/*
 * Isomorphic SmartGWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */

package com.smartgwt.sample.showcase.client.dataintegration.java.datasource;

import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.types.ListGridEditEvent;
import com.smartgwt.sample.showcase.client.PanelFactory;
import com.smartgwt.sample.showcase.client.ShowcasePanel;
import com.smartgwt.sample.showcase.client.SourceEntity;


public class ORMDataSourceSample extends ShowcasePanel {
    private static final String DESCRIPTION = "<p>This example shows an entirely custom DataSource that connects SmartGWT Server to Hibernate (note that this is just an example of the principles involved - SmartGWT Server's built-in Hibernate " +
            "support is considerably more sophisticated than the simple adapter shown here). It is created by extending <code>BasicDataSource</code> and implementing the four core CRUD methods. In this case, we connect DataSource requests to Hibernate " +
            "<code>Criteria</code> queries and the <code>saveOrUpdate</code> method.</p>" +
            "<p>This implementation, though simple, is fully functional and could be used unchanged in a real application. It supports all four CRUD operations, plus data pagination, server-side sorting and filtering, client cache synchronization, " +
            "and of course it is actually persisting the data to a real database. Creating an equivalent adapter for Toplink or Ibatis or some other ORM solution would be a fairly simple matter of replacing the Hibernate-specific code in this example " +
            "with the equivalent specifics from the other ORM system.</p>" +
            "<p>As with the other custom DataSource examples, note how the <code>ORMDataSource.java</code> code deals entirely in native Java objects - even entirely custom DataSources benefit from SmartGWT Server's robust and comprehensive Javascript<->Java translation.</p>";

    public static class Factory implements PanelFactory {

        private String id;

        public Canvas create() {
            ORMDataSourceSample panel = new ORMDataSourceSample();
            id = panel.getID();
            return panel;
        }

        public String getID() {
            return id;
        }

        public String getDescription() {
            return DESCRIPTION;
        }
    }

    protected boolean isTopIntro() {
        return true;
    }

    public Canvas getViewPanel() {
        final ListGrid listGrid = new ListGrid();
        listGrid.setWidth(700);
        listGrid.setHeight(224);
        listGrid.setAlternateRecordStyles(true);
        listGrid.setDataSource(DataSource.get("ormDataSource_country"));
        listGrid.setAutoFetchData(true);
        listGrid.setShowFilterEditor(true);
        listGrid.setCanEdit(true);
        listGrid.setEditEvent(ListGridEditEvent.CLICK);
        listGrid.setCanRemoveRecords(true);

        ListGridField countryCode = new ListGridField("countryCode", "Code", 50);
        ListGridField countryName = new ListGridField("countryName", "Country");
        ListGridField capital = new ListGridField("capital", "Capital");
        ListGridField area = new ListGridField("area", "Area (km&amp;sup2;)");
        ListGridField population = new ListGridField("population", "Population");
        ListGridField gdp = new ListGridField("gdp", "GDP($M)");

        listGrid.setFields(countryCode, countryName, capital, area, population, gdp);

        IButton newButton = new IButton("Add New");
        newButton.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                listGrid.startEditingNew();
            }
        });


        VLayout layout = new VLayout(15);
        layout.addMember(listGrid);
        layout.addMember(newButton);

        return layout;
    }

    public SourceEntity[] getSourceUrls() {
        return new SourceEntity[] {
                new SourceEntity("ORMDataSource.java", JAVA, "source/datasource/ORMDataSource.java.html", true),
                new SourceEntity("Country.java", JAVA, "source/beans/Country.java.html", true),
                new SourceEntity("WEB-INF/web.xml", JAVA, "source/ds/common/web.xml.html", true)
        };
    }
    public String getIntro() {
        return DESCRIPTION;
    }
}