/*
 Isomorphic SmartClient web presentation layer
 Online documentation data
 Copyright 2000-2008 Isomorphic Software, Inc. All rights reserved.
*/


var docItems = {
    "method:Layout.getMember":{
        ref:"method:Layout.getMember",
        flags:"",
        description:"\n Given a numerical index or a member ID, return a pointer to the appropriate member.\n <p>\n If passed a member Canvas, just returns it.\n\n",
        name:"getMember",
        seeAlso:[
            {ref:"method:Layout.getMemberNumber", description:null}
        ],
        params:[
            {description:"identifier for the required member", optional:false, name:"memberID",
             defaultValue:null, type:"string | number | Canvas"}
        ],
        returns:{description:"member widget", type:"Canvas"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:ComboBoxItem.pickListProperties":{
        ref:"attr:ComboBoxItem.pickListProperties",
        flags:"IRA",
        description:"If specified this properties block will be applied to the ${isc.DocUtils.linkForRef('class:PickListMenu','pickList')}\n created for this Form Item.",
        name:"pickListProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:ListGridField.includeInRecordSummaryFields":{
        ref:"attr:ListGridField.includeInRecordSummaryFields",
        flags:"IR",
        description:"If this listGrid has any fields of type <code>\"summary\"</code> and\n this field will be ${isc.DocUtils.linkForRef('attr:ListGridField.includeInRecordSummary','included')} in summary calculations\n by default, this attribute provides an opportunity to explicitly specify which summary fields\n the record should be displayed in.\n <P>\n Specified as an array of fieldNames. If set, this field value will only be included for\n record summary value calculations for summary fields whos name is included in this array.",
        name:"includeInRecordSummaryFields",
        seeAlso:[],
        valueType:"array of fieldNames",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:ColorPicker.autoCenterOnShow":{
        ref:"attr:ColorPicker.autoCenterOnShow",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ColorPicker.autoPosition')} is false, this property controls whether to automatically center the\n colorPicker every time it is reshown with the show() method.",
        name:"autoCenterOnShow",
        seeAlso:[
            {ref:"attr:ColorPicker.autoPosition", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:UploadItem.setValue":{
        ref:"method:UploadItem.setValue",
        flags:"",
        description:" Attempting to set the value for an upload form item is disallowed for security reasons.\n Therefore this method will just log a warning, and not modify the value of the item.\n",
        name:"setValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:UploadItem"
    },
    "attr:ListGridRecord.detailDS":{
        ref:"attr:ListGridRecord.detailDS",
        flags:"IRWA",
        description:"The default value of ${isc.DocUtils.linkForRef('attr:ListGrid.recordDetailDSProperty')}.",
        name:"detailDS",
        seeAlso:[],
        valueType:"DataSource",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "attr:TileLayout.expandMargins":{
        ref:"attr:TileLayout.expandMargins",
        flags:"IR",
        description:"With ${isc.DocUtils.linkForRef('type:LayoutPolicy')}:\"fit\", should margins be expanded so that tiles fill the\n available space in the TileLayout on the breadth axis?\n This can also affect ${isc.DocUtils.linkForRef('attr:TileLayout.tileWidth')} or ${isc.DocUtils.linkForRef('attr:TileLayout.tileHeight')}. See those properties for details.",
        name:"expandMargins",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "method:ResultSet.sortByProperty":{
        ref:"method:ResultSet.sortByProperty",
        flags:"",
        description:" Sort this ResultSet by a property of each record.\n <P>\n Sorting is performed on the client for a ResultSet that has a full cache for the current\n filter criteria.  Otherwise, sorting is performed by the server, and changing the sort order\n will invalidate the cache.\n <P>\n <b>NOTE:</b> normalizers are not supported by ResultSets in \"paged\" mode, although valueMaps\n in the DataSource are respected by the SQLDataSource.\n\n",
        name:"sortByProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to sort by", optional:false, name:"property",
             defaultValue:null, type:"string"},
            {description:"true == sort ascending, false == sort descending", optional:false, name:"up",
             defaultValue:null, type:"boolean"},
            {description:"May be specified as a function, with signature \n              <code>normalize(item, propertyName, context)</code>, where <code>item</code> is\n              a pointer to the item in the array, <code>propertyName</code> is the \n              property by which the array is being sorted, and <code>context</code> is the\n              arbitrary context passed into this method. Normalizer function should return\n              the value normalized for sorting.<br>\n              May also be specified as a ValueMap which maps property values to sortable values.",
             optional:true, name:"normalizer", defaultValue:null, type:"function or ValueMap"},
            {description:"Callers may pass an arbitrary context into the sort method, which                          will then be made available to the normalizer function",
             optional:true, name:"context", defaultValue:null, type:"any"}
        ],
        returns:{description:"the list itself", type:"List"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["sorting"]
    },
    "attr:Calendar.chosenDate":{
        ref:"attr:Calendar.chosenDate",
        flags:"IRW",
        description:"The date for which events are displayed in the day, week, and month tabs of \n the calendar.  Default is today.",
        name:"chosenDate",
        setter:["setChosenDate"],
        seeAlso:[],
        valueType:"Date",
        defaultValue:"'Today'",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["date"]
    },
    "method:ListGrid.getDragTrackerIcon":{
        ref:"method:ListGrid.getDragTrackerIcon",
        flags:"",
        description:" Return an icon to display as a drag tracker when the user drags some record.<br>\n Default implementation: \n If ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')} is specified for the title field of this grid \n (see ${isc.DocUtils.linkForRef('method:ListGrid.getTitleField')}), the appropriate value icon will be displayed.\n If no appropriate valueIcon can be found, the icon will be derived from\n ${isc.DocUtils.linkForRef('attr:ListGrid.trackerImage')}.<br>\n If multiple records are selected, only the first record is examined for valueIcons.\n <p>\n Note: Only called if ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')} is set to <code>\"icon\"</code>. \n",
        name:"getDragTrackerIcon",
        seeAlso:[],
        params:[
            {description:"Records being dragged", optional:false, name:"records", defaultValue:null,
             type:"Array of ListGridRecord"}
        ],
        returns:{description:"Image URL of icon to display", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dragTracker"]
    },
    "attr:FormItem.startRow":{
        ref:"attr:FormItem.startRow",
        flags:"IRW",
        description:"Whether this item should always start a new row in the form layout.",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "attr:ToolbarItem.buttonProperties":{
        ref:"attr:ToolbarItem.buttonProperties",
        flags:"IRA",
        description:"Default properties for this toolbar's buttons.",
        name:"buttonProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ToolbarItem"
    },
    "method:ListGrid.freezeField":{
        ref:"method:ListGrid.freezeField",
        flags:"",
        description:" Freeze the indicated field, so that it remains in place and visible when horizontal\n scrolling occurs.\n\n",
        name:"freezeField",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"field", defaultValue:null,
             type:"ListGridField or colNum or ListGridField.name"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["frozenFields"]
    },
    "classAttr:Validator.mustBeLongerThan":{
        ref:"classAttr:Validator.mustBeLongerThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>lengthRange</code> type validator\n returns false becaues the value passed in has fewer than <code>validator.min</code> characters.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeLongerThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be more than ${min} characters\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.optionTextMatchStyle":{
        ref:"attr:ListGridField.optionTextMatchStyle",
        flags:"IR",
        description:"For fields with an ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')}, where\n ${isc.DocUtils.linkForRef('attr:ListGridField.autoFetchDisplayMap')} is true, this property will govern\n the <code>textMatchStyle</code> attribute of the ${isc.DocUtils.linkForRef('class:DSRequest')} parameter passed to \n ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} when retrieving the remote data set to be used as \n a basis for thiis field's valueMap.",
        name:"optionTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "method:ColorPicker.getOpacity":{
        ref:"method:ColorPicker.getOpacity",
        flags:"",
        description:" Returns the opacity of the currently-selected color, as an integer from 0-100. If \n opacity is switched off, this is always 100.\n",
        name:"getOpacity",
        seeAlso:[],
        returns:{description:"opacity value", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:MenuItem.iconWidth":{
        ref:"attr:MenuItem.iconWidth",
        flags:"IR",
        description:"The width applied to this item's icon.  The default of <code>16</code> can be changed\n for all MenuItems by overriding ${isc.DocUtils.linkForRef('attr:Menu.iconWidth')}.",
        name:"iconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"object:MenuItem",
        groups:["menuIcons"]
    },
    "method:Button.getTitle":{
        ref:"method:Button.getTitle",
        flags:"A",
        description:" Return the title - text/HTML drawn inside the component.\n <p>\n Default is to simply return this.title.\n",
        name:"getTitle",
        seeAlso:[],
        returns:{description:"HTML for the title", type:"string"},
        type:"method",
        definingClass:"class:Button"
    },
    "classMethod:XMLTools.loadXML":{
        ref:"classMethod:XMLTools.loadXML",
        flags:"",
        description:" Load an XML document from the origin server or from a foreign server by relaying through the\n origin server.  An asynchronous callback provides both the XML document and raw text of the\n response.\n <P>\n Relaying through the origin server requires that the ISC HttpProxyServlet be installed and\n accessible.\n \n",
        name:"loadXML",
        seeAlso:[],
        params:[
            {description:"URL to load the schema from", optional:false, name:"URL", defaultValue:null,
             type:"URL"},
            {description:"callback to fire when the XML is loaded.  Signature is                             callback(xmlDoc, xmlText)",
             optional:false, name:"callback", defaultValue:null, type:"callback"},
            {description:"additional properties to set on the RPCRequest                                         that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"RPCRequest"}
        ],
        type:"classMethod",
        definingClass:"class:XMLTools"
    },
    "attr:Img.src":{
        ref:"attr:Img.src",
        flags:"IRW",
        description:"The base filename for the image.\n <P>\n If <code>img.state</code> is set, it's value will be appended to the URL before the\n file extension.  \n <P>\n For example, given an <code>src</code> of \"findIcon.gif\" and a state of\n \"Disabled\", the resulting image name would be \"findIcon_Disabled.gif\".  Compound states\n such as \"Selected\", \"Focused\" and \"Over\" or \"Down\" will have an intervening underscore,\n resulting in, for example, <code>\"findIcon_Selected_Down.gif\"</code>.",
        name:"src",
        setter:["setSrc"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"blank.gif\"",
        type:"attr",
        definingClass:"class:Img",
        groups:["appearance"]
    },
    "attr:ImgButton.iconAlign":{
        ref:"attr:ImgButton.iconAlign",
        flags:"IR",
        description:"If this button is showing an icon should it be right or left aligned?",
        name:"iconAlign",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:DynamicForm.showInlineErrors":{
        ref:"attr:DynamicForm.showInlineErrors",
        flags:"IRW",
        description:"If true, field errors are written into the form next to the item(s) where the errors\n occurred.  Errors may appear as text or just an icon (via ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText')}:false}.\n <P>\n If false, errors are written at the top of the form.\n <P>\n To do some other kind of error display, override ${isc.DocUtils.linkForRef('method:DynamicForm.showErrors')}.",
        name:"showInlineErrors",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "method:ListGrid.collapseRecord":{
        ref:"method:ListGrid.collapseRecord",
        flags:"",
        description:" Collapses a given ${isc.DocUtils.linkForRef('object:ListGridRecord','record')} which has been previously expanded using\n ${isc.DocUtils.linkForRef('method:ListGrid.expandRecord')}.\n \n",
        name:"collapseRecord",
        seeAlso:[],
        params:[
            {description:"record to collapse", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:Dialog.setToolbarButtons":{
        ref:"method:Dialog.setToolbarButtons",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons')} for this dialog.\n",
        name:"setToolbarButtons",
        seeAlso:[],
        params:[
            {description:"buttons for the toolbar", optional:false, name:"newButtons",
             defaultValue:"null", type:"array"}
        ],
        type:"method",
        definingClass:"class:Dialog"
    },
    "attr:Hilite.title":{
        ref:"attr:Hilite.title",
        flags:"IRW",
        description:"User-visible title for this hilite.  Used for interfaces such as menus that can enable or\n disable hilites.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "method:DynamicForm.setError":{
        ref:"method:DynamicForm.setError",
        deprecated:"This method has been deprecated as of SmartClient release 5.7.  Use ${isc.DocUtils.linkForRef('method:DynamicForm.addFieldErrors')} or ${isc.DocUtils.linkForRef('method:DynamicForm.setFieldErrors')} instead",
        flags:"A",
        description:"          Sets error message(s) for the specified itemName to the error string or array of\n          strings. You must call form.markForRedraw() to display the new error message(s).<br>\n          <b>Note:</b> you can call this multiple times for an individual itemName\n			 which will result in an array of errors being remembered.\n\n",
        name:"setError",
        seeAlso:[],
        params:[
            {description:"name of the item to set", optional:false, name:"itemName", defaultValue:null,
             type:"string"},
            {description:"error message string or array of strings", optional:false,
             name:"errorMessage", defaultValue:null, type:"string|array"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "attr:TreeGrid.showRoot":{
        ref:"attr:TreeGrid.showRoot",
        flags:"IR",
        description:"Specifies whether the root node should be displayed in the treeGrid.\n <P>\n This property is only available for \"children\" modelType trees, hence is not allowed for\n trees that load data from the server dynamically via ${isc.DocUtils.linkForRef('method:TreeGrid.fetchData')}.  \n <P>\n To get the equivalent of a visible \"root\" node in a tree that loads data dynamically,\n add a singular, top-level parent to the data.  However, note that this top-level parent\n will technically be the only child of root, and the implicit root object will be\n returned by ${isc.DocUtils.linkForRef('method:Tree.getRoot','this.data.getRoot()')}.",
        name:"showRoot",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeField"]
    },
    "method:ListGrid.setData":{
        ref:"method:ListGrid.setData",
        flags:"",
        description:" Initialize the data object with the given array. Observes methods of the data object\n so that when the data changes, the listGrid will redraw automatically.\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"data to show in the list", optional:false, name:"newData", defaultValue:null,
             type:"List of ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["data"]
    },
    "attr:ComboBoxItem.showAllOptions":{
        ref:"attr:ComboBoxItem.showAllOptions",
        flags:"IR",
        description:"If true, even non-matching options will be shown, with configurable \n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.separatorRows','separator rows')} in between.  Not valid for\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.optionDataSource','databound pickLists')}.",
        name:"showAllOptions",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:FilterBuilder.getCriteria":{
        ref:"method:FilterBuilder.getCriteria",
        flags:"",
        description:" Get the criteria entered by the user.\n \n",
        name:"getCriteria",
        seeAlso:[],
        returns:{description:null, type:"AdvancedCriteria"},
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "attr:ListGrid.includeInSummaryProperty":{
        ref:"attr:ListGrid.includeInSummaryProperty",
        flags:"IRW",
        description:"Property name on a record that will be checked to determine whether a record should\n be included when calculating totals for the ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary','grid summary')}.",
        name:"includeInSummaryProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"includeInSummary\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "classAttr:GroupingMessages.upcomingTomorrowTitle":{
        ref:"classAttr:GroupingMessages.upcomingTomorrowTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs tomorrow,\n relative to the current date.",
        name:"upcomingTomorrowTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Tomorrow\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:Canvas.getScrollTop":{
        ref:"method:Canvas.getScrollTop",
        flags:"A",
        description:" Get the number of pixels this Canvas is scrolled from its top edge.\n",
        name:"getScrollTop",
        seeAlso:[],
        returns:{description:"scrollTop", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "scrolling"]
    },
    "attr:ListGrid.dataPageSize":{
        ref:"attr:ListGrid.dataPageSize",
        flags:"IRW",
        description:"When using data paging, how many records to fetch at a time.  The value of this\n attribute is passed on to the auto-constructed ${isc.DocUtils.linkForRef('class:ResultSet')} object for this\n component.  In effect, this gives you control over the ${isc.DocUtils.linkForRef('attr:ResultSet.resultSize')}\n attribute for this component.\n <P>\n <b>Note</b> that regardless of the <code>dataPageSize</code> setting, a component will always fetch\n all of data that it needs to draw.  Settings such as\n ${isc.DocUtils.linkForRef('attr:ListGrid.showAllRecords','showAllRecords:true')},\n ${isc.DocUtils.linkForRef('attr:ListGrid.drawAllMaxCells','drawAllMaxCells')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio','drawAheadRatio')} can cause more rows than the configured\n <code>dataPageSize</code> to be fetched.",
        name:"dataPageSize",
        seeAlso:[
            {ref:"attr:ResultSet.resultSize", description:null}
        ],
        valueType:"integer",
        defaultValue:"75",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('databoundFetch')}"],
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "group:ancestry":{
        ref:"group:ancestry",
        refs:["type:DisplayNodeType"],
        description:"Parent/child relationships",
        name:"ancestry",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Log.echoLeaf":{
        ref:"classMethod:Log.echoLeaf",
        flags:"",
        description:" Return a very short (generally less than 40 characters) string representation of any object,\n suitable for viewing by a developer for debugging purposes.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echoLeaf\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoLeaf",
        seeAlso:[
            {ref:"method:Class.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "method:DetailViewer.exportData":{
        ref:"method:DetailViewer.exportData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','DataSource')} to \n retrieve data that matches the current filter and sort criteria for this component, then \n exports the resulting data to a file or window in the requested format.\n <P>\n For more information on exporting data, see ${isc.DocUtils.linkForRef('method:DataSource.exportData')}.\n\n",
        name:"exportData",
        seeAlso:[],
        params:[
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:DetailViewer",
        groups:["dataBoundComponentMethods"]
    },
    "group:menuBasic":{
        ref:"group:menuBasic",
        refs:["method:MenuItem.action", "attr:MenuItem.title", "attr:MenuItem.submenu", 
         "attr:MenuItem.isSeparator","attr:MenuItem.enabled"],
        name:"menuBasic",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Date.setShortDatetimeDisplayFormat":{
        ref:"classMethod:Date.setShortDatetimeDisplayFormat",
        flags:"",
        description:"  Set the default short format for datetime values. After calling this method, subsequent calls to \n ${isc.DocUtils.linkForRef('method:Date.toShortDateTime')} will return a string formatted according to this format \n specification. Note that this will be the standard datetime format used by\n SmartClient components.\n <P>\n The <code>format</code> parameter may be either a ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} string, or \n a function. If passed a function, this function will be executed in the scope of the Date\n and should return the formatted string.\n <P>\n Initial default format is <code>\"toUSShortDateTime\"</code>.\n\n",
        name:"setShortDatetimeDisplayFormat",
        seeAlso:[],
        params:[
            {description:"new formatter", optional:false, name:"format", defaultValue:null,
             type:"DateDisplayFormat | function"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('dateFormat')}", 
         "${isc.DocUtils.linkForExampleId('customDateFormat')}"],
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:GridRenderer.getRowHeight":{
        ref:"method:GridRenderer.getRowHeight",
        flags:"",
        description:" Return the height this row should be.  Default is this.cellHeight. If\n ${isc.DocUtils.linkForRef('attr:GridRenderer.fixedRowHeights')} is false, the row may be rendered taller than this\n specified size.\n\n",
        name:"getRowHeight",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number", optional:false, name:"rowNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"height in pixels", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:RestDataSource.fetchDataURL":{
        ref:"attr:RestDataSource.fetchDataURL",
        flags:"IR",
        description:"Custom dataURL for fetch type operations",
        name:"fetchDataURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:PickList.filterLocally":{
        ref:"attr:PickList.filterLocally",
        flags:"IRA",
        description:"If <code>filterLocally</code> is set for this item, and this item is showing options \n from a dataSource, fetch the entire set of options from the server, and use these values\n to map the item value to the appropriate display value. Also use <code>\"local\"</code>\n type filtering on drop down list of options.\n <P>\n This means data will only be fetched once from the server, and then filtered on the\n client.",
        name:"filterLocally",
        seeAlso:[
            {ref:"attr:FormItem.filterLocally", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "type:TileLayoutPolicy":{
        ref:"type:TileLayoutPolicy",
        values:[
            {description:"Each line has the same number of tiles, based on ${isc.DocUtils.linkForRef('attr:TileLayout.tilesPerLine')} if set, otherwise,\n based on fitting as many tiles per line as possible consistent with\n ${isc.DocUtils.linkForRef('attr:TileLayout.tileSize','tileSize')} and ${isc.DocUtils.linkForRef('attr:TileLayout.tileMargin')}. \n <P>\n Tiles that do not specify a size will be sized to fill available space.",
             value:"\"fit\""},
            {description:"Tiles are laid out with varying numbers of tiles per line according to each tile's size,\n the ${isc.DocUtils.linkForRef('attr:TileLayout.tileMargin','tileMargin')}, and the available space.\n <P>\n Tiles are never resized by the TileLayout and ${isc.DocUtils.linkForRef('attr:TileLayout.tileSize')} is ignored.",
             value:"\"flow\""}
        ],
        description:"Policy for laying out tiles. \n <P>\n Because a TileLayout can be either horizontally or vertically oriented, the general term\n \"line\" is used to mean either a row or column of tiles. \n <P>\n <b>NOTE</b>: for typical form layouts (a variety of input fields and buttons in a\n tabular layout with col-spanning and row-spanning support), use a ${isc.DocUtils.linkForRef('class:DynamicForm')} and\n see the ${isc.DocUtils.linkForRef('group:formLayout')} topic.",
        name:"TileLayoutPolicy",
        seeAlso:[],
        type:"type"
    },
    "attr:ListGrid.sortDirection":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.sortDirection",
        description:"Sorting direction of this ListGrid. If specified when the ListGrid is initialized,\n this property will be the default sorting direction for the ${isc.DocUtils.linkForRef('attr:ListGrid.sortField')}.\n May be overridden by specifying ${isc.DocUtils.linkForRef('attr:ListGridField.sortDirection')}.\n <P>\n After initialization, this property will be updated on ${isc.DocUtils.linkForRef('method:ListGrid.sort')} to reflect\n the current sort direction of the grid.",
        name:"sortDirection",
        setter:["listGrid.sort"],
        valueType:"SortDirection",
        seeAlso:[
            {ref:"type:SortDirection", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        defaultValue:"Array.ASCENDING",
        groups:["sorting"],
        definingClass:"class:ListGrid"
    },
    "attr:ResultTree.disableCacheSync":{
        ref:"attr:ResultTree.disableCacheSync",
        flags:"IRA",
        description:"By default when the data of this ResultTree's dataSource is modified, the ResultTree will\n be updated to display these changes.\n Set this flag to true to disable this behavior.",
        name:"disableCacheSync",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ResultTree",
        groups:["cacheSync"]
    },
    "method:ListGrid.getEditValues":{
        ref:"method:ListGrid.getEditValues",
        flags:"",
        description:" Returns the current set of unsaved edits for a given row being edited.\n\n",
        name:"getEditValues",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"valuesID", defaultValue:null, type:"number | Object"}
        ],
        returns:{description:"Current editValues object for the row.  This contains the current                     edit values in {fieldName1:value1, fieldName2:value2} format.",
         type:"object"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "classAttr:ValidatorTypes.lengthRange":{
        ref:"classAttr:ValidatorTypes.lengthRange",
        flags:"IR",
        description:"This validator type applies to string values only.  If the value is a string value\n  validation will fail if the strings length falls outside the range specified by \n  <code>validator.max</code> and <code>validator.min</code>.<br>\n  Note that non-string values will always pass validation by this validator type.<br>\n Note that the <code>errorMessage</code> for this validator will be evaluated as\n a dynamicString - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"lengthRange",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:OperationBinding.operationType":{
        ref:"attr:OperationBinding.operationType",
        flags:"IR",
        description:"Which operationType this operationBinding is for.  This property is only settable on an\n operationBinding, not a DataSource as a whole.",
        name:"operationType",
        seeAlso:[],
        valueType:"DSOperationType",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:RPCRequest.useXmlHttpRequest":{
        ref:"attr:RPCRequest.useXmlHttpRequest",
        deprecated:"As of SmartClient 5.5, use ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')}.  If you specify a value for this property, it will take precedence over ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')}.",
        flags:"IRWA",
        description:"Selects the default http transport for this RPCRequest.  If set to true, this request will use\n XMLHttpRequest for the transport to the server.  If set to false it will use a hidden frame.  If\n left unset, the transport mechanism is determined from the RPCManager default set in \n ${isc.DocUtils.linkForRef('classAttr:RPCManager.useXmlHttpRequest')}\n <p>\n If you're using queueing, note that all requests in the queue must use the same transport.\n If you attempt to send a request via a different transport than those that are currently on\n the queue, it will be sent to the server separately, ahead of the queue, and a warning will\n be logged to the Developer Console.\n <p>\n If you specify <code>true</code> for this attribute and XMLHttp is not available, a warning\n will be logged to the Developer Console and RPCManager will attempt to use the frames\n transport for this request.  Note that some features like\n ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} requre the XMLHttp transport and will not work if the\n XMLHttp transport is unavailable (this can happen if the end user is using Internet Explorer\n and has disabled ActiveX).  You can query the availability of XMLHttp by calling\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.xmlHttpRequestAvailable')}",
        name:"useXmlHttpRequest",
        seeAlso:[
            {ref:"classAttr:RPCManager.useXmlHttpRequest", description:null},
            {ref:"classMethod:RPCManager.xmlHttpRequestAvailable", description:null}
        ],
        valueType:"boolean",
        defaultValue:"RPCManager.useXmlHttpRequest",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:Canvas.getHoverHTML":{
        ref:"method:Canvas.getHoverHTML",
        flags:"",
        description:" If <code>this.showHover</code> is true, when the user holds the mouse over this Canvas for\n long enough to trigger a hover event, a hover canvas is shown by default. This method returns\n the contents of that hover canvas. Default implementation returns <code>this.prompt</code> -\n override for custom hover HTML. Note that returning <code>null</code> or an empty string will\n suppress the hover canvas altogether.\n",
        name:"getHoverHTML",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:Mail.templateFile":{
        ref:"attr:Mail.templateFile",
        flags:"IR",
        description:"Qualified name of the file containing the message template, relative to webroot.  Like all\n the other <code>String</code> properties of <code>Mail</code>, you can use Velocity\n substitution variables in this property.  The content itself (ie, the contents of the file\n named in this property) can also contain Velocity substitutions, of course.\n <p>\n You must specify either this property or ${isc.DocUtils.linkForRef('attr:Mail.messageTemplate','messageTemplate')}, but\n clearly it makes no sense to specify them botm.",
        name:"templateFile",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "method:Calendar.getCurrentViewName":{
        ref:"method:Calendar.getCurrentViewName",
        flags:"",
        description:" Get the name of the visible view. Either 'day', 'week', or 'month'.\n\n",
        name:"getCurrentViewName",
        seeAlso:[],
        returns:{description:"The name of the visible view.", type:"ViewName"},
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:ListGridField.suppressValueIcon":{
        ref:"attr:ListGridField.suppressValueIcon",
        flags:"IRW",
        description:"If this field has a valueIcons property specified, setting this property to true will \n prevent the valueIcon being written out into this field's cells.",
        name:"suppressValueIcon",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null},
            {ref:"attr:ListGridField.showValueIconOnly", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:ResultSet.dropCacheOnUpdate":{
        ref:"attr:ResultSet.dropCacheOnUpdate",
        flags:"IRA",
        description:"Whether to discard all cached rows when a modification operation (add, update, remove)\n occurs on the ResultSet's DataSource.\n <P>\n A ResultSet that has a complete cache for the current filter criteria can potentially\n incorporate a newly created or updated row based on the data that the server returns\n when a modification operation completes.  However this is not always possible for\n ResultSets that show some types of joins, or when the server cannot easily return update\n data.  In this case set <code>dropCacheOnUpdate</code> to cause the cache to be\n discarded when an update occurs.\n <P>\n <code>dropCacheOnUpdate</code> can be set either directly on a ResultSet, or on a\n DataSource in order to affect all ResultSets on that DataSource.",
        name:"dropCacheOnUpdate",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["cacheSync"]
    },
    "method:DatabaseBrowser.getResults":{
        ref:"method:DatabaseBrowser.getResults",
        flags:"",
        description:" \n",
        name:"getResults",
        seeAlso:[],
        type:"method",
        definingClass:"class:DatabaseBrowser"
    },
    "group:validtion":{
        ref:"group:validtion",
        refs:["method:ListGrid.clearRowErrors"],
        name:"validtion",
        seeAlso:[],
        type:"group"
    },
    "attr:TreeNode.parentId":{
        ref:"attr:TreeNode.parentId",
        flags:"IR",
        description:"For trees with modelType:\"parent\", this property specifies the unique ID of this node's \n parent node.\n The unique ID of a node, together with the unique ID of its parent is used by\n ${isc.DocUtils.linkForRef('method:Tree.linkNodes')} to link a list of nodes into a tree.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')}.",
        name:"parentId",
        seeAlso:[
            {ref:"attr:TreeNode.id", description:null},
            {ref:"method:Tree.linkNodes", description:null},
            {ref:"attr:Tree.modelType", description:null},
            {ref:"attr:Tree.parentIdField", description:null}
        ],
        valueType:"String or Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:ListGrid.freezeFieldText":{
        ref:"attr:ListGrid.freezeFieldText",
        flags:"IRWA",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','this.canFreezeFields')} is true, this string will be shown as the\n title for the menu item to freeze a currently unfrozen field.\n <P>\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>viewer</code> available as a variable mapped to\n the ListGrid instance, and <code>field</code> as a variable pointing to the ListGrid field.\n <P>\n Default value returns \"Freeze \" + the field's summary title.",
        name:"freezeFieldText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Freeze \\${viewer.getSummaryTitle(field)}\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "method:Tree.getAllNodes":{
        ref:"method:Tree.getAllNodes",
        flags:"",
        description:" Get all the nodes that exist in the tree under a particular node, as a flat list, in\n depth-first traversal order.\n\n",
        name:"getAllNodes",
        seeAlso:[],
        params:[
            {description:"soptional node to start from.  Default is root.", optional:true, name:"node",
             defaultValue:null, type:null}
        ],
        returns:{description:"all the nodes that exist in the tree", type:"Array of TreeNode"},
        type:"method",
        definingClass:"class:Tree"
    },
    "classMethod:Page.getScreenWidth":{
        ref:"classMethod:Page.getScreenWidth",
        flags:"",
        description:"		Get the width of the user's screen, in pixels.\n",
        name:"getScreenWidth",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:Scrollbar.skinImgDir":{
        ref:"attr:Scrollbar.skinImgDir",
        flags:"IRA",
        description:"Where are the skin images for the Scrollbar.  This is local to the ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','overall skin directory')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/Scrollbar/\"",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["images"]
    },
    "method:ColorPicker.setSaturation":{
        ref:"method:ColorPicker.setSaturation",
        flags:"",
        description:" Sets the Saturation of the selected color\n",
        name:"setSaturation",
        seeAlso:[
            {ref:"method:ColorPicker.getSaturation", description:null}
        ],
        params:[
            {description:"An integer between 0 and 240", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:Canvas.edgeCenterBackgroundColor":{
        ref:"attr:Canvas.edgeCenterBackgroundColor",
        flags:"IR",
        description:"Background color for the center section only.  Can be used as a surrogate background\n color for the decorated Canvas, if the Canvas is set to partially overlap the edges and\n hence can't show a background color itself without occluding media.",
        name:"edgeCenterBackgroundColor",
        seeAlso:[],
        valueType:"Color",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "attr:GridRenderer.showAllRows":{
        ref:"attr:GridRenderer.showAllRows",
        flags:"IRA",
        description:"Whether all rows should be drawn all at once, or only rows visible in the viewport.<br><br>\n\n Drawing all rows causes longer initial rendering time, but allows smoother vertical scrolling.\n With a very large number of rows, showAllRows will become too slow.",
        name:"showAllRows",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "attr:TabSet.tabBarThickness":{
        ref:"attr:TabSet.tabBarThickness",
        flags:"IRW",
        description:"Thickness of tabBar, applies to either orientation (specifies height for horizontal,\n width for vertical orientation).  Note that overriding this value for TabSets that are\n skinned with images generally means providing new media for the borders.",
        name:"tabBarThickness",
        seeAlso:[],
        valueType:"number",
        defaultValue:"21",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBar"]
    },
    "method:Class.logError":{
        ref:"method:Class.logError",
        flags:"",
        description:" Log a message at \"error\" priority\n \n",
        name:"logError",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "method:Tree.getParent":{
        ref:"method:Tree.getParent",
        flags:"",
        description:"\n Returns the parent of this node.\n\n",
        name:"getParent",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"parent of this node", type:"node"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:Canvas.mouseOut":{
        ref:"method:Canvas.mouseOut",
        flags:"A",
        description:"\n Executed when the mouse leaves this widget.  No default implementation.\n\n",
        name:"mouseOut",
        seeAlso:[],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customMouseEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:StatefulCanvas.setAutoFit":{
        ref:"method:StatefulCanvas.setAutoFit",
        flags:"",
        description:" Setter method for the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.autoFit')} property. Pass in true or false to turn\n autoFit on or off. When autoFit is set to <code>false</code>, canvas will be resized to\n it's previously specified size.\n",
        name:"setAutoFit",
        seeAlso:[],
        params:[
            {description:"New autoFit setting.", optional:false, name:"autoFit", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas"
    },
    "attr:FormItem.iconPrompt":{
        ref:"attr:FormItem.iconPrompt",
        flags:"IRWA",
        description:"Default prompt (and tooltip-text) for icons.",
        name:"iconPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:DynamicForm.cancelParamValue":{
        ref:"attr:DynamicForm.cancelParamValue",
        flags:"IRW",
        description:"The value of the special field sent to the server as part of ${isc.DocUtils.linkForRef('method:DynamicForm.cancel')}",
        name:"cancelParamValue",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"cancel\"",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "method:Layout.setLayoutMargin":{
        ref:"method:Layout.setLayoutMargin",
        flags:"",
        description:" Method to force a reflow of the layout after directly assigning a value to any of the\n layout*Margin properties. Takes no arguments.\n\n",
        name:"setLayoutMargin",
        seeAlso:[],
        type:"method",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "attr:FormItem.showDisabled":{
        ref:"attr:FormItem.showDisabled",
        flags:"IRWA",
        description:"When this item is disabled, should it be re-styled to indicate its disabled state?",
        name:"showDisabled",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:Selection.deselectAll":{
        ref:"method:Selection.deselectAll",
        flags:"",
        description:"			Deselect ALL records of the list\n",
        name:"deselectAll",
        seeAlso:[],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:Calendar.dayHeaderBaseStyle":{
        ref:"attr:Calendar.dayHeaderBaseStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to the day headers of the month view.\n This style will have \"Dark\", \"Over\", \"Selected\", or \"Disabled\"\n appended to it according to the state of the cell.",
        name:"dayHeaderBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calMonthDayHeader\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "method:FilterBuilder.filterChanged":{
        ref:"method:FilterBuilder.filterChanged",
        flags:"",
        description:" Handler fired when there is a change() event fired on any FormItem within the \n filterBuilder. \n\n",
        name:"filterChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "attr:ListGrid.ungroupText":{
        ref:"attr:ListGrid.ungroupText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, and\n ${isc.DocUtils.linkForRef('attr:ListGrid.isGrouped','this.isGrouped')} is true, this\n attribute will be shown as the title for the menu item to ungroup the grid.",
        name:"ungroupText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Ungroup\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "group:shadow":{
        ref:"group:shadow",
        refs:["method:Canvas.setShowShadow", "attr:Canvas.showShadow", "attr:Canvas.shadowDepth", 
         "attr:Canvas.shadowOffset","attr:Canvas.shadowSoftness", "attr:Canvas.shadowImage"],
        name:"shadow",
        seeAlso:[],
        type:"group"
    },
    "attr:DataSource.showLocalFieldsOnly":{
        ref:"attr:DataSource.showLocalFieldsOnly",
        flags:"IR",
        description:"For a DataSource that inherits ${isc.DocUtils.linkForRef('attr:DataSource.fields')} from another DataSource \n (via ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}), indicates that only the fields listed in this DataSource\n should be shown.  All other inherited parent fields will be marked \"hidden:true\".",
        name:"showLocalFieldsOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["fields"]
    },
    "classMethod:Log.logWarn":{
        ref:"classMethod:Log.logWarn",
        flags:"",
        description:" Log a message at \"warn\" priority\n \n",
        name:"logWarn",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:ListGrid.booleanImageHeight":{
        ref:"attr:ListGrid.booleanImageHeight",
        flags:"IRWA",
        description:"Height for the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} and the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanFalseImage')}\n Note: If ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} is unset, the ${isc.DocUtils.linkForRef('attr:CheckboxItem.checkedImage')}\n will be used to indicate a true value in a boolean field. In this case this property is\n ignored in favor of ${isc.DocUtils.linkForRef('attr:CheckboxItem.valueIconHeight')}.",
        name:"booleanImageHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "method:ImgButton.setIcon":{
        ref:"method:ImgButton.setIcon",
        flags:"",
        description:" Change the icon being shown next to the title text.\n",
        name:"setIcon",
        seeAlso:[],
        params:[
            {description:"URL of new icon", optional:false, name:"icon", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "type:PromptStyle":{
        ref:"type:PromptStyle",
        values:[
            {description:"Displays a centered modal prompt with text specified by                 ${isc.DocUtils.linkForRef('attr:RPCRequest.prompt')}",
             value:"\"dialog\""},
            {description:"Changes the current cursor to the style specified by                 ${isc.DocUtils.linkForRef('attr:RPCRequest.promptCursor')}",
             value:"\"cursor\""}
        ],
        description:null,
        name:"PromptStyle",
        seeAlso:[],
        type:"type"
    },
    "class:XJSONDataSource":{
        methods:[],
        ref:"class:XJSONDataSource",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:[],
        description:"A DataSource preconfigured to use the ${isc.DocUtils.linkForRef('type:RPCTransport','\"scriptInclude\"')} transport\n for cross-domain calls to JSON services, such as those provided by Yahoo!.  \n <P>\n To use this DataSource, provide the URL of the service as ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')}, and\n provide ${isc.DocUtils.linkForRef('attr:DataSource.fields','fields')} that describe the structure of the data you want to\n extract from the service's response.\n <P>\n ${isc.DocUtils.linkForRef('attr:DataSource.recordXPath')} and ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')} can be used to extract\n data from the JSON structure returned by the service.  See\n ${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-Side Data Integration')} for an overview of how to\n control what parts of the JSON structure are included in the ${isc.DocUtils.linkForRef('class:DSResponse')} object, and\n hence provided to ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}s that are bound to this DataSource.\n <P>\n This XJSONDataSource is really a subclass of DataSource with just a few property settings:\n <pre>\n    dataFormat : \"json\",\n    dataTransport : \"scriptInclude\"\n    callbackParam : \"callback\"\n </pre>\n <P>\n If you are also writing the server side code to respond to requests from this DataSource,\n see the \n <a href='http://developer.yahoo.net/common/json.html#callbackparam' onclick=\"window.open('http://developer.yahoo.net/common/json.html#callbackparam');return false;\">tutorial provided by Yahoo!</a> \n for a good overview of how this transport mechanism works.  Note, as indicated in the\n tutorial above, the server is responsible for writing out not just the data, but also a\n JavaScript function call that tells the client that the response has arrived.  The client\n passes the name of the function to call as the \"callback\" URL parameter.\n <P>\n NOTE: if you use this DataSource to contact Yahoo web services, remember to include\n output=json in the dataURL, as well as a\n <a href='http://developer.yahoo.net/' onclick=\"window.open('http://developer.yahoo.net/');return false;\">Yahoo developer ID</a>.",
        name:"XJSONDataSource",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('jsonYahooWebServices')}"]
    },
    "attr:SelectItem.optionDataSource":{
        ref:"attr:SelectItem.optionDataSource",
        flags:"IRA",
        description:"If set, this FormItem will derive data to show in the PickList by fetching records from\n the specified <code>optionDataSource</code>.  The fetched data will be used as a\n ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} by extracting the\n ${isc.DocUtils.linkForRef('attr:FormItem.valueField','valueField')} and ${isc.DocUtils.linkForRef('attr:FormItem.displayField','displayField')} in\n the loaded records, to derive one valueMap entry per record loaded from the\n optionDataSource.  Multiple fields from the fetched data may be shown in the pickList by\n setting ${isc.DocUtils.linkForRef('attr:SelectItem.pickListFields')}.\n <P>\n The data will be retrieved via a \"fetch\" operation on the DataSource, passing the \n ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')} (if set) as criteria, and passing\n ${isc.DocUtils.linkForRef('attr:SelectItem.optionFilterContext')} (if set) as DSRequest properties.\n <P>\n The fetch will be triggered when the pickList is first shown, or, you can set\n ${isc.DocUtils.linkForRef('attr:SelectItem.autoFetchData','autoFetchData:true')} to fetch when the FormItem is\n first drawn.  \n <P>\n Note that providing an initial value when\n ${isc.DocUtils.linkForRef('attr:FormItem.fetchMissingValues','fetchMissingValues')} is enabled, or enabling\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption','defaultToFirstOption')}, can also cause a fetch to\n be initiated immediately upon form creation.  You can also call ${isc.DocUtils.linkForRef('method:PickList.fetchData')}\n at any time to manually trigger a fetch.\n <P>\n Data paging is automatically enabled if the optionDataSource supports it.  As the\n pickList is scrolled by the user, requests for additional data will be automatically\n issued.\n <P>\n For a pickList attached to a ComboBoxItem, new fetches are issued as the user types,\n with criteria set as described under ${isc.DocUtils.linkForRef('method:ComboBoxItem.getPickListFilterCriteria')}.\n If your dataSource is not capable of filtering results by search criteria (eg the\n dataSource is backed by an XML flat file), you can set ${isc.DocUtils.linkForRef('attr:SelectItem.filterLocally')} to have the\n entire dataset loaded up front and filtering performed in the browser.  This disables\n data paging.\n <P>\n Note that if a normal, static ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} is <b>also</b> specified\n for the field (either directly in the form item or as part of the field definition in\n the dataSource), it will be preferred to the data derived from the optionDataSource for\n whatever mappings are present.",
        name:"optionDataSource",
        seeAlso:[],
        valueType:"DataSource | String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "method:DetailViewer.getCellCSSText":{
        ref:"method:DetailViewer.getCellCSSText",
        flags:"",
        description:" Return CSS text for styling this cell, which will be applied in addition to the CSS class\n for the cell, as overrides.\n <p>\n \"CSS text\" means semicolon-separated style settings, suitable for inclusion in a CSS\n stylesheet or in a STYLE attribute of an HTML element.\n\n",
        name:"getCellCSSText",
        seeAlso:[],
        params:[
            {description:"actual value of this cell", optional:false, name:"value", defaultValue:null,
             type:"any"},
            {description:"field object for this cell", optional:false, name:"field", defaultValue:null,
             type:"DetailViewerField"},
            {description:"record object for this cell", optional:false, name:"record",
             defaultValue:null, type:"Record"},
            {description:"the viewer instance to which this cell belongs", optional:false,
             name:"viewer", defaultValue:null, type:"DetailViewer"}
        ],
        returns:{description:"CSS text to add to this cell", type:"CSSText"},
        type:"method",
        definingClass:"class:DetailViewer",
        groups:["apperance"]
    },
    "attr:HibernateBrowser.excludeSubstring":{
        ref:"attr:HibernateBrowser.excludeSubstring",
        flags:"IR",
        description:"If set, specifies a substring which must NOT exist in an entity name for it to be included in\n this HibernateBrowser.  If this property is set to a List of strings, entity names are\n excluded if they match any one of the strings.  The comparison is case-insensitive.\n <p>\n For example, <code>excludeSubstring: [\"Order\", \"inv\"]</code> would exclude all the following \n entity names: \"ORDERS\", \"Inventory\", \"client_invoicing\"\n <p>\n Note that if you specify both include and exclude criteria and they conflict (ie,\n according to the criteria you set, an entity should be both included and excluded), \n exclude wins.",
        name:"excludeSubstring",
        seeAlso:[],
        valueType:"String | List of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "attr:RadioGroupItem.vertical":{
        ref:"attr:RadioGroupItem.vertical",
        flags:"IRW",
        description:"True == display options vertically, false == display in a single row",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RadioGroupItem",
        groups:["appearance"]
    },
    "attr:ListGrid.listEndEditAction":{
        ref:"attr:ListGrid.listEndEditAction",
        flags:"IRW",
        description:"If the user is editing the last record in this listGrid, and attempts to navigate \n beyond the last row either by tabbing off the last editable field, or using the down\n arrow key, this property determines what action to take:<ul>\n <li>\"next\": start editing a new record at the end of the list.\n <li>\"done\": hide the editor\n <li>\"stop\": leave focus in the cell being edited\n <li>\"none\": no action\n </ul>",
        name:"listEndEditAction",
        seeAlso:[],
        valueType:"RowEndEditAction",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('enterNewRows')}"],
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:TabSet.showPaneContainerEdges":{
        ref:"attr:TabSet.showPaneContainerEdges",
        flags:"IRWA",
        description:"Should the paneContainer for this tabset show ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','edges')}.",
        name:"showPaneContainerEdges",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "method:TreeGrid.folderClosed":{
        ref:"method:TreeGrid.folderClosed",
        flags:"",
        description:"\n This method is called when a folder is closed either via the user manipulating the\n expand/collapse control in the UI or via ${isc.DocUtils.linkForRef('method:TreeGrid.closeFolder')}.  You can return\n <code>false</code> to cancel the close.\n\n",
        name:"folderClosed",
        seeAlso:[],
        params:[
            {description:"the folder (record) that is being closed", optional:false, name:"node",
             defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"false to cancel the close, true to all it to proceed", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:DataSource.getTypeOperators":{
        ref:"method:DataSource.getTypeOperators",
        flags:"",
        description:" Get the list of ${isc.DocUtils.linkForRef('type:OperatorId')}s available on this DataSource for the given ${isc.DocUtils.linkForRef('type:FieldType')}.\n <P>\n If ${isc.DocUtils.linkForRef('method:DataSource.setTypeOperators')} has been called for this DataSource and Fieldtype, returns that\n list, otherwise, returns the set of valid operators for the ${isc.DocUtils.linkForRef('type:FieldType')} as specified by \n ${isc.DocUtils.linkForRef('attr:SimpleType.validOperators')}, otherwise, the system-wide set of valid operators for the\n type as registered via ${isc.DocUtils.linkForRef('classMethod:DataSource.addSearchOperator')}.\n \n",
        name:"getTypeOperators",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"typeName", defaultValue:null, type:"FieldType"}
        ],
        returns:{description:"available Operators", type:"Array of OperatorId"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "method:List.first":{
        ref:"method:List.first",
        flags:"",
        description:" Return the first item in this list\n\n",
        name:"first",
        seeAlso:[],
        returns:{description:"first item in the list", type:"any"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "classMethod:RPCManager.clearTransaction":{
        ref:"classMethod:RPCManager.clearTransaction",
        flags:"A",
        description:" Erase all client-side record of a transaction, such that any response from the server\n will be ignored.\n <P>\n A transaction means a batch of one or more RPCRequests that have already been sent to\n the server via ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendQueue')}.\n <P>\n You can retrieve the id of the current transaction, if there is one, by \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.getCurrentTransactionId','getCurrentTransactionId()')} before the \n transaction is sent.\n \n",
        name:"clearTransaction",
        seeAlso:[
            {ref:"group:relogin", description:null}
        ],
        params:[
            {description:"id of the transaction to be cleared", optional:false, name:"transactionNum",
             defaultValue:null, type:"id"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "method:ListGrid.focusInFilterEditor":{
        ref:"method:ListGrid.focusInFilterEditor",
        flags:"",
        description:" If the filter editor (${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')}) is visible for this grid, \n this method will explictly put focus into the specified field in the filter editor.\n",
        name:"focusInFilterEditor",
        seeAlso:[],
        params:[
            {description:"Name of the field to put focus into. If unspecified focus will go                             to the first field in the editor",
             optional:true, name:"fieldName", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "method:ListGrid.unsort":{
        ref:"method:ListGrid.unsort",
        flags:"",
        description:" Turn sorting off, typically because data has changed and is no longer sorted.\n <p>\n Calling <code>unsort()</code> disables visual indication of which column is the sort column,\n and calls <code>unsort()</code> on the underlying dataset.\n <P>\n Note that a grid viewing a paged dataset may not be able to support <code>unsort()</code>\n because the sort order is what establishes the row numbering that allows data to be\n fetched in batches.\n <P>\n <code>unsort()</code> is automatically called when records are dropped or value of the\n sorted column is changed.\n\n",
        name:"unsort",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["sorting"]
    },
    "group:viewFile":{
        ref:"group:viewFile",
        refs:[],
        description:null,
        name:"viewFile",
        seeAlso:[],
        type:"group"
    },
    "attr:DynamicForm.wrapItemTitles":{
        ref:"attr:DynamicForm.wrapItemTitles",
        flags:"IRW",
        description:"Whether titles for form items should wrap.  If not specified, titles will wrap by\n default.  Can be overridden for individual items via ${isc.DocUtils.linkForRef('attr:FormItem.wrapTitle')}",
        name:"wrapItemTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "group:allowedOperations":{
        ref:"group:allowedOperations",
        title:"Allowed Operations",
        refs:["attr:Calendar.canCreateEvents", "attr:Calendar.canEditEvents", 
         "attr:Calendar.canDeleteEvents"],
        description:null,
        name:"allowedOperations",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Class.logFatal":{
        ref:"classMethod:Class.logFatal",
        flags:"",
        description:" Log a message at \"fatal\" priority\n \n",
        name:"logFatal",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "type:ObjectLiteral":{
        ref:"type:ObjectLiteral",
        description:"An \"Object literal\" is JavaScript shorthand for defining a JavaScript Object with a set\n of properties.  For example, code like this:\n <pre>\n    var request = {\n        actionURL : \"/foo.do\",\n        showPrompt:false\n    };</pre>\n .. is equivalent to ..\n <pre>\n    var request = new Object();\n    request.actionURL = \"/foo.do\";\n    request.showPrompt = false;</pre>\n In situations where a set of ${isc.DocUtils.linkForRef('type:Properties','properties')} may be passed to a\n method, the Object literal notation is much more compact.  For example:\n <pre>\n    isc.RPCManager.sendRequest({\n        actionURL : \"/foo.do\",\n        showPrompt:false\n    });</pre>\n <b>NOTE:</b> if you have a 'trailing comma' in an object literal, like so:\n <pre>\n    var request = {\n        actionURL : \"/foo.do\",\n        showPrompt:false, // TRAILING COMMA\n    };</pre>\n This is considered a syntax error by Internet Explorer, but not by Firefox.  This is by\n far the #1 cause of Internet Explorer-specific errors that do not occur in other\n browsers.  Pay special attention to this error, and, if you can, install the\n JSSyntaxScannerFilter into your development environment (as described in the\n ${isc.DocUtils.linkForRef('group:iscInstall','deployment instructions')}).",
        name:"ObjectLiteral",
        seeAlso:[],
        type:"type"
    },
    "attr:GridRenderer.emptyMessageStyle":{
        ref:"attr:GridRenderer.emptyMessageStyle",
        flags:"IRW",
        description:"The CSS style name applied to the emptyMessage string if displayed.",
        name:"emptyMessageStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["emptyMessage"]
    },
    "attr:CubeGrid.dataSource":{
        ref:"attr:CubeGrid.dataSource",
        flags:"IR",
        description:"Name of the DataSource to use to fetch CubeGrid data.\n <p>\n The DataSource should have a field named after each facetId.  The CubeGrid will submit\n requests for data as DataSource \"fetch\" operations that request \n ${isc.DocUtils.linkForRef('object:CellRecord','cellRecords')} only for currently visible area\n (including ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio','drawAheadRatio')}).  The ${isc.DocUtils.linkForRef('type:Criteria')} passed\n in each fetch operation will be a ${isc.DocUtils.linkForRef('type:FacetValueMap')} that corresponds to a rectangular\n swath of cells the CubeGrid needs data for, for example:<pre>\n      { region:\"west\", product:[\"chair\", \"table\"], timePeriod:\"Q1 2004\" }\n </pre>\n Note that in the criteria above, the Array value for the \"product\" facet indicates the\n CubeGrid needs cellRecords for both the \"chair\" and \"table\" facetValues.\n <P>\n ${isc.DocUtils.linkForRef('attr:CubeGrid.fixedFacetValues')}, if any, are always sent in each \"fetch\" operation.\n <P>\n Scrolling, expanding/collapsing tree facets, pivoting, and changing the currently\n visibile facets may all trigger data requests.\n <P>\n The CubeGrid will generally attempt to fetch a minimal set of data to fill the viewport,\n sending multiple fetch operations in a batch (via ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','queueing')})\n which minimally describe newly revealed areas without specifying each individual cell.\n The CubeGrid will automatically handle being provided <i>more</i> data than was asked\n for, so server-side fetch-ahead policies can be implemented without any client-side\n customization.\n <P>\n Note that the ${isc.DocUtils.linkForRef('attr:DataSource.serverType','SQL connector')} shipped with the SmartClient\n SDK is capable of responding to the CubeGrid's data requests without writing any custom\n server code.",
        name:"dataSource",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:ListGrid.fetchDelay":{
        ref:"attr:ListGrid.fetchDelay",
        flags:"IRWA",
        description:"If we're showing the filterEditor (${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')} is true), and we're\n re-filtering on every keypress (${isc.DocUtils.linkForRef('attr:ListGrid.filterOnKeypress')} is true), this \n property is the delay in milliseconds between the user changing the filter and the \n filter request being kicked off. If multiple changes are made to the filter \n within this fetch delay, only the most recent will actually cause a re-filter",
        name:"fetchDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"300",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "method:TileGrid.setFieldState":{
        ref:"method:TileGrid.setFieldState",
        flags:"",
        description:" Sets some presentation properties (visibility, width, userFormula and userSummary) of the \n grid fields based on the ${isc.DocUtils.linkForRef('type:listGridFieldState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to ${isc.DocUtils.linkForRef('method:TileGrid.getFieldState')}.\n\n",
        name:"setFieldState",
        seeAlso:[
            {ref:"method:TileGrid.getFieldState", description:null}
        ],
        params:[
            {description:"state to apply to the grid's fields.", optional:false, name:"fieldState",
             defaultValue:null, type:"listGridFieldState"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["viewState"]
    },
    "classMethod:Log.setPriority":{
        ref:"classMethod:Log.setPriority",
        flags:"",
        description:" Set the priority of messages that will be visible for this log category.\n <P>\n After calling setPriority, any messages logged to the given category whose priority is\n below the specified priority will not appear in the Log.\n\n",
        name:"setPriority",
        seeAlso:[
            {ref:"classMethod:Log.isEnabledFor",
             description:"to check whether a category would allow a log at a given priority"}
        ],
        params:[
            {description:"category name", optional:false, name:"category", defaultValue:null,
             type:"String"},
            {description:"priority level to set", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"},
            {description:"Optional ISC class or instance - if passed the priority will be set for logging \n      occurring on the class or instance only.",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "classAttr:ValidatorTypes.hasRelatedRecord":{
        ref:"classAttr:ValidatorTypes.hasRelatedRecord",
        flags:"IR",
        description:"Returns true if the record implied by a relation exists.  The relation can be \n  derived automatically from the ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')} attribute of \n  the field being validated, or you can specify it manually via \n  <code>validator.relatedDataSource</code> and <code>validator.relatedField</code>.\n  <p>\n  You can specify at DataSource level that this validator should be automatically \n  applied to all fields that specify a ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} -\n  see ${isc.DocUtils.linkForRef('attr:DataSource.validateRelatedRecords')}.\n  <p>\n  By default, validators of this type have ${isc.DocUtils.linkForRef('attr:Validator.serverOnly','serverOnly')} \n  set to <code>true</code> and do not run on the client.\n  <p>\n  Note that this validation is generally unnecessary for data coming from a UI.  The \n  typical UI uses a ${isc.DocUtils.linkForRef('class:SelectItem')} or ${isc.DocUtils.linkForRef('class:ComboBoxItem')} with an \n  ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')} for user entry, such that the user \n  can't accidentally enter a related record if that doesn't exist, and a typical SQL \n  schema will include constraints that prevent a bad insert if the user attempts to \n  circumvent the UI.  The primary purpose of declaring this validation explicitly is \n  to provide clear, friendly error messages for use cases such as ${isc.DocUtils.linkForRef('class:BatchUploader')}, \n  where values aren't individually chosen by the user.",
        name:"hasRelatedRecord",
        seeAlso:[
            {ref:"attr:DataSource.validateRelatedRecords", description:null}
        ],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:Menu.submenuImage":{
        ref:"attr:Menu.submenuImage",
        flags:"IRWA",
        description:"Default image to use for the submenu indicator. Valid src, width and height must be\n specified. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.<br>\n If ${isc.DocUtils.linkForRef('attr:Menu.submenuDirection')} is set to <code>\"left\"</code>, the image src will have\n the suffix <code>\"_left\"</code> appended to it.",
        name:"submenuImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:Canvas.resizeBy":{
        ref:"method:Canvas.resizeBy",
        flags:"",
        description:"			Resizes the widget, adding deltaX to its width and deltaY to its height (moves the right\n          and/or bottom sides of the widget).\n",
        name:"resizeBy",
        seeAlso:[],
        params:[
            {description:"amount to resize horizontally (may be negative)", optional:true,
             name:"deltaX", defaultValue:null, type:"number"},
            {description:"amount to resize vertically (may be negative)", optional:true, name:"deltaY",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"whether the component actually changed size", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('resize')}"],
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Array.first":{
        ref:"method:Array.first",
        flags:"",
        description:" Return the first item in this list\n\n",
        name:"first",
        seeAlso:[],
        returns:{description:"first item in the list", type:"any"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:RestDataSource.removeDataURL":{
        ref:"attr:RestDataSource.removeDataURL",
        flags:"IR",
        description:"Custom dataURL for remove type operations",
        name:"removeDataURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:RPCRequest.serverOutputAsString":{
        ref:"attr:RPCRequest.serverOutputAsString",
        flags:"IRWA",
        description:"Setting this flag makes the body of the HTTP response available as a String in the\n ${isc.DocUtils.linkForRef('attr:RPCRequest.callback')} as ${isc.DocUtils.linkForRef('attr:RPCResponse.data')}.  This is typically only useful if you\n are sending a request that will <b>not</b> be received by the SmartClient Java Server,\n however in that case, set ${isc.DocUtils.linkForRef('attr:RPCRequest.useSimpleHttp')}:true instead, which implies\n <code>serverOutputAsString:true</code>.\n <P>\n <code>serverOutputAsString:true</code> allows you to, for example, load the contents of\n static files off your webserver into a string for processing on the client with no server\n support.  The ${isc.DocUtils.linkForRef('attr:RPCRequest.actionURL')} must be in the same domain as the current page for\n this to work.\n <p>\n This feature relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.\n <p>\n Generally this API is used for either ${isc.DocUtils.linkForRef('group:nonJavaBackend','non-Java backends')} \n or for advanced usage such as content that requires processing before it can be used in\n SmartClient components (such as client-side web scraping).  Note that SmartClient provides\n higher-level APIs for loading common types of data, see eg ${isc.DocUtils.linkForRef('class:HTMLFlow')} for HTML content,\n ${isc.DocUtils.linkForRef('class:ViewLoader')} for loading SmartClient components, ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXML')} for loading\n XML, ${isc.DocUtils.linkForRef('attr:RPCRequest.evalResult')} for loading <a href='http://www.json.org/' onclick=\"window.open('http://www.json.org/');return false;\">JSON</a>, and\n ${isc.DocUtils.linkForRef('class:DataSource')} for loading structured data in various formats.",
        name:"serverOutputAsString",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:FormItem.suppressValueIcon":{
        ref:"attr:FormItem.suppressValueIcon",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} is set, this property may be set to prevent the value\n icons from showing up next to the form items value",
        name:"suppressValueIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "method:Canvas.disable":{
        ref:"method:Canvas.disable",
        flags:"",
        description:" Disables this widget and any children and peers of this widget.\n",
        name:"disable",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "object:groupNode":{
        methods:[],
        ref:"object:groupNode",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/ListGrid"],
        attrs:["attr:groupNode.groupMembers", "attr:groupNode.groupTitle", "attr:groupNode.groupValue"],
        description:"An auto-generated subclass of ${isc.DocUtils.linkForRef('object:TreeNode')} representing the group nodes\n in a grouped listgrid.",
        name:"groupNode",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        type:"object",
        groups:["grouping"]
    },
    "attr:ComboBoxItem.filterWithValue":{
        ref:"attr:ComboBoxItem.filterWithValue",
        flags:"RA",
        description:"Read-only property set by the ComboBoxItem to indicate whether we should use the \n current typed-in value as part of the filter criteria returned by \n ${isc.DocUtils.linkForRef('method:ComboBoxItem.getPickListFilterCriteria')}.  You can check this flag in order to\n mimic the ComboBoxItem's default behavior if you provide a custom implementation of\n <code>getPickListFilterCriteria()</code>.",
        name:"filterWithValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"varies",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:FormulaBuilder.titleField":{
        ref:"attr:FormulaBuilder.titleField",
        flags:"IR",
        description:"TextItem that allows users to set the title for this field.",
        name:"titleField",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:OperationBinding.customValueFields":{
        ref:"attr:OperationBinding.customValueFields",
        flags:"IR",
        description:"Specifies, for this operationBinding only, a list of extra field names that should be included in\n the default ${isc.DocUtils.linkForRef('attr:OperationBinding.selectClause','selectClause')} SmartClient generates.  Here, \"extra\" refers to\n fields that would not normally be included because they are marked as \n ${isc.DocUtils.linkForRef('attr:DataSourceField.customSQL','customSQL')}.\n You can specify this property as either an array or a string containing a comma-separated list \n of field names (eg, \"foo, bar, baz\").\n <p>\n This property is only applicable to DataSources of type \"sql\".",
        name:"customValueFields",
        seeAlso:[
            {ref:"attr:OperationBinding.customCriteriaFields", description:null}
        ],
        valueType:"String or Array",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "classAttr:RPCManager.defaultPrompt":{
        ref:"classAttr:RPCManager.defaultPrompt",
        flags:"IRW",
        description:"If showPrompt is enabled for a given transaction, this is the defaultPrompt to be shown\n to the user in a modal dialog while the transaction occurs.\n May be overridden at the request level via ${isc.DocUtils.linkForRef('attr:RPCRequest.prompt')}.<br>\n More targetted default prompts are also supported for certain code-paths. See the following \n set of properties for details:<ul>\n <li>${isc.DocUtils.linkForRef('classAttr:RPCManager.removeDataPrompt')}</li>\n <li>${isc.DocUtils.linkForRef('classAttr:RPCManager.saveDataPrompt')}</li>\n <li>${isc.DocUtils.linkForRef('classAttr:RPCManager.fetchDataPrompt')}</li>\n </ul>",
        name:"defaultPrompt",
        seeAlso:[
            {ref:"classAttr:RPCManager.showPrompt", description:null},
            {ref:"classAttr:RPCManager.promptStyle", description:null},
            {ref:"classAttr:RPCManager.promptCursor", description:null},
            {ref:"attr:RPCRequest.showPrompt", description:null},
            {ref:"attr:RPCRequest.prompt", description:null},
            {ref:"attr:RPCRequest.promptStyle", description:null},
            {ref:"attr:RPCRequest.promptCursor", description:null}
        ],
        valueType:"string",
        defaultValue:"\"Contacting Server...\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["rpcPrompt", "i18nMessages"]
    },
    "group:mouseEvents":{
        ref:"group:mouseEvents",
        refs:["classMethod:EventHandler.getTarget", "classMethod:EventHandler.getDragTarget", 
         "classMethod:EventHandler.getX","classMethod:EventHandler.getY", "classMethod:EventHandler.leftButtonDown", 
         "classMethod:EventHandler.rightButtonDown"],
        name:"mouseEvents",
        seeAlso:[],
        type:"group"
    },
    "attr:SummaryBuilder.showHelpIcon":{
        ref:"attr:SummaryBuilder.showHelpIcon",
        flags:"IR",
        description:"Whether to show the help icon that appears after the ${isc.DocUtils.linkForRef('attr:SummaryBuilder.formulaField')}.",
        name:"showHelpIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:Splitbar.vSrc":{
        ref:"attr:Splitbar.vSrc",
        flags:"IR",
        description:"Base URL for the image if ${isc.DocUtils.linkForRef('attr:StretchImg.vertical')} is true and \n ${isc.DocUtils.linkForRef('attr:StretchImg.src')} is unset.",
        name:"vSrc",
        seeAlso:[
            {ref:"attr:StretchImg.src", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["appearance"]
    },
    "attr:TileLayout.tileVMargin":{
        ref:"attr:TileLayout.tileVMargin",
        flags:"IR",
        description:"Vertical margin in between tiles.  See ${isc.DocUtils.linkForRef('attr:TileLayout.tileMargin')}.",
        name:"tileVMargin",
        setter:["setTileVMargin"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "method:StatefulCanvas.deselect":{
        ref:"method:StatefulCanvas.deselect",
        flags:"",
        description:" Deselect this object.\n",
        name:"deselect",
        seeAlso:[],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "method:CubeGrid.deselectFacetValue":{
        ref:"method:CubeGrid.deselectFacetValue",
        flags:"",
        description:"\n  deselect the header for a given facet value\n\n  <br><i>methodType</i> action\n",
        name:"deselectFacetValue",
        seeAlso:[],
        params:[
            {description:"ID of facet", optional:false, name:"facetId", defaultValue:null, type:"string"},
            {description:"ID of facetValue to select", optional:false, name:"facetValueId",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:DynamicForm.itemChanged":{
        ref:"method:DynamicForm.itemChanged",
        flags:"",
        description:" Handler fired when there is a changed() event fired on a FormItem within this form.\n <P>\n Fires after the change() handler on the FormItem itself, and only if the item did not\n cancel the change event and chooses to allow it to propagate to the form as a whole. \n\n",
        name:"itemChanged",
        seeAlso:[],
        params:[
            {description:"the FormItem where the change event occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"new value for the FormItem", optional:false, name:"newValue",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "method:TabSet.tabIconClick":{
        ref:"method:TabSet.tabIconClick",
        flags:"",
        description:" Method fired when the user clicks the icon for a tab, as specified via ${isc.DocUtils.linkForRef('attr:Tab.icon')}.\n <P>\n Default behavior will fire <code>icon.click()</code> if specified, with two parameters\n <code>tab</code> (a pointer to the tab object and <code>tabSet</code> a pointer to the tabSet\n instance.\n",
        name:"tabIconClick",
        seeAlso:[],
        params:[
            {description:"with click handler being fired", optional:false, name:"tab",
             defaultValue:null, type:"Tab"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:ColorPicker.crosshairImageURL":{
        ref:"attr:ColorPicker.crosshairImageURL",
        flags:"IR",
        description:"The location of the crosshair image file",
        name:"crosshairImageURL",
        seeAlso:[],
        valueType:"text",
        defaultValue:"\"[SKIN]ColorPicker/crosshair.png\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:ColumnTree.getColumnProperties":{
        ref:"method:ColumnTree.getColumnProperties",
        flags:"A",
        description:" Additional properties to apply to the ListGrid that will show the indicated column.\n Returns nothing by default. This method can be overrideen to allow, for example, different \n styling, icons, or row heights per column.\n",
        name:"getColumnProperties",
        seeAlso:[],
        params:[
            {description:"parent node for the nodes to be shown in the column", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"index of the column", optional:false, name:"colNum", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"properties to be applied to the column", type:"ListGrid Properties"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "attr:TextAreaItem.height":{
        ref:"attr:TextAreaItem.height",
        flags:"IRW",
        description:"default height of this item",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["appearance"]
    },
    "attr:RichTextItem.startRow":{
        ref:"attr:RichTextItem.startRow",
        flags:"IRW",
        description:"By default RichTextItems take up an entire row",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RichTextItem"
    },
    "attr:DataSource.inheritsFrom":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.inheritsFrom",
        description:"ID of another DataSource this DataSource inherits its ${isc.DocUtils.linkForRef('attr:DataSource.fields')} from.\n <P>\n Local fields (fields defined in this DataSource) are added to inherited fields \n to form the full set of fields.  Fields with the same name are merged in the same way\n that ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields','databound component fields')} are merged with\n DataSource fields.\n <P>\n The default order of the combined fields is new local fields first (including any fields\n present in the parent DataSource which the local DataSource re-declares), then parent\n fields.  You can set ${isc.DocUtils.linkForRef('attr:DataSource.useParentFieldOrder')} to instead use the parent's field\n order, with new local fields appearing last.  You can set ${isc.DocUtils.linkForRef('attr:DataSource.showLocalFieldsOnly')} to\n have all non-local fields hidden.\n <P>\n Note that <b>only fields are inherited</b> - other properties such as dataURL and\n dataFormat are not.  You can use ordinary inheritance, that is, creating a subclass of\n DataSource, in order to share properties such as dataURL across a series of DataSources\n that also inherit fields from each other via <code>inheritsFrom</code>.\n <P>\n This feature can be used for:\n <ul>\n <li>creating a customized view (eg, only certain fields shown) which will be used by\n multiple ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databound components')}.\n <li>adding presentation-specific attributes to metadata that has been automatically\n derived from ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema','XML Schema')} or other metadata formats\n <li>modelling object subclassing and extension in server-side code and storage systems\n <li>modelling relational database joins, and the equivalents in other systems\n <li>creating hooks for others to customize your application in a maintainable way.  For\n example, if you have a dataSource \"employee\", you can create a dataSource\n \"customizedEmployee\" which inherits from \"employee\" but does not initially define any\n fields, and bind all ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databound components')} to\n \"customizedEmployee\".  Customizations of fields (including appearance changes, field\n order, new fields, hiding of fields, and custom validation rules) can be added to\n \"customizedEmployee\", so that they are kept separtely from the original field data and\n have the best possible chance of working with future versions of the \"employee\"\n dataSource.\n </ul>",
        name:"inheritsFrom",
        valueType:"DataSource ID",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('schemaChaining')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["fields"],
        definingClass:"class:DataSource"
    },
    "method:GridRenderer.setColumnWidths":{
        ref:"method:GridRenderer.setColumnWidths",
        flags:"",
        description:"\n Sets the width of all columns in the grid.\n\n",
        name:"setColumnWidths",
        seeAlso:[],
        params:[
            {description:"array of new widths - one for each column.", optional:false,
             name:"newWidths", defaultValue:null, type:"Array"}
        ],
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:FacetValue.isMinimizeValue":{
        ref:"attr:FacetValue.isMinimizeValue",
        flags:"IR",
        description:"Used to determine which facetValue is to be shown when the facet is minimized.\n <P>\n See ${isc.DocUtils.linkForRef('attr:CubeGrid.canMinimizeFacets')}.",
        name:"isMinimizeValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "method:DataSource.performCustomOperation":{
        ref:"method:DataSource.performCustomOperation",
        flags:"",
        description:" Perform a custom DataSource operation against this DataSource.  A custom operation is\n anything that is not one of the 4 CRUD operations - generally, something that is more \n complex than a fetch or an update to a single record.  <code>customOperation</code>s \n are an alternative to using ${isc.DocUtils.linkForRef('class:RPCRequest','RPCs')}; anything that can be sent to\n the server as a plain RPC can instead be framed as a DataSource \n <code>customOperation</code>.\n\n",
        name:"performCustomOperation",
        seeAlso:[],
        params:[
            {description:"the operation ID", optional:false, name:"operationId", defaultValue:null,
             type:"String"},
            {description:"data to pass to the server", optional:false, name:"data", defaultValue:null,
             type:"Record Properties"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "method:DynamicForm.saveData":{
        ref:"method:DynamicForm.saveData",
        flags:"",
        description:"\n Validate and then save the form's current values to the ${isc.DocUtils.linkForRef('class:DataSource')} this form is\n bound to.\n <p>\n If client-side validators are defined, they are executed first, and if any errors are\n found the save is aborted and the form will show the errors.\n <p>\n If client-side validation passes, a ${isc.DocUtils.linkForRef('class:DSRequest')} will be sent, exactly as though\n ${isc.DocUtils.linkForRef('method:DataSource.addData')} or ${isc.DocUtils.linkForRef('method:DataSource.updateData')} had been called with \n ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','the form's values')} as data.  The\n ${isc.DocUtils.linkForRef('attr:DSRequest.operationType')} will be either \"update\" or \"add\", depending on the\n current ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')}.\n <P>\n On either a client-side or server-side validation failure, validation errors will be\n displayed in the form.  Visible items within a DynamicForms will be redrawn to display\n errors. Validation failure occuring on hidden items, or DataSource fields with no \n associated form items may be handled via ${isc.DocUtils.linkForRef('method:DynamicForm.handleHiddenValidationErrors')}\n or ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')}.\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.validationURL','validationURL')} can be set to do validation against a\n different URL from where the form will ultimately save, as part of an incremental\n upgrade strategy for Struts and Struts-like applications.\n\n",
        name:"saveData",
        seeAlso:[],
        params:[
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('addOperation')}"],
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "attr:RPCRequest.httpHeaders":{
        ref:"attr:RPCRequest.httpHeaders",
        flags:"IRW",
        description:"HTTP headers to send, as a Object mapping Header name -> Header value, eg<br>\n { \"Content-Type\" : \"text/xml\" }\n <P>\n Valid with the xmlHttpRequest ${isc.DocUtils.linkForRef('attr:RPCRequest.transport','transport')} only.",
        name:"httpHeaders",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:ServerObject.lookupStyle":{
        ref:"attr:ServerObject.lookupStyle",
        flags:"IR",
        description:"Specifies the mechanism for locating the class instance on which to invoke the method.\n Valid values are as follows:\n <ul>\n <li>\"spring\": For use with the <a href='http://springframework.com' onclick=\"window.open('http://springframework.com');return false;\">Spring framework</a>.\n ${isc.DocUtils.linkForRef('attr:ServerObject.bean')} contains the name of the bean to invoke.  Which application context\n is used can be configured via web.xml (see the example web.xml in the SDK).\n <li>\"new\": A new instance of the class specified by ${isc.DocUtils.linkForRef('attr:ServerObject.className')} will be\n created and the DMI method will be invoked on that instance (unless the specified method is\n static, in which case no instance is created, but the class specified by\n ${isc.DocUtils.linkForRef('attr:ServerObject.className')} is still used).\n <li>\"factory\": A custom factory provides the class instance on which the DMI method is to be\n invoked.  In this case, ${isc.DocUtils.linkForRef('attr:ServerObject.className')} specifies the className of the factory\n that will provide the instance on which the DMI method is to be invoked.  The class\n specified by ${isc.DocUtils.linkForRef('attr:ServerObject.className')} must provide exactly one method named\n <code>create</code> that must return the class instance on which you wish the DMI method to\n be invoked.  Like the DMI methods, the <code>create</code> method can request a standard set of\n values as arguments.  See ${isc.DocUtils.linkForRef('class:DMI')} for a list of available values.\n <li> \"attribute\": The instance on which the DMI method is to be invoked is looked up in the\n scope defined by ${isc.DocUtils.linkForRef('attr:ServerObject.attributeScope')} via the attribute name specified in\n ${isc.DocUtils.linkForRef('attr:ServerObject.attributeName')}.\n </ul>",
        name:"lookupStyle",
        seeAlso:[
            {ref:"attr:ServerObject.className", description:null},
            {ref:"attr:ServerObject.attributeName", description:null},
            {ref:"attr:ServerObject.attributeScope", description:null}
        ],
        valueType:"\"new\"|\"factory\"|\"attribute\"|\"spring\"",
        defaultValue:"\"new\"",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "method:SectionStack.setSectionTitle":{
        ref:"method:SectionStack.setSectionTitle",
        flags:"",
        description:" Changes the title of a SectionHeader.\n\n",
        name:"setSectionTitle",
        seeAlso:[],
        params:[
            {description:"ID or index of the section whose title you want to change", optional:false,
             name:"section", defaultValue:null, type:"String or Number"},
            {description:"new title for the SectionHeader", optional:false, name:"newTitle",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:ListGridField.validators":{
        ref:"attr:ListGridField.validators",
        flags:"IRW",
        description:"Array of ${isc.DocUtils.linkForRef('class:Validator')} objects for this field.  When the user edits cells in\n this field, these validators will be applied to the edited value.<br>\n Note: for databound listGrids, this property may be specified on the \n ${isc.DocUtils.linkForRef('class:DataSourceField')}, enabling both client and server side validation.",
        name:"validators",
        seeAlso:[
            {ref:"class:Validator", description:null},
            {ref:"attr:ListGridField.required", description:null}
        ],
        valueType:"array of Validator",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataValidation')}"],
        definingClass:"object:ListGridField",
        groups:["gridValidation"]
    },
    "method:ListGrid.cellContextClick":{
        ref:"method:ListGrid.cellContextClick",
        flags:"",
        description:" Called when a cell receives a contextclick event.\n\n",
        name:"cellContextClick",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('cellClicks')}"],
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "group:eclipseIntegration":{
        ref:"group:eclipseIntegration",
        title:"Eclipse Integration",
        refs:[],
        description:"If you're developing with Eclipse, you can use the SmartClient personality for Eclipse ATF\n    (<a href='http://eclipse.org/atf' onclick=\"window.open('http://eclipse.org/atf');return false;\">Ajax Toolkit Framework</a>).  Here's how to set it up:\n    <p>\n    1. If you haven't already, download and install ATF from \n      <a href='http://www.eclipse.org/atf/downloads/index.php' onclick=\"window.open('http://www.eclipse.org/atf/downloads/index.php');return false;\">http://www.eclipse.org/atf/downloads/index.php</a>.\n      The SmartClient personality has been tested against the ATF 0.1 Milestone Build, but is\n      also known to work with later releases.<p>\n    2. Download the SmartClient personality: \n        <a href='http://smartclient.com/product/SmartClient_ATF_Personality_1.0.zip' onclick=\"window.open('http://smartclient.com/product/SmartClient_ATF_Personality_1.0.zip');return false;\">SmartClient_ATF_Personality_1.0.zip</a><p>\n    3. Unzip the above archive into your eclipse installation directory.  This will create\n    three directories under the plugins directory: <i>com.smartClient.core_1.0.0,\n    com.smartClient.runtime_1.0.0, com.smartClient.ui_1.0.0</i>.<p>\n    4. For the next step, you will need to have downloaded and unzipped a copy of the\n    SmartClient Evaluation SDK, available from\n    <a href='http://smartclient.com/product/download.jsp' onclick=\"window.open('http://smartclient.com/product/download.jsp');return false;\">http://smartclient.com/product/download.jsp</a>.  You can of course also use\n    any other licensed copy of SmartClient.  From the SDK, copy the <i>isomorphic</i>\n    directory from smartclientRuntime into the\n    <i>eclipse/plugins/com.smartClient.runtime_1.0.0/resources</i> directory.\n    <p>\n    You're all set.  When you next start Eclipse, you'll be able to select ATF->SmartClient as\n    a new Project Facet during project creation or add it as a facet to an existing project.\n    You can then create new SmartClient Applications by right-clicking on your project and\n    selecting New->Other->SmartClient->SmartClient Application.  And you'll be able to use the\n    provided snippets to quickly create components.  If you have any problems, please post to\n    the <a href='http://forums.smartclient.com' onclick=\"window.open('http://forums.smartclient.com');return false;\">forums</a>.",
        name:"eclipseIntegration",
        seeAlso:[],
        type:"group"
    },
    "attr:HeaderSpan.valign":{
        ref:"attr:HeaderSpan.valign",
        flags:"IR",
        description:"Vertical alignment of the title of this headerSpan.\n <P>\n Defaults to listGrid.headerSpanVAlign if unset.",
        name:"valign",
        seeAlso:[],
        valueType:"vAlign",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "method:SectionItem.expandSection":{
        ref:"method:SectionItem.expandSection",
        flags:"",
        description:" Expands a section, showing all the items contained within the section.\n",
        name:"expandSection",
        seeAlso:[],
        type:"method",
        definingClass:"class:SectionItem"
    },
    "attr:DynamicForm.saveOnEnter":{
        ref:"attr:DynamicForm.saveOnEnter",
        flags:"IRW",
        description:"If <code>true</code>, when the user hits the Enter key while focussed in a text-item in\n this form, we automatically submit the form to the server using the \n ${isc.DocUtils.linkForRef('method:DynamicForm.submit')} method.",
        name:"saveOnEnter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "method:ListGrid.setCriteria":{
        ref:"method:ListGrid.setCriteria",
        flags:"",
        description:" Sets this component's filter criteria.\n Default implementation calls this.data.setCriteria()\n",
        name:"setCriteria",
        seeAlso:[],
        params:[
            {description:"new criteria to show", optional:false, name:"", defaultValue:null,
             type:"Criteria or AdvancedCriteria"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "type:XMLElement":{
        ref:"type:XMLElement",
        description:"An XMLElement represents one complete XML tag, including any subelements contained between\n the start and end tags.\n <P>\n XMLElement is a native object supplied directly by the browser.  The SmartClient-supported\n interfaces for this object include methods that take an XMLElement as an argument (such as\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectNodes')}).  If you want to retrieve XML data and display it in a\n SmartClient component, read about ${isc.DocUtils.linkForRef('group:clientDataIntegration','XML Data Binding')}.  To\n extract data as JavaScript Objects from XML, see ${isc.DocUtils.linkForRef('classMethod:XMLTools.toJS')}.  Direct\n manipulation of XMLElements objects is subject to cross-browser inconsistencies, bugs,\n memory leaks and performance issues.",
        name:"XMLElement",
        seeAlso:[],
        type:"type"
    },
    "attr:DataSourceField.rootValue":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSourceField.rootValue",
        description:"For a field that is a foreignKey establishing a tree relationship, what value indicates a\n root-level node.  Defaults to null.\n <P>\n Note that the rootValue may be overridden on a specific ResultTree instance by setting\n ${isc.DocUtils.linkForRef('attr:ResultTree.rootNode')}, or if the ResultTree is auto-generated by a ${isc.DocUtils.linkForRef('class:TreeGrid')},\n by setting ${isc.DocUtils.linkForRef('attr:TreeGrid.treeRootValue')}. This allows a component to navigate a subtree of\n the hierarchical data from this dataSource starting at a particular node.",
        name:"rootValue",
        valueType:"any",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('treeLoadXML')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["dataSourceRelations"],
        definingClass:"class:DataSourceField"
    },
    "attr:Canvas.canDrop":{
        ref:"attr:Canvas.canDrop",
        flags:"IRWA",
        description:"Indicates that this object can be dropped on top of other widgets. Only valid if \n      canDrag or canDragReposition is true.",
        name:"canDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:DataSourceField.canExport":{
        ref:"attr:DataSourceField.canExport",
        flags:"IR",
        description:"Dictates whether the data in this field be exported.  Explicitly setting \n <i>canExport</i> to false overrides the setting on any component-fields, such\n as ${isc.DocUtils.linkForRef('attr:ListGridField.canExport','ListGrid fields')}.",
        name:"canExport",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "classMethod:Page.goBack":{
        ref:"classMethod:Page.goBack",
        flags:"",
        description:" Go back in the browser's history.<br><br>\n\n If the history is empty and the window.opener is set, we assume we're a child window and just\n close the window.\n\n",
        name:"goBack",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:ColorPicker.modeToggleButton":{
        ref:"attr:ColorPicker.modeToggleButton",
        flags:"R",
        description:"\"More\"/\"Less\" button for the ColorPicker",
        name:"modeToggleButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ListGrid.showEmptyMessage":{
        ref:"attr:ListGrid.showEmptyMessage",
        flags:"IRW",
        description:"Indicates whether the text of the emptyMessage property should be displayed if no data is\n available.",
        name:"showEmptyMessage",
        seeAlso:[
            {ref:"attr:ListGrid.emptyMessage", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('emptyGrid')}"],
        definingClass:"class:ListGrid",
        groups:["emptyMessage"]
    },
    "class:DSResponse":{
        methods:[],
        ref:"class:DSResponse",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:DSResponse.clientContext", "attr:DSResponse.data", "attr:DSResponse.status", 
         "attr:DSResponse.errors","attr:DSResponse.startRow", "attr:DSResponse.endRow", "attr:DSResponse.totalRows", 
         "attr:DSResponse.invalidateCache"],
        description:"Response sent by the server in response to a ${isc.DocUtils.linkForRef('class:DSRequest','DataSource request')}.  Contains\n all the properties available on the basic ${isc.DocUtils.linkForRef('class:RPCResponse')}, in addition to the\n properties listed here.",
        name:"DSResponse",
        seeAlso:[],
        type:"class"
    },
    "group:dataType":{
        ref:"group:dataType",
        refs:["attr:DataSourceField.required", "attr:DataSourceField.valueMap", 
         "attr:DataSourceField.validators","attr:DataSourceField.length", "attr:SimpleType.valueMap"],
        name:"dataType",
        seeAlso:[],
        type:"group"
    },
    "attr:TabSet.rightEdgeSizes":{
        ref:"attr:TabSet.rightEdgeSizes",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeSizes</code>, \n <code>rightEdgeSizes</code>, <code>topEdgeSizes</code> and <code>bottomEdgeSizes</code> \n propertes allow the sizes of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge sizes where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"rightEdgeSizes",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:MenuButton.title":{
        ref:"attr:MenuButton.title",
        flags:"IRW",
        description:"Default title for the button.",
        name:"title",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Show Menu\"",
        type:"attr",
        definingClass:"class:MenuButton",
        groups:["i18nMessages"]
    },
    "group:defaults":{
        ref:"group:defaults",
        refs:[],
        name:"defaults",
        seeAlso:[],
        type:"group"
    },
    "method:ActiveXControl.getPluginHandle":{
        ref:"method:ActiveXControl.getPluginHandle",
        flags:"",
        description:"  Returns a handle to the element for this ISC ActiveX control object.\n\n",
        name:"getPluginHandle",
        seeAlso:[],
        type:"method",
        definingClass:"class:ActiveXControl"
    },
    "attr:Layout.members":{
        ref:"attr:Layout.members",
        flags:"IRW",
        getter:["getMembers"],
        description:"An array of canvases that will be contained within this layout. You can set the\n following properties on these canvases (in addition to the standard component\n properties):\n <ul>\n  <li>layoutAlign--specifies the member's alignment along the breadth axis; valid\n  values are \"top\", \"center\" and \"bottom\" for a horizontal layout and \"left\", \"center\"\n  and \"right\" for a vertical layout (see ${isc.DocUtils.linkForRef('attr:Layout.defaultLayoutAlign')} for default\n  implementation.)\n  <li>showResizeBar--set to true to show a resize bar (default is false)\n </ul>\n Height and width settings found on members are interpreted by the Layout according to\n the ${isc.DocUtils.linkForRef('attr:Layout.vPolicy','layout policy')}.",
        name:"members",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout"
    },
    "classAttr:Operators.iStartsWithTitle":{
        ref:"classAttr:Operators.iStartsWithTitle",
        flags:"IR",
        description:"Title for the \"iStartsWith\" operator",
        name:"iStartsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"starts with\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classMethod:Class.echoLeaf":{
        ref:"classMethod:Class.echoLeaf",
        flags:"",
        description:" Return a very short (generally less than 40 characters) string representation of any object,\n suitable for viewing by a developer for debugging purposes.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echoLeaf\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoLeaf",
        seeAlso:[
            {ref:"method:Class.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "classMethod:Page.getAppDir":{
        ref:"classMethod:Page.getAppDir",
        flags:"",
        description:" Returns the base URL of the application, which is the page URL minus the last non-directory \n path component.  For example, if the page is loaded from\n <code>http://foo.com/bar/zoo.jsp</code>, appDir will be <code>http://foo.com/bar/</code>.\n <P>\n If other page-wide URLs such as ${isc.DocUtils.linkForRef('classMethod:Page.setIsomorphicDir')} are specified as\n relative paths, they are considered relative to this URL.\n\n",
        name:"getAppDir",
        seeAlso:[],
        returns:{description:"URL for page-specific files.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files"]
    },
    "attr:ListGridRecord.canAcceptDrop":{
        ref:"attr:ListGridRecord.canAcceptDrop",
        flags:"IR",
        description:"When set to <code>false</code>, other records cannot be dropped on (i.e., inserted\n via drag and drop) immediately before this record.",
        name:"canAcceptDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "attr:TreeGrid.animateFolderSpeed":{
        ref:"attr:TreeGrid.animateFolderSpeed",
        flags:"IRW",
        description:"When animating folder opening / closing, this property designates the speed of the\n animation in pixels shown (or hidden) per second. Takes presidence over the \n ${isc.DocUtils.linkForRef('attr:TreeGrid.animateFolderTime')} property, which allows the developer to specify a\n duration for the animation rather than a speed.",
        name:"animateFolderSpeed",
        seeAlso:[
            {ref:"attr:TreeGrid.animateFolderTime", description:null}
        ],
        valueType:"number",
        defaultValue:"1000",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('animateTree')}"],
        definingClass:"class:TreeGrid",
        groups:["animation"]
    },
    "method:LoginDialog.register":{
        ref:"method:LoginDialog.register",
        flags:"",
        description:" Called if the user clicks on the ${isc.DocUtils.linkForRef('attr:LoginDialog.registrationItem','registration link')}\n on the login form. Implement this method to allow the user to register for a\n new account.\n",
        name:"register",
        seeAlso:[],
        params:[
            {description:"Current values of form fields", optional:false, name:"values",
             defaultValue:null, type:"Object"},
            {description:"Form on which the link was clicked", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:LoginDialog"
    },
    "attr:Mail.bcc":{
        ref:"attr:Mail.bcc",
        flags:"IR",
        description:"Comma-separated list of \"BCC\" recipients.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n If you specify a \"Bcc:\" line in the body of your email, recipient addresses will be parsed \n and this property will be ignored.",
        name:"bcc",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "method:ListGrid.resizeField":{
        ref:"method:ListGrid.resizeField",
        flags:"A",
        description:" Resize a particular field to a new width.\n\n",
        name:"resizeField",
        seeAlso:[],
        params:[
            {description:"Number of the field to resize", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"},
            {description:"New width of the field", optional:false, name:"newWidth", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Flashlet.classID":{
        ref:"attr:Flashlet.classID",
        flags:"IR",
        description:"This attribute specifies the clsid of the outer &lt;object&gt; tag.\n  <p>\n  The default classID is: \"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\"\n  <p>",
        name:"classID",
        seeAlso:[],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "attr:ListGrid.expansionMode":{
        ref:"attr:ListGrid.expansionMode",
        flags:"IRWA",
        description:"The ${isc.DocUtils.linkForRef('type:ExpansionMode')} for records in this grid.\n Default <code>null</code> value means no expansion.",
        name:"expansionMode",
        seeAlso:[],
        valueType:"ExpansionMode",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:BatchUploader.cancelButton":{
        ref:"attr:BatchUploader.cancelButton",
        flags:"IR",
        description:"Button that cancels the uncommitted upload",
        name:"cancelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "classAttr:RPCResponse.STATUS_LOGIN_REQUIRED":{
        ref:"classAttr:RPCResponse.STATUS_LOGIN_REQUIRED",
        flags:"R",
        description:"Indicates that a login is required before this RPCRequest can proceed.\n <P>\n Applications do not directly set this status code, instead, to trigger the relogin flow,\n return the loginRequiredMarker in the response sent by your server when login is\n required.  See the ${isc.DocUtils.linkForRef('group:relogin','Relogin Overview')} for details.",
        name:"STATUS_LOGIN_REQUIRED",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-7",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "class:FilterBuilder":{
        methods:["method:FilterBuilder.removeClause", "method:FilterBuilder.setTopOperator", 
         "method:FilterBuilder.getFieldOperators","method:FilterBuilder.getValueFieldProperties", "method:FilterBuilder.getCriteria", 
         "method:FilterBuilder.setCriteria","method:FilterBuilder.clearCriteria", "method:FilterBuilder.addCriterion", 
         "method:FilterBuilder.search","method:FilterBuilder.filterChanged"],
        ref:"class:FilterBuilder",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:FilterBuilder.dataSource", "attr:FilterBuilder.criteria", 
         "attr:FilterBuilder.saveOnEnter","attr:FilterBuilder.showFieldTitles", "attr:FilterBuilder.validateOnChange", 
         "attr:FilterBuilder.showRemoveButton","attr:FilterBuilder.removeButtonPrompt", "attr:FilterBuilder.removeButton", 
         "attr:FilterBuilder.showAddButton","attr:FilterBuilder.addButtonPrompt", "attr:FilterBuilder.addButton", 
         "attr:FilterBuilder.allowEmpty","attr:FilterBuilder.topOperator", "attr:FilterBuilder.radioOptions", 
         "attr:FilterBuilder.topOperatorAppearance","attr:FilterBuilder.radioOperatorForm", "attr:FilterBuilder.topOperatorForm", 
         "attr:FilterBuilder.defaultSubClauseOperator","attr:FilterBuilder.clauseStack", "attr:FilterBuilder.rangeSeparator", 
         "attr:FilterBuilder.showSubClauseButton","attr:FilterBuilder.subClauseButtonTitle", "attr:FilterBuilder.subClauseButtonPrompt", 
         "attr:FilterBuilder.subClauseButton","attr:FilterBuilder.bracket"],
        description:"A form that allows the user to input advanced search criteria, including operators on\n field values such as \"less than\", and sub-clauses using \"AND\" and \"OR\" operators.\n <P>\n A FilterBuilder produces an ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} object, which the ${isc.DocUtils.linkForRef('class:DataSource')}\n subsystem can use to filter datasets, including the ability to perform such filtering within\n the browser for datasets that are completely loaded.",
        name:"FilterBuilder",
        seeAlso:[],
        type:"class"
    },
    "attr:ColorItem.pickerIconSrc":{
        ref:"attr:ColorItem.pickerIconSrc",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n src of the picker icon image to be displayed.",
        name:"pickerIconSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/ColorPicker_icon.png\"",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "attr:DataSourceField.inapplicable":{
        ref:"attr:DataSourceField.inapplicable",
        flags:"IR",
        description:"For use in ${isc.DocUtils.linkForRef('group:componentSchema')}, a field ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom','inherited')} from\n another schema can be redeclared with this property set in order to indicate that the\n property should not be used.\n <P>\n This is primarily used to influence ${isc.DocUtils.linkForRef('group:visualBuilder')}.  For simple type properties,\n this avoids the property appearing in the Component Editor.  \n <P>\n For fields that hold subcomponents, this prevents inappropriate drag and drop.  For example,\n a custom class called <code>MyDialog</code> may automatically create a series of children,\n and not allow arbitrary other children to be added.  In this case, the inherited property\n ${isc.DocUtils.linkForRef('attr:Canvas.children')} should be marked inapplicable in order to prevent arbitrary\n components being dropped onto a <code>MyDialog</code> instance.",
        name:"inapplicable",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentSchema"]
    },
    "classMethod:RPCManager.queueSent":{
        ref:"classMethod:RPCManager.queueSent",
        flags:"",
        description:"\n This method is called by the RPCManager every time it sends a queue of requests to the \n server (note that if you not using queuing, the system simply sends queues containing\n just one request, so this API is valid regardless).<p>\n There is no default implementation of this method; it is simply an override point.  \n It is intended to be used by user code that needs to be notified when SmartClient sends \n requests to the server.  Note that the list of ${isc.DocUtils.linkForRef('class:RPCRequest')}s passed to this\n method is strictly <b>read-only</b>.\n\n",
        name:"queueSent",
        seeAlso:[],
        params:[
            {description:"The queue of ${isc.DocUtils.linkForRef('class:RPCRequest')}s that was sent",
             optional:false, name:"requests", defaultValue:null, type:"List of RPCRequest"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "classMethod:Class.setLogPriority":{
        ref:"classMethod:Class.setLogPriority",
        flags:"",
        description:" Set the priority of messages that will be visible for some log category, when logged on\n this Class or Instance object.<br>\n If called with no category, this priority will be applied to every logged message on this\n object<br>\n To set the visible log priority for some category across the entire page, use\n <code>isc.Log.setPriority()</code> instead.\n",
        name:"setLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "group:title":{
        ref:"group:title",
        refs:["method:DataSource.getAutoTitle", "classMethod:DataSource.getAutoTitle", 
         "attr:FormItem.showTitle","attr:FormItem.titleOrientation", "attr:FormItem.titleAlign", "attr:FormItem.wrapTitle", 
         "attr:FormItem.titleStyle","attr:FormItem.titleClassName", "attr:FormItem.titleErrorClassName"],
        name:"title",
        seeAlso:[],
        type:"group"
    },
    "classMethod:RPCManager.xmlHttpRequestAvailable":{
        ref:"classMethod:RPCManager.xmlHttpRequestAvailable",
        flags:"",
        description:"\n Returns true if the XMLHttpRequest object is available, false otherwise.  See\n ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information on when XMLHttpRequest parser may\n not available and what features are\n impacted as a result.\n\n",
        name:"xmlHttpRequestAvailable",
        seeAlso:[],
        returns:{description:"true if XMLHttpRequest is available, false otherwise.", type:"boolean"},
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "method:ListGrid.setHeaderSpanTitle":{
        ref:"method:ListGrid.setHeaderSpanTitle",
        flags:"",
        description:" Update the title of a ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpan')} dynamically.\n\n",
        name:"setHeaderSpanTitle",
        seeAlso:[],
        params:[
            {description:"name of the headerSpan, as specified via ${isc.DocUtils.linkForRef('attr:HeaderSpan.name')}.",
             optional:false, name:"name", defaultValue:null, type:"String"},
            {description:"new title for the headerSpan", optional:false, name:"newTitle",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "attr:TileLayout.dragDataAction":{
        ref:"attr:TileLayout.dragDataAction",
        flags:"IRW",
        description:"Indicates what to do with data dragged into another DataBoundComponent. See\n          DragDataAction type for details.",
        name:"dragDataAction",
        seeAlso:[],
        valueType:"DragDataAction",
        defaultValue:"Canvas.MOVE",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('gridsDragMove')}", 
         "${isc.DocUtils.linkForExampleId('gridsDragCopy')}"],
        definingClass:"class:TileLayout",
        groups:["dragging"]
    },
    "method:CubeGrid.removeFacet":{
        ref:"method:CubeGrid.removeFacet",
        flags:"",
        description:"\n Remove a facet from the current view, using a fixed value from that facet.  For example, remove\n the \"months\" facet from the view, collapsing to just January, or total for all months.\n\n",
        name:"removeFacet",
        seeAlso:[
            {ref:"method:CubeGrid.addFacet", description:null},
            {ref:"attr:CubeGrid.fixedFacetValues", description:null}
        ],
        params:[
            {description:"facetId to remove", optional:false, name:"facetId", defaultValue:null,
             type:"id"},
            {description:"New fixed value for the facet, to be added to cubeGrid.fixedFacetValues.  Default is the\n     rollup value for the facet.",
             optional:true, name:"fixedFacetValueId", defaultValue:null, type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "method:MultiSortPanel.sortChanged":{
        ref:"method:MultiSortPanel.sortChanged",
        flags:"",
        description:" Fired whenever the sort configuration changes.  The single parameter is an array of\n ${isc.DocUtils.linkForRef('object:SortSpecifier')}s that represent the list of sort-levels as they appear after \n whatever change has occurred.\n\n",
        name:"sortChanged",
        seeAlso:[],
        params:[
            {description:"The current sort configuration, after any changes", optional:false,
             name:"sortLevels", defaultValue:null, type:"Array of SortSpecifier"}
        ],
        type:"method",
        definingClass:"class:MultiSortPanel"
    },
    "type:clickMaskMode":{
        ref:"type:clickMaskMode",
        values:[
            {description:"When the mask recieves a click, it will fire its click action,                   and cancel the event, leaving the clickMask up.",
             value:"\"hard\""},
            {description:"When the mask recieves a click, it will fire its click action,                   then dismiss the clickMask and allow the event to proceed to its target.",
             value:"\"soft\""}
        ],
        description:"Passed as a parameter to ${isc.DocUtils.linkForRef('method:Canvas.showClickMask')} to determine the masks behavior\n when clicked.",
        name:"clickMaskMode",
        seeAlso:[],
        type:"type",
        groups:["clickMask"]
    },
    "attr:Calendar.data":{
        ref:"attr:Calendar.data",
        flags:"IRW",
        description:"A List of CalendarEvent objects, specifying the data to be used to populate the\n calendar.  \n <p>\n This property will typically not be explicitly specified for databound Calendars, where\n the data is returned from the server via databound component methods such as\n ${isc.DocUtils.linkForRef('method:Calendar.fetchData')}. In this case the data objects will be set to a \n ${isc.DocUtils.linkForRef('class:ResultSet','resultSet')} rather than a simple array.",
        name:"data",
        setter:["setData"],
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"List of CalendarEvent",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["data"]
    },
    "attr:WSRequest.data":{
        ref:"attr:WSRequest.data",
        flags:"IR",
        description:"Data to be serialized to XML to form the SOAP body.",
        name:"data",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WSRequest"
    },
    "type:Encoding":{
        ref:"type:Encoding",
        values:[
            {description:"normal form encoding (\"application/x-www-form-urlencoded\")",
             value:"DynamicForm.NORMAL"},
            {description:"form encoding for forms with INPUT file elements, that is, forms                              that upload files (\"multipart/form-data\")",
             value:"DynamicForm.MULTIPART"}
        ],
        description:"Form encodying types - these translate to Form ENCTYPE parameters.",
        name:"Encoding",
        seeAlso:[],
        type:"type",
        groups:["submitting"]
    },
    "attr:Canvas.accessKey":{
        ref:"attr:Canvas.accessKey",
        flags:"IRWA",
        description:"If specified this governs the accessKey for the widget.\n This should be set to a character - when a user hits Alt+[accessKey], or in Mozilla\n Firefox 2.0 and above, Shift+Alt+[accessKey], focus will be given to the widget in\n question.",
        name:"accessKey",
        setter:["setAccessKey"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "method:ListGrid.getRowErrors":{
        ref:"method:ListGrid.getRowErrors",
        flags:"",
        description:"  Returns any currently stored validation errors for this row in the following format:<br>\n  &nbsp;&nbsp;<code>{fieldName:[array of error messages], ...}</code>\n",
        name:"getRowErrors",
        seeAlso:[
            {ref:"method:ListGrid.getCellErrors", description:null}
        ],
        params:[
            {description:"index of row to check for validation errors.", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"object showing validation error arrays by field for the row passed in                       - if no validation errors stored for the row, null is returned.",
         type:"object"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:DataBoundComponent.showComplexFields":{
        ref:"attr:DataBoundComponent.showComplexFields",
        flags:"IRWA",
        description:"Whether to show fields of non-atomic types when a DataBoundComponent is given a\n DataSource but no <code>component.fields</code>.\n <p>\n If true, the component will show fields that declare a complex type, for example, a\n field 'shippingAddress' that declares type 'Address', where 'Address' is the ID of a\n DataSource that declares the fields of a shipping address (city, street name, etc).\n <P>\n Such fields may need custom formatters or editors in order to create a usable interface,\n for example, an Address field in a ListGrid might use a custom formatter to combine the\n relevant fields of an address into one column, and might use a pop-up dialog for\n editing.",
        name:"showComplexFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "method:Class.setDefaultLogPriority":{
        ref:"method:Class.setDefaultLogPriority",
        flags:"",
        description:" Set the default priority of logging for messages logged on this Class or Instance object.\n All categories for which there is no explicit, instance level logging priority set will\n log at this level on this object.<br>  \n To set the default visible log priority across the entire page, use\n <code>isc.Log.setDefaultPriority()</code> instead.\n",
        name:"setDefaultLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "method:PickList.getValueFieldName":{
        ref:"method:PickList.getValueFieldName",
        flags:"",
        description:" Getter method to retrieve the ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} for this item.\n If unset, default behavior will return the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this field.\n",
        name:"getValueFieldName",
        seeAlso:[],
        returns:{description:"fieldName to use a \"value field\" in records from this items               ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}",
         type:"string"},
        type:"method",
        definingClass:"interface:PickList",
        groups:["display_values"]
    },
    "method:SelectItem.defaultDynamicValue":{
        ref:"method:SelectItem.defaultDynamicValue",
        flags:"A",
        description:" Expression evaluated to determine the ${isc.DocUtils.linkForRef('attr:SelectItem.defaultValue')} when no value is\n provided for this item. To default to the first option use\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption')} instead.\n",
        name:"defaultDynamicValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:SelectItem"
    },
    "method:DynamicForm.setAction":{
        ref:"method:DynamicForm.setAction",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:DynamicForm.action','action')} for this form.\n",
        name:"setAction",
        seeAlso:[],
        params:[
            {description:"New action URL", optional:false, name:"action", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "classAttr:GroupingMessages.upcomingBeforeTitle":{
        ref:"classAttr:GroupingMessages.upcomingBeforeTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs before\n the current date.",
        name:"upcomingBeforeTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Before\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:ListGrid.chartColumn":{
        ref:"method:ListGrid.chartColumn",
        flags:"",
        description:" Chart a single column of data, with each cell value labelled by a value from another\n column.\n\n",
        name:"chartColumn",
        seeAlso:[],
        params:[
            {description:"name of the ListGridField to use as a data", optional:false,
             name:"dataColumn", defaultValue:null, type:"String"},
            {description:"name of the ListGridField to use as labels for data", optional:false,
             name:"labelColumn", defaultValue:null, type:"String"},
            {description:"properties to pass to the created chart", optional:true,
             name:"chartProperties", defaultValue:null, type:"Chart Properties"}
        ],
        returns:{description:"created Chart instance", type:"Chart"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ListGrid.isSeparatorProperty":{
        ref:"attr:ListGrid.isSeparatorProperty",
        flags:"IRW",
        description:"If <code>record[this.isSeparatorProperty]</code> is set for some record, the \n record will be displayed as a simple separator row.",
        name:"isSeparatorProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"isSeparator\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Number.stringify":{
        ref:"method:Number.stringify",
        flags:"",
        description:"\n Return this number as a string padded out to digits length.\n\n",
        name:"stringify",
        seeAlso:[],
        params:[
            {description:"Number of digits to pad to.  (Default is 2)", optional:true, name:"digits",
             defaultValue:"2", type:"number"}
        ],
        returns:{description:"Padded string version of the number", type:"string"},
        type:"method",
        examples:["var str = myNumberVar.stringify(2);"],
        definingClass:"object:Number",
        groups:["stringProcessing"]
    },
    "attr:TreeGrid.customIconProperty":{
        ref:"attr:TreeGrid.customIconProperty",
        flags:"IRW",
        description:"This property allows the developer to rename the \n ${isc.DocUtils.linkForRef('attr:TreeNode.icon','default node.icon')} property.",
        name:"customIconProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"icon\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "method:DataView.dataViewLoaded":{
        ref:"method:DataView.dataViewLoaded",
        flags:"A",
        description:"\n Executed when the dataView has loaded all dependencies (such as DataSources or WebServices).\n No default implementation.\n\n",
        name:"dataViewLoaded",
        seeAlso:[],
        type:"method",
        definingClass:"class:DataView"
    },
    "attr:TabSet.closeTabIconSize":{
        ref:"attr:TabSet.closeTabIconSize",
        flags:"IR",
        description:"Size in pixels of the icon for closing tabs, displayed when ${isc.DocUtils.linkForRef('attr:TabSet.canCloseTabs')} is true.",
        name:"closeTabIconSize",
        seeAlso:[],
        valueType:"int",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:HeaderSpan.height":{
        ref:"attr:HeaderSpan.height",
        flags:"IR",
        description:"Height of this headerSpan.  Defaults to ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpanHeight')}.",
        name:"height",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "method:Img.setSrc":{
        ref:"method:Img.setSrc",
        flags:"",
        description:" Changes the URL of this image and redraws it.\n <P>\n Does nothing if the src has not changed - if <code>src</code> has not changed but other\n state has changed such that the image needs updating, call ${isc.DocUtils.linkForRef('method:Img.resetSrc')} instead.\n\n",
        name:"setSrc",
        seeAlso:[],
        params:[
            {description:"new URL for the image", optional:false, name:"URL", defaultValue:null,
             type:"URL"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('loadImages')}"],
        definingClass:"class:Img",
        groups:["appearance"]
    },
    "type:OperatorId":{
        ref:"type:OperatorId",
        values:[
            {description:"exactly equal to", value:"\"equals\""},
            {description:"not equal to", value:"\"notEqual\""},
            {description:"Greater than", value:"\"greaterThan\""},
            {description:"Less than", value:"\"lessThan\""},
            {description:"Greater than or equal to", value:"\"greaterOrEqual\""},
            {description:"Less than or equal to", value:"\"lessOrEqual\""},
            {description:"Contains as sub-string (match case)", value:"\"contains\""},
            {description:"Starts with (match case)", value:"\"startsWith\""},
            {description:"Ends with (match case)", value:"\"endsWith\""},
            {description:"Contains as sub-string (case insensitive)", value:"\"iContains\""},
            {description:"Starts with (case insensitive)", value:"\"iStartsWith\""},
            {description:"Ends with (case insensitive)", value:"\"iEndsWith\""},
            {description:"Does not contain as sub-string (match case)", value:"\"notContains\""},
            {description:"Does not start with (match case)", value:"\"notStartsWith\""},
            {description:"Does not end with (match case)", value:"\"notEndsWith\""},
            {description:"Does not contain as sub-string (case insensitive)",
             value:"\"iNotContains\""},
            {description:"Does not start with (case insensitive)", value:"\"iNotStartsWith\""},
            {description:"Does not end with (case insensitive)", value:"\"iNotEndsWith\""},
            {description:"Regular expression match", value:"\"regexp\""},
            {description:"Regular expression match (case insensitive)", value:"\"iregexp\""},
            {description:"value is null", value:"\"isNull\""},
            {description:"value is non-null.  Note empty string (\"\") is non-null",
             value:"\"notNull\""},
            {description:"value is in a set of values.  Specify criterion.value as an Array",
             value:"\"inSet\""},
            {description:"value is not in a set of values.  Specify criterion.value as an Array",
             value:"\"notInSet\""},
            {description:"matches another field (specify fieldName as criterion.value)",
             value:"\"equalsField\""},
            {description:"does not match another field (specified fieldName as criterion.value)",
             value:"\"notEqualField\""},
            {description:"all subcriteria (criterion.criteria) are true", value:"\"and\""},
            {description:"all subcriteria (criterion.criteria) are false", value:"\"not\""},
            {description:"at least one subcriteria (criterion.criteria) is true", value:"\"or\""},
            {description:"shortcut for \"greaterThan\" + \"lessThan\" + \"and\".  Specify criterion.start         and criterion.end",
             value:"\"between\""},
            {description:"shortcut for \"greaterOrEqual\" + \"lessOrEqual\" + \"and\".  Specify         criterion.start and criterion.end",
             value:"\"betweenInclusive\""}
        ],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        description:"An operator is used as part of a ${isc.DocUtils.linkForRef('object:Criterion')} when specifying ${isc.DocUtils.linkForRef('object:AdvancedCriteria')}.\n <P>\n This list of operators indicates the set of operators built into SmartClient DataSources,\n which can be used for both client and server-side filtering.   You can extend the list of\n operators with ${isc.DocUtils.linkForRef('method:DataSource.addSearchOperator')}.",
        name:"OperatorId",
        seeAlso:[],
        type:"type",
        serverDS:"allowed",
        groups:["advancedFilter"]
    },
    "attr:DataBoundComponent.addFormulaFieldText":{
        ref:"attr:DataBoundComponent.addFormulaFieldText",
        flags:"IRW",
        description:"Text for a menu item allowing users to add a formula field",
        name:"addFormulaFieldText",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Add formula column...\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["i18nMessages"]
    },
    "attr:FormItem.align":{
        ref:"attr:FormItem.align",
        flags:"IRW",
        description:"Alignment of this item in its cell. Note that the alignment of content within this item\n can be separately controlled via ${isc.DocUtils.linkForRef('attr:FormItem.textAlign')} (typically only applies to items\n showing a \"textBox\", such as ${isc.DocUtils.linkForRef('class:TextItem','textItems')} or ${isc.DocUtils.linkForRef('class:SelectItem','selectItems')}).",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.LEFT",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:FormItem.change":{
        ref:"method:FormItem.change",
        flags:"",
        description:" Called when a FormItem's value is about to change as the result of user interaction.  This\n method fires after the user performed an action that would change the value of this field,\n but before the element itself is changed.  \n <P>\n Returning false cancels the change.  Note that if what you want to do is\n <b>transform</b> the user's input, for example, automatically change separator\n characters to a standard separator character, you should implement\n ${isc.DocUtils.linkForRef('method:FormItem.transformInput','transformInput')} rather than using a combination of\n change() and setValue() to accomplish the same thing.  Returning false from\n <code>change</code> is intended for rejecting input entirely, such as typing invalid\n characters.\n <p>\n Note that if you ask the form for the current value in this handler, you will get the old\n value because the change has not yet been commited.  The new value is available as a\n parameter to this method.\n\n",
        name:"change",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"The new value of the form item", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"The previous value of the form item", optional:false, name:"oldValue",
             defaultValue:null, type:"any"}
        ],
        returns:{description:"In your handler, return false to cancel the change, true to allow the change",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('fieldEnableDisable')}"],
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "method:Date.getWeek":{
        ref:"method:Date.getWeek",
        flags:"",
        description:"      Returns an integer containing the week number\n",
        name:"getWeek",
        seeAlso:[],
        returns:{description:"week number, starting with 1", type:"integer"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:Tree.openAll":{
        ref:"method:Tree.openAll",
        flags:"",
        description:"\n Open all nodes under a particular node.\n\n",
        name:"openAll",
        seeAlso:[],
        params:[
            {description:"node from which to open folders (if not specified, the root                              node is used)",
             optional:true, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('parentLinking')}"],
        definingClass:"class:Tree"
    },
    "attr:BlurbItem.clipValue":{
        ref:"attr:BlurbItem.clipValue",
        flags:"IRW",
        description:"If true, text that exceeds the specified size of the form item will be clipped",
        name:"clipValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:BlurbItem",
        groups:["appearance"]
    },
    "method:Dialog.closeClick":{
        ref:"method:Dialog.closeClick",
        flags:"",
        description:" Handles a click on the close button of this window. The default implementation\n hides the window and returns false to cancel bubbling.  Override this method if you want\n other actions to be taken.\n",
        name:"closeClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:Canvas.disabledCursor":{
        ref:"attr:Canvas.disabledCursor",
        flags:"IRWA",
        description:"Specifies the cursor image to display when the mouse pointer is\n      over this widget if this widget is disabled. It corresponds to the CSS cursor \n      attribute. See Cursor type for different cursors.",
        name:"disabledCursor",
        seeAlso:[],
        valueType:"Cursor",
        defaultValue:"Canvas.DEFAULT",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["cues"]
    },
    "group:CompoundFormItem_skinning":{
        ref:"group:CompoundFormItem_skinning",
        refs:[],
        description:"When skinning basic FormItems like SelectItem and TextItem, consider that compound form\n items like DateItem and ComboBox reuse simpler items like SelectItem and TextItem, so adding\n a border to SelectItem would also apply a border to each select item within DateItem.<br>\n To avoid such side-effects, if you want to add styling to all SelectItems used in your \n application, you can create an application-specific subclass like MySelectItem and apply \n properties there.<br>",
        name:"CompoundFormItem_skinning",
        seeAlso:[],
        type:"group"
    },
    "method:Calendar.addEvent":{
        ref:"method:Calendar.addEvent",
        flags:"",
        description:" Create a new event in this calendar instance.\n\n",
        name:"addEvent",
        seeAlso:[],
        params:[
            {description:"start date of event, or CalendarEvent Object", optional:false,
             name:"startDate", defaultValue:null, type:"Date or Object"},
            {description:"end date of event", optional:false, name:"endDate", defaultValue:null,
             type:"Date"},
            {description:"name of event", optional:false, name:"name", defaultValue:null, type:"String"},
            {description:"description of event", optional:false, name:"description", defaultValue:null,
             type:"String"},
            {description:"new values of additional fields to be updated", optional:false,
             name:"otherFields", defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:DynamicForm.showErrorIcons":{
        ref:"attr:DynamicForm.showErrorIcons",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "method:Array.sortByProperty":{
        ref:"method:Array.sortByProperty",
        flags:"",
        description:" Sort a list of objects by a given property of each item.\n <P>\n The optional normalizer, if passed as a function, is called for each item in the List, and\n should return whatever value should be used for sorting, which does not have to agree with\n the property value. By passing a normalizer function you can achieve any kind of sorting\n you'd like, including sorting by multiple properties.\n <P>\n NOTE: string sort is case INsensitive by default\n\n",
        name:"sortByProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to sort by", optional:false, name:"property",
             defaultValue:null, type:"string"},
            {description:"true == sort ascending, false == sort descending", optional:false, name:"up",
             defaultValue:null, type:"boolean"},
            {description:"May be specified as a function, with signature \n              <code>normalize(item, propertyName, context)</code>, where <code>item</code> is\n              a pointer to the item in the array, <code>propertyName</code> is the \n              property by which the array is being sorted, and <code>context</code> is the\n              arbitrary context passed into this method. Normalizer function should return\n              the value normalized for sorting.<br>\n              May also be specified as a ValueMap which maps property values to sortable values.",
             optional:true, name:"normalizer", defaultValue:null, type:"function or ValueMap"},
            {description:"Callers may pass an arbitrary context into the sort method, which                          will then be made available to the normalizer function",
             optional:true, name:"context", defaultValue:null, type:"any"}
        ],
        returns:{description:"the list itself", type:"List"},
        type:"method",
        definingClass:"object:Array",
        groups:["sorting"]
    },
    "classAttr:Validator.mustBeShorterThan":{
        ref:"classAttr:Validator.mustBeShorterThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>lengthRange</code> type validator\n returns false becaues the value passed in has more than <code>validator.max</code> characters.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeShorterThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be less than ${max} characters\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "type:DragDataAction":{
        ref:"type:DragDataAction",
        values:[
            {description:"Don't do anything, resulting in the same data being in both lists.",
             value:"\"none\""},
            {description:"Copy the data leaving the original in our list.", value:"Canvas.COPY"},
            {description:"Remove the data from this list so it can be moved into the other list.",
             value:"Canvas.MOVE"}
        ],
        description:"What do we do with data that's been dropped into another list?",
        name:"DragDataAction",
        seeAlso:[],
        type:"type",
        groups:["drag"]
    },
    "method:ResultSet.invalidateCache":{
        ref:"method:ResultSet.invalidateCache",
        flags:"A",
        description:" Manually invalidate this ResultSet's cache.\n <P>\n Generally a ResultSet will observe and incorporate updates to the DataSource that provides it's\n records, but when this is not possible, <code>invalidateCache()</code> allows manual cache\n invalidation.\n <P>\n <code>invalidateCache()</code> fires <code>dataChanged()</code>, which may cause components \n using this ResultSet to request new data for display, triggering server fetches.\n",
        name:"invalidateCache",
        seeAlso:[],
        type:"method",
        definingClass:"class:ResultSet"
    },
    "method:Menu.showContextMenu":{
        ref:"method:Menu.showContextMenu",
        flags:"",
        description:" Show this menu as a context menu, that is, immediately adjacent to the current mouse position.\n\n",
        name:"showContextMenu",
        seeAlso:[],
        returns:{description:"false == stop processing this event", type:"boolean"},
        type:"method",
        definingClass:"class:Menu",
        groups:["visibility"]
    },
    "attr:DateItem.pickerConstructor":{
        ref:"attr:DateItem.pickerConstructor",
        flags:"IR",
        description:"SmartClient class for the ${isc.DocUtils.linkForRef('attr:FormItem.picker')} autoChild displayed to allow the user\n to directly select dates.",
        name:"pickerConstructor",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"DateChooser\"",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "attr:FormulaBuilder.helpTextIntro":{
        ref:"attr:FormulaBuilder.helpTextIntro",
        flags:"IR",
        description:"Text that appears in the hover from the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.helpIcon')}, as a pre-amble to the list of\n available functions.",
        name:"helpTextIntro",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"For basic arithmetic, type in symbols (+-/%) directly.<P>The following functions are also available:\"",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["i18nMessages"]
    },
    "attr:TileGrid.autoFetchData":{
        ref:"attr:TileGrid.autoFetchData",
        flags:"IR",
        description:"If true, when this component is first drawn, automatically call <code>this.fetchData()</code>.\n Criteria for this fetch may be picked up from ${isc.DocUtils.linkForRef('attr:TileGrid.initialCriteria')}, and textMatchStyle may\n be specified via ${isc.DocUtils.linkForRef('attr:TileGrid.autoFetchTextMatchStyle')}.\n <P>\n <span style='color:red'>NOTE:</span> if <code>autoFetchData</code> is set, calling\n ${isc.DocUtils.linkForRef('method:TileGrid.fetchData')} before draw will cause two requests to be issued, one from the manual\n call to fetchData() and one from the autoFetchData setting.  The second request will use\n only ${isc.DocUtils.linkForRef('attr:TileGrid.initialCriteria')} and not any other criteria or settings from the first request.\n Generally, turn off autoFetchData if you are going to manually call fetchData() at any time.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"method:TileGrid.fetchData", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fetchOperation')}"],
        definingClass:"class:TileGrid",
        groups:["databinding"]
    },
    "attr:Window.hiliteHeaderSrc":{
        ref:"attr:Window.hiliteHeaderSrc",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Window.showHeaderBackground')} is true, this governs the URL of the image to \n use in the header's highlighted state when the window is ${isc.DocUtils.linkForRef('method:Window.flash','flashed')}",
        name:"hiliteHeaderSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]Window/headerGradient_hilite.gif\" | null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:GridRenderer.showHover":{
        ref:"attr:GridRenderer.showHover",
        flags:"RW",
        description:"If true, and canHover is also true, when the user hovers over a cell, hover text will pop up\n next to the mouse.  The contents of the hover is determined by ${isc.DocUtils.linkForRef('method:GridRenderer.cellHoverHTML')}.",
        name:"showHover",
        seeAlso:[
            {ref:"attr:GridRenderer.canHover", description:null},
            {ref:"method:GridRenderer.cellHoverHTML", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:GridRenderer.cellContextClick":{
        ref:"method:GridRenderer.cellContextClick",
        flags:"A",
        description:" Called when a cell receives a contextclick event.\n\n",
        name:"cellContextClick",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "class:ValidatorTypes":{
        methods:[],
        ref:"class:ValidatorTypes",
        classAttrs:["classAttr:ValidatorTypes.isBoolean", "classAttr:ValidatorTypes.isString", 
         "classAttr:ValidatorTypes.isInteger","classAttr:ValidatorTypes.isFloat", "classAttr:ValidatorTypes.requiredIf", 
         "classAttr:ValidatorTypes.matchesField","classAttr:ValidatorTypes.isOneOf", "classAttr:ValidatorTypes.integerRange", 
         "classAttr:ValidatorTypes.lengthRange","classAttr:ValidatorTypes.contains", "classAttr:ValidatorTypes.doesntContain", 
         "classAttr:ValidatorTypes.substringCount","classAttr:ValidatorTypes.regexp", "classAttr:ValidatorTypes.mask", 
         "classAttr:ValidatorTypes.dateRange","classAttr:ValidatorTypes.floatRange", "classAttr:ValidatorTypes.floatPrecision", 
         "classAttr:ValidatorTypes.isUnique","classAttr:ValidatorTypes.hasRelatedRecord", "classAttr:ValidatorTypes.serverCustom"],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Validator"],
        attrs:[],
        description:"The set of standard validator types available on all fields.<br>\n  To make use of some standard validator type for a field in a DataSource, or \n  DynamicForm instance, specify the <code>validators</code> property to an array \n  containing a validator definition where the <code>type</code> property is set to \n  the appropriate type.  \n  A custom error message can be specified for any validator type by setting the\n  <code>errorMessage</code> property on the validator definition object, and some\n  validator types make use of additional properties on the validator definition \n  object such as <code>max</code> or <code>min</code>.<br>\n  For example, to make use of the <code>integerRange</code> validator type:<br><br><code>\n  &nbsp;&nbsp;field:{<br>\n  &nbsp;&nbsp;&nbsp;&nbsp;validators:[<br>\n  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{type:\"integerRange\", min:1, max:100}<br>\n  &nbsp;&nbsp;&nbsp;&nbsp;]<br>\n  &nbsp;&nbsp;}\n  </code><br><br>",
        name:"ValidatorTypes",
        seeAlso:[],
        type:"class"
    },
    "attr:FilterBuilder.rangeSeparator":{
        ref:"attr:FilterBuilder.rangeSeparator",
        flags:"IR",
        description:"For operators that check that a value is within a range, text to show between the start and\n end input fields for specifying the limits of the range.",
        name:"rangeSeparator",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"and\"",
        type:"attr",
        definingClass:"class:FilterBuilder",
        groups:["i18nMessages"]
    },
    "method:GridRenderer.cellSelectionChanged":{
        ref:"method:GridRenderer.cellSelectionChanged",
        flags:"A",
        description:" Called when (cell-based) selection changes within this grid.\n\n",
        name:"cellSelectionChanged",
        seeAlso:[],
        params:[
            {description:"Array of cells whos selected state was modified.", optional:false,
             name:"cellList", defaultValue:null, type:"array"}
        ],
        returns:{description:"Returning false will prevent the GridRenderer styling from being updated                      to reflect the selection change.",
         type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["selection"]
    },
    "method:GridRenderer.rowHover":{
        ref:"method:GridRenderer.rowHover",
        flags:"A",
        description:" Called when the mouse hovers over a row if this.canHover is true. \n  Returning false will suppress the hover text from being shown if this.showHover is true.\n\n",
        name:"rowHover",
        seeAlso:[
            {ref:"attr:GridRenderer.canHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event (default behavior of showing the hover)",
         type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:ListGrid.shrinkForFreeze":{
        ref:"attr:ListGrid.shrinkForFreeze",
        flags:"IRWA",
        description:"If this list grid is showing any ${isc.DocUtils.linkForRef('attr:ListGridField.frozen','frozen')} fields, and a horizontal\n scrollbar is visible at the bottom of the liquid columns, should an equivalent scrollbar gap\n be left visible below the frozen columns?<br>\n Note that if set to <code>true</code> any backgroundColor or border applied to the ListGrid\n will show up below the bottom row of the frozen column(s).",
        name:"shrinkForFreeze",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["frozenFields"]
    },
    "attr:ListGridField.frozen":{
        ref:"attr:ListGridField.frozen",
        flags:"IR",
        description:"Whether this field should be \"frozen\" for the purposes of horizontal scrolling.  See\n ${isc.DocUtils.linkForRef('group:frozenFields')}.",
        name:"frozen",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["frozenFields"]
    },
    "method:Date.getShortMonthName":{
        ref:"method:Date.getShortMonthName",
        flags:"",
        description:" Return the abbreviated (up to 3 chars) name of the month for this date (Jan, Feb, etc)\n To modify the value returned by this method, set ${isc.DocUtils.linkForRef('classAttr:Date.shortMonthNames')}\n",
        name:"getShortMonthName",
        seeAlso:[],
        params:[
            {description:"Number of characters to return (Defaults to 3, can't be                                  longer than 3)",
             optional:false, name:"length", defaultValue:null, type:"number"}
        ],
        returns:{description:"Abbreviated month name (3 character string)", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:Canvas.setBackgroundImage":{
        ref:"method:Canvas.setBackgroundImage",
        flags:"",
        description:"			Sets the background to an image file given by newImage. This URL should be given as a\n          string relative to the image directory for the page (./images by default).\n",
        name:"setBackgroundImage",
        seeAlso:[],
        params:[
            {description:"new URL (local to Page image directory) for background image",
             optional:false, name:"newImage", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:Canvas.setPadding":{
        ref:"method:Canvas.setPadding",
        flags:"",
        description:" Set the CSS padding of this component, in pixels.  Padding provides space between the border\n and the component's contents.\n <P>\n This property sets the same thickness of padding on every side.  Differing per-side\n padding can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.\n <P>\n",
        name:"setPadding",
        seeAlso:[],
        params:[
            {description:"new padding in pixels", optional:false, name:"newPadding", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:StretchImgButton.setAutoFit":{
        ref:"method:StretchImgButton.setAutoFit",
        flags:"",
        description:" Setter method for the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.autoFit')} property. Pass in true or false to turn\n autoFit on or off. When autoFit is set to <code>false</code>, canvas will be resized to\n it's previously specified size.\n",
        name:"setAutoFit",
        seeAlso:[],
        params:[
            {description:"New autoFit setting.", optional:false, name:"autoFit", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton"
    },
    "attr:ListGrid.titleField":{
        ref:"attr:ListGrid.titleField",
        flags:"IRW",
        getter:["getTitleField"],
        description:"Best field to use for a user-visible title for an individual record from this grid.\n If ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')} is non null, this property may be specified on the \n dataSource instead.\n <p>\n If not explicitly set, titleField looks for fields named \"title\", \"name\", and \"id\" \n in that order.  If a field exists with one of those names, it becomes the titleField.  \n If not, then the first field is designated as the titleField.",
        name:"titleField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ListGrid.expansionDetailRelated":{
        ref:"attr:ListGrid.expansionDetailRelated",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:HLayout')} appearing in a record's expanded section\n when ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>detailRelated</code>.  \n This component contains two other autoChild components,\n a ${isc.DocUtils.linkForRef('class:DetailViewer')} for viewing fields from the record which are not already \n present in the grid and a separate embedded ${isc.DocUtils.linkForRef('class:ListGrid')} for displaying other \n data related to this record via record.detailDS.  See ${isc.DocUtils.linkForRef('attr:ListGrid.expansionDetails')} \n and ${isc.DocUtils.linkForRef('attr:ListGrid.expansionRelated')} for more information.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionDetailRelatedProperties</code> and \n <code>listGrid.expansionDetailRelatedDefaults</code>.\n <P>\n Note, however, that this is a multi-instance component (potentially one per record), \n so it is created using ${isc.DocUtils.linkForRef('method:Class.createAutoChild','createAutoChild()')} not \n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')}, and no default single instance is created by\n name on the grid.",
        name:"expansionDetailRelated",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:ListGrid.rowHasChanges":{
        ref:"method:ListGrid.rowHasChanges",
        flags:"",
        description:" If this listGrid can be edited, this method will return true if the row passed in has been\n edited, but the edits have not yet been saved to the ListGrid's data object.\n",
        name:"rowHasChanges",
        seeAlso:[],
        params:[
            {description:"index of row to check for changes", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true if the row has changes.", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:javaModuleDependencies":{
        ref:"group:javaModuleDependencies",
        title:"Java Module Dependencies",
        treeLocation:["Concepts/Deploying SmartClient"],
        refs:[],
        description:"The following is a description of what functionality is contained in each SmartClient JAR\n file, and which other SmartClient JARs and third party libraries are required to enable that\n functionality.  Note that actual filenames in WEB-INF/lib will typically contain the version\n number of the library, whereas the library names below simply list the base name.\n <ul>\n <li><b>isomorphic_core_rpc</b>: This is the core SmartClient module.  It provides the RPC, DMI,\n and DataSource support.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-cli<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-lang<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-collections<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-pool<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-codec<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isc-jakarta-oro<br>\n &nbsp;&nbsp;&nbsp;&nbsp;log4j<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-jxpath<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-httpclient<br>\n &nbsp;&nbsp;&nbsp;&nbsp;velocity<br>\n &nbsp;&nbsp;<u>Optionally Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;xercesImpl - if you're using JDK &lt; 1.5<br>\n &nbsp;&nbsp;&nbsp;&nbsp;xml-apis - if you're using JDK &lt; 1.5<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-fileupload - if you plan to use the upload form item with\n SmartClient processing.<br>\n &nbsp;&nbsp;&nbsp;&nbsp;mail - if you plan to use the Mail messaging feature<br>\n &nbsp;&nbsp;&nbsp;&nbsp;activation - if you plan to use the Mail messaging feature and you\n are using a JDK &lt; 1.6<br>\n </li>\n <li><b>isomorphic_web_services</b>: Web services examples only.  Contains code backing the\n SmartClientOperations.wsdl example.  Do not deploy in production.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;axis<br>\n &nbsp;&nbsp;&nbsp;&nbsp;axis-schema<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-discovery<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-logging<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jaxrpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;saaj<br>\n &nbsp;&nbsp;&nbsp;&nbsp;wsdl4j<br>\n </li>\n <li><b>isomorphic_tools</b>: Contains back-end logic for the \"Admin Console\" tool visible in\n the Developer Console, and also standalone from the SDK home page.  This is an SDK-only\n tool, not intended for production deployment, and further intended for rapid prototyping\n only.  Do not deploy this module to production, but you may use it in your development if\n you're using the SmartClient rapid prototyping SQL engine.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_sql<br>\n </li>\n <li><b>isomorphic_embedded_tomcat</b>: This contains the bootstrap class for the Embedded\n Tomcat engine shipped with the SmartClient SDK.  It is not intended to be used in any way\n other than via the start_embedded_tomcat.bat|sh|command scripts in the webRoot of the\n SDK.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;All JARs under WEB-INF/embeddedTomcat/lib<br>\n </li>\n <li><b>isomorphic_struts</b>: Contains the ValidationAction and ValidationFailureAction\n classes that implement RPC-based validation of DynamicForms using the Struts\n ValidatorPlugIn.  If you're not using Struts or if you don't need this particular feature of\n SmartClient, you do not need this module or its dependencies.  An example of this style of\n validation is available here: <a href='/examples/struts/forms/welcome.do' onclick=\"window.open('/examples/struts/forms/welcome.do');return false;\">/examples/struts/forms/welcome.do</a> - read the\n info on this page, and follow the \"Dynamic Form (With RPC-based Validation)\" Link for\n the actual example.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;struts<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-digester<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-beanutils<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-fileupload<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-logging<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-validator<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jakarta-oro<br>\n </li>\n <li><b>isomorphic_spring</b>: Required for ${isc.DocUtils.linkForRef('class:DMI')} dispatches to Spring beans (via\n ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')} : \"spring\").<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;spring<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-logging<br>\n </li>\n <li><b>isomorphic_examples</b>: Contains code for various SmartClient examples shown in the\n SDK.  Do not deploy to production.  The source code for the compiled classes in this module\n are present in the SDK in various locations and typically linked to directly from the\n examples that use them.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_sql<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_struts<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_spring<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_hibernate<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-logging<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jstl<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jakarta-taglib-standard<br>\n </li>\n <li><b>isomorphic_realtime_messaging</b>: Server support required for the SmartClient\n Realtime Messaging Module.  Install this if you're using this 'push' technology.  For more\n information, see: ${isc.DocUtils.linkForDocNode('Messaging QuickRef')}:<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n </li>\n <li><b>isomorphic_hibernate</b>: Contains support for Hibernate DataSources as described\n here: ${isc.DocUtils.linkForRef('group:hibernateIntegration')}.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_sql<br> \n &nbsp;&nbsp;&nbsp;&nbsp;hibernate3<br>\n &nbsp;&nbsp;&nbsp;&nbsp;commons-logging<br>\n &nbsp;&nbsp;&nbsp;&nbsp;dom4j<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jdbc2_0-stdext<br>\n &nbsp;&nbsp;&nbsp;&nbsp;asm<br>\n &nbsp;&nbsp;&nbsp;&nbsp;asm-attrs<br>\n &nbsp;&nbsp;&nbsp;&nbsp;jta<br>\n &nbsp;&nbsp;&nbsp;&nbsp;cglib<br>\n &nbsp;&nbsp;&nbsp;&nbsp;antlr<br>\n &nbsp;&nbsp;<u>Optionally Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;hsqldb - not required for linking, but needed for Hibernate to\n actually connect to databases of this type, and hsqldb is what's used in the SDK examples.\n Note that for production use you will need a JDBC driver for your database type.<br>\n </li>\n <li><b>isomorphic_sql</b>: The SmartClient SQLDataSource.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br> \n &nbsp;&nbsp;&nbsp;&nbsp;commons-dbcp<br> \n &nbsp;&nbsp;&nbsp;&nbsp;hsqldb<br> \n </li>\n <li><b>isomorphic_js_parser</b>: A parser capable of reading a JSON byte stream and creating\n an in-memory Java object structure to match.  Used by any mechanism that relies on\n JSON-style configuration.  Examples include FileAssembly definitions in JSON format, any use\n of the rulesFile with a URIRegexFilter (Java Servlet) or subclass.  Generally not\n required.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br> \n </li>\n <li><b>isomorphic_compression</b>: This is a part of the Network Performance Module.  The\n isomorphic_compression module is required for dynamic and static compression of various\n assets delivered to the browser.  For more information, see: \n ${isc.DocUtils.linkForRef('group:compression')}.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br> \n </li>\n <li><b>isomorphic_assembly</b>: This is part of the Network Performance Module.  The\n isomorphic_assembly module is required for file assembly and stripping.  For more\n information, see: ${isc.DocUtils.linkForRef('group:fileAssembly')}.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br> \n &nbsp;&nbsp;<u>Optionally Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_obfuscation - for code obfuscation<br> \n </li>\n <li><b>isomorphic_obfuscation</b>: This is part of the Network Performance Module.  The\n isomorphic_obfuscation module is required to enable obfuscation of code assembled by the\n File Assembly system.  For more information, see: \n ${isc.DocUtils.linkForRef('group:compression')}.<br>\n &nbsp;&nbsp;<u>Requires</u>:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;isomorphic_core_rpc<br> \n </li>\n </ul>",
        name:"javaModuleDependencies",
        seeAlso:[],
        type:"group"
    },
    "attr:DynamicForm.hiliteRequiredFields":{
        ref:"attr:DynamicForm.hiliteRequiredFields",
        flags:"IRW",
        description:"Indicates whether the titles of required items in this form should use the special\n prefix and suffix specified by the next two properties, instead of the standard\n prefix and suffix.",
        name:"hiliteRequiredFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "attr:ListGridField.align":{
        ref:"attr:ListGridField.align",
        flags:"IRW",
        description:"Horizontal alignment for field's column header: \"left\", \"right\"\n or \"center\". Applied to the column header title and cells by default. A separate\n alignment for cells can be specified via ${isc.DocUtils.linkForRef('attr:ListGridField.cellAlign')}.<br>\n If null, values are left-aligned. If this field is editable, the\n alignment of cells in the body will also be reflected in any editors for the field.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "attr:Canvas.mouseStillDownDelay":{
        ref:"attr:Canvas.mouseStillDownDelay",
        flags:"IRWA",
        description:"Amount of time (in msec) between 'mouseStillDown' events for this object",
        name:"mouseStillDownDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["events"]
    },
    "attr:FacetValue.title":{
        ref:"attr:FacetValue.title",
        flags:"IRW",
        description:"user-visible title of this facetValue.  Shown on the field header.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:ListGrid.canReorderRecords":{
        ref:"attr:ListGrid.canReorderRecords",
        flags:"IRW",
        description:"Indicates whether records can be reordered by dragging within this listGrid.",
        name:"canReorderRecords",
        seeAlso:[
            {ref:"attr:ListGridRecord.canDrag", description:null},
            {ref:"attr:ListGridRecord.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}", 
         "${isc.DocUtils.linkForExampleId('gridsDragReorder')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "attr:DateChooser.baseWeekdayStyle":{
        ref:"attr:DateChooser.baseWeekdayStyle",
        flags:"IRW",
        description:"Base css style applied to weekdays. Will have \"Over\", \"Selected\" and \"Down\"\n suffix appended as the user interacts with buttons.  Defaults to ${isc.DocUtils.linkForRef('attr:DateChooser.baseButtonStyle')}.",
        name:"baseWeekdayStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:DataSourceField.foreignKey":{
        ref:"attr:DataSourceField.foreignKey",
        flags:"IR",
        description:"Declares that this field holds values that can be matched to values from another DataSource\n field, to create a relationship between records from different DataSources or even records\n within the same DataSource.\n <p>\n The format of <code>foreignKey</code> is\n <code>&lt;dataSourceId&gt;.&lt;fieldName&gt;</code>.<br>\n For a foreignKey within the same dataSource, you can omit the <code>dataSourceId</code>\n and just specify <code>&lt;fieldName&gt;</code>.<br>\n For example, to create a tree relationship within a DataSource:\n <pre>\n   DataSource.create({\n     ID:\"supplyItem\",\n     fields : [\n       {name:\"itemId\", type:\"sequence\", primaryKey:true},\n       {name:\"parentId\", type:\"integer\", foreignKey:\"itemId\"},\n       ...\n     ]\n   });\n </pre>\n <P>\n <code>foreignKey</code> declarations also allow other automatic behaviors by\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}, such as ${isc.DocUtils.linkForRef('method:ListGrid.fetchRelatedData')}.",
        name:"foreignKey",
        seeAlso:[],
        valueType:"String",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["dataSourceRelations"]
    },
    "attr:MenuItem.isSeparator":{
        ref:"attr:MenuItem.isSeparator",
        flags:"IR",
        description:"When set to <code>true</code>, this menu item shows a horizontal separator instead of\n the ${isc.DocUtils.linkForRef('attr:MenuItem.title')} text.  Typically specified as the only property of a menu item,\n since the separator will not respond to mouse events.",
        name:"isSeparator",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"],
        definingClass:"object:MenuItem",
        groups:["menuBasic"]
    },
    "attr:ListGrid.expansionFieldTrueImage":{
        ref:"attr:ListGrid.expansionFieldTrueImage",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is set to <code>true</code>, this property\n determines the image to display in the expansion field for expanded rows.\n If unset, the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} will be used.",
        name:"expansionFieldTrueImage",
        seeAlso:[
            {ref:"attr:ListGrid.expansionFieldFalseImage", description:null},
            {ref:"attr:ListGrid.expansionFieldImageWidth", description:null},
            {ref:"attr:ListGrid.expansionFieldImageHeight", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:ColorPicker.opacitySliderLabel":{
        ref:"attr:ColorPicker.opacitySliderLabel",
        flags:"IR",
        description:"The label shown next to the opacity slider. Ignored if \n ${isc.DocUtils.linkForRef('attr:ColorPicker.supportsTransparency')} is false.",
        name:"opacitySliderLabel",
        seeAlso:[],
        valueType:"text",
        defaultValue:"\"Opacity\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ListGrid.autoFitMaxWidth":{
        ref:"attr:ListGrid.autoFitMaxWidth",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')} is set to <code>\"horizontal\"</code> or <code>\"both\"</code>\n this property provides an upper limit on how far the ListGrid will expand horizontally to\n accomodate its content. \n If content exceeds this width, scrollbars will be introduced as usual.  \n In addition to this property, ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxColumns')} allows you to limit \n horizontal expansion based on the number of columns to be rendered.",
        name:"autoFitMaxWidth",
        setter:["setAutoFitMaxWidth"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "method:TabSet.getTabPane":{
        ref:"method:TabSet.getTabPane",
        flags:"",
        description:" Returns the pane for a given tab.\n\n",
        name:"getTabPane",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null,
             type:"object | number | ID | Tab"}
        ],
        returns:{description:"the tab pane", type:"Canvas"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "object:Browser":{
        methods:[],
        ref:"object:Browser",
        classAttrs:["classAttr:Browser.isSupported"],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:[],
        description:"Object containing flags indicating basic attributes of the browser.",
        name:"Browser",
        seeAlso:[],
        type:"object"
    },
    "method:GridRenderer.cellOut":{
        ref:"method:GridRenderer.cellOut",
        flags:"A",
        description:" Called when the mouse pointer leaves a cell\n\n",
        name:"cellOut",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:DataSource.applyFilter":{
        ref:"method:DataSource.applyFilter",
        flags:"",
        description:" Returns records in the passed Array that match the provided filter\n ${isc.DocUtils.linkForRef('type:Criteria','criteria')}.\n <P>\n By default:\n <ul>\n <li> any criteria that do not correspond to a DataSource field are ignored\n <li> any null or empty string criteria are ignored\n <li> all other criteria are passed to ${isc.DocUtils.linkForRef('method:DataSource.fieldMatchesFilter')}\n </ul>\n This method is called by ${isc.DocUtils.linkForRef('method:ResultSet.applyFilter')} to provide filtering when a\n ResultSet has a complete cache and filtering can be performed client-side.  You may want\n to override this method in order to mimic the filtering behavior that your server performs.\n\n",
        name:"applyFilter",
        seeAlso:[],
        params:[
            {description:"the list of rows", optional:false, name:"data", defaultValue:null,
             type:"Array"},
            {description:"the filter criteria", optional:false, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"optional dataSource request properties", optional:true,
             name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        returns:{description:"the list of matching rows", type:"Array"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "object:Array":{
        methods:["method:Array.get", "method:Array.getLength", "method:Array.isEmpty", "method:Array.first", 
         "method:Array.last","method:Array.indexOf", "method:Array.lastIndexOf", "method:Array.contains", 
         "method:Array.containsAll","method:Array.intersect", "method:Array.equals", "method:Array.getItems", 
         "method:Array.getRange","method:Array.duplicate", "method:Array.set", "method:Array.addAt", 
         "method:Array.removeAt","method:Array.add", "method:Array.addList", "method:Array.setLength", 
         "method:Array.addListAt","method:Array.remove", "method:Array.removeList", "method:Array.dataChanged", 
         "method:Array.getProperty","method:Array.getValueMap", "method:Array.map", "method:Array.setProperty", 
         "method:Array.clearProperty","method:Array.getUniqueItems", "method:Array.findIndex", "method:Array.findNextIndex", 
         "method:Array.find","method:Array.containsProperty", "method:Array.findAll", "method:Array.slide", 
         "method:Array.slideRange","method:Array.makeIndex", "method:Array.sortByProperty", "method:Array.unsort", 
         "method:Array.max","method:Array.min", "method:Array.sum", "method:Array.and", "method:Array.or"],
        ref:"object:Array",
        classAttrs:["classAttr:Array.LOADING", "classAttr:Array.CASE_INSENSITIVE", 
         "classAttr:Array.DATE_VALUES","classAttr:Array.DATETIME_VALUES"],
        classMethods:["classMethod:Array.isLoading", "classMethod:Array.compareAscending", 
         "classMethod:Array.compareDescending"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Generic extensions to JavaScript Arrays.  You can call these on any Array.\n <P>\n JavaScript's native Array is retrofitted to support the <code>List</code> API.",
        name:"Array",
        implementsInterfaces:["List"],
        seeAlso:[
            {ref:"interface:List", description:null}
        ],
        type:"object"
    },
    "method:Button.deselect":{
        ref:"method:Button.deselect",
        flags:"",
        description:" Select this object.\n",
        name:"deselect",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:DataBoundComponent.showHiddenFields":{
        ref:"attr:DataBoundComponent.showHiddenFields",
        flags:"IRW",
        description:"Whether to show fields marked <code>hidden:true</code> when a DataBoundComponent is given a\n DataSource but no <code>component.fields</code>.\n <p>\n The <code>hidden</code> property is used on DataSource fields to mark fields that are\n never of meaning to an end user.",
        name:"showHiddenFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "attr:BlurbItem.textBoxStyle":{
        ref:"attr:BlurbItem.textBoxStyle",
        flags:"IRW",
        description:"Base css style for this item.",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"staticTextItem\"",
        type:"attr",
        definingClass:"class:BlurbItem",
        groups:["appearance"]
    },
    "method:Window.setTitle":{
        ref:"method:Window.setTitle",
        flags:"",
        description:"          Sets the title text that appears in the window header; the header will be redrawn\n          if necessary.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:"null",
             type:"string"}
        ],
        type:"method",
        definingClass:"class:Window",
        groups:["header"]
    },
    "attr:DateChooser.monthMenuStyle":{
        ref:"attr:DateChooser.monthMenuStyle",
        flags:"IR",
        description:"Style for the pop-up year menu.",
        name:"monthMenuStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"dateChooserMonthMenu\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:ListGrid.generateDoubleClickOnSpace":{
        ref:"attr:ListGrid.generateDoubleClickOnSpace",
        flags:"IRWA",
        description:"If true, when the user navigates to a cell using arrow keys and hits Space, \n the cell will respond to a double click event.",
        name:"generateDoubleClickOnSpace",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:TreeGrid.toggleFolder":{
        ref:"method:TreeGrid.toggleFolder",
        flags:"",
        description:"          Opens the folder specified by node if it's closed, and closes it if it's open.\n          TreeGrid will redraw if there's a change in the folder's open/closed state.\n\n",
        name:"toggleFolder",
        seeAlso:[],
        params:[
            {description:"node to toggle", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:StatefulCanvas.align":{
        ref:"attr:StatefulCanvas.align",
        flags:"IRW",
        description:"Horizontal alignment of this component's title.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["appearance"]
    },
    "method:TreeGrid.folderClick":{
        ref:"method:TreeGrid.folderClick",
        flags:"",
        description:"\n This method is called when a folder record is clicked on.\n\n",
        name:"folderClick",
        seeAlso:[
            {ref:"method:TreeGrid.nodeClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which folderClick() occurred.", optional:false,
             name:"viewer", defaultValue:null, type:"TreeGrid"},
            {description:"The folder (record) that was clicked", optional:false, name:"folder",
             defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the click occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "object:TreeGridField":{
        methods:[],
        ref:"object:TreeGridField",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/TreeGrid"],
        attrs:["attr:TreeGridField.treeField", "attr:TreeGridField.canExport"],
        description:"An object literal with a particular set of properties used to configure the display of\n and interaction with the columns of a ${isc.DocUtils.linkForRef('class:TreeGrid')}.\n ${isc.DocUtils.linkForRef('class:TreeGrid')} is a subclass of ${isc.DocUtils.linkForRef('class:ListGrid')} and as a result, for all fields except\n the field containing the ${isc.DocUtils.linkForRef('class:Tree')} itself (specified by\n ${isc.DocUtils.linkForRef('attr:TreeGridField.treeField')}, all properties settable on\n ${isc.DocUtils.linkForRef('object:ListGridField')} apply to TreeGridField as well.\n <p>\n This class documents just those properties that are specific to TreeGridFields - see\n ${isc.DocUtils.linkForRef('object:ListGridField')} for the set of inherited properties.",
        name:"TreeGridField",
        seeAlso:[
            {ref:"object:ListGridField", description:null},
            {ref:"attr:TreeGrid.fields", description:null},
            {ref:"method:ListGrid.setFields", description:null}
        ],
        type:"object"
    },
    "attr:DynamicForm.formSubmitFailedWarning":{
        ref:"attr:DynamicForm.formSubmitFailedWarning",
        flags:"IRWA",
        description:"Warning to display to the user if an attempt to ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm','natively submit')} a\n form is unable to submit to the server. The most common cause for this failure is that the user\n has typed an invalid file-path into an upload type field.",
        name:"formSubmitFailedWarning",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Form was unable to be submitted. The most likely cause for this is an invalid value in an upload field.\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["i18nMessages"]
    },
    "attr:DSResponse.endRow":{
        ref:"attr:DSResponse.endRow",
        flags:"R",
        description:"End row of returned server results, when using paged result fetching\n <p>\n Note that startRow and endRow are zero-based - the first record is row zero.",
        name:"endRow",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["paging"]
    },
    "attr:ListGridRecord.linkText":{
        ref:"attr:ListGridRecord.linkText",
        flags:"IRW",
        description:"The HTML to display in this row for fields with fieldType set to link. This overrides\n  ${isc.DocUtils.linkForRef('attr:ListGridField.linkText')}.",
        name:"linkText",
        seeAlso:[
            {ref:"type:ListGridFieldType", description:null},
            {ref:"type:FieldType", description:null},
            {ref:"attr:ListGridField.linkText", description:null},
            {ref:"attr:ListGrid.linkTextProperty", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord",
        groups:["display_values"]
    },
    "attr:FormItem.height":{
        ref:"attr:FormItem.height",
        flags:"IRW",
        description:"Default height of this item.",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formLayoutFilling')}"],
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "method:DataBoundComponent.deselectAllRecords":{
        ref:"method:DataBoundComponent.deselectAllRecords",
        flags:"",
        description:"\n Deselect all records\n\n",
        name:"deselectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "attr:TreeGrid.connectorImage":{
        ref:"attr:TreeGrid.connectorImage",
        flags:"IR",
        description:"The base filename for connector icons shown when ${isc.DocUtils.linkForRef('attr:TreeGrid.showConnectors')} is true.\n Connector icons are rendered into the title field of each row and show the dotted\n hierarchy lines between siblings of the same parent node. For each node, a connector icon\n may be shown:<ul>\n <li>As an opener icon for folder nodes, next to the folder icon</li>\n <li>In place of an opener icon for leaf nodes, next to the leaf icon</li>\n <li>As a standalone vertical continuation line in the indent to the left of the node, to show\n     a connection between some ancestor node's siblings (only relevant if\n     ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors')} is true).</li>\n </ul>\n Note that ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors')} governs whether connector lines will be\n displayed for all indent levels, or just for the innermost level of the tree.\n <P>\n The filenames for these icons are assembled from this base filename and the state of the\n node.  Assuming the connectorImage is set to <code>{baseName}.{extension}</code>, the\n full set of images to be displayed will be:\n <P>\n <code>{baseName}_ancestor[_rtl].{extension}</code> if ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors')}\n  is true, this is the URL for the vertical continuation image to be displayed at the\n  appropriate indent levels for ancestor nodes with subsequent children.\n <P>\n For nodes with no children:\n <ul>\n <li><code>{baseName}_single[_rtl].{extension}</code>: Shown when there is no connector line\n  attached to the parent or previous sibling, and no connector line to the next sibling. For\n  ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors','showFullConnectors:true')} trees, there will always be a\n  connector leading to the parent or previous sibling if its present in the tree so this\n  icon can only be displayed for the first row.</li>\n <li><code>{baseName}_start[_rtl].{extension}</code>:  Shown when the there is no connector\n  line attached to the parent or previous sibling, but there is a connector to the next\n  sibling. As with <code>_single</code> this will only ever be used for the first row if\n  ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors')} is true</li>\n <li><code>{baseName}_end[_rtl].{extension}</code>:  Shown if we are not showing a connector \n  line attached to the next sibling of this node (but are showing a connection to the previous\n  sibling or parent).</li>\n <li><code>{baseName}_middle[_rtl].{extension}</code>:  Shown where the we have a connector\n  line leading to both the previous sibling (or parent) and the next sibling.\n </ul>\n For folders with children. Note that if ${isc.DocUtils.linkForRef('attr:TreeGrid.showFullConnectors')} is false, open\n folders will never show a connector to subsequent siblings:\n <ul>\n <li><code>{baseName}_opened_single[_rtl].{extension}</code> opened folder node with \n  children when no connector line is shown attaching to either the folder's previous sibling\n  or parent, or to any subsequent siblings.</li>\n <li><code>{baseName}_opened_start[_rtl].{extension}</code>:  opened folder with children\n  when the there is no connector line attached to the parent or previous sibling, but there \n  is a connector to the next sibling.</li>\n <li><code>{baseName}_opened_end[_rtl].{extension}</code>:  opened folder with children \n  if we are not showing a connector line attached to the next sibling of this node (but are\n  showing a connection to the previous sibling or parent).</li>\n <li><code>{baseName}_opened_middle[_rtl].{extension}</code>: opened folder with children \n  where the we have a connector line leading to both the previous sibling (or parent) and the\n  next sibling.\n </ul>\n <ul>\n <li><code>{baseName}_closed_single[_rtl].{extension}</code> closed folder node with \n  children when no connector line is shown attaching to either the folder's previous sibling\n  or parent, or to any subsequent siblings.</li>\n <li><code>{baseName}_closed_start[_rtl].{extension}</code>:  closed folder with children\n  when the there is no connector line attached to the parent or previous sibling, but there \n  is a connector to the next sibling.</li>\n <li><code>{baseName}_closed_end[_rtl].{extension}</code>:  closed folder with children \n  if we are not showing a connector line attached to the next sibling of this node (but are\n  showing a connection to the previous sibling or parent).</li>\n <li><code>{baseName}_closed_middle[_rtl].{extension}</code>: closed folder with children \n  where the we have a connector line leading to both the previous sibling (or parent) and the\n  next sibling.\n </ul>\n (Note '[_rtl]' means that \"_rtl\" will be attached if isRTL() is true for this widget).",
        name:"connectorImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]connector.gif\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:DataSourceField.javaKeyClass":{
        ref:"attr:DataSourceField.javaKeyClass",
        flags:"IR",
        description:"See ${isc.DocUtils.linkForRef('attr:DataSourceField.javaClass')} - when auto-populating of Java Beans / POJOs based on \n inbound DSRequest data, for a field of Map type, <code>javaKeyClass</code> can be used to\n specify a particular concrete class for the map keys.  If not specified, and a concrete\n type cannot be discovered using Java Reflection, <code>java.lang.Object</code> is used.\n\n Note that <code>javaKeyClass</code> take precedence over generics if both are used.",
        name:"javaKeyClass",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "method:TreeGrid.getSelectedPaths":{
        ref:"method:TreeGrid.getSelectedPaths",
        flags:"",
        description:" Returns a snapshot of the current selection within this treeGrid as \n a ${isc.DocUtils.linkForRef('type:listGridSelectedState')} object.<br>\n This object can be passed to ${isc.DocUtils.linkForRef('method:TreeGrid.setSelectedPaths')} to reset this grid's selection\n the current state (assuming the same data is present in the grid).<br>\n",
        name:"getSelectedPaths",
        seeAlso:[
            {ref:"method:TreeGrid.setSelectedPaths", description:null}
        ],
        returns:{description:"current state of this grid's selection", type:"listGridSelectedState"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "group:loadXMLSchemaTag":{
        ref:"group:loadXMLSchemaTag",
        title:"&lt;isomorphic:loadXMLSchema&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n Load an XML Schema (.xsd) file and create a ${isc.DocUtils.linkForRef('class:SchemaSet')} object representing the loaded\n definitions. This tag works just like ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')}, except it's\n synchronous and the result is server-cacheable.\n <p>\n Note that this JSP tag must be surrounded by &lt;SCRIPT&gt; tags in the JSP because it\n generates JavaScript code.  Like other tags that generate JavaScript code, this tag can be\n used in a JSP that is included from your main page in order to create separate\n cacheability.  For example:<pre>\n     &lt;SCRIPT SRC=\"myXMLSchemaDefinitions.jsp\"&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>url</b><br>\n <i>value format</i>: URL or URI\n <i>default value</i>: NONE\n <p>\n This attribute specifies the URL or URI of the XML Schema file to fetch and translate.  This\n can be either a remote URL - e.g: <code>http://host:port/schemaFile.xsd</code> or a relative\n or absolute URI to a file local to this container - e.g: <code>/some/schemaFile.xsd</code>\n or <code>../some/schemaFile.xsd</code>.  If the url is a remote URL, then an HTTP request\n will be made for the file.  If it is local, it will be fetched from disk using standard\n Servlet APIs (<code>ServletContext.getResourceAsStream()</code>).\n <p>\n <b>cache</b><br>\n <i>value format</i>: Integer (number of seconds to cache result)\n <i>default value</i>: 3600 (1 hour)\n <p>\n This attribute specifies the number of seconds for which the fetched XML Schema is cacheable\n on the server.  Fetching an XML Schema file from a remote server can cause a significant\n delay in JSP processing, and XML Schema files rarely change outside of a development\n environment.  Set this value to zero to disable caching.",
        name:"loadXMLSchemaTag",
        seeAlso:[
            {ref:"classMethod:XMLTools.loadXMLSchema", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "attr:DynamicForm.requiredTitlePrefix":{
        ref:"attr:DynamicForm.requiredTitlePrefix",
        flags:"IRW",
        description:"The string prepended to the title of every required item in this form if\n highlightRequiredFields is true.",
        name:"requiredTitlePrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"<B>\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "attr:ColorPicker.colorArray":{
        ref:"attr:ColorPicker.colorArray",
        flags:"IR",
        description:"Array containing 40 HTML color strings, which will be shown as the basic color \n selection boxes.",
        name:"colorArray",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[...]",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "group:facetExpansion":{
        ref:"group:facetExpansion",
        refs:["attr:CubeGrid.canCollapseFacets", "attr:CubeGrid.canMinimizeFacets"],
        description:null,
        name:"facetExpansion",
        seeAlso:[],
        type:"group"
    },
    "method:WebService.callOperation":{
        ref:"method:WebService.callOperation",
        flags:"",
        description:" Invoke a web service operation.\n <P>\n The <code>data</code> parameter will be serialized to XML to form the input message for\n the operation, as described by ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}.  Namespacing,\n element ordering, and SOAP encoding rules are automatically followed.  If the web\n service you are trying to contact requires a complicated nested structure, consider\n using ${isc.DocUtils.linkForRef('attr:WSRequest.useFlatFields')} to simplify the required JavaScript input data.\n <P>\n The <code>resultType</code> selects what part of the message should be decoded to\n JavaScript and made available as the \"data\" variable in the callback.  The\n <code>resultType</code> parameter can be either:\n <ul>\n <li> an XPath.  \"data\" will be always be an Array, containing the selected elements as\n decoded by ${isc.DocUtils.linkForRef('classMethod:XMLTools.toJS')}.  All properties will have String value.\n <li> the name of an XML Schema type found somewhere in the response.  You can use the\n WSDL tab of the Developer Console to analyze the WSDL file for an appropriate type name.\n \"data\" will be an Array, containing the decoded elements as decoded by\n ${isc.DocUtils.linkForRef('method:DataSource.recordsFromXML')}.  In this case, since the XML Schema type of the\n selected data is known, properties will have correct type (eg \"date\" fields will\n have JavaScript Date objects)\n <li> null.  \"data\" will an Object representing the entire &lt;SOAP:Body&gt; as decoded\n to JavaScript.  As above, properties will have correct type.\n </ul>\n In the callback, you also receive the XML document returned by the web service as\n \"xmlDoc\".\n <P>\n NOTE: <code>callOperation()</code> is appropriate for simple operations that do not\n involve DataBound Components, such as logging into a web service, or retrieving simple\n String data.  <code>callOperation()</code> can also be used to retrieve small, read-only\n datasets such as the option list for a SelectItem, but only if the dataset is guaranteed\n to remain small enough for paging to be unnecessary.  For any larger datasets or\n anything that will be edited, DataSource integration is more appropriate.\n\n",
        name:"callOperation",
        seeAlso:[],
        params:[
            {description:"Name of the operation to invoke", optional:false, name:"operationName",
             defaultValue:null, type:"String"},
            {description:"data to serialize as XML to form the inbound message of                                  the operation",
             optional:false, name:"data", defaultValue:null, type:"Object"},
            {description:"Type, Element name, or XPath that                                  should be selected from the result.  For XPaths, see\n                                  ${isc.DocUtils.linkForRef('attr:WSRequest.xmlNamespaces')} for available namespace\n                                  prefixes and how to add more.",
             optional:false, name:"resultType", defaultValue:null, type:"Type or ElementName or XPath"},
            {description:"Callback to invoke on completion.  Signature                                  callback(data, xmlDoc, rpcResponse, wsRequest)",
             optional:false, name:"callback", defaultValue:null, type:"Callback"},
            {description:"Additional properties for the WSRequest, such                                      as HTTPHeaders",
             optional:false, name:"requestProperties", defaultValue:null, type:"WSRequest Properties"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('wsdlOperation')}"],
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "method:FormItem.enable":{
        ref:"method:FormItem.enable",
        flags:"",
        description:" Set this item to be enabled at runtime.\n",
        name:"enable",
        seeAlso:[
            {ref:"attr:FormItem.disabled", description:null}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["enable"]
    },
    "classMethod:DataSource.applyRecordSummaryFunction":{
        ref:"classMethod:DataSource.applyRecordSummaryFunction",
        flags:"",
        description:" Applies a ${isc.DocUtils.linkForRef('type:RecordSummaryFunction')} to a record and returns the result.\n",
        name:"applyRecordSummaryFunction",
        seeAlso:[],
        params:[
            {description:"Summary Function or identifier for   registered recordSummaryFunction to execute. If passed in as an explicit function\n   record, fields and summaryField parameters will be passed through to the function.",
             optional:false, name:"summaryFunction", defaultValue:null, type:"SummaryFunction"},
            {description:"Record to retrieve a summary for", optional:false, name:"record",
             defaultValue:null, type:"DataSourceRecord"},
            {description:"Set of fields to include in the summary", optional:false, name:"fields",
             defaultValue:null, type:"Array of DataSourceFields"},
            {description:"field in which this summary will be displayed.", optional:false,
             name:"summaryField", defaultValue:null, type:"DataSourceField"}
        ],
        returns:{description:"summary value for the record", type:"any"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:TabBar.breadth":{
        ref:"attr:TabBar.breadth",
        flags:"IRW",
        description:"Breadth of the tabBar (including baseline breadth)",
        name:"breadth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"21",
        type:"attr",
        definingClass:"class:TabBar"
    },
    "method:ListGrid.getFilterEditorValueMap":{
        ref:"method:ListGrid.getFilterEditorValueMap",
        flags:"A",
        description:"\n  If we're showing the filter (query-by-example) row for this ListGrid, this method is\n  used to determine the valueMap to display in the filter row for this field.\n  Default implementation will return the field.filterEditorValueMap if specified, or\n  field.valueMap.\n\n",
        name:"getFilterEditorValueMap",
        seeAlso:[],
        params:[
            {description:"field definition field for which we need a valueMap", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"ValueMap for the edit field (or null if no valueMap required)", type:"ValueMap"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "method:ColumnTree.getColumn":{
        ref:"method:ColumnTree.getColumn",
        flags:"A",
        description:" Advanced API - get the ListGrid representing the indicated column.\n",
        name:"getColumn",
        seeAlso:[],
        params:[
            {description:"column number, or parent node of the nodes shown in the column",
             optional:false, name:"column", defaultValue:null, type:"int or TreeNode"}
        ],
        returns:{description:"ListGrid that renders the indicated column, or null if column is not shown",
         type:"ListGrid"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "attr:TileGrid.tile":{
        ref:"attr:TileGrid.tile",
        flags:"IR",
        getter:["getTile"],
        description:"A TileGrid automatically creates one tile per record in the dataset, via the\n ${isc.DocUtils.linkForRef('type:AutoChild')} pattern.\n <P>\n By default, the ${isc.DocUtils.linkForRef('class:SimpleTile')} class will be used, and will contain content as rendered\n by a ${isc.DocUtils.linkForRef('class:DetailViewer')}, based on the provided ${isc.DocUtils.linkForRef('attr:TileGrid.fields')} (or on the default\n set of fields).\n <P>\n To create a completely different appearance, override ${isc.DocUtils.linkForRef('attr:TileGrid.tileConstructor')} with the name\n of the custom SmartClient class to use for each tile.  For example, subclass\n ${isc.DocUtils.linkForRef('class:SimpleTile')} and override ${isc.DocUtils.linkForRef('method:Canvas.getInnerHTML','getInnerHTML()')}, returning custom\n HTML for each tile.\n <pre>\n     isc.defineClass(\"MyCustomTile\", \"SimpleTile\").addProperties({\n        getInnerHTML : function () {\n           return this.Super(\"getInnerHTML\", arguments) +\n                this.getRecord().width + \" x \" + this.getRecord().height;\n        }\n     });\n\n     isc.TileGrid.create({\n        tileConstructor:\"MyCustomTile\"\n     });\n </pre>\n <P>\n Note that you can also override tile behaviors on a per-record basis, via\n ${isc.DocUtils.linkForRef('attr:TileRecord.tileConstructor')} and ${isc.DocUtils.linkForRef('attr:TileRecord.tileProperties')}.",
        name:"tile",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:Label.contents":{
        ref:"attr:Label.contents",
        flags:"IRW",
        description:"The contents of a canvas or label widget. Any HTML string is acceptable.",
        name:"contents",
        setter:["setContents"],
        seeAlso:[
            {ref:"attr:Label.dynamicContents", description:null}
        ],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        definingClass:"class:Label",
        groups:["contents"]
    },
    "method:ListGrid.getExpansionField":{
        ref:"method:ListGrid.getExpansionField",
        flags:"",
        description:" Returns the specially generated expansion field used when ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is\n true.\n <P>\n Called during ${isc.DocUtils.linkForRef('method:ListGrid.setFields')}, this method can be overridden to add advanced dynamic\n defaults to the expansion field (call Super, modify the default field returned by Super,\n return the modified field).  Normal customization can be handled by just setting\n ${isc.DocUtils.linkForRef('type:AutoChild')} properties, as mentioned under the docs for ${isc.DocUtils.linkForRef('attr:ListGrid.expansionField')}.\n\n",
        name:"getExpansionField",
        seeAlso:[],
        returns:{description:null, type:"ListGridField"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:CubeGrid.selectAllFacetValues":{
        ref:"method:CubeGrid.selectAllFacetValues",
        flags:"",
        description:"\n select/deselect all headers in a headerBar (specified by facetId) or\n all headerBars (if no facetId)\n\n  <br><i>methodType</i> action\n",
        name:"selectAllFacetValues",
        seeAlso:[],
        params:[
            {description:"ID of facet - if null, selects all headerbars' headers", optional:true,
             name:"facetId", defaultValue:null, type:"string"},
            {description:"new selection state - if null defaults to true", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:Canvas.setShowResizeBar":{
        ref:"method:Canvas.setShowResizeBar",
        flags:"",
        description:" When this Canvas is included as a member in a Layout, dynamically updates whether a \n resizeBar should be shown after this member in the layout, to allow it to be resized.\n",
        name:"setShowResizeBar",
        seeAlso:[],
        params:[
            {description:"setting for this.showResizeBar", optional:false, name:"show",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["layoutMember"]
    },
    "attr:ListGridField.valueIconSize":{
        ref:"attr:ListGridField.valueIconSize",
        flags:"IRW",
        description:"Default width and height of value icons in this field.\n Takes precidence over valueIconWidth, valueIconHeight and valueIconSize specified at\n the ListGrid level.\n Can be overridden via ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconWidth')} and {ListGridField.valueIconHeight}",
        name:"valueIconSize",
        seeAlso:[
            {ref:"attr:ListGrid.valueIconSize", description:null},
            {ref:"attr:ListGridField.valueIconWidth", description:null},
            {ref:"attr:ListGridField.valueIconHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:CalendarEvent.startDate":{
        ref:"attr:CalendarEvent.startDate",
        flags:"IRW",
        description:"Date object which represents the start date of a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.\n The name of this field within the CalendarEvent can be changed via \n ${isc.DocUtils.linkForRef('attr:Calendar.startDateField')}",
        name:"startDate",
        seeAlso:[],
        valueType:"Date",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "method:TreeGrid.recordDoubleClick":{
        ref:"method:TreeGrid.recordDoubleClick",
        flags:"",
        description:"\n Handle a doubleClick on a tree node - override of ListGrid stringMethod of same name.  If\n the node is a folder, this implementation calls ${isc.DocUtils.linkForRef('method:TreeGrid.toggleFolder')} on it.  If\n the node is a leaf, calls ${isc.DocUtils.linkForRef('method:TreeGrid.openLeaf')} on it.\n \n",
        name:"recordDoubleClick",
        seeAlso:[
            {ref:"method:ListGrid.recordDoubleClick", description:null}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:TreeGrid.folderDrop":{
        ref:"method:TreeGrid.folderDrop",
        flags:"A",
        description:"\n This method processes the drop on a folder in the TreeGrid.  The default implementation\n works as follows:\n <P>\n If the nodes originate in this tree and the ${isc.DocUtils.linkForRef('attr:TreeGrid.dragDataAction')} is \"none\" or\n \"move\", then the nodes are simply reordered in this TreeGrid.  Otherwise (if\n ${isc.DocUtils.linkForRef('attr:TreeGrid.dragDataAction')} is \"copy\" or \"none\"), this method calls\n ${isc.DocUtils.linkForRef('method:ListGrid.transferDragData')} on the <code>sourceWidget</code> and adds the returned\n rows to this TreeGrid.\n <P>\n In either case, the new row(s) appear in the <code>folder</code> at the <code>index</code>\n specified by the arguments of the same name.\n <P>\n For dataBound treeGrids, folderDrop() will initiate ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}\n to update remote datasets.  For nodes moved within the tree, an \"update\" operation will\n be submitted to update the ${isc.DocUtils.linkForRef('attr:Tree.parentIdField','parentId')} field of the moved node(s). \n For nodes added to a tree, \"add\" DataSource requests will be submitted with the dropped\n node(s) as dsRequest.data.  For all drops onto databound treeGrids from other databound \n components, if ${isc.DocUtils.linkForRef('attr:DataBoundComponent.addDropValues','addDropValues')} is true, \n ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues','getDropValues')} will be called for every item \n being dropped.\n <P>\n As a special case, if the <code>sourceWidget</code> is also databound, and a \n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} relationship is declared from the\n <code>sourceWidget</code>'s DataSource to this TreeGrid's DataSource, the interaction will\n be treated as a \"drag recategorization\" use case such as files being placed in folders,\n employees being assigned to teams, etc.  \"update\" DSRequests will be submitted that\n change the foreignKey field in the dropped records to point to the tree folder that was the\n target of the drop.  In this case no change will be made to the Tree data as such, only to\n the dropped records. \n\n",
        name:"folderDrop",
        seeAlso:[],
        params:[
            {description:"List of nodes being dropped", optional:false, name:"nodes", defaultValue:null,
             type:"List of TreeNode"},
            {description:"The folder being dropped on", optional:false, name:"folder",
             defaultValue:null, type:"TreeNode"},
            {description:"Within the folder being dropped on, the index at which the drop is                        occurring.  Only passed if ${isc.DocUtils.linkForRef('attr:TreeGrid.canReorderRecords')} is true.",
             optional:false, name:"index", defaultValue:null, type:"integer"},
            {description:"The component that is the source of the nodes (where the nodes                              were dragged from).",
             optional:false, name:"sourceWidget", defaultValue:null, type:"Canvas"},
            {description:"optional callback to be fired when the transfer process has                        completed.  The callback will be passed a single parameter \"records\",\n                       the list of nodes actually transferred to this component (it is called\n                       \"records\" because this is logic shared with ${isc.DocUtils.linkForRef('class:ListGrid')})",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('treeDropEvents')}"],
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.getSelection":{
        ref:"method:ListGrid.getSelection",
        flags:"",
        description:" Returns all selected records, as an Array.\n\n",
        name:"getSelection",
        seeAlso:[],
        returns:{description:"list of records, empty list if nothing selected",
         type:"Array of ListGridRecord"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:DataBoundComponent.getSort":{
        ref:"method:DataBoundComponent.getSort",
        flags:"",
        description:" Return the ${isc.DocUtils.linkForRef('object:SortSpecifier')}s representing the current sort configuration of this\n component.\n",
        name:"getSort",
        seeAlso:[],
        returns:{description:"sortSpecifiersThe current sort specification for this component",
         type:"Array of SortSpecifier"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "method:DateChooser.getData":{
        ref:"method:DateChooser.getData",
        flags:"",
        description:" Get the current value of the picker.\n <P>\n See ${isc.DocUtils.linkForRef('method:DateChooser.dataChanged')} for how to respond to the user picking a date.\n\n",
        name:"getData",
        seeAlso:[],
        returns:{description:"current date", type:"Date"},
        type:"method",
        definingClass:"class:DateChooser"
    },
    "attr:OperationBinding.lineBreakStyle":{
        ref:"attr:OperationBinding.lineBreakStyle",
        flags:"IR",
        description:"The style of line-breaks to use in the exported output.  See ${isc.DocUtils.linkForRef('type:LineBreakStyle')} for\n more information.",
        name:"lineBreakStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "method:DynamicForm.valuesAreValid":{
        ref:"method:DynamicForm.valuesAreValid",
        flags:"",
        description:" Method to determine whether the current form values would pass validation.\n This method will run validators on the form's values and return a value indicating whether\n validation was successful.\n <P>\n Unlike ${isc.DocUtils.linkForRef('method:DynamicForm.validate')} this method will not store the errors on the DynamicForm\n or display them to the user.\n",
        name:"valuesAreValid",
        seeAlso:[],
        params:[
            {description:"Should validators be processed for non-visible fields         such as dataSource fields with no associated item or fields with visibility set to\n         <code>\"hidden\"</code>?",
             optional:false, name:"validateHiddenFields", defaultValue:null, type:"boolean"},
            {description:"If unset, this method returns a simple boolean value indicating success or failure of validation. If this parameter is passed, this method will return\n an object mapping each field name to the errors(s) encountered on validation failure, or null\n if validation was successful.",
             optional:false, name:"returnErrors", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"Boolean value indiciation validation success, or if  <code>returnErrors</code> was specified, an object containing the generated errors mapped to\n their fieldNames.",
         type:"boolean | object | null"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:CubeGrid.alternateRecordStyles":{
        ref:"attr:CubeGrid.alternateRecordStyles",
        flags:"IRW",
        description:"Whether alternating rows should be drawn in alternating styles, in order to create a \"ledger\"\n effect for easier reading.  If enabled, the cell style for alternate rows will have \"Dark\"\n appended to it.",
        name:"alternateRecordStyles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "attr:ListGrid.summaryRowDataSource":{
        ref:"attr:ListGrid.summaryRowDataSource",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, by default summary values are calculated on the \n client based on the current data-set for the grid (see ${isc.DocUtils.linkForRef('method:ListGrid.getGridSummary')} and\n ${isc.DocUtils.linkForRef('method:ListGrid.getGridSummaryFunction')}).\n <P>\n In some cases however it may make sense to calculate summary values on the server and retrieve\n them via a dataSource fetch. If set, this property specifies a dataSource to fetch against for\n the summary row. The dataSource should return a single record with summary data for each\n field for which summary data should be shown. Note that specifying this property completely\n bypasses the standard client-side grid summary calculation logic.\n <P>\n The fetch may be further customized via ${isc.DocUtils.linkForRef('attr:ListGrid.summaryRowCriteria')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.summaryRowFetchRequestProperties')}",
        name:"summaryRowDataSource",
        seeAlso:[],
        valueType:"DataSource",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "group:iscServer":{
        ref:"group:iscServer",
        title:"SmartClient Server Summary",
        treeLocation:["Java Server Reference"],
        refs:[],
        description:"The SmartClient Server is a set of Java libraries, servlets and tools that provide the key\n server-side components needed to build a complete application in the modern web\n architecture.\n <P>\n The SmartClient Server can be ${isc.DocUtils.linkForRef('group:iscInstall','integrated')} into any pre-existing Java\n application, and is designed to rapidly connect SmartClient visual components to\n pre-existing Java business logic or persistence engines.  SmartClient's Ajax request\n processing facilities can be easily integrated into Struts Actions,\n ${isc.DocUtils.linkForRef('group:springIntegration','Spring controllers')}, or custom servlets and JSPs.\n <P>\n Alternatively, the SmartClient Server provides a complete SQL and Hibernate-based\n persistence engine for new applications, with out-of-the-box servlets for processing Ajax\n data requests.\n <P>\n The SmartClient Server is optional, and SmartClient's client-side Ajax engine can be\n integrated with any server that provides HTTP access, using XML, JSON, SOAP or proprietary\n data protocols.  However any server in a modern web application will be required to provide\n most or all of the features of the SmartClient Server (described below), and the SmartClient\n Server represents a best-of-breed implementation of these facilities with a long history of\n high-volume production deployments.\n <P>\n <h4>Server enforcement of ${isc.DocUtils.linkForRef('class:Validator','Validators')}</h4>\n <P>\n Data passed from the browser can be automatically validated by the SmartClient Server.\n In contrast, when using ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side integration')},\n data arrives as HTTP params or XML messages, and you must parse values into the correct\n types (eg java.util.Date) and validate them, or use a server framework that does so.\n <P>\n <h4>High Speed Data Delivery / Data Compression</h4>\n <P>\n The SmartClient Server delivers data to and from the browser using a proprietary, maximally\n efficient protocol, providing simple Java APIs for sending and receiving data.\n <P>\n SmartClient's data protocol is:\n <ul>\n <li> automatically compressed: provides 6-8x improvement in bandwidth utilization\n <li> efficient on the server: high speed data serialization for any Java Object\n <li> efficient in the browser: faster than ordinary XML or JSON data delivery\n <li> minimal: facilities for ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields','trimming')} and\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','extracting')} only the data you want the browser to see\n </ul>\n <P>\n <h4>Transparent upload support</h4>\n <P>\n SmartClient provides special client and server-side support for ${isc.DocUtils.linkForRef('group:upload','file\n upload')}, which allows single and multiple-file HTTP uploads to be performed as a background\n Ajax request without reloading the page or launching sub-windows.\n <P>\n Uploaded files arrive at the SmartClient server as Java InputStreams accessible from the\n DSRequest object, and can optionally be automatically stored via SmartClient's SQL\n subsystem.\n <P>\n <h4>Transparent Queuing / \"Batch\" Operations</h4>\n <P>\n Any request transmitted to the SmartClient Server can be combined into a \"queue\" transmitted\n as a single HTTP request, with in-order execution for all queued operations. \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','startQueue()')} starts a queue and\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendQueue','sendQueue()')} transmits it; queueing is transparent to the code\n that initiates the individual requests.  This enables:\n <ul>\n <li> re-use of data access operations across different screens\n <li> easy implementation of transaction boundaries\n <li> simplified saving and loading of screens with complex, master-detail views\n <li> guaranteed in-order processing of operations\n <li> more efficient network usage\n </ul>\n <P>\n <h4>Visual Builder</h4>\n <P>\n ${isc.DocUtils.linkForRef('group:visualBuilder','Visual Builder')} is included with the SmartClient Server, and uses\n server features such as automatic SQL binding to provide a rapid prototyping environment.\n <P>\n <h4>Automatic Bi-directional Java &lt; - &gt; JavaScript serialization and translation</h4>\n <P>\n Provides a powerful, type-safe ${isc.DocUtils.linkForRef('attr:RPCRequest.data','data transmission mechanism')} for moving\n data between a Java server and the browser. \n <P>\n Any Java objects, including Java Beans, POJOs, Java Collections, XML DOMs and all Java\n primitives, with any level of nesting, can be automatically serialized and delivered as\n JavaScript Objects to the SmartClient client-side components.\n <P>\n JavaScript Objects existing in the browser can likewise be automatically transmitted to a\n Java Server and translated to Java Objects, with any level of nesting and automatic\n preservation of primitive types.\n <P>\n <h4>SQL and Hibernate connectors</h4>\n <P>\n DataSources of serverType:\"sql\" or serverType:\"hibernate\" can generate and execute queries\n against popular SQL engines or against the Hibernate ORM system, providing SmartClient's\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}s with the four standard CRUD operations (create, retrieve, update,\n delete) without writing any server-side code.  For rapid prototyping, these DataSources can\n even generate SQL tables based on the DataSource declaration, using the\n ${isc.DocUtils.linkForRef('group:adminConsole')} visual tool.\n <P>\n Server-side APIs allow server-side modification of the request before it is executed (for\n example, to enforce security) and post-processing of the request after execution (for\n example, to provide calculated values).\n <P>\n Both serverType:\"sql\" and serverType:\"hibernate\" support the field-operator-value queries\n that can be generated by using the ${isc.DocUtils.linkForRef('class:FilterBuilder')} component (see\n ${isc.DocUtils.linkForExampleId('filterBuilderBracket', 'example')}).\n <P>\n <h4>Rich, Standardized Request / Response protocol</h4>\n <P>\n The SmartClient Server provides a standardized request and response protocol designed for\n data-oriented \"CRUD\" operations (create, retrieve, update, delete).\n <P>\n This standardized protocol automatically handles ${isc.DocUtils.linkForRef('class:DSRequest','request metadata')} (paging\n parameters, requested sort order, original values of data being modified) and\n ${isc.DocUtils.linkForRef('class:DSResponse','response metadata')} (error handling, cache management, session expiration etc).\n <P>\n Similar to the impact of Struts on early web applications, this standardized protocol avoids\n developers in different groups inventing their own incompatible and redundant\n request/response protocols, and allows developers to more easily learn code they didn't\n author.\n <P>\n <h4>Bi-directional XPath binding to Java Objects</h4>\n <P>\n Most UI designs do not directly reflect the underlying Object model and so some degree of\n translation is necessary in order to populate UI components with data and apply user changes\n to the Java Object model.  This is often accomplished with brittle, difficult to understand\n data translation code sprinkled throughout the system, done in a different way for every\n screen or component.\n <P>\n SmartClient provides a standard, ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','XPath-based approach')} to\n adapting any Java-based Object model to the requirements of the UI design.  Data relevant to\n the application UI is centrally extracted in the server-side ${isc.DocUtils.linkForRef('class:DataSource')} layer, so\n that all UI components have a consistent, unified view of the data model for both loading\n <b>and</b> saving data.\n <P>\n <h4>Broadest possible browser support</h4>\n <P>\n The SmartClient Server can compensate for facilities \n ${isc.DocUtils.linkForRef('group:platformDependencies','missing or disabled in certain browsers')},\n including ActiveX being disabled in IE6 and missing XML support in some versions\n of Apple's Safari browser.\n <P>\n <h4>Transparent Proxying</h4>\n <P>\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendProxied','Proxying')} allows SmartClient applications to access web\n services, RSS feeds, HTML content and other data services in a secure manner regardless of\n where they are located: across the enterprise or publicly available.\n <P>\n <h4>Optional ${isc.DocUtils.linkForRef('group:networkPerformance','Network Performance')} Module</h4>\n <P>\n Provides:\n <ul>\n <li> compressed delivery of SmartClient runtime, application logic and other assets such as CSS\n <li> ${isc.DocUtils.linkForRef('class:FileLoader','background download')} of SmartClient and other assets for zero\n user-perceived load time\n <li> on-the-fly stripping, combining and obfuscation of JavaScript (application code and data)\n <li> browser cache control\n </ul>\n <P>\n <h4>Optional Messaging Module (aka server push)</h4>\n <P>\n The ${isc.DocUtils.linkForDocNode('messaging', 'Messaging')} module allows the server to \"push\" messages to the client, without\n client polling, for real-time monitoring/dashboarding applications.",
        name:"iscServer",
        seeAlso:[],
        type:"group"
    },
    "method:Class.map":{
        ref:"method:Class.map",
        flags:"",
        description:"\n Call <code>method</code> on each item in <code>argsList</code> and return the Array of results.\n\n",
        name:"map",
        seeAlso:[],
        params:[
            {description:"Name of the method on this instance which should be called on each element of the Array",
             optional:false, name:"methodName", defaultValue:null, type:"string"},
            {description:"Array of items to call the method on", optional:false, name:"items",
             defaultValue:null, type:"Array"}
        ],
        returns:{description:"Array of results, one per element in the passed \"items\" Array", type:"Array"},
        type:"method",
        definingClass:"class:Class"
    },
    "method:MultiSortDialog.validate":{
        ref:"method:MultiSortDialog.validate",
        flags:"",
        description:" Validate that no two ${isc.DocUtils.linkForRef('object:SortSpecifier')}s sort on the same \n ${isc.DocUtils.linkForRef('attr:SortSpecifier.property','property')}.\n\n",
        name:"validate",
        seeAlso:[],
        returns:{description:"True if validation succeeds, false if any property is used twice",
         type:"boolean"},
        type:"method",
        definingClass:"class:MultiSortDialog"
    },
    "method:TileLayout.setTileMargin":{
        ref:"method:TileLayout.setTileMargin",
        flags:"",
        description:" sets the vertical and horizontal margin of tiles\n",
        name:"setTileMargin",
        seeAlso:[],
        params:[
            {description:"margin", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "type:ListGridFieldType":{
        ref:"type:ListGridFieldType",
        values:[
            {description:"Simple text rendering for view.  For editing a text entry field is shown. If the length of the field (as specified by the ${isc.DocUtils.linkForRef('attr:DataSourceField.length')}\n attribute) is larger than the value specified by ${isc.DocUtils.linkForRef('attr:ListGrid.longTextEditorThreshold')}, a\n text input icon is shown that, when clicked on (or field is focused in) opens a larger\n editor that expands outside the boundaries of the cell (textarea by default, but\n overrideable via ${isc.DocUtils.linkForRef('attr:ListGrid.longTextEditorType')}).",
             value:"\"text\""},
            {description:"For viewing and editing a checkbox is shown with a check mark for the <code>true</code> value and no check mark for the <code>false</code> value.  See\n ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} for customization.",
             value:"\"boolean\""},
            {description:"Same as <code>text</code>.  Consider setting ${isc.DocUtils.linkForRef('attr:ListGridField.editorType','editorType')} to use a ${isc.DocUtils.linkForRef('class:SpinnerItem')}.",
             value:"\"integer\""},
            {description:"Same as <code>text</code>.  Consider setting ${isc.DocUtils.linkForRef('attr:ListGridField.editorType','editorType')} to use a ${isc.DocUtils.linkForRef('class:SpinnerItem')}.",
             value:"\"float\""},
            {description:"Expected to contain <code>Date</code> type data. Dates will be formatted using\n ${isc.DocUtils.linkForRef('attr:ListGridField.displayFormat')} if specified, otherwise ${isc.DocUtils.linkForRef('attr:ListGrid.dateFormatter')}.\n If both these attributes are unset, dates are formatted using the standard \n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat','short display format')} for dates.\n <P>\n For editing, by default a ${isc.DocUtils.linkForRef('class:DateItem')} is used with ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} set\n to true, providing textual date entry plus a pop-up date picker. The\n ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat','displayFormat')} and ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat','inputFormat')}\n for the editor will be picked up from the ListGridField, if specified.",
             value:"\"date\""},
            {description:"Expected to contain Time data encoded in javascript <code>Date</code> objects.\n Times will be formatted using ${isc.DocUtils.linkForRef('attr:ListGridField.displayFormat')} if specified, \n otherwise ${isc.DocUtils.linkForRef('attr:ListGrid.timeFormatter')}.\n <P>\n If both these attributes are unset, times are formatted using the standard \n ${isc.DocUtils.linkForRef('classAttr:Time.shortDisplayFormat','short display format')} for times.\n <P>\n For editing, by default a ${isc.DocUtils.linkForRef('class:TimeItem')} is used. The\n ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat','displayFormat')} for the editor will be picked up from \n the ListGridField, if specified.",
             value:"\"time\""},
            {description:"Same as <code>text</code>", value:"\"sequence\""},
            {description:"Renders a clickable html link (using an HTML anchor tag: &lt;A&gt;). The target URL is the value of the field, which is also the default display value.  You\n can override the display value by setting ${isc.DocUtils.linkForRef('attr:ListGridRecord.linkText')} or\n ${isc.DocUtils.linkForRef('attr:ListGridField.linkText')}.\n <P>\n Clicking the link opens the URL in a new window by default.  To change this behavior,\n you can set <code>field.target</code>, which works indentically to the \"target\"\n attribute on an HTML anchor (&lt;A&gt;) tag.  See ${isc.DocUtils.linkForRef('attr:ListGridField.target')} for more\n information.\n <P>\n In inline edit mode, this type works like a text field.\n <P>\n To create a link not covered by this feature, consider using\n ${isc.DocUtils.linkForRef('method:ListGridField.formatCellValue')} along with ${isc.DocUtils.linkForRef('method:Canvas.linkHTML')}, or simply\n ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle','styling the field')} to look like a link, and providing\n interactivity via ${isc.DocUtils.linkForRef('method:ListGridField.recordClick','field.recordClick()')}.",
             value:"\"link\""},
            {description:"Renders a different image in each row based on the value of the field.  If this URL is not absolute, it is assumed to be relative to\n ${isc.DocUtils.linkForRef('attr:ListGridField.imageURLPrefix')} if specified. The size of the image is controlled by\n ${isc.DocUtils.linkForRef('attr:ListGridField.imageSize')}, ${isc.DocUtils.linkForRef('attr:ListGridField.imageWidth')},\n ${isc.DocUtils.linkForRef('attr:ListGridField.imageHeight')} (and by the similarly-named global default\n attributes on the ListGrid itself).\n <P>\n You can also specify the following attributes on the field: <code>activeAreaHTML</code>, and\n <code>extraStuff</code> - these are passed to ${isc.DocUtils.linkForRef('method:Canvas.imgHTML')} to generate the\n final URL.\n <P>\n Note if you want to display icons <b>in addition to</b> the normal cell value, you\n can use ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons','valueIcons')} instead.",
             value:"\"image\""},
            {description:"Shows ${isc.DocUtils.linkForRef('attr:ListGridField.icon','field.icon')} in every cell, and also in the header.  Useful for a field that is used as a button, for example, launches a detail\n window or removes a row.  Implement a ${isc.DocUtils.linkForRef('method:ListGridField.recordClick','field.recordClick')}\n to define a behavior for the button.\n <P>\n NOTE: for a field that shows different icons depending on the field value, see\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')}.\n <P>\n <code>type:\"icon\"</code> also defaults to a small field width, accomodating just the icon\n with padding, and to a blank header title, so that the header shows the icon only.  \n <P>\n ${isc.DocUtils.linkForRef('attr:ListGridField.iconWidth','field.iconWidth')} and related properties configure\n the size of the icon both in the header and in body cells.\n <P> \n If you want the icon to appear only in body cells and not in the header, set\n ${isc.DocUtils.linkForRef('attr:ListGridField.cellIcon','field.cellIcon')} instead, leaving field.icon null.",
             value:"\"icon\""},
            {description:"For viewing, the grid renders a 'view' icon (looking glass) followed by a 'download' icon and then the name of the file is displayed in text.  If the user clicks the\n 'view' icon, a new browser window is opened and the file is streamed to that browser\n instance, using ${isc.DocUtils.linkForRef('method:DataSource.viewFile')}.  For images and other file types with\n known handlers, the content is typically displayed inline - otherwise the browser will\n ask the user how to handle the content.  If the download icon is clicked,\n ${isc.DocUtils.linkForRef('method:DataSource.downloadFile')} is used to cause the browser to show a \"save\" dialog.\n There is no inline editing mode for this field type.",
             value:"\"binary\""},
            {description:"Same as <code>binary</code>", value:"\"imageFile\""},
            {description:"Show a calculated summary based on other field values within the  current record. See ${isc.DocUtils.linkForRef('attr:ListGridField.recordSummaryFunction')} for more information",
             value:"\"summary\""}
        ],
        description:"ListGrids format data for viewing and editing based on the <i>type</i> attribute of the\n field.  This table describes how the ListGrid deals with the various built-in types.",
        name:"ListGridFieldType",
        seeAlso:[
            {ref:"attr:ListGridField.type", description:null},
            {ref:"type:FieldType", description:null}
        ],
        type:"type",
        examples:["${isc.DocUtils.linkForExampleId('gridsDataTypes')}"]
    },
    "attr:SimpleType.name":{
        ref:"attr:SimpleType.name",
        flags:"IR",
        description:"Name of the type, used to refer to the type from ${isc.DocUtils.linkForRef('attr:DataSourceField.name','field.name')}.",
        name:"name",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleType"
    },
    "attr:ImgSplitbar.canDrag":{
        ref:"attr:ImgSplitbar.canDrag",
        flags:"IRW",
        description:"<code>canDrag</code> set to true to allow dragging of the split bar. Dragging the\n Splitbar will resize it's ${isc.DocUtils.linkForRef('attr:Splitbar.target','target')}",
        name:"canDrag",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "attr:ImgTab.align":{
        ref:"attr:ImgTab.align",
        flags:"IRW",
        description:"Alignment of title text",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:ImgTab",
        groups:["positioning"]
    },
    "method:SectionItem.collapseSection":{
        ref:"method:SectionItem.collapseSection",
        flags:"",
        description:" Collapse a sectionItem, and hide all the items within the section (not including\n the header).\n",
        name:"collapseSection",
        seeAlso:[],
        type:"method",
        definingClass:"class:SectionItem"
    },
    "method:GridRenderer.getCellStyle":{
        ref:"method:GridRenderer.getCellStyle",
        flags:"",
        description:" Return the CSS class for a cell. By default this method has the following implementation:<br>\n - return any custom style for the record (see ${isc.DocUtils.linkForRef('attr:GridRenderer.recordCustomStyleProperty')}) \n   if defined.<br>\n - create a style name based on the result of ${isc.DocUtils.linkForRef('method:GridRenderer.getBaseStyle')} and the \n   state of the record.<br>\n The state of the record is indicated by adding a suffix to the base style.\n There are four independant boolean states, which are combined in the order given:\n <ol>\n <li>\"Disabled\" : whether the cell is disabled; enable by setting the \"enabled\" flag on record\n     returned by getCellRecord\n <li>\"Selected\" : whether cell is selected; enable by passing a Selection object as \"selection\"\n <li>\"Over\" : mouse is over this cell; enable with showRollovers\n <li>\"Dark\" : alternating color bands; enable with alternateRowStyles\n </ol>\n For example, with a baseStyle of \"myCell\", a cell which is selected, which the mouse is over,\n and which is in a dark-colored band will get a styleName of myCellSelectedOverDark.\n <p>\n Cell Styles customizable by:\n <ul>\n <li>attaching a custom style to a record by setting \n    <code>record[this.recordCustomStyleProperty]</code> to some valid CSS style name.  \n <li>modifying the base style returned by getBaseStyle() [see that method for further \n     documentation on this]\n <li>overriding this function\n </ul>\n\n",
        name:"getCellStyle",
        seeAlso:[],
        params:[
            {description:"record object for this row and column", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"number of the row", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"number of the column", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS style for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["appearance"]
    },
    "group:menuKeys":{
        ref:"group:menuKeys",
        refs:["attr:MenuItem.keys", "attr:MenuItem.keyTitle"],
        name:"menuKeys",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.setDragTracker":{
        ref:"method:Canvas.setDragTracker",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance')} is set to <code>\"tracker\"</code>, this method will be called\n (if defined), when the user starts to drag this widget. It is an opportunity to update the\n drag tracker to display something relative to this canvas.  Typical implementation will\n be to call ${isc.DocUtils.linkForRef('classMethod:EventHandler.setDragTracker')}, passing in the desired custom tracker HTML \n as a string\n",
        name:"setDragTracker",
        seeAlso:[],
        returns:{description:"Return false to suppress bubbling, and prevent <code>setDragTracker()</code>                      from being called on this widget's ancestors.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragTracker')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "classMethod:Date.create":{
        ref:"classMethod:Date.create",
        flags:"",
        description:"  Create a new <code>Date</code> object - synonym for <code>new Date(arguments)</code>\n",
        name:"create",
        seeAlso:[],
        returns:{description:"Date object", type:"Date"},
        type:"classMethod",
        definingClass:"class:Date"
    },
    "classMethod:RPCManager.startQueue":{
        ref:"classMethod:RPCManager.startQueue",
        flags:"",
        description:"\n Start queuing requests.  When queuing requests, an HTTP request will not be sent to\n the server until RPCManager.sendQueue() is called.\n <p>\n All requests in a given queue must go to the same actionURL and use the same transport\n (XMLHttp or frames).  If a request specifies a different actionURL or transport than\n that of the requests currently on the queue, it will be sent to the server separately,\n ahead of the queue, and a warning will be logged to the Developer Console.\n <p>\n Note that the server must process all requests sent as part of the queue before any\n response is sent to the client.  You should avoid batching a request that will take a\n long time to process on the server with any other requests because it will delay the\n response of all requests in the queue.\n\n",
        name:"startQueue",
        seeAlso:[
            {ref:"classMethod:RPCManager.sendQueue", description:null}
        ],
        params:[
            {description:"whether queuing should be enabled, default true.  Passing false                      will disable queuing but not send the queue yet, so that any\n                      queued requests will be sent along with the next\n                      send()/sendRequest()",
             optional:true, name:"shouldQueue", defaultValue:null, type:null}
        ],
        returns:{description:"whether queuing was already enabled before we called", type:"boolean"},
        type:"classMethod",
        definingClass:"class:RPCManager",
        requiresModules:["SCServer"]
    },
    "attr:Canvas.snapResizeToGrid":{
        ref:"attr:Canvas.snapResizeToGrid",
        flags:"IRW",
        description:"Causes this canvas to snap to its parent's grid when resizing.\n Note that this value defaults to the Canvas's \n ${isc.DocUtils.linkForRef('attr:Canvas.snapToGrid','snapToGrid')} value if undefined.",
        name:"snapResizeToGrid",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:JSONEncoder.encodeDate":{
        ref:"method:JSONEncoder.encodeDate",
        flags:"",
        description:" Encode a JavaScript Date value.\n <P>\n By default, follows the ${isc.DocUtils.linkForRef('attr:JSONEncoder.dateFormat')} setting.  Override to do custom\n encoding.\n \n",
        name:"encodeDate",
        seeAlso:[],
        params:[
            {description:"JavaScript date object to be serialized", optional:false, name:"theDate",
             defaultValue:null, type:"Date"}
        ],
        returns:{description:"value to be included in result.  <b>If this value is intended to appear                  as a String it should include quotes (\")</b>",
         type:"String"},
        type:"method",
        definingClass:"class:JSONEncoder"
    },
    "attr:FormItem.showIcons":{
        ref:"attr:FormItem.showIcons",
        flags:"IRWA",
        description:"Set to false to suppress writing out any ${isc.DocUtils.linkForRef('attr:FormItem.icons')} for this item.",
        name:"showIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:ServerObject.ID":{
        ref:"attr:ServerObject.ID",
        flags:"IR",
        description:"You can optionally specify an ID on the ServerObject config block - in which case you can\n use that value as the \"className\" argument when calling ${isc.DocUtils.linkForRef('classMethod:DMI.call')}.  This allows you to\n hide the name of the server-side class used as the factory or implementer of the DMI from\n the browser as a security precaution.",
        name:"ID",
        seeAlso:[
            {ref:"attr:ServerObject.className", description:null},
            {ref:"classMethod:DMI.call", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "attr:DataSource.enumTranslateStrategy":{
        ref:"attr:DataSource.enumTranslateStrategy",
        flags:"IA",
        description:"Sets the strategy this DataSource uses to translate Java enumerated types (objects of type\n enum) to and from Javascript.\n \n This property is only applicable if you are using the SmartClient server",
        name:"enumTranslateStrategy",
        seeAlso:[],
        valueType:"EnumTranslateStrategy",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:StretchImgButton.iconSize":{
        ref:"attr:StretchImgButton.iconSize",
        flags:"IR",
        description:"Size in pixels of the icon image.\n <P>\n The <code>iconWidth</code> and <code>iconHeight</code> properties can be used to\n configure width and height separately.",
        name:"iconSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:LoginDialog.loginFailureItem":{
        ref:"attr:LoginDialog.loginFailureItem",
        flags:"IR",
        description:"Field item containing login error message (if required) in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"loginFailureItem",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:ColorPicker.modeToggleButtonConstructor":{
        ref:"attr:ColorPicker.modeToggleButtonConstructor",
        flags:"IRWA",
        description:"The class of the mode toggle button. It is intended that you use either IButton or \n      Button - other classes are unlikely to work correctly.",
        name:"modeToggleButtonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"IButton",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "group:toolbar":{
        ref:"group:toolbar",
        refs:["attr:Dialog.showToolbar"],
        name:"toolbar",
        seeAlso:[],
        type:"group"
    },
    "method:TreeGrid.getNodeTitle":{
        ref:"method:TreeGrid.getNodeTitle",
        flags:"",
        description:"\n Returns the title to show for a node in the tree column.  If the field specifies the\n <code>name</code> attribute, then the current <code>node[field.name]</code> is returned.\n Otherwise, the result of calling ${isc.DocUtils.linkForRef('method:Tree.getTitle')} on the node is called.\n <br><br>\n You can override this method to return a custom title for node titles in the tree column.\n\n",
        name:"getNodeTitle",
        seeAlso:[
            {ref:"method:Tree.getTitle", description:null}
        ],
        params:[
            {description:"The node for which the title is being requested.", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"The index of the node.", optional:false, name:"recordNum", defaultValue:null,
             type:"Number"},
            {description:"The field for which the title is being requested.", optional:false,
             name:"field", defaultValue:null, type:"DSField"}
        ],
        returns:{description:"the title to display.", type:"HTML"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "classMethod:Validator.addValidators":{
        ref:"classMethod:Validator.addValidators",
        flags:"A",
        description:"  Add several new validator types at once, as though ${isc.DocUtils.linkForRef('classMethod:Validator.addValidator')} were called\n  several times.\n \n",
        name:"addValidators",
        seeAlso:[
            {ref:"classMethod:Validator.addValidator", description:null}
        ],
        params:[
            {description:"Set of validators to add.  This parameter should      be a JavaScript object where the property names are validator type names, and the\n      property values are condition functions or expressions, for example:<br>\n      &nbsp;&nbsp;&nbsp;<code>{type1:condition1, type2:condition2}</code><br>.",
             optional:false, name:"newValidators", defaultValue:null, type:"object"}
        ],
        type:"classMethod",
        definingClass:"class:Validator",
        groups:["validation"]
    },
    "method:TreeGrid.getExtraIcon":{
        ref:"method:TreeGrid.getExtraIcon",
        flags:"A",
        description:" Get an additional icon to show between the open icon and folder/node icon for a particular \n node.\n\n",
        name:"getExtraIcon",
        seeAlso:[],
        params:[
            {description:"tree node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"URL for the extra icon (null if none required)", type:"URL"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:Selection.deselectRange":{
        ref:"method:Selection.deselectRange",
        flags:"",
        description:"			Deselect range of records from <code>start</code> to <code>end</code>, non-inclusive\n\n",
        name:"deselectRange",
        seeAlso:[],
        params:[
            {description:"start index to select", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end index (non-inclusive)", optional:false, name:"end", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:ColumnTree.folderIcon":{
        ref:"attr:ColumnTree.folderIcon",
        flags:"IRW",
        description:"The URL of the base icon for all folder nodes in this columnTree. Note that this URL will\n have ${isc.DocUtils.linkForRef('attr:ColumnTree.openIconSuffix')} or ${isc.DocUtils.linkForRef('attr:ColumnTree.closedIconSuffix')} appended to \n indicate state changes if appropriate - see documentation on  ${isc.DocUtils.linkForRef('attr:ColumnTree.showOpenIcons')}",
        name:"folderIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]folder.gif\"",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "attr:ListGrid.fastCellUpdates":{
        ref:"attr:ListGrid.fastCellUpdates",
        flags:"I",
        description:"Whether to speed up dynamic styling at the expense of slightly slower drawing.\n <P>\n <code>fastCellUpdates</code> speeds up the dynamic styling system used by rollovers,\n selections, and custom styling that calls ${isc.DocUtils.linkForRef('method:GridRenderer.refreshCellStyle')}, at\n the cost of slightly slower draw() and redraw() times.",
        name:"fastCellUpdates",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "classAttr:ValidatorTypes.floatPrecision":{
        ref:"classAttr:ValidatorTypes.floatPrecision",
        flags:"IR",
        description:"Tests whether the value for this field is a floating point number with the \n  appropriate number of decimal places - specified in <code>validator.precision</code>\n  If the value is of higher precision, if <code>validator.roundToPrecision</code> \n  is specified, the value will be rounded to the specified number of decimal places\n  and validation will pass, otherwise validation will fail.",
        name:"floatPrecision",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:ListGrid.fieldVisibilitySubmenuTitle":{
        ref:"attr:ListGrid.fieldVisibilitySubmenuTitle",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canPickFields','this.canPickFields')} is true, this\n attribute will be shown as the title for the menu item which contains a submenu with items \n allowing the user to show and hide fields in the grid.",
        name:"fieldVisibilitySubmenuTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Columns\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:Window.headerSrc":{
        ref:"attr:Window.headerSrc",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Window.showHeaderBackground')} is <code>true</code>, this property provides\n the URL of the background image for the header.",
        name:"headerSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]Window/headerGradient.gif\" | null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:DSRequest.exportFields":{
        ref:"attr:DSRequest.exportFields",
        flags:"IR",
        description:"The list of field-names to export.  If provided, the field-list in the exported output is \n limited and sorted as per the list.\n <P>\n If exportFields is not provided, the exported output includes all visible fields \n from the DataSource (field.hidden=false), sorted in the order they're defined.",
        name:"exportFields",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:Array.clearProperty":{
        ref:"method:Array.clearProperty",
        flags:"",
        description:" Delete property in each item in this array.\n",
        name:"clearProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to clear", optional:false, name:"property",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"returns true if any of the properties in the array had a value for the     specified property.",
         type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["iteration"]
    },
    "attr:FacetValue.width":{
        ref:"attr:FacetValue.width",
        flags:"IR",
        description:"Width of the cube grid facet value in pixels",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"facet.width",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:ListGrid.isGrouped":{
        ref:"attr:ListGrid.isGrouped",
        flags:"R",
        description:"True if this listgrid is grouped, false otherwise",
        name:"isGrouped",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:TabSet.tabBarPosition":{
        ref:"attr:TabSet.tabBarPosition",
        flags:"IR",
        description:"Which side of the TabSet the TabBar should appear on.",
        name:"tabBarPosition",
        seeAlso:[],
        valueType:"Side",
        defaultValue:"Canvas.TOP",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('tabsOrientation')}"],
        definingClass:"class:TabSet",
        groups:["tabBar"]
    },
    "attr:ListGrid.leaveScrollbarGap":{
        ref:"attr:ListGrid.leaveScrollbarGap",
        flags:"IRW",
        description:"Whether to leave a gap for the vertical scrollbar, even when it's not present.\n <P>\n Note that if leaveScrollbarGap is false and vertical scrolling is introduced, fields\n will be resized to fit the smaller body area if possible, in order to avoid horizontal\n scrolling also being required.",
        name:"leaveScrollbarGap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('autofitRows')}"],
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:Tree.pathDelim":{
        ref:"attr:Tree.pathDelim",
        flags:"IRWA",
        description:"Specifies the delimiter between node names.  The pathDelim is used to construct a unique\n path to each node. A path can be obtained for any node by calling\n ${isc.DocUtils.linkForRef('method:Tree.getPath')} and can be used to find any node in the tree by calling\n ${isc.DocUtils.linkForRef('method:Tree.find')}.  Note that you can also hand-construct a path - in other words\n you are not required to call ${isc.DocUtils.linkForRef('method:Tree.getPath')} in order to later use\n ${isc.DocUtils.linkForRef('method:Tree.find')} to retrieve it.\n <br><br>\n The pathDelim can be any character or sequence of characters, but must be a unique string\n with respect to the text that can appear in the ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')} that's used\n for naming the nodes.  So for example, if you have the following tree:\n <pre>\n one\n   two\n     three/four\n </pre>\n Then you will be unable to find the <code>three/four</code> node using\n ${isc.DocUtils.linkForRef('method:Tree.find')} if your tree is using the default pathDelim of /.\n In such a case, you can use a different pathDelim for the tree.  For example if you used |\n for the path delim, then you can find the <code>three/four</code> node in the tree above by\n calling <code>tree.find(\"one|two|three/four\")</code>.\n <br><br>\n The pathDelim is used only by ${isc.DocUtils.linkForRef('method:Tree.getPath')} and ${isc.DocUtils.linkForRef('method:Tree.find')} and\n does not affect any aspect of the tree structure or other forms of tree navigation (such as\n via ${isc.DocUtils.linkForRef('method:Tree.getChildren')}).",
        name:"pathDelim",
        seeAlso:[
            {ref:"attr:Tree.nameProperty", description:null},
            {ref:"method:Tree.find", description:null}
        ],
        valueType:"string",
        defaultValue:"\"/\"",
        type:"attr",
        definingClass:"class:Tree"
    },
    "attr:ComboBoxItem.completeOnTab":{
        ref:"attr:ComboBoxItem.completeOnTab",
        flags:"IRW",
        description:"If true, when the pickList is showing, the user can select the current value by hitting\n the <code>Tab</code> key.",
        name:"completeOnTab",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:FormItem.blurItem":{
        ref:"method:FormItem.blurItem",
        flags:"",
        description:"			Takes focus from this form item's focusable element.\n",
        name:"blurItem",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling", "focus"]
    },
    "attr:TabSet.symmetricEdges":{
        ref:"attr:TabSet.symmetricEdges",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n this property determines whether the same edge media will be used regardless of the tab\n bar position, or whether different media should be used (necessary if the edge appearance is\n not symmetrical on all sides).\n <P>\n If this property is set to false the paneContainer edge image URLs will be prefixed with\n the tabBarPosition of the tabSet - for example <code>\"[SKIN]edge_top_T.gif\"</code> rather\n than just <code>\"[SKIN]edge_T.gif\"</code>.\n <P>\n When <code>symmetricEdges</code> is false, custom edge sizes for the pane container may be\n specified via ${isc.DocUtils.linkForRef('attr:TabSet.topEdgeSizes')} et al, and custom edge offsets via \n ${isc.DocUtils.linkForRef('attr:TabSet.topEdgeOffsets')} et al.",
        name:"symmetricEdges",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:TileLayout.autoWrapLines":{
        ref:"attr:TileLayout.autoWrapLines",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('type:LayoutPolicy')} is \"flow\", should we automatically start a new line when there's\n not enough room to fit the next tile on the same line?\n <P>\n If set to false, a new line will only be started if a tile specifies tile.startLine or\n tile.endLine.",
        name:"autoWrapLines",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutPolicy"]
    },
    "attr:DataSourceField.multiple":{
        ref:"attr:DataSourceField.multiple",
        flags:"IR",
        description:"Indicates that this field should always be Array-valued.  If the value derived from\n ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','XML or JSON data')} is singular, it will be wrapped in an Array.\n <P>\n Specifically for XML serialization and deserialization, <code>multiple:true</code> behaves\n similarly to the \n <a href='http://www.google.com/search?hl=en&q=soap+array' onclick=\"window.open('http://www.google.com/search?hl=en&q=soap+array');return false;\">SOAP array idiom</a>, that is,\n there will be a \"wrapper element\" named after the field name, whose contents will be several\n elements of the specified ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field.type')}.  \n <P>\n For example, ${isc.DocUtils.linkForRef('attr:Layout.members')} is declared with <code>type:\"Canvas\",\n multiple:true</code>.  The correct XML format is thus:\n <pre>\n &lt;VLayout&gt;\n     &lt;members&gt;\n         &lt;Canvas ID=\"myCanvas\" ... /&gt;\n         &lt;ListGrid ID=\"myGrid\" .../&gt;\n         &lt;Toolstrip ID=\"myToolStrip\" ... /&gt;\n     &lt;/members&gt;\n &lt;/VLayout&gt;\n </pre>\n <P>\n See ${isc.DocUtils.linkForRef('attr:DataSourceField.childTagName')} for customizing the tagName used for subelements.",
        name:"multiple",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["xmlSerialize", "componentSchema"]
    },
    "attr:ListGrid.canDragRecordsOut":{
        ref:"attr:ListGrid.canDragRecordsOut",
        flags:"IRW",
        description:"Indicates whether records can be dragged from this listGrid and dropped elsewhere.",
        name:"canDragRecordsOut",
        seeAlso:[
            {ref:"attr:ListGridRecord.canDrag", description:null},
            {ref:"attr:ListGridRecord.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "attr:RichTextEditor.controlGroups":{
        ref:"attr:RichTextEditor.controlGroups",
        flags:"IRA",
        description:"An array of control group names (strings) specifying which groups of controls should \n  be included in the editor toolbar.<br>\n  For each control group name, this[controlGroupName] should be defined as an array of \n  ${isc.DocUtils.linkForRef('type:ControlName')}s, allowing the controlGroup to be customized.",
        name:"controlGroups",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[\"fontControls\", \"formatControls\", \"styleControls\", \"colorControls\"]",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('RichTextEditor')}"],
        definingClass:"class:RichTextEditor"
    },
    "classMethod:Page.unregisterKey":{
        ref:"classMethod:Page.unregisterKey",
        flags:"",
        description:" Clears an action registered to fire on a specific a keyPress event via the ${isc.DocUtils.linkForRef('classMethod:Page.registerKey')}\n method. \n",
        name:"unregisterKey",
        seeAlso:[
            {ref:"classMethod:Page.registerKey", description:null}
        ],
        params:[
            {description:"Name of key to clear registry enties for.", optional:false, name:"actionID",
             defaultValue:null, type:"KeyName"},
            {description:"target specified when the action was registered for the key.",
             optional:true, name:"target", defaultValue:null, type:"object"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["KeyRegistry"]
    },
    "attr:Window.modalMask":{
        ref:"attr:Window.modalMask",
        flags:"R",
        description:"A ScreenSpan instance used to darken the rest of a page when a modal window is\n active. To use, set ${isc.DocUtils.linkForRef('attr:Window.showModalMask')} to true, add a CSS style \n \"modalMask\" to the active skin (generally with background-color set), \n and adjust ${isc.DocUtils.linkForRef('attr:Window.modalMaskOpacity')}.",
        name:"modalMask",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["modal", "appearance"]
    },
    "attr:ListGrid.recordEnabledProperty":{
        ref:"attr:ListGrid.recordEnabledProperty",
        flags:"IR",
        description:"Property name on a record that will be checked to determine whether a record is enabled.\n <P>\n Setting this property on a record will effect the visual style and interactivity of\n the record.  If set to <code>false</code> the record (row in a ${isc.DocUtils.linkForRef('class:ListGrid')} or\n ${isc.DocUtils.linkForRef('class:TreeGrid')}) will not highlight when the mouse moves over it, nor will it respond to\n mouse clicks.",
        name:"recordEnabledProperty",
        seeAlso:[
            {ref:"attr:ListGridRecord.enabled", description:null}
        ],
        valueType:"string",
        defaultValue:"\"enabled\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disabledRows')}"],
        definingClass:"class:ListGrid"
    },
    "class:ValuesManager":{
        exampleConfig:"ValuesManager.xml",
        methods:["method:ValuesManager.setDataSource", "method:ValuesManager.getDataSource", 
         "method:ValuesManager.getItem","method:ValuesManager.getMembers", "method:ValuesManager.getMember", 
         "method:ValuesManager.getMemberForField","method:ValuesManager.validate", "method:ValuesManager.getValidatedValues", 
         "method:ValuesManager.setErrors","method:ValuesManager.addFieldErrors", "method:ValuesManager.setFieldErrors", 
         "method:ValuesManager.clearErrors","method:ValuesManager.clearFieldErrors", "method:ValuesManager.getErrors", 
         "method:ValuesManager.getFieldErrors","method:ValuesManager.hasErrors", "method:ValuesManager.hasFieldErrors", 
         "method:ValuesManager.showErrors","method:ValuesManager.showFieldErrors", "method:ValuesManager.getValues", 
         "method:ValuesManager.setValues","method:ValuesManager.clearValues", "method:ValuesManager.getMemberValues", 
         "method:ValuesManager.setMemberValues","method:ValuesManager.rememberValues", "method:ValuesManager.resetValues", 
         "method:ValuesManager.valuesHaveChanged","method:ValuesManager.getValue", "method:ValuesManager.setValue", 
         "method:ValuesManager.clearValue","method:ValuesManager.addMember", "method:ValuesManager.addMembers", 
         "method:ValuesManager.removeMember","method:ValuesManager.removeMembers", "method:ValuesManager.handleHiddenValidationErrors", 
         "method:ValuesManager.editNewRecord","method:ValuesManager.editRecord", "method:ValuesManager.setSaveOperationType", 
         "method:ValuesManager.isNewRecord","method:ValuesManager.getSaveOperationType", "method:ValuesManager.editSelectedData", 
         "method:ValuesManager.saveData","method:ValuesManager.submit", "method:ValuesManager.cancel", 
         "method:ValuesManager.filterData","method:ValuesManager.fetchData"],
        type:"class",
        ref:"class:ValuesManager",
        classAttrs:[],
        treeLocation:["Client Reference/Forms"],
        classMethods:[],
        description:"The ValuesManager manages data from multiple member forms.\n <P>\n If a single logical form needs to be separated into multiple DynamicForm instances for\n Layout purposes (for example, spanning one logical form across multiple Tabs), a\n ValuesManager can be used to make the forms act as one logical form, supporting all\n value-related APIs otherwise called on DynamicForm directly.\n <P>\n A ValuesManager has no visual representation - it is strictly a logical entity, and the\n member forms provide the user interface.  You can initialize a ValuesManager with a set of\n member forms (by setting ${isc.DocUtils.linkForRef('attr:ValuesManager.members')} at init) or add and remove member\n forms dynamically.\n <P>\n Calling ${isc.DocUtils.linkForRef('method:ValuesManager.setValues')} on a ValuesManager will automatically route new\n field values to whichever member form is showing an editor for that field.  Likewise,\n calling ${isc.DocUtils.linkForRef('method:ValuesManager.validate')} will validate all member forms, and\n ${isc.DocUtils.linkForRef('method:ValuesManager.saveData')} will initiate a save operation which aggregates values from\n all member forms.\n <P>\n Like a DynamicForm, a ValuesManager can be databound by setting\n ${isc.DocUtils.linkForRef('attr:ValuesManager.dataSource')}.  In this case all member forms must also be bound to the\n same DataSource.\n <P>\n In general, when working with a ValuesManager and its member forms, call APIs on the\n ValuesManager whenever you are dealing with values that span multiple forms, and only call\n APIs on member forms that are specific to that form or its fields.\n <P>\n Note that, just as a DynamicForm can track values that are not shown in any FormItem, a\n ValuesManager may track values for which there is no FormItem in any member form.  However,\n when using a ValuesManager these extra values are only allowed on the ValuesManager itself.\n Member forms will not track values for which they do not have FormItems.",
        attrs:["attr:ValuesManager.dataSource", "attr:ValuesManager.members", 
         "attr:ValuesManager.unknownErrorMessage","attr:ValuesManager.disableValidation", "attr:ValuesManager.saveOperationType"],
        name:"ValuesManager",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('formSplitting')}"]
    },
    "attr:ListGrid.unfreezeFieldText":{
        ref:"attr:ListGrid.unfreezeFieldText",
        flags:"IRWA",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','this.canFreezeFields')} is true, this string will be shown as the\n title for the menu item to unfreeze a currently frozen field.\n <P>\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>viewer</code> available as a variable mapped to\n the ListGrid instance, and <code>field</code> as a variable pointing to the ListGrid field.\n <P>\n Default value returns \"Unfreeze \" + the field's summary title.",
        name:"unfreezeFieldText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Unfreeze \\${viewer.getSummaryTitle(field)}\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "method:ListGridField.getEditorValueMap":{
        ref:"method:ListGridField.getEditorValueMap",
        flags:"",
        description:" Optional stringMethod to get a ${isc.DocUtils.linkForRef('attr:ListGridField.valueMap')} for a specific field.\n If present this method will be called from ${isc.DocUtils.linkForRef('method:ListGrid.getEditorValueMap')} and the resulting\n valueMap will be used instead of any static specified valueMap for the field.\n",
        name:"getEditorValueMap",
        seeAlso:[],
        params:[
            {description:"Field values for record being edited. Note that this will include                        the current edit values for fields that have not yet been saved.\n                        May be null, if editing a new record.",
             optional:false, name:"values", defaultValue:null, type:"object"},
            {description:"pointer to the listGridField", optional:false, name:"field",
             defaultValue:null, type:"ListGridField"},
            {description:"pointer back to this ListGrid instance.", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"ValueMap for the field (or null if no valueMap required)", type:"valueMap"},
        type:"method",
        definingClass:"object:ListGridField"
    },
    "method:Array.setLength":{
        ref:"method:Array.setLength",
        flags:"",
        description:" Set the length of this list.\n <P>\n If the length of the list is shortened, any elements past the new length of the list are removed.\n If the length is increased, all positions past the old length have the value\n <code>undefined</code>.\n\n",
        name:"setLength",
        seeAlso:[],
        params:[
            {description:"new length", optional:false, name:"length", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "method:ListGridField.cellChanged":{
        ref:"method:ListGridField.cellChanged",
        flags:"",
        description:" Callback fired when field changes value as the result of a cell edit.  Fired only on\n successful save of edit, when the new value doesn't match the value before editing.<br>\n <p>\n Same signature as ${isc.DocUtils.linkForRef('method:ListGrid.cellChanged')}, but defined on a per-field\n basis.\n\n",
        name:"cellChanged",
        seeAlso:[
            {ref:"method:ListGrid.cellChanged", description:null}
        ],
        params:[
            {description:"record for the cell being changed", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"new value for the cell", optional:false, name:"newValue", defaultValue:null,
             type:"any"},
            {description:"old value for the cell", optional:false, name:"oldValue", defaultValue:null,
             type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"},
            {description:"grid where cell was changed.", optional:false, name:"grid", defaultValue:null,
             type:"ListGrid"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:FormItem.itemHoverHTML":{
        ref:"method:FormItem.itemHoverHTML",
        flags:"A",
        description:"  If defined, this method should return the HTML to display in a hover canvas when the \n  user holds the mousepointer over this item.  Return null to suppress the hover \n  canvas altogether.<br>\n  If not defined, <code>dynamicForm.itemHoverHTML()</code> will be evaluated to \n  determine hover content instead.\n\n",
        name:"itemHoverHTML",
        seeAlso:[
            {ref:"attr:FormItem.prompt", description:null},
            {ref:"method:FormItem.itemHover", description:null},
            {ref:"method:FormItem.titleHoverHTML", description:null}
        ],
        params:[
            {description:"Pointer to this item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"This items form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('itemHoverHTML')}"],
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "method:StretchImg.setState":{
        ref:"method:StretchImg.setState",
        flags:"",
        description:" Set the specified image's state to newState and update the displayed image given by\n whichPart, or set the state for all images to newState and update the displayed images\n if whichPart is not provided.\n",
        name:"setState",
        seeAlso:[],
        params:[
            {description:"name for the new state (\"off\", \"down\", etc)", optional:false,
             name:"newState", defaultValue:null, type:"string"},
            {description:"name of the piece to set (\"start\", \"stretch\" or \"end\")											if not specified, sets them all",
             optional:true, name:"whichPart", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "method:ResultTree.dataArrived":{
        ref:"method:ResultTree.dataArrived",
        flags:"",
        description:" This callback fires whenever the resultTree receives new nodes from the server, after\n the new nodes have been integrated into the tree.\n \n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"The parentNode for which children were just loaded", optional:false,
             name:"parentNode", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:ResultTree"
    },
    "attr:Hilite.criteria":{
        ref:"attr:Hilite.criteria",
        flags:"IR",
        description:"Criteria defining what records this hilite should apply to.",
        name:"criteria",
        seeAlso:[],
        valueType:"Criteria or AdvancedCriteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "classMethod:Class.setProperties":{
        ref:"classMethod:Class.setProperties",
        flags:"",
        description:"	Apply a set of properties to a class object, calling the appropriate setter class methods if\n	any are found.\n\n",
        name:"setProperties",
        seeAlso:[],
        params:[
            {description:"objects with properties to add (think named parameters).										all the properties of each argument will be applied one after another\n										so later properties will override",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"object"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:DynamicForm.method":{
        ref:"attr:DynamicForm.method",
        flags:"IRW",
        description:"The mechanism by which form data is sent to the action URL. See FormMethod type\n for details.\n <p>\n <b>NOTE:</b> this is used only in the very rare case that a form is used to submit data\n directly to a URL.  Normal server contact is through \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')}.",
        name:"method",
        setter:["setMethod"],
        seeAlso:[],
        valueType:"FormMethod",
        defaultValue:"DynamicForm.POST",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "classMethod:RPCManager.resendTransaction":{
        ref:"classMethod:RPCManager.resendTransaction",
        flags:"A",
        description:" Resend a suspended transaction to the server.  See ${isc.DocUtils.linkForRef('classMethod:RPCManager.suspendTransaction')} for\n context.  \n <P>\n Note that the transaction must have been previously suspended, and in particular\n suspended validly according to the rules described in the docs for\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.suspendTransaction')}, or undefined results will occur.\n <P>\n You can resend <b>all</b> suspended transactions by calling\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction')} with no arguments.\n\n",
        name:"resendTransaction",
        seeAlso:[
            {ref:"group:relogin", description:null}
        ],
        params:[
            {description:"id of the transaction to be re-sent, or null to resend all                              suspended transactions",
             optional:true, name:"transactionNum", defaultValue:null, type:"id"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "attr:DetailViewer.data":{
        ref:"attr:DetailViewer.data",
        flags:"IRW",
        description:"A single record object or an array of them, specifying data. Note that DetailViewers do\n not observe changes to the data array (in other words they will not automatically\n re-draw when the data provided via this property is altered).",
        name:"data",
        setter:["setData"],
        seeAlso:[],
        valueType:"Array of DetailViewerRecord",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["basics"]
    },
    "method:ActiveXControl.getPluginID":{
        ref:"method:ActiveXControl.getPluginID",
        flags:"",
        description:"  Returns the ID for this ISC ActiveX control object.\n  If the <code>id</code> property was specified for the object, that will be used, otherwise \n  the ID will be auto-generated.\n",
        name:"getPluginID",
        seeAlso:[],
        type:"method",
        definingClass:"class:ActiveXControl"
    },
    "attr:FormItemIcon.neverDisable":{
        ref:"attr:FormItemIcon.neverDisable",
        flags:"IRWA",
        description:"If <code>icon.neverDisable</code> is true, when this form item is disabled, the \n  icon will remain enabled. \n  Note that disabling the entire form will disable all items, together with their \n  icons including those marked as neverDisable - this property only has an effect \n  if the form is enabled and a specific item is disabled within it.",
        name:"neverDisable",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "classAttr:RPCManager.defaultTransport":{
        ref:"classAttr:RPCManager.defaultTransport",
        flags:"IRW",
        description:"Selects the transport use for RPC requests by default.  You can override this setting on\n a per-request basis by setting ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')}.",
        name:"defaultTransport",
        seeAlso:[
            {ref:"attr:RPCRequest.transport", description:null}
        ],
        valueType:"RPCTransport",
        defaultValue:"\"xmlHttpRequest\"",
        type:"classAttr",
        definingClass:"class:RPCManager"
    },
    "class:ImgSplitbar":{
        methods:[],
        ref:"class:ImgSplitbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:ImgSplitbar.target", "attr:ImgSplitbar.vertical", "attr:ImgSplitbar.canDrag", 
         "attr:ImgSplitbar.canCollapse","attr:ImgSplitbar.skinImgDir", "attr:ImgSplitbar.src", "attr:ImgSplitbar.hSrc", 
         "attr:ImgSplitbar.vSrc"],
        description:"Subclass of the ${isc.DocUtils.linkForRef('class:Img')} class. As with the ${isc.DocUtils.linkForRef('class:Splitbar')} class, \n  widgets of this class can be displayed as a resize-bar for widgets \n  in Layouts where showResizeBar is set to true. Provides a different appearance from\n  the <code>Splitbar</code> class.<br>\n  To specify the resizeBar class for some layout, use the ${isc.DocUtils.linkForRef('attr:Layout.resizeBarClass')}\n  property.",
        name:"ImgSplitbar",
        seeAlso:[
            {ref:"class:Layout", description:null},
            {ref:"class:Splitbar", description:null}
        ],
        type:"class"
    },
    "attr:OperationBinding.mail":{
        ref:"attr:OperationBinding.mail",
        flags:"IR",
        description:"Definition of an email message that will be sent as an after-effect of selecting or updating\n data.\n <p>\n Note that if a fixed number of different messages need to be sent, multiple \n <code>&lt;mail&gt;</code> tags may be specified.  For example, one mail could be sent to an \n admin address, and a different message to every member of a user group.",
        name:"mail",
        seeAlso:[],
        valueType:"Mail",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:OperationBinding",
        groups:["mail"]
    },
    "method:ValuesManager.getDataSource":{
        ref:"method:ValuesManager.getDataSource",
        flags:"A",
        description:" Returns the dataSource for this valuesManager.  Will return null if this is not a \n data-bound valuesManager instance.\n",
        name:"getDataSource",
        seeAlso:[],
        returns:{description:"Datasource object for this valuesManager.", type:"DataSource"},
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "method:GridRenderer.getCellValue":{
        ref:"method:GridRenderer.getCellValue",
        flags:"",
        description:" Return the HTML to display in this cell.  Implementing this is required to get a non-empty\n grid.\n\n",
        name:"getCellValue",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"HTML to display in this cell", type:"string"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "classMethod:Validator.addValidator":{
        ref:"classMethod:Validator.addValidator",
        flags:"A",
        description:" Add a new validator type that can be specified as ${isc.DocUtils.linkForRef('attr:Validator.type')} anywhere\n validators are declared, such as ${isc.DocUtils.linkForRef('attr:DataSourceField.validators')} or\n ${isc.DocUtils.linkForRef('attr:FormItem.validators')}.\n <br>\n The <code>condition</code> argument should be a method of the same signature as\n ${isc.DocUtils.linkForRef('method:Validator.condition')}.\n\n",
        name:"addValidator",
        seeAlso:[
            {ref:"classMethod:Validator.addValidators", description:null}
        ],
        params:[
            {description:"type name for the new validator", optional:false, name:"type",
             defaultValue:null, type:"String"},
            {description:"function or expression to evaluate to determine whether                                 validation was successful",
             optional:false, name:"condition", defaultValue:null, type:"StringMethod"}
        ],
        type:"classMethod",
        definingClass:"class:Validator",
        groups:["validation"]
    },
    "attr:ImgButton.autoFit":{
        ref:"attr:ImgButton.autoFit",
        flags:"IRW",
        description:"If true, ignore the specified size of this widget and always size just large\n enough to accomodate the title.  If <code>setWidth()</code> is explicitly called on an\n autoFit:true button, autoFit will be reset to <code>false</code>.\n <P>\n Note that for StretchImgButton instances, autoFit will occur horizontally only, as \n unpredictable vertical sizing is likely to distort the media. If you do want vertical \n auto-fit, this can be achieved by simply setting a small height, and having \n overflow:\"visible\"",
        name:"autoFit",
        setter:["setAutoFit"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["sizing"]
    },
    "method:ListGrid.showField":{
        ref:"method:ListGrid.showField",
        flags:"",
        description:" Force a field to be shown.<br><br>\n\n NOTE: If a field.showIf expression exists, it will be destroyed.\n\n",
        name:"showField",
        seeAlso:[],
        params:[
            {description:"field to show", optional:false, name:"field", defaultValue:null,
             type:"field name or ListGridField"},
            {description:"If passed, don't resize non-explicitly sized columns                                       to fill the available space.",
             optional:true, name:"suppressRelayout", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('columnOrder')}"],
        definingClass:"class:ListGrid"
    },
    "attr:SelectItem.defaultToFirstOption":{
        ref:"attr:SelectItem.defaultToFirstOption",
        flags:"IRW",
        description:"Select the first option as the default value for this SelectItem. \n <P>\n If options are derived from a dataSource, the first value returned by the server will be\n used, otherwise the first value in the valueMap.  Note that setting this property to\n true will trigger a fetch at soon as the form is created, because the form will try to\n establish a default value at that time.\n <P>\n If enabled, this setting overrides ${isc.DocUtils.linkForRef('attr:SelectItem.defaultValue')} and\n ${isc.DocUtils.linkForRef('method:SelectItem.defaultDynamicValue')}.",
        name:"defaultToFirstOption",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:SectionHeader.icon":{
        ref:"attr:SectionHeader.icon",
        flags:"IRA",
        description:"Base filename of the icon that represents open and closed states. The default settings\n also change the icon for disabled sections, so a total of four images are required\n (opened, closed, Disabled_opened, Disabled_closed).\n <P>\n Not shown if ${isc.DocUtils.linkForRef('attr:SectionStackSection.canCollapse')} is false.",
        name:"icon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]SectionHeader/opener.gif\"",
        type:"attr",
        definingClass:"class:SectionHeader"
    },
    "classMethod:Log.clearPriority":{
        ref:"classMethod:Log.clearPriority",
        flags:"",
        description:" Clear the priority setting for a particular category, so that the category's effective\n priority returns to <code>Log.defaultPriority</code><br>\n If the optional second parameter is passed, the specific priority setting for the \n category on that object will be cleared, so logs in that category on that object will\n be logged at the global priority level for the category.\n\n",
        name:"clearPriority",
        seeAlso:[],
        params:[
            {description:"category name", optional:false, name:"category", defaultValue:null,
             type:"String"},
            {description:"Optional instance or class object - if passed                                        clear logging priority for the appropriate category\n                                        on that object.",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:TestFunctionResult.failedExecution":{
        ref:"attr:TestFunctionResult.failedExecution",
        flags:"IRW",
        description:"Set to true if callng the formula or summary format resulted in a JavaScript Error.\n This would generally indicate a reference to non-existant data values.  See \n ${isc.DocUtils.linkForRef('attr:TestFunctionResult.failedGeneration')} for other types of failure.\n <P>\n When set to true, ${isc.DocUtils.linkForRef('attr:TestFunctionResult.errorText')} contains the exception message.",
        name:"failedExecution",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "method:ListGrid.headerClick":{
        ref:"method:ListGrid.headerClick",
        flags:"A",
        description:" Handle a click in the list header.\n <P>\n By default, calls ${isc.DocUtils.linkForRef('method:ListGrid.sort')} to sort by the field that was clicked, or to reverse the\n direction of sorting if already sorted by that field.\n\n",
        name:"headerClick",
        seeAlso:[],
        params:[
            {description:"field number for the header that was clicked", optional:false,
             name:"fieldNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["sorting", "events", "gridHeader"]
    },
    "method:DetailViewer.formatCellValue":{
        ref:"method:DetailViewer.formatCellValue",
        flags:"",
        description:" Optional method to format the value to display for cells in this DetailViewer.\n Note that if ${isc.DocUtils.linkForRef('method:DetailViewerField.formatCellValue')} is specified this method will not\n be called for cells within that field.\n",
        name:"formatCellValue",
        seeAlso:[],
        params:[
            {description:"the raw value of the cell (may be formatted by   ${isc.DocUtils.linkForRef('method:DetailViewerField.formatCellValue')}",
             optional:false, name:"value", defaultValue:null, type:"string"},
            {description:"the record being displayed", optional:false, name:"record", defaultValue:null,
             type:"detailViewerRecord"},
            {description:"the field being displayed", optional:false, name:"field", defaultValue:null,
             type:"detailViewerField"}
        ],
        type:"method",
        definingClass:"class:DetailViewer",
        groups:["i18nMessages"]
    },
    "attr:PickList.pickListWidth":{
        ref:"attr:PickList.pickListWidth",
        flags:"IRW",
        description:"Default width to show the pickList.\n If not specified, the width of this form item's element will be used instead.",
        name:"pickListWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('listComboBox')}"],
        definingClass:"interface:PickList"
    },
    "method:TileGrid.recordClick":{
        ref:"method:TileGrid.recordClick",
        flags:"",
        description:" Executed when the tileGrid receives a 'click' event on a\n tile. The default implementation does nothing -- override to perform some action\n when any record is clicked.<br>\n A record event handler can be specified either as\n a function to execute, or as a string of script to evaluate. If the handler is defined\n as a string of script, all the parameters below will be available as variables for use\n in the script.<br>\n If you want to cancel the click based on the parameters, return false. Otherwise, return \n true so that the click event be registered with the tile.\n\n",
        name:"recordClick",
        seeAlso:[],
        params:[
            {description:"the TileGrid itself", optional:false, name:"viewer", defaultValue:null,
             type:"TileGrid"},
            {description:"the tile that was clicked on", optional:false, name:"tile", defaultValue:null,
             type:"Canvas"},
            {description:"the record that was clicked on", optional:false, name:"record",
             defaultValue:null, type:"TileRecord"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('tilingEditing')}"],
        definingClass:"class:TileGrid",
        groups:["events"]
    },
    "attr:Label.showDownIcon":{
        ref:"attr:Label.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "method:DataSource.addData":{
        ref:"method:DataSource.addData",
        flags:"",
        description:" Perform an \"add\" DataSource operation against this DataSource, to create a new DataSource\n record.\n\n",
        name:"addData",
        seeAlso:[],
        params:[
            {description:"new record", optional:false, name:"newRecord", defaultValue:null,
             type:"Record"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "method:ListGrid.selectRecord":{
        ref:"method:ListGrid.selectRecord",
        flags:"",
        description:" Select/deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Note that this method selects records unconditionally, allowing multiple selected \n records, even when ${isc.DocUtils.linkForRef('attr:ListGrid.selectionType')} is \"single\".  To enforce mutually-exclusive \n record-selection, use ${isc.DocUtils.linkForRef('method:ListGrid.selectSingleRecord')}.\n",
        name:"selectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "classMethod:FileLoader.loadJSFiles":{
        ref:"classMethod:FileLoader.loadJSFiles",
        flags:"",
        description:"\n Loads the specified JS files into the context of the current page.\n\n",
        name:"loadJSFiles",
        seeAlso:[],
        params:[
            {description:"List of URLs pointing to JS files to load.  This can either be a                                string with comma separated URLs or an Array of URLs.",
             optional:false, name:"URLs", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified URLs                                      has completed loading.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "class:ImgTab":{
        methods:[],
        ref:"class:ImgTab",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:ImgTab.capSize", "attr:ImgTab.skinImgDir", "attr:ImgTab.baseStyle", 
         "attr:ImgTab.titleStyle","attr:ImgTab.src", "attr:ImgTab.showRollOver", "attr:ImgTab.showFocus", 
         "attr:ImgTab.showFocused","attr:ImgTab.align"],
        description:"Specialized StretchImgButton used by TabSet/TabBar for tabs",
        name:"ImgTab",
        seeAlso:[],
        type:"class"
    },
    "attr:OperationBinding.exportAs":{
        ref:"attr:OperationBinding.exportAs",
        flags:"IR",
        description:"The format in which the data should be exported.  Default is \"csv\".\n See ${isc.DocUtils.linkForRef('type:ExportFormat')} for more information.",
        name:"exportAs",
        seeAlso:[],
        valueType:"ExportFormat",
        defaultValue:"\"csv\"",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "attr:DataSourceField.group":{
        ref:"attr:DataSourceField.group",
        flags:"IR",
        description:"For use in ${isc.DocUtils.linkForRef('group:componentSchema')}, indicates what group to place the property in when\n editing in Visual Builder.",
        name:"group",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentSchema"]
    },
    "method:FormItem.click":{
        ref:"method:FormItem.click",
        flags:"",
        description:" Called when this FormItem is clicked on.\n <P>\n Note: <code>click()</code> is available on StaticTextItem, BlurbItems, ButtonItem, and\n derivatives.  Other form items (such as HiddenItem) do not support <code>click()</code>.\n\n",
        name:"click",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "method:ColumnTree.getRecord":{
        ref:"method:ColumnTree.getRecord",
        flags:"",
        description:" Retrieve a record by index.  \n <P>\n If <code>colNum</code> is passed, returns the record found in that column at that index,\n or null if the column doesn't exist or the index is too high.\n <P>\n With no <code>colNum</code> parameter, a record's index is it's position counting from the\n first record of the first column and including all records in each column. Note that both\n index and colNum are zero-based - so the first column is column 0, not column 1.\n \n",
        name:"getRecord",
        seeAlso:[],
        params:[
            {description:"index of record to return.", optional:false, name:"index", defaultValue:null,
             type:"integer"},
            {description:"optional index of the column", optional:true, name:"colNum",
             defaultValue:null, type:"integer"}
        ],
        returns:{description:"node at the specified index", type:"TreeNode"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "group:eventBubbling":{
        ref:"group:eventBubbling",
        refs:[],
        name:"eventBubbling",
        seeAlso:[],
        type:"group"
    },
    "attr:FormItemIcon.width":{
        ref:"attr:FormItemIcon.width",
        flags:"IRW",
        description:"If set, this property determines the width of this icon in px.\n      If unset the form item's <code>iconWidth</code> property will be used instead.",
        name:"width",
        seeAlso:[
            {ref:"attr:FormItem.iconWidth", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "attr:Scrollbar.trackEndWidth":{
        ref:"attr:Scrollbar.trackEndWidth",
        flags:"IRA",
        description:"The minimum pixel width of the track end segments (if enabled with showTrackEnds).",
        name:"trackEndWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"12",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["track"]
    },
    "attr:ImgButton.state":{
        flags:"IRWA",
        getter:["getState"],
        type:"attr",
        ref:"attr:ImgButton.state",
        description:"Current \"state\" of this widget. StatefulCanvases will have a different appearance based\n on their current state. By default this is handled by changing the css className applied to\n the StatefulCanvas - see ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')} for a description of how this is\n done.<P>\n For ${isc.DocUtils.linkForRef('class:Img')} or ${isc.DocUtils.linkForRef('class:StretchImg')} based subclasses of StatefulCanvas, the \n appearance may also be updated by changing the src of the rendered image. See\n ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:StretchImgButton.src')} for a description of how the URL \n is modified to reflect the state of the widget in this case.",
        name:"state",
        setter:["setState"],
        valueType:"State",
        seeAlso:[
            {ref:"type:State", description:null},
            {ref:"group:state", description:null}
        ],
        defaultValue:"\"\"",
        groups:["state"],
        definingClass:"class:ImgButton"
    },
    "method:ValuesManager.saveData":{
        ref:"method:ValuesManager.saveData",
        flags:"",
        description:"\n Validate and then save the form's current values to the ${isc.DocUtils.linkForRef('class:DataSource')} this form is\n bound to.\n <p>\n If client-side validators are defined, they are executed first, and if any errors are\n found the save is aborted and the form will show the errors.\n <p>\n If client-side validation passes, a ${isc.DocUtils.linkForRef('class:DSRequest')} will be sent, exactly as though\n ${isc.DocUtils.linkForRef('method:DataSource.addData')} or ${isc.DocUtils.linkForRef('method:DataSource.updateData')} had been called with \n ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','the form's values')} as data.  The\n ${isc.DocUtils.linkForRef('attr:DSRequest.operationType')} will be either \"update\" or \"add\", depending on the\n current ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')}.\n <P>\n On either a client-side or server-side validation failure, validation errors will be\n displayed in the form.  Visible items within a DynamicForms will be redrawn to display\n errors. Validation failure occuring on hidden items, or DataSource fields with no \n associated form items may be handled via ${isc.DocUtils.linkForRef('method:DynamicForm.handleHiddenValidationErrors')}\n or ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')}.\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.validationURL','validationURL')} can be set to do validation against a\n different URL from where the form will ultimately save, as part of an incremental\n upgrade strategy for Struts and Struts-like applications.\n\n",
        name:"saveData",
        seeAlso:[],
        params:[
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('addOperation')}"],
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "group:formIcons":{
        ref:"group:formIcons",
        refs:["method:FormItemIcon.click", "method:FormItemIcon.keyPress", "method:FormItem.iconClick", 
         "method:FormItem.iconKeyPress","object:FormItemIcon", "attr:FormItem.icons", "attr:FormItem.defaultIconSrc", 
         "attr:FormItem.showOverIcons","attr:FormItem.showFocusedIcons", "attr:FormItem.iconVAlign", "attr:FormItem.iconHeight", 
         "attr:FormItem.iconWidth","attr:FormItem.iconPrompt", "attr:FormItem.showIcons", "attr:FormItemIcon.src", 
         "attr:FormItemIcon.showOver","attr:FormItemIcon.showFocused", "attr:FormItemIcon.showFocusedWithItem", 
         "attr:FormItemIcon.neverDisable","attr:FormItemIcon.width", "attr:FormItemIcon.height", "attr:FormItemIcon.prompt", 
         "attr:TextAreaItem.iconVAlign"],
        name:"formIcons",
        seeAlso:[],
        type:"group"
    },
    "method:MultiView.getSelection":{
        ref:"method:MultiView.getSelection",
        flags:"",
        description:" Returns all selected records, as an Array.\n\n",
        name:"getSelection",
        seeAlso:[],
        returns:{description:"list of records, empty list if nothing selected",
         type:"Array of ListGridRecord"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:TextItem.maskPadChar":{
        ref:"attr:TextItem.maskPadChar",
        flags:"IRWA",
        description:"Character that is used to fill required empty mask positions\n to display text while control has no focus.",
        name:"maskPadChar",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\" \"",
        type:"attr",
        definingClass:"class:TextItem"
    },
    "type:SCClassName":{
        ref:"type:SCClassName",
        description:"Name of a SmartClient Class, that is, a Class that has been created via\n ${isc.DocUtils.linkForRef('classMethod:isc.defineClass')}, including Classes built into SmartClient, such as \"ListGrid\".",
        name:"SCClassName",
        seeAlso:[],
        type:"type"
    },
    "method:Progressbar.getLength":{
        ref:"method:Progressbar.getLength",
        flags:"",
        description:" Returns the current width of a horizontal progressbar, or height of a vertical progressbar.\n\n",
        name:"getLength",
        seeAlso:[],
        returns:{description:"the length of the progressbar", type:"number"},
        type:"method",
        definingClass:"class:Progressbar"
    },
    "attr:TabBar.baseLineCapSize":{
        ref:"attr:TabBar.baseLineCapSize",
        flags:"IR",
        description:"Set ${isc.DocUtils.linkForRef('attr:StretchImg.capSize')} for the ${isc.DocUtils.linkForRef('group:baseLine')} stretchImg.",
        name:"baseLineCapSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:TabBar",
        groups:["baseLine"]
    },
    "attr:DataSource.useHttpProxy":{
        ref:"attr:DataSource.useHttpProxy",
        flags:"IR",
        description:"Like ${isc.DocUtils.linkForRef('attr:OperationBinding.useHttpProxy')}, but serves as a default for this DataSource\n that may be overriden by individual operationBindings.",
        name:"useHttpProxy",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration"]
    },
    "attr:StretchImgButton.showSelectedIcon":{
        ref:"attr:StretchImgButton.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:DetailViewerField.type":{
        ref:"attr:DetailViewerField.type",
        flags:"IR",
        description:"Specifies the type of this DetailViewerField.  By default (value is <code>null</code>)\n the field shows a field title on the left and the field value on the right.  There are\n two special values for this attribute:\n <ul>\n <li>\"header\" - If you specify type \"header\", the field spans both the field name and\n field value columns and contains text defined in the ${isc.DocUtils.linkForRef('attr:DetailViewerField.value')}\n attribute with the style specifed by ${isc.DocUtils.linkForRef('attr:DetailViewer.headerStyle')}.  You can use this\n field type as a titled separator.\n <li>\"separator\" - If you specify type \"separator\", the field spans both the field name\n and the field value columns with no text, and is styled using the style specified via\n ${isc.DocUtils.linkForRef('attr:DetailViewer.separatorStyle')}.  The height of the separator field can be controlled\n via ${isc.DocUtils.linkForRef('attr:DetailViewerField.height')}.\n <li>\"image\" For viewing, a thumbnail image is rendered in the field.  The URL of the\n image is the value of the field, and should be absolute. The size of the image is\n controlled by ${isc.DocUtils.linkForRef('attr:DetailViewerField.imageSize')},\n ${isc.DocUtils.linkForRef('attr:DetailViewerField.imageWidth')}, ${isc.DocUtils.linkForRef('attr:DetailViewerField.imageHeight')}\n </ul>",
        name:"type",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "class:Mail":{
        methods:[],
        ref:"class:Mail",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:["attr:Mail.templateFile", "attr:Mail.messageTemplate", "attr:Mail.to", "attr:Mail.cc", 
         "attr:Mail.bcc","attr:Mail.from", "attr:Mail.replyTo", "attr:Mail.subject", "attr:Mail.multiple", 
         "attr:Mail.messageData"],
        description:"A <code>Mail</code> object describes an email that will be sent automatically by the \n SmartClient Server as part of a normal ${isc.DocUtils.linkForRef('class:DataSource')} operation.  Mail definitions\n are part of an ${isc.DocUtils.linkForRef('class:OperationBinding')}, and they can only be specified in a \n DataSource's XML definition.  Despite the declarative nature of mail definitions, templating\n support via Velocity makes it very flexible; it is very easy to include values from records\n just fetched from permanent storage by this operation, for example.  See \n ${isc.DocUtils.linkForRef('group:velocitySupport')} for an overview of SmartClient Server's pervasive support for \n templated declarative operations using Velocity. \n <P>\n The default context for the message is the dsResponse.data; that is, $foo in the\n message refers to the field \"foo\" in the DataSource response (the equivalent of\n calling dsResponse.getFieldValue(\"foo\") in Java).  You also have access to the normal\n Velocity variables described in the overview linked to above.\n <P>\n If multiple records appear in the dsResponse, one message will be sent per record\n in the response unless \"multiple\" is set to false.  This allows a set of\n recipients to be selected by a \"fetch\" operationBinding and messages sent to each.\n This also means that if no data is selected by a fetch or affected by an update,\n no mail is sent.\n <P>\n If you have the Transaction Chaining feature, all of the variables normally\n accessible in other Velocity contexts are available in the message template as\n well, for example, $responseData.last('order').orderId.  See \n ${isc.DocUtils.linkForRef('attr:DSRequestModifier.value','transaction chaining')} for more details.\n <p>\n You can provide the body of the email directly in the ${isc.DocUtils.linkForRef('attr:Mail.messageTemplate','messageTemplate')}\n tag, or you can provide a filename in the ${isc.DocUtils.linkForRef('attr:Mail.templateFile','templateFile')} attribute,\n which tells SmartClient to use the contents of that file as the body of the email.  If you \n provide neither of these, your email will have no body; if you provide both, SmartClient will\n ignore the file and just use the content directly provided.\n <p>",
        name:"Mail",
        seeAlso:[],
        type:"class",
        groups:["mail"]
    },
    "method:TabSet.scrollBack":{
        ref:"method:TabSet.scrollBack",
        flags:"",
        description:" If there is not enough space to display all the tabs in this tabSet, this method will \n scroll the previous tab (that first tab that is clipped at the beginning of the tab-bar) \n into view.\n",
        name:"scrollBack",
        seeAlso:[],
        type:"method",
        definingClass:"class:TabSet"
    },
    "classAttr:ValidatorTypes.isOneOf":{
        ref:"classAttr:ValidatorTypes.isOneOf",
        flags:"IR",
        description:"Tests whether the value for this field matches any value from an arbitrary\n list of acceptable values.  The set of acceptable values is specified via\n the <code>list</code> property on the validator, which should be set to an array of\n values. If validator.list is not supplied, the valueMap for the field will be used.\n If there is no valueMap, not providing validator.list is an error.",
        name:"isOneOf",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:ListGrid.loadingMessage":{
        ref:"attr:ListGrid.loadingMessage",
        flags:"IR",
        description:"If you have a databound listGrid and you scroll out of the currently loaded dataset, by\n default you will see blank rows until the server returns the data for those rows.  The\n loadingMessage attribute allows you to specify arbitrary html that will be shown in each\n such \"blank\" record while the data for that record is loading.",
        name:"loadingMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&amp;nbsp;\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["emptyMessage", "i18nMessages"]
    },
    "method:Button.setBaseStyle":{
        ref:"method:Button.setBaseStyle",
        flags:"",
        description:" Sets the base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n",
        name:"setBaseStyle",
        seeAlso:[],
        params:[
            {description:"new base style", optional:false, name:"style", defaultValue:null,
             type:"className"}
        ],
        type:"method",
        definingClass:"class:Button"
    },
    "method:Array.max":{
        ref:"method:Array.max",
        flags:"",
        description:"\n 			Returns the largest number in the array, skipping non-numeric values.\n\n			If the start and/or end are given, searches the specified subset of the list.\n\n",
        name:"max",
        seeAlso:[],
        params:[
            {description:"optional start index (default is 0)", optional:true, name:"start",
             defaultValue:null, type:"number"},
            {description:"optional end index (default is list.length)", optional:true, name:"end",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"maximum of all items in the list, or null if all values are non-numeric",
         type:"number"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "method:Array.getUniqueItems":{
        ref:"method:Array.getUniqueItems",
        flags:"",
        description:" Return a list of each unique item in this list exactly once.\n <P>\n Returns in the same order they were found in the list.\n <P>\n Usage example:<br>\n &nbsp;&nbsp;&nbsp;&nbsp;uniqueList = myArray.getProperty(\"foo\").getUniqueItems();\n\n",
        name:"getUniqueItems",
        seeAlso:[],
        returns:{description:"list of each unique item in the list", type:"array"},
        type:"method",
        definingClass:"object:Array",
        groups:["subset"]
    },
    "method:CubeGrid.facetRemoved":{
        ref:"method:CubeGrid.facetRemoved",
        flags:"",
        description:" Notification fired when a facet is removed.\n \n",
        name:"facetRemoved",
        seeAlso:[
            {ref:"method:CubeGrid.facetAdded", description:"<br><i>methodType</i>  handler"}
        ],
        params:[
            {description:"facetId that was removed", optional:false, name:"facetId", defaultValue:null,
             type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "method:TextItem.setSelectionRange":{
        ref:"method:TextItem.setSelectionRange",
        flags:"",
        description:" Puts focus into this form item and selects characters between the given indices.\n Only applies to drawn text based items.\n",
        name:"setSelectionRange",
        seeAlso:[],
        params:[
            {description:"selection starting character index", optional:false, name:"start",
             defaultValue:null, type:"integer"},
            {description:"end of selection character index", optional:false, name:"end",
             defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:TextItem"
    },
    "method:Array.map":{
        ref:"method:Array.map",
        flags:"",
        description:" Return an array where the value of item <code>i</code> is the result of calling the provided\n function on item <code>i</code> in this array.\n <P>\n The function to call can either be provided directly as a function object, in which case it\n is invoked with the item as the first argument, or can be provided as the String name of a\n method present on each item, which will be invoked.  In the latter case, if any item is null\n or lacks the named method, null will be returned for that item.\n <P>\n Examples:<PRE>\n    // line up widgets at 20 pixels from page edge\n    [widget1, widget2].map(\"setPageLeft\", 20);\n\n    // find furthest right widget\n    [widget1, widget2].map(\"getPageRight\").max();\n </PRE>\n \n",
        name:"map",
        seeAlso:[],
        params:[
            {description:"function object, or name of method", optional:false, name:"method",
             defaultValue:null, type:"string or function"},
            {description:"arguments to pass to the function or method                                           invoked on each item",
             optional:true, name:"(arguments 1-N)", defaultValue:null, type:"any"}
        ],
        returns:{description:"array of returned values", type:"array"},
        type:"method",
        definingClass:"object:Array",
        groups:["iteration"]
    },
    "method:Canvas.getSnapTo":{
        ref:"method:Canvas.getSnapTo",
        flags:"",
        description:" Return the snapTo value of this object\n\n",
        name:"getSnapTo",
        seeAlso:[],
        returns:{description:"snapTo", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "type:Object":{
        ref:"type:Object",
        description:"An ordinary JavaScript as obtained by \"new Object()\" or via \n ${isc.DocUtils.linkForRef('type:ObjectLiteral','Object Literal')} syntax.\n <P>\n Methods that return Objects or take Objects as parameters make use of the ability of a\n JavaScript Object to contain an arbitrary set of named properties, without requiring\n declaration in advance.  This capability makes it possible to use a JavaScript Object much\n like a HashMap in Java or .NET, but without the need to call get() or set() to create and\n retrieve properties.\n <P>\n For example if you created an Object using ${isc.DocUtils.linkForRef('type:ObjectLiteral','Object Literal')} syntax\n like so:\n <pre>\n    var request = {\n        actionURL : \"/foo.do\",\n        showPrompt:false\n    };\n </pre>\n You could then access it's properties like so:\n <pre>\n    var myActionURL = request.actionURL;\n    var myShowPrompt = request.showPrompt;\n </pre>\n .. and you could assign new values to those properties like so:\n <pre>\n    request.actionURL = \"<i>newActionURL</i>\";\n    request.showPrompt = <i>newShowPromptSetting</i>;\n </pre>\n Note that while JavaScript allows you to get and set properties in this way on any Object,\n SmartClient components require that if a setter or getter exists, it must be called, or no\n action will occur.  For example, if you had a ${isc.DocUtils.linkForRef('class:ListGrid')} and you wanted to change the\n ${isc.DocUtils.linkForRef('attr:ListGrid.showHeader','showHeader')} property:\n <pre>\n     myListGrid.setShowHeader(false); // correct\n     myListGrid.showHeader = false; // incorrect (nothing happens)\n </pre>\n All documented attributes have ${isc.DocUtils.linkForRef('group:flags','flags')} (eg IRW) that indicate when direct\n property access is allowed or not.",
        name:"Object",
        seeAlso:[],
        type:"type"
    },
    "classMethod:Hover.show":{
        ref:"classMethod:Hover.show",
        flags:"",
        description:" Displays a standard Hover canvas containing the specified HTML content.<br>\n This method may also be called to modify the content of the hover if it is already showing.\n Call ${isc.DocUtils.linkForRef('classMethod:Hover.hide')} to hide the canvas again.<br>\n A common use case for calling this method is to asynchronously fetch detail data from the\n server about some component, and display it in the Hover canvas when the data is returned.\n Note that in this case you will typically need to verify that the user is still hovering \n over the component in question before calling Hover.show() - if the user has moved the mouse \n off the component, the information will not apply to whatever is now under the mouse. \n Suggested approaches for handling this are to either use a ${isc.DocUtils.linkForRef('method:Canvas.mouseOut')} handler\n to track when the user moves off the component, or checking ${isc.DocUtils.linkForRef('classMethod:EventHandler.getTarget')}\n as part of the asynchronous callback\n <p>\n The default Hover canvas position will be based on the mouse pointer position, adjusted by\n ${isc.DocUtils.linkForRef('classAttr:Hover.leftOffset')} and ${isc.DocUtils.linkForRef('classAttr:Hover.topOffset')}. If this position would render the\n Hover canvas partially clipped, it will be automatically modified to ensure the Hover \n is entirely visible.\n",
        name:"show",
        seeAlso:[],
        params:[
            {description:"contents for the hover", optional:false, name:"contents", defaultValue:null,
             type:"html"},
            {description:"object containing attributes for managing the hover canvas'   appearance. Valid properties include:<ul>\n  <li>left, top, width, height\n  <li>baseStyle\n  <li>opacity\n  <li>wrap\n  <li>moveWithMouse [overrides ${isc.DocUtils.linkForRef('classAttr:Hover.moveWithMouse')}]\n </ul>",
             optional:false, name:"properties", defaultValue:null, type:"Label Properties"}
        ],
        type:"classMethod",
        definingClass:"class:Hover"
    },
    "attr:MenuButton.showMenuButtonImage":{
        ref:"attr:MenuButton.showMenuButtonImage",
        flags:"IR",
        description:"show menu button image (up / down arrowhead) for this menu button",
        name:"showMenuButtonImage",
        setter:["setShowMenuButtonImage"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "method:Canvas.resized":{
        ref:"method:Canvas.resized",
        flags:"",
        description:"  Observable method called whenever a Canvas changes size. Note that if this canvas is\n ${isc.DocUtils.linkForRef('attr:Canvas.overflow','overflow:\"visible\"')}, and is waiting for a queued redraw (see \n ${isc.DocUtils.linkForRef('method:Canvas.isDirty')}), the value for ${isc.DocUtils.linkForRef('method:Canvas.getVisibleWidth')} and\n ${isc.DocUtils.linkForRef('method:Canvas.getVisibleHeight')} will be unreliable until <code>redraw()</code> fires.\n",
        name:"resized",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:Date.setFormatter":{
        ref:"method:Date.setFormatter",
        deprecated:"As of SmartClient 5.5 use the static methods              ${isc.DocUtils.linkForRef('classMethod:Date.setNormalDisplayFormat')} and \n              ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat')} to set default formatters for all dates",
        flags:"",
        description:"  Set the formatter for this date object to the method name passed in.  After this call\n  wherever appropriate SmartClient components will use this formatter function to return \n  the date as a string.\n",
        name:"setFormatter",
        seeAlso:[],
        params:[
            {description:"name of a date formatter method on this Date", optional:false,
             name:"functionName", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:ListGrid.cellClick":{
        ref:"method:ListGrid.cellClick",
        flags:"",
        description:" Called when a cell receives a click event.\n\n",
        name:"cellClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('cellClicks')}"],
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:Slider.value":{
        ref:"attr:Slider.value",
        flags:"IRW",
        getter:["getValue"],
        description:"The slider value. This value should lie between the minValue and maxValue and increases as\n the thumb is moved up (for a vertical slider) or right (for a horizontal slider) unless\n flipValues is set to true.",
        name:"value",
        setter:["setValue"],
        seeAlso:[
            {ref:"attr:Slider.minValue", description:null},
            {ref:"attr:Slider.maxValue", description:null},
            {ref:"attr:Slider.flipValues", description:null},
            {ref:"attr:Slider.showValue", description:null}
        ],
        valueType:"float",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:Slider"
    },
    "method:Canvas.scrollByPercent":{
        ref:"method:Canvas.scrollByPercent",
        flags:"",
        description:"  Scroll this widget by some percentage of scroll size in either (or both) direction(s).\n\n",
        name:"scrollByPercent",
        seeAlso:[],
        params:[
            {description:"Percentage to scroll horizontally. Will accept either                                      a numeric percent value, or a string like \"10%\".",
             optional:false, name:"dX", defaultValue:null, type:"number | string"},
            {description:"Percentage to scroll horizontally. Will accept either                                      a numeric percent value, or a string like \"10%\".",
             optional:false, name:"dY", defaultValue:null, type:"number | string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:Canvas.getVSnapOrigin":{
        ref:"method:Canvas.getVSnapOrigin",
        flags:"",
        description:" Get an offset to be used when calculating snap positioning. Returns 0 by default.\n\n",
        name:"getVSnapOrigin",
        seeAlso:[
            {ref:"method:Canvas.getVSnapPosition", description:null}
        ],
        params:[
            {description:"the child that is being snapped", optional:true, name:"snapChild",
             defaultValue:null, type:"Canvas"}
        ],
        returns:{description:"The offset to use when snapping", type:"integer"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:FacetValue.collapsed":{
        ref:"attr:FacetValue.collapsed",
        flags:"IR",
        description:"For tree facets, initial collapse state for this node.  Defaults to facet.collapsed",
        name:"collapsed",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"facet.collapsed",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:MenuItem.enabled":{
        ref:"attr:MenuItem.enabled",
        flags:"IR",
        description:"Affects the visual style and interactivity of the menu item.  If set to\n <code>false</code>, the menu item will not respond to mouse rollovers or clicks.\n <p>\n If you need to set this state dynamically, use ${isc.DocUtils.linkForRef('method:MenuItem.enableIf')} instead.",
        name:"enabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:MenuItem",
        groups:["menuBasic"]
    },
    "attr:Canvas.hoverDelay":{
        ref:"attr:Canvas.hoverDelay",
        flags:"IRW",
        description:"If <code>this.canHover</code> is true, how long should the mouse be kept over this\n widget before the hover event is fired",
        name:"hoverDelay",
        seeAlso:[
            {ref:"attr:Canvas.canHover", description:null},
            {ref:"method:Canvas.hover", description:null}
        ],
        valueType:"number",
        defaultValue:"300",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:WSRequest.xmlResult":{
        ref:"attr:WSRequest.xmlResult",
        flags:"IR",
        description:"Valid only with ${isc.DocUtils.linkForRef('method:WebService.callOperation')}.  If set, do not transform XML results to\n JavaScript.  Instead just return the XML nodes selected by the passed XPath or recordName,\n or all nodes within the SOAP body if no XPath was passed.",
        name:"xmlResult",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:WSRequest"
    },
    "method:ResultSet.find":{
        ref:"method:ResultSet.find",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.find')}.  Checks only loaded rows and will not trigger a fetch.\n",
        name:"find",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"first matching object or null if not found", type:"Object"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access", "find"]
    },
    "attr:FormItem.form":{
        ref:"attr:FormItem.form",
        flags:"R",
        description:"A Read-Only pointer to this formItem's DynamicForm widget.",
        name:"form",
        seeAlso:[],
        valueType:"DynamicForm",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:DataBoundComponent.dragDataAction":{
        ref:"attr:DataBoundComponent.dragDataAction",
        flags:"IRW",
        description:"Indicates what to do with data dragged into another DataBoundComponent. See\n          DragDataAction type for details.",
        name:"dragDataAction",
        seeAlso:[],
        valueType:"DragDataAction",
        defaultValue:"Canvas.MOVE",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('gridsDragMove')}", 
         "${isc.DocUtils.linkForExampleId('gridsDragCopy')}"],
        definingClass:"interface:DataBoundComponent",
        groups:["dragging"]
    },
    "method:DataBoundComponent.deselectRecords":{
        ref:"method:DataBoundComponent.deselectRecords",
        flags:"",
        description:"\n Deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecords(records, false)</code>\n\n",
        name:"deselectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to deselect", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "classAttr:Operators.greaterThanTitle":{
        ref:"classAttr:Operators.greaterThanTitle",
        flags:"IR",
        description:"Title for the \"greaterThan\" operator",
        name:"greaterThanTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"greater than\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.showGridSummary":{
        ref:"attr:ListGridField.showGridSummary",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, should this field show a summary value.\n If unset, this field will show a summary value in the summary row if an\n explicit ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')} is specified or if a\n ${isc.DocUtils.linkForRef('classMethod:SimpleType.getDefaultSummaryFunction','default summary function')} is defined\n for the specified field type.",
        name:"showGridSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "object:AdvancedCriteria":{
        methods:[],
        type:"object",
        inheritsFrom:"Criterion",
        ref:"object:AdvancedCriteria",
        classAttrs:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        classMethods:[],
        description:"AdvancedCriteria is a format for representing search criteria which may include\n operators on field values such as \"less than\", or may include sub-clauses such as several \n criteria applied to fields joined by an \"OR\" operator.\n <P>\n SmartClient DataSources can use AdvancedCriteria to search a list of ${isc.DocUtils.linkForRef('object:Record')}s, and\n the SmartClient Java Server can translate AdvancedCriteria to either SQL or Hibernate\n queries (<b>Note:</b> The server-side AdvancedCriteria handling feature is only available \n with the <b>Power</b> and <b>Enterprise</b> Editions of SmartClient; the Pro Edition is \n limited to ordinary criteria handling on the server side).<p>\n If the entire dataset is cached locally, SmartClient can perform AdvancedCriteria filtering \n on the client, avoiding a server call.\n <P>\n An AdvancedCriteria is an ordinary JavaScript object which can be created directly\n with JavaScript literal notation.  For example:\n <pre>\n var advancedCriteria = {\n        _constructor:\"AdvancedCriteria\",\n        operator:\"and\",\n        criteria:[\n            // this is a Criterion\n            { fieldName:\"salary\", operator:\"lessThan\", value:\"80000\" },\n            { operator:\"or\", criteria:[\n                  { fieldName:\"title\", operator:\"iContains\", value:\"Manager\" },\n                  { fieldName:\"reports\", operator:\"notNull\" }\n              ]  \n            }\n        ]\n    }\n </pre>\n An AdvancedCriteria is in effect a ${isc.DocUtils.linkForRef('object:Criterion')} that has been marked with \n _constructor:\"AdvancedCriteria\" to mark it as complete criteria.\n <P>\n When passed to the SmartClient Server, AdvancedCriteria are automatically translated\n to Java Objects, identically to other JavaScript data, by the rules given under \n ${isc.DocUtils.linkForRef('attr:RPCRequest.data')}.\n <P>\n Other servers may receive AdvancedCriteria in the most convenient format, for example,\n a ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} might translate AdvancedCriteria directly into\n a SQL-like language, or serialize to XML using ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}.",
        attrs:[],
        name:"AdvancedCriteria",
        seeAlso:[],
        serverDS:"allowed",
        groups:["advancedFilter"]
    },
    "attr:StretchImgButton.title":{
        flags:"IRW",
        getter:["getTitle"],
        type:"attr",
        ref:"attr:StretchImgButton.title",
        description:"The text title to display in this button.",
        name:"title",
        setter:["setTitle"],
        valueType:"string",
        seeAlso:[],
        defaultValue:"varies",
        groups:["basics"],
        definingClass:"class:StretchImgButton"
    },
    "method:HTMLFlow.setContentsURL":{
        ref:"method:HTMLFlow.setContentsURL",
        flags:"",
        description:" Change the URL this component loads content from.  Triggers a fetch for content from the new\n URL.\n <p>\n Can also be called with no arguments to reload content from the existing ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsURL')}.\n <P>\n This feature relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.\n\n",
        name:"setContentsURL",
        seeAlso:[],
        params:[
            {description:"URL to retrieve contents from", optional:true, name:"url", defaultValue:null,
             type:"URL"},
            {description:"Parameters to send to the contentsURL.  Merged with                            <code>component.contentsURLParams</code> if both are set.",
             optional:true, name:"params", defaultValue:null, type:"Object"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('loadHtmlPages')}"],
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:ToolStripResizer.skinImgDir":{
        ref:"attr:ToolStripResizer.skinImgDir",
        flags:"IR",
        description:"Path to resizer image.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"images/ToolStrip/\"",
        type:"attr",
        definingClass:"class:ToolStripResizer"
    },
    "attr:Canvas.scrollbarSize":{
        ref:"attr:Canvas.scrollbarSize",
        flags:"IRWA",
        getter:["getScrollbarSize"],
        description:"How thick should we make the scrollbars for this canvas.<br>\n			NOTE: has no effect if showCustomScrollbars is false.",
        name:"scrollbarSize",
        seeAlso:[
            {ref:"method:Canvas.getScrollbarSize", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:FormItem.setLeft":{
        ref:"method:FormItem.setLeft",
        flags:"A",
        description:" For a form with ${isc.DocUtils.linkForRef('attr:DynamicForm.itemLayout','itemLayout')}:\"absolute\" only, set the left\n coordinate of this form item.\n <P>\n Causes the form to redraw.\n",
        name:"setLeft",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "classMethod:RPCManager.sendRequest":{
        ref:"classMethod:RPCManager.sendRequest",
        flags:"",
        description:"\n Send the passed <code>RPCRequest</code> to the server.  If queuing is in effect, this queues\n the request instead.\n\n",
        name:"sendRequest",
        seeAlso:[],
        params:[
            {description:"RPCRequest to send to the server", optional:false, name:"rpcRequest",
             defaultValue:null, type:"RPCRequest Properties"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "attr:Calendar.eventNameFieldTitle":{
        ref:"attr:Calendar.eventNameFieldTitle",
        flags:"IR",
        description:"The title for the event name field in the quick event dialog",
        name:"eventNameFieldTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Event Name\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "classMethod:RPCManager.loginRequired":{
        ref:"classMethod:RPCManager.loginRequired",
        flags:"",
        description:" Called when a session timeout is encountered while trying to do a background RPC.  See\n ${isc.DocUtils.linkForRef('group:relogin','Relogin')}.\n <P>\n The transaction with the passed <code>transactionId</code> is suspended, and should either\n be ${isc.DocUtils.linkForRef('classMethod:RPCManager.clearTransaction','cleared')} or ${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction','resent')}\n after the user has been re-authenticated.  \n <P>\n The <code>rpcRequest</code> parameter can be used to determine whether the suspended\n transaction can simply be dropped (eg, it's periodic polling request).\n <P>\n The <code>rpcResponse</code> parameter has rpcResponse.data set to the raw text of the\n response that triggered <code>loginRequired()</code>.  Some very advanced relogin strategies\n may need to inspect the raw response to get information needed for re-authentication.\n\n",
        name:"loginRequired",
        seeAlso:[],
        params:[
            {description:"id of the transaction", optional:false, name:"transactionNum",
             defaultValue:null, type:"id"},
            {description:"first RPCRequest of the transaction", optional:false, name:"rpcRequest",
             defaultValue:null, type:"RPCRequest"},
            {description:"RPCResponse containing the session timeout response that                                  caused loginRequired() to be invoked",
             optional:false, name:"rpcResponse", defaultValue:null, type:"RPCResponse"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager",
        groups:["relogin"]
    },
    "attr:ListGridField.inputFormat":{
        ref:"attr:ListGridField.inputFormat",
        flags:"IRWA",
        description:"For fields of type <code>\"date\"</code>, if this is an editable listGrid, this property \n allows you to specify the ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat','inputFormat')} applied to the editor \n for this field.",
        name:"inputFormat",
        seeAlso:[
            {ref:"attr:ListGridField.displayFormat", description:null}
        ],
        valueType:"DateInputFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:Canvas.keyPress":{
        ref:"method:Canvas.keyPress",
        flags:"A",
        description:"\n Executed when a key is pressed and released on a focusable widget (${isc.DocUtils.linkForRef('attr:Canvas.canFocus')}:\n true). \n <P>\n Use ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKey')} to find out the ${isc.DocUtils.linkForRef('type:KeyName','keyName')} of the key that\n was pressed, and use ${isc.DocUtils.linkForRef('classMethod:EventHandler.shiftKeyDown')} and related functions to determine\n whether modifier keys were down.\n\n",
        name:"keyPress",
        seeAlso:[
            {ref:"attr:Canvas.canFocus", description:null}
        ],
        returns:{description:"false to suppress native behavior in response to the keyPress, and prevent                    this event from bubbling to this widget's parent, or true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('keyboardEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:StatefulCanvas.setDisabled":{
        ref:"method:StatefulCanvas.setDisabled",
        flags:"A",
        description:" Enable or disable this object\n",
        name:"setDisabled",
        seeAlso:[],
        params:[
            {description:"true if this widget is to be disabled", optional:false, name:"disabled",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["enable", "state"]
    },
    "method:CubeGrid.facetLabelOut":{
        ref:"method:CubeGrid.facetLabelOut",
        flags:"",
        description:" StringMethod handler fired when mouseout occurs over a facet label\n\n",
        name:"facetLabelOut",
        seeAlso:[],
        params:[
            {description:"ID of the appropriate facet", optional:false, name:"facetId",
             defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "attr:ColorPicker.colorButtonSize":{
        ref:"attr:ColorPicker.colorButtonSize",
        flags:"IR",
        description:"Width and height of the basic color boxes (they are always square, and they are\n all the same size).",
        name:"colorButtonSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Selection.deselect":{
        ref:"method:Selection.deselect",
        flags:"",
        description:"			Deselect a particular item\n",
        name:"deselect",
        seeAlso:[],
        params:[
            {description:"object to select", optional:false, name:"item", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:MenuButton.menuAnimationEffect":{
        ref:"attr:MenuButton.menuAnimationEffect",
        flags:"IRWA",
        description:"Allows you to specify an animation effect to apply to the menu when it is being shown.\n Valid options are \"none\" (no animation), \"fade\", \"slide\" and \"wipe\".\n If unspecified falls through to <code>menu.showAnimationEffect</code>",
        name:"menuAnimationEffect",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "type:Selected":{
        ref:"type:Selected",
        values:[
            {description:"StatefulCanvas should show focused state", value:"StatefulCanvas.FOCUSED"},
            {description:"StatefulCanvas is selected", value:"StatefulCanvas.SELECTED"},
            {description:"StatefulCanvas is not selected", value:"StatefulCanvas.UNSELECTED"}
        ],
        description:null,
        name:"Selected",
        seeAlso:[],
        type:"type",
        groups:["state"]
    },
    "method:DataBoundComponent.setSort":{
        ref:"method:DataBoundComponent.setSort",
        flags:"",
        description:" Sort this component by a list of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.  If the component's data is not a \n ${isc.DocUtils.linkForRef('class:ResultSet')}, only the first specifier is applied.\n \n",
        name:"setSort",
        seeAlso:[],
        params:[
            {description:"List of ${isc.DocUtils.linkForRef('object:SortSpecifier')} objects, one    per sort-field and direction",
             optional:false, name:"sortSpecifiers", defaultValue:null, type:"Array of SortSpecifier"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:DateChooser.alternateStyleSuffix":{
        ref:"attr:DateChooser.alternateStyleSuffix",
        flags:"IRW",
        description:"The text appended to the style name when using ${isc.DocUtils.linkForRef('attr:DateChooser.alternateWeekStyles')}.",
        name:"alternateStyleSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Dark\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:FormItem.focusInItem":{
        ref:"method:FormItem.focusInItem",
        flags:"",
        description:"			Move the keyboard focus into this item's focusable element\n",
        name:"focusInItem",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling", "focus"]
    },
    "method:Class.observe":{
        ref:"method:Class.observe",
        flags:"",
        description:" Take an arbitrary action whenever a method is called on an instance.<br><br>\n\n When you observe some method of another object, eg:<br>\n			<code>thisObject.observe(thatObject, \"someMethod\", \"observer.foo()\")</code><br><br>\n\n When <code>thatObject.someMethod()</code> is called,<br>\n			<code>thisObject.foo()</code> <br>\n will be called automatically, after the observed method completes.<br><br>\n\n Action is typically a string expression.  Available variables:\n <ul>\n    <li> observed: target of the observation, that is, object passed to observe()\n    <li> observer: object that observes, that is, object that observe() was called on\n    <li> returnVal: return value of observed function\n </ul>\n\n An unlimited number of observers can observe any message, they will all be notified\n automatically in the order that the observations were set up.<br><br>\n\n NOTES: \n - observation also works on JavaScript Array objects\n - a method may trigger an observation of itself by another object, either through code \n   within the method itself or within an observer's action.  In this case the observation\n   will be set up, but the new observation action will not fire as part of this thread.\n   When the method is called again in the future the newly added observer will be fired.\n\n\n",
        name:"observe",
        seeAlso:[
            {ref:"method:Class.ignore", description:null}
        ],
        params:[
            {description:"object to observe", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"name of the method to observe", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"String for the function to call.										In this string, \n											<code>observer</code> is the object that is observing, \n											<code>this</code> is the object that is being observed\n\n										If <code>action</code> is not specified, \n											<code>observer.methodName()</code> will be called.",
             optional:true, name:"action", defaultValue:null, type:"string"}
        ],
        returns:{description:"true == observation set up, false == observation not set up", type:"boolean"},
        type:"method",
        definingClass:"class:Class",
        groups:["observation"]
    },
    "method:ListGrid.getRecord":{
        ref:"method:ListGrid.getRecord",
        flags:"A",
        description:" Return the pointer to a particular record by record number.\n Synonym for ${isc.DocUtils.linkForRef('method:ListGrid.getCellRecord')}.\n\n",
        name:"getRecord",
        seeAlso:[
            {ref:"method:ListGrid.getCellRecord", description:null},
            {ref:"method:ListGrid.getEditedRecord", description:null}
        ],
        params:[
            {description:"row index of record to return.", optional:false, name:"recordNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Record object for the row.", type:"ListGridRecord"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:TabSet.rightEdgeOffsets":{
        ref:"attr:TabSet.rightEdgeOffsets",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeOffsets</code>, \n <code>rightEdgeOffsets</code>, <code>topEdgeOffsets</code> and <code>bottomEdgeOffsets</code> \n propertes allow the offsets of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge offsets where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"rightEdgeOffsets",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:Window.closeButton":{
        ref:"attr:Window.closeButton",
        flags:"R",
        description:"Button show in the header that will close this Window by calling ${isc.DocUtils.linkForRef('method:Window.closeClick')}.",
        name:"closeButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "method:MultiView.deselectRecord":{
        ref:"method:MultiView.deselectRecord",
        flags:"",
        description:"\n Deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecord(record, false)</code>\n\n",
        name:"deselectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to deselect", optional:false, name:"record",
             defaultValue:null, type:"Record | number"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:ListGrid.canPickFields":{
        ref:"attr:ListGrid.canPickFields",
        flags:"IRW",
        description:"Indicates whether the field picker item and submenu should be present in the header\n context menu",
        name:"canPickFields",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["sorting"]
    },
    "attr:TabSet.showPartialEdges":{
        ref:"attr:TabSet.showPartialEdges",
        flags:"IRA",
        description:"If the paneContainer for this tab set is showing ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','edges')}, setting this\n attribute to <code>true</code> will set the paneContainer to show\n ${isc.DocUtils.linkForRef('attr:Canvas.customEdges','customEdges')} for the three sides opposing the tabBarPosition.",
        name:"showPartialEdges",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:DataBoundComponent.useAllDataSourceFields":{
        ref:"attr:DataBoundComponent.useAllDataSourceFields",
        flags:"IRW",
        description:"If true, the set of fields given by the \"default binding\" (see \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}) is used, with any fields specified in\n <code>component.fields</code> acting as overrides that can suppress or modify the\n display of individual fields, without having to list the entire set of fields that\n should be shown.\n <P>\n If <code>component.fields</code> contains fields that are not found in the DataSource,\n they will be shown after the most recently referred to DataSource field.  If the new\n fields appear first, they will be shown first.\n <P>\n ${isc.DocUtils.linkForExampleId('validationFieldBinding', 'This example')} shows a mixture of component\n fields and DataSource fields, and how they interact for validation.",
        name:"useAllDataSourceFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('validationFieldBinding')}"],
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "attr:Canvas.snapAxis":{
        ref:"attr:Canvas.snapAxis",
        flags:"IRW",
        description:"Describes which axes to apply snap-to-grid to.\n Valid values are Canvas.HORIZONTAL, Canvas.VERTICAL and Canvas.BOTH",
        name:"snapAxis",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"attr:Canvas.snapResizeToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapResizeToGrid", description:null}
        ],
        valueType:"string",
        defaultValue:"Canvas.BOTH",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Toolbar.addButtons":{
        ref:"method:Toolbar.addButtons",
        flags:"",
        description:" Add a list of buttons to the toolbar\n",
        name:"addButtons",
        seeAlso:[],
        params:[
            {description:"list of button object initializers.", optional:true, name:"buttons",
             defaultValue:null, type:"Array of objects"},
            {description:"position to add the new buttons at", optional:true, name:"position",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Toolbar"
    },
    "attr:TabSet.scrollerHSrc":{
        ref:"attr:TabSet.scrollerHSrc",
        flags:"IR",
        description:"If this TabSet is showing ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller','tab scroller buttons')}, and \n ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is true, this property governs the base\n URL for the tab bar back and forward scroller button images for horizontal tab bars [IE for\n tab sets with ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} set to \"top\" or \"bottom\"].\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is false, \n ${isc.DocUtils.linkForRef('attr:TabSet.scrollerSrc')} will be used instead.\n <P>\n To get the path to the image to display, this base URL will be modified as follows:\n <ul>\n <li>If appropriate a state suffix of <code>\"Down\"</code> or <code>\"Disabled\"</code> will be\n     appended.</li>\n <li>A suffix of <code>\"forward\"</code> or <code>\"back\"</code> will be appended for the\n     forward or backward scrolling button.</li>\n </ul>\n For example - if the scrollerHSrc is set to <code>\"[SKIN]hscroll.gif\"</code>, the image\n displayed for the back-scroller button on a tabSet with <code>tabBarPosition</code> set to\n \"top\" and <code>symmetricScroller</code> set to true would be one of \n <code>\"[SKIN]hscroll_back.gif\"</code>, <code>\"[SKIN]hscroll_Down_back.gif\"</code>,\n and <code>\"[SKIN]hscroll_Disabled_back.gif\"</code>.\n <P>\n Note that for best results the media should be sized to match the scroller button sizes, \n determined by ${isc.DocUtils.linkForRef('attr:TabSet.tabBarThickness')} and ${isc.DocUtils.linkForRef('attr:TabSet.scrollerButtonSize')}.",
        name:"scrollerHSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricScroller", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]hscroll.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "method:Canvas.setClassName":{
        ref:"method:Canvas.setClassName",
        deprecated:"As of SmartClient version 5.5, use ${isc.DocUtils.linkForRef('method:Canvas.setStyleName')} instead.",
        flags:"A",
        description:" Sets the CSS class for this widget\n",
        name:"setClassName",
        seeAlso:[],
        params:[
            {description:"new CSS class name (must be defined previously)", optional:false,
             name:"newClass", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "group:dynamicLoading":{
        ref:"group:dynamicLoading",
        refs:[],
        name:"dynamicLoading",
        seeAlso:[],
        type:"group"
    },
    "attr:MenuButton.showMenuBelow":{
        ref:"attr:MenuButton.showMenuBelow",
        flags:"IRW",
        description:"The menu drops down below the menu button.\n			Set to false if the menu should appear above the menu button.",
        name:"showMenuBelow",
        setter:["setShowMenuBelow"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "attr:TextAreaItem.width":{
        ref:"attr:TextAreaItem.width",
        flags:"IRW",
        description:"default width of this item",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"150",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["appearance"]
    },
    "attr:RestDataSource.xmlRecordXPath":{
        ref:"attr:RestDataSource.xmlRecordXPath",
        flags:"IR",
        description:"<code>recordXPath</code> mapping to the data node of XML returned by the server.\n Applies if this.dataFormat is set to <code>\"xml\"</code>.<br>\n The default value will pick up data from a response structured as follows:<br>\n <pre>\n &lt;response&gt;\n    &lt;status&gt;0&lt;/status&gt;\n    &lt;data&gt;\n      &lt;record&gt;\n          &lt;field1&gt;value&lt;/field1&gt;\n          &lt;field2&gt;value&lt;/field2&gt;\n      &lt;/record&gt;\n      &lt;record&gt;\n          &lt;field1&gt;value&lt;/field1&gt;\n          &lt;field2&gt;value&lt;/field2&gt;\n      &lt;/record&gt;\n    &lt;/data&gt;\n &lt;/response&gt;\n </pre>",
        name:"xmlRecordXPath",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"/response/data/*\"",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:DynamicForm.titleOrientation":{
        flags:"IRW",
        getter:["getTitleOrientation"],
        type:"attr",
        ref:"attr:DynamicForm.titleOrientation",
        description:"Default orientation for titles for items in this form.  ${isc.DocUtils.linkForRef('type:TitleOrientation')}\n lists valid options.\n <P>\n Note that titles on the left or right take up a cell in tabular\n ${isc.DocUtils.linkForRef('group:formLayout','form layouts')}, but titles on top do not.",
        name:"titleOrientation",
        valueType:"TitleOrientation",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('formLayoutTitles')}"],
        defaultValue:"\"left\"",
        groups:["formTitles"],
        definingClass:"class:DynamicForm"
    },
    "attr:TabSet.paneContainerClassName":{
        ref:"attr:TabSet.paneContainerClassName",
        flags:"IRW",
        description:"CSS style used for the paneContainer.",
        name:"paneContainerClassName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "class:ComboBoxItem":{
        methods:["method:ComboBoxItem.defaultDynamicValue", "method:ComboBoxItem.fetchData", 
         "method:ComboBoxItem.getDisplayFieldName","method:ComboBoxItem.getValueFieldName", "method:ComboBoxItem.getSelectedRecord", 
         "method:ComboBoxItem.getPickListFilterCriteria","method:ComboBoxItem.getClientPickListData", 
         "method:ComboBoxItem.filterClientPickListData","method:ComboBoxItem.dataArrived"],
        type:"class",
        ref:"class:ComboBoxItem",
        classAttrs:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        classMethods:[],
        description:"The Combobox is a text input field which can show a list of options via a drop-down\n PickList.\n <p>\n The set of options will be filtered based on the current value in the text field, so only\n options that match what has been typed so far will be displayed.\n The set of options can be derived from a ValueMap or dynamically retrieved from a\n dataSource.  See the ${isc.DocUtils.linkForRef('interface:PickList')} interface for further settings.\n <P>\n Note that to select the first option as a default value for the item,\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.defaultToFirstOption')} may be set.",
        attrs:["attr:ComboBoxItem.defaultValue", "attr:ComboBoxItem.defaultToFirstOption", 
         "attr:ComboBoxItem.showHintInField","attr:ComboBoxItem.mask", "attr:ComboBoxItem.maskSaveLiterals", 
         "attr:ComboBoxItem.maskPadChar","attr:ComboBoxItem.maskPromptChar", "attr:ComboBoxItem.maskOverwriteMode", 
         "attr:ComboBoxItem.showPickerIcon","attr:ComboBoxItem.pickerIconWidth", "attr:ComboBoxItem.pickerIconHeight", 
         "attr:ComboBoxItem.pickerIconSrc","attr:ComboBoxItem.showPickListOnKeypress", "attr:ComboBoxItem.completeOnTab", 
         "attr:ComboBoxItem.optionDataSource","attr:ComboBoxItem.pickListFields", "attr:ComboBoxItem.optionFilterContext", 
         "attr:ComboBoxItem.optionOperationId","attr:ComboBoxItem.displayField", "attr:ComboBoxItem.valueField", 
         "attr:ComboBoxItem.filterLocally","attr:ComboBoxItem.pickListCriteria", "attr:ComboBoxItem.showOptionsFromDataSource", 
         "attr:ComboBoxItem.pickListProperties","attr:ComboBoxItem.sortField", "attr:ComboBoxItem.autoFetchData", 
         "attr:ComboBoxItem.filterWithValue","attr:ComboBoxItem.textMatchStyle", "attr:ComboBoxItem.showAllOptions", 
         "attr:ComboBoxItem.separatorRows"],
        implementsInterfaces:["PickList"],
        name:"ComboBoxItem",
        seeAlso:[
            {ref:"interface:PickList", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('listComboBox')}", 
         "${isc.DocUtils.linkForExampleId('listComboBox')}"]
    },
    "attr:SelectItem.showHintInField":{
        ref:"attr:SelectItem.showHintInField",
        flags:"IRWA",
        description:"If showing a hint for this form item, should it be shown within the field?\n <P>CSS style for the hint is ${isc.DocUtils.linkForRef('attr:SelectItem.textBoxStyle')} with the suffix\n \"Hint\" appended to it.",
        name:"showHintInField",
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "type:DragIntersectStyle":{
        ref:"type:DragIntersectStyle",
        values:[
            {description:"Look for drop targets that are under the current mouse cursor position.",
             value:"\"mouse\""},
            {description:"Look for drop targets by intersection of the entire rect of the drag               target with the droppable target.",
             value:"\"rect\""}
        ],
        description:"Different styles of determining intersection: with mouse or entire rect of target",
        name:"DragIntersectStyle",
        seeAlso:[],
        type:"type",
        groups:["dragdrop"]
    },
    "group:enable":{
        ref:"group:enable",
        title:"Enabling and Disabling",
        refs:["method:FormItem.setDisabled", "method:FormItem.isDisabled", "method:FormItem.enable", 
         "method:FormItem.disable","method:Canvas.setEnabled", "method:Canvas.setDisabled", "method:Canvas.enable", 
         "method:Canvas.disable","method:Canvas.isDisabled", "method:Canvas.isEnabled", "method:StatefulCanvas.setDisabled", 
         "method:Button.setDisabled","method:ImgButton.setDisabled", "method:StretchImgButton.setDisabled", 
         "attr:Canvas.disabled","attr:Canvas.enabled"],
        description:"Disabled components do not respond to mouse or keyboard events, and change appearance to\n indicate they are disabled.",
        name:"enable",
        seeAlso:[],
        type:"group"
    },
    "attr:CubeGrid.canSortFacets":{
        ref:"attr:CubeGrid.canSortFacets",
        flags:"IRW",
        description:"If true, sort controls will be shown on FacetHeaders.\n <P>\n When clicked, sort controls call ${isc.DocUtils.linkForRef('method:CubeGrid.sortByFacetId')}.",
        name:"canSortFacets",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:CubeGrid.hiliteCell":{
        ref:"method:CubeGrid.hiliteCell",
        flags:"",
        description:"\n      apply a hilite to a specific cell.\n      Note: can be called either as \n          hiliteCell(cellObject, hiliteID) or \n          hiliteCell(row, column, hiliteID)\n\n      <br><i>methodType</i> action\n",
        name:"hiliteCell",
        seeAlso:[],
        params:[
            {description:"cell to hilite / row of cell to hilite", optional:false, name:"cellObj",
             defaultValue:null, type:"cell object | row number"},
            {description:"hilite ID / column of cell to hilite", optional:false, name:"hiliteID",
             defaultValue:null, type:"string | column number"},
            {description:"optional third parameter - hilite ID.", optional:true, name:"",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true if the cell was successfully hilited.", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["hiliting"]
    },
    "method:ComboBoxItem.getPickListFilterCriteria":{
        ref:"method:ComboBoxItem.getPickListFilterCriteria",
        flags:"A",
        description:" ${isc.DocUtils.linkForRef('group:stringMethods','StringMethod')} to return filter criteria for options displayed for\n this item.  \n <P>\n The criteria returned by this method are used to decide which options should appear in\n the drop-down ${isc.DocUtils.linkForRef('interface:PickList')} shown by this ComboBox.\n <P>\n While the user is typing in a value, this method will return the typed-in value as part\n of the criteria, so that only matching values are shown.  Matching is determined by the\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.textMatchStyle','textMatchStyle')}.  \n <P>\n If the user explicitly shows the down-down pickList, via either clicking on the drop\n down icon or using the <i>Ctrl+Arrow Down</i> key combination, the typed-in value is \n ignored for filtering.\n <P>\n If included in the criteria, the typed-in value will be included as a value for the \n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.displayField','displayField')} (or for the\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.valueField','valueField')} if <code>this.displayField</code> is\n unspecified).\n <P>\n Static criteria, specified via ${isc.DocUtils.linkForRef('attr:ComboBoxItem.pickListCriteria','pickListCriteria')},\n will always be included.\n <P>\n If you are implementing your own <code>getPickListFilterCriteria()</code> the\n <b>read-only</b> property ${isc.DocUtils.linkForRef('attr:ComboBoxItem.filterWithValue','this.filterWithValue')} can\n be read to determine whether the ComboBox would ordinarily ignore the typed-in value for\n filtering.  Note that in addition to cases where the user explicitly shows the pickList,\n <code>filterWithValue</code> will also be <code>true</code> during a call to \n ${isc.DocUtils.linkForRef('method:ComboBoxItem.fetchData')} on a databound comboBox.\n    \n",
        name:"getPickListFilterCriteria",
        seeAlso:[],
        returns:{description:"criteria to be used for databound or local filtering", type:"Criteria"},
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "method:GridRenderer.rowDoubleClick":{
        ref:"method:GridRenderer.rowDoubleClick",
        flags:"A",
        description:" Called when a row receives a double click event.\n\n",
        name:"rowDoubleClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:OperationBinding.spoofResponses":{
        ref:"attr:OperationBinding.spoofResponses",
        flags:"IR",
        description:"For a DataSource contacting a ${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace','WSDL web service')}, setting\n this flag means the DataSource doesn't actually attempt to contact the server but generates\n a sample response instead, based on the XML Schema of the response message embedded in the\n WSDL.\n <P>\n The spoofed response will include all complexType elements and will fill in appropriate\n values by type for all simpleType elements, although the spoofed data will not conform to\n all xs:restriction declarations (eg xs:pattern).\n <P>\n Note that if your WSDL does not fully describe the response format (some WSDL services just\n have a placeholder &lt;xs:any&gt; element), SmartClient can only produce a partial\n response.  To use a hand-generated sample response, just save an XML file to disk and use\n the ${isc.DocUtils.linkForRef('attr:OperationBinding.dataURL')} setting to point to it.",
        name:"spoofResponses",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "method:TileLayout.setTileVMargin":{
        ref:"method:TileLayout.setTileVMargin",
        flags:"",
        description:" sets the vertical margin of tiles\n",
        name:"setTileVMargin",
        seeAlso:[],
        params:[
            {description:"width", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "attr:CanvasItem.canvas":{
        ref:"attr:CanvasItem.canvas",
        flags:"IRW",
        description:"The canvas that will be displayed inside this item.  You can pass an instance you've \n already created, or its global ID as a String.  If none is passed, one will be \n auto-created using the overrideable defaults: ${isc.DocUtils.linkForRef('attr:CanvasItem.canvasProperties')} and\n ${isc.DocUtils.linkForRef('attr:CanvasItem.canvasConstructor')}",
        name:"canvas",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CanvasItem"
    },
    "attr:DataSource.testData":{
        deprecated:"in favor of ${isc.DocUtils.linkForRef('attr:DataSource.cacheData')}",
        flags:"IRW",
        type:"attr",
        ref:"attr:DataSource.testData",
        description:"For a client-only DataSource, a set of records to use as a dataset, specified as an\n Array of JavaScript Objects.\n <P>\n See ${isc.DocUtils.linkForRef('group:clientOnlyDataSources','this discussion')} for ways to populate a\n client-only DataSource with test data.",
        name:"testData",
        seeAlso:[
            {ref:"attr:DataSource.clientOnly", description:null}
        ],
        valueType:"Array of Object",
        defaultValue:"null",
        serverDS:"allowed",
        examples:["${isc.DocUtils.linkForExampleId('localDataSource')}"],
        definingClass:"class:DataSource",
        groups:["clientOnlyDataSources"]
    },
    "type:ControlName":{
        ref:"type:ControlName",
        values:[
            {description:"A button to make the current selection bold.", value:"\"boldSelection\""},
            {description:"A button to make the current selection italic.",
             value:"\"italicSelection\""},
            {description:"A button to make the current selection underlined.",
             value:"\"underlineSelection\""},
            {description:"A select item allowing the user to change the font of the current                     text selection.",
             value:"\"fontSelector\""},
            {description:"A select item allowing the user to change the font                       size of the current text selection.",
             value:"\"fontSizeSelector\""},
            {description:"A button to left-align the selected text.", value:"\"alignLeft\""},
            {description:"A button to right-align the selected text.", value:"\"alignRight\""},
            {description:"A button to center the selected text", value:"\"alignCenter\""},
            {description:"A button to justify the selected line of text.", value:"\"justify\""},
            {description:"A color-picker allowing the user to set the text color", value:"\"color\""},
            {description:"A color picker allowing the user to set the text background  color",
             value:"\"backgroundColor\""}
        ],
        description:"<code>ControlNames</code> are strings used to specify which UI controls should appear\n  in the editor toolbar.<br>",
        name:"ControlName",
        seeAlso:[],
        type:"type"
    },
    "attr:ListGrid.editOnFocus":{
        ref:"attr:ListGrid.editOnFocus",
        flags:"IRWA",
        description:"Should we start editing when this widget receives focus (if this ListGrid supports\n editing)?\n <P>\n Note that this property being set to true will cause editing to occur on a single\n click, even if ${isc.DocUtils.linkForRef('attr:ListGrid.editEvent')} is <code>\"doubleClick\"</code>, because single clicking\n the grid will place keyboard focus there automatically.",
        name:"editOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:MultiSortPanel.maxLevels":{
        ref:"attr:MultiSortPanel.maxLevels",
        flags:"IR",
        description:"The maximum number of levels of sorting that can be applied.  Since each sort-property or\n field-name can be used only once in a given multi-sort operation, if no maxLevels value\n or a value larger than the total number of available properties is specified, it will \n default to the total number of available properties.",
        name:"maxLevels",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "method:Canvas.mouseMove":{
        ref:"method:Canvas.mouseMove",
        flags:"A",
        description:"\n Executed when the mouse moves within this widget.  No default implementation.\n\n",
        name:"mouseMove",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customMouseEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:TabSet.setTabPane":{
        ref:"method:TabSet.setTabPane",
        flags:"",
        description:" Apply a new ${isc.DocUtils.linkForRef('attr:Tab.pane','pane')} to an existing tab in this tabSet\n",
        name:"setTabPane",
        seeAlso:[],
        params:[
            {description:"Tab to update (may be referenced by ID or index)", optional:false,
             name:"tab", defaultValue:null, type:"number | string | Tab"},
            {description:"new Pane for the tab", optional:false, name:"pane", defaultValue:null,
             type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:Scrollbar.showCorner":{
        ref:"attr:Scrollbar.showCorner",
        flags:"IRA",
        description:"If true, displays a corner piece at the bottom end of a vertical scrollbar, or the\n right end of a horizontal scrollbar. This is typically set only when both horizontal\n and vertical scrollbars are displayed and about the same corner.",
        name:"showCorner",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["corner"]
    },
    "attr:DataBoundComponent.dataFetchMode":{
        ref:"attr:DataBoundComponent.dataFetchMode",
        flags:"IRW",
        description:"How to fetch and manage records retrieve from the server.  See ${isc.DocUtils.linkForRef('type:FetchMode')}.\n <P>\n This setting only applies to the ${isc.DocUtils.linkForRef('class:ResultSet')} automatically created by calling\n ${isc.DocUtils.linkForRef('method:DataBoundComponent.fetchData')}.  If a pre-existing ResultSet is passed to setData() instead, it's\n existing setting for ${isc.DocUtils.linkForRef('attr:ResultSet.fetchMode')} applies.",
        name:"dataFetchMode",
        seeAlso:[],
        valueType:"FetchMode",
        defaultValue:"\"paged\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "method:TileGrid.deselectRecords":{
        ref:"method:TileGrid.deselectRecords",
        flags:"",
        description:"\n Deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecords(records, false)</code>\n\n",
        name:"deselectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to deselect", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "method:ListGrid.getExpansionComponent":{
        ref:"method:ListGrid.getExpansionComponent",
        flags:"",
        description:" Gets the embedded-component to show as a given record's expansionComponent.  This \n component is then housed in ${isc.DocUtils.linkForRef('attr:ListGrid.expansionLayout','a VLayout')} and embedded \n into a record's row.\n <P>\n By default, this method returns one of a set of builtin components, according to the \n value of ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')}.  You can override this method \n to return any component you wish to provide as an expansionComponent.\n \n",
        name:"getExpansionComponent",
        seeAlso:[],
        params:[
            {description:"record to get the expansionComponent for", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"}
        ],
        returns:{description:"the component to embed", type:"Canvas or Canvas Properties"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "class:LayoutSpacer":{
        methods:[],
        ref:"class:LayoutSpacer",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:[],
        description:"Add a LayoutSpacer to a Layout to take up space just like a normal member, without actually\n drawing anything.  Semantically equivalent to using an empty canvas, but higher performance\n for this particular use case.",
        name:"LayoutSpacer",
        seeAlso:[],
        type:"class"
    },
    "attr:Window.bodyConstructor":{
        ref:"attr:Window.bodyConstructor",
        flags:"IRWA",
        description:"The name of the widget class (as a string) to use for the body. If unset the appropriate\n constructor type will be determined as follows:<br>\n - if ${isc.DocUtils.linkForRef('attr:Window.items')} is defined as an array of widgets, and ${isc.DocUtils.linkForRef('attr:Window.contentLayout')} \n   is not set to <code>\"none\"</code>, bodyConstructor defaults to a ${isc.DocUtils.linkForRef('class:VLayout')}<br>\n - if ${isc.DocUtils.linkForRef('attr:Window.src')} is set, bodyConstructor defaults to an ${isc.DocUtils.linkForRef('class:HTMLFlow')}<br>\n - otherwise bodyConstructor will default to a simple ${isc.DocUtils.linkForRef('class:Canvas')}<br>\n Note that if this property is overridden for some window, the specified constructor \n should be a subclass of one of these defaults to ensure the window renders out as \n expected.",
        name:"bodyConstructor",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "attr:Operator.ID":{
        ref:"attr:Operator.ID",
        flags:"IR",
        description:"Unique id for an operator, which appears within ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} as the\n ${isc.DocUtils.linkForRef('object:Operator')} property.\n <P>\n A list of built-in identifiers is ${isc.DocUtils.linkForRef('type:OperatorId','here')}.",
        name:"ID",
        seeAlso:[],
        valueType:"OperatorId",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "attr:Mail.to":{
        ref:"attr:Mail.to",
        flags:"IR",
        description:"Comma-separated list of recipients.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n If you specify a \"To:\" line in the body of your email, recipient addresses will be parsed \n and this property will be ignored.",
        name:"to",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "method:ValuesManager.rememberValues":{
        ref:"method:ValuesManager.rememberValues",
        flags:"",
        description:"			Make a snapshot of the current set of values, so we can reset to them later.\n			Creates a new object, then adds all non-method properties of values\n			to the new object.  Use <code>resetValues()</code> to revert to these values.\n          Note that this method is automatically called when the values for this form are\n          set programmatically via a call to ${isc.DocUtils.linkForRef('method:DynamicForm.setValues')}.\n\n",
        name:"rememberValues",
        seeAlso:[],
        returns:{description:"copy of current form values", type:"object"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "attr:Menu.autoDraw":{
        ref:"attr:Menu.autoDraw",
        flags:"IRWA",
        description:"Menus will not draw on initialization, until they're explicitly show()n",
        name:"autoDraw",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Menu"
    },
    "type:LogicalOperator":{
        ref:"type:LogicalOperator",
        values:[
            {description:"true if all criteria are true", value:"\"and\""},
            {description:"true if any criteria are true", value:"\"or\""},
            {description:"true if all criteria are false", value:"\"not\""}
        ],
        description:"Operators that can evaluate a set of criteria and produce a combined result.",
        name:"LogicalOperator",
        seeAlso:[],
        type:"type"
    },
    "attr:TreeGrid.data":{
        ref:"attr:TreeGrid.data",
        flags:"IRW",
        description:"A ${isc.DocUtils.linkForRef('class:Tree')} object containing of nested ${isc.DocUtils.linkForRef('object:TreeNode')}s to \n display as rows in this TreeGrid.  \n The <code>data</code> property will typically not be explicitly specified for \n databound TreeGrids, where the data is returned from the server via databound component\n methods such as <code>fetchData()</code>",
        name:"data",
        setter:["setData"],
        seeAlso:[],
        valueType:"Tree",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["data"]
    },
    "attr:SpinnerItem.maskPromptChar":{
        ref:"attr:SpinnerItem.maskPromptChar",
        flags:null,
        description:"Not applicable to a SpinnerItem.",
        name:"maskPromptChar",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:SpinnerItem"
    },
    "class:SimpleTile":{
        methods:["method:SimpleTile.getInnerHTML", "method:SimpleTile.getRecord"],
        ref:"class:SimpleTile",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/TileGrid"],
        attrs:["attr:SimpleTile.baseStyle", "attr:SimpleTile.creator"],
        description:"Default class used by a ${isc.DocUtils.linkForRef('class:TileGrid')} to render each tile.  See ${isc.DocUtils.linkForRef('attr:TileGrid.tile')}.\n <P>\n SimpleTiles should not be created directly, instead, use a TileGrid and provide data and\n SimpleTile instances are created for you.",
        name:"SimpleTile",
        seeAlso:[],
        type:"class"
    },
    "group:errorIcon":{
        ref:"group:errorIcon",
        refs:["attr:FormItem.errorIconHeight", "attr:FormItem.errorIconWidth", 
         "attr:FormItem.errorIconSrc","attr:FormItem.showErrorIcon"],
        name:"errorIcon",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.chartData":{
        ref:"method:ListGrid.chartData",
        flags:"",
        description:" Chart the data in this listGrid as a multi-series chart.\n <P>\n Each row provides a series of data.  Each series of data is labelled by a value from one\n column, called the <code>labelField</code>.  \n <P>\n For example, cell values are sales figures, and fields are \"Product\", \"August\",\n \"September\", \"October\".  In this case each row gives a series: sales figures for each of 3\n months.  The <code>labelField</code> in this case is the \"Product\" field, meaning each row\n represents sales figures for each of 3 months for a particular product.  This dataset\n can be charted via any multi-series chart: stacked or clustered bar or column chart, line\n chart with multiple lines, or area chart (stacked lines).\n <P>\n By default, all visible fields other than the label field are assumed to be labels for\n series values, but an explicit list of fields can be provided as <code>dataFields</code>.\n <P>\n By default, all data is charted if all data is loaded, otherwise, data visible in the\n viewport is charted.  An explicit set of rows can be provided via <code>dataRows</code>.\n \n",
        name:"chartData",
        seeAlso:[],
        params:[
            {description:"name of the field", optional:false, name:"labelField", defaultValue:null,
             type:"String"},
            {description:"optional list of fields to use as labels.  By      default, all fields are used.",
             optional:true, name:"dataFields", defaultValue:null, type:"Array of String"},
            {description:"set of records to chart.  Can be obtained by eg      ${isc.DocUtils.linkForRef('method:ResultSet.getRange','grid.data.getRange()')}.",
             optional:true, name:"dataRows", defaultValue:null, type:"Array of ListGridRecord"},
            {description:"properties to pass to the created chart", optional:true,
             name:"chartProperties", defaultValue:null, type:"Chart Properties"},
            {description:"if true, use the labelField as the \"first\" set of labels,      for example, as the bar labels in a stacked bar chart, whereas the second set of labels\n      would appear as the legend.",
             optional:true, name:"labelFieldFirst", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"created Chart instance", type:"Chart"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('gridCharting')}"],
        definingClass:"class:ListGrid"
    },
    "attr:DateChooser.showCancelButton":{
        ref:"attr:DateChooser.showCancelButton",
        flags:"IRW",
        description:"Determines whether the \"Cancel\" button will be displayed.",
        name:"showCancelButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:FormItem.top":{
        ref:"attr:FormItem.top",
        flags:"IRWA",
        description:"Top coordinate of this item in pixels.  Applies only when the containing DynamicForm\n sets <code>itemLayout:\"absolute\"</code>.",
        name:"top",
        setter:["setTop"],
        seeAlso:[],
        valueType:"integer",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:FormItem.hint":{
        flags:"IRW",
        type:"attr",
        ref:"attr:FormItem.hint",
        description:"Specifies \"hint\" string to show next to the form item to indicate something to the user.\n This string generally appears to the right of the form item.",
        name:"hint",
        setter:["setHint"],
        valueType:"String",
        seeAlso:[
            {ref:"attr:FormItem.hintStyle", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('formHints')}"],
        defaultValue:"null",
        groups:["appearance"],
        definingClass:"class:FormItem"
    },
    "attr:ListGridField.canExport":{
        ref:"attr:ListGridField.canExport",
        flags:"IR",
        description:"Dictates whether the data in this field be exported.  Explicitly set this\n  to false to prevent exporting.  Has no effect if the underlying \n  ${isc.DocUtils.linkForRef('attr:DataSourceField.canExport','dataSourceField')} is explicitly set to \n  canExport: false.",
        name:"canExport",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:SectionStack.itemEndIndent":{
        ref:"attr:SectionStack.itemEndIndent",
        flags:"IRW",
        description:"Size, in pixels, of indentation of all member items relative to the end of\n the alignment axis. For instance, for left-aligned members, \n itemStartIndent specifies indentation for every item from the right side of the\n section stack.",
        name:"itemEndIndent",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"undefined",
        type:"attr",
        definingClass:"class:SectionStack",
        groups:["layoutMember"]
    },
    "method:List.addList":{
        ref:"method:List.addList",
        flags:"",
        description:" Add a list of items to this array.\n <P>\n Note: you can specify that a subset range be added by passing start and end indices\n\n",
        name:"addList",
        seeAlso:[],
        params:[
            {description:"list of items to add", optional:false, name:"list", defaultValue:null,
             type:"array"},
            {description:"optional start index in list", optional:true, name:"listStartRow",
             defaultValue:null, type:"number"},
            {description:"optional end index in list (non-inclusive)", optional:true,
             name:"listEndRow", defaultValue:null, type:"number"}
        ],
        returns:{description:"list of items that were added", type:"list"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "method:Canvas.showNextTo":{
        ref:"method:Canvas.showNextTo",
        flags:"",
        description:" Show this widget next to another widget.\n",
        name:"showNextTo",
        seeAlso:[],
        params:[
            {description:"Canvas to show next to", optional:false, name:"otherWidget",
             defaultValue:null, type:"Canvas"},
            {description:"which side to show on, defaults to \"right\"", optional:true, name:"side",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "classMethod:Page.isRTL":{
        ref:"classMethod:Page.isRTL",
        flags:"",
        description:"\n Return whether the page text direction is right to left.  If you set \"DIR=RTL\" in the BODY tag of\n the page, then this method will return true.  If you set \"DIR=LTR\" then this method will return\n false.\n    \n",
        name:"isRTL",
        seeAlso:[],
        returns:{description:"true if Page text direction is RTL, false otherwise", type:"boolean"},
        type:"classMethod",
        definingClass:"class:Page"
    },
    "method:TreeGrid.dataArrived":{
        ref:"method:TreeGrid.dataArrived",
        flags:"",
        description:" Notification method fired whenever this TreeGrid recieves new data nodes from the \n dataSource. Only applies to databound TreeGrids where ${isc.DocUtils.linkForRef('attr:TreeGrid.data')} is a \n ${isc.DocUtils.linkForRef('class:ResultTree')} - either explicitly created and applied via ${isc.DocUtils.linkForRef('method:TreeGrid.setData')} or\n automatically generated via a ${isc.DocUtils.linkForRef('method:TreeGrid.fetchData','fetchData()')} call.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"The parentNode for which children were just loaded", optional:false,
             name:"parentNode", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:GridRenderer.getCellCSSText":{
        ref:"method:GridRenderer.getCellCSSText",
        flags:"A",
        description:" Return CSS text for styling this cell, which will be applied in addition to the CSS class\n for the cell, as overrides.\n <p>\n \"CSS text\" means semicolon-separated style settings, suitable for inclusion in a CSS\n stylesheet or in a STYLE attribute of an HTML element.\n\n",
        name:"getCellCSSText",
        seeAlso:[
            {ref:"method:GridRenderer.getCellStyle", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS text for this cell", type:"string"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "method:Scrollbar.setScrollTarget":{
        ref:"method:Scrollbar.setScrollTarget",
        flags:"",
        description:"          Sets or clears the scrollbar's scrollTarget. If no argument is provided, then the\n          scrollTarget will be set to the scrollbar itself.\n\n",
        name:"setScrollTarget",
        seeAlso:[],
        params:[
            {description:"target canvas to be scrolled", optional:true, name:"newTarget",
             defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:Scrollbar",
        groups:["scroll"]
    },
    "attr:ServerObject.dropExtraFields":{
        ref:"attr:ServerObject.dropExtraFields",
        flags:"IR",
        description:"By default, for DMI DSResponses, DSResponse.data is filtered on the server to just the set\n of fields defined on the DataSource.  This behavior can be overridden in several ways - see\n the overview in ${isc.DocUtils.linkForRef('class:DMI')} for details.  The value of this attribute overrides\n ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')}.",
        name:"dropExtraFields",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "method:Canvas.getFullDataPath":{
        ref:"method:Canvas.getFullDataPath",
        flags:"",
        description:" Returns a fully qualified ${isc.DocUtils.linkForRef('type:dataPath')} for this canvas. This is calculated by combining\n the canvas' specified ${isc.DocUtils.linkForRef('attr:Canvas.dataPath')} with the <code>dataPath</code> of any parent \n canvases up to whichever canvas has a specified ${isc.DocUtils.linkForRef('attr:Canvas.valuesManager')} specified to actually\n manage values from this component.\n",
        name:"getFullDataPath",
        seeAlso:[],
        returns:{description:"fully qualified dataPath for this component", type:"dataPath"},
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:FormItemIcon.click":{
        ref:"method:FormItemIcon.click",
        flags:"",
        description:"      StringMethod action to fire when this icon is clicked\n      If unset the form item's <code>iconClick</code> method will be fired instead (if\n      specified).\n",
        name:"click",
        seeAlso:[],
        params:[
            {description:"The Dynamic Form to which this icon's item belongs.", optional:false,
             name:"form", defaultValue:null, type:"DynamicForm"},
            {description:"The Form Item containing this icon", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"A pointer to the form item icon clicked", optional:false, name:"icon",
             defaultValue:null, type:"FormItemIcon"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('formIcons')}"],
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "method:DataSource.processResponse":{
        ref:"method:DataSource.processResponse",
        flags:"A",
        description:" Process a dsResponse for a request initiated by a DataSource with\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','dataProtocol:\"clientCustom\"')}.\n <code>requestId</code> parameter should be dsRequest.requestId as found on the dsRequest\n passed to ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}.\n <P>\n You must provide a response for both error and non-error cases.  For an error case, a\n sufficient response is:\n <pre>\n { status : -1 }\n </pre>\n",
        name:"processResponse",
        seeAlso:[],
        params:[
            {description:"requestId attribute from the associated dataSource request object",
             optional:false, name:"requestId", defaultValue:null, type:"string"},
            {description:"Configuration for the dsResponse", optional:false, name:"dsResponse",
             defaultValue:null, type:"DSResponse properties"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:ListGrid.longTextEditorType":{
        ref:"attr:ListGrid.longTextEditorType",
        flags:"IRW",
        description:"When the length of the field specified by ${isc.DocUtils.linkForRef('attr:DataSourceField.length')} exceeds \n <code>this.longTextEditorThreshold</code> show an edit field of this type\n rather than the standard text field when the field enters inline edit mode.",
        name:"longTextEditorType",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"PopUpTextAreaItem\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Canvas.keepInParentRect":{
        ref:"attr:Canvas.keepInParentRect",
        flags:"IRWA",
        description:"Constrains drag-resizing and drag-repositioning of this canvas to either the rect of its\n parent (if set to true) or an arbitrary rect (if set to a [Left,Top,Width,Height] rect\n array).  If this canvas has no parent, constrains dragging to within the browser window.\n <p>\n Affects target and outline dragAppearance, not tracker.\n <p>\n Note: keepInParentRect affects only user drag interactions, not programmatic moves.",
        name:"keepInParentRect",
        seeAlso:[],
        valueType:"boolean or rect",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:SectionStack.headerHeight":{
        ref:"attr:SectionStack.headerHeight",
        flags:"IR",
        description:"Height of headers for sections.",
        name:"headerHeight",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "method:ListGrid.setShowGridSummary":{
        ref:"method:ListGrid.setShowGridSummary",
        flags:"",
        description:" Setter for the ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} attribute\n",
        name:"setShowGridSummary",
        seeAlso:[],
        params:[
            {description:"new value for this.showGridSummary", optional:false, name:"showGridSummary",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "classMethod:isA.RegularExpression":{
        ref:"classMethod:isA.RegularExpression",
        flags:"",
        description:"\n	Is <code>object</code> a Regular Expression (RegExp) object?\n\n",
        name:"RegularExpression",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Boolean", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:MultiSortPanel.getSort":{
        ref:"method:MultiSortPanel.getSort",
        flags:"",
        description:" Returns all configured sorting levels, as an array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n\n",
        name:"getSort",
        seeAlso:[],
        returns:{description:"the SortSpecifiers for all configured sorting levels",
         type:"Array of SortSpecifier"},
        type:"method",
        definingClass:"class:MultiSortPanel"
    },
    "attr:ActiveXControl.codeBase":{
        ref:"attr:ActiveXControl.codeBase",
        flags:"IR",
        description:"Specifies the URL from which to load the ActiveX control.",
        name:"codeBase",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ActiveXControl"
    },
    "attr:DataBoundComponent.dataPageSize":{
        ref:"attr:DataBoundComponent.dataPageSize",
        flags:"IRW",
        description:"When using data paging, how many records to fetch at a time.  The value of this\n attribute is passed on to the auto-constructed ${isc.DocUtils.linkForRef('class:ResultSet')} object for this\n component.  In effect, this gives you control over the ${isc.DocUtils.linkForRef('attr:ResultSet.resultSize')}\n attribute for this component.\n <P>\n <b>Note</b> that regardless of the <code>dataPageSize</code> setting, a component will always fetch\n all of data that it needs to draw.  Settings such as\n ${isc.DocUtils.linkForRef('attr:ListGrid.showAllRecords','showAllRecords:true')},\n ${isc.DocUtils.linkForRef('attr:ListGrid.drawAllMaxCells','drawAllMaxCells')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio','drawAheadRatio')} can cause more rows than the configured\n <code>dataPageSize</code> to be fetched.",
        name:"dataPageSize",
        seeAlso:[
            {ref:"attr:ResultSet.resultSize", description:null}
        ],
        valueType:"number",
        defaultValue:"75",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "method:Array.addAt":{
        ref:"method:Array.addAt",
        flags:"",
        description:" Add a single item to this array at a specific position in the list, sliding other items over\n to fit.\n\n",
        name:"addAt",
        seeAlso:[],
        params:[
            {description:"object to add", optional:false, name:"obj", defaultValue:null, type:"object"},
            {description:"position in the list to add at", optional:false, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"object that was added", type:"object"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "attr:SummaryBuilder.formulaField":{
        ref:"attr:SummaryBuilder.formulaField",
        flags:"IR",
        description:"TextItem that users type into when entering a formula.",
        name:"formulaField",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:ListGrid.skinImgDir":{
        ref:"attr:ListGrid.skinImgDir",
        flags:"IRWA",
        description:"Where do 'skin' images (those provided with the class) live?",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/ListGrid/\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance", "images"]
    },
    "method:DataBoundComponent.disableHiliting":{
        ref:"method:DataBoundComponent.disableHiliting",
        flags:"",
        description:" Disable all hilites.\n\n",
        name:"disableHiliting",
        seeAlso:[],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["hiliting"]
    },
    "attr:SummaryBuilder.showAutoHideCheckBox":{
        ref:"attr:SummaryBuilder.showAutoHideCheckBox",
        flags:"IR",
        description:"Whether to show a checkbox offering the user the ability to automatically\n hide any fields involved in the formula.",
        name:"showAutoHideCheckBox",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:FacetValue.align":{
        ref:"attr:FacetValue.align",
        flags:"IR",
        description:"Default alignment for facet label title, and cells for this facetValue.\n Can be overridden by setting titleAlign or cellAlign on the facetValue.",
        name:"align",
        seeAlso:[],
        valueType:"alignEnum",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:RestDataSource.addDataURL":{
        ref:"attr:RestDataSource.addDataURL",
        flags:"IR",
        description:"Custom dataURL for add type operations",
        name:"addDataURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:TreeGrid.nodeIcon":{
        flags:"IRW",
        type:"attr",
        ref:"attr:TreeGrid.nodeIcon",
        description:"The filename of the default icon for all leaf nodes in this grid. To specify a \n custom image for an individual node, set the ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconProperty')} directly on\n the node.",
        name:"nodeIcon",
        setter:["setNodeIcon"],
        valueType:"SCImgURL",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        defaultValue:"\"[SKIN]file.gif\"",
        groups:["treeIcons"],
        definingClass:"class:TreeGrid"
    },
    "classMethod:Timer.setTimeout":{
        ref:"classMethod:Timer.setTimeout",
        flags:"",
        description:"    \n Execute an action in a given amount of time.  This method wraps the native setTimeout() method,\n correcting for browser-specific memory leaks.\n\n",
        name:"setTimeout",
        seeAlso:[
            {ref:"classMethod:Timer.clear", description:null}
        ],
        params:[
            {description:"Function to be called when delay has elapsed. \n				     Can also be a string representation of an expression.\n	    			 Passing a string is preferred.",
             optional:false, name:"action", defaultValue:null, type:"string expression or function"},
            {description:"Time until action is executed (in milliseconds). If not specified, the                       default is 100 milliseconds.",
             optional:false, name:"delay", defaultValue:null, type:"number"}
        ],
        returns:{description:"Reference to the timerEvent created. Note that this reference is provided 							 only so that it can be used as an argument for Timer.clear().",
         type:"timerEvent"},
        type:"classMethod",
        definingClass:"class:Timer"
    },
    "attr:Facet.titleAlign":{
        ref:"attr:Facet.titleAlign",
        flags:"IR",
        description:"Alignment of facet label title",
        name:"titleAlign",
        seeAlso:[
            {ref:"attr:CubeGrid.facetTitleAlign", description:null}
        ],
        valueType:"alignEnum",
        defaultValue:"cubeGrid.facetTitleAlign",
        type:"attr",
        definingClass:"class:Facet"
    },
    "method:Class.echo":{
        ref:"method:Class.echo",
        flags:"",
        description:" Return a short string representation of any object, suitable for viewing by a developer for\n debugging purposes.\n <P>\n If passed an object containing other objects, echo will not recurse into subobjects,\n summarizing them instead via echoLeaf().\n <P>\n NOTE: echo() is used to generate the output shown in the Log window when evaluating an\n expression.\n <P>\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n \n",
        name:"echo",
        seeAlso:[
            {ref:"classMethod:Log.echoAll", description:null},
            {ref:"classMethod:Log.echoLeaf", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"method",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:ListGrid.getCriteria":{
        ref:"method:ListGrid.getCriteria",
        flags:"",
        description:" Retrieves the current criteria for this component (may be null)\n",
        name:"getCriteria",
        seeAlso:[],
        returns:{description:"current filter criteria", type:"Criteria"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:Layout.removeMember":{
        ref:"method:Layout.removeMember",
        flags:"",
        description:"\n Removes the specified member from the layout. If it has a resize bar, the bar will be\n destroyed.\n\n",
        name:"removeMember",
        seeAlso:[],
        params:[
            {description:"the canvas to be removed from the layout", optional:false, name:"member",
             defaultValue:null, type:"canvas"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "class:MultiFilePicker":{
        methods:[],
        ref:"class:MultiFilePicker",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items/MultiFileItem"],
        attrs:[],
        description:"The MultiFilePicker is a pop-up picker used by the ${isc.DocUtils.linkForRef('class:MultiFileItem')} to allow the user to\n enter several files for upload.",
        name:"MultiFilePicker",
        seeAlso:[],
        type:"class",
        groups:["upload"]
    },
    "method:StretchImgButton.getTitle":{
        ref:"method:StretchImgButton.getTitle",
        flags:"A",
        description:" Return the title - text/HTML drawn inside the component.\n <p>\n Default is to simply return this.title.\n",
        name:"getTitle",
        seeAlso:[],
        returns:{description:"HTML for the title", type:"string"},
        type:"method",
        definingClass:"class:StretchImgButton"
    },
    "classMethod:MultiSortDialog.askForSort":{
        ref:"classMethod:MultiSortDialog.askForSort",
        flags:"",
        description:" Launches a MultiSortDialog and obtains a sort-definition from the user.\n\n",
        name:"askForSort",
        seeAlso:[],
        params:[
            {description:"A source for Fields which the user can choose to sort by", optional:false,
             name:"fieldSource", defaultValue:null,
             type:"Array of Field or DataSource or DataBoundComponent"},
            {description:"The initial sort definition.", optional:false, name:"initialSort",
             defaultValue:null, type:"Array of SortSpecifier"},
            {description:"Called when the user defines and accepts one or more  ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.  Single parameter <code>sortLevels</code> is an Array of \n SortSpecifier or null if the user cancelled the dialog.",
             optional:false, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"classMethod",
        definingClass:"class:MultiSortDialog"
    },
    "attr:StretchImgButton.showRollOverIcon":{
        ref:"attr:StretchImgButton.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:ToolStripSeparator.hSrc":{
        ref:"attr:ToolStripSeparator.hSrc",
        flags:"IRW",
        description:"Image for horizontally oriented separator (for vertical toolstrips).",
        name:"hSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]hseparator.png\"",
        type:"attr",
        definingClass:"class:ToolStripSeparator"
    },
    "attr:Layout.overflow":{
        ref:"attr:Layout.overflow",
        flags:"IR",
        description:"Normal ${isc.DocUtils.linkForRef('type:Overflow')} settings can be used on layouts, for example, an\n overflow:auto Layout will scroll if members exceed its specified size, whereas an\n overflow:visible Layout will grow to accomodate members.",
        name:"overflow",
        seeAlso:[],
        valueType:"Overflow",
        defaultValue:"\"visible\"",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "method:DynamicForm.getErrors":{
        ref:"method:DynamicForm.getErrors",
        flags:"",
        description:" Returns the current set of validation errors for this form.\n",
        name:"getErrors",
        seeAlso:[],
        returns:{description:"Errors are returned as an object of the format<br> <code>{fieldName:errors, fieldName:errors}</code><br>\n where each <code>errors</code> object will be either an error message string or an array\n of error message strings.",
         type:"object"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "method:DynamicForm.getSaveOperationType":{
        ref:"method:DynamicForm.getSaveOperationType",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('type:DSOperationType')} to be performed when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is\n called. Valid options are <code>\"add\"</code> or <code>\"update\"</code>.\n <P>\n If a ${isc.DocUtils.linkForRef('class:DSRequest')} configuration object is passed in containing an explicit operationType\n this will be returned. Otherwise ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')} will be returned.\n\n",
        name:"getSaveOperationType",
        seeAlso:[],
        params:[
            {description:"Optional DSRequest config block for the  save operation", optional:true,
             name:"requestProperties", defaultValue:null, type:"DSRequest properties"}
        ],
        returns:{description:"Operation type for the save request.", type:"DSOperationType"},
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "method:Canvas.animateResize":{
        ref:"method:Canvas.animateResize",
        flags:"",
        description:" Animate a resize of this canvas from its current size to the specified size\n",
        name:"animateResize",
        seeAlso:[],
        params:[
            {description:"new width (or null for unchanged)", optional:false, name:"width",
             defaultValue:null, type:"number"},
            {description:"new height (or null for unchanged)", optional:false, name:"height",
             defaultValue:null, type:"number"},
            {description:"When the resize completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated resize", optional:true, name:"duration",
             defaultValue:null, type:"number"},
            {description:"Optional acceleration effect to apply to the resize", optional:true,
             name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateResize')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:Calendar.eventClick":{
        ref:"method:Calendar.eventClick",
        flags:"",
        description:" Called whenever an event is clicked on in the day, week or month views.\n <P>\n By default a dialog appears showing details for the event, and offering the ability to\n edit events which are editable.  Return false to cancel the default action. This is a good\n place to, for example, show a completely customized event dialog instead of the default one.\n\n",
        name:"eventClick",
        seeAlso:[],
        params:[
            {description:"event that was clicked on", optional:false, name:"event", defaultValue:null,
             type:"CalendarEvent"},
            {description:"view where the event was clicked on: \"day\", \"week\" or \"month\"",
             optional:false, name:"viewName", defaultValue:null, type:"String"}
        ],
        returns:{description:"false to cancel the default action", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "attr:StretchImgButton.capSize":{
        ref:"attr:StretchImgButton.capSize",
        flags:"IRW",
        description:"How big are the end pieces by default",
        name:"capSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"12",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["appearance"]
    },
    "attr:ListGrid.initialCriteria":{
        ref:"attr:ListGrid.initialCriteria",
        flags:"IR",
        description:"Criteria to be used when ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchData')} is set.",
        name:"initialCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ListGridRecord.enabled":{
        ref:"attr:ListGridRecord.enabled",
        flags:"IR",
        description:"Default property name denoting whether this record is enabled. Property name may be modified\n for some grid via ${isc.DocUtils.linkForRef('attr:ListGrid.recordEnabledProperty')}.",
        name:"enabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disabledRows')}"],
        definingClass:"object:ListGridRecord"
    },
    "attr:GridRenderer.scrollRedrawDelay":{
        ref:"attr:GridRenderer.scrollRedrawDelay",
        flags:"IRW",
        description:"While drag scrolling in an incrementally rendered grid, time in milliseconds to wait\n before redrawing, after the last mouse movement by the user.",
        name:"scrollRedrawDelay",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"75",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "attr:Label.showSelectedIcon":{
        ref:"attr:Label.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "attr:DSRequest.resultSet":{
        ref:"attr:DSRequest.resultSet",
        flags:"R",
        description:"For advanced use in integrating dataset paging with web services,\n the ResultSet that issued this \"fetch\" DSRequest is automatically made available as the\n <code>resultSet</code> property.\n <P>\n This property can only be read.  There is no meaning to setting this property\n yourself.",
        name:"resultSet",
        seeAlso:[],
        valueType:"ResultSet",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "classAttr:Date.shortDayNames":{
        ref:"classAttr:Date.shortDayNames",
        flags:"IRWA",
        description:"This property may be set to an array of names of days of the week. <br>\n For example:\n <pre>\n [\"Sun\", \"Mon\", \"Tue\", \"Wed\", \"Thu\", \"Fri\", \"Sat\"]\n </pre>\n The appropriate day name will then be returned from ${isc.DocUtils.linkForRef('method:Date.getShortDayName')}, and may \n be used whenever SmartClient components display day-names (for example in the \n ${isc.DocUtils.linkForRef('class:DateItem','DateItem class')}).<br>\n Note: For US based applications the first item in the array should be the name for Sunday, \n then Monday, Tuesday, etc. For browsers with different locales this may vary. \n To determine the first day for some locale, you can run the following code:\n <pre>\n    alert(new Date(2000, 0, 2).getDay());\n </pre>\n You should see an alert with a number between zero and 6. This represents the numerical\n 'day' value for Sunday for your browser's locale, since Jan 2nd 2000 was a Sunday. \n Therefore if this code alerted the number 6, Sunday should appear last in your list \n of day-names, and Monday first.",
        name:"shortDayNames",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"classAttr",
        definingClass:"class:Date",
        groups:["i18nMessages"]
    },
    "attr:MultiSortPanel.invalidListPrompt":{
        ref:"attr:MultiSortPanel.invalidListPrompt",
        flags:"IR",
        description:"This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed.\n <P>\n Default value returns <P>\n <code>\n <i>Columns may only be used once: <code>[some field's title]</code> is used multiple times</i>",
        name:"invalidListPrompt",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Columns may only be used once: '\\${title}' is used multiple times.\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "method:DynamicForm.fetchData":{
        ref:"method:DynamicForm.fetchData",
        flags:"",
        description:" Retrieve data that matches the provided criteria, and edit the first record returned\n \n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Window.headerStyle":{
        ref:"attr:Window.headerStyle",
        flags:"IRWA",
        description:"Style for the Window header.",
        name:"headerStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"WindowHeader\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:Calendar.saveButtonTitle":{
        ref:"attr:Calendar.saveButtonTitle",
        flags:"IR",
        description:"The title for the save button in the quick event dialog and the event editor",
        name:"saveButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Save Event\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.headerButtonConstructor":{
        ref:"attr:ListGrid.headerButtonConstructor",
        flags:"IR",
        description:"Widget class for this ListGrid's header buttons. If unset constructor will be \n picked up directly standard ${isc.DocUtils.linkForRef('class:Toolbar')} button constructor.",
        name:"headerButtonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:DateChooser.nextYearIconHeight":{
        ref:"attr:DateChooser.nextYearIconHeight",
        flags:"IRW",
        description:"Height of the icon for the next year button",
        name:"nextYearIconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:SectionStack.animateSections":{
        ref:"attr:SectionStack.animateSections",
        flags:"IRW",
        description:"If true, sections are animated during expand/collapse and addition/removal of\n SectionItems is likewise animated.",
        name:"animateSections",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('animateSections')}"],
        definingClass:"class:SectionStack",
        groups:["animation"]
    },
    "method:SimpleTile.getRecord":{
        ref:"method:SimpleTile.getRecord",
        flags:"",
        description:" Return the record that this tile should render.\n <P>\n NOTE: a TileGrid that is doing data paging may reuse tiles with different records, so a\n subclass of SimpleTile should not cache the record returned by getRecord().\n <P>\n",
        name:"getRecord",
        seeAlso:[],
        returns:{description:"the TileRecord associated with this tile", type:"TileRecord"},
        type:"method",
        definingClass:"class:SimpleTile"
    },
    "method:List.containsAll":{
        ref:"method:List.containsAll",
        flags:"",
        description:" Return whether this list contains all the item in the specified list.\n\n",
        name:"containsAll",
        seeAlso:[],
        params:[
            {description:"items to look for", optional:false, name:"list", defaultValue:null,
             type:"List"}
        ],
        returns:{description:"whether all items were found", type:"boolean"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:ListGridField.editorValueIconHeight":{
        ref:"attr:ListGridField.editorValueIconHeight",
        flags:"IRW",
        description:"When some cell in this field is being edited, setting this property will specify the\n height for value icons in the cell's editor. If unset, the editor's valueIcon width and\n height will be determined in the same way as it would be for a static cell.",
        name:"editorValueIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:Operator.titleProperty":{
        ref:"attr:Operator.titleProperty",
        flags:"IR",
        description:"Name of a property on the ${isc.DocUtils.linkForRef('class:Operators')} class that provides the title for this operator.",
        name:"titleProperty",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "attr:CubeGrid.cellAlign":{
        ref:"attr:CubeGrid.cellAlign",
        flags:"IRW",
        description:"default align for cell values (in body)",
        name:"cellAlign",
        seeAlso:[],
        valueType:"alignEnum",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:DataSource.requiresRole":{
        ref:"attr:DataSource.requiresRole",
        flags:"IR",
        description:"Similar to ${isc.DocUtils.linkForRef('attr:OperationBinding.requiresRole')}, but controls access to the DataSource as a\n whole.",
        name:"requiresRole",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSource",
        groups:["auth"],
        requiresModules:["SCServer"]
    },
    "method:Tree.getFolders":{
        ref:"method:Tree.getFolders",
        flags:"",
        description:"\n Returns all the first-level folders of a node.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')},\n so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} interface\n instead of as a native Javascript Array.\n\n",
        name:"getFolders",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"List of immediate children that are folders", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:Canvas.getScrollWidth":{
        ref:"method:Canvas.getScrollWidth",
        flags:"A",
        description:" Returns the scrollable width of the widget's contents, including children, ignoring\n clipping.\n",
        name:"getScrollWidth",
        seeAlso:[],
        returns:{description:"the scrollable width of the widget's contents", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:TextItem.characterCasing":{
        ref:"attr:TextItem.characterCasing",
        flags:"IRWA",
        description:"Should entered characters be converted to upper or lowercase?\n Also applies to values applied with ${isc.DocUtils.linkForRef('method:FormItem.setValue')}.\n <P>\n Note: character casing cannot be used at the same time as a ${isc.DocUtils.linkForRef('attr:TextItem.mask')}.",
        name:"characterCasing",
        seeAlso:[],
        valueType:"CharacterCasing",
        defaultValue:"TextItem.DEFAULT",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formFilters')}"],
        definingClass:"class:TextItem"
    },
    "classMethod:isc.showLoginDialog":{
        ref:"classMethod:isc.showLoginDialog",
        flags:"",
        description:" Handle a complete login interaction with a typical login dialog asking for username and\n password credentials using the ${isc.DocUtils.linkForRef('class:LoginDialog')} class.\n <P>\n As with other convenience methods that show Dialogs, such as ${isc.DocUtils.linkForRef('classMethod:isc.warn')},\n the dialog is shown and the function immediately returns.  When the user responds, the\n provided callback function is called.\n <P>\n If the user clicks the \"Log in\" button, the credentials entered by the user are passed to\n the provided \"loginFunc\" as an Object with properties \"username\" and \"password\" (NOTE: both\n property names are all lowercase), as the variable \"credentials\".  For example:\n <pre>{ username: \"barney\", password: \"rUbbL3\" }</pre>\n <P>\n The \"loginFunc\" should then attempt to log in by whatever means is necessary.  The second\n parameter to the loginFunc, \"dialogCallback\", is a function, which must be called <i>whether\n login succeeds or fails</i> with a true/false value indicating whether login succeeded.\n <P>\n If the login dialog is dismissable (settable as properties.dismissable, default false) and\n the user dismisses it, the loginFunc will be fired with null for the credentials.\n <P>\n The following code shows typical usage.  This code assumes you have created a global\n function sendCredentials() that send credentials to some authentication system and fires a\n callback function with the result:\n <pre>\n isc.showLoginDialog(function (credentials, dialogCallback) {\n     if (credentials == null) return; // dismissed\n\n     // send credentials    \n     sendCredentials(credentials, function (loginSucceeded) {\n         // report success or failure\n         dialogCallback(loginSucceeded);\n     })\n })\n </pre>\n The login dialog has several built-in behaviors:\n <ul>\n <li> keyboard focus is automatically placed in the username field\n <li> hitting enter in the username field proceeds to the password field\n <li> hitting enter in the password field submits (fires the provided callback)\n </ul>\n In addition to normal properties supported by Dialog/Window, the following special\n properties can be passed:\n <ul>\n <li><code>username</code>: initial value for the username field\n <li><code>password</code>: initial value for the password field\n <li><code>usernameTitle</code>: title for the username field\n <li><code>passwordTitle</code>: title for the password field\n <li><code>errorMessage</code>: default error message on login failure\n <li><code>loginButtonTitle</code>: title for the login button\n <li><code>dismissable</code>: whether the dialog can be dismissed, default false\n <li><code>errorStyle</code>: CSS style for the error message, if shown\n </ul>\n See below for links to the default values for these properties.\n\n",
        name:"showLoginDialog",
        seeAlso:[
            {ref:"classAttr:LoginDialog.LOGIN_TITLE", description:null},
            {ref:"classAttr:LoginDialog.USERNAME_TITLE", description:null},
            {ref:"classAttr:LoginDialog.PASSWORD_TITLE", description:null},
            {ref:"classAttr:LoginDialog.LOGIN_BUTTON_TITLE", description:null},
            {ref:"classAttr:LoginDialog.LOGIN_ERROR_MESSAGE", description:null}
        ],
        params:[
            {description:"Function to call to attempt login.  Recieves parameters                                      \"credentials\" and \"dialogCallback\", described above",
             optional:false, name:"loginFunc", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog", optional:true, name:"properties",
             defaultValue:null, type:"LoginDialog Properties"}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "method:Array.min":{
        ref:"method:Array.min",
        flags:"",
        description:"          \n 			Returns the smallest number in the array, skipping non-numeric values.\n\n			If the start and/or end are given, searches the specified subset of the list.\n\n",
        name:"min",
        seeAlso:[],
        params:[
            {description:"optional start index (default is 0)", optional:true, name:"start",
             defaultValue:null, type:"number"},
            {description:"optional end index (default is list.length)", optional:true, name:"end",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"minimum of all items in the list, or null if all values are non-numeric",
         type:"number"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "method:ListGrid.getCellAlign":{
        ref:"method:ListGrid.getCellAlign",
        flags:"",
        description:" Return the horizontal alignment for cell contents. Default implementation returns \n ${isc.DocUtils.linkForRef('attr:ListGridField.cellAlign')} if specified, otherwise ${isc.DocUtils.linkForRef('attr:ListGridField.align')}.\n\n",
        name:"getCellAlign",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null}
        ],
        params:[
            {description:"this cell's record", optional:false, name:"record", defaultValue:null,
             type:"listGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"Horizontal alignment of cell contents: 'right', 'center', or 'left'",
         type:"Alignment"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ButtonItem.autoFit":{
        ref:"attr:ButtonItem.autoFit",
        flags:"IR",
        description:"Should the button auto fit to its title. Maps to ${isc.DocUtils.linkForRef('attr:Button.autoFit')} attribute.\n Note that if an explicit width or height is specified for this item, it will be respected,\n disabling autoFit behavior",
        name:"autoFit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ButtonItem"
    },
    "attr:ListGrid.expansionFieldImageHeight":{
        ref:"attr:ListGrid.expansionFieldImageHeight",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is set to <code>true</code>, this property\n may be set to govern the height of the expansion image displayed to indicate whether a row \n is expanded. If unset, the expansionField image will be sized to match the\n ${isc.DocUtils.linkForRef('attr:ListGrid.booleanImageHeight')} for this grid.",
        name:"expansionFieldImageHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:DataSourceField.canFilter":{
        ref:"attr:DataSourceField.canFilter",
        flags:"IR",
        description:"Should the user be able to filter data by this field.\n Effects whether this field will show up in dataBoundComponents with UI for filtering data.",
        name:"canFilter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "method:Canvas.parentResized":{
        ref:"method:Canvas.parentResized",
        flags:"",
        description:" Fires when the interior size of the parent changes, including parent resize and\n scrollbar introduction or removal.\n <p>\n This method allows a child to implement a layout policy that can be used within any \n parent, such as a Resizer component that always snaps to the parent's\n bottom-right corner.  The default implementation of this method applies a child's\n percent sizes, if any, or implements layout based on the ${isc.DocUtils.linkForRef('attr:Canvas.snapTo')} property\n",
        name:"parentResized",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:FormItem.endRow":{
        ref:"attr:FormItem.endRow",
        flags:"IRW",
        description:"Whether this item should end the row it's in in the form layout",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "attr:TreeGrid.customIconOpenProperty":{
        ref:"attr:TreeGrid.customIconOpenProperty",
        flags:"IRWA",
        description:"This property allows the developer to rename the \n ${isc.DocUtils.linkForRef('attr:TreeNode.showOpenIcon','default node.showOpenIcon')} property.",
        name:"customIconOpenProperty",
        seeAlso:[
            {ref:"attr:TreeGrid.customIconProperty", description:null},
            {ref:"attr:TreeGrid.showCustomIconOpen", description:null}
        ],
        valueType:"string",
        defaultValue:"\"showOpenIcon\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "group:serverDataIntegration":{
        ref:"group:serverDataIntegration",
        title:"Server DataSource Integration",
        treeLocation:["Concepts/Client-Server Integration", "Java Server Reference"],
        refs:["attr:DataSource.dataFormat", "attr:DataSource.dataProtocol", 
         "attr:DataSource.requestProperties","attr:DataSource.serverType", "attr:DataSource.tableName", "attr:DataSource.dbName", 
         "attr:DataSource.serverObject","attr:OperationBinding.requestProperties", "type:DSDataFormat", "type:DSServerType"],
        description:"Server Data Integration means:\n <ul>\n <li> You ${isc.DocUtils.linkForRef('group:iscInstall','install')} the \n      ${isc.DocUtils.linkForRef('group:iscServer','SmartClient Java Server Framework')} into any J2SE/J2EE\n      environment, including any existing web application\n <li> You ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','create DataSources')} via an XML declaration,\n possibly on-the-fly from ${isc.DocUtils.linkForRef('group:metadataImport','existing metadata')}.  \n <li> Server communication for components bound to these DataSources is handled\n automatically with a highly efficient, compressed protocol.  You work with clean Java APIs\n instead of dealing with the details of XML or JSON over HTTP.\n <li> You can use built-in connectors for SQL, Hibernate and other common data providers\n without writing any code, or you can easily build your own connectors in Java.\n <li> Whether using the built-in connectors or custom connectors, declarations in your\n DataSource control a large set of server features that can make common types of business\n logic entirely declarative\n </ul>\n This approach is in contrast to \n ${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-side Data Integration')} in which client-side\n DataSources are configured to send and receive HTTP messages containing XML, JSON\n or other content.\n <P>\n <B>Server-side Request Processing</B>\n <P>\n Client-side ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')} will send\n ${isc.DocUtils.linkForRef('class:DSRequest','DSRequests')} to the SmartClient Server as background communications transparent\n to the user.  Integrating SmartClient's DataSource layer with your data model is a matter of\n handling these DSRequests and sending back DSResponses, in order to fulfill the 4 basic\n operations of the ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource Protocol')}.\n <P>\n Out of the box, SmartClient is set up to route all DSRequests through a special servlet\n called <code>IDACall</code>.  Requests that go through <code>IDACall</code> have the \n following lifecycle:\n <ul>\n <li>The overall HTTP request is received by the IDACall servlet.  SmartClient supports \n queuing of transactions, so each HTTP request might contain multiple DSRequests.</li>\n <li>IDACall sets up an instance of <code>RPCManager</code> to manage the processing of\n the entire queue of transactions.  For every DSRequest in the queue, this RPCManager:</li>\n   <ul>\n   <li>Validates the DSRequest</li>\n   <li>Sends the DSRequest through ${isc.DocUtils.linkForRef('class:DMI')} - in other words, your code - if this is \n       configured in the DataSource.  As described later in this section, your code can\n       perform some custom logic here: either completely fulfilling the request, or\n       alternatively modifying the request and causing the default\n       processing of the request to continue</li>\n   <li>Calls the DataSource's <code>execute</code> method to obtain a DSResponse.</li>\n   </ul>\n <li>Having processed all requests, the RPCManager now serializes all the DSResponses \n and sends them back to the browser as a single HTTP response</li>\n </ul>\n <p>\n This basic request handling flow can be customized at a number of points:\n <ul>\n <li>If you need an overarching authentication service, this is best implemented using \n <a href=http://java.sun.com/products/servlet/Filters.html>servlet Filters</a> to intercept\n unauthenticated requests before they reach the <code>IDACall</code> servlet</li>\n <li>If you are not using one of the built-in persistence mechanisms (SQL and Hibernate), \n hook into the <code>IDACall</code> flow by \n ${isc.DocUtils.linkForRef('group:writeCustomDataSource','writing a custom DataSource')}.  This approach lets you write and \n maintain the minimal amount of custom code, while taking full advantage of\n DataSource-agnostic features of the SmartClient Server, like validation, queuing,\n transaction chaining, support for Velocity templating, and so on.</li>\n <li>Custom validation can be added by writing a custom DataSource (extending SQLDataSource\n or HibernateDataSource if appropriate) and overriding its <code>validate()</code> method, \n as described ${isc.DocUtils.linkForRef('attr:DataSource.serverConstructor','here')}.</li>\n <li>General custom business logic can be added in a number of ways, both declaratively and\n programmatically:</li>\n <ul>\n   <li>The &lt;criteria&gt; and &lt;values&gt; properties of an ${isc.DocUtils.linkForRef('class:OperationBinding')} \n       allow you to dynamically set data values at transaction-processing time, using \n       built-in ${isc.DocUtils.linkForRef('group:velocitySupport','Velocity support')}</li>\n   <li>Override the <code>execute()</code> method of the DataSource to add extra processing\n       before and/or after the SmartClient processing</li>\n   <li>Use ${isc.DocUtils.linkForRef('attr:DataSource.value','Transaction Chaining')} to dynamically set data values \n       according to the results of earlier transactions\n   <li>For SQL DataSources, use ${isc.DocUtils.linkForRef('group:customQuerying','SQL Templating')} to change, add \n       to or even completely replace the SQL sent to the database, including calling\n       stored procedures</li>\n   <li>Use ${isc.DocUtils.linkForRef('class:DMI','Direct Method Invocation')} to call directly into your own Java \n       classes.  As descibed in the DMI discussion linked to above, DMI calls can be used \n       in conjunction with normal <code>DSRequest</code> process flow, thus enabling you\n       to add custom logic to built-in DataSources without having to create your own \n       overridden versions of them</li>\n </ul><br>\n <li>If you need to use a Front Controller servlet for some other reason than authentication -\n for example, you are using Spring, Struts, or some other similar system which requires that \n all requests go through some particular servlet - just call \n <code>RPCManager.processRequest()</code> within your Spring Controller, Struts Action, or \n whatever the equivalent is in the framework in use.\n <p>\n However, note carefully that taking this approach is often a sign that the SmartClient \n architecture has not been correctly understood.  SmartClient is architected for \n <em>client-server</em> data communication, as opposed to early web MVC frameworks which \n do everything on the server.  In particular, it is absolutely incorrect to represent every \n individual DataSource operation - or even every DataSource - as a separate Struts Action \n or Spring Controller, because this implies different URLs for different operations.  All \n DataSource operations should go through a single URL in order to allow \n ${isc.DocUtils.linkForRef('class:RPCManager','transaction queuing')} - see these \n ${isc.DocUtils.linkForExampleId('transactionsFolder', 'Queuing examples')}.</li>\n </ul>\n <P>\n For more information on the DMI subsystem, see the ${isc.DocUtils.linkForRef('class:DMI')} class and the \n ${isc.DocUtils.linkForExampleId('DMI', 'DMI example')} in the Feature Explorer.\n <P>\n Note that, as you continue to integrate your prototype with your backend, you can use a\n mixture of DataSources that have been fully integrated with your backend and DataSources\n that are running in \"client-only\" mode (see ${isc.DocUtils.linkForRef('group:clientOnlyDataSources')}).\n <P>\n <b>Important methods for handling DataSource requests</b>\n <P>\n The basic flow of logic for handling DataSource requests is:\n <P>\n <table class=\"normal\" border=1 width=\"700\">\n <tr>\n <td>1. Determine operation type (Fetch, Add, Update, Remove) for a single request.  Not \n necessary if you follow the recommendations for \n ${isc.DocUtils.linkForRef('group:writeCustomDataSource','writing a custom DataSource')} and provide your \n implementation via <code>executeFetch(), executeAdd()</code>, et al.</td>\n <td>dsRequest.getOperationType()</td>\n </tr>\n\n <tr>\n <td>2. Get inbound values (Add, Update) and/or criteria (Fetch, Update, Remove) for this\n request.</td>\n <td>dsRequest.getFieldValue()<br>\n dsRequest.getValues()<br> \n dsRequest.getCriteria()</td>\n </tr>\n\n <tr>\n <td>3. Business logic, validation, calls to data and service tiers... anything you can code.\n </td>\n <td><b>execute custom logic</b></td>\n </tr>\n\n <tr>\n <td>4. Set status and data for the response.</td>\n <td>dsResponse.setStatus()<br>\n dsResponse.setData()</td>\n </tr>\n </table>\n <P>\n For more information, see the ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager documentation')}, and the \n ${isc.DocUtils.linkForExampleId('ormDataSource', 'Custom ORM DataSource example')}.",
        name:"serverDataIntegration",
        seeAlso:[],
        type:"group"
    },
    "method:Button.action":{
        ref:"method:Button.action",
        flags:"",
        description:" This property contains the default 'action' for the Button to fire when activated.\n",
        name:"action",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button"
    },
    "attr:Window.resizer":{
        ref:"attr:Window.resizer",
        flags:"R",
        description:"ImgButton-based resizer, shown in the footer.",
        name:"resizer",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:DataBoundComponent.useFlatFields":{
        ref:"attr:DataBoundComponent.useFlatFields",
        flags:"IR",
        description:"The <code>useFlatFields</code> flag causes all simple type fields anywhere in a nested\n set of DataSources to be exposed as a flat list for form binding.  \n <P>\n <code>useFlatFields</code> is typically used with imported metadata, such as \n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema','XML Schema')} from a \n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL','WSDL-described web servce')}, as a means of eliminating levels of XML\n nesting that aren't meaningful in a user interface, without the cumbersome and fragile\n process of mapping form fields to XML structures.\n <P>\n For example, having called ${isc.DocUtils.linkForRef('method:WebService.getInputDS')} to retrieve the input message\n schema for a web service operation whose input message looks like this:\n <pre>\n &lt;FindServices&gt;\n     &lt;searchFor&gt;search text&lt;/searchFor&gt;\n     &lt;Options&gt;\n         &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;\n     &lt;/Options&gt;\n     &lt;IncludeInSearch&gt;\n         &lt;serviceName&gt;true&lt;/serviceName&gt;\n         &lt;documentation&gt;true&lt;/documentation&gt;\n         &lt;keywords&gt;true&lt;/keywords&gt;\n     &lt;/IncludeInSearch&gt;\n &lt;/FindServices&gt;\n </pre>\n Setting <code>useFlatFields</code> on a ${isc.DocUtils.linkForRef('class:DynamicForm')} that is bound to this input\n message schema would result in 5 ${isc.DocUtils.linkForRef('class:FormItem','FormItems')} reflecting the 5 simple type\n fields in the message.\n <P>\n For this form, the result of ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form.getValues()')} might look\n like:\n <P>\n <pre>{\n    searchFor: \"search text\",\n    caseSensitive: false,\n    serviceName: true,\n    documentation : true,\n    keywords : true\n }</pre>\n When contacting a ${isc.DocUtils.linkForRef('class:WebService','WSDL web service')}, these values can be automatically\n mapped to the structure of the input message for a web service operation by setting\n ${isc.DocUtils.linkForRef('attr:WSRequest.useFlatFields')} (for use with ${isc.DocUtils.linkForRef('method:WebService.callOperation')}) or by setting\n ${isc.DocUtils.linkForRef('attr:DSRequest.useFlatFields')} (for use with a ${isc.DocUtils.linkForRef('class:DataSource')} that is\n ${isc.DocUtils.linkForRef('group:wsdlBinding','bound to a WSDL web service')} via\n ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation')}).  \n <P>\n Using these two facilities in conjunction (component.useFlatFields and\n request.useFlatFields) allows gratuitous nesting to be consistently bypassed in both the user\n presentation and when providing the data for XML messages.\n <P>\n You can also set ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields')} to automatically enable \n \"flattened\" XML serialization (request.useFlatFields) for all DataSource requests of a\n particular operationType.\n <P>\n Note that <code>useFlatFields</code> is not generally recommended for use with structures\n where multiple simple type fields exist with the same name, however if used with such a\n structure, the first field to use a given name wins.  \"first\" means the first field\n encountered in a depth first search.  \"wins\" means only the first field will be present as a\n field when data binding.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:Img.setImageType":{
        ref:"method:Img.setImageType",
        flags:"",
        description:" Change the style of image rendering.\n\n",
        name:"setImageType",
        seeAlso:[],
        params:[
            {description:"new style of image rendering", optional:false, name:"imageType",
             defaultValue:null, type:"ImageStyle"}
        ],
        type:"method",
        definingClass:"class:Img"
    },
    "classAttr:Page.defaultUnsupportedBrowserURL":{
        ref:"classAttr:Page.defaultUnsupportedBrowserURL",
        flags:"IRWA",
        description:"On a call to <code>Page.checkBrowserAndRedirect()</code>, if no explicit URL\n      is passed in, and the browser is not supported by ISC, redirect to this URL.",
        name:"defaultUnsupportedBrowserURL",
        seeAlso:[
            {ref:"classMethod:Page.checkBrowserAndRedirect", description:null}
        ],
        valueType:"URL",
        defaultValue:"\"[SKIN]/unsupported_browser.html\"",
        type:"classAttr",
        definingClass:"class:Page",
        groups:["files"]
    },
    "classMethod:isc.clearPrompt":{
        ref:"classMethod:isc.clearPrompt",
        flags:"",
        description:"\n	Clear the modal prompt being shown to the user.\n\n",
        name:"clearPrompt",
        seeAlso:[
            {ref:"classAttr:Dialog.Prompt", description:null}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "attr:ColumnTree.column":{
        ref:"attr:ColumnTree.column",
        flags:"IR",
        getter:["getColumn"],
        description:"Instance of ListGrid used to display each column of the tree.",
        name:"column",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColumnTree"
    },
    "attr:MultiView.dataSource":{
        ref:"attr:MultiView.dataSource",
        flags:"IRW",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        definingClass:"class:MultiView",
        groups:["databinding"]
    },
    "group:clientData":{
        ref:"group:clientData",
        refs:[],
        name:"clientData",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGridRecord.canDrag":{
        ref:"attr:ListGridRecord.canDrag",
        flags:"IR",
        description:"When set to <code>false</code>, this record cannot be dragged. If canDrag is false for\n any record in the current selection, none of the records will be draggable.",
        name:"canDrag",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "group:compression":{
        ref:"group:compression",
        title:"Compression",
        treeLocation:["Optional Modules/Network Performance"],
        refs:[],
        description:"Compression helps reduce the sizes of various data fetched from the server.  Most modern web\n browsers can handle compressed responses of certain content types. The time it takes to\n decompress these responses on a client system is negligible compared to the time saved\n by reducing the number of bits on the wire, especially for slow connections.\n <p>\n If you're not using the SmartClient Java back-end, there are several compression solutions\n available, depending on your server of choice.  Microsoft's IIS has built-in compression\n capability, please check the reference manual for details.  If you're using Apache, you can\n use <a href='http://sourceforge.net/projects/mod-gzip/' onclick=\"window.open('http://sourceforge.net/projects/mod-gzip/');return false;\">mod_gzip</a> or \n use <a href='http://httpd.apache.org/docs/2.0/mod/mod_deflate.html' onclick=\"window.open('http://httpd.apache.org/docs/2.0/mod/mod_deflate.html');return false;\">mod_deflate</a>.  Some\n servlet containers also natively support dynamic compression.\n <p>\n The SmartClient Java back-end supports two types of response compression:\n <ul>\n <li>Pre-compressed static content served via the ISC FileDownload servlet.\n <li>On-the-fly compression of arbitrary content using the CompressionFilter.\n </ul>\n <u>Serving pre-compressed files</u>\n <p>\n To serve pre-compressed static content via FileDownload, register the FileDownload servlet\n in your web.xml as follows:\n <pre>\n     &lt;servlet&gt;\n       &lt;servlet-name&gt;FileDownload&lt;/servlet-name&gt;\n       &lt;servlet-class&gt;com.isomorphic.servlet.FileDownload&lt;/servlet-class&gt;\n     &lt;/servlet&gt;\n </pre>\n Then map any resource that you want to serve compressed to the FileDownload servlet in your\n web.xml.  Typically, you'll want to serve all SmartClient modules compressed.  You can do so\n by adding the following servlet-mapping directive to your web.xml:\n <pre>\n     &lt;servlet-mapping&gt;\n       &lt;servlet-name&gt;FileDownload&lt;/servlet-name&gt;\n       &lt;url-pattern&gt;/isomorphic/system/modules/*&lt;/url-pattern&gt;\n     &lt;/servlet-mapping&gt;\n </pre>\n Finally, you'll need to create pre-compressed versions of your files alongside the\n uncompressed versions.  If you're using the FileAssembler mechanism, it can create\n pre-compressed files for you automatically.  For all other files, you can use any program\n that uses the gzip encoding.  The compressed file must have exactly the same filename as the\n uncompressed version, with a '.gz' extension.  Note that it's important that both the\n compressed and uncompressed versions be present alongside each other because there are cases\n where serving compressed content is not possible (for example HTTP 1.0 requests) - for those\n situations it's important that the uncompressed files be available to be served to the\n client.  The FileDownload filter automatically detects whether or not compression is\n possible.\n <p>\n <u>Dynamic Compression</u>\n <p>\n Dynamic Compression requires the optional Network Performance module.  To use Dynamic\n Compression, register the CompressionFilter filter in your web.xml as follows:\n <pre>\n     &lt;filter&gt;\n         &lt;filter-name&gt;CompressionFilter&lt;/filter-name&gt;\n         &lt;filter-class&gt;com.isomorphic.servlet.CompressionFilter&lt;/filter-class&gt;\n     &lt;/filter&gt;\n </pre>\n Then map any resource that you want dynamically compressed to this filter.  Note that the\n CompressionFilter knows the mime types that are compressible and will automatically ignore\n any stream that sets a content-encoding header, and it automatically figures out if the\n current request is an include or forward (and doesn't compress in that case), so it's safe\n to simply map it to all resources as follows:\n <pre>\n     &lt;filter-mapping&gt;\n         &lt;filter-name&gt;CompressionFilter&lt;/filter-name&gt;\n         &lt;url-pattern&gt;/*&lt;/url-pattern&gt;\n     &lt;/filter-mapping&gt;\n </pre>\n You can register the CompressionFilter anywhere in your filter chain, but be aware that\n if any filters in front wrap and inspect the HttpServletResponse output stream, they will\n be inspecting the compressed response.  Filters are typically applied in the order in which\n they are listed in web.xml. \n <p>\n <u>Compressable mime types and compatibility</u>\n <p>\n The FileDownload servlet and CompressionFilter filter can serve the following mime-types\n compressed: text/html, text/xml, application/x-javascript, text/javascript, text/ecmascript,\n image/svg+xml.  If your files are not being compressed, make sure your servlet container\n has a mime type mapping that identifies it as one of the above file types.\n <p>\n Compression for the mime types listed above is supported on all browsers supported by\n SmartClient.  There is one exception: compression of javascript files for IE versions older\n than IE6 Service Pack 2 requires that the CompressionFilter be registerd to dynamically\n compress the page that loads these javascript files.",
        name:"compression",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.animateScrollAcceleration":{
        ref:"attr:Canvas.animateScrollAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated scroll.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateScrollAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:HibernateBrowser.includeSubstring":{
        ref:"attr:HibernateBrowser.includeSubstring",
        flags:"IR",
        description:"If set, specifies a substring which must exist in an entity name for it to be included in\n this HibernateBrowser.  If this property is set to a List of strings, entity names are\n included if they match any one of the strings.  The comparison is case-insensitive.\n <p>\n For example, <code>includeSubstring: [\"Order\", \"inv\"]</code> would match all the following \n entity names: \"ORDERS\", \"Inventory\", \"client_invoicing\"",
        name:"includeSubstring",
        seeAlso:[],
        valueType:"String | List of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "attr:HeaderItem.defaultValue":{
        ref:"attr:HeaderItem.defaultValue",
        flags:"IRW",
        description:"Header text",
        name:"defaultValue",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Header\"",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "attr:SectionHeader.baseStyle":{
        ref:"attr:SectionHeader.baseStyle",
        flags:"IRA",
        description:"CSS class for the section header.",
        name:"baseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"sectionHeader\"",
        type:"attr",
        definingClass:"class:SectionHeader"
    },
    "attr:Window.footer":{
        ref:"attr:Window.footer",
        flags:"R",
        description:"Optional footer for the window, providing space for a resizer and status bar.",
        name:"footer",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:ListGridField.valueIcons":{
        ref:"attr:ListGridField.valueIcons",
        flags:"IRW",
        description:"This property is a mapping between data values to ${isc.DocUtils.linkForRef('type:SCImgURL','SCImgURLs')} for icons to display\n for those data values.  For example, given a field named \"status\" with possible values\n \"Normal\", \"Slow\", \"Offline\", the follow definition would show various icons for that\n field:\n <P>\n <pre>\n fields : [\n     { name:\"status\", \n       valueIcons: {\n           Normal : \"greenIcon.png\",\n           Slow : \"yellowIcon.png\",\n           Offline : \"redIcon.png\"\n       }\n     },\n     ... other fields ...\n ]\n </pre>\n <P>\n <code>valueIcons</code> can either be displayed alongside the normal value or can\n replace the normal field value so that only the icon is shown.  See\n ${isc.DocUtils.linkForRef('attr:ListGridField.showValueIconOnly')}.\n <P>\n If inline editing is enabled for this field, editors displayed for this field will also\n show valueIcons.  This may be overridden by explicitly setting\n ${isc.DocUtils.linkForRef('attr:ListGridField.editorValueIcons')}.\n <P>\n Note that the following attributes related to valueIcon styling will also be picked up\n by the editor from the ListGridField object unless explicitly specified via the\n equivalent <code>editor_</code> attributes:<br>\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconWidth')}<br>\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconHeight')}<br>\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconSize')}<br>\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconLeftPadding')}<br>\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconRightPadding')}<br>            \n ${isc.DocUtils.linkForRef('attr:ListGridField.imageURLPrefix')}<br>    \n ${isc.DocUtils.linkForRef('attr:ListGridField.imageURLSuffix')}",
        name:"valueIcons",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:Mail.messageData":{
        ref:"attr:Mail.messageData",
        flags:"IRA",
        description:"Allows you to specify an alternative source of substitution values for the templating \n process (by default, the data record(s) returned by the DataSource operation are used).\n If set, this ${isc.DocUtils.linkForRef('type:VelocityExpression')} should evaluate to a Java <code>Map</code>\n object, or a Java <code>List</code> containing only <code>Map</code>s.\n <p>\n For example, assuming you had suitable context stored in a <code>Map</code> that is\n held in the current session as attribute \"foo\", the following would cause the mail \n templating system to look in there for values when running substitution:\n <pre>\n     messageData=\"$session.foo\"\n </pre>",
        name:"messageData",
        seeAlso:[],
        valueType:"VelocityExpression",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "attr:DataSource.dataTransport":{
        ref:"attr:DataSource.dataTransport",
        flags:"IR",
        description:"Transport to use for all operations on this DataSource. Defaults to\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')}.  This would typically only be set to\n enable \"scriptInclude\" transport for contacting ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','JSON')} web\n services hosted on servers other than the origin server.\n <p>\n When using the \"scriptInclude\" transport, be sure to set ${isc.DocUtils.linkForRef('attr:DataSource.callbackParam')} or\n ${isc.DocUtils.linkForRef('attr:OperationBinding.callbackParam')} to match the name of the query parameter name expected\n by your JSON service provider.",
        name:"dataTransport",
        seeAlso:[
            {ref:"type:RPCTransport", description:null},
            {ref:"attr:DataSource.callbackParam", description:null}
        ],
        valueType:"RPCTransport",
        defaultValue:"RPCManager.defaultTransport",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration"]
    },
    "class:ServerObject":{
        methods:[],
        type:"class",
        requiresModules:["SCServer"],
        ref:"class:ServerObject",
        classAttrs:[],
        treeLocation:["Client Reference/RPC"],
        classMethods:[],
        description:"The ServerObject tells the ISC server how to find or create a server-side object involved in\n ${isc.DocUtils.linkForRef('class:DMI')} (Direct Method Invocation).  \n <p>\n A ServerObject declaration appears in the XML definition of a ${isc.DocUtils.linkForRef('class:DataSource')} (for\n responding to ${isc.DocUtils.linkForRef('class:DSRequest')}s) or in an Application configuration file (.app.xml) for\n responding to ${isc.DocUtils.linkForRef('class:RPCRequest')}s.",
        attrs:["attr:ServerObject.lookupStyle", "attr:ServerObject.className", "attr:ServerObject.ID", 
         "attr:ServerObject.bean","attr:ServerObject.attributeName", "attr:ServerObject.attributeScope", 
         "attr:ServerObject.visibleMethods","attr:ServerObject.dropExtraFields", "attr:ServerObject.targetXPath"],
        name:"ServerObject",
        seeAlso:[
            {ref:"class:DMI", description:null}
        ],
        serverDS:"only"
    },
    "group:jsStringTag":{
        ref:"group:jsStringTag",
        title:"&lt;isomorphic:jsString&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i>JavaScript\n <p>\n This tag takes everything in its body and outputs a correctly-escaped JavaScript string.\n This is useful for capturing HTML for display in a Canvas or Label, for example.\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>var</b><br>\n <i>value format</i>: Any legal JavaScript variable name\n <i>default value</i>: NONE\n <p>\n If specified, the escaped string is assigned to a newly created variable of the specified\n name.  e.g: var foo = \"bar\";\n <p>\n <b>filename</b><br>\n <i>value format</i>: webRoot-relative path to file\n <i>default value</i>: NONE\n <p>\n If specified, the resulting string content is loaded from the specified file instead of\n from the tag body.",
        name:"jsStringTag",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "method:ValuesManager.hasErrors":{
        ref:"method:ValuesManager.hasErrors",
        flags:"",
        description:" Are there any errors associated with any fields in this valuesManager?\n",
        name:"hasErrors",
        seeAlso:[],
        returns:{description:"returns true if there are any oustanding validation errors, false                   otherwise.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "method:Slider.setValue":{
        ref:"method:Slider.setValue",
        flags:"",
        description:" Sets the slider value to newValue and moves the slider thumb to the appropriate position for this\n value. Sends the 'sliderMove' event to the sliderTarget.\n\n",
        name:"setValue",
        seeAlso:[],
        params:[
            {description:"the new value", optional:false, name:"newValue", defaultValue:null,
             type:"float"},
            {description:"do not animate the slider thumb to the new value", optional:false,
             name:"noAnimation", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Slider"
    },
    "method:FacetValue.formatCellValue":{
        ref:"method:FacetValue.formatCellValue",
        flags:"",
        description:" Formatter to apply to values displayed for cells under this facetValue.\n <P>\n Can only be set on the ${isc.DocUtils.linkForRef('attr:CubeGrid.metricFacetId','metric facet')}, or if no metric facet\n is specified, on the innermost column facet.\n <P>\n If a single, grid-wide formatting style is desired implement\n ${isc.DocUtils.linkForRef('method:ListGrid.formatCellValue','grid.formatCellValue()')} instead.\n\n",
        name:"formatCellValue",
        seeAlso:[],
        params:[
            {description:"raw value for the cell being", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"record object for the cell.   Note: If this is a new cell that has not been saved, in an \n   editable grid, it has no associated record object. In this case the edit values will\n   be passed in as this parameter.",
             optional:false, name:"record", defaultValue:null, type:"CellRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"formatted value to display in the cell.", type:"string"},
        type:"method",
        definingClass:"class:FacetValue"
    },
    "method:DateItem.setSelectionRange":{
        ref:"method:DateItem.setSelectionRange",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is true, falls through to standard\n ${isc.DocUtils.linkForRef('method:FormItem.setSelectionRange')} implementation on this items freeform text entry field.\n Otherwise has no effect.\n",
        name:"setSelectionRange",
        seeAlso:[],
        params:[
            {description:"character index for start of new selection", optional:false, name:"start",
             defaultValue:null, type:"integer"},
            {description:"character index for end of new selection", optional:false, name:"end",
             defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:DateItem"
    },
    "class:Label":{
        methods:["method:Label.setIconOrientation", "method:Label.setIcon", "method:Label.setContents"],
        ref:"class:Label",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:Label.contents", "attr:Label.dynamicContents", "attr:Label.align", 
         "attr:Label.valign","attr:Label.wrap", "attr:Label.autoFit", "attr:Label.icon", "attr:Label.iconSize", 
         "attr:Label.iconWidth","attr:Label.iconHeight", "attr:Label.iconOrientation", "attr:Label.iconAlign", 
         "attr:Label.iconSpacing","attr:Label.showDisabledIcon", "attr:Label.showRollOverIcon", "attr:Label.showFocusedIcon", 
         "attr:Label.showDownIcon","attr:Label.showSelectedIcon"],
        description:"Labels display a small amount of ${isc.DocUtils.linkForRef('attr:Label.align','alignable')} ${isc.DocUtils.linkForRef('attr:Label.contents','text')}\n with optional ${isc.DocUtils.linkForRef('attr:Label.icon','icon')} and ${isc.DocUtils.linkForRef('attr:Label.autoFit','autoFit')}.\n <P>\n For a general-purpose container for HTML content, use ${isc.DocUtils.linkForRef('class:HTMLFlow')} or ${isc.DocUtils.linkForRef('class:HTMLPane')}\n instead.",
        name:"Label",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('label')}"]
    },
    "attr:ListGrid.emptyMessageStyle":{
        ref:"attr:ListGrid.emptyMessageStyle",
        flags:"IRW",
        description:"The CSS style name applied to the ${isc.DocUtils.linkForRef('attr:ListGrid.emptyMessage')} if displayed.",
        name:"emptyMessageStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"emptyMessage\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["emptyMessage"]
    },
    "method:Class.logDebug":{
        ref:"method:Class.logDebug",
        flags:"",
        description:" Log a message at \"debug\" priority\n <P>\n A method named log<i>Priority</i> exists for each priority level, on every ISC Class and\n instance of an ISC Class.  Messages logged on a Class or instance have a default\n category of the classname.  Messages logged on an instance will also automatically\n incorporate the instance ID.  General best practice is to call logDebug() et al as\n \"this.logDebug\" whenever \"this\" is an instance, or as \"Log.logDebug\" otherwise.\n\n",
        name:"logDebug",
        seeAlso:[
            {ref:"classMethod:Log.echo", description:"for dumping datastructures to the log"},
            {ref:"classMethod:Log.setPriority",
             description:"for controlling what messages appear in the log"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:TileGrid.animateTileChange":{
        ref:"attr:TileGrid.animateTileChange",
        flags:"IRWA",
        description:"If set, when the dataset changes due to filtering, sorting or other actions, any tiles that\n were showing before and after the change will animate from their old positions to their new\n positions.",
        name:"animateTileChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TileGrid",
        groups:["appearance"]
    },
    "attr:ListGridRecord._baseStyle":{
        ref:"attr:ListGridRecord._baseStyle",
        flags:"IRW",
        description:"Name of a CSS style to use as the ${isc.DocUtils.linkForRef('attr:ListGrid.baseStyle')} for all cells for this\n particular record.  \n <P>\n The styleName specified with have suffixes appended to it as the record changes state\n (\"Over\", \"Selected\" and so forth) as described by ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle')}.  For a\n single, fixed style for a record, use ${isc.DocUtils.linkForRef('attr:ListGridRecord.customStyle')} instead.\n <P>\n See ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle')} for an overview of various ways to customize styling,\n both declarative and programmatic.\n <P>\n If this property is changed after draw(), to refresh the grid call\n ${isc.DocUtils.linkForRef('method:ListGrid.refreshRow')} (or ${isc.DocUtils.linkForRef('method:ListGrid.markForRedraw')} if several rows are\n being refreshed).\n <P>\n If your application's data uses the \"_baseStyle\" attribute for something else, the\n property name can be changed via ${isc.DocUtils.linkForRef('attr:ListGrid.recordBaseStyleProperty')}.",
        name:"_baseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "classMethod:isc.defineClass":{
        ref:"classMethod:isc.defineClass",
        flags:"",
        description:" Shortcut for <code>isc.ClassFactory.defineClass()</code>.\n",
        name:"defineClass",
        seeAlso:[
            {ref:"classMethod:ClassFactory.defineClass", description:null}
        ],
        params:[
            {description:"Name for the new class.", optional:false, name:"className", defaultValue:null,
             type:"string"},
            {description:"Optional SuperClass Class object or name", optional:true, name:"superClass",
             defaultValue:null, type:"Class"}
        ],
        returns:{description:"Returns the new Class object.", type:"Class"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:Calendar.timelineViewTitle":{
        ref:"attr:Calendar.timelineViewTitle",
        flags:"IR",
        description:"The title for the timeline view",
        name:"timelineViewTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Timeline\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "classMethod:Class.logInfo":{
        ref:"classMethod:Class.logInfo",
        flags:"",
        description:" Log a message at \"info\" priority\n \n",
        name:"logInfo",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:ListGrid.saveAllEdits":{
        ref:"method:ListGrid.saveAllEdits",
        flags:"",
        description:" Save a number of outstanding edits for this ListGrid. If no rows are specifed, all \n outstanding edits will be saved\n",
        name:"saveAllEdits",
        seeAlso:[],
        params:[
            {description:"optionally specify which rows to save", optional:true, name:"rows",
             defaultValue:null, type:"array of numbers"},
            {description:"If specified this callback will be fired on a successful save  of the specified rows. Note that if there are no pending edits to be saved this callback will\n  not fire - you can check for this condition using ${isc.DocUtils.linkForRef('method:ListGrid.hasChanges')} or \n  ${isc.DocUtils.linkForRef('method:ListGrid.rowHasChanges')}.",
             optional:true, name:"saveCallback", defaultValue:null, type:"callback"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ListGrid.animateRemoveRecord":{
        ref:"attr:ListGrid.animateRemoveRecord",
        flags:"IRW",
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} is enabled, should records be animated out of view\n when they are removed by the user?",
        name:"animateRemoveRecord",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Canvas.enable":{
        ref:"method:Canvas.enable",
        flags:"",
        description:" Enables this widget and any children / peers of this widget.\n",
        name:"enable",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "class:PickListMenu":{
        methods:[],
        ref:"class:PickListMenu",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:PickListMenu.emptyMessage", "attr:PickListMenu.dataProperties"],
        description:"Subclass of ScrollingMenu used by FormItems which implement ${isc.DocUtils.linkForRef('interface:PickList')} to display\n a list of selectable options.<br>\n Note that PickListMenu ultimately inherits from ${isc.DocUtils.linkForRef('class:ListGrid')}.",
        name:"PickListMenu",
        seeAlso:[],
        type:"class"
    },
    "attr:DynamicForm.unknownErrorMessage":{
        ref:"attr:DynamicForm.unknownErrorMessage",
        flags:"IRW",
        description:"The error message for a failed validator that does not specify its own errorMessage.",
        name:"unknownErrorMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Invalid value\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation", "i18nMessages"]
    },
    "attr:Validator.stopIfFalse":{
        ref:"attr:Validator.stopIfFalse",
        flags:"IR",
        description:"Normally, all validators defined for a field will be run even if one of the validators has\n already failed.  However, if <code>stopIfFalse</code> is set, validation will not proceed\n beyond this validator if the check fails.\n <P>\n This is useful to prevent expensive validators from being run unnecessarily, or to allow\n custom validators that don't need to be robust about handling every conceivable type of\n value.",
        name:"stopIfFalse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Validator"
    },
    "method:DataSource.getDataProtocol":{
        ref:"method:DataSource.getDataProtocol",
        flags:"A",
        description:" Returns the appropriate ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')} for a ${isc.DocUtils.linkForRef('class:DSRequest')}\n",
        name:"getDataProtocol",
        seeAlso:[],
        params:[
            {description:"DataSource Request object", optional:false, name:"dsRequest",
             defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:"DataProtocol to be used for this request operation.", type:"DSProtocol"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:RPCRequest.contentType":{
        ref:"attr:RPCRequest.contentType",
        flags:"IRW",
        description:"Valid with the xmlHttpRequest transport only and only when\n ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')} is set to \"POST\".",
        name:"contentType",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"application/x-www-form-urlencoded\"",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:Facet.isTree":{
        ref:"attr:Facet.isTree",
        flags:"IR",
        description:"Marks this facet as a hierarchical facet.\n <P>\n If set, ${isc.DocUtils.linkForRef('attr:Facet.values','facet.value')} will be linked as for a\n ${isc.DocUtils.linkForRef('attr:Tree.modelType','modelType:\"parent\"')} Tree, using\n ${isc.DocUtils.linkForRef('attr:FacetValue.id','facetValue.id')} and\n ${isc.DocUtils.linkForRef('attr:FacetValue.parentId','facetValue.parentId')}.\n Expand/collapse controls will be shown allowing navigation of the facet's values.\n <P>\n The CubeGrid's ${isc.DocUtils.linkForRef('attr:CubeGrid.dataSource','load on demand')} system automatically avoids\n fetching data for facetValues that are not currently visible due to the expand/collapse\n state of a tree facet.\n <P>\n Initial open/close state can be controlled via\n ${isc.DocUtils.linkForRef('attr:Facet.collapsed','facet.collapsed')} and ${isc.DocUtils.linkForRef('attr:FacetValue.collapsed')}.",
        name:"isTree",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Menu.autoDismiss":{
        ref:"attr:Menu.autoDismiss",
        flags:"IRW",
        description:"When true, when a menu item is chosen (via mouse click or keyboard), the menu is not \n automatically hidden, staying in place for further interactivity",
        name:"autoDismiss",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:ComboBoxItem.fetchData":{
        ref:"method:ComboBoxItem.fetchData",
        flags:"",
        description:" Only applies to databound items (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}).<br>\n Performs a fetch type operation on this item's DataSource to retrieve the set of valid\n options for the item, based on the current ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')}.\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Callback to fire when the fetch completes. Callback will               fire with 4 parameters:<ul>\n  <li><code>item</code> a pointer to the form item\n  <li><code>dsResponse</code> the ${isc.DocUtils.linkForRef('class:DSResponse')} returned by the server\n  <li><code>data</code> the raw data returned by the server\n  <li><code>dsRequest</code> the ${isc.DocUtils.linkForRef('class:DSRequest')} sent to the server\n  </ul>",
             optional:false, name:"callback", defaultValue:null, type:"callback"},
            {description:"properties to apply to the              dsRequest for this fetch.",
             optional:false, name:"requestProperties", defaultValue:null, type:"dsRequest properties"}
        ],
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "attr:Validator.clientOnly":{
        ref:"attr:Validator.clientOnly",
        flags:"IR",
        description:"Indicates this validator runs on the client only.\n <p>\n Normally, if the server is trying to run validators and finds a validator that it can't\n execute, for safety reasons validation is considered to have failed.  Use this flag to\n explicitly mark a validator that only needs to run on the client.",
        name:"clientOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Validator"
    },
    "group:monthViewFormatting":{
        ref:"group:monthViewFormatting",
        refs:["method:Calendar.getDayBodyHTML"],
        name:"monthViewFormatting",
        seeAlso:[],
        type:"group"
    },
    "attr:RowSpacerItem.startRow":{
        ref:"attr:RowSpacerItem.startRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RowSpacerItem",
        groups:["appearance"]
    },
    "attr:GridRenderer.canSelectOnRightMouse":{
        ref:"attr:GridRenderer.canSelectOnRightMouse",
        flags:"RW",
        description:"If true, rightMouseDown events will fire 'selectOnRightMouseDown()' for the appropriate cells.",
        name:"canSelectOnRightMouse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:FormItem.isDisabled":{
        ref:"method:FormItem.isDisabled",
        flags:"A",
        description:"  Is this item disabled?\n",
        name:"isDisabled",
        seeAlso:[
            {ref:"attr:FormItem.disabled", description:null}
        ],
        returns:{description:"disabledtrue if this item is be disabled", type:"boolean"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["enable"]
    },
    "method:DynamicForm.setValues":{
        ref:"method:DynamicForm.setValues",
        flags:"",
        description:" Replaces the current values of the entire form with the values passed in.\n <P>\n Note: when working with a form that is saving to a DataSource, you would typically call\n either ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} for an existing record, or ${isc.DocUtils.linkForRef('method:DynamicForm.editNewRecord')} for a new\n record.  In addition to setting the current values of the form, these APIs establish the\n ${isc.DocUtils.linkForRef('attr:DSRequest.operationType')} used to save (\"update\" vs \"add\").\n <P>\n Values should be provided as an Object containing the new values as properties, where each\n propertyName is the name of a ${isc.DocUtils.linkForRef('group:Items','form item')} in the form, and each property value is\n the value to apply to that form item via ${isc.DocUtils.linkForRef('method:FormItem.setValue')}. \n <P>\n Values with no corresponding form item may also be passed, will be tracked by the form\n and returned by subsequent calls to ${isc.DocUtils.linkForRef('method:DynamicForm.getValues')}.\n <P>\n Any ${isc.DocUtils.linkForRef('class:FormItem')} for which a value is not provided will revert to its\n ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue','defaultValue')}.  To cause all FormItems to revert to default\n values, pass null.\n <P>\n This method also calls ${isc.DocUtils.linkForRef('method:DynamicForm.rememberValues')} so that a subsequent later call to\n ${isc.DocUtils.linkForRef('method:DynamicForm.resetValues')} will revert to the passed values.\n\n",
        name:"setValues",
        seeAlso:[],
        params:[
            {description:"values for the form, or null to reset all items to default values",
             optional:true, name:"newData", defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:DataSourceField.validators":{
        ref:"attr:DataSourceField.validators",
        flags:"IR",
        description:"Validators to be applied to this field.\n <p>\n Validators are applied whenever there is an attempt to save changes to a field.\n <p>\n For the available set of built-in validators, and how to define a custom validator, see the\n ${isc.DocUtils.linkForRef('class:Validator')} class.",
        name:"validators",
        seeAlso:[
            {ref:"class:Validator", description:null}
        ],
        valueType:"Array of Validator",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["dataType", "validation"]
    },
    "method:Class.ignore":{
        ref:"method:Class.ignore",
        flags:"A",
        description:"		Stop observing a method on some other object.\n\n",
        name:"ignore",
        seeAlso:[
            {ref:"method:Class.observe", description:null}
        ],
        params:[
            {description:"object to observe", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"name of the method to observe", optional:false, name:"methodName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true == observation stopped, false == no change made", type:"boolean"},
        type:"method",
        definingClass:"class:Class",
        groups:["observation"]
    },
    "attr:CubeGrid.canResizeColumns":{
        ref:"attr:CubeGrid.canResizeColumns",
        flags:"IRW",
        description:"If true, body columns can be resized via the innermost column headers.",
        name:"canResizeColumns",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:GridRenderer.alternateRowFrequency":{
        ref:"attr:GridRenderer.alternateRowFrequency",
        flags:"IRW",
        description:"The number of consecutive rows to draw in the same style before alternating, when\n alternateRowStyles is true.",
        name:"alternateRowFrequency",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:FilterBuilder.removeButtonPrompt":{
        ref:"attr:FilterBuilder.removeButtonPrompt",
        flags:"IR",
        description:"The hover prompt text for the remove button.",
        name:"removeButtonPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Remove\"",
        type:"attr",
        definingClass:"class:FilterBuilder",
        groups:["i18nMessages"]
    },
    "attr:TextItem.showHintInField":{
        ref:"attr:TextItem.showHintInField",
        flags:"IRWA",
        description:"If showing hint for this form item, should it be shown within the field?\n <P>CSS style for the hint is ${isc.DocUtils.linkForRef('attr:TextItem.textBoxStyle')} with the suffix\n \"Hint\" appended to it.",
        name:"showHintInField",
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["appearance"]
    },
    "attr:OperationBinding.callbackParam":{
        ref:"attr:OperationBinding.callbackParam",
        flags:"IR",
        description:"Applies only to dataFormat: \"json\".  Specifies the name of the query parameter that\n tells your JSON service what function to call as part of the response for this operation.\n <P>\n Typically set once for the DataSource as a whole via ${isc.DocUtils.linkForRef('attr:DataSource.callbackParam')}.",
        name:"callbackParam",
        seeAlso:[
            {ref:"attr:DataSource.callbackParam", description:null}
        ],
        valueType:"String",
        defaultValue:"\"callback\"",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "method:String.contains":{
        ref:"method:String.contains",
        flags:"",
        description:"			Returns true if this string contains the specified substring.\n",
        name:"contains",
        seeAlso:[],
        params:[
            {description:"string to look for", optional:false, name:"substring", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"true == this string contains the substring", type:"boolean"},
        type:"method",
        definingClass:"class:String",
        groups:["stringProcessing"]
    },
    "attr:CubeGrid.canMinimizeColumns":{
        ref:"attr:CubeGrid.canMinimizeColumns",
        flags:"IRW",
        description:"If true, allow columns in the grid body to be minimized (reduced to the width of the\n minimize control) by clicking on a minimize control in the innermost column headers.",
        name:"canMinimizeColumns",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:StretchImgButton.showFocusedIcon":{
        ref:"attr:StretchImgButton.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "class:HTMLFlow":{
        methods:["method:HTMLFlow.setContentsURL", "method:HTMLFlow.loadingContent", 
         "method:HTMLFlow.handleError","method:HTMLFlow.transformHTML", "method:HTMLFlow.contentLoaded"],
        type:"class",
        ref:"class:HTMLFlow",
        classAttrs:[],
        treeLocation:["Client Reference/Foundation"],
        classMethods:[],
        description:"Use the HTMLFlow component to display HTML content that should expand to its natural size\n without scrolling.  \n <p>\n HTML content can be loaded and reloaded from a URL via the property\n <code>contentsURL</code>.  This method of loading is for simple HTML content\n only; SmartClient components should be loaded via the ${isc.DocUtils.linkForRef('class:ViewLoader')} class.\n <P>\n NOTE: Since the size of an HTMLFlow component is determined by its HTML contents, this\n component will draw at varying sizes if given content of varying size.  When using HTMLFlow\n components within a Layout, consider what will happen if the HTMLFlow renders at various\n sizes.  An HTMLFlow which can expand should be placed in a container where other components\n can render smaller, where the container is allowed to scroll, or where there is padding to\n expand into.",
        attrs:["attr:HTMLFlow.contents", "attr:HTMLFlow.dynamicContents", "attr:HTMLFlow.contentsURL", 
         "attr:HTMLFlow.loadingMessage","attr:HTMLFlow.contentsURLParams", "attr:HTMLFlow.httpMethod", 
         "attr:HTMLFlow.contentsType","attr:HTMLFlow.allowCaching", "attr:HTMLFlow.evalScriptBlocks", 
         "attr:HTMLFlow.captureSCComponents"],
        name:"HTMLFlow",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('htmlFlow')}"],
        groups:["contentLoading"]
    },
    "attr:DynamicForm.valuesManager":{
        ref:"attr:DynamicForm.valuesManager",
        flags:"IA",
        description:"If set at init time, this dynamicForm will be created as a member form for the\n specified valuesManager.  To update the valuesManager to which a form belongs after init\n use <code>valuesManager.addMember(form)</code> and \n <code>valuesManager.removeMember(form)</code>",
        name:"valuesManager",
        seeAlso:[
            {ref:"class:ValuesManager", description:null}
        ],
        valueType:"ValuesManager instance or global ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formValuesManager"]
    },
    "type:RPCTransport":{
        ref:"type:RPCTransport",
        values:[
            {description:"Uses the XMLHttpRequest object to make the request to the server. Note that in some browsers with certain configurations, this transport may not be\n available.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.  This transport is\n not useful with file uploads.  Cannot be used to target cross-domain URLs directly.",
             value:"\"xmlHttpRequest\""},
            {description:"Write a SCRIPT tag into the DOM with a SRC attribute that targets an arbitrary URL.  This transport is the only one that allows direct cross-domain URL\n access.  \n <P>\n For ${isc.DocUtils.linkForRef('attr:RPCRequest.callback')} to work, the server being contacted must support the ability\n to generate JavaScript code in the response that will call a JavaScript function generated\n by SmartClient.  SmartClient passes the name of the function to call via a URL parameter,\n which can be controlled with ${isc.DocUtils.linkForRef('attr:RPCRequest.callbackParam')}.",
             value:"\"scriptInclude\""},
            {description:"Available with SmartClient Server only.  An HTML form is dynamically assembled that targets a hidden IFRAME.  This mechanism is supported on all\n browsers and cannot be disabled by end users.  \n <P>\n If using the SmartClient Server and using \n ${isc.DocUtils.linkForRef('group:serverDataIntegration','Server-side data integration')}, the \"hiddenFrame\" transport\n is automatically used for all RPCManager and DataSource requests if the \"xmlHttpRequest\"\n transport is not available.\n <P>\n Cannot be used to target cross-domain URLs directly.",
             value:"\"hiddenFrame\""}
        ],
        description:"SmartClient supports multiple RPC transports for maximum compatibility and feature richness.\n All of transports use HTTP as the underlying protocol, but use different mechanisms for\n sending the HTTP request and processing the response.  The transport is typically\n auto-selected for by based on the feature being used and the current browser settings.  For\n advanced use cases, ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')} and ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')} are\n exposed as override points.\n <p>",
        name:"RPCTransport",
        seeAlso:[],
        type:"type"
    },
    "attr:Tree.defaultNodeTitle":{
        ref:"attr:Tree.defaultNodeTitle",
        flags:"IRW",
        description:"Title assigned to nodes without a ${isc.DocUtils.linkForRef('attr:Tree.titleProperty')} value or a\n ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')} value.",
        name:"defaultNodeTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Untitled\"",
        type:"attr",
        definingClass:"class:Tree"
    },
    "class:HStack":{
        methods:[],
        exampleConfig:"HStack.xml",
        ref:"class:HStack",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:[],
        description:"A subclass of Layout that simply stacks members on the horizontal axis without trying to\n manage their width.  On the vertical axis, any members that do not have explict heights will\n be sized to match the height of the stack.",
        name:"HStack",
        seeAlso:[
            {ref:"attr:Layout.hPolicy", description:null}
        ],
        type:"class"
    },
    "method:ListGrid.getEditCol":{
        ref:"method:ListGrid.getEditCol",
        flags:"",
        description:"\n  Returns the index of the column being edited or null if there is no edit col.\n\n",
        name:"getEditCol",
        seeAlso:[],
        returns:{description:"Index of the current edit column", type:"number"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:autoChildren":{
        ref:"group:autoChildren",
        refs:["method:Class.addAutoChild", "method:Class.createAutoChild", "type:AutoChild"],
        description:"An autoChild is an automatically generated subcomponent that a component creates to\n handle part of its presentation or functionality.\n <P>\n An example is the Window component and its subcomponent the \"header\".\n <P>\n AutoChildren support a standard set of properties that can be used to customize or skin\n them.\n <P>\n This topic explains how to use the autoChild system when creating custom components in\n order to create maximum flexibility.  To learn how to use the autoChild system with \n pre-existing components, ${isc.DocUtils.linkForRef('type:AutoChild','go here')}.  \n <P>\n Before reading this topic, be sure you have read the ${isc.DocUtils.linkForDocNode('QuickStart Guide')}\n material on creating custom components and have reviewed the provided examples.\n <P>\n <h3>Basics</h3>\n <P>\n The following is an example of creating subcomponents <b>without</b> using the AutoChild\n pattern.  In this case a fictitious \"Portlet\" class is being created, which uses an\n instance of isc.Label to serve as it's header.\n <pre>\n isc.defineClass(\"Portlet\", \"VLayout\").addProperties({\n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         this.headerLabel = isc.Label.create({\n             autoDraw:false,\n             contents: this.title, \n             styleName: this.titleStyleName,\n             portlet:this,\n             click : function () { this.portlet.bringToFront() },\n             wrap:false,  \n             overflow:\"hidden\", \n             width:\"100%\"\n         });\n         this.addMember(this.headerLabel);\n         ...\n </pre>\n While straightforward, this approach provides limited flexibility to someone using the\n \"Portlet\" class.  There is no way to:\n <ol>\n <li> avoid creating the headerLabel, for a \"headerless\" portlet\n <li> use a different, more advanced class as a header (eg, StretchImgButton or a custom \n class)\n <li> skin the headerLabel, other than CSS (rounded corners, animations, etc, wouldn't be\n possible)\n <li> change it's layout behavior (eg enable autoSize)\n <li> add or override event handlers\n </ol>\n Let's imagine we wanted to add some of the above features.  We could change the code\n like so:\n <P>\n <pre>\n isc.defineClass(\"Portlet\", \"VLayout\").addProperties({\n     <b>showHeaderLabel:true,</b>\n     <b>headerLabelConstructor:isc.Label,</b>\n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         <b>if (this.showHeaderLabel) {</b>\n             this.headerLabel = this.headerLabelConstructor.create({\n                 autoDraw:false,\n                 contents: this.title, \n                 styleName: this.titleStyleName,\n                 portlet:this,\n                 click : function () { this.portlet.bringToFront() },\n                 wrap:false,  \n                 overflow:\"hidden\", \n                 width:\"100%\"\n             }<b>, this.headerLabelProperties</b>);\n             this.addMember(this.headerLabel);\n         <b>}</b>\n         ...\n </pre>\n Our additions solve our initial concerns:\n <ul>\n <li> <code>showHeaderLabel:false</code> can be set to suppress the header label\n <li> <code>headerLabelConstructor</code> allows you to switch to a different class\n <li> <code>headerLabelProperties</code> give you a means to add arbitrary properties\n (skinning properties, sizing properties, event handlers, etc)\n </ul>\n However, the code is becoming more verbose and repetitive, and we've created a few\n additional properties that now need documentation and testing.  This extra work is going\n to be multiplied by every subcomponent we create where we want this kind of flexibility.\n <P>\n Enter the AutoChild system: the purpose of the AutoChild system is to define a standard\n pattern for creating subcomponents with maximum flexibility.  This means:\n <ul>\n <li> developers creating custom components write less code, have less to test and less\n to document\n <li> developers can more easily understand each other's code for custom components,\n because it follows a standard pattern\n <li> developers <b>using</b> custom components have a standard pattern for\n customization, instead of learning customization APIs for every component separately\n </ul>\n The code below uses the autoChild system to create the \"headerLabel\" subcomponent.  This\n version of the code would still respect all of the customization properties from earlier\n examples (<code>headerLabelProperties</code> et al) and offers several additional degrees\n of flexibility still to be explained, yet it's significantly shorter.  More importantly,\n this code is less redundant; the \"boilerplate\" code is gone and what's left is just the\n actual settings for the headerLabel subcomponent.\n <pre>\n isc.defineClass(\"Portlet\", \"VLayout\").addProperties({\n     headerLabelDefaults : {\n         _constructor:isc.Label,\n         click : function () { this.creator.bringToFront() },\n         wrap:false,  \n         overflow:\"hidden\", \n         width:\"100%\"\n     },\n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         this.addAutoChild(\"headerLabel\", {\n             contents: this.title, \n             styleName: this.titleStyleName\n         });\n         ...\n </pre>\n <P>\n The documentation for ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')} explains why this code\n will still respect the <code>showHeaderLabel</code> flag and other customization\n properties even though they aren't mentioned specifically.\n <P>\n <h3>Subclassing a component with autoChildren</h3>\n <P>\n If you are subclassing a component that has an autoChild and you want to change\n defaults for that autoChild, the correct way to do so is to use\n ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','changeDefaults()')}:\n <pre>\n isc.defineClass(\"MyWindow\", \"Window\");\n isc.MyWindow.changeDefaults(\"headerDefaults\", { layoutMargin:10 });\n isc.MyWindow.addProperties({ \n    ...\n </pre>\n <P>\n <code>changeDefaults()</code> creates a copy of the superclass defaults and applies your\n changes, which is important because you want to inherit the superclass behavior without\n affecting the superclass, and yet apply overrides.\n <P>\n The following code sample indicates two common \n <span style=\"color:red;font-weight:bold\">incorrect</span> patterns for working with\n defaults, and the consequences of each: \n <pre>\n isc.defineClass(\"MyWindow\", \"Window\").addProperties({\n     // NO.  Superclass behavior / settings for autoChild\n     // won't be inherited.  Use changeDefaults() instead.\n     headerDefaults : { ... },\n \n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         // NO.  \"headerDefaults\" object is shared across the class,\n         // changing it affects all instances created from here on.\n         // Pass dynamic defaults to addAutoChild() instead\n         this.headerDefaults.myProperty = this.newValue;\n         ...\n });\n </pre>\n <b>defaults vs properties</b>\n <P>\n For AutoChildren, defaults and properties both provide similar means of adding\n properties to an AutoChild, and the distinction between them is primarily one of\n convention: a class that uses AutoChildren should never define a default value for\n <i>autoChildName</i>Properties, so that instances can freely specify\n <i>autoChildName</i>Properties without overriding built-in behavior.\n <pre>\n isc.defineClass(\"MyWindow\", \"Window\").addProperties({\n     // NO.  Any further use of \"headerProperties\", in\n     // instances or in subclasses, would wipe out behavior\n     headerProperties : { ... },\n </pre>\n <P>\n By consistently using ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')} whenever you override autoChild\n defaults in a subclass, you ensure that your classes can in turn be subclassed and\n extended uniformly.\n <P>\n <h3>autoParents and creation order</h3>\n <P>\n The AutoChild pattern can create an entire hierarchy of generated subcomponents.  For\n example, the ${isc.DocUtils.linkForRef('class:Window')} class included with SmartClient uses several AutoChildren as\n part of the overall header structure: separate autoChildren for the minimize button,\n close button, and then the header itself, a Layout-derived class that contains all other\n header controls. \n <P>\n To facilitate construction of hierarchies of autoChildren, the special\n <code>autoParent</code> property may appear in either defaults or properties for an\n autoChild, and indicates the name of another autoChild that should used as a parent.\n For example, to create a \"closeButton\" autoChild that will be a member of the \"header\"\n autoChild:\n <P>\n <pre>\n isc.defineClass(\"Portlet\", \"VLayout\").addProperties({\n     headerDefaults : {\n         _constructor:isc.HLayout,\n         ...\n     },\n     closeButtonDefaults : {\n         <b>autoParent:\"header\",</b>\n         _constructor:isc.ImgButton,\n         ...\n     },\n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         this.addAutoChild(\"header\");\n         this.addAutoChild(\"closeButton\");\n         ...\n </pre>\n <P>\n In addition to cutting down on code and making inter-autoChild relationships clearer,\n using <code>autoParent</code> rather than manual calls to addMember() allows a\n subclass of your component to potentially completely rearrange the autoChildren you have\n defined, while retaining their behavior.\n <P>\n When using <code>autoParent</code> to arrange autoChildren, create parents first, then\n children.\n <P>\n <b>Tip:</b> if you want all of the behaviors of\n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')} <i>except</i> automatically adding the\n autoChild to a parent, set <code>autoParent:\"none\"</code>.\n <P>\n <b>special case: TabSets and SectionStacks</b>\n <p>\n An autoChild that appears as a ${isc.DocUtils.linkForRef('attr:Tab.pane')} or\n ${isc.DocUtils.linkForRef('attr:SectionStackSection.items','section item')} does not have a clear way to refer to it's\n tab or section via the <code>autoParent</code> property.  For this special case, the\n TabSet and SectionStack components allow tab.pane / section.items to contain the special\n string \"autoChild:<i>autoChildName</i>\".  This will cause the corresponding autoChild to be\n automatically created when the tab is selected or section expanded.\n <P>\n For example:\n <pre>\n isc.defineClass(\"Portlet\", \"VLayout\").addProperties({\n     ...\n     mainTabsDefaults : {\n         _constructor:isc.TabSet,\n         tabs : [\n             { title:\"First Pane\", pane:\"autoChild:firstPane\" }\n         ]\n     },\n     firstPaneDefaults : {\n         ...\n     },\n     initWidget : function () {\n         this.Super(\"initWidget\", arguments);\n\n         // this automatically creates firstPane as an autoChild\n         this.addAutoChild(\"mainTabs\");\n         ...\n </pre>",
        name:"autoChildren",
        seeAlso:[],
        type:"group"
    },
    "method:Layout.drop":{
        ref:"method:Layout.drop",
        flags:"A",
        description:" Layouts have built-in handling of component drag and drop.  See the discussion in\n ${isc.DocUtils.linkForRef('class:Layout')} on how it works.  If you override this builtin implementation of drop() and\n you're using the built-in dropLine via ${isc.DocUtils.linkForRef('attr:Layout.canDropComponents')}:true, be sure to call\n ${isc.DocUtils.linkForRef('method:Layout.hideDropLine')} to hide the dropLine after doing your custom drop() handling.\n\n",
        name:"drop",
        seeAlso:[],
        returns:{description:"Returning false will cancel the drop entirely", type:"boolean"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:EdgedCanvas.centerBackgroundColor":{
        ref:"attr:EdgedCanvas.centerBackgroundColor",
        flags:"IR",
        description:"Background color for the center section only.  Can be used as a surrogate background\n color for the decorated Canvas, if the Canvas is set to partially overlap the edges and\n hence can't show a background color itself without occluding media.",
        name:"centerBackgroundColor",
        seeAlso:[],
        valueType:"cssColor",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "group:prompt":{
        ref:"group:prompt",
        refs:[],
        name:"prompt",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.clearRowErrors":{
        ref:"method:ListGrid.clearRowErrors",
        flags:"",
        description:"  Clear any stored validation errors for some row\n",
        name:"clearRowErrors",
        seeAlso:[
            {ref:"method:ListGrid.setRowErrors", description:null}
        ],
        params:[
            {description:"index of row to clear validation error for", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["validtion"]
    },
    "method:ListGrid.displayHeaderContextMenu":{
        ref:"method:ListGrid.displayHeaderContextMenu",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu')} is <code>true</code> this method is fired\n when the user right-clicks on the header for this grid.<br>\n Default implementation will display a menu with entries derived from\n ${isc.DocUtils.linkForRef('method:ListGrid.getHeaderContextMenuItems')} for the appropriate column.\n",
        name:"displayHeaderContextMenu",
        seeAlso:[
            {ref:"attr:ListGrid.showHeaderContextMenu", description:null}
        ],
        params:[
            {description:"which button in the header received the right-click event                         (may be the sorter button)",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"Optional 2-element array specifying position at which the menu should                           be shown. If this is not passed in the menu will be shown at the\n                           mouseEvent position (default context menu behavior).",
             optional:true, name:"position", defaultValue:null, type:"Array"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridHeader"]
    },
    "attr:DetailViewer.showEmptyField":{
        ref:"attr:DetailViewer.showEmptyField",
        flags:"IRWA",
        description:"Whether to show the field when the value is null",
        name:"showEmptyField",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:Canvas.scrollToRight":{
        ref:"method:Canvas.scrollToRight",
        flags:"",
        description:" Horizontally scrolls the content of the widget to the end of its content\n\n",
        name:"scrollToRight",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:CubeGrid.deselectAll":{
        ref:"method:CubeGrid.deselectAll",
        flags:"",
        description:"\n  deselect all cells and facetValues\n\n      <br><i>methodType</i> action\n",
        name:"deselectAll",
        seeAlso:[],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "classAttr:Operators.orTitle":{
        ref:"classAttr:Operators.orTitle",
        flags:"IR",
        description:"Title for the \"or\" operator",
        name:"orTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Match Any\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ImgSplitbar.canCollapse":{
        ref:"attr:ImgSplitbar.canCollapse",
        flags:"IRW",
        description:"If this property is true, a click on the Splitbar will collapse its \n ${isc.DocUtils.linkForRef('attr:Splitbar.target','target')}, hiding it and shifting the Splitbar and other members\n of the layout across to fill the newly available space. If the target is already hidden\n a click will expand it again (showing it at it's normal size).",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "attr:ListGridField.defaultGroupingMode":{
        ref:"attr:ListGridField.defaultGroupingMode",
        flags:"IR",
        description:"Default groupingMode used when the user does not specify a mode or grouping is triggered\n programmatically and ${isc.DocUtils.linkForRef('attr:ListGridField.groupingMode','field.groupingMode')} is unset.\n See ${isc.DocUtils.linkForRef('attr:ListGridField.groupingModes','field.groupingModes')}.",
        name:"defaultGroupingMode",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "group:cacheSync":{
        ref:"group:cacheSync",
        refs:["attr:DSResponse.invalidateCache", "attr:ResultSet.initialData", 
         "attr:ResultSet.initialLength","attr:ResultSet.allRows", "attr:ResultSet.updateCacheFromRequest", 
         "attr:ResultSet.dropCacheOnUpdate","attr:ResultSet.disableCacheSync", "attr:ResultSet.neverDropUpdatedRows", 
         "attr:ResultSet.updatePartialCache","attr:ResultTree.updateCacheFromRequest", "attr:ResultTree.disableCacheSync"],
        name:"cacheSync",
        seeAlso:[],
        type:"group"
    },
    "group:submitting":{
        ref:"group:submitting",
        refs:["method:DynamicForm.submitForm", "method:DynamicForm.submitValues", 
         "method:SearchForm.search","method:DynamicForm.cancel", "method:ValuesManager.cancel", "attr:DynamicForm.action", 
         "attr:DynamicForm.target","attr:DynamicForm.method", "attr:DynamicForm.encoding", "attr:DynamicForm.canSubmit", 
         "attr:DynamicForm.saveOnEnter","type:FormMethod", "type:Encoding"],
        description:"Direct submission of forms to a target URL\n <P>\n <b>NOTE:</b> directly submitting forms is only done for specialized purposes, such as\n integration with certain legacy systems.  Normal form usage contacts the server via \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')}, through the RPCManager system.",
        name:"submitting",
        seeAlso:[],
        type:"group"
    },
    "method:Layout.showMember":{
        ref:"method:Layout.showMember",
        flags:"",
        description:" Show the specified member, firing the specified callback when the hide is complete.\n <P>\n Members can always be directly shown via <code>member.show()</code>, but if\n ${isc.DocUtils.linkForRef('attr:Layout.animateMembers','animation')} is enabled, animation will only occur if showMember() is\n called to show the member.\n\n",
        name:"showMember",
        seeAlso:[],
        params:[
            {description:"Member to show", optional:false, name:"member", defaultValue:null,
             type:"Canvas"},
            {description:"action to fire when the member has been shown", optional:false,
             name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:TextItem.maskPromptChar":{
        ref:"attr:TextItem.maskPromptChar",
        flags:"IRWA",
        description:"Character that is used to fill required empty mask positions\n to display text while control has focus.",
        name:"maskPromptChar",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"_\"",
        type:"attr",
        definingClass:"class:TextItem"
    },
    "attr:SelectOtherItem.separatorTitle":{
        ref:"attr:SelectOtherItem.separatorTitle",
        flags:"IRW",
        description:"Title for the separator between normal items and the <code>Other...</code> item in the drop\n down list.  Selecting this item will not change the FormItem's value.",
        name:"separatorTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"--------------------\"",
        type:"attr",
        definingClass:"class:SelectOtherItem",
        groups:["appearance"]
    },
    "attr:FilterBuilder.addButton":{
        ref:"attr:FilterBuilder.addButton",
        flags:"IR",
        description:"An ImgButton that allows new clauses to be added if ${isc.DocUtils.linkForRef('attr:FilterBuilder.showAddButton')}\n is set.",
        name:"addButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:TreeGrid.showCustomIconOpen":{
        ref:"attr:TreeGrid.showCustomIconOpen",
        flags:"IRWA",
        description:"Should folder nodes showing custom icons (set via the ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconProperty')}),\n show open state images when the folder is opened.\n If true, the ${isc.DocUtils.linkForRef('attr:TreeGrid.openIconSuffix')} will be appended to the image URL\n (so <code>\"customFolder.gif\"</code> might be replaced with \n <code>\"customFolder_open.gif\"</code>).<br>\n <b>Note</b> that the ${isc.DocUtils.linkForRef('attr:TreeGrid.closedIconSuffix')} is never appended to custom folder icons.<br>\n Can be overridden at the node level via the default property ${isc.DocUtils.linkForRef('attr:TreeNode.showOpenIcon')}\n and that property can be renamed via ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconOpenProperty')}.",
        name:"showCustomIconOpen",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "method:ListGrid.groupBy":{
        ref:"method:ListGrid.groupBy",
        flags:"",
        description:" Display the current set of records grouped by their values for the given field or\n fields. With no arguments, disables all grouping.\n <P>\n Grouping tranforms the current dataset into a Tree on the fly, then provides a familiar tree\n interface for exploring the grouped data.\n <P>\n Grouping works automatically with any dataset, providing simple default grouping based on\n each field's declared type.  However, you can use the\n ${isc.DocUtils.linkForRef('method:ListGridField.getGroupValue','field.getGroupValue()')} API to control how records are\n grouped, and the ${isc.DocUtils.linkForRef('method:ListGridField.getGroupTitle','field.getGroupTitle()')} API to control\n how groups are titled.\n <P>\n Grouping can be performed programmatically via this API, or you can set\n ${isc.DocUtils.linkForRef('attr:ListGrid.canGroupBy','grid.canGroupBy:true')} to enable menus that allow the user to perform\n grouping. To group a grid automatically, instantiate the grid with a\n  ${isc.DocUtils.linkForRef('attr:ListGrid.groupByField')} setting.\n <P>\n While grouped, the automatically created Tree is available as\n ${isc.DocUtils.linkForRef('attr:ListGrid.groupTree','grid.groupTree')} and the original dataset is availabe as\n ${isc.DocUtils.linkForRef('attr:ListGrid.originalData','grid.originalData')}.\n\n",
        name:"groupBy",
        seeAlso:[],
        params:[
            {description:"name of fields to group by", optional:true, name:"arguments 0-N",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dynamicGrouping')}"],
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:Layout.layoutLeftMargin":{
        ref:"attr:Layout.layoutLeftMargin",
        flags:"IRW",
        description:"Space outside of all members, on the left-hand side.  Defaults to ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')}.\n <P>\n Requires a manual call to <code>setLayoutMargin()</code> if changed on the fly.",
        name:"layoutLeftMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "method:Class.getClass":{
        ref:"method:Class.getClass",
        flags:"",
        description:"	\n	Gets a pointer to the class object for this instance\n\n",
        name:"getClass",
        seeAlso:[],
        returns:{description:"Class object that was used to construct this object", type:"Class"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:Splitbar.skinImgDir":{
        ref:"attr:Splitbar.skinImgDir",
        flags:"IR",
        description:"Default directory for skin images (those defined by the class), relative to the\n Page-wide ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','skinDir')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["images"]
    },
    "attr:Calendar.leadingDateField":{
        ref:"attr:Calendar.leadingDateField",
        flags:"IR",
        description:"The name of the end date field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"leadingDateField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"leadingDate\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "group:filterEditor":{
        ref:"group:filterEditor",
        refs:["method:ListGrid.focusInFilterEditor", "method:ListGrid.filterByEditor", 
         "method:ListGrid.setShowFilterEditor","method:ListGrid.getFilterEditorValueMap", "method:ListGrid.getFilterEditorType", 
         "attr:ListGridField.canFilter","attr:ListGridField.filterEditorValueMap", "attr:ListGridField.filterEditorType", 
         "attr:ListGridField.defaultFilterValue","attr:ListGridField.filterEditorProperties", "attr:ListGridField.filterOnKeypress", 
         "attr:ListGrid.fetchDelay","attr:ListGrid.showFilterEditor", "attr:ListGrid.filterEditorHeight", 
         "attr:ListGrid.filterByCell","attr:ListGrid.filterOnKeypress"],
        name:"filterEditor",
        seeAlso:[],
        type:"group"
    },
    "attr:RPCRequest.promptCursor":{
        ref:"attr:RPCRequest.promptCursor",
        flags:"IRW",
        description:"Controls the cursor shown when ${isc.DocUtils.linkForRef('classAttr:RPCManager.promptStyle')} is set to\n <code>\"cursor\"</code> for this request only. Defaults to ${isc.DocUtils.linkForRef('classAttr:RPCManager.promptCursor')}.\n <p>\n In Safari, IE 5.5 and Firefox 1.0 the default value is \"wait\", on all other platforms it is\n \"progress\".  The reason for this split is that the above-mentioned browsers do not support\n CSS2.1 - which is required for the \"progress\" cursor type.",
        name:"promptCursor",
        seeAlso:[
            {ref:"classAttr:RPCManager.promptCursor", description:null}
        ],
        valueType:"String",
        defaultValue:"browser-dependent",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["rpcPrompt"]
    },
    "attr:ListGrid.bodyBackgroundColor":{
        ref:"attr:ListGrid.bodyBackgroundColor",
        flags:"IRW",
        description:"Background color applied to the ListGrid body (that is, the area of the grid where\n data values are rendered).<br>\n Note that this will typically not be visible to the user unless there are few enough\n rows that there is visible space in the body below the last row. To style data cells,\n override ${isc.DocUtils.linkForRef('attr:ListGrid.baseStyle')} instead.",
        name:"bodyBackgroundColor",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"white\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "method:Canvas.updateHover":{
        ref:"method:Canvas.updateHover",
        flags:"A",
        description:" If this canvas is currently showing a hover (see ${isc.DocUtils.linkForRef('method:Canvas.handleHover')}), this method\n can be called to update the contents of the hover. Has no effect if the hover canvas is not\n showing for this widget.\n",
        name:"updateHover",
        seeAlso:[],
        params:[
            {description:"Option to specify new HTML for the hover. If not passed, the result   of ${isc.DocUtils.linkForRef('method:Canvas.getHoverHTML','this.getHoverHTML()')} will be used instead. Note that if the\n   hover HTML is empty the hover will be hidden.",
             optional:true, name:"hoverHTML", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:Canvas.setLeft":{
        ref:"method:Canvas.setLeft",
        flags:"",
        description:"			Set the left coordinate of this object, relative to its enclosing context, in pixels.\n			NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()\n          instead\n",
        name:"setLeft",
        seeAlso:[],
        params:[
            {description:"new left coordinate", optional:false, name:"left", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:Selection.select":{
        ref:"method:Selection.select",
        flags:"",
        description:"			Select a particular item\n",
        name:"select",
        seeAlso:[],
        params:[
            {description:"object to select", optional:false, name:"item", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:DateItem.maskDateSeparator":{
        ref:"attr:DateItem.maskDateSeparator",
        flags:"IA",
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} and ${isc.DocUtils.linkForRef('attr:DateItem.useMask')} are both <code>true</code>\n this value is the separator between date components.",
        name:"maskDateSeparator",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"/\"",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["basics"]
    },
    "attr:FormItem.editorType":{
        ref:"attr:FormItem.editorType",
        flags:"IR",
        description:"Name of the FormItem to use for editing, eg \"TextItem\" or \"SelectItem\".\n <P>\n The type of FormItem to use for editing is normally derived automatically from\n ${isc.DocUtils.linkForRef('attr:FormItem.type','field.type')}, which is the data type of the field, by the rules\n explained ${isc.DocUtils.linkForRef('type:FormItemType','here')}.",
        name:"editorType",
        seeAlso:[
            {ref:"type:FormItemType", description:null},
            {ref:"type:FieldType", description:null}
        ],
        valueType:"FormItem class",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:ListGrid.headerBarStyle":{
        ref:"attr:ListGrid.headerBarStyle",
        flags:"IR",
        description:"Set the CSS style used for the header as a whole.",
        name:"headerBarStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "classMethod:EventHandler.setDragTracker":{
        ref:"classMethod:EventHandler.setDragTracker",
        flags:"",
        description:" Set the HTML for the drag tracker that follows the mouse during a drag and drop interaction.\n <P>\n Your canvas can use this routine to set the drag tracker to whatever HTML you want like so:\n <pre>\n    dragStart : function () {\n        isc.EventHandler.setDragTracker('Your contents here');\n    }\n </pre>\n\n",
        name:"setDragTracker",
        seeAlso:[],
        params:[
            {description:"HTML for the tracker", optional:false, name:"html", defaultValue:null,
             type:"string"},
            {description:"new width for the tracker", optional:true, name:"newWidth", defaultValue:null,
             type:"number"},
            {description:"new height for the tracker", optional:true, name:"newHeight",
             defaultValue:null, type:"number"},
            {description:"x-offset for the tracker", optional:true, name:"offsetX", defaultValue:null,
             type:"number"},
            {description:"y-offset for the tracker", optional:true, name:"offsetY", defaultValue:null,
             type:"number"},
            {description:"Opportunity to pass in a free-form set of properties                                       for the dragTracker",
             optional:true, name:"properties", defaultValue:null, type:"object"}
        ],
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["dragDrop", "dragTracker"]
    },
    "attr:Canvas.percentBox":{
        ref:"attr:Canvas.percentBox",
        flags:"IRA",
        description:"Governs the model to be used when sizing canvases with percentage width or height, or \n positioning widgets with a specified ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')}.\n <P>\n Only affects widgets with a a specified ${isc.DocUtils.linkForRef('attr:Canvas.percentSource','percentSource')}, or \n widgets that have ${isc.DocUtils.linkForRef('attr:Canvas.snapTo')} set and are peers of some \n ${isc.DocUtils.linkForRef('attr:Canvas.masterElement','other canvas')}.\n <P>\n Determines whether the coordinates used for sizing (for percentage sized widgets) and\n positioning (if <code>snapTo</code> is set) should be relative to the visible size or the\n viewport size of the percentSource or masterElement widget.",
        name:"percentBox",
        seeAlso:[],
        valueType:"PercentBoxModel",
        defaultValue:"\"visible\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:ListGrid.headerSpan":{
        ref:"attr:ListGrid.headerSpan",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpans')} are created via the ${isc.DocUtils.linkForRef('type:AutoChild')} pattern, hence\n <code>headerSpanConstructor</code>, <code>headerSpanDefaults</code> and\n <code>headerSpanProperties</code> are valid.",
        name:"headerSpan",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "attr:DataSourceField.showFileInline":{
        ref:"attr:DataSourceField.showFileInline",
        flags:"IR",
        description:"For a field of type:\"imageFile\", indicates whether to stream the image and display it\n inline or to display the View and Download icons.",
        name:"showFileInline",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "method:Tree.getParents":{
        ref:"method:Tree.getParents",
        flags:"",
        description:"\n Given a node, return an array of the node's parents with the immediate parent first.  The\n node itself is not included in the result.  For example, for the following tree:\n <pre>\n root\n   foo\n     bar\n </pre>\n Calling <code>tree.getParents(bar)</code> would return: <code>[foo, root]</code>.  Note that\n the returned array will contain references to the nodes, not the names.\n\n",
        name:"getParents",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"array of node's parents", type:"Array"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:StatefulCanvas.iconWidth":{
        ref:"attr:StatefulCanvas.iconWidth",
        flags:"IR",
        description:"Width in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:ListGrid.validateByCell":{
        ref:"attr:ListGrid.validateByCell",
        flags:"IRW",
        description:"Whether client-side validation checks should be performed when the user moves between\n cells in the current edit row.  If unset, defaults to ${isc.DocUtils.linkForRef('attr:ListGrid.editByCell')}.\n <P>\n Note that validation always occurs when a row is to be saved, so setting\n ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell')}:true forces validation on cell transitions.  To completely disable\n automatic validation, set ${isc.DocUtils.linkForRef('attr:ListGrid.neverValidate')}:true.",
        name:"validateByCell",
        seeAlso:[
            {ref:"group:editing", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "method:ListGrid.removeSelectedData":{
        ref:"method:ListGrid.removeSelectedData",
        flags:"",
        description:" Remove the currently selected records from this component.\n If this is a databound grid, the records will be removed directly from the DataSource.\n <P>\n If no records are selected, no action is taken. The grid will automatically be\n updated if the record deletion succeeds.\n\n",
        name:"removeSelectedData",
        seeAlso:[],
        params:[
            {description:"callback to fire when the data has been removed", optional:true,
             name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('removeOperation')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Canvas.snapHGap":{
        ref:"attr:Canvas.snapHGap",
        flags:"IRW",
        description:"The horizontal grid size to use, in pixels, when snap-to-grid is enabled.",
        name:"snapHGap",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"attr:Canvas.snapResizeToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapResizeToGrid", description:null}
        ],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:GridRenderer.getEventColumn":{
        ref:"method:GridRenderer.getEventColumn",
        flags:"",
        description:" Returns the column number of the most recent mouse event.\n",
        name:"getEventColumn",
        seeAlso:[],
        params:[
            {description:"optional x-coordinate to obtain column number for, in lieue of the x                        coordinate of the last mouse event",
             optional:true, name:"x", defaultValue:null, type:"number"}
        ],
        returns:{description:"column number, or -2 if beyond last drawn column", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events", "selection"]
    },
    "group:debug":{
        ref:"group:debug",
        refs:["method:Class.getCallTrace", "method:Class.getStackTrace", "method:Class.echoLeaf", 
         "method:Class.echo","method:Class.echoAll", "class:Log", "classMethod:Class.getCallTrace", 
         "classMethod:Log.getCallTrace","classMethod:Class.getStackTrace", "classMethod:Log.getStackTrace", 
         "classMethod:Class.echoLeaf","classMethod:Log.echoLeaf", "classMethod:Class.echo", "classMethod:Log.echo", 
         "classMethod:Class.echoAll","classMethod:Log.echoAll", "classMethod:Log.show", "classMethod:Log.traceMethod", 
         "classMethod:Log.timeMethod"],
        description:"Support for debugging and logging",
        name:"debug",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.snapTo":{
        flags:"IRW",
        getter:["getSnapTo"],
        type:"attr",
        ref:"attr:Canvas.snapTo",
        description:"Position this widget such that it is aligned with (\"snapped to\") an edge of its \n ${isc.DocUtils.linkForRef('attr:Canvas.masterElement','masterElement')} (if specified), or its\n ${isc.DocUtils.linkForRef('attr:Canvas.parentElement','parentElement')}.\n <P>\n Note that this property also impacts the sizing of this widget. If this widgets size\n is specifed as a percent value, and has no explicit\n ${isc.DocUtils.linkForRef('attr:Canvas.percentSource')}, sizing will be calculated based on the size of the \n masterElement when snapTo is set.\n <P>\n Possible values: BR, BL, TR, TL, R, L, B, T, C where B=Bottom, T=Top, L=Left, R=right\n and C=center",
        name:"snapTo",
        setter:["setSnapTo"],
        valueType:"String",
        seeAlso:[
            {ref:"attr:Canvas.snapEdge", description:null},
            {ref:"attr:Canvas.percentBox", description:null}
        ],
        defaultValue:"null",
        groups:["sizing"],
        definingClass:"class:Canvas"
    },
    "class:TileGrid":{
        methods:["method:TileGrid.fetchData", "method:TileGrid.filterData", 
         "method:TileGrid.invalidateCache","method:TileGrid.removeSelectedData", "method:TileGrid.getSelection", 
         "method:TileGrid.getSelectedRecord","method:TileGrid.selectRecord", "method:TileGrid.deselectRecord", 
         "method:TileGrid.selectRecords","method:TileGrid.deselectRecords", "method:TileGrid.selectAllRecords", 
         "method:TileGrid.deselectAllRecords","method:TileGrid.anySelected", "method:TileGrid.getTileHTML", "method:TileGrid.getTile", 
         "method:TileGrid.recordClick","method:TileGrid.recordDoubleClick", "method:TileGrid.selectionChanged", 
         "method:TileGrid.addTile","method:TileGrid.removeTile", "method:TileGrid.addData", "method:TileGrid.removeData", 
         "method:TileGrid.setDragTracker","method:TileGrid.getTitleField", "method:TileGrid.getDragTrackerTitle", 
         "method:TileGrid.transferDragData","method:TileGrid.getDragData", "method:TileGrid.transferSelectedData", 
         "method:TileGrid.setFieldState","method:TileGrid.getFieldState", "method:TileGrid.dataArrived", 
         "method:TileGrid.exportData"],
        ref:"class:TileGrid",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:TileGrid.fields", "attr:TileGrid.showDetailFields", "attr:TileGrid.tileValueStyle", 
         "attr:TileGrid.tileValueAlign","attr:TileGrid.data", "attr:TileGrid.dataSource", "attr:TileGrid.autoFetchData", 
         "attr:TileGrid.autoFetchTextMatchStyle","attr:TileGrid.initialCriteria", "attr:TileGrid.selectionType", "attr:TileGrid.tile", 
         "attr:TileGrid.tileConstructor","attr:TileGrid.showAllRecords", "attr:TileGrid.animateTileChange", 
         "attr:TileGrid.styleName","attr:TileGrid.dragTrackerStyle", "attr:TileGrid.dragDataAction"],
        description:"A TileGrid is a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} that displays a list of objects as a set\n of \"tiles\", where each tile represents one object, and the tiles are laid out in a grid with\n multiple tiles per row.  Each tile displays one or more properties of the object it\n represents.",
        name:"TileGrid",
        implementsInterfaces:["DataBoundComponent"],
        seeAlso:[],
        type:"class"
    },
    "method:MultiSortDialog.getSortLevel":{
        ref:"method:MultiSortDialog.getSortLevel",
        flags:"",
        description:" Return a ${isc.DocUtils.linkForRef('object:SortSpecifier')} object for the requested levelNum.\n\n",
        name:"getSortLevel",
        seeAlso:[],
        params:[
            {description:"The index of the level to return a SortSpecifier for", optional:false,
             name:"levelNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"A SortSpecifier representing the requested levelNum", type:"SortSpecifier"},
        type:"method",
        definingClass:"class:MultiSortDialog"
    },
    "attr:ListGridRecord.includeInSummary":{
        ref:"attr:ListGridRecord.includeInSummary",
        flags:"IRW",
        description:"If specified as false this record should be ignored when calculating summary totals \n to be shown in the ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary','summary row')} for this grid.\n <P>\n Note that <code>includeInTotals</code> is the default property name for this attribute,\n but it may be modified via ${isc.DocUtils.linkForRef('listGrid.includeInTotalsProperty')}.",
        name:"includeInSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "attr:TileLayout.tileHMargin":{
        ref:"attr:TileLayout.tileHMargin",
        flags:"IR",
        description:"Horizontal margin in between tiles.  See ${isc.DocUtils.linkForRef('attr:TileLayout.tileMargin')}.",
        name:"tileHMargin",
        setter:["setTileHMargin"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "attr:DataSource.dataFormat":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.dataFormat",
        description:"Indicates the format to be used for HTTP requests and responses when\n fulfilling DSRequests (eg, when ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} is called).",
        name:"dataFormat",
        valueType:"DSDataFormat",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('jsonDataSource')}", 
         "${isc.DocUtils.linkForExampleId('simpleJSON')}"],
        serverDS:"allowed",
        defaultValue:"\"iscServer\"",
        groups:["clientDataIntegration", "serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "attr:MultiSortPanel.addLevelButton":{
        ref:"attr:MultiSortPanel.addLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for adding new levels\n to the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.addLevelButtonProperties</code> and \n <code>multiSortPanel.addLevelButtonDefaults</code>.",
        name:"addLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "attr:Canvas.edgeCursorMap":{
        ref:"attr:Canvas.edgeCursorMap",
        flags:"IRWA",
        description:"Cursor to use when over each edge of a Canvas that is drag resizable.\n <P>\n To disable drag resize cursors, set the edgeCursorMap property to null.",
        name:"edgeCursorMap",
        seeAlso:[
            {ref:"attr:Canvas.resizeFrom", description:null}
        ],
        valueType:"object",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:Canvas.hoverWidth":{
        ref:"attr:Canvas.hoverWidth",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.showHover','this.showHover')} is true, this property can be used to customize the\n width of the hover canvas shown.",
        name:"hoverWidth",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customHovers')}"],
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:CubeGrid.innerHeaderBaseStyle":{
        ref:"attr:CubeGrid.innerHeaderBaseStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:Button.baseStyle','baseStyle')} for the buttons in the innermost column header for \n this cubeGrid.",
        name:"innerHeaderBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"innerHeader",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "method:Array.lastIndexOf":{
        ref:"method:Array.lastIndexOf",
        flags:"",
        description:" Return the position in the list of the last instance of the specified object.\n <p>\n If pos is specified, starts looking before that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"lastIndexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"earliest index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:FormItem.iconWidth":{
        ref:"attr:FormItem.iconWidth",
        flags:"IRWA",
        getter:["getIconWidth"],
        description:"Default width for form item icons",
        name:"iconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:DSRequest.resultTree":{
        ref:"attr:DSRequest.resultTree",
        flags:"R",
        description:"For advanced use in integrating trees that \n ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand','load data on demand')} with web services,\n the ResultTree that issued this \"fetch\" DSRequest is automatically made available as the\n <code>resultTree</code> property.\n <P>\n This property can only be read.  There is no meaning to setting this property\n yourself.",
        name:"resultTree",
        seeAlso:[],
        valueType:"ResultTree",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:ValuesManager.members":{
        ref:"attr:ValuesManager.members",
        flags:"IRW",
        getter:["getMembers"],
        description:"The set of member forms for this valuesManager.  These can be specified at init time via\n the <code>members</code> property, or updated at runtime via <code>addMember()</code> and\n <code>removeMember()</code>.<br>\n Note: Alternatively a form can be initialized as a member of a valuesManager by setting\n the <code>valuesManager</code> property of the form to a pre-defined valuesManager\n instance.",
        name:"members",
        seeAlso:[
            {ref:"method:ValuesManager.addMember", description:null},
            {ref:"method:ValuesManager.removeMember", description:null}
        ],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ValuesManager"
    },
    "method:DynamicForm.submitValues":{
        ref:"method:DynamicForm.submitValues",
        flags:"",
        description:" Triggered when a SubmitItem is included in the form is submitted and gets pressed.\n \n",
        name:"submitValues",
        seeAlso:[
            {ref:"method:DynamicForm.submit", description:null}
        ],
        params:[
            {description:"the form values", optional:false, name:"values", defaultValue:null,
             type:"object"},
            {description:"the form being submitted", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "attr:ColorItem.pickerIconProperties":{
        ref:"attr:ColorItem.pickerIconProperties",
        flags:"IRWA",
        description:"Properties for the picker icon. Default object specifies a prompt for the picker icon",
        name:"pickerIconProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "method:ResultSet.getSort":{
        ref:"method:ResultSet.getSort",
        flags:"",
        description:" Return the current sort-specification for this ResultSet as an Array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n \n",
        name:"getSort",
        seeAlso:[],
        returns:{description:"the list of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s currently applied to   this ResultSet",
         type:"Array of SortSpecifier"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:TimeItem.maskOverwriteMode":{
        ref:"attr:TimeItem.maskOverwriteMode",
        flags:"IRWA",
        description:"Internal-use only for a TimeItem.",
        name:"maskOverwriteMode",
        seeAlso:[
            {ref:"attr:TimeItem.useMask", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TimeItem"
    },
    "method:StatefulCanvas.select":{
        ref:"method:StatefulCanvas.select",
        flags:"",
        description:" Select this object.\n",
        name:"select",
        seeAlso:[],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "object:CellRecord":{
        methods:[],
        ref:"object:CellRecord",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Analytics/CubeGrid"],
        attrs:["attr:CellRecord.enabled"],
        description:"A CellRecord represents the data for one cell of the body area.\n <P>\n Each CellRecord should be an object that minimally has a property named after each\n visible facetId with the value being a facetValueId from that facet, and also a value\n for ${isc.DocUtils.linkForRef('attr:CubeGrid.valueProperty')}.\n <P>\n Cell records can contain any other properties desired, such as cell ids, or values for\n facets not initially shown.",
        name:"CellRecord",
        seeAlso:[],
        type:"object",
        requiresModules:["Analytics"]
    },
    "attr:DynamicForm.itemLayout":{
        ref:"attr:DynamicForm.itemLayout",
        flags:"IRWA",
        description:"Layout style to use with this form.  \n <P>\n The default of \"table\" uses a tabular layout similar to HTML tables, but with much more\n powerful control over sizing, item visibility and reflow, overflow handling, etc.\n <P>\n <code>itemLayout:\"absolute\"</code> allows absolute positioning of every form item.  This\n provides maximum flexibility in placement, with the following limitations:<ul>\n <li> titles, which normally take up an adjacent cell, are not shown.  Use\n      StaticTextItems to show titles\n <li> no automatic reflow when showing or hiding items.  ${isc.DocUtils.linkForRef('method:FormItem.setLeft')}\n      and ${isc.DocUtils.linkForRef('method:FormItem.setTop')} can be used for manual reflow.\n <li> only pixel and percent sizes are allowed, no \"*\".  Percent widths mean percentage\n      of the overall form size rather than the column size\n <li> with different font styling or internationalized titles, items may overlap that did\n      not overlap in the skin used at design time\n </ul>",
        name:"itemLayout",
        seeAlso:[],
        valueType:"FormLayoutType",
        defaultValue:"\"table\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formLayout"]
    },
    "attr:TileGrid.showAllRecords":{
        ref:"attr:TileGrid.showAllRecords",
        flags:"IR",
        description:"Whether tiles are created and drawn for all records, or only for those currently visible.",
        name:"showAllRecords",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TileGrid",
        groups:["basics"]
    },
    "attr:DateChooser.prevYearIcon":{
        ref:"attr:DateChooser.prevYearIcon",
        flags:"IR",
        description:"Icon for the previous year button",
        name:"prevYearIcon",
        seeAlso:[
            {ref:"attr:DateChooser.showDoubleYearIcon", description:null}
        ],
        valueType:"URL",
        defaultValue:"\"[SKIN]doubleArrow_left.gif\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:TreeGrid.openLeaf":{
        ref:"method:TreeGrid.openLeaf",
        flags:"A",
        description:" Executed when a leaf node receives a 'doubleClick' event. This handler must be\n specified as a function, whose single parameter is a reference to the relevant leaf node in\n the tree's data.<br>\n See the ListGrid Widget Class for inherited recordClick and recordDoubleClick events.\n\n",
        name:"openLeaf",
        seeAlso:[
            {ref:"class:ListGrid", description:null}
        ],
        params:[
            {description:"node to open", optional:false, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.canExpandRecord":{
        ref:"method:ListGrid.canExpandRecord",
        flags:"",
        description:" Indicates whether a given record or rowNum can be expanded.  The default implementation\n checks the value of ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} and \n <code>record[${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecordProperty')}]</code>.\n <P>\n Override this method for more specific control over individual record expansion.\n \n",
        name:"canExpandRecord",
        seeAlso:[],
        params:[
            {description:"record to work with", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"rowNum of the record to work with", optional:false, name:"rowNum",
             defaultValue:null, type:"Number"}
        ],
        returns:{description:"true if the record can be expanded", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "classMethod:XMLTools.selectObjects":{
        ref:"classMethod:XMLTools.selectObjects",
        flags:"A",
        description:" Applies an XPath expression to JavaScript objects, returning matching objects.\n <P>\n Both child and attribute names are interpreted as property names, and array access notation\n can be used to select elements from Arrays.  For example:<pre>\n     var results = {\n        searchResults:[\n            { title:\"Page One\", relevance:6.3 },\n            { title:\"Page Two\", relevance:5.2, \n              summary: \"Summary of Page One\" }\n        ]\n     };\n\n     // returns the \"searchResults\" two-item Array\n     isc.XMLTools.selectObjects(results, \"/searchResults\");\n\n     // returns the first item under \"searchResults\", in an Array (NOTE: in XPath, Array\n     // index starts at 1, not 0)\n     isc.XMLTools.selectObjects(results, \"/searchResults[1]\");\n\n     // returns [\"Page One\"]\n     isc.XMLTools.selectObjects(results, \"/searchResults[1]/title\");\n\n     // also returns [\"Page One\"]\n     isc.XMLTools.selectObjects(results, \"/searchResults[1]@title\");\n </pre>\n A limited form of XPath \"predicates\", that is, expressions with brackets that filter\n returned objects, is allowed.  A predicate can be either:\n <ul>\n <li> a number only, eg [5], for Array access\n <li> the XPath function call \"last()\", eg [last()], to retrieve the last item\n <li> a property name (*without* any leading \"@\"), meaning that the property contains a value\n      that is considered \"true\" in JavaScript.  For example: [summary]\n <li> a property name, comparison operator, and either a number or String literal, for\n      example, [name = \"bob\"].  In this case the property can also be the XPath function\n      position(), for example, [position() > 5]\n </ul>\n Some examples of using simple predicates with the sample data above:\n <pre>\n     // returns an Array with only the first result\n     isc.XMLTools.selectObjects(results, \"/searchResults[relevance > 5.5]\");\n \n     // return an Array with only the second result, since the first has no summary\n     isc.XMLTools.selectObjects(results, \"/searchResults[summary]\");\n </pre>\n Details of the XPath -> Objects mapping:\n <ul> \n <li> JavaScript Object properties are considered element children, and text children do not\n      exist (in the XML model, text children exist *between* element children, but nothing\n      exists between JavaScript properties)\n <li> The contents of Array-valued properties are considered immediate element children (this\n      is consistent with the predicate \"[5]\" acting like Array access)\n <li> \"*\" in XML selects all element children, so \"*\" in Object XPath selects the values of\n      all properties, that is, ${isc.DocUtils.linkForRef('classMethod:isc.getValues','isc.getValues(object)')}, except\n      that Array-valued properties are \"flattened\" into the returned list.\n </ul>\n\n",
        name:"selectObjects",
        seeAlso:[],
        params:[
            {description:"Object to select results from", optional:false, name:"object",
             defaultValue:null, type:"Object"},
            {description:"XPath expression", optional:false, name:"xPath", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"Array of matching objects, or null for no match", type:"Array"},
        type:"classMethod",
        definingClass:"class:XMLTools"
    },
    "method:ListGrid.getSelectedState":{
        ref:"method:ListGrid.getSelectedState",
        flags:"",
        description:" Returns a snapshot of the current selection within this listGrid as \n a ${isc.DocUtils.linkForRef('type:listGridSelectedState')} object.<br>\n This object can be passed to ${isc.DocUtils.linkForRef('method:ListGrid.setSelectedState')} to reset this grid's selection\n the current state (assuming the same data is present in the grid).<br>\n",
        name:"getSelectedState",
        seeAlso:[
            {ref:"method:ListGrid.setSelectedState", description:null}
        ],
        returns:{description:"current state of this grid's selection", type:"listGridSelectedState"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:Tree.rootValue":{
        ref:"attr:Tree.rootValue",
        flags:"IR",
        description:"If you are using the \"parent\" modelType and did not specify a root node via ${isc.DocUtils.linkForRef('attr:Tree.root')}\n with an id (${isc.DocUtils.linkForRef('attr:Tree.idField')}), then you can provide the root node's id via this property.\n See the example in ${isc.DocUtils.linkForRef('attr:Tree.data')} for more info.",
        name:"rootValue",
        seeAlso:[
            {ref:"attr:Tree.data", description:null}
        ],
        valueType:"string|number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "attr:Splitbar.canCollapse":{
        ref:"attr:Splitbar.canCollapse",
        flags:"IRW",
        description:"If this property is true, a click on the Splitbar will collapse its \n ${isc.DocUtils.linkForRef('attr:Splitbar.target','target')}, hiding it and shifting the Splitbar and other members\n of the layout across to fill the newly available space. If the target is already hidden\n a click will expand it again (showing it at it's normal size).",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Splitbar"
    },
    "attr:TabSet.pickerButtonSize":{
        ref:"attr:TabSet.pickerButtonSize",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true, this property governs the size of tab-picker\n button. Applied as the width of buttons if the tabBar is horizontal, or the height\n if tabBar is vertical. Note that the other dimension is determined by\n ${isc.DocUtils.linkForRef('attr:TabSet.tabBarThickness','this.tabBarThickness')}",
        name:"pickerButtonSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "method:DynamicForm.focusInItem":{
        ref:"method:DynamicForm.focusInItem",
        flags:"",
        description:" Move the keyboard focus into a particular item.\n",
        name:"focusInItem",
        seeAlso:[],
        params:[
            {description:"Item (or reference to) item to focus in.", optional:false, name:"itemName",
             defaultValue:null, type:"number|itemName|formItem"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["eventHandling", "focus"]
    },
    "group:event":{
        ref:"group:event",
        refs:[],
        name:"event",
        seeAlso:[],
        type:"group"
    },
    "group:dateFormatting":{
        ref:"group:dateFormatting",
        refs:["method:Date.getShortDayName", "method:Date.getShortMonthName", "method:Date.getShortYear", 
         "method:Date.getWeek","method:Date.toDateStamp", "method:Date.toNormalDate", "method:Date.toShortDate", 
         "method:Date.toShortDateTime","method:Date.setDefaultDateSeparator", "method:Date.getDefaultDateSeparator", 
         "method:Date.toUSShortDate","method:Date.toUSShortDateTime", "method:Date.toEuropeanShortDate", 
         "method:Date.toEuropeanShortDateTime","method:Date.toJapanShortDate", "method:Date.toJapanShortDateTime", 
         "method:Date.toSerializeableDate","method:Date.setFormatter", "method:Date.toPrettyString", "classMethod:Date.parseInput", 
         "classMethod:Date.setNormalDisplayFormat","classMethod:Date.setShortDisplayFormat", "classMethod:Date.setShortDatetimeDisplayFormat"],
        name:"dateFormatting",
        seeAlso:[],
        type:"group"
    },
    "group:grouping":{
        ref:"group:grouping",
        refs:["method:ListGridField.getGroupValue", "method:ListGridField.getGroupTitle", 
         "method:ListGrid.groupBy","object:groupNode", "attr:groupNode.groupMembers", "attr:groupNode.groupTitle", 
         "attr:groupNode.groupValue","attr:ListGrid.originalData", "attr:ListGrid.groupTree", "attr:ListGrid.groupStartOpen", 
         "attr:ListGridField.groupingModes","attr:ListGridField.groupingMode", "attr:ListGridField.defaultGroupingMode", 
         "attr:ListGridField.groupPrecision","attr:ListGridField.groupGranularity", "attr:ListGrid.groupNodeStyle", 
         "attr:ListGrid.groupIcon","attr:ListGrid.groupIconSize", "attr:ListGrid.groupIndentSize", 
         "attr:ListGrid.groupLeadingIndent","attr:ListGrid.canGroupBy", "attr:ListGrid.groupByMaxRecords", "attr:ListGrid.isGrouped", 
         "attr:ListGrid.nullGroupTitle","attr:ListGrid.groupByField"],
        name:"grouping",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Page.getScrollHeight":{
        ref:"classMethod:Page.getScrollHeight",
        flags:"",
        description:"		Get the height of the window contents as they have been drawn.\n		If the page scrolls, this may be larger than the page.getHeight().\n",
        name:"getScrollHeight",
        seeAlso:[],
        returns:{description:"height of the page as drawn", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "method:ListGrid.setEditValue":{
        ref:"method:ListGrid.setEditValue",
        flags:"A",
        description:"\n  Modifies a field value being tracked as an unsaved user edit.<P>\n  Use for suggested or reformatted values for edits that remain unsaved.\n\n",
        name:"setEditValue",
        seeAlso:[],
        params:[
            {description:"Row number (or edit values ID)", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"Column number of cell, or name of field                                           having editValue updated",
             optional:false, name:"colNum", defaultValue:null, type:"number | string"},
            {description:"New value for the appropriate field.", optional:false, name:"value",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:TileGrid.selectionType":{
        ref:"attr:TileGrid.selectionType",
        flags:"IRW",
        description:"Defines a tileGrid's clickable-selection behavior.",
        name:"selectionType",
        seeAlso:[
            {ref:"type:SelectionStyle", description:null}
        ],
        valueType:"SelectionStyle",
        defaultValue:"Selection.MULTIPLE",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('tilingEditing')}"],
        definingClass:"class:TileGrid",
        groups:["selection", "appearance"]
    },
    "group:headerSpan":{
        ref:"group:headerSpan",
        refs:["method:ListGrid.getHeaderSpanContextMenuItems", "method:ListGrid.setHeaderSpans", 
         "method:ListGrid.setHeaderSpanTitle","object:HeaderSpan", "attr:ListGrid.headerSpans", "attr:ListGrid.headerSpanHeight", 
         "attr:ListGrid.headerSpanVAlign","attr:ListGrid.unspannedHeaderVAlign", "attr:ListGrid.headerSpanConstructor", 
         "attr:ListGrid.headerSpan","attr:HeaderSpan.name", "attr:HeaderSpan.fields", "attr:HeaderSpan.title", 
         "attr:HeaderSpan.height","attr:HeaderSpan.valign", "attr:HeaderSpan.align", "attr:ListGrid.showTreeColumnPicker"],
        name:"headerSpan",
        seeAlso:[],
        type:"group"
    },
    "attr:CubeGrid.simpleDeselect":{
        ref:"attr:CubeGrid.simpleDeselect",
        flags:"IRW",
        description:"If true, clicking on the existing selection causes it to be entirely deselected.",
        name:"simpleDeselect",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:Splitbar.src":{
        ref:"attr:Splitbar.src",
        flags:"IR",
        description:"The base URL for the image. \n <P>\n As with ${isc.DocUtils.linkForRef('attr:Img.src')}, the ${isc.DocUtils.linkForRef('type:State')} of the component is added to this URL.  Then,\n the image segment name as specified by ${isc.DocUtils.linkForRef('attr:StretchImg.items')} is added.\n <P>\n For example, for a stretchImg in \"Over\" state with a <code>src</code> of \"button.png\"\n and a segment name of \"stretch\", the resulting URL would be \"button_Over_stretch.png\".",
        name:"src",
        seeAlso:[
            {ref:"attr:StretchImg.hSrc", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["appearance"]
    },
    "group:dataSourceRelations":{
        ref:"group:dataSourceRelations",
        title:"Relations",
        refs:["attr:DataSourceField.primaryKey", "attr:DataSourceField.foreignKey", 
         "attr:DataSourceField.childrenProperty","attr:DataSourceField.rootValue", "attr:DataSource.childrenField"],
        description:"Properties for declaring relations between DataSource and within DataSources, such as\n master-detail relationship (a SalesOrder contains 0 or more OrderItems) and tree\n relationships (records can be linked by a parentId field).",
        name:"dataSourceRelations",
        seeAlso:[
            {ref:"attr:DataSourceField.foreignKey", description:null}
        ],
        type:"group"
    },
    "attr:Calendar.eventEditor":{
        ref:"attr:Calendar.eventEditor",
        flags:"R",
        description:"An autochild of type ${isc.DocUtils.linkForRef('class:DynamicForm')} which displays ${isc.DocUtils.linkForRef('object:CalendarEvent','event data')}. \n This form is created within the ${isc.DocUtils.linkForRef('attr:Calendar.eventEditorLayout','event editor layout')}",
        name:"eventEditor",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:RPCRequest.evalVars":{
        ref:"attr:RPCRequest.evalVars",
        flags:"IRWA",
        description:"If you've set ${isc.DocUtils.linkForRef('attr:RPCRequest.evalResult')} : true, then the property values of this object\n will be available in the evaluation scope of the result under the variable names\n specified by the property names. \n <p>\n So e.g. if evalVars is: <code>{foo: \"bar\"}</code> then a reference to the\n variable <code>foo</code> in the result will evaluate to <code>\"bar\"</code>.",
        name:"evalVars",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["viewLoading"]
    },
    "method:DataSource.evaluateCriterion":{
        ref:"method:DataSource.evaluateCriterion",
        flags:"",
        description:" Evaluate the given criterion with respect to the passed record.\n <P>\n Typically called by the ${isc.DocUtils.linkForRef('method:Operator.condition','condition')} function of a custom\n ${isc.DocUtils.linkForRef('object:Operator')} to evaluate ${isc.DocUtils.linkForRef('attr:Criterion.criteria','sub-criteria')}.\n\n",
        name:"evaluateCriterion",
        seeAlso:[],
        params:[
            {description:"record to evaluate", optional:false, name:"record", defaultValue:null,
             type:"Record"},
            {description:"criterion to use", optional:false, name:"criterion", defaultValue:null,
             type:"Criterion"}
        ],
        returns:{description:"whether the record meets the supplied ${isc.DocUtils.linkForRef('object:Criterion')}",
         type:"boolean"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "classMethod:Log.show":{
        ref:"classMethod:Log.show",
        flags:"",
        description:" Open the Developer Console.\n <P>\n The Developer Console should <b>always</b> be open while developing any ISC-enabled\n application, because ISC logs many important errors and warnings to the Developer Console.\n <P>\n In Internet Explorer, the Developer Console is able to log a stack trace for every JS error,\n including errors that occur in non-ISC code.\n <P>\n NOTE: if you have the Microsoft JavaScript Debugger installed, ISC will be unable to log\n stack traces on JS errors until you go to Tools->Internet Options->Advanced Tab and check\n \"Disable script debugging\".  The ability to see stack traces in the Developer Console is\n generally much more useful for debugging ISC-based applications than the generic Javascript\n Debugging facilities.\n \n",
        name:"show",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "attr:ListGrid.expansionFieldImageWidth":{
        ref:"attr:ListGrid.expansionFieldImageWidth",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is set to <code>true</code>, this property\n may be set to govern the width of the expansion image displayed to indicate whether a row \n is expanded. If unset, the expansionField image will be sized to match the\n ${isc.DocUtils.linkForRef('attr:ListGrid.booleanImageWidth')} for this grid.",
        name:"expansionFieldImageWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:ListGridField.showIf":{
        ref:"method:ListGridField.showIf",
        flags:"",
        description:" An optional ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')} which if provided, is evaluated to\n conditionally determine whether this field should be displayed. \n Evaluated on initial draw, then reevaluated on explicit \n calls to <code>listGrid.refreshFields()</code> or <code>listGrid.setFields()</code>. \n <P>\n Use 'showIf:\"false\"' to set a ListGrid field to initially hidden.\n <P>\n Note that explicit calls to ${isc.DocUtils.linkForRef('method:ListGrid.showField','grid.showField()')} or hideField()\n will wipe out the <code>showIf</code> expression, as will the end user showing and\n hiding columns via the ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','header contextMenu')}.\n <P>\n Also note that fields marked as ${isc.DocUtils.linkForRef('attr:DataSourceField.detail','detail:true')} will be hidden by\n default even if ${isc.DocUtils.linkForRef('attr:ListGrid.showDetailFields')} is <code>true</code>. To show detail fields\n inherited from a DataSource, include an explicit field definition for the field and\n set this property to return <code>true</code>.\n",
        name:"showIf",
        seeAlso:[
            {ref:"method:ListGrid.refreshFields", description:null}
        ],
        params:[
            {description:"A pointer to the listGrid containing the field", optional:false, name:"list",
             defaultValue:null, type:"ListGrid"},
            {description:"the ListGridField object", optional:false, name:"field", defaultValue:null,
             type:"ListGridField"},
            {description:"the index of the field", optional:false, name:"fieldNum", defaultValue:null,
             type:"integer"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "classAttr:Array.LOADING":{
        ref:"classAttr:Array.LOADING",
        flags:"IRA",
        description:"Marker value returned by Lists that manage remote datasets, indicating the requested data is\n being loaded. Note that the recommended test for loading data is to call ${isc.DocUtils.linkForRef('classMethod:Array.isLoading')}\n rather than compare to this value directly.",
        name:"LOADING",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"loading\"",
        type:"classAttr",
        definingClass:"object:Array"
    },
    "attr:Toolbar.canResizeItems":{
        ref:"attr:Toolbar.canResizeItems",
        flags:"IRWA",
        description:"If true, items (buttons) can be resized by dragging on them.",
        name:"canResizeItems",
        setter:["setCanResizeItems"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["dragndrop"]
    },
    "attr:MenuButton.menuButtonImage":{
        ref:"attr:MenuButton.menuButtonImage",
        flags:"IRA",
        description:"Image for menu button indicating that the button expands a menu.  This image is shown\n for menus expanding down from the button.   Menu direction is controlled by\n ${isc.DocUtils.linkForRef('attr:MenuButton.showMenuBelow')}.",
        name:"menuButtonImage",
        seeAlso:[
            {ref:"attr:MenuButton.menuButtonImageUp", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]menu_button.gif\"",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "attr:SortSpecifier.direction":{
        ref:"attr:SortSpecifier.direction",
        flags:"IR",
        description:"The direction in which this specifier should sort.",
        name:"direction",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SortSpecifier"
    },
    "attr:TextItem.mask":{
        ref:"attr:TextItem.mask",
        flags:"IRWA",
        description:"Input mask used to filter text entry.\n <P>\n Sample masks:\n <UL>\n <LI>Phone number: (###) ###-####</LI>\n <LI>Social Security number: ###-##-####\n <LI>First name: &gt;?&lt;??????????</LI>\n <LI>Date: ##/##/####</LI>\n <LI>State: &gt;LL</LI>\n </UL>\n Overview of available mask characters\n <P>\n <table class=\"normal\">\n <tr><th>Character</th><th>Description</th></tr>\n <tr><td>0</td><td>Digit (0 through 9) or plus [+] or minus [-] signs</td></tr>\n <tr><td>9</td><td>Digit or space</td></tr>\n <tr><td>#</td><td>Digit</td></tr>\n <tr><td>L</td><td>Letter (A through Z)</td></tr>\n <tr><td>?</td><td>Letter (A through Z) or space</td></tr>\n <tr><td>A</td><td>Letter or digit</td></tr>\n <tr><td>a</td><td>Letter or digit</td></tr>\n <tr><td>C</td><td>Any character or space</td></tr>\n <tr><td>&nbsp;</td></tr>\n <tr><td>&lt;</td><td>Causes all characters that follow to be convered to lowercase</td></tr>\n <tr><td>&gt;</td><td>Causes all characters that follow to be convered to uppercase</td></tr>\n </table>\n <P>\n Any character not matching one of the above mask characters or that is escaped\n with a backslash (\\) is considered to be a literal.\n <P>\n Custom mask characters can be defined by standard regular expression character set\n or range. For example, a hexadecimal color code mask could be:\n <UL>\n <LI>Color: \\#>[0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F]</LI>\n </UL>\n <P>\n Note: input mask cannot be used at the same time as a ${isc.DocUtils.linkForRef('attr:TextItem.keyPressFilter')}.",
        name:"mask",
        setter:["setMask"],
        seeAlso:[
            {ref:"attr:TextItem.keyPressFilter", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('maskedTextItem')}"],
        definingClass:"class:TextItem"
    },
    "type:PageEvent":{
        ref:"type:PageEvent",
        values:[
            {description:"Fires repeatedly (every 10 ms by default) when the system is idle (i.e.,\n        not busy running other scripts) after the page is loaded.",
             value:"\"idle\""},
            {description:"Fires when the page has finished loading. It corresponds to the\n        browser 'load' event normally handled by window.onload.",
             value:"\"load\""},
            {description:"Fires when the page is exited or unloaded. It corresponds to the\n        browser 'unload' event normally handled by window.onunload.",
             value:"\"unload\""},
            {description:"Fires when the browser window is resized by the user. It corresponds\n        to the browser 'resize' event normally handled by window.onresize.",
             value:"\"resize\""},
            {description:"Fires when the left mouse button is pressed on the Page.",
             value:"\"mouseDown\""},
            {description:"Fires when the right mouse button is pressed on the Page.",
             value:"\"rightMouseDown\""},
            {description:"Fires when the mouse moves on the Page.", value:"\"mouseMove\""},
            {description:"Fires when the left mouse button released on the Page.",
             value:"\"mouseUp\""},
            {description:"Fires when the user clicks the mouse on the Page.", value:"\"cick\""},
            {description:"Fires when the uesr double-clicks on the Page.", value:"\"doubleClick\""},
            {description:"Fires when the right mouse button is clicked on the page.  If your event handler\n        for this event returns false, the native browser context menu will be suppressed.<br>\n        Note: On the Macintosh platform, <code>Command+Click</code> may be used instead\n        of right-button click to trigger a context menu event.<br>\n        On the Opera browser, <code>Ctrl+Shift+Click</code> should be used instead of\n        right-button click.",
             value:"\"showContextMenu\""},
            {description:"Fires when a user presses a key on the keyboard. \n see classMethod:Page.setEvent()\n see classMethod:Page.clearEvent()",
             value:"\"keyPress\""}
        ],
        description:"Events registerable via ${isc.DocUtils.linkForRef('classMethod:Page.setEvent')}",
        name:"PageEvent",
        seeAlso:[],
        type:"type"
    },
    "method:Canvas.intersects":{
        ref:"method:Canvas.intersects",
        flags:"",
        description:"			Returns true if the rectangles of this widget and the specified widget overlap.\n",
        name:"intersects",
        seeAlso:[],
        params:[
            {description:"other canvas to test for intersection", optional:false, name:"other",
             defaultValue:null, type:"canvas"}
        ],
        returns:{description:"true if this canvas intersects other; false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:LoginDialog.formFields":{
        ref:"attr:LoginDialog.formFields",
        flags:"IR",
        description:"Customizes the fields present in the dialog, or specifies new fields to be\n present, in the same manner as with ${isc.DocUtils.linkForRef('attr:DynamicForm.fields')}.",
        name:"formFields",
        seeAlso:[
            {ref:"attr:DataBoundComponent.fields", description:null}
        ],
        valueType:"Array of FormItem Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "classMethod:EventHandler.getKeyEventCharacterValue":{
        ref:"classMethod:EventHandler.getKeyEventCharacterValue",
        flags:"",
        description:"          Returns the numeric characterValue reported by the browser.\n          Only available on keyPress events, and only for character (or ascii control) keys\n",
        name:"getKeyEventCharacterValue",
        seeAlso:[],
        returns:{description:"Numeric character value reported by the browser                   (ASCII value of the key pressed)",
         type:"integer"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "attr:PickList.pickListBaseStyle":{
        ref:"attr:PickList.pickListBaseStyle",
        flags:"IR",
        description:"Base Style for pickList cells.  As with ListGrid Cells, will have 'over', 'selected'\n and 'disabled' appended on changes of state for the cells.",
        name:"pickListBaseStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"pickListCell\"",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "method:ValuesManager.getMemberForField":{
        ref:"method:ValuesManager.getMemberForField",
        flags:"",
        description:" Given a fieldName or dataPath, this method will find the member responsible for\n interacting with that field's value.\n If no form is found, returns null.\n",
        name:"getMemberForField",
        seeAlso:[],
        params:[
            {description:"fieldName or dataPath to check", optional:false, name:"fieldName",
             defaultValue:null, type:"string || dataPath"}
        ],
        returns:{description:"member responsible for displaying this field (may be null).", type:"Canvas"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "attr:Progressbar.vertical":{
        ref:"attr:Progressbar.vertical",
        flags:"IRW",
        description:"Indicates whether this is a vertical or horizontal progressbar.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Progressbar",
        groups:["appearance"]
    },
    "attr:ListGrid.editEvent":{
        ref:"attr:ListGrid.editEvent",
        flags:"IRW",
        description:"Event that will trigger inline editing, see ${isc.DocUtils.linkForRef('type:ListGridEditEvent')} for options.\n <P>\n Note this setting has no effect unless ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit')} has been set to enable editing.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:ListGrid.editOnFocus')} and ${isc.DocUtils.linkForRef('method:ListGrid.startEditing')}.",
        name:"editEvent",
        seeAlso:[],
        valueType:"ListGridEditEvent",
        defaultValue:"\"doubleClick\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('editByRow')}"],
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:widgetEvents":{
        ref:"group:widgetEvents",
        refs:["method:Canvas.showContextMenu", "method:Canvas.hideContextMenu", 
         "method:Canvas.mouseOver","method:Canvas.mouseDown", "method:Canvas.rightMouseDown", "method:Canvas.mouseStillDown", 
         "method:Canvas.mouseMove","method:Canvas.mouseOut", "method:Canvas.mouseUp", "method:Canvas.click", 
         "method:Canvas.doubleClick","method:Canvas.mouseWheel", "method:Canvas.dragRepositionStart", 
         "method:Canvas.dragRepositionMove","method:Canvas.dragRepositionStop", "method:Canvas.dragResizeStart", 
         "method:Canvas.dragResizeMove","method:Canvas.dragResizeStop", "method:Canvas.dragStart", "method:Canvas.dragMove", 
         "method:Canvas.dragStop","method:Canvas.dropOver", "method:Canvas.dropMove", "method:Canvas.dropOut", 
         "method:Canvas.drop","method:Canvas.keyDown", "method:Canvas.keyUp", "method:Canvas.keyPress"],
        name:"widgetEvents",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.setSelectedState":{
        ref:"method:ListGrid.setSelectedState",
        flags:"",
        description:" Reset this grid's selection to match the ${isc.DocUtils.linkForRef('type:listGridSelectedState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to \n ${isc.DocUtils.linkForRef('method:ListGrid.getSelectedState')}.\n\n",
        name:"setSelectedState",
        seeAlso:[
            {ref:"method:ListGrid.getSelectedState", description:null}
        ],
        params:[
            {description:"Object describing the desired selection state of                                              the grid",
             optional:false, name:"selectedState", defaultValue:null, type:"listGridSelectedState"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "method:Layout.reflowNow":{
        ref:"method:Layout.reflowNow",
        flags:"A",
        description:" Layout members according to current settings, immediately.\n <br>\n Generally, when changes occur that require a layout to reflow (such as members being shown\n or hidden), the Layout will reflow only after a delay, so that multiple changes cause only\n one reflow.  To remove this delay for cases where it is not helpful, reflowNow() can be\n called.\n",
        name:"reflowNow",
        seeAlso:[],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:TreeGrid.openerImage":{
        ref:"attr:TreeGrid.openerImage",
        flags:"IR",
        description:"The base filename of the opener icon for the folder node when 'showConnectors' is false\n for this TreeGrid.<br>\n The opener icon is displayed beside the folder icon in the Tree column for folder nodes.\n Clicking on this icon will toggle the open state of the folder.<br>\n The filenames for these icons are assembled from this base filename and the state of the\n node, as follows:<br>\n If the openerImage is set to <code>{baseName}.{extension}</code>, \n <code>{baseName}_opened.{extension}</code> will be displayed next to opened folders, and\n <code>{baseName}_closed.{extension}</code> will be displayed next to closed folders, or\n if this page is in RTL mode, <code>{baseName}_opened_rtl.{extension}</code> and\n <code>{baseName}_closed_rtl.{extension}</code> will be used.",
        name:"openerImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]opener.gif\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:MultiSortDialog.addLevelButtonTitle":{
        ref:"attr:MultiSortDialog.addLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the addLevelButton",
        name:"addLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Add Level\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "classAttr:Operators.andTitle":{
        ref:"classAttr:Operators.andTitle",
        flags:"IR",
        description:"Title for the \"and\" operator",
        name:"andTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Match All\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "group:hiliting":{
        ref:"group:hiliting",
        title:"Hiliting Overview",
        refs:["method:CubeGrid.hiliteCell", "method:CubeGrid.hiliteCellList", 
         "method:CubeGrid.hiliteFacetValue","method:DataBoundComponent.enableHilite", "method:DataBoundComponent.disableHilite", 
         "method:DataBoundComponent.enableHiliting","method:DataBoundComponent.disableHiliting"],
        description:"Hiliting means special visual styling which is applied to specific data values that meet\n certain criteria.\n <P>\n A ${isc.DocUtils.linkForRef('object:Hilite')} definition contains styling information such as ${isc.DocUtils.linkForRef('attr:Hilite.cssText')} and\n ${isc.DocUtils.linkForRef('attr:Hilite.htmlBefore')} that define what the hilite looks like, as well as properties\n defining where the hilite is applied.\n <P>\n A hilite can be applied to data <b>either</b> by defining ${isc.DocUtils.linkForRef('attr:Hilite.criteria','criteria')}\n or by explicitly including markers on the data itself.  \n <P>\n Hiliting rules such as hiliting different ranges of values with different colors can be\n accomplished entirely client-side by defining ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} in hilite definitions\n that pick out values to be highlighted.\n <P>\n Hiliting rules that require server-side calculations can be achieved by assigning a\n ${isc.DocUtils.linkForRef('attr:Hilite.id')} to a hilite definition, and setting the\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.hiliteProperty')} on the records that should show that highlight.\n This can be used, for example, to hilite the record with the maximum value for a dataset\n that the application will load incrementally.",
        name:"hiliting",
        seeAlso:[],
        type:"group"
    },
    "attr:ImgButton.selected":{
        ref:"attr:ImgButton.selected",
        flags:"IRW",
        description:"Whether this component is selected.  For some components, selection affects appearance.",
        name:"selected",
        setter:["setSelected"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:SortSpecifier.property":{
        ref:"attr:SortSpecifier.property",
        flags:"IR",
        description:"The property name, eg a ${isc.DocUtils.linkForRef('object:ListGridField','field name')}, to which this sortSpecifier applies.",
        name:"property",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SortSpecifier"
    },
    "attr:ValuesManager.disableValidation":{
        ref:"attr:ValuesManager.disableValidation",
        flags:"IRWA",
        description:"If set to true, client-side validators will not run on the form when validate() is\n called.  Server-side validatiors (if any) will still run on attempted save.",
        name:"disableValidation",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null},
            {ref:"method:DynamicForm.submit", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ValuesManager",
        groups:["validation"]
    },
    "interface:List":{
        methods:["method:List.get", "method:List.getLength", "method:List.isEmpty", "method:List.first", 
         "method:List.last","method:List.indexOf", "method:List.lastIndexOf", "method:List.findIndex", 
         "method:List.findNextIndex","method:List.find", "method:List.findAll", "method:List.contains", 
         "method:List.containsAll","method:List.intersect", "method:List.equals", "method:List.getItems", 
         "method:List.getRange","method:List.duplicate", "method:List.set", "method:List.addAt", "method:List.removeAt", 
         "method:List.add","method:List.addList", "method:List.setLength", "method:List.addListAt", 
         "method:List.remove","method:List.removeList", "method:List.sort", "method:List.getProperty", 
         "method:List.sortByProperty","method:List.getValueMap", "method:List.dataChanged"],
        ref:"interface:List",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"An interface for an ordered collection of items.\n <P>\n This is the interface that is expected by list-oriented display components such as the\n ListGrid.  The JavaScript native Array object is retrofitted to support the List interface.\n Also, a valid List can be created by mixing the List interface into any class that supports:\n <ul>\n <li> for read-only support: get(position), getLength()\n <li> for modifiable support: set(position), addAt(object, position), removeAt(position)\n </ul>\n <P>\n NOTE: this interface is compatible with the java.util.List interface, except that:\n <ul>\n <li> for removal by index, removeAt() must be called instead of remove().  In Java, remove()\n is an overloaded method that takes either an int or Object, whereas in JavaScript, a\n Number is an Object.\n <li> Iterators don't exist.\n </ul> \n Some methods from the Java List interface have been omitted from the documentation to avoid\n redundancy.",
        name:"List",
        seeAlso:[],
        type:"interface"
    },
    "classMethod:EventHandler.leftButtonDown":{
        ref:"classMethod:EventHandler.leftButtonDown",
        flags:"",
        description:"			Returns true if the left mouse button is being pressed.\n\n",
        name:"leftButtonDown",
        seeAlso:[],
        returns:{description:"true == left button is down, false == up", type:"boolean"},
        type:"classMethod",
        platformNotes:"Mac:		Macintosh platform generally has only one mouse button - ISC considers it to be the \"left\"\n		mouse button, so this method will return true if the mouse is down on a single-button mouse Mac.",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "method:Canvas.dropOver":{
        ref:"method:Canvas.dropOver",
        flags:"A",
        description:"\n Executed when the compatible dragged object is first moved over this drop target. Your\n implementation can use this to show a custom visual indication that the object can be\n dropped here.\n\n",
        name:"dropOver",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customDrag')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:CubeGrid.getEditValues":{
        ref:"method:CubeGrid.getEditValues",
        flags:"",
        description:" Returns the current set of unsaved edits for a given row being edited.\n\n",
        name:"getEditValues",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"valuesID", defaultValue:null, type:"number | Object"},
            {description:"colNum of the record being edited. Only required if valuesID                         is passed in as a rowNum.",
             optional:false, name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"Current editValues object for the row.  This contains the current                     edit values in {fieldName1:value1, fieldName2:value2} format.",
         type:"object"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:ListGrid.headerMenuButtonConstructor":{
        ref:"attr:ListGrid.headerMenuButtonConstructor",
        flags:"IRA",
        description:"Constrocutor for the  ${isc.DocUtils.linkForRef('attr:ListGrid.headerMenuButton')}. If unset a standard \"Button\" will\n be rendered out. Note that this property may be overridden by different skins.",
        name:"headerMenuButtonConstructor",
        seeAlso:[],
        valueType:"className",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "attr:Calendar.eventOverlapIdenticalStartTimes":{
        ref:"attr:Calendar.eventOverlapIdenticalStartTimes",
        flags:"IR",
        description:"When set to true, events that start at the same time will not overlap each other to prevent \n events having their close button hidden",
        name:"eventOverlapIdenticalStartTimes",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "method:Selection.selectRange":{
        ref:"method:Selection.selectRange",
        flags:"",
        description:"			Select range of records from <code>start</code> to <code>end</code>, non-inclusive.\n",
        name:"selectRange",
        seeAlso:[],
        params:[
            {description:"start index to select", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end index (non-inclusive)", optional:false, name:"end", defaultValue:null,
             type:"number"},
            {description:"optional new selection state to set.  True means                                      selected, false means unselected.  Defaults to true.",
             optional:true, name:"newState", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:MultiSortPanel.getSortLevel":{
        ref:"method:MultiSortPanel.getSortLevel",
        flags:"",
        description:" Return a ${isc.DocUtils.linkForRef('object:SortSpecifier')} object for the requested levelNum.\n\n",
        name:"getSortLevel",
        seeAlso:[],
        params:[
            {description:"The index of the level to return a SortSpecifier for", optional:false,
             name:"levelNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"A SortSpecifier representing the requested levelNum", type:"SortSpecifier"},
        type:"method",
        definingClass:"class:MultiSortPanel"
    },
    "classMethod:Page.setAppImgDir":{
        ref:"classMethod:Page.setAppImgDir",
        flags:"",
        description:" Specify the directory for app-specific images.\n <P>\n This becomes the default location where any SmartClient component will load images from\n unless the special \"[SKIN]\" prefix is used to indicate that an image is part of a skin.\n <P>\n Default is \"[APP]images/\"\n\n",
        name:"setAppImgDir",
        seeAlso:[],
        params:[
            {description:"New imgDir URL.", optional:true, name:"URL", defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "method:ListGrid.hasChanges":{
        ref:"method:ListGrid.hasChanges",
        flags:"",
        description:" Whether the grid as a whole has any unsaved edits, in any row.\n",
        name:"hasChanges",
        seeAlso:[],
        returns:{description:"returns true of any unsaved edits are present", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Layout.hPolicy":{
        ref:"attr:Layout.hPolicy",
        flags:"IRWA",
        description:"Sizing policy applied to members on horizonal axis",
        name:"hPolicy",
        seeAlso:[],
        valueType:"LayoutPolicy",
        defaultValue:"\"fill\"",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:OperationBinding.invalidateCache":{
        ref:"attr:OperationBinding.invalidateCache",
        flags:"IR",
        description:"If set, every invocation of this operationBinding will invalidate the local cache, forcing\n a server visit to refresh the data.",
        name:"invalidateCache",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "attr:ListGrid.canResizeFields":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.canResizeFields",
        description:"Indicates whether fields in this listGrid can be resized by dragging header\n          fields.",
        name:"canResizeFields",
        setter:["setCanResizeFields"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('columnSize')}"],
        defaultValue:"true",
        groups:["dragging"],
        definingClass:"class:ListGrid"
    },
    "attr:Window.showFooter":{
        ref:"attr:Window.showFooter",
        flags:"IRW",
        description:"If true, show a footer for this Window, including resizer, statusBar, etc.\n This setting is commonly overridden for skinning purposes.",
        name:"showFooter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('windowFooter')}"],
        definingClass:"class:Window",
        groups:["appearance", "footer"]
    },
    "type:XMLDocument":{
        ref:"type:XMLDocument",
        description:"XMLDocument is the \"parsed\" or object form of XML, which allows XML to be navigated as\n a tree of nodes with attributes, namespaces and other metadata, as opposed to being\n manipulated as just a String.\n <P> \n XMLDocument is a native object supplied directly by the browser.  The SmartClient-supported\n interfaces for this object are methods that take an XMLDocument as an argument (such as\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectNodes')}).  If you want to retrieve XML data and display it in a\n SmartClient component, read about ${isc.DocUtils.linkForRef('group:clientDataIntegration','XML Data Binding')}.  To\n extract data as JavaScript Objects from XML, see ${isc.DocUtils.linkForRef('classMethod:XMLTools.toJS')}.  Direct\n manipulation of XMLDocument is subject to cross-browser inconsistencies, bugs, memory leaks\n and performance issues.",
        name:"XMLDocument",
        seeAlso:[],
        type:"type"
    },
    "attr:ListGridField.isRemoveField":{
        ref:"attr:ListGridField.isRemoveField",
        flags:"IRA",
        description:"If set to true and ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} is true, this field will be rendered\n as the remove-field for this grid. In most common usage scenarios this field will essentially\n be a placeholder indicating where the remove field should be rendered, meaning properties \n other than <code>isRemoveField</code>, such as <code>name</code> or <code>title</code>, may\n be left unset.",
        name:"isRemoveField",
        seeAlso:[
            {ref:"attr:ListGrid.canRemoveRecords", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:Canvas.setPercentSource":{
        ref:"method:Canvas.setPercentSource",
        flags:"A",
        description:" Setter method for the ${isc.DocUtils.linkForRef('attr:Canvas.percentSource','percentSource')} attribute.\n",
        name:"setPercentSource",
        seeAlso:[],
        params:[
            {description:"eterNew percent source (if ommitted existing                                      percentSource will just be cleared).",
             optional:true, name:"sourceWidget", defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:DynamicForm.rightTitlePrefix":{
        ref:"attr:DynamicForm.rightTitlePrefix",
        flags:"IRW",
        description:"The string prepended to the title of an item in this form if its\n          titleOrientation property is set to \"right\".",
        name:"rightTitlePrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\":&nbsp;\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "group:componentBinding":{
        ref:"group:componentBinding",
        title:"Component Binding",
        refs:["attr:DataSourceField.title", "attr:DataSourceField.canEdit", 
         "attr:DataSourceField.canSave","attr:DataSourceField.editorType", "attr:DataSourceField.hidden", 
         "attr:DataSourceField.ignore","attr:DataSourceField.detail"],
        description:"Properties that control how a DataBound Component binds to this DataSource",
        name:"componentBinding",
        seeAlso:[],
        type:"group"
    },
    "method:SectionStack.getVisibleSections":{
        ref:"method:SectionStack.getVisibleSections",
        flags:"",
        description:"\n Returns the list of currently visible sections.  The list items are section IDs.\n\n",
        name:"getVisibleSections",
        seeAlso:[],
        returns:{description:"list of hidden sections", type:"List"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "classMethod:EventHandler.getKey":{
        ref:"classMethod:EventHandler.getKey",
        flags:"",
        description:"			Return the name of the key for the event passed in.\n			Note that this is only set reliably for keyboard events.\n\n",
        name:"getKey",
        seeAlso:[],
        returns:{description:"Key Name", type:"KeyName"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "attr:ListGridField.baseStyle":{
        ref:"attr:ListGridField.baseStyle",
        flags:"IRW",
        description:"Custom base style to apply to all cells in this field instead of ${isc.DocUtils.linkForRef('attr:ListGrid.baseStyle')}\n  To override the baseStyle at the row level, use \n  ${isc.DocUtils.linkForRef('attr:ListGrid.recordBaseStyleProperty','record[listGrid.recordBaseStyleProperty]')}\n  instead.",
        name:"baseStyle",
        seeAlso:[
            {ref:"attr:ListGrid.recordBaseStyleProperty", description:null}
        ],
        valueType:"cssClass",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('gridCells')}"],
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "attr:Calendar.eventDialog":{
        ref:"attr:Calendar.eventDialog",
        flags:"R",
        description:"An autochild of type ${isc.DocUtils.linkForRef('class:Window')} that displays a quick event entry form within a \n popup window.",
        name:"eventDialog",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:ImgButton.setState":{
        ref:"method:ImgButton.setState",
        flags:"",
        description:" Set the 'state' of this object, this changes it's appearance.\n\n",
        name:"setState",
        seeAlso:[
            {ref:"method:ImgButton.setDisabled", description:"which also affects state values."}
        ],
        params:[
            {description:"new state", optional:false, name:"newState", defaultValue:null, type:"State"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state", "appearance"]
    },
    "attr:ListGrid.expansionFieldFalseImage":{
        ref:"attr:ListGrid.expansionFieldFalseImage",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is set to <code>true</code>, this property\n determines the image to display in the expansion field for collapsed rows.\n If unset, the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanFalseImage')} will be used.",
        name:"expansionFieldFalseImage",
        seeAlso:[
            {ref:"attr:ListGrid.expansionFieldTrueImage", description:null},
            {ref:"attr:ListGrid.expansionFieldImageWidth", description:null},
            {ref:"attr:ListGrid.expansionFieldImageHeight", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:ViewFileItem.showFileInline":{
        ref:"attr:ViewFileItem.showFileInline",
        flags:"IR",
        description:"Indicates whether to stream the image and display it\n inline or to display the View and Download icons.",
        name:"showFileInline",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ViewFileItem"
    },
    "attr:Canvas.animateAcceleration":{
        ref:"attr:Canvas.animateAcceleration",
        flags:"IRWA",
        description:"Default acceleration effect to apply to all animations on this Canvas.\n Can be overridden by setting animationAcceleration for specific animations or by passing\n an acceleration function directly into the appropriate method.",
        name:"animateAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"\"smoothEnd\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:TileGrid.selectRecords":{
        ref:"method:TileGrid.selectRecords",
        flags:"",
        description:"\n Select/deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n\n",
        name:"selectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to select", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "method:ListGrid.refreshFields":{
        ref:"method:ListGrid.refreshFields",
        flags:"",
        description:" Re-evaluates ${isc.DocUtils.linkForRef('method:ListGridField.showIf')} for each field, dynamically showing and \n hiding the appropriate set of fields\n",
        name:"refreshFields",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:CubeGrid.cellSelectionChanged":{
        ref:"method:CubeGrid.cellSelectionChanged",
        flags:"",
        description:"\n      Handler/Notification function for cell selection change\n      May update header button styles.\n\n      <br><i>methodType</i> handler\n",
        name:"cellSelectionChanged",
        seeAlso:[],
        params:[
            {description:"array of cells with new selection state", optional:false, name:"cellList",
             defaultValue:null, type:"Array of cells"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:ListGridField.imageURLSuffix":{
        ref:"attr:ListGridField.imageURLSuffix",
        flags:"IRWA",
        description:"If any cells in this field are showing a value icon (see: ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')})\n or this is has ${isc.DocUtils.linkForRef('type:ListGridFieldType')} set to <code>\"image\"</code>, this the value\n of this property will be appended to the end of the URL for the icon displayed.<br>\n Typical usage might be to append a file type such as <code>\".gif\"</code> to the\n filename of the image.<br>\n For editable fields, this property will also be passed through to any editors as\n ${isc.DocUtils.linkForRef('attr:FormItem.imageURLSuffix')}.",
        name:"imageURLSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('imageType')}"],
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:StretchImgButton.labelVPad":{
        ref:"attr:StretchImgButton.labelVPad",
        flags:"IRW",
        description:"The padding for a StretchImgButton's label is determined as follows.\n <P>\n If <code>labelHPad</code> is set it will specify the horizontal padding applied to the\n label. Similarly if <code>labelVPad</code> is set it will specify the vertical padding\n for the label, regardless of the button's ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical','vertical')} setting.\n <P>\n Otherwise <code>labelLengthPad</code> can be set to specify the label padding along the\n length axis (ie: horizontal padding if ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical')} is false,\n otherwise vertical padding), and \n <code>labelBreadthPad</code> can be set to specify the label padding along the other axis.\n <P>\n Otherwise the padding on the length axis will match the ${isc.DocUtils.linkForRef('attr:StretchImgButton.capSize')} and\n will be set to zero on the breadth axis.\n <P>\n So by default the label will be sized to match the center image of the StretchImgButton, but\n these settings allow the label to partially or wholly overlap the caps.",
        name:"labelVPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "attr:StretchImgButton.labelLengthPad":{
        ref:"attr:StretchImgButton.labelLengthPad",
        flags:"IRW",
        description:"The padding for a StretchImgButton's label is determined as follows.\n <P>\n If <code>labelHPad</code> is set it will specify the horizontal padding applied to the\n label. Similarly if <code>labelVPad</code> is set it will specify the vertical padding\n for the label, regardless of the button's ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical','vertical')} setting.\n <P>\n Otherwise <code>labelLengthPad</code> can be set to specify the label padding along the\n length axis (ie: horizontal padding if ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical')} is false,\n otherwise vertical padding), and \n <code>labelBreadthPad</code> can be set to specify the label padding along the other axis.\n <P>\n Otherwise the padding on the length axis will match the ${isc.DocUtils.linkForRef('attr:StretchImgButton.capSize')} and\n will be set to zero on the breadth axis.\n <P>\n So by default the label will be sized to match the center image of the StretchImgButton, but\n these settings allow the label to partially or wholly overlap the caps.",
        name:"labelLengthPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "method:Array.remove":{
        ref:"method:Array.remove",
        flags:"",
        description:" Remove first instance of the passed object from this array, sliding other items around to\n fill gaps.\n\n",
        name:"remove",
        seeAlso:[],
        params:[
            {description:"item to remove", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"true if a matching object was found and removed, false if no matching object was found and the list remains unchanged.",
         type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "attr:MultiSortDialog.applyButtonTitle":{
        ref:"attr:MultiSortDialog.applyButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the applyButton",
        name:"applyButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Apply\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "method:Canvas.scrollToLeft":{
        ref:"method:Canvas.scrollToLeft",
        flags:"",
        description:" Horizontally scrolls the content of the widget to 0\n\n",
        name:"scrollToLeft",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:ImgButton.select":{
        ref:"method:ImgButton.select",
        flags:"",
        description:" Select this object.\n",
        name:"select",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:Chart.title":{
        ref:"attr:Chart.title",
        flags:"IR",
        description:"Title for the chart as a whole.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "method:GridRenderer.refreshCellStyle":{
        ref:"method:GridRenderer.refreshCellStyle",
        flags:"",
        description:" Refresh the styling of an individual cell without redrawing the grid.\n <P>\n The cell's CSS class and CSS text will be refreshed, to the current values returned by\n getCellStyle() and getCellCSSText() respectively.\n <P>\n The cell's contents (as returned by getCellValue()) will <b>not</b> be refreshed.  To\n refresh both styling and contents, call refreshCell() instead.\n\n",
        name:"refreshCellStyle",
        seeAlso:[
            {ref:"method:GridRenderer.refreshCell", description:"to update cell contents too"}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column number of cell to refresh", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["appearance"]
    },
    "method:Canvas.setCursor":{
        ref:"method:Canvas.setCursor",
        flags:"",
        description:"			Sets the cursor for this widget to cursor. See the cursor property\n          for possible values.\n",
        name:"setCursor",
        seeAlso:[],
        params:[
            {description:"new cursor", optional:false, name:"newCursor", defaultValue:null,
             type:"Cursor"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["cues"]
    },
    "group:subset":{
        ref:"group:subset",
        refs:["method:Array.getUniqueItems"],
        name:"subset",
        seeAlso:[],
        type:"group"
    },
    "method:List.isEmpty":{
        ref:"method:List.isEmpty",
        flags:"",
        description:" Return whether or not this array is empty\n\n",
        name:"isEmpty",
        seeAlso:[],
        returns:{description:"true == this array is empty, false == some items in the array", type:"boolean"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "method:SectionStack.addItem":{
        ref:"method:SectionStack.addItem",
        flags:"",
        description:" Add a canvas as an item to a section.\n",
        name:"addItem",
        seeAlso:[],
        params:[
            {description:"ID or index of the section to add item to", optional:false, name:"section",
             defaultValue:null, type:"String or Number"},
            {description:"Item to insert into the section", optional:false, name:"item",
             defaultValue:null, type:"Canvas"},
            {description:"Index into section to insert item", optional:false, name:"index",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:Canvas.animateScrollTime":{
        ref:"attr:Canvas.animateScrollTime",
        flags:"IRWA",
        description:"Default time for performing an animated scroll.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateScrollTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:ListGrid.checkboxFieldTrueImage":{
        ref:"attr:ListGrid.checkboxFieldTrueImage",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance')} is set to <code>\"checkbox\"</code> this property\n determines the image to display in the checkbox field for a selected row.\n If unset, the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} will be used.",
        name:"checkboxFieldTrueImage",
        seeAlso:[
            {ref:"attr:ListGrid.checkboxFieldFalseImage", description:null},
            {ref:"attr:ListGrid.checkboxFieldImageWidth", description:null},
            {ref:"attr:ListGrid.checkboxFieldImageHeight", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "group:buttonIcon":{
        ref:"group:buttonIcon",
        title:"Button Icon",
        refs:["method:PanelHeader.iconClick", "method:StatefulCanvas.setIconOrientation", 
         "method:StatefulCanvas.setIcon","method:Button.setIconOrientation", "method:Button.setIcon", "method:Button.iconClick", 
         "method:Label.setIconOrientation","method:Label.setIcon", "method:ImgButton.setIconOrientation", "method:ImgButton.setIcon", 
         "method:StretchImgButton.setIconOrientation","method:StretchImgButton.setIcon", "method:StretchImgButton.iconClick", 
         "attr:ListGridField.iconSpacing","attr:ListGridField.showDisabledIcon", "attr:ListGridField.showRollOverIcon", 
         "attr:ListGridField.showFocusedIcon","attr:ListGridField.showDownIcon", "attr:ListGridField.showSelectedIcon", 
         "attr:StatefulCanvas.icon","attr:StatefulCanvas.iconSize", "attr:StatefulCanvas.iconWidth", 
         "attr:StatefulCanvas.iconHeight","attr:StatefulCanvas.iconOrientation", "attr:StatefulCanvas.iconSpacing", 
         "attr:StatefulCanvas.showDisabledIcon","attr:StatefulCanvas.showRollOverIcon", "attr:StatefulCanvas.showDownIcon", 
         "attr:StatefulCanvas.showSelectedIcon","attr:StatefulCanvas.showFocusedIcon", "attr:Button.icon", "attr:Button.iconSize", 
         "attr:Button.iconWidth","attr:Button.iconHeight", "attr:Button.iconOrientation", "attr:Button.iconAlign", 
         "attr:Button.iconSpacing","attr:Button.showDisabledIcon", "attr:Button.showRollOverIcon", 
         "attr:Button.showFocusedIcon","attr:Button.showDownIcon", "attr:Button.showSelectedIcon", "attr:Label.icon", 
         "attr:Label.iconSize","attr:Label.iconWidth", "attr:Label.iconHeight", "attr:Label.iconOrientation", 
         "attr:Label.iconAlign","attr:Label.iconSpacing", "attr:Label.showDisabledIcon", "attr:Label.showRollOverIcon", 
         "attr:Label.showFocusedIcon","attr:Label.showDownIcon", "attr:Label.showSelectedIcon", "attr:ImgButton.icon", 
         "attr:ImgButton.iconSize","attr:ImgButton.iconWidth", "attr:ImgButton.iconHeight", "attr:ImgButton.iconOrientation", 
         "attr:ImgButton.iconAlign","attr:ImgButton.iconSpacing", "attr:ImgButton.showDisabledIcon", 
         "attr:ImgButton.showRollOverIcon","attr:ImgButton.showFocusedIcon", "attr:ImgButton.showDownIcon", 
         "attr:ImgButton.showSelectedIcon","attr:StretchImgButton.icon", "attr:StretchImgButton.iconSize", 
         "attr:StretchImgButton.iconWidth","attr:StretchImgButton.iconHeight", "attr:StretchImgButton.iconOrientation", 
         "attr:StretchImgButton.iconAlign","attr:StretchImgButton.iconSpacing", "attr:StretchImgButton.showDisabledIcon", 
         "attr:StretchImgButton.showRollOverIcon","attr:StretchImgButton.showFocusedIcon", "attr:StretchImgButton.showDownIcon", 
         "attr:StretchImgButton.showSelectedIcon"],
        description:"Control over optional icons shown in Buttons, Labels and other contexts",
        name:"buttonIcon",
        seeAlso:[],
        type:"group"
    },
    "attr:Tree.nameProperty":{
        ref:"attr:Tree.nameProperty",
        flags:"IRW",
        description:"Name of the property on a ${isc.DocUtils.linkForRef('object:TreeNode')} that holds a name for the node that is unique\n among it's immediate siblings, thus allowing a unique path to be used to identify the node,\n similar to a file system.  Default value is \"name\".  See ${isc.DocUtils.linkForRef('attr:TreeNode.name')} for usage.",
        name:"nameProperty",
        seeAlso:[
            {ref:"attr:TreeNode.name", description:null}
        ],
        valueType:"string",
        defaultValue:"\"name\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "attr:StretchImgButton.hiliteAccessKey":{
        ref:"attr:StretchImgButton.hiliteAccessKey",
        flags:"IRW",
        description:"If set to true, if the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.title','title')} of this button contains the\n specified ${isc.DocUtils.linkForRef('attr:Canvas.accessKey','accessKey')}, when the title is displayed to the user\n it will be modified to include HTML to underline the accessKey.<br>\n Note that this property may cause titles that include HTML (rather than simple strings)\n to be inappropriately modified, so should be disabled if your title string includes\n HTML characters.",
        name:"hiliteAccessKey",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "method:CubeGrid.getFacetValuesRow":{
        ref:"method:CubeGrid.getFacetValuesRow",
        flags:"",
        description:" Get the index of the first row in the grid that matches the specified FacetValueMap.<br>\n The facetValues passed in should contain values for at least one row facet. If values are \n sparse (values not specified for every row facet), the first row matching the specified\n facet values will be returned.\n",
        name:"getFacetValuesRow",
        seeAlso:[],
        params:[
            {description:"facet values to find", optional:false, name:"facetValues", defaultValue:null,
             type:"facetValueMap"}
        ],
        returns:{description:"index of first row in the grid that matches the facet values passed in.",
         type:"integer"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:Validator.serverOnly":{
        ref:"attr:Validator.serverOnly",
        flags:"IR",
        description:"Indicates this validator runs on the server only.",
        name:"serverOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Validator"
    },
    "attr:TextItem.maskOverwriteMode":{
        ref:"attr:TextItem.maskOverwriteMode",
        flags:"IRWA",
        description:"During entry into masked field, should keystrokes overwrite current position value?\n By default new keystrokes are inserted into the field.",
        name:"maskOverwriteMode",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextItem"
    },
    "classMethod:Log.setLogPriority":{
        ref:"classMethod:Log.setLogPriority",
        flags:"",
        description:" Set the priority of messages that will be visible for some log category, when logged on\n this Class or Instance object.<br>\n If called with no category, this priority will be applied to every logged message on this\n object<br>\n To set the visible log priority for some category across the entire page, use\n <code>isc.Log.setPriority()</code> instead.\n",
        name:"setLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "method:ListGrid.selectionChanged":{
        ref:"method:ListGrid.selectionChanged",
        flags:"",
        description:" Called when (row-based) selection changes within this grid. Note this method fires for\n each record for which selection is modified - so when a user clicks inside a grid this\n method will typically fire twice (once for the old record being deselected, and once for\n the new record being selected).\n\n",
        name:"selectionChanged",
        seeAlso:[],
        params:[
            {description:"record for which selection changed", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"New selection state (true for selected, false for unselected)",
             optional:false, name:"state", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('multipleSelect')}"],
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:Canvas.dragRepositionCursor":{
        ref:"attr:Canvas.dragRepositionCursor",
        flags:"IRWA",
        description:"Cursor to switch to if the mouse is over a widget that is drag repositionable.",
        name:"dragRepositionCursor",
        seeAlso:[],
        valueType:"Cursor",
        defaultValue:"Canvas.MOVE",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:StretchImg.vSrc":{
        ref:"attr:StretchImg.vSrc",
        flags:"IRW",
        description:"Base URL for the image if ${isc.DocUtils.linkForRef('attr:StretchImg.vertical')} is true and \n ${isc.DocUtils.linkForRef('attr:StretchImg.src')} is unset.",
        name:"vSrc",
        seeAlso:[
            {ref:"attr:StretchImg.src", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "group:clientOnlyDataSources":{
        ref:"group:clientOnlyDataSources",
        title:"Client Only DataSources",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        refs:["attr:DataSource.clientOnly", "attr:DataSource.testData"],
        description:"For prototyping purposes, a \"client-only\" DataSource can be created that has no permanent\n storage and never contacts the server, instead using a set of test data to respond to\n requests in the same manner as a server-based DataSource might.\n <P>\n The client-side interface to a client-only DataSource is identical to a server-based\n DataSource, including asynchronous responses, so that a client-only DataSource can be\n replaced by a server-based DataSource without code changes.  The only difference is that\n changes to records belonging to the DataSource persist only until the page is reloaded.\n <p>\n Client-only DataSources allow you to create a complete prototype application in an .html\n file that does not require a server.\n <p>\n The <code>clientOnly</code> property is specified to create a client-only DataSource, and\n the <code>testData</code> property should contain the test dataset, as an Array of Objects,\n one per DataSource record.  For example:\n <pre>\n   isc.DataSource.create({\n       ID:\"supplyItem\",\n       fields: ...,\n       clientOnly:true,\n       testData:[\n          {itemName:\"Pencil\", cost:5.50},\n          ...\n       ]\n   });\n </pre>\n If you have existing test data in XML (see the ${isc.DocUtils.linkForRef('group:adminConsole','Admin Console')} for\n expected format),\n you can use the XML->JS translation engine to load it into a client-only DataSource, like so:\n <pre>\n   isc.DataSource.create({\n     ID:\"solutions\",\n     fields: ...,\n     clientOnly : true,\n     testData : \n         &lt;isomorphic:XML filename=\"shared/ds/test_data/solutions.data.xml\"/&gt;\n   });\n </pre>\n Another useful practice is to specify both the clientOnly DataSource and it's test data in\n XML, so that the ${isc.DocUtils.linkForRef('group:adminConsole','Admin Console')} can later be used to import the\n DataSource and its test data into a SQL Database.  An idiom for accomplishing this is:\n <pre>\n   &lt;isomorphic:loadDS name=\"solutions\"/&gt;\n   isc.DataSource.getDataSource(\"solutions\").addProperties({\n     clientOnly : true,\n     testData : \n        &lt;isomorphic:XML filename=\"shared/ds/test_data/solutions.data.xml\"/&gt;\n   });\n </pre>\n Finally, if you specify your DataSource as <code>clientOnly: true</code>, omit testData\n entirely, and provide either a ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')} or a <code>testFileName</code>, the\n DataSource will lazily make a one-time fetch against the specified data file the first time\n an operation is called on it.  From then on, the DataSource will work against the local\n cache created from this initial request.  This is a quick way to prototype against some test\n data that may eventually be returned from an arbitrary back-end.",
        name:"clientOnlyDataSources",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.doubleClick":{
        ref:"method:Canvas.doubleClick",
        flags:"A",
        description:"\n Executed when the left mouse button is clicked twice in rapid succession (within\n ${isc.DocUtils.linkForRef('attr:Canvas.doubleClickDelay')} by default) in this object.\n\n",
        name:"doubleClick",
        seeAlso:[
            {ref:"attr:Canvas.doubleClickDelay", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "type:SortDirection":{
        ref:"type:SortDirection",
        values:[
            {description:"Sort in ascending order (eg: A-Z, larger items later in the list)",
             value:"\"ascending\""},
            {description:"Sort in descending order (eg: Z-A, larger items earlier in the list)",
             value:"\"descending\""}
        ],
        description:null,
        name:"SortDirection",
        seeAlso:[],
        type:"type",
        groups:["sorting"]
    },
    "group:sizing":{
        ref:"group:sizing",
        refs:["method:FormItem.getIconHeight", "method:FormItem.getIconWidth", 
         "method:ListGrid.setHeaderHeight","method:Canvas.setRect", "method:Canvas.getWidth", "method:Canvas.setWidth", 
         "method:Canvas.getHeight","method:Canvas.setHeight", "method:Canvas.getRight", "method:Canvas.setRight", 
         "method:Canvas.getBottom","method:Canvas.setBottom", "method:Canvas.getScrollWidth", "method:Canvas.getScrollHeight", 
         "method:Canvas.getViewportWidth","method:Canvas.getViewportHeight", "method:Canvas.getInnerHeight", 
         "method:Canvas.getInnerWidth","method:Canvas.getInnerContentHeight", "method:Canvas.getInnerContentWidth", 
         "method:Canvas.getVisibleWidth","method:Canvas.getVisibleHeight", "method:Canvas.resizeBy", 
         "method:Canvas.setPercentSource","method:Canvas.layoutChildren", "method:Canvas.resizeTo", "method:Canvas.parentResized", 
         "method:Canvas.setOverflow","method:Canvas.adjustForContent", "classMethod:Page.resizeTo", "classMethod:Page.getWidth", 
         "classMethod:Page.getHeight","classMethod:Page.getScrollWidth", "classMethod:Page.getScrollHeight", 
         "classMethod:Page.getScrollLeft","classMethod:Page.getScrollTop", "attr:GridRenderer.fixedColumnWidths", 
         "attr:GridRenderer.autoFit","attr:ListGrid.bodyOverflow", "attr:ListGrid.headerHeight", "attr:Menu.defaultWidth", 
         "attr:Menu.cellHeight","attr:TileLayout.tileSize", "attr:TileLayout.tileWidth", "attr:TileLayout.tileHeight", 
         "attr:TileLayout.overflow","attr:Canvas.width", "attr:Canvas.height", "attr:Canvas.defaultWidth", 
         "attr:Canvas.defaultHeight","attr:Canvas.minWidth", "attr:Canvas.maxWidth", "attr:Canvas.minHeight", 
         "attr:Canvas.maxHeight","attr:Canvas.overflow", "attr:Canvas.percentSource", "attr:Canvas.percentBox", 
         "attr:Canvas.snapTo","attr:Canvas.snapEdge", "attr:Canvas.snapOffsetLeft", "attr:Canvas.snapOffsetTop", 
         "attr:StatefulCanvas.autoFit","attr:Button.autoFit", "attr:Img.size", "attr:Label.wrap", "attr:Label.autoFit", 
         "attr:ToolStrip.height","attr:ImgButton.autoFit", "attr:StretchImgButton.autoFit", "type:Overflow"],
        description:"Width and height of an object",
        name:"sizing",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.cellHeight":{
        ref:"attr:ListGrid.cellHeight",
        flags:"IRW",
        description:"The default height of each row in pixels.",
        name:"cellHeight",
        seeAlso:[
            {ref:"method:GridRenderer.getRowHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('multilineValues')}"],
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "group:labels":{
        ref:"group:labels",
        refs:["attr:DetailViewer.labelPrefix", "attr:DetailViewer.labelSuffix", 
         "attr:DetailViewer.wrapLabel","attr:DetailViewer.wrapValues"],
        name:"labels",
        seeAlso:[],
        type:"group"
    },
    "attr:DetailViewer.recordsPerBlock":{
        ref:"attr:DetailViewer.recordsPerBlock",
        flags:"IRW",
        description:"The number of records to display in a block. A block is a horizontal row on a page\n          containing one or more records, as specified by the value of recordsPerBlock. The\n          height of a block is equal to the height of a single record. The default setting of\n          1 causes each record to appear by itself in a vertical row. Setting recordsPerBlock\n          to 2 would cause records to appear side by side in groups of two.\n          Use a value of \"*\" to indicate all records.",
        name:"recordsPerBlock",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "attr:Canvas.position":{
        ref:"attr:Canvas.position",
        flags:"IRWA",
        description:"Absolute or relative, corresponding to the \"absolute\" (with respect to parent) or\n \"relative\" (with respect to document flow) values for the CSS position attribute.\n <P>\n Setting <code>position:\"relative\"</code> enables SmartClient components to be embedded\n directly into the native HTML flow of a page, causing the component to be rendered \n within an existing DOM structure. \n This attribute should only be set to <code>\"relative\"</code> on a top level component \n (a component with no ${isc.DocUtils.linkForRef('attr:Canvas.parentElement')}). \n <P>\n There are 2 ways to embed relatively positioned canvases in the DOM - by default the\n component will be written out inline when it gets ${isc.DocUtils.linkForRef('method:Canvas.draw','drawn()n')}. For example\n to embed a canvas in an HTML table you could use this code: \n <pre>\n &lt;table&gt;\n   &lt;tr&gt;\n     &lt;td&gt;\n       &lt;script&gt;\n         isc.Canvas.create({autoDraw:true, backgroundColor:\"red\", position:\"relative\"});\n       &lt;/script&gt;\n     &lt;td&gt;\n   &lt;/tr&gt;\n &lt;/table&gt;\n </pre>\n Alternatively you can make use of the ${isc.DocUtils.linkForRef('attr:Canvas.htmlElement')} attribute.\n <P>\n Relative positioning is intended as a short-term integration scenario while incrementally\n upgrading existing applications.\n Note that relative positioning is not used to manage layout within SmartClient components -\n instead the ${isc.DocUtils.linkForRef('class:Layout')} class would typically be used.\n For best consistency and flexibility across browsers, all SmartClient layout managers\n use absolute positioning.\n <P>\n For canvases with a specified ${isc.DocUtils.linkForRef('attr:Canvas.htmlElement')}, this attribute defaults to\n <code>\"relative\"</code>. In all other cases the default value will be \n <code>\"aboslute\"</code>.",
        name:"position",
        seeAlso:[],
        valueType:"Positioning",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('inlineComponents')}"],
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:FormItem.clearValue":{
        ref:"method:FormItem.clearValue",
        flags:"",
        description:" Clear the value for this form item.\n <P>\n Note that if a default value is specified, value will be set to that default value,\n otherwise value will be cleared, (and removed from the containing form's values).\n",
        name:"clearValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:LoginDialog.passwordItem":{
        ref:"attr:LoginDialog.passwordItem",
        flags:"IR",
        description:"Password field item in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"passwordItem",
        seeAlso:[
            {ref:"attr:LoginDialog.allowBlankPassword", description:null},
            {ref:"attr:LoginDialog.passwordItemTitle", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "classMethod:Log.echo":{
        ref:"classMethod:Log.echo",
        flags:"",
        description:" Return a short string representation of any object, suitable for viewing by a developer for\n debugging purposes.\n <P>\n If passed an object containing other objects, echo will not recurse into subobjects,\n summarizing them instead via echoLeaf().\n <P>\n NOTE: echo() is used to generate the output shown in the Log window when evaluating an\n expression.\n <P>\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n \n",
        name:"echo",
        seeAlso:[
            {ref:"classMethod:Log.echoAll", description:null},
            {ref:"classMethod:Log.echoLeaf", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "method:CubeGrid.makeChart":{
        ref:"method:CubeGrid.makeChart",
        flags:"",
        description:" Chart the portion of the dataset indicated by <code>fixedFacetValues</code>, for all \n values of the <code>variableFacets</code>.\n <P>\n One, two or more variableFacets may be passed.  Two variable facets for a column chart will\n result in ${isc.DocUtils.linkForRef('attr:Chart.stacked','stacking')} or clustering.  Three facets or more may be\n supported by some ${isc.DocUtils.linkForRef('attr:CubeGrid.chartType','chartTypes')} or\n ${isc.DocUtils.linkForRef('attr:CubeGrid.chartConstructor','charting engines')}.\n \n",
        name:"makeChart",
        seeAlso:[],
        params:[
            {description:"set of facet values to hold constant.  Pass null to                                         chart the entire dataset.",
             optional:false, name:"fixedFacetValues", defaultValue:null, type:"FacetValueMap"},
            {description:"set of facets to be charted", optional:false, name:"variableFacets",
             defaultValue:null, type:"Array of FacetIds"},
            {description:"properties to pass through to the created                        ${isc.DocUtils.linkForRef('interface:Chart')}",
             optional:false, name:"chartProperties", defaultValue:null, type:"Chart properties"}
        ],
        returns:{description:"created chart instance", type:"Chart"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:Tree.discardParentlessNodes":{
        ref:"attr:Tree.discardParentlessNodes",
        flags:"IRA",
        description:"If this tree has ${isc.DocUtils.linkForRef('attr:Tree.modelType','modelType:\"parent\"')}, should nodes in the data array for the\n tree be dropped if they have an explicitly specified value for the ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')}\n which doesn't match any other nodes in the tree. If set to false these nodes will be added as\n children of the root node.",
        name:"discardParentlessNodes",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Tree"
    },
    "attr:SectionStackSection.resizeable":{
        ref:"attr:SectionStackSection.resizeable",
        flags:"I",
        description:"If set to false, then the items in this section will not be resized by sectionHeader\n repositioning.  You may also set this flag directly on any of the items in any section to\n cause that item to not be resizeable.",
        name:"resizeable",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('resizeSections')}"],
        definingClass:"object:SectionStackSection"
    },
    "attr:DetailViewerField.imageSize":{
        ref:"attr:DetailViewerField.imageSize",
        flags:"IRW",
        description:"Size of images shown for fieldTypes image in this field.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n height.  For example, if <code>field.imageSize</code> is \"logoSize\",\n <code>record.logoSize</code> will control the size of the image.",
        name:"imageSize",
        seeAlso:[
            {ref:"attr:DetailViewerField.imageWidth", description:null},
            {ref:"attr:DetailViewerField.imageHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField",
        groups:["imageColumns"]
    },
    "attr:MultiSortDialog.optionsGrid":{
        ref:"attr:MultiSortDialog.optionsGrid",
        flags:"IR",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ListGrid')} allowing the user to configure a set of \n ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.optionsGridProperties</code> and \n <code>multiSortPanel.optionsGridDefaults</code>.",
        name:"optionsGrid",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:MenuButton.setShowMenuBelow":{
        ref:"method:MenuButton.setShowMenuBelow",
        flags:"",
        description:" Setter for the 'showMenuButtonBelow' property - determines whether the menu will be\n shown above or below the menubutton.\n",
        name:"setShowMenuBelow",
        seeAlso:[],
        params:[
            {description:"True if the menu should be shown below the menubutton.", optional:false,
             name:"below", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:MenuButton"
    },
    "method:PickList.getPickListFilterCriteria":{
        ref:"method:PickList.getPickListFilterCriteria",
        flags:"A",
        description:" ${isc.DocUtils.linkForRef('group:stringMethods','StringMethod')} to return a set of filter criteria to be applied to\n the data displayed in the pickList when it is shown.\n <P>\n If this is a databound item the criteria will be passed as criteria to\n ${isc.DocUtils.linkForRef('method:DataSource.fetchData')}.  Otherwise an equivalent client-side filter will be\n performed on the data returned by ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')}.\n <P>\n By default combines ${isc.DocUtils.linkForRef('attr:FormItem.optionCriteria')} with\n ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')} if specified, otherwise an empty \n set of criteria so all records will be displayed.\n\n",
        name:"getPickListFilterCriteria",
        seeAlso:[],
        returns:{description:"criteria to be used for databound or local filtering", type:"Criteria"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundDependentSelects')}"],
        definingClass:"interface:PickList"
    },
    "method:Array.equals":{
        ref:"method:Array.equals",
        flags:"",
        description:" Return whether this list is equal to another list.\n <P>\n Two lists are equal only if they have the same length and all contained items are in the same\n order and are also equal.\n\n",
        name:"equals",
        seeAlso:[],
        params:[
            {description:"list to check for equality", optional:false, name:"list", defaultValue:null,
             type:"List"}
        ],
        returns:{description:"whether the specified list is equal to this list", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:Tab.pickerTitle":{
        ref:"attr:Tab.pickerTitle",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true for this TabSet, if set this property will determine\n the title of the picker menu item for this tab. If unset, ${isc.DocUtils.linkForRef('attr:Tab.title')} will be used\n instead",
        name:"pickerTitle",
        seeAlso:[
            {ref:"attr:TabSet.showTabPicker", description:null},
            {ref:"attr:Tab.title", description:null}
        ],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab",
        groups:["tabBarControls"]
    },
    "attr:CubeGrid.defaultFacetWidth":{
        ref:"attr:CubeGrid.defaultFacetWidth",
        flags:"IRW",
        description:"default width of inner column headers",
        name:"defaultFacetWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "method:ListGrid.hasErrors":{
        ref:"method:ListGrid.hasErrors",
        flags:"",
        description:" Does this grid currently have errors associated with editValues for any row in the grid.\n",
        name:"hasErrors",
        seeAlso:[
            {ref:"method:ListGrid.rowHasErrors", description:null},
            {ref:"method:ListGrid.cellHasErrors", description:null}
        ],
        returns:{description:"true if there are unresolved errors, false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:ColumnTree.fields":{
        ref:"attr:ColumnTree.fields",
        flags:"IRW",
        description:"An array of field objects, specifying the order, layout, dynamic calculation, and\n sorting behavior of each field in each column in the columnTree object. In ColumnTrees, \n the fields array specifies sub-columns within each main column. \n Each field in the fields array is a ListGridField object.\n <p>\n If ${isc.DocUtils.linkForRef('attr:ColumnTree.dataSource')} is also set, this value acts as a set of overrides as\n explained in ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}.",
        name:"fields",
        seeAlso:[
            {ref:"object:ListGridField", description:null}
        ],
        valueType:"Array of ListGridField",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["databinding"]
    },
    "method:ListGrid.rowEditorExit":{
        ref:"method:ListGrid.rowEditorExit",
        flags:"",
        description:" Callback fired when the user attempts to navigate away from the current edit row, \n or complete the current edit.\n <P>\n Return false from this method to cancel the default behavior (Saving / cancelling the\n current edit / moving to the next edit cell).\n\n",
        name:"rowEditorExit",
        seeAlso:[
            {ref:"method:ListGridField.editorExit", description:null}
        ],
        params:[
            {description:"How was the edit completion fired?", optional:false,
             name:"editCompletionEvent", defaultValue:null, type:"EditCompletionEvent"},
            {description:"record for the cell being edited", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"new values for the record [Note that fields that have                                 not been edited will not be included in this object]",
             optional:false, name:"newValues", defaultValue:null, type:"object"},
            {description:"row number for the row being left", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Returning false from this method will cancel the default behavior                      (for example saving the row) and leave the editor visible and focus\n                      in this edit cell.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:ValuesManager.cancel":{
        ref:"method:ValuesManager.cancel",
        flags:"",
        description:"\n This method exists for clean integration with existing server frameworks that have a 'cancel'\n feature which typically clears session state associated with the form.  When this method is\n called, an RPC is sent to the server with a parameter named\n ${isc.DocUtils.linkForRef('attr:DynamicForm.cancelParamName')} with the value\n ${isc.DocUtils.linkForRef('attr:DynamicForm.cancelParamValue')}.<p>\n\n Note that no other form data is sent.  By default the current top-level page is replaced with the\n reply.  If you wish to ignore the server reply instead, call this method like this:\n <pre>\n dynamicFormInstance.cancel({ignoreTimeout: true, target: null});\n </pre>\n\n",
        name:"cancel",
        seeAlso:[],
        params:[
            {description:"additional properties to set on the RPCRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["submitting"]
    },
    "method:Label.setIconOrientation":{
        ref:"method:Label.setIconOrientation",
        flags:"",
        description:" Changes the orientation of the icon relative to the text of the button.\n\n",
        name:"setIconOrientation",
        seeAlso:[],
        params:[
            {description:"The new orientation of the icon relative to the text of the button.",
             optional:false, name:"orientation", defaultValue:null, type:"\"left\" or \"right\""}
        ],
        type:"method",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "method:List.findAll":{
        ref:"method:List.findAll",
        flags:"",
        description:" Find all objects where property == value in the object.\n <P>\n Pass an Object as the <code>propertyName</code> argument to match multiple properties.\n\n",
        name:"findAll",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"all matching Objects or null if none found", type:"Array"},
        type:"method",
        definingClass:"interface:List",
        groups:["access", "find"]
    },
    "method:DOMGrid.setRootElement":{
        ref:"method:DOMGrid.setRootElement",
        flags:"",
        description:" Set the root element (or document) to view in the tree.\n",
        name:"setRootElement",
        seeAlso:[],
        params:[
            {description:"new root element", optional:false, name:"rootElement", defaultValue:null,
             type:"DOMElement"}
        ],
        type:"method",
        definingClass:"class:DOMGrid"
    },
    "method:CubeGrid.getEditedRecord":{
        ref:"method:CubeGrid.getEditedRecord",
        flags:"",
        description:" Returns the combination of unsaved edits (if any) and original values (if any) for a given\n cell being edited.\n <P>\n The returned value is never null, and can be freely modified.\n\n",
        name:"getEditedRecord",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"valuesID", defaultValue:null, type:"number | Object"},
            {description:"colNum of the record being edited. Only required if the records                          rowNum is passed in as the first parameter",
             optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"A copy of the record with unsaved edits included", type:"Object"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "method:ListGridField.getFieldTitle":{
        ref:"method:ListGridField.getFieldTitle",
        flags:"",
        description:" If your derivation of the field title is more complex than specifying a static string,\n you can specify a getFieldTitle() method on your field to return the title string.\n Otherwise you can use the ${isc.DocUtils.linkForRef('attr:ListGridField.title')} attribute on the field to specify the title.\n <P>\n You can use ${isc.DocUtils.linkForRef('method:ListGrid.setFieldProperties','setFieldProperties()')} to dynamically\n update the title.\n\n",
        name:"getFieldTitle",
        seeAlso:[
            {ref:"attr:ListGridField.title", description:null}
        ],
        params:[
            {description:"pointer back to the ListGrid", optional:false, name:"viewer",
             defaultValue:null, type:"ListGrid"},
            {description:"index of this field in the grid's fields array.", optional:false,
             name:"fieldNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "classMethod:DataSource.setTypeOperators":{
        ref:"classMethod:DataSource.setTypeOperators",
        flags:"",
        description:" Set the list of valid ${isc.DocUtils.linkForRef('type:OperatorId')}s for a given FieldType.\n \n",
        name:"setTypeOperators",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"typeName", defaultValue:null, type:"FieldType"},
            {description:"available Operators", optional:false, name:"operators", defaultValue:null,
             type:"Array of OperatorId"}
        ],
        type:"classMethod",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "attr:CubeGrid.skinImgDir":{
        ref:"attr:CubeGrid.skinImgDir",
        flags:"IR",
        description:"Default directory for skin images (those defined by the class), relative to the\n Page-wide ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','skinDir')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"images/CubeGrid/\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["images"]
    },
    "group:viewLoading":{
        ref:"group:viewLoading",
        refs:["method:ViewLoader.setViewURL", "class:ViewLoader", "attr:RPCRequest.evalResult", 
         "attr:RPCRequest.evalVars","attr:ViewLoader.loadingMessage"],
        name:"viewLoading",
        seeAlso:[],
        type:"group"
    },
    "method:Tree.addList":{
        ref:"method:Tree.addList",
        flags:"",
        description:"\n Add a list of nodes to some parent.\n\n",
        name:"addList",
        seeAlso:[
            {ref:"group:sharingNodes", description:null}
        ],
        params:[
            {description:"The list of nodes to add", optional:false, name:"nodeList", defaultValue:null,
             type:"List of TreeNode"},
            {description:"Parent of the nodes being added.  You can pass                                          in either the ${isc.DocUtils.linkForRef('object:TreeNode')} itself, or a path to\n	                                        the node (as a String), in which case a\n	                                        ${isc.DocUtils.linkForRef('method:Tree.find')} is performed to find\n	                                        the node.",
             optional:false, name:"parent", defaultValue:null, type:"String or TreeNode"},
            {description:"Position of the new nodes in the children list. If not	                                specified, the nodes will be added at the end of the list.",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        returns:{description:"List of added nodes.", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:ListGrid.booleanTrueImage":{
        ref:"attr:ListGrid.booleanTrueImage",
        flags:"IRWA",
        description:"Image to display for a true value in a boolean field.\n <P>\n To turn this off explicitly set ${isc.DocUtils.linkForRef('attr:ListGridField.suppressValueIcon')} to true.\n <P>\n If this and ${isc.DocUtils.linkForRef('attr:ListGrid.booleanFalseImage')} are undefined, this will be set to\n ${isc.DocUtils.linkForRef('attr:CheckboxItem.checkedImage')}.",
        name:"booleanTrueImage",
        seeAlso:[
            {ref:"attr:ListGrid.booleanFalseImage", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:PickTreeItem.displayField":{
        ref:"attr:PickTreeItem.displayField",
        flags:"IR",
        description:"Specifies an alternative field from which display values should be retrieved for this\n item.<br>\n If this item is not databound (${isc.DocUtils.linkForRef('attr:PickTreeItem.dataSource')} is unset), this is\n implemented by picking up the value of the specified field from the\n ${isc.DocUtils.linkForRef('attr:PickTreeItem.valueTree')}.<br>\n Otherwise this item will attempt to map its underlying value to a display value\n by retrieving a record from the ${isc.DocUtils.linkForRef('attr:PickTreeItem.dataSource')} where the \n ${isc.DocUtils.linkForRef('attr:PickTreeItem.valueField')} matches this item's value, and displaying the \n <code>displayField</code> value from that record.<br>",
        name:"displayField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:PickTreeItem"
    },
    "method:DynamicForm.getField":{
        ref:"method:DynamicForm.getField",
        flags:"",
        description:" Synonym for dynamicForm.getItem()\n\n",
        name:"getField",
        seeAlso:[
            {ref:"method:DynamicForm.getItem", description:null}
        ],
        params:[
            {description:"name of the item you're looking for", optional:false, name:"itemName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"FormItem object or null if not found", type:"FormItem"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["items"]
    },
    "classMethod:EventHandler.rightButtonDown":{
        ref:"classMethod:EventHandler.rightButtonDown",
        flags:"",
        description:"			Returns true if the right mouse button is being pressed.\n\n",
        name:"rightButtonDown",
        seeAlso:[],
        returns:{description:"true == right button is down, false == up", type:"boolean"},
        type:"classMethod",
        platformNotes:"Mac:		Macintosh platform generally has only one mouse button, and the\n		control key being held down serves the same purpose of the \n		right mouse button on Windows.  This is taken into account automatically.<br>\n      Opera: \n      The Opera browser does not pass right mouse button events to JavaScript code by default\n      (the user must explictly enable this bahavior via a menu item). Therefore we \n      treat <b>Shift+Ctrl+Click</b> as a context click in Opera.",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "attr:StretchImgButton.showDownIcon":{
        ref:"attr:StretchImgButton.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:ListGrid.valueIconLeftPadding":{
        ref:"attr:ListGrid.valueIconLeftPadding",
        flags:"IRW",
        description:"How much padding should there be on the left of valueIcons by default\n Can be overridden at the field level",
        name:"valueIconLeftPadding",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "method:ListGrid.setRowErrors":{
        ref:"method:ListGrid.setRowErrors",
        flags:"",
        description:" Set the validation errors for some row (replacing any pre-existant validation errors)\n",
        name:"setRowErrors",
        seeAlso:[
            {ref:"method:ListGrid.getRowErrors", description:null},
            {ref:"method:ListGrid.setFieldError", description:null}
        ],
        params:[
            {description:"row to add validation error for", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"validation errors for the row in the format                           <code>{fieldName:errorMessage, ...}</code> <br>or <br>\n                          <code>{fieldName:[errorMessage1, errorMessage2], ...}</code>",
             optional:false, name:"errors", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:ListGridField.displayFormat":{
        ref:"attr:ListGridField.displayFormat",
        flags:"IRWA",
        description:"For fields of type <code>\"date\"</code>, set this property to a valid\n ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} to specify how the date should be formatted.<br>\n For fields of type <code>\"time\"</code>, set this property to a valid \n ${isc.DocUtils.linkForRef('type:timeFormatter','timeFormatter')} to specify how the time should be formatted.<br>\n If unset, display format may be set at the listGrid level via ${isc.DocUtils.linkForRef('attr:ListGrid.dateFormatter')}\n or ${isc.DocUtils.linkForRef('attr:ListGrid.timeFormatter')}.\n <p>\n If this field is editable the displayFormat will also be passed to the editor created\n to edit this field.  For dates you may also need to set ${isc.DocUtils.linkForRef('attr:ListGridField.inputFormat')}.",
        name:"displayFormat",
        seeAlso:[
            {ref:"attr:ListGridField.inputFormat", description:null},
            {ref:"attr:ListGrid.dateFormatter", description:null},
            {ref:"attr:ListGrid.timeFormatter", description:null}
        ],
        valueType:"varies",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:Toolbar.buttons":{
        ref:"attr:Toolbar.buttons",
        flags:"IRW",
        description:"An array of button object initializers. See the Button Widget Class for standard\n button properties. The following additional properties can also be specified for\n button sizing and positioning on the toolbar itself:<br><br>\n <ul><li>width--Specifies the width of this button as an absolute number of pixels, a\n named property of the toolbar that specifies an absolute number of pixels, a\n percentage of the remaining space (e.g. '60%'), or \"*\" (default) to allocate an\n equal portion of the remaining space.\n <li>height--Specifies the height of this button.\n <li>extraSpace--Specifies an optional amount of extra space, in pixels, to separate\n this button from the next button in the toolbar.</ul>",
        name:"buttons",
        setter:["setButtons"],
        seeAlso:[
            {ref:"method:Toolbar.addButtons", description:null},
            {ref:"method:Toolbar.removeButtons", description:null},
            {ref:"class:Button", description:null}
        ],
        valueType:"array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Toolbar"
    },
    "attr:Calendar.showDateChooser":{
        ref:"attr:Calendar.showDateChooser",
        flags:"IR",
        description:"Determines whether the ${isc.DocUtils.linkForRef('attr:Calendar.dateChooser','dateChooser')} is displayed.",
        name:"showDateChooser",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["visibility"]
    },
    "attr:DSRequest.useFlatHeaderFields":{
        ref:"attr:DSRequest.useFlatHeaderFields",
        flags:"IRW",
        description:"Cause the ${isc.DocUtils.linkForRef('attr:DSRequest.useFlatFields','useFlatFields')} XML serialization behavior to be used\n for <b>all</b> soap headers in the request.  See also ${isc.DocUtils.linkForRef('attr:DSRequest.headerData')}.",
        name:"useFlatHeaderFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest",
        groups:["flatFields"]
    },
    "method:WebService.getSoapMessage":{
        ref:"method:WebService.getSoapMessage",
        flags:"A",
        description:" Return the SOAP message that will be formed from this WSRequest.\n \n",
        name:"getSoapMessage",
        seeAlso:[],
        params:[
            {description:"web service request object", optional:false, name:"wsRequest",
             defaultValue:null, type:"WSRequest Properties"}
        ],
        returns:{description:"SOAP message", type:"String"},
        type:"method",
        definingClass:"class:WebService"
    },
    "method:Calendar.setChosenDate":{
        ref:"method:Calendar.setChosenDate",
        flags:"",
        description:" Set the current date for which the calendar will display events.\n\n",
        name:"setChosenDate",
        seeAlso:[],
        params:[
            {description:"the new date to set as the current date", optional:false, name:"newDate",
             defaultValue:null, type:"Date"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:OperationBinding.preventHTTPCaching":{
        ref:"attr:OperationBinding.preventHTTPCaching",
        flags:"IR",
        description:"Configures ${isc.DocUtils.linkForRef('attr:DataSource.preventHTTPCaching')} on a per-operationType basis.",
        name:"preventHTTPCaching",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "attr:Window.src":{
        ref:"attr:Window.src",
        flags:"IRW",
        description:"A URL to load as content for the Window's body.  If specified, this\n attribute will take precedence over the items attribute.\n <P>\n Note that setting window.src is essentially a shortcut for setting ${isc.DocUtils.linkForRef('attr:Window.items')}\n to a single HTMLflow with a specified ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsURL','contentsURL')}.",
        name:"src",
        setter:["setSrc"],
        seeAlso:[
            {ref:"attr:Window.contentsType", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "attr:Canvas.className":{
        deprecated:"In favor or ${isc.DocUtils.linkForRef('attr:Canvas.styleName')} as of SmartClient release 5.5",
        flags:"IRW",
        type:"attr",
        ref:"attr:Canvas.className",
        description:"The CSS class applied to this widget as a whole.",
        name:"className",
        setter:["setClassName"],
        valueType:"CSSStyleName",
        seeAlso:[],
        defaultValue:"\"normal\"",
        groups:["appearance"],
        definingClass:"class:Canvas"
    },
    "attr:DataSource.resultSetClass":{
        ref:"attr:DataSource.resultSetClass",
        flags:"IRA",
        description:"Class for ResultSets used by this datasource.  If null, defaults to using\n ${isc.DocUtils.linkForRef('class:ResultSet')}.\n <P>\n This can be set to a custom subclass of ResultSet that, for example, hangs onto to extra\n information necessary for integration with web services.",
        name:"resultSetClass",
        seeAlso:[],
        valueType:"Class Object",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "object:CalendarEvent":{
        methods:[],
        ref:"object:CalendarEvent",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Calendar"],
        attrs:["attr:CalendarEvent.startDate", "attr:CalendarEvent.endDate", "attr:CalendarEvent.name", 
         "attr:CalendarEvent.description","attr:CalendarEvent.canEdit", "attr:CalendarEvent.eventWindowStyle"],
        description:"An object which represents an event to occur at a specific time, displayed within the\n calendar.",
        name:"CalendarEvent",
        seeAlso:[],
        type:"object",
        groups:["data"]
    },
    "attr:DataSourceField.canEdit":{
        ref:"attr:DataSourceField.canEdit",
        flags:"IR",
        description:"Whether this field can ever be edited by the user.  If set to <code>false</code>, no\n DataBound component will ever try to offer an editing interface for this field.",
        name:"canEdit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "class:ColorItem":{
        methods:["method:ColorItem.pickerColorSelected"],
        exampleConfig:"ColorItem.xml",
        ref:"class:ColorItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:ColorItem.showPickerIcon", "attr:ColorItem.pickerIconWidth", 
         "attr:ColorItem.pickerIconHeight","attr:ColorItem.pickerIconSrc", "attr:ColorItem.pickerIconProperties", 
         "attr:ColorItem.defaultPickerMode","attr:ColorItem.allowComplexMode", "attr:ColorItem.supportsTransparency"],
        description:"FormItem for selecting a color via a pop-up ColorPicker.",
        name:"ColorItem",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGridField.target":{
        ref:"attr:ListGridField.target",
        flags:"IRW",
        description:"By default, clicking a link rendered by this item opens it in a new browser window.  You \n can alter this behavior by setting this property.  The value of this property will be \n passed as the value to the <code>target</code> attribute of the anchor tag used to render \n the link.\n <P>\n If you set listGridField.target to \"javascript\", the default behaviour is to catch and \n consume mouse-clicks that would result in the link being followed.  Instead, the\n ${isc.DocUtils.linkForRef('method:ListGrid.cellClick')} event is fired for the containing cell.",
        name:"target",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"_blank\"",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "classAttr:GroupingMessages.upcomingNextMonthTitle":{
        ref:"classAttr:GroupingMessages.upcomingNextMonthTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs next month,\n relative to the current date.",
        name:"upcomingNextMonthTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Next Month\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:ValuesManager.editSelectedData":{
        ref:"method:ValuesManager.editSelectedData",
        flags:"",
        description:"\n Edit the record selected in the specified selection component (typically a\n ${isc.DocUtils.linkForRef('class:ListGrid')}).\n <P>\n Updates the values of this editor to match the selected record's values.\n <P>\n If this form has a dataSource, then saving via ${isc.DocUtils.linkForRef('method:ValuesManager.saveData')} will use the \n \"update\" operation type.\n\n",
        name:"editSelectedData",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"the ListGrid or ID of a ${isc.DocUtils.linkForRef('class:ListGrid')} whose currently selected\n     record(s) is/are to be edited",
             optional:false, name:"selectionComponent", defaultValue:null, type:"ListGrid or ID"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('updateOperation')}"],
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "method:ListGrid.setFieldProperties":{
        ref:"method:ListGrid.setFieldProperties",
        flags:"",
        description:" Set properties for a particular field, such as the title.\n <P>\n NOTE: to resize a field, use resizeField() instead.\n\n",
        name:"setFieldProperties",
        seeAlso:[],
        params:[
            {description:"name of the field, or index.", optional:false, name:"fieldNum",
             defaultValue:null, type:"number or String"},
            {description:"properties to apply to the header", optional:false, name:"properties",
             defaultValue:null, type:"Button Properties"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:Img.resetSrc":{
        ref:"method:Img.resetSrc",
        flags:"A",
        description:" Refresh the image being shown.  Call this when the ${isc.DocUtils.linkForRef('attr:Img.src')} attribute has not changed, but\n other state that affects the image URL (such as being selected) has changed.\n\n",
        name:"resetSrc",
        seeAlso:[],
        type:"method",
        definingClass:"class:Img",
        groups:["appearance"]
    },
    "attr:Calendar.showQuickEventDialog":{
        ref:"attr:Calendar.showQuickEventDialog",
        flags:"IR",
        description:"Determines whether the quick event dialog is displayed when a time is clicked. If this is\n false, the full event editor is displayed.",
        name:"showQuickEventDialog",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["editing"]
    },
    "method:Array.duplicate":{
        ref:"method:Array.duplicate",
        flags:"A",
        description:" Return an Array that is a shallow copy of the list, that is, containing the same items.\n\n",
        name:"duplicate",
        seeAlso:[],
        returns:{description:"new array, pointing to the same items", type:"Array"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "method:Canvas.containsFocus":{
        ref:"method:Canvas.containsFocus",
        flags:"",
        description:" Returns true if the keyboard focus is in this Canvas or any child of this Canvas.\n",
        name:"containsFocus",
        seeAlso:[],
        returns:{description:"whether this Canvas contains the keyboard focus", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "attr:BatchUploader.requestProperties":{
        ref:"attr:BatchUploader.requestProperties",
        flags:"IRW",
        description:"Object containing properties to send with every \"add\" request this batchUploader sends.",
        name:"requestProperties",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:ImgButton.showTitle":{
        ref:"attr:ImgButton.showTitle",
        flags:"IRWA",
        description:"Determines whether any specified ${isc.DocUtils.linkForRef('method:StatefulCanvas.getTitle','title')} will be \n displayed for this component.<br>\n Applies to Image-based components only, where the title will be rendered out in a label\n floating over the component",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton"
    },
    "attr:FormItem.name":{
        ref:"attr:FormItem.name",
        flags:"IR",
        description:"Name for this form field.\n <br><br>\n The FormItem's name determines the name of the property it edits within the form.",
        name:"name",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["basics"]
    },
    "method:ListGrid.setFieldState":{
        ref:"method:ListGrid.setFieldState",
        flags:"",
        description:" Sets some presentation properties (visibility, width, userFormula and userSummary) of the \n listGrid fields based on the ${isc.DocUtils.linkForRef('type:listGridFieldState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to ${isc.DocUtils.linkForRef('method:ListGrid.getFieldState')}.\n\n",
        name:"setFieldState",
        seeAlso:[
            {ref:"method:ListGrid.getFieldState", description:null}
        ],
        params:[
            {description:"state to apply to the listGrid's fields.", optional:false, name:"fieldState",
             defaultValue:null, type:"listGridFieldState"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:TextItem.width":{
        ref:"attr:TextItem.width",
        flags:"IRW",
        description:"Default width for fields.",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"150",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["appearance"]
    },
    "method:Calendar.eventRemoved":{
        ref:"method:Calendar.eventRemoved",
        flags:"",
        description:" Notification fired whenever a user removes an event\n <P>\n In a calendar with a DataSource, eventRemoved() fires <b>after</b> the event has\n been successfully removed from the server\n\n",
        name:"eventRemoved",
        seeAlso:[],
        params:[
            {description:"the event that was removed", optional:false, name:"event", defaultValue:null,
             type:"CalendarEvent"}
        ],
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "attr:ListGridField.groupGranularity":{
        ref:"attr:ListGridField.groupGranularity",
        flags:"IR",
        description:"Granularity of grouping for numeric fields.\n <P>\n Groups will be formed based on ranges of values of size <code>groupGranularity</code>.  For\n example, if groupGranularity were 1000, groups would be 0-1000, 1000-2000, etc.",
        name:"groupGranularity",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "classMethod:EventHandler.getY":{
        ref:"classMethod:EventHandler.getY",
        flags:"",
        description:"			Return the page-relative Y (vertical) coordinate of an event.\n\n",
        name:"getY",
        seeAlso:[],
        returns:{description:"y-coordinate in page coordinate space", type:"number"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "method:MenuItem.click":{
        ref:"method:MenuItem.click",
        flags:"",
        description:" Executed when this menu item is clicked by the user. The click handler must be specified\n as a function or string of script.  Return false to suppress the ${isc.DocUtils.linkForRef('method:Menu.itemClick')}\n handler if specified.\n\n",
        name:"click",
        seeAlso:[],
        params:[
            {description:"for a menu shown as a context menu, the Canvas the menu was shown                        on.  Otherwise the ${isc.DocUtils.linkForRef('class:Menu')} instance of which this\n                        ${isc.DocUtils.linkForRef('object:MenuItem')} is a member.",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"The ${isc.DocUtils.linkForRef('object:MenuItem')} that was clicked on.",
             optional:false, name:"item", defaultValue:null, type:"MenuItem"},
            {description:"The ${isc.DocUtils.linkForRef('class:Menu')} instance of which this ${isc.DocUtils.linkForRef('object:MenuItem')} is a                          member.",
             optional:false, name:"menu", defaultValue:null, type:"Menu"},
            {description:"Index of the column the user clicked. May be null if the                          user activated the menu via a keyboard event.",
             optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"object:MenuItem",
        groups:["menuItemEvents"]
    },
    "classMethod:Time.setNormalDisplayFormat":{
        ref:"classMethod:Time.setNormalDisplayFormat",
        flags:"",
        description:" Sets the default format for strings returned by ${isc.DocUtils.linkForRef('classMethod:Time.toTime')}.\n",
        name:"setNormalDisplayFormat",
        seeAlso:[],
        params:[
            {description:"Optional custom formatter to use. Will accept  a function (which will be passed a pointer to the date to perform the conversion), or\n  a string designating a standard formatter",
             optional:false, name:"formatter", defaultValue:null, type:"timeFormatter | function"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "attr:TreeGrid.canDropOnLeaves":{
        ref:"attr:TreeGrid.canDropOnLeaves",
        flags:"IRWA",
        description:"Whether drops are allowed on leaf nodes.\n <P>\n Dropping is ordinarily not allowed on leaf nodes unless ${isc.DocUtils.linkForRef('attr:TreeGrid.canReorderRecords')} is\n set.  \n <P>\n The default action for a drop on a leaf node is to place the node in that leaf's parent\n folder.  This can be customized by overriding ${isc.DocUtils.linkForRef('method:TreeGrid.folderDrop')}.\n <P>\n Note that enabling <code>canDropOnLeaves</code> is usually only appropriate where you\n intend to add a custom ${isc.DocUtils.linkForRef('method:TreeGrid.folderDrop')} implementation that <b>does not</b> add a\n child node under the leaf.  If you want to add a child nodes to a leaf, instead of\n enabling canDropOnLeaves, use empty folders isntead - see ${isc.DocUtils.linkForRef('method:Tree.isFolder')} for how\n to control whether a node is considered a folder.",
        name:"canDropOnLeaves",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "method:Canvas.getHSnapOrigin":{
        ref:"method:Canvas.getHSnapOrigin",
        flags:"",
        description:" Get an offset to be used when calculating snap positioning. Returns 0 by default.\n\n",
        name:"getHSnapOrigin",
        seeAlso:[
            {ref:"method:Canvas.getHSnapPosition", description:null}
        ],
        params:[
            {description:"the child that is being snapped", optional:true, name:"snapChild",
             defaultValue:null, type:"Canvas"}
        ],
        returns:{description:"The offset to use when snapping", type:"integer"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:TimeItem.mask":{
        ref:"attr:TimeItem.mask",
        flags:"IRWA",
        description:"Internal-use only for a TimeItem.",
        name:"mask",
        setter:["setMask"],
        seeAlso:[
            {ref:"attr:TimeItem.useMask", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TimeItem"
    },
    "method:CubeGrid.getFacet":{
        ref:"method:CubeGrid.getFacet",
        flags:"",
        description:" Get a facet definition by facetId.  Constant time.\n\n",
        name:"getFacet",
        seeAlso:[
            {ref:"class:Facet", description:null}
        ],
        params:[
            {description:"the id of the facet to retrieve", optional:false, name:"facetId",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the Facet if found, or null", type:"Facet"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "method:StatefulCanvas.setIconOrientation":{
        ref:"method:StatefulCanvas.setIconOrientation",
        flags:"",
        description:" Changes the orientation of the icon relative to the text of the button.\n\n",
        name:"setIconOrientation",
        seeAlso:[],
        params:[
            {description:"The new orientation of the icon relative to the text of the button.",
             optional:false, name:"orientation", defaultValue:null, type:"\"left\" or \"right\""}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "method:SectionStack.reorderSection":{
        ref:"method:SectionStack.reorderSection",
        deprecated:"As of SmartClient version 5.5, use ${isc.DocUtils.linkForRef('method:SectionStack.moveSection')}.",
        flags:"",
        description:"\n Reorder the sections by shifting the specified section to a new position\n\n",
        name:"reorderSection",
        seeAlso:[],
        params:[
            {description:"Section to move.  You can pass the position                       of the section in the SectionStack or the ID of the section.",
             optional:false, name:"section", defaultValue:null, type:"position|sectionId"},
            {description:"new position index for the section.", optional:false, name:"position",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:SelectItem.showOptionsFromDataSource":{
        ref:"attr:SelectItem.showOptionsFromDataSource",
        flags:"IRWA",
        description:"If this item is part of a databound form, and has a specified <code>valueMap</code>,\n by default we show the valueMap options in the pickList for the item.\n Setting this property to true will ensure that the options displayed in our pickList\n are derived from the form's <code>dataSource</code>.",
        name:"showOptionsFromDataSource",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["databinding"]
    },
    "attr:Splitbar.target":{
        ref:"attr:Splitbar.target",
        flags:"R",
        description:"When a <code>Splitbar</code> is created by a layout, the <code>target</code> property\n of the Splitbar will be a pointer to the member for which it is acting as a resizeBar.\n The Splitbar will be positioned next to its target, and will resize it on drag completion.",
        name:"target",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar"
    },
    "attr:TileGrid.dataSource":{
        ref:"attr:TileGrid.dataSource",
        flags:"IRW",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        definingClass:"class:TileGrid",
        groups:["databinding"]
    },
    "attr:Canvas.edgeShowCenter":{
        ref:"attr:Canvas.edgeShowCenter",
        flags:"IR",
        description:"Whether to show media in the center section, that is, behind the decorated Canvas.",
        name:"edgeShowCenter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('corners')}"],
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "attr:Toolbar.canAcceptDrop":{
        ref:"attr:Toolbar.canAcceptDrop",
        flags:"IRWA",
        description:"If true, items (buttons) can be dropped into this toolbar, and the toolbar will\n show a drop line at the drop location.  Override drop() to decide what happens when the\n item is dropped.",
        name:"canAcceptDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["dragndrop"]
    },
    "method:DataBoundComponent.getFieldNum":{
        ref:"method:DataBoundComponent.getFieldNum",
        flags:"",
        description:" Find the index of a currently visible field.\n\n",
        name:"getFieldNum",
        seeAlso:[],
        params:[
            {description:"field name or field", optional:false, name:"fieldID", defaultValue:null,
             type:"String || Field"}
        ],
        returns:{description:"index of field within currently visible fields", type:"int"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:ListGrid.dragTrackerMode":{
        ref:"attr:ListGrid.dragTrackerMode",
        flags:"IRA",
        description:"When records are being dragged from within a ListGrid, what sort of drag-tracker\n should be displayed?<br>\n Note that if multiple records are being dragged the displayed tracker will be\n based on the first selected record.",
        name:"dragTrackerMode",
        seeAlso:[],
        valueType:"DragTrackerMode",
        defaultValue:"\"icon\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["dragTracker"]
    },
    "attr:ListGridField.canSort":{
        ref:"attr:ListGridField.canSort",
        flags:"IRW",
        description:"Enables or disables sorting by this column. If false, neither interactive\n  nor scripted (via the sort() method) instructions will sort the listGrid by this\n  column.",
        name:"canSort",
        seeAlso:[
            {ref:"method:ListGrid.sort", description:null},
            {ref:"attr:ListGrid.canSort", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["sorting"]
    },
    "method:List.find":{
        ref:"method:List.find",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.findIndex')}, but returns the object itself instead of its index.\n\n",
        name:"find",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"first matching object or null if not found", type:"Object"},
        type:"method",
        definingClass:"interface:List",
        groups:["access", "find"]
    },
    "attr:TreeGrid.separateFolders":{
        ref:"attr:TreeGrid.separateFolders",
        flags:"IR",
        description:"Specifies whether folders and leaves should be segregated in the treeGrid display.\n          With separateFolders:true and sortDirection:\"descending\", folders are displayed\n          before their sibling leaves; with sortDirection:\"ascending\", leaves are displayed\n          before their sibling folders.",
        name:"separateFolders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeField"]
    },
    "method:ColumnTree.setData":{
        ref:"method:ColumnTree.setData",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('class:Tree')} object this ColumnTree will view\n\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"Tree to show", optional:false, name:"newData", defaultValue:null, type:"Tree"}
        ],
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "method:StretchImgButton.setBaseStyle":{
        ref:"method:StretchImgButton.setBaseStyle",
        flags:"",
        description:" Sets the base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n",
        name:"setBaseStyle",
        seeAlso:[],
        params:[
            {description:"new base style", optional:false, name:"style", defaultValue:null,
             type:"className"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton"
    },
    "attr:ComboBoxItem.valueField":{
        ref:"attr:ComboBoxItem.valueField",
        flags:"IRW",
        getter:["FormItem.getValueFieldName"],
        description:"If this form item maps data values to display values by retrieving the \n ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} values from an \n ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}, this property \n denotes the the field to use as the underlying data value in records from the \n optionDataSource.<br>\n If unset, assumed to be the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this form item.",
        name:"valueField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem",
        groups:["display_values"]
    },
    "attr:Img.name":{
        ref:"attr:Img.name",
        flags:"IA",
        description:"The value of this attribute is specified as the value of the 'name' attribute in the\n resulting HTML.\n <p>\n Note: this attribute is ignored if the imageType is set to \"tile\"",
        name:"name",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"main\"",
        type:"attr",
        definingClass:"class:Img"
    },
    "attr:ListGrid.linkTextProperty":{
        ref:"attr:ListGrid.linkTextProperty",
        flags:"IRW",
        description:"Property name on a record that will hold the link text for that record.\n <br>\n This property is configurable to avoid possible collision with data values in the record.",
        name:"linkTextProperty",
        seeAlso:[
            {ref:"type:ListGridFieldType", description:null},
            {ref:"type:FieldType", description:null},
            {ref:"attr:ListGridField.linkText", description:null}
        ],
        valueType:"string",
        defaultValue:"\"linkText\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["display_values"]
    },
    "method:ListGrid.getSortState":{
        ref:"method:ListGrid.getSortState",
        flags:"",
        description:" Returns a snapshot of the current sort state (sort field and order) within this listGrid as \n a ${isc.DocUtils.linkForRef('type:listGridSortState')} object.<br>\n This object can be passed to ${isc.DocUtils.linkForRef('method:ListGrid.setSortState')} to reset this grid's sort to\n the current state (assuming the same fields are present in the grid).<br>\n",
        name:"getSortState",
        seeAlso:[
            {ref:"method:ListGrid.setSortState", description:null}
        ],
        returns:{description:"current sort state for the grid.", type:"listGridSortState"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:DynamicForm.cancelParamName":{
        ref:"attr:DynamicForm.cancelParamName",
        flags:"IRW",
        description:"The name of the special field sent to the server as part of ${isc.DocUtils.linkForRef('method:DynamicForm.cancel')}",
        name:"cancelParamName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"org.apache.struts.taglib.html.CANCEL\"",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "classMethod:isc.clone":{
        ref:"classMethod:isc.clone",
        flags:"",
        description:" Create a deep clone of an object that can be edited without affecting the original\n <br>\n All mutable types, including Objects, Arrays and Dates, are copied.  All immutable types are\n just preserved by reference.\n <br>\n Does not handle looping references (will infinite loop).\n\n",
        name:"clone",
        seeAlso:[],
        params:[
            {description:"object to clone", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"cloned object", type:"object"},
        type:"classMethod",
        definingClass:"class:isc",
        groups:["serialization"]
    },
    "attr:Canvas.customEdges":{
        ref:"attr:Canvas.customEdges",
        flags:"IR",
        description:"Array of side names (\"T\", \"B\", \"L\", \"R\") specifying which sides of the decorated\n component should show edges.  For example: <pre>\n      customEdges : [\"T\", \"B\"]\n </pre>\n .. would show edges only on the top and bottom of a component.\n <P>\n The default of <code>null</code> means edges will be shown on all sides.",
        name:"customEdges",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "classMethod:Class.getDefaultLogPriority":{
        ref:"classMethod:Class.getDefaultLogPriority",
        flags:"",
        description:" Retrieves the default priority of messages for this class or instance.\n",
        name:"getDefaultLogPriority",
        seeAlso:[],
        returns:{description:"default priority for logging messages on this object.", type:"LogPriority"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:FilterBuilder.allowEmpty":{
        ref:"attr:FilterBuilder.allowEmpty",
        flags:"IR",
        description:"If set to false, the last clause cannot be removed.",
        name:"allowEmpty",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "class:DataSource":{
        methods:["method:DataSource.getLegalChildTags", "method:DataSource.transformRequest", 
         "method:DataSource.getUpdatedData","method:DataSource.getDataProtocol", "method:DataSource.processResponse", 
         "method:DataSource.getXMLRequestBody","method:DataSource.xmlSerialize", "method:DataSource.recordsFromXML", 
         "method:DataSource.transformResponse","method:DataSource.fetchRecord", "method:DataSource.fetchData", 
         "method:DataSource.filterData","method:DataSource.exportData", "method:DataSource.getClientOnlyDataSource", 
         "method:DataSource.addData","method:DataSource.updateData", "method:DataSource.removeData", 
         "method:DataSource.performCustomOperation","method:DataSource.downloadFile", "method:DataSource.viewFile", 
         "method:DataSource.handleError","method:DataSource.getField", "method:DataSource.getFieldNames", 
         "method:DataSource.getPrimaryKeyField","method:DataSource.getPrimaryKeyFieldName", "method:DataSource.getAutoTitle", 
         "method:DataSource.getClientOnlyResponse","method:DataSource.applyFilter", "method:DataSource.fieldMatchesFilter", 
         "method:DataSource.supportsTextMatchStyle","method:DataSource.compareCriteria", "method:DataSource.addSearchOperator", 
         "method:DataSource.getSearchOperator","method:DataSource.getTypeOperators", "method:DataSource.setTypeOperators", 
         "method:DataSource.getFieldOperators","method:DataSource.getFieldOperatorMap", "method:DataSource.getTypeOperatorMap", 
         "method:DataSource.evaluateCriterion"],
        ref:"class:DataSource",
        classAttrs:["classAttr:DataSource.serializeTimeAsDatetime"],
        classMethods:["classMethod:DataSource.getDataSource", "classMethod:DataSource.get", 
         "classMethod:DataSource.load","classMethod:DataSource.getSortBy", "classMethod:DataSource.getSortSpecifiers", 
         "classMethod:DataSource.addSearchOperator","classMethod:DataSource.setTypeOperators", "classMethod:DataSource.getAutoTitle", 
         "classMethod:DataSource.convertCriteria","classMethod:DataSource.combineCriteria", 
         "classMethod:DataSource.applyRecordSummaryFunction","classMethod:DataSource.registerRecordSummaryFunction"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:DataSource.ID", "attr:DataSource.addGlobalId", "attr:DataSource.dataFormat", 
         "attr:DataSource.dataProtocol","attr:DataSource.useHttpProxy", "attr:DataSource.callbackParam", 
         "attr:DataSource.requestProperties","attr:DataSource.dataTransport", "attr:DataSource.preventHTTPCaching", 
         "attr:DataSource.serverType","attr:DataSource.tableName", "attr:DataSource.dbName", "attr:DataSource.quoteColumnNames", 
         "attr:DataSource.jsonPrefix","attr:DataSource.jsonSuffix", "attr:DataSource.fields", "attr:DataSource.inheritsFrom", 
         "attr:DataSource.useFlatFields","attr:DataSource.showLocalFieldsOnly", "attr:DataSource.useParentFieldOrder", 
         "attr:DataSource.dropExtraFields","attr:DataSource.sendExtraFields", "attr:DataSource.autoDeriveSchema", 
         "attr:DataSource.serviceNamespace","attr:DataSource.schemaNamespace", "attr:DataSource.recordXPath", 
         "attr:DataSource.dataURL","attr:DataSource.tagName", "attr:DataSource.clientOnly", "attr:DataSource.testData", 
         "attr:DataSource.criteriaPolicy","attr:DataSource.resultTreeClass", "attr:DataSource.resultSetClass", 
         "attr:DataSource.useLocalValidators","attr:DataSource.showPrompt", "attr:DataSource.enumTranslateStrategy", 
         "attr:DataSource.enumConstantProperty","attr:DataSource.enumOrdinalProperty", "attr:DataSource.autoDeriveTitles", 
         "attr:DataSource.canMultiSort","attr:DataSource.requiresAuthentication", "attr:DataSource.requiresRole", 
         "attr:DataSource.requires","attr:DataSource.title", "attr:DataSource.pluralTitle", "attr:DataSource.titleField", 
         "attr:DataSource.iconField","attr:DataSource.childrenField", "attr:DataSource.serverConstructor", 
         "attr:DataSource.qualifyColumnNames","attr:DataSource.validateRelatedRecords", "attr:DataSource.beanClassName", 
         "attr:DataSource.globalNamespaces","attr:DataSource.resultBatchSize", "attr:DataSource.serverObject", 
         "attr:DataSource.operationBindings","attr:DataSource.strictSQLFiltering"],
        description:"A DataSource is data-provider-independant description of a set of objects\n that will be loaded, edited and saved within the user interface of your application.\n <P>\n Each DataSource consists of a list of ${isc.DocUtils.linkForRef('attr:DataSource.fields','fields')} that make up a\n DataSource <code>record</code>, along with ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field types')}, \n ${isc.DocUtils.linkForRef('attr:DataSourceField.validators','validation rules')}, \n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','relationships')} to other DataSources, and other\n metadata.\n <P>\n The abstract object description provided by a DataSource is easily mapped to a variety of\n backend object models and storage schemes.  The following table shows analogous terminology\n across systems.\n <table border=1 class=\"normal\">\n <tr>\n   <td>Isomorphic SmartClient</td>\n   <td>Relational Database</td>\n   <td>Enterprise Java Beans (EJB)</td>\n   <td>Entity/Relationship Modelling</td>\n   <td>OO/UML</td>\n   <td>XML Schema/WSDL</td>\n   <td>LDAP</td>\n </tr><tr>\n   <td>DataSource</td>\n   <td>Table</td>\n   <td>EJB class</td>\n   <td>Entity</td>\n   <td>Class</td>\n   <td>Element Schema (ComplexType)</td>\n   <td>Objectclass</td>\n </tr><tr>\n   <td>Record</td>\n   <td>Row</td>\n   <td>EJB instance</td>\n   <td>Entity instance</td>\n   <td>Class instance/Object</td>\n   <td>Element instance (ComplexType)</td>\n   <td>Entry</td>\n </tr><tr>\n   <td>Field</td>\n   <td>Column</td>\n   <td>Property</td>\n   <td>Attribute</td>\n   <td>Property/Attribute</td>\n   <td>Attribute or Element (SimpleType)</td>\n   <td>Attribute</td>\n </tr></table>\n <P>\n DataSources can be ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','declared')} in either JavaScript or XML\n format, and can also be ${isc.DocUtils.linkForRef('group:metadataImport','imported')} from existing metadata formats,\n including XML Schema.\n <P>\n <i>Data Binding</i> is the process by which\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','Data Binding-capable UI components')} can automatically configure\n themselves for viewing, editing and saving data described by DataSources.  DataBinding is\n covered in the ${isc.DocUtils.linkForDocNode('QuickStartGuide', 'QuickStart Guide')}, Chapter 6, <i>Data\n Binding</i>.\n <P>\n ${isc.DocUtils.linkForRef('group:clientServerIntegration','Data Integration')} is the process by which a DataSource\n can be connected to server systems such as SQL DataBases, Java Object models, WSDL web\n services and other data providers.  Data Integration comes in two variants: client-side and\n server-side.  ${isc.DocUtils.linkForRef('group:serverDataIntegration','Server-side integration')} uses the\n SmartClient Java-based server to connect to data represented by Java Objects or\n JDBC-accessible databases.  ${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-side integration')}\n connects SmartClient DataSources to XML, JSON or other formats accessible via HTTP.\n <P>\n DataSources have a concept of ${isc.DocUtils.linkForRef('group:dataSourceOperations','4 core operations')} (\"fetch\",\n \"add\", \"update\" and \"remove\") that can be performed on the set of objects represented by a\n DataSource.  Once a DataSource has been integrated with your data store, databinding-capable \n UI components can leverage the 4 core DataSource operations to provide many complete user\n interactions without the need to configure how each individual component loads and saves\n data.\n <P>\n These interactions include ${isc.DocUtils.linkForRef('class:ListGrid','grid views')}, ${isc.DocUtils.linkForRef('class:TreeGrid','tree views')}, \n ${isc.DocUtils.linkForRef('class:DetailViewer','detail views')}, ${isc.DocUtils.linkForRef('class:DynamicForm','form')}-based \n ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord','editing')} and ${isc.DocUtils.linkForRef('method:DynamicForm.saveData','saving')},\n grid-based ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit','editing')} and ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell','saving')},\n and custom interactions provided by ${isc.DocUtils.linkForExampleId('patternReuse')} custom\n databinding-capable components.",
        name:"DataSource",
        seeAlso:[
            {ref:"interface:DataBoundComponent",
             description:"for information on DataBound Components"},
            {ref:"group:dataSourceDeclaration", description:"for how to create DataSources"}
        ],
        type:"class",
        serverDS:"allowed"
    },
    "attr:SummaryBuilder.helpIcon":{
        ref:"attr:SummaryBuilder.helpIcon",
        flags:"IRA",
        description:"Icon that appears after the ${isc.DocUtils.linkForRef('attr:SummaryBuilder.formulaField')}, showing help on hover.",
        name:"helpIcon",
        seeAlso:[],
        valueType:"AutoChild FormItemIcon",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "class:Operators":{
        methods:[],
        ref:"class:Operators",
        classAttrs:["classAttr:Operators.equalsTitle", "classAttr:Operators.notEqualTitle", 
         "classAttr:Operators.greaterThanTitle","classAttr:Operators.lessThanTitle", "classAttr:Operators.greaterOrEqualTitle", 
         "classAttr:Operators.lessOrEqualTitle","classAttr:Operators.betweenTitle", "classAttr:Operators.betweenInclusiveTitle", 
         "classAttr:Operators.iContainsTitle","classAttr:Operators.iStartsWithTitle", "classAttr:Operators.iEndsWithTitle", 
         "classAttr:Operators.containsTitle","classAttr:Operators.startsWithTitle", "classAttr:Operators.endsWithTitle", 
         "classAttr:Operators.iNotContainsTitle","classAttr:Operators.iNotStartsWithTitle", "classAttr:Operators.iNotEndsWithTitle", 
         "classAttr:Operators.notContainsTitle","classAttr:Operators.notStartsWithTitle", "classAttr:Operators.notEndsWithTitle", 
         "classAttr:Operators.isNullTitle","classAttr:Operators.notNullTitle", "classAttr:Operators.regexpTitle", 
         "classAttr:Operators.iregexpTitle","classAttr:Operators.inSetTitle", "classAttr:Operators.notInSetTitle", 
         "classAttr:Operators.equalsFieldTitle","classAttr:Operators.notEqualFieldTitle", "classAttr:Operators.andTitle", 
         "classAttr:Operators.notTitle","classAttr:Operators.orTitle"],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:[],
        description:"The Operators class provides a collection of class-level properties that can be used to\n modify the descriptions associated with the logical ${isc.DocUtils.linkForRef('object:Operator')}s used in \n ${isc.DocUtils.linkForRef('object:AdvancedCriteria')}. This is primarily intended for internationalization.\n <p>\n To add or override an operator's description, use\n Operators.${isc.DocUtils.linkForRef('classMethod:Class.addClassProperties','addClassProperties()')}.  For example:\n <p>\n <code>Operators.addClassProperties({lessOrEqualTitle: \"Less than or equal to\"});</code>",
        name:"Operators",
        seeAlso:[],
        type:"class"
    },
    "class:MultiSortPanel":{
        methods:["method:MultiSortPanel.getNumLevels", "method:MultiSortPanel.getSortLevel", 
         "method:MultiSortPanel.getSort","method:MultiSortPanel.validate", "method:MultiSortPanel.sortChanged"],
        ref:"class:MultiSortPanel",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:MultiSortPanel.fields", "attr:MultiSortPanel.addLevelButtonTitle", 
         "attr:MultiSortPanel.deleteLevelButtonTitle","attr:MultiSortPanel.copyLevelButtonTitle", "attr:MultiSortPanel.invalidListPrompt", 
         "attr:MultiSortPanel.propertyFieldTitle","attr:MultiSortPanel.directionFieldTitle", "attr:MultiSortPanel.ascendingTitle", 
         "attr:MultiSortPanel.descendingTitle","attr:MultiSortPanel.firstSortLevelTitle", "attr:MultiSortPanel.otherSortLevelTitle", 
         "attr:MultiSortPanel.addLevelButton","attr:MultiSortPanel.deleteLevelButton", "attr:MultiSortPanel.copyLevelButton", 
         "attr:MultiSortPanel.levelUpButton","attr:MultiSortPanel.levelDownButton", "attr:MultiSortPanel.optionsGrid", 
         "attr:MultiSortPanel.initialSort","attr:MultiSortPanel.maxLevels"],
        description:"A widget that allows the user to set up complex sorting arrangments by defining a group of\n ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n <P>\n Each ${isc.DocUtils.linkForRef('object:SortSpecifier')} applies to a single property and direction - so, for instance, in \n a grid with two columns, <code>year</code> and <code>monthNumber</code>, you could sort first \n by <code>year</code> in descending order and then by <code>monthNumber</code> in ascending \n order.  This would producing a grid sorted by year from largest (most \n recent) to smallest (least recent) and, within each year, by monthNumber from smallest \n (January) to largest (December).",
        name:"MultiSortPanel",
        seeAlso:[],
        type:"class"
    },
    "attr:Slider.maxValueLabel":{
        ref:"attr:Slider.maxValueLabel",
        flags:"IRW",
        description:"The text displayed in the label for the maximum value of the slider. If left as null, then\n slider.maxValue will be displayed.",
        name:"maxValueLabel",
        seeAlso:[
            {ref:"attr:Slider.showRange", description:null},
            {ref:"attr:Slider.maxValue", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:MultiSortDialog.copyLevelButton":{
        ref:"attr:MultiSortDialog.copyLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for duplicating levels\n in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.copyLevelButtonProperties</code> and \n <code>multiSortPanel.copyLevelButtonDefaults</code>.",
        name:"copyLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "attr:ComboBoxItem.pickerIconHeight":{
        ref:"attr:ComboBoxItem.pickerIconHeight",
        flags:"IRW",
        description:"Don't specify an explicit height for the picker icon - instead have it size to match the\n height of the combo box item.",
        name:"pickerIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "type:Properties":{
        ref:"type:Properties",
        description:"When the type for a parameter mentions \"properties\" as in \"ListGrid Properties\" or\n \"RPCRequest Properties\", it means that the expected value is a JavaScript Object\n containing any set of properties generally legal when creating an object of that type.\n <P>\n For example, the first parameter of ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendRequest')} is of type\n \"RPCRequest Properties\".  This means it should be called like:\n <pre>\n    isc.RPCManager.sendRequest({\n        actionURL : \"/foo.do\",\n        showPrompt:false\n    });</pre>\n ${isc.DocUtils.linkForRef('attr:RPCRequest.actionURL','actionURL')} and ${isc.DocUtils.linkForRef('attr:RPCRequest.showPrompt','showPrompt')} are \n properties of ${isc.DocUtils.linkForRef('class:RPCRequest')}.  \n <P>\n Note that the notation shown above is an example of a\n ${isc.DocUtils.linkForRef('type:ObjectLiteral','JavaScript object literal')}.",
        name:"Properties",
        seeAlso:[],
        type:"type"
    },
    "group:buttons":{
        ref:"group:buttons",
        refs:["method:Window.closeClick", "method:Dialog.saveData", "method:Dialog.closeClick", 
         "method:Dialog.cancelClick","method:Dialog.okClick", "method:Dialog.applyClick", "method:Dialog.yesClick", 
         "method:Dialog.noClick","method:Dialog.doneClick", "method:Toolbar.getButton"],
        name:"buttons",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Log.getCallTrace":{
        ref:"classMethod:Log.getCallTrace",
        flags:"",
        description:" Returns a one-line summary of the current method call, showing method name and passed\n arguments. \n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getCallTrace()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"getCallTrace",
        seeAlso:[],
        params:[
            {description:"arguments object from the call to trace.  On IE, defaults to the                             calling function's arguments",
             optional:true, name:"args", defaultValue:null, type:"Arguments"}
        ],
        type:"classMethod",
        platformNotes:"IE only.",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "method:Canvas.setTop":{
        ref:"method:Canvas.setTop",
        flags:"",
        description:" Set the top coordinate of this object, relative to its enclosing context, in pixels.\n <P>\n NOTE: if you're setting multiple coordinates, use setRect() or moveTo() instead\n\n",
        name:"setTop",
        seeAlso:[],
        params:[
            {description:"new top coordinate", optional:false, name:"top", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:ListGrid.setSort":{
        ref:"method:ListGrid.setSort",
        flags:"A",
        description:"  This method sorts the grid on one or more fields.  Pass in an array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s\n to have the grid's data sorted by the fields in each specifier.property and in \n the directions specified.  The grid can be sorted by any combinatioon of fields, including\n fields specified in the fields array, whether visible or hidden, and \n ${isc.DocUtils.linkForRef('attr:DataSource.fields','unused fields from the underlying dataSource')}, if there is one.  If\n multiple fields are sorted, those that are visible show a directional icon and a small \n ${isc.DocUtils.linkForRef('attr:ListGrid.sortNumeralStyle','sort-numeral')} \n indicating that field's index in the sort configuration.\n <P>See ${isc.DocUtils.linkForRef('method:ListGrid.addSort','addSort()')} and ${isc.DocUtils.linkForRef('method:ListGrid.toggleSort','toggleSort()')} APIs\n for information on making changes to the current sort configuration.\n",
        name:"setSort",
        seeAlso:[],
        params:[
            {description:"Array of ${isc.DocUtils.linkForRef('object:SortSpecifier')} objects",
             optional:false, name:"sortSpecifiers", defaultValue:null, type:"Array of SortSpecifier"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "class:SubmitItem":{
        methods:[],
        ref:"class:SubmitItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:SubmitItem.title"],
        description:"Button that saves the data in the form, by calling ${isc.DocUtils.linkForRef('method:DynamicForm.submit')} when clicked.\n ${isc.DocUtils.linkForRef('method:DynamicForm.submit')} for details on how to control what happens when a form is\n submitted.",
        name:"SubmitItem",
        seeAlso:[
            {ref:"group:operations", description:null}
        ],
        type:"class"
    },
    "method:DataBoundComponent.disableHilite":{
        ref:"method:DataBoundComponent.disableHilite",
        flags:"",
        description:" Disable a hilite\n\n",
        name:"disableHilite",
        seeAlso:[],
        params:[
            {description:"ID of hilite to disable", optional:false, name:"hiliteID", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["hiliting"]
    },
    "attr:Layout.vertical":{
        ref:"attr:Layout.vertical",
        flags:"IRW",
        description:"Should this layout appear with members stacked vertically or horizontally. Defaults to \n <code>false</code> if unspecified.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:FormulaBuilder.editMode":{
        ref:"attr:FormulaBuilder.editMode",
        flags:"IR",
        description:"Are we editing an existing field?",
        name:"editMode",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:StretchImgButton.setDisabled":{
        ref:"method:StretchImgButton.setDisabled",
        flags:"",
        description:" Enable or disable this object\n",
        name:"setDisabled",
        seeAlso:[],
        params:[
            {description:"true if this widget is to be disabled", optional:false, name:"disabled",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["enable", "state"]
    },
    "method:DataBoundComponent.transferSelectedData":{
        ref:"method:DataBoundComponent.transferSelectedData",
        flags:"",
        description:" Simulates a drag / drop type transfer of the selected records in some other component to this\n component, without requiring any user interaction.  This method acts on the dropped records \n exactly as if they had been dropped in an actual drag / drop interaction, including any \n special databound behavior invoked by calling\n ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues','getDropValues')} for each dropped record.\n <P>\n To transfer <b>all</b> data in, for example, a ${isc.DocUtils.linkForRef('class:ListGrid')}, call grid.selection.selectAll() first.\n <P>\n Note that drag/drop type transfers of records between components are asynchronous operations:\n SmartClient may need to perform server turnarounds to establish whether dropped records \n already exist in the target component.  Therefore, it is possible to issue a call to \n transferSelectedData() and/or the ${isc.DocUtils.linkForRef('method:ListGrid.drop','drop()')} method of a databound \n component whilst a transfer is still active.  When this happens, SmartClient adds the \n second and subsequent transfer requests to a queue and runs them one after the other.  If \n you want to be notified when a transfer process has actually completed, either provide a \n callback to this method or implement ${isc.DocUtils.linkForRef('method:DataBoundComponent.dropComplete')}.\n <P>\n See the ${isc.DocUtils.linkForRef('group:dragging')} documentation for an overview of list grid drag/drop data\n transfer.\n \n",
        name:"transferSelectedData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"},
            {description:"target index (drop position) of the rows within this grid.", optional:true,
             name:"index", defaultValue:null, type:"integer"},
            {description:"optional callback to be fired when the transfer process has                        completed.  The callback will be passed a single parameter \"records\",\n                       the list of records actually transferred to this component.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}"],
        definingClass:"interface:DataBoundComponent",
        groups:["dragging"]
    },
    "method:Canvas.drop":{
        ref:"method:Canvas.drop",
        flags:"A",
        description:"\n Executed when the mouse button is released over a compatible drop target at the end of\n a drag sequence. Your widget should implement whatever it wants to do when receiving a\n drop here. For example, in a file moving interface, a drop might mean that you should\n move or copy the dragged file into the folder it was dropped on, or dropping something in\n a trash can might mean to clear it from the screen.\n\n",
        name:"drop",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null},
            {ref:"classMethod:EventHandler.getDragTarget", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:ResultSet.findAll":{
        ref:"method:ResultSet.findAll",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.findAll')}.  Checks only loaded rows and will not trigger a fetch.\n",
        name:"findAll",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"all matching Objects or null if none found", type:"Array"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access", "find"]
    },
    "classMethod:SchemaSet.get":{
        ref:"classMethod:SchemaSet.get",
        flags:"A",
        description:" Retrieve a SchemaSet object by it's schemaNamespace.\n\n",
        name:"get",
        seeAlso:[],
        params:[
            {description:"uri from the \"targetNamespace\" attribute of the &lt;xsd:schema&gt; element from the XML Schema or WSDL file this SchemaSet was derived\n from.",
             optional:false, name:"schemaNamespace", defaultValue:null, type:"String"}
        ],
        returns:{description:"the requested SchemaSet, or null if not loaded", type:"SchemaSet"},
        type:"classMethod",
        definingClass:"class:SchemaSet"
    },
    "classMethod:Log.setDefaultLogPriority":{
        ref:"classMethod:Log.setDefaultLogPriority",
        flags:"",
        description:" Set the default priority of logging for messages logged on this Class or Instance object.\n All categories for which there is no explicit, instance level logging priority set will\n log at this level on this object.<br>  \n To set the default visible log priority across the entire page, use\n <code>isc.Log.setDefaultPriority()</code> instead.\n",
        name:"setDefaultLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:StretchImg.hSrc":{
        ref:"attr:StretchImg.hSrc",
        flags:"IRW",
        description:"Base URL for the image if ${isc.DocUtils.linkForRef('attr:StretchImg.vertical')} is false and \n ${isc.DocUtils.linkForRef('attr:StretchImg.src')} is unset.",
        name:"hSrc",
        seeAlso:[
            {ref:"attr:StretchImg.src", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "method:GridRenderer.cellOver":{
        ref:"method:GridRenderer.cellOver",
        flags:"A",
        description:" Called when the mouse pointer enters a cell\n\n",
        name:"cellOver",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:Flashlet.params":{
        ref:"attr:Flashlet.params",
        flags:"IR",
        description:"A map of key/value pairs to pass to the flashlet as parameters.  Note that these will be set\n  on the outer &lt;object&gt; element as well as the inner &lt;embed&gt; element.",
        name:"params",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "method:ComboBoxItem.getDisplayFieldName":{
        ref:"method:ComboBoxItem.getDisplayFieldName",
        flags:"A",
        description:" Returns the ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} for this form item.\n If unset, and ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is explicitly specified, this method\n will return the title field for the <code>optionDataSource</code>\n",
        name:"getDisplayFieldName",
        seeAlso:[],
        returns:{description:"display field name, or null", type:"String"},
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "attr:Calendar.datePickerButton":{
        ref:"attr:Calendar.datePickerButton",
        flags:"IR",
        description:"The datePickerIcon is an ImgButton that appears above the week/day/month views of the\n calendar and offers alternative access to a ${isc.DocUtils.linkForRef('class:DateChooser')} to pick the current day.",
        name:"datePickerButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:Calendar.showDayHeaders":{
        ref:"attr:Calendar.showDayHeaders",
        flags:"IR",
        description:"If false, the month view does not show day headers",
        name:"showDayHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["visibility"]
    },
    "method:CubeGrid.clearEditValue":{
        ref:"method:CubeGrid.clearEditValue",
        flags:"A",
        description:"\n  Clear a field value being tracked as an unsaved user edit for some cell.<P>\n  The saved record value will be displayed in the the appropriate cell instead.\n\n",
        name:"clearEditValue",
        seeAlso:[],
        params:[
            {description:"Row number, primary keys object for the record,                                              or editValues object",
             optional:false, name:"editValuesID", defaultValue:null, type:"number | object"},
            {description:"Column number for the cell in question. Only required if the                              first parameter is a row number.",
             optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:Menu.submenuDisabledImage":{
        ref:"attr:Menu.submenuDisabledImage",
        flags:"IRWA",
        description:"Default image to use for the submenu indicator when item is disabled. Valid src, width and\n height must be specified. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.<br>\n If ${isc.DocUtils.linkForRef('attr:Menu.submenuDirection')} is set to <code>\"left\"</code>, the image src will have\n the suffix <code>\"_left\"</code> appended to it.",
        name:"submenuDisabledImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Menu"
    },
    "group:fields":{
        ref:"group:fields",
        refs:["attr:DataSource.fields", "attr:DataSource.inheritsFrom", "attr:DataSource.useFlatFields", 
         "attr:DataSource.showLocalFieldsOnly","attr:DataSource.useParentFieldOrder", "attr:DataSource.autoDeriveSchema"],
        name:"fields",
        seeAlso:[],
        type:"group"
    },
    "attr:ImgButton.iconOrientation":{
        ref:"attr:ImgButton.iconOrientation",
        flags:"IR",
        description:"If this button is showing an icon should it appear to the left or right of the title?\n valid options are <code>\"left\"</code> and <code>\"right\"</code>.",
        name:"iconOrientation",
        setter:["setIconOrientation"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "method:CubeGrid.facetHasSelection":{
        ref:"method:CubeGrid.facetHasSelection",
        flags:"",
        description:"\n return whether any facet value for this facet is selected in headers.  If no facetId passed,\n return whether any facet has a selection\n\n  <br><i>methodType</i> tester\n",
        name:"facetHasSelection",
        seeAlso:[],
        params:[
            {description:"Id for facet to test", optional:false, name:"facetId", defaultValue:null,
             type:"string"}
        ],
        returns:{description:"true if any facet value in this header is selected", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:Toolbar.getButton":{
        ref:"method:Toolbar.getButton",
        flags:"",
        description:"          Retrieves a button widget instance (within this toolbar) from the ID / index / \n          descriptor object for the button (as with the getButtonNumber() method)\n          This provides a way to access a toolbar button's properties and methods directly.\n",
        name:"getButton",
        seeAlso:[
            {ref:"method:Toolbar.getButtonNumber", description:null}
        ],
        params:[
            {description:"identifier for the button to retrieve", optional:false, name:"index",
             defaultValue:null, type:"number | string | object"}
        ],
        returns:{description:"the button, or null if the button wasn't found", type:"Button"},
        type:"method",
        definingClass:"class:Toolbar",
        groups:["buttons"]
    },
    "method:ListGrid.setViewState":{
        ref:"method:ListGrid.setViewState",
        flags:"",
        description:" Reset this grid's view state to match the ${isc.DocUtils.linkForRef('type:listGridViewState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to \n ${isc.DocUtils.linkForRef('method:ListGrid.getViewState')}.\n\n",
        name:"setViewState",
        seeAlso:[
            {ref:"method:ListGrid.getViewState", description:null}
        ],
        params:[
            {description:"Object describing the desired view state for the grid", optional:false,
             name:"viewState", defaultValue:null, type:"listGridViewState"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:ListGrid.autoFitExtraRecords":{
        ref:"attr:ListGrid.autoFitExtraRecords",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')} is set to <code>\"vertical\"</code> or <code>\"both\"</code> \n this property specifies the number of additional records for which the grid \n will expeand. If more\n records are present, scrolling will be introduced to reach them as normal.",
        name:"autoFitExtraRecords",
        setter:["setAutoFitExtraRecords"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "attr:ListGridField.valueIconOrientation":{
        ref:"attr:ListGridField.valueIconOrientation",
        flags:"IRW",
        description:"If we're showing a valueIcon for this field should it appear to the left or the right\n of the text?  By default the icon will appear to the left of the textual value - \n set this to \"right\" to show the icon on the right of the text.\n Has no effect if ${isc.DocUtils.linkForRef('attr:ListGridField.showValueIconOnly')} is true",
        name:"valueIconOrientation",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "method:Canvas.isEnabled":{
        ref:"method:Canvas.isEnabled",
        deprecated:"As of SmartClient version 5.5 deprecated in favor of ${isc.DocUtils.linkForRef('method:Canvas.isDisabled')}.",
        flags:"",
        description:" Returns true if the widget and all widgets above it in the containment hierarchy are enabled.\n Returns false otherwise.\n",
        name:"isEnabled",
        seeAlso:[],
        returns:{description:"true if the widget and all widgets above it in the containment hierarchy                      are enabled; false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "classAttr:RPCManager.timeoutErrorMessage":{
        ref:"classAttr:RPCManager.timeoutErrorMessage",
        flags:"IRW",
        description:"Default message displayed to user when an opration fails to return from the server within\n the timeout period specified by ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTimeout')}.",
        name:"timeoutErrorMessage",
        seeAlso:[
            {ref:"classAttr:RPCManager.defaultTimeout", description:null}
        ],
        valueType:"string",
        defaultValue:"\"Operation timed out\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["i18nMessages"]
    },
    "attr:SectionHeader.controlsLayout":{
        ref:"attr:SectionHeader.controlsLayout",
        flags:"IR",
        description:"A ${isc.DocUtils.linkForRef('class:Layout')} containing specified ${isc.DocUtils.linkForRef('attr:SectionHeader.controls')} if any.  Sets\n ${isc.DocUtils.linkForRef('attr:Layout.membersMargin')}:5, ${isc.DocUtils.linkForRef('attr:Layout.defaultLayoutAlign')}:\"center\", and\n RTL-sensitive ${isc.DocUtils.linkForRef('attr:Layout.align')} (right by default).",
        name:"controlsLayout",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SectionHeader"
    },
    "attr:Canvas.backgroundColor":{
        ref:"attr:Canvas.backgroundColor",
        flags:"IRW",
        description:"The background color for this widget. It corresponds to the CSS background-color\n attribute. You can set this property to an RGB value (e.g. #22AAFF) or a named color\n (e.g. red) from a list of browser supported color names.",
        name:"backgroundColor",
        setter:["setBackgroundColor"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "attr:ListGridField.valueMap":{
        ref:"attr:ListGridField.valueMap",
        flags:"IRW",
        description:"Array of legal values for this field, or an Object where each property maps a stored\n value to a user-displayable value.<br>\n Note that if this field is editable (see ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit')}, \n ${isc.DocUtils.linkForRef('attr:ListGridField.canEdit')}), editors displayed for this field will pick up their\n valueMap either from this value or from ${isc.DocUtils.linkForRef('attr:ListGridField.editorValueMap')}.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap')}.",
        name:"valueMap",
        seeAlso:[
            {ref:"method:ListGrid.setValueMap", description:null},
            {ref:"method:ListGrid.getDisplayValue", description:null}
        ],
        valueType:"Array of Object",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('listType')}"],
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "group:event handling":{
        ref:"group:event handling",
        refs:["method:TreeGrid.willAcceptDrop", "method:StatefulCanvas.getActionType", 
         "method:StatefulCanvas.setActionType","method:StatefulCanvas.addToRadioGroup", "method:StatefulCanvas.removeFromRadioGroup", 
         "method:Button.addToRadioGroup","method:Button.removeFromRadioGroup", "method:Button.setActionType", 
         "method:Button.getActionType","method:Toolbar.itemClick", "method:ImgButton.addToRadioGroup", 
         "method:ImgButton.removeFromRadioGroup","method:ImgButton.setActionType", "method:ImgButton.getActionType", 
         "method:StretchImgButton.addToRadioGroup","method:StretchImgButton.removeFromRadioGroup", "method:StretchImgButton.setActionType", 
         "method:StretchImgButton.getActionType","attr:StatefulCanvas.actionType", "attr:StatefulCanvas.radioGroup", 
         "attr:Button.radioGroup","attr:Button.actionType", "attr:ImgButton.radioGroup", "attr:ImgButton.actionType", 
         "attr:StretchImgButton.radioGroup","attr:StretchImgButton.actionType", "type:SelectionType"],
        name:"event handling",
        seeAlso:[],
        type:"group"
    },
    "attr:SelectItem.addUnknownValues":{
        ref:"attr:SelectItem.addUnknownValues",
        flags:"IRWA",
        description:"If we're setting the value of a select item and the value isn't a legal value in the\n valueMap, whether we should allow the value (creating a new option for it) or reject it.\n <P>\n Exception: If the value is set to <code>null</code> but there is no null entry in the\n valueMap for this item, setting <code>addUnknownValues</code> to true will not cause\n a null option to show up at the top of the select item pickList. Whether an empty \n option is shown in the pickList is governed by ${isc.DocUtils.linkForRef('attr:SelectItem.allowEmptyValue')}\n instead.",
        name:"addUnknownValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formDependentSelects')}"],
        definingClass:"class:SelectItem"
    },
    "attr:DataBoundComponent.addSummaryFieldText":{
        ref:"attr:DataBoundComponent.addSummaryFieldText",
        flags:"IRW",
        description:"Text for a menu item allowing users to add a formula field",
        name:"addSummaryFieldText",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Add summary column...\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["i18nMessages"]
    },
    "class:Splitbar":{
        methods:[],
        ref:"class:Splitbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:Splitbar.target", "attr:Splitbar.vertical", "attr:Splitbar.src", 
         "attr:Splitbar.hSrc","attr:Splitbar.vSrc", "attr:Splitbar.capSize", "attr:Splitbar.skinImgDir", 
         "attr:Splitbar.showGrip","attr:Splitbar.gripImgSuffix", "attr:Splitbar.showDownGrip", 
         "attr:Splitbar.showRollOverGrip","attr:Splitbar.showClosedGrip", "attr:Splitbar.canDrag", "attr:Splitbar.canCollapse", 
         "attr:Splitbar.cursor","attr:Splitbar.vResizeCursor", "attr:Splitbar.hResizeCursor"],
        description:"Subclass of the ${isc.DocUtils.linkForRef('class:StretchImg')} class. As with the ${isc.DocUtils.linkForRef('class:ImgSplitbar')} class, \n  widgets of this class can be displayed as a resize-bar for widgets \n  in Layouts where showResizeBar is set to true. Provides a different appearance from\n  the <code>ImgSplitbar</code> class.<br>\n  To specify the resizeBar class for some layout, use the ${isc.DocUtils.linkForRef('attr:Layout.resizeBarClass')}\n  property.",
        name:"Splitbar",
        seeAlso:[
            {ref:"class:Layout", description:null},
            {ref:"class:ImgSplitbar", description:null}
        ],
        type:"class"
    },
    "attr:LoginDialog.showRegistrationLink":{
        ref:"attr:LoginDialog.showRegistrationLink",
        flags:"IR",
        description:"If true, display a ${isc.DocUtils.linkForRef('class:LinkItem')} (${isc.DocUtils.linkForRef('attr:LoginDialog.registrationItem')})\n meant for the user to click if the user wishes to reigster a new account.\n The text of the link is controlled by\n ${isc.DocUtils.linkForRef('attr:LoginDialog.registrationItemTitle')}. If clicked, the link will fire\n ${isc.DocUtils.linkForRef('method:LoginDialog.register')}.",
        name:"showRegistrationLink",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:CubeGrid.getCellFacetValues":{
        ref:"method:CubeGrid.getCellFacetValues",
        flags:"",
        description:" Given a cell coordinate within this CubeGrid return a ${isc.DocUtils.linkForRef('type:FacetValueMap','FacetValueMap')}\n indicating the facet values for the cell.\n",
        name:"getCellFacetValues",
        seeAlso:[],
        params:[
            {description:"row index of the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"row index of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"facet values for the specified cell. Returns null if the specified cell    is not present in the grid.",
         type:"FacetValueMap"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "method:Button.setState":{
        ref:"method:Button.setState",
        flags:"",
        description:" Set the 'state' of this object, this changes it's appearance.\n\n",
        name:"setState",
        seeAlso:[
            {ref:"method:Button.setDisabled", description:"which also affects state values."}
        ],
        params:[
            {description:"new state", optional:false, name:"newState", defaultValue:null, type:"State"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["state", "appearance"]
    },
    "attr:BlurbItem.wrap":{
        ref:"attr:BlurbItem.wrap",
        flags:"IRW",
        description:"If true, item contents can wrap. If false, all the contents should appear on a single line.",
        name:"wrap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BlurbItem",
        groups:["appearance"]
    },
    "attr:DataSourceField.sequenceName":{
        ref:"attr:DataSourceField.sequenceName",
        flags:"IR",
        description:"For a DataSource with ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType:\"sql\"')} with a field of type\n \"sequence\", the name of the SQL sequence that should be used when inserting new records into\n this table.\n <P>\n Note that this is never required for SQL tables that are generated from SmartClient\n DataSources (a default sequence name of tableName + \"_\" + columnName is chosen), and is \n never required for databases where inserting null into a sequence column is sufficient\n (MySQL, SQL Server, DB2 and others).  \n <P>\n You would only need to set sequenceName if you are integrating with a pre-existing table\n stored in a database where the sequence must be named for insertion to work (Oracle,\n Postgres) OR you are trying to use the same sequence across multiple DataSources.",
        name:"sequenceName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["sqlDataSource"]
    },
    "method:ImgButton.getTitle":{
        ref:"method:ImgButton.getTitle",
        flags:"A",
        description:" Return the title - text/HTML drawn inside the component.\n <p>\n Default is to simply return this.title.\n",
        name:"getTitle",
        seeAlso:[],
        returns:{description:"HTML for the title", type:"string"},
        type:"method",
        definingClass:"class:ImgButton"
    },
    "attr:Label.icon":{
        ref:"attr:Label.icon",
        flags:"IRW",
        description:"Optional icon to be shown with the button title text.  \n <P>\n Specify as the partial URL to an image, relative to the imgDir of this component.",
        name:"icon",
        setter:["setIcon"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "classMethod:EventHandler.getX":{
        ref:"classMethod:EventHandler.getX",
        flags:"",
        description:"			Return the page-relative X (horizontal) coordinate of an event.\n\n",
        name:"getX",
        seeAlso:[],
        returns:{description:"x-coordinate in page coordinate space", type:"number"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "method:Canvas.getPageRight":{
        ref:"method:Canvas.getPageRight",
        flags:"",
        description:" Return the page-relative right coordinate of this object, in pixels.\n		\n",
        name:"getPageRight",
        seeAlso:[],
        returns:{description:"GLOBAL right coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:Canvas.top":{
        flags:"IRW",
        getter:["getTop"],
        type:"attr",
        ref:"attr:Canvas.top",
        description:"Number of pixels the top of the widget is offset down from its default drawing context\n (either its parent's top-left corner, or the document flow, depending on the value of\n the ${isc.DocUtils.linkForRef('attr:Canvas.position')} property).\n <P>\n Can also be set as a percentage, specified as a String ending in '%', eg, \"50%\".  In\n this case the top coordinate is considered as a percentage of the specified height of\n the ${isc.DocUtils.linkForRef('attr:Canvas.parentElement','parent')}.",
        name:"top",
        setter:["setTop"],
        valueType:"Number or String",
        seeAlso:[],
        defaultValue:"0",
        groups:["positioning"],
        definingClass:"class:Canvas"
    },
    "method:ImgButton.setTitle":{
        ref:"method:ImgButton.setTitle",
        flags:"",
        description:" Set the title.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["appearance"]
    },
    "classMethod:Log.logInfo":{
        ref:"classMethod:Log.logInfo",
        flags:"",
        description:" Log a message at \"info\" priority\n \n",
        name:"logInfo",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "method:TreeGrid.nodeClick":{
        ref:"method:TreeGrid.nodeClick",
        flags:"",
        description:"\n This method is called when a leaf or folder record is clicked on.  Note that if you set\n up a callback for <code>nodeClick()</code> and e.g. ${isc.DocUtils.linkForRef('method:TreeGrid.leafClick')}, then\n both will fire (in that order) if a leaf is clicked on.\n\n",
        name:"nodeClick",
        seeAlso:[
            {ref:"method:TreeGrid.folderClick", description:null},
            {ref:"method:TreeGrid.leafClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which leafClick() occurred.", optional:false, name:"viewer",
             defaultValue:null, type:"TreeGrid"},
            {description:"The node (record) that was clicked", optional:false, name:"node",
             defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the click occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('treeDropEvents')}"],
        definingClass:"class:TreeGrid"
    },
    "method:ImgButton.getActionType":{
        ref:"method:ImgButton.getActionType",
        flags:"",
        description:" Return the 'actionType' for this canvas (radio / checkbox / button)\n",
        name:"getActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state", "event handling"]
    },
    "attr:SectionStackSection.ID":{
        ref:"attr:SectionStackSection.ID",
        flags:"IR",
        description:"Optional ID for the section.  This can be used later in calls to ${isc.DocUtils.linkForRef('class:SectionStack')} APIs such as\n ${isc.DocUtils.linkForRef('method:SectionStack.expandSection')} and ${isc.DocUtils.linkForRef('method:SectionStack.collapseSection')}.",
        name:"ID",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "method:ColorPicker.getLuminosity":{
        ref:"method:ColorPicker.getLuminosity",
        flags:"",
        description:" Returns the Luminosity (brightness) of the currently-selected color, as an \n integer from 0-240\n",
        name:"getLuminosity",
        seeAlso:[
            {ref:"method:ColorPicker.setLuminosity", description:null}
        ],
        returns:{description:"luminosity value", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "method:DynamicForm.getItem":{
        ref:"method:DynamicForm.getItem",
        flags:"",
        description:" Retrieve a ${isc.DocUtils.linkForRef('class:FormItem')} in this form by it's ${isc.DocUtils.linkForRef('attr:FormItem.name','name')},\n ${isc.DocUtils.linkForRef('attr:FormItem.dataPath','dataPath')}, or index within\n the ${isc.DocUtils.linkForRef('attr:DynamicForm.items','items')} array.\n <P>\n FormItems that also have a ${isc.DocUtils.linkForRef('attr:FormItem.ID')} may be accessed directly as a global variable\n <code>window[itemID]</code> or just <code>itemID</code>\n\n",
        name:"getItem",
        seeAlso:[
            {ref:"method:DynamicForm.getItem", description:null}
        ],
        params:[
            {description:"name of the item you're looking for", optional:false, name:"itemName",
             defaultValue:null, type:"string or int"}
        ],
        returns:{description:"FormItem object or null if not found", type:"FormItem"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["items"]
    },
    "group:clientServerIntegration":{
        ref:"group:clientServerIntegration",
        title:"Client-Server Integration",
        treeLocation:["Concepts"],
        refs:[],
        description:"Like client-server desktop applications, SmartClient browser-based applications interact\n with remote data and services via background communication channels. Background requests\n retrieve chunks of data rather than new HTML pages, and update your visual components in\n place rather than rebuilding the entire user interface.\n <P>\n <b>DataSources</b>\n <p>\n First you must create ${isc.DocUtils.linkForRef('class:DataSource','DataSources')} that describe the objects from\n your object model that will be loaded or manipulated within your application.  All of \n SmartClient's most powerful functionality builds on the concept of a DataSource, and because \n of SmartClient's databinding framework (see ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}), it's as easy to \n create a DataSource that can configure an unlimited number of components as it is to \n configure a single component.\n <P>\n For background information on how to create DataSources, ${isc.DocUtils.linkForRef('interface:DataBoundComponent','bind')}\n components to DataSources and initiate ${isc.DocUtils.linkForRef('class:DSRequest')}s, please see the <em>Data \n Binding</em> chapter of the <em>SmartClient Quickstart Guide</em>.\n <P>\n <b>Data Integration</b>\n <P>\n DataSources provide a data-provider agnostic API to SmartClient Visual Components that \n allow them to perform the 4 CRUD operations (<b>C</b>reate, <b>R</b>etrieve, \n <b>U</b>pdate, <b>D</b>elete).  By \"agnostic\" we mean that the implementation details - \n the nuts and bolts of how a given DataSource actually retrieves or updates data - are \n unknown to bound SmartClient components.  One effect of this is that DataSources are \n \"pluggable\": they can be replaced without affecting the User Interface.\n <p>\n When a visual component, or your own custom code, performs a CRUD operation on a DataSource,\n the DataSource creates a ${isc.DocUtils.linkForRef('class:DSRequest')} (DataSource Request) representing the operation.\n \"Data Integration\" is the process of fulfilling that DSRequest by creating a corresponding\n ${isc.DocUtils.linkForRef('class:DSResponse')} (DataSource Response), by using a variety of possible approaches to \n connect to the ultimate data provider.  \n <p>\n There are two main approaches to integrating DataSources with your server technology: \n <ul>\n <li><b>Server-side integration</b>: DataSource requests from the browser arrive as Java \n Objects on the server. You deliver responses to the browser by returning Java Objects. The\n various server-side integration possibilities are discussed later in this article.</li>\n <li>${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-side integration')}: DataSource requests arrive as \n simple HTTP requests which your server code receives directly (in Java, you use the \n Servlet API or .jsps to handle the requests). Responses are sent as XML or JSON which you \n directly generate.</li>\n </ul>\n The possible approaches are summarized in the diagram below. Paths 2, 3 and 4 are \n client-side integration approaches, and path 1 includes all server-side integration \n approaches. \n <p>\n <img src=\"skin/ClientServerIntegration.png\" width=\"866px\" height=\"495px\">\n <p>\n SmartClient supports, out of the box, codeless connectivity to various kinds of common data\n providers, including SQL and Hibernate.  SmartClient also provides functionality and tools\n for accelerated integration with broad categories of data providers, such as Java\n Object-based persistence mechanisms (JPA, EJB, Ibatis, in-house written systems), and REST \n and WSDL web services in XML or JSON formats.  Ultimately, a DataSource can be connected to \n anything that is accessible via HTTP or HTTPS, and also to in-browser persistence engines \n such as <a href='http://gears.google.com' onclick=\"window.open('http://gears.google.com');return false;\">Google Gears</a>.\n <p>\n <b>Choosing a Data Integration Approach</b><p>\n This section aims to help you decide which of the many possible data integration approaches\n is best for your particular circumstances.  The recommendations given here will guide you\n to the approach that involves the least effort.<p>\n <img src=\"skin/dataIntegrationFlowchart.png\" width=\"640px\" height=\"300px\">\n <p>\n <ul>\n <li>If you have a Java server</li>\n <ul><li>If your ultimate storage is a SQL database</li>\n   <ul>\n       <li>If you are already committed to Hibernate, use the Hibernate DataSource</li>\n       <li>Otherwise, use the SQL DataSource</li>\n       <li>Be sure to read the overview of ${isc.DocUtils.linkForRef('group:sqlVsJPA','SQL DataSource vs JPA')} and\n           other technologies.  If you ultimately decide not to use the SQL or \n           Hibernate DataSource, write a ${isc.DocUtils.linkForRef('group:writeCustomDataSource','custom DataSource')}</li>\n       <li>Derive DataSource definitions from existing tables using \n           ${isc.DocUtils.linkForExampleId('sqlWizard', 'Visual Builder wizards')} or the Batch DataSource \n           Generator tool.  Or, generate tables from DataSource definitions you create by \n           hand</li>\n   </ul>\n     <li>If your ultimate storage is not SQL, write a \n         ${isc.DocUtils.linkForRef('group:writeCustomDataSource','custom DataSource')}</li>\n     <li>Whether or not your storage is SQL, add business logic either declaratively in the \n         DataSource definition, via ${isc.DocUtils.linkForRef('class:DMI')}, or any combination of the two:\n   <ul><li>The &lt;criteria&gt; and &lt;values&gt; properties of an ${isc.DocUtils.linkForRef('class:OperationBinding')}\n           allow you to dynamically set data values at transaction-processing time, using \n           built-in ${isc.DocUtils.linkForRef('group:velocitySupport','Velocity support')}</li>\n       <li>Override the <code>validate()</code> method of the DataSource to provide extra\n           custom validations - just call <code>super</code> to obtain the list of errors \n           derived from SmartClient validations, then add to that list as required with your\n           own custom code</li>\n       <li>Override the <code>execute()</code> method of the DataSource to add extra processing\n           either before or after the SmartClient processing</li>\n       <li>Use ${isc.DocUtils.linkForRef('attr:DSRequestModifier.value','Transaction Chaining')} to dynamically set\n           data values according to the results of earlier transactions</li>\n       <li>For SQL DataSources, use ${isc.DocUtils.linkForRef('group:customQuerying','SQL Templating')} to change, \n           add to or even completely replace the SQL sent to the database, and to implement\n           special query requirements</li>\n       <li>For Hibernate DataSources, use ${isc.DocUtils.linkForRef('attr:OperationBinding.customHQL','custom HQL queries')}\n           to implement special query requirements</li>\n   </ul>\n         Read more about the server-side request processing flow and how to customize it in\n         ${isc.DocUtils.linkForRef('group:serverDataIntegration','the server integration overview')}.\n </ul>\n </ul>\n <ul>\n <li>If you do not have a Java server</li>\n   <ul><li>If you are not obliged to use a pre-existing network protocol, use the \n           ${isc.DocUtils.linkForRef('class:RestDataSource')}</li>\n       <li>Otherwise, use ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side data integration')} features\n           to create a custom client-side DataSource that adapts the DataSource protocol to \n           your existing services</li>\n   </ul>\n </ul>\n <p><br>\n <b>RPCs: Unstructured Server Communication</b>\n <P>\n SmartClient also supports \"unstructured\" client-server operations.  These \n ${isc.DocUtils.linkForRef('class:RPCRequest')}s (Remote Procedure Call Requests) are a low-level, very flexible \n mechanism for custom client-server communications.  In an nutshell, RPCRequests:\n <ul>\n <li> may contain arbitrary data\n <li> are always initiated by custom code (a call to ${isc.DocUtils.linkForRef('classMethod:RPCManager.send')}), and have\n their responses handled by custom code (the callback passed to <code>send()</code>)\n </ul>\n <P>\n RPCRequests are relatively rare.  Most client-server communications are better done in a \n structured fashion using a ${isc.DocUtils.linkForRef('class:DSRequest')} (DataSource Request).  Note that <em>any</em>\n RPCRequest can alternatively be framed as a ${isc.DocUtils.linkForRef('method:DataSource.fetchData','DataSource fetch')}; \n depending on the circumstances, this may be more convenient.\n <P>\n See the ${isc.DocUtils.linkForRef('class:RPCManager')} documentation for further information on RPCRequests.",
        name:"clientServerIntegration",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.peers":{
        ref:"attr:Canvas.peers",
        flags:"IRWA",
        description:"Array of all Canvii that are peers of this Canvas.",
        name:"peers",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "method:TabSet.addTabs":{
        ref:"method:TabSet.addTabs",
        flags:"A",
        description:" Add one or more tabs\n",
        name:"addTabs",
        seeAlso:[
            {ref:"method:TabSet.addTab", description:null}
        ],
        params:[
            {description:"new tab or tabs", optional:false, name:"tabs", defaultValue:null,
             type:"Tab or Array of Tab"},
            {description:"position where tab should be added (or array of positions)", optional:false,
             name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:DateChooser.prevMonthIcon":{
        ref:"attr:DateChooser.prevMonthIcon",
        flags:"IR",
        description:"Icon for the previous month button",
        name:"prevMonthIcon",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"[SKIN]arrow_left.gif\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:ResultSet.lengthIsKnown":{
        ref:"method:ResultSet.lengthIsKnown",
        flags:"",
        description:" Whether the ResultSet actually knows how many records are available from the server.\n The ResultSet will not know how many records are available when initially fetching and \n filtering data. Note that the value returned from ${isc.DocUtils.linkForRef('method:ResultSet.getLength')} will be \n an arbitrary, large value if the actual length is not known.\n",
        name:"lengthIsKnown",
        seeAlso:[],
        returns:{description:"whether length is known", type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "group:EventRegistry":{
        ref:"group:EventRegistry",
        refs:["classMethod:Page.setEvent", "classMethod:Page.clearEvent", "type:FireStyle"],
        name:"EventRegistry",
        seeAlso:[],
        type:"group"
    },
    "class:ToolbarItem":{
        methods:[],
        exampleConfig:"ToolbarItem.xml",
        ref:"class:ToolbarItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:ToolbarItem.buttons", "attr:ToolbarItem.showTitle", "attr:ToolbarItem.startRow", 
         "attr:ToolbarItem.endRow","attr:ToolbarItem.colSpan", "attr:ToolbarItem.vertical", 
         "attr:ToolbarItem.buttonConstructor","attr:ToolbarItem.buttonProperties", "attr:ToolbarItem.buttonBaseStyle"],
        description:"Set of horizontally arranged buttons.",
        name:"ToolbarItem",
        seeAlso:[],
        type:"class"
    },
    "attr:CubeGrid.autoSelectValues":{
        ref:"attr:CubeGrid.autoSelectValues",
        flags:"IR",
        description:"Whether to select cells in the body when row or column headers are selected.<br><br>\n Legal values: \"both\", \"rows\", \"cols\", \"none\" (or false)",
        name:"autoSelectValues",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"both\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:Layout.membersMargin":{
        ref:"attr:Layout.membersMargin",
        flags:"IRW",
        description:"Space between each member of the layout.\n <P>\n Requires a manual call to <code>reflow()</code> if changed on the fly.",
        name:"membersMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"0",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('userSizing')}"],
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "attr:Window.dismissOnOutsideClick":{
        ref:"attr:Window.dismissOnOutsideClick",
        flags:"IRW",
        description:"If true, a click outside the bounds of the Window will have the same effect as\n      pressing its cancel button.<br>\n      <b>Note:</b> Applies only to modal windows.",
        name:"dismissOnOutsideClick",
        seeAlso:[
            {ref:"attr:Window.isModal", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["modal"]
    },
    "method:Canvas.moveAbove":{
        ref:"method:Canvas.moveAbove",
        flags:"",
        description:" Puts this widget just above the specified widget in the stacking order, so it appears in\n front of the specified widget if both widgets have the same parent.\n",
        name:"moveAbove",
        seeAlso:[],
        params:[
            {description:"canvas to move above", optional:false, name:"canvas", defaultValue:null,
             type:"Canvas or subclass"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('layer')}"],
        definingClass:"class:Canvas",
        groups:["zIndex"]
    },
    "attr:Canvas.animateResizeAcceleration":{
        ref:"attr:Canvas.animateResizeAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated resize.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateResizeAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:CellSelection.deselectCell":{
        ref:"method:CellSelection.deselectCell",
        flags:"",
        description:"			Deselect a particular cell\n",
        name:"deselectCell",
        seeAlso:[],
        params:[
            {description:"row index of the cell to select", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of the cell to select", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection"]
    },
    "attr:SelectItem.valueField":{
        flags:"IRW",
        getter:["FormItem.getValueFieldName"],
        type:"attr",
        ref:"attr:SelectItem.valueField",
        description:"If this form item maps data values to display values by retrieving the \n ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} values from an \n ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}, this property \n denotes the the field to use as the underlying data value in records from the \n optionDataSource.<br>\n If unset, assumed to be the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this form item.",
        name:"valueField",
        valueType:"string",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('relatedRecords')}"],
        defaultValue:"null",
        groups:["display_values"],
        definingClass:"class:SelectItem"
    },
    "method:FormItem.setValueMap":{
        ref:"method:FormItem.setValueMap",
        flags:"A",
        description:" Set the valueMap for this item.\n",
        name:"setValueMap",
        seeAlso:[
            {ref:"attr:FormItem.valueMap", description:null}
        ],
        params:[
            {description:"new valueMap", optional:false, name:"valueMap", defaultValue:null,
             type:"Array or Object"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["valueMap"]
    },
    "attr:Layout.animateMembers":{
        ref:"attr:Layout.animateMembers",
        flags:"IRW",
        description:"If true when members are added / removed, they should be animated as they are shown\n or hidden in position",
        name:"animateMembers",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('animateLayout')}"],
        definingClass:"class:Layout",
        groups:["animation"]
    },
    "class:Page":{
        methods:[],
        ref:"class:Page",
        classAttrs:["classAttr:Page.defaultUnsupportedBrowserURL", "classAttr:Page.unsupportedBrowserAction"],
        classMethods:["classMethod:Page.isLoaded", "classMethod:Page.setTitle", "classMethod:Page.getAppDir", 
         "classMethod:Page.setAppImgDir","classMethod:Page.getAppImgDir", "classMethod:Page.setAppFilesDir", 
         "classMethod:Page.getAppFilesDir","classMethod:Page.setIsomorphicDir", "classMethod:Page.getIsomorphicDir", 
         "classMethod:Page.setSkinDir","classMethod:Page.getSkinDir", "classMethod:Page.getSkinImgDir", 
         "classMethod:Page.getImgURL","classMethod:Page.getURL", "classMethod:Page.isRTL", "classMethod:Page.loadStyleSheet", 
         "classMethod:Page.resizeTo","classMethod:Page.moveTo", "classMethod:Page.scrollTo", "classMethod:Page.getWidth", 
         "classMethod:Page.getHeight","classMethod:Page.getScrollWidth", "classMethod:Page.getScrollHeight", 
         "classMethod:Page.getScrollLeft","classMethod:Page.getScrollTop", "classMethod:Page.getScreenWidth", 
         "classMethod:Page.getScreenHeight","classMethod:Page.goBack", "classMethod:Page.waitFor", "classMethod:Page.waitForMultiple", 
         "classMethod:Page.checkBrowserAndRedirect","classMethod:Page.getUnsupportedBrowserPromptString", "classMethod:Page.setEvent", 
         "classMethod:Page.clearEvent","classMethod:Page.registerKey", "classMethod:Page.unregisterKey"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Provides information about the page you're loaded in.  We define \"page\" here to be roughly\n 	equivalent to the browser window or frame the libraries have been loaded in.",
        name:"Page",
        seeAlso:[],
        type:"class"
    },
    "attr:StatefulCanvas.autoFit":{
        ref:"attr:StatefulCanvas.autoFit",
        flags:"IRW",
        description:"If true, ignore the specified size of this widget and always size just large\n enough to accomodate the title.  If <code>setWidth()</code> is explicitly called on an\n autoFit:true button, autoFit will be reset to <code>false</code>.\n <P>\n Note that for StretchImgButton instances, autoFit will occur horizontally only, as \n unpredictable vertical sizing is likely to distort the media. If you do want vertical \n auto-fit, this can be achieved by simply setting a small height, and having \n overflow:\"visible\"",
        name:"autoFit",
        setter:["setAutoFit"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["sizing"]
    },
    "attr:ImgButton.iconSpacing":{
        ref:"attr:ImgButton.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:HiddenItem.showTitle":{
        ref:"attr:HiddenItem.showTitle",
        flags:"IRW",
        description:"we never show a separate title cell for hidden fields",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:HiddenItem",
        groups:["appearance"]
    },
    "method:Canvas.setHtmlPosition":{
        ref:"method:Canvas.setHtmlPosition",
        flags:"",
        description:" Setter for the ${isc.DocUtils.linkForRef('attr:Canvas.htmlPosition')}.\n",
        name:"setHtmlPosition",
        seeAlso:[],
        params:[
            {description:"New htmlPosition for this canvas", optional:false, name:"element",
             defaultValue:null, type:"DrawPosition"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["htmlElement"]
    },
    "attr:TreeGrid.treeFieldTitle":{
        ref:"attr:TreeGrid.treeFieldTitle",
        flags:"IR",
        description:"Visible title for the tree column (field).",
        name:"treeFieldTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Name\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeField"]
    },
    "attr:OperationBinding.serverMethod":{
        ref:"attr:OperationBinding.serverMethod",
        flags:"IR",
        description:"The name of the method to invoke on the ${isc.DocUtils.linkForRef('class:ServerObject')} for this operationBinding.",
        name:"serverMethod",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        requiresModules:["SCServer"]
    },
    "method:ImgButton.setBaseStyle":{
        ref:"method:ImgButton.setBaseStyle",
        flags:"",
        description:" Sets the base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n",
        name:"setBaseStyle",
        seeAlso:[],
        params:[
            {description:"new base style", optional:false, name:"style", defaultValue:null,
             type:"className"}
        ],
        type:"method",
        definingClass:"class:ImgButton"
    },
    "attr:ListGrid.groupTree":{
        ref:"attr:ListGrid.groupTree",
        flags:"R",
        description:"The data tree that results from a call to  ${isc.DocUtils.linkForRef('method:ListGrid.groupBy')}.\n This will be a ${isc.DocUtils.linkForRef('class:ResultTree')} if ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')} is\n present, otherwise it will be a ${isc.DocUtils.linkForRef('class:Tree')}.",
        name:"groupTree",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "class:JSONEncoder":{
        methods:["method:JSONEncoder.encode", "method:JSONEncoder.encodeDate"],
        ref:"class:JSONEncoder",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:JSONEncoder.dateFormat", "attr:JSONEncoder.strictQuoting", 
         "attr:JSONEncoder.circularReferenceMode","attr:JSONEncoder.circularReferenceMarker", "attr:JSONEncoder.prettyPrint"],
        description:"Class for encoding objects as JSON strings.",
        name:"JSONEncoder",
        seeAlso:[],
        type:"class"
    },
    "classAttr:RPCManager.saveDataPrompt":{
        ref:"classAttr:RPCManager.saveDataPrompt",
        flags:"IRW",
        description:"Default prompt displayed to the user while an opreration is running to save data to\n the server.<br>\n Displayed as a result of the ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} code path.",
        name:"saveDataPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Saving form...\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["i18nMessages"]
    },
    "attr:Scrollbar.showTrackEnds":{
        ref:"attr:Scrollbar.showTrackEnds",
        flags:"IRA",
        description:"If true, the scrollbar uses a 5-segment rather than 3-segment image representation,\n where the 3 interior image segments have the same state (Down, Over, etc), independent\n of the two outermost image segments.  \n <P>\n This allows certain advanced skinning designs where the track-as-such (space in which\n the thumb may be dragged) has curved endcaps, and is also visually stateful (that is,\n changes when the mouse goes down, without affecting the appearance of the outermost\n segments).",
        name:"showTrackEnds",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["track"]
    },
    "attr:SelectItem.showFocused":{
        ref:"attr:SelectItem.showFocused",
        flags:"IRWA",
        description:"When this item recieves focus, should it be re-styled to indicate it has focus?",
        name:"showFocused",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true, [IRWA]",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "attr:MultiView.currentDBC":{
        ref:"attr:MultiView.currentDBC",
        flags:"R",
        description:"The actual ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} currently displaying it's UI.",
        name:"currentDBC",
        seeAlso:[],
        valueType:"DataBoundComponent",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiView"
    },
    "method:ListGrid.reorderFields":{
        ref:"method:ListGrid.reorderFields",
        flags:"A",
        description:" Reorder a set of adjacent fields, from start to end exclusive at the end, by distance\n moveDelta.<br><br>\n\n NOTE: start and end coordinates are in terms of the currently visible fields, not the full set of\n fields.\n\n",
        name:"reorderFields",
        seeAlso:[],
        params:[
            {description:"Start of the range of fields to move, inclusive", optional:false,
             name:"start", defaultValue:null, type:"number"},
            {description:"End of the range of fields to move, non-inclusive", optional:false,
             name:"end", defaultValue:null, type:"number"},
            {description:"Distance to move by", optional:false, name:"moveDelta", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:TreeGrid.animateFolders":{
        ref:"attr:TreeGrid.animateFolders",
        flags:"IRW",
        description:"If true, when folders are opened / closed children will be animated into view.",
        name:"animateFolders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('animateTree')}"],
        definingClass:"class:TreeGrid",
        groups:["animation"]
    },
    "attr:ListGridRecord.singleCellValue":{
        ref:"attr:ListGridRecord.singleCellValue",
        flags:"IRW",
        description:"Default property name denoting the single value to display for all fields of this row.\n If this property is set for some record, the record will be displayed as a single \n cell spanning every column in the grid, with contents set to the value of this\n property.<br>\n Note: this attribute name is governed by ${isc.DocUtils.linkForRef('attr:ListGrid.singleCellValueProperty')}.",
        name:"singleCellValue",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "classAttr:Time.shortDisplayFormat":{
        ref:"classAttr:Time.shortDisplayFormat",
        flags:"RWA",
        description:"Standard formatter to be used when converting a date to a time-string via ${isc.DocUtils.linkForRef('classMethod:Time.toShortTime')}",
        name:"shortDisplayFormat",
        setter:["setShortDisplayFormat"],
        seeAlso:[],
        valueType:"timeFormatter | function",
        defaultValue:"\"toShortTime\"",
        type:"classAttr",
        definingClass:"class:Time"
    },
    "attr:OperationBinding.cacheSyncOperation":{
        ref:"attr:OperationBinding.cacheSyncOperation",
        flags:"IR",
        description:"For an operationBinding of ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"add\" or \"update\", this \n property is the ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId','operationId')} of a \"fetch\" operationBinding to use \n for cache synchronization purposes (ie, to fetch the row most recently added or changed).\n This property, along with ${isc.DocUtils.linkForRef('attr:OperationBinding.useForCacheSync','useForCacheSync')} and \n ${isc.DocUtils.linkForRef('attr:OperationBinding.canSyncCache','canSyncCache')} is provided so that you can use custom database operations\n without sacrificing the benefits of SmartClient's automatic cache synchronization.\n <p>\n This property is only applicable to DataSources of type \"sql\".",
        name:"cacheSyncOperation",
        seeAlso:[
            {ref:"attr:OperationBinding.useForCacheSync", description:null},
            {ref:"attr:OperationBinding.canSyncCache", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "method:Calendar.getActiveDay":{
        ref:"method:Calendar.getActiveDay",
        flags:"",
        description:" Gets the day of the week (0-6) that the mouse is currently over.\n\n",
        name:"getActiveDay",
        seeAlso:[
            {ref:"method:Calendar.getActiveTime", description:null}
        ],
        returns:{description:"the day that the mouse is currently over", type:"integer"},
        type:"method",
        definingClass:"class:Calendar"
    },
    "method:TabSet.getTabObject":{
        ref:"method:TabSet.getTabObject",
        flags:"",
        description:" Get the tab Object originally passed to ${isc.DocUtils.linkForRef('attr:TabSet.tabs')}, by index or ID.\n If passed a tab Object, just returns it.\n\n",
        name:"getTabObject",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"int | ID | Tab"}
        ],
        returns:{description:"the tab, or null if not found", type:"Tab"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:RPCRequest.omitNullMapValuesInResponse":{
        ref:"attr:RPCRequest.omitNullMapValuesInResponse",
        flags:"IRWA",
        description:"If enabled, the server omits any key/value pairs in map that have null values from the\n response.  This can reduce the size of the response when many fields have null values.\n <p>\n To enable this globally for all responses you can set RPCManager.omitNullMapValuesInResponse\n in server.properties.",
        name:"omitNullMapValuesInResponse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:DSResponse.clientContext":{
        ref:"attr:DSResponse.clientContext",
        flags:"R",
        description:"The ${isc.DocUtils.linkForRef('attr:DSRequest.clientContext')} object as set on the ${isc.DocUtils.linkForRef('class:DSRequest')}.",
        name:"clientContext",
        seeAlso:[
            {ref:"attr:DSRequest.clientContext", description:null},
            {ref:"attr:RPCResponse.clientContext", description:null}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse"
    },
    "method:ListGridField.formatEditorValue":{
        ref:"method:ListGridField.formatEditorValue",
        flags:"",
        description:" Return the value to display in cells of this field which are being edited.\n <P>\n <i>Example usage</i>: converting a stored value in cents (100) to a dollar-and-cents \n value in the editor (1.00)\n <P>\n The value passed to this method is the raw value for the cell.\n <P>\n <code>formatEditorValue</code> takes precedence over ${isc.DocUtils.linkForRef('method:ListGrid.formatEditorValue')}\n defined at the grid level for cells in this field.\n <P>\n To convert the formatted value displayed within an editor back to a raw value, implement\n ${isc.DocUtils.linkForRef('method:ListGridField.parseEditorValue')} as well.\n\n",
        name:"formatEditorValue",
        seeAlso:[
            {ref:"method:ListGridField.formatCellValue", description:null},
            {ref:"method:ListGrid.formatEditorValue", description:null},
            {ref:"method:ListGridField.parseEditorValue", description:null}
        ],
        params:[
            {description:"raw value for the cell being edited", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"Record object for the cell. Note: If this is a new row that has not been saved, in an \n   editable grid, it has no associated record object. In this case the edit values will\n   be passed in as this parameter.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"},
            {description:"A pointer to the ListGrid displaying the cell", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid instance"}
        ],
        returns:{description:"formatted value to display in the editor", type:"any"},
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:CubeGrid.facetLabelOver":{
        ref:"method:CubeGrid.facetLabelOver",
        flags:"",
        description:" StringMethod handler fired when mouseover occurs over a facet label\n\n\n",
        name:"facetLabelOver",
        seeAlso:[],
        params:[
            {description:"ID of the appropriate facet", optional:false, name:"facetId",
             defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "attr:Label.iconSize":{
        ref:"attr:Label.iconSize",
        flags:"IR",
        description:"Size in pixels of the icon image.\n <P>\n The <code>iconWidth</code> and <code>iconHeight</code> properties can be used to\n configure width and height separately.",
        name:"iconSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "attr:DetailViewerField.valueMap":{
        ref:"attr:DetailViewerField.valueMap",
        flags:"IR",
        description:"A property list (or an expression that evaluates to a property list)\n specifying a mapping of internal values to display values for the field (row).",
        name:"valueMap",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "attr:FacetValue.canCollapse":{
        ref:"attr:FacetValue.canCollapse",
        flags:"IR",
        description:"For individual parent facetValues within a hierarchical facet, this flag controls\n whether an expand/collapse control will be shown.",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:ViewLoader.viewURL":{
        ref:"attr:ViewLoader.viewURL",
        flags:"IR",
        description:"URL to load components from.",
        name:"viewURL",
        setter:["setViewURL"],
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('viewLoading')}"],
        definingClass:"class:ViewLoader"
    },
    "method:CubeGrid.deselectCell":{
        ref:"method:CubeGrid.deselectCell",
        flags:"",
        description:"\n  deselect a single cell - accepts cell ID or cell record.\n\n      <br><i>methodType</i> action\n",
        name:"deselectCell",
        seeAlso:[],
        params:[
            {description:"cell to deselect", optional:false, name:"cell", defaultValue:null,
             type:"ID | object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:DataBoundComponent.getDropValues":{
        ref:"method:DataBoundComponent.getDropValues",
        flags:"",
        description:" Returns the \"drop values\" to apply to a record dropped on this component prior to update.  Only\n applicable to databound components - see ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dropValues')} for more details.  If multiple records \n are being dropped, this method is called for each of them in turn.\n <P>\n The default implementation of this method returns the following:\n <UL>\n <LI>Nothing, if ${isc.DocUtils.linkForRef('attr:DataBoundComponent.addDropValues')} is false</LI>\n <LI>dropValues, if that property is set.  If the component's criteria object is applicable (as explained\n in the next item), it is merged into dropValues, with properties in dropValues taking precedence.</LI>\n <LI>The component's criteria object, if the most recent textMatchStyle for the component was \"exact\" \n     and it is simple criteria (ie, not an AdvancedCriteria object)</LI>\n <LI>Otherwise nothing</LI>\n </UL>\n <P>\n You can override this method if you need more complex setting of drop values than can be \n provided by simply supplying a dropValues object.\n \n",
        name:"getDropValues",
        seeAlso:[],
        params:[
            {description:"record being dropped", optional:false, name:"record", defaultValue:null,
             type:"Record"},
            {description:"dataSource the record being dropped is bound to", optional:false,
             name:"sourceDS", defaultValue:null, type:"DataSource"},
            {description:"record being dropped on", optional:false, name:"targetRecord",
             defaultValue:null, type:"Record"},
            {description:"index of record being dropped on", optional:false, name:"index",
             defaultValue:null, type:"int"},
            {description:"widget where dragging began", optional:false, name:"sourceWidget",
             defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "method:DataSource.updateData":{
        ref:"method:DataSource.updateData",
        flags:"",
        description:" Perform an \"update\" DataSource operation against this DataSource, to update values in an\n existing DataSource record.\n\n",
        name:"updateData",
        seeAlso:[],
        params:[
            {description:"updated record", optional:false, name:"updatedRecord", defaultValue:null,
             type:"Record Properties"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "method:Canvas.moveBelow":{
        ref:"method:Canvas.moveBelow",
        flags:"",
        description:" Puts this widget just below the specified widget in the stacking order, so it appears\n behind the specified widget if both widgets have the same parent.\n",
        name:"moveBelow",
        seeAlso:[],
        params:[
            {description:"canvas to move below", optional:false, name:"canvas", defaultValue:null,
             type:"Canvas or subclass"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('layer')}"],
        definingClass:"class:Canvas",
        groups:["zIndex"]
    },
    "method:Menu.getItemNum":{
        ref:"method:Menu.getItemNum",
        flags:"",
        description:" Given a MenuItem, return it's index in the items array.\n\n",
        name:"getItemNum",
        seeAlso:[],
        params:[
            {description:"index of the item or MenuItem itself", optional:false, name:"item",
             defaultValue:null, type:"MenuItem | int"}
        ],
        returns:{description:"index of the item, or -1 if not defined.", type:"int"},
        type:"method",
        definingClass:"class:Menu",
        groups:["menuItems"]
    },
    "attr:Applet.altHTML":{
        ref:"attr:Applet.altHTML",
        flags:"IR",
        description:"This text is displayed if the browser does not understand the tag used for the applet html\n  element.",
        name:"altHTML",
        seeAlso:[
            {ref:"attr:Applet.alt", description:null},
            {ref:"attr:Applet.useTag", description:null}
        ],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:ListGrid.discardEditsSaveButtonTitle":{
        ref:"attr:ListGrid.discardEditsSaveButtonTitle",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.confirmDiscardEdits')} is true this is the title for the save button\n appearing in the lost edits confirmation dialog. Override this for localization if necessary.",
        name:"discardEditsSaveButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Save\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing", "i18nMessages"]
    },
    "attr:ViewLoader.viewURLParams":{
        ref:"attr:ViewLoader.viewURLParams",
        flags:"IR",
        description:"Parameters to be sent to the viewURL when fetching the view.",
        name:"viewURLParams",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ViewLoader"
    },
    "classMethod:Log.traceMethod":{
        ref:"classMethod:Log.traceMethod",
        flags:"",
        description:"\n  Observe a method on an object, logging a stack trace whenever the method is called.\n  <P>\n  Call a second time with identical arguments to disable tracing.\n\n",
        name:"traceMethod",
        seeAlso:[],
        params:[
            {description:"object to observe", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"name of the method to observe", optional:false, name:"methodName",
             defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "attr:Canvas.showEdges":{
        ref:"attr:Canvas.showEdges",
        flags:"IR",
        description:"Whether an ${isc.DocUtils.linkForRef('class:EdgedCanvas')} should be used to show image-based edges around this\n component.",
        name:"showEdges",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "attr:DataSource.requires":{
        ref:"attr:DataSource.requires",
        flags:"IR",
        description:"Indicates that the specified ${isc.DocUtils.linkForRef('type:VelocityExpression')} must evaluate to true for a user \n to access this DataSource.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:OperationBinding.requires')}.",
        name:"requires",
        seeAlso:[],
        valueType:"VelocityExpression",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSource",
        groups:["auth"]
    },
    "method:DynamicForm.submit":{
        ref:"method:DynamicForm.submit",
        flags:"",
        description:" <code>submit()</code> is automatically called when a ${isc.DocUtils.linkForRef('class:SubmitItem')} included in the\n form is clicked, or, if ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOnEnter','saveOnEnter')} is set, when the\n \"Enter\" key is pressed in a text input.  Submit can also be manually called.\n <P>\n If ${isc.DocUtils.linkForRef('method:DynamicForm.submitValues','form.submitValues()')} exists, it will be called, then\n immediately return.\n <P>\n Otherwise, default behavior varies based on ${isc.DocUtils.linkForRef('attr:DynamicForm.canSubmit','form.canSubmit')}: if\n <code>canSubmit</code> is false, ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} will be called to\n handle saving via SmartClient databinding.  \n <P>\n If <code>canSubmit</code> is true, the form will be submitted like an ordinary HTML\n form via ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm')}.\n <P>\n The parameters to <code>submit()</code> apply only if <code>submit()</code> will be\n calling ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')}.  If you override <code>submit()</code>, you can safely\n ignore the parameters as SmartClient framework code does not pass them.\n \n",
        name:"submit",
        seeAlso:[
            {ref:"method:DynamicForm.submitValues", description:null}
        ],
        params:[
            {description:"callback to invoke on completion.                                          [Ignored if this.canSubmit is true]",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued\n                                          [Ignored if this.canSubmit is true]",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "method:RichTextEditor.setValue":{
        ref:"method:RichTextEditor.setValue",
        flags:"",
        description:"  Updates the current value of the edit area.\n",
        name:"setValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:RichTextEditor"
    },
    "attr:Calendar.datePickerHoverText":{
        ref:"attr:Calendar.datePickerHoverText",
        flags:"IR",
        description:"The text to be displayed when a user hovers over the ${isc.DocUtils.linkForRef('attr:Calendar.datePickerButton','date picker')}\n toolbar button",
        name:"datePickerHoverText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Choose a date\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "group:dragTracker":{
        ref:"group:dragTracker",
        refs:["method:ListGrid.setDragTracker", "method:ListGrid.getDragTrackerIcon", 
         "method:ListGrid.getDragTrackerTitle","method:TreeGrid.getDragTrackerIcon", "method:TileGrid.setDragTracker", 
         "method:TileGrid.getDragTrackerTitle","method:DataBoundComponent.setDragTracker", "classMethod:EventHandler.setDragTracker", 
         "attr:ListGrid.dragTrackerMode","attr:ListGrid.trackerImage", "type:DragTrackerMode"],
        name:"dragTracker",
        seeAlso:[],
        type:"group"
    },
    "class:SelectItem":{
        methods:["method:SelectItem.defaultDynamicValue", "method:SelectItem.showPicker", 
         "method:SelectItem.fetchData","method:SelectItem.getDisplayFieldName", "method:SelectItem.getValueFieldName", 
         "method:SelectItem.dataArrived","method:SelectItem.getSelectedRecord"],
        ref:"class:SelectItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:SelectItem.height", "attr:SelectItem.addUnknownValues", 
         "attr:SelectItem.defaultValue","attr:SelectItem.defaultToFirstOption", "attr:SelectItem.showPickerIcon", 
         "attr:SelectItem.textBoxStyle","attr:SelectItem.controlStyle", "attr:SelectItem.pickerIconStyle", 
         "attr:SelectItem.showFocused","attr:SelectItem.pickerIconWidth", "attr:SelectItem.pickerIconHeight", 
         "attr:SelectItem.pickerIconSrc","attr:SelectItem.pickerIconProperties", "attr:SelectItem.hiliteOnFocus", 
         "attr:SelectItem.hiliteColor","attr:SelectItem.hiliteTextColor", "attr:SelectItem.pickButtonWidth", 
         "attr:SelectItem.pickButtonHeight","attr:SelectItem.pickButtonSrc", "attr:SelectItem.showOver", 
         "attr:SelectItem.allowEmptyValue","attr:SelectItem.autoFetchData", "attr:SelectItem.showHintInField", 
         "attr:SelectItem.optionDataSource","attr:SelectItem.pickListFields", "attr:SelectItem.optionFilterContext", 
         "attr:SelectItem.optionOperationId","attr:SelectItem.displayField", "attr:SelectItem.valueField", 
         "attr:SelectItem.filterLocally","attr:SelectItem.pickListCriteria", "attr:SelectItem.showOptionsFromDataSource", 
         "attr:SelectItem.pickListProperties","attr:SelectItem.sortField", "attr:SelectItem.multiple"],
        description:"FormItem that allows picking between several mutually exclusive options via a select list.\n <P>\n Options may be derived from a <code>dataSource</code> or a <code>valueMap</code>.\n <P>\n Note that to select the first option as a default value for the item,\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption')} may be set.",
        name:"SelectItem",
        implementsInterfaces:["PickList"],
        seeAlso:[
            {ref:"attr:PickList.optionDataSource", description:null},
            {ref:"attr:FormItem.valueMap", description:null}
        ],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('selectItem')}"]
    },
    "attr:StretchImgButton.iconHeight":{
        ref:"attr:StretchImgButton.iconHeight",
        flags:"IR",
        description:"Height in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:Layout.enforcePolicy":{
        ref:"attr:Layout.enforcePolicy",
        flags:"IRWA",
        description:"Whether the layout policy is continuously enforced as new members are added or removed\n and as members are resized.\n <p>\n This setting implies that any member that resizes larger, or any added member, will take\n space from other members in order to allow the overall layout to stay the same size.",
        name:"enforcePolicy",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:TileRecord.tileConstructor":{
        ref:"attr:TileRecord.tileConstructor",
        flags:"IRW",
        description:"SmartClient Class to use to construct the tile for this particular record.",
        name:"tileConstructor",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TileRecord"
    },
    "method:DataSource.removeData":{
        ref:"method:DataSource.removeData",
        flags:"",
        description:" Perform a \"remove\" DataSource operation against this DataSource, to delete an existing\n DataSource record.\n\n",
        name:"removeData",
        seeAlso:[],
        params:[
            {description:"primary key values of record to delete,                                             (or complete record)",
             optional:false, name:"data", defaultValue:null, type:"Record PrimaryKeys"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "attr:Dialog.styleName":{
        ref:"attr:Dialog.styleName",
        flags:"IRW",
        description:"Style of the Dialog background",
        name:"styleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"dialogBackground\"",
        type:"attr",
        definingClass:"class:Dialog",
        groups:["appearance"]
    },
    "attr:FormItem.cellStyle":{
        ref:"attr:FormItem.cellStyle",
        flags:"IRW",
        description:"CSS style applied to the form item as a whole, including the text element, any icons, and\n any hint text for the item. Applied to the cell containing the form item.\n <P>\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"cellStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"formCell\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:FormItem.pickerConstructor":{
        ref:"attr:FormItem.pickerConstructor",
        flags:"IRW",
        description:"Class name of the picker to be created.",
        name:"pickerConstructor",
        seeAlso:[],
        valueType:"SCClassName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:Tab.icon":{
        ref:"attr:Tab.icon",
        flags:"IRW",
        description:"If specified, this tab will show an icon next to the tab title.  Note that as with \n ${isc.DocUtils.linkForRef('attr:Button.icon')}, the URL of a tabs icon will be updated to reflect disabled state.<br>\n If desired a click handler may be assigned to the icon, which will be fired when the user\n clicks the tab. This method takes a single parameter <code>tab</code>, a pointer to the tab\n object.",
        name:"icon",
        seeAlso:[
            {ref:"method:TabSet.tabIconClick", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('tabsOrientation')}"],
        definingClass:"object:Tab"
    },
    "attr:FormItem.optionCriteria":{
        ref:"attr:FormItem.optionCriteria",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, and this property may be used\n to specify criteria to pass to the datasource when\n performing the fetch operation on the dataSource to obtain a data-value to display-value\n mapping",
        name:"optionCriteria",
        seeAlso:[],
        valueType:"criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "class:DateTimeItem":{
        methods:[],
        ref:"class:DateTimeItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:DateTimeItem.useTextField", "attr:DateTimeItem.displayFormat", 
         "attr:DateTimeItem.inputFormat"],
        description:"Subclass of DateItem for manipulating ${isc.DocUtils.linkForRef('type:FieldType','datetimes')}.",
        name:"DateTimeItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('dateItem')}"]
    },
    "attr:FormulaBuilder.autoHideCheckBox":{
        ref:"attr:FormulaBuilder.autoHideCheckBox",
        flags:"IR",
        description:"CheckBox that, when selected, hides columns in the component that are used in this formula.",
        name:"autoHideCheckBox",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "classAttr:Array.DATE_VALUES":{
        ref:"classAttr:Array.DATE_VALUES",
        flags:null,
        description:"This is a built-in comparator for the ${isc.DocUtils.linkForRef('method:Array.find','find')} and ${isc.DocUtils.linkForRef('method:Array.findIndex','findIndex')}\n methods of Array.  Passing this comparator to those methods will find instances where Dates  \n in the search criteria match Dates in the array members (ordinarily, Javascript only regards\n Dates as equal if they refer to the exact same object).  This comparator compares <i>logical</i>\n dates; the time elements of the values being compared are ignored, so two Dates representing\n different times on the same day will be considered equal.",
        name:"DATE_VALUES",
        seeAlso:[
            {ref:"classAttr:Array.DATETIME_VALUES", description:null}
        ],
        valueType:"Function",
        defaultValue:"See below",
        type:"classAttr",
        definingClass:"object:Array"
    },
    "attr:TextItem.height":{
        ref:"attr:TextItem.height",
        flags:"IRW",
        description:"Default height for text items.",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"19",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["appearance"]
    },
    "attr:Slider.animateThumbInit":{
        ref:"attr:Slider.animateThumbInit",
        flags:"IRW",
        description:"If thumb animation is enabled, should the thumb be animated to its initial value?",
        name:"animateThumbInit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Slider",
        groups:["animation"]
    },
    "attr:Calendar.showWorkday":{
        ref:"attr:Calendar.showWorkday",
        flags:"IR",
        description:"If set, causes the calendar to use ${isc.DocUtils.linkForRef('attr:Calendar.workdayBaseStyle')}\n for cells falling within the workday as defined by ${isc.DocUtils.linkForRef('attr:Calendar.workdayStart')} and ${isc.DocUtils.linkForRef('attr:Calendar.workdayEnd')},\n in both the ${isc.DocUtils.linkForRef('attr:Calendar.weekView')} and ${isc.DocUtils.linkForRef('attr:Calendar.dayView')}.",
        name:"showWorkday",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday"]
    },
    "attr:ListGridField.showDefaultContextMenu":{
        ref:"attr:ListGridField.showDefaultContextMenu",
        flags:"IRW",
        description:"When set to false, this field will not show a context menu in it's header.",
        name:"showDefaultContextMenu",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:RestDataSource.metaDataPrefix":{
        ref:"attr:RestDataSource.metaDataPrefix",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:RestDataSource.sendMetaData')} is true, this attribute is used to specify\n the prefix to apply to 'meta data' properties when assembling parameters to send to the \n server.  Applies to operations where OperationBinding.dataProtocol is set to \n <code>\"getParams\"</code> or <code>\"postParams\"</code> only.",
        name:"metaDataPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"_\"",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "method:TreeGrid.recordDrop":{
        ref:"method:TreeGrid.recordDrop",
        flags:"",
        description:" The superclass event ${isc.DocUtils.linkForRef('method:ListGrid.recordDrop')} does not fire on a TreeGrid, use\n ${isc.DocUtils.linkForRef('method:TreeGrid.folderDrop')} instead.\n\n",
        name:"recordDrop",
        seeAlso:[],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.filterByEditor":{
        ref:"method:ListGrid.filterByEditor",
        flags:"",
        description:" If the filter editor (${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')}) is visible for this grid, \n this method will perform a filter based on the current values in the editor.\n",
        name:"filterByEditor",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "attr:Canvas.backgroundPosition":{
        ref:"attr:Canvas.backgroundPosition",
        flags:"IRW",
        description:"Specifies how the background image should be positioned on the widget.\n      It corresponds to the CSS background-position attribute. If unset,\n      no background-position attribute is specified if a background image is\n      specified.",
        name:"backgroundPosition",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:TileGrid.invalidateCache":{
        ref:"method:TileGrid.invalidateCache",
        flags:"",
        description:" Invalidate the current data cache for this databound component via a call to\n <code>this.data.invalidateCache()</code>. If necessary, this will cause a new fetch to \n be performed with the current set of criteria for this component.\n <P>\n Has no effect if this component is not showing a set of filtered data.\n \n",
        name:"invalidateCache",
        seeAlso:[],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DataBoundComponent.dropValues":{
        flags:"IRWA",
        getter:["getDropValues"],
        type:"attr",
        ref:"attr:DataBoundComponent.dropValues",
        description:"When an item is dropped on this component, and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.addDropValues')} is true and both \n          the source and target widgets are databound, either to the same DataSource or \n          to different DataSources that are related via a foreign key, this object \n          provides the \"drop values\" that SmartClient will apply to the dropped object \n          before updating it.\n          <P>\n          If this property is not defined, SmartClient defaults to returning the selection\n          criteria currently in place for this component.  Thus, any databound items (for example, \n          rows from other grids bound to the same DataSource) dropped on the grid will,\n          by default, be subjected to an update that makes them conform to the grid's \n          current filter criteria.",
        name:"dropValues",
        valueType:"Object",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('listRecategorize')}"],
        defaultValue:"null",
        groups:["dragging"],
        definingClass:"interface:DataBoundComponent"
    },
    "attr:ListGridField.validateOnChange":{
        ref:"attr:ListGridField.validateOnChange",
        flags:"IRW",
        description:"If set to true, any ${isc.DocUtils.linkForRef('attr:ListGridField.validators')} for this field will be run whenever\n the value of the field is changed.\n <P>\n Analgous to the ${isc.DocUtils.linkForRef('attr:FormItem.validateOnChange')} property.",
        name:"validateOnChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["gridValidation"]
    },
    "method:ColumnTree.getIcon":{
        ref:"method:ColumnTree.getIcon",
        flags:"",
        description:" Get the appropriate icon for a node.\n <P>\n By default icons are derived from ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')} and ${isc.DocUtils.linkForRef('attr:ColumnTree.nodeIcon')}.\n Custom icons for individual nodes can be overriden by setting the ${isc.DocUtils.linkForRef('attr:ColumnTree.customIconProperty')}\n on a node.\n <p>\n If you want to suppress icons altogether, provide an override of this method that simply\n returns null.\n <p> \n Note that the full icon URL will be derived by applying ${isc.DocUtils.linkForRef('method:Canvas.getImgURL')} to the\n value returned from this method.\n\n",
        name:"getIcon",
        seeAlso:[],
        params:[
            {description:"tree node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"URL for the icon to show for this node", type:"URL"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "attr:HiddenItem.rowSpan":{
        ref:"attr:HiddenItem.rowSpan",
        flags:"IRW",
        description:"hidden fields don't take up any rows",
        name:"rowSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:HiddenItem",
        groups:["appearance"]
    },
    "method:CubeGrid.facetValueHoverHTML":{
        ref:"method:CubeGrid.facetValueHoverHTML",
        flags:"",
        description:"\n      Get the HTML for the facetValue button hover.  Default implementation returns null.\n\n      <br><i>methodType</i> callback\n",
        name:"facetValueHoverHTML",
        seeAlso:[],
        params:[
            {description:"facetValues object for the button", optional:false, name:"facetValues",
             defaultValue:null, type:"facetValues object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "method:Tree.hasChildren":{
        ref:"method:Tree.hasChildren",
        flags:"",
        description:"\n Returns true if this node has any children.\n\n",
        name:"hasChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node has children", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:TreeGrid.setOpenState":{
        ref:"method:TreeGrid.setOpenState",
        flags:"",
        description:" Reset this set of open folders within this grid's data to match the \n ${isc.DocUtils.linkForRef('type:treeGridOpenState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to \n ${isc.DocUtils.linkForRef('method:TreeGrid.getOpenState')}.\n\n",
        name:"setOpenState",
        seeAlso:[
            {ref:"method:TreeGrid.getOpenState", description:null}
        ],
        params:[
            {description:"Object describing the desired set of open folders.", optional:false,
             name:"openState", defaultValue:null, type:"treeGridOpenState"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "classMethod:Class.clearLogPriority":{
        ref:"classMethod:Class.clearLogPriority",
        flags:"",
        description:" Clear this object's priority setting for a particular category, so that the category's \n effective priority returns to the specified priority for this category at the Log level\n (or <code>Log.defaultPriority</code> if not set).<br>\n To clear the Page-level priority setting for this log category use \n <code>isc.Log.clearPriority()</code> instead.\n\n",
        name:"clearLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.clearPriority", description:null}
        ],
        params:[
            {description:"Category name. If not specified, all logging on this object                              will revert to default priority settings.",
             optional:false, name:"category", defaultValue:null, type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:DataBoundComponent.anySelected":{
        ref:"method:DataBoundComponent.anySelected",
        flags:"",
        description:" Whether at least one item is selected\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "attr:Canvas.dynamicContents":{
        ref:"attr:Canvas.dynamicContents",
        flags:"IRWA",
        description:"Dynamic contents allows the contents string to be treated as a simple, but powerful\n template.  When this attribute is set to true, expressions of the form \\${arbitrary JS\n here} are replaced by the result of the evaluation of the JS code inside the curly\n brackets.  This evaluation happens at draw time.  If you want to trigger a re-evaluation\n of the expressions in the contents string you can call markForRedraw() on the canvas.\n <p>\n You can use this feature to build some simple custom components. For example, let's say\n you want to show the value of a Slider in a Canvas somewhere on the screen.  You can do\n this by observing the valueChanged() method on the slider and calling setContents() on\n your canvas with the new string or you can set the contents of the canvas to something\n like:\n <p><code>\n \"The slider value is \\${sliderInstance.getValue()}.\"\n </code><p>\n Next you set dynamicContents: true on the canvas, observe valueChanged() on the slider\n and call canvas.markForRedraw() in that observation.  This approach is cleaner than\n setContents() when the Canvas is aggregating several values or dynamic expressions.\n Like so:\n <p>\n <pre>\n Slider.create({\n     ID: \"mySlider\"\n });\n\n Canvas.create({\n     ID: \"myCanvas\",\n     dynamicContents: true,\n     contents: \"The slider value is \\${mySlider.getValue()}.\"\n });\n     \n myCanvas.observe(mySlider, \"valueChanged\", \n                  \"observer.markForRedraw()\");\n </pre>\n You can embed an arbitrary number of dynamic expressions in the contents string.  The\n search and replace is optimized for speed.\n <p>\n If an error occurs during the evaluation of one of the expressions, a warning is logged\n to the ISC Developer Console and the error string is embedded in place of the expected\n value in the Canvas.\n <p>\n The value of a function is its return value.  The value of any variable is the same as\n that returned by its toString() representation.\n <p>\n Inside the evalution contentext, <code>this</code> points to the canvas instance that\n has the dynamicContents string as its contents - in other words the canvas instance on\n which the template is declared.",
        name:"dynamicContents",
        seeAlso:[
            {ref:"attr:Canvas.contents", description:null},
            {ref:"attr:Canvas.dynamicContentsVars", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dynamicContents')}"],
        definingClass:"class:Canvas",
        groups:["contents"]
    },
    "attr:ListGrid.bodyOverflow":{
        flags:"IRWA",
        type:"attr",
        ref:"attr:ListGrid.bodyOverflow",
        description:"Overflow setting for the \"body\", that is, the area of the grid where data values are\n rendered.\n <P>\n By setting both the grid itself and the body to overflow:visible, it is possible to\n \"auto-fit\" to the rendered height or width of the rows.  Note that in this case\n <code>grid.width</code> and <code>grid.height</code> act as minimums.",
        name:"bodyOverflow",
        setter:["setBodyOverflow"],
        valueType:"Overflow",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('autofitRows')}"],
        defaultValue:"Canvas.AUTO",
        groups:["sizing"],
        definingClass:"class:ListGrid"
    },
    "attr:Chart.data":{
        ref:"attr:Chart.data",
        flags:"IR",
        description:"Dataset for this chart.\n <P>\n Two basic formats are supported:\n <ul>\n <li> \"Standard model\": <code>data</code> is an array of CellRecords where each record\n contains one data value.  Each record also contains a property named after each facetId\n whose value is a facetValueId from that facet.\n <p>\n For example, with a facet with id \"regions\" and facetValues \"west\", \"north\" and \"east\", and\n with ${isc.DocUtils.linkForRef('attr:Chart.valueProperty')} with it's default value \"_value\", the <code>data</code> property\n could be:\n <pre>\n    isc.Chart.create({\n       facets:[{ id:\"regions\" }],\n       data : [\n          {regions:\"west\", _value:4},\n          {regions:\"north\", _value:2},\n          {regions:\"east\", _value:5}\n       ]\n    })</pre>\n If there were a second facet with id \"product\" and facetValues \"cars\" and \"trucks\", a Chart\n with a complete set of values would be:\n <pre>\n    isc.Chart.create({\n       facets:[{ id:\"regions\" }, { id:\"product\" }],\n       data : [\n          {product:\"cars\", regions:\"west\", _value:4},\n          {product:\"cars\", regions:\"north\", _value:2},\n          {product:\"cars\", regions:\"east\", _value:5},\n          {product:\"trucks\", regions:\"west\", _value:1},\n          {product:\"trucks\", regions:\"north\", _value:9},\n          {product:\"trucks\", regions:\"east\", _value:3}\n       ]\n    })</pre>\n This 2 facet (or \"2 dimensional\") dataset, if rendered as a bar chart, would use stacked or\n clustered bars and a legend.\n <P>\n\n <li> \"Inlined facet\": <code>data</code> is a single CellRecord or Array of CellRecords where\n each record contains multiple data values.  In this case, one facet definition is considered\n \"inlined\", meaning that the facetValueIds from this facet appear as properties in each\n record, and each such property holds one data value.  For example, a complete chart\n definition whose dataset is equivalent to the previous example would be:\n <pre>\n    isc.Chart.create({\n       facets: [{ \n          inlinedValues:true,\n          values : [ { id:\"west\" }, { id:\"north\" }, { id : \"east\" } ]\n       }],\n       data : { west:4, north:5, east:2 }\n    })</pre>\n Note that the property \"inlinedValues\" must be set on the facet definition, and the set of\n facetValueIds must be specified outside of the data array (with the \"standard model\",\n facetValueIds can be automatically derived from the data).  \n <P>\n A two facet chart with an inlined facet can be defined as follows:\n <pre>\n    isc.Chart.create({\n       facets: [\n         { \n            inlinedValues:true,\n            values : [ { id:\"west\" }, { id:\"north\" }, { id : \"east\" } ]\n         }, \n         { id:\"product\" }\n       ],\n       data : [\n           { product:\"cars\", west:4, north:5, east:2 },\n           { product:\"trucks\", west:1, north:9, east:3 }\n       ]\n    })</pre>\n </ul>\n Comparing between the formats, the \"standard model\" format treats all facets identically,\n which can be of use when integrating with server technology that likewise treats all facets\n identically.  The \"inlined facet\" format is a more compact data representation and allows\n easier conversion from data displayed in a ListGrid.",
        name:"data",
        setter:["setData"],
        seeAlso:[],
        valueType:"Array of CellRecord or CellRecord",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:ListGridField.valueIconHeight":{
        ref:"attr:ListGridField.valueIconHeight",
        flags:"IRW",
        description:"Height for value icons for this listGrid field.\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconSize')}, ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconHeight')}, and\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconSize')}.",
        name:"valueIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "classAttr:RPCManager.promptStyle":{
        ref:"classAttr:RPCManager.promptStyle",
        flags:"IRW",
        description:"Controls the default prompt style.  Overrideable by ${isc.DocUtils.linkForRef('attr:RPCRequest.promptStyle')}.",
        name:"promptStyle",
        seeAlso:[
            {ref:"attr:RPCRequest.promptStyle", description:null}
        ],
        valueType:"PromptStyle",
        defaultValue:"\"dialog\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["rpcPrompt"]
    },
    "method:Class.addPropertyList":{
        ref:"method:Class.addPropertyList",
        flags:"",
        description:"\n	Add properties to this instance.\n\n",
        name:"addPropertyList",
        seeAlso:[],
        params:[
            {description:"listarray of objects with properties to add", optional:true, name:"",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"the object after properties have been added to it", type:"object"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:ImgButton.showFocusedIcon":{
        ref:"attr:ImgButton.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:ImgButton.radioGroup":{
        ref:"attr:ImgButton.radioGroup",
        flags:"IRWA",
        description:"String identifier for this canvas's mutually exclusive selection group.",
        name:"radioGroup",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonRadioToggle')}"],
        definingClass:"class:ImgButton",
        groups:["state", "event handling"]
    },
    "attr:Window.footerHeight":{
        ref:"attr:Window.footerHeight",
        flags:"IR",
        description:"The height of the footer, in pixels.",
        name:"footerHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"18",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "footer"]
    },
    "type:TopOperatorAppearance":{
        ref:"type:TopOperatorAppearance",
        values:[
            {description:"radio buttons appear at the top of the form", value:"\"radio\""},
            {description:"a SelectItem appears with a \"bracket\" spanning all top-level clauses, exactly the same appearance used for showing\n ${isc.DocUtils.linkForRef('attr:FilterBuilder.showSubClauseButton','subClauses')}, if enabled.",
             value:"\"bracket\""},
            {description:"no interface is shown.  The top-level operator is expected to be shown to the user outside the FilterBuilder, and, if editable, ${isc.DocUtils.linkForRef('method:FilterBuilder.setTopOperator')}\n should be called to update it",
             value:"\"none\""}
        ],
        description:"Interface to use for the showing and editing the ${isc.DocUtils.linkForRef('attr:FilterBuilder.topOperator','top-level operator')} of a\n FilterBuilder.",
        name:"TopOperatorAppearance",
        seeAlso:[],
        type:"type"
    },
    "method:DataBoundComponent.selectRecords":{
        ref:"method:DataBoundComponent.selectRecords",
        flags:"",
        description:"\n Select/deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n\n",
        name:"selectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to select", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "method:PickList.fetchData":{
        ref:"method:PickList.fetchData",
        flags:"",
        description:" Only applies to databound items (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}).<br>\n Performs a fetch type operation on this item's DataSource to retrieve the set of valid\n options for the item, based on the current ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')}.\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Callback to fire when the fetch completes. Callback will               fire with 4 parameters:<ul>\n  <li><code>item</code> a pointer to the form item\n  <li><code>dsResponse</code> the ${isc.DocUtils.linkForRef('class:DSResponse')} returned by the server\n  <li><code>data</code> the raw data returned by the server\n  <li><code>dsRequest</code> the ${isc.DocUtils.linkForRef('class:DSRequest')} sent to the server\n  </ul>",
             optional:false, name:"callback", defaultValue:null, type:"callback"},
            {description:"properties to apply to the              dsRequest for this fetch.",
             optional:false, name:"requestProperties", defaultValue:null, type:"dsRequest properties"}
        ],
        type:"method",
        definingClass:"interface:PickList"
    },
    "attr:Slider.labelWidth":{
        ref:"attr:Slider.labelWidth",
        flags:"IRW",
        description:"The width of the labels used to display the minimum, maximum and current values of the\n slider.",
        name:"labelWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"50",
        type:"attr",
        definingClass:"class:Slider"
    },
    "method:DynamicForm.getFields":{
        ref:"method:DynamicForm.getFields",
        flags:"",
        description:" Method to retrieve the ${isc.DocUtils.linkForRef('attr:DynamicForm.fields','items')} for this DynamicForm. \n\n",
        name:"getFields",
        seeAlso:[],
        returns:{description:null, type:"Array of FormItem"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["elements"]
    },
    "method:Tree.getPath":{
        ref:"method:Tree.getPath",
        flags:"",
        description:"\n Returns the path of a node - a path has the following format:\n <code>([name][pathDelim]?)*</code>\n <br><br>\n For example, in this tree:\n <pre>\n root\n   foo\n     bar\n </pre>\n Assuming that ${isc.DocUtils.linkForRef('attr:Tree.pathDelim')} is the default <code>/</code>, the <code>bar</code>\n node would have the path <code>root/foo/bar</code> and the path for the <code>foo</code>\n node would be <code>root/foo</code>.\n <br><br>\n Once you have a path to a node, you can call find(path) to retrieve a reference to the node\n later.\n\n",
        name:"getPath",
        seeAlso:[
            {ref:"method:Tree.getParentPath", description:null}
        ],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"path to the node", type:"string"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:Slider.vertical":{
        ref:"attr:Slider.vertical",
        flags:"IRW",
        description:"Indicates whether this is a vertical or horizontal slider.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:Slider"
    },
    "attr:Window.showHeaderBackground":{
        ref:"attr:Window.showHeaderBackground",
        flags:"IRA",
        description:"Should the window header show a background image? Default value is true for all browsers\n except for Internet Explorer.<br>\n If set to true the image source is derived from ${isc.DocUtils.linkForRef('attr:Window.headerSrc')} and \n ${isc.DocUtils.linkForRef('attr:Window.hiliteHeaderSrc')}, otherwise the background will be styled according to \n ${isc.DocUtils.linkForRef('attr:Window.headerStyle')} / ${isc.DocUtils.linkForRef('attr:Window.hiliteHeaderStyle')}.",
        name:"showHeaderBackground",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"varies",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:ListGrid.checkboxFieldImageWidth":{
        ref:"attr:ListGrid.checkboxFieldImageWidth",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance')} is set to <code>\"checkbox\"</code> this property\n may be set to govern the width of the checkbox image displayed to indicate whether a row is\n selected. If unset, the checkboxField image will be sized to match the\n ${isc.DocUtils.linkForRef('attr:ListGrid.booleanImageWidth')} for this grid.",
        name:"checkboxFieldImageWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "attr:DynamicForm.minColWidth":{
        ref:"attr:DynamicForm.minColWidth",
        flags:"IRW",
        description:"Minimum width of a form column.",
        name:"minColWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "attr:CubeGrid.hideEmptyFacetValues":{
        ref:"attr:CubeGrid.hideEmptyFacetValues",
        flags:"IRW",
        description:"This causes the headers for any combination of facetValues for which there are no \n cellRecords to be suppressed.\n (Note: Valid only for CubeGrids that specify the complete dataset upfront - \n  don't use load on demand)",
        name:"hideEmptyFacetValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "class:SpacerItem":{
        methods:[],
        ref:"class:SpacerItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:SpacerItem.showTitle", "attr:SpacerItem.width", "attr:SpacerItem.height"],
        description:"A SpacerItem takes up a single cell in the FormLayout, of arbitrary size.",
        name:"SpacerItem",
        seeAlso:[],
        type:"class"
    },
    "attr:FormItem.valueIconRightPadding":{
        ref:"attr:FormItem.valueIconRightPadding",
        flags:"IRW",
        description:"If we're showing a value icon, this attribute governs the amount of space between the \n icon and the label of the form item",
        name:"valueIconRightPadding",
        seeAlso:[
            {ref:"attr:FormItem.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"3",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:Canvas.canDrag":{
        ref:"attr:Canvas.canDrag",
        flags:"IRWA",
        description:"Indicates whether this widget can initiate custom drag-and-drop operations (other than\n      reposition or resize). Normally canDragReposition or canDragResize would be used\n      instead of this property.\n      Note: this property may be manipulated by higher-level dragging semantics.",
        name:"canDrag",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:ColumnTree.nodeIcon":{
        ref:"attr:ColumnTree.nodeIcon",
        flags:"IRW",
        description:"The filename of the default icon for all leaf nodes in this grid. To specify a \n custom image for an individual node, set the ${isc.DocUtils.linkForRef('attr:ColumnTree.customIconProperty')} directly on\n the node.",
        name:"nodeIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]file.gif\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('millerColumns')}"],
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "type:DateDisplayFormat":{
        ref:"type:DateDisplayFormat",
        values:[
            {description:"Default native browser 'toString()' implementation. May vary by browser.<br>\n <i>Example</i>: <code>Fri Nov 04 2005 11:03:00 GMT-0800 (Pacific Standard Time)</code>",
             value:"toString"},
            {description:"Default native browser 'toLocaleString()' implementation. May vary by browser.\n <i>Example</i>: <code>Friday, November 04, 2005 11:03:00 AM</code>",
             value:"toLocaleString"},
            {description:"Short date in format MM/DD/YYYY.<br> <i>Example</i>: <code>11/4/2005</code>",
             value:"toUSShortDate"},
            {description:"Short date with time in format MM/DD/YYYY HH:MM<br> <i>Example</i>: <code>11/4/2005 11:03</code>",
             value:"toUSShortDateTime"},
            {description:"Short date in format DD/MM/YYYY.<br> <i>Example</i>: <code>4/11/2005</code>",
             value:"toEuropeanShortDate"},
            {description:"Short date with time in format DD/MM/YYYY HH:MM<br> <i>Example</i>: <code>4/11/2005 11:03</code>",
             value:"toEuropeanShortDateTime"},
            {description:"Short date in format YYYY/MM/DD.<br> <i>Example</i>: <code>2005/11/4</code>",
             value:"toJapanShortDate"},
            {description:"Short date with time in format YYYY/MM/DD HH:MM<br> <i>Example</i>: <code>2005/11/4 11:03</code>",
             value:"toJapanShortDateTime"},
            {description:"Date in the format YYYY-MM-DD HH:MM:SS<br> <i>Example</i>: <code>2005-11-04 11:09:15</code>",
             value:"toSerializeableDate"},
            {description:"Date in the format &lt;YYYYMMDD&gt;T&lt;HHMMSS&gt;Z <i>Example</i>: <code>20051104T111001Z</code>\n <br>\n <br>\n Note: In addition to these standard formats, custom formatting can be set by passing\n a function directly to ${isc.DocUtils.linkForRef('classMethod:Date.setNormalDisplayFormat')} et al. This \n function will then be executed whenever the appropriate formatter method is called [eg \n ${isc.DocUtils.linkForRef('method:Date.toNormalDate')}], in the scope of the date instance in question.",
             value:"toDateStamp"}
        ],
        description:"Valid display formats for dates.  These strings are the names of formatters which can be \n passed to <code>Date.setNormalDisplayFormat()</code> or <code>Date.setShortDisplayFormat()</code>\n and will be subsequently used as default long or short formatters for date objects by\n SmartClient components.<br>\n Default set of valid display formats is as follows:<br><br>",
        name:"DateDisplayFormat",
        seeAlso:[],
        type:"type"
    },
    "attr:ListGrid.checkboxField":{
        ref:"attr:ListGrid.checkboxField",
        flags:"IR",
        getter:["getCheckboxField"],
        description:"Returns the specially generated checkbox field used when ${isc.DocUtils.linkForRef('type:SelectionAppearance')} is\n \"checkbox\".  Created via the ${isc.DocUtils.linkForRef('type:AutoChild')} pattern so that\n <code>checkboxFieldDefaults</code> and <code>checkboxFieldProperties</code> are available\n for skinning purposes.\n <P>\n This field will render an icon to indicate the selected state of each row, which, when\n clicked will toggle the selection state. The icon src may be configured using \n ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxFieldTrueImage')} and ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxFieldFalseImage')}, as\n well as ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxFieldImageWidth')} and ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxFieldImageHeight')}.\n <P>\n The checkboxField can be detected by calling ${isc.DocUtils.linkForRef('method:ListGrid.isCheckboxField')} on any\n ListGridField object.",
        name:"checkboxField",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "method:Progressbar.setBreadth":{
        ref:"method:Progressbar.setBreadth",
        flags:"",
        description:" Sets the breadth of the progressbar to newLength. This is the height of a horizontal progressbar,\n or the width of a vertical progressbar.\n\n",
        name:"setBreadth",
        seeAlso:[],
        params:[
            {description:"the new breadth of the progressbar", optional:false, name:"newBreadth",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Progressbar"
    },
    "attr:ListGridField.imageURLPrefix":{
        ref:"attr:ListGridField.imageURLPrefix",
        flags:"IRWA",
        description:"If this field has type [${isc.DocUtils.linkForRef('type:ListGridFieldType')}] set to <code>\"image\"</code>\n and the URL for the image displayed is not absolute, the path of the URL will be relative \n to this string<br>\n Alternatively, if this field displays any valueIcons, this prefix will be applied to\n the beginning of any ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')} when determining the\n URL for the image.",
        name:"imageURLPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('imageType')}"],
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "method:Menu.itemClick":{
        ref:"method:Menu.itemClick",
        flags:"A",
        description:"          Executed when a menu item with no click handler is clicked by the user. This\n          itemClick handler must be specified as a function. It is passed an item parameter that\n          is a reference to the clicked menu item.\n\n",
        name:"itemClick",
        seeAlso:[],
        params:[
            {description:"pointer to the item in question", optional:false, name:"item",
             defaultValue:null, type:"object"},
            {description:"Index of the column clicked by the user. May be null if                                  this menu item was activated in response to a keyboard event.",
             optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"false if event processing should be stopped, true to continue", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('menuColumns')}"],
        definingClass:"class:Menu"
    },
    "method:ListGrid.setAlternateRecordStyles":{
        ref:"method:ListGrid.setAlternateRecordStyles",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.alternateRecordStyles')}\n",
        name:"setAlternateRecordStyles",
        seeAlso:[],
        params:[
            {description:"New value for <code>this.alternateRecordStyles</code>", optional:false,
             name:"alternateStyles", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ListGrid.autoFitMaxRecords":{
        ref:"attr:ListGrid.autoFitMaxRecords",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')} is set to <code>\"vertical\"</code> or <code>\"both\"</code> this\n property provides the maximum number of records for which the ListGrid will expand. If more\n records are present, scrolling will be introduced to reach them as normal.\n If unset, by default the ListGrid will expand to accomodate as many records as are present.",
        name:"autoFitMaxRecords",
        setter:["setAutoFitMaxRecords"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"50",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "class:SchemaSet":{
        methods:["method:SchemaSet.getSchema"],
        ref:"class:SchemaSet",
        classAttrs:[],
        classMethods:["classMethod:SchemaSet.get"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:SchemaSet.schemaNamespace"],
        description:"A set of schema derived from the &lt;xsd:schema&gt; element in a WSDL or XML schema file\n loaded by ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')} or ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')}.",
        name:"SchemaSet",
        seeAlso:[],
        type:"class"
    },
    "method:DataBoundComponent.addFormulaField":{
        ref:"method:DataBoundComponent.addFormulaField",
        flags:"",
        description:" Convenience method to display a ${isc.DocUtils.linkForRef('class:FormulaBuilder')} to create a new Formula Field.  This \n is equivalent to calling ${isc.DocUtils.linkForRef('method:DataBoundComponent.editFormulaField','editFormulaField()')} with \n no paramater.\n\n",
        name:"addFormulaField",
        seeAlso:[],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["formulaFields"]
    },
    "method:CubeGrid.saveAllEdits":{
        ref:"method:CubeGrid.saveAllEdits",
        flags:"",
        description:" Save a number of outstanding edits for this CubeGrids. If no cells are specifed, all \n outstanding edits will be saved\n",
        name:"saveAllEdits",
        seeAlso:[],
        params:[
            {description:"optionally specify which cells to save. Each cell  should be specified as a 2 element array in the format <code>[rowNum,colNum]</code>.",
             optional:true, name:"cells", defaultValue:null, type:"array of 2-element arrays"},
            {description:"If specified this callback will be fired on a successful save  of the specified rows. Note that if there are no pending edits to be saved this callback will\n  not fire - you can check for this condition using ${isc.DocUtils.linkForRef('method:CubeGrid.hasChanges')} or \n  ${isc.DocUtils.linkForRef('method:CubeGrid.recordHasChanges')}.",
             optional:true, name:"saveCallback", defaultValue:null, type:"callback"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "classAttr:Date.weekendDays":{
        ref:"classAttr:Date.weekendDays",
        flags:"IR",
        description:"Days that are considered \"weekend\" days.   Values should be the integers returned by the\n JavaScript built-in Date.getDay(), eg, 0 in Sunday and 6 is Saturday.  Override to\n accomodate different workweeks such as Saudi Arabia (Saturday -> Wednesday) or Israel \n (Sunday -> Thurday).",
        name:"weekendDays",
        seeAlso:[],
        valueType:"Array of int",
        defaultValue:"[0, 6]",
        type:"classAttr",
        definingClass:"class:Date"
    },
    "attr:MenuItem.icon":{
        ref:"attr:MenuItem.icon",
        flags:"IR",
        description:"The base filename for this item's custom icon. If both this property and\n ${isc.DocUtils.linkForRef('attr:MenuItem.checked')} are both specified, only icon specified by this property will be\n displayed. The path to the loaded skin directory and the skinImgDir are prepended to\n this filename to form the full URL.\n <p>\n If you need to set this state dynamically, use ${isc.DocUtils.linkForRef('method:MenuItem.dynamicIcon')} instead.",
        name:"icon",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"],
        definingClass:"object:MenuItem",
        groups:["menuIcons"]
    },
    "attr:ViewLoader.viewRPCProperties":{
        ref:"attr:ViewLoader.viewRPCProperties",
        flags:"IRA",
        description:"RPCRequest properties to be sent with every RPCRequest issued by the ViewLoader.  Very\n advanced; could be used to, for example, set HTTP headers.",
        name:"viewRPCProperties",
        seeAlso:[],
        valueType:"RPCRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ViewLoader"
    },
    "method:Canvas.dropOut":{
        ref:"method:Canvas.dropOut",
        flags:"A",
        description:"\n Executed when the dragged object is moved out of the rectangle of this drop target. If you\n have set a visual indication in dropOver or dropMove, you should reset it to its normal\n state in dropOut.\n\n",
        name:"dropOut",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customDrag')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:ListGridField.showFileInline":{
        ref:"attr:ListGridField.showFileInline",
        flags:"IR",
        description:"For a field of type:\"imageFile\", indicates whether to stream the image and display it\n inline or to display the View and Download icons.",
        name:"showFileInline",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:TreeGrid.setData":{
        ref:"method:TreeGrid.setData",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('class:Tree')} object this TreeGrid will view and manipulate.\n\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"Tree to show", optional:false, name:"newData", defaultValue:null, type:"Tree"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:MultiSortPanel.propertyFieldTitle":{
        ref:"attr:MultiSortPanel.propertyFieldTitle",
        flags:"IR",
        description:"The title-text to appear in the header of the \"property\" field.",
        name:"propertyFieldTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Column\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "attr:Slider.thumbThinWidth":{
        ref:"attr:Slider.thumbThinWidth",
        flags:"IRW",
        description:"The dimension of the thumb parallel to the slider track.",
        name:"thumbThinWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"17",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:ListGrid.dragDataAction":{
        ref:"attr:ListGrid.dragDataAction",
        flags:"IRW",
        description:"Indicates what to do with data dragged into another DataBoundComponent. See\n          DragDataAction type for details.",
        name:"dragDataAction",
        seeAlso:[],
        valueType:"DragDataAction",
        defaultValue:"Canvas.MOVE",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('gridsDragMove')}", 
         "${isc.DocUtils.linkForExampleId('gridsDragCopy')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "class:EdgedCanvas":{
        methods:[],
        ref:"class:EdgedCanvas",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:EdgedCanvas.skinImgDir", "attr:EdgedCanvas.edgeImage", "attr:EdgedCanvas.edgeColor", 
         "attr:EdgedCanvas.customEdges","attr:EdgedCanvas.showCenter", "attr:EdgedCanvas.edgeSize", "attr:EdgedCanvas.edgeLeft", 
         "attr:EdgedCanvas.edgeRight","attr:EdgedCanvas.edgeTop", "attr:EdgedCanvas.edgeBottom", "attr:EdgedCanvas.edgeOffset", 
         "attr:EdgedCanvas.edgeOffsetTop","attr:EdgedCanvas.edgeOffsetBottom", "attr:EdgedCanvas.edgeOffsetLeft", 
         "attr:EdgedCanvas.edgeOffsetRight","attr:EdgedCanvas.centerBackgroundColor", "attr:EdgedCanvas.edgeStyleName", 
         "attr:EdgedCanvas.addEdgeStyleSuffix"],
        description:"EdgedCanvas acts as a decorative, image-based frame around another single Canvas.",
        name:"EdgedCanvas",
        seeAlso:[],
        type:"class",
        groups:["imageEdges"]
    },
    "attr:Window.canDragReposition":{
        ref:"attr:Window.canDragReposition",
        flags:"IRW",
        description:"If true, this Window may be moved around by the user by dragging on the Window header.  \n Note that if the header is not showing, the Window can't be drag-repositioned regardless\n of this setting.",
        name:"canDragReposition",
        seeAlso:[
            {ref:"attr:Window.showHeader", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["dragging"]
    },
    "attr:Scrollbar.autoEnable":{
        ref:"attr:Scrollbar.autoEnable",
        flags:"IRWA",
        description:"If true, this scrollbar will automatically enable when the scrollTarget is\n scrollable (i.e., when the contents of the scrollTarget exceed its clip size in the\n direction relevant to this scrollbar), and automatically disable when the\n scrollTarget is not scrollable. Set this property to false for full manual control\n over a scrollbar's enabled state.",
        name:"autoEnable",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Scrollbar"
    },
    "method:Button.setAutoFit":{
        ref:"method:Button.setAutoFit",
        flags:"",
        description:" Setter method for the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.autoFit')} property. Pass in true or false to turn\n autoFit on or off. When autoFit is set to <code>false</code>, canvas will be resized to\n it's previously specified size.\n",
        name:"setAutoFit",
        seeAlso:[],
        params:[
            {description:"New autoFit setting.", optional:false, name:"autoFit", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Button"
    },
    "method:ListGrid.setBodyStyleName":{
        ref:"method:ListGrid.setBodyStyleName",
        flags:"",
        description:" Update the ${isc.DocUtils.linkForRef('attr:ListGrid.bodyStyleName','bodyStyleName')} for this listGrid.\n",
        name:"setBodyStyleName",
        seeAlso:[],
        params:[
            {description:"new body style name", optional:false, name:"styleName", defaultValue:null,
             type:"CSSStyleName"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:StretchImgButton.wrap":{
        ref:"attr:StretchImgButton.wrap",
        flags:"IRW",
        description:"Should the title for this button wrap? If unset, default behavior is to allow wrapping\n if this.vertical is true, otherwise disallow wrapping",
        name:"wrap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "method:Canvas.resizeTo":{
        ref:"method:Canvas.resizeTo",
        flags:"",
        description:"			Resizes the widget to the specified width and height (moves the right and/ or bottom\n          sides of the widget). The width and height parameters can be expressed as a percentage\n          of viewport size or as the number of pixels.\n",
        name:"resizeTo",
        seeAlso:[],
        params:[
            {description:"new width for canvas", optional:true, name:"width", defaultValue:null,
             type:"number"},
            {description:"new height for canvas", optional:true, name:"height", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether the size actually changed", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('resize')}"],
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "classAttr:RPCManager.useCursorTracker":{
        ref:"classAttr:RPCManager.useCursorTracker",
        flags:"IRW",
        description:"If true, an image is shown to the right of the cursor when ${isc.DocUtils.linkForRef('attr:RPCRequest.promptStyle')} is\n set to \"cursor\", otherwise the cursor itself is modified via css to the value of\n ${isc.DocUtils.linkForRef('attr:RPCRequest.promptCursor')}.  The default is platform-dependent.  In Safari, IE 5.5 and\n Firefox 1.0 the default is true, on all other platforms it is false.  The reason for\n this split is that, the above browsers require that the cursor move before CSS settings are\n re-evaluated - this means the progress cursor can stick until the user moves the mouse.\n <p>\n This value can be overridden on a per-request basis via ${isc.DocUtils.linkForRef('classAttr:RPCRequest.useCursorTracker')}.",
        name:"useCursorTracker",
        seeAlso:[
            {ref:"classAttr:RPCRequest.useCursorTracker", description:null}
        ],
        valueType:"boolean",
        defaultValue:"platform-dependent",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["rpcPrompt"]
    },
    "attr:ListGrid.headerMenuButtonHeight":{
        ref:"attr:ListGrid.headerMenuButtonHeight",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, this property governs the height of the \n auto-generated <code>headerMenuButton</code>",
        name:"headerMenuButtonHeight",
        seeAlso:[],
        valueType:"measure",
        defaultValue:"\"100%\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "attr:ListGrid.gridSummaryRecordProperty":{
        ref:"attr:ListGrid.gridSummaryRecordProperty",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, this attribute will be set to true on the\n record object representing the grid summary row.",
        name:"gridSummaryRecordProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"isGridSummary\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:DateChooser.prevMonthIconHeight":{
        ref:"attr:DateChooser.prevMonthIconHeight",
        flags:"IR",
        description:"Height of the icon for the previous month button",
        name:"prevMonthIconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:Canvas.getInnerHTML":{
        ref:"method:Canvas.getInnerHTML",
        flags:"A",
        description:"      Return the inner HTML for this canvas. Called when the canvas is drawn or redrawn; \n      override to customize.\n",
        name:"getInnerHTML",
        seeAlso:[],
        returns:{description:"HTML contents of this canvas", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "classAttr:Operators.iNotEndsWithTitle":{
        ref:"classAttr:Operators.iNotEndsWithTitle",
        flags:"IR",
        description:"Title for the \"iNotEndsWith\" operator",
        name:"iNotEndsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not end with\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classAttr:ValidatorTypes.isString":{
        ref:"classAttr:ValidatorTypes.isString",
        flags:"IR",
        description:"Validation will fail if the value is not a string value.",
        name:"isString",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:Window.canFocusInHeaderButtons":{
        ref:"attr:Window.canFocusInHeaderButtons",
        flags:"IRWA",
        description:"If true, the user can give the header buttons keyboard focus (by clicking on\n      them and including them in the tabOrder)",
        name:"canFocusInHeaderButtons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["focus", "header"]
    },
    "attr:ColorPicker.defaultOpacity":{
        ref:"attr:ColorPicker.defaultOpacity",
        flags:"IR",
        description:"The initial opacity value for the component, as a percentage value between 0 and 100",
        name:"defaultOpacity",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ToolStrip.members":{
        ref:"attr:ToolStrip.members",
        flags:"IR",
        description:"Array of components that will be contained within this Toolstrip, like\n ${isc.DocUtils.linkForRef('attr:Layout.members')}, with the following special behaviors:\n <ul>\n <li>the String \"separator\" will cause a separator to be created (instance of \n ${isc.DocUtils.linkForRef('attr:ToolStrip.separatorClass')})\n <li>the String \"resizer\" will cause a resizer to be created (instance of\n ${isc.DocUtils.linkForRef('attr:ToolStrip.resizeBarClass')}).  This is equivalent to setting\n ${isc.DocUtils.linkForRef('attr:Canvas.showResizeBar','showResizeBar:true')} on the preceding member.\n </ul>",
        name:"members",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('toolstrip')}"],
        definingClass:"class:ToolStrip"
    },
    "attr:Toolbar.buttonDefaults":{
        ref:"attr:Toolbar.buttonDefaults",
        flags:"IRWA",
        description:"Settings to apply to all buttons of a toolbar. Properties that can be applied to\n button objects can be applied to all buttons of a toolbar by specifying them in\n buttonDefaults using the following syntax:<br>\n <code>buttonDefaults:{property1:value1, property2:value2, ...}</code><br>\n See the Button Widget Class for standard button properties.",
        name:"buttonDefaults",
        seeAlso:[
            {ref:"class:Button", description:null}
        ],
        valueType:"object",
        defaultValue:"varies",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["appearance"]
    },
    "method:GridRenderer.findColNum":{
        ref:"method:GridRenderer.findColNum",
        flags:"",
        description:" Given a record displayed in this grid, find and return the colNum in which the record\n appears.\n <P>\n As with ${isc.DocUtils.linkForRef('method:GridRenderer.getCellRecord')} implementing this method is optional as a valid\n grid may be created without any notion of records, or records may not be displayed in a\n single column (as with the ${isc.DocUtils.linkForRef('class:ListGrid','ListGrid')} class where each record is \n displayed in an entire row.  \n\n",
        name:"findColNum",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"index of the column containing the record or -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:CheckboxItem.allowEmptyValue":{
        ref:"attr:CheckboxItem.allowEmptyValue",
        flags:"IRW",
        description:"By default checkboxes allow the user to toggle between true and false values only.\n Setting this property to true will allow the user to toggle between three values - \n <code>true</code>, <code>false</code> and unset.",
        name:"allowEmptyValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CheckboxItem"
    },
    "attr:RPCRequest.evalResult":{
        ref:"attr:RPCRequest.evalResult",
        flags:"IRWA",
        description:"This works similarly to ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} except the resulting String\n is automatically evaluated as JavaScript.  The result of the evaluation is then passed to\n any specified ${isc.DocUtils.linkForRef('attr:RPCRequest.callback')} as ${isc.DocUtils.linkForRef('attr:RPCResponse.data')}.\n <p>\n This feature can be used to dynamically load new application modules into a running\n application.  An RPCRequest with <code>evalResult</code> enabled can be used to fetch a\n static .js file or JavaScript dynamically generated by the server.  The returned JavaScript\n can contain anything that a JavaScript file loaded at init time can contain, including new\n views and new SmartClient class definitions.\n <p>\n <i>Example usage with ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendRequest')}:</i>\n <pre>\n isc.RPCManager.sendRequest({\n     evalResult:true,\n     actionURL:\"js/loadLabel.js\",\n     evalVars:{var1:\"A Value\"}\n });\n </pre>\n This call would execute the code from <code>loadLabel.js</code>, and make the variable\n <code>var1</code> available to that code. Therefore if the .js file contained this code:\n <pre>\n isc.Label.create({\n     contents:var1\n })\n </pre>\n A label would be created with contents set to the value of <code>var1</code> - the string\n <code>\"A Value\"</code>.\n \n <p>\n This feature relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.",
        name:"evalResult",
        seeAlso:[
            {ref:"class:ViewLoader", description:null},
            {ref:"attr:RPCRequest.evalVars", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["viewLoading"]
    },
    "method:PanelHeader.setBaseStyle":{
        ref:"method:PanelHeader.setBaseStyle",
        flags:"",
        description:" Sets the base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n",
        name:"setBaseStyle",
        seeAlso:[],
        params:[
            {description:"new base style", optional:false, name:"style", defaultValue:null,
             type:"className"}
        ],
        type:"method",
        definingClass:"class:PanelHeader"
    },
    "type:DrawPosition":{
        ref:"type:DrawPosition",
        values:[
            {description:"insert before the target element", value:"\"beforeBegin\""},
            {description:"insert as the target element's first child", value:"\"afterBegin\""},
            {description:"insert as the target element's last child", value:"\"beforeEnd\""},
            {description:"insert after the target element", value:"\"afterEnd\""},
            {description:"replace the target element", value:"\"replace\""}
        ],
        description:null,
        name:"DrawPosition",
        seeAlso:[],
        type:"type",
        groups:["drawContext"]
    },
    "attr:FormulaBuilder.cancelButton":{
        ref:"attr:FormulaBuilder.cancelButton",
        flags:"IR",
        description:"Button to Cancel this FormulaBuilder.  The formula is not tested, formulaBuilder.cancelled\n is set to true and formulaBuilder.fireOnClose is fired.",
        name:"cancelButton",
        seeAlso:[],
        valueType:"AutoChild Button",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:Menu.showIcons":{
        ref:"attr:Menu.showIcons",
        flags:"IRW",
        description:"A boolean, indicating whether the checkmark/custom icon column should be displayed.\n If showIcons is not set, the menu will show the icon column only if one of its items\n specifies an icon, checked, checkIf, or dynamicIcon property.",
        name:"showIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:FormItem.showFocusedIcons":{
        ref:"attr:FormItem.showFocusedIcons",
        flags:"IRWA",
        description:"If we're showing icons, should we change their image source to the appropriate\n  <i>focused</i>  source when this item has focus?  Can be overridden on a per\n  icon basis by the formItemIcon <code>showFocused</code> property.",
        name:"showFocusedIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:DynamicForm.saveOperationType":{
        ref:"attr:DynamicForm.saveOperationType",
        flags:"IRW",
        getter:["getSaveOperationType"],
        description:"Default ${isc.DocUtils.linkForRef('type:DSOperationType')} to be performed when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is called.\n This property is automatically set on a call to ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} or\n ${isc.DocUtils.linkForRef('method:DynamicForm.editNewRecord')}, or may be set directly via \n ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')}.\n <P>\n If <code>saveOperationType</code> is unset, the form will heuristically determine\n whether an \"add\" or \"update\" operation is intended based on whether the primaryKey field\n is present and editable.",
        name:"saveOperationType",
        setter:["setSaveOperationType"],
        seeAlso:[],
        valueType:"DSOperationType",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "attr:ListGrid.headerSpanVAlign":{
        ref:"attr:ListGrid.headerSpanVAlign",
        flags:"IR",
        description:"Default alignment for ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpans')} with no\n ${isc.DocUtils.linkForRef('attr:HeaderSpan.valign')} specified.",
        name:"headerSpanVAlign",
        seeAlso:[],
        valueType:"vAlign",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "method:CubeGrid.getEditedCell":{
        ref:"method:CubeGrid.getEditedCell",
        flags:"",
        description:" Returns the current value of a cell. If the cell has an outstanding edit value, this will\n be returned, otherwise the underlying value of the record will be returned.\n\n",
        name:"getEditedCell",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"record", defaultValue:null, type:"number | Object"},
            {description:"colNum of the cell. Only required if the first parameter is                              a rowNum",
             optional:false, name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"Current edit value, or underlying value for the cell.", type:"any"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:Slider.trackImageType":{
        ref:"attr:Slider.trackImageType",
        flags:"IRW",
        description:"The imageType setting for the slider track.",
        name:"trackImageType",
        seeAlso:[
            {ref:"type:ImageStyle", description:null},
            {ref:"attr:StretchImg.imageType", description:null}
        ],
        valueType:"ImageStyle",
        defaultValue:"\"stretch\"",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:FilterBuilder.radioOptions":{
        ref:"attr:FilterBuilder.radioOptions",
        flags:"IR",
        description:"Logical operators to allow if we have a ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')} of \"radio\".",
        name:"radioOptions",
        seeAlso:[],
        valueType:"Array of OperatorId",
        defaultValue:"[\"and\", \"or\", \"not\"]",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:TreeNode.id":{
        ref:"attr:TreeNode.id",
        flags:"IR",
        description:"Specifies the unique ID of this node.  \n <P>\n Required for trees with ${isc.DocUtils.linkForRef('attr:Tree.modelType')} \"parent\".  With modelType:\"parent\", the unique\n ID of a node, together with the unique ID of its parent (see ${isc.DocUtils.linkForRef('attr:TreeNode.parentId')}) is\n used by ${isc.DocUtils.linkForRef('method:Tree.linkNodes')} to link a list of nodes into a tree.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.idField')}.",
        name:"id",
        seeAlso:[
            {ref:"attr:TreeNode.parentId", description:null},
            {ref:"method:Tree.linkNodes", description:null},
            {ref:"attr:Tree.modelType", description:null},
            {ref:"attr:Tree.idField", description:null}
        ],
        valueType:"String or Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:OperationBinding.qualifyColumnNames":{
        ref:"attr:OperationBinding.qualifyColumnNames",
        flags:"IR",
        description:"Specifies, for this specific operationBinding, whether to qualify column names with table\n names in any SQL we generate.  Overrides the ${isc.DocUtils.linkForRef('attr:DataSource.qualifyColumnNames')} property.\n Only applicable to dataSources of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\".",
        name:"qualifyColumnNames",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding"
    },
    "attr:FormItem.filterLocally":{
        ref:"attr:FormItem.filterLocally",
        flags:"IRA",
        description:"If this form item is mapping data values to a display value by fetching them from a \n dataSource (see ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}, ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} \n and ${isc.DocUtils.linkForRef('attr:FormItem.fetchMissingValues')}), setting this property to true ensures that when\n the form item value is set, entire data-set from the dataSource is loaded at once and \n used as a valueMap, rather than just loading the display value for the current value.\n This avoids the need to perform fetches each time setValue() is called with a new value.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:PickList.filterLocally')} for behavior on form items such as SelectItems\n that show pick-lists.",
        name:"filterLocally",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "attr:Canvas.matchElement":{
        ref:"attr:Canvas.matchElement",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.htmlElement')} is specified, should this canvas initially be drawn\n at the same dimensions as the htmlElement?<br>\n Note: setting this property will not force the canvas to resize if the element\n subsequently resizes (for example due to page reflow).",
        name:"matchElement",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "method:CubeGrid.getSelectedCellIds":{
        ref:"method:CubeGrid.getSelectedCellIds",
        flags:"",
        description:"\n  return an array of the IDs of all selected cell records\n\n  <br><i>methodType</i> getter\n",
        name:"getSelectedCellIds",
        seeAlso:[],
        returns:{description:"array of the selected cell IDs", type:"Array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:DSRequest.exportFooter":{
        ref:"attr:DSRequest.exportFooter",
        flags:"IR",
        description:"Optional text to appear at the end of the file.",
        name:"exportFooter",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:ListGrid.bodyStyleName":{
        ref:"attr:ListGrid.bodyStyleName",
        flags:"IR",
        description:"CSS style used for the body of this grid.  If applying a background-color to the body\n via a CSS style applied using this property, be sure to set \n ${isc.DocUtils.linkForRef('attr:ListGrid.bodyBackgroundColor')} to <code>null</code>.",
        name:"bodyStyleName",
        setter:["setBodyStyleName"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "classMethod:DataSource.getSortBy":{
        ref:"classMethod:DataSource.getSortBy",
        flags:"",
        description:" Given an array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s, return a simple list of Strings in the format\n expected by ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')}.\n\n",
        name:"getSortBy",
        seeAlso:[],
        params:[
            {description:"The list of specifiers to return in   sortBy format", optional:false,
             name:"sortSpecifiers", defaultValue:null, type:"Array of SortSpecifier"}
        ],
        returns:{description:"An array of sort-definitions in the format expected by  ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')}",
         type:"Array of String"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "method:List.getValueMap":{
        ref:"method:List.getValueMap",
        flags:"",
        description:" Get a map of the form <code>{ item[idField] -&gt; item[displayField] }</code>, for all \n items in the list.  Note that if more than one item has the same <code>idProperty</code>, \n the value for the later item in the list will clobber the value for the earlier item.\n\n",
        name:"getValueMap",
        seeAlso:[],
        params:[
            {description:"Property to use as ID (data value) in the valueMap", optional:false,
             name:"idField", defaultValue:null, type:"string"},
            {description:"Property to use a display value in the valueMap", optional:false,
             name:"displayField", defaultValue:null, type:"string"}
        ],
        returns:{description:"valueMap object", type:"object"},
        type:"method",
        definingClass:"interface:List"
    },
    "method:ListGrid.recordClick":{
        ref:"method:ListGrid.recordClick",
        flags:"",
        description:" Executed when the listGrid receives a 'click' event on an enabled, non-separator\n record. The default implementation does nothing -- override to perform some action\n when any record or field is clicked.<br>\n A record event handler can be specified either as\n a function to execute, or as a string of script to evaluate. If the handler is defined\n as a string of script, all the parameters below will be available as variables for use\n in the script.<br>\n	To do something specific if a particular field is clicked, add a recordClick\n	method or string of script to that field (same parameters) when you're setting up\n	the list.<br>\n	<b>Notes:</b><ul>\n  <li>This will not be called if the click is below the last item of the list.</li>\n  <li>This method is called from the default implementaiton of\n  ${isc.DocUtils.linkForRef('method:ListGrid.rowClick')}, so if that method is overridden\n  this method may not be fired.</li></ul>\n",
        name:"recordClick",
        seeAlso:[
            {ref:"method:ListGrid.rowClick", description:null}
        ],
        params:[
            {description:"the listGrid that contains the click event", optional:false, name:"viewer",
             defaultValue:null, type:"listGrid"},
            {description:"the record that was clicked on", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"number of the record clicked on in the current set of                                  displayed records (starts with 0)",
             optional:false, name:"recordNum", defaultValue:null, type:"number"},
            {description:"the field that was clicked on (field definition)", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"number of the field clicked on in the listGrid.fields                                  array",
             optional:false, name:"fieldNum", defaultValue:null, type:"number"},
            {description:"value of the cell (after valueMap, etc. applied)", optional:false,
             name:"value", defaultValue:null, type:"object"},
            {description:"raw value of the cell (before valueMap, etc applied)", optional:false,
             name:"rawValue", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:Menu.setItemTitle":{
        ref:"method:Menu.setItemTitle",
        flags:"",
        description:" Sets the title of a particular menu item to the string specified by newTitle and\n redraws the menu if necessary.\n\n",
        name:"setItemTitle",
        seeAlso:[],
        params:[
            {description:"MenuItem in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem or number"},
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        returns:{description:"true if the title was changed, and false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:Menu"
    },
    "attr:ServerObject.attributeScope":{
        ref:"attr:ServerObject.attributeScope",
        flags:"IR",
        description:"Specifies the scope in which the DMI instance is to be looked up.  Valid values are: \n <code>\"request\"</code>, <code>\"session\"</code>, and <code>\"application\"</code>.  If\n <code>attributeScope</code> is left out of the <code>ServerObject</code> definition, then\n all scopes are searched in the order in which they are listed above.\n <p>\n This attribute is consulted only when the value of ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')} is\n <code>\"attribute\"</code>.",
        name:"attributeScope",
        seeAlso:[
            {ref:"attr:ServerObject.attributeName", description:null},
            {ref:"attr:ServerObject.lookupStyle", description:null}
        ],
        valueType:"\"request\"|\"session\"|\"application\"",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "attr:ListGrid.groupByText":{
        ref:"attr:ListGrid.groupByText",
        flags:"IRWA",
        getter:["getGroupByText"],
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canGroupBy','this.canGroupBy')} is true, this string will be shown as the\n title for the menu item to toggle the group by setting for a field.\n <P>\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>viewer</code> available as a variable mapped to\n the ListGrid instance, and <code>field</code> as a variable pointing to the ListGrid field.\n <P>\n Default value returns \"Group by \" + the field's summary title.",
        name:"groupByText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Group by \\${title}\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "classAttr:RPCRequest.useCursorTracker":{
        ref:"classAttr:RPCRequest.useCursorTracker",
        flags:"IRW",
        description:"If true, an image is shown to the right of the cursor when ${isc.DocUtils.linkForRef('attr:RPCRequest.promptStyle')} is\n set to \"cursor\", otherwise the cursor itself is modified via css to the value of\n ${isc.DocUtils.linkForRef('attr:RPCRequest.promptCursor')}.  The default is platform-dependent.  In Safari, IE 5.5 and\n Firefox 1.0 the default is true, on all other platforms it is false.  The reason for this\n split is that the above browsers require that the cursor move before CSS settings are\n re-evaluated - this means the progress cursor can stick until the user moves the mouse.\n <p>\n If left unspecified, the default value is set by ${isc.DocUtils.linkForRef('classAttr:RPCManager.useCursorTracker')}.",
        name:"useCursorTracker",
        seeAlso:[
            {ref:"classAttr:RPCManager.useCursorTracker", description:null}
        ],
        valueType:"boolean",
        defaultValue:"platform-dependent",
        type:"classAttr",
        definingClass:"class:RPCRequest",
        groups:["rpcPrompt"]
    },
    "attr:TreeGrid.showDropIcons":{
        ref:"attr:TreeGrid.showDropIcons",
        flags:"IRW",
        description:"If true, when the user drags a droppable target over a folder in this TreeGrid, show \n a different icon folder icon.\n This is achieved by appending the ${isc.DocUtils.linkForRef('attr:TreeGrid.dropIconSuffix')} onto the\n ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} URL (for example <code>\"[SKIN]/folder.gif\"</code> may be\n replaced by <code>\"[SKIN]/folder_drop.gif\"</code>).",
        name:"showDropIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:ListGrid.invalidSummaryValue":{
        ref:"attr:ListGrid.invalidSummaryValue",
        flags:"IRWA",
        description:"Value to display to the user if showing summary values (through ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')},\n ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} or ${isc.DocUtils.linkForRef('type:ListGridFieldType','listGridFieldType:\"summary\"')}), and\n the summary function returns <code>\"null\"</code> (implying it was unable to calculate a\n valid summary value).",
        name:"invalidSummaryValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&amp;nbsp;\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Applet.getPluginHandle":{
        ref:"method:Applet.getPluginHandle",
        flags:"A",
        description:" \n  Returns a handle to the applet DOM element (valid only after the component has been drawn).  \n\n",
        name:"getPluginHandle",
        seeAlso:[],
        type:"method",
        definingClass:"class:Applet"
    },
    "attr:DynamicForm.nestedListEditorType":{
        ref:"attr:DynamicForm.nestedListEditorType",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('class:FormItem')} class to use for any list-type complex fields on this DynamicForm.\n List-type fields are denoted by marking them <code>multiple: true</code> in the \n DataSource.",
        name:"nestedListEditorType",
        seeAlso:[
            {ref:"attr:DynamicForm.nestedEditorType", description:null}
        ],
        valueType:"String",
        defaultValue:"\"NestedListEditorItem\"",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "attr:Canvas.cursor":{
        flags:"IRWA",
        type:"attr",
        ref:"attr:Canvas.cursor",
        description:"Specifies the cursor image to display when the mouse pointer is\n      over this widget. It corresponds to the CSS cursor attribute. See Cursor type for\n      different cursors.",
        name:"cursor",
        setter:["setCursor"],
        valueType:"Cursor",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}", 
         "${isc.DocUtils.linkForExampleId('cursors')}"],
        defaultValue:"Canvas.DEFAULT",
        groups:["cues"],
        definingClass:"class:Canvas"
    },
    "method:TileGrid.filterData":{
        ref:"method:TileGrid.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:FormulaBuilder.autoTestDelay":{
        ref:"attr:FormulaBuilder.autoTestDelay",
        flags:"IRWA",
        description:"When ${isc.DocUtils.linkForRef('attr:FormulaBuilder.autoTest')} is true, this property indicates the delay in \n milliseconds between a user pausing and ${isc.DocUtils.linkForRef('method:FormulaBuilder.testFunction','testFunction()')}\n being called.\n <P>\n The default is 200 milliseconds.",
        name:"autoTestDelay",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"200",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:StretchImgButton.getState":{
        ref:"method:StretchImgButton.getState",
        flags:"",
        description:" Return the state of this StatefulCanvas\n",
        name:"getState",
        seeAlso:[],
        returns:{description:null, type:"State"},
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "classMethod:Class.getStackTrace":{
        ref:"classMethod:Class.getStackTrace",
        flags:"",
        description:" Returns a \"stack trace\" - one line per method in the current call stack, showing the method\n name and any parameters passed.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getStackTrace\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n <P>\n This API can currently only be supported in Internet Explorer.  See\n ${isc.DocUtils.linkForRef('group:debugging')} for information on how to get equivalent information from other\n browsers.\n\n",
        name:"getStackTrace",
        seeAlso:[],
        returns:{description:"stack trace.  Use eg ${isc.DocUtils.linkForRef('method:Class.logWarn')} to log to the Developer Console.",
         type:"String"},
        type:"classMethod",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:DataSource.fetchData":{
        ref:"method:DataSource.fetchData",
        flags:"",
        description:" Perform a \"fetch\" DataSource operation against this DataSource, sending search criteria\n and retrieving matching records.\n <P>\n In contrast to ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}, which creates a ${isc.DocUtils.linkForRef('class:ResultSet')} to manage\n the returned data, calling <code>dataSource.fetchData()</code> provides the returned\n data in the callback as a simple JavaScript Array of JavaScript Objects.  Calling\n <code>dataSource.fetchData()</code> does not automatically update any visual components or\n caches: code in the callback passed to <code>fetchData()</code> decides what to do with\n the returned data. \n <P>\n For example, given a ListGrid \"myGrid\" and a DataSource \"employees\", the following code\n would populate \"myGrid\" with data fetched from the DataSource:\n <pre>\n    isc.DataSource.get(\"employees\").fetchData(null, \"myGrid.setData(data)\");\n </pre>\n Unlike calling <code>myGrid.fetchData()</code>, which creates a ${isc.DocUtils.linkForRef('class:ResultSet')}, the\n data provided to the grid is \"disconnected\" data, unmanaged by SmartClient's databinding\n facilities and safe to directly modify.  This is useful when, for example, a ListGrid is\n being used as a more sophisticated version of HTML's multi-select component.\n <P>\n Disconnected datasets may be used to populate various visual components.  For example,\n while an individual FormItem can be configured to fetch\n ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} options from a DataSource via the\n ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')} property, the following code shows\n storing a dataset to derive valueMaps from later:\n <pre>\n    isc.DataSource.get(\"countries\").fetchData(null, \"window.countries = data\");\n\n    ... later, a form is created dynamically ...\n\n    function showForm() {\n       isc.DynamicForm.create({\n           items : [\n              { name:\"country\", title:\"Pick Country\",\n                valueMap: window.countries.getValueMap(\"countryId\", \"countryName\")\n              },\n       ...\n </pre>\n <P>\n You can also create a ResultSet from the data retrieved from <code>fetchData()</code>,\n like so:\n <pre>\n    isc.DataSource.get(\"countries\").fetchData(null,\n        function (dsResponse, data) {\n           isc.ResultSet.create({\n              dataSource:\"countries\",\n              allRows:data\n           })\n        }\n    )\n </pre>\n <P>\n This gives you a dataset that supports client-side filtering (via\n ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')}), can provide \n ${isc.DocUtils.linkForRef('method:ResultSet.getValueMap','filtered valueMaps')}, will \n ${isc.DocUtils.linkForRef('attr:ResultSet.disableCacheSync','automatically reflect updates')} to the DataSource made via\n other components, and can be re-used with multiple visual components.\n <P>\n See also the server-side com.isomorphic.js.JSTranslater class in the\n ${isc.DocUtils.linkForDocNode('javaServerReference', 'Java Server Reference')} for other, similar approaches\n involving dumping data into the page during initial page load.  <b>Note:</b> care should\n be taken when using this approach.  Large datasets degrade the basic performance of some\n browsers, so use ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource','optionDataSource')} and similar\n facilities to manage datasets that may become very large.\n <P>\n <b>Data-Driven Visual Component Creation</b>\n <P>\n <code>DataSource.fetchData()</code> can also be used to create SmartClient components in\n a data-driven way.  Many properties on SmartClient visual components are configured via\n an Array of Objects - the same data format that <code>dataSource.fetchData()</code>\n returns.  These include ${isc.DocUtils.linkForRef('attr:ListGrid.fields')}, ${isc.DocUtils.linkForRef('attr:TabSet.tabs')},\n ${isc.DocUtils.linkForRef('attr:DynamicForm.items')}, ${isc.DocUtils.linkForRef('attr:Facet.values')} and even ${isc.DocUtils.linkForRef('attr:DataSource.fields')}.\n <P>\n For example, if you had a DataSource \"myFormFields\" whose fields included the\n basic properties of ${isc.DocUtils.linkForRef('class:FormItem','FormItems')} (name, title, type, etc), this example\n code would create a form based on stored field definitions, loaded from the\n \"myFormFields\" DataSource on the fly:\n <pre>\n    isc.DataSource.get(\"myFormFields\").fetchData(null, \n        \"isc.DynamicForm.create({ items:data })\"\n    )\n </pre>\n This capability to dynamically create visual components from dynamically fetched data\n provides a foundation for creating interfaces that can be customized by end users.\n See also ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "classAttr:Log.defaultPriority":{
        ref:"classAttr:Log.defaultPriority",
        flags:"IRWA",
        getter:["getDefaultPriority"],
        description:"Any logs below this priority will be suppressed, unless a more specific setting exists for\n the category.",
        name:"defaultPriority",
        setter:["setDefaultPriority"],
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        valueType:"LogPriority",
        defaultValue:"Log.WARN",
        type:"classAttr",
        definingClass:"class:Log"
    },
    "classAttr:Operators.notEqualTitle":{
        ref:"classAttr:Operators.notEqualTitle",
        flags:"IR",
        description:"Title for the \"notEqual\" operator",
        name:"notEqualTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"not equal\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:DateChooser.todayButtonTitle":{
        ref:"attr:DateChooser.todayButtonTitle",
        flags:"IRW",
        description:"Title for \"Today\" button.",
        name:"todayButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Today\"",
        type:"attr",
        definingClass:"class:DateChooser",
        groups:["i18nMessages"]
    },
    "method:ListGrid.cellOut":{
        ref:"method:ListGrid.cellOut",
        flags:"",
        description:" Called when the mouse pointer leaves a cell\n\n",
        name:"cellOut",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:Canvas.skinImgDir":{
        ref:"attr:Canvas.skinImgDir",
        flags:"IRWA",
        description:"Default directory for skin images (those defined by the class), relative to the\n Page-wide ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','skinDir')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "attr:CubeGrid.facetValueHoverWidth":{
        ref:"attr:CubeGrid.facetValueHoverWidth",
        flags:"IRWA",
        description:"If specified and <code>this.showHover</code> is true, this is the default width to apply\n to hover tips shown for facetValues. If unset, hover canvas will be sized to \n <code>this.hoverWidth</code> if specified instead.",
        name:"facetValueHoverWidth",
        seeAlso:[
            {ref:"attr:Canvas.hoverWidth", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "method:TreeGrid.setViewState":{
        ref:"method:TreeGrid.setViewState",
        flags:"",
        description:" Overridden to take a ${isc.DocUtils.linkForRef('type:treeGridViewState')} object.\n\n",
        name:"setViewState",
        seeAlso:[
            {ref:"method:TreeGrid.getViewState", description:null}
        ],
        params:[
            {description:"Object describing the desired view state for the grid", optional:false,
             name:"viewState", defaultValue:null, type:"treeGridViewState"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "attr:StretchImgButton.src":{
        ref:"attr:StretchImgButton.src",
        flags:"IRW",
        description:"Base URL for the image.  By default, StretchImgButtons consist of three image parts: A\n start image (displayed at the top or left), a scaleable central image and an end image\n displayed at the bottom or right.\n <P>\n The images displayed in the stretchImgButton are derived from this property in the \n following way:\n <P>\n <ul>\n <li> When the button is in its standard state the suffixes \"_start\", \"_end\" and \n      \"_stretch\" are applied to the src (before the file extension), so by default \n      the images displayed will be \"button_start.gif\" (sized to be \n      <code>this.capSize</code> by the specified width of the stretchImgButton), \n      \"button_stretch.gif\" (stretched to the necessary width) and \"button_end.gif\" \n      (sized the same as the start image).\n <li> As the button's state changes, the images will have suffixes appended <b>before</b>\n      the \"_start\" / \"_end\" / \"_stretch\" to represent the button state. Possible states \n      are \"Down\", \"Over\", \"Selected\" \"Focused\" and \"Disabled\". Note that \"Selected\" and\n      \"Focused\" are compound states which may be applied in addition to \"Down\" etc.\n </ul>\n For example the center piece of a selected stretchImgButton with the mouse hovering\n over it might have the URL: <code>\"button_Selected_Down_stretch.gif\"</code>.\n <P>\n Media should be present for each possible state of the _start, _end and _stretch images.",
        name:"src",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"button.gif\"",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "classAttr:Dialog.NO_BUTTON_TITLE":{
        ref:"classAttr:Dialog.NO_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"No\"</code> button.",
        name:"NO_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"No\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:Calendar.addEventButtonHoverText":{
        ref:"attr:Calendar.addEventButtonHoverText",
        flags:"IR",
        description:"The text to be displayed when a user hovers over the ${isc.DocUtils.linkForRef('attr:Calendar.addEventButton','add event')}\n toolbar button",
        name:"addEventButtonHoverText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Add an event\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "method:Calendar.setShowWeekends":{
        ref:"method:Calendar.setShowWeekends",
        flags:"",
        description:"  Setter for ${isc.DocUtils.linkForRef('attr:Calendar.showWeekends')} to change this property at runtime.\n\n",
        name:"setShowWeekends",
        seeAlso:[],
        type:"method",
        definingClass:"class:Calendar"
    },
    "method:Canvas.animateScroll":{
        ref:"method:Canvas.animateScroll",
        flags:"",
        description:" Animate a scroll from the current scroll position to the specified position.\n",
        name:"animateScroll",
        seeAlso:[],
        params:[
            {description:"desired final left scroll postion", optional:false, name:"scrollLeft",
             defaultValue:null, type:"number"},
            {description:"desired final top scroll postion", optional:false, name:"scrollTop",
             defaultValue:null, type:"number"},
            {description:"When the scroll completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated scroll", optional:true, name:"duration",
             defaultValue:null, type:"number"},
            {description:"Optional acceleration to bias the animation ratios", optional:true,
             name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:Button.autoFit":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Button.autoFit",
        description:"If true, ignore the specified size of this widget and always size just large\n enough to accomodate the title.  If <code>setWidth()</code> is explicitly called on an\n autoFit:true button, autoFit will be reset to <code>false</code>.\n <P>\n Note that for StretchImgButton instances, autoFit will occur horizontally only, as \n unpredictable vertical sizing is likely to distort the media. If you do want vertical \n auto-fit, this can be achieved by simply setting a small height, and having \n overflow:\"visible\"",
        name:"autoFit",
        setter:["setAutoFit"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonAutoFit')}"],
        defaultValue:"null",
        groups:["sizing"],
        definingClass:"class:Button"
    },
    "attr:OperationBinding.customHQL":{
        ref:"attr:OperationBinding.customHQL",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"hibernate\", this \n property can be specified on an operationBinding to indicate that the server should run \n user-specified HQL (Hibernate Query Language), rather than the Hibernate criteria query or \n <code>saveOrUpdate</code> call it would normally generate to satisfy a dataSource operation\n via Hibernate. \n <p>\n Note that inserting new records via HQL is often impractical, due to intentional restrictions \n in the language (it is only possible to perform an insert expressed in terms of a SELECT; the\n \"VALUES\" construct commonly used when inserting new rows singly is not supported).  If you are \n intending to use customHQL, we recommend that you avoid doing so for ${isc.DocUtils.linkForRef('class:OperationBinding')}s \n with ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"add\", unless you have a special \n requirement such as a bulk insert; if you need custom queries to perform inserts \n on \"hibernate\" dataSources, we recommend you use ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL','customSQL')},\n which is valid for \"hibernate\" DataSources as well as \"sql\" dataSources.\n <p>\n For other operations on \"hibernate\" dataSources, however, HQL has the advantage of \n being more portable across different database engines than is plain SQL.",
        name:"customHQL",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null},
            {ref:"attr:DataSourceField.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:ListGrid.autoFitMaxHeight":{
        ref:"attr:ListGrid.autoFitMaxHeight",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')} is set to <code>\"vertical\"</code> or <code>\"both\"</code> this\n property provides an upper limit on how far the ListGrid will expand vertically to accomodate\n its content. If content exceeds this height, scrollbars will be introduced as usual.  \n In addition to this property, ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxRecords')} allows you to limit vertical\n expansion based on the number of rows to be rendered.",
        name:"autoFitMaxHeight",
        setter:["setAutoFitMaxHeight"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "method:Tree.add":{
        ref:"method:Tree.add",
        flags:"",
        description:"\n Add a single node under the specified parent\n\n",
        name:"add",
        seeAlso:[
            {ref:"group:sharingNodes", description:null},
            {ref:"method:Tree.addList", description:null}
        ],
        params:[
            {description:"node to add", optional:false, name:"node", defaultValue:null, type:"TreeNode"},
            {description:"Parent of the node being added.  You can pass                                          in either the ${isc.DocUtils.linkForRef('object:TreeNode')} itself, or a path to\n	                                        the node (as a String), in which case a\n	                                        ${isc.DocUtils.linkForRef('method:Tree.find')} is performed to find\n	                                        the node.",
             optional:false, name:"parent", defaultValue:null, type:"String or TreeNode"},
            {description:"Position of the new node in the children list. If not	                                specified, the node will be added at the end of the list.",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        returns:{description:"The added node. Will return null if the node was not added (typically    because the specified <code>parent</code> could not be found in the tree).",
         type:"TreeNode or null"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:ImgButton.deselect":{
        ref:"method:ImgButton.deselect",
        flags:"",
        description:" Select this object.\n",
        name:"deselect",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:Tab.title":{
        ref:"attr:Tab.title",
        flags:"IRW",
        description:"Specifies the title of the this tab.  To change the title after the TabSet has been\n created, call ${isc.DocUtils.linkForRef('method:TabSet.setTabTitle')}.",
        name:"title",
        seeAlso:[
            {ref:"method:TabSet.setTabTitle", description:null}
        ],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab"
    },
    "attr:TileGrid.tileValueStyle":{
        ref:"attr:TileGrid.tileValueStyle",
        flags:"IR",
        description:"When using the default ${isc.DocUtils.linkForRef('class:SimpleTile')}, CSS style for each value shown within a tile.",
        name:"tileValueStyle",
        seeAlso:[],
        valueType:"CSSClassName",
        defaultValue:"\"tileValue\"",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:MenuItem.keys":{
        ref:"attr:MenuItem.keys",
        flags:"IR",
        description:"Shortcut key(s) to fire the menu item action. Each key can be defined as a ${isc.DocUtils.linkForRef('type:KeyIdentifier')}.\n To apply multiple shortcut keys to this item, set this property to an array of such key\n identifiers.",
        name:"keys",
        seeAlso:[],
        valueType:"KeyIdentifier | Array of KeyIdentifer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:MenuItem",
        groups:["menuKeys"]
    },
    "attr:Calendar.eventWindow":{
        ref:"attr:Calendar.eventWindow",
        flags:"A",
        description:"To display events in day and week views, the Calendar creates instance of ${isc.DocUtils.linkForRef('class:EventWindow')}\n for each event.  Use the ${isc.DocUtils.linkForRef('type:AutoChild')} system to customize these windows.",
        name:"eventWindow",
        seeAlso:[],
        valueType:"AutoChild EventWindow",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:Calendar.nextButton":{
        ref:"attr:Calendar.nextButton",
        flags:"IR",
        description:"The nextButton is an ImgButton that appears above the week/day/month views of the\n calendar and allows the user to move the calendar forwards in time.",
        name:"nextButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "group:sorting":{
        ref:"group:sorting",
        refs:["method:ResultSet.sortByProperty", "method:List.sortByProperty", 
         "method:ListGridField.sortNormalizer","method:ListGrid.headerClick", "method:ListGrid.unsort", "method:ListGrid.sort", 
         "method:Array.sortByProperty","method:Array.unsort", "classMethod:Array.compareAscending", 
         "classMethod:Array.compareDescending","attr:PickList.sortField", "attr:SelectItem.sortField", "attr:ComboBoxItem.sortField", 
         "attr:ListGridField.canSort","attr:ListGridField.sortDirection", "attr:ListGrid.canSort", "attr:ListGrid.sortFieldNum", 
         "attr:ListGrid.sortField","attr:ListGrid.sortDirection", "attr:ListGrid.showSortArrow", 
         "attr:ListGrid.canPickFields","type:SortArrow", "type:SortDirection"],
        description:null,
        name:"sorting",
        seeAlso:[],
        type:"group"
    },
    "method:TreeGrid.transferSelectedData":{
        ref:"method:TreeGrid.transferSelectedData",
        flags:"",
        description:" Simulates a drag / drop type transfer of the selected records in some other grid to this\n treeGrid, without requiring any user interaction.<br>\n See the ${isc.DocUtils.linkForRef('group:dragging')} documentation for an overview of grid drag/drop data\n transfer.\n",
        name:"transferSelectedData",
        seeAlso:[],
        params:[
            {description:"source grid from which the records will be tranferred", optional:false,
             name:"sourceGrid", defaultValue:null, type:"ListGrid"},
            {description:"parent node into which records should be dropped - if null    records will be transferred as children of the root node.",
             optional:true, name:"folder", defaultValue:null, type:"TreeNode"},
            {description:"target index (drop position) within the parent folder", optional:true,
             name:"index", defaultValue:null, type:"integer"},
            {description:"optional callback to be fired when the transfer process has                        completed.  The callback will be passed a single parameter \"records\",\n                       the list of nodes actually transferred to this component (it is called \n                       \"records\" because this logic is shared with ${isc.DocUtils.linkForRef('class:ListGrid')}).",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragTree')}"],
        definingClass:"class:TreeGrid",
        groups:["dragging"]
    },
    "classAttr:ValidatorTypes.isUnique":{
        ref:"classAttr:ValidatorTypes.isUnique",
        flags:"IR",
        description:"Returns true if the value for this field is unique across the whole DataSource.\n  By default, validators of this type have ${isc.DocUtils.linkForRef('attr:Validator.serverOnly','serverOnly')} \n  set to <code>true</code> and do not run on the client.",
        name:"isUnique",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "group:cellStyling":{
        ref:"group:cellStyling",
        refs:["attr:CubeGrid.baseStyle", "attr:GridRenderer.cellHeight", 
         "attr:GridRenderer.fixedRowHeights","attr:GridRenderer.wrapCells", "attr:GridRenderer.cellPadding", 
         "attr:GridRenderer.baseStyle","attr:GridRenderer.alternateRowStyles", "attr:GridRenderer.alternateRowFrequency", 
         "attr:GridRenderer.emptyCellValue","attr:ListGrid.emptyCellValue", "attr:ListGrid.cellHeight", 
         "attr:ListGrid.fixedRecordHeights","attr:ListGrid.fixedFieldWidths", "attr:ListGrid.wrapCells", "attr:ListGrid.cellPadding", 
         "attr:ListGrid.baseStyle","attr:ListGrid.alternateRecordStyles", "attr:ListGrid.alternateRecordFrequency"],
        name:"cellStyling",
        seeAlso:[],
        type:"group"
    },
    "attr:DataBoundComponent.fetchOperation":{
        ref:"attr:DataBoundComponent.fetchOperation",
        flags:"IRW",
        description:"Operation ID this component should use when performing fetch operations.",
        name:"fetchOperation",
        seeAlso:[
            {ref:"attr:DSRequest.operationId", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["operations"]
    },
    "attr:TileGrid.dragDataAction":{
        ref:"attr:TileGrid.dragDataAction",
        flags:"IRW",
        description:"Indicates what to do with data dragged into another DataBoundComponent. See\n          DragDataAction type for details.",
        name:"dragDataAction",
        seeAlso:[],
        valueType:"DragDataAction",
        defaultValue:"Canvas.MOVE",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('gridsDragMove')}", 
         "${isc.DocUtils.linkForExampleId('gridsDragCopy')}"],
        definingClass:"class:TileGrid",
        groups:["dragging"]
    },
    "attr:ListGrid.frozenHeaderTitleStyle":{
        ref:"attr:ListGrid.frozenHeaderTitleStyle",
        flags:"IR",
        description:"If this listGrid contains any frozen fields, this property can be used to apply a custom\n headerTitleStyle to the frozen fields set of fields. If unset, the standard headerTitleStyle\n will be used for both frozen and unfrozen cells.",
        name:"frozenHeaderTitleStyle",
        seeAlso:[
            {ref:"attr:ListGrid.headerTitleStyle", description:null},
            {ref:"attr:ListGridField.frozen", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance", "frozenFields"]
    },
    "attr:Canvas.animateMoveAcceleration":{
        ref:"attr:Canvas.animateMoveAcceleration",
        flags:"IRWA",
        description:"Default acceleration effect for performing an animated move.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateMoveAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:Canvas.getOffsetX":{
        ref:"method:Canvas.getOffsetX",
        flags:"",
        description:"	Return the X-coordinate of the last event relative to the left edge of the content of this\n	Canvas.<br><br>\n\n  NOTE: To get a coordinate relative to the <b>viewport</b> of this Canvas, subtract\n  this.getScrollLeft()\n\n",
        name:"getOffsetX",
        seeAlso:[],
        returns:{description:null, type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["events", "positioning"]
    },
    "method:ListGrid.hideField":{
        ref:"method:ListGrid.hideField",
        flags:"",
        description:" Force a field to be hidden.<br><br>\n\n NOTE: If a field.showIf expression exists, it will be destroyed.\n \n",
        name:"hideField",
        seeAlso:[],
        params:[
            {description:"field to hide", optional:false, name:"field", defaultValue:null,
             type:"field name or ListGridField"},
            {description:"if passed, don't relayout non-explicit sized fields                                       to fit the available space",
             optional:true, name:"suppressRelayout", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('columnOrder')}"],
        definingClass:"class:ListGrid"
    },
    "method:Canvas.getOffsetY":{
        ref:"method:Canvas.getOffsetY",
        flags:"",
        description:"	Return the Y-coordinate of the last event, relative to the top edge of the content of this\n	Canvas.<br><br>\n\n  NOTE: To get a coordinate relative to the <b>viewport</b> of this Canvas, subtract\n  this.getScrollTop()\n\n",
        name:"getOffsetY",
        seeAlso:[],
        returns:{description:null, type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["events", "positioning"]
    },
    "method:Calendar.dateChanged":{
        ref:"method:Calendar.dateChanged",
        flags:"",
        description:" Fires whenever the user changes the current date, including picking a specific date or\n navigating to a new week or month.\n",
        name:"dateChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:Calendar"
    },
    "method:CubeGrid.resizeFacetValue":{
        ref:"method:CubeGrid.resizeFacetValue",
        flags:"",
        description:" Resizes all columns for the provided facetValueId, which must be a facetValueId from the\n innermost column facet.\n\n",
        name:"resizeFacetValue",
        seeAlso:[],
        params:[
            {description:"facetValueId of columns to be resized", optional:false, name:"facetValueId",
             defaultValue:null, type:"id"},
            {description:"column's new width", optional:false, name:"newWidth", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "group:orientation":{
        ref:"group:orientation",
        refs:["type:Orientation"],
        name:"orientation",
        seeAlso:[],
        type:"group"
    },
    "attr:CubeGrid.facetValueAlign":{
        ref:"attr:CubeGrid.facetValueAlign",
        flags:"IRW",
        description:"default alignment for facet values (in headers)",
        name:"facetValueAlign",
        seeAlso:[],
        valueType:"alignEnum",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:CheckboxItem.valueMap":{
        ref:"attr:CheckboxItem.valueMap",
        flags:"IRW",
        description:"Object defining how the checkbox \"checked\" state will be mapped to the field value.<br>\n Checkboxes only support 2 states. By default a checked checkbox will have\n value <code>true</code>, and an unchecked one will have value <code>false</code>\n (or <code>null</code> if there is no default value and the value has never been set).\n <p>\n A valueMap can modify this in 2 ways:<br>\n - If the desired checked/unchecked values can be resolved to <code>true</code> \n   and <code>false</code> directly in JavaScript, the valueMap may be specified as\n   a 2-element array containing these values. Examples of this woud include<br>\n   <code>&nbsp;&nbsp;[0,1]</code>: \n   In this case an unchecked checkbox would have value <code>0</code> and a checked box\n   would have value <code>1</code><br>\n   <code>&nbsp;&nbsp;[null,\"Some String\"]</code>:\n   In this case an unchecked checkbox would have value <code>null</code> and a checked \n    box would have value <code>\"Some String\"</code><br>\n - More arbitrary data values can be resolved to checked / unchecked values via an\n   object mapping the abitrary data values to display values of <code>true</code> and\n   <code>false</code>. For example:<br>\n   <code>&nbsp;&nbsp;{\"A\":false, \"B\":true}</code><br>\n    In this case an unchecked checkbox would have value \"A\", and a checked box \n    would have value \"B\"\n <p>\n Note: ValueMaps in other formats will be ignored by the CheckboxItem class. To update\n the valueMap at runtime, always use ${isc.DocUtils.linkForRef('method:CheckboxItem.setValueMap')}",
        name:"valueMap",
        setter:["setValueMap"],
        seeAlso:[],
        valueType:"object | Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["valueMap"]
    },
    "group:containment":{
        ref:"group:containment",
        refs:["method:Canvas.addChild", "method:Canvas.removePeer", "method:Canvas.removeChild", 
         "method:Canvas.addPeer","method:Canvas.getParentElements", "method:Canvas.contains", "attr:Canvas.parentElement", 
         "attr:Canvas.topElement","attr:Canvas.masterElement", "attr:Canvas.children", "attr:Canvas.peers"],
        description:"Canvases containing other objects as children or peers",
        name:"containment",
        seeAlso:[],
        type:"group"
    },
    "classAttr:Array.CASE_INSENSITIVE":{
        ref:"classAttr:Array.CASE_INSENSITIVE",
        flags:null,
        description:"This is a built-in comparator for the ${isc.DocUtils.linkForRef('method:Array.find','find')} and ${isc.DocUtils.linkForRef('method:Array.findIndex','findIndex')}\n methods of Array.  Passing this comparator to those methods will find case-insensitively,\n so, eg, <code>find(\"foo\", \"bar\")</code> would find objects with a \"foo\" property set to \n \"Bar\", \"BAR\" or \"bar\"",
        name:"CASE_INSENSITIVE",
        seeAlso:[],
        valueType:"Function",
        defaultValue:"See below",
        type:"classAttr",
        definingClass:"object:Array"
    },
    "group:creation":{
        ref:"group:creation",
        refs:[],
        name:"creation",
        seeAlso:[],
        type:"group"
    },
    "method:DynamicForm.editNewRecord":{
        ref:"method:DynamicForm.editNewRecord",
        flags:"",
        description:"\n Prepare to edit a new record by clearing the current set of values (or replacing them \n with initialValues if specified).\n <br>\n This method will also call ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')} to ensure\n subsequent calls to <code>saveData()</code> will use an <code>add</code> rather than\n an <code>update</code> operation.\n\n",
        name:"editNewRecord",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"initial set of values for the editor as a map of field names to their corresponding\n     values",
             optional:true, name:"initialValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "method:SliderItem.setMaxValue":{
        ref:"method:SliderItem.setMaxValue",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.maxValue','maximum value')} of the slider \n\n",
        name:"setMaxValue",
        seeAlso:[],
        params:[
            {description:"the new maximum value", optional:false, name:"newValue", defaultValue:null,
             type:"float"}
        ],
        type:"method",
        definingClass:"class:SliderItem"
    },
    "classMethod:Page.getScrollLeft":{
        ref:"classMethod:Page.getScrollLeft",
        flags:"",
        description:"		Get the amount that the browser window has been scrolled horizontally.\n",
        name:"getScrollLeft",
        seeAlso:[],
        returns:{description:"horizontal scroll amount", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "class:SectionItem":{
        exampleConfig:"SectionItem.xml",
        methods:["method:SectionItem.isExpanded", "method:SectionItem.expandSection", 
         "method:SectionItem.collapseSection"],
        type:"class",
        ref:"class:SectionItem",
        classAttrs:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        classMethods:[],
        description:"Header item for a collapsable section in a ${isc.DocUtils.linkForRef('class:DynamicForm')}.  Each <code>SectionItem</code>\n is associated with a number of other <code>item</code>s in the form, which will be shown or\n hidden as a group when the section is expanded or collapsed.  Clicking on a\n <code>SectionItem</code> will expand or collapse the section.\n <P>\n To make a form where only one section is expanded at a time, set\n ${isc.DocUtils.linkForRef('attr:DynamicForm.sectionVisibilityMode')} to \"mutex\".",
        attrs:["attr:SectionItem.defaultValue", "attr:SectionItem.itemIds", 
         "attr:SectionItem.sectionVisible","attr:SectionItem.sectionExpanded", "attr:SectionItem.sectionHeaderClass", 
         "attr:SectionItem.canCollapse"],
        name:"SectionItem",
        seeAlso:[
            {ref:"attr:DynamicForm.sectionVisibilityMode", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('formSections')}"]
    },
    "attr:TileRecord.tileProperties":{
        ref:"attr:TileRecord.tileProperties",
        flags:"IRW",
        description:"Additional properties to be passed when creating a tile for this record.",
        name:"tileProperties",
        seeAlso:[],
        valueType:"Canvas Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TileRecord"
    },
    "method:Layout.reflow":{
        ref:"method:Layout.reflow",
        flags:"A",
        description:" Layout members according to current settings.\n <P>\n Members will reflow automatically when the layout is resized, members resize, the list of\n members changes or members change visibility.  It is only necessary to manually call\n <code>reflow()</code> after changing settings on the layout, for example,\n <code>layout.reverseOrder</code>.\n\n",
        name:"reflow",
        seeAlso:[],
        params:[
            {description:"reason reflow() had to be called (appear in logs if enabled)",
             optional:true, name:"reason", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "method:TabSet.closeClick":{
        ref:"method:TabSet.closeClick",
        flags:"",
        description:" When ${isc.DocUtils.linkForRef('attr:TabSet.canCloseTabs')} is set, method fired when the user clicks the \"close\" icon for a\n tab.\n <P>\n Default implementation will remove the tab from the tabSet via ${isc.DocUtils.linkForRef('method:TabSet.removeTab')}.\n\n",
        name:"closeClick",
        seeAlso:[],
        params:[
            {description:"tab to close", optional:false, name:"tab", defaultValue:null, type:"Tab"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "class:RPCManager":{
        methods:[],
        ref:"class:RPCManager",
        classAttrs:["classAttr:RPCManager.defaultTimeout", "classAttr:RPCManager.defaultPrompt", 
         "classAttr:RPCManager.timeoutErrorMessage","classAttr:RPCManager.removeDataPrompt", "classAttr:RPCManager.saveDataPrompt", 
         "classAttr:RPCManager.promptStyle","classAttr:RPCManager.useCursorTracker", "classAttr:RPCManager.promptCursor", 
         "classAttr:RPCManager.fetchDataPrompt","classAttr:RPCManager.showPrompt", "classAttr:RPCManager.actionURL", 
         "classAttr:RPCManager.useXmlHttpRequest","classAttr:RPCManager.defaultTransport", "classAttr:RPCManager.credentialsURL"],
        classMethods:["classMethod:RPCManager.queueSent", "classMethod:RPCManager.xmlHttpRequestAvailable", 
         "classMethod:RPCManager.send","classMethod:RPCManager.sendProxied", "classMethod:RPCManager.sendRequest", 
         "classMethod:RPCManager.startQueue","classMethod:RPCManager.getCurrentTransactionId", "classMethod:RPCManager.cancelQueue", 
         "classMethod:RPCManager.clearTransaction","classMethod:RPCManager.suspendTransaction", "classMethod:RPCManager.resendTransaction", 
         "classMethod:RPCManager.sendQueue","classMethod:RPCManager.handleError", "classMethod:RPCManager.handleTransportError", 
         "classMethod:RPCManager.loginRequired"],
        treeLocation:["Client Reference/RPC"],
        attrs:[],
        description:"RPCManager is a static singleton class that manages transparent client/server RPC (remote\n procedure call).  This class provides a generic, low-level client/server communication\n integration point.  You can use it to send arbitrary data to a URL of your choosing on the\n server and optionally be called back with server-returned data when the server replies.  \n You can process the RPC request in a JSP, Servlet or Filter on the server.\n <P>\n SmartClient's powerful ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')} automatically issue\n RPCs as necessary, based on the \n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource protocol')}.  To integrate DataBoundComponents\n with your server, ${isc.DocUtils.linkForRef('group:clientServerIntegration','start here')}.\n <P>\n <u>Simple example (client code):</u>\n <P><code>\n var data = { here: \"is some data\", to: [\"send to the server\"]};<br>\n RPCManager.sendRequest({ data: data, callback: \"myCallback(data)\", actionURL: \"/rpcHandler.jsp\"});<br>\n function myCallback(data) { alert(\"response from the server: \" + data); }\n </code><br><br>\n <u>Simple example (server code: /rpcHandler.jsp):</u>\n <br><br><code>\n RPCManager rpc = new RPCManager(request, response, out);<br>\n Object data = rpc.getData();<br>\n System.out.println(\"client sent: \" + data.toString());<br>\n rpc.send(\"here's a response\");<br>\n </code>\n <P>\n Note that, while the example above uses the SmartClient Java Server, the RPCManager is also\n capable of issuing RPCs that do not require a SmartClient server.  See\n ${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-Side Data Integration')} for details.\n <P>\n <u><b>Queuing</b></u>\n <br>\n Because of browser limitations on the total number of simultaneous HTTP connections to a given\n server, batching multiple RPC requests into a single HTTP request is highly advisable whenever\n possible.  The RPCManager provides a queuing mechanism that allows this.\n <br><br>\n <u>Queuing example (client code):</u>\n <br><br><code>\n RPCManager.startQueue();<br>\n RPCManager.send(\"a string of data\", \"myCallback(data)\", {actionURL: \"/rpcHandler.jsp\"});<br>\n RPCManager.sendRequest({ data: [\"some\", \"more data\", 2], callback: \"myCallback(data)\", actionURL: \"/rpcHandler.jsp\"});<br>\n RPCManager.sendRequest({ data: \"different callback\", callback: \"myCallback2(data)\", actionURL: \"/rpcHandler.jsp\"});<br>\n RPCManager.sendQueue()<br>\n function myCallback(data) { alert(\"response from the server: \" + data); }<br>\n function myCallback2(data) { alert(\"response from the server (other callback): \" + data); }\n </code><br><br>\n <u>Queuing example (server code: /rpcHandler.jsp):</u>\n <br><br><code>\n RPCManager rpc = new RPCManager(request, response, out);<br>\n for(Iterator i = rpc.getRequests().iterator(); i.hasNext();) {<br>\n &nbsp;&nbsp;&nbsp;&nbsp;RPCRequest rpcRequest = (RPCRequest)i.next();<br>\n &nbsp;&nbsp;&nbsp;&nbsp;Object data = rpcRequest.getData();<br>\n &nbsp;&nbsp;&nbsp;&nbsp;System.out.println(\"client sent:\" + data.toString());<br>\n &nbsp;&nbsp;&nbsp;&nbsp;//send back the data sent to us by the client<br>\n &nbsp;&nbsp;&nbsp;&nbsp;rpc.send(rpcRequest, new RPCResponse(data));<br>\n }<br>\n </code>\n <br><br>\n <u><b>Error Handling</b></u>\n <br>\n The ${isc.DocUtils.linkForRef('class:RPCResponse')} object has an integer status field that the RPCManager inspects when\n the response is received from the server. If the value of this field is less than zero, the\n request is considered to have failed.  Otherwise it is considered to have succeeded.  This\n value is settable via the setStatus() method call on the RPCResponse server-side object.\n <br><br>\n If the status field shows a failure, the RPCManager will, by default, show a dialog with the\n contents of the ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} field (which is assumed to contain a \n meaningful description of the error that occured).  If you specified a callback in your\n RPCRequest, it will <b>not</b> be called if the status shows a failure (see below for how to\n change this).\n <br><br>\n If the status field shows success, the RPCManager takes no special action.\n <br><br>\n The built-in status codes and default behavior are there for convenience.  You can choose to\n completely ignore it and handle errors as you see fit (for example by encoding them into the data\n field returned by the server, and always setting the RPCResponse status field to a success\n value).  In fact, the status field is automatically set to a success code\n (RPCResponse.STATUS_SUCCESS) by the constructor of the RPCResponse object on the server. \n <br><br>\n If you choose to use the status field, but want to handle the errors yourself in your callback\n (and suppress the default error dialog popped up by the RPCManager), simply specify the\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError','willHandleError:true')} on your RPCRequest object.  This\n allows you to use the RPCManager.sendError() convenience methods on the server without the\n default error handling behavior on the client.",
        name:"RPCManager",
        seeAlso:[],
        type:"class"
    },
    "method:DataSourceField.getFieldValue":{
        ref:"method:DataSourceField.getFieldValue",
        flags:"A",
        description:" Function or JavaScript expression used to retrieve the field's value from the XML element or\n JSON record returned from a web service.\n <P>\n This is an advanced API for use when a ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','valueXPath')} setting\n is insufficient to derive a field's value, yet an implementation of\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} is overkill.\n\n",
        name:"getFieldValue",
        seeAlso:[],
        params:[
            {description:"record object selected from web service response data                                      by ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')}",
             optional:false, name:"record", defaultValue:null, type:"Object or XMLElement"},
            {description:"default value derived by the method described in                    ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')}",
             optional:false, name:"value", defaultValue:null, type:"any"},
            {description:"DataSourceField definition", optional:false, name:"field", defaultValue:null,
             type:"DataSourceField"},
            {description:"name of the DataSource field", optional:false, name:"fieldName",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["clientDataIntegration"]
    },
    "method:ListGrid.rowEditorEnter":{
        ref:"method:ListGrid.rowEditorEnter",
        flags:"",
        description:" Callback fired when the user starts editing a new row.\n\n",
        name:"rowEditorEnter",
        seeAlso:[
            {ref:"method:ListGrid.editorEnter", description:null}
        ],
        params:[
            {description:"record for the cell being edited", optional:false, name:"record",
             defaultValue:null, type:"ListGridField"},
            {description:"edit values for the current row", optional:false, name:"editValues",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:DynamicForm.cancel":{
        ref:"method:DynamicForm.cancel",
        flags:"",
        description:"\n This method exists for clean integration with existing server frameworks that have a 'cancel'\n feature which typically clears session state associated with the form.  When this method is\n called, an RPC is sent to the server with a parameter named\n ${isc.DocUtils.linkForRef('attr:DynamicForm.cancelParamName')} with the value\n ${isc.DocUtils.linkForRef('attr:DynamicForm.cancelParamValue')}.<p>\n\n Note that no other form data is sent.  By default the current top-level page is replaced with the\n reply.  If you wish to ignore the server reply instead, call this method like this:\n <pre>\n dynamicFormInstance.cancel({ignoreTimeout: true, target: null});\n </pre>\n\n",
        name:"cancel",
        seeAlso:[],
        params:[
            {description:"additional properties to set on the RPCRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "attr:Layout.stackZIndex":{
        ref:"attr:Layout.stackZIndex",
        flags:"IR",
        description:"For use in conjunction with ${isc.DocUtils.linkForRef('attr:Layout.memberOverlap')}, controls the z-stacking order of\n members.\n <P>\n If \"lastOnTop\", members stack from the first member at bottom to the last member at\n top. If \"firstOnTop\", members stack from the last member at bottom to the first member\n at top.",
        name:"stackZIndex",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout"
    },
    "method:Class.setLogPriority":{
        ref:"method:Class.setLogPriority",
        flags:"",
        description:" Set the priority of messages that will be visible for some log category, when logged on\n this Class or Instance object.<br>\n If called with no category, this priority will be applied to every logged message on this\n object<br>\n To set the visible log priority for some category across the entire page, use\n <code>isc.Log.setPriority()</code> instead.\n",
        name:"setLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:FormulaBuilder.showHelpIcon":{
        ref:"attr:FormulaBuilder.showHelpIcon",
        flags:"IR",
        description:"Whether to show the help icon that appears after the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.formulaField')}.",
        name:"showHelpIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:FormItem.dataPath":{
        ref:"attr:FormItem.dataPath",
        flags:"IR",
        getter:["getDataPath"],
        description:"dataPath for this item. Allows the user to edit details nested data structures in a\n flat set of form fields",
        name:"dataPath",
        seeAlso:[],
        valueType:"dataPath",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:Canvas.mouseUp":{
        ref:"method:Canvas.mouseUp",
        flags:"A",
        description:"\n Executed when the left mouse is released on this widget.  No default implementation.\n\n",
        name:"mouseUp",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customMouseEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:WebService.getOutputHeaderSchema":{
        ref:"method:WebService.getOutputHeaderSchema",
        flags:"",
        description:" Get the schema for each part of the SOAP header for the output message of a given\n operation, as a mapping from part name to schema.  For example, given WSDL like:\n <pre>\n     &lt;soap:header part=\"SessionHeader\"/&gt;\n     &lt;soap:header part=\"CallOptions\"/&gt;\n </pre>\n The following schema would be returned:\n <pre>\n     { SessionHeader : <i>sessionHeaderPartSchema</i>,\n       CallOptions : <i>callOptionsPartSchema</i> }\n </pre>\n The schema are instances of ${isc.DocUtils.linkForRef('class:DataSource')} that can be inspected to discover the\n elements and types that are legal in that header part, and can construct a valid SOAP\n header part if ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} is invoked.\n\n",
        name:"getOutputHeaderSchema",
        seeAlso:[],
        params:[
            {description:"name of an operation from this web service", optional:false,
             name:"operationName", defaultValue:null, type:"String"}
        ],
        returns:{description:"mapping from partName to schema", type:"Object"},
        type:"method",
        definingClass:"class:WebService"
    },
    "attr:StretchImgButton.showRollOver":{
        ref:"attr:StretchImgButton.showRollOver",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes over this object?",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "attr:ListGrid.booleanImageWidth":{
        ref:"attr:ListGrid.booleanImageWidth",
        flags:"IRWA",
        description:"Width for the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} and ${isc.DocUtils.linkForRef('attr:ListGrid.booleanFalseImage')}.\n Note: If ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} is unset, the ${isc.DocUtils.linkForRef('attr:CheckboxItem.checkedImage')}\n will be used to indicate a true value in a boolean field. In this case this property is\n ignored in favor of ${isc.DocUtils.linkForRef('attr:CheckboxItem.valueIconWidth')}.",
        name:"booleanImageWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:LoginDialog.title":{
        ref:"attr:LoginDialog.title",
        flags:"IR",
        description:"Specifies the title of the dialog box.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Dialog.LOGIN_TITLE",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:ListGridField.editorValueMap":{
        ref:"attr:ListGridField.editorValueMap",
        flags:"IRW",
        getter:["getEditorValueMap"],
        description:"A valueMap to use for editors shown for this field.  By default if this is not\n specified ${isc.DocUtils.linkForRef('attr:ListGridField.valueMap','field.valueMap')} will be used instead.\n <P>\n Dynamic valueMaps can be provided by implementing ${isc.DocUtils.linkForRef('method:ListGrid.getEditorValueMap')}.",
        name:"editorValueMap",
        seeAlso:[
            {ref:"method:ListGrid.getEditorValueMap", description:null},
            {ref:"method:ListGrid.setEditorValueMap", description:null}
        ],
        valueType:"Array or Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "object:FormItemIcon":{
        methods:["method:FormItemIcon.click", "method:FormItemIcon.keyPress"],
        type:"object",
        ref:"object:FormItemIcon",
        classAttrs:[],
        treeLocation:["Client Reference/Forms/Form Items/FormItem"],
        classMethods:[],
        description:"Form item icon descriptor objects used by Form Items to specify the appearance and\n  behavior of icons displayed after the item in the page flow.",
        attrs:["attr:FormItemIcon.src", "attr:FormItemIcon.showOver", "attr:FormItemIcon.showFocused", 
         "attr:FormItemIcon.showFocusedWithItem","attr:FormItemIcon.neverDisable", "attr:FormItemIcon.width", "attr:FormItemIcon.height", 
         "attr:FormItemIcon.prompt"],
        name:"FormItemIcon",
        seeAlso:[
            {ref:"attr:FormItem.icons", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('formIcons')}"],
        groups:["formIcons"]
    },
    "class:HiddenItem":{
        methods:[],
        ref:"class:HiddenItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:HiddenItem.showTitle", "attr:HiddenItem.colSpan", "attr:HiddenItem.rowSpan"],
        description:"HiddenItems track a value but have no visible appearance and do not take up space in the form\n layout.\n <P>\n When using SmartClient databinding it is usually not necessary to use a HiddenItem, since\n the DynamicForm will track values for which no actual form control exists, and will submit\n these 'extra' values when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is called.  HiddenItems only apply\n to forms that are submitted like ordinary HTML forms, via the\n ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm')} method.",
        name:"HiddenItem",
        seeAlso:[],
        type:"class"
    },
    "attr:Canvas.defaultWidth":{
        ref:"attr:Canvas.defaultWidth",
        flags:"IRWA",
        description:"For custom components, establishes a default width for the component.\n <P>\n For a component that should potentially be sized automatically by a Layout, set this\n property rather than ${isc.DocUtils.linkForRef('attr:Canvas.width')} directly, because Layouts regard a width setting as\n an explicit size that shouldn't be changed.",
        name:"defaultWidth",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Canvas.layoutChildren":{
        ref:"method:Canvas.layoutChildren",
        flags:"A",
        description:"\n <code>layoutChildren()</code> is where a Canvas should implement a sizing policy for it's\n Canvas children.  Since <code>layoutChildren</code> calls parentResized() on its children,\n ${isc.DocUtils.linkForRef('method:Canvas.parentResized')} is a good place for a child to implement a layout policy that\n can be used within any parent.\n <P>\n Recommended practice for a Canvas that manages Canvas children is to create those children\n without any initial coordinate or size settings and do all sizing when layoutChildren() is\n called.\n <P>\n layoutChildren() is always called at least once before children are drawn, and is called\n automatically whenever the viewport size changes (which includes both resizing and\n introduction/removal of scrolling).  layoutChildren() can also be manually invoked in any\n other component-specific situation which changes the layout.\n <P>\n NOTE: layoutChildren() may be called before draw() if a widget is resized before draw(), so\n be sure to avoid errors such as assuming that any children you automatically create have\n already been created.\n <P>\n NOTE: auto-sizing: layoutChildren() is also called once during the initial draw(), before\n children are drawn, with a \"reason\" of \"initial draw\".  During this invocation of\n layoutChildren() it is legal to directly draw children (call child.draw()), which is\n otherwise never allowed.  This allows a Canvas to implement an auto-sizing layout policy by\n drawing some children before deciding on the sizes of remaining children, which is far more\n efficient than drawing all children and resizing some of them after they are drawn.\n",
        name:"layoutChildren",
        seeAlso:[],
        params:[
            {description:"reason why layoutChildren() is being called, passed when framework                        code invokes layoutChildren()",
             optional:false, name:"reason", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:CubeGrid.deselectFacetValues":{
        ref:"method:CubeGrid.deselectFacetValues",
        flags:"",
        description:"\n deselect the header showing a given set of facet values\n\n  <br><i>methodType</i> action\n",
        name:"deselectFacetValues",
        seeAlso:[],
        params:[
            {description:"facetVales to deselect", optional:false, name:"facetValues",
             defaultValue:null, type:"facetValues object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "group:scrolling":{
        ref:"group:scrolling",
        refs:["method:Canvas.getScrollLeft", "method:Canvas.getScrollTop", 
         "method:Canvas.getScrollbarSize","method:Canvas.getScrollBottom", "method:Canvas.getScrollRight", 
         "method:Canvas.scrollToTop","method:Canvas.scrollToBottom", "method:Canvas.scrollToLeft", 
         "method:Canvas.scrollToRight","method:Canvas.scrollBy", "method:Canvas.scrollByPercent", "method:Canvas.scrollTo", 
         "method:Canvas.scrolled","method:Canvas.scrollToPercent", "attr:Canvas.showCustomScrollbars", 
         "attr:Canvas.scrollbarSize"],
        description:"Scrolling and clipping of objects when part of the object is obscured",
        name:"scrolling",
        seeAlso:[],
        type:"group"
    },
    "method:Selection.selectList":{
        ref:"method:Selection.selectList",
        flags:"",
        description:" Select an array of items (subset of the entire list)\n",
        name:"selectList",
        seeAlso:[],
        params:[
            {description:"listarray of objects to select", optional:true, name:"", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:DynamicForm.titleHoverHTML":{
        ref:"method:DynamicForm.titleHoverHTML",
        flags:"A",
        description:"  Retrieves the HTML to display in a hover canvas when the user holds the mousepointer over\n  some item's title.  Return null to suppress the hover canvas altogether.<br>\n  Default implementation returns the prompt for the item if defined.  If no prompt is defined\n  and the item title is clipped, the item title will be shown in a hover by default.<br>\n  Can be overridden via <code>item.titleHoverHTML()</code>\n\n",
        name:"titleHoverHTML",
        seeAlso:[
            {ref:"attr:FormItem.prompt", description:null},
            {ref:"method:FormItem.titleHoverHTML", description:null}
        ],
        params:[
            {description:"Item the user is hovering over.", optional:false, name:"item",
             defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:StretchImgButton.showDown":{
        ref:"attr:StretchImgButton.showDown",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes down in this object?",
        name:"showDown",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "classMethod:Log.logDebug":{
        ref:"classMethod:Log.logDebug",
        flags:"",
        description:" Log a message at \"debug\" priority\n <P>\n A method named log<i>Priority</i> exists for each priority level, on every ISC Class and\n instance of an ISC Class.  Messages logged on a Class or instance have a default\n category of the classname.  Messages logged on an instance will also automatically\n incorporate the instance ID.  General best practice is to call logDebug() et al as\n \"this.logDebug\" whenever \"this\" is an instance, or as \"Log.logDebug\" otherwise.\n\n",
        name:"logDebug",
        seeAlso:[
            {ref:"classMethod:Log.echo", description:"for dumping datastructures to the log"},
            {ref:"classMethod:Log.setPriority",
             description:"for controlling what messages appear in the log"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "method:FormItem.getIconWidth":{
        ref:"method:FormItem.getIconWidth",
        flags:"A",
        description:"	Takes an icon definition object, and returns the width for that icon in px.\n",
        name:"getIconWidth",
        seeAlso:[],
        params:[
            {description:"icon definition object for this item.", optional:false, name:"icon",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"width of the form item icon in px", type:"number"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["sizing"]
    },
    "classMethod:Log.getLogPriorities":{
        ref:"classMethod:Log.getLogPriorities",
        flags:"",
        description:" Get all priority settings as an object mapping category names to priority levels.\n\n",
        name:"getLogPriorities",
        seeAlso:[],
        params:[
            {description:"Optional param to get priorities specific to                                            some ISC class or instance.",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"},
            {description:"If this method is retrieving the priorities specific                                   to logging for some class or instance, this parameter\n                                  can be used to view only the overrides to the default\n                                  log priorites on this object.",
             optional:true, name:"overridesOnly", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"priority settings", type:"Object"},
        type:"classMethod",
        definingClass:"class:Log"
    },
    "method:StatefulCanvas.setBaseStyle":{
        ref:"method:StatefulCanvas.setBaseStyle",
        flags:"",
        description:" Sets the base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n",
        name:"setBaseStyle",
        seeAlso:[],
        params:[
            {description:"new base style", optional:false, name:"style", defaultValue:null,
             type:"className"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas"
    },
    "method:CubeGrid.selectCell":{
        ref:"method:CubeGrid.selectCell",
        flags:"",
        description:"\n  select / deselect a single cell - accepts cell ID or cell record.\n\n      <br><i>methodType</i> action\n",
        name:"selectCell",
        seeAlso:[],
        params:[
            {description:"cell to select / deselect", optional:false, name:"cell", defaultValue:null,
             type:"ID | object"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "group:servletDetails":{
        ref:"group:servletDetails",
        title:"The Core and Optional SmartClient servlets",
        treeLocation:["Concepts/Deploying SmartClient"],
        refs:[],
        description:"The following is a description of the servlets and filters you'll find in the web.xml file\n contained in the smartclientRuntime and what they do:\n <p>\n <i>Core Functionality</i>\n <ul>\n <li>Init servlet- <b>required:</b> initializes the SmartClient server componentry from config files\n at container startup time.  Notice that this servlet has no mapping - it's purely an\n initialization servlet.\n <li>IDACall servlet - <b>required</b> for ${isc.DocUtils.linkForRef('class:DMI')}, built-in RPC operations and built-in DataSource\n operations to work.  All databound examples in the SDK use this servlet.  If you're planning on\n using a custom actionURL for all your RPC requests, then you don't need this servlet.\n <li>FileDownload servlet - required for serving the Isomorphic framework code compressed and with\n caching headers as well as for serving skin images with caching headers.  It is highly\n recommended that you use this for production but is not required. \n <li>PreCache servlet - loads resources into memory on container startup.  Not required, but if you\n exclude this servlet there may be a slow response to the very first request.\n <li>jsp-config section - the iscTaglib registration block is required to use\n &lt;isomorphic&gt; tags, and the *.isc and *.rpc mappings.  These are optional, if you want\n to use these as handy development tools.\n </ul>\n <p>\n <i>Optional Functionality</i>\n <ul>\n <li>HttpProxy - used by the RPCManager when sending AJAX RPCs to a server other than the\n server that serves the main application page.  You need to install this servlet if, for\n example, your application will be querying web services exposed by servers other than the\n server that is serving the rest of the application.  See the javadoc for this servlet for\n various configuration options, such as how to restrict the URLs that are allowed to be\n proxied. \n <li>MessagingServlet - used by the realtime messaging system.  If you're planning\n on using this subsystem, you'll need this servlet.\n <li>CompressionFilter - required if you want to use dynamic compression of html and js\n files.\n <li>JSSyntaxScannerFilter - development tool that looks for trailing commas in JS\n source (scans html files for &lt;script&gt; tags and scans .js files in their entirety).\n This is a useful development tool, but should not be included in production.\n <li>NoCacheFilter - development tool that makes any content it intercepts non-cacheable in\n order to ensure developers are looking at the latest version of a file when modifying\n examples.  Not for production use.\n <li>DataSourceLoader - a servlet that returns the definition of one or more DataSources in\n JavaScript notation.  This servlet is provided as an alternative to using the \n <code>&lt;isomorphic:loadDS&gt;</code> JSP tag, and is particularly suitable in environments\n where JSP tags can't be used for some reason (such as with SmartGWT).  See \n ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','Creating DataSources')} for more details.\n </ul>\n Note that not all of the servlets and filters listed under <i>Optional Functionality</i>\n above are present in the web.xml that ships with the smartclientRuntime - if you need to use\n any of these, copy their configuration from the web.xml available under the WEB-INF\n directory of smartclientSDK.  Other servlets, filters and configuration files from the\n smartclientSDK should not be copied to your deployment, simply because the SDK includes many\n developer tools that are not extensively audited from a security standpoint.",
        name:"servletDetails",
        seeAlso:[],
        type:"group"
    },
    "group:skins":{
        ref:"group:skins",
        refs:["classMethod:Page.setSkinDir", "classMethod:Page.loadStyleSheet"],
        description:"Properties used to manage to the the overall appearance of the application.\n <P>\n A \"skin\" consists of \n <ul>\n <li><code>skin_styles.css</code>: a .css file defining the set of \n     classes to apply to SmartClient components' visual elements</li>\n <li><code>images/</code>: a directory of image files used as part of visual \n     components</li>\n <li><code>load_skin.js</code>: a .js file containing overrides for various \n     SmartClient component properties that affect the appearance of those components.</li>\n </ul>\n Skins are loaded via the <code>skin</code> attribute of the ${isc.DocUtils.linkForRef('group:loadISCTag')} or \n by including the appropriate <code>load_skin.js</code> source file with a standard script\n include tag.\n <P>\n To create a custom skin, we suggest making a complete copy of an existing skin, then\n modifying the media, css class definitions and component property overrides you wish to\n change.\n <P>\n Note that the <code>load_skin.js</code> file contains a ${isc.DocUtils.linkForRef('classMethod:Page.setSkinDir')} \n directive to set up the skin dir (used to ensure media is retrieved from the appropriate\n directory), and a ${isc.DocUtils.linkForRef('classMethod:Page.loadStyleSheet')} directive to load the .css file.\n <P>\n See the ${isc.DocUtils.linkForRef('group:skinning','Skinning Overview')} for more information.",
        name:"skins",
        seeAlso:[
            {ref:"group:appearance", description:null},
            {ref:"group:images", description:null},
            {ref:"group:files", description:null}
        ],
        type:"group"
    },
    "class:ListGrid":{
        exampleConfig:"ListGrid.xml",
        methods:["method:ListGrid.markForRedraw", "method:ListGrid.getVisibleRows", 
         "method:ListGrid.getDrawnRows","method:ListGrid.getCellStyle", "method:ListGrid.getCellCSSText", 
         "method:ListGrid.refreshCellStyle","method:ListGrid.cellOver", "method:ListGrid.rowOver", "method:ListGrid.cellOut", 
         "method:ListGrid.rowOut","method:ListGrid.cellHover", "method:ListGrid.rowHover", "method:ListGrid.cellHoverHTML", 
         "method:ListGrid.cellContextClick","method:ListGrid.rowContextClick", "method:ListGrid.cellMouseDown", 
         "method:ListGrid.rowMouseDown","method:ListGrid.cellMouseUp", "method:ListGrid.rowMouseUp", "method:ListGrid.cellClick", 
         "method:ListGrid.cellDoubleClick","method:ListGrid.getFormulaFieldValue", "method:ListGrid.getSummaryFieldValue", 
         "method:ListGrid.getExpansionField","method:ListGrid.isExpansionField", "method:ListGrid.canExpandRecord", 
         "method:ListGrid.expandRecord","method:ListGrid.collapseRecord", "method:ListGrid.getExpansionComponent", 
         "method:ListGrid.getRelatedDataSource","method:ListGrid.setData", "method:ListGrid.setSelectionType", 
         "method:ListGrid.setSelectionAppearance","method:ListGrid.setBodyOverflow", "method:ListGrid.setBodyStyleName", 
         "method:ListGrid.setAlternateBodyStyleName","method:ListGrid.setAlternateRecordStyles", "method:ListGrid.setAutoFitData", 
         "method:ListGrid.setAutoFitExtraRecords","method:ListGrid.setAutoFitMaxRecords", "method:ListGrid.setAutoFitMaxHeight", 
         "method:ListGrid.setAutoFitMaxColumns","method:ListGrid.setAutoFitMaxWidth", "method:ListGrid.setFieldProperties", 
         "method:ListGrid.setFieldTitle","method:ListGrid.showField", "method:ListGrid.hideField", "method:ListGrid.fieldIsVisible", 
         "method:ListGrid.getTitleField","method:ListGrid.setFields", "method:ListGrid.focusInFilterEditor", 
         "method:ListGrid.filterByEditor","method:ListGrid.getFieldState", "method:ListGrid.setFieldState", 
         "method:ListGrid.getCheckboxField","method:ListGrid.isCheckboxField", "method:ListGrid.getSelectedState", 
         "method:ListGrid.setSelectedState","method:ListGrid.getSortState", "method:ListGrid.setSortState", 
         "method:ListGrid.getViewState","method:ListGrid.setViewState", "method:ListGrid.refreshFields", 
         "method:ListGrid.getFieldWidth","method:ListGrid.getBaseStyle", "method:ListGrid.getCellValue", 
         "method:ListGrid.getCellAlign","method:ListGrid.getCellVAlign", "method:ListGrid.getValueIcon", 
         "method:ListGrid.rowClick","method:ListGrid.rowDoubleClick", "method:ListGrid.selectRecord", 
         "method:ListGrid.deselectRecord","method:ListGrid.selectRecords", "method:ListGrid.deselectRecords", 
         "method:ListGrid.selectAllRecords","method:ListGrid.deselectAllRecords", "method:ListGrid.anySelected", 
         "method:ListGrid.selectSingleRecord","method:ListGrid.keyPress", "method:ListGrid.bodyKeyPress", "method:ListGrid.getFocusRow", 
         "method:ListGrid.recordClick","method:ListGrid.recordDoubleClick", "method:ListGrid.setShowGridSummary", 
         "method:ListGrid.getGridSummaryFunction","method:ListGrid.getGridSummary", "method:ListGrid.setShowFilterEditor", 
         "method:ListGrid.getFilterEditorValueMap","method:ListGrid.getFilterEditorType", "method:ListGrid.setFilterEditorCriteria", 
         "method:ListGrid.getFilterEditorCriteria","method:ListGrid.canEditCell", "method:ListGrid.startEditing", "method:ListGrid.stopHover", 
         "method:ListGrid.getEditorValueMap","method:ListGrid.getEditorValueIcons", "method:ListGrid.setEditorValueMap", 
         "method:ListGrid.getEditorType","method:ListGrid.refreshCell", "method:ListGrid.refreshRow", 
         "method:ListGrid.startEditingNew","method:ListGrid.getAllEditRows", "method:ListGrid.getEditValues", 
         "method:ListGrid.getEditedRecord","method:ListGrid.getEditedCell", "method:ListGrid.setEditValues", 
         "method:ListGrid.setEditValue","method:ListGrid.getEditValue", "method:ListGrid.clearEditValue", 
         "method:ListGrid.getEditRow","method:ListGrid.getEditCol", "method:ListGrid.getEditField", 
         "method:ListGrid.cancelEditing","method:ListGrid.endEditing", "method:ListGrid.discardAllEdits", 
         "method:ListGrid.discardEdits","method:ListGrid.saveEdits", "method:ListGrid.rowHasChanges", "method:ListGrid.hasChanges", 
         "method:ListGrid.cellHasChanges","method:ListGrid.saveAllEdits", "method:ListGrid.validateRow", 
         "method:ListGrid.validateCell","method:ListGrid.getRequiredFieldMessage", "method:ListGrid.hasErrors", 
         "method:ListGrid.rowHasErrors","method:ListGrid.cellHasErrors", "method:ListGrid.getRowErrors", 
         "method:ListGrid.getCellErrors","method:ListGrid.setFieldError", "method:ListGrid.setRowErrors", 
         "method:ListGrid.clearFieldError","method:ListGrid.clearRowErrors", "method:ListGrid.removeData", 
         "method:ListGrid.setDragTracker","method:ListGrid.getDragTrackerIcon", "method:ListGrid.getDragTrackerTitle", 
         "method:ListGrid.willAcceptDrop","method:ListGrid.transferSelectedData", "method:ListGrid.recordDrop", 
         "method:ListGrid.transferDragData","method:ListGrid.getDragData", "method:ListGrid.getRecord", 
         "method:ListGrid.getCellRecord","method:ListGrid.getTotalRows", "method:ListGrid.getAllFields", 
         "method:ListGrid.getFieldName","method:ListGrid.getField", "method:ListGrid.getFieldNum", 
         "method:ListGrid.getSummaryTitle","method:ListGrid.setValueMap", "method:ListGrid.getDisplayValue", 
         "method:ListGrid.getRecordIndex","method:ListGrid.getEventRow", "method:ListGrid.getEventColumn", 
         "method:ListGrid.getDropIndex","method:ListGrid.selectionChanged", "method:ListGrid.setCanFreezeFields", 
         "method:ListGrid.freezeField","method:ListGrid.unfreezeField", "method:ListGrid.toggleFrozen", 
         "method:ListGrid.setHeaderHeight","method:ListGrid.setShowHeader", "method:ListGrid.headerClick", 
         "method:ListGrid.reorderField","method:ListGrid.reorderFields", "method:ListGrid.setCanResizeFields", 
         "method:ListGrid.resizeField","method:ListGrid.displayHeaderContextMenu", "method:ListGrid.getToggleFreezeText", 
         "method:ListGrid.getGroupByText","method:ListGrid.getHeaderSpanContextMenuItems", 
         "method:ListGrid.getHeaderContextMenuItems","method:ListGrid.unsort", "method:ListGrid.sort", "method:ListGrid.toggleSort", 
         "method:ListGrid.addSort","method:ListGrid.getSortSpecifier", "method:ListGrid.getSortFieldCount", 
         "method:ListGrid.isSortField","method:ListGrid.getSortNumeralHTML", "method:ListGrid.setSort", 
         "method:ListGrid.chartData","method:ListGrid.chartRow", "method:ListGrid.chartColumn", "method:ListGrid.groupBy", 
         "method:ListGrid.getGroupNodeHTML","method:ListGrid.ungroup", "method:ListGrid.setHeaderSpans", 
         "method:ListGrid.setHeaderSpanTitle","method:ListGrid.cellChanged", "method:ListGrid.editComplete", 
         "method:ListGrid.editFailed","method:ListGrid.editorEnter", "method:ListGrid.rowEditorEnter", 
         "method:ListGrid.editorExit","method:ListGrid.rowEditorExit", "method:ListGrid.formatCellValue", 
         "method:ListGrid.formatEditorValue","method:ListGrid.parseEditorValue", "method:ListGrid.fieldStateChanged", 
         "method:ListGrid.dataArrived","method:ListGrid.fetchData", "method:ListGrid.exportData", "method:ListGrid.filterData", 
         "method:ListGrid.fetchRelatedData","method:ListGrid.clearCriteria", "method:ListGrid.getCriteria", 
         "method:ListGrid.setCriteria","method:ListGrid.invalidateCache", "method:ListGrid.willFetchData", 
         "method:ListGrid.addData","method:ListGrid.updateData", "method:ListGrid.removeSelectedData", 
         "method:ListGrid.getSelection","method:ListGrid.getSelectedRecord"],
        type:"class",
        ref:"class:ListGrid",
        classAttrs:[],
        treeLocation:["Client Reference/Grids"],
        classMethods:[],
        description:"A ListGrid is a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} that displays a list of objects in a grid, where\n each row represents one object and each cell in the row represents one property.",
        attrs:["attr:ListGrid.fastCellUpdates", "attr:ListGrid.styleName", "attr:ListGrid.data", 
         "attr:ListGrid.recordEnabledProperty","attr:ListGrid.canExpandRecordProperty", "attr:ListGrid.fields", 
         "attr:ListGrid.defaultFields","attr:ListGrid.dataSource", "attr:ListGrid.autoFetchDisplayMap", 
         "attr:ListGrid.saveLocally","attr:ListGrid.useAllDataSourceFields", "attr:ListGrid.showDetailFields", 
         "attr:ListGrid.titleField","attr:ListGrid.dataProperties", "attr:ListGrid.originalData", "attr:ListGrid.groupTree", 
         "attr:ListGrid.groupStartOpen","attr:ListGrid.groupNodeStyle", "attr:ListGrid.groupIcon", "attr:ListGrid.groupIconSize", 
         "attr:ListGrid.groupIndentSize","attr:ListGrid.groupLeadingIndent", "attr:ListGrid.canGroupBy", 
         "attr:ListGrid.groupByMaxRecords","attr:ListGrid.isGrouped", "attr:ListGrid.nullGroupTitle", "attr:ListGrid.groupByField", 
         "attr:ListGrid.valueIconSize","attr:ListGrid.valueIconWidth", "attr:ListGrid.valueIconHeight", 
         "attr:ListGrid.valueIconLeftPadding","attr:ListGrid.valueIconRightPadding", "attr:ListGrid.imageSize", 
         "attr:ListGrid.headerSpans","attr:ListGrid.headerSpanHeight", "attr:ListGrid.headerSpanVAlign", 
         "attr:ListGrid.unspannedHeaderVAlign","attr:ListGrid.headerSpanConstructor", "attr:ListGrid.headerSpan", 
         "attr:ListGrid.showTreeColumnPicker","attr:ListGrid.modalEditing", "attr:ListGrid.fetchDelay", "attr:ListGrid.showAllRecords", 
         "attr:ListGrid.drawAllMaxCells","attr:ListGrid.drawAheadRatio", "attr:ListGrid.quickDrawAheadRatio", 
         "attr:ListGrid.scrollRedrawDelay","attr:ListGrid.dataPageSize", "attr:ListGrid.body", "attr:ListGrid.bodyOverflow", 
         "attr:ListGrid.bodyBackgroundColor","attr:ListGrid.bodyStyleName", "attr:ListGrid.emptyCellValue", "attr:ListGrid.cellHeight", 
         "attr:ListGrid.fixedRecordHeights","attr:ListGrid.enforceVClipping", "attr:ListGrid.fixedFieldWidths", 
         "attr:ListGrid.autoFitData","attr:ListGrid.autoFitMaxHeight", "attr:ListGrid.autoFitMaxRecords", 
         "attr:ListGrid.autoFitExtraRecords","attr:ListGrid.autoFitMaxWidth", "attr:ListGrid.autoFitMaxColumns", 
         "attr:ListGrid.leaveScrollbarGap","attr:ListGrid.wrapCells", "attr:ListGrid.cellPadding", "attr:ListGrid.dateFormatter", 
         "attr:ListGrid.datetimeFormatter","attr:ListGrid.dateInputFormat", "attr:ListGrid.timeFormatter", 
         "attr:ListGrid.linkTextProperty","attr:ListGrid.baseStyle", "attr:ListGrid.editFailedBaseStyle", 
         "attr:ListGrid.editFailedCSSText","attr:ListGrid.editPendingBaseStyle", "attr:ListGrid.editPendingCSSText", 
         "attr:ListGrid.recordCustomStyleProperty","attr:ListGrid.recordBaseStyleProperty", "attr:ListGrid.frozenBaseStyle", 
         "attr:ListGrid.shrinkForFreeze","attr:ListGrid.alternateRecordStyles", "attr:ListGrid.alternateBodyStyleName", 
         "attr:ListGrid.alternateRecordFrequency","attr:ListGrid.generateClickOnSpace", "attr:ListGrid.generateClickOnEnter", 
         "attr:ListGrid.generateDoubleClickOnSpace","attr:ListGrid.generateDoubleClickOnEnter", "attr:ListGrid.arrowKeyAction", 
         "attr:ListGrid.showRollOver","attr:ListGrid.canHover", "attr:ListGrid.showHover", "attr:ListGrid.hoverStyle", 
         "attr:ListGrid.selection","attr:ListGrid.selectionAppearance", "attr:ListGrid.canSelectAll", 
         "attr:ListGrid.selectionType","attr:ListGrid.canDragSelect", "attr:ListGrid.canDragSelectText", 
         "attr:ListGrid.checkboxField","attr:ListGrid.checkboxFieldTrueImage", "attr:ListGrid.checkboxFieldFalseImage", 
         "attr:ListGrid.checkboxFieldImageWidth","attr:ListGrid.checkboxFieldImageHeight", "attr:ListGrid.showEmptyMessage", 
         "attr:ListGrid.emptyMessage","attr:ListGrid.emptyMessageStyle", "attr:ListGrid.loadingDataMessage", 
         "attr:ListGrid.loadingDataMessageStyle","attr:ListGrid.loadingMessage", "attr:ListGrid.singleCellValueProperty", 
         "attr:ListGrid.isSeparatorProperty","attr:ListGrid.showFilterEditor", "attr:ListGrid.filterEditorHeight", 
         "attr:ListGrid.autoFetchTextMatchStyle","attr:ListGrid.canEdit", "attr:ListGrid.recordEditProperty", 
         "attr:ListGrid.alwaysShowEditors","attr:ListGrid.editByCell", "attr:ListGrid.saveByCell", "attr:ListGrid.validateByCell", 
         "attr:ListGrid.validateOnChange","attr:ListGrid.neverValidate", "attr:ListGrid.canRemoveRecords", 
         "attr:ListGrid.removeIcon","attr:ListGrid.animateRemoveRecord", "attr:ListGrid.animateRemoveTime", 
         "attr:ListGrid.animateRemoveSpeed","attr:ListGrid.removeFieldTitle", "attr:ListGrid.removeFieldDefaults", 
         "attr:ListGrid.removeFieldProperties","attr:ListGrid.filterByCell", "attr:ListGrid.filterOnKeypress", 
         "attr:ListGrid.waitForSave","attr:ListGrid.stopOnErrors", "attr:ListGrid.autoSaveEdits", 
         "attr:ListGrid.confirmCancelEditing","attr:ListGrid.cancelEditingConfirmationMessage", "attr:ListGrid.confirmDiscardEdits", 
         "attr:ListGrid.confirmDiscardEditsMessage","attr:ListGrid.discardEditsSaveButtonTitle", "attr:ListGrid.rowEndEditAction", 
         "attr:ListGrid.listEndEditAction","attr:ListGrid.enterKeyEditAction", "attr:ListGrid.escapeKeyEditAction", 
         "attr:ListGrid.editEvent","attr:ListGrid.editOnFocus", "attr:ListGrid.selectOnEdit", 
         "attr:ListGrid.longTextEditorThreshold","attr:ListGrid.longTextEditorType", "attr:ListGrid.header", "attr:ListGrid.headerHeight", 
         "attr:ListGrid.minFieldWidth","attr:ListGrid.showHeader", "attr:ListGrid.headerBarStyle", 
         "attr:ListGrid.headerBackgroundColor","attr:ListGrid.headerButtonConstructor", "attr:ListGrid.headerBaseStyle", 
         "attr:ListGrid.headerTitleStyle","attr:ListGrid.frozenHeaderBaseStyle", "attr:ListGrid.frozenHeaderTitleStyle", 
         "attr:ListGrid.headerButtonDefaults","attr:ListGrid.headerButtonProperties", "attr:ListGrid.sorterConstructor", 
         "attr:ListGrid.sorterDefaults","attr:ListGrid.sorterProperties", "attr:ListGrid.canSort", "attr:ListGrid.sortFieldNum", 
         "attr:ListGrid.sortField","attr:ListGrid.sortDirection", "attr:ListGrid.showSortArrow", 
         "attr:ListGrid.canPickFields","attr:ListGrid.canFreezeFields", "attr:ListGrid.canAddFormulaFields", 
         "attr:ListGrid.canAddSummaryFields","attr:ListGrid.showHeaderContextMenu", "attr:ListGrid.showHeaderMenuButton", 
         "attr:ListGrid.headerMenuButtonConstructor","attr:ListGrid.headerMenuButton", "attr:ListGrid.headerMenuButtonIcon", 
         "attr:ListGrid.headerMenuButtonIconWidth","attr:ListGrid.headerMenuButtonIconHeight", "attr:ListGrid.headerMenuButtonWidth", 
         "attr:ListGrid.headerMenuButtonHeight","attr:ListGrid.canDragRecordsOut", "attr:ListGrid.canAcceptDroppedRecords", 
         "attr:ListGrid.canReorderRecords","attr:ListGrid.canReorderFields", "attr:ListGrid.canResizeFields", 
         "attr:ListGrid.dragTrackerMode","attr:ListGrid.resizeFieldsInRealTime", "attr:ListGrid.dragDataAction", 
         "attr:ListGrid.skinImgDir","attr:ListGrid.sortAscendingImage", "attr:ListGrid.sortDescendingImage", 
         "attr:ListGrid.trackerImage","attr:ListGrid.booleanTrueImage", "attr:ListGrid.booleanFalseImage", 
         "attr:ListGrid.booleanImageWidth","attr:ListGrid.booleanImageHeight", "attr:ListGrid.canExpandRecords", 
         "attr:ListGrid.detailField","attr:ListGrid.expansionMode", "attr:ListGrid.expansionField", 
         "attr:ListGrid.expansionFieldTrueImage","attr:ListGrid.expansionFieldFalseImage", "attr:ListGrid.expansionFieldImageWidth", 
         "attr:ListGrid.expansionFieldImageHeight","attr:ListGrid.expansionDetailField", "attr:ListGrid.expansionDetails", 
         "attr:ListGrid.expansionRelated","attr:ListGrid.expansionEditor", "attr:ListGrid.expansionDetailRelated", 
         "attr:ListGrid.expansionLayout","attr:ListGrid.recordDetailDSProperty", "attr:ListGrid.childExpansionMode", 
         "attr:ListGrid.expansionCanEdit","attr:ListGrid.showGridSummary", "attr:ListGrid.invalidSummaryValue", 
         "attr:ListGrid.includeInSummaryProperty","attr:ListGrid.gridSummaryRecordProperty", "attr:ListGrid.groupSummaryRecordProperty", 
         "attr:ListGrid.summaryRowDataSource","attr:ListGrid.summaryRowCriteria", "attr:ListGrid.summaryRowFetchRequestProperties", 
         "attr:ListGrid.recordSummaryBaseStyle","attr:ListGrid.summaryRow", "attr:ListGrid.summaryRowHeight", 
         "attr:ListGrid.summaryRowStyle","attr:ListGrid.dragTrackerStyle", "attr:ListGrid.freezeOnRightText", 
         "attr:ListGrid.freezeOnLeftText","attr:ListGrid.sortFieldAscendingText", "attr:ListGrid.sortFieldDescendingText", 
         "attr:ListGrid.clearSortFieldText","attr:ListGrid.clearAllSortingText", "attr:ListGrid.clearFilterText", 
         "attr:ListGrid.fieldVisibilitySubmenuTitle","attr:ListGrid.freezeFieldText", "attr:ListGrid.unfreezeFieldText", 
         "attr:ListGrid.groupByText","attr:ListGrid.ungroupText", "attr:ListGrid.canMultiSort", "attr:ListGrid.initialSort", 
         "attr:ListGrid.sortNumeralStyle","attr:ListGrid.showGroupSummary", "attr:ListGrid.groupSummaryStyle", 
         "attr:ListGrid.autoFetchData","attr:ListGrid.initialCriteria"],
        implementsInterfaces:["DataBoundComponent"],
        name:"ListGrid",
        seeAlso:[]
    },
    "attr:Calendar.cancelButtonTitle":{
        ref:"attr:Calendar.cancelButtonTitle",
        flags:"IR",
        description:"The title for the cancel button in the event editor",
        name:"cancelButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Cancel\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "classMethod:Class.logIsWarnEnabled":{
        ref:"classMethod:Class.logIsWarnEnabled",
        flags:"",
        description:" Check whether a message logged at \"warn\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsWarnEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:SectionStack.sections":{
        ref:"attr:SectionStack.sections",
        flags:"IR",
        getter:["getSections"],
        description:"List of sections of components managed by this SectionStack.",
        name:"sections",
        seeAlso:[
            {ref:"method:SectionStack.getSections", description:null}
        ],
        valueType:"Array of SectionStackSection Properties",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"],
        definingClass:"class:SectionStack"
    },
    "attr:Menu.showSubmenus":{
        ref:"attr:Menu.showSubmenus",
        flags:"IRW",
        description:"A boolean, indicating whether the submenu indicator column should be displayed. If\n showSubmenus is not set, the menu will show the indicator column only if one of its\n items specifies a submenu property. If showSubmenus is false, the submenu arrows\n will not be displayed, but submenus will still appear on rollover.",
        name:"showSubmenus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:SimpleType.editorType":{
        ref:"attr:SimpleType.editorType",
        flags:"IR",
        description:"Classname of the FormItem that should be the default for editing values of this type (eg\n \"SelectItem\").\n <P>\n You can create a simple custom FormItem by adding default ${isc.DocUtils.linkForRef('attr:FormItem.icons')} that\n launch custom value picking dialogs (an example is in the <i>QuickStart\n Guide</i>, Chapter 9, <i>Extending SmartClient</i>).  By setting simpleType.editorType\n to the name of your custom FormItem, forms will automatically use the custom FormItem,\n as will grids performing ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit','inline editing')}.",
        name:"editorType",
        seeAlso:[],
        valueType:"FormItem ClassName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleType"
    },
    "attr:FormItem.pickerIconSrc":{
        ref:"attr:FormItem.pickerIconSrc",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n src of the picker icon image to be displayed.",
        name:"pickerIconSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:Canvas.snapEdge":{
        flags:"IRW",
        getter:["getSnapEdge"],
        type:"attr",
        ref:"attr:Canvas.snapEdge",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')} is defined to this widget, this property can be used to\n define which edge of this widget should be snapped to an edge of the master or parent \n element.\n <P>\n If unspecified the, default snapTo behavior is set up to align the \"snapTo\" edge of this \n widget with the snapTo edge of the master or parent.",
        name:"snapEdge",
        setter:["setSnapEdge"],
        valueType:"String",
        seeAlso:[
            {ref:"attr:Canvas.snapTo", description:null}
        ],
        defaultValue:"null",
        groups:["sizing"],
        definingClass:"class:Canvas"
    },
    "method:Tree.indexOf":{
        ref:"method:Tree.indexOf",
        flags:"",
        description:" Return the position in the list of the first instance of the specified object.\n <p>\n If pos is specified, starts looking after that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"indexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"earliest index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"last index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:Tree",
        groups:["access"]
    },
    "attr:TreeGrid.folderIcon":{
        ref:"attr:TreeGrid.folderIcon",
        flags:"IRW",
        description:"The URL of the base icon for all folder nodes in this treeGrid. Note that this URL will\n have ${isc.DocUtils.linkForRef('attr:TreeGrid.openIconSuffix')}, ${isc.DocUtils.linkForRef('attr:TreeGrid.closedIconSuffix')} or \n ${isc.DocUtils.linkForRef('attr:TreeGrid.dropIconSuffix')} appended to indicate state changes if appropriate - \n see documentation on  ${isc.DocUtils.linkForRef('attr:TreeGrid.showOpenIcons')} and ${isc.DocUtils.linkForRef('attr:TreeGrid.showDropIcons')}.",
        name:"folderIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]folder.gif\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:ListGrid.editPendingCSSText":{
        ref:"attr:ListGrid.editPendingCSSText",
        flags:"IRWA",
        description:"Custom CSS text to be applied to cells with pending edits that have not yet been\n  submitted.<br>\n For further customization of styling for cells with pending edits use\n <code>this.editPendingBaseStyle</code> instead.",
        name:"editPendingCSSText",
        seeAlso:[
            {ref:"attr:ListGrid.editFailedBaseStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"\"border:color:#0066CC;\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:CubeGrid.canMoveFacets":{
        ref:"attr:CubeGrid.canMoveFacets",
        flags:"IRW",
        description:"Whether row and column facets can be rearranged by the user, by dragging and dropping the\n facet labels.",
        name:"canMoveFacets",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:Button.showDown":{
        ref:"attr:Button.showDown",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes down in this object?",
        name:"showDown",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonStates')}"],
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:LoginDialog.usernameItemTitle":{
        ref:"attr:LoginDialog.usernameItemTitle",
        flags:"IR",
        description:"Specifies the title of the \"usernameItem\" field of the ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"usernameItemTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Dialog.USERNAME_TITLE",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:ListGrid.endEditing":{
        ref:"method:ListGrid.endEditing",
        flags:"",
        description:"\n Complete the current edit by storing the value and hiding the inline editor. Note that\n if ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')} is true, the value will be saved to the server. \n\n",
        name:"endEditing",
        seeAlso:[
            {ref:"method:ListGrid.startEditing", description:null}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:GridRenderer.snapInsideBorder":{
        ref:"attr:GridRenderer.snapInsideBorder",
        flags:"IRW",
        description:"If true, snap-to-cell drops will snap the dropped object inside the selected cell's border.\n If false, snap-to-cell drops will snap the dropped object to the edge of the selected cell, \n regardless of borders",
        name:"snapInsideBorder",
        seeAlso:[
            {ref:"attr:GridRenderer.snapToCells", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["dragdrop"]
    },
    "attr:OperationBinding.outputs":{
        ref:"attr:OperationBinding.outputs",
        flags:"IR",
        description:"Specifies, for this operationBinding only, the list of field names that should be returned\n to the client.  Typically this will be a subset of the ${isc.DocUtils.linkForRef('attr:DataSource.fields')}, but note \n that this is not a requirement; <code>outputs</code> can include fields that are not defined\n in the DataSource's field list.  In this case, the server will return extra fields even if \n ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')} is true.\n <p>\n You specify this property as a string containing a comma-separated list of field names \n (eg, \"foo, bar, baz\")",
        name:"outputs",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:OperationBinding"
    },
    "attr:CubeGrid.bodyMinWidth":{
        ref:"attr:CubeGrid.bodyMinWidth",
        flags:"IRWA",
        description:"Minimum width for the body of this cubeGrid",
        name:"bodyMinWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "method:HTMLFlow.handleError":{
        ref:"method:HTMLFlow.handleError",
        flags:"",
        description:"\n This method is called when a transport error occurs.  Typically, this is the result of the\n server returning an HTTP error code such as 404 - document not found.  You can inspect the\n RPCResponse object for the reasons for the error and take appropriate action.  Typical\n properties to look at are rpcResponse.status, and rpcResponse.httpResponseCode.\n <p>\n This method is called from the response processing pipeline.  If you want to provide your\n own HTML response that should be rendered into this component as the result of the error,\n you can do so by setting rpcResponse.data to your HTML string.  Returning false from this\n method suppresses any further response handling.  The default implementation of this method\n causes an error message to be logged to the Developer Console and sets the HTML to the error\n string.\n\n",
        name:"handleError",
        seeAlso:[],
        params:[
            {description:"The RPCRequest that was made to the server", optional:false,
             name:"rpcRequest", defaultValue:null, type:"RPCRequest"},
            {description:"The RPCResponse that was received", optional:false, name:"rpcResponse",
             defaultValue:null, type:"RPCResponse"}
        ],
        returns:{description:"false to suppress further response processing", type:"boolean"},
        type:"method",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:EdgedCanvas.showCenter":{
        ref:"attr:EdgedCanvas.showCenter",
        flags:"IR",
        description:"Whether to show media in the center section, that is, behind the decorated Canvas.",
        name:"showCenter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "method:Tree.linkNodes":{
        ref:"method:Tree.linkNodes",
        flags:"",
        description:"\n This method is provided as a mechanism to link new nodes into a tree of modelType\n \"parent\".  This method takes as a parameter a list of nodes that must contain, at a minimum, \n a unique ID (keyed by ${isc.DocUtils.linkForRef('attr:Tree.idField')}) and a parent ID (keyed by\n ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')}).  Based on this information, the list of nodes is integrated\n into the tree structure.\n\n",
        name:"linkNodes",
        seeAlso:[
            {ref:"attr:Tree.data", description:null},
            {ref:"attr:Tree.modelType", description:null}
        ],
        params:[
            {description:"list of nodes to link into the tree.", optional:false, name:"nodes",
             defaultValue:null, type:"Array of TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "group:dragResize":{
        ref:"group:dragResize",
        refs:["classMethod:Canvas.getEventEdge"],
        name:"dragResize",
        seeAlso:[],
        type:"group"
    },
    "attr:FormItem.titleOrientation":{
        ref:"attr:FormItem.titleOrientation",
        flags:"IRW",
        description:"On which side of this item should the title be placed.  ${isc.DocUtils.linkForRef('type:TitleOrientation')}\n lists valid options.\n <P>\n Note that titles on the left or right take up a cell in tabular\n ${isc.DocUtils.linkForRef('group:formLayout','form layouts')}, but titles on top do not.",
        name:"titleOrientation",
        seeAlso:[
            {ref:"attr:DynamicForm.titleOrientation", description:null}
        ],
        valueType:"TitleOrientation",
        defaultValue:"Canvas.LEFT",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "method:Toolbar.getButtonNumber":{
        ref:"method:Toolbar.getButtonNumber",
        flags:"A",
        description:"			get the index of a button in the buttons array<p>\n          The button can be specified as - \n          <ul>\n          <li>an index within this.buttons (just returned)\n          <li>the ID property of a button\n          <li>a pointer to the button descriptor object in this.buttons\n          <li>the actual button widget in this.members\n          </ul><p>\n			returns -1 if not found\n\n",
        name:"getButtonNumber",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"button", defaultValue:null,
             type:"number | string  | button object | button widget"}
        ],
        returns:{description:"index of the button in question", type:"number"},
        type:"method",
        definingClass:"class:Toolbar"
    },
    "attr:ListGrid.sortFieldNum":{
        deprecated:"as of version 7.0 in favor of ${isc.DocUtils.linkForRef('attr:ListGrid.sortField')}",
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.sortFieldNum",
        description:"Specifies the number of the field by which to sort this listGrid. Column numbers\n          start at 0 for the left-most column.",
        name:"sortFieldNum",
        valueType:"number",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        defaultValue:"null",
        groups:["sorting"],
        definingClass:"class:ListGrid"
    },
    "method:FormItem.updateState":{
        ref:"method:FormItem.updateState",
        flags:"A",
        description:" Update the visual state of a FormItem to reflect any changes in state or any changes in\n style settings (eg ${isc.DocUtils.linkForRef('attr:FormItem.textBoxStyle')}). \n <P>\n Calls to <code>updateState()</code> normally occur automatically as a consequence of\n focus changes, items becoming disabled, etc.  This method is advanced and intended only\n for use in workarounds.\n\n",
        name:"updateState",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:DatabaseBrowser.includeSubstring":{
        ref:"attr:DatabaseBrowser.includeSubstring",
        flags:"IR",
        description:"If set, specifies a substring which must exist in a table name for it to be included in\n this DatabaseBrowser.  If this property is set to a List of strings, table names are\n included if they match any one of the strings.  The comparison is case-insensitive.\n <p>\n For example, <code>includeSubstring: [\"E\", \"qry\"]</code> would match all the following \n table names: \"table\", \"QryTbl\", \"QRY\", \"ORDERS\"",
        name:"includeSubstring",
        seeAlso:[],
        valueType:"String | List of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:FormItem.errorMessageWidth":{
        ref:"attr:FormItem.errorMessageWidth",
        flags:"IRW",
        description:"When ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} and ${isc.DocUtils.linkForRef('attr:FormItem.showErrorText')} are both true and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorOrientation')} is \"left\" or \"right\", errorMessageWidth is the amount to reduce\n the width of the editor to accomodate the error message and icon.",
        name:"errorMessageWidth",
        seeAlso:[],
        valueType:"int",
        defaultValue:"80",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["validation"]
    },
    "method:DataBoundComponent.getField":{
        ref:"method:DataBoundComponent.getField",
        flags:"",
        description:" Return a field by a field index or field name.\n\n",
        name:"getField",
        seeAlso:[],
        params:[
            {description:"field index or field.name", optional:false, name:"fieldID", defaultValue:null,
             type:"String || Number"}
        ],
        returns:{description:"Field description", type:"object"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:SectionItem.sectionExpanded":{
        ref:"attr:SectionItem.sectionExpanded",
        flags:"IR",
        description:"Whether this form section should be initially collapsed. Can be set programatically\n via ${isc.DocUtils.linkForRef('method:SectionItem.expandSection')} and ${isc.DocUtils.linkForRef('method:SectionItem.collapseSection')}.",
        name:"sectionExpanded",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formSections')}"],
        definingClass:"class:SectionItem"
    },
    "attr:StatefulCanvas.iconOrientation":{
        ref:"attr:StatefulCanvas.iconOrientation",
        flags:"IR",
        description:"If this button is showing an icon should it appear to the left or right of the title?\n valid options are <code>\"left\"</code> and <code>\"right\"</code>.",
        name:"iconOrientation",
        setter:["setIconOrientation"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "class:FusionChart":{
        methods:["method:FusionChart.getDataColor"],
        type:"class",
        requiresModules:["Analytics"],
        ref:"class:FusionChart",
        classAttrs:[],
        treeLocation:["Optional Modules/Analytics"],
        classMethods:[],
        description:"Component that wraps the FusionCharts charting engine.\n <P>\n Provides multiple-series (area, bar, column, line, radar) and single-series (doughnut, pie)\n chart types. These charts all depict a single continuous dimension (eg revenue), broken down\n by one or two discrete dimensions (eg product, region).",
        attrs:["attr:FusionChart.chartsBaseURL", "attr:FusionChart.chartURL", 
         "attr:FusionChart.fusionVersion","attr:FusionChart.chartProperties", "attr:FusionChart.dataColors"],
        implementsInterfaces:["Chart"],
        name:"FusionChart",
        seeAlso:[]
    },
    "method:GridRenderer.cellMouseDown":{
        ref:"method:GridRenderer.cellMouseDown",
        flags:"A",
        description:" Called when a cell receives a mousedown event.\n \n",
        name:"cellMouseDown",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:FormItem.showFocused":{
        ref:"attr:FormItem.showFocused",
        flags:"IRWA",
        description:"When this item recieves focus, should it be re-styled to indicate it has focus?",
        name:"showFocused",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:MenuItem.checkIf":{
        ref:"method:MenuItem.checkIf",
        flags:"",
        description:" Contains the condition that will check or uncheck the curent menuItem. The handler must be specified\n as a function or string of script.  Return false to uncheck the menuItem or true to check it\n <p>\n If you don't need to set this state dynamically, use ${isc.DocUtils.linkForRef('attr:MenuItem.checked')} instead.\n <p>\n May be defined as a ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')}.\n <p>\n",
        name:"checkIf",
        seeAlso:[],
        params:[
            {description:"${isc.DocUtils.linkForRef('attr:Menu.target','target')} attribute for the top level menu.",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"${isc.DocUtils.linkForRef('class:Menu','menu')} contains the reference to the menu that contains the current item",
             optional:false, name:"menu", defaultValue:null, type:"Menu"},
            {description:"contains the reference to the current item", optional:false, name:"item",
             defaultValue:null, type:"MenuItem"}
        ],
        returns:{description:"Return true to show a checkmark by this menu item", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('menuDynamicItems')}"],
        definingClass:"object:MenuItem",
        groups:["dynamicMenuItem"]
    },
    "attr:ImgButton.align":{
        ref:"attr:ImgButton.align",
        flags:"IRW",
        description:"Horizontal alignment of this component's title.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["appearance"]
    },
    "attr:CubeGrid.canReorderColumns":{
        ref:"attr:CubeGrid.canReorderColumns",
        flags:"IRW",
        description:"If true, body columns can be reordered via the innermost column headers.",
        name:"canReorderColumns",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:Canvas.shadowDepth":{
        ref:"attr:Canvas.shadowDepth",
        flags:"IR",
        description:"Depth of the shadow, or the virtual height above the page of the widget throwing the shadow.\n <P>\n This is a single parameter that can be used to control both <code>shadowSoftness</code> and\n <code>shadowOffset</code>.",
        name:"shadowDepth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"4",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["shadow"]
    },
    "method:Canvas.setSnapTo":{
        ref:"method:Canvas.setSnapTo",
        flags:"",
        description:" Set the snapTo property of this canvas, and handle repositioning.\n\n",
        name:"setSnapTo",
        seeAlso:[],
        params:[
            {description:"new snapTo value", optional:false, name:"snapTo", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:Calendar.eventMoved":{
        ref:"method:Calendar.eventMoved",
        flags:"",
        description:" Called when an event is moved via dragging by a user.  Return false to disallow the move.\n",
        name:"eventMoved",
        seeAlso:[],
        params:[
            {description:"new date and time that event is being moved to", optional:false,
             name:"newDate", defaultValue:null, type:"Date"},
            {description:null, optional:false, name:"event", defaultValue:null, type:"CalendarEvent"}
        ],
        returns:{description:"return false to disallow the move.", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "attr:MenuItem.title":{
        ref:"attr:MenuItem.title",
        flags:"IR",
        description:"The text displayed for the menu item",
        name:"title",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:MenuItem",
        groups:["menuBasic"]
    },
    "attr:ListGrid.stopOnErrors":{
        ref:"attr:ListGrid.stopOnErrors",
        flags:"IRWA",
        description:"If this is an editable listGrid, this property determines how failure to save due to \n validation errors should be displayed to the user.\n <P>\n If this property is true, when validation errors occur the errors will be displayed\n to the user in an alert, and focus will be returned to the first cell to fail validation.\n <P>\n If false, this the cells that failed validation will be silently styled with the \n editFailedBaseStyle.<br>\n <b>Note:</b> stopOnErrors being set to true implies that 'waitForSave' is also true.\n We will not dismiss the editor until save has completed if stopOnErrors is true.",
        name:"stopOnErrors",
        seeAlso:[
            {ref:"attr:ListGrid.waitForSave", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:ListGrid.clearFieldError":{
        ref:"method:ListGrid.clearFieldError",
        flags:"",
        description:" Clears any validation errors for some cell.\n",
        name:"clearFieldError",
        seeAlso:[
            {ref:"method:ListGrid.setFieldError", description:null}
        ],
        params:[
            {description:"row index of cell to add validation error for", optional:false,
             name:"rowNum", defaultValue:null, type:"number"},
            {description:"col index or field name of cell to add validation error for",
             optional:false, name:"fieldName", defaultValue:null, type:"number | string"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "class:FacetValue":{
        methods:["method:FacetValue.getCellValue", "method:FacetValue.formatCellValue"],
        ref:"class:FacetValue",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Analytics/CubeGrid"],
        attrs:["attr:FacetValue.id", "attr:FacetValue.title", "attr:FacetValue.parentId", 
         "attr:FacetValue.canEdit","attr:FacetValue.collapsed", "attr:FacetValue.canCollapse", 
         "attr:FacetValue.isMinimizeValue","attr:FacetValue.width", "attr:FacetValue.align", "attr:FacetValue.cellAlign", 
         "attr:FacetValue.borderBefore","attr:FacetValue.borderAfter", "attr:FacetValue.selectionBoundary", 
         "attr:FacetValue.titleHilite"],
        description:"Facet value definition object made use of by the CubeGrid class (contained by facets)",
        name:"FacetValue",
        seeAlso:[],
        type:"class",
        requiresModules:["Analytics"]
    },
    "method:List.dataChanged":{
        ref:"method:List.dataChanged",
        flags:"A",
        description:" Method called when this array changes in some way.  Observe the method to react to changes in\n this list.\n <P>\n Note: dataChanged() will only fire when items are added, removed or rearranged.  If a list\n contains objects, dataChanged() will not fire if changes are made to objects within the list\n without changing their position within the list.  If an observer of dataChanged() needs to react\n to such a change, you can manually fire dataChanged() by simply calling it.\n <P>\n Note: may be called multiple times as the result of a multi-item add or remove, etc.\n\n",
        name:"dataChanged",
        seeAlso:[],
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:ListGrid.canSort":{
        ref:"attr:ListGrid.canSort",
        flags:"IRW",
        description:"Enables or disables interactive sorting behavior for this listGrid. Does not\n          affect sorting by direct calls to the sort method.",
        name:"canSort",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["sorting"]
    },
    "class:LinkItem":{
        methods:["method:LinkItem.setLinkTitle"],
        ref:"class:LinkItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:LinkItem.target", "attr:LinkItem.linkTitle"],
        description:"A FormItem that displays an HTML link",
        name:"LinkItem",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.clearFilterText":{
        ref:"attr:ListGrid.clearFilterText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, \n and a ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor','filter-editor')} is visible, this\n attribute will be shown as the menu item title to clear any existing filter.  This\n menu-item is displayed only in the context menu for the sorter button.",
        name:"clearFilterText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Clear Filter\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:DSRequest.requestId":{
        ref:"attr:DSRequest.requestId",
        flags:"RA",
        description:"Automatically generated unique ID for this request. This ID will be required by developers \n making use of the ${isc.DocUtils.linkForRef('type:DSProtocol','\"clientCustom\" dataProtocol')}.",
        name:"requestId",
        seeAlso:[],
        valueType:"String",
        defaultValue:"varies",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:FormItem.ID":{
        ref:"attr:FormItem.ID",
        flags:"IRW",
        description:"Global identifier for referring to the formItem in JavaScript.  The ID property is\n optional if you do not need to refer to the widget from JavaScript, or can refer to it\n indirectly (for example, via <code>form.getItem(\"<i>itemName</i>\")</code>).\n <P>\n An internal, unique ID will automatically be created upon instantiation for any formItem\n where one is not provided.",
        name:"ID",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["basics"]
    },
    "attr:ListGrid.valueIconHeight":{
        ref:"attr:ListGrid.valueIconHeight",
        flags:"IRW",
        description:"Height for value icons for this listGrid.\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconSize')}.\n Can be overridden at the field level",
        name:"valueIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "type:SortArrow":{
        ref:"type:SortArrow",
        values:[
            {description:"Don't show a sort arrow at all.", value:"\"none\""},
            {description:"Display sort arrow in the upper-right corner (above the scrollbar) only.",
             value:"\"corner\""},
            {description:"Display sort arrow above each field header only.", value:"\"field\""},
            {description:"Display sort arrow above each field header AND in corner above scrollbar.BOTH:\"both\", // NOTE: Canvas establishes this constant",
             value:"\"both\""}
        ],
        description:"Do we display an arrow for the sorted field ?",
        name:"SortArrow",
        seeAlso:[],
        type:"type",
        groups:["sorting", "appearance"]
    },
    "attr:PickList.emptyPickListHeight":{
        ref:"attr:PickList.emptyPickListHeight",
        flags:"IRW",
        description:"Height for an empty pick list (showing the empty message), if the pick list has no\n records and ${isc.DocUtils.linkForRef('attr:PickList.hideEmptyPickList')} is <code>false</code>.",
        name:"emptyPickListHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "method:Window.addMember":{
        ref:"method:Window.addMember",
        flags:"A",
        description:"\n  Add a canvas to the layout, optionally at a specific position.\n\n",
        name:"addMember",
        seeAlso:[
            {ref:"method:Window.addMembers", description:null}
        ],
        params:[
            {description:"the canvas object to be added to the layout", optional:false,
             name:"newMember", defaultValue:null, type:"canvas"},
            {description:"the position in the layout to place newMember (starts with 0);                                  if omitted, it will be added at the last position",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "method:ListGrid.getGroupNodeHTML":{
        ref:"method:ListGrid.getGroupNodeHTML",
        flags:"",
        description:" Returns the HTML code necessary to render a group node, including icon, title, and padding.\n The amount of the padding is at least ${isc.DocUtils.linkForRef('attr:ListGrid.groupLeadingIndent')} pixels, and\n an additional ${isc.DocUtils.linkForRef('attr:ListGrid.groupIndentSize')} pixels for each increasing level of the\n node.\n",
        name:"getGroupNodeHTML",
        seeAlso:[],
        params:[
            {description:"Specified group node", optional:false, name:"node", defaultValue:null,
             type:"Object"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Layout.managePercentBreadth":{
        ref:"attr:Layout.managePercentBreadth",
        flags:"IR",
        description:"If set, a Layout with breadthPolicy:\"fill\" will specially interpret a percentage breadth on\n a member as a percentage of available space excluding the ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')}.  If false,\n percentages work exactly as for a non-member, with layoutMargins, if any, ignored.",
        name:"managePercentBreadth",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Layout"
    },
    "method:Canvas.animateFade":{
        ref:"method:Canvas.animateFade",
        flags:"",
        description:" Animate a change in opacity from the widget's current opacity to the specified opacity.\n",
        name:"animateFade",
        seeAlso:[],
        params:[
            {description:"desired final opacity", optional:false, name:"opacity", defaultValue:null,
             type:"number"},
            {description:"When the fade completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated fade", optional:true, name:"duration",
             defaultValue:null, type:"number"},
            {description:"Optional animation acceleration to bias the ratios", optional:true,
             name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateFade')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "type:MenuFieldID":{
        ref:"type:MenuFieldID",
        values:[
            {description:"Displays the icon field for the menu. This field contains the items                  specified icon (if there is one), or if the item is checked, the \n                  checkmark icon for the item.",
             value:"\"icon\""},
            {description:"Displays the item's title", value:"\"title\""},
            {description:"Displays the key field for the menu. This field contains the name or                title of any shortcut keys for this menu item.",
             value:"\"key\""},
            {description:"Field to display the submenu image for items that have a submenu.",
             value:"\"subMenu\""}
        ],
        description:"Simple string identifiers for standard menu fields.",
        name:"MenuFieldID",
        seeAlso:[],
        type:"type"
    },
    "classAttr:ValidatorTypes.dateRange":{
        ref:"classAttr:ValidatorTypes.dateRange",
        flags:"IR",
        description:"Tests whether the value for a date field is within the range specified.\n Range is inclusive, and is specified via <code>validator.min</code> and\n <code>validator.max</code>, which should be dates.<br>\n Note that the <code>errorMessage</code> for this validator will be evaluated as\n a dynamicString - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"dateRange",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "method:Canvas.getScrollRight":{
        ref:"method:Canvas.getScrollRight",
        flags:"",
        description:" Returns the scrollLeft required to scroll horizontally to the end of this widget's content.\n",
        name:"getScrollRight",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:ValuesManager.validate":{
        ref:"method:ValuesManager.validate",
        flags:"",
        description:" Validate the current set of values for this values manager against validators defined\n in the member forms. For databound valuesManagers, also perform validation against any\n validators defined on datasource fields.\n <P>\n Note that if validation errors occur for a value that is not shown in any member forms,\n those errors will cause a warning and ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')} will be\n called.  This can occur if:<br>\n - A datasource field has no correspending item in any member form<br>\n - The item in question is hidden<br>\n - The member form containing the item is hidden.\n\n",
        name:"validate",
        seeAlso:[],
        returns:{description:"true if all validation passed", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('formSplitting')}"],
        definingClass:"class:ValuesManager"
    },
    "method:Canvas.getPageBottom":{
        ref:"method:Canvas.getPageBottom",
        flags:"",
        description:" Return the page-relative bottom coordinate of this object, in pixels.\n",
        name:"getPageBottom",
        seeAlso:[],
        returns:{description:"GLOBAL bottom coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:GridRenderer.fastCellUpdates":{
        ref:"attr:GridRenderer.fastCellUpdates",
        flags:"I",
        description:"Whether to speed up dynamic styling at the expense of slightly slower drawing.\n <P>\n <code>fastCellUpdates</code> speeds up the dynamic styling system used by rollovers,\n selections, and custom styling that calls ${isc.DocUtils.linkForRef('method:GridRenderer.refreshCellStyle')}, at\n the cost of slightly slower draw() and redraw() times.",
        name:"fastCellUpdates",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer"
    },
    "classMethod:MathFunction.getDefaultFunctionIndex":{
        ref:"classMethod:MathFunction.getDefaultFunctionIndex",
        flags:"",
        description:" Returns an index of all default registered functions by name, ordered by \n ${isc.DocUtils.linkForRef('attr:MathFunction.defaultSortPosition')}.\n \n",
        name:"getDefaultFunctionIndex",
        seeAlso:[],
        returns:{description:null, type:"Index"},
        type:"classMethod",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "attr:Canvas.dragOpacity":{
        ref:"attr:Canvas.dragOpacity",
        flags:"IRWA",
        description:"If this widget has dragAppearance <code>\"target\"</code>, this value specifies the\n opacity to render the target while it is being dragged. A null value implies we do\n not modify the opacity.",
        name:"dragOpacity",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragEffects')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "classMethod:XMLTools.parseXML":{
        ref:"classMethod:XMLTools.parseXML",
        flags:"",
        description:" Parse XML text into an ${isc.DocUtils.linkForRef('type:XMLDocument')}.  Parse errors, if any, are reported to the log.\n\n",
        name:"parseXML",
        seeAlso:[],
        params:[
            {description:"XML text to be parsed", optional:false, name:"xmlText", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"resulting XMLDocument", type:"XMLDocument"},
        type:"classMethod",
        definingClass:"class:XMLTools"
    },
    "method:ColorPicker.colorSelected":{
        ref:"method:ColorPicker.colorSelected",
        flags:"",
        description:" Override this method to be notified when the user selects a color\n either by clicking a basic color box in simple mode, or by clicking \n the OK button in complex mode. It is not intended that client code \n call this method.\n",
        name:"colorSelected",
        seeAlso:[
            {ref:"method:ColorPicker.colorChanged", description:null}
        ],
        params:[
            {description:"The color selected, in HTML format", optional:false, name:"color",
             defaultValue:null, type:"String"},
            {description:"The opacity selected, from 0-100.", optional:false, name:"opacity",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:DataSourceField.detail":{
        ref:"attr:DataSourceField.detail",
        flags:"IR",
        description:"Whether this field should be considered a \"detail\" field by a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}.\n <P>\n Detail fields won't be shown by default in a DataBoundComponent where \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.showDetailFields')} is false.  This allows for some DataBound\n components, like a ${isc.DocUtils.linkForRef('class:ListGrid')}, to show a summary view of records which displays only \n the most commonly viewed fields by default, while other DataBoundComponents, like a\n ${isc.DocUtils.linkForRef('class:DetailViewer')}, show all fields by default.\n <P>\n In addition, the ${isc.DocUtils.linkForRef('method:FormItem.showIf','formItem.showIf')} property is supported in multiple\n components for conditional visibility - see for example ${isc.DocUtils.linkForRef('method:ListGridField.showIf')} and\n ${isc.DocUtils.linkForRef('method:FormItem.showIf')}).",
        name:"detail",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "attr:Calendar.scrollToWorkday":{
        ref:"attr:Calendar.scrollToWorkday",
        flags:"IR",
        description:"If set, causes the ${isc.DocUtils.linkForRef('attr:Calendar.workdayStart','workday hours')} to be sized to fill the available space\n in the day view and week view, and automatically scrolls these views to the start of the\n workday when the calendar is first displayed and whenever the user switches to a new day or\n week.",
        name:"scrollToWorkday",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday"]
    },
    "method:ColorPicker.getHtmlColor":{
        ref:"method:ColorPicker.getHtmlColor",
        flags:"",
        description:" Returns the currently-selected color, in HTML color representation form, as a string.\n HTML color representation is a hash sign, followed by the red, green and blue elements\n of the color in 2-digit hex form - for example \"#F17F1D\"\n",
        name:"getHtmlColor",
        seeAlso:[
            {ref:"method:ColorPicker.setHtmlColor", description:null}
        ],
        returns:{description:"HTML color value", type:"String"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "method:FormItem.doubleClick":{
        ref:"method:FormItem.doubleClick",
        flags:"",
        description:" Called when this FormItem is double-clicked.\n\n",
        name:"doubleClick",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "type:DefaultQueryClause":{
        ref:"type:DefaultQueryClause",
        values:[
            {description:"The column names to select, for a fetch operation only",
             value:"\"$defaultSelectClause\""},
            {description:"The table name(s) to select from or update",
             value:"\"$defaultTablesClause\""},
            {description:"The \"where\" condition, which will be derived from supplied                                 criteria or a primary key value, depending on the type of \n                                operation",
             value:"\"$defaultWhereClause\""},
            {description:"The column names to update and the update values, for an                                update or add operation",
             value:"\"$defaultValuesClause\""},
            {description:"The column names to sort by, for a fetch operation only",
             value:"\"$defaultOrderClause\""}
        ],
        description:"The Velocity variable names of the \"pieces\" of SQL that SmartClient generates to form a\n complete fetch or update query.  You can use these variables in you own custom queries \n and query clause overrides to build on the SmartClient functionality.  See \n ${isc.DocUtils.linkForRef('group:customQuerying')} for a full discussion.",
        name:"DefaultQueryClause",
        seeAlso:[],
        type:"type",
        serverDS:"allowed",
        groups:["customQuerying"]
    },
    "method:SectionStack.hideSection":{
        ref:"method:SectionStack.hideSection",
        flags:"",
        description:" \n Hides a section or sections.  This includes the section header and its items.  The space\n vacated by this action is reassigned to the nearest visible section item above this\n section.  If there are no visible section items above this section, the space is\n reassigned to the nearest visible section item below this section.\n\n",
        name:"hideSection",
        seeAlso:[
            {ref:"method:SectionStack.collapseSection", description:null}
        ],
        params:[
            {description:"Section(s) to hide.  For this parameter, you can pass the position \n                      of the section in the SectionStack, the ID of the section, or a\n                      List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"},
            {description:"callback to fire when the section has been hidden", optional:true,
             name:"callback", defaultValue:null, type:null}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsShowAndHide')}"],
        definingClass:"class:SectionStack"
    },
    "attr:ColumnTree.showOpenIcons":{
        ref:"attr:ColumnTree.showOpenIcons",
        flags:"IRW",
        description:"If true, show a different icon for <code>open</code> folders than closed folders.\n This is achieved by appending the ${isc.DocUtils.linkForRef('attr:ColumnTree.openIconSuffix')} onto the \n ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')} URL [for example <code>\"[SKIN]/folder.gif\"</code> might be \n replaced by <code>\"[SKIN]/folder_open.gif\"</code>.<br>\n <b>Note</b> If this property is set to <code>false</code> the same icon is shown for\n open folders as for closed folders, unless a custom folder icon was specified. This will be\n determined by ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')} plus the ${isc.DocUtils.linkForRef('attr:ColumnTree.closedIconSuffix')}.",
        name:"showOpenIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('millerColumns')}"],
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "method:ResultSet.rowIsLoaded":{
        ref:"method:ResultSet.rowIsLoaded",
        flags:"A",
        description:" Whether the given row has been loaded.\n <p>\n Unlike get(), will not trigger a server fetch.  \n\n",
        name:"rowIsLoaded",
        seeAlso:[],
        params:[
            {description:"row to check", optional:false, name:"rowNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"true if the given row has been loaded, false if it has not been                   loaded or is still in the process of being loaded",
         type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "method:CubeGrid.facetValueContextClick":{
        ref:"method:CubeGrid.facetValueContextClick",
        flags:"",
        description:"    StringMethod handler fired when context click occurs over a facetValue button in a header.\n  <br><i>methodType</i> handler\n",
        name:"facetValueContextClick",
        seeAlso:[],
        params:[
            {description:"facetValuesObject for the appropriate header button", optional:false,
             name:"facetValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "method:DataBoundComponent.editSummaryField":{
        ref:"method:DataBoundComponent.editSummaryField",
        flags:"",
        description:" Method to display a ${isc.DocUtils.linkForRef('class:SummaryBuilder')} to edit a Summary Field.  If the function is called\n without a paramater, a new field will be created when the summary is saved.\n\n",
        name:"editSummaryField",
        seeAlso:[],
        params:[
            {description:"Field to edit or null to add a new summary column", optional:false,
             name:"field", defaultValue:null, type:"Field"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["summaryFields"]
    },
    "attr:StretchImg.showGrip":{
        ref:"attr:StretchImg.showGrip",
        flags:"IRA",
        description:"Should we show a \"grip\" image floating above the center of this widget?",
        name:"showGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["grip"]
    },
    "attr:Window.isModal":{
        ref:"attr:Window.isModal",
        flags:"IRW",
        description:"If true, when shown this Window will intercept and block events to all other existing\n components on the page.\n <P>\n Use ${isc.DocUtils.linkForRef('attr:Window.showModalMask')} to darken all other elements on the screen when a modal dialog\n is showing.\n <P>\n Chained modal windows - that is, modal windows that launch other modal windows - are\n allowed.  You can accomplish this by simply creating a second modal Window while a modal\n Window is showing.\n <P>\n Note only top-level Windows (Windows without parents) can be modal.",
        name:"isModal",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('modality')}"],
        definingClass:"class:Window",
        groups:["modal"]
    },
    "class:TimeItem":{
        methods:["method:TimeItem.setMask"],
        exampleConfig:"TimeItem.xml",
        ref:"class:TimeItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:TimeItem.displayFormat", "attr:TimeItem.useMask", "attr:TimeItem.mask", 
         "attr:TimeItem.maskSaveLiterals","attr:TimeItem.maskOverwriteMode"],
        description:"FormItem for editing times in a text field.  \n <P>\n ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat')} allows you to set the display format for this field.  See also\n ${isc.DocUtils.linkForRef('classMethod:Time.setNormalDisplayFormat')} for system-wide settings.\n <P>\n TimeItem automatically accepts both 12 and 24 hour time as well as partial times and a\n variety of possible time value separators.  Examples:\n <pre>\n		11:34:45 AM	=> 11:34:45\n		1:3:5 AM	=> 01:30:50\n		1:3p		=> 13:30:00\n		11 34 am	=> 11:34:00\n		11-34		=> 11:34:00\n		113445		=> 11:34:45\n		13445		=> 01:34:45\n		1134		=> 11:34:00\n		134			=> 01:34:00\n </pre>\n <P>\n Values entered by the user are stored as JavaScript <code>Date</code> objects in GMT/UTC\n time regardless of the local time zone.  Note if you log, alert() or otherwise convert a\n Date into a String, the default behavior is to show it in local time.  To see the correct\n time value, use the JavaScript built-in APIs dateValue.getUTCHours/Minutes/Seconds() or the\n ${isc.DocUtils.linkForRef('classMethod:Time.toShortTime','formatting APIs on the Time class')}.",
        name:"TimeItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('timeItem')}"]
    },
    "type:LineBreakStyle":{
        ref:"type:LineBreakStyle",
        values:[
            {description:"Use the default line-break style of the server OS", value:"\"default\""},
            {description:"Use UNIX-style line-breaks (LF only)", value:"\"unix\""},
            {description:"Use MAC-style line-breaks (CR only)", value:"\"mac\""},
            {description:"Use DOS-style line-breaks (both CR & LF)", value:"\"dos\""}
        ],
        description:"The style of line-breaks to use when exporting data",
        name:"LineBreakStyle",
        seeAlso:[],
        type:"type",
        serverDS:"allowed"
    },
    "class:CubeGrid":{
        exampleConfig:"CubeGrid.xml",
        methods:["method:CubeGrid.getEventRow", "method:CubeGrid.getEventColumn", 
         "method:CubeGrid.getCellStyle","method:CubeGrid.getBaseStyle", "method:CubeGrid.setEditValues", 
         "method:CubeGrid.setEditValue","method:CubeGrid.getEditValue", "method:CubeGrid.getEditedRecord", 
         "method:CubeGrid.getEditedCell","method:CubeGrid.getEditValues", "method:CubeGrid.clearEditValue", 
         "method:CubeGrid.saveEdits","method:CubeGrid.getAllEditCells", "method:CubeGrid.discardAllEdits", 
         "method:CubeGrid.recordHasChanges","method:CubeGrid.hasChanges", "method:CubeGrid.saveAllEdits", "method:CubeGrid.getFacet", 
         "method:CubeGrid.getFacetValue","method:CubeGrid.getRollupValue", "method:CubeGrid.getCellFacetValues", 
         "method:CubeGrid.getFacetValuesColumn","method:CubeGrid.getFacetValuesRow", "method:CubeGrid.getRowFacetValues", 
         "method:CubeGrid.getColumnFacetValues","method:CubeGrid.getCellRecord", "method:CubeGrid.getCellRow", 
         "method:CubeGrid.getCellColumn","method:CubeGrid.getCellCoordinates", "method:CubeGrid.facetLabelHoverHTML", 
         "method:CubeGrid.facetValueHoverHTML","method:CubeGrid.closeFacet", "method:CubeGrid.closeColumn", 
         "method:CubeGrid.facetValueSelectionChanged","method:CubeGrid.getSelectedCells", "method:CubeGrid.getSelectedCellIds", 
         "method:CubeGrid.anyCellSelected","method:CubeGrid.cellIsSelected", "method:CubeGrid.selectCell", 
         "method:CubeGrid.deselectCell","method:CubeGrid.selectCells", "method:CubeGrid.deselectCells", 
         "method:CubeGrid.selectAllCells","method:CubeGrid.deselectAllCells", "method:CubeGrid.getSelectedFacetValues", 
         "method:CubeGrid.facetValuesSelected","method:CubeGrid.facetHasSelection", "method:CubeGrid.getFacetsHavingSelection", 
         "method:CubeGrid.selectFacetValues","method:CubeGrid.deselectFacetValues", "method:CubeGrid.selectFacetValue", 
         "method:CubeGrid.deselectFacetValue","method:CubeGrid.selectAllFacetValues", "method:CubeGrid.deselectAllFacetValues", 
         "method:CubeGrid.deselectAll","method:CubeGrid.resizeFacetValue", "method:CubeGrid.hiliteCell", 
         "method:CubeGrid.hiliteCellList","method:CubeGrid.hiliteFacetValue", "method:CubeGrid.setFacetTitle", 
         "method:CubeGrid.setFacetTitleAlign","method:CubeGrid.setFacetValueTitle", "method:CubeGrid.setFacetValueTitleAlign", 
         "method:CubeGrid.getFacetValueLayout","method:CubeGrid.getRowFacetLayout", "method:CubeGrid.getColumnFacetLayout", 
         "method:CubeGrid.makeChart","method:CubeGrid.addRowFacet", "method:CubeGrid.addColumnFacet", 
         "method:CubeGrid.addFacet","method:CubeGrid.removeFacet", "method:CubeGrid.setFixedFacetValue", 
         "method:CubeGrid.facetValueOver","method:CubeGrid.facetValueOut", "method:CubeGrid.facetValueHover", 
         "method:CubeGrid.facetValueContextClick","method:CubeGrid.facetValueReordered", "method:CubeGrid.facetAdded", 
         "method:CubeGrid.facetRemoved","method:CubeGrid.facetMoved", "method:CubeGrid.facetLabelOver", 
         "method:CubeGrid.facetLabelOut","method:CubeGrid.facetLabelHover", "method:CubeGrid.cellSelectionChanged"],
        type:"class",
        requiresModules:["Analytics"],
        ref:"class:CubeGrid",
        classAttrs:[],
        treeLocation:["Optional Modules/Analytics"],
        classMethods:[],
        description:"The CubeGrid is an interactive grid component that presents very large, multi-dimensional\n data sets (also known as data cubes) for reporting or analytic applications.\n <P>\n CubeGrids are often called crosstabs, for their cross-tabular display of data dimensions in\n stacked/nested rows and columns, or pivot tables, for their ability to \"pivot\" dimensions\n between rows and columns to view a data cube from different perspectives. They are typically\n used in the querying and reporting front-ends of data warehousing, decision support, OLAP,\n and business intelligence systems.\n <P>\n <B>Multi-Dimensional Data Terminology</B>\n <P>\n The CubeGrid refers to the dimensions of a data cube as facets, to the possible values in\n each facet as facet values, and to the values within the data cube as data values or cell\n values. Equivalent terms that are commonly used in data warehousing or business intelligence\n systems include:<br>\n <b>facet:</b> dimension, attribute, feature<br>\n <b>facet value:</b> dimension member, attribute value, feature value<br>\n <b>cell value:</b> data value, metric value, measure\n <P>\n <B>Visual Structure</B>\n <P>\n Like the ListGrid and TreeGrid components, the CubeGrid displays data values in a tabular\n \"body\" with adjacent \"headers\".  While the ListGrid and TreeGrid display rows of records\n with field values, the CubeGrid displays a body of individual cell values, each associated\n with a combination of facet values. The facet values for a cell are displayed in the column\n headers above the cell and row headers to the left of the cell. CubeGrids can display an\n arbitrary number of facets, by stacking multiple levels of row and/or column headers.\n <P>\n Except for the innermost column facet, each facet in a CubeGrid has a facet label adjacent\n to its row or column headers. The facet labels serve two main purposes: they display the\n titles of the facets, and they provide drag-and-drop reordering or pivoting of facets within\n the . The row facet labels also provide interactive selection, resizing, and other\n operations on the columns of row facet values.\n <P>\n The innermost column headers provide special behaviors and controls for manipulating the\n columns of data in a CubeGrid. End users may select, resize, reorder, minimize, maximize, or\n auto-fit the columns of data via mouse interactions with these headers. Customizable\n indicators and controls may be included at the top of each innermost column header.\n <P>\n If a CubeGrid is not large enough to display all of its cell values, horizontal and/or\n vertical scrollbars will appear below and to the right of the body. The body of the CubeGrid\n may be scrolled on either axis. The headers are \"frozen\" from scrolling on one axis - row\n headers only scroll vertically, while column headers only scroll horizontally - so the facet\n values for the visible cells are always displayed.\n <P>\n <B>Data Loading</B>\n <P>\n In order to allow intuitive and efficient navigation through very large datasets, the\n CubeGrid supports adding and removing facets from the display on the fly, and also tree-like\n expanding and collapsing both within facets and across facets, all integrated with a\n load-on-demand system that only loads visible data, and works with standard ISC DataSources.",
        attrs:["attr:CubeGrid.styleName", "attr:CubeGrid.bodyStyleName", "attr:CubeGrid.baseStyle", 
         "attr:CubeGrid.alternateRecordStyles","attr:CubeGrid.skinImgDir", "attr:CubeGrid.innerHeaderBaseStyle", 
         "attr:CubeGrid.colHeaderBaseStyle","attr:CubeGrid.rowHeaderBaseStyle", "attr:CubeGrid.facetValueHoverWidth", 
         "attr:CubeGrid.facetLabelHoverWidth","attr:CubeGrid.facetValueHoverHeight", "attr:CubeGrid.facetLabelHoverHeight", 
         "attr:CubeGrid.facetValueHoverAlign","attr:CubeGrid.facetLabelHoverAlign", "attr:CubeGrid.facetValueHoverVAlign", 
         "attr:CubeGrid.facetLabelHoverVAlign","attr:CubeGrid.facetValueHoverStyle", "attr:CubeGrid.facetLabelHoverStyle", 
         "attr:CubeGrid.fixedFacetValues","attr:CubeGrid.hilites", "attr:CubeGrid.dataSource", "attr:CubeGrid.facets", 
         "attr:CubeGrid.canSelectValues","attr:CubeGrid.canSelectHeaders", "attr:CubeGrid.autoSelectHeaders", 
         "attr:CubeGrid.autoSelectValues","attr:CubeGrid.simpleDeselect", "attr:CubeGrid.columnFacets", "attr:CubeGrid.rowFacets", 
         "attr:CubeGrid.hideEmptyFacetValues","attr:CubeGrid.data", "attr:CubeGrid.valueProperty", "attr:CubeGrid.cellIdProperty", 
         "attr:CubeGrid.canEdit","attr:CubeGrid.editByCell", "attr:CubeGrid.saveByCell", "attr:CubeGrid.canReorderColumns", 
         "attr:CubeGrid.canResizeColumns","attr:CubeGrid.canCollapseFacets", "attr:CubeGrid.canMinimizeFacets", 
         "attr:CubeGrid.canSortFacets","attr:CubeGrid.canMinimizeColumns", "attr:CubeGrid.canMoveFacets", 
         "attr:CubeGrid.autoSizeHeaders","attr:CubeGrid.facetTitleAlign", "attr:CubeGrid.facetValueAlign", 
         "attr:CubeGrid.cellAlign","attr:CubeGrid.wrapFacetTitles", "attr:CubeGrid.wrapFacetValueTitles", 
         "attr:CubeGrid.padTitles","attr:CubeGrid.defaultFacetWidth", "attr:CubeGrid.rollupValue", 
         "attr:CubeGrid.metricFacetId","attr:CubeGrid.valueTitle", "attr:CubeGrid.bodyMinWidth", "attr:CubeGrid.bodyMinHeight", 
         "attr:CubeGrid.chartConstructor","attr:CubeGrid.chartType", "attr:CubeGrid.chartConfirmThreshold"],
        name:"CubeGrid",
        seeAlso:[
            {ref:"class:Facet", description:null},
            {ref:"class:FacetValue", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('analytics')}"]
    },
    "attr:ListGridField.type":{
        ref:"attr:ListGridField.type",
        flags:"IRW",
        description:"ListGrids picks a renderer for the view and edit mode of a field based on this attribute.",
        name:"type",
        seeAlso:[
            {ref:"type:ListGridFieldType", description:null},
            {ref:"type:FieldType", description:null}
        ],
        valueType:"ListGridFieldType",
        defaultValue:"\"text\"",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:Canvas.focusChanged":{
        ref:"method:Canvas.focusChanged",
        flags:"",
        description:" Notification function fired when this widget receives or loses keyboard focus.\n",
        name:"focusChanged",
        seeAlso:[],
        params:[
            {description:"If true this widget now has keyboard focus", optional:false, name:"hasFocus",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "attr:TextItem.maskSaveLiterals":{
        ref:"attr:TextItem.maskSaveLiterals",
        flags:"IRWA",
        description:"Should entered mask value be saved with embedded literals?",
        name:"maskSaveLiterals",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextItem"
    },
    "method:LoginDialog.loginFunc":{
        ref:"method:LoginDialog.loginFunc",
        flags:"",
        description:" User-supplied callback function to process login transactions. \n <p>If the user clicks the \"Log in\" button, the credentials entered by the user are passed to\n loginFunc as an Object with properties \"username\" and \"password\" (NOTE: both\n property names are all lowercase), as the variable \"credentials\".  For example:\n <pre>{ username: \"barney\", password: \"rUbbL3\" }</pre>\n <P>\n This function should then attempt to log in by whatever means is necessary.  The second\n parameter to the loginFunc, \"dialogCallback\", is a function, which must be called <i>whether\n login succeeds or fails</i> with a true/false value indicating whether login succeeded.\n <P>\n If the login dialog is dismissable (settable as properties.dismissable, default false) and\n the user dismisses it, loginFunc will be fired with null for the credentials.\n <P>\n The following code shows typical usage.  This code assumes you have created a global\n function sendCredentials() that send credentials to some authentication system and fires a\n callback function with the result:\n <pre>\n ...\n loginFunc : function (credentials, dialogCallback) {\n     if (credentials == null) return; // dismissed\n\n     // send credentials    \n     sendCredentials(credentials, function (loginSucceeded) {\n         // report success or failure\n         dialogCallback(loginSucceeded);\n     })\n })\n ...\n </pre>\n",
        name:"loginFunc",
        seeAlso:[],
        params:[
            {description:"Login credentials supplied by the user", optional:false, name:"credentials",
             defaultValue:null, type:"Object"},
            {description:"Function that must be called once the login transaction completes",
             optional:false, name:"dialogCallback", defaultValue:null, type:"Function"}
        ],
        type:"method",
        definingClass:"class:LoginDialog"
    },
    "attr:BatchUploader.uploadDataSource":{
        ref:"attr:BatchUploader.uploadDataSource",
        flags:"IR",
        description:"DataSource used to save uploaded records.  Should have an operation of type \"add\".\n <p>\n Be careful to note that this is the DataSource representing your data as it will be \n persisted to your server.  It is completely different from the special \"bacthUpload\"\n DataSource which is used purely as a medium to upload the raw data to the server in\n the first place.",
        name:"uploadDataSource",
        seeAlso:[],
        valueType:"DataSource",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:FormulaBuilder.testButton":{
        ref:"attr:FormulaBuilder.testButton",
        flags:"IR",
        description:"Button to Test the formula by generating it's function and executing it",
        name:"testButton",
        seeAlso:[],
        valueType:"AutoChild Button",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:BatchUploader.uploadButton":{
        ref:"attr:BatchUploader.uploadButton",
        flags:"IR",
        description:"Button that triggers the upload.",
        name:"uploadButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:OperationBinding.useHttpProxy":{
        ref:"attr:OperationBinding.useHttpProxy",
        flags:"IR",
        description:"Whether to use the ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendProxied','HttpProxy')} servlet to send requests\n described by this operationBinding.  If unset, automatically detects whether using the\n HttpProxy is necessary based on the same-origin policy.\n <P>\n Valid only with ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')} settings other than ISCServer.",
        name:"useHttpProxy",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "method:Canvas.setImage":{
        ref:"method:Canvas.setImage",
        flags:"A",
        description:" Set the URL of an image element by name.\n <p>\n The image element must have been created from HTML generated by calling\n <code>canvas.imgHTML()</code> on this particular Canvas.\n\n",
        name:"setImage",
        seeAlso:[],
        params:[
            {description:"name of the image to change, as originally passed to                              <code>imgHTML</code>",
             optional:false, name:"identifier", defaultValue:null, type:"string"},
            {description:"URL for the image", optional:false, name:"URL", defaultValue:null,
             type:"SCImgURL"},
            {description:"optional image directory, overrides the default for this Canvas",
             optional:true, name:"imgDir", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "attr:GridRenderer.emptyCellValue":{
        ref:"attr:GridRenderer.emptyCellValue",
        flags:"IRW",
        description:"Value to show in empty cells (when getCellValue returns null)",
        name:"emptyCellValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:PickList.sortField":{
        ref:"attr:PickList.sortField",
        flags:"IR",
        description:"Specifies the field by which this item should be initially sorted.  Can be set to \n either a ${isc.DocUtils.linkForRef('attr:ListGridField.name','field name')} or the index of the field in the fields \n Array. Note that if <code>sortField</code> is initally specified as a number, it will be\n converted to a string (field name) after the item is initialized.\n <P>\n To programmatically change sort field or direction after initialization, call ${isc.DocUtils.linkForRef('sort')}.",
        name:"sortField",
        seeAlso:[],
        valueType:"String or integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        definingClass:"interface:PickList",
        groups:["sorting"]
    },
    "attr:ImgTab.src":{
        ref:"attr:ImgTab.src",
        flags:"IRW",
        description:"Base URL for tab images",
        name:"src",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"tab.gif\"",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "type:FacetValueMap":{
        ref:"type:FacetValueMap",
        description:"A JavaScript Object where each property name is a facetId and each property value is a\n facetValueId for that facet.  \n <P>\n The facetId -> facetValueId mappings in a FacetValueMap describe a specific slice of the\n dataset.  If mappings are included for all facets, a FacetValueMap describes a unique\n cell.  If some facets are omitted, it describes a row, column, or set of rectangular\n areas, or equivalently, a particular row or column header (if all facetIds in the map are\n displayed on the same axis)\n <P>\n FacetValueMaps are used in various contexts to describe headers, datasets to be loaded,\n screen regions, etc.",
        name:"FacetValueMap",
        seeAlso:[],
        type:"type"
    },
    "method:TabBar.selectTab":{
        ref:"method:TabBar.selectTab",
        flags:"",
        description:" Select a tab\n",
        name:"selectTab",
        seeAlso:[],
        params:[
            {description:"index of tab to select", optional:false, name:"tabNum", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:TabBar"
    },
    "class:WebService":{
        methods:["method:WebService.getOperationNames", "method:WebService.getSchema", 
         "method:WebService.callOperation","method:WebService.getSoapMessage", "method:WebService.getInputDS", 
         "method:WebService.getInputHeaderSchema","method:WebService.getOutputHeaderSchema", "method:WebService.getHeaderData", 
         "method:WebService.getFetchDS","method:WebService.setLocation"],
        ref:"class:WebService",
        classAttrs:[],
        classMethods:["classMethod:WebService.get", "classMethod:WebService.getByName"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:WebService.serviceNamespace", "attr:WebService.globalNamespaces"],
        description:"Class representing a WebService definition derived from a WSDL file.\n <P>\n A Web Service object allows you to invoke operations (via\n ${isc.DocUtils.linkForRef('method:WebService.callOperation','callOperation()')}), inspect schema declared in the\n WSDL file (${isc.DocUtils.linkForRef('method:WebService.getSchema','getSchema()')}), and perform simple read-only\n databinding ${isc.DocUtils.linkForRef('method:WebService.getFetchDS')}.  \n <P>\n Once a WebService has been loaded, a DataSource can be declared with a\n ${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')} to connect it to the web service, allowing DataSource\n data to be loaded and saved to the web service using\n ${isc.DocUtils.linkForRef('class:OperationBinding','operationBindings')}.",
        name:"WebService",
        seeAlso:[],
        type:"class",
        groups:["webService"]
    },
    "attr:Calendar.showOtherDays":{
        ref:"attr:Calendar.showOtherDays",
        flags:"IR",
        description:"If set to true, in the month view, days that fall in an adjacent month are still shown with\n a header and body area, and are interactive.  Otherwise days from other months are rendered\n in the ${isc.DocUtils.linkForRef('attr:Calendar.otherDayBlankStyle')} and are non-interactive.",
        name:"showOtherDays",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["visibility"]
    },
    "attr:Scrollbar.cornerSize":{
        ref:"attr:Scrollbar.cornerSize",
        flags:"IR",
        description:"Allows the size of the corner segment to be set independently of the ${isc.DocUtils.linkForRef('attr:Scrollbar.btnSize')}.",
        name:"cornerSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["corner"]
    },
    "attr:EdgedCanvas.edgeRight":{
        ref:"attr:EdgedCanvas.edgeRight",
        flags:"IR",
        description:"Height in pixels for right corners and edges.  Defaults to edgeSize when unset.",
        name:"edgeRight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:PickTreeItem.valueField":{
        ref:"attr:PickTreeItem.valueField",
        flags:"IR",
        description:"Which field in the tree-data should be returned as this item's value?\n If unspecified, the path will be used",
        name:"valueField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('pickTree')}"],
        definingClass:"class:PickTreeItem"
    },
    "attr:TreeNode.canAcceptDrop":{
        ref:"attr:TreeNode.canAcceptDrop",
        flags:"IRA",
        description:"Governs whether dragged data (typicaly other <code>treeNode</code>s) may be dropped over\n this node. Only has an effect if this node is displayed in a ${isc.DocUtils.linkForRef('class:TreeGrid')} where\n ${isc.DocUtils.linkForRef('attr:TreeGrid.canAcceptDroppedRecords')}, ${isc.DocUtils.linkForRef('attr:TreeGrid.canReorderRecords')} or \n ${isc.DocUtils.linkForRef('attr:TreeGrid.canReparentNodes')} is true.",
        name:"canAcceptDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:SummaryBuilder.helpTextIntro":{
        ref:"attr:SummaryBuilder.helpTextIntro",
        flags:"IR",
        description:"Text that appears in the hover from the ${isc.DocUtils.linkForRef('attr:SummaryBuilder.helpIcon')}, as a pre-amble to the list of\n available format-tokens.",
        name:"helpTextIntro",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"The following functions are available:\"",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["i18nMessages"]
    },
    "attr:BatchUploader.partialCommitError":{
        ref:"attr:BatchUploader.partialCommitError",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:BatchUploader.partialCommit')} is set to \"prevent\", the text to display to the user if they try \n to commit a dataset containing errors.  By default, this text is \"There are errors in your \n data.  Please correct all errors before clicking Commit\"",
        name:"partialCommitError",
        seeAlso:[],
        valueType:"String",
        defaultValue:"See below",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:Toolbar.deselectButton":{
        ref:"method:Toolbar.deselectButton",
        flags:"",
        description:"	Deselects the specified button from the toolbar, where buttonID is the index of\n  the button's object initializer. The button will be redrawn if necessary.\n  The button identifier can be a number (index), string (id), or object (widget or init block),\n as with the getButtonNumber() method.\n",
        name:"deselectButton",
        seeAlso:[
            {ref:"method:Toolbar.getButtonNumber", description:null}
        ],
        params:[
            {description:"Button / Button identifier", optional:false, name:"buttonID",
             defaultValue:null, type:"number | string | object | canvas"}
        ],
        type:"method",
        definingClass:"class:Toolbar",
        groups:["selection"]
    },
    "attr:Window.autoCenter":{
        ref:"attr:Window.autoCenter",
        flags:"IRW",
        description:"If true, this Window widget will automatically be centered on the page when shown.\n      If false, it will show up in the last position it was placed (either programatically,\n      or by user interaction).",
        name:"autoCenter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"autoCenter",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "location"]
    },
    "class:Log":{
        methods:[],
        ref:"class:Log",
        classAttrs:["classAttr:Log.defaultPriority", "classAttr:Log.stackTracePriority"],
        classMethods:["classMethod:Log.getCallTrace", "classMethod:Log.getStackTrace", 
         "classMethod:Log.echoLeaf","classMethod:Log.echo", "classMethod:Log.echoAll", "classMethod:Log.logDebug", 
         "classMethod:Log.logInfo","classMethod:Log.logWarn", "classMethod:Log.logError", "classMethod:Log.logFatal", 
         "classMethod:Log.setLogPriority","classMethod:Log.setDefaultLogPriority", "classMethod:Log.getDefaultLogPriority", 
         "classMethod:Log.applyLogPriorities","classMethod:Log.getLogPriorities", "classMethod:Log.getPriority", 
         "classMethod:Log.setPriority","classMethod:Log.setDefaultPriority", "classMethod:Log.getDefaultPriority", 
         "classMethod:Log.clearPriority","classMethod:Log.isEnabledFor", "classMethod:Log.show", "classMethod:Log.clear", 
         "classMethod:Log.traceMethod","classMethod:Log.timeMethod"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"A logging system similar to the Java log4j package: messages are logged with a \"category\" and\n \"priority\", and developers can dynamically set which log messages are being displayed.\n <P>\n 5 log priorities are available, with the following general meaning:\n <ul>\n <li> \"debug\": diagnostic info which is only likely to be understood by a developer with\n source access, or would occur too frequently for normal usage\n <li> \"info\": reports of significant events in the normal operation of the subsystem\n <li> \"warn\": some kind of problem is likely to occur, an API appears is apparently being\n misused or will yield a partial or very slow result\n <li> \"error\": a definite error has occurred which may be recoverable\n <li> \"fatal\": total failure with no possibility of recovery\n </ul>\n <P>\n Log categories do not need to be declared in advance - you can simply make up a category name and\n start logging to it, and control whether that category's messages will be displayed via\n <code>setPriority()</code>.\n <P>\n <b>NOTE:</b> to open the Developer Console in any page that loads ISC, type\n javascript:isc.Log.show() in the URL bar - this URL is bookmarkable.  \n <P>\n The Developer Console should <b>always</b> be open while developing any ISC-enabled application,\n because ISC logs many important errors and warnings to the Developer Console.\n <P>\n In Internet Explorer, the Developer Console is able to log a stack trace for every JS error,\n including errors that occur in non-ISC code.\n <P>\n NOTE: if you have the Microsoft JavaScript Debugger installed, ISC will be unable to log stack\n traces on JS errors until you go to Tools->Internet Options->Advanced Tab and check \"Disable\n script debugging\".  The ability to see stack traces in the Developer Console is generally much\n more useful for debugging ISC-based applications than the generic Javascript Debugging\n facilities.",
        name:"Log",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        type:"class",
        groups:["debug"]
    },
    "group:image":{
        ref:"group:image",
        refs:[],
        description:"Utilities to render images",
        name:"image",
        seeAlso:[],
        type:"group"
    },
    "attr:StatefulCanvas.showSelectedIcon":{
        ref:"attr:StatefulCanvas.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:ResultSet.fetchOperation":{
        ref:"attr:ResultSet.fetchOperation",
        flags:"IR",
        description:"Operation this dataModel should use to fetch sets of objects",
        name:"fetchOperation",
        seeAlso:[],
        valueType:"Operation or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "attr:Dialog.showToolbar":{
        ref:"attr:Dialog.showToolbar",
        flags:"IRW",
        description:"Whether to show a toolbar of buttons at the bottom of the Dialog.",
        name:"showToolbar",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Dialog",
        groups:["appearance", "toolbar"]
    },
    "attr:DetailViewer.emptyMessage":{
        ref:"attr:DetailViewer.emptyMessage",
        flags:"IRW",
        description:"The string to display in the body of a detailViewer with no records.",
        name:"emptyMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"No items to display.\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["emptyMessage"]
    },
    "attr:RestDataSource.prettyPrintJSON":{
        ref:"attr:RestDataSource.prettyPrintJSON",
        flags:"IR",
        description:"When using dataFormat:\"json\" and dataProtocol:\"postMessage\" should we use the\n ${isc.DocUtils.linkForRef('attr:JSONEncoder.prettyPrint')} feature to enable indented, highly readable JSON messages.\n <P>\n True by default because the bandwidth involved is generally neglible and the benefits for\n troubleshooting are key.",
        name:"prettyPrintJSON",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "group:clipping":{
        ref:"group:clipping",
        refs:[],
        name:"clipping",
        seeAlso:[],
        type:"group"
    },
    "attr:Hilite.fieldName":{
        ref:"attr:Hilite.fieldName",
        flags:"IR",
        description:"Name of the field that hilite should be applied to.  \n <P>\n If unset, hilite is applied to every field of the record.",
        name:"fieldName",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "method:DynamicForm.filterData":{
        ref:"method:DynamicForm.filterData",
        flags:"",
        description:" Retrieve data that matches the provided criteria, and edit the first record returned.<br>\n Differs from ${isc.DocUtils.linkForRef('method:DynamicForm.fetchData')} in that a case insensitive substring match\n will be performed against the criteria to retrieve the data.\n \n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Window.showMaximizeButton":{
        ref:"attr:Window.showMaximizeButton",
        flags:"IRW",
        description:"If true, show a maximize button in the header - clicking it maximizes the Window",
        name:"showMaximizeButton",
        setter:["setShowMaximizeButton"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:ResultSet.resultSize":{
        ref:"attr:ResultSet.resultSize",
        flags:"IRWA",
        description:"How many rows to retrieve at once.\n <P>\n Applicable only with <code>fetchMode: \"paged\"</code>.  When a paged ResultSet is asked\n for rows that have not yet been loaded, it will fetch adjacent rows that are likely to\n be required soon, in batches of this size.",
        name:"resultSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"75",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching"]
    },
    "method:ValuesManager.setSaveOperationType":{
        ref:"method:ValuesManager.setSaveOperationType",
        flags:"",
        description:" Setter for the default ${isc.DocUtils.linkForRef('type:DSOperationType')} when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is called.\n Note that this property can also be set by calling ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} or \n ${isc.DocUtils.linkForRef('method:DynamicForm.editNewRecord')}\n\n",
        name:"setSaveOperationType",
        seeAlso:[],
        params:[
            {description:"Operation type to use as a default. Valid values are  <code>\"add\"</code> or <code>\"update\"</code>.",
             optional:false, name:"operationType", defaultValue:null, type:"DSOperationType"}
        ],
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "group:cellEditing":{
        ref:"group:cellEditing",
        refs:["attr:CubeGrid.canEdit", "attr:CubeGrid.editByCell", "attr:CubeGrid.saveByCell"],
        name:"cellEditing",
        seeAlso:[],
        type:"group"
    },
    "attr:SummaryBuilder.editMode":{
        ref:"attr:SummaryBuilder.editMode",
        flags:"IR",
        description:"Are we editing an existing field?",
        name:"editMode",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:ListGrid.canDragSelectText":{
        ref:"attr:ListGrid.canDragSelectText",
        flags:"IRW",
        description:"If this property is true, users can drag the mouse to select text within grid rows.\n This is mutually exclusive with \n ${isc.DocUtils.linkForRef('canDragReorder','rearranging rows or cells by dragging')}, and with \n ${isc.DocUtils.linkForRef('attr:ListGrid.canDragSelect','drag selection of rows')}.",
        name:"canDragSelectText",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:ListGrid.setAutoFitMaxHeight":{
        ref:"method:ListGrid.setAutoFitMaxHeight",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxHeight')}.\n",
        name:"setAutoFitMaxHeight",
        seeAlso:[],
        params:[
            {description:"Maximum height in px we'll expand to accomodate if  ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','auto fit')} is enabled vertically.",
             optional:false, name:"height", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "method:StatefulCanvas.removeFromRadioGroup":{
        ref:"method:StatefulCanvas.removeFromRadioGroup",
        flags:"A",
        description:" Remove this widget from the specified mutually exclusive selection group with the ID\n passed in.\n No-op's if this widget is not a member of the groupID passed in.\n If no groupID is passed in, defaults to removing from whatever radioGroup this widget\n is a member of.\n",
        name:"removeFromRadioGroup",
        seeAlso:[],
        params:[
            {description:"- optional radio group ID (to ensure the widget is removed                                        from the appropriate group.",
             optional:true, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state", "event handling"]
    },
    "attr:MenuItem.canSelectParent":{
        ref:"attr:MenuItem.canSelectParent",
        flags:"IR",
        description:"A MenuItem that has a submenu normally cannot be selected, instead clicking or hitting Enter\n while keyboard focus is on the item shows the submenu.  Setting canSelectParent:true allows\n a menu item with a submenu to be selected directly.",
        name:"canSelectParent",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:MenuItem"
    },
    "attr:Button.iconHeight":{
        ref:"attr:Button.iconHeight",
        flags:"IR",
        description:"Height in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:ListGridField.showRollOverIcon":{
        ref:"attr:ListGridField.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "attr:ListGridField.editorValueIcons":{
        ref:"attr:ListGridField.editorValueIcons",
        flags:"IRW",
        description:"When some cell in this field is being edited, setting this property will specify the\n value icons to display in the cell's editor. If unset, the editor's valueIcons\n will be determined in the same way as it would be for a static cell.",
        name:"editorValueIcons",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:BatchUploader.uploadForm":{
        ref:"attr:BatchUploader.uploadForm",
        flags:"IR",
        description:"Form used to specify file to upload, and any additional fields required.",
        name:"uploadForm",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:OperationBinding.serverObject":{
        ref:"attr:OperationBinding.serverObject",
        flags:"IR",
        description:"Optional ServerObject declaration that specifies the ServerObject configuration for this\n operationBinding.  In the absense of a serverObject specification here, the one specified on\n the DataSource itself is used by default, if present (${isc.DocUtils.linkForRef('attr:DataSource.serverObject')}).  If\n neither is present, then Direct Method Invocation will not be enabled for this operationBinding.",
        name:"serverObject",
        seeAlso:[
            {ref:"attr:DataSource.serverObject", description:null}
        ],
        valueType:"ServerObject",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        requiresModules:["SCServer"]
    },
    "method:Date.toUSShortDate":{
        ref:"method:Date.toUSShortDate",
        flags:"",
        description:"			Return this date in the format: <code>MM/DD/YYYY</code>\n",
        name:"toUSShortDate",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:DetailViewerField.imageWidth":{
        ref:"attr:DetailViewerField.imageWidth",
        flags:"IRW",
        description:"Width of images shown for fieldTypes image in this field.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n width.  For example, if <code>field.imageWidth</code> is \"logoWidth\",\n <code>record.logoWidth</code> will control the width of the image.",
        name:"imageWidth",
        seeAlso:[
            {ref:"attr:DetailViewerField.imageSize", description:null},
            {ref:"attr:DetailViewerField.imageHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField",
        groups:["imageColumns"]
    },
    "method:Canvas.hoverHidden":{
        ref:"method:Canvas.hoverHidden",
        flags:"A",
        description:" If ${isc.DocUtils.linkForRef('attr:Canvas.showHover','showHover')} is true for this canvas, this notification method will be\n fired whenever the hover shown in response to ${isc.DocUtils.linkForRef('method:Canvas.handleHover','handleHover()')} is \n hidden. This method may be observed or overridden.\n",
        name:"hoverHidden",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:ListGrid.getAllEditRows":{
        ref:"method:ListGrid.getAllEditRows",
        flags:"",
        description:" Returns an array of every rowNum for which we have pending (unsubmitted) edits.\n",
        name:"getAllEditRows",
        seeAlso:[],
        returns:{description:"Array of rowNums for rows with edit values pending submission.", type:"array"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:CubeGrid.canCollapseFacets":{
        ref:"attr:CubeGrid.canCollapseFacets",
        flags:"IRW",
        description:"If true, hierarchical facets will show expand/collapse controls to allow the user to\n expand and collapse the tree of facetValues for that facet.",
        name:"canCollapseFacets",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetExpansion"]
    },
    "group:scroll":{
        ref:"group:scroll",
        refs:["method:Scrollbar.setScrollTarget"],
        name:"scroll",
        seeAlso:[],
        type:"group"
    },
    "method:ResultSet.applyFilter":{
        ref:"method:ResultSet.applyFilter",
        flags:"A",
        description:" The ResultSet will call applyFilter() when it needs to determine whether rows match the\n current filter criteria.\n <P>\n Default behavior is to call ${isc.DocUtils.linkForRef('method:DataSource.applyFilter')} to determine which rows match\n that provided criteria.\n <P>\n Override this method or ${isc.DocUtils.linkForRef('method:DataSource.applyFilter')} to implement your own client-side\n filtering behavior.\n\n",
        name:"applyFilter",
        seeAlso:[],
        params:[
            {description:"the list of rows", optional:false, name:"data", defaultValue:null,
             type:"Array"},
            {description:"the filter criteria", optional:false, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"dataSource request properties", optional:true, name:"requestProperties",
             defaultValue:null, type:"DSRequest Properties"}
        ],
        returns:{description:"the list of matching rows", type:"Array"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "group:platformDependencies":{
        ref:"group:platformDependencies",
        title:"Platform Dependencies",
        treeLocation:["/Client Reference/System"],
        refs:[],
        description:"Client-side processing of web services, XML parsing, and some UI loading mechanisms rely on\n a native in-browser XML parser and/or the XMLHttpRequest object - one or both of which will\n not be available if the end user disables ActiveX support in Internet Explorer.  Note that\n these features do not require plugins or downloads of any kind - IE simply exposes certain\n built-in functionality like the XML parser and XMLHttpRequest through the ActiveX\n interface.  Disabling ActiveX also disables all browser plugins such as Flash, Java, SVG, etc.\n <p>\n Barring ActiveX being disabled, the XMLHttpRequest object is available to SmartClient on all\n supported browsers and an XML parser is available on all supported browsers except Safari\n versions prior to 3.0.3.\n <p>\n SmartClient client-server communication is not affected by the lack of an XML parser or the\n XMLHttpRequest object, but the <code>xmlHttpRequest</code> transport will not be available\n if the XMLHttpRequest object is not available.  Instead, the <code>hiddenFrame</code> or the\n <code>scriptInclude</code> transports are used for client-server communication.\n <p>\n <b><u>XML Parser</u></b>\n <p>\n If an XML Parser is not available to SmartClient, all client-side web service bindings and\n related methods will be unavailable.  Turning off ActiveX disables integration paths 2 and 3\n in the diagram below.  If you want to bind to web services and require deployment to IE\n without ActiveX (or you need to support Safari pre 3.0.3), you'll need to do all XML processing on the\n server and use either the SmartClient DSRequest or JSON operation pathways (integration\n paths 1 and 4 in the diagram below).  See the discussion in ${isc.DocUtils.linkForRef('group:clientServerIntegration')}\n for more information on the integration paths shown in the diagram below.\n <p>\n You call ${isc.DocUtils.linkForRef('classMethod:XMLTools.nativeXMLAvailable')} to check for the avialability of a native XML\n parser at runtime.\n <p>\n <img src=\"${isc.DocViewer.instance.referenceRoot}skin/ds_bindings.png\" width=763 height=475>\n <p>\n <b><u>XMLHttpRequest</u></b>\n <p>\n The XMLHttpRequest object is used for the <code>xmlHttpRequest</code> ${isc.DocUtils.linkForRef('type:RPCTransport')}.\n Safari, Mozilla, Firefox, and IE 7 provide a native XMLHttpRequest implementation that is\n not affected by ActiveX being disabled (although the native IE 7 implementation can still be\n explicitly disabled by the end user).  IE 5.5 and IE 6.0 rely on the ActiveX bridge to\n support XMLHttpRequest, so if ActiveX is disabled in these browsers, XMLHttpRequest will not\n be available.\n <p>\n The lack of the XMLHttpRequest objects affects UI loading features like ${isc.DocUtils.linkForRef('class:ViewLoader')},\n and ${isc.DocUtils.linkForRef('class:HTMLFlow')} when used in remote loading mode (via ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsURL')},\n ${isc.DocUtils.linkForRef('method:HTMLFlow.setContentsURL')}, but does not affect the typical client/server communication\n pathways (integration paths 1 and 5 in the diagram above).\n <p>\n Also affected are low level features ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')},\n ${isc.DocUtils.linkForRef('attr:RPCRequest.evalResult')}, and ${isc.DocUtils.linkForRef('attr:RPCResponse.httpResponseCode')}.\n <p>\n In all of the above cases, it is possible to use the <code>hiddenFrame</code> transport to\n support these features when XMLHttpRequest is not available.  SmartClient will automatically\n send the request using the <code>hiddenFrame</code> transport when it detects that\n XMLHttpRequest is unavailable.  To support the above features, you'll need to use the\n RPCManager APIs on the server to send back the data that would normally be returned by\n XMLHttpRequest.  Since XMLHttpRequest cannot target URLs outside of the current domain, this\n strategy applies also to using the above features with cross-domain URLs.\n <p>\n You can call ${isc.DocUtils.linkForRef('classMethod:RPCManager.xmlHttpRequestAvailable')} to check for the availability of\n XMLHttpRequest at runtime.",
        name:"platformDependencies",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.setRect":{
        ref:"method:Canvas.setRect",
        flags:"",
        description:" Set all four coordinates, relative to the enclosing context, at once.\n <P>\n Moves the widget so that its top-left corner is at the specified top-left coordinates,\n then resizes it to the specified width and height.\n\n",
        name:"setRect",
        seeAlso:[],
        params:[
            {description:"new left coordinate, Array of coordinates                                                  in parameter order, or Object with left,\n                                                  top, width, height properties.  If an Array\n                                                  or Object is passed, the remaining\n                                                  parameters are ignored",
             optional:true, name:"left", defaultValue:null, type:"number, Array, Object"},
            {description:"new top coordinate", optional:true, name:"top", defaultValue:null,
             type:"number"},
            {description:"new width", optional:true, name:"width", defaultValue:null, type:"number"},
            {description:"new height", optional:true, name:"height", defaultValue:null, type:"number"}
        ],
        returns:{description:"whether the component's size actually changed", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "sizing"]
    },
    "method:CubeGrid.getAllEditCells":{
        ref:"method:CubeGrid.getAllEditCells",
        flags:"",
        description:" Method to determine which records currently have pending (unsubmitted) edits.\n Returns an array of 2 element arrays indicating the <code>[rowNum,colNum]</code> of the\n cells in question.\n",
        name:"getAllEditCells",
        seeAlso:[],
        returns:{description:"Array of <code>[rowNum,colNum]</code> arrays for cells with edit               values pending submission.",
         type:"array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:DataSourceField.validOperators":{
        ref:"attr:DataSourceField.validOperators",
        flags:"IR",
        description:"List of operators valid on this field.  \n <P>\n If not specified, all operators that are valid for the field type are allowed.",
        name:"validOperators",
        seeAlso:[],
        valueType:"Array of OperatorId",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["advancedFilter"]
    },
    "classMethod:Class.changeDefaults":{
        ref:"classMethod:Class.changeDefaults",
        flags:"A",
        description:" \n Changes a set of defaults defined as a JavaScript Object.  For these kind of properties,\n simply calling ${isc.DocUtils.linkForRef('method:Class.addProperties')} would replace the original Object\n with yours, wiping out settings required for the basic functionality of the component.\n This method instead applies your overrides over the existing properties, without\n destroying non-overridden properties.\n <p>\n For example let's say you have a component that's defined as follows\n <pre>\n isc.defineClass(\"MyComponent\");\n isc.MyComponent.addProperties({\n     simpleProperty: \"some value\",\n     propertyBlock : {\n       foo: \"bar\",\n       zoo: \"moo\"\n     }\n }\n </pre>\n If you wanted to override simpleProperty, you can just call ${isc.DocUtils.linkForRef('method:Class.addProperties')}\n like this:\n <pre>\n isc.MyComponent.addProperties({\n     simpleProperty: \"my override\"\n });\n </pre>\n If you want to override the value of <code>propertyBlock.moo</code> above,\n but you don't want to clobber the value of <code>propertyBlock.zoo</code>.  If you use\n the above pattern like so:\n <pre>\n isc.MyComponent.addProperties({\n     propertyBlock: {\n         foo: \"new value\",\n         zoo: \"moo\"\n     }\n });\n </pre>\n You need to re-specify the value of <code>propertyBlock.zoo</code> which you didn't want\n to override.  Failing to re-specify it would destroy the value.\n <p>\n Instead of re-specifying the value, you can use this method to modify the value of\n <code>foo</code> - like this:\n <pre>\n isc.MyComponent.changeDefaults(\"propertyBlock\", {\n     foo: \"new value\"\n });\n </pre>\n <p>\n See also the ${isc.DocUtils.linkForRef('type:AutoChild')} system for information about standard sets of defaults\n that are available for customization.\n\n",
        name:"changeDefaults",
        seeAlso:[],
        params:[
            {description:"name of the property to change", optional:false, name:"defaultsName",
             defaultValue:null, type:"String"},
            {description:"overrides for defaults", optional:false, name:"newDefaults",
             defaultValue:null, type:"Object"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "group:dynamicMenuItem":{
        ref:"group:dynamicMenuItem",
        refs:["method:MenuItem.enableIf", "method:MenuItem.checkIf", "method:MenuItem.dynamicTitle", 
         "method:MenuItem.dynamicIcon"],
        name:"dynamicMenuItem",
        seeAlso:[],
        type:"group"
    },
    "classAttr:RPCResponse.STATUS_TRANSPORT_ERROR":{
        ref:"classAttr:RPCResponse.STATUS_TRANSPORT_ERROR",
        flags:"R",
        description:"This response code is usable only with the XMLHttpRequest transport and indicates that\n the server returned an HTTP response code outside the range 200-299 (all of these statuses\n indicate success, but ordinarily only 200 is used).  To get the actual\n response code, you can query rpcResponse.httpResponseCode in your callback.\n <p>\n Note that currently this error code will never occur for the <code>hiddenFrame</code>\n transport - instead, use ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_SERVER_TIMEOUT')} to detect\n <code>hiddenFrame</code> transport errors.",
        name:"STATUS_TRANSPORT_ERROR",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"-90",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "type:DSCallback":{
        ref:"type:DSCallback",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        description:"A ${isc.DocUtils.linkForRef('type:Callback')} to evaluate when a DataSource request completes.\n <P>\n The parameters available in the DSCallback expression are:\n <ul>\n <li>dsResponse: a ${isc.DocUtils.linkForRef('class:DSResponse')} instance with metadata about the returned data\n <li>data: data returned to satisfy the DataSource request.  See the\n     ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource operations')} topic for expected results\n     for each type of DataSource operation\n <li>dsRequest: the ${isc.DocUtils.linkForRef('class:DSRequest')} that was sent.  You can use\n     ${isc.DocUtils.linkForRef('attr:DSRequest.clientContext')} to track state during the server turnaround.\n </ul>\n For example, if you had a DynamicForm with ID \"myForm\" and you wanted to retrieve a \n record from a DataSource \"myUsers\", where each record has a \"userId\" field:\n <pre>\n     myUsers.fetchData({ userId : \"fred\" }, \"myForm.setValues(data)\");\n </pre>\n or\n <pre>\n     myUsers.fetchData({ userId : \"fred\" }, function (dsResponse, data, dsRequest) {\n                                              myForm.setValues(data);\n                                            });\n </pre>\n <P>\n Note that if the request encounters a low-level error (such as 500 server error), by\n default the callback will <b>not</b> be fired, instead, ${isc.DocUtils.linkForRef('method:DataSource.handleError')}\n is called to invoke the default system-wide error handling.  Set\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError','willHandleError')}:true to have your callback invoked\n regardless of whether there are errors, however, make sure your callback properly\n handles malformed responses when ${isc.DocUtils.linkForRef('attr:DSResponse.status')} is non-zero.",
        name:"DSCallback",
        seeAlso:[],
        type:"type"
    },
    "classAttr:Validator.mustBeGreaterThan":{
        ref:"classAttr:Validator.mustBeGreaterThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>integerRange</code> type validator\n  returns false because the value passed in is less than the specified minimum.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeGreaterThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be at least ${min}\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:SchemaSet.schemaNamespace":{
        ref:"attr:SchemaSet.schemaNamespace",
        flags:"R",
        description:"Namespace of this SchemaSet, derived from the <code>targetNamespace</code>\n attribute of the <code>&lt;schema&gt;</code> element.",
        name:"schemaNamespace",
        seeAlso:[],
        valueType:"URI",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SchemaSet",
        groups:["webService"]
    },
    "method:Tree.getDescendantFolders":{
        ref:"method:Tree.getDescendantFolders",
        flags:"",
        description:"\n Returns the list of all descendants of a node that are folders.  This works just like\n ${isc.DocUtils.linkForRef('method:Tree.getDescendants')}, except leaf nodes are not part of the returned list.\n Like ${isc.DocUtils.linkForRef('method:Tree.getDescendants')}, this method can be very slow for large trees.\n Generally, ${isc.DocUtils.linkForRef('method:Tree.find')} in combination with ${isc.DocUtils.linkForRef('method:Tree.getFolders')} \n will be much faster.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')},\n so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} interface\n instead of as a native Javascript Array.\n\n",
        name:"getDescendantFolders",
        seeAlso:[],
        params:[
            {description:"node in question (the root node is assumed if none is specified)",
             optional:true, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"List of descendants of the node that are folders.", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "classAttr:GroupingMessages.byDayOfMonthTitle":{
        ref:"classAttr:GroupingMessages.byDayOfMonthTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by day of month.",
        name:"byDayOfMonthTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Day of Month\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:FusionChart.chartsBaseURL":{
        ref:"attr:FusionChart.chartsBaseURL",
        flags:"IR",
        description:"Base URL where FusionCharts are installed.  SmartClient expects to find the FusionCharts\n \".swf\" files under this URL.\n <P>\n The default value indicates that SmartClient will look for a FusionCharts directory\n parallel to the \"isomorphic/\" directory.\n <P>\n Note that the URL formation logic automatically compensates for various inconsistencies and\n typos in the default names for the chart .swf files, as well as differences between\n FusionCharts 2.3 and 3.0.   The chart .swf files should be left exactly as found in the\n FusionCharts distribution.  If you have renamed the charts for other purposes, you\n can set ${isc.DocUtils.linkForRef('attr:FusionChart.chartURL')} to the full path to any given chart type.",
        name:"chartsBaseURL",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"[ISOMORPHIC]/../FusionCharts/\"",
        type:"attr",
        definingClass:"class:FusionChart"
    },
    "attr:Window.headerLabel":{
        ref:"attr:Window.headerLabel",
        flags:"R",
        description:"Label that shows Window title in header.",
        name:"headerLabel",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "method:DynamicForm.itemChange":{
        ref:"method:DynamicForm.itemChange",
        flags:"",
        description:" Handler fired when there is a change() event fired on a FormItem within this form.\n <P>\n Fires after the change() handler on the FormItem itself, and only if the item did not\n cancel the change event and chooses to allow it to propagate to the form as a whole. \n\n",
        name:"itemChange",
        seeAlso:[],
        params:[
            {description:"the FormItem where the change event occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"new value for the FormItem", optional:false, name:"newValue",
             defaultValue:null, type:"any"},
            {description:"value the FormItem had previous to this change() event", optional:false,
             name:"oldValue", defaultValue:null, type:"any"}
        ],
        returns:{description:"return false to cancel the change, or true to allow it", type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "method:Window.setShowMinimizeButton":{
        ref:"method:Window.setShowMinimizeButton",
        flags:"",
        description:" Dynamically update ${isc.DocUtils.linkForRef('attr:Window.showMinimizeButton')} to show / hide the minimizeButton\n",
        name:"setShowMinimizeButton",
        seeAlso:[
            {ref:"attr:Window.headerControls", description:null},
            {ref:"attr:Window.showMinimizeButton", description:null}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "attr:Tree.showRoot":{
        ref:"attr:Tree.showRoot",
        flags:"IRW",
        description:"Controls whether a call to ${isc.DocUtils.linkForRef('method:Tree.getOpenList')} includes the root node.  Since view\n components such as ${isc.DocUtils.linkForRef('class:TreeGrid')} use <code>getOpenList()</code> to display the currently\n visible tree, <code>showRoot</code> controls whether the root node is shown to the user.\n <P>\n All Trees must have a single, logical root, however, most applications want to show multiple\n nodes at the top level.  <code>showRoot:false</code>, the default setting, prevents the logical\n root from being shown, so that the displayed tree begins with the children of root.\n <P>\n You can set <code>showRoot:true</code> to show the single, logical root node as the only\n top-level node.  This property is only meaningful for Trees where you supplied a value for\n ${isc.DocUtils.linkForRef('attr:Tree.root')}, otherwise, you will see an automatically generated root node that is\n meaningless to the user.",
        name:"showRoot",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Tree"
    },
    "object:TestFunctionResult":{
        methods:[],
        ref:"object:TestFunctionResult",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/FormulaBuilder"],
        attrs:["attr:TestFunctionResult.failedGeneration", "attr:TestFunctionResult.failedExecution", 
         "attr:TestFunctionResult.emptyTestValue","attr:TestFunctionResult.errorText", "attr:TestFunctionResult.result", 
         "attr:TestFunctionResult.record"],
        description:"A TestFunctionResult is an ordinary JavaScript Object with properties that indicate the \n status of an attempt to generate and execute a function for ${isc.DocUtils.linkForRef('class:FormulaBuilder')} and \n it's subclasses.\n <P>\n Because TestFunctionResult is always an ordinary JavaScript Object, it supports the\n normal behaviors of JavaScript Objects, including accessing and assigning to properties\n via dot notation:\n <pre>\n     var propValue = testFunctionResult.<i>propName</i>;\n     testFunctionResult.<i>propName</i> = newValue;\n </pre>\n <P>",
        name:"TestFunctionResult",
        seeAlso:[],
        type:"object",
        groups:["formulaFields"]
    },
    "attr:FilterBuilder.showAddButton":{
        ref:"attr:FilterBuilder.showAddButton",
        flags:"IR",
        description:"If set, a button will be shown underneath all current clauses allowing a new clause to be\n added.",
        name:"showAddButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Button.iconClick":{
        ref:"method:Button.iconClick",
        flags:"",
        description:" If this button is showing an ${isc.DocUtils.linkForRef('attr:Button.icon','icon')}, a separate click handler for the\n icon may be defined as <code>this.iconClick</code>.\n Returning false will suppress the standard button click handling code.\n",
        name:"iconClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:Chart.valueTitle":{
        ref:"attr:Chart.valueTitle",
        flags:"IR",
        description:"A label for the data values, such as \"Sales in Thousands\", typically used as the label for\n the value axis.",
        name:"valueTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:ListGrid.summaryRow":{
        ref:"attr:ListGrid.summaryRow",
        flags:"RA",
        description:"Automatically generated ListGrid for displaying grid summary information (see\n ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')}).\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.summaryRowProperties</code> and <code>listGrid.summaryRowDefaults</code>",
        name:"summaryRow",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "classAttr:RPCManager.defaultTimeout":{
        ref:"classAttr:RPCManager.defaultTimeout",
        flags:"RW",
        description:"In milliseconds, how long the RPCManager waits for an RPC request to complete before\n returning an error.  If set to zero, the RPCManager will not enforce a timeout, but note that\n most browsers enforce their own timeouts on HTTP requests.\n <p>\n For the \"xmlHttpRequest\" transport, this timeout can only happen if the server actually fails to\n respond within the specified number of milliseconds.  For the \"hiddenFrames\" transport,\n this timeout will occur for non-200 (HTTP_OK) responses.",
        name:"defaultTimeout",
        seeAlso:[],
        valueType:"number",
        defaultValue:"240000 [4 minutes]",
        type:"classAttr",
        definingClass:"class:RPCManager"
    },
    "method:Dialog.cancelClick":{
        ref:"method:Dialog.cancelClick",
        flags:"",
        description:" Handle a click on the 'cancel' button of this Dialog.\n Default implementation is to return null and hide the Dialog.\n Override to do something else.\n",
        name:"cancelClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:CubeGrid.facets":{
        ref:"attr:CubeGrid.facets",
        flags:"I",
        description:"Facet definitions for this CubeGrid.  Facets, also called \"dimensions\", are orthogonal\n aspects of the data model.\n <P>\n For example, you can look at profit by the facets \"plant and product\" or by \"product and\n plant\" and it's the same number, because the facets - plant and product - are the same.\n What would change the profit numbers would be to remove a facet, called \"summarizing\",\n or add a new facet, called \"drilling down\".  For example, showing profit by plant and\n product, you could \"drill down\" by adding the region facet, which would divide profit\n among each region.  Or you could remove the \"plant\" facet, showing total profit for each\n \"product\", summed across all plants.",
        name:"facets",
        seeAlso:[
            {ref:"method:CubeGrid.getFacet", description:null},
            {ref:"class:Facet", description:null},
            {ref:"method:CubeGrid.getFacetValue", description:null},
            {ref:"class:FacetValue", description:null}
        ],
        valueType:"Array of Facet",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:List.lastIndexOf":{
        ref:"method:List.lastIndexOf",
        flags:"",
        description:" Return the position in the list of the last instance of the specified object.\n <p>\n If pos is specified, starts looking before that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"lastIndexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"earliest index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:CheckboxItem.valueIconHeight":{
        ref:"attr:CheckboxItem.valueIconHeight",
        flags:"IRW",
        description:"Height of the checkbox image.",
        name:"valueIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"13",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["valueIcons"]
    },
    "attr:ListGrid.headerMenuButton":{
        ref:"attr:ListGrid.headerMenuButton",
        flags:"RA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, when the user rolls over the header buttons in this\n grid the headerMenuButton will be shown over the header button in question. When clicked\n this button will display the standard header context menu (see\n ${isc.DocUtils.linkForRef('method:ListGrid.displayHeaderContextMenu')}).\n <P>\n ${isc.DocUtils.linkForRef('group:headerMenuButton','Several properties')} exist to customize the appearance of the\n headerMenuButton. Also see the ${isc.DocUtils.linkForRef('type:AutoChild')} documentation for information on how \n to make freeform modifications to autoChild widgets",
        name:"headerMenuButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "method:ListGrid.getGroupByText":{
        ref:"method:ListGrid.getGroupByText",
        flags:"",
        description:" If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canGroupBy','this.canGroupBy')} is true, this string will be shown as the\n title for the menu item to toggle the group by setting for a field.<br>\n Default implementation evaulates and returns the dynamic ${isc.DocUtils.linkForRef('attr:ListGrid.groupByText')}\n string.\n",
        name:"getGroupByText",
        seeAlso:[],
        params:[
            {description:"field to get the menu item title for", optional:false, name:"field",
             defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"Title to show in the menu item", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:DetailViewer.loadingMessageStyle":{
        ref:"attr:DetailViewer.loadingMessageStyle",
        flags:"IRWA",
        description:"CSS style to use for the ${isc.DocUtils.linkForRef('attr:DetailViewer.loadingMessage')}.",
        name:"loadingMessageStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"normal\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["emptyMessage"]
    },
    "class:ColumnTree":{
        methods:["method:ColumnTree.fetchData", "method:ColumnTree.filterData", 
         "method:ColumnTree.nodeSelected","method:ColumnTree.shouldShowHeader", "method:ColumnTree.getColumnTitle", 
         "method:ColumnTree.getRecord","method:ColumnTree.getIcon", "method:ColumnTree.getNodeTitle", "method:ColumnTree.getData", 
         "method:ColumnTree.setData","method:ColumnTree.getColumn", "method:ColumnTree.getColumnProperties", 
         "method:ColumnTree.selectRecord","method:ColumnTree.deselectRecord", "method:ColumnTree.selectRecords", 
         "method:ColumnTree.deselectRecords","method:ColumnTree.selectAllRecords", "method:ColumnTree.deselectAllRecords", 
         "method:ColumnTree.anySelected"],
        ref:"class:ColumnTree",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:ColumnTree.dataSource", "attr:ColumnTree.data", "attr:ColumnTree.autoFetchData", 
         "attr:ColumnTree.autoFetchTextMatchStyle","attr:ColumnTree.initialCriteria", "attr:ColumnTree.folderIcon", 
         "attr:ColumnTree.customIconProperty","attr:ColumnTree.nodeIcon", "attr:ColumnTree.openIconSuffix", 
         "attr:ColumnTree.closedIconSuffix","attr:ColumnTree.showOpenIcons", "attr:ColumnTree.showCustomIconOpen", 
         "attr:ColumnTree.customIconOpenProperty","attr:ColumnTree.loadDataOnDemand", "attr:ColumnTree.column", "attr:ColumnTree.fields", 
         "attr:ColumnTree.showHeaders","attr:ColumnTree.firstColumnTitle", "attr:ColumnTree.showNodeCount"],
        description:"The SmartClient system supports hierarchical data (also referred to as tree data\n due to its \"branching\" organization) with:\n <ul>\n   <li> the ${isc.DocUtils.linkForRef('class:Tree')} class, which manipulates hierarchical data sets\n   <li> the TreeGrid widget class, which extends the ListGrid class to visually\n        present tree data in an expandable/collapsible format.\n   <li> the ColumnTree widget class, which visually\n        presents tree data in a so-called \n        \"<a href=http://en.wikipedia.org/wiki/Miller_Columns>Miller Column</a>\" format.\n </ul>\n For information on DataBinding Trees, see ${isc.DocUtils.linkForRef('group:treeDataBinding')}.\n <p>\n A ColumnTree shows a single branch of the underlying ${isc.DocUtils.linkForRef('class:Tree')} horizontally, from \n left to right. Thus, the leftmost column shows all the top-level nodes. When the user clicks\n one of those nodes, a new column is shown immediately to the right of the top-level column,\n showing the selected node's children. Additional columns are shown as required to present\n lower-level children. The behavior of ColumnTree is similar to that of the Browser interface\n in the Apple&trade; iTunes&trade; application.",
        name:"ColumnTree",
        implementsInterfaces:["DataBoundComponent"],
        seeAlso:[],
        type:"class"
    },
    "attr:FormItem.type":{
        ref:"attr:FormItem.type",
        flags:"IR",
        description:"The DynamicForm picks a field renderer based on the type of the field (and sometimes other\n attributes of the field).",
        name:"type",
        seeAlso:[
            {ref:"type:FormItemType", description:null},
            {ref:"type:FieldType", description:null}
        ],
        valueType:"FormItemType",
        defaultValue:"\"text\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:Menu.getItems":{
        ref:"method:Menu.getItems",
        flags:"",
        description:" Get all the MenuItems in the current menu as an Array.\n\n",
        name:"getItems",
        seeAlso:[],
        returns:{description:null, type:"Array of MenuItem"},
        type:"method",
        definingClass:"class:Menu"
    },
    "attr:SelectItem.displayField":{
        flags:"IRW",
        getter:["FormItem.getDisplayFieldName"],
        type:"attr",
        ref:"attr:SelectItem.displayField",
        description:"Specifies an alternative field from which display values should be retrieved for this\n item.\n <P>\n The display field can be either another field value in the same record or a field that\n must be retrieved from a related ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}.\n <P>\n If this item is not databound (${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is unset), or bound \n to the same dataSource as the form as a whole, this item will call\n ${isc.DocUtils.linkForRef('method:DynamicForm.getValue','form.getValue()')} \n the form named after is implemented by picking up the\n value of the specified field from the Form's values object.\n <P>\n Otherwise this item will attempt to map its underlying value to a display value\n by retrieving a record from the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} where the \n ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} matches this item's value, and displaying the \n <code>displayField</code> value from that record.\n Note that if <code>optionDataSource</code> is set and this value is not\n set, ${isc.DocUtils.linkForRef('method:FormItem.getDisplayFieldName')} will return the dataSource title field by default.\n <P>\n This essentially enables the specified <code>optionDataSource</code> to be used as\n a server based ${isc.DocUtils.linkForRef('group:valueMap')}.",
        name:"displayField",
        valueType:"string",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('relatedRecords')}"],
        defaultValue:"null",
        groups:["display_values"],
        definingClass:"class:SelectItem"
    },
    "attr:Criterion.fieldName":{
        ref:"attr:Criterion.fieldName",
        flags:"IR",
        description:"Name of the field in each ${isc.DocUtils.linkForRef('object:Record')} that this criterion applies to.  Not applicable for a\n criterion with ${isc.DocUtils.linkForRef('attr:Criterion.criteria','sub-criteria')}.",
        name:"fieldName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "class:Class":{
        methods:["method:Class.init", "method:Class.getID", "method:Class.getClass", 
         "method:Class.getSuperClass","method:Class.getClassName", "method:Class.addProperties", "method:Class.addPropertyList", 
         "method:Class.setProperty","method:Class.setProperties", "method:Class.isA", "method:Class.observe", 
         "method:Class.ignore","method:Class.isObserving", "method:Class.evaluate", "method:Class.fireCallback", 
         "method:Class.delayCall","method:Class.addAutoChild", "method:Class.createAutoChild", "method:Class.map", 
         "method:Class.Super","method:Class.toString", "method:Class.getCallTrace", "method:Class.getStackTrace", 
         "method:Class.echoLeaf","method:Class.echo", "method:Class.echoAll", "method:Class.logDebug", 
         "method:Class.logInfo","method:Class.logWarn", "method:Class.logError", "method:Class.logFatal", 
         "method:Class.logIsEnabledFor","method:Class.logIsDebugEnabled", "method:Class.logIsInfoEnabled", 
         "method:Class.logIsWarnEnabled","method:Class.logIsErrorEnabled", "method:Class.setLogPriority", 
         "method:Class.setDefaultLogPriority","method:Class.getDefaultLogPriority", "method:Class.clearLogPriority"],
        ref:"class:Class",
        classAttrs:[],
        classMethods:["classMethod:Class.create", "classMethod:Class.getClassName", 
         "classMethod:Class.getSuperClass","classMethod:Class.registerStringMethods", "classMethod:Class.evaluate", 
         "classMethod:Class.addClassProperties","classMethod:Class.addProperties", "classMethod:Class.addPropertyList", 
         "classMethod:Class.changeDefaults","classMethod:Class.setProperties", "classMethod:Class.isA", "classMethod:Class.Super", 
         "classMethod:Class.map","classMethod:Class.getInstanceProperty", "classMethod:Class.setInstanceProperty", 
         "classMethod:Class.fireCallback","classMethod:Class.delayCall", "classMethod:Class.toString", 
         "classMethod:Class.getCallTrace","classMethod:Class.getStackTrace", "classMethod:Class.echoLeaf", "classMethod:Class.echo", 
         "classMethod:Class.echoAll","classMethod:Class.logDebug", "classMethod:Class.logInfo", "classMethod:Class.logWarn", 
         "classMethod:Class.logError","classMethod:Class.logFatal", "classMethod:Class.logIsEnabledFor", 
         "classMethod:Class.logIsDebugEnabled","classMethod:Class.logIsInfoEnabled", "classMethod:Class.logIsWarnEnabled", 
         "classMethod:Class.logIsErrorEnabled","classMethod:Class.setLogPriority", "classMethod:Class.setDefaultLogPriority", 
         "classMethod:Class.getDefaultLogPriority","classMethod:Class.clearLogPriority"],
        treeLocation:["Client Reference/System"],
        attrs:["attr:Class.addPropertiesOnCreate"],
        description:"The Class object is root of the Isomorphic SmartClient inheritance tree -- it includes\n functionality for creating instances, adding methods and properties, getting prototypes,\n etc.<br><br>\n\n To add functionality to ALL classes, add them to Class.<br><br>\n\n To create a Class, call <code>ClassFactory.defineClass(\"MyClass\", \"MySuperClass\")</code>\n <P>\n <code>defineClass</code> will return the created class, and make it available as\n <code>isc.MyClass</code>, and as the global variable <code>MyClass</code> if not in\n ${isc.DocUtils.linkForRef('class:isc','portal mode')}.\n <P>\n You can then:\n <UL>\n		<LI>add class-level (static) properties and methods to the class:\n				<code>MyClass.addClassProperties()</code>	\n			these methods and properties are accessed through the Class variable itself, eg:\n				<code>MyClass.someStaticMethod()</code> or <code>MyClass.someStaticProperty</code>\n\n		<LI>add default instance properties and methods to the class:\n				<code>MyClass.addProperties()</code>\n			these methods and properties are accessed through a class instance, eg:\n				<code>var myInstance = MyClass.create();</code>\n				<code>myInstance.someInstanceMethod()</code>\n\n		<LI>create new instances of this class:\n				<code>var myInstance = MyClass.create()</code>\n </UL>\n NOTE: as a convention, all class names begin with a capital letter and all instances begin\n with a lower case letter.",
        name:"Class",
        seeAlso:[],
        type:"class"
    },
    "method:TreeGrid.willAcceptDrop":{
        ref:"method:TreeGrid.willAcceptDrop",
        flags:"A",
        description:" \n This method overrides ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')} and works as follows:\n <br><br>\n First, ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')} (the superclass definition) is consulted.  If it\n returns false, then this method returns false immediately.<br>\n This handles the following cases:<br>\n - reordering of records withing this TreeGrid when ${isc.DocUtils.linkForRef('attr:ListGrid.canReorderRecords')} is true<br>\n - accepting dropped records from another dragTarget when ${isc.DocUtils.linkForRef('attr:ListGrid.canAcceptDroppedRecords')} \n   is true and the dragTarget gives us a valid set of records to drop into place.<br>\n - disallowing drop over disabled nodes, or nodes with <code>canAcceptDrop:false</code>\n <br>\n This method will also return false if the drop occurred over a leaf node whos immediate \n parent has <code>canAcceptDrop</code> set to <code>false</code><br>\n If ${isc.DocUtils.linkForRef('attr:TreeGrid.canReparentNodes')} is true, and the user is dragging a node from one folder\n to another, this method will return true to allow the change of parent folder.<br>\n <br><br>\n Otherwise this method returns true.\n\n",
        name:"willAcceptDrop",
        seeAlso:[],
        returns:{description:"true if this component will accept a drop of the dragData", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["event handling"]
    },
    "attr:Canvas.maxWidth":{
        ref:"attr:Canvas.maxWidth",
        flags:"IRWA",
        description:"Maximum width that this Canvas can be resized to.",
        name:"maxWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10000",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:FormItem.imageURLSuffix":{
        ref:"attr:FormItem.imageURLSuffix",
        flags:"IRWA",
        description:"Suffix to apply to the end of any ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} when determining the URL for\n the image. A common usage would be to specify a suffix of <code>\".gif\"</code> in which\n case the <code>valueIcons</code> property would map values to the names of images without\n the <code>\".gif\"</code> extension.",
        name:"imageURLSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "method:Array.makeIndex":{
        ref:"method:Array.makeIndex",
        flags:"A",
        description:" Make an index for the items in this Array by a particular property of each item.\n <P>\n Returns an Object with keys for each distinct listItem[property] value.  Each key will point\n to an array of items that share that property value.  The sub-array will be in the same order\n that they are in this list.\n\n",
        name:"makeIndex",
        seeAlso:[],
        params:[
            {description:"names of the property to index by", optional:false, name:"property",
             defaultValue:null, type:"strings"},
            {description:"if true, we always make an array for every index.  if false, we make an Array only\n              when more than one item has the same value for the index property",
             optional:false, name:"alwaysMakeArray", defaultValue:"false", type:"boolean"}
        ],
        returns:{description:"index object", type:"object"},
        type:"method",
        definingClass:"object:Array"
    },
    "method:StretchImg.setSrc":{
        ref:"method:StretchImg.setSrc",
        flags:"",
        description:" Changes the base ${isc.DocUtils.linkForRef('attr:StretchImg.src')} for this stretchImg, redrawing if necessary.\n\n",
        name:"setSrc",
        seeAlso:[],
        params:[
            {description:"new URL for the image", optional:false, name:"src", defaultValue:null,
             type:"URL"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('loadImages')}"],
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "attr:SimpleType.validators":{
        ref:"attr:SimpleType.validators",
        flags:"IR",
        description:"Validators to apply to value of this type.",
        name:"validators",
        seeAlso:[],
        valueType:"Array of Validator",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleType",
        groups:["validation"]
    },
    "method:Canvas.setAccessKey":{
        ref:"method:Canvas.setAccessKey",
        flags:"A",
        description:" Set the accessKey for this canvas.\n <P>\n The accessKey can be set to any alphanumeric character (symbols not supported)\n Having set an accessKey, the canvas will be given focus when the user hits \n Alt+[accessKey], or in Mozilla Firefox 2.0 and above, Shift+Alt+[accessKey].\n\n",
        name:"setAccessKey",
        seeAlso:[],
        params:[
            {description:"Character to use as an accessKey for this widget.  Case Insensitive.",
             optional:false, name:"accessKey", defaultValue:null, type:null}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "attr:FormItem.hintClassName":{
        ref:"attr:FormItem.hintClassName",
        deprecated:"As of SmartClient version 5.5, deprecated in favor of ${isc.DocUtils.linkForRef('attr:FormItem.hintStyle')}",
        flags:"IRW",
        description:"CSS class for the \"hint\" string.",
        name:"hintClassName",
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"formHint\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:CubeGrid.facetLabelHoverAlign":{
        ref:"attr:CubeGrid.facetLabelHoverAlign",
        flags:"IRWA",
        description:"Allows the developer to override the horizontal text alignment of hover tips shown for\n facetLabels.  If unspecified the hover canvas content alignment will be set by\n <code>this.hoverAlign</code> if specified.",
        name:"facetLabelHoverAlign",
        seeAlso:[
            {ref:"attr:Canvas.hoverAlign", description:null}
        ],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "classMethod:SimpleType.registerSummaryFunction":{
        ref:"classMethod:SimpleType.registerSummaryFunction",
        flags:"",
        description:" Registers a new ${isc.DocUtils.linkForRef('type:summaryFunction')} by name. After calling this method,\n developers may specify the name passed in as a standard summaryFunction\n (for example in ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')}).\n",
        name:"registerSummaryFunction",
        seeAlso:[],
        params:[
            {description:"name for the newly registered summaryFunction", optional:false,
             name:"functionName", defaultValue:null, type:"string"},
            {description:"New summary function. This function should take 2 parameters -   <code>records</code>: an array of records for which a summary must be generated, \n   and <code>field</code>: a field definition - and return a summary value for the field.",
             optional:false, name:"method", defaultValue:null, type:"function"}
        ],
        type:"classMethod",
        definingClass:"class:SimpleType"
    },
    "group:members":{
        ref:"group:members",
        refs:["method:ValuesManager.getMembers", "method:ValuesManager.getMember", 
         "method:ValuesManager.getMemberForField","method:ValuesManager.addMember", "method:ValuesManager.addMembers", 
         "method:ValuesManager.removeMember","method:ValuesManager.removeMembers"],
        name:"members",
        seeAlso:[],
        type:"group"
    },
    "classMethod:FormItem.create":{
        ref:"classMethod:FormItem.create",
        flags:"",
        description:" FormItem.create() should never be called directly, instead, create a ${isc.DocUtils.linkForRef('class:DynamicForm')}\n and specify form items via ${isc.DocUtils.linkForRef('attr:DynamicForm.items','form.items')}.\n\n",
        name:"create",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:FormItem"
    },
    "attr:DataBoundComponent.canAddSummaryFields":{
        ref:"attr:DataBoundComponent.canAddSummaryFields",
        flags:"IRW",
        description:"Adds an item to the header context menu allowing users to launch a dialog to define a new\n text field that can contain both user-defined text and the formatted values present in other \n fields, using the ${isc.DocUtils.linkForRef('class:SummaryBuilder')}.\n <P>\n User-added summary fields can be persisted via ${isc.DocUtils.linkForRef('method:ListGrid.getFieldState')} and \n ${isc.DocUtils.linkForRef('method:ListGrid.setFieldState')}.",
        name:"canAddSummaryFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["summaryFields"]
    },
    "method:Canvas.setCanFocus":{
        ref:"method:Canvas.setCanFocus",
        flags:"A",
        description:" Change whether a widget can accept keyboard focus. \n",
        name:"setCanFocus",
        seeAlso:[
            {ref:"attr:Canvas.canFocus", description:null}
        ],
        params:[
            {description:"whether the widget should now accept focus", optional:false, name:"canFocus",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "attr:Facet.width":{
        ref:"attr:Facet.width",
        flags:"IR",
        description:"Integer number of pixels.  For row facets, width of headers.",
        name:"width",
        seeAlso:[
            {ref:"attr:CubeGrid.defaultFacetWidth", description:null}
        ],
        valueType:"number",
        defaultValue:"cubeGrid.defaultFacetWidth",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:ListGrid.expansionField":{
        ref:"attr:ListGrid.expansionField",
        flags:"IRWA",
        getter:["getExpansionField"],
        description:"The field providing the facility to expand and collapse rows.",
        name:"expansionField",
        seeAlso:[],
        valueType:"AutoChild ListGridField",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "classMethod:Time.toTime":{
        ref:"classMethod:Time.toTime",
        flags:"",
        description:" Given a date object, return the time associated with the date as a string.\n If no formatter is passed, use the standard formatter set up via ${isc.DocUtils.linkForRef('classMethod:Time.setNormalDisplayFormat')}\n",
        name:"toTime",
        seeAlso:[],
        params:[
            {description:"Date to convert to a time string.", optional:false, name:"date",
             defaultValue:null, type:"Date"},
            {description:"Optional custom formatter to use. Will accept  a function (which will be passed a pointer to the date to perform the conversion), or\n  a string designating a standard formatter",
             optional:true, name:"formatter", defaultValue:null, type:"timeFormatter | function"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "attr:FilterBuilder.topOperator":{
        ref:"attr:FilterBuilder.topOperator",
        flags:"IRW",
        description:"Default logical operator for all top-level clauses in the FilterBuilder.\n <P>\n May be able to be changed by the user via the UI, according to ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')}.",
        name:"topOperator",
        setter:["setTopOperator"],
        seeAlso:[],
        valueType:"LogicalOperator",
        defaultValue:"\"and\"",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Array.sum":{
        ref:"method:Array.sum",
        flags:"",
        description:" 			Returns the sum of the numbers in the array, skipping non-numeric values.\n\n			If the start and/or end are given, uses only the specified subset of the list.\n\n",
        name:"sum",
        seeAlso:[],
        params:[
            {description:"optional start index (default is 0)", optional:true, name:"start",
             defaultValue:null, type:"number"},
            {description:"optional end index (default is list.length)", optional:true, name:"end",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"sum of all items in the list", type:"number"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "method:ListGrid.parseEditorValue":{
        ref:"method:ListGrid.parseEditorValue",
        flags:"",
        description:" Method used to convert the value displayed in an editor for some cell being edited into\n a raw value for saving.<br>\n If <code>parseEditorValue</code> is defined at the field level for some cell being edited,\n the field level method will be used to parse the edit value and this method will not\n be called for that cell.<br>\n\n",
        name:"parseEditorValue",
        seeAlso:[
            {ref:"method:ListGridField.parseEditorValue", description:null},
            {ref:"method:ListGrid.formatEditorValue", description:null}
        ],
        params:[
            {description:"value displayed in the editor for the cell", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"record object for the row being edited. May be null if this                          is a new row being added to the end of the list.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"value in raw format", type:"any"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:FormItemIcon.showOver":{
        ref:"attr:FormItemIcon.showOver",
        flags:"IRWA",
        description:"Should this icon's image switch to the appropriate \"over\" source when the user rolls\n  over or focuses on the icon?",
        name:"showOver",
        seeAlso:[
            {ref:"attr:FormItem.showOverIcons", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "attr:Splitbar.capSize":{
        ref:"attr:Splitbar.capSize",
        flags:"IR",
        description:"If the default items are used, capSize is the size in pixels of the first and last\n          images in this stretchImg.",
        name:"capSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["appearance"]
    },
    "attr:Canvas.snapHDirection":{
        ref:"attr:Canvas.snapHDirection",
        flags:"IRW",
        description:"The horizontal snap direction.\n Set this value to Canvas.BEFORE to snap to the nearest gridpoint to the left;\n set it to Canvas.AFTER to snap to the nearest gridpoint to the right; and set\n it to Canvas.NEAREST to snap to the nearest gridpoint in either direction.",
        name:"snapHDirection",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"attr:Canvas.snapResizeToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapResizeToGrid", description:null}
        ],
        valueType:"string",
        defaultValue:"Canvas.AFTER",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:LoginDialog.registrationItemTitle":{
        ref:"attr:LoginDialog.registrationItemTitle",
        flags:"IR",
        description:"Specifies the contents of the registration link (if configured) on\n the ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"registrationItemTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"LoginDialog.registrationItemTitle",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "classMethod:SimpleType.setDefaultSummaryFunction":{
        ref:"classMethod:SimpleType.setDefaultSummaryFunction",
        flags:"",
        description:" Set up a default summary function for some field type.\n <P>\n Note that the following default summary functions are set up when SmartClient initializes:\n <br>- <code>\"integer\"</code> defaults to <code>\"sum\"</code>\n <br>- <code>\"float\"</code> defaults to <code>\"sum\"</code>.\n\n",
        name:"setDefaultSummaryFunction",
        seeAlso:[],
        params:[
            {description:"type name", optional:false, name:"typeName", defaultValue:null, type:"string"},
            {description:"summary function to set as the default for   this data type.",
             optional:false, name:"summaryFunction", defaultValue:null, type:"SummaryFunction"}
        ],
        type:"classMethod",
        definingClass:"class:SimpleType"
    },
    "type:SQLType":{
        ref:"type:SQLType",
        values:[
            {description:"The custom SQL or HQL is read-only", value:"\"query\""},
            {description:"The custom SQL or HQL updates data", value:"\"update\""}
        ],
        description:"The types of custom query that can be handled by SmartClient's built-in \"sql\" and \n \"hibernate\" DataSources.  Note, only applies to ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType')}\n \"custom\".",
        name:"SQLType",
        seeAlso:[],
        type:"type",
        groups:["customQuerying"]
    },
    "attr:PickList.valueIconField":{
        ref:"attr:PickList.valueIconField",
        flags:"IRWA",
        description:"For Databound formItems, this property determines which column \n ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} should show up in for this formItem's pickList.<br>\n If unset, valueIcons show up in the ${isc.DocUtils.linkForRef('attr:PickList.displayField')} column if specified, \n otherwise the ${isc.DocUtils.linkForRef('attr:PickList.valueField')} column.<br>\n In most cases only the <code>displayField</code> or <code>valueField</code> will be visible.\n This property is typically only required if custom ${isc.DocUtils.linkForRef('attr:PickList.pickListFields')} \n have been specified for this item.",
        name:"valueIconField",
        seeAlso:[
            {ref:"attr:FormItem.valueIcons", description:null},
            {ref:"attr:PickList.pickListFields", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:Window.keepInParentRect":{
        ref:"attr:Window.keepInParentRect",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Window.canDragReposition')} or ${isc.DocUtils.linkForRef('attr:Window.canDragResize')} is true, should the\n windows size and position be constrained such that it remains within the viewport of\n its parent element (or for top level windows, within the viewport of the browser window)?\n <br>\n Can also be set to an array denoting an arbitrary rect [Left,Top,Width,Height] beyond\n which the window cannot be moved.\n <p>\n Note: keepInParentRect affects only user drag interactions, not programmatic moves.",
        name:"keepInParentRect",
        seeAlso:[
            {ref:"attr:Window.canDragReposition", description:null}
        ],
        valueType:"boolean or rect",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["dragdrop"]
    },
    "method:Tree.setRoot":{
        ref:"method:Tree.setRoot",
        flags:"",
        description:"\n Set the root node of the tree. \n\n",
        name:"setRoot",
        seeAlso:[],
        params:[
            {description:"new root node", optional:false, name:"newRoot", defaultValue:null,
             type:"TreeNode"},
            {description:"set to true to automatically open the new root node.", optional:false,
             name:"autoOpen", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "method:StretchImgButton.setState":{
        ref:"method:StretchImgButton.setState",
        flags:"",
        description:" Set the 'state' of this object, this changes it's appearance.\n\n",
        name:"setState",
        seeAlso:[
            {ref:"method:StretchImgButton.setDisabled",
             description:"which also affects state values."}
        ],
        params:[
            {description:"new state", optional:false, name:"newState", defaultValue:null, type:"State"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state", "appearance"]
    },
    "method:Tree.getOpenList":{
        ref:"method:Tree.getOpenList",
        flags:"",
        description:"\n Return a flattened list of nodes that are open under some parent, including the parent\n itself.  If the passed in node is a leaf, this method returns null\n\n",
        name:"getOpenList",
        seeAlso:[],
        params:[
            {description:"node in question", optional:true, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"flattened list of open nodes", type:"List of TreeNode"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:animationShowEffect.effect":{
        ref:"attr:animationShowEffect.effect",
        flags:"IR",
        description:"Effect to apply",
        name:"effect",
        seeAlso:[],
        valueType:"animateShowEffectId",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:animationShowEffect"
    },
    "group:sharingNodes":{
        ref:"group:sharingNodes",
        title:"Sharing Nodes",
        refs:[],
        description:"For local Trees, that is, Trees that don't use load on demand, SmartClient supports setting\n up the Tree structure by setting properties such as \"childrenProperty\", directly on data\n nodes.  This allows for simpler, faster structures for many common tree uses, but can create\n confusion if nodes need to be shared across Trees.\n <P>\n <b>using one node in two places in one Tree</b>\n <P>\n To do this, either clone the shared node like so:<pre>\n\n     tree.add(isc.addProperties({}, sharedNode));\n\n </pre> or place the shared data in a shared subobject instead.\n <P>\n <b>sharing nodes or subtrees across Trees</b>\n <P>\n Individual nodes within differing tree structures can be shared by two Trees only if\n ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')}, ${isc.DocUtils.linkForRef('attr:Tree.childrenProperty')}, and ${isc.DocUtils.linkForRef('attr:Tree.openProperty')} have\n different values in each Tree.\n <P>\n As a special case of this, two Trees can maintain different open state across a single\n read-only structure as long as just \"openProperty\" has a different value in each Tree.",
        name:"sharingNodes",
        seeAlso:[],
        type:"group"
    },
    "attr:WSRequest.useFlatFields":{
        ref:"attr:WSRequest.useFlatFields",
        flags:"IR",
        description:"When <code>useFlatFields</code> is set for a request to be sent to a WSDL web service, when\n creating the input XML message to send to the web service, properties in\n ${isc.DocUtils.linkForRef('attr:WSRequest.data','request.data')} will be used as the values for XML elements of the same name, at\n any level of nesting.\n <P>\n <code>useFlatFields</code> allows you to ignore gratuitous XML message structure, such as\n extra levels of nested elements, and provides some insulation against changes in the\n required structure of the input message.\n <P>\n For example, given this input message:\n <pre>\n &lt;FindServices&gt;\n     &lt;searchFor&gt;search text&lt;/searchFor&gt;\n     &lt;Options&gt;\n         &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;\n     &lt;/Options&gt;\n     &lt;IncludeInSearch&gt;\n         &lt;serviceName&gt;true&lt;/serviceName&gt;\n         &lt;documentation&gt;true&lt;/documentation&gt;\n         &lt;keywords&gt;true&lt;/keywords&gt;\n     &lt;/IncludeInSearch&gt;\n &lt;/FindServices&gt;\n </pre>\n If <code>useFlatFields</code> were <b>not</b> set, in order to fill out this message\n correctly, <code>request.data</code> would need to be:\n <pre>{\n    searchFor: \"search text\",\n    Options : {\n        caseSensitive: false,\n    },\n    IncludeInSearch : {\n        serviceName: true,\n        documentation : true,\n        keywords : true\n    }\n }</pre>\n However if useFlatFields were set, <code>request.data</code> could be just:\n <pre>{\n    searchFor: \"search text\",\n    caseSensitive: false,\n    serviceName: true,\n    documentation : true,\n    keywords : true\n }</pre>\n <code>useFlatFields</code> is often set when the input data comes from a ${isc.DocUtils.linkForRef('class:DynamicForm')}\n to avoid the cumbersome and fragile process of mapping input fields to an XML structure.\n <P>\n ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields')} can also be set to cause <b>all</b> dsRequests of a\n particular type to <code>useFlatFields</code> automatically.\n <P>\n For ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}, \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useFlatFields','component.useFlatFields')} can be set use \"flattened\"\n binding to fields of a WSDL message or XML Schema.\n <P>\n Note that <code>useFlatFields</code> is not generally recommended for use with XML input\n messages where multiple simple type fields exist with the same name, however if used in this\n way, the first field to use a given name wins.  \"first\" means the first field encountered in a\n depth first search.  \"wins\" means only the first field will be populated in the generated\n XML message.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WSRequest",
        groups:["flatFields"]
    },
    "method:Canvas.setOpacity":{
        ref:"method:Canvas.setOpacity",
        flags:"",
        description:"			Sets the opacity for the widget to the newOpacity value. This newOpacity\n          value must be within the range of 0 (transparent) to 100 (opaque). <br>\n          In Internet Explorer, any other filter effects will be wiped out. \n",
        name:"setOpacity",
        seeAlso:[],
        params:[
            {description:"new opacity level", optional:false, name:"newOpacity", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('translucency')}"],
        definingClass:"class:Canvas",
        groups:["cues"]
    },
    "method:Array.findIndex":{
        ref:"method:Array.findIndex",
        flags:"",
        description:" Find the index of the first Object where property == value in the object.\n <P>\n Pass an Object instead to match multiple properties.\n <P>\n Note: for string values, matches are case sensitive.\n\n",
        name:"findIndex",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"object:Array",
        groups:["access", "find"]
    },
    "method:List.sort":{
        ref:"method:List.sort",
        flags:"",
        description:" Sorts the elements of the List in place.\n <P>\n The optional comparator function should take two parameters \"a\" and \"b\" which are the two list\n items to compare, and should return:\n <ul>\n <li> a value less than zero, if \"a\" is less than \"b\" such that \"a\" should appear earlier in the\n      list\n <li> zero, if \"a\" and \"b\" are equal\n <li> a value greater than zero, if \"a\" is greater than \"b\" such that \"b\" should appear earlier in\n      the list\n </ul>\n \n",
        name:"sort",
        seeAlso:[],
        params:[
            {description:"comparator function to use", optional:true, name:"comparator",
             defaultValue:null, type:"function"}
        ],
        returns:{description:"the list itself", type:"List"},
        type:"method",
        definingClass:"interface:List"
    },
    "attr:JSONEncoder.dateFormat":{
        ref:"attr:JSONEncoder.dateFormat",
        flags:"IR",
        description:"Format for encoding JavaScript Date values in JSON.  See ${isc.DocUtils.linkForRef('type:JSONDateFormat')} for\n valid options, or override ${isc.DocUtils.linkForRef('method:JSONEncoder.encodeDate')} to do something custom.",
        name:"dateFormat",
        seeAlso:[],
        valueType:"JSONDateFormat",
        defaultValue:"\"xmlSchema\"",
        type:"attr",
        definingClass:"class:JSONEncoder"
    },
    "attr:TreeGrid.showConnectors":{
        ref:"attr:TreeGrid.showConnectors",
        flags:"IRW",
        description:"Should this treeGrid show connector lines illustrating the tree's hierachy?\n <P>\n For the set of images used to show connectors, see ${isc.DocUtils.linkForRef('attr:TreeGrid.connectorImage')}.\n <P>\n <b>Note</b>: in order for connector images to be perfectly connected, all styles for\n cells must have no top or bottom border or padding.  If you see small gaps in connector\n lines, check your CSS files.  See the example below for an example of correct\n configuration, including example CSS.",
        name:"showConnectors",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('connectors')}"],
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "method:DetailViewer.viewSelectedData":{
        ref:"method:DetailViewer.viewSelectedData",
        flags:"",
        description:"\n Displays the currently selected record(s) of the selectionComponent widget (typically a\n listGrid) in the detailViewer.\n\n",
        name:"viewSelectedData",
        seeAlso:[],
        params:[
            {description:"the ListGrid or ID of a ${isc.DocUtils.linkForRef('class:ListGrid')} whose currently selected\n     record(s) is/are to be edited",
             optional:false, name:"selectionComponent", defaultValue:null, type:"ListGrid or ID"}
        ],
        type:"method",
        definingClass:"class:DetailViewer",
        groups:["dataBoundComponentMethods"]
    },
    "attr:FilterBuilder.showFieldTitles":{
        ref:"attr:FilterBuilder.showFieldTitles",
        flags:"IR",
        description:"If true (the default), show field titles in the drop-down box used to select a field for querying.\n If false, show actual field names instead.",
        name:"showFieldTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "group:xmlSchema":{
        ref:"group:xmlSchema",
        refs:["method:DataSource.getLegalChildTags", "classMethod:XMLTools.loadXMLSchema", 
         "classMethod:XMLTools.loadWSDL"],
        name:"xmlSchema",
        seeAlso:[],
        type:"group"
    },
    "attr:ColumnTree.dataSource":{
        ref:"attr:ColumnTree.dataSource",
        flags:"IRW",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        definingClass:"class:ColumnTree",
        groups:["databinding"]
    },
    "class:ActiveXControl":{
        methods:["method:ActiveXControl.getPluginID", "method:ActiveXControl.getPluginHandle"],
        ref:"class:ActiveXControl",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Client Bridges"],
        attrs:["attr:ActiveXControl.id", "attr:ActiveXControl.params", "attr:ActiveXControl.uuid", 
         "attr:ActiveXControl.classID","attr:ActiveXControl.codeBase"],
        description:"ISC Abstraction for ActiveX controls",
        name:"ActiveXControl",
        seeAlso:[],
        type:"class",
        requiresModules:["PluginBridges"]
    },
    "attr:DataSource.iconField":{
        ref:"attr:DataSource.iconField",
        flags:"IR",
        description:"Designates a field of ${isc.DocUtils.linkForRef('type:FieldType','type')}:\"image\" as the field to use when rendering a\n record as an image, for example, in a ${isc.DocUtils.linkForRef('class:TileGrid')}.\n <p>\n For example, for a DataSource of employees, a \"photo\" field of type \"image\" should be\n designated as the iconField.\n <p>\n If not explicitly set, iconField looks for fields named \"picture\", \"thumbnail\", \"icon\",\n \"image\" and \"img\", in that order, and will use any of these fields as the iconField if it\n exists and has type \"image\". \n <P>\n To avoid any field being used as the iconField, set iconField to <code>null</code>.",
        name:"iconField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:TabSet.getTab":{
        ref:"method:TabSet.getTab",
        flags:"",
        description:" Get the live Canvas representing a tab by index or ID.  \n If passed a tab Canvas, just returns it.\n <P>\n Note that live Tab instances are not available until ${isc.DocUtils.linkForRef('method:Canvas.draw','draw()')}.\n <P>\n The returned Tab is considered an internal component of the TabSet.  In order to maximize\n forward compatibility, manipulate tabs through APIs such as a ${isc.DocUtils.linkForRef('method:TabSet.setTabTitle')} instead.\n Also note that a super-lightweight TabSet implementation may not use a separate Canvas per\n Tab, and code that accesses an manipulates Tabs as Canvases won't be compatible with that\n implementation.\n\n",
        name:"getTab",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"int | ID | Canvas"}
        ],
        returns:{description:"the tab Canvas, or null if not found or TabSet not drawn yet", type:"Tab"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:DSRequest.useFlatFields":{
        ref:"attr:DSRequest.useFlatFields",
        flags:"IR",
        description:"When <code>useFlatFields</code> is set for a request to be sent to a WSDL web service, when\n creating the input XML message to send to the web service, properties in\n ${isc.DocUtils.linkForRef('attr:DSRequest.data','request.data')} will be used as the values for XML elements of the same name, at\n any level of nesting.\n <P>\n <code>useFlatFields</code> allows you to ignore gratuitous XML message structure, such as\n extra levels of nested elements, and provides some insulation against changes in the\n required structure of the input message.\n <P>\n For example, given this input message:\n <pre>\n &lt;FindServices&gt;\n     &lt;searchFor&gt;search text&lt;/searchFor&gt;\n     &lt;Options&gt;\n         &lt;caseSensitive&gt;false&lt;/caseSensitive&gt;\n     &lt;/Options&gt;\n     &lt;IncludeInSearch&gt;\n         &lt;serviceName&gt;true&lt;/serviceName&gt;\n         &lt;documentation&gt;true&lt;/documentation&gt;\n         &lt;keywords&gt;true&lt;/keywords&gt;\n     &lt;/IncludeInSearch&gt;\n &lt;/FindServices&gt;\n </pre>\n If <code>useFlatFields</code> were <b>not</b> set, in order to fill out this message\n correctly, <code>request.data</code> would need to be:\n <pre>{\n    searchFor: \"search text\",\n    Options : {\n        caseSensitive: false,\n    },\n    IncludeInSearch : {\n        serviceName: true,\n        documentation : true,\n        keywords : true\n    }\n }</pre>\n However if useFlatFields were set, <code>request.data</code> could be just:\n <pre>{\n    searchFor: \"search text\",\n    caseSensitive: false,\n    serviceName: true,\n    documentation : true,\n    keywords : true\n }</pre>\n <code>useFlatFields</code> is often set when the input data comes from a ${isc.DocUtils.linkForRef('class:DynamicForm')}\n to avoid the cumbersome and fragile process of mapping input fields to an XML structure.\n <P>\n ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields')} can also be set to cause <b>all</b> dsRequests of a\n particular type to <code>useFlatFields</code> automatically.\n <P>\n For ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}, \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useFlatFields','component.useFlatFields')} can be set use \"flattened\"\n binding to fields of a WSDL message or XML Schema.\n <P>\n Note that <code>useFlatFields</code> is not generally recommended for use with XML input\n messages where multiple simple type fields exist with the same name, however if used in this\n way, the first field to use a given name wins.  \"first\" means the first field encountered in a\n depth first search.  \"wins\" means only the first field will be populated in the generated\n XML message.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest",
        groups:["flatFields"]
    },
    "method:PickList.dataArrived":{
        ref:"method:PickList.dataArrived",
        flags:"",
        description:" If this item is showing a dataBound pickList, this notification method will be fired \n when new data arrives from the server.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"index of first row returned by the server", optional:false, name:"startRow",
             defaultValue:null, type:"number"},
            {description:"index of last row returned by the server", optional:false, name:"endRow",
             defaultValue:null, type:"number"},
            {description:"pointer to this pickList's data", optional:false, name:"data",
             defaultValue:null, type:"ResultSet"}
        ],
        type:"method",
        definingClass:"interface:PickList"
    },
    "classMethod:Class.evaluate":{
        ref:"classMethod:Class.evaluate",
        flags:"",
        description:" Evaluate a string of script and return the result.    \n <P>\n This method is a wrapper around the native javascript method <code>eval()</code>. It\n papers over some native issues to ensure evaluation of script behaves consistently across\n browsers\n\n",
        name:"evaluate",
        seeAlso:[],
        params:[
            {description:"the expression to be evaluated", optional:false, name:"expression",
             defaultValue:null, type:"string"},
            {description:"Optional mapping of argument names to values - each key will      be available as a local variable when the script is executed.",
             optional:false, name:"evalArgs", defaultValue:null, type:"object"}
        ],
        returns:{description:"the result of the eval", type:"any"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:DateItem.useMask":{
        ref:"attr:DateItem.useMask",
        flags:"IA",
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is not <code>false</code> this property determines if\n an input mask should be used. The format of the mask is determined by the \n ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat')} or ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} (in that order).\n <p>NOTE: If neither ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat')} nor ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')}\n is set (default), the mask for input format MDY is used.",
        name:"useMask",
        seeAlso:[
            {ref:"attr:DateItem.maskDateSeparator", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["basics"]
    },
    "method:ListGrid.setAutoFitExtraRecords":{
        ref:"method:ListGrid.setAutoFitExtraRecords",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitExtraRecords')}.\n",
        name:"setAutoFitExtraRecords",
        seeAlso:[],
        params:[
            {description:"Number of extra rows beyond the data-size we'll expand to  accomodate if ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','auto fit')} is enabled vertically.",
             optional:false, name:"extraRecords", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "method:StatefulCanvas.getState":{
        ref:"method:StatefulCanvas.getState",
        flags:"A",
        description:" Return the state of this StatefulCanvas\n",
        name:"getState",
        seeAlso:[],
        returns:{description:null, type:"State"},
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "attr:RPCRequest.callback":{
        ref:"attr:RPCRequest.callback",
        flags:"IRW",
        description:"If you expect to receive a response to your RPC request, you can specify a callback that\n will be called with an instance or RPCResponse class as sent by the server.  Queuing does\n not affect callbacks in any way - your specified callback will be invoked for each\n RPCRequest that contained a callback regardless of whether the request was sent as part of a\n queue or not.\n <P>\n Note that if the request encounters an error (such as 500 server error), by default the\n callback will <b>not</b> be fired, instead, ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')} is called to\n invoke the default system-wide error handling.  Set ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError')}:true to have your\n callback invoked regardless of whether there are errors, however, make sure your callback\n properly handles malformed responses when ${isc.DocUtils.linkForRef('attr:RPCResponse.status')} is non-zero.",
        name:"callback",
        seeAlso:[],
        valueType:"RPCCallback",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "classAttr:Dialog.OK_BUTTON_TITLE":{
        ref:"classAttr:Dialog.OK_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"OK\"</code> button.",
        name:"OK_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"OK\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:FormItem.required":{
        ref:"attr:FormItem.required",
        flags:"IR",
        description:"Whether a non-empty value is required for this field to pass validation.\n <P>\n If the user does not fill in the required field, the ${isc.DocUtils.linkForRef('classAttr:Validator.requiredField')}\n error message will be shown.\n <P>\n <b>Note:</b> if specified on a FormItem, <code>required</code> is only enforced on the\n client.  <code>required</code> should generally be specified on a\n ${isc.DocUtils.linkForRef('class:DataSourceField')}.",
        name:"required",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formShowAndHide')}"],
        definingClass:"class:FormItem",
        groups:["validation"]
    },
    "attr:Menu.data":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Menu.data",
        description:"An array of menuItem objects, specifying the menu items this menu should show.",
        name:"data",
        setter:["setData"],
        valueType:"Array of MenuItem",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"],
        defaultValue:"null",
        groups:["data"],
        definingClass:"class:Menu"
    },
    "method:Tree.isRoot":{
        ref:"method:Tree.isRoot",
        flags:"",
        description:"\n Return true if the passed node is the root node.\n\n",
        name:"isRoot",
        seeAlso:[],
        params:[
            {description:"node to test", optional:false, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"true if the node is the root node", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:Layout.animateMemberTime":{
        ref:"attr:Layout.animateMemberTime",
        flags:"IRWA",
        description:"If specified this is the duration of show/hide animations when members are being shown\n or hidden due to being added / removed from this layout.",
        name:"animateMemberTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["animation"]
    },
    "attr:DateItem.useTextField":{
        ref:"attr:DateItem.useTextField",
        flags:"IRW",
        description:"Should we show the date in a text field, or as 3 select boxes?",
        name:"useTextField",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dateItem')}"],
        definingClass:"class:DateItem",
        groups:["basics"]
    },
    "attr:Calendar.otherDayHeaderBaseStyle":{
        ref:"attr:Calendar.otherDayHeaderBaseStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to the day headers of the month view.\n This style will have \"Dark\", \"Over\", \"Selected\", or \"Disabled\"\n appended to it according to the state of the cell.",
        name:"otherDayHeaderBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calMonthDayHeader\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "attr:ListGrid.selectOnEdit":{
        ref:"attr:ListGrid.selectOnEdit",
        flags:"IRWA",
        description:"When the user starts editing a row, should the row also be selected?\n  <P>\n  Note that  when this attribute is set to <code>true</code>, other all other rows in the grid\n  will be deselected when a record is selected due to editing.",
        name:"selectOnEdit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:DataSourceField.xmlAttribute":{
        ref:"attr:DataSourceField.xmlAttribute",
        flags:"IR",
        description:"Indicates that ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} should serialize this value as an XML\n attribute.\n <P>\n Note this does not need to be declared in order for DataSource records to be derived from\n XML data: a field will be populated with either an attribute or subelement with matching\n name.",
        name:"xmlAttribute",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["xmlSerialize", "componentSchema"]
    },
    "classAttr:LoginDialog.LOGIN_TITLE":{
        ref:"classAttr:LoginDialog.LOGIN_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed by ${isc.DocUtils.linkForRef('classMethod:isc.showLoginDialog')}.\n A custom title can alternatively be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"LOGIN_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Please log in\"",
        type:"classAttr",
        definingClass:"class:LoginDialog",
        groups:["i18nMessages"]
    },
    "class:Hover":{
        methods:[],
        ref:"class:Hover",
        classAttrs:["classAttr:Hover.moveWithMouse", "classAttr:Hover.leftOffset", "classAttr:Hover.topOffset", 
         "classAttr:Hover.hoverCanvasDefaults"],
        classMethods:["classMethod:Hover.show", "classMethod:Hover.hide"],
        treeLocation:["Client Reference/Control"],
        attrs:[],
        description:"The Hover class handles showing a simple SmartClient canvas containing arbitrary HTML, or\n triggering some other action in response to a user holding the mouse-pointer (or hovering)\n over a specific widget.",
        name:"Hover",
        seeAlso:[],
        type:"class"
    },
    "classMethod:XMLTools.selectString":{
        ref:"classMethod:XMLTools.selectString",
        flags:"A",
        description:" Retrieve a string value from an XML element or document based on an XPath expression.\n <P>\n If more than one node matches, only the first node's value will be returned.\n <P>\n Namespacing works as described under ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectNodes')}\n <P>\n <b>NOTE:</b> this API cannot be supported on the Safari web browser for versions prior to\n 3.0.3.\n\n",
        name:"selectString",
        seeAlso:[],
        params:[
            {description:"Native XMLElement,document, or xml string                                                        to select from",
             optional:false, name:"element", defaultValue:null,
             type:"XMLElement or XMLDocument or String"},
            {description:"XPath expression to use to select nodes", optional:false, name:"expression",
             defaultValue:null, type:"XPath"},
            {description:"namespace mapping used by the expression", optional:true, name:"namespaces",
             defaultValue:null, type:"prefix -> URI mapping"}
        ],
        returns:{description:"result of the XPath, in String form", type:"String"},
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('xmlServerValidationErrors')}"],
        definingClass:"class:XMLTools",
        groups:["xmlTransform"]
    },
    "attr:Slider.trackWidth":{
        ref:"attr:Slider.trackWidth",
        flags:"IRW",
        description:"The thickness of the track. This is the width, for a vertical slider, or the height, for\n a horizontal slider.",
        name:"trackWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:DetailViewer.labelStyle":{
        ref:"attr:DetailViewer.labelStyle",
        flags:"IRW",
        description:"CSS style for a normal detail label",
        name:"labelStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detailLabel\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:ListGrid.addData":{
        ref:"method:ListGrid.addData",
        flags:"",
        description:" Perform a DataSource \"add\" operation to add new records to this component's DataSource.\n\n",
        name:"addData",
        seeAlso:[],
        params:[
            {description:"new record", optional:false, name:"newRecord", defaultValue:null,
             type:"Object"},
            {description:"method to call on operation completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundAdd')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:EBay.setSandboxToken":{
        ref:"method:EBay.setSandboxToken",
        flags:"",
        description:"\n Sets the token to be used by queries to the eBay sandbox environment.\n \n",
        name:"setSandboxToken",
        seeAlso:[],
        params:[
            {description:"Your sanbox authToken, provided to you by eBay.", optional:false,
             name:"authToken", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:EBay"
    },
    "classMethod:isc.getIconRect":{
        ref:"classMethod:isc.getIconRect",
        flags:"A",
        description:"  Returns the size / position of an icon with respect to the widget rendering out the\n  form item as an array of coordinates.\n",
        name:"getIconRect",
        seeAlso:[],
        params:[
            {description:"icon definition for the icon you want to determine the                               position of (defaults to first icon in this.icons).",
             optional:false, name:"icon", defaultValue:null, type:"object"}
        ],
        returns:{description:"four element array representing the Left, Top, Width, and Height of                      the icon in px.",
         type:"array"},
        type:"method",
        definingClass:"class:isc"
    },
    "method:ListGrid.cellOver":{
        ref:"method:ListGrid.cellOver",
        flags:"",
        description:" Called when the mouse pointer enters a cell\n\n",
        name:"cellOver",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:SchemaSet.getSchema":{
        ref:"method:SchemaSet.getSchema",
        flags:"A",
        description:" Get the schema definition of any complexType or element of complexType defined within\n the &lt;schema&gt; element this SchemaSet represents.\n\n",
        name:"getSchema",
        seeAlso:[],
        params:[
            {description:"name of the schema to retrieve", optional:false, name:"schemaName",
             defaultValue:null, type:"String"},
            {description:"optional type of schema to return, either \"element\" for                              xs:element definitions only or \"type\" for xs:complexType\n                              definitions.  If unspecified, either will be returned,\n                              with types preferred if names collide",
             optional:true, name:"schemaType", defaultValue:null, type:"String"}
        ],
        returns:{description:"the schema if found, or null", type:"DataSource"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('xmlSchemaImport')}"],
        definingClass:"class:SchemaSet"
    },
    "method:TabSet.setTabIcon":{
        ref:"method:TabSet.setTabIcon",
        flags:"A",
        description:" Changes the icon for a tab\n",
        name:"setTabIcon",
        seeAlso:[],
        params:[
            {description:"tab to update", optional:false, name:"tab", defaultValue:null,
             type:"Tab | number | ID"},
            {description:"new icon", optional:false, name:"icon", defaultValue:null, type:"SCImgURL"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "method:TileGrid.setDragTracker":{
        ref:"method:TileGrid.setDragTracker",
        flags:"",
        description:" Sets the custom tracker HTML to display next to the mouse when the user initiates a drag\n operation on this component. Default implementation will examine ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')}\n and set the custom drag tracker to display the appropriate HTML based on the selected record.\n <br>\n To display custom drag tracker HTML, this method may be overridden - call \n ${isc.DocUtils.linkForRef('classMethod:EventHandler.setDragTracker')} to actually update the drag tracker HTML.\n",
        name:"setDragTracker",
        seeAlso:[],
        returns:{description:"returns false by default to suppress 'setDragTracker' on any ancestors                   of this component.",
         type:"boolean"},
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dragTracker"]
    },
    "group:iscInstall":{
        ref:"group:iscInstall",
        title:"Deploying SmartClient",
        treeLocation:["Concepts"],
        refs:[],
        description:"This overview serves as a how-to for installing SmartClient into your web application.\n Evaluators are urged to use the SmartClient SDK with the embedded tomcat servlet engine\n during evaluation rather than pursue installation into an existing web application up front,\n however, reading this document and the related ${isc.DocUtils.linkForRef('group:clientServerIntegration','server\n integration')} materials is recommended to get an overview.\n <P>\n SmartClient has two pieces - the client components that run in the browser and the server\n components that run in a J2SE compatible container.  You don't need to use a Java back-end\n to use SmartClient, but the SDK comes with some examples that assume the presence of the\n Java back-end and, for some examples, a SQL Database.  If you will be using SmartClient with\n a Java back-end, see below for the list of J2SE application servers supported by the Java\n implementation of the SmartClient server.\n <p>\n The SDK contains two top-level directories: <code>smartclientSDK</code> and\n <code>smartclientRuntime</code>.  The <code>smartclientSDK</code> directory contains the\n membedded servlet engine, embedded database, examples, and documentation.  The\n <code>smartclientRuntime</code> directory contains just the client and server components of\n the SmartClient product - use the contents of this directory when deploying SmartClient into\n your application environment.\n <p>\n <u><b>Client integration</b></u>\n <p>\n To install the client-side portion of SmartClient, simply copy the <code>isomorphic</code>\n directory from the smartclientRuntime webroot to the webroot of your application.  Having\n done this you can use SmartClient components on your pages regardless of the technologies\n used on your back-end and you can bind to server-side componentry backed by arbitrary\n technology - see the <i>Data Integration</i> section of the\n ${isc.DocUtils.linkForRef('group:clientServerIntegration')} section for more information.\n <p>\n <u><b>Server integration</b></u>\n <p>\n SmartClient can be ${isc.DocUtils.linkForRef('group:nonJavaBackend','integrated with any backend')} without\n installing the SmartClient Java server.  However, the SmartClient server accelerates Java\n integration and provides other useful facilities, including server-push and network\n performance enhancements.  See the ${isc.DocUtils.linkForRef('group:iscServer','SmartClient Server summary')} for\n details.\n <P>\n To integrate the server portion of SmartClient, you need to follow the steps below for each\n application (WAR) that uses SmartClient.  Note that, if installing into an environment that\n uses multiple WARs, installation of SmartClient JARs into a directory shared by multiple\n applications is not supported.  Installation of a separate WAR with client-side SmartClient\n modules for maintaining cache coherence across applications using the same version of ISC is\n supported - contact Isomorphic support for more details on how to set that up.\n <p>\n <span style='color:red'>Note:</span> Some of the instructions below ask you to copy files\n into the WEB-INF/classes folder.  If you're using an IDE such as Eclipse that attempts to\n manage the WEB-INF/classes folder, we recommend that you copy these files to the src/\n directory of your project (next to the top-level folder for your java namespace) such that\n your IDE auto-deploys them to the WEB-INF/classes folder.  We have seen cases of tools like\n Eclipse periodically deleting files that are checked into to WEB-INF/classes directly.\n <p>\n <ul>\n <li> Copy all files from the WEB-INF/lib directory of the smartclientRuntime to your\n WEB-INF/lib.  SmartClient\n leverages a number of third-party libraries - if there are conflicts with the versions you\n want to use, use the versions you want - SmartClient has minimal dependencies on these\n libraries.  Contact Isomorphic with any specific questions about compatibility.\n See the \"Java Module Dependencies\" section below for a dependency tree that will\n help you select a minimal set of JARs to deploy.\n <li> Copy the WEB-INF/classes/log4j.isc.config.xml from the smartclientRuntime to your\n WEB-INF/classes\n directory.  This file contains the SmartClient server log configuration.\n <li> Copy the WEB-INF/classes/server.properties from the smartclientRuntime to your WEB-INF/classes\n directory.  This file contains settings for basic file locations such the location of\n webroot, the SmartClient SQL engine and DMI.  The version under smartclientRuntime has a\n basic, secure configuration.  See the version of server.properties under the\n smartclientSDK directory for sample SQL and other settings.\n <li> Copy the WEB-INF/iscTaglib.xml from the smartclientRuntime to your WEB-INF directory.\n This file enables the use of custom SmartClient tags in your JSPs.  \n <li> Merge portions of the WEB-INF/web.xml into your application's web.xml.  To use\n SmartClient JSP tags like &lt;loadISC&gt;, you'll need to merge the &lt;jsp-config&gt;\n section from web.xml.  Also there are some mandatory and optional servlets and filters to\n merge - see below.\n <li> <b>Power and Enterprise Editions only</b>.  Copy the shared/ds/batchUpload.ds.xml file\n to the same location in your target webapp directory.  This file is a utility DataSource\n that is used to provide the initial upload functionality of the ${isc.DocUtils.linkForRef('class:BatchUploader')} \n component - strictly speaking, you only need to perform this step if you intend to use that \n component.\n </ul>\n <p>\n See ${isc.DocUtils.linkForRef('group:servletDetails','Core and Optional SmartClient servlets')} for details of additional \n changes you may need to make to your applications <code>web.xml</code> file.  See \n ${isc.DocUtils.linkForRef('group:javaModuleDependencies','Java Module Dependencies')} for details of the .JAR files that\n comprise the SmartClient Server, and their dependencies on various third-party libraries.\n <p>\n <u><b>Troubleshooting</b></u>\n <p>\n This section covers some common problems with possible solutions. You may also need to refer to the\n documentation for your specific application server, web server, or database. If you experience any problems\n installing and configuring SmartClient in your environment, please post on the \n <a href=\"http://forums.smartclient.com/\" target='_blank'>SmartClient forums</a> for\n assistance.\n <p>\n <table width=\"90%\" class=\"normal\" align=\"center\" border=\"1\" cellpadding=\"5\">\n <tr bgcolor=\"#808080\">\n 	<td width=\"30%\"><b>Problem</b></td>\n 	<td width=\"30%\"><b>Possible Causes</b></td>\n 	<td width=\"40%\"><b>Solution</b></td>\n </tr><tr>\n 	<td rowspan='3'>Browser displays a generic \"page cannot be displayed\" or \"unable to locate the server\" message.</td>\n 	<td>Servlet engine not started.</td>\n 	<td>Start your application server.</td>\n </tr><tr>\n 	<td>Missing/incorrect port for servlet engine in URL.</td>\n 	<td>Check the startup messages, logs, or documentation for the servlet engine to determine what port it is using.</td>\n </tr><tr>\n 	<td>Host name is incorrect.</td>\n 	<td>Check whether other pages on the host can be accessed.\n 		Try the base URL http://[host name]:[port number] to see whether the servlet engine or webserver is functioning.</td>\n </tr><tr>\n 	<td>Browser displays a 404 or other page/file not found error.</td>\n 	<td>Incorrect URL.</td>\n 	<td>Check for errors in the URL, including capitalization.</td>\n </tr><tr>\n 	<td>Server error: taglib not defined</td>\n 	<td>Missing iscTaglib.xml or iscTaglib.xml not referenced in web.xml</td>\n 	<td>Copy WEB-INF/iscTaglib.xml from smartclientRuntime to your deployment WEB-INF directory\n 	and make sure that you have merged the &lt;jsp-config&gt; section from the\n 	smartclientRuntime web.xml</td>\n </tr><tr>\n  <td>ClassNotFound or other Java Exceptions in the server log.</td>\n  <td>Missing JAR files</td>\n  <td>Verify every .jar from the smartclientRuntime WEB-INF/lib directory has been copied to\n  your deployment WEB-INF/lib.  Although you might later be able to remove some .jars, for\n  initial installation testing, copy every .jar</td>\n </tr><tr>\n  <td>\"isc\" is not defined JS error</td>\n  <td>Incorrect URLs to SmartClient modules</td>\n  <td>Use View Source to look at SCRIPT includes (e.g. for ISC_Core.js), try those URLs\n  directly in the browser to verify the files are correctly deployed</td>\n </tr>\n </table>\n <P>\n <u><b>Caching Considerations</b></u>\n <p>\n When upgrading from one SmartClient release to the next, you want to make sure that the user\n picks up the new version on next access, but you also want to keep the ISC modules cacheable\n so they're not refetched on every access.\n <p>\n SmartClient deals with this problem by appending a version string as a query parameter to\n each module load directive.  This is done by the &lt;isomorphic:loadISC&gt; and\n &lt;isomorphic:loadModules&gt; tags automatically.  As long as you make sure that the file\n that contains these tags is non-cacheable, you will get the desired behavior.\n <p>\n <u><b>Supported J2SE Containers</b></u>\n <p>\n Below is the list of J2SE containters that have been tested to be compatible with this\n version of SmartClient.  Installation in these containers is supported for deployment by\n Isomorphic.  If your application server is not on this list, please contact us at the\n <a href=\"http://forums.smartclient.com\" target='_blank'>SmartClient forums</a> to see if we can support\n your deployment.  In general, the Java portion of ISC should work on servlet containers that\n comply with servlet specification version 2.3 and up and utilize a JVM no older than version\n 1.4.\n <p>\n Supported J2SE Containers:\n <p>\n <table class='normal'>\n <tr><td width=40></td><td width=200><i>Apache Tomcat 4.0.x, 4.1.x, 5.0.x, 5.5x, 6.0.x</i></td><td></td></tr>\n <tr><td></td><td><i>Apache Geronimo 1.x, 2.x</i></td><td></td></tr>\n <tr><td></td><td><i>BEA WebLogic 6.x, 7x, 8.x, 9.x, 10.x</i></td><td></td></tr>\n <tr><td></td><td><i>Caucho Resin 2.1.x, 3.0.x, 3.1.x</i></td><td></td></tr>\n <tr><td></td><td><i>IBM WebSphere 5.x, 6.x</i></td><td></td></tr>\n <tr><td></td><td><i>IBM WebSphere Community Edition 1.x</i></td><td></td></tr>\n <tr><td></td><td><i>JBoss 3.2.x, 4.0.x, 4.2.x</i></td><td></td></tr>\n <tr><td></td><td><i>Mortbay Jetty 4.x, 5.x, 6.x</i></td><td></td></tr>\n <tr><td></td><td><i>Oracle Containers for J2EE (OC4J) 9.x, 10.x, 11.x</i></td><td></td></tr>\n <tr><td></td><td><i>Oracle Application Server 10g 9.x, 10.x</i></td><td></td></tr>\n <tr><td></td><td><i>Sun Application Server 8.x, 9.x</i></td><td></td></tr>\n </table>",
        name:"iscInstall",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGridRecord.isGridSummary":{
        ref:"attr:ListGridRecord.isGridSummary",
        flags:"IRW",
        description:"This attribute will automatically be set to true for the record representing the \n grid-level summary row shown if ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true.\n <P>\n Note that <code>isGridSummary</code> is the default property name for this attribute but\n it may be modified by setting ${isc.DocUtils.linkForRef('attr:ListGrid.gridSummaryRecordProperty')}",
        name:"isGridSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "group:line":{
        ref:"group:line",
        refs:[],
        name:"line",
        seeAlso:[],
        type:"group"
    },
    "attr:DateChooser.weekendHeaderStyle":{
        ref:"attr:DateChooser.weekendHeaderStyle",
        flags:"IRW",
        description:"Optional CSS style applied to the day-of-week headers for weekend days. If unset \n ${isc.DocUtils.linkForRef('attr:DateChooser.headerStyle')} will be applied to both weekdays and weekend days.",
        name:"weekendHeaderStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:TileGrid.getSelectedRecord":{
        ref:"method:TileGrid.getSelectedRecord",
        flags:"",
        description:" Return the first selected record in this component\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"first selected record, or null if nothing selected", type:"TileRecord"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "class:ToolStripResizer":{
        methods:[],
        ref:"class:ToolStripResizer",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/ToolStrip"],
        attrs:["attr:ToolStripResizer.skinImgDir", "attr:ToolStripResizer.vSrc", 
         "attr:ToolStripResizer.hSrc"],
        description:"Simple subclass of ImgSplitbar with appearance appropriate for a ToolStrip resizer.",
        name:"ToolStripResizer",
        seeAlso:[],
        type:"class"
    },
    "attr:MenuBar.tabIndex":{
        ref:"attr:MenuBar.tabIndex",
        flags:"IRWA",
        description:"By default exclude menubars from the page's tab order. To include a menubar in the page's\n tab order, set tabIndex to an explicit tab index, or <code>null</code> for automatically \n assigned tabIndex",
        name:"tabIndex",
        seeAlso:[],
        valueType:"number",
        defaultValue:"-1",
        type:"attr",
        definingClass:"class:MenuBar"
    },
    "method:ColorPicker.setRed":{
        ref:"method:ColorPicker.setRed",
        flags:"",
        description:" Sets the Red element of the selected color\n",
        name:"setRed",
        seeAlso:[
            {ref:"method:ColorPicker.getRed", description:null}
        ],
        params:[
            {description:"An integer between 0 and 255", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "method:StatefulCanvas.setActionType":{
        ref:"method:StatefulCanvas.setActionType",
        flags:"A",
        description:" Update the 'actionType' for this canvas (radio / checkbox / button)\n If the canvas is currently selected, and the passed in actionType is 'button'\n this method will deselect the canvas.\n",
        name:"setActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state", "event handling"]
    },
    "attr:Splitbar.cursor":{
        ref:"attr:Splitbar.cursor",
        flags:"IRW",
        description:"Splitbars' cursors are set at init time based on whether they are to be used for vertical or\n horizontal resize.  To customize the cursor for this class, modify \n ${isc.DocUtils.linkForRef('attr:Splitbar.vResizeCursor')} or ${isc.DocUtils.linkForRef('attr:Splitbar.hResizeCursor')} rather than this property.",
        name:"cursor",
        seeAlso:[],
        valueType:"Cursor",
        defaultValue:"\"hand\"",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["cursor"]
    },
    "method:Canvas.focus":{
        ref:"method:Canvas.focus",
        flags:"",
        description:" If this canvas can accept focus, give it keyboard focus. After this method, the canvas\n will appear focussed and will receive keyboard events.\n",
        name:"focus",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "attr:ListGrid.drawAllMaxCells":{
        ref:"attr:ListGrid.drawAllMaxCells",
        flags:"IRWA",
        description:"If drawing all rows would cause less than <code>drawAllMaxCells</code> cells to be rendered,\n the full dataset will instead be drawn even if ${isc.DocUtils.linkForRef('attr:ListGrid.showAllRecords','showAllRecords')}\n is false and the viewport size and ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio')} setting would normally have caused\n incremental rendering to be used.\n <P>\n The <code>drawAllMaxCells</code> setting prevents incremental rendering from being used in\n situations where it's really unnecessary, such as a 40 row, 5 column dataset (only 200\n cells) which happens to be in a grid with a viewport showing only 20 or so rows.\n Incremental rendering causes a brief \"flash\" during scrolling as the visible portion of the\n dataset is redrawn, and a better scrolling experience can be obtained in this situation by\n drawing the entire dataset up front, which in this example would have neglible effect on\n initial draw time.\n <P>\n <code>drawAllMaxCells:0</code> disables this features.  You may want to disable this feature\n if performance is an issue and:\n <ul>\n <li> you are very frequently redraw a grid\n <li> you do a lot of computation when rendering each cell (eg formulas)\n <li> you are showing many grids on one screen and the user won't scroll most of them\n </ul>",
        name:"drawAllMaxCells",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"250",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "method:DataSource.exportData":{
        ref:"method:DataSource.exportData",
        flags:"",
        description:" Perform a \"fetch\" DataSource operation against this DataSource, sending search criteria,\n retrieving matching records and exporting the results.  See \n ${isc.DocUtils.linkForRef('attr:OperationBinding.exportResults')} or ${isc.DocUtils.linkForRef('attr:DSRequest.exportResults')} and for more\n information.\n\n",
        name:"exportData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "method:GridRenderer.selectionChanged":{
        ref:"method:GridRenderer.selectionChanged",
        flags:"A",
        description:" Called when (row-based) selection changes within this grid. Note this method fires for\n each record for which selection is modified - so when a user clicks inside a grid this\n method will typically fire twice (once for the old record being deselected, and once for\n the new record being selected).\n\n",
        name:"selectionChanged",
        seeAlso:[],
        params:[
            {description:"record for which selection changed", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"New selection state (true for selected, false for unselected)",
             optional:false, name:"state", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["selection"]
    },
    "method:Layout.hasMember":{
        ref:"method:Layout.hasMember",
        flags:"",
        description:"		    Returns true if the layout includes the specified canvas.\n",
        name:"hasMember",
        seeAlso:[],
        params:[
            {description:"the canvas to check for", optional:false, name:"canvas", defaultValue:null,
             type:"canvas"}
        ],
        returns:{description:"true if the layout includes the specified canvas", type:"boolean"},
        type:"method",
        definingClass:"class:Layout"
    },
    "group:editing":{
        ref:"group:editing",
        title:"Grid Editing",
        refs:["method:CubeGrid.setEditValue", "method:CubeGrid.getEditValue", 
         "method:CubeGrid.getEditedRecord","method:CubeGrid.getEditedCell", "method:CubeGrid.getEditValues", 
         "method:CubeGrid.clearEditValue","method:CubeGrid.saveEdits", "method:CubeGrid.getAllEditCells", 
         "method:CubeGrid.discardAllEdits","method:CubeGrid.recordHasChanges", "method:CubeGrid.hasChanges", 
         "method:CubeGrid.saveAllEdits","method:DynamicForm.getEditorType", "method:ListGridField.defaultDynamicValue", 
         "method:ListGridField.editorEnter","method:ListGridField.editorExit", "method:ListGridField.cellChanged", 
         "method:ListGridField.formatEditorValue","method:ListGridField.parseEditorValue", "method:ListGridField.change", 
         "method:ListGridField.changed","method:ListGrid.canEditCell", "method:ListGrid.startEditing", 
         "method:ListGrid.getEditorValueMap","method:ListGrid.setEditorValueMap", "method:ListGrid.getEditorType", 
         "method:ListGrid.startEditingNew","method:ListGrid.getAllEditRows", "method:ListGrid.getEditValues", 
         "method:ListGrid.getEditedRecord","method:ListGrid.getEditedCell", "method:ListGrid.setEditValue", 
         "method:ListGrid.getEditValue","method:ListGrid.clearEditValue", "method:ListGrid.getEditRow", 
         "method:ListGrid.getEditCol","method:ListGrid.getEditField", "method:ListGrid.cancelEditing", 
         "method:ListGrid.endEditing","method:ListGrid.discardAllEdits", "method:ListGrid.discardEdits", 
         "method:ListGrid.saveEdits","method:ListGrid.rowHasChanges", "method:ListGrid.hasChanges", 
         "method:ListGrid.cellHasChanges","method:ListGrid.saveAllEdits", "method:ListGrid.cellChanged", 
         "method:ListGrid.editComplete","method:ListGrid.editFailed", "method:ListGrid.editorEnter", 
         "method:ListGrid.rowEditorEnter","method:ListGrid.editorExit", "method:ListGrid.rowEditorExit", 
         "method:ListGrid.formatEditorValue","method:ListGrid.parseEditorValue", "attr:Calendar.eventSnapGap", 
         "attr:Calendar.showQuickEventDialog","attr:Calendar.eventEditorFields", "attr:Calendar.eventDialogFields", 
         "attr:ListGridField.editorImageURLPrefix","attr:ListGridField.editorImageURLSuffix", "attr:ListGridField.icons", 
         "attr:ListGridField.editorIconWidth","attr:ListGridField.editorIconHeight", "attr:ListGridField.defaultIconSrc", 
         "attr:ListGridField.iconVAlign","attr:ListGridField.canEdit", "attr:ListGridField.defaultValue", 
         "attr:ListGridField.enterKeyEditAction","attr:ListGridField.escapeKeyEditAction", "attr:ListGridField.editorType", 
         "attr:ListGridField.editorProperties","attr:ListGrid.modalEditing", "attr:ListGridField.editorValueMap", "attr:ListGrid.canEdit", 
         "attr:ListGrid.recordEditProperty","attr:ListGrid.alwaysShowEditors", "attr:ListGrid.editByCell", "attr:ListGrid.saveByCell", 
         "attr:ListGrid.waitForSave","attr:ListGrid.stopOnErrors", "attr:ListGrid.autoSaveEdits", 
         "attr:ListGrid.confirmCancelEditing","attr:ListGrid.cancelEditingConfirmationMessage", "attr:ListGrid.confirmDiscardEdits", 
         "attr:ListGrid.confirmDiscardEditsMessage","attr:ListGrid.discardEditsSaveButtonTitle", "attr:ListGrid.rowEndEditAction", 
         "attr:ListGrid.listEndEditAction","attr:ListGrid.enterKeyEditAction", "attr:ListGrid.escapeKeyEditAction", 
         "attr:ListGrid.editEvent","attr:ListGrid.editOnFocus", "attr:ListGrid.selectOnEdit", "attr:ListGridField.canToggle", 
         "attr:ListGrid.longTextEditorThreshold","attr:ListGrid.longTextEditorType", "type:RowEndEditAction", "type:EnterKeyEditAction", 
         "type:EscapeKeyEditAction","type:EditCompletionEvent", "type:ListGridEditEvent"],
        description:"Data being displayed by a grid may be edited within the grid, by showing editing\n interfaces embedded inside the cells of the grid.\n <P>\n <b>Enabling editing</b>\n <P>\n Editing is enabled when ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit','canEdit')} is <code>true</code>.  When enabled,\n the user can begin editing via the \n ${isc.DocUtils.linkForRef('attr:ListGrid.editEvent','editEvent')}, typically click or double-click.  Editing can also be triggered\n programmatically by a call to ${isc.DocUtils.linkForRef('method:ListGrid.startEditing','startEditing()')} or\n ${isc.DocUtils.linkForRef('method:ListGrid.startEditingNew','startEditingNew()')}.\n <P>\n <b>New record creation</b>\n <P>\n By default, editing is restricted to existing records.  Setting ${isc.DocUtils.linkForRef('attr:ListGrid.listEndEditAction')} to\n \"next\" allows the user to create new records by simply navigating off the end of the dataset\n with the keyboard.  Editing of new records can also be initiated with \n ${isc.DocUtils.linkForRef('method:ListGrid.startEditingNew')}, for example, from a button outside the grid.\n <P>\n <b>Saving changes</b>\n <P>\n Saving of changes is triggered automatically when the user navigates out of the row or cell\n being edited (based on ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell')}) or when the user ends editing.   For\n a \"mass update\" interface, automatic saving of changes can be disabled entirely via \n ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits','autoSaveEdits:false')}, in which case a manual call to \n ${isc.DocUtils.linkForRef('method:ListGrid.saveEdits','saveEdits()')} or ${isc.DocUtils.linkForRef('method:ListGrid.saveAllEdits','saveAllEdits()')} is required\n to trigger saving. \n <P>\n If a grid has no DataSource, saving means that the properties of the ${isc.DocUtils.linkForRef('object:ListGridRecord')}s\n in ${isc.DocUtils.linkForRef('attr:ListGrid.data','grid.data')} are directly changed.\n <P>\n For a grid with a DataSource, saving will be accomplished by using DataSource \"update\"\n operations for existing records, and DataSource \"add\" operations for new records.  If multiple\n records have been edited and ${isc.DocUtils.linkForRef('method:ListGrid.saveAllEdits','saveAllEdits()')} is called, \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','request queuing')} will be automatically used to enable all\n edits to be saved in one HTTP turnaround (if using the SmartClient Server).\n <P>\n By default, a grid will send only updated fields and the primaryKey field as part of \n ${isc.DocUtils.linkForRef('attr:DSRequest.data')} so that the server can discern which fields the user actually changed.\n However, the grid always includes the original field values in the\n dsRequest as ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues')}.\n <P>\n Note that although it is possible to load DataSource data without actually declaring a\n ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey field')}, a primaryKey must be declared for\n editing and saving.  The primaryKey value is how SmartClient identifies the changed\n record to the server.\n <P>\n <b>Validation</b>\n <P>\n Any time saving is attempted, validation is automatically triggered.  Values entered by the\n user will be checked against the ${isc.DocUtils.linkForRef('attr:ListGridField.validators')} and the\n ${isc.DocUtils.linkForRef('attr:DataSourceField.validators')}. Any invalid values abort an attempted save.\n <P>\n Similar to editing and saving, validation can be done on row transitions or on cell\n transitions by setting ${isc.DocUtils.linkForRef('attr:ListGrid.validateByCell','validateByCell')}, or can be disabled entirely\n via ${isc.DocUtils.linkForRef('attr:ListGrid.neverValidate','neverValidate:true')}.\n <P>\n <b>Editability of cells</b>\n <P>\n Editors will either be shown for the complete row or for a single cell based on \n ${isc.DocUtils.linkForRef('class:ListGrid','editByCell,editByCell')}.  Whether a cell can be edited can be controlled on a\n per field basis by setting ${isc.DocUtils.linkForRef('attr:ListGridField.canEdit','field.canEdit')}, or on a per-record basis\n by setting ${isc.DocUtils.linkForRef('attr:ListGrid.recordEditProperty','recordEditProperty')} on a \n ${isc.DocUtils.linkForRef('object:ListGridRecord','record')}, or can be controlled on an arbitrary, programmatic basis via\n an override of ${isc.DocUtils.linkForRef('method:ListGrid.canEditCell')}.\n <P>\n Cells which are not editable just display the cell's current value.\n <P>\n <b>Keyboard Navigation</b>\n <P>\n Full keyboard navigation is supported by default, including Tab and Shift-Tab to navigate\n between cells in a row, and Up Arrow and Down Arrow to traverse rows.  Several properties\n on both grids and fields, all named *EditAction, control navigation behavior of certain keys\n (eg Enter).\n <P>\n You can use ${isc.DocUtils.linkForRef('method:ListGrid.startEditing','startEditing(<i>rowNum</i>, <i>colNum</i>)')} to \n programmatically move editing to a particular cell, for example, during a \n ${isc.DocUtils.linkForRef('method:ListGridField.changed','field.changed()')} event.\n <P>\n <b>editValues (unsaved changes)</b>\n <P>\n The term \"editValues\" means changes that the user has made to the dataset which have not\n been saved.  The grid manages and stores editValues separately from the data itself in order\n to allow the user to revert to original values, and in order to enable to grid to send only\n updated fields to the server.  \n <P>\n Because editValues are stored separately, if you directly access the dataset (eg via \n <code>grid.data.get()</code>) you will see the records without the user's unsaved changes.\n Many APIs exist for retrieving and managing editValues (search for editValue).\n For the common case of needing to access the record-as-edited, you can call \n ${isc.DocUtils.linkForRef('method:ListGrid.getEditedRecord','grid.getEditedRecord(rowNum)')}.\n <P>\n When accessing and manipulating edited data, you should think carefully about whether\n you want to be working with the original data or with the edited version.  Values entered\n by the user may not have been validated yet, or may have failed validation, hence you may\n find a String value in a field of type \"date\" or \"int\", which could cause naive formatters or\n totalling functions to crash.\n <P>\n Setting editValues is fully equivalent to the user making changes to data via the editing UI.\n If you <i>also</i> allow editing external to the grid, setting editValues is one way to \n combine changes from external editors into the grid's edits, so that you can do a single \n save.\n <P>\n <b>Customizing Cell Editors</b>\n <P>\n When a cell is being edited, the editor displayed in the cell will be a ${isc.DocUtils.linkForRef('class:FormItem')}.\n The editor type for the cell will be determined by ${isc.DocUtils.linkForRef('method:ListGrid.getEditorType')} based on the\n specified ${isc.DocUtils.linkForRef('attr:ListGridField.editorType')} or ${isc.DocUtils.linkForRef('attr:ListGridField.type','data type')} for the field in\n question.\n <P>\n You can customize the editor by setting ${isc.DocUtils.linkForRef('attr:ListGridField.editorProperties')} to a set of\n properties that is valid for that FormItem type.  Custom FormItem classes are also allowed,\n for example, you may use ${isc.DocUtils.linkForRef('attr:FormItem.icons')} to create an icon that launches a separate\n ${isc.DocUtils.linkForRef('class:Dialog')} in order to provide an arbitrary interface that allows the user to select the\n value for a field.\n <P>\n <b>Events</b>\n <P>\n Editing triggers several events which you can provide handlers for in order to customize\n editing behavior.  Some of the most popular are ${isc.DocUtils.linkForRef('method:ListGridField.change','field.change()')},\n ${isc.DocUtils.linkForRef('method:ListGridField.changed','field.changed()')} for detecting changes made by the user,\n ${isc.DocUtils.linkForRef('method:ListGrid.cellChanged')} for detecting changes that have been successfully saved,\n and ${isc.DocUtils.linkForRef('method:ListGrid.editorEnter')} and ${isc.DocUtils.linkForRef('method:ListGrid.editorExit','editorExit()')} for detecting user\n navigation during editing.\n <P>\n You can also install event handlers directly on the FormItem-based editors used in the grid\n via ${isc.DocUtils.linkForRef('attr:ListGridField.editorProperties','editorProperties')} as mentioned above.  When handling\n events on items, or which involve items, be aware that in addition to standard \n ${isc.DocUtils.linkForRef('class:FormItem')} APIs, editors have the following properties:\n <P>\n - <code>rowNum</code>: The rowNum of the record being edited.<br>\n - <code>colNum</code>: The colNum of the cell being edited.<br>\n - <code>grid</code>: A pointer back to the listGrid containing the record.",
        name:"editing",
        seeAlso:[],
        type:"group"
    },
    "classAttr:GroupingMessages.bySecondsTitle":{
        ref:"classAttr:GroupingMessages.bySecondsTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a time field by seconds.",
        name:"bySecondsTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Seconds\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:DataSource.fieldMatchesFilter":{
        ref:"method:DataSource.fieldMatchesFilter",
        flags:"",
        description:" Compares a criteria value to a field value and returns whether they match, as follows:\n <ul>\n <li> any non-String filter value is directly compared (==) to the field value\n <li> any String filter value is compared according to\n      ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} in the passed <code>requestProperties</code>,\n      regardless of the actual field type\n <li> if the filter value is an Array, it matches if the field value is contained in the\n      Array\n </ul>\n",
        name:"fieldMatchesFilter",
        seeAlso:[],
        params:[
            {description:"field value to be compared", optional:false, name:"fieldValue",
             defaultValue:null, type:"object"},
            {description:"filter value to be compared", optional:false, name:"filterValue",
             defaultValue:null, type:"object"},
            {description:"optional dataSource request properties", optional:true,
             name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        returns:{description:"true if the filter and field values match, false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:TabSet.leftEdgeOffsets":{
        ref:"attr:TabSet.leftEdgeOffsets",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeOffsets</code>, \n <code>rightEdgeOffsets</code>, <code>topEdgeOffsets</code> and <code>bottomEdgeOffsets</code> \n propertes allow the offsets of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge offsets where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"leftEdgeOffsets",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "method:FormItem.disable":{
        ref:"method:FormItem.disable",
        flags:"",
        description:" Set this item to be disabled at runtime.\n",
        name:"disable",
        seeAlso:[
            {ref:"attr:FormItem.disabled", description:null}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["enable"]
    },
    "method:Canvas.click":{
        ref:"method:Canvas.click",
        flags:"A",
        description:"\n Executed when the left mouse is clicked (pressed and then released) on this widget.  No default\n implementation.\n\n",
        name:"click",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:GridRenderer.refreshCell":{
        ref:"method:GridRenderer.refreshCell",
        flags:"A",
        description:" Refresh an individual cell without redrawing the grid.\n <P>\n The cell's value, CSS class, and CSS text will be refreshed, to the current values returned\n by getCellValue(), getCellStyle() and getCellCSSText() respectively.\n\n",
        name:"refreshCell",
        seeAlso:[
            {ref:"method:GridRenderer.refreshCellStyle", description:"to update just styling"}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column number of cell to refresh", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["appearance"]
    },
    "attr:Canvas.snapVGap":{
        ref:"attr:Canvas.snapVGap",
        flags:"IRW",
        description:"The vertical grid size to use, in pixels, when snap-to-grid is enabled.",
        name:"snapVGap",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"attr:Canvas.snapResizeToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapResizeToGrid", description:null}
        ],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:ComboBoxItem.pickListFields":{
        ref:"attr:ComboBoxItem.pickListFields",
        flags:"IRA",
        description:"This property allows the developer to specify which field[s] will be displayed in the \n drop down list of options.\n <P>\n Only applies to databound pickLists (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}, or picklists\n with custom data set up via the advanced ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')} method.\n <P>\n If this property is unset, we display the ${isc.DocUtils.linkForRef('attr:PickList.displayField')}, if specified, \n otherwise the ${isc.DocUtils.linkForRef('attr:PickList.valueField')}.\n <P>\n If there are multiple fields, column headers will be shown for each field, the\n height of which can be customized via the ${isc.DocUtils.linkForRef('attr:PickList.pickListHeaderHeight')} attribute.\n <P>\n Each field to display should be specified as a ${isc.DocUtils.linkForRef('object:ListGridField')} object. Note that\n unlike in ${isc.DocUtils.linkForRef('class:ListGrid','listGrids')}, dataSource fields marked as \n ${isc.DocUtils.linkForRef('attr:DataSourceField.hidden','hidden:true')} will be hidden by default in pickLists. To\n override this behavior, ensure that you specify an explicit value for \n ${isc.DocUtils.linkForRef('method:ListGridField.showIf','showIf')}.",
        name:"pickListFields",
        seeAlso:[
            {ref:"attr:ComboBoxItem.valueField", description:null},
            {ref:"attr:PickList.pickListHeaderHeight", description:null}
        ],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:DataSource.showPrompt":{
        ref:"attr:DataSource.showPrompt",
        flags:"IRW",
        description:"Whether RPCRequests sent by this DataSource should enable \n ${isc.DocUtils.linkForRef('attr:RPCRequest.showPrompt')} if it's unset.",
        name:"showPrompt",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:Array.removeList":{
        ref:"method:Array.removeList",
        flags:"",
        description:" Remove all instances of objects in the specified list from this list, sliding the remaining\n objects around to fill gaps.\n\n",
        name:"removeList",
        seeAlso:[],
        params:[
            {description:"list of items to remove", optional:false, name:"list", defaultValue:null,
             type:"array"}
        ],
        returns:{description:"list of items passed in", type:"list"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "attr:DSRequest.callback":{
        ref:"attr:DSRequest.callback",
        flags:"IR",
        description:"Data, such as search criteria or an updated record, to be acted upon.  Contents differ by\n <code>operationType</code>, see ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource Operations')} for\n details.\n <P>\n This field is generally filled in by passing the \"data\" argument to methods such as\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}.",
        name:"callback",
        seeAlso:[],
        valueType:"DSCallback",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:SimpleType.shortDisplayFormatter":{
        ref:"method:SimpleType.shortDisplayFormatter",
        flags:"",
        description:" Formatter for values of this type when compact display is required, for example, in a\n ${isc.DocUtils.linkForRef('class:ListGrid')} or ${isc.DocUtils.linkForRef('class:TreeGrid')}.\n <P>\n When this formatter is called, the SimpleType object is available as \"this\".  \n <P>\n A formatter can make itself configurable on a per-component or per-field basis by\n checking properties on the component or field.  For example, a formatter for account IDs\n may want to omit a prefix in views where it is redundant, and could check a flag\n listGridField.omitAccountIdPrefix for this purpose.\n\n",
        name:"shortDisplayFormatter",
        seeAlso:[],
        params:[
            {description:"value to be formatted", optional:false, name:"value", defaultValue:null,
             type:"any"},
            {description:"field descriptor from the component calling the formatter, if                      applicable.  Depending on the calling component, this could be a\n                      ${isc.DocUtils.linkForRef('object:ListGridField')}, ${isc.DocUtils.linkForRef('object:TreeGridField')}, etc",
             optional:true, name:"field", defaultValue:null, type:"Field"},
            {description:"component calling this formatter, if applicable", optional:true,
             name:"component", defaultValue:null, type:"DataBoundComponent"},
            {description:"Full record, if applicable", optional:true, name:"record", defaultValue:null,
             type:"Object"}
        ],
        type:"method",
        definingClass:"class:SimpleType"
    },
    "attr:SelectItem.autoFetchData":{
        ref:"attr:SelectItem.autoFetchData",
        flags:"IRA",
        description:"If this select item retrieves its options from a <code>dataSource</code>, should options\n be fetched from the server when the item is first drawn, or should this fetch be\n delayed until the user opens the pickList.\n <P>\n The default is true in order to allow the user to select a value via keyboard input\n while keyboard focus is on the SelectItem but the pickList has not actually been shown.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"attr:PickList.optionDataSource", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "method:Canvas.getVSnapPosition":{
        ref:"method:Canvas.getVSnapPosition",
        flags:"A",
        description:" Override this method to provide a custom snap-to grid.  Note that you do not need to do\n this if your grid is regular (ie, grid points are every x pixels) - regular grids should be \n defined using ${isc.DocUtils.linkForRef('attr:Canvas.snapHGap')} and ${isc.DocUtils.linkForRef('attr:Canvas.snapVGap')}.\n You should only override this method if you want to provide support for a grid of \n irregularly-placed points\n\n",
        name:"getVSnapPosition",
        seeAlso:[],
        params:[
            {description:"y-coordinate of the drag event relative to the inside of this widget",
             optional:false, name:"coordinate", defaultValue:null, type:"integer"},
            {description:"\"before\" or \"after\" denoting whether the returned coordinate should   match the top or bottom edge of the current square. If unset ${isc.DocUtils.linkForRef('attr:Canvas.snapHDirection')} will\n   be used by default",
             optional:true, name:"direction", defaultValue:null, type:"string"}
        ],
        returns:{description:"The vertical coordinate to snap to", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:StretchImgButton.iconClick":{
        ref:"method:StretchImgButton.iconClick",
        flags:"",
        description:" If this button is showing an ${isc.DocUtils.linkForRef('attr:StretchImgButton.icon','icon')}, a separate click\n handler for the icon may be defined as <code>this.iconClick</code>.\n Returning false will suppress the standard button click handling code.\n",
        name:"iconClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:FormItem.valueIconWidth":{
        ref:"attr:FormItem.valueIconWidth",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} is specified, use this property to specify a width for\n the value icon written out.",
        name:"valueIconWidth",
        seeAlso:[
            {ref:"attr:FormItem.valueIconHeight", description:null},
            {ref:"attr:FormItem.valueIconSize", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "type:FetchMode":{
        ref:"type:FetchMode",
        values:[
            {description:"All records that match the current filter are fetched.  Sorting is                performed on the client.",
             value:"\"basic\""},
            {description:"Only requested ranges of records are fetched, with predictive fetch                ahead.  Sorting is performed on the server.",
             value:"\"paged\""},
            {description:"All records available from the DataSource are fetched.  Filtering by                search criteria and sorting are both performed on the client.",
             value:"\"local\""}
        ],
        description:"Mode of fetching records from the server.\n <P>\n Generally, \"paged\" mode should be used unless the maximum number of records is\n guaranteed to be small.",
        name:"FetchMode",
        seeAlso:[],
        type:"type",
        groups:["fetching"]
    },
    "attr:DataSource.clientOnly":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.clientOnly",
        description:"Whether to act as a \"client-only\" DataSource, which has no permanent storage and never\n contacts the server, instead using a set of test data to respond to requests in the same\n manner as a server-based DataSource might.\n <P>\n See ${isc.DocUtils.linkForRef('group:clientOnlyDataSources','this discussion')} for ways to populate a\n client-only DataSource with test data.",
        name:"clientOnly",
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('localDataSource')}"],
        serverDS:"allowed",
        defaultValue:"false",
        groups:["clientOnlyDataSources"],
        definingClass:"class:DataSource"
    },
    "method:Canvas.hover":{
        ref:"method:Canvas.hover",
        flags:"",
        description:" If <code>canHover</code> is true for this widget, the <code>hover</code> string method will\n be fired when the user hovers over this canvas. If this method returns false, it will\n suppress the default behavior of showing a hover canvas if <code>this.showHover</code> \n is true.\n",
        name:"hover",
        seeAlso:[
            {ref:"attr:Canvas.canHover", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:ColorPicker.swatchWidth":{
        ref:"attr:ColorPicker.swatchWidth",
        flags:"IR",
        description:"Displayed width of the color swatch image. The default width is approximately\n that used by the Windows&#174; XP color picking window",
        name:"swatchWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"170",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:ListGridField.parseEditorValue":{
        ref:"method:ListGridField.parseEditorValue",
        flags:"",
        description:" Method used to convert the value displayed in an editor for some cell in this field into\n a raw value for saving.<br>\n Takes precedence over <code>parseEditorValue</code> defined at the grid level.\n\n",
        name:"parseEditorValue",
        seeAlso:[
            {ref:"method:ListGrid.parseEditorValue", description:null},
            {ref:"method:ListGridField.formatEditorValue", description:null}
        ],
        params:[
            {description:"value displayed in the editor for the cell", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"record object for the row being edited. May be null if this                          is a new row being added to the end of the list.",
             optional:false, name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"},
            {description:"A pointer to the ListGrid displaying the cell", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid instance"}
        ],
        returns:{description:"raw value for the field derived from formatted value in editor", type:"any"},
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "type:EditCompletionEvent":{
        ref:"type:EditCompletionEvent",
        values:[
            {description:"User clicked outside editor during edit.", value:"ListGrid.CLICK_OUTSIDE"},
            {description:"User started editing another row by clicking on it", value:"ListGrid.CLICK"},
            {description:"User started editing another row by double                               clicking",
             value:"ListGrid.DOUBLE_CLICK"},
            {description:"Enter pressed.", value:"ListGrid.ENTER_KEYPRESS"},
            {description:"User pressed Escape.", value:"ListGrid.ESCAPE_KEYPRESS"},
            {description:"Up arrow key pressed.", value:"ListGrid.UP_ARROW_KEYPRESS"},
            {description:"down arrow key.", value:"ListGrid.DOWN_ARROW_KEYPRESS"},
            {description:"User pressed Tab.", value:"ListGrid.TAB_KEYPRESS"},
            {description:"User pressed Shift+Tab.", value:"ListGrid.SHIFT_TAB_KEYPRESS"},
            {description:"Edit moved to a different field (same row)",
             value:"ListGrid.EDIT_FIELD_CHANGE"},
            {description:"Edit completed via explicit function call", value:"ListGrid.PROGRAMMATIC"}
        ],
        description:"What event / user interaction type caused cell editing to complete.",
        name:"EditCompletionEvent",
        seeAlso:[],
        type:"type",
        groups:["editing"]
    },
    "attr:Canvas.snapOnDrop":{
        ref:"attr:Canvas.snapOnDrop",
        flags:"IRWA",
        description:"When this canvas is dropped onto an object supporting snap-to-grid, should it snap\n to the grid (true, the default) or just drop wherever the mouse is (false).",
        name:"snapOnDrop",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"method:Canvas.shouldSnapOnDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "classAttr:RPCResponse.STATUS_FAILURE":{
        ref:"classAttr:RPCResponse.STATUS_FAILURE",
        flags:"R",
        description:"Indicates a generic failure on the server.  \n See the error handling section in ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager documentation')}\n for more information.",
        name:"STATUS_FAILURE",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-1",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "method:Window.setShowHeaderIcon":{
        ref:"method:Window.setShowHeaderIcon",
        flags:"",
        description:" Dynamically update ${isc.DocUtils.linkForRef('attr:Window.showHeaderIcon')} to show / hide the headerIcon\n",
        name:"setShowHeaderIcon",
        seeAlso:[
            {ref:"attr:Window.headerControls", description:null},
            {ref:"attr:Window.showHeaderIcon", description:null}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "attr:Scrollbar.endThumbOverlap":{
        ref:"attr:Scrollbar.endThumbOverlap",
        flags:"IRA",
        description:"Number of pixels the thumb is allowed to overlap the buttons at the end of the track.\n Default prevents doubling of 1px borders.  Set higher to allow media that shows curved\n joins between the track button and ScrollThumb.",
        name:"endThumbOverlap",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "attr:ListGridField.icons":{
        ref:"attr:ListGridField.icons",
        flags:"IRA",
        description:"If this field is editable, this property can be used to specify \n ${isc.DocUtils.linkForRef('attr:FormItem.icons','icons')} to be displayed in the editors displayed for this field",
        name:"icons",
        seeAlso:[],
        valueType:"Array of FormItemIcon Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:RadioGroupItem.itemProperties":{
        ref:"attr:RadioGroupItem.itemProperties",
        flags:"IR",
        description:"Map of properties to apply to generated items within this RadioGroup. This allows you to \n customize the generated radio items for this item.",
        name:"itemProperties",
        seeAlso:[],
        valueType:"RadioItem Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RadioGroupItem"
    },
    "method:ValuesManager.getItem":{
        ref:"method:ValuesManager.getItem",
        flags:"",
        description:" Retrieve a ${isc.DocUtils.linkForRef('class:FormItem')} from this ValuesManager.\n <P>\n Takes a field ${isc.DocUtils.linkForRef('attr:FormItem.name','name')} or ${isc.DocUtils.linkForRef('type:dataPath')}, and searches through the\n members of this valuesManager for an editor for that field. If found the appropriate\n formItem will be returned. Note that if a dataPath is passed in, it should be the full\n data path for the item, including any canvas level ${isc.DocUtils.linkForRef('attr:Canvas.dataPath','dataPath')} specified\n on the member form containing this form item.\n <br>Note: Unlike the <code>DynamicForm</code> class, this method will not return an \n item by index\n",
        name:"getItem",
        seeAlso:[],
        params:[
            {description:"item fieldName or dataPath identifier", optional:false, name:"itemID",
             defaultValue:null, type:"fieldName | dataPath"}
        ],
        returns:{description:"form item for editing/displaying the appropriate field, or null if   no formItem can be found for the field.",
         type:"formItem"},
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "attr:Layout.dropLine":{
        ref:"attr:Layout.dropLine",
        flags:"R",
        description:"Line showed to mark the drop position when components are being dragged onto this Layout.\n A simple Canvas typically styled via CSS.  The default dropLine.styleName is\n \"layoutDropLine\".",
        name:"dropLine",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragMove')}"],
        definingClass:"class:Layout"
    },
    "classMethod:DataSource.getSortSpecifiers":{
        ref:"classMethod:DataSource.getSortSpecifiers",
        flags:"",
        description:" Return a an array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s, given an array of Strings in the format\n expected by ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')}.\n\n",
        name:"getSortSpecifiers",
        seeAlso:[],
        params:[
            {description:"A list of sortBy strings in the format expected by ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')}",
             optional:false, name:"sortBy", defaultValue:null, type:"Array of String"}
        ],
        returns:{description:"An array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s equivalent to the passed in string array",
         type:"Array of String"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:Layout.layoutBottomMargin":{
        ref:"attr:Layout.layoutBottomMargin",
        flags:"IRW",
        description:"Space outside of all members, on the bottom side.  Defaults to ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')}.\n <P>\n Requires a manual call to <code>setLayoutMargin()</code> if changed on the fly.",
        name:"layoutBottomMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "method:Dialog.saveData":{
        ref:"method:Dialog.saveData",
        flags:"A",
        description:" Method to save this Dialog's data. Called from <code>okClick()</code>, \n <code>applyClick()</code>.\n No default implementation - override to perform some action if required.\n      \n",
        name:"saveData",
        seeAlso:[
            {ref:"method:Dialog.okClick", description:null},
            {ref:"method:Dialog.applyClick", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:RichTextEditor.colorControls":{
        ref:"attr:RichTextEditor.colorControls",
        flags:"IRA",
        description:"Control group for modifying text color / background color. \n  Consists of an array of ${isc.DocUtils.linkForRef('type:ControlName')}s.\n  To display this group of controls for some RichTextEditor,\n  include <code>\"formatControls\"</code> in the ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} array",
        name:"colorControls",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[\"color\", \"backgroundColor\"]",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "method:ImgButton.setSelected":{
        ref:"method:ImgButton.setSelected",
        flags:"",
        description:" Select this object.\n",
        name:"setSelected",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:FilterBuilder.subClauseButtonPrompt":{
        ref:"attr:FilterBuilder.subClauseButtonPrompt",
        flags:"IR",
        description:"The hover prompt text for the subClauseButton.",
        name:"subClauseButtonPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Add Subclause\"",
        type:"attr",
        definingClass:"class:FilterBuilder",
        groups:["i18nMessages"]
    },
    "attr:CubeGrid.facetLabelHoverVAlign":{
        ref:"attr:CubeGrid.facetLabelHoverVAlign",
        flags:"IRWA",
        description:"Allows the developer to override the vertical text alignment of hover tips shown for\n facetLabels.  If unspecified the hover canvas content alignment will be set by\n <code>this.hoverVAlign</code> if specified.",
        name:"facetLabelHoverVAlign",
        seeAlso:[
            {ref:"attr:Canvas.hoverVAlign", description:null}
        ],
        valueType:"VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "method:Canvas.blur":{
        ref:"method:Canvas.blur",
        flags:"",
        description:" If this canvas has keyboard focus, blur it. After this method, the canvas\n will no longer appear focussed and will stop recieving keyboard events.\n",
        name:"blur",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "method:Canvas.visibleAtPoint":{
        ref:"method:Canvas.visibleAtPoint",
        flags:"A",
        description:" Does this widget contain the specified global (x,y) coordinates, and have no other widgets\n also at the specified position, obscuring this one?  This is commonly used for (for example)\n drag and drop interactions.\n\n",
        name:"visibleAtPoint",
        seeAlso:[],
        params:[
            {description:"GLOBAL x-coordinate", optional:false, name:"x", defaultValue:null,
             type:"number"},
            {description:"GLOBAL y-coordinate", optional:false, name:"y", defaultValue:null,
             type:"number"},
            {description:"point lies within our viewport rather than                                              just our drawn area",
             optional:true, name:"withinViewport", defaultValue:null, type:"boolean"},
            {description:"If passed ignore widget(s), do not check whether                                           those widgets occludes this one.",
             optional:true, name:"ignoreWidgets", defaultValue:null, type:"canvas"}
        ],
        returns:{description:"true if this object contains the specified point; false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "classAttr:LoginDialog.PASSWORD_TITLE":{
        ref:"classAttr:LoginDialog.PASSWORD_TITLE",
        flags:"IRW",
        description:"Default title for the ${isc.DocUtils.linkForRef('attr:LoginDialog.passwordItem','\"passwordItem\"')} field in the\n dialog displayed by ${isc.DocUtils.linkForRef('classMethod:isc.showLoginDialog')}.",
        name:"PASSWORD_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Password\"",
        type:"classAttr",
        definingClass:"class:LoginDialog",
        groups:["i18nMessages"]
    },
    "attr:Canvas.contextMenu":{
        ref:"attr:Canvas.contextMenu",
        flags:"IRWA",
        description:"Context menu to show for this object, an instance of the Menu widget.",
        name:"contextMenu",
        seeAlso:[
            {ref:"method:Canvas.showContextMenu", description:null}
        ],
        valueType:"Menu",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('contextMenus')}"],
        definingClass:"class:Canvas",
        groups:["cues"]
    },
    "attr:DataSource.resultBatchSize":{
        ref:"attr:DataSource.resultBatchSize",
        flags:"IRWA",
        description:"Very advanced: for servers that do not support paging, and must return large numbers of XML\n records in one HTTP response, SmartClient breaks up the processing of the response\n in order to avoid the \"script running slowly\" dialog appearing for an end user.\n <P>\n If you have a relatively small number of records with a great deal of properties or\n subojects on each record, and you have not set ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')} to eliminate unused\n data, and you see the \"script running slowly\" dialog, you may need to set this number\n lower.",
        name:"resultBatchSize",
        seeAlso:[],
        valueType:"int",
        defaultValue:"150",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:CubeGrid.canSelectValues":{
        ref:"attr:CubeGrid.canSelectValues",
        flags:"IRW",
        description:"Determines whether cell values in the body can be selected.",
        name:"canSelectValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:Canvas.shadowImage":{
        ref:"attr:Canvas.shadowImage",
        flags:"IRA",
        description:"Base name of the series of images for the sides, corners, and center of the shadow.\n <P>\n The actual image names fetched for the dropShadow combine the segment name and the\n <code>shadowDepth</code> setting.  For example, given \"ds.png\" as the base name, a depth of\n 4, and the top-left segment of the shadow, we'd use \"ds4_TL.png\".\n <P>\n The names for segments are the same as those given for controlling resizable edges; see\n ${isc.DocUtils.linkForRef('attr:Canvas.resizeFrom')}.  The center segment has the name \"center\".  The center segment is\n the only segment that doesn't include the depth in the URL, so the final image name for the\n center given a baseName of \"ds.png\" would be just \"ds_center.png\".",
        name:"shadowImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]ds.png\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["shadow"]
    },
    "classAttr:GroupingMessages.byQuarterTitle":{
        ref:"classAttr:GroupingMessages.byQuarterTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by quarter.",
        name:"byQuarterTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Quarter\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:CubeGrid.fixedFacetValues":{
        ref:"attr:CubeGrid.fixedFacetValues",
        flags:"IR",
        description:"A ${isc.DocUtils.linkForRef('type:FacetValueMap')} describing the set of facet values that should be regarded as\n \"fixed\" in this cubeGrid.  These are used as fixed criteria for load on demand, and also\n allow using a dataset with more facets in it than are currently shown in the grid.",
        name:"fixedFacetValues",
        seeAlso:[
            {ref:"method:CubeGrid.addFacet", description:null},
            {ref:"method:CubeGrid.removeFacet", description:null}
        ],
        valueType:"FacetValueMap",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "group:flatFields":{
        ref:"group:flatFields",
        refs:["attr:DSRequest.useFlatFields", "attr:DSRequest.useFlatHeaderFields", 
         "attr:WSRequest.useFlatFields"],
        name:"flatFields",
        seeAlso:[],
        type:"group"
    },
    "classAttr:Operators.betweenTitle":{
        ref:"classAttr:Operators.betweenTitle",
        flags:"IR",
        description:"Title for the \"between\" operator",
        name:"betweenTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"between\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:DataSource.title":{
        ref:"attr:DataSource.title",
        flags:"IRW",
        description:"User-visible name for this DataSource.\n <P>\n For example, for the supplyItem DataSource, \"Supply Item\".\n <P>\n Defaults to <code>dataSource.ID</code>.",
        name:"title",
        seeAlso:[],
        valueType:"string",
        defaultValue:"dataSource.ID",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["titles"]
    },
    "method:Dialog.yesClick":{
        ref:"method:Dialog.yesClick",
        flags:"",
        description:" Handle a click on the 'yes' button of this Dialog.\n Default implementation is to return <code>true</code>.\n Override to do something else\n",
        name:"yesClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "method:Operator.condition":{
        ref:"method:Operator.condition",
        flags:"",
        description:" Method which actually evaluates whether a given record meets a ${isc.DocUtils.linkForRef('object:Criterion')}.\n <P>\n For operators that act on ${isc.DocUtils.linkForRef('attr:Criterion.criteria','sub-criteria')}, call\n ${isc.DocUtils.linkForRef('method:DataSource.evaluateCriterion')} to evaluate sub-criteria.\n <P>\n Because criteria are sometimes applied to user-entered data that has not been validated, a\n robust <code>condition()</code> function should expect that data found in a ${isc.DocUtils.linkForRef('object:Record')}\n may be null, NaN, not the correct type (eg \"NA\" for a type:\"date\" field) or otherwise out of\n the expected range.\n <P>\n Note that an Operator has access both to the ${isc.DocUtils.linkForRef('object:Criterion')} object, allowing operators\n that act on more than one field or perform calculations, and access to the ${isc.DocUtils.linkForRef('object:Operator')}\n object itself, allowing a <code>condition()</code> function to be shared across a range of\n related operators with different ${isc.DocUtils.linkForRef('type:OperatorId')}s.\n \n",
        name:"condition",
        seeAlso:[],
        params:[
            {description:"value from the field supplied as ${isc.DocUtils.linkForRef('attr:Criterion.fieldName')}, if   applicable",
             optional:false, name:"value", defaultValue:null, type:"any"},
            {description:"record being evaluated", optional:false, name:"record", defaultValue:null,
             type:"Record"},
            {description:"fieldName supplied as ${isc.DocUtils.linkForRef('attr:Criterion.fieldName')}, if applicable",
             optional:false, name:"fieldName", defaultValue:null, type:"any"},
            {description:"criterion definition", optional:false, name:"criterion", defaultValue:null,
             type:"Criterion"},
            {description:"operator definition", optional:false, name:"operator", defaultValue:null,
             type:"Operator"}
        ],
        returns:{description:"whether the field passes this criteria", type:"boolean"},
        type:"method",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "method:DataSource.getUpdatedData":{
        ref:"method:DataSource.getUpdatedData",
        flags:"A",
        description:" Helper method to retrieve the updated data from a successful dataSource update or add \n operation.\n",
        name:"getUpdatedData",
        seeAlso:[],
        params:[
            {description:"Update request object passed to the server. Note that this  request may have been modified by ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}",
             optional:false, name:"dsRequest", defaultValue:null, type:"DSRequest"},
            {description:"Response returned by the server", optional:false, name:"dsResponse",
             defaultValue:null, type:"DSResponse"},
            {description:"If ${isc.DocUtils.linkForRef('attr:DSResponse.data')} is empty, should data be  derived from the submitted request.",
             optional:false, name:"useDataFromRequest", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"data (DataSourceRecord || Array of DataSourceRecords} Updated data.", type:null},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:TreeGrid.customIconDropProperty":{
        ref:"attr:TreeGrid.customIconDropProperty",
        flags:"IRWA",
        description:"This property allows the developer to rename the \n ${isc.DocUtils.linkForRef('attr:TreeNode.showDropIcon','default node.showDropIcon')} property.",
        name:"customIconDropProperty",
        seeAlso:[
            {ref:"attr:TreeGrid.customIconProperty", description:null},
            {ref:"attr:TreeGrid.showCustomIconDrop", description:null}
        ],
        valueType:"string",
        defaultValue:"\"showDropIcon\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:CheckboxItem.uncheckedImage":{
        ref:"attr:CheckboxItem.uncheckedImage",
        flags:"IRW",
        description:"URL for the image to display when this checkbox is not selected, or unchecked",
        name:"uncheckedImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/unchecked.gif\"",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["appearance"]
    },
    "attr:OperationBinding.useForCacheSync":{
        ref:"attr:OperationBinding.useForCacheSync",
        flags:"IR",
        description:"For an operationBinding of ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"fetch\" which specifies no\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId','operationId')}, this property determines whether the operationBinding \n should be used for cache synchronization purposes (ie, to retrieve the record most recently\n added or changed).  This property has no effect on an operationBinding that specifies an \n operationId - see ${isc.DocUtils.linkForRef('attr:OperationBinding.cacheSyncOperation')}.\n <p>\n In order to work correctly with SmartClient's cache synchronization system, an \n operationBinding marked useForCacheSync should have the following properties:\n <ul>\n <li>Able to complete its retrieval using no context other than the values of the primary key\n fields declared in the dataSource (these will be provided in the $criteria object passed to \n the operation)</li>\n <li>Returns the entire record, including any values that may require joins to other tables\n or other complexities</li>\n </ul>\n This property is only applicable to DataSources of type \"sql\".",
        name:"useForCacheSync",
        seeAlso:[
            {ref:"attr:OperationBinding.cacheSyncOperation", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:SpinnerItem.step":{
        ref:"attr:SpinnerItem.step",
        flags:"IRW",
        description:"How much should the value be incremented / decremented when the user hits the icons\n  to increase / decrease the value?",
        name:"step",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('spinnerItem')}"],
        definingClass:"class:SpinnerItem"
    },
    "classAttr:ValidatorTypes.floatRange":{
        ref:"classAttr:ValidatorTypes.floatRange",
        flags:"IR",
        description:"Tests whether the value for this field is a floating point number within the range \n  specified.  The <code>max</code> and <code>min</code> properties on the validator\n  are used to determine the acceptable range.<br>\n Note that the <code>errorMessage</code> for this validator will be evaluated as\n a dynamicString - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"floatRange",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:ImgTab.skinImgDir":{
        ref:"attr:ImgTab.skinImgDir",
        flags:"IRW",
        description:"Base path for the images.   <B>Note</B> that when used within a TabSet, the\n ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')} is appended as an additional path segment, yielding\n \"images/Tab/top/\" et al.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/Tab/\"",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "attr:TileLayout.dragLine":{
        ref:"attr:TileLayout.dragLine",
        flags:"R",
        description:"Canvas used to display a drop indicator when a another canvas is dragged over this widget.",
        name:"dragLine",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout"
    },
    "method:ColumnTree.anySelected":{
        ref:"method:ColumnTree.anySelected",
        flags:"",
        description:" Whether at least one item is selected in the supplied column (the first column if none is passed)\n\n",
        name:"anySelected",
        seeAlso:[],
        params:[
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"true == at least one item is selected in the supplied column,  false == nothing at all is selected in the supplied column (note that there may be selections\n in other columns in this columnTree)",
         type:"Boolean"},
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "class:Window":{
        methods:["method:Window.setShowCloseButton", "method:Window.setShowMinimizeButton", 
         "method:Window.setShowMaximizeButton","method:Window.setShowHeaderIcon", "method:Window.setTitle", "method:Window.setStatus", 
         "method:Window.setSrc","method:Window.addItem", "method:Window.removeItem", "method:Window.addItems", 
         "method:Window.removeItems","method:Window.addMember", "method:Window.addMembers", "method:Window.setAutoSize", 
         "method:Window.shouldDismissOnEscape","method:Window.centerInPage", "method:Window.flash", "method:Window.minimize", 
         "method:Window.restore","method:Window.maximize", "method:Window.closeClick"],
        exampleConfig:"Window.xml",
        ref:"class:Window",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:Window.canDragReposition", "attr:Window.keepInParentRect", 
         "attr:Window.canDragResize","attr:Window.isModal", "attr:Window.modalMask", "attr:Window.showModalMask", 
         "attr:Window.modalMaskOpacity","attr:Window.modalMaskStyle", "attr:Window.autoCenter", 
         "attr:Window.dismissOnOutsideClick","attr:Window.dismissOnEscape", "attr:Window.body", "attr:Window.showBody", 
         "attr:Window.bodyStyle","attr:Window.bodyColor", "attr:Window.hiliteBodyColor", "attr:Window.items", 
         "attr:Window.src","attr:Window.contentsType", "attr:Window.bodyConstructor", "attr:Window.bodyDefaults", 
         "attr:Window.contentLayout","attr:Window.autoSize", "attr:Window.header", "attr:Window.showHeader", 
         "attr:Window.headerBackground","attr:Window.showHeaderBackground", "attr:Window.headerStyle", "attr:Window.headerSrc", 
         "attr:Window.headerControls","attr:Window.hiliteHeaderStyle", "attr:Window.hiliteHeaderSrc", "attr:Window.headerLabel", 
         "attr:Window.showTitle","attr:Window.title", "attr:Window.headerLabelDefaults", "attr:Window.headerIcon", 
         "attr:Window.showHeaderIcon","attr:Window.headerIconDefaults", "attr:Window.canFocusInHeaderButtons", 
         "attr:Window.closeButton","attr:Window.showCloseButton", "attr:Window.minimizeButton", 
         "attr:Window.showMinimizeButton","attr:Window.minimized", "attr:Window.defaultMinimizeHeight", "attr:Window.minimizeHeight", 
         "attr:Window.animateMinimize","attr:Window.minimizeTime", "attr:Window.minimizeAcceleration", 
         "attr:Window.restoreButton","attr:Window.maximized", "attr:Window.maximizeButton", "attr:Window.showMaximizeButton", 
         "attr:Window.footer","attr:Window.showFooter", "attr:Window.footerHeight", "attr:Window.statusBar", 
         "attr:Window.showStatusBar","attr:Window.resizer", "attr:Window.showResizer", "attr:Window.status"],
        description:"A general purpose Window class for implementing dialogs, portlets, alerts, prompts, wizards\n and desktop-like windowing interfaces.\n <P>\n Windows can contain arbitrary SmartClient components, configured via the ${isc.DocUtils.linkForRef('attr:Window.items')}\n property.  Windows may be ${isc.DocUtils.linkForRef('attr:Window.isModal','modal')} or non-modal.\n <P>\n Windows provide a series of highly configurable and skinnable ${isc.DocUtils.linkForRef('type:AutoChild','autoChildren')}\n including a header, various header controls, footer, and corner resizer.\n <P>\n The more specialized ${isc.DocUtils.linkForRef('class:Dialog')} subclass of Window has additional functionality targetted\n at simple prompts and confirmations, such as buttons with default actions, and single-method\n ${isc.DocUtils.linkForRef('classMethod:isc.warn','shortcuts')} for common application dialogs.",
        name:"Window",
        seeAlso:[],
        type:"class"
    },
    "classMethod:isc.getErrorsHTML":{
        ref:"classMethod:isc.getErrorsHTML",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is false, the form will render all errors in a list at\n the top of the form. This method returns the HTML for this list of errors.\n",
        name:"getErrorsHTML",
        seeAlso:[],
        params:[
            {description:"Map of field names to error messages. Each field may contain a single                        error message (string) or an array of errors",
             optional:false, name:"errors", defaultValue:null, type:"object"}
        ],
        returns:{description:"error HTML.", type:"HTML"},
        type:"method",
        definingClass:"class:isc",
        groups:["validation"]
    },
    "classMethod:DataSource.getAutoTitle":{
        ref:"classMethod:DataSource.getAutoTitle",
        flags:"",
        description:" Utility method to derive a reasonable user-visible title from an identifier.\n <P>\n The following approach is taken:\n <ul>\n <li> any underscores (_) or dollar signs ($) become spaces, except that there will never\n      be either a leading or trailing space.\n <li> if the fieldName is either entirely uppercase or lowercase, all words separated by\n      spaces are given a leading capital letter.  Example USER_NAME or user_name -> \"User\n      Name\".\n <li> if there is any use of mixed case, camelCaps convention is assumed, and the field name\n      is split into separate words based on 1) everywhere an uppercase letter appears after a\n      lowercase letter 2) everywhere a series of uppercase letters ends.  Letter case will\n      not be modified, with the exception that the first word will have it's first letter \n      capitalized.  Example: useHTTPProxy -> \"Use HTTP Proxy\"\n </ul>\n\n",
        name:"getAutoTitle",
        seeAlso:[],
        params:[
            {description:"identifier for which a title is desired.", optional:false, name:"identifier",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"auto-derived title", type:"String"},
        type:"classMethod",
        definingClass:"class:DataSource",
        groups:["title"]
    },
    "attr:ListGrid.recordSummaryBaseStyle":{
        ref:"attr:ListGrid.recordSummaryBaseStyle",
        flags:"IRWA",
        description:"If showing any record summary fields (IE: fields of ${isc.DocUtils.linkForRef('type:ListGridFieldType','type:\"summary\"')}),\n this attribute specifies a custom base style to apply to cells in the summary field",
        name:"recordSummaryBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"recordSummaryCell\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "classAttr:Validator.notAString":{
        ref:"classAttr:Validator.notAString",
        flags:"IRA",
        description:"Default error message to display when standard <code>isString</code> type validator\n  returns false.",
        name:"notAString",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be a string.\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "type:DragTrackerMode":{
        ref:"type:DragTrackerMode",
        values:[
            {description:"Don't display a drag tracker at all", value:"\"none\""},
            {description:"Display an icon to represent the record(s) being dragged. Icon src is              derived from ${isc.DocUtils.linkForRef('method:ListGrid.getDragTrackerIcon')}",
             value:"\"icon\""},
            {description:"Display a title for the record being dragged. Title derived from              ${isc.DocUtils.linkForRef('method:ListGrid.getDragTrackerTitle')}",
             value:"\"title\""},
            {description:"Display the entire record being dragged", value:"\"record\""}
        ],
        description:"When records are being dragged from within a ListGrid, what sort of drag-tracker\n should be displayed?",
        name:"DragTrackerMode",
        seeAlso:[],
        type:"type",
        groups:["dragTracker"]
    },
    "attr:FormulaBuilder.samplePrompt":{
        ref:"attr:FormulaBuilder.samplePrompt",
        flags:"IRWA",
        getter:["getSamplePrompt"],
        description:"This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed.\n <P>\n Default value returns <P>\n <code>\n <i>For Record: + the value of the rows title-field <br>\n Output: + the result of he generated function<br>\n </i>\n </code>",
        name:"samplePrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"<nobr>For record: \\${title}</nobr><br><nobr>Output: \\${output}</nobr>\"",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["i18nMessages"]
    },
    "attr:ColorPicker.lumWidth":{
        ref:"attr:ColorPicker.lumWidth",
        flags:"IR",
        description:"Width of the Luminosity bar",
        name:"lumWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"15",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:FormItem.colSpan":{
        ref:"attr:FormItem.colSpan",
        flags:"IRW",
        description:"Number of columns that this item spans.  \n <P>\n The <code>colSpan</code> setting does not include the title shown for items with\n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')}:true, so the effective <code>colSpan</code> is one higher than this\n setting for items that are showing a title and whose ${isc.DocUtils.linkForRef('type:TitleOrientation')} is either\n \"left\" or \"right\".",
        name:"colSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "method:ListGrid.fieldIsVisible":{
        ref:"method:ListGrid.fieldIsVisible",
        flags:"",
        description:" Check whether a field is currently visible\n\n",
        name:"fieldIsVisible",
        seeAlso:[],
        params:[
            {description:"field to be checked", optional:false, name:"field", defaultValue:null,
             type:"field name or field object"}
        ],
        returns:{description:"true if the field is currently visible, false otherwise.", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DataSource.enumConstantProperty":{
        ref:"attr:DataSource.enumConstantProperty",
        flags:"IA",
        description:"The name of the property this DataSource uses for constant name when translating\n Java enumerated types to and from Javascript, if the ${isc.DocUtils.linkForRef('type:EnumTranslateStrategy')} is set\n to \"bean\".  Defaults to \"_constant\" if not set.\n <p>\n  This property is only applicable if you are using the SmartClient server",
        name:"enumConstantProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:Hilite.htmlAfter":{
        ref:"attr:Hilite.htmlAfter",
        flags:"IR",
        description:"HTML to append to the end of cell values where this hilite is applied.",
        name:"htmlAfter",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "class:BlurbItem":{
        methods:[],
        ref:"class:BlurbItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:BlurbItem.showTitle", "attr:BlurbItem.colSpan", "attr:BlurbItem.textBoxStyle", 
         "attr:BlurbItem.wrap","attr:BlurbItem.clipValue"],
        description:"FormItem intended for inserting blurbs of instructional HTML into DynamicForms.\n <p>\n Set the <code>defaultValue</code> of this item to the HTML you want to embed in the form.",
        name:"BlurbItem",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.header":{
        ref:"attr:ListGrid.header",
        flags:"R",
        description:"A Toolbar used to manager the headers shown for each column of the grid.",
        name:"header",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader"]
    },
    "classAttr:RPCManager.actionURL":{
        ref:"classAttr:RPCManager.actionURL",
        flags:"RW",
        description:"The actionURL specifies the URL to which the RPC request will be sent.  Note that if you\n override this global default and your application uses DataSource databound components,\n you'll need to dispatch the DataSource requests from your RPC handler.  Your other option is\n to specify a url on a per-request basis.",
        name:"actionURL",
        seeAlso:[
            {ref:"attr:RPCRequest.actionURL", description:null}
        ],
        valueType:"URL",
        defaultValue:"RPCManager.actionURL",
        type:"classAttr",
        definingClass:"class:RPCManager"
    },
    "attr:Flashlet.pluginsPage":{
        ref:"attr:Flashlet.pluginsPage",
        flags:"IR",
        description:"This attribute specifies the page the user should go to to get the plugin required to view\n  this flashlet.\n  <p>\n  The default pluginsPage is: \"http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash\"\n  <p>",
        name:"pluginsPage",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "attr:RestDataSource.sendMetaData":{
        ref:"attr:RestDataSource.sendMetaData",
        flags:"IR",
        description:"Should operation meta data be included when assmebling parameters to send \n to the server? If true, meta data parameters will be prefixed with the \n ${isc.DocUtils.linkForRef('attr:RestDataSource.metaDataPrefix')}.<br>\n Applies to operations where OperationBinding.dataProtocol is set to \n <code>\"getParams\"</code> or <code>\"postParams\"</code> only.",
        name:"sendMetaData",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "classAttr:ValidatorTypes.requiredIf":{
        ref:"classAttr:ValidatorTypes.requiredIf",
        flags:"IR",
        description:"RequiredIf type validators should be specified with an <code>expression</code>\n  property set to a ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')}, which takes three\n  parameters:<ul>\n  <li>item - the DynamicForm item on which the error occurred (may be null)\n  <li>validator - a pointer to the validator object\n  <li>value - the value of the field in question</ul>\n  When validation is perfomed, the expression will be evaluated (or executed) - if it\n  returns <code>true</code>, the field will be treated as a required field, so validation \n  will fail if the field has no value.",
        name:"requiredIf",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        examples:["${isc.DocUtils.linkForExampleId('conditionallyRequired')}"],
        definingClass:"class:ValidatorTypes"
    },
    "classMethod:XMLTools.selectNodes":{
        ref:"classMethod:XMLTools.selectNodes",
        flags:"",
        description:" Retrieve a set of nodes from an XML element or document based on an XPath expression.\n <P>\n If the target document is namespaced, namespace prefixes declared in the document element of\n the target document will be available, as well as the default namespace, if declared, under\n the prefix \"default\".\n <P>\n To declare your own namespace prefixes, provide a prefix to URI mapping as a simple JS\n Object, for example:\n <pre>\n   {\n      az : \"http://webservices.amazon.com/AWSECommerceService/2005-03-23\",\n      xsd : \"http://www.w3.org/2001/XMLSchema\"\n   }\n </pre>\n <P>\n <b>NOTE:</b> this API cannot be supported on the Safari web browser for versions earlier\n than 3.0.3.\n\n",
        name:"selectNodes",
        seeAlso:[],
        params:[
            {description:"Native XMLElement,document, or xml string                                                        to select from",
             optional:false, name:"element", defaultValue:null,
             type:"XMLElement or XMLDocument or String"},
            {description:"XPath expression to use to select nodes", optional:false, name:"expression",
             defaultValue:null, type:"XPath"},
            {description:"namespace mapping used by the expression", optional:true, name:"namespaces",
             defaultValue:null, type:"prefix -> URI mapping"}
        ],
        returns:{description:"list of nodes matching XPath", type:"Array"},
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('xmlServerValidationErrors')}"],
        definingClass:"class:XMLTools",
        groups:["xmlTransform"]
    },
    "method:ColorPicker.setGreen":{
        ref:"method:ColorPicker.setGreen",
        flags:"",
        description:" Sets the Green element of the selected color\n",
        name:"setGreen",
        seeAlso:[
            {ref:"method:ColorPicker.getGreen", description:null}
        ],
        params:[
            {description:"An integer between 0 and 255", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:ListGrid.headerButtonDefaults":{
        ref:"attr:ListGrid.headerButtonDefaults",
        flags:"IRA",
        description:"Defaults to apply to all header buttons. To modify this object, \n use ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','ListGrid.changeDefaults()')} \n rather than replacing with an entirely new object.",
        name:"headerButtonDefaults",
        seeAlso:[],
        valueType:"Button Properties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:WSRequest.headerData":{
        ref:"attr:WSRequest.headerData",
        flags:"IR",
        description:"Data to be serialized to form the SOAP headers, as a map from the header part name to the\n data.  For example, given WSDL like this:\n <pre>\n     &lt;soap:header part=\"SessionHeader\" message=\"tns:HeaderMessage\"/&gt;\n     &lt;soap:header part=\"CallOptions\" message=\"tns:HeaderMessage/&gt;\n </pre>\n <code>headerData</code> like this might be provided:\n <pre>\n     dsRequest.headerData = \n         { SessionHeader : <i>data</i>\n           CallOptions : <i>data</i> };\n </pre>\n The provided data will be serialized to XML by the\n ${isc.DocUtils.linkForRef('method:WebService.getInputHeaderSchema','SOAP header schema')} via\n ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}",
        name:"headerData",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WSRequest"
    },
    "method:ListGrid.getSortNumeralHTML":{
        ref:"method:ListGrid.getSortNumeralHTML",
        flags:"",
        description:" When multiple fields are sorted, this method returns the HTML for the sort-numeral that \n appears after the sort-arrows in the header-buttons of sorted fields.  If you don't want\n sort-numerals in the header-buttons, you can override this\n method to return null or an empty string.\n",
        name:"getSortNumeralHTML",
        seeAlso:[],
        params:[
            {description:"The name of a sort-field to get the   ${isc.DocUtils.linkForRef('attr:ListGrid.sortNumeralStyle','sortNumeral')} HTML for.",
             optional:false, name:"fieldName", defaultValue:null, type:"String"}
        ],
        returns:{description:"The HTML for this field's sortNumeral", type:"String"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Operator.fieldTypes":{
        ref:"attr:Operator.fieldTypes",
        flags:"IR",
        description:"List of types that this Operator is valid for.\n <P>\n If omitted, the operator is assumed to be valid for all FieldTypes unless a list of \n FieldTypes is passed to ${isc.DocUtils.linkForRef('method:DataSource.addSearchOperator')}.",
        name:"fieldTypes",
        seeAlso:[],
        valueType:"Array of FieldType",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "attr:ImgProperties.height":{
        ref:"attr:ImgProperties.height",
        flags:"IRW",
        description:"Specifies the height of the image.",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "method:CubeGrid.selectFacetValues":{
        ref:"method:CubeGrid.selectFacetValues",
        flags:"",
        description:"\n select/deselect the header showing a given set of facet values\n\n  <br><i>methodType</i> action\n",
        name:"selectFacetValues",
        seeAlso:[],
        params:[
            {description:"facetValues to select the header of", optional:false, name:"facetValues",
             defaultValue:null, type:"facetValues object"},
            {description:"new selection state - if null defaults to true", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:FormItem.pickerIconDefaults":{
        ref:"attr:FormItem.pickerIconDefaults",
        flags:"IRWA",
        description:"Block of default properties to apply to the pickerIcon for this widget.\n Intended for class-level customization: To modifying this value we recommend using\n ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')} rather than directly assigning a value to the property.",
        name:"pickerIconDefaults",
        seeAlso:[],
        valueType:"FormItemIcon Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:DetailViewer.styleName":{
        ref:"attr:DetailViewer.styleName",
        flags:"IRW",
        description:"CSS style for the component as a whole.",
        name:"styleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detailViewer\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:Class.getClassName":{
        ref:"method:Class.getClassName",
        flags:"",
        description:"	\n	Gets the name of this class as a string.\n\n",
        name:"getClassName",
        seeAlso:[],
        returns:{description:"String name of this instance's Class object.", type:"string"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:MenuButton.hiliteAccessKey":{
        ref:"attr:MenuButton.hiliteAccessKey",
        flags:"IR",
        description:"If this MenuButton has a specified ${isc.DocUtils.linkForRef('attr:Canvas.accessKey','accessKey')}, underline it\n in the title of the button by default",
        name:"hiliteAccessKey",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "attr:ListGrid.clearAllSortingText":{
        ref:"attr:ListGrid.clearAllSortingText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, this\n attribute will be shown as the menu item title to clear any existing sort on all fields.  This\n menu-item is displayed only in the context menu for the sorter button.",
        name:"clearAllSortingText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Clear All Sorting\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:ColorItem.pickerIconWidth":{
        ref:"attr:ColorItem.pickerIconWidth",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"18",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "method:Canvas.imgHTML":{
        ref:"method:Canvas.imgHTML",
        flags:"A",
        description:" Generates the HTML for an image unique to this Canvas.\n <P>\n The full URL for the image will be formed according to the rules documented for\n <code>${isc.DocUtils.linkForRef('method:Canvas.getImgURL')}</code>.\n <P>\n The created image will have an identifier unique to this Canvas, and subsequent calls to\n <code>${isc.DocUtils.linkForRef('method:Canvas.getImage')}</code> and\n <code>${isc.DocUtils.linkForRef('method:Canvas.setImage')}</code> \n with the name passed to this function will act on the image object produced by the HTML\n returned from this call.\n\n",
        name:"imgHTML",
        seeAlso:[],
        params:[
            {description:"URL local to the skin or application directory.<br>		NOTE: instead of passing several parameters, you can pass an object as the 'src'\n      parameter with properties for all the various function parameters with, eg:<br>\n      canvas.imgHTML( {src:\"foo\", width:10, height:10} );",
             optional:false, name:"src", defaultValue:null, type:"SCImgURL"},
            {description:"width of the image", optional:true, name:"width", defaultValue:null,
             type:"number"},
            {description:"height of the image", optional:true, name:"height", defaultValue:null,
             type:"number"},
            {description:"name for the image", optional:true, name:"name", defaultValue:null,
             type:"string"},
            {description:"additional attributes to write in the tag", optional:true, name:"extraStuff",
             defaultValue:null, type:"string"},
            {description:"image-specific image directory to override the default                                  for this Canvas",
             optional:true, name:"imgDir", defaultValue:null, type:"string"}
        ],
        returns:{description:"HTML to draw the image.", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "method:ListGrid.getFilterEditorType":{
        ref:"method:ListGrid.getFilterEditorType",
        flags:"A",
        description:"\n  If we're showing the filter (query-by-example) row for this ListGrid, this method is\n  used to determine the type of form item to display in the filter edit row for this field.\n  Default implementation will return the field.filterEditorType if specified, or\n  field.valueMap.\n\n",
        name:"getFilterEditorType",
        seeAlso:[],
        params:[
            {description:"field definition field for which we need a valueMap", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"ValueMap for the edit field (or null if no valueMap required)", type:"ValueMap"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "attr:DSRequest.dataSource":{
        ref:"attr:DSRequest.dataSource",
        flags:"IR",
        description:"DataSource this DSRequest will act on.\n <P>\n This property is generally automatically populated, for example when calling\n ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} the dataSource property is set to the target\n DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "group:thumb":{
        ref:"group:thumb",
        refs:["attr:Scrollbar.allowThumbDownState", "attr:Scrollbar.allowThumbOverState", 
         "attr:Scrollbar.thumbMinSize","attr:Scrollbar.thumbOverlap", "attr:Scrollbar.startThumbOverlap", 
         "attr:Scrollbar.endThumbOverlap","attr:Scrollbar.thumbInset"],
        name:"thumb",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.getScrollLeft":{
        ref:"method:Canvas.getScrollLeft",
        flags:"A",
        description:" Get the number of pixels this Canvas is scrolled from its left edge.\n",
        name:"getScrollLeft",
        seeAlso:[],
        returns:{description:"scrollLeft", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "scrolling"]
    },
    "method:List.findIndex":{
        ref:"method:List.findIndex",
        flags:"",
        description:" Find the index of the first Object where property == value in the object.\n <P>\n Pass an Object instead to match multiple properties.\n <P>\n Note: for string values, matches are case sensitive.\n\n",
        name:"findIndex",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"interface:List",
        groups:["access", "find"]
    },
    "classMethod:isc.addDefaults":{
        ref:"classMethod:isc.addDefaults",
        flags:"",
        description:"\n Copy any properties that do not already have a value in destination.  Null and zero values\n are not overwritten, but 'undef' values will be.\n\n",
        name:"addDefaults",
        seeAlso:[],
        params:[
            {description:"Object to which properties will be added.", optional:false,
             name:"destination", defaultValue:null, type:"Object"},
            {description:"Object from which properties will be added.", optional:false, name:"source",
             defaultValue:null, type:"Object"}
        ],
        returns:{description:"The destination object is returned.", type:"Object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "group:dragDrop":{
        ref:"group:dragDrop",
        refs:["classMethod:EventHandler.setDragTracker"],
        name:"dragDrop",
        seeAlso:[],
        type:"group"
    },
    "attr:DynamicForm.colWidths":{
        ref:"attr:DynamicForm.colWidths",
        flags:"IRW",
        description:"An array of widths for the columns of items in this form's layout grid. \n <P>\n If specified, these widths should sum to the total width of the form (form.width).\n If not specified, we assume every other column will contain form item titles, and so\n should have <code>form.titleWidth</code>, and all other columns should share the\n remaining space.\n <P>\n Acceptable values for each element in the array are:<br>\n <ul>\n <li>A number (e.g. 100) representing the number of pixel widths to allocate to a\n     column.\n <li>A percent (e.g. 20%) representing the percentage of the total form.width to\n     allocate to a column.\n <li>\"*\" (all) to allocate remaining width (form.width minus all specified column\n     widths). Multiple columns can use \"*\", in which case remaining width is divided\n     between all columns marked \"*\".\n </ul>",
        name:"colWidths",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('columnSpanning')}"],
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "class:HLayout":{
        methods:[],
        exampleConfig:"HLayout.xml",
        ref:"class:HLayout",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:[],
        description:"A subclass of Layout that applies a sizing policy along the horizontal axis, interpreting\n  percent and \"*\" sizes as proportions of the width of the layout. HLayouts will set any members\n  that do not have explict heights to match the layout.",
        name:"HLayout",
        seeAlso:[
            {ref:"attr:Layout.hPolicy", description:null}
        ],
        type:"class"
    },
    "attr:Canvas.dynamicContentsVars":{
        ref:"attr:Canvas.dynamicContentsVars",
        flags:"IRWA",
        description:"An optional map of name:value parameters that will be evailable within the scope of the\n dynamicContents evaluation.  For example - if you have e.g:\n <pre>\n Canvas.create({\n   dynamicContents: true,\n   dynamicContentsVars: {\n       name: \"Bob\"\n   },\n   contents: \"hello \\${name}\"\n });\n </pre>\n The above will create a canvas with contents <code>hello Bob</code>.  You can add, remove, and\n change values in the dynamicContentsVars object literal, just call\n <code>markForRedraw()</code> on the canvas to have the dynamicContents template re-evaluated.\n <p>\n Note that <code>this</code> is always evailable inside a dynamic contents string and points to\n the canvas instance containing the dynamic contents.\n <p>\n Used only if ${isc.DocUtils.linkForRef('attr:Canvas.dynamicContents')} : true has been set.",
        name:"dynamicContentsVars",
        seeAlso:[
            {ref:"attr:Canvas.dynamicContents", description:null}
        ],
        valueType:"ValueMap",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "method:CubeGrid.facetLabelHover":{
        ref:"method:CubeGrid.facetLabelHover",
        flags:"",
        description:" StringMethod handler fired from hover over a facet label\n\n",
        name:"facetLabelHover",
        seeAlso:[],
        params:[
            {description:"ID of the appropriate facet", optional:false, name:"facetId",
             defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "attr:EdgedCanvas.addEdgeStyleSuffix":{
        ref:"attr:EdgedCanvas.addEdgeStyleSuffix",
        flags:"IRW",
        description:"If specified, the ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeStyleName')} will be treated as a base style name and\n appended with following suffixes to support separate styling per cell:\n <P>\n <code>_TL</code> (top left cell)<br>\n <code>_T</code> (top center cell)<br>\n <code>_TR</code> (top right cell)<br>\n <code>_L</code> (middle left cell)<br>\n <code>_C</code> (center cell)<br>\n <code>_R</code> (middle right cell)<br>\n <code>_BL</code> (bottom left cell)<br>\n <code>_B</code> (bottom center cell)<br>\n <code>_BR</code> (bottom right cell)",
        name:"addEdgeStyleSuffix",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["im ageEdgeStyles", "imageEdges"]
    },
    "group:autoFitData":{
        ref:"group:autoFitData",
        refs:["method:ListGrid.setAutoFitData", "method:ListGrid.setAutoFitExtraRecords", 
         "method:ListGrid.setAutoFitMaxRecords","method:ListGrid.setAutoFitMaxHeight", "method:ListGrid.setAutoFitMaxColumns", 
         "method:ListGrid.setAutoFitMaxWidth","attr:ListGrid.autoFitData", "attr:ListGrid.autoFitMaxHeight", 
         "attr:ListGrid.autoFitMaxRecords","attr:ListGrid.autoFitExtraRecords", "attr:ListGrid.autoFitMaxWidth", 
         "attr:ListGrid.autoFitMaxColumns"],
        name:"autoFitData",
        seeAlso:[],
        type:"group"
    },
    "attr:GridRenderer.autoFit":{
        ref:"attr:GridRenderer.autoFit",
        flags:"IRWA",
        description:"If true, make columns only wide enough to fit content, ignoring any widths specified.\n Overrides fixedFieldWidths.",
        name:"autoFit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["sizing"]
    },
    "attr:ComboBoxItem.showOptionsFromDataSource":{
        ref:"attr:ComboBoxItem.showOptionsFromDataSource",
        flags:"IRWA",
        description:"If this item is part of a databound form, and has a specified <code>valueMap</code>,\n by default we show the valueMap options in the pickList for the item.\n Setting this property to true will ensure that the options displayed in our pickList\n are derived from the form's <code>dataSource</code>.",
        name:"showOptionsFromDataSource",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem",
        groups:["databinding"]
    },
    "method:List.getRange":{
        ref:"method:List.getRange",
        flags:"",
        description:" Return the items between position start and end, non-inclusive at the end.\n\n",
        name:"getRange",
        seeAlso:[],
        params:[
            {description:"start position", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end position", optional:false, name:"end", defaultValue:null, type:"number"}
        ],
        returns:{description:"subset of the array from start -> end-1", type:"Array"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:ListGrid.expansionCanEdit":{
        ref:"attr:ListGrid.expansionCanEdit",
        flags:"IRWA",
        description:"For ${isc.DocUtils.linkForRef('type:ExpansionMode','expansionModes')} that show another grid or tree, is that \n component editable?\n <P>The default value for this property is <code>false</code>.",
        name:"expansionCanEdit",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:FusionChart.chartURL":{
        ref:"attr:FusionChart.chartURL",
        flags:"IR",
        description:"Full URL to the chart.  Needed only if you have renamed the charts such that automatic\n URL formation won't work.",
        name:"chartURL",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FusionChart"
    },
    "method:HibernateBrowser.getGeneratedDataSource":{
        ref:"method:HibernateBrowser.getGeneratedDataSource",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('class:DataSource')} most recently auto-derived by this HibernateBrowser.  This will\n correspond to the currently-selected class mapping.\n",
        name:"getGeneratedDataSource",
        seeAlso:[],
        type:"method",
        definingClass:"class:HibernateBrowser"
    },
    "attr:MultiSortPanel.levelDownButton":{
        ref:"attr:MultiSortPanel.levelDownButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ImgButton')} providing a mechanism for moving existing\n sort-levels down in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.levelDownButtonProperties</code> and \n <code>multiSortPanel.levelDownButtonDefaults</code>.",
        name:"levelDownButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "method:ListGrid.startEditing":{
        ref:"method:ListGrid.startEditing",
        flags:"A",
        description:" Start inline editing at the provided coordinates.\n <p>\n Invoked when a cell is editable and the <code>editEvent</code> occurs on that cell.  Can\n also be invoked explicitly.\n <P>\n If this method is called while editing is already in progress, the value from the current\n editCell will either be stored locally as a temporary edit value, or saved via 'saveEdits()'\n depending on <code>this.saveByCell</code>, and the position of the new edit cell.<br>\n Will update the UI to show the editor for the new cell, and put focus in it unless \n explicitly suppressed by the optional <code>suppressFocus</code> parameter.\n\n",
        name:"startEditing",
        seeAlso:[
            {ref:"method:ListGrid.canEditCell", description:null},
            {ref:"attr:ListGrid.editEvent", description:null}
        ],
        params:[
            {description:"Row number of the cell to edit.  Defaults to first                                  editable row",
             optional:true, name:"rowNum", defaultValue:null, type:"number"},
            {description:"Column number of the cell to edit.  Defaults to first                                  editable column",
             optional:true, name:"colNum", defaultValue:null, type:"number"},
            {description:"If passed this parameter suppresses the default                                   behavior of focussing in the edit form item when \n                                  the editor is shown.",
             optional:true, name:"suppressFocus", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if we are editing the cell, false if not editing for some reason",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "object:DetailViewerRecord":{
        methods:[],
        ref:"object:DetailViewerRecord",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/DetailViewer"],
        attrs:[],
        description:"A DetailViewerRecord is an object literal with properties that define the values for the\n various fields of a ${isc.DocUtils.linkForRef('class:DetailViewer')}.\n <p>\n For example a DetailViewer that defines the following fields:\n <pre>\n fields : [\n     {name: \"field1\"},\n     {name: \"field2\"}\n ],\n </pre>\n Might have the following data:\n <pre>\n data : [\n     {field1: \"foo\", field2: \"bar\"},\n     {field1: \"field1 value\", field2: \"field2 value\"}\n ]\n </pre>\n Each element in the data array above is an instance of DetailViewerRecord - notice that\n these are specified simply as object literals with properties.",
        name:"DetailViewerRecord",
        seeAlso:[],
        type:"object"
    },
    "attr:ImgSectionHeader.controlsLayout":{
        ref:"attr:ImgSectionHeader.controlsLayout",
        flags:"IR",
        description:"A ${isc.DocUtils.linkForRef('class:Layout')} containing specified ${isc.DocUtils.linkForRef('attr:ImgSectionHeader.controls')} if any.  Sets\n ${isc.DocUtils.linkForRef('attr:Layout.membersMargin')}:5, ${isc.DocUtils.linkForRef('attr:Layout.defaultLayoutAlign')}:\"center\", and\n RTL-sensitive ${isc.DocUtils.linkForRef('attr:Layout.align')} (right by default).",
        name:"controlsLayout",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgSectionHeader"
    },
    "classAttr:Browser.isSupported":{
        ref:"classAttr:Browser.isSupported",
        flags:"R",
        description:"Whether SmartClient supports the current browser.\n <P>\n Note that this flag will only be available on browsers that at least support basic\n JavaScript.",
        name:"isSupported",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"varies by browser",
        type:"classAttr",
        definingClass:"object:Browser"
    },
    "method:TileLayout.setTileSize":{
        ref:"method:TileLayout.setTileSize",
        flags:"",
        description:" sets the height and width of tiles\n",
        name:"setTileSize",
        seeAlso:[],
        params:[
            {description:"size", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "attr:ListGridField.cellIcon":{
        ref:"attr:ListGridField.cellIcon",
        flags:"IR",
        description:"For a field of type:\"icon\" only, set the icon that appears in body cells.  Unless\n setting ${isc.DocUtils.linkForRef('attr:ListGridField.icon','field.icon')}, setting field.cellIcon will not show an\n icon in the header.",
        name:"cellIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:DynamicForm.formSubmitFailed":{
        ref:"method:DynamicForm.formSubmitFailed",
        flags:"A",
        description:" Method called when an attempt to ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm','natively submit')} a\n form is unable to submit to the server. Default behavior is to display the\n ${isc.DocUtils.linkForRef('attr:DynamicForm.formSubmitFailedWarning')} in a warning dialog.\n The most common cause for this failure is that the user\n has typed an invalid file-path into an upload type field.\n",
        name:"formSubmitFailed",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["i18nMessages"]
    },
    "attr:Tree.openProperty":{
        ref:"attr:Tree.openProperty",
        flags:"IRWA",
        description:"The property consulted by the default implementation of ${isc.DocUtils.linkForRef('method:Tree.isOpen')} to determine if the\n node is open or not.  By default, this property is auto-generated for you, but you can set\n it to a custom value if you want to declaratively specify this state, but be careful - if\n you display this Tree in multiple TreeGrids at the same time, the open state will not be\n tracked independently - see ${isc.DocUtils.linkForRef('group:sharingNodes')} for more info on this.",
        name:"openProperty",
        seeAlso:[
            {ref:"group:sharingNodes", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('initialData')}"],
        definingClass:"class:Tree",
        groups:["openList"]
    },
    "method:TileGrid.getFieldState":{
        ref:"method:TileGrid.getFieldState",
        flags:"",
        description:" Returns a snapshot of the current presentation of this grid's fields as \n a ${isc.DocUtils.linkForRef('type:listGridFieldState')} object.\n <P>\n This object can be passed to ${isc.DocUtils.linkForRef('method:TileGrid.setFieldState')} to reset this grid's fields to\n the current state.\n <P>\n Note that the information stored includes the current width and visibility of each of this \n grid's fields.\n\n",
        name:"getFieldState",
        seeAlso:[
            {ref:"method:TileGrid.setFieldState", description:null}
        ],
        returns:{description:"current state of this grid's fields.", type:"listGridFieldState"},
        type:"method",
        definingClass:"class:TileGrid",
        groups:["viewState"]
    },
    "method:ValuesManager.getErrors":{
        ref:"method:ValuesManager.getErrors",
        flags:"",
        description:" Returns the set of errors for this valuesManager.\n Errors will be returned as an object of the format <br>\n <code>{field1:errors, field2:errors}</code><br>\n Where each errors object is either a single error message or an array of error message\n strings.\n",
        name:"getErrors",
        seeAlso:[],
        returns:{description:"Object contining mapping from field names to error strings. Returns null                  if there are no errors for this valuesManager.",
         type:"object"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:ListGridField.iconSpacing":{
        ref:"attr:ListGridField.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "method:TileGrid.selectionChanged":{
        ref:"method:TileGrid.selectionChanged",
        flags:"A",
        description:" Called when selection changes within this tileGrid. Note this method fires for\n each record for which selection is modified - so when a user clicks inside a tileGrid this\n method will typically fire twice (once for the old record being deselected, and once for\n the new record being selected).\n\n",
        name:"selectionChanged",
        seeAlso:[],
        params:[
            {description:"record for which selection changed", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"New selection state (true for selected, false for unselected)",
             optional:false, name:"state", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "class:EventHandler":{
        methods:[],
        ref:"class:EventHandler",
        classAttrs:[],
        classMethods:["classMethod:EventHandler.getWheelDelta", "classMethod:EventHandler.setDragTracker", 
         "classMethod:EventHandler.getDragRect","classMethod:EventHandler.getTarget", "classMethod:EventHandler.getDragTarget", 
         "classMethod:EventHandler.getX","classMethod:EventHandler.getY", "classMethod:EventHandler.leftButtonDown", 
         "classMethod:EventHandler.rightButtonDown","classMethod:EventHandler.getKeyEventCharacterValue", 
         "classMethod:EventHandler.getKeyEventCharacter","classMethod:EventHandler.getKey", "classMethod:EventHandler.shiftKeyDown", 
         "classMethod:EventHandler.ctrlKeyDown","classMethod:EventHandler.altKeyDown"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"The ISC system provides a predictable cross-browser event-handling mechanism for ISC\n widgets. Events can be handled both at the page level (i.e., globally), and at the level of\n individual widgets.\n <p>\n With the exception of a few page-specific events ('load', 'unload', 'idle' and 'resize'),\n events are processed in the following sequence:\n <p>\n 1. The event is sent to any global (page-level) event handlers. These handlers can cancel\n further propagation of the event by returning false.  You can register to listen for any of the\n events linked in the seeAlso section (below) by calling ${isc.DocUtils.linkForRef('classMethod:Page.setEvent')}\n method.\n <p>\n 2. If the event occurred on a form element or a link, it is passed on to the browser so that\n the element will perform its default action. No widget receives the event.\n <p>\n 3. If the event occurred on an enabled widget (but not on a form element or link inside\n the widget), it is sent to that widget's event handler, if any. This handler can cancel\n further propagation of the event by returning false.  An \"enabled\" widget is any widget that\n defines an event handler for one of the supported events.  Interceptable events are defined in\n the ${isc.DocUtils.linkForRef('class:Canvas#methods#events','\"widgetEvents\" section of Canvas')}.\n <p>\n 4. The event is \"bubbled\" up to the widget's parent in the containment hierarchy, if any.\n Again, the parent's handler for the event can cancel further propagation by returning\n false. This step is repeated, with the event \"bubbling\" up through the containment\n hierarchy, until a top-level widget is reached or the event is explicitly canceled.\n In brief, the ISC event model offers the best features of browser event models:\n <ul>\n <li> Page-first event handling allows you to reliably process or cancel any event before it\n      affects the objects on the page.\n <li> Event \"bubbling\" ensures that parent widgets receive events sent to their children,\n      and allows you to create generalized parent-level handlers rather than duplicating\n      code in each child.\n </ul>\n Note: Canceling propagation of an event may cancel its side effects as well, including the\n generation of other (synthetic) events. For example, if a global mouseDown handler returns\n false, drag-and-drop events will not be generated. Specific effects are discussed in the\n descriptions of the various events in the following sections.\n <p>\n SmartClient libraries will not interfere with native event handling when events occur\n outside of a target widget. You can therefore have HTML that is not ISC-based on the same\n page as widget objects that will react to native events as you would expect.\n <p>\n You can use isc.Event as an alias for isc.EventHandler.",
        name:"EventHandler",
        seeAlso:[
            {ref:"type:PageEvent", description:null},
            {ref:"classMethod:Page.setEvent", description:null},
            {ref:"classMethod:Page.clearEvent", description:null},
            {ref:"class:Canvas#methods#widgetEvents", description:null}
        ],
        type:"class"
    },
    "attr:ListGrid.nullGroupTitle":{
        ref:"attr:ListGrid.nullGroupTitle",
        flags:"IRW",
        description:"Default alias to use for groups with no value",
        name:"nullGroupTitle",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"String",
        defaultValue:"'-none-'",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:ImgTab.titleStyle":{
        ref:"attr:ImgTab.titleStyle",
        flags:"IR",
        description:"Like ${isc.DocUtils.linkForRef('attr:StretchImgButton.titleStyle')}, can set to provide a separate style for the\n title text.\n <P>\n If set and the ImgTab is ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical','vertical')}, a \"v\" will be\n automatically prepended to the style name (hence \"tabTitle\" -> \"vtabTitle\").",
        name:"titleStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "group:upload":{
        ref:"group:upload",
        title:"Uploading Files",
        treeLocation:["Client Reference/Forms/Form Items/FileItem"],
        refs:["class:UploadItem", "class:FileItem", "class:MultiFileItem", "class:MultiFilePicker", 
         "class:ViewFileItem"],
        description:"SmartClient provides special client and server-side support for file upload that allows\n uploaded files to be treated like ordinary DataSource fields.  This includes:\n <ul>\n <li>the ${isc.DocUtils.linkForRef('class:FileItem')} and ${isc.DocUtils.linkForRef('class:MultiFileItem')} FormItems that enable users to upload one or\n more files as a background operation, without leaving the current page\n <li>server-side support that allows binary uploads to be treated as a normal DataSource field\n values, with all other aspects of server-side integration unchanged\n <li>built-in SQL DataSource support that can store and retrieve uploaded files from SQL\n databases\n </ul>\n The following documentation assumes you are using the SmartClient Java Server.  If you are\n not, skip to the sections near the end of this document.\n <P>\n <b>Single file upload: \"binary\" field type</b>\n <P>\n To use SmartClient's client-server upload system, you use a DataSource field of\n ${isc.DocUtils.linkForRef('attr:DataSourceField.type','type')} \"binary\".  By default, a DynamicForm bound to a DataSource\n with a field of type \"binary\" will use the ${isc.DocUtils.linkForRef('class:FileItem')}, which displays a standard HTML\n &lt;input type=\"upload\"&gt; form control.\n <P>\n When you call ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} on a DynamicForm containing a FileItem,\n SmartClient processes the save identically to a saveData() call that did not include a file\n upload:\n <ul>\n <li> if you are using the built-in SQL connectors via serverType:\"sql\", the file will be\n saved to SQL as described under ${isc.DocUtils.linkForRef('type:FieldType','field type \"binary\"')}.\n <li> if you have server-side business logic, the inbound request may be routed to your\n business logic via RPCManager dispatch or DMI declarations as normal, your business logic\n will receive a normal DSRequest, and you are expected to provide a normal DSResponse.\n </ul>\n <P>\n Client-side callbacks, such as the callback passed to saveData(), fire normally.\n <P>\n <b>Processing File Uploads with server-side business logic</b>\n <P>\n Server-side business logic that processes file uploads may retrieve upload files via the\n server side API dsRequest.getUploadedFile(<i>fieldName</i>).  The uploaded file is returned\n as an instance of ISCFileItem, which provides access to a Java InputStream as well as\n metadata about the file (size, name).  \n See the ${isc.DocUtils.linkForDocNode('javaServerReference', 'Java Server Reference')} for details.\n <P>\n <span style=\"color:red;font-weight:bold;\">NOTE:</span> request processing engines such as\n Struts may parse the inbound request before SmartClient receives it.  If you are creating an\n RPCManager object inside of a Struts Action and the file being uploaded is not available via\n <code>dsRequest.getUploadedFile()</code>, this is likely to be the problem, and you should\n use SmartClient ${isc.DocUtils.linkForRef('class:DMI')} instead.\n <P>\n Server-side validation errors may be provided, including validation errors for the uploaded\n file (such as too large or invalid content), and will be displayed in the form that\n attempted an upload.\n <P>\n Be aware of the following special concerns when processing file uploads:\n <ul>\n <li> if you provide your own Java Servlet or JSP that creates an instance of RPCManager in\n order process SmartClient requests, many APIs of the HttpServletRequest are not safe to call\n before you have created the RPCManager, passing in the HttpServletRequest.  These include\n getReader(), getParameter() and other commonly called methods.  This is a limitation of\n Java Servlets, not specific to SmartClient\n <li> unlike other DataSource \"add\" and \"update\" operations, you are not expected to return\n the file as part of the data returned in the DSResponse\n </ul>\n <P>\n <b>Multi file upload: MultiFileItem</b>\n <P>\n The MultiFileItem provides an interface for a user to save one or more files that are\n related to a DataSource record, where each file is represented by a record in a\n related DataSource.\n <P>\n See the ${isc.DocUtils.linkForRef('class:MultiFileItem')} docs for details.\n <P>\n <b>Upload without the SmartClient Server</b>\n <P>\n If it is acceptable that the application will do a full-page reload after the upload\n completes, you can simply:\n <ul>\n <li> set ${isc.DocUtils.linkForRef('attr:DynamicForm.encoding')} to \"multipart\"\n <li> include an ${isc.DocUtils.linkForRef('class:UploadItem')} to get a basic HTML upload control\n <li> set ${isc.DocUtils.linkForRef('attr:DynamicForm.action')} to a URL where you have deployed server-side code to\n handle the upload\n <li> call ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm')} to cause the form to be submitted\n </ul>\n This cause the DynamicForm component to submit to the form.action URL like an ordinary HTML\n &lt;form&gt; element.  Many \n <a href='http://www.google.com/search?q=html+file+upload+example' onclick=\"window.open('http://www.google.com/search?q=html+file+upload+example');return false;\">online tutorials</a>\n are available which explain how to handle HTML form file upload in various server-side\n technologies.\n <P>\n Note that when you submitForm(), the only values that will be sent to your actionURL are \n values for which actual FormItems exist.  This differs from saveData(), in which the\n entire set of ${isc.DocUtils.linkForRef('attr:DynamicForm.values','form values')} are always sent.  To handle submitting\n extra values, use ${isc.DocUtils.linkForRef('class:HiddenItem')}s.\n <P>\n For further details, see the ${isc.DocUtils.linkForRef('class:UploadItem')} docs.\n <P>\n <b>Background upload without the SmartClient Server</b>\n <P>\n Achieving background file upload without using the SmartClient server is also possible\n although considerably more advanced.  In addition to the steps above, create a hidden\n &lt;iframe&gt; element in the page, and use ${isc.DocUtils.linkForRef('attr:DynamicForm.target')} to target the form\n submission at this IFRAME.  In order receive a callback notification when the upload\n completes, after processing the file upload, your server should output HTML content for the\n IFRAME that includes a &lt;SCRIPT&gt; block which will navigate out of the IFRAME (generally\n via the JavaScript global \"top\") and call a global method you have declared as a callback.",
        name:"upload",
        seeAlso:[],
        type:"group"
    },
    "method:CubeGrid.closeColumn":{
        ref:"method:CubeGrid.closeColumn",
        flags:"",
        description:"\n  Handler fired when column is closed\n\n      <br><i>methodType</i> handler\n",
        name:"closeColumn",
        seeAlso:[],
        params:[
            {description:"FacetValues for the appropriate col.", optional:false,
             name:"headerFacetValues", defaultValue:null, type:"facetValues object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["columnControls"]
    },
    "attr:FormItem.globalTabIndex":{
        ref:"attr:FormItem.globalTabIndex",
        flags:"IRWA",
        description:"TabIndex for the form item within the page. Takes presidence over any local tab index\n specified as ${isc.DocUtils.linkForRef('attr:FormItem.tabIndex','item.tabIndex')}.",
        name:"globalTabIndex",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["focus"]
    },
    "type:SelectionStyle":{
        ref:"type:SelectionStyle",
        values:[
            {description:"don't select at all", value:"Selection.NONE"},
            {description:"select only one item at a time", value:"Selection.SINGLE"},
            {description:"select one or more items", value:"Selection.MULTIPLE"},
            {description:"select one or more items as a toggle 								  so you don't need to hold down control keys to select \n                                  more than one object",
             value:"Selection.SIMPLE"}
        ],
        description:"Different styles of selection that a list, etc. might support",
        name:"SelectionStyle",
        seeAlso:[],
        type:"type",
        groups:["selection"]
    },
    "method:Date.toJapanShortDateTime":{
        ref:"method:Date.toJapanShortDateTime",
        flags:"",
        description:"			Return this date in the format: <code>YYYY/MM/DD HH:MM:SS</code>\n",
        name:"toJapanShortDateTime",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:ResultSet.dataArrived":{
        ref:"method:ResultSet.dataArrived",
        flags:"",
        description:" Notification fired when data has arrived from the server and has been successfully\n integrated into the cache.\n <P>\n When <code>dataArrived()</code> fires, an immediate call to <code>getRange()</code> with\n the <code>startRow</code> and <code>endRow</code> passed as arguments will return a List\n with no <code>Array.LOADING</code> markers.\n\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"starting index of rows that have just loaded", optional:false,
             name:"startRow", defaultValue:null, type:"number"},
            {description:"ending index of rows that have just loaded, non-inclusive", optional:false,
             name:"endRow", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:CheckboxItem.titleStyle":{
        ref:"attr:CheckboxItem.titleStyle",
        flags:"IRW",
        description:"Base CSS class for this item's title cell.\n <P>\n <B>Note:</B> This styling applies to the standard form item title cell for this item - it\n does not apply to item's ${isc.DocUtils.linkForRef('attr:CheckboxItem.showLabel','label')}.\n To modify the styling for that text, use ${isc.DocUtils.linkForRef('attr:CheckboxItem.textBoxStyle')} instead.",
        name:"titleStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["appearance"]
    },
    "attr:MultiSortPanel.firstSortLevelTitle":{
        ref:"attr:MultiSortPanel.firstSortLevelTitle",
        flags:"IR",
        description:"The title-text to appear in the first column for the first sort-level.",
        name:"firstSortLevelTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Sort by\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "attr:DataSource.jsonSuffix":{
        ref:"attr:DataSource.jsonSuffix",
        flags:"IRA",
        description:"Allows you to specify an arbitrary suffix string to apply to all json format responses \n sent from the server to this application.<br>\n The inclusion of such a suffix ensures your code is not directly executable outside of\n your application, as a preventitive measure against\n <a href='http://www.google.com/search?q=javascript+hijacking' onclick=\"window.open('http://www.google.com/search?q=javascript+hijacking');return false;\">javascript hijacking</a>.\n <br>\n Only applies to responses formatted as json objects. Does not apply to responses returned\n via scriptInclude type transport.",
        name:"jsonSuffix",
        seeAlso:[
            {ref:"attr:OperationBinding.dataFormat", description:null},
            {ref:"attr:OperationBinding.dataTransport", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:Window.removeItem":{
        ref:"method:Window.removeItem",
        flags:"A",
        description:"			Removes a widget from the window.\n",
        name:"removeItem",
        seeAlso:[],
        params:[
            {description:"the widget to be removed", optional:false, name:"item", defaultValue:null,
             type:"Canvas"}
        ],
        returns:{description:"the array of widgets removed", type:"array"},
        type:"method",
        definingClass:"class:Window",
        groups:["windowItems"]
    },
    "attr:StaticTextItem.wrap":{
        ref:"attr:StaticTextItem.wrap",
        flags:"IRW",
        description:"If true, item contents can wrap. If false, all the contents should appear on a single line.",
        name:"wrap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StaticTextItem",
        groups:["appearance"]
    },
    "method:ResultSet.allRowsCached":{
        ref:"method:ResultSet.allRowsCached",
        flags:"A",
        description:" Do we have a complete client-side cache of all records for this DataSource?\n <P>\n Becomes true only when the ResultSet obtains a complete cache after a fetch with empty\n criteria.\n\n",
        name:"allRowsCached",
        seeAlso:[],
        returns:{description:"whether all rows are cached", type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "method:Tree.getDescendants":{
        ref:"method:Tree.getDescendants",
        flags:"",
        description:"\n Returns the list of all descendants of a node.  Note: this method can be very slow,\n especially on large trees because it assembles a list of all descendants recursively.\n Generally, ${isc.DocUtils.linkForRef('method:Tree.find')} in combination with ${isc.DocUtils.linkForRef('method:Tree.getChildren')} will\n be much faster.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')},\n so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} interface\n instead of as a native Javascript Array.\n\n",
        name:"getDescendants",
        seeAlso:[],
        params:[
            {description:"node in question (the root node is assumed if none is specified)",
             optional:true, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"List of descendants of the node.", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:MathFunction.name":{
        ref:"attr:MathFunction.name",
        flags:"IR",
        description:"Name of the function (what the user actually types).  For example, a name of \"min\" would\n indicate that the user types \"min(someValue)\" to use this function.\n <P>\n Limited to lowercase characters only in this release.",
        name:"name",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "method:StretchImgButton.setActionType":{
        ref:"method:StretchImgButton.setActionType",
        flags:"",
        description:" Update the 'actionType' for this canvas (radio / checkbox / button)\n If the canvas is currently selected, and the passed in actionType is 'button'\n this method will deselect the canvas.\n",
        name:"setActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state", "event handling"]
    },
    "classMethod:Page.waitFor":{
        ref:"classMethod:Page.waitFor",
        flags:"",
        description:" Wait for a method to fire on an object.\n <P>\n <code>waitFor</code> is similar ${isc.DocUtils.linkForRef('method:Class.observe','observation')}, but fires once only.\n\n",
        name:"waitFor",
        seeAlso:[],
        params:[
            {description:"any SmartClient object, eg, a ListGrid", optional:false, name:"object",
             defaultValue:null, type:"Object"},
            {description:"name of a method on that object", optional:false, name:"methodName",
             defaultValue:null, type:"String"},
            {description:"Callback to fire when the observed method completes", optional:false,
             name:"callback", defaultValue:null, type:"Function"},
            {description:"Optional timeout period (in milliseconds). If you want a timeout,                           you must also provide a timeoutCallback",
             optional:true, name:"timeout", defaultValue:null, type:"Number"},
            {description:"Callback to fire if the timeout period elapses before the                                      observed method completes",
             optional:true, name:"timeoutCallback", defaultValue:null, type:"Function"}
        ],
        returns:{description:"whether observation succeeded.  Observation may fail due to null object,                   non-existant method or similar bad parameters",
         type:"boolean"},
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:ColorPicker.defaultPickMode":{
        ref:"attr:ColorPicker.defaultPickMode",
        flags:"IR",
        description:"The ColorPicker can operate in either a \"simple\" mode (where it displays just the\n 40 basic colors and allows the user to click one), or a \"complex\" mode (where the\n user can specify a color from anywhere in the spectrum, with an optional alpha \n element). The defaultPickMode attribute specifies which of these two modes is \n in force when the picker first loads.",
        name:"defaultPickMode",
        seeAlso:["ColorPicker.allowComplexMode()"],
        valueType:"text",
        defaultValue:"\"simple\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Selection.selectSingle":{
        ref:"method:Selection.selectSingle",
        flags:"",
        description:" Select a single item and deselect everything else\n",
        name:"selectSingle",
        seeAlso:[],
        params:[
            {description:"object to select", optional:false, name:"item", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:DynamicForm.itemHoverStyle":{
        ref:"attr:DynamicForm.itemHoverStyle",
        flags:"IRW",
        description:"CSS Style for hovers shown for items",
        name:"itemHoverStyle",
        seeAlso:[
            {ref:"attr:FormItem.hoverStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"formHover\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:PickList.emptyPickListMessage":{
        ref:"attr:PickList.emptyPickListMessage",
        flags:"IRWA",
        description:"Empty message to display in the pickList if \n ${isc.DocUtils.linkForRef('attr:PickList.hideEmptyPickList')} is <code>false</code>.",
        name:"emptyPickListMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.autoFetchData":{
        ref:"attr:ListGrid.autoFetchData",
        flags:"IR",
        description:"If true, when this component is first drawn, automatically call <code>this.fetchData()</code>.\n Criteria for this fetch may be picked up from ${isc.DocUtils.linkForRef('attr:ListGrid.initialCriteria')}, and textMatchStyle may\n be specified via ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchTextMatchStyle')}.\n <P>\n <span style='color:red'>NOTE:</span> if <code>autoFetchData</code> is set, calling\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} before draw will cause two requests to be issued, one from the manual\n call to fetchData() and one from the autoFetchData setting.  The second request will use\n only ${isc.DocUtils.linkForRef('attr:ListGrid.initialCriteria')} and not any other criteria or settings from the first request.\n Generally, turn off autoFetchData if you are going to manually call fetchData() at any time.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"method:ListGrid.fetchData", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fetchOperation')}"],
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "attr:Calendar.baseStyle":{
        ref:"attr:Calendar.baseStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to the grid cells of the day and week views\n of the calendar. This style will have \"Dark\", \"Over\", \"Selected\", or \"Disabled\"\n appended to it according to the state of the cell.",
        name:"baseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calendar\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "method:MenuItem.enableIf":{
        ref:"method:MenuItem.enableIf",
        flags:"",
        description:" Contains the condition that will enable or disable the curent menuItem. The handler must be specified\n as a function or string of script.  Return false to disable the menuItem or true to enable it\n <p>\n If you don't need to set this state dynamically, use ${isc.DocUtils.linkForRef('attr:MenuItem.enabled')} instead.\n <p>\n May be defined as a ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')}.\n <p>\n",
        name:"enableIf",
        seeAlso:[],
        params:[
            {description:"${isc.DocUtils.linkForRef('attr:Menu.target','target')} attribute for the top level menu.",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"${isc.DocUtils.linkForRef('class:Menu','menu')} contains the reference to the menu that contains the current item",
             optional:false, name:"menu", defaultValue:null, type:"Menu"},
            {description:"contains the reference to the current item", optional:false, name:"item",
             defaultValue:null, type:"MenuItem"}
        ],
        returns:{description:"Return true to show a checkmark by this menu item", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('menuDynamicItems')}"],
        definingClass:"object:MenuItem",
        groups:["dynamicMenuItem"]
    },
    "method:DataSource.getFieldOperators":{
        ref:"method:DataSource.getFieldOperators",
        flags:"",
        description:" Get the list of ${isc.DocUtils.linkForRef('type:OperatorId')}s available for this field.\n <P>\n By default, if ${isc.DocUtils.linkForRef('attr:DataSourceField.validOperators','field.validOperators')} is set,\n returns that list, otherwise returns result of ${isc.DocUtils.linkForRef('method:DataSource.getTypeOperators')}.\n",
        name:"getFieldOperators",
        seeAlso:[],
        params:[
            {description:"Field (or field name) to obtain operators for", optional:false, name:"field",
             defaultValue:null, type:"string or DataSourceField"}
        ],
        returns:{description:"available Operators", type:"Array of OperatorId"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "attr:StretchImg.capSize":{
        ref:"attr:StretchImg.capSize",
        flags:"IRW",
        description:"If the default items are used, capSize is the size in pixels of the first and last\n          images in this stretchImg.",
        name:"capSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "method:Canvas.shouldSnapOnDrop":{
        ref:"method:Canvas.shouldSnapOnDrop",
        flags:"A",
        description:" Override this method to give programmatic control over whether or not the parameter \n <code>dragTarget</code> should snap to this object's grid when dropped.  Note that this only applies\n if snap-to-grid is enabled on either <code>dragTarget</code> or this object.  See \n ${isc.DocUtils.linkForRef('attr:Canvas.snapToGrid')} and ${isc.DocUtils.linkForRef('attr:Canvas.childrenSnapToGrid')}.\n <P>\n The default implementation simply returns true.\n\n",
        name:"shouldSnapOnDrop",
        seeAlso:[],
        params:[
            {description:"The object about to be dropped", optional:false, name:"dragTarget",
             defaultValue:null, type:"isc.Canvas"}
        ],
        returns:{description:"true if <code>dragTarget</code> should snap to this object's grid; otherwise false",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:ViewLoader.allowCaching":{
        ref:"attr:ViewLoader.allowCaching",
        flags:"IR",
        description:"By default a ViewLoader will explicitly prevent browser caching.\n <P>\n Set to true to allow browser caching <b>if the browser would normally do so</b>, in other\n words, if the HTTP headers returned with the response indicate that the response can be\n cached.",
        name:"allowCaching",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ViewLoader"
    },
    "attr:Canvas.showDragShadow":{
        ref:"attr:Canvas.showDragShadow",
        flags:"IRWA",
        description:"When this widget is dragged, if its dragAppearance is <code>\"target\"</code>, should\n we show a shadow behind the canvas during the drag.",
        name:"showDragShadow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragEffects')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:CubeGrid.selectFacetValue":{
        ref:"method:CubeGrid.selectFacetValue",
        flags:"",
        description:"\n  select/deselect the header for a given facet value\n\n  <br><i>methodType</i> action\n",
        name:"selectFacetValue",
        seeAlso:[],
        params:[
            {description:"ID of facet", optional:false, name:"facetId", defaultValue:null, type:"string"},
            {description:"ID of facetValue to select", optional:false, name:"facetValueId",
             defaultValue:null, type:"string"},
            {description:"new selection state - if null defaults to true", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:Tree.getLength":{
        ref:"method:Tree.getLength",
        flags:"",
        description:"\n Returns the number of items in the current open list.\n\n",
        name:"getLength",
        seeAlso:[
            {ref:"method:Tree.getOpenList", description:null}
        ],
        returns:{description:"number of items in open list", type:"number"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:groupNode.groupMembers":{
        ref:"attr:groupNode.groupMembers",
        flags:"R",
        description:"Array of ListGridRecord that belong to this group, or, for multi-field grouping, array\n of groupNodes of subgroups under this groupNode.",
        name:"groupMembers",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Array of ListGridRecord or GroupNode",
        defaultValue:"see below",
        type:"attr",
        definingClass:"object:groupNode",
        groups:["grouping"]
    },
    "group:dataBoundComponentMethods":{
        ref:"group:dataBoundComponentMethods",
        title:"DataBound Component Methods",
        treeLocation:["Client Reference/Data Binding"],
        refs:["method:MultiView.fetchData", "method:MultiView.filterData", 
         "method:MultiView.fetchRelatedData","method:MultiView.removeSelectedData", "method:MultiView.anySelected", 
         "method:MultiView.getSelection","method:MultiView.selectRecord", "method:MultiView.selectRecords", 
         "method:MultiView.selectAllRecords","method:MultiView.deselectRecord", "method:MultiView.deselectRecords", 
         "method:MultiView.deselectAllRecords","method:DynamicForm.editNewRecord", "method:DynamicForm.editRecord", 
         "method:DynamicForm.editSelectedData","method:DynamicForm.submit", "method:DynamicForm.saveData", "method:DynamicForm.fetchData", 
         "method:DynamicForm.filterData","method:ValuesManager.editNewRecord", "method:ValuesManager.editRecord", 
         "method:ValuesManager.editSelectedData","method:ValuesManager.saveData", "method:ValuesManager.submit", 
         "method:ValuesManager.filterData","method:ValuesManager.fetchData", "method:DetailViewer.viewSelectedData", 
         "method:Calendar.fetchData","method:Calendar.filterData", "method:ListGrid.removeData", "method:TileGrid.fetchData", 
         "method:TileGrid.filterData","method:TileGrid.invalidateCache", "method:TileGrid.removeSelectedData", 
         "method:TileGrid.addData","method:TileGrid.removeData", "method:ColumnTree.fetchData", 
         "method:ColumnTree.filterData","method:ListGrid.fetchData", "method:ListGrid.exportData", "method:ListGrid.filterData", 
         "method:ListGrid.fetchRelatedData","method:ListGrid.clearCriteria", "method:ListGrid.getCriteria", 
         "method:ListGrid.setCriteria","method:ListGrid.invalidateCache", "method:ListGrid.willFetchData", 
         "method:ListGrid.addData","method:ListGrid.updateData", "method:ListGrid.removeSelectedData", 
         "method:TreeGrid.fetchData","method:TreeGrid.filterData", "method:TileGrid.exportData", 
         "method:DetailViewer.exportData"],
        description:"An Action Method initiates an orchestrated client-server flow that stores or retrieves data\n and updates one or more components.\n <P>\n For example, the ${isc.DocUtils.linkForRef('method:DynamicForm.saveData','editor.saveData()')} Action Method saves the\n record currently being edited in the form, transparently handling the trip to the server,\n standard error conditions such as validation errors (whether the validation error\n happens on the client or server), and update of client-side caches.\n <P>\n Action Methods are available on DataBoundComponents.",
        name:"dataBoundComponentMethods",
        seeAlso:[
            {ref:"interface:DataBoundComponent", description:null}
        ],
        type:"group"
    },
    "attr:EdgedCanvas.edgeLeft":{
        ref:"attr:EdgedCanvas.edgeLeft",
        flags:"IR",
        description:"Height in pixels for left corners and edges.  Defaults to edgeSize when unset.",
        name:"edgeLeft",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "method:TileLayout.transferDragData":{
        ref:"method:TileLayout.transferDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method is called to transfer a set of records that\n were dropped onto some other component.  This method is called after the set of records has\n been copied to the other component.  Whether or not this component's data is modified is \n determined by the value of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dragDataAction')}.\n <P>\n With a <code>dragDataAction</code> of \"move\", a databound component will issue \"remove\"\n dsRequests against its DataSource to actually remove the data, via\n ${isc.DocUtils.linkForRef('method:DataSource.removeData')}.\n\n",
        name:"transferDragData",
        seeAlso:[
            {ref:"method:DataBoundComponent.getDragData", description:null},
            {ref:"method:ListGrid.willAcceptDrop", description:null}
        ],
        returns:{description:"Array of objects that were dragged out of this ListGrid.", type:"Array"},
        type:"method",
        definingClass:"class:TileLayout"
    },
    "attr:ToolStrip.resizeBarSize":{
        ref:"attr:ToolStrip.resizeBarSize",
        flags:"IRA",
        description:"Thickness of the resizeBars in pixels",
        name:"resizeBarSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"14",
        type:"attr",
        definingClass:"class:ToolStrip"
    },
    "attr:ListGridField.summaryFunction":{
        ref:"attr:ListGridField.summaryFunction",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} or ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, \n this attribute can be used to specify\n an explicit ${isc.DocUtils.linkForRef('type:SummaryFunction')} for calculating the summary value to\n display.",
        name:"summaryFunction",
        seeAlso:[],
        valueType:"SummaryFunction",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:Button.setTitle":{
        ref:"method:Button.setTitle",
        flags:"",
        description:" Set the title.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["appearance"]
    },
    "attr:Calendar.canDeleteEvents":{
        ref:"attr:Calendar.canDeleteEvents",
        flags:"IR",
        description:"If true, users can delete existing events. Defaults to ${isc.DocUtils.linkForRef('attr:Calendar.canEditEvents')}",
        name:"canDeleteEvents",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["allowedOperations"]
    },
    "method:DynamicForm.reset":{
        ref:"method:DynamicForm.reset",
        flags:"",
        description:"\n Resets values to the state it was the last time <code>setValues()</code> or\n <code>rememberValues()</code> was called. If neither of those methods has been called,\n values will be set back to their inital values at init time.\n\n",
        name:"reset",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "method:Tree.getDescendantLeaves":{
        ref:"method:Tree.getDescendantLeaves",
        flags:"",
        description:"\n Returns the list of all descendants of a node that are leaves.  This works just like\n ${isc.DocUtils.linkForRef('method:Tree.getDescendants')}, except folders are not part of the returned list.\n Folders are still recursed into, just not returned.  Like ${isc.DocUtils.linkForRef('method:Tree.getDescendants')},\n this method can be very slow for large trees.  Generally, ${isc.DocUtils.linkForRef('method:Tree.find')} in\n combination with ${isc.DocUtils.linkForRef('method:Tree.getLeaves')} will be much faster.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')},\n so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} interface\n instead of as a native Javascript Array.\n\n",
        name:"getDescendantLeaves",
        seeAlso:[],
        params:[
            {description:"node in question (the root node is assumed if none specified)",
             optional:true, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"List of descendants of the node that are leaves.", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:ListGrid.setValueMap":{
        ref:"method:ListGrid.setValueMap",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('attr:ListGridField.valueMap','valueMap')} for a field.\n See also the ${isc.DocUtils.linkForRef('method:ListGrid.setEditorValueMap','setEditorValueMap()')}\n and ${isc.DocUtils.linkForRef('method:ListGrid.getEditorValueMap','getEditorValueMap()')} methods which allow further \n customization of the valueMap displayed while the field is in edit mode.\n\n",
        name:"setValueMap",
        seeAlso:[],
        params:[
            {description:"Name or number of field to update", optional:false, name:"fieldID",
             defaultValue:null, type:"string || number"},
            {description:"ValueMap for the field", optional:false, name:"map", defaultValue:null,
             type:"object"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:TreeGrid.closedIconSuffix":{
        ref:"attr:TreeGrid.closedIconSuffix",
        flags:"IRW",
        description:"This suffix will be appended to the ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} for closed folders.\n If ${isc.DocUtils.linkForRef('attr:TreeGrid.showOpenIcons')} is set to <code>false</code> this suffix will also be\n appended to open folders' icons.",
        name:"closedIconSuffix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"closed\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "group:noFrames":{
        ref:"group:noFrames",
        title:"Don't Misuse Frames",
        refs:[],
        description:"Loading the SmartClient framework into multiple frames or iframes within the same browser is\n not a supported configuration, or more accurately, not a <i>supportable</i> configuration,\n for the following reasons:\n <ul>\n <li> each additional frame multiplies the memory footprint and reduces speed\n <li> having multiple frames prevents drag and drop between components in different frames\n <li> modality handling (eg modal dialogs) doesn't automatically take into account multiple\n frames (consider tabbing order, nested modality and other issues, you'll see it's not\n realistic to provide automatic cross-frame modality handling)\n <li> inter-frame communication triggers several browser bugs: memory leaks, performance\n issues, intermittent crashes in some browsers, inconsistencies in basic JavaScript operators\n such as \"typeof\", and problems with form focus handling in IE, among many other bugs\n </ul>\n None of these problems are specific to SmartClient.  They happen with Ajax frameworks in\n general as well as other RIA technologies.  This is why no successful Ajax application has\n ever used the approach of double-loading a component framework into multiple frames.\n <P>\n The recommended ${isc.DocUtils.linkForRef('group:smartArchitecture','SmartClient Architecture')} involves loading as many\n SmartClient-based application views as possible in the first page load, then showing and\n hiding different views as the user navigates through the application.\n <P>\n If, for whatever reason, you cannot follow the SmartClient Architecture and must load new\n SmartClient-based views by contacting the server each time, use the ${isc.DocUtils.linkForRef('class:ViewLoader')} class\n to load new views, never frames.\n <P>\n Note that the use of IFrames is appropriate in certain circumstances, including loading\n certain types of content within an ${isc.DocUtils.linkForRef('class:HTMLFlow','contentsType,HTMLFlow')}.  The only\n prohibited usage is loading the SmartClient framework into multiple frames within the same\n browser.",
        name:"noFrames",
        seeAlso:[],
        type:"group"
    },
    "attr:ColorPicker.autoPosition":{
        ref:"attr:ColorPicker.autoPosition",
        flags:"IR",
        description:"If true, causes the ColorPicker to appear near where the mouse was last clicked.\n If false, the ColorPicker is centered on first show; depending on the value of \n ${isc.DocUtils.linkForRef('attr:ColorPicker.autoCenterOnShow')}, it either reappears wherever it was last shown after hide/show(), \n or centered regardless of where it was last shown.",
        name:"autoPosition",
        seeAlso:[
            {ref:"attr:ColorPicker.autoCenterOnShow", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:DateChooser.prevYearIconHeight":{
        ref:"attr:DateChooser.prevYearIconHeight",
        flags:"IR",
        description:"Height of the icon for the previous year button",
        name:"prevYearIconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:Window.addItems":{
        ref:"method:Window.addItems",
        flags:"",
        description:"			Adds an array of widgets to the window.\n",
        name:"addItems",
        seeAlso:[],
        params:[
            {description:"an array of widgets to be added", optional:false, name:"items",
             defaultValue:null, type:"Array of Canvas"}
        ],
        returns:{description:"array of widgets added", type:"array"},
        type:"method",
        definingClass:"class:Window",
        groups:["windowItems"]
    },
    "group:chartAppearance":{
        ref:"group:chartAppearance",
        refs:["attr:Chart.threeD", "attr:Chart.shouldAnimateShow", "attr:Chart.labelValues"],
        name:"chartAppearance",
        seeAlso:[],
        type:"group"
    },
    "attr:OperationBinding.whereClause":{
        ref:"attr:OperationBinding.whereClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n WHERE clause to use when constructing the SQL query to perform this operation.  The\n property should be a valid expression in the syntax of the underlying database.  The \n server will insert the text of this property immediately after the \"WHERE\" token.\n <p>\n You may find the SmartClient-provided <b>$criteria</b> variable of particular use with\n this property.\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"whereClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "method:List.getProperty":{
        ref:"method:List.getProperty",
        flags:"",
        description:" Return a new Array where the value of item i is the value of \"property\" of item i in this\n array.  If an item doesn't have that property or is null, return item will be null.\n\n",
        name:"getProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to look for", optional:false, name:"property",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"array of the values of property in each item of this list", type:"Array"},
        type:"method",
        definingClass:"interface:List",
        groups:["iteration"]
    },
    "class:OperationBinding":{
        methods:[],
        ref:"class:OperationBinding",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:["attr:OperationBinding.operationType", "attr:OperationBinding.operationId", 
         "attr:OperationBinding.serverMethod","attr:OperationBinding.methodArguments", "attr:OperationBinding.serverObject", 
         "attr:OperationBinding.wsOperation","attr:OperationBinding.dataURL", "attr:OperationBinding.dataProtocol", 
         "attr:OperationBinding.dataFormat","attr:OperationBinding.dataTransport", "attr:OperationBinding.useHttpProxy", 
         "attr:OperationBinding.callbackParam","attr:OperationBinding.requestProperties", "attr:OperationBinding.preventHTTPCaching", 
         "attr:OperationBinding.defaultParams","attr:OperationBinding.useFlatFields", "attr:OperationBinding.recordXPath", 
         "attr:OperationBinding.recordName","attr:OperationBinding.spoofResponses", "attr:OperationBinding.xmlNamespaces", 
         "attr:OperationBinding.responseDataSchema","attr:OperationBinding.exportResults", "attr:OperationBinding.exportAs", 
         "attr:OperationBinding.exportFilename","attr:OperationBinding.lineBreakStyle", "attr:OperationBinding.exportFields", 
         "attr:OperationBinding.selectClause","attr:OperationBinding.tableClause", "attr:OperationBinding.whereClause", 
         "attr:OperationBinding.groupClause","attr:OperationBinding.orderClause", "attr:OperationBinding.valuesClause", 
         "attr:OperationBinding.customSQL","attr:OperationBinding.customHQL", "attr:OperationBinding.customValueFields", 
         "attr:OperationBinding.customCriteriaFields","attr:OperationBinding.excludeCriteriaFields", "attr:OperationBinding.useForCacheSync", 
         "attr:OperationBinding.cacheSyncOperation","attr:OperationBinding.canSyncCache", "attr:OperationBinding.sqlType", 
         "attr:OperationBinding.allowMultiUpdate","attr:OperationBinding.qualifyColumnNames", "attr:OperationBinding.invalidateCache", 
         "attr:OperationBinding.outputs","attr:OperationBinding.criteria", "attr:OperationBinding.values", 
         "attr:OperationBinding.requiresAuthentication","attr:OperationBinding.requiresRole", "attr:OperationBinding.requires", 
         "attr:OperationBinding.mail"],
        description:"An operationBinding tells a DataSource how to execute one of the basic DS operations: fetch,\n add, update, remove.  See ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings')}.",
        name:"OperationBinding",
        seeAlso:[],
        type:"class",
        serverDS:"allowed"
    },
    "attr:CheckboxItem.unsetImage":{
        ref:"attr:CheckboxItem.unsetImage",
        flags:"IRW",
        description:"URL for the image to display when this checkbox is unset. Note that if \n ${isc.DocUtils.linkForRef('attr:CheckboxItem.showUnsetImage')} is false or ${isc.DocUtils.linkForRef('attr:CheckboxItem.allowEmptyValue')} is false\n the ${isc.DocUtils.linkForRef('attr:CheckboxItem.uncheckedImage')} will be used for null values rather than this\n image.",
        name:"unsetImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/unsetcheck.gif\"",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["appearance"]
    },
    "method:ListGrid.getGridSummaryFunction":{
        ref:"method:ListGrid.getGridSummaryFunction",
        flags:"A",
        description:" Determines the ${isc.DocUtils.linkForRef('type:SummaryFunction')} to use when calculating per-field summary values\n describing multiple records in this grid.\n Used to determine the summary function to use for both ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')}.\n <P>. \n Default implementation picks up ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')} if explicitly specified,\n otherwise checks for a default summary function based on field type (see\n ${isc.DocUtils.linkForRef('classMethod:SimpleType.setDefaultSummaryFunction')})\n",
        name:"getGridSummaryFunction",
        seeAlso:[],
        params:[
            {description:"field to check for summary function", optional:false, name:"field",
             defaultValue:null, type:"listGridField"}
        ],
        returns:{description:"summary function for the field in question", type:"SummaryFunction"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ImgTab.showFocus":{
        ref:"attr:ImgTab.showFocus",
        deprecated:"as of SmartClient 6.1 in favor of ${isc.DocUtils.linkForRef('attr:ImgTab.showFocused')}",
        flags:"IRW",
        description:"Should we visibly change state when the tab recieves keyboard focus?",
        name:"showFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "attr:ImgButton.iconHeight":{
        ref:"attr:ImgButton.iconHeight",
        flags:"IR",
        description:"Height in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "method:ListGrid.rowMouseDown":{
        ref:"method:ListGrid.rowMouseDown",
        flags:"",
        description:" Called when a row receives a mousedown event.\n \n",
        name:"rowMouseDown",
        seeAlso:[],
        params:[
            {description:"record object returned from 'getCellRecord()'", optional:false,
             name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "type:TextMatchStyle":{
        ref:"type:TextMatchStyle",
        values:[
            {description:"test for exact match", value:"\"exact\""},
            {description:"test for case-insenstive substring match", value:"\"substring\""},
            {description:"test for the beginning of the value matching the search criteria",
             value:"\"startsWith\""}
        ],
        description:"For \"fetch\" operations, how search criteria should be interpreted for text fields.",
        name:"TextMatchStyle",
        seeAlso:[],
        type:"type"
    },
    "method:DataBoundComponent.getSummaryFieldValue":{
        ref:"method:DataBoundComponent.getSummaryFieldValue",
        flags:"",
        description:" Get the computed value of a ${isc.DocUtils.linkForRef('attr:DataBoundComponent.canAddSummaryFields','summary field')}.\n",
        name:"getSummaryFieldValue",
        seeAlso:[],
        params:[
            {description:"field that has a summary format", optional:false, name:"field",
             defaultValue:null, type:"Field"},
            {description:"record to use to compute formula value", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"formula result", type:"Number"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:SliderItem.minValue":{
        ref:"attr:SliderItem.minValue",
        flags:"IR",
        description:"The minimum slider value. The slider value is equal to minValue when the thumb is at the\n bottom or left of the slider (unless flipValues is true, in which case the minimum value\n is at the top/right of the slider)",
        name:"minValue",
        setter:["setMinValue"],
        seeAlso:[
            {ref:"attr:Slider.flipValues", description:null}
        ],
        valueType:"float",
        defaultValue:"1",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"],
        definingClass:"class:SliderItem"
    },
    "attr:DatabaseBrowser.catalog":{
        ref:"attr:DatabaseBrowser.catalog",
        flags:"IR",
        description:"The name of the database catalog to use, if the RDBMS in use recognizes the concept of\n multiple catalogs.  Leave this property unset to include all catalogs.\n <p>\n Note that this property only applies where ${isc.DocUtils.linkForRef('attr:DatabaseBrowser.serverType')} is \"sql\"",
        name:"catalog",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:StretchImgButton.labelBreadthPad":{
        ref:"attr:StretchImgButton.labelBreadthPad",
        flags:"IRW",
        description:"The padding for a StretchImgButton's label is determined as follows.\n <P>\n If <code>labelHPad</code> is set it will specify the horizontal padding applied to the\n label. Similarly if <code>labelVPad</code> is set it will specify the vertical padding\n for the label, regardless of the button's ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical','vertical')} setting.\n <P>\n Otherwise <code>labelLengthPad</code> can be set to specify the label padding along the\n length axis (ie: horizontal padding if ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical')} is false,\n otherwise vertical padding), and \n <code>labelBreadthPad</code> can be set to specify the label padding along the other axis.\n <P>\n Otherwise the padding on the length axis will match the ${isc.DocUtils.linkForRef('attr:StretchImgButton.capSize')} and\n will be set to zero on the breadth axis.\n <P>\n So by default the label will be sized to match the center image of the StretchImgButton, but\n these settings allow the label to partially or wholly overlap the caps.",
        name:"labelBreadthPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "method:Array.or":{
        ref:"method:Array.or",
        flags:"",
        description:" Returns true if at least one value between the start and end indices is true.\n\n",
        name:"or",
        seeAlso:[],
        params:[
            {description:"optional start index (default is 0)", optional:true, name:"start",
             defaultValue:null, type:"number"},
            {description:"optional end index (default is list.length)", optional:true, name:"end",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"at least one of the items is true", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "attr:Canvas.ID":{
        ref:"attr:Canvas.ID",
        flags:"IR",
        getter:["getID"],
        description:"Global identifier for referring to a widget in JavaScript.  The ID property is optional if\n you do not need to refer to the widget from JavaScript, or can refer to it indirectly\n (for example, by storing the reference returned by ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')}).\n <P>\n An internal, unique ID will automatically be created upon instantiation for any canvas\n where one is not provided. \n <P>\n The ID property should be unique in the global scope. If <code>window[<i>ID</i>]</code> \n is already assigned to something else a warning will be logged using the developer console,\n and the existing reference will be replaced, calling ${isc.DocUtils.linkForRef('method:Canvas.destroy','destroy()')} on the\n previous object if it is a SmartClient Class instance.\n <P>\n Automatically generated IDs will be unique as long as the canvases they refer to remain\n active - once a canvas with an automatically generated ID has been destroyed, its ID may be\n reused for the next canvas created with no explicitly specified ID.",
        name:"ID",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["basics"]
    },
    "method:MultiSortPanel.validate":{
        ref:"method:MultiSortPanel.validate",
        flags:"",
        description:" Validate that no two ${isc.DocUtils.linkForRef('object:SortSpecifier')}s sort on the same \n ${isc.DocUtils.linkForRef('attr:SortSpecifier.property','property')}.\n\n",
        name:"validate",
        seeAlso:[],
        returns:{description:"True if validation succeeds, false if any property is used twice",
         type:"boolean"},
        type:"method",
        definingClass:"class:MultiSortPanel"
    },
    "method:Calendar.updateEvent":{
        ref:"method:Calendar.updateEvent",
        flags:"",
        description:" Update an event in this calendar.\n\n",
        name:"updateEvent",
        seeAlso:[],
        params:[
            {description:"The event object to remove from the calendar", optional:false, name:"event",
             defaultValue:null, type:"CalendarEvent"},
            {description:"start date of event", optional:false, name:"startDate", defaultValue:null,
             type:"Date"},
            {description:"end date of event", optional:false, name:"endDate", defaultValue:null,
             type:"Date"},
            {description:"name of event", optional:false, name:"name", defaultValue:null, type:"String"},
            {description:"description of event", optional:false, name:"description", defaultValue:null,
             type:"String"},
            {description:"new values of additional fields to be updated", optional:false,
             name:"otherFields", defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:ListGrid.minFieldWidth":{
        ref:"attr:ListGrid.minFieldWidth",
        flags:"IRW",
        description:"Minimum size, in pixels, for ListGrid headers.",
        name:"minFieldWidth",
        seeAlso:[],
        valueType:"int",
        defaultValue:"10",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "type:PercentBoxModel":{
        ref:"type:PercentBoxModel",
        values:[
            {description:"use coordinates relative to the {${isc.DocUtils.linkForRef('method:Canvas.getVisibleHeight')},visibleHeight} and width of the other canvas",
             value:"\"visible\""},
            {description:"use coordinates relative to the {${isc.DocUtils.linkForRef('method:Canvas.getViewportHeight')},viewportHeight} and width of the other canvas",
             value:"\"viewport\""}
        ],
        description:"Determines sizing model when sizing / positioning a canvas relative to its\n ${isc.DocUtils.linkForRef('attr:Canvas.percentBox','percentBox')}.",
        name:"PercentBoxModel",
        seeAlso:[],
        type:"type"
    },
    "method:Button.setIcon":{
        ref:"method:Button.setIcon",
        flags:"",
        description:" Change the icon being shown next to the title text.\n",
        name:"setIcon",
        seeAlso:[],
        params:[
            {description:"URL of new icon", optional:false, name:"icon", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:FormItem.pickerIconWidth":{
        ref:"attr:FormItem.pickerIconWidth",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconWidth",
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:RadioGroupItem.textBoxStyle":{
        ref:"attr:RadioGroupItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class applied to the text for items within this radiogroup.",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"labelAnchor\"",
        type:"attr",
        definingClass:"class:RadioGroupItem",
        groups:["appearance"]
    },
    "attr:ListGridField.showDownIcon":{
        ref:"attr:ListGridField.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "attr:CheckboxItem.textBoxStyle":{
        ref:"attr:CheckboxItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class for this item's title text",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"labelAnchor\"",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["appearance"]
    },
    "method:GridRenderer.getNearestColToEvent":{
        ref:"method:GridRenderer.getNearestColToEvent",
        flags:"",
        description:"			Returns the nearest column to the event coordinates\n",
        name:"getNearestColToEvent",
        seeAlso:[],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events", "selection"]
    },
    "method:CubeGrid.facetAdded":{
        ref:"method:CubeGrid.facetAdded",
        flags:"",
        description:"      Notification fired when a new facet is added.\n \n",
        name:"facetAdded",
        seeAlso:[
            {ref:"method:CubeGrid.facetRemoved", description:"<br><i>methodType</i>  handler"}
        ],
        params:[
            {description:"facetId that was added", optional:false, name:"facetId", defaultValue:null,
             type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "group:workday":{
        ref:"group:workday",
        refs:["attr:Calendar.workdayBaseStyle", "attr:Calendar.workdayStart", "attr:Calendar.workdayEnd", 
         "attr:Calendar.showWorkday","attr:Calendar.workdays", "attr:Calendar.scrollToWorkday"],
        name:"workday",
        seeAlso:[],
        type:"group"
    },
    "method:DataSource.setTypeOperators":{
        ref:"method:DataSource.setTypeOperators",
        flags:"",
        description:" Set the list of ${isc.DocUtils.linkForRef('type:OperatorId')}s valid for a given FieldType.\n \n",
        name:"setTypeOperators",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"typeName", defaultValue:null, type:"FieldType"},
            {description:"available Operators", optional:false, name:"operators", defaultValue:null,
             type:"Array of OperatorId"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "attr:DataSource.enumOrdinalProperty":{
        ref:"attr:DataSource.enumOrdinalProperty",
        flags:"IA",
        description:"The name of the property this DataSource uses for ordinal number when translating\n Java enumerated types to and from Javascript, if the ${isc.DocUtils.linkForRef('type:EnumTranslateStrategy')} is set\n to \"bean\".  Defaults to \"_ordinal\" if not set.\n <p>\n  This property is only applicable if you are using the SmartClient server",
        name:"enumOrdinalProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:DynamicForm.longTextEditorThreshold":{
        ref:"attr:DynamicForm.longTextEditorThreshold",
        flags:"IRW",
        description:"When creating form items for fields with text type data, if the specified length of the\n field exceeds this threshold we will create form item of type \n <code>this.longTextEditorType</code> (a TextAreaItem by default), rather than a simple\n text item.  Overridden by explicitly specifying <code>editorType</code> for the field.",
        name:"longTextEditorThreshold",
        seeAlso:[],
        valueType:"number",
        defaultValue:"255",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["appearance"]
    },
    "attr:ColumnTree.customIconOpenProperty":{
        ref:"attr:ColumnTree.customIconOpenProperty",
        flags:"IRWA",
        description:"This property allows the developer to rename the \n ${isc.DocUtils.linkForRef('attr:TreeNode.showOpenIcon','default node.showOpenIcon')} property.",
        name:"customIconOpenProperty",
        seeAlso:[
            {ref:"attr:TreeGrid.customIconProperty", description:null},
            {ref:"attr:TreeGrid.showCustomIconOpen", description:null}
        ],
        valueType:"string",
        defaultValue:"\"showOpenIcon\"",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "method:Canvas.getViewportHeight":{
        ref:"method:Canvas.getViewportHeight",
        flags:"A",
        description:"  Returns the height of the viewport onto the scrollable content.\n\n",
        name:"getViewportHeight",
        seeAlso:[],
        returns:{description:"height of the viewport, in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "classAttr:FileLoader.defaultSkin":{
        ref:"classAttr:FileLoader.defaultSkin",
        flags:"RW",
        description:"Default skin for  ${isc.DocUtils.linkForRef('classMethod:FileLoader.loadISC')}/${isc.DocUtils.linkForRef('classMethod:FileLoader.cacheISC')} to\n use if one is not provided by the user.",
        name:"defaultSkin",
        seeAlso:[
            {ref:"classMethod:FileLoader.cacheISC", description:null},
            {ref:"classMethod:FileLoader.loadISC", description:null}
        ],
        valueType:"String",
        defaultValue:"\"standard\"",
        type:"classAttr",
        definingClass:"class:FileLoader"
    },
    "attr:FormItem.hoverOpacity":{
        ref:"attr:FormItem.hoverOpacity",
        flags:"IRW",
        description:"Opacity for any hover shown for this item",
        name:"hoverOpacity",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverOpacity", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:TestFunctionResult.errorText":{
        ref:"attr:TestFunctionResult.errorText",
        flags:"IRW",
        description:"If the formula or summary format caused a JavaScript error, this contains the JavaScript error text.",
        name:"errorText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "class:StatefulCanvas":{
        methods:["method:StatefulCanvas.setBaseStyle", "method:StatefulCanvas.setState", 
         "method:StatefulCanvas.getState","method:StatefulCanvas.setSelected", "method:StatefulCanvas.select", 
         "method:StatefulCanvas.deselect","method:StatefulCanvas.isSelected", "method:StatefulCanvas.getActionType", 
         "method:StatefulCanvas.setActionType","method:StatefulCanvas.addToRadioGroup", "method:StatefulCanvas.removeFromRadioGroup", 
         "method:StatefulCanvas.setDisabled","method:StatefulCanvas.getStateSuffix", "method:StatefulCanvas.setIconOrientation", 
         "method:StatefulCanvas.setAutoFit","method:StatefulCanvas.getTitle", "method:StatefulCanvas.setTitle", 
         "method:StatefulCanvas.setIcon"],
        ref:"class:StatefulCanvas",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:StatefulCanvas.title", "attr:StatefulCanvas.redrawOnStateChange", 
         "attr:StatefulCanvas.selected","attr:StatefulCanvas.state", "attr:StatefulCanvas.showRollOver", 
         "attr:StatefulCanvas.showFocus","attr:StatefulCanvas.showFocused", "attr:StatefulCanvas.showFocusedAsOver", 
         "attr:StatefulCanvas.showDown","attr:StatefulCanvas.showDisabled", "attr:StatefulCanvas.actionType", 
         "attr:StatefulCanvas.radioGroup","attr:StatefulCanvas.baseStyle", "attr:StatefulCanvas.align", "attr:StatefulCanvas.valign", 
         "attr:StatefulCanvas.autoFit","attr:StatefulCanvas.icon", "attr:StatefulCanvas.iconSize", 
         "attr:StatefulCanvas.iconWidth","attr:StatefulCanvas.iconHeight", "attr:StatefulCanvas.iconOrientation", 
         "attr:StatefulCanvas.iconSpacing","attr:StatefulCanvas.showDisabledIcon", "attr:StatefulCanvas.showRollOverIcon", 
         "attr:StatefulCanvas.showDownIcon","attr:StatefulCanvas.showSelectedIcon", "attr:StatefulCanvas.showFocusedIcon", 
         "attr:StatefulCanvas.showOverCanvas","attr:StatefulCanvas.overCanvas", "attr:StatefulCanvas.overCanvasConstructor", 
         "attr:StatefulCanvas.overCanvasDefaults"],
        description:"A component that has a set of possible states, and which presents itself differently according to\n which state it is in.  An example is a button, which can be \"up\", \"down\", \"over\" or \"disabled\".",
        name:"StatefulCanvas",
        seeAlso:[],
        type:"class"
    },
    "method:Canvas.setShowShadow":{
        ref:"method:Canvas.setShowShadow",
        flags:"",
        description:" Method to update ${isc.DocUtils.linkForRef('attr:Canvas.showShadow')}.\n",
        name:"setShowShadow",
        seeAlso:[],
        params:[
            {description:"true if the shadow should be visible false if not", optional:false,
             name:"showShadow", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["shadow"]
    },
    "attr:ComboBoxItem.pickListCriteria":{
        ref:"attr:ComboBoxItem.pickListCriteria",
        flags:"IRWA",
        description:"If this item has a databound pick-list (for example ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} is\n set), this property can be used to provide static filter criteria when retrieving the data\n for the pickList.",
        name:"pickListCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:GridRenderer.fixedColumnWidths":{
        ref:"attr:GridRenderer.fixedColumnWidths",
        flags:"IRWA",
        description:"Should we horizontally clip cell contents, or allow columns to expand horizontally to\n show all contents?<br><br>\n\n If we allow columns to expand, the column width is treated as a minimum.",
        name:"fixedColumnWidths",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["sizing"]
    },
    "method:Canvas.show":{
        ref:"method:Canvas.show",
        flags:"",
        description:" Sets this widget's visibility to \"inherit\", so that it becomes visible if all it's parents\n are visible or it has no parents.\n <P>\n If the widget has not yet been drawn (and doesn't have a parent or master), this method\n calls the draw method as well.\n\n",
        name:"show",
        seeAlso:[],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('showAndHide')}"],
        definingClass:"class:Canvas",
        groups:["visibility"]
    },
    "attr:Button.wrap":{
        ref:"attr:Button.wrap",
        flags:"IRW",
        description:"A boolean indicating whether the button's title should word-wrap, if necessary.",
        name:"wrap",
        setter:["setWrap"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["basics"]
    },
    "classAttr:Time.UTCHoursOffset":{
        ref:"classAttr:Time.UTCHoursOffset",
        deprecated:"As of 7.0 this attribute has been deprecated in favor of ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone')}",
        flags:"IRA",
        description:"Time data is stored in Date type objects. \n In order to have times display consistently across browsers all times assumed to be stored\n as UTC times.\n This property allows you to specify an timezone offset between the stored time \n and the displayed time.",
        name:"UTCHoursOffset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"classAttr",
        definingClass:"class:Time"
    },
    "attr:HiddenItem.colSpan":{
        ref:"attr:HiddenItem.colSpan",
        flags:"IRW",
        description:"hidden fields don't take up any columns",
        name:"colSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:HiddenItem",
        groups:["appearance"]
    },
    "object:TreeNode":{
        methods:[],
        ref:"object:TreeNode",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/TreeGrid", "Client Reference/System/Tree"],
        attrs:["attr:TreeNode.enabled", "attr:TreeNode.canDrag", "attr:TreeNode.canAcceptDrop", 
         "attr:TreeNode.isFolder","attr:TreeNode.name", "attr:TreeNode.title", "attr:TreeNode.id", "attr:TreeNode.parentId", 
         "attr:TreeNode.children","attr:TreeNode.icon", "attr:TreeNode.showOpenIcon", "attr:TreeNode.showDropIcon"],
        description:"Every node in the tree is represented by a TreeNode object which is an object literal with a\n set of properties that configure the node.\n <p>\n When a Tree is supplied as ${isc.DocUtils.linkForRef('attr:TreeGrid.data')} to ${isc.DocUtils.linkForRef('class:TreeGrid')}, you can also set\n properties from ${isc.DocUtils.linkForRef('object:ListGridRecord')} on the TreeNode (e.g. setting\n ${isc.DocUtils.linkForRef('attr:ListGridRecord.enabled')}:<code>false</code> on the node).",
        name:"TreeNode",
        seeAlso:[],
        type:"object"
    },
    "attr:Facet.borderAfter":{
        ref:"attr:Facet.borderAfter",
        flags:"IR",
        description:"CSS line style to apply as a border after this facet, eg \"1px dashed blue\"",
        name:"borderAfter",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:DynamicForm.nestedEditorType":{
        ref:"attr:DynamicForm.nestedEditorType",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('class:FormItem')} class to use for any singular (ie, non-list) complex fields \n on this DynamicForm.",
        name:"nestedEditorType",
        seeAlso:[
            {ref:"attr:DynamicForm.nestedListEditorType", description:null}
        ],
        valueType:"String",
        defaultValue:"\"NestedEditorItem\"",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "method:ValuesManager.submit":{
        ref:"method:ValuesManager.submit",
        flags:"",
        description:" <code>submit()</code> is automatically called when a ${isc.DocUtils.linkForRef('class:SubmitItem')} included in the\n form is clicked, or, if ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOnEnter','saveOnEnter')} is set, when the\n \"Enter\" key is pressed in a text input.  Submit can also be manually called.\n <P>\n If ${isc.DocUtils.linkForRef('method:DynamicForm.submitValues','form.submitValues()')} exists, it will be called, then\n immediately return.\n <P>\n Otherwise, default behavior varies based on ${isc.DocUtils.linkForRef('attr:DynamicForm.canSubmit','form.canSubmit')}: if\n <code>canSubmit</code> is false, ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} will be called to\n handle saving via SmartClient databinding.  \n <P>\n If <code>canSubmit</code> is true, the form will be submitted like an ordinary HTML\n form via ${isc.DocUtils.linkForRef('method:DynamicForm.submitForm')}.\n <P>\n The parameters to <code>submit()</code> apply only if <code>submit()</code> will be\n calling ${isc.DocUtils.linkForRef('method:ValuesManager.saveData')}.  If you override <code>submit()</code>, you can safely\n ignore the parameters as SmartClient framework code does not pass them.\n \n",
        name:"submit",
        seeAlso:[
            {ref:"method:DynamicForm.submitValues", description:null}
        ],
        params:[
            {description:"callback to invoke on completion.                                          [Ignored if this.canSubmit is true]",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued\n                                          [Ignored if this.canSubmit is true]",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "attr:CubeGrid.chartType":{
        ref:"attr:CubeGrid.chartType",
        flags:"IRW",
        description:"Default type of chart to plot.",
        name:"chartType",
        seeAlso:[],
        valueType:"ChartType",
        defaultValue:"\"Column\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:Canvas.childrenSnapToGrid":{
        ref:"attr:Canvas.childrenSnapToGrid",
        flags:"IRW",
        description:"If true, causes this canvas's children to snap to its grid when dragging.\n This behavior can be overridden on a per-child basis by setting the \n ${isc.DocUtils.linkForRef('attr:Canvas.snapToGrid','snapToGrid')} value on the child.",
        name:"childrenSnapToGrid",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:BatchUploader.uploadFieldPrefix":{
        ref:"attr:BatchUploader.uploadFieldPrefix",
        flags:"IRW",
        description:"String to prepend to the names of the additional fields specified in ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadFormFields')} \n before they are stored in the HttpSession on the server.  This property provides a basic\n namespace facility, allowing you to avoid name collisions with existing session attributes.\n <P>\n Example usage: if you have an additional field called \"someDate\" and you set \n uploadFieldPrefix to \"myFields_\", your additionalFormField will be available as an\n HttpSession attribute called \"myFields_someDate\"",
        name:"uploadFieldPrefix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "classAttr:LoginDialog.USERNAME_TITLE":{
        ref:"classAttr:LoginDialog.USERNAME_TITLE",
        flags:"IRW",
        description:"Default title for the ${isc.DocUtils.linkForRef('attr:LoginDialog.usernameItem','\"usernameItem\"')} field in the \n dialog displayed by ${isc.DocUtils.linkForRef('classMethod:isc.showLoginDialog')}.",
        name:"USERNAME_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Username\"",
        type:"classAttr",
        definingClass:"class:LoginDialog",
        groups:["i18nMessages"]
    },
    "attr:Canvas.htmlPosition":{
        ref:"attr:Canvas.htmlPosition",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.htmlElement')} is specified, this attribute specifies the position where \n the canvas should be inserted relative to the <code>htmlElement</code> in the DOM.",
        name:"htmlPosition",
        setter:["setHtmlPosition"],
        seeAlso:[],
        valueType:"DrawPosition",
        defaultValue:"\"afterBegin\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["htmlElement", "positioning"]
    },
    "method:ListGrid.setEditValues":{
        ref:"method:ListGrid.setEditValues",
        flags:"",
        description:"\n This method sets up a set of editValues for some row / cell.  It differs from \n 'setEditValue()' in that:<br>\n &nbsp;- it takes values for multiple fields<br> \n &nbsp;- it clears out any previous edit values for the record<br>\n\n",
        name:"setEditValues",
        seeAlso:[],
        params:[
            {description:"Row number for the record being edited", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"New values for the row", optional:false, name:"values", defaultValue:null,
             type:"any"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DynamicForm.requiredRightTitlePrefix":{
        ref:"attr:DynamicForm.requiredRightTitlePrefix",
        flags:"IRW",
        description:"The string prepended to the title of every required item in this form if\n highlightRequiredFields is true and the titleOrientation property is set to \"right\".",
        name:"requiredRightTitlePrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"<B>:&nbsp;\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "type:ExportDisplay":{
        ref:"type:ExportDisplay",
        values:[
            {description:"Show the Save As dialog and download the file", value:"\"download\""},
            {description:"Show the data in a new browser window", value:"\"window\""}
        ],
        description:"Method to use for displaying the exported data.",
        name:"ExportDisplay",
        seeAlso:[],
        type:"type",
        serverDS:"allowed"
    },
    "method:FilterBuilder.setCriteria":{
        ref:"method:FilterBuilder.setCriteria",
        flags:"",
        description:" Set new criteria for editing.  \n <P>\n An interface for editing the provided criteria will be generated identically to what happens\n when initialized with ${isc.DocUtils.linkForRef('type:Criteria')}.\n <P>\n Any existing criteria entered by the user will be discarded.  \n \n",
        name:"setCriteria",
        seeAlso:[],
        params:[
            {description:"new criteria.  Pass null or {} to effectively reset the                                    filterBuilder to it's initial state when no criteria are\n                                    specified",
             optional:false, name:"criteria", defaultValue:null, type:"AdvancedCriteria"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "method:ValuesManager.getMembers":{
        ref:"method:ValuesManager.getMembers",
        flags:"",
        description:"  Retrieves an array of pointers to all the members for this valuesManager.\n",
        name:"getMembers",
        seeAlso:[],
        returns:{description:"array of member forms", type:"array"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "method:GridRenderer.getCellRecord":{
        ref:"method:GridRenderer.getCellRecord",
        flags:"",
        description:" Return the record that holds the value for this cell.\n <P>\n Implementing <code>getCellRecord</code> is optional: the actual HTML placed into each\n grid cell comes from <code>getCellValue</code>, and a valid grid can be created without any\n notion of \"records\" at all.\n <p>\n If you do implement <code>getCellRecord</code>, the value you return is passed to you as the\n \"record\" parameter in other methods.\n\n",
        name:"getCellRecord",
        seeAlso:[],
        params:[
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"record for this cell", type:"object"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "method:TextAreaItem.deselectValue":{
        ref:"method:TextAreaItem.deselectValue",
        flags:"",
        description:" If this item currently has focus, clear the current selection. leaving focus in the item.\n Has no effect if the item is undrawn or unfocussed.\n Only applies to text-based items.\n",
        name:"deselectValue",
        seeAlso:[],
        params:[
            {description:"By default the text insertion cursor will be moved to the end of the   current value - pass in this parameter to move to the start instead",
             optional:true, name:"start", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:TextAreaItem"
    },
    "attr:ComboBoxItem.autoFetchData":{
        ref:"attr:ComboBoxItem.autoFetchData",
        flags:"IRA",
        description:"If this combo box retrieves its options from a <code>dataSource</code>, should options\n be fetched from the server when the item is first written out, or should this fetch be\n delayed until the user opens the pickList.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"attr:PickList.optionDataSource", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:Canvas.isDisabled":{
        ref:"method:Canvas.isDisabled",
        flags:"",
        description:" Is this canvas disabled? Note that the disabled state is inherited - this method will return\n true if this widget, or any of its ancestors are marked disabled.\n",
        name:"isDisabled",
        seeAlso:[],
        returns:{description:"true if the widget or any widget above it in the containment hierarchy                      are disabled.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "attr:ListGrid.validateOnChange":{
        ref:"attr:ListGrid.validateOnChange",
        flags:"IRW",
        description:"If true, validation will be perfomed on each edited cell when each editor's \n \"change\" handler is fired.",
        name:"validateOnChange",
        seeAlso:[
            {ref:"attr:ListGridField.validateOnChange", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "group:safeSkinning":{
        ref:"group:safeSkinning",
        title:"Safe Skinning",
        refs:[],
        description:"The skinning mechanism is extremely powerful and gives you the ability to change\n internal functionality of components.  While this is useful for workarounds, you should\n think through any properties you override, considering what will happen with future\n versions of SmartClient, where the defaults may change or be expanded.\n <P>\n The following kinds of overrides are generally very safe:\n <ul>\n <li> Change ${isc.DocUtils.linkForRef('attr:Canvas.styleName','styleName')} or ${isc.DocUtils.linkForRef('attr:Button.baseStyle','baseStyle')} to\n provide a custom CSS style or series of styles\n <li> Change a media path such as the ${isc.DocUtils.linkForRef('attr:Img.src','src')} of the \n ${isc.DocUtils.linkForRef('attr:Window.minimizeButton')}.\n <li> Change the size of any part of the UI that has a fixed pixel size, such as\n the height and width of the ${isc.DocUtils.linkForRef('attr:Window.minimizeButton')}, especially when this is done\n to match the size of media you have created\n <li> Set properties such as ${isc.DocUtils.linkForRef('attr:Button.showRollOver')} that cause a component to\n visually react to more or fewer UI states (disabled, over, down, etc)\n </ul>\n The following should be very carefully considered:\n <ul>\n <li> Adding custom behaviors by passing in event handlers such as \n (eg ${isc.DocUtils.linkForRef('method:Canvas.showContextMenu','showContextMenu()')}).  If future versions of the\n component add more functionality, you may prevent new features from functioning, cause\n them to function only partially, or break.\n <P>\n If you want to ensure that you do not break new functionality added in future SmartClient\n versions, be sure to call ${isc.DocUtils.linkForRef('method:Class.Super','Super()')} for methods you override, and do not\n prevent events from bubbling.\n <P>\n If you want to ensure that <b>only</b> your custom behavior is used if a future version\n of a SmartClient component adds functionality, override all methods involved in the\n interaction, even if your methods do nothing.  For example, for a custom drop\n interaction, override dropOver, dropMove, dropOut and drop, even if you do nothing on\n dropMove().  Then, do not call Super() if there is no superclass behavior required for\n the interaction you've implemented.  Also, for any event handlers (such as drop())\n return false if you consider your code to have completely handled the event (no\n parent component should react).\n </ul>\n The following are not recommended:\n <ul>\n <li> Providing a global ${isc.DocUtils.linkForRef('attr:Canvas.ID','ID')} to a subcomponent (only works once).\n <li> Overriding ${isc.DocUtils.linkForRef('attr:Canvas.backgroundColor')}, ${isc.DocUtils.linkForRef('attr:Canvas.border','border')},\n ${isc.DocUtils.linkForRef('attr:Canvas.margin','margin')}, ${isc.DocUtils.linkForRef('attr:Canvas.padding','padding')}, or in general any single\n attribute otherwise controlled by CSS.  Future SmartClient versions may change the base\n CSS style, rendering your single-property customization senseless.  Change the entire\n CSS style via ${isc.DocUtils.linkForRef('attr:Canvas.styleName','styleName')} instead.\n </ul>",
        name:"safeSkinning",
        seeAlso:[],
        type:"group"
    },
    "attr:GridRenderer.emptyMessageTableStyle":{
        ref:"attr:GridRenderer.emptyMessageTableStyle",
        flags:"IRW",
        description:"CSS styleName for the table as a whole if we're showing the empty message",
        name:"emptyMessageTableStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["emptyMessage"]
    },
    "attr:FormItem.wrapTitle":{
        ref:"attr:FormItem.wrapTitle",
        flags:"IRW",
        description:"If specified determines whether this items title should wrap.\n Overrides ${isc.DocUtils.linkForRef('attr:DynamicForm.wrapItemTitles','wrapItemTitles')} at the DynamicForm level.",
        name:"wrapTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "attr:TreeNode.name":{
        ref:"attr:TreeNode.name",
        flags:"IR",
        description:"Provides a name for the node that is unique among it's immediate siblings, thus allowing a\n unique path to be used to identify the node, similar to a file system.  See\n ${isc.DocUtils.linkForRef('method:Tree.getPath')}.\n <p>\n If the nameProperty is not set on a given node, the ${isc.DocUtils.linkForRef('attr:TreeNode.id')} will be used instead.  If\n this is also missing, ${isc.DocUtils.linkForRef('method:Tree.getName')} and ${isc.DocUtils.linkForRef('method:Tree.getPath')} will auto-generate a\n unique name for you.  Thus names are not required, but if the dataset you are using already\n has usable names for each node, using them can make APIs such as ${isc.DocUtils.linkForRef('method:Tree.find')} more\n useful.  Alternatively, if your dataset has unique ids consider providing those as\n ${isc.DocUtils.linkForRef('attr:TreeNode.id')}.\n <P>\n If a value provided for the nameProperty of a node (e.g. node.name) is not a\n string, it will be converted to a string by the Tree via \"\"+value.\n <p>\n This property is also used as the default title for the node (see ${isc.DocUtils.linkForRef('method:Tree.getTitle')})\n if ${isc.DocUtils.linkForRef('attr:TreeNode.title')} is not specified.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')}.",
        name:"name",
        seeAlso:[
            {ref:"attr:Tree.nameProperty", description:null},
            {ref:"attr:Tree.pathDelim", description:null},
            {ref:"method:Tree.getPath", description:null},
            {ref:"method:Tree.getTitle", description:null}
        ],
        valueType:"String",
        defaultValue:"null, but see below",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:ResultSet.fetchMode":{
        ref:"attr:ResultSet.fetchMode",
        flags:"IRA",
        description:"Mode of fetching records from the server. If unset, will default to <code>\"local\"</code>\n if ${isc.DocUtils.linkForRef('attr:ResultSet.allRows')} is specified, otherwise <code>\"paged\"</code>.",
        name:"fetchMode",
        seeAlso:[
            {ref:"type:FetchMode", description:null}
        ],
        valueType:"FetchMode",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching"]
    },
    "type:DSServerType":{
        ref:"type:DSServerType",
        values:[
            {description:"Saves via Hibernate, by automatically generating a Hibernate configuration based on a\n SmartClient DataSource file (<i>dataSourceID</i>.ds.xml).  See\n ${isc.DocUtils.linkForRef('group:hibernateIntegration')} for details.",
             value:"\"hibernate\""},
            {description:"Uses SmartClient's ${isc.DocUtils.linkForRef('group:sqlDataSource','built-in SQL connectors')} to talk to\n relational databases.  You can use this if your DataSource connects to a relational\n database table.",
             value:"\"sql\""},
            {description:"Requests will be delivered to the server and you are expected to write Java code to\n create a valid response.  Throws an error if the server side method dsRequest.execute()\n is called. This is appropriate if you intend an entirely custom implementation, and you\n want an error thrown if there is an attempt to call an operation you have not\n implemented.",
             value:"\"generic\""}
        ],
        description:"Indicates what the ISC server will do with a DataSource request if you call\n dsRequest.execute() in server code.\n <P>\n If you use a Java-based persistence layer not provided by SmartClient, such as\n Hibernate, EJB or your own custom object model, you don't need to set\n <code>dataSource.serverType</code> and should follow the\n ${isc.DocUtils.linkForRef('group:serverDataIntegration','integration instructions')}.\n <P>\n If you want to use the SmartClient SQL engine for persistence, set\n <code>dataSource.serverType</code> to \"sql\", and see the instructions on\n ${isc.DocUtils.linkForRef('group:sqlDataSource','SQL DataSources')}.",
        name:"DSServerType",
        seeAlso:[],
        type:"type",
        serverDS:"allowed",
        groups:["serverDataIntegration"]
    },
    "classAttr:Validator.mustBeLaterThan":{
        ref:"classAttr:Validator.mustBeLaterThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>dateRange</code> type validator\n  returns false because the value passed in is greater than the specified minimum date.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeLaterThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be later than ${min}\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:CubeGrid.rollupValue":{
        ref:"attr:CubeGrid.rollupValue",
        flags:"IR",
        getter:["getRollupValue"],
        description:"facetValueId of the default rollupValue for each facet.  Can be overriden per facet via\n facet.rollupValue",
        name:"rollupValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"sum\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:DateItem.inputFormat":{
        ref:"attr:DateItem.inputFormat",
        flags:"IRW",
        getter:["getInputFormat"],
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is <code>true</code> this property can be used to specify\n the input format for date strings. \n If unset, the input format will be determined based on the specified\n ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} if possible (see ${isc.DocUtils.linkForRef('method:DateItem.getInputFormat')}), otherwise\n picked up from the Date class (see ${isc.DocUtils.linkForRef('classMethod:Date.setInputFormat')}).\n <P>\n Should be set to a standard ${isc.DocUtils.linkForRef('type:DateInputFormat')} or\n a function which will take a date string as a parameter and return a Javascript Date object.",
        name:"inputFormat",
        seeAlso:[
            {ref:"attr:DateItem.displayFormat", description:null}
        ],
        valueType:"DateInputFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "type:FormItemBaseStyle":{
        ref:"type:FormItemBaseStyle",
        description:"This string is the base css class name applied to a FormItem (or some part of a form item).\n The style name will be modified as the 'state' of the form item changes. Specifically:<ul>\n <li>If ${isc.DocUtils.linkForRef('attr:FormItem.showFocused')} is true, when the form item recieves focus, this\n     style will be have the suffix <code>\"Focused\"</code> appended to it.</li>\n <li>If ${isc.DocUtils.linkForRef('attr:FormItem.showErrorStyle')} is true, if the form item has errors, this\n     style will be have the suffix <code>\"Error\"</code> appended to it.</li>\n <li>If ${isc.DocUtils.linkForRef('attr:FormItem.showDisabled')} is true, when the form item is disabled, this\n     style will be have the suffix <code>\"Disabled\"</code> appended to it.</li></ul>\n So for example if the <code>cellStyle</code> for some form item is set to <code>\"formCell\"</code>\n and <code>FormItem.showFocused</code> is true, when the form item recieves focus, the\n form item's cell will have the <code>\"formCellDisabled\"</code> style applied to it.",
        name:"FormItemBaseStyle",
        seeAlso:[],
        type:"type",
        groups:["appearance"]
    },
    "method:Window.flash":{
        ref:"method:Window.flash",
        flags:"A",
        description:"          Makes the window header flash if it's visible; if there's no header, or the header\n          is hidden, makes the window body flash instead.\n          <p>\n			This method is executed when users click outside the bounds of a modal window\n			so they'll notice that they have to do something with the window.\n",
        name:"flash",
        seeAlso:[],
        type:"method",
        definingClass:"class:Window",
        groups:["modal"]
    },
    "attr:ColorItem.defaultPickerMode":{
        ref:"attr:ColorItem.defaultPickerMode",
        flags:"IR",
        description:"The ${isc.DocUtils.linkForRef('class:ColorPicker')} associated with this ColorItem can operate in either a \"simple\" mode \n (where it displays just the 40 basic colors and allows the user to click one), or a \n \"complex\" mode (where the user can specify a color from anywhere in the spectrum, with \n an optional alpha element). The defaultPickMode attribute specifies which of these two \n modes is in force when the picker first loads.",
        name:"defaultPickerMode",
        seeAlso:["ColorPicker.defaultPickMode()"],
        valueType:"text",
        defaultValue:"\"simple\"",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "method:MenuItem.action":{
        ref:"method:MenuItem.action",
        flags:"",
        description:" Action to fire when this menu is activated.\n",
        name:"action",
        seeAlso:[],
        type:"method",
        definingClass:"object:MenuItem",
        groups:["menuBasic"]
    },
    "attr:FormulaBuilder.field":{
        ref:"attr:FormulaBuilder.field",
        flags:"IR",
        description:"The Field object representing the field being created or edited.",
        name:"field",
        seeAlso:[],
        valueType:"Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "classMethod:Page.setAppFilesDir":{
        ref:"classMethod:Page.setAppFilesDir",
        flags:"",
        description:" Specify the directory for miscellaneous app-specific files <b>other than</b> images, such as\n ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsURL','HTML fragments')}, ${isc.DocUtils.linkForRef('class:ViewLoader','loadable views')}, \n XML or JSON flat data files, videos, etc.\n <P>\n This URL also becomes available via the prefix \"[APPFILES]\" for ${isc.DocUtils.linkForRef('attr:RPCRequest.actionURL')}.\n <P>\n Defaults to the value of ${isc.DocUtils.linkForRef('classMethod:Page.getAppDir')}, that is, the current directory.\n\n",
        name:"setAppFilesDir",
        seeAlso:[],
        params:[
            {description:"New app files URL.", optional:true, name:"URL", defaultValue:null,
             type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "attr:DateChooser.alternateWeekStyles":{
        ref:"attr:DateChooser.alternateWeekStyles",
        flags:"IRW",
        description:"Whether alternating weeks should be drawn in alternating styles. If enabled, the cell style\n for alternate rows will have ${isc.DocUtils.linkForRef('attr:DateChooser.alternateStyleSuffix')} appended to it.",
        name:"alternateWeekStyles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:Menu.fields":{
        ref:"attr:Menu.fields",
        flags:"IRWA",
        description:"Array of columns to show for this menu.<br>\n Standard menu fields may be included by specifying ${isc.DocUtils.linkForRef('type:MenuFieldID','MenuFieldIDs')}\n directly. Additional custom fields may be specifed as ${isc.DocUtils.linkForRef('object:ListGridField')} objects.<br>\n If this property is unset, default behavior will show the\n ${isc.DocUtils.linkForRef('type:MenuFieldID','standard set of fields')}, with the exception of any that have\n been suppressed via ${isc.DocUtils.linkForRef('attr:Menu.showIcons')}, ${isc.DocUtils.linkForRef('attr:Menu.showKeys')} and ${isc.DocUtils.linkForRef('attr:Menu.showSubmenus')}",
        name:"fields",
        seeAlso:[],
        valueType:"Array of MenuFieldID | Array of ListGridFields",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('menuColumns')}"],
        definingClass:"class:Menu"
    },
    "group:emptyMessage":{
        ref:"group:emptyMessage",
        refs:["attr:DetailViewer.showEmptyMessage", "attr:DetailViewer.emptyMessage", 
         "attr:DetailViewer.emptyMessageStyle","attr:DetailViewer.loadingMessage", "attr:DetailViewer.loadingMessageStyle", 
         "attr:GridRenderer.showEmptyMessage","attr:GridRenderer.emptyMessage", "attr:GridRenderer.emptyMessageTableStyle", 
         "attr:GridRenderer.emptyMessageStyle","attr:ListGrid.showEmptyMessage", "attr:ListGrid.emptyMessage", 
         "attr:ListGrid.emptyMessageStyle","attr:ListGrid.loadingMessage"],
        name:"emptyMessage",
        seeAlso:[],
        type:"group"
    },
    "method:Number.toCurrencyString":{
        ref:"method:Number.toCurrencyString",
        flags:"",
        description:" Return this number as a currency-formatted string.\n",
        name:"toCurrencyString",
        seeAlso:[],
        params:[
            {description:"Currency symbol, can be set to an empty string. If unset <code>\"$\"</code> will be used.",
             optional:true, name:"currencyChar", defaultValue:null, type:"string"},
            {description:"Decimal separator symbol. If unset <code>\".\"</code> will be used.",
             optional:true, name:"decimalChar", defaultValue:null, type:"string"},
            {description:"Should decimal portion be padded out to two digits? True  by default.",
             optional:true, name:"padDecimal", defaultValue:null, type:"boolean"},
            {description:"Should currency symbol come at the end of the string?  If unspecified, currency symbol will be shown at the beginning of the string.",
             optional:true, name:"currencyCharLast", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"Currency-formatted string version of the number", type:"string"},
        type:"method",
        definingClass:"object:Number",
        groups:["stringProcessing"]
    },
    "attr:PickList.pickListFields":{
        ref:"attr:PickList.pickListFields",
        flags:"IRA",
        description:"This property allows the developer to specify which field[s] will be displayed in the \n drop down list of options.\n <P>\n Only applies to databound pickLists (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}, or picklists\n with custom data set up via the advanced ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')} method.\n <P>\n If this property is unset, we display the ${isc.DocUtils.linkForRef('attr:PickList.displayField')}, if specified, \n otherwise the ${isc.DocUtils.linkForRef('attr:PickList.valueField')}.\n <P>\n If there are multiple fields, column headers will be shown for each field, the\n height of which can be customized via the ${isc.DocUtils.linkForRef('attr:PickList.pickListHeaderHeight')} attribute.\n <P>\n Each field to display should be specified as a ${isc.DocUtils.linkForRef('object:ListGridField')} object. Note that\n unlike in ${isc.DocUtils.linkForRef('class:ListGrid','listGrids')}, dataSource fields marked as \n ${isc.DocUtils.linkForRef('attr:DataSourceField.hidden','hidden:true')} will be hidden by default in pickLists. To\n override this behavior, ensure that you specify an explicit value for \n ${isc.DocUtils.linkForRef('method:ListGridField.showIf','showIf')}.",
        name:"pickListFields",
        seeAlso:[
            {ref:"attr:PickList.valueField", description:null},
            {ref:"attr:PickList.pickListHeaderHeight", description:null}
        ],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:ColorPicker.opacityText":{
        ref:"attr:ColorPicker.opacityText",
        flags:"IR",
        description:"The text to show underneath the selected color box, so that it can \n be seen through semi-transparent colors. If you do not want such text, set \n this value to blank. This value is irrelevant if \n ${isc.DocUtils.linkForRef('attr:ColorPicker.supportsTransparency')} is false.",
        name:"opacityText",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "group:formValuesManager":{
        ref:"group:formValuesManager",
        refs:["attr:DynamicForm.valuesManager"],
        name:"formValuesManager",
        seeAlso:[],
        type:"group"
    },
    "attr:TreeGrid.folderOpenImage":{
        ref:"attr:TreeGrid.folderOpenImage",
        deprecated:"as part of SmartClient release 5.5 in favor of ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')}",
        flags:"IRW",
        description:"The filename of the default icon for all open folder nodes in this treeGrid.",
        name:"folderOpenImage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "method:TileGrid.exportData":{
        ref:"method:TileGrid.exportData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','DataSource')} to \n retrieve data that matches the current filter and sort criteria for this component, then \n exports the resulting data to a file or window in the requested format.\n <P>\n For more information on exporting data, see ${isc.DocUtils.linkForRef('method:DataSource.exportData')}.\n\n",
        name:"exportData",
        seeAlso:[],
        params:[
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:FormItem.getIconHeight":{
        ref:"method:FormItem.getIconHeight",
        flags:"A",
        description:"	Takes an icon definition object, and returns the height for that icon in px.\n",
        name:"getIconHeight",
        seeAlso:[],
        params:[
            {description:"icon definition object for this item.", optional:false, name:"icon",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"height of the form item icon in px", type:"number"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["sizing"]
    },
    "method:TreeGrid.getCellAlign":{
        ref:"method:TreeGrid.getCellAlign",
        flags:"",
        description:" Return the horizontal alignment for cell contents. Default implementation will always\n left-align the special ${isc.DocUtils.linkForRef('attr:TreeGridField.treeField')} [or right-align if the page is in\n RTL mode] - otherwise will return ${isc.DocUtils.linkForRef('attr:ListGridField.cellAlign')} if specified, otherwise\n ${isc.DocUtils.linkForRef('attr:ListGridField.align')}.\n\n\n",
        name:"getCellAlign",
        seeAlso:[],
        params:[
            {description:"this cell's record", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"Horizontal alignment of cell contents: 'right', 'center', or 'left'",
         type:"Alignment"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:RPCResponse.data":{
        ref:"attr:RPCResponse.data",
        flags:"R",
        description:"The data sent by the server.\n <P>\n When communicating with the SmartClient server, rpcResponse.data is the data passed to the\n server-side method RPCResponse.setData() by your Java code, as translated into JavaScript\n objects by the rules described under ${isc.DocUtils.linkForRef('attr:RPCRequest.data')}.\n <P>\n When not communicating with the SmartClient server (${isc.DocUtils.linkForRef('attr:RPCRequest.useSimpleHttp')} or\n another flag that implies it is set), rpcResponse.data contains the raw HTTP response body.\n For an exception, see ${isc.DocUtils.linkForRef('attr:RPCRequest.evalResult')}.",
        name:"data",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCResponse"
    },
    "attr:TabSet.closeTabIcon":{
        ref:"attr:TabSet.closeTabIcon",
        flags:"IR",
        description:"Default src for the close icon for tabs to display if ${isc.DocUtils.linkForRef('attr:TabSet.canCloseTabs')} is true.",
        name:"closeTabIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"[SKIN]/TabSet/close.png",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:Canvas.animateRectTime":{
        ref:"attr:Canvas.animateRectTime",
        flags:"IRWA",
        description:"Default time for performing an animated setRect.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateRectTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:RichTextEditor.styleControls":{
        ref:"attr:RichTextEditor.styleControls",
        flags:"IRA",
        description:"Default text styling control group. Consists of an array of \n  ${isc.DocUtils.linkForRef('type:ControlName')}s. To display this group of controls for some RichTextEditor,\n  include <code>\"styleControls\"</code> in the ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} array",
        name:"styleControls",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[\"boldSelection\", \"italicSelection\", \"underlineSelection\"]",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "attr:ListGrid.groupIconSize":{
        ref:"attr:ListGrid.groupIconSize",
        flags:"IRW",
        description:"Default width and height of group icons for this ListGrid.",
        name:"groupIconSize",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:ListGrid.recordEditProperty":{
        ref:"attr:ListGrid.recordEditProperty",
        flags:"IRWA",
        description:"Property name on a record that should be checked to determine whether the record may be\n edited.\n <br>\n This property is configurable to avoid possible collision with data values in record.\n With the default setting of \"_canEdit\", a record can be set non-editable by ensuring\n record._canEdit == false.\n <br>\n For controlling editability for the entire grid or for a field, set grid.canEdit or\n field.canEdit.",
        name:"recordEditProperty",
        seeAlso:[
            {ref:"attr:ListGrid.canEdit", description:null},
            {ref:"attr:ListGridField.canEdit", description:null},
            {ref:"method:ListGrid.canEditCell", description:null}
        ],
        valueType:"string",
        defaultValue:"\"_canEdit\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ResultTree.dataSource":{
        ref:"attr:ResultTree.dataSource",
        flags:"IR",
        description:"What ${isc.DocUtils.linkForRef('class:DataSource')} is this resultTree associated with?",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "classAttr:RPCResponse.STATUS_SERVER_TIMEOUT":{
        ref:"classAttr:RPCResponse.STATUS_SERVER_TIMEOUT",
        flags:"R",
        description:"Indicates a request timed out with no server response.\n <p>\n This is a client-only error code - never sent by the server (since it's the server\n that times out).\n <p>\n NOTE that if using <code>hiddenFrame</code> as the transport (not the default), a\n malformed response such as a \"500 Server Error\" or 404 errors will be reported as a\n timeout.",
        name:"STATUS_SERVER_TIMEOUT",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"-100",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:Window.showModalMask":{
        ref:"attr:Window.showModalMask",
        flags:"IR",
        description:"If true, displays a translucent mask over the rest of the page when a modal window\n is displayed.",
        name:"showModalMask",
        seeAlso:[
            {ref:"attr:Window.modalMask", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["modal", "appearance"]
    },
    "group:tableLayout":{
        ref:"group:tableLayout",
        refs:["attr:DynamicForm.numCols", "attr:DynamicForm.fixedColWidths", 
         "attr:DynamicForm.colWidths","attr:DynamicForm.minColWidth", "attr:DynamicForm.cellPadding", 
         "attr:DynamicForm.cellBorder","attr:DynamicForm.sectionVisibilityMode"],
        description:"Manipulating the values stored in the form.",
        name:"tableLayout",
        seeAlso:[],
        type:"group"
    },
    "attr:FormItem.titleErrorClassName":{
        ref:"attr:FormItem.titleErrorClassName",
        deprecated:"As of SmartClient Version 5.5, use ${isc.DocUtils.linkForRef('attr:FormItem.titleStyle')} instead",
        flags:"IRW",
        description:"CSS class for a form item's title when a validation error is showing.",
        name:"titleErrorClassName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"formTitleError\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "attr:Window.contentsType":{
        ref:"attr:Window.contentsType",
        flags:"IR",
        description:"If this window has ${isc.DocUtils.linkForRef('attr:Window.src')} specified, this property can be used to indicate\n whether the source is a standalone HTML page or an HTML fragment.\n <P>\n This is similar to the ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType')} property - be sure to read the\n HTMLFlow documentation to understand circumstances where contentsType:\"page\" is\n <b>unsafe and not recommended</b>.",
        name:"contentsType",
        seeAlso:[
            {ref:"attr:Window.src", description:null}
        ],
        valueType:"string",
        defaultValue:"\"page\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "attr:MathFunction.defaultSortPosition":{
        ref:"attr:MathFunction.defaultSortPosition",
        flags:"IR",
        description:"Indicates the sort-order of this MathFunction in an index returned from static method \n ${isc.DocUtils.linkForRef('classMethod:MathFunction.getDefaultFunctionIndex')}.    Unlike , the result\n is an index of  to return\n a list of",
        name:"defaultSortPosition",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"-1",
        type:"attr",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "attr:SelectItem.pickListFields":{
        ref:"attr:SelectItem.pickListFields",
        flags:"IRA",
        description:"This property allows the developer to specify which field[s] will be displayed in the \n drop down list of options.\n <P>\n Only applies to databound pickLists (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}, or picklists\n with custom data set up via the advanced ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')} method.\n <P>\n If this property is unset, we display the ${isc.DocUtils.linkForRef('attr:PickList.displayField')}, if specified, \n otherwise the ${isc.DocUtils.linkForRef('attr:PickList.valueField')}.\n <P>\n If there are multiple fields, column headers will be shown for each field, the\n height of which can be customized via the ${isc.DocUtils.linkForRef('attr:PickList.pickListHeaderHeight')} attribute.\n <P>\n Each field to display should be specified as a ${isc.DocUtils.linkForRef('object:ListGridField')} object. Note that\n unlike in ${isc.DocUtils.linkForRef('class:ListGrid','listGrids')}, dataSource fields marked as \n ${isc.DocUtils.linkForRef('attr:DataSourceField.hidden','hidden:true')} will be hidden by default in pickLists. To\n override this behavior, ensure that you specify an explicit value for \n ${isc.DocUtils.linkForRef('method:ListGridField.showIf','showIf')}.",
        name:"pickListFields",
        seeAlso:[
            {ref:"attr:SelectItem.valueField", description:null},
            {ref:"attr:PickList.pickListHeaderHeight", description:null}
        ],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('relatedRecords')}"],
        definingClass:"class:SelectItem"
    },
    "attr:ResultSet.useClientFiltering":{
        ref:"attr:ResultSet.useClientFiltering",
        flags:"IRWA",
        description:"Whether to filter data locally when all DataSource records have been loaded (that is,\n criteria is blank and cache is complete).\n <P>\n This may need to be disabled if client-side filtering differs from server-side filtering\n in a way that affects functionality or is surprising.\n <P>\n This setting is distinct from <code>fetchMode:\"local\"</code>, which explicitly loads all\n available DataSource records up front.\n <P>\n See ${isc.DocUtils.linkForRef('method:ResultSet.applyFilter')} for default filtering behavior.\n <P>\n <b>NOTE:</b> even with useClientFiltering false, client-side filtering will be used\n during cache sync to determine if an updated or added row matches the current criteria.\n To avoid relying on client-side filtering in this case, either:<br>\n - avoid returning update data when the updated row doesn't match the current filter<br>\n - set dropCacheOnUpdate<br>",
        name:"useClientFiltering",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "method:SummaryBuilder.save":{
        ref:"method:SummaryBuilder.save",
        flags:"",
        description:" Call to finish working, test the formula and call \n ${isc.DocUtils.linkForRef('method:FormulaBuilder.fireOnClose','fireOnClose()')}.  Called automatically\n when the Save button is clicked.\n\n",
        name:"save",
        seeAlso:[],
        type:"method",
        definingClass:"class:SummaryBuilder",
        groups:["formulaFields"]
    },
    "method:DynamicForm.getValue":{
        ref:"method:DynamicForm.getValue",
        flags:"",
        description:"  Returns the value stored in the form for some field.\n  Shorthand for dynamicForm.getValues()[fieldName];\n",
        name:"getValue",
        seeAlso:[],
        params:[
            {description:"name of the field for which you're retrieving a value. Nested          values may be retrieved by passing in a ${isc.DocUtils.linkForRef('type:dataPath')}",
             optional:false, name:"fieldName", defaultValue:null, type:"string"}
        ],
        returns:{description:"value of the field", type:"any"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dateItem')}"],
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ListGridField.iconWidth":{
        ref:"attr:ListGridField.iconWidth",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGridField.icon')} is specified, this property can be used to specify the \n width of the icon to be displayed in the ListGrid header button. \n (See ${isc.DocUtils.linkForRef('attr:StatefulCanvas.iconWidth')})<br>\n If this field is editable, and ${isc.DocUtils.linkForRef('attr:ListGridField.editorIconWidth')} is unset, this \n property will be passed onto the editors for this field as ${isc.DocUtils.linkForRef('attr:FormItem.iconWidth')}, \n which will effect the default size for ${isc.DocUtils.linkForRef('attr:ListGridField.icons','icons')} displayed \n in the editor.",
        name:"iconWidth",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null},
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:ValuesManager.editRecord":{
        ref:"method:ValuesManager.editRecord",
        flags:"",
        description:"\n Edit an existing record.  Updates this editors values to match the values of the record \n passed in, via ${isc.DocUtils.linkForRef('method:ValuesManager.setValues')}.\n <P>\n This method will also call ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')} to ensure \n subsequent calls to <code>saveData()</code> will use an <code>update</code> rather than\n an <code>add</code> operation.\n\n",
        name:"editRecord",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"the record to be edited as a map of field names to their corresponding values",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "classAttr:Time.AMIndicator":{
        ref:"classAttr:Time.AMIndicator",
        flags:"RWA",
        description:"String appended to times to indicate am (when not using 24 hour format).",
        name:"AMIndicator",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\" am\"",
        type:"classAttr",
        definingClass:"class:Time",
        groups:["i18nMessages"]
    },
    "method:ColumnTree.filterData":{
        ref:"method:ColumnTree.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DynamicForm.showTitlesWithErrorMessages":{
        ref:"attr:DynamicForm.showTitlesWithErrorMessages",
        flags:"IRW",
        description:"Indicates whether on validation failure, the error message displayed to the\n          user should be prepended with the title for the item.",
        name:"showTitlesWithErrorMessages",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "method:DetailViewerField.showIf":{
        ref:"method:DetailViewerField.showIf",
        flags:"",
        description:"\n If specified on a field, this method is evaluated at draw time to determine whether or\n not to show this particular field.\n <p>\n This method can be specified either as a function or a string that will be\n auto-converted to a function.\n\n",
        name:"showIf",
        seeAlso:[],
        params:[
            {description:"The DetailViewer", optional:false, name:"viewer", defaultValue:null,
             type:"DetailViewer"},
            {description:null, optional:false, name:"valueList", defaultValue:null,
             type:"List of DetailViewerRecord"}
        ],
        returns:{description:"true to show the field, false to not show it.", type:"boolean"},
        type:"method",
        definingClass:"object:DetailViewerField"
    },
    "classAttr:ValidatorTypes.isFloat":{
        ref:"classAttr:ValidatorTypes.isFloat",
        flags:"IR",
        description:"Tests whether the value for this field is a valid floating point number.",
        name:"isFloat",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:CubeGrid.chartConfirmThreshold":{
        ref:"attr:CubeGrid.chartConfirmThreshold",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('method:CubeGrid.makeChart')} is called with a chart specification that will show more than\n <code>chartConfirmThreshold</code> data elements, the user will be presented with a\n ${isc.DocUtils.linkForRef('classMethod:isc.confirm','confirmation dialog')}.\n <P>\n Set to 0 to disable this confirmation.",
        name:"chartConfirmThreshold",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"2000",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:List.addListAt":{
        ref:"method:List.addListAt",
        flags:"",
        description:" Add list of items list to this array at item pos.  All items after array[pos] will slide down to\n fit new items.\n\n",
        name:"addListAt",
        seeAlso:[],
        params:[
            {description:"new array of items", optional:false, name:"list", defaultValue:null,
             type:"array"},
            {description:"position in this list to put the new items", optional:false, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"the list of items that was added", type:"array"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:FormulaBuilder.helpIcon":{
        ref:"attr:FormulaBuilder.helpIcon",
        flags:"IRA",
        description:"Icon that appears after the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.formulaField')}, showing help on hover.",
        name:"helpIcon",
        seeAlso:[],
        valueType:"AutoChild FormItemIcon",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "group:i18n":{
        ref:"group:i18n",
        title:"Internationalization and Localization (i18n,l10n)",
        treeLocation:["Concepts"],
        refs:[],
        description:"<b><u>Internationalizing SmartClient UI Components</u></b>\n <p>\n Internationalizing a SmartClient application involves creating locale-specific\n versions of all strings, images, and possibly colors.  In some cases, component layout may\n be affected as well (for example date field order differs between some locales).\n <p>\n <u><b>Character Encodings</b></u>\n <p>\n In order to deliver content in different languages to the browser and have it be displayed\n correctly, it is important that the browser and server agree on the character encoding being\n used to send the page.  \n <p>\n Generally speaking, you can use whatever character encoding you prefer, so long as\n you're consistent about serving your files to the browser with exactly the same encoding\n as was used to save the file to disk.  Keep in mind though that the character encoding you\n choose must be supported by all browsers you intend to support.  Isomorphic recommends that,\n where possible, you use the UTF-8 encoding.  Regardless of which character encoding you\n choose, keep in mind that for the browser to correctly display non-ASCII characters, you\n <b>must</b> explicitly set the character encoding of your page to match the encoding you\n used to save the file.  Browsers have built-in heuristics to guess the character encoding,\n but they can't be relied upon to work 100% of the time.\n <p>\n There are two ways to explicitly tell the browser what character\n encoding is being used for a given page.  One is to set the \"Content-Type\" header - e.g. as\n follows in a raw HTTP response to set UTF-8 character encoding and a text/html mime type:\n <pre>\n Content-Type: text/html; charset=UTF-8\n </pre>\n If you're using Java on the back-end, you can set this header out of JSP as follows:\n <pre>\n &lt;%@ page contentType=\"text/html; charset=UTF-8\"%&gt;\n </pre>\n Or using Java Servlet APIs in a Servlet as follows:\n <pre>\n response.setContentType(\"text/html; charset=UTF-8\");\n </pre>\n Note that the latter needs to be done before any content is written to the response output\n stream.\n <p>\n The other approach to setting the content encoding is to use an HTML META tag to embed it\n in the page itself (note that this applies to HTML documents only).  The META tag must go\n inside the &lt;HEAD&gt; HTML element - e.g. as follows:\n <pre>\n &lt;HEAD&gt;\n &lt;META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\"&gt;\n &lt;/HEAD&gt;\n </pre>\n Isomorphic recommends that you use <b>both</b> of these approaches for maximum\n compatibility.  For example, if you omit the META tag approach, documents saved by the user\n using the browser save function may not render correctly when subsequently read from disk\n because HTTP headers are not available in that context.  And conversely not providing the\n HTTP header can result in application servings applying their own default and incorrect\n content encoding.\n <p>\n If you're using a given character encoding pervasively in your pages, you can also configure\n your web server or application server to use that character encoding as the default for all\n pages of a given mime type or some other criteria (depending on the capability of your\n server) - for example on a per-directory basis.\n <p>\n For more information on character encodings, character sets, and working with HTML, please\n see W3C's tutorial here:\n <a href='http://www.w3.org/International/tutorials/tutorial-char-enc/' onclick=\"window.open('http://www.w3.org/International/tutorials/tutorial-char-enc/');return false;\">http://www.w3.org/International/tutorials/tutorial-char-enc/</a>\n <p>\n <u><b>String localization</b></u>\n <p>\n All text appearing in SmartClient UI components is customizeable at the component prototype\n level and at the instance level.  For example, Button component has a default title of\n \"Untitled Button\" that is changed at the instance level when the button is created (using\n JavaScript syntax):\n <pre>\n isc.Button.create({ title: \"Click me!\"});\n </pre>\n Likewise, the default title can be changed like so:\n <pre>\n isc.Button.create({});  // this produces a button with the title \"Untitled Button\"\n isc.Button.addProperties({ title: \"Titled Button\"});\n isc.Button.create({});  // this now produces a button with the title \"Titled Button\"\n </pre>\n Using these mechanisms, you can set locale-specific text on any SmartClient component.\n <p>\n We'll use Java-based internationalization examples here because the SDK ships with a Java\n backend - but note that Java is not required to internationalize your SmartClient\n application - see the \"Resources\" section at the end of this overview for links to\n internationalization resources of other popular back-end technologies.  All those\n techonologies provide mechanisms for fetching locale-specific strings that are compatible\n with SmartClient because ultimately internationalizing a SmartClient application is not much\n different from internationalizing an HTML page.\n <p>\n So, to give a Java-based example, let's say we want to create a button on a page in a manner\n that permits future localization.  Let's say we currently have the following code in our JSP\n page: \n <pre>\n &lt;SCRIPT&gt;\n isc.Button.create({ title: \"Click me!\" });\n &lt;/SCRIPT&gt;\n </pre>\n One standards-based approach is to use the i18n component of the JSTL Core package (Java\n Server Pages Standard Tag Library) to replace that static \"Click me!\" text with a localized\n reference.  This can be done as follows:\n <pre>\n &lt;%@ taglib prefix=\"fmt\" uri=\"/WEB-INF/fmt.tld\" %&gt;\n &lt;SCRIPT&gt;\n isc.Button.create({ title: &lt;fmt:message key=\"buttonTitle\"&gt; });\n &lt;/SCRIPT&gt;\n </pre>\n Assuming you've set the current Locale and provided a resource bundle with the relevant\n translated text, the above would create a button with a title that comes from the locale-specific\n bundle you provided.  See Sun's i18n tutorial:\n <a href='http://java.sun.com/docs/books/tutorial/i18n/' onclick=\"window.open('http://java.sun.com/docs/books/tutorial/i18n/');return false;\">http://java.sun.com/docs/books/tutorial/i18n/</a> and the JSTL home page:\n <a href='http://java.sun.com/products/jsp/jstl/' onclick=\"window.open('http://java.sun.com/products/jsp/jstl/');return false;\">http://java.sun.com/products/jsp/jstl/</a> for more information on how to set up\n Resource Bundles.\n <p>\n If you're developing in XML, the mechanism is much the same.  Let's say we have the following\n XML code in our JSP:\n <pre>\n &lt;isomorphic:XML&gt;\n &lt;Button title=\"Click me!\"/&gt;\n &lt;/isomorphic:XML&gt;\n </pre>\n We can localize this button by using JSTL as before by writing it like this:\n <pre>\n &lt;%@ taglib prefix=\"fmt\" uri=\"/WEB-INF/fmt.tld\" %&gt;\n &lt;isomorphic:XML&gt;\n &lt;Button&gt;\n   &lt;title&gt;&lt;fmt:message key=\"buttonTitle\"&gt;&lt;/title&gt;\n &lt;/Button&gt;\n &lt;/isomorphic:XML&gt;\n </pre>\n Note that the XML must be well-formed so we can't just replace \"Click me!\" with the\n <code>fmt</code> tag because we would end up with embedded double quotes.\n <p>\n NOTE: even though you are dynamically adding localized data to your presentation using\n JSTL/JSPs, you can still allow the browser to cache the entire presentation layer (described\n in detail ${isc.DocUtils.linkForRef('group:smartArchitecture','here')}.  Your bootstrap JSP (the application start\n page) can use a &lt;SCRIPT SRC=&gt; tag to load the localized presentation code as\n cacheable JavaScript, for example, &lt;SCRIPT SRC=/mainUI.jsp?locale=[localeName]&gt;.  This\n works with both XML and JS component descriptors since both are ultimately delivered to the\n browser as JavaScript.\n <p>\n <u><b>Framework message localization</b></u>\n <p>\n In addition to titles and other strings displayed on SmartClient widget instances, \n SmartClient displays a number of standard prompts and error messages in various cases, all\n of which are fully customizeable for localization purposes.<br>\n A complete list of these standard messages is presented ${isc.DocUtils.linkForRef('group:i18nMessages','here')}.<br>\n Customizing these standard messages is very similar to customizing any other strings, such\n as the <code>Button.title</code> attribute described above.  To take an example, the \n ${isc.DocUtils.linkForRef('classAttr:RPCManager.timeoutErrorMessage')} is displayed to the user in a warning dialog\n when a SmartClient operation fails to complete. By default this is set to \n <code>\"Operation Timed Out\"</code> but can be modified using the ${isc.DocUtils.linkForRef('classMethod:Class.addClassProperties')} \n method, as follows:\n <pre>\n isc.RPCManager.addClassProperties({timeoutErrorMessage:\"Custom Timeout Message\"});\n </pre>\n <p>\n <u><b>DataSource localization</b></u>\n <p>\n DataSources can be created in ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','several ways')}.  DataSources\n created directly in JavaScript can be internationalized via the techniques described above.\n DataSources which are declared in XML (.ds.xml files) and are read by the ISC server, which\n are normally loaded into a .jsp page via the <code>&lt;isomorphic:loadDS&gt;</code> JSP tag,\n can instead be loaded and interpreted as .jsp files via the technique described below.  This\n allows JSTL and other JSP tags to be used to internationalize the titles and validation\n error messages in XML DataSources.\n <P>\n For example, given the following DataSource located in /shared/ds/supplyItem.ds.xml:\n <pre>\n &lt;DataSource&gt;\n     &lt;fields&gt;\n         &lt;field name=\"itemName\"&gt;\n             &lt;title&gt;Item Name&lt;/title&gt;\n             &lt;validators&gt;\n                 &lt;Validator type=\"lengthRange\" max=\"40\"&gt;\n                     &lt;errorMessage&gt;Must be 40 characters or less.&lt;/errorMessage&gt;\n                 &lt;/Validator&gt;\n             &lt;/validators&gt;\n         &lt;/field&gt;\n     &lt;/fields&gt;\n &lt;/DataSource&gt;\n </pre>\n To localize the title and validator error string of the <code>itemName</code> field \n via the same JSTL strategy we've been using, first add the following to your web.xml to\n allow DataSource files to be interpreted as JSPs:\n <pre>\n &lt;jsp-config&gt;\n     &lt;jsp-property-group&gt;\n       &lt;url-pattern&gt;/shared/ds/*&lt;/url-pattern&gt;\n     &lt;/jsp-property-group&gt;     \n &lt;/jsp-config&gt;\n </pre>\n Next change the DataSource definition as follows:\n <pre>\n &lt;!--\n &lt;%@ taglib prefix=\"fmt\" uri=\"/WEB-INF/fmt.tld\" %&gt;\n --&gt;\n &lt;DataSource xmlns:fmt=\"urn:jsptld:/WEB-INF/fmt.tld\"&gt;\n     &lt;fields&gt;\n         &lt;field name=\"itemName\"&gt;\n             &lt;title&gt;&lt;fmt:message key=\"itemTitle\"&gt;&lt;/title&gt;\n             &lt;validators&gt;\n                 &lt;Validator type=\"lengthRange\" max=\"40\"&gt;\n                     &lt;errorMessage&gt;&lt;fmt:message key=\"itemLengthRangeValidator\"/&gt;&lt;/errorMessage&gt;\n                 &lt;/Validator&gt;\n             &lt;/validators&gt;\n         &lt;/field&gt;\n     &lt;/fields&gt;\n &lt;/DataSource&gt;\n </pre>\n Note that the XML comment around the taglib declaration is intentional.  It is there to make\n sure the JSP parser sees the tag library declaration, while the file remains valid XML.\n If you need to use multiple JSP tag libraries to achieve your goals, simply add additional\n taglib declarations inside the XML comment and be sure to register the tag namespace in the\n DataSource tag via <code>xmlns:tagName</code> as done above for the <code>fmt</code>\n namespace. Instead of using the <code>&lt;isomorphic:loadDS&gt;</code> JSP tag to load this\n DataSource, you would load it as follows in your JSP:\n <pre>\n &lt;SCRIPT&gt;\n &lt;isomorphic:XML&gt;\n &lt;jsp:include page=\"/shared/ds/supplyItem.ds.xml\"&gt;&lt;/jsp:include&gt;\n &lt;/isomorphic:XML&gt;\n &lt;/SCRIPT&gt;\n </pre> \n This makes it possible to internationalize field titles as well as validation error messages\n for built-in validators.  To internationalize custom server-side validation errors, simply\n provide internationalized strings when calling <code>DSResponse.setErrorReport()</code> to\n report validation errors (see the JavaDoc for that documentation).\n <p>\n <u><b>Support for Right-to-Left (RTL) languages</b></u>\n <P>\n SmartClient includes beta quality support for RTL langauges, with known issues in certain\n skins and components.  To enable, simply set <code>dir=\"rtl\"</code> on the HTML element:\n <pre>\n    &lt;HTML dir=\"rtl\"&gt;\n </pre>\n ListGrid columns, horizontal layouts, scrolling directions, etc will reverse order\n automatically.\n <P>\n If you need production-quality RTL support for your application, visit the<a\n href=\"http://forums.smartclient.com\">SmartClient forums</a> for details of \n known limitations.\n <P>\n <u><b>Image, CSS localization</b></u>\n <p>\n Most SmartClient components use a mixture of text, CSS and images to render.  If you wish to\n provide locale-specific images or use locale-specific CSS, you can create localized copies\n of a SmartClient skin named after the locale and modify images as you see fit.  A skin is\n specified at load time using either the <code>skin</code> attribute of the\n <code>&lt;isomorphic:loadISC&gt;</code> tag or by loading the skin\n directly using a script tag like so: &lt;SCRIPT SRC=/isomorphic/skins/[skin]/load_skin.js&gt;.\n If you're using the <code>loadISC</code> tag, you can specify a locale-specific skin like so:\n <pre>\n &lt;isomorphic:loadISC skin=\"[localeName]\"/&gt;\n </pre>\n Or you can use a script tag like so:\n <pre>\n &lt;SCRIPT SRC=/isomorphic/skins/[localeName]/load_skin.js&gt;&lt;/SCRIPT&gt;\n </pre>\n Where the <code>[localeName]</code> is the name of the current locale - you'll have this in the\n current page context as you'll need it to use the JSTL &lt;fmt:setLocale&gt; tag.\n\n Resources:\n <p>\n <b><u>Java</u></b>\n <ul>\n <li>Sun's i18n tutorial: <a href='http://java.sun.com/docs/books/tutorial/i18n/' onclick=\"window.open('http://java.sun.com/docs/books/tutorial/i18n/');return false;\">http://java.sun.com/docs/books/tutorial/i18n/</a>\n <li>JSTL home page: <a href='http://java.sun.com/products/jsp/jstl/' onclick=\"window.open('http://java.sun.com/products/jsp/jstl/');return false;\">http://java.sun.com/products/jsp/jstl/</a>\n <li>Apache JSTL \"Getting Started\": \n     <a href='http://jakarta.apache.org/taglibs/doc/standard-doc/standard/GettingStarted.html' onclick=\"window.open('http://jakarta.apache.org/taglibs/doc/standard-doc/standard/GettingStarted.html');return false;\">http://jakarta.apache.org/taglibs/doc/standard-doc/standard/GettingStarted.html</a>\n <li>Apache taglibs: <a href='http://jakarta.apache.org/taglibs/' onclick=\"window.open('http://jakarta.apache.org/taglibs/');return false;\">http://jakarta.apache.org/taglibs/</a>\n </ul>\n <p>\n\n <b><u>.NET</u></b>\n <ul>\n <li>MSDN Developer Resource: \"Developing World-Ready Applications\":\n <a href='http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpcondesigningglobalapplications.asp' onclick=\"window.open('http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpcondesigningglobalapplications.asp');return false;\">http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpcondesigningglobalapplications.asp</a>\n </ul>\n\n <b><u>PHP</u></b>\n <ul>\n <li>PHP Gettext manual: <a href='http://us2.php.net/gettext' onclick=\"window.open('http://us2.php.net/gettext');return false;\">http://us2.php.net/gettext</a>\n <li>O'Reilly Gettext tutorial: <a href='http://www.onlamp.com/pub/a/php/2002/06/13/php.html' onclick=\"window.open('http://www.onlamp.com/pub/a/php/2002/06/13/php.html');return false;\">http://www.onlamp.com/pub/a/php/2002/06/13/php.html</a>\n </ul>\n <p>\n\n <b><u>Perl</u></b>\n <ul>\n <li>Perl, Unicode and i18n FAQ: <a href='http://rf.net/~james/perli18n.html' onclick=\"window.open('http://rf.net/~james/perli18n.html');return false;\">http://rf.net/~james/perli18n.html</a>\n </ul>",
        name:"i18n",
        seeAlso:[],
        type:"group"
    },
    "attr:Calendar.monthViewTitle":{
        ref:"attr:Calendar.monthViewTitle",
        flags:"IR",
        description:"The title for the month view",
        name:"monthViewTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Month\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "method:DataBoundComponent.selectAllRecords":{
        ref:"method:DataBoundComponent.selectAllRecords",
        flags:"",
        description:" Select all records\n\n",
        name:"selectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "attr:Class.addPropertiesOnCreate":{
        ref:"attr:Class.addPropertiesOnCreate",
        flags:"RA",
        description:"Controls whether arguments passed to ${isc.DocUtils.linkForRef('classMethod:Class.create')} are assumed to be\n Objects containing properties that should be added to the newly created instance.  This\n behavior is how <code>create()</code> works with almost all SmartClient widgets and\n other components, allowing the convenient shorthand of setting a batch of properties via\n an ${isc.DocUtils.linkForRef('type:ObjectLiteral','JavaScript Object Literal')} passed to create().\n <P>\n The setting defaults to true if unset.  To disable this behavior for a custom class,\n such that <code>create()</code> works more like typical constructors found in Java and\n other languages, use:\n <pre>\n     isc.[i]ClassName[/i].addProperties({ addPropertiesOnCreate:false })\n </pre>\n <P>\n Note that it is not valid to disable this behavior for any subclass of ${isc.DocUtils.linkForRef('class:Canvas')}\n (Canvas relies on this property).\n <p>\n Regardless of the setting for <code>addPropertiesOnCreate</code>, all arguments passed to\n ${isc.DocUtils.linkForRef('classMethod:Class.create')} are still passed on to ${isc.DocUtils.linkForRef('method:Class.init')}.",
        name:"addPropertiesOnCreate",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"undefined",
        type:"attr",
        definingClass:"class:Class"
    },
    "attr:MultiSortDialog.deleteLevelButtonTitle":{
        ref:"attr:MultiSortDialog.deleteLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the deleteLevelButton",
        name:"deleteLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Delete Level\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "classMethod:Page.setEvent":{
        ref:"classMethod:Page.setEvent",
        flags:"",
        description:" Register to be called whenever a given type of event occurs, at the page level.\n <p>\n This includes events that also occur on widgets (like \"click\") and events that only occur at\n the page level (\"resize\" and \"load\").\n <p>\n For events that also occur on widgets, page level event registrations will fire BEFORE the\n event handlers on Canvases.   If your action returns <code>false</code>, this will prevent\n the event from getting to the intended Canvas.\n <p>\n Capturing events on widgets can be done by setting one of the event methods available on Canvas\n (and hence available to all widget classes).  See ${isc.DocUtils.linkForRef('group:widgetEvents')}.\n\n",
        name:"setEvent",
        seeAlso:[
            {ref:"class:EventHandler", description:null},
            {ref:"classMethod:EventHandler.getX", description:null},
            {ref:"classMethod:EventHandler.getY", description:null}
        ],
        params:[
            {description:"event type to register for (\"mouseDown\", \"load\", etc)", optional:false,
             name:"eventType", defaultValue:null, type:"pageEvent"},
            {description:"string to be eval'd when event fires						(function)			function to be executed when event fires\n                      (object)            an object to call on which a method named \"page\" +\n                                          eventType will be called",
             optional:false, name:"action", defaultValue:null, type:"string"},
            {description:"Flag to set automatic removal of the event after												it fires one or more times",
             optional:true, name:"fireStyle", defaultValue:null, type:"FireStyle"},
            {description:"optional - if an object was passed in as the second                                          parameter, this is a name of a method to call on that\n                                          object.",
             optional:true, name:"functionName", defaultValue:null, type:"string"}
        ],
        returns:{description:"ID number of this event, may be used to remove the event later										via a call to <code>Page.clearEvent()</code>",
         type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["EventRegistry"]
    },
    "method:FormulaBuilder.setFormula":{
        ref:"method:FormulaBuilder.setFormula",
        flags:"",
        description:" Call to set the formula-string in this FormulaBuilder.\n <P>\n Note that calling setFormula() will update the UI, generate the formula's funtion and \n test it automatically.\n\n",
        name:"setFormula",
        seeAlso:[],
        params:[
            {description:"The new formula-string for this builder", optional:false, name:"",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "group:date":{
        ref:"group:date",
        refs:["attr:Calendar.chosenDate", "attr:Calendar.firstDayOfWeek", "attr:Calendar.workdayStart", 
         "attr:Calendar.workdayEnd"],
        name:"date",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.hideClickMask":{
        ref:"method:Canvas.hideClickMask",
        flags:"",
        description:" Hides the click mask associated with this canvas.\n",
        name:"hideClickMask",
        seeAlso:[
            {ref:"method:Canvas.showClickMask", description:null}
        ],
        params:[
            {description:"optional ID of specific clickMask to hide. If not passed,                       defaults to hiding the click mask associated with this widget only.",
             optional:true, name:"ID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["clickMask"]
    },
    "attr:Splitbar.hResizeCursor":{
        ref:"attr:Splitbar.hResizeCursor",
        flags:"IR",
        description:"Cursor to display if this Splitbar is to be used for horizontal resize of widgets.",
        name:"hResizeCursor",
        seeAlso:[],
        valueType:"Cursor",
        defaultValue:"\"col-resize\"",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["cursor"]
    },
    "attr:Button.iconSize":{
        ref:"attr:Button.iconSize",
        flags:"IR",
        description:"Size in pixels of the icon image.\n <P>\n The <code>iconWidth</code> and <code>iconHeight</code> properties can be used to\n configure width and height separately.",
        name:"iconSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:RPCResponse.httpResponseCode":{
        ref:"attr:RPCResponse.httpResponseCode",
        flags:"R",
        description:"This attribute (avialable when using the the <code>xmlHttpRequest</code> transport) contains\n the HTTP response code sent by the server.\n <p>\n Note that this is different from ${isc.DocUtils.linkForRef('attr:RPCResponse.status')} - that attribute is used to\n indicate a status code for the RPC itself whereas httpResponseCode is the raw HTTP response\n code for the HTTP request that contained the RPCRequest.\n <p>\n This feature relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.\n <p>\n If you're using this attribute, you'll typically want to avoid the default error\n handling response of RPCManager.  To do so, set\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError')} to <code>true</code>.",
        name:"httpResponseCode",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCResponse"
    },
    "attr:EdgedCanvas.customEdges":{
        ref:"attr:EdgedCanvas.customEdges",
        flags:"IR",
        description:"Array of side names (\"T\", \"B\", \"L\", \"R\") specifying which sides of the decorated\n component should show edges.  For example: <pre>\n      customEdges : [\"T\", \"B\"]\n </pre>\n .. would show edges only on the top and bottom of a component.\n <P>\n The default of <code>null</code> means edges will be shown on all sides.",
        name:"customEdges",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "method:FormItem.getValueFieldName":{
        ref:"method:FormItem.getValueFieldName",
        flags:"",
        description:" Getter method to retrieve the ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} for this item.\n If unset, default behavior will return the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this field.\n",
        name:"getValueFieldName",
        seeAlso:[],
        returns:{description:"fieldName to use a \"value field\" in records from this items               ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}",
         type:"string"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "method:ListGrid.keyPress":{
        ref:"method:ListGrid.keyPress",
        flags:"",
        description:" Handle a keyPress event on the ListGrid as a whole.  \n <P>\n Note that the majority of keyboard handling for a ListGrid is performed by\n ${isc.DocUtils.linkForRef('method:ListGrid.bodyKeyPress')} and most overrides are better performed there.\n\n",
        name:"keyPress",
        seeAlso:[],
        params:[
            {description:"the event details", optional:false, name:"event", defaultValue:null,
             type:"ISCEvent"}
        ],
        returns:{description:"return value for the keyPress event.", type:"EventReturn"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:SectionStack.canTabToHeaders":{
        ref:"attr:SectionStack.canTabToHeaders",
        flags:"IRA",
        description:"If true, the headers for the sections (if shown) will be included in the page's tab\n order for accessibility.",
        name:"canTabToHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "attr:ListGrid.editPendingBaseStyle":{
        ref:"attr:ListGrid.editPendingBaseStyle",
        flags:"RA",
        description:"A base name for the CSS class applied to cells containing pending (unsaved) edits<br>\n  As with the default 'baseStyle' property, this style will have \"Dark\", \"Over\", \"Selected\", \n  or \"Disabled\" appended to it according to the state of the cell.\n  If this property is null, cells with pending edits will pick up custom css text to\n  be applied on top of the normal base style from <code>this.editPendingCSSText</code>",
        name:"editPendingBaseStyle",
        seeAlso:[
            {ref:"attr:ListGrid.baseStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:SectionStack.sectionHeaderClass":{
        ref:"attr:SectionStack.sectionHeaderClass",
        flags:"IRA",
        description:"Name of the Canvas subclass to use as a header that labels the section and allows\n showing and hiding.  The default class can be skinned, or trivial subclasses created to\n allow different appearances for headers in different SectionStacks.\n <P>\n Very advanced developers can use the following information to create custom header\n classes.\n <P>\n The SectionStack will instantiate this class, giving the following properties on init:\n <ul>\n <li><code>layout</code>: the SectionStack\n <li><code>expanded</code>: true or false\n <li><code>hidden</code>: true or false\n <li><code>title</code>: section title\n </ul>\n From then on, when the sectionHeader is clicked on, it should call\n ${isc.DocUtils.linkForRef('method:SectionStack.sectionHeaderClick')}.\n <br>\n Whenever the section is hidden or shown, sectionHeader.setExpanded(true|false) will be\n called if implemented.",
        name:"sectionHeaderClass",
        seeAlso:[],
        valueType:"Classname",
        defaultValue:"\"SectionHeader\"",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "group:data":{
        ref:"group:data",
        refs:["method:Calendar.setData", "method:ListGrid.setData", "method:ListGrid.getCellValue", 
         "method:ListGrid.getDragData","method:Menu.setData", "method:Menu.setItems", "method:TileLayout.getDragData", 
         "method:TileGrid.getDragData","method:DataBoundComponent.getDragData", "object:CalendarEvent", "attr:Calendar.data", 
         "attr:ListGrid.data","attr:ListGridField.name", "attr:TreeGrid.data", "attr:Menu.data", "attr:Menu.items", 
         "attr:TileGrid.data","attr:ColumnTree.data"],
        description:null,
        name:"data",
        seeAlso:[],
        type:"group"
    },
    "method:ResultSet.setSort":{
        ref:"method:ResultSet.setSort",
        flags:"",
        description:" Sort this ResultSet by a list of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s. \n <P>\n\n",
        name:"setSort",
        seeAlso:[],
        params:[
            {description:"List of ${isc.DocUtils.linkForRef('object:SortSpecifier')} objects, one    per sort-field and direction",
             optional:false, name:"sortSpecifiers", defaultValue:null, type:"Array of SortSpecifier"}
        ],
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:FormulaBuilder.cancelled":{
        ref:"attr:FormulaBuilder.cancelled",
        flags:"R",
        description:"Was the builder operation cancelled?  Set to true when the user cancels with the cancel\n button or the dialog's close-button.",
        name:"cancelled",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:ValuesManager.removeMember":{
        ref:"method:ValuesManager.removeMember",
        flags:"",
        description:"  Remove a member form from this valuesManager, so its values are no longer managed\n  by this instance.\n  This does not clear the values associated with the form from the valuesManager - they\n  will still be available via <code>valuesManager.getValues()</code>, but will not be\n  updated as the form is manipulated.\n",
        name:"removeMember",
        seeAlso:[
            {ref:"method:ValuesManager.removeMembers", description:null}
        ],
        params:[
            {description:"form (or ID of form) to remove from this valuesManager", optional:false,
             name:"member", defaultValue:null, type:"DynamicForm | string"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "attr:FormItem.valueIconSize":{
        ref:"attr:FormItem.valueIconSize",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} is specified, this property may be used to specify both\n the width and height of the icon written out.\n Note that ${isc.DocUtils.linkForRef('attr:FormItem.valueIconWidth')} and ${isc.DocUtils.linkForRef('attr:FormItem.valueIconHeight')} take\n presidence over this value, if specified.",
        name:"valueIconSize",
        seeAlso:[
            {ref:"attr:FormItem.valueIconWidth", description:null},
            {ref:"attr:FormItem.valueIconHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:Layout.placeHolderDefaults":{
        ref:"attr:Layout.placeHolderDefaults",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:Layout.showDragPlaceHolder','this.showDragPlaceHolder')} is true, this \n defaults object determines the default appearance of the placeholder displayed\n when the user drags a widget out of this layout.<br>\n Default value for this property sets the placeholder ${isc.DocUtils.linkForRef('attr:Canvas.styleName','styleName')} to\n <code>\"layoutPlaceHolder\"</code><br>\n To modify this object, use ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')}",
        name:"placeHolderDefaults",
        seeAlso:[],
        valueType:"canvas properties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Layout",
        groups:["dragdrop"]
    },
    "attr:TextItem.keyPressFilter":{
        ref:"attr:TextItem.keyPressFilter",
        flags:"IRWA",
        description:"Sets a keypress filter regular expression to limit valid characters\n that can be entered by the user. If defined, keys that match the\n regular expression are allowed; all others are suppressed. The\n filter is applied after character casing, if defined.\n <P>\n Note: keypress filtering cannot be used at the same time as a ${isc.DocUtils.linkForRef('attr:TextItem.mask')}.",
        name:"keyPressFilter",
        setter:["setKeyPressFilter"],
        seeAlso:[
            {ref:"attr:TextItem.characterCasing", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formFilters')}"],
        definingClass:"class:TextItem"
    },
    "attr:ListGrid.body":{
        ref:"attr:ListGrid.body",
        flags:"R",
        description:"GridRenderer used to render the dataset.",
        name:"body",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:Criterion.operator":{
        ref:"attr:Criterion.operator",
        flags:"IR",
        description:"Operator this criterion applies.",
        name:"operator",
        seeAlso:[],
        valueType:"OperatorId",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "method:TileGrid.anySelected":{
        ref:"method:TileGrid.anySelected",
        flags:"",
        description:" Whether at least one item is selected\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "type:VisibilityMode":{
        ref:"type:VisibilityMode",
        values:[
            {description:"Only one section can be expanded at a time.", value:"\"mutex\""},
            {description:"Multiple sections can be expanded at the same time, and will share space.",
             value:"\"multiple\""}
        ],
        description:"Settings for whether multiple sections can be in the expanded state simultaneously.",
        name:"VisibilityMode",
        seeAlso:[],
        type:"type"
    },
    "attr:Menu.iconWidth":{
        ref:"attr:Menu.iconWidth",
        flags:"IRW",
        description:"The default width applied to custom icons in this menu. This is used whenever\n          item.iconWidth is not specified.",
        name:"iconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:FilterBuilder.bracket":{
        ref:"attr:FilterBuilder.bracket",
        flags:"IR",
        description:"Widget used as a \"bracket\" to hint to the user that a subclause groups several\n field-by-field filter criteria under one logical operator.\n <P>\n By default, a simple CSS-style Canvas with borders on three sides.  A vertical StretchImg\n could provide a more elaborate appearance.",
        name:"bracket",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Canvas.contains":{
        ref:"method:Canvas.contains",
        flags:"A",
        description:"      Returns true if element is a descendant of this widget (i.e., exists below this widget in\n      the containment hierarchy); and false otherwise.\n",
        name:"contains",
        seeAlso:[],
        params:[
            {description:"the canvas to be tested", optional:false, name:"canvas", defaultValue:null,
             type:"canvas"},
            {description:"If passed this method will return true if the the canvas                                parameter is a pointer to this widget.",
             optional:true, name:"testSelf", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if specified element is a descendant of this canvas; false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "method:TileGrid.recordDoubleClick":{
        ref:"method:TileGrid.recordDoubleClick",
        flags:"",
        description:" Executed when the tileGrid receives a 'doubleclick' event on a\n tile. The default implementation does nothing -- override to perform some action\n when any record is doubleclicked.<br>\n A record event handler can be specified either as\n a function to execute, or as a string of script to evaluate. If the handler is defined\n as a string of script, all the parameters below will be available as variables for use\n in the script.<br>\n If you want to cancel the doubleclick based on the parameters, return false. Otherwise, return \n true so that the doubleclick event be registered with the tile.\n\n",
        name:"recordDoubleClick",
        seeAlso:[],
        params:[
            {description:"the TileGrid itself", optional:false, name:"viewer", defaultValue:null,
             type:"TileGrid"},
            {description:"the tile that was doubleclicked on", optional:false, name:"tile",
             defaultValue:null, type:"Canvas"},
            {description:"the record that was doubleclicked on", optional:false, name:"record",
             defaultValue:null, type:"TileRecord"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('tilingEditing')}"],
        definingClass:"class:TileGrid",
        groups:["events"]
    },
    "classAttr:LoginDialog.LOGIN_BUTTON_TITLE":{
        ref:"classAttr:LoginDialog.LOGIN_BUTTON_TITLE",
        flags:"IRW",
        description:"Default title for login button in the dialog displayed by \n ${isc.DocUtils.linkForRef('classMethod:isc.showLoginDialog')}.",
        name:"LOGIN_BUTTON_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Log in\"",
        type:"classAttr",
        definingClass:"class:LoginDialog",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.confirmCancelEditing":{
        ref:"attr:ListGrid.confirmCancelEditing",
        flags:"IRW",
        description:"If this is an editable listGrid, when the user attempts to cancel an edit, should we\n display a confirmation prompt before discarding the edited values for the record?",
        name:"confirmCancelEditing",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ListGrid.generateClickOnEnter":{
        ref:"attr:ListGrid.generateClickOnEnter",
        flags:"IRWA",
        description:"If true, when the user navigates to a cell using arrow keys and hits Enter, \n the cell will respond to a click event.",
        name:"generateClickOnEnter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "group:xmlTag":{
        ref:"group:xmlTag",
        title:"&lt;isomorphic:XML&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n This tag converts SmartClient UI components declaratively specified in the body of the tag to\n JavaScript for execution in the browser.\n <p>\n The XML->JS translation works just like with ${isc.DocUtils.linkForRef('group:loadUITag')}, but the XML is read\n from the body of the tag.  If you wish, you can also specify an external filename, and XML\n will be read from that file, in addition to any XML encountered in the body of the tag.  If\n you do specify that an external file should be read, it is read from a path starting in \n your webroot (as opposed to the <code>&lt;loadUI&gt;</code> tag, which looks in \n <code>shared/ui</code>) \n \n <p>\n Note that this JSP tag must be surrounded by &lt;SCRIPT&gt; tags in the JSP because it\n generates JavaScript code.  Like other tags that generate JavaScript code, this tag can be\n used in a JSP that is included from your main page in order to create separate\n cacheability.  For example:<pre>\n     &lt;SCRIPT SRC=\"myUIDefinitions.jsp\"&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n Example of using this tag :\n <pre>\n &lt;isomorphic:XML&gt;\n &lt;Canvas&nbsp;backgroundColor=\"black\"/&gt;\n &lt;/isomorphic:XML&gt;\n </pre>\n Would output the following JavaScript code:\n <pre>\n Canvas.create({\n   backgroundColor: \"black\"\n });\n </pre>\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>filename</b><br>\n <i>value format</i>: String - name of XML file to load (including the \"XML\" extension)<br>\n <i>default value</i>: NONE\n <p>\n This optional attribute specifies the name of an XML file to read and convert.\n <p>\n An example that specifies both a filename and some XML in the tag body:\n <pre>\n &lt;isomorphic:XML name=\"test.xml\"&gt;\n   &lt;Canvas backgroundColor=\"red\"/&gt;\n &lt;/isomorphic:XML&gt;\n </pre>",
        name:"xmlTag",
        seeAlso:[
            {ref:"group:loadUITag", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "attr:LoginDialog.loginButtonTitle":{
        ref:"attr:LoginDialog.loginButtonTitle",
        flags:"IR",
        description:"Specifies the contents of the login submission button of the ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"loginButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Dialog.LOGIN_BUTTON_TITLE",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:TextAreaItem.setSelectionRange":{
        ref:"method:TextAreaItem.setSelectionRange",
        flags:"",
        description:" Puts focus into this form item and selects characters between the given indices.\n Only applies to drawn text based items.\n",
        name:"setSelectionRange",
        seeAlso:[],
        params:[
            {description:"selection starting character index", optional:false, name:"start",
             defaultValue:null, type:"integer"},
            {description:"end of selection character index", optional:false, name:"end",
             defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:TextAreaItem"
    },
    "class:Applet":{
        methods:["method:Applet.getPluginHandle"],
        ref:"class:Applet",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Client Bridges"],
        attrs:["attr:Applet.name", "attr:Applet.code", "attr:Applet.src", "attr:Applet.codeBase", 
         "attr:Applet.archive","attr:Applet.alt", "attr:Applet.altHTML", "attr:Applet.params", "attr:Applet.mayScript", 
         "attr:Applet.scriptable","attr:Applet.classID", "attr:Applet.objectCodeBase", "attr:Applet.extraHTML", 
         "attr:Applet.useTag"],
        description:"SmartClient abstraction for Applets.  This class provides Java&lt;-&gt;SmartClient\n  interoperability.  When embedding Applets into SmartClient-based pages, please use this\n  wrapper class to instantiate the Applet.  Doing so will provide the following benefits:\n  <p>\n  <ul>\n  <li>Drag and drop interactions on the page can target the Applet (can drop on applet).\n  <li>When a drag interaction transits an Applet, SmartClient will ensure that the dragTarget\n  appears above the Applet (prevents burn-through).\n  <li>Allows you to easily embed the Applet into SmartClient containers such as Windows and\n  Layouts, providing automatic masking to ensure the correct z-index order.  But see notes\n  below for limitations.\n  </ul>\n  <p>\n  <u>Known issues</u><br><br>\n  <ul>\n  <li>Resizing Applets can cause Firefox to lock up and require the user to terminate the\n  process manually.  The cause is currently unknown, but likely causes include applets\n  attempting to control their own size, or to attempt to re-render on resize.  More recent\n  JVMs (1.6+) appear to make this problem less frequent.  This may also simply be caused by\n  applet code being unable to deal with certain combination sizes and crashing.\n  <li>In Firefox, when using a JVM older than 1.6, Applets cannot be resized smaller than\n  their originally drawn size.  Doing so causes them to stick at the originally drawn size,\n  and to overflow the boundary of any SmartClient component they are embedded in.\n  <li>In Firefox, Applets do not handle the z-index ordering correctly with respect to\n  IFRAMEs or other Applets.  Therefore, in Firefox you cannot layer Applets on top of each\n  other or on top of SmartClient components containing IFRAMEs and have them render in the\n  correct order.\n  </ul>\n\n  For a general overview of applets and an applet tutorial see: \n  <a href='http://java.sun.com/docs/books/tutorial/applet/' onclick=\"window.open('http://java.sun.com/docs/books/tutorial/applet/');return false;\">http://java.sun.com/docs/books/tutorial/applet/</a>",
        name:"Applet",
        seeAlso:[],
        type:"class",
        requiresModules:["PluginBridges"]
    },
    "attr:Button.iconWidth":{
        ref:"attr:Button.iconWidth",
        flags:"IR",
        description:"Width in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:Window.contentLayout":{
        ref:"attr:Window.contentLayout",
        flags:"IRWA",
        description:"The layout policy that should be used for widgets within the Window body.\n <P>\n Valid values are \"vertical\", \"horizontal\", \"none\".  If the body is a Layout, this\n controls how the items are stacked in the body by setting ${isc.DocUtils.linkForRef('attr:Layout.vertical')}.  \n See ${isc.DocUtils.linkForRef('attr:Window.bodyConstructor')} for details.",
        name:"contentLayout",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"vertical\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance"]
    },
    "method:TileGrid.transferDragData":{
        ref:"method:TileGrid.transferDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method is called to transfer a set of records that\n were dropped onto some other component.  This method is called after the set of records has\n been copied to the other component.  Whether or not this component's data is modified is \n determined by the value of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dragDataAction')}.\n <P>\n With a <code>dragDataAction</code> of \"move\", a databound component will issue \"remove\"\n dsRequests against its DataSource to actually remove the data, via\n ${isc.DocUtils.linkForRef('method:DataSource.removeData')}.\n\n",
        name:"transferDragData",
        seeAlso:[
            {ref:"method:DataBoundComponent.getDragData", description:null},
            {ref:"method:ListGrid.willAcceptDrop", description:null}
        ],
        returns:{description:"Array of objects that were dragged out of this ListGrid.", type:"Array"},
        type:"method",
        definingClass:"class:TileGrid"
    },
    "method:ListGrid.transferDragData":{
        ref:"method:ListGrid.transferDragData",
        flags:"A",
        description:"\n During a drag-and-drop interaction, this method is called to transfer a set of records that\n were dropped onto some other component.  This method is called after the set of records has\n been copied to the other component.  Whether or not this component's data is modified is \n determined by the value of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dragDataAction')}.\n <P>\n With a <code>dragDataAction</code> of \"move\", a databound component will issue \"remove\"\n dsRequests against its DataSource to actually remove the data, via\n ${isc.DocUtils.linkForRef('method:DataSource.removeData')}.\n\n",
        name:"transferDragData",
        seeAlso:[
            {ref:"method:DataBoundComponent.getDragData", description:null},
            {ref:"method:ListGrid.willAcceptDrop", description:null}
        ],
        returns:{description:"Array of objects that were dragged out of this ListGrid.", type:"Array"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Facet.align":{
        ref:"attr:Facet.align",
        flags:"IR",
        description:"Default alignment for facet label title, and cells for this facet.\n Can be overridden at the facetValue level, or by setting titleAlign or cellAlign\n on the facet.",
        name:"align",
        seeAlso:[
            {ref:"attr:Facet.titleAlign", description:null},
            {ref:"attr:Facet.cellAlign", description:null}
        ],
        valueType:"alignEnum",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "type:Side":{
        ref:"type:Side",
        values:[
            {description:"Left side", value:"Canvas.LEFT"},
            {description:"Right side", value:"Canvas.RIGHT"},
            {description:"Top side", value:"Canvas.TOP"},
            {description:"Bottom side", value:"Canvas.BOTTOM"}
        ],
        description:"Side of a component.",
        name:"Side",
        seeAlso:[],
        type:"type"
    },
    "group:jsfIntegration":{
        ref:"group:jsfIntegration",
        title:"Integration with JSF",
        treeLocation:["Concepts"],
        refs:[],
        description:"SmartClient can be used within JSF applications to add AJAX richness and interactivity.\n <P>\n Because JSF is a pre-AJAX architecture, the recommended approach in adding\n SmartClient to JSF applications is to create pages that use SmartClient components exclusively,\n so that older, server-based JSF components do not introduce full-page refreshes.\n <P>\n JSF pages that render components on the server access data via JSF Expression Language.\n SmartClient-based JSF pages can similarly load initial data by using JSTL, as shown in \n <a href='/examples/server_integration/#jstlList' onclick=\"window.open('/examples/server_integration/#jstlList');return false;\">this example</a>, where a ListGrid is\n populated by JSTL access to Java Beans stored in the JSP <code>pageContext</code>.  \n <P>\n Once a SmartClient JSF page has loaded, SmartClient components will request data via background\n HTTP requests that load only data, not a complete page.  The \n ${isc.DocUtils.linkForRef('class:DMI','Direct Method Invocation')} system can be used to declaratively map SmartClient's\n background data requests directly to Java Methods. The SmartClient server automatically\n translates inbound request data into Java Objects that are passed to the method you specify,\n and the Java method return value is automatically translated into data for SmartClient\n components.\n <P>\n <h4>Incorporating server-side JSF components into a SmartClient JSF page</h4>\n <P>\n An ${isc.DocUtils.linkForRef('class:HTMLFlow')} or ${isc.DocUtils.linkForRef('class:HTMLPane')} component can be used to incorporate server-generated\n content within a SmartClient-based page.  With ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType','contentsType')} set to\n \"page\", the HTMLPane/Flow will act like a standalone page-within-a-page (via a\n SmartClient-managed HTML IFRAME element), allowing interactive server-side JSF components to\n participate normally, with limitations as discussed under the documentation for \n ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType','contentsType')}.",
        name:"jsfIntegration",
        seeAlso:[],
        type:"group"
    },
    "attr:MultiSortPanel.deleteLevelButtonTitle":{
        ref:"attr:MultiSortPanel.deleteLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the deleteLevelButton",
        name:"deleteLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Delete Level\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "method:DataSource.downloadFile":{
        ref:"method:DataSource.downloadFile",
        flags:"",
        description:" Download a file stored in a field of type:\"binary\" in a DataSource record.\n <P>\n This will trigger the browser's \"Save As\" dialog and allow the user to save the file.\n\n",
        name:"downloadFile",
        seeAlso:[],
        params:[
            {description:"Record or value of primary key field                                                   for record containing the file to\n                                                   download.",
             optional:false, name:"data", defaultValue:null, type:"Record or PKValue"},
            {description:"optional name of the binary field containing the                                         file.  If not provided, the first binary field\n                                         is used",
             optional:true, name:"fieldName", defaultValue:null, type:"String"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "class:SimpleType":{
        methods:["method:SimpleType.shortDisplayFormatter", "method:SimpleType.normalDisplayFormatter"],
        ref:"class:SimpleType",
        classAttrs:[],
        classMethods:["classMethod:SimpleType.getType", "classMethod:SimpleType.registerSummaryFunction", 
         "classMethod:SimpleType.setDefaultSummaryFunction","classMethod:SimpleType.getDefaultSummaryFunction", 
         "classMethod:SimpleType.applySummaryFunction"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:SimpleType.validOperators", "attr:SimpleType.name", "attr:SimpleType.inheritsFrom", 
         "attr:SimpleType.validators","attr:SimpleType.valueMap", "attr:SimpleType.editorType"],
        description:"An atomic type such as a string or number, that is generally stored, displayed and\n manipulated as a single value.\n <P>\n SimpleTypes can be created at any time, and subsequently referred to as a \n ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field type')} in ${isc.DocUtils.linkForRef('class:DataSource','DataSources')} and\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}.  This allows you to define\n ${isc.DocUtils.linkForRef('attr:SimpleType.validators','validation')}, ${isc.DocUtils.linkForRef('method:SimpleType.normalDisplayFormatter','formatting')}\n and ${isc.DocUtils.linkForRef('attr:SimpleType.editorType','editing')} behaviors for a type to be reused across all\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}.\n <P>\n Note that the term \"simpleType\" is used in the same sense as in\n <a href='XML Schema' onclick=\"window.open('XML Schema');return false;\">http://www.w3.org/TR/xmlschema-0/</a>, and\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')} will create new SimpleType definitions.\n <P>\n An ${isc.DocUtils.linkForExampleId('customSimpleType', 'example')} is here.",
        name:"SimpleType",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('customSimpleType')}"]
    },
    "attr:CubeGrid.colHeaderBaseStyle":{
        ref:"attr:CubeGrid.colHeaderBaseStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:Button.baseStyle','baseStyle')} for the buttons in this grid's column headers.\n <P>\n Exception: The innermost column\n header will always be styled using ${isc.DocUtils.linkForRef('attr:CubeGrid.innerHeaderBaseStyle')}.",
        name:"colHeaderBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"colHeader",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "attr:Window.showHeader":{
        ref:"attr:Window.showHeader",
        flags:"IRWA",
        description:"If true, show a ${isc.DocUtils.linkForRef('attr:Window.header')} for this Window. \n <P>\n Note that in certain Smartclient skins ${isc.DocUtils.linkForRef('attr:Window.showHeaderBackground')} may be set to\n <code>false</code> and the header's appearance implemented as part of the\n window's ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','edge media')}. In this case suppressing the header can be achieved\n by overriding the edge media as well as setting this property to false. For example, to\n create a headerless window with a similar appearance to a ${isc.DocUtils.linkForRef('class:Menu')} in the\n <code><i>TreeFrog</i></code> skin, the following attributes could be used:\n <pre>\n      showHeader:false,\n      edgeImage:\"[SKIN]/Menu/m.png\",\n      edgeSize:10, edgeTop:17, edgeBottom:17,\n      edgeCenterBackgroundColor:\"#F7F7F7\"\n </pre>",
        name:"showHeader",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:ListGrid.headerButtonProperties":{
        ref:"attr:ListGrid.headerButtonProperties",
        flags:"IRA",
        description:"Properties to apply to all header buttons.\n Overrides defaults applied via  ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonDefaults')}.",
        name:"headerButtonProperties",
        seeAlso:[],
        valueType:"Button Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "method:DynamicForm.clearValue":{
        ref:"method:DynamicForm.clearValue",
        flags:"",
        description:" Clears the value for some field via a call to ${isc.DocUtils.linkForRef('method:FormItem.clearValue')} if appropriate.\n If there is no item associated with the field name, the field will just be cleared within\n our stored set of values.\n",
        name:"clearValue",
        seeAlso:[],
        params:[
            {description:"Name of the field being cleared. A ${isc.DocUtils.linkForRef('type:dataPath')} may be used for  clearing details of nested data structures.",
             optional:false, name:"fieldName", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "class:ToolStripSeparator":{
        methods:[],
        ref:"class:ToolStripSeparator",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/ToolStrip"],
        attrs:["attr:ToolStripSeparator.skinImgDir", "attr:ToolStripSeparator.vSrc", 
         "attr:ToolStripSeparator.hSrc"],
        description:"Simple subclass of Img with appearance appropriate for a ToolStrip separator",
        name:"ToolStripSeparator",
        seeAlso:[],
        type:"class"
    },
    "attr:CellRecord.enabled":{
        ref:"attr:CellRecord.enabled",
        flags:"IR",
        description:"Default property name denoting whether this record is enabled. Property name may be modified\n for some grid via ${isc.DocUtils.linkForRef('attr:ListGrid.recordEnabledProperty')}.",
        name:"enabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disabledRows')}"],
        definingClass:"object:CellRecord"
    },
    "method:StretchImgButton.removeFromRadioGroup":{
        ref:"method:StretchImgButton.removeFromRadioGroup",
        flags:"",
        description:" Remove this widget from the specified mutually exclusive selection group with the ID\n passed in.\n No-op's if this widget is not a member of the groupID passed in.\n If no groupID is passed in, defaults to removing from whatever radioGroup this widget\n is a member of.\n",
        name:"removeFromRadioGroup",
        seeAlso:[],
        params:[
            {description:"- optional radio group ID (to ensure the widget is removed                                        from the appropriate group.",
             optional:true, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state", "event handling"]
    },
    "method:ListGrid.clearEditValue":{
        ref:"method:ListGrid.clearEditValue",
        flags:"A",
        description:"\n  Clear a field value being tracked as an unsaved user edit.<P>\n  The saved record value will be displayed in the the appropriate cell instead.\n  Will also discard any validation errors for the specified field / row.\n\n",
        name:"clearEditValue",
        seeAlso:[],
        params:[
            {description:"Row number, primary keys object for the record,                                              or editValues object",
             optional:false, name:"editValuesID", defaultValue:null, type:"number | object"},
            {description:"Column number, or Name of field for which                                       the value is to be cleared",
             optional:false, name:"colNum", defaultValue:null, type:"number | string"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "classAttr:Dialog.ASK_FOR_VALUE_TITLE":{
        ref:"classAttr:Dialog.ASK_FOR_VALUE_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed by ${isc.DocUtils.linkForRef('classMethod:isc.askForValue')}.\n A custom title can alternatively be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"ASK_FOR_VALUE_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Please enter a value\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:FormItem.showFocusedPickerIcon":{
        ref:"attr:FormItem.showFocusedPickerIcon",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, should the picker icon show\n a focused image when the form item has focus?",
        name:"showFocusedPickerIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "type:JSONDateFormat":{
        ref:"type:JSONDateFormat",
        values:[
            {description:"dates are is encoded as a String in <a target=_blank        href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema date format</a> in UTC,\n        for example, \"2005-08-02\" for logical date fields or \"2005-08-01T21:35:48.350\"\n        for <code>datetime</code> fields. See ${isc.DocUtils.linkForRef('group:dateFormatAndStorage','Date format and\n        storage')} for more information.",
             value:"\"xmlSchema\""},
            {description:"dates are encoded as raw JavaScript code for creating a Date object,        that is:\n <pre>\n    new Date(1238792738633)\n </pre>\n This is not strictly valid JSON, but if eval()d, will result in an identical date object,\n regardless of timezone.",
             value:"\"dateConstructor\""}
        ],
        description:"Format for encoding dates in JSON.  Note you can override ${isc.DocUtils.linkForRef('method:JSONEncoder.encodeDate')}\n for a custom format.",
        name:"JSONDateFormat",
        seeAlso:[],
        type:"type"
    },
    "attr:groupNode.groupValue":{
        ref:"attr:groupNode.groupValue",
        flags:"R",
        description:"The value from which groups are computed for a field, \n which results from ${isc.DocUtils.linkForRef('method:ListGridField.getGroupValue')}",
        name:"groupValue",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"any",
        defaultValue:"see below",
        type:"attr",
        definingClass:"object:groupNode",
        groups:["grouping"]
    },
    "method:ValuesManager.fetchData":{
        ref:"method:ValuesManager.fetchData",
        flags:"",
        description:" Retrieve data that matches the provided criteria, and edit the first record returned\n \n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "type:ReorderPosition":{
        ref:"type:ReorderPosition",
        values:[
            {description:"Drop the drag-item before the target-row", value:"ListGrid.BEFORE"},
            {description:"Drop the drag-item after the target-row", value:"ListGrid.AFTER"},
            {description:"Drop the drag-item over (onto) the target-row", value:"ListGrid.OVER"}
        ],
        description:"Controls where a drag-item should be dropped in relation to the target row",
        name:"ReorderPosition",
        seeAlso:[],
        type:"type",
        groups:["dragging"]
    },
    "method:Selection.deselectList":{
        ref:"method:Selection.deselectList",
        flags:"",
        description:"			Deselect an array of items (subset of the entire list)\n",
        name:"deselectList",
        seeAlso:[],
        params:[
            {description:"listarray of objects to select", optional:true, name:"", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:ListGrid.recordDrop":{
        ref:"method:ListGrid.recordDrop",
        flags:"",
        description:" Process a drop of one or more records on a ListGrid record.\n <P>\n This method can be overriden to provide custom drop behaviors, and is a more appropriate\n override point than the lower level ${isc.DocUtils.linkForRef('method:Canvas.drop')} handler.\n <P>\n If this is a self-drop, records are simply reordered.\n <P>\n For a drop from another widget, ${isc.DocUtils.linkForRef('method:ListGrid.transferDragData')} is called, which\n depending on the ${isc.DocUtils.linkForRef('attr:ListGrid.dragDataAction','dragDataAction')} specified on the source\n widget, may either remove the source records from the original list\n (<code>dragDataAction:\"move\"</code>) or just provide a copy to this list\n (<code>dragDataAction:\"copy\"</code>).\n <P>\n If this grid is databound, the new records will be added to the dataset by calling\n ${isc.DocUtils.linkForRef('method:DataSource.addData')}.  Further, if the new records were dragged from another \n databound component, and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.addDropValues','addDropValues')}\n is true, ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues','getDropValues')} will be called for every item \n being dropped.  \n <P>\n Note that reordering records has no effect on a databound grid.\n <P>\n The newly dropped data is then selected automatically.\n <P>\n NOTE: for a drop beyond the last visible record of a ListGrid, <code>targetRecord</code>\n will be null and the <code>index</code> will be one higher than the last record.  This\n includes a drop into an empty ListGrid, where <code>index</code> will be 0.\n \n",
        name:"recordDrop",
        seeAlso:[],
        params:[
            {description:"records being dropped", optional:false, name:"dropRecords", defaultValue:null,
             type:"List of ListGridRecord"},
            {description:"record being dropped on.  May be null", optional:false, name:"targetRecord",
             defaultValue:null, type:"ListGridRecord"},
            {description:"index of record being dropped on", optional:false, name:"index",
             defaultValue:null, type:"int"},
            {description:"widget where dragging began", optional:false, name:"sourceWidget",
             defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ToolStripResizer.hSrc":{
        ref:"attr:ToolStripResizer.hSrc",
        flags:"IRW",
        description:"Image for horizontal resizer for a vertical Toolstrip",
        name:"hSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]hresizer.png\"",
        type:"attr",
        definingClass:"class:ToolStripResizer"
    },
    "attr:SectionStack.scrollSectionIntoView":{
        ref:"attr:SectionStack.scrollSectionIntoView",
        flags:"IR",
        description:"If an expanded or shown section expands past the current viewport and this property is\n true, then the viewport will auto-scroll to fit as much of the section content into the\n viewport without scrolling the top of the section out of the viewport.",
        name:"scrollSectionIntoView",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "attr:CubeGrid.canMinimizeFacets":{
        ref:"attr:CubeGrid.canMinimizeFacets",
        flags:"IRW",
        description:"If true, when multiple facets are shown on a side, all facetValues in the second level\n of headers or higher will show controls to \"minimize\" the values of the next facet.\n Minimizing means showing only one, or very few, of the next facet's values.\n <P>\n Set ${isc.DocUtils.linkForRef('attr:FacetValue.isMinimizeValue')} to indicate which facetValues should be\n shown when a facet is minimized.",
        name:"canMinimizeFacets",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetExpansion"]
    },
    "attr:SliderItem.vertical":{
        ref:"attr:SliderItem.vertical",
        flags:"IR",
        description:"Indicates whether this is a vertical or horizontal slider.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:SliderItem"
    },
    "attr:Canvas.appImgDir":{
        ref:"attr:Canvas.appImgDir",
        flags:"IRWA",
        description:"Default directory for app-specific images, relative to the Page-wide\n ${isc.DocUtils.linkForRef('classMethod:Page.getAppImgDir','appImgDir')}.",
        name:"appImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "attr:Canvas.snapVDirection":{
        ref:"attr:Canvas.snapVDirection",
        flags:"IRW",
        description:"The vertical snap direction.\n Set this value to Canvas.BEFORE to snap to the nearest gridpoint above;\n set it to Canvas.AFTER to snap to the nearest gridpoint below; and set\n it to Canvas.NEAREST to snap to the nearest gridpoint in either direction.",
        name:"snapVDirection",
        seeAlso:[
            {ref:"attr:Canvas.snapToGrid", description:null},
            {ref:"attr:Canvas.snapResizeToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapToGrid", description:null},
            {ref:"attr:Canvas.childrenSnapResizeToGrid", description:null}
        ],
        valueType:"string",
        defaultValue:"Canvas.AFTER",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:ImgButton.showFocus":{
        ref:"attr:ImgButton.showFocus",
        deprecated:"as of SmartClient version 6.1 in favor of ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')}",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  Note that by default the\n <code>over</code> state is used to indicate focus.",
        name:"showFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "group:display":{
        ref:"group:display",
        refs:["method:ListGrid.getFieldName", "method:ListGrid.getFieldNum", 
         "method:ListGrid.getSummaryTitle"],
        name:"display",
        seeAlso:[],
        type:"group"
    },
    "method:Date.duplicate":{
        ref:"method:Date.duplicate",
        flags:"A",
        description:"      Copy the value of this date into a new Date() object for independant manipulation\n",
        name:"duplicate",
        seeAlso:[],
        type:"method",
        definingClass:"class:Date"
    },
    "attr:SectionStackSection.expanded":{
        ref:"attr:SectionStackSection.expanded",
        flags:"I",
        description:"Sections default to the collapsed state unless ${isc.DocUtils.linkForRef('attr:SectionStackSection.showHeader')} is\n set to <code>false</code> in which case they default to the expanded state.  This\n attribute allows you to explicitly control the expand/collapse state of the\n section by overriding the above default behavior.",
        name:"expanded",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "group:gridLayout":{
        ref:"group:gridLayout",
        refs:["method:CubeGrid.setFacetTitleAlign", "method:CubeGrid.setFacetValueTitleAlign", 
         "attr:CubeGrid.canResizeColumns","attr:CubeGrid.canMinimizeColumns", "attr:CubeGrid.autoSizeHeaders", 
         "attr:CubeGrid.facetTitleAlign","attr:CubeGrid.facetValueAlign", "attr:CubeGrid.cellAlign", 
         "attr:CubeGrid.wrapFacetTitles","attr:CubeGrid.wrapFacetValueTitles", "attr:CubeGrid.padTitles", 
         "attr:CubeGrid.defaultFacetWidth","attr:CubeGrid.bodyMinWidth", "attr:CubeGrid.bodyMinHeight"],
        description:null,
        name:"gridLayout",
        seeAlso:[],
        type:"group"
    },
    "attr:FacetValue.borderBefore":{
        ref:"attr:FacetValue.borderBefore",
        flags:"IR",
        description:"CSS line style to apply as a border before this facetValue",
        name:"borderBefore",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "method:Window.addMembers":{
        ref:"method:Window.addMembers",
        flags:"A",
        description:"\n  Add one or more canvases to the layout, optionally at specific positions.\n\n  NOTE: depending on the layout policy, adding a new member may cause existing members to\n  resize.\n\n",
        name:"addMembers",
        seeAlso:[],
        params:[
            {description:"array of canvases to be added, or reference to single                                          canvas",
             optional:false, name:"newMembers", defaultValue:null, type:"Array or object"},
            {description:"position to add newMembers; if omitted newMembers                                          will be added at the last position",
             optional:true, name:"position", defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "classMethod:XMLTools.selectNumber":{
        ref:"classMethod:XMLTools.selectNumber",
        flags:"A",
        description:" Retrieve a numeric value from an XML element or document based on an XPath expression.\n <P>\n If more than one node matches, only the first node's value will be returned.\n <P>\n Namespacing works as described under ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectNodes')}\n <P>\n <b>NOTE:</b> this API cannot be supported on the Safari web browser for versions prior to\n 3.0.3.\n\n",
        name:"selectNumber",
        seeAlso:[],
        params:[
            {description:"Native XMLElement,document, or xml string                                                        to select from",
             optional:false, name:"element", defaultValue:null,
             type:"XMLElement or XMLDocument or String"},
            {description:"XPath expression to use to select nodes", optional:false, name:"expression",
             defaultValue:null, type:"XPath"},
            {description:"namespace mapping used by the expression", optional:true, name:"namespaces",
             defaultValue:null, type:"prefix -> URI mapping"}
        ],
        returns:{description:"result of the XPath, in Number form", type:"Number"},
        type:"classMethod",
        definingClass:"class:XMLTools",
        groups:["xmlTransform"]
    },
    "attr:Canvas.styleName":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Canvas.styleName",
        description:"The CSS class applied to this widget as a whole.",
        name:"styleName",
        setter:["setStyleName"],
        valueType:"CSSStyleName",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('styles')}"],
        defaultValue:"\"normal\"",
        groups:["appearance"],
        definingClass:"class:Canvas"
    },
    "attr:ListGridField.headerBaseStyle":{
        ref:"attr:ListGridField.headerBaseStyle",
        flags:"IRW",
        description:"Custom base style to apply to this field's header button instead of \n ${isc.DocUtils.linkForRef('attr:ListGrid.headerBaseStyle')}.<br>\n Note that depending on the header button constructor, you may have to override\n ${isc.DocUtils.linkForRef('attr:ListGridField.headerTitleStyle')} as well.",
        name:"headerBaseStyle",
        seeAlso:[],
        valueType:"cssClass",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:Button.setIconOrientation":{
        ref:"method:Button.setIconOrientation",
        flags:"",
        description:" Changes the orientation of the icon relative to the text of the button.\n\n",
        name:"setIconOrientation",
        seeAlso:[],
        params:[
            {description:"The new orientation of the icon relative to the text of the button.",
             optional:false, name:"orientation", defaultValue:null, type:"\"left\" or \"right\""}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:ListGrid.frozenBaseStyle":{
        ref:"attr:ListGrid.frozenBaseStyle",
        flags:"IRW",
        description:"If this listGrid contains any frozen fields, this property can be used to apply a custom\n baseStyle to all cells in those frozen fields. If unset, the standard base style will be\n used for both frozen and unfrozen cells.",
        name:"frozenBaseStyle",
        seeAlso:[
            {ref:"attr:ListGrid.baseStyle", description:null},
            {ref:"attr:ListGridField.frozen", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance", "frozenFields"]
    },
    "method:Tree.getTitle":{
        ref:"method:Tree.getTitle",
        flags:"",
        description:"\n Return the title of a node -- the name as it should be presented to the user.  This method\n works as follows:\n <ul>\n <li> If a ${isc.DocUtils.linkForRef('attr:Tree.titleProperty')} is set on the node, the value of that property is\n returned.\n <li> Otherwise, if the ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')} is set on the node, that value is\n returned, minus any trailing ${isc.DocUtils.linkForRef('attr:Tree.pathDelim')}.\n <li> Finally, if none of the above yielded a title, the value of\n ${isc.DocUtils.linkForRef('attr:Tree.defaultNodeTitle')} is returned.\n </ul>\n You can override this method to return the title of your choice for a given node.\n <br><br>\n To override the title for an auto-constructed tree (for example, in a databound TreeGrid),\n override ${isc.DocUtils.linkForRef('method:TreeGrid.getNodeTitle')} instead.\n\n",
        name:"getTitle",
        seeAlso:[
            {ref:"method:TreeGrid.getNodeTitle", description:null}
        ],
        params:[
            {description:"node for which the title is being requested", optional:false, name:"node",
             defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"title to display", type:"string"},
        type:"method",
        definingClass:"class:Tree"
    },
    "classMethod:isc.askForValue":{
        ref:"classMethod:isc.askForValue",
        flags:"",
        description:" Show a modal dialog with a text entry box, asking the user to enter a value.\n <P>\n As with other convenience methods that show Dialogs, such as ${isc.DocUtils.linkForRef('classMethod:isc.warn')},\n the dialog is shown and the function immediately returns.  When the user responds, the\n provided callback is called.\n <P>\n If the user clicks OK, the value typed in is passed to the callback (including the empty\n string (\"\") if nothing was entered.  If the user clicks cancel, the value passed to the\n callback is null.\n <P>\n A default value for the text field can be passed via <code>properties.defaultValue</code>.\n <P>\n Keyboard focus is automatically placed in the text entry field, and hitting the enter key is\n the equivalent of pressing OK.\n\n",
        name:"askForValue",
        seeAlso:[
            {ref:"method:Dialog.okClick", description:null},
            {ref:"method:Dialog.cancelClick", description:null},
            {ref:"classAttr:Dialog.ASK_FOR_VALUE_TITLE", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"Callback to fire when the                                       user clicks a button to dismiss the dialog.\n                                      This has the single parameter 'value', indicating the \n                                      user entry, or null if cancel was pressed or the window\n                                      closed",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog.                                      To set ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons','custom buttons')} for\n                                      the Dialog, set properties.toolbarButtons to an array of\n                                      buttons\n										eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "attr:FormItem.selectOnFocus":{
        ref:"attr:FormItem.selectOnFocus",
        flags:"IRW",
        description:"Allows the ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus','selectOnFocus')} behavior to be configured on a\n per-FormItem basis.  Normally all items in a form default to the value of\n ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus')}.",
        name:"selectOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["focus"]
    },
    "class:StretchImgButton":{
        methods:["method:StretchImgButton.getTitle", "method:StretchImgButton.setTitle", 
         "method:StretchImgButton.setIconOrientation","method:StretchImgButton.setIcon", "method:StretchImgButton.setAutoFit", 
         "method:StretchImgButton.setBaseStyle","method:StretchImgButton.setTitleStyle", "method:StretchImgButton.select", 
         "method:StretchImgButton.deselect","method:StretchImgButton.isSelected", "method:StretchImgButton.setSelected", 
         "method:StretchImgButton.addToRadioGroup","method:StretchImgButton.removeFromRadioGroup", "method:StretchImgButton.setActionType", 
         "method:StretchImgButton.getActionType","method:StretchImgButton.setState", "method:StretchImgButton.setDisabled", 
         "method:StretchImgButton.getState","method:StretchImgButton.action", "method:StretchImgButton.iconClick"],
        ref:"class:StretchImgButton",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:StretchImgButton.title", "attr:StretchImgButton.wrap", "attr:StretchImgButton.icon", 
         "attr:StretchImgButton.iconSize","attr:StretchImgButton.iconWidth", "attr:StretchImgButton.iconHeight", 
         "attr:StretchImgButton.iconOrientation","attr:StretchImgButton.iconAlign", "attr:StretchImgButton.iconSpacing", 
         "attr:StretchImgButton.showDisabledIcon","attr:StretchImgButton.showRollOverIcon", "attr:StretchImgButton.showFocusedIcon", 
         "attr:StretchImgButton.showDownIcon","attr:StretchImgButton.showSelectedIcon", "attr:StretchImgButton.autoFit", 
         "attr:StretchImgButton.baseStyle","attr:StretchImgButton.titleStyle", "attr:StretchImgButton.selected", 
         "attr:StretchImgButton.radioGroup","attr:StretchImgButton.actionType", "attr:StretchImgButton.state", 
         "attr:StretchImgButton.showDisabled","attr:StretchImgButton.showDown", "attr:StretchImgButton.showFocus", 
         "attr:StretchImgButton.showFocused","attr:StretchImgButton.showRollOver", "attr:StretchImgButton.align", 
         "attr:StretchImgButton.valign","attr:StretchImgButton.showTitle", "attr:StretchImgButton.labelHPad", 
         "attr:StretchImgButton.labelVPad","attr:StretchImgButton.labelLengthPad", "attr:StretchImgButton.labelBreadthPad", 
         "attr:StretchImgButton.hiliteAccessKey","attr:StretchImgButton.src", "attr:StretchImgButton.vertical", 
         "attr:StretchImgButton.capSize"],
        description:"A StretchImg that behaves like a button, going through up/down/over state transitions in response\n to user events.  Supports an optional title, and will auto-size to accomodate the title text if\n <code>overflow</code> is set to \"visible\".\n <P>\n Examples of use include fancy buttons, poplist headers, and tabs.",
        name:"StretchImgButton",
        seeAlso:[],
        type:"class"
    },
    "attr:MathFunction.jsFunction":{
        ref:"attr:MathFunction.jsFunction",
        flags:"IR",
        description:"Javascript method to perform the calculation associated with this function",
        name:"jsFunction",
        seeAlso:[],
        valueType:"Function",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "attr:CalendarEvent.endDate":{
        ref:"attr:CalendarEvent.endDate",
        flags:"IRW",
        description:"Date object which represents the end date of a ${isc.DocUtils.linkForRef('object:CalendarEvent')}\n The name of this field within the CalendarEvent can be changed via \n ${isc.DocUtils.linkForRef('attr:Calendar.endDateField')}",
        name:"endDate",
        seeAlso:[],
        valueType:"Date",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "classAttr:Operators.notNullTitle":{
        ref:"classAttr:Operators.notNullTitle",
        flags:"IR",
        description:"Title for the \"notNull\" operator",
        name:"notNullTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"is not null\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.imageHeight":{
        ref:"attr:ListGridField.imageHeight",
        flags:"IRW",
        description:"Height of image shown for fieldTypes image and imageFile in this field.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n height.  For example, if <code>field.imageHeight</code> is \"logoHeight\",\n <code>record.logoHeight</code> will control the heigt of the image.",
        name:"imageHeight",
        seeAlso:[
            {ref:"attr:ListGrid.imageSize", description:null},
            {ref:"attr:ListGridField.imageSize", description:null},
            {ref:"attr:ListGridField.imageWidth", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:DateItem.textAlign":{
        ref:"attr:DateItem.textAlign",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is <code>true</code> this property governs the alignment\n of text within the text field.",
        name:"textAlign",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.RIGHT",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["appearance"]
    },
    "attr:ListGrid.quickDrawAheadRatio":{
        ref:"attr:ListGrid.quickDrawAheadRatio",
        flags:"IRW",
        description:"Alternative to ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio')}, to be used when the user\n is rapidly changing the grids viewport (for example drag scrolling through the grid).\n If unspecified ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio')} will be used in all cases",
        name:"quickDrawAheadRatio",
        seeAlso:[],
        valueType:"float",
        defaultValue:"1.3",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "attr:FormItem.valueMap":{
        ref:"attr:FormItem.valueMap",
        flags:"IRW",
        description:"In a form, valueMaps are used for FormItem types that allow the user to pick from a\n limited set of values, such as a SelectItem.  The valueMap can be either an Array of\n legal values or an Object where each property maps a stored value to a user-displayable\n value.\n <P>\n To set the initial selection for a form item with a valueMap, use\n ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue')}.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap')}.",
        name:"valueMap",
        setter:["setValueMap"],
        seeAlso:[],
        valueType:"Array or Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueMap"]
    },
    "attr:ComboBoxItem.showHintInField":{
        ref:"attr:ComboBoxItem.showHintInField",
        flags:"IRWA",
        description:"If showing a hint for this form item, should it be shown within the field?\n <P>CSS style for the hint is ${isc.DocUtils.linkForRef('attr:SelectItem.textBoxStyle')} with the suffix\n \"Hint\" appended to it.",
        name:"showHintInField",
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem",
        groups:["appearance"]
    },
    "classMethod:Page.getAppImgDir":{
        ref:"classMethod:Page.getAppImgDir",
        flags:"",
        description:" Return the directory for app-specific images.\n\n",
        name:"getAppImgDir",
        seeAlso:[],
        returns:{description:"URL for page-specific images.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "method:Button.setVAlign":{
        ref:"method:Button.setVAlign",
        flags:"",
        description:" Sets the vertical alignment of this buttons content.\n",
        name:"setVAlign",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["positioning"]
    },
    "method:SectionStack.sectionHeaderClick":{
        ref:"method:SectionStack.sectionHeaderClick",
        flags:"A",
        description:" Method intended to be called by the sectionHeader when it is clicked on.\n\n",
        name:"sectionHeaderClick",
        seeAlso:[],
        params:[
            {description:"the sectionHeader clicked on", optional:false, name:"sectionHeader",
             defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "method:EBay.setProductionToken":{
        ref:"method:EBay.setProductionToken",
        flags:"",
        description:"\n Sets the token to be used by queries to the eBay production environment.\n \n",
        name:"setProductionToken",
        seeAlso:[],
        params:[
            {description:"Your production authToken, provided to you by eBay.", optional:false,
             name:"authToken", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:EBay"
    },
    "class:History":{
        methods:[],
        ref:"class:History",
        classAttrs:[],
        classMethods:["classMethod:History.registerCallback", "classMethod:History.getCurrentHistoryId", 
         "classMethod:History.getHistoryData","classMethod:History.setHistoryTitle", "classMethod:History.addHistoryEntry"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"This class provides synthetic history support.  Using this class, you can create history\n entries at any point and be called back when the user next navigates to any of these history\n entires via any of the browser mechanisms that enable navigation: back/forward buttons,\n history dropdown and bookmarks.\n <p>\n The history entries created using this mechanism work just like history entries created\n natively by the browser, except you get a callback whenever a transition occurs.  This\n implementation correctly handles \"deep\" history - i.e. it correctly maintains forward and\n back history when the user navigates forward or back away from the page that uses this\n module.\n <p>\n This module is usable independent of the rest of SmartClient - you can use it on pages that\n don't load any other modules.\n <p>\n This module currently does not work in Safari, but works in all other browsers supported by\n SmartClient.  Also, currently, if you set document.domain on the top-level page, the History\n mechanism will behave sub-obtimally in IE - three clicks one the forward/back buttons will\n be required to transition to the next history entry.\n <p>\n <b>Usage overview</b><br>\n Synthetic history entries are added to the browser history via ${isc.DocUtils.linkForRef('classMethod:History.addHistoryEntry')}.\n When this method is called, the page's URL will be modified and the native browser back button\n will become active.<br>\n The ${isc.DocUtils.linkForRef('classMethod:History.registerCallback')} allows the developer to register a callback method to\n fire when the user navigates to these generated history entries. This method will be fired\n with an appropriate history ID when the user hits the back-button or explicitly navigates to\n the URL generated for some synthetic history entry.",
        name:"History",
        seeAlso:[],
        type:"class"
    },
    "method:List.contains":{
        ref:"method:List.contains",
        flags:"",
        description:" Return if this list contains the specified object.\n <P>\n If pos is specified, starts looking after that position.\n\n",
        name:"contains",
        seeAlso:[],
        params:[
            {description:"item to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"optional position in the list to look after", optional:true, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == item was found, false == not found", type:"boolean"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "type:HashAlgorithm":{
        ref:"type:HashAlgorithm",
        values:[
            {description:"Message Digest algorithm 5", value:"\"MD5\""},
            {description:"Secure Hashing Algorithm (SHA-1)", value:"\"SHA\""}
        ],
        description:"Algorithms to use when hashing field values (see ${isc.DocUtils.linkForRef('attr:DataSourceField.storeWithHash')})",
        name:"HashAlgorithm",
        seeAlso:[],
        type:"type"
    },
    "attr:DataBoundComponent.addDropValues":{
        ref:"attr:DataBoundComponent.addDropValues",
        flags:"IRW",
        description:"Indicates whether to add \"drop values\" to items dropped on this component, if both \n          the source and target widgets are databound, either to the same DataSource or \n          to different DataSources that are related via a foreign key.  \"Drop values\" are \n          properties of the dropped item that you wish to change (and persist) as a \n          result of the item being dropped on this grid.\n          <P>\n          If this value is true and this component is databound, ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues')} will \n          be called for every databound item dropped on this grid, and an update performed\n          on the item",
        name:"addDropValues",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('listRecategorize')}"],
        definingClass:"interface:DataBoundComponent",
        groups:["dragging"]
    },
    "attr:StretchImgButton.autoFit":{
        ref:"attr:StretchImgButton.autoFit",
        flags:"IRW",
        description:"If true, ignore the specified size of this widget and always size just large\n enough to accomodate the title.  If <code>setWidth()</code> is explicitly called on an\n autoFit:true button, autoFit will be reset to <code>false</code>.\n <P>\n Note that for StretchImgButton instances, autoFit will occur horizontally only, as \n unpredictable vertical sizing is likely to distort the media. If you do want vertical \n auto-fit, this can be achieved by simply setting a small height, and having \n overflow:\"visible\"",
        name:"autoFit",
        setter:["setAutoFit"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["sizing"]
    },
    "attr:ListGridField.summaryTitle":{
        ref:"attr:ListGridField.summaryTitle",
        flags:"IRWA",
        getter:["getSummaryTitle"],
        description:"Optional long summary title for this field, provided in addition to \n ${isc.DocUtils.linkForRef('attr:ListGridField.title')}. This gives the developer an option to use a very short,\n or empty title for the ListGrid column (where space may be a factor), but have a longer \n value available to be used elsewhere.<br>\n By default this value will be used for the title of the context-menu item\n for showing/hiding the listGrid field when the user right-clicks on the ListGrid header.",
        name:"summaryTitle",
        seeAlso:[
            {ref:"attr:ListGridField.title", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:FormulaBuilder.testFunction":{
        ref:"method:FormulaBuilder.testFunction",
        flags:"",
        description:" Test the formula by generating it's function and trying to run it \n",
        name:"testFunction",
        seeAlso:[],
        returns:{description:"result of the function", type:"string"},
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:CubeGrid.addFacet":{
        ref:"method:CubeGrid.addFacet",
        flags:"",
        description:"\n Add a facet to the view, into the row or column facets (intoRows true or false), at index\n \"index\".  Handles the facet already being in the view (does a pivot).\n <P>\n The facet being added should currently have a fixed facet value (unless it's already part of the\n view), which will be removed from cubeGrid.fixedFacetValues.\n\n",
        name:"addFacet",
        seeAlso:[
            {ref:"method:CubeGrid.removeFacet", description:null},
            {ref:"attr:CubeGrid.fixedFacetValues", description:null}
        ],
        params:[
            {description:"facetId to add.  Definition must have been provided at init time.",
             optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"whether to add facet as a row facet", optional:true, name:"intoRows",
             defaultValue:"true", type:"boolean"},
            {description:"index to add the facet at.  0 = outermost (default innermost)",
             optional:true, name:"index", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "method:TextItem.selectValue":{
        ref:"method:TextItem.selectValue",
        flags:"",
        description:" Put focus in this item and select the entire value.\n Only applies to text based items\n",
        name:"selectValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:TextItem"
    },
    "attr:Canvas.opacity":{
        ref:"attr:Canvas.opacity",
        flags:"IRWA",
        description:"Renders the widget to be partly transparent. A widget's opacity property may\n      be set to any number between 0 (transparent) to 100 (opaque).\n		Null means don't specify opacity directly, 100 is fully opaque.\n		Note that heavy use of opacity may slow down your browser.\n		See canvas.setOpacity() for details.",
        name:"opacity",
        setter:["setOpacity"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["cues"]
    },
    "attr:Scrollbar.hSrc":{
        ref:"attr:Scrollbar.hSrc",
        flags:"IR",
        description:"Base URL for the images used for the horizontal scrollbar track and end buttons.  \n <P>\n See ${isc.DocUtils.linkForRef('attr:StretchImg.items')} for a general explanation of how this base URL is\n transformed into various pieces and states.\n <P>\n For a normal 3-segment track, the suffixes \"_start\", \"_track\" and \"_end\" are added to\n this URL.  The \"start\" and \"end\" images should appear to be buttons (the user can click\n on these segments to scroll slowly).  The \"track\" segment provides a background for the\n space in which the thumb can be dragged, and can also be clicked on to scroll quickly.\n <P>\n For a 5-segment track (${isc.DocUtils.linkForRef('attr:Scrollbar.showTrackEnds')}:true), the suffixes are \"_start\", \"_track_start\",\n \"_track\", \"_track_end\" and \"_end\".",
        name:"hSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]hscroll.gif\"",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["images"]
    },
    "attr:MenuItem.submenu":{
        ref:"attr:MenuItem.submenu",
        flags:"IR",
        description:"A reference to another menu, to display as a submenu when the mouse cursor hovers over\n this menu item.",
        name:"submenu",
        seeAlso:[],
        valueType:"Menu",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('subMenus')}"],
        definingClass:"object:MenuItem",
        groups:["menuBasic"]
    },
    "object:ClassFactory":{
        methods:[],
        ref:"object:ClassFactory",
        classAttrs:[],
        classMethods:["classMethod:ClassFactory.defineClass", "classMethod:ClassFactory.getClass", 
         "classMethod:ClassFactory.newInstance"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Sets up a real inheritance structure for Javascript objects.\n	We separate out class objects from prototypes, so each gets its own inheritance chain.  \n	This allows us to set up superclass calls, maintain class vs. instance variables and more!\n\n	The ClassFactory is a singleton object that holds the miscellaneous pieces of our inheritance\n	mechanism.\n\n	Your main interaction with the ClassFactory is to create new classes:\n		<code>ClassFactory.defineClass(\"MyClass\", \"mySuperClass\");</code>",
        name:"ClassFactory",
        seeAlso:[
            {ref:"class:Class", description:null}
        ],
        type:"object"
    },
    "attr:DSRequest.componentId":{
        ref:"attr:DSRequest.componentId",
        flags:"IR",
        description:"For requests submitted by a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}, the ${isc.DocUtils.linkForRef('attr:Canvas.ID')} of the\n submitting component.\n <P>\n This ID will be present for operations including automatic saves by a ListGrid \n ${isc.DocUtils.linkForRef('group:editing','during editing')}, or calls to\n ${isc.DocUtils.linkForRef('method:DynamicForm.saveData','form.saveData()')}.  It will not be present for a direct\n call to a DataSource method such as ${isc.DocUtils.linkForRef('method:DataSource.fetchData')}.\n <P>\n Note this is the component's <b>String</b> ID - you can retrieve the component itself\n via ${isc.DocUtils.linkForRef('classMethod:Canvas.getById')}.",
        name:"componentId",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:Tree.getName":{
        ref:"method:Tree.getName",
        flags:"",
        description:"\n Get the 'name' of a node.  This is node[${isc.DocUtils.linkForRef('attr:Tree.nameProperty')}].  If that value has not\n been set on the node, a unique value (within this parent) will be auto-generated and\n returned.\n	\n",
        name:"getName",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"name of the node", type:"string"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:HeaderItem.endRow":{
        ref:"attr:HeaderItem.endRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "method:Layout.getMemberSizes":{
        ref:"method:Layout.getMemberSizes",
        flags:"",
        description:" \n",
        name:"getMemberSizes",
        seeAlso:[],
        returns:{description:"array of member sizes", type:"Array"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:HTMLFlow.dynamicContents":{
        ref:"attr:HTMLFlow.dynamicContents",
        flags:"IRWA",
        description:"Dynamic contents allows the contents string to be treated as a simple, but powerful\n template.  When this attribute is set to true, expressions of the form \\${arbitrary JS\n here} are replaced by the result of the evaluation of the JS code inside the curly\n brackets.  This evaluation happens at draw time.  If you want to trigger a re-evaluation\n of the expressions in the contents string you can call markForRedraw() on the canvas.\n <p>\n You can use this feature to build some simple custom components. For example, let's say\n you want to show the value of a Slider in a Canvas somewhere on the screen.  You can do\n this by observing the valueChanged() method on the slider and calling setContents() on\n your canvas with the new string or you can set the contents of the canvas to something\n like:\n <p><code>\n \"The slider value is \\${sliderInstance.getValue()}.\"\n </code><p>\n Next you set dynamicContents: true on the canvas, observe valueChanged() on the slider\n and call canvas.markForRedraw() in that observation.  This approach is cleaner than\n setContents() when the Canvas is aggregating several values or dynamic expressions.\n Like so:\n <p>\n <pre>\n Slider.create({\n     ID: \"mySlider\"\n });\n\n Canvas.create({\n     ID: \"myCanvas\",\n     dynamicContents: true,\n     contents: \"The slider value is \\${mySlider.getValue()}.\"\n });\n     \n myCanvas.observe(mySlider, \"valueChanged\", \n                  \"observer.markForRedraw()\");\n </pre>\n You can embed an arbitrary number of dynamic expressions in the contents string.  The\n search and replace is optimized for speed.\n <p>\n If an error occurs during the evaluation of one of the expressions, a warning is logged\n to the ISC Developer Console and the error string is embedded in place of the expected\n value in the Canvas.\n <p>\n The value of a function is its return value.  The value of any variable is the same as\n that returned by its toString() representation.\n <p>\n Inside the evalution contentext, <code>this</code> points to the canvas instance that\n has the dynamicContents string as its contents - in other words the canvas instance on\n which the template is declared.",
        name:"dynamicContents",
        seeAlso:[
            {ref:"attr:HTMLFlow.contents", description:null},
            {ref:"attr:Canvas.dynamicContentsVars", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dynamicContents')}"],
        definingClass:"class:HTMLFlow",
        groups:["contents"]
    },
    "method:StretchImgButton.setTitle":{
        ref:"method:StretchImgButton.setTitle",
        flags:"",
        description:" Set the title.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["appearance"]
    },
    "attr:ListGridField.defaultIconSrc":{
        ref:"attr:ListGridField.defaultIconSrc",
        flags:"IRA",
        description:"If this field is editable, this property will be passed to editors displayed for\n cells within this field as ${isc.DocUtils.linkForRef('attr:FormItem.defaultIconSrc')}.",
        name:"defaultIconSrc",
        seeAlso:[
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:DetailViewer.blockSeparator":{
        ref:"attr:DetailViewer.blockSeparator",
        flags:"IRW",
        description:"A string (HTML acceptable) that will be written to a page to separate blocks.",
        name:"blockSeparator",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"<BR><BR>\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:Canvas.setBorder":{
        ref:"method:Canvas.setBorder",
        flags:"",
        description:" Set the CSS border of this component, as a CSS string including border-width,\n border-style, and/or color (eg \"2px solid blue\").\n <P>\n This property applies the same border to all four sides of this component.  Different\n per-side borders can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.\n\n",
        name:"setBorder",
        seeAlso:[],
        params:[
            {description:"new border to set to (eg: \"2px solid black\")", optional:false,
             name:"newBorder", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:SectionStack.expandSection":{
        ref:"method:SectionStack.expandSection",
        flags:"",
        description:" \n Expands a section or sections.  This action shows all the items assigned to the section.\n If the section is currently hidden, it is shown first and then expanded.  Calling this\n method is equivalent to the user clicking on the SectionHeader of a collapsed section.\n\n",
        name:"expandSection",
        seeAlso:[
            {ref:"method:SectionStack.showSection", description:null},
            {ref:"attr:SectionStack.scrollSectionIntoView", description:null}
        ],
        params:[
            {description:"Section(s) to expand.  For this parameter, you can pass the position \n                      of the section in the SectionStack, the ID of the section, or a\n                      List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"},
            {description:"callback to fire when the section has been expanded.", optional:true,
             name:"callback", defaultValue:null, type:null}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"],
        definingClass:"class:SectionStack"
    },
    "attr:TabBar.tabs":{
        ref:"attr:TabBar.tabs",
        flags:"IR",
        description:"Tab for this TabBar.",
        name:"tabs",
        seeAlso:[],
        valueType:"Array of Tab Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabBar"
    },
    "attr:SectionStackSection.hidden":{
        ref:"attr:SectionStackSection.hidden",
        flags:"I",
        description:"Sections default to the visible state.  This\n attribute allows you to explicitly control the visible/hidden state of the\n section by overriding the above default behavior.",
        name:"hidden",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "attr:PickList.optionFilterContext":{
        ref:"attr:PickList.optionFilterContext",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, and this property is\n not null, this will be passed to the datasource as ${isc.DocUtils.linkForRef('class:DSRequest')} properties when\n performing the filter operation on the dataSource to obtain the set of options for the\n list.",
        name:"optionFilterContext",
        seeAlso:[],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "classMethod:isA.Instance":{
        ref:"classMethod:isA.Instance",
        flags:"",
        description:"\n	Is <code>object</code> an instance of some class?\n\n",
        name:"Instance",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is an instance of some class", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:Slider.labelSpacing":{
        ref:"attr:Slider.labelSpacing",
        flags:"IRW",
        description:"The space around the labels used to display the minimum, maximum and current values of the\n slider.",
        name:"labelSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"5",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:FormItem.showErrorIcon":{
        ref:"attr:FormItem.showErrorIcon",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:FormItem.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["errorIcon", "validation", "appearance"]
    },
    "group:performance":{
        ref:"group:performance",
        refs:["attr:GridRenderer.showAllRows", "attr:GridRenderer.showAllColumns", 
         "attr:GridRenderer.drawAllMaxCells","attr:GridRenderer.scrollRedrawDelay", "attr:GridRenderer.drawAheadRatio", 
         "attr:GridRenderer.quickDrawAheadRatio","attr:ListGrid.fastCellUpdates", "attr:ListGrid.showAllRecords", 
         "attr:ListGrid.drawAllMaxCells","attr:ListGrid.drawAheadRatio", "attr:ListGrid.quickDrawAheadRatio", 
         "attr:ListGrid.scrollRedrawDelay","attr:ListGrid.dataPageSize"],
        name:"performance",
        seeAlso:[],
        type:"group"
    },
    "group:loadState":{
        ref:"group:loadState",
        refs:["method:ResultTree.loadChildren", "method:ResultTree.unloadChildren", 
         "method:Tree.loadChildren","method:Tree.unloadChildren", "method:Tree.reloadChildren"],
        name:"loadState",
        seeAlso:[],
        type:"group"
    },
    "method:MultiView.selectAllRecords":{
        ref:"method:MultiView.selectAllRecords",
        flags:"",
        description:" Select all records\n\n",
        name:"selectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:SelectItem.textBoxStyle":{
        ref:"attr:SelectItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class name for a form item's text box element.\n <P>\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"textBoxStyle",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"selectItemText\", [IRA]",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "attr:Canvas.mouseStillDownInitialDelay":{
        ref:"attr:Canvas.mouseStillDownInitialDelay",
        flags:"IRWA",
        description:"Amount of time (in msec) before mouseStillDown events start to be fired for this object.",
        name:"mouseStillDownInitialDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"400",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["events"]
    },
    "attr:ListGridField.canFilter":{
        ref:"attr:ListGridField.canFilter",
        flags:"IRW",
        description:"If showing a filter row for this ListGrid, should the filter criteria for this \n      field be editable",
        name:"canFilter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disableFilter')}"],
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "attr:MultiSortPanel.directionFieldTitle":{
        ref:"attr:MultiSortPanel.directionFieldTitle",
        flags:"IR",
        description:"The title-text to appear in the header of the \"direction\" field.",
        name:"directionFieldTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Order\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "method:ResultSet.compareCriteria":{
        ref:"method:ResultSet.compareCriteria",
        flags:"",
        description:" Default behavior is to call ${isc.DocUtils.linkForRef('method:DataSource.compareCriteria')} to determine whether new\n criteria is guaranteed more restrictive, equivalent to the old criteria, or not guaranteed\n more restrictive, returning 1, 0 or -1 respectively.\n <P>\n Override this method or ${isc.DocUtils.linkForRef('method:DataSource.compareCriteria')} to implement your own client-side\n filtering behavior.\n\n",
        name:"compareCriteria",
        seeAlso:[
            {ref:"type:CriteriaPolicy", description:null}
        ],
        params:[
            {description:"new filter criteria", optional:false, name:"newCriteria", defaultValue:null,
             type:"Criteria"},
            {description:"old filter criteria", optional:false, name:"oldCriteria", defaultValue:null,
             type:"Criteria"},
            {description:"dataSource request properties", optional:true, name:"requestProperties",
             defaultValue:null, type:"DSRequest Properties"},
            {description:"overrides ${isc.DocUtils.linkForRef('type:CriteriaPolicy')}", optional:true,
             name:"policy", defaultValue:null, type:"string"}
        ],
        returns:{description:"0 if the filters are equivalent, 1 if newFilter is guaranteed more                      restrictive, and -1 if newFilter is not guaranteed more restrictive",
         type:"Number"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:TreeNode.isFolder":{
        ref:"attr:TreeNode.isFolder",
        flags:"IR",
        description:"Set to <code>true</code> or a string that is not equal to (ignoring case)\n <code>\"false\"</code> to explicitly mark this node as a folder.  See ${isc.DocUtils.linkForRef('method:Tree.isFolder')} for\n a full description of how the ${isc.DocUtils.linkForRef('class:Tree')} determines whether a node is a folder or not.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty')}.",
        name:"isFolder",
        seeAlso:[
            {ref:"attr:Tree.isFolderProperty", description:null}
        ],
        valueType:"Boolean or String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:RPCResponse.transactionNum":{
        ref:"attr:RPCResponse.transactionNum",
        flags:"R",
        description:"ID of the transaction sent to the server via ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendQueue')} containing the\n ${isc.DocUtils.linkForRef('class:RPCRequest')} associated with this response.",
        name:"transactionNum",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCResponse"
    },
    "attr:Canvas.hoverWrap":{
        ref:"attr:Canvas.hoverWrap",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, this property can be used to customize the\n whether content in the hover canvas is displayed in a single line, or wraps.",
        name:"hoverWrap",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:SelectItem.pickListProperties":{
        ref:"attr:SelectItem.pickListProperties",
        flags:"IRA",
        description:"If specified this properties block will be applied to the ${isc.DocUtils.linkForRef('class:PickListMenu','pickList')}\n created for this Form Item.",
        name:"pickListProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:DataSource.beanClassName":{
        ref:"attr:DataSource.beanClassName",
        flags:"IR",
        description:"This property has different meanings depending on the ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')}:\n <p>\n <b>For DataSources with serverType \"hibernate\"</b><br>\n The name of the Java bean or POJO class that is mapped in Hibernate.  This will typically \n be the fully-qualified class name - eg <code>com.foo.MyClass</code> - but it may be the \n simple class name - just <code>MyClass</code> - or it may be some other value.  It all \n depends on how your classes have been mapped in Hibernate.\n <p>\n Note:  If you are intending to use Hibernate as a data-access layer only, \n you do not need to create Hibernate mappings or Java objects: SmartClient will generate\n everything it needs on the fly.\n <p>\n <b>For DataSources with serverType \"sql\"</b><br>\n If set, results from the database will be used to create one instance of the indicated Java\n bean per database row.  Otherwise a Map is used to represent each row retrieved from SQL.\n <P>\n With this feature active, a DSResponse from this DataSource will contain a Collection of\n instances of the indicated <code>beanClassName</code>, available via DSResponse.getData().\n This creates a couple of possibilities:\n <P>\n <dl>\n <dt>Add business logic for derived properties, such as computed formulas</dt>\n <dd>For example, declare a DataSourceField named \"revenueProjection\".  By default this\n field will call getRevenueProjection() on your bean to retrieve the value to send to the\n client.  Your implementation of getRevenueProjection() could apply some kind of formula to\n other values loaded from the database.</dd>\n <dt>Call business logic on retrieved beans via DMI</dt>\n <dd>By adding a ${isc.DocUtils.linkForRef('class:DMI')} method that calls DSRequest.execute() to retrieve a DSResponse,\n you have an opportunity to call business logic methods on the beans representing each\n row affected by the DSRequest.  For example, notify a related BPEL process of changes to\n certain fields.</dd>\n </dl>\n <P>\n By using <code>beanClassName</code> on a specific ${isc.DocUtils.linkForRef('class:OperationBinding')}, you can:\n <ul>\n <li>Use a bean to represent your data only when it matters; for example, avoid the overhead\n     of using a bean for \"fetch\" operations, but do use a bean for \"update\" operations so\n     that you can execute relevant business logic after the update completes.</li>\n <li>Skip the use of beans for complex reporting queries that produce results unrelated to\n     your persistent object model.  Set beanClassName to blank (\"\") on a specific\n     operationBinding to override DataSource.beanClassName for that specific operation.</li>\n <li>For SQL joins that produce additional data fields, use a special, operation-specific\n     bean that represents a join of multiple entities and contains business logic specific\n     to that joined dataset</li>\n </ul>\n <P>\n Note that setting <code>beanClassName</code> has no effect on the DSRequest.  However,\n ${isc.DocUtils.linkForRef('class:DMI')} has a built-in facility for populating a bean with the inbound\n ${isc.DocUtils.linkForRef('attr:DSRequest.data')} - just declare the bean as a method argument.\n <p>\n <b>For DataSources with serverType \"generic\"</b><br>\n ${isc.DocUtils.linkForRef('group:visualBuilder','Visual Builder')} sets this property when it creates a generic \n DataSource using the Javabean Wizard.  It has no built-in server-side effects.",
        name:"beanClassName",
        seeAlso:["OperationBinding.beanClassName"],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "group:monthViewEvents":{
        ref:"group:monthViewEvents",
        refs:["method:Calendar.dayBodyClick", "method:Calendar.dayHeaderClick", 
         "method:Calendar.eventChanged","method:Calendar.eventRemoved", "method:Calendar.eventAdded", "method:Calendar.eventClick", 
         "method:Calendar.eventRemoveClick","method:Calendar.eventMoved", "method:Calendar.eventResized"],
        name:"monthViewEvents",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.containsPoint":{
        ref:"method:Canvas.containsPoint",
        flags:"A",
        description:" Return whether or not this object contains the specified global (x,y) coordinates.\n <P>\n Will return false if any parentElement does not contain the specified point, (EG:\n you're hovering over an element's absolute location, but it is scrolled out of \n view in a parent element)\n\n",
        name:"containsPoint",
        seeAlso:[],
        params:[
            {description:"GLOBAL x-coordinate", optional:false, name:"x", defaultValue:null,
             type:"number"},
            {description:"GLOBAL y-coordinate", optional:false, name:"y", defaultValue:null,
             type:"number"},
            {description:"point lies specificly within our viewport                                              (drawn area excluding margins and scrollbars if\n                                              present)",
             optional:true, name:"withinViewport", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if this object contains the specified point; false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:RestDataSource.dataURL":{
        ref:"attr:RestDataSource.dataURL",
        flags:"IR",
        description:"Default URL to contact to fulfill all DSRequests.  \n RestDataSources also allow per-operationType dataURLs to be set via\n <ul>\n <li>${isc.DocUtils.linkForRef('attr:RestDataSource.fetchDataURL')}</li>\n <li>${isc.DocUtils.linkForRef('attr:RestDataSource.addDataURL')}</li> \n <li>${isc.DocUtils.linkForRef('attr:RestDataSource.updateDataURL')}</li>\n <li>${isc.DocUtils.linkForRef('attr:RestDataSource.removeDataURL')}</li>\n </ul>",
        name:"dataURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:Dialog.messageStyle":{
        ref:"attr:Dialog.messageStyle",
        flags:"IA",
        description:"Style to apply to the message text shown in the center of the dialog",
        name:"messageStyle",
        seeAlso:[],
        valueType:"CSSStyle",
        defaultValue:"\"normal\"",
        type:"attr",
        definingClass:"class:Dialog"
    },
    "attr:FormulaBuilder.dataSource":{
        ref:"attr:FormulaBuilder.dataSource",
        flags:"IRW",
        description:"DataSource providing the available fields for the formulaBuilder.\n <P>\n By default the formulaBuilder will include <b>only</b> fields of numeric type or derived\n from a numeric type.  Set ${isc.DocUtils.linkForRef('attr:FormulaBuilder.fields')} to override this.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:MultiSortPanel.deleteLevelButton":{
        ref:"attr:MultiSortPanel.deleteLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for deleting levels\n from the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.deleteLevelButtonProperties</code> and \n <code>multiSortPanel.deleteLevelButtonDefaults</code>.",
        name:"deleteLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "attr:SectionStack.overflow":{
        ref:"attr:SectionStack.overflow",
        flags:"IR",
        description:"Normal ${isc.DocUtils.linkForRef('type:Overflow')} settings can be used on layouts, for example, an\n overflow:auto Layout will scroll if sections are resized to exceed the specified size,\n whereas an overflow:visible Layout will grow to accomodate the resized sections.",
        name:"overflow",
        seeAlso:[],
        valueType:"Overflow",
        defaultValue:"\"hidden\"",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "attr:Canvas.masterElement":{
        ref:"attr:Canvas.masterElement",
        flags:"RA",
        description:"This Canvas's \"master\" (the Canvas to which it was added as a peer), if any.",
        name:"masterElement",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "attr:TreeGrid.cantDragIntoChildMessage":{
        ref:"attr:TreeGrid.cantDragIntoChildMessage",
        flags:"IR",
        description:"Message displayed when user attempts to drop a node into a child of itself.",
        name:"cantDragIntoChildMessage",
        seeAlso:[
            {ref:"attr:TreeGrid.canDragRecordsOut", description:null},
            {ref:"attr:TreeGrid.canAcceptDroppedRecords", description:null},
            {ref:"attr:TreeGrid.canReorderRecords", description:null}
        ],
        valueType:"String",
        defaultValue:"\"You can't drag an item into one of it's children.\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["i18nMessages"]
    },
    "attr:DataSource.validateRelatedRecords":{
        ref:"attr:DataSource.validateRelatedRecords",
        flags:"IR",
        description:"If true, indicates that the SmartClient Server should automatically apply a \n ${isc.DocUtils.linkForRef('classAttr:ValidatorTypes.hasRelatedRecord','hasRelatedRecord')} validator to every field \n on this dataSource that has a ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} defined.",
        name:"validateRelatedRecords",
        seeAlso:[
            {ref:"classAttr:ValidatorTypes.hasRelatedRecord", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSource"
    },
    "attr:ListGridField.cellAlign":{
        ref:"attr:ListGridField.cellAlign",
        flags:"IRW",
        description:"Horizontal alignment for cells in this field's column: \"left\", \"right\"\n or \"center\".<br>\n If null, alignment is derived from ${isc.DocUtils.linkForRef('attr:ListGridField.align')}. If this field is editable, \n the alignment of cells in the body will also be reflected in any editors for the field.",
        name:"cellAlign",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "attr:Canvas.animateShowEffect":{
        ref:"attr:Canvas.animateShowEffect",
        flags:"IRWA",
        description:"Default animation effect to use if ${isc.DocUtils.linkForRef('method:Canvas.animateShow')} is called without an\n explicit <code>effect</code> parameter",
        name:"animateShowEffect",
        seeAlso:[],
        valueType:"animateShowEffectId | animateShowEffect",
        defaultValue:"\"wipe\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "group:resizing":{
        ref:"group:resizing",
        refs:["attr:Window.canDragResize"],
        name:"resizing",
        seeAlso:[],
        type:"group"
    },
    "attr:RestDataSource.recordXPath":{
        ref:"attr:RestDataSource.recordXPath",
        flags:"IRW",
        description:"For RestDataSources, by default, either the ${isc.DocUtils.linkForRef('attr:RestDataSource.xmlRecordXPath')} or \n ${isc.DocUtils.linkForRef('attr:RestDataSource.jsonRecordXPath')} is used based on the ${isc.DocUtils.linkForRef('attr:RestDataSource.dataFormat')}\n setting.\n <P>\n Note that you can also apply record xpath binding via\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath')}.",
        name:"recordXPath",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:ListGrid.showHeaderMenuButton":{
        ref:"attr:ListGrid.showHeaderMenuButton",
        flags:"IR",
        description:"If set to true and ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','showHeaderContextMenu')} is true, the\n ${isc.DocUtils.linkForRef('attr:ListGrid.headerMenuButton')} will be displayed when the user rolls\n over the header buttons in this grid.",
        name:"showHeaderMenuButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "method:PanelHeader.setTitleStyle":{
        ref:"method:PanelHeader.setTitleStyle",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:PanelHeader.titleStyle')}, which is applied to the title text.\n",
        name:"setTitleStyle",
        seeAlso:[],
        params:[
            {description:"new title style", optional:false, name:"style", defaultValue:null,
             type:"CSSStyleName"}
        ],
        type:"method",
        definingClass:"class:PanelHeader"
    },
    "attr:FormItem.emptyValueIcon":{
        ref:"attr:FormItem.emptyValueIcon",
        flags:"IRW",
        description:"This property allows the developer to specify an icon to display when this item has\n no value. It is configured in the same way as any other valueIcon \n (see ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')})",
        name:"emptyValueIcon",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:Calendar.autoFetchTextMatchStyle":{
        ref:"attr:Calendar.autoFetchTextMatchStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:Calendar.autoFetchData')} is <code>true</code>, this attribute allows the developer to\n specify a textMatchStyle for the initial ${isc.DocUtils.linkForRef('method:Calendar.fetchData')} call.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["databinding"]
    },
    "method:Tree.getRoot":{
        ref:"method:Tree.getRoot",
        flags:"",
        description:"\n Returns the root node of the tree.\n\n",
        name:"getRoot",
        seeAlso:[],
        returns:{description:"the root node", type:"TreeNode"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:SelectItem.pickerIconHeight":{
        ref:"attr:SelectItem.pickerIconHeight",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:SelectItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset, the picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["pickerIcon"]
    },
    "group:loadWSDLTag":{
        ref:"group:loadWSDLTag",
        title:"&lt;isomorphic:loadWSDL&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n Load a WSDL file and create an instance of ${isc.DocUtils.linkForRef('class:WebService')} that allows invoking operations\n and binding DataSources to web service operations. This tag works just like\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')}, except it's synchronous and the result is server-cacheable.\n <p>\n Note that this JSP tag must be surrounded by &lt;SCRIPT&gt; tags in the JSP because it\n generates JavaScript code.  Like other tags that generate JavaScript code, this tag can be\n used in a JSP that is included from your main page in order to create separate\n cacheability.  For example:<pre>\n     &lt;SCRIPT SRC=\"myWebServiceDefinition.jsp\"&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>url</b><br>\n <i>value format</i>: URL or URI\n <i>default value</i>: NONE\n <p>\n This attribute specifies the URL or URI of the WSDL file to fetch and translate.  This can\n be either a remote URL - e.g: <code>http://host:port/wsdlFile.wsdl</code> or a relative or\n absolute URI to a file local to this container - e.g: <code>/some/wsdlFile.wsdl</code> or\n <code>../some/wsdlFile.wsdl</code>.  If the url is a remote URL, then an HTTP request will\n be made for the file.  If it is local, it will be fetched from disk using standard Servlet\n APIs (<code>ServletContext.getResourceAsStream()</code>).\n <p>\n <b>cache</b><br>\n <i>value format</i>: Integer (number of seconds to cache result)\n <i>default value</i>: 3600 (1 hour)\n <p>\n This attribute specifies the number of seconds for which the fetched WSDL is cacheable on\n the server.  Fetching a WSDL file from a remote server can cause a significant delay in JSP\n processing, and WSDL files rarely change outside of a development environment.  Set this\n value to zero to disable caching.",
        name:"loadWSDLTag",
        seeAlso:[
            {ref:"group:wsdlBinding", description:null},
            {ref:"classMethod:XMLTools.loadWSDL", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "attr:TreeGridField.canExport":{
        ref:"attr:TreeGridField.canExport",
        flags:"IR",
        description:"Dictates whether the data in this field be exported.  Explicitly set this\n  to false to prevent exporting.  Has no effect if the underlying \n  ${isc.DocUtils.linkForRef('attr:DataSourceField.canExport','dataSourceField')} is explicitly set to \n  canExport: false.",
        name:"canExport",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeGridField"
    },
    "method:SectionStack.showSection":{
        ref:"method:SectionStack.showSection",
        flags:"",
        description:" \n Shows a section or sections.  This includes the section header and its items.  If the\n section is collapsed, only the header is shown.  If the section is expanded, the section\n header and all items are shown.\n\n",
        name:"showSection",
        seeAlso:[
            {ref:"method:SectionStack.expandSection", description:null},
            {ref:"attr:SectionStack.scrollSectionIntoView", description:null}
        ],
        params:[
            {description:"Section(s) to show.  For this parameter, you can pass the position \n                      of the section in the SectionStack, the ID of the section, or a\n                      List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"},
            {description:"callback to fire when the sections have been expanded.", optional:true,
             name:"callback", defaultValue:null, type:null}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsShowAndHide')}"],
        definingClass:"class:SectionStack"
    },
    "attr:DateItem.startDate":{
        ref:"attr:DateItem.startDate",
        flags:"IRW",
        description:"Minimum date the selectors will allow the user to pick.",
        name:"startDate",
        seeAlso:[],
        valueType:"Date",
        defaultValue:"1/1/1995",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["appearance"]
    },
    "attr:Mail.messageTemplate":{
        ref:"attr:Mail.messageTemplate",
        flags:"IR",
        description:"The text that will form the body of the mail message.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n You must specify either this property or ${isc.DocUtils.linkForRef('attr:Mail.templateFile','templateFile')}, but clearly\n it makes no sense to specify them both.",
        name:"messageTemplate",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "attr:FormItem.optionDataSource":{
        flags:"IR",
        getter:["getOptionDataSource"],
        type:"attr",
        ref:"attr:FormItem.optionDataSource",
        description:"If set, this FormItem will map stored values to display values as though a\n ${isc.DocUtils.linkForRef('type:ValueMap')} were specified, by fetching records from the \n specified <code>optionDataSource</code> and extracting the\n ${isc.DocUtils.linkForRef('attr:FormItem.valueField','valueField')} and \n ${isc.DocUtils.linkForRef('attr:FormItem.displayField','displayField')} in loaded records, to derive one\n valueMap entry per record loaded from the optionDataSource.\n <P>\n With the default setting of ${isc.DocUtils.linkForRef('attr:FormItem.fetchMissingValues')}, fetches will be initiated against\n the optionDataSource any time the FormItem has a non-null value and no corresponding\n display value is available.  This includes when the form is first initialized, as well\n as any subsequent calls to ${isc.DocUtils.linkForRef('method:FormItem.setValue')}, such as may happen when\n ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} is called.  Retrieved values are automatically cached by\n the FormItem.\n <P>\n Note that if a normal, static ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} is <b>also</b> specified for\n the field (either directly in the form item or as part of the field definition in the\n dataSource), it will be preferred to the data derived from the optionDataSource for\n whatever mappings are present.\n <P>\n In a databound form, if ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} is specified for a FormItem and \n <code>optionDataSource</code> is unset, <code>optionDataSource</code> will default to\n the form's current DataSource",
        name:"optionDataSource",
        valueType:"DataSource | String",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('listComboBox')}"],
        defaultValue:"null",
        groups:["display_values"],
        definingClass:"class:FormItem"
    },
    "method:Class.getCallTrace":{
        ref:"method:Class.getCallTrace",
        flags:"",
        description:" Returns a one-line summary of the current method call, showing method name and passed\n arguments. \n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getCallTrace()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"getCallTrace",
        seeAlso:[],
        params:[
            {description:"arguments object from the call to trace.  On IE, defaults to the                             calling function's arguments",
             optional:true, name:"args", defaultValue:null, type:"Arguments"}
        ],
        type:"method",
        platformNotes:"IE only.",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "attr:StatefulCanvas.overCanvas":{
        ref:"attr:StatefulCanvas.overCanvas",
        flags:"R",
        description:"Auto generated child widget to be shown when the user rolls over this canvas if \n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showOverCanvas')} is true. See documentation for ${isc.DocUtils.linkForRef('type:AutoChild')}\n for information on how to customize this canvas.",
        name:"overCanvas",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas"
    },
    "attr:DetailViewerField.title":{
        ref:"attr:DetailViewerField.title",
        flags:"IR",
        description:"The title of the field as displayed on the left-hand side.  If left unspecified, the\n title of the field is derived by looking up the value of\n ${isc.DocUtils.linkForRef('attr:DetailViewer.fieldIdProperty')} on this field.  So, by default, the title of a field\n is the value of its \"name\" property.",
        name:"title",
        seeAlso:[
            {ref:"attr:DetailViewer.fieldIdProperty", description:null}
        ],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "attr:LoginDialog.showCloseButton":{
        ref:"attr:LoginDialog.showCloseButton",
        flags:"IRW",
        description:"Do not set LoginDialog.showCloseButton; it is controlled by the \n ${isc.DocUtils.linkForRef('attr:LoginDialog.dismissable')}\n property.",
        name:"showCloseButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:ImgButton.showDownIcon":{
        ref:"attr:ImgButton.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "class:UploadItem":{
        methods:["method:UploadItem.setValue"],
        exampleConfig:"UploadItem.xml",
        ref:"class:UploadItem",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"FormItem that creates an HTML &lt;input type=\"upload\"&gt; control, with an interface that\n allows a user to pick a file from his machine to upload to the server.\n <P>\n See the ${isc.DocUtils.linkForRef('group:upload','Uploading Files')} overview for details.\n <P>\n Note: if a form containing an UploadItem is ${isc.DocUtils.linkForRef('method:Canvas.redraw','redrawn')} (which may\n happen if other form items are shown or hidden, the form is\n ${isc.DocUtils.linkForRef('attr:Canvas.redrawOnResize','resized')}, or other items show validation errors) then the value\n in the upload item is lost (because an HTML upload field may not be created with a value).\n For this reason, if you are building a form that combines an UploadItem with other FormItems\n that could trigger redraw()s, recommended practice is to place each UploadItem in a distinct\n DynamicForm instance and create the visual appearance of a single logical form via combining\n the DynamicForms in a ${isc.DocUtils.linkForRef('class:Layout')}.",
        name:"UploadItem",
        seeAlso:[],
        type:"class",
        groups:["upload"]
    },
    "method:DynamicForm.hasErrors":{
        ref:"method:DynamicForm.hasErrors",
        flags:"",
        description:" Return whether this form currently has any validation errors.<br>\n Validation errors are set up automatically by validation, or may be explicitly set via\n ${isc.DocUtils.linkForRef('method:DynamicForm.setErrors')} or ${isc.DocUtils.linkForRef('method:DynamicForm.setFieldErrors')}.\n",
        name:"hasErrors",
        seeAlso:[],
        returns:{description:"true == form currently has validation errors.", type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "method:Canvas.dragResizeStart":{
        ref:"method:Canvas.dragResizeStart",
        flags:"A",
        description:"\n Executed when resize dragging first starts. No default implementation.  \n Create this handler to set things up for the drag resize.\n\n",
        name:"dragResizeStart",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to cancel the drag reposition action", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "type:ViewName":{
        ref:"type:ViewName",
        values:[
            {description:"day view", value:"\"day\""},
            {description:"week view", value:"\"week\""},
            {description:"month view", value:"\"month\""}
        ],
        description:"The names of the Calendar views.",
        name:"ViewName",
        seeAlso:[],
        type:"type"
    },
    "attr:FormItemIcon.height":{
        ref:"attr:FormItemIcon.height",
        flags:"IRW",
        description:"If set, this property determines the height of this icon in px.\n      If unset the form item's <code>iconHeight</code> property will be used instead.",
        name:"height",
        seeAlso:[
            {ref:"attr:FormItem.iconHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "attr:Canvas.animateHideAcceleration":{
        ref:"attr:Canvas.animateHideAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated hide.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateHideAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:SliderItem.setMinValue":{
        ref:"method:SliderItem.setMinValue",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.minValue','minimum value')} of the slider \n\n",
        name:"setMinValue",
        seeAlso:[],
        params:[
            {description:"the new minimum value", optional:false, name:"newValue", defaultValue:null,
             type:"float"}
        ],
        type:"method",
        definingClass:"class:SliderItem"
    },
    "method:DataSource.getAutoTitle":{
        ref:"method:DataSource.getAutoTitle",
        flags:"",
        description:" Return a reasonable user-visible title given a fieldName.  Called when\n ${isc.DocUtils.linkForRef('attr:DataSource.autoDeriveTitles')} is true and by default, calls the class method\n ${isc.DocUtils.linkForRef('classMethod:DataSource.getAutoTitle')}.  Override to provide a different policy\n for auto-deriving titles for a particular DataSource or subclass of DataSource.\n\n",
        name:"getAutoTitle",
        seeAlso:[],
        params:[
            {description:"identifier for which a title is desired.", optional:false, name:"identifier",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"auto-derived title", type:"String"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["title"]
    },
    "attr:ColorItem.showPickerIcon":{
        ref:"attr:ColorItem.showPickerIcon",
        flags:"IRW",
        description:"Should the pick button icon be shown for choosing colors from a ColorPicker",
        name:"showPickerIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "attr:DataSource.fields":{
        ref:"attr:DataSource.fields",
        flags:"IR",
        description:"The list of fields that compose records from this DataSource.\n <P>\n Each DataSource field can have type, user-visible title, validators, and other metadata\n attached.",
        name:"fields",
        seeAlso:[
            {ref:"class:DataSourceField", description:null}
        ],
        valueType:"Array of DataSourceField",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["fields"]
    },
    "classMethod:Class.create":{
        ref:"classMethod:Class.create",
        flags:"",
        description:"\n Create an instance of this class.  \n <P>\n All arguments passed to this method are passed on to the ${isc.DocUtils.linkForRef('method:Class.init')} instance \n method.  Unless ${isc.DocUtils.linkForRef('attr:Class.addPropertiesOnCreate')} is set to <code>false</code>, all\n arguments passed to this method must be Objects and all properties on those\n objects will be copied to the newly created instance before ${isc.DocUtils.linkForRef('method:Class.init')} is\n called.  If there are overlapping properties in the passed arguments, the last wins.\n <p>\n Any return value from ${isc.DocUtils.linkForRef('method:Class.init')} is thrown away.\n <p>\n Note: Generally, you would not override this method.  If you want to specify a\n constructor for your class, provide an override for ${isc.DocUtils.linkForRef('method:Class.init')} for generic\n classes or ${isc.DocUtils.linkForRef('method:Canvas.initWidget')} for any subclasses of UI components\n (i.e. descendants of ${isc.DocUtils.linkForRef('class:Canvas')}.\n\n",
        name:"create",
        seeAlso:[],
        params:[
            {description:"Any arguments passed will be passed along to the init() routine of the instance.\n      Unless ${isc.DocUtils.linkForRef('attr:Class.addPropertiesOnCreate')} is set to false, any arguments passed to\n      this method must be of type Object.",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"any"}
        ],
        returns:{description:"New instance of this class, whose init() routine has already been called",
         type:"object"},
        type:"classMethod",
        examples:["<code>var myInstance = MyClass.create()</code>", 
         "${isc.DocUtils.linkForExampleId('create')}"],
        definingClass:"class:Class"
    },
    "method:ListGridField.getGroupValue":{
        ref:"method:ListGridField.getGroupValue",
        flags:"",
        description:" Return the value which records should be grouped by.  \n <P>\n All records for which getGroupValue() returns the same value appear in the same\n group.  Default is the result of ${isc.DocUtils.linkForRef('method:ListGrid.getCellValue')}.\n <P>\n While any type of value may be returned, avoiding the use of string values may\n result in improved performance. In this case, ${isc.DocUtils.linkForRef('method:ListGridField.getGroupTitle')}\n may be implemented to map a numeric group value into a legible string.\n\n",
        name:"getGroupValue",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null},
            {ref:"method:ListGridField.getGroupTitle", description:null}
        ],
        params:[
            {description:"raw value for the cell, from the record for the row", optional:false,
             name:"value", defaultValue:null, type:"any"},
            {description:"Record object for the cell. Note: If this is a new row that has not been saved, in an \n   editable grid, it has no associated record object. In this case the edit values will\n   be passed in as this parameter (see ${isc.DocUtils.linkForRef('method:ListGrid.getEditValues')})",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"Field object for which to get group value", optional:false, name:"field",
             defaultValue:null, type:"Object"},
            {description:"The name of the field", optional:false, name:"fieldName", defaultValue:null,
             type:"String"},
            {description:"the ListGrid displaying the cell", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"Group value to which this record belongs", type:"any"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customGrouping')}"],
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "method:Canvas.removeChild":{
        ref:"method:Canvas.removeChild",
        flags:"",
        description:"	Remove a child from its parent if it has one.\n",
        name:"removeChild",
        seeAlso:[],
        params:[
            {description:"Child canvas to remove from this parent.", optional:false, name:"child",
             defaultValue:null, type:"Canvas instance"},
            {description:"If the child canvas was assigned a name, it should be passed in here                          to ensure this[name] is cleared",
             optional:true, name:"name", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "group:errorHandling":{
        ref:"group:errorHandling",
        refs:["method:DataSource.handleError", "attr:DSResponse.status", "attr:DSResponse.errors"],
        name:"errorHandling",
        seeAlso:[],
        type:"group"
    },
    "attr:ToolbarItem.colSpan":{
        ref:"attr:ToolbarItem.colSpan",
        flags:"IRW",
        description:"these items span all columns",
        name:"colSpan",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["appearance"]
    },
    "attr:LoginDialog.loginButton":{
        ref:"attr:LoginDialog.loginButton",
        flags:"IR",
        description:"Login submission button in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"loginButton",
        seeAlso:[
            {ref:"attr:LoginDialog.loginButtonTitle", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:FormulaBuilder.getTestRecord":{
        ref:"method:FormulaBuilder.getTestRecord",
        flags:"",
        description:" Gets the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.testRecord','test record')} for this formula.\n",
        name:"getTestRecord",
        seeAlso:[],
        returns:{description:"the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.testRecord','testRecord')} for this formula",
         type:"Object"},
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "classAttr:FileLoader.defaultModules":{
        ref:"classAttr:FileLoader.defaultModules",
        flags:"RW",
        description:"Default list of modules for\n ${isc.DocUtils.linkForRef('classMethod:FileLoader.loadISC')}/${isc.DocUtils.linkForRef('classMethod:FileLoader.cacheISC')} to use if none are\n provided by the user.",
        name:"defaultModules",
        seeAlso:[
            {ref:"classMethod:FileLoader.cacheISC", description:null},
            {ref:"classMethod:FileLoader.loadISC", description:null}
        ],
        valueType:"String",
        defaultValue:"\"Core,Foundation,Containers,Grids,Forms,DataBinding\"",
        type:"classAttr",
        definingClass:"class:FileLoader"
    },
    "method:DataSource.transformRequest":{
        ref:"method:DataSource.transformRequest",
        flags:"A",
        description:" For a dataSource using ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side data integration')},\n return the data that should be sent to the ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')}.\n <P>\n By default, HTTP requests sent to non-SmartClient servers do not include DSRequest\n metadata such as ${isc.DocUtils.linkForRef('attr:DSRequest.startRow')}, ${isc.DocUtils.linkForRef('attr:DSRequest.endRow','endRow')},\n and ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues','oldValues')}.  Only the core\n ${isc.DocUtils.linkForRef('group:dataSourceOperations','datasource protocol data')} is sent, such as the criteria\n passed to ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} or the updated values submitted by\n ${isc.DocUtils.linkForRef('method:DynamicForm.saveData','form.saveData()')}.  \n <P>\n transformRequest() allows you to transform dsRequest metadata into a\n format understood by your server and include it in the HTTP request, so that you can\n integrate DataSource features such as data paging with servers that support such\n features.\n <P>\n How the data is actually sent to the URL is controlled by \n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')}.  If using the \"getParams\" or\n \"postParams\" protocol, data is expected to be a JavaScript Object where each property\n will become a GET or POST'd parameter.  If using dataProtocol:\"soap\" or \"postXML\", data\n will be serialized as an XML message by ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}.\n <P>\n As an example, if you have a dataURL that can return paged data given URL parameters\n \"start\" and \"end\", you could implement transformRequest like so:\n <pre>\n   isc.DataSource.create({\n      ... \n      transformRequest : function (dsRequest) {\n         if (dsRequest.operationType == \"fetch\") {\n             var params = {\n                start : dsRequest.startRow,\n                end : dsRequest.endRow\n             };\n             // combine paging parameters with criteria\n             return isc.addProperties({}, dsRequest.data, params);\n         }\n      }\n   });\n </pre>\n Other reasons to implement transformRequest():\n <ul>\n <li> transform a ${isc.DocUtils.linkForRef('type:Criteria')} object into the custom query language of a web\n      service\n <li> add a session id to requests that require authentication\n <li> detect colliding updates by sending both updated values and the values the user\n      originally retrieved before editing began (available as ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues')})\n </ul>\n <i>Special case:</i> If the <code>dataProtocol</code> is <code>\"clientCustom\"</code>\n the SmartClient system will not attempt to send data to the server in any way. Instead \n transformRequest should be implemented such that it accesses or updates the underlying\n data-set and calls ${isc.DocUtils.linkForRef('method:DataSource.processResponse')} when the operation is complete. This\n setting allows straightforward integration with non SmartClient comm mechanisms that \n directly send requests to the server (such as GWT-RPC), or handle data manipulation without\n sending HTTP at all (such as Google Gears).\n <P>\n Note: The ${isc.DocUtils.linkForRef('class:RestDataSource')} class overrides transformRequest() to handle xml-serializing\n the request (including meta data) into a standard format.\n\n",
        name:"transformRequest",
        seeAlso:[],
        params:[
            {description:"the DSRequest being processed", optional:false, name:"dsRequest",
             defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:"data to be sent to the dataURL", type:"any"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:DataSourceField.editorType":{
        ref:"attr:DataSourceField.editorType",
        flags:"IR",
        description:"Sets the default FormItem to be used whenever this field is edited (whether in a grid, form,\n or other component).\n <P>\n If unset, a FormItem will be automatically chosen based on the type of the field, by the\n rules explained ${isc.DocUtils.linkForRef('type:FormItemType','here')}.",
        name:"editorType",
        seeAlso:[],
        valueType:"FormItem className",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "attr:FormItem.showErrorText":{
        ref:"attr:FormItem.showErrorText",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:FormItem.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorText",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["validation", "appearance"]
    },
    "attr:FormulaBuilder.testRecord":{
        ref:"attr:FormulaBuilder.testRecord",
        flags:"IRA",
        getter:["getTestRecord"],
        description:"Record to use when testing the formula dynamically (if ${isc.DocUtils.linkForRef('attr:FormulaBuilder.autoTest')} is enabled) or when\n showing samples of formula output.\n <P>\n If not specified, the selected record in the component that launched the FormulaBuilder will\n be used, or if there's no selection, the first visible row, or with no component, a dummy\n data row derived automatically from the provided DataSource.",
        name:"testRecord",
        seeAlso:[],
        valueType:"Record",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:Snapbar.showRollOver":{
        ref:"attr:Snapbar.showRollOver",
        flags:"IRW",
        description:"Snapbars show rollover styling.",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar"
    },
    "method:ColorPicker.setSupportsTransparency":{
        ref:"method:ColorPicker.setSupportsTransparency",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('attr:ColorPicker.supportsTransparency')} flag.\n",
        name:"setSupportsTransparency",
        seeAlso:[],
        params:[
            {description:"Set to true to enable transparency/opacity", optional:false,
             name:"transparencyFlag", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:ImgProperties.src":{
        ref:"attr:ImgProperties.src",
        flags:"IRW",
        description:"Specifies the URL of the image local to the skin or application directory.",
        name:"src",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "group:fileAssembly":{
        ref:"group:fileAssembly",
        title:"File Assembly",
        treeLocation:["Optional Modules/Network Performance"],
        refs:[],
        description:"File assembly concatenates multiple files into a single response, reducing the number\n of HTTP fetches required to load a page. This concatenation also improves the\n compressability of responses, since the compression algorithm may reduce redundancy across a\n larger data set.\n <p>\n The ISC FileAssembly service provides configuration-driven file assembly, with integrated\n compression and JavaScript stripping services.  Assemblies are specified in an xml format\n and map a URI to a set of files to concatenate together.  There also additional flags to\n enable stripping and compression.  \n <p>\n You can use the FileAssembler to create assemblies of javascript and css.  Since it's\n basically a file concatenator with some specialized services for javascript files, you could\n use it to assembly anything else, but javascript and css are probably the main things to\n consider.\n <p>\n The FileAssembler has a development mode and a production\n packaging mode.  In the development mode, there is a servlet that you can register at\n virtual URLs that are assembled on the fly by the FileAssembler based on its configuration.\n For production packaging, you can use the same configuration file to generate static,\n pre-compressed versions of these files.  The format of the file is as follows:\n <pre>\n &lt;FileAssembly&gt;\n     &lt;assemblies&gt;\n         &lt;FileAssemblyEntry uri=\"/myAssembly.js\"&gt;\n             &lt;compress&gt;true&lt;/compress&gt;\n             &lt;components&gt;\n                 &lt;component&gt;\n                     &lt;type&gt;file&lt;/type&gt;\n                     &lt;fileName&gt;copyright.txt&lt;/fileName&gt;\n                 &lt;/component&gt;\n                 &lt;component&gt;\n                     &lt;type&gt;datasource&lt;/type&gt;\n                     &lt;name&gt;myDatasource1&lt;/name&gt;\n                 &lt;/component&gt;\n                 &lt;component&gt;\n                     &lt;type&gt;ds&lt;/type&gt;\n                     &lt;name&gt;myDatasource2&lt;/name&gt;\n                 &lt;/component&gt;\n                 &lt;component&gt;\n                     &lt;type&gt;file&lt;/type&gt;\n                     &lt;fileName&gt;myPublicCode.js&lt;/fileName&gt;\n                 &lt;/component&gt;\n                 &lt;component&gt;\n                     &lt;type&gt;file&lt;/type&gt;\n                     &lt;fileName&gt;myPrivateCode.js&lt;/fileName&gt;\n                     &lt;jsStripping&gt;full&lt;/jsStripping&gt;\n                     &lt;jsObfuscation&gt;1&lt;/jsObfuscation&gt;\n                 &lt;/component&gt;\n             &lt;/components&gt;\n         &lt;/FileAssemblyEntry&gt;\n     &lt;/assemblies&gt;\n &lt;/FileAssembly&gt;\n </pre>\n You can specify as many FileAssemblyEntry blocks as you want, simply place them alongside\n each other under the &lt;assemblies&gt; element.  The above example states that the\n /myAssembly.js file should be assembled from the various components listed above. Component\n blocks support the following attributes:\n <p>\n <table border='1' class='normal'>\n <tr><td><b>Name</b></td><td><b>Values</b></td><td><b>Description</b></td></tr>\n <tr><td>type</td><td>file<br>xmlFile<br>ds|datasource<br>ui</td>\n     <td>any file<br>XML file<br>ISC DataSource file<br>ISC UI file<br></td></tr>\n <tr><td>name</td><td>file identifier</td>\n     <td>identifier for a ds or ui file</td></tr>\n <tr><td>fileName</td><td>file path</td><td>path to a file or xmlFile (relative to webRoot)</td></tr>\n <tr><td>jsStripping</td><td>none<br>partial<br>full</td>\n     <td>level of stripping (whitespace, comment, and delimiter removal) applied to file</td></tr>\n </table>\n <p>\n All component types except file are converted from valid ISC XML format to JavaScript format\n by the FileAssembler.  The <code>jsStripping</code> attribute only applies to assembled\n javascript files.\n <p>\n Once you've created the configuration file, you'll probably want to place it somewhere in\n webRoot.  Generally, it can be placed anywhere you like, but the development-time\n FileAssembly servlet requires the file be placed under webRoot. The default location is\n /isomorphicConfig/fileAssembly.xml and can be changed via the <code>configFile</code>\n init-param of the FileAssembly servlet.  To enable development mode support for\n FileAssembly, you'll need to register the FileAssembly servlet in your web.xml as follows:\n <pre>\n     &lt;servlet&gt;\n       &lt;servlet-name&gt;FileAssembly&lt;/servlet-name&gt;\n       &lt;servlet-class&gt;com.isomorphic.servlet.FileAssembly&lt;/servlet-class&gt;\n     &lt;/servlet&gt;\n </pre>\n Now, for every FileAssemblyEntry URI you specified in your fileAssembly.xml, you'll need\n corresponding mappings in web.xml, to tell the servlet container to send requests for that\n file to the FileAssembler.  For example, assuming your fileAssembly.xml contained the\n contents shown above, you'd register /myAssembly.js in your web.xml with the FileAssembler as\n follows:\n <pre>\n     &lt;servlet-mapping&gt;\n       &lt;servlet-name&gt;FileAssembly&lt;/servlet-name&gt;\n       &lt;url-pattern&gt;/myAssembly.js&lt;/url-pattern&gt;\n     &lt;/servlet-mapping&gt;\n </pre>\n You'll need a similar entry for any other entry in your fileAssembly.xml.  Note that this is\n for development only - we'll pregenerate static files for production.  Once you've done the\n above, you can load /myAssembly.js into your page via a standard SCRIPT block, like so:\n <pre>\n &lt;script src=\"/myAssembly.js\"&gt;&lt;/script&gt;\n </pre>\n Now, whenever requests are made for this file, the FileAssembler will check the timestamps\n of all the components of this assembly, rebuild the assembled file if required, and serve it\n to the browser as if it was a static file on disk.\n <p>\n To create static assembled files for production, you can use the fileAssembler command line\n tool provided as part of the SDK.  This is a Java-based command line tool located in\n smartclientSDK/WEB-INF/bin.  There are three versions of the tool in that directory -\n fileAssembler.bat is for Windows platforms, fileAssembler.command is for MacOS platforms, and\n fileAssembler.sh is for *nix platforms.  Running this tool with the --help argument will show\n you the command line switches available.  Typicaly, you'll need to specify:\n <ul>\n <li>--config &lt;file&gt; - location of the fileAssembly.xml configuration file\n <li>--webRoot &lt;dir&gt; - location of the webRoot that the fileAssembler uses to access the\n component files.\n <li>--outputDir &lt;dir&gt; - location of directory where you want output files to go - typically\n this is the webRoot of your production build.  These are the files you'll deploy to\n production.\n </ul>\n Finally, since the generated files are likely to be compressed and they're static, you'll\n probably want to map the FileDownload servlet to serve them in your production web.xml",
        name:"fileAssembly",
        seeAlso:[],
        type:"group",
        requiresModules:["NetworkPerformance"]
    },
    "method:RichTextEditor.doWarn":{
        ref:"method:RichTextEditor.doWarn",
        flags:"",
        description:" Display a warning if Rich Text Editing is not fully supported in this browser.\n Default behavior logs a warning to the devloper console - Override this if a user-visible\n warning is required \n",
        name:"doWarn",
        seeAlso:[],
        type:"method",
        definingClass:"class:RichTextEditor"
    },
    "attr:SummaryBuilder.autoHideCheckBoxLabel":{
        ref:"attr:SummaryBuilder.autoHideCheckBoxLabel",
        flags:"IRW",
        description:"Text label for the checkbox that allows the user to automatically hide the\n fields used in the summary format.",
        name:"autoHideCheckBoxLabel",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Auto hide fields used in summary\"",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["i18nMessages"]
    },
    "method:DatabaseBrowser.getGeneratedDataSource":{
        ref:"method:DatabaseBrowser.getGeneratedDataSource",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('class:DataSource')} most recently auto-derived by this DatabaseBrowser.  This will\n correspond to the currently-selected table.\n",
        name:"getGeneratedDataSource",
        seeAlso:[],
        type:"method",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:TabSet.pickerButtonVSrc":{
        ref:"attr:TabSet.pickerButtonVSrc",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true, and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricPickerButton')} is \n set to true, this property governs the base URL for the picker\n button image, when displayed in a verricaL tab-bar [IE ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')} is\n set to <code>\"LEFT\"</code> or <code>\"right\"</code>].\n <P>\n Note that if <code>symmetricPickerButton</code> is false, the ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonSrc')}\n property will be used instead.\n <P>\n This base URL will have a suffix of <code>\"Down\"</code> appended when the user holds the\n mouse down over the button, and <code>\"Disabled\"</code> if the tabset as a whole is \n disabled.",
        name:"pickerButtonVSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricPickerButton", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]vpicker.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "attr:Calendar.eventWindowStyleField":{
        ref:"attr:Calendar.eventWindowStyleField",
        flags:"IR",
        description:"The name of the field used to override ${isc.DocUtils.linkForRef('attr:Calendar.eventWindowStyle')} for an individual\n ${isc.DocUtils.linkForRef('object:CalendarEvent')}.  See ${isc.DocUtils.linkForRef('attr:CalendarEvent.eventWindowStyle')}.",
        name:"eventWindowStyleField",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"eventWindowStyle\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent", "appearance"]
    },
    "attr:ImgButton.showDown":{
        ref:"attr:ImgButton.showDown",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes down in this object?",
        name:"showDown",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:DataSourceField.maxFileSize":{
        ref:"attr:DataSourceField.maxFileSize",
        flags:"IRW",
        description:"For fields of a type that are represented by binary data initially uploaded from a file\n (currently \"binary\" and \"imageFile\", see ${isc.DocUtils.linkForRef('type:FieldType')}), this sets the maximum \n file size allowed, in bytes.",
        name:"maxFileSize",
        seeAlso:[],
        valueType:"Integer",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "attr:TabSet.symmetricScroller":{
        ref:"attr:TabSet.symmetricScroller",
        flags:"IR",
        description:"If this TabSet is showing ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller','tab scroller buttons')}, this property \n determines whether the ${isc.DocUtils.linkForRef('attr:TabSet.scrollerHSrc')} and ${isc.DocUtils.linkForRef('attr:TabSet.scrollerVSrc')} media\n will be used for vertical and horizontal tab-bar scroller buttons, or whether separate\n media should be used for each possible ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} based\n on the ${isc.DocUtils.linkForRef('attr:TabSet.scrollerSrc')} property for this tabSet.",
        name:"symmetricScroller",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "group:concat":{
        ref:"group:concat",
        refs:[],
        name:"concat",
        seeAlso:[],
        type:"group"
    },
    "attr:Flashlet.src":{
        ref:"attr:Flashlet.src",
        flags:"IR",
        description:"Location from which to load the Flashlet.",
        name:"src",
        setter:["setSrc"],
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "attr:ImgSplitbar.src":{
        ref:"attr:ImgSplitbar.src",
        flags:"IR",
        description:"The base filename for the image.\n <P>\n If <code>img.state</code> is set, it's value will be appended to the URL before the\n file extension.  \n <P>\n For example, given an <code>src</code> of \"findIcon.gif\" and a state of\n \"Disabled\", the resulting image name would be \"findIcon_Disabled.gif\".  Compound states\n such as \"Selected\", \"Focused\" and \"Over\" or \"Down\" will have an intervening underscore,\n resulting in, for example, <code>\"findIcon_Selected_Down.gif\"</code>.",
        name:"src",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgSplitbar",
        groups:["appearance"]
    },
    "method:Canvas.showClickMask":{
        ref:"method:Canvas.showClickMask",
        flags:"",
        description:" Show a clickMask over the entire screen that intercepts mouse clicks and fires some action.\n The mask created will be associated with this canvas - calling this method multiple times\n will not show multiple (stacked) clickMasks if the mask associated with this canvas is \n already up.<br><br>\n\n The clickMask useful for modal dialogs, menus and similar uses, where any click outside of\n some Canvas should either be suppressed (as in a modal dialog) or just cause something (like\n dismissing a menu). \n \n",
        name:"showClickMask",
        seeAlso:[
            {ref:"method:Canvas.hideClickMask", description:null}
        ],
        params:[
            {description:"action to fire when the user clicks on the mask", optional:false,
             name:"clickAction", defaultValue:null, type:"callback"},
            {description:"whether to automatically hide the clickMask on mouseDown                                      and suppress the mouseDown event from reaching\n                                      the target under the mouse",
             optional:false, name:"mode", defaultValue:null, type:"clickMaskMode"},
            {description:"initially unmasked targets for this clickMask. Note that if this is a\n  <code>\"hard\"</code> mask, unmasked children of masked parents are not supported\n  so any non-top-level widgets passed in will have their parents unmasked.\n  Children of masked parents can never be masked.",
             optional:false, name:"unmaskedTargets", defaultValue:null,
             type:"widget | array of widgets"}
        ],
        returns:{description:"clickMask ID", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["clickMask"]
    },
    "attr:ColumnTree.openIconSuffix":{
        ref:"attr:ColumnTree.openIconSuffix",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ColumnTree.showOpenIcons')} is true, this suffix will be appended to the\n ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')} for open folders in this grid.",
        name:"openIconSuffix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"open\"",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "attr:FormItem.showHint":{
        ref:"attr:FormItem.showHint",
        flags:"IRWA",
        description:"If a hint is defined for this form item, should it be shown?",
        name:"showHint",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:Canvas.mouseDown":{
        ref:"method:Canvas.mouseDown",
        flags:"A",
        description:"\n Executed when the left mouse down is pressed on this widget.  No default implementation.\n\n",
        name:"mouseDown",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        platformNotes:"If the end user system has only one mouse button, then it is considered the \"left\"                mouse button (and this method would execute when it is pressed on this widget).",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:Canvas.setDisabled":{
        ref:"method:Canvas.setDisabled",
        flags:"A",
        description:" set the disabled state of this object\n",
        name:"setDisabled",
        seeAlso:[],
        params:[
            {description:"new disabled state of this object - pass <code>true</code> to disable the widget",
             optional:false, name:"disabled", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "method:Tree.closeFolders":{
        ref:"method:Tree.closeFolders",
        flags:"",
        description:"\n Close a set of folders, specified by path or as pointers to nodes.\n\n",
        name:"closeFolders",
        seeAlso:[],
        params:[
            {description:"List of nodes or node paths.", optional:false, name:"nodeList",
             defaultValue:null, type:"List of TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "method:ListGrid.rowOut":{
        ref:"method:ListGrid.rowOut",
        flags:"",
        description:" Called when the mouse pointer leaves a row\n\n",
        name:"rowOut",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "type:RecordSummaryFunction":{
        ref:"type:RecordSummaryFunction",
        values:[
            {description:"Iterates through each field, picking up the numeric field values from the record and summing them.",
             value:"sum"},
            {description:"Iterates through each field, picking up the numeric field values from the record and calculating the mean value.",
             value:"avg"},
            {description:"Iterates through each field, picking up the numeric field values from the record and calculating the maximum value.",
             value:"max"},
            {description:"Iterates through each field, picking up the numeric field values from the record and calculating the minimum value.",
             value:"min"},
            {description:"Iterates through each field, picking up the numeric field values from the record and multiplying them together.",
             value:"multiplier"}
        ],
        description:"Function to produce a summary value based on field values within a record.\n Example usage is record-level summaries in \n ${isc.DocUtils.linkForRef('type:ListGridFieldType','\"summary\" type ListGrid fields')}.\n <P>\n SummaryFunctions may be specified in one of 2 ways:<ul>\n <li>as an explicit function or executable\n ${isc.DocUtils.linkForRef('type:stringMethod')}. Takes 3 parameters:\n <br><code>record</code> the record for which the value is being calculated\n <br><code>fields</code> Array of listGridFields from which summaries should be calculated\n <br><code>summaryField</code> pointer to the summary type field on which the summary function\n is being run.</li>\n <li>As a registered RecordSummaryFunction identifier</li></ul>\n Note that ${isc.DocUtils.linkForRef('classMethod:DataSource.registerRecordSummaryFunction')} may be used to expand the\n set of registered RecordSummaryFunctions.",
        name:"RecordSummaryFunction",
        seeAlso:[],
        type:"type"
    },
    "attr:SectionStack.itemIndent":{
        ref:"attr:SectionStack.itemIndent",
        flags:"IRW",
        description:"Size, in pixels, of indentation of all member items. Items will be offset\n and reduced in width by this amount. Overridden by\n ${isc.DocUtils.linkForRef('attr:SectionStack.itemStartIndent')} or ${isc.DocUtils.linkForRef('attr:SectionStack.itemEndIndent')}.\n Setting itemIndent is equivalent to setting itemStartIndent to the same amount\n and itemEndIndent to 0.",
        name:"itemIndent",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:SectionStack",
        groups:["layoutMember"]
    },
    "method:MultiView.deselectRecords":{
        ref:"method:MultiView.deselectRecords",
        flags:"",
        description:"\n Deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecords(records, false)</code>\n\n",
        name:"deselectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to deselect", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "method:TileLayout.setTileWidth":{
        ref:"method:TileLayout.setTileWidth",
        flags:"",
        description:" sets the width of tiles\n",
        name:"setTileWidth",
        seeAlso:[],
        params:[
            {description:"width", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "attr:ListGridField.editorImageURLPrefix":{
        ref:"attr:ListGridField.editorImageURLPrefix",
        flags:"IRWA",
        description:"When some cell in this field is being edited, this property can be used to apply \n an explicit ${isc.DocUtils.linkForRef('attr:FormItem.imageURLPrefix')} to the editor in question. \n This can be used to modify the valueIcons within the editor.<br>\n If unset, but ${isc.DocUtils.linkForRef('attr:ListGridField.imageURLPrefix')} is specified, that will be used\n instead.",
        name:"editorImageURLPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:ListGrid.updateData":{
        ref:"method:ListGrid.updateData",
        flags:"",
        description:" Perform a DataSource \"update\" operation to update existing records in this component's\n DataSource.\n\n",
        name:"updateData",
        seeAlso:[],
        params:[
            {description:"updated record", optional:false, name:"updatedRecord", defaultValue:null,
             type:"Object"},
            {description:"method to call on operation completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundUpdate')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "type:Positioning":{
        ref:"type:Positioning",
        values:[
            {description:"The canvas is absolutely positioned with respect to its parentElement.",
             value:"Canvas.ABSOLUTE"},
            {description:"The canvas is relatively positioned according to the document flow.",
             value:"Canvas.RELATIVE"}
        ],
        description:null,
        name:"Positioning",
        seeAlso:[],
        type:"type",
        groups:["positioning"]
    },
    "attr:Splitbar.vertical":{
        ref:"attr:Splitbar.vertical",
        flags:"R",
        description:"Is this split bar vertically orientated?<br>\n When a <code>Splitbar</code> is created by a layout to be the resizeBar for some\n member of the layout, the <code>vertical</code> property will be set to <code>true</code>\n if the layout is horizontal, meaning this resizeBar will be taller than it is wide, \n and will allow horizontal resizing of the member.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar"
    },
    "method:ListGrid.getDisplayValue":{
        ref:"method:ListGrid.getDisplayValue",
        flags:"",
        description:" Given a field with a specified ${isc.DocUtils.linkForRef('attr:ListGridField.valueMap')} or ${isc.DocUtils.linkForRef('attr:ListGridField.displayField')}\n this method will return the display value for any underlying data value.\n",
        name:"getDisplayValue",
        seeAlso:[],
        params:[
            {description:"Field or field identifier with valueMap", optional:false, name:"fieldID",
             defaultValue:null, type:"string || number || ListGridField"},
            {description:"Data value for this field", optional:false, name:"valueFieldValue",
             defaultValue:null, type:"any"}
        ],
        returns:{description:"Display value associated with the specified valueFieldValue", type:"string"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "type:CriteriaPolicy":{
        ref:"type:CriteriaPolicy",
        values:[
            {description:"Cache is dropped whenever criteria changes.", value:"\"dropOnChange\""},
            {description:"Cache is retained as along as the only changes to criteria                              make the criteria more restrictive as determined by\n                              ${isc.DocUtils.linkForRef('method:ResultSet.compareCriteria')}.",
             value:"\"dropOnShortening\""}
        ],
        description:null,
        name:"CriteriaPolicy",
        seeAlso:[],
        type:"type"
    },
    "method:CubeGrid.setEditValues":{
        ref:"method:CubeGrid.setEditValues",
        flags:"",
        description:" Set the temporary edit values for some cell in the cubeGrid.<br>\n Note that only the ${isc.DocUtils.linkForRef('attr:CubeGrid.valueProperty','this.valueProperty')} of the object passed in\n will be displayed in the cell.\n",
        name:"setEditValues",
        seeAlso:[],
        params:[
            {description:"2 element array of the form                   <code>[rowNum,colNum]</code> indicating the record being edited",
             optional:false, name:"cell", defaultValue:null, type:"Array of numbers"},
            {description:"New values for the record", optional:false, name:"values", defaultValue:null,
             type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:ResultSet.useClientSorting":{
        ref:"attr:ResultSet.useClientSorting",
        flags:"IRWA",
        description:"Whether to sort data locally when all records matching the current criteria have been\n cached.\n <P>\n This may need to be disabled if client-side sort order differs from server-side sort\n order in a way that affects functionality or is surprising.",
        name:"useClientSorting",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "method:ListGrid.cellMouseUp":{
        ref:"method:ListGrid.cellMouseUp",
        flags:"",
        description:" Called when a cell receives a mouseup event.\n\n",
        name:"cellMouseUp",
        seeAlso:[],
        params:[
            {description:"Record object (retrieved from getCellRecord(rowNum, colNum))",
             optional:false, name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "classMethod:ColorPicker.getSharedColorPicker":{
        ref:"classMethod:ColorPicker.getSharedColorPicker",
        flags:"",
        description:" Returns the shared global ColorPicker. \n Many applications will only need one ColorPicker instance; for such use \n cases, it is a good idea to use the shared object for performance reasons.\n <p>\n The optional second parameter to this method indicates whether the shared picker\n should retain the state (mode, color and opacity) it was in last time it was used, \n or revert to defaults.\n Generally, you will want the picker to revert to default state; this gives the\n same user experience as creating a new instance without incurring the overhead.\n However, some use cases will benefit from the picker remembering what the user\n did last time.\n",
        name:"getSharedColorPicker",
        seeAlso:[],
        params:[
            {description:"Creation properties for the global ColorPicker object", optional:false,
             name:"properties", defaultValue:null, type:"Object"},
            {description:"Should we keep the current state?                                    If false (or not provided), revert to default state",
             optional:true, name:"keepCurrentState", defaultValue:null, type:"boolean"}
        ],
        type:"classMethod",
        definingClass:"class:ColorPicker"
    },
    "method:ListGrid.setFilterEditorCriteria":{
        ref:"method:ListGrid.setFilterEditorCriteria",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')} is true, this method will update the criteria shown\n in the <code>filterEditor</code> without performing a filter.\n",
        name:"setFilterEditorCriteria",
        seeAlso:[],
        params:[
            {description:"New criteria to show", optional:false, name:"criteria", defaultValue:null,
             type:"Criteria or AdvancedCriteria"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "group:error handling":{
        ref:"group:error handling",
        refs:[],
        name:"error handling",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Class.getSuperClass":{
        ref:"classMethod:Class.getSuperClass",
        flags:"",
        description:"	\n	Gets a pointer to the superClass' Class object.\n\n",
        name:"getSuperClass",
        seeAlso:[],
        returns:{description:"Class object for superclass.", type:"Class"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:BatchUploader.partialCommit":{
        ref:"attr:BatchUploader.partialCommit",
        flags:"IRW",
        description:"Specifies what action to take if the user attempts to commit a partially validated set\n of data (ie, one that still contains some errors).",
        name:"partialCommit",
        seeAlso:[],
        valueType:"PartialCommitOption",
        defaultValue:"\"prompt\"",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:Button.removeFromRadioGroup":{
        ref:"method:Button.removeFromRadioGroup",
        flags:"",
        description:" Remove this widget from the specified mutually exclusive selection group with the ID\n passed in.\n No-op's if this widget is not a member of the groupID passed in.\n If no groupID is passed in, defaults to removing from whatever radioGroup this widget\n is a member of.\n",
        name:"removeFromRadioGroup",
        seeAlso:[],
        params:[
            {description:"- optional radio group ID (to ensure the widget is removed                                        from the appropriate group.",
             optional:true, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["state", "event handling"]
    },
    "group:loadUITag":{
        ref:"group:loadUITag",
        title:"&lt;isomorphic:loadUI&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n This tag converts SmartClient UI components declaratively specified in an XML \"UI\" file to\n JavaScript for execution in the browser.\n <p>\n The XML->JS translation works just like with ${isc.DocUtils.linkForRef('group:xmlTag')}, except the XML is read\n from an external file instead of from the body of the tag.\n <p>\n Note that this JSP tag must be surrounded by &lt;SCRIPT&gt; tags in the JSP because it\n generates JavaScript code.  Like other tags that generate JavaScript code, this tag can be\n used in a JSP that is included from your main page in order to create separate\n cacheability.  For example:<pre>\n     &lt;SCRIPT SRC=\"myUIDefinitions.jsp\"&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>name</b><br>\n <i>value format</i>: String - name of UI file to load (minus extension)<br>\n <i>default value</i>: NONE\n <p>\n This attribute specifies the name of the file that contains the UI components to translate.\n UI files are located in <code>[webroot]/shared/ui</code> by default.  This location is\n changeable in <code>[webroot]/WEB-INF/classes/server.properties</code> by setting the config\n parameter <code>project.ui</code> to the directory where your UI files are located.   \n We recommend that for prototying, at least, you use the default directory.\n <p>\n For example:\n <pre>\n &lt;isomorphic:loadUI name=\"test\"/&gt;\n </pre>\n Would translate declarative XML in the file <code>[webroot]/shared/ui/test.ui.xml</code> to\n JavaScript and output the results into the JSP output stream at the location of the tag.",
        name:"loadUITag",
        seeAlso:[
            {ref:"group:xmlTag", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "attr:Calendar.canEditField":{
        ref:"attr:Calendar.canEditField",
        flags:"IR",
        description:"Name of the field on each ${isc.DocUtils.linkForRef('object:CalendarEvent')} that determines editability.",
        name:"canEditField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"canEdit\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "attr:Toolbar.canReorderItems":{
        ref:"attr:Toolbar.canReorderItems",
        flags:"IRWA",
        description:"If true, items can be reordered by dragging on them.",
        name:"canReorderItems",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["dragndrop"]
    },
    "group:hovers":{
        ref:"group:hovers",
        refs:["method:ListGridField.hoverHTML", "method:Canvas.handleHover", "method:Canvas.updateHover", 
         "method:Canvas.hoverHidden","method:Canvas.hover", "method:Canvas.getHoverHTML", "attr:ListGrid.canHover", 
         "attr:ListGrid.showHover","attr:ListGrid.hoverStyle", "attr:Canvas.prompt", "attr:Canvas.canHover", 
         "attr:Canvas.hoverDelay","attr:Canvas.showHover", "attr:Canvas.hoverWidth", "attr:Canvas.hoverHeight", 
         "attr:Canvas.hoverAlign","attr:Canvas.hoverVAlign", "attr:Canvas.hoverWrap", "attr:Canvas.hoverStyle", 
         "attr:Canvas.hoverOpacity","attr:Canvas.hoverMoveWithMouse"],
        name:"hovers",
        seeAlso:[],
        type:"group"
    },
    "attr:MultiView.currentView":{
        ref:"attr:MultiView.currentView",
        flags:"IRW",
        description:"Class-name of the ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} that represents the current View.  The \n default is ${isc.DocUtils.linkForRef('class:ListGrid')}.",
        name:"currentView",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ListGrid\"",
        type:"attr",
        definingClass:"class:MultiView"
    },
    "attr:SelectItem.controlStyle":{
        ref:"attr:SelectItem.controlStyle",
        flags:"IRW",
        description:"Base CSS class name for a form item's control box (surrounds text box and picker).\n <P>\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"controlStyle",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"selectItemControl\", [IRA]",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "type:ValueMap":{
        ref:"type:ValueMap",
        description:"A ValueMap defines the set of legal values for a field, and optionally allows you to provide\n a mapping from stored values to values as seen by the end user.\n <P>\n A valueMap can be specified as either an Array of legal values, or as an ${isc.DocUtils.linkForRef('type:Object')}\n where each property maps a stored value to a user-displayable value.  See\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap')} for how to express a ValueMap in\n ${isc.DocUtils.linkForRef('group:componentXML','Component XML')}.\n <P>\n A ValueMap can be entirely static or entirely dynamic, with many options in between.  For\n example, a ValueMap may be:\n <ul>\n <li> statically defined in a JavaScript or XML file.  Such a valueMap changes only when\n application code is upgraded.\n <li> generated dynamically by server code when the application first loads, for example,\n by generating JavaScript or XML dynamically in a .jsp or .asp file.  Such a valueMap may be\n different for each session and for each user.\n <li> loaded on demand from a DataSource, via the\n ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource','optionDataSource')} property, or via a call to\n ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} where a valueMap is derived dynamically from the returned data\n (see ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} for an example).  Such a valueMap may be updated at any\n time, for example, every time the user changes a related field while editing data.\n </ul>\n See also the ${isc.DocUtils.linkForRef('group:smartArchitecture','SmartClient Architecture Overview')} to understand\n the best architecture from a performance and caching perspective.",
        name:"ValueMap",
        seeAlso:[],
        type:"type",
        serverDS:"allowed"
    },
    "attr:ListGrid.defaultFields":{
        ref:"attr:ListGrid.defaultFields",
        flags:"IRA",
        description:"An array of listGrid field configuration objects.  When a listGrid is initialized, if this\n property is set and there is no value for the <code>fields</code> attribute, this.fields will\n be defaulted to a generated array of field objects duplicated from this array.\n <P>\n This property is useful for cases where a standard set of fields will be displayed\n in multiple listGrids - for example a subclass of ListGrid intended to display a particular\n type of data:<br>\n In this example we would not assign a single ${isc.DocUtils.linkForRef('attr:ListGrid.fields')} array directly to the\n class via <code>addProperties()</code> as every generated instance of this class would\n then point to the same fields array object. This would cause unexpected behavior such as\n changes to the field order in one grid effecting other grids on the page.<br>\n Instead we could use <code>addProperties()</code> on our new subclass to set\n <code>defaultFields</code> to a standard array of fields to display. Each generated instance\n of the subclass would then show up with default fields duplicated from this array.",
        name:"defaultFields",
        seeAlso:[],
        valueType:"Array of ListGridField Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:FormItem.showValueIconOnly":{
        ref:"attr:FormItem.showValueIconOnly",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} is set, this property may be set to show the valueIcon\n only and prevent the standard form item element or text from displaying",
        name:"showValueIconOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "method:Array.containsAll":{
        ref:"method:Array.containsAll",
        flags:"",
        description:" Return whether this list contains all the item in the specified list.\n\n",
        name:"containsAll",
        seeAlso:[],
        params:[
            {description:"items to look for", optional:false, name:"list", defaultValue:null,
             type:"List"}
        ],
        returns:{description:"whether all items were found", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "method:CubeGrid.facetMoved":{
        ref:"method:CubeGrid.facetMoved",
        flags:"",
        description:" Notification fired when a facet is moved.\n \n",
        name:"facetMoved",
        seeAlso:[
            {ref:"method:CubeGrid.facetRemoved", description:null},
            {ref:"method:CubeGrid.facetAdded", description:"<br><i>methodType</i>  handler"}
        ],
        params:[
            {description:"facetId which moved", optional:false, name:"facetId", defaultValue:null,
             type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "method:CubeGrid.recordHasChanges":{
        ref:"method:CubeGrid.recordHasChanges",
        flags:"",
        description:" If this cubeGrid can be edited, this method will return true if the record passed in has been\n edited, but the edits have not yet been saved to the CubeGrid's data object.\n",
        name:"recordHasChanges",
        seeAlso:[],
        params:[
            {description:"row index of record to check for changes", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of the record to check for changes", optional:false,
             name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"true if the record has been edited but not yet saved", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:Scrollbar.trackEndHeight":{
        ref:"attr:Scrollbar.trackEndHeight",
        flags:"IRA",
        description:"The minimum pixel height of the track end segments (if enabled with showTrackEnds).",
        name:"trackEndHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"12",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["track"]
    },
    "attr:TabSet.scrollerSrc":{
        ref:"attr:TabSet.scrollerSrc",
        flags:"IR",
        description:"If this TabSet is showing ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller','tab scroller buttons')}, and \n ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is false, this property governs the base\n URL for the tab bar back and forward scroller button images.\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is true, \n ${isc.DocUtils.linkForRef('attr:TabSet.scrollerHSrc')} and ${isc.DocUtils.linkForRef('attr:TabSet.scrollerVSrc')} will be used instead.\n <P>\n To get the path to the image to display, this base URL will be modified as follows:\n <ul>\n <li>If appropriate a state suffix of <code>\"Down\"</code> or <code>\"Disabled\"</code> will be\n     appended.</li>\n <li>The ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} for this tabSet will be appended.</li>\n <li>A suffix of <code>\"forward\"</code> or <code>\"back\"</code> will be appended for the\n     forward or backward scrolling button.</li>\n </ul>\n For example - if the scrollerSrc is set to <code>\"[SKIN]scroll.gif\"</code>, the image\n displayed for the back-scroller button on a tabSet with <code>tabBarPosition</code> set to\n \"top\" and <code>symmetricScroller</code> set to false would be one of \n <code>\"[SKIN]scroll_top_back.gif\"</code>, <code>\"[SKIN]scroll_Down_top_back.gif\"</code>,\n and <code>\"[SKIN]scroll_Disabled_top_back.gif\"</code>.\n <P>\n Note that for best results the media should be sized to match the scroller button sizes, \n determined by ${isc.DocUtils.linkForRef('attr:TabSet.tabBarThickness')} and ${isc.DocUtils.linkForRef('attr:TabSet.scrollerButtonSize')}.",
        name:"scrollerSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricScroller", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/scroll.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "method:SelectItem.dataArrived":{
        ref:"method:SelectItem.dataArrived",
        flags:"",
        description:" If this item is showing a dataBound pickList, this notification method will be fired \n when new data arrives from the server.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"index of first row returned by the server", optional:false, name:"startRow",
             defaultValue:null, type:"number"},
            {description:"index of last row returned by the server", optional:false, name:"endRow",
             defaultValue:null, type:"number"},
            {description:"pointer to this pickList's data", optional:false, name:"data",
             defaultValue:null, type:"ResultSet"}
        ],
        type:"method",
        definingClass:"class:SelectItem"
    },
    "classMethod:WebService.get":{
        ref:"classMethod:WebService.get",
        flags:"",
        description:" Retrieve a WebService object by the targetNamespace declared on the &lt;wsdl:definitions&gt;\n element in the WSDL file from which the WebService was derived.\n <P>\n If you have more than one &lt;wsdl:service&gt; in the same target namespace, use\n ${isc.DocUtils.linkForRef('classMethod:WebService.getByName')} to disambiguate.\n\n",
        name:"get",
        seeAlso:[],
        params:[
            {description:"uri from the \"targetNamespace\" attribute of the &lt;wsdl:definitions&gt; element in the WSDL file",
             optional:false, name:"serviceNamespace", defaultValue:null, type:"String"}
        ],
        returns:{description:"the requested WebService, or null if not loaded", type:"WebService"},
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('wsdlBinding')}"],
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "method:ListGrid.setBodyOverflow":{
        ref:"method:ListGrid.setBodyOverflow",
        flags:"A",
        description:" Update the ${isc.DocUtils.linkForRef('attr:ListGrid.bodyOverflow','bodyOverflow')} for this listGrid.\n",
        name:"setBodyOverflow",
        seeAlso:[],
        params:[
            {description:"new overflow setting for the body", optional:false, name:"overflow",
             defaultValue:null, type:"Overflow"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "classMethod:EventHandler.shiftKeyDown":{
        ref:"classMethod:EventHandler.shiftKeyDown",
        flags:"",
        description:"			Return true if the shift key is being held down.\n			Note that this is only set reliably for keyboard events.\n\n",
        name:"shiftKeyDown",
        seeAlso:[],
        returns:{description:"true == shift key is down", type:"boolean"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "attr:Canvas.autoDraw":{
        ref:"attr:Canvas.autoDraw",
        flags:"IR",
        description:"If true, this canvas will draw itself immediately after it is created.\n <P>\n <b>Note</b> that you should turn this OFF for any canvases that are provided as children\n of other canvases, or they will draw initially, then be clear()ed and drawn again when\n added as children, causing a large performance penalty.  \n <P>\n For example, the following code is incorrect and will cause extra draw()s:\n <P>\n <pre>\n     isc.Layout.create({\n         members : [\n             isc.ListGrid.create()\n         ]\n     });\n </pre>\n It should instead be:\n <pre>\n     isc.Layout.create({\n         members : [\n             isc.ListGrid.create(<b>{ autoDraw: false }</b>)\n         ]\n     });\n </pre>\n In order to avoid unwanted autoDrawing systematically, it is recommend that you call\n ${isc.DocUtils.linkForRef('classMethod:isc.setAutoDraw','isc.setAutoDraw(false)')} immediately after SmartClient is loaded\n and before any components are created, then set <code>autoDraw:true</code> or call\n draw() explicitly to draw components.  \n <P>\n Otherwise, if the global setting for autoDraw remains <code>true</code>, you must set\n autoDraw:false, as shown above, on every component in your application that \n should not immediately draw: all Canvas children, Layout members, Window items, Tab\n panes, etc, however deeply nested.  Forgetting to set autoDraw:false will result in one\n more clear()s - these are reported on the Results tab of the \n ${isc.DocUtils.linkForRef('group:debugging','Developer Console')}, and can be tracked to individual components by\n using the \"clears\" log category in the Developer Console.",
        name:"autoDraw",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('autodraw')}"],
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "method:WebService.getFetchDS":{
        ref:"method:WebService.getFetchDS",
        flags:"",
        description:" Retrieve a DataSource that provides read-only access to records returned by a web\n service operation.\n <P>\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBound Components')} can be bound to the returned\n DataSource, and the ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} method can be invoked\n to retrieve data from the web service.\n <P>\n The returned DataSource is only capable of the \"fetch\" \n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource operation')}, not \"update\", \"add\" or\n \"remove\".  To create a DataSource capable of full read-write access, use\n ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings')} with the\n ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','wsOperation')} property set to associate each\n DataSource operation with a web service operation. \n\n",
        name:"getFetchDS",
        seeAlso:[],
        params:[
            {description:"name of the web service operation to invoke to fetch                               records",
             optional:false, name:"operationName", defaultValue:null, type:"String"},
            {description:"tag or type name of the XML element to be returned as                               DataSource records",
             optional:false, name:"resultType", defaultValue:null, type:"String"},
            {description:"Optional additional properties for the operationType:\"fetch\" \n ${isc.DocUtils.linkForRef('class:OperationBinding','operationBinding')} which this method automatically creates.  This\n can be used to set properties such as ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields')} or\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath')}",
             optional:true, name:"operationBindingProperties", defaultValue:null,
             type:"OperationBinding Properties"}
        ],
        type:"method",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "attr:LoginDialog.usernameItem":{
        ref:"attr:LoginDialog.usernameItem",
        flags:"IR",
        description:"Username field item in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"usernameItem",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "group:Items":{
        ref:"group:Items",
        refs:[],
        name:"Items",
        seeAlso:[],
        type:"group"
    },
    "method:TabSet.tabSelected":{
        ref:"method:TabSet.tabSelected",
        flags:"",
        description:" Notification fired when a tab is selected. Note that this will only fire if \n this tabSet is drawn. If a tab is selected before <code>TabSet.draw()</code> \n is called, the <code>tabSelected()</code> notification will fire on \n <code>draw()</code>\n",
        name:"tabSelected",
        seeAlso:[],
        params:[
            {description:"number of the tab", optional:false, name:"tabNum", defaultValue:null,
             type:"number"},
            {description:"pane for this tab", optional:false, name:"tabPane", defaultValue:null,
             type:"Canvas"},
            {description:"id of the tab", optional:false, name:"ID", defaultValue:null, type:"id"},
            {description:"the tab object (not tab button instance)", optional:false, name:"tab",
             defaultValue:null, type:"tab"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:Snapbar.showDown":{
        ref:"attr:Snapbar.showDown",
        flags:"IRW",
        description:"Snapbars show mouse-down styling.",
        name:"showDown",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar"
    },
    "method:FormItem.setValue":{
        ref:"method:FormItem.setValue",
        flags:"",
        description:" Set the value of the form item to the value passed in\n <p>\n NOTE: for valueMap'd items, newValue should be data value not displayed value\n",
        name:"setValue",
        seeAlso:[],
        params:[
            {description:"value to set the element to", optional:false, name:"newValue",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:ValuesManager.getSaveOperationType":{
        ref:"method:ValuesManager.getSaveOperationType",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('type:DSOperationType')} to be performed when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is\n called. Valid options are <code>\"add\"</code> or <code>\"update\"</code>.\n <P>\n If a ${isc.DocUtils.linkForRef('class:DSRequest')} configuration object is passed in containing an explicit operationType\n this will be returned. Otherwise ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')} will be returned.\n\n",
        name:"getSaveOperationType",
        seeAlso:[],
        params:[
            {description:"Optional DSRequest config block for the  save operation", optional:true,
             name:"requestProperties", defaultValue:null, type:"DSRequest properties"}
        ],
        returns:{description:"Operation type for the save request.", type:"DSOperationType"},
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "attr:JSONEncoder.circularReferenceMarker":{
        ref:"attr:JSONEncoder.circularReferenceMarker",
        flags:"IR",
        description:"The string marker used to represent circular references.  See ${isc.DocUtils.linkForRef('attr:JSONEncoder.circularReferenceMode')}.",
        name:"circularReferenceMarker",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"$$BACKREF$$\"",
        type:"attr",
        definingClass:"class:JSONEncoder"
    },
    "classMethod:isc.echo":{
        ref:"classMethod:isc.echo",
        flags:"",
        description:" Same as ${isc.DocUtils.linkForRef('classMethod:Log.echo')}.\n\n",
        name:"echo",
        seeAlso:[],
        params:[
            {description:"object to echo", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "method:StatefulCanvas.getTitle":{
        ref:"method:StatefulCanvas.getTitle",
        flags:"A",
        description:" Return the title - text/HTML drawn inside the component.\n <p>\n Default is to simply return this.title.\n",
        name:"getTitle",
        seeAlso:[],
        returns:{description:"HTML for the title", type:"string"},
        type:"method",
        definingClass:"class:StatefulCanvas"
    },
    "method:ListGridField.hoverHTML":{
        ref:"method:ListGridField.hoverHTML",
        flags:"",
        description:" StringMethod override point for returning HTML to be shown in hovers over cells in the\n column described by this field.\n <P>\n Called only when ${isc.DocUtils.linkForRef('attr:ListGrid.canHover','canHover')} and ${isc.DocUtils.linkForRef('attr:ListGrid.showHover','showHover')}\n are both true.\n <P>\n The value of \"this\" within the method will by the ${isc.DocUtils.linkForRef('object:ListGridField','field definition')}.\n\n",
        name:"hoverHTML",
        seeAlso:[],
        params:[
            {description:"record being hovered over", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"value of the cell being hovered over", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"row number where hover occurred", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column number where hover occurred", optional:false, name:"colNum",
             defaultValue:null, type:"number"},
            {description:"ListGrid this field is a part of", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"HTML to show in the hover", type:"HTML"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('valueHoverTips')}"],
        definingClass:"object:ListGridField",
        groups:["hovers"]
    },
    "attr:Button.iconOrientation":{
        flags:"IR",
        type:"attr",
        ref:"attr:Button.iconOrientation",
        description:"If this button is showing an icon should it appear to the left or right of the title?\n valid options are <code>\"left\"</code> and <code>\"right\"</code>.",
        name:"iconOrientation",
        setter:["setIconOrientation"],
        valueType:"string",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonIcons')}"],
        defaultValue:"\"left\"",
        groups:["buttonIcon"],
        definingClass:"class:Button"
    },
    "attr:FacetValue.cellAlign":{
        ref:"attr:FacetValue.cellAlign",
        flags:"IR",
        description:"default alignment of cells (in the body) for this facetValue",
        name:"cellAlign",
        seeAlso:[],
        valueType:"alignEnum",
        defaultValue:"facet.cellAlign",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:StatefulCanvas.overCanvasDefaults":{
        ref:"attr:StatefulCanvas.overCanvasDefaults",
        flags:"IRWA",
        description:"Default properties for this widgets ${isc.DocUtils.linkForRef('attr:StatefulCanvas.overCanvas','overCanvas')}. To modify\n these defaults, use ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')}",
        name:"overCanvasDefaults",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"{ ... }",
        type:"attr",
        definingClass:"class:StatefulCanvas"
    },
    "group:images":{
        ref:"group:images",
        refs:["method:Canvas.getImgURL", "method:Canvas.imgHTML", "method:Canvas.getImage", 
         "method:Canvas.setImage","classMethod:Page.setAppImgDir", "classMethod:Page.getAppImgDir", 
         "classMethod:Page.setAppFilesDir","classMethod:Page.getAppFilesDir", "classMethod:Page.setSkinDir", 
         "classMethod:Page.getSkinDir","classMethod:Page.getSkinImgDir", "classMethod:Page.getImgURL", "classMethod:Page.getURL", 
         "classMethod:Page.loadStyleSheet","attr:CubeGrid.skinImgDir", "attr:ListGrid.skinImgDir", "attr:Canvas.appImgDir", 
         "attr:Canvas.skinImgDir","attr:TabSet.skinImgDir", "attr:Scrollbar.skinImgDir", "attr:Scrollbar.cornerSrc", 
         "attr:Scrollbar.hSrc","attr:Scrollbar.vSrc", "attr:Splitbar.skinImgDir", "attr:ImgSplitbar.skinImgDir"],
        description:"Refering to and loading images.\n <P>\n The two main URL settings relevent to loading images are:<br>\n * imgDir (where application-specific images live)<br>\n * skinImgDir (where system supplied images live)<br>",
        name:"images",
        seeAlso:[],
        type:"group"
    },
    "attr:Tree.data":{
        ref:"attr:Tree.data",
        flags:"IR",
        description:"Optional initial data for the tree. How this data is interpreted depends on this tree's\n ${isc.DocUtils.linkForRef('attr:Tree.modelType')}.\n <P>\n If <code>modelType</code> is <code>\"parent\"</code>, the list that you provide will be passed \n to ${isc.DocUtils.linkForRef('method:Tree.linkNodes')}, integrating the nodes into the tree.\n <p>\n In this case the root node may be supplied explicity via ${isc.DocUtils.linkForRef('attr:Tree.root')}, or auto generated,\n picking up its <code>id</code> via ${isc.DocUtils.linkForRef('attr:Tree.rootValue')}. Any nodes in the data with no\n explicitly specified ${isc.DocUtils.linkForRef('attr:TreeNode.parentId')} will be added as children to this root element.\n <P>\n To create this tree:\n <pre>\n foo\n   bar\n zoo\n </pre>\n with modelType:\"parent\", you can do this:\n <pre>\n Tree.create({\n   data: [\n     {name: \"foo\", id: \"foo\"},\n     {name: \"bar\", id: \"bar\", parentId: \"foo\"},\n     {name: \"zoo\", id: \"zoo\"}\n });\n </pre>\n Or this (explicitly specified root):\n <pre>\n Tree.create({\n   root: {id: \"root\"},\n   data: [\n     {name: \"foo\", id: \"foo\", parentId: \"root\"},\n     {name: \"bar\", id: \"bar\", parentId: \"foo\"},\n     {name: \"zoo\", id: \"zoo\", parentId: \"root\"}\n });\n </pre>\n Or this (explicitly specified rootValue):\n <pre>\n Tree.create({\n   rootValue: \"root\",\n   data: [\n     {name: \"foo\", id: \"foo\", parentId: \"root\"},\n     {name: \"bar\", id: \"bar\", parentId: \"foo\"},\n     {name: \"zoo\", id: \"zoo\", parentId: \"root\"}\n });\n </pre>\n Specifying the root node explicitly allows you to give it a name, changing the way path\n derivation works (see ${isc.DocUtils.linkForRef('attr:Tree.root')} for more on naming the root node).\n <P>\n For <code>modelType:\"children\"</code> trees, the data passed in will be assumed to be an \n array of children of the tree's root node.",
        name:"data",
        seeAlso:[
            {ref:"attr:Tree.modelType", description:null},
            {ref:"object:TreeNode", description:null}
        ],
        valueType:"List of TreeNode",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "method:ComboBoxItem.filterClientPickListData":{
        ref:"method:ComboBoxItem.filterClientPickListData",
        flags:"",
        description:" Returns the data to display in the pick list.\n <P>\n The default implementation applies the criteria returned by \n ${isc.DocUtils.linkForRef('method:PickList.getPickListFilterCriteria')} to the data returned by\n ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')}.  A record passes the filter if it has a\n matching value for all fields in the criteria object.  Matching is performed according\n to ${isc.DocUtils.linkForRef('type:TextMatchStyle')}.\n <P>\n If ${isc.DocUtils.linkForRef('attr:PickList.showAllOptions')} is set, all values are shown, with matching values\n shown below a ${isc.DocUtils.linkForRef('attr:PickList.separatorRows','separator')}.\n\n",
        name:"filterClientPickListData",
        seeAlso:[],
        returns:{description:"array of record objects to display in the pickList",
         type:"Array of ListGridRecord"},
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "attr:DateChooser.baseBottomButtonStyle":{
        ref:"attr:DateChooser.baseBottomButtonStyle",
        flags:"IRW",
        description:"CSS style to apply to the buttons at the bottom of the DateChooser (\"Today\" and\n \"Cancel\").  If null, the CSS style specified in ${isc.DocUtils.linkForRef('attr:DateChooser.baseButtonStyle')} is used.",
        name:"baseBottomButtonStyle",
        seeAlso:[],
        valueType:"CSSClassName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:DynamicForm.getItems":{
        ref:"method:DynamicForm.getItems",
        flags:"",
        description:" Method to retrieve the ${isc.DocUtils.linkForRef('attr:DynamicForm.fields','items')} for this DynamicForm. \n\n",
        name:"getItems",
        seeAlso:[],
        returns:{description:null, type:"Array of FormItem"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["elements"]
    },
    "classAttr:RPCManager.showPrompt":{
        ref:"classAttr:RPCManager.showPrompt",
        flags:"RW",
        description:"If set to <code>true</code>, the RPCManager will block the UI with a modal dialog containing\n the text from RPCManager.defaultPrompt (or the per-RPCRequest override) until the RPC to the\n server completes.\n <br><br>\n If set to <code>false</code>, the RPC happens transparently, allowing the user to continue\n interacting with the UI",
        name:"showPrompt",
        seeAlso:[
            {ref:"classAttr:RPCManager.defaultPrompt", description:null},
            {ref:"attr:RPCRequest.showPrompt", description:null}
        ],
        valueType:"boolean",
        defaultValue:"RPCManager.showPrompt",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["rpcPrompt"]
    },
    "attr:Canvas.extraSpace":{
        ref:"attr:Canvas.extraSpace",
        flags:"IR",
        description:"When this Canvas is included as a member in a Layout, extra blank space that should be left\n after this member in a Layout.",
        name:"extraSpace",
        seeAlso:[
            {ref:"class:LayoutSpacer", description:"for more control"}
        ],
        valueType:"positiveInteger",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["layoutMember"]
    },
    "attr:TreeGrid.parentAlreadyContainsChildMessage":{
        ref:"attr:TreeGrid.parentAlreadyContainsChildMessage",
        flags:"IR",
        description:"Message displayed when user attempts to drag a node into a parent that already contains\n a child of the same name.",
        name:"parentAlreadyContainsChildMessage",
        seeAlso:[
            {ref:"attr:TreeGrid.canDragRecordsOut", description:null},
            {ref:"attr:TreeGrid.canAcceptDroppedRecords", description:null},
            {ref:"attr:TreeGrid.canReorderRecords", description:null}
        ],
        valueType:"String",
        defaultValue:"\"This item already contains a child item with that name.\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["i18nMessages"]
    },
    "classMethod:isc.propertyDefined":{
        ref:"classMethod:isc.propertyDefined",
        flags:"",
        description:"\n	Is some property specified on the object passed in?  This will return true if \n  <code>object[propertyName]</code> has ever been set to any value, and not deleted.<br>\n  May return true even if <code>object[propertyName] === undefined</code> if the property \n  is present on the object and has been explicitly set to undefined.\n\n",
        name:"propertyDefined",
        seeAlso:[],
        params:[
            {description:"Object to test", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"Which property is being tested for?", optional:false, name:"propertyName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true if property is defined", type:"boolean"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "method:TabSet.removeTabs":{
        ref:"method:TabSet.removeTabs",
        flags:"A",
        description:" Remove one or more tabs.  The pane(s) associated with the removed tab(s) is automatically\n destroyed when you call this method.\n\n",
        name:"removeTabs",
        seeAlso:[
            {ref:"method:TabSet.removeTab", description:null}
        ],
        params:[
            {description:"list of tabs, tabIDs, or tab numbers", optional:false, name:"tabs",
             defaultValue:null, type:"Tab | ID | number"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:OperationBinding.excludeCriteriaFields":{
        ref:"attr:OperationBinding.excludeCriteriaFields",
        flags:"IR",
        description:"Specifies, for this operationBinding only, a list of field names that should be excluded from\n the default ${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','whereClause')} SmartClient generates.  The idea behind this is \n that you can then use these criteria manually in a complex query - for example, in the \n WHERE clause of a subquery.\n <p>\n You can specify this property as either an array or a string containing a comma-separated list \n of field names (eg, \"foo, bar, baz\").  Note that if a field is included in both \n excludeCriteriaFields and ${isc.DocUtils.linkForRef('attr:OperationBinding.customCriteriaFields','customCriteriaFields')}, \n customCriteriaFields wins.\n <p>\n This property is only applicable to DataSources of type \"sql\".",
        name:"excludeCriteriaFields",
        seeAlso:[
            {ref:"attr:OperationBinding.customCriteriaFields", description:null}
        ],
        valueType:"String or Array",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:StretchImgButton.showFocused":{
        ref:"attr:StretchImgButton.showFocused",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  If\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is <code>true</code>, the <b><code>\"over\"</code></b>\n will be used to indicate focus. Otherwise a separate <b><code>\"focused\"</code></b> state\n will be used.",
        name:"showFocused",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "attr:Layout.defaultResizeBars":{
        ref:"attr:Layout.defaultResizeBars",
        flags:"IR",
        description:"Policy for whether resize bars are shown on members by default.",
        name:"defaultResizeBars",
        seeAlso:[],
        valueType:"LayoutResizeBarPolicy",
        defaultValue:"\"marked\"",
        type:"attr",
        definingClass:"class:Layout"
    },
    "attr:RPCResponse.clientContext":{
        ref:"attr:RPCResponse.clientContext",
        flags:"R",
        description:"The ${isc.DocUtils.linkForRef('attr:RPCRequest.clientContext')} object as set on the ${isc.DocUtils.linkForRef('class:RPCRequest')}.",
        name:"clientContext",
        seeAlso:[
            {ref:"attr:RPCRequest.clientContext", description:null}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCResponse"
    },
    "method:TreeGrid.getIcon":{
        ref:"method:TreeGrid.getIcon",
        flags:"",
        description:" Get the appropriate icon for a node.\n <P>\n By default icons are derived from ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} and ${isc.DocUtils.linkForRef('attr:TreeGrid.nodeIcon')}.\n Custom icons for individual nodes can be overriden by setting the ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconProperty')}\n on a node.\n <p>\n If you want to suppress icons altogether, provide an override of this method that simply\n returns null.\n <p> \n Note that the full icon URL will be derived by applying ${isc.DocUtils.linkForRef('method:Canvas.getImgURL')} to the\n value returned from this method.\n\n",
        name:"getIcon",
        seeAlso:[],
        params:[
            {description:"tree node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"URL for the icon to show for this node", type:"URL"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:DateChooser.cancelClick":{
        ref:"method:DateChooser.cancelClick",
        flags:"",
        description:" Fired when the user clicks the cancel button in this date chooser. Default implementation\n clears the date chooser.\n",
        name:"cancelClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:DateChooser"
    },
    "attr:SectionStack.showExpandControls":{
        ref:"attr:SectionStack.showExpandControls",
        flags:"IRW",
        description:"Whether to show the Expand/Collapse controls in the headers of sections.  If false, hides\n the expand/collapse controls and, instead, treats a click anywhere on the header as if \n it were a click on the expand control.",
        name:"showExpandControls",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "method:FilterBuilder.clearCriteria":{
        ref:"method:FilterBuilder.clearCriteria",
        flags:"",
        description:" Clear all current criteria.\n",
        name:"clearCriteria",
        seeAlso:[],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "classMethod:isc.getIconPageRect":{
        ref:"classMethod:isc.getIconPageRect",
        flags:"A",
        description:"  Returns the size / position of an icon on the page as an array of coordinates.\n",
        name:"getIconPageRect",
        seeAlso:[],
        params:[
            {description:"icon definition for the icon you want to determine the                               position of (defaults to first icon in this.icons).",
             optional:false, name:"icon", defaultValue:null, type:"object"}
        ],
        returns:{description:"four element array representing the Left, Top, Width, and Height of                      the icon in px.",
         type:"array"},
        type:"method",
        definingClass:"class:isc"
    },
    "classAttr:Date.shortMonthNames":{
        ref:"classAttr:Date.shortMonthNames",
        flags:"IRWA",
        description:"This property may be set to an array of names of months.<br>\n For example:\n <pre>\n [\"Jan\", \"Feb\", \"Mar\", \"Apr\", \"May\", \"Jun\", \"Jul\", \"Aug\", \"Sep\", \"Oct\", \"Nov\", \"Dec\"]\n </pre>\n The appropriate month name will then be returned from ${isc.DocUtils.linkForRef('method:Date.getShortMonthName')}, \n and may be used whenever  SmartClient components display month-names (for example in the \n ${isc.DocUtils.linkForRef('class:DateItem','DateItem class')}).",
        name:"shortMonthNames",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"classAttr",
        definingClass:"class:Date",
        groups:["i18nMessages"]
    },
    "method:DynamicForm.rememberValues":{
        ref:"method:DynamicForm.rememberValues",
        flags:"",
        description:"			Make a snapshot of the current set of values, so we can reset to them later.\n			Creates a new object, then adds all non-method properties of values\n			to the new object.  Use <code>resetValues()</code> to revert to these values.\n          Note that this method is automatically called when the values for this form are\n          set programmatically via a call to ${isc.DocUtils.linkForRef('method:DynamicForm.setValues')}.\n\n",
        name:"rememberValues",
        seeAlso:[],
        returns:{description:"copy of current form values", type:"object"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:SelectItem.pickButtonHeight":{
        ref:"attr:SelectItem.pickButtonHeight",
        deprecated:"As of SmartClient version 5.5, pickButtonHeight has been deprecated in   favor of ${isc.DocUtils.linkForRef('attr:SelectItem.pickerIconHeight')}.",
        flags:"IRWA",
        description:"How large should the pick button be rendered?",
        name:"pickButtonHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:RPCRequest.clientContext":{
        ref:"attr:RPCRequest.clientContext",
        flags:"IRW",
        description:"An object to be held onto for the duration of the RPC turnaround to track\n application-specific context.\n <br>\n When an RPC turnaround completes, the <code>clientContext</code> is available in the\n ${isc.DocUtils.linkForRef('type:RPCCallback')} as <code>rpcResponse.clientContext</code>.  The\n <code>clientContext</code> is never sent to the server.\n <br>\n The <code>clientContext</code> is useful for holding onto state that will be used when the\n ${isc.DocUtils.linkForRef('type:RPCCallback')} fires, such as the name of a component that will receive the\n returned data.",
        name:"clientContext",
        seeAlso:[
            {ref:"attr:RPCResponse.clientContext", description:null}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:DSRequest.lineBreakStyle":{
        ref:"attr:DSRequest.lineBreakStyle",
        flags:"IR",
        description:"The style of line-breaks to use in the exported output.  See ${isc.DocUtils.linkForRef('type:LineBreakStyle')} for\n more information.",
        name:"lineBreakStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:Applet.alt":{
        ref:"attr:Applet.alt",
        flags:"IR",
        description:"This text is displayed if the browser understands the tag used for the applet html element,\n  but can't run the applet for some reason.",
        name:"alt",
        seeAlso:[
            {ref:"attr:Applet.altHTML", description:null},
            {ref:"attr:Applet.useTag", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:Calendar.dayViewTitle":{
        ref:"attr:Calendar.dayViewTitle",
        flags:"IR",
        description:"The title for the day view",
        name:"dayViewTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Day\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "attr:Tree.autoOpenRoot":{
        ref:"attr:Tree.autoOpenRoot",
        flags:"IRW",
        description:"If true, the root node is automatically opened when the tree is created or\n ${isc.DocUtils.linkForRef('method:Tree.setRoot')} is called.",
        name:"autoOpenRoot",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Tree"
    },
    "method:ResultTree.unloadChildren":{
        ref:"method:ResultTree.unloadChildren",
        flags:"",
        description:"			Unload the children of a node\n",
        name:"unloadChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:ResultTree",
        groups:["loadState"]
    },
    "attr:animationShowEffect.startFrom":{
        ref:"attr:animationShowEffect.startFrom",
        flags:"IR",
        description:"For show animations of type <code>\"wipe\"</code> and\n   <code>\"slide\"</code> this attribute specifies where the wipe / slide should originate.\n   Valid values are <code>\"T\"</code> (vertical animation from the top down, the \n   default behavior), and <code>\"L\"</code> (horizontal animation from the left side).",
        name:"startFrom",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:animationShowEffect"
    },
    "class:NativeCheckboxItem":{
        methods:[],
        ref:"class:NativeCheckboxItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:NativeCheckboxItem.textBoxStyle", "attr:NativeCheckboxItem.showLabel"],
        description:"A checkbox for manipulating 2-valued fields based on the native checkbox element.",
        name:"NativeCheckboxItem",
        seeAlso:[],
        type:"class"
    },
    "attr:CubeGrid.canSelectHeaders":{
        ref:"attr:CubeGrid.canSelectHeaders",
        flags:"IRW",
        description:"Determines whether row or column facetValue headers can be selected.",
        name:"canSelectHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:GridRenderer.rowClick":{
        ref:"method:GridRenderer.rowClick",
        flags:"A",
        description:" Called when a row receives a click event.\n\n",
        name:"rowClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:DateChooser.prevMonthIconWidth":{
        ref:"attr:DateChooser.prevMonthIconWidth",
        flags:"IR",
        description:"Width of the icon for the previous month button",
        name:"prevMonthIconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:StretchImgButton.setIconOrientation":{
        ref:"method:StretchImgButton.setIconOrientation",
        flags:"",
        description:" Changes the orientation of the icon relative to the text of the button.\n\n",
        name:"setIconOrientation",
        seeAlso:[],
        params:[
            {description:"The new orientation of the icon relative to the text of the button.",
             optional:false, name:"orientation", defaultValue:null, type:"\"left\" or \"right\""}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "method:Canvas.getSnapEdge":{
        ref:"method:Canvas.getSnapEdge",
        flags:"",
        description:" Return the snapEdge value of this object\n\n",
        name:"getSnapEdge",
        seeAlso:[],
        returns:{description:"snapEdge", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "group:draw":{
        ref:"group:draw",
        refs:[],
        name:"draw",
        seeAlso:[],
        type:"group"
    },
    "attr:LoginDialog.passwordItemTitle":{
        ref:"attr:LoginDialog.passwordItemTitle",
        flags:"IR",
        description:"Specifies the title of the \"passwordItem\" field of the ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"passwordItemTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Dialog.PASSWORD_TITLE",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:DataSource.pluralTitle":{
        ref:"attr:DataSource.pluralTitle",
        flags:"IR",
        description:"User-visible plural name for this DataSource.\n <P>\n For example, for the supplyItem DataSource, \"Supply Items\".\n <P>\n Defaults to (<code>dataSource.title</code> or <code>dataSource.ID</code>) + \"s\".",
        name:"pluralTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"dataSource.ID",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["titles"]
    },
    "method:Layout.getMembers":{
        ref:"method:Layout.getMembers",
        flags:"",
        description:" Get the Array of members.<br><br>\n \n NOTE: the returned array should not be modified.\n",
        name:"getMembers",
        seeAlso:[],
        returns:{description:"the Array of members", type:"Array"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:RowSpacerItem.showTitle":{
        ref:"attr:RowSpacerItem.showTitle",
        flags:"IRW",
        description:"we never show a separate title cell for separators",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RowSpacerItem",
        groups:["appearance"]
    },
    "attr:OperationBinding.valuesClause":{
        ref:"attr:OperationBinding.valuesClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n set of values to add or update,  for use when constructing the SQL query to perform this \n operation.  The property should be one of the following, depending on the \n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')}:\n <p>\n For \"add\" operations, the syntax that would be valid for an INSERT INTO query: a \n comma-separated list of column names enclosed in parentheses, followed by a comma-separated\n list of new values, enclosed in parentheses and preceded by the token \"VALUES\".  For example:\n <p>\n <code>&lt;valuesClause&gt;(name, age) VALUES(\"Jane Doe\", 48)&lt;/valuesClause&gt;</code>\n <p>\n For \"update\" operations, the syntax that would be valid for an UPDATE query: a \n comma-separated list of expressions equating a column name to its new value.  For example:\n <p>\n <code>&lt;valuesClause&gt;name=\"Jane Doe\", age=48&lt;/valuesClause&gt;</code>\n <p>\n You may find the SmartClient-provided <b>$values</b> variable of particular use with\n this property.\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"valuesClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "classMethod:History.setHistoryTitle":{
        ref:"classMethod:History.setHistoryTitle",
        flags:"",
        description:"\n Sets the title associated with all history entries.  This is the string that appears in the\n history drop-down.  If left unset, this default to the history id that is passed into\n ${isc.DocUtils.linkForRef('classMethod:History.addHistoryEntry')}.\n <p>\n Note: Currently, this works in IE only.  You may call this method in all other browsers,\n but it will not change what's displayed in the history drop-down.\n\n",
        name:"setHistoryTitle",
        seeAlso:[],
        params:[
            {description:"The title to show in the history drop-down.", optional:false, name:"title",
             defaultValue:null, type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:History"
    },
    "method:FormItem.keyPress":{
        ref:"method:FormItem.keyPress",
        flags:"",
        description:" StringMethod fired when the user presses a key while focused in this form item.\n\n",
        name:"keyPress",
        seeAlso:[],
        params:[
            {description:"Item over which the keypress occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"Pointer to the item's form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"},
            {description:"Name of the key pressed (Example: <code>\"A\"</code>, <code>\"Enter\"</code>)",
             optional:false, name:"keyName", defaultValue:null, type:"KeyName"},
            {description:"If this was a character key, this is the numeric value        for the character",
             optional:false, name:"characterValue", defaultValue:null, type:"number"}
        ],
        returns:{description:"return false to attempt to cancel the event.  Note for general purpose                   APIs for managing whether user input is allowed, use ${isc.DocUtils.linkForRef('method:FormItem.change')} \n                   or ${isc.DocUtils.linkForRef('method:FormItem.transformInput')} instead.",
         type:"boolean"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "attr:MultiSortDialog.copyLevelButtonTitle":{
        ref:"attr:MultiSortDialog.copyLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the copyLevelButton",
        name:"copyLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Copy Level\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "method:TileGrid.deselectAllRecords":{
        ref:"method:TileGrid.deselectAllRecords",
        flags:"",
        description:"\n Deselect all records\n\n",
        name:"deselectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "method:TileGrid.getDragTrackerTitle":{
        ref:"method:TileGrid.getDragTrackerTitle",
        flags:"",
        description:" Return \"title\" HTML to display as a drag tracker when the user drags some record.<br>\n Default implementation will display the cell value for the title field (see \n ${isc.DocUtils.linkForRef('method:ListGrid.getTitleField')}) for the record(s) being dragged (including any\n icons / custom formatting / styling, etc).\n <p>\n Note: Only called if ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')} is set to <code>\"title\"</code>.\n",
        name:"getDragTrackerTitle",
        seeAlso:[],
        params:[
            {description:"First selected record being dragged", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"row index of first record being dragged", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Title for the row. Default implementation looks at the value of the                  title-field cell for the row.",
         type:"string"},
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dragTracker"]
    },
    "attr:GridRenderer.drawAllMaxCells":{
        ref:"attr:GridRenderer.drawAllMaxCells",
        flags:"IRWA",
        description:"If drawing all rows would cause less than <code>drawAllMaxCells</code> cells to be rendered,\n the full dataset will instead be drawn even if ${isc.DocUtils.linkForRef('attr:ListGrid.showAllRecords','showAllRecords')}\n is false and the viewport size and ${isc.DocUtils.linkForRef('attr:GridRenderer.drawAheadRatio')} setting would normally have caused\n incremental rendering to be used.\n <P>\n The <code>drawAllMaxCells</code> setting prevents incremental rendering from being used in\n situations where it's really unnecessary, such as a 40 row, 5 column dataset (only 200\n cells) which happens to be in a grid with a viewport showing only 20 or so rows.\n Incremental rendering causes a brief \"flash\" during scrolling as the visible portion of the\n dataset is redrawn, and a better scrolling experience can be obtained in this situation by\n drawing the entire dataset up front, which in this example would have neglible effect on\n initial draw time.\n <P>\n <code>drawAllMaxCells:0</code> disables this features.  You may want to disable this feature\n if performance is an issue and:\n <ul>\n <li> you are very frequently redraw a grid\n <li> you do a lot of computation when rendering each cell (eg formulas)\n <li> you are showing many grids on one screen and the user won't scroll most of them\n </ul>",
        name:"drawAllMaxCells",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"250",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "attr:ListGridField.canGroupBy":{
        ref:"attr:ListGridField.canGroupBy",
        flags:"IRW",
        description:"Determines whether this field will be groupable in the header context menu.",
        name:"canGroupBy",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:RPCRequest.prompt":{
        ref:"attr:RPCRequest.prompt",
        flags:"IRW",
        description:"Overrides RPCManager.defaultPrompt for this request only.  If you're using queuing, note that the\n prompt string from the first request in the queue is the one that is shown to the user.",
        name:"prompt",
        seeAlso:[
            {ref:"classAttr:RPCManager.defaultPrompt", description:null},
            {ref:"classAttr:RPCManager.showPrompt", description:null},
            {ref:"classAttr:RPCManager.promptStyle", description:null},
            {ref:"classAttr:RPCManager.promptCursor", description:null},
            {ref:"attr:RPCRequest.showPrompt", description:null},
            {ref:"attr:RPCRequest.promptStyle", description:null},
            {ref:"attr:RPCRequest.promptCursor", description:null}
        ],
        valueType:"string",
        defaultValue:"RPCManager.defaultPrompt",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["rpcPrompt"]
    },
    "group:drag":{
        ref:"group:drag",
        refs:["type:DragDataAction"],
        name:"drag",
        seeAlso:[],
        type:"group"
    },
    "attr:ImgButton.showDisabled":{
        ref:"attr:ImgButton.showDisabled",
        flags:"IRW",
        description:"Should we visibly change state when disabled?",
        name:"showDisabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:TreeNode.showDropIcon":{
        ref:"attr:TreeNode.showDropIcon",
        flags:"IRWA",
        description:"For folder nodes showing custom icons (set via ${isc.DocUtils.linkForRef('attr:TreeNode.icon')}),\n this property allows the developer to specify on a per-node basis whether a\n drop state icon should be displayed when the \n user drop-hovers over this folder.<br>\n Set <code>node.showDropIcon</code> to true to show the drop state\n icon, or false to suppress this.<br>\n If not specified, this behavior is determined by ${isc.DocUtils.linkForRef('attr:TreeGrid.showCustomIconDrop')}\n for this node.\n <P>You can change the name of this property by setting \n ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconDropProperty')}.",
        name:"showDropIcon",
        seeAlso:[
            {ref:"attr:TreeGrid.customIconProperty", description:null},
            {ref:"attr:TreeGrid.showCustomIconDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:TreeNode",
        groups:["treeIcons"]
    },
    "attr:ComboBoxItem.pickerIconWidth":{
        ref:"attr:ComboBoxItem.pickerIconWidth",
        flags:"IRW",
        description:"Default the picker icon to a fixed with (15px).",
        name:"pickerIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"15",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:Snapbar.showClosedGrip":{
        ref:"attr:Snapbar.showClosedGrip",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether the grip image\n displayed should show the <code>\"Closed\"</code> state when the ${isc.DocUtils.linkForRef('attr:Splitbar.target')} \n is hidden.",
        name:"showClosedGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar",
        groups:["grip"]
    },
    "attr:ImgButton.showFocused":{
        ref:"attr:ImgButton.showFocused",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  If\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is <code>true</code>, the <b><code>\"over\"</code></b>\n will be used to indicate focus. Otherwise a separate <b><code>\"focused\"</code></b> state\n will be used.",
        name:"showFocused",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:Canvas.canDragReposition":{
        ref:"attr:Canvas.canDragReposition",
        flags:"IRWA",
        description:"Indicates whether this widget can be moved by a user of your application by simply\n      dragging with the mouse.",
        name:"canDragReposition",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragEffects')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:ImgButton.removeFromRadioGroup":{
        ref:"method:ImgButton.removeFromRadioGroup",
        flags:"",
        description:" Remove this widget from the specified mutually exclusive selection group with the ID\n passed in.\n No-op's if this widget is not a member of the groupID passed in.\n If no groupID is passed in, defaults to removing from whatever radioGroup this widget\n is a member of.\n",
        name:"removeFromRadioGroup",
        seeAlso:[],
        params:[
            {description:"- optional radio group ID (to ensure the widget is removed                                        from the appropriate group.",
             optional:true, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state", "event handling"]
    },
    "method:DetailViewerField.getCellStyle":{
        ref:"method:DetailViewerField.getCellStyle",
        flags:"",
        description:" Optional method to return the CSS class for cells in this field. If specified, this method\n will be called from ${isc.DocUtils.linkForRef('method:DetailViewer.getCellStyle')}, and should return a css class name.\n\n",
        name:"getCellStyle",
        seeAlso:[],
        params:[
            {description:"actual value of this cell", optional:false, name:"value", defaultValue:null,
             type:"string"},
            {description:"field object for this cell", optional:false, name:"field", defaultValue:null,
             type:"object"},
            {description:"record object for this cell", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"the viewer instance to which this cell belongs", optional:false,
             name:"viewer", defaultValue:null, type:"DetailViewer"}
        ],
        returns:{description:"CSS style for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"object:DetailViewerField",
        groups:["appearance"]
    },
    "attr:ListGrid.checkboxFieldFalseImage":{
        ref:"attr:ListGrid.checkboxFieldFalseImage",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance')} is set to <code>\"checkbox\"</code> this property\n determines the image to display in the checkbox field for an unselected row.\n If unset, the ${isc.DocUtils.linkForRef('attr:ListGrid.booleanFalseImage')} will be used.",
        name:"checkboxFieldFalseImage",
        seeAlso:[
            {ref:"attr:ListGrid.checkboxFieldTrueImage", description:null},
            {ref:"attr:ListGrid.checkboxFieldImageWidth", description:null},
            {ref:"attr:ListGrid.checkboxFieldImageHeight", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "method:GridRenderer.cellHoverHTML":{
        ref:"method:GridRenderer.cellHoverHTML",
        flags:"A",
        description:" StringMethod to dynamically assemble an HTML string to show in a hover window over the\n appropriate cell/record when this.canHover and this.showHover are both true.\n Called when the mouse hovers over a cell.\n\n",
        name:"cellHoverHTML",
        seeAlso:[
            {ref:"attr:GridRenderer.canHover", description:null},
            {ref:"attr:GridRenderer.showHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"the html to be shown inside the hover for this cell", type:"html"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:Layout.getDropPosition":{
        ref:"method:Layout.getDropPosition",
        flags:"A",
        description:"\n Get the position a new member would be dropped.  This drop position switches in the\n middle of each member, and both edges (before beginning, after end) are legal drop positions\n <p>\n Use this method to obtain the drop position for e.g. a custom drop handler.\n\n",
        name:"getDropPosition",
        seeAlso:[],
        returns:{description:"the position a new member would be dropped", type:"integer"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:HeaderSpan.name":{
        ref:"attr:HeaderSpan.name",
        flags:"IR",
        description:"Name for this headerSpan, for use in APIs like ${isc.DocUtils.linkForRef('method:ListGrid.setHeaderSpanTitle')}.\n <P>\n Name is optional, but if specified, must be unique for this ListGrid (but not globally\n unique).",
        name:"name",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "method:String.endsWith":{
        ref:"method:String.endsWith",
        flags:"",
        description:"			Returns true if this string ends with another string.\n",
        name:"endsWith",
        seeAlso:[],
        params:[
            {description:"other string to check", optional:false, name:"substring", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"true == this string ends with substring", type:"boolean"},
        type:"method",
        definingClass:"class:String",
        groups:["stringProcessing"]
    },
    "group:positioning":{
        ref:"group:positioning",
        refs:["method:Canvas.setSnapTo", "method:Canvas.getSnapTo", "method:Canvas.setSnapEdge", 
         "method:Canvas.getSnapEdge","method:Canvas.setRect", "method:Canvas.getLeft", "method:Canvas.setLeft", 
         "method:Canvas.getTop","method:Canvas.setTop", "method:Canvas.getRight", "method:Canvas.getBottom", 
         "method:Canvas.getScrollLeft","method:Canvas.getScrollTop", "method:Canvas.setPageLeft", "method:Canvas.setPageTop", 
         "method:Canvas.getPageLeft","method:Canvas.getPageTop", "method:Canvas.getPageRight", "method:Canvas.getPageBottom", 
         "method:Canvas.containsPoint","method:Canvas.visibleAtPoint", "method:Canvas.intersects", "method:Canvas.intersectsRect", 
         "method:Canvas.containsEvent","method:Canvas.getOffsetX", "method:Canvas.getOffsetY", "method:Canvas.moveBy", 
         "method:Canvas.moveTo","method:Canvas.getVSnapOrigin", "method:Canvas.getHSnapOrigin", 
         "method:Canvas.setOverflow","method:Button.setAlign", "method:Button.setVAlign", "attr:Canvas.htmlElement", 
         "attr:Canvas.htmlPosition","attr:Canvas.position", "attr:Canvas.left", "attr:Canvas.top", "attr:ImgTab.align", 
         "attr:Label.align","attr:Label.valign", "type:Positioning"],
        description:"Horizontal and vertical location on the page",
        name:"positioning",
        seeAlso:[],
        type:"group"
    },
    "attr:Layout.showDragPlaceHolder":{
        ref:"attr:Layout.showDragPlaceHolder",
        flags:"IRW",
        description:"If set to true, when a member is dragged out of layout, a visible placeholder canvas \n will be displayed in place of the dragged widget for the duration of the drag and drop\n interaction.",
        name:"showDragPlaceHolder",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragMove')}"],
        definingClass:"class:Layout",
        groups:["dragdrop"]
    },
    "method:TimeItem.setMask":{
        ref:"method:TimeItem.setMask",
        flags:"",
        description:" A custom mask cannot be defined for a time item.\n",
        name:"setMask",
        seeAlso:[
            {ref:"attr:TimeItem.useMask", description:null}
        ],
        type:"method",
        definingClass:"class:TimeItem"
    },
    "attr:FormItem.shouldSaveValue":{
        ref:"attr:FormItem.shouldSaveValue",
        flags:"IR",
        description:"Should this item's value be saved in the form's values and hence returned from\n ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form.getValues()')}?\n <p>\n <code>shouldSaveValue:false</code> is used to mark formItems which do not correspond to\n the underlying data model and should not save a value into the form's\n ${isc.DocUtils.linkForRef('attr:DynamicForm.values','values')}.  Example includes visual separators, password re-type fields,\n or checkboxes used to show/hide other form items.\n <p>\n A <code>shouldSaveValue:false</code> item should be given a value either via\n ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue')} or by calling can use\n ${isc.DocUtils.linkForRef('method:DynamicForm.setValue','form.setValue(item, value)')} or \n ${isc.DocUtils.linkForRef('method:FormItem.setValue','formItem.setValue(value)')}.  Providing a value via\n ${isc.DocUtils.linkForRef('attr:DynamicForm.values','form.values')} or ${isc.DocUtils.linkForRef('method:DynamicForm.setValues','form.setValues()')} \n will automatically switch the item to <code>shouldSaveValue:true</code>.\n <P>\n Note that <ul>\n <li>if an item is shouldSaveValue true, but has no name, a warning is logged, and \n     shouldSaveValue will be set to false.\n </li></ul>",
        name:"shouldSaveValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formValues"]
    },
    "method:CubeGrid.selectAllCells":{
        ref:"method:CubeGrid.selectAllCells",
        flags:"",
        description:"\n Select all cells\n\n      <br><i>methodType</i> action\n",
        name:"selectAllCells",
        seeAlso:[],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:DataSource.ID":{
        ref:"attr:DataSource.ID",
        flags:"IR",
        description:"Unique identifier for this DataSource.  Required for all DataSources.\n\n DataSources will make themselves available as JavaScript globals under the same name\n as their ID only if <code>addGlobalId</code> is set.",
        name:"ID",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["identity"]
    },
    "attr:TabSet.tabBar":{
        ref:"attr:TabSet.tabBar",
        flags:"R",
        description:"TabBar for this TabSet, an instance of ${isc.DocUtils.linkForRef('class:TabBar')}.",
        name:"tabBar",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "method:Canvas.getScrollHeight":{
        ref:"method:Canvas.getScrollHeight",
        flags:"A",
        description:"			Returns the scrollable height of the widget's contents, including children, ignoring\n          clipping.\n",
        name:"getScrollHeight",
        seeAlso:[],
        returns:{description:"height of the element that can scroll", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Canvas.addChild":{
        ref:"method:Canvas.addChild",
        flags:"",
        description:" Adds newChild as a child of this widget, set up a named object reference (i.e., this[name])\n to the new widget if name argument is provided, and draw the child if this widget has been\n drawn already.\n <P>\n If newChild has a parent it will be removed from it. If it has a master, it will be detached\n from it if the master is a child of a different parent. If newChild has peers, they'll be\n added to this widget as children as well.\n\n",
        name:"addChild",
        seeAlso:[],
        params:[
            {description:"new child canvas to add", optional:false, name:"newChild", defaultValue:null,
             type:"canvas"},
            {description:"name to assign to child (eg: this[name] == child)", optional:true,
             name:"name", defaultValue:null, type:"string"},
            {description:"if false, child will not automatically be drawn (only                                          for advanced use)",
             optional:true, name:"autoDraw", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"the new child, or null if it couldn't be added", type:"canvas"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "classAttr:Array.DATETIME_VALUES":{
        ref:"classAttr:Array.DATETIME_VALUES",
        flags:null,
        description:"This is a built-in comparator for the ${isc.DocUtils.linkForRef('method:Array.find','find')} and ${isc.DocUtils.linkForRef('method:Array.findIndex','findIndex')}\n methods of Array.  Passing this comparator to those methods will find instances where Dates  \n in the search criteria match Dates in the array members (ordinarily, Javascript only regards\n Dates as equal if they refer to the exact same object).  This comparator compares entire\n date values, including the time elements of the values being compared, so two Dates \n representing different times on the same day (even if they are only a millisecond apart) will \n be considered equal.",
        name:"DATETIME_VALUES",
        seeAlso:[
            {ref:"classAttr:Array.DATE_VALUES", description:null}
        ],
        valueType:"Function",
        defaultValue:"See below",
        type:"classAttr",
        definingClass:"object:Array"
    },
    "attr:Applet.objectCodeBase":{
        ref:"attr:Applet.objectCodeBase",
        flags:"IR",
        description:"This attribute is used only with useTag: \"object\".  In combination with \n  ${isc.DocUtils.linkForRef('attr:Applet.classID')} this tag specifies the minimum JVM version required to view\n  this applet and the URL from which the JVM can be downloaded.\n  <p>\n  The default settings match the default behavior of the &lt;applet&gt; tag which requires JVM\n  1.3 or higher.\n  <p>\n  The default objectCodeBase is: \"http://java.sun.com/products/plugin/1.3/jinstall-13-win32.cab#Version=1,3,0,0\"\n  <p>\n  For notes on this feature see: <a href='http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie' onclick=\"window.open('http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie');return false;\">http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie</a>",
        name:"objectCodeBase",
        seeAlso:[
            {ref:"attr:Applet.classID", description:null}
        ],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:ListGridField.iconOrientation":{
        ref:"attr:ListGridField.iconOrientation",
        flags:"IR",
        description:"If this field is showing an icon, should it appear to the left or right of the title?<br>\n Valid options are <code>\"left\"</code> or <code>\"right\"</code>",
        name:"iconOrientation",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"string",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:TabSet.showTabScroller":{
        ref:"attr:TabSet.showTabScroller",
        flags:"IR",
        description:"If there is not enough space to display all the tab-buttons in this tabSet, should \n scroller buttons be displayed to allow access to tabs that are clipped?",
        name:"showTabScroller",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "group:dataBinding":{
        ref:"group:dataBinding",
        refs:["attr:ListGrid.autoFetchTextMatchStyle", "attr:TreeGrid.autoFetchTextMatchStyle"],
        name:"dataBinding",
        seeAlso:[],
        type:"group"
    },
    "attr:OperationBinding.dataTransport":{
        ref:"attr:OperationBinding.dataTransport",
        flags:"IR",
        description:"Transport to use for this operation.  Defaults to ${isc.DocUtils.linkForRef('attr:DataSource.dataTransport')}, which in\n turn defaults to ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')}.  This would typically only be set to\n enable \"scriptInclude\" transport for contacting ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','JSON')} web\n services hosted on servers other than the origin server.\n <p>\n When using the \"scriptInclude\" transport, be sure to set ${isc.DocUtils.linkForRef('attr:DataSource.callbackParam')} or\n ${isc.DocUtils.linkForRef('attr:OperationBinding.callbackParam')} to match the name of the query parameter name expected\n by your JSON service provider.",
        name:"dataTransport",
        seeAlso:[
            {ref:"type:RPCTransport", description:null},
            {ref:"attr:DataSource.callbackParam", description:null}
        ],
        valueType:"RPCTransport",
        defaultValue:"RPCManager.defaultTransport",
        type:"attr",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:FormItem.imageURLPrefix":{
        ref:"attr:FormItem.imageURLPrefix",
        flags:"IRWA",
        description:"Prefix to apply to the beginning of any ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} when determining the\n URL for the image.\n Will not be applied if the <code>valueIcon</code> URL is absolute.",
        name:"imageURLPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:SelectItem.pickButtonWidth":{
        ref:"attr:SelectItem.pickButtonWidth",
        deprecated:"As of SmartClient version 5.5, pickButtonWidth has been deprecated in   favor of ${isc.DocUtils.linkForRef('attr:SelectItem.pickerIconWidth')}.",
        flags:"IRWA",
        description:"How large should the pick button be rendered?",
        name:"pickButtonWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:ToolStrip.resizeBarClass":{
        ref:"attr:ToolStrip.resizeBarClass",
        flags:"IRW",
        description:"Customized resizeBar with typical appearance for a ToolStrip",
        name:"resizeBarClass",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ToolStripResizer\"",
        type:"attr",
        definingClass:"class:ToolStrip"
    },
    "attr:DataBoundComponent.dataSource":{
        flags:"IRW",
        type:"attr",
        ref:"attr:DataBoundComponent.dataSource",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        setter:["setDataSource"],
        valueType:"DataSource or ID",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        defaultValue:"null",
        groups:["databinding"],
        definingClass:"interface:DataBoundComponent"
    },
    "method:CubeGrid.getFacetsHavingSelection":{
        ref:"method:CubeGrid.getFacetsHavingSelection",
        flags:"",
        description:"\n  return the list of facets that have any selection in their headers\n\n  <br><i>methodType</i> getter\n",
        name:"getFacetsHavingSelection",
        seeAlso:[],
        returns:{description:"list of facets that have any selection in their headers", type:"Array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:ResultSet.getProperty":{
        ref:"method:ResultSet.getProperty",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.getProperty')}.  Checks only loaded rows and will not trigger a fetch.\n",
        name:"getProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to look for", optional:false, name:"property",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"array of the values of property in each item of this list", type:"Array"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["iteration"]
    },
    "attr:DatabaseBrowser.serverType":{
        ref:"attr:DatabaseBrowser.serverType",
        flags:"IR",
        description:"The type of database server this DatabaseBrowser should connect to. Valid values are \n \"sql\" (SmartClient's own built-in support for SQL databases) or \"hibernate\"",
        name:"serverType",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"sql\"",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "method:Canvas.getParentElements":{
        ref:"method:Canvas.getParentElements",
        flags:"A",
        description:"      Returns an array of object references to all ancestors of this widget in the containment\n      hierarchy, starting with the direct parent and ending with the top element.\n",
        name:"getParentElements",
        seeAlso:[],
        returns:{description:"array of parentElements, closest first; empty array if no parentElements",
         type:"array"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "method:FormItem.parseEditorValue":{
        ref:"method:FormItem.parseEditorValue",
        flags:"",
        description:" An optional ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')} to map a user-entered display value\n to a data value for storage.  This method only applies to form items which show a\n freeform text entry area, such at the ${isc.DocUtils.linkForRef('class:TextItem')} or ${isc.DocUtils.linkForRef('class:TextAreaItem')}.\n <P>\n See also ${isc.DocUtils.linkForRef('method:FormItem.formatEditorValue')}\n",
        name:"parseEditorValue",
        seeAlso:[],
        params:[
            {description:"value as entered by the user", optional:false, name:"value",
             defaultValue:null, type:"string"},
            {description:"pointer to the dynamicForm containing this item", optional:false,
             name:"form", defaultValue:null, type:"DynamicForm"},
            {description:"pointer to this item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"}
        ],
        returns:{description:"Data value to store for this item.", type:"any"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:Canvas.sendToBack":{
        ref:"method:Canvas.sendToBack",
        flags:"",
        description:" Puts this widget at the bottom of the stacking order, so it appears behind all other\n widgets in the same parent.\n",
        name:"sendToBack",
        seeAlso:[],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('layer')}"],
        definingClass:"class:Canvas",
        groups:["zIndex"]
    },
    "attr:TileLayout.tiles":{
        ref:"attr:TileLayout.tiles",
        flags:"IR",
        description:"List of tiles to lay out.",
        name:"tiles",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout"
    },
    "group:sqlVsJPA":{
        ref:"group:sqlVsJPA",
        title:"SQL DataSource vs JPA, EJB, Ibatis and other technologies",
        treeLocation:["Concepts"],
        refs:[],
        description:"If you are free to choose which persistence mechanism your application will use, you should\n consider using the SmartClient SQL DataSource instead of a more heavyweight, bean-based \n solution.  This article discusses the advantages of doing so.\n <p>\n <b>Simplicity</b>\n <p>\n With the SmartClient SQL DataSource, simple CRUD connectivity can be set up via a \n ${isc.DocUtils.linkForExampleId('sqlWizard', 'wizard')} and requires zero server side code.  Only a DataSource \n descriptor (.ds.xml file) needs to exist; this descriptor can be generated by the wizard \n or created by hand.  The descriptor actually serves double duty by also providing the \n configuration for UI components - in other words, this is information that you would need \n to express anyway.\n <p>\n Semi-technical product managers, testers, business analysts and IT staff who have no\n familiarity with Java can easily comprehend DataSource definitions and even customized\n SQL queries, allowing them to go further with prototyping efforts, provide more specific\n feedback and capture more relevant diagnostics when reporting issues.\n <p>\n This level of simplicity is lost when using more heavyweight systems.  JPA / EJB \n best practices indicate creation of a bean class for every domain object, as well as \n related \"services\" or \"session beans\", DTOs \n (<a href='http://en.wikipedia.org/wiki/Data_Transfer_Object' onclick=\"window.open('http://en.wikipedia.org/wiki/Data_Transfer_Object');return false;\">Data Transfer Objects</a>) and \n other unnecessary scaffolding.  Ibatis avoids some of this scaffolding, but requires every \n SQL query to be written by hand.  In contrast the SQL DataSource supports basic CRUD queries\n out of the box.\n <p>\n <b>Performance</b>\n <p>\n Systems like JPA work nicely when dealing with a single object at a time, but enterprise\n applications routinely work with lists or trees of objects that draw data from multiple\n tables.  In these situations, it's trivial to express an efficient SQL query for retrieving\n the desired results (as shown in ${isc.DocUtils.linkForExampleId('largeValueMap', 'this example')}).  Fetching the\n same data using getter methods on Java Beans often leads to nightmare performance scenarios\n (such as 3 or more separate SQL queries per object retrieved).\n <P>\n Trying to \"trick\" the persistence system into generating efficient queries doesn't make\n sense - this just leads to a far more complex and fragile solution that now requires deep\n knowledge of how the ORM system generates SQL as well as SQL itself. \n <P>\n SQLDataSource allows you to directly write SQL when it makes sense, and \n ${isc.DocUtils.linkForRef('attr:DataSource.beanClassName','to use beans')} when object oriented approaches are\n clearer and simpler.  When you do write SQL directly, you override just the parts of the\n query that you need to change - you still leverage SQLDataSource's ability to generate\n cross-database SQL for complex search criteria, efficient data paging and sorting, even in a\n complex reporting query (see ${isc.DocUtils.linkForExampleId('dynamicReporting', 'this example')}).\n <p>\n <b>Portability</b>\n <p>\n SmartClient DataSources provide cross-database portability like JPA and other solutions.\n However, DataSources can also be replaced with an entirely different integration strategy or\n entirely different server platform, such as a SOA architecture where the browser contacts\n WSDL web services directly.  The clear data requirements definition represented by a\n DataSource makes such drastic technology changes much easier with the SQL DataSource than\n with any other technology.\n <p>\n <b>Power</b>\n <p>\n The SQL DataSource has out of the box support for server-side advanced filtering without\n the need to write any code (see the \n ${isc.DocUtils.linkForExampleId('filterBuilderBracket', 'SQL Advanced Filtering example')}), and SmartClient\n provides ${isc.DocUtils.linkForRef('class:FilterBuilder','pre-built user interfaces for filtering')}.  The effort\n required to develop similar functionality with another persistence mechanism would vary from \n substantial to spectacular.\n <p>\n You can leverage advanced, automatic SQL generation, such as advanced filter criteria, \n GROUP BY and ORDER BY clauses, and selection of row ranges, even in very heavily customized\n queries.  The ${isc.DocUtils.linkForExampleId('dynamicReporting', 'Dynamic Reporting example')} shows this.\n <p>\n With the SQL DataSource and Transaction Chaining, you can chain together multiple SQL \n queries, or a mixture of SQL queries and other data access, with simple declarations right \n in the DataSource, as ${isc.DocUtils.linkForExampleId('queuedAdd', 'this example')} demonstrates.\n <p>\n Because you write the SQL, you can use database-specific features when absolutely \n necessary.  Features such as query optimizer hints or stored procedures are thus accessible\n but, importantly, are within the same processing model used for all other data access.\n <p>\n <b>Security</b>\n <p>\n Because the central DataSource definition expresses all the available operations, how they\n are performed and who has access to them, things are clear and simple.  It's much easier to \n understand and audit a DataSource definition than a slew of Java classes.\n <p>\n There is no information leakage from server to client with the SQL DataSource.  All \n server-side declarations, such as SQL templates, are automatically stripped out of the \n DataSource definition before the browser sees it.\n <p>\n Custom SQL in a SmartClient SQL DataSource is protected from SQL injection attacks.  It is\n impossible for a developer to write a SQL template that is vulnerable to SQL injection \n without going through the ${isc.DocUtils.linkForRef('group:velocitySupport','$rawValue')} feature, a rarely used\n feature that is very prominently flagged in the documentation as requiring special care.\n Other ORM systems tend to require hand-coded SQL queries for advanced use cases such as\n reporting; these hand-written queries are where most security holes appear.  By providing a\n safe environment for SQL customizations, SQL DataSource removes these risks.",
        name:"sqlVsJPA",
        seeAlso:[],
        type:"group"
    },
    "method:DataBoundComponent.getTitleField":{
        ref:"method:DataBoundComponent.getTitleField",
        flags:"",
        description:" Method to return the fieldName which represents the \"title\" for records in this\n Component.<br>\n If this.titleField is explicitly specified it will always be used.\n Otherwise, default implementation will check ${isc.DocUtils.linkForRef('attr:DataSource.titleField')} for databound\n compounds.<br>\n For non databound components returns the first defined field name of <code>\"title\"</code>, \n <code>\"name\"</code>, or <code>\"id\"</code>. If we dont find any field-names that match these\n titles, the first field in the component will be used instead.\n",
        name:"getTitleField",
        seeAlso:[],
        returns:{description:"fieldName for title field for this component.", type:"string"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "group:flags":{
        ref:"group:flags",
        title:"Flag Abbreviations",
        refs:[],
        description:"<ul>\n <li> <b>I</b>: property can be initialized (provided in constructor block)\n <li> <b>R</b>: property can be read.  If a getter method exists, it must be called.\n <li> <b>W</b>: property can be written to after initialization.  If a setter method\n exists, it must be called.  If no setter method exists,\n ${isc.DocUtils.linkForRef('method:Class.setProperty','setProperty()')} must be called.\n </ul>",
        name:"flags",
        seeAlso:[],
        type:"group"
    },
    "method:TileGrid.transferSelectedData":{
        ref:"method:TileGrid.transferSelectedData",
        flags:"",
        description:" Simulates a drag / drop type transfer of the selected records in some other component to this\n component, without requiring any user interaction.  This method acts on the dropped records \n exactly as if they had been dropped in an actual drag / drop interaction, including any \n special databound behavior invoked by calling\n ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues','getDropValues')} for each dropped record.\n <P>\n To transfer <b>all</b> data in, for example, a ${isc.DocUtils.linkForRef('class:ListGrid')}, call grid.selection.selectAll() first.\n <P>\n Note that drag/drop type transfers of records between components are asynchronous operations:\n SmartClient may need to perform server turnarounds to establish whether dropped records \n already exist in the target component.  Therefore, it is possible to issue a call to \n transferSelectedData() and/or the ${isc.DocUtils.linkForRef('method:ListGrid.drop','drop()')} method of a databound \n component whilst a transfer is still active.  When this happens, SmartClient adds the \n second and subsequent transfer requests to a queue and runs them one after the other.  If \n you want to be notified when a transfer process has actually completed, either provide a \n callback to this method or implement ${isc.DocUtils.linkForRef('method:DataBoundComponent.dropComplete')}.\n <P>\n See the ${isc.DocUtils.linkForRef('group:dragging')} documentation for an overview of list grid drag/drop data\n transfer.\n \n",
        name:"transferSelectedData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"},
            {description:"target index (drop position) of the rows within this grid.", optional:true,
             name:"index", defaultValue:null, type:"integer"},
            {description:"optional callback to be fired when the transfer process has                        completed.  The callback will be passed a single parameter \"records\",\n                       the list of records actually transferred to this component.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}"],
        definingClass:"class:TileGrid",
        groups:["dragging"]
    },
    "method:CubeGrid.hiliteFacetValue":{
        ref:"method:CubeGrid.hiliteFacetValue",
        flags:"",
        description:"\n      apply a hilite to all cells corresponding to a facetValue.\n\n      <br><i>methodType</i> action\n",
        name:"hiliteFacetValue",
        seeAlso:[],
        params:[
            {description:"facet ID", optional:false, name:"facetID", defaultValue:null, type:"string"},
            {description:"facet value ID", optional:false, name:"facetValueID", defaultValue:null,
             type:"string"},
            {description:"hilite ID", optional:false, name:"hiliteID", defaultValue:null, type:"string"}
        ],
        returns:{description:"true if the cells were successfully hilited.", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["hiliting"]
    },
    "attr:DateChooser.showWeekends":{
        ref:"attr:DateChooser.showWeekends",
        flags:"IR",
        description:"Whether weekend days should be shown.  Which days are considered weekends is controlled\n by ${isc.DocUtils.linkForRef('classAttr:Date.weekendDays')}.",
        name:"showWeekends",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "group:docViewerHelp":{
        ref:"group:docViewerHelp",
        title:"SmartClient Reference Overview",
        refs:[],
        description:"Welcome to the SmartClient Reference.  Click on a topic in the tree on the left to show\n documentation in this pane or type some search terms into the search box in the upper left\n and hit Enter.\n <p>\n To bring back this help page, click the blue question mark in the upper right corner.",
        name:"docViewerHelp",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.setMargin":{
        ref:"method:Canvas.setMargin",
        flags:"",
        description:" Set the CSS Margin, in pixels, for this component.  Margin provides blank space outside of\n the border.\n <P>\n This property sets the same thickness of margin on every side.  Differing per-side\n margins can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.\n <P>\n Note that the specified size of the widget will be the size <b>including</b> the margin\n thickness on each side.\n \n",
        name:"setMargin",
        seeAlso:[],
        params:[
            {description:"new margin in pixels", optional:false, name:"margin", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:ListGrid.getToggleFreezeText":{
        ref:"method:ListGrid.getToggleFreezeText",
        flags:"",
        description:" If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','this.canFreezeFields')} is true, this string will be shown as the\n title for the menu item to toggle whether a field is frozen or unfrozen.\n <P>\n Default implementation evaluates and returns ${isc.DocUtils.linkForRef('attr:ListGrid.freezeFieldText')} or\n ${isc.DocUtils.linkForRef('attr:ListGrid.unfreezeFieldText')} depending on whether the field is currently frozen.\n",
        name:"getToggleFreezeText",
        seeAlso:[],
        params:[
            {description:"field to get the menu item title for", optional:false, name:"field",
             defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"Title to show in the menu item", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "method:Canvas.containsEvent":{
        ref:"method:Canvas.containsEvent",
        flags:"",
        description:"			Return true if the last event's mouse coordinates are within the bounds of this component.\n		NOTE: Z-ordering is not considered for the purposes of this test.  If the coordinate you're\n		testing is occluded by other component, but the X,Y coordiates are still within the bounds\n		of that component, this method will return true.\n\n",
        name:"containsEvent",
        seeAlso:[],
        returns:{description:"true if the event occurred within the bounds of this component", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["events", "positioning"]
    },
    "attr:ListGridField.showDisabledIcon":{
        ref:"attr:ListGridField.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "type:SendMethod":{
        ref:"type:SendMethod",
        values:[
            {description:"GET method (regular URL encoding)", value:"\"GET\""},
            {description:"POST method (form field encoding)", value:"\"POST\""}
        ],
        description:null,
        name:"SendMethod",
        seeAlso:[],
        type:"type",
        groups:["communication"]
    },
    "method:ResultSet.getCriteria":{
        ref:"method:ResultSet.getCriteria",
        flags:"",
        description:" Get the current criteria for this ResultSet.\n",
        name:"getCriteria",
        seeAlso:[],
        returns:{description:"current criteria", type:"Criteria"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:ListGridField.formatGroupSummary":{
        ref:"attr:ListGridField.formatGroupSummary",
        flags:"IR",
        description:"Optional stringMethod to format the group level summary values for this field displayed via\n ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')}.\n Takes a single parameter <code>value</code> and should return the formatted version\n of that value\n <P>\n Note that for fields with a specified summary function of \"count\", if this method is \n undefined we default to formatting the count value by appending \n <code>field.pluralTitle</code> if defined, otherwise <code>field.title</code> to the\n numeric count value returned by the standard count function. To change this behavior for\n such fields, specify an explicit 'formatGridSummary' and/or 'formatGroupSummary' method",
        name:"formatGroupSummary",
        seeAlso:[],
        valueType:"stringMethod",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:Chart.getValue":{
        ref:"method:Chart.getValue",
        flags:"",
        description:" Lookup a data value by the set of matching facetValues expressed as a\n ${isc.DocUtils.linkForRef('type:FacetValueMap')}.  Automatically handles the ${isc.DocUtils.linkForRef('attr:Chart.data','inlinedFacet')}, if any.\n <P>\n This method is designed to be called by a concrete Chart implementation.\n <P>\n As a special case, if ${isc.DocUtils.linkForRef('attr:Chart.data')} is a single Object and the only facet is\n ${isc.DocUtils.linkForRef('attr:Chart.data','inlined')}, any value can be used as the single facetId required.  For example, a\n legal FacetValueMap in this case would be {inlined:\"west\"}.\n\n",
        name:"getValue",
        seeAlso:[],
        params:[
            {description:"set of facetValues describing the data value to retrieve", optional:false,
             name:"criteria", defaultValue:null, type:"FacetValueMap"}
        ],
        returns:{description:"matching value from the dataset, or null if no value exists", type:"any"},
        type:"method",
        definingClass:"interface:Chart"
    },
    "method:DataBoundComponent.setDragTracker":{
        ref:"method:DataBoundComponent.setDragTracker",
        flags:"",
        description:" Sets the custom tracker HTML to display next to the mouse when the user initiates a drag\n operation on this component. Default implementation will examine ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')}\n and set the custom drag tracker to display the appropriate HTML based on the selected record.\n <br>\n To display custom drag tracker HTML, this method may be overridden - call \n ${isc.DocUtils.linkForRef('classMethod:EventHandler.setDragTracker')} to actually update the drag tracker HTML.\n",
        name:"setDragTracker",
        seeAlso:[],
        returns:{description:"returns false by default to suppress 'setDragTracker' on any ancestors                   of this component.",
         type:"boolean"},
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["dragTracker"]
    },
    "group:strutsIntegration":{
        ref:"group:strutsIntegration",
        title:"Integration with Struts",
        treeLocation:["Concepts"],
        refs:[],
        description:"<b>Overview of SmartClient integration with Struts and other popular server-side\n frameworks.</b>\n <p>\n Current and upcoming server-side frameworks solve a lot of problems endemic to the past\n generation of web applications.  Piles of JSPs and Servlets have been replaced by MVC and\n other paradigms that enhance developer productivity.  Unfortunately the end-user\n presentation remains vanilla HTML.  SmartClient solves this problem by providing rich\n databindable components. SmartClient was designed to integrate cleanly with existing\n server-side frameworks.  Because SmartClient components only require an HTML context to\n render, they can be used with any server-side frameworks that use HTML for presentation.\n <p>\n SmartClient provides a rich UI by doing more work on the client (browser) rather than\n sending every user action to the server for re-rendering.  Instead of doing page transitions\n to redraw the whole page, SmartClient sends RPC (Remote Procedure Call) requests (or AJAX\n requests) to the server while the UI allows the user to continue interacting with the\n system.  Current server-side frameworks, on the other hand are typically designed around\n page transitions - for example in Struts user actions are typically mapped to URLs that\n dispatch through a central servlet and ultimately return new HTML to be rendered by the\n browser.  The problem with page transitions is that they destroy client-side state and\n introduce client-server latency (and generally use more bandwidth since HTML is sent over\n the wire rather than just data) - essentially destroying a large part of the Rich\n Internet Application (RIA) experience.\n <p>\n Fortunately, there's a way to get the best of both worlds - to leverage the power of your\n favorite server-side framework and combine it with the SmartClient UI.  There are several\n approaches to integrating SmartClient into an existing framework:\n <p>\n <u><b>Plug-replacing HTML components with SmartClient components</b></u>\n <p>\n SmartClient components can be instructed to draw at page load time using by specifying\n <code>position: \"relative\"</code> at construction time.  This enables you to replace any\n chunk of HTML with a SmartClient component - the new component simply inserts its HTML in\n the page flow during page load.  This is the easiest integration option - you get a better\n UI with minimal work.  The downside is that you don't get the full power of a rich client\n because most user actions will still trigger a page transition.\n <p>\n <u><b>Eliminating page transitions</b></u>\n <p>\n Most SmartClient components can accept new data (or even dynamically pre-fetch and expire\n data) without needing to be recreated.  For example - let's say you want to draw a grid on a\n page.  In a traditional server-side-rendered application the server would generate all of\n the html with \"next 20 records\" and \"previous 20 records\" buttons. When the user wants to\n see the next set of data, he clicks one of the buttons and the server replaces the entire\n page with a new grid that contains the next/previous 20 records.  In a SmartClient\n application, you would create a databound ListGrid.  Based on its configuration this grid\n will fetch the first N (say 20) records and display a scrollbar for the user to scroll\n through the data.  When the user scrolls off the last cached record the ListGrid\n automatically sends an RPC to the server asking for the next 20 records.  This RPC (fetch)\n is performed without destroying the page the user is currently looking at - it just happens\n seamlessly in the background.  If the user now scrolls back to the first 20 records -\n they're already cached in the grid, so no fetch is performed.  Of course, in a real world\n application, it's typical that a page has hundreds of components and in a server-side-only\n rendering all of them need to be rebuilt by the server and resent to the client when a piece\n of data in just one needs to be updated.  SmartClient components can intelligently update\n just their data without the need to redraw the whole page.\n <p>\n The plug-replacement strategy listed above gives us a SmartClient component in place of a\n raw HTML rendering.  Now we need to databind that component so that actions like scrolling a\n grid or validating a form don't cause a page transition.  To do this, you need to set up a \n Struts Action that will handle <b>all</b> SmartClient <code>RPCRequest</code>s and \n <code>DSRequest</code>s.  This is important, as requests need to be sent to the same URL \n to enable queuing to work.  In your Action class, you simply need to invoke \n <code>RPCManager.processRequest()</code> to hook straight into the normal \n <code>DSRequest</code> processing flow.\n <p>\n The SDK contains a simple example of doing form validation without incurring a page\n transition. These examples also show how to populate e.g. field names using the struts-bean\n taglib and how to set validation errors using the standard Struts Validation plugin.\n Point your browser to <a href='/examples/struts/forms' onclick=\"window.open('/examples/struts/forms');return false;\">/examples/struts/forms</a> in the SmartClient SDK to take a\n look.",
        name:"strutsIntegration",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.setAutoFitMaxColumns":{
        ref:"method:ListGrid.setAutoFitMaxColumns",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxColumns')}.\n",
        name:"setAutoFitMaxColumns",
        seeAlso:[],
        params:[
            {description:"Maximum number of fields we'll expand to accomodate if  ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','auto fit')} is enabled horizontally.",
             optional:false, name:"maxColumns", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "attr:LoginDialog.registrationItem":{
        ref:"attr:LoginDialog.registrationItem",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('class:LinkItem')} to page requesting new user registration in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.\n <p>To handle user clicks on this link, implement ${isc.DocUtils.linkForRef('method:LoginDialog.register')}.\n <p>To handle a user click as a physical link to another page, set\n ${isc.DocUtils.linkForRef('linkItem.defaultValue')} via loginDialog.registrationItemProperties:\n <code>\n registrationItemProperties: {\n     defaultValue: \"register.html\"\n },\n </code>",
        name:"registrationItem",
        seeAlso:[
            {ref:"attr:LoginDialog.showRegistrationLink", description:null},
            {ref:"attr:LoginDialog.registrationItemTitle", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:StatefulCanvas.valign":{
        ref:"attr:StatefulCanvas.valign",
        flags:"IRW",
        description:"Vertical alignment of this component's title.",
        name:"valign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["appearance"]
    },
    "method:DataSource.getTypeOperatorMap":{
        ref:"method:DataSource.getTypeOperatorMap",
        flags:"",
        description:" Get the list of ${isc.DocUtils.linkForRef('object:Operator')}s available for this ${isc.DocUtils.linkForRef('type:FieldType')}, as a ${isc.DocUtils.linkForRef('type:ValueMap')} from\n ${isc.DocUtils.linkForRef('type:OperatorId')} to the ${isc.DocUtils.linkForRef('attr:Operator.title')} specified for the ${isc.DocUtils.linkForRef('object:Operator')}, or\n the corresponding property in ${isc.DocUtils.linkForRef('class:Operators')} if ${isc.DocUtils.linkForRef('attr:Operator.titleProperty')} is set.\n <P>\n This valueMap is suitable for use in a UI for building queries, similar to the\n ${isc.DocUtils.linkForRef('class:FilterBuilder')}, and optionally omits operators marked ${isc.DocUtils.linkForRef('attr:Operator.hidden')}:true.\n <p>\n It is also possible to have this function return only operators of a given \n ${isc.DocUtils.linkForRef('type:OperatorValueType')}, or everything except operators of that type. This is useful, \n for example, if you want to return all the logical operators (like \"and\"), or everything \n except the logical operators.\n\n",
        name:"getTypeOperatorMap",
        seeAlso:[
            {ref:"method:DataSource.getFieldOperatorMap", description:null}
        ],
        params:[
            {description:"Type to obtain operator map for. Defaults to \"text\" if not passed.",
             optional:true, name:"type", defaultValue:null, type:"FieldType"},
            {description:"whether to include Operators marked hidden:true", optional:true,
             name:"includeHidden", defaultValue:null, type:"boolean"},
            {description:"If passed, returns only operators of this ${isc.DocUtils.linkForRef('type:OperatorValueType')}",
             optional:true, name:"valueType", defaultValue:null, type:"OperatorValueType"},
            {description:"If set, reverses the meaning of the <code>valueType</code>         parameter, so operators of that ${isc.DocUtils.linkForRef('type:OperatorValueType')} are the only ones omitted",
             optional:true, name:"omitValueType", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"mapping from ${isc.DocUtils.linkForRef('type:OperatorId')} to title, as described above",
         type:"ValueMap"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "class:SelectOtherItem":{
        methods:[],
        exampleConfig:"SelectOtherItem.xml",
        ref:"class:SelectOtherItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:SelectOtherItem.separatorTitle", "attr:SelectOtherItem.separatorValue", 
         "attr:SelectOtherItem.otherTitle","attr:SelectOtherItem.otherValue", "attr:SelectOtherItem.selectOtherPrompt", 
         "attr:SelectOtherItem.dialogWidth"],
        description:"FormItem that shows a list of options, plus an \"Other...\" option that allows them to enter\n another value.",
        name:"SelectOtherItem",
        seeAlso:[],
        type:"class"
    },
    "method:Button.isSelected":{
        ref:"method:Button.isSelected",
        flags:"",
        description:" Find out if this object is selected\n",
        name:"isSelected",
        seeAlso:[],
        returns:{description:null, type:"boolean"},
        type:"method",
        definingClass:"class:Button",
        groups:["state"]
    },
    "method:Class.logInfo":{
        ref:"method:Class.logInfo",
        flags:"",
        description:" Log a message at \"info\" priority\n \n",
        name:"logInfo",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:ListGrid.saveLocally":{
        ref:"attr:ListGrid.saveLocally",
        flags:"IRA",
        description:"For grids with a specified ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')}, this property can be set to \n <code>true</code> to avoid the grid from attempting to save / retrieve data from the\n server.  In this case the grid's data should be specified as an array via \n the ${isc.DocUtils.linkForRef('attr:ListGrid.data')} attribute, and the datasource will simply act as a schema to\n describe the set of fields visible in the grid. \n ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit','Inline edits')}, or removals via the\n ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} mechanism will update the local data array rather than\n attempting to perform operations against the dataSource.",
        name:"saveLocally",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "attr:ColorPicker.cancelButton":{
        ref:"attr:ColorPicker.cancelButton",
        flags:"R",
        description:"Cancel button for the ColorPicker",
        name:"cancelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:Canvas.hoverOpacity":{
        ref:"attr:Canvas.hoverOpacity",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, should the hover canvas be shown with opacity\n other than 100?",
        name:"hoverOpacity",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customHovers')}"],
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:ListGrid.fetchRelatedData":{
        ref:"method:ListGrid.fetchRelatedData",
        flags:"",
        description:" Based on the relationship between the DataSource this component is bound to and the\n DataSource specified as the \"schema\" argument, call fetchData() to retrieve records in this\n grid that are related to the passed-in record.\n <P>\n Relationships between DataSources are declared via ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')}.\n <P>\n For example, given a DataSource \"orders\" and another DataSource \"orderItems\", where\n \"orderItems\" declared a field \"orderId\" pointing to the primary key field of the \"orders\"\n DataSource, there is a set of records from the \"orderItems\" DataSource related to any given\n record from the \"order\" DataSource.  If this component were bound to \"orderItems\" and a\n record from the \"orders\".\n\n",
        name:"fetchRelatedData",
        seeAlso:[],
        params:[
            {description:"DataSource record", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"schema of the DataSource record, or                            DataBoundComponent already bound to that schema",
             optional:false, name:"schema", defaultValue:null, type:"Canvas or DataSource or ID"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Button.actionType":{
        flags:"IRW",
        getter:["getActionType"],
        type:"attr",
        ref:"attr:Button.actionType",
        description:"Behavior on state changes -- BUTTON, RADIO or CHECKBOX",
        name:"actionType",
        setter:["setActionType"],
        valueType:"SelectionType",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonRadioToggle')}"],
        defaultValue:"\"button\"",
        groups:["state", "event handling"],
        definingClass:"class:Button"
    },
    "attr:ListGridField.headerTitleStyle":{
        ref:"attr:ListGridField.headerTitleStyle",
        flags:"IRW",
        description:"Custom titleStyle to apply to this field's header button instead of \n ${isc.DocUtils.linkForRef('attr:ListGrid.headerTitleStyle')}.<br>\n Note that this will typically only have an effect if \n ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor')} is set to ${isc.DocUtils.linkForRef('class:StretchImgButton')} or a subclass \n thereof.",
        name:"headerTitleStyle",
        seeAlso:[],
        valueType:"cssClass",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "attr:ToolbarItem.endRow":{
        ref:"attr:ToolbarItem.endRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["appearance"]
    },
    "class:ColorPicker":{
        methods:["method:ColorPicker.setSupportsTransparency", "method:ColorPicker.getRed", 
         "method:ColorPicker.getGreen","method:ColorPicker.getBlue", "method:ColorPicker.getHue", 
         "method:ColorPicker.getSaturation","method:ColorPicker.getLuminosity", "method:ColorPicker.getHtmlColor", 
         "method:ColorPicker.getOpacity","method:ColorPicker.setRed", "method:ColorPicker.setGreen", "method:ColorPicker.setBlue", 
         "method:ColorPicker.setHue","method:ColorPicker.setSaturation", "method:ColorPicker.setLuminosity", 
         "method:ColorPicker.setHtmlColor","method:ColorPicker.setOpacity", "method:ColorPicker.colorChanged", 
         "method:ColorPicker.colorSelected"],
        ref:"class:ColorPicker",
        classAttrs:[],
        classMethods:["classMethod:ColorPicker.getSharedColorPicker"],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:ColorPicker.okButton", "attr:ColorPicker.okButtonConstructor", 
         "attr:ColorPicker.cancelButton","attr:ColorPicker.cancelButtonConstructor", "attr:ColorPicker.modeToggleButton", 
         "attr:ColorPicker.modeToggleButtonConstructor","attr:ColorPicker.defaultColor", "attr:ColorPicker.colorButtonSize", 
         "attr:ColorPicker.colorButtonBaseStyle","attr:ColorPicker.colorArray", "attr:ColorPicker.swatchWidth", 
         "attr:ColorPicker.swatchHeight","attr:ColorPicker.lumWidth", "attr:ColorPicker.supportsTransparency", 
         "attr:ColorPicker.opacityText","attr:ColorPicker.swatchImageURL", "attr:ColorPicker.crosshairImageURL", 
         "attr:ColorPicker.basicColorLabel","attr:ColorPicker.selectedColorLabel", "attr:ColorPicker.opacitySliderLabel", 
         "attr:ColorPicker.defaultOpacity","attr:ColorPicker.autoPosition", "attr:ColorPicker.autoCenterOnShow", 
         "attr:ColorPicker.defaultPickMode","attr:ColorPicker.allowComplexMode"],
        description:"The ColorPicker widget allows the user to select a color from anywhere in the \n color spectrum. It also supports selecting the alpha (opacity) value of the \n color.  The picker supports a simple mode - which allows for one-click selection\n from a standard palette of colors - and a complex mode which allow the user to\n define any conceivable color. It is possible for the user to switch from simple\n mode to complex by interacting with the widget.  In general, the widget provides\n very similar functionality to the color picker dialogs found in graphics packages\n and other desktop software.",
        name:"ColorPicker",
        seeAlso:[],
        type:"class"
    },
    "attr:TreeGrid.fileImage":{
        ref:"attr:TreeGrid.fileImage",
        deprecated:"as part of SmartClient release 5.5 in favor of ${isc.DocUtils.linkForRef('attr:TreeGrid.nodeIcon')}",
        flags:"IRW",
        description:"The filename of the default icon for all leaf nodes in this treeGrid. Use the\n node.icon property (null by default) to specify a custom image for an individual\n node.",
        name:"fileImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]file.gif\"",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "method:Canvas.setStyleName":{
        ref:"method:Canvas.setStyleName",
        flags:"A",
        description:" Sets the CSS class for this widget\n",
        name:"setStyleName",
        seeAlso:[],
        params:[
            {description:"new CSS class name (must be defined previously)", optional:false,
             name:"newStyle", defaultValue:null, type:"string"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('styles')}"],
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "attr:DSRequest.endRow":{
        ref:"attr:DSRequest.endRow",
        flags:"IR",
        description:"End row of requested results, used only with fetch operations.\n <p>\n Note that startRow and endRow are zero-based, so startRow: 0, endRow: 1 is a request\n for the first two records.",
        name:"endRow",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest",
        groups:["paging"]
    },
    "method:ViewLoader.handleError":{
        ref:"method:ViewLoader.handleError",
        flags:"",
        description:"\n This method is called when a transport error occurs.  Typically, this is the result of the\n server returning an HTTP error code such as 404 - document not found.  You can inspect the\n RPCResponse object for the reasons for the error and take appropriate action.  Typical\n properties to look at are rpcResponse.status, and rpcResponse.httpResponseCode.\n <p>\n This method is called from the response processing pipeline.  If you want to provide your\n own HTML response as the result of the error, you can do\n so by setting rpcResponse.data to your HTML string.  Returning false from this method\n suppresses any further response handling.  The default implementation of this method causes\n an error message to be logged to the Developer Console and sets the HTML to the error\n string.\n\n",
        name:"handleError",
        seeAlso:[],
        params:[
            {description:"The RPCRequest that was made to the server", optional:false,
             name:"rpcRequest", defaultValue:null, type:"RPCRequest"},
            {description:"The RPCResponse that was received", optional:false, name:"rpcResponse",
             defaultValue:null, type:"RPCResponse"}
        ],
        returns:{description:"false to suppress further response processing", type:"boolean"},
        type:"method",
        definingClass:"class:ViewLoader",
        groups:["contentLoading"]
    },
    "attr:DataSource.useParentFieldOrder":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.useParentFieldOrder",
        description:"For a DataSource that inherits ${isc.DocUtils.linkForRef('attr:DataSource.fields')} from another DataSource \n (via ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}), indicates that the parent's field order should be used\n instead of the order of the fields as declared in this DataSource.  New fields, if any,\n are placed at the end.",
        name:"useParentFieldOrder",
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('schemaChaining')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["fields"],
        definingClass:"class:DataSource"
    },
    "method:ValuesManager.setValues":{
        ref:"method:ValuesManager.setValues",
        flags:"",
        description:" Replaces the current values of the ValuesManager and all member forms with the values\n passed in.\n <P>\n Values should be provided as an Object containing the new values as properties, where\n each propertyName is the name of a ${isc.DocUtils.linkForRef('group:Items','form item')} in one of the member forms,\n and each property value is the value to apply to that form item via\n ${isc.DocUtils.linkForRef('method:FormItem.setValue')}.\n <P>\n Values with no corresponding form item may also be passed, will be tracked by the\n valuesManager and returned by subsequent calls to ${isc.DocUtils.linkForRef('method:ValuesManager.getValues')}.\n <P>\n Any ${isc.DocUtils.linkForRef('class:FormItem')} for which a value is not provided will revert to its\n ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue','defaultValue')}.  To cause all FormItems to revert to default\n values, pass null.\n <P>\n This method also calls ${isc.DocUtils.linkForRef('method:ValuesManager.rememberValues')} so that a subsequent later call to\n ${isc.DocUtils.linkForRef('method:ValuesManager.resetValues')} will revert to the passed values.\n \n",
        name:"setValues",
        seeAlso:[],
        params:[
            {description:"new set of values for this values manager.", optional:false, name:"values",
             defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "attr:HibernateBrowser.dataGrid":{
        ref:"attr:HibernateBrowser.dataGrid",
        flags:"IR",
        description:"Instance of ListGrid used to display the actual data in the database table associated with\n the mapping selected in the ${isc.DocUtils.linkForRef('attr:HibernateBrowser.mappingTree','mappingTree')}.",
        name:"dataGrid",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "method:DataBoundComponent.dragComplete":{
        ref:"method:DataBoundComponent.dragComplete",
        flags:"",
        description:" This method is invoked on the source component whenever a drag operation or \n ${isc.DocUtils.linkForRef('method:DataBoundComponent.transferSelectedData')} completes.  This method is called when the entire chain of \n operations - including, for databound components, server-side updates and subsequent \n integration of the changes into the client-side cache - has completed.<p>\n There is no default implementation of this method; you are intended to override it if you \n are interested in being notified when drag operations complete.\n\n",
        name:"dragComplete",
        seeAlso:[
            {ref:"method:DataBoundComponent.dropComplete", description:null}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["dragging"]
    },
    "classMethod:FileLoader.cacheModules":{
        ref:"classMethod:FileLoader.cacheModules",
        flags:"",
        description:"\n Caches the specified SmartClient modules.\n\n",
        name:"cacheModules",
        seeAlso:[],
        params:[
            {description:"List of SmartClient modules to cache.  This can either be a                                  string with comma separated module names or an Array of module\n                                  names.",
             optional:false, name:"modules", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified                                      modules has been cached.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "classAttr:Operators.equalsTitle":{
        ref:"classAttr:Operators.equalsTitle",
        flags:"IR",
        description:"Title for the \"equals\" operator",
        name:"equalsTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"equals\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:MenuItem.checked":{
        ref:"attr:MenuItem.checked",
        flags:"IR",
        description:"If true, this item displays a standard checkmark image to the left of its title.  You\n can set the checkmark image URL by setting ${isc.DocUtils.linkForRef('attr:Menu.checkmarkImage')}.\n <p>\n If you need to set this state dynamically, use ${isc.DocUtils.linkForRef('method:MenuItem.checkIf')} instead.",
        name:"checked",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"],
        definingClass:"object:MenuItem",
        groups:["menuIcons"]
    },
    "classAttr:RPCResponse.STATUS_UNKNOWN_HOST_ERROR":{
        ref:"classAttr:RPCResponse.STATUS_UNKNOWN_HOST_ERROR",
        flags:"R",
        description:"This response code only occurs when using the HTTP proxy.  It is issued by the proxy \n servlet when the target host is unknown (ie, cannot be resolved through DNS).  This\n response probably indicates that you are attempting to contact a nonexistent server \n (though it might mean that you have DNS problems).\n <p>\n Note that this status will be reported alongside an httpResponseCode of 591 - this is \n an artificial httpResponseCode used by the proxy to signal this particular condition;\n it is not a valid HTTP response code sent by a webserver.",
        name:"STATUS_UNKNOWN_HOST_ERROR",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"-91",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:StretchImgButton.iconSpacing":{
        ref:"attr:StretchImgButton.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "group:minimize":{
        ref:"group:minimize",
        refs:["attr:Window.minimizeHeight"],
        name:"minimize",
        seeAlso:[],
        type:"group"
    },
    "attr:Chart.subTitle":{
        ref:"attr:Chart.subTitle",
        flags:"IR",
        description:"Subtitle for the chart as a whole, typically used to describe fixed facet values (such as\n \"for Q1, 1999\").\n <P>\n The subTitle should appear under the title in less emphasized text.",
        name:"subTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:ListGrid.confirmDiscardEdits":{
        ref:"attr:ListGrid.confirmDiscardEdits",
        flags:"IRW",
        description:"For editable listGrids, outstanding unsaved edits when the user performs a new filter\n or sort will be discarded. This flag determines whether we should display a confirmation\n dialog with options to save or discard the edits, or cancel the action in this case.",
        name:"confirmDiscardEdits",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ListGrid.styleName":{
        ref:"attr:ListGrid.styleName",
        flags:"IRW",
        description:"Default CSS class",
        name:"styleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"listGrid\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:Criterion.value":{
        ref:"attr:Criterion.value",
        flags:"IR",
        description:"Value to be used in the application of this criterion.\n <P>\n Value may be required or not required, or may be an Array, according to the\n ${isc.DocUtils.linkForRef('type:OperatorValueType')} of the operator.",
        name:"value",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "method:ListGrid.validateRow":{
        ref:"method:ListGrid.validateRow",
        flags:"",
        description:" Validate the current set of edit values for the row in question.\n <P>\n Called when the user moves to a new edit row, or when an edited record is to be saved if\n client side validation is enabled for this grid.\n <P>\n This method may also be called directly to perform row level validation at any time.\n\n",
        name:"validateRow",
        seeAlso:[],
        params:[
            {description:"index of row to be validated.", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"returns true if validation was successful (no errors encountered), false                    otherwise.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:ListGrid.showAllRecords":{
        ref:"attr:ListGrid.showAllRecords",
        flags:"IRW",
        description:"Whether all rows should be drawn all at once, or only rows visible in the viewport.<br><br>\n\n Drawing all rows causes longer initial rendering time, but allows smoother vertical scrolling.\n With a very large number of rows, showAllRows will become too slow.",
        name:"showAllRecords",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('autofitRows')}"],
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "classAttr:GroupingMessages.upcomingThisWeekTitle":{
        ref:"classAttr:GroupingMessages.upcomingThisWeekTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs this week,\n relative to the current date.",
        name:"upcomingThisWeekTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"This Week\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:RichTextItem.showTitle":{
        ref:"attr:RichTextItem.showTitle",
        flags:"IR",
        description:"Don't show the title for rich text items by default",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RichTextItem"
    },
    "attr:ListGridField.filterOnKeypress":{
        ref:"attr:ListGridField.filterOnKeypress",
        flags:"IRWA",
        description:"If we're showing the filterEditor (listGrid.showFilterEditor is true), this property \n determines whether this list should be filtered every time the user edits the value of\n the filter editor for this field.",
        name:"filterOnKeypress",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "method:Canvas.setRight":{
        ref:"method:Canvas.setRight",
        flags:"",
        description:" Resizes the widget horizontally to position its right side at the specified coordinate.\n <P>\n NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()\n instead\n",
        name:"setRight",
        seeAlso:[],
        params:[
            {description:"new right coordinate", optional:false, name:"right", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:ListGridField.canHide":{
        ref:"attr:ListGridField.canHide",
        flags:"IR",
        description:"Whether this field can be hidden via the header context menu.",
        name:"canHide",
        seeAlso:[
            {ref:"method:ListGrid.getHeaderContextMenuItems", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:ListGrid.getEditorType":{
        ref:"method:ListGrid.getEditorType",
        flags:"A",
        description:"\n  Returns the form item type (Class Name) to display for a field when it is displayed in the \n  editor while editing some record.<br>\n  Default implementation will return field.editorType if specified.\n  If not specified, the default form item for the appropriate data type will be displayed\n  - can be overriden to provide a different specific form item type for some field based on \n  the record/field data.\n\n",
        name:"getEditorType",
        seeAlso:[],
        params:[
            {description:"field definition field for which we need a valueMap", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"current edit values for the record (may be null, if editing a                               new record)",
             optional:false, name:"values", defaultValue:null, type:"object"}
        ],
        returns:{description:"form item type for the edit field", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ToolStrip.styleName":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ToolStrip.styleName",
        description:"The CSS class applied to this widget as a whole.",
        name:"styleName",
        setter:["canvas.setStyleName"],
        valueType:"CSSClassName",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('styles')}"],
        defaultValue:"\"toolStrip\"",
        groups:["appearance"],
        definingClass:"class:ToolStrip"
    },
    "object:SortSpecifier":{
        methods:[],
        ref:"object:SortSpecifier",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:SortSpecifier.property", "attr:SortSpecifier.direction", 
         "attr:SortSpecifier.normalizer","attr:SortSpecifier.context"],
        description:"A Javascript object defining the details of a single sort operation.\n <P>\n You can convert between SortSpecifiers and the string format required by \n ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')} by calling ${isc.DocUtils.linkForRef('classMethod:DataSource.getSortBy')} and \n ${isc.DocUtils.linkForRef('classMethod:DataSource.getSortSpecifiers')}.",
        name:"SortSpecifier",
        seeAlso:[],
        type:"object"
    },
    "attr:DateChooser.prevYearIconWidth":{
        ref:"attr:DateChooser.prevYearIconWidth",
        flags:"IR",
        description:"Width of the icon for the previous year button",
        name:"prevYearIconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"14",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:Class.echoLeaf":{
        ref:"method:Class.echoLeaf",
        flags:"",
        description:" Return a very short (generally less than 40 characters) string representation of any object,\n suitable for viewing by a developer for debugging purposes.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echoLeaf\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoLeaf",
        seeAlso:[
            {ref:"method:Class.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"method",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:ListGrid.getAllFields":{
        ref:"method:ListGrid.getAllFields",
        flags:"",
        description:" Get the complete array of fields for this ListGrid, including fields that are not currently\n visible or were specified implicitly via ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')}.\n <P>\n This list of fields is only valid once the ListGrid has been ${isc.DocUtils.linkForRef('method:Canvas.draw','drawn')} or once\n ${isc.DocUtils.linkForRef('method:ListGrid.setFields')} has been called explicitly.  If called earlier, only the list of directly\n specified fields will be returned (the Array passed to create()).\n <P>\n This Array should be treated as <b>read-only</b>.  To modify the set of visible fields, use\n ${isc.DocUtils.linkForRef('method:ListGrid.showField')}, ${isc.DocUtils.linkForRef('method:ListGrid.hideField')} and related APIs.  To update properties of individual\n fields, use ${isc.DocUtils.linkForRef('method:ListGrid.setFieldProperties')} or more specific APIs such as\n ${isc.DocUtils.linkForRef('method:ListGrid.setFieldTitle')}.\n\n",
        name:"getAllFields",
        seeAlso:[],
        returns:{description:"Array of all fields in the ListGrid", type:"Array of ListGridField"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:ListGrid.validateCell":{
        ref:"method:ListGrid.validateCell",
        flags:"",
        description:" Validate the current edit value for the cell in question.  Called when the user moves to a\n new edit cell if ${isc.DocUtils.linkForRef('attr:ListGrid.validateByCell')} is true.<br>\n This method may also be called directly to perform cell level validation at any time.\n",
        name:"validateCell",
        seeAlso:[],
        params:[
            {description:"index of row to be validated.", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"field name (or column index) of field to be validated", optional:false,
             name:"fieldName", defaultValue:null, type:"string | number"}
        ],
        returns:{description:"returns true if validation was successful (no errors encountered), false                    otherwise.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "method:CubeGrid.getCellRecord":{
        ref:"method:CubeGrid.getCellRecord",
        flags:"A",
        description:" Return the pointer to a particular record by record and column number.<br>\n",
        name:"getCellRecord",
        seeAlso:[
            {ref:"method:ListGrid.getRecord", description:null},
            {ref:"method:ListGrid.getEditedRecord", description:null}
        ],
        params:[
            {description:"row index of record to return.", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of record to return.", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Record object for the row.", type:"ListGridRecord"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "class:GridRenderer":{
        methods:["method:GridRenderer.getTotalRows", "method:GridRenderer.setColumnWidth", 
         "method:GridRenderer.setColumnWidths","method:GridRenderer.getCellStyle", "method:GridRenderer.cellIsEnabled", 
         "method:GridRenderer.refreshCellStyle","method:GridRenderer.refreshCell", "method:GridRenderer.refreshRow", 
         "method:GridRenderer.getEventRow","method:GridRenderer.getEventColumn", "method:GridRenderer.getNearestRowToEvent", 
         "method:GridRenderer.getNearestColToEvent","method:GridRenderer.getVisibleRows", "method:GridRenderer.getDrawnRows", 
         "method:GridRenderer.getCellRecord","method:GridRenderer.getCellValue", "method:GridRenderer.findRowNum", 
         "method:GridRenderer.findColNum","method:GridRenderer.getBaseStyle", "method:GridRenderer.getCellCSSText", 
         "method:GridRenderer.getRowHeight","method:GridRenderer.getRowSpan", "method:GridRenderer.cellOut", 
         "method:GridRenderer.cellOver","method:GridRenderer.rowOut", "method:GridRenderer.rowOver", 
         "method:GridRenderer.cellContextClick","method:GridRenderer.rowContextClick", "method:GridRenderer.cellMouseDown", 
         "method:GridRenderer.rowMouseDown","method:GridRenderer.cellMouseUp", "method:GridRenderer.rowMouseUp", 
         "method:GridRenderer.cellClick","method:GridRenderer.cellDoubleClick", "method:GridRenderer.rowClick", 
         "method:GridRenderer.rowDoubleClick","method:GridRenderer.cellHover", "method:GridRenderer.rowHover", 
         "method:GridRenderer.cellHoverHTML","method:GridRenderer.selectionChanged", "method:GridRenderer.cellSelectionChanged"],
        ref:"class:GridRenderer",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:GridRenderer.totalRows", "attr:GridRenderer.showAllRows", 
         "attr:GridRenderer.showAllColumns","attr:GridRenderer.drawAllMaxCells", "attr:GridRenderer.scrollRedrawDelay", 
         "attr:GridRenderer.drawAheadRatio","attr:GridRenderer.quickDrawAheadRatio", "attr:GridRenderer.cellHeight", 
         "attr:GridRenderer.fixedRowHeights","attr:GridRenderer.fixedColumnWidths", "attr:GridRenderer.autoFit", 
         "attr:GridRenderer.wrapCells","attr:GridRenderer.cellPadding", "attr:GridRenderer.canSelectOnRightMouse", 
         "attr:GridRenderer.canHover","attr:GridRenderer.showHover", "attr:GridRenderer.baseStyle", 
         "attr:GridRenderer.alternateRowStyles","attr:GridRenderer.alternateRowFrequency", "attr:GridRenderer.emptyCellValue", 
         "attr:GridRenderer.showEmptyMessage","attr:GridRenderer.emptyMessage", "attr:GridRenderer.emptyMessageTableStyle", 
         "attr:GridRenderer.emptyMessageStyle","attr:GridRenderer.fastCellUpdates", "attr:GridRenderer.snapToCells", 
         "attr:GridRenderer.snapInsideBorder","attr:GridRenderer.recordCustomStyleProperty"],
        description:"A flexible, high-speed table that offers consistent cross-platform sizing, clipping, and events.",
        name:"GridRenderer",
        seeAlso:[],
        type:"class"
    },
    "method:ListGridField.getRecordSummary":{
        ref:"method:ListGridField.getRecordSummary",
        flags:"A",
        description:" Only applies to ${isc.DocUtils.linkForRef('type:ListGridFieldType','summary type')} fields. If specified , this\n method will be called to generated the record summary value to be displayed for each row\n in this field.\n <P>\n Note that if specified, this is called instead of making use of the\n ${isc.DocUtils.linkForRef('attr:ListGridField.recordSummaryFunction')}.\n <P>\n If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} or ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, this\n field's value in the summary row[s] will still be calculated by calling this method.\n In this case, the record object passed in will contain summary values for each field.\n If custom handling is required for this case, it may be detected by checking the\n record object's ${isc.DocUtils.linkForRef('attr:ListGridRecord.isGroupSummary')} and ${isc.DocUtils.linkForRef('attr:ListGridRecord.isGridSummary')}\n attributes.\n",
        name:"getRecordSummary",
        seeAlso:[],
        params:[
            {description:"record for which a summary is being generated", optional:false,
             name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"pointer to the summary type field", optional:false, name:"field",
             defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"summary value to display", type:"any"},
        type:"method",
        definingClass:"object:ListGridField"
    },
    "classMethod:isc.echoLeaf":{
        ref:"classMethod:isc.echoLeaf",
        flags:"",
        description:" Same as ${isc.DocUtils.linkForRef('classMethod:Log.echoLeaf')}.\n\n",
        name:"echoLeaf",
        seeAlso:[],
        params:[
            {description:"object to echo", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:TileLayout.layoutPolicy":{
        ref:"attr:TileLayout.layoutPolicy",
        flags:"IR",
        description:"Policy for laying out tiles.  See ${isc.DocUtils.linkForRef('type:TileLayoutPolicy')} for options.",
        name:"layoutPolicy",
        seeAlso:[],
        valueType:"TileLayoutPolicy",
        defaultValue:"\"fit\"",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutPolicy"]
    },
    "method:ColorItem.pickerColorSelected":{
        ref:"method:ColorItem.pickerColorSelected",
        flags:"",
        description:"  Store the color value selected by the user from the color picker.  You will need to \n  override this method if you wish to capture opacity information from the ${isc.DocUtils.linkForRef('class:ColorPicker')}\n",
        name:"pickerColorSelected",
        seeAlso:[],
        params:[
            {description:"the selected color as a string", optional:false, name:"color",
             defaultValue:null, type:"color"},
            {description:"the selected opacity, from 0 (transparent) to 100 (opaque).                            Only applicable if ${isc.DocUtils.linkForRef('attr:ColorItem.supportsTransparency')} is true.",
             optional:false, name:"opacity", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColorItem"
    },
    "classMethod:isc.echoAll":{
        ref:"classMethod:isc.echoAll",
        flags:"",
        description:" Same as ${isc.DocUtils.linkForRef('classMethod:Log.echoAll')}.\n\n",
        name:"echoAll",
        seeAlso:[],
        params:[
            {description:"object to echo", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:RichTextEditor.formatControls":{
        ref:"attr:RichTextEditor.formatControls",
        flags:"IRA",
        description:"Default text formatting control group. Consists of an array of \n  ${isc.DocUtils.linkForRef('type:ControlName')}s. To display this group of controls for some RichTextEditor,\n  include <code>\"formatControls\"</code> in the ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} array",
        name:"formatControls",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[\"alignLeft\", \"alignRight\", \"alignCenter\", \"justify\"]",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "group:contents":{
        ref:"group:contents",
        refs:["attr:HTMLFlow.contents", "attr:HTMLFlow.dynamicContents", "attr:Canvas.contents", 
         "attr:Canvas.dynamicContents","attr:Label.contents", "attr:Label.dynamicContents"],
        name:"contents",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.escapeKeyEditAction":{
        ref:"attr:ListGrid.escapeKeyEditAction",
        flags:"IRW",
        description:"What to do when a user hits escape while editing a cell:<ul>\n <li>\"cancel\": close the editor and discard the current set of edit values\n <li>\"done\": just close the editor (the edit is complete, but the edited values are retained).\n </ul>\n Note that if ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')} is true, this may cause a save of the current edit values",
        name:"escapeKeyEditAction",
        seeAlso:[],
        valueType:"EscapeKeyEditAction",
        defaultValue:"\"cancel\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "classAttr:ValidatorTypes.isInteger":{
        ref:"classAttr:ValidatorTypes.isInteger",
        flags:"IR",
        description:"Tests whether the value for this field is a whole number.  If \n  <code>validator.convertToInteger</code> is true, float values will be converted \n  into integers and validation will succeed.",
        name:"isInteger",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:StretchImg.src":{
        ref:"attr:StretchImg.src",
        flags:"IRW",
        description:"The base URL for the image. \n <P>\n As with ${isc.DocUtils.linkForRef('attr:Img.src')}, the ${isc.DocUtils.linkForRef('type:State')} of the component is added to this URL.  Then,\n the image segment name as specified by ${isc.DocUtils.linkForRef('attr:StretchImg.items')} is added.\n <P>\n For example, for a stretchImg in \"Over\" state with a <code>src</code> of \"button.png\"\n and a segment name of \"stretch\", the resulting URL would be \"button_Over_stretch.png\".",
        name:"src",
        setter:["setSrc"],
        seeAlso:[
            {ref:"attr:StretchImg.hSrc", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "attr:TreeNode.icon":{
        ref:"attr:TreeNode.icon",
        flags:"IRW",
        description:"This Property allows the developer to customize the icon displayed next to a node.\n Set <code>node.icon</code> to the URL of the desired icon to display and\n it will be shown instead of the standard ${isc.DocUtils.linkForRef('attr:TreeGrid.nodeIcon')} for this node.<br>\n Note that if ${isc.DocUtils.linkForRef('attr:TreeNode.showOpenIcon')} and/or ${isc.DocUtils.linkForRef('attr:TreeNode.showDropIcon')} \n is true for this node, customized icons for folder nodes will be appended with the \n ${isc.DocUtils.linkForRef('attr:TreeGrid.openIconSuffix')} or ${isc.DocUtils.linkForRef('attr:TreeGrid.dropIconSuffix')} suffixes on state change \n as with the standard ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} for this treeGrid.  Also note that for\n custom folder icons, the ${isc.DocUtils.linkForRef('attr:TreeGrid.closedIconSuffix')} will never be appended.\n <P>You can change the name of this property by setting \n ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconProperty')}.",
        name:"icon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode",
        groups:["treeIcons"]
    },
    "method:Label.setIcon":{
        ref:"method:Label.setIcon",
        flags:"",
        description:" Change the icon being shown next to the title text.\n",
        name:"setIcon",
        seeAlso:[],
        params:[
            {description:"URL of new icon", optional:false, name:"icon", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "method:ValuesManager.setMemberValues":{
        ref:"method:ValuesManager.setMemberValues",
        flags:"",
        description:" Set the values for some member form.\n",
        name:"setMemberValues",
        seeAlso:[],
        params:[
            {description:"ID of the member form to update", optional:false, name:"ID",
             defaultValue:null, type:"string"},
            {description:"new values for the form", optional:false, name:"values", defaultValue:null,
             type:"object"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "classMethod:Class.logDebug":{
        ref:"classMethod:Class.logDebug",
        flags:"",
        description:" Log a message at \"debug\" priority\n <P>\n A method named log<i>Priority</i> exists for each priority level, on every ISC Class and\n instance of an ISC Class.  Messages logged on a Class or instance have a default\n category of the classname.  Messages logged on an instance will also automatically\n incorporate the instance ID.  General best practice is to call logDebug() et al as\n \"this.logDebug\" whenever \"this\" is an instance, or as \"Log.logDebug\" otherwise.\n\n",
        name:"logDebug",
        seeAlso:[
            {ref:"classMethod:Log.echo", description:"for dumping datastructures to the log"},
            {ref:"classMethod:Log.setPriority",
             description:"for controlling what messages appear in the log"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:HeaderItem.showTitle":{
        ref:"attr:HeaderItem.showTitle",
        flags:"IRW",
        description:"Don't show a separate title cell for headers",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "attr:DataSource.tableName":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:DataSource.tableName",
        description:"For DataSources using the ${isc.DocUtils.linkForRef('group:sqlDataSource','SmartClient SQL engine')} for\n persistence, what database table name to use.  The default is to use the DataSource ID\n as the table name.",
        name:"tableName",
        valueType:"String",
        seeAlso:[],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "attr:Canvas.canSelectText":{
        ref:"attr:Canvas.canSelectText",
        flags:"IRWA",
        description:"Whether native drag selection of contained text is allowed within this Canvas.\n <P>\n Note that setting this property to <code>false</code> will not avoid text selection\n which is initiated outside this Canvas from continuing into this Canvas, even if text\n selection began in another Canvas.",
        name:"canSelectText",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["events"]
    },
    "attr:Canvas.disabled":{
        flags:"IRWA",
        getter:["isDisabled"],
        type:"attr",
        ref:"attr:Canvas.disabled",
        description:"If set to true, the widget will be disabled. A widget is only considered enabled \n      if it is individually enabled and all parents above it in the containment hierarchy \n      are enabled. This allows you to enable or disable all components of a complex \n      nested widget by enabling or disabling the top-level parent only.",
        name:"disabled",
        setter:["setDisabled"],
        valueType:"boolean",
        seeAlso:[],
        defaultValue:"false",
        groups:["enable"],
        definingClass:"class:Canvas"
    },
    "method:GridRenderer.cellHover":{
        ref:"method:GridRenderer.cellHover",
        flags:"A",
        description:" Called when the mouse hovers over a cell if this.canHover is true. \n  Returning false will suppress the hover text from being shown if this.showHover is true.\n\n",
        name:"cellHover",
        seeAlso:[
            {ref:"attr:GridRenderer.canHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "classMethod:Time.setShortDisplayFormat":{
        ref:"classMethod:Time.setShortDisplayFormat",
        flags:"",
        description:" Sets the default format for strings returned by ${isc.DocUtils.linkForRef('classMethod:Time.toShortTime')}.\n",
        name:"setShortDisplayFormat",
        seeAlso:[],
        params:[
            {description:"Optional custom formatter to use. Will accept  a function (which will be passed a pointer to the date to perform the conversion), or\n  a string designating a standard formatter",
             optional:false, name:"formatter", defaultValue:null, type:"timeFormatter | function"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "method:Selection.getSelection":{
        ref:"method:Selection.getSelection",
        flags:"",
        description:"			Return an ordered array of all of the selected items\n",
        name:"getSelection",
        seeAlso:[],
        returns:{description:"list of selected items", type:"array"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:ListGrid.setHeaderHeight":{
        ref:"method:ListGrid.setHeaderHeight",
        flags:"",
        description:" Modify the height of a listGrid. To hide the header set height to zero.\n",
        name:"setHeaderHeight",
        seeAlso:[],
        params:[
            {description:"new height for the header", optional:false, name:"height", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["sizing", "gridHeader"]
    },
    "attr:Canvas.canDragResize":{
        ref:"attr:Canvas.canDragResize",
        flags:"IRWA",
        description:"Indicates whether this widget can be resized by dragging on the edges and/or corners of\n      the widget with the mouse.",
        name:"canDragResize",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragResize')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:CalendarEvent.name":{
        ref:"attr:CalendarEvent.name",
        flags:"IRW",
        description:"String which represents the name of a ${isc.DocUtils.linkForRef('object:CalendarEvent')}\n The name of this field within the CalendarEvent can be changed via \n ${isc.DocUtils.linkForRef('attr:Calendar.nameField')}",
        name:"name",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "attr:DetailViewerField.imageURLPrefix":{
        ref:"attr:DetailViewerField.imageURLPrefix",
        flags:"IRWA",
        description:"If this field has type set to <code>\"image\"</code>\n and the URL for the image displayed is not absolute, the path of the URL will be relative\n to this string<br>",
        name:"imageURLPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField",
        groups:["imageColumns"]
    },
    "classAttr:Operators.lessThanTitle":{
        ref:"classAttr:Operators.lessThanTitle",
        flags:"IR",
        description:"Title for the \"lessThan\" operator",
        name:"lessThanTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"less than\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "method:ResultSet.getValueMap":{
        ref:"method:ResultSet.getValueMap",
        flags:"",
        description:" Get a map of the form <code>{ item[idField] -&gt; item[displayField] }</code>, for all \n items in the list.  If more than one item has the same <code>idProperty</code>, \n the value for the later item in the list will clobber the value for the earlier item.\n <P>\n If this method is called when the ${isc.DocUtils.linkForRef('method:ResultSet.allMatchingRowsCached','cache is incomplete')}, it\n will trigger fetches, and will return a valueMap reflecting only the currently loaded rows.\n\n",
        name:"getValueMap",
        seeAlso:[
            {ref:"method:ResultSet.allMatchingRowsCached", description:null}
        ],
        params:[
            {description:"Property to use as ID (data value) in the valueMap", optional:false,
             name:"idField", defaultValue:null, type:"string"},
            {description:"Property to use as a display value in the valueMap", optional:false,
             name:"displayField", defaultValue:null, type:"string"}
        ],
        returns:{description:"valueMap object", type:"object"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "class:WSDataSource":{
        methods:[],
        ref:"class:WSDataSource",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:[],
        description:"A WSDataSource is a DataSource that is preconfigured to contact the WSDL web service built\n into the SDK (see isomorphic/system/schema/SmartClientOperations.wsdl).  This WSDL service\n can be easily implemented on Java and non-Java backends.  \n <P>\n WSDataSource supports all 4 DataSource operations (fetch, add, update, remove) and can be\n used with ListGrids, DynamicForms and other ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}s just like other\n DataSources.\n <P>\n Note that WSDataSource is specifically designed for use with SmartClientOperations.wsdl.  If\n you are trying to connect to a pre-existing WSDL service, start with just ${isc.DocUtils.linkForRef('class:DataSource')}, \n not WSDataSource, and see the ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL Integration')} chapter for an\n overview.",
        name:"WSDataSource",
        seeAlso:[],
        type:"class"
    },
    "method:Canvas.intersectsRect":{
        ref:"method:Canvas.intersectsRect",
        flags:"",
        description:"			Returns true if the rectangle of this widget intersects with the rectangle coordinates\n          passed in, and false otherwise.\n",
        name:"intersectsRect",
        seeAlso:[],
        params:[
            {description:"left coord of rect (or rect array)", optional:false, name:"left",
             defaultValue:null, type:"number, array"},
            {description:"top coord of rect", optional:false, name:"top", defaultValue:null,
             type:"number"},
            {description:"width of rect", optional:false, name:"width", defaultValue:null, type:"number"},
            {description:"height of rect", optional:false, name:"height", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"true if this canvas intersects the rectangle passed in; false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:FormItem.titleClassName":{
        ref:"attr:FormItem.titleClassName",
        deprecated:"As of SmartClient Version 5.5, use ${isc.DocUtils.linkForRef('attr:FormItem.titleStyle')} instead",
        flags:"IRW",
        description:"CSS class for the form item's title.",
        name:"titleClassName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"formTitle\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "attr:FormItem.hoverWidth":{
        ref:"attr:FormItem.hoverWidth",
        flags:"IRW",
        description:"Option to specify a width for any hover shown for this item.",
        name:"hoverWidth",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverWidth", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:Window.showStatusBar":{
        ref:"attr:Window.showStatusBar",
        flags:"IRW",
        description:"If true, show a statusBar for this Window, including resizer.",
        name:"showStatusBar",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "footer"]
    },
    "attr:Window.minimizeHeight":{
        ref:"attr:Window.minimizeHeight",
        flags:"IRWA",
        description:"Height for the window when minimized.\n If unset the window will shrink to the height of the header, if present, otherwise\n ${isc.DocUtils.linkForRef('attr:Window.defaultMinimizeHeight','this.defaultMinimizeHeight')}",
        name:"minimizeHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "minimize"]
    },
    "attr:ImgTab.showRollOver":{
        ref:"attr:ImgTab.showRollOver",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes over this tab",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "method:Date.getShortDayName":{
        ref:"method:Date.getShortDayName",
        flags:"",
        description:" Return the abbreviated (up to 3 chars) day of week name for this date (Mon, Tue, etc).\n To modify the value returned by this method, set ${isc.DocUtils.linkForRef('classAttr:Date.shortDayNames')}\n\n",
        name:"getShortDayName",
        seeAlso:[],
        params:[
            {description:"Number of characters to return (Defaults to 3, can't be                                   longer than 3)",
             optional:false, name:"length", defaultValue:null, type:"number"}
        ],
        returns:{description:"Abbreviated day name", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:Tree.titleProperty":{
        ref:"attr:Tree.titleProperty",
        flags:"IRW",
        description:"Name of the property on a ${isc.DocUtils.linkForRef('object:TreeNode')} that holds the title of the node as it should be\n shown to the user.  Default value is \"title\".  See ${isc.DocUtils.linkForRef('attr:TreeNode.title')} for usage.",
        name:"titleProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"title\"",
        type:"attr",
        definingClass:"class:Tree"
    },
    "group:tabBarScrolling":{
        ref:"group:tabBarScrolling",
        refs:["attr:TabSet.symmetricScroller", "attr:TabSet.scrollerSrc", "attr:TabSet.scrollerHSrc", 
         "attr:TabSet.scrollerVSrc","attr:TabSet.symmetricPickerButton", "attr:TabSet.pickerButtonSrc", 
         "attr:TabSet.pickerButtonHSrc","attr:TabSet.pickerButtonVSrc"],
        name:"tabBarScrolling",
        seeAlso:[],
        type:"group"
    },
    "attr:Window.defaultMinimizeHeight":{
        ref:"attr:Window.defaultMinimizeHeight",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:Window.minimizeHeight')} is unset, by the window will shrink to the height of the\n header when minimized.\n <BR>\n If there is no header, the <code>defaultMinimizeHeight</code> will be used instead.",
        name:"defaultMinimizeHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "classMethod:DMI.call":{
        ref:"classMethod:DMI.call",
        flags:"",
        description:"\n Calls a server-side DMI method.  At a minimum, you need to specify the appID (.app.xml\n file), ${isc.DocUtils.linkForRef('attr:ServerObject.className')} or ${isc.DocUtils.linkForRef('attr:ServerObject.ID')} and methodName to call.\n Arguments and callback are optional.  There are two ways to invoke this method:\n <pre>\n DMI.call(appID, className, methodName, \n          arg1, arg2 ...argN, callback);\n </pre>\n or:\n <pre>\n DMI.call({\n     appID: appID,\n     className: className,\n     methodName: methodName,\n     arguments: [arg1, arg2, ...argN], //optional\n     callback: callback, //optional\n     requestParams: requestProps // optional\n });\n </pre>\n If you use the first signature, you must either specify a callback or if you don't want a\n callback, pass a <code>null</code> as the last argument.  The second signature allows you to\n specify requestParams that are applied to the ${isc.DocUtils.linkForRef('class:RPCRequest')} generated by this DMI call.\n This allows you to override some defaults - for example to suppress the \"Contacting Server\"\n prompt, change it's text; change the timeout or set any other property settable on\n ${isc.DocUtils.linkForRef('class:RPCRequest')}.\n <p>\n Note that you can use the ${isc.DocUtils.linkForRef('group:loadDMIStubsTag')} tag to bind all methods of\n <code>ServerObjects</code> defined in a given .app.xml file and call methods on them\n directly.\n\n",
        name:"call",
        seeAlso:[],
        params:[
            {description:"the appID (.app.xml file to look in) or comprehensive request                          object as documented above.",
             optional:false, name:"appID", defaultValue:null, type:"string or Object"},
            {description:"${isc.DocUtils.linkForRef('attr:ServerObject.className')} or ${isc.DocUtils.linkForRef('attr:ServerObject.ID')}",
             optional:false, name:"className", defaultValue:null, type:"string"},
            {description:"the name of the method to call on the serverObject", optional:false,
             name:"methodName", defaultValue:null, type:"string"},
            {description:"The next N-1 params specify arguments to the server-side method.",
             optional:true, name:"args", defaultValue:null, type:"any"},
            {description:"The callback of the response.  If you do not want a callback, you                          must specify a <code>null</code> value for this parameter when\n                          using the first signature (documented above).",
             optional:false, name:"callback", defaultValue:null, type:"RPCCallback"}
        ],
        returns:{description:"the RPCRequest that was sent to the server.", type:"RPCRequest"},
        type:"classMethod",
        definingClass:"class:DMI"
    },
    "method:CubeGrid.addRowFacet":{
        ref:"method:CubeGrid.addRowFacet",
        flags:"",
        description:" Add a row facet to the view at index \"index\".  Handles the facet already being in the view\n (does a pivot).<br><br>\n\n The facet being added should currently have a fixed facet value (unless it's already part of the\n view), which will be removed from cubeGrid.fixedFacetValues.\n\n <br><i>methodType</i>  action\n",
        name:"addRowFacet",
        seeAlso:[
            {ref:"method:CubeGrid.removeFacet", description:null},
            {ref:"attr:CubeGrid.fixedFacetValues", description:null}
        ],
        params:[
            {description:"facetId to add.  Definition must have been provided at init time.",
             optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"index to add the facet at.  0 = outermost (default innermost)",
             optional:true, name:"index", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "method:PickList.filterClientPickListData":{
        ref:"method:PickList.filterClientPickListData",
        flags:"",
        description:" Returns the data to display in the pick list.\n <P>\n The default implementation applies the criteria returned by \n ${isc.DocUtils.linkForRef('method:PickList.getPickListFilterCriteria')} to the data returned by\n ${isc.DocUtils.linkForRef('method:PickList.getClientPickListData')}.  A record passes the filter if it has a\n matching value for all fields in the criteria object.  Matching is performed according\n to ${isc.DocUtils.linkForRef('type:TextMatchStyle')}.\n <P>\n If ${isc.DocUtils.linkForRef('attr:PickList.showAllOptions')} is set, all values are shown, with matching values\n shown below a ${isc.DocUtils.linkForRef('attr:PickList.separatorRows','separator')}.\n\n",
        name:"filterClientPickListData",
        seeAlso:[],
        returns:{description:"array of record objects to display in the pickList",
         type:"Array of ListGridRecord"},
        type:"method",
        definingClass:"interface:PickList"
    },
    "attr:Mail.subject":{
        ref:"attr:Mail.subject",
        flags:"IR",
        description:"The string to appear in the subject line of the message.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n If you specify a \"Subject:\" line in the body of your email, the subject will be parsed \n and this property will be ignored.",
        name:"subject",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "method:Canvas.getHSnapPosition":{
        ref:"method:Canvas.getHSnapPosition",
        flags:"A",
        description:"\n Override this method to provide a custom snap-to grid.  Note that you do not need to do\n this if your grid is regular (ie, grid points are every x pixels); regular grids should be \n defined using ${isc.DocUtils.linkForRef('attr:Canvas.snapHGap')} and ${isc.DocUtils.linkForRef('attr:Canvas.snapVGap')}.\n You should only override this method if you want to provide support for a grid of \n irregularly-placed points\n\n",
        name:"getHSnapPosition",
        seeAlso:[],
        params:[
            {description:"x-coordinate of the drag event relative to the inside of this widget",
             optional:false, name:"coordinate", defaultValue:null, type:"integer"},
            {description:"\"before\" or \"after\" denoting whether the returned coordinate should   match the left or right edge of the current square. If unset ${isc.DocUtils.linkForRef('attr:Canvas.snapHDirection')} will\n   be used by default",
             optional:true, name:"direction", defaultValue:null, type:"string"}
        ],
        returns:{description:"The horizontal coordinate to snap to", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Layout.hideMember":{
        ref:"method:Layout.hideMember",
        flags:"",
        description:" Hide the specified member, firing the specified callback when the hide is complete.\n <P>\n Members can always be directly hidden via <code>member.hide()</code>, but if\n ${isc.DocUtils.linkForRef('attr:Layout.animateMembers','animation')} is enabled, animation will only occur if hideMember() is\n called to hide the member.\n\n",
        name:"hideMember",
        seeAlso:[],
        params:[
            {description:"Member to hide", optional:false, name:"member", defaultValue:null,
             type:"Canvas"},
            {description:"callback to fire when the member is hidden.", optional:false,
             name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:Facet.cellAlign":{
        ref:"attr:Facet.cellAlign",
        flags:"IR",
        description:"Default alignment of cells (in the body) for this facet",
        name:"cellAlign",
        seeAlso:[
            {ref:"attr:CubeGrid.cellAlign", description:null}
        ],
        valueType:"alignEnum",
        defaultValue:"cubeGrid.cellAlign",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Calendar.autoFetchData":{
        ref:"attr:Calendar.autoFetchData",
        flags:"IR",
        description:"If true, when this component is first drawn, automatically call <code>this.fetchData()</code>.\n Criteria for this fetch may be picked up from ${isc.DocUtils.linkForRef('attr:Calendar.initialCriteria')}, and textMatchStyle may\n be specified via ${isc.DocUtils.linkForRef('attr:Calendar.autoFetchTextMatchStyle')}.\n <P>\n <span style='color:red'>NOTE:</span> if <code>autoFetchData</code> is set, calling\n ${isc.DocUtils.linkForRef('method:Calendar.fetchData')} before draw will cause two requests to be issued, one from the manual\n call to fetchData() and one from the autoFetchData setting.  The second request will use\n only ${isc.DocUtils.linkForRef('attr:Calendar.initialCriteria')} and not any other criteria or settings from the first request.\n Generally, turn off autoFetchData if you are going to manually call fetchData() at any time.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"method:Calendar.fetchData", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fetchOperation')}"],
        definingClass:"class:Calendar",
        groups:["databinding"]
    },
    "attr:Canvas.hoverHeight":{
        ref:"attr:Canvas.hoverHeight",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, this property can be used to customize the\n height of the hover canvas shown.",
        name:"hoverHeight",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:FilterBuilder.clauseStack":{
        ref:"attr:FilterBuilder.clauseStack",
        flags:"IR",
        description:"VStack of all clauses that are part of this FilterBuilder",
        name:"clauseStack",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Canvas.dragStart":{
        ref:"method:Canvas.dragStart",
        flags:"A",
        description:" Executed when dragging first starts. Your widget can use this opportunity to set\n things up for the drag, such as setting the drag tracker. Returning false from this\n event handler will cancel the drag action entirely.\n <p>\n A drag action is considered to be begin when the mouse has moved\n ${isc.DocUtils.linkForRef('attr:Canvas.dragStartDistance')} with the left mouse down.\n\n",
        name:"dragStart",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to cancel drag action.", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragPan')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:ListGrid.getSortFieldCount":{
        ref:"method:ListGrid.getSortFieldCount",
        flags:"A",
        description:"  Returns the number of fields involved in this grid's current sort configuration.\n",
        name:"getSortFieldCount",
        seeAlso:[],
        returns:{description:"the number of fields this grid is currently sorted on.", type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:TileGrid.removeData":{
        ref:"method:TileGrid.removeData",
        flags:"",
        description:" Perform a DataSource \"remove\" operation to remove records from this component's\n DataSource.\n\n",
        name:"removeData",
        seeAlso:[],
        params:[
            {description:"primary key values of record to delete,                                           (or complete record)",
             optional:false, name:"data", defaultValue:null, type:"Object"},
            {description:"method to call on operation completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:ListGrid.headerMenuButtonWidth":{
        ref:"attr:ListGrid.headerMenuButtonWidth",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, this property governs the width of the \n auto-generated <code>headerMenuButton</code>",
        name:"headerMenuButtonWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "attr:ListGrid.headerHeight":{
        ref:"attr:ListGrid.headerHeight",
        flags:"IRW",
        description:"The height of this listGrid's header, in pixels.",
        name:"headerHeight",
        setter:["setHeaderHeight"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"22",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "sizing"]
    },
    "method:Canvas.linkHTML":{
        ref:"method:Canvas.linkHTML",
        flags:"A",
        description:" Generates the HTML for a standard link element\n\n",
        name:"linkHTML",
        seeAlso:[],
        params:[
            {description:"URL for the link to point to", optional:false, name:"href", defaultValue:null,
             type:"string"},
            {description:"Text to for the link (defaults to the href)", optional:true, name:"text",
             defaultValue:null, type:"string"},
            {description:"Target window for the link (defaults to opening in a new, unnamed window)",
             optional:true, name:"target", defaultValue:null, type:"string"},
            {description:"optional ID for the link element to be written out", optional:true,
             name:"ID", defaultValue:null, type:"string"},
            {description:"optional tabIndex for the link", optional:true, name:"tabIndex",
             defaultValue:null, type:"number"},
            {description:"optional accessKey for the link", optional:true, name:"accessKey",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"HTML for the link", type:"string"},
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:FormulaBuilder.fireOnClose":{
        ref:"method:FormulaBuilder.fireOnClose",
        flags:"A",
        description:" Override to execute a callback function when the Formula is Cancelled or Saved.\n \n",
        name:"fireOnClose",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:OperationBinding.useFlatFields":{
        ref:"attr:OperationBinding.useFlatFields",
        flags:"IR",
        description:"Setting <code>useFlatFields</code> on an operationBinding is equivalent to setting\n ${isc.DocUtils.linkForRef('attr:DSRequest.useFlatFields')} on all DataSource requests with the same ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType')}\n as this <code>operationBinding</code>.\n <P>\n Typical usage is to combine operationBinding.useFlatFields with \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useFlatFields','searchForm.useFlatFields')}, with the ${isc.DocUtils.linkForRef('class:SearchForm')}\n bound to the ${isc.DocUtils.linkForRef('method:WebService.getInputDS','input message')} of the web service operation set\n as ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation')}.  This allows gratuitous nesting to be consistently\n bypassed in both the user presentation and in the actual XML messaging.\n <P>\n Note that <code>useFlatFields</code> is not generally recommended for use with input messages\n where multiple simple type fields exist with the same name, however if used in this way,\n the first field to use a given name wins.  \"first\" means the first field encountered in a\n depth first search.  \"wins\" means only the first field will be available in data binding,\n and only the first field will be populated in the generated XML message.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "method:Menu.setItemIcon":{
        ref:"method:Menu.setItemIcon",
        flags:"",
        description:" Sets the icon and disabled icon (if specified) for a particular menu item and redraws\n the menu if necessary. Returns true if the icon changed.\n\n",
        name:"setItemIcon",
        seeAlso:[],
        params:[
            {description:"MenuItem in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem or number"},
            {description:"new icon URL", optional:false, name:"newIcon", defaultValue:null,
             type:"string"},
            {description:"new icon URL for disabled image", optional:true, name:"newDisabledIcon",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true == something changed, redraw is called for", type:"boolean"},
        type:"method",
        definingClass:"class:Menu"
    },
    "classMethod:Log.clear":{
        ref:"classMethod:Log.clear",
        flags:"",
        description:" Clear all currently displayed Log messages\n",
        name:"clear",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:EdgedCanvas.edgeStyleName":{
        ref:"attr:EdgedCanvas.edgeStyleName",
        flags:"IRW",
        description:"Optional property specifying the CSS ClassName to apply to the various parts of this edged canvas\n (top, bottom, corners, sides and center). To\n apply separate styles for each part, use ${isc.DocUtils.linkForRef('attr:EdgedCanvas.addEdgeStyleSuffix')}.",
        name:"edgeStyleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdgeStyles", "imageEdges"]
    },
    "method:ListGridField.changed":{
        ref:"method:ListGridField.changed",
        flags:"",
        description:" If this field is editable, any ${isc.DocUtils.linkForRef('method:FormItem.changed','changed')} handler specified\n on the ListGridField will be passed onto the editors for this field.\n",
        name:"changed",
        seeAlso:[
            {ref:"method:ListGridField.change", description:null},
            {ref:"method:ListGrid.cellChanged", description:null}
        ],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the editor (form item) itself (also available as \"this\").                              Note that in addition to the standard FormItem APIs available\n                              on the editor, it also has:<br>\n                              - a pointer back to the containing listGrid \n                              [<code>item.grid</code>]<br>\n                              - the colNum being edited [<code>item.colNum</code>]<br>\n                              - the rowNum being edited [<code>item.rowNum</code>]",
             optional:false, name:"item", defaultValue:null, type:"FormItem"},
            {description:"The current value (after the change).", optional:false, name:"value",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:ViewLoader.getView":{
        ref:"method:ViewLoader.getView",
        flags:"",
        description:" Retrieve the current view.  May be null if the view has not yet been loaded, or has been\n explicitly set to null.\n\n",
        name:"getView",
        seeAlso:[],
        returns:{description:"the current view", type:"Canvas"},
        type:"method",
        definingClass:"class:ViewLoader",
        groups:["contentLoading"]
    },
    "type:DSProtocol":{
        ref:"type:DSProtocol",
        values:[
            {description:"Data is added to the dataURL, with each property in the data becoming an HTTP parameter,\n   eg http://service.com/search?keyword=foo",
             value:"\"getParams\""},
            {description:"Data is POST'd to the dataURL, with each property becoming an HTTP parameter, \n   exactly as an HTML form would submit them if it had one input field per property in the\n   data.",
             value:"\"postParams\""},
            {description:"Data is serialized as XML via ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} and POST'd as the\n   HTTP request body with contentType \"text/xml\".",
             value:"\"postXML\""},
            {description:"Data is serialized as XML via ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}, wrapped in a SOAP\n   envelope, and POST'd as the HTTP request body with contentType \"text/xml\".  Generally\n   only used in connection with a ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL web service')}.",
             value:"\"soap\""},
            {description:"dsRequest.data is assumed to be a String set up by ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}\n   and is POST'd as the HTTP request body.",
             value:"\"postMessage\""},
            {description:"This setting entirely bypasses the SmartClient comm system. Instead of the DataSource sending\n   an HTTP request to the server, the developer is expected to implement\n   ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} to perform their own custom data manipulation logic, and\n   then call ${isc.DocUtils.linkForRef('method:DataSource.processResponse')} to handle the results of this action.",
             value:"\"clientCustom\""}
        ],
        description:"${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')} affects how the data in the DSRequest (${isc.DocUtils.linkForRef('attr:DSRequest.data')})\n is sent to the ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')}.  Listed below are the valid values for\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')} and their behavior.\n <P>\n Note that, when using the SmartClient server, data is automatically translated from\n JavaScript to Java according to the rules described ${isc.DocUtils.linkForRef('attr:RPCRequest.data','here')};\n dataProtocol does not apply and is ignored.\n <P>\n If you are integrating with a ${isc.DocUtils.linkForRef('class:RestDataSource','REST')} server that requires the more\n obscure ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')}s of \"PUT\", \"DELETE\" or \"HEAD\", you can specify these\n httpMethod settings via ${isc.DocUtils.linkForRef('attr:OperationBinding.requestProperties')}.  dataProtocol settings\n that mention \"GET\" or \"POST\" are compatible with these additional HTTP methods as well.\n Typical ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings','operationBindings')} for a REST server that uses\n \"PUT\" and \"DELETE\" are as follows:\n <pre>\n    operationBindings:[\n       {operationType:\"fetch\", dataProtocol:\"getParams\"},\n       {operationType:\"add\", dataProtocol:\"postParams\"},\n       {operationType:\"remove\", dataProtocol:\"getParams\", requestProperties:{httpMethod:\"DELETE\"}},\n       {operationType:\"update\", dataProtocol:\"postParams\", requestProperties:{httpMethod:\"PUT\"}}\n    ],\n </pre>",
        name:"DSProtocol",
        seeAlso:[
            {ref:"attr:OperationBinding.dataProtocol", description:null}
        ],
        type:"type",
        serverDS:"allowed",
        groups:["clientDataIntegration"]
    },
    "attr:Scrollbar.vSrc":{
        ref:"attr:Scrollbar.vSrc",
        flags:"IR",
        description:"Base URL for the images used for the vertical scrollbar track and end buttons.  See\n ${isc.DocUtils.linkForRef('attr:Scrollbar.hSrc')} for usage.",
        name:"vSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]vscroll.gif\"",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["images"]
    },
    "attr:RPCRequest.promptStyle":{
        ref:"attr:RPCRequest.promptStyle",
        flags:"IRW",
        description:"Controls the prompt style for this request only.  Defaults to\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.promptStyle')}.",
        name:"promptStyle",
        seeAlso:[
            {ref:"classAttr:RPCManager.promptStyle", description:null}
        ],
        valueType:"PromptStyle",
        defaultValue:"RPCManager.promptStyle",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["rpcPrompt"]
    },
    "attr:Canvas.hoverAlign":{
        ref:"attr:Canvas.hoverAlign",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, this property can be used to customize the\n alignment of content in the hover canvas.",
        name:"hoverAlign",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:Layout.getDropComponent":{
        ref:"method:Layout.getDropComponent",
        flags:"",
        description:" When ${isc.DocUtils.linkForRef('attr:Layout.canDropComponents')} is true, this method will be called when a component is\n dropped onto the layout to determine what component to add as a new layout member.\n <P>\n By default, the actual component being dragged (isc.EventHandler.getDragTarget()) will be\n added to the layout.  For a different behavior, such as wrapping dropped components in\n Windows, or creating components on the fly from dropped data, override this method.  \n <P>\n You can also return false to cancel the drop.\n\n",
        name:"getDropComponent",
        seeAlso:[],
        params:[
            {description:"current drag target", optional:false, name:"dragTarget", defaultValue:null,
             type:"Canvas"},
            {description:"index of the drop in the list of current members", optional:false,
             name:"dropPosition", defaultValue:null, type:"int"}
        ],
        returns:{description:"the component to add to the layout, or null to cancel the drop", type:"Canvas"},
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:EdgedCanvas.skinImgDir":{
        ref:"attr:EdgedCanvas.skinImgDir",
        flags:"IR",
        description:"Standard skin directory for edge images (sides and corners).",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/edges/\"",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "method:FormulaBuilder.save":{
        ref:"method:FormulaBuilder.save",
        flags:"",
        description:" Call to finish working, test the formula and call \n ${isc.DocUtils.linkForRef('method:FormulaBuilder.fireOnClose','fireOnClose()')}.  Called automatically\n when the Save button is clicked.\n\n",
        name:"save",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:Window.maximizeButton":{
        ref:"attr:Window.maximizeButton",
        flags:"R",
        description:"Button that will make this Window fill the browser via ${isc.DocUtils.linkForRef('method:Window.maximize')}.",
        name:"maximizeButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:SliderItem.maxValue":{
        ref:"attr:SliderItem.maxValue",
        flags:"IR",
        description:"The maximum slider value. The slider value is equal to maxValue when the thumb is at the\n top or right of the slider (unless flipValues is true, in which case the maximum value\n is at the bottom/left of the slider)",
        name:"maxValue",
        setter:["setMaxValue"],
        seeAlso:[
            {ref:"attr:Slider.flipValues", description:null}
        ],
        valueType:"float",
        defaultValue:"100",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"],
        definingClass:"class:SliderItem"
    },
    "attr:TimeItem.maskSaveLiterals":{
        ref:"attr:TimeItem.maskSaveLiterals",
        flags:"IRWA",
        description:"Internal-use only for a TimeItem.",
        name:"maskSaveLiterals",
        seeAlso:[
            {ref:"attr:TimeItem.useMask", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TimeItem"
    },
    "attr:ListGrid.resizeFieldsInRealTime":{
        ref:"attr:ListGrid.resizeFieldsInRealTime",
        flags:"IRWA",
        description:"True == we redraw the list viewer in real time as fields are being resized.\n		This can be slow with a large list and/or on some platforms.",
        name:"resizeFieldsInRealTime",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"Browser.isIE && isc.Browser.isWin",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "attr:TreeGrid.iconSize":{
        ref:"attr:TreeGrid.iconSize",
        flags:"IRW",
        description:"The standard size (same height and width, in pixels) of node icons in this\n          treeGrid.",
        name:"iconSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:HTMLFlow.httpMethod":{
        ref:"attr:HTMLFlow.httpMethod",
        flags:"IRW",
        description:"Selects the HTTP method that will be used when fetching content.  Valid values are \"POST\"\n and \"GET\".",
        name:"httpMethod",
        seeAlso:[],
        valueType:"SendMethod",
        defaultValue:"\"GET\"",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "method:ListGridField.recordClick":{
        ref:"method:ListGridField.recordClick",
        flags:"",
        description:"\n Executed when this field is clicked on.  Note that if ${isc.DocUtils.linkForRef('method:ListGrid.recordClick')} is\n also defined, it will be fired for fields that define a recordClick handler if the\n field-level handler returns true. Return false to prevent the grid-level handler from firing.\n\n",
        name:"recordClick",
        seeAlso:[
            {ref:"method:ListGrid.recordClick", description:null}
        ],
        params:[
            {description:"the listGrid that contains the click event", optional:false, name:"viewer",
             defaultValue:null, type:"ListGrid"},
            {description:"the record that was clicked on", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"number of the record clicked on in the current set of                                  displayed records (starts with 0)",
             optional:false, name:"recordNum", defaultValue:null, type:"number"},
            {description:"the field that was clicked on (field definition)", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"number of the field clicked on in the listGrid.fields                                  array",
             optional:false, name:"fieldNum", defaultValue:null, type:"number"},
            {description:"value of the cell (after valueMap, etc. applied)", optional:false,
             name:"value", defaultValue:null, type:"object"},
            {description:"raw value of the cell (before valueMap, etc applied)", optional:false,
             name:"rawValue", defaultValue:null, type:"object"}
        ],
        returns:{description:"false to stop event bubbling", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('recordClicks')}"],
        definingClass:"object:ListGridField",
        groups:["events"]
    },
    "method:FormItem.showIf":{
        ref:"method:FormItem.showIf",
        flags:"A",
        description:" Expression that's evaluated to see if an item should be dynamically hidden.\n <p>\n <code>showIf()</code> is evaluated whenever the form draws or redraws.\n\n",
        name:"showIf",
        seeAlso:[],
        params:[
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"current value of the form item", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the current set of values for the form as a whole", optional:false,
             name:"values", defaultValue:null, type:"Object"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('formShowAndHide')}"],
        definingClass:"class:FormItem"
    },
    "method:TileGrid.fetchData":{
        ref:"method:TileGrid.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component.\n <P>\n If there are a large number of matching records, paging will automatically be enabled, so\n that initially a smaller number of records will be retrieved and further records will\n be fetched as the user navigates the dataset.\n <P>\n When first called, this method will create a ${isc.DocUtils.linkForRef('class:ResultSet')}, which will be\n configured based on component settings such as ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fetchOperation')}\n and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataPageSize')}, as well as the general purpose\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')}.  The ResultSet is then available as\n <code>component.data</code>.\n <P>\n Subsequent calls to fetchData() will simply call ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')}\n on the created ResultSet with the passed criteria.\n <P>\n In some cases fetchData() will not need to contact the server as the new criteria can be\n satisfied by performing a client-side filter against the currently cached set of data.\n You can determine whether criteria will cause a fetch by calling \n ${isc.DocUtils.linkForRef('method:ResultSet.willFetchData')}.  If you need to force a server fetch, you can call\n ${isc.DocUtils.linkForRef('method:ListGrid.invalidateCache','invalidateCache()')} to do so.\n <P>\n This method takes an optional callback parameter (set to a ${isc.DocUtils.linkForRef('type:DSCallback')}) to fire when\n the fetch completes. Note that this callback will not fire if no server fetch is performed.\n In this case the data is updated synchronously, so as soon as this method completes you\n can interact with the new data. If necessary, you can use\n {dataBoundComponent.willFetchData,willFetchData()} to determine whether or not a server\n fetch will occur when <code>fetchData()</code> is called with new criteria.\n <P>\n In addition to the callback parameter for this method, developers can use \n ${isc.DocUtils.linkForRef('method:ListGrid.dataArrived','dataArrived()')} to be notified every time data is loaded.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundFetch')}"],
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DataSourceField.primaryKey":{
        ref:"attr:DataSourceField.primaryKey",
        flags:"IR",
        description:"Whether this field holds a value unique across all records in this DataSource.\n <p>\n A DataSource that can <i>only</i> perform the \"fetch\" operation does not require a\n primaryKey.  If a DataSource allows modification of DataSource records through add, update\n and remove DataSource operations, exactly one field must be marked as the primary key.\n <P>\n SmartClient requires a primary key value to uniquely identify records when communicating\n updates or deletions to the server.  There is no requirement that the primaryKey field be\n mapped to an actual \"primary key\" in your object model, web service, or database.  The only\n requirement is that values of the primaryKey field be unique for a given browser instance\n for the lifetime of the page.  \n <P>\n If your underlying data model has no primaryKey or has multiple primaryKey(s), you can\n generate unique values or combine multiple values into a single field that serves as a\n unique key, and declare that field as the single primary key instead.  For example, if your\n underlying data model has two primary keys \"firstName\" and \"lastName\", you can use\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} to combine them to into a new field \"name_key\":\n <pre>\n isc.DataSource.create({\n    fields:[\n        {name:\"name_key\", primaryKey:true},\n        {name:\"firstName\"},\n        {name:\"lastName\"},\n        ... other fields ...\n    ],\n    transformResponse : function (dsResponse) {\n        var data = dsResponse.data;\n        for (var i = 0 ; i &lt; data.length; i++) {\n            data[i].name_key = data[i].firstName + \":\" +\n                               data[i].lastName;\n        }\n    }\n });\n </pre>\n <P>\n Here it is assumed that \":\" is not a legal character in either the firstName or lastName\n field.  Updates to records from this DataSource will then be sent with name_key as the\n primary key value, and the server can split the value to obtain the separate key values.\n <P>\n If using ISC's ${isc.DocUtils.linkForRef('group:sqlDataSource','SQL engine')} and generating SQL tables using the\n \"Databases\" tab of the Developer Console, the table column generated from a primaryKey field\n will have a unique constraint applied in the database table.",
        name:"primaryKey",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["dataSourceRelations"]
    },
    "attr:Scrollbar.thumbInset":{
        ref:"attr:Scrollbar.thumbInset",
        flags:"IRA",
        description:"Inset of the thumb relative to the track.  An inset of N pixels means the thumb is 2N\n pixels smaller in breadth than the track.",
        name:"thumbInset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "classMethod:Class.Super":{
        ref:"classMethod:Class.Super",
        flags:"",
        description:"\n	Call the SuperClass implementation of a class method.\n\n",
        name:"Super",
        seeAlso:[],
        params:[
            {description:"name of the superclass method to call", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"native \"arguments\" object, or array of                                           arguments to pass to the Super call",
             optional:false, name:"args", defaultValue:null, type:"arguments or Array"},
            {description:"native \"arguments\" object, required if an Array is                                  passed for the \"args\" parameter in lieue of the native\n                                  arguments object",
             optional:true, name:"nativeArgs", defaultValue:null, type:"arguments"}
        ],
        returns:{description:"return value of the superclass call", type:"any"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:Window.headerIconDefaults":{
        ref:"attr:Window.headerIconDefaults",
        flags:"IRWA",
        description:"This is an object literal property block specifying the various properties of the\n headerIcon - the icon that appears at the top left of the window and is by default the\n Isomorphic logo.  Overrideable defaults are as follows:\n <ul>\n <li>width - default to <code>16</code> and specifies the width of the headerIcon.\n <li>height - default to <code>14</code> and specifies the height of the headerIcon.\n <li>src - defaults to <code>\"[SKIN]/Window/minimize.gif\"</code> and specifies the image\n for the headerIcon.\n </ul>\n You can override the the above propertites by calling ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')}.",
        name:"headerIconDefaults",
        seeAlso:[],
        valueType:"object",
        defaultValue:"...",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "method:DataSource.getClientOnlyDataSource":{
        ref:"method:DataSource.getClientOnlyDataSource",
        flags:"",
        description:" Produces a clientOnly \"copy\" of a particular subset of data from a normal \n DataSource, via calling fetchData() to fetch matching rows, and constructing \n a clientOnly DataSource that ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')} the original DataSource.\n <P>\n This clientOnly \"copy\" can be useful in situations where you want to allow \n a series of local changes without immediately committing to the server.  \n See also ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')} for more fine-grained tracking of \n edits (eg, special styling for uncommitted changes).\n <P>\n The new DataSource is returned via the \"callback\" argument.\n\n",
        name:"getClientOnlyDataSource",
        seeAlso:[],
        params:[
            {description:"The criteria for the clientOnly DS", optional:false, name:"criteria",
             defaultValue:null, type:"Criteria"},
            {description:"The callback to fire passing the clientOnly DS", optional:false,
             name:"callback", defaultValue:null, type:"Callback"},
            {description:"Properties to pass through to the DSRequest", optional:false,
             name:"requestProperties", defaultValue:null, type:"DSRequest Properties"},
            {description:"Properties to pass through to the clientOnly DS", optional:false,
             name:"dataSourceProperties", defaultValue:null, type:"DataSource Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:DetailViewer.loadingMessage":{
        ref:"attr:DetailViewer.loadingMessage",
        flags:"IRW",
        description:"The string to display in the body of a detailViewer which is loading records.",
        name:"loadingMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["emptyMessage"]
    },
    "method:MenuBar.removeMenus":{
        ref:"method:MenuBar.removeMenus",
        flags:"",
        description:" Dynamically remove menus from the menuBar. Will update the visible set of buttons as \n appropriate.\n",
        name:"removeMenus",
        seeAlso:[],
        params:[
            {description:"Array of menus to remove (will accept actual Menu components,                       or numbers representing the index of the menus in the current menus array)",
             optional:false, name:"menus", defaultValue:null, type:"array"}
        ],
        type:"method",
        definingClass:"class:MenuBar"
    },
    "method:ViewLoader.viewLoaded":{
        ref:"method:ViewLoader.viewLoaded",
        flags:"",
        description:" StringMethod fired when the view has been loaded.  Has no default implementation.  May be\n observed or overridden to fire custom logic when loading completes.\n\n",
        name:"viewLoaded",
        seeAlso:[],
        params:[
            {description:"the view that was loaded", optional:false, name:"view", defaultValue:null,
             type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:ViewLoader",
        groups:["contentLoading"]
    },
    "attr:DataSource.globalNamespaces":{
        ref:"attr:DataSource.globalNamespaces",
        flags:"IRW",
        description:"Namespaces definitions to add to the root element of outbound XML messages sent to a web\n service, as a mapping from namespace prefix to namespace URI.\n <P>\n The default value is:\n <pre>\n   globalNamespaces : {\n      xsi: \"http://www.w3.org/2001/XMLSchema-instance\",\n      xsd: \"http://www.w3.org/2001/XMLSchema\"\n   },\n </pre>\n This default value allows the use of the xsi:type and xsi:nil attributes without further\n declarations.\n <P>\n Note that some web services will only accept specific revisions of the XML Schema URI.\n If xsi-namespaced attributes seem to be ignored by an older webservice, try the URI\n \"http://www.w3.org/1999/XMLSchema-instance\" instead.",
        name:"globalNamespaces",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"...",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "group:xmlTransform":{
        ref:"group:xmlTransform",
        refs:["classMethod:XMLTools.selectNodes", "classMethod:XMLTools.selectString", 
         "classMethod:XMLTools.selectNumber","classMethod:XMLTools.transformNodes", "classMethod:XMLTools.serializeToString"],
        name:"xmlTransform",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.getCellStyle":{
        ref:"method:ListGrid.getCellStyle",
        flags:"",
        description:" Return the CSS class for a cell. By default this method has the following implementation:<br>\n - return any custom style for the record (see ${isc.DocUtils.linkForRef('attr:GridRenderer.recordCustomStyleProperty')}) \n   if defined.<br>\n - create a style name based on the result of ${isc.DocUtils.linkForRef('method:GridRenderer.getBaseStyle')} and the \n   state of the record.<br>\n The state of the record is indicated by adding a suffix to the base style.\n There are four independant boolean states, which are combined in the order given:\n <ol>\n <li>\"Disabled\" : whether the cell is disabled; enable by setting the \"enabled\" flag on record\n     returned by getCellRecord\n <li>\"Selected\" : whether cell is selected; enable by passing a Selection object as \"selection\"\n <li>\"Over\" : mouse is over this cell; enable with showRollovers\n <li>\"Dark\" : alternating color bands; enable with alternateRowStyles\n </ol>\n For example, with a baseStyle of \"myCell\", a cell which is selected, which the mouse is over,\n and which is in a dark-colored band will get a styleName of myCellSelectedOverDark.\n <p>\n Cell Styles customizable by:\n <ul>\n <li>attaching a custom style to a record by setting \n    <code>record[this.recordCustomStyleProperty]</code> to some valid CSS style name.  \n <li>modifying the base style returned by getBaseStyle() [see that method for further \n     documentation on this]\n <li>overriding this function\n </ul>\n\n",
        name:"getCellStyle",
        seeAlso:[
            {ref:"method:ListGrid.getBaseStyle", description:null}
        ],
        params:[
            {description:"record object for this row and column", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"number of the row", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"number of the column", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS style for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:FormulaBuilder.saveButton":{
        ref:"attr:FormulaBuilder.saveButton",
        flags:"IR",
        description:"Button to Save the formula, by generating it's function, testing it and firing \n formulaBuilder.fireOnClose",
        name:"saveButton",
        seeAlso:[],
        valueType:"AutoChild Button",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:ListGrid.enterKeyEditAction":{
        ref:"attr:ListGrid.enterKeyEditAction",
        flags:"IRW",
        description:"What to do when a user hits enter while editing a cell:\n <ul>\n <li>\"nextCell\": start editing the next editable cell in this record (or the first\n     editable cell in the next record if focus is in the last editable cell in the row)\n <li>\"nextRow\": start editing the same field in the next row (skipping any rows where\n      that would be a non-editable cell.\n <li>\"nextRowStart\": start editing the first editable cell in the next row.\n <li>\"done\": hide the editor (editing is complete)\n </ul>\n Note that if this.autoSaveEdits is true, this may cause a save of the current edit values",
        name:"enterKeyEditAction",
        seeAlso:[],
        valueType:"EnterKeyEditAction",
        defaultValue:"\"done\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:JSONEncoder.encode":{
        ref:"method:JSONEncoder.encode",
        flags:"",
        description:" Serialize an object as a JSON string.\n\n",
        name:"encode",
        seeAlso:[],
        params:[
            {description:"object to serialize", optional:false, name:"object", defaultValue:null,
             type:"any"}
        ],
        returns:{description:"object encoded as a JSON String", type:"String"},
        type:"method",
        definingClass:"class:JSONEncoder"
    },
    "attr:StretchImgButton.icon":{
        ref:"attr:StretchImgButton.icon",
        flags:"IRW",
        description:"Optional icon to be shown with the button title text.  \n <P>\n Specify as the partial URL to an image, relative to the imgDir of this component.",
        name:"icon",
        setter:["setIcon"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:DateItem.displayFormat":{
        ref:"attr:DateItem.displayFormat",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is <code>true</code> this property can be used to \n customize the format in which dates are displayed.<br>\n Should be set to a standard ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} or\n a function which will return a formatted date string.\n <P>\n If unset, the standard shortDate format as set up via ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat')}\n will be used.\n <P>\n <B>NOTE: you may need to update the ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat','inputFormat')} to ensure the\n DateItem is able to parse user-entered date strings back into Dates</B>",
        name:"displayFormat",
        seeAlso:[
            {ref:"attr:DateItem.inputFormat", description:null}
        ],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "group:calendarEvent":{
        ref:"group:calendarEvent",
        refs:["attr:Calendar.nameField", "attr:Calendar.descriptionField", 
         "attr:Calendar.startDateField","attr:Calendar.endDateField", "attr:Calendar.leadingDateField", 
         "attr:Calendar.trailingDateField","attr:Calendar.eventTypeField", "attr:Calendar.eventWindowStyleField", 
         "attr:Calendar.canEditField","attr:Calendar.eventAutoArrange", "attr:Calendar.eventOverlap", 
         "attr:Calendar.eventOverlapPercent","attr:Calendar.eventOverlapIdenticalStartTimes"],
        name:"calendarEvent",
        seeAlso:[],
        type:"group"
    },
    "attr:TextItem.textBoxStyle":{
        ref:"attr:TextItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class name for this item's input element.\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"textItem\"",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["appearance"]
    },
    "attr:ListGrid.sortField":{
        ref:"attr:ListGrid.sortField",
        flags:"IR",
        description:"Specifies the field by which this grid should be initially sorted.  Can be set to either a\n ${isc.DocUtils.linkForRef('attr:ListGridField.name','field name')} or the index of the field in the fields Array. Note that\n if <code>sortField</code> is initally specified as a number, it will be converted to a\n string (field name) after list grid initialization.\n <P>\n To programmatically change sort field or direction after initialization, call ${isc.DocUtils.linkForRef('method:ListGrid.sort')}.",
        name:"sortField",
        seeAlso:[],
        valueType:"String or integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        definingClass:"class:ListGrid",
        groups:["sorting"]
    },
    "class:PickTreeItem":{
        methods:[],
        ref:"class:PickTreeItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:PickTreeItem.dataSource", "attr:PickTreeItem.valueTree", 
         "attr:PickTreeItem.loadDataOnDemand","attr:PickTreeItem.displayField", "attr:PickTreeItem.valueField", 
         "attr:PickTreeItem.emptyMenuMessage","attr:PickTreeItem.canSelectParentItems"],
        description:"FormItem that allows picking a value from a hierarchical data model.",
        name:"PickTreeItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('pickTree')}"]
    },
    "group:menuItemEvents":{
        ref:"group:menuItemEvents",
        refs:["method:MenuItem.click"],
        name:"menuItemEvents",
        seeAlso:[],
        type:"group"
    },
    "attr:GridRenderer.wrapCells":{
        ref:"attr:GridRenderer.wrapCells",
        flags:"IRWA",
        description:"Should content within cells be allowed to wrap?",
        name:"wrapCells",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "method:ListGridField.defaultDynamicValue":{
        ref:"method:ListGridField.defaultDynamicValue",
        flags:"",
        description:" If this field ${isc.DocUtils.linkForRef('attr:ListGridField.canEdit','can be edited')}, this property can be used to\n set a dynamic default value which will show up in editors for this field. \n Will be applied to the editor for the field as ${isc.DocUtils.linkForRef('method:FormItem.defaultDynamicValue')}\n\n",
        name:"defaultDynamicValue",
        seeAlso:[],
        params:[
            {description:"The editor for the cell itself (also available as \"this\").                              Note that in addition to the standard FormItem APIs available\n                              on the editor, it also has:<br>\n                              - a pointer back to the containing listGrid \n                              [<code>item.grid</code>]<br>\n                              - the colNum being edited [<code>item.colNum</code>]<br>\n                              - the rowNum being edited [<code>item.rowNum</code>]",
             optional:false, name:"item", defaultValue:null, type:"FormItem"},
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the current set of values for the form as a whole", optional:false,
             name:"values", defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:ListGrid.alternateBodyStyleName":{
        ref:"attr:ListGrid.alternateBodyStyleName",
        flags:"IRWA",
        description:"Optional css style to apply to the body if ${isc.DocUtils.linkForRef('attr:ListGrid.alternateRecordStyles')} is true \n for this grid. If unset ${isc.DocUtils.linkForRef('attr:ListGrid.bodyStyleName')} will be used to style the body\n regardless of the ${isc.DocUtils.linkForRef('attr:ListGrid.alternateRecordStyles','alternateRecordStyles')} setting.",
        name:"alternateBodyStyleName",
        setter:["setAlternateBodyStyleName"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:StretchImgButton.vertical":{
        ref:"attr:StretchImgButton.vertical",
        flags:"IRW",
        description:"Default is a horizontal button.  Vertical StretchImgButtons are allowed, but title text,\n if any, will not be automatically rotated.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["appearance"]
    },
    "classAttr:RPCManager.useXmlHttpRequest":{
        ref:"classAttr:RPCManager.useXmlHttpRequest",
        deprecated:"As of SmartClient 5.5, use ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')}.  If you specify a value for this property, it will take precedence over\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')} for requests that do not specify a\n ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')} or ${isc.DocUtils.linkForRef('attr:RPCRequest.useXmlHttpRequest')}.",
        flags:"RW",
        description:"Selects the default http transport for all RPC requests.  If set to true, RPCManager\n will use XMLHttp for requests to the server.  If set to false, it will use hidden\n frames.  Overrideable on a per-request basis via ${isc.DocUtils.linkForRef('attr:RPCRequest.useXmlHttpRequest')}.\n <p>\n Note that if the end user disables ActiveX controls in Internet Explorer, the XMLHttpRequest\n object will not be available and SmartClient will automatically fall back on frames\n communication.",
        name:"useXmlHttpRequest",
        seeAlso:[
            {ref:"attr:RPCRequest.useXmlHttpRequest", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true if XMLHttpRequest is supported, false otherwise",
        type:"classAttr",
        definingClass:"class:RPCManager"
    },
    "method:List.setLength":{
        ref:"method:List.setLength",
        flags:"",
        description:" Set the length of this list.\n <P>\n If the length of the list is shortened, any elements past the new length of the list are removed.\n If the length is increased, all positions past the old length have the value\n <code>undefined</code>.\n\n",
        name:"setLength",
        seeAlso:[],
        params:[
            {description:"new length", optional:false, name:"length", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "classAttr:Operators.notEqualFieldTitle":{
        ref:"classAttr:Operators.notEqualFieldTitle",
        flags:"IR",
        description:"Title for the \"notEqualField\" operator",
        name:"notEqualFieldTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"differs from field\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:TestFunctionResult.record":{
        ref:"attr:TestFunctionResult.record",
        flags:"IRW",
        description:"Set to the record that was used when testing the generated function.  This is the record\n selected by ${isc.DocUtils.linkForRef('method:FormulaBuilder.getTestRecord')}.",
        name:"record",
        seeAlso:[],
        valueType:"Record",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "class:DateChooser":{
        methods:["method:DateChooser.setData", "method:DateChooser.getData", 
         "method:DateChooser.dataChanged","method:DateChooser.cancelClick", "method:DateChooser.todayClick"],
        ref:"class:DateChooser",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:DateChooser.headerHeight", "attr:DateChooser.skinImgDir", 
         "attr:DateChooser.prevYearIcon","attr:DateChooser.prevYearIconWidth", "attr:DateChooser.prevYearIconHeight", 
         "attr:DateChooser.prevMonthIcon","attr:DateChooser.prevMonthIconWidth", "attr:DateChooser.prevMonthIconHeight", 
         "attr:DateChooser.nextYearIcon","attr:DateChooser.nextYearIconWidth", "attr:DateChooser.nextYearIconHeight", 
         "attr:DateChooser.nextMonthIcon","attr:DateChooser.nextMonthIconWidth", "attr:DateChooser.nextMonthIconHeight", 
         "attr:DateChooser.showDoubleYearIcon","attr:DateChooser.yearMenuStyle", "attr:DateChooser.monthMenuStyle", 
         "attr:DateChooser.showTodayButton","attr:DateChooser.showCancelButton", "attr:DateChooser.todayButtonTitle", 
         "attr:DateChooser.cancelButtonTitle","attr:DateChooser.todayButtonHeight", "attr:DateChooser.disableWeekends", 
         "attr:DateChooser.showWeekends","attr:DateChooser.firstDayOfWeek", "attr:DateChooser.baseButtonStyle", 
         "attr:DateChooser.baseWeekdayStyle","attr:DateChooser.baseWeekendStyle", "attr:DateChooser.alternateWeekStyles", 
         "attr:DateChooser.alternateStyleSuffix","attr:DateChooser.headerStyle", "attr:DateChooser.weekendHeaderStyle", 
         "attr:DateChooser.baseNavButtonStyle","attr:DateChooser.baseBottomButtonStyle"],
        description:"Simple interactive calendar interface used to pick a date.\n Used by the ${isc.DocUtils.linkForRef('class:DateItem')} class.",
        name:"DateChooser",
        seeAlso:[],
        type:"class"
    },
    "classMethod:Class.registerStringMethods":{
        ref:"classMethod:Class.registerStringMethods",
        flags:"",
        description:"\n	Register a method, or set of methods, that can be provided to instances of this class as\n	Strings (containing a JavaScript expression) and will be automatically converted into\n	functions.\n  <p>\n  For example:\n  <pre>\n  isc.MyClass.registerStringMethods({\n      myStringMethod: \"arg1, arg2\"\n  });\n  </pre>\n\n",
        name:"registerStringMethods",
        seeAlso:[
            {ref:"group:stringMethods", description:null}
        ],
        params:[
            {description:"If this is a string, name of the property to register                                  If this is an object, assume passing in a set of name/value\n                                  pairs to register",
             optional:false, name:"methodName", defaultValue:null, type:"object"},
            {description:"named arguments for the property in a comma separated string                                  (not used if methodName is an object)",
             optional:false, name:"argumentString", defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:DataSource.dataProtocol":{
        flags:"IR",
        getter:["getDataProtocol"],
        type:"attr",
        ref:"attr:DataSource.dataProtocol",
        description:"Controls the format in which inputs are sent to the dataURL when fulfilling DSRequests.\n May be overridden for individual request types using\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','operation bindings')}.",
        name:"dataProtocol",
        valueType:"DSProtocol",
        seeAlso:[],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration", "serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "classMethod:Time.getDefaultDisplayTimezone":{
        ref:"classMethod:Time.getDefaultDisplayTimezone",
        flags:"",
        description:" Returns the default display timezone set up by ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone')}.\n If no explicit timezone has been set this will return the browser locale timezone offset.\n",
        name:"getDefaultDisplayTimezone",
        seeAlso:[],
        returns:{description:"String of the format <code>+/-HH:MM</code>", type:"string"},
        type:"classMethod",
        definingClass:"class:Time"
    },
    "group:display_values":{
        ref:"group:display_values",
        refs:["method:FormItem.getValueFieldName", "method:FormItem.getOptionDataSource", 
         "method:PickList.getValueFieldName","method:SelectItem.getValueFieldName", "method:ComboBoxItem.getValueFieldName", 
         "method:ListGridField.getCellValue","method:ListGridField.formatCellValue", "method:ListGrid.formatCellValue", 
         "attr:FormItem.emptyDisplayValue","attr:FormItem.optionDataSource", "attr:FormItem.valueField", "attr:FormItem.displayField", 
         "attr:FormItem.fetchMissingValues","attr:FormItem.filterLocally", "attr:PickList.valueField", "attr:PickList.displayField", 
         "attr:SelectItem.displayField","attr:SelectItem.valueField", "attr:ComboBoxItem.displayField", 
         "attr:ComboBoxItem.valueField","attr:ListGridRecord.linkText", "attr:ListGrid.autoFetchDisplayMap", 
         "attr:ListGridField.linkText","attr:ListGridField.valueMap", "attr:ListGridField.emptyCellValue", 
         "attr:ListGridField.autoFetchDisplayMap","attr:ListGridField.optionTextMatchStyle", "attr:ListGridField.optionFilterContext", 
         "attr:ListGridField.optionOperationId","attr:ListGridField.optionDataSource", "attr:ListGridField.valueField", 
         "attr:ListGridField.displayField","attr:ListGrid.linkTextProperty"],
        name:"display_values",
        seeAlso:[],
        type:"group"
    },
    "attr:DSRequest.exportHeader":{
        ref:"attr:DSRequest.exportHeader",
        flags:"IR",
        description:"Optional text to appear at the beginning of the file.",
        name:"exportHeader",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:ListGridField.getSummaryTitle":{
        ref:"method:ListGridField.getSummaryTitle",
        flags:"A",
        description:" Optional string method to return a long summary title for this field, if a dynamic\n summary title is required for this field.  \n\n",
        name:"getSummaryTitle",
        seeAlso:[
            {ref:"attr:ListGridField.summaryTitle", description:null},
            {ref:"attr:ListGridField.title", description:null}
        ],
        params:[
            {description:"pointer back to the ListGrid", optional:false, name:"viewer",
             defaultValue:null, type:"listGrid"},
            {description:"pointer to the field object", optional:false, name:"field", defaultValue:null,
             type:"listGridField"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:DynamicForm.getEditorType":{
        ref:"method:DynamicForm.getEditorType",
        flags:"A",
        description:"\n Returns the form item type (Class Name) to be created for some field.<br>\n By default <code>field.editorType</code> will be used if present - otherwise backs off to\n deriving the appropriate form item type from the data type of the field (see\n ${isc.DocUtils.linkForRef('type:FormItemType')} for details.\n\n",
        name:"getEditorType",
        seeAlso:[],
        params:[
            {description:"field definition for which we are deriving form item type.", optional:false,
             name:"field", defaultValue:null, type:"object"}
        ],
        returns:{description:"form item type for the field", type:"string"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["editing"]
    },
    "attr:DSRequest.data":{
        ref:"attr:DSRequest.data",
        flags:"IR",
        description:"Data, such as search criteria or an updated record, to be acted upon.  Contents differ by\n <code>operationType</code>, see ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource Operations')} for\n details.\n <P>\n This field is generally filled in by passing the \"data\" argument to methods such as\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}.",
        name:"data",
        seeAlso:[],
        valueType:"Object or Array of Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:DataSource.transformResponse":{
        ref:"method:DataSource.transformResponse",
        flags:"A",
        description:" Modify the DSResponse object derived from the response returned from the \n ${isc.DocUtils.linkForRef('attr:DataSource.dataURL','dataURL')}.\n <P>\n This is an override point that makes it possible to use DataSource features such as\n paging with web services that support such features, by allowing you to fill in metadata\n fields in the DSResponse object (such as ${isc.DocUtils.linkForRef('attr:DSResponse.startRow')}) based on\n service-specific metadata fields contained in the service's response.\n <P>\n The DSResponse passed to this method already has ${isc.DocUtils.linkForRef('attr:DSResponse.data')}, which is\n derived differently depending on the ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','dataFormat')} setting:\n <ul>\n <li> <code>dataFormat:\"xml\"</code> : either the\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')} or \n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordName','recordName')} is used to select the XML elements\n that represent DataSource records.  The selected XML elements are passed to\n ${isc.DocUtils.linkForRef('method:DataSource.recordsFromXML')}, which transforms the XML elements to typed\n JavaScript data using the DataSource as a schema.\n <li> <code>dataFormat:\"json\"</code> : the\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')}, if specified, is used to select\n records from the returned JSON data via ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectObjects')}.  \n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')} is used to derive correctly typed field values.\n <li> <code>dataFormat:\"custom\"</code> : <code>dsResponse.data</code> is the raw response\n in String form.  It must be parsed into an Array of Objects for subsequent processing to\n work.\n </ul> \n <P>\n In addition to <code>dsResponse.data</code>, ${isc.DocUtils.linkForRef('attr:DSResponse.status')} is defaulted\n to 0 (indicating no error), and ${isc.DocUtils.linkForRef('attr:DSResponse.startRow')} is assumed to be zero,\n with ${isc.DocUtils.linkForRef('attr:DSResponse.endRow','endRow')} and ${isc.DocUtils.linkForRef('attr:DSResponse.totalRows','totalRows')}\n both set to <code>dsResponse.data.length - 1</code>, that is, the returned data is\n assumed to be all records that matched the filter criteria.\n <P>\n Examples of using this API include:\n <ul>\n <li> setting ${isc.DocUtils.linkForRef('attr:DSResponse.startRow','startRow')}, \n      ${isc.DocUtils.linkForRef('attr:DSResponse.endRow','endRow')} and ${isc.DocUtils.linkForRef('attr:DSResponse.totalRows','totalRows')}\n      to allow paging for a service that supports it.  For example, if an XML service\n      returns a \"resultRow\" tag that contained the row number of the first row of the\n      returned results:<pre>\n    dsResponse.startRow = \n        isc.XMLTools.selectNumber(xmlData, \"//resultRow\");\n </pre>\n <li> setting ${isc.DocUtils.linkForRef('attr:DSResponse.status')} to recognized ISC error values based on\n      service-specific errors, in order to trigger standard ISC error handling.  For\n      example, setting <code>dsResponse.status</code> to\n      ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_VALIDATION_ERROR')} and filling in\n      ${isc.DocUtils.linkForRef('attr:DSResponse.errors')} in order to cause validation errors to be shown in\n      forms and grids.\n <li> for services that either do not return cache update data, or return partial data,\n      using ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues')} to create cache update data (whether this is\n      appropriate is application-specific), or setting\n      ${isc.DocUtils.linkForRef('attr:DSResponse.invalidateCache')}.\n </ul>\n <P>\n <span STYLE=\"color:red\">NOTE:</span> this method is NOT an appropriate time to call\n methods on visual components such as grids, initiate new DSRequests or RPCRequests, or\n in general do anything other than fill in fields on the DSResponse based on data that is\n already available.  Any actions that need to be taken as a result of the web\n service response should be implemented exactly as for a DataSource where\n <code>transformResponse()</code> has not been overriden, that is, use the callback\n passed to high-level methods such as \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','<code>grid.fetchData()</code>')}, and do error\n handling via either ${isc.DocUtils.linkForRef('method:DataSource.handleError')} or by setting\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError','willHandleError')}.\n\n",
        name:"transformResponse",
        seeAlso:[],
        params:[
            {description:"default DSResponse derived from the response data", optional:false,
             name:"dsResponse", defaultValue:null, type:"DSResponse"},
            {description:"DSRequest object that initiated this request", optional:false,
             name:"dsRequest", defaultValue:null, type:"DSRequest"},
            {description:"XML document or JSON objects returned by the web                                   service",
             optional:false, name:"data", defaultValue:null, type:"XMLDocument or JSON"}
        ],
        returns:{description:"response derived", type:"DSResponse"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('xmlServerValidationErrors')}"],
        definingClass:"class:DataSource"
    },
    "object:DetailViewerField":{
        methods:["method:DetailViewerField.formatCellValue", "method:DetailViewerField.showIf", 
         "method:DetailViewerField.getCellStyle"],
        ref:"object:DetailViewerField",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/DetailViewer"],
        attrs:["attr:DetailViewerField.imageSize", "attr:DetailViewerField.imageWidth", 
         "attr:DetailViewerField.imageHeight","attr:DetailViewerField.imageURLPrefix", "attr:DetailViewerField.type", 
         "attr:DetailViewerField.title","attr:DetailViewerField.valueMap", "attr:DetailViewerField.value", 
         "attr:DetailViewerField.height","attr:DetailViewerField.showFileInline", "attr:DetailViewerField.canExport"],
        description:"An object literal with a particular set of properties used to configure the display of\n and interaction with the rows of a ${isc.DocUtils.linkForRef('class:DetailViewer')}.",
        name:"DetailViewerField",
        seeAlso:[],
        type:"object"
    },
    "group:autoDraw":{
        ref:"group:autoDraw",
        refs:["classMethod:isc.setAutoDraw"],
        name:"autoDraw",
        seeAlso:[],
        type:"group"
    },
    "attr:StretchImgButton.labelHPad":{
        ref:"attr:StretchImgButton.labelHPad",
        flags:"IRW",
        description:"The padding for a StretchImgButton's label is determined as follows.\n <P>\n If <code>labelHPad</code> is set it will specify the horizontal padding applied to the\n label. Similarly if <code>labelVPad</code> is set it will specify the vertical padding\n for the label, regardless of the button's ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical','vertical')} setting.\n <P>\n Otherwise <code>labelLengthPad</code> can be set to specify the label padding along the\n length axis (ie: horizontal padding if ${isc.DocUtils.linkForRef('attr:StretchImgButton.vertical')} is false,\n otherwise vertical padding), and \n <code>labelBreadthPad</code> can be set to specify the label padding along the other axis.\n <P>\n Otherwise the padding on the length axis will match the ${isc.DocUtils.linkForRef('attr:StretchImgButton.capSize')} and\n will be set to zero on the breadth axis.\n <P>\n So by default the label will be sized to match the center image of the StretchImgButton, but\n these settings allow the label to partially or wholly overlap the caps.",
        name:"labelHPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "attr:SelectItem.allowEmptyValue":{
        ref:"attr:SelectItem.allowEmptyValue",
        flags:"IRW",
        description:"If set to true, always show an empty option in this item's pickList, allowing the user\n to clear the value (even if there is no empty entry in the valueMap for the item).\n <P>\n The empty value will be displayed with the\n ${isc.DocUtils.linkForRef('attr:FormItem.emptyDisplayValue','emptyDisplayValue')}.\n <P>\n With a ${isc.DocUtils.linkForRef('attr:SelectItem.optionDataSource','databound selectItem')}, enabling\n <code>allowEmptyValue</code> disables data paging - all data matching the\n ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria','current criteria')} will be requested.",
        name:"allowEmptyValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "type:DialogButtons":{
        ref:"type:DialogButtons",
        values:[
            {description:"Button object to fire dialog's \"okClick()\" method on click.              Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.OK_BUTTON_TITLE')}.",
             value:"OK"},
            {description:"Button object to fire dialog's \"applyClick()\" method on click.              Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.APPLY_BUTTON_TITLE')}.",
             value:"APPLY"},
            {description:"Button object to fire dialog's \"yesClick()\" method on click                  Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.YES_BUTTON_TITLE')}.",
             value:"YES"},
            {description:"Button object to fire dialog's \"noClick()\" method on click.              Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.NO_BUTTON_TITLE')}.",
             value:"NO"},
            {description:"Button object to fire dialog's \"cancelClick()\" method on click.                  Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.CANCEL_BUTTON_TITLE')}.",
             value:"CANCEL"},
            {description:"Button object to fire dialog's \"doneClick()\" method on click.                  Title derived from ${isc.DocUtils.linkForRef('classAttr:Dialog.DONE_BUTTON_TITLE')}.",
             value:"DONE"}
        ],
        description:"Default buttons that you can use in your Dialogs.\n <P>\n On click these call canonical methods that you can override in your Dialog.\n <P>\n Refer to these buttons via the syntax <code>isc.Dialog.OK</code> when passing them into\n ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons')} or into the <code>properties</code> argument of helper\n methods such as ${isc.DocUtils.linkForRef('classMethod:isc.say')}.",
        name:"DialogButtons",
        seeAlso:[],
        type:"type"
    },
    "method:Canvas.destroy":{
        ref:"method:Canvas.destroy",
        flags:"A",
        description:"  Permanently destroy a Canvas.<br><br>\n  \n  This does everything that clear() does, but unlike clear(), the Canvas cannot be draw()'n again,\n  cannot be referenced by it's global ID, and is eligible for garbage collection (assuming that\n  application code is not holding a reference to the Canvas).<br><br>\n\n  Any attempt to call a method on a destroyed Canvas will generally result in an error.  If your\n  application is forced to hold onto Canvas's that might be destroy()d without warning, you can\n  avoid errors by checking for the ${isc.DocUtils.linkForRef('attr:Canvas.destroyed')} property.  If you override certain Canvas\n  methods, your code may be called while a Canvas is being destroy()d; in this case you can avoid\n  extra work (and possibly errors) by checking for the ${isc.DocUtils.linkForRef('attr:Canvas.destroying')} property.\n  <P>\n  Note that <code>destroy()</code> should not be called directly in event handling code for this\n  canvas. For this reason, wherever possible we recommend using ${isc.DocUtils.linkForRef('method:Canvas.markForDestroy')}\n  instead of calling this method directly. \n \n",
        name:"destroy",
        seeAlso:[
            {ref:"method:Canvas.markForDestroy", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:TreeGrid.leafClick":{
        ref:"method:TreeGrid.leafClick",
        flags:"",
        description:"\n This method is called when a leaf record is clicked on.\n\n",
        name:"leafClick",
        seeAlso:[
            {ref:"method:TreeGrid.nodeClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which leafClick() occurred.", optional:false, name:"viewer",
             defaultValue:null, type:"TreeGrid"},
            {description:"The leaf (record) that was clicked", optional:false, name:"leaf",
             defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the click occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:SectionItem.sectionHeaderClass":{
        ref:"attr:SectionItem.sectionHeaderClass",
        flags:"IRA",
        description:"Name of the Canvas subclass to use as a header that labels the section and allows\n showing and hiding.  The default class be skinned, or trivial subclasses created to\n allow different appearances for SectionItems in different forms.\n Very advanced developers can use the following information to create custom header\n classes.",
        name:"sectionHeaderClass",
        seeAlso:[],
        valueType:"Classname",
        defaultValue:"\"SectionHeader\"",
        type:"attr",
        definingClass:"class:SectionItem"
    },
    "method:Tree.isDescendantOf":{
        ref:"method:Tree.isDescendantOf",
        flags:"",
        description:"			Is one node a descendant of the other?\n\n",
        name:"isDescendantOf",
        seeAlso:[],
        params:[
            {description:"child node", optional:false, name:"child", defaultValue:null, type:"TreeNode"},
            {description:"parent node", optional:false, name:"parent", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true == parent is an ancestor of child", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:Slider.length":{
        ref:"attr:Slider.length",
        flags:"IRW",
        description:"Used to set slider height if vertical, slider width if horizontal.\n Applied to the slider track, not necessarily the entire widget.\n Overridden by an explicit width/height specification for the widget.",
        name:"length",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"200",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:Button.iconAlign":{
        ref:"attr:Button.iconAlign",
        flags:"IR",
        description:"If this button is showing an icon should it be right or left aligned?",
        name:"iconAlign",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "classAttr:RPCManager.removeDataPrompt":{
        ref:"classAttr:RPCManager.removeDataPrompt",
        flags:"IRW",
        description:"Default prompt displayed to user while an opration is running to remove data from\n the server.<br>\n Displayed as a result of the ${isc.DocUtils.linkForRef('method:ListGrid.removeSelectedData')} code path.",
        name:"removeDataPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Deleting Record(s)...\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["i18nMessages"]
    },
    "attr:Canvas.animateHideEffect":{
        ref:"attr:Canvas.animateHideEffect",
        flags:"IRWA",
        description:"Default animation effect to use if ${isc.DocUtils.linkForRef('method:Canvas.animateHide')} is called without an\n explicit <code>effect</code> parameter",
        name:"animateHideEffect",
        seeAlso:[],
        valueType:"animateShowEffectId | animateShowEffect",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "classAttr:Log.stackTracePriority":{
        ref:"classAttr:Log.stackTracePriority",
        flags:"IRWA",
        description:"At this priority and above, a stack trace will be included automatically along with the log\n message itself.",
        name:"stackTracePriority",
        seeAlso:[],
        valueType:"LogPriority",
        defaultValue:"Log.ERROR",
        type:"classAttr",
        definingClass:"class:Log"
    },
    "classAttr:GroupingMessages.byYearTitle":{
        ref:"classAttr:GroupingMessages.byYearTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by year.",
        name:"byYearTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Year\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:Calendar.selectedCellStyle":{
        ref:"attr:Calendar.selectedCellStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to a cell that is selected via a mouse drag.",
        name:"selectedCellStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calendarCellSelected\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "attr:CheckboxItem.showUnsetImage":{
        ref:"attr:CheckboxItem.showUnsetImage",
        flags:"IRA",
        description:"Determines what image to display when the value for this checkbox is unset.\n Set to true to display the ${isc.DocUtils.linkForRef('attr:CheckboxItem.unsetImage')} for null values, or false to\n use the ${isc.DocUtils.linkForRef('attr:CheckboxItem.uncheckedImage')} for both null and explicitly unchecked values.\n <P>\n If this attribute is not set, the ${isc.DocUtils.linkForRef('attr:CheckboxItem.unsetImage')} for null values if \n ${isc.DocUtils.linkForRef('attr:CheckboxItem.allowEmptyValue')} is true for this item, otherwise the unchecked\n image will be used.",
        name:"showUnsetImage",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CheckboxItem"
    },
    "class:RichTextEditor":{
        methods:["method:RichTextEditor.doWarn", "method:RichTextEditor.richEditorSupported", 
         "method:RichTextEditor.getValue","method:RichTextEditor.setValue"],
        ref:"class:RichTextEditor",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:RichTextEditor.value", "attr:RichTextEditor.controlGroups", 
         "attr:RichTextEditor.styleControls","attr:RichTextEditor.fontControls", "attr:RichTextEditor.fontNames", 
         "attr:RichTextEditor.fontSizes","attr:RichTextEditor.formatControls", "attr:RichTextEditor.colorControls"],
        description:"RichTextEditing component.  Provides a rich-text editing area along with UI for executing\n rich-text commands on the text.<br>\n The HTML generated from this component may vary by browser, and, as with any HTML \n value created on the client, we recommend values be sanitized on the server before \n storing and displaying to other users.<br>\n Note: This component has limited support on the Safari browser.",
        name:"RichTextEditor",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('RichTextEditor')}"]
    },
    "attr:Splitbar.showClosedGrip":{
        ref:"attr:Splitbar.showClosedGrip",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether the grip image\n displayed should show the <code>\"Closed\"</code> state when the ${isc.DocUtils.linkForRef('attr:Splitbar.target')} \n is hidden.",
        name:"showClosedGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["grip"]
    },
    "attr:Menu.checkmarkDisabledImage":{
        ref:"attr:Menu.checkmarkDisabledImage",
        flags:"IRWA",
        description:"Default image to display for disabled checkmarked items. See ${isc.DocUtils.linkForRef('object:ImgProperties')}\n for format. Valid src, width and height must be specified.",
        name:"checkmarkDisabledImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:Canvas.enabled":{
        deprecated:"As of SmartClient version 5.5 this property has been deprecated. The   ${isc.DocUtils.linkForRef('attr:Canvas.disabled')} property will be used to govern enabled/disabled state instead \n  if <code>this.enabled</code> has not been changed from its default value.",
        flags:"IRWA",
        type:"attr",
        ref:"attr:Canvas.enabled",
        description:"If set to true, this widget will be enabled, if set to false, or null, this \n widget will be disabled.",
        name:"enabled",
        setter:["setEnabled"],
        valueType:"boolean",
        seeAlso:[],
        defaultValue:"\"unset\"",
        groups:["enable"],
        definingClass:"class:Canvas"
    },
    "attr:ListGrid.canGroupBy":{
        ref:"attr:ListGrid.canGroupBy",
        flags:"IRW",
        description:"If false, grouping via context menu will be disabled.",
        name:"canGroupBy",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:OperationBinding.responseDataSchema":{
        ref:"attr:OperationBinding.responseDataSchema",
        flags:"IRA",
        description:"Optional schema describing how to extract DataSource records from the XML elements selected.\n <P>\n Once a set of XML elements have been selected via <code>recordXPath</code> or\n <code>recordName</code>, those elements are normally transformed to JavaScript objects\n using the <code>fields</code> of the DataSource that owns the operationBinding.  A\n <code>responseDataSchema</code> can be specified instead if the XML differs in some way between\n different DataSource operations, such that different values for\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','field.valueXPath')} may be necessary to extract the same\n DataSource record from slightly different XML structures.",
        name:"responseDataSchema",
        seeAlso:[],
        valueType:"DataSource",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:Canvas.showResizeBar":{
        flags:"IR",
        type:"attr",
        ref:"attr:Canvas.showResizeBar",
        description:"When this Canvas is included as a member in a Layout, whether a resizeBar should be shown\n after this member in the layout, to allow it to be resized.\n <p>\n By default the resize bar acts on the Canvas that it is declared on.  If you want the resize\n bar to instead act on the next member of the Layout (e.g. to collapse down or to the right),\n set ${isc.DocUtils.linkForRef('attr:Canvas.resizeBarTarget')} as well.",
        name:"showResizeBar",
        setter:["setShowResizeBar"],
        valueType:"boolean",
        seeAlso:[
            {ref:"attr:Canvas.resizeBarTarget", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('layoutNesting')}"],
        defaultValue:"false",
        groups:["layoutMember"],
        definingClass:"class:Canvas"
    },
    "classMethod:isc.firstKey":{
        ref:"classMethod:isc.firstKey",
        flags:"",
        description:" Return the first property name in a given Object, according to for..in iteration order.\n\n",
        name:"firstKey",
        seeAlso:[],
        params:[
            {description:"Object to get properties from", optional:false, name:"object",
             defaultValue:null, type:"Object"}
        ],
        returns:{description:"first property name, or null if Object has no properties", type:"String"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:TileGrid.tileConstructor":{
        ref:"attr:TileGrid.tileConstructor",
        flags:"IR",
        description:"Classname of a SmartClient component to use for each tile rendered by this TileGrid.  Tiles\n are created by the ${isc.DocUtils.linkForRef('type:AutoChild')} pattern; see ${isc.DocUtils.linkForRef('attr:TileGrid.tile')}.\n <P> \n Any subclass of Canvas is allowed, but typically any custom class will derive from\n ${isc.DocUtils.linkForRef('class:SimpleTile')}.",
        name:"tileConstructor",
        seeAlso:[],
        valueType:"SCClass",
        defaultValue:"\"SimpleTile\"",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "method:TreeGrid.setSelectedPaths":{
        ref:"method:TreeGrid.setSelectedPaths",
        flags:"",
        description:" Reset this grid's selection to match the ${isc.DocUtils.linkForRef('type:listGridSelectedState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to \n ${isc.DocUtils.linkForRef('method:TreeGrid.getSelectedPaths')}.\n\n",
        name:"setSelectedPaths",
        seeAlso:[
            {ref:"method:TreeGrid.getSelectedPaths", description:null}
        ],
        params:[
            {description:"Object describing the desired selection state of                                              the grid",
             optional:false, name:"selectedPaths", defaultValue:null, type:"listGridSelectedState"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "attr:RPCRequest.bypassCache":{
        ref:"attr:RPCRequest.bypassCache",
        flags:"IRWA",
        description:"For xmlHttp transport + httpMethod: \"GET\" only, set to true to force a conditional\n GET request even if the browser thinks it has a current cached response.",
        name:"bypassCache",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:Tree.isLeaf":{
        ref:"method:Tree.isLeaf",
        flags:"",
        description:"\n Returns true if the passed in node is a leaf.\n\n",
        name:"isLeaf",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node is a leaf", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:DataSourceField.tableName":{
        ref:"attr:DataSourceField.tableName",
        flags:"IR",
        description:"The table name to use when qualifying the column name for this field during server-side SQL\n query generation.  Only applicable to \"sql\" dataSources, and only has an effect when \n ${isc.DocUtils.linkForRef('attr:DataSource.qualifyColumnNames')} or ${isc.DocUtils.linkForRef('attr:OperationBinding.qualifyColumnNames')} is in\n force.\n <p>\n <b>Note:</b> This property is not normally required; the SmartClient server will ordinarily \n qualify all column names correctly without assistance. This property is useful when \n qualification can't be done automatically - for example, when you have a custom \n ${isc.DocUtils.linkForRef('attr:OperationBinding.tableClause','tableClause')} that renames tables in the query using aliases.",
        name:"tableName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "classMethod:isc.eval":{
        ref:"classMethod:isc.eval",
        flags:"",
        description:" Evaluate a string of script and return the result. Falls through to\n ${isc.DocUtils.linkForRef('classMethod:Class.evaluate','Class.evaluate()')}\n\n",
        name:"eval",
        seeAlso:[],
        params:[
            {description:"Expression to evaluate", optional:false, name:"expression", defaultValue:null,
             type:"string"}
        ],
        returns:{description:"Result of evaluating the expression passed in", type:"any"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:Progressbar.percentDone":{
        ref:"attr:Progressbar.percentDone",
        flags:"IRW",
        description:"Number from 0 to 100, inclusive, for the percentage to be displayed graphically in\n this progressbar.",
        name:"percentDone",
        setter:["setPercentDone"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:Progressbar",
        groups:["appearance"]
    },
    "method:MenuBar.showMenu":{
        ref:"method:MenuBar.showMenu",
        flags:"",
        description:" Shows (opens) a menu.\n",
        name:"showMenu",
        seeAlso:[],
        params:[
            {description:"menu to show (may be specified as a menu object, or index of                                   the menu from ${isc.DocUtils.linkForRef('attr:MenuBar.menus','this.menus')}).",
             optional:false, name:"menu", defaultValue:null, type:"Menu | integer"}
        ],
        type:"method",
        definingClass:"class:MenuBar"
    },
    "attr:DetailViewer.fields":{
        ref:"attr:DetailViewer.fields",
        flags:"IRW",
        description:"An array of field objects, specifying the order and type of fields to display in this\n DetailViewer.  In DetailViewers, the fields specify rows.",
        name:"fields",
        seeAlso:[],
        valueType:"List of DetailViewerField",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DetailViewer"
    },
    "class:DSRequest":{
        methods:[],
        ref:"class:DSRequest",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:DSRequest.dataSource", "attr:DSRequest.operationType", "attr:DSRequest.data", 
         "attr:DSRequest.callback","attr:DSRequest.startRow", "attr:DSRequest.endRow", "attr:DSRequest.sortBy", 
         "attr:DSRequest.textMatchStyle","attr:DSRequest.oldValues", "attr:DSRequest.componentId", "attr:DSRequest.operationId", 
         "attr:DSRequest.requestId","attr:DSRequest.clientContext", "attr:DSRequest.parentNode", "attr:DSRequest.resultTree", 
         "attr:DSRequest.resultSet","attr:DSRequest.useFlatFields", "attr:DSRequest.useFlatHeaderFields", 
         "attr:DSRequest.headerData","attr:DSRequest.exportResults", "attr:DSRequest.exportAs", "attr:DSRequest.exportFilename", 
         "attr:DSRequest.exportDisplay","attr:DSRequest.lineBreakStyle", "attr:DSRequest.exportFields", 
         "attr:DSRequest.exportHeader","attr:DSRequest.exportFooter"],
        description:"Request sent to the server to initiate a \n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource operation')}.  All properties which are legal on\n ${isc.DocUtils.linkForRef('class:RPCRequest')} are legal, in addition to the properties listed here.",
        name:"DSRequest",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        type:"class",
        inheritsFrom:"RPCRequest"
    },
    "attr:ListGrid.recordCustomStyleProperty":{
        ref:"attr:ListGrid.recordCustomStyleProperty",
        flags:"IRW",
        description:"Denotes the name of a property that can be set on records to display a custom style.\n For example if this property is set to <code>\"customStyle\"</code>, setting \n <code>record.customStyle</code> to a css styleName will cause the record in question to\n render out with that styling applied to it.  Note that this will be a static \n style - it will not be modified as the state of the record (selected / over etc) changes.",
        name:"recordCustomStyleProperty",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null},
            {ref:"attr:ListGrid.recordBaseStyleProperty", description:null}
        ],
        valueType:"String",
        defaultValue:"\"customStyle\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:LoginDialog.allowBlankPassword":{
        ref:"attr:LoginDialog.allowBlankPassword",
        flags:"IR",
        description:"If true, the login form will allow blank passwords to be submitted. Otherwise\n the form fails to be validated until the user enters at least one character into\n the password field.",
        name:"allowBlankPassword",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:ListGrid.getSummaryFieldValue":{
        ref:"method:ListGrid.getSummaryFieldValue",
        flags:"",
        description:" Get the computed value of a ${isc.DocUtils.linkForRef('attr:ListGrid.canAddSummaryFields','summary field')}.\n",
        name:"getSummaryFieldValue",
        seeAlso:[],
        params:[
            {description:"field that has a summary format", optional:false, name:"field",
             defaultValue:null, type:"Field"},
            {description:"record to use to compute formula value", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"formula result", type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:RadioGroupItem.setValueDisabled":{
        ref:"method:RadioGroupItem.setValueDisabled",
        flags:"",
        description:" Disable or Enable a specific option within this radioGroup\n",
        name:"setValueDisabled",
        seeAlso:[],
        params:[
            {description:"value of option to disable", optional:false, name:"value", defaultValue:null,
             type:"any"},
            {description:"true to disable the option, false to enable it", optional:false,
             name:"disabled", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:RadioGroupItem"
    },
    "method:CubeGrid.facetValueReordered":{
        ref:"method:CubeGrid.facetValueReordered",
        flags:"",
        description:" \n      Notification fired when a facet or facetValueGroup is reordered\n\n      <br><i>methodType</i> handler\n",
        name:"facetValueReordered",
        seeAlso:[],
        params:[
            {description:"facetValueGroupId or facetId", optional:false, name:"groupId",
             defaultValue:null, type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "method:ValuesManager.clearValue":{
        ref:"method:ValuesManager.clearValue",
        flags:"",
        description:" Clear the value for some field.\n",
        name:"clearValue",
        seeAlso:[],
        params:[
            {description:"Which field to set the value for", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "classAttr:ValidatorTypes.doesntContain":{
        ref:"classAttr:ValidatorTypes.doesntContain",
        flags:"IR",
        description:"Determine whether a string value does not contain some substring specified via \n <code>validator.substring</code>.",
        name:"doesntContain",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:TabSet.paneContainerOverflow":{
        ref:"attr:TabSet.paneContainerOverflow",
        flags:"IRWA",
        description:"Specifies the overflow of the pane container (the component that holds the pane contents\n for all tabs).  By default this is set to \"auto\", meaning the pane container will\n automatically introduce scrolling when the pane contents exceed the TabSet's specified\n size.\n <p>\n For other values and their meaning, see ${isc.DocUtils.linkForRef('type:Overflow')}",
        name:"paneContainerOverflow",
        setter:["setPaneContainerOverflow"],
        seeAlso:[],
        valueType:"Overflow",
        defaultValue:"Canvas.AUTO",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "method:Tree.isFolder":{
        ref:"method:Tree.isFolder",
        flags:"",
        description:"\n Determines whether a particular node is a folder.  The logic works as follows:<br><br>\n <ul>\n <li> If the ${isc.DocUtils.linkForRef('object:TreeNode')} has a value for the ${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty')}\n (${isc.DocUtils.linkForRef('attr:TreeNode.isFolder')} by default) that value is returned.\n <li> Next, the existence of the ${isc.DocUtils.linkForRef('attr:Tree.childrenProperty')} (by default\n ${isc.DocUtils.linkForRef('attr:TreeNode.children')}) is checked on the ${isc.DocUtils.linkForRef('object:TreeNode')}.  If the node has the children\n property defined (regardless of whether it actually has any children), then isFolder()\n returns true for that node.\n </ul>\n <P>\n You can override this method to provide your own intepretation of what constitutes a folder.\n\n",
        name:"isFolder",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node is a folder", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:SectionStack.visibilityMode":{
        ref:"attr:SectionStack.visibilityMode",
        flags:"IRW",
        description:"Whether multiple sections can be visible at once",
        name:"visibilityMode",
        seeAlso:[
            {ref:"type:VisibilityMode", description:null}
        ],
        valueType:"VisibilityMode",
        defaultValue:"\"mutex\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"],
        definingClass:"class:SectionStack"
    },
    "classAttr:RPCManager.credentialsURL":{
        ref:"classAttr:RPCManager.credentialsURL",
        flags:"RWA",
        description:"Specifies URL where credentials should be submitted to attempt relogin when session\n timeout is encountered during a background RPC.  See ${isc.DocUtils.linkForRef('group:relogin','Relogin')}",
        name:"credentialsURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"RPCManager.credentialsURL",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["relogin"]
    },
    "attr:LinkItem.target":{
        ref:"attr:LinkItem.target",
        flags:"IRW",
        description:"By default, clicking a link rendered by this item opens it in a new browser window.  You \n can alter this behavior by setting this property.  The value of this property will be \n passed as the value to the <code>target</code> attribute of the anchor tag used to render \n the link.\n <P>\n If you set linkItem.target to \"javascript\", the default behaviour is to catch and consume\n mouse-clicks that would result in the link being followed.  Instead, the\n ${isc.DocUtils.linkForRef('method:FormItem.click')} event is fired.",
        name:"target",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"_blank\"",
        type:"attr",
        definingClass:"class:LinkItem"
    },
    "attr:ListGrid.canAcceptDroppedRecords":{
        ref:"attr:ListGrid.canAcceptDroppedRecords",
        flags:"IRW",
        description:"Indicates whether records can be dropped into this listGrid.",
        name:"canAcceptDroppedRecords",
        seeAlso:[
            {ref:"attr:ListGridRecord.canDrag", description:null},
            {ref:"attr:ListGridRecord.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "attr:ComboBoxItem.showPickListOnKeypress":{
        ref:"attr:ComboBoxItem.showPickListOnKeypress",
        flags:"IRW",
        description:"Should the list of options be displayed whenever the user types into the \n the combo-box textArea, or only when the user clicks on the pick button or uses the \n explicit <code>Alt+Arrow Down</code> key combination?",
        name:"showPickListOnKeypress",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:TabSet.pickerButtonHSrc":{
        ref:"attr:TabSet.pickerButtonHSrc",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true, and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricPickerButton')} is \n set to true, this property governs the base URL for the picker\n button image, when displayed in a horizontal tab-bar [IE ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')} is\n set to <code>\"top\"</code> or <code>\"bottom\"</code>].\n <P>\n Note that if <code>symmetricPickerButton</code> is false, the ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonSrc')}\n property will be used instead.\n <P>\n This base URL will have a suffix of <code>\"Down\"</code> appended when the user holds the\n mouse down over the button, and <code>\"Disabled\"</code> if the tabset as a whole is \n disabled.",
        name:"pickerButtonHSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricPickerButton", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]hpicker.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "method:Array.contains":{
        ref:"method:Array.contains",
        flags:"",
        description:" Return if this list contains the specified object.\n <P>\n If pos is specified, starts looking after that position.\n\n",
        name:"contains",
        seeAlso:[],
        params:[
            {description:"item to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"optional position in the list to look after", optional:true, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == item was found, false == not found", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:ListGrid.confirmDiscardEditsMessage":{
        ref:"attr:ListGrid.confirmDiscardEditsMessage",
        flags:"IRW",
        description:"If <code>this.confirmDiscardEdits</code> is true, this property can be used to customize the\n error message string displayed to the user in a dialog with options to \n cancel the action, or save or discard pending edits in response to sort/filter actions\n that would otherwise drop unsaved edit values.",
        name:"confirmDiscardEditsMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"This action will discard all unsaved edited values for this list.\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing", "i18nMessages"]
    },
    "attr:ComboBoxItem.pickerIconSrc":{
        ref:"attr:ComboBoxItem.pickerIconSrc",
        flags:"IRWA",
        description:"Src for the picker icon.",
        name:"pickerIconSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/ComboBoxItem_PickButton_icon.gif\"",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "classMethod:RPCManager.suspendTransaction":{
        ref:"classMethod:RPCManager.suspendTransaction",
        flags:"A",
        description:" Suspends the current transaction, such that all processing of the transaction is halted,\n any remaining ${isc.DocUtils.linkForRef('attr:RPCRequest.callback','callbacks')} in the transaction won't fire, and\n the transaction can never ${isc.DocUtils.linkForRef('attr:RPCRequest.timeout','timeout')}.\n <P>\n <code>suspendTransaction()</code> is typically used to handle total failures for an\n entire transaction, such as HTTP status 500, or session timeout resulting in\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.loginRequired','loginRequired()')} being called.  In both cases the intent\n is to put the transaction on hold so that a transient problem can be resolved, and then\n the transaction can be re-sent successfully.  By using suspendTransaction(), components\n that submitted requests never realize there was a transient failure, and so error\n handling logic does not have to be implemented in every component.\n <P>\n Generally you can only validly suspend a transaction from either\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.loginRequired')} or ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')}, and in the case of\n handleError(), only when the first response in the transaction has an error.  Suspending\n and re-sending a partially processed transaction means that some responses will be\n processed twice, with undefined results for requests issued automatically by UI\n components.\n <P>\n A suspended transaction must ultimately be either cleared via\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.clearTransaction')} or re-sent via ${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction')} or memory will be\n leaked.\n\n",
        name:"suspendTransaction",
        seeAlso:[
            {ref:"classMethod:RPCManager.resendTransaction", description:null}
        ],
        params:[
            {description:"transaction to delay.  Defaults to the                      current transaction if there is one",
             optional:true, name:"transaction", defaultValue:null, type:"transaction Obj or ID"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "classAttr:GroupingMessages.upcomingNextWeekTitle":{
        ref:"classAttr:GroupingMessages.upcomingNextWeekTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs next week, \n relative to the current date.",
        name:"upcomingNextWeekTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Next Week\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:TreeGrid.dropIconSuffix":{
        ref:"attr:TreeGrid.dropIconSuffix",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:TreeGrid.showDropIcons')} is true, this suffix will be appended to the\n ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} when the user drop-hovers over some folder.",
        name:"dropIconSuffix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"drop\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:FormItem.fetchMissingValues":{
        ref:"attr:FormItem.fetchMissingValues",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} is specified for this item, should the item \n perform a fetch on the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} to retrieve the display value\n for the item when ${isc.DocUtils.linkForRef('method:FormItem.setValue')} is called with a value for which we\n do not already have a mapped display value?\n <P>\n As noted under ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}, you may <b>also</b> specify a ${isc.DocUtils.linkForRef('type:ValueMap')}\n and it will be consulted first before fetching from the <code>optionDataSource</code>.",
        name:"fetchMissingValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "method:Layout.addMember":{
        ref:"method:Layout.addMember",
        flags:"",
        description:"\n  Add a canvas to the layout, optionally at a specific position.\n\n",
        name:"addMember",
        seeAlso:[
            {ref:"method:Layout.addMembers", description:null}
        ],
        params:[
            {description:"the canvas object to be added to the layout", optional:false,
             name:"newMember", defaultValue:null, type:"canvas"},
            {description:"the position in the layout to place newMember (starts with 0);                                  if omitted, it will be added at the last position",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:ResultTree.rootNode":{
        ref:"attr:ResultTree.rootNode",
        flags:"IR",
        description:"This attribute may be used to specify a root value for the parentIdField of this resultTree.\n This overrides the default ${isc.DocUtils.linkForRef('attr:DataSourceField.rootValue')} for this tree, allowing\n a component to navigate a tree starting at a specific node.\n <P>\n May be overridden via ${isc.DocUtils.linkForRef('attr:TreeGrid.treeRootValue')} for ResultTrees generated by a TreeGrid\n component.",
        name:"rootNode",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "attr:CubeGrid.facetLabelHoverHeight":{
        ref:"attr:CubeGrid.facetLabelHoverHeight",
        flags:"IRWA",
        description:"If specified and <code>this.showHover</code> is true, this is the default height to apply\n to hover tips shown for facetLabels. If unset, hover canvas will be sized to \n <code>this.hoverHeight</code> if specified instead.",
        name:"facetLabelHoverHeight",
        seeAlso:[
            {ref:"attr:Canvas.hoverHeight", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:ButtonItem.showTitle":{
        ref:"attr:ButtonItem.showTitle",
        flags:"IRW",
        description:"Buttons do not show a title by default.",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ButtonItem",
        groups:["appearance"]
    },
    "type:FireStyle":{
        ref:"type:FireStyle",
        values:[
            {description:"Call the registered handler any time the event occurs", value:"null"},
            {description:"Call the registered handler the first time the event                               occurs, then unregister the handler as though\n                               ${isc.DocUtils.linkForRef('classMethod:Page.clearEvent')} had been called",
             value:"Page.FIRE_ONCE"}
        ],
        description:"Flags to set automatic removal of events from the page event registry.",
        name:"FireStyle",
        seeAlso:[
            {ref:"classMethod:Page.setEvent", description:null}
        ],
        type:"type",
        groups:["EventRegistry"]
    },
    "attr:GridRenderer.alternateRowStyles":{
        ref:"attr:GridRenderer.alternateRowStyles",
        flags:"IRW",
        description:"Whether alternating rows should be drawn in alternating styles, in order to create a \"ledger\"\n effect for easier reading.  If enabled, the cell style for alternate rows will have \"Dark\"\n appended to it.",
        name:"alternateRowStyles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:Facet.showParentsLast":{
        ref:"attr:Facet.showParentsLast",
        flags:"IR",
        description:"Indicates internal hierarchy should be displayed in reverse of normal tree order (so\n that parents follow children)",
        name:"showParentsLast",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Tree.defaultIsFolder":{
        ref:"attr:Tree.defaultIsFolder",
        flags:"IR",
        description:"Controls whether nodes are assumed to be folders or leaves by default.\n <P>\n Nodes that have children or have the ${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty')} set to true will be considered\n folders by default.  Other nodes will be considered folders or leaves by default according\n to this setting.",
        name:"defaultIsFolder",
        seeAlso:[
            {ref:"attr:TreeGrid.loadDataOnDemand", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Tree"
    },
    "method:ColorPicker.setHue":{
        ref:"method:ColorPicker.setHue",
        flags:"",
        description:" Sets the Hue of the selected color\n",
        name:"setHue",
        seeAlso:[
            {ref:"method:ColorPicker.getHue", description:null}
        ],
        params:[
            {description:"An integer between 0 and 239", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "group:paging":{
        ref:"group:paging",
        refs:["attr:DSResponse.startRow", "attr:DSResponse.endRow", "attr:DSResponse.totalRows", 
         "attr:DSRequest.startRow","attr:DSRequest.endRow"],
        name:"paging",
        seeAlso:[],
        type:"group"
    },
    "attr:TreeGrid.initialData":{
        ref:"attr:TreeGrid.initialData",
        flags:"IRA",
        description:"You can specify the initial set of data for a databound TreeGrid using this property.\n The value of this attribute should be a list of <code>parentId</code> linked\n ${isc.DocUtils.linkForRef('object:TreeNode')}s in a format equivalent to that documented on ${isc.DocUtils.linkForRef('attr:Tree.data')}.",
        name:"initialData",
        seeAlso:[
            {ref:"object:TreeNode", description:null},
            {ref:"attr:Tree.data", description:null}
        ],
        valueType:"List of TreeNode",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('initialData')}"],
        definingClass:"class:TreeGrid"
    },
    "attr:ServerObject.bean":{
        ref:"attr:ServerObject.bean",
        flags:"IR",
        description:"For use when ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')} is <code>\"spring\"</code>, id of the bean to ask\n Spring to create.",
        name:"bean",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "attr:TabSet.animateTabScrolling":{
        ref:"attr:TabSet.animateTabScrolling",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller')} is true, should tabs be scrolled into view via an \n animation when the user interacts with the scroller buttons?",
        name:"animateTabScrolling",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "attr:ServerObject.className":{
        ref:"attr:ServerObject.className",
        flags:"IR",
        description:"Specifies the fully-qualified class name that provides the server-side endpoint of the DMI\n (${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')}:\"new\") or the class name of the factory that produces the\n DMI instance (${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')}:\"factory\").\n <p>\n This is one of the values that you need to pass to ${isc.DocUtils.linkForRef('classMethod:DMI.call')} to invoke the DMI from\n the client.\n <p>\n The value of this attribute is used only for <code>\"new\"</code> and <code>\"factory\"</code>\n values of ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')}.",
        name:"className",
        seeAlso:[
            {ref:"attr:ServerObject.lookupStyle", description:null},
            {ref:"attr:ServerObject.ID", description:null},
            {ref:"classMethod:DMI.call", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "method:CubeGrid.setFacetTitleAlign":{
        ref:"method:CubeGrid.setFacetTitleAlign",
        flags:"",
        description:" Set the align of a facet title (appears in facet label)\n\n",
        name:"setFacetTitleAlign",
        seeAlso:[],
        params:[
            {description:"facet to update", optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"new alignment for facet title", optional:false, name:"align",
             defaultValue:null, type:"alignEnum"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:Facet.borderBefore":{
        ref:"attr:Facet.borderBefore",
        flags:"IR",
        description:"CSS line style to apply as a border before this facet, eg \"1px dashed blue\"",
        name:"borderBefore",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:ListGrid.headerTitleStyle":{
        ref:"attr:ListGrid.headerTitleStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:StretchImgButton.titleStyle')} to apply to the buttons in the header, and the sorter,\n for this ListGrid.\n Note that this will typically only have an effect if \n ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor')} is set to ${isc.DocUtils.linkForRef('class:StretchImgButton')} or a subclass \n thereof.",
        name:"headerTitleStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:DataSourceField.length":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSourceField.length",
        description:"Maximum number of characters allowed.  Applicable only to fields of text type.",
        name:"length",
        valueType:"number",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('longText')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["dataType"],
        definingClass:"class:DataSourceField"
    },
    "classAttr:Operators.lessOrEqualTitle":{
        ref:"classAttr:Operators.lessOrEqualTitle",
        flags:"IR",
        description:"Title for the \"lessOrEqual\" operator",
        name:"lessOrEqualTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"less than or equal to\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:Facet.values":{
        ref:"attr:Facet.values",
        flags:"IRW",
        description:"Array of facetValue definitions.",
        name:"values",
        seeAlso:[
            {ref:"class:FacetValue", description:null}
        ],
        valueType:"Array of FacetValue",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Window.minimizeAcceleration":{
        ref:"attr:Window.minimizeAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated minimize / maximize.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"minimizeAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header", "animation"]
    },
    "attr:ImgSectionHeader.background":{
        ref:"attr:ImgSectionHeader.background",
        flags:"R",
        description:"Background of the section header, based on a StretchImg.",
        name:"background",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgSectionHeader"
    },
    "attr:DetailViewer.blockStyle":{
        ref:"attr:DetailViewer.blockStyle",
        flags:"IRW",
        description:"CSS style for each block (one record's worth of data).",
        name:"blockStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detailBlock\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:Class.getSuperClass":{
        ref:"method:Class.getSuperClass",
        flags:"",
        description:"	\n	Gets a pointer to the class object for this instance's superclass.\n\n",
        name:"getSuperClass",
        seeAlso:[],
        returns:{description:"Class object for superclass.", type:"Class"},
        type:"method",
        definingClass:"class:Class"
    },
    "method:Canvas.getImgURL":{
        ref:"method:Canvas.getImgURL",
        flags:"A",
        description:" Return the full URL for an image to be drawn in this canvas.\n <P>\n If the passed URL begins with the special prefix \"[SKIN]\", it will have the\n widget.skinImgDir and Page.skinImgDir prepended.  Otherwise the image is assumed to be\n application-specific, and will have the widget.appImgDir and Page.appImgDir automatically\n prepended.\n <P>\n Note that if passed an absolute path (starting with \"/\" or \"http://\" for example), no extra\n image directory information will be prepended to the generated URL.// \n		\n",
        name:"getImgURL",
        seeAlso:[],
        params:[
            {description:"URL local to skin or application image directory", optional:false,
             name:"URL", defaultValue:null, type:"string"},
            {description:"optional image directory to override the default for this Canvas",
             optional:true, name:"imgDir", defaultValue:null, type:"string"}
        ],
        returns:{description:"URL to use", type:"string"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "method:ImgButton.action":{
        ref:"method:ImgButton.action",
        flags:"",
        description:" This property contains the default 'action' for the Button to fire when activated.\n",
        name:"action",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton"
    },
    "classMethod:isc.setAutoDraw":{
        ref:"classMethod:isc.setAutoDraw",
        flags:"",
        description:" Set the global default setting for ${isc.DocUtils.linkForRef('attr:Canvas.autoDraw')}.\n <p>\n After calling <code>isc.setAutoDraw()</code>, any newly created Canvas which is not given an\n explicit setting for ${isc.DocUtils.linkForRef('attr:Canvas.autoDraw','autoDraw')} will follow the new default setting.\n <P>\n autoDraw:false is the recommended default setting for most applications since it ensures\n that extra draws will not occur when developers inadvertently omit the autoDraw:false\n setting on child components.\n\n",
        name:"setAutoDraw",
        seeAlso:[
            {ref:"attr:Canvas.autoDraw", description:null}
        ],
        params:[
            {description:"whether autoDraw should be enabled or disabled.  Defaults to true.",
             optional:true, name:"enable", defaultValue:null, type:null}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["autoDraw"]
    },
    "attr:OperationBinding.criteria":{
        ref:"attr:OperationBinding.criteria",
        flags:"IR",
        description:"A list of ${isc.DocUtils.linkForRef('object:DSRequestModifier')}s that will be used to modify the critieria of each \n ${isc.DocUtils.linkForRef('class:DSRequest')} that uses this operationBinding.",
        name:"criteria",
        seeAlso:[
            {ref:"attr:OperationBinding.values", description:null}
        ],
        valueType:"List of DSRequestModifier",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "attr:DataSourceField.hidden":{
        ref:"attr:DataSourceField.hidden",
        flags:"IR",
        description:"Whether this field should be hidden from users by default within a DataBound component.\n This is generally used for internal IDs and other fields not meaningful to users.\n <P>\n See ${isc.DocUtils.linkForRef('attr:DataSourceField.detail')} for fields that should be hidden in a summary view such as\n a ${isc.DocUtils.linkForRef('class:ListGrid')}, but still available to the user.",
        name:"hidden",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "attr:DetailViewer.dataFetchMode":{
        ref:"attr:DetailViewer.dataFetchMode",
        flags:"IRW",
        description:"DetailViewers do not yet support paging, and will fetch and render all available\n records.",
        name:"dataFetchMode",
        seeAlso:[],
        valueType:"FetchMode",
        defaultValue:"\"basic\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["databinding"]
    },
    "attr:FormItem.hoverVAlign":{
        ref:"attr:FormItem.hoverVAlign",
        flags:"IRW",
        description:"Vertical text alignment  for text displayed in this item's hover canvas, if shown.",
        name:"hoverVAlign",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverVAlign", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:DataSource.jsonPrefix":{
        ref:"attr:DataSource.jsonPrefix",
        flags:"IRA",
        description:"Allows you to specify an arbitrary prefix string to apply to all json format responses \n sent from the server to this application.<br>\n The inclusion of such a prefix ensures your code is not directly executable outside of\n your application, as a preventitive measure against\n <a href='http://www.google.com/search?q=javascript+hijacking' onclick=\"window.open('http://www.google.com/search?q=javascript+hijacking');return false;\">javascript hijacking</a>.\n <br>\n Only applies to responses formatted as json objects. Does not apply to responses returned\n via scriptInclude type transport.<br>\n Note: If the prefix / suffix served by your backend is not a constant, you can use \n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataFormat','dataFormat:\"custom\"')} instead and explicitly parse\n the prefix out as part of ${isc.DocUtils.linkForRef('method:DataSource.transformResponse','transformResponse()')}.",
        name:"jsonPrefix",
        seeAlso:[
            {ref:"attr:OperationBinding.dataFormat", description:null},
            {ref:"attr:OperationBinding.dataTransport", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:ComboBoxItem.defaultDynamicValue":{
        ref:"method:ComboBoxItem.defaultDynamicValue",
        flags:"A",
        description:" Expression evaluated to determine the ${isc.DocUtils.linkForRef('attr:ComboBoxItem.defaultValue')} when no value is \n provided for this item. To default to the first option use\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.defaultToFirstOption')} instead.\n",
        name:"defaultDynamicValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "classMethod:Log.applyLogPriorities":{
        ref:"classMethod:Log.applyLogPriorities",
        flags:"",
        description:" Apply a batch a batch of priority settings, as a object mapping category names to priority\n levels.\n\n",
        name:"applyLogPriorities",
        seeAlso:[],
        params:[
            {description:"priority settings for multiple categories", optional:false, name:"settings",
             defaultValue:null, type:"Object"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "group:elements":{
        ref:"group:elements",
        refs:["method:DynamicForm.setItems", "method:DynamicForm.setFields", 
         "method:DynamicForm.getFields","method:DynamicForm.getItems"],
        description:"Manipulating native form elements",
        name:"elements",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.filterData":{
        ref:"method:ListGrid.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundFilter')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:Canvas.dragRepositionMove":{
        ref:"method:Canvas.dragRepositionMove",
        flags:"A",
        description:"\n Executed every time the mouse moves while drag-repositioning. If this method\n does not return false, the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline if\n ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance')} is set to \"outline\" will automatically be moved as appropriate\n whenever the mouse moves.\n\n",
        name:"dragRepositionMove",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to suppress auto-move of the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} or outline.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:StretchImgButton.align":{
        ref:"attr:StretchImgButton.align",
        flags:"IRW",
        description:"Horizontal alignment of this component's title.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["appearance"]
    },
    "attr:ListGridField.showFocusedIcon":{
        ref:"attr:ListGridField.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "attr:ListGrid.generateClickOnSpace":{
        ref:"attr:ListGrid.generateClickOnSpace",
        flags:"IRWA",
        description:"If true, when the user navigates to a cell using arrow keys and hits space, \n the cell will respond to a click event.",
        name:"generateClickOnSpace",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ListGridField.showGroupSummary":{
        ref:"attr:ListGridField.showGroupSummary",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, should this field show a summary value\n in a summary row when the grid is grouped?\n If unset, this field will show a summary value in the summary row if an\n explicit ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')} is specified or if a\n ${isc.DocUtils.linkForRef('classMethod:SimpleType.getDefaultSummaryFunction','default summary function')} is defined\n for the specified field type.",
        name:"showGroupSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:ImgButton.labelHPad":{
        ref:"attr:ImgButton.labelHPad",
        flags:"IRW",
        description:"Horizontal padding to be applied to this widget's label. If this value is null, \n the label will be given a horizontal padding of zero.\n <p>\n The specified amount of padding is applied to the left and right edges of the button, so\n the total amount of padding is 2x the specified value.",
        name:"labelHPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton"
    },
    "classMethod:Page.resizeTo":{
        ref:"classMethod:Page.resizeTo",
        flags:"",
        description:"		Resize the outer portion of the window to a specific width and height.\n",
        name:"resizeTo",
        seeAlso:[],
        params:[
            {description:"new width for the window", optional:false, name:"width", defaultValue:null,
             type:"number"},
            {description:"new height for the window", optional:false, name:"height", defaultValue:null,
             type:"number"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "attr:Canvas.showHover":{
        ref:"attr:Canvas.showHover",
        flags:"IRW",
        description:"If <code>this.canHover</code> is true, should we show the global hover canvas by default\n when the user hovers over this canvas?",
        name:"showHover",
        seeAlso:[
            {ref:"method:Canvas.getHoverHTML", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:Canvas.dragAppearance":{
        ref:"attr:Canvas.dragAppearance",
        flags:"IRWA",
        description:"Visual appearance to show when the object is being dragged.",
        name:"dragAppearance",
        seeAlso:[],
        valueType:"DragAppearance",
        defaultValue:"EventHandler.OUTLINE",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Canvas.getLeft":{
        ref:"method:Canvas.getLeft",
        flags:"",
        description:"			Return the left coordinate of this object, relative to its enclosing context, in pixels.\n",
        name:"getLeft",
        seeAlso:[],
        returns:{description:"left coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:TreeGrid.leafContextClick":{
        ref:"method:TreeGrid.leafContextClick",
        flags:"",
        description:"\n This method is called when a context click occurs on a leaf record.\n\n",
        name:"leafContextClick",
        seeAlso:[
            {ref:"method:TreeGrid.nodeContextClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which the contexclick occurred.", optional:false,
             name:"viewer", defaultValue:null, type:"TreeGrid"},
            {description:"The leaf (record) on which the contextclick occurred.", optional:false,
             name:"leaf", defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the contextclick occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:ListGrid.modalEditing":{
        ref:"attr:ListGrid.modalEditing",
        flags:"IRWA",
        description:"If this property is true, any mouse click outside of the open cell editors\n      will end editing mode, hiding the cell editors and saving any changes to those\n      cell values.",
        name:"modalEditing",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('modalEditing')}"],
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:ColorPicker.setLuminosity":{
        ref:"method:ColorPicker.setLuminosity",
        flags:"",
        description:" Sets the Luminosity (brightness) of the selected color\n",
        name:"setLuminosity",
        seeAlso:[
            {ref:"method:ColorPicker.getLuminosity", description:null}
        ],
        params:[
            {description:"An integer between 0 and 240", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "method:Window.maximize":{
        ref:"method:Window.maximize",
        flags:"",
        description:" Maximize the window. Fired when the user clicks the maximize button if \n ${isc.DocUtils.linkForRef('attr:Window.showMaximizeButton','this.showMaximizeButton')} is true.<br>\n Default implementation moves the window to <code>0, 0</code> and resizes the window to \n <code>\"100%\"</code> on both axes, so it will fill the browser window (or the parentElement\n of the Window instance, if appropriate).<br>\n If ${isc.DocUtils.linkForRef('attr:Window.animateMinimize','animateMinimize')} is true, the maximize will be animated.\n A restore button will be displayed in place of the maximize button when the window is \n maximized.\n \n",
        name:"maximize",
        seeAlso:[],
        type:"method",
        definingClass:"class:Window"
    },
    "attr:ListGridField.iconVAlign":{
        ref:"attr:ListGridField.iconVAlign",
        flags:"IRA",
        description:"If this field is editable, this property will be passed to editors displayed for\n cells within this field as ${isc.DocUtils.linkForRef('attr:FormItem.iconVAlign')}.",
        name:"iconVAlign",
        seeAlso:[
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:TabBar.defaultTabSize":{
        ref:"attr:TabBar.defaultTabSize",
        flags:"IR",
        description:"Default size (length) in pixels for tabs within this tabBar",
        name:"defaultTabSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"80",
        type:"attr",
        definingClass:"class:TabBar"
    },
    "method:Canvas.getWidth":{
        ref:"method:Canvas.getWidth",
        flags:"",
        description:" Return the width of this object, in pixels.\n",
        name:"getWidth",
        seeAlso:[],
        returns:{description:"width", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:TileLayout.addTile":{
        ref:"method:TileLayout.addTile",
        flags:"",
        description:" Add a tile to the layout, dynamically.\n\n",
        name:"addTile",
        seeAlso:[],
        params:[
            {description:"new tile to add", optional:false, name:"tile", defaultValue:null,
             type:"Canvas"},
            {description:"position where the tile should be added.  Defaults to adding the tile at the end.",
             optional:true, name:"index", defaultValue:null, type:"int"}
        ],
        type:"method",
        definingClass:"class:TileLayout"
    },
    "method:SelectItem.showPicker":{
        ref:"method:SelectItem.showPicker",
        flags:"",
        description:" Method to show a picker for this item. By default this method is called if the user\n clicks on a ${isc.DocUtils.linkForRef('attr:SelectItem.showPickerIcon','pickerIcon')}.  May also be called programatically.\n <P>\n Default implementation lazily creates and shows the ${isc.DocUtils.linkForRef('attr:FormItem.picker','Picker Autochild')}.\n May be overridden to implement some custom picker for this item.\n\n",
        name:"showPicker",
        seeAlso:[],
        type:"method",
        definingClass:"class:SelectItem"
    },
    "method:ListGrid.cellHover":{
        ref:"method:ListGrid.cellHover",
        flags:"",
        description:" Called when the mouse hovers over a cell if this.canHover is true. \n  Returning false will suppress the hover text from being shown if this.showHover is true.\n\n",
        name:"cellHover",
        seeAlso:[
            {ref:"attr:ListGrid.canHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:Menu.setData":{
        ref:"method:Menu.setData",
        flags:"",
        description:" Change the set of items to display in this menu\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"new items for this menu", optional:false, name:"items", defaultValue:null,
             type:"array of MenuItems"}
        ],
        type:"method",
        definingClass:"class:Menu",
        groups:["data"]
    },
    "attr:Canvas.layoutAlign":{
        ref:"attr:Canvas.layoutAlign",
        flags:"IR",
        description:"When this Canvas is included as a member in a Layout, layoutAlign controls alignment on the\n breadth axis of the layout.  Default is \"left\" for a VLayout, \"top\" for an HLayout.",
        name:"layoutAlign",
        seeAlso:[],
        valueType:"Alignment or VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["layoutMember"]
    },
    "attr:Layout.paddingAsLayoutMargin":{
        ref:"attr:Layout.paddingAsLayoutMargin",
        flags:"IRWA",
        description:"If this widget has padding specified (as ${isc.DocUtils.linkForRef('attr:Canvas.padding','this.padding')} or in the\n CSS style applied to this layout), should it show up as space outside the members,\n similar to layoutMargin?\n <P>\n If this setting is false, padding will not affect member positioning (as CSS padding\n normally does not affect absolutely positioned children).  Leaving this setting true\n allows a designer to more effectively control layout purely from CSS.\n <P>\n Note that ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')} if specified, takes precidence over this value.",
        name:"paddingAsLayoutMargin",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "classMethod:Page.setTitle":{
        ref:"classMethod:Page.setTitle",
        flags:"",
        description:"		Set the title of the page, which is typically shown as part of the browser window title\n",
        name:"setTitle",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:ComboBoxItem.defaultToFirstOption":{
        ref:"attr:ComboBoxItem.defaultToFirstOption",
        flags:"IRW",
        description:"Select the first option as the default value for this ComboBoxItem. If options are derived\n from a dataSource, the first value returned by the server will be used, otherwise the first\n value in the valueMap. If enabled, this setting overrides ${isc.DocUtils.linkForRef('attr:ComboBoxItem.defaultValue')}\n and ${isc.DocUtils.linkForRef('method:ComboBoxItem.defaultDynamicValue')}.",
        name:"defaultToFirstOption",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:MultiSortDialog.addLevelButton":{
        ref:"attr:MultiSortDialog.addLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for adding new levels\n to the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.addLevelButtonProperties</code> and \n <code>multiSortPanel.addLevelButtonDefaults</code>.",
        name:"addLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:MenuButton.setShowMenuButtonImage":{
        ref:"method:MenuButton.setShowMenuButtonImage",
        flags:"",
        description:" Setter for the 'showMenuButtonImage' property - shows/hides the menu button image\n at runtime\n",
        name:"setShowMenuButtonImage",
        seeAlso:[],
        params:[
            {description:"Should the image be shown", optional:false, name:"show", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:MenuButton"
    },
    "method:CubeGrid.discardAllEdits":{
        ref:"method:CubeGrid.discardAllEdits",
        flags:"A",
        description:"\n",
        name:"discardAllEdits",
        seeAlso:[],
        params:[
            {description:"allows you to specify which cell(s) to drop                   edits for. Each record should be identified as an array containing \n                  <code>[rowNum,colNum]</code>",
             optional:true, name:"records", defaultValue:null, type:"array of 2-element arrays"},
            {description:"By default this method will hide the editor if                              it is currently showing for any row in the grid. Passing in\n                              this parameter will leave the editor visible (and just reset\n                              the edit values underneath the editor).",
             optional:true, name:"dontHideEditor", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "classAttr:Operators.equalsFieldTitle":{
        ref:"classAttr:Operators.equalsFieldTitle",
        flags:"IR",
        description:"Title for the \"equalsField\" operator",
        name:"equalsFieldTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"matches other field\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.selection":{
        ref:"attr:ListGrid.selection",
        flags:"RA",
        getter:["getSelection"],
        description:"The Selection object associated with the listGrid.",
        name:"selection",
        seeAlso:[],
        valueType:"Selection",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:ColorPicker.setBlue":{
        ref:"method:ColorPicker.setBlue",
        flags:"",
        description:" Sets the Blue element of the selected color\n",
        name:"setBlue",
        seeAlso:[
            {ref:"method:ColorPicker.getBlue", description:null}
        ],
        params:[
            {description:"An integer between 0 and 255", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "classAttr:Operators.notStartsWithTitle":{
        ref:"classAttr:Operators.notStartsWithTitle",
        flags:"IR",
        description:"Title for the \"notStartsWith\" operator",
        name:"notStartsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not start with (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:Canvas.edgeOffset":{
        ref:"attr:Canvas.edgeOffset",
        flags:"IR",
        description:"Amount the contained Canvas should be offset.  Defaults to edgeSize; set to less than\n edgeSize to allow the contained Canvas to overlap the edge and corner media.",
        name:"edgeOffset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('corners')}"],
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "class:Facet":{
        methods:[],
        ref:"class:Facet",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Analytics/CubeGrid"],
        attrs:["attr:Facet.id", "attr:Facet.title", "attr:Facet.values", "attr:Facet.rollupValue", 
         "attr:Facet.isTree","attr:Facet.showParentsLast", "attr:Facet.collapsed", "attr:Facet.canCollapse", 
         "attr:Facet.canMinimize","attr:Facet.width", "attr:Facet.align", "attr:Facet.titleAlign", "attr:Facet.cellAlign", 
         "attr:Facet.borderBefore","attr:Facet.borderAfter", "attr:Facet.selectionBoundary", "attr:Facet.summaryValue", 
         "attr:Facet.summaryTitle","attr:Facet.synchColumnLayout"],
        description:"Facet definition object made use of by the CubeGrid class.",
        name:"Facet",
        seeAlso:[],
        type:"class",
        requiresModules:["Analytics"]
    },
    "attr:StretchImg.imageType":{
        ref:"attr:StretchImg.imageType",
        flags:"IRW",
        description:"Indicates whether the image should be tiled/cropped, stretched, or centered when the\n          size of this widget does not match the size of the image. See ImageStyle for\n          details.",
        name:"imageType",
        seeAlso:[],
        valueType:"ImageStyle",
        defaultValue:"Img.STRETCH",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "attr:Canvas.maxHeight":{
        ref:"attr:Canvas.maxHeight",
        flags:"IRWA",
        description:"Maximum height that this Canvas can be resized to.",
        name:"maxHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10000",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:ValuesManager.isNewRecord":{
        ref:"method:ValuesManager.isNewRecord",
        flags:"",
        description:" Returns true if ${isc.DocUtils.linkForRef('attr:ValuesManager.saveOperationType')} is currently \"add\".  See\n ${isc.DocUtils.linkForRef('attr:ValuesManager.saveOperationType')}.\n\n",
        name:"isNewRecord",
        seeAlso:[],
        returns:{description:"whether this form will use an \"add\" operation when saving", type:"boolean"},
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "classMethod:Class.delayCall":{
        ref:"classMethod:Class.delayCall",
        flags:"",
        description:"  This is a helper to delay a call to a method on some target by a specified\n  amount of time.  Can be used to delay a call to a static method on this class by \n  ommiting the <code>target</code> parameter.\n",
        name:"delayCall",
        seeAlso:[],
        params:[
            {description:"name of the method to call", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"array of arguments to pass to the method in question", optional:true,
             name:"arrayArgs", defaultValue:null, type:"array"},
            {description:"Number of ms to delay the call by - defaults to zero (so just pulls                        execution of the method out of the current execution thread.",
             optional:true, name:"time", defaultValue:null, type:"number"},
            {description:"Target to fire the method on - if unspecified assume this is                          a call to a classMethod on this Class.",
             optional:true, name:"target", defaultValue:null, type:"object"}
        ],
        returns:{description:"Timer ID for the delayed call - can be passed to                       ${isc.DocUtils.linkForRef('classMethod:Timer.clear')} to cancel the call before it executes",
         type:"string"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:ListGrid.getCellErrors":{
        ref:"method:ListGrid.getCellErrors",
        flags:"",
        description:"  Returns the current set of errors for this cell. \n",
        name:"getCellErrors",
        seeAlso:[],
        params:[
            {description:"index of row to check for validation errors.", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"field to check for validation errors - can be fieldName                                           or index of the column.",
             optional:false, name:"fieldName", defaultValue:null, type:"string | number"}
        ],
        returns:{description:"array of error messages (strings) for the specified cell. If no validation                   errors are present, returns null.",
         type:"array"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:FormulaBuilder.instructionsTextStart":{
        ref:"attr:FormulaBuilder.instructionsTextStart",
        flags:"IRW",
        description:"Text label for the start of the instruction text that appears in the instructions label.",
        name:"instructionsTextStart",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"The following fields are available for use in this \"",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["i18nMessages"]
    },
    "attr:Button.iconSpacing":{
        ref:"attr:Button.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:TileGrid.styleName":{
        ref:"attr:TileGrid.styleName",
        flags:"IR",
        description:"Style for the overall TileGrid component.",
        name:"styleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"tileGrid\"",
        type:"attr",
        definingClass:"class:TileGrid",
        groups:["appearance"]
    },
    "attr:ListGrid.showTreeColumnPicker":{
        ref:"attr:ListGrid.showTreeColumnPicker",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans')} are in use, whether to show a hierarchical column picker\n that includes both headerSpans and normal headers, with normal headers indented under\n headerSpans similarly to how a ${isc.DocUtils.linkForRef('class:TreeGrid')} displays a Tree.\n <P>\n If <code>showTreeColumnPicker</code> is false, no column picker will be shown on the\n headerSpan itself, and the column picker for a clicked on a normal field header will include\n only normal fields.",
        name:"showTreeColumnPicker",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "attr:FormItem.rejectInvalidValueOnChange":{
        ref:"attr:FormItem.rejectInvalidValueOnChange",
        flags:"IRWA",
        description:"If validateOnChange is true, and validation fails for this item on change, with no suggested\n value, should we revert to the previous value, or continue to display the bad value entered\n by the user. May be set at the item or form level.",
        name:"rejectInvalidValueOnChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:ListGridField.valueField":{
        ref:"attr:ListGridField.valueField",
        flags:"IRW",
        description:"Specifies the ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')} field used to retrieve the stored\n values that are to be mapped to the display values (specified by\n ${isc.DocUtils.linkForRef('attr:ListGridField.displayField')}). Note that if this field is editable this will also\n be applied to this field's editors.",
        name:"valueField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "group:nonJavaBackend":{
        ref:"group:nonJavaBackend",
        title:".NET, PHP, Serverless Integration",
        treeLocation:["Concepts"],
        refs:[],
        description:"SmartClient is fully functional without installation of the Java server included in the SDK,\n and can be integrated with any server technology.  This topic provides pointers to\n documentation that is most relevant for this type of integration.\n <P>\n <h4>Installation</h4>\n <P>\n As described in ${isc.DocUtils.linkForRef('group:iscInstall','<i>Deploying SmartClient</i>')}, for a client-only\n integration, installation consists of just copying a directory of JavaScript and media files\n to your webserver.\n <P>\n <h4>Creating Components</h4>\n <P>\n SmartClient components can be included in any .html page, including dynamically generated pages\n produced by .php or .asp files.  The SmartClient libraries can be included in the page as\n follows:\n <pre>\n &lt;HTML&gt;&lt;HEAD&gt;\n &lt;SCRIPT&gt;var isomorphicDir=\"../isomorphic/\";&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_Core.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_Foundation.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_Containers.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_Grids.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_Forms.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/system/modules/ISC_DataBinding.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=../isomorphic/skins/SmartClient/load_skin.js&gt;&lt;/SCRIPT&gt;\n &lt;/HEAD&gt;&lt;BODY&gt;\n ...</pre>\n SmartClient components can then be created via normal JavaScript:\n <pre>\n &lt;SCRIPT&gt;\n isc.Button.create({\n     title:\"Button\",\n     click:\"isc.say('Hello World')\"\n });\n &lt;/SCRIPT&gt;</pre>\n This approach is discussed in more detail in the \n <a href='/docs/SmartClient_Quick_Start_Guide.pdf' onclick=\"window.open('/docs/SmartClient_Quick_Start_Guide.pdf');return false;\">QuickStart Guide</a>, Chapter 4, <i>Coding</i>.\n Note that JavaScript-based component instantiation is currently the recommended approach, and\n most examples are provided in the JavaScript format.\n <P>\n <h4>Data Loading / Data Binding</h4>\n <P>\n The primary focus of SmartClient integration is connecting DataSource operations to your\n server.  The ${isc.DocUtils.linkForRef('group:clientDataIntegration','Client-side Data Integration')} chapter covers\n the key approaches, including cookbook approaches for REST-based integration with any server\n that can return XML or JSON over HTTP.\n <P>\n <h4>Simple RPCs (non-DataSource requests)</h4>\n <P>\n You can implement simple RPCs as web service operations: use\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')} to load the service definition, and then use\n ${isc.DocUtils.linkForRef('method:WebService.callOperation')} to call the operations.  Note that some server frameworks\n allow the generation of WSDL from server method signatures.\n <P>\n Alternatively, if your backend is capable of outputting \n <a href='http://www.json.org/' onclick=\"window.open('http://www.json.org/');return false;\">JSON</a> (JavaScript Object Notation), you can use \n ${isc.DocUtils.linkForRef('attr:RPCRequest.evalResult')} to directly turn JSON results into live JavaScript objects.\n ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} lets you load arbitrary server results, including\n JSON results that need to be processed before they can be eval()'d.\n <P>\n <h4>HTTPProxy: Cross-site or cross-port data loading</h4>\n <P>\n If you develop a prototype using the SmartClient SDK and SmartClient Java Server, and then\n you migrate the prototype to another server technology, you need to be aware that the\n SmartClient Java Server includes an HTTPProxy servlet that allows SmartClient interfaces to\n contact servers other than the origin server (bypassing what is called the\n <a href='http://www.google.com/search?q=same+origin+policy' onclick=\"window.open('http://www.google.com/search?q=same+origin+policy');return false;\">\"same origin policy\"</a>).\n <P>\n SmartClient uses the HttpProxy automatically when needed, so it may not be obvious that the\n HTTPProxy is in use.  Then, your migrated application will encounter errors attempting to\n contact the HTTPProxy servlet.\n <P>\n To avoid these errors, ensure that all services that your application uses are accessed\n using the same hostname and port as the page was loaded from.  In particular, watch for WSDL\n files, which contain the service URL - you may need to use ${isc.DocUtils.linkForRef('method:WebService.setLocation')}\n to ensure that the web service URL and page URL match.\n <P>\n If your production application really does need to access services or content hosted on\n other servers, typical practice is to pursue normal SmartClient integration with your\n server, then write server-side code that contacts other hosts on behalf of your SmartClient\n interface.",
        name:"nonJavaBackend",
        seeAlso:[],
        type:"group"
    },
    "group:adminConsole":{
        ref:"group:adminConsole",
        title:"Admin Console",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        refs:[],
        description:"The Admin Console allows you to configure database access for DataSources that use\n SmartClient's built-in ${isc.DocUtils.linkForRef('group:sqlDataSource','SQL engine')}.\n <P>\n NOTE: You must have the Isomorphic SmartClient package installed and your servlet engine\n started. \n <p>\n Direct your browser to the following URL to access the Admin Console: \n <p>\n &nbsp;&nbsp;<a href='http://localhost:8080/tools/adminConsole.jsp' onclick=\"window.open('http://localhost:8080/tools/adminConsole.jsp');return false;\">http://localhost:8080/tools/adminConsole.jsp</a>\n <p>\n The common default servlet engine port 8080 is used in the URL given above. Adjust your URL\n as necessary if you are using a different port and replace localhost with the machine name\n running the servlet engine if you are accessing it from a remote machine.\n <p>\n Note that the Admin Console interface is also available in the Developer Console as the\n \"DataBases\" tab.\n <P>\n <b>Test Data</b>\n <p>\n You can create a test file that contains a sample dataset which can be imported into your\n database table with the Admin Console.\n <p>\n The test file to use with your DataSource is specified in the <code>testFileName</code>\n DataSource configuration property. The test file uses the extension .data.xml.\n <p>\n The test data file should consist of a top-level &lt;List&gt; element containing a series of XML\n elements named after your DataSource's ID, each of which creates one DataSource\n record. Values for each field are given within tags named after the field name. \n <p>\n For example, the following XML is from the supplyItem.data.xml test data file supplied with\n the Isomorphic SmartClient package. This file is located in\n [webroot]/examples/shared/ds/test_data/.\n <p>\n <pre>\n &lt;List&gt;\n  &lt;supplyItem&gt;\n      &lt;description&gt;A revolutionary cushion-grip ballpoint pen that reduces \n          required gripping power, relieving stress and alleviating writing \n          fatigue. Excellent for people who suffer from arthritis or carpal \n          tunnel syndrome. Medium point, black ink. Refillable.&lt;/description&gt;\n      &lt;category&gt;1&lt;/category&gt;\n      &lt;itemRef&gt;ODC 204-502-153&lt;/itemRef&gt;\n      &lt;maxQuantity&gt;5&lt;/maxQuantity&gt;\n      &lt;requiresJustification&gt;0&lt;/requiresJustification&gt;\n      &lt;itemName&gt;Dr. Grip Pens -- Blue Barrel&lt;/itemName&gt;\n      &lt;itemID&gt;1&lt;/itemID&gt;\n      &lt;unitCost&gt;4.99&lt;/unitCost&gt;\n  &lt;/supplyItem&gt;\n  &lt;supplyItem&gt;\n      &lt;description&gt;A revolutionary cushion-grip ballpoint pen that reduces \n          required gripping power, relieving stress and alleviating writing \n          fatigue. Excellent for people who suffer from arthritis or carpal \n          tunnel syndrome. Medium point, black ink. Refillable.&lt;/description&gt;\n      &lt;category&gt;1&lt;/category&gt;\n      &lt;itemRef&gt;ODC 204-708-834&lt;/itemRef&gt;\n      &lt;maxQuantity&gt;5&lt;/maxQuantity&gt;\n      &lt;requiresJustification&gt;0&lt;/requiresJustification&gt;\n      &lt;itemName&gt;Dr. Grip Pens -- Black Barrel&lt;/itemName&gt;\n      &lt;itemID&gt;2&lt;/itemID&gt;\n      &lt;unitCost&gt;4.99&lt;/unitCost&gt;\n  &lt;/supplyItem&gt;\n  &lt;supplyItem&gt;\n      &lt;description&gt;Personalized business cards for all your networking \n          needs.&lt;/description&gt;\n      &lt;category&gt;2&lt;/category&gt;\n      &lt;itemRef&gt;&lt;/itemRef&gt;\n      &lt;maxQuantity&gt;500&lt;/maxQuantity&gt;\n      &lt;requiresJustification&gt;1&lt;/requiresJustification&gt;\n      &lt;itemName&gt;Personalized business cards -- 500 count&lt;/itemName&gt;\n      &lt;itemID&gt;3&lt;/itemID&gt;\n      &lt;unitCost&gt;25.00&lt;/unitCost&gt;\n  &lt;/supplyItem&gt;\n  ...\n &lt;List/&gt;\n </pre>\n \n Data for a tree-like DataSource can be specified with the same format.\n The following code example is from the supplyCategory.data.xml test data file. This file\n is also located in [webroot]/examples/shares/ds/test_data/. \n\n <pre>\n &lt;List&gt;\n     &lt;supplyCategory&gt;\n     	&lt;itemName&gt;Office Paper Products&lt;/itemName&gt;\n     	&lt;parentID&gt;root&lt;/parentID&gt;\n     &lt;/supplyCategory&gt;\n     &lt;supplyCategory&gt;\n     	&lt;itemName&gt;Calculator Rolls&lt;/itemName&gt;\n     	&lt;parentID&gt;Office Paper Products&lt;/parentID&gt;\n     &lt;/supplyCategory&gt;\n     &lt;supplyCategory&gt;\n     	&lt;itemName&gt;Adding Machine/calculator Roll&lt;/itemName&gt;\n     	&lt;parentID&gt;Calculator Rolls&lt;/parentID&gt;\n     &lt;/supplyCategory&gt;\n     . . .\n &lt;/List&gt;\n </pre>\n \n Notice that all records must define values for the itemName primary key field and for the\n parentID field that establishes the tree relationship.",
        name:"adminConsole",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "method:CubeGrid.facetValueHover":{
        ref:"method:CubeGrid.facetValueHover",
        flags:"",
        description:"    StringMethod handler fired when mouse hovers over a facetValue button in a header.\n  <br><i>methodType</i> handler\n",
        name:"facetValueHover",
        seeAlso:[],
        params:[
            {description:"facetValuesObject for the appropriate header button", optional:false,
             name:"facetValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "method:TextItem.setKeyPressFilter":{
        ref:"method:TextItem.setKeyPressFilter",
        flags:"",
        description:" Set the keyPressFilter for this item\n",
        name:"setKeyPressFilter",
        seeAlso:[],
        params:[
            {description:"new keyPress filter for the item", optional:false, name:"filter",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:TextItem"
    },
    "classMethod:isc.logEchoAll":{
        ref:"classMethod:isc.logEchoAll",
        flags:"",
        description:" Logs the echoed object (using ${isc.DocUtils.linkForRef('classMethod:isc.echoAll')}) as a warning, prefixed with an\n optional message.\n\n",
        name:"logEchoAll",
        seeAlso:[
            {ref:"classMethod:Log.logWarn", description:"for logging info"}
        ],
        params:[
            {description:"object to echo", optional:false, name:"value", defaultValue:null, type:"any"},
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:DynamicForm.errorOrientation":{
        ref:"attr:DynamicForm.errorOrientation",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true, where should the error icon and text appear\n relative to form items?  Valid options are <code>\"top\"</code>, \n <code>\"bottom\"</code>, <code>\"left\"</code> or <code>\"right\"</code>.<br>\n May be overridden at the item level via ${isc.DocUtils.linkForRef('attr:FormItem.errorOrientation')}.",
        name:"errorOrientation",
        seeAlso:[],
        valueType:"align",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation", "appearance"]
    },
    "attr:DataSource.operationBindings":{
        ref:"attr:DataSource.operationBindings",
        flags:"IR",
        description:"Optional array of OperationBindings, which provide instructions to the DataSource about how each\n DSOperation is to be performed.\n <P>\n When using the SmartClient Server, OperationBindings are specified in your DataSource\n descriptor (.ds.xml file) and control server-side behavior such as what Java object to route\n DSRequest to (${isc.DocUtils.linkForRef('attr:OperationBinding.serverObject')}) or customizations to SQL / HQL queries\n (${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} and ${isc.DocUtils.linkForRef('attr:OperationBinding.customHQL')}).  See the \n ${isc.DocUtils.linkForExampleId('javaDataIntegration', 'Java Integration samples')}.\n <P>\n For DataSources bound to WSDL-described web services using\n ${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')}, OperationBindings are used to bind each DataSource\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} to an\n ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','operation')} of a WSDL-described \n ${isc.DocUtils.linkForRef('class:WebService','web service')}, so that a DataSource can both fetch and save data to a web\n service.\n <P>\n For example, this code accomplishes part of the binding to the \n <a href='http://www.google.com/search?q=sforce+partner+wsdl' onclick=\"window.open('http://www.google.com/search?q=sforce+partner+wsdl');return false;\">SalesForce partner web services</a>\n <pre>\n isc.DataSource.create({\n    serviceNamespace : \"urn:partner.soap.sforce.com\",\n    operationBindings : [\n        { operationType:\"fetch\", wsOperation:\"query\", recordName: \"sObject\" },\n        { operationType:\"update\", wsOperation:\"update\", recordName: \"SaveResult\" },\n        { operationType:\"add\", wsOperation:\"create\", recordName: \"SaveResult\" },\n        { operationType:\"remove\", wsOperation:\"delete\", recordName: \"DeleteResult\" }\n    ],\n    ...\n }); \n </pre>\n NOTE: additional code is required to handle authentication and other details, see the\n complete code in isomorphicSDK/examples/databinding/SalesForce.\n <P>\n For DataSources that contact non-WSDL-described XML or JSON services, OperationBindings can\n be used to separately configure the URL, HTTP method, input and output processing for each\n operationType.  This makes it possible to fetch JSON data from one URL for the \"fetch\"\n operationType and save to a web service for the \"update\" operationType, while appearing as a\n single integrated DataSource to a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} such as an\n ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit','editable ListGrid')}.\n <P>\n If no operationBinding is defined for a given DataSource operation, all of the properties\n which are valid on the operationBinding are checked for on the DataSource itself.  \n <P>\n This also means that for a read-only DataSource, that is, a DataSource only capable of fetch\n operations, operationBindings need not be specified, and instead all operationBinding\n properties can be set on the DataSource itself.  An example of using OperationBinding\n properties directly on the DataSource in order to read an RSS feed can be found here:\n <P>\n ${isc.DocUtils.linkForStandaloneExample('/examples/databinding/rss_databinding.html', '/examples/databinding/rss_databinding.html')}",
        name:"operationBindings",
        seeAlso:[
            {ref:"class:OperationBinding", description:null}
        ],
        valueType:"Array of OperationBinding",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:Canvas.hoverMoveWithMouse":{
        ref:"attr:Canvas.hoverMoveWithMouse",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, should this widget's hover canvas be moved with\n the mouse while visible?",
        name:"hoverMoveWithMouse",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:ListGridField.editorExit":{
        ref:"method:ListGridField.editorExit",
        flags:"A",
        description:" Callback fired when the user attempts to navigate away from the current edit cell, \n or complete the current edit.<br>\n Return false from this method to cancel the default behavior (Saving / cancelling the\n current edit / moving to the next edit cell)\n\n",
        name:"editorExit",
        seeAlso:[
            {ref:"method:ListGrid.editorExit", description:null}
        ],
        params:[
            {description:"What interaction triggered this                                                          edit cell exit",
             optional:false, name:"editCompletionEvent", defaultValue:null, type:"EditCompletionEvent"},
            {description:"record for the cell being edited", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"new value for the cell being edited", optional:false, name:"newValue",
             defaultValue:null, type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"},
            {description:"ListGrid to which this field belongs", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"Returning false from this method will cancel the default behavior                      (for example saving the row) and leave the editor visible and focus\n                      in this edit cell.",
         type:"boolean"},
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:LoginDialog.lostPasswordItem":{
        ref:"attr:LoginDialog.lostPasswordItem",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('class:LinkItem')} to page requesting forgotten password in ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.\n <p>To handle user clicks on this link, implement ${isc.DocUtils.linkForRef('method:LoginDialog.lostPassword')}.\n <p>To handle a user click as a physical link to another page, set\n ${isc.DocUtils.linkForRef('linkItem.defaultValue')} via loginDialog.lostPasswordItemProperties:\n <code>\n lostPasswordItemProperties: {\n     defaultValue: \"register.html\"\n },\n </code>",
        name:"lostPasswordItem",
        seeAlso:[
            {ref:"attr:LoginDialog.showLostPasswordLink", description:null},
            {ref:"attr:LoginDialog.lostPasswordItemTitle", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:ResultSet.criteria":{
        ref:"attr:ResultSet.criteria",
        flags:"IRW",
        getter:["getCriteria"],
        description:"Filter criteria used whenever records are retrieved.\n <P>\n Use ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria')} to change the criteria after initialization.",
        name:"criteria",
        setter:["setCriteria"],
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "attr:OperationBinding.requires":{
        ref:"attr:OperationBinding.requires",
        flags:"IR",
        description:"Indicates that the specified ${isc.DocUtils.linkForRef('type:VelocityExpression')} must be true for a user to access\n this operationBinding.\n <P>\n As with ${isc.DocUtils.linkForRef('attr:OperationBinding.requiresRole')}, if there an operationBinding that is the\n default operationBinding for the operationType, its <code>requires</code> expression is\n assumed to apply to all other operationBindings of the same type unless they explicitly set\n <code>requires=\"\"</code>\n <P>\n ${isc.DocUtils.linkForRef('attr:DataSource.requires')}, if specified, applies before\n <code>operationBinding.requires</code> is evaluated.  In this case, both <code>requires</code>\n expressions must be true for the request to be accepted.",
        name:"requires",
        seeAlso:[],
        valueType:"VelocityExpression",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["auth"]
    },
    "method:TextItem.getHint":{
        ref:"method:TextItem.getHint",
        flags:"A",
        description:"	Returns the hint text for this item. Default implementation returns ${isc.DocUtils.linkForRef('attr:FormItem.hint')}, or\n  null if there is no hint to show.\n\n",
        name:"getHint",
        seeAlso:[],
        returns:{description:"HTML to show as the hint for the item", type:"HTML"},
        type:"method",
        definingClass:"class:TextItem",
        groups:["appearance"]
    },
    "attr:Applet.archive":{
        ref:"attr:Applet.archive",
        flags:"IR",
        description:"URL to the applet JAR or ZIP archive.",
        name:"archive",
        seeAlso:[
            {ref:"attr:Applet.code", description:null},
            {ref:"attr:Applet.codeBase", description:null}
        ],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "method:ListGrid.invalidateCache":{
        ref:"method:ListGrid.invalidateCache",
        flags:"",
        description:" Invalidate the current data cache for this databound component via a call to\n <code>this.data.invalidateCache()</code>. If necessary, this will cause a new fetch to \n be performed with the current set of criteria for this component.\n <P>\n Has no effect if this component is not showing a set of filtered data.\n \n",
        name:"invalidateCache",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:DynamicForm.getValues":{
        ref:"method:DynamicForm.getValues",
        flags:"",
        description:" An Object containing the values of the form as properties, where each propertyName is\n the name of a ${isc.DocUtils.linkForRef('group:Items','form item')} in the form, and each property value is the value\n held by that form item.\n\n",
        name:"getValues",
        seeAlso:[],
        returns:{description:"values in the form", type:"Object"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ToolStrip.separatorSize":{
        ref:"attr:ToolStrip.separatorSize",
        flags:"IR",
        description:"Separator thickness in pixels",
        name:"separatorSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"8",
        type:"attr",
        definingClass:"class:ToolStrip"
    },
    "method:Window.setAutoSize":{
        ref:"method:Window.setAutoSize",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:Window.autoSize')}\n",
        name:"setAutoSize",
        seeAlso:[],
        params:[
            {description:"true if the window should auto-size to its content", optional:false,
             name:"autoSize", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "group:metadata":{
        ref:"group:metadata",
        refs:["method:CubeGrid.setFacetTitle", "method:CubeGrid.setFacetValueTitle"],
        description:null,
        name:"metadata",
        seeAlso:[],
        type:"group"
    },
    "object:Tab":{
        methods:[],
        ref:"object:Tab",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/TabSet"],
        attrs:["attr:Tab.title", "attr:Tab.pickerTitle", "attr:Tab.pane", "attr:Tab.ID", "attr:Tab.width", 
         "attr:Tab.disabled","attr:Tab.icon", "attr:Tab.canClose", "attr:Tab.closeIcon"],
        description:"Tabs are specified as objects, not class instances.  For example, when\n developing in JavaScript, a typical initialization block for a TabSet would look like\n this:\n <pre>\n TabSet.create({\n     tabs: [\n         {title: \"tab1\", pane: \"pane1\"},\n         {title: \"tab2\"}\n     ]\n });\n </pre>\n And in XML:\n <pre>\n &lt;TabSet&gt;\n    &lt;tabs&gt;\n        &lt;Tab title=\"tab1\" pane=\"pane1\"/&gt;\n        &lt;Tab title=\"tab2\"/&gt;\n    &lt;/tabs&gt;\n &lt;/TabSet&gt;\n </pre>",
        name:"Tab",
        seeAlso:[],
        type:"object"
    },
    "attr:Canvas.edgeBackgroundColor":{
        ref:"attr:Canvas.edgeBackgroundColor",
        flags:"IR",
        description:"Background color for the EdgedCanvas created to decorate this component.  This can be used\n to provide an underlying \"tint\" color for translucent edge media",
        name:"edgeBackgroundColor",
        seeAlso:[],
        valueType:"Color",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "method:Toolbar.itemClick":{
        ref:"method:Toolbar.itemClick",
        flags:"A",
        description:"	Called when one of the buttons recieved a click event\n",
        name:"itemClick",
        seeAlso:[],
        params:[
            {description:"pointer to the button in question", optional:false, name:"item",
             defaultValue:null, type:"button"},
            {description:"number of the button in question", optional:false, name:"itemNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Toolbar",
        groups:["event handling"]
    },
    "attr:DetailViewerField.showFileInline":{
        ref:"attr:DetailViewerField.showFileInline",
        flags:"IR",
        description:"For a field of type:\"imageFile\", indicates whether to stream the image and display it\n inline or to display the View and Download icons.",
        name:"showFileInline",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "classMethod:EventHandler.altKeyDown":{
        ref:"classMethod:EventHandler.altKeyDown",
        flags:"",
        description:"			Return true if the alt (option) key is being held down.\n			Note that this is only set reliably for keyboard events.\n\n",
        name:"altKeyDown",
        seeAlso:[],
        returns:{description:"true == alt key is down", type:"boolean"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "method:GridRenderer.getDrawnRows":{
        ref:"method:GridRenderer.getDrawnRows",
        flags:"",
        description:" Get the rows that are currently drawn (exist in the DOM), as an array of [firstRowNum,\n lastRowNum].  \n <P>\n The drawn rows differ from the ${isc.DocUtils.linkForRef('method:GridRenderer.getVisibleRows','visibleRows')} because of\n ${isc.DocUtils.linkForRef('attr:GridRenderer.drawAheadRatio','drawAhead')}.  The drawn rows are the apppropriate range to consider if\n you need to, eg, using ${isc.DocUtils.linkForRef('method:GridRenderer.refreshCell')} to update all the cells in a column.\n <P>\n If the grid is undrawn or the ${isc.DocUtils.linkForRef('attr:GridRenderer.emptyMessage')} is currently shown, returns\n [null,null];\n\n",
        name:"getDrawnRows",
        seeAlso:[],
        returns:{description:null, type:"Array"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "method:DataBoundComponent.enableHilite":{
        ref:"method:DataBoundComponent.enableHilite",
        flags:"",
        description:" Enable / disable a ${isc.DocUtils.linkForRef('attr:DataBoundComponent.hilites','hilite')}\n\n",
        name:"enableHilite",
        seeAlso:[],
        params:[
            {description:"ID of hilite to enable", optional:false, name:"hiliteID", defaultValue:null,
             type:"string"},
            {description:"new enabled state to apply - if null, defaults to true", optional:true,
             name:"enable", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["hiliting"]
    },
    "group:valueMap":{
        ref:"group:valueMap",
        refs:["method:FormItem.getDisplayValue", "method:FormItem.setValueMap", 
         "method:CheckboxItem.setValueMap","attr:FormItem.valueMap", "attr:CheckboxItem.valueMap"],
        description:"A ${isc.DocUtils.linkForRef('type:ValueMap')} defines the set of legal values for a field, and optionally allows you to provide\n a mapping from stored values to values as seen by the end user.",
        name:"valueMap",
        seeAlso:[],
        type:"group"
    },
    "attr:TextAreaItem.textBoxStyle":{
        ref:"attr:TextAreaItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class to apply to this item's input element.\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"textItem\"",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["appearance"]
    },
    "attr:Facet.id":{
        ref:"attr:Facet.id",
        flags:"IR",
        description:"id of this facet.  Any string or number.",
        name:"id",
        seeAlso:[],
        valueType:"String | Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "class:DMI":{
        methods:[],
        ref:"class:DMI",
        classAttrs:[],
        classMethods:["classMethod:DMI.call"],
        treeLocation:["Client Reference/RPC"],
        attrs:[],
        description:"Direct Method Invocation (DMI) allows background HTTP requests to directly\n invoke methods on server-side objects via XML configuration.\n <P>\n DMI is an alternative to the ${isc.DocUtils.linkForRef('classAttr:RPCManager.actionURL')} approach where your server code\n receives a generalized ${isc.DocUtils.linkForRef('class:RPCRequest','request object')} which you route to appropriate\n methods yourself.  Which interface (DMI or RPCManager) you choose is largely a matter of\n preference - they provide equivalent functionality.  Note that there are also several\n approaches for ${isc.DocUtils.linkForRef('group:nonJavaBackend','non-Java backends')} and/or Java backends not\n running the ISC server.\n <p>\n When using DMI, inbound request data is translated to Java objects and passed as method\n parameters to the designated method, with available request data matched to each declared\n parameter by Java type.  The return value of your method is automatically wrapped as a valid\n response and delivered to the browser.\n <P>\n <u><b>DataSource DMI</b></u>\n <br>\n To enable DMI for a given DataSource, simply include a <code>&lt;serverObject&gt;</code>\n configuration block in that DataSource's configuration either at\n ${isc.DocUtils.linkForRef('attr:DataSource.serverObject')} or on a particular operationBinding via\n ${isc.DocUtils.linkForRef('attr:OperationBinding.serverObject')}.  The ServerObject specifies the target of the method\n invocation and ${isc.DocUtils.linkForRef('attr:OperationBinding.serverMethod')} specifies the method that will be\n called.\n <P>\n For example, the following Datasource DMI declaration would route \"fetch\" operations for\n this DataSource to the method \"fetch\" on an object stored in the servlet session under the\n name \"beanFetcher\":\n <pre>\n &lt;DataSource&gt;\n   &lt;operationBindings&gt;\n       &lt;binding operationType=\"fetch\" serverMethod=\"fetch\"&gt;\n           &lt;serverObject  \n                lookupStyle=\"attribute\" \n                attributeScope=\"session\" \n                attributeName=\"beanFetcher\"/&gt;\n       &lt;/binding&gt;\n   &lt;/operationBindings&gt;\n   ...\n &lt;/DataSource&gt;\n </pre>\n Method overloading is not supported - there must be exactly one method on the target class\n with the name specified in ${isc.DocUtils.linkForRef('attr:OperationBinding.serverMethod')}.  The method must be public,\n but can be either an instance or static method.  If no operationBinding is specified or the\n operationBinding does not specify a <code>serverMethod</code> then it defaults to the name of\n the operation (eg \"fetch\").\n <p>\n By default, the DSResponse data sent back by DataSource DMIs is filtered to just the set of\n fields specified on the DataSource.  This allows you to simply return beans that potentially\n have getter methods for fields other than are defined in the DataSource without that\n (potentially private) data being sent to the client.  If you want to disable this\n functionality, you can do so on a per-operation basis by setting\n ${isc.DocUtils.linkForRef('attr:ServerObject.dropExtraFields')}, on a per-DataSource level by setting\n ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')}, or globally by setting the config parameter\n <code>DMI.dropExtraFields</code> to <code>false</code> in\n <code>[webroot]/WEB-INF/classes/server.properties</code>.  Non-DMI DSResponse data is, by\n default, not filtered in this manner for backward compatibility reasons.  If you want to\n enable this type of filtering for non-DMI DSResponse data, you can do so by setting the\n config parameter <code>DSResponse.dropExtraFields</code> to <code>true</code> in\n <code>[webroot]/WEB-INF/classes/server.properties</code>.  <code>DMI.dropExtraFields</code>\n and <code>DSResponse.dropExtraFields</code> can be enabled/disabled independently of each\n other - that is, setting one does not side-effect the other.  <code>server.properties</code>\n settings can be overridden by an explicit setting in ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')} which\n in turn can be overridden by an explicit setting in ${isc.DocUtils.linkForRef('attr:ServerObject.dropExtraFields')} (this\n last one for DMI only since non-DMI operations don't have a serverObject context).\n <p>\n <u><b>DataSource DMI and regular RPCManager dispatch</b></u><br>\n It is possible to use DMI to incorporate your own code into what is otherwise the regular \n process flow of an ordinary, non-DMI DataSource request.  This is particularly valuable if\n you are using the built-in SQL or Hibernate DataSources, because it allows you to inject\n extra functionality (validations, processing steps, messages to other systems, anything you\n can code) into a fetch or update request that is otherwise handled for you by the SmartClient \n Server.\n <p>\n To do this, just configure an operationBinding for DMI, as described above.  Then, in your\n server-side method, invoke <code>execute()</code> on the <code>DSRequest</code> your method\n is passed.  If you create a DMI method that does nothing <b>but</b> invoke\n <code>dsRequest.execute()</code>, then you have a DMI method that behaves exactly like the\n normal RPCManager dispatch code.  Customizing \"normal RPCManager dispatch code\" is now a \n simple matter of adding logic to your DMI method.  See \n ${isc.DocUtils.linkForExampleId('userSpecificData', 'this example')} of this technique in the Feature Explorer.\n <p>\n <u><b>RPC DMI</b></u>\n <br>\n RPC DMI makes a set of methods from a server-side class available as client-side methods for\n direct invocation.  RPC DMI also uses a ${isc.DocUtils.linkForRef('class:ServerObject')} configuration block to specify\n the server-side DMI end-point, but in the case of RPCs, the ${isc.DocUtils.linkForRef('class:ServerObject')} definition\n goes into an <code>rpcBindings</code> section of an <code>Application</code> definition in a\n .app.xml file.  For an example, see the <code>example.app.xml</code> file in the /shared/app\n directory of the SmartClient SDK.  The only difference between the RPC DMI\n ServerObject definition and the DataSource DMI version is the addition of the\n ${isc.DocUtils.linkForRef('attr:ServerObject.visibleMethods')} block that specifies which methods are callable on this\n ServerObject.  This section is not consulted for DataSource DMIs because the\n ${isc.DocUtils.linkForRef('attr:OperationBinding.serverMethod')} is used to specify the callable method in that case.\n <p>\n <u><b>Method Invocation</b></u>\n <br>\n SmartClient can pass a set of stock context variables to your DMI method and also performs\n some type adaptation logic to make this interface more flexible.  For DataSource DMI, you\n can declare your method to take any number of the following types of arguments and they will\n be passed to you:\n <ul>\n <li>HttpServletRequest\n <li>HttpServletResponse\n <li>ServletContext\n <li>HttpSession\n <li>RPCManager\n <li>DSRequest\n <li>RequestContext (from com.isomorphic.servlet)\n <li>DataSource (same as DSRequest.getDataSource())\n <li>Map (same as DSRequest.getValues())\n <li>Bean (auto-populated from DSRequest.getValues())\n </ul>\n DataSource DMI methods can return any of the following types of values:\n <ul>\n <li>DSResponse (used as the DSResponse verbatim)\n <li>List (valid response to a fetch operation - gets auto-popuplated into a DSResponse for\n you via setData())\n <li>Map or Bean (valid response to add, update, remove operations - gets auto-populated\n into a DSResponse for you via setData()).\n </ul>\n Note that to take advantage of some SmartClient features like paging and custom validation,\n you need to return a DSResponse and provide the required metadata (like\n startRow/endRow/totalRows for paging).  You can simply return a <code>List</code> instead,\n but this won't work for large datasets.\n <p>\n So, for example, all of the following DataSource DMI method signatures are valid:\n <pre>\n public List myFetch(Map criteria)\n public List myFetch(SupplyItem criteria)\n public DSResponse myAdd(HttpSession session, \n                         DataSource ds, \n                         DSRequest dsRequest)\n </pre>\n <p>\n See\n <a href='/examples/server_integration/#customDataSourceIntegrationDMI' onclick=\"window.open('/examples/server_integration/#customDataSourceIntegrationDMI');return false;\">the supplyItemDMI example</a>\n for an example of DataSource DMI.\n <p>\n <p>\n RPC DMIs work slighly differently.  Unlike DataSource DMIs, RPC DMIs can have an arbitrary\n number of required arguments, and also some optional context arguments.  For example, let's\n say you call a method from the client like so (note that there's a cleaner way to invoke\n DMIs if you use the ${isc.DocUtils.linkForRef('group:loadDMIStubsTag')} JSP tag):\n <pre>\n DMI.call(\"myApp\", \"com.foo.MyClass\", \"doSomething\",\n          1, \"zoo\", [1, 2, 3], \"clientCallback()\");\n </pre>\n The server-side implementation of method <code>doSomething</code> must take a least three\n arguments of the type used above - specifically a Number, String, and List.  SmartClient\n will try to adapt arguments where possible - so for example the first argument can be a Long\n or an Integer instead and the invocation will still work.  Also, an object literal passed\n from the client becomes a Map on the server and will be automatically applied to a bean if\n the method argument takes a Bean in that position.  See ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} for a table\n of type conversions.  You can use native types in the server-side signature for things like\n Integer, Long, etc - so e.g. you can specify your method taking an int or long. In addition\n to the required arguments, you can pass the following optional arguments:\n <ul>\n <li>HttpServletRequest\n <li>HttpServletResponse\n <li>HttpSession\n <li>RPCManager\n <li>RPCRequest\n </ul>\n See\n <a href='/examples/server_integration/#genericRPCIntegrationDMI' onclick=\"window.open('/examples/server_integration/#genericRPCIntegrationDMI');return false;\">the getTimeStampDMI example</a>\n for an example of RPC DMI.",
        name:"DMI",
        seeAlso:[
            {ref:"group:loadDMIStubsTag", description:null},
            {ref:"class:ServerObject", description:null},
            {ref:"attr:DataSource.serverObject", description:null},
            {ref:"attr:OperationBinding.serverObject", description:null},
            {ref:"group:clientServerIntegration", description:null}
        ],
        type:"class",
        requiresModules:["SCServer"]
    },
    "classAttr:Operators.iEndsWithTitle":{
        ref:"classAttr:Operators.iEndsWithTitle",
        flags:"IR",
        description:"Title for the \"iEndsWith\" operator",
        name:"iEndsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ends with\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ComboBoxItem.textMatchStyle":{
        ref:"attr:ComboBoxItem.textMatchStyle",
        flags:"IR",
        description:"When applying filter criteria to pickList data, what type of matching to use.  Legal\n values are \"substring\" (value contains user input) or \"startsWith\" (value starts with\n user input.  Both matches are case insensitive.\n <P>\n For a databound pickList (${isc.DocUtils.linkForRef('attr:ComboBoxItem.optionDataSource')} set), <code>textMatchStyle</code> is\n sent to the server as ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')}.\n <P>\n For a non-databound pickList, <code>textMatchStyle</code> is applied by \n ${isc.DocUtils.linkForRef('method:ComboBoxItem.filterClientPickListData')}.",
        name:"textMatchStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"startsWith\"",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:Canvas.setPageTop":{
        ref:"method:Canvas.setPageTop",
        flags:"",
        description:" Set the page-relative top coordinate of this widget.\n \n",
        name:"setPageTop",
        seeAlso:[],
        params:[
            {description:"new top coordinate in pixels", optional:false, name:"top", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:ComboBoxItem.sortField":{
        ref:"attr:ComboBoxItem.sortField",
        flags:"IR",
        description:"Specifies the field by which this item should be initially sorted.  Can be set to \n either a ${isc.DocUtils.linkForRef('attr:ListGridField.name','field name')} or the index of the field in the fields \n Array. Note that if <code>sortField</code> is initally specified as a number, it will be\n converted to a string (field name) after the item is initialized.\n <P>\n To programmatically change sort field or direction after initialization, call ${isc.DocUtils.linkForRef('sort')}.",
        name:"sortField",
        seeAlso:[],
        valueType:"String or integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        definingClass:"class:ComboBoxItem",
        groups:["sorting"]
    },
    "method:Dialog.okClick":{
        ref:"method:Dialog.okClick",
        flags:"",
        description:" Handle a click on the 'ok' button of this Dialog.\n Default implementation is to call <code>saveData()</code>, hide the Dialog, then return\n <code>true</code>.  \n Override to do something else.\n",
        name:"okClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:DSRequest.sortBy":{
        ref:"attr:DSRequest.sortBy",
        flags:"IR",
        description:"Array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s on which the DSRequest should sort it's data.  For example, \n to sort by the field \"department\" in ascending order, followed by the field \"userName\" in \n descending order, pass the following SortSpecifier array:\n <P>\n <code>\n [<br>\n     { property: \"department\", direction: \"ascending\" },<br>\n     { property: \"userName\", direction: \"descending\" }<br>\n ]\n </code>",
        name:"sortBy",
        seeAlso:[],
        valueType:"Array of SortSpecifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:ListGrid.rowOver":{
        ref:"method:ListGrid.rowOver",
        flags:"",
        description:" Called when the mouse pointer enters a row\n\n",
        name:"rowOver",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:CubeGrid.getFacetValue":{
        ref:"method:CubeGrid.getFacetValue",
        flags:"",
        description:" Get facet value definition by facetId and facetValueId.  Constant time.\n\n",
        name:"getFacetValue",
        seeAlso:[
            {ref:"class:FacetValue", description:null}
        ],
        params:[
            {description:"the id of the facet to retrieve", optional:false, name:"facetId",
             defaultValue:null, type:"String"},
            {description:"the id of the facet value to retrieve", optional:false, name:"facetValueId",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the FacetValue if found, or null", type:"FacetValue"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:FormulaBuilder.fieldKey":{
        ref:"attr:FormulaBuilder.fieldKey",
        flags:"IR",
        description:"ListGrid displaying the list of available fields and their corresponding formula keys.",
        name:"fieldKey",
        seeAlso:[],
        valueType:"AutoChild ListGrid",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:ListGrid.unfreezeField":{
        ref:"method:ListGrid.unfreezeField",
        flags:"",
        description:" Unfreeze a frozen field, so that it will now scroll along with other fields when horizontal\n scrolling occurs.\n\n",
        name:"unfreezeField",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"field", defaultValue:null,
             type:"ListGridField or colNum or ListGridField.name"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["frozenFields"]
    },
    "attr:Slider.labelHeight":{
        ref:"attr:Slider.labelHeight",
        flags:"IRW",
        description:"The height of the labels used to display the minimum, maximum and current values of the\n slider.",
        name:"labelHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:Slider"
    },
    "group:modal":{
        ref:"group:modal",
        refs:["method:Window.flash", "attr:Window.isModal", "attr:Window.modalMask", 
         "attr:Window.showModalMask","attr:Window.modalMaskOpacity", "attr:Window.modalMaskStyle", 
         "attr:Window.dismissOnOutsideClick"],
        name:"modal",
        seeAlso:[],
        type:"group"
    },
    "attr:RPCResponse.status":{
        ref:"attr:RPCResponse.status",
        flags:"R",
        description:"Status code for this response.  Status codes less than zero are considered errors by the\n RPCManager, those greater than or equal to zero are considered successes.  Please see the\n error handling section the ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager docs')} for more information on\n what the RPCManager does with the status code and how you can override this behavior.\n <P>\n When using the SmartClient server you can set the rpcResponse.status by calling the\n server-side method RPCResponse.setStatus().  \n <P>\n When not using the SmartClient server, the RPCManager makes no assumptions about the\n structure of the response, so the status code just reflects the\n ${isc.DocUtils.linkForRef('attr:RPCResponse.httpResponseCode')}: status will be \n ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_TRANSPORT_ERROR','STATUS_TRANSPORT_ERROR')} if an HTTP-level error\n occurred such as \"500 server error\".  If you have a status code you need to transmit you can\n simply embed it in the response (as part of ${isc.DocUtils.linkForRef('attr:RPCResponse.data')}) and interpret it from\n the callback.\n <P>\n With or without the SmartClient server, the ${isc.DocUtils.linkForRef('group:relogin')} status codes (such as \n ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_LOGIN_REQUIRED')}) are triggered whenever special markers, such as the\n loginRequiredMarker, appear in the body of the response.  See the ${isc.DocUtils.linkForRef('group:relogin','Relogin\n Overview')} for details.",
        name:"status",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCResponse"
    },
    "attr:Canvas.minHeight":{
        ref:"attr:Canvas.minHeight",
        flags:"IRWA",
        description:"Minimum height that this Canvas can be resized to.\n <P>\n Note that a Canvas with overflow:\"visible\" has an implicit minimize size based on it's\n contents.",
        name:"minHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "group:xmlSerialize":{
        ref:"group:xmlSerialize",
        refs:["attr:DataSourceField.xmlAttribute", "attr:DataSourceField.multiple", 
         "attr:DataSourceField.childTagName"],
        name:"xmlSerialize",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Class.map":{
        ref:"classMethod:Class.map",
        flags:"",
        description:"\n Call <code>method</code> on each item in <code>argsList</code> and return the Array of results.\n\n",
        name:"map",
        seeAlso:[],
        params:[
            {description:"Name of the method on this instance which should be called on each element of the Array",
             optional:false, name:"methodName", defaultValue:null, type:"string"},
            {description:"Array of items to call the method on", optional:false, name:"items",
             defaultValue:null, type:"Array"}
        ],
        returns:{description:"Array of results, one per element in the passed \"items\" Array", type:"Array"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:ButtonItem.height":{
        ref:"attr:ButtonItem.height",
        flags:"IRW",
        description:"By default buttonItems are sized to match their content (see ${isc.DocUtils.linkForRef('attr:ButtonItem.autoFit')}).\n Specifying an explicit size for the button will disable this behavior.",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ButtonItem",
        groups:["appearance"]
    },
    "attr:ListGrid.initialSort":{
        ref:"attr:ListGrid.initialSort",
        flags:"IR",
        description:"An array of ${isc.DocUtils.linkForRef('object:SortSpecifier')} objects used to set up the initial sort configuration for\n this grid.",
        name:"initialSort",
        seeAlso:[],
        valueType:"Array of SortSpecifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:Canvas.border":{
        ref:"attr:Canvas.border",
        flags:"IRW",
        description:"Set the CSS border of this component, as a CSS string including border-width,\n border-style, and/or color (eg \"2px solid blue\").\n <P>\n This property applies the same border to all four sides of this component.  Different\n per-side borders can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.",
        name:"border",
        setter:["setBorder"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "attr:ListGrid.summaryRowStyle":{
        ref:"attr:ListGrid.summaryRowStyle",
        flags:"IRWA",
        description:"${isc.DocUtils.linkForRef('attr:ListGrid.baseStyle')} for the ${isc.DocUtils.linkForRef('attr:ListGrid.summaryRow')}",
        name:"summaryRowStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"gridSummaryCell\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "classMethod:isc.logEcho":{
        ref:"classMethod:isc.logEcho",
        flags:"",
        description:" Logs the echoed object (using ${isc.DocUtils.linkForRef('classMethod:isc.echo')}) as a warning, prefixed with an\n optional message.\n\n",
        name:"logEcho",
        seeAlso:[
            {ref:"classMethod:Log.logWarn", description:"for logging info"}
        ],
        params:[
            {description:"object to echo", optional:false, name:"value", defaultValue:null, type:"any"},
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:StatefulCanvas.redrawOnStateChange":{
        ref:"attr:StatefulCanvas.redrawOnStateChange",
        flags:"IRWA",
        description:"Whether this widget needs to redraw to reflect state change",
        name:"redrawOnStateChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "attr:DateChooser.showTodayButton":{
        ref:"attr:DateChooser.showTodayButton",
        flags:"IRW",
        description:"Determines whether the \"Today\" button will be displayed, allowing the user to select \n the current date.",
        name:"showTodayButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "classMethod:Page.getHeight":{
        ref:"classMethod:Page.getHeight",
        flags:"",
        description:" Get the height of the visible portion of the window, not including browser chrome or the\n scrollbar area.\n",
        name:"getHeight",
        seeAlso:[],
        params:[
            {description:"the window object", optional:true, name:"wd", defaultValue:null, type:"object"}
        ],
        returns:{description:"height of the page", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "attr:GridRenderer.baseStyle":{
        ref:"attr:GridRenderer.baseStyle",
        flags:"IR",
        getter:["getBaseStyle"],
        description:"The base name for the CSS class applied to cells. This style will have \"Dark\", \n \"Over\", \"Selected\", or \"Disabled\" appended to it according to the state of the cell.",
        name:"baseStyle",
        seeAlso:[
            {ref:"method:GridRenderer.getCellStyle", description:null},
            {ref:"method:GridRenderer.getBaseStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"cell\"",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:DataSourceField.propertiesOnly":{
        ref:"attr:DataSourceField.propertiesOnly",
        flags:"IR",
        description:"For use in ${isc.DocUtils.linkForRef('group:componentSchema')} for fields that contain other components, this flag\n suppresses auto-construction for subcomponents that appear under this field.\n <P>\n For example, the ${isc.DocUtils.linkForRef('class:VLayout')} schema sets this for its ${isc.DocUtils.linkForRef('attr:Layout.members','members')}\n property, so that when a VLayout is constructed via XML as follows:\n <pre>\n &lt;VLayout&gt;\n     &lt;members&gt;\n         &lt;ListGrid ID=\"myGrid\" .../&gt;\n         &lt;Toolstrip ID=\"myToolStrip\" ... /&gt;\n     &lt;/members&gt;\n &lt;/VLayout&gt;\n </pre>\n The ListGrid and ToolStrip do not construct themselves automatically.  Instead, the VLayout\n receives the properties of the ListGrid and ToolStrip as ordinary JavaScript Objects, with\n the special property <code>_constructor</code> set to the name of the class that should be\n constructed.",
        name:"propertiesOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentSchema"]
    },
    "attr:ListGrid.canMultiSort":{
        ref:"attr:ListGrid.canMultiSort",
        flags:"IRW",
        description:"When true, indicates that this ListGrid supports multi-level sorting.",
        name:"canMultiSort",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:ListGrid"
    },
    "method:CubeGrid.hasChanges":{
        ref:"method:CubeGrid.hasChanges",
        flags:"",
        description:" Determines whether any cells in this cubeGrid have been edited but not yet saved to the\n underlying data set.\n",
        name:"hasChanges",
        seeAlso:[],
        returns:{description:"true if any record in the grid has been edited but not yet saved",
         type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:DynamicForm.items":{
        flags:"IRW",
        getter:["getItems"],
        type:"attr",
        ref:"attr:DynamicForm.items",
        description:"Synonym for ${isc.DocUtils.linkForRef('attr:DynamicForm.fields')}",
        name:"items",
        setter:["setItems"],
        valueType:"Array of FormItem Properties",
        seeAlso:[
            {ref:"attr:DynamicForm.fields", description:null}
        ],
        defaultValue:"null",
        groups:["items"],
        definingClass:"class:DynamicForm"
    },
    "class:AutoFitButton":{
        methods:[],
        ref:"class:AutoFitButton",
        deprecated:"As of Isomorphic SmartClient version 5.5, autoFit behavior can be achieved using the Button class instead by setting the property ${isc.DocUtils.linkForRef('attr:Button.autoFit')} to true.",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control/Button"],
        attrs:[],
        description:"A button that automatically sizes to the length of its title.  Implemented via the \n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.autoFit')} property.",
        name:"AutoFitButton",
        seeAlso:[
            {ref:"class:Button", description:null}
        ],
        type:"class"
    },
    "attr:DataSourceField.storeWithHash":{
        ref:"attr:DataSourceField.storeWithHash",
        flags:"IR",
        description:"If set, causes the field to be securely hashed before saving on an \"add\" or \"update\" \n operation.",
        name:"storeWithHash",
        seeAlso:[],
        valueType:"HashAlgorithm",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSourceField"
    },
    "type:timeFormatter":{
        ref:"type:timeFormatter",
        values:[
            {description:"String will display with seconds and am/pm indicator:<code>[H]H:MM:SS am|pm</code>. <br>\n  Example: <code>3:25:15 pm</code>",
             value:"toTime"},
            {description:"String will display with seconds in 24 hour time: <code>[H]H:MM:SS</code>. <br>\n  Example: <code>15:25:15</code>",
             value:"to24HourTime"},
            {description:"String will display with seconds, with a 2 digit hour and am/pm indicator: \n  <code>HH:MM:SS am|pm</code> <br>\n  Example: <code>03:25:15 pm</code>",
             value:"toPaddedTime"},
            {description:"String will display with seconds, with a 2 digit hour in 24 hour format: \n  <code>HH:MM:SS</code> <br>\n  Examples: <code>15:25:15</code>, <code>03:16:45</code>",
             value:"toPadded24HourTime"},
            {description:"String will have no seconds and be in 12 hour format:<code>[H]H:MM am|pm</code><br>\n  Example: <code>3:25 pm</code>",
             value:"toShortTime"},
            {description:"String will have no seconds and be in 24 hour format: <code>[H]H:MM</code><br>\n  Example:<code>15:25</code>",
             value:"toShort24HourTime"},
            {description:"String will have no seconds and will display a 2 digit hour, in 12 hour clock format:\n  <code>HH:MM am|pm</code><br>\n  Exmaple: <code>03:25 pm</code>",
             value:"toShortPaddedTime"},
            {description:"String will have no seconds and will display with a 2 digit hour in 24 hour clock format:\n <code>HH:MM</code><br>\n Examples: <code>15:25</code>, <code>03:16</code>",
             value:"toShortPadded24HourTime"}
        ],
        description:"String designating a standard time formatter for displaying the times associated with \n dates strings.",
        name:"timeFormatter",
        seeAlso:[],
        type:"type"
    },
    "group:writeCustomDataSource":{
        ref:"group:writeCustomDataSource",
        title:"Integrating the SmartClient Server framework with custom Java persistence logic",
        treeLocation:["Concepts"],
        refs:[],
        description:"Out of the box, and with no code to write, SmartClient supports SQL and Hibernate for \n persistence.  For other Java-based persistence systems, such as EJB, JPA, or systems\n proprietary to your company, you write a custom DataSource class in Java.  In most cases, it\n is possible to write a single, generic DataSource class that provides access to all data\n that is a available from a given persistence mechanism; for example, a single DataSource\n class can typically be written for accessing all Entity Beans available via EJB.\n <p>\n Note that a majority of the features of the SmartClient Server framework apply even when\n using your own persistence mechanism.  As with the features supported by SmartClient's\n browser-based visual components, SmartClient's server-side features rely only on the \n concept of a DataSource and not on the details of the ultimate persistence mechanism.  Hence\n they are usable with a custom DataSource regardless of the final data provider.\n <p>\n We provide a complete working example of a custom DataSource in the SmartClient Feature\n Explorer; you can see it in action ${isc.DocUtils.linkForExampleId('ormDataSource', 'here')}.  This example \n \"ormDataSource\" is an adaptor for Hibernate which supports the 4 CRUD operations, \n data paging, server-side sort and filter, and which participates correctly in \n ${isc.DocUtils.linkForRef('class:ResultSet','cache synchronization')}.  The code required is minimal, and the approaches\n taken generalize to any ORM system.  Studying the Java source code for this DataSource -\n which is available in the \"ORMDataSource.java\" tab in the example linked to above - is the\n best way to get a start on implementing your own custom DataSource.\n <p>\n <ul>\n <li><code>ORMDataSource</code> extends <code>BasicDataSource</code>.\n <li><code>ORMDataSource</code> is primarily an implementation of four key methods:\n     <code>executeFetch</code>, <code>executeAdd</code>, <code>executeUpdate</code> and \n     <code>executeRemove</code>.  All the logic related to the actual CRUD data operation\n     takes place in one of these methods.  This is the recommended approach.</li>\n <li>The class also implements the <code>execute</code> method.  This is an override of the\n     method that is actually called by the framework, and as such is an appropriate place to\n     set up shared objects that will be used in more than one CRUD operation, and to perform\n     shared pre- and post-processing.  As you can see, the example is setting up a Hibernate\n     session and transaction, and then calling <code>super.execute</code> - this calls back\n     into the framework and ultimately leads to the appropriate data operation method being\n     called.</li>\n <li>Note how each of the <code>executeXxx</code> methods conforms to the \n     ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource protocol')}.  To take <code>executeFetch</code> as \n     an example, note how it:\n     <ul><li>Retrieves the criteria for the fetch from the supplied <code>DSRequest</code></li>\n         <li>Implements logic to obey the <code>startRow</code>, <code>endRow</code> and \n             <code>batchSize</code> values.  This is only necessary for a DataSource that \n             intends to support automatic data paging.</li>\n         <li>Retrieves <code>sortByFields</code> from the supplied <code>DSrequest</code>, \n             and uses that value to change the order of the resultset.  This is only \n             necessary for a DataSource that intends to support server-side sorting.</li>\n         <li>Populates <code>startRow</code>, <code>endRow</code> and <code>totalRows</code>\n             on the <code>DSResponse</code>.</li>\n         <li>Populates the <code>DSResponse</code>'s <code>data</code> member with the list of \n             objects retrieved by the Hibernate call.</li>\n     </ul><br>\n     These are the only parts of this method that are of significance as far as SmartClient \n     is concerned - the rest of the method is concerned with communicating with the \n     data provider, which is of no interest to SmartClient as long as the method conforms to\n     the DataSource protocol for a \"fetch\" operation.</li>\n </ul>\n <p><br>\n <b>The DataSource descriptor</b>\n <p>\n Once your custom DataSource is implemented, you need to to create a descriptor for each \n instance of the DataSource.  As noted above, it is generally possible to write one custom \n DataSource class that is capable of handling all data access for a particular persistence \n mechanism.  DataSource descriptors, on the other hand, are written per entity.\n <p>\n A DataSource descriptor is an XML file with the special suffix <code>.ds.xml</code>.  The \n descriptor for a custom DataSource is, for the most part, identical to the descriptor for \n a built-in DataSource: it is the central place where you describe the DataSource instance\n to the system - its fields, validations, security constraints, special data operations, \n transaction chaining expressions and so on (see the ${isc.DocUtils.linkForRef('class:DataSource','DataSource docs')} \n for full details).\n <p>\n One property that is always required for a custom DataSource is \n ${isc.DocUtils.linkForRef('attr:DataSource.serverConstructor','serverConstructor')}.  This fully-qualified class \n name tells SmartClient what to instantiate when data operations for this DataSource arrive\n on the server - in other words, it is how you tell SmartClient to use your custom class.  \n In the ${isc.DocUtils.linkForExampleId('ormDataSource', 'ORM DataSource example')}, on the \n <code>ormDataSource_country</code> tab, you will see how we use this property to tie the\n <code>ormDataSource_country</code> DataSource <em>instance</em> to the \n <code>ormDataSource</code> DataSource <em>implementation</em>.\n <p>\n Finally, if your data model is based on Javabeans, or on POJOs that broadly follow the \n Javabean conventions (basically, if they have private state variables accessible via public \n getters and setters), SmartClient can automatically generate basic DataSource definitions \n for your beans that will only need minimal change (ie, specifying a \n <code>serverConstructor</code>) to be fully operational.  Both the \n ${isc.DocUtils.linkForExampleId('javabeanWizard', 'Visual Builder Javabean Wizard')} and the Batch DataSource \n Generator can create DataSource descriptors from existing beans.\n <p>\n <b>Server framework features relevant to custom DataSources</b>\n <P>\n The vast majority of the SmartClient Server framework's key features are not specific to the\n built-in SQL and Hibernate connectors, and still apply even when using a custom persistence\n mechanism.  See ${isc.DocUtils.linkForRef('group:featuresCustomPersistence','this overview')} of which features apply\n when using a custom persistence mechanism and how best to leverage those features.",
        name:"writeCustomDataSource",
        seeAlso:[],
        type:"group"
    },
    "method:DataBoundComponent.enableHiliting":{
        ref:"method:DataBoundComponent.enableHiliting",
        flags:"",
        description:" Enable all hilites.\n\n",
        name:"enableHiliting",
        seeAlso:[],
        params:[
            {description:"new enabled state to apply - if null, defaults to true", optional:true,
             name:"enable", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["hiliting"]
    },
    "attr:DateItem.selectorFormat":{
        ref:"attr:DateItem.selectorFormat",
        flags:"IRW",
        description:"If showing date selectors rather than the date text field (so when \n <code>this.useTextField</code> is false), this property allows customization of the \n order of the day, month and year selector fields.  If unset these fields will match the\n specified inputFormat for this item.\n <P>\n Note: selectors may be ommitted entirely by setting selectorFormat to (for example) \n <code>\"MD\"</code>. In this case the value for the omitted selector will match the\n ${isc.DocUtils.linkForRef('defaultValue')} specified for the item.  For example, if the selector format is \"MD\"\n (month and day only), the year comes from the Date specified as the defaultValue.",
        name:"selectorFormat",
        seeAlso:[],
        valueType:"DateItemSelectorFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "attr:DynamicForm.autoFetchData":{
        ref:"attr:DynamicForm.autoFetchData",
        flags:"IR",
        description:"If true, when this component is first drawn, automatically call <code>this.fetchData()</code>.\n Criteria for this fetch may be picked up from ${isc.DocUtils.linkForRef('attr:DynamicForm.initialCriteria')}, and textMatchStyle may\n be specified via ${isc.DocUtils.linkForRef('attr:DynamicForm.autoFetchTextMatchStyle')}.\n <P>\n <span style='color:red'>NOTE:</span> if <code>autoFetchData</code> is set, calling\n ${isc.DocUtils.linkForRef('method:DynamicForm.fetchData')} before draw will cause two requests to be issued, one from the manual\n call to fetchData() and one from the autoFetchData setting.  The second request will use\n only ${isc.DocUtils.linkForRef('attr:DynamicForm.initialCriteria')} and not any other criteria or settings from the first request.\n Generally, turn off autoFetchData if you are going to manually call fetchData() at any time.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"method:DynamicForm.fetchData", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["databinding"]
    },
    "attr:ListGridField.sortDirection":{
        ref:"attr:ListGridField.sortDirection",
        flags:"IRW",
        description:"Specifies the default sorting direction for this column. If specified on the\n ${isc.DocUtils.linkForRef('attr:ListGrid.sortField','default sort field')} for the listGrid, sorting occurs\n automatically, otherwise this will be the default direction when the user clicks the\n field header, or calls ${isc.DocUtils.linkForRef('method:ListGrid.sort')} without specifying an explicit sort direction.\n <P>\n Overrides ListGrid.sortDirection",
        name:"sortDirection",
        seeAlso:[
            {ref:"type:SortDirection", description:null}
        ],
        valueType:"SortDirection",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["sorting"]
    },
    "attr:Window.title":{
        ref:"attr:Window.title",
        flags:"IRW",
        description:"title for this Window, shown in the header (if drawn)",
        name:"title",
        setter:["setTitle"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Untitled Window\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "headerLabel", "i18nMessages"]
    },
    "method:Canvas.setBackgroundColor":{
        ref:"method:Canvas.setBackgroundColor",
        flags:"",
        description:"			Sets the background color of this widget to newColor.\n",
        name:"setBackgroundColor",
        seeAlso:[],
        params:[
            {description:"new color to set the widget's background to", optional:false,
             name:"newColor", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "attr:DataSourceField.javaCollectionClass":{
        ref:"attr:DataSourceField.javaCollectionClass",
        flags:"IR",
        description:"See ${isc.DocUtils.linkForRef('attr:DataSourceField.javaClass')} - when auto-populating of Java Beans / POJOs based on \n inbound DSRequest data, for a field of type Collection or Map, \n <code>javaCollectionClass</code> can \n be used to specify a particular concrete class to use.  If not specified, and a concrete\n Collection or Map class cannot be discovered using Java Reflection, the following concrete\n classes are used:\n <ul>\n <li> <code>java.util.ArrayList</code> is used for fields of type <code>List</code>\n <li> <code>java.util.HashSet</code> is used for fields of type <code>Set</code>\n <li> <code>java.util.LinkedList</code> is used for fields of type <code>Queue</code>\n <li> <code>org.apache.commons.collections.map.LinkedMap</code> is used for fields of type <code>Map</code>\n <li> <code>java.util.ArrayList</code> is used for fields that are otherwise of type <code>Collection</code>\n </ul>\n Note that this value is used even if the target Collection or Map is declared as a concrete\n class.  So, for example, if you set <code>javaCollectionClass</code> to \n <code>java.util.LinkedList</code> but your setter method accepts a \n <code>java.util.ArrayList</code>, you will get a ClassCastException.",
        name:"javaCollectionClass",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "attr:TabSet.tabBarAlign":{
        ref:"attr:TabSet.tabBarAlign",
        flags:"IR",
        description:"Alignment of the tabBar.\n <P>\n If the position of the tabBar is \"top\" or \"bottom\", then alignment must be \"left\" or\n \"right\" and defaults to \"left\".\n <P>\n If the position of the tabBar is \"left\" or \"right\", then the alignment must be \"top\" or\n \"bottom\" and defaults to \"top\".",
        name:"tabBarAlign",
        seeAlso:[],
        valueType:"Side",
        defaultValue:"see below",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('tabsAlign')}"],
        definingClass:"class:TabSet",
        groups:["tabBar"]
    },
    "method:DynamicForm.getItemErrorHTML":{
        ref:"method:DynamicForm.getItemErrorHTML",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true, this method is called for each item in the form\n and returns the error HTML to be written out next to the item.<br>\n Default implementation falls through to ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')} on the item in question.\n",
        name:"getItemErrorHTML",
        seeAlso:[],
        params:[
            {description:"Form item for which the HTML should be retrieved", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"Error message to display for the item, or array of error message                              strings.",
             optional:false, name:"error", defaultValue:null, type:"string | array"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:SummaryBuilder.testRecord":{
        ref:"attr:SummaryBuilder.testRecord",
        flags:"IRA",
        description:"Record to use when showing sample output for the format string.\n <P>\n If not specified, the selected record in the component that launched the SummaryBuilder will\n be used, or if there's no selection, the first visible row, or with no component, a dummy\n data row derived automatically from the provided DataSource.",
        name:"testRecord",
        seeAlso:[],
        valueType:"Record",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["formulaFields"]
    },
    "attr:ListGridField.showHover":{
        ref:"attr:ListGridField.showHover",
        flags:"IRW",
        description:"Whether to show hovers for this field.  The default hover will be the contents of the\n cell the user is hovering over, and can be customized via\n ${isc.DocUtils.linkForRef('method:ListGridField.hoverHTML','field.hoverHTML()')}.\n <P>\n ${isc.DocUtils.linkForRef('attr:ListGrid.canHover')} can be set to true to cause hovers to be shown for all fields\n by default.  In this case, <code>field.showHover</code> can be set to false to suppress\n hovers for an individual field.\n <P>\n All hovers can be disabled, regardless of other settings, by setting\n ${isc.DocUtils.linkForRef('attr:ListGrid.showHover')} to false.",
        name:"showHover",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('valueHoverTips')}"],
        definingClass:"object:ListGridField"
    },
    "method:ListGrid.getEditValue":{
        ref:"method:ListGrid.getEditValue",
        flags:"",
        description:"\n Returns the current temporary locally stored edit value for some field within a record \n being edited.\n\n",
        name:"getEditValue",
        seeAlso:[],
        params:[
            {description:"index of the row for which the editValue should be returned",
             optional:false, name:"rowNum", defaultValue:null, type:"number"},
            {description:"index of the field, or fieldName, for which value should be                               returned",
             optional:false, name:"colNum", defaultValue:null, type:"number | string"}
        ],
        returns:{description:"edit value for the field in question", type:"any"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:autoComplete":{
        ref:"group:autoComplete",
        refs:[],
        name:"autoComplete",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.showHover":{
        ref:"attr:ListGrid.showHover",
        flags:"IRW",
        description:"If true, and canHover is also true, when the user hovers over a cell, hover text will pop up\n next to the mouse.  The contents of the hover is determined by ${isc.DocUtils.linkForRef('method:ListGrid.cellHoverHTML')}.",
        name:"showHover",
        seeAlso:[
            {ref:"attr:ListGrid.canHover", description:null},
            {ref:"method:ListGrid.cellHoverHTML", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["hovers"]
    },
    "method:CubeGrid.deselectCells":{
        ref:"method:CubeGrid.deselectCells",
        flags:"",
        description:"\n  Deselect cells that match a list of facetValues.  \n  Also supports an explicit list of cells or cellIds\n\n      <br><i>methodType</i> action\n",
        name:"deselectCells",
        seeAlso:[],
        params:[
            {description:"cells to deselect", optional:false, name:"cellList", defaultValue:null,
             type:"Array of cells | facetValues object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:ListGrid.ungroup":{
        ref:"method:ListGrid.ungroup",
        flags:"",
        description:" Removes the grouping from the listgrid, restoring its original data\n\n",
        name:"ungroup",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ComboBoxItem.maskOverwriteMode":{
        ref:"attr:ComboBoxItem.maskOverwriteMode",
        flags:null,
        description:"Not applicable to a ComboBoxItem.",
        name:"maskOverwriteMode",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:SelectItem.optionOperationId":{
        ref:"attr:SelectItem.optionOperationId",
        flags:"IR",
        description:"If this item has a specified <code>optionDataSource</code>, this attribute may be set\n to specify an explicit ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} when performing a fetch against the\n option dataSource to pick up display value mapping.",
        name:"optionOperationId",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:StatefulCanvas.iconSpacing":{
        ref:"attr:StatefulCanvas.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:MultiFileItem.removeButton":{
        ref:"attr:MultiFileItem.removeButton",
        flags:"IR",
        description:"Button for removing files.  Supports the properties of a ${isc.DocUtils.linkForRef('object:FormItemIcon')}.",
        name:"removeButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiFileItem"
    },
    "method:FormItem.titleHover":{
        ref:"method:FormItem.titleHover",
        flags:"A",
        description:"  Optional stringMethod to fire when the user hovers over this item's title.\n  Return false to suppress default behavior of showing a hover canvas containing the\n  HTML returned by <code>formItem.titleHoverHTML()</code> / \n  <code>form.titleHoverHTML()</code>.\n\n",
        name:"titleHover",
        seeAlso:[
            {ref:"method:FormItem.itemHover", description:null},
            {ref:"method:FormItem.titleHoverHTML", description:null}
        ],
        params:[
            {description:"Pointer to this item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"This items form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:GridRenderer.emptyMessage":{
        ref:"attr:GridRenderer.emptyMessage",
        flags:"IRW",
        description:"The string to display in the body of a listGrid with an empty data array, if\n showEmptyMessage is true.",
        name:"emptyMessage",
        seeAlso:[
            {ref:"attr:GridRenderer.showEmptyMessage", description:null},
            {ref:"attr:GridRenderer.emptyMessageStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["emptyMessage", "i18nMessages"]
    },
    "classAttr:Validator.notAnInteger":{
        ref:"classAttr:Validator.notAnInteger",
        flags:"IRA",
        description:"Default error message to display when standard <code>isInteger</code> type validator\n  returns false.",
        name:"notAnInteger",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be a whole number.\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:SimpleType.validOperators":{
        ref:"attr:SimpleType.validOperators",
        flags:"IR",
        description:"Set of search operators valid for this type.  \n <P>\n If not specified, the ${isc.DocUtils.linkForRef('attr:SimpleType.inheritsFrom','inherited')} type's operators will be used, finally\n defaulting to the default operators for the basic types (eg, integer).",
        name:"validOperators",
        seeAlso:[],
        valueType:"Array of OperatorId",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleType",
        groups:["advancedFilter"]
    },
    "attr:TestFunctionResult.failedGeneration":{
        ref:"attr:TestFunctionResult.failedGeneration",
        flags:"IRW",
        description:"Set to true if there is a syntax error in the formula or summary being checked.\n <P>\n When set to true, ${isc.DocUtils.linkForRef('attr:TestFunctionResult.errorText')} contains the exception message.",
        name:"failedGeneration",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "attr:ToolbarItem.buttonConstructor":{
        ref:"attr:ToolbarItem.buttonConstructor",
        flags:"IRWA",
        description:"Use AutoFitButtons rather than simple Buttons for this toolbar's buttons by default.",
        name:"buttonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"AutoFitButton",
        type:"attr",
        definingClass:"class:ToolbarItem"
    },
    "method:DynamicForm.getFieldErrors":{
        ref:"method:DynamicForm.getFieldErrors",
        flags:"",
        description:" Returns the current set of validation errors for some field in this form.\n",
        name:"getFieldErrors",
        seeAlso:[],
        returns:{description:"Error message string, or if there is more than one error      associated with this field, array of error message strings.",
         type:"string | array of strings"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "class:PropertySheet":{
        methods:[],
        ref:"class:PropertySheet",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms"],
        attrs:[],
        description:"Editor with a minimalist appearance, tuned for editing large numbers of properties in a\n constrained space.",
        name:"PropertySheet",
        seeAlso:[],
        type:"class"
    },
    "classMethod:Page.getImgURL":{
        ref:"classMethod:Page.getImgURL",
        flags:"",
        description:" Return the full URL for app-specific or skin image.\n <P>\n To use a skin image, start the URL with \"[SKIN]\".  Any other relative URL is assumed\n relative to the ${isc.DocUtils.linkForRef('classMethod:Page.getAppImgDir','appImgDir')}.\n\n",
        name:"getImgURL",
        seeAlso:[],
        params:[
            {description:"Local file name for the image.", optional:false, name:"src",
             defaultValue:null, type:"SCImgURL"},
            {description:"User-specified image directory,										local to", optional:true,
             name:"imgDir", defaultValue:null, type:"string"}
        ],
        returns:{description:"URL for the image.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "group:relogin":{
        ref:"group:relogin",
        title:"Relogin",
        treeLocation:["Client Reference/RPC"],
        refs:["classAttr:RPCManager.credentialsURL", "classMethod:RPCManager.loginRequired", 
         "attr:RPCRequest.containsCredentials"],
        description:"For applications that require authentication and use session timeout, a background RPC may\n occur with an expired session.  The ideal handling of this scenario is that, with the attempted\n transaction still suspended, a modal dialog is used to re-authenticate the user, and then the\n original transaction is resumed without loss of data or context.  SmartClient makes it easy to\n implement this ideal handling <i>without</i> having to implement session timeout handling in\n every codepath that contacts the server, by providing central notification of session timeout,\n and the ability to re-send a transaction that encountered session timeout.\n <P>\n When session timeout occurs, typical authentication systems intercept requests to protected\n resources and return a page that tells the user that login is required.  With a background RPC\n this page won't be shown to the user.\n <P>\n To enable SmartClient to detect that session timeout has occurred, a snippet of HTML called the\n loginRequiredMarker must be added to the session timeout response sent by the authentication\n system.  The loginRequiredMarker is standardized and does not need to be customized for your\n deployment.  It can be added to an HTML page without visual effect, or if the response sent on\n session timeout is not an HTML page, simply embedded in any other type of response, or sent\n as the entirety of the response.  The loginRequiredMarker is in\n isomorphic/login/loginRequiredMarker.html.\n <P>\n When SmartClient detects the loginRequired marker, the transaction that encountered session\n timeout is put on hold, and ${isc.DocUtils.linkForRef('classMethod:RPCManager.loginRequired')} is called.  At this point you\n have a few options:\n <ol>\n \n <li> Leave the SmartClient application and take the user to the login page, by simply doing a\n <code>window.location.replace(<i>myLoginURL</i>)</code>, the simplest but least user friendly\n option.\n \n <li> Open a new browser window that goes to your plain HTML login form (or offer a link that\n opens such a browser window), using a modal dialog in the application page that prompts the\n user to login before continuing, then re-send the intercepted transaction\n (${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction')} when the user indicates he has logged in.\n This is simple, does not drop context, but is not seamless.\n \n <li> Use a SmartClient interface, typically a DynamicForm in a Window, to collect credentials,\n perform login as a background RPC, and on success re-send the intercepted transaction\n (${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction')}.  A complete example of this, which assumes\n an authentication system that can take credentials as HTTP POST params, is included in the SDK\n as isomorphic/login/reloginFlow.js.\n \n </ol>\n <B>Authentication via background RPC form POST</B>\n <P>\n The approach shown in reloginFlow.js posts the credentials gathered from the user to\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.credentialsURL')}.  To make this work with an authentication system that can\n accept credentials via HTTP POST:\n <ol>\n <li> set the RPCManager.credentialsURL to the URL where credentials should be POST'd\n <li> include reloginFlow.js in your page, modified, if necessary, so that the names of the\n USERNAME and PASSWORD params match what your authentication system uses\n <li> configure your authentication system to send back the loginSuccessMarker as part of a\n successful login response, and the loginRequiredMarker as part of a failed login response\n </ol>\n If your authentication system can accept POST'd credentials at any URL it protects, the last\n step may be as simple as configuring the loginSuccessMarker file itself as a protected\n resource (isomorphic/login/loginSuccess.html).\n <P>\n <B>Authentication via background SmartClient server RPC/DMI</B>\n <P>\n If you are using the SmartClient Java server and your authentication system allows you to mark\n a user as authenticated from Java, you can perform a normal RPC or DMI with the credentials\n gathered from the user and send back success or failure indications as normal RPC or DMI\n responses.  This can be useful if, in addition to logging in, you want to send back additional\n data.\n <P>\n <B>Advanced: concurrency</B>\n <P>\n If, after loginRequired() has fired and before the user has re-authenticated, you send\n additional RPCs to protected URLs, you will get additional loginRequired() notifications.  This\n may happen to applications that poll for data or periodically save without user action.  You\n may wish to avoid this by setting an application-specific flag to avoid firing requests during\n the relogin process.  However, you must ultimately either\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.resendTransaction','resend')} or ${isc.DocUtils.linkForRef('classMethod:RPCManager.clearTransaction','discard')}\n every transaction for which loginRequired() fires, or you will have a memory leak due to\n suspended transactions.\n <P>\n Note also that there is no requirement that the relogin process blocks user interaction.\n Applications that access multiple services may choose to simply show an unobtrusive error\n indication such that the user can log back in at his liesure, or even log the user back in\n automatically.",
        name:"relogin",
        seeAlso:[],
        type:"group"
    },
    "attr:StatefulCanvas.selected":{
        ref:"attr:StatefulCanvas.selected",
        flags:"IRW",
        description:"Whether this component is selected.  For some components, selection affects appearance.",
        name:"selected",
        setter:["setSelected"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "method:String.startsWith":{
        ref:"method:String.startsWith",
        flags:"",
        description:"			Returns true if this string starts with another string.\n",
        name:"startsWith",
        seeAlso:[],
        params:[
            {description:"other string to check", optional:false, name:"substring", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"true == this string starts with substring", type:"boolean"},
        type:"method",
        definingClass:"class:String",
        groups:["stringProcessing"]
    },
    "method:TreeGrid.fetchData":{
        ref:"method:TreeGrid.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component as a\n tree.\n <P>\n This method will create a ${isc.DocUtils.linkForRef('class:ResultTree')} to manage tree data, which will\n subsequently be available as <code>treeGrid.data</code>.  DataSource records\n returned by the \"fetch\" operation are linked into a tree structure according to\n ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')} and\n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} declarations on DataSource fields.  See the\n ${isc.DocUtils.linkForRef('group:treeDataBinding')} topic for complete details.\n <P>\n By default, the created ResultTree will use folder-by-folder load on demand, asking the\n server for the children of each folder as the user opens it.\n <P>\n The ${isc.DocUtils.linkForRef('class:ResultTree')} created by <code>fetchData()</code> can be customized by setting\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')} to an Object containing properties and methods to apply to\n the created ResultTree.  For example, the property that determines whether a node is a\n folder (${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty','isFolderProperty')}) can be customized, or\n level-by-level loading can be disabled via\n ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand','loadDataOnDemand:false')}.\n <P>\n The callback passed to <code>fetchData</code> will fire once, the first time that data is\n loaded from the server.  If using folder-by-folder load on demand, use the\n ${isc.DocUtils.linkForRef('method:ResultTree.dataArrived')} notification to be notified each time new nodes are loaded.\n <P>\n Note that, if criteria are passed to <code>fetchData()</code>, they will be passed every\n time a new \"fetch\" operation is sent to the server.  This allows you to retrieve multiple\n different tree structures from the same DataSource.  However note that the server is expected\n to always respond with an intact tree - returned nodes which do not have parents are dropped\n from the dataset and not displayed.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:FacetValue.selectionBoundary":{
        ref:"attr:FacetValue.selectionBoundary",
        flags:"IR",
        description:"Selection boundary determining what facets / facetValues can be selected together by\n drag selection / shift+click selection",
        name:"selectionBoundary",
        seeAlso:[],
        valueType:"enum",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "method:ListGrid.setFieldTitle":{
        ref:"method:ListGrid.setFieldTitle",
        flags:"",
        description:" Change the title of a field after the grid is created.\n\n",
        name:"setFieldTitle",
        seeAlso:[],
        params:[
            {description:"name of the field, or index.", optional:false, name:"fieldNum",
             defaultValue:null, type:"integer or String"},
            {description:"new title", optional:false, name:"title", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "classMethod:SimpleType.getDefaultSummaryFunction":{
        ref:"classMethod:SimpleType.getDefaultSummaryFunction",
        flags:"",
        description:" Retrieves the default summary function for some field type.\n",
        name:"getDefaultSummaryFunction",
        seeAlso:[],
        params:[
            {description:"type name", optional:false, name:"typeName", defaultValue:null, type:"string"}
        ],
        returns:{description:"default summary function for this data type.", type:"SummaryFunction"},
        type:"classMethod",
        definingClass:"class:SimpleType"
    },
    "attr:FormItemIcon.showFocused":{
        ref:"attr:FormItemIcon.showFocused",
        flags:"IRWA",
        description:"Should this icon's image switch to the appropriate \"focused\" source when the user puts\n focus on the form item or icon?",
        name:"showFocused",
        seeAlso:[
            {ref:"attr:FormItem.showFocusedIcons", description:null},
            {ref:"attr:FormItemIcon.showFocusedWithItem", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "method:Selection.selectAll":{
        ref:"method:Selection.selectAll",
        flags:"",
        description:"			Select ALL records of the list\n",
        name:"selectAll",
        seeAlso:[],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:ListGridField.optionDataSource":{
        ref:"attr:ListGridField.optionDataSource",
        flags:"IRW",
        description:"Derive a ${isc.DocUtils.linkForRef('type:ValueMap')} by fetching records from another DataSource and extracting\n the ${isc.DocUtils.linkForRef('attr:ListGridField.valueField','valueField')} and \n ${isc.DocUtils.linkForRef('attr:ListGridField.displayField','displayField')} in the loaded records, to derive one\n valueMap entry per record loaded from the optionDataSource.\n <P>\n Unlike the similar use of ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} for ${isc.DocUtils.linkForRef('interface:PickList','pickLists')}\n used during editing or filtering, <code>listGridField.optionDataSource</code> causes the\n entire set of records from the optionDataSource to be fetched, without paging.   Hence\n listGridField.optionDataSource is appropriate only for smaller valueMaps.  For very\n large valueMap situations, such as an accountId field that should be displayed as an\n accountName where there are thousands of accounts, the recommended approach is:\n <ul>\n <li> do not set listGridField.optionDataSource\n <li> declare two fields in the DataSource, eg \"accountId\" and \"accountName\".\n <li> Set the ${isc.DocUtils.linkForRef('attr:ListGridField.displayField')} attribute on the data field to the\n      name of the display field.\n <li> When fetching records for display in a grid, have your server send back values for \n      both fields, but show only the data field (\"accountId\") in the grid.\n </ul>\n In this case the cells in the accountId field will show the record value from the \n accountName field.  This approach means the valueMap will never be loaded in its\n entirety, instead, each loaded record contains the valueMapping for that one record, as\n a pair of fields within the record.\n <P>\n Note that it is key that the server return <b>both</b> the underlying stored value\n <b>and</b> the display value, as suggested above, because this approach allows the\n ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} property to be used to provide paged valueMaps during\n inline editing and ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor','inline filtering')}.  Note that\n pickList.optionDataSource is a distinct setting from listGridField.optionDataSource,\n settable via ${isc.DocUtils.linkForRef('attr:ListGridField.editorProperties')} (for editing) and \n ${isc.DocUtils.linkForRef('attr:ListGridField.filterEditorProperties','field.filterEditorProperties')} (for\n filtering).",
        name:"optionDataSource",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "attr:ListGrid.scrollRedrawDelay":{
        ref:"attr:ListGrid.scrollRedrawDelay",
        flags:"IRW",
        description:"While drag scrolling in an incrementally rendered grid, time in milliseconds to wait\n before redrawing, after the last mouse movement by the user.",
        name:"scrollRedrawDelay",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"75",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "method:ListGrid.getVisibleRows":{
        ref:"method:ListGrid.getVisibleRows",
        flags:"",
        description:" Get the rows that are currently visible in the viewport, as an array of \n [firstRowNum, lastRowNum]. If the grid contains no records, will return [-1,-1];\n",
        name:"getVisibleRows",
        seeAlso:[],
        returns:{description:null, type:"Array"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:FormItem.width":{
        ref:"attr:FormItem.width",
        flags:"IRW",
        description:"By default, items take up the entire width of their cell",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"\"*\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('columnSpanning')}"],
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "attr:TreeGrid.displayNodeType":{
        ref:"attr:TreeGrid.displayNodeType",
        flags:"IRW",
        description:"Specifies the type of nodes displayed in the treeGrid.",
        name:"displayNodeType",
        seeAlso:[
            {ref:"type:DisplayNodeType", description:"for options"}
        ],
        valueType:"DisplayNodeType",
        defaultValue:"Tree.FOLDERS_AND_LEAVES",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeField"]
    },
    "attr:ListGrid.showFilterEditor":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.showFilterEditor",
        description:"Should this listGrid display a filter row.  If true, this ListGrid\n will be drawn with a single editable row, (separate from the body) with a filter button.\n Values entered into this row are used as filter criteria to filter this List's data on\n enter-keypress or filter button click. ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchTextMatchStyle')} determines\n the textMatchStyle for the request passed to ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}.\n <P>\n Note that if ${isc.DocUtils.linkForRef('method:ListGrid.filterData')} or ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} is called directly\n while the filter editor is showing, the filter editor values will be updated to reflect the\n new set of criteria. If you wish to retain the user entered filter criteria and \n programatically modify a subset of field values programatically this can be achieved by\n deriving new criteria by copying the existing set of criteria and adding other changes - \n something like this:\n <pre><code>\n   var newCriteria = isc.clone(myListGrid.getCriteria());\n   isc.addProperties(newCriteria, {\n      field1:\"new value1\",\n      field2:\"new value2\"\n   });\n   myListGrid.setCriteria(newCriteria);\n </code></pre>\n <P>\n Also note that if you call <code>filterData()</code> and pass in criteria for dataSource \n fields that are not present in the ListGrid, these criteria will continue to be applied along\n with the user visible criteria.",
        name:"showFilterEditor",
        setter:["setShowFilterEditor"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('filter')}"],
        defaultValue:"null",
        groups:["filterEditor"],
        definingClass:"class:ListGrid"
    },
    "method:TileLayout.getDragData":{
        ref:"method:TileLayout.getDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method returns the set of records being dragged out\n of the component.  In the default implementation, this is the list of currently selected\n records.<p>\n \n This method is consulted by ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')}.\n\n",
        name:"getDragData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"}
        ],
        returns:{description:"Array of ${isc.DocUtils.linkForRef('object:Record')}s that are currently selected.",
         type:"Array of Record"},
        type:"method",
        definingClass:"class:TileLayout",
        groups:["dragging", "data"]
    },
    "classMethod:History.addHistoryEntry":{
        ref:"classMethod:History.addHistoryEntry",
        flags:"",
        description:"\n Call this method to add a synthetic history entry.  The new history entry is added in the\n history stack after the currently visible page - in exactly the same way as the browser\n would treat a new page transition at this point.  In other words, if the user has navigated\n ten pages using, say, a mixture of synthetic and real history entries, then presses back\n five times and then triggers a call to this method, the history entry will be created at the\n 6th position in the history stack and any history entries forward of that will be destroyed.\n <p>\n This method must be called with an id.  This id can be any string - it will be URL-encoded\n and added to the current page URL as an anchor (e.g. #foo).  This URL change allows the user\n to bookmark this particular application state.  When the user next navigates to this history\n entry, the id you supplied here will be passed back to the callback you supplied via\n ${isc.DocUtils.linkForRef('classMethod:History.registerCallback')}.\n <p>\n You may also optionally supply some arbitrary data to associate with this history entry.\n If you do this, the data you passed in will be passed back to you as part of the callback\n you specified via ${isc.DocUtils.linkForRef('classMethod:History.registerCallback')}.  This data object can be anything you\n want, but there are some caveats:\n <ul>\n <li>As long as the user has not navigated away from the top-level page (i.e. the user is\n navigating within synthetic history entries only), whatever data you pass in will be handed\n back to you.\n <li>When the user navigates away from the current page, SmartClient will attempt to\n serialize the data into a string so that when/if the user comes back to this history entry,\n it can be deserialized and passed back to your logic.  To take advantage of this, you need\n to make sure that your data is serializeable.  As long as your data is a native datatype\n (String, Number, Boolean) or a collection of such datatypes (collections meaning object\n literals and arrays), then it will serialize correctly.  Things like pointers to the\n document object and functions cannot be serialized.\n <li>In order for the serialization to occur on a page transition, you must have the\n SmartClient Core module loaded on the page at the time of the transition.  If it's not\n available, the data will be lost, but you will still get a callback with the id you specify\n if the user navigates back to this history entry later.\n <li>The data associated with this history entry will persist as long as at least one\n instance of the browser remains open on the user's machine.  Once the user closes all\n browser instances, the data will be lost. \n <li>Also, the user can trigger a history callback at any time by navigating to a bookmarked\n history entry that may have been created in a past session, such that no data is associated\n with that id in the current session.  How you choose to handle that situation is up to you.\n </ul>\n <p>\n You're always guaranteed to receive the id you associate with a history entry in the\n callback that you specify, but the data you associated may or may not be available, so be\n careful about how you use it. Note that by passing the <code>requiresData</code> parameter\n to ${isc.DocUtils.linkForRef('classMethod:History.registerCallback')} you can suppress the callback from firing unless the stored\n data object is actually available.\n\n",
        name:"addHistoryEntry",
        seeAlso:[],
        params:[
            {description:"The id you want to associate with this history entry.  This value will appear as an anchor reference at the end of the URL string.  For example, if you pass in\n \"foo\" as the id, the URL will then have a #foo tacked on the end of it.  This id will be\n passed back to the callback you specified in ${isc.DocUtils.linkForRef('classMethod:History.registerCallback')} when the user\n navigates to this history entry in the future.",
             optional:false, name:"id", defaultValue:null, type:"string"},
            {description:"The title to show in the history drop-down for this history entry.  If not specified, the <code>id</code> is used, unless you've set an explicit history title via\n ${isc.DocUtils.linkForRef('classMethod:History.setHistoryTitle')}.  Note: this currently works in IE only.  You may pass a\n title in any other browser, but it will not change what's displayed in the history\n drop-down.",
             optional:true, name:"title", defaultValue:null, type:"string"},
            {description:"Arbitrary data to associate with this history entry.  When the user next navigates to this history entry, this data will be provided as an argument to your callback\n function.  Note that the SmartClient Core module is also required to be loaded on the page\n for this particular feature to work.",
             optional:true, name:"data", defaultValue:null, type:"any"}
        ],
        type:"classMethod",
        definingClass:"class:History"
    },
    "attr:ListGridField.showSelectedIcon":{
        ref:"attr:ListGridField.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["buttonIcon"]
    },
    "attr:RPCRequest.willHandleError":{
        ref:"attr:RPCRequest.willHandleError",
        flags:"IRW",
        description:"With willHandleError:false, rpcResponses that indicate an error go through centralized\n handling in the RPCManager and rpcRequest.callback is never invoked.\n <P>\n Setting willHandleError:true means that your rpcRequest.callback will receive rpcResponses\n that have an error status and must handle them.\n <P>\n See also the error handling section in the ${isc.DocUtils.linkForRef('class:RPCManager')} docs.",
        name:"willHandleError",
        seeAlso:[
            {ref:"class:RPCManager", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "group:hoverTips":{
        ref:"group:hoverTips",
        refs:["method:CubeGrid.facetLabelHoverHTML", "method:CubeGrid.facetValueHoverHTML", 
         "attr:CubeGrid.facetValueHoverWidth","attr:CubeGrid.facetLabelHoverWidth", "attr:CubeGrid.facetValueHoverHeight", 
         "attr:CubeGrid.facetLabelHoverHeight","attr:CubeGrid.facetValueHoverAlign", "attr:CubeGrid.facetLabelHoverAlign", 
         "attr:CubeGrid.facetValueHoverVAlign","attr:CubeGrid.facetLabelHoverVAlign", "attr:CubeGrid.facetValueHoverStyle", 
         "attr:CubeGrid.facetLabelHoverStyle"],
        description:null,
        name:"hoverTips",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.alternateRecordFrequency":{
        ref:"attr:ListGrid.alternateRecordFrequency",
        flags:"IRW",
        description:"The number of consecutive rows to draw in the same style before alternating, when\n alternateRowStyles is true.",
        name:"alternateRecordFrequency",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "attr:EdgedCanvas.edgeOffsetRight":{
        ref:"attr:EdgedCanvas.edgeOffsetRight",
        flags:"IRA",
        description:"Amount the contained Canvas should be offset from the right.  Defaults to the size for\n the right edge.  Set smaller to allow the contained Canvas to overlap the edge and\n corner media.",
        name:"edgeOffsetRight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:BatchUploader.warnOnCancel":{
        ref:"attr:BatchUploader.warnOnCancel",
        flags:"IRW",
        description:"If set, indicates that a warning dialog should be shown when Cancel is clicked, asking \n the user to confirm that this is really what they want to do. The actual warning message\n is specified with ${isc.DocUtils.linkForRef('attr:BatchUploader.cancelConfirmMessage')}",
        name:"warnOnCancel",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:ResultSet.allMatchingRowsCached":{
        ref:"method:ResultSet.allMatchingRowsCached",
        flags:"A",
        description:" Do we have a complete client-side cache of records for the current filter criteria?\n <P>\n Returns false if this is a paged data set, and the entire set of records that match\n the current criteria has not been retrieved from the server.\n\n",
        name:"allMatchingRowsCached",
        seeAlso:[],
        returns:{description:"whether all matching rows are cached", type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "object:Criterion":{
        methods:[],
        type:"object",
        ref:"object:Criterion",
        classAttrs:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        classMethods:[],
        description:"An object representing a criterion to apply to a record.  \n <P>\n A criterion is part of the definiton of an ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} object, which is used to\n filter records according to search criteria. \n <P>\n A criterion consists of an ${isc.DocUtils.linkForRef('attr:Criterion.operator')} and typically a\n ${isc.DocUtils.linkForRef('attr:DataSourceField.name','fieldName')} from a\n ${isc.DocUtils.linkForRef('object:Record')} and a ${isc.DocUtils.linkForRef('attr:Criterion.value','value')} to compare to.  However some operators\n either don't require a value (eg, isNull) or act on other criteria rather than directly on a\n ${isc.DocUtils.linkForRef('object:Record')}'s fields (eg, the \"and\" and \"or\" logical operators).",
        attrs:["attr:Criterion.operator", "attr:Criterion.fieldName", "attr:Criterion.value", 
         "attr:Criterion.criteria","attr:Criterion.start", "attr:Criterion.end"],
        name:"Criterion",
        seeAlso:[],
        serverDS:"allowed",
        groups:["advancedFilter"]
    },
    "method:CubeGrid.getCellColumn":{
        ref:"method:CubeGrid.getCellColumn",
        flags:"",
        description:" Given a record in this grid, this method returns the colNum in which the record is displayed.\n",
        name:"getCellColumn",
        seeAlso:[],
        params:[
            {description:"record to find coordinates for", optional:false, name:"cellRecord",
             defaultValue:null, type:"record"}
        ],
        returns:{description:"Column number for the record. Returns -1 if the record is not found.",
         type:"integer"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "type:LayoutResizeBarPolicy":{
        ref:"type:LayoutResizeBarPolicy",
        values:[
            {description:"resize bars are only shown on members marked                 ${isc.DocUtils.linkForRef('attr:Canvas.showResizeBar','showResizeBar:true')}",
             value:"\"marked\""},
            {description:"resize bars are shown on all resizable members that are not explicitly marked              showResizeBar:false, except the last member.  Appropriate for a\n              ${isc.DocUtils.linkForRef('type:LayoutPolicy')} of \"fill\" (VLayout, HLayout) since the overall space will\n              always be filled.",
             value:"\"middle\""},
            {description:"resize bars are shown on all resizable members that are not explicitly marked              showResizeBar:false, including the last member.  Can be appropriate for a\n              ${isc.DocUtils.linkForRef('type:LayoutPolicy')} of \"none\" (VStack, HStack) since the overall size of the\n              layout is dictated by it's member's sizes.",
             value:"\"all\""},
            {description:"resize bars are not shown even if members are marked with                 ${isc.DocUtils.linkForRef('attr:Canvas.showResizeBar','showResizeBar:true')}",
             value:"\"none\""}
        ],
        description:"Policy for whether resize bars are shown on members by default.",
        name:"LayoutResizeBarPolicy",
        seeAlso:[],
        type:"type"
    },
    "attr:ColorPicker.supportsTransparency":{
        ref:"attr:ColorPicker.supportsTransparency",
        flags:"IR",
        description:"Determines whether to show the opacity slider. This allows the user to select colors\n with an alpha element (ie, semi-transparent colors). If this attribute is set to false,\n no opacity slider is shown, and all colors are completely opaque.",
        name:"supportsTransparency",
        setter:["setSupportsTransparency"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Canvas.getPageTop":{
        ref:"method:Canvas.getPageTop",
        flags:"A",
        description:" Returns the page-relative top coordinate of the widget on the page, in pixels \n",
        name:"getPageTop",
        seeAlso:[],
        returns:{description:"GLOBAL top coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "method:GridRenderer.getTotalRows":{
        ref:"method:GridRenderer.getTotalRows",
        flags:"",
        description:" Return the total number of rows in the grid.<br><br>\n\n NOTE: in order to create a valid grid, you must either provide a totalRows value or implement\n getTotalRows()\n\n",
        name:"getTotalRows",
        seeAlso:[
            {ref:"attr:GridRenderer.totalRows", description:null}
        ],
        returns:{description:null, type:"number"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:DetailViewerField.imageHeight":{
        ref:"attr:DetailViewerField.imageHeight",
        flags:"IRW",
        description:"Height of image shown for fieldTypes image in this field.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n height.  For example, if <code>field.imageHeight</code> is \"logoHeight\",\n <code>record.logoHeight</code> will control the height of the image.",
        name:"imageHeight",
        seeAlso:[
            {ref:"attr:DetailViewerField.imageSize", description:null},
            {ref:"attr:DetailViewerField.imageWidth", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField",
        groups:["imageColumns"]
    },
    "method:Canvas.mouseOver":{
        ref:"method:Canvas.mouseOver",
        flags:"A",
        description:"\n Executed when mouse enters this widget.  No default implementation.\n\n",
        name:"mouseOver",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "group:pickerIcon":{
        ref:"group:pickerIcon",
        refs:["attr:SelectItem.pickerIconWidth", "attr:SelectItem.pickerIconHeight", 
         "attr:SelectItem.pickerIconSrc","attr:SelectItem.pickerIconProperties"],
        name:"pickerIcon",
        seeAlso:[],
        type:"group"
    },
    "method:TreeGrid.folderOpened":{
        ref:"method:TreeGrid.folderOpened",
        flags:"",
        description:"\n This method is called when a folder is opened either via the user manipulating the\n expand/collapse control in the UI or via ${isc.DocUtils.linkForRef('method:TreeGrid.openFolder')}.  You can return\n <code>false</code> to cancel the open.\n\n",
        name:"folderOpened",
        seeAlso:[],
        params:[
            {description:"the folder (record) that is being opened", optional:false, name:"node",
             defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"false to cancel the open, true to all it to proceed", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:ComboBoxItem.showPickerIcon":{
        ref:"attr:ComboBoxItem.showPickerIcon",
        flags:"IRWA",
        description:"Show the picker icon for the ComboBox.",
        name:"showPickerIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:Tree.dataChanged":{
        ref:"method:Tree.dataChanged",
        flags:"A",
        description:"\n Called when the structure of this tree is changed in any way.  Intended to be observed.\n <br><br>\n Note that on a big change (many items being added or deleted) this may be called multiple times\n\n",
        name:"dataChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:Tree"
    },
    "type:listGridViewState":{
        ref:"type:listGridViewState",
        description:"An object containing the \"view state\" information for a listGrid.<br>\n This object contains state information reflecting<br>\n - ${isc.DocUtils.linkForRef('type:listGridFieldState')}<br>\n - ${isc.DocUtils.linkForRef('type:listGridSortState')}<br>\n - ${isc.DocUtils.linkForRef('type:listGridSelectedState')}<br>\n for the grid.<br>\n Note that this object is not intended to be interrogated directly, but may be stored \n (for example) as a blob on the server for view state persistence across sessions.",
        name:"listGridViewState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "method:TabSet.setPaneContainerOverflow":{
        ref:"method:TabSet.setPaneContainerOverflow",
        flags:"",
        description:" Update ${isc.DocUtils.linkForRef('attr:TabSet.paneContainerOverflow')} after creation.\n\n",
        name:"setPaneContainerOverflow",
        seeAlso:[],
        params:[
            {description:"new overflow setting", optional:false, name:"newOverflow", defaultValue:null,
             type:"Overflow"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:ListGridField.groupingModes":{
        ref:"attr:ListGridField.groupingModes",
        flags:"IR",
        description:"If set, provides several possible styles of grouping that are valid for this field.  For\n example, a field of type:\"date\" might be able to be grouped by week, month, or by day of\n week.\n <P>\n If <code>groupingModes</code> are present and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canGroupBy','grouping is enabled')}, the menu for this field includes a\n submenu of possible grouping modes generated from the <code>groupingModes</code> valueMap.\n When the user selects a particular grouping mode,\n ${isc.DocUtils.linkForRef('attr:ListGridField.groupingMode','field.groupingMode')} is set to the user's chosen mode,\n and this choice can be detected via the <code>field</code> parameter to\n ${isc.DocUtils.linkForRef('method:ListGridField.getGroupValue')} in order to provide different modes of grouping.\n <P>\n The user may also choose to group records without specifying a grouping mode, in this case,\n the ${isc.DocUtils.linkForRef('attr:ListGridField.defaultGroupingMode')} is used.\n <P>\n Note that <code>getGroupValue</code>, <code>groupingModes</code> et al can be specified on\n ${isc.DocUtils.linkForRef('class:SimpleType')} declaration, and the different grouping modes that are offered\n automatically for various common types are defined this way.",
        name:"groupingModes",
        seeAlso:[],
        valueType:"ValueMap",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "attr:Calendar.initialCriteria":{
        ref:"attr:Calendar.initialCriteria",
        flags:"IR",
        description:"Criteria to be used when ${isc.DocUtils.linkForRef('attr:Calendar.autoFetchData')} is set.",
        name:"initialCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:DataBoundComponent.removeOperation":{
        ref:"attr:DataBoundComponent.removeOperation",
        flags:"IRW",
        description:"Operation ID this component should use when performing remove operations.",
        name:"removeOperation",
        seeAlso:[
            {ref:"attr:DSRequest.operationId", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["operations"]
    },
    "attr:DSResponse.invalidateCache":{
        ref:"attr:DSResponse.invalidateCache",
        flags:"R",
        description:"Optional flag that can be set by the server to force ResultSets to drop any caches of\n records from the DataSource that was the target of the operation.",
        name:"invalidateCache",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["cacheSync"]
    },
    "classMethod:Page.getScrollTop":{
        ref:"classMethod:Page.getScrollTop",
        flags:"",
        description:"		Get the amount that the browser window has been scrolled vertically.\n",
        name:"getScrollTop",
        seeAlso:[],
        returns:{description:"vertical scroll amount", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "method:FormItem.blur":{
        ref:"method:FormItem.blur",
        flags:"",
        description:" Called when this FormItem loses focus.\n \n",
        name:"blur",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "attr:StatefulCanvas.state":{
        flags:"IRWA",
        getter:["getState"],
        type:"attr",
        ref:"attr:StatefulCanvas.state",
        description:"Current \"state\" of this widget. StatefulCanvases will have a different appearance based\n on their current state. By default this is handled by changing the css className applied to\n the StatefulCanvas - see ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')} for a description of how this is\n done.<P>\n For ${isc.DocUtils.linkForRef('class:Img')} or ${isc.DocUtils.linkForRef('class:StretchImg')} based subclasses of StatefulCanvas, the \n appearance may also be updated by changing the src of the rendered image. See\n ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:StretchImgButton.src')} for a description of how the URL \n is modified to reflect the state of the widget in this case.",
        name:"state",
        setter:["setState"],
        valueType:"State",
        seeAlso:[
            {ref:"type:State", description:null},
            {ref:"group:state", description:null}
        ],
        defaultValue:"\"\"",
        groups:["state"],
        definingClass:"class:StatefulCanvas"
    },
    "method:ListGrid.setCanFreezeFields":{
        ref:"method:ListGrid.setCanFreezeFields",
        flags:"",
        description:" Setter method for ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields')}\n",
        name:"setCanFreezeFields",
        seeAlso:[],
        params:[
            {description:"New value for <code>listGrid.canFreezeFields</code>", optional:false,
             name:"canFreeze", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:DataSource.addSearchOperator":{
        ref:"method:DataSource.addSearchOperator",
        flags:"",
        description:" Add a new search operator, only to this DataSource.  \n <P>\n If an existing ${isc.DocUtils.linkForRef('object:Operator')} is passed, restricts the set of FieldTypes to which that\n operator can be applied in this DataSource.\n <P>\n See also ${isc.DocUtils.linkForRef('classMethod:DataSource.addSearchOperator')} for adding operators to all\n DataSources.\n\n",
        name:"addSearchOperator",
        seeAlso:[],
        params:[
            {description:"definition of the operator to add", optional:false, name:"operator",
             defaultValue:null, type:"Operator"},
            {description:"types to which this operator applies", optional:true, name:"types",
             defaultValue:null, type:"Array of FieldType"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "attr:RPCRequest.transport":{
        ref:"attr:RPCRequest.transport",
        flags:"IRWA",
        description:"Selects the transport used for this RPCRequest.  If unset, the value of\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')} will be used.\n <p>\n If you're using queueing, note that all requests in the queue must use the same transport.\n If you attempt to send a request via a different transport than those that are currently on\n the queue, it will be sent to the server separately, ahead of the queue, and a warning will\n be logged to the Developer Console.\n <p>\n If you specify an unknown transport, an error will be logged to the DeveloperConsole and\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.defaultTransport')} will be used instead.\n <p>\n If you specify the <code>xmlHttpRequest</code> transport and it is not available, a warning will be\n logged to the Developer Console and the RPCManager will attempt to use the\n <code>hiddenFrame</code> transport instead for this request.  Note that some features like\n ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} requre the <code>xmlHttpRequest</code> transport and will not\n work if the <code>xmlHttpRequest</code> transport is unavailable (this can happen if the end user is\n using Internet Explorer and has disabled ActiveX).  You can check whether or not the\n <code>xmlHttpRequest</code> transport is currently available by calling\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.xmlHttpRequestAvailable')}.",
        name:"transport",
        seeAlso:[
            {ref:"classAttr:RPCManager.defaultTransport", description:null}
        ],
        valueType:"RPCTransport",
        defaultValue:"RPCManager.defaultTransport",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:TileLayout.getTile":{
        ref:"method:TileLayout.getTile",
        flags:"",
        description:" Retrieve a tile by index.  \n <P>\n The TileLayout consistently uses this method to access tiles, in order to allow subclasses\n to create tiles on demand.\n\n",
        name:"getTile",
        seeAlso:[],
        params:[
            {description:"index of the tile", optional:false, name:"index", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"the tile", type:"Canvas"},
        type:"method",
        definingClass:"class:TileLayout"
    },
    "method:MultiView.selectRecords":{
        ref:"method:MultiView.selectRecords",
        flags:"",
        description:"\n Select/deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n\n",
        name:"selectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to select", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:ColumnTree.closedIconSuffix":{
        ref:"attr:ColumnTree.closedIconSuffix",
        flags:"IRW",
        description:"This suffix will be appended to the ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')} for closed folders.\n If ${isc.DocUtils.linkForRef('attr:ColumnTree.showOpenIcons')} is set to <code>false</code> this suffix will also be\n appended to open folders' icons.",
        name:"closedIconSuffix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"closed\"",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "attr:SelectItem.filterLocally":{
        ref:"attr:SelectItem.filterLocally",
        flags:"IRA",
        description:"If <code>filterLocally</code> is set for this item, and this item is showing options \n from a dataSource, fetch the entire set of options from the server, and use these values\n to map the item value to the appropriate display value. Also use <code>\"local\"</code>\n type filtering on drop down list of options.\n <P>\n This means data will only be fetched once from the server, and then filtered on the\n client.",
        name:"filterLocally",
        seeAlso:[
            {ref:"attr:FormItem.filterLocally", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "type:State":{
        ref:"type:State",
        values:[
            {description:"state when mouse is not acting on this StatefulCanvas",
             value:"StatefulCanvas.STATE_UP"},
            {description:"state when mouse is down", value:"StatefulCanvas.STATE_DOWN"},
            {description:"state when mouse is over", value:"StatefulCanvas.STATE_OVER"},
            {description:"disabled", value:"StatefulCanvas.STATE_DISABLED"}
        ],
        description:"Constants for the standard states for a StatefulCanvas.",
        name:"State",
        seeAlso:[],
        type:"type",
        groups:["state"]
    },
    "attr:Canvas.contents":{
        flags:"IRWA",
        getter:["getContents"],
        type:"attr",
        ref:"attr:Canvas.contents",
        description:"The contents of a canvas or label widget. Any HTML string is acceptable.",
        name:"contents",
        setter:["setContents"],
        valueType:"string",
        seeAlso:[
            {ref:"attr:Canvas.dynamicContents", description:null}
        ],
        defaultValue:"\"&nbsp;\"",
        groups:["contents"],
        definingClass:"class:Canvas"
    },
    "method:SimpleTile.getInnerHTML":{
        ref:"method:SimpleTile.getInnerHTML",
        flags:"",
        description:" The default implementation will call ${isc.DocUtils.linkForRef('method:TileGrid.getTileHTML')}.\n",
        name:"getInnerHTML",
        seeAlso:[],
        returns:{description:"HTML contents for the tile, as a String", type:"HTML"},
        type:"method",
        definingClass:"class:SimpleTile"
    },
    "attr:TabBar.baseLineSrc":{
        ref:"attr:TabBar.baseLineSrc",
        flags:"IR",
        description:"Sets ${isc.DocUtils.linkForRef('attr:StretchImg.src')} for the ${isc.DocUtils.linkForRef('group:baseLine')} StretchImg.",
        name:"baseLineSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]baseline.gif\"",
        type:"attr",
        definingClass:"class:TabBar",
        groups:["baseLine"]
    },
    "attr:ImgSectionHeader.controls":{
        ref:"attr:ImgSectionHeader.controls",
        flags:"IR",
        description:"Custom controls to be shown on top of this section header.\n <P>\n These controls are shown in the ${isc.DocUtils.linkForRef('attr:ImgSectionHeader.controlsLayout')}.",
        name:"controls",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sectionControls')}"],
        definingClass:"class:ImgSectionHeader"
    },
    "classMethod:Date.parseInput":{
        ref:"classMethod:Date.parseInput",
        flags:"",
        description:" Parse a date passed in as a string, returning the approprate date object.\n",
        name:"parseInput",
        seeAlso:[],
        params:[
            {description:"date value as a string", optional:false, name:"dateString", defaultValue:null,
             type:"string"},
            {description:"Format of the date string being passed.                                      If not passed, the default date input format as set up\n                                      via setInputFormat() will be used.",
             optional:true, name:"format", defaultValue:null, type:"DateInputFormat"},
            {description:"For date formats that support a 2 digit                                  year, if parsed year is 2 digits and less than this\n                                  number, assume year to be 20xx rather than 19xx",
             optional:true, name:"centuryThreshold", defaultValue:null, type:"number"},
            {description:"If the string passed in was not a valid date, in some cases we can convert to a\n          valid date (for example incrementing the year if the month is greater than 12).\n          This optional parameter will suppress such conversions - anything that doesn't\n          parse directly to a valid date will simply return null.",
             optional:true, name:"suppressConversion", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"date value, or null if the string could not be parsed to a valid date.",
         type:"Date"},
        type:"classMethod",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "type:OperatorValueType":{
        ref:"type:OperatorValueType",
        values:[
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.value')} should contain a value of the same type as the                    field.",
             value:"\"fieldType\""},
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.value')} should be the name of another field in the record",
             value:"\"fieldName\""},
            {description:"no criterion.value or other setting required (used for operators like                    isNull).",
             value:"\"none\""},
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.criteria')} should be an Array of criteria (used for logical                   operators like \"and\").",
             value:"\"criteria\""},
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.start')} and ${isc.DocUtils.linkForRef('attr:Criterion.end')} should contain start and                     end values, both of the same type as the field.",
             value:"\"valueRange\""},
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.value')} should contain an Array of valid field values.",
             value:"\"valueSet\""},
            {description:"${isc.DocUtils.linkForRef('attr:Criterion.value')} should contain a value which is not necessarily the                 same type as the field (used for regexp and similar operators).  \n                 ${isc.DocUtils.linkForRef('attr:Operator.editorType')} can be specified as a FormItem to use to\n                 enter a value for the criterion.",
             value:"\"custom\""}
        ],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        description:"Indicates the kind of value expected in a ${isc.DocUtils.linkForRef('object:Criterion')} that uses this operator.",
        name:"OperatorValueType",
        seeAlso:[],
        type:"type",
        serverDS:"allowed",
        groups:["advancedFilter"]
    },
    "attr:ListGrid.summaryRowHeight":{
        ref:"attr:ListGrid.summaryRowHeight",
        flags:"IR",
        description:"Height for the ${isc.DocUtils.linkForRef('attr:ListGrid.summaryRow','summary row autoChild')}.",
        name:"summaryRowHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:DataSource.requiresAuthentication":{
        ref:"attr:DataSource.requiresAuthentication",
        flags:"IR",
        description:"Whether a user must be authenticated in order to access this DataSource.  This establishes a\n default for the DataSource as a whole; individual ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings')} within the\n DataSource may still override this setting by explicitly setting\n ${isc.DocUtils.linkForRef('attr:OperationBinding.requiresAuthentication')}.\n <P>\n Whether the user is authenticated is determined by calling\n <code>httpServletRequest.getRemoteUser()</code>, hence works with both simple J2EE security\n (realms and form-based authentication) and JAAS (Java Authentication & Authorization\n Service).\n <P>\n If you wish to use an authentication scheme that does not make use of the servlet API's \n standards, SmartClient Server also implements the <code>setAuthenticated</code> method\n on <code>RPCManager</code>.  You can use this API to tell SmartClient that all the \n requests in the queue currently being processed are associated with an authenticated\n user; in this case, SmartClient will not attempt to authenticate the user via \n <code>httpServletRequest.getRemoteUser()</code>\n <P>\n You can set the default value for this property via setting \"authentication.defaultRequired\"\n in server.properties.  This allows you to, for example, cause all DataSources to require\n authentication for all operations by default.\n <P>\n Note that setting this property does not automatically cause an authentication mechanism to\n appear - you still need to separately configure an authentication system.  Likewise, setting\n requiresAuthentication=\"false\" does not automatically allow users to bypass your authentication\n mechanism - you need to set up a URL that will accept DSRequests and process them similar to\n the default \"IDACall\" servlet, and which is not protected by the authentication system.  See\n ${isc.DocUtils.linkForRef('group:servletDetails','Deploying SmartClient')} for details on the IDACall servlet.",
        name:"requiresAuthentication",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSource",
        groups:["auth"],
        requiresModules:["SCServer"]
    },
    "attr:ToolStripSeparator.skinImgDir":{
        ref:"attr:ToolStripSeparator.skinImgDir",
        flags:"IR",
        description:"Path to separator image.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"images/ToolStrip/\"",
        type:"attr",
        definingClass:"class:ToolStripSeparator"
    },
    "attr:TabSet.canCloseTabs":{
        ref:"attr:TabSet.canCloseTabs",
        flags:"IRW",
        description:"Should tabs in this tabSet show an icon allowing the user to dismiss the tab by\n clicking on it directly. May be overridden for individual tabs by setting \n ${isc.DocUtils.linkForRef('attr:Tab.canClose')}.\n <P>\n The URL for this icon's image will be derived from  ${isc.DocUtils.linkForRef('attr:TabSet.closeTabIcon')} by \n default, but may be overridden by explicitly specifying ${isc.DocUtils.linkForRef('attr:Tab.closeIcon')}.\n <P>\n <b>Note</b>: Currently, tabs can only show a single icon, so a closable tab will show\n the close icon only even if ${isc.DocUtils.linkForRef('attr:Tab.icon')} is set.  To work around this, add the icon\n as an HTML &lt;img&gt; tag to the ${isc.DocUtils.linkForRef('attr:Tab.title')} property, for example:\n <pre>\n    title : \"<span>\" + isc.Canvas.imgHTML(\"myIcon.png\") + \" Tab Title</span>\"\n </pre>",
        name:"canCloseTabs",
        seeAlso:[
            {ref:"method:TabSet.closeClick", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:Slider.title":{
        ref:"attr:Slider.title",
        flags:"IRW",
        description:"Optional display title for the slider.",
        name:"title",
        seeAlso:[
            {ref:"attr:Slider.showTitle", description:null}
        ],
        valueType:"String",
        defaultValue:"\"Set Value\"",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:SelectItem.pickerIconWidth":{
        ref:"attr:SelectItem.pickerIconWidth",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:SelectItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset, the picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["pickerIcon"]
    },
    "method:List.set":{
        ref:"method:List.set",
        flags:"",
        description:" Change the array element at a particular position.\n <P>\n set() can be used to expand the length of the list.\n\n",
        name:"set",
        seeAlso:[],
        params:[
            {description:"position in the list to change", optional:false, name:"pos",
             defaultValue:null, type:"number"},
            {description:"new value for that position", optional:false, name:"obj", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"whatever's at that position, null if not found", type:"object"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:Window.dismissOnEscape":{
        ref:"attr:Window.dismissOnEscape",
        flags:"IRW",
        description:"Should this window be dismissed (same effect as pressing the \"Cancel\" button) when the \n user presses the \"Escape\" key? Behavior will only occur while the window or one of its\n descendants has focus, and does not cancel the Escape keypress.\n <P>\n If unset default behavior depends on whether a close / cancel button is visible for\n this item.",
        name:"dismissOnEscape",
        seeAlso:[
            {ref:"method:Window.shouldDismissOnEscape", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:Canvas.shadowOffset":{
        ref:"attr:Canvas.shadowOffset",
        flags:"IRA",
        description:"Offset of the shadow.  Defaults to half of <code>shadowDepth</code> if unset.\n <P>\n Because of the blurred edges, a shadow is larger than the originating component by\n 2xsoftness.  An <code>shadowOffset</code> of 0 means that the shadow will extend around the\n originating component equally in all directions.",
        name:"shadowOffset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('shadows')}"],
        definingClass:"class:Canvas",
        groups:["shadow"]
    },
    "method:CubeGrid.facetValueOut":{
        ref:"method:CubeGrid.facetValueOut",
        flags:"",
        description:"    StringMethod handler fired when mouseout occurs for a facetValues header button\n  <br><i>methodType</i> handler\n",
        name:"facetValueOut",
        seeAlso:[],
        params:[
            {description:"facetValuesObject for the appropriate header button", optional:false,
             name:"facetValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "method:Tree.find":{
        ref:"method:Tree.find",
        flags:"",
        description:"\n Find a node within this tree using a string path or by attribute value(s).  This method can be\n called with 1 or 2 arguments. If a single String\n argument is supplied, the value of the argument is treated as the path to the node.  If a \n single argument of type Object is provided, it is treated as a set of field name/value \n pairs to search for (see ${isc.DocUtils.linkForRef('method:List.find')}).\n <br>\n If 2 arguments are supplied, this method will treat the first argument as a fieldName, and\n return the first node encountered where <code>node[fieldName]</code> matches the second \n argument.  So for example, given this tree:\n <pre>\n foo\n   zoo\n     bar\n   moo\n     bar\n </pre>\n Assuming your ${isc.DocUtils.linkForRef('attr:Tree.pathDelim')} is the default <code>/</code> and <code>foo</code>\n is the name of the root node, then\n <code>tree.find(\"foo/moo/bar\")</code> would return the <code>bar</code> node under the\n <code>moo</code> node.\n <br><br>\n <code>tree.find(\"name\", \"bar\")</code> would return the first <code>bar</code> node because\n it is the first one in the list whose <code>name</code> (default value of\n ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')}) property matches the value\n <code>bar</code>.  The two argument usage is generally more interesting when your tree nodes\n have some custom unique property that you wish to search on.  For example if your tree nodes\n had a unique field called \"UID\", their serialized form would look something like this:\n <pre>\n { name: \"foo\", children: [...], UID:\"someUniqueId\"}\n </pre> \n You could then call <code>tree.find(\"UID\", \"someUniqueId\")</code> to find that node.  Note\n that the value doesn't have to be a string - it can be any valid JS value, but since this\n data generally comes from the server, the typical types are string, number, and boolean.\n <br><br>\n The usage where you pass a single object is interesting when your tree nodes have a number\n of custom properties that you want to search for in combination.  Say your tree nodes had \n properties for \"color\" and \"shape\"; <code>tree.find({color: \"green\", shape: \"circle\"})</code>\n would return the first node in the tree where both properties matched.\n <br><br>\n When searching by path, trailing path delimiters are ignored.  So for example\n <code>tree.find(\"foo/zoo/bar\")</code> is equivalent to\n <code>tree.find(\"foo/zoo/bar/\")</code>\n \n",
        name:"find",
        seeAlso:[
            {ref:"attr:Tree.root", description:null},
            {ref:"attr:Tree.pathDelim", description:null},
            {ref:"attr:Tree.nameProperty", description:null}
        ],
        params:[
            {description:"Either the path to the node to be found, or the name of                                      a field which should match the value passed as a second\n                                      parameter",
             optional:false, name:"fieldNameOrPath", defaultValue:null, type:"string"},
            {description:"If specified, this is the desired value for the                                    appropriate field",
             optional:true, name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"the node matching the supplied criteria or null if not found", type:"object"},
        type:"method",
        definingClass:"class:Tree",
        groups:["location"]
    },
    "group:operations":{
        ref:"group:operations",
        title:"Operations Overview",
        treeLocation:["Client Reference/Data Binding"],
        refs:["method:DataSource.fetchData", "method:DataSource.filterData", 
         "method:DataSource.exportData","method:DataSource.addData", "method:DataSource.updateData", 
         "method:DataSource.removeData","method:DataSource.performCustomOperation", "classMethod:RPCManager.handleError", 
         "attr:DSRequest.operationId","attr:DataBoundComponent.fetchOperation", "attr:DataBoundComponent.updateOperation", 
         "attr:DataBoundComponent.addOperation","attr:DataBoundComponent.removeOperation"],
        description:"SmartClient Operations are dynamic, transparent communications made from the client-side\n SmartClient system running in the browser, to the server-side SmartClient system running in\n a servlet engine, or to other non-SmartClient servers available via HTTP. Operations are\n used to load new data or new behavior into a running SmartClient application. Operations are\n also used to save data entered by users, and in general, to get the result of any process\n which must be run on the server for security reasons.\n <br><br>\n\n <b>RPC Operations</b>\n <br><br>\n RPC Operations are low-level communications that send and retrieve arbitrary data.  RPC\n Operations are supported by the ${isc.DocUtils.linkForRef('class:RPCManager')} class, which when used with the\n SmartClient server, provides Java to JavaScript ${isc.DocUtils.linkForRef('attr:RPCRequest.data','2-way translation')} of\n basic data structures.  The RPCManager also provides a mechanism for client-side code to be\n invoked when an operation completes (called a \"callback\").  RPC Operations are intended for\n unstructured data; data that is ultimately destined for display in SmartClient components\n will generally come from DataSource operations.\n <br><br>\n\n <b>DataSource Operations and DataBound Components</b>\n <br><br>\n A ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource Operation')} is an operation that acts on a\n DataSource, performing one of the basic actions that makes sense on a set of similar\n records: \"fetch\", \"add\", \"update\" or \"remove\".  Unlike RPC operations, DataSource operations\n have specific request data and response data, for example, in the \"fetch\" DataSource\n operation, the request data is expected to be search criteria, and the response data is\n expected to be a list of matching DataSource records.  Although DataSource operations can be\n invoked manually from the client, they are generally automatically invoked by DataBound\n components.\n <br><br>\n DataBound Components are components that understand DataSources.  Databound components\n configured with a DataSource are able to offer complete user interactions without further\n configuration (extensive customization is also supported).\n <br><br>\n For example, given a DataSource, the ListGrid component supports a sophisticated inline\n editing interaction, complete with automatically chosen editors like date pickers for dates,\n type-aware validation, saving, and error reporting. \n <br><br>\n A DataBound component supporting an interaction such as inline editing will automatically\n submit DataSource operations to the server at appropriate times.\n <br><br>\n\n <b>DataSource Operation Integration</b>\n <br><br>\n Integrating DataSource operations with an existing system is best approached by implementing\n the the 4 basic DataSource operations in terms of your existing object model or data store.\n With these 4 operations implemented, the entire range of user interactions supported by\n SmartClient ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databinding-capable components')} becomes applicable to\n your server.  At that point authentication, authorization and other business rules can be\n layered on top.\n <br><br>\n\n <b>Built-in SQL Connectivity</b>\n <br><br>\n The SmartClient Server comes with a built-in ${isc.DocUtils.linkForRef('group:sqlDataSource','SQLDataSource')}\n which can be used without any server-side code needing to be written.  In contrast,\n any operation which uses custom server-side code is called a \"Custom Operation\".\n <br><br>\n Generally it makes sense to prototype an application using Built-in DataSource Operations,\n then on the backend, create Custom DataSource Operations to retrieve data from the data\n store you will use in production (though don't rule out using the SQL DataSource in \n production - see ${isc.DocUtils.linkForRef('group:sqlVsJPA','this discussion')} of the advantages of doing so}.  \n As you switch from using Built-in DataSources to Custom Operations, no client-side code \n changes will be required, because the client cares only about the DataSource definition, \n not the data store which the data is ultimately retrieved from.\n <br><br>\n\n <b>Data Managers: ResultSet and ResultTree</b>\n <br><br>\n Data Managers manage datasets retrieved from DataSources.  Data Managers are automatically\n created by DataBound components, but can be created directly when more control is needed.\n <br><br>\n Data Managers provide load-on-demand for datasets too large to be loaded on the client,\n automatically invoking DataSource operations as necessary to retrieve data as it is\n requested, and optionally fetching ahead to anticipate further requests.   Data Managers\n will automatically perform actions locally when it is possible, for example, a sort\n action can be performed locally with a complete cache.  Data Managers also automatically\n manage the consistency of the client-side cache, observing update operations performed\n against DataSources and integrating updated rows automatically.\n <br><br>",
        name:"operations",
        seeAlso:[
            {ref:"class:RPCManager", description:"for RPC Operations"},
            {ref:"interface:DataBoundComponent",
             description:"for information on DataBound Components"},
            {ref:"group:dataSourceOperations",
             description:"for more information on DataSource Operations"},
            {ref:"group:clientServerIntegration",
             description:"for information on integrating DataSource Operations with existing servers"},
            {ref:"method:DataSource.fetchData",
             description:"for manually invoked DataSource operations"},
            {ref:"class:ResultSet", description:"for managing lists of records"},
            {ref:"class:ResultTree", description:"for managing trees of records"}
        ],
        type:"group"
    },
    "method:ValuesManager.handleHiddenValidationErrors":{
        ref:"method:ValuesManager.handleHiddenValidationErrors",
        flags:"A",
        description:" Method to display validation error messages for a valuesManager when there is not\n currently visible form item displaying the errors.\n This will be called when validation fails for<br>\n - a field in a hidden or undrawn member form<br>\n - a hidden field in a visible member form<br>\n - for databound ValuesManagers, a datasource field with specified validators, but not\n   associated item in any member.<br>\n Implement this to provide custom validation error handling for these fields.<br>\n By default hidden validation errors will be logged as warnings in the developerConsole.\n Return false from this method to suppress that behavior.\n",
        name:"handleHiddenValidationErrors",
        seeAlso:[],
        params:[
            {description:"The set of errors returned - this is an object of the form<br>                      &nbsp;&nbsp;<code>{fieldName:errors}</code><br>\n                      Where the 'errors' object is either a single string or an array\n                      of strings containing the error messages for the field.",
             optional:false, name:"errors", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "group:menuItems":{
        ref:"group:menuItems",
        refs:["method:Menu.getItem", "method:Menu.getItemNum"],
        name:"menuItems",
        seeAlso:[],
        type:"group"
    },
    "classMethod:FileLoader.cacheImgStates":{
        ref:"classMethod:FileLoader.cacheImgStates",
        flags:"",
        description:"\n Caches a set of state images derived from a base image.  This is useful for caching a set of\n images for a single component.  For example this code:\n <pre>\n isc.FileLoader.cacheImgStates(\"/isomorphic/skins/SmartClient/images/TreeGrid/opener.png\",\n                               \"closed,opening,opened\");\n </pre>\n Will cause the following images to be cached:\n <pre>\n /isomorphic/skins/SmartClient/images/TreeGrid/opener_closed.png\n /isomorphic/skins/SmartClient/images/TreeGrid/opener_opening.png\n /isomorphic/skins/SmartClient/images/TreeGrid/opener_opened.png\n </pre>\n\n",
        name:"cacheImgStates",
        seeAlso:[
            {ref:"classMethod:FileLoader.cacheStretchImgStates", description:null}
        ],
        params:[
            {description:"List of base image URLs from which stateful names are to be                                    derived.  This can either be a\n                                    string with comma separated baseURLs or an Array of baseURLs.",
             optional:false, name:"baseURLs", defaultValue:null, type:"String or Array"},
            {description:"List of states to load.  If none specified, The following                                    default list will be used:\n                                    \"Down,Over,Selected,Selected_Down,Selected_Over\".  This can\n                                    either be a string with comma separated states or an Array\n                                    of states.",
             optional:true, name:"states", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified URLs                                      has completed caching.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "attr:StretchImg.showTitle":{
        ref:"attr:StretchImg.showTitle",
        flags:"IRWA",
        description:"Determines whether any specified ${isc.DocUtils.linkForRef('method:StatefulCanvas.getTitle','title')} will be \n displayed for this component.<br>\n Applies to Image-based components only, where the title will be rendered out in a label\n floating over the component",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImg"
    },
    "attr:TabSet.bottomEdgeOffsets":{
        ref:"attr:TabSet.bottomEdgeOffsets",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeOffsets</code>, \n <code>rightEdgeOffsets</code>, <code>topEdgeOffsets</code> and <code>bottomEdgeOffsets</code> \n propertes allow the offsets of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge offsets where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"bottomEdgeOffsets",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:DataSource.dbName":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:DataSource.dbName",
        description:"For DataSources using the ${isc.DocUtils.linkForRef('group:sqlDataSource','SmartClient SQL engine')} for\n persistence, which database configuration to use.  Database configurations can be\n created using the \"Databases\" tab in the Developer Console.  If unset, the default\n database configuration is used (which is also settable using the \"Databases\" tab).",
        name:"dbName",
        valueType:"String",
        seeAlso:[],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "attr:ListGrid.loadingDataMessageStyle":{
        ref:"attr:ListGrid.loadingDataMessageStyle",
        flags:"IRW",
        description:"The CSS style name applied to the loadingDataMessage string if displayed.",
        name:"loadingDataMessageStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"loadingDataMessage\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["loadingDataMessage"]
    },
    "classMethod:Page.waitForMultiple":{
        ref:"classMethod:Page.waitForMultiple",
        flags:"",
        description:" Wait for methods to fire on multiple objects.\n <P>\n <code>waitForMultiple</code> is similar to ${isc.DocUtils.linkForRef('classMethod:Page.waitFor')}, except that it does not fire\n its callback until all of the provided methods have fired.\n\n",
        name:"waitForMultiple",
        seeAlso:[],
        params:[
            {description:"an array of objects, each of which consists of two properties:<br/>                        \"object\": any SmartClient object, eg a ListGrid<br/>\n                        \"method\": name of a method on that object",
             optional:false, name:"objects", defaultValue:null, type:"Array"},
            {description:"Callback to fire when all observed methods have fired", optional:false,
             name:"callback", defaultValue:null, type:"Function"},
            {description:"Optional timeout period (in milliseconds). If you want a timeout,                           you must also provide a timeoutCallback",
             optional:true, name:"timeout", defaultValue:null, type:"Number"},
            {description:"Callback to fire if the timeout period elapses before all                                      observed methods have fired",
             optional:true, name:"timeoutCallback", defaultValue:null, type:"Function"}
        ],
        returns:{description:"whether observation succeeded.  Observation may fail due to null objects,                   non-existant methods or similar bad parameters",
         type:"boolean"},
        type:"classMethod",
        definingClass:"class:Page"
    },
    "method:Array.and":{
        ref:"method:Array.and",
        flags:"",
        description:" Returns true if all values between the start and end indices are true.\n\n",
        name:"and",
        seeAlso:[],
        params:[
            {description:"optional start index (default is 0)", optional:true, name:"start",
             defaultValue:null, type:"number"},
            {description:"optional end index (default is list.length)", optional:true, name:"end",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"all of the items in the array are true", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "attr:Hilite.id":{
        ref:"attr:Hilite.id",
        flags:"IR",
        description:"Unique id for this hilite definition.  \n <P>\n For hilites that include ${isc.DocUtils.linkForRef('attr:Hilite.criteria')} this is not required.\n <P>\n If you are explicitly marking records for hiliting, set\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.hiliteProperty')} on the record to this id.",
        name:"id",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "method:ColorPicker.colorChanged":{
        ref:"method:ColorPicker.colorChanged",
        flags:"",
        description:" Override this method to be kept informed when the ColorPicker changes in real-time \n (for example, if you need to update your own GUI accordingly). Then use the \n getXxxx() methods (for example, ${isc.DocUtils.linkForRef('method:ColorPicker.getBlue','getBlue()')} or \n ${isc.DocUtils.linkForRef('method:ColorPicker.getLuminosity','getLuminosity()')})to obtain current state as required. \n",
        name:"colorChanged",
        seeAlso:[
            {ref:"method:ColorPicker.colorSelected", description:null}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:ImgButton.labelVPad":{
        ref:"attr:ImgButton.labelVPad",
        flags:"IRW",
        description:"Vertical padding to be applied to this widget's label. If this value is null, \n the label will be given a vertial padding of zero.\n <p>\n The specified amount of padding is applied to the top and bottom edges of the button, so\n the total amount of padding is 2x the specified value.",
        name:"labelVPad",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton"
    },
    "attr:ResultTree.defaultNewNodesToRoot":{
        ref:"attr:ResultTree.defaultNewNodesToRoot",
        flags:"IRWA",
        description:"This attribute governs how to handle cache-synch when a new node is added to this dataSource \n with no explicit parentId.\n <P>\n If set to <code>true</code>, when a new node is added to this dataSource via\n ${isc.DocUtils.linkForRef('method:DataSource.addData')}, with no explicit parentId, the node will be added as a \n child of the root node of this result tree. Otherwise it will be ignored.\n <P>\n Similar logic applies to ${isc.DocUtils.linkForRef('method:DataSource.updateData','updated nodes')} - if this property is\n true and the parentId of an updated node is cleared, it will be moved to become a child of\n root, otherwise it will be dropped from the tree.",
        name:"defaultNewNodesToRoot",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "attr:Canvas.animateHideTime":{
        ref:"attr:Canvas.animateHideTime",
        flags:"IRWA",
        description:"Default time for performing an animated hide.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateHideTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:ListGrid.canAddSummaryFields":{
        ref:"attr:ListGrid.canAddSummaryFields",
        flags:"IRW",
        description:"Adds an item to the header context menu allowing users to launch a dialog to define a new\n text field that can contain both user-defined text and the formatted values present in other \n fields, using the ${isc.DocUtils.linkForRef('class:SummaryBuilder')}.\n <P>\n User-added summary fields can be persisted via ${isc.DocUtils.linkForRef('method:ListGrid.getFieldState')} and \n ${isc.DocUtils.linkForRef('method:ListGrid.setFieldState')}.",
        name:"canAddSummaryFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["summaryFields"]
    },
    "attr:FormItem.operator":{
        ref:"attr:FormItem.operator",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('type:OperatorId')} to be used when ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')} is called.\n <P>\n <code>item.operator</code> can be used to create a form that offers search functions such\n as date range filtering, without the more advanced user interface of the ${isc.DocUtils.linkForRef('class:FilterBuilder')}.\n <P>\n When <code>item.operator</code> is set for any FormItem in a form,\n <code>form.getValuesAsCriteria()</code> will return an ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} object\n instead of a normal ${isc.DocUtils.linkForRef('type:Criteria')} object.  Each FormItem will produce one\n ${isc.DocUtils.linkForRef('object:Criterion')} affecting the DataSource field specified by ${isc.DocUtils.linkForRef('attr:FormItem.criteriaField')}.\n The criteria produced by the FormItems will be grouped under the logical operator\n provided by ${isc.DocUtils.linkForRef('attr:DynamicForm.operator')}.\n <P>\n if <code>operator</code> is set for some fields but not others, the default operator is\n \"equals\" for fields with a valueMap or an optionDataSource, and for fields of type \"enum\"\n (or of a type that inherits from \"enum\").  The default operator for all other fields is\n \"iContains\" (case-insensitive match) \n <P>\n <b>Note:</b> Advanced criteria will only be created for items in a Databound form. If this\n item is part of a form with no ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','dataSource')}, the \n <code>operator</code> attribute will have no effect.",
        name:"operator",
        seeAlso:[],
        valueType:"OperatorId",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:MultiSortDialog.getNumLevels":{
        ref:"method:MultiSortDialog.getNumLevels",
        flags:"",
        description:" Return the number of levels of sorting that have been configured.\n\n",
        name:"getNumLevels",
        seeAlso:[],
        returns:{description:"The number of levels of sorting that have been configured", type:"number"},
        type:"method",
        definingClass:"class:MultiSortDialog"
    },
    "class:FormulaBuilder":{
        methods:["method:FormulaBuilder.setFormula", "method:FormulaBuilder.getHelpText", 
         "method:FormulaBuilder.getUpdatedFieldObject","method:FormulaBuilder.testFunction", "method:FormulaBuilder.getTestRecord", 
         "method:FormulaBuilder.save","method:FormulaBuilder.fireOnClose", "method:FormulaBuilder.getSamplePrompt"],
        ref:"class:FormulaBuilder",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:FormulaBuilder.dataSource", "attr:FormulaBuilder.fields", 
         "attr:FormulaBuilder.field","attr:FormulaBuilder.editMode", "attr:FormulaBuilder.formulaField", 
         "attr:FormulaBuilder.titleField","attr:FormulaBuilder.showHelpIcon", "attr:FormulaBuilder.helpIcon", 
         "attr:FormulaBuilder.autoHideCheckBoxLabel","attr:FormulaBuilder.showAutoHideCheckBox", "attr:FormulaBuilder.autoHideCheckBox", 
         "attr:FormulaBuilder.builderTypeText","attr:FormulaBuilder.helpTextIntro", "attr:FormulaBuilder.mathFunctions", 
         "attr:FormulaBuilder.cancelled","attr:FormulaBuilder.fieldKey", "attr:FormulaBuilder.instructionsTextStart", 
         "attr:FormulaBuilder.instructions","attr:FormulaBuilder.cancelButton", "attr:FormulaBuilder.testButton", 
         "attr:FormulaBuilder.saveButton","attr:FormulaBuilder.autoTest", "attr:FormulaBuilder.autoTestDelay", 
         "attr:FormulaBuilder.testRecord","attr:FormulaBuilder.samplePrompt"],
        description:"Shows an interface allowing a user to enter simple formulas by typing them into a text\n field.\n <P>\n Available values for the formula are determined by the DataSource fields, and are given\n simple single-letter aliases (such as \"A\", \"B\", ...) similar to column names in Excel.\n The set of available values is shown in the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.fieldKey')} as a simple\n mapping between the ${isc.DocUtils.linkForRef('attr:DataSourceField.title','field title')} and it's short name.\n <P>\n By default, available math functions are shown in a hover from the\n ${isc.DocUtils.linkForRef('attr:FormulaBuilder.helpIcon','helpIcon')} that appears after the formula field.",
        name:"FormulaBuilder",
        seeAlso:[],
        type:"class",
        groups:["formulaFields"]
    },
    "method:FormItem.setValueIcons":{
        ref:"method:FormItem.setValueIcons",
        flags:"",
        description:" Set the valueIcons for this item\n",
        name:"setValueIcons",
        seeAlso:[],
        params:[
            {description:"mapping of logical values for this item to icon src URLs", optional:false,
             name:"map", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "group:dataSourceDeclaration":{
        ref:"group:dataSourceDeclaration",
        title:"Creating DataSources",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        refs:[],
        description:"DataSources can be specified in XML format, in which case the ISC server is used to load the\n DataSource in a JSP file, or DataSources can be directly created on the client via\n JavaScript.\n <P>\n Whether a DataSource is specified in XML or JS, identical requests will ultimately be\n submitted to the server, so the server integration pattern is the same.  However,\n DataSources created in XML can be loaded and used by the SmartClient Server,\n enabling many features including synchronized client-server validation, request bundling, \n file upload, and optional automatic SQL/Hibernate connectivity (see the\n ${isc.DocUtils.linkForRef('group:iscServer','Server Summary')} for details).\n <P>\n DataSources created on the client use the same style of creation as DataBound components:\n <pre>\n    isc.DataSource.create({\n        ID:\"supplyItem\",\n        fields:[\n            {name:\"itemName\", ... }\n            ...\n        ]\n    });\n </pre>\n Reference for all properties that can be set for DataSources, their fields and validators is\n given in the ${isc.DocUtils.linkForRef('class:DataSource')} class reference.\n <P>\n XML DataSources use a direct analogue of this format:\n <pre>\n     &lt;DataSource ID=\"supplyItem\"&gt;\n         &lt;fields&gt;\n             &lt;field name=\"itemName\" type=\"text\" title=\"Item\"/&gt;\n             &lt;field name=\"SKU\"      type=\"text\" title=\"SKU\"&gt;\n                 &lt;validators&gt;\n                     &lt;validator type=\"integerRange\" ... /&gt;\n                 &lt;/validators&gt;\n             &lt;/field&gt;\n         &lt;/fields&gt;\n     &lt;/DataSource&gt;\n </pre>\n XML DataSources are loaded via a special JSP tag supported by the SmartClient Server:\n <pre>\n     &lt;%&#64; taglib uri=\"/WEB-INF/iscTaglib.xml\" prefix=\"isomorphic\" %&gt;\n     ...\n     &lt;SCRIPT&gt;\n     &lt;isomorphic:loadDS ID=\"supplyItem\"/&gt;\n     &lt;/SCRIPT&gt;\n </pre>\n Alternatively, XML DataSources can be loaded by targetting a special servlet provided by \n the SmartClient Server.  This servlet yields exactly the same Javascript as the equivalent\n <code>&lt;isomorphic:loadDS/&gt;</code>, so the two methods are interchangeable.  The\n servlet-based method is ideal in environments where JSP tags cannot be used (SmartGWT is \n one such environment).  Example usage:\n <pre>\n     &lt;SCRIPT SRC=isomorphic/DataSourceLoader?dataSource=supplyItem,employees,worldDS&lt;/SCRIPT&gt;\n </pre>\n When loading an XML DataSource, by default, the ISC Server will look for a file named\n <code>&lt;dataSourceId&gt;.ds.xml</code> in the <code>/shared/ds</code> subdirectory under\n webroot.  The location of this directory can be changed, or individual DataSources can be\n placed in arbitrary locations.  For more information, see\n <code>[webroot]/WEB-INF/classes/server.properties</code>.\n <p>",
        name:"dataSourceDeclaration",
        seeAlso:[
            {ref:"class:DataSource", description:null},
            {ref:"group:loadDSTag", description:null}
        ],
        type:"group"
    },
    "attr:RPCRequest.actionURL":{
        ref:"attr:RPCRequest.actionURL",
        flags:"IRW",
        description:"Overrides RPCManager.actionURL for this request only.  If you're using queuing, note that queues\n as per-URL - in other words all RPCRequests in a queue must go to a single URL.  If you attempt\n to send a request with an actionURL that is different from those already in the queue, it\n will be sent to the server separately, ahead of the queue,  and a warning will be logged to\n the Developer Console.",
        name:"actionURL",
        seeAlso:[
            {ref:"classAttr:RPCManager.actionURL", description:null}
        ],
        valueType:"URL",
        defaultValue:"RPCManager.actionURL",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:TabSet.paneContainer":{
        ref:"attr:TabSet.paneContainer",
        flags:"R",
        description:"Container where the component specified by ${isc.DocUtils.linkForRef('attr:Tab.pane')} is shown.\n <P>\n Note: paneContainer and showEdges:true for rounded tabsets: you can enable decorative\n image-based edges on the paneContainer by setting ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','showEdges:true')}\n via paneContainerDefaults (to skin all tabsets) or paneContainerProperties (to use\n edges on one instance).  In this structure, the ${isc.DocUtils.linkForRef('group:baseLine')} should use media\n that matches the appearance of the decorative edges and fully overlaps the edge of the\n paneContainer that it is adjacent to.  In the most typical appearance (symmetric edges\n on all 4 sides), both ${isc.DocUtils.linkForRef('attr:TabBar.baseLineCapSize')} and ${isc.DocUtils.linkForRef('attr:TabBar.baseLineThickness')}\n match the ${isc.DocUtils.linkForRef('attr:Canvas.edgeSize','edgeSize')} set on the paneContainer.  See the\n load_skin.js file for the \"SmartClient\" skin for an example of setting all relevant\n properties.\n <P>\n To disable edges for a particular TabSet, which you may want to do for a TabSet that\n is already within a clearly defined container, configure the paneContainer to show only\n it's top edge:\n <pre>\n      paneContainerProperties : { customEdges:[\"T\"] },\n </pre>\n To completely flatten even the top edge of the TabSet:\n <pre>\n      paneContainerProperties : { customEdges:[\"T\"] },\n      tabBarProperties :{ baseLineCapSize:0 },\n </pre>\n This \"flattens\" the baseLine so that only the center image is used.",
        name:"paneContainer",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "classMethod:Log.setDefaultPriority":{
        ref:"classMethod:Log.setDefaultPriority",
        flags:"",
        description:" Set the default priority of messages that will be visible.\n\n",
        name:"setDefaultPriority",
        seeAlso:[],
        params:[
            {description:"priority level to set", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"},
            {description:"Optional ISC class or instance - if passed the default priority will be set for logging \n      occurring on the class or instance only.",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:ColumnTree.firstColumnTitle":{
        ref:"attr:ColumnTree.firstColumnTitle",
        flags:"IR",
        description:"A title for the leftmost column if ${isc.DocUtils.linkForRef('attr:ColumnTree.showHeaders')} is set (the remaining columns\n have their titles derived from the item selected in the column to the left). Ignored\n if ${isc.DocUtils.linkForRef('attr:ColumnTree.showHeaders')} is not set.<br><br>\n Note: if you do not want a heading for the first column leave this attribute at its \n default value of \" \". If you set it to null or the empty string, SmartClient will\n fall back to displaying the field's name in the heading.",
        name:"firstColumnTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\" \"",
        type:"attr",
        definingClass:"class:ColumnTree"
    },
    "method:Canvas.dragStop":{
        ref:"method:Canvas.dragStop",
        flags:"A",
        description:" Executed when the mouse button is released at the end of the drag. Your widget can\n use this opportunity to fire code based on the last location of the drag or reset any\n visual state that was sent.\n\n",
        name:"dragStop",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to cancel drag interaction.", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:TreeGrid.rowClick":{
        ref:"method:TreeGrid.rowClick",
        flags:"",
        description:"\n This override to ${isc.DocUtils.linkForRef('method:ListGrid.rowClick')}.  This implementation calls through to the\n ${isc.DocUtils.linkForRef('method:TreeGrid.nodeClick')}, ${isc.DocUtils.linkForRef('method:TreeGrid.folderClick')}, ${isc.DocUtils.linkForRef('method:TreeGrid.leafClick')} methods, as\n appropriate unless the click was on the expand/collapse control of a folder - in which case\n those callbacks are not fired.\n <p>\n Do not override this method unless you need a rowClick callback that fires even when the\n user clicks on the expand/collapse control.  If you do override this method, be sure to call\n <code>return this.Super(\"rowClick\", arguments);</code> at the end of your override to\n preserver other handler that are called from the superclass (for example,\n ${isc.DocUtils.linkForRef('method:ListGrid.recordClick')}.\n <p>\n\n",
        name:"rowClick",
        seeAlso:[
            {ref:"method:TreeGrid.nodeClick", description:null},
            {ref:"method:TreeGrid.folderClick", description:null},
            {ref:"method:TreeGrid.leafClick", description:null},
            {ref:"method:ListGrid.recordClick", description:null}
        ],
        params:[
            {description:"record that was clicked on", optional:false, name:"record", defaultValue:null,
             type:"TreeNode"},
            {description:"index of the row where the click occurred", optional:false, name:"recordNum",
             defaultValue:null, type:"number"},
            {description:"index of the col where the click occurred", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:FormItem.valueIconLeftPadding":{
        ref:"attr:FormItem.valueIconLeftPadding",
        flags:"IRW",
        description:"If we're showing a value icon, this attribute governs the amount of space between the \n icon and the left edge of the form item cell",
        name:"valueIconLeftPadding",
        seeAlso:[
            {ref:"attr:FormItem.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:ListGrid.fields":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.fields",
        description:"An array of field objects, specifying the order, layout, formatting, and\n sorting behavior of each field in the listGrid object.  In ListGrids, the fields\n array specifies columns.  Each field in the fields array is a ListGridField object. \n Any listGrid that will display data should have at least one visible field.\n <p>\n If ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')} is also set, this value acts as a set of overrides as\n explained in ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}.",
        name:"fields",
        setter:["setFields"],
        valueType:"Array of ListGridField",
        seeAlso:[
            {ref:"object:ListGridField", description:null},
            {ref:"method:ListGrid.setFields", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('listGridFields')}", 
         "${isc.DocUtils.linkForExampleId('mergedFields')}"],
        defaultValue:"null",
        groups:["databinding"],
        definingClass:"class:ListGrid"
    },
    "attr:Button.align":{
        ref:"attr:Button.align",
        flags:"IRW",
        description:"Horizontal alignment of this component's title.",
        name:"align",
        setter:["setAlign"],
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:Button",
        groups:["appearance"]
    },
    "method:DynamicForm.validateData":{
        ref:"method:DynamicForm.validateData",
        flags:"",
        description:"\n Perform validation on the client and the server.\n\n",
        name:"validateData",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:ViewLoader.httpMethod":{
        ref:"attr:ViewLoader.httpMethod",
        flags:"IRW",
        description:"Selects the HTTP method that will be used when fetching content.  Valid values are \"POST\"\n and \"GET\".",
        name:"httpMethod",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"GET\"",
        type:"attr",
        definingClass:"class:ViewLoader",
        groups:["contentLoading"]
    },
    "attr:SelectItem.hiliteColor":{
        ref:"attr:SelectItem.hiliteColor",
        deprecated:"As of SmartClient version 5.5, if ${isc.DocUtils.linkForRef('attr:SelectItem.showFocused')} is true,  styling will be updated for this form item on focus. The hiliting effect can therefore\n  be achieved via ${isc.DocUtils.linkForRef('attr:SelectItem.textBoxStyle')} and ${isc.DocUtils.linkForRef('attr:SelectItem.controlStyle')} \n  instead.",
        flags:"IRWA",
        description:"Background color to apply to the select item's selected value when the SelectItem \n receives focus, if <code>hiliteOnFocus</code> is true.",
        name:"hiliteColor",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"#316AC5\"",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:Facet.summaryTitle":{
        ref:"attr:Facet.summaryTitle",
        flags:"IR",
        description:"title for facet summary.",
        name:"summaryTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"cubeGrid.summaryTitle",
        type:"attr",
        definingClass:"class:Facet"
    },
    "classMethod:isc.getIconLeft":{
        ref:"classMethod:isc.getIconLeft",
        flags:"A",
        description:"  Returns the (offset) left-coordinate of an icon within its containing widget.\n",
        name:"getIconLeft",
        seeAlso:[],
        params:[
            {description:"icon definition", optional:false, name:"icon", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"icon left position in px", type:"number"},
        type:"method",
        definingClass:"class:isc"
    },
    "method:ListGrid.cancelEditing":{
        ref:"method:ListGrid.cancelEditing",
        flags:"",
        description:"\n  Cancel the current edit without saving.\n\n",
        name:"cancelEditing",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:gridHeader":{
        ref:"group:gridHeader",
        refs:["method:ListGrid.setHeaderHeight", "method:ListGrid.setShowHeader", 
         "method:ListGrid.headerClick","method:ListGrid.displayHeaderContextMenu", "method:ListGrid.getHeaderContextMenuItems", 
         "attr:ListGrid.header","attr:ListGrid.headerHeight", "attr:ListGrid.showHeader", "attr:ListGrid.headerBarStyle", 
         "attr:ListGrid.headerBackgroundColor","attr:ListGrid.headerButtonConstructor", "attr:ListGrid.headerBaseStyle", 
         "attr:ListGrid.headerTitleStyle","attr:ListGrid.frozenHeaderBaseStyle", "attr:ListGrid.frozenHeaderTitleStyle", 
         "attr:ListGrid.headerButtonDefaults","attr:ListGrid.headerButtonProperties", "attr:ListGrid.sorterConstructor", 
         "attr:ListGrid.sorterDefaults","attr:ListGrid.sorterProperties", "attr:ListGrid.showHeaderContextMenu"],
        description:"Properties and methods related to the ListGrid header. ListGrid headers are implemented\n as a ${isc.DocUtils.linkForRef('class:Toolbar')} of buttons shown at the top of the ListGrid \n (one button per column).<br>\n The toolbar header provides UI for interacting with the ListGrid fields directly (sorting,\n resizing, reordering columns, etc).",
        name:"gridHeader",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.getDrawnRows":{
        ref:"method:ListGrid.getDrawnRows",
        flags:"",
        description:" Get the rows that are currently drawn (exist in the DOM), as an array of [firstRowNum,\n lastRowNum].  \n <P>\n The drawn rows differ from the ${isc.DocUtils.linkForRef('method:ListGrid.getVisibleRows','visibleRows')} because of\n ${isc.DocUtils.linkForRef('attr:ListGrid.drawAheadRatio','drawAhead')}.  The drawn rows are the apppropriate range to consider if\n you need to, eg, using ${isc.DocUtils.linkForRef('method:ListGrid.refreshCell')} to update all the cells in a column.\n <P>\n If the grid is undrawn or the ${isc.DocUtils.linkForRef('attr:ListGrid.emptyMessage')} is currently shown, returns\n [null,null];\n\n",
        name:"getDrawnRows",
        seeAlso:[],
        returns:{description:null, type:"Array"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:SelectItem.getValueFieldName":{
        ref:"method:SelectItem.getValueFieldName",
        flags:"A",
        description:" Getter method to retrieve the ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} for this item.\n If unset, default behavior will return the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this field.\n",
        name:"getValueFieldName",
        seeAlso:[],
        returns:{description:"fieldName to use a \"value field\" in records from this items               ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}",
         type:"string"},
        type:"method",
        definingClass:"class:SelectItem",
        groups:["display_values"]
    },
    "method:Tree.findById":{
        ref:"method:Tree.findById",
        flags:"A",
        description:"\n Find the node with the specified ID.  Specifically, it returns the node whose idField\n matches the id passed to this method. If the tree is using the \"parent\" modelType, this\n lookup will be constant-time.  For all other modelTypes, the tree will be searched\n recursively.\n\n",
        name:"findById",
        seeAlso:[
            {ref:"attr:Tree.idField", description:null},
            {ref:"method:Tree.find", description:null}
        ],
        params:[
            {description:"ID of the node to return.", optional:false, name:"id", defaultValue:null,
             type:"string"}
        ],
        returns:{description:"node with appropriate ID, or null if not found.", type:"object"},
        type:"method",
        definingClass:"class:Tree",
        groups:["location"]
    },
    "classMethod:isc.confirm":{
        ref:"classMethod:isc.confirm",
        flags:"",
        description:" Show a modal dialog with a message, icon, and \"OK\" and \"Cancel\" buttons.\n <P>\n The callback will receive boolean true for an OK button click, or null for a Cancel click or\n if the Dialog is dismissed via the close button.\n <P>\n Note: this does not override the native window.confirm() method.\n\n",
        name:"confirm",
        seeAlso:[
            {ref:"classAttr:Dialog.Warn", description:null},
            {ref:"classMethod:isc.warn", description:null},
            {ref:"method:Dialog.okClick", description:null},
            {ref:"method:Dialog.cancelClick", description:null},
            {ref:"classAttr:Dialog.CONFIRM_TITLE", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"Callback to fire when the                                       user clicks a button to dismiss the dialog.\n                                      This has the single parameter 'value', indicating the \n                                      value returned by the Warn dialog from 'okClick()' etc.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog.                                      To set ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons','custom buttons')} for\n                                      the Dialog, set properties.toolbarButtons to an array of\n                                      buttons\n										eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('dialogs')}"],
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "method:Canvas.setOverflow":{
        ref:"method:Canvas.setOverflow",
        flags:"A",
        description:" Update the ${isc.DocUtils.linkForRef('attr:Canvas.overflow','overflow')} of a Canvas after it has been created.\n",
        name:"setOverflow",
        seeAlso:[],
        params:[
            {description:"New overflow value.", optional:false, name:"newOverflow", defaultValue:null,
             type:"Overflow"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "sizing", "sizing"]
    },
    "method:Button.addToRadioGroup":{
        ref:"method:Button.addToRadioGroup",
        flags:"",
        description:" Add this widget to the specified mutually exclusive selection group with the ID\n passed in.\n Selecting this widget will then deselect any other StatefulCanvases with the same\n radioGroup ID.\n StatefulCanvases can belong to only one radioGroup, so this method will remove from \n any other radiogroup of which this button is already a member.\n",
        name:"addToRadioGroup",
        seeAlso:[],
        params:[
            {description:"- ID of the radiogroup to which this widget should be added",
             optional:false, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["state", "event handling"]
    },
    "attr:ListGrid.valueIconRightPadding":{
        ref:"attr:ListGrid.valueIconRightPadding",
        flags:"IRW",
        description:"How much padding should there be on the right of valueIcons by default",
        name:"valueIconRightPadding",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:DateItem.pickerIconPrompt":{
        ref:"attr:DateItem.pickerIconPrompt",
        flags:"IR",
        description:"Prompt to show when the user hovers the mouse over the picker icon for this DateItem. May\n be overridden for localization of your application.",
        name:"pickerIconPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Show Date Chooser\"",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["i18nMessages"]
    },
    "attr:Canvas.autoShowParent":{
        ref:"attr:Canvas.autoShowParent",
        flags:"IRWA",
        description:"If set to true, the widget's parent (if any) will automatically be shown whenever the\n      widget is shown.",
        name:"autoShowParent",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "class:ResultSet":{
        methods:["method:ResultSet.allMatchingRowsCached", "method:ResultSet.allRowsCached", 
         "method:ResultSet.getValueMap","method:ResultSet.getLength", "method:ResultSet.indexOf", "method:ResultSet.get", 
         "method:ResultSet.getRange","method:ResultSet.lengthIsKnown", "method:ResultSet.rowIsLoaded", 
         "method:ResultSet.rangeIsLoaded","method:ResultSet.findByKey", "method:ResultSet.setCriteria", 
         "method:ResultSet.getCriteria","method:ResultSet.compareCriteria", "method:ResultSet.willFetchData", 
         "method:ResultSet.sortByProperty","method:ResultSet.getSort", "method:ResultSet.setSort", "method:ResultSet.filterLocalData", 
         "method:ResultSet.applyFilter","method:ResultSet.invalidateCache", "method:ResultSet.transformData", 
         "method:ResultSet.dataArrived","method:ResultSet.findAll", "method:ResultSet.find", "method:ResultSet.findIndex", 
         "method:ResultSet.findNextIndex","method:ResultSet.getProperty"],
        ref:"class:ResultSet",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:ResultSet.fetchMode", "attr:ResultSet.initialData", "attr:ResultSet.initialLength", 
         "attr:ResultSet.allRows","attr:ResultSet.resultSize", "attr:ResultSet.fetchDelay", "attr:ResultSet.dataSource", 
         "attr:ResultSet.fetchOperation","attr:ResultSet.criteria", "attr:ResultSet.criteriaPolicy", 
         "attr:ResultSet.useClientSorting","attr:ResultSet.useClientFiltering", "attr:ResultSet.updateCacheFromRequest", 
         "attr:ResultSet.dropCacheOnUpdate","attr:ResultSet.disableCacheSync", "attr:ResultSet.neverDropUpdatedRows", 
         "attr:ResultSet.updatePartialCache"],
        description:"ResultSets are an implementation of the ${isc.DocUtils.linkForRef('interface:List')} interface that automatically fetches \n DataSource records when items are requested from the List.  ResultSets provide robust,\n customizable, high-performance cache management for ListGrids and other built-in SmartClient\n components, and can be used as cache managers by custom components.\n <P>\n ResultSets manage data paging, that is, loading records in batches as the user navigates\n the data set.  A ResultSet will switch to using client-side sorting and filtering when\n possible to improve responsiveness and reduce server load.  ResultSets also participate in\n automatic cache synchronization, observing operations on DataSources and automatically\n updating their caches.\n <P>\n <b>Creation</b>\n <P>\n A ResultSet can be passed to any component that expects a List, and the List APIs can be\n called directly on the ResultSet as long as the caller is able to deal with asynchronous\n loading; see ${isc.DocUtils.linkForRef('method:ResultSet.getRange')}.\n <P>\n Generally ResultSets do not need to be created directly, but are created by DataBound\n components as an automatic consequence of calling \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')}.  \n For example, the ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} causes ${isc.DocUtils.linkForRef('attr:ListGrid.data')} to become an\n automatically created <code>ResultSet</code> object.  Automatically created ResultSets\n can be customized via properties on ListGrids such as ${isc.DocUtils.linkForRef('attr:ListGrid.dataPageSize')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')}.  All ResultSets for a given DataSource may also be \n customized via setting ${isc.DocUtils.linkForRef('attr:DataSource.resultSetClass')} to the name of a ResultSet \n ${isc.DocUtils.linkForRef('classMethod:isc.defineClass','subclass')} in which \n ${isc.DocUtils.linkForRef('classMethod:Class.addProperties','defaults have been changed')}.\n <P>\n A ResultSet defaults to using data paging, setting ${isc.DocUtils.linkForRef('attr:DSRequest.startRow')} and\n ${isc.DocUtils.linkForRef('attr:DSRequest.endRow')} in issued dsRequests.  Server code may always return more rows than\n the ResultSet requests and the ResultSet will correctly integrate those rows based on\n ${isc.DocUtils.linkForRef('attr:DSResponse.startRow')}/${isc.DocUtils.linkForRef('attr:DSResponse.endRow','endRow')}.  \n Hence the server can always avoid paging mode by simply returning all matching rows.\n <P>\n A ResultSet can be created directly with just the ID of a ${isc.DocUtils.linkForRef('class:DataSource')}:\n <pre>\n     isc.ResultSet.create({\n         dataSource : \"<i>dataSourceID</i>\"\n     })\n </pre>\n <P>\n Directly created ResultSets are typically used by custom components, or as a means of\n managing datasets that will be used by several components.\n <P>\n When created directly rather than via a dataBoundComponent, a newly created ResultSet will\n not issue it's first \"fetch\" ${isc.DocUtils.linkForRef('class:DSRequest')} until data is accessed (for example, via\n ${isc.DocUtils.linkForRef('method:ResultSet.get','get()')}).  \n <P>\n <b>Paging and total dataset length</b>\n <P>\n When using data paging, the server communicates the total number of records that match the\n current search criteria by setting ${isc.DocUtils.linkForRef('attr:DSResponse.totalRows')}.  The ResultSet will then\n return this number from ${isc.DocUtils.linkForRef('method:ResultSet.getLength','getLength()')}, and ListGrids and other\n components will show a scrollbar that allows the user to jump to the end of the dataset\n directly.\n <P>\n However, the ResultSet does not require that the server calculate the true length of the\n dataset, which can be costly for an extremely large, searchable dataset.  Instead, the\n server <i>may</i> simply advertise a <code>totalRows</code> value that is one page larger\n than the last row loaded.  This results in a UI sometimes called \"progressive loading\",\n where the user may load more rows by scrolling past the end of the currently loaded rows,\n but is not allowed to skip to the end of the dataset.\n <P>\n No client-side settings are required to enable this mode - it is entirely server-driven.\n However, it is usually coupled with ${isc.DocUtils.linkForRef('attr:ListGrid.canSort','disabling sorting')}, since\n server-side sorting would also force the server to traverse the entire dataset.\n <P>\n <b>Client-side Sorting and Filtering</b>\n <P>\n If a ResultSet obtains a full cache for the current set of filter criteria, it will \n automatically switch to client-side sorting, and will also use client-side filtering \n if the filter criteria are later changed but appear to be <i>more restrictive</i> than the\n criteria in use when the ResultSet obtained a full cache.\n <P>\n The ${isc.DocUtils.linkForRef('attr:ResultSet.useClientSorting','useClientSorting')} and \n ${isc.DocUtils.linkForRef('attr:ResultSet.useClientFiltering','useClientFiltering')} flags can be used to disable\n client-side sorting and filtering respectively if these behaviors don't match server-based\n sorting and filtering.  However, because client-side sorting and filtering radically improve\n responsiveness and reduce server load, it is better to customize the ResultSet so that it\n can match server-side sorting and filtering behaviors.\n <P>\n Sorting behavior is primarily customized via the \"sort normalizer\" passed to\n ${isc.DocUtils.linkForRef('method:ResultSet.sortByProperty')}, either via direct calls on a standalone ResultSet, or via\n ${isc.DocUtils.linkForRef('method:ListGridField.sortNormalizer')} for a ListGrid-managed ResultSet.\n <P>\n By default, client-side filtering interprets the ${isc.DocUtils.linkForRef('type:Criteria','criteria')} passed to\n ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')} as a set of field values that records must match\n (similarly to the built-in SQL/Hibernate connectors built into the SmartClient Server).\n Custom client-side filtering logic can be implemented by overriding\n ${isc.DocUtils.linkForRef('method:ResultSet.applyFilter','applyFilter()')}.  Overriding\n ${isc.DocUtils.linkForRef('method:ResultSet.compareCriteria','compareCriteria()')} allows you to control when the ResultSet\n uses client-side vs server-side filtering, and the ResultSet has two default \n ${isc.DocUtils.linkForRef('attr:ResultSet.criteriaPolicy','criteria policies')} built-in.\n <P>\n <b>Updates and Automatic Cache Synchronization</b>\n <P>\n ResultSets observe any successful \"update\", \"add\" or \"remove\" dsRequests against their\n DataSource, regardless of the component that initiated them.  A ResultSet with a full cache\n for the current filter criteria will integrate updates into the cache automatically.\n <P>\n Updated rows that no longer match the current filter criteria will be removed\n automatically.  To prevent this, you can set ${isc.DocUtils.linkForRef('attr:ResultSet.neverDropUpdatedRows')}.\n Added rows will similarly be added to the cache only if they match current filter criteria.\n <P>\n Note that the client-side filtering described above is also used to determine whether \n updated or added rows should be in the cache.  If any aspect of automated cache update is\n ever incorrect, ${isc.DocUtils.linkForRef('attr:ResultSet.dropCacheOnUpdate','dropCacheOnUpdate')} can be set for the\n ResultSet or ${isc.DocUtils.linkForRef('attr:DSResponse.invalidateCache')} can be set for an individual dsResponse.\n <P>\n <b>Data Paging with partial cache</b>\n <P>\n When in paging mode with a partial cache, a ResultSet relies on server side sorting, setting \n ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy')} to the current sort field and direction.  In order for the cache to \n remain coherant, row numbering must continue to agree between server and client as new\n fetches are issued, otherwise, duplicate rows or missing rows may occur.  \n <P>\n If concurrent modifications by other users are allowed, generally the server should set\n ${isc.DocUtils.linkForRef('attr:DSResponse.invalidateCache')} to clear the cache when concurrent modification is\n detected.\n <P>\n In paging mode with a partial cache, any successful \"update\" or \"add\" operation may cause\n client and server row numbering to become out of sync.  This happens because the update\n may affect the sort order, and client and server cannot be guaranteed to match for sets of\n records that have equivalent values for the sort field.\n <P>\n For this reason, after an \"add\" or \"update\" operation with a partial cache, the ResultSet\n will automatically mark cache for invalidation the next time a fetch operation is performed.\n Alternatively, if ${isc.DocUtils.linkForRef('attr:ResultSet.updatePartialCache')} is set to false, the ResultSet will\n simply invalidate cache immediately in this circumstance.",
        name:"ResultSet",
        implementsInterfaces:["List"],
        seeAlso:[
            {ref:"interface:DataBoundComponent", description:null},
            {ref:"group:dataBoundComponentMethods", description:null},
            {ref:"attr:DataSource.resultSetClass",
             description:"to customize all ResultSets for a given DataSource"},
            {ref:"method:ResultSet.getRange",
             description:"for information on handling asynchronous loading"}
        ],
        type:"class"
    },
    "attr:DataSource.serverType":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:DataSource.serverType",
        description:"For a DataSource stored in .xml format on the ISC server, indicates what server-side\n connector to use to execute requests, that is, what happens if you call\n dsRequest.execute() in server code.",
        name:"serverType",
        valueType:"DSServerType",
        seeAlso:[],
        serverDS:"allowed",
        defaultValue:"\"generic\"",
        groups:["serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "attr:ListGrid.detailField":{
        ref:"attr:ListGrid.detailField",
        flags:"IRWA",
        description:"The field whose contents to show in the expanded portion of a record when \n ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords','canExpandRecords')} is <code>true</code> and \n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>detailField</code>.",
        name:"detailField",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:Menu.target":{
        ref:"attr:Menu.target",
        flags:"IRW",
        description:"Optional target canvas for this menu. Available as a parameter to dynamic menuItem configuration\n methods such as ${isc.DocUtils.linkForRef('method:MenuItem.checkIf')}.\n <P>\n If this item has any ${isc.DocUtils.linkForRef('attr:MenuItem.submenu','submenus')} the <code>target</code> will be propogated down\n to these child menus.",
        name:"target",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:SelectOtherItem.otherValue":{
        ref:"attr:SelectOtherItem.otherValue",
        flags:"IRWA",
        description:"Data value for the <code>Other...</code> item. If necessary this value may be changed to\n ensure it doesn't collide with any data values in this item's\n ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')}.",
        name:"otherValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"***other***\"",
        type:"attr",
        definingClass:"class:SelectOtherItem",
        groups:["appearance"]
    },
    "group:drawing":{
        ref:"group:drawing",
        refs:["method:FormItem.getFieldName", "method:FormItem.getTitle", "method:FormItem.isDrawn", 
         "method:ListGrid.markForRedraw","method:Canvas.isDrawn", "method:Canvas.getInnerHTML", "method:Canvas.draw", 
         "method:Canvas.isDirty","method:Canvas.markForRedraw", "method:Canvas.redraw", "attr:Canvas.autoDraw", 
         "attr:Canvas.redrawOnResize"],
        description:"Rendering an object on the page",
        name:"drawing",
        seeAlso:[],
        type:"group"
    },
    "group:apperance":{
        ref:"group:apperance",
        refs:["method:DetailViewer.getCellCSSText", "method:DetailViewer.getCellStyle"],
        name:"apperance",
        seeAlso:[],
        type:"group"
    },
    "attr:MenuItem.iconHeight":{
        ref:"attr:MenuItem.iconHeight",
        flags:"IR",
        description:"The height applied to this item's icon.  The default of <code>16</code> can be changed\n for all MenuItems by overriding ${isc.DocUtils.linkForRef('attr:Menu.iconHeight')}.",
        name:"iconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"object:MenuItem",
        groups:["menuIcons"]
    },
    "attr:Criterion.criteria":{
        ref:"attr:Criterion.criteria",
        flags:"IR",
        description:"For a criterion with an operator that acts on other criteria (eg \"and\", \"or\"), a list of\n sub-criteria that are grouped together by the operator.",
        name:"criteria",
        seeAlso:[],
        valueType:"Array of Criterion",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "attr:FormItem.hoverDelay":{
        ref:"attr:FormItem.hoverDelay",
        flags:"IRWA",
        description:"If specified, this is the number of miliseconds to wait between the user rolling over \n this form item, and triggering any hover action for it.<br>\n If not specified <code>this.form.itemHoverDelay</code> will be used instead.",
        name:"hoverDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:BatchUploader.cancelConfirmMessage":{
        ref:"attr:BatchUploader.cancelConfirmMessage",
        flags:"IRW",
        description:"Confirmation message to show if the user clicks the \"Cancel\" button and ${isc.DocUtils.linkForRef('attr:BatchUploader.warnOnCancel')}\n is true.  Defaults to \"You will lose any work you have done on this data. Proceed anyway?\"",
        name:"cancelConfirmMessage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"See below",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:DSRequest.headerData":{
        ref:"attr:DSRequest.headerData",
        flags:"IRW",
        description:"For DataSources using SOAP messaging with a WSDL web service, data to be serialized to form\n SOAP headers, as a map from the header part name to the data.  See\n ${isc.DocUtils.linkForRef('attr:WSRequest.headerData')} for more information.\n <P>\n SOAP headers typically contain request metadata such as a session id for authentication, and\n so <code>dsRequest.headerData</code> is typically populated by\n ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}, or, for data that applies to every request sent to the\n server, by ${isc.DocUtils.linkForRef('method:WebService.getHeaderData')}.",
        name:"headerData",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:List.add":{
        ref:"method:List.add",
        flags:"",
        description:" Add an object to this list, at the end\n\n",
        name:"add",
        seeAlso:[],
        params:[
            {description:"object to add", optional:false, name:"object", defaultValue:null, type:"any"}
        ],
        returns:{description:"pointer to the object passed in", type:"any"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "class:ViewFileItem":{
        methods:[],
        ref:"class:ViewFileItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:ViewFileItem.showFileInline"],
        description:"Item for displaying the contents of \"imageFile\" fields in DynamicForms. \n <P>\n Displays one of two UIs, according to the value of \n ${isc.DocUtils.linkForRef('attr:ViewFileItem.showFileInline','showFileInline')}.  If showFileInline is false, this Item\n displays the View and Download icons and the filename.  Otherwise, it streams the image-file \n and displays it inline.",
        name:"ViewFileItem",
        seeAlso:[],
        type:"class",
        groups:["upload"]
    },
    "method:Canvas.setPageLeft":{
        ref:"method:Canvas.setPageLeft",
        flags:"",
        description:" Set the page-relative left coordinate of this widget.\n \n",
        name:"setPageLeft",
        seeAlso:[],
        params:[
            {description:"new left coordinate in pixels", optional:false, name:"left",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:Scrollbar.cornerSrc":{
        ref:"attr:Scrollbar.cornerSrc",
        flags:"IR",
        description:"URL for the corner image, a singular image that appears in the corner when both h and v\n scrollbars are showing.",
        name:"cornerSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]corner.gif\"",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["images"]
    },
    "method:Calendar.dayHeaderClick":{
        ref:"method:Calendar.dayHeaderClick",
        flags:"",
        description:" Called when the header area of a day in the month view is clicked on.\n <P>\n By default, moves to the day tab and shows the clicked days events.\n Return false to cancel this action.\n <P>\n Not called if the day falls outside the current month and ${isc.DocUtils.linkForRef('attr:Calendar.showOtherDays')} is false.\n\n",
        name:"dayHeaderClick",
        seeAlso:[],
        params:[
            {description:"JavaScript Date object representing this day", optional:false, name:"date",
             defaultValue:null, type:"Date"},
            {description:"events that fall on this day", optional:false, name:"events",
             defaultValue:null, type:"Array of CalendarEvent"},
            {description:"the calendar itself", optional:false, name:"calendar", defaultValue:null,
             type:"Calendar"},
            {description:"the row number to which the parameter date belongs", optional:false,
             name:"rowNum", defaultValue:null, type:"Integer"},
            {description:"the column number to which the parameter date belongs", optional:false,
             name:"colNum", defaultValue:null, type:"Integer"}
        ],
        returns:{description:"return false to cancel the action", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "method:ListGrid.setFields":{
        ref:"method:ListGrid.setFields",
        flags:"A",
        description:" Sets the fields array and/or field widths to newFields and sizes, respectively.<br><br>\n\n If newFields is specified, it is assumed that the new fields may have nothing in common with\n the old fields, and the component is substantially rebuilt.  Consider the following methods\n for more efficient, more incremental changes: resizeField, reorderField, showField,\n hideField, setFieldProperty.\n\n",
        name:"setFields",
        seeAlso:[],
        params:[
            {description:"array of fields to draw", optional:true, name:"newFields", defaultValue:null,
             type:"List of ListGridField"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:PickList.getOptionDataSource":{
        ref:"method:PickList.getOptionDataSource",
        flags:"",
        description:" Pick-lists can derive their data directly from a valueMap, or retrieve data from a \n dataSource to display as options.\n \n This method will return the dataSource used to populate the pickList, or null if \n none is specified (implies this list will derive its data from the valueMap for the item).\n Default implementation  will return ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} if specified,\n otherwise if this is a field with a specified <code>foreignKey</code> in a databound\n form, returns the dataSource for the <code>foreignKey</code>.\n Otherwise picks up <code>this.form.dataSource</code> if set.\n\n",
        name:"getOptionDataSource",
        seeAlso:[],
        returns:{description:"DataSource to use for fetching options", type:"DataSource"},
        type:"method",
        definingClass:"interface:PickList"
    },
    "type:Callback":{
        ref:"type:Callback",
        description:"A <code>Callback</code> is an arbitrary action to be fired - usually passed into a \n method to be fired asynchronously as a notificaction of some event.<br>\n The <code>callback</code> can be defined in the following formats:<ul>\n <li>a function</li>\n <li>A string containing an expression to evaluate</li>\n <li>An object with the following properties:<br>\n     - target: fire in the scope of this target - when the action fires,\n       the target will be available as <code>this</code>.<br>\n     - methodName: if specified we'll check for a method on the target object with this \n       name.<br>\n  </li></ul>\n <code>Callbacks</code> are fired via the ${isc.DocUtils.linkForRef('classMethod:Class.fireCallback')} method, which allows\n named parameters to be passed into the callback at runtime. If the Callback was specified\n as a string of script, these parameters are available as local variables at eval time.<br>\n For specific SmartClient methods that make use of <code>Callback</code> objects, see\n local documentation for information on parameters and scope.",
        name:"Callback",
        seeAlso:[],
        type:"type"
    },
    "attr:ServerObject.targetXPath":{
        ref:"attr:ServerObject.targetXPath",
        flags:"IR",
        description:"If set, the SmartClient server will use JXPath to call your server code.  The\n <code>JXPathContext</code> (start point) will be the object arrived at by applying the \n ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle','lookupStyle')} and related ServerObject properties. The \n intention of this property is to allow easier access to your existing Java objects \n and reduce the need to write SmartClient-specific server code.",
        name:"targetXPath",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "method:ListGridField.formatCellValue":{
        ref:"method:ListGridField.formatCellValue",
        flags:"",
        description:" Return the HTML to display in cells of this field.\n <P>\n Given the raw value for this field as taken from the record Formatter to apply to the\n static values displayed in cells for this field.\n <P>\n <i>Example usage</i>: formatting a currency value stored in cents (so \"100\" to \"$1.00\")<br>\n The value passed to this method is the raw value for the cell.<br>\n Takes precedence over <code>formatCellValue</code> defined at the grid level for cells\n in this field.\n <P>\n Note: this formatter will not be applied to the values displayed in cells being edited.\n The ${isc.DocUtils.linkForRef('method:ListGridField.formatEditorValue','formatEditorValue()')} is provided for that purpose.\n\n",
        name:"formatCellValue",
        seeAlso:[
            {ref:"method:ListGrid.formatCellValue", description:null},
            {ref:"method:ListGridField.formatEditorValue", description:null}
        ],
        params:[
            {description:"raw value for the cell, from the record for the row", optional:false,
             name:"value", defaultValue:null, type:"any"},
            {description:"Record object for the cell. Note: If this is a new row that has not been saved, in an \n   editable grid, it has no associated record object. In this case the edit values will\n   be passed in as this parameter (see ${isc.DocUtils.linkForRef('method:ListGrid.getEditValues')})",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"},
            {description:"the ListGrid displaying the cell", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"HTML to display in the cell", type:"String"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('formatValues')}"],
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "attr:ListGrid.dataSource":{
        ref:"attr:ListGrid.dataSource",
        flags:"IRW",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "method:PickList.getClientPickListData":{
        ref:"method:PickList.getClientPickListData",
        flags:"A",
        description:" Returns the set of data to be displayed in this item's PickList.\n <P>\n This method will be called for non-databound form items implementing the PickList\n interface.  The default implementation will derive data from the item's valueMap - \n can be overridden to allow a custom set of options to be displayed.\n <P>\n Note that for PickLists that filter data based on user input\n (${isc.DocUtils.linkForRef('class:ComboBoxItem','ComboBox')}), this method should return the data <b>before\n filtering</b>.  To customize the data returned after filtering, override\n ${isc.DocUtils.linkForRef('method:PickList.filterClientPickListData')} instead.\n <P>\n As an example, for a formItem with ${isc.DocUtils.linkForRef('attr:PickList.valueField')} set to \"valueFieldName\", the\n default implementation would take a valueMap like the following:\n <pre>\n     valueMap: { value1: \"display 1\", value2: \"display 2\" }\n </pre>\n .. and returning the following set of records: \n <pre>\n     [\n          { valueFieldName : \"value1\" },\n          { valueFieldName : \"value2\" }\n     ]\n </pre>\n Due to the valueMap, these records will appear as a two row pickList displayed as\n \"display 1\" and \"display 2\".\n\n",
        name:"getClientPickListData",
        seeAlso:[],
        returns:{description:"Array of record objects to be displayed in the           pickList. Note that when a user picks a record from the list, the value of the\n           field matching <code>item.valueField</code> will be picked. Also note that the\n           fields to be displayed can be customized via <code>item.pickListFields</code>",
         type:"Array of ListGridRecord"},
        type:"method",
        definingClass:"interface:PickList"
    },
    "type:listGridFieldState":{
        ref:"type:listGridFieldState",
        description:"An object containing the stored presentation information for the fields of a listGrid.\n Information contained in a <code>listGridFieldState</code> object includes the \n visibility and widths of the listGrid's fields.<br>\n Note that this object is a JavaScript string, and may be stored (for example) as a blob \n on the server for state persistence across sessions.",
        name:"listGridFieldState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "method:ValuesManager.hasFieldErrors":{
        ref:"method:ValuesManager.hasFieldErrors",
        flags:"",
        description:" Are there any errors associated with a field in this valuesManager?\n",
        name:"hasFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to check for errors", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"returns true if there are any oustanding validation errors, false                   otherwise.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:CubeGrid.autoSizeHeaders":{
        ref:"attr:CubeGrid.autoSizeHeaders",
        flags:"IRW",
        description:"Automatically size row headers to fit wrapped text",
        name:"autoSizeHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "group:loadDMIStubsTag":{
        ref:"group:loadDMIStubsTag",
        title:"&lt;isomorphic:loadDMIStubs&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n Creates global bindings for all serverObjects defined in the <code>rpcBindings</code>\n section of the .app.xml file specified by the <code>ID</code> or <code>name</code> attribute of\n this tag.  Once you've loaded your <code>rpcBindings</code> using this tag, you can call\n methods on the <code>ServerObjects</code> defined there directly.  For example, you can load\n the example.app.xml (located in /shared/app directory of the webRoot of the SDK) like this:\n <pre>\n &lt;isomorphic:loadDMIStubs ID=\"example\"/&gt;\n </pre>\n Whereas using ${isc.DocUtils.linkForRef('classMethod:DMI.call')} you would have had to invoke the <code>getTimeStamp</code>\n method like this:\n <pre>\n DMI.call(\"example\", \"GetTimeStampDMI\", \"getTimeStamp\", new Date(), \"alert(data)\";\n </pre>\n Having loaded the stubs of the <code>example</code> .app.xml, you can then call\n <code>getTimeStamp</code> like this:\n <pre>\n GetTimeStampDMI.getTimeStamp(new Date(), \"alert(data)\");\n </pre>\n or this:\n <pre>\n GetTimeStamp.getTimeStamp({\n     arguments: [new Date()],\n     callback: \"alert(data)\"\n });\n </pre>\n or this:\n <pre>\n GetTimeStamp.call({\n     methodName: \"getTimeStamp\",\n     arguments: [new Date()],\n     callback: \"alert(data)\"\n });\n </pre>\n As with ${isc.DocUtils.linkForRef('classMethod:DMI.call')}, the last argument must be the callback - if you don't want a\n callback, simply specify <code>null</code> as the callback.  The name of the global binding\n created will be the same as the ${isc.DocUtils.linkForRef('attr:ServerObject.ID')} or the non-qualified name of the\n ${isc.DocUtils.linkForRef('attr:ServerObject.className')} (java namespace, if any, will be stripped).\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>ID or name</b><br>\n <i>value format</i>: String - name of .app.xml file to load (minus the .app.xml extension)<br>\n <i>default value</i>: NONE\n <p>\n This attribute specifies the name of the file that contains the rpcBindings to load.\n UI files are located in <code>[webroot]/shared/app</code> by default.  This location is\n changeable in <code>[webroot]/WEB-INF/classes/server.properties</code> by setting the config\n parameter <code>project.apps</code> to the directory where your .app.xml files are located.   \n We recommend that for prototying, at least, you use the default directory.",
        name:"loadDMIStubsTag",
        seeAlso:[
            {ref:"class:DMI", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "method:ColumnTree.selectRecord":{
        ref:"method:ColumnTree.selectRecord",
        flags:"",
        description:"\n Select/deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n\n",
        name:"selectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"},
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "attr:ListGridField.valueIconWidth":{
        ref:"attr:ListGridField.valueIconWidth",
        flags:"IRW",
        description:"Width for value icons for this listGrid field.\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconSize')}, ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconWidth')}, and\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconSize')}.",
        name:"valueIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "attr:BatchUploader.dataURL":{
        ref:"attr:BatchUploader.dataURL",
        flags:"IRW",
        description:"If set, the batchUploader will copy this value to the queue of \"add\" requests it sends \n to the server to actually populate the data.  You can use this facility to route the \n queue to your own server-side logic, for example to add pre- or post-processing.",
        name:"dataURL",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:FilterBuilder.dataSource":{
        ref:"attr:FilterBuilder.dataSource",
        flags:"IRW",
        description:"DataSource this filter should use for field definitions and available ${isc.DocUtils.linkForRef('object:Operator')}s.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:FormItem.left":{
        ref:"attr:FormItem.left",
        flags:"IRWA",
        description:"Left coordinate of this item in pixels.  Applies only when the containing DynamicForm\n sets <code>itemLayout:\"absolute\"</code>.",
        name:"left",
        setter:["setLeft"],
        seeAlso:[],
        valueType:"integer",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:FilterBuilder.defaultSubClauseOperator":{
        ref:"attr:FilterBuilder.defaultSubClauseOperator",
        flags:"IR",
        description:"Default operator for subclauses added via the ${isc.DocUtils.linkForRef('attr:FilterBuilder.subClauseButton')}.",
        name:"defaultSubClauseOperator",
        seeAlso:[],
        valueType:"LogicalOperator",
        defaultValue:"\"or\"",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "group:wsdlBinding":{
        ref:"group:wsdlBinding",
        title:"WSDL Binding",
        treeLocation:["Client Reference/Data Binding"],
        refs:["attr:DataSource.serviceNamespace", "attr:DataSource.schemaNamespace"],
        description:"SmartClient supports automated integration with WSDL-described web services.  This support\n consists of:\n <ul>\n <li> creation of SOAP XML messages from JavaScript application data, with automatic\n namespacing, and support for both \"literal\" and \"encoded\" SOAP messaging, and \"document\" and\n \"rpc\" WSDL-SOAP bindings\n <li> automatic decode of SOAP XML messages to JavaScript objects, with strong typing (eg an\n XML schema \"date\" type becomes a JavaScript Date object)\n <li> ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema','import of XML Schema')} (contained in WSDL, or external),\n including translating XML Schema \"restrictions\" to ISC ${isc.DocUtils.linkForRef('class:Validator','Validators')}\n </ul>\n <P>\n WSDL services can be contacted by using ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')} or the\n ${isc.DocUtils.linkForRef('group:loadWSDLTag','&lt;isc:loadWSDL&gt; JSP tag')} to load the service definition, then\n invoking methods on the resulting ${isc.DocUtils.linkForRef('class:WebService')} object.\n <P>\n ${isc.DocUtils.linkForRef('method:WebService.callOperation')} can be used to manually invoke operations for\n custom processing (example using ${isc.DocUtils.linkForExampleId('wsdlOperation', 'public zipcode service')},\n examples using .NET at\n <a href='/examples/databinding/dotNET/temperatureConvert.jsp' onclick=\"window.open('/examples/databinding/dotNET/temperatureConvert.jsp');return false;\">/examples/databinding/dotNET/temperatureConvert.jsp</a>).\n <P>\n <b>Fetch-only DataSource binding</b>\n <P>\n To bind a component to a web service operation, call\n <P>\n &nbsp;&nbsp;${isc.DocUtils.linkForRef('method:WebService.getFetchDS','WebService.getFetchDS(<i>operationName,elementName</i>)')}\n <P>\n to obtain a DataSource which describes the structure of an XML element or XML Schema type\n named <i>elementName</i>, which appears in the response message for the operation named\n <i>operationName</i>.  A component bound to this DataSource will show fields corresponding\n to the structure of the chosen XML element or type, that is, one field per subelement or\n attribute.  ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} called on this DataSource (or on a\n component bound to it) will invoke the specified web service operation, using the\n ${isc.DocUtils.linkForRef('type:Criteria')} passed to fetchData() to fill out the input message via\n ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}, and using the specified XML element from the response\n message as data.\n <P>\n Similarly, ${isc.DocUtils.linkForRef('method:WebService.getInputDS','WebService.getInputDS(<i>operationName</i>)')} returns\n a DataSource suitable for binding to a form that a user will fill out to provide inputs to\n the specified web service operation.  Typical use is to let the user fill in the form, then\n pass the results of ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form.getValues()')} to\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} as criteria.\n <P>\n If the input message to the web service has extra nesting, consider using\n the ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields','useFlatFields')} property to simplify the inputs\n required for <code>fetchData()</code>, and/or to simplify form databinding via\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useFlatFields','component.useFlatFields')}.\n <P>\n Note that the WSDL tab in the Developer Console can provide a clean, simplified view of any \n WSDL file, making it easier to pick out the appropriate <code>operationName</code> and\n <code>elementName</code> parameters to pass to <code>getFetchDS()</code> and other\n ${isc.DocUtils.linkForRef('class:WebService')} methods.\n <P>\n Take a look at the ${isc.DocUtils.linkForExampleId('wsdlBinding', 'Google SOAP Search example')} and the\n <a href='/examples/databinding/dotNET/customerSearch.jsp' onclick=\"window.open('/examples/databinding/dotNET/customerSearch.jsp');return false;\">.NET example</a>\n (/examples/databinding/dotNET/customerSearch.jsp).\n <P>\n <b>Binding with Customized Presentation</b>\n <P>\n Because XML Schema lacks key presentation metadata such as user-viewable titles, typically\n you cannot directly use the DataSources derived from XML Schema embedded in a WSDL file to\n drive visual component DataBinding in your final application.\n <P>\n You can create a DataSource that has custom fields <b>and</b> invokes a web\n service operation by setting ${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')} to match the targetNamespace\n of the ${isc.DocUtils.linkForRef('class:WebService')} (found on the <code>&lt;definitions&gt;</code> element from the\n WSDL file), and setting ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','wsOperation')} to the name of the\n web service operation to invoke.  <code>fetchData()</code> called on such a DataSource will\n invoke the web service operation named by ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','wsOperation')},\n just like a DataSource returned by ${isc.DocUtils.linkForRef('method:WebService.getFetchDS')}.\n <P>\n In contrast to <code>getFetchDS()</code>, creating a DataSource in this way gives you the\n opportunity to:\n <ul>\n <li> declare arbitrary fields, with SmartClient presentation attributes such as titles and\n formatters\n <li> extract any data from the response message, via\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','operationBinding.recordXPath')} and \n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','field.valueXPath')}, and transform it with\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse','transformResponse()')}\n <li> transform the inbound data, if necessary, in order to add metadata such as\n ${isc.DocUtils.linkForRef('attr:DSRequest.startRow')} for paging, or a sessionId for a service requiring authentication\n </ul>\n These techniques are shown in the ${isc.DocUtils.linkForExampleId('wsdlBinding', 'Google SOAP Search example')}.\n <P>\n <b>XML Schema Reuse</b>\n <P>\n Having loaded a WSDL file, all of the XML Schema definitions within the service definition\n get translated to SmartClient ${isc.DocUtils.linkForRef('class:DataSource','DataSources')} and\n ${isc.DocUtils.linkForRef('class:SimpleType','SimpleTypes')} via the rules described by ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')},\n and are available to you via ${isc.DocUtils.linkForRef('method:WebService.getSchema')} and ${isc.DocUtils.linkForRef('attr:DataSourceField.type')}. \n <P>\n You can use the ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')} property to create DataSources that extend\n from XML schema definitions, then add presentation metadata not found in XML schema.\n <P>\n Even if you choose to declare all fields manually, you can leverage XML Schema\n &lt;simpleType&gt; definitions by setting ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field.type')} to the name\n of an XML Schema simple type embedded in the WSDL file.\n <P>\n <b>Round Trip Binding [fetch -> edit -> save]</b>\n <P>\n For full read-write integration with a service that supports the basic\n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource operations')} on persistent data, \n ${isc.DocUtils.linkForRef('class:OperationBinding','OperationBindings')} can be declared for each DataSource operation, and\n the ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','wsOperation')} property can be used to to bind each\n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource operation')} (fetch, update, add, remove) to a\n corresponding web service operation.\n <P>\n For example, this code accomplishes part of the binding to the \n <a href='http://www.google.com/search?q=sforce+partner+wsdl' onclick=\"window.open('http://www.google.com/search?q=sforce+partner+wsdl');return false;\">SalesForce partner web services</a>\n (additional code is required to handle authentication and other details):\n <pre>\n isc.DataSource.create({\n    serviceNamespace : \"urn:partner.soap.sforce.com\",\n    operationBindings : [\n        { operationType:\"fetch\", wsOperation:\"query\", recordName: \"sObject\" },\n        { operationType:\"update\", wsOperation:\"update\", recordName: \"SaveResult\" },\n        { operationType:\"add\", wsOperation:\"create\", recordName: \"SaveResult\" },\n        { operationType:\"remove\", wsOperation:\"delete\", recordName: \"DeleteResult\" }\n    ],\n    ...\n }); \n </pre>\n NOTE: additional code is required to handle authentication and other details, see the\n complete code in isomorphicSDK/examples/databinding/SalesForce.\n <P>\n In this usage, any DSRequest performed on this DataSource invokes the web service operation\n named by the <code>wsOperation</code> property on the corresponding operationBinding, and\n ${isc.DocUtils.linkForRef('attr:DSRequest.data')} is serialized via ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} to form the input\n message to send to the web service.  For example, if a ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is\n invoked and triggers a DSRequest with operationType:\"add\", the DataSource above will invoke\n the \"create\" operation, and ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form.values')} will become\n ${isc.DocUtils.linkForRef('attr:DSRequest.data')} and be serialized to form the input message of the \"create\" web\n service operation.\n <P>\n Typical usage is:\n <ol>\n <li> declare a DataSource that represents the fields of the object as you want them\n represented in the UI.  This DataSource is considered the \"entity DataSource\".  It may\n extend from an XML Schema complex type via ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}.\n <li> use ${isc.DocUtils.linkForRef('class:OperationBinding','operationBindings')} to configure the entity DataSource to\n call the appropriate web service operations for each DataSource operation, and extract\n results via\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')}/${isc.DocUtils.linkForRef('attr:OperationBinding.recordName','recordName')}\n <li> bind components as follows:\n <ul>\n <li> bind ${isc.DocUtils.linkForRef('class:ListGrid','grids')} to the entity DataSource\n <li> bind ${isc.DocUtils.linkForRef('class:SearchForm','SearchForms')} to the input message of the fetch operation\n (obtained via ${isc.DocUtils.linkForRef('method:WebService.getInputDS','webService.getInputDS(\"operationName\")')}.  This is\n done because search inputs are frequently unrelated to the structure of the objects being\n searched for\n <li> bind forms use for editing (\"add\" and \"update\" operations) to the entity DataSource\n </ul>\n <li> use\n ${isc.DocUtils.linkForRef('method:DataSource.transformRequest','transformRequest')}/${isc.DocUtils.linkForRef('method:DataSource.transformResponse','transformResponse')}, \n ${isc.DocUtils.linkForRef('attr:OperationBinding.useFlatFields')} and ${isc.DocUtils.linkForRef('attr:OperationBinding.responseDataSchema')} to\n handle inconsistencies between the WSDL operations and the data you want in the presentation\n layer.\n </ol>\n A complete example of binding to the SalesForce \"partner\" web service, including\n authentication via SOAP headers, saving data and cache sync, inline editing, validation\n error handling and data paging, can be found in [webroot]/examples/databinding/SalesForce.\n <P>\n This requires a SalesForce account.  SalesForce currently offers \n <a href='http://www.google.com/search?hl=en&q=salesforce+developer+account' onclick=\"window.open('http://www.google.com/search?hl=en&q=salesforce+developer+account');return false;\">free developer accounts</a>.\n Please note: this application deals with <b>live data</b> and if you using inline editing\n <b>it will save to SalesForce</b>.\n <P>\n <b>Deployment</b>\n <P>\n For best performance, using the ${isc.DocUtils.linkForRef('group:loadWSDLTag','&lt;isc:loadWSDL&gt; JSP tag')}\n is recommended, as it automatically caches a translated form of the WSDL file.  If you are\n not using the SmartClient server, the WSDL tab in the Developer Console allows you\n to save a .js file representing a WebService object, which can then be loaded and cached\n like a normal JavaScript file.\n <P>\n <B>Creating New WSDL Services</B>\n <P>\n If you have no existing WSDL web service but would like to use web services for integration,\n you can implement the \"SmartClientOperations\" web service described by the\n ${isc.DocUtils.externalLink(isc.Page.getIsomorphicDir()+\"system/schema/SmartClientOperations.wsdl\",\"WSDL file\")} \n included in the SDK.  This simple, 4 operation web service can support any number of\n DataSources.  In this case, you create your DataSources as client-side instances of\n ${isc.DocUtils.linkForRef('class:WSDataSource')} (general client-side DataSource creation is described under\n ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','Creating DataSources')}).  To change the URL where ISC\n expects to find the SmartClientOperations web service, use ${isc.DocUtils.linkForRef('method:WebService.setLocation')}\n like so:<pre>\n      var service = isc.WebService.get(\"urn:operations.smartclient.com\");\n      service.setLocation(\"myURL\");\n </pre>\n <P>\n To implement a web service <b>starting from a WSDL file</b>:\n <ul>\n <li>In the .NET framework, you will use the Web Services Description Language Tool \n <a href='http://www.google.com/search?q=wsdl.exe' onclick=\"window.open('http://www.google.com/search?q=wsdl.exe');return false;\">(wsdl.exe)</a> to generate C# stubs that\n you will add business logic to\n <li>In Java, <a href='http://ws.apache.org/axis/' onclick=\"window.open('http://ws.apache.org/axis/');return false;\">Apache Axis</a> can be used to generate\n Java stubs for implementing a web service\n <li>In Perl, the <a href='http://soaplite.com' onclick=\"window.open('http://soaplite.com');return false;\">SOAP:Lite</a> module can be used to\n implement web services without code generation\n <li>for PHP, the NuSoap module can likewise be used to implement web services without code\n generation\n </ul>",
        name:"wsdlBinding",
        seeAlso:[],
        type:"group"
    },
    "method:CubeGrid.getRowFacetValues":{
        ref:"method:CubeGrid.getRowFacetValues",
        flags:"",
        description:" Return a ${isc.DocUtils.linkForRef('type:FacetValueMap','FacetValueMap')} indicating the facet values for a specific \n row in the grid\n",
        name:"getRowFacetValues",
        seeAlso:[],
        params:[
            {description:"index of the row", optional:false, name:"rowNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"facet values for the specified row. Returns null if the specified row    is not present in the grid.",
         type:"FacetValueMap"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:SimpleType.inheritsFrom":{
        ref:"attr:SimpleType.inheritsFrom",
        flags:"IR",
        description:"Name of another SimpleType from which this type should inherit.\n <P>\n Validators, if any, will be combined.  All other SimpleType properties default to the\n inherited type's value.",
        name:"inheritsFrom",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customSimpleType')}"],
        definingClass:"class:SimpleType"
    },
    "method:ListGrid.discardEdits":{
        ref:"method:ListGrid.discardEdits",
        flags:"A",
        description:"\n Cancel outstanding edits for the specified rows, discarding edit values, and hiding editors \n if appropriate.<br>\n Note that if this method is called on a new edit row (created via \n ${isc.DocUtils.linkForRef('method:ListGrid.startEditingNew')} for example), which has not yet been saved, this method\n will remove the row entirely.\n \n",
        name:"discardEdits",
        seeAlso:[],
        params:[
            {description:"Row to cancel", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"Column to cancel. Note that this parameter is ignored in ListGrids but                        may be required in subclasses of ListGrid where each cell represents\n                        one record in the data set (EG CubeGrid)",
             optional:false, name:"colNum", defaultValue:null, type:"number"},
            {description:"By default this method will hide the editor if                              it is currently showing for the row in question. Passing in\n                              this parameter will leave the editor visible (and just reset\n                              the edit values underneath the editor).",
             optional:true, name:"dontHideEditor", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:ValuesManager.clearErrors":{
        ref:"method:ValuesManager.clearErrors",
        flags:"",
        description:"			Clears all errors from member forms.\n",
        name:"clearErrors",
        seeAlso:[],
        params:[
            {description:"If true, clear any visible error messages.", optional:false,
             name:"showErrors", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "method:SimpleType.normalDisplayFormatter":{
        ref:"method:SimpleType.normalDisplayFormatter",
        flags:"",
        description:" Normal formatter for values of this type used in a ${isc.DocUtils.linkForRef('class:StaticTextItem')} or\n ${isc.DocUtils.linkForRef('class:DetailViewer')}.\n <P>\n When this formatter is called, the SimpleType object is available as \"this\".  \n <P>\n A formatter can make itself configurable on a per-component or per-field basis by\n checking properties on the component or field.  For example, a formatter for account IDs\n may want to omit a prefix in views where it is redundant, and could check a flag\n detailViewer.omitAccountIdPrefix for this purpose.\n\n",
        name:"normalDisplayFormatter",
        seeAlso:[],
        params:[
            {description:"value to be formatted", optional:false, name:"value", defaultValue:null,
             type:"any"},
            {description:"field descriptor from the component calling the formatter, if                      applicable.  Depending on the calling component, this could be a\n                      ${isc.DocUtils.linkForRef('class:FormItem')}, ${isc.DocUtils.linkForRef('object:DetailViewerField')}, etc",
             optional:true, name:"field", defaultValue:null, type:"Field"},
            {description:"component calling this formatter, if applicable", optional:true,
             name:"component", defaultValue:null, type:"DataBoundComponent"},
            {description:"Full record, if applicable", optional:true, name:"record", defaultValue:null,
             type:"Object"}
        ],
        type:"method",
        definingClass:"class:SimpleType"
    },
    "method:FormItem.getValue":{
        ref:"method:FormItem.getValue",
        flags:"",
        description:" Return this form items value.\n <p>\n For valueMap'd items, or items with 'mapValueToDisplay()' / 'mapDisplayToValue()'\n defined, the data value rather than the display value is returned.\n",
        name:"getValue",
        seeAlso:[],
        returns:{description:"value of this element", type:"any"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["formValues"]
    },
    "class:MultiFileItem":{
        methods:[],
        ref:"class:MultiFileItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:MultiFileItem.dataSource", "attr:MultiFileItem.removeButton", 
         "attr:MultiFileItem.editButton"],
        description:"The MultiFileItem provides an interface for a user to save one or more files that are\n related to a DataSource record, where each file is represented by a record in a\n related DataSource.\n <P>\n Use MultiFileItem when a record may have one or more files associated with it (such as\n attachments to an email message) where no information about the files needs to be stored other\n than the files themselves.  If you have several fields associated with each file (such as an\n uploaded document with a version, comments and processes associated with it), consider\n instead an ordinary DataSource with on field of type \"binary\", and using the ${isc.DocUtils.linkForRef('class:FileItem')}\n for upload.\n <P>\n See the ${isc.DocUtils.linkForRef('group:upload','Uploading Files')} overview for more information on upload.\n <P>\n <b>DataSource Setup</B>\n <P>\n In a relationship sometimes called a \"master-detail\" relationship, the MultiFileItem stores\n files in a \"detail\" DataSource which are related to a \"master\" DataSource record being\n edited by the form which contains the MultiFileItem.\n <P>\n To use a MultiFileItem:\n <ul>\n <li> declare a \"detail\" DataSource to store the related files.  At a minimum, this\n DataSource must have:\n <ul>\n <li> a ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')} field\n <li> a field declaring a ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} relationship to the\n primaryKey of the \"master\" DataSource\n <li> a field of type \"binary\"\n </ul>\n <li> ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','bind')} a DynamicForm to the \"master\" DataSource \n <li> in the DynamicForm bound to the \"master\" DataSource, declare a field with\n ${isc.DocUtils.linkForRef('attr:FormItem.editorType','editorType')}:\"MultiFileItem\" and a <code>dataSource</code>\n property set to the ID of the \"detail\" DataSource\n </ul>\n An example \"detail\" DataSource for storing files is shown below.  This \"detail\" DataSource\n assumes a \"master\" DataSource with ${isc.DocUtils.linkForRef('attr:DataSource.ID')} \"masterRecord\" and with a primaryKey\n field \"id\".\n <code>\n   &lt;DataSource ID=\"uploadedFiles\" serverType=\"sql\"&gt;\n     &lt;fields&gt;\n        &lt;field name=\"fileId\" type=\"sequence\" primaryKey=\"true\" hidden=\"true\"/&gt;\n        &lt;field name=\"masterRecordId\" type=\"number\" foreignKey=\"masterRecord.id\" hidden=\"true\"/&gt;\n        &lt;field name=\"file\" type=\"binary\" title=\"File\"/&gt;\n     &lt;/fields&gt;\n   &lt;/DataSource&gt;\n </code>\n <P>\n Aside from a single \"binary\" field, the \"detail\" DataSource should generally have only\n hidden fields, as shown above.  Additional internal fields (such as a \"lastUpdated\" field)\n may be added, but will not be editable via MultiFileItem. \n <P>\n <b>Display</b>\n <P>\n The MultiFileItem appears as a list of files related to the current record.  An optional\n button, the ${isc.DocUtils.linkForRef('attr:MultiFileItem.removeButton','removeButton')} allows removing files.  A\n second optional button, the ${isc.DocUtils.linkForRef('attr:MultiFileItem.editButton','editButton')}, launches a\n picker for uploading further files.\n <P>\n <b>Saving</b>\n <P>\n In all cases, uploading a new file is an \"add\" DSRequest against the\n ${isc.DocUtils.linkForRef('attr:MultiFileItem.dataSource')}.\n <P>\n The MultiFileItem has two modes, according to whether the \"master\" record is being newly created\n via an \"add\" operation or whether the master record is pre-existing (\"update\" operation).\n <P>\n If the master record is pre-existing, each file added by the user is uploaded as soon as the\n user exits the picker launched from the edit button, and the list of files shown in the main\n form reflects the actual list of stored files.  \n <P>\n If the master record is being newly created, files are not actually uploaded until\n <b>after</b> the master record is confirmed saved, and the list of fields shown in the main\n form reflects files which will be uploaded after the master record is saved.\n <P>\n In both cases, if there are multiple files to upload, they are uploaded one at a time, as a\n series of separate \"add\" DSRequests against the ${isc.DocUtils.linkForRef('attr:MultiFileItem.dataSource')}.\n <P>\n Also in both cases, deletion of any file is immediate.  In the case of a pre-existing master\n record, all files shown actually exist as DataSource records, and deletion is performed as a\n \"remove\" DSRequest against the ${isc.DocUtils.linkForRef('attr:MultiFileItem.dataSource')}.",
        name:"MultiFileItem",
        seeAlso:[],
        type:"class",
        groups:["upload"]
    },
    "method:DataSource.viewFile":{
        ref:"method:DataSource.viewFile",
        flags:"",
        description:" Display a file stored in a field of type:\"binary\" in a new browser window.\n <P>\n This will open a new browser window to show the file.  Depending on the file type,\n the user's installed plugins and applications, and the user's browser settings, this may\n cause the file to be actually displayed in the new browser window, or may prompt the\n user to either launch an external application to view the file or save the file to disk.\n\n",
        name:"viewFile",
        seeAlso:[],
        params:[
            {description:"Record or value of primary key field                                                   for record containing the file to\n                                                   view.",
             optional:false, name:"data", defaultValue:null, type:"Record or PKValue"},
            {description:"optional name of the binary field containing the                                         file.  If not provided, the first binary field\n                                         is used",
             optional:true, name:"fieldName", defaultValue:null, type:"String"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:Button.icon":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Button.icon",
        description:"Optional icon to be shown with the button title text.  \n <P>\n Specify as the partial URL to an image, relative to the imgDir of this component.",
        name:"icon",
        setter:["setIcon"],
        valueType:"SCImgURL",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonIcons')}"],
        defaultValue:"null",
        groups:["buttonIcon"],
        definingClass:"class:Button"
    },
    "method:ColumnTree.deselectRecord":{
        ref:"method:ColumnTree.deselectRecord",
        flags:"",
        description:"\n Deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecord(record, false)</code>\n\n",
        name:"deselectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to deselect", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "method:StretchImgButton.setSelected":{
        ref:"method:StretchImgButton.setSelected",
        flags:"",
        description:" Select this object.\n",
        name:"setSelected",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "method:CubeGrid.cellIsSelected":{
        ref:"method:CubeGrid.cellIsSelected",
        flags:"",
        description:"\n Determine whether the cell passed in is seselected in this cubeGrid\n\n  <br><i>methodType</i> tester\n",
        name:"cellIsSelected",
        seeAlso:[],
        params:[
            {description:"cell to test", optional:false, name:"cell", defaultValue:null,
             type:"cell record object"}
        ],
        returns:{description:"true if any cells are selected", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:CubeGrid.deselectAllCells":{
        ref:"method:CubeGrid.deselectAllCells",
        flags:"",
        description:"\n Deselect all cells\n\n      <br><i>methodType</i> action\n",
        name:"deselectAllCells",
        seeAlso:[],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "classMethod:Class.logWarn":{
        ref:"classMethod:Class.logWarn",
        flags:"",
        description:" Log a message at \"warn\" priority\n \n",
        name:"logWarn",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:TileGrid.showDetailFields":{
        ref:"attr:TileGrid.showDetailFields",
        flags:"IR",
        description:"By default, TileGrids will not show fields marked ${isc.DocUtils.linkForRef('attr:DataSourceField.detail','detail:true')}\n in the DataSource.  See also ${isc.DocUtils.linkForRef('attr:TileGrid.fields')}.",
        name:"showDetailFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:ListGrid.recordBaseStyleProperty":{
        ref:"attr:ListGrid.recordBaseStyleProperty",
        flags:"IRWA",
        description:"This attribute allows custom base styles to be displayed on a per-record basis.\n To specify a custom base-style for some record set \n <code>record[listGrid.recordBaseStyleProperty]</code> to the desired base style name - \n for example if <code>recordBaseStyleProperty</code> is <code>\"_baseStyle\"</code>, set\n <code>record._baseStyle</code> to the custom base style name.",
        name:"recordBaseStyleProperty",
        seeAlso:[
            {ref:"attr:ListGrid.baseStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"\"_baseStyle\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:StatefulCanvas.showFocusedAsOver":{
        ref:"attr:StatefulCanvas.showFocusedAsOver",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused','showFocused')} is true for this widget, should the\n <code>\"over\"</code> state be used to indicate the widget as focused. If set to false,\n a separate <code>\"focused\"</code> state will be used.",
        name:"showFocusedAsOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "attr:FormItem.displayField":{
        ref:"attr:FormItem.displayField",
        flags:"IR",
        getter:["FormItem.getDisplayFieldName"],
        description:"Specifies an alternative field from which display values should be retrieved for this\n item.\n <P>\n The display field can be either another field value in the same record or a field that\n must be retrieved from a related ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}.\n <P>\n If this item is not databound (${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is unset), or bound \n to the same dataSource as the form as a whole, this item will call\n ${isc.DocUtils.linkForRef('method:DynamicForm.getValue','form.getValue()')} \n the form named after is implemented by picking up the\n value of the specified field from the Form's values object.\n <P>\n Otherwise this item will attempt to map its underlying value to a display value\n by retrieving a record from the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} where the \n ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} matches this item's value, and displaying the \n <code>displayField</code> value from that record.\n Note that if <code>optionDataSource</code> is set and this value is not\n set, ${isc.DocUtils.linkForRef('method:FormItem.getDisplayFieldName')} will return the dataSource title field by default.\n <P>\n This essentially enables the specified <code>optionDataSource</code> to be used as\n a server based ${isc.DocUtils.linkForRef('group:valueMap')}.",
        name:"displayField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "attr:Canvas.showCustomScrollbars":{
        ref:"attr:Canvas.showCustomScrollbars",
        flags:"IRWA",
        description:"Whether to use the browser's native scrollbars or SmartClient-based scrollbars.\n <P>\n SmartClient-based scrollbars are skinnable, giving you complete control over look and\n feel.  SmartClient-based scrollbars also enable some interactions not possible with\n native scrollbars, such as ${isc.DocUtils.linkForRef('attr:ListGrid.fixedRecordHeights','variable height records')}\n in grids in combination with ${isc.DocUtils.linkForRef('attr:ListGrid.dataPageSize','data paging')}.\n <P>\n Native browser scrollbars are slightly faster simply because there are less SmartClient\n components that need to be created, drawn and updated.  Each visible SmartClient-based\n scrollbar on the screen has roughly the impact of two StretchImgButtons.\n <P>\n SmartClient is always aware of the size of the scrollbar, regardless of whether native\n or custom scrollbars are used, and regardless of what operating system and/or operating\n system \"theme\" or \"skin\" is in use.  This means SmartClient will correctly report the\n ${isc.DocUtils.linkForRef('method:Canvas.getViewportHeight','viewport size')}, that is, the interior area of the\n widget excluding space taken by scrollbars, which is key for exactly filling a component\n with content without creating unnecessary scrolling.\n <P>\n The <code>showCustomScrollbars</code> setting is typically overridden in load_skin.js in\n order to change the default for all SmartClient components at once, like so:\n <pre>\n     isc.Canvas.addProperties({ showCustomScrollbars:false });\n </pre>",
        name:"showCustomScrollbars",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:OperationBinding.exportFields":{
        ref:"attr:OperationBinding.exportFields",
        flags:"IR",
        description:"The list of field-names to export.  If provided, the field-list in the exported output is \n limited and sorted as per the list.\n <P>\n If exportFields is not provided, the exported output includes all visible fields \n from the DataSource (field.hidden=false), sorted in the order they're defined.",
        name:"exportFields",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "method:ListGrid.startEditingNew":{
        ref:"method:ListGrid.startEditingNew",
        flags:"A",
        description:"\n Start editing a new row, after the last pre-existing record in the current set of data.\n <P>\n This new row will be saved via the \"add\" ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource\n operation')}.\n <P>\n If editing is already underway elsewhere in the grid, startEditingNew() behaves just like\n ${isc.DocUtils.linkForRef('method:ListGrid.startEditing')}.\n\n",
        name:"startEditingNew",
        seeAlso:[
            {ref:"method:ListGrid.startEditing", description:null}
        ],
        params:[
            {description:"Optional initial set of properties for the new record", optional:true,
             name:"newValues", defaultValue:null, type:"object"},
            {description:"Whether to suppress the default behavior of moving focus                                   to the newly shown editor.",
             optional:true, name:"suppressFocus", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:WebService.getOperationNames":{
        ref:"method:WebService.getOperationNames",
        flags:"",
        description:null,
        name:"getOperationNames",
        seeAlso:[],
        returns:{description:"names of the available operations supported by this service (array of strings)",
         type:"Array"},
        type:"method",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "method:Date.toEuropeanShortDateTime":{
        ref:"method:Date.toEuropeanShortDateTime",
        flags:"",
        description:" Return this date in the format: <code>DD/MM/YYYY HH:MM</code>.\n",
        name:"toEuropeanShortDateTime",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:DynamicForm.operator":{
        ref:"attr:DynamicForm.operator",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('attr:FormItem.operator')} has been set for any ${isc.DocUtils.linkForRef('class:FormItem')} in this form, what\n logical operator should be applied across the ${isc.DocUtils.linkForRef('object:Criterion','criteria')} produced by the form\n items?  Only applicable to forms that have a ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','dataSource')}.",
        name:"operator",
        seeAlso:[],
        valueType:"OperationId",
        defaultValue:"\"and\"",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "attr:Tree.root":{
        flags:"IRW",
        getter:["getRoot"],
        type:"attr",
        ref:"attr:Tree.root",
        description:"If you're using the \"parent\" modelType, you can provide the root node configuration via this\n property.  If you don't provide it, one will be auto-created for you with an empty name.\n Read on for a description of what omitting the name property on the root node means for path\n derivation.\n <p>\n If you're using the \"children\" modelType, you can provide the initial tree data via this\n property.  So, for example, to construct the following tree:\n <pre>\n foo\n   bar\n zoo\n </pre>\n You would initialize the tree as follows: \n <pre>\n Tree.create({\n     root: { name:\"root\", children: [\n         { name:\"foo\", children: [\n             { name: \"bar\" }\n         ]},\n         { name: \"zoo\" }\n     ]}\n });\n </pre>\n Note that if you provide a <code>name</code> property for the root node, then the path to\n any node underneath it will start with that name.  So in the example above, the path to the\n <code>bar</code> node would be <code>root/foo/bar</code> (assuming you're using the default\n ${isc.DocUtils.linkForRef('attr:Tree.pathDelim')}.  If you omit the name attribute on the root node, then it's name\n is automatically set to the ${isc.DocUtils.linkForRef('attr:Tree.pathDelim')} value.  So in the example above, if\n you omitted <code>name:\"root\"</code>, then the path to the <code>bar</code> node would be\n <code>/foo/bar</code>.\n <br><br>\n Note: if you initialize a Tree with no <code>root</code> value, a root node will be\n auto-created for you.  You can then call ${isc.DocUtils.linkForRef('method:Tree.add')} to construct the tree.",
        name:"root",
        setter:["setRoot"],
        valueType:"TreeNode",
        seeAlso:[
            {ref:"attr:Tree.modelType", description:null},
            {ref:"method:Tree.setRoot", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('childrenArrays')}"],
        defaultValue:"null",
        definingClass:"class:Tree"
    },
    "classAttr:ValidatorTypes.mask":{
        ref:"classAttr:ValidatorTypes.mask",
        flags:"IR",
        description:"Validate against a regular expression mask, specified as <code>validator.mask</code>.\n  If validation is successful a transformation can also be specified via the\n  <code>validator.transformTo</code> property. This should be set to a string in the\n  standard format for string replacement via the native JavaScript <code>replace()</code>\n  method.",
        name:"mask",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        examples:["${isc.DocUtils.linkForExampleId('valueTransform')}"],
        definingClass:"class:ValidatorTypes"
    },
    "attr:MultiSortDialog.initialSort":{
        ref:"attr:MultiSortDialog.initialSort",
        flags:"IR",
        description:"The initial sort configuration to show in the ${isc.DocUtils.linkForRef('attr:MultiSortPanel.optionsGrid')}.",
        name:"initialSort",
        seeAlso:[],
        valueType:"Array of SortSpecifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:Canvas.animateRect":{
        ref:"method:Canvas.animateRect",
        flags:"",
        description:" Animate a reposition / resize of this canvas from its current size and postion.\n",
        name:"animateRect",
        seeAlso:[],
        params:[
            {description:"new left position (or null for unchanged)", optional:false, name:"left",
             defaultValue:null, type:"number"},
            {description:"new top position (or null for unchanged)", optional:false, name:"top",
             defaultValue:null, type:"number"},
            {description:"new width (or null for unchanged)", optional:false, name:"width",
             defaultValue:null, type:"number"},
            {description:"new height (or null for unchanged)", optional:false, name:"height",
             defaultValue:null, type:"number"},
            {description:"When the setRect completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated setRect", optional:true, name:"duration",
             defaultValue:null, type:"number"},
            {description:"Optional acceleration effect to apply to the animation", optional:true,
             name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateZoom')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:DataSource.getPrimaryKeyField":{
        ref:"method:DataSource.getPrimaryKeyField",
        flags:"",
        description:" Returns a pointer to the primaryKey field for this DataSource\n",
        name:"getPrimaryKeyField",
        seeAlso:[],
        returns:{description:"primary key field object", type:"DataSourceField"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "method:GridRenderer.rowMouseUp":{
        ref:"method:GridRenderer.rowMouseUp",
        flags:"A",
        description:" Called when a row receives a mouseup event.\n\n",
        name:"rowMouseUp",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:DataSource.getSearchOperator":{
        ref:"method:DataSource.getSearchOperator",
        flags:"",
        description:" Get the ${isc.DocUtils.linkForRef('object:Operator')} definition for an ${isc.DocUtils.linkForRef('type:OperatorId')}.\n\n",
        name:"getSearchOperator",
        seeAlso:[],
        params:[
            {description:"the id of the operator", optional:false, name:"operatorId", defaultValue:null,
             type:"OperatorId"}
        ],
        returns:{description:"the ${isc.DocUtils.linkForRef('object:Operator')} definition", type:"Operator"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "group:access":{
        ref:"group:access",
        refs:["method:ResultSet.getLength", "method:ResultSet.indexOf", "method:ResultSet.get", 
         "method:ResultSet.getRange","method:ResultSet.findAll", "method:ResultSet.find", "method:ResultSet.findIndex", 
         "method:ResultSet.findNextIndex","method:List.get", "method:List.getLength", "method:List.isEmpty", "method:List.first", 
         "method:List.last","method:List.indexOf", "method:List.lastIndexOf", "method:List.findIndex", 
         "method:List.findNextIndex","method:List.find", "method:List.findAll", "method:List.contains", 
         "method:List.containsAll","method:List.equals", "method:List.getItems", "method:List.getRange", 
         "method:List.duplicate","method:Tree.indexOf", "method:Tree.lastIndexOf", "method:Array.get", 
         "method:Array.getLength","method:Array.isEmpty", "method:Array.first", "method:Array.last", "method:Array.indexOf", 
         "method:Array.lastIndexOf","method:Array.contains", "method:Array.containsAll", "method:Array.equals", 
         "method:Array.getItems","method:Array.getRange", "method:Array.duplicate", "method:Array.findIndex", 
         "method:Array.findNextIndex","method:Array.find", "method:Array.findAll"],
        name:"access",
        seeAlso:[],
        type:"group"
    },
    "attr:PickTreeItem.loadDataOnDemand":{
        ref:"attr:PickTreeItem.loadDataOnDemand",
        flags:"IRA",
        description:"If this is a databound item, should the load our set of possible options be loaded\n on demand (as submenus are displayed), or upfront?",
        name:"loadDataOnDemand",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:PickTreeItem"
    },
    "attr:Button.baseStyle":{
        ref:"attr:Button.baseStyle",
        flags:"IRW",
        description:"Base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n <P>\n When the component changes state (eg becomes disabled), a suffix will be appended to this\n style name, reflecting the following states: \"Over\", \"Down\", or \"Disabled\".\n <P>\n If the widget is selected, the suffixes will be \"Selected\", \"SelectedOver\", etc.\n <P>\n If the widget has focus and ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')} is true, and\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is false, the suffixes will be \"Focused\",\n \"FocusedOver\", etc, or if the widget is both selected and focused, \"SelectedFocused\",\n \"SelectedFocusedOver\", etc.\n <P>\n For example, if <code>baseStyle</code> is set to \"button\", this component is\n ${isc.DocUtils.linkForRef('method:Button.isSelected','selected')} and the mouse cursor is over this component, the style\n \"buttonSelectedOver\" will be used.",
        name:"baseStyle",
        setter:["setBaseStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"button\"",
        type:"attr",
        definingClass:"class:Button"
    },
    "attr:ImgTab.baseStyle":{
        ref:"attr:ImgTab.baseStyle",
        flags:"IR",
        description:null,
        name:"baseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"tab\"",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "attr:CheckboxItem.showLabel":{
        ref:"attr:CheckboxItem.showLabel",
        flags:"IRW",
        description:"Should we should the label text next to the checkbox item.",
        name:"showLabel",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CheckboxItem"
    },
    "type:ImageStyle":{
        ref:"type:ImageStyle",
        values:[
            {description:"Center (and don't stretch at all) the image if smaller than its enclosing frame.CENTER:\"center\",",
             value:"Canvas.CENTER"},
            {description:"Tile (repeat) the image if smaller than its enclosing frame.",
             value:"Canvas.TILE"},
            {description:"Stretch the image to the size of its enclosing frame.",
             value:"Canvas.STRETCH"},
            {description:"Allow the image to have natural size", value:"Canvas.NORMAL"}
        ],
        description:null,
        name:"ImageStyle",
        seeAlso:[],
        type:"type",
        groups:["appearance"]
    },
    "method:ValuesManager.setErrors":{
        ref:"method:ValuesManager.setErrors",
        flags:"A",
        description:" Sets validation errors for this valuesManager to the specified set of errors.\n Errors should be of the format:<br>\n <code>{field1:errors, field2:errors}</code><br>\n where each <code>errors</code> object is either a single error message string or an\n array of error messages.<br>\n If <code>showErrors</code> is passed in, error messages will be displayed in the \n appropriate member form items. For fields with no visible form item, \n ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')} will be fired instead.<br>\n Note that if <code>showErrors</code> is false, errors may be shown at any time via\n a call to ${isc.DocUtils.linkForRef('method:ValuesManager.showErrors')}.\n\n",
        name:"setErrors",
        seeAlso:[],
        params:[
            {description:"list of errors as an object with the field namesas keys", optional:false,
             name:"errors", defaultValue:null, type:"object"},
            {description:"If true display errors now.", optional:false, name:"showErrors",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:FormItem.valueField":{
        ref:"attr:FormItem.valueField",
        flags:"IR",
        getter:["FormItem.getValueFieldName"],
        description:"If this form item maps data values to display values by retrieving the \n ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} values from an \n ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}, this property \n denotes the the field to use as the underlying data value in records from the \n optionDataSource.<br>\n If unset, assumed to be the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this form item.",
        name:"valueField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "type:ChartType":{
        ref:"type:ChartType",
        values:[
            {description:"Values represented by vertical columns.  Typically supports stacking to                  represent two facets.  May support simultaneous stacking and clustering for\n                  3 facets.",
             value:"Column"},
            {description:"Values represented by horizontal bars.  Typically supports stacking to                  represent two facets.  May support simultaneous stacking and clustering for\n                  3 facets.",
             value:"Bar"},
            {description:"Values represented by a line between data points.", value:"Line"},
            {description:"Circular chart with wedges representing values.  Single facet only",
             value:"Pie"},
            {description:"Like a pie chart with a central hole.  Single facet only", value:"Doughnut"}
        ],
        description:"Known chart types.  These are visual representations of data, not separate data models,\n although some chart types are only capable of showing a single facet of data.\n <P>\n Concrete charting implementations may use any value for <code>chartType</code> but should\n support the provided <code>chartType</code> values for charts that correspond to the visual\n presentation described below, to enable easy switching between charting engines.",
        name:"ChartType",
        seeAlso:[],
        type:"type"
    },
    "attr:PickList.separatorRows":{
        ref:"attr:PickList.separatorRows",
        flags:"IR",
        description:"Array of records to show between matching and non-matching rows in the PickList.\n <P>\n Not valid for ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource','databound pickLists')}.",
        name:"separatorRows",
        seeAlso:[],
        valueType:"Array of ListGridRecord",
        defaultValue:"[{isSeparator:true}]",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "method:StretchImgButton.addToRadioGroup":{
        ref:"method:StretchImgButton.addToRadioGroup",
        flags:"",
        description:" Add this widget to the specified mutually exclusive selection group with the ID\n passed in.\n Selecting this widget will then deselect any other StatefulCanvases with the same\n radioGroup ID.\n StatefulCanvases can belong to only one radioGroup, so this method will remove from \n any other radiogroup of which this button is already a member.\n",
        name:"addToRadioGroup",
        seeAlso:[],
        params:[
            {description:"- ID of the radiogroup to which this widget should be added",
             optional:false, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state", "event handling"]
    },
    "attr:StaticTextItem.clipValue":{
        ref:"attr:StaticTextItem.clipValue",
        flags:"IRW",
        description:"If true, text that exceeds the specified size of the form item will be clipped",
        name:"clipValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StaticTextItem",
        groups:["appearance"]
    },
    "attr:DynamicForm.autoFocus":{
        ref:"attr:DynamicForm.autoFocus",
        flags:"IRW",
        description:"If true, when the form is drawn, focus will automatically be put into the first focusable\n element in the form.<br>\n Note that to put focus in a different item you can explicitly call \n <code>dynamicForm.focusInItem(<i>itemName</i>)</code>",
        name:"autoFocus",
        seeAlso:[
            {ref:"method:DynamicForm.focusInItem", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["focus"]
    },
    "method:ListGrid.fetchData":{
        ref:"method:ListGrid.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component.\n <P>\n If there are a large number of matching records, paging will automatically be enabled, so\n that initially a smaller number of records will be retrieved and further records will\n be fetched as the user navigates the dataset.\n <P>\n When first called, this method will create a ${isc.DocUtils.linkForRef('class:ResultSet')}, which will be\n configured based on component settings such as ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fetchOperation')}\n and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataPageSize')}, as well as the general purpose\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')}.  The ResultSet is then available as\n <code>component.data</code>.\n <P>\n Subsequent calls to fetchData() will simply call ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')}\n on the created ResultSet with the passed criteria.\n <P>\n In some cases fetchData() will not need to contact the server as the new criteria can be\n satisfied by performing a client-side filter against the currently cached set of data.\n You can determine whether criteria will cause a fetch by calling \n ${isc.DocUtils.linkForRef('method:ResultSet.willFetchData')}.  If you need to force a server fetch, you can call\n ${isc.DocUtils.linkForRef('method:ListGrid.invalidateCache','invalidateCache()')} to do so.\n <P>\n This method takes an optional callback parameter (set to a ${isc.DocUtils.linkForRef('type:DSCallback')}) to fire when\n the fetch completes. Note that this callback will not fire if no server fetch is performed.\n In this case the data is updated synchronously, so as soon as this method completes you\n can interact with the new data. If necessary, you can use\n {dataBoundComponent.willFetchData,willFetchData()} to determine whether or not a server\n fetch will occur when <code>fetchData()</code> is called with new criteria.\n <P>\n In addition to the callback parameter for this method, developers can use \n ${isc.DocUtils.linkForRef('method:ListGrid.dataArrived','dataArrived()')} to be notified every time data is loaded.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundFetch')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:ListGrid.canExpandRecordProperty":{
        ref:"attr:ListGrid.canExpandRecordProperty",
        flags:"IR",
        description:"Property name on a record that will be checked to determine whether a record can be expanded.",
        name:"canExpandRecordProperty",
        seeAlso:[
            {ref:"attr:ListGridRecord.canExpand", description:null}
        ],
        valueType:"string",
        defaultValue:"\"canExpand\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:CubeGrid.autoSelectHeaders":{
        ref:"attr:CubeGrid.autoSelectHeaders",
        flags:"IRW",
        description:"If true, when multiple facets appear on one side in a nested headers presentation, the\n selection state of parent/child headers are automatically kept in sync.",
        name:"autoSelectHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:FormItem.optionOperationId":{
        ref:"attr:FormItem.optionOperationId",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, this attribute may be set\n to specify an explicit ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} when performing a fetch against the\n option dataSource to pick up display value mapping.",
        name:"optionOperationId",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:TabSet.getSelectedTab":{
        ref:"method:TabSet.getSelectedTab",
        flags:"A",
        description:" Returns the currently selected tab object.  This is the object literal used to configure the\n tab, rather than the tab button widget.\n",
        name:"getSelectedTab",
        seeAlso:[],
        returns:{description:"the currently selected Tab object", type:"Tab"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "group:featureExplorerOverview":{
        ref:"group:featureExplorerOverview",
        title:"Feature Explorer Overview",
        refs:[],
        description:"The ${isc.DocUtils.linkForDocNode('FeatureExplorer', 'Feature Explorer')} is an example shell designed to help you\n explore the capabilities of SmartClient.  Read on for a brief overview, including specific\n instructions for using the example code in your own standalone application.\n <p>\n The tree on the left of the Feature Explorer contains examples grouped by logical\n categories.  Selecting an example brings it up on the right side of the screen, inside a\n tabbed view.  The default \"View\" tab shows the interactive example itself, while other\n tabs show the underlying source.  The \"JS\" tab shows the source for the example.  There is\n sometimes also an \"XML\" tab that shows the corresponding version in \n ${isc.DocUtils.linkForRef('group:componentXML','Component XML')} format.  For databound examples, there are also\n frequently additional tabs that show the ${isc.DocUtils.linkForRef('class:DataSource','DataSources')} associated with the\n example.\n <p>\n <b><u>How to create a standalone application using code from the Feature Explorer</u></b>\n <p>\n The Feature Explorer shell is designed to show many examples in one place and to enable\n experimentation by providing a way for you to modify example code and try out your changes.\n As a result, the Feature Explorer requires some basic server support and its examples omit\n the usual SmartClient module includes that have to be in place for using SmartClient\n components standalone.\n <p>\n If you'd like to use example code in your application to get started quickly, create a page\n with SmartClient includes and then take the code from the \"JS\" tab and place it between\n &lt;SCRIPT&gt; blocks as described ${isc.DocUtils.linkForRef('group:nonJavaBackend','here')}.  If the example also\n includes a datasource, place the datasource definitions in the same file before the\n component code.  Note that DataSources (and components) written in XML require the optional\n SmartClient server.  If you're using the server, you can include them on your page using the \n ${isc.DocUtils.linkForRef('group:loadDSTag')} tag.\n <p>\n <b><u>Feature Explorer difference in the LGPL package</u></b>\n <p>\n The LGPL edition of SmartClient does not include the SmartClient Java Server as part of the\n licensed software, but a trimmed down server is included in the package to support the\n Feature Explorer shell.  There are some examples that use DataSources that would normally\n use the SmartClient server for persistence.  In the LGPL package, these DataSources are\n automatically turned into ${isc.DocUtils.linkForRef('group:clientOnlyDataSources','Client Only DataSources')} and\n the Feature Explorer loads the data for these one-time from the dataURL or testFileName\n attributes specified on the DataSource.  Subsequent DataSource operations work against this\n client-side dataset, which is why changes to the data aren't permanent in these examples.\n <p>\n Check out the ${isc.DocUtils.linkForRef('group:clientServerIntegration','Client-Server Integration')} overview topic for\n an overview of your DataBinding options.",
        name:"featureExplorerOverview",
        seeAlso:[],
        type:"group"
    },
    "group:identity":{
        ref:"group:identity",
        refs:["attr:DataSource.ID", "attr:DataSource.addGlobalId"],
        name:"identity",
        seeAlso:[],
        type:"group"
    },
    "method:MultiView.recordClick":{
        ref:"method:MultiView.recordClick",
        flags:"",
        description:" Handles a click on a record.\n\n",
        name:"recordClick",
        seeAlso:[],
        params:[
            {description:"Clicked record", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"component which recieved the click event", optional:false, name:"view",
             defaultValue:null, type:"DataBoundComponent"}
        ],
        type:"method",
        definingClass:"class:MultiView"
    },
    "method:CheckboxItem.setValueMap":{
        ref:"method:CheckboxItem.setValueMap",
        flags:"",
        description:" Setter method to apply a valueMap to a checkbox item.<br>\n Note that if this method is overridden, the override must call\n <code>this.Super(\"setValueMap\", arguments);</code> to maintain functionality in this\n class.\n",
        name:"setValueMap",
        seeAlso:[
            {ref:"attr:CheckboxItem.valueMap", description:null}
        ],
        type:"method",
        definingClass:"class:CheckboxItem",
        groups:["valueMap"]
    },
    "method:ListGrid.getFieldWidth":{
        ref:"method:ListGrid.getFieldWidth",
        flags:"",
        description:" Returns a numeric value for the width of some field within this listGrid.\n",
        name:"getFieldWidth",
        seeAlso:[],
        params:[
            {description:"Index or fieldName of field for which width is to be determined.",
             optional:false, name:"fieldNum", defaultValue:null, type:"Number | String"}
        ],
        returns:{description:"width of the field in px, or null if the width can't be determined",
         type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DataSource.requestProperties":{
        ref:"attr:DataSource.requestProperties",
        flags:"IR",
        description:"Additional properties to pass through to the ${isc.DocUtils.linkForRef('class:DSRequest')}s made by this\n DataSource.\n <p>\n These properties are applied before ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} is called.",
        name:"requestProperties",
        seeAlso:[
            {ref:"class:DSRequest", description:null},
            {ref:"attr:OperationBinding.requestProperties", description:null}
        ],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration", "serverDataIntegration"]
    },
    "attr:Canvas.animateRectAcceleration":{
        ref:"attr:Canvas.animateRectAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated move and resize.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateRectAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:OperationBinding.exportFilename":{
        ref:"attr:OperationBinding.exportFilename",
        flags:"IR",
        description:"The name of the file to save the exported data into.",
        name:"exportFilename",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "group:modification":{
        ref:"group:modification",
        refs:["method:List.set", "method:List.addAt", "method:List.removeAt", "method:List.add", 
         "method:List.addList","method:List.setLength", "method:List.addListAt", "method:List.remove", 
         "method:List.removeList","method:List.dataChanged", "method:Array.set", "method:Array.addAt", 
         "method:Array.removeAt","method:Array.add", "method:Array.addList", "method:Array.setLength", 
         "method:Array.addListAt","method:Array.remove", "method:Array.removeList", "method:Array.dataChanged"],
        name:"modification",
        seeAlso:[],
        type:"group"
    },
    "classAttr:GroupingMessages.upcomingTodayTitle":{
        ref:"classAttr:GroupingMessages.upcomingTodayTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs today,\n relative to the current date.",
        name:"upcomingTodayTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Today\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:RestDataSource.operationBindings":{
        ref:"attr:RestDataSource.operationBindings",
        flags:"IR",
        description:"RestDataSource OperationBindings set to specify default dataProtocol per operationType.\n Default databindings are:\n <pre>\n   operationBindings : [\n     {operationType:\"fetch\", dataProtocol:\"getParams\"},\n     {operationType:\"add\", dataProtocol:\"postParams\"},\n     {operationType:\"remove\", dataProtocol:\"postParams\"},\n     {operationType:\"update\", dataProtocol:\"postParams\"} \n   ],\n </pre>\n If you are integrating with a ${isc.DocUtils.linkForRef('class:RestDataSource','REST')} server that requires the more\n obscure ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')}s of \"PUT\", \"DELETE\" or \"HEAD\", you can specify these\n httpMethod settings via ${isc.DocUtils.linkForRef('attr:OperationBinding.requestProperties')}.  dataProtocol settings\n that mention \"GET\" or \"POST\" are compatible with these additional HTTP methods as well.\n Typical ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings','operationBindings')} for a REST server that uses\n \"PUT\" and \"DELETE\" are as follows:\n <pre>\n   operationBindings:[\n     {operationType:\"fetch\", dataProtocol:\"getParams\"},\n     {operationType:\"add\", dataProtocol:\"postParams\"},\n     {operationType:\"remove\", dataProtocol:\"getParams\", requestProperties:{httpMethod:\"DELETE\"}},\n     {operationType:\"update\", dataProtocol:\"postParams\", requestProperties:{httpMethod:\"PUT\"}}\n   ],\n </pre>",
        name:"operationBindings",
        seeAlso:[],
        valueType:"Array of OperationBinding",
        defaultValue:"[...]",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:HTMLFlow.contentsURLParams":{
        ref:"attr:HTMLFlow.contentsURLParams",
        flags:"IRW",
        description:"Parameters to be sent to the contentsURL when fetching content.",
        name:"contentsURLParams",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "classAttr:Time.PMIndicator":{
        ref:"classAttr:Time.PMIndicator",
        flags:"RWA",
        description:"String appended to times to indicate am (when not using 24 hour format).",
        name:"PMIndicator",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\" pm\"",
        type:"classAttr",
        definingClass:"class:Time",
        groups:["i18nMessages"]
    },
    "attr:Selection.selectionRangeNotLoadedMessage":{
        ref:"attr:Selection.selectionRangeNotLoadedMessage",
        flags:"IRWA",
        description:"Message to display to the user in a <code>warn</code> dialog if ${isc.DocUtils.linkForRef('method:Selection.selectRange')} is\n called for a selection on a ResultSet, where the range of records to be selected has not been\n loaded.",
        name:"selectionRangeNotLoadedMessage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Can't select that many records at once.&lt;br&gt;&lt;br&gt;Please try working in smaller batches.",
        type:"attr",
        definingClass:"class:Selection",
        groups:["i18nMessages"]
    },
    "classMethod:EventHandler.ctrlKeyDown":{
        ref:"classMethod:EventHandler.ctrlKeyDown",
        flags:"",
        description:"			Return true if the control key is being held down.\n			Note that this is only set reliably for keyboard events.\n\n",
        name:"ctrlKeyDown",
        seeAlso:[],
        returns:{description:"true == control key is down", type:"boolean"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "group:layoutPolicy":{
        ref:"group:layoutPolicy",
        refs:["attr:TileLayout.layoutPolicy", "attr:TileLayout.orientation", 
         "attr:TileLayout.tilesPerLine","attr:TileLayout.autoWrapLines", "attr:Layout.overflow", "attr:Layout.orientation", 
         "attr:Layout.vertical","attr:Layout.vPolicy", "attr:Layout.hPolicy", "attr:Layout.minMemberSize", 
         "attr:Layout.enforcePolicy","attr:Layout.defaultLayoutAlign", "attr:Layout.align", "attr:Layout.reverseOrder"],
        name:"layoutPolicy",
        seeAlso:[],
        type:"group"
    },
    "attr:FacetValue.canEdit":{
        ref:"attr:FacetValue.canEdit",
        flags:"IRW",
        description:"Whether cells for this facetValue can be edited.  Defaults to cubeGrid.canEdit",
        name:"canEdit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"cubeGrid.canEdit",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:FormulaBuilder.autoHideCheckBoxLabel":{
        ref:"attr:FormulaBuilder.autoHideCheckBoxLabel",
        flags:"IRW",
        description:"Text label for the checkbox that allows the user to automatically hide the\n fields used in the formula.",
        name:"autoHideCheckBoxLabel",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Auto hide fields used in formula\"",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["i18nMessages"]
    },
    "method:DateItem.getInputFormat":{
        ref:"method:DateItem.getInputFormat",
        flags:"A",
        description:" If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is <code>true</code> this method returns a\n standard ${isc.DocUtils.linkForRef('type:DateInputFormat')}, or parsing function, determining how values entered\n by the user are to be converted to Javascript Date objects.\n <P>\n If an explicit ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat')} has been specified it will be returned.\n <P>\n Otherwise, if an explicit ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} has been specified as one of\n <code>toUSShortDate</code>, <code>toEuropeanShortDate</code> or <code>toJapanShortDate</code>\n this method will return the appropriate DateInputFormat to parse strings entered in the\n specified displayFormat.\n <P>\n If no inputFormat can be derived, the method will return null, meaning the standard input\n format for Date objects (specified via ${isc.DocUtils.linkForRef('classMethod:Date.setInputFormat')}) will be used.\n",
        name:"getInputFormat",
        seeAlso:[],
        returns:{description:"expected format of date strings to parse", type:"DateInputFormat"},
        type:"method",
        definingClass:"class:DateItem"
    },
    "attr:DetailViewer.labelSuffix":{
        ref:"attr:DetailViewer.labelSuffix",
        flags:"IRW",
        description:"text to put after a label",
        name:"labelSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\":\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["labels"]
    },
    "group:files":{
        ref:"group:files",
        refs:["classAttr:Page.defaultUnsupportedBrowserURL", "classMethod:Page.getAppDir", 
         "classMethod:Page.setAppImgDir","classMethod:Page.getAppImgDir", "classMethod:Page.setAppFilesDir", 
         "classMethod:Page.getAppFilesDir","classMethod:Page.setIsomorphicDir", "classMethod:Page.getIsomorphicDir", 
         "classMethod:Page.setSkinDir","classMethod:Page.getSkinDir", "classMethod:Page.getSkinImgDir", 
         "classMethod:Page.getImgURL","classMethod:Page.getURL", "classMethod:Page.loadStyleSheet"],
        description:"Refering to and loading other files.\n <P>\n The two main URL settings relevant to file loading are:<br>\n * appDir  (where application-specific files live)<br>\n * isomorphicDir (where system supplied files live)<br>",
        name:"files",
        seeAlso:[],
        type:"group"
    },
    "attr:DataSource.callbackParam":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.callbackParam",
        description:"Applies only to dataFormat: \"json\".  Specifies the name of the query parameter that\n tells your JSON service what function to call as part of the response.",
        name:"callbackParam",
        valueType:"String",
        seeAlso:[
            {ref:"attr:DataSource.dataFormat", description:null},
            {ref:"attr:DataSource.operationBindings", description:null},
            {ref:"attr:OperationBinding.callbackParam", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('xmlEditSave')}"],
        serverDS:"allowed",
        defaultValue:"\"callback\"",
        groups:["clientDataIntegration"],
        definingClass:"class:DataSource"
    },
    "classMethod:Class.getClassName":{
        ref:"classMethod:Class.getClassName",
        flags:"",
        description:"\n	Gets the name of this class as a string.\n\n",
        name:"getClassName",
        seeAlso:[],
        returns:{description:"name of the class", type:"string"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:GridRenderer.getVisibleRows":{
        ref:"method:GridRenderer.getVisibleRows",
        flags:"",
        description:" Get the rows that are currently visible in the viewport, as an array of \n [firstRowNum, lastRowNum]. If the grid contains no records, will return [-1,-1];\n",
        name:"getVisibleRows",
        seeAlso:[],
        returns:{description:null, type:"Array"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "classMethod:Timer.clear":{
        ref:"classMethod:Timer.clear",
        flags:"",
        description:"\n Cancels the processing of a timerEvent if it has not already fired.\n\n",
        name:"clear",
        seeAlso:[],
        params:[
            {description:"timerEvent object previously returned from Timer.setTimeout()",
             optional:false, name:"timerEvent", defaultValue:null, type:"object"}
        ],
        type:"classMethod",
        definingClass:"class:Timer"
    },
    "method:Canvas.animateHide":{
        ref:"method:Canvas.animateHide",
        flags:"",
        description:" Hide a canvas by shrinking it vertically to zero height over a period of time.\n This method will not fire if the widget is already drawn and visible, or has overflow\n other than <code>\"visible\"</code> or <code>\"hidden\"</code>.\n",
        name:"animateHide",
        seeAlso:[],
        params:[
            {description:"How should the content of the  window be hidden during the hide? If ommitted, default behavior can be configured via\n  ${isc.DocUtils.linkForRef('attr:Canvas.animateHideEffect')}",
             optional:true, name:"effect", defaultValue:null,
             type:"animateShowEffectId | animateShowEffect"},
            {description:"When the hide completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation.",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated hide.  If unset, duration will be   picked up from ${isc.DocUtils.linkForRef('attr:Canvas.animateHideTime')}",
             optional:true, name:"duration", defaultValue:null, type:"number"},
            {description:"Optional acceleration effect function to bias   the animation ratios.  If unset, acceleration will be picked up from\n   ${isc.DocUtils.linkForRef('attr:Canvas.animateShowTime')}",
             optional:true, name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateWipe')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:Label.autoFit":{
        ref:"attr:Label.autoFit",
        flags:"IRW",
        description:"If true, ignore the specified size of this widget and always size just large\n enough to accomodate the title.  If <code>setWidth()</code> is explicitly called on an\n autoFit:true button, autoFit will be reset to <code>false</code>.\n <P>\n Note that for StretchImgButton instances, autoFit will occur horizontally only, as \n unpredictable vertical sizing is likely to distort the media. If you do want vertical \n auto-fit, this can be achieved by simply setting a small height, and having \n overflow:\"visible\"",
        name:"autoFit",
        setter:["StatefulCanvas.setAutoFit"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Label",
        groups:["sizing"]
    },
    "attr:Menu.showAnimationEffect":{
        ref:"attr:Menu.showAnimationEffect",
        flags:"IRWA",
        description:"When this menu is shown how should it animate into view? By default the menu will just\n show at the specified size/position. Options for animated show effects are <code>\"fade\"</code>\n to fade from transparent to visible, <code>\"slide\"</code> to slide the menu into view,\n or <code>\"wipe\"</code> to have the menu grow into view, revealing its content as it\n grows. Can be overridden by passing the 'animationEffect' parameter to 'menu.show()'",
        name:"showAnimationEffect",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:StatefulCanvas.addToRadioGroup":{
        ref:"method:StatefulCanvas.addToRadioGroup",
        flags:"A",
        description:" Add this widget to the specified mutually exclusive selection group with the ID\n passed in.\n Selecting this widget will then deselect any other StatefulCanvases with the same\n radioGroup ID.\n StatefulCanvases can belong to only one radioGroup, so this method will remove from \n any other radiogroup of which this button is already a member.\n",
        name:"addToRadioGroup",
        seeAlso:[],
        params:[
            {description:"- ID of the radiogroup to which this widget should be added",
             optional:false, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state", "event handling"]
    },
    "method:TextItem.deselectValue":{
        ref:"method:TextItem.deselectValue",
        flags:"",
        description:" If this item currently has focus, clear the current selection. leaving focus in the item.\n Has no effect if the item is undrawn or unfocussed.\n Only applies to text-based items.\n",
        name:"deselectValue",
        seeAlso:[],
        params:[
            {description:"By default the text insertion cursor will be moved to the end of the   current value - pass in this parameter to move to the start instead",
             optional:true, name:"start", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:TextItem"
    },
    "attr:TreeGrid.canAcceptDroppedRecords":{
        ref:"attr:TreeGrid.canAcceptDroppedRecords",
        flags:"IRW",
        description:"Indicates whether records can be dropped into this listGrid.",
        name:"canAcceptDroppedRecords",
        seeAlso:[
            {ref:"attr:TreeNode.canDrag", description:null},
            {ref:"attr:TreeNode.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragReparent')}"],
        definingClass:"class:TreeGrid",
        groups:["dragdrop"]
    },
    "method:ListGrid.selectRecords":{
        ref:"method:ListGrid.selectRecords",
        flags:"",
        description:"\n Select/deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n\n",
        name:"selectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to select", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "class:AutoFitTextAreaItem":{
        methods:[],
        ref:"class:AutoFitTextAreaItem",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"Class for editable multi-line text areas (uses HTML <code>&lt;TEXTAREA&gt;</code> object)\n automatically expands to accomodate its content",
        name:"AutoFitTextAreaItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('textAreaItem')}"]
    },
    "group:clientDataIntegration":{
        ref:"group:clientDataIntegration",
        title:"Client-side Data Integration",
        treeLocation:["Concepts/Client-Server Integration", "Client Reference/Data Binding"],
        refs:["method:DataSourceField.getFieldValue", "attr:DataSource.dataFormat", 
         "attr:DataSource.dataProtocol","attr:DataSource.useHttpProxy", "attr:DataSource.callbackParam", 
         "attr:DataSource.requestProperties","attr:DataSource.dataTransport", "attr:DataSource.dropExtraFields", 
         "attr:DataSource.sendExtraFields","attr:DataSource.serviceNamespace", "attr:DataSource.schemaNamespace", 
         "attr:DataSource.recordXPath","attr:DataSource.dataURL", "attr:DataSource.tagName", "attr:DataSourceField.valueXPath", 
         "attr:OperationBinding.operationType","attr:OperationBinding.operationId", "attr:OperationBinding.wsOperation", 
         "attr:OperationBinding.dataURL","attr:OperationBinding.dataProtocol", "attr:OperationBinding.dataFormat", 
         "attr:OperationBinding.dataTransport","attr:OperationBinding.useHttpProxy", "attr:OperationBinding.callbackParam", 
         "attr:OperationBinding.requestProperties","attr:OperationBinding.defaultParams", "attr:OperationBinding.useFlatFields", 
         "attr:OperationBinding.recordXPath","attr:OperationBinding.recordName", "attr:OperationBinding.spoofResponses", 
         "attr:OperationBinding.xmlNamespaces","attr:OperationBinding.responseDataSchema", "type:DSDataFormat", "type:DSProtocol"],
        description:"SmartClient supports declarative, XPath-based binding of visual components to any server\n capable of returning XML or JSON responses over HTTP, without the need for the\n ${isc.DocUtils.linkForRef('group:serverDataIntegration','SmartClient server')}.\n <P>\n This approach is called Client-Side Data Integration, which means:\n <ul>\n <li> You ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','create DataSources')} in JavaScript \n which describe the data to be loaded and manipulated in the user interface. The\n JavaScript that creates these DataSources may be dynamically generated and/or existing\n metadata may be ${isc.DocUtils.linkForRef('group:metadataImport','imported')}.\n <li> You configure DataSources, via property and method overrides, to send appropriate\n HTTP requests to your server, and to parse HTTP responses from your server, in order to\n fulfill the 4 core operations of the ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource Protocol')}.\n <li> These DataSources are then bound to \n ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databinding-capable UI components')}, which can provide a variety of\n complete user interactions (form-based editing, grid-based editing, load on demand, ..)\n based on these 4 core operations\n </ul>\n <P>\n <h4>Approaches and platforms</h4>\n <P>\n <B>WSDL integration (.NET and others)</B>\n <P>\n If you have pre-existing WSDL services or would like to generate web services for\n SmartClient to consume, the ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL Binding Overview')} covers possible\n approaches.\n <P>\n WSDL binding is the most popular approach for integration with the .NET platform.\n You can \n <a href='http://www.google.com/search?q=create+web+service+visual+studio' onclick=\"window.open('http://www.google.com/search?q=create+web+service+visual+studio');return false;\">use Visual Studio to create web services</a>\n from existing server-side methods, and then use SmartClient's \n ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL Binding')} system to connect to those web services.  \n <P>\n Here are a couple of examples of integration with .NET web services:\n <a href='/examples/databinding/dotNET/temperatureConvert.jsp' onclick=\"window.open('/examples/databinding/dotNET/temperatureConvert.jsp');return false;\">temperature conversion service</a>,\n and <a href='/examples/databinding/dotNET/customerSearch.jsp' onclick=\"window.open('/examples/databinding/dotNET/customerSearch.jsp');return false;\">customer search service</a>.\n Both use the ${isc.DocUtils.linkForRef('method:WebService.callOperation')} method to query the web service. <b>Note:</b>\n These examples will only work if there's a web service running at the WSDL URLs used in the examples.\n <P>\n <B>REST integration (PHP and others)</B>\n <P>\n For PHP and other server technologies (Cold Fusion, Ruby, Python, Perl..), integration\n is based on simple XML or JSON delivered over HTTP, sometimes called the\n <a href='http://www.google.com/search?hl=en&q=REST+HTTP' onclick=\"window.open('http://www.google.com/search?hl=en&q=REST+HTTP');return false;\">REST</a> (REpresentational State\n Transfer) pattern.\n <P> \n When using this, you create a dynamic web page that generates XML or JSON data for\n SmartClient to consume.\n <P>\n SmartClient DataSources can be configured to work with any pre-existing XML or JSON formats\n your application is already using; see below.\n <P>\n For new applications, the ${isc.DocUtils.linkForRef('class:RestDataSource')} provides a complete XML or JSON-based\n protocol that supports all of the features of SmartClient's databinding layer (data paging,\n server-side validation errors, automatic cache synchronization, etc).  To use the\n RestDataSource, simply write server code that can parse RestDataSource requests and produce\n the required responses; example requests and responses are ${isc.DocUtils.linkForRef('class:RestDataSource','provided')}.\n <P>\n <h4>Consuming Existing XML and JSON formats</h4>\n <p>\n For WSDL web services, see the ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL binding topic')} first.\n <P>\n To display XML or JSON data in a visual component such as a ListGrid, you bind the component\n to a ${isc.DocUtils.linkForRef('class:DataSource')} which provides the ${isc.DocUtils.linkForRef('attr:DataSource.dataURL','URL')} of the service, as\n well as a declaration of how to form inputs to the service and how to interpret service\n responses as DataSource records.\n <P>\n An XPath expression, the ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')}, is applied to\n the service response to select the XML elements or JSON objects that should be interpreted\n as DataSource records.  Then, for each field of the DataSource, an optional\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')} can be declared which selects the value for the field\n from within each of the XML elements or JSON objects selected by the recordXPath.  If no\n valueXPath is specified, the field name itself is taken as an XPath, which will select the\n same-named subelement or property from the record element or object.\n <P>\n For example, the following code defines a DataSource that a ListGrid could bind to in order\n to display an RSS 2.0 feed.\n <PRE>\n    isc.DataSource.create({\n        dataURL:feedURL,\n        recordXPath:\"//item\",\n        fields:[\n            { name:\"title\" },\n            { name:\"link\" },\n            { name:\"description\" }\n        ]\n    });\n </PRE>\n A representative slice of an RSS 2.0 feed follows:\n <pre>\n     &lt;?xml version=\"1.0\" encoding=\"iso-8859-1\" ?&gt; \n     &lt;rss version=\"2.0\"&gt;\n     &lt;channel&gt;\n       &lt;title&gt;feed title&lt;/title&gt; \n       ...\n       &lt;item&gt;\n         &lt;title&gt;article title&lt;/title&gt; \n         &lt;link&gt;url of article&lt;/link&gt; \n         &lt;description&gt;\n            article description\n         &lt;/description&gt; \n       &lt;/item&gt;\n       &lt;item&gt;\n          ...\n </pre>\n Here, the recordXPath selects a list of &lt;item&gt; elements.  Since the intended values\n for each DataSource field appear as simple subelements of each &lt;item&gt; element (eg\n &lt;description&gt;), the field name is sufficient to select the correct values, and no\n explicit valueXPath needs to be specified.\n <P>\n A running version of this example is available here: ${isc.DocUtils.linkForExampleId('rssFeed')}.  Further\n examples of simple XML or JSON data loading using files stored on disk as the\n \"service\" to contact: the ${isc.DocUtils.linkForExampleId('simpleJSON', 'Simple JSON')} example shows loading\n data from a JSON file into a databound grid, and the\n ${isc.DocUtils.linkForExampleId('xpathBinding', 'XPath Binding example')} shows loading XML and processing it with\n XPaths.\n <P>\n <H4>Round Tripping: Loading, Editing and Saving</H4>\n <P>\n For WSDL web services, see the ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL binding topic')} first.\n <P>\n When a user triggers a DSRequest (eg, completes an inline edit in a grid), the request\n data will be sent to the dataURL.  The ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource protocol')}\n describes request and response data expected for each operation type.  \n <P>\n By using settings such as ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')}, you can control how\n DSRequests are sent to your backend so that you can handle them most easily.  By using the\n same properties used to initially load data (eg\n ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')}), you can control how SmartClient forms the\n DSResponses that are then interpreted by ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databound components')}.\n <P>\n <b>Controlling how DSRequests are sent</b>\n <P>\n According to the ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','protocol')} being used, the\n ${isc.DocUtils.linkForRef('group:dataSourceOperations','DataSource request data')}, if any, either becomes HTTP\n params (sent by GET or POST), or an XML message as put together by\n ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}.  For a DataSource invoking a WSDL-described web\n service, XML serialization automatically handles namespacing and SOAP encoding.\n <P>\n Note that, by default, just ${isc.DocUtils.linkForRef('attr:DSRequest.data')} is sent, not any of the metadata such\n as ${isc.DocUtils.linkForRef('attr:DSRequest.startRow')}.  This can be customized via\n ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}.\n <P>\n The URL to contact is set via the ${isc.DocUtils.linkForRef('attr:OperationBinding.dataURL','dataURL')}\n property.  If using a Web Service, the <code>dataURL</code> defaults to the service location\n URL embedded in the WSDL file.\n <P>\n For example, in the default configuration for non-WSDL binding, since\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','dataProtocol')} is \"getParams\", ${isc.DocUtils.linkForRef('attr:DSRequest.data')} is\n sent as HTTP params in an HTTP \"GET\" operation.  Given:\n <ul>\n <li> changes to an existing record, hence an \"update\" request\n <li> a ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')} field of \"id\" with value \"5\" on the\n record to be updated\n <li> a field \"age\" being changed to \"32\"\n <li> \"dataURL\" of \"save.php\"\n </ul>\n You will see an HTTP GET to the URL <code>save.php?id=5&age=32</code>.\n <P>\n <b>Forming a DSResponse from the response data</b>\n <P>\n A ${isc.DocUtils.linkForRef('class:DSResponse')} is created from the response data by using XPath\n expressions declared in the schema (${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath','recordXPath')} and \n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath','valueXPath')}) to extract DataSource record and field\n values.\n <P>\n See the ${isc.DocUtils.linkForExampleId('xmlEditSave', '\"Edit and Save\"')} example for sample XML responses for\n all four operationTypes.  \n <P>\n Similar to input processing, by default DataSource layer metadata, such as\n ${isc.DocUtils.linkForRef('attr:DSResponse.startRow')}, is not extracted from the response data.  You can\n implement ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} to fill out the metadata fields of\n the ${isc.DocUtils.linkForRef('class:DSResponse')}, in order to allow more DataSource features, such as paging and\n validation errors, to be used with a web service that supports such features.  \n <P>\n See the ${isc.DocUtils.linkForExampleId('xmlServerValidationErrors', 'XML')} and \n ${isc.DocUtils.linkForExampleId('jsonServerValidationErrors', 'JSON')} versions of the transformResponse()\n example for an example of providing validation errors in XML or JSON responses.",
        name:"clientDataIntegration",
        seeAlso:[],
        type:"group"
    },
    "class:TreeGrid":{
        exampleConfig:"TreeGrid.xml",
        methods:["method:TreeGrid.toggleFolder", "method:TreeGrid.openFolder", 
         "method:TreeGrid.closeFolder","method:TreeGrid.getOpenState", "method:TreeGrid.setOpenState", 
         "method:TreeGrid.getSelectedPaths","method:TreeGrid.setSelectedPaths", "method:TreeGrid.getViewState", 
         "method:TreeGrid.setViewState","method:TreeGrid.setData", "method:TreeGrid.canEditCell", "method:TreeGrid.rowClick", 
         "method:TreeGrid.recordDoubleClick","method:TreeGrid.openLeaf", "method:TreeGrid.transferDragData", 
         "method:TreeGrid.getDragTrackerIcon","method:TreeGrid.willAcceptDrop", "method:TreeGrid.getEventRow", 
         "method:TreeGrid.getDropFolder","method:TreeGrid.transferSelectedData", "method:TreeGrid.recordDrop", 
         "method:TreeGrid.folderDrop","method:TreeGrid.getCellAlign", "method:TreeGrid.getNodeTitle", 
         "method:TreeGrid.getOpenIcon","method:TreeGrid.getExtraIcon", "method:TreeGrid.getIcon", "method:TreeGrid.setNodeIcon", 
         "method:TreeGrid.folderOpened","method:TreeGrid.folderClosed", "method:TreeGrid.folderClick", "method:TreeGrid.leafClick", 
         "method:TreeGrid.nodeClick","method:TreeGrid.folderContextClick", "method:TreeGrid.leafContextClick", 
         "method:TreeGrid.nodeContextClick","method:TreeGrid.dataArrived", "method:TreeGrid.fetchData", "method:TreeGrid.filterData"],
        type:"class",
        ref:"class:TreeGrid",
        classAttrs:[],
        treeLocation:["Client Reference/Grids"],
        classMethods:[],
        description:"The SmartClient system supports hierarchical data (also referred to as tree data\n due to its \"branching\" organization) with:\n <ul>\n   <li> the ${isc.DocUtils.linkForRef('class:Tree')} class, which manipulates hierarchical data sets\n   <li> the TreeGrid widget class, which extends the ListGrid class to visually\n        present tree data in an expandable/collapsible format.\n </ul>\n For information on DataBinding Trees, see ${isc.DocUtils.linkForRef('group:treeDataBinding')}.\n <p>\n A TreeGrid works just like a ${isc.DocUtils.linkForRef('class:ListGrid')}, except one column (specified by\n ${isc.DocUtils.linkForRef('attr:TreeGridField.treeField')} shows a heirarchical ${isc.DocUtils.linkForRef('class:Tree')}.  A TreeGrid is not limited\n to displaying just the ${isc.DocUtils.linkForRef('class:Tree')} column - you can define additional columns (via\n ${isc.DocUtils.linkForRef('attr:TreeGrid.fields')}) which will render just like the columns of a ${isc.DocUtils.linkForRef('class:ListGrid')}, and\n support all of the functionality of ListGrid columns, such as\n ${isc.DocUtils.linkForRef('method:ListGridField.formatCellValue','formatters')}.\n <p>\n Except where explicitly overridden, ${isc.DocUtils.linkForRef('class:ListGrid')} methods, callbacks, and properties\n apply to TreeGrids as well.  The ${isc.DocUtils.linkForRef('class:ListGrid')} defines some methods as taking/returning\n ${isc.DocUtils.linkForRef('object:ListGridField')} and ${isc.DocUtils.linkForRef('object:ListGridRecord')}.  When using those methods in a TreeGrid,\n those types will be ${isc.DocUtils.linkForRef('object:TreeGridField')} and ${isc.DocUtils.linkForRef('object:TreeNode')}, respectively.",
        attrs:["attr:TreeGrid.animateFolders", "attr:TreeGrid.animateFolderTime", 
         "attr:TreeGrid.animateFolderSpeed","attr:TreeGrid.animateFolderEffect", "attr:TreeGrid.dataSource", "attr:TreeGrid.data", 
         "attr:TreeGrid.initialData","attr:TreeGrid.loadDataOnDemand", "attr:TreeGrid.autoFetchTextMatchStyle", 
         "attr:TreeGrid.treeRootValue","attr:TreeGrid.fields", "attr:TreeGrid.treeFieldTitle", "attr:TreeGrid.showRoot", 
         "attr:TreeGrid.separateFolders","attr:TreeGrid.displayNodeType", "attr:TreeGrid.canDragRecordsOut", 
         "attr:TreeGrid.canAcceptDroppedRecords","attr:TreeGrid.canReorderRecords", "attr:TreeGrid.canDropOnLeaves", 
         "attr:TreeGrid.canReparentNodes","attr:TreeGrid.dragDataAction", "attr:TreeGrid.parentAlreadyContainsChildMessage", 
         "attr:TreeGrid.cantDragIntoSelfMessage","attr:TreeGrid.cantDragIntoChildMessage", "attr:TreeGrid.indentSize", 
         "attr:TreeGrid.iconSize","attr:TreeGrid.openerIconSize", "attr:TreeGrid.folderIcon", "attr:TreeGrid.dropIconSuffix", 
         "attr:TreeGrid.openIconSuffix","attr:TreeGrid.closedIconSuffix", "attr:TreeGrid.nodeIcon", "attr:TreeGrid.showOpenIcons", 
         "attr:TreeGrid.showDropIcons","attr:TreeGrid.customIconProperty", "attr:TreeGrid.customIconOpenProperty", 
         "attr:TreeGrid.customIconDropProperty","attr:TreeGrid.showCustomIconOpen", "attr:TreeGrid.showCustomIconDrop", 
         "attr:TreeGrid.folderOpenImage","attr:TreeGrid.folderClosedImage", "attr:TreeGrid.folderDropImage", 
         "attr:TreeGrid.fileImage","attr:TreeGrid.manyItemsImage", "attr:TreeGrid.showConnectors", 
         "attr:TreeGrid.showFullConnectors","attr:TreeGrid.openerImage", "attr:TreeGrid.connectorImage"],
        implementsInterfaces:["DataBoundComponent"],
        name:"TreeGrid",
        seeAlso:[]
    },
    "attr:Calendar.previousButton":{
        ref:"attr:Calendar.previousButton",
        flags:"IR",
        description:"The previousButton is an ImgButton that appears above the week/day/month views of the\n calendar and allows the user to move the calendar backwards in time.",
        name:"previousButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:ListGridField.icon":{
        ref:"attr:ListGridField.icon",
        flags:"IR",
        description:"Optional icon to show next to the title for this field. \n Should be set to a URL to an image. Relative paths will be evaluated starting at\n the imgDir of this component. This URL is partial - it may be updated to indicate\n the current disabled (etc) state of the field.\n <P>\n If ${isc.DocUtils.linkForRef('attr:ListGridField.type','field.type')} is set to \"icon\", this icon will also be shown\n in every cell of this field - see also ${isc.DocUtils.linkForRef('attr:ListGridField.cellIcon','field.cellIcon')}.",
        name:"icon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:FusionChart.fusionVersion":{
        ref:"attr:FusionChart.fusionVersion",
        flags:"IR",
        description:"Version of FusionCharts to assume.  If version is \"2.3\", different names are used for\n the .swf files for each chart (to match the default names in FusionCharts 2.3), and a\n lower version of Flash is required (6.0).",
        name:"fusionVersion",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"3.0\"",
        type:"attr",
        definingClass:"class:FusionChart"
    },
    "attr:Window.showMinimizeButton":{
        ref:"attr:Window.showMinimizeButton",
        flags:"IRW",
        description:"If true, show a minimize button in the header--clicking it minimizes the Window.",
        name:"showMinimizeButton",
        setter:["setShowMinimizeButton"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "method:ListGrid.getSortSpecifier":{
        ref:"method:ListGrid.getSortSpecifier",
        flags:"A",
        description:"  Returns the ${isc.DocUtils.linkForRef('object:SortSpecifier')} for the passed fieldName, or null if the field is not \n sorted.\n",
        name:"getSortSpecifier",
        seeAlso:[],
        params:[
            {description:"The name of a field, visible, hidden or existing only in the dataSource",
             optional:false, name:"fieldName", defaultValue:null, type:"String"}
        ],
        returns:{description:"True if the passed field is sorted, false otherwise", type:"Boolean"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "group:velocitySupport":{
        ref:"group:velocitySupport",
        title:"Velocity context variables",
        refs:["type:VelocityExpression"],
        description:"The SmartClient Server provides a number of standard context variables for use in the \n Velocity templates you write to implement ${isc.DocUtils.linkForRef('group:customQuerying','custom queries')}, \n ${isc.DocUtils.linkForRef('attr:DSRequestModifier.value','transaction chaining')},  \n ${isc.DocUtils.linkForRef('attr:OperationBinding.requires','dynamic security checking')} and \n ${isc.DocUtils.linkForRef('class:Mail','templated mail messages')}.  These are:\n <ul>\n <li><b>$currentDate</b>. The current date/time with millisecond precision</li>\n <li><b>$transactionDate</b>. The date/time that this transaction started, with millisecond \n precision.  If you are not using ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','queuing')}, this value will be\n identical to <b>$currentDate</b></li>\n <li><b>$servletRequest</b>. The associated <code>HttpServletRequest</code></li> \n <li><b>$session</b>. The associated <code>HttpSession</code></li>\n <li><b>$httpParameters</b>. This variable gives you access to the parameters Map of the \n associated <code>HttpServletRequest</code>; it is an alternate form of \n <code>$servletRequest.getParameter</code></li>\n <li><b>$requestAttributes</b>. This variable gives you access to the attributes Map of the \n associated <code>HttpServletRequest</code>; it is an alternate form of \n <code>$servletRequest.getAttribute</code></li>\n <li><b>$sessionAttributes</b>. This variable gives you access to the attributes Map of the \n associated <code>HttpSession</code>; it is an alternate form of \n <code>$session.getAttribute</code></li>\n <li><b>$dataSources</b>. This variable gives you access to SmartClient ${isc.DocUtils.linkForRef('class:DataSources')}s.  \n You access a dataSource by suffixing its name to the <code>$dataSources</code> designation.\n For example, <code>$dataSources.supplyItem</code> refers to the DataSource object called \n \"supplyItem\".  You can use this approach to execute any valid DataSource method. One especially\n useful method in this context is <code>hasRecord(fieldName, value)</code> - see the \n server-side Javadocs for more details.\n <li><b>$util</b> - A <code>DataTools</code> object, giving you access to all of that\n     class's useful helper functions</li>\n </ul>\n All of these variables (other than the two dates) represent objects that can contain other \n objects (attributes or parameters).  They all implement the <code>Map</code> interface, so\n you can use the Velocity \"property\" shorthand notation to access them.  The following usage\n examples show five different ways to return the value of the session attribute named \"foo\":\n <pre>\n    $session.foo\n    $session.get(\"foo\")\n    $session.getAttribute(\"foo\")\n    $sessionAttributes.foo\n    $sessionAttributes.get(\"foo\")\n </pre>\n In the case of <code>$servletRequest</code>, the shorthand approach accesses the attributes \n - you need to use either <code>$httpParameters</code> or <code>$servletRequest.getParameter</code>\n to access parameters. These examples all return the value of the HTTP parameter named \"bar\":\n <pre>\n    $httpParameters.bar\n    $httpParameters.get(\"bar\")\n    $servletRequest.getParameter(\"bar\")\n </pre>\n When you use these Velocity variables in a ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL','customSQL')} \n clause or SQL snippet such as a ${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','whereClause')}, all of \n these template variables return values that have been correctly quoted and escaped according\n  to the syntax of the underlying database.  We do this because \"raw\" values are vulnerable to \n <a href=\"http://en.wikipedia.org/wiki/SQL_injection\">SQL injection attacks</a>.\n If you need access to the raw value of a variable in a SQL template, you can use the \n <b>$rawValue</b> qualifier in front of any of the template variables, like this:<br><br><code>\n &nbsp;&nbsp;$rawValue.session.foo</code>\n <p>\n This also works for the <b>$criteria</b> and <b>$values</b> context variables (see \n ${isc.DocUtils.linkForRef('group:customQuerying')} for details of these variables).  So:<br><br><code>\n &nbsp;&nbsp;$rawValue.criteria.customerName</code>\n </p>\n Note that <code>$rawValue</code> is only available in SQL templates.  It is not needed in\n other contexts, such as Transaction Chaining, because the value is not escaped and \n quoted in these contexts.\n <p>\n <b>Warning</b>:  Whenever you access a template variable for use in a SQL statement, bear \n in mind that it is <b>dangerous</b> to use <code>$rawValue</code>.  There are some cases \n where using the raw value is necessary, but even so, all such cases are likely to be vulnerable \n to injection attacks.  Generally, the presence of <code>$rawValue</code> in a SQL template \n should be viewed as a red flag.\n <p>\n Finally, some example usages of these values.  \n These ${isc.DocUtils.linkForRef('attr:OperationBinding.values','values')} clauses set \"price\" to a value extracted from the \n session, and \"lastUpdated\" to the date/time that this transaction started:<br><br><code>\n &nbsp;&nbsp;&lt;values fieldName=\"price\" value=\"$session.somePrice\" /&gt;<br>\n &nbsp;&nbsp;&lt;values fieldName=\"lastUpdated\" value=\"$transactionDate\" /&gt;\n </code><p>\n This whereClause selects some users based on various values passed in the criteria and \n as HTTP parameters:<br><br><code>\n &nbsp;&nbsp;&lt;whereClause&gt;department = $httpParameters.userDept AND dob >= $criteria.dateOfBirth&lt;/whereClause&gt;\n </code><p>\n This whereClause selects some users based on various values obtained from the \n servletRequest's attributes, using a number of equivalent techniques for accessing the \n attributes:<pre>\n   &lt;whereClause&gt;\n         department = $servletRequest.dept \n     AND startDate >= $requestAttributes.dateOfBirth \n     AND salary < $servletRequest.getAttribute(\"userSalary\")\n   &lt;/whereClause&gt;\n </pre><p>",
        name:"velocitySupport",
        seeAlso:[],
        type:"group"
    },
    "classMethod:DataSource.registerRecordSummaryFunction":{
        ref:"classMethod:DataSource.registerRecordSummaryFunction",
        flags:"",
        description:" Register a new standard ${isc.DocUtils.linkForRef('type:RecordSummaryFunction')}. This will then be available\n by calling ${isc.DocUtils.linkForRef('DataSource.applySummaryFunction')} and passing in just the new method name.\n",
        name:"registerRecordSummaryFunction",
        seeAlso:[],
        params:[
            {description:"identifier for the new summary function", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"new summary function implementation.  This method should take 3 parameters: <code>record</code> (the record for which the\n  summary is being generated), <code>fields</code> (an array of fields to include in the\n  generated summary) and <code>summaryField</code> (a pointer to the field in which the\n  summary will be displayed [may be null].",
             optional:false, name:"summaryFunction", defaultValue:null, type:"function | stringMethod"}
        ],
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:CubeGrid.wrapFacetValueTitles":{
        ref:"attr:CubeGrid.wrapFacetValueTitles",
        flags:"IRW",
        description:"Whether to allow text wrapping on facet value titles",
        name:"wrapFacetValueTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:SliderItem.numValues":{
        ref:"attr:SliderItem.numValues",
        flags:"IR",
        description:"The number of discrete values represented by slider. If specified, the range of valid\n values (between <code>minValue</code> and <code>maxValue</code>) will be divided into\n this many steps. As the thumb is moved along the track it will only select these values\n and appear to jump between the steps.",
        name:"numValues",
        setter:["setNumValues"],
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"],
        definingClass:"class:SliderItem"
    },
    "attr:ButtonItem.endRow":{
        ref:"attr:ButtonItem.endRow",
        flags:"IRW",
        description:"These items are in a row by themselves by default",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ButtonItem",
        groups:["formLayout"]
    },
    "method:ListGrid.stopHover":{
        ref:"method:ListGrid.stopHover",
        flags:"",
        description:" Notification that the user is no longer hovering over some cell. Hides the current hover canvas\n if one is showing. \n",
        name:"stopHover",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ToolbarItem.startRow":{
        ref:"attr:ToolbarItem.startRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["appearance"]
    },
    "class:TabSet":{
        methods:["method:TabSet.setPaneContainerOverflow", "method:TabSet.getPaneContainerEdges", 
         "method:TabSet.setTabTitle","method:TabSet.setTabIcon", "method:TabSet.enableTab", "method:TabSet.disableTab", 
         "method:TabSet.setTabProperties","method:TabSet.addTab", "method:TabSet.addTabs", "method:TabSet.setTabPane", 
         "method:TabSet.removeTab","method:TabSet.removeTabs", "method:TabSet.closeClick", "method:TabSet.tabIconClick", 
         "method:TabSet.getTabObject","method:TabSet.getTab", "method:TabSet.getTabPane", "method:TabSet.getTabNumber", 
         "method:TabSet.updateTab","method:TabSet.scrollForward", "method:TabSet.scrollBack", "method:TabSet.getSelectedTab", 
         "method:TabSet.getSelectedTabNumber","method:TabSet.selectTab", "method:TabSet.tabSelected", "method:TabSet.tabDeselected"],
        ref:"class:TabSet",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:TabSet.tabs", "attr:TabSet.tabBar", "attr:TabSet.tabProperties", 
         "attr:TabSet.tabBarPosition","attr:TabSet.tabBarAlign", "attr:TabSet.tabBarThickness", "attr:TabSet.selectedTab", 
         "attr:TabSet.canCloseTabs","attr:TabSet.closeTabIcon", "attr:TabSet.closeTabIconSize", "attr:TabSet.tabBarControls", 
         "attr:TabSet.showTabScroller","attr:TabSet.showTabPicker", "attr:TabSet.animateTabScrolling", 
         "attr:TabSet.scrollerButtonSize","attr:TabSet.pickerButtonSize", "attr:TabSet.skinImgDir", "attr:TabSet.symmetricScroller", 
         "attr:TabSet.scrollerSrc","attr:TabSet.scrollerHSrc", "attr:TabSet.scrollerVSrc", 
         "attr:TabSet.symmetricPickerButton","attr:TabSet.pickerButtonSrc", "attr:TabSet.pickerButtonHSrc", 
         "attr:TabSet.pickerButtonVSrc","attr:TabSet.paneContainer", "attr:TabSet.paneContainerClassName", 
         "attr:TabSet.paneContainerOverflow","attr:TabSet.symmetricEdges", "attr:TabSet.leftEdgeSizes", "attr:TabSet.topEdgeSizes", 
         "attr:TabSet.bottomEdgeSizes","attr:TabSet.rightEdgeSizes", "attr:TabSet.leftEdgeOffsets", 
         "attr:TabSet.rightEdgeOffsets","attr:TabSet.topEdgeOffsets", "attr:TabSet.bottomEdgeOffsets", 
         "attr:TabSet.showPaneContainerEdges","attr:TabSet.showPartialEdges", "attr:TabSet.destroyPanes"],
        description:"The TabSet class allows components on several panes to share the same space. The tabs at \n the top can be selected by the user to show each pane. \n <P>\n Tabs are configured via the <code>tabs</code> property, each of which has a\n <code>pane</code> property which will be displayed in the main pane when that tab is\n selected.",
        name:"TabSet",
        seeAlso:[],
        type:"class"
    },
    "attr:DSResponse.errors":{
        ref:"attr:DSResponse.errors",
        flags:"R",
        description:"Server-side validation errors for an attempted \"update\" or \"add\" operation, as a JS Object\n where each property name is a field name from the record and each property value is an error\n message to be shown to the user.  For example:<pre>\n     dsResponse.errors = {\n         userId : \"A user with this userId already exists\",\n         orderId : \"No Order with ID '6A18294' exists\"\n     }\n </pre>\n The Java API DSResponse.addError(fieldName, errorMessage) is used to send server-side errors\n to the client.  See the Java Server Reference for details.",
        name:"errors",
        seeAlso:[
            {ref:"method:DataSource.handleError", description:null}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["errorHandling"]
    },
    "attr:DynamicForm.showErrorStyle":{
        ref:"attr:DynamicForm.showErrorStyle",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorStyle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:ImgButton.showRollOver":{
        ref:"attr:ImgButton.showRollOver",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes over this object?",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:FormItem.defaultValue":{
        ref:"attr:FormItem.defaultValue",
        flags:"IRW",
        description:"Value used when no value is provided for this item. Note that whenever this item's value\n is cleared programatically (for example via <code>item.setValue(null)</code>), it will be\n reverted to the <code>defaultValue</code>. Developers should use the\n ${isc.DocUtils.linkForRef('attr:DynamicForm.values')} object if their intention is to provide an initial value for a\n field in a form rather than a value to use in place of <code>null</code>.",
        name:"defaultValue",
        seeAlso:[
            {ref:"method:FormItem.defaultDynamicValue", description:null}
        ],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fieldEnableDisable')}"],
        definingClass:"class:FormItem",
        groups:["basics"]
    },
    "attr:Slider.roundPrecision":{
        ref:"attr:Slider.roundPrecision",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:Slider.roundValues')} is false, the slider value will be rounded to this number of\n decimal places. If set to null the value will not be rounded",
        name:"roundPrecision",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:Slider"
    },
    "class:ImgButton":{
        methods:["method:ImgButton.getTitle", "method:ImgButton.setTitle", 
         "method:ImgButton.setIconOrientation","method:ImgButton.setIcon", "method:ImgButton.setAutoFit", "method:ImgButton.setBaseStyle", 
         "method:ImgButton.select","method:ImgButton.deselect", "method:ImgButton.isSelected", "method:ImgButton.setSelected", 
         "method:ImgButton.addToRadioGroup","method:ImgButton.removeFromRadioGroup", "method:ImgButton.setActionType", 
         "method:ImgButton.getActionType","method:ImgButton.setState", "method:ImgButton.setDisabled", "method:ImgButton.getState", 
         "method:ImgButton.action"],
        ref:"class:ImgButton",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:ImgButton.title", "attr:ImgButton.hiliteAccessKey", "attr:ImgButton.icon", 
         "attr:ImgButton.iconSize","attr:ImgButton.iconWidth", "attr:ImgButton.iconHeight", "attr:ImgButton.iconOrientation", 
         "attr:ImgButton.iconAlign","attr:ImgButton.iconSpacing", "attr:ImgButton.showDisabledIcon", 
         "attr:ImgButton.showRollOverIcon","attr:ImgButton.showFocusedIcon", "attr:ImgButton.showDownIcon", 
         "attr:ImgButton.showSelectedIcon","attr:ImgButton.autoFit", "attr:ImgButton.baseStyle", "attr:ImgButton.selected", 
         "attr:ImgButton.radioGroup","attr:ImgButton.actionType", "attr:ImgButton.state", "attr:ImgButton.showDisabled", 
         "attr:ImgButton.showDown","attr:ImgButton.showFocus", "attr:ImgButton.showFocused", "attr:ImgButton.showRollOver", 
         "attr:ImgButton.align","attr:ImgButton.valign", "attr:ImgButton.showTitle", "attr:ImgButton.labelHPad", 
         "attr:ImgButton.labelVPad","attr:ImgButton.src"],
        description:"A Img that behaves like a button, going through up/down/over state transitions in response to\n user events.  Supports an optional title, and will auto-size to accomodate the title text if\n <code>overflow</code> is set to \"visible\".\n <P>\n An examples of use is Window minimize/close buttons.",
        name:"ImgButton",
        seeAlso:[],
        type:"class"
    },
    "method:ListGrid.editorExit":{
        ref:"method:ListGrid.editorExit",
        flags:"",
        description:" Callback fired when the user attempts to navigate away from the current edit cell, \n or complete the current edit.\n <P>\n Return false from this method to cancel the default behavior (Saving / cancelling the\n current edit / moving to the next edit cell).\n <P>\n This callback is typically used to dynamically update values or value maps for related\n fields (via ${isc.DocUtils.linkForRef('method:ListGrid.setEditValue')} and ${isc.DocUtils.linkForRef('method:ListGrid.setEditorValueMap')} respectively, or to\n implement custom navigation (via ${isc.DocUtils.linkForRef('method:ListGrid.startEditing','startEditing(rowNum,colNum)')}.\n <P>\n Can be overriden at the field level as field.editorExit.\n\n",
        name:"editorExit",
        seeAlso:[
            {ref:"method:ListGridField.editorExit", description:null}
        ],
        params:[
            {description:"How was the edit completion fired?", optional:false,
             name:"editCompletionEvent", defaultValue:null, type:"EditCompletionEvent"},
            {description:"record for the cell being edited", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"new value for the cell being edited", optional:false, name:"newValue",
             defaultValue:null, type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"Returning false from this method will cancel the default behavior                      (for example saving the row) and leave the editor visible and focus\n                      in this edit cell.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:dragndrop":{
        ref:"group:dragndrop",
        refs:["attr:Toolbar.canReorderItems", "attr:Toolbar.canResizeItems", 
         "attr:Toolbar.canAcceptDrop"],
        name:"dragndrop",
        seeAlso:[],
        type:"group"
    },
    "classAttr:GroupingMessages.byHoursTitle":{
        ref:"classAttr:GroupingMessages.byHoursTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a time field by hours.",
        name:"byHoursTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Hours\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "class:DateItem":{
        methods:["method:DateItem.setSelectionRange", "method:DateItem.getSelectionRange", 
         "method:DateItem.selectValue","method:DateItem.deselectValue", "method:DateItem.getInputFormat"],
        exampleConfig:"DateItem.xml",
        ref:"class:DateItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:DateItem.pickerConstructor", "attr:DateItem.useTextField", "attr:DateItem.textAlign", 
         "attr:DateItem.useMask","attr:DateItem.maskDateSeparator", "attr:DateItem.enforceDate", 
         "attr:DateItem.invalidDateStringMessage","attr:DateItem.pickerIconPrompt", "attr:DateItem.startDate", "attr:DateItem.endDate", 
         "attr:DateItem.centuryThreshold","attr:DateItem.dateFormatter", "attr:DateItem.displayFormat", "attr:DateItem.inputFormat", 
         "attr:DateItem.selectorFormat"],
        description:"Item for manipulating Dates.\n <p>\n Can be rendered as a text field, or as 3 selects for day, month, year.  Includes optional\n pop-up picker.",
        name:"DateItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('dateItem')}"]
    },
    "attr:CubeGrid.chartConstructor":{
        ref:"attr:CubeGrid.chartConstructor",
        flags:"IR",
        description:"Name of the SmartClient Class to be used when creating charts.  Must support the\n ${isc.DocUtils.linkForRef('interface:Chart')} interface.",
        name:"chartConstructor",
        seeAlso:[],
        valueType:"Classname",
        defaultValue:"\"FusionChart\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:ListGrid.reorderField":{
        ref:"method:ListGrid.reorderField",
        flags:"A",
        description:" Reorder a particular field \n\n",
        name:"reorderField",
        seeAlso:[],
        params:[
            {description:"Number of the field to reorder", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"},
            {description:"New position for that field", optional:false, name:"moveToPosition",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:ListGrid.setShowFilterEditor":{
        ref:"method:ListGrid.setShowFilterEditor",
        flags:"",
        description:" Setter for the ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')} property. Allows the filter editor to be\n shown or hidden at runtime.\n",
        name:"setShowFilterEditor",
        seeAlso:[],
        params:[
            {description:"true if the filter editor should be shown, false if it should be hidden",
             optional:false, name:"value", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "attr:animationShowEffect.endsAt":{
        ref:"attr:animationShowEffect.endsAt",
        flags:"IR",
        description:"For hide animations of type <code>\"wipe</code> and \n   <code>\"slide\"</code> this attribute specifies where the wipe / slide should finish.\n   Valid options are <code>\"T\"</code> (vertical animation upwards to the top of the canvas,\n   the default behavior) and <code>\"L\"</code> (horizontal animation to the left of the\n   canvas).",
        name:"endsAt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:animationShowEffect"
    },
    "attr:Canvas.minWidth":{
        ref:"attr:Canvas.minWidth",
        flags:"IRWA",
        description:"Minimum width that this Canvas can be resized to.\n <P>\n Note that a Canvas with overflow:\"visible\" has an implicit minimize size based on it's\n contents.",
        name:"minWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Class.isA":{
        ref:"method:Class.isA",
        flags:"",
        description:"\n	Returns whether this object is of a particular class by class name, either as a direct\n	instance of that class or as subclass of that class, or by implementing an interface\n  that has been mixed into the class.<br><br>\n\n	NOTE: this only applies to ISC's class system, eg:  <code>myInstance.isA(\"Object\")</code> will be\n	false.\n\n",
        name:"isA",
        seeAlso:[],
        params:[
            {description:"Class name to test against", optional:false, name:"className",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"whether this object is of that Class                                   or a subClass of that Class",
         type:"boolean"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:CubeGrid.baseStyle":{
        ref:"attr:CubeGrid.baseStyle",
        flags:"IR",
        getter:["getBaseStyle"],
        description:"The base name for the CSS class applied to cells. This style will have \"Dark\", \n \"Over\", \"Selected\", or \"Disabled\" appended to it according to the state of the cell.",
        name:"baseStyle",
        seeAlso:[
            {ref:"method:CubeGrid.getCellStyle", description:null},
            {ref:"method:CubeGrid.getBaseStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"cubeCell\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["cellStyling"]
    },
    "attr:Canvas.resizeBarTarget":{
        ref:"attr:Canvas.resizeBarTarget",
        flags:"IR",
        description:"When this Canvas is included as a member in a Layout, and ${isc.DocUtils.linkForRef('attr:Canvas.showResizeBar')} is set to\n <code>true</code> so that a resizeBar is created, <code>resizeBarTarget:\"next\"</code> can be\n set to indicate that the resizeBar should resize the next member of the layout rather than\n this one.  For resizeBars that support hiding their target member when clicked on, \n <code>resizeBarTarget:\"next\"</code> also means that the next member will be the one hidden.\n <P>\n This is typically used to create a 3-way split pane, where left and right-hand sections can\n be resized or hidden to allow a center section to expand.\n <P>\n <b>NOTE:</b> as with any Layout, to ensure all available space is used, one or more members\n must maintain a flexible size (eg 75%, or *).  In a two pane Layout with a normal resize\n bar, to fill all space after a user resizes, the member on the <b>right</b> should have\n flexible size.  With resizeBarTarget:\"next\", the member on the <b>left</b> should have\n flexible size.",
        name:"resizeBarTarget",
        seeAlso:[
            {ref:"attr:Canvas.showResizeBar", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["layoutMember"]
    },
    "method:ListGrid.rowClick":{
        ref:"method:ListGrid.rowClick",
        flags:"A",
        description:"\n Event handler for when rows in the body are clicked upon. The default implementation handles\n firing ${isc.DocUtils.linkForRef('method:ListGrid.startEditing')} if appropriate, and fires\n ${isc.DocUtils.linkForRef('method:ListGridField.recordClick')} and/or ${isc.DocUtils.linkForRef('method:ListGrid.recordClick')} if set. Developers\n should typically implement recordClick rather than overriding this method.\n\n",
        name:"rowClick",
        seeAlso:[
            {ref:"method:ListGrid.recordClick", description:null}
        ],
        params:[
            {description:"record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"index of the row where the click occurred", optional:false, name:"recordNum",
             defaultValue:null, type:"number"},
            {description:"index of the col where the click occurred", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"},
            {description:"indicates whether this was a synthesized record                                              click in response to a keyboard event",
             optional:true, name:"keyboardGenerated", defaultValue:null, type:"boolean"}
        ],
        returns:{description:null, type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events", "events"]
    },
    "class:Time":{
        methods:[],
        ref:"class:Time",
        classAttrs:["classAttr:Time.UTCHoursOffset", "classAttr:Time.displayFormat", 
         "classAttr:Time.shortDisplayFormat","classAttr:Time.AMIndicator", "classAttr:Time.PMIndicator"],
        classMethods:["classMethod:Time.setDefaultDisplayTimezone", "classMethod:Time.getDefaultDisplayTimezone", 
         "classMethod:Time.toTime","classMethod:Time.toShortTime", "classMethod:Time.parseInput", 
         "classMethod:Time.createDate","classMethod:Time.setShortDisplayFormat", "classMethod:Time.setNormalDisplayFormat", 
         "classMethod:Time.compareTimes"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Helper methods and system-wide defaults for dealing with time values and time display formats.\n Note that Times are stored as JavaScript date objects on the client, though only the\n time information is typically displayed to the user. See \n ${isc.DocUtils.linkForRef('group:dateFormatAndStorage','Date format and storage')} for more information on working with\n <code>\"time\"</code> type fields.",
        name:"Time",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.summaryRowFetchRequestProperties":{
        ref:"attr:ListGrid.summaryRowFetchRequestProperties",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, and a ${isc.DocUtils.linkForRef('listGrid.summaryDataSource')} is specified\n this property may be used to customize the fetch request used when retrieving summary data\n to show in the summary row. An example use case might be specifying a \n ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} to perform a custom fetch operation which retrieved only summary\n values based on criteria.",
        name:"summaryRowFetchRequestProperties",
        seeAlso:[],
        valueType:"DSRequest properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ResultSet.disableCacheSync":{
        ref:"attr:ResultSet.disableCacheSync",
        flags:"IRA",
        description:"By default when the data of this ResultSet's dataSource is modified, the ResultSet will\n be updated to display these changes.\n Set this flag to true to disable this behavior.",
        name:"disableCacheSync",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["cacheSync"]
    },
    "method:DynamicForm.validate":{
        ref:"method:DynamicForm.validate",
        flags:"",
        description:" Validates the form without submitting it, and redraws the form to display error messages\n if there are any validation errors. Returns true if validation succeeds, or false if\n validation fails.<br>\n For databound forms, any Datasource field validators will be run even if there is no \n associated item in the form.<br>\n Validators will also be run on hidden form items<br>\n In both these cases, validation failure can be handled via \n ${isc.DocUtils.linkForRef('method:DynamicForm.handleHiddenValidationErrors')}\n\n",
        name:"validate",
        seeAlso:[],
        params:[
            {description:"Should validators be processed for non-visible fields         such as dataSource fields with no associated item or fields with visibility set to\n         <code>\"hidden\"</code>?",
             optional:false, name:"validateHiddenFields", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if validation succeeds, or false if validation fails.", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('validationType')}"],
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "classAttr:Validator.mustBeExactLength":{
        ref:"classAttr:Validator.mustBeExactLength",
        flags:"IRA",
        description:"Default error message to display when standard <code>lengthRange</code> type validator\n has <code>validator.max</code> and <code>validator.min</code> set to the same value,\n and returns false because the value passed is not the same length as these limits.<br>\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeExactLength",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be exactly ${max} characters\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:SummaryBuilder.dataSource":{
        ref:"attr:SummaryBuilder.dataSource",
        flags:"IRW",
        description:"DataSource providing the available fields for the formulaBuilder.\n <P>\n By default the formulaBuilder will include <b>only</b> fields of numeric type or derived\n from a numeric type.  Set ${isc.DocUtils.linkForRef('attr:FormulaBuilder.fields')} to override this.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "class:Toolbar":{
        methods:["method:Toolbar.setButtons", "method:Toolbar.addButtons", "method:Toolbar.removeButtons", 
         "method:Toolbar.getButton","method:Toolbar.getButtonNumber", "method:Toolbar.setCanResizeItems", 
         "method:Toolbar.selectButton","method:Toolbar.deselectButton", "method:Toolbar.itemClick", 
         "method:Toolbar.itemDragResized"],
        ref:"class:Toolbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:Toolbar.buttons", "attr:Toolbar.vertical", "attr:Toolbar.buttonConstructor", 
         "attr:Toolbar.canReorderItems","attr:Toolbar.canResizeItems", "attr:Toolbar.canAcceptDrop", "attr:Toolbar.buttonDefaults"],
        description:"A Toolbar creates a vertical or horizontal strip of similar components (typically Buttons)\n and provides managed resizing and reordering behavior over those components.\n <p>\n If you are creating a bar with a mixture of different elements (eg some MenuButtons, some\n Labels, some Buttons, some custom components), you want to use a ${isc.DocUtils.linkForRef('class:ToolStrip')}.  A\n Toolbar is better suited for managing a set of highly similar, interchangeable components,\n such as ListGrid headers.",
        name:"Toolbar",
        seeAlso:[],
        type:"class"
    },
    "interface:DataBoundComponent":{
        methods:["method:DataBoundComponent.dragComplete", "method:DataBoundComponent.dropComplete", 
         "method:DataBoundComponent.getField","method:DataBoundComponent.getFieldNum", "method:DataBoundComponent.setDataSource", 
         "method:DataBoundComponent.findByKey","method:DataBoundComponent.selectRecord", "method:DataBoundComponent.deselectRecord", 
         "method:DataBoundComponent.selectRecords","method:DataBoundComponent.deselectRecords", "method:DataBoundComponent.selectAllRecords", 
         "method:DataBoundComponent.deselectAllRecords","method:DataBoundComponent.anySelected", "method:DataBoundComponent.enableHilite", 
         "method:DataBoundComponent.disableHilite","method:DataBoundComponent.enableHiliting", "method:DataBoundComponent.disableHiliting", 
         "method:DataBoundComponent.transferRecords","method:DataBoundComponent.getDropValues", "method:DataBoundComponent.transferDragData", 
         "method:DataBoundComponent.getDragData","method:DataBoundComponent.transferSelectedData", 
         "method:DataBoundComponent.setDragTracker","method:DataBoundComponent.addFormulaField", "method:DataBoundComponent.editFormulaField", 
         "method:DataBoundComponent.getFormulaFieldValue","method:DataBoundComponent.addSummaryField", "method:DataBoundComponent.editSummaryField", 
         "method:DataBoundComponent.getSummaryFieldValue","method:DataBoundComponent.getRecordIndex", "method:DataBoundComponent.getTitleFieldValue", 
         "method:DataBoundComponent.getTitleField","method:DataBoundComponent.getSort", "method:DataBoundComponent.setSort"],
        ref:"interface:DataBoundComponent",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:DataBoundComponent.dataSource", "attr:DataBoundComponent.dataFetchMode", 
         "attr:DataBoundComponent.dataPageSize","attr:DataBoundComponent.fields", "attr:DataBoundComponent.useAllDataSourceFields", 
         "attr:DataBoundComponent.showHiddenFields","attr:DataBoundComponent.showDetailFields", "attr:DataBoundComponent.showComplexFields", 
         "attr:DataBoundComponent.fetchOperation","attr:DataBoundComponent.updateOperation", "attr:DataBoundComponent.addOperation", 
         "attr:DataBoundComponent.removeOperation","attr:DataBoundComponent.exportFields", "attr:DataBoundComponent.exportAll", 
         "attr:DataBoundComponent.preventDuplicates","attr:DataBoundComponent.duplicateDragMessage", "attr:DataBoundComponent.addDropValues", 
         "attr:DataBoundComponent.dropValues","attr:DataBoundComponent.useFlatFields", "attr:DataBoundComponent.hilites", 
         "attr:DataBoundComponent.hiliteProperty","attr:DataBoundComponent.dragDataAction", "attr:DataBoundComponent.dragTrackerStyle", 
         "attr:DataBoundComponent.canAddFormulaFields","attr:DataBoundComponent.addFormulaFieldText", 
         "attr:DataBoundComponent.editFormulaFieldText","attr:DataBoundComponent.canAddSummaryFields", 
         "attr:DataBoundComponent.addSummaryFieldText","attr:DataBoundComponent.editSummaryFieldText"],
        description:"A DataBoundComponent is a widget that can configure itself for viewing or editing objects which\n share a certain schema by \"binding\" to the schema for that object (called a \"DataSource\").\n <P>\n A schema (or DataSource) describes an object as consisting of a set of properties (or\n \"fields\").\n <P>\n DataBoundComponents have a ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','common set of APIs')} for\n dealing with binding to DataSources, \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields','overriding or augmenting')} the schema information\n provided by a DataSource, and manipulating objects or sets of object from the DataSource.\n <P>\n The following visual components currently support databinding:<pre>\n   ${isc.DocUtils.linkForRef('class:DynamicForm')}\n   ${isc.DocUtils.linkForRef('class:DetailViewer')}\n   ${isc.DocUtils.linkForRef('class:ListGrid')}\n   ${isc.DocUtils.linkForRef('class:TreeGrid')}\n   ${isc.DocUtils.linkForRef('class:TileGrid')}\n   ${isc.DocUtils.linkForRef('class:ColumnTree')}\n   ${isc.DocUtils.linkForRef('class:CubeGrid')}\n </pre>\n The following non-visual components also support databinding:<pre>\n   ${isc.DocUtils.linkForRef('class:ValuesManager')}\n   ${isc.DocUtils.linkForRef('class:ResultSet')}\n   ${isc.DocUtils.linkForRef('class:ResultTree')}\n </pre>",
        name:"DataBoundComponent",
        seeAlso:[],
        type:"interface"
    },
    "classMethod:RPCManager.handleError":{
        ref:"classMethod:RPCManager.handleError",
        flags:"",
        description:" <code>handleError</code> will be called if ${isc.DocUtils.linkForRef('attr:RPCResponse.status')} is negative and\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError')} was not set.\n <p>\n This method is called for both ${isc.DocUtils.linkForRef('class:DSResponse')}s and ${isc.DocUtils.linkForRef('class:RPCResponse')}s that have a\n non-success status.  You can check whether the response is a DSResponse by checking\n <code>response.isDSResponse</code>.\n <p>\n By default <code>handleError()</code> always logs a warning.  In addition, if\n ${isc.DocUtils.linkForRef('attr:RPCResponse.data','response.data')} was set to a String, a warning dialog will be\n shown to the user with response.data as the message, which allows the server to send\n user error messages back without writing custom client-side error handling.\n <p>\n To do custom error handling that is specific to a particular component or type of\n request, set ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError')} and deal with errors in the\n rpcRequest.callback.  To change the default system-wide error handling, override this\n method.  Note that since <code>handleError()</code> is a class method, to override it\n you will call ${isc.DocUtils.linkForRef('classMethod:Class.addClassProperties','addClassProperties()')} rather than\n addProperties(), like so:\n <pre>\n     isc.RPCManager.addClassProperties({\n         handleError : function (response, request) { .. custom handling .. }\n     })\n </pre>\n <p>\n If you're using the xmlHttpRequest ${isc.DocUtils.linkForRef('attr:RPCRequest.transport')}, you can access the \n <a href='http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html' onclick=\"window.open('http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html');return false;\">HTTP status code</a>\n of the response (eg 404 Not Found or 500 Server Error) as ${isc.DocUtils.linkForRef('attr:RPCResponse.httpResponseCode')}.\n <P>\n For very advanced usage, the response.xmlHttpRequest contains the native XMLHttpRequest\n object used to make the request.  Accessing this object is subject to possible\n cross-platform bugs and inconsistencies, and Isomorphic recommends that you wrap any\n access to the XMLHttpRequest object in a try/catch block because some browsers may throw\n exceptions when certain attributes of this object are accessed.  For example, if you try\n to access XMLHttpRequest.status (for the HTTP status code) when the network cable \n is unpluged in Windows, you'll get an Exception in Firefox.\n\n",
        name:"handleError",
        seeAlso:[
            {ref:"method:DataSource.handleError", description:null}
        ],
        params:[
            {description:"the RPCResponse or DSResponse object returned from the server",
             optional:false, name:"response", defaultValue:null, type:"Response"},
            {description:"the RPCRequest or DSRequest that was sent to the server", optional:false,
             name:"request", defaultValue:null, type:"Request"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager",
        groups:["operations"]
    },
    "attr:SerializationContext.useFlatFields":{
        ref:"attr:SerializationContext.useFlatFields",
        flags:"IR",
        description:"Enables flat serialization mode, as described for ${isc.DocUtils.linkForRef('attr:DSRequest.useFlatFields')}.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SerializationContext"
    },
    "attr:CubeGrid.columnFacets":{
        ref:"attr:CubeGrid.columnFacets",
        flags:"IRW",
        description:"The list of facets that will appear on top of the body",
        name:"columnFacets",
        seeAlso:[],
        valueType:"array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:ImgButton.valign":{
        ref:"attr:ImgButton.valign",
        flags:"IRW",
        description:"Vertical alignment of this component's title.",
        name:"valign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["appearance"]
    },
    "method:ListGrid.getFormulaFieldValue":{
        ref:"method:ListGrid.getFormulaFieldValue",
        flags:"",
        description:" Get the computed value of a ${isc.DocUtils.linkForRef('attr:ListGrid.canAddFormulaFields','formula field')}.\n",
        name:"getFormulaFieldValue",
        seeAlso:[],
        params:[
            {description:"field that has a formula", optional:false, name:"field", defaultValue:null,
             type:"Field"},
            {description:"record to use to compute formula value", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"formula result", type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:CubeGrid.facetValueHoverVAlign":{
        ref:"attr:CubeGrid.facetValueHoverVAlign",
        flags:"IRWA",
        description:"Allows the developer to override the vertical text alignment of hover tips shown for\n facet values.  If unspecified the hover canvas content alignment will be set by\n <code>this.hoverVAlign</code> if specified.",
        name:"facetValueHoverVAlign",
        seeAlso:[
            {ref:"attr:Canvas.hoverVAlign", description:null}
        ],
        valueType:"VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:ListGrid.generateDoubleClickOnEnter":{
        ref:"attr:ListGrid.generateDoubleClickOnEnter",
        flags:"IRWA",
        description:"If true, when the user navigates to a cell using arrow keys and hits Enter, \n the cell will respond to a double click event.",
        name:"generateDoubleClickOnEnter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:FormItem.getTitle":{
        ref:"method:FormItem.getTitle",
        flags:"A",
        description:"	Return the title of this formItem\n",
        name:"getTitle",
        seeAlso:[],
        returns:{description:"title for the formItem", type:"HTML"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["drawing"]
    },
    "method:List.intersect":{
        ref:"method:List.intersect",
        flags:"",
        description:" Return the list of items that are in both this list and the passed-in list(s).\n\n",
        name:"intersect",
        seeAlso:[],
        params:[
            {description:"lists to intersect with", optional:false, name:"lists", defaultValue:null,
             type:"all List arguments"}
        ],
        returns:{description:"intersection", type:"List"},
        type:"method",
        definingClass:"interface:List",
        groups:["arrayMath"]
    },
    "attr:FormulaBuilder.showAutoHideCheckBox":{
        ref:"attr:FormulaBuilder.showAutoHideCheckBox",
        flags:"IR",
        description:"Whether to show a checkbox offering the user the ability to automatically\n hide any fields involved in the formula.",
        name:"showAutoHideCheckBox",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:Array.add":{
        ref:"method:Array.add",
        flags:"",
        description:" Add an object to this list, at the end\n\n",
        name:"add",
        seeAlso:[],
        params:[
            {description:"object to add", optional:false, name:"object", defaultValue:null, type:"any"}
        ],
        returns:{description:"pointer to the object passed in", type:"any"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "method:ListGrid.rowMouseUp":{
        ref:"method:ListGrid.rowMouseUp",
        flags:"",
        description:" Called when a row receives a mouseup event.\n\n",
        name:"rowMouseUp",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:CubeGrid.getColumnFacetLayout":{
        ref:"method:CubeGrid.getColumnFacetLayout",
        flags:"",
        description:" Get the current heights of the column facets, as an object of the form:\n <pre>\n [ {facetId:facetId, height:currentHeight}, ... ]\n </pre>\n\n",
        name:"getColumnFacetLayout",
        seeAlso:[],
        returns:{description:"array of {facetId:facetId, height:height} objects", type:"array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:EdgedCanvas.edgeBottom":{
        ref:"attr:EdgedCanvas.edgeBottom",
        flags:"IR",
        description:"Height in pixels for bottom corners and edges.  Defaults to edgeSize when unset.",
        name:"edgeBottom",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:ComboBoxItem.separatorRows":{
        ref:"attr:ComboBoxItem.separatorRows",
        flags:"IR",
        description:"Array of records to show between matching and non-matching rows in the PickList.\n <P>\n Not valid for ${isc.DocUtils.linkForRef('attr:ComboBoxItem.optionDataSource','databound pickLists')}.",
        name:"separatorRows",
        seeAlso:[],
        valueType:"Array of ListGridRecord",
        defaultValue:"[{isSeparator:true}]",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:StretchImgButton.radioGroup":{
        ref:"attr:StretchImgButton.radioGroup",
        flags:"IRWA",
        description:"String identifier for this canvas's mutually exclusive selection group.",
        name:"radioGroup",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state", "event handling"]
    },
    "attr:RestDataSource.updateDataURL":{
        ref:"attr:RestDataSource.updateDataURL",
        flags:"IR",
        description:"Custom dataURL for update type operations",
        name:"updateDataURL",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:ColumnTree.showNodeCount":{
        ref:"attr:ColumnTree.showNodeCount",
        flags:"IR",
        description:"If set, and ${isc.DocUtils.linkForRef('attr:ColumnTree.showHeaders')} is also set, each column's header will show \n a count of the number of nodes in that column",
        name:"showNodeCount",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ColumnTree"
    },
    "group:componentXML":{
        ref:"group:componentXML",
        title:"Component XML",
        treeLocation:["Concepts"],
        refs:[],
        description:"As covered in the <i>QuickStart Guide</i> Chapter 4, <i>Coding</i>, SmartClient \n components can be created in either XML or JavaScript format.  This section covers some of\n the details of using the XML format, called \"SmartClient component XML\".\n <P>\n <b>Referring to previously created components</b>\n <P>\n To refer to another component by ID in XML, use &lt;Canvas ref=/&gt;.  For example:\n <pre>\n &lt;Canvas ID=\"myCanvas\"/&gt;\n &lt;Canvas ID=\"myCanvas2\"/&gt;\n &lt;VLayout&gt;\n     &lt;members&gt;\n         &lt;Canvas ref=\"myCanvas\"/&gt;\n         &lt;Canvas ref=\"myCanvas2\"/&gt;\n     &lt;/members&gt;\n &lt;/VLayout&gt;\n </pre>\n <P>\n <b>JavaScript expressions</b>\n <P>\n To embed a JavaScript expression into component XML, use the &lt;JS&gt; tag.  For example:\n <pre>\n &lt;VLayout&gt;\n     &lt;width&gt;&lt;JS&gt;isc.Page.getWidth() - 20&lt;/JS&gt;&lt;/width&gt;\n &lt;/VLayout&gt;\n </pre>\n Note that, like all component XML properties, the <code>width</code> property can be\n specified either as an XML attribute or as a subelement.  Expressing it as a subelement, as\n shown above, allows the &lt;JS&gt; tag to be used.\n <P>\n <b>Embedding Methods</b>\n <P>\n For ${isc.DocUtils.linkForRef('group:stringMethods','StringMethods')} such as ${isc.DocUtils.linkForRef('method:ListGrid.recordClick')},\n JavaScript code can be used as an ordinary element value:\n <pre>\n &lt;ListGrid&gt;\n     &lt;recordClick&gt;if (record.age > 65) doSomething()&lt;/recordClick&gt;\n &lt;/ListGrid&gt;\n </pre>\n To embed an actual function definition, use the &lt;JS&gt; tag described above.  For\n example:\n <pre>\n &lt;ListGrid&gt;\n     &lt;recordClick&gt;&lt;JS&gt;function (viewer, record, recordNum, field) {\n          if (record.age > 65) doSomething();\n     }&lt;/JS&gt;&lt;/recordClick&gt;\n &lt;/ListGrid&gt;\n </pre>\n Unfortunately, characters commonly used in JavaScript code, such as ampersand (&amp;), are\n not legal inside XML element or attribute values.  For example, the expression \"record !=\n null && record.age > 65\" must be written as shown below, or it is not considered valid XML:\n <P>\n <pre>\n &lt;ListGrid&gt;\n     &lt;recordClick&gt;\n         if (record.status != null &amp;amp;&amp;amp; record.age > 65) doSomething()\n     &lt;/recordClick&gt;\n &lt;/ListGrid&gt;\n </pre>\n An alternative, for larger blocks of code, is to use the XML standard \"CDATA\" (character\n data) processing directive, which allows ampersand and other characters to be used without\n special notation:\n <pre>\n &lt;ListGrid&gt;\n     &lt;recordClick&gt;&lt;![CDATA[\n         if (record.status != null && record.age > 65) doSomething()\n     ]]&gt;&lt;/recordClick&gt;\n &lt;/ListGrid&gt;\n </pre>\n <P>\n Overall, embedding code in XML can be awkward.  Isomorphic generally recommends that\n signficant chunks of JavaScript code, such as non-trivial custom components, be moved to\n separate, purely JavaScript files, while code embedded in component XML is limited to simple\n expressions and short functions.\n <P>\n <b>Troubleshooting</b>\n <P>\n XML parsing errors, which happen when XML is not well-formed and would be rejected by any\n standard XML parser, are reported in the server-side log, and, when possible, also in the\n client-side log within the \"Results\" tab of the Developer Console.\n <P>\n Other issues with component XML can result from incorrect use of SmartClient component XML\n tags.  For example, you may specify a property and it may appear to have no effect even\n though it clearly works in other, JavaScript-based examples.  If you get this symptom, you\n can troubleshoot by looking at the JavaScript code SmartClient generates from component XML.\n <P>\n SmartClient always translates SmartClient component XML to JavaScript format before\n execution.  This is done automatically by the &lt;isomorphic:XML&gt; tag for XML embedded in\n a .jsp file, and you can \"View Source\" with your browser to see the generated XML.  You can\n also use the \"Eval XML\" section in the \"Results\" tab of the Developer Console to see\n the generated JavaScript (\"Show JS\" button) and to dynamically execute component XML (\"Eval\n XML\" button).  These facilities will help you troubleshoot issues with incorrectly specified\n XML.\n <P>\n <b>Custom Properties</b>\n <P>\n If you specify a custom property on a component in XML, for example:\n <pre>\n &lt;Canvas myProperty=\"false\"/&gt;\n </pre>\n The value of the property will be a JavaScript String.  In the above example, it would be\n the string \"false\", which is considered a boolean true value in the JavaScript language.\n If you want a different JavaScript type, you can force a property to be interpreted as a\n given type by using the \"xsi:type\" attribute:\n <pre>\n &lt;Canvas&gt;\n     &lt;myProperty xsi:type=\"xsd:boolean\"&gt;false&lt;/myProperty&gt;\n &lt;/Canvas&gt;\n </pre>\n The same notation works when you want to declare that an entire subobject has a given type.\n For example, this would cause the custom property \"myListGrid\" to have a live\n ${isc.DocUtils.linkForRef('class:ListGrid')} instance as it's value.  All of the properties on the &lt;myListGrid&gt; tag\n will be correctly interpreted as ListGrid properties and have the correct types.\n <pre>\n &lt;Canvas&gt;\n     &lt;myListGrid xsi:type=\"ListGrid\" width=\"500\" height=\"600\"/&gt;\n &lt;/Canvas&gt;\n </pre>\n If you do not want an actual live ListGrid, but rather a JavaScript Object containing\n properties for later construction of a ListGrid, use the <code>propertiesOnly</code>\n attribute.  For example, this code would cause the property \"listGridProperties\" to be a\n JavaScript Object with properties \"width\" and \"height\", whose values would be JavaScript\n Numbers.\n <pre>\n &lt;Canvas&gt;\n     &lt;listGridProperties xsi:type=\"ListGrid\" propertiesOnly=\"true\" \n                          width=\"500\" height=\"600\"/&gt;\n &lt;/Canvas&gt;\n </pre>\n For your reference: \"xsi\" stands for \"XML Schema Instance\"; this notation derives from XML\n Schema standards for explicitly specifying type inline.\n <P>\n <B>Custom Components</B>\n <P>\n If you use ${isc.DocUtils.linkForRef('classMethod:ClassFactory.defineClass','defineClass()')} to define a new component class\n \"MyListGrid\" which is a subclass of the built-in component ListGrid, you can create it in\n XML as shown below:\n <pre>\n &lt;ListGrid constructor=\"MyListGrid\" width=\"500\"/&gt;\n </pre>\n By using the &lt;ListGrid&gt; tag you advertise that properties should be interpreted \n as <code>ListGrid</code> properties.  By specifying <code>constructor</code>\n you tell SmartClient what class to ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')}.\n <P>\n <b>Component Schema</b>\n <P>\n Instead of using the <code>constructor</code> and <code>xsi:type</code> attributes for\n custom components and custom properties, you can create a ${isc.DocUtils.linkForRef('group:componentSchema')} that\n describes the custom component.  Declaring a component schema allows you to use your\n component just like the built-in SmartClient components, and also allows your component to\n be used within ${isc.DocUtils.linkForRef('group:visualBuilder')}.",
        name:"componentXML",
        seeAlso:[],
        type:"group"
    },
    "method:Calendar.getDayBodyHTML":{
        ref:"method:Calendar.getDayBodyHTML",
        flags:"",
        description:" Return the HTML to be shown in the body of a day in the month view.\n <P>\n Default is to render a series of links that call ${isc.DocUtils.linkForRef('method:Calendar.eventClick')} to provide details\n and/or an editing interface for the events.\n <P>\n <code>getDayBodyHTML()</code> is not called for days outside of the current month if\n ${isc.DocUtils.linkForRef('attr:Calendar.showOtherDays')} is false.\n\n",
        name:"getDayBodyHTML",
        seeAlso:[],
        params:[
            {description:"JavaScript Date object representing this day", optional:false, name:"date",
             defaultValue:null, type:"Date"},
            {description:"events that fall on this day", optional:false, name:"events",
             defaultValue:null, type:"Array of CalendarEvent"},
            {description:"the calendar itself", optional:false, name:"calendar", defaultValue:null,
             type:"Calendar"},
            {description:"the row number to which the parameter date belongs", optional:false,
             name:"rowNum", defaultValue:null, type:"Integer"},
            {description:"the column number to which the parameter date belongs", optional:false,
             name:"colNum", defaultValue:null, type:"Integer"}
        ],
        returns:{description:"HTML to display", type:"HTML"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewFormatting"]
    },
    "class:EventWindow":{
        methods:[],
        ref:"class:EventWindow",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"Subclass of Window used to display events within a ${isc.DocUtils.linkForRef('class:Calendar')}.  Customize via\n ${isc.DocUtils.linkForRef('attr:Calendar.eventWindow')}.",
        name:"EventWindow",
        seeAlso:[],
        type:"class"
    },
    "method:DataSource.handleError":{
        ref:"method:DataSource.handleError",
        flags:"A",
        description:" If you define this method on a DataSource, it will be called whenever the server returns\n a DSResponse with a status other than ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_SUCCESS')}.  You can use\n this hook to do DataSource-specific error handling.  Unless you return\n <code>false</code> from this method, ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')} will be called by\n SmartClient right after this method completes.\n \n",
        name:"handleError",
        seeAlso:[
            {ref:"classMethod:RPCManager.handleError", description:null}
        ],
        params:[
            {description:"the DSResponse or DSResponse object returned from the server",
             optional:false, name:"response", defaultValue:null, type:"DSResponse"},
            {description:"the DSRequest or DSRequest that was sent to the server", optional:false,
             name:"request", defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:"false to suppress ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')}",
         type:"boolean"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["errorHandling"]
    },
    "attr:StatefulCanvas.showFocused":{
        ref:"attr:StatefulCanvas.showFocused",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  If\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is <code>true</code>, the <b><code>\"over\"</code></b>\n will be used to indicate focus. Otherwise a separate <b><code>\"focused\"</code></b> state\n will be used.",
        name:"showFocused",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "method:Tree.isOpen":{
        ref:"method:Tree.isOpen",
        flags:"",
        description:"\n Whether a particular node is open or closed (works for leaves and folders).\n\n",
        name:"isOpen",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node is open", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:StretchImgButton.getActionType":{
        ref:"method:StretchImgButton.getActionType",
        flags:"",
        description:" Return the 'actionType' for this canvas (radio / checkbox / button)\n",
        name:"getActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state", "event handling"]
    },
    "attr:PickListMenu.emptyMessage":{
        ref:"attr:PickListMenu.emptyMessage",
        flags:"IRW",
        description:"The string to display in the body of a listGrid with an empty data array, if\n showEmptyMessage is true.",
        name:"emptyMessage",
        seeAlso:["showEmptyMessage", "emptyMessageStyle"],
        valueType:"string",
        defaultValue:"\"No Items To Show\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('emptyGrid')}"],
        definingClass:"class:PickListMenu",
        groups:["i18nMessages"]
    },
    "attr:FormItem.pickerProperties":{
        ref:"attr:FormItem.pickerProperties",
        flags:"IRW",
        description:"Default properties for the picker.",
        name:"pickerProperties",
        seeAlso:[],
        valueType:"Canvas Properties",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "class:VLayout":{
        methods:[],
        exampleConfig:"VLayout.xml",
        ref:"class:VLayout",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:[],
        description:"A subclass of Layout that applies a sizing policy along the vertical axis, interpreting\n  percent and \"*\" sizes as proportions of the height of the layout. VLayouts will set any\n  members that do not have explict widths to match the layout.",
        name:"VLayout",
        seeAlso:[
            {ref:"attr:Layout.vPolicy", description:null}
        ],
        type:"class"
    },
    "attr:RichTextItem.colSpan":{
        ref:"attr:RichTextItem.colSpan",
        flags:"IRW",
        description:"By default RichTextItems take up an entire row",
        name:"colSpan",
        seeAlso:[],
        valueType:"number | string",
        defaultValue:"\"*\"",
        type:"attr",
        definingClass:"class:RichTextItem"
    },
    "method:Button.getActionType":{
        ref:"method:Button.getActionType",
        flags:"",
        description:" Return the 'actionType' for this canvas (radio / checkbox / button)\n",
        name:"getActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["state", "event handling"]
    },
    "attr:MultiSortPanel.addLevelButtonTitle":{
        ref:"attr:MultiSortPanel.addLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the addLevelButton",
        name:"addLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Add Level\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "method:Array.getRange":{
        ref:"method:Array.getRange",
        flags:"",
        description:" Return the items between position start and end, non-inclusive at the end.\n\n",
        name:"getRange",
        seeAlso:[],
        params:[
            {description:"start position", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end position", optional:false, name:"end", defaultValue:null, type:"number"}
        ],
        returns:{description:"subset of the array from start -> end-1", type:"Array"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "classMethod:isc.ask":{
        ref:"classMethod:isc.ask",
        flags:"",
        description:" Show a modal dialog with a message, icon, and \"Yes\" and \"No\" buttons.\n <P>\n The callback will receive boolean true for an OK button click, boolean false for a No button\n click, or null if the Dialog is dismissed via the close button.\n\n",
        name:"ask",
        seeAlso:[
            {ref:"classAttr:Dialog.Warn", description:null},
            {ref:"classMethod:isc.warn", description:null},
            {ref:"method:Dialog.yesClick", description:null},
            {ref:"method:Dialog.noClick", description:null},
            {ref:"classAttr:Dialog.ASK_TITLE", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"Callback to fire when the                                       user clicks a button to dismiss the dialog.\n                                      This has the single parameter 'value', indicating the \n                                      value returned by the Warn dialog from 'okClick()' etc.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog.                                      To set ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons','custom buttons')} for\n                                      the Dialog, set properties.toolbarButtons to an array \n                                      of buttons\n										eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('dialogs')}"],
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "attr:Calendar.eventEditorLayout":{
        ref:"attr:Calendar.eventEditorLayout",
        flags:"R",
        description:"An autochild of type ${isc.DocUtils.linkForRef('class:Window')} that displays the full \n ${isc.DocUtils.linkForRef('attr:Calendar.eventEditor','event editor')}",
        name:"eventEditorLayout",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:ListGrid.getSelectedRecord":{
        ref:"method:ListGrid.getSelectedRecord",
        flags:"",
        description:" Return the first selected record in this component.<br><br>\n This method is appropriate if <code>${isc.DocUtils.linkForRef('attr:ListGrid.selectionType')}</code> is\n <code>\"single\"</code>, or if you only care about the first selected record in\n a multiple-record selection. To access all selected records, use\n <code>${isc.DocUtils.linkForRef('method:ListGrid.getSelection')}</code> instead.\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"first selected record, or null if nothing selected", type:"ListGridRecord"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:Canvas.init":{
        ref:"method:Canvas.init",
        flags:"A",
        description:"\n This method performs some basic initialization common to all UI components.  To do custom UI\n component initialization, you should generally override ${isc.DocUtils.linkForRef('method:Canvas.initWidget')}.  This\n method does the following, in order:\n <ul>\n <li>Sets up a global reference to this instance as described in ${isc.DocUtils.linkForRef('attr:Canvas.ID')}.\n <li>Ensures certain numeric properties have numeric values (e.g. width, height, padding,\n margin)\n <li>Ensures ${isc.DocUtils.linkForRef('attr:Canvas.children')} and ${isc.DocUtils.linkForRef('attr:Canvas.peers')} are Arrays.\n <li>Calls ${isc.DocUtils.linkForRef('method:Canvas.initWidget')}\n <li>Creates ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','edges')} and ${isc.DocUtils.linkForRef('attr:Canvas.showShadow','shadow')}, if so configured.\n <li>Calls ${isc.DocUtils.linkForRef('method:Canvas.draw')} if ${isc.DocUtils.linkForRef('attr:Canvas.autoDraw')} is set on instance or globally.\n </ul>\n Unless you're in an advanced scenario where you need to inject code before the above\n logic executes, place your initialization logic in initWidget() rather than init().  If you\n do decided to override this method, you must call the superclass implementation like so:\n <pre>\n    this.Super(\"init\", arguments);\n </pre>\n\n",
        name:"init",
        seeAlso:[],
        params:[
            {description:"All arguments initially passed to ${isc.DocUtils.linkForRef('classMethod:Class.create')}",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "classMethod:Log.timeMethod":{
        ref:"classMethod:Log.timeMethod",
        flags:"",
        description:" \n  Observe a method on an object, logging execution time whenever the method is called.\n <P>\n  Call a second time with identical arguments to disable tracing.\n\n",
        name:"timeMethod",
        seeAlso:[],
        params:[
            {description:"object to observe", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"name of the method to observe", optional:false, name:"methodName",
             defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "method:TileGrid.getTileHTML":{
        ref:"method:TileGrid.getTileHTML",
        flags:"",
        description:" When using the default ${isc.DocUtils.linkForRef('class:SimpleTile')} class as ${isc.DocUtils.linkForRef('attr:TileGrid.tileConstructor')}, this\n method provides the HTML to be displayed within each tile.  See ${isc.DocUtils.linkForRef('attr:TileGrid.tile')}.\n \n",
        name:"getTileHTML",
        seeAlso:[],
        params:[
            {description:"the tile for which HTML should be retrieved", optional:false,
             name:"tileRecord", defaultValue:null, type:"TileRecord"}
        ],
        returns:{description:"HTML contents for the tile, as a String", type:"HTML"},
        type:"method",
        definingClass:"class:TileGrid"
    },
    "type:LayoutPolicy":{
        ref:"type:LayoutPolicy",
        values:[
            {description:"Layout does not try to size members on the axis at all, merely stacking them (length\n  axis) and leaving them at default breadth.",
             value:"Layout.NONE"},
            {description:"Layout sizes members so that they fill the specified size of the layout.  The rules\n  are:\n  <ul>\n  <li> Any component given an initial pixel size, programmatically resized to a specific\n  pixel size, or drag resized by user action is left at that exact size\n  <li> Any component that ${isc.DocUtils.linkForRef('attr:Button.autoFit','autofits')} is given exactly the space it\n  needs, never forced to take up more.\n  <li> All other components split the remaining space equally, or according to their\n  relative percentages.\n  </ul>",
             value:"Layout.FILL"}
        ],
        description:"Policy controlling how the Layout will manage member sizes on this axis.\n  <P>\n  See also ${isc.DocUtils.linkForRef('attr:Layout.overflow')}.",
        name:"LayoutPolicy",
        seeAlso:[],
        type:"type"
    },
    "method:Canvas.animateMove":{
        ref:"method:Canvas.animateMove",
        flags:"",
        description:" Animate a reposition of this canvas from its current position to the specified position\n",
        name:"animateMove",
        seeAlso:[],
        params:[
            {description:"new left position (or null for unchanged)", optional:false, name:"left",
             defaultValue:null, type:"number"},
            {description:"new top position (or null for unchanged)", optional:false, name:"top",
             defaultValue:null, type:"number"},
            {description:"When the move completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated move", optional:true, name:"duration",
             defaultValue:null, type:"number"},
            {description:"Optional acceleration effect to bias the ratios", optional:true,
             name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateMove')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:ListGrid.autoFetchTextMatchStyle":{
        ref:"attr:ListGrid.autoFetchTextMatchStyle",
        flags:"IR",
        description:"When this grid is initially filtered via ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchData')}, or filtered by the user \n via the ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor','filterEditor')}, this attribute can be used to set the\n <code>textMatchStyle</code> on the dsRequest passed to <code>fetchData()</code>.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"\"substring\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["dataBinding"]
    },
    "method:MultiView.fetchData":{
        ref:"method:MultiView.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component.\n <P>\n If there are a large number of matching records, paging will automatically be enabled, so\n that initially a smaller number of records will be retrieved and further records will\n be fetched as the user navigates the dataset.\n <P>\n When first called, this method will create a ${isc.DocUtils.linkForRef('class:ResultSet')}, which will be\n configured based on component settings such as ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fetchOperation')}\n and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataPageSize')}, as well as the general purpose\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')}.  The ResultSet is then available as\n <code>component.data</code>.\n <P>\n Subsequent calls to fetchData() will simply call ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')}\n on the created ResultSet with the passed criteria.\n <P>\n In some cases fetchData() will not need to contact the server as the new criteria can be\n satisfied by performing a client-side filter against the currently cached set of data.\n You can determine whether criteria will cause a fetch by calling \n ${isc.DocUtils.linkForRef('method:ResultSet.willFetchData')}.  If you need to force a server fetch, you can call\n ${isc.DocUtils.linkForRef('method:ListGrid.invalidateCache','invalidateCache()')} to do so.\n <P>\n This method takes an optional callback parameter (set to a ${isc.DocUtils.linkForRef('type:DSCallback')}) to fire when\n the fetch completes. Note that this callback will not fire if no server fetch is performed.\n In this case the data is updated synchronously, so as soon as this method completes you\n can interact with the new data. If necessary, you can use\n {dataBoundComponent.willFetchData,willFetchData()} to determine whether or not a server\n fetch will occur when <code>fetchData()</code> is called with new criteria.\n <P>\n In addition to the callback parameter for this method, developers can use \n ${isc.DocUtils.linkForRef('method:ListGrid.dataArrived','dataArrived()')} to be notified every time data is loaded.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "method:Canvas.getInnerHeight":{
        ref:"method:Canvas.getInnerHeight",
        flags:"A",
        description:"  Returns the amount of space available for (an) absolutely positioned child widget(s) or \n  HTML content, without introducing clipping, scrolling or overflow.<br>\n  This is the space within the viewport of the widget (including padding, but excluding \n  margins, borders or scrollbars) rendered at its specified size.\n\n",
        name:"getInnerHeight",
        seeAlso:[
            {ref:"method:Canvas.getInnerWidth", description:null},
            {ref:"method:Canvas.getInnerContentHeight", description:null},
            {ref:"method:Canvas.getInnerContentWidth", description:null}
        ],
        returns:{description:"inner height of the widget in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:DataSource.sendExtraFields":{
        ref:"attr:DataSource.sendExtraFields",
        flags:"IR",
        description:"Analagous to ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields')}, for data sent to the server.  Setting this\n attribute to false ensures that for any records in the data object, only fields that\n correspond to declared dataSource fields will be present on the dsRequest data object passed\n to ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} and ultimately sent to the server.",
        name:"sendExtraFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration"]
    },
    "attr:ResultSet.updatePartialCache":{
        ref:"attr:ResultSet.updatePartialCache",
        flags:"IRA",
        description:"If set to true, updated and added rows will be integrated into the client-side cache\n even if paging is enabled and cache is partial.  If <code>updatePartialCache</code> is\n false, the cache will be invalidated and new data fetched.\n <P>\n If updatePartialCache is enabled and an \"add\" or \"update\" operation succeeds with a partial\n cache:\n <ul>\n <li> updated rows will remain in their current position.  No attempt will be made to sort\n them into a new position even if the sort field was updated.\n <li> newly added rows will be added at either the end (first preference) or beginning of\n the dataset if that part of the dataset is cached and was most recently requested.\n If not, the new row is added at the end of the most recently requested contiguously\n cached range.\n </ul>\n The cache will then be dropped the next time rows are fetched, to prevent problems with\n inconsistent row numbering between the server and client, which could otherwise lead to\n duplicate rows or rows being skipped entirely.",
        name:"updatePartialCache",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["cacheSync"]
    },
    "attr:TimeItem.useMask":{
        ref:"attr:TimeItem.useMask",
        flags:"IRA",
        description:"If true, a data entry mask will be enabled in the field based on the\n ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat')}.\n <p>\n Note that if a non-padded ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat')} is specified, it\n will be changed to the corresponding padded version (ex. \"toShort24HourTime\"\n will be changed to \"toShortPadded24HourTime\").",
        name:"useMask",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TimeItem"
    },
    "group:corner":{
        ref:"group:corner",
        refs:["attr:Scrollbar.showCorner", "attr:Scrollbar.cornerSize"],
        name:"corner",
        seeAlso:[],
        type:"group"
    },
    "attr:WebService.serviceNamespace":{
        ref:"attr:WebService.serviceNamespace",
        flags:"R",
        description:"Namespace of this WebService, derived from the <code>targetNamespace</code>\n attribute of the <code>&lt;wsdl:definitions&gt;</code> element.",
        name:"serviceNamespace",
        seeAlso:[],
        valueType:"URI",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "method:TreeGrid.getDragTrackerIcon":{
        ref:"method:TreeGrid.getDragTrackerIcon",
        flags:"",
        description:" Return an icon to display as a drag tracker when the user drags some node(s).<br>\n Default implementation:<br>\n If multiple nodes are selected and ${isc.DocUtils.linkForRef('attr:TreeGrid.manyItemsImage')} is defined, this\n image wlll be returned<br>\n Otherwise returns the result of ${isc.DocUtils.linkForRef('method:TreeGrid.getIcon')} for the first node being \n dragged.\n <p>\n Note: Only called if ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')} is set to <code>\"icon\"</code>. \n",
        name:"getDragTrackerIcon",
        seeAlso:[],
        params:[
            {description:"Records being dragged", optional:false, name:"records", defaultValue:null,
             type:"Array of ListGridRecord"}
        ],
        returns:{description:"Image URL of icon to display", type:"string"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["dragTracker"]
    },
    "method:LoginDialog.lostPassword":{
        ref:"method:LoginDialog.lostPassword",
        flags:"",
        description:" Called if the user clicks on the ${isc.DocUtils.linkForRef('attr:LoginDialog.lostPasswordItem','\"Lost Password\"')} link\n on the login form. Implement this method to allow the user to request the password \n be resent or reset.\n",
        name:"lostPassword",
        seeAlso:[],
        params:[
            {description:"Current values of form fields", optional:false, name:"values",
             defaultValue:null, type:"Object"},
            {description:"Form on which the link was clicked", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:LoginDialog"
    },
    "method:Canvas.getPageLeft":{
        ref:"method:Canvas.getPageLeft",
        flags:"A",
        description:" Returns the page-relative left coordinate of the widget on the page, in pixels.\n",
        name:"getPageLeft",
        seeAlso:[],
        returns:{description:"global left coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:Button.showRollOver":{
        ref:"attr:Button.showRollOver",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes over this object?",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonStates')}"],
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:Canvas.dragTarget":{
        ref:"attr:Canvas.dragTarget",
        flags:"IRWA",
        description:"A different widget that should be actually dragged when dragging initiates on this\n      widget. One example of this is to have a child widget that drags its parent, as with a\n      drag box. Because the parent automatically repositions its children, setting the drag\n      target of the child to the parent and then dragging the child will result in both\n      widgets being moved.<br>\n      Valid dragTarget values are:<br>\n      - <code>null</code> (default) [this widget is its own drag target]<br>\n      - Pointer to another widget, or widget ID<br>\n      - <code>\"parent\"</code> drag target is this widget's \n         ${isc.DocUtils.linkForRef('attr:Canvas.parentElement','parentElement')}<br>\n      - <code>\"top\"</code> drag target is this widget's \n         ${isc.DocUtils.linkForRef('attr:Canvas.topElement','topElement')}<br>",
        name:"dragTarget",
        seeAlso:[
            {ref:"classMethod:EventHandler.getDragTarget", description:null}
        ],
        valueType:"Canvas | String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Button.setAlign":{
        ref:"method:Button.setAlign",
        flags:"",
        description:" Sets the (horizontal) alignment of this buttons content.\n",
        name:"setAlign",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["positioning"]
    },
    "classAttr:DataSource.serializeTimeAsDatetime":{
        ref:"classAttr:DataSource.serializeTimeAsDatetime",
        flags:"IRA",
        description:"Specifies how time field values should be serialized when being sent to the server for\n dataSources with dataFormat <code>\"xml\"</code> or <code>\"json\"</code>.\n If <code>false</code> the time field will be serialized as a logical time object in\n UTC, using the standard truncated XML Schema format: <code>\"HH:MM:SS\"</code>.\n If <code>true</code> the time field will be serialized as a complete dateTime object\n matching the value of the underlying JavaScript date object representing this time on the\n client.",
        name:"serializeTimeAsDatetime",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"classAttr",
        definingClass:"class:DataSource"
    },
    "classMethod:isA.Object":{
        ref:"classMethod:isA.Object",
        flags:"",
        description:"\n	Is <code>object</code> a non-null object<br><br>\n\n  NOTE: This will return false for objects of type number, string, boolean, function, \n  undefined.  It will also return false for the null value.\n  For other objects, including arrays, regular expressions, date objects, and JS object \n  literals it will return true.\n\n",
        name:"Object",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is an object", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:groupNode.groupTitle":{
        ref:"attr:groupNode.groupTitle",
        flags:"R",
        description:"The computed title for the group, which results from ${isc.DocUtils.linkForRef('method:ListGridField.getGroupTitle')}",
        name:"groupTitle",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"HTML",
        defaultValue:"see below",
        type:"attr",
        definingClass:"object:groupNode",
        groups:["grouping"]
    },
    "method:ListGrid.addSort":{
        ref:"method:ListGrid.addSort",
        flags:"A",
        description:"  Adds another ${isc.DocUtils.linkForRef('object:SortSpecifier')} to this grid's sort configuration and resorts.\n",
        name:"addSort",
        seeAlso:[],
        params:[
            {description:"A SortSpecifier object indicating an additional field     and direction to sort by",
             optional:false, name:"sortSpecifier", defaultValue:null, type:"SortSpecifier"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "classMethod:RPCManager.sendQueue":{
        ref:"classMethod:RPCManager.sendQueue",
        flags:"",
        description:"\n Send all currently queued requests to the server.  You need only call this method if you are\n using queuing otherwise your requests are synchronously submitted to the server.\n <br><br>\n NOTE: if you aren't the caller who first enables queuing (startQueue() returns\n true), you should in general avoid calling sendQueue(), because whoever was\n first to enable queuing may have more requests to add to the same queue.\n\n",
        name:"sendQueue",
        seeAlso:[
            {ref:"classMethod:RPCManager.send", description:null},
            {ref:"classMethod:RPCManager.sendRequest", description:null},
            {ref:"classMethod:RPCManager.startQueue", description:null}
        ],
        params:[
            {description:"Callback to fire when the queued operations complete. Callback will be fired with 1 paramter: <code>responses</code> an array of ${isc.DocUtils.linkForRef('class:DSResponse')} or \n ${isc.DocUtils.linkForRef('class:RPCResponse')} objects that were part of the transaction fired by this method.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager",
        requiresModules:["SCServer"]
    },
    "attr:PickList.pickListHeaderHeight":{
        ref:"attr:PickList.pickListHeaderHeight",
        flags:"IRW",
        description:"If this pick list is showing multiple fields, this property determines the height of\n the column headers for those fields. Set to zero to suppress the headers entirely.",
        name:"pickListHeaderHeight",
        seeAlso:[
            {ref:"attr:PickList.pickListFields", description:null}
        ],
        valueType:"number",
        defaultValue:"22",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:TreeGrid.treeRootValue":{
        ref:"attr:TreeGrid.treeRootValue",
        flags:"IRA",
        description:"For databound trees, use this attribute to supply a ${isc.DocUtils.linkForRef('attr:DataSourceField.rootValue')} for this\n component's generated data object.\n <P> \n This property allows you to have a particular component navigate a tree starting from any\n given node as the root.",
        name:"treeRootValue",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["databinding"]
    },
    "attr:ListGrid.canAddFormulaFields":{
        ref:"attr:ListGrid.canAddFormulaFields",
        flags:"IRW",
        description:"Adds an item to the header context menu allowing users to launch a dialog to define a new\n field based on values present in other fields, using the ${isc.DocUtils.linkForRef('class:FormulaBuilder')}.\n <P>\n User-added formula fields can be persisted via ${isc.DocUtils.linkForRef('method:ListGrid.getFieldState')} and \n ${isc.DocUtils.linkForRef('method:ListGrid.setFieldState')}.",
        name:"canAddFormulaFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["formulaFields"]
    },
    "type:DragAppearance":{
        ref:"type:DragAppearance",
        values:[
            {description:"No default drag appearance is indicated. Your custom dragging routines should\n      implement some behavior that indicates that the user is in a dragging situation,\n      and where the mouse is.",
             value:"\"none\""},
            {description:"A \"drag tracker\" object is automatically shown and moved around with the\n      mouse. This is generally set to an icon that represents what is being dragged. The\n      default tracker is a 10 pixel black square, but you can customize this icon. This\n      dragAppearance is not recommended for use with drag resizing or drag moving.",
             value:"\"tracker\""},
            {description:"The target object is actually moved, resized, etc. in real time. This is\n      recommended for drag repositioning, but not for drag resizing of complex objects.",
             value:"\"target\""},
            {description:"An outline the size of the target object is moved, resized, etc. with the\n      mouse. This is recommended for drag resizing, especially for objects that take a\n      significant amount of time to draw.",
             value:"\"outline\""}
        ],
        description:"Different types of effects for showing dragging behavior.",
        name:"DragAppearance",
        seeAlso:[],
        type:"type",
        groups:["dragdrop"]
    },
    "classMethod:Log.isEnabledFor":{
        ref:"classMethod:Log.isEnabledFor",
        flags:"",
        description:" Would a message logged to the given category at the given priority appear in the Log?\n <P>\n NOTE: if there is no specific priority setting for a given category, the\n <code>Log.defaultPriority</code> is used.\n\n",
        name:"isEnabledFor",
        seeAlso:[],
        params:[
            {description:"category name", optional:false, name:"category", defaultValue:null,
             type:"String"},
            {description:"priority level to check", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:ImgProperties.extraStuff":{
        ref:"attr:ImgProperties.extraStuff",
        flags:"IRW",
        description:"Specifies the additional attributes to write in the tag.",
        name:"extraStuff",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "classAttr:Operators.inSetTitle":{
        ref:"classAttr:Operators.inSetTitle",
        flags:"IR",
        description:"Title for the \"inSet\" operator",
        name:"inSetTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"is one of\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:StatefulCanvas.title":{
        flags:"IRW",
        getter:["getTitle"],
        type:"attr",
        ref:"attr:StatefulCanvas.title",
        description:"The text title to display in this button.",
        name:"title",
        setter:["setTitle"],
        valueType:"string",
        seeAlso:[],
        defaultValue:"varies",
        groups:["basics"],
        definingClass:"class:StatefulCanvas"
    },
    "attr:ActiveXControl.classID":{
        ref:"attr:ActiveXControl.classID",
        flags:"IR",
        description:"This sets the value of the classID property on the object.  This is meant to give you\n  complete control over the generated HTML.  In practice it may be more handy to set the uuid\n  property on this object and let the classID be generated from that.",
        name:"classID",
        seeAlso:[
            {ref:"attr:ActiveXControl.uuid", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ActiveXControl"
    },
    "attr:SelectOtherItem.dialogWidth":{
        ref:"attr:SelectOtherItem.dialogWidth",
        flags:"IRW",
        description:"Width for the \"other value\" prompt dialog.",
        name:"dialogWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"250",
        type:"attr",
        definingClass:"class:SelectOtherItem"
    },
    "type:EnterKeyEditAction":{
        ref:"type:EnterKeyEditAction",
        values:[
            {description:"end editing", value:"\"done\""},
            {description:"edit the next editable cell in the record", value:"\"nextCell\""},
            {description:"edit the same field in the next editable record", value:"\"nextRow\""},
            {description:"edit the first editable cell in next editable record",
             value:"\"nextRowStart\""}
        ],
        description:"What to do when a user hits enter while editing a cell",
        name:"EnterKeyEditAction",
        seeAlso:[],
        type:"type",
        groups:["editing"]
    },
    "method:RichTextEditor.richEditorSupported":{
        ref:"method:RichTextEditor.richEditorSupported",
        flags:"",
        description:" Does this browser support the full RichTextEditor feature set.\n Returns false for browsers in which some features are not natively supported\n - currently Safari and Opera.\n",
        name:"richEditorSupported",
        seeAlso:[],
        returns:{description:"false if this browser doesn't fully support RichTextEditing", type:"boolean"},
        type:"method",
        definingClass:"class:RichTextEditor"
    },
    "method:Toolbar.setButtons":{
        ref:"method:Toolbar.setButtons",
        flags:"",
        description:" Apply a new set of buttons to render in this toolbar as ${isc.DocUtils.linkForRef('attr:Toolbar.buttons')}.\n \n",
        name:"setButtons",
        seeAlso:[],
        params:[
            {description:"List of button object initializers", optional:true, name:"newButtons",
             defaultValue:null, type:"Array of objects"}
        ],
        type:"method",
        definingClass:"class:Toolbar"
    },
    "attr:ToolStrip.separatorClass":{
        ref:"attr:ToolStrip.separatorClass",
        flags:"IRW",
        description:"Class to create when the string \"separator\" appears in ${isc.DocUtils.linkForRef('attr:ToolStrip.members')}.",
        name:"separatorClass",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ToolStripSeparator\"",
        type:"attr",
        definingClass:"class:ToolStrip"
    },
    "method:Array.getProperty":{
        ref:"method:Array.getProperty",
        flags:"",
        description:" Return a new Array where the value of item i is the value of \"property\" of item i in this\n array.  If an item doesn't have that property or is null, return item will be null.\n\n",
        name:"getProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to look for", optional:false, name:"property",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"array of the values of property in each item of this list", type:"Array"},
        type:"method",
        definingClass:"object:Array",
        groups:["iteration"]
    },
    "type:FormItemType":{
        ref:"type:FormItemType",
        values:[
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:TextItem')}, unless the length of the field (as specified by ${isc.DocUtils.linkForRef('attr:DataSourceField.length')} attribute) is larger than the value\n specified by ${isc.DocUtils.linkForRef('attr:DynamicForm.longTextEditorThreshold')}, a\n ${isc.DocUtils.linkForRef('class:TextAreaItem')} is shown.",
             value:"\"text\""},
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:CheckboxItem')}",
             value:"\"boolean\""},
            {description:"Same as <code>text</code> by default.                     Consider setting editorType:${isc.DocUtils.linkForRef('class:SpinnerItem')}.",
             value:"\"integer\""},
            {description:"Same as <code>text</code> by default.                     Consider setting editorType:${isc.DocUtils.linkForRef('class:SpinnerItem')}.",
             value:"\"float\""},
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:DateItem')}",
             value:"\"date\""},
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:TimeItem')}",
             value:"\"time\""},
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:SelectItem')}.  Also true for any field that                   specifies a ${isc.DocUtils.linkForRef('attr:FormItem.valueMap')}.  \n                   Consider setting editorType:${isc.DocUtils.linkForRef('class:ComboBoxItem')}.",
             value:"\"enum\""},
            {description:"Same as <code>text</code>", value:"\"sequence\""},
            {description:"If ${isc.DocUtils.linkForRef('attr:DataSourceField.canEdit')}<code>:false</code> is set on the field, the value is rendered as a ${isc.DocUtils.linkForRef('class:LinkItem')}.  Otherwise the field is rendered as a\n ${isc.DocUtils.linkForRef('class:TextItem')}.",
             value:"\"link\""},
            {description:null, value:"\"image\""},
            {description:null, value:"\"imageFile\""},
            {description:"Rendered as a ${isc.DocUtils.linkForRef('class:UploadItem')}",
             value:"\"binary\""}
        ],
        description:"DynamicForms automatically choose the FormItem type for a field based on the\n <code>type</code> property of the field.  The table below describes the default FormItem\n chosen for various values of the <code>type</code> property.\n <P>\n You can also set ${isc.DocUtils.linkForRef('attr:FormItem.editorType','field.editorType')} to the classname of a\n ${isc.DocUtils.linkForRef('class:FormItem')} to override this default mapping.  You can alternatively override\n ${isc.DocUtils.linkForRef('method:DynamicForm.getEditorType')} to create a form with different rules for which\n FormItems are chosen.\n <P>\n If a field is marked ${isc.DocUtils.linkForRef('attr:DataSourceField.canEdit','canEdit:false')} it will generally use a\n ${isc.DocUtils.linkForRef('class:StaticTextItem')} rather than the FormItem type listed below (but see exceptions\n below).  If you want to create a read-only field, for example, a field that renders like\n a normal TextItem but cannot be changed, don't set canEdit:false, but implement a\n ${isc.DocUtils.linkForRef('method:FormItem.change')} that always returns false, thus cancelling all changes.\n <P>",
        name:"FormItemType",
        seeAlso:[
            {ref:"attr:FormItem.type", description:null},
            {ref:"type:FieldType", description:null}
        ],
        type:"type"
    },
    "attr:DataSourceField.javaClass":{
        ref:"attr:DataSourceField.javaClass",
        flags:"IR",
        description:"For use with the SmartClient server when populating Java Beans / POJOs based on data\n contained in a DSRequest, <code>javaClass</code> specifies the fully qualified Java\n className to be created and passed to the setter for the Java Bean Property with the same\n name as this field.   <code>javaClass</code> is used both when manually calling\n DataSource.setProperties() and when auto-populating POJO arguments of a ${isc.DocUtils.linkForRef('class:DMI')} method.\n <P>\n The Java class to create does not normally have to be specified: SmartClient will\n use Java reflection to inspect the type of argument expected by a setter method and will\n attempt conversion of inbound data to that type.  As described in the documentation for\n DataTools.setProperties(), this works for almost all typical cases.  However\n <code>field.javaClass</code> is useful for:\n <ul>\n <li> subobject of abstract or interface type: in this case Java Reflection is not sufficient\n to discover the concrete type that should be instantiated, and <code>javaClass</code> should be\n set instead.</li>\n <li> subobject of Collection or Map type, when Java generics are not used or the Collection\n member type or Map value type is abstract.  When Java generics are used (for example the\n setter takes an argument is of type Collection&lt;SomePOJO&gt; or Map&lt;KeyType,SomePOJO&gt;,\n SmartClient will automatically attempt to convert inbound data to the type of the members of\n the Collection or values of the Map.  Without generics, <code>javaClass</code> needs to be\n specified.  Note that <code>javaClass</code> will take precendence over generics if both \n are used. Also note that ${isc.DocUtils.linkForRef('attr:DataSourceField.javaCollectionClass')} can be specified if a\n particular Collection or Map type is needed, and ${isc.DocUtils.linkForRef('attr:DataSourceField.javaKeyClass')} can be\n specified for a field of type <code>java.util.Map</code>.</li>\n </ul>",
        name:"javaClass",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "attr:ListGrid.fixedFieldWidths":{
        ref:"attr:ListGrid.fixedFieldWidths",
        flags:"IRWA",
        description:"Should we horizontally clip cell contents, or allow columns to expand horizontally to\n show all contents?\n <P>\n If we allow columns to expand, the column width is treated as a minimum.\n <P>\n NOTE: the header does not automatically respond to expanded field widths",
        name:"fixedFieldWidths",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "attr:ImgTab.capSize":{
        ref:"attr:ImgTab.capSize",
        flags:"IRW",
        description:"How big are the end pieces by default",
        name:"capSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:ImgTab",
        groups:["appearance"]
    },
    "method:FormItem.keyUp":{
        ref:"method:FormItem.keyUp",
        flags:"",
        description:" StringMethod fired in response to a keyup while focused in this form item.\n\n",
        name:"keyUp",
        seeAlso:[],
        params:[
            {description:"Item over which the keyup occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"Pointer to the item's form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"},
            {description:"Name of the key pressed (Example: <code>\"A\"</code>, <code>\"Enter\"</code>)",
             optional:false, name:"keyName", defaultValue:null, type:"KeyName"}
        ],
        returns:{description:"return false to attempt to cancel the event.  Note for general purpose                   APIs for managing whether user input is allowed, use ${isc.DocUtils.linkForRef('method:FormItem.change')} \n                   or ${isc.DocUtils.linkForRef('method:FormItem.transformInput')} instead.",
         type:"boolean"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "method:TreeGrid.getEventRow":{
        ref:"method:TreeGrid.getEventRow",
        flags:"",
        description:" Returns the row number of the most recent mouse event.\n",
        name:"getEventRow",
        seeAlso:[],
        params:[
            {description:"optional y-coordinate to obtain row number, in lieue of the y                        coordinate of the last mouse event",
             optional:true, name:"y", defaultValue:null, type:"number"}
        ],
        returns:{description:"row number, or -2 if beyond last drawn row", type:"number"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["events"]
    },
    "attr:Canvas.canDropBefore":{
        ref:"attr:Canvas.canDropBefore",
        flags:"IRWA",
        description:"When explicitly set to false, disallows drop before this member in the Layout.",
        name:"canDropBefore",
        seeAlso:[
            {ref:"class:Layout", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["layoutMember"]
    },
    "attr:BlurbItem.colSpan":{
        ref:"attr:BlurbItem.colSpan",
        flags:"IRW",
        description:"By default, texts span all remaining columns",
        name:"colSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"\"*\"",
        type:"attr",
        definingClass:"class:BlurbItem",
        groups:["appearance"]
    },
    "classMethod:XMLTools.serializeToString":{
        ref:"classMethod:XMLTools.serializeToString",
        flags:"",
        description:" Takes an XMLDocument and returns it as a String.\n <P>\n This method is not supported on the Safari web browser versions prior to 3.0.3.\n\n",
        name:"serializeToString",
        seeAlso:[],
        params:[
            {description:"XML document to apply the transform to", optional:false,
             name:"inputDocument", defaultValue:null, type:"XMLDocument"}
        ],
        returns:{description:"XML document as a String", type:"String"},
        type:"classMethod",
        definingClass:"class:XMLTools",
        groups:["xmlTransform"]
    },
    "attr:DynamicForm.errorItemCellStyle":{
        ref:"attr:DynamicForm.errorItemCellStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is false we show all errors for the form item in \n a single item rendered at the top of the form.<br>\n This attribute specifies the cellStyle to apply to this item.",
        name:"errorItemCellStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"formCellError\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:ServerObject.attributeName":{
        ref:"attr:ServerObject.attributeName",
        flags:"IR",
        description:"Specifies the name of the attribute by which to look up the DMI instance.  This attribute is\n consulted only when the value of ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')} is <code>\"attribute\"</code>.",
        name:"attributeName",
        seeAlso:[
            {ref:"attr:ServerObject.attributeScope", description:null},
            {ref:"attr:ServerObject.lookupStyle", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "group:networkPerformance":{
        ref:"group:networkPerformance",
        title:"Network Performance",
        treeLocation:["Optional Modules"],
        refs:[],
        description:"This section describes various methods for optimizing delivery of your SmartClient-based\n application to the browser.  The various techniques discussed here should enable you to\n drastically reduce the time to load your SmartClient based application and to cut down on\n bandwidth costs and server-side CPU usage.\n <P>\n <i>Note that in addition to these network performance considerations, you should also be aware\n of the recommended application design practices discussed in ${isc.DocUtils.linkForRef('group:smartArchitecture')}.</i>\n <P>\n\n There are three main ways of improving the performance of your application:\n <p>\n <u><b>Compression</b></u>\n <p>\n You'll want to deliver as much of your application assets compressed as possible.  For a start, the\n SmartClient modules come pre-compressed.  The ${isc.DocUtils.linkForRef('group:compression')} section describes how\n to make sure that SmartClient modules and your application logic are delivered compressed\n and how to enable dynamic compression of your dynamic content.\n <p>\n Compression drastically reduces transfer times to the browser.  Compression ratios for css\n and javascript files can be as high as 8:1.\n\n <p>\n <u><b>Caching</b></u>\n <p>\n The ${isc.DocUtils.linkForRef('group:caching')} section deals with delivering as much content as possible with\n caching headers.  Setting these headers allows the end-user's browser and any intermediary\n proxies to keep a local copy of the file across browser reloads, ensuring the subsequent\n visits require almost no fetches from the server.  Eliminating these fetches is also\n important for the reasons explained under File Assembly below.\n \n <p>\n <u><b>File Assembly</b></u>\n <p>\n Modern browsers limit the number of HTTP connections they use to fetch the assets required\n to render your pages - typically to just 2 connections.  As a result, connection latency can\n play a much higher role than available bandwidth and the two connection limit effectively\n drastically limits the user's usable bandwidth.  For this reason, it's important to minimize\n the number of HTTP requests made to the server.  The File Assembly services make it easy to\n deliver multiple css and javascript file via one HTTP response.  Assembling files together\n also helps to increase the compression ratio because the commpression engine has more\n redundant data to work with.\n\n <p>\n <u><b>FileLoader</b></u>\n <p>\n The FileLoader is a standalone client-side module that allows you to performing caching or\n loading of various resources in the background on any page.  The typical usage pattern is to\n use this module to pre-cache SmartClient modules, skin images, and application logic while\n the user is, for example filling in a login form.  See the ${isc.DocUtils.linkForRef('class:FileLoader')} documentation\n on usage.\n\n <p>\n <u><b>SSL Considerations</b></u>\n <p>\n Some quirks to be aware of when optimizing for pages servied via HTTPS:\n <ul>\n <li>If your top-level https page loads an image via http, IE will pop a warning to the user\n asking if it's ok to serve the \"unsecure\" content.  Firefox does not pop an alert, bug shows\n a different lock icon to indicate that not all content is served from a secure server.\n <li>Firefox will not write images fetched via SSL to disk, even if expires headers are set.\n But it will cache them in memory for the duration of the browser session.  This makes the\n use of the ${isc.DocUtils.linkForRef('class:FileLoader')} even more important for SSL applications.  IE will cache SSL\n content across browser restarts.\n <li>If your only concern is with actual security and not with perception, it makes a lot of\n sense to serve the top-level page via plain HTTP and then use the SmartClient RPC mechanism\n to fetch sensitive data with no-cache headers via HTTPS.  Unfortunately this almost never\n practical because most users look for the secure lock icon on the top-level page and may\n become worried that the application is not secure if they don't see it.\n </ul>\n\n <p>\n <u><b>Other Tips</b></u>\n <p>\n 1.  Modern applications often use a number of cookies to keep track of session state,\n application state, etc.  If you don't limit the paths these cookies are served for, you can\n easily end up with these cookies being sent to the server with every HTTP request as\n part of the HTTP headers.  This can severely limit bandwidth where a large number of\n requests are made - typically for skin images.  Also note that most technologies used by\n end-users to connect to the internet such as DSL and Cable are half-duplex, meaning that\n downstream must pause for upstream traffic to be sent and these connections also are\n asymmetric, allowing much faster download speeds than upload speeds.  As a result, the\n limited upstream capability of most connections ends up limiting the download bandwidth by\n delayng the download of other assets required to render the page.\n <p>\n 2.  Whenever possible, don't sent the \"Cache-Control\" header to \"private\" - this tells\n intermediary proxies to not allow caching of the specified content, eliminating that cache\n as a useful intermediary for other users behind it.\n <p>\n 3.  For a good overview of performance considerations, see this post:\n <a href='http://www.die.net/musings/page_load_time/' onclick=\"window.open('http://www.die.net/musings/page_load_time/');return false;\">http://www.die.net/musings/page_load_time/</a>",
        name:"networkPerformance",
        seeAlso:[],
        type:"group"
    },
    "classMethod:XMLTools.transformNodes":{
        ref:"classMethod:XMLTools.transformNodes",
        flags:"",
        description:" Apply an XSLT Stylesheet to an XML Document.\n <P>\n This method cannot currently be supported on the Safari web browser versions prior to\n 3.0.3.\n\n",
        name:"transformNodes",
        seeAlso:[],
        params:[
            {description:"XML document to apply the transform to", optional:false,
             name:"inputDocument", defaultValue:null, type:"XMLDocument"},
            {description:"XSLT stylesheet to use for transform", optional:false, name:"styleSheet",
             defaultValue:null, type:"XMLDocument"}
        ],
        returns:{description:"stylesheet output", type:"String"},
        type:"classMethod",
        definingClass:"class:XMLTools",
        groups:["xmlTransform"]
    },
    "attr:OperationBinding.values":{
        ref:"attr:OperationBinding.values",
        flags:"IR",
        description:"A list of ${isc.DocUtils.linkForRef('object:DSRequestModifier')}s that will be used to modify the values object of each \n ${isc.DocUtils.linkForRef('class:DSRequest')} that uses this operationBinding.",
        name:"values",
        seeAlso:[
            {ref:"attr:OperationBinding.criteria", description:null}
        ],
        valueType:"List of DSRequestModifier",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "classAttr:RPCResponse.STATUS_LOGIN_INCORRECT":{
        ref:"classAttr:RPCResponse.STATUS_LOGIN_INCORRECT",
        flags:"R",
        description:"Indicates that the RPC has been intercepted by an authenticator that requires the user\n to log in.",
        name:"STATUS_LOGIN_INCORRECT",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-5",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:ListGrid.dateInputFormat":{
        ref:"attr:ListGrid.dateInputFormat",
        flags:"IRWA",
        description:"If this is an editable listGrid, this property will specify the \n ${isc.DocUtils.linkForRef('attr:DateItem.inputFormat','inputFormat')} applied to editors for fields of type \n <code>\"date\"</code>",
        name:"dateInputFormat",
        seeAlso:[
            {ref:"attr:ListGrid.dateFormatter", description:null}
        ],
        valueType:"DateInputFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:CubeGrid.setEditValue":{
        ref:"method:CubeGrid.setEditValue",
        flags:"A",
        description:"\n Set the edit value for some cell in the cube grid.<p>\n Note that cubeGrids display one record per cell - the value passed in should be the \n desired edit value for the ${isc.DocUtils.linkForRef('attr:CubeGrid.valueProperty')} of the record.\n\n",
        name:"setEditValue",
        seeAlso:[],
        params:[
            {description:"Row number", optional:false, name:"rowNum", defaultValue:null, type:"number"},
            {description:"Column number", optional:false, name:"colNum", defaultValue:null,
             type:"number"},
            {description:"New value for the record", optional:false, name:"value", defaultValue:null,
             type:"any"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "method:ResultTree.loadChildren":{
        ref:"method:ResultTree.loadChildren",
        flags:"",
        description:" 			Load the children of a given node.\n\n 			This implementation does nothing other than mark the node as loaded\n",
        name:"loadChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"},
            {description:"Optional callback (stringMethod) to fire when loading                       completes. Has a single param <code>node</code> - the node whose \n                      children have been loaded, and is fired in the scope of the Tree.",
             optional:true, name:"callback", defaultValue:null, type:"callback"}
        ],
        type:"method",
        definingClass:"class:ResultTree",
        groups:["loadState"]
    },
    "method:DataSource.getXMLRequestBody":{
        ref:"method:DataSource.getXMLRequestBody",
        flags:"A",
        description:" Get the XML to be posted to the dataURL based on the passed DSRequest.\n <P>\n This API is intended to be overriden in order to integrate with web services that\n expect XML messages rather than simple HTTP parameters, but lack a\n WSDL description.  For WSDL-described web services, having loaded the service\n description, SmartClient knows the correct XML message structure, so customization is\n best done by modifying the JavaScript data that is used to form the message.\n\n",
        name:"getXMLRequestBody",
        seeAlso:[
            {ref:"classMethod:XMLTools.loadWSDL",
             description:"for integration with WSDL-described web services"}
        ],
        params:[
            {description:"the request to encode as an XML message.", optional:false, name:"dsRequest",
             defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:"the entire XML request as a String, including SOAP envelope if SOAP is used",
         type:"String"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:Canvas.hoverStyle":{
        ref:"attr:Canvas.hoverStyle",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, this property can be used to specify the\n css style to apply to the hover canvas.",
        name:"hoverStyle",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customHovers')}"],
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:FormItem.disabled":{
        flags:"IRW",
        type:"attr",
        ref:"attr:FormItem.disabled",
        description:"Whether this item is disabled.  Can be updated at runtime via the <code>setDisabled()</code>\n method.  Note that if the widget containing this formItem is disabled, the formItem will\n behave in a disabled manner regardless of the setting of the item.disabled property.\n <p>\n Note that not all items can be disabled, and not all browsers show an obvious disabled style\n for native form elements.",
        name:"disabled",
        setter:["setDisabled"],
        valueType:"boolean",
        seeAlso:[
            {ref:"method:FormItem.setDisabled", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('fieldEnableDisable')}"],
        defaultValue:"false",
        groups:["appearance"],
        definingClass:"class:FormItem"
    },
    "attr:ButtonItem.icon":{
        ref:"attr:ButtonItem.icon",
        flags:"IR",
        description:"Optional icon image to display on the button for this item.  See ${isc.DocUtils.linkForRef('attr:Button.icon')}.",
        name:"icon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ButtonItem"
    },
    "attr:TileLayout.tileWidth":{
        ref:"attr:TileLayout.tileWidth",
        flags:"IR",
        description:"Width of each tile in pixels.  See ${isc.DocUtils.linkForRef('attr:TileLayout.tileSize')}.\n If ${isc.DocUtils.linkForRef('type:LayoutPolicy')} is \"fit\", ${isc.DocUtils.linkForRef('attr:TileLayout.expandMargins')} is false, ${isc.DocUtils.linkForRef('attr:TileLayout.tilesPerLine')} is set,\n ${isc.DocUtils.linkForRef('type:Orientation')} is \"horizontal\",\n and tileWidth is not set, tileWidth will be computed automatically based on ${isc.DocUtils.linkForRef('attr:TileLayout.tilesPerLine')}.",
        name:"tileWidth",
        setter:["setTileWidth"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["sizing"]
    },
    "attr:DateChooser.yearMenuStyle":{
        ref:"attr:DateChooser.yearMenuStyle",
        flags:"IR",
        description:"Style for the pop-up year menu.",
        name:"yearMenuStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"dateChooserYearMenu\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "group:metadataImport":{
        ref:"group:metadataImport",
        title:"Metadata Import",
        treeLocation:["Client Reference/Data Binding"],
        refs:[],
        description:"In SmartClient, metadata is expressed through ${isc.DocUtils.linkForRef('class:DataSource','DataSources')}, which in turn\n drive ${isc.DocUtils.linkForRef('interface:DataBoundComponent','DataBoundComponents')}.  If you have existing metadata, there\n are several possible approaches to transforming it to SmartClient DataSources, either one\n time or on the fly.\n <P>\n There are two possible targets for metadata import: XML format or JavaScript format.\n The XML format is more general purpose, since the ISC server can transform it to JavaScript\n via the ${isc.DocUtils.linkForRef('group:loadDSTag','loadDS tag')}, and DataSources in XML format can be used by the\n ISC server for server-side validation (this split is covered in more detail under\n ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','Data Source Declaration')}).\n <P>\n You may also transform your metadata dynamically (while the application is running in\n production) or statically (one time ever or at packaging time).  Generally for a static\n or dynamic transform targetting JavaScript format you will want to produce one .js file\n containing all your DataSource definitions, to be loaded by your application via a normal\n &lt;SCRIPT SRC&gt; tag.  For a static transform targetting XML format, you will want to\n produce a series of .ds.xml files and place them in the directories expected by the ISC\n server (see ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','DataSource Declaration')}).  Staticly generated\n XML DataSources can be delivered to the browser as a single .js file via a .jsp containing\n several ${isc.DocUtils.linkForRef('group:loadDSTag','<code>loadDS</code> tags')}.\n <P>\n If you want to do dynamic transform targetting XML format and use ISC server-side\n validation, the server-side API DataSource.fromXML() can be used to create a DataSource\n dynamically from XML, so that you can then call DataSource.validate().  Either the XML\n DataSource definition or the live DataSource itself can be passed to the server-side API\n XML.toJS() to produce JavaScript.\n <P>\n How to actually produce JavaScript or XML DataSource definitions from your existing metadata\n depends on the format of your metadata.\n <P>\n <b>XML Schema</b>\n <P>\n The method ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')} and the \n ${isc.DocUtils.linkForRef('group:loadXMLSchemaTag','loadXMLSchema JSP tag')} provide dynamic transform of XML Schema\n to JavaScript.  This is essentially accomplished by running\n <code>isomorphic/system/schema/schemaTranslator.xsl</code> on the XML schema file to produce\n XML DataSource definitions, and then translating those to JavaScript.  You can run the\n <code>schemaTranslator</code> stylesheet using any standard XSLT processor and capture the\n XML output.\n <P>\n <b>Java Beans</b>\n <P>\n Metadata available via Java's \"reflection\" APIs allows a basic DataSource to be generated\n from Java beans.  Sample Java code can be found in\n <code>examples/server_integraton/DataSourceGenerator.java</code>.  See also the last section\n on this page for an automatic option for generating DataSource definitions via Reflection.\n <P>\n <b>Other XML formats</b>\n <P>\n If you are familiar with XSLT or other XML transform languages, you\n could use it to do an XML to XML transform, and then use XML.toJS() to get to JavaScript.\n <P>\n <b>Schema represented as Java Objects</b>\n <P>\n If you are targetting XML, hand-coded generation of DataSource XML is straightforward, and\n from XML you can use XML.toJS() to get to JavaScript.\n <P>\n <b>Database Schema, Hibernate mappings or Java class definitions</b>\n <P>\n <em><b>Note: This is an Enterprise feature.</b>  It is not available to users of the LGPL,\n Pro or Power Editions of SmartClient.  Please see the <a href=http://smartclient.com/licensing>\n licensing page</a> for details.</em>\n <p>\n If you have existing database tables, Hibernate classes or Java POJOs that follow the Javabean\n semantics, you can use these to automatically produce basic XML DataSource definitions.  \n We provide the Batch DataSource Generator, a supported tool that makes use of SmartClient \n Server APIs to generate XML DataSource definitions and save them as <code>.ds.xml</code> files \n in your <code>shared/ds</code> folder.  The source for the tool is provided in \n <code>tools/batchDSGenerator.jsp</code>.  \n <p>\n If your metadata source is existing database tables, the Batch DS Generator can also extract \n data from those tables and save it as test data in your <code>shared/ds/test_data</code> \n folder.  With this option, you have a complete round-trip facility from a populated database \n table to a client-side DataSource that does not require a server. The database\n table can be recreated and repopulated from the XML files using the SmartClient Admin \n Console, giving you a portable schema and dataset that can be used to initialize any database.\n <p>\n The tool is also able to output the DataSource definitions and test data in Javascript format\n rather than XML, for direct inclusion in client-side programs.  In this mode, it will create \n <code>.ds.js</code> and <code>.data.js</code> files, rather than files with a <code>.xml</code>\n extension.\n <p>\n To use the tool, direct your web browser to the <code>tools/batchDSGenerator.jsp</code> \n resource of your SmartClient server.  If you pass no parameters to the JSP, a SmartClient \n window will be shown, prompting you for the various generation parameters.  Alternatively, \n you can effect a true batch operation by passing these parameters in directly.  The source \n code is extensively commented and explains the meanings of all the parameters should you wish\n to take this latter approach.\n <p>\n Example simple usage (via the UI):<br/>\n <code>http://localhost:8080/tools/batchDSGenerator.jsp</code>\n <p>\n Example batch usage (providing the parameters by hand):<br/>\n <code>http://localhost:8080/tools/batchDSGenerator.jsp?dbName=Mysql&tableName=foo&tableName=bar&className=com.bar.foo.SomeClass&overwrite=true</code>",
        name:"metadataImport",
        seeAlso:[],
        type:"group"
    },
    "attr:FormItem.titleStyle":{
        ref:"attr:FormItem.titleStyle",
        flags:"IRW",
        description:"Base CSS class name for a form item's title.",
        name:"titleStyle",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"formTitle\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "attr:SelectItem.showPickerIcon":{
        ref:"attr:SelectItem.showPickerIcon",
        flags:"IRW",
        description:"Should we show a special 'picker' icon for this form item. Picker icons are customizeable\n via ${isc.DocUtils.linkForRef('attr:SelectItem.pickerIconProperties')}. By default they will be rendered inside the \n Form Item's \"control box\" area, and will call ${isc.DocUtils.linkForRef('method:SelectItem.showPicker')} when clicked.",
        name:"showPickerIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:ListGrid.removeIcon":{
        ref:"attr:ListGrid.removeIcon",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} is enabled, default icon to show in\n the auto-generated field that allows removing records.",
        name:"removeIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/actions/remove.png\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:DataSource.dataURL":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.dataURL",
        description:"Default URL to contact to fulfill all DSRequests.  Can also be set on a\n per-operationType basis via ${isc.DocUtils.linkForRef('attr:OperationBinding.dataURL')}.\n <P>\n NOTE: Best practice is to use the same <code>dataURL</code> for all DataSources which\n fulfill DSRequests via the server-side RPCManager API.  Otherwise, cross-DataSource\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','operation queueing')} will not be possible.",
        name:"dataURL",
        valueType:"URL",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('jsonDataSource')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration"],
        definingClass:"class:DataSource"
    },
    "classMethod:EventHandler.getDragRect":{
        ref:"classMethod:EventHandler.getDragRect",
        flags:"",
        description:" During a drag with ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance','dragAppearance')} of either \"target\" or\n \"outline\", returns the page-relative coordinates of whatever element is being dragged.\n <P>\n Calling this method allows you to write drag and drop logic that works identically even if\n <code>dragAppearance</code> is subsequently changed.\n\n",
        name:"getDragRect",
        seeAlso:[],
        returns:{description:"global (page-relative) coordinates and size of the dragged element, as a                       4-element array [left,top,width,height], or null if not dragging",
         type:"Rect"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["dragdrop"]
    },
    "method:ColumnTree.nodeSelected":{
        ref:"method:ColumnTree.nodeSelected",
        flags:"",
        description:" Called when a node is selected in any column.  Default behavior is to show the next level\n of the tree in a column to the right of the current column.\n <P>\n The new column will be created if it is not already showing.  Any columns further to the\n right, showing deeper levels of the tree, will be removed.\n",
        name:"nodeSelected",
        seeAlso:[],
        params:[
            {description:"nodethe node that was selected", optional:true, name:"TreeNode",
             defaultValue:null, type:null}
        ],
        returns:{description:"override and return false to cancel the default action", type:"boolean"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "classMethod:RPCManager.handleTransportError":{
        ref:"classMethod:RPCManager.handleTransportError",
        flags:"",
        description:" Method to handle server error responses to submitted transactions.\n When the server responds to a submitted transaction with an HTTP error code this method will\n be called before any individual response callbacks are fired, regardless of whether\n ${isc.DocUtils.linkForRef('attr:RPCRequest.willHandleError')} was specified on the submitted request[s].\n <P>\n This provides the developer with an opportunity to handle a server error by (for example)\n suspending and resubmitting the transaction before any other handling occurs.\n <P>\n The default implementation takes no action - by default transport errors \n are handled via ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')}, or by the standard request callback\n methods, depending on request.willHandleError. \n To perform custom handing for transport errors this classMethod may be overridden as\n follows\n <pre>\n     isc.RPCManager.addClassProperties({\n         handleTransportError : function (transactionNum, status, httpResponseCode, httpResponseText) \n         {\n                .. custom handling .. \n         }\n     })\n </pre>\n <P>\n Note: This method only applies to operations submitted via\n ${isc.DocUtils.linkForRef('type:RPCTransport','XMLHttpRequest')} - it is not possible to provide similar error handling\n for other transports.\n\n",
        name:"handleTransportError",
        seeAlso:[],
        params:[
            {description:"The submitted client-server transaction number", optional:false,
             name:"transactionNum", defaultValue:null, type:"integer"},
            {description:"The RPCResponse status code", optional:false, name:"status",
             defaultValue:null, type:"integer"},
            {description:"The HTTP Response code reported by the server", optional:false,
             name:"httpResponseCode", defaultValue:null, type:"integer"},
            {description:"The raw HTTP Response text", optional:false, name:"httpResponseText",
             defaultValue:null, type:"text"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "classAttr:Hover.leftOffset":{
        ref:"classAttr:Hover.leftOffset",
        flags:"RW",
        description:"When positioining the hover canvas, this will be the default left offset from the \n mousepointer, if no explicit position was passed to the ${isc.DocUtils.linkForRef('classMethod:Hover.show')} method",
        name:"leftOffset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"15",
        type:"classAttr",
        definingClass:"class:Hover"
    },
    "attr:TileGrid.autoFetchTextMatchStyle":{
        ref:"attr:TileGrid.autoFetchTextMatchStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TileGrid.autoFetchData')} is <code>true</code>, this attribute allows the developer to\n specify a textMatchStyle for the initial ${isc.DocUtils.linkForRef('method:TileGrid.fetchData')} call.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"\"substring\"",
        type:"attr",
        definingClass:"class:TileGrid",
        groups:["databinding"]
    },
    "group:databinding":{
        ref:"group:databinding",
        title:"DataBinding",
        refs:["attr:PickListMenu.dataProperties", "attr:PickList.showOptionsFromDataSource", 
         "attr:SelectItem.showOptionsFromDataSource","attr:ComboBoxItem.showOptionsFromDataSource", "attr:MultiView.dataSource", 
         "attr:Calendar.autoFetchData","attr:Calendar.autoFetchTextMatchStyle", "attr:DetailViewer.dataFetchMode", 
         "attr:ListGrid.fields","attr:ListGrid.dataSource", "attr:ListGrid.saveLocally", 
         "attr:ListGrid.useAllDataSourceFields","attr:ListGrid.showDetailFields", "attr:ListGrid.dataProperties", 
         "attr:TreeGrid.dataSource","attr:TreeGrid.loadDataOnDemand", "attr:TreeGrid.treeRootValue", "attr:TreeGrid.fields", 
         "attr:TileGrid.dataSource","attr:TileGrid.autoFetchData", "attr:TileGrid.autoFetchTextMatchStyle", 
         "attr:ColumnTree.dataSource","attr:ColumnTree.autoFetchData", "attr:ColumnTree.autoFetchTextMatchStyle", 
         "attr:ColumnTree.loadDataOnDemand","attr:ColumnTree.fields", "attr:DataBoundComponent.dataSource", 
         "attr:DataBoundComponent.dataFetchMode","attr:DataBoundComponent.dataPageSize", "attr:DataBoundComponent.fields", 
         "attr:DataBoundComponent.useAllDataSourceFields","attr:DataBoundComponent.showHiddenFields", "attr:DataBoundComponent.showDetailFields", 
         "attr:DataBoundComponent.showComplexFields","attr:ListGrid.autoFetchData", "attr:DynamicForm.autoFetchData", 
         "attr:DynamicForm.autoFetchTextMatchStyle"],
        description:"DataBinding means the automatic, highly customizable process of 'binding' a UI component\n to a DataSource, so that a UI component displays, edits and saves DataSource records\n using appropriate formatters, editors, validation rules, and persistance logic.",
        name:"databinding",
        seeAlso:[
            {ref:"interface:DataBoundComponent", description:null}
        ],
        type:"group"
    },
    "method:ResultSet.willFetchData":{
        ref:"method:ResultSet.willFetchData",
        flags:"",
        description:" Will changing the criteria for this resultSet require fetching new data from the server, \n or can the new criteria be satisfied from data already cached on the client?<br>\n Second <code>textMatchStyle</code> parameter determines whether a change of text-match style\n will require a server fetch - for example if filter is being changed between\n an exact match (from e.g: ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}) and a substring match \n (from e.g: ${isc.DocUtils.linkForRef('method:ListGrid.filterData')}).<br>\n This method can be used to determine whether ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} or \n ${isc.DocUtils.linkForRef('method:ListGrid.filterData')} would cause a server side fetch when passed a certain set of \n criteria.\n",
        name:"willFetchData",
        seeAlso:[],
        params:[
            {description:"new criteria to test.", optional:false, name:"newCriteria", defaultValue:null,
             type:"Criteria"},
            {description:"New text match style. If not passed, assumes       textMatchStyle will not be modified.",
             optional:true, name:"textMatchStyle", defaultValue:null, type:"TextMatchStyle"}
        ],
        returns:{description:"true if server fetch would be required to satisfy new criteria.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:DetailViewer.headerStyle":{
        ref:"attr:DetailViewer.headerStyle",
        flags:"IRW",
        description:"CSS style for a header",
        name:"headerStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detailHeader\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "attr:NativeCheckboxItem.textBoxStyle":{
        ref:"attr:NativeCheckboxItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class applied to this item's title text (rendered next to the checkbox element).",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"labelAnchor\"",
        type:"attr",
        definingClass:"class:NativeCheckboxItem",
        groups:["appearance"]
    },
    "method:Slider.valueIsChanging":{
        ref:"method:Slider.valueIsChanging",
        flags:"A",
        description:" Call this method in your ${isc.DocUtils.linkForRef('method:Slider.valueChanged')} handler to determine whether the\n value change is due to an ongoing drag interaction (true) or due to a thumb-release,\n mouse click, keypress, or programmatic event (false). You may choose to execute temporary or\n partial updates while the slider thumb is dragged, and final updates or persistence of the value\n in response to the other events.\n\n",
        name:"valueIsChanging",
        seeAlso:[],
        returns:{description:"true if user is still dragging the slider thumb, false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Slider"
    },
    "classAttr:RPCManager.promptCursor":{
        ref:"classAttr:RPCManager.promptCursor",
        flags:"IRW",
        description:"Controls the default cursor shown when ${isc.DocUtils.linkForRef('classAttr:RPCManager.promptStyle')} is set to\n <code>\"cursor\"</code>.  Overrideable by ${isc.DocUtils.linkForRef('attr:RPCRequest.promptCursor')}.\n <p>\n In Safari, IE 5.5 and Firefox 1.0 the default value is \"wait\", on all other platforms it is\n \"progress\".  The reason for this split is that the above-mentioned browsers do not support\n CSS2.1 - which is required for the \"progress\" cursor type.",
        name:"promptCursor",
        seeAlso:[
            {ref:"attr:RPCRequest.promptCursor", description:null}
        ],
        valueType:"String",
        defaultValue:"browser-dependent",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["rpcPrompt"]
    },
    "classMethod:FileLoader.cacheShadows":{
        ref:"classMethod:FileLoader.cacheShadows",
        flags:"",
        description:"\n Caches a set of shadow images at various depths.  For example this code:\n <pre>\n isc.FileLoader.cacheShadows(\"/isomorphic/skins/SmartClient/images/shared/shadows\", \"5\");\n </pre>\n Will cause the following images to be cached:\n <pre>\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_TL.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_T.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_TR.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_L.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_R.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_BL.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_B.png\n /isomorphic/skins/SmartClient/images/shared/shadows/ds5_BR.png\n </pre>\n\n",
        name:"cacheShadows",
        seeAlso:[],
        params:[
            {description:"Base directory containing the shadow images.", optional:false,
             name:"baseDir", defaultValue:null, type:"String"},
            {description:"List of depths to load. These values corresponds to the Canvas                                    attribute ${isc.DocUtils.linkForRef('attr:Canvas.shadowDepth')}.   This can either be\n                                    a string with comma separated depths or an Array of depths.",
             optional:false, name:"depths", defaultValue:null, type:"String or Array"},
            {description:"Name of the base shadow image from which list of URLs is                                    generated.  If not specified, this defaults to \"ds.png\".",
             optional:true, name:"baseShadowImage", defaultValue:null, type:"String"},
            {description:"Optional code to execute when the last of the specified URLs                                      has completed caching.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "classMethod:RPCManager.send":{
        ref:"classMethod:RPCManager.send",
        flags:"",
        description:"\n This method is a convenience wrapper on <code>RPCManager.sendRequest()</code> - it calls\n through to sendRequest().\n\n",
        name:"send",
        seeAlso:[
            {ref:"classMethod:RPCManager.sendRequest", description:null},
            {ref:"class:RPCRequest", description:null}
        ],
        params:[
            {description:"data to be passed to the server", optional:false, name:"data",
             defaultValue:null, type:"any"},
            {description:"method to call on RPC completion", optional:true, name:"callback",
             defaultValue:null, type:"RPCCallback"},
            {description:"object literal containing any additional properties                                        you want to set - these will be applied to the\n                                        RPCRequest object that will be auto-created for you.",
             optional:true, name:"requestParams", defaultValue:null, type:"Object"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "method:Class.init":{
        ref:"method:Class.init",
        flags:"A",
        description:"	\n Initialize a new instance of this Class.  This method is called automatically by\n ${isc.DocUtils.linkForRef('classMethod:Class.create')}.  \n <p>\n Override this method to provide initialization logic for your class.  If your class is\n a subclass of a UI component (i.e. descendant of ${isc.DocUtils.linkForRef('class:Canvas')}), override\n ${isc.DocUtils.linkForRef('method:Canvas.initWidget')} instead. \n\n",
        name:"init",
        seeAlso:[],
        params:[
            {description:"All arguments initially passed to ${isc.DocUtils.linkForRef('classMethod:Class.create')}",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:DynamicForm.encoding":{
        ref:"attr:DynamicForm.encoding",
        flags:"IRWA",
        description:"encoding for the form, use MULTIPART_ENCODING for file upload forms",
        name:"encoding",
        seeAlso:[],
        valueType:"Encoding",
        defaultValue:"DynamicForm.NORMAL",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "attr:DataBoundComponent.dragTrackerStyle":{
        ref:"attr:DataBoundComponent.dragTrackerStyle",
        flags:"IRW",
        description:"CSS Style to apply to the drag tracker when dragging occurs on this component.",
        name:"dragTrackerStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"gridDragTracker\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:WebService.getSchema":{
        ref:"method:WebService.getSchema",
        flags:"",
        description:" Get the schema definition of any complexType or element of complexType defined in any\n &lt;schema&gt; blocks in the WSDL file this WebService represents.\n \n",
        name:"getSchema",
        seeAlso:[],
        params:[
            {description:"name of type or element", optional:false, name:"schemaName",
             defaultValue:null, type:"String"},
            {description:"optional type of schema to return, either \"element\" for                              xs:element definitions only or \"type\" for xs:complexType\n                              definitions.  If unspecified, either will be returned,\n                              with types preferred if names collide",
             optional:true, name:"schemaType", defaultValue:null, type:"String"}
        ],
        returns:{description:"requested schema", type:"DataSource"},
        type:"method",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "method:ValuesManager.getMemberValues":{
        ref:"method:ValuesManager.getMemberValues",
        flags:"",
        description:" Returns the subset of this valuesManager's values associated with some member form.\n  \n",
        name:"getMemberValues",
        seeAlso:[],
        params:[
            {description:"ID of the member form for which we want to retrieve the values.",
             optional:false, name:"ID", defaultValue:null, type:"string"}
        ],
        returns:{description:"a map of the values for the appropriate member form.", type:"object"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "attr:BatchUploader.grid":{
        ref:"attr:BatchUploader.grid",
        flags:"IR",
        description:"Grid which will show a preview of data to be uploaded, with errors flagged",
        name:"grid",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:DateItem.getSelectionRange":{
        ref:"method:DateItem.getSelectionRange",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is true, falls through to standard\n ${isc.DocUtils.linkForRef('method:FormItem.getSelectionRange')} implementation on this items freeform text entry field.\n Otherwise has no effect.\n",
        name:"getSelectionRange",
        seeAlso:[],
        returns:{description:"2 element array indicating start/end character index of current selection  within our text entry field. Returns null if this item is undrawn or doesn't have focus.",
         type:"array"},
        type:"method",
        definingClass:"class:DateItem"
    },
    "attr:FormItemIcon.showFocusedWithItem":{
        ref:"attr:FormItemIcon.showFocusedWithItem",
        flags:"IRWA",
        description:"If this icon will be updated to show focus (see ${isc.DocUtils.linkForRef('attr:FormItemIcon.showFocused')}, \n ${isc.DocUtils.linkForRef('attr:FormItem.showFocusedIcons')}), this property governs whether the focused state should\n be shown when the item as a whole recieves focus or just if the icon recieves focus.\n If this property is unset, default behavior is to show focused state when the item\n recieves focus.",
        name:"showFocusedWithItem",
        seeAlso:[
            {ref:"attr:FormItem.showFocusedIcons", description:null},
            {ref:"attr:FormItemIcon.showFocused", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "attr:PickList.showAllOptions":{
        ref:"attr:PickList.showAllOptions",
        flags:"IR",
        description:"If true, even non-matching options will be shown, with configurable \n ${isc.DocUtils.linkForRef('attr:PickList.separatorRows','separator rows')} in between.  Not valid for\n ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource','databound pickLists')}.",
        name:"showAllOptions",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:DynamicForm.validationURL":{
        ref:"attr:DynamicForm.validationURL",
        flags:"IRW",
        description:"validationURL can be set to do server-side validation against a different URL from where\n the form will ultimately save, as part of an incremental upgrade strategy for Struts and\n Struts-like applications.  \n <P>\n If set, calling ${isc.DocUtils.linkForRef('method:DynamicForm.submit')} causes an RPC to be sent to this URL to\n perform server-side validation of the form values.  If the validation fails, the\n validation errors returned by the server are rendered in the form.  If the validation\n succeeds, the form is submitted to the URL specified by ${isc.DocUtils.linkForRef('attr:DynamicForm.action')}.\n <p>\n The form values are available on the server as request parameters (just like a normal form\n submit) and also as the values of a DSRequest sent as an RPC alongside the normal\n submit.\n <p>\n The expected response to this request is a DSResponse sent via the RPC mechanism.  If\n validation is successful, an empty response with the STATUS_SUCCESS status code is\n sufficient.  If there are validation errors, the DSResponse should have the status set to \n STATUS_VALIDATION_ERROR and the errors should be set on the response via the\n addError()/setErrorReport() API on DSResponse.  See the javadoc for DSResponse for\n details.\n <P>\n See the Struts examples in <code>[webroot]/examples/struts</code> for usage examples.",
        name:"validationURL",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null},
            {ref:"method:DynamicForm.submit", description:null}
        ],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:Canvas.width":{
        flags:"IRW",
        getter:["getWidth"],
        type:"attr",
        ref:"attr:Canvas.width",
        description:"Size for this component's horizontal dimension.\n <P>\n Can be a number of pixels, or a percentage like \"50%\". Percentage sizes are resolved to\n pixel values as follows:\n <UL>\n <LI>If a canvas has a specified ${isc.DocUtils.linkForRef('attr:Canvas.percentSource','percentSource')}, sizing will be\n     a percentage of the size of that widget (see also ${isc.DocUtils.linkForRef('attr:Canvas.percentBox')}).</LI>\n <LI>Otherwise, if a canvas has a ${isc.DocUtils.linkForRef('attr:Canvas.masterElement','masterElement')}, and\n     ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')} is set for the widget, sizing will be a percentage of\n     the size of that widget (see also ${isc.DocUtils.linkForRef('attr:Canvas.percentBox')}).</LI>\n <LI>Otherwise if this is a child of some other canvas, percentages will be based on the \n     inner size of the ${isc.DocUtils.linkForRef('attr:Canvas.parentElement','parentElement')}'s viewport.</LI>\n <LI>Otherwise, for top level widgets, sizing is calculated as a percentage of page size.</LI>\n </UL>\n <P>\n ${isc.DocUtils.linkForRef('class:Layout','Layouts')} may specially interpret percentage sizes on their children,\n and also allow \"*\" as a size.\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:Canvas.overflow','overflow')} is set to \"visible\", this size is a\n minimum, and the component may overflow to show all content and/or children.\n <P>\n If trying to establish a default width for a custom component, set\n ${isc.DocUtils.linkForRef('attr:Canvas.defaultWidth','defaultWidth')} instead.",
        name:"width",
        setter:["setWidth"],
        valueType:"Number or String",
        seeAlso:[],
        defaultValue:"null",
        groups:["sizing"],
        definingClass:"class:Canvas"
    },
    "method:ListGridField.editorEnter":{
        ref:"method:ListGridField.editorEnter",
        flags:"A",
        description:" Callback fired when the user first starts editing a cell.\n <P>\n This callback is typically used to establish dynamic default values via\n ${isc.DocUtils.linkForRef('method:ListGrid.setEditValue')} or ${isc.DocUtils.linkForRef('method:ListGrid.setEditValues')}.\n\n",
        name:"editorEnter",
        seeAlso:[],
        params:[
            {description:"record for the cell being edited.  <b>Will be null</b>                                for a new, unsaved record.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"value for the cell being edited", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"integer"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"integer"},
            {description:"ListGrid to which this field belongs", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:ListGrid.groupIndentSize":{
        ref:"attr:ListGrid.groupIndentSize",
        flags:"IRW",
        description:"Default number of pixels by which to indent subgroups relative to parent group.",
        name:"groupIndentSize",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null},
            {ref:"method:ListGrid.getGroupNodeHTML", description:null}
        ],
        valueType:"Number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "method:SummaryBuilder.fireOnClose":{
        ref:"method:SummaryBuilder.fireOnClose",
        flags:"A",
        description:" Override to execute a callback function when the Format is Cancelled or Saved.\n \n",
        name:"fireOnClose",
        seeAlso:[],
        type:"method",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "group:featuresCustomPersistence":{
        ref:"group:featuresCustomPersistence",
        title:"Using SmartClient Server framework with custom persistence logic",
        treeLocation:["Concepts"],
        refs:[],
        description:"The vast majority of the SmartClient Server framework's key features are not specific to the\n built-in SQL and Hibernate connectors, and still apply even when using a custom persistence\n mechanism.\n <P>\n See the listing below of major features and how to apply them with custom persistence:\n <p>\n <b>Server Data Binding:</b> Using the SmartClient Server framework means that the starting\n point for connecting to custom persistence logic is a clean Java API.  SmartClient provides\n Java <code>DSRequest</code> and <code>DSResponse</code> objects with all of the methods\n necessary to handle data paging, sorting, validation error reporting, and other features.\n In most cases, you can fulfill a DSResponse by simply returning one of your Java\n business objects rather than worrying about how to encode objects to XML or JSON.\n Communication with the browser is automatically handled with an efficient, compressed\n protocol.  \n ${isc.DocUtils.linkForExampleId('ormDataSource', 'Custom DataSource example')}, \n ${isc.DocUtils.linkForExampleId('DMI', 'DMI example')}\n <p>\n <b>Data Selection (No DTOs):</b> When using a DataSource, Java data you return in your\n <code>DSResponse</code> is automatically trimmed to just the fields declared in the\n DataSource before delivery to the browser (see\n ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields','dropExtraFields')}).  This eliminates the need to create \n redundant <a href='http://en.wikipedia.org/wiki/Data_transfer_object' onclick=\"window.open('http://en.wikipedia.org/wiki/Data_transfer_object');return false;\">Data Transfer Objects</a> \n to express the list of fields that need to be delivered to the UI - the DataSource already\n has this information, and can serve two purposes by both configuring UI components and\n trimming relevant data, from a single definition.\n <P>\n Furthermore, DataSources can extract specific fields from complex nested \n object graphs via XPath expressions, for both loading and saving of data.\n ${isc.DocUtils.linkForExampleId('flattenedBeans', 'XPath Binding example')}\n <p>\n <b>Server Validation:</b> Both client and server validation are driven from declarations in\n a single DataSource definition.  You already have a need to declare validators to\n drive SmartClient's client-side validation; when you use the SmartClient Server framework\n you get automatic server-side enforcement of the same validation rules, without the need to\n write any additional code.\n ${isc.DocUtils.linkForExampleId('serverValidation', 'Server Validation example')}\n <p>\n <b>Queuing:</b> Queuing allows multiple data load or save requests from different UI \n components to be transparently combined into a single HTTP request with guaranteed in-order\n execution.  The type of DataSource handling each request is not important, so queuing will\n work with your custom DataSource; in fact, a single queue could contain operations to be \n handled by many different types of DataSource.  \n ${isc.DocUtils.linkForExampleId('transactionsFolder', 'Queuing examples')}\n <p>\n <b>Transaction Chaining:</b> allows one request in a queue of operations to incorporate\n values from previously executed requests in the same queue.  This allows a series of\n dependent operations - such as fetching a value from one DataSource to be used in a query on\n another - to be defined with simple declarations right in the DataSource definition, with no\n need to write a custom Java class to manually pass data between the different operations.\n Since Transaction Chaining works strictly in terms of DataSource requests and responses and\n knows nothing about the underlying persistence mechanism, it works with any persistence\n strategy.\n <p>\n <b>Java / JS Reflection:</b> Any Java object can be delivered to the browser as a JavaScript \n object, and vice versa.  As a developer of a custom DataSource, you do not need to concern \n yourself with dealing with translations to and from JSON or XML; you work directly with\n native Java objects.  For example, a method you write to fulfill a \"fetch\" operation can\n simply return a <code>Collection</code> of Java beans; the SmartClient Server framework would \n transparently handle converting this into a matching Javascript structure.\n ${isc.DocUtils.linkForExampleId('masterDetail', 'Saving nested objects example')}\n <p>\n <b>Visual Builder:</b> The DataSource Wizards in Visual Builder are pluggable; we provide\n wizards for SQL and Hibernate DataSources, and it is easy to write a new wizard to integrate \n your custom DataSource into Visual Builder.  ${isc.DocUtils.linkForExampleId('sqlWizard', 'SQL Wizard screenshots')},\n ${isc.DocUtils.linkForExampleId('hibernateWizard', 'Hibernate Wizard screenshots')}\n <p>\n <b>Batch DataSource Generator:</b> If the persistence scheme you are implementing your \n custom DataSource for is based on collections of Javabeans, the Batch DataSource Generator \n can generate DataSource definition files for instances of your custom DataSource.  This is\n out of the box behavior, but you can also alter and extend the DataSource Generator to suit \n your exact needs - we supply the source and it has been specifically designed to be easy \n to modify.\n <p>\n <b>Batch Uploader:</b> A user interface for end-to-end batch upload of data as a pre-built, \n customizable component.  This component - like any SmartClient databound component - only\n cares that a DataSource is a DataSource, so custom DataSources will work just like built-in\n ones.  ${isc.DocUtils.linkForExampleId('batchUpload', 'Batch Uploader example')}\n <p>\n <b>File Upload:</b> Single and multiple file uploads can be handled as a normal DataSource \n operation, including normal handling of validation errors. Optional automatic storage to SQL \n (no server code required) means you can upload to SQL tables for holding files, which can be\n related to Java Objects by id (eg, a User's uploaded files). \n ${isc.DocUtils.linkForExampleId('upload', 'File Upload example')}\n <p>\n <b>Export:</b> Allows any grid component to export its current dataset in CSV, XML or JSON\n format.  This feature works by issuing the DataSource with an ordinary \"fetch\", and then \n changing the <code>DSResponse</code> to send back an import file rather than a resultset.\n Accordingly, this just works with custom DataSources.  ${isc.DocUtils.linkForExampleId('export', 'Export example')}\n <p>\n <b>HTTP Proxy:</b> The HTTP Proxy allows an application to access web services hosted on \n remote servers which are not normally accessible to web applications due to the \n <a href='http://www.google.com/search?q=same+origin+policy' onclick=\"window.open('http://www.google.com/search?q=same+origin+policy');return false;\">\"same origin policy\"</a>).\n This is a general feature of the SmartClient Server framework that does not directly apply\n to DataSources.  ${isc.DocUtils.linkForExampleId('rssFeed', 'HTTP Proxy example')}\n <p>\n <b>Lightweight Persistence / Reporting:</b> Even while using a custom DataSource to connect\n to a custom ORM system, you can still make use of the SQL DataSource for simple storage-only\n entities where an object-based representation is a waste of time.  You can also do\n this for reporting scenarios that don't correspond to the object model.",
        name:"featuresCustomPersistence",
        seeAlso:[],
        type:"group"
    },
    "classMethod:isc.makeReverseMap":{
        ref:"classMethod:isc.makeReverseMap",
        flags:"",
        description:" Given a key:value map, return a new map as value:key.\n <P>\n If the same value appears more than once, the key will correspond to the last instance of that\n value.\n\n",
        name:"makeReverseMap",
        seeAlso:[],
        params:[
            {description:"object of key:value pairs", optional:false, name:"valueMap",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"reversed value map", type:"object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "type:SCImgURL":{
        ref:"type:SCImgURL",
        description:"Properties that refer to images by URL, such as ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:Button.icon')}, are\n specially interpreted in SmartClient to allow for simpler and more uniform image URLs,\n and to allow applications to be restructured more easily.\n <P>\n <b>the application image directory</b>\n <P>\n When specifying URLs to image files via SmartClient component properties such as\n ${isc.DocUtils.linkForRef('attr:StretchImg.src')}, any relative path is assumed to be relative to the \"application image\n directory\" (<code>appImgDir</code>).  The application image directory can be set via\n ${isc.DocUtils.linkForRef('classMethod:Page.setAppImgDir')}, and defaults to \"images/\", representing the typical practice of\n placing images in a subdirectory relative to the URL at which the application is accessed.\n <P>\n For applications that may be launched from multiple URLs, the <code>appImgDir</code> can be \n set to the correct relative path to the image directory by calling\n ${isc.DocUtils.linkForRef('classMethod:Page.setAppImgDir')} before any SmartClient components are created.  This enables\n applications or components of an application to be launched from multiple locations, or to\n be relocated, without changing any image URLs supplied to SmartClient components.\n <P>\n <b>the \"[SKIN]\" URL prefix</b>\n <P>\n The special prefix \"[SKIN]\" can be used to refer to images within the skin folder\n whenever image URLs are supplied to SmartClient components.\n <P>\n The value of \"[SKIN]\" is the combination of: \n <ul>\n <li> the \"skin directory\", established in <code>load_skin.js</code> via ${isc.DocUtils.linkForRef('classMethod:Page.setSkinDir')},\n plus..\n <li> the setting for ${isc.DocUtils.linkForRef('attr:Canvas.skinImgDir','skinImgDir')} on the component where you set an\n image URL property\n </ul>\n <code>skinImgDir</code> defaults to \"images/\", so creating an ${isc.DocUtils.linkForRef('class:Img')} component with\n ${isc.DocUtils.linkForRef('attr:Img.src')} set to \"[SKIN]myButton/button.gif\" will expand to <code>Page.getSkinDir() +\n \"/images/myButton/button.gif\"</code>.\n <P>\n Some components that use a large number of images use <code>skinImgDir</code> to group them\n together and make it possible to relocate all the media for the component with a single setting.\n For example, the ${isc.DocUtils.linkForRef('class:TreeGrid')} class sets <code>skinImgDir</code> to \"images/TreeGrid/\".\n This allows ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} to be set to just \"[SKIN]folder.gif\" but refer to\n <code>Page.getSkinDir() + \"/images/TreeGrid/folder.gif\"</code>.\n <P>\n A custom subclass of TreeGrid can set <code>skinImgDir</code> to a different path, such as \n \"/images/MyTreeGrid\", to source all media from a different location.\n <P>\n TIPS:\n <ul>\n <li> subcomponents may not share the parent component's setting for skinImgDir.  For\n example, the ${isc.DocUtils.linkForRef('attr:Window.minimizeButton')} has the default setting for \"skinImgDir\"\n (\"images/\"), so the ${isc.DocUtils.linkForRef('attr:Img.src','src')} property used with this component is set to\n \"[SKIN]/Window/minimize.png\" (in the \"SmartClient\" sample skin).\n <li> for a particular image, the skinImgDir setting on the component may not be\n convenient.  The prefix \"[SKINIMG]\" can be used to refer to <code>Page.getSkinDir() +\n \"/images\"</code> regardless of the setting for <code>skinImgDir</code>\n </ul>\n <B>Stateful image URLs</B>\n <P>\n Many image URLs in SmartClient are \"stateful\", meaning that the actual URL used to fetch an\n image will vary according to the component's state (eg, \"Disabled\"), generally, by adding a\n suffix to the image URL.  See the ${isc.DocUtils.linkForRef('group:skinning','Skinning Overview')} for more\n information on statefulness and the ${isc.DocUtils.linkForRef('attr:Img.src')} documentation for information on how\n stateful image URLs are formed.",
        name:"SCImgURL",
        seeAlso:[],
        type:"type"
    },
    "attr:RPCRequest.containsCredentials":{
        ref:"attr:RPCRequest.containsCredentials",
        flags:"IRWA",
        description:"For use during ${isc.DocUtils.linkForRef('group:relogin','Relogin')}, this property marks this request an attempt to\n login, therefore a response containing the <code>loginRequiredMarker</code> is a normal\n condition and should result in the status code ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_LOGIN_INCORRECT')}\n rather than a call to ${isc.DocUtils.linkForRef('classMethod:RPCManager.loginRequired','loginRequired()')}.\n <P>\n It is not required to set <code>containsCredentials</code>, however, it does typically\n simplify relogin logic by separating the handling of RPCs that are login attempts from RPCs\n that are not.",
        name:"containsCredentials",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["relogin"]
    },
    "attr:Menu.checkmarkImage":{
        ref:"attr:Menu.checkmarkImage",
        flags:"IRWA",
        description:"Default image to display for checkmarked items. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.\n Valid src, width and height must be specified.",
        name:"checkmarkImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:Menu"
    },
    "classMethod:FileLoader.cacheEdgeImages":{
        ref:"classMethod:FileLoader.cacheEdgeImages",
        flags:"",
        description:"\n Caches a set of edge images derived from a base image.\n For example this code:\n <pre>\n isc.FileLoader.cacheEdgeImages(\"/isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6.png\");\n </pre>\n Will cause the following images to be cached:\n <pre>\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_TL.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_T.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_TR.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_L.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_R.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_BL.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_B.png\n /isomorphic/skins/SmartClient/images/edges/rounded/frame/A3B2CC/6_BR.png\n </pre>\n\n",
        name:"cacheEdgeImages",
        seeAlso:[],
        params:[
            {description:"List of base image URLs from which position-specific edge                                    names are to be derived.  This can either be a\n                                    string with comma separated baseURLs or an Array of baseURLs.",
             optional:false, name:"baseURLs", defaultValue:null, type:"String or Array"},
            {description:"If true, also causes the center image to be cached.  Default                                    is false.",
             optional:true, name:"showCenter", defaultValue:null, type:"Boolean"},
            {description:"List of edges to load.  If none specified, The following                                    default list will be used:\n                                    \"TL,T,TR,L,R,BL,B,BR\".  The list of valid values is the\n                                    set of extensions specified by\n                                    ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeImage')}. This can\n                                    either be a string with comma separated edge extension or an\n                                    Array of edge extensions.",
             optional:true, name:"edges", defaultValue:null, type:"String or Array"},
            {description:"List of colors to cache.  If not specified, no color is                                     specified.  The list of valid colors are those that you have\n                                     specified as ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeColor')}.\n                                    This can either be a string with comma separated colors or an Array\n                                    of colors.",
             optional:true, name:"colors", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified URLs                                      has completed caching.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "group:layoutMember":{
        ref:"group:layoutMember",
        refs:["method:Canvas.setShowResizeBar", "method:Layout.getMemberOffset", 
         "method:Layout.getMemberDefaultBreadth","attr:Canvas.canDropBefore", "attr:Layout.defaultLayoutAlign", "attr:Layout.memberOverlap", 
         "attr:Canvas.layoutAlign","attr:Canvas.showResizeBar", "attr:Canvas.resizeBarTarget", "attr:Canvas.extraSpace", 
         "attr:SectionStack.itemIndent","attr:SectionStack.itemStartIndent", "attr:SectionStack.itemEndIndent"],
        description:"Properties that can be set on members of a layout to control how the layout is done",
        name:"layoutMember",
        seeAlso:[],
        type:"group"
    },
    "attr:OperationBinding.selectClause":{
        ref:"attr:OperationBinding.selectClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n SELECT clause to use when constructing the SQL query to perform this operation.  The\n property should be a comma-separated list of column names and/or expressions, and you \n can refer to any scalar function supported by the underlying database.  The server will \n insert the text of this property immediately after the \"SELECT\" token.\n <p>\n Note that if you also specify a ${isc.DocUtils.linkForRef('attr:OperationBinding.groupClause','groupClause')}, you can use aggregate \n functions such as SUM and COUNT in the selectClause.\n <p>\n This property is only applicable to operationBindings of \n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"fetch\".\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"selectClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "type:animateShowEffectId":{
        ref:"type:animateShowEffectId",
        values:[
            {description:"content slides into or out of view as the widget grows or shrinks",
             value:"\"slide\""},
            {description:"content is revealed or wiped as the widget grows or shrinks",
             value:"\"wipe\""},
            {description:"widget's opacity smoothly fades into or out of view", value:"\"fade\""},
            {description:"widget moves into position from offscreen", value:"\"fly\""}
        ],
        description:"String specifying effect to apply during an animated show or hide.",
        name:"animateShowEffectId",
        seeAlso:[],
        type:"type"
    },
    "attr:OperationBinding.customSQL":{
        ref:"attr:OperationBinding.customSQL",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\" or \"hibernate\", this \n property can be specified on an operationBinding to indicate that the server should run \n user-specified SQL, rather than the query it would normally generate to satisfy a \n dataSource operation.  This property allows you provide a fully-customized query; an\n alternative is to provide custom \"pieces\" to the query generator, via properties such as\n ${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','whereClause')} and ${isc.DocUtils.linkForRef('attr:OperationBinding.valuesClause','valuesClause')}.  See \n ${isc.DocUtils.linkForRef('group:customQuerying')} for more details.\n <p>\n For a dataSource of type \"sql\", the SmartClient server generates a number of useful query \n \"pieces\" and makes them available to your custom SQL code via the Velocity templating \n language (note that this is not available for \"hibernate\" dataSources).  These are \n documented ${isc.DocUtils.linkForRef('type:DefaultQueryClause','here')}.\n <p>\n We also make the template variables <b>$criteria</b> and <b>$values</b> available, to give\n you direct access to the supplied criteria, and to the new field values for update and add\n operations.  These variables are available to both \"sql\" and \"hibernate\" dataSources.\n <p>\n Note that you should use this feature with care. In particular, writing customSQL code\n that makes use of a particular database engine's features or syntax will make your \n application less portable.  One way to avoid tying your query to a particular database's\n name-quoting idiosyncracies (every database product seems to have a different idea about \n how to treat unquoted names, and which characters to use to quote names) is to use the \n template variables <b>$fields</b> and <b>$qfields</b>, whioh provide the names of your\n dataSource fields correctly quoted in the syntax of the underlying database.\n <p>\n See ${isc.DocUtils.linkForRef('group:customQuerying')} for an overview of writing custom queries and clauses.\n <p>\n <h3>Examples</h3>\n An example using the SmartClient-supplied query pieces.  This custom query will give \n exactly the same result as the SmartClient-generated query:\n <p>\n <code>\n &lt;operationBinding operationId=\"customFetch\" operationType=\"fetch\"&gt;<br>\n &nbsp;&nbsp;&lt;customSQL&gt;<br>\n &nbsp;&nbsp;&nbsp;&nbsp;SELECT $defaultSelectClause FROM $defaultTablesClause\n &nbsp;&nbsp;&nbsp;&nbsp;WHERE $defaultWhereClause ORDER BY $defaultOrderClause\n &nbsp;&nbsp;&lt;/customSQL&gt;<br>\n &lt;/operationBinding&gt;<br>\n </code>\n <p>\n An example using the SmartClient-supplied <b>$criteria</b> template variable:\n <p>\n <code>\n &lt;operationBinding operationId=\"customFetch\" operationType=\"fetch\"&gt;<br>\n &nbsp;&nbsp;&lt;customSQL&gt;<br>\n &nbsp;&nbsp;&nbsp;&nbsp;SELECT foo, bar, baz FROM MyTable WHERE bar > $criteria.someValue<br>\n &nbsp;&nbsp;&lt;/customSQL&gt;<br>\n &lt;/operationBinding&gt;<br>\n </code>\n <p>\n An update example:\n <p>\n <code>\n &lt;operationBinding operationId=\"myUpdateOp\" operationType=\"update\"&gt;<br>\n &nbsp;&nbsp;&lt;customSQL&gt;<br>\n &nbsp;&nbsp;&nbsp;&nbsp;UPDATE $defaultTableClause SET $defaultValuesClause WHERE bar \n &lt;= $criteria.someValue<br>\n &nbsp;&nbsp;&lt;/customSQL&gt;<br>\n &lt;/operationBinding&gt;<br>\n </code>",
        name:"customSQL",
        seeAlso:[
            {ref:"attr:OperationBinding.customHQL", description:null},
            {ref:"attr:DataSourceField.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "object:TileRecord":{
        methods:[],
        ref:"object:TileRecord",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:TileRecord.tileConstructor", "attr:TileRecord.tileProperties"],
        description:"A TileRecord is a JavaScript Object whose properties contain values for each\n TileField. A TileRecord may have additional properties which affect the record's\n appearance or behavior, or which hold data for use by custom logic or other, related\n components.",
        name:"TileRecord",
        seeAlso:[],
        type:"object"
    },
    "attr:StretchImg.items":{
        ref:"attr:StretchImg.items",
        flags:"IRW",
        description:"The list of images to display as an array of objects specifying the image names and\n sizes.\n <P>\n The \"name\" is appended as a suffix to the ${isc.DocUtils.linkForRef('attr:StretchImg.src')} URL in order to fetch separate\n media files for each image.\n <P>\n The height and width can be set to a number, \"*\" (remaining space, divided amongst all\n images that specify \"*\") or to the name of a property on the StretchImg component, such\n as ${isc.DocUtils.linkForRef('attr:StretchImg.capSize')}.  \n <P>\n Height or width is only used on the axis on which images are stacked.  For example, if\n ${isc.DocUtils.linkForRef('attr:StretchImg.vertical')} is true, images stack vertically and heights are used to size images on\n the vertical axis, but all images will have width matching the overall component size.\n <P>\n For example, the default setting for <code>items</code>, which is used to produce\n stretchable buttons and headers with fixed-size endcaps, is as follows:\n <pre>\n   items:[\n        {height:\"capSize\", name:\"start\", width:\"capSize\"},\n        {height:\"*\", name:\"stretch\", width:\"*\"},\n        {height:\"capSize\", name:\"end\", width:\"capSize\"}\n   ]\n </pre>",
        name:"items",
        seeAlso:[],
        valueType:"Array of Object",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "attr:FormItem.showErrorStyle":{
        ref:"attr:FormItem.showErrorStyle",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:FormItem.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorStyle",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["validation", "appearance"]
    },
    "method:DataBoundComponent.getFormulaFieldValue":{
        ref:"method:DataBoundComponent.getFormulaFieldValue",
        flags:"",
        description:" Get the computed value of a ${isc.DocUtils.linkForRef('attr:DataBoundComponent.canAddFormulaFields','formula field')}.\n",
        name:"getFormulaFieldValue",
        seeAlso:[],
        params:[
            {description:"field that has a formula", optional:false, name:"field", defaultValue:null,
             type:"Field"},
            {description:"record to use to compute formula value", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"formula result", type:"Number"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:PickList.optionDataSource":{
        ref:"attr:PickList.optionDataSource",
        flags:"IRA",
        getter:["getOptionDataSource"],
        description:"If set, this FormItem will derive data to show in the PickList by fetching records from\n the specified <code>optionDataSource</code>.  The fetched data will be used as a\n ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} by extracting the\n ${isc.DocUtils.linkForRef('attr:FormItem.valueField','valueField')} and ${isc.DocUtils.linkForRef('attr:FormItem.displayField','displayField')} in\n the loaded records, to derive one valueMap entry per record loaded from the\n optionDataSource.  Multiple fields from the fetched data may be shown in the pickList by\n setting ${isc.DocUtils.linkForRef('attr:PickList.pickListFields')}.\n <P>\n The data will be retrieved via a \"fetch\" operation on the DataSource, passing the \n ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')} (if set) as criteria, and passing\n ${isc.DocUtils.linkForRef('attr:PickList.optionFilterContext')} (if set) as DSRequest properties.\n <P>\n The fetch will be triggered when the pickList is first shown, or, you can set\n ${isc.DocUtils.linkForRef('attr:SelectItem.autoFetchData','autoFetchData:true')} to fetch when the FormItem is\n first drawn.  \n <P>\n Note that providing an initial value when\n ${isc.DocUtils.linkForRef('attr:FormItem.fetchMissingValues','fetchMissingValues')} is enabled, or enabling\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption','defaultToFirstOption')}, can also cause a fetch to\n be initiated immediately upon form creation.  You can also call ${isc.DocUtils.linkForRef('method:PickList.fetchData')}\n at any time to manually trigger a fetch.\n <P>\n Data paging is automatically enabled if the optionDataSource supports it.  As the\n pickList is scrolled by the user, requests for additional data will be automatically\n issued.\n <P>\n For a pickList attached to a ComboBoxItem, new fetches are issued as the user types,\n with criteria set as described under ${isc.DocUtils.linkForRef('method:ComboBoxItem.getPickListFilterCriteria')}.\n If your dataSource is not capable of filtering results by search criteria (eg the\n dataSource is backed by an XML flat file), you can set ${isc.DocUtils.linkForRef('attr:PickList.filterLocally')} to have the\n entire dataset loaded up front and filtering performed in the browser.  This disables\n data paging.\n <P>\n Note that if a normal, static ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} is <b>also</b> specified\n for the field (either directly in the form item or as part of the field definition in\n the dataSource), it will be preferred to the data derived from the optionDataSource for\n whatever mappings are present.",
        name:"optionDataSource",
        seeAlso:[],
        valueType:"DataSource | String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:ToolStrip.vertical":{
        ref:"attr:ToolStrip.vertical",
        flags:"IRW",
        description:"Indicates whether the components are drawn horizontally from left to right (false), or\n vertically from top to bottom (true).",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ToolStrip",
        groups:["appearance"]
    },
    "method:DynamicForm.addFieldErrors":{
        ref:"method:DynamicForm.addFieldErrors",
        flags:"",
        description:" Adds field validation error[s] to the specified field. Errors passed in will be added\n to any existing errors on the field caused by validation or a previous call to this method.\n <br>\n The errors parameter may be passed in as a string (a single error message), or an array of\n strings.<br>\n The showErrors parameter allows the errors to be displayed immediately. Alternatively, call\n ${isc.DocUtils.linkForRef('method:DynamicForm.showFieldErrors')} to display the errors for this field.\n",
        name:"addFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to apply the new errors to", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"errors to apply to the field in question", optional:false, name:"errors",
             defaultValue:null, type:"string | array of strings"},
            {description:"If true this method will fall through to ${isc.DocUtils.linkForRef('method:DynamicForm.showFieldErrors')}  to update the display",
             optional:false, name:"show", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "object:DSRequestModifier":{
        methods:[],
        ref:"object:DSRequestModifier",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:["attr:DSRequestModifier.fieldName", "attr:DSRequestModifier.value"],
        description:"An object that modifies a dsRequest based on several available \n values, including preceding responses from the same queue. You provide a list of \n DSRequestModifiers as the ${isc.DocUtils.linkForRef('attr:OperationBinding.criteria')} or ${isc.DocUtils.linkForRef('attr:OperationBinding.values')}\n of an ${isc.DocUtils.linkForRef('class:OperationBinding')}.\n <P>\n A DSRequestModifier consists of a ${isc.DocUtils.linkForRef('attr:DSRequestModifier.fieldName','fieldName')} and a\n ${isc.DocUtils.linkForRef('attr:DSRequestModifier.value','value')}.  The value can be static, or it can be an expression\n in the Velocity template language, which will be resolved at runtime, immediately before the\n DSRequest is executed.",
        name:"DSRequestModifier",
        seeAlso:[],
        type:"object",
        serverDS:"allowed"
    },
    "class:SummaryBuilder":{
        methods:["method:SummaryBuilder.setSummary", "method:SummaryBuilder.getHelpText", 
         "method:SummaryBuilder.fireOnClose","method:SummaryBuilder.save"],
        ref:"class:SummaryBuilder",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:SummaryBuilder.dataSource", "attr:SummaryBuilder.fields", 
         "attr:SummaryBuilder.editMode","attr:SummaryBuilder.field", "attr:SummaryBuilder.formulaField", 
         "attr:SummaryBuilder.titleField","attr:SummaryBuilder.showHelpIcon", "attr:SummaryBuilder.helpIcon", 
         "attr:SummaryBuilder.showAutoHideCheckBox","attr:SummaryBuilder.autoHideCheckBox", "attr:SummaryBuilder.testRecord", 
         "attr:SummaryBuilder.autoHideCheckBoxLabel","attr:SummaryBuilder.helpTextIntro"],
        description:"Shows an interface allowing a user to create or edit fields by typing simple\n format-strings into a text field.  The format-strings can include the values of other fields\n and additional text as required.\n <P>\n Available values for the format-string are determined by the DataSource fields, and are given\n simple single-letter aliases (such as \"A\", \"B\", ...) similar to column names in Excel.\n The set of available values is shown in the ${isc.DocUtils.linkForRef('attr:FormulaBuilder.fieldKey')} as a simple\n mapping between the ${isc.DocUtils.linkForRef('attr:DataSourceField.title','field title')} and it's short name.\n <P>\n To include a field in the format-string, prefix it with a hash sign (#).",
        name:"SummaryBuilder",
        seeAlso:[],
        type:"class",
        groups:["summaryFields"]
    },
    "attr:DataBoundComponent.updateOperation":{
        ref:"attr:DataBoundComponent.updateOperation",
        flags:"IRW",
        description:"Operation ID this component should use when performing update operations.",
        name:"updateOperation",
        seeAlso:[
            {ref:"attr:DSRequest.operationId", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["operations"]
    },
    "attr:ComboBoxItem.defaultValue":{
        ref:"attr:ComboBoxItem.defaultValue",
        flags:"IRW",
        description:"Static default value for this ComboBoxItem. To default to the first option use\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.defaultToFirstOption')} instead.",
        name:"defaultValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:ColorPicker.getBlue":{
        ref:"method:ColorPicker.getBlue",
        flags:"",
        description:" Returns the Blue element of the currently-selected color, as an integer from 0-255\n",
        name:"getBlue",
        seeAlso:[
            {ref:"method:ColorPicker.setBlue", description:null}
        ],
        returns:{description:"blue color component", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "classMethod:Class.logIsDebugEnabled":{
        ref:"classMethod:Class.logIsDebugEnabled",
        flags:"",
        description:" Check whether a message logged at \"debug\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsDebugEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:ListGrid.datetimeFormatter":{
        ref:"attr:ListGrid.datetimeFormatter",
        flags:"IRW",
        description:"Display format to use for fields specified as type 'datetime'.  Default is to use the\n system-wide default date time format, configured via\n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDatetimeDisplayFormat')}.  Specify any\n valid ${isc.DocUtils.linkForRef('type:DateDisplayFormat')}, or\n function to change the display format for datetimes used by this grid.\n If specified as  a function, this function will be executed in the scope of the Date\n and should return the formatted string.<br>\n May also be specified at the field level via\n ${isc.DocUtils.linkForRef('attr:ListGridField.displayFormat')}.<br>\n If this field is editable the dateFormatter will also be passed to the editor created\n to edit this field as ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat','displayFormat')}.\n In this case you may also need to set ${isc.DocUtils.linkForRef('attr:ListGrid.dateInputFormat')}.",
        name:"datetimeFormatter",
        seeAlso:[
            {ref:"attr:ListGridField.displayFormat", description:null}
        ],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "method:CubeGrid.hiliteCellList":{
        ref:"method:CubeGrid.hiliteCellList",
        flags:"",
        description:"\n      apply a hilite to an array of cells.\n\n      <br><i>methodType</i> action\n",
        name:"hiliteCellList",
        seeAlso:[],
        params:[
            {description:"cells to hilite", optional:false, name:"cellObjList", defaultValue:null,
             type:"Array of cell objects"},
            {description:"ID of hilite to apply to cells", optional:false, name:"hiliteID",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true if the cells were successfully hilited.", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["hiliting"]
    },
    "method:Canvas.getVisibleHeight":{
        ref:"method:Canvas.getVisibleHeight",
        flags:"A",
        description:"      Return the visible height of the Canvas.\n\n",
        name:"getVisibleHeight",
        seeAlso:[],
        returns:{description:"visible height in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:ListGrid.setAlternateBodyStyleName":{
        ref:"method:ListGrid.setAlternateBodyStyleName",
        flags:"",
        description:" Update the ${isc.DocUtils.linkForRef('attr:ListGrid.alternateBodyStyleName','alternateBodyStyleName')} for this listGrid.\n",
        name:"setAlternateBodyStyleName",
        seeAlso:[],
        params:[
            {description:"new body style name when showing alternateRecordStyles", optional:false,
             name:"styleName", defaultValue:null, type:"CSSStyleName"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:ColorPicker.getSaturation":{
        ref:"method:ColorPicker.getSaturation",
        flags:"",
        description:" Returns the Saturation of the currently-selected color, as an integer from 0-240\n",
        name:"getSaturation",
        seeAlso:[
            {ref:"method:ColorPicker.setSaturation", description:null}
        ],
        returns:{description:"saturation value", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "method:RestDataSource.transformResponse":{
        ref:"method:RestDataSource.transformResponse",
        flags:"",
        description:" RestDataSource implements transformResponse in order to extract data and meta-data\n properties from server responses, as described in the \n ${isc.DocUtils.linkForRef('class:RestDataSource','RestDataSource overview')}.\n <P>\n You can override <code>transformResponse()</code> in order to further modify the\n response, but if you do so, call ${isc.DocUtils.linkForRef('method:Class.Super','Super()')} as shown below or you will\n wipe out the built-in response processing behavior of RestDataSource.\n <pre>\n transformResponse : function (dsResponse, dsRequest, data) {        \n     var dsResponse = this.Super(\"transformResponse\", arguments);\n     // ... do something to dsResponse ...\n     return dsResponse;\n }\n </pre> \n",
        name:"transformResponse",
        seeAlso:[],
        params:[
            {description:"default DSResponse derived from the response data", optional:false,
             name:"dsResponse", defaultValue:null, type:"DSResponse"},
            {description:"DSRequest object that initiated this request", optional:false,
             name:"dsRequest", defaultValue:null, type:"DSRequest"},
            {description:"XML document or JSON objects returned by the web                                   service",
             optional:false, name:"data", defaultValue:null, type:"XMLDocument or JSON"}
        ],
        returns:{description:"response derived", type:"DSResponse"},
        type:"method",
        definingClass:"class:RestDataSource"
    },
    "attr:CubeGrid.canEdit":{
        ref:"attr:CubeGrid.canEdit",
        flags:"IRW",
        description:"Whether cells can be edited in this grid.  Can be override on a per-facetValue basis.",
        name:"canEdit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["cellEditing"]
    },
    "attr:DateChooser.headerStyle":{
        ref:"attr:DateChooser.headerStyle",
        flags:"IRW",
        description:"CSS style applied to the day-of-week headers. By default this applies to all days of the \n week. To apply a separate style to weekend headers, set ${isc.DocUtils.linkForRef('attr:DateChooser.weekendHeaderStyle')}",
        name:"headerStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"dateChooserButtonDisabled\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:DateChooser.nextMonthIconHeight":{
        ref:"attr:DateChooser.nextMonthIconHeight",
        flags:"IRW",
        description:"Height of the icon for the next month button",
        name:"nextMonthIconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:ListGrid.transferSelectedData":{
        ref:"method:ListGrid.transferSelectedData",
        flags:"",
        description:" Simulates a drag / drop type transfer of the selected records in some other component to this\n component, without requiring any user interaction.  This method acts on the dropped records \n exactly as if they had been dropped in an actual drag / drop interaction, including any \n special databound behavior invoked by calling\n ${isc.DocUtils.linkForRef('method:DataBoundComponent.getDropValues','getDropValues')} for each dropped record.\n <P>\n To transfer <b>all</b> data in, for example, a ${isc.DocUtils.linkForRef('class:ListGrid')}, call grid.selection.selectAll() first.\n <P>\n Note that drag/drop type transfers of records between components are asynchronous operations:\n SmartClient may need to perform server turnarounds to establish whether dropped records \n already exist in the target component.  Therefore, it is possible to issue a call to \n transferSelectedData() and/or the ${isc.DocUtils.linkForRef('method:ListGrid.drop','drop()')} method of a databound \n component whilst a transfer is still active.  When this happens, SmartClient adds the \n second and subsequent transfer requests to a queue and runs them one after the other.  If \n you want to be notified when a transfer process has actually completed, either provide a \n callback to this method or implement ${isc.DocUtils.linkForRef('method:DataBoundComponent.dropComplete')}.\n <P>\n See the ${isc.DocUtils.linkForRef('group:dragging')} documentation for an overview of list grid drag/drop data\n transfer.\n \n",
        name:"transferSelectedData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"},
            {description:"target index (drop position) of the rows within this grid.", optional:true,
             name:"index", defaultValue:null, type:"integer"},
            {description:"optional callback to be fired when the transfer process has                        completed.  The callback will be passed a single parameter \"records\",\n                       the list of records actually transferred to this component.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragListMove')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "group:appearance":{
        ref:"group:appearance",
        refs:["method:CubeGrid.getCellStyle", "method:TextItem.getHint", "method:ButtonItem.setTitle", 
         "method:DetailViewerField.getCellStyle","method:GridRenderer.getCellStyle", "method:GridRenderer.cellIsEnabled", 
         "method:GridRenderer.refreshCellStyle","method:GridRenderer.refreshCell", "method:GridRenderer.refreshRow", 
         "method:ListGrid.getCellStyle","method:ListGrid.refreshCellStyle", "method:ListGridField.getFieldTitle", 
         "method:ListGridField.showIf","method:ListGridField.getSummaryTitle", "method:ListGrid.refreshCell", 
         "method:ListGrid.refreshRow","method:Canvas.setPadding", "method:Canvas.setBackgroundColor", 
         "method:Canvas.setBackgroundImage","method:Canvas.setBorder", "method:Canvas.setClassName", "method:Canvas.setStyleName", 
         "method:Window.setStatus","method:Window.setSrc", "method:Window.centerInPage", "method:StatefulCanvas.setState", 
         "method:StatefulCanvas.setTitle","method:Button.setTitle", "method:Button.setState", "method:Img.setSrc", 
         "method:Img.resetSrc","method:StretchImg.setState", "method:StretchImg.setSrc", "method:ImgButton.setTitle", 
         "method:ImgButton.setState","method:StretchImgButton.setTitle", "method:StretchImgButton.setState", 
         "attr:CubeGrid.styleName","attr:CubeGrid.bodyStyleName", "attr:CubeGrid.alternateRecordStyles", 
         "attr:CubeGrid.innerHeaderBaseStyle","attr:CubeGrid.colHeaderBaseStyle", "attr:CubeGrid.rowHeaderBaseStyle", 
         "attr:DateChooser.firstDayOfWeek","attr:DynamicForm.longTextEditorThreshold", "attr:DynamicForm.longTextEditorType", 
         "attr:DynamicForm.errorOrientation","attr:FormItem.type", "attr:FormItem.editorType", "attr:FormItem.dateFormatter", 
         "attr:FormItem.timeFormatter","attr:FormItem.visible", "attr:FormItem.disabled", "attr:FormItem.align", 
         "attr:FormItem.textAlign","attr:FormItem.showErrorIcon", "attr:FormItem.showErrorText", 
         "attr:FormItem.showErrorStyle","attr:FormItem.errorOrientation", "attr:FormItem.hint", "attr:FormItem.showHint", 
         "attr:FormItem.showFocused","attr:FormItem.showDisabled", "attr:FormItem.cellStyle", "attr:FormItem.hintStyle", 
         "attr:FormItem.textBoxStyle","attr:FormItem.cellClassName", "attr:FormItem.errorCellClassName", 
         "attr:FormItem.hintClassName","attr:TextItem.width", "attr:TextItem.height", "attr:TextItem.textBoxStyle", 
         "attr:TextItem.showHintInField","attr:BlurbItem.showTitle", "attr:BlurbItem.colSpan", "attr:BlurbItem.textBoxStyle", 
         "attr:BlurbItem.wrap","attr:BlurbItem.clipValue", "attr:ButtonItem.height", "attr:ButtonItem.baseStyle", 
         "attr:ButtonItem.showTitle","attr:SelectItem.height", "attr:SelectOtherItem.separatorTitle", 
         "attr:SelectOtherItem.separatorValue","attr:SelectOtherItem.otherTitle", "attr:SelectOtherItem.otherValue", 
         "attr:SelectItem.textBoxStyle","attr:SelectItem.controlStyle", "attr:SelectItem.pickerIconStyle", 
         "attr:SelectItem.showFocused","attr:SelectItem.showHintInField", "attr:SelectItem.multiple", 
         "attr:CheckboxItem.titleStyle","attr:CheckboxItem.textBoxStyle", "attr:CheckboxItem.checkedImage", 
         "attr:CheckboxItem.uncheckedImage","attr:CheckboxItem.unsetImage", "attr:NativeCheckboxItem.textBoxStyle", 
         "attr:HeaderItem.defaultValue","attr:HeaderItem.showTitle", "attr:HeaderItem.textBoxStyle", "attr:HeaderItem.colSpan", 
         "attr:HeaderItem.startRow","attr:HeaderItem.endRow", "attr:HiddenItem.showTitle", "attr:HiddenItem.colSpan", 
         "attr:HiddenItem.rowSpan","attr:StaticTextItem.wrap", "attr:StaticTextItem.clipValue", 
         "attr:StaticTextItem.textBoxStyle","attr:StaticTextItem.outputAsHTML", "attr:RadioGroupItem.vertical", 
         "attr:RadioGroupItem.textBoxStyle","attr:RadioGroupItem.wrap", "attr:DateItem.textAlign", "attr:DateItem.startDate", 
         "attr:DateItem.endDate","attr:DateItem.centuryThreshold", "attr:SpacerItem.showTitle", "attr:SpacerItem.width", 
         "attr:SpacerItem.height","attr:RowSpacerItem.showTitle", "attr:RowSpacerItem.colSpan", 
         "attr:RowSpacerItem.startRow","attr:RowSpacerItem.endRow", "attr:TextAreaItem.wrap", "attr:TextAreaItem.width", 
         "attr:TextAreaItem.height","attr:TextAreaItem.textBoxStyle", "attr:TextAreaItem.showHintInField", 
         "attr:ToolbarItem.showTitle","attr:ToolbarItem.startRow", "attr:ToolbarItem.endRow", "attr:ToolbarItem.colSpan", 
         "attr:ToolbarItem.buttonBaseStyle","attr:ComboBoxItem.showHintInField", "attr:Calendar.baseStyle", 
         "attr:Calendar.dayHeaderBaseStyle","attr:Calendar.dayBodyBaseStyle", "attr:Calendar.otherDayHeaderBaseStyle", 
         "attr:Calendar.otherDayBodyBaseStyle","attr:Calendar.otherDayBlankStyle", "attr:Calendar.selectedCellStyle", 
         "attr:Calendar.eventWindowStyle","attr:Calendar.workdayBaseStyle", "attr:Calendar.eventWindowStyleField", 
         "attr:Calendar.weekEventBorderOverlap","attr:DetailViewer.recordsPerBlock", "attr:DetailViewer.blockSeparator", 
         "attr:DetailViewer.showEmptyField","attr:DetailViewer.emptyCellValue", "attr:DetailViewer.styleName", 
         "attr:DetailViewer.blockStyle","attr:DetailViewer.labelStyle", "attr:DetailViewer.cellStyle", 
         "attr:DetailViewer.headerStyle","attr:DetailViewer.separatorStyle", "attr:ListGrid.styleName", "attr:ListGridField.type", 
         "attr:ListGridField.title","attr:ListGridField.canHide", "attr:ListGridField.summaryTitle", 
         "attr:ListGridField.width","attr:ListGridField.align", "attr:ListGridField.headerBaseStyle", 
         "attr:ListGridField.headerTitleStyle","attr:ListGridField.cellAlign", "attr:ListGridField.baseStyle", 
         "attr:ListGrid.bodyBackgroundColor","attr:ListGrid.bodyStyleName", "attr:ListGrid.leaveScrollbarGap", 
         "attr:ListGrid.dateFormatter","attr:ListGrid.datetimeFormatter", "attr:ListGrid.timeFormatter", 
         "attr:ListGrid.editFailedBaseStyle","attr:ListGrid.editFailedCSSText", "attr:ListGrid.editPendingBaseStyle", 
         "attr:ListGrid.editPendingCSSText","attr:ListGrid.recordBaseStyleProperty", "attr:ListGrid.frozenBaseStyle", 
         "attr:ListGrid.showRollOver","attr:ListGrid.selectionType", "attr:ListGrid.showHeader", "attr:ListGrid.headerBarStyle", 
         "attr:ListGrid.headerBackgroundColor","attr:ListGrid.headerButtonConstructor", "attr:ListGrid.headerBaseStyle", 
         "attr:ListGrid.headerTitleStyle","attr:ListGrid.frozenHeaderBaseStyle", "attr:ListGrid.frozenHeaderTitleStyle", 
         "attr:ListGrid.headerButtonDefaults","attr:ListGrid.headerButtonProperties", "attr:ListGrid.sorterConstructor", 
         "attr:ListGrid.sorterDefaults","attr:ListGrid.sorterProperties", "attr:ListGrid.showSortArrow", 
         "attr:ListGrid.skinImgDir","attr:ListGrid.sortAscendingImage", "attr:ListGrid.sortDescendingImage", 
         "attr:TreeGrid.indentSize","attr:TileLayout.animateTileChange", "attr:TileGrid.selectionType", 
         "attr:TileGrid.animateTileChange","attr:TileGrid.styleName", "attr:Canvas.autoShowParent", "attr:Canvas.visibility", 
         "attr:Canvas.className","attr:Canvas.styleName", "attr:Canvas.margin", "attr:Canvas.padding", "attr:Canvas.border", 
         "attr:Canvas.backgroundColor","attr:Canvas.backgroundImage", "attr:Canvas.backgroundRepeat", 
         "attr:Canvas.backgroundPosition","attr:ImgTab.capSize", "attr:Window.modalMask", "attr:Window.showModalMask", 
         "attr:Window.modalMaskOpacity","attr:Window.modalMaskStyle", "attr:Window.autoCenter", "attr:Window.showBody", 
         "attr:Window.bodyStyle","attr:Window.bodyColor", "attr:Window.hiliteBodyColor", "attr:Window.items", 
         "attr:Window.src","attr:Window.contentsType", "attr:Window.bodyConstructor", "attr:Window.bodyDefaults", 
         "attr:Window.contentLayout","attr:Window.autoSize", "attr:Window.showHeader", "attr:Window.showHeaderBackground", 
         "attr:Window.headerStyle","attr:Window.headerSrc", "attr:Window.hiliteHeaderStyle", "attr:Window.hiliteHeaderSrc", 
         "attr:Window.showTitle","attr:Window.title", "attr:Window.headerLabelDefaults", "attr:Window.showHeaderIcon", 
         "attr:Window.headerIconDefaults","attr:Window.showCloseButton", "attr:Window.showMinimizeButton", "attr:Window.minimized", 
         "attr:Window.defaultMinimizeHeight","attr:Window.minimizeHeight", "attr:Window.animateMinimize", "attr:Window.minimizeTime", 
         "attr:Window.minimizeAcceleration","attr:Window.maximized", "attr:Window.showMaximizeButton", "attr:Window.showFooter", 
         "attr:Window.footerHeight","attr:Window.showStatusBar", "attr:Window.showResizer", "attr:Window.status", 
         "attr:Dialog.styleName","attr:Dialog.showToolbar", "attr:StatefulCanvas.align", "attr:StatefulCanvas.valign", 
         "attr:Button.align","attr:Button.valign", "attr:Img.src", "attr:Img.activeAreaHTML", "attr:Img.imageType", 
         "attr:StretchImg.vertical","attr:StretchImg.capSize", "attr:StretchImg.src", "attr:StretchImg.hSrc", 
         "attr:StretchImg.vSrc","attr:StretchImg.imageType", "attr:StretchImg.items", "attr:Progressbar.percentDone", 
         "attr:Progressbar.length","attr:Progressbar.breadth", "attr:Progressbar.vertical", "attr:Progressbar.src", 
         "attr:Toolbar.vertical","attr:Toolbar.buttonConstructor", "attr:Toolbar.buttonDefaults", 
         "attr:ToolStrip.styleName","attr:ToolStrip.vertical", "attr:ImgButton.align", "attr:ImgButton.valign", 
         "attr:ImgButton.src","attr:StretchImgButton.align", "attr:StretchImgButton.valign", 
         "attr:StretchImgButton.vertical","attr:StretchImgButton.capSize", "attr:Splitbar.src", "attr:Splitbar.hSrc", 
         "attr:Splitbar.vSrc","attr:Splitbar.capSize", "attr:ImgSplitbar.src", "type:FormItemBaseStyle", 
         "type:SortArrow","type:Alignment", "type:VerticalAlignment", "type:Direction", "type:ImageStyle", 
         "type:CSSStyleName"],
        description:"Properties defining an object's apperance",
        name:"appearance",
        seeAlso:[],
        type:"group"
    },
    "attr:DetailViewerField.canExport":{
        ref:"attr:DetailViewerField.canExport",
        flags:"IR",
        description:"Dictates whether the data in this field be exported.  Explicitly set this\n  to false to prevent exporting.  Has no effect if the underlying \n  ${isc.DocUtils.linkForRef('attr:DataSourceField.canExport','dataSourceField')} is explicitly set to \n  canExport: false.",
        name:"canExport",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "method:SelectItem.getSelectedRecord":{
        ref:"method:SelectItem.getSelectedRecord",
        flags:"",
        description:" Returns the entire record object associated with the current value for this item\n (or null if no matching record exists in the PickList data).\n <P>\n Most commonly used for databound pickListItems to retrieve the values of other fields\n in the record.\n\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"record object", type:"object"},
        type:"method",
        definingClass:"class:SelectItem"
    },
    "attr:MultiSortPanel.levelUpButton":{
        ref:"attr:MultiSortPanel.levelUpButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ImgButton')} providing a mechanism for moving existing\n sort-levels up in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.levelUpButtonProperties</code> and \n <code>multiSortPanel.levelUpButtonDefaults</code>.",
        name:"levelUpButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "attr:TabBar.baseLineThickness":{
        ref:"attr:TabBar.baseLineThickness",
        flags:"IR",
        description:"Thickness of the baseLine, in pixels.  This should be set to match the media specified\n by ${isc.DocUtils.linkForRef('attr:TabBar.baseLineSrc')}.  The baseLineThickness also determines the degree of overlap with\n the TabSet's paneContainer when using decorative edges - see ${isc.DocUtils.linkForRef('attr:TabSet.paneContainer')}\n for details.",
        name:"baseLineThickness",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:TabBar",
        groups:["baseLine"]
    },
    "classAttr:Operators.regexpTitle":{
        ref:"classAttr:Operators.regexpTitle",
        flags:"IR",
        description:"Title for the \"regexp\" operator",
        name:"regexpTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"matches expression (exact case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classMethod:isA.nonemptyString":{
        ref:"classMethod:isA.nonemptyString",
        flags:"",
        description:"\n	Is <code>object</code> a non-empty String?<br><br>\n	\n",
        name:"nonemptyString",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a non-empty string", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:FormItem.getValueIcon":{
        ref:"method:FormItem.getValueIcon",
        flags:"",
        description:" If specified this stringMethod allows the developer to specify the image source for an \n icon to be displayed for the current form item value.\n <P>\n Takes presidence over ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')}\n\n",
        name:"getValueIcon",
        seeAlso:[],
        params:[
            {description:"value of the item for which an item should be returned.", optional:false,
             name:"value", defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:ResultSet.criteriaPolicy":{
        ref:"attr:ResultSet.criteriaPolicy",
        flags:"IRWA",
        description:"Decides under what conditions the cache should be dropped when the ${isc.DocUtils.linkForRef('type:Criteria')}\n changes.",
        name:"criteriaPolicy",
        seeAlso:[
            {ref:"type:Criteria", description:null},
            {ref:"attr:DataSource.criteriaPolicy", description:null}
        ],
        valueType:"CriteriaPolicy",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "method:DynamicForm.getTitleOrientation":{
        ref:"method:DynamicForm.getTitleOrientation",
        flags:"A",
        description:" Return the orientation of the title for a specific item or the default title orientation if\n no item is passed.\n \n",
        name:"getTitleOrientation",
        seeAlso:[],
        params:[
            {description:"item to check", optional:true, name:"item", defaultValue:null, type:"FormItem"}
        ],
        returns:{description:"orientation of the title, or null if an item is passed and has no                            title",
         type:"TitleOrientation"},
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "attr:LoginDialog.errorMessage":{
        ref:"attr:LoginDialog.errorMessage",
        flags:"IR",
        description:"Specifies the default error message displayed on the login form when \n authentication fails.",
        name:"errorMessage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"Dialog.LOGIN_ERROR_MESSAGE",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:SimpleTile.baseStyle":{
        ref:"attr:SimpleTile.baseStyle",
        flags:"IR",
        description:"CSS style for the tile as a whole.  As with ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')}, suffixes\n are appended to this style to represent various states (\"Over\", \"Selected\", etc).",
        name:"baseStyle",
        seeAlso:[],
        valueType:"CSSClassName",
        defaultValue:"\"simpleTile\"",
        type:"attr",
        definingClass:"class:SimpleTile"
    },
    "attr:TreeGrid.indentSize":{
        ref:"attr:TreeGrid.indentSize",
        flags:"IRW",
        description:"The amount of indentation (in pixels) to add to a node's icon/title for each level\n          down in this tree's hierarchy.",
        name:"indentSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["appearance"]
    },
    "attr:GridRenderer.recordCustomStyleProperty":{
        ref:"attr:GridRenderer.recordCustomStyleProperty",
        flags:"IRW",
        description:"Denotes the name of a property that can be set on records to display a custom style.\n For example if this property is set to <code>\"customStyle\"</code>, setting \n <code>record.customStyle</code> to a css styleName will cause the record in question to\n render out with that styling applied to it.  Note that this will be a static \n style - it will not be modified as the state of the record (selected / over etc) changes.",
        name:"recordCustomStyleProperty",
        seeAlso:[
            {ref:"method:GridRenderer.getCellStyle", description:null}
        ],
        valueType:"\"customStyle\"",
        defaultValue:"string",
        type:"attr",
        definingClass:"class:GridRenderer"
    },
    "method:GridRenderer.cellIsEnabled":{
        ref:"method:GridRenderer.cellIsEnabled",
        flags:"A",
        description:" Whether this cell should be considered enabled.  Affects whether events will fire for the\n cell, and the default styling behavior in getCellStyle. \n\n",
        name:"cellIsEnabled",
        seeAlso:[],
        params:[
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether this record is enabled or not", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["selection", "appearance"]
    },
    "attr:Layout.align":{
        ref:"attr:Layout.align",
        flags:"IRW",
        description:"Alignment of all members in this Layout on the length axis (vertical for a VLayout,\n horizontal for an HLayout).  Defaults to \"top\" for vertical Layouts, and \"left\" for\n horizontal Layouts.\n <P>\n For alignment on the breadth axis, see ${isc.DocUtils.linkForRef('attr:Layout.defaultLayoutAlign')} and\n ${isc.DocUtils.linkForRef('attr:Canvas.layoutAlign')}.\n <P>\n When attempting to center components be sure that you have set a specific size on the\n component(s) involved.  If components fill all available space in the layout, centering\n looks the same as not centering.\n <P>\n Similarly, if a component has no visible boundary (like a border), it can appear similar\n to when it's not centered if the component is larger than you expect - use the Watch tab\n in the Developer Console to see the component's extents visually.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment or VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "method:StatefulCanvas.getActionType":{
        ref:"method:StatefulCanvas.getActionType",
        flags:"A",
        description:" Return the 'actionType' for this canvas (radio / checkbox / button)\n",
        name:"getActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state", "event handling"]
    },
    "attr:TreeGrid.cantDragIntoSelfMessage":{
        ref:"attr:TreeGrid.cantDragIntoSelfMessage",
        flags:"IR",
        description:"Message displayed when user attempts to drop a dragged node onto itself.",
        name:"cantDragIntoSelfMessage",
        seeAlso:[
            {ref:"attr:TreeGrid.canDragRecordsOut", description:null},
            {ref:"attr:TreeGrid.canAcceptDroppedRecords", description:null},
            {ref:"attr:TreeGrid.canReorderRecords", description:null}
        ],
        valueType:"String",
        defaultValue:"\"You can't drag an item into itself.\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["i18nMessages"]
    },
    "method:CubeGrid.getEventRow":{
        ref:"method:CubeGrid.getEventRow",
        flags:"",
        description:" Returns the row number of the most recent mouse event.\n",
        name:"getEventRow",
        seeAlso:[],
        params:[
            {description:"optional y-coordinate to obtain row number, in lieue of the y                        coordinate of the last mouse event",
             optional:true, name:"y", defaultValue:null, type:"number"}
        ],
        returns:{description:"row number, or -2 if beyond last drawn row", type:"number"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events", "selection"]
    },
    "attr:DynamicForm.fixedColWidths":{
        ref:"attr:DynamicForm.fixedColWidths",
        flags:"IRW",
        description:"If true, we ensure that column widths are at least as large as you specify them.  This\n means that if any single column overflows (due to, eg, a long unbreakable title),\n the form as a whole overflows.\n <P>\n If false, columns will have their specified sizes as long as no column overflows.  If\n any column overflows, space will be taken from any other columns that aren't filling the\n available room, until there is no more free space, in which case the form as a whole\n overflows.",
        name:"fixedColWidths",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "attr:ListGrid.valueIconSize":{
        ref:"attr:ListGrid.valueIconSize",
        flags:"IRW",
        description:"Default width and height of value icons for this ListGrid.\n Can be overridden at the listGrid level via explicit ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconHeight')}, or at the field level via ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconSize')},\n ${isc.DocUtils.linkForRef('attr:ListGridField.valueIconWidth')} and {ListGridField.valueIconHeight}",
        name:"valueIconSize",
        seeAlso:[
            {ref:"attr:ListGrid.valueIconWidth", description:null},
            {ref:"attr:ListGrid.valueIconHeight", description:null},
            {ref:"attr:ListGridField.valueIconSize", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:CubeGrid.facetValueHoverHeight":{
        ref:"attr:CubeGrid.facetValueHoverHeight",
        flags:"IRWA",
        description:"If specified and <code>this.showHover</code> is true, this is the default height to apply\n to hover tips shown for facetValues. If unset, hover canvas will be sized to \n <code>this.hoverHeight</code> if specified instead.",
        name:"facetValueHoverHeight",
        seeAlso:[
            {ref:"attr:Canvas.hoverHeight", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "method:Tree.openFolder":{
        ref:"method:Tree.openFolder",
        flags:"",
        description:"\n Open a particular node\n\n",
        name:"openFolder",
        seeAlso:[
            {ref:"method:ResultTree.dataArrived", description:null}
        ],
        params:[
            {description:"node to open", optional:false, name:"node", defaultValue:null, type:"TreeNode"},
            {description:"Optional callback (stringMethod) to fire when loading                       completes. Has a single param <code>node</code> - the node whose \n                      children have been loaded, and is fired in the scope of the Tree.",
             optional:true, name:"callback", defaultValue:null, type:"callback"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:Window.header":{
        ref:"attr:Window.header",
        flags:"R",
        description:"Header for the Window, based on an HLayout. The header contains the title and some standard\n controls for the window, which may be configured via ${isc.DocUtils.linkForRef('attr:Window.headerControls')}.",
        name:"header",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:JSONEncoder.prettyPrint":{
        ref:"attr:JSONEncoder.prettyPrint",
        flags:"IR",
        description:"Whether to add indentation to the returned JSON string.  This makes the returned JSON much\n easier to read but adds size.  Note that when delivering JSON responses compressed, the size\n difference between prettyPrinted JSON and normal JSON is negligible.",
        name:"prettyPrint",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:JSONEncoder"
    },
    "attr:Canvas.showShadow":{
        flags:"IR",
        type:"attr",
        ref:"attr:Canvas.showShadow",
        description:"Whether to show a drop shadow for this Canvas",
        name:"showShadow",
        setter:["setShowShadow"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('shadows')}"],
        defaultValue:"false",
        groups:["shadow"],
        definingClass:"class:Canvas"
    },
    "class:SpinnerItem":{
        methods:[],
        ref:"class:SpinnerItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:SpinnerItem.mask", "attr:SpinnerItem.maskSaveLiterals", 
         "attr:SpinnerItem.maskPadChar","attr:SpinnerItem.maskPromptChar", "attr:SpinnerItem.maskOverwriteMode", 
         "attr:SpinnerItem.step","attr:SpinnerItem.max", "attr:SpinnerItem.min"],
        description:"Item for picking a number. Includes arrow buttons to increase / decrease the value",
        name:"SpinnerItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('spinnerItem')}"]
    },
    "method:Class.createAutoChild":{
        ref:"method:Class.createAutoChild",
        flags:"",
        description:" Unconditionally creates and returns a component created according to the \"AutoChild\"\n pattern.\n <P>\n In addition to applying defaults and properties as described under the\n ${isc.DocUtils.linkForRef('type:AutoChild','AutoChild overview')}, the created autoChild:\n <ul>\n <li> is automatically <code>autoDraw:false</code> \n <li> has a <code>creator</code> property that points to this component, for easy\n authoring of event handlers (eg click:\"this.creator.doSomething()\")\n </ul>\n <P>\n Unlike ${isc.DocUtils.linkForRef('method:Class.addAutoChild')}, <code>createAutoChild()</code> does not create a\n this.<i>autoChildName</i> reference to the component, check a show<i>AutoChildName</i>\n flag, or automatically add the autoChild via ${isc.DocUtils.linkForRef('method:Canvas.addChild')}.  \n <P>\n General you use <code>createAutoChild</code> rather than addAutoChild when:\n <ul>\n <li> you are going to create several autoChildren with a common set of defaults (for\n example the ${isc.DocUtils.linkForRef('attr:ColumnTree.column','column')} autoChild of the ${isc.DocUtils.linkForRef('class:ColumnTree')}).\n <li> children need to be created before their parents (eg, for layout/auto-sizing\n reasons)\n </ul>\n\n",
        name:"createAutoChild",
        seeAlso:[],
        params:[
            {description:"name of the autoChild", optional:false, name:"childName", defaultValue:null,
             type:"String"},
            {description:"dynamic properties for the autoChild", optional:false, name:"defaults",
             defaultValue:null, type:"Properties"}
        ],
        returns:{description:"created autoChild", type:"Class"},
        type:"method",
        definingClass:"class:Class",
        groups:["autoChildren"]
    },
    "attr:FilterBuilder.criteria":{
        ref:"attr:FilterBuilder.criteria",
        flags:"IRW",
        getter:["getCriteria"],
        description:"Initial criteria.\n <P>\n When initialized with criteria, appropriate clauses for editing the provided criteria will\n be automatically generated.\n <P>\n Note that empty or partial criteria are allowed, for example, criteria that specify\n ${isc.DocUtils.linkForRef('attr:Criterion.fieldName')} only will generate an expression with the operator not chosen\n yet, and a ${isc.DocUtils.linkForRef('object:Criterion')} with a logical operator (\"and\" or \"or\") but not\n ${isc.DocUtils.linkForRef('attr:Criterion.criteria','subcriteria')} defined will generate an empty subclause.",
        name:"criteria",
        setter:["setCriteria"],
        seeAlso:[],
        valueType:"AdvancedCriteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "classMethod:isc.warn":{
        ref:"classMethod:isc.warn",
        flags:"",
        description:" Show a modal dialog with a message, icon, and \"OK\" button.\n <P>\n The callback will receive boolean true for an OK button click, or null if the Dialog is\n dismissed via the close button.\n\n",
        name:"warn",
        seeAlso:[
            {ref:"classAttr:Dialog.Warn", description:null},
            {ref:"classMethod:isc.say", description:null},
            {ref:"classMethod:isc.ask", description:null},
            {ref:"method:Dialog.okClick", description:null},
            {ref:"classAttr:Dialog.WARN_TITLE", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"Optional Callback to fire when the user                                       dismisses the dialog. This has the single parameter\n                                      'value', indicating the value returned by the Warn\n                                      dialog from 'okClick()' etc.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog.                                      To set ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons','custom buttons')} for\n                                      the Dialog, set properties.toolbarButtons to \n                                      an array of buttons\n										eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "method:FacetValue.getCellValue":{
        ref:"method:FacetValue.getCellValue",
        flags:"",
        description:" Callout to determine custom value to display for cells displayed for this facetValue\n <P>\n Can only be set on the ${isc.DocUtils.linkForRef('attr:CubeGrid.metricFacetId','metric facet')}, or if no metric facet\n is specified, on the innermost column facet.\n\n",
        name:"getCellValue",
        seeAlso:[],
        params:[
            {description:"this facetValue's CubeGrid", optional:false, name:"viewer", defaultValue:null,
             type:"CubeGrid"},
            {description:"cell record", optional:false, name:"record", defaultValue:null, type:"object"},
            {description:"row value for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column value for the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"HTML to display", type:"string"},
        type:"method",
        definingClass:"class:FacetValue"
    },
    "method:TreeGrid.transferDragData":{
        ref:"method:TreeGrid.transferDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method is called to transfer a set of records that\n were dropped onto some other component.  This method is called after the set of records has\n been copied to the other component.  Whether or not this component's data is modified is \n determined by the value of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dragDataAction')}.\n <P>\n With a <code>dragDataAction</code> of \"move\", a databound component will issue \"remove\"\n dsRequests against its DataSource to actually remove the data, via\n ${isc.DocUtils.linkForRef('method:DataSource.removeData')}.\n\n",
        name:"transferDragData",
        seeAlso:[
            {ref:"method:DataBoundComponent.getDragData", description:null},
            {ref:"method:ListGrid.willAcceptDrop", description:null}
        ],
        returns:{description:"Array of objects that were dragged out of this ListGrid.", type:"Array"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "group:dragdrop":{
        ref:"group:dragdrop",
        refs:["method:Canvas.setDragTracker", "method:Canvas.willAcceptDrop", 
         "method:Canvas.getHSnapPosition","method:Canvas.getVSnapPosition", "method:Canvas.shouldSnapOnDrop", 
         "method:DataBoundComponent.transferRecords","classMethod:EventHandler.getDragRect", "classMethod:Canvas.getEventEdge", 
         "attr:GridRenderer.snapToCells","attr:GridRenderer.snapInsideBorder", "attr:TreeGrid.canDragRecordsOut", 
         "attr:TreeGrid.canAcceptDroppedRecords","attr:TreeGrid.canReorderRecords", "attr:TreeGrid.canReparentNodes", 
         "attr:TreeGrid.manyItemsImage","attr:Canvas.canDrag", "attr:Canvas.dragStartDistance", "attr:Canvas.dragIntersectStyle", 
         "attr:Canvas.canDragReposition","attr:Canvas.dragRepositionCursor", "attr:Canvas.canDragResize", "attr:Canvas.resizeFrom", 
         "attr:Canvas.edgeCursorMap","attr:Canvas.keepInParentRect", "attr:Canvas.dragAppearance", "attr:Canvas.dragType", 
         "attr:Canvas.dragTarget","attr:Canvas.showDragShadow", "attr:Canvas.dragOpacity", "attr:Canvas.canDrop", 
         "attr:Canvas.canAcceptDrop","attr:Canvas.dropTypes", "attr:Canvas.snapToGrid", "attr:Canvas.snapResizeToGrid", 
         "attr:Canvas.childrenSnapToGrid","attr:Canvas.childrenSnapResizeToGrid", "attr:Canvas.snapHGap", "attr:Canvas.snapVGap", 
         "attr:Canvas.snapHDirection","attr:Canvas.snapVDirection", "attr:Canvas.snapAxis", "attr:Canvas.snapOnDrop", 
         "attr:Window.keepInParentRect","attr:Layout.canDropComponents", "attr:Layout.dropLineThickness", 
         "attr:Layout.showDragPlaceHolder","attr:Layout.placeHolderProperties", "attr:Layout.placeHolderDefaults", 
         "type:DragAppearance","type:DragIntersectStyle"],
        description:"Dragging objects and dropping them onto other objects",
        name:"dragdrop",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Page.getUnsupportedBrowserPromptString":{
        ref:"classMethod:Page.getUnsupportedBrowserPromptString",
        flags:"",
        description:" Returns the text for the prompt shown to user from ${isc.DocUtils.linkForRef('classMethod:Page.checkBrowserAndRedirect')}\n if they are accessing this page in an unsupported\n browser and ${isc.DocUtils.linkForRef('classAttr:Page.unsupportedBrowserAction')} is set to <code>\"confirm\"</code>. May be\n overridden to return a different message.\n",
        name:"getUnsupportedBrowserPromptString",
        seeAlso:[
            {ref:"classMethod:Page.checkBrowserAndRedirect", description:null}
        ],
        returns:{description:"Unsupported browser message.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["i18nMessages"]
    },
    "classMethod:Class.logError":{
        ref:"classMethod:Class.logError",
        flags:"",
        description:" Log a message at \"error\" priority\n \n",
        name:"logError",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:ListGridField.canToggle":{
        ref:"attr:ListGridField.canToggle",
        flags:"IRWA",
        description:"Allows a boolean or ${isc.DocUtils.linkForRef('type:ValueMap','valueMapped')} field to be edited without going into\n edit mode. When this attribute is set, if clicking on the field will change the value - for\n boolean fields toggling between <code>true</code> and <code>false</code>, and for valueMapped\n fields, advancing the value to the next option in the valueMap.\n <P>\n To enable this feature, ${isc.DocUtils.linkForRef('attr:ListGrid.canEdit')} must be set to true.  Note that you can\n enable toggling only (without allowing the user to edit other fields) by just setting\n ${isc.DocUtils.linkForRef('attr:ListGrid.editEvent','grid.editEvent:\"none\"')}.\n <P>\n If ${isc.DocUtils.linkForRef('attr:ListGrid.editEvent')} is set to \"click\", when the user clicks on the field, \n the value will be toggled, and inline editing will be triggered as usual.\n Otherwise the toggled value will be saved immediately to the server, or if \n ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')} has been set to false, will be stored as an edit value\n for the record.",
        name:"canToggle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:ValuesManager.getFieldErrors":{
        ref:"method:ValuesManager.getFieldErrors",
        flags:"",
        description:" Returns any validation errors for some field in this valuesManager.\n Errors will be returned as either a string (a single error message), or an array \n of strings. If no errors are present, will return null.\n",
        name:"getFieldErrors",
        seeAlso:[],
        params:[
            {description:"fieldName to check for errors", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"error messages for the field passed in", type:"string | array of strings"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "classAttr:Operators.notInSetTitle":{
        ref:"classAttr:Operators.notInSetTitle",
        flags:"IR",
        description:"Title for the \"notInSet\" operator",
        name:"notInSetTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"is not one of\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "method:Window.addItem":{
        ref:"method:Window.addItem",
        flags:"A",
        description:"			Adds a widget to the window. \n",
        name:"addItem",
        seeAlso:[],
        params:[
            {description:"the widget to be added", optional:false, name:"item", defaultValue:null,
             type:"Canvas"}
        ],
        returns:{description:"array of widgets added", type:"array"},
        type:"method",
        definingClass:"class:Window",
        groups:["windowItems"]
    },
    "method:Dialog.doneClick":{
        ref:"method:Dialog.doneClick",
        flags:"",
        description:" Handle a click on the 'done' button of this Dialog.\n Default implementation is to hide the dialog then return <code>true</code>.\n Override to do something else.\n",
        name:"doneClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:FormItem.defaultIconSrc":{
        ref:"attr:FormItem.defaultIconSrc",
        flags:"IRWA",
        description:"Default icon image source.     \n Specify as the partial URL to an image, relative to the imgDir of this component.\n To specify image source for a specific icon use the <code>icon.src</code> property.<br>\n If this item is drawn in the disabled state, the url will be modified by adding \n \"_Disabled\" to get a disabled state image for the icon.\n If <code>icon.showOver</code> is true, this url will be modified by adding \"_Over\" to get\n an over state image for the icon.",
        name:"defaultIconSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/default_formItem_icon.gif\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "method:SelectItem.fetchData":{
        ref:"method:SelectItem.fetchData",
        flags:"",
        description:" Only applies to databound items (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}).<br>\n Performs a fetch type operation on this item's DataSource to retrieve the set of valid\n options for the item, based on the current ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')}.\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Callback to fire when the fetch completes. Callback will               fire with 4 parameters:<ul>\n  <li><code>item</code> a pointer to the form item\n  <li><code>dsResponse</code> the ${isc.DocUtils.linkForRef('class:DSResponse')} returned by the server\n  <li><code>data</code> the raw data returned by the server\n  <li><code>dsRequest</code> the ${isc.DocUtils.linkForRef('class:DSRequest')} sent to the server\n  </ul>",
             optional:false, name:"callback", defaultValue:null, type:"callback"},
            {description:"properties to apply to the              dsRequest for this fetch.",
             optional:false, name:"requestProperties", defaultValue:null, type:"dsRequest properties"}
        ],
        type:"method",
        definingClass:"class:SelectItem"
    },
    "class:FileItem":{
        methods:[],
        ref:"class:FileItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:[],
        description:"Binary data interface for use in DynamicForms. Allows users to select a single file for\n upload.\n <P>\n See the ${isc.DocUtils.linkForRef('group:upload','Upload Overview')} for information on using this control.",
        name:"FileItem",
        seeAlso:[],
        type:"class",
        groups:["upload"]
    },
    "attr:Button.showFocused":{
        ref:"attr:Button.showFocused",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  If\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is <code>true</code>, the <b><code>\"over\"</code></b>\n will be used to indicate focus. Otherwise a separate <b><code>\"focused\"</code></b> state\n will be used.",
        name:"showFocused",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:ListGridField.showValueIconOnly":{
        ref:"attr:ListGridField.showValueIconOnly",
        flags:"IRW",
        description:"If this field has a valueIcons property specified, setting this property causes \n the valueIcon for each value to be displayed in the cell without also showing the\n record's value for the field.\n <P>\n If unset the default behavior is to show the icon only if an explicit valueMap is\n specified as well in addition to a valueIcons map, otherwise show both the valueIcon and\n value for the cell.\n <P>\n Note that if this field is editable ${isc.DocUtils.linkForRef('attr:FormItem.showValueIconOnly')} will be passed\n through to editors displayed in this field.",
        name:"showValueIconOnly",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null},
            {ref:"attr:ListGridField.suppressValueIcon", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "method:GridRenderer.rowContextClick":{
        ref:"method:GridRenderer.rowContextClick",
        flags:"A",
        description:" Called when a row receives a contextclick event.\n",
        name:"rowContextClick",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:ComboBoxItem.maskPromptChar":{
        ref:"attr:ComboBoxItem.maskPromptChar",
        flags:null,
        description:"Not applicable to a ComboBoxItem.",
        name:"maskPromptChar",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:ListGrid.exportData":{
        ref:"method:ListGrid.exportData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','DataSource')} to \n retrieve data that matches the current filter and sort criteria for this component, then \n exports the resulting data to a file or window in the requested format.\n <P>\n For more information on exporting data, see ${isc.DocUtils.linkForRef('method:DataSource.exportData')}.\n\n",
        name:"exportData",
        seeAlso:[],
        params:[
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "method:ListGrid.getEditedRecord":{
        ref:"method:ListGrid.getEditedRecord",
        flags:"",
        description:" Returns the combination of unsaved edits (if any) and original values (if any) for a given\n row being edited.\n <P>\n The returned value is never null, and can be freely modified.\n\n",
        name:"getEditedRecord",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"valuesID", defaultValue:null, type:"number | Object"}
        ],
        returns:{description:"A copy of the record with unsaved edits included", type:"Object"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:FormItem.show":{
        ref:"method:FormItem.show",
        flags:"I",
        description:" Show this form item.\n <BR><BR>\n This will cause the form to redraw.  If this item had an item.showIf expression, it will\n be destroyed.\n",
        name:"show",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:Tree.lastIndexOf":{
        ref:"method:Tree.lastIndexOf",
        flags:"",
        description:" Return the position in the list of the last instance of the specified object.\n <p>\n If pos is specified, starts looking before that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"lastIndexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"earliest index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:Tree",
        groups:["access"]
    },
    "attr:DateTimeItem.inputFormat":{
        ref:"attr:DateTimeItem.inputFormat",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is <code>true</code> this property can be used to specify\n the input format for date strings. \n If unset, the input format will be determined based on the specified\n ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} if possible (see ${isc.DocUtils.linkForRef('method:DateItem.getInputFormat')}), otherwise\n picked up from the Date class (see ${isc.DocUtils.linkForRef('classMethod:Date.setInputFormat')}).\n <P>\n Should be set to a standard ${isc.DocUtils.linkForRef('type:DateInputFormat')} or\n a function which will take a date string as a parameter and return a Javascript Date object.",
        name:"inputFormat",
        seeAlso:[
            {ref:"attr:DateItem.displayFormat", description:null}
        ],
        valueType:"DateInputFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateTimeItem"
    },
    "attr:ListGridField.escapeKeyEditAction":{
        ref:"attr:ListGridField.escapeKeyEditAction",
        flags:"IRW",
        description:"What to do when a user hits escape while editing this field?<br>\n Overrides the <code>escapeKeyEditAction</code> as specified at the listGrid level while \n focus is in this field.",
        name:"escapeKeyEditAction",
        seeAlso:[],
        valueType:"EscapeKeyEditAction",
        defaultValue:"\"cancel\"",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "group:basics":{
        ref:"group:basics",
        refs:["attr:FormItem.name", "attr:FormItem.title", "attr:FormItem.defaultValue", 
         "attr:FormItem.ID","attr:FormItem.prompt", "attr:DateItem.useTextField", "attr:DateItem.useMask", 
         "attr:DateItem.maskDateSeparator","attr:DateTimeItem.useTextField", "attr:DataSourceField.name", "attr:DataSourceField.type", 
         "attr:DetailViewer.data","attr:TileGrid.showAllRecords", "attr:Canvas.ID", "attr:StatefulCanvas.title", 
         "attr:Button.title","attr:Button.wrap", "attr:ImgButton.title", "attr:StretchImgButton.title"],
        description:"Basic definition of a DataSource field",
        name:"basics",
        seeAlso:[],
        type:"group"
    },
    "method:List.last":{
        ref:"method:List.last",
        flags:"",
        description:" Return the last item in this list\n\n",
        name:"last",
        seeAlso:[],
        returns:{description:"last item in the list", type:"any"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:ListGrid.groupByField":{
        ref:"attr:ListGrid.groupByField",
        flags:"IR",
        description:"List of fields to group grid records. If only a single field is used, that field\n may be specified as a string. After initialization, use ${isc.DocUtils.linkForRef('method:ListGrid.groupBy')} \n to update the grouping field list, instead of modifying groupByField directly.",
        name:"groupByField",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"String or Array of String",
        defaultValue:"see below",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dynamicGrouping')}"],
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "group:viewState":{
        ref:"group:viewState",
        refs:["method:ListGrid.getFieldState", "method:ListGrid.setFieldState", 
         "method:ListGrid.getSelectedState","method:ListGrid.setSelectedState", "method:ListGrid.getSortState", 
         "method:ListGrid.setSortState","method:ListGrid.getViewState", "method:ListGrid.setViewState", 
         "method:TreeGrid.getOpenState","method:TreeGrid.setOpenState", "method:TreeGrid.getSelectedPaths", 
         "method:TreeGrid.setSelectedPaths","method:TreeGrid.getViewState", "method:TreeGrid.setViewState", 
         "method:TileGrid.setFieldState","method:TileGrid.getFieldState", "type:listGridFieldState", "type:listGridSelectedState", 
         "type:listGridSortState","type:listGridViewState", "type:treeGridOpenState", "type:treeGridViewState"],
        name:"viewState",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.refreshRow":{
        ref:"method:ListGrid.refreshRow",
        flags:"",
        description:" Refresh an entire row of cells without redrawing the grid.\n <P>\n The cells' values, CSS classes, and CSS text will be refreshed, to the current values\n returned by getCellValue(), getCellStyle() and getCellCSSText() respectively.\n\n",
        name:"refreshRow",
        seeAlso:[
            {ref:"method:ListGrid.refreshCellStyle", description:"to update just styling"},
            {ref:"method:ListGrid.refreshCell", description:null}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:StretchImgButton.showDisabledIcon":{
        ref:"attr:StretchImgButton.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "method:DetailViewerField.formatCellValue":{
        ref:"method:DetailViewerField.formatCellValue",
        flags:"",
        description:" Optional method to format the value to display for this field's cells. Takes precedence over\n ${isc.DocUtils.linkForRef('method:DetailViewer.formatCellValue')} for cells in this field.\n",
        name:"formatCellValue",
        seeAlso:[],
        params:[
            {description:"the raw value of the cell", optional:false, name:"value", defaultValue:null,
             type:"string"},
            {description:"the record being displayed", optional:false, name:"record", defaultValue:null,
             type:"detailViewerRecord"},
            {description:"the field being displayed", optional:false, name:"field", defaultValue:null,
             type:"detailViewerField"},
            {description:"the detailViewer containing this field", optional:false, name:"viewer",
             defaultValue:null, type:"detailViewer"}
        ],
        type:"method",
        definingClass:"object:DetailViewerField",
        groups:["i18nMessages"]
    },
    "attr:FilterBuilder.removeButton":{
        ref:"attr:FilterBuilder.removeButton",
        flags:"IR",
        description:"The clause removal ImgButton that appears after each clause if\n ${isc.DocUtils.linkForRef('attr:FilterBuilder.showRemoveButton')} is set.",
        name:"removeButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Array.findNextIndex":{
        ref:"method:Array.findNextIndex",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:Array.findIndex')}, but inspects a range from startIndex to endIndex.\n\n",
        name:"findNextIndex",
        seeAlso:[],
        params:[
            {description:"first index to consider", optional:false, name:"startIndex",
             defaultValue:null, type:"int"},
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"endIndex", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"object:Array",
        groups:["access", "find"]
    },
    "classMethod:Class.isA":{
        ref:"classMethod:Class.isA",
        flags:"",
        description:"\n Returns whether this class object is the provided class or is a subclass of the provided\n class, or implements the provided interface.\n\n",
        name:"isA",
        seeAlso:[],
        params:[
            {description:"Class name to test against", optional:false, name:"className",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true == this Class is a subclass of the provided classname", type:"boolean"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:Slider.getValue":{
        ref:"method:Slider.getValue",
        flags:"",
        description:" Returns the current slider value.\n\n",
        name:"getValue",
        seeAlso:[],
        returns:{description:"current slider value", type:"float"},
        type:"method",
        definingClass:"class:Slider"
    },
    "attr:StretchImgButton.iconOrientation":{
        ref:"attr:StretchImgButton.iconOrientation",
        flags:"IR",
        description:"If this button is showing an icon should it appear to the left or right of the title?\n valid options are <code>\"left\"</code> and <code>\"right\"</code>.",
        name:"iconOrientation",
        setter:["setIconOrientation"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:DataBoundComponent.editFormulaFieldText":{
        ref:"attr:DataBoundComponent.editFormulaFieldText",
        flags:"IRW",
        description:"Text for a menu item allowing users to edit a formula field",
        name:"editFormulaFieldText",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Edit formula...\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["i18nMessages"]
    },
    "attr:DataSourceField.name":{
        ref:"attr:DataSourceField.name",
        flags:"IR",
        description:"Name for this field.  \n <p>\n The field name is also the property in each DataSource record which holds the value for this\n field.\n <P>\n Must be unique across all fields within the DataSource.",
        name:"name",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["basics"]
    },
    "group:textDirection":{
        ref:"group:textDirection",
        refs:[],
        name:"textDirection",
        seeAlso:[],
        type:"group"
    },
    "attr:RestDataSource.jsonRecordXPath":{
        ref:"attr:RestDataSource.jsonRecordXPath",
        flags:"IR",
        description:"<code>recordXPath</code> mapping to the data node of json returned by the server.\n Applies if this.dataFormat is set to <code>\"json\"</code><br>\n The default value will pick up data from a response structured as follows:<br>\n <pre>\n {response:\n  {status:0,\n   data:[\n      {field1:\"value\", field2:\"value\"},\n      {field1:\"value\", field2:\"value\"}\n   ]\n }\n </pre>",
        name:"jsonRecordXPath",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"/response/data\"",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "method:Calendar.selectTab":{
        ref:"method:Calendar.selectTab",
        flags:"",
        description:" Fires whenever the user changes the current date, including picking a specific date or\n navigating to a new week or month.\n\n",
        name:"selectTab",
        seeAlso:[],
        params:[
            {description:"the index of the tab to select", optional:false, name:"tabnum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:Canvas.animateResizeTime":{
        ref:"attr:Canvas.animateResizeTime",
        flags:"IRWA",
        description:"Default time for performing an animated resize.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateResizeTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "class:RPCResponse":{
        methods:[],
        ref:"class:RPCResponse",
        classAttrs:["classAttr:RPCResponse.STATUS_SUCCESS", "classAttr:RPCResponse.STATUS_FAILURE", 
         "classAttr:RPCResponse.STATUS_VALIDATION_ERROR","classAttr:RPCResponse.STATUS_LOGIN_INCORRECT", 
         "classAttr:RPCResponse.STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED","classAttr:RPCResponse.STATUS_LOGIN_REQUIRED", 
         "classAttr:RPCResponse.STATUS_LOGIN_SUCCESS","classAttr:RPCResponse.STATUS_UPDATE_WITHOUT_PK_ERROR", 
         "classAttr:RPCResponse.STATUS_TRANSPORT_ERROR","classAttr:RPCResponse.STATUS_UNKNOWN_HOST_ERROR", 
         "classAttr:RPCResponse.STATUS_CONNECTION_RESET_ERROR","classAttr:RPCResponse.STATUS_SERVER_TIMEOUT"],
        classMethods:[],
        treeLocation:["Client Reference/RPC"],
        attrs:["attr:RPCResponse.data", "attr:RPCResponse.status", "attr:RPCResponse.httpResponseCode", 
         "attr:RPCResponse.clientContext","attr:RPCResponse.transactionNum"],
        description:"Encapsulates an RPC response from the server.  Instances of this class are automatically created\n and optionally passed to you in the callback you specify as part of your RPCRequest.",
        name:"RPCResponse",
        seeAlso:[
            {ref:"class:RPCRequest", description:null},
            {ref:"type:RPCCallback", description:null}
        ],
        type:"class"
    },
    "attr:DetailViewer.showEmptyMessage":{
        ref:"attr:DetailViewer.showEmptyMessage",
        flags:"IRWA",
        description:"Show ${isc.DocUtils.linkForRef('attr:DetailViewer.emptyMessage')} when there is no data to display?",
        name:"showEmptyMessage",
        seeAlso:[
            {ref:"attr:DetailViewer.emptyMessage", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["emptyMessage"]
    },
    "method:Canvas.getTop":{
        ref:"method:Canvas.getTop",
        flags:"",
        description:"			Return the top coordinate of this object, relative to its enclosing context, in pixels.\n",
        name:"getTop",
        seeAlso:[],
        returns:{description:"top coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:CubeGrid.facetLabelHoverStyle":{
        ref:"attr:CubeGrid.facetLabelHoverStyle",
        flags:"IRWA",
        description:"Allows the developer to override the css class applied to  hover tips shown for\n facet labels.  If unspecified, and <code>this.hoverStyle</code> is not null, that \n css class will be applied to facet label hovers instead.",
        name:"facetLabelHoverStyle",
        seeAlso:[
            {ref:"attr:Canvas.hoverStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:Canvas.padding":{
        ref:"attr:Canvas.padding",
        flags:"IRW",
        description:"Set the CSS padding of this component, in pixels.  Padding provides space between the\n border and the component's contents.\n <P>\n This property sets the same thickness of padding on every side.  Differing per-side\n padding can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.\n <P>\n Note that CSS padding does not affect the placement of ${isc.DocUtils.linkForRef('attr:Canvas.children')}.  To\n provide a blank area around children, either use ${isc.DocUtils.linkForRef('attr:Canvas.margin','CSS margins')} or use\n a ${isc.DocUtils.linkForRef('class:Layout')} as the parent instead, and use properties such as\n ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')} to create blank space.",
        name:"padding",
        setter:["setPadding"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "attr:FormItemIcon.prompt":{
        ref:"attr:FormItemIcon.prompt",
        flags:"IRWA",
        description:"If set, this property will be displayed as a prompt (and tooltip text) for this form\n item icon.\n <P>\n If unset the form item's <code>iconPrompt</code> property will be used instead.",
        name:"prompt",
        seeAlso:[
            {ref:"attr:FormItem.iconPrompt", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "method:Canvas.dropMove":{
        ref:"method:Canvas.dropMove",
        flags:"A",
        description:"\n Executed whenever the compatible dragged object is moved over this drop target. You\n can use this to show a custom visual indication of where the drop would occur within the\n widget.\n\n",
        name:"dropMove",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customDrag')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:Class.logIsErrorEnabled":{
        ref:"method:Class.logIsErrorEnabled",
        flags:"",
        description:" Check whether a message logged at \"error\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsErrorEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "classAttr:Dialog.Warn":{
        ref:"classAttr:Dialog.Warn",
        flags:"A",
        description:"A singleton Dialog instance that will show text to the user and provide buttons for their\n response.  The Dialog will expand to show all the text that you put into it.<br>\n This can be used in cases where a developer would alternatively make use of the native\n JavaScript <code>alert()</code> and <code>confirm()</code> methods.  The main differences\n between those methods and using the Warn object are:<br>\n - The Warn object can be customized by modifying which buttons are visible, the style \n   applied to it, etc.<br>\n - The <code>isc.ask()</code> and <code>isc.warn()</code> methods are asynchronous - rather \n   than returning a value indicating the user's response, a callback method will be fired\n   when the user interacts with the dialog.<br><br>\n\n Notes:<br>\n  Because this is a singleton object, properties set on the Warn object directly will persist \n  each time it is shown.<br>\n  Developers should use the <code>warn()</code> or <code>ask()</code> methods to show and\n  hide this object rather than manipulating the Dialog directly.",
        name:"Warn",
        seeAlso:[
            {ref:"classMethod:isc.warn", description:null},
            {ref:"classMethod:isc.ask", description:null}
        ],
        valueType:"Dialog Properties",
        defaultValue:"dialog instance properties",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["Prompting"]
    },
    "method:ListGrid.setFieldError":{
        ref:"method:ListGrid.setFieldError",
        flags:"",
        description:" Set a validation error for some cell.\n",
        name:"setFieldError",
        seeAlso:[
            {ref:"method:ListGrid.getCellErrors", description:null},
            {ref:"method:ListGrid.setRowErrors", description:null}
        ],
        params:[
            {description:"row index of cell to add validation error for", optional:false,
             name:"rowNum", defaultValue:null, type:"number"},
            {description:"col index or field name of cell to add validation error for",
             optional:false, name:"fieldName", defaultValue:null, type:"number | string"},
            {description:"validation error/errors for the cell.", optional:false, name:"errorMessage",
             defaultValue:null, type:"string | array of strings"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:ComboBoxItem.displayField":{
        ref:"attr:ComboBoxItem.displayField",
        flags:"IRWA",
        getter:["FormItem.getDisplayFieldName"],
        description:"Specifies an alternative field from which display values should be retrieved for this\n item.\n <P>\n The display field can be either another field value in the same record or a field that\n must be retrieved from a related ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}.\n <P>\n If this item is not databound (${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is unset), or bound \n to the same dataSource as the form as a whole, this item will call\n ${isc.DocUtils.linkForRef('method:DynamicForm.getValue','form.getValue()')} \n the form named after is implemented by picking up the\n value of the specified field from the Form's values object.\n <P>\n Otherwise this item will attempt to map its underlying value to a display value\n by retrieving a record from the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} where the \n ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} matches this item's value, and displaying the \n <code>displayField</code> value from that record.\n Note that if <code>optionDataSource</code> is set and this value is not\n set, ${isc.DocUtils.linkForRef('method:FormItem.getDisplayFieldName')} will return the dataSource title field by default.\n <P>\n This essentially enables the specified <code>optionDataSource</code> to be used as\n a server based ${isc.DocUtils.linkForRef('group:valueMap')}.",
        name:"displayField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem",
        groups:["display_values"]
    },
    "method:Calendar.filterData":{
        ref:"method:Calendar.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:Calendar",
        groups:["dataBoundComponentMethods"]
    },
    "method:StatefulCanvas.setTitle":{
        ref:"method:StatefulCanvas.setTitle",
        flags:"",
        description:" Set the title.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["appearance"]
    },
    "attr:HeaderItem.startRow":{
        ref:"attr:HeaderItem.startRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "attr:FormItem.errorIconSrc":{
        ref:"attr:FormItem.errorIconSrc",
        flags:"IRW",
        description:"URL of the image to show as an error icon, if we're showing icons when validation\n errors occur.",
        name:"errorIconSrc",
        seeAlso:[
            {ref:"attr:FormItem.showErrorIcon", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/validation_error_icon.png\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["errorIcon"]
    },
    "attr:Hilite.htmlBefore":{
        ref:"attr:Hilite.htmlBefore",
        flags:"IR",
        description:"HTML to prepend to cell values where this hilite is applied.",
        name:"htmlBefore",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "classMethod:ClassFactory.newInstance":{
        ref:"classMethod:ClassFactory.newInstance",
        flags:"",
        description:"\n Given the name of a class, create an instance of that class.\n	\n",
        name:"newInstance",
        seeAlso:[],
        params:[
            {description:"Name of a class.							(ClassObject)	Actual class object to use.",
             optional:false, name:"className", defaultValue:null, type:"string"},
            {description:"Properties to apply to the instance.", optional:true, name:"props",
             defaultValue:null, type:"object"},
            {description:"More properties to apply to the instance.", optional:true, name:"props2",
             defaultValue:null, type:"object"},
            {description:"Yet more properties to apply to the instance.", optional:true, name:"props3",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"Pointer to the new class.", type:"class"},
        type:"classMethod",
        definingClass:"object:ClassFactory"
    },
    "group:i18nMessages":{
        ref:"group:i18nMessages",
        refs:["method:DynamicForm.formSubmitFailed", "method:DetailViewerField.formatCellValue", 
         "method:DetailViewer.formatCellValue","method:ListGrid.getRequiredFieldMessage", "method:ListGrid.getSummaryTitle", 
         "method:ListGrid.getToggleFreezeText","method:ListGrid.getGroupByText", "method:FormulaBuilder.getSamplePrompt", 
         "classAttr:Validator.notABoolean","classAttr:Validator.notAString", "classAttr:Validator.notAnInteger", 
         "classAttr:Validator.notADecimal","classAttr:Validator.notADate", "classAttr:Validator.mustBeLessThan", 
         "classAttr:Validator.mustBeGreaterThan","classAttr:Validator.mustBeLaterThan", "classAttr:Validator.mustBeEarlierThan", 
         "classAttr:Validator.mustBeShorterThan","classAttr:Validator.mustBeLongerThan", "classAttr:Validator.mustBeExactLength", 
         "classAttr:Validator.requiredField","classAttr:Validator.notOneOf", "classAttr:RPCManager.defaultPrompt", 
         "classAttr:RPCManager.timeoutErrorMessage","classAttr:RPCManager.removeDataPrompt", "classAttr:RPCManager.saveDataPrompt", 
         "classAttr:RPCManager.fetchDataPrompt","classAttr:Operators.equalsTitle", "classAttr:Operators.notEqualTitle", 
         "classAttr:Operators.greaterThanTitle","classAttr:Operators.lessThanTitle", "classAttr:Operators.greaterOrEqualTitle", 
         "classAttr:Operators.lessOrEqualTitle","classAttr:Operators.betweenTitle", "classAttr:Operators.betweenInclusiveTitle", 
         "classAttr:Operators.iContainsTitle","classAttr:Operators.iStartsWithTitle", "classAttr:Operators.iEndsWithTitle", 
         "classAttr:Operators.containsTitle","classAttr:Operators.startsWithTitle", "classAttr:Operators.endsWithTitle", 
         "classAttr:Operators.iNotContainsTitle","classAttr:Operators.iNotStartsWithTitle", "classAttr:Operators.iNotEndsWithTitle", 
         "classAttr:Operators.notContainsTitle","classAttr:Operators.notStartsWithTitle", "classAttr:Operators.notEndsWithTitle", 
         "classAttr:Operators.isNullTitle","classAttr:Operators.notNullTitle", "classAttr:Operators.regexpTitle", 
         "classAttr:Operators.iregexpTitle","classAttr:Operators.inSetTitle", "classAttr:Operators.notInSetTitle", 
         "classAttr:Operators.equalsFieldTitle","classAttr:Operators.notEqualFieldTitle", "classAttr:Operators.andTitle", 
         "classAttr:Operators.notTitle","classAttr:Operators.orTitle", "classAttr:Date.shortDayNames", 
         "classAttr:Date.shortMonthNames","classAttr:Time.AMIndicator", "classAttr:Time.PMIndicator", 
         "classAttr:Dialog.OK_BUTTON_TITLE","classAttr:Dialog.APPLY_BUTTON_TITLE", "classAttr:Dialog.YES_BUTTON_TITLE", 
         "classAttr:Dialog.NO_BUTTON_TITLE","classAttr:Dialog.CANCEL_BUTTON_TITLE", "classAttr:Dialog.DONE_BUTTON_TITLE", 
         "classAttr:Dialog.CONFIRM_TITLE","classAttr:Dialog.SAY_TITLE", "classAttr:Dialog.WARN_TITLE", "classAttr:Dialog.ASK_TITLE", 
         "classAttr:Dialog.ASK_FOR_VALUE_TITLE","classAttr:LoginDialog.LOGIN_TITLE", "classAttr:LoginDialog.USERNAME_TITLE", 
         "classAttr:LoginDialog.PASSWORD_TITLE","classAttr:LoginDialog.LOGIN_BUTTON_TITLE", "classAttr:LoginDialog.LOGIN_ERROR_MESSAGE", 
         "classAttr:GroupingMessages.upcomingTodayTitle","classAttr:GroupingMessages.upcomingTomorrowTitle", 
         "classAttr:GroupingMessages.upcomingThisWeekTitle","classAttr:GroupingMessages.upcomingNextWeekTitle", 
         "classAttr:GroupingMessages.upcomingNextMonthTitle","classAttr:GroupingMessages.upcomingBeforeTitle", 
         "classAttr:GroupingMessages.upcomingLaterTitle","classAttr:GroupingMessages.byDayTitle", "classAttr:GroupingMessages.byWeekTitle", 
         "classAttr:GroupingMessages.byMonthTitle","classAttr:GroupingMessages.byQuarterTitle", "classAttr:GroupingMessages.byYearTitle", 
         "classAttr:GroupingMessages.byDayOfMonthTitle","classAttr:GroupingMessages.byUpcomingTitle", "classAttr:GroupingMessages.byHoursTitle", 
         "classAttr:GroupingMessages.byMinutesTitle","classAttr:GroupingMessages.bySecondsTitle", 
         "classAttr:GroupingMessages.byMilisecondsTitle","classMethod:Page.getUnsupportedBrowserPromptString", "attr:DateChooser.todayButtonTitle", 
         "attr:DateChooser.cancelButtonTitle","attr:DynamicForm.errorsPreamble", "attr:DynamicForm.unknownErrorMessage", 
         "attr:DynamicForm.formSubmitFailedWarning","attr:PickListMenu.emptyMessage", "attr:PickList.emptyPickListMessage", 
         "attr:SelectOtherItem.otherTitle","attr:SelectOtherItem.selectOtherPrompt", "attr:DateItem.invalidDateStringMessage", 
         "attr:DateItem.pickerIconPrompt","attr:ValuesManager.unknownErrorMessage", "attr:FilterBuilder.removeButtonPrompt", 
         "attr:FilterBuilder.addButtonPrompt","attr:FilterBuilder.rangeSeparator", "attr:FilterBuilder.subClauseButtonTitle", 
         "attr:FilterBuilder.subClauseButtonPrompt","attr:MultiSortPanel.addLevelButtonTitle", "attr:MultiSortPanel.deleteLevelButtonTitle", 
         "attr:MultiSortPanel.copyLevelButtonTitle","attr:MultiSortPanel.invalidListPrompt", "attr:MultiSortPanel.propertyFieldTitle", 
         "attr:MultiSortPanel.directionFieldTitle","attr:MultiSortPanel.ascendingTitle", "attr:MultiSortPanel.descendingTitle", 
         "attr:MultiSortPanel.firstSortLevelTitle","attr:MultiSortPanel.otherSortLevelTitle", "attr:MultiSortDialog.addLevelButtonTitle", 
         "attr:MultiSortDialog.deleteLevelButtonTitle","attr:MultiSortDialog.copyLevelButtonTitle", "attr:MultiSortDialog.applyButtonTitle", 
         "attr:MultiSortDialog.cancelButtonTitle","attr:MultiSortDialog.invalidListPrompt", "attr:Calendar.invalidDateMessage", 
         "attr:Calendar.dayViewTitle","attr:Calendar.weekViewTitle", "attr:Calendar.monthViewTitle", 
         "attr:Calendar.timelineViewTitle","attr:Calendar.eventNameFieldTitle", "attr:Calendar.saveButtonTitle", 
         "attr:Calendar.detailsButtonTitle","attr:Calendar.cancelButtonTitle", "attr:Calendar.previousButtonHoverText", 
         "attr:Calendar.nextButtonHoverText","attr:Calendar.addEventButtonHoverText", "attr:Calendar.datePickerHoverText", 
         "attr:Selection.selectionRangeNotLoadedMessage","attr:GridRenderer.emptyMessage", "attr:ListGrid.emptyMessage", 
         "attr:ListGrid.loadingDataMessage","attr:ListGrid.loadingMessage", "attr:ListGrid.removeFieldTitle", 
         "attr:ListGrid.cancelEditingConfirmationMessage","attr:ListGrid.confirmDiscardEditsMessage", "attr:ListGrid.discardEditsSaveButtonTitle", 
         "attr:ListGrid.freezeOnRightText","attr:ListGrid.freezeOnLeftText", "attr:ListGrid.sortFieldAscendingText", 
         "attr:ListGrid.sortFieldDescendingText","attr:ListGrid.clearSortFieldText", "attr:ListGrid.clearAllSortingText", 
         "attr:ListGrid.clearFilterText","attr:ListGrid.fieldVisibilitySubmenuTitle", "attr:ListGrid.freezeFieldText", 
         "attr:ListGrid.unfreezeFieldText","attr:ListGrid.groupByText", "attr:ListGrid.ungroupText", 
         "attr:TreeGrid.parentAlreadyContainsChildMessage","attr:TreeGrid.cantDragIntoSelfMessage", "attr:TreeGrid.cantDragIntoChildMessage", 
         "attr:MenuButton.title","attr:FormulaBuilder.autoHideCheckBoxLabel", "attr:FormulaBuilder.helpTextIntro", 
         "attr:FormulaBuilder.instructionsTextStart","attr:FormulaBuilder.samplePrompt", "attr:SummaryBuilder.autoHideCheckBoxLabel", 
         "attr:SummaryBuilder.helpTextIntro","attr:DataBoundComponent.addFormulaFieldText", 
         "attr:DataBoundComponent.editFormulaFieldText","attr:DataBoundComponent.addSummaryFieldText", 
         "attr:DataBoundComponent.editSummaryFieldText","attr:Window.title", "attr:Button.title"],
        description:"The properties in this group are default system messages that a localized \n application will want to override on a per-locale basis",
        name:"i18nMessages",
        seeAlso:[
            {ref:"group:i18n", description:null}
        ],
        type:"group"
    },
    "attr:StatefulCanvas.iconSize":{
        ref:"attr:StatefulCanvas.iconSize",
        flags:"IR",
        description:"Size in pixels of the icon image.\n <P>\n The <code>iconWidth</code> and <code>iconHeight</code> properties can be used to\n configure width and height separately.",
        name:"iconSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "method:ListGrid.cellChanged":{
        ref:"method:ListGrid.cellChanged",
        flags:"",
        description:" Fires after user edits have been successfully saved to the server, when the new value\n doesn't match the value before editing.\n <p>\n If you want immediate notification of a changes <b>before</b> changes has been saved to the\n server, implement ${isc.DocUtils.linkForRef('method:ListGridField.change','field.change()')} or\n ${isc.DocUtils.linkForRef('method:ListGridField.changed','field.changed()')} instead.\n <P>\n You can supply this method on the listGrid instance or on the listGridField(s) that you\n want to receive cellChanged events for.  If both a field and the listGrid define a\n cellChanged method and that field receives an edit save, only the one defined on the\n field is called.\n\n",
        name:"cellChanged",
        seeAlso:[
            {ref:"method:ListGridField.cellChanged", description:null}
        ],
        params:[
            {description:"record for the cell being changed", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"new value for the cell", optional:false, name:"newValue", defaultValue:null,
             type:"any"},
            {description:"old value for the cell", optional:false, name:"oldValue", defaultValue:null,
             type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"},
            {description:"grid where cell was changed.  Also available as \"this\"", optional:false,
             name:"grid", defaultValue:null, type:"ListGrid"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:TabSet.selectedTab":{
        ref:"attr:TabSet.selectedTab",
        flags:"IRW",
        getter:["getSelectedTab"],
        description:"Specifies the index of the initially selected tab.",
        name:"selectedTab",
        seeAlso:[],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBar"]
    },
    "method:ListGrid.getEditedCell":{
        ref:"method:ListGrid.getEditedCell",
        flags:"",
        description:" Returns the current value of a cell. If the cell has an outstanding edit value, this will\n be returned, otherwise the underlying value of the record will be returned.\n\n",
        name:"getEditedCell",
        seeAlso:[],
        params:[
            {description:"rowNum of the record being edited, or an Object                                      containing values for all the record's primary keys",
             optional:false, name:"record", defaultValue:null, type:"number | Object"},
            {description:"colNum or fieldName of the cell", optional:false, name:"field",
             defaultValue:null, type:"number | string"}
        ],
        returns:{description:"Current edit value, or underlying value for the cell", type:"any"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:visibility":{
        ref:"group:visibility",
        refs:["method:Menu.showContextMenu", "method:Menu.hideContextMenu", "method:Menu.showSubmenu", 
         "method:Canvas.show","method:Canvas.hide", "method:Canvas.isVisible", "attr:Calendar.showDateChooser", 
         "attr:Calendar.disableWeekends","attr:Calendar.showWeekends", "attr:Calendar.showDayHeaders", 
         "attr:Calendar.showOtherDays","type:Visibility"],
        description:"Whether an object can be seen",
        name:"visibility",
        seeAlso:[],
        type:"group"
    },
    "attr:SectionStackSection.items":{
        ref:"attr:SectionStackSection.items",
        flags:"I",
        description:"List of Canvases that constitute the section.  These Canvases will be shown and hidden\n together.",
        name:"items",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "class:DynamicForm":{
        exampleConfig:"DynamicForm.xml",
        methods:["method:DynamicForm.getEditorType", "method:DynamicForm.setItems", 
         "method:DynamicForm.setFields","method:DynamicForm.getFields", "method:DynamicForm.getItems", 
         "method:DynamicForm.setValues","method:DynamicForm.rememberValues", "method:DynamicForm.resetValues", 
         "method:DynamicForm.clearValues","method:DynamicForm.valuesHaveChanged", "method:DynamicForm.getValues", 
         "method:DynamicForm.getValuesAsCriteria","method:DynamicForm.getValuesAsAdvancedCriteria", "method:DynamicForm.getItem", 
         "method:DynamicForm.getField","method:DynamicForm.getValue", "method:DynamicForm.setValue", 
         "method:DynamicForm.clearValue","method:DynamicForm.showItem", "method:DynamicForm.hideItem", 
         "method:DynamicForm.setValueMap","method:DynamicForm.getErrors", "method:DynamicForm.getFieldErrors", 
         "method:DynamicForm.setErrors","method:DynamicForm.setError", "method:DynamicForm.addFieldErrors", 
         "method:DynamicForm.setFieldErrors","method:DynamicForm.clearFieldErrors", "method:DynamicForm.clearErrors", 
         "method:DynamicForm.hasErrors","method:DynamicForm.hasFieldErrors", "method:DynamicForm.getItemErrorHTML", 
         "method:DynamicForm.getTitleOrientation","method:DynamicForm.submitForm", "method:DynamicForm.formSubmitFailed", 
         "method:DynamicForm.setAction","method:DynamicForm.setTarget", "method:DynamicForm.setMethod", 
         "method:DynamicForm.validate","method:DynamicForm.valuesAreValid", "method:DynamicForm.getValidatedValues", 
         "method:DynamicForm.showErrors","method:DynamicForm.showFieldErrors", "method:DynamicForm.focusInItem", 
         "method:DynamicForm.itemHoverHTML","method:DynamicForm.titleHoverHTML", "method:DynamicForm.itemChanged", 
         "method:DynamicForm.itemChange","method:DynamicForm.itemKeyPress", "method:DynamicForm.submitValues", 
         "method:DynamicForm.handleHiddenValidationErrors","method:DynamicForm.editNewRecord", "method:DynamicForm.editRecord", 
         "method:DynamicForm.editSelectedData","method:DynamicForm.validateData", "method:DynamicForm.reset", "method:DynamicForm.cancel", 
         "method:DynamicForm.submit","method:DynamicForm.saveData", "method:DynamicForm.isNewRecord", 
         "method:DynamicForm.setSaveOperationType","method:DynamicForm.getSaveOperationType", "method:DynamicForm.fetchData", 
         "method:DynamicForm.filterData"],
        type:"class",
        ref:"class:DynamicForm",
        classAttrs:[],
        treeLocation:["Client Reference/Forms"],
        classMethods:[],
        description:"The DynamicForm manages a collection of FormItems which represent user input controls.  The\n DynamicForm provides layout, value management, validation and databinding for the controls\n it manages.\n <P>\n To create a DynamicForm, set ${isc.DocUtils.linkForRef('attr:DynamicForm.fields')} to an Array of Objects describing the\n FormItems you want to use.  For example:\n <pre>\n    isc.DynamicForm.create({\n        fields:[\n            {name:\"userName\", type:\"text\"},  // creates a TextItem\n            {name:\"usState\", type:\"select\"}  // creates a SelectItem\n        ]\n    })\n </pre>\n The item <code>name</code> is an identifier for the item that must be unique just within\n this form.  It is used:\n <ul>\n <li> as the property name under which the item's value is stored in the form (the form's\n      current values are accessible as ${isc.DocUtils.linkForRef('method:DynamicForm.getValues','form.getValues()')}\n <li> when retrieving the FormItem's current value (via\n      ${isc.DocUtils.linkForRef('method:DynamicForm.getValue','form.getValue()')}) \n <li> to retrieve the item itself via ${isc.DocUtils.linkForRef('method:DynamicForm.getItem','form.getItem()')}\n </ul>\n The item <code>type</code> controls what kind of FormItem is created.  See\n ${isc.DocUtils.linkForRef('type:FormItemType')}.\n <P>\n The ${isc.DocUtils.linkForRef('attr:FormItem.title','title')} and ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue','defaultValue')} are also\n commonly specified.  All FormItems share a common set of properties for controlling\n ${isc.DocUtils.linkForRef('group:formLayout','form layout')}.  Other properties common to all FormItems are\n documented on the ${isc.DocUtils.linkForRef('class:FormItem')} class, and properties specific to particular FormItems are\n documented on the respective FormItems.  \n <P>\n NOTE: For very simple forms consisting of exactly one item, you still use a DynamicForm.\n See the \"fontSelector\" form in the ${isc.DocUtils.linkForExampleId('toolstrip', 'Toolstrip example')}.",
        attrs:["attr:DynamicForm.items", "attr:DynamicForm.fields", "attr:DynamicForm.values", 
         "attr:DynamicForm.itemLayout","attr:DynamicForm.numCols", "attr:DynamicForm.fixedColWidths", 
         "attr:DynamicForm.colWidths","attr:DynamicForm.minColWidth", "attr:DynamicForm.cellPadding", 
         "attr:DynamicForm.cellBorder","attr:DynamicForm.sectionVisibilityMode", "attr:DynamicForm.longTextEditorThreshold", 
         "attr:DynamicForm.longTextEditorType","attr:DynamicForm.valuesManager", "attr:DynamicForm.titleOrientation", 
         "attr:DynamicForm.titlePrefix","attr:DynamicForm.rightTitlePrefix", "attr:DynamicForm.titleSuffix", 
         "attr:DynamicForm.rightTitleSuffix","attr:DynamicForm.titleWidth", "attr:DynamicForm.wrapItemTitles", 
         "attr:DynamicForm.showInlineErrors","attr:DynamicForm.showErrorIcons", "attr:DynamicForm.showErrorText", 
         "attr:DynamicForm.showErrorStyle","attr:DynamicForm.errorOrientation", "attr:DynamicForm.errorItemProperties", 
         "attr:DynamicForm.errorItemCellStyle","attr:DynamicForm.errorsPreamble", "attr:DynamicForm.showTitlesWithErrorMessages", 
         "attr:DynamicForm.hiliteRequiredFields","attr:DynamicForm.requiredTitlePrefix", "attr:DynamicForm.requiredRightTitlePrefix", 
         "attr:DynamicForm.requiredTitleSuffix","attr:DynamicForm.requiredRightTitleSuffix", "attr:DynamicForm.itemHoverDelay", 
         "attr:DynamicForm.itemHoverWidth","attr:DynamicForm.itemHoverHeight", "attr:DynamicForm.itemHoverAlign", 
         "attr:DynamicForm.itemHoverVAlign","attr:DynamicForm.itemHoverStyle", "attr:DynamicForm.itemHoverOpacity", 
         "attr:DynamicForm.errors","attr:DynamicForm.validateOnChange", "attr:DynamicForm.rejectInvalidValueOnChange", 
         "attr:DynamicForm.unknownErrorMessage","attr:DynamicForm.autoFocus", "attr:DynamicForm.selectOnFocus", 
         "attr:DynamicForm.canFocus","attr:DynamicForm.validationURL", "attr:DynamicForm.disableValidation", 
         "attr:DynamicForm.cancelParamName","attr:DynamicForm.cancelParamValue", "attr:DynamicForm.action", "attr:DynamicForm.target", 
         "attr:DynamicForm.method","attr:DynamicForm.encoding", "attr:DynamicForm.canSubmit", "attr:DynamicForm.saveOnEnter", 
         "attr:DynamicForm.operator","attr:DynamicForm.showComplexFieldsRecursively", "attr:DynamicForm.nestedEditorType", 
         "attr:DynamicForm.nestedListEditorType","attr:DynamicForm.formSubmitFailedWarning", "attr:DynamicForm.saveOperationType", 
         "attr:DynamicForm.autoFetchData","attr:DynamicForm.autoFetchTextMatchStyle", "attr:DynamicForm.initialCriteria"],
        implementsInterfaces:["DataBoundComponent"],
        name:"DynamicForm",
        seeAlso:[]
    },
    "attr:Calendar.nameField":{
        ref:"attr:Calendar.nameField",
        flags:"IR",
        description:"The name of the name field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"nameField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"name\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "class:ButtonItem":{
        methods:["method:ButtonItem.click", "method:ButtonItem.setTitle"],
        exampleConfig:"ButtonItem.xml",
        ref:"class:ButtonItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:ButtonItem.height", "attr:ButtonItem.baseStyle", "attr:ButtonItem.icon", 
         "attr:ButtonItem.showTitle","attr:ButtonItem.startRow", "attr:ButtonItem.endRow", "attr:ButtonItem.buttonConstructor", 
         "attr:ButtonItem.autoFit","attr:ButtonItem.buttonProperties"],
        description:"FormItem for adding a Button to a form.",
        name:"ButtonItem",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.canHover":{
        ref:"attr:ListGrid.canHover",
        flags:"IRW",
        description:"If true, cellHover and rowHover events will fire when the user leaves the mouse over a \n row / cell.",
        name:"canHover",
        seeAlso:[
            {ref:"attr:ListGrid.showHover", description:null},
            {ref:"attr:ListGridField.showHover", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('valueHoverTips')}"],
        definingClass:"class:ListGrid",
        groups:["hovers"]
    },
    "class:Menu":{
        methods:["method:Menu.itemClick", "method:Menu.showContextMenu", "method:Menu.setData", 
         "method:Menu.setItems","method:Menu.hideContextMenu", "method:Menu.getItem", "method:Menu.setItemProperties", 
         "method:Menu.getItemNum","method:Menu.getItems", "method:Menu.showSubmenu", "method:Menu.getSubmenu", 
         "method:Menu.setItemEnabled","method:Menu.setItemChecked", "method:Menu.setItemTitle", "method:Menu.setItemIcon"],
        ref:"class:Menu",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:Menu.data", "attr:Menu.items", "attr:Menu.target", "attr:Menu.fields", 
         "attr:Menu.defaultWidth","attr:Menu.cellHeight", "attr:Menu.autoDraw", "attr:Menu.useKeys", "attr:Menu.showKeys", 
         "attr:Menu.showIcons","attr:Menu.showSubmenus", "attr:Menu.submenuDirection", "attr:Menu.emptyMessage", 
         "attr:Menu.iconWidth","attr:Menu.iconHeight", "attr:Menu.showAnimationEffect", "attr:Menu.submenuImage", 
         "attr:Menu.submenuDisabledImage","attr:Menu.checkmarkImage", "attr:Menu.checkmarkDisabledImage", 
         "attr:Menu.canSelectParentItems","attr:Menu.autoDismiss", "attr:Menu.menuButtonWidth"],
        description:"The Menu widget class implements interactive menu widgets, with optional icons, submenus,\n and shortcut keys.\n <p>\n A Menu is initialized with an Array of items, specified as menu.data, each of which represents\n one row in the menu's display and specifies the action to take when that menu item is selected.\n <p>\n Generally to create a context menu for a component, provide a Menu instance for the \n <code>contextMenu</code> property.  Note that some components have special context menu support\n because they have distinct regions or because they have a default set of context menu actions\n available.\n <p>\n If you want a button that pops up a menu when clicked, or a bar of such buttons, see the\n MenuButton and MenuBar classes.",
        name:"Menu",
        seeAlso:[
            {ref:"attr:Menu.data", description:null},
            {ref:"attr:Canvas.contextMenu", description:null},
            {ref:"class:MenuButton", description:null},
            {ref:"class:MenuBar", description:null}
        ],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"]
    },
    "attr:SelectOtherItem.otherTitle":{
        ref:"attr:SelectOtherItem.otherTitle",
        flags:"IRW",
        description:"Title for the <code>Other...</code> item. When this item is selected, the user will be \n shown a prompt allowing them to enter a new value for the item.",
        name:"otherTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Other...\"",
        type:"attr",
        definingClass:"class:SelectOtherItem",
        groups:["appearance", "i18nMessages"]
    },
    "attr:BatchUploader.uploadFileLabel":{
        ref:"attr:BatchUploader.uploadFileLabel",
        flags:"IR",
        description:"Title to display next to the ${isc.DocUtils.linkForRef('class:FileItem')} field where the user enters a filename to upload",
        name:"uploadFileLabel",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Upload File\"",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:Canvas.keyUp":{
        ref:"method:Canvas.keyUp",
        flags:"A",
        description:"\n Executed when a key is released on a focusable widget (${isc.DocUtils.linkForRef('attr:Canvas.canFocus')}: true). \n <P>\n Use ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKey')} to find out the ${isc.DocUtils.linkForRef('type:KeyName','keyName')} of the key that\n was pressed, and use ${isc.DocUtils.linkForRef('classMethod:EventHandler.shiftKeyDown')} and related functions to determine\n whether modifier keys were down.\n\n",
        name:"keyUp",
        seeAlso:[
            {ref:"attr:Canvas.canFocus", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:Chart.getFacetValue":{
        ref:"method:Chart.getFacetValue",
        flags:"",
        description:" Get facet value definition by facetId and facetValueId.\n",
        name:"getFacetValue",
        seeAlso:[
            {ref:"class:FacetValue", description:null}
        ],
        params:[
            {description:"the id of the facet to retrieve", optional:false, name:"facetId",
             defaultValue:null, type:"String"},
            {description:"the id of the facet value to retrieve", optional:false, name:"facetValueId",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the FacetValue if found, or null", type:"FacetValue"},
        type:"method",
        definingClass:"interface:Chart"
    },
    "method:ListGrid.expandRecord":{
        ref:"method:ListGrid.expandRecord",
        flags:"",
        description:" Expands a given ${isc.DocUtils.linkForRef('object:ListGridRecord','record')} by creating a subcomponent and inserting it \n in to the record's grid-row.  A number of built-in ${isc.DocUtils.linkForRef('type:ExpansionMode','expansionModes')} \n are supported by the default implementation of \n ${isc.DocUtils.linkForRef('method:ListGrid.getExpansionComponent','getExpansionComponent()')} and you can override\n that method to provide your own expansion behavior.\n \n",
        name:"expandRecord",
        seeAlso:[],
        params:[
            {description:"record to expand", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:SectionStack.canResizeSections":{
        ref:"attr:SectionStack.canResizeSections",
        flags:"IRA",
        description:"Whether sections can be drag resized by the user dragging the section header.\n <P>\n Note that, with <code>canResizeSections:true</code>, not all sections can be resized:\n sections that contain only ${isc.DocUtils.linkForRef('attr:Button.autoFit','autofitting')} components or that are\n marked with ${isc.DocUtils.linkForRef('attr:SectionStackSection.resizeable','section.resizeable:false')} will not be\n resizeable.",
        name:"canResizeSections",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SectionStack"
    },
    "attr:SectionStackSection.title":{
        ref:"attr:SectionStackSection.title",
        flags:"IR",
        description:"Title to show for the section",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "attr:ImgProperties.imgDir":{
        ref:"attr:ImgProperties.imgDir",
        flags:"IRW",
        description:"Specifies the image-specific image directory to override the default.",
        name:"imgDir",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "method:ListGrid.rowHover":{
        ref:"method:ListGrid.rowHover",
        flags:"",
        description:" Called when the mouse hovers over a row if this.canHover is true. \n  Returning false will suppress the hover text from being shown if this.showHover is true.\n\n",
        name:"rowHover",
        seeAlso:[
            {ref:"attr:ListGrid.canHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event (default behavior of showing the hover)",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:CubeGrid.wrapFacetTitles":{
        ref:"attr:CubeGrid.wrapFacetTitles",
        flags:"IRW",
        description:"Whether to allow text wrapping on facet titles",
        name:"wrapFacetTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "classAttr:ValidatorTypes.serverCustom":{
        ref:"classAttr:ValidatorTypes.serverCustom",
        flags:"IR",
        description:"Evaluates the Velocity expression provided in \n  ${isc.DocUtils.linkForRef('attr:Validator.serverCondition','serverCondition')} on the server side.  By default, \n  validators of this type have ${isc.DocUtils.linkForRef('attr:Validator.serverOnly','serverOnly')} set to\n  <code>true</code> and do not run on the client.",
        name:"serverCustom",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:Window.status":{
        ref:"attr:Window.status",
        flags:"IRW",
        description:"Text to show in the status bar of the window (if one is visible)",
        name:"status",
        setter:["setStatus"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance"]
    },
    "group:drawContext":{
        ref:"group:drawContext",
        refs:["type:DrawPosition"],
        name:"drawContext",
        seeAlso:[],
        type:"group"
    },
    "group:componentSchema":{
        ref:"group:componentSchema",
        title:"Component Schema",
        treeLocation:["Concepts/Component XML"],
        refs:["attr:DataSourceField.xmlAttribute", "attr:DataSourceField.multiple", 
         "attr:DataSourceField.childTagName","attr:DataSourceField.propertiesOnly", "attr:DataSourceField.inapplicable", 
         "attr:DataSourceField.group"],
        description:"A component schema is a special type of DataSource that describes a custom component.  \n <P>\n Declaring a component schema for your custom component allows you to:\n <ul>\n <li> use simpler XML when creating your custom component: avoid having to specify the\n <code>constructor</code> and <code>xsi:type</code> attributes as described under\n ${isc.DocUtils.linkForRef('group:componentXML')}\n <li> use your custom component within ${isc.DocUtils.linkForRef('group:visualBuilder')}\n </ul>\n <P>\n <b>Example of a Component Schema</b>\n <P>\n It's most basic form, a component schema for a custom subclass of ListGrid called\n \"MyListGrid\" looks like this:\n <pre>\n &lt;DataSource serverType=\"component\" ID=\"MyListGrid\" \n             inheritsFrom=\"ListGrid\" instanceConstructor=\"MyListGrid\"/&gt;\n </pre>\n With this definition saved as \"MyListGrid.ds.xml\" in the project dataSources directory\n ([webroot]/shared/ds/ by default), you can now create an instance of MyListGrid with just:\n <pre>\n &lt;MyListGrid width=\"500\"/&gt;\n </pre>\n Note: you may need to restart your servlet engine/J2EE container before this example will\n work.\n <P>\n The attributes set directly on the DataSource tag have special meaning for a component\n schema definition:\n <ul>\n <li>${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')}=\"component\" indicates this DataSource describes\n a component, as opposed to a SQL table or other data provider\n <li>${isc.DocUtils.linkForRef('attr:DataSource.ID','ID')} means the tagName that will be used to create your custom\n component.  This must match the first component of the filename (ID=\"MyListGrid\" means the\n filename must be MyListGrid.ds.xml) and typically also matches the name of the class.\n <li>${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}=\"ListGrid\" inherits the ListGrid property definitions via\n ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')}.  \n <li>instanceConstructor=\"MyListGrid\" indicates the SmartClient class that\n ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')} should be called on to construct an instance.\n <li>showLocalFieldsOnly is a boolean that, when set to true, tells the ${isc.DocUtils.linkForRef('class:VisualBuilder')}\n to show only the fields declared in this schema in the component editor.  Otherwise fields\n inherited via ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom')} (all the way up the chain) are also included.\n <li>showSuperClassEvents is a boolean that, like showLocalFieldsOnly, optionally restricts\n the list of events shown in the Events tab of the ${isc.DocUtils.linkForRef('class:VisualBuilder')} to those defined in\n this schema only.\n <li>showSuperClassActions is a boolean that optionally restricts the list of actions shown\n in the menu when you map a component Event to a component Action within ${isc.DocUtils.linkForRef('class:VisualBuilder')}\n to those defined in this schema only.\n </ul>\n <P>\n <b>Declaring custom properties</b>\n <P>\n Custom properties are declared via ${isc.DocUtils.linkForRef('attr:DataSource.fields','fields')} as for an ordinary\n ${isc.DocUtils.linkForRef('class:DataSource')}.  As with ordinary DataSources, it is legal to redeclare inherited fields\n in order to modify properties such as ${isc.DocUtils.linkForRef('attr:DataSourceField.editorType','field.editorType')}.\n <P>\n The following DataSourceField properties have special significance when a component schema\n is used to process ${isc.DocUtils.linkForRef('group:componentXML','component XML')}:\n <ul>\n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field.type')} declares the type of the field, and hence the\n type of the JavaScript value your custom class will be initialized with.  In order to\n declare subcomponents, can be set to the name of another component (built-in or custom).  \n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.multiple','field.multiple')} declares that the field should always be\n array-valued even when only a single value is provided.  Also indicates that the field name\n should be used as a \"wrapper tag\" in the XML format for the component.\n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.propertiesOnly','field.propertiesOnly')}.  For fields that hold\n subcomponents, suppresses auto-construction to avoid ${isc.DocUtils.linkForRef('attr:Canvas.autoDraw','double drawing')}\n and to allow subcomponents to be modified by their parent component before they are created\n and drawn\n </ul>\n When a component is edited within Visual Builder, the DataSource properties that normally\n influence databound forms will influence the Component Editor (for example, field.title,\n field.editorType).  In addition, the following properties have special signficance in\n component editing and component drag and drop:\n <ul>\n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.inapplicable','field.inapplicable')} indicates that an inherited\n field is inapplicable in this component.\n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.group','field.group')} indicates what group the property should be\n placed in when editing in Visual Builder.\n <li> ${isc.DocUtils.linkForRef('attr:DataSourceField.xmlAttribute','field.xmlAttribute')}: indicates this field should\n serialize as an XML attribute.  Note that when constructing the component from XML, either\n an attribute or a subelement will continue to be accepted as means of specifying the field\n value, so this property is primarily set in order to make code generated by Visual Builder\n maximally compact or to make it conform to externally set standards.\n </ul>\n <P>\n <b>Declaring Events and Actions</b>\n <P>\n Events and Actions are declared via a methods array.  In order for a method to be considered\n an event, it needs to have a method definition in the methods array (or be publicly\n documented in the SmartClient reference) and have been added to\n the class as a ${isc.DocUtils.linkForRef('group:stringMethods','StringMethod')} via ${isc.DocUtils.linkForRef('classMethod:Class.registerStringMethods')}.\n <p>\n In order for a method to be considered an action, it needs to have a method definition in\n the methods array and have the <code>action</code> property set to <code>true</code>.  For\n example, the following is a definition of the 'hide' action available on any Canvas, as\n copied from Canvas.ds.xml:\n <pre>\n     &lt;methods&gt;\n         &lt;method name=\"hide\" title=\"Hide\" action=\"true\"/&gt;\n     &lt;/methods&gt;\n </pre>\n For custom component actions, an array of expected parameters may be specified using the\n <code>params</code> attribute. Each entry in this array should have a specified type.\n By doing this, you allow the visual builder to pass parameters through to actions when setting \n up events that call actions (possibly on another component).\n For example if you had a component with a custom action that expected to be passed a single\n parameter of type ${isc.DocUtils.linkForRef('object:ListGridRecord')} you could define it as follows:\n <pre>\n     &lt;method name=\"showRecordDetails\" title=\"Show Record Details\" action=\"true\"&gt;\n         &lt;params&gt;\n             &lt;param type=\"ListGridRecord\"/&gt;\n         &lt;params&gt;\n     &lt;/method&gt;\n </pre>\n If a user working within the visualBuilder then added ListGrid to the page and used the \"+\" icon\n to associate the ${isc.DocUtils.linkForRef('method:ListGrid.recordClick','recordClick')} event with this custom method, the\n Visual Builder logic would automatically associate the parameters available in the fired event\n (in this case <code>recordClick</code>) with the expected parameters for the action to fire \n by type. So the <code>record</code> parameter of the event (known to be of type \n <code>ListGridRecord</code>) would be passed through to this custom <i>showRecordDetails</i>\n action as the first parameter.",
        name:"componentSchema",
        seeAlso:[],
        type:"group"
    },
    "attr:LoginDialog.lostPasswordItemTitle":{
        ref:"attr:LoginDialog.lostPasswordItemTitle",
        flags:"IR",
        description:"Specifies the contents of the password request button (if configured) on\n the ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}.",
        name:"lostPasswordItemTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"LoginDialog.lostPasswordItemTitle",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:TileGrid.getDragData":{
        ref:"method:TileGrid.getDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method returns the set of records being dragged out\n of the component.  In the default implementation, this is the list of currently selected\n records.<p>\n \n This method is consulted by ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')}.\n\n",
        name:"getDragData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"}
        ],
        returns:{description:"Array of ${isc.DocUtils.linkForRef('object:Record')}s that are currently selected.",
         type:"Array of Record"},
        type:"method",
        definingClass:"class:TileGrid",
        groups:["dragging", "data"]
    },
    "attr:LoginDialog.dismissable":{
        ref:"attr:LoginDialog.dismissable",
        flags:"IR",
        description:"If true, allow the user to close the LoginDialog by pressing Escape or by pressing\n a Close button on the upper right corner (only visible when true).\n <p>If the Dialog is dismissed,\n ${isc.DocUtils.linkForRef('method:LoginDialog.loginFunc')} is called with null arguments.\n <p>Note that this attribute overrides the dismissOnEscape and showCloseButton\n attributes.",
        name:"dismissable",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "attr:ColorPicker.okButtonConstructor":{
        ref:"attr:ColorPicker.okButtonConstructor",
        flags:"IRWA",
        description:"The class of the \"OK\" button. It is intended that you use either IButton or \n      Button - other classes are unlikely to work correctly.",
        name:"okButtonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"IButton",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ImgSplitbar.hSrc":{
        ref:"attr:ImgSplitbar.hSrc",
        flags:"IR",
        description:"Default src to display when ${isc.DocUtils.linkForRef('attr:ImgSplitbar.vertical')} is false, \n and ${isc.DocUtils.linkForRef('attr:ImgSplitbar.src')} is unset.",
        name:"hSrc",
        seeAlso:[
            {ref:"attr:ImgSplitbar.src", description:null}
        ],
        valueType:"string",
        defaultValue:"[SKIN]hgrip.png",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "attr:ListGridRecord.customStyle":{
        ref:"attr:ListGridRecord.customStyle",
        flags:"IRW",
        description:"Name of a CSS style to use for all cells for this particular record.  \n <P>\n Note that using this property assigns a single, fixed style to the record, so rollover\n and selection styling are disabled.  To provide a series of stateful styles for a record\n use ${isc.DocUtils.linkForRef('attr:ListGridRecord._baseStyle')} instead.\n <P>\n See ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle')} for an overview of various ways to customize styling,\n both declarative and programmatic.\n <P>\n If this property is changed after draw(), to refresh the grid call\n ${isc.DocUtils.linkForRef('method:ListGrid.refreshRow')} (or ${isc.DocUtils.linkForRef('method:ListGrid.markForRedraw')} if several rows are\n being refreshed).\n <P>\n If your application's data uses the \"customStyle\" attribute for something else, the\n property name can be changed via ${isc.DocUtils.linkForRef('attr:ListGrid.recordCustomStyleProperty')}.",
        name:"customStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "attr:Window.restoreButton":{
        ref:"attr:Window.restoreButton",
        flags:"R",
        description:"ImgButton that restores the Window via ${isc.DocUtils.linkForRef('method:Window.restore')}.",
        name:"restoreButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:ToolStripSeparator.vSrc":{
        ref:"attr:ToolStripSeparator.vSrc",
        flags:"IRW",
        description:"Image for vertically oriented separator (for horizontal toolstrips).",
        name:"vSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]separator.png\"",
        type:"attr",
        definingClass:"class:ToolStripSeparator"
    },
    "group:headerMenuButton":{
        ref:"group:headerMenuButton",
        refs:["attr:ListGrid.showHeaderMenuButton", "attr:ListGrid.headerMenuButtonConstructor", 
         "attr:ListGrid.headerMenuButton","attr:ListGrid.headerMenuButtonIcon", "attr:ListGrid.headerMenuButtonIconWidth", 
         "attr:ListGrid.headerMenuButtonIconHeight","attr:ListGrid.headerMenuButtonWidth", "attr:ListGrid.headerMenuButtonHeight"],
        name:"headerMenuButton",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.editByCell":{
        ref:"attr:ListGrid.editByCell",
        flags:"IRW",
        description:"Determines whether when the user edits a cell in this listGrid the entire row becomes\n editable, or just the cell that received the edit event.\n <P>\n No effect if this.canEdit is false or null.",
        name:"editByCell",
        seeAlso:[
            {ref:"attr:ListGrid.canEdit", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('editByCell')}"],
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Facet.title":{
        ref:"attr:Facet.title",
        flags:"IRW",
        description:"User-visible title of this facet.  Shown on the facet label in the CubeGrid.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "method:CubeGrid.getRowFacetLayout":{
        ref:"method:CubeGrid.getRowFacetLayout",
        flags:"",
        description:" Get the current widths of the row facets, as an object of the form:\n <pre>\n [ {facetId:facetId, width:currentWidth }, ... ]\n </pre>\n\n",
        name:"getRowFacetLayout",
        seeAlso:[],
        returns:{description:"array of {facetId:facetId, width:width} objects", type:"array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:DynamicForm.titlePrefix":{
        ref:"attr:DynamicForm.titlePrefix",
        flags:"IRW",
        description:"The string prepended to the title of every item in this form.",
        name:"titlePrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "classMethod:ClassFactory.defineClass":{
        ref:"classMethod:ClassFactory.defineClass",
        flags:"",
        description:"\n Create a new SmartClient class, which can then be used to create instances of this\n object type, via ${isc.DocUtils.linkForRef('classMethod:Class.create')}.\n <P>\n The new Class is returned by <code>defineClass</code>, is available as\n <code>isc.<i>ClassName</i></code> and is also available in global scope if not in\n ${isc.DocUtils.linkForRef('class:isc','portal mode')}.  Typically, ${isc.DocUtils.linkForRef('classMethod:addProperties')} is then\n called to establish different defaults in the new class, or to add methods.  For\n example:\n <pre>\n    isc.defineClass(\"MyListGrid\", \"ListGrid\").addProperties({\n        headerHeight : 40, // change default for listGrid.headerHeight\n\n        // override listGrid.recordClick\n        recordClick : function (viewer, record) { \n           isc.say(record.description);\n        }\n    })\n    isc.MyListGrid.create(); // create an instance of the new class\n </pre>\n <P>\n See also ${isc.DocUtils.linkForRef('method:Class.Super','Super()')} for calling superclass methods.\n <P>\n NOTE: <code>isc.defineClass()</code> also creates a new function\n <code>${isc.DocUtils.linkForRef('object:isA','class:isA')}.<i>ClassName()</i></code> object for identifying instances of\n this Class.\n\n",
        name:"defineClass",
        seeAlso:[],
        params:[
            {description:"Name for the new class.", optional:false, name:"className", defaultValue:null,
             type:"string"},
            {description:"Optional SuperClass Class object or name", optional:true, name:"superClass",
             defaultValue:null, type:"Class"}
        ],
        returns:{description:"Returns the new Class object.", type:"Class"},
        type:"classMethod",
        definingClass:"object:ClassFactory"
    },
    "attr:DynamicForm.showErrorText":{
        ref:"attr:DynamicForm.showErrorText",
        flags:"IRW",
        description:"${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorIcons','showErrorIcons')}, \n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText','showErrorText')}, and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorStyle','showErrorStyle')} control how validation errors are\n displayed next to form items when ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true.\n These properties are boolean values, and can be set on a DynamicForm to control the \n behavior form-wide, or set on individual FormItems.\n <P>\n The HTML displayed next to a form item with errors is generated by \n ${isc.DocUtils.linkForRef('method:FormItem.getErrorHTML')}.\n The default implemenation of that method respects <code>showErrorIcons</code> and\n <code>showErrorText</code> as follows:\n <P>\n <code>showErrorIcons</code>, or <code>showErrorIcon</code> at the FormItem level controls\n whether an error icon should appear next to fields which have validation errors.  The icon's\n appearance is governed by ${isc.DocUtils.linkForRef('attr:FormItem.errorIconSrc')}, ${isc.DocUtils.linkForRef('attr:FormItem.errorIconWidth')} and\n ${isc.DocUtils.linkForRef('attr:FormItem.errorIconHeight')}\n <P>\n <code>showErrorText</code> determines whether the text of the validation error should be\n displayed next to fields which have validation errors. The attribute\n ${isc.DocUtils.linkForRef('attr:DynamicForm.showTitlesWithErrorMessages')} may be set to prefix error messages with the \n form item's title + <code>\":\"</code> (may be desired if the item has \n ${isc.DocUtils.linkForRef('attr:FormItem.showTitle')} set to false).\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')} controls where the error HTML should appear relative \n to form items. Therefore the combination of ${isc.DocUtils.linkForRef('attr:DynamicForm.showErrorText')}<code>:false</code> and\n ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')}<code>:\"left\"</code> creates a compact validation error display\n consisting of just an icon, to the left of the item with the error message\n available via a hover (similar appearance to ListGrid validation error display).  \n <P>\n In addition to this, <code>showErrorStyle</code> determines whether fields  with validation\n errors should have special styling applied to them. See ${isc.DocUtils.linkForRef('type:FormItemBaseStyle')} for a \n discussion for how error styling is calculated.",
        name:"showErrorText",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "class:RadioItem":{
        methods:[],
        ref:"class:RadioItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items/RadioGroupItem"],
        attrs:[],
        description:"Form item representing a member of a radio group, subclassed from ${isc.DocUtils.linkForRef('class:NativeCheckboxItem')}.\n RadioItems items are created and managed automatically by ${isc.DocUtils.linkForRef('class:RadioGroupItem')} instances\n and should not be instantiated directly.",
        name:"RadioItem",
        seeAlso:[],
        type:"class"
    },
    "type:EdgeSizes":{
        ref:"type:EdgeSizes",
        description:"Object used to specify custom edge sizes or offsets.\n Specified as an object where <code>defaultSize</code> will map to the default edge size or \n offset for the canvsa (${isc.DocUtils.linkForRef('attr:Canvas.edgeSize')}, or ${isc.DocUtils.linkForRef('attr:Canvas.edgeOffset')} and\n <code>top</code>, <code>left</code>, <code>right</code> and\n <code>bottom</code> will map to the\n ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeTop','edgeTop')}/${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeOffsetTop','edgeOffsetTop')}, \n ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeLeft','edgeLeft')}/${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeOffsetLeft','edgeOffsetLeft')},\n ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeRight','edgeRight')}/${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeOffsetRight','edgeOffsetRight')},\n and ${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeBottom','edgeBottom')}/${isc.DocUtils.linkForRef('attr:EdgedCanvas.edgeOffsetBottom','edgeOffsetBottom')}\n attributes on the paneContainer respectively. Note that not all these properties have to be\n set - if unset standard edge sizing rules will apply.",
        name:"EdgeSizes",
        seeAlso:[],
        type:"type"
    },
    "method:PickList.getDisplayFieldName":{
        ref:"method:PickList.getDisplayFieldName",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} for this form item.\n If unset, and ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is explicitly specified, this method\n will return the title field for the <code>optionDataSource</code>\n",
        name:"getDisplayFieldName",
        seeAlso:[],
        returns:{description:"display field name, or null", type:"String"},
        type:"method",
        definingClass:"interface:PickList"
    },
    "method:TileGrid.getTitleField":{
        ref:"method:TileGrid.getTitleField",
        flags:"",
        description:" Method to return the fieldName which represents the \"title\" for records in this\n Component.<br>\n If this.titleField is explicitly specified it will always be used.\n Otherwise, default implementation will check ${isc.DocUtils.linkForRef('attr:DataSource.titleField')} for databound\n compounds.<br>\n For non databound components returns the first defined field name of <code>\"title\"</code>, \n <code>\"name\"</code>, or <code>\"id\"</code>. If we dont find any field-names that match these\n titles, the first field in the component will be used instead.\n",
        name:"getTitleField",
        seeAlso:[],
        returns:{description:"fieldName for title field for this component.", type:"string"},
        type:"method",
        definingClass:"class:TileGrid"
    },
    "attr:Mail.replyTo":{
        ref:"attr:Mail.replyTo",
        flags:"IR",
        description:"An email address to indicate as the reply-to address on the message.  Like all other \n <code>String</code> properties of <code>Mail</code>, you can use Velocity substitution \n variables in this property.",
        name:"replyTo",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "classMethod:Page.getIsomorphicDir":{
        ref:"classMethod:Page.getIsomorphicDir",
        flags:"",
        description:"		Return the root directory for Isomorphic-specific files.\n\n",
        name:"getIsomorphicDir",
        seeAlso:[],
        returns:{description:"IsomorphicDir URL.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files"]
    },
    "classMethod:isc.sortObject":{
        ref:"classMethod:isc.sortObject",
        flags:"",
        description:" Given a simple javascript object, return that object sorted by keys, such that when iterating\n through the properties of the object, they will show up in sorted order.<br>\n Usage example - may be used to sort a ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','formItem valueMap')} defined\n as an object.\n",
        name:"sortObject",
        seeAlso:[],
        params:[
            {description:"Object to sort", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"Comparitor function to use when sorting the objects keys", optional:true,
             name:"comparitor", defaultValue:null, type:"function"}
        ],
        returns:{description:"sorted version of the object passed in.", type:"object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:ListGrid.dataProperties":{
        ref:"attr:ListGrid.dataProperties",
        flags:"IRWA",
        description:"For databound ListGrids, this attribute can be used to customize the ${isc.DocUtils.linkForRef('class:ResultSet')}\n object created for this grid when data is fetched",
        name:"dataProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "attr:DSRequest.clientContext":{
        ref:"attr:DSRequest.clientContext",
        flags:"IRW",
        description:"An object to be held onto for the duration of the DSRequest turnaround to track\n application-specific context.\n <P>\n When a DataSource request completes, the <code>clientContext</code> is available in the\n ${isc.DocUtils.linkForRef('type:DSCallback')} as <code>dsResponse.clientContext</code>.  The\n <code>clientContext</code> is never sent to the server.\n <br>\n The <code>clientContext</code> is useful for holding onto state that will be used when the\n ${isc.DocUtils.linkForRef('type:DSCallback')} fires, such as the name of a component that will receive the\n returned data.",
        name:"clientContext",
        seeAlso:[
            {ref:"attr:DSResponse.clientContext", description:null},
            {ref:"attr:RPCRequest.clientContext", description:null}
        ],
        valueType:"Object",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:DataBoundComponent.selectRecord":{
        ref:"method:DataBoundComponent.selectRecord",
        flags:"",
        description:"\n Select/deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n\n",
        name:"selectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "group:find":{
        ref:"group:find",
        refs:["method:ResultSet.findAll", "method:ResultSet.find", "method:ResultSet.findIndex", 
         "method:ResultSet.findNextIndex","method:List.findIndex", "method:List.findNextIndex", "method:List.find", 
         "method:List.findAll","method:Array.findIndex", "method:Array.findNextIndex", "method:Array.find", 
         "method:Array.containsProperty","method:Array.findAll"],
        name:"find",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.setDragTracker":{
        ref:"method:ListGrid.setDragTracker",
        flags:"",
        description:" Sets the custom tracker HTML to display next to the mouse when the user initiates a drag\n operation on this component. Default implementation will examine ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')}\n and set the custom drag tracker to display the appropriate HTML based on the selected record.\n <br>\n To display custom drag tracker HTML, this method may be overridden - call \n ${isc.DocUtils.linkForRef('classMethod:EventHandler.setDragTracker')} to actually update the drag tracker HTML.\n",
        name:"setDragTracker",
        seeAlso:[],
        returns:{description:"returns false by default to suppress 'setDragTracker' on any ancestors                   of this component.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dragTracker"]
    },
    "classMethod:Page.getWidth":{
        ref:"classMethod:Page.getWidth",
        flags:"",
        description:" Get the width of the visible portion of the window, not including browser chrome or the\n scrollbar area.\n",
        name:"getWidth",
        seeAlso:[],
        params:[
            {description:"the window object", optional:true, name:"wd", defaultValue:null, type:"object"}
        ],
        returns:{description:"width of the page", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "method:CubeGrid.setFacetValueTitle":{
        ref:"method:CubeGrid.setFacetValueTitle",
        flags:"",
        description:" Set the title for a facet value\n\n",
        name:"setFacetValueTitle",
        seeAlso:[],
        params:[
            {description:"facet to update", optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"facetValue to update", optional:false, name:"facetValueId", defaultValue:null,
             type:"id"},
            {description:"title for the facet", optional:false, name:"newTitle", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["metadata"]
    },
    "group:imageEdges":{
        ref:"group:imageEdges",
        refs:["class:EdgedCanvas", "attr:Canvas.showEdges", "attr:Canvas.edgeSize", 
         "attr:Canvas.edgeOffset","attr:Canvas.edgeImage", "attr:Canvas.customEdges", "attr:Canvas.edgeBackgroundColor", 
         "attr:Canvas.edgeOpacity","attr:Canvas.edgeShowCenter", "attr:Canvas.edgeCenterBackgroundColor", 
         "attr:EdgedCanvas.skinImgDir","attr:EdgedCanvas.edgeImage", "attr:EdgedCanvas.edgeColor", "attr:EdgedCanvas.customEdges", 
         "attr:EdgedCanvas.showCenter","attr:EdgedCanvas.edgeSize", "attr:EdgedCanvas.edgeLeft", "attr:EdgedCanvas.edgeRight", 
         "attr:EdgedCanvas.edgeTop","attr:EdgedCanvas.edgeBottom", "attr:EdgedCanvas.edgeOffset", 
         "attr:EdgedCanvas.edgeOffsetTop","attr:EdgedCanvas.edgeOffsetBottom", "attr:EdgedCanvas.edgeOffsetLeft", 
         "attr:EdgedCanvas.edgeOffsetRight","attr:EdgedCanvas.centerBackgroundColor", "attr:EdgedCanvas.edgeStyleName", 
         "attr:EdgedCanvas.addEdgeStyleSuffix"],
        name:"imageEdges",
        seeAlso:[],
        type:"group"
    },
    "attr:Splitbar.showRollOverGrip":{
        ref:"attr:Splitbar.showRollOverGrip",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether to show the\n 'Over' state on the grip image when the user rolls over on this widget. \n Has no effect if ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showRollOver')} is false",
        name:"showRollOverGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["grip"]
    },
    "attr:SummaryBuilder.field":{
        ref:"attr:SummaryBuilder.field",
        flags:"IR",
        description:"The Field object representing the field being created or edited.",
        name:"field",
        seeAlso:[],
        valueType:"Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:ListGrid.selectionType":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.selectionType",
        description:"Defines a listGrid's clickable-selection behavior.  \n <P>\n The default selection appearance is governed by ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance')}: if\n selectionAppearance is \"checkbox\", this will be \"simple\", otherwise, this will be\n \"multiple\".",
        name:"selectionType",
        setter:["setSelectionType"],
        valueType:"SelectionStyle",
        seeAlso:[
            {ref:"type:SelectionStyle", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('multipleSelect')}"],
        defaultValue:"null",
        groups:["selection", "appearance"],
        definingClass:"class:ListGrid"
    },
    "attr:ResultTree.loadDataOnDemand":{
        ref:"attr:ResultTree.loadDataOnDemand",
        flags:"IR",
        description:"Does this resultTree load data incrementally as folders within the tree are opened, or\n is it all loaded in a single request?",
        name:"loadDataOnDemand",
        seeAlso:[
            {ref:"attr:TreeGrid.loadDataOnDemand", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "attr:Applet.name":{
        ref:"attr:Applet.name",
        flags:"IR",
        description:"Sets the 'name' attribute on the applet object.  If a name is not provided it will be\n  auto-generated.  Note that in general you don't need to set this.  If you have a handle to\n  your ISC Applet object you can simply call ${isc.DocUtils.linkForRef('method:Applet.getPluginHandle')} to get a\n  handle to the element.",
        name:"name",
        seeAlso:[
            {ref:"method:Applet.getPluginHandle", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:FormItem.prompt":{
        ref:"attr:FormItem.prompt",
        flags:"IRW",
        description:"This text is shown as a tooltip prompt when the cursor hovers over this item.",
        name:"prompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["basics"]
    },
    "attr:ListGrid.dateFormatter":{
        ref:"attr:ListGrid.dateFormatter",
        flags:"IRW",
        description:"Display format to use for fields specified as type 'date'.  Default is to use the\n system-wide default short date format, configured via\n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat')}.  Specify any valid ${isc.DocUtils.linkForRef('type:DateDisplayFormat')}, or\n function to change the display format for dates used by this grid.\n If specified as  a function, this function will be executed in the scope of the Date\n and should return the formatted string.<br>\n May also be specified at the field level via\n ${isc.DocUtils.linkForRef('attr:ListGridField.displayFormat')}.<br>\n If this field is editable the dateFormatter will also be passed to the editor created\n to edit this field as ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat','displayFormat')}.\n In this case you may also need to set ${isc.DocUtils.linkForRef('attr:ListGrid.dateInputFormat')}.",
        name:"dateFormatter",
        seeAlso:[
            {ref:"attr:ListGridField.displayFormat", description:null}
        ],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "group:KeyRegistry":{
        ref:"group:KeyRegistry",
        refs:["classMethod:Page.registerKey", "classMethod:Page.unregisterKey"],
        name:"KeyRegistry",
        seeAlso:[],
        type:"group"
    },
    "attr:Flashlet.name":{
        ref:"attr:Flashlet.name",
        flags:"IR",
        description:"Sets the 'name' attribute on the flashlet object.  If a name is not provided it will be\n  auto-generated.  Note that in general you don't need to set this.  If you have a handle to\n  your ISC Flashlet object you can simply call ${isc.DocUtils.linkForRef('method:Flashlet.getPluginHandle')} to get a\n  handle to the element.",
        name:"name",
        seeAlso:[
            {ref:"method:Flashlet.getPluginHandle", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "attr:RichTextEditor.fontControls":{
        ref:"attr:RichTextEditor.fontControls",
        flags:"IRA",
        description:"Default font control group. Consists of an array of \n  ${isc.DocUtils.linkForRef('type:ControlName')}s. To display this group of controls for some RichTextEditor,\n  include <code>\"fontControls\"</code> in the ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} array",
        name:"fontControls",
        seeAlso:[],
        valueType:"array",
        defaultValue:"[\"fontSelector\", \"fontSizeSelector\"]",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "attr:SectionStackSection.canCollapse":{
        ref:"attr:SectionStackSection.canCollapse",
        flags:"I",
        description:"This attribute controls whether or not the expand/collapse UI control is shown on the\n header of this section.  Any section can still be expanded/collapsed programmatically,\n regardless of this setting.",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"],
        definingClass:"object:SectionStackSection"
    },
    "group:xmlClientVsServer":{
        ref:"group:xmlClientVsServer",
        refs:[],
        description:"Server benefits\n - faster client-side processing\n Server neutral\n - heavy customization of XML transform, if any, written in Java\n\n Client benefits\n - faster server-side processing\n Client neutral\n - heavy customization of XML transform, if any, written in JavaScript",
        name:"xmlClientVsServer",
        seeAlso:[],
        type:"group"
    },
    "type:SummaryFunction":{
        ref:"type:SummaryFunction",
        values:[
            {description:"Iterates through the set of records, picking up and summing all numeric values for the specified field. Returns null to indicate invalid summary value if\n any non numeric field values are encountered.",
             value:"sum"},
            {description:"Iterates through the set of records, picking up all numeric values for the specified field and determining the mean value. Returns null to indicate invalid\n summary value if any non numeric field values are encountered.",
             value:"avg"},
            {description:"Iterates through the set of records, picking up all values for the specified field and finding the maximum value. Handles numeric fields and\n date type fields only. Returns null to indicate invalid\n summary value if any non numeric/date field values are encountered.",
             value:"max"},
            {description:"Iterates through the set of records, picking up all values for the specified field and finding the minimum value.  Handles numeric fields and\n date type fields only. Returns null to indicate invalid summary value if\n any non numeric field values are encountered.",
             value:"min"},
            {description:"Iterates through the set of records, picking up all numeric values for the specified field and multiplying them together.\n Returns null to indicate invalid summary value if\n any non numeric field values are encountered.",
             value:"multiplier"},
            {description:"Returns a numeric count of the total number of records passed in.",
             value:"count"}
        ],
        description:"Function to produce a summary value based on an array of records and a field definition.\n An example usage is the ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary','listGrid summary row')}, where\n a row is shown at the bottom of the listGrid containing summary information about each\n column.\n <P>\n SummaryFunctions may be specified in one of 2 ways:<ul>\n <li>as an explicit function or executable\n ${isc.DocUtils.linkForRef('type:stringMethod')}, which will be passed <code>records</code> (an array of records)\n and <code>field</code> (the field definition for which the summary is required).</li>\n <li>as a standard SummaryFunction identifier</li></ul>",
        name:"SummaryFunction",
        seeAlso:[],
        type:"type"
    },
    "method:ResultSet.findByKey":{
        ref:"method:ResultSet.findByKey",
        flags:"",
        description:" Attempt to find the record in the resultSet that has a primary key value that matches the \n passed in parameter value. Only the locally cached data will be searched. \n Checks only loaded rows and will not trigger a fetch. Returns null if there is no match, \n data is not loaded, or there is no ${isc.DocUtils.linkForRef('attr:ResultSet.dataSource','dataSource')}.\n\n",
        name:"findByKey",
        seeAlso:[],
        params:[
            {description:"primary key value to search for", optional:false, name:"keyValue",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the record with a matching primary key field, or null if not found",
         type:"Record"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:Hilite.disabled":{
        ref:"attr:Hilite.disabled",
        flags:"IRW",
        description:"Whether this hilite is currently disabled.\n <P>\n Hilites can be programmatically enabled and disabled via ${isc.DocUtils.linkForRef('method:DataBoundComponent.enableHilite')}.",
        name:"disabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "attr:MultiSortPanel.descendingTitle":{
        ref:"attr:MultiSortPanel.descendingTitle",
        flags:"IR",
        description:"The title-text to appear in the \"direction\" field's SelectItem for a \"descending\" sort",
        name:"descendingTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Descending\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "group:clickMask":{
        ref:"group:clickMask",
        refs:["method:Canvas.showClickMask", "method:Canvas.hideClickMask", "method:Canvas.clickMaskUp", 
         "type:clickMaskMode"],
        name:"clickMask",
        seeAlso:[],
        type:"group"
    },
    "attr:TileLayout.orientation":{
        ref:"attr:TileLayout.orientation",
        flags:"IR",
        description:"Direction of tiling.  See also ${isc.DocUtils.linkForRef('type:TileLayoutPolicy')}.",
        name:"orientation",
        seeAlso:[],
        valueType:"Orientation",
        defaultValue:"\"horizontal\"",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutPolicy"]
    },
    "method:PickList.getSelectedRecord":{
        ref:"method:PickList.getSelectedRecord",
        flags:"",
        description:" Returns the entire record object associated with the current value for this item\n (or null if no matching record exists in the PickList data).\n <P>\n Most commonly used for databound pickListItems to retrieve the values of other fields\n in the record.\n\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"record object", type:"object"},
        type:"method",
        definingClass:"interface:PickList"
    },
    "method:DateItem.selectValue":{
        ref:"method:DateItem.selectValue",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is true, falls through to standard\n ${isc.DocUtils.linkForRef('method:FormItem.selectValue')} implementation on this items freeform text entry field.\n Otherwise has no effect.\n",
        name:"selectValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:DateItem"
    },
    "classMethod:isA.emptyArray":{
        ref:"classMethod:isA.emptyArray",
        flags:"",
        description:"\n Is <code>object</code> an Array with no items?\n\n",
        name:"emptyArray",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is an empty array", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:DateItem.dateFormatter":{
        ref:"attr:DateItem.dateFormatter",
        flags:"IA",
        description:"If <code>dateFormatter</code> is set at init time, it will be used instead of\n ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} to govern how dates are displayed in this item.",
        name:"dateFormatter",
        seeAlso:[],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "classMethod:isA.List":{
        ref:"classMethod:isA.List",
        flags:"",
        description:" Does <code>object</code> implement the  <code>List</code> interface?\n",
        name:"List",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"<code>true</code> if the object is an Array or belongs to another class that                   implements the <code>List</code> API.",
         type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:ListGridField.getGridSummary":{
        ref:"method:ListGridField.getGridSummary",
        flags:"A",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, and this method is specified it will be\n called to generate the summary value to be displayed in the grid summary row. Note that\n this is called instead of making use of the ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')}.\n <P>\n If this grid is grouped, and ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, this method\n will be passed a third parameter - an array of group-level summaries.\n",
        name:"getGridSummary",
        seeAlso:[],
        params:[
            {description:"records for which a summary is being generated", optional:false,
             name:"records", defaultValue:null, type:"Array of ListGridRecord"},
            {description:"pointer to the field for which summary value is being generated",
             optional:false, name:"field", defaultValue:null, type:"ListGridField"},
            {description:"If this grid is grouped and  ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is specified, this parameter contains an array of already-\n  calculated summary values for each group in the grid. Each element in this array will\n  be an object containing calculated summary values for each field in the grid, as well as\n  a specified groupValue and groupName, allowing the developer to determine which group this\n  summary value comes from",
             optional:true, name:"groupSummaries", defaultValue:null, type:"Array of objects"}
        ],
        returns:{description:"summary value to display", type:"any"},
        type:"method",
        definingClass:"object:ListGridField"
    },
    "method:DynamicForm.clearErrors":{
        ref:"method:DynamicForm.clearErrors",
        flags:"",
        description:"	Clears all errors for this DynamicForm.\n",
        name:"clearErrors",
        seeAlso:[],
        params:[
            {description:"If true, redraw the form to clear any visible error messages.",
             optional:false, name:"show", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "classMethod:Array.isLoading":{
        ref:"classMethod:Array.isLoading",
        flags:"A",
        description:" Is the object passed in a loading marker value? For use with Lists that manage remote\n datasets, to indicate that a record has not yet been retrieved from the server. A typical\n use case might be to check if a row has been loaded in a ListGrid - for example:\n <P>\n <code>\n if (Array.isLoading(myList.getRecord(0))) isc.warn(\"Please wait for the data to load.\");\n </code>\n",
        name:"isLoading",
        seeAlso:[],
        params:[
            {description:"data to test.", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        type:"classMethod",
        definingClass:"object:Array"
    },
    "group:fill":{
        ref:"group:fill",
        refs:[],
        name:"fill",
        seeAlso:[],
        type:"group"
    },
    "attr:DateTimeItem.useTextField":{
        ref:"attr:DateTimeItem.useTextField",
        flags:"IRW",
        description:"DateTimeItems show datetime values in a freeform text entry area.",
        name:"useTextField",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DateTimeItem",
        groups:["basics"]
    },
    "method:CellSelection.getSelectedCells":{
        ref:"method:CellSelection.getSelectedCells",
        flags:"",
        description:"  Returns an array of the currently selected cells.  Each cell is returned as a 2 element\n  array in the form <code>[rowNum, colNum]</code>.\n",
        name:"getSelectedCells",
        seeAlso:[],
        returns:{description:"an array of the selected cells, as 2 element arrays", type:"array"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection", "selection"]
    },
    "attr:Canvas.edgeSize":{
        ref:"attr:Canvas.edgeSize",
        flags:"IR",
        description:"Size in pixels for corners and edges",
        name:"edgeSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "attr:Progressbar.length":{
        flags:"IR",
        getter:["getLength"],
        type:"attr",
        ref:"attr:Progressbar.length",
        description:"Length of the progressbar in pixels. This is effectively height for a vertical\n progressbar, or width for a horizontal progressbar.\n <P>\n This property must be set instead of setting <code>width</code> or <code>height</code>.",
        name:"length",
        setter:["setLength"],
        valueType:"number",
        seeAlso:[],
        defaultValue:"100",
        groups:["appearance"],
        definingClass:"class:Progressbar"
    },
    "attr:PickList.animationTime":{
        ref:"attr:PickList.animationTime",
        flags:"IRWA",
        description:"If this.animatePickList is true - this specifies the duration of the animation effect\n applied when showing the pickList",
        name:"animationTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"200",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "type:DSDataFormat":{
        ref:"type:DSDataFormat",
        values:[
            {description:"Make an HTTP request in a format recognized by the ISC server and expect ISC server\n response format.  ${isc.DocUtils.linkForRef('group:serverDataIntegration','Server-side integration')} can then be\n used to connect to any kind of data store or web service.",
             value:"\"iscServer\""},
            {description:"Expect XML responses.  Request format depends on the setting for\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','protocol')}.  This is the correct setting when\n consuming RSS feeds, XML web services (whether SOAP, REST, XML-RPC or custom format),\n and XML flat files directly from the browser.\n <P>\n Values for \"date\", \"time\" or \"datetime\" fields in responses should be specified in the\n applicable <a target=_blank href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema\n date format</a>.\n If no timezone is explicitly specified, dates / datetimes received by the client are \n assumed to be GMT. Note that \"date\" type fields represent logical dates and may omit\n time information entirely, and \"time\" type fields may omit date information.\n See ${isc.DocUtils.linkForRef('group:dateFormatAndStorage','Date and Time Format and storage')} for more information\n on how date values are serialized in requests sent to the server.\n  \n <P>\n A DSResponse will be derived from the returned XML via the process described under\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')}.",
             value:"\"xml\""},
            {description:"Expect response in JSON <a href='http://json.org' onclick=\"window.open('http://json.org');return false;\">(Java Script Object Notation)</a>\n format, ready to be eval()'d. Response should either be a naked object literal:<br>\n <code>&nbsp;&nbsp;&nbsp;&nbsp;{ \"property\":\"value1\", \"property2\" : \"value2\", ... }</code><br>\n or a string that evals to return a valid response object:<br>\n <code>&nbsp;&nbsp;&nbsp;&nbsp;var response = { \"property\":\"value1\", \"property2\" : \"value2\", ... }</code>\n <br>\n <code>&nbsp;&nbsp;&nbsp;&nbsp;response;</code><br>\n <P>\n A DSResponse will be derived from the returned JSON via the process described under\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')}.\n <P>\n As with <code>\"xml\"</code> responses, values for \"date\" or \"datetime\" fields should be\n specified as a String in \n <a target=_blank href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema date format</a>\n and may include a timezone.  In the absence of a timezone they will be assumed to be GMT.\n <P>\n Request format depends on the setting for ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','protocol')}.  \n See also ${isc.DocUtils.linkForRef('class:XJSONDataSource')}.",
             value:"\"json\""},
            {description:"SmartClient will not attempt to parse the response, instead,\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} must be implemented.\n <code>transformResponse</code> will receive the \"data\" parameter as a String, and must\n parse this String into an Array of Objects, which should be set as\n ${isc.DocUtils.linkForRef('attr:DSResponse.data')}.  Request format depends on the setting for\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','protocol')}.\n <P>\n Note that, unlike either the \"json\" or \"xml\" settings of <code>dataFormat</code>, you\n are responsible for ensuring that parsed values are the correct type, for example, using\n the JavaScript built-ins <code>parseInt</code> and <code>parseFloat</code> on integer\n and decimal values respectively, and using <code>new Date()</code> to construct valid\n Dates.",
             value:"\"custom\""}
        ],
        description:"Indicates the format to be used for HTTP requests and responses when\n fulfilling DSRequests (eg, when ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} is called).\n <P>\n Note that ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','request queuing')} is only available for\n \"iscServer\" requests.",
        name:"DSDataFormat",
        seeAlso:[],
        type:"type",
        serverDS:"allowed",
        examples:["${isc.DocUtils.linkForExampleId('simpleJSON')}"],
        groups:["clientDataIntegration", "serverDataIntegration"]
    },
    "classMethod:MathFunction.registerFunction":{
        ref:"classMethod:MathFunction.registerFunction",
        flags:"",
        description:" Registers a new math function for use with FormulaFields.\n",
        name:"registerFunction",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"newFunction", defaultValue:null,
             type:"MathFunction"}
        ],
        type:"classMethod",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "attr:Chart.threeD":{
        ref:"attr:Chart.threeD",
        flags:"IR",
        description:"Whether to show chart in a 3D appearance, for charts that support this.",
        name:"threeD",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"interface:Chart",
        groups:["chartAppearance"]
    },
    "classMethod:FileLoader.cacheISC":{
        ref:"classMethod:FileLoader.cacheISC",
        flags:"",
        description:"\n Caches the specified SmartClient modules and skin.  Calling this method is equivalent to using\n the <code>loadISC</code> JSP tag with <code>cacheOnly=\"true\"</code>.\n\n",
        name:"cacheISC",
        seeAlso:[],
        params:[
            {description:"Name of the skin to load.  If not specified, the skin specified by the                        default ${isc.DocUtils.linkForRef('classAttr:FileLoader.defaultSkin')} will be used.\n                        Instead of a skin name, you can specify a skinDir - this works just\n                        like ${isc.DocUtils.linkForRef('classMethod:Page.setSkinDir')} - the skin assumed to be a skinDir if\n                        there's at least one forward slash (/) in the name.",
             optional:true, name:"skin", defaultValue:null, type:"String"},
            {description:"List of modules to load.  If not specified, the list of modules                                   specified by the default\n                                   ${isc.DocUtils.linkForRef('classAttr:FileLoader.defaultModules')} will be used.  You\n                                   can specify modules as \"Core,Foundation\" or as [\"Core\",\n                                   \"Foundation\"]",
             optional:true, name:"modules", defaultValue:null, type:"String or List"},
            {description:"Optional code to execute when all specified modules and                                      skin have been cached.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "attr:SimpleTile.creator":{
        ref:"attr:SimpleTile.creator",
        flags:"IR",
        description:"The ${isc.DocUtils.linkForRef('class:TileGrid')} that created this SimpleTile.",
        name:"creator",
        seeAlso:[],
        valueType:"TileGrid",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleTile"
    },
    "attr:ListGrid.rowEndEditAction":{
        ref:"attr:ListGrid.rowEndEditAction",
        flags:"IRW",
        description:"If the user is editing a record in this listGrid, and attempts to navigate to a field\n beyond the end of the row, via tab (or shift-tab off the first editable field), this \n property determines what action to take:<ul>\n <li>\"next\": start editing the next (or previous) record in the list\n <li>\"same\": put focus back into the first editable field of the same record.\n <li>\"done\": hide the editor\n <li>\"stop\": leave focus in the cell being edited\n <li>\"none\": no action\n </ul>",
        name:"rowEndEditAction",
        seeAlso:[],
        valueType:"RowEndEditAction",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "class:CellSelection":{
        methods:["method:CellSelection.cellIsSelected", "method:CellSelection.anySelected", 
         "method:CellSelection.getSelectedCells","method:CellSelection.selectCell", "method:CellSelection.deselectCell", 
         "method:CellSelection.selectCellList","method:CellSelection.deselectCellList", "method:CellSelection.selectSingleCell"],
        ref:"class:CellSelection",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System/Selection"],
        attrs:[],
        description:"Maintains a representation of selection over a 2-dimensional grid of objects.<br>\n  Automtatically created to manage cell-selection on ${isc.DocUtils.linkForRef('class:CubeGrid')} widgets.",
        name:"CellSelection",
        seeAlso:[],
        type:"class"
    },
    "method:Class.delayCall":{
        ref:"method:Class.delayCall",
        flags:"",
        description:"  This is a helper to delay a call to some method on this object by some specified\n  amount of time.\n",
        name:"delayCall",
        seeAlso:[],
        params:[
            {description:"name of the method to call", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"array of arguments to pass to the method in question", optional:true,
             name:"arrayArgs", defaultValue:null, type:"array"},
            {description:"Number of ms to delay the call by - defaults to zero (so just pulls                        execution of the method out of the current execution thread.",
             optional:true, name:"time", defaultValue:null, type:"number"}
        ],
        returns:{description:"Timer ID for the delayed call - can be passed to                       ${isc.DocUtils.linkForRef('classMethod:Timer.clear')} to cancel the call before it executes",
         type:"string"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:SectionHeader.controls":{
        ref:"attr:SectionHeader.controls",
        flags:"IR",
        description:"Custom controls to be shown on top of this section header.\n <P>\n These controls are shown in the ${isc.DocUtils.linkForRef('attr:SectionHeader.controlsLayout')}.",
        name:"controls",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sectionControls')}"],
        definingClass:"class:SectionHeader"
    },
    "method:Class.addProperties":{
        ref:"method:Class.addProperties",
        flags:"",
        description:"	\n 	Add properties or methods to this specific instance.  \n	Properties with the same name as existing properties will override.\n\n",
        name:"addProperties",
        seeAlso:[],
        params:[
            {description:"Object containing name:value pairs to be added to this object",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"object"}
        ],
        returns:{description:"the object after properties have been added to it", type:"object"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:StretchImgButton.iconWidth":{
        ref:"attr:StretchImgButton.iconWidth",
        flags:"IR",
        description:"Width in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "group:checkboxField":{
        ref:"group:checkboxField",
        refs:["method:ListGrid.getCheckboxField", "method:ListGrid.isCheckboxField", 
         "attr:ListGrid.checkboxField","attr:ListGrid.checkboxFieldTrueImage", "attr:ListGrid.checkboxFieldFalseImage", 
         "attr:ListGrid.checkboxFieldImageWidth","attr:ListGrid.checkboxFieldImageHeight"],
        name:"checkboxField",
        seeAlso:[],
        type:"group"
    },
    "attr:OperationBinding.methodArguments":{
        ref:"attr:OperationBinding.methodArguments",
        flags:"IR",
        description:"You can explicitly declare the arguments to be passed to\n ${isc.DocUtils.linkForRef('attr:OperationBinding.serverMethod')} using this attribute.  This isn't required - in the\n absense of <code>methodArguments</code>, the DMI implementation will still automatically\n pass a stock set of arguments to your method (see the overview in ${isc.DocUtils.linkForRef('class:ServerObject')}), but\n specifying arguments explicitly gives you more flexibility in what can be passed.\n <p>\n The format for specifying <code>methodArguments</code> is as a comma separated list of VTL\n (Velocity Template Language) expressions.  See the\n <a href='http://jakarta.apache.org/velocity/docs/vtl-reference-guide.html' onclick=\"window.open('http://jakarta.apache.org/velocity/docs/vtl-reference-guide.html');return false;\">VTL Reference</a>\n and <a href='http://jakarta.apache.org/velocity/docs/user-guide.html' onclick=\"window.open('http://jakarta.apache.org/velocity/docs/user-guide.html');return false;\">Velocity User Guide</a> \n for an overview of how to use VTL.\n <p>\n The Velocity context is pre-populated with the following variables - you can pass these\n verbatim as arguments, or call methods on these objects and pass the resulting values:\n <ul>\n <li>dsRequest: instance of the current DSRequest\n <li>request: the current HttpServletRequest\n <li>response: the current HttpServletResponse\n <li>rpcManager: the instance of RPCManager for this request\n <li>dataSource: a DataSource instance for this request\n </ul>\n So, for example, if you had a method signature like the following:\n <p><code>\n public DSResponse fetch(SupplyItem criteria, long startRow, long endRow)\n </code><p>\n You can invoke it by specifying <code>methodArguments</code> as follows:\n <p><code>\n methodArguments=\"$dsRequest.criteria, $dsRequest.startRow, $dsRequest.endRow\"\n </code></p>\n Without <code>methodArguments</code>, there would be no way for you to specify\n <code>startRow/endRow</code> as arguments.  You could, of course, simply declare the method\n to take a <code>DSRequest</code> object and call <code>getStartRow()/getEndRow()</code> in\n the body of the method.",
        name:"methodArguments",
        seeAlso:[
            {ref:"class:ServerObject", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding"
    },
    "classMethod:Class.logIsErrorEnabled":{
        ref:"classMethod:Class.logIsErrorEnabled",
        flags:"",
        description:" Check whether a message logged at \"error\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsErrorEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "classAttr:ValidatorTypes.substringCount":{
        ref:"classAttr:ValidatorTypes.substringCount",
        flags:"IR",
        description:"Determine whether a string value contains some substring multiple times.\n  The substring to check for is specified via <code>validator.substring</code>.\n  The <code>validator.operator</code> property allows you to specify how to test\n  the number of substring occurrances. Valid values for this property are\n  <code>==</code>, <code>!=</code>, <code>&lt;</code>, <code>&lt;=</code>,\n  <code>&gt;</code>, <code>&gt;=</code>.<br>\n  The number of matches to check for is specified via <code>validator.count</code>.",
        name:"substringCount",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "method:Date.getShortYear":{
        ref:"method:Date.getShortYear",
        flags:"",
        description:"      Return a 2 digit year for this date.\n",
        name:"getShortYear",
        seeAlso:[],
        returns:{description:"year number, padded to 2 characters", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "class:Canvas":{
        methods:["method:Canvas.init", "method:Canvas.initWidget", "method:Canvas.isDrawn", 
         "method:Canvas.getID","method:Canvas.getInnerHTML", "method:Canvas.draw", "method:Canvas.setHtmlElement", 
         "method:Canvas.setHtmlPosition","method:Canvas.isDirty", "method:Canvas.markForRedraw", "method:Canvas.redraw", 
         "method:Canvas.clear","method:Canvas.destroy", "method:Canvas.markForDestroy", "method:Canvas.addChild", 
         "method:Canvas.removePeer","method:Canvas.removeChild", "method:Canvas.addPeer", "method:Canvas.setSnapTo", 
         "method:Canvas.getSnapTo","method:Canvas.setSnapEdge", "method:Canvas.getSnapEdge", 
         "method:Canvas.getParentElements","method:Canvas.contains", "method:Canvas.showClickMask", "method:Canvas.hideClickMask", 
         "method:Canvas.clickMaskUp","method:Canvas.setRect", "method:Canvas.getLeft", "method:Canvas.setLeft", 
         "method:Canvas.getTop","method:Canvas.setTop", "method:Canvas.getWidth", "method:Canvas.setWidth", 
         "method:Canvas.getHeight","method:Canvas.setHeight", "method:Canvas.getRight", "method:Canvas.setRight", 
         "method:Canvas.getBottom","method:Canvas.setBottom", "method:Canvas.getScrollWidth", "method:Canvas.getScrollHeight", 
         "method:Canvas.getScrollLeft","method:Canvas.getScrollTop", "method:Canvas.setPageLeft", "method:Canvas.setPageTop", 
         "method:Canvas.getPageLeft","method:Canvas.getPageTop", "method:Canvas.getPageRight", "method:Canvas.getPageBottom", 
         "method:Canvas.setMargin","method:Canvas.setPadding", "method:Canvas.containsPoint", "method:Canvas.visibleAtPoint", 
         "method:Canvas.intersects","method:Canvas.intersectsRect", "method:Canvas.containsEvent", "method:Canvas.getOffsetX", 
         "method:Canvas.getOffsetY","method:Canvas.getScrollbarSize", "method:Canvas.getViewportWidth", 
         "method:Canvas.getViewportHeight","method:Canvas.getInnerHeight", "method:Canvas.getInnerWidth", 
         "method:Canvas.getInnerContentHeight","method:Canvas.getInnerContentWidth", "method:Canvas.getVisibleWidth", 
         "method:Canvas.getVisibleHeight","method:Canvas.moveBy", "method:Canvas.moveTo", "method:Canvas.getVSnapOrigin", 
         "method:Canvas.getHSnapOrigin","method:Canvas.showNextTo", "method:Canvas.resizeBy", "method:Canvas.resized", 
         "method:Canvas.setPercentSource","method:Canvas.layoutChildren", "method:Canvas.resizeTo", "method:Canvas.parentResized", 
         "method:Canvas.setDragTracker","method:Canvas.shouldDragScroll", "method:Canvas.setOverflow", 
         "method:Canvas.adjustForContent","method:Canvas.getScrollBottom", "method:Canvas.getScrollRight", 
         "method:Canvas.scrollToTop","method:Canvas.scrollToBottom", "method:Canvas.scrollToLeft", 
         "method:Canvas.scrollToRight","method:Canvas.scrollBy", "method:Canvas.scrollByPercent", "method:Canvas.scrollTo", 
         "method:Canvas.scrolled","method:Canvas.scrollToPercent", "method:Canvas.show", "method:Canvas.hide", 
         "method:Canvas.isVisible","method:Canvas.setEnabled", "method:Canvas.setDisabled", "method:Canvas.enable", 
         "method:Canvas.disable","method:Canvas.isDisabled", "method:Canvas.isEnabled", "method:Canvas.setCanFocus", 
         "method:Canvas.focus","method:Canvas.blur", "method:Canvas.containsFocus", "method:Canvas.setAccessKey", 
         "method:Canvas.setTabIndex","method:Canvas.bringToFront", "method:Canvas.sendToBack", "method:Canvas.moveAbove", 
         "method:Canvas.moveBelow","method:Canvas.getContents", "method:Canvas.setContents", 
         "method:Canvas.setBackgroundColor","method:Canvas.setBackgroundImage", "method:Canvas.setBorder", "method:Canvas.setOpacity", 
         "method:Canvas.setCursor","method:Canvas.handleHover", "method:Canvas.updateHover", "method:Canvas.hoverHidden", 
         "method:Canvas.hover","method:Canvas.getHoverHTML", "method:Canvas.setClassName", "method:Canvas.setStyleName", 
         "method:Canvas.showContextMenu","method:Canvas.hideContextMenu", "method:Canvas.mouseOver", "method:Canvas.mouseDown", 
         "method:Canvas.rightMouseDown","method:Canvas.mouseStillDown", "method:Canvas.mouseMove", "method:Canvas.mouseOut", 
         "method:Canvas.mouseUp","method:Canvas.click", "method:Canvas.doubleClick", "method:Canvas.mouseWheel", 
         "method:Canvas.dragRepositionStart","method:Canvas.dragRepositionMove", "method:Canvas.dragRepositionStop", 
         "method:Canvas.dragResizeStart","method:Canvas.dragResizeMove", "method:Canvas.dragResizeStop", "method:Canvas.dragStart", 
         "method:Canvas.dragMove","method:Canvas.dragStop", "method:Canvas.dropOver", "method:Canvas.dropMove", 
         "method:Canvas.dropOut","method:Canvas.drop", "method:Canvas.keyDown", "method:Canvas.keyUp", 
         "method:Canvas.keyPress","method:Canvas.willAcceptDrop", "method:Canvas.getHSnapPosition", 
         "method:Canvas.getVSnapPosition","method:Canvas.shouldSnapOnDrop", "method:Canvas.getImgURL", "method:Canvas.imgHTML", 
         "method:Canvas.getImage","method:Canvas.setImage", "method:Canvas.linkHTML", "method:Canvas.setEdgeOpacity", 
         "method:Canvas.setShowShadow","method:Canvas.focusChanged", "method:Canvas.setDataPath", "method:Canvas.getFullDataPath", 
         "method:Canvas.animateMove","method:Canvas.animateResize", "method:Canvas.animateRect", "method:Canvas.animateFade", 
         "method:Canvas.animateScroll","method:Canvas.animateShow", "method:Canvas.animateHide", "method:Canvas.setShowResizeBar"],
        exampleConfig:"Canvas.xml",
        ref:"class:Canvas",
        classAttrs:["classAttr:Canvas.TAB_INDEX_FLOOR"],
        classMethods:["classMethod:Canvas.getEventEdge", "classMethod:Canvas.getById", 
         "classMethod:Canvas.hiliteCharacter"],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:Canvas.ID", "attr:Canvas.autoDraw", "attr:Canvas.parentElement", 
         "attr:Canvas.topElement","attr:Canvas.masterElement", "attr:Canvas.children", "attr:Canvas.peers", 
         "attr:Canvas.htmlElement","attr:Canvas.htmlPosition", "attr:Canvas.matchElement", "attr:Canvas.position", 
         "attr:Canvas.left","attr:Canvas.top", "attr:Canvas.width", "attr:Canvas.height", "attr:Canvas.defaultWidth", 
         "attr:Canvas.defaultHeight","attr:Canvas.minWidth", "attr:Canvas.maxWidth", "attr:Canvas.minHeight", 
         "attr:Canvas.maxHeight","attr:Canvas.autoShowParent", "attr:Canvas.visibility", "attr:Canvas.canSelectText", 
         "attr:Canvas.className","attr:Canvas.styleName", "attr:Canvas.contents", "attr:Canvas.dynamicContents", 
         "attr:Canvas.dynamicContentsVars","attr:Canvas.margin", "attr:Canvas.padding", "attr:Canvas.border", 
         "attr:Canvas.backgroundColor","attr:Canvas.backgroundImage", "attr:Canvas.backgroundRepeat", 
         "attr:Canvas.backgroundPosition","attr:Canvas.appImgDir", "attr:Canvas.skinImgDir", "attr:Canvas.cursor", 
         "attr:Canvas.disabledCursor","attr:Canvas.opacity", "attr:Canvas.overflow", "attr:Canvas.showCustomScrollbars", 
         "attr:Canvas.scrollbarSize","attr:Canvas.disabled", "attr:Canvas.enabled", "attr:Canvas.redrawOnResize", 
         "attr:Canvas.canFocus","attr:Canvas.tabIndex", "attr:Canvas.accessKey", "attr:Canvas.contextMenu", 
         "attr:Canvas.prompt","attr:Canvas.canDrag", "attr:Canvas.dragStartDistance", "attr:Canvas.canDragScroll", 
         "attr:Canvas.dragScrollDelay","attr:Canvas.dragIntersectStyle", "attr:Canvas.canDragReposition", 
         "attr:Canvas.dragRepositionCursor","attr:Canvas.canDragResize", "attr:Canvas.resizeFrom", "attr:Canvas.canHover", 
         "attr:Canvas.hoverDelay","attr:Canvas.showHover", "attr:Canvas.hoverWidth", "attr:Canvas.hoverHeight", 
         "attr:Canvas.hoverAlign","attr:Canvas.hoverVAlign", "attr:Canvas.hoverWrap", "attr:Canvas.hoverStyle", 
         "attr:Canvas.hoverOpacity","attr:Canvas.hoverMoveWithMouse", "attr:Canvas.edgeCursorMap", 
         "attr:Canvas.keepInParentRect","attr:Canvas.dragAppearance", "attr:Canvas.dragType", "attr:Canvas.dragTarget", 
         "attr:Canvas.showDragShadow","attr:Canvas.dragOpacity", "attr:Canvas.canDrop", "attr:Canvas.canAcceptDrop", 
         "attr:Canvas.canDropBefore","attr:Canvas.dropTypes", "attr:Canvas.mouseStillDownInitialDelay", 
         "attr:Canvas.mouseStillDownDelay","attr:Canvas.doubleClickDelay", "attr:Canvas.noDoubleClicks", "attr:Canvas.percentSource", 
         "attr:Canvas.percentBox","attr:Canvas.snapTo", "attr:Canvas.snapEdge", "attr:Canvas.snapOffsetLeft", 
         "attr:Canvas.snapOffsetTop","attr:Canvas.snapToGrid", "attr:Canvas.snapResizeToGrid", "attr:Canvas.childrenSnapToGrid", 
         "attr:Canvas.childrenSnapResizeToGrid","attr:Canvas.snapHGap", "attr:Canvas.snapVGap", "attr:Canvas.snapHDirection", 
         "attr:Canvas.snapVDirection","attr:Canvas.snapAxis", "attr:Canvas.snapOnDrop", "attr:Canvas.destroyed", 
         "attr:Canvas.destroying","attr:Canvas.showEdges", "attr:Canvas.edgeSize", "attr:Canvas.edgeOffset", 
         "attr:Canvas.edgeImage","attr:Canvas.customEdges", "attr:Canvas.edgeBackgroundColor", "attr:Canvas.edgeOpacity", 
         "attr:Canvas.edgeShowCenter","attr:Canvas.edgeCenterBackgroundColor", "attr:Canvas.showShadow", 
         "attr:Canvas.shadowDepth","attr:Canvas.shadowOffset", "attr:Canvas.shadowSoftness", "attr:Canvas.shadowImage", 
         "attr:Canvas.valuesManager","attr:Canvas.dataPath", "attr:Canvas.animateTime", "attr:Canvas.animateAcceleration", 
         "attr:Canvas.animateShowEffect","attr:Canvas.animateHideEffect", "attr:Canvas.animateMoveTime", 
         "attr:Canvas.animateResizeTime","attr:Canvas.animateRectTime", "attr:Canvas.animateFadeTime", 
         "attr:Canvas.animateScrollTime","attr:Canvas.animateShowTime", "attr:Canvas.animateHideTime", 
         "attr:Canvas.animateMoveAcceleration","attr:Canvas.animateResizeAcceleration", "attr:Canvas.animateRectAcceleration", 
         "attr:Canvas.animateScrollAcceleration","attr:Canvas.animateShowAcceleration", "attr:Canvas.animateHideAcceleration", 
         "attr:Canvas.layoutAlign","attr:Canvas.showResizeBar", "attr:Canvas.resizeBarTarget", "attr:Canvas.extraSpace"],
        description:"Canvas is the base abstraction for cross-browser DHTML drawing.\n		All DHTML widgets inherit from the Canvas class.",
        name:"Canvas",
        seeAlso:[],
        type:"class"
    },
    "method:Array.unsort":{
        ref:"method:Array.unsort",
        flags:"A",
        description:"		Turn sorting off for this array, indicating that the current sort\n		order should be preserved.  Return true if this is supported in thist List.\n\n		Some implementations may not support this -- they should return false\n		to indicate to the caller that sort order must be maintained (eg: in\n		the case where sort order is dervied from the server, etc).\n\n",
        name:"unsort",
        seeAlso:[],
        returns:{description:"true == list supports unsorting, false == not supported.", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["sorting"]
    },
    "attr:PickList.pickListCriteria":{
        ref:"attr:PickList.pickListCriteria",
        flags:"IRWA",
        description:"If this item has a databound pick-list (for example ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} is\n set), this property can be used to provide static filter criteria when retrieving the data\n for the pickList.",
        name:"pickListCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "method:CellSelection.cellIsSelected":{
        ref:"method:CellSelection.cellIsSelected",
        flags:"",
        description:"			Return true if a particular item is selected\n",
        name:"cellIsSelected",
        seeAlso:[],
        params:[
            {description:"row index of the cell to check", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of the cell to check", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == object is selected								false == object is not selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection", "selection"]
    },
    "attr:DataSourceField.exportTitle":{
        ref:"attr:DataSourceField.exportTitle",
        flags:"IR",
        description:"Optional different field-title used for exports.",
        name:"exportTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "attr:StretchImgButton.selected":{
        ref:"attr:StretchImgButton.selected",
        flags:"IRW",
        description:"Whether this component is selected.  For some components, selection affects appearance.",
        name:"selected",
        setter:["setSelected"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "method:DynamicForm.hideItem":{
        ref:"method:DynamicForm.hideItem",
        flags:"",
        description:" Hide a form item via ${isc.DocUtils.linkForRef('method:FormItem.hide')}\n",
        name:"hideItem",
        seeAlso:[],
        params:[
            {description:"Name of the item to show", optional:false, name:"itemName", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "method:CubeGrid.getEditValue":{
        ref:"method:CubeGrid.getEditValue",
        flags:"",
        description:"\n Returns the current temporary locally stored edit value for a cell being edited.\n Note this is the ${isc.DocUtils.linkForRef('attr:CubeGrid.valueProperty','valueProperty')} that will be saved for the\n cell in question.\n\n",
        name:"getEditValue",
        seeAlso:[],
        params:[
            {description:"index of the row for which the editValue should be returned",
             optional:false, name:"rowNum", defaultValue:null, type:"number"},
            {description:"index of column for which value should be returned", optional:false,
             name:"colNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"edit value for the cell", type:"any"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "attr:TileLayout.paddingAsLayoutMargin":{
        ref:"attr:TileLayout.paddingAsLayoutMargin",
        flags:"IRWA",
        description:"If this widget has padding specified (as ${isc.DocUtils.linkForRef('attr:Canvas.padding','this.padding')} or in the\n CSS style applied to this layout), should it show up as space outside the members,\n similar to layoutMargin?\n <P>\n If this setting is false, padding will not affect member positioning (as CSS padding\n normally does not affect absolutely positioned children).  Leaving this setting true\n allows a designer to more effectively control layout purely from CSS.\n <P>\n Note that ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')} if specified, takes precidence over this value.",
        name:"paddingAsLayoutMargin",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "attr:DataBoundComponent.canAddFormulaFields":{
        ref:"attr:DataBoundComponent.canAddFormulaFields",
        flags:"IRW",
        description:"Adds an item to the header context menu allowing users to launch a dialog to define a new\n field based on values present in other fields, using the ${isc.DocUtils.linkForRef('class:FormulaBuilder')}.\n <P>\n User-added formula fields can be persisted via ${isc.DocUtils.linkForRef('method:ListGrid.getFieldState')} and \n ${isc.DocUtils.linkForRef('method:ListGrid.setFieldState')}.",
        name:"canAddFormulaFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["formulaFields"]
    },
    "attr:Window.maximized":{
        ref:"attr:Window.maximized",
        flags:"IRW",
        description:"Is this window maximized. If true at init time, the window will be drawn maximized.\n To set this property at runtime use ${isc.DocUtils.linkForRef('method:Window.maximize')} or ${isc.DocUtils.linkForRef('method:Window.restore')}.",
        name:"maximized",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "method:ListGrid.getEditRow":{
        ref:"method:ListGrid.getEditRow",
        flags:"",
        description:"\n  Returns the index of the row being edited or null if there is no current edit row.\n\n",
        name:"getEditRow",
        seeAlso:[],
        returns:{description:"Index of the current edit row", type:"number"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Window.bodyStyle":{
        ref:"attr:Window.bodyStyle",
        flags:"IRW",
        description:"Style of the Window body.",
        name:"bodyStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"windowBody\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "method:Canvas.setEnabled":{
        ref:"method:Canvas.setEnabled",
        deprecated:"As of Smartclient version 5.5, deprecated in favor of ${isc.DocUtils.linkForRef('method:Canvas.setDisabled')}",
        flags:"A",
        description:" set the enabled state of this object.\n\n",
        name:"setEnabled",
        seeAlso:[],
        params:[
            {description:"pass false to disable or anything else to enable", optional:false,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["enable"]
    },
    "attr:TreeGrid.autoFetchTextMatchStyle":{
        ref:"attr:TreeGrid.autoFetchTextMatchStyle",
        flags:"IR",
        description:"With ${isc.DocUtils.linkForRef('loadDataOnDemand:true')}, TreeGrids fetch data by selecting the child nodes of\n each parent, which should be exact match, so we default to\n <code>autoFetchTextMatchStyle:\"exact\"</code>.\n See ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchTextMatchStyle')} for details.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"\"exact\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["dataBinding"]
    },
    "method:ValuesManager.addMembers":{
        ref:"method:ValuesManager.addMembers",
        flags:"",
        description:"  Add multiple new member forms to this valuesManager.\n",
        name:"addMembers",
        seeAlso:[
            {ref:"method:ValuesManager.addMember", description:null}
        ],
        params:[
            {description:"array of forms to add to this valuesManager as members.", optional:false,
             name:"members", defaultValue:null, type:"array"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "attr:MultiSortPanel.copyLevelButtonTitle":{
        ref:"attr:MultiSortPanel.copyLevelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the copyLevelButton",
        name:"copyLevelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Copy Level\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "classAttr:Hover.moveWithMouse":{
        ref:"classAttr:Hover.moveWithMouse",
        flags:"RWA",
        description:"When the Hover canvas is shown by default, should it move as the user moves the\n mouse pointer?<br>\n May be overridden by including a <code>moveWithMouse</code> attribute on the \n properties block passed to ${isc.DocUtils.linkForRef('classMethod:Hover.show')}",
        name:"moveWithMouse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"classAttr",
        definingClass:"class:Hover"
    },
    "method:Layout.reorderMembers":{
        ref:"method:Layout.reorderMembers",
        flags:"",
        description:" Move a range of members to a new position\n		    \n",
        name:"reorderMembers",
        seeAlso:[],
        params:[
            {description:"beginning of range of members to move", optional:false, name:"start",
             defaultValue:null, type:"number"},
            {description:"end of range of members to move, non-inclusive", optional:false, name:"end",
             defaultValue:null, type:"number"},
            {description:"new position to move the members to", optional:false, name:"newPosition",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "method:Window.setShowCloseButton":{
        ref:"method:Window.setShowCloseButton",
        flags:"",
        description:" Dynamically update ${isc.DocUtils.linkForRef('attr:Window.showCloseButton')} to show / hide the closeButton\n",
        name:"setShowCloseButton",
        seeAlso:[
            {ref:"attr:Window.headerControls", description:null},
            {ref:"attr:Window.showCloseButton", description:null}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "method:ListGrid.getFieldName":{
        ref:"method:ListGrid.getFieldName",
        flags:"A",
        description:" Given a column number or field id, return the field name of a field.\n",
        name:"getFieldName",
        seeAlso:[],
        params:[
            {description:"number or id of the field.", optional:false, name:"colNum", defaultValue:null,
             type:"number or id"}
        ],
        returns:{description:"Name of the field.", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["display"]
    },
    "class:MenuButton":{
        methods:["method:MenuButton.setShowMenuButtonImage", "method:MenuButton.setShowMenuBelow"],
        ref:"class:MenuButton",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:MenuButton.title", "attr:MenuButton.height", "attr:MenuButton.showMenuButtonImage", 
         "attr:MenuButton.menuButtonImage","attr:MenuButton.menuButtonImageUp", "attr:MenuButton.hiliteAccessKey", 
         "attr:MenuButton.showMenuBelow","attr:MenuButton.menu", "attr:MenuButton.menuAnimationEffect"],
        description:"Simple subclass of button associated with a menu widget (gets shown below the button).",
        name:"MenuButton",
        seeAlso:[],
        type:"class"
    },
    "method:CubeGrid.selectCells":{
        ref:"method:CubeGrid.selectCells",
        flags:"",
        description:"\n  Select/deselect cells that match a list of facetValues.  \n  Also supports an explicit list of cells or cellIds\n\n      <br><i>methodType</i> action\n",
        name:"selectCells",
        seeAlso:[],
        params:[
            {description:"cells to select", optional:false, name:"cellList", defaultValue:null,
             type:"Array of cells | facetValues object"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "classMethod:MathFunction.getRegisteredFunctionIndex":{
        ref:"classMethod:MathFunction.getRegisteredFunctionIndex",
        flags:"",
        description:" Returns an index of all registered functions by name\n \n",
        name:"getRegisteredFunctionIndex",
        seeAlso:[],
        returns:{description:null, type:"Index"},
        type:"classMethod",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "method:SearchForm.search":{
        ref:"method:SearchForm.search",
        flags:"",
        description:" Triggered when a SubmitItem is included in the form is submitted and gets pressed.\n \n",
        name:"search",
        seeAlso:[
            {ref:"method:DynamicForm.submit", description:null},
            {ref:"method:DynamicForm.submitValues", description:null}
        ],
        params:[
            {description:"the search criteria from the form", optional:false, name:"criteria",
             defaultValue:null, type:"Criteria"},
            {description:"the form being submitted", optional:false, name:"form", defaultValue:null,
             type:"SearchForm"}
        ],
        type:"method",
        definingClass:"class:SearchForm",
        groups:["submitting"]
    },
    "attr:ListGrid.groupLeadingIndent":{
        ref:"attr:ListGrid.groupLeadingIndent",
        flags:"IRW",
        description:"Default number of pixels by which to indent all groups.",
        name:"groupLeadingIndent",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null},
            {ref:"method:ListGrid.getGroupNodeHTML", description:null}
        ],
        valueType:"Number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "class:BrowserPlugin":{
        methods:[],
        ref:"class:BrowserPlugin",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Client Bridges"],
        attrs:[],
        description:"Container for a Browser Plugin.",
        name:"BrowserPlugin",
        seeAlso:[],
        type:"class",
        requiresModules:["PluginBridges"]
    },
    "attr:ListGridField.recordSummaryFunction":{
        ref:"attr:ListGridField.recordSummaryFunction",
        flags:"IR",
        description:"Only applies to fields of type <code>\"summary\"</code>.\n Fields of this type will display a calculated value based on the other field values\n within the current record.\n <P>\n This attribute specifies how the summary field value will be calculated. See \n ${isc.DocUtils.linkForRef('type:RecordSummaryFunction')} for valid options.\n <P>\n A subset of the ListGrid's fields will be passed to the RecordSummaryFunction. \n Which fields to include is determined based on ${isc.DocUtils.linkForRef('attr:ListGridField.includeInRecordSummary')}\n <P>\n If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} or ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, this\n field's value in the summary row[s] will still be calculated by calling this method.\n In this case, the record object passed in will contain summary values for each field.\n If custom handling is required for this case, it may be detected by checking the\n record object's ${isc.DocUtils.linkForRef('attr:ListGridRecord.isGroupSummary')} and ${isc.DocUtils.linkForRef('attr:ListGridRecord.isGridSummary')}\n attributes.",
        name:"recordSummaryFunction",
        seeAlso:[],
        valueType:"RecordSummaryFunction",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:FormItem.emptyDisplayValue":{
        ref:"attr:FormItem.emptyDisplayValue",
        flags:"IRW",
        description:"Text to display when this form item has a null or undefined value.",
        name:"emptyDisplayValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "attr:RPCRequest.ignoreTimeout":{
        ref:"attr:RPCRequest.ignoreTimeout",
        flags:"IRWA",
        description:"When set to true, no reply is expected from the server.  However, if a reply is received, it will\n be processed.<p>\n\n Note: setting this to true, forces ${isc.DocUtils.linkForRef('attr:RPCRequest.sendNoQueue')} to <code>true</code> for\n this request.",
        name:"ignoreTimeout",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:Dialog.noClick":{
        ref:"method:Dialog.noClick",
        flags:"",
        description:" Handle a click on the 'no' button of this Dialog.\n Default implementation is to return <code>false</code>.\n Override to do something else.\n",
        name:"noClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "attr:LoginDialog.items":{
        ref:"attr:LoginDialog.items",
        flags:"IR",
        description:"Specifies the dialog contents. By default, the dialog only contains\n ${isc.DocUtils.linkForRef('attr:LoginDialog.loginForm')}. If desired, additional widgets may be placed before/after\n the loginForm. To specify these widgets as ${isc.DocUtils.linkForRef('group:autoChildren')}, use the syntax\n \"autoChild:<i>childName</i>\" ${isc.DocUtils.linkForRef('group:autoChildren','as used for panes/items of \n Tabs/SectionStacks')}.",
        name:"items",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"[\"autoChild:loginForm\"]",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:StretchImgButton.deselect":{
        ref:"method:StretchImgButton.deselect",
        flags:"",
        description:" Select this object.\n",
        name:"deselect",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "attr:DataSource.useLocalValidators":{
        ref:"attr:DataSource.useLocalValidators",
        flags:"IRWA",
        description:"Whether to attempt validation on the client at all for this DataSource.  If unset (the\n default), client-side validation is enabled.\n <p>\n Disabling client-side validation entirely is a good way to test server-side validation.",
        name:"useLocalValidators",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["validation"]
    },
    "attr:FilterBuilder.addButtonPrompt":{
        ref:"attr:FilterBuilder.addButtonPrompt",
        flags:"IR",
        description:"The hover prompt text for the add button.",
        name:"addButtonPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Add\"",
        type:"attr",
        definingClass:"class:FilterBuilder",
        groups:["i18nMessages"]
    },
    "class:GroupingMessages":{
        methods:[],
        ref:"class:GroupingMessages",
        classAttrs:["classAttr:GroupingMessages.upcomingTodayTitle", 
         "classAttr:GroupingMessages.upcomingTomorrowTitle","classAttr:GroupingMessages.upcomingThisWeekTitle", 
         "classAttr:GroupingMessages.upcomingNextWeekTitle","classAttr:GroupingMessages.upcomingNextMonthTitle", 
         "classAttr:GroupingMessages.upcomingBeforeTitle","classAttr:GroupingMessages.upcomingLaterTitle", "classAttr:GroupingMessages.byDayTitle", 
         "classAttr:GroupingMessages.byWeekTitle","classAttr:GroupingMessages.byMonthTitle", "classAttr:GroupingMessages.byQuarterTitle", 
         "classAttr:GroupingMessages.byYearTitle","classAttr:GroupingMessages.byDayOfMonthTitle", 
         "classAttr:GroupingMessages.byUpcomingTitle","classAttr:GroupingMessages.byHoursTitle", "classAttr:GroupingMessages.byMinutesTitle", 
         "classAttr:GroupingMessages.bySecondsTitle","classAttr:GroupingMessages.byMilisecondsTitle"],
        classMethods:[],
        treeLocation:["Client Reference/Grids/ListGrid"],
        attrs:[],
        description:"Grouping titles that will be displayed when data is grouped\n in a ${isc.DocUtils.linkForRef('class:ListGrid')}.",
        name:"GroupingMessages",
        seeAlso:[],
        type:"class"
    },
    "method:DataSource.getField":{
        ref:"method:DataSource.getField",
        flags:"",
        description:" Return the field definition object.\n",
        name:"getField",
        seeAlso:[],
        params:[
            {description:"Name of the field to retrieve", optional:false, name:"fieldName",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"field object", type:"DataSourceField"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "group:printing":{
        ref:"group:printing",
        refs:[],
        name:"printing",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.backgroundImage":{
        ref:"attr:Canvas.backgroundImage",
        flags:"IR",
        description:"URL for a background image for this widget (corresponding to the CSS \"background-image\"\n attribute).",
        name:"backgroundImage",
        setter:["setBackgroundImage"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:Date.setDefaultDateSeparator":{
        ref:"method:Date.setDefaultDateSeparator",
        flags:"",
        description:" Sets a new default separator that will be used when formatting dates. By default, this\n is a forward slash character: \"/\"\n",
        name:"setDefaultDateSeparator",
        seeAlso:[],
        params:[
            {description:"separator to use in dates", optional:false, name:"separator",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:ResultSet.transformData":{
        ref:"method:ResultSet.transformData",
        flags:"",
        description:" <code>transformData()</code> provides an opportunity to modify data that has been\n returned from the server, before it has been integrated into the client-side cache.\n <P>\n If data is not immediately suited for client-side use when it is returned from the\n ultimate data store, this method allows it to be transformed on the client so that such\n transform operations do not impact server scalability.\n <P>\n It is legal for <code>transformData()</code> to modify not only the records, but also\n their number (by modifying startRow and endRow on the ${isc.DocUtils.linkForRef('class:DSResponse')} object).\n <P>\n See also ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} for an alternative entry point which\n applies to all DSResponses for a DataSource.\n\n",
        name:"transformData",
        seeAlso:[],
        params:[
            {description:"data returned from the server", optional:false, name:"newData",
             defaultValue:null, type:"any"},
            {description:"the DSResponse object returned by the                             server",
             optional:false, name:"dsResponse", defaultValue:null, type:"DSResponse"}
        ],
        returns:{description:"the modified data, ready to be cached", type:"Array of Objects"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:DataSourceField.type":{
        ref:"attr:DataSourceField.type",
        flags:"IR",
        description:"Type of this field.  Required for all DataSource fields.\n <P>\n Field type may imply automatic validators (for example, an integer field cannot accept the\n value \"foo\").  Field type also affects the default behaviors of DataBound components, for\n example, if a field is declared as type \"date\", components that edit that field will\n automatically choose a date-editing interface with pop-up date picker.",
        name:"type",
        seeAlso:[],
        valueType:"FieldType",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["basics"]
    },
    "attr:Calendar.dayBodyBaseStyle":{
        ref:"attr:Calendar.dayBodyBaseStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to the day body of the month view\n of the calendar. This style will have \"Dark\", \"Over\", \"Selected\", or \"Disabled\"\n appended to it according to the state of the cell.",
        name:"dayBodyBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calMonthDayBody\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "method:CubeGrid.deselectAllFacetValues":{
        ref:"method:CubeGrid.deselectAllFacetValues",
        flags:"",
        description:"\n deselect all headers in a headerBar (specified by facetId) or\n all headerBars (if no facetId)\n\n  <br><i>methodType</i> action\n",
        name:"deselectAllFacetValues",
        seeAlso:[],
        params:[
            {description:"ID of facet - if null, selects all headerbars' headers", optional:true,
             name:"facetId", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "class:HTMLPane":{
        methods:[],
        ref:"class:HTMLPane",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:[],
        description:"Use the HTMLPane component to display HTML content in a pane of specified size. If the HTML\n content is larger than the size of the pane, the pane will provide scrollbars for viewing\n clipped content.\n <P>\n You can set the size of an HTMLPane directly via the width and height properties, or\n indirectly by placing the HTMLPane in a container component (${isc.DocUtils.linkForRef('class:Layout')}, ${isc.DocUtils.linkForRef('class:Window')}, \n ${isc.DocUtils.linkForRef('class:SectionStack')}, etc) that manages the sizes of its members.",
        name:"HTMLPane",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('htmlPane')}"]
    },
    "group:track":{
        ref:"group:track",
        refs:["attr:Scrollbar.btnSize", "attr:Scrollbar.showTrackEnds", "attr:Scrollbar.trackEndWidth", 
         "attr:Scrollbar.trackEndHeight"],
        name:"track",
        seeAlso:[],
        type:"group"
    },
    "classAttr:RPCResponse.STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED":{
        ref:"classAttr:RPCResponse.STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED",
        flags:"R",
        description:"Indicates that too many authentication attempts have been made and the server refuses to\n accept any more login attempts.",
        name:"STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-6",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:PickTreeItem.canSelectParentItems":{
        ref:"attr:PickTreeItem.canSelectParentItems",
        flags:"IRW",
        description:"If true, clicking or pressing Enter on a menu item that has a submenu will\n		select that item (with standard behavior of hiding the menus, calling click\n		handlers, etc) instead of showing the submenu.",
        name:"canSelectParentItems",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('treesEditing')}"],
        definingClass:"class:PickTreeItem",
        groups:["selection"]
    },
    "type:VelocityExpression":{
        ref:"type:VelocityExpression",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        description:"An expression in the <a href=http://velocity.apache.org/engine/releases/velocity-1.5/user-guide.html>\n Velocity Template Language</a> (VTL).  For more information on SmartClient's Velocity support, \n see ${isc.DocUtils.linkForRef('group:velocitySupport','Velocity support')}.\n <p>\n Note that a <code>VelocityExpression</code> must often evaluate to a particular type of value \n to be useful.  For example, ${isc.DocUtils.linkForRef('attr:DataSource.requires')} must evaluate to true or false \n (Boolean objects or strings containing those two words), and ${isc.DocUtils.linkForRef('attr:Mail.messageData')} must\n evaluate to a Java <code>Map</code> object, or a Java <code>List</code> containing only \n <code>Map</code>s.",
        name:"VelocityExpression",
        seeAlso:[],
        type:"type",
        groups:["velocitySupport"]
    },
    "attr:FusionChart.chartProperties":{
        ref:"attr:FusionChart.chartProperties",
        flags:"IRA",
        description:"Properties to passthrough to the 'graph' element of the XML generated for FusionCharts.\n <P>\n Some automatic conversions are done:\n <ul>\n <li> boolean values become '0' and '1' as FusionCharts expects </li>\n <li> color values with a leading '#' have the '#' stripped to match FusionCharts </li>\n </ul>",
        name:"chartProperties",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FusionChart"
    },
    "method:FormItem.keyDown":{
        ref:"method:FormItem.keyDown",
        flags:"",
        description:" StringMethod fired in response to a keydown while focused in this form item.\n\n",
        name:"keyDown",
        seeAlso:[],
        params:[
            {description:"Item over which the keydown occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"Pointer to the item's form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"},
            {description:"Name of the key pressed (Example: <code>\"A\"</code>, <code>\"Enter\"</code>)",
             optional:false, name:"keyName", defaultValue:null, type:"KeyName"}
        ],
        returns:{description:"return false to attempt to cancel the event.  Note for general purpose                   APIs for managing whether user input is allowed, use ${isc.DocUtils.linkForRef('method:FormItem.change')} \n                   or ${isc.DocUtils.linkForRef('method:FormItem.transformInput')} instead.",
         type:"boolean"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "method:Class.logWarn":{
        ref:"method:Class.logWarn",
        flags:"",
        description:" Log a message at \"warn\" priority\n \n",
        name:"logWarn",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:Hilite.cssText":{
        ref:"attr:Hilite.cssText",
        flags:"IR",
        description:"CSS text to be applied to cells where this hilite is applied, for example,\n \"background-color:#FF0000\"",
        name:"cssText",
        seeAlso:[],
        valueType:"CSSText",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "method:Layout.setVisibleMember":{
        ref:"method:Layout.setVisibleMember",
        flags:"",
        description:" Hide all other members and make the single parameter member visible.\n\n",
        name:"setVisibleMember",
        seeAlso:[],
        params:[
            {description:"member to show", optional:false, name:"member", defaultValue:null,
             type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:Layout.leaveScrollbarGap":{
        ref:"attr:Layout.leaveScrollbarGap",
        flags:"IR",
        description:"Whether to leave a gap for a vertical scrollbar even when one is not actually present.\n <P>\n This setting avoids the layout resizing all members when the vertical scrollbar is\n introduced or removed, which can avoid unnecessary screen shifting and improve\n performance.",
        name:"leaveScrollbarGap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Layout"
    },
    "attr:Calendar.eventAutoArrange":{
        ref:"attr:Calendar.eventAutoArrange",
        flags:"IR",
        description:"If set to true, enables the auto-arrangement of events that share time in the calendar.  The\n default is true.",
        name:"eventAutoArrange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "method:List.addAt":{
        ref:"method:List.addAt",
        flags:"",
        description:" Add a single item to this array at a specific position in the list, sliding other items over\n to fit.\n\n",
        name:"addAt",
        seeAlso:[],
        params:[
            {description:"object to add", optional:false, name:"obj", defaultValue:null, type:"object"},
            {description:"position in the list to add at", optional:false, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"object that was added", type:"object"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:Applet.classID":{
        ref:"attr:Applet.classID",
        flags:"IR",
        description:"This attribute is used only with useTag: \"object\".  In combination with \n  ${isc.DocUtils.linkForRef('attr:Applet.objectCodeBase')} this tag specifies the minimum JVM version required to view\n  this applet and the URL from which the JVM can be downloaded.\n  <p>\n  The default settings match the default behavior of the &lt;applet&gt; tag which requires JVM\n  1.3 or higher.\n  <p>\n  The default classID is: \"clsid:8AD9C840-044E-11D1-B3E9-00805F499D93\"\n  <p>\n  For notes on this feature see: <a href='http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie' onclick=\"window.open('http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie');return false;\">http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html#in-ie</a>",
        name:"classID",
        seeAlso:[
            {ref:"attr:Applet.objectCodeBase", description:null}
        ],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:StatefulCanvas.showDisabled":{
        ref:"attr:StatefulCanvas.showDisabled",
        flags:"IRW",
        description:"Should we visibly change state when disabled?",
        name:"showDisabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "attr:ListGrid.sortNumeralStyle":{
        ref:"attr:ListGrid.sortNumeralStyle",
        flags:"IRWA",
        description:"When multiple fields are sorted, the Style to apply to the numeral that appears after the \n sort-arrows in the header-buttons of sorted fields.",
        name:"sortNumeralStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"sortNumeral\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:CalendarEvent.eventWindowStyle":{
        ref:"attr:CalendarEvent.eventWindowStyle",
        flags:"IR",
        description:"CSS style series to use for the draggable event window that represents this event.  If\n specified, overrides ${isc.DocUtils.linkForRef('attr:Calendar.eventWindowStyle')} for this specific event.\n <P>\n The name of this field within the CalendarEvent can be changed via \n ${isc.DocUtils.linkForRef('attr:Calendar.eventWindowStyleField')}",
        name:"eventWindowStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "method:MultiView.filterData":{
        ref:"method:MultiView.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Calendar.firstDayOfWeek":{
        ref:"attr:Calendar.firstDayOfWeek",
        flags:"IRW",
        description:"The numeric day (0-6) which the calendar should consider as the first day of the week",
        name:"firstDayOfWeek",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["date"]
    },
    "class:SVG":{
        methods:[],
        ref:"class:SVG",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Client Bridges"],
        attrs:["attr:SVG.pluginsPage", "attr:SVG.src"],
        description:"ISC Abstraction for SVG controls",
        name:"SVG",
        seeAlso:[],
        type:"class",
        requiresModules:["PluginBridges"]
    },
    "attr:Calendar.eventOverlap":{
        ref:"attr:Calendar.eventOverlap",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('attr:Calendar.eventAutoArrange')} is true, setting eventOverlap to true causes events that \n share timeslots to overlap each other by a percentage of their width, specified by \n ${isc.DocUtils.linkForRef('attr:Calendar.eventOverlapPercent')}.  The default is true.",
        name:"eventOverlap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "method:FormItem.itemHover":{
        ref:"method:FormItem.itemHover",
        flags:"A",
        description:"  Optional stringMethod to fire when the user hovers over this item.\n  Return false to suppress default behavior of showing a hover canvas containing the\n  HTML returned by <code>formItem.itemHoverHTML()</code> / \n  <code>form.itemHoverHTML()</code>.\n\n",
        name:"itemHover",
        seeAlso:[
            {ref:"method:FormItem.titleHover", description:null},
            {ref:"method:FormItem.itemHoverHTML", description:null}
        ],
        params:[
            {description:"Pointer to this item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"This items form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "classMethod:Class.echoAll":{
        ref:"classMethod:Class.echoAll",
        flags:"",
        description:" Like echo(), except that if passed an Array, echoAll() will echo() every element of the\n Array.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoAll",
        seeAlso:[
            {ref:"method:Class.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:Array.removeAt":{
        ref:"method:Array.removeAt",
        flags:"",
        description:" Remove the item at the specified position, rearranging all subsequent items to fill the gap\n\n",
        name:"removeAt",
        seeAlso:[],
        params:[
            {description:"position to remove", optional:false, name:"pos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"item that was removed", type:"any"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "attr:Slider.roundValues":{
        ref:"attr:Slider.roundValues",
        flags:"IRW",
        description:"Specifies whether the slider value should be rounded to the nearest integer.  If set to\n false, values will be rounded to a fixed number of decimal places controlled by\n ${isc.DocUtils.linkForRef('attr:Slider.roundPrecision')}.",
        name:"roundValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider"
    },
    "method:ResultSet.rangeIsLoaded":{
        ref:"method:ResultSet.rangeIsLoaded",
        flags:"A",
        description:" Whether the given range of rows has been loaded.\n\n Unlike getRange(), will not trigger a server fetch.  \n\n",
        name:"rangeIsLoaded",
        seeAlso:[],
        params:[
            {description:"start position, inclusive", optional:false, name:"startRow",
             defaultValue:null, type:"number"},
            {description:"end position, exclusive", optional:false, name:"endRow", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"true if all rows in the given range have been loaded, false if any rows in                   the range have not been loaded or are still in the process of being loaded",
         type:"boolean"},
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:SpinnerItem.maskOverwriteMode":{
        ref:"attr:SpinnerItem.maskOverwriteMode",
        flags:null,
        description:"Not applicable to a SpinnerItem.",
        name:"maskOverwriteMode",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:SpinnerItem"
    },
    "attr:SliderItem.roundPrecision":{
        ref:"attr:SliderItem.roundPrecision",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:Slider.roundValues')} is false, the slider value will be rounded to this number of\n decimal places. If set to null the value will not be rounded",
        name:"roundPrecision",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"],
        definingClass:"class:SliderItem"
    },
    "attr:TabSet.skinImgDir":{
        ref:"attr:TabSet.skinImgDir",
        flags:"IR",
        description:"Default directory for skin images (those defined by the class), relative to the\n Page-wide ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','skinDir')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"images/TabSet/\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["images"]
    },
    "classMethod:DataSource.load":{
        ref:"classMethod:DataSource.load",
        flags:"",
        description:" Load a DataSource or an array of DataSources using the DataSourceLoader servlet.  When\n a callback is specified, this is fired after the DataSources are loaded.  If\n no loading occurs because the requested DataSource(s) are already loaded, a warning is \n logged and the callback is fired.\n <P>\n To force reloading of DataSources that have already been loaded,\n pass <code>true</code> in the forceReload parameter.\n\n",
        name:"load",
        seeAlso:[],
        params:[
            {description:"DataSource ID or Array of DataSource IDs", optional:false, name:"dsID",
             defaultValue:null, type:"String or Array of String"},
            {description:"Callback to fire after DataSource loading completes", optional:false,
             name:"callback", defaultValue:null, type:"Callback"},
            {description:"Forcibly reload a dataSource if it's already loaded", optional:false,
             name:"forceReload", defaultValue:null, type:"Boolean"}
        ],
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:StretchImgButton.titleStyle":{
        ref:"attr:StretchImgButton.titleStyle",
        flags:"IRW",
        description:"CSS style applied to the title text only.  Defaults to ${isc.DocUtils.linkForRef('attr:StretchImgButton.baseStyle')} when unset.\n <P>\n With a separate <code>titleStyle</code> and ${isc.DocUtils.linkForRef('attr:StretchImgButton.baseStyle')} set, you can provide a\n backgroundColor via <code>baseStyle</code> that will allow translucent .png media to be\n \"tinted\" by the underlying background color, so that a single set of media can provide\n range of color options.  In this usage, the <code>titleStyle</code> should generally not\n specify a background color as this would block out the media that appears behind the\n title.",
        name:"titleStyle",
        setter:["setTitleStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "attr:ColumnTree.data":{
        flags:"IRW",
        getter:["getData"],
        type:"attr",
        ref:"attr:ColumnTree.data",
        description:"A ${isc.DocUtils.linkForRef('class:Tree')} object consisting of nested ${isc.DocUtils.linkForRef('object:TreeNode')}s to \n display in this ColumnTree.  \n The <code>data</code> property will typically not be explicitly specified for \n databound ColumnTrees, where the data is returned from the server via databound component\n methods such as <code>fetchData()</code>",
        name:"data",
        setter:["setData"],
        valueType:"Tree",
        seeAlso:[],
        defaultValue:"null",
        groups:["data"],
        definingClass:"class:ColumnTree"
    },
    "attr:ListGrid.sortDescendingImage":{
        ref:"attr:ListGrid.sortDescendingImage",
        flags:"IRWA",
        description:"Image to show when sorting descending. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.",
        name:"sortDescendingImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:PickList.valueField":{
        ref:"attr:PickList.valueField",
        flags:"IRA",
        getter:["FormItem.getValueFieldName"],
        description:"If this form item maps data values to display values by retrieving the \n ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} values from an \n ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}, this property \n denotes the the field to use as the underlying data value in records from the \n optionDataSource.<br>\n If unset, assumed to be the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this form item.",
        name:"valueField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList",
        groups:["display_values"]
    },
    "method:ColorPicker.setOpacity":{
        ref:"method:ColorPicker.setOpacity",
        flags:"",
        description:" Sets the Opacity of the selected color. Ignored if opacity is switched off.\n",
        name:"setOpacity",
        seeAlso:[
            {ref:"method:ColorPicker.getOpacity", description:null}
        ],
        params:[
            {description:"An integer between 0 and 100", optional:false, name:"newValue",
             defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:ListGrid.animateRemoveSpeed":{
        ref:"attr:ListGrid.animateRemoveSpeed",
        flags:"IRW",
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.animateRemoveRecord','animating record removal')}, this property \n designates the speed of the animation in pixels per second. Takes presidence over the\n ${isc.DocUtils.linkForRef('attr:ListGrid.animateRemoveTime')} property, which allows the developer to specify a\n duration for the animation rather than a speed.",
        name:"animateRemoveSpeed",
        seeAlso:[
            {ref:"attr:ListGrid.animateRemoveRecord", description:null}
        ],
        valueType:"number",
        defaultValue:"200",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["animation"]
    },
    "method:DateChooser.setData":{
        ref:"method:DateChooser.setData",
        flags:"",
        description:" Set the picker to show the given date.\n \n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"new value", optional:false, name:"date", defaultValue:null, type:"Date"}
        ],
        type:"method",
        definingClass:"class:DateChooser"
    },
    "attr:DatabaseBrowser.title":{
        ref:"attr:DatabaseBrowser.title",
        flags:"IR",
        description:"A title to show in the header button of the DatabaseBrowser's schema tree.  If not set,\n defaults to the name of the connected database",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:LinkItem.linkTitle":{
        ref:"attr:LinkItem.linkTitle",
        flags:"IRW",
        description:"Optional title text to display for this item's link. If unspecified the value of the item\n will be the title text as well as the target of the link.",
        name:"linkTitle",
        setter:["setLinkTitle"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LinkItem"
    },
    "attr:DateChooser.cancelButtonTitle":{
        ref:"attr:DateChooser.cancelButtonTitle",
        flags:"IRW",
        description:"Title for the cancellation button.",
        name:"cancelButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Cancel\"",
        type:"attr",
        definingClass:"class:DateChooser",
        groups:["i18nMessages"]
    },
    "attr:Facet.canCollapse":{
        ref:"attr:Facet.canCollapse",
        flags:"IR",
        description:"For tree facets, whether expand/collapse controls should be shown.",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"facet.isTree",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Window.canDragResize":{
        ref:"attr:Window.canDragResize",
        flags:"IRW",
        description:"Can the window be drag-resized? If true the window may be drag resized from its edges,\n and if showing, via the resiszer icon in the footer.",
        name:"canDragResize",
        seeAlso:[
            {ref:"attr:Window.showResizer", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["dragging", "resizing"]
    },
    "method:Slider.setNumValues":{
        ref:"method:Slider.setNumValues",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.numValues','number of values')} for the slider \n\n",
        name:"setNumValues",
        seeAlso:[],
        params:[
            {description:"the new number of values", optional:false, name:"newNumValues",
             defaultValue:null, type:"float"}
        ],
        type:"method",
        definingClass:"class:Slider"
    },
    "method:DynamicForm.editSelectedData":{
        ref:"method:DynamicForm.editSelectedData",
        flags:"",
        description:"\n Edit the record selected in the specified selection component (typically a\n ${isc.DocUtils.linkForRef('class:ListGrid')}).\n <P>\n Updates the values of this editor to match the selected record's values.\n <P>\n If this form has a dataSource, then saving via ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} will use the \n \"update\" operation type.\n\n",
        name:"editSelectedData",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"the ListGrid or ID of a ${isc.DocUtils.linkForRef('class:ListGrid')} whose currently selected\n     record(s) is/are to be edited",
             optional:false, name:"selectionComponent", defaultValue:null, type:"ListGrid or ID"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('updateOperation')}"],
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "method:RichTextEditor.getValue":{
        ref:"method:RichTextEditor.getValue",
        flags:"",
        description:"  Retrieves the current value of the edit area.\n",
        name:"getValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:RichTextEditor"
    },
    "class:DataPathItem":{
        methods:[],
        ref:"class:DataPathItem",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"TextItem subclass for managing a DataPath",
        name:"DataPathItem",
        seeAlso:[],
        type:"class"
    },
    "attr:DSRequest.exportAs":{
        ref:"attr:DSRequest.exportAs",
        flags:"IR",
        description:"The format in which the data should be exported.  See ${isc.DocUtils.linkForRef('type:ExportFormat')} for more \n information.",
        name:"exportAs",
        seeAlso:[],
        valueType:"ExportFormat",
        defaultValue:"\"csv\"",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:DynamicForm.setItems":{
        ref:"method:DynamicForm.setItems",
        flags:"",
        description:" Synonym for ${isc.DocUtils.linkForRef('method:DynamicForm.setFields')}\n\n",
        name:"setItems",
        seeAlso:[],
        params:[
            {description:"list of new items to show in the form", optional:false, name:"itemList",
             defaultValue:null, type:"Array of FormItem Properties"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["elements"]
    },
    "attr:Slider.thumbSrc":{
        ref:"attr:Slider.thumbSrc",
        flags:"IRW",
        description:"The base filename for the slider thumb images.\n The filenames for the thumb icons are assembled from this base filename and the state of the\n thumb, as follows:<br>\n Assume the thumbSrc is set to <code>{baseName}.{extension}</code><br>\n The full set of images to be displayed is:<br>\n For horizontal sliders:\n <ul>\n <li><code>h{baseName}.{extension}</code>: default enabled appearance.\n <li><code>h{baseName}_down.{extension}</code>:  appearance when the slider is enabled and the\n     thumb is clicked.\n <li><code>h{baseName}_off.{extension}</code>:  appearance when the slider is disabled.\n </ul>\n For vertical sliders:\n <ul>\n <li><code>v{baseName}.{extension}</code>: default enabled appearance.\n <li><code>v{baseName}_down.{extension}</code>:  appearance when the slider is enabled and the\n     thumb is clicked.\n <li><code>v{baseName}_off.{extension}</code>:  appearance when the slider is disabled.\n </ul>",
        name:"thumbSrc",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"thumb.gif\"",
        type:"attr",
        definingClass:"class:Slider"
    },
    "class:DataView":{
        methods:["method:DataView.dataViewLoaded"],
        ref:"class:DataView",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:[],
        description:"A DataView is an encapsulation of the components required to present a particular view of \n your data and the dataSources and/or webservices required to obtain and manage that data",
        name:"DataView",
        seeAlso:[],
        type:"class"
    },
    "method:DynamicForm.showItem":{
        ref:"method:DynamicForm.showItem",
        flags:"",
        description:" Show a form item via ${isc.DocUtils.linkForRef('method:FormItem.show')}\n",
        name:"showItem",
        seeAlso:[],
        params:[
            {description:"Name of the item to show", optional:false, name:"itemName", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "group:statusCodes":{
        ref:"group:statusCodes",
        refs:["classAttr:RPCResponse.STATUS_SUCCESS", "classAttr:RPCResponse.STATUS_FAILURE", 
         "classAttr:RPCResponse.STATUS_VALIDATION_ERROR","classAttr:RPCResponse.STATUS_LOGIN_INCORRECT", 
         "classAttr:RPCResponse.STATUS_MAX_LOGIN_ATTEMPTS_EXCEEDED","classAttr:RPCResponse.STATUS_LOGIN_REQUIRED", 
         "classAttr:RPCResponse.STATUS_LOGIN_SUCCESS","classAttr:RPCResponse.STATUS_UPDATE_WITHOUT_PK_ERROR", 
         "classAttr:RPCResponse.STATUS_TRANSPORT_ERROR","classAttr:RPCResponse.STATUS_UNKNOWN_HOST_ERROR", 
         "classAttr:RPCResponse.STATUS_CONNECTION_RESET_ERROR","classAttr:RPCResponse.STATUS_SERVER_TIMEOUT"],
        description:"Status codes returned by the server as rpcResponse.status.<br>\n See the error handling doc section in ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager')} for more \n information on these codes",
        name:"statusCodes",
        seeAlso:[],
        type:"group"
    },
    "attr:Slider.numValues":{
        ref:"attr:Slider.numValues",
        flags:"IRW",
        description:"The number of discrete values represented by slider. If specified, the range of valid\n values (between <code>minValue</code> and <code>maxValue</code>) will be divided into\n this many steps. As the thumb is moved along the track it will only select these values\n and appear to jump between the steps.",
        name:"numValues",
        setter:["setNumValues"],
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:Slider"
    },
    "attr:DataSourceField.required":{
        ref:"attr:DataSourceField.required",
        flags:"IR",
        description:"Indicates this field must be non-null in order for a record to pass validation.\n <P>\n Note that <code>required</code> should not be set for a server-generated field, such as a\n sequence, or validation will fail on the client.",
        name:"required",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["dataType"]
    },
    "group:dataChanged":{
        ref:"group:dataChanged",
        title:"Data Changes",
        refs:[],
        description:"Operations that change the Array",
        name:"dataChanged",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.dragMove":{
        ref:"method:Canvas.dragMove",
        flags:"A",
        description:" Executed every time the mouse moves while dragging this canvas.\n\n",
        name:"dragMove",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to cancel drag interaction.", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dragPan')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:Window.restore":{
        ref:"method:Window.restore",
        flags:"",
        description:" Restores the window to its specified height and width after a call to ${isc.DocUtils.linkForRef('method:Window.minimize')} or \n ${isc.DocUtils.linkForRef('method:Window.maximize')}. Called from a click on the restore button shown in place of the \n minimize or maximize button when the window is minimized or maximized.<br>\n Resizing will occur as an animation if ${isc.DocUtils.linkForRef('attr:Window.animateMinimize')} is true.\n",
        name:"restore",
        seeAlso:[],
        type:"method",
        definingClass:"class:Window"
    },
    "attr:DatabaseBrowser.excludeSubstring":{
        ref:"attr:DatabaseBrowser.excludeSubstring",
        flags:"IR",
        description:"If set, specifies a substring which must NOT exist in a table name for it to be included in\n this DatabaseBrowser.  If this property is set to a List of strings, table names are\n excluded if they match any one of the strings.  The comparison is case-insensitive.\n <p>\n For example, <code>excludeSubstring: [\"E\", \"qry\"]</code> would exclude all the following \n table names: \"table\", \"QryTbl\", \"QRY\", \"ORDERS\"\n <p>\n Note that if you specify both include and exclude criteria and they conflict (ie,\n according to the criteria you set, a table should be both included and excluded), \n exclude wins.",
        name:"excludeSubstring",
        seeAlso:[],
        valueType:"String | List of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "method:ListGrid.isExpansionField":{
        ref:"method:ListGrid.isExpansionField",
        flags:"",
        description:" Identifies whether the passed-in field is the specially generated\n ${isc.DocUtils.linkForRef('attr:ListGrid.expansionField','expansionField')} used when ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is\n true.  Use this method in your custom event handlers to avoid inappropriately\n performing actions when the expansionField is clicked on.\n \n",
        name:"isExpansionField",
        seeAlso:[],
        params:[
            {description:"field to test", optional:false, name:"field", defaultValue:null,
             type:"ListGridField"}
        ],
        returns:{description:"whether the provided field is the expansion field", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:ListGrid.longTextEditorThreshold":{
        ref:"attr:ListGrid.longTextEditorThreshold",
        flags:"IRW",
        description:"When the length of the field specified by ${isc.DocUtils.linkForRef('attr:DataSourceField.length')} exceeds this\n value, the ListGrid shows an edit field of type ${isc.DocUtils.linkForRef('attr:ListGrid.longTextEditorType')}\n rather than the standard text field when the field enters inline edit mode.",
        name:"longTextEditorThreshold",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"255",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:StretchImgButton.action":{
        ref:"method:StretchImgButton.action",
        flags:"",
        description:" This property contains the default 'action' for the Button to fire when activated.\n",
        name:"action",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton"
    },
    "method:Array.last":{
        ref:"method:Array.last",
        flags:"",
        description:" Return the last item in this list\n\n",
        name:"last",
        seeAlso:[],
        returns:{description:"last item in the list", type:"any"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:DataSource.autoDeriveSchema":{
        ref:"attr:DataSource.autoDeriveSchema",
        flags:"IR",
        description:"For a DataSource with serverType: \"sql\" or serverType: \"hibernate\", automatically derive\n the dataSource's schema (field definitions) from the SQL table specified in \n ${isc.DocUtils.linkForRef('attr:DataSource.tableName')}.  This causes SmartClient to create a \"super\" DataSource, which\n this dataSource then automatically ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom','inheritsFrom')}.  This allows you to \n override auto-derived schema as required.\n <p>\n This property is only applicable if you are using the SmartClient server.",
        name:"autoDeriveSchema",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["fields"]
    },
    "attr:SummaryBuilder.titleField":{
        ref:"attr:SummaryBuilder.titleField",
        flags:"IR",
        description:"TextItem that allows users to set the title for this field.",
        name:"titleField",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:DynamicForm.itemHoverDelay":{
        ref:"attr:DynamicForm.itemHoverDelay",
        flags:"IRW",
        description:"If the user rolls over an item, how long a delay before we fire any hover action / show\n a hover for that item?",
        name:"itemHoverDelay",
        seeAlso:[
            {ref:"attr:FormItem.hoverDelay", description:null}
        ],
        valueType:"number",
        defaultValue:"500",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:Calendar.eventSnapGap":{
        ref:"attr:Calendar.eventSnapGap",
        flags:"IR",
        description:"Determines how many minutes an event can be moved or resized by.",
        name:"eventSnapGap",
        seeAlso:[],
        valueType:"Integer",
        defaultValue:"30",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["editing"]
    },
    "group:dataChanges":{
        ref:"group:dataChanges",
        refs:[],
        name:"dataChanges",
        seeAlso:[],
        type:"group"
    },
    "method:TileGrid.addTile":{
        ref:"method:TileGrid.addTile",
        flags:"",
        description:" This is not allowed for tileGrid. Instead, use ${isc.DocUtils.linkForRef('method:TileGrid.addData')}.\n\n",
        name:"addTile",
        seeAlso:[],
        type:"method",
        definingClass:"class:TileGrid"
    },
    "type:DateItemSelectorFormat":{
        ref:"type:DateItemSelectorFormat",
        values:[
            {description:"Output fields in day, month, year order.", value:"DateItem.DAY_MONTH_YEAR"},
            {description:"Output fields in month, day, year order.", value:"DateItem.MONTH_DAY_YEAR"},
            {description:"Output fields in year, month, day order.", value:"DateItem.YEAR_MONTH_DAY"},
            {description:"Output only day, month fields.", value:"DateItem.DAY_MONTH"},
            {description:"Output only month, day fields.", value:"DateItem.MONTH_DAY"},
            {description:"Output only year, month fields.", value:"DateItem.YEAR_MONTH"},
            {description:"Output only month, year fields.", value:"DateItem.YEAR_MONTH"}
        ],
        description:"Order of pickers and which pickers are present when using a DateItem with\n ${isc.DocUtils.linkForRef('useTextField')} false.",
        name:"DateItemSelectorFormat",
        seeAlso:[],
        type:"type"
    },
    "attr:HTMLFlow.captureSCComponents":{
        ref:"attr:HTMLFlow.captureSCComponents",
        flags:"IR",
        description:"If true, SmartClient components created while executing the loaded HTML are captured\n for rendering inside the HTMLFlow.\n <P>\n Only applies when contentsType is <b>not</b> \"page\".",
        name:"captureSCComponents",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "method:Flashlet.getPluginHandle":{
        ref:"method:Flashlet.getPluginHandle",
        flags:"A",
        description:" \n  Returns a handle to the flashlet DOM element (valid only after the component has been drawn).  \n\n",
        name:"getPluginHandle",
        seeAlso:[],
        type:"method",
        definingClass:"class:Flashlet"
    },
    "attr:StatefulCanvas.iconHeight":{
        ref:"attr:StatefulCanvas.iconHeight",
        flags:"IR",
        description:"Height in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "method:Class.logIsEnabledFor":{
        ref:"method:Class.logIsEnabledFor",
        flags:"",
        description:" Check whether a message logged at the given priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsEnabledFor",
        seeAlso:[],
        params:[
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "group:eventHandling":{
        ref:"group:eventHandling",
        refs:["method:DynamicForm.focusInItem", "method:FormItem.focusInItem", 
         "method:FormItem.blurItem","method:FormItem.focus", "method:FormItem.blur", "method:FormItem.click", 
         "method:FormItem.doubleClick","method:FormItem.change", "method:FormItem.changed", "method:FormItem.keyPress", 
         "method:FormItem.keyDown","method:FormItem.keyUp", "method:ButtonItem.click"],
        name:"eventHandling",
        seeAlso:[],
        type:"group"
    },
    "group:tileLayout":{
        ref:"group:tileLayout",
        refs:["method:TileLayout.setTileSize", "method:TileLayout.setTileWidth", 
         "method:TileLayout.setTileHeight","method:TileLayout.setTileMargin", "method:TileLayout.setTileHMargin", 
         "method:TileLayout.setTileVMargin"],
        name:"tileLayout",
        seeAlso:[],
        type:"group"
    },
    "class:Tree":{
        methods:["method:Tree.isRoot", "method:Tree.linkNodes", "method:Tree.getRoot", 
         "method:Tree.setRoot","method:Tree.getName", "method:Tree.getTitle", "method:Tree.getPath", 
         "method:Tree.getParentPath","method:Tree.getParent", "method:Tree.getParents", "method:Tree.getLevel", 
         "method:Tree.isFolder","method:Tree.isLeaf", "method:Tree.findById", "method:Tree.find", 
         "method:Tree.getChildren","method:Tree.getFolders", "method:Tree.getLeaves", "method:Tree.hasChildren", 
         "method:Tree.hasFolders","method:Tree.hasLeaves", "method:Tree.isDescendantOf", "method:Tree.getDescendants", 
         "method:Tree.getDescendantFolders","method:Tree.getDescendantLeaves", "method:Tree.dataChanged", "method:Tree.add", 
         "method:Tree.addList","method:Tree.move", "method:Tree.remove", "method:Tree.removeList", 
         "method:Tree.loadChildren","method:Tree.unloadChildren", "method:Tree.reloadChildren", "method:Tree.isOpen", 
         "method:Tree.openFolder","method:Tree.openFolders", "method:Tree.closeFolder", "method:Tree.closeFolders", 
         "method:Tree.openAll","method:Tree.closeAll", "method:Tree.getOpenList", "method:Tree.getAllNodes", 
         "method:Tree.getLength","method:Tree.indexOf", "method:Tree.lastIndexOf"],
        ref:"class:Tree",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:["attr:Tree.modelType", "attr:Tree.isFolderProperty", "attr:Tree.defaultIsFolder", 
         "attr:Tree.reportCollisions","attr:Tree.pathDelim", "attr:Tree.nameProperty", "attr:Tree.titleProperty", 
         "attr:Tree.idField","attr:Tree.parentIdField", "attr:Tree.childrenProperty", "attr:Tree.openProperty", 
         "attr:Tree.data","attr:Tree.rootValue", "attr:Tree.root", "attr:Tree.discardParentlessNodes", 
         "attr:Tree.showRoot","attr:Tree.autoOpenRoot", "attr:Tree.defaultNodeTitle"],
        description:"A Tree is a data model representing a set of objects linked into a hierarchy.\n <P>\n A Tree has no visual presentation, it is displayed by a ${isc.DocUtils.linkForRef('class:TreeGrid')} or ${isc.DocUtils.linkForRef('class:ColumnTree')}\n when supplied as ${isc.DocUtils.linkForRef('attr:TreeGrid.data')} or ${isc.DocUtils.linkForRef('attr:ColumnTree.data')}.\n <P>\n A Tree can be constructed out of a List of objects interlinked by IDs or via explicitly\n specified Arrays of child objects.  See ${isc.DocUtils.linkForRef('attr:Tree.modelType')} for an explanation of how\n to pass data to a Tree.\n <P>\n Typical usage is to call ${isc.DocUtils.linkForRef('method:TreeGrid.fetchData')} to cause automatic creation of a \n ${isc.DocUtils.linkForRef('class:ResultTree')}, which is a type of Tree that automatically handles loading data on \n demand.  For information on DataBinding Trees, see ${isc.DocUtils.linkForRef('group:treeDataBinding')}.",
        name:"Tree",
        seeAlso:[],
        type:"class"
    },
    "method:Window.shouldDismissOnEscape":{
        ref:"method:Window.shouldDismissOnEscape",
        flags:"",
        description:" Should this window be dismissed (same effect as pressing the \"Cancel\" button) when the \n user presses the \"Escape\" key?<br>\n Default behavior: if ${isc.DocUtils.linkForRef('attr:Window.dismissOnEscape')} is set, just return it. Otherwise return\n true if this window is showing a \"close\" control in the header \n (see ${isc.DocUtils.linkForRef('attr:Window.headerControls')}).\n",
        name:"shouldDismissOnEscape",
        seeAlso:[],
        returns:{description:"true if the window should be dismissed when the user hits escape",
         type:"boolean"},
        type:"method",
        definingClass:"class:Window"
    },
    "method:SectionStack.getSectionHeader":{
        ref:"method:SectionStack.getSectionHeader",
        flags:"",
        description:" Return the SectionHeader for a section.\n <P>\n This will be an instance of the ${isc.DocUtils.linkForRef('attr:SectionStack.sectionHeaderClass')}.  Since different\n SectionStacks may use different header classes, be careful about what APIs you rely on\n for the section header unless you have explicitly set the\n <code>sectionHeaderClass</code>.  In particular, use APIs such as\n ${isc.DocUtils.linkForRef('method:SectionStack.setSectionTitle')} to manipulate headers indirectly wherever possible, as high\n performance SectionStacks designed for very large numbers of sections may cache and\n re-use headers or use other strategies that would make it invalid to store a pointer to\n a section header, assume the header is a layout member, etc.\n\n",
        name:"getSectionHeader",
        seeAlso:[],
        params:[
            {description:"ID or index of the section for which you want the header", optional:false,
             name:"section", defaultValue:null, type:"String or Number"}
        ],
        returns:{description:"the section header indicated", type:"SectionHeader"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "method:ListGrid.toggleFrozen":{
        ref:"method:ListGrid.toggleFrozen",
        flags:"",
        description:" Freeze or unfreeze the indicated freeze according to whether it is currently frozen.\n <P>\n Called when the ListGrid freezes or unfreezes fields by user action.\n\n",
        name:"toggleFrozen",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"field", defaultValue:null,
             type:"ListGridField or colNum or ListGridField.name"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["frozenFields"]
    },
    "method:ListGrid.getCheckboxField":{
        ref:"method:ListGrid.getCheckboxField",
        flags:"",
        description:" Returns the specially generated checkbox field used when ${isc.DocUtils.linkForRef('type:SelectionAppearance')} is\n \"checkbox\".\n <P>\n Called during ${isc.DocUtils.linkForRef('method:ListGrid.setFields')}, this method can be overridden to add advanced dynamic\n defaults to the checkbox field (call Super, modify the default field returned by Super,\n return the modified field).  Normal customization can be handled by just setting\n ${isc.DocUtils.linkForRef('type:AutoChild')} properties, as mentioned under the docs for ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxField')}.\n\n",
        name:"getCheckboxField",
        seeAlso:[],
        returns:{description:null, type:"ListGridField"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "method:Canvas.hideContextMenu":{
        ref:"method:Canvas.hideContextMenu",
        flags:"A",
        description:"\n The default implementation of this method hides the contextMenu currently being shown for this\n component (which occurs when the mouse button that toggles the context menu is released).\n Override if you want some other behavior.	\n\n",
        name:"hideContextMenu",
        seeAlso:[
            {ref:"method:Canvas.showContextMenu", description:null},
            {ref:"method:Menu.hideContextMenu", description:null},
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:Window.showHeaderIcon":{
        ref:"attr:Window.showHeaderIcon",
        flags:"IRW",
        description:"If true, we show an icon on the left in the header.",
        name:"showHeaderIcon",
        setter:["setShowHeaderIcon"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "method:Canvas.getInnerContentWidth":{
        ref:"method:Canvas.getInnerContentWidth",
        flags:"A",
        description:"  Returns the amount of space available for interior content (or relatively positioned child\n  widget(s)) without introducing clipping, scrolling or overflow.<br>\n  This is the space within the viewport of the widget (not including padding, and excluding \n  margins, borders or scrollbars) rendered at its specified size.\n\n",
        name:"getInnerContentWidth",
        seeAlso:[
            {ref:"method:Canvas.getInnerContentHeight", description:null},
            {ref:"method:Canvas.getInnerHeight", description:null},
            {ref:"method:Canvas.getInnerWidth", description:null}
        ],
        returns:{description:"inner height of the widget in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:ColumnTree.getData":{
        ref:"method:ColumnTree.getData",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('class:Tree')} object this ColumnTree is viewing\n\n",
        name:"getData",
        seeAlso:[],
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "classAttr:Dialog.DONE_BUTTON_TITLE":{
        ref:"classAttr:Dialog.DONE_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"Done\"</code> button.",
        name:"DONE_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"Done\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.expansionRelated":{
        ref:"attr:ListGrid.expansionRelated",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ListGrid')} for displaying data related to a record\n in it's expanded section when\n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>related</code>.  \n The ${isc.DocUtils.linkForRef('class:DataSource')} containing the related data is provided by \n ${isc.DocUtils.linkForRef('method:ListGrid.getRelatedDataSource','getRelatedDataSource()')} which, by default,\n returns the DataSource referred to in ${isc.DocUtils.linkForRef('attr:ListGridRecord.detailDS')}.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionRelatedProperties</code> and \n <code>listGrid.expansionRelatedDefaults</code>.\n <P>\n Note, however, that this is a multi-instance component (potentially one per record), \n so it is created using ${isc.DocUtils.linkForRef('method:Class.createAutoChild','createAutoChild()')} not \n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')}, and no default single instance is created by \n name on the grid.",
        name:"expansionRelated",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:Calendar.eventRemoveClick":{
        ref:"method:Calendar.eventRemoveClick",
        flags:"",
        description:" Called whenever the close icon of an event is clicked within the day or week view. Return\n false to cancel the removal, or true to allow it.\n <P>\n Implement this method to do something like, for example, showing a confirmation dialog \n before an event is removed.\n\n",
        name:"eventRemoveClick",
        seeAlso:[],
        params:[
            {description:"event that was clicked on", optional:false, name:"event", defaultValue:null,
             type:"CalendarEvent"},
            {description:"view where the event was clicked on: \"day\", \"week\" or \"month\"",
             optional:false, name:"viewName", defaultValue:null, type:"String"}
        ],
        returns:{description:"false to cancel the removal", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "method:Validator.condition":{
        ref:"method:Validator.condition",
        flags:"",
        description:" For a validator that is not one of the built-in ${isc.DocUtils.linkForRef('class:ValidatorTypes')}, a function or\n String expression to evaluate to see if this validator passes or fails.\n <p>\n Because the validator declaration itself is passed as a parameter to\n <code>condition()</code>, you can effectively parameterize the validator.  For example, to\n create a validator that checks that the value is after a certain date:<pre> \n     { type:\"custom\", afterDate:new Date(), \n       condition:\"value.getTime() > validator.afterDate.getTime()\" }\n </pre>\n Reusable validators, like the above, can be registered as a standard validatorType by\n calling ${isc.DocUtils.linkForRef('classMethod:Validator.addValidator')}.\n <P>\n Note that, if a field is declared with a builtin ${isc.DocUtils.linkForRef('type:FieldType')}, the value passed in\n will already have been converted to the specified type, if possible.\n\n",
        name:"condition",
        seeAlso:[],
        params:[
            {description:"FormItem or DataSourceField on which this                                           validator was declared.  NOTE: FormItem will not\n                                           be available during a save performed without a\n                                           form (eg programmatic save) or if the field \n                                           is not available in the form.",
             optional:false, name:"item", defaultValue:null, type:"DataSourceField or FormItem"},
            {description:"Validator declaration from eg                              ${isc.DocUtils.linkForRef('attr:DataSourceField.validators')}.",
             optional:false, name:"validator", defaultValue:null, type:"Validator"},
            {description:"value to validate", optional:false, name:"value", defaultValue:null,
             type:"any"}
        ],
        returns:{description:"whether the value passed validation.  True for passed, false for fail.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Validator"
    },
    "attr:MultiSortPanel.ascendingTitle":{
        ref:"attr:MultiSortPanel.ascendingTitle",
        flags:"IR",
        description:"The title-text to appear in the \"direction\" field's SelectItem for an \"ascending\" sort",
        name:"ascendingTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Ascending\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "attr:FormItem.showOverIcons":{
        ref:"attr:FormItem.showOverIcons",
        flags:"IRWA",
        description:"If we're showing icons, should we change their image source to the appropriate <i>over</i>\n  source when the user rolls over (or puts focus onto) them?  Can be overridden on a per\n  icon basis by the formItemIcon <code>showOver</code> property.",
        name:"showOverIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:ImgTab.showFocused":{
        ref:"attr:ImgTab.showFocused",
        flags:"IRW",
        description:"Should we visibly change state when the tab recieves keyboard focus?",
        name:"showFocused",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgTab"
    },
    "attr:Calendar.eventTypeField":{
        ref:"attr:Calendar.eventTypeField",
        flags:"IR",
        description:"The name of the field which will determine the row in which this event will be displayed\n for the ${isc.DocUtils.linkForRef('attr:Calendar.timelineView')}",
        name:"eventTypeField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"eventType\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "class:RowSpacerItem":{
        methods:[],
        ref:"class:RowSpacerItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:RowSpacerItem.showTitle", "attr:RowSpacerItem.colSpan", 
         "attr:RowSpacerItem.startRow","attr:RowSpacerItem.endRow"],
        description:"Form item that renders as a blank row in the form layout.<br>\n Set ${isc.DocUtils.linkForRef('attr:RowSpacerItem.startRow')} to <code>false</code> to create a rowSpacer that simply\n takes up every remaining column in the current row rather than starting a new row.",
        name:"RowSpacerItem",
        seeAlso:[],
        type:"class"
    },
    "class:ScrollThumb":{
        methods:[],
        ref:"class:ScrollThumb",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation/ScrollBar"],
        attrs:[],
        description:"Class used for the draggable \"thumb\" of a scrollbar.  Do not use directly; this class is\n documented only for skinning purposes.",
        name:"ScrollThumb",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.sorterDefaults":{
        ref:"attr:ListGrid.sorterDefaults",
        flags:"IRA",
        description:"Defaults to apply to the corner sort button. To modify this object, use\n ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','ListGrid.changeDefaults()')} rather than replacing with an\n entirely new object.",
        name:"sorterDefaults",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:FilterBuilder.subClauseButtonTitle":{
        ref:"attr:FilterBuilder.subClauseButtonTitle",
        flags:"IR",
        description:"The hover title text of the subClauseButton",
        name:"subClauseButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"+()\"",
        type:"attr",
        definingClass:"class:FilterBuilder",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.headerSpans":{
        ref:"attr:ListGrid.headerSpans",
        flags:"IR",
        description:"Header spans are a second level of headers that appear above the normal ListGrid headers,\n spanning one or more listGrid fields in a manner similar to a column-spanning cell in an \n HTML table.\n <P>\n A header span can be created by simply naming the fields the header should span.  The\n example below creates a headerSpan that spans the first two fields of the ListGrid.\n <pre>\n    isc.ListGrid.create({\n        headerHeight:40,\n        fields : [\n            { name:\"field1\" },\n            { name:\"field2\" },\n            { name:\"field3\" }\n        ],\n        headerSpans : [\n            { \n                fields: [\"field1\", \"field2\"],\n                title: \"Field 1 and 2\"\n            }\n        ]\n    });\n </pre>\n Header spans will automatically react to resizing of the headers they span, and will be\n hidden automatically when all of the spanned fields are hidden.\n <P>\n Header spans appear in the ${isc.DocUtils.linkForRef('attr:ListGrid.header','header')} area of the ListGrid, sharing space\n with the existing headers, so it's typical to set ${isc.DocUtils.linkForRef('attr:ListGrid.headerHeight')} to\n approximately double its normal height when using headerSpans.\n <P>\n See ${isc.DocUtils.linkForRef('object:HeaderSpan')} for many properties that allow the control of the appearance of\n headerSpans.  Note that headerSpans are created via the ${isc.DocUtils.linkForRef('type:AutoChild')} pattern, hence\n you can change the SmartClient component being used, or any of it's properties.\n <P>\n Neither headerSpans themselves nor the fields within them may be drag reordered, but other\n unspanned headers may be.\n <P>\n Note that headerSpans primarily provide a visual cue for grouping multiple headers \n together.  If you have an OLAP, data \"cube\" or multi-dimensional data model, the\n ${isc.DocUtils.linkForRef('class:CubeGrid')} component is the right choice.",
        name:"headerSpans",
        setter:["setHeaderSpans"],
        seeAlso:[],
        valueType:"Array of HeaderSpan",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "attr:Label.dynamicContents":{
        ref:"attr:Label.dynamicContents",
        flags:"IRWA",
        description:"Dynamic contents allows the contents string to be treated as a simple, but powerful\n template.  When this attribute is set to true, expressions of the form \\${arbitrary JS\n here} are replaced by the result of the evaluation of the JS code inside the curly\n brackets.  This evaluation happens at draw time.  If you want to trigger a re-evaluation\n of the expressions in the contents string you can call markForRedraw() on the canvas.\n <p>\n You can use this feature to build some simple custom components. For example, let's say\n you want to show the value of a Slider in a Canvas somewhere on the screen.  You can do\n this by observing the valueChanged() method on the slider and calling setContents() on\n your canvas with the new string or you can set the contents of the canvas to something\n like:\n <p><code>\n \"The slider value is \\${sliderInstance.getValue()}.\"\n </code><p>\n Next you set dynamicContents: true on the canvas, observe valueChanged() on the slider\n and call canvas.markForRedraw() in that observation.  This approach is cleaner than\n setContents() when the Canvas is aggregating several values or dynamic expressions.\n Like so:\n <p>\n <pre>\n Slider.create({\n     ID: \"mySlider\"\n });\n\n Canvas.create({\n     ID: \"myCanvas\",\n     dynamicContents: true,\n     contents: \"The slider value is \\${mySlider.getValue()}.\"\n });\n     \n myCanvas.observe(mySlider, \"valueChanged\", \n                  \"observer.markForRedraw()\");\n </pre>\n You can embed an arbitrary number of dynamic expressions in the contents string.  The\n search and replace is optimized for speed.\n <p>\n If an error occurs during the evaluation of one of the expressions, a warning is logged\n to the ISC Developer Console and the error string is embedded in place of the expected\n value in the Canvas.\n <p>\n The value of a function is its return value.  The value of any variable is the same as\n that returned by its toString() representation.\n <p>\n Inside the evalution contentext, <code>this</code> points to the canvas instance that\n has the dynamicContents string as its contents - in other words the canvas instance on\n which the template is declared.",
        name:"dynamicContents",
        seeAlso:[
            {ref:"attr:Label.contents", description:null},
            {ref:"attr:Canvas.dynamicContentsVars", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dynamicContents')}"],
        definingClass:"class:Label",
        groups:["contents"]
    },
    "method:ValuesManager.getValue":{
        ref:"method:ValuesManager.getValue",
        flags:"",
        description:" Returns the value for some field.\n",
        name:"getValue",
        seeAlso:[],
        params:[
            {description:"Which value to be returned", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"current value of the appropriate field", type:"any"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "classAttr:Validator.notADecimal":{
        ref:"classAttr:Validator.notADecimal",
        flags:"IRA",
        description:"Default error message to display when standard <code>isFloat</code> type validator\n  returns false.",
        name:"notADecimal",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be a valid decimal.\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:Calendar.eventOverlapPercent":{
        ref:"attr:Calendar.eventOverlapPercent",
        flags:"IR",
        description:"The size of the overlap, presented as a percentage of the width of events sharing timeslots",
        name:"eventOverlapPercent",
        seeAlso:[],
        valueType:"number",
        defaultValue:"10",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "method:MultiView.fetchRelatedData":{
        ref:"method:MultiView.fetchRelatedData",
        flags:"",
        description:" Based on the relationship between the DataSource this component is bound to and the\n DataSource specified as the \"schema\" argument, call fetchData() to retrieve records in this\n grid that are related to the passed-in record.\n <P>\n Relationships between DataSources are declared via ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')}.\n <P>\n For example, given a DataSource \"orders\" and another DataSource \"orderItems\", where\n \"orderItems\" declared a field \"orderId\" pointing to the primary key field of the \"orders\"\n DataSource, there is a set of records from the \"orderItems\" DataSource related to any given\n record from the \"order\" DataSource.  If this component were bound to \"orderItems\" and a\n record from the \"orders\".\n\n",
        name:"fetchRelatedData",
        seeAlso:[],
        params:[
            {description:"DataSource record", optional:false, name:"record", defaultValue:null,
             type:"ListGridRecord"},
            {description:"schema of the DataSource record, or                            DataBoundComponent already bound to that schema",
             optional:false, name:"schema", defaultValue:null, type:"Canvas or DataSource or ID"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "classMethod:Log.logError":{
        ref:"classMethod:Log.logError",
        flags:"",
        description:" Log a message at \"error\" priority\n \n",
        name:"logError",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:Menu.menuButtonWidth":{
        ref:"attr:Menu.menuButtonWidth",
        flags:"IR",
        description:"For a menu that has a ${isc.DocUtils.linkForRef('class:MenuButton')} generated for it automatically (for example when\n included in a ${isc.DocUtils.linkForRef('class:MenuBar')}, the width that the MenuButton should have.  If unset, the\n MenuButton will be as wide as <code>menu.width</code>.",
        name:"menuButtonWidth",
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Menu"
    },
    "class:ResultTree":{
        methods:["method:ResultTree.loadChildren", "method:ResultTree.unloadChildren", 
         "method:ResultTree.invalidateCache","method:ResultTree.dataArrived"],
        ref:"class:ResultTree",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:ResultTree.dataSource", "attr:ResultTree.loadDataOnDemand", 
         "attr:ResultTree.defaultIsFolder","attr:ResultTree.rootNode", "attr:ResultTree.discardParentlessNodes", 
         "attr:ResultTree.defaultNewNodesToRoot","attr:ResultTree.updateCacheFromRequest", "attr:ResultTree.disableCacheSync"],
        description:"ResultTrees are an implementation of the ${isc.DocUtils.linkForRef('class:Tree')} API, used to handle hierarchical\n data, whose nodes are DataSource records which are retrieved from a server.",
        name:"ResultTree",
        seeAlso:[],
        type:"class"
    },
    "attr:PickList.animatePickList":{
        ref:"attr:PickList.animatePickList",
        flags:"IRWA",
        description:"If true, when the pickList is shown, it will be shown via an animated reveal effect",
        name:"animatePickList",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:Window.minimizeButton":{
        ref:"attr:Window.minimizeButton",
        flags:"R",
        description:"ImgButton shown in the header that will minimize this Window by calling ${isc.DocUtils.linkForRef('method:Window.minimize')}.",
        name:"minimizeButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "method:EBay.setAuthToken":{
        ref:"method:EBay.setAuthToken",
        flags:"",
        description:"\n eBay web services require an authentication token to be passed with every request.  You\n can obtain a token from <a href='http://developer.ebay.com/' onclick=\"window.open('http://developer.ebay.com/');return false;\">http://developer.ebay.com/</a>.  Before using any\n eBay web services, call this method to set the token provided to you by eBay.\n \n",
        name:"setAuthToken",
        seeAlso:[],
        params:[
            {description:"Your authToken, provided to you by eBay.", optional:false, name:"authToken",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:EBay"
    },
    "method:FilterBuilder.addCriterion":{
        ref:"method:FilterBuilder.addCriterion",
        flags:"",
        description:" Add a new criterion, including recursively adding sub-criteria for a criterion that\n contains other criteria.\n \n",
        name:"addCriterion",
        seeAlso:[],
        params:[
            {description:"new criterion to be added", optional:false, name:"criterion",
             defaultValue:null, type:"Criterion"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "type:CSSStyleName":{
        ref:"type:CSSStyleName",
        description:"CSS class name to apply to some HTML element on this page. This is a string that should\n match the css class defined for the page in an external stylesheet or in inline \n html &lt;STYLE&gt; tags.\n <P>\n As a general rule, wherever it is possible to provide a CSS styleName (such as\n ${isc.DocUtils.linkForRef('attr:Canvas.styleName')} or ${isc.DocUtils.linkForRef('attr:Button.baseStyle')}, your CSS style can specify border,\n margins, padding, and any CSS attributes controlling background or text styling.  You\n should not specify any CSS properties related to positioning, clipping, sizing or\n visibility (such as \"overflow\", \"position\", \"display\", \"visibility\" and \"float\"), using\n the SmartClient APIs for this kind of control.\n <P>\n Because text wrapping cannot be consistently controlled cross-browser from CSS alone,\n you should use SmartClient properties such as ${isc.DocUtils.linkForRef('attr:Button.wrap')} instead of the\n corresponding CSS properties, when provided.\n <P>\n Content contained within SmartClient components can use arbitrary CSS, with the \n caveat that the content should be tested on all supported browsers, just as content\n outside of SmartClient must be.",
        name:"CSSStyleName",
        seeAlso:[],
        type:"type",
        examples:["${isc.DocUtils.linkForExampleId('consistentSizing')}"],
        groups:["appearance"]
    },
    "method:FormItem.isDrawn":{
        ref:"method:FormItem.isDrawn",
        flags:"",
        description:" Returns true if this item has been written out into the DOM.\n\n",
        name:"isDrawn",
        seeAlso:[],
        returns:{description:"whether this item is drawn", type:"boolean"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["drawing"]
    },
    "attr:OperationBinding.groupClause":{
        ref:"attr:OperationBinding.groupClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n GROUP BY clause to use when constructing the SQL query to perform this operation.  The\n property should be a comma-separated list of column names and/or expressions, forming\n a valid GROUP BY clause in the syntax of the underlying database.  The server will \n insert the text of this property immediately after the \"GROUP BY\" token.\n <p>\n Note that specifying this property enables you to use aggregate functions (such as \n COUNT and SUM) in your ${isc.DocUtils.linkForRef('attr:OperationBinding.selectClause','selectClause')}. Also note that care is required \n when using groupClause to ensure that the selectClause contains the fields you are grouping\n by.  Failure to do this correctly will result in a runtime SQL error.\n <p>\n This property is only applicable to operationBindings of \n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"fetch\".\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"groupClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null},
            {ref:"attr:OperationBinding.selectClause", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "classMethod:Class.setInstanceProperty":{
        ref:"classMethod:Class.setInstanceProperty",
        flags:"",
        description:"\n	Sets a named property from the instance defaults for this object.\n\n",
        name:"setInstanceProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to return", optional:false, name:"property",
             defaultValue:null, type:"string"},
            {description:"value to set to", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:Toolbar.buttonConstructor":{
        ref:"attr:Toolbar.buttonConstructor",
        flags:"IRWA",
        description:"Default constructor for toolbar items.",
        name:"buttonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"Button",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["appearance"]
    },
    "attr:CubeGrid.cellIdProperty":{
        ref:"attr:CubeGrid.cellIdProperty",
        flags:"IR",
        description:"Name of the property in a cell record that holds it's unique ID.  Note cell record IDs are\n optional.",
        name:"cellIdProperty",
        seeAlso:[],
        valueType:"id",
        defaultValue:"\"ID\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:OperationBinding.dataProtocol":{
        ref:"attr:OperationBinding.dataProtocol",
        flags:"IR",
        description:"Controls the format in which inputs are sent to the dataURL.\n <p>\n When a DataSource operation such as fetchData() is invoked on this DataSource or a component\n bound to this DataSource, the data passed to the operation, if any, will be sent to the\n <code>dataURL</code>.  The <code>dataProtocol</code> property controls the format in\n which the data is sent: SOAP message, HTTP GET or POST of parameters, etc.\n <P>\n The <code>dataProtocol</code> property need not be set for a DataSource with a WebService\n ( ${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')} is set), in this case, SOAP messaging is used by\n default.\n <P>\n Developers may completely bypass the SmartClient comm system by setting dataProtocol to\n <code>\"clientCustom\"</code>. In this case SmartClient will not attempt to send any data\n to the server after calling ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}. Instead the developer is\n expected to implement <code>transformRequest()</code> such that it performs the necessary\n data action outside of SmartClient, and then calls ${isc.DocUtils.linkForRef('method:DataSource.processResponse')}, \n passing in the ${isc.DocUtils.linkForRef('attr:DSRequest.requestId')} and an appropriate set of DSResponse properties to\n indicate the result of the action.\n <P>\n NOTE: when ${isc.DocUtils.linkForRef('attr:OperationBinding.dataFormat')} is \"iscServer\", <code>dataProtocol</code> is not consulted.\n Instead, SmartClient uses a proprietary wire format to communicate with the SmartClient\n server, and the server-side DSRequest and DSResponse objects should be used to access\n request data and form responses.",
        name:"dataProtocol",
        seeAlso:[
            {ref:"type:DSProtocol", description:null}
        ],
        valueType:"DSProtocol",
        defaultValue:"\"getParams\"",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "classAttr:GroupingMessages.upcomingLaterTitle":{
        ref:"classAttr:GroupingMessages.upcomingLaterTitle",
        flags:"IRW",
        description:"When a ${isc.DocUtils.linkForRef('class:ListGrid')} is grouped by a date field in 'Upcoming' mode, \n this is the group title for all records in which the grouped date field occurs later than\n one month after today's date.",
        name:"upcomingLaterTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Later\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:Array.containsProperty":{
        ref:"method:Array.containsProperty",
        flags:"",
        description:"  Determine whether this array contains any members where the property passed in matches the value\n  passed in.\n\n",
        name:"containsProperty",
        seeAlso:[],
        params:[
            {description:"property to look for							(object)	key:value pairs to look for",
             optional:false, name:"property", defaultValue:null, type:"string"},
            {description:"value to compare against (if property is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"true if this array contains an object with the appropriate property value",
         type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["find"]
    },
    "attr:Hilite.htmlValue":{
        ref:"attr:Hilite.htmlValue",
        flags:"IR",
        description:"Value to show <b>in place of</b> the actual value from the record, for a record that matches\n this hilite.\n <P>\n This can be used to take ranges of numeric values and simplify them to \"Low\", \"Medium\",\n \"High\" or similar textual values, translate very small or very large values to \"Outlier\" or\n \"Negligible\", and similar use cases.",
        name:"htmlValue",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Hilite"
    },
    "attr:OperationBinding.canSyncCache":{
        ref:"attr:OperationBinding.canSyncCache",
        flags:"IR",
        description:"For an operationBinding of ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"add\" or \"update\", this \n property indicates to the server that cache synchronization will work after this operation\n has performed its updates, without the need for an explicit \n ${isc.DocUtils.linkForRef('attr:OperationBinding.cacheSyncOperation','cacheSyncOperation')}.  This property tells the server that the \n default cache sync operation (either a custom one declared with no operationId, as described\n in the docs for ${isc.DocUtils.linkForRef('attr:OperationBinding.useForCacheSync')}, or the default one that SmartClient generates) will \n return the correct record to the client, so the client's cache does not need to be \n invalidated.\n <p>\n This property is only applicable to DataSources of type \"sql\".",
        name:"canSyncCache",
        seeAlso:[
            {ref:"attr:OperationBinding.useForCacheSync", description:null},
            {ref:"attr:OperationBinding.cacheSyncOperation", description:null}
        ],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:Window.hiliteBodyColor":{
        ref:"attr:Window.hiliteBodyColor",
        flags:"IRW",
        description:"Highlight color for the Window body (shown when the body is flashed).",
        name:"hiliteBodyColor",
        seeAlso:[
            {ref:"method:Window.flash", description:null}
        ],
        valueType:"string",
        defaultValue:"\"#EEEEEE\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "attr:FormItem.cellClassName":{
        ref:"attr:FormItem.cellClassName",
        deprecated:"As of SmartClient version 5.5, deprecated in favor of ${isc.DocUtils.linkForRef('attr:FormItem.cellStyle')}",
        flags:"IRW",
        description:"CSS class for a form item's cell in the form layout",
        name:"cellClassName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"formCell\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:ListGrid.canEditCell":{
        ref:"method:ListGrid.canEditCell",
        flags:"A",
        description:" Can this cell be edited?\n <P>\n The default implementation of <code>canEditCell()</code> respects the various property\n settings affecting editability: ${isc.DocUtils.linkForRef('attr:ListGridField.canEdit','field.canEdit')} disables editing\n for a field, a record with the ${isc.DocUtils.linkForRef('attr:ListGrid.recordEditProperty','recordEditProperty')} set to\n false is not editable, and disabled records are not editable.\n <P>\n You can override this method to control editability on a cell-by-cell basis.  For example,\n if you had a grid that allows editing of \"orders\", and you had a field \"shipDate\" that is\n normally editable, but should not be editable if the order is already \"complete\", you might\n implement <code>canEditCell()</code> as follows:\n <P>\n <pre>\n   isc.ListGrid.create({\n       ...\n       canEditCell : function (rowNum, colNum) {\n           var record = this.getRecord(rowNum),\n               fieldName = this.getFieldName(colNum);\n           if (fieldName == \"shipDate\" && \n               record.orderStatus == \"complete\") \n           {\n               return false;   \n           }\n           // use default rules for all other fields\n           return this.Super(\"canEditCell\", arguments);\n       }\n   });\n </pre>\n <P>\n Notes on providing custom implementations:\n <ul>\n <li> In order to allow complete control over editing, <code>canEditCell()</code> is called\n very frequently.  If you see delays on row to row navigation, check that your implementation\n is efficient\n <li> If you change the editability of a cell on the fly, for example, during \n ${isc.DocUtils.linkForRef('method:ListGrid.editorExit')} on another cell, call refreshCell() to show or hide the editor\n <li> If this ListGrid allows new records to be created, <code>canEditCell()</code> may be\n called when there is no record available, in which case getRecord() will return null.  The\n values input so far by the user are available via ${isc.DocUtils.linkForRef('method:ListGrid.getEditValues')}. \n </ul>\n\n",
        name:"canEditCell",
        seeAlso:[],
        params:[
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"Whether to allow editing this cell", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:ListGridField.groupingMode":{
        ref:"attr:ListGridField.groupingMode",
        flags:"IR",
        description:"For a field that allows multiple ${isc.DocUtils.linkForRef('attr:ListGridField.groupingModes','grouping modes')}, \n the current grouping mode.\n <P>\n This property is set when a user chooses a particular grouping mode, and may be set on\n ListGrid creation to affect the initial grouping.",
        name:"groupingMode",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "method:DataBoundComponent.deselectRecord":{
        ref:"method:DataBoundComponent.deselectRecord",
        flags:"",
        description:"\n Deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecord(record, false)</code>\n\n",
        name:"deselectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to deselect", optional:false, name:"record",
             defaultValue:null, type:"Record | number"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["selection"]
    },
    "attr:FormItem.hoverAlign":{
        ref:"attr:FormItem.hoverAlign",
        flags:"IRW",
        description:"Text alignment  for text displayed in this item's hover canvas, if shown.",
        name:"hoverAlign",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverAlign", description:null}
        ],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "class:Img":{
        methods:["method:Img.setImageType", "method:Img.setSrc", "method:Img.resetSrc"],
        ref:"class:Img",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:Img.name", "attr:Img.src", "attr:Img.activeAreaHTML", "attr:Img.imageType", 
         "attr:Img.imageHeight","attr:Img.imageWidth", "attr:Img.size", "attr:Img.showTitle"],
        description:"The Img widget class implements a simple widget that displays a single image.",
        name:"Img",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('img')}"]
    },
    "attr:ListGrid.arrowKeyAction":{
        ref:"attr:ListGrid.arrowKeyAction",
        flags:"IRWA",
        description:"Action to perform when the listGrid has keyboard focus (but not editing focus) and a user\n presses the up or down arrow key. Possible values are:\n <ul>\n <li><code>select</code> : select the next row in the list (calls <code>recordClick</code> handler)</li>\n <li><code>focus</code> : move focus to the next row in the list without changing the selection</li>\n <li><code>activate</code> : select and activate the next row in the list (calls\n  <code>recordDoubleClick</code> handler)</li>\n <li><code>none</code> : no action</li>\n </ul>",
        name:"arrowKeyAction",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"select\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:ListGrid.freezeOnRightText":{
        ref:"attr:ListGrid.freezeOnRightText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','this.canFreezeFields')} is true, this string will be shown as the\n title for the menu item to freeze fields on the right of the scrollable body.",
        name:"freezeOnRightText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Freeze on right\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "method:HTMLFlow.transformHTML":{
        ref:"method:HTMLFlow.transformHTML",
        flags:"",
        description:" Override to modify the loaded HTML before it is rendered.\n\n",
        name:"transformHTML",
        seeAlso:[],
        params:[
            {description:"the html as loaded from the server return (HTML) html to be rendered",
             optional:false, name:"html", defaultValue:null, type:"HTML"}
        ],
        type:"method",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:ColumnTree.loadDataOnDemand":{
        ref:"attr:ColumnTree.loadDataOnDemand",
        flags:"IR",
        description:"For databound columnTree instances, should the entire tree of data be loaded on initial \n fetch, or should each coluimn be loaded as needed. If unset the default \n ResultTree.loadDataOnDemand setting will be used.",
        name:"loadDataOnDemand",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["databinding"]
    },
    "method:Calendar.eventChanged":{
        ref:"method:Calendar.eventChanged",
        flags:"",
        description:" Notification fired whenever a user changes an event, whether by dragging the event or by\n editing it in a dialog.\n <P>\n In a calendar with a DataSource, eventChanged() fires <b>after</b> the updated event has\n been successfully saved to the server\n\n",
        name:"eventChanged",
        seeAlso:[],
        params:[
            {description:"the event that changed", optional:false, name:"event", defaultValue:null,
             type:"CalendarEvent"}
        ],
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "method:DynamicForm.getValidatedValues":{
        ref:"method:DynamicForm.getValidatedValues",
        flags:"",
        description:" Call ${isc.DocUtils.linkForRef('method:DynamicForm.validate')} to check for validation errors. If no errors are found,\n return the current values for this form, otherwise return null.\n",
        name:"getValidatedValues",
        seeAlso:[],
        returns:{description:"current values or null if validation failed.", type:"object|null"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "attr:ListGridRecord.isGroupSummary":{
        ref:"attr:ListGridRecord.isGroupSummary",
        flags:"IRW",
        description:"This attribute will automatically be set to true for records representing  \n group-level summary rows shown if ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true.\n <P>\n Note that <code>isGroupSummary</code> is the default property name for this attribute but\n it may be modified by setting ${isc.DocUtils.linkForRef('attr:ListGrid.groupSummaryRecordProperty')}",
        name:"isGroupSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "object:ImgProperties":{
        methods:[],
        ref:"object:ImgProperties",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation/Canvas"],
        attrs:["attr:ImgProperties.src", "attr:ImgProperties.width", "attr:ImgProperties.height", 
         "attr:ImgProperties.name","attr:ImgProperties.extraStuff", "attr:ImgProperties.imgDir"],
        description:"A set of properties that can be used to create an image.",
        name:"ImgProperties",
        seeAlso:[],
        type:"object"
    },
    "classAttr:LoginDialog.LOGIN_ERROR_MESSAGE":{
        ref:"classAttr:LoginDialog.LOGIN_ERROR_MESSAGE",
        flags:"IRW",
        description:"Default error message displayed on failed login in the dialog shown by \n ${isc.DocUtils.linkForRef('classMethod:isc.showLoginDialog')}.",
        name:"LOGIN_ERROR_MESSAGE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Invalid username or password\"",
        type:"classAttr",
        definingClass:"class:LoginDialog",
        groups:["i18nMessages"]
    },
    "attr:FormItem.validateOnChange":{
        ref:"attr:FormItem.validateOnChange",
        flags:"IRW",
        description:"If true, form items will be validated when each item's \"change\" handler is fired\n as well as when the entire form is submitted or validated.\n <p>\n Note that this property can also be set at the form, level;\n If true at either level the valiator will be fired on change - displaying errors and\n rejecting the change on validation failure.",
        name:"validateOnChange",
        seeAlso:[
            {ref:"attr:DynamicForm.validateOnChange", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["changeHandling"]
    },
    "attr:Canvas.valuesManager":{
        ref:"attr:Canvas.valuesManager",
        flags:"IRWA",
        description:"${isc.DocUtils.linkForRef('class:ValuesManager')} for managing values displayed in this component.\n If specified at initialization time, this component will be added to the valuesManager via\n ${isc.DocUtils.linkForRef('method:ValuesManager.addMember')}.\n <P>\n ValuesManagers allow different fields of a single object to be displayed or edited\n across multiple UI components. Given a single values object, a valuesManager will handle\n determining the appropriate field values for its member components and displaying them / \n responding to edits if the components support this.\n <P>\n Data may be derived simply from the specified fieldNames within the member components, or for\n complex nested data structures can be specified by both component and field-level\n ${isc.DocUtils.linkForRef('attr:Canvas.dataPath')}.\n <P>\n Note that components may be automatically bound to an existing valuesManager attached to a \n parent component if dataPath is specified. See ${isc.DocUtils.linkForRef('attr:Canvas.dataPath')} for more information.\n Also note that if a databound component has a specified dataSource and dataPath but no specified\n valuesManager object one will be automatically generated as part of the databinding process",
        name:"valuesManager",
        seeAlso:[],
        valueType:"ValuesManager",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "attr:ResultTree.defaultIsFolder":{
        ref:"attr:ResultTree.defaultIsFolder",
        flags:"IR",
        description:"Controls whether nodes are assumed to be folders or leaves by default.\n <P>\n Nodes that have children or have the ${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty','isFolderProperty')} set\n to true will always be considered folders.  Other nodes will be considered folders or\n leaves by default according to this setting.\n <P>\n If <code>defaultIsFolder</code> is unset, the ResultTree will automatically set it to\n match the value of ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand')}.  This means that, when using\n folder-by-folder load on demand (<code>loadDataOnDemand:true</code>), by default a newly\n loaded node will be considered to be a folder that has not loaded its children yet.  \n <P>\n When not using folder-by-folder load on demand, by default a newly loaded node is\n considered a leaf.  If you set <code>defaultIsFolder:true</code> explicitly, by default\n a newly loaded node is considered to be a folder with no children.\n <P> \n See ${isc.DocUtils.linkForRef('method:Tree.isFolder')} for details on how to explicitly mark nodes as folders or leaves.",
        name:"defaultIsFolder",
        seeAlso:[
            {ref:"attr:TreeGrid.loadDataOnDemand", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "classAttr:GroupingMessages.byWeekTitle":{
        ref:"classAttr:GroupingMessages.byWeekTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by week.",
        name:"byWeekTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Week\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:SectionItem.itemIds":{
        ref:"attr:SectionItem.itemIds",
        flags:"IR",
        description:"IDs of the items that should be considered a member of this section.",
        name:"itemIds",
        seeAlso:[],
        valueType:"Array of ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SectionItem"
    },
    "attr:GridRenderer.showAllColumns":{
        ref:"attr:GridRenderer.showAllColumns",
        flags:"IRA",
        description:"Whether all columns should be drawn all at once, or only columns visible in the viewport.<br><br>\n\n Drawing all columns causes longer initial rendering time, but allows smoother horizontal\n scrolling.  With a very large number of columns, showAllColumns will become too slow.",
        name:"showAllColumns",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "group:treeDataBinding":{
        ref:"group:treeDataBinding",
        title:"Tree DataBinding",
        treeLocation:["Client Reference/Data Binding"],
        refs:[],
        description:"The SmartClient ${isc.DocUtils.linkForRef('class:TreeGrid')} component is a visual representation of a tree and requires\n a ${isc.DocUtils.linkForRef('class:Tree')} or ${isc.DocUtils.linkForRef('class:ResultTree')} datatype passed via the ${isc.DocUtils.linkForRef('attr:TreeGrid.data')} attribute to\n initialize the tree view.  The ${isc.DocUtils.linkForRef('class:Tree')} datatype is used when you want to provide all of\n the tree nodes in one shot at initialization time.  The ${isc.DocUtils.linkForRef('class:ResultTree')} datatype is used\n when you want portions of the tree to be loaded on demand from the server.\n <p>\n <b><u>Providing all data to the Tree at creation</u></b>\n <p>\n The simplest mechanism by which to initialize a Tree is to simply provide all the data\n up-front when the Tree itself is created.  Depending on the format of your tree data, this\n can be done by setting ${isc.DocUtils.linkForRef('attr:Tree.root')} or ${isc.DocUtils.linkForRef('attr:Tree.data')}.  This functionality is provided\n by the ${isc.DocUtils.linkForRef('class:Tree')} class.\n <p>\n For examples of this type of databinding, see the following SDK examples:\n <ul>\n <li><a href='/examples/components/treeGrid_init.html' onclick=\"window.open('/examples/components/treeGrid_init.html');return false;\">TreeGrid Initialization Example</a>\n <li><a href='/examples/server_integration/#jstlTree' onclick=\"window.open('/examples/server_integration/#jstlTree');return false;\">TreeGrid Initialization with JSTL</a>\n </ul>\n <p>\n <b><u>Loading Tree nodes on demand</u></b>\n <p>\n In this mode, tree nodes are loaded on-demand the first time a user expands a folder.  This\n approach is necessary for large trees.  This functionality is provided by the\n ${isc.DocUtils.linkForRef('class:ResultTree')} class.\n <p>\n ResultTrees require that every node in the tree have an ${isc.DocUtils.linkForRef('attr:Tree.idField','id')} that is\n unique tree-wide.  When the user expands a folder whose contents have not yet been loaded\n from the server (or you programmatically call openFolder() on such a node), the client\n automatically sends a ${isc.DocUtils.linkForRef('class:DSRequest')} to the server to ask for all immediate children of\n that node.  The ${isc.DocUtils.linkForRef('class:DSRequest')} criteria will specify the id of the node for which children\n are being requested via the <code>parentId</code> property (see ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')}).\n This client is asking the server: \"give me all nodes whose parentId is the id of this\n node\".\n <P>\n If there are no pre-existing node ids in the dataset you are loading, you must generate node\n ids (because the client needs some way to identify nodes when talking to the server).\n Generally a node id should contain whatever information is required to fetch the node and\n it's children on the server.  One typical approach is to use the path to the node as a node\n id.  For XML datasets in particular, the path may be a valid XPath, so that server-side\n lookup of child nodes is just a matter of applying the node id as an XPath to a server-side\n XMLDocument.\n <P>\n ${isc.DocUtils.linkForRef('class:ResultTree')}s are created for you by the ${isc.DocUtils.linkForRef('class:TreeGrid')} when you set\n ${isc.DocUtils.linkForRef('attr:TreeGrid.dataSource')}, but you can pass an initial dataset to a databound TreeGrid by\n setting ${isc.DocUtils.linkForRef('attr:TreeGrid.initialData')}.  The ${isc.DocUtils.linkForRef('attr:Tree.idField')} is derived from the dataSource\n you provide to the TreeGrid - the first field marked as\n ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey')}:true becomes the idField of the ResultTree.\n The ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')} is found by looking for a field that has a\n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')} property pointing to the idField.\n <P>\n If you do not provide ${isc.DocUtils.linkForRef('attr:TreeGrid.initialData')}, the first DSRequest you receive will be a\n request for the nodes under root.  The id of the root node of the tree is the value of the\n <code>rootValue</code> attribute on the ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')} of the Tree DataSource. \n <p>\n For examples of this type of databinding, see the following SDK examples:\n <ul>\n <li><a href='/examples/databinding/tree_databinding.jsp' onclick=\"window.open('/examples/databinding/tree_databinding.jsp');return false;\">TreeGrid DataBinding Example</a>\n <li><a href='/examples/server_integration/#xml2JSLOD' onclick=\"window.open('/examples/server_integration/#xml2JSLOD');return false;\">TreeGrid XML DataBinding</a>\n </ul>\n <P>\n <b><u>Multi-Level load on demand</u></b>\n <P>\n The ResultTree's DSRequests ask for the immediate children of a node only (by specifying\n <code>parentId</code> in the criteria). Any nodes returned whose <code>parentId</code> field\n value is unset or matches this criterion will be added to the tree as immediate children of the\n node. However you are also free to return multiple levels of children.  This can be done by\n simply returning a flat list of descendents with valid id's and parentId's, exactly as though \n you were initializing a multi-level tree via ${isc.DocUtils.linkForRef('attr:Tree.data')}.  \n <P>\n Note that when receiving multiple levels of children, the ResultTree's assumption is that\n if any children are loaded for a parent, then that parent is considered fully loaded.\n <P>\n When loading children for a given parent node, the ResultTree calls\n ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} on its DataSource.  For custom code that may need to reference\n the parentNode or tree in some way, the parent node whose children are being loaded is\n available on the dsRequest instance in the DataSource flow as dsRequest.parentNode, where it\n can be inspected during ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')}.",
        name:"treeDataBinding",
        seeAlso:[],
        type:"group"
    },
    "attr:DataSource.autoDeriveTitles":{
        ref:"attr:DataSource.autoDeriveTitles",
        flags:"IR",
        description:"If set, titles are automatically derived from ${isc.DocUtils.linkForRef('attr:DataSourceField.name','field.name')} for any \n field that does not have a ${isc.DocUtils.linkForRef('attr:DataSourceField.title','field.title')} and is not marked\n ${isc.DocUtils.linkForRef('attr:DataSourceField.hidden','hidden')}:true, by calling the method ${isc.DocUtils.linkForRef('method:DataSource.getAutoTitle')}.",
        name:"autoDeriveTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:Canvas.setBottom":{
        ref:"method:Canvas.setBottom",
        flags:"",
        description:" Resizes the widget vertically to position its bottom edge at the specified coordinate.\n <P>\n NOTE: if you're setting multiple coordinates, use setRect(), moveTo() or resizeTo()\n instead\n\n",
        name:"setBottom",
        seeAlso:[],
        params:[
            {description:"new bottom coordinate", optional:false, name:"bottom", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:DateChooser.baseNavButtonStyle":{
        ref:"attr:DateChooser.baseNavButtonStyle",
        flags:"IRW",
        description:"CSS style to apply to navigation buttons and date display at the top of the\n component. If null, the CSS style specified in ${isc.DocUtils.linkForRef('attr:DateChooser.baseButtonStyle')} is used.",
        name:"baseNavButtonStyle",
        seeAlso:[],
        valueType:"CSSClassName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:ValuesManager.addMember":{
        ref:"method:ValuesManager.addMember",
        flags:"",
        description:"\n Add a new member form to this valuesManager.\n This form's values will subsequently be available through this valuesManager.  <br>\n Note on pre-existant values:\n If the valuesManager has a value specified for some field, for which the member form has\n an item, this value will be applied to the member form.  This is true whether the item\n has a value or not.<br>\n However if the member form has a value for some field, and the ValuesManager does not\n have a specified value for the same field, we allow the valuesManager to pick up the \n value from the member form.    \n\n",
        name:"addMember",
        seeAlso:[
            {ref:"method:ValuesManager.addMembers", description:null}
        ],
        params:[
            {description:"form (or ID of form) to add to                                           this valuesManager as a member.",
             optional:false, name:"member", defaultValue:null, type:"DynamicForm | String"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "attr:Button.state":{
        flags:"IRWA",
        getter:["getState"],
        type:"attr",
        ref:"attr:Button.state",
        description:"Current \"state\" of this widget. StatefulCanvases will have a different appearance based\n on their current state. By default this is handled by changing the css className applied to\n the StatefulCanvas - see ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')} for a description of how this is\n done.<P>\n For ${isc.DocUtils.linkForRef('class:Img')} or ${isc.DocUtils.linkForRef('class:StretchImg')} based subclasses of StatefulCanvas, the \n appearance may also be updated by changing the src of the rendered image. See\n ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:StretchImgButton.src')} for a description of how the URL \n is modified to reflect the state of the widget in this case.",
        name:"state",
        setter:["setState"],
        valueType:"State",
        seeAlso:[
            {ref:"type:State", description:null},
            {ref:"group:state", description:null}
        ],
        defaultValue:"\"\"",
        groups:["state"],
        definingClass:"class:Button"
    },
    "method:TreeGrid.setNodeIcon":{
        ref:"method:TreeGrid.setNodeIcon",
        flags:"",
        description:" Set the icon for a particular treenode to a specified URL\n\n",
        name:"setNodeIcon",
        seeAlso:[],
        params:[
            {description:"tree node", optional:false, name:"node", defaultValue:null, type:"TreeNode"},
            {description:"path to the resource", optional:false, name:"icon", defaultValue:null,
             type:"SCImgUrl"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:ListGrid.emptyMessage":{
        ref:"attr:ListGrid.emptyMessage",
        flags:"IRW",
        description:"The string to display in the body of a listGrid with an empty data array, if\n showEmptyMessage is true.",
        name:"emptyMessage",
        seeAlso:[
            {ref:"attr:ListGrid.showEmptyMessage", description:null},
            {ref:"attr:ListGrid.emptyMessageStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"\"No items to show.\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('emptyGrid')}"],
        definingClass:"class:ListGrid",
        groups:["emptyMessage", "i18nMessages"]
    },
    "group:observation":{
        ref:"group:observation",
        title:"Observation",
        refs:["method:Class.observe", "method:Class.ignore", "method:Class.isObserving"],
        description:"Observation is the ability to take an action whenever a method is called.",
        name:"observation",
        seeAlso:[],
        type:"group"
    },
    "attr:MultiSortDialog.invalidListPrompt":{
        ref:"attr:MultiSortDialog.invalidListPrompt",
        flags:"IR",
        description:"This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed.\n <P>\n Default value returns <P>\n <code>\n <i>Columns may only be used once: <code>[some field's title]</code> is used multiple times</i>",
        name:"invalidListPrompt",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Columns may only be used once: '\\${title}' is used multiple times.\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.canDragSelect":{
        ref:"attr:ListGrid.canDragSelect",
        flags:"IRW",
        description:"If this property is true, users can drag the mouse to select several rows or cells.\n  This is mutually exclusive with rearranging rows or cells by dragging.",
        name:"canDragSelect",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragListSelect')}"],
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:DynamicForm.rejectInvalidValueOnChange":{
        ref:"attr:DynamicForm.rejectInvalidValueOnChange",
        flags:"IRWA",
        description:"If validateOnChange is true, and validation fails for an item on change, with no suggested\n value, should we revert to the previous value, or continue to display the bad value entered\n by the user. May be set at the item or form level.",
        name:"rejectInvalidValueOnChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "attr:Layout.layoutMargin":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Layout.layoutMargin",
        description:"Space outside of all members. This attribute, along with ${isc.DocUtils.linkForRef('attr:Layout.layoutLeftMargin')} \n and related properties does not have a true setter method.<br>\n It may be assigned directly at runtime. After setting the property, \n ${isc.DocUtils.linkForRef('method:Layout.setLayoutMargin')} may be called with no arguments to reflow the layout.",
        name:"layoutMargin",
        setter:["setLayoutMargin"],
        valueType:"integer",
        seeAlso:[
            {ref:"attr:Layout.layoutLeftMargin", description:null},
            {ref:"attr:Layout.layoutRightMargin", description:null},
            {ref:"attr:Layout.layoutBottomMargin", description:null},
            {ref:"attr:Layout.layoutTopMargin", description:null},
            {ref:"attr:Layout.paddingAsLayoutMargin", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('userSizing')}"],
        defaultValue:"null",
        groups:["layoutMargin"],
        definingClass:"class:Layout"
    },
    "group:dateFormatAndStorage":{
        ref:"group:dateFormatAndStorage",
        title:"Date and Time Format and Storage",
        treeLocation:["Concepts"],
        refs:[],
        description:"The SmartClient system has the following features for handling Date and Time type values\n within DataSources and databound components.\n <P>\n DataSources and databound components may define fields of type <code>date</code>,\n <code>time</code>, or <code>datetime</code>.\n <P>\n Fields of type ${isc.DocUtils.linkForRef('type:FieldType','date')} are considered to be logical Dates. System wide\n formatting for dates may be controlled via the ${isc.DocUtils.linkForRef('classMethod:Date.setNormalDisplayFormat')} and\n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat')} methods.  Components also support applying custom display\n formats for specific date fields, typically achieved via the <code>dateFormatter</code> or\n <code>field.displayFormat</code> attributes.\n <P>\n On the client date type fields are stored as JavaScript Date objects. When formatted for display\n to the user, they are typically displayed without any time information.\n <P>\n When communicating with a non SmartClient server via an \"xml\" DataSource, date field\n values in requests will be serialized out as logical date information only (omitting time)\n in the standard truncated\n <a target=_blank href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema date format</a> -\n <code>YYYY-MM-DD</code>.\n Date values received from the server in responses are expected to be in the same format, though\n if time information is included in the returned date-string it will be understood. \n This matches the default date serialization behavior for \"json\" dataSources, though dates may\n also be communicated as a JavaScript date instantiation string \n (EG: <code>new Date(1238792738633)</code>). See ${isc.DocUtils.linkForRef('attr:JSONEncoder.dateFormat')}.\n <P>\n Fields of type ${isc.DocUtils.linkForRef('type:FieldType','datetime')} are dates with full time information. System wide formatting\n for datetimes may be controlled via the ${isc.DocUtils.linkForRef('classMethod:Date.setShortDatetimeDisplayFormat')} method, or\n at a component level by <code>datetimeFormatter</code> or <code>field.displayFormat</code>\n attributes. DateTimes willbe displayed to the user in local time as set up via \n ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone')}. Note that if not explicitly set, this will default\n to the browser native timezone.\n <P>\n On the client datetime type fields are stored as JavaScript Date objects. \n When communicating with a non SmartClient server via an \"xml\" DataSource, datetime field\n values in requests will be serialized out as full datetimes in UTC using the standard\n <a target=_blank href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema date format</a> -\n <code>YYYY-MM-DDTHH:MM:SS</code>.\n Date values received from the server in responses are expected to also be in XML Schema date\n format - and assumed to be in UTC time unless an explicit timezone offset is specified on\n the datetime string (EG:<code>2006-01-10T12:22:04-04:00</code>).\n As with dates, \"json\" format dataSources use the same XML Schema format by default but may\n use JavaScript date instantiaion strings instead.\n <P>\n Fields of type ${isc.DocUtils.linkForRef('type:FieldType','time')} are logical time values. These are stored on the client as\n JavaScript date objects, but only the time information is displayed to the user.\n Time formatting is handled by the ${isc.DocUtils.linkForRef('class:Time')} class APIs. By default times are displayed to users\n in the display timezone set up via ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone')} [if not explicitly\n set, defaults to native browser local time].<br>\n When communicating with a non SmartClient server via an \"xml\" DataSource, time field\n values in requests will be serialized out as full times in UTC using the standard\n <a target=_blank href=\"http://www.w3.org/TR/xmlschema-2/#dateTime\">XML Schema date / time\n format</a> - <code>HH:MM:SS</code>. Note that the flag ${isc.DocUtils.linkForRef('classAttr:DataSource.serializeTimeAsDatetime')}\n may be set to serialize all times as full datetimes rather than just time strings.\n Time values received from the server in responses are expected to be in the same format, in UTC,\n or with an explicit timezone offset specified (for example <code>\"22:01:45-01:00\"</code>)\n <P>\n When communicating with the SmartClient server fields of type <code>date</code>,\n <code>datetime</code> and <code>time</code> are all automatically translated to Java date objects\n on the server side.",
        name:"dateFormatAndStorage",
        seeAlso:[],
        type:"group"
    },
    "classMethod:DataSource.getDataSource":{
        ref:"classMethod:DataSource.getDataSource",
        flags:"",
        description:" Lookup a DataSource by ID.\n\n",
        name:"getDataSource",
        seeAlso:[],
        params:[
            {description:"DataSource ID", optional:false, name:"ID", defaultValue:null,
             type:"identifier"}
        ],
        returns:{description:"the DataSource with this ID, if loaded, otherwise null.", type:"DataSource"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "classMethod:RPCManager.getCurrentTransactionId":{
        ref:"classMethod:RPCManager.getCurrentTransactionId",
        flags:"",
        description:" Returns the id of the current transaction (a queue of requests).\n <P>\n This method must be called after startQueue() has been called and\n at least one request has been issued.\n \n",
        name:"getCurrentTransactionId",
        seeAlso:[],
        returns:{description:"the transactionNum of the current transaction.", type:"number"},
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "attr:DateChooser.nextYearIcon":{
        ref:"attr:DateChooser.nextYearIcon",
        flags:"IR",
        description:"Icon for the next year button",
        name:"nextYearIcon",
        seeAlso:[
            {ref:"attr:DateChooser.showDoubleYearIcon", description:null}
        ],
        valueType:"URL",
        defaultValue:"\"[SKIN]doubleArrow_right.gif\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:TileGrid.removeSelectedData":{
        ref:"method:TileGrid.removeSelectedData",
        flags:"",
        description:" Remove the currently selected records from this component.\n If this is a databound grid, the records will be removed directly from the DataSource.\n <P>\n If no records are selected, no action is taken. The grid will automatically be\n updated if the record deletion succeeds.\n\n",
        name:"removeSelectedData",
        seeAlso:[],
        params:[
            {description:"callback to fire when the data has been removed", optional:true,
             name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('removeOperation')}"],
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "classMethod:Date.setShortDisplayFormat":{
        ref:"classMethod:Date.setShortDisplayFormat",
        flags:"",
        description:" Set the default short format for dates. After calling this method, subsequent calls to \n ${isc.DocUtils.linkForRef('method:Date.toShortDate')} will return a string formatted according to this format \n specification. Note that this will be the standard short date format used by\n SmartClient components.\n <P>\n The <code>format</code> parameter may be either a ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} string, or \n a function. If passed a function, this function will be executed in the scope of the Date\n and should return the formatted string.\n <P>\n Initial default shortDateFormat is <code>\"toUSShortDate\"</code>.\n\n",
        name:"setShortDisplayFormat",
        seeAlso:[],
        params:[
            {description:"new formatter", optional:false, name:"format", defaultValue:null,
             type:"DateDisplayFormat | function"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('dateFormat')}", 
         "${isc.DocUtils.linkForExampleId('customDateFormat')}"],
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:FusionChart.getDataColor":{
        ref:"method:FusionChart.getDataColor",
        flags:"",
        description:" Get a color from the ${isc.DocUtils.linkForRef('attr:FusionChart.dataColors')} Array, or white (\"FFFFFF\") if the index is beyond the\n end of the Array.  \n <P>\n Override to provide a dynamic color generation scheme.\n\n",
        name:"getDataColor",
        seeAlso:[],
        params:[
            {description:"index of the visual element to be colored", optional:false, name:"index",
             defaultValue:null, type:"Number"}
        ],
        returns:{description:"color value in hexadecimal RRGGBB format (with no leading '#')", type:"String"},
        type:"method",
        definingClass:"class:FusionChart"
    },
    "attr:ListGrid.alternateRecordStyles":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.alternateRecordStyles",
        description:"Whether alternating rows should be drawn in alternating styles, in order to create a \"ledger\"\n effect for easier reading.  If enabled, the cell style for alternate rows will have \"Dark\"\n appended to it.",
        name:"alternateRecordStyles",
        setter:["setAlternateRecordStyles"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('gridCells')}"],
        defaultValue:"false",
        groups:["cellStyling"],
        definingClass:"class:ListGrid"
    },
    "method:ButtonItem.click":{
        ref:"method:ButtonItem.click",
        flags:"",
        description:" Called when a ButtonItem is clicked on.\n\n",
        name:"click",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:ButtonItem",
        groups:["eventHandling"]
    },
    "method:CellSelection.deselectCellList":{
        ref:"method:CellSelection.deselectCellList",
        flags:"",
        description:"			deselect an array of cells\n\n",
        name:"deselectCellList",
        seeAlso:[],
        params:[
            {description:"listArray of cells to deselect. Each cell can be specified                                      as a 2 element array <code>[rowNum, colNum]</code>",
             optional:true, name:"", defaultValue:null, type:"array"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection"]
    },
    "attr:Canvas.dragIntersectStyle":{
        ref:"attr:Canvas.dragIntersectStyle",
        flags:"IRWA",
        description:"This indicates how the system will test for droppable targets: either by intersection\n with the mouse or intersection with the rectangle of the dragMoveTarget.",
        name:"dragIntersectStyle",
        seeAlso:[],
        valueType:"DragIntersectStyle",
        defaultValue:"\"mouse\"",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:MultiSortPanel.getNumLevels":{
        ref:"method:MultiSortPanel.getNumLevels",
        flags:"",
        description:" Return the number of levels of sorting that have been configured.\n\n",
        name:"getNumLevels",
        seeAlso:[],
        returns:{description:"The number of levels of sorting that have been configured", type:"number"},
        type:"method",
        definingClass:"class:MultiSortPanel"
    },
    "attr:Slider.minValueLabel":{
        ref:"attr:Slider.minValueLabel",
        flags:"IRW",
        description:"The text displayed in the label for the minimum value of the slider. If left as null, then\n slider.minValue will be displayed.",
        name:"minValueLabel",
        seeAlso:[
            {ref:"attr:Slider.showRange", description:null},
            {ref:"attr:Slider.minValue", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:ListGrid.alwaysShowEditors":{
        ref:"attr:ListGrid.alwaysShowEditors",
        flags:"IR",
        description:"When this attribute is set, editors will be rendered into every row of the grid rather than\n showing up in a single record at a time.\n This attribute is only valid when ${isc.DocUtils.linkForRef('attr:ListGrid.editByCell')} is false",
        name:"alwaysShowEditors",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "class:EBay":{
        methods:["method:EBay.setAuthToken", "method:EBay.setSandboxToken", 
         "method:EBay.setProductionToken","method:EBay.setUseProduction"],
        ref:"class:EBay",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Connectors"],
        attrs:[],
        description:"WebService object representing the eBay trading web service.",
        name:"EBay",
        seeAlso:[],
        type:"class"
    },
    "method:FormItem.iconClick":{
        ref:"method:FormItem.iconClick",
        flags:"",
        description:"      StringMethod.\n      Default action to fire when the user clicks on a form item icon. May be overridden\n      by setting <code>click</code> on the form item icon directly.\n",
        name:"iconClick",
        seeAlso:[],
        params:[
            {description:"a pointer to this item's form", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"a pointer to this form item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"a pointer to the icon that recieved the click event.", optional:false,
             name:"icon", defaultValue:null, type:"FormItemIcon"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "method:Button.setDisabled":{
        ref:"method:Button.setDisabled",
        flags:"",
        description:" Enable or disable this object\n",
        name:"setDisabled",
        seeAlso:[],
        params:[
            {description:"true if this widget is to be disabled", optional:false, name:"disabled",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Button",
        groups:["enable", "state"]
    },
    "attr:Label.valign":{
        ref:"attr:Label.valign",
        flags:"IRW",
        description:"Vertical alignment of label text. See VerticalAlignment type for details.",
        name:"valign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:Label",
        groups:["positioning"]
    },
    "attr:RPCRequest.sendNoQueue":{
        ref:"attr:RPCRequest.sendNoQueue",
        flags:"IRWA",
        description:"When set to true, this request is sent to the server immediately, bypassing any current queue.",
        name:"sendNoQueue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:ListGrid.setAutoFitMaxWidth":{
        ref:"method:ListGrid.setAutoFitMaxWidth",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxWidth')}.\n",
        name:"setAutoFitMaxWidth",
        seeAlso:[],
        params:[
            {description:"Width in px we'll expand to accomodate if  ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','auto fit')} is enabled horizontally.",
             optional:false, name:"width", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "method:LinkItem.setLinkTitle":{
        ref:"method:LinkItem.setLinkTitle",
        flags:"",
        description:" Method to set the linkTitle for this item\n",
        name:"setLinkTitle",
        seeAlso:[],
        params:[
            {description:"new linkTitle for this item", optional:false, name:"title", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:LinkItem"
    },
    "group:tabBarControls":{
        ref:"group:tabBarControls",
        refs:["attr:Tab.pickerTitle", "attr:TabSet.tabBarControls", "attr:TabSet.showTabScroller", 
         "attr:TabSet.showTabPicker","attr:TabSet.animateTabScrolling", "attr:TabSet.scrollerButtonSize", 
         "attr:TabSet.pickerButtonSize"],
        name:"tabBarControls",
        seeAlso:[],
        type:"group"
    },
    "class:TabBar":{
        methods:["method:TabBar.selectTab"],
        ref:"class:TabBar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/TabSet"],
        attrs:["attr:TabBar.tabs", "attr:TabBar.breadth", "attr:TabBar.buttonConstructor", 
         "attr:TabBar.baseLineThickness","attr:TabBar.baseLineSrc", "attr:TabBar.baseLineCapSize", "attr:TabBar.defaultTabSize"],
        description:"Shows a set of Tabs.  TabBars are automatically created by TabSets and shouldn't be used\n directly.  The TabBar is documented for skinning purposes.",
        name:"TabBar",
        seeAlso:[],
        type:"class"
    },
    "classMethod:isA.Function":{
        ref:"classMethod:isA.Function",
        flags:"",
        description:"\n	Is <code>object</code> a Function object?\n\n",
        name:"Function",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Function", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:ListGrid.rowHasErrors":{
        ref:"method:ListGrid.rowHasErrors",
        flags:"",
        description:" Does the specified row have unresolved errors?\n",
        name:"rowHasErrors",
        seeAlso:[
            {ref:"method:ListGrid.hasErrors", description:null},
            {ref:"method:ListGrid.cellHasErrors", description:null}
        ],
        params:[
            {description:"rowNum to check for errors", optional:false, name:"rowNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"true if there are unresolved errors, false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "method:ResultSet.getRange":{
        ref:"method:ResultSet.getRange",
        flags:"",
        description:" Return the items between position start and end, non-inclusive at the end, possibly \n containing markers for records that haven't loaded yet.\n <P>\n Calling getRange for records that have not yet loaded will trigger an asynchronous fetch.  The\n returned data will contain the marker value <code>Array.LOADING</code> as a placeholder for\n records being fetched.  If any rows needed to be fetched, <code>dataArrived()</code> will\n fire when they arrive.\n\n",
        name:"getRange",
        seeAlso:[
            {ref:"classAttr:Array.LOADING", description:null},
            {ref:"method:ResultSet.dataArrived", description:null}
        ],
        params:[
            {description:"start position", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end position", optional:false, name:"end", defaultValue:null, type:"number"}
        ],
        returns:{description:"subset of the array from start -> end-1", type:"Array"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access"]
    },
    "attr:ListGrid.autoFetchDisplayMap":{
        ref:"attr:ListGrid.autoFetchDisplayMap",
        flags:"IRW",
        description:"If true, for fields where ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')} is specified,\n a valueMap will be automatically created by making a ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} call\n against the specified dataSource and extracting a valueMap from the returned records\n based on the displayField and valueField.\n <P>\n If set to false, valueMaps will not be automatically fetched.  In this case, setting\n field.optionDataSource is effectively a shortcut for setting optionDataSource on\n the editor via ${isc.DocUtils.linkForRef('attr:ListGridField.editorProperties')}.\n <P>\n Can also be disabled on a per-field basis with ${isc.DocUtils.linkForRef('attr:ListGridField.autoFetchDisplayMap')}.",
        name:"autoFetchDisplayMap",
        seeAlso:[
            {ref:"attr:ListGridField.autoFetchDisplayMap", description:null},
            {ref:"attr:ListGridField.optionDataSource", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["display_values"]
    },
    "attr:FormItem.tabIndex":{
        ref:"attr:FormItem.tabIndex",
        flags:"IRW",
        description:"tabIndex for the form item within the form.  Auto-assigned if not specified.<br>\n Note: This controls the tab order within the form.\n <P>\n To specify the tabindex of an item within the page as a whole (not just this form), use\n globalTabIndex instead.",
        name:"tabIndex",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["focus"]
    },
    "attr:Label.iconOrientation":{
        ref:"attr:Label.iconOrientation",
        flags:"IR",
        description:"If this button is showing an icon should it appear to the left or right of the title?\n valid options are <code>\"left\"</code> and <code>\"right\"</code>.",
        name:"iconOrientation",
        setter:["setIconOrientation"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"left\"",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "classMethod:isA.Interface":{
        ref:"classMethod:isA.Interface",
        flags:"",
        description:"\n	Is <code>object</code> an interface object?\n\n",
        name:"Interface",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Interface Object", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:Canvas.tabIndex":{
        ref:"attr:Canvas.tabIndex",
        flags:"IRWA",
        description:"If specified this governs the tabIndex of the widget in the page's tab order.\n Note that by default SmartClient auto-assigns tab-indices, ensuring focusable widgets\n are reachable by tabbing in the order in which they are drawn on the page.\n <code>canvas.tabIndex</code> cannot be set to greater than \n ${isc.DocUtils.linkForRef('classAttr:Canvas.TAB_INDEX_FLOOR')} - as we reserve the values above this range for\n auto-assigned tab-indices.",
        name:"tabIndex",
        setter:["setTabIndex"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "method:DateChooser.dataChanged":{
        ref:"method:DateChooser.dataChanged",
        flags:"",
        description:" Method to override or observe in order to be notified when a user picks a date value.\n <P>\n Has no default behavior (so no need to call Super).\n <P>\n Use ${isc.DocUtils.linkForRef('method:DateChooser.getData')} to get the current date value.\n \n",
        name:"dataChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:DateChooser"
    },
    "attr:Layout.dropLineThickness":{
        ref:"attr:Layout.dropLineThickness",
        flags:"IRA",
        description:"Thickness, in pixels of the dropLine shown during drag and drop when\n ${isc.DocUtils.linkForRef('attr:Layout.canDropComponents')} is set to <code>true</code>.  See the discussion in\n ${isc.DocUtils.linkForRef('class:Layout')} for more info.",
        name:"dropLineThickness",
        seeAlso:[
            {ref:"class:Layout", description:null}
        ],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragMove')}"],
        definingClass:"class:Layout",
        groups:["dragdrop"]
    },
    "method:Tree.getLevel":{
        ref:"method:Tree.getLevel",
        flags:"A",
        description:"\n Return the number of levels deep this node is in the tree.  For example, for this tree:\n <pre>\n root\n   foo\n     bar\n </pre>\n Calling <code>tree.getLevel(bar)</code> will return <code>2</code>. \n\n",
        name:"getLevel",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"number of parents the node has", type:"number"},
        type:"method",
        definingClass:"class:Tree"
    },
    "classMethod:isA.ClassObject":{
        ref:"classMethod:isA.ClassObject",
        flags:"",
        description:"\n	Is <code>object</code> a class object?\n\n",
        name:"ClassObject",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Class Object", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:DynamicForm.requiredTitleSuffix":{
        ref:"attr:DynamicForm.requiredTitleSuffix",
        flags:"IRW",
        description:"The string appended to the title of every required item in this form if\n highlightRequiredFields is true.",
        name:"requiredTitleSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\" :</B>\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "method:Selection.deselectItem":{
        ref:"method:Selection.deselectItem",
        flags:"",
        description:" Deselect a particular item by its position in the list\n\n",
        name:"deselectItem",
        seeAlso:[],
        params:[
            {description:"index of the item to be selected", optional:false, name:"position",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:FormItem.iconVAlign":{
        ref:"attr:FormItem.iconVAlign",
        flags:"IRWA",
        description:"How should icons be aligned vertically for this form item.",
        name:"iconVAlign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"\"bottom\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:Label.align":{
        ref:"attr:Label.align",
        flags:"IRW",
        description:"Horizontal alignment of label text. See Alignment type for details.",
        name:"align",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.LEFT",
        type:"attr",
        definingClass:"class:Label",
        groups:["positioning"]
    },
    "classMethod:Class.echo":{
        ref:"classMethod:Class.echo",
        flags:"",
        description:" Return a short string representation of any object, suitable for viewing by a developer for\n debugging purposes.\n <P>\n If passed an object containing other objects, echo will not recurse into subobjects,\n summarizing them instead via echoLeaf().\n <P>\n NOTE: echo() is used to generate the output shown in the Log window when evaluating an\n expression.\n <P>\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n \n",
        name:"echo",
        seeAlso:[
            {ref:"classMethod:Log.echoAll", description:null},
            {ref:"classMethod:Log.echoLeaf", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "attr:ListGrid.editFailedCSSText":{
        ref:"attr:ListGrid.editFailedCSSText",
        flags:"IRWA",
        description:"Custom CSS text to be applied to cells when editing has failed.<br>\n  If this listGrid is editable, this css text will be applied to any edited cells for which\n  validation failed, on top of the base style for the cell.<br>\n For further customization of styling for cells that failed editing validation, use\n <code>this.editFailedBaseStyle</code> instead.",
        name:"editFailedCSSText",
        seeAlso:[
            {ref:"attr:ListGrid.editFailedBaseStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"\"color:red;border:1px solid red;\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "method:Date.toPrettyString":{
        ref:"method:Date.toPrettyString",
        deprecated:"As of SmartClient 5.5 use ${isc.DocUtils.linkForRef('method:Date.toShortDate')} instead",
        flags:"",
        description:"			Return this date in the format: <code>MM/DD/YY HH:MM</code>\n",
        name:"toPrettyString",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:FormItem.errorIconWidth":{
        ref:"attr:FormItem.errorIconWidth",
        flags:"IRW",
        description:"Height of the error icon, if we're showing icons when validation errors occur.",
        name:"errorIconWidth",
        seeAlso:[
            {ref:"attr:FormItem.showErrorIcon", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["errorIcon"]
    },
    "attr:Canvas.childrenSnapResizeToGrid":{
        ref:"attr:Canvas.childrenSnapResizeToGrid",
        flags:"IRW",
        description:"If true, causes this canvas's children to snap to its grid when resizing.\n This behavior can be overridden on a per-child basis by setting the \n ${isc.DocUtils.linkForRef('attr:Canvas.snapToGrid','snapToGrid')} or \n ${isc.DocUtils.linkForRef('attr:Canvas.snapResizeToGrid','snapResizeToGrid')} value on the child.",
        name:"childrenSnapResizeToGrid",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:ListGrid.headerSpanHeight":{
        ref:"attr:ListGrid.headerSpanHeight",
        flags:"IR",
        description:"Default height for a ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpan')} with no height specified.\n <P>\n If <code>headerSpanHeight</code> is not specified (the default), headerSpans will be 1/2\n of ${isc.DocUtils.linkForRef('attr:ListGrid.headerHeight')}.",
        name:"headerSpanHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "method:GridRenderer.getRowSpan":{
        ref:"method:GridRenderer.getRowSpan",
        flags:"A",
        description:" Return how many rows this cell should span.  Default is 1. \n <P>\n NOTE: if using horizontal incremental rendering, <code>getRowSpan()</code> may be called for\n a rowNum <b>in the middle of a spanning cell</b>, and should return the remaining span from\n that rowNum onward.\n <P>\n NOTE: if a cell spans multiple rows, getCellRecord/Style/etc will be called with the topmost\n row coordinates only.\n\n",
        name:"getRowSpan",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"number of cells to span", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:FilterBuilder.saveOnEnter":{
        ref:"attr:FilterBuilder.saveOnEnter",
        flags:"IR",
        description:"If true, when the user hits the Enter key while focussed in a text-item in this \n FilterBuilder, we automatically invoke the user-supplied ${isc.DocUtils.linkForRef('method:FilterBuilder.search')} method.",
        name:"saveOnEnter",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:FormItem.focus":{
        ref:"method:FormItem.focus",
        flags:"",
        description:" Called when this FormItem receives focus.\n \n",
        name:"focus",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "group:loadModulesTag":{
        ref:"group:loadModulesTag",
        title:"&lt;isomorphic:loadModules&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> HTML\n <p>\n This tag works just like ${isc.DocUtils.linkForRef('group:loadISCTag')} except it does not load a skin.  All other\n attributes are supported on this tag just as on <code>loadISC</code>.  This tag is useful if\n you have a single \"header\" JSP that has the <code>loadISC</code> tag that you then include in other\n SmartClient-enabled JSPs that require additional modules.  The JSPs that require additional\n modules can then use the <code>loadModules</code> to load additional SmartClient modules.",
        name:"loadModulesTag",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "classMethod:Flashlet.flashAvailable":{
        ref:"classMethod:Flashlet.flashAvailable",
        flags:"",
        description:" Is Shockwave flash installed on this browser?\n",
        name:"flashAvailable",
        seeAlso:[],
        returns:{description:"true if Flash is installed.", type:"boolean"},
        type:"classMethod",
        definingClass:"class:Flashlet"
    },
    "group:expansionField":{
        ref:"group:expansionField",
        refs:["method:ListGrid.getExpansionField", "method:ListGrid.isExpansionField", 
         "method:ListGrid.canExpandRecord","method:ListGrid.expandRecord", "method:ListGrid.collapseRecord", 
         "method:ListGrid.getExpansionComponent","attr:ListGrid.canExpandRecordProperty", "attr:ListGridRecord.canExpand", 
         "attr:ListGrid.canExpandRecords","attr:ListGrid.detailField", "attr:ListGrid.expansionMode", "attr:ListGrid.expansionField", 
         "attr:ListGrid.expansionFieldTrueImage","attr:ListGrid.expansionFieldFalseImage", "attr:ListGrid.expansionFieldImageWidth", 
         "attr:ListGrid.expansionFieldImageHeight","attr:ListGrid.expansionDetailField", "attr:ListGrid.expansionDetails", 
         "attr:ListGrid.expansionRelated","attr:ListGrid.expansionEditor", "attr:ListGrid.expansionDetailRelated", 
         "attr:ListGrid.expansionLayout","attr:ListGrid.childExpansionMode", "attr:ListGrid.expansionCanEdit", "type:ExpansionMode"],
        name:"expansionField",
        seeAlso:[],
        type:"group"
    },
    "class:Timer":{
        methods:[],
        ref:"class:Timer",
        classAttrs:[],
        classMethods:["classMethod:Timer.setTimeout", "classMethod:Timer.clear"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"The Timer class provides a predictable cross-browser system for creating\n timed events.",
        name:"Timer",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.showHeaderContextMenu":{
        ref:"attr:ListGrid.showHeaderContextMenu",
        flags:"IR",
        description:"Whether to show a context menu on the header with standard items for showing and hiding\n fields.",
        name:"showHeaderContextMenu",
        seeAlso:[
            {ref:"method:ListGrid.displayHeaderContextMenu", description:null},
            {ref:"method:ListGrid.getHeaderContextMenuItems", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader"]
    },
    "attr:SpinnerItem.max":{
        ref:"attr:SpinnerItem.max",
        flags:"IRW",
        description:"Maximum valid value for this item",
        name:"max",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('spinnerItem')}"],
        definingClass:"class:SpinnerItem"
    },
    "method:CubeGrid.getCellCoordinates":{
        ref:"method:CubeGrid.getCellCoordinates",
        flags:"",
        description:" Given a record in this grid, this method returns the coordinates of the cell in which the \n record is displayed as a 2 element array.\n",
        name:"getCellCoordinates",
        seeAlso:[],
        params:[
            {description:"record to find coordinates for", optional:false, name:"cellRecord",
             defaultValue:null, type:"record"}
        ],
        returns:{description:"2 element array containing <code>[rowNum,colNum]</code> for the cell, or                  <code>null</code> if the record is not found.",
         type:"array"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:Calendar.monthView":{
        ref:"attr:Calendar.monthView",
        flags:"R",
        description:"Listgrid used to display events that pertain to a given month.",
        name:"monthView",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "classMethod:isc.logWarn":{
        ref:"classMethod:isc.logWarn",
        flags:"",
        description:" Same as ${isc.DocUtils.linkForRef('classMethod:Log.logWarn')}.\n\n",
        name:"logWarn",
        seeAlso:[],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in, defaults to \"Log\"", optional:true, name:"category",
             defaultValue:null, type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:isc"
    },
    "method:DataSource.getClientOnlyResponse":{
        ref:"method:DataSource.getClientOnlyResponse",
        flags:"",
        description:" Return a \"spoofed\" response for a ${isc.DocUtils.linkForRef('attr:DataSource.clientOnly')} DataSource.\n <P>\n The default implementation will use ${isc.DocUtils.linkForRef('attr:DataSource.testData')} to provide an appropriate\n response, by using ${isc.DocUtils.linkForRef('method:DataSource.applyFilter','client-side filtering')} for a \"fetch\" request, and\n by modifying the <code>testData</code> for other requests.\n <P>\n Override this method to provide simulations of other server-side behavior, such as\n modifying other records, or to implement <b>synchronous</b> client-side data providers\n (such as Google Gears).  For <b>asynchronous</b> third-party data providers, such as\n GWT-RPC, HTML5 sockets, or bridges to plug-in based protocols (Java, Flash,\n Silverlight..), use ${isc.DocUtils.linkForRef('type:DSProtocol','dataProtocol:\"clientCustom\"')} instead. \n <P>\n Overriding this method is also a means of detecting that a normal DataSource (not\n clientOnly) would be contacting the server.\n\n",
        name:"getClientOnlyResponse",
        seeAlso:[],
        params:[
            {description:"DataSource request to respond to", optional:false, name:"request",
             defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:null, type:"DSResponse"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:PickList.textMatchStyle":{
        ref:"attr:PickList.textMatchStyle",
        flags:"IR",
        description:"When applying filter criteria to pickList data, what type of matching to use.  Legal\n values are \"substring\" (value contains user input) or \"startsWith\" (value starts with\n user input.  Both matches are case insensitive.\n <P>\n For a databound pickList (${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} set), <code>textMatchStyle</code> is\n sent to the server as ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')}.\n <P>\n For a non-databound pickList, <code>textMatchStyle</code> is applied by \n ${isc.DocUtils.linkForRef('method:PickList.filterClientPickListData')}.",
        name:"textMatchStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"startsWith\"",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "group:titles":{
        ref:"group:titles",
        refs:["attr:DataSource.title", "attr:DataSource.pluralTitle", "attr:DataSource.titleField"],
        name:"titles",
        seeAlso:[],
        type:"group"
    },
    "attr:Layout.defaultLayoutAlign":{
        ref:"attr:Layout.defaultLayoutAlign",
        flags:"IR",
        description:"Specifies the default alignment for layout members on the breadth axis (horizontal axis\n for a VLayout, vertical axis for an HLayout).  Can be overridden on a per-member basis\n by setting ${isc.DocUtils.linkForRef('attr:Canvas.layoutAlign')}.\n <P>\n If unset, default member layout alignment will be \"top\" for a horizontal layout, and\n \"left\" for a vertical layout, or \"right\" if in ${isc.DocUtils.linkForRef('classMethod:Page.isRTL','RTL')} mode.\n <P>\n When attempting to center components be sure that you have set a specific size on the\n component(s) involved.  If components fill all available space in the layout, centering\n looks the same as not centering.\n <P>\n Similarly, if a component has no visible boundary (like a border), it can appear similar\n to when it's not centered if the component is larger than you expect - use the Watch tab\n in the Developer Console to see the component's extents visually.",
        name:"defaultLayoutAlign",
        seeAlso:[],
        valueType:"Alignment or VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMember", "layoutPolicy"]
    },
    "classMethod:Page.registerKey":{
        ref:"classMethod:Page.registerKey",
        flags:"",
        description:" Fire some action when the Page recieves a keyPress event from a certain key.<br>\n Note that if a widget has keyboard focus, this action will fire only after any widget-level\n keyPress handlers have fired and bubbled the event up to the top of their ancestor chain.<br>\n Multiple actions can be registered to fire on a single keyPress using this method, and can\n be associated with different <code>target</code> objects (which will then be available as\n a parameter when the action is fired).<br>\n This differs from calling ${isc.DocUtils.linkForRef('classMethod:Page.setEvent')} with the <code>\"keyPress\"</code>\n events registered via <code>setEvent()</code> will fire <i>before</i> widget level handlers \n respond to the event, and will fire for every <code>keyPress</code> event, not just those\n triggerred by some specific key or key-combination.\n \n \n",
        name:"registerKey",
        seeAlso:[
            {ref:"method:Canvas.keyPress", description:null},
            {ref:"classMethod:Page.setEvent", description:null},
            {ref:"classMethod:Page.unregisterKey", description:null}
        ],
        params:[
            {description:"key name or identifier object.", optional:false, name:"key",
             defaultValue:null, type:"KeyIdentifier"},
            {description:"Action to fire when key is pressed.              This can be a string of script to evaluate or a javascript function.<br>\n              This action will be passed 2 parameters: The name of the key pressed will be \n              available as the first parameter or <code>key</code> keyword. The target \n              passed into this method will be available as the second parameter or \n             <code>target</code> keyword.",
             optional:false, name:"action", defaultValue:null, type:"string"},
            {description:"If specified this object will be made available to the                                  action fired as a parameter.",
             optional:true, name:"target", defaultValue:null, type:"any"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["KeyRegistry"]
    },
    "attr:Splitbar.showDownGrip":{
        ref:"attr:Splitbar.showDownGrip",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether to show the\n 'Down' state on the grip image when the user mousedown's on this widget. \n Has no effect if ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showDown')} is false",
        name:"showDownGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["grip"]
    },
    "attr:WSRequest.wsOperation":{
        ref:"attr:WSRequest.wsOperation",
        flags:"IR",
        description:"Name of the web service operation to invoke.",
        name:"wsOperation",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WSRequest"
    },
    "method:List.equals":{
        ref:"method:List.equals",
        flags:"",
        description:" Return whether this list is equal to another list.\n <P>\n Two lists are equal only if they have the same length and all contained items are in the same\n order and are also equal.\n\n",
        name:"equals",
        seeAlso:[],
        params:[
            {description:"list to check for equality", optional:false, name:"list", defaultValue:null,
             type:"List"}
        ],
        returns:{description:"whether the specified list is equal to this list", type:"boolean"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:Layout.canDropComponents":{
        ref:"attr:Layout.canDropComponents",
        flags:"IRA",
        description:"Layouts provide a default implementation of a drag and drop interaction.  If you set\n ${isc.DocUtils.linkForRef('attr:Canvas.canAcceptDrop','canAcceptDrop')}:true and <code>canDropComponents:true</code>\n on a Layout, when a droppable Canvas (${isc.DocUtils.linkForRef('attr:Canvas.canDrop','canDrop:true')} is dragged over\n the layout will show a dropLine (a simple insertion line) at the drop location.  \n <P>\n When the drop occurs, the dragTarget (obtained using\n ${isc.DocUtils.linkForRef('classMethod:EventHandler.getDragTarget')}) is added as a member of this layout at the location\n shown by the dropLine (calculated by ${isc.DocUtils.linkForRef('method:Layout.getDropPosition')}).  This default\n behavior allows either members or external components that have\n ${isc.DocUtils.linkForRef('attr:Canvas.canDragReposition')} (or ${isc.DocUtils.linkForRef('attr:Canvas.canDrag')}) and ${isc.DocUtils.linkForRef('attr:Canvas.canDrop')} set\n to <code>true</code> to be added to or reordered within the Layout.\n <P>\n You can control the thickness of the dropLine via ${isc.DocUtils.linkForRef('attr:Layout.dropLineThickness')} and\n you can customize the style using css styling in the skin file (look for .layoutDropLine in\n skin_styles.css for your skin).  \n <P>  \n If you want to dynamically create a component to be added to the Layout in response to a\n drop event you can do so as follows: \n <pre>\n isc.VLayout.create({\n   ...various layout properties...\n   canDropComponents: true,\n   drop : function () {\n     // create the new component \n     var newMember = isc.Canvas.create(); \n     // add to the layout at the current drop position \n     // (the dropLine will be showing here)\n     this.addMember(newMember, this.getDropPosition());  \n     // hide the dropLine that was automatically shown \n     // by builtin SmartClient methods\n     this.hideDropLine();\n   }\n });\n </pre>\n If you want to completely suppress the builtin drag and drop logic, but still receive drag\n and drop events for your own custom implementation, set ${isc.DocUtils.linkForRef('attr:Canvas.canAcceptDrop')} to\n <code>true</code> and <code>canDropComponents</code> to <code>false</code> on your Layout.",
        name:"canDropComponents",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Layout",
        groups:["dragdrop"]
    },
    "method:Array.indexOf":{
        ref:"method:Array.indexOf",
        flags:"",
        description:" Return the position in the list of the first instance of the specified object.\n <p>\n If pos is specified, starts looking after that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"indexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"earliest index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"last index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "classMethod:History.registerCallback":{
        ref:"classMethod:History.registerCallback",
        flags:"",
        description:"\n Registers a callback to be called when the user navigates to a synthetic history entry.\n <p>\n If the SmartClient Core module is loaded on the page where you're using the History module,\n you can use any format acceptable to ${isc.DocUtils.linkForRef('method:Class.fireCallback')} as the callback.  The\n parameters 'id' and 'data' will be passed to your callback, in that order.\n <p>\n If the SmartClient Core module is not loaded on the page, you can use one of the following\n formats:\n <ul>\n <li>A function that takes an id and a data argument, in that order.\n <li>An object literal with a property named 'method' whose value is a function that takes\n an id and a data argument, in that order; and a property named 'target' that specifies the\n object on which the callback function should be applied.  So, e.g:\n <pre>\n {target: myObj, method: myObj.myFunction}\n </pre>\n </ul>\n The user can navigate to a synthetic history entry (and trip this callback) in one of two ways:\n <ul>\n <li>When ${isc.DocUtils.linkForRef('classMethod:History.addHistoryEntry')} method is called, a new URL associated with the\n     history entry is generated, and the browser's back/forward navigation buttons become active.\n     The user can then navigate back to a stored history entry via standard browser history\n     navigation, or by explicitly hitting the appropriate URL. In this case both the ID and\n     data parameter passed to ${isc.DocUtils.linkForRef('classMethod:History.addHistoryEntry')} will be available when the\n     callback fires.</li>\n <li>Alternatively the user can store a generated history URL (for example in a browser bookmark)\n     and navigate directly to it in a new browser session. In this case the 'addHistoryEntry()'\n     may not have been fired within the browser session. This callback will still fire with the\n     appropriate history ID but the data parameter will be null. You can disable this behavior\n     by passing in the <code>requiresData</code> parameter.</li>\n </ul>\n\n If this method is called before the page has loaded, and the page initially has a URL with \n a history ID, the callback will be fired with the appropriate ID on page load.\n However if a history callback is registered after the page has loaded, it will not be fired\n until the user moves to a new synthetic history entry. If you wish to explicitly check the\n current URL for a history entry, you can use the ${isc.DocUtils.linkForRef('classMethod:History.getCurrentHistoryId')} method.\n <p>\n When the user transitions to the history entry immediately before the first synthetic\n history entry, the callback is fired with an id of null.\n \n",
        name:"registerCallback",
        seeAlso:[],
        params:[
            {description:"The callback to invoke when the user navigates to a synthetic history entry.",
             optional:false, name:"callback", defaultValue:null, type:"String or Object"},
            {description:"If passed, this callback will only be fired if the user is  navigating to a history entry that was explicitly generated in this browser session.",
             optional:false, name:"requiresData", defaultValue:null, type:"boolean"}
        ],
        type:"classMethod",
        definingClass:"class:History"
    },
    "type:CharacterCasing":{
        ref:"type:CharacterCasing",
        values:[
            {description:"No character translation", value:"TextItem.DEFAULT"},
            {description:"Map characters to uppercase", value:"TextItem.UPPER"},
            {description:"Map characters to lowercase", value:"TextItem.LOWER"}
        ],
        description:null,
        name:"CharacterCasing",
        seeAlso:[],
        type:"type",
        groups:["validation"]
    },
    "attr:FilterBuilder.topOperatorAppearance":{
        ref:"attr:FilterBuilder.topOperatorAppearance",
        flags:"IR",
        description:"How to display and edit the ${isc.DocUtils.linkForRef('attr:FilterBuilder.topOperator','top-level operator')} for this FilterBuilder.\n <P>\n See ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')} for a list of options.",
        name:"topOperatorAppearance",
        seeAlso:[],
        valueType:"TopOperatorAppearance",
        defaultValue:"\"bracket\"",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:TabSet.selectTab":{
        ref:"method:TabSet.selectTab",
        flags:"",
        description:"	Select a tab\n",
        name:"selectTab",
        seeAlso:[],
        params:[
            {description:"tab to select", optional:false, name:"tab", defaultValue:null,
             type:"number | ID | Tab"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('tabsOrientation')}"],
        definingClass:"class:TabSet"
    },
    "method:ListGrid.getBaseStyle":{
        ref:"method:ListGrid.getBaseStyle",
        flags:"",
        description:" Return the base stylename for this cell.  Has the following implementation by default:\n <ul>\n <li>If ${isc.DocUtils.linkForRef('attr:ListGrid.editFailedBaseStyle','this.editFailedBaseStyle')} is defined, and the\n     cell is displaying a validation error return this value.</li>\n <li>If ${isc.DocUtils.linkForRef('attr:ListGrid.editPendingBaseStyle','this.editFailedPendingStyle')} is defined, and\n     the cell is displaying an edit value that has not yet been saved (see \n     ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')}) return this value.</li>\n <li>Otherwise return ${isc.DocUtils.linkForRef('attr:ListGrid.recordBaseStyleProperty','record[listGrid.recordBaseStyleProperty]')},\n     if defined, otherwise ${isc.DocUtils.linkForRef('attr:ListGridField.baseStyle','field.baseStyle')}, \n     or finally ${isc.DocUtils.linkForRef('attr:ListGrid.baseStyle','this.baseStyle')}</li>\n </ul>\n\n",
        name:"getBaseStyle",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null}
        ],
        params:[
            {description:"Record associated with this cell. May be <code>null</code>                               for a new edit row at the end of this grid's data set.",
             optional:false, name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS class for this cell", type:"CSSStyleName"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('replaceStyle')}"],
        definingClass:"class:ListGrid"
    },
    "group:keyboardEvents":{
        ref:"group:keyboardEvents",
        title:"Keyboard Events",
        treeLocation:["Concepts"],
        refs:["classMethod:EventHandler.getKeyEventCharacterValue", 
         "classMethod:EventHandler.getKeyEventCharacter","classMethod:EventHandler.getKey", "classMethod:EventHandler.shiftKeyDown", 
         "classMethod:EventHandler.ctrlKeyDown","classMethod:EventHandler.altKeyDown"],
        description:"SmartClient allows keybaord events to be captured at the page level via \n ${isc.DocUtils.linkForRef('classMethod:Page.registerKey')} or ${isc.DocUtils.linkForRef('classMethod:Page.setEvent')} or at the widget level\n via ${isc.DocUtils.linkForRef('method:Canvas.keyDown')}, ${isc.DocUtils.linkForRef('method:Canvas.keyPress')}, and ${isc.DocUtils.linkForRef('method:Canvas.keyUp')}.\n <P>\n Details about the key events can be retrieved via static methods on the EventHandler class\n including ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKey')}, ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKeyEventCharacter')} and\n ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKeyEventCharacterValue')}.\n <P>\n As with other SmartClient event handling code, returning <code>false</code> will suppress the\n default native browser behavior.<br>\n <b>Note:</b> browsers do not allow cancellation of some keys' default behaviors.\n These cases vary by browser, and wherever native cancellation is supported, returning false\n from your event handler should be sufficient to suppress the behavior.\n <br>\n Some specific cases where default behavior cancellation is not always possible include:\n <ul><li>Some function keys (<code>f1, f3, f5,</code> etc) which trigger native browser behavior.\n         [These can be suppressed in Internet Explorer and Mozilla Firefox but not in some other\n          browsers such as Safari / Chrome, etc]</li>\n     <li>Some accelerator key combos such as <code>Alt+f3</code></li>\n     <li>The \"Meta\" key (the <code>Windows</code> / <code>Apple</code> key to show OS level menu)\n     </li>\n </ul>\n If you do want to include functionality for these keys in your application, we'd recommend \n testing against your expected users' browser types. It is also worth considering whether by\n changing the functionality of these standard browser keys you may provide an unexpected \n user experience (for example a user may press \"f5\" in an attempt to reload the application\n and be surprised by this triggering some alternative functionality in your application).",
        name:"keyboardEvents",
        seeAlso:[],
        type:"group"
    },
    "method:StretchImgButton.setTitleStyle":{
        ref:"method:StretchImgButton.setTitleStyle",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:StretchImgButton.titleStyle')}, which is applied to the title text.\n",
        name:"setTitleStyle",
        seeAlso:[],
        params:[
            {description:"new title style", optional:false, name:"style", defaultValue:null,
             type:"CSSStyleName"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton"
    },
    "attr:Canvas.dragType":{
        ref:"attr:Canvas.dragType",
        flags:"IRWA",
        description:"The \"type\" of thing given as a string that can be dragged from this widget. If\n      specified, this will be matched up with the dropTypes of droppable widgets as detailed\n      in the dropTypes property.",
        name:"dragType",
        seeAlso:[
            {ref:"attr:Canvas.dropTypes", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Canvas.getViewportWidth":{
        ref:"method:Canvas.getViewportWidth",
        flags:"A",
        description:"  Returns the width of the viewport onto the scrollable content.\n\n",
        name:"getViewportWidth",
        seeAlso:[],
        returns:{description:"width of the viewport, in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:TreeGrid.showOpenIcons":{
        ref:"attr:TreeGrid.showOpenIcons",
        flags:"IRW",
        description:"If true, show a different icon for <code>open</code> folders than closed folders.\n This is achieved by appending the ${isc.DocUtils.linkForRef('attr:TreeGrid.openIconSuffix')} onto the \n ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} URL [for example <code>\"[SKIN]/folder.gif\"</code> might be \n replaced by <code>\"[SKIN]/folder_open.gif\"</code>.<br>\n <b>Note</b> If this property is set to <code>false</code> the same icon is shown for\n open folders as for closed folders, unless a custom folder icon was specified. This will be\n determined by ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} plus the ${isc.DocUtils.linkForRef('attr:TreeGrid.closedIconSuffix')}.",
        name:"showOpenIcons",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "method:CubeGrid.getSelectedFacetValues":{
        ref:"method:CubeGrid.getSelectedFacetValues",
        flags:"",
        description:"\n  returns an array of facetValues objects indicating the headers that are selected in the\n  headerBar for this facet.  If facetId is not passed, returns selection for all\n  facets.\n\n  <br><i>methodType</i> getter\n",
        name:"getSelectedFacetValues",
        seeAlso:[],
        params:[
            {description:"Id for facet for which we are getting selected facetValues.",
             optional:false, name:"facetId", defaultValue:null, type:"string"}
        ],
        returns:{description:"selected facetValues", type:"Array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:DynamicForm.itemHoverOpacity":{
        ref:"attr:DynamicForm.itemHoverOpacity",
        flags:"IRW",
        description:"Opacity for hovers shown for items",
        name:"itemHoverOpacity",
        seeAlso:[
            {ref:"attr:FormItem.hoverOpacity", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "method:Operator.compareCriteria":{
        ref:"method:Operator.compareCriteria",
        flags:"",
        description:" Compare two criteria, both of which use this operator, and report whether the newCriteria is\n definitely more restrictive than the previous criteria.\n <P>\n This is used by the ${isc.DocUtils.linkForRef('class:ResultSet')} to understand whether client-side filtering can\n continue using cached data, or whether server-side filtering must be used instead.\n \n",
        name:"compareCriteria",
        seeAlso:[],
        params:[
            {description:"new criterion", optional:false, name:"newCriterion", defaultValue:null,
             type:"Criterion"},
            {description:"previous criterion", optional:false, name:"oldCriterion", defaultValue:null,
             type:"Criterion"}
        ],
        returns:{description:"0 if the criteria are equivalent, 1 if newCriterion is guaranteed more                      restrictive, and -1 if newCriterion is not guaranteed more restrictive",
         type:"Number"},
        type:"method",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "method:Window.setStatus":{
        ref:"method:Window.setStatus",
        flags:"",
        description:"			Sets the text in the status bar of the window, redrawing if necessary.\n",
        name:"setStatus",
        seeAlso:[],
        params:[
            {description:"new text for the status bar", optional:false, name:"statusString",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Window",
        groups:["appearance"]
    },
    "attr:ListGrid.timeFormatter":{
        ref:"attr:ListGrid.timeFormatter",
        flags:"IRW",
        description:"Display format to use for fields specified as type 'time'.  May also be specified at \n the field level via ${isc.DocUtils.linkForRef('attr:ListGridField.displayFormat')}.<br>\n If unset, time fields will be formatted based on the system wide \n ${isc.DocUtils.linkForRef('classAttr:Time.shortDisplayFormat')}.<br>\n If this field is editable, the timeFormatter will also be passed to the editor\n created to edit any time type fields as ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat')}",
        name:"timeFormatter",
        seeAlso:[],
        valueType:"timeFormatter",
        defaultValue:"\"toShortPaddedTime\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "type:dataPath":{
        ref:"type:dataPath",
        description:"String specifying a nested data field structure.\n <P>\n Each dataPath string is a slash-delimited set of field identifiers, for example\n <code>\"id1/id2/id3\"</code>. DataPaths may be applied directly to a\n ${isc.DocUtils.linkForRef('attr:Canvas.dataPath','component')}, and/or to a databound component field specification.\n A datapath denotes a path to a nested field value in a hierarchical structure, giving\n developers the opportunity to easily view or edit nested data structures.\n Specifically:\n <ul><li>if the component is viewing or editing a record, the value for fields \n         will be derived from a nested structure of records</li>\n     <li>if the component is bound to a dataSource, field attributes may be picked up by\n         following the dataPath to a field definition on another dataSource</li></ul>\n <b>Examples:</b><br>\n If a dynamicForm is defined with the following fields:\n <pre>\n    [\n      { name:\"name\" },\n      { name:\"street\", dataPath:\"address/street\" }\n    ]\n </pre>\n If the <code>\"name\"</code> field is set to <i>\"Joe Smith\"</i> and the <code>\"street\"</code> field\n is set to <i>\"1221 High Street\"</i>, when the values for this form are retrieved via a\n <code>getValues()</code> call they will retun an object in the following format:\n <pre>\n    {name:\"Joe Smith\", address:{street:\"1221 High Street\"}}\n </pre>\n <P>\n For databound components, dataPath also provides a way to pick up field attributes from nested\n dataSources. Given the following dataSource definitions:\n <pre>\n  isc.DataSource.create({\n      ID:\"contacts\",\n      fields:[\n          {name:\"name\"},\n          {name:\"email\"},\n          {name:\"organization\"},\n          {name:\"phone\"},\n          {name:\"address\", type:\"Address\"}\n      ]\n  });\n \n  isc.DataSource.create({\n      ID:\"Address\",\n      fields:[\n          {name:\"street\"},\n          {name:\"city\"},\n          {name:\"state\"},\n          {name:\"zip\"}\n      ]\n  });\n  </pre>\n and a databound component bound to the 'contacts' dataSource, specifying a field with a dataPath\n of <code>\"address/street\"</code> would ensure the field attributes were derived from the \n \"street\" field of the 'Address' dataSource.\n <P>\n dataPaths are also cumulative. In other words if a component has a specified dataPath, \n the dataPath of any fields it contains will be appended to that component level path when\n accessing data. For example the following form:\n <pre>\n      isc.Dynamicform.create({\n          dataPath:\"contact\",\n          fields:[\n              {dataPath:\"address/email\"}\n          ]\n </pre>\n Might be used to edit a data structure similar to this:\n <pre>{contact:{name:'Ed Jones', address:{state:\"CA\", email:\"ed@ed.jones.com\"}}}</pre>\n Nested canvases can also have dataPaths specified, which will similarly be combined. See\n the ${isc.DocUtils.linkForRef('attr:Canvas.dataPath')} attribute for more information and examples of this.",
        name:"dataPath",
        seeAlso:[],
        type:"type"
    },
    "attr:ResultSet.updateCacheFromRequest":{
        ref:"attr:ResultSet.updateCacheFromRequest",
        flags:"IRA",
        description:"When a successful Add, Update or Remove type operation fires on this ResultSet's \n dataSource, if ${isc.DocUtils.linkForRef('attr:DSResponse.data')} is unset, should we integrate the submitted\n data values (from the request) into our data-set? This attribute will be passed to\n ${isc.DocUtils.linkForRef('method:DataSource.getUpdatedData')} as the <code>useDataFromRequest</code> parameter.",
        name:"updateCacheFromRequest",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["cacheSync"]
    },
    "attr:Calendar.eventDialogFields":{
        ref:"attr:Calendar.eventDialogFields",
        flags:"IR",
        description:"The set of fields for the ${isc.DocUtils.linkForRef('attr:Calendar.eventDialog','event dialog')}. \n <p>\n The default set of fields are: \n <pre>\n    {name: \"name\", title: \"Event Name\", type: nameType, width: 250 },\n    {name: \"save\", title: \"Save Event\", type: \"SubmitItem\", endRow: false},\n    {name: \"details\", title: \"Edit Details\", type: \"button\", startRow: false}\n </pre>\n See the Customized Binding example below for more information on altering default datasource \n fields within forms.",
        name:"eventDialogFields",
        seeAlso:[],
        valueType:"Array of FormItem",
        defaultValue:"see below",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customCalendar')}", 
         "${isc.DocUtils.linkForExampleId('validationFieldBinding')}"],
        definingClass:"class:Calendar",
        groups:["editing"]
    },
    "attr:DataBoundComponent.exportAll":{
        ref:"attr:DataBoundComponent.exportAll",
        flags:"IRW",
        description:"Setting exportAll to true prevents the component from passing it's list of fields to the \n export call.  The result is the export of all visible fields from ${isc.DocUtils.linkForRef('attr:DataSource.fields')}.\n <P>\n If exportAll is false, an export operation will first consider \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.exportFields')}, if it's set, and fall back on all visible fields from\n ${isc.DocUtils.linkForRef('attr:DataSource.fields')} otherwise.",
        name:"exportAll",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:ListGrid.setHeaderSpans":{
        ref:"method:ListGrid.setHeaderSpans",
        flags:"",
        description:" Update the headerSpans configuration on the grid dynamically.\n\n",
        name:"setHeaderSpans",
        seeAlso:[],
        params:[
            {description:"same configuration block as that passed to                                   ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans')}.",
             optional:false, name:"name", defaultValue:null, type:"Array of HeaderSpan"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "method:DynamicForm.showErrors":{
        ref:"method:DynamicForm.showErrors",
        flags:"",
        description:" If this form has any outstanding validation errors, show them now.<br>\n This method is called when the set of errors are changed by ${isc.DocUtils.linkForRef('method:DynamicForm.setErrors')} or\n ${isc.DocUtils.linkForRef('method:DynamicForm.validate')}.<br>\n Default implementation will redraw the form to display error messages and call\n ${isc.DocUtils.linkForRef('method:DynamicForm.handleHiddenValidationErrors','handleHiddenValidationErrors()')} to\n display errors with no visible field.<br>\n Note that this method may be overridden to perform custom display of validation errors.  \n",
        name:"showErrors",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "method:SummaryBuilder.setSummary":{
        ref:"method:SummaryBuilder.setSummary",
        flags:"",
        description:" Call to set the format-string in this SummaryBuilder.\n <P>\n Note that calling setSummary() will update the UI, generate the summary's funtion and \n test it automatically.\n\n",
        name:"setSummary",
        seeAlso:[],
        params:[
            {description:"The new format-string for the summary", optional:false, name:"",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:SummaryBuilder",
        groups:["formulaFields"]
    },
    "type:JSONCircularReferenceMode":{
        ref:"type:JSONCircularReferenceMode",
        values:[
            {description:"circular references in Arrays will be represented as a null entry, and objects               will have a property with a null value",
             value:"\"omit\""},
            {description:"leave a string marker, the ${isc.DocUtils.linkForRef('attr:JSONEncoder.circularReferenceMarker')},                 wherever a circular reference is found",
             value:"\"marker\""},
            {description:"leave a string marker <i>followed by</i> the path to the first occurrence of               the circular reference from the top of the object tree that was serialized.\n               This potentially allows the original object graph to be reconstructed.",
             value:"\"path\""}
        ],
        description:"What the ${isc.DocUtils.linkForRef('class:JSONEncoder')} should do when it encounters a circular reference in an object\n structure.",
        name:"JSONCircularReferenceMode",
        seeAlso:[],
        type:"type"
    },
    "attr:DynamicForm.rightTitleSuffix":{
        ref:"attr:DynamicForm.rightTitleSuffix",
        flags:"IRW",
        description:"The string appended to the title of an item in this form if its titleOrientation\n          property is set to \"right\".",
        name:"rightTitleSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "classMethod:Page.getScreenHeight":{
        ref:"classMethod:Page.getScreenHeight",
        flags:"",
        description:"		Get the height of the user's screen, in pixels.\n",
        name:"getScreenHeight",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:ComboBoxItem.maskSaveLiterals":{
        ref:"attr:ComboBoxItem.maskSaveLiterals",
        flags:null,
        description:"Not applicable to a ComboBoxItem.",
        name:"maskSaveLiterals",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:Menu.showKeys":{
        ref:"attr:Menu.showKeys",
        flags:"IRW",
        description:"A boolean, indicating whether the shortcut key column should be displayed. If\n showKeys is not set, the menu will show the key column only if one of its items\n specifies a keys property. If showKeys is false, the keys will not be displayed,\n but will still function.",
        name:"showKeys",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:ListGrid.setSortState":{
        ref:"method:ListGrid.setSortState",
        flags:"",
        description:" Reset this grid's sort state (sort field and direction) to match the \n ${isc.DocUtils.linkForRef('type:listGridSortState')} object passed in.<br>\n Used to restore previous state retrieved from the grid by a call to \n ${isc.DocUtils.linkForRef('method:ListGrid.getSortState')}.\n\n",
        name:"setSortState",
        seeAlso:[
            {ref:"method:ListGrid.getSortState", description:null}
        ],
        params:[
            {description:"Object describing the desired sort state for the grid.", optional:false,
             name:"sortState", defaultValue:null, type:"listGridSortState"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:CubeGrid.saveByCell":{
        ref:"attr:CubeGrid.saveByCell",
        flags:"R",
        description:"CubeGrids only support editing by cell",
        name:"saveByCell",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["cellEditing"]
    },
    "attr:FormItem.dateFormatter":{
        ref:"attr:FormItem.dateFormatter",
        flags:"IRWA",
        description:"Display format to use for date type values within this formItem. Default is to use the\n system-wide default short date format, configured via\n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDisplayFormat')}.  Specify any valid ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} to \n change the format used by this item.<br>\n Note: if both <code>dateFormatter</code> and ${isc.DocUtils.linkForRef('attr:FormItem.timeFormatter','timeFormatter')}\n are specified on an item, Date type values will be formatted as dates using \n <code>dateFormatter</code>.<br>\n See also ${isc.DocUtils.linkForRef('attr:DateItem.displayFormat')} for formatting dates displayed in DateItems.",
        name:"dateFormatter",
        seeAlso:[],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:DynamicForm.canFocus":{
        ref:"attr:DynamicForm.canFocus",
        flags:"IRWA",
        description:"DynamicForms are considered to have focus if any of their form items have focus.\n Note that setting <code>dynamicForm.canFocus</code> to false will have no effect on\n whether form items within the form may recieve focus. This property will only govern\n whether the form may recieve focus if the form contains no focusable items.",
        name:"canFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["focus"]
    },
    "attr:GridRenderer.quickDrawAheadRatio":{
        ref:"attr:GridRenderer.quickDrawAheadRatio",
        flags:"IRWA",
        description:"Alternative to ${isc.DocUtils.linkForRef('attr:GridRenderer.drawAheadRatio')}, to be used when the user\n is rapidly changing the grids viewport (for example drag scrolling through the grid).\n If unspecified ${isc.DocUtils.linkForRef('attr:GridRenderer.drawAheadRatio')} will be used in all cases",
        name:"quickDrawAheadRatio",
        seeAlso:[],
        valueType:"float",
        defaultValue:"1.3",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "group:fetching":{
        ref:"group:fetching",
        refs:["attr:ResultSet.fetchMode", "attr:ResultSet.initialData", "attr:ResultSet.initialLength", 
         "attr:ResultSet.allRows","attr:ResultSet.resultSize", "attr:ResultSet.fetchDelay", "type:FetchMode"],
        name:"fetching",
        seeAlso:[],
        type:"group"
    },
    "method:Menu.setItems":{
        ref:"method:Menu.setItems",
        flags:"",
        description:" Synonym for ${isc.DocUtils.linkForRef('method:Menu.setData')}.\n",
        name:"setItems",
        seeAlso:[],
        params:[
            {description:"new items for this menu", optional:false, name:"items", defaultValue:null,
             type:"array of MenuItems"}
        ],
        type:"method",
        definingClass:"class:Menu",
        groups:["data"]
    },
    "method:ListGrid.setAutoFitMaxRecords":{
        ref:"method:ListGrid.setAutoFitMaxRecords",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxRecords')}.\n",
        name:"setAutoFitMaxRecords",
        seeAlso:[],
        params:[
            {description:"Maximum number of rows we'll expand to accomodate if  ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','auto fit')} is enabled vertically.",
             optional:false, name:"maxRecords", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "classMethod:isc.say":{
        ref:"classMethod:isc.say",
        flags:"",
        description:" Show a modal dialog with a message, icon, and \"OK\" button.  Intended for notifications which\n are not really warnings (default icon is less severe).\n <P>\n The callback will receive boolean true for an OK button click, or null if the Dialog is\n dismissed via the close button.\n\n",
        name:"say",
        seeAlso:[
            {ref:"classAttr:Dialog.Warn", description:null},
            {ref:"classMethod:isc.warn", description:null},
            {ref:"classMethod:isc.ask", description:null},
            {ref:"method:Dialog.okClick", description:null},
            {ref:"classAttr:Dialog.SAY_TITLE", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"Optional Callback to fire when the user                                       dismisses the dialog. This has the single parameter\n                                      'value', indicating the value returned by the Warn\n                                      dialog from 'okClick()' etc.",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"additional properties for the Dialog.                                      To set ${isc.DocUtils.linkForRef('attr:Dialog.toolbarButtons','custom buttons')} for\n                                      the Dialog, set properties.toolbarButtons to an array of\n                                      buttons\n										eg:	{ buttons : [Dialog.OK, Dialog.CANCEL] }",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "group:state":{
        ref:"group:state",
        refs:["method:StatefulCanvas.setState", "method:StatefulCanvas.getState", 
         "method:StatefulCanvas.setSelected","method:StatefulCanvas.select", "method:StatefulCanvas.deselect", 
         "method:StatefulCanvas.isSelected","method:StatefulCanvas.getActionType", "method:StatefulCanvas.setActionType", 
         "method:StatefulCanvas.addToRadioGroup","method:StatefulCanvas.removeFromRadioGroup", "method:StatefulCanvas.setDisabled", 
         "method:Button.select","method:Button.deselect", "method:Button.isSelected", "method:Button.setSelected", 
         "method:Button.addToRadioGroup","method:Button.removeFromRadioGroup", "method:Button.setActionType", 
         "method:Button.getActionType","method:Button.setState", "method:Button.setDisabled", "method:Button.getState", 
         "method:ImgButton.select","method:ImgButton.deselect", "method:ImgButton.isSelected", "method:ImgButton.setSelected", 
         "method:ImgButton.addToRadioGroup","method:ImgButton.removeFromRadioGroup", "method:ImgButton.setActionType", 
         "method:ImgButton.getActionType","method:ImgButton.setState", "method:ImgButton.setDisabled", "method:ImgButton.getState", 
         "method:StretchImgButton.select","method:StretchImgButton.deselect", "method:StretchImgButton.isSelected", 
         "method:StretchImgButton.setSelected","method:StretchImgButton.addToRadioGroup", "method:StretchImgButton.removeFromRadioGroup", 
         "method:StretchImgButton.setActionType","method:StretchImgButton.getActionType", "method:StretchImgButton.setState", 
         "method:StretchImgButton.setDisabled","method:StretchImgButton.getState", "attr:StatefulCanvas.redrawOnStateChange", 
         "attr:StatefulCanvas.selected","attr:StatefulCanvas.state", "attr:StatefulCanvas.showRollOver", 
         "attr:StatefulCanvas.showFocus","attr:StatefulCanvas.showFocused", "attr:StatefulCanvas.showFocusedAsOver", 
         "attr:StatefulCanvas.showDown","attr:StatefulCanvas.showDisabled", "attr:StatefulCanvas.actionType", 
         "attr:StatefulCanvas.radioGroup","attr:Button.selected", "attr:Button.radioGroup", "attr:Button.actionType", 
         "attr:Button.state","attr:Button.showDisabled", "attr:Button.showDown", "attr:Button.showFocused", 
         "attr:Button.showRollOver","attr:ImgButton.selected", "attr:ImgButton.radioGroup", "attr:ImgButton.actionType", 
         "attr:ImgButton.state","attr:ImgButton.showDisabled", "attr:ImgButton.showDown", "attr:ImgButton.showFocus", 
         "attr:ImgButton.showFocused","attr:ImgButton.showRollOver", "attr:StretchImgButton.selected", 
         "attr:StretchImgButton.radioGroup","attr:StretchImgButton.actionType", "attr:StretchImgButton.state", 
         "attr:StretchImgButton.showDisabled","attr:StretchImgButton.showDown", "attr:StretchImgButton.showFocus", 
         "attr:StretchImgButton.showFocused","attr:StretchImgButton.showRollOver", "type:State", "type:SelectionType", "type:Selected"],
        description:"Change of state and it's consequences for presentation.",
        name:"state",
        seeAlso:[],
        type:"group"
    },
    "class:Dialog":{
        methods:["method:Dialog.setToolbarButtons", "method:Dialog.saveData", "method:Dialog.closeClick", 
         "method:Dialog.cancelClick","method:Dialog.okClick", "method:Dialog.applyClick", "method:Dialog.yesClick", 
         "method:Dialog.noClick","method:Dialog.doneClick"],
        exampleConfig:"Dialog.xml",
        ref:"class:Dialog",
        classAttrs:["classAttr:Dialog.OK_BUTTON_TITLE", "classAttr:Dialog.APPLY_BUTTON_TITLE", 
         "classAttr:Dialog.YES_BUTTON_TITLE","classAttr:Dialog.NO_BUTTON_TITLE", "classAttr:Dialog.CANCEL_BUTTON_TITLE", 
         "classAttr:Dialog.DONE_BUTTON_TITLE","classAttr:Dialog.CONFIRM_TITLE", "classAttr:Dialog.SAY_TITLE", 
         "classAttr:Dialog.WARN_TITLE","classAttr:Dialog.ASK_TITLE", "classAttr:Dialog.ASK_FOR_VALUE_TITLE", 
         "classAttr:Dialog.Prompt","classAttr:Dialog.Warn"],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:Dialog.styleName", "attr:Dialog.messageStyle", "attr:Dialog.showToolbar", 
         "attr:Dialog.toolbarButtons"],
        description:"Dialogs are a specialized version of ${isc.DocUtils.linkForRef('class:Window')} used for small windows such as\n alerts, prompts, and confirmations.  They can be modal or modeless (via the\n ${isc.DocUtils.linkForRef('attr:Window.isModal','isModal')} property) and will contain various children by default\n (\"titlebar\", \"resizer\", etc).\n <P>\n NOTE: If you are building a custom component that will add components to the Window via\n ${isc.DocUtils.linkForRef('method:Window.addItem','addItem()')}, in most cases it makes sense",
        name:"Dialog",
        seeAlso:[],
        type:"class"
    },
    "attr:HeaderSpan.title":{
        ref:"attr:HeaderSpan.title",
        flags:"IR",
        description:"Title for this headerSpan.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "attr:Progressbar.src":{
        ref:"attr:Progressbar.src",
        flags:"IRW",
        description:"The base file name for the progressbar image.",
        name:"src",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]progressbar.gif\"",
        type:"attr",
        definingClass:"class:Progressbar",
        groups:["appearance"]
    },
    "attr:DataBoundComponent.duplicateDragMessage":{
        ref:"attr:DataBoundComponent.duplicateDragMessage",
        flags:"IR",
        description:"Message to show when a user attempts to transfer duplicate records into this component, and\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.preventDuplicates')} is enabled.",
        name:"duplicateDragMessage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Duplicates not allowed\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:Canvas.setHtmlElement":{
        ref:"method:Canvas.setHtmlElement",
        flags:"",
        description:" Setter for the ${isc.DocUtils.linkForRef('attr:Canvas.htmlElement')}.\n",
        name:"setHtmlElement",
        seeAlso:[],
        params:[
            {description:"New htmlElement for this canvas, or Null to clear the existing      htmlElement",
             optional:false, name:"element", defaultValue:null, type:"DOM element"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["htmlElement"]
    },
    "method:FormItem.getDisplayValue":{
        ref:"method:FormItem.getDisplayValue",
        flags:"",
        description:" Returns this item's value with any valueMap applied to it - the value as currently\n displayed to the user.\n",
        name:"getDisplayValue",
        seeAlso:[],
        params:[
            {description:"optional stored value to be mapped to a display value.  Default is to                use the form's current value",
             optional:true, name:"value", defaultValue:null, type:null}
        ],
        returns:{description:"value displayed to the user", type:"any"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["valueMap"]
    },
    "class:LoginDialog":{
        methods:["method:LoginDialog.register", "method:LoginDialog.lostPassword", 
         "method:LoginDialog.loginFunc"],
        ref:"class:LoginDialog",
        classAttrs:["classAttr:LoginDialog.LOGIN_TITLE", "classAttr:LoginDialog.USERNAME_TITLE", 
         "classAttr:LoginDialog.PASSWORD_TITLE","classAttr:LoginDialog.LOGIN_BUTTON_TITLE", "classAttr:LoginDialog.LOGIN_ERROR_MESSAGE"],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:LoginDialog.dismissable", "attr:LoginDialog.dismissOnEscape", 
         "attr:LoginDialog.showCloseButton","attr:LoginDialog.allowBlankPassword", "attr:LoginDialog.showLostPasswordLink", 
         "attr:LoginDialog.showRegistrationLink","attr:LoginDialog.title", "attr:LoginDialog.errorStyle", 
         "attr:LoginDialog.usernameItemTitle","attr:LoginDialog.passwordItemTitle", "attr:LoginDialog.loginButtonTitle", 
         "attr:LoginDialog.lostPasswordItemTitle","attr:LoginDialog.registrationItemTitle", "attr:LoginDialog.errorMessage", 
         "attr:LoginDialog.items","attr:LoginDialog.formFields", "attr:LoginDialog.loginFailureItem", 
         "attr:LoginDialog.usernameItem","attr:LoginDialog.lostPasswordItem", "attr:LoginDialog.registrationItem", 
         "attr:LoginDialog.passwordItem","attr:LoginDialog.loginButton", "attr:LoginDialog.loginForm"],
        description:"Handle a complete login interaction with a typical login dialog asking for username and\n password credentials. Use this\n class to quickly present a traditional username/password authentication mechanism in a\n SmartClient window.\n <p>\n To adapt this class to your requirements, first implement LoginDialog.loginFunc to submit\n the username and password to the authentication mechanism of your choice, calling\n dialogCallback once the authentication process completes.",
        name:"LoginDialog",
        seeAlso:[
            {ref:"classMethod:isc.showLoginDialog", description:null}
        ],
        type:"class",
        groups:["Prompting"]
    },
    "class:WSRequest":{
        methods:[],
        ref:"class:WSRequest",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:WSRequest.wsOperation", "attr:WSRequest.data", "attr:WSRequest.useFlatFields", 
         "attr:WSRequest.xmlNamespaces","attr:WSRequest.xmlResult", "attr:WSRequest.headerData"],
        description:"A WSRequest (or \"web service request\") is an extended RPCRequest will additional properties\n application to WSDL/SOAP web services.\n <P>\n All properties which are legal on ${isc.DocUtils.linkForRef('class:RPCRequest')} are legal on a WSRequest, in\n addition to the properties listed here.",
        name:"WSRequest",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        type:"class"
    },
    "attr:Tree.idField":{
        ref:"attr:Tree.idField",
        flags:"IRA",
        description:"Name of the property on a ${isc.DocUtils.linkForRef('object:TreeNode')} that holds an id for the node which is unique\n across the entire Tree.  Required for all nodes for trees with modelType \"parent\".\n Default value is \"id\".  See ${isc.DocUtils.linkForRef('attr:TreeNode.id')} for usage.",
        name:"idField",
        seeAlso:[
            {ref:"attr:TreeNode.id", description:null}
        ],
        valueType:"string",
        defaultValue:"\"id\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "group:dsResponse":{
        ref:"group:dsResponse",
        refs:["attr:DSResponse.data"],
        name:"dsResponse",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.getImage":{
        ref:"method:Canvas.getImage",
        flags:"A",
        description:" Retrieve a native image element by name.\n <P>\n The image element must have been created from HTML generated by calling\n <code>canvas.imgHTML()</code> on this particular Canvas.\n \n",
        name:"getImage",
        seeAlso:[],
        params:[
            {description:"name of the image to get, as originally passed to                              <code>imgHTML</code>",
             optional:false, name:"identifier", defaultValue:null, type:"string"}
        ],
        returns:{description:"DOM image object if found, else null", type:"object"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["images"]
    },
    "method:Canvas.isDirty":{
        ref:"method:Canvas.isDirty",
        flags:"A",
        description:" Returns whether a canvas is waiting to be redrawn. Will return true if \n ${isc.DocUtils.linkForRef('method:Canvas.markForRedraw')} has been called, but this canvas has not yet been\n redrawn.\n",
        name:"isDirty",
        seeAlso:[],
        returns:{description:"true is this canvas needs to be redrawn; false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "attr:FormItem.errorOrientation":{
        ref:"attr:FormItem.errorOrientation",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is true, where should the error icon and text appear\n relative to the form item itself. Valid options are <code>\"top\"</code>, \n <code>\"bottom\"</code>, <code>\"left\"</code> or <code>\"right\"</code>.<br>\n If unset the orientation will be derived from ${isc.DocUtils.linkForRef('attr:DynamicForm.errorOrientation')}.",
        name:"errorOrientation",
        seeAlso:[],
        valueType:"align",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["validation", "appearance"]
    },
    "method:ViewLoader.setViewURL":{
        ref:"method:ViewLoader.setViewURL",
        flags:"",
        description:" Change the URL this component loads a view from.  Triggers a fetch from the new URL.\n <P>\n Can also be called with no arguments to reload the view from the existing\n ${isc.DocUtils.linkForRef('attr:ViewLoader.viewURL')}.\n\n",
        name:"setViewURL",
        seeAlso:[],
        params:[
            {description:"URL to retrieve view from", optional:true, name:"url", defaultValue:null,
             type:"URL"},
            {description:"Parameters to send to the viewURL.  Merged with                           <code>component.viewURLParams</code> if both are set.",
             optional:true, name:"params", defaultValue:null, type:"Object"},
            {description:"Additional properties for the RPCRequest                         sent by the ViewLoader.  Very advanced; could be used to, for\n                        example, set HTTP headers.",
             optional:true, name:"rpcProperties", defaultValue:null, type:"RPCRequest Properties"}
        ],
        type:"method",
        definingClass:"class:ViewLoader",
        groups:["viewLoading"]
    },
    "attr:PickList.hideEmptyPickList":{
        ref:"attr:PickList.hideEmptyPickList",
        flags:"IRW",
        description:"If this pickList contains no options, should it be hidden?\n If unset, default behavior is to allow the empty pickList to show if it is databound.",
        name:"hideEmptyPickList",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "group:jspTags":{
        ref:"group:jspTags",
        title:"SmartClient JSP Tags",
        refs:[],
        description:"The SmartClient Java Server component ships with a number of custom JSP tags designed to\n make development with SmartClient easier.  The custom tags are defined in\n <code>[webroot]/WEB-INF/iscTaglib.xml</code> in the SDK package.  To make use of these tags, make\n sure you have the following entry in your web.xml (the SDK already contains this entry by\n default):<br>\n <pre>\n &lt;taglib&gt;\n     &lt;taglib-uri&gt;isomorphic&lt;/taglib-uri&gt; \n     &lt;taglib-location&gt;/WEB-INF/iscTaglib.xml&lt;/taglib-location&gt; \n &lt;/taglib&gt;\n </pre>\n <p>\n Also, the JSP page making use of the tag must load the custom taglib via the following\n directive:<br>\n <pre>\n &lt;%@ taglib uri=\"/WEB-INF/iscTaglib.xml\" prefix=\"isomorphic\" %&gt;\n </pre>\n All SmartClient JSP tags produce either HTML or JavaScript output, so you can easily see\n what any given tag is generating by doing a \"View->Source\" in your browser after browsing to\n the JSP that contains your tag.  Tags that produce HTML must be located in the HTML BODY\n context in your JSP - that is, outside of any <code>&lt;SCRIPT&gt;</code> tags and inside\n <code>&lt;BODY&gt;</code> tags.  Tags that produce JavaScript must be located inside\n <code>&lt;SCRIPT&gt;</code> tags.",
        name:"jspTags",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "method:DataSource.getLegalChildTags":{
        ref:"method:DataSource.getLegalChildTags",
        flags:"",
        description:" For a DataSource that describes a DOM structure, the list of legal child elements that can\n be contained by the element described by this DataSource.\n <p>\n For a DataSource described by XML schema, this is the list of legal subelements <b>of\n complexType</b> (elements of simpleType become DataSourceFields with atomic type).\n <p>\n Note that currently, if an XML schema file contains ordering constraints, DataSources\n derived from XML Schema do not capture these constraints.\n\n",
        name:"getLegalChildTags",
        seeAlso:[],
        type:"method",
        definingClass:"class:DataSource",
        groups:["xmlSchema"]
    },
    "method:ListGrid.toggleSort":{
        ref:"method:ListGrid.toggleSort",
        flags:"A",
        description:"  Toggles the sort-direction of the field with the passed name and resorts the grid.\n",
        name:"toggleSort",
        seeAlso:[],
        params:[
            {description:"The name of a field, visible, hidden or existing only in the dataSource",
             optional:false, name:"fieldName", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:Window.setShowMaximizeButton":{
        ref:"method:Window.setShowMaximizeButton",
        flags:"",
        description:" Dynamically update ${isc.DocUtils.linkForRef('attr:Window.showMaximizeButton')} to show / hide the maximizeButton\n",
        name:"setShowMaximizeButton",
        seeAlso:[
            {ref:"attr:Window.headerControls", description:null},
            {ref:"attr:Window.showMaximizeButton", description:null}
        ],
        type:"method",
        definingClass:"class:Window"
    },
    "classMethod:Page.clearEvent":{
        ref:"classMethod:Page.clearEvent",
        flags:"",
        description:"	Clear event(s) under the given eventType.<p>\n	To clear all events, omit the ID parameter.  To clear a specific event,\n	pass the ID that was returned by Page.setEvent().\n",
        name:"clearEvent",
        seeAlso:[
            {ref:"class:EventHandler", description:null}
        ],
        params:[
            {description:"event type to clear", optional:false, name:"eventType", defaultValue:null,
             type:"PageEvent, Event"},
            {description:"ID of the event to clear. 										If not specified, all events in eventType will be cleared.",
             optional:true, name:"ID", defaultValue:null, type:"number"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["EventRegistry"]
    },
    "attr:CubeGrid.hilites":{
        ref:"attr:CubeGrid.hilites",
        flags:"IRW",
        description:"Hilites to be applied to the data for this component.  See ${isc.DocUtils.linkForRef('group:hiliting')}.",
        name:"hilites",
        seeAlso:[],
        valueType:"Array of Hilite",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:ListGridField.linkText":{
        ref:"attr:ListGridField.linkText",
        flags:"IRW",
        description:"The HTML to display in cells of this field if the fieldType is set to link. Can be\n  overridden by ${isc.DocUtils.linkForRef('attr:ListGridRecord.linkText')}.",
        name:"linkText",
        seeAlso:[
            {ref:"type:ListGridFieldType", description:null},
            {ref:"type:FieldType", description:null},
            {ref:"attr:ListGridRecord.linkText", description:null},
            {ref:"attr:ListGrid.linkTextProperty", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('linkImage')}"],
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "classMethod:XMLTools.nativeXMLAvailable":{
        ref:"classMethod:XMLTools.nativeXMLAvailable",
        flags:"",
        description:"\n Returns true if the current browser exposes an XML parser that can be used for SmartClient\n XML operations like web service bindings and XML processing.  See\n ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information on when the XML parser may not\n available and what features are impacted as a result.\n\n",
        name:"nativeXMLAvailable",
        seeAlso:[],
        returns:{description:"true if native XML processing is available, false otherwise.", type:"boolean"},
        type:"classMethod",
        definingClass:"class:XMLTools"
    },
    "classMethod:isA.Boolean":{
        ref:"classMethod:isA.Boolean",
        flags:"",
        description:"\n	Is <code>object</code> a Boolean object?\n\n",
        name:"Boolean",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Boolean", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:Canvas.redraw":{
        ref:"method:Canvas.redraw",
        flags:"A",
        description:" Redraws the widget immediately with its current property values.  \n\n Generally, if you want a Canvas to redraw, call markForRedraw() - this will cause the Canvas to\n be redrawn when current processing ends, so that a series of modifications made to a Canvas will\n cause it to redraw only once.\n\n Only call redraw() directly if you need immediate responsiveness, for example you are redrawing\n in response to continuous mouse motion.\n\n",
        name:"redraw",
        seeAlso:[],
        params:[
            {description:"reason for performing the redraw", optional:true, name:"reason",
             defaultValue:"\"no reason provided\"", type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "group:stringMethods":{
        ref:"group:stringMethods",
        title:"String Methods Overview",
        treeLocation:["Client Reference/System"],
        refs:[],
        description:"A method flagged as a String Method can be specified as a String containing a valid\n JavaScript expression.  This expression will automatically be converted to a function with a\n return value matching the value of the last statement.  Providing a String is not required -\n you may use a real function instead.\n <p>\n For example - suppose you wanted to override the <code>leafClick()</code> method on\n the TreeGrid.  Normally you would do so as follows:<br>\n\n <pre>\n TreeGrid.create({\n     ...\n     leafClick : function(viewer, leaf, recordNum) { \n         if(leaf.name == 'zoo') { \n             alert(1); \n         } else {\n             alert(2);\n         }\n     }\n });\n </pre>\n\n Since leafClick is a stringMethod, however, you can shorten this to:<br>\n <pre>\n TreeGrid.create({\n     ...\n     leafClick : \"if(leaf.name == 'zoo') { alert(1); } else { alert(2); }\";\n });\n </pre>",
        name:"stringMethods",
        seeAlso:[],
        type:"group"
    },
    "method:CubeGrid.getCellStyle":{
        ref:"method:CubeGrid.getCellStyle",
        flags:"",
        description:" Return the CSS class for a cell. By default this method has the following implementation:<br>\n - return any custom style for the record (see ${isc.DocUtils.linkForRef('attr:GridRenderer.recordCustomStyleProperty')}) \n   if defined.<br>\n - create a style name based on the result of ${isc.DocUtils.linkForRef('method:GridRenderer.getBaseStyle')} and the \n   state of the record.<br>\n The state of the record is indicated by adding a suffix to the base style.\n There are four independant boolean states, which are combined in the order given:\n <ol>\n <li>\"Disabled\" : whether the cell is disabled; enable by setting the \"enabled\" flag on record\n     returned by getCellRecord\n <li>\"Selected\" : whether cell is selected; enable by passing a Selection object as \"selection\"\n <li>\"Over\" : mouse is over this cell; enable with showRollovers\n <li>\"Dark\" : alternating color bands; enable with alternateRowStyles\n </ol>\n For example, with a baseStyle of \"myCell\", a cell which is selected, which the mouse is over,\n and which is in a dark-colored band will get a styleName of myCellSelectedOverDark.\n <p>\n Cell Styles customizable by:\n <ul>\n <li>attaching a custom style to a record by setting \n    <code>record[this.recordCustomStyleProperty]</code> to some valid CSS style name.  \n <li>modifying the base style returned by getBaseStyle() [see that method for further \n     documentation on this]\n <li>overriding this function\n </ul>\n\n",
        name:"getCellStyle",
        seeAlso:[],
        params:[
            {description:"record object for this row and column", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"number of the row", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"number of the column", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS style for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "method:WebService.getHeaderData":{
        ref:"method:WebService.getHeaderData",
        flags:"",
        description:" Override this method to return data that should be serialized as SOAP headers for the\n current operation, such as a sessionId.\n <P>\n Format of the returned data is the same as that documented for\n ${isc.DocUtils.linkForRef('attr:DSRequest.headerData')}.\n <P>\n The object passed to this method will be a true DSRequest in the case of a DataSource\n operation, or just an Object with a \"data\" property for web service operations\n initiated by ${isc.DocUtils.linkForRef('method:WebService.callOperation')}.\n <P>\n If <code>headerData</code> is instead provided via either dsRequest.headerData or as\n part of the <code>requestProperties</code> parameter to\n ${isc.DocUtils.linkForRef('method:WebService.callOperation','callOperation()')}, this method will never be called.\n\n",
        name:"getHeaderData",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"dsRequest", defaultValue:null, type:"DSRequest"}
        ],
        returns:{description:"data for SOAP headers", type:"Object"},
        type:"method",
        definingClass:"class:WebService"
    },
    "attr:ListGridField.name":{
        ref:"attr:ListGridField.name",
        flags:"IRW",
        description:"Name of this field.  Must be unique within this ListGrid.\n <P>\n The name of field is also the property in each record which holds the value for that\n field.\n <P>\n If a ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')} is specified and the DataSource has a field with the\n same name, the ListGridField and DataSourceField are merged so that properties on the\n ListGridField",
        name:"name",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["data"]
    },
    "attr:ToolbarItem.showTitle":{
        ref:"attr:ToolbarItem.showTitle",
        flags:"IRW",
        description:"Don't show a title for toolbars",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["appearance"]
    },
    "method:FormItemIcon.keyPress":{
        ref:"method:FormItemIcon.keyPress",
        flags:"",
        description:"      StringMethod action to fire when this icon has focus and recieves a keypress\n      event.\n      If unset the form item's <code>iconKeyPress</code> method will be fired instead \n      (if specified).\n",
        name:"keyPress",
        seeAlso:[],
        params:[
            {description:"Name of the key pressed", optional:false, name:"keyName", defaultValue:null,
             type:"string"},
            {description:"character produced by the keypress", optional:false, name:"character",
             defaultValue:null, type:"character"},
            {description:"The Dynamic Form to which this icon's item belongs.", optional:false,
             name:"form", defaultValue:null, type:"DynamicForm"},
            {description:"The Form Item containing this icon", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"A pointer to the form item icon", optional:false, name:"icon",
             defaultValue:null, type:"FormItemIcon"}
        ],
        type:"method",
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "method:Tree.hasLeaves":{
        ref:"method:Tree.hasLeaves",
        flags:"",
        description:"\n  Return whether this node has any children that are leaves.\n\n",
        name:"hasLeaves",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node has children that are leaves", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:ListGrid.childExpansionMode":{
        ref:"attr:ListGrid.childExpansionMode",
        flags:"IRWA",
        description:"For ${isc.DocUtils.linkForRef('type:ExpansionMode','expansionModes')} that show another grid or tree, what the \n child's expansionMode should be.\n <P>Default value <code>null</code> means no further expansion.",
        name:"childExpansionMode",
        seeAlso:[],
        valueType:"ExpansionMode",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:ListGridField.imageWidth":{
        ref:"attr:ListGridField.imageWidth",
        flags:"IRW",
        description:"Width of images shown for fieldTypes image and imageFile in this field.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n width.  For example, if <code>field.imageWidth</code> is \"logoWidth\",\n <code>record.logoWidth</code> will control the width of the image.",
        name:"imageWidth",
        seeAlso:[
            {ref:"attr:ListGrid.imageSize", description:null},
            {ref:"attr:ListGridField.imageSize", description:null},
            {ref:"attr:ListGridField.imageHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "group:frozenFields":{
        ref:"group:frozenFields",
        title:"Frozen Fields",
        refs:["method:ListGrid.freezeField", "method:ListGrid.unfreezeField", 
         "method:ListGrid.toggleFrozen","attr:ListGridField.frozen", "attr:ListGridField.canFreeze", 
         "attr:ListGrid.frozenBaseStyle","attr:ListGrid.shrinkForFreeze", "attr:ListGrid.frozenHeaderBaseStyle", 
         "attr:ListGrid.frozenHeaderTitleStyle","attr:ListGrid.canFreezeFields"],
        description:"Frozen fields are fields that do not scroll horizontally with other fields, remaining on\n the screen while other fields may be scrolled off.  This feature is typically used to\n allow basic identifying information (like an \"accountId\") to remain on screen while the\n user scrolls through a large number of related fields.\n <P>\n Fields can be programmatically frozen via setting\n ${isc.DocUtils.linkForRef('attr:ListGridField.frozen','field.frozen')} to true when the grid is created, or\n dynamically frozen and unfrozen via ${isc.DocUtils.linkForRef('method:ListGrid.freezeField','freezeField()')} and\n ${isc.DocUtils.linkForRef('method:ListGrid.unfreezeField','unfreezeField()')}.\n The setting ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','canFreezeFields')} enables a user interface to\n allow end users to dynamically freeze and unfreeze fields.\n <P>\n The frozen fields feature is not compatible with the following features:\n <ul>\n <li> variable height auto-sizing records\n      (${isc.DocUtils.linkForRef('attr:ListGrid.fixedRecordHeights','fixedRecordHeights:false')})\n <li> ${isc.DocUtils.linkForRef('group:autoFitData')}:\"horizontal\", as well as headers that autoFit to titles\n      (normally enabled via <code>field.overflow:\"visible\"</code>)\n <li> the ${isc.DocUtils.linkForRef('class:CubeGrid')} subclass of ListGrid\n <li> nested grids\n </ul>\n The frozen fields feature <b>is</b> compatible with column resize and reorder, selection\n and multi-selection, loading data on demand, inline editing, drag and drop and reorder\n of records, the ${isc.DocUtils.linkForRef('class:TreeGrid')} subclass of ListGrid, and all dynamic styling-related and\n formatting-related features.",
        name:"frozenFields",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.cancelEditingConfirmationMessage":{
        ref:"attr:ListGrid.cancelEditingConfirmationMessage",
        flags:"IRW",
        description:"If this is an editable listGrid, and <code>this.confirmCancelEditing</code> is true\n this property is used as the message to display in the confirmation dismissal prompt.",
        name:"cancelEditingConfirmationMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"Cancelling this edit will clear unsaved edit values for this record. Continue?",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing", "i18nMessages"]
    },
    "attr:ButtonItem.baseStyle":{
        ref:"attr:ButtonItem.baseStyle",
        flags:"IRW",
        description:"Optional <code>baseStyle</code> will be applied to the button.",
        name:"baseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ButtonItem",
        groups:["appearance"]
    },
    "attr:Canvas.shadowSoftness":{
        ref:"attr:Canvas.shadowSoftness",
        flags:"IRA",
        description:"Softness, or degree of blurring, of the shadow.\n <P>\n A shadow with <code>softness:x</code> is 2x pixels larger in each direction than the element\n throwing the shadow, and the media for each edge should be x pixels wide/tall.\n <P>\n Defaults to <code>shadowDepth</code> if unset.",
        name:"shadowSoftness",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('shadows')}"],
        definingClass:"class:Canvas",
        groups:["shadow"]
    },
    "method:Canvas.setTabIndex":{
        ref:"method:Canvas.setTabIndex",
        flags:"A",
        description:"  Assign an explicit tabIndex to this widget.\n",
        name:"setTabIndex",
        seeAlso:[
            {ref:"attr:Canvas.tabIndex", description:null}
        ],
        params:[
            {description:"New tabIndex for this widget. Must be less than                           ${isc.DocUtils.linkForRef('classAttr:Canvas.TAB_INDEX_FLOOR')} to avoid interfering with auto-assigned\n                          tab indices on the page.",
             optional:false, name:"tabIndex", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "method:Calendar.next":{
        ref:"method:Calendar.next",
        flags:"",
        description:" Move to the next day, week, or month, depending on which tab is selected.\n\n",
        name:"next",
        seeAlso:[],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:Menu.emptyMessage":{
        ref:"attr:Menu.emptyMessage",
        flags:"IRWA",
        description:"Message to show when a menu is shown with no items.",
        name:"emptyMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"[Empty menu]\"",
        type:"attr",
        definingClass:"class:Menu"
    },
    "attr:DetailViewerField.value":{
        ref:"attr:DetailViewerField.value",
        flags:"IR",
        description:"When a field specifies its ${isc.DocUtils.linkForRef('attr:DetailViewerField.type')} to be \"header\", the value of\n this attribute specifies the header text.",
        name:"value",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"undefined\"",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "class:CanvasItem":{
        methods:[],
        ref:"class:CanvasItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:CanvasItem.canvas", "attr:CanvasItem.canvasConstructor", 
         "attr:CanvasItem.canvasDefaults","attr:CanvasItem.canvasProperties"],
        description:"Form item which renders a Canvas inline in a DynamicForm instance.",
        name:"CanvasItem",
        seeAlso:[],
        type:"class"
    },
    "method:CubeGrid.facetValueSelectionChanged":{
        ref:"method:CubeGrid.facetValueSelectionChanged",
        flags:"",
        description:"\n      Handler/Notification function for facetValue selection change\n      (no default implementation)\n\n      <br><i>methodType</i> handler\n",
        name:"facetValueSelectionChanged",
        seeAlso:[],
        params:[
            {description:"facetValues with new selection state", optional:false, name:"facetValues",
             defaultValue:null, type:"facetValues object"},
            {description:"new selection state", optional:false, name:"newState", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:EdgedCanvas.edgeOffset":{
        ref:"attr:EdgedCanvas.edgeOffset",
        flags:"IRA",
        description:"Amount the contained Canvas should be offset.  Defaults to edgeSize; set to less than\n edgeSize to allow the contained Canvas to overlap the edge and corner media.",
        name:"edgeOffset",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:ListGrid.showSortArrow":{
        ref:"attr:ListGrid.showSortArrow",
        flags:"IRW",
        description:"Indicates whether a sorting arrow should appear for the listGrid, and its\n          location. See SortArrow type for details.<br>\n          Clicking the sort arrow reverses the direction of sorting for the current sort\n          column (if any), or sorts the listGrid by its first sortable column. The arrow\n          image on the button indicates the current direction of sorting.\n          If undefined, the sort arrow will show up in the sorted field, and the\n          corner sort button will be displayed if a vertical scrollbar is being displayed",
        name:"showSortArrow",
        seeAlso:[],
        valueType:"SortArrow",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["sorting", "appearance"]
    },
    "attr:TileLayout.animateTileChange":{
        ref:"attr:TileLayout.animateTileChange",
        flags:"IRWA",
        description:"If set, tiles animate to their new positions when a tile is added, removed, or reordered via\n drag and drop.",
        name:"animateTileChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["appearance"]
    },
    "attr:Layout.memberOverlap":{
        ref:"attr:Layout.memberOverlap",
        flags:"IR",
        description:"Number of pixels by which each member should overlap the preceding member, used for\n creating an \"stack of cards\" appearance for the members of a Layout.\n <P>\n <code>memberOverlap</code> can be used in conjunction with ${isc.DocUtils.linkForRef('attr:Layout.stackZIndex')} to create\n a particular visual stacking order.\n <P>\n Note that overlap of individual members can be accomplished with a negative setting for\n ${isc.DocUtils.linkForRef('attr:Canvas.extraSpace')}.",
        name:"memberOverlap",
        seeAlso:[],
        valueType:"positiveInteger",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMember"]
    },
    "method:ColorPicker.setHtmlColor":{
        ref:"method:ColorPicker.setHtmlColor",
        flags:"",
        description:" Changes the selected color to the one represented by the supplied HTML color \n string. Note that ths method only accepts the parameter if it represents a \n valid color (otherwise it is simply ignored).\n",
        name:"setHtmlColor",
        seeAlso:[
            {ref:"method:ColorPicker.getHtmlColor", description:null}
        ],
        params:[
            {description:"A string in HTML color representation format (#RRGGBB)", optional:false,
             name:"newValue", defaultValue:null, type:"text"}
        ],
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:Button.showDisabledIcon":{
        ref:"attr:Button.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "group:values":{
        ref:"group:values",
        refs:[],
        description:"Manipulating the values stored in the form.",
        name:"values",
        seeAlso:[],
        type:"group"
    },
    "attr:Menu.defaultWidth":{
        ref:"attr:Menu.defaultWidth",
        flags:"IRW",
        description:"The default menu width.",
        name:"defaultWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"150",
        type:"attr",
        definingClass:"class:Menu",
        groups:["sizing"]
    },
    "classAttr:RPCResponse.STATUS_CONNECTION_RESET_ERROR":{
        ref:"classAttr:RPCResponse.STATUS_CONNECTION_RESET_ERROR",
        flags:"R",
        description:"This response code only occurs when using the HTTP proxy.  It is issued by the proxy\n servlet when the attempt to contact the target server results in a Java SocketException.\n This response probably indicates that the target server is currently down.\n <p>\n Note that this status will be reported alongside an httpResponseCode of 592 - this is \n an artificial httpResponseCode used by the proxy to signal this particular condition;\n it is not a valid HTTP response code sent by a webserver.",
        name:"STATUS_CONNECTION_RESET_ERROR",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"-92",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:DSResponse.status":{
        ref:"attr:DSResponse.status",
        flags:"R",
        description:"Same meaning as ${isc.DocUtils.linkForRef('attr:RPCResponse.status')}, except DSResponses have additional error codes,\n such as ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_VALIDATION_ERROR','validation failure')}.",
        name:"status",
        seeAlso:[
            {ref:"group:dataSourceOperations",
             description:"for standard return types for DataSource Operations"}
        ],
        valueType:"Number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["errorHandling"]
    },
    "attr:DynamicForm.disableValidation":{
        ref:"attr:DynamicForm.disableValidation",
        flags:"IRW",
        description:"If set to true, client-side validators will not run on the form when validate() is\n called.  Server-side validatiors (if any) will still run on attempted save.",
        name:"disableValidation",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null},
            {ref:"method:DynamicForm.submit", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "class:HeaderItem":{
        methods:[],
        exampleConfig:"HeaderItem.xml",
        ref:"class:HeaderItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:HeaderItem.defaultValue", "attr:HeaderItem.showTitle", 
         "attr:HeaderItem.textBoxStyle","attr:HeaderItem.colSpan", "attr:HeaderItem.startRow", "attr:HeaderItem.endRow"],
        description:"FormItem for showing a header within a DynamicForm.\n <p>\n Set the <code>defaultValue</code> of this item to the HTML you want to embed in the form.",
        name:"HeaderItem",
        seeAlso:[],
        type:"class"
    },
    "method:ListGrid.refreshCell":{
        ref:"method:ListGrid.refreshCell",
        flags:"",
        description:" Refresh an individual cell without redrawing the grid.\n <P>\n The cell's value, CSS class, and CSS text will be refreshed, to the current values returned\n by getCellValue(), getCellStyle() and getCellCSSText() respectively.\n\n",
        name:"refreshCell",
        seeAlso:[
            {ref:"method:ListGrid.refreshCellStyle", description:"to update just styling"}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column number of cell to refresh", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('calculatedCellValue')}"],
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:ColorPicker.defaultColor":{
        ref:"attr:ColorPicker.defaultColor",
        flags:"IR",
        description:"The default color. This is the color that is selected when the picker first loads",
        name:"defaultColor",
        seeAlso:[],
        valueType:"text",
        defaultValue:"#808080",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:TreeGrid.openIconSuffix":{
        ref:"attr:TreeGrid.openIconSuffix",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:TreeGrid.showOpenIcons')} is true, this suffix will be appended to the\n ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')} for open folders in this grid.",
        name:"openIconSuffix",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"open\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:ResultSet.initialLength":{
        ref:"attr:ResultSet.initialLength",
        flags:"IA",
        description:"Initial value of the data set length.\n <P>\n To create a ResultSet with it's cache partly filled, see ${isc.DocUtils.linkForRef('attr:ResultSet.initialData')}.",
        name:"initialLength",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching", "cacheSync"]
    },
    "attr:Chart.facets":{
        ref:"attr:Chart.facets",
        flags:"IR",
        description:"An Array of facets, exactly analogous to ${isc.DocUtils.linkForRef('attr:CubeGrid.facets')}, except that:\n <ul>\n <li> the \"inlinedValues\" property can be set on a facet to change data representation as\n described under ${isc.DocUtils.linkForRef('attr:Chart.data')}.\n <li> for a non-inlined facet, Charts support auto-derivation of facetValues from the data.\n </ul>",
        name:"facets",
        seeAlso:[],
        valueType:"Array of Facet or Facet",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:Canvas.visibility":{
        flags:"IRW",
        getter:["isVisible"],
        type:"attr",
        ref:"attr:Canvas.visibility",
        description:"Controls widget visibility when the widget is initialized. See Visibility type for\n      details.",
        name:"visibility",
        setter:["show", "hide"],
        valueType:"Visibility",
        seeAlso:[],
        defaultValue:"Canvas.INHERIT",
        groups:["appearance"],
        definingClass:"class:Canvas"
    },
    "method:Canvas.mouseWheel":{
        ref:"method:Canvas.mouseWheel",
        flags:"A",
        description:"\n Executed when the mouse wheel is actuated.\n\n",
        name:"mouseWheel",
        seeAlso:[
            {ref:"classMethod:EventHandler.getWheelDelta", description:null},
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        platformNotes:"Not all end user systems have mouse wheels.",
        examples:["${isc.DocUtils.linkForExampleId('customMouseEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:ListGrid.getEditorValueMap":{
        ref:"method:ListGrid.getEditorValueMap",
        flags:"A",
        description:"\n  Returns the valueMap to display for a field when it is displayed in the editor while\n  editing some record.<br>\n  Called when a user starts to edit a field, or whenever the field valueMap is updated via\n  a call to ${isc.DocUtils.linkForRef('method:ListGrid.setValueMap')} or ${isc.DocUtils.linkForRef('method:ListGrid.setEditorValueMap')}.\n  Default implementation will return the <code>field.editorValueMap</code> if specified, otherwise\n  <code>field.valueMap</code> - can be overriden to provide a\n  different specific valueMap for some field based on the record/field data.\n\n",
        name:"getEditorValueMap",
        seeAlso:[],
        params:[
            {description:"field definition field for which we need a valueMap", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"Field values for record being edited. Note that this will include                              the current edit values for fields that have not yet been saved.\n                              May be null, if editing a new record.",
             optional:false, name:"values", defaultValue:null, type:"object"}
        ],
        returns:{description:"ValueMap for the edit field (or null if no valueMap required)", type:"ValueMap"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:CanvasItem.canvasProperties":{
        ref:"attr:CanvasItem.canvasProperties",
        flags:"IRW",
        description:"Properties to apply to this canvas on creation if this.canvas is not already a canvas \n instance.",
        name:"canvasProperties",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:CanvasItem"
    },
    "attr:Scrollbar.scrollTarget":{
        ref:"attr:Scrollbar.scrollTarget",
        flags:"IRWA",
        description:"The widget whose contents should be scrolled by this scrollbar. The scrollbar thumb\n is sized according to the amount of visible vs. scrollable content in this widget.",
        name:"scrollTarget",
        setter:["setScrollTarget"],
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Scrollbar"
    },
    "attr:DataSource.addGlobalId":{
        ref:"attr:DataSource.addGlobalId",
        flags:"IRA",
        description:"Whether to make this DataSource available as a global variable for convenience.",
        name:"addGlobalId",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DataSource",
        groups:["identity"]
    },
    "method:FormItem.formatEditorValue":{
        ref:"method:FormItem.formatEditorValue",
        flags:"",
        description:" An optional ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')} to map this item's current data value \n to a display value.\n <P>\n Note that this only applies to items which show a freeform entry area, such as a\n ${isc.DocUtils.linkForRef('class:TextItem')} or ${isc.DocUtils.linkForRef('class:TextAreaItem')}. For display values which will not be directly \n manipulated by the user, use ${isc.DocUtils.linkForRef('method:FormItem.formatValue')} instead.\n <P>\n See also ${isc.DocUtils.linkForRef('method:FormItem.parseEditorValue')}.\n\n",
        name:"formatEditorValue",
        seeAlso:[],
        params:[
            {description:"Underlying data value to format. May be null.", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"The record currently being edited by this form.      Essentially the form's current values object.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"pointer to the DynamicForm", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"},
            {description:"pointer to the FormItem", optional:false, name:"item", defaultValue:null,
             type:"FormItem"}
        ],
        returns:{description:"display value to show in the editor.", type:"string"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:FormItem.validators":{
        ref:"attr:FormItem.validators",
        flags:"IR",
        description:"Validators for this form item.  \n <P>\n <b>Note:</b> these validators will only be run on the client; to\n do real client-server validation, validators must be specified on the DataSource.",
        name:"validators",
        seeAlso:[],
        valueType:"Array of Validator",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:ListGrid.setSelectionType":{
        ref:"method:ListGrid.setSelectionType",
        flags:"A",
        description:" Changes selectionType on the fly.\n",
        name:"setSelectionType",
        seeAlso:[],
        params:[
            {description:"New selection style.", optional:false, name:"selectionType",
             defaultValue:null, type:"SelectionStyle"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DatabaseBrowser.dbName":{
        ref:"attr:DatabaseBrowser.dbName",
        flags:"IR",
        description:"The name of the database configuration to use.  Database configurations are set up \n through the \"Databases\" tab of the Developer Console.  If not set, defaults to the \n current default database configuration.\n <p>\n Note that this property only applies where ${isc.DocUtils.linkForRef('attr:DatabaseBrowser.serverType')} is \"sql\"",
        name:"dbName",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:Img.showTitle":{
        ref:"attr:Img.showTitle",
        flags:"IRWA",
        description:"Determines whether any specified ${isc.DocUtils.linkForRef('method:StatefulCanvas.getTitle','title')} will be \n displayed for this component.<br>\n Applies to Image-based components only, where the title will be rendered out in a label\n floating over the component",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Img"
    },
    "attr:DataSource.serverConstructor":{
        ref:"attr:DataSource.serverConstructor",
        flags:"IR",
        description:"Specifies the fully-qualified name of the DataSource subclass that should be instantiated\n server-side for this dataSource. This property allows you to write and use custom DataSource \n subclasses on the server.\n <p>\n One reason you might wish to do this would be to override the validate() method to provide \n some arbitrary custom validation (such as complex database lookups, validation embedded in \n legacy applications, etc).  It is also possible - though obviously a more substantial task -\n to override the execute() method in your custom DataSource.  This is one way of creating \n a completely customized DataSource implementation.\n <p>\n <b>Note:</b> If you use this property, you are responsible for making sure that it refers to\n a valid server-side class that extends <code>com.isomorphic.datasource.BasicDataSource</code>.\n If your implementation relies on methods or state only present in certain specialized \n subclasses of DataSource (for example, you want the normal behavior and features of a \n HibernateDataSource, but with a specialized validate() method), then you shoud extend the \n subclass rather than the base class.",
        name:"serverConstructor",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:DynamicForm.setFieldErrors":{
        ref:"method:DynamicForm.setFieldErrors",
        flags:"",
        description:" Set field validation error[s] for some field.<br>\n The errors parameter may be passed in as a string (a single error message), or an array of\n strings.<br>\n The showErrors parameter allows the errors to be displayed immediately. Alternatively, an\n explicit call to ${isc.DocUtils.linkForRef('method:DynamicForm.showFieldErrors')} will display the errors for this field.\n",
        name:"setFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to apply the new errors to", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"errors to apply to the field in question", optional:false, name:"errors",
             defaultValue:null, type:"string | array of strings"},
            {description:"If true this method will fall through to ${isc.DocUtils.linkForRef('method:DynamicForm.showFieldErrors')}  to update the display",
             optional:false, name:"show", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "attr:ListGridField.optionOperationId":{
        ref:"attr:ListGridField.optionOperationId",
        flags:"IR",
        description:"If this field has an optionDataSource specified and \n ${isc.DocUtils.linkForRef('attr:ListGridField.autoFetchDisplayMap','autoFetchDisplayMap')} is set, this attribute\n provides a way to customize the ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} passed to \n <code>dataSource.fetchData()</code> when retrieving the display map from the option \n dataSource.",
        name:"optionOperationId",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "classAttr:Dialog.CONFIRM_TITLE":{
        ref:"classAttr:Dialog.CONFIRM_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed in response to the ${isc.DocUtils.linkForRef('classMethod:isc.confirm')} method.\n Note that a custom title can be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"CONFIRM_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Confirm\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:Canvas.destroyed":{
        ref:"attr:Canvas.destroyed",
        flags:"RA",
        description:"If this property is set to <code>true</code>, the ${isc.DocUtils.linkForRef('method:Canvas.destroy','destroy()')} method\n has been called on this canvas. This implies the canvas is no longer valid. Its ID has been\n removed from global scope, and calling standard canvas APIs on it is likely to result in \n errors.",
        name:"destroyed",
        seeAlso:[
            {ref:"method:Canvas.destroy", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "attr:ListGrid.sorterConstructor":{
        ref:"attr:ListGrid.sorterConstructor",
        flags:"IR",
        description:"Widget class for the corner sort button, if showing. For consistent appearance, this\n is usually set to match ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor')}",
        name:"sorterConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"Button",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:TileLayout.tileSize":{
        ref:"attr:TileLayout.tileSize",
        flags:"IR",
        description:"Size of each tile in pixels.  Depending on the ${isc.DocUtils.linkForRef('type:LayoutPolicy')}, <code>tileSize</code>\n may be taken as a maximum, minimum or exact size of tiles, or may be irrelevant.\n <P>\n Width and height may be separately set via ${isc.DocUtils.linkForRef('attr:TileLayout.tileHeight')} and ${isc.DocUtils.linkForRef('attr:TileLayout.tileWidth')}.",
        name:"tileSize",
        setter:["setTileSize"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"50",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["sizing"]
    },
    "attr:ImgButton.iconWidth":{
        ref:"attr:ImgButton.iconWidth",
        flags:"IR",
        description:"Width in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "method:Canvas.willAcceptDrop":{
        ref:"method:Canvas.willAcceptDrop",
        flags:"A",
        description:"\n Returns true if the widget object being dragged can be dropped on this widget, and\n false otherwise.  The default implementation of this method simply compares the\n ${isc.DocUtils.linkForRef('attr:Canvas.dragType')} of the <code>dragTarget</code> (the component being dragged from)\n with the list of ${isc.DocUtils.linkForRef('attr:Canvas.dropTypes')} on this Canvas.  If the ${isc.DocUtils.linkForRef('attr:Canvas.dropTypes')}\n list contains the ${isc.DocUtils.linkForRef('attr:Canvas.dragType')} value, then this method returns true.  Otherwise it\n returns false.\n\n",
        name:"willAcceptDrop",
        seeAlso:[
            {ref:"attr:Canvas.dragType", description:null},
            {ref:"attr:Canvas.dropTypes", description:null},
            {ref:"attr:Canvas.dragTarget", description:null},
            {ref:"method:Canvas.drop", description:null}
        ],
        returns:{description:"true if the widget object being dragged can be dropped on this widget,                      false otherwise",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:SelectItem.pickerIconSrc":{
        ref:"attr:SelectItem.pickerIconSrc",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:SelectItem.showPickerIcon')} is true for this item, this property governs the\n src of the picker icon image to be displayed.",
        name:"pickerIconSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/SelectItem_PickButton_icon.gif\"",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["pickerIcon"]
    },
    "attr:ListGrid.headerMenuButtonIcon":{
        ref:"attr:ListGrid.headerMenuButtonIcon",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, this property governs the icon shown on the\n auto-generated <code>headerMenuButton</code>",
        name:"headerMenuButtonIcon",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"[SKIN]/ListGrid/sort_descending.gif\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "method:GridRenderer.cellDoubleClick":{
        ref:"method:GridRenderer.cellDoubleClick",
        flags:"A",
        description:" Called when a cell receives a double click event.\n\n",
        name:"cellDoubleClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "classAttr:ValidatorTypes.integerRange":{
        ref:"classAttr:ValidatorTypes.integerRange",
        flags:"IR",
        description:"Tests whether the value for this field is a whole number within the range \n  specified.  The <code>max</code> and <code>min</code> properties on the validator\n  are used to determine the acceptable range.",
        name:"integerRange",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        examples:["${isc.DocUtils.linkForExampleId('validationBuiltins')}"],
        definingClass:"class:ValidatorTypes"
    },
    "attr:EdgedCanvas.edgeOffsetLeft":{
        ref:"attr:EdgedCanvas.edgeOffsetLeft",
        flags:"IRA",
        description:"Amount the contained Canvas should be offset from the left.  Defaults to the size for\n the left edge.  Set smaller to allow the contained Canvas to overlap the edge and\n corner media.",
        name:"edgeOffsetLeft",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:MenuButton.height":{
        ref:"attr:MenuButton.height",
        flags:"IRW",
        description:"Default height of the button.",
        name:"height",
        seeAlso:[],
        valueType:"Number or String",
        defaultValue:"22",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "group:baseLine":{
        ref:"group:baseLine",
        refs:["attr:TabBar.baseLineThickness", "attr:TabBar.baseLineSrc", "attr:TabBar.baseLineCapSize"],
        description:"The baseLine is StretchImg that is placed along the edge of the TabBar that borders on\n the pane, occluding the pane's actual border but matching it exactly.  The selected tab\n is in front of the baseLine, and the rest are behind it.",
        name:"baseLine",
        seeAlso:[],
        type:"group"
    },
    "method:ColumnTree.getColumnTitle":{
        ref:"method:ColumnTree.getColumnTitle",
        flags:"",
        description:" Returns the title to show for the header of indicated column.  Only called if\n ${isc.DocUtils.linkForRef('method:ColumnTree.shouldShowHeader')} returns true for this column.\n <P>\n By default, returns ${isc.DocUtils.linkForRef('attr:ColumnTree.firstColumnTitle')} for the first column, and for subsequent\n columns, the result of ${isc.DocUtils.linkForRef('method:Tree.getTitle','this.data.getTitle()')} called on the\n <code>node</code> passed to this function.\n \n",
        name:"getColumnTitle",
        seeAlso:[],
        params:[
            {description:"parent node for the nodes to be shown in the column", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"index of the column", optional:false, name:"colNum", defaultValue:null,
             type:"int"}
        ],
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "attr:RPCRequest.timeout":{
        ref:"attr:RPCRequest.timeout",
        flags:"IRWA",
        description:"Overrides RPCManager.defaultTimeout for this request only.  If you're using queuing, note\n that the timeout setting derived from the last request in the queue is used for the entire\n queue.  If you want to override the timeout for the queue, make sure to set your override at\n least on the last request in the queue.",
        name:"timeout",
        seeAlso:[
            {ref:"classAttr:RPCManager.defaultTimeout", description:null}
        ],
        valueType:"number",
        defaultValue:"RPCManager.defaultTimeout",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:Layout.addMembers":{
        ref:"method:Layout.addMembers",
        flags:"",
        description:"\n  Add one or more canvases to the layout, optionally at specific positions.\n\n  NOTE: depending on the layout policy, adding a new member may cause existing members to\n  resize.\n\n",
        name:"addMembers",
        seeAlso:[],
        params:[
            {description:"array of canvases to be added, or reference to single                                          canvas",
             optional:false, name:"newMembers", defaultValue:null, type:"Array or object"},
            {description:"position to add newMembers; if omitted newMembers                                          will be added at the last position",
             optional:true, name:"position", defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:Slider.thumbThickWidth":{
        ref:"attr:Slider.thumbThickWidth",
        flags:"IRW",
        description:"The dimension of the thumb perpendicular to the slider track.",
        name:"thumbThickWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"23",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:CheckboxItem.valueIconWidth":{
        ref:"attr:CheckboxItem.valueIconWidth",
        flags:"IRW",
        description:"Width of the checkbox image.",
        name:"valueIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"13",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["valueIcons"]
    },
    "attr:TreeGrid.folderClosedImage":{
        ref:"attr:TreeGrid.folderClosedImage",
        deprecated:"as part of SmartClient release 5.5 in favor of ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')}",
        flags:"IRW",
        description:"The filename of the default icon for all closed folder nodes in this treeGrid. Use\n the node.icon property (null by default) to specify a custom image for an individual\n folder node. The same custom image will be used for both the open and closed folder\n images.",
        name:"folderClosedImage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "attr:GridRenderer.totalRows":{
        ref:"attr:GridRenderer.totalRows",
        flags:"IRW",
        getter:["getTotalRows"],
        description:"Total number of rows in the grid.<br><br>\n\n NOTE: in order to create a valid grid, you must either provide a totalRows value or implement\n getTotalRows()",
        name:"totalRows",
        seeAlso:[
            {ref:"method:GridRenderer.getTotalRows", description:null}
        ],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:GridRenderer"
    },
    "method:TileGrid.getTile":{
        ref:"method:TileGrid.getTile",
        flags:"",
        description:" Returns the tile for the passed record or record index.\n <P>\n If ${isc.DocUtils.linkForRef('attr:TileGrid.showAllRecords')} is false, this may return null.\n",
        name:"getTile",
        seeAlso:[],
        params:[
            {description:"record or index of record in this.data", optional:false, name:"tile",
             defaultValue:null, type:"TileRecord or int"}
        ],
        returns:{description:"tile for this record", type:"Canvas"},
        type:"method",
        definingClass:"class:TileGrid"
    },
    "method:DataBoundComponent.getRecordIndex":{
        ref:"method:DataBoundComponent.getRecordIndex",
        flags:"",
        description:" Get the index of the provided record.\n <P>\n Override in subclasses to provide more specific behaviour, for instance, when data holds a\n large number of records\n\n",
        name:"getRecordIndex",
        seeAlso:[],
        params:[
            {description:"the record whose index is to be retrieved", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"indexindex of the record, or -1 if not found", type:"Number"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "method:ListGrid.getDropIndex":{
        ref:"method:ListGrid.getDropIndex",
        flags:"",
        description:" Return the drop-index for a given row and reorderPosition.\n <P>\n When there are no rows in the grid, getDropIndex() returns zero.  \n <P>\n If parameter <i>recordNum</i> is not passed, the current event row is used, see\n ${isc.DocUtils.linkForRef('method:ListGrid.getEventRow')}. \n <P>\n Parameter ${isc.DocUtils.linkForRef('type:ReorderPosition','reorderPosition')} indicates where the drop-item should appear \n in relation to the row at index <i>recordNum</i>.  If no reorderPosition is provided, it is \n calculated based on the physical position of the mouse in the drop-target row when the drop \n occurs; if the mouse is in the top half of a row, the drop-index is before  \n <i>recordNum</i>.  Otherwise, the drop-index is after <i>recordNum</i>.\n\n",
        name:"getDropIndex",
        seeAlso:[],
        params:[
            {description:"Index of the record to drop onto", optional:true, name:"recordNum",
             defaultValue:null, type:"number"},
            {description:"Where to drop in relation to <i>recordNum</i>", optional:true,
             name:"reorderPosition", defaultValue:null, type:"ReorderPosition"}
        ],
        returns:{description:"The calculated drop-index", type:"number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:ListGrid.getDragTrackerTitle":{
        ref:"method:ListGrid.getDragTrackerTitle",
        flags:"",
        description:" Return \"title\" HTML to display as a drag tracker when the user drags some record.<br>\n Default implementation will display the cell value for the title field (see \n ${isc.DocUtils.linkForRef('method:ListGrid.getTitleField')}) for the record(s) being dragged (including any\n icons / custom formatting / styling, etc).\n <p>\n Note: Only called if ${isc.DocUtils.linkForRef('attr:ListGrid.dragTrackerMode')} is set to <code>\"title\"</code>.\n",
        name:"getDragTrackerTitle",
        seeAlso:[],
        params:[
            {description:"First selected record being dragged", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"row index of first record being dragged", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Title for the row. Default implementation looks at the value of the                  title-field cell for the row.",
         type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dragTracker"]
    },
    "method:ListGrid.getRequiredFieldMessage":{
        ref:"method:ListGrid.getRequiredFieldMessage",
        flags:"",
        description:" Returns the message to display when a user attempts to save a required field with an\n empty value.  By default returns ${isc.DocUtils.linkForRef('classAttr:Validator.requiredField')}.\n",
        name:"getRequiredFieldMessage",
        seeAlso:[],
        params:[
            {description:"definition of the field being edited", optional:false, name:"field",
             defaultValue:null, type:"object"},
            {description:"record object being edited", optional:false, name:"record", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"\"Field is required\"", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["i18nMessages", "gridValidation"]
    },
    "attr:FormItem.title":{
        ref:"attr:FormItem.title",
        flags:"IRW",
        getter:["getTitle"],
        description:"User visible title for this form item.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["basics"]
    },
    "method:FormItem.stopHover":{
        ref:"method:FormItem.stopHover",
        flags:"A",
        description:" This method is fired when the user rolls off this item (or the title for this item) and\n will clear any hover canvas shown by the item.\n",
        name:"stopHover",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:Button.title":{
        flags:"IRW",
        getter:["getTitle"],
        type:"attr",
        ref:"attr:Button.title",
        description:"The text title to display in this button.",
        name:"title",
        setter:["setTitle"],
        valueType:"string",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonStates')}"],
        defaultValue:"varies",
        groups:["basics", "i18nMessages"],
        definingClass:"class:Button"
    },
    "method:ListGrid.getCellRecord":{
        ref:"method:ListGrid.getCellRecord",
        flags:"A",
        description:" Return the pointer to a particular record by record number.<br>\n Notes:<br>\n - If this is a databound grid, and the record for some row has not yet been loaded, \n   returns the ${isc.DocUtils.linkForRef('classAttr:Array.LOADING')} marker, and a fetch will be initialized to retrieve the\n   record from the server.<br>\n - If this is a new row in editable ListGrid, and has not yet been saved, this method will\n   return null.\n",
        name:"getCellRecord",
        seeAlso:[
            {ref:"method:ListGrid.getRecord", description:null},
            {ref:"method:ListGrid.getEditedRecord", description:null}
        ],
        params:[
            {description:"row index of record to return.", optional:false, name:"recordNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"Record object for the row.", type:"ListGridRecord"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Validator.errorMessage":{
        ref:"attr:Validator.errorMessage",
        flags:"IR",
        description:"Text to display if the value does not pass this validation check.\n <P>\n If unspecified, default error messages exist for all built-in validators, and a generic\n message will be used for a custom validator that is not passed.",
        name:"errorMessage",
        seeAlso:[],
        valueType:"errorMessage",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('conditionallyRequired')}"],
        definingClass:"class:Validator"
    },
    "attr:Calendar.invalidDateMessage":{
        ref:"attr:Calendar.invalidDateMessage",
        flags:"IR",
        description:"The message to display in the ${isc.DocUtils.linkForRef('attr:Calendar.eventEditor')} when the 'To' date is greater than\n the 'From' date and a save is attempted.",
        name:"invalidDateMessage",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"\"From must be before To\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "type:ExpansionMode":{
        ref:"type:ExpansionMode",
        values:[
            {description:"Show a single field's value in an ${isc.DocUtils.linkForRef('class:HTMLFlow')}. Field  to use is ${isc.DocUtils.linkForRef('attr:ListGrid.detailField')}.",
             value:"\"detailField\""},
            {description:"Show a ${isc.DocUtils.linkForRef('class:DetailViewer')} displaying those fields from the  record which are not already displayed in the grid.",
             value:"\"details\""},
            {description:"Show a separate ${isc.DocUtils.linkForRef('class:ListGrid')} containing related-records. See ${isc.DocUtils.linkForRef('attr:ListGridRecord.detailDS')} and ${isc.DocUtils.linkForRef('attr:ListGrid.recordDetailDSProperty')} for more \n information.",
             value:"\"related\""},
            {description:"Show a ${isc.DocUtils.linkForRef('class:DynamicForm')} to edit those fields from the record which not already present in the grid.",
             value:"\"editor\""},
            {description:"Show a ${isc.DocUtils.linkForRef('class:DetailViewer')} displaying those fields from the record not already displayed in the grid, together with a separate\n ${isc.DocUtils.linkForRef('class:ListGrid')} containing related-records.",
             value:"\"detailRelated\""}
        ],
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords','canExpandRecords')} is true, ExpansionMode\n  dictates the type of UI to be displayed in the expanded portion of the row.\n <P>There are a number of builtin ExpansionModes and you can override \n ${isc.DocUtils.linkForRef('method:ListGrid.getExpansionComponent','getExpansionComponent()')} to create your own\n expansion behaviors.",
        name:"ExpansionMode",
        seeAlso:[],
        type:"type",
        groups:["expansionField"]
    },
    "attr:Tree.parentIdField":{
        ref:"attr:Tree.parentIdField",
        flags:"IRA",
        description:"For trees with modelType \"parent\", this property specifies the name of the property\n that contains the unique parent ID of a node.  Default value is \"parentId\".  See\n ${isc.DocUtils.linkForRef('attr:TreeNode.parentId')} for usage.",
        name:"parentIdField",
        seeAlso:[
            {ref:"attr:TreeNode.parentId", description:null}
        ],
        valueType:"string",
        defaultValue:"\"parentId\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "class:FormItem":{
        methods:["method:FormItem.getFieldName", "method:FormItem.getDataPath", "method:FormItem.getTitle", 
         "method:FormItem.getIconHeight","method:FormItem.getIconWidth", "method:FormItem.setLeft", "method:FormItem.setTop", 
         "method:FormItem.updateState","method:FormItem.getErrorHTML", "method:FormItem.isDrawn", "method:FormItem.showPicker", 
         "method:FormItem.show","method:FormItem.hide", "method:FormItem.getDisplayValue", "method:FormItem.formatValue", 
         "method:FormItem.formatEditorValue","method:FormItem.parseEditorValue", "method:FormItem.setValueMap", 
         "method:FormItem.setValueIcons","method:FormItem.getValueFieldName", "method:FormItem.getDisplayFieldName", 
         "method:FormItem.getOptionDataSource","method:FormItem.setValue", "method:FormItem.clearValue", "method:FormItem.setHint", 
         "method:FormItem.setHintStyle","method:FormItem.getValue", "method:FormItem.getCriterion", "method:FormItem.stopHover", 
         "method:FormItem.itemHover","method:FormItem.titleHover", "method:FormItem.titleHoverHTML", 
         "method:FormItem.itemHoverHTML","method:FormItem.setDisabled", "method:FormItem.isDisabled", "method:FormItem.enable", 
         "method:FormItem.disable","method:FormItem.focusInItem", "method:FormItem.blurItem", "method:FormItem.showIf", 
         "method:FormItem.defaultDynamicValue","method:FormItem.focus", "method:FormItem.blur", "method:FormItem.click", 
         "method:FormItem.doubleClick","method:FormItem.iconClick", "method:FormItem.iconKeyPress", "method:FormItem.change", 
         "method:FormItem.changed","method:FormItem.transformInput", "method:FormItem.keyPress", "method:FormItem.keyDown", 
         "method:FormItem.keyUp","method:FormItem.getValueIcon"],
        ref:"class:FormItem",
        classAttrs:[],
        classMethods:["classMethod:FormItem.create"],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:FormItem.type", "attr:FormItem.editorType", "attr:FormItem.name", 
         "attr:FormItem.dataPath","attr:FormItem.title", "attr:FormItem.defaultValue", "attr:FormItem.ID", 
         "attr:FormItem.emptyDisplayValue","attr:FormItem.valueMap", "attr:FormItem.optionDataSource", 
         "attr:FormItem.optionFilterContext","attr:FormItem.optionCriteria", "attr:FormItem.optionOperationId", 
         "attr:FormItem.valueField","attr:FormItem.displayField", "attr:FormItem.fetchMissingValues", 
         "attr:FormItem.filterLocally","attr:FormItem.dateFormatter", "attr:FormItem.timeFormatter", "attr:FormItem.valueIcons", 
         "attr:FormItem.emptyValueIcon","attr:FormItem.showValueIconOnly", "attr:FormItem.suppressValueIcon", 
         "attr:FormItem.valueIconWidth","attr:FormItem.valueIconHeight", "attr:FormItem.valueIconSize", 
         "attr:FormItem.valueIconLeftPadding","attr:FormItem.valueIconRightPadding", "attr:FormItem.imageURLPrefix", 
         "attr:FormItem.imageURLSuffix","attr:FormItem.form", "attr:FormItem.showPickerIcon", 
         "attr:FormItem.showFocusedPickerIcon","attr:FormItem.pickerIconDefaults", "attr:FormItem.pickerIconProperties", 
         "attr:FormItem.pickerIconSrc","attr:FormItem.pickerIconWidth", "attr:FormItem.pickerIconHeight", "attr:FormItem.picker", 
         "attr:FormItem.pickerConstructor","attr:FormItem.pickerProperties", "attr:FormItem.validators", "attr:FormItem.required", 
         "attr:FormItem.visible","attr:FormItem.disabled", "attr:FormItem.accessKey", "attr:FormItem.tabIndex", 
         "attr:FormItem.globalTabIndex","attr:FormItem.selectOnFocus", "attr:FormItem.showTitle", "attr:FormItem.titleOrientation", 
         "attr:FormItem.titleAlign","attr:FormItem.wrapTitle", "attr:FormItem.validateOnChange", 
         "attr:FormItem.rejectInvalidValueOnChange","attr:FormItem.width", "attr:FormItem.height", "attr:FormItem.colSpan", 
         "attr:FormItem.rowSpan","attr:FormItem.startRow", "attr:FormItem.endRow", "attr:FormItem.align", 
         "attr:FormItem.textAlign","attr:FormItem.left", "attr:FormItem.top", "attr:FormItem.icons", 
         "attr:FormItem.defaultIconSrc","attr:FormItem.showOverIcons", "attr:FormItem.showFocusedIcons", 
         "attr:FormItem.iconVAlign","attr:FormItem.iconHeight", "attr:FormItem.iconWidth", "attr:FormItem.prompt", 
         "attr:FormItem.iconPrompt","attr:FormItem.showIcons", "attr:FormItem.errorIconHeight", "attr:FormItem.errorIconWidth", 
         "attr:FormItem.errorIconSrc","attr:FormItem.showErrorIcon", "attr:FormItem.showErrorText", 
         "attr:FormItem.showErrorStyle","attr:FormItem.errorOrientation", "attr:FormItem.hint", "attr:FormItem.showHint", 
         "attr:FormItem.showFocused","attr:FormItem.showDisabled", "attr:FormItem.cellStyle", "attr:FormItem.hintStyle", 
         "attr:FormItem.titleStyle","attr:FormItem.textBoxStyle", "attr:FormItem.cellClassName", 
         "attr:FormItem.errorCellClassName","attr:FormItem.titleClassName", "attr:FormItem.titleErrorClassName", 
         "attr:FormItem.hintClassName","attr:FormItem.hoverDelay", "attr:FormItem.hoverWidth", "attr:FormItem.hoverHeight", 
         "attr:FormItem.hoverAlign","attr:FormItem.hoverVAlign", "attr:FormItem.hoverStyle", "attr:FormItem.hoverOpacity", 
         "attr:FormItem.operator","attr:FormItem.criteriaField", "attr:FormItem.errorMessageWidth", 
         "attr:FormItem.shouldSaveValue"],
        description:"An Item that can participate in a DynamicForm, managing some value.\n <P>\n FormItems are never directly created, instead, FormItem descriptors are passed to a\n DynamicForm.  See the ${isc.DocUtils.linkForRef('class:DynamicForm')} documentation for details.",
        name:"FormItem",
        seeAlso:[],
        type:"class"
    },
    "method:Canvas.getVisibleWidth":{
        ref:"method:Canvas.getVisibleWidth",
        flags:"A",
        description:"      Return the visible width of the Canvas.\n\n",
        name:"getVisibleWidth",
        seeAlso:[],
        returns:{description:"visible width in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Class.logIsInfoEnabled":{
        ref:"method:Class.logIsInfoEnabled",
        flags:"",
        description:" Check whether a message logged at \"info\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsInfoEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:OperationBinding.allowMultiUpdate":{
        ref:"attr:OperationBinding.allowMultiUpdate",
        flags:"IR",
        description:"Ordinarily, \"update\" and \"remove\" operations are only allowed for ${isc.DocUtils.linkForRef('class:DataSource')}s\n that have a ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')}, and all primary key values\n are present in the request.  This is because an update of a DataSource with no primary key, \n or an update request that has missing primary key values, cannot be guaranteed to affect \n only one record.\n <p>\n Setting this property on an operationBinding circumvents this restriction for that operation\n only.\n <p>\n <b>Warning:</b> Be aware that this is a potentially dangerous setting and should be used\n with care.  With this flag set, you have no guarantee that an update will not change or \n remove every row in a table.",
        name:"allowMultiUpdate",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:OperationBinding"
    },
    "group:dynamicString":{
        ref:"group:dynamicString",
        refs:[],
        name:"dynamicString",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.setContents":{
        ref:"method:Canvas.setContents",
        flags:"",
        description:" Changes the contents of a widget to newContents, an HTML string.\n <P>\n When ${isc.DocUtils.linkForRef('attr:Canvas.dynamicContents','dynamicContents')} is set, <code>setContents()</code> can\n also be called with no arguments to cause contents to be re-evaluated.\n\n",
        name:"setContents",
        seeAlso:[],
        params:[
            {description:"an HTML string to be set as the contents of this widget", optional:true,
             name:"newContents", defaultValue:null, type:"string"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('setContents')}"],
        definingClass:"class:Canvas"
    },
    "method:ListGridField.getCellValue":{
        ref:"method:ListGridField.getCellValue",
        deprecated:"As of SmartClient 5.5, use ${isc.DocUtils.linkForRef('method:ListGridField.formatCellValue')}.",
        flags:"",
        description:" A stringMethod which returns the cell value to display for this field for some record.\n If defined, called by ListGrid.getCellValue().  Called in the scope of the field object.\n\n Deprecated as of Jan 12 05 in favor of field.formatCellValue, because 'getCellValue()'\n is a lower-level API which handles (for example) returning the HTML for editors within\n the cell.\n  \n",
        name:"getCellValue",
        seeAlso:[
            {ref:"method:ListGrid.getCellValue", description:null},
            {ref:"method:ListGridField.formatCellValue", description:null}
        ],
        params:[
            {description:"the ListGrid for which we're returning a cellValue", optional:false,
             name:"viewer", defaultValue:null, type:"ListGrid"},
            {description:"the current record object", optional:false, name:"record", defaultValue:null,
             type:"object"},
            {description:"row-index of the current record", optional:false, name:"recordNum",
             defaultValue:null, type:"nubmer"},
            {description:"current field object", optional:false, name:"field", defaultValue:null,
             type:"listGridField"},
            {description:"column-index of the current field", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"},
            {description:"unformatted value for this field, determined via                           ListGrid.getRawCellValue()",
             optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"value to display in the ListGrid cell", type:"any"},
        type:"method",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "type:VerticalAlignment":{
        ref:"type:VerticalAlignment",
        values:[
            {description:"At the top of the container", value:"Canvas.TOP"},
            {description:"Center within container.", value:"Canvas.CENTER"},
            {description:"At the bottom of the container", value:"Canvas.BOTTOM"}
        ],
        description:null,
        name:"VerticalAlignment",
        seeAlso:[],
        type:"type",
        groups:["appearance"]
    },
    "method:FormItem.getErrorHTML":{
        ref:"method:FormItem.getErrorHTML",
        flags:"A",
        description:" Output the HTML for an error message in a form element. Default behavior respects\n ${isc.DocUtils.linkForRef('attr:FormItem.showErrorIcon')} and ${isc.DocUtils.linkForRef('attr:FormItem.showErrorText')} as described in the\n documentation for those attributes.\n",
        name:"getErrorHTML",
        seeAlso:[],
        params:[
            {description:"error message string or array of error messages", optional:false,
             name:"error", defaultValue:null, type:"string | array"}
        ],
        returns:{description:"HTML to display the error", type:"HTML"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "object:animationShowEffect":{
        methods:[],
        ref:"object:animationShowEffect",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:["attr:animationShowEffect.effect", "attr:animationShowEffect.startFrom", 
         "attr:animationShowEffect.endsAt"],
        description:"Configuration object for effect to apply during an animated show or hide.",
        name:"animationShowEffect",
        seeAlso:[],
        type:"object"
    },
    "attr:SortSpecifier.normalizer":{
        ref:"attr:SortSpecifier.normalizer",
        flags:"IR",
        description:"A normalizer function which this sortSpecifier will use to sort.",
        name:"normalizer",
        seeAlso:[],
        valueType:"Function",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SortSpecifier"
    },
    "attr:SelectItem.optionFilterContext":{
        ref:"attr:SelectItem.optionFilterContext",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, and this property is\n not null, this will be passed to the datasource as ${isc.DocUtils.linkForRef('class:DSRequest')} properties when\n performing the filter operation on the dataSource to obtain the set of options for the\n list.",
        name:"optionFilterContext",
        seeAlso:[],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:ListGridField.canSortClientOnly":{
        ref:"attr:ListGridField.canSortClientOnly",
        flags:"IRW",
        description:"When true, this field can only be used for sorting if the data is entirely client-side.",
        name:"canSortClientOnly",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "group:gridValidation":{
        ref:"group:gridValidation",
        refs:["method:ListGrid.validateRow", "method:ListGrid.validateCell", 
         "method:ListGrid.getRequiredFieldMessage","method:ListGrid.hasErrors", "method:ListGrid.rowHasErrors", 
         "method:ListGrid.cellHasErrors","method:ListGrid.getRowErrors", "method:ListGrid.getCellErrors", 
         "method:ListGrid.setFieldError","method:ListGrid.setRowErrors", "method:ListGrid.clearFieldError", 
         "attr:ListGridField.validators","attr:ListGridField.validateOnChange", "attr:ListGridField.required", 
         "attr:ListGrid.validateByCell","attr:ListGrid.validateOnChange", "attr:ListGrid.neverValidate"],
        description:"ListGrids support automatic validation of edited cells / records. This group is a collection \n of APIs related to the validation subsystem.\n <p>\n Default validation occurs in response to the user navigating between edit cells \n (see ${isc.DocUtils.linkForRef('attr:ListGrid.validateByCell')}) or whenever edited values are to be committed to the\n server for saving.  Standard validation can also be triggered for a cell or row programatically at\n any time.<br>\n When standard validation occurs, ${isc.DocUtils.linkForRef('attr:ListGridField.validators')} will be run on each\n cell to be validated.<br>\n In addition to this standard behavior developers can add custom errors to fields\n via ${isc.DocUtils.linkForRef('method:ListGrid.setFieldError')} / ${isc.DocUtils.linkForRef('method:ListGrid.setFieldError')}.",
        name:"gridValidation",
        seeAlso:[
            {ref:"group:editing", description:null}
        ],
        type:"group"
    },
    "attr:CheckboxItem.showTitle":{
        ref:"attr:CheckboxItem.showTitle",
        flags:"IR",
        description:"CheckboxItem has special behavior for titles, see ${isc.DocUtils.linkForRef('attr:CheckboxItem.labelAsTitle')}.",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CheckboxItem"
    },
    "attr:SpinnerItem.min":{
        ref:"attr:SpinnerItem.min",
        flags:"IRW",
        description:"Minimum valid value for this item",
        name:"min",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('spinnerItem')}"],
        definingClass:"class:SpinnerItem"
    },
    "attr:RestDataSource.dataFormat":{
        ref:"attr:RestDataSource.dataFormat",
        flags:"IR",
        description:"Expected format for server responses. RestDataSources handle <code>\"json\"</code> and\n <code>\"xml\"</code> format responses by default. See class overview documentation for \n examples of responses in each format.",
        name:"dataFormat",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"xml\"",
        type:"attr",
        definingClass:"class:RestDataSource"
    },
    "attr:Scrollbar.allowThumbOverState":{
        ref:"attr:Scrollbar.allowThumbOverState",
        flags:"IRA",
        description:"If true, the thumb's appearance changes when the user rolls over it.",
        name:"allowThumbOverState",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "method:TabSet.getTabNumber":{
        ref:"method:TabSet.getTabNumber",
        flags:"",
        description:" Get the index of a tab, from the tab or tabID.  If passed a number, just returns it.\n",
        name:"getTabNumber",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"number | ID | tab"}
        ],
        returns:{description:"the index of the tab, or -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "group:rpcPrompt":{
        ref:"group:rpcPrompt",
        refs:["classAttr:RPCRequest.useCursorTracker", "classAttr:RPCManager.defaultPrompt", 
         "classAttr:RPCManager.promptStyle","classAttr:RPCManager.useCursorTracker", "classAttr:RPCManager.promptCursor", 
         "classAttr:RPCManager.showPrompt","attr:RPCRequest.promptStyle", "attr:RPCRequest.promptCursor", "attr:RPCRequest.prompt", 
         "attr:RPCRequest.showPrompt"],
        description:"The properties in this group all deal with setting and styling a modal prompt during an RPC\n call to the server.",
        name:"rpcPrompt",
        seeAlso:[],
        type:"group"
    },
    "method:Tree.hasFolders":{
        ref:"method:Tree.hasFolders",
        flags:"",
        description:"\n Return true if this this node has any children that are folders.\n\n",
        name:"hasFolders",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the node has children that are folders", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:FusionChart.dataColors":{
        ref:"attr:FusionChart.dataColors",
        flags:"IR",
        description:"An array of colors to use for a series of visual elements representing data (eg columns,\n bars, pie slices), any of which may be adjacent to any other.\n <P>\n Colors are expressed as hexademical RRGGBB Strings <b>with no leading '#' character</b>.",
        name:"dataColors",
        seeAlso:[],
        valueType:"Array of CSSColor",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:FusionChart"
    },
    "method:DynamicForm.setErrors":{
        ref:"method:DynamicForm.setErrors",
        flags:"A",
        description:" Setter for validation errors on this form. Errors passed in should be a Javascript object\n of the format<br>\n <code>{fieldName1:errors, fieldName2:errors}</code><br>\n Where the <code>errors</code> value may be either a string (single error message) or an\n array of strings (if multiple errors should be applied to the field in question).\n",
        name:"setErrors",
        seeAlso:[],
        params:[
            {description:"list of errors as an object with the field names as keys", optional:false,
             name:"errors", defaultValue:null, type:"object"},
            {description:"If true redraw form to display errors now. Otherwise errors can be displayed by calling\n      ${isc.DocUtils.linkForRef('method:DynamicForm.showErrors')}<br>\n      Note: When the errors are shown, \n      ${isc.DocUtils.linkForRef('method:DynamicForm.handleHiddenValidationErrors','handleHiddenValidationErrors()')} will\n      be fired for errors on hidden fields, or with no associated formItem.",
             optional:false, name:"showErrors", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "classMethod:Page.getSkinDir":{
        ref:"classMethod:Page.getSkinDir",
        flags:"",
        description:"		Return the directory for media that's part of the skin\n\n",
        name:"getSkinDir",
        seeAlso:[],
        returns:{description:"base URL for skin media", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "attr:FilterBuilder.subClauseButton":{
        ref:"attr:FilterBuilder.subClauseButton",
        flags:"IR",
        description:"Button allowing the user to add subclauses grouped by a ${isc.DocUtils.linkForRef('type:LogicalOperator')}.",
        name:"subClauseButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:ListGrid.clearSortFieldText":{
        ref:"attr:ListGrid.clearSortFieldText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, this\n attribute will be shown as the menu item title to clear an existing sort on this field.",
        name:"clearSortFieldText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Clear Sort\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "classAttr:RPCResponse.STATUS_UPDATE_WITHOUT_PK_ERROR":{
        ref:"classAttr:RPCResponse.STATUS_UPDATE_WITHOUT_PK_ERROR",
        flags:"R",
        description:"Indicates that the client attempted an update or remove operation without providing \n primary key field(s)",
        name:"STATUS_UPDATE_WITHOUT_PK_ERROR",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-9",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "object:ListGridRecord":{
        methods:[],
        ref:"object:ListGridRecord",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/ListGrid"],
        attrs:["attr:ListGridRecord.enabled", "attr:ListGridRecord.canExpand", 
         "attr:ListGridRecord.isSeparator","attr:ListGridRecord.customStyle", "attr:ListGridRecord._baseStyle", 
         "attr:ListGridRecord.singleCellValue","attr:ListGridRecord.canDrag", "attr:ListGridRecord.canAcceptDrop", 
         "attr:ListGridRecord.linkText","attr:ListGridRecord.detailDS", "attr:ListGridRecord.includeInSummary", 
         "attr:ListGridRecord.isGridSummary","attr:ListGridRecord.isGroupSummary"],
        description:"A ListGridRecord is a JavaScript Object whose properties contain values for each\n ${isc.DocUtils.linkForRef('object:ListGridField')}.  A ListGridRecord may have additional properties which affect the\n record's appearance or behavior, or which hold data for use by custom logic or other,\n related components.\n <p>\n For example a ListGrid that defines the following fields:\n <pre>\n fields : [\n     {name: \"field1\"},\n     {name: \"field2\"}\n ],\n </pre>\n Might have the following data:\n <pre>\n data : [\n     {field1: \"foo\", field2: \"bar\", customProperty:5},\n     {field1: \"field1 value\", field2: \"field2 value\", enabled:false}\n ]\n </pre>\n Each line of code in the <code>data</code> array above creates one JavaScript Object via\n JavaScript {type:ObjectLiteral,object literal} notation.  These JavaScript Objects are\n used as ListGridRecords.\n <P>\n Both records shown above have properties whose names match the name property of a\n ListGridField, as well as additional properties.  The second record will be disabled due to\n <code>enabled:false</code>; the first record has a property \"customProperty\" which will\n have no effect by default but which may be accessed by custom logic.\n <P>\n After a ListGrid is created and has loaded data, records may be accessed via\n ${isc.DocUtils.linkForRef('attr:ListGrid.data')}, for example, listGrid.data.get(0) retrieves the first record.\n ListGridRecords are also passed to many events, such as\n ${isc.DocUtils.linkForRef('method:ListGrid.cellClick','cellClick()')}.\n <P>\n A ListGridRecord is always an ordinary JavaScript Object regardless of how the grid's\n dataset is loaded (static data, java server, XML web service, etc), and so supports the\n normal behaviors of JavaScript Objects, including accessing and assigning to properties\n via dot notation:\n <pre>\n     var fieldValue = record.<i>fieldName</i>;\n     record.<i>fieldName</i> = newValue;\n </pre>\n <P>\n Note however that simply assigning a value to a record won't cause the display to be\n automatically refreshed - ${isc.DocUtils.linkForRef('method:ListGrid.refreshCell')} needs to be called.  Also,\n consider ${isc.DocUtils.linkForRef('group:editing','editValues vs saved values')} when directly modifying\n ListGridRecords.\n <P>\n See the attributes in the API tab for the full list of special properties on\n ListGridRecords that will affect the grid's behavior.",
        name:"ListGridRecord",
        seeAlso:[
            {ref:"attr:ListGrid.data", description:null}
        ],
        type:"object",
        inheritsFrom:"Record"
    },
    "attr:Applet.mayScript":{
        ref:"attr:Applet.mayScript",
        flags:"IR",
        description:"Usable only in IE: if true allows Java->JS LiveConnect (other platforms that support\n  LiveConnect have it enabled by default and do not allow it to be disabled).",
        name:"mayScript",
        seeAlso:[
            {ref:"attr:Applet.scriptable", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        platformNotes:"Relevant on IE only.",
        definingClass:"class:Applet"
    },
    "attr:ListGrid.valueIconWidth":{
        ref:"attr:ListGrid.valueIconWidth",
        flags:"IRW",
        description:"Width for value icons for this listGrid.\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconSize')}.\n Can be overridden at the field level",
        name:"valueIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:SelectItem.multiple":{
        ref:"attr:SelectItem.multiple",
        flags:"IRW",
        description:"If true, this selectItem will be displayed as a tall list of options, and the user\n will be able to select multiple options.\n <P>\n In this case the logical value of the formItem, as retrieved by \n ${isc.DocUtils.linkForRef('method:FormItem.getValue','getValue()')} and set via ${isc.DocUtils.linkForRef('method:FormItem.setValue','setValue()')}, is \n an Array of Strings reflecting the selected values.\n <P>\n Note: <code>multiple:true</code> SelectItems do not currently support optionDataSource\n binding.  You can get around this by calling DataSource.fetchData() directly and creating\n a valueMap for the item from the resulting array.",
        name:"multiple",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "method:Array.set":{
        ref:"method:Array.set",
        flags:"",
        description:" Change the array element at a particular position.\n <P>\n set() can be used to expand the length of the list.\n\n",
        name:"set",
        seeAlso:[],
        params:[
            {description:"position in the list to change", optional:false, name:"pos",
             defaultValue:null, type:"number"},
            {description:"new value for that position", optional:false, name:"obj", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"whatever's at that position, null if not found", type:"object"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "class:ResetItem":{
        methods:[],
        ref:"class:ResetItem",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"Button that resets the form to default values, by calling\n <code>DynamicForm.resetValues()</code>\n \n If you define a click handler on this item, you can return false to cancel the reset.",
        name:"ResetItem",
        seeAlso:[],
        type:"class"
    },
    "type:Overflow":{
        ref:"type:Overflow",
        values:[
            {description:"Content that extends beyond the widget's width or height is                              displayed.\n                              Note: To have the content be sized only by the drawn size of \n                              the content set the overflow to be Canvas.VISIBLE and specify\n                              a small size, allowing the size to expand to the size required\n                              by the content.\n                              Leaving the width / height for the widget undefined will use the\n                              default value of 100, and setting the size to zero may cause the\n                              widget not to draw.",
             value:"Canvas.VISIBLE"},
            {description:"Content that extends beyond the widget's width or height is                              clipped (hidden).",
             value:"Canvas.HIDDEN"},
            {description:"Horizontal and/or vertical scrollbars are displayed only if                              necessary. Content that extends beyond the remaining visible\n                              area is clipped.",
             value:"Canvas.AUTO"},
            {description:"Horizontal and vertical scrollbars are always drawn inside the                              widget. Content that extends beyond the remaining visible area\n                              is clipped, and can be accessed via scrolling.",
             value:"Canvas.SCROLL"},
            {description:"Clip horizontally but extend the canvas's clip region                              vertically if necessary.",
             value:"Canvas.CLIP_H"},
            {description:"Clip vertically but extend the canvas's clip region                              horizontally if necessary.",
             value:"Canvas.CLIP_V"}
        ],
        description:null,
        name:"Overflow",
        seeAlso:[],
        type:"type",
        groups:["sizing"]
    },
    "attr:ImgButton.title":{
        flags:"IRW",
        getter:["getTitle"],
        type:"attr",
        ref:"attr:ImgButton.title",
        description:"The text title to display in this button.",
        name:"title",
        setter:["setTitle"],
        valueType:"string",
        seeAlso:[],
        defaultValue:"varies",
        groups:["basics"],
        definingClass:"class:ImgButton"
    },
    "method:Canvas.hide":{
        ref:"method:Canvas.hide",
        flags:"",
        description:"			Sets the widget's CSS visibility attribute to \"hidden\".\n",
        name:"hide",
        seeAlso:[],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('showAndHide')}"],
        definingClass:"class:Canvas",
        groups:["visibility"]
    },
    "method:Flashlet.setSrc":{
        ref:"method:Flashlet.setSrc",
        flags:"",
        description:" \n  Sets the source file for the flash component  \n\n",
        name:"setSrc",
        seeAlso:[],
        type:"method",
        definingClass:"class:Flashlet"
    },
    "attr:SectionItem.defaultValue":{
        ref:"attr:SectionItem.defaultValue",
        flags:"IRW",
        description:"Section items show their <code>value</code> as title text for the section.\n Therefore the simplest way to specify this text on the form item directly is via the\n <code>defaultValue</code> attribute.",
        name:"defaultValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Section Header\"",
        type:"attr",
        definingClass:"class:SectionItem"
    },
    "classAttr:Canvas.TAB_INDEX_FLOOR":{
        ref:"classAttr:Canvas.TAB_INDEX_FLOOR",
        flags:"R",
        description:"Specifies the lower limit for automatically assigned tab indices for focusable canvii.",
        name:"TAB_INDEX_FLOOR",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"1000",
        type:"classAttr",
        definingClass:"class:Canvas",
        groups:["focus"]
    },
    "group:dataSourceOperations":{
        ref:"group:dataSourceOperations",
        title:"DataSource Operations",
        treeLocation:["Client Reference/Data Binding"],
        refs:[],
        description:"A DataSource Operation is a type of ${isc.DocUtils.linkForRef('group:operations','operation')} that acts on the set\n of stored objects represented by a ${isc.DocUtils.linkForRef('class:DataSource')}, performing one of the basic actions\n that makes sense on a set of similar records: \"fetch\", \"add\", \"update\" or \"remove\".  There\n is also a fifth DataSource Operation, \"custom\", which is intended for arbitrary server\n operations that are more complex than a fetch of some records, or an update to a single\n record.\n <P>\n Each DataSource operation has specific request and response data, for example, in the\n \"fetch\" DataSource operation, the request data is expected to be search criteria, and the\n response data is expected to be a list of matching DataSource records.  Listed below are the\n request data and response data for each DataSource operation type, and what they mean.  \n <P>\n DataSource records are represented on the client by a JavaScript Object,\n where each property in the Object maps a DataSource field name to the field value - hence\n the DataSource operations below are in essence a way of exchanging records from client to\n server and back.\n <P>\n If you are using ${isc.DocUtils.linkForRef('group:serverDataIntegration','server-side data integration')} with the\n SmartClient Java server, see the ${isc.DocUtils.linkForDocNode('javaServerReference', 'Java Server Reference')} for\n information about how DataSource Requests arrive on the server (specifically\n com.isomophic.datasource.DSRequest) and how to provide responses \n (specifically com.isomorphic.datasource.DSResponse.setData()).\n <P>\n If you are using ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side data integration')} to directly\n consume services that use XML, JSON or other formats, see the \"Editing and Saving\" section\n of the ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side data integration')} topic.\n <P>\n\n <b>fetch</b>\n <ul>\n <li>Request data: filter criteria, as an Object\n <li>Response data: matching records, as an Array of Objects\n </ul>\n\n <b>add</b>\n <ul>\n <li>Request data: new record, as an Object\n <li>Response data: new record as stored, as an Object\n </ul>\n\n <b>update</b>\n <ul>\n <li>Request data: primary keys of record to update, and new values (or just complete updated\n record), as an Object\n <li>Response data: new record as stored, as an Object\n </ul>\n\n <b>remove</b>\n <ul>\n <li>Request data: primary keys of record to delete, as an Object\n <li>Response data: minimally the primary keys of deleted record (can be complete record), as\n an Object\n </ul>\n\n <b>custom</b>\n <ul>\n <li>Request data: whatever the custom operation requires\n <li>Response data: custom operations can return whatever they like, including nothing.  \n Custom operations are like RPC calls in this respect - the exchanged data is unstructured, \n so it is up to you to make sure the client and server agree.  Note also that, because of\n this unstructured data exchange, cache synchronization does not work with custom operations.\n </ul>",
        name:"dataSourceOperations",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.saveByCell":{
        ref:"attr:ListGrid.saveByCell",
        flags:"IRW",
        description:"Whether edits should be saved whenever the user moves between cells in the current edit\n row.\n <P>\n If unset, defaults to ${isc.DocUtils.linkForRef('attr:ListGrid.editByCell','this.editByCell')}.\n <P>\n To avoid automatic saving entirely, set ${isc.DocUtils.linkForRef('attr:ListGrid.autoSaveEdits')}:false.",
        name:"saveByCell",
        seeAlso:[
            {ref:"attr:ListGrid.editByCell", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:MultiView.selectRecord":{
        ref:"method:MultiView.selectRecord",
        flags:"",
        description:"\n Select/deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n\n",
        name:"selectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "group:fieldState":{
        ref:"group:fieldState",
        refs:["method:ListGrid.fieldStateChanged"],
        name:"fieldState",
        seeAlso:[],
        type:"group"
    },
    "attr:TileLayout.overflow":{
        ref:"attr:TileLayout.overflow",
        flags:"IR",
        description:"Normal ${isc.DocUtils.linkForRef('type:Overflow')} settings can be used on TileLayouts, for example, an\n overflow:auto TileLayout will scroll if members exceed its specified size, whereas an\n overflow:visible TileLayout will grow to accomodate members.",
        name:"overflow",
        seeAlso:[],
        valueType:"Overflow",
        defaultValue:"\"visible\"",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["sizing"]
    },
    "attr:Canvas.htmlElement":{
        ref:"attr:Canvas.htmlElement",
        flags:"IRWA",
        description:"If specified as a pointer to an HTML element present in the DOM, this canvas will be\n rendered inside that element on ${isc.DocUtils.linkForRef('method:Canvas.draw','draw()')}.\n <P>\n <i>NOTES:</i><br>\n This feature is intended for temporary integration with legacy page architectures only;\n the native browser's reaction to DOM insertion is unspecified and unsupported. For\n consistent cross-browser layout and positioning semantics, use Canvas parents\n (especially Layouts) and use absolute positioning at top level.\n <P>\n Persistence of htmlElement: If ${isc.DocUtils.linkForRef('attr:Canvas.htmlPosition')} is set to <code>\"replace\"</code>\n the htmlElement will be removed from the DOM when the canvas is drawn - therefore the\n htmlElement attribute will be cleared at this time.\n Otherwise if a Canvas is clear()d and then draw()n again it will \n be rendered inside the same htmlElement.<br>\n If a Canvas is added as a child to Canvas parent, its htmlElement will be dropped.\n <P>\n ${isc.DocUtils.linkForRef('attr:Canvas.position')} should typically be set to <code>\"relative\"</code> if the widget \n is to be rendered inline within a standard page.",
        name:"htmlElement",
        setter:["setHtmlElement"],
        seeAlso:[],
        valueType:"HTML Element",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["htmlElement", "positioning"]
    },
    "method:DataSource.xmlSerialize":{
        ref:"method:DataSource.xmlSerialize",
        flags:"A",
        description:" Serialize a JavaScript object as XML.\n <P>\n The JavaScript Object passed to ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} becomes an XML\n element named after the ${isc.DocUtils.linkForRef('attr:DataSource.tagName')} (or ${isc.DocUtils.linkForRef('attr:DataSource.ID')} if\n tagName is unset).  Each property of the object becomes a subElement.  For example,\n using a DataSource to serialize like this:\n <pre>\n     var inputObject = {\n        startRow : 5,\n        endRow : 50,\n        data : [\n           { field1 : \"value1\", field2: new Date() },\n           { field1 : \"value3\", field2: null }\n        ]\n     };\n     var myDS = isc.DataSource.create({ tagName:\"DSRequest\" });\n     myDS.xmlSerialize(inputObject);\n </pre>\n .. produces the following XML:\n <pre>\n     &lt;DSRequest&gt;\n         &lt;startRow&gt;5&lt;/startRow&gt;\n         &lt;endRow&gt;50&lt;/endRow&gt;\n         &lt;data&gt;\n             &lt;field1&gt;value1&lt;/field1&gt;\n             &lt;field2&gt;2005-10-14T18:01:16&lt;/field2&gt;\n         &lt;/data&gt;\n         &lt;data&gt;\n             &lt;field1&gt;value3&lt;/field1&gt;\n             &lt;field2&gt;&lt;/field2&gt;\n         &lt;/data&gt;\n     &lt;/DSRequest&gt;\n </pre>\n <P>\n Various properties on the DataSource and DataSourceField can affect how serialization is\n performed, see for example ${isc.DocUtils.linkForRef('attr:DataSource.tagName')},\n ${isc.DocUtils.linkForRef('attr:DataSource.schemaNamespace')}, \n ${isc.DocUtils.linkForRef('attr:DataSourceField.xmlAttribute')}, \n ${isc.DocUtils.linkForRef('attr:DataSourceField.multiple')} and ${isc.DocUtils.linkForRef('attr:DataSourceField.childTagName')}.\n By setting the ${isc.DocUtils.linkForRef('attr:DataSourceField.type','type of a field')} to the ID of another\n DataSource which has further XML serialization settings, you can control serialization of\n nested structures.\n <P>\n If you are working with a WSDL-described web service, XML serialization is performed\n automatically by APIs like ${isc.DocUtils.linkForRef('method:WebService.callOperation')} - you only need to\n know about serialization in order to understand how to put together JavaScript data that\n will fill in an XML message properly, and for simple messages, setting\n ${isc.DocUtils.linkForRef('attr:DSRequest.useFlatFields')} makes that unnecessary as well.   \n <P>\n <b>Note:</b> when trying to send data to a web service, it is best to avoid putting\n together any XML yourself, instead modify the JavaScript data being fed to ISC's SOAP\n engine.  This is because the WSDL and SOAP rules for correctly namespacing and encoding\n Web Service messages are very complex and are subject to change with new versions of the\n web service you are contacting, whereas the data itself is easy to manipulate and less\n likely to change.\n <P>\n To troubleshoot message formation, you can set the log category \"xmlSerialize\" to \n <code>INFO</code> or <code>DEBUG</code> level in order to see diagnostics about XML\n message formation, and you can use the RPC tab in the Developer Console to see the\n actual messages being passed to web services.\n\n",
        name:"xmlSerialize",
        seeAlso:[],
        params:[
            {description:"data to be serialized", optional:false, name:"data", defaultValue:null,
             type:"any"},
            {description:"options for the serialization engine", optional:false, name:"flags",
             defaultValue:null, type:"SerializationContext"}
        ],
        returns:{description:"data as serialized to XML", type:"String"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:RPCRequest.useSimpleHttp":{
        ref:"attr:RPCRequest.useSimpleHttp",
        flags:"IRWA",
        description:"When set to true, assume the request is not going to the SmartClient server, and hence send\n a simple HTTP request that does not use SmartClient-specific request encoding.\n <P>\n Values specified in ${isc.DocUtils.linkForRef('attr:RPCRequest.params')} are sent to to the server as HTTP request\n parameters.  If ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')} is \"GET\", parameters appear in the request URL, otherwise\n if httpMethod is \"POST\", parameters are encoded in the request body (exactly like an HTML form\n does).  These parameters are then accessible via typical server-side APIs for retrieving\n HTTP parameters, eg, servletRequest.getParameter(paramName) in Java Servlets.  \n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')} method is POST and ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} is supplied,\n ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} is assumed to be a string to post as the HTTP request body, and\n ${isc.DocUtils.linkForRef('attr:RPCRequest.params')} are sent as URL parameters instead.  This usage is for sending\n custom request bodies such as the XML payloads used for SOAP.  In this case,\n ${isc.DocUtils.linkForRef('attr:RPCRequest.contentType')} is typically also set to indicate the content type of the\n request body.\n <p>\n Setting <code>useSimpleHttp</code> to true also automatically sets\n ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} to true as well.",
        name:"useSimpleHttp",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:DataSourceField.canSave":{
        ref:"attr:DataSourceField.canSave",
        flags:"IR",
        description:"Whether values in this field can be updated and saved to the dataSource.\n <P>\n If set to <code>false</code>, this field will default to being non-editable in standard editing\n components (${isc.DocUtils.linkForRef('class:DynamicForm')}, editable ${isc.DocUtils.linkForRef('class:ListGrid')}), but will be editable when displayed\n for filtering purposes only (in a ${isc.DocUtils.linkForRef('class:SearchForm')} or ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor','ListGrid\n filter editor')}.\n <P>\n Note: if ${isc.DocUtils.linkForRef('attr:DataSourceField.canEdit')} is explicitly specified it will take precedence over this\n behavior.",
        name:"canSave",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "group:errors":{
        ref:"group:errors",
        refs:["method:DynamicForm.getErrors", "method:DynamicForm.getFieldErrors", 
         "method:DynamicForm.setErrors","method:DynamicForm.setError", "method:DynamicForm.addFieldErrors", 
         "method:DynamicForm.setFieldErrors","method:DynamicForm.clearFieldErrors", "method:DynamicForm.clearErrors", 
         "method:DynamicForm.hasErrors","method:DynamicForm.hasFieldErrors", "method:DynamicForm.getValidatedValues", 
         "method:DynamicForm.showErrors","method:DynamicForm.showFieldErrors", "method:ValuesManager.getValidatedValues", 
         "method:ValuesManager.setErrors","method:ValuesManager.addFieldErrors", "method:ValuesManager.setFieldErrors", 
         "method:ValuesManager.clearErrors","method:ValuesManager.clearFieldErrors", "method:ValuesManager.getErrors", 
         "method:ValuesManager.getFieldErrors","method:ValuesManager.hasErrors", "method:ValuesManager.hasFieldErrors", 
         "method:ValuesManager.showErrors","method:ValuesManager.showFieldErrors"],
        description:"Validation errors and how they are shown",
        name:"errors",
        seeAlso:[],
        type:"group"
    },
    "attr:CanvasItem.canvasDefaults":{
        ref:"attr:CanvasItem.canvasDefaults",
        flags:"IRW",
        description:"Default properties for the canvas if this.canvas is not already a canvas instance.",
        name:"canvasDefaults",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:CanvasItem"
    },
    "method:Window.removeItems":{
        ref:"method:Window.removeItems",
        flags:"",
        description:"			Removes an array of widgets from the window.\n",
        name:"removeItems",
        seeAlso:[],
        params:[
            {description:"an array of widgets to be removed", optional:false, name:"items",
             defaultValue:null, type:"array of canvases"}
        ],
        returns:{description:"the array of widgets removed", type:"array"},
        type:"method",
        definingClass:"class:Window",
        groups:["windowItems"]
    },
    "method:ColumnTree.shouldShowHeader":{
        ref:"method:ColumnTree.shouldShowHeader",
        flags:"",
        description:" Whether the indicated column should show a header.  Returns this.showHeaders by default,\n override for different behavior.\n \n",
        name:"shouldShowHeader",
        seeAlso:[],
        params:[
            {description:"parent node for the nodes to be shown in the column", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"index of the column", optional:false, name:"colNum", defaultValue:null,
             type:"int"}
        ],
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "method:SectionStack.addSection":{
        ref:"method:SectionStack.addSection",
        flags:"",
        description:"\n Add a section to the SectionStack.\n\n",
        name:"addSection",
        seeAlso:[],
        params:[
            {description:"Initialization block                  for the section or a list of initialization blocks to add.",
             optional:false, name:"sections", defaultValue:null,
             type:"SectionStackSection Properties | List of SectionStackSection Properties"},
            {description:"index for the new section(s) (if not specified, the section                      will be added at the end of the SectionStack).",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsAddAndRemove')}"],
        definingClass:"class:SectionStack"
    },
    "attr:ListGridField.canEdit":{
        ref:"attr:ListGridField.canEdit",
        flags:"IRW",
        description:"Can this field be edited? May be overridden by setting the\n      'canEdit' property at the listGrid level",
        name:"canEdit",
        seeAlso:[
            {ref:"attr:ListGrid.canEdit", description:null},
            {ref:"attr:ListGrid.recordEditProperty", description:null},
            {ref:"method:ListGrid.canEditCell", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disableEditing')}"],
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "interface:PickList":{
        methods:["method:PickList.fetchData", "method:PickList.getOptionDataSource", 
         "method:PickList.getPickListFilterCriteria","method:PickList.getValueFieldName", "method:PickList.getDisplayFieldName", 
         "method:PickList.getClientPickListData","method:PickList.dataArrived", "method:PickList.filterClientPickListData", 
         "method:PickList.getSelectedRecord"],
        ref:"interface:PickList",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:PickList.pickListHeight", "attr:PickList.emptyPickListHeight", 
         "attr:PickList.emptyPickListMessage","attr:PickList.hideEmptyPickList", "attr:PickList.pickListWidth", 
         "attr:PickList.pickListBaseStyle","attr:PickList.animatePickList", "attr:PickList.animationTime", "attr:PickList.fetchDelay", 
         "attr:PickList.pickListProperties","attr:PickList.pickListHeaderHeight", "attr:PickList.valueField", 
         "attr:PickList.displayField","attr:PickList.pickListFields", "attr:PickList.valueIconField", 
         "attr:PickList.pickListCriteria","attr:PickList.optionDataSource", "attr:PickList.showOptionsFromDataSource", 
         "attr:PickList.optionFilterContext","attr:PickList.filterLocally", "attr:PickList.sortField", 
         "attr:PickList.cachePickListResults","attr:PickList.textMatchStyle", "attr:PickList.showAllOptions", 
         "attr:PickList.separatorRows"],
        description:"Interface to show a drop-down list of pickable options. Used by the ${isc.DocUtils.linkForRef('class:SelectItem')} and\n ${isc.DocUtils.linkForRef('class:ComboBoxItem')} classes. The generated drop down list of options will be an instance of\n ${isc.DocUtils.linkForRef('class:PickListMenu')}.",
        name:"PickList",
        seeAlso:[],
        type:"interface"
    },
    "method:TreeGrid.getDropFolder":{
        ref:"method:TreeGrid.getDropFolder",
        flags:"",
        description:" When the user is dragging a droppable element over this grid, this method returns the folder\n which would contain the item if dropped. This is the current drop node if the user is hovering\n over a folder, or the node's parent if the user is hovering over a leaf.\n",
        name:"getDropFolder",
        seeAlso:[],
        returns:{description:"target drop folder", type:"node"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["events"]
    },
    "attr:GridRenderer.cellPadding":{
        ref:"attr:GridRenderer.cellPadding",
        flags:"IRW",
        description:"The amount of empty space, in pixels, surrounding each value in its cell.",
        name:"cellPadding",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "classMethod:Page.getURL":{
        ref:"classMethod:Page.getURL",
        flags:"",
        description:" Return a full URL for a relative path that uses a special prefix such as \"[APPFILES]\" or\n \"[SKIN]\".\n <P>\n For images, use ${isc.DocUtils.linkForRef('classMethod:Page.getImgURL')} instead.\n\n",
        name:"getURL",
        seeAlso:[],
        params:[
            {description:"Local file name for the image.", optional:false, name:"fileName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"URL for the image.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "method:Window.centerInPage":{
        ref:"method:Window.centerInPage",
        flags:"A",
        description:" Centers the Window in the page. This is called automatically in window.show() if\n Window.autoCenter is true.\n Note - if the Window is a child of another widget, we center in the parent widget\n rather than centering in the page.\n",
        name:"centerInPage",
        seeAlso:[
            {ref:"attr:Window.autoCenter", description:null}
        ],
        type:"method",
        definingClass:"class:Window",
        groups:["appearance"]
    },
    "attr:DynamicForm.showComplexFieldsRecursively":{
        ref:"attr:DynamicForm.showComplexFieldsRecursively",
        flags:"IR",
        description:"If set, this <code>DynamicForm</code> will set both \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.showComplexFields','showComplexFields')} and \n <code>showComplexFieldsRecursively</code> on any nested component used for showing/editing\n a complex field.  Thus any of this form's items that handle complex fields will themselves\n also show complex fields.  This allows for handling of field structures of any complexity.\n <p>\n If set, this value automatically sets ${isc.DocUtils.linkForRef('attr:DataBoundComponent.showComplexFields','showComplexFields')}\n as well.",
        name:"showComplexFieldsRecursively",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "method:Chart.setData":{
        ref:"method:Chart.setData",
        flags:"",
        description:" Change the dataset for this chart on the fly.  May or may not be supported by concrete chart\n implementations.\n\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"new dataset", optional:false, name:"newData", defaultValue:null,
             type:"Array of CellRecord or CellRecord"}
        ],
        type:"method",
        definingClass:"interface:Chart"
    },
    "attr:Tab.disabled":{
        ref:"attr:Tab.disabled",
        flags:"IRW",
        description:"If specified, this tab will initially be rendered in a disabled state. To enable or\n disable tabs on the fly use the ${isc.DocUtils.linkForRef('method:TabSet.enableTab')}, and ${isc.DocUtils.linkForRef('method:TabSet.disableTab')}\n methods.",
        name:"disabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab"
    },
    "method:List.removeList":{
        ref:"method:List.removeList",
        flags:"",
        description:" Remove all instances of objects in the specified list from this list, sliding the remaining\n objects around to fill gaps.\n\n",
        name:"removeList",
        seeAlso:[],
        params:[
            {description:"list of items to remove", optional:false, name:"list", defaultValue:null,
             type:"array"}
        ],
        returns:{description:"list of items passed in", type:"list"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "method:Canvas.dragResizeStop":{
        ref:"method:Canvas.dragResizeStop",
        flags:"A",
        description:"\n Executed when the mouse button is released at the end of the drag resize. Your\n widget can use this opportunity to fire custom code based upon where the\n mouse button was released, etc. \n <p>\n Returning true from this handler will cause the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline if\n ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance')} is set to \"outline\") to be left at its current size. Returning\n false from this handler will cause it to snap back to its original location size\n\n",
        name:"dragResizeStop",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to snap the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline) back to its                   original size or true to leave it at the current cursor position.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "classAttr:Operators.iContainsTitle":{
        ref:"classAttr:Operators.iContainsTitle",
        flags:"IR",
        description:"Title for the \"iContains\" operator",
        name:"iContainsTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"contains\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "method:Class.toString":{
        ref:"method:Class.toString",
        flags:"",
        description:"\n  The default toString() for instances reports that you have an instance of a class and prints\n  the instance ID if present.\n",
        name:"toString",
        seeAlso:[],
        type:"method",
        definingClass:"class:Class"
    },
    "method:DataSource.getPrimaryKeyFieldName":{
        ref:"method:DataSource.getPrimaryKeyFieldName",
        flags:"",
        description:" Returns the primary key fieldName for this DataSource\n",
        name:"getPrimaryKeyFieldName",
        seeAlso:[],
        returns:{description:"primary key field name", type:"string"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "method:TabSet.addTab":{
        ref:"method:TabSet.addTab",
        flags:"A",
        description:" Add a tab\n",
        name:"addTab",
        seeAlso:[
            {ref:"method:TabSet.addTabs", description:null}
        ],
        params:[
            {description:"new tab", optional:false, name:"tab", defaultValue:null, type:"Tab"},
            {description:"position where tab should be added", optional:true, name:"position",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('tabsAddAndRemove')}"],
        definingClass:"class:TabSet"
    },
    "method:SectionStack.removeItem":{
        ref:"method:SectionStack.removeItem",
        flags:"",
        description:" Remove an item from a section.\n",
        name:"removeItem",
        seeAlso:[],
        params:[
            {description:"ID or index of the section to remove item from", optional:false,
             name:"section", defaultValue:null, type:"String or Number"},
            {description:"Item to remove", optional:false, name:"item", defaultValue:null, type:"Canvas"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:Canvas.prompt":{
        ref:"attr:Canvas.prompt",
        flags:"IRW",
        description:"Prompt displayed in hover canvas if ${isc.DocUtils.linkForRef('attr:Canvas.showHover','showHover')} is true.",
        name:"prompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customHovers')}"],
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:TabSet.getPaneContainerEdges":{
        ref:"method:TabSet.getPaneContainerEdges",
        flags:"A",
        description:" If the paneContainer for this tab set is showing ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','edges')}, this \n method can be used to specify (dynamically) which ${isc.DocUtils.linkForRef('attr:Canvas.customEdges','customEdges')} to\n show. Called when the pane creator is created.\n <P>\n Default implementation will return null unless ${isc.DocUtils.linkForRef('attr:TabSet.showPartialEdges','showPartialEdges')}\n is true, in which case it will return the three edges opposite the\n ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')}.\n",
        name:"getPaneContainerEdges",
        seeAlso:[],
        returns:{description:"array of custom edges to show", type:"array"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "method:CubeGrid.anyCellSelected":{
        ref:"method:CubeGrid.anyCellSelected",
        flags:"",
        description:"\n Determine whether any cells are selected in this cubeGrid\n\n  <br><i>methodType</i> tester\n",
        name:"anyCellSelected",
        seeAlso:[],
        returns:{description:"true if any cells are selected", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:ListGrid.discardAllEdits":{
        ref:"method:ListGrid.discardAllEdits",
        flags:"A",
        description:"\n Cancel outstanding edits, discarding edit values, and hiding editors for the record[s] passed\n in if appropriate.\n If no rows are passed in all outstanding edit values will be dropped.\n \n",
        name:"discardAllEdits",
        seeAlso:[],
        params:[
            {description:"allows you to specify which row(s) to drop edits for", optional:true,
             name:"rows", defaultValue:null, type:"array of numbers"},
            {description:"By default this method will hide the editor if                              it is currently showing for any row in the grid. Passing in\n                              this parameter will leave the editor visible (and just reset\n                              the edit values underneath the editor).",
             optional:true, name:"dontHideEditor", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Label.showRollOverIcon":{
        ref:"attr:Label.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "method:DynamicForm.itemKeyPress":{
        ref:"method:DynamicForm.itemKeyPress",
        flags:"",
        description:" Handler fired when a FormItem within this form receives a keypress event.\n <P>\n Fires after the keyPress handler on the FormItem itself, and only if the item did not\n cancel the event and chooses to allow it to propagate to the form as a whole. \n \n",
        name:"itemKeyPress",
        seeAlso:[],
        params:[
            {description:"the FormItem where the change event occurred", optional:false, name:"item",
             defaultValue:null, type:"FormItem"},
            {description:"name of the key that was pressed (EG: \"A\", \"Space\")", optional:false,
             name:"keyName", defaultValue:null, type:"string"},
            {description:"numeric character value of the pressed key.", optional:false,
             name:"characterValue", defaultValue:null, type:"number"}
        ],
        returns:{description:"return false to cancel the keyPress, or true to allow it", type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "attr:TabSet.topEdgeSizes":{
        ref:"attr:TabSet.topEdgeSizes",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeSizes</code>, \n <code>rightEdgeSizes</code>, <code>topEdgeSizes</code> and <code>bottomEdgeSizes</code> \n propertes allow the sizes of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge sizes where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"topEdgeSizes",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "method:Canvas.getContents":{
        ref:"method:Canvas.getContents",
        flags:"",
        description:"      Returns the contents of a Canvas. The contents are an HTML string.\n",
        name:"getContents",
        seeAlso:[],
        returns:{description:"contents of this Canvas", type:"HTML"},
        type:"method",
        definingClass:"class:Canvas"
    },
    "classMethod:SimpleType.getType":{
        ref:"classMethod:SimpleType.getType",
        flags:"",
        description:" Retrieve a simpleType definition by type name\n",
        name:"getType",
        seeAlso:[],
        params:[
            {description:"the <code>name</code> of the simpleType to return", optional:false,
             name:"typeName", defaultValue:null, type:"string"}
        ],
        returns:{description:"ssimple type object", type:"SimpleType"},
        type:"classMethod",
        definingClass:"class:SimpleType"
    },
    "attr:FormItem.rowSpan":{
        ref:"attr:FormItem.rowSpan",
        flags:"IRW",
        description:"Number of rows that this item spans",
        name:"rowSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formLayout"]
    },
    "method:ListGrid.selectSingleRecord":{
        ref:"method:ListGrid.selectSingleRecord",
        flags:"",
        description:" Select a single ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index, and deselect everything else.\n When programmatic selection of records is a requirement and ${isc.DocUtils.linkForRef('attr:ListGrid.selectionType')} \n is \"single\", use this method rather than ${isc.DocUtils.linkForRef('method:ListGrid.selectRecord','selectRecord()')} to \n enforce mutually-exclusive record-selection.\n\n",
        name:"selectSingleRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:ImgButton.showSelectedIcon":{
        ref:"attr:ImgButton.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:ListGrid.filterOnKeypress":{
        ref:"attr:ListGrid.filterOnKeypress",
        flags:"IRWA",
        description:"If we're showing the filterEditor (this.showFilterEditor is true), this property \n determines whether this list should be filtered every time the user modifies the value\n in a field of the filter-editor. Can also be set at the field level.",
        name:"filterOnKeypress",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "attr:Operator.requiresServer":{
        ref:"attr:Operator.requiresServer",
        flags:"IR",
        description:"Whether this operator needs to be executed on the server side.\n <P>\n This implies that if a ${isc.DocUtils.linkForRef('object:Criterion')} using this operator is either introduced into\n ${isc.DocUtils.linkForRef('object:AdvancedCriteria','criteria')} or is changed, the server will need to be contacted to\n perform filtering.",
        name:"requiresServer",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "attr:Calendar.weekEventBorderOverlap":{
        ref:"attr:Calendar.weekEventBorderOverlap",
        flags:"IR",
        description:"Augments the width of week event windows slightly to avoid duplicate adjacent borders\n between events.",
        name:"weekEventBorderOverlap",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "classMethod:Date.compareLogicalDates":{
        ref:"classMethod:Date.compareLogicalDates",
        flags:"",
        description:" Compare two dates, normalizing out the time elements so that only the date elements are \n considered; returns 0 if equal, -1 if the first date is greater (later), or 1 if\n the second date is greater.\n",
        name:"compareLogicalDates",
        seeAlso:[],
        params:[
            {description:"first date to compare", optional:false, name:"date1", defaultValue:null,
             type:"date"},
            {description:"second date to compare", optional:false, name:"date2", defaultValue:null,
             type:"date"}
        ],
        returns:{description:"0 if equal, -1 if first date &gt; second date, 1 if second date &gt;                      first date.  Returns false if either argument is not a date",
         type:"number"},
        type:"classMethod",
        definingClass:"class:Date"
    },
    "class:MenuBar":{
        methods:["method:MenuBar.setMenus", "method:MenuBar.addMenus", "method:MenuBar.removeMenus", 
         "method:MenuBar.showMenu"],
        ref:"class:MenuBar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:MenuBar.menus", "attr:MenuBar.tabIndex"],
        description:"A MenuBar is a bar of buttons used to show a set of menus.",
        name:"MenuBar",
        seeAlso:[],
        type:"class"
    },
    "attr:Applet.src":{
        ref:"attr:Applet.src",
        flags:"IR",
        description:"A URL that points to the class of this applet.  This is the same as the 'code' attribute.\n  Note that if you specify both the src and the code attributes, the src attribute will be\n  ignored in favor of the code attribute.",
        name:"src",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:Chart.shouldAnimateShow":{
        ref:"attr:Chart.shouldAnimateShow",
        flags:"IR",
        description:"Whether the chart should animate when shown (eg, bars or columns grow into place)",
        name:"shouldAnimateShow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"interface:Chart",
        groups:["chartAppearance"]
    },
    "attr:FormItem.icons":{
        ref:"attr:FormItem.icons",
        flags:"IRW",
        description:"An array of descriptor objects for icons to display in a line after this form item.\n  These icons are clickable images, often used to display some kind of helper for \n  populating a form item.",
        name:"icons",
        seeAlso:[
            {ref:"object:FormItemIcon", description:null}
        ],
        valueType:"Array of FormItemIcon Properties",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formIcons')}"],
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "attr:Snapbar.gripImgSuffix":{
        ref:"attr:Snapbar.gripImgSuffix",
        flags:"IRA",
        description:"Overridden from ${isc.DocUtils.linkForRef('attr:Splitbar.gripImgSuffix')} to simplify providing custom grip media\n for this widget.",
        name:"gripImgSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"snap\"",
        type:"attr",
        definingClass:"class:Snapbar"
    },
    "attr:ListGrid.sorterProperties":{
        ref:"attr:ListGrid.sorterProperties",
        flags:"IRA",
        description:"Properties to apply to the sorter button. Overrides defaults applied via \n ${isc.DocUtils.linkForRef('attr:ListGrid.sorterDefaults')}.",
        name:"sorterProperties",
        seeAlso:[],
        valueType:"Button Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:SearchForm.showFilterFieldsOnly":{
        ref:"attr:SearchForm.showFilterFieldsOnly",
        flags:"IRWA",
        description:"If this attribute is true any ${isc.DocUtils.linkForRef('attr:DataSourceField.canFilter','canFilter:false')} fields\n specified on the dataSource will not be shown unless explicitly included in this component's\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields','fields array')}",
        name:"showFilterFieldsOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SearchForm"
    },
    "attr:OperationBinding.recordXPath":{
        ref:"attr:OperationBinding.recordXPath",
        flags:"IR",
        description:"For an XML or JSON DataSource, XPath expression used to retrieve the objects that will\n become DataSource records.\n <p>\n For example, an \"ItemSearch\" web service might return a \"Results\" structure containing\n metadata along with the set of Items that one might want to display in a grid.  An XPath\n expression like \"/Results/Items\" could be used to retrieve just the Items, which would then\n become DataSource records.\n <p>\n For a JSON web service, the <code>recordXPath</code> is applied to the returned JSON data\n via ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectObjects')}.  Only limited XPath syntax is allowed; see \n ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectObjects','selectObjects()')} for details.\n <P>\n For processing XML results, see ${isc.DocUtils.linkForRef('attr:OperationBinding.xmlNamespaces')} for information\n on the namespaces that are available in this XPath expression.  If you are contacting a WSDL\n web service, note that ${isc.DocUtils.linkForRef('attr:OperationBinding.recordName')} is an alternative way to specify\n which records should be selected by their tagName or type, and this is usually simpler.\n <P>\n To learn about XPath, try the following search:\n <a href=\"http://www.google.com/search?q=xpath+tutorial\" target=\"_blank\"\n >http://www.google.com/search?q=xpath+tutorial</a>",
        name:"recordXPath",
        seeAlso:[],
        valueType:"XPathExpression",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:DateItem.endDate":{
        ref:"attr:DateItem.endDate",
        flags:"IRW",
        description:"Maximum date the selectors will allow the user to pick.",
        name:"endDate",
        seeAlso:[],
        valueType:"Date",
        defaultValue:"12/31/2015",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["appearance"]
    },
    "attr:TabBar.buttonConstructor":{
        ref:"attr:TabBar.buttonConstructor",
        flags:"AIRW",
        description:"SmartClient component used for the tabs of the tabBar. \n Must be Button or Button subclass.",
        name:"buttonConstructor",
        seeAlso:[],
        valueType:"class",
        defaultValue:"ImgTab",
        type:"attr",
        definingClass:"class:TabBar"
    },
    "attr:HTMLFlow.contents":{
        ref:"attr:HTMLFlow.contents",
        flags:"IRW",
        description:"The contents of a canvas or label widget. Any HTML string is acceptable.",
        name:"contents",
        seeAlso:[
            {ref:"attr:HTMLFlow.dynamicContents", description:null}
        ],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contents"]
    },
    "method:CubeGrid.facetValuesSelected":{
        ref:"method:CubeGrid.facetValuesSelected",
        flags:"",
        description:"\n return whether the header indicated by the set of facetValues is selected\n\n  <br><i>methodType</i> tester\n",
        name:"facetValuesSelected",
        seeAlso:[],
        params:[
            {description:"facetValues to test", optional:false, name:"facetValues", defaultValue:null,
             type:"facetValues object"}
        ],
        returns:{description:"true if the header is selected", type:"boolean"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "attr:TabSet.pickerButtonSrc":{
        ref:"attr:TabSet.pickerButtonSrc",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true, this property governs the base URL for the picker\n button image, when ${isc.DocUtils.linkForRef('attr:TabSet.symmetricPickerButton')} is set to false\n <P>\n Note that if <code>symmetricPickerButton</code> is true, the ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonHSrc')} \n and ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonVSrc')} properties will be used instead.\n <P>\n To get the path to the image to display, this base URL will be modified as follows:\n <ul>\n <li>If appropriate a state suffix of <code>\"Down\"</code> or <code>\"Disabled\"</code> will be\n     appended.</li>\n <li>The ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} for this tabSet will be appended.</li>\n </ul>",
        name:"pickerButtonSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricPickerButton", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/picker.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "attr:Slider.sliderTarget":{
        ref:"attr:Slider.sliderTarget",
        flags:"IRW",
        description:"The target widget for the <code>sliderMove</code> event generated when the slider thumb \n is moved.",
        name:"sliderTarget",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:FormItem.hoverHeight":{
        ref:"attr:FormItem.hoverHeight",
        flags:"IRW",
        description:"Option to specify a height for any hover shown for this item.",
        name:"hoverHeight",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverHeight", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:StretchImgButton.showFocus":{
        ref:"attr:StretchImgButton.showFocus",
        deprecated:"as of SmartClient version 6.1 in favor of ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')}",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  Note that by default the\n <code>over</code> state is used to indicate focus.",
        name:"showFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "attr:ListGrid.selectionAppearance":{
        ref:"attr:ListGrid.selectionAppearance",
        flags:"IRW",
        description:"How selection of rows should be presented to the user.\n <P>\n For <code>selectionAppearance:\"checkbox\"</code> with multiple selection\n allowed, you would typically use ${isc.DocUtils.linkForRef('type:SelectionType')}:\"simple\" (the default).  Because \n <code>selectionType</code> and <code>selectionAppearance</code> are unrelated, \n the combination of <code>selectionAppearance:\"checkbox\"</code> and\n <code>selectionType:\"multiple\"</code> results in a grid where multiple selection can\n only be achieved via shift-click or ctrl-click.  \n <P>\n If using \"checkbox\", see also ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxField')} for customization APIs.",
        name:"selectionAppearance",
        setter:["setSelectionAppearance"],
        seeAlso:[],
        valueType:"SelectionAppearance",
        defaultValue:"\"rowStyle\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:DataSource.schemaNamespace":{
        ref:"attr:DataSource.schemaNamespace",
        flags:"R",
        description:"For a DataSource derived from WSDL or XML schema, the XML namespace this schema belongs\n to.  This is a read-only attribute automatically present on DataSources returned from\n ${isc.DocUtils.linkForRef('method:SchemaSet.getSchema')} and ${isc.DocUtils.linkForRef('method:WebService.getSchema')}.",
        name:"schemaNamespace",
        seeAlso:[],
        valueType:"URN",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["wsdlBinding", "clientDataIntegration"]
    },
    "classMethod:RPCManager.cancelQueue":{
        ref:"classMethod:RPCManager.cancelQueue",
        flags:"",
        description:" Cancel a queue of requests (also called a transaction).\n <P>\n If a transactionId is passed, that transaction will be cancelled, otherwise, the current \n (not yet sent) transaction is cancelled.  You can retrieve the id of the current \n transaction, if there is one, by calling \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.getCurrentTransactionId','getCurrentTransactionId()')} before the\n transaction has been sent.\n <P>\n Note that cancelQueue() calls ${isc.DocUtils.linkForRef('classMethod:RPCManager.clearTransaction','clearTransaction()')}\n and attempts to abort the request.  However, note also that whilst cancelling a \n transaction that has already been sent will not necessarily stop the HTTP request that \n has been issued - this is only possible on some browsers and with some transports - it \n will reliably cause SmartClient to ignore any response returned by the server and not \n fire any callbacks that have been passed in.\n \n",
        name:"cancelQueue",
        seeAlso:[],
        params:[
            {description:"transactionId of the queue.", optional:true, name:"transactionNum",
             defaultValue:null, type:"id"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager"
    },
    "attr:DataSource.titleField":{
        ref:"attr:DataSource.titleField",
        flags:"IR",
        description:"Best field to use for a user-visible title for an individual record from this dataSource.\n <p>\n For example, for a DataSource of employees, a \"full name\" field would probably most clearly\n label an employee record.\n <p>\n If not explicitly set, titleField looks for fields named \"title\", \"label\", \"name\", and \"id\"\n in that order.  If a field exists with one of those names, it becomes the titleField.  If not,\n then the first field is designated as the titleField.",
        name:"titleField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["titles"]
    },
    "attr:DetailViewer.cellStyle":{
        ref:"attr:DetailViewer.cellStyle",
        flags:"IRW",
        getter:["getCellStyle"],
        description:"CSS style for a normal value",
        name:"cellStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detail\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:DetailViewer.getCellStyle":{
        ref:"method:DetailViewer.getCellStyle",
        flags:"",
        description:" Return the CSS class for a cell. Default implementation calls\n ${isc.DocUtils.linkForRef('method:DetailViewerField.getCellStyle','getCellStyle()')} on the field if defined, otherwise\n returns ${isc.DocUtils.linkForRef('attr:DetailViewer.cellStyle','this.cellStyle')}\n",
        name:"getCellStyle",
        seeAlso:[],
        params:[
            {description:"actual value of this cell", optional:false, name:"value", defaultValue:null,
             type:"any"},
            {description:"field object for this cell", optional:false, name:"field", defaultValue:null,
             type:"DetailViewerField"},
            {description:"record object for this cell", optional:false, name:"record",
             defaultValue:null, type:"Record"},
            {description:"the viewer instance to which this cell belongs", optional:false,
             name:"viewer", defaultValue:null, type:"DetailViewer"}
        ],
        returns:{description:"CSS style for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:DetailViewer",
        groups:["apperance"]
    },
    "method:FilterBuilder.getValueFieldProperties":{
        ref:"method:FilterBuilder.getValueFieldProperties",
        flags:"",
        description:" Override to return properties for the FormItem used for the \"value\" field, that is, the\n user-entered value that becomes ${isc.DocUtils.linkForRef('attr:Criterion.value')}.\n \n",
        name:"getValueFieldProperties",
        seeAlso:[],
        params:[
            {description:"type of the DataSource field for this filter row", optional:false,
             name:"type", defaultValue:null, type:"FieldType"},
            {description:"name of the DataSource field for this filter row", optional:false,
             name:"type", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "method:DataSource.recordsFromXML":{
        ref:"method:DataSource.recordsFromXML",
        flags:"A",
        description:" Transform a list of XML elements to DataSource records.\n <P>\n <code>recordsFromXML()</code> will create a List of DataSource records in the form of\n JavaScript objects.  The value for each field is extracted from the XML according to the\n rules described under ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')}. \n <P>\n Derived JavaScript values for each field will be the appropriate JavaScript type, eg,\n for a field of \"date\" type, the JS value will be a JavaScript Date object.\n <P>\n Note that if the set of nodes exceeds ${isc.DocUtils.linkForRef('attr:DataSource.resultBatchSize')}, this method\n will break the XML processing logic into multiple threads and therefore complete\n asynchronously. In this case the return value will be null, and the callback parameter\n must be used to process the extracted records.\n \n",
        name:"recordsFromXML",
        seeAlso:[],
        params:[
            {description:"XML elements to transform, eg, the result of a                                       call to ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectNodes')}",
             optional:false, name:"elements", defaultValue:null, type:"List of XMLElement"},
            {description:"Callback to fire when the transfrom completes. Takes a single    parameter <code>records</code> - the array of DataSource records derived from the\n    XML elements.",
             optional:false, name:"calback", defaultValue:null, type:"Callback"}
        ],
        returns:{description:"List of DataSource records derived from the XML elements, or null   if this method completed the XML processing asynchronously.",
         type:"List"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "method:List.getLength":{
        ref:"method:List.getLength",
        flags:"",
        description:" Return the number of items in this list\n\n",
        name:"getLength",
        seeAlso:[],
        returns:{description:"number of items in the list", type:"Number"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "method:Slider.setMaxValue":{
        ref:"method:Slider.setMaxValue",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.maxValue','maximum value')} of the slider \n\n",
        name:"setMaxValue",
        seeAlso:[],
        params:[
            {description:"the new maximum value", optional:false, name:"newValue", defaultValue:null,
             type:"float"}
        ],
        type:"method",
        definingClass:"class:Slider"
    },
    "method:StretchImgButton.select":{
        ref:"method:StretchImgButton.select",
        flags:"",
        description:" Select this object.\n",
        name:"select",
        seeAlso:[],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "class:Scrollbar":{
        methods:["method:Scrollbar.setScrollTarget"],
        ref:"class:Scrollbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:Scrollbar.btnSize", "attr:Scrollbar.autoEnable", "attr:Scrollbar.showCorner", 
         "attr:Scrollbar.allowThumbDownState","attr:Scrollbar.allowThumbOverState", "attr:Scrollbar.showTrackEnds", 
         "attr:Scrollbar.thumbMinSize","attr:Scrollbar.trackEndWidth", "attr:Scrollbar.trackEndHeight", 
         "attr:Scrollbar.thumbOverlap","attr:Scrollbar.startThumbOverlap", "attr:Scrollbar.endThumbOverlap", 
         "attr:Scrollbar.thumbInset","attr:Scrollbar.skinImgDir", "attr:Scrollbar.cornerSrc", "attr:Scrollbar.cornerSize", 
         "attr:Scrollbar.hSrc","attr:Scrollbar.vSrc", "attr:Scrollbar.scrollTarget"],
        description:"The Scrollbar widget implements cross-platform, image-based scrollbars that control the\n scrolling of content in other widgets.  Scrollbar widgets are created and displayed\n automatically for widgets that require them, based on settings for ${isc.DocUtils.linkForRef('attr:Canvas.overflow')}.\n <P>\n The scrollbar's appearance is based on a ${isc.DocUtils.linkForRef('class:StretchImg')} for the \"track\", which consists\n of two fixed size buttons and a stretchable center segment, and the ${isc.DocUtils.linkForRef('class:ScrollThumb')},\n the draggable portion of the scrollbar, also a StretchImg, with an optional\n ${isc.DocUtils.linkForRef('attr:StretchImg.showGrip','grip')}.",
        name:"Scrollbar",
        seeAlso:[],
        type:"class"
    },
    "method:ComboBoxItem.getClientPickListData":{
        ref:"method:ComboBoxItem.getClientPickListData",
        flags:"",
        description:" Returns the set of data to be displayed in this item's PickList.\n <P>\n This method will be called for non-databound form items implementing the PickList\n interface.  The default implementation will derive data from the item's valueMap - \n can be overridden to allow a custom set of options to be displayed.\n <P>\n Note that for PickLists that filter data based on user input\n (${isc.DocUtils.linkForRef('class:ComboBoxItem','ComboBox')}), this method should return the data <b>before\n filtering</b>.  To customize the data returned after filtering, override\n ${isc.DocUtils.linkForRef('method:ComboBoxItem.filterClientPickListData')} instead.\n <P>\n As an example, for a formItem with ${isc.DocUtils.linkForRef('attr:ComboBoxItem.valueField')} set to \"valueFieldName\", the\n default implementation would take a valueMap like the following:\n <pre>\n     valueMap: { value1: \"display 1\", value2: \"display 2\" }\n </pre>\n .. and returning the following set of records: \n <pre>\n     [\n          { valueFieldName : \"value1\" },\n          { valueFieldName : \"value2\" }\n     ]\n </pre>\n Due to the valueMap, these records will appear as a two row pickList displayed as\n \"display 1\" and \"display 2\".\n\n",
        name:"getClientPickListData",
        seeAlso:[],
        returns:{description:"Array of record objects to be displayed in the           pickList. Note that when a user picks a record from the list, the value of the\n           field matching <code>item.valueField</code> will be picked. Also note that the\n           fields to be displayed can be customized via <code>item.pickListFields</code>",
         type:"Array of ListGridRecord"},
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "attr:ListGrid.checkboxFieldImageHeight":{
        ref:"attr:ListGrid.checkboxFieldImageHeight",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance')} is set to <code>\"checkbox\"</code> this property\n may be set to govern the height of the checkbox image displayed to indicate whether a row is\n selected. If unset, the checkboxField image will be sized to match the\n ${isc.DocUtils.linkForRef('attr:ListGrid.booleanImageHeight')} for this grid.",
        name:"checkboxFieldImageHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "classMethod:Date.setNormalDisplayFormat":{
        ref:"classMethod:Date.setNormalDisplayFormat",
        flags:"",
        description:" Set the default formatter for date objects to the method name passed in.  After calling this\n method, subsequent calls to ${isc.DocUtils.linkForRef('method:Date.toNormalDate')} will return a string formatted\n according to this format specification. Note: this will be the standard long date format used \n by SmartClient components.<br>\n The <code>format</code> parameter may be either a ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} string, or \n a function. If passed a function, this function will be executed in the scope of the Date\n and should return the formatted string.<br>\n Initial default normalDisplayFormat is <code>\"toLocaleString\"</code>\n",
        name:"setNormalDisplayFormat",
        seeAlso:[],
        params:[
            {description:"new formatter", optional:false, name:"format", defaultValue:null,
             type:"DateDisplayFormat | function"}
        ],
        type:"classMethod",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:DetailViewer.datetimeFormatter":{
        ref:"attr:DetailViewer.datetimeFormatter",
        flags:"IR",
        description:"Display format to use for fields specified as type 'datetime'. Default is to use\n the system-wide default datetime format configured via \n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDatetimeDisplayFormat')}",
        name:"datetimeFormatter",
        seeAlso:[],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DetailViewer"
    },
    "method:Class.addAutoChild":{
        ref:"method:Class.addAutoChild",
        flags:"",
        description:" Creates a component according to the \"AutoChild\" pattern, and adds it to this component.\n <P>\n See the ${isc.DocUtils.linkForRef('group:autoChildren','AutoChild usage overview')} to understand the general\n purpose and usage of this method.\n <P>\n <code>addAutoChild()</code> takes the following actions:\n <ol>\n <li> checks whether this.<i>autoChildName</i> is already populated, and returns it if so\n <li> checks when there is a show<i>AutoChildName</i> with the value false, and if so\n returns without creating a component  \n <li> calls ${isc.DocUtils.linkForRef('method:Class.createAutoChild')} to create the component\n <li> sets this.<i>autoChildName</i> to the created component\n <li> adds the component either to this component, or to some other parent, specified\n by the \"autoParent\" property in the autoChild's defaults.  The \"autoParent\" property may\n be \"none\" indicating the autoChild should not be automatically added.\n </ol>\n <P>\n When adding an autoChild to a ${isc.DocUtils.linkForRef('class:Layout')} subclass,\n ${isc.DocUtils.linkForRef('method:Layout.addMember','addMember()')} will be called instead of the normal\n ${isc.DocUtils.linkForRef('method:Canvas.addChild','addChild()')}.  To prevent this behavior,\n <code>addAsChild:true</code> can be set in the autoChild defaults.  Similarly,\n <code>addAsPeer:true</code> may be set to add an autoChild as a peer.\n <P>\n <b>Tip:</b> because <code>addAutoChild()</code>\n checks specifically for show<i>AutoChildName</i>:false, you do not have to add\n show<i>AutoChildName</i>:true in order for an autoChild to be shown by default; leaving\n the property undefined is sufficient.\n \n",
        name:"addAutoChild",
        seeAlso:[],
        params:[
            {description:"name of the autoChild", optional:false, name:"childName", defaultValue:null,
             type:"String"},
            {description:"dynamic properties for the autoChild", optional:false, name:"defaults",
             defaultValue:null, type:"Properties"}
        ],
        returns:{description:"created autoChild", type:"Class"},
        type:"method",
        definingClass:"class:Class",
        groups:["autoChildren"]
    },
    "attr:SummaryBuilder.autoHideCheckBox":{
        ref:"attr:SummaryBuilder.autoHideCheckBox",
        flags:"IR",
        description:"CheckBox that, when selected, hides columns in the component that are used in this formula.",
        name:"autoHideCheckBox",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:PanelHeader.titleStyle":{
        ref:"attr:PanelHeader.titleStyle",
        flags:"IRW",
        description:"CSS style applied to the title text only.  Defaults to ${isc.DocUtils.linkForRef('attr:PanelHeader.baseStyle')} when unset.\n <P>\n With a separate <code>titleStyle</code> and ${isc.DocUtils.linkForRef('attr:PanelHeader.baseStyle')} set, you can provide a\n backgroundColor via <code>baseStyle</code> that will allow translucent .png media to be\n \"tinted\" by the underlying background color, so that a single set of media can provide\n range of color options.  In this usage, the <code>titleStyle</code> should generally not\n specify a background color as this would block out the media that appears behind the\n title.",
        name:"titleStyle",
        setter:["setTitleStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:PanelHeader"
    },
    "method:Canvas.clickMaskUp":{
        ref:"method:Canvas.clickMaskUp",
        flags:"",
        description:" Determines whether a clickmask is showing\n",
        name:"clickMaskUp",
        seeAlso:[
            {ref:"method:Canvas.showClickMask", description:null}
        ],
        params:[
            {description:"optional ID of specific clickMask to check. If not passed,                       checks for the click mask associated with this widget only.",
             optional:true, name:"ID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["clickMask"]
    },
    "classAttr:Validator.notOneOf":{
        ref:"classAttr:Validator.notOneOf",
        flags:"IRA",
        description:"Default error message to display when standard <code>isOneOf</code> type validator\n is not passed.",
        name:"notOneOf",
        seeAlso:[],
        valueType:"string",
        defaultValue:"'Not a valid option'",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:FacetValue.titleHilite":{
        ref:"attr:FacetValue.titleHilite",
        flags:"IR",
        description:"Hilite style to apply to the title for this facetValue.  See\n ${isc.DocUtils.linkForRef('attr:CubeGrid.hilites')}.",
        name:"titleHilite",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "method:ValuesManager.resetValues":{
        ref:"method:ValuesManager.resetValues",
        flags:"",
        description:"\n Same as ${isc.DocUtils.linkForRef('method:DynamicForm.reset')}.\n\n",
        name:"resetValues",
        seeAlso:[],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "group:validation":{
        ref:"group:validation",
        refs:["classMethod:isc.getErrorsHTML", "method:DynamicForm.getItemErrorHTML", 
         "method:DynamicForm.validate","method:DynamicForm.valuesAreValid", "method:DynamicForm.validateData", 
         "classMethod:Validator.addValidator","classMethod:Validator.addValidators", "attr:DynamicForm.showInlineErrors", 
         "attr:DynamicForm.showErrorIcons","attr:DynamicForm.showErrorText", "attr:DynamicForm.showErrorStyle", 
         "attr:DynamicForm.errorOrientation","attr:DynamicForm.errorItemProperties", "attr:DynamicForm.errorItemCellStyle", 
         "attr:DynamicForm.errorsPreamble","attr:DynamicForm.showTitlesWithErrorMessages", "attr:DynamicForm.errors", 
         "attr:DynamicForm.validateOnChange","attr:DynamicForm.unknownErrorMessage", "attr:DynamicForm.validationURL", 
         "attr:DynamicForm.disableValidation","attr:FormItem.required", "attr:FormItem.showErrorIcon", "attr:FormItem.showErrorText", 
         "attr:FormItem.showErrorStyle","attr:FormItem.errorOrientation", "attr:FormItem.errorMessageWidth", 
         "attr:TextItem.length","attr:ValuesManager.unknownErrorMessage", "attr:ValuesManager.disableValidation", 
         "attr:DataSource.useLocalValidators","attr:DataSourceField.validators", "attr:SimpleType.validators", "type:CharacterCasing"],
        description:"Validation",
        name:"validation",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.animateFadeTime":{
        ref:"attr:Canvas.animateFadeTime",
        flags:"IRWA",
        description:"Default time for performing an animated fade.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateFadeTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:StatefulCanvas.showRollOverIcon":{
        ref:"attr:StatefulCanvas.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "method:CubeGrid.getRollupValue":{
        ref:"method:CubeGrid.getRollupValue",
        flags:"",
        description:" Get the facetValue definition for the facetValue to show when this facet is \"rolled up\" under\n another facet, during a breakout.<br><br>\n\n A facet is not required to have a rollup value, and if it does not have one, then rollups will\n simply be blank rows.  The facetValueId of the rollup value can be declared as\n cubeGrid.rollupValue or facet.rollupValue.  \n\n",
        name:"getRollupValue",
        seeAlso:[
            {ref:"attr:CubeGrid.rollupValue", description:null}
        ],
        returns:{description:"rolled up facet value definition", type:"facetValue"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "method:ListGrid.getEditField":{
        ref:"method:ListGrid.getEditField",
        flags:"",
        description:"\n  Returns the field object associated with cell currently being edited\n\n",
        name:"getEditField",
        seeAlso:[],
        returns:{description:"Field object definition", type:"object"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:Slider.showTitle":{
        ref:"attr:Slider.showTitle",
        flags:"IRW",
        description:"Indicates whether the slider's title should be displayed. The default position for this label\n is to the left of a horizontal slider, or above a vertical slider.",
        name:"showTitle",
        seeAlso:[
            {ref:"attr:Slider.title", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:Progressbar.breadth":{
        flags:"IR",
        getter:["getBreadth"],
        type:"attr",
        ref:"attr:Progressbar.breadth",
        description:"Thickness of the progressbar in pixels. This is effectively width for a vertical\n progressbar, or height for a horizontal progressbar.\n <P>\n This property must be set instead of setting <code>width</code> or <code>height</code>.",
        name:"breadth",
        setter:["setBreadth"],
        valueType:"number",
        seeAlso:[],
        defaultValue:"20",
        groups:["appearance"],
        definingClass:"class:Progressbar"
    },
    "group:treeField":{
        ref:"group:treeField",
        refs:["attr:TreeGridField.treeField", "attr:TreeGrid.treeFieldTitle", "attr:TreeGrid.showRoot", 
         "attr:TreeGrid.separateFolders","attr:TreeGrid.displayNodeType"],
        name:"treeField",
        seeAlso:[],
        type:"group"
    },
    "method:Date.toNormalDate":{
        ref:"method:Date.toNormalDate",
        flags:"",
        description:" Returns the date as a formatted string using the format set up via the \n <code>setNormalDisplayFormat()</code> method. Note that the default formatter for this\n method is <code>\"toLocaleString\"</code>.\n",
        name:"toNormalDate",
        seeAlso:[],
        params:[
            {description:"Optional Format for the date returned", optional:false, name:"format",
             defaultValue:null, type:"DateDisplayFormat"}
        ],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:SectionStack.sectionIsExpanded":{
        ref:"method:SectionStack.sectionIsExpanded",
        flags:"",
        description:"\n Returns true if the specified section is expanded, false if it is collapsed.\n\n",
        name:"sectionIsExpanded",
        seeAlso:[],
        params:[
            {description:"Section for which you want to obtain information.\n                      For this parameter, you can pass the position of the section in the\n                      SectionStack, or the ID of the section.",
             optional:false, name:"section", defaultValue:null, type:"sectionId|position"}
        ],
        returns:{description:"true if the section is expanded, false if it is not.", type:"boolean"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "class:RestDataSource":{
        methods:["method:RestDataSource.transformRequest", "method:RestDataSource.transformResponse"],
        ref:"class:RestDataSource",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:RestDataSource.dataFormat", "attr:RestDataSource.xmlRecordXPath", 
         "attr:RestDataSource.jsonRecordXPath","attr:RestDataSource.recordXPath", "attr:RestDataSource.prettyPrintJSON", 
         "attr:RestDataSource.operationBindings","attr:RestDataSource.dataURL", "attr:RestDataSource.fetchDataURL", 
         "attr:RestDataSource.updateDataURL","attr:RestDataSource.addDataURL", "attr:RestDataSource.removeDataURL", 
         "attr:RestDataSource.sendMetaData","attr:RestDataSource.metaDataPrefix"],
        description:"The RestDataSource implements the 4 core DataSource operations using a simple protocol of\n XML or JSON requests and responses sent over HTTP, which can be easily fulfilled by any HTTP\n server technology.\n <P>\n RestDataSource is named for the\n <a href='http://www.google.com/search?hl=en&q=REST+HTTP' onclick=\"window.open('http://www.google.com/search?hl=en&q=REST+HTTP');return false;\">REST</a> (REpresentational State\n Transfer) pattern, which in brief says that simple messages passed over HTTP is a sufficient\n protocol for many web applications, without the need for further protocols such as WSDL or\n SOAP.\n <P>\n A RestDataSource is used just like a normal DataSource.  RestDataSources are pre-configured,\n using the general-purpose databinding facilities of DataSources, to expect a particular\n format for responses and to send requests in a specific format.   These request and\n response formats represent Isomorphic's recommended best practices for binding SmartClient\n to backends which do not already support a similar, pre-existing request and response\n format and where the SmartClient Java Server cannot be used.  \n <P>\n If you have a pre-existing REST or WSDL service which is difficult to change, consider\n adapting SmartClient to the existing service instead, by starting with a normal\n ${isc.DocUtils.linkForRef('class:DataSource')} and using the \n ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side data integration')} facilities to create a\n mapping between SmartClient's ${isc.DocUtils.linkForRef('class:DSRequest')} and ${isc.DocUtils.linkForRef('class:DSResponse')} objects and the message\n formats of your existing services.\n <P>\n RestDataSource is typically used with PHP, Ruby, Python, Perl or custom server technologies,\n and represents an alternative to installing the SmartClient Server in a Java technology\n stack, or using ${isc.DocUtils.linkForRef('group:wsdlBinding','WSDL-based binding')} with .NET or other WSDL-capable\n technologies.\n <P>\n The request and response formats used by the RestDataSource allow for many of the available\n features of SmartClient's databinding system to be used, including data paging, searching &\n sorting, ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues','long transactions')}, \n ${isc.DocUtils.linkForRef('class:ResultSet','automatic cache sync')} and ${isc.DocUtils.linkForRef('group:relogin','relogin')}.  However advanced\n features such as ${isc.DocUtils.linkForRef('group:upload','uploading / binary fields')},\n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','queuing')} and transaction chaining,\n ${isc.DocUtils.linkForRef('method:ListGrid.exportData','export')} and all ${isc.DocUtils.linkForRef('group:iscServer','server-based features')} aren't\n available with RestDataSource and need to be re-implemented as needed.\n <P>\n <span style=\"font-weight:bold;font-size:16px;\">Examples</span>\n <p>\n <b>XML formatted responses:</b>\n <P>\n RestDataSource expects a response like the following in response to a \"fetch\" request:\n <pre>\n &lt;response&gt;\n    &lt;status&gt;0&lt;/status&gt;\n    &lt;startRow&gt;0&lt;/startRow&gt;\n    &lt;endRow&gt;76&lt;/endRow&gt;\n    &lt;totalRows&gt;546&lt;/totalRows&gt;\n    &lt;data&gt;\n      &lt;record&gt;\n          &lt;field1&gt;value&lt;/field1&gt;\n          &lt;field2&gt;value&lt;/field2&gt;\n      &lt;/record&gt;\n      &lt;record&gt;\n          &lt;field1&gt;value&lt;/field1&gt;\n          &lt;field2&gt;value&lt;/field2&gt;\n      &lt;/record&gt;\n      <i>... 75 total records ... </i>\n    &lt;/data&gt;\n &lt;/response&gt;\n </pre>\n The &lt;status&gt; element indicates whether the fetch operation was successful \n (see ${isc.DocUtils.linkForRef('group:statusCodes')}).\n <P>\n The &lt;data&gt; element contains a list of record nodes, each of which represents a record\n returned by the server.  The optional &lt;startRow&gt;, &lt;endRow&gt; and &lt;totalRows&gt;\n elements are needed only if data paging is in use, and populate the\n ${isc.DocUtils.linkForRef('attr:DSResponse.startRow','startRow')}, ${isc.DocUtils.linkForRef('attr:DSResponse.endRow','endRow')} and\n ${isc.DocUtils.linkForRef('attr:DSResponse.totalRows','totalRows')} properties of the ${isc.DocUtils.linkForRef('class:DSResponse')}.\n <P>\n Note: for a more compact format, simple field values may be specified on record \n nodes directly as attributes - in this case a record element might be structured like this:\n <pre>\n     &lt;record field1=\"value\" field2=\"value\" /&gt;\n </pre>\n <p>\n Note that a RestDataSource will bypass browser caching of all responses by default.  See\n ${isc.DocUtils.linkForRef('attr:DataSource.preventHTTPCaching')}.\n <p>\n Successful \"add\" or \"update\" request responses are similar in format - in this case the data\n element would be expected to contain a single record object containing the details of the\n record, as saved on the server.\n <P>\n The response from a \"remove\" operation would again include status and data elements, but in\n this case, only the primary key field value(s) of the removed record would be expected to be \n present under the data element.\n <p>\n If a validation failure occurred on the server, the response would\n have status set to ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_VALIDATION_ERROR')} [<code>-4</code>],\n and any validation errors could be included as per-field sub-elements of an \"errors\"\n element.  For a validation error, the response is not expected to contain any\n &lt;data&gt; element.  \n <P>\n A response showing a validation error might look like this:\n <pre>\n &lt;response&gt;\n    &lt;status&gt;-4&lt;/status&gt;\n    &lt;errors&gt;\n      &lt;field1&gt;\n          &lt;errorMessage&gt;A validation error occurred for this field&lt;/errorMessage&gt;\n      &lt;/field1&gt;\n    &lt;/errors&gt;\n &lt;/response&gt;\n </pre>\n <P>\n An unrecoverable error, such as an unexpected server failure, can be flagged by setting\n &lt;status&gt; to -1 and setting &lt;data&gt; to an error message.  In this case the\n &lt;errors&gt; element is not used (it's specific to validation errors).  An unrecoverable\n error causes all response processing to be skipped and ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')} to be\n invoked, which by default will show the provided error message as an alert using\n ${isc.DocUtils.linkForRef('classMethod:isc.warn')}.\n <p>\n <b>JSON formatted responses:</b>\n <P>\n JSON format responses are expected to contain the same data / meta-data as XMLresponses,\n encapsulated a simple object with a <code>\"response\"</code> attribute.<br>\n The response to a \"fetch\" request would therefore have this format:<br>\n <pre>\n {    \n    response:{\n       status:0,\n       startRows:0,\n       endRow:76,\n       totalRows:546,\n       data:[\n           {field1:\"value\", field2:\"value\"},\n           {field1:\"value\", field2:\"value\"},\n           <i>... 75 total records ...</i>\n       ]\n    }\n }\n </pre>\n The structure successful for \"add\", \"update\" and \"remove\" responses would be similar, though\n the data array would be expected to contain only a single object, representing the values as\n saved.  This allows the server to return values such as an auto-generated sequence\n primaryKey, a last modified timestamp, or similar server-generated field values.\n <P>\n For a remove, only the value for the primaryKey field[s] would be required.\n <p>\n For a validation error, the <code>status</code> attribute would be set to \n ${isc.DocUtils.linkForRef('classAttr:RPCResponse.STATUS_VALIDATION_ERROR')} [<code>-4</code>], and errors would\n be specified in the <code>errors</code> attribute of the response. For example:\n <pre>\n {    response:\n      {   status:-4,\n          errors: \n              {   field1:{errorMessage:\"A validation error on field1\"},\n                  field2:{errorMessage:\"A validation error on field2\"}\n              }\n      }\n }\n </pre>\n An array of errors may also be returned for a single field, like this:\n <pre>\n {    response:\n      {   status:-4,\n          errors: \n              {   field1:[\n                      {errorMessage:\"First error on field1\"},\n                      {errorMessage:\"Second error on field1\"}\n                  ]\n              }\n      }\n }\n </pre>\n <P>\n As with the XML format above, an unrecoverable error is indicated by setting the\n <code>status</code> attribute to -1 and the <code>data</code> property to the error message.\n <P>\n <b>Server inbound data formats</b>\n <P>\n The format of data sent to the server is determined by the ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol')}\n specified for the operation. Request data is sent as parameters if the format is \n specified as <code>\"getParams\"</code> or <code>\"postParams\"</code>.\n <P>\n In this case, the parameters sent to the server will consist of the DSRequest's data, and any\n parameters explicitly specified on the DSRequest object (as ${isc.DocUtils.linkForRef('attr:RPCRequest.params')}.<br> \n If ${isc.DocUtils.linkForRef('attr:RestDataSource.sendMetaData')} is true, the DSRequest meta \n data properties will also be present as parameters, prefixed with \n ${isc.DocUtils.linkForRef('attr:RestDataSource.metaDataPrefix')}.\n <P>\n Example URL constructed with the metaDataPrefix set to <code>\"_\"</code> (the default):\n <p>\n <code>\n &nbsp;&nbsp;&nbsp;<i>[dataURL]</i>?field1=value1&_operationType=fetch&_startRow=0&_endRow=50&_sortBy=-field2&_dataSource=dsName\n </code>\n <p>\n In this case the server would be able to separate the request's data from the meta data \n via the <code>\"_\"</code> prefix.\n <P>\n If data is sent to the server via the <code>\"postMessage\"</code> dataProtocol, the data will\n be serialized as an XML or JSON message according to the <code>dataFormat</code> setting.\n Both XML and JSON messages will contain request metadata such as startRow and endRow, and\n will appear exactly as though the subset of the ${isc.DocUtils.linkForRef('class:DSRequest')} that is meaningful to the\n server had been passed to ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} or ${isc.DocUtils.linkForRef('classMethod:JSON.encode')}\n respectively.\n <P>\n An example of an XML message might look like this:\n <pre>\n    &lt;request&gt;\n        &lt;data&gt;\n            &lt;countryDS&gt;\n                &lt;countryCode&gt;US&lt;/countryCode&gt;\n                &lt;countryName&gt;Edited Value&lt;/countryName&gt;\n                &lt;capital&gt;Edited Value&lt;/capital&gt;\n                &lt;continent&gt;Edited Value&lt;/continent&gt;\n            &lt;/countryDS&gt;\n        &lt;/data&gt;\n        &lt;dataSource&gt;countryDS&lt;/dataSource&gt;\n        &lt;operationType&gt;update&lt;/operationType&gt;\n    &lt/request&gt;\n </pre>\n The ${isc.DocUtils.linkForRef('attr:RestDataSource.operationBindings','default OperationBindings')} for a RestDataSource\n specify dataProtocol as \"getParams\" for the fetch operation, and \"postParams\" for update,\n add and remove operations.\n <P>\n <b>Hierachical (Tree) data:</b>\n <P>\n To create a hierachical DataSource, in the DataSource's <code>fields</code> array, a field \n must be specified as the parent id field - the field which will contain a pointer to the\n id of each node's parent. \n This can be achieved by setting the ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')} and the \n ${isc.DocUtils.linkForRef('attr:DataSourceField.rootValue')} attributes on the field definition. For example:\n <pre>\n RestDataSource.create({\n    ID:\"supplyItem\",\n    fields : [\n        {name:\"itemId\", type:\"sequence\", primaryKey:true},\n        {name:\"parentId\", type:\"integer\", foreignKey:\"supplyItem.itemId\", rootValue:0},\n        ...\n    ]\n });\n </pre>\n Tree Data is then treated on the server as a flat list of records linked by parent id.<br>\n Tree data is typically displayed using a dataBound ${isc.DocUtils.linkForRef('class:TreeGrid')} component. TreeGrids\n automatically create a ${isc.DocUtils.linkForRef('class:ResultTree')} data object, which requests data directly\n from the DataSource.  ResultTrees load data on demand, only requesting currently visible \n (open) nodes from the server. This is handled by including a specified value for the parent \n id field in the request criteria.<br>\n To implement a standard load-on-demand tree RestDataSource back end, you should therefore \n simply return the set of nodes that match the criteria passed in. \n For example, if your DataSource was defined as the \"supplyItem\" code snippet above, \n a fetch request for all children of a node with <code>itemId</code> set to <code>12</code> \n would have <code>\"parentId\"</code> set to <code>12</code> in the request criteria.\n A valid response would then contain all the records that matched this criteria. For example:\n <pre>\n &lt;response&gt;\n    &lt;status&gt;0&lt;/status&gt;\n    &lt;data&gt;\n      &lt;record&gt;\n          &lt;itemId&gt;15&lt;/itemId&gt;\n          &lt;parentId&gt;12&lt;/parentId&gt;\n      &lt;/record&gt;\n      &lt;record&gt;\n          &lt;itemId&gt;16&lt;/itemId&gt;\n          &lt;parentId&gt;12&lt;/parentId&gt;\n      &lt;/record&gt;\n    &lt;/data&gt;\n &lt;/response&gt;\n </pre>\n The structure of responses for Add, Update and Delete type requests will be the \n same regardless of whether the data is hierachical. However you should be aware that \n the underlying data storage may need to be managed slightly differently in some cases.<br>\n Specifically, Add and Update operations may change the structure of the tree by returning a \n new parent id field value for the modified node. Depending on how your data is stored you \n may need to include special back-end logic to handle this.<br>\n Also, if a user deletes a folder within a databound tree, any children of that folder will \n also be dropped from the tree, and can be removed from the back-end data storage.<br>\n Note: For a general overview of binding components to Tree structured data, see \n ${isc.DocUtils.linkForRef('group:treeDataBinding','Tree Databinding')}.",
        name:"RestDataSource",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('restEditSave')}"]
    },
    "attr:Img.imageHeight":{
        ref:"attr:Img.imageHeight",
        flags:"IR",
        description:"Explicit size for the image, for ${isc.DocUtils.linkForRef('attr:Img.imageType')} settings that would normally use the\n image's natural size (applies to ${isc.DocUtils.linkForRef('attr:Img.imageType')} \"center\" and \"normal\" only).",
        name:"imageHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Img"
    },
    "group:im ageEdgeStyles":{
        ref:"group:im ageEdgeStyles",
        refs:["attr:EdgedCanvas.addEdgeStyleSuffix"],
        name:"im ageEdgeStyles",
        seeAlso:[],
        type:"group"
    },
    "method:StatefulCanvas.setSelected":{
        ref:"method:StatefulCanvas.setSelected",
        flags:"",
        description:" Set this object to be selected or deselected.\n",
        name:"setSelected",
        seeAlso:[],
        params:[
            {description:"new boolean value of whether or not the object is                                          selected.",
             optional:false, name:"newIsSelected", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "method:HibernateBrowser.getResults":{
        ref:"method:HibernateBrowser.getResults",
        flags:"",
        description:" \n",
        name:"getResults",
        seeAlso:[],
        type:"method",
        definingClass:"class:HibernateBrowser"
    },
    "attr:OperationBinding.dataFormat":{
        ref:"attr:OperationBinding.dataFormat",
        flags:"IR",
        description:"Format for response data for this operation.\n <P>\n Typically set once for the DataSource as a whole via ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat')}.",
        name:"dataFormat",
        seeAlso:[],
        valueType:"DSDataFormat",
        defaultValue:"\"iscServer\"",
        type:"attr",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:StatefulCanvas.icon":{
        ref:"attr:StatefulCanvas.icon",
        flags:"IRW",
        description:"Optional icon to be shown with the button title text.  \n <P>\n Specify as the partial URL to an image, relative to the imgDir of this component.",
        name:"icon",
        setter:["setIcon"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:DateChooser.baseWeekendStyle":{
        ref:"attr:DateChooser.baseWeekendStyle",
        flags:"IRW",
        description:"Base css style applied to weekends. Will have \"Over\", \"Selected\" and \"Down\"\n suffix appended as the user interacts with buttons.  Defaults to ${isc.DocUtils.linkForRef('attr:DateChooser.baseWeekdayStyle')}.",
        name:"baseWeekendStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:Layout.reorderMember":{
        ref:"method:Layout.reorderMember",
        flags:"",
        description:" Shift a member of the layout to a new position\n		    \n",
        name:"reorderMember",
        seeAlso:[],
        params:[
            {description:"current position of the member to move to a new position", optional:false,
             name:"memberNum", defaultValue:null, type:"number"},
            {description:"new position to move the member to", optional:false, name:"newPosition",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:StatefulCanvas.overCanvasConstructor":{
        ref:"attr:StatefulCanvas.overCanvasConstructor",
        flags:"IRWA",
        description:"Constructor class name for this widgets ${isc.DocUtils.linkForRef('attr:StatefulCanvas.overCanvas','overCanvas')}",
        name:"overCanvasConstructor",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Canvas\"",
        type:"attr",
        definingClass:"class:StatefulCanvas"
    },
    "attr:MultiSortDialog.levelDownButton":{
        ref:"attr:MultiSortDialog.levelDownButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ImgButton')} providing a mechanism for moving existing\n sort-levels down in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.levelDownButtonProperties</code> and \n <code>multiSortPanel.levelDownButtonDefaults</code>.",
        name:"levelDownButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "attr:StretchImgButton.iconAlign":{
        ref:"attr:StretchImgButton.iconAlign",
        flags:"IR",
        description:"If this button is showing an icon should it be right or left aligned?",
        name:"iconAlign",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "attr:TreeNode.canDrag":{
        ref:"attr:TreeNode.canDrag",
        flags:"IRA",
        description:"Governs whether this node can be dragged. Only has an effect if this node is displayed in\n a ${isc.DocUtils.linkForRef('class:TreeGrid')} where ${isc.DocUtils.linkForRef('attr:TreeGrid.canDragRecordsOut')}, ${isc.DocUtils.linkForRef('attr:TreeGrid.canReorderRecords')}\n or ${isc.DocUtils.linkForRef('attr:TreeGrid.canReparentNodes')} is <code>true</code>.",
        name:"canDrag",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:Window.headerBackground":{
        ref:"attr:Window.headerBackground",
        flags:"R",
        description:"Img background component for the header, for gradient or image-based display",
        name:"headerBackground",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "class:BatchUploader":{
        methods:[],
        ref:"class:BatchUploader",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:BatchUploader.uploadForm", "attr:BatchUploader.uploadDataSource", 
         "attr:BatchUploader.uploadOperation","attr:BatchUploader.uploadFormFields", "attr:BatchUploader.uploadFileLabel", 
         "attr:BatchUploader.uploadButton","attr:BatchUploader.grid", "attr:BatchUploader.gridFields", 
         "attr:BatchUploader.commitButton","attr:BatchUploader.cancelButton", "attr:BatchUploader.requestProperties", 
         "attr:BatchUploader.dataURL","attr:BatchUploader.uploadFieldPrefix", "attr:BatchUploader.partialCommit", 
         "attr:BatchUploader.partialCommitPrompt","attr:BatchUploader.partialCommitError", "attr:BatchUploader.warnOnCancel", 
         "attr:BatchUploader.cancelConfirmMessage"],
        description:"The BatchUploader handles the upload, validation, review and saving of a dataset\n expressed in CSV or other upload formats.\n <P>\n By default, a BatchUploader consists of a single ${isc.DocUtils.linkForRef('class:FileItem')} form field.\n This form field will upload a file using the special \"batchUpload\" built-in DataSource.\n The uploaded file data will be parsed and validated using the\n ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadDataSource','uploadDataSource')},\n then streamed back to the browser, along with any errors, for display in a ListGrid.  \n <P>\n The user can then correct any errors and submit the final dataset, which will be\n added to the DataSource via a series of \"add\" DSRequests, all submitted as a single\n HTTP request via ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','request queuing')}.\n <P>\n Additional form fields can be added to the form that uploads the data file via \n ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadFormFields','uploadFormFields')}.  Values entered into these fields\n are not included in the \"add\" DSRequests used to store the uploaded records.  Instead, they\n are stored as HttpSession attributes with the names corresponding to the \n names of the specified <code>uploadFormFields</code> (optionally with a \n ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadFieldPrefix','prefix')} applied, in case\n this is necessary to avoid name collisions in the session).  This allows any custom logic\n for the \"add\" operation to access these additional fields via httpSession.getAttribute().  \n <P>\n Because all records are saved in a single HTTP request, a similar strategy of \n storing data as servletRequest or session attributes allows reuse of objects required to\n perform the \"add\" operations (such as field values common to all added records,\n or a SQL connection or transaction manager). \n <p>\n If ${isc.DocUtils.linkForRef('attr:DataSourceField.uploadFieldName','uploadFieldName')} is set on any of the \n ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadDataSource','uploadDataSource')}'s fields, the BatchUploader will use that name to \n map the uploaded file's content.\n <p>\n A couple of server-side techniques are interesting in conjunction with the BatchUploader.\n One is to set the ${isc.DocUtils.linkForRef('attr:DataSource.serverConstructor')} property to point at your own class\n that inherits from <code>com.isomorphic.datasource.DataSource</code>.  The most interesting\n reason for doing this is to override the <code>validate</code> method and provide complete\n custom validation - for example, checking relations to other tables.\n <p>\n Another technique is to handle the initial SmartClient call in your own servlet, by setting the \n ${isc.DocUtils.linkForRef('attr:BatchUploader.dataURL','dataURL')} property.  You than handle the add requests\n with a combination of your own code and SmartClient server API calls.  This is a \n good way to add special pre- and post-processing to the normal server-side flow.\n <p>\n The BatchUploader sample, provided as one of the databinding examples, demonstrates both of\n these techniques.  We use a special subclass of DataSource and override the validate method\n as described above; we also set ${isc.DocUtils.linkForRef('attr:BatchUploader.dataURL','dataURL')} to point at a JSP that\n wraps the queue of SmartClient requests inside a single database transaction (a servlet\n would be more usual, but we use a JSP for ease of modification)\n <p>\n <b>Note:</b> The special \"batchUpload\" DataSource, which should reside in the shared/ds \n folder of your application's webroot (see ${isc.DocUtils.linkForRef('group:iscInstall')}), \n is not part of your application's data flow, and it has nothing to do with the \n ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadDataSource','uploadDataSource')} you use to actually persist the validated\n and error-corrected data: it is simply a means to uploading the raw data in the first place.\n Normally, you should simply ignore its presence and treat it as an internal detail of the \n SmartClient framework.\n <p>\n However, there are circumstances in which you may wish to change it to achieve specific aims.\n For example, you may wish to override the Java class it invokes, in order to insert your own\n security or other validation logic into the initial upload flow.  This is entirely in keeping\n with the design, but we regard it as an out-of-the-ordinary use-case: normal usage is simply \n to ignore the presence of the batchUpload DataSource.",
        name:"BatchUploader",
        seeAlso:[],
        type:"class"
    },
    "attr:ImgButton.showRollOverIcon":{
        ref:"attr:ImgButton.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:DataSource.serverObject":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:DataSource.serverObject",
        description:"For Direct Method Invocation (DMI) binding, declares the ServerObject to use as the default\n target for all ${isc.DocUtils.linkForRef('attr:DataSource.operationBindings')}.  Specifying this\n attribute in an XML DataSource stored on the server enables DMI for this DataSource.",
        name:"serverObject",
        valueType:"ServerObject",
        seeAlso:[],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["serverDataIntegration"],
        definingClass:"class:DataSource"
    },
    "method:Canvas.clear":{
        ref:"method:Canvas.clear",
        flags:"A",
        description:" Remove all visual representation of a Canvas.\n <P>\n This is far more expensive than hide(), because in order to become visible again, the Canvas must\n be draw()n again.  Generally, application code has no reason to call clear(); if you want to\n temporarily hide a Canvas, use hide() and show(), and if you want to permanently destroy a\n Canvas, use destroy().\n <P>\n Note: a clear() will happen as part of moving a Canvas to a different parent.  See addChild().\n\n",
        name:"clear",
        seeAlso:[
            {ref:"method:Canvas.addChild", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:SectionStack.sectionIsVisible":{
        ref:"method:SectionStack.sectionIsVisible",
        flags:"",
        description:"\n Returns true if the specified section is visible, false if it is not.  A section is\n visible if it shows a header and the header is visible or if it has items and the first\n item is visible.\n\n",
        name:"sectionIsVisible",
        seeAlso:[],
        params:[
            {description:"Section for which you want to obtain visibility information.\n                      For this parameter, you can pass the position of the section in the\n                      SectionStack, or the ID of the section.",
             optional:false, name:"section", defaultValue:null, type:"sectionId|position"}
        ],
        returns:{description:"true if the section is visible, false if it is not.", type:"boolean"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:ListGridField.filterEditorValueMap":{
        ref:"attr:ListGridField.filterEditorValueMap",
        flags:"IRW",
        description:"If this listGrid is showing a filter row, this property can be used to specify a\n  mapping of internal data to/from display values to be in the appropriate filter\n  row form item.",
        name:"filterEditorValueMap",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "method:StatefulCanvas.setIcon":{
        ref:"method:StatefulCanvas.setIcon",
        flags:"",
        description:" Change the icon being shown next to the title text.\n",
        name:"setIcon",
        seeAlso:[],
        params:[
            {description:"URL of new icon", optional:false, name:"icon", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:ColumnTree.initialCriteria":{
        ref:"attr:ColumnTree.initialCriteria",
        flags:"IR",
        description:"Criteria to be used when ${isc.DocUtils.linkForRef('attr:ColumnTree.autoFetchData')} is set.",
        name:"initialCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColumnTree"
    },
    "attr:BatchUploader.commitButton":{
        ref:"attr:BatchUploader.commitButton",
        flags:"IR",
        description:"Button that commits changes once the user is happy with the data.",
        name:"commitButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:CubeGrid.getFacetValuesColumn":{
        ref:"method:CubeGrid.getFacetValuesColumn",
        flags:"",
        description:" Get the index of the first column in the grid that matches the specified FacetValueMap.<br>\n The facetValues passed in should contain values for at least one column facet. If values are \n sparse (values not specified for every column facet), the first column matching the specified\n facet values will be returned.\n",
        name:"getFacetValuesColumn",
        seeAlso:[],
        params:[
            {description:"facet values to find", optional:false, name:"facetValues", defaultValue:null,
             type:"facetValueMap"}
        ],
        returns:{description:"index of first column in the grid that matches the facet values passed in.",
         type:"integer"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:HTMLFlow.contentsType":{
        ref:"attr:HTMLFlow.contentsType",
        flags:"IR",
        description:"The default setting of 'null' or 'fragment' indicates that HTML loaded from\n ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsURL')} is assumed to be an HTML fragment rather than a complete page.  Set to\n \"page\" to load HTML as a standalone page, via an IFRAME.  \n <P>\n <code>contentsType:\"page\"</code> should only be used for controlled HTML content, and only\n when such content cannot be delivered as an HTML fragment instead (the default).  To\n dynamically load SmartClient components, use ${isc.DocUtils.linkForRef('class:ViewLoader')}, <b>never</b> this mechanism\n (click ${isc.DocUtils.linkForRef('group:noFrames','here')} for why).\n <P>\n Loading HTML content as a fragment is less resource intensive and avoids visual artifacts\n such as translucent media becoming opaque or disappearing when placed over an IFRAME.  \n <P>\n Loading third-party, uncontrolled content could lead to the surrounding page disappearing if\n a user clicks on an HTML link with <code>target=_top</code>.\n <P>\n With <code>contentsType:\"page\"</code>, ${isc.DocUtils.linkForRef('attr:HTMLFlow.loadingMessage')} is not supported, and only\n \"GET\" is supported for ${isc.DocUtils.linkForRef('attr:HTMLFlow.httpMethod','httpMethod')}.",
        name:"contentsType",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:CubeGrid.facetTitleAlign":{
        ref:"attr:CubeGrid.facetTitleAlign",
        flags:"IRW",
        description:"default alignment for facet labels",
        name:"facetTitleAlign",
        setter:["setFacetTitleAlign"],
        seeAlso:[],
        valueType:"alignEnum",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "classAttr:Validator.notADate":{
        ref:"classAttr:Validator.notADate",
        flags:"IRA",
        description:"Default error message to display when standard <code>isDate</code> type validator\n  returns false.",
        name:"notADate",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be a date.\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.title":{
        ref:"attr:ListGridField.title",
        flags:"IRW",
        description:"A title for this field, to display in the header of the listGrid object.  Alternately you can\n specify a ${isc.DocUtils.linkForRef('method:ListGridField.getFieldTitle')} method on the field to return the HTML for the field title.",
        name:"title",
        seeAlso:[
            {ref:"method:ListGridField.getFieldTitle", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "method:ListGrid.getRecordIndex":{
        ref:"method:ListGrid.getRecordIndex",
        flags:"",
        description:" Get the index of the provided record.\n <P>\n This is essentially the same as calling listGrid.data.indexOf(record), except that \n the currently visible range of records is checked first.  This is important for\n responsiveness in functions that respond to user actions when the user is working near the\n end of a very large dataset (eg 500k records).\n\n",
        name:"getRecordIndex",
        seeAlso:[],
        params:[
            {description:"the record whose index is to be retrieved", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"}
        ],
        returns:{description:"indexindex of the record, or -1 if not found", type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DynamicForm.errorItemProperties":{
        ref:"attr:DynamicForm.errorItemProperties",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is false we show all errors for the form item in \n a single item rendered at the top of the form.<br>\n This attribute contains a properties block for this item.",
        name:"errorItemProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "attr:ListGrid.removeFieldProperties":{
        ref:"attr:ListGrid.removeFieldProperties",
        flags:"IR",
        description:"Configuration properties for the \"remove field\" displayed when\n ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} is enabled. These configuration settings will be overlaid\n on top of the ${isc.DocUtils.linkForRef('attr:ListGrid.removeFieldDefaults')}.",
        name:"removeFieldProperties",
        seeAlso:[],
        valueType:"ListGridField properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:DetailViewerField.height":{
        ref:"attr:DetailViewerField.height",
        flags:"IR",
        description:"For ${isc.DocUtils.linkForRef('attr:DetailViewerField.type')}: <code>\"separator\"</code>, this attribute specifies\n the height of the separator.",
        name:"height",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:DetailViewerField"
    },
    "group:cursor":{
        ref:"group:cursor",
        refs:["attr:Splitbar.cursor", "attr:Splitbar.vResizeCursor", "attr:Splitbar.hResizeCursor"],
        name:"cursor",
        seeAlso:[],
        type:"group"
    },
    "attr:ImgButton.actionType":{
        flags:"IRW",
        getter:["getActionType"],
        type:"attr",
        ref:"attr:ImgButton.actionType",
        description:"Behavior on state changes -- BUTTON, RADIO or CHECKBOX",
        name:"actionType",
        setter:["setActionType"],
        valueType:"SelectionType",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('buttonRadioToggle')}"],
        defaultValue:"\"button\"",
        groups:["state", "event handling"],
        definingClass:"class:ImgButton"
    },
    "attr:FormulaBuilder.builderTypeText":{
        ref:"attr:FormulaBuilder.builderTypeText",
        flags:"IR",
        description:"Indicates whether to use \"formula\" or some other keyword in various captions and text",
        name:"builderTypeText",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"formula\"",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:ListGrid.groupSummaryRecordProperty":{
        ref:"attr:ListGrid.groupSummaryRecordProperty",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, this attribute will be set to true on each\n record object representing a group-level summary row.",
        name:"groupSummaryRecordProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"isGroupSummary\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Menu.showSubmenu":{
        ref:"method:Menu.showSubmenu",
        flags:"A",
        description:" Show the submenu for the specified item, if it has one.\n <P>\n Normally triggered automatically by user interaction.\n\n",
        name:"showSubmenu",
        seeAlso:[],
        params:[
            {description:"the item in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem | number"}
        ],
        type:"method",
        definingClass:"class:Menu",
        groups:["visibility"]
    },
    "group:skinning":{
        ref:"group:skinning",
        title:"Skinning / Theming",
        treeLocation:["Concepts"],
        refs:[],
        description:"Skinning (aka \"theming\" or \"branding\") is the process of modifying SmartClient's default\n look and feel to match the desired look and feel for your application.  SmartClient supports\n an extremely powerful and simple skinning system that allows designers with a basic grasp of\n CSS and JavaScript to skin any SmartClient component.\n <P>\n <h4>Basics</h4>\n <P>\n <ul>\n <li> SmartClient components create their visual appearance by dynamically generating HTML,\n within the browser, using JavaScript.\n\n <li> the HTML generated by SmartClient components contains CSS style names and URLs to\n images\n\n <li> SmartClient components can be skinned by replacing the CSS styles and images that\n the components use by default, or by using JavaScript properties to configure\n components to use new CSS styles and new image URLs.\n\n <li> You can change the appearance of an individual SmartClient component by passing \n properties to ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')}, or you can skin all components of the\n same class at once, by using ${isc.DocUtils.linkForRef('classMethod:Class.addProperties','addProperties()')} and \n ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','changeDefaults()')} to change the defaults for the class.\n\n <li> A \"skin\" consists of:\n <ul>\n <li> a single CSS stylesheet containing all CSS styles used by SmartClient components\n (<code>skin_styles.css</code>)\n <li> a single JavaScript file that sets component defaults (<code>load_skin.js</code>)\n <li> a directory tree of images organized by component\n </ul>\n \n <li>\n The example skins that come with SmartClient are in\n <code>isomorphicSDK/isomorphic/skins</code>.  The standard filesystem layout for a skin is:\n <pre>\n    isomorphic/skins\n        skin_styles.css\n        load_skin.js\n        images/\n            ListGrid/\n                sort_ascending.gif\n                ...\n            Tab/\n            ... other directories containing\n                component or shared media ...\n </pre>\n <li> A skin is loaded via a &lt;SCRIPT SRC=&gt; tag that loads load_skin.js, or, if using\n the SmartClient server, by specifying the \"skin\" property of the ${isc.DocUtils.linkForRef('group:loadISCTag')}.\n load_skin.js loads the stylesheet and sets the CSS styleNames and media URLs that\n SmartClient components will use.\n </ul>\n <P>\n <h4>Modifying Skins</h4>\n <P>\n To modify a skin, first create a copy of one of the skins that comes with the SmartClient\n SDK, then modify the copy.  Full instructions are provided in Chapter 9 of the \n ${isc.DocUtils.linkForDocNode('QuickStartGuide', 'QuickStart Guide')}.\n <P>\n <h4>Locating Skinning Properties</h4>\n <P>\n <b>Starting from the name of the component</b>\n <P>\n Given a SmartClient component that you want to skin, use the search feature of the SmartClient\n Reference to locate it, and open the \"Instance APIs\" tab.\n <ul>\n <li> for properties that set CSS styles, look for properties whose name includes \"style\", eg\n ${isc.DocUtils.linkForRef('attr:Button.baseStyle')}\n <li> for properties that control URLs to media, look for properties whose name includes\n \"src\", \"image\" or \"icon\", such as ${isc.DocUtils.linkForRef('attr:Img.src')}\n <li> for subcomponents that also support skinning, look for properties of type \"AutoChild\"\n and check the reference for the type of the AutoChild for settable properties.  For example,\n ${isc.DocUtils.linkForRef('attr:Window.minimizeButton')} is an ImgButton and therefore supports ${isc.DocUtils.linkForRef('attr:ImgButton.src')}.\n </ul>\n <b>TIP</b>: the Instance APIs tab allows you to search within just the current class, limit \n the display to just properties or methods, and sort by type.\n <P>\n <b>Starting from a running example</b>\n <P>\n Open the Developer Console and use the Watch Tab to locate the component or subcomponent you \n want to skin, then locate it in the documentation, as above.\n <P>\n If you don't find the component in the documentation, it may be a custom component specific\n to your organization.  To find the base SmartClient component for a component named\n \"MyComponent\", use the following code to find out the name of the superclass:\n <pre>\n     isc.<i>MyComponent</i>.getSuperClass().getClassName()\n </pre>\n Repeat this until you arrive at a SmartClient built-in class.  You can execute this code in\n the \"Eval JS\" area of the Results pane of the Developer Console.\n <P>\n Specific browsers offer alternate approaches to quickly discover the images or style names\n being used for a part of a SmartClient component's appearance: \n <ul>\n <li> the Firefox browser offers a dialog via Tools->\"Page Info\" that gives a manifest of\n media used in the page.\n <li> the <a href='http://www.getfirebug.com/' onclick=\"window.open('http://www.getfirebug.com/');return false;\">Firebug</a> extension for Firefox has an\n \"Inspect\" feature that allows you to see the HTML, CSS and media in use for a given area of\n the screen\n <li> right clicking (option-click on a Mac) on an image and choosing \"Properties\" shows a\n dialog that provides the image URL in most browsers.  Tips:\n <ul>\n <li> if a SmartClient component is showing text over an image, right-click at the very edge of\n the underlying image to get image properties rather than information about the text label\n <li> on some browsers, in order to see the full image URL, you may need to drag select the \n partial URL of the image shown in the properties dialog\n </ul>\n </ul>\n <P>\n <h4>Image URLs in SmartClient</h4>\n <P>\n Properties that refer to images by URL, such as ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:Button.icon')}, are\n specially interpreted in SmartClient to allow for simpler and more uniform image URLs,\n and to allow applications to be restructured more easily.\n <P>\n Unlike the URL used with an HTML &lt;IMG&gt; element, the image URL passed to a SmartClient\n component is not assumed to be relative to the current page.  See ${isc.DocUtils.linkForRef('type:SCImgURL')} for a\n full explanation of the default application image directory, and the meaning of the \"[SKIN]\"\n prefix.\n <P>\n <h4>Specifying Image URLs</h4>\n <P>\n Default image URLs for SmartClient components are specified in <code>load_skin.js</code> via\n JavaScript, using calls to ${isc.DocUtils.linkForRef('classMethod:Class.addProperties')} and\n ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')}.  For example, the <code>load_skin.js</code> file\n from the \"SmartClient\" sample skin includes the following code to establish the media used by\n ${isc.DocUtils.linkForRef('attr:Window.minimizeButton')}:\n <pre>\n    isc.Window.changeDefaults(\"minimizeButtonDefaults\", { \n         src:\"[SKIN]/Window/minimize.png\"\n    });\n </pre>\n <P>\n <h4>Specifying Image Sizes</h4>\n <P>\n Many SmartClient components must know some image sizes in advance, in order to allow those\n components to autosize to data or content.\n <P>\n For example, the ${isc.DocUtils.linkForRef('class:ImgTab')}s used in ${isc.DocUtils.linkForRef('class:TabSet')}s are capable of automatically sizing\n to a variable length ${isc.DocUtils.linkForRef('attr:Tab.title')}.  To make this possible, SmartClient must know the\n sizes of the images used as \"endcaps\" on each tab in advance.\n <P>\n Like image URLs, image sizes are specified in <code>load_skin.js</code>.  The following code\n sample establishes the default size of the \"endcaps\" for tabs, by setting a default value\n for ${isc.DocUtils.linkForRef('attr:ImgTab.capSize')}:\n <pre>\n     isc.ImgTab.addProperties({\n         capSize:4\n     })\n </pre>\n <P>\n <h4>CSS usage in SmartClient</h4>\n <P>\n In SmartClient, screen layout and sizing are controlled via JavaScript, and appearance via\n CSS and images.  \n <P>\n CSS borders, margins and padding applied to SmartClient components can be treated as purely\n visual properties with no effect on sizing or layout.  Unlike HTML elements, a SmartClient\n component will always have the exact size you specify via JavaScript, regardless of browser\n platform, browser compatibility mode, or borders, margins, or padding, all of which normally\n affect the final size of an HTML element. \n <P>\n For this reason, SmartClient skinning requires only novice-level familiarity with CSS, as CSS\n is used principally for colors and fonts.  See ${isc.DocUtils.linkForRef('type:CSSStyleName','this discussion')} for\n further details on what properties should be set via CSS vs via JavaScript.\n <P>\n <h4>Statefulness and Suffixes</h4>\n <P>\n Some components or areas within components, including buttons and the cells within a grid, are\n \"stateful\", meaning that they can be in one of a set of states each of which has a distinct\n visual appearance.\n <P>\n Stateful components switch the CSS styles or image URLs they are using as they transition\n from state to state, appending state information as suffixes on the style names or URL.\n See ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:Button.baseStyle')} for details and examples.\n <P>\n SmartClient has built-in logic to manage a series of state transitions, such as:\n <ul>\n <li> \"rollover\": showing a different appearance when the mouse is over a component\n <li> \"button down\": showing a different appearance when the mouse is pressed over a\n component\n <li> \"disabled\": showing a different appearance when a component cannot be interacted with\n <li> \"selected\": showing one of a set of components in a different state to indicate\n selection\n </ul>\n Flags on some components, such as ${isc.DocUtils.linkForRef('attr:ImgButton.showRollOver')}, allow you to control whether the\n component will switch CSS style or image URL when the component transitions into a given state.\n <P>\n <h4>StretchImg: 3-segment stretchable images</h4>\n <P>\n A ${isc.DocUtils.linkForRef('class:StretchImg')} is SmartClient component that renders out a compound image composed of 3\n image files: two fixed-size endcaps images and a stretchable center segment.  Like stateful\n components, the names of each image segment is appended to the image URL as a suffix.  See\n ${isc.DocUtils.linkForRef('attr:StretchImg.src')} for details.\n <P>\n <h4>EdgedCanvas</h4>\n <P>\n Similar to a StretchImg, an ${isc.DocUtils.linkForRef('class:EdgedCanvas')} provides an image-based decorative edge\n around and/or behind another component, with up to 9 segments (a 3x3 grid).  Decorative\n edges can be added to any component by setting ${isc.DocUtils.linkForRef('attr:Canvas.showEdges','showEdges:true')}.\n EdgedCanvas is also used to construct dropshadows, which can be enabled on any component via\n ${isc.DocUtils.linkForRef('attr:Canvas.showShadow','showShadow:true')}.\n <P>\n <h4>Multiple looks for the same component type</h4>\n <P>\n In some cases you need to create two variations in appearance for a component with the same\n behavior.  For example, you may want to create a specialized Window, called \"PaletteWindow\",\n that behaves like a normal Window but has a very compact look & feel.  To create a\n separately skinnable component for PaletteWindow, use ${isc.DocUtils.linkForRef('classMethod:isc.defineClass')}.  For\n example:\n <pre>\n    isc.defineClass(\"PaletteWindow\", \"Window\");\n    isc.PaletteWindow.addProperties({\n        showFooter:false,\n        ...\n    })\n </pre>",
        name:"skinning",
        seeAlso:[],
        type:"group"
    },
    "method:GridRenderer.rowOver":{
        ref:"method:GridRenderer.rowOver",
        flags:"A",
        description:" Called when the mouse pointer enters a row\n\n",
        name:"rowOver",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:ValuesManager.setDataSource":{
        ref:"method:ValuesManager.setDataSource",
        flags:"A",
        description:" Specifies a dataSource for this valuesManager.  This dataSource will then be used for\n validation and client-server flow methods.\n",
        name:"setDataSource",
        seeAlso:[],
        params:[
            {description:"Datasource object or identifier to bind to.", optional:false,
             name:"dataSource", defaultValue:null, type:"string | DataSource"}
        ],
        type:"method",
        definingClass:"class:ValuesManager"
    },
    "attr:Slider.trackSrc":{
        ref:"attr:Slider.trackSrc",
        flags:"IRW",
        description:"The base filename for the slider track images.\n The filenames for the track icons are assembled from this base filename and the state of the\n slider, as follows:<br>\n Assume the trackSrc is set to <code>{baseName}.{extension}</code><br>\n The full set of images to be displayed is:<br>\n For horizontal sliders:\n <ul>\n <li><code>h{baseName}_start.{extension}</code>: start (left edge) of the track for a slider\n     that is enabled.\n <li><code>h{baseName}_stretch.{extension}</code>:  the track for an enabled slider; this may\n     be centered, tiled, or stretched.\n <li><code>h{baseName}_end.{extension}</code>:  end (right edge) of the track for a slider\n     that is enabled.\n <li><code>h{baseName}_off_start.{extension}</code>: start (left edge) of the track for a slider\n     that is disabled.\n <li><code>h{baseName}_off_stretch.{extension}</code>:  the track for a disabled slider; this\n     may be centered, tiled, or stretched.\n <li><code>h{baseName}_off_end.{extension}</code>:  end (right edge) of the track for a slider\n     that is disabled.\n </ul>\n For vertical sliders:\n <ul>\n <li><code>v{baseName}_start.{extension}</code>: start (bottom edge) of the track for a slider\n     that is enabled.\n <li><code>v{baseName}_stretch.{extension}</code>:  the track for an enabled slider; this may\n     be centered, tiled, or stretched.\n <li><code>v{baseName}_end.{extension}</code>:  end (top edge) of the track for a slider\n     that is enabled.\n <li><code>v{baseName}_off_start.{extension}</code>: start (bottom edge) of the track for a slider\n     that is disabled.\n <li><code>v{baseName}_off_stretch.{extension}</code>:  the track for a disabled slider; this\n     may be centered, tiled, or stretched.\n <li><code>v{baseName}_off_end.{extension}</code>:  end (top edge) of the track for a slider\n     that is disabled.\n </ul>",
        name:"trackSrc",
        seeAlso:[
            {ref:"attr:Slider.trackImageType", description:null}
        ],
        valueType:"String",
        defaultValue:"\"track.gif\"",
        type:"attr",
        definingClass:"class:Slider"
    },
    "type:TreeModelType":{
        ref:"type:TreeModelType",
        values:[
            {description:"In this model, each node has an ID unique across the whole tree and a parent ID that points to its parent.  The name of the unique ID property can be specified\n via ${isc.DocUtils.linkForRef('attr:Tree.idField')} and the name of the parent ID property can be specified via\n ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')}.  The initial set of nodes can be passed in as a list to\n ${isc.DocUtils.linkForRef('attr:Tree.data')} and also added as a list later via ${isc.DocUtils.linkForRef('method:Tree.linkNodes')}.\n Whether or not a given node is a folder is determined by the value of the property specified\n by ${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty')}.\n <br><br>\n The \"parent\" modelType is best for integrating with relational storage (because nodes can\n map easily to rows in a table) and collections of Beans and is the model used for DataBound\n trees.",
             value:"\"parent\""},
            {description:"In this model, nodes specify their children as a list of nodes.  The property that holds the children nodes is determined by ${isc.DocUtils.linkForRef('attr:Tree.childrenProperty')}.\n Nodes are not required to have an ID that is unique across the whole tree (in fact, no ID is\n required at all).  Node names (specified by the ${isc.DocUtils.linkForRef('attr:Tree.nameProperty')}, unique within\n their siblings, are optional but not required.  Whether or not a given node is a folder is\n determined by the presense of the children list (${isc.DocUtils.linkForRef('attr:Tree.childrenProperty')}).",
             value:"\"children\""}
        ],
        description:null,
        name:"TreeModelType",
        seeAlso:[],
        type:"type"
    },
    "method:ListGrid.editComplete":{
        ref:"method:ListGrid.editComplete",
        flags:"",
        description:" Callback fired when inline edits have been successfully saved.\n <P>\n No default implementation.\n\n",
        name:"editComplete",
        seeAlso:[],
        params:[
            {description:"current index of the row that was saved", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"index of the column that was saved, if applicable", optional:false,
             name:"colNum", defaultValue:null, type:"number"},
            {description:"new values that were saved", optional:false, name:"newValues",
             defaultValue:null, type:"Record"},
            {description:"the complete original values from before the save occurred", optional:false,
             name:"oldValues", defaultValue:null, type:"Record"},
            {description:"Event that led to the save", optional:false, name:"editCompletionEvent",
             defaultValue:null, type:"editCompletionEvent"},
            {description:"for DataSource saves, DSResponse object returned", optional:true,
             name:"dsResponse", defaultValue:null, type:"DSResponse"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:TreeNode.enabled":{
        ref:"attr:TreeNode.enabled",
        flags:"IR",
        description:"Default property name denoting whether this record is enabled. Property name may be modified\n for some grid via ${isc.DocUtils.linkForRef('attr:ListGrid.recordEnabledProperty')}.",
        name:"enabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('disabledRows')}"],
        definingClass:"object:TreeNode"
    },
    "group:rects":{
        ref:"group:rects",
        refs:[],
        name:"rects",
        seeAlso:[],
        type:"group"
    },
    "method:CubeGrid.getColumnFacetValues":{
        ref:"method:CubeGrid.getColumnFacetValues",
        flags:"",
        description:" Return a ${isc.DocUtils.linkForRef('type:FacetValueMap','FacetValueMap')} indicating the facet values for a specific \n column in the grid\n",
        name:"getColumnFacetValues",
        seeAlso:[],
        params:[
            {description:"index of the column", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"facet values for the specified column. Returns null if the specified       column is not present in the grid.",
         type:"FacetValueMap"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:Canvas.canHover":{
        ref:"attr:Canvas.canHover",
        flags:"IRW",
        description:"Will this Canvas fire hover events when the user hovers over it, or one of its children?",
        name:"canHover",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null},
            {ref:"method:Canvas.hover", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:Canvas.bringToFront":{
        ref:"method:Canvas.bringToFront",
        flags:"",
        description:" Puts this widget at the top of the stacking order, so it appears in front of all other\n widgets in the same parent.\n",
        name:"bringToFront",
        seeAlso:[],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('layer')}"],
        definingClass:"class:Canvas",
        groups:["zIndex"]
    },
    "classMethod:Page.moveTo":{
        ref:"classMethod:Page.moveTo",
        flags:"",
        description:"		Move the window to a specified top and left in screen coordinates.\n\n",
        name:"moveTo",
        seeAlso:[],
        params:[
            {description:"new left coordinate for window", optional:false, name:"left",
             defaultValue:null, type:"number"},
            {description:"new top coordinate for window", optional:false, name:"top", defaultValue:null,
             type:"number"}
        ],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "method:Class.evaluate":{
        ref:"method:Class.evaluate",
        flags:"",
        description:" \n Evaluate a string of script in the scope of this instance (so <code>this</code>\n is available as a pointer to the instance).\n\n",
        name:"evaluate",
        seeAlso:[
            {ref:"classMethod:Class.evaluate", description:null}
        ],
        params:[
            {description:"the expression to be evaluated", optional:false, name:"expression",
             defaultValue:null, type:"string"},
            {description:"Optional mapping of argument names to values - each key will      be available as a local variable when the script is executed.",
             optional:false, name:"evalArgs", defaultValue:null, type:"object"}
        ],
        returns:{description:"the result of the eval", type:"any"},
        type:"method",
        definingClass:"class:Class"
    },
    "class:FileLoader":{
        methods:[],
        ref:"class:FileLoader",
        classAttrs:["classAttr:FileLoader.defaultModules", "classAttr:FileLoader.defaultSkin"],
        classMethods:["classMethod:FileLoader.cacheISC", "classMethod:FileLoader.loadISC", 
         "classMethod:FileLoader.loadJSFiles","classMethod:FileLoader.loadModules", "classMethod:FileLoader.cacheFiles", 
         "classMethod:FileLoader.cacheModules","classMethod:FileLoader.cacheImgStates", "classMethod:FileLoader.cacheStretchImgStates", 
         "classMethod:FileLoader.cacheEdgeImages","classMethod:FileLoader.cacheShadows"],
        treeLocation:["Optional Modules/Network Performance"],
        attrs:[],
        description:"This class enables background (deferred) loading and caching of JS, CSS and Image files.  It is\n designed to work standalone from the rest of the SmartClient framework to provide a lightweight\n caching and loading mechanism for SmartClient modules as well as user-built application\n modules/fragments.\n <p>\n The most common usage scenarios are:\n <ul>\n   <li> Caching JS, CSS, Image files in the browser in anticipation of a transition to a page\n   that requires these files.  For example, a plain HTML (non-SmartClient) login page or\n   landing page can begin caching SmartClient in the background while allowing the user\n   to login, or giving the user something to read.  Normally, loading SmartClient or other\n   large JavaScript files would block page loading and display.  By loading SmartClient in\n   the background only after a simple HTML landing page has loaded, you can completely\n   eliminate perceived download time associated with loading a rich UI application, making a\n   much larger difference in user experience than any difference in framework/application size.\n   <li> Loading a multi-phase UI.  In this scenario, an initial rendering of a page is done with\n   minimal data transfer to the browser.  Then JS, CSS, and Image files are fetched in the\n   background to provide richer UI components.  During this time the user can continue to normally\n   interact with the initial page.  Once loading is complete, the UI is updated with richer\n   components.\n </ul>\n\n The recommended usage pattern is to use the <code>loadISC</code> custom tag provided as part of\n the SmartClient SDK.  You can specify <code>cacheOnly=\"true\"</code> to loadISC to cache the\n SmartClient framework in the background or alternately <code>defer=\"true\"</code> to load the\n SmartClient framework and make it available in the current page.  You can specify the\n <code>onload</code> attribute of the tag to provide a JavaScript callback to your code that will\n be called when the framework loading is complete.\n <p>\n If you're not working in a JSP environment, you can use the\n ${isc.DocUtils.linkForRef('classMethod:FileLoader.cacheISC')}/${isc.DocUtils.linkForRef('classMethod:FileLoader.loadISC')} APIs to accomplish\n the same effect as the <code>loadISC</code> JSP tag.\n <p>\n Additional APIs are provided for performing dynamic caching and loading of other JS, CSS, and\n Image files to improve the performance of your application.  See below.\n <p>\n <b>You must set <code>window.isomorphicDir</code> before loading and using this module unless the\n default of '../isomorphic/' is accepable.  E.g. if your html file is in your toplevel webroot\n directory, then your HTML file should say (note the trailing slash):</b>\n <pre>\n &lt;SCRIPT&gt;window.isomorphicDir='isomorphic/'&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/modules/ISC_FileLoader.js&gt;&lt;/SCRIPT&gt;\n </pre>\n \n This module is usable independent of the rest of SmartClient - you can use it on pages that\n don't load any other modules.  In practice, the general pattern is to use this module on\n static HTML pages such as a login page to pre-cache SmartClient modules, application logic,\n skin files, and css so that once the user logs in, there's no latency to load the rich UI.\n <p>\n Note: You can also reference this class via the alias isc.FL",
        name:"FileLoader",
        seeAlso:[],
        type:"class",
        requiresModules:["NetworkPerformance"]
    },
    "attr:TimeItem.displayFormat":{
        ref:"attr:TimeItem.displayFormat",
        flags:"IRW",
        description:"What format should this item's time string be presented in?",
        name:"displayFormat",
        seeAlso:[],
        valueType:"timeFormatter",
        defaultValue:"\"toShort24HourTime\"",
        type:"attr",
        definingClass:"class:TimeItem"
    },
    "method:Class.Super":{
        ref:"method:Class.Super",
        flags:"",
        description:"\n Call the SuperClass implementation of an instance method.  For example:\n <pre>\n    isc.defineClass(\"MyButton\", \"Button\").addProperties({\n        // this override causes no change in behavior because it just \n        // calls Super and returns whatever the superclass would return\n        getTitle : function () {\n            return this.Super(\"getTitle\", arguments);\n        },\n\n        // this override would add \"foo\" to the titles of all buttons\n        getTitle : function () {\n            // add code here to take actions before the superclass method is invoked\n\n            var superResult = return this.Super(\"getTitle\", arguments);\n\n            // add code here to take action after the superclass method is invoked\n\n            return superResult + \"foo\";\n        }\n\n    })\n </pre>\n Note that Super is always called with the name of the current method.  You cannot call\n the Super class implementation of another method directly.\n <P>\n It is <b>required</b> to always pass the native 'arguments' object to Super.  Arguments\n is a JavaScript builtin that is available within any JavaScript function - see any\n JavaScript Reference for details.\n <P>\n See also ${isc.DocUtils.linkForRef('classMethod:ClassFactory.defineClass','defineClass()')} and\n ${isc.DocUtils.linkForRef('classMethod:Class.addProperties','addProperties')} for the basics of creating classes\n and overriding methods.\n\n",
        name:"Super",
        seeAlso:[],
        params:[
            {description:"name of the superclass method to call", optional:false, name:"methodName",
             defaultValue:null, type:"string"},
            {description:"native \"arguments\" object, or array of                                           arguments to pass to the Super call",
             optional:false, name:"args", defaultValue:null, type:"arguments or Array"},
            {description:"native \"arguments\" object, required if an Array is                                  passed for the \"args\" parameter in lieue of the native\n                                  arguments object",
             optional:true, name:"nativeArgs", defaultValue:null, type:"arguments"}
        ],
        returns:{description:"return value of the superclass call", type:"any"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:StatefulCanvas.showDownIcon":{
        ref:"attr:StatefulCanvas.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "attr:ColorItem.allowComplexMode":{
        ref:"attr:ColorItem.allowComplexMode",
        flags:"IR",
        description:"Should \"complex\" mode be allowed for the ${isc.DocUtils.linkForRef('class:ColorPicker')} window associated with  \n this ColorItem?<p>\n If false, no \"More\" button is shown on the simple picker",
        name:"allowComplexMode",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "attr:Scrollbar.startThumbOverlap":{
        ref:"attr:Scrollbar.startThumbOverlap",
        flags:"IRA",
        description:"Number of pixels the thumb is allowed to overlap the buttons at the start of the track.\n Default prevents doubling of 1px borders.  Set higher to allow media that shows curved\n joins between the track button and ScrollThumb.",
        name:"startThumbOverlap",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "attr:Calendar.previousButtonHoverText":{
        ref:"attr:Calendar.previousButtonHoverText",
        flags:"IR",
        description:"The text to be displayed when a user hovers over the ${isc.DocUtils.linkForRef('attr:Calendar.previousButton','previous')}\n toolbar button",
        name:"previousButtonHoverText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Previous\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "attr:Slider.maxValue":{
        ref:"attr:Slider.maxValue",
        flags:"IRW",
        description:"The maximum slider value. The slider value is equal to maxValue when the thumb is at the\n top or right of the slider (unless flipValues is true, in which case the maximum value\n is at the bottom/left of the slider)",
        name:"maxValue",
        setter:["setMaxValue"],
        seeAlso:[
            {ref:"attr:Slider.flipValues", description:null}
        ],
        valueType:"float",
        defaultValue:"100",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:Slider"
    },
    "attr:Calendar.workdays":{
        ref:"attr:Calendar.workdays",
        flags:"IR",
        description:"Array of days that are considered workdays when ${isc.DocUtils.linkForRef('attr:Calendar.showWorkday')} is true",
        name:"workdays",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"[1,2,3,4,5]",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday"]
    },
    "attr:OperationBinding.requiresRole":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:OperationBinding.requiresRole",
        description:"Comma-separated list of user roles that are allowed to invoke the operation described by\n this operationBinding.\n <P>\n Whether the current user has a given role is determined by calling the standard Java\n servlets method <code>httpServletRequest.isUserInRole()</code>, hence works with both simple\n J2EE security (realms and form-based authentication) and JAAS (Java Authentication &\n Authorization Service).\n <P>\n If you wish to use a role-based security scheme that does not make use of the servlet API's \n standards, SmartClient Server also implements the <code>setUserRoles</code> method\n on <code>RPCManager</code>.  You can use this API to tell SmartClient that all the \n requests in the queue currently being processed are associated with a user who has the roles\n you supply; in this case, SmartClient will not attempt to resolve the user's roles via \n <code>httpServletRequest.isUserInRole()</code>.\n <P>\n If there an operationBinding declared for a given operationType which does not have an\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId')}, that is, it is the default operationBinding for the type, then any other\n operationBinding of the same type is assumed to have the same setting for\n <code>requiresRole</code> as the default operationBinding for the operationType.  For\n example, given these declarations:\n <pre>\n     &lt;operationBinding operationType=\"fetch\" requiresRole=\"manager\"&gt;\n           ... settings ...\n      &lt;/operationBinding&gt;\n     &lt;operationBinding operationType=\"fetch\" operationId=\"fetchWithExtraFields\"&gt;\n           ... settings ...\n      &lt;/operationBinding&gt;\n </pre>\n The second operationBinding requires the \"manager\" role even though there is no explicit\n <code>requiresRole</code> declaration.  To prevent the \"manager\" role being required by the\n second operationBinding, add <code>requireRole=\"\"</code>.\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:DataSource.requiresRole')} is set, all operations on the DataSource require\n the roles set for the DataSource as a whole, even if they declare individual\n <code>requiresRole</code> attributes.\n <P>\n This property is valid only for a server-side DataSource when using the SmartClient Server.",
        name:"requiresRole",
        valueType:"String",
        seeAlso:[],
        serverDS:"only",
        defaultValue:"null",
        groups:["auth"],
        definingClass:"class:OperationBinding"
    },
    "method:DataSource.getFieldNames":{
        ref:"method:DataSource.getFieldNames",
        flags:"",
        description:" Retrieves the list of fields declared on this DataSource.\n\n",
        name:"getFieldNames",
        seeAlso:[],
        params:[
            {description:"If true, returns only those fields that are                                 not marked as hidden",
             optional:false, name:"excludeHidden", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"names of all fields declared on this DataSource", type:"Array of String"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "classMethod:XMLTools.loadWSDL":{
        ref:"classMethod:XMLTools.loadWSDL",
        flags:"A",
        description:" Load a WSDL file and create an instance of WebService that allows invoking operations and\n binding DataSources to web service operations.\n <P>\n The created WebService object is available in the callback as the single parameter\n \"service\", or can be retrieved via <code>WebService.get(serviceNamespace)</code>.\n <P>\n XML Schema present in the WSDL file will also be processed as described in\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')}.  However note that <b>imported</b> XML Schema\n (&lt;xs:import&gt; tag) will not be automatically loaded and must be loaded manually using\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')} before the loaded service will be usable.  This is because the WSDL\n spec allows but does not require a valid URL to be provided for loading imported XML Schema.\n <P>\n NOTE: unless you are building an application that dynamically contacts WSDL web services\n without prior knowledge, instead of calling loadWSDL(), you should either:\n <ul>\n <li> use the ${isc.DocUtils.linkForRef('group:loadWSDLTag')} tag to eliminate the need for an asynchronous\n download of a WSDL file as part of application startup, <b>OR</b>\n <li> use the \"WSDL\" tab in the Developer Console to obtain the WebService definition as a\n JavaScript file that can be loaded via a normal HTML &lt;SCRIPT SRC=&gt; tag and/or combined\n with other JavaScript files.\n </ul>\n Platform notes:\n <ul>\n <li> loadWSDL() is not supported in Safari 2.0 (but is supported in Safari 3.0.3 and greater)\n However, you can use either approach mentioned above (loadWSDLTag or JavaScript file) with \n Safari pre 3.0.3.\n <li> if you are using a non-Java server, in order to obtain a JavaScript file representing a\n web service, you must run the Developer Console in the Java-based SmartClient SDK\n </ul>\n\n",
        name:"loadWSDL",
        seeAlso:[],
        params:[
            {description:"URL to load the WSDL file from", optional:false, name:"wsdlURL",
             defaultValue:null, type:"URL"},
            {description:"signature is callback(service)", optional:false, name:"callback",
             defaultValue:null, type:"Callback"},
            {description:"additional properties to set on the RPCRequest                                         that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"RPCRequest"},
            {description:"if set, xsd:import statements will be processed                                  automatically to load dependent XSD files where a\n                                  \"location\" is specified.  The callback will not fire until\n                                  all dependencies have been loaded",
             optional:false, name:"autoLoadImports", defaultValue:null, type:"boolean"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('WSDLDataSource')}"],
        definingClass:"class:XMLTools",
        groups:["xmlSchema"]
    },
    "type:KeyIdentifier":{
        ref:"type:KeyIdentifier",
        description:"Identifiers for keys pressed by the user used by various methods.<br>\n Valid <code>keyIdentifier</code>s can be either ${isc.DocUtils.linkForRef('type:KeyName')} strings, or objects.<br>\n If a <code>keyIdentifier</code> is specified as an object, it should have the following\n properties:<br>\n - <code>keyName</code>: name of the key<br>\n - <code>ctrlKey</code>: optional boolean - true if ctrl is down.<br>\n - <code>shiftKey</code>: optional boolean - true if shift is down.<br>\n - <code>altKey</code>: optional boolean - true if alt is down.",
        name:"KeyIdentifier",
        seeAlso:[
            {ref:"type:KeyName", description:null}
        ],
        type:"type"
    },
    "attr:DataSourceField.valueXPath":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSourceField.valueXPath",
        description:"XPath expression used to retrieve the field's value.\n <P>\n This XPath expression will be evaluated in the scope of the record objects selected by \n the ${isc.DocUtils.linkForRef('attr:DataSource.recordXPath')}.  For XML data \n (${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','dataFormat:\"xml\"')}) this means a call to\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectString')} passing the selected XML element.  For JSON data\n (${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','dataFormat:\"json\"')}), this means a call to \n ${isc.DocUtils.linkForRef('classMethod:XMLTools.selectObjects')} passing the selected JSON object.\n <P>\n In the absence of a <code>valueXPath</code>, for JSON data the value for the field will\n be the value of the same-named property in the record object selected by\n ${isc.DocUtils.linkForRef('attr:DataSource.recordXPath','recordXPath')}.  \n <P>\n For XML data, the value will be the attribute or subelement named after the field name.\n For example, for a field \"author\" on a record element &lt;book&gt;, the following structures\n require no valueXPath:\n <pre>\n    &lt;book author=\"Mark Jones\"/&gt;\n\n    &lt;book&gt;\n        &lt;author&gt;Mark Jones&lt;/author&gt;\n    &lt;/book&gt;\n </pre>\n <P>\n If <code>valueXPath</code> is not required for your field because of the default handling\n described above, don't specify it, as it's slightly slower.\n <p>\n To learn about XPath, try the following search:\n <a href=\"http://www.google.com/search?q=xpath+tutorial\" target=\"_blank\"\n >http://www.google.com/search?q=xpath+tutorial</a>\n <P>\n <B>Using valueXPath with the SmartClient server</B>\n <p>\n If you're using the SmartClient server to return data via the DSResponse object (or\n indirectly doing so using DataSource DMI), the valueXPath you specify on the DataSource\n fields will be applied to the data you return via the \n <a href='http://jakarta.apache.org/commons/jxpath/' onclick=\"window.open('http://jakarta.apache.org/commons/jxpath/');return false;\">JXPath</a> library.\n <P>\n If you are returning Java Beans as your DSResponse data, normally each dataSource field\n receives the value of the same-named Java Bean property, that is, a field \"zipCode\" is\n populated by looking for \"getZipCode()\" on the objects passed as DSResponse data.  You can\n use <code>valueXPath</code> to retrieve properties from subobjects, so long as a chain of\n getter methods exists that corresponds to the valueXPath.  For example, a valueXPath of\n \"address/zipCode\" expects to call \"getAddress()\" on the bean(s) passed to\n DSResponse.setData(), followed by \"getZipCode()\" on whatever object \"getAddress()\" returns.\n <P>\n When you are saving data, the inbound DSRequest values, available as a Java Map, will use \n just dataSource field names as Map keys, not the valueXPath used to derive them.  However,\n to achieve bidirectional valueXPath binding, you can use the server-side method\n dataSource.setProperties() to use the valueXPath when setting properties on your server\n object model.  When applied as a setter, an XPath like \"address/zipCode\" attempts\n \"getAddress()\" followed by \"setZipCode()\" on the returned object.  JXPath also has some\n ability to auto-create intervening objects if they are missing, such as auto-creating an\n \"address\" subobject when applying \"address/zipCode\" as a valueXPath.\n <P>\n See the\n <a href='http://jakarta.apache.org/commons/jxpath/' onclick=\"window.open('http://jakarta.apache.org/commons/jxpath/');return false;\">JXPath</a> library documentation for\n complete details, including other types of server object models supported, such as\n server-side XML.",
        name:"valueXPath",
        valueType:"XPathExpression",
        seeAlso:["DataSource.setProperties() (Java API)", "DSResponse.setData() (Java API)"],
        examples:["${isc.DocUtils.linkForExampleId('xpathBinding')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration"],
        definingClass:"class:DataSourceField"
    },
    "attr:Snapbar.showGrip":{
        ref:"attr:Snapbar.showGrip",
        flags:"IRW",
        description:"Should we show a \"grip\" image floating above the center of this widget?",
        name:"showGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar",
        groups:["grip"]
    },
    "classMethod:Log.getPriority":{
        ref:"classMethod:Log.getPriority",
        flags:"",
        description:" Return the priority setting for a particular category.\n <P>\n If there is no priority setting specific to this category, <code>null</code> will be\n returned, NOT <code>Log.defaultPriority</code>.\n\n",
        name:"getPriority",
        seeAlso:[],
        params:[
            {description:"category name", optional:false, name:"category", defaultValue:null,
             type:"String"},
            {description:"Optional class or instance to check for                                               specific log priority overrides",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"}
        ],
        returns:{description:"priority setting", type:"LogPriority"},
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:DataBoundComponent.hilites":{
        ref:"attr:DataBoundComponent.hilites",
        flags:"IRW",
        description:"Hilites to be applied to the data for this component.  See ${isc.DocUtils.linkForRef('group:hiliting')}.",
        name:"hilites",
        seeAlso:[],
        valueType:"Array of Hilite",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:Date.toEuropeanShortDate":{
        ref:"method:Date.toEuropeanShortDate",
        flags:"",
        description:"			Return this date in the format: <code>DD/MM/YYYY</code>\n",
        name:"toEuropeanShortDate",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:FormItem.getDataPath":{
        ref:"method:FormItem.getDataPath",
        flags:"A",
        description:" Return the dataPath for the this formItem.\n",
        name:"getDataPath",
        seeAlso:[],
        returns:{description:"dataPath for this form item", type:"dataPath"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:ListGrid.enforceVClipping":{
        ref:"attr:ListGrid.enforceVClipping",
        flags:"IRW",
        description:"For performance reasons, even when ${isc.DocUtils.linkForRef('attr:ListGrid.fixedRecordHeights')} is set, vertical clipping\n is not enforced by default for some kinds of content (such as images) on all browsers.\n Set ${isc.DocUtils.linkForRef('attr:ListGrid.enforceVClipping','enforceVClipping:true')} to enforce clipping for all types of\n content on all browsers.\n <P>\n This additional setting is likely to be phased out as browsers improve.",
        name:"enforceVClipping",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Calendar.setTimelineRange":{
        ref:"method:Calendar.setTimelineRange",
        flags:"",
        description:" Sets the range over which the timeline will display events\n",
        name:"setTimelineRange",
        seeAlso:[],
        params:[
            {description:"start of range", optional:false, name:"start", defaultValue:null, type:"date"},
            {description:"end of range", optional:false, name:"end", defaultValue:null, type:"date"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:RadioGroupItem.wrap":{
        ref:"attr:RadioGroupItem.wrap",
        flags:"IRW",
        description:"Should the text for items within this radio group wrap?",
        name:"wrap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RadioGroupItem",
        groups:["appearance"]
    },
    "group:summaryFields":{
        ref:"group:summaryFields",
        refs:["method:SummaryBuilder.getHelpText", "method:SummaryBuilder.fireOnClose", 
         "method:DataBoundComponent.addSummaryField","method:DataBoundComponent.editSummaryField", "class:SummaryBuilder", 
         "attr:ListGrid.canAddSummaryFields","attr:SummaryBuilder.dataSource", "attr:SummaryBuilder.fields", 
         "attr:SummaryBuilder.editMode","attr:SummaryBuilder.field", "attr:SummaryBuilder.formulaField", 
         "attr:SummaryBuilder.titleField","attr:SummaryBuilder.showHelpIcon", "attr:SummaryBuilder.helpIcon", 
         "attr:SummaryBuilder.showAutoHideCheckBox","attr:SummaryBuilder.autoHideCheckBox", "attr:DataBoundComponent.canAddSummaryFields"],
        name:"summaryFields",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.booleanFalseImage":{
        ref:"attr:ListGrid.booleanFalseImage",
        flags:"IRWA",
        description:"Image to display for a false value in a boolean field. Default <code>null</code> value\n means no image will be displayed\n <P>\n To turn this off explicitly set ${isc.DocUtils.linkForRef('attr:ListGridField.suppressValueIcon')} to true\n <P>\n If this and ${isc.DocUtils.linkForRef('attr:ListGrid.booleanTrueImage')} are undefined, this will be set to\n ${isc.DocUtils.linkForRef('attr:CheckboxItem.uncheckedImage')}.",
        name:"booleanFalseImage",
        seeAlso:[
            {ref:"attr:ListGrid.booleanTrueImage", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:Applet.extraHTML":{
        ref:"attr:Applet.extraHTML",
        flags:"IR",
        description:"Any additional HTML you want to throw into the tag that specifies the applet element.",
        name:"extraHTML",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Applet"
    },
    "method:ListGrid.getTotalRows":{
        ref:"method:ListGrid.getTotalRows",
        flags:"",
        description:" Return the total number of rows in the grid.\n <P>\n Note that, when creating new rows via inline editing, this can be more than the total number\n of rows in the dataset (that is, grid.data.getLength())\n",
        name:"getTotalRows",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:SpacerItem.width":{
        ref:"attr:SpacerItem.width",
        flags:"IRW",
        description:"default width for the spacer",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:SpacerItem",
        groups:["appearance"]
    },
    "attr:DataSourceField.childTagName":{
        ref:"attr:DataSourceField.childTagName",
        flags:"IR",
        description:"For a field that is ${isc.DocUtils.linkForRef('attr:DataSourceField.multiple','multiple:\"true\"')}, controls the name of the XML tag used for each\n subelement during ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')}.\n <P>\n If unset, the default tag name is \"value\" for a field of simple type, and for a field of\n DataSource type, is the tagName or ID of the DataSource (as though\n <code>xmlSerialize()</code> were called on the child DataSource).",
        name:"childTagName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["xmlSerialize", "componentSchema"]
    },
    "attr:PickList.fetchDelay":{
        ref:"attr:PickList.fetchDelay",
        flags:"IRWA",
        description:"For a ComboBox or other pickList that accepts user-entered criteria, how many\n millseconds to wait after the last user keystroke before fetching data from the server.\n The default setting will initiate a fetch if the user stops typing or pauses briefly.",
        name:"fetchDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"200",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "method:DynamicForm.showFieldErrors":{
        ref:"method:DynamicForm.showFieldErrors",
        flags:"",
        description:" If this form has any outstanding validation errors for the field passed in, show them now.\n Called when field errors are set directly via ${isc.DocUtils.linkForRef('method:DynamicForm.setFieldErrors')} / \n ${isc.DocUtils.linkForRef('method:DynamicForm.addFieldErrors')} / ${isc.DocUtils.linkForRef('method:DynamicForm.clearFieldErrors')}.<br>\n Default implementation simply falls through to ${isc.DocUtils.linkForRef('method:DynamicForm.showErrors')}.\n",
        name:"showFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to show errors for", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "attr:FormItem.textAlign":{
        ref:"attr:FormItem.textAlign",
        flags:"IRW",
        description:"Alignment of the text / content within this form item. Note that ${isc.DocUtils.linkForRef('attr:FormItem.align')} may\n be used to control alignment of the entire form item within its cell. May not apply to all\n form item types.",
        name:"textAlign",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"Canvas.LEFT",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:TreeGrid.closeFolder":{
        ref:"method:TreeGrid.closeFolder",
        flags:"",
        description:" Closes a folder.\n\n",
        name:"closeFolder",
        seeAlso:[
            {ref:"method:TreeGrid.openFolder", description:null},
            {ref:"method:TreeGrid.folderClosed", description:null}
        ],
        params:[
            {description:"node to close", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.getTitleField":{
        ref:"method:ListGrid.getTitleField",
        flags:"",
        description:" Method to return the fieldName which represents the \"title\" for records in this\n Component.<br>\n If this.titleField is explicitly specified it will always be used.\n Otherwise, default implementation will check ${isc.DocUtils.linkForRef('attr:DataSource.titleField')} for databound\n compounds.<br>\n For non databound components returns the first defined field name of <code>\"title\"</code>, \n <code>\"name\"</code>, or <code>\"id\"</code>. If we dont find any field-names that match these\n titles, the first field in the component will be used instead.\n",
        name:"getTitleField",
        seeAlso:[],
        returns:{description:"fieldName for title field for this component.", type:"string"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:OperationBinding.xmlNamespaces":{
        flags:"IR",
        type:"attr",
        ref:"attr:OperationBinding.xmlNamespaces",
        description:"Optional object declaring namespace prefixes for use in ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath')} and\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueXPath')} XPath expressions.\n <P>\n <code>xmlNamespaces</code> should be specified as a mapping from namespace prefix to\n namespace URI, for example:\n <pre>\n    xmlNamespaces : {\n        az : \"http://webservices.amazon.com/AWSECommerceService/2005-03-23\"\n    }\n </pre>\n By default, all namespaces declared on the document element (outermost element of the\n response) are made available with the prefix used in the document itself.\n <P>\n Then, for non-WSDL-described XML results, if there is a default namespace on the document\n element, it is made available with the special prefix \"default\".\n <P>\n For results of WSDL-described operations, the prefix \"service\" means the service namespace,\n that is, the \"targetNamespace\" on the &lt;definitions&gt; element from the WSDL file.  The prefix\n \"schema\" means the namespace of the outermost element in the output message for the current\n operation.  \"default\" will be the schema namespace if there is one, otherwise the service\n namespace.\n <P>\n For basic information on XML Namespaces and their use in XPath, try the following search:\n <a href=\"http://www.google.com/search?q=XPath+xml+namespaces\" target=\"_blank\"\n >http://www.google.com/search?q=XPath+xml+namespaces</a>",
        name:"xmlNamespaces",
        valueType:"Object",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('xmlYahooWebServices')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration"],
        definingClass:"class:OperationBinding"
    },
    "method:FormItem.hide":{
        ref:"method:FormItem.hide",
        flags:"I",
        description:" Hide this form item.\n <BR><BR>\n This will cause the form to redraw.  If this item had an item.showIf expression, it will\n be destroyed.\n",
        name:"hide",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:TabSet.setTabTitle":{
        ref:"method:TabSet.setTabTitle",
        flags:"A",
        description:" Changes the title of a tab\n",
        name:"setTabTitle",
        seeAlso:[],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"Tab | number | ID"},
            {description:"new title", optional:false, name:"title", defaultValue:null, type:"HTML"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('titleChange')}"],
        definingClass:"class:TabSet"
    },
    "method:Toolbar.itemDragResized":{
        ref:"method:Toolbar.itemDragResized",
        flags:"",
        description:" Observable, overrideable method - called when one of the Toolbar buttons is drag resized.\n\n",
        name:"itemDragResized",
        seeAlso:[],
        params:[
            {description:"the index of the item that was resized", optional:false, name:"itemNum",
             defaultValue:null, type:"number"},
            {description:"the new size of the item", optional:false, name:"newSize", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Toolbar"
    },
    "attr:TreeGrid.showCustomIconDrop":{
        ref:"attr:TreeGrid.showCustomIconDrop",
        flags:"IRWA",
        description:"Should folder nodes showing custom icons (set via the ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconProperty')},\n default ${isc.DocUtils.linkForRef('attr:TreeNode.icon')}),\n show drop state images when the user is drop-hovering over the folder.\n If true, the ${isc.DocUtils.linkForRef('attr:TreeGrid.dropIconSuffix')} will be appended to the image URL\n (so <code>\"customFolder.gif\"</code> might be replaced with \n <code>\"customFolder_drop.gif\"</code>).<br>\n Can be overridden at the node level via the default property ${isc.DocUtils.linkForRef('attr:TreeNode.showDropIcon')}\n and that property can be renamed via ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconDropProperty')}.",
        name:"showCustomIconDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "classAttr:ValidatorTypes.isBoolean":{
        ref:"classAttr:ValidatorTypes.isBoolean",
        flags:"IR",
        description:"Validation will fail if this field is non-empty and has a non-boolean value.",
        name:"isBoolean",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "classMethod:isc.getKeys":{
        ref:"classMethod:isc.getKeys",
        flags:"",
        description:"\n	Return all keys (property names) of a given object\n\n",
        name:"getKeys",
        seeAlso:[],
        params:[
            {description:"object to get properties from", optional:false, name:"object",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"String names of all properties.  NOTE: never null", type:"Array"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:TabSet.scrollerVSrc":{
        ref:"attr:TabSet.scrollerVSrc",
        flags:"IR",
        description:"If this TabSet is showing ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller','tab scroller buttons')}, and \n ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is true, this property governs the base\n URL for the tab bar back and forward scroller button images for vertical tab bars [IE for\n tab sets with ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} set to \"left\" or \"right\"].\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:TabSet.symmetricScroller','symmetricScroller')} is false, \n ${isc.DocUtils.linkForRef('attr:TabSet.scrollerSrc')} will be used instead.\n <P>\n To get the path to the image to display, this base URL will be modified as follows:\n <ul>\n <li>If appropriate a state suffix of <code>\"Down\"</code> or <code>\"Disabled\"</code> will be\n     appended.</li>\n <li>A suffix of <code>\"forward\"</code> or <code>\"back\"</code> will be appended for the\n     forward or backward scrolling button.</li>\n </ul>\n For example - if the scrollerVSrc is set to <code>\"[SKIN]vscroll.gif\"</code>, the image\n displayed for the back-scroller button on a tabSet with <code>tabBarPosition</code> set to\n \"left\" and <code>symmetricScroller</code> set to true would be one of \n <code>\"[SKIN]vscroll_back.gif\"</code>, <code>\"[SKIN]vscroll_Down_back.gif\"</code>,\n and <code>\"[SKIN]vscroll_Disabled_back.gif\"</code>.\n <P>\n Note that for best results the media should be sized to match the scroller button sizes, \n determined by ${isc.DocUtils.linkForRef('attr:TabSet.tabBarThickness')} and ${isc.DocUtils.linkForRef('attr:TabSet.scrollerButtonSize')}.",
        name:"scrollerVSrc",
        seeAlso:[
            {ref:"attr:TabSet.symmetricScroller", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]vscroll.gif\"",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "method:ListGridField.change":{
        ref:"method:ListGridField.change",
        flags:"",
        description:" If this field is editable, any ${isc.DocUtils.linkForRef('method:FormItem.change','change')} handler specified\n on the ListGridField will be passed onto the editors for this field.\n",
        name:"change",
        seeAlso:[
            {ref:"method:ListGridField.changed", description:null},
            {ref:"method:ListGrid.cellChanged", description:null}
        ],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the editor (form item) itself (also available as \"this\").                              Note that in addition to the standard FormItem APIs available\n                              on the editor, it also has:<br>\n                              - a pointer back to the containing listGrid \n                              [<code>item.grid</code>]<br>\n                              - the colNum being edited [<code>item.colNum</code>]<br>\n                              - the rowNum being edited [<code>item.rowNum</code>]",
             optional:false, name:"item", defaultValue:null, type:"FormItem"},
            {description:"The new value of the form item", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"The previous value of the form item", optional:false, name:"oldValue",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:Applet.useTag":{
        ref:"attr:Applet.useTag",
        flags:"IR",
        values:[
            {description:"Use the &lt;applet&gt; tag.  Works on all browsers.", value:"\"applet\""},
            {description:"Use the &lt;object&gt; tag.  Works on IE.", value:"\"object\""}
        ],
        description:"Specifies the type of tag to use for the applet html.  See\n  <a href='http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html' onclick=\"window.open('http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html');return false;\">http://java.sun.com/j2se/1.4.2/docs/guide/plugin/developer_guide/using_tags.html</a> for a\n  discussion of different tags.",
        name:"useTag",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"applet\"",
        type:"attr",
        definingClass:"class:Applet"
    },
    "method:DynamicForm.getValuesAsCriteria":{
        ref:"method:DynamicForm.getValuesAsCriteria",
        flags:"",
        description:" Return search criteria based on the current set of values within this form.\n <p>\n The returned search criteria will be either a simple ${isc.DocUtils.linkForRef('type:Criteria')} object or an\n ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} object if ${isc.DocUtils.linkForRef('attr:FormItem.operator')} has been set for any item in the\n form (see ${isc.DocUtils.linkForRef('attr:FormItem.operator')} for details).  Note that AdvancedCriteria objects will\n only be returned if this form is bound to an appropriate ${isc.DocUtils.linkForRef('class:DataSource')} (ie, a \n dataSource which contains the fields you're trying to use as criteria).\n <P>\n The returned criteria object can then be used to filter data via methods such as\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}, ${isc.DocUtils.linkForRef('method:DataSource.fetchData')}, or, for more advanced usage,\n ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria')}.\n <P>\n Note that any form field which the user has left blank is omitted as criteria, that is,\n a blank field is assumed to mean \"allow any value for this field\" and not \"this field must\n be blank\".  Examples of empty values include a blank text field or SelectItem with an empty\n selection.\n\n",
        name:"getValuesAsCriteria",
        seeAlso:[],
        returns:{description:"a ${isc.DocUtils.linkForRef('type:Criteria')} object, or ${isc.DocUtils.linkForRef('object:AdvancedCriteria')}",
         type:"Criteria or AdvancedCriteria"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ListGrid.groupSummaryStyle":{
        ref:"attr:ListGrid.groupSummaryStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:ListGridRecord.customStyle')} for the group-level summary row displayed when \n ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true.",
        name:"groupSummaryStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"gridSummaryCell\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ListGridField.valueIconLeftPadding":{
        ref:"attr:ListGridField.valueIconLeftPadding",
        flags:"IRW",
        description:"How much padding should there be on the left of valueIcons for this field\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconLeftPadding')}",
        name:"valueIconLeftPadding",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "classAttr:RPCResponse.STATUS_SUCCESS":{
        ref:"classAttr:RPCResponse.STATUS_SUCCESS",
        flags:"R",
        description:"Indicates successful completion of the request.  This is the default status and is\n automatically used by the RPCResponse on the server unless you override it with\n setStatus().\n <br><br>\n See the error handling section in ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager documentation')}\n for more information.",
        name:"STATUS_SUCCESS",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"0",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "attr:ActiveXControl.params":{
        ref:"attr:ActiveXControl.params",
        flags:"IR",
        description:"A map of key/value pairs to pass to the Active X control as parameters.",
        name:"params",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ActiveXControl"
    },
    "attr:TabSet.leftEdgeSizes":{
        ref:"attr:TabSet.leftEdgeSizes",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeSizes</code>, \n <code>rightEdgeSizes</code>, <code>topEdgeSizes</code> and <code>bottomEdgeSizes</code> \n propertes allow the sizes of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge sizes where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"leftEdgeSizes",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:DataSource.qualifyColumnNames":{
        ref:"attr:DataSource.qualifyColumnNames",
        flags:"IR",
        description:"For dataSources of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", determines whether we qualify column\n names with table names in any SQL we generate.  This property can be overridden on specific\n operationBindings.",
        name:"qualifyColumnNames",
        seeAlso:[
            {ref:"attr:OperationBinding.qualifyColumnNames", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:DataBoundComponent.addOperation":{
        ref:"attr:DataBoundComponent.addOperation",
        flags:"IRW",
        description:"Operation ID this component should use when performing add operations.",
        name:"addOperation",
        seeAlso:[
            {ref:"attr:DSRequest.operationId", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["operations"]
    },
    "attr:ImgProperties.name":{
        ref:"attr:ImgProperties.name",
        flags:"IRW",
        description:"Specifies the name of the image. This is an identifier unique to the canvas, and subsequent\n calls to <code>${isc.DocUtils.linkForRef('method:Canvas.getImage')}</code> and\n <code>${isc.DocUtils.linkForRef('method:Canvas.setImage')}</code> \n with this name will act on the image object created using this ImgProperties object.",
        name:"name",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "attr:Window.items":{
        ref:"attr:Window.items",
        flags:"IR",
        description:"The contents of the Window body. Can be specified three different ways:\n      <ul><li>an Array of Canvases that will become the children of the Window's body when it\n      is initialized; the canvases in this array should be created, but not drawn (autodraw:\n      false).\n      <li>a single canvas that will become a child of the Window body.\n      <li>a string that will be set as the body's contents.</ul>",
        name:"items",
        seeAlso:[
            {ref:"attr:Window.body", description:null}
        ],
        valueType:"Array of Canvas, Canvas or String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "group:focus":{
        ref:"group:focus",
        title:"Focus",
        refs:["method:DynamicForm.focusInItem", "method:FormItem.focusInItem", 
         "method:FormItem.blurItem","method:Canvas.focus", "method:Canvas.blur", "method:Canvas.containsFocus", 
         "method:Canvas.setAccessKey","method:Canvas.setTabIndex", "method:Canvas.focusChanged", 
         "classAttr:Canvas.TAB_INDEX_FLOOR","attr:DynamicForm.autoFocus", "attr:DynamicForm.selectOnFocus", 
         "attr:DynamicForm.canFocus","attr:FormItem.accessKey", "attr:FormItem.tabIndex", "attr:FormItem.globalTabIndex", 
         "attr:FormItem.selectOnFocus","attr:TextItem.selectOnFocus", "attr:TextAreaItem.selectOnFocus", "attr:Canvas.canFocus", 
         "attr:Canvas.tabIndex","attr:Canvas.accessKey", "attr:Window.canFocusInHeaderButtons"],
        description:"Focus is the ability to become the target of keyboard input events.\n <P>\n A widget normally receives focus by being clicked on or tabbed to.",
        name:"focus",
        seeAlso:[],
        type:"group"
    },
    "classMethod:JSON.encode":{
        ref:"classMethod:JSON.encode",
        flags:"",
        description:" Serialize an object as a JSON string.\n <P>\n Automatically handles circular references - see ${isc.DocUtils.linkForRef('attr:JSONEncoder.circularReferenceMode')}.\n\n",
        name:"encode",
        seeAlso:[],
        params:[
            {description:"object to serialize", optional:false, name:"object", defaultValue:null,
             type:"any"},
            {description:"optional settings for encoding", optional:true, name:"settings",
             defaultValue:null, type:"JSONEncoder Properties"}
        ],
        returns:{description:"object encoded as a JSON String", type:"String"},
        type:"classMethod",
        definingClass:"class:JSON"
    },
    "attr:FormItem.hoverStyle":{
        ref:"attr:FormItem.hoverStyle",
        flags:"IRW",
        description:"Explict CSS Style for any hover shown for this item.",
        name:"hoverStyle",
        seeAlso:[
            {ref:"attr:DynamicForm.itemHoverStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:Label.iconAlign":{
        ref:"attr:Label.iconAlign",
        flags:"IR",
        description:"If this button is showing an icon should it be right or left aligned?",
        name:"iconAlign",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "attr:DSRequest.exportFilename":{
        ref:"attr:DSRequest.exportFilename",
        flags:"IR",
        description:"The name of the file to save the exported data into.",
        name:"exportFilename",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:DateChooser.nextYearIconWidth":{
        ref:"attr:DateChooser.nextYearIconWidth",
        flags:"IR",
        description:"Width of the icon for the next year button",
        name:"nextYearIconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"14",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "classAttr:Operators.notEndsWithTitle":{
        ref:"classAttr:Operators.notEndsWithTitle",
        flags:"IR",
        description:"Title for the \"notEndsWith\" operator",
        name:"notEndsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not end with (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.defaultValue":{
        ref:"attr:ListGridField.defaultValue",
        flags:"IRW",
        description:"If this field ${isc.DocUtils.linkForRef('attr:ListGridField.canEdit','can be edited')}, this property can be used to\n specify a default value for this field's editor when adding new rows to the grid.",
        name:"defaultValue",
        seeAlso:[
            {ref:"method:ListGrid.startEditingNew", description:null}
        ],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:RPCRequest.showPrompt":{
        ref:"attr:RPCRequest.showPrompt",
        flags:"IRW",
        description:"Overrides RPCManager.showPrompt for this request only.  If you're using queuing, note that\n if any of the requests in the queue specify showPrompt:true, then a prompt will be shown for\n the entire queue with the prompt text of the first request in the queue to specify a custom\n prompt if promptStyle is set to \"dialog\".  If promptStyle is set to \"cursor\" for the request\n that specified showPrompt: true, then the entire queue uses the \"cursor\" style for the\n prompt.",
        name:"showPrompt",
        seeAlso:[
            {ref:"classAttr:RPCManager.showPrompt", description:null}
        ],
        valueType:"boolean",
        defaultValue:"RPCManager.showPrompt",
        type:"attr",
        definingClass:"class:RPCRequest",
        groups:["rpcPrompt"]
    },
    "attr:Canvas.height":{
        flags:"IRW",
        getter:["getHeight"],
        type:"attr",
        ref:"attr:Canvas.height",
        description:"Size for this component's vertical dimension.\n <P>\n Can be a number of pixels, or a percentage like \"50%\". See documentation for\n ${isc.DocUtils.linkForRef('attr:Canvas.width')} for details on who percentage values are resolved actual size.\n <P>\n Note that if ${isc.DocUtils.linkForRef('attr:Canvas.overflow','overflow')} is set to \"visible\", this size is a\n minimum, and the component may overflow to show all content and/or children.\n <P>\n If trying to establish a default height for a custom component, set\n ${isc.DocUtils.linkForRef('attr:Canvas.defaultHeight','defaultHeight')} instead.",
        name:"height",
        setter:["setHeight"],
        valueType:"Number or String",
        seeAlso:[],
        defaultValue:"null",
        groups:["sizing"],
        definingClass:"class:Canvas"
    },
    "method:ListGrid.setCanResizeFields":{
        ref:"method:ListGrid.setCanResizeFields",
        flags:"",
        description:" Setter method for updating ${isc.DocUtils.linkForRef('attr:ListGrid.canResizeFields')} at runtime.\n",
        name:"setCanResizeFields",
        seeAlso:[],
        params:[
            {description:"new value for this.canResizeFields", optional:false, name:"canResize",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:Slider.trackCapSize":{
        ref:"attr:Slider.trackCapSize",
        flags:"IRW",
        description:"The height of vertical slider start and end images, or width of horizontal slider start and\n end images.",
        name:"trackCapSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:ListGrid.drawAheadRatio":{
        ref:"attr:ListGrid.drawAheadRatio",
        flags:"IRW",
        description:"How far should we render rows ahead of the currently visible area?  This is expressed as a\n ratio from viewport size to rendered area size.<br><br>\n \n Tweaking drawAheadRatio allows you to make tradeoffs between continuous scrolling speed vs\n initial render time and render time when scrolling by large amounts.<br><br>\n\n NOTE: Only applies when showAllRows is false.",
        name:"drawAheadRatio",
        seeAlso:[],
        valueType:"float",
        defaultValue:"1.3",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('databoundFetch')}"],
        definingClass:"class:ListGrid",
        groups:["performance"]
    },
    "method:DynamicForm.setTarget":{
        ref:"method:DynamicForm.setTarget",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:DynamicForm.target','target')} for this form.\n",
        name:"setTarget",
        seeAlso:[],
        params:[
            {description:"New submission target", optional:false, name:"target", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "classAttr:Dialog.ASK_TITLE":{
        ref:"classAttr:Dialog.ASK_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed in response to the ${isc.DocUtils.linkForRef('classMethod:isc.ask')} method.\n Note that a custom title can be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"ASK_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Question\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "method:ListGrid.getCellVAlign":{
        ref:"method:ListGrid.getCellVAlign",
        flags:"",
        description:" Return the vertical alignment for cell contents.\n Expected values are: 'top', 'center', or 'bottom'\n\n",
        name:"getCellVAlign",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null}
        ],
        params:[
            {description:"this cell's record", optional:false, name:"record", defaultValue:null,
             type:"listgridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"Vertical alignment of cell contents: 'right', 'center', or 'left'",
         type:"Alignment"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:DataSource.filterData":{
        ref:"method:DataSource.filterData",
        flags:"",
        description:" Perform a \"fetch\" DataSource operation against this DataSource, sending search criteria\n and retrieving matching records.\n <P>\n This is identical to ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is set to \"substring\" to cause case insensitive\n substring matching (if the server respects this setting).\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on                                                       the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource",
        groups:["operations"]
    },
    "attr:DateItem.centuryThreshold":{
        ref:"attr:DateItem.centuryThreshold",
        flags:"IRW",
        description:"Only used if we're showing the date in a text field. When parsing a date, if the year\n is specified with only 2 digits and is less than the centuryThreshold, then the year will\n be assumed to be 20xx; otherwise it will be interpreted according to default browser\n behaviour, which will consider it to be 19xx.",
        name:"centuryThreshold",
        seeAlso:[],
        valueType:"number",
        defaultValue:"25",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["appearance"]
    },
    "attr:ToolStripResizer.vSrc":{
        ref:"attr:ToolStripResizer.vSrc",
        flags:"IRW",
        description:"Image for resizer",
        name:"vSrc",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]resizer.png\"",
        type:"attr",
        definingClass:"class:ToolStripResizer"
    },
    "method:ListGrid.rowContextClick":{
        ref:"method:ListGrid.rowContextClick",
        flags:"",
        description:" Called when a row receives a contextclick event.\n",
        name:"rowContextClick",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('recordClicks')}"],
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:Selection.multipleSelected":{
        ref:"method:Selection.multipleSelected",
        flags:"",
        description:"	Whether multiple items are selected\n",
        name:"multipleSelected",
        seeAlso:[],
        returns:{description:"true == more than one item is selected								false == no items are selected, or only one item is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:TreeGrid.folderContextClick":{
        ref:"method:TreeGrid.folderContextClick",
        flags:"",
        description:"\n This method is called when a context click occurs on a folder record.\n\n",
        name:"folderContextClick",
        seeAlso:[
            {ref:"method:TreeGrid.nodeContextClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which the contexclick occurred.", optional:false,
             name:"viewer", defaultValue:null, type:"TreeGrid"},
            {description:"The folder (record) on which the contextclick occurred.", optional:false,
             name:"folder", defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the contextclick occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.rowDoubleClick":{
        ref:"method:ListGrid.rowDoubleClick",
        flags:"A",
        description:" Event handler for when a body record is double-clicked.\n <P>\n Default implementation fires 'editCell' if appropriate, and handles firing\n 'recordDoubleClick' stringMethod if defined at the field or LG level (That method has a\n different signature from this one)\n\n",
        name:"rowDoubleClick",
        seeAlso:[
            {ref:"method:ListGrid.recordDoubleClick", description:null}
        ],
        params:[
            {description:"record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"index of the row where the click occurred", optional:false, name:"recordNum",
             defaultValue:null, type:"number"},
            {description:"index of the col where the click occurred", optional:false, name:"fieldNum",
             defaultValue:null, type:"number"},
            {description:"indicates whether this was a synthesized record                                              doubleclick in response to a keyboard event",
             optional:true, name:"keyboardGenerated", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"false if first click not on same record; true otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "method:CellSelection.anySelected":{
        ref:"method:CellSelection.anySelected",
        flags:"",
        description:"			Is anything in the list selected?\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection", "selection"]
    },
    "type:CriteriaCombineOperator":{
        ref:"type:CriteriaCombineOperator",
        values:[
            {description:null, value:"\"and\""},
            {description:null, value:"\"or\""}
        ],
        description:"The logical operator to use when combining criteria objects with the \n ${isc.DocUtils.linkForRef('classMethod:DataSource.combineCriteria')} method.",
        name:"CriteriaCombineOperator",
        seeAlso:[],
        type:"type"
    },
    "attr:DetailViewer.separatorStyle":{
        ref:"attr:DetailViewer.separatorStyle",
        flags:"IRW",
        description:"CSS style for a separator",
        name:"separatorStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"detail\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "method:Calendar.eventResized":{
        ref:"method:Calendar.eventResized",
        flags:"",
        description:" Called when an event is resized via dragging by a user.  Return false to disallow the\n resize.\n",
        name:"eventResized",
        seeAlso:[],
        params:[
            {description:"new end date and time that event is being resized to", optional:false,
             name:"newDate", defaultValue:null, type:"Date"},
            {description:null, optional:false, name:"event", defaultValue:null, type:"CalendarEvent"}
        ],
        returns:{description:"return false to disallow the resize", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "method:Toolbar.setCanResizeItems":{
        ref:"method:Toolbar.setCanResizeItems",
        flags:"",
        description:" Setter for updating ${isc.DocUtils.linkForRef('attr:Toolbar.canResizeItems')} at runtime.\n",
        name:"setCanResizeItems",
        seeAlso:[],
        params:[
            {description:"New value for this.canResizeItems", optional:false, name:"canResizeItems",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Toolbar"
    },
    "classMethod:Page.loadStyleSheet":{
        ref:"classMethod:Page.loadStyleSheet",
        flags:"",
        description:"		Load a styleSheet for this application.  \n\n		The styleSheetURL parameter can use any special directories, eg:<br>\n      &nbsp;&nbsp;<code>Page.loadStylesheet(\"[SKIN]/skin_styles.css\")</code><br>\n      or<br>\n      &nbsp;&nbsp;<code>Page.loadStylesheet(\"[APP]/app_styles.css\")</code>.\n      <P>\n		If you don't specify a special directory, the app directory\n			will be assumed.\n		<P>\n		Note:   If the document's ONLOAD handler has already fired, this\n				will have no effect.\n\n",
        name:"loadStyleSheet",
        seeAlso:[],
        params:[
            {description:"URL to the stylesheet.", optional:false, name:"styleSheetURL",
             defaultValue:null, type:"URL"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["skins", "files", "images"]
    },
    "method:ButtonItem.setTitle":{
        ref:"method:ButtonItem.setTitle",
        flags:"",
        description:" Set the title.\n",
        name:"setTitle",
        seeAlso:[],
        params:[
            {description:"new title", optional:false, name:"newTitle", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ButtonItem",
        groups:["appearance"]
    },
    "method:ValuesManager.getValidatedValues":{
        ref:"method:ValuesManager.getValidatedValues",
        flags:"",
        description:" Call ${isc.DocUtils.linkForRef('method:ValuesManager.validate')} to check for validation errors. If no errors are found,\n return the current values for this valuesManager, otherwise return null.\n",
        name:"getValidatedValues",
        seeAlso:[],
        returns:{description:"current values or null if validation failed.", type:"object|null"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:TabSet.tabs":{
        ref:"attr:TabSet.tabs",
        flags:"IRW",
        description:"An array of tab objects, specifying the title and pane contents of each tab in the\n TabSet.  When developing in JavaScript, tabs are specified as an array of object\n literals, not instances - see ${isc.DocUtils.linkForRef('object:Tab')}.\n <p>\n You can add and remove tabs after creating the TabSet by calling ${isc.DocUtils.linkForRef('method:TabSet.addTab')}",
        name:"tabs",
        seeAlso:[],
        valueType:"Array of Tab",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('tabsOrientation')}"],
        definingClass:"class:TabSet"
    },
    "attr:EdgedCanvas.edgeTop":{
        ref:"attr:EdgedCanvas.edgeTop",
        flags:"IR",
        description:"Height in pixels for top corners and edges.  Defaults to edgeSize when unset.",
        name:"edgeTop",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:Window.headerLabelDefaults":{
        ref:"attr:Window.headerLabelDefaults",
        flags:"IRWA",
        description:"This is an object literal property block specifying various properties of the header\n label that displays the ${isc.DocUtils.linkForRef('attr:Window.title')}.  Overrideable defaults are as follows:\n <ul>\n <li>styleName- defaults to <code>\"windowHeaderText\"</code> and specifies the css style\n that is used  to render the ${isc.DocUtils.linkForRef('attr:Window.title')} text.\n </ul>\n You can override the the above propertites by calling ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')}.",
        name:"headerLabelDefaults",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "headerLabel"]
    },
    "attr:ColorItem.supportsTransparency":{
        ref:"attr:ColorItem.supportsTransparency",
        flags:"IRW",
        description:"Determines whether the ${isc.DocUtils.linkForRef('class:ColorPicker')} associated with this ColorItem allows the user \n to set transparency/opacity information whilst selecting a color. If false, no opacity\n slider is shown and all colors are 100% opaque.<p>\n <b>Note</b> ColorItems are representations of HTML color strings, they do not implicitly\n support transparency.  Setting supportsTransparency to true just allows the user to \n set opacity with the picker; if you actually want to capture that information,  you will\n also need to override ${isc.DocUtils.linkForRef('method:ColorItem.pickerColorSelected')}.",
        name:"supportsTransparency",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "type:listGridSelectedState":{
        ref:"type:listGridSelectedState",
        description:"An object containing the stored selection information for a listGrid.\n Note that this object is not intended to be interrogated directly, but may be stored \n (for example) as a blob on the server for state persistence across sessions.",
        name:"listGridSelectedState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "class:DataSourceField":{
        methods:["method:DataSourceField.getFieldValue"],
        ref:"class:DataSourceField",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:["attr:DataSourceField.name", "attr:DataSourceField.type", "attr:DataSourceField.required", 
         "attr:DataSourceField.valueMap","attr:DataSourceField.validators", "attr:DataSourceField.length", 
         "attr:DataSourceField.valueXPath","attr:DataSourceField.title", "attr:DataSourceField.canEdit", 
         "attr:DataSourceField.canSave","attr:DataSourceField.canFilter", "attr:DataSourceField.editorType", 
         "attr:DataSourceField.hidden","attr:DataSourceField.ignore", "attr:DataSourceField.detail", 
         "attr:DataSourceField.primaryKey","attr:DataSourceField.foreignKey", "attr:DataSourceField.childrenProperty", 
         "attr:DataSourceField.rootValue","attr:DataSourceField.sequenceName", "attr:DataSourceField.summaryFunction", 
         "attr:DataSourceField.xmlAttribute","attr:DataSourceField.multiple", "attr:DataSourceField.childTagName", 
         "attr:DataSourceField.propertiesOnly","attr:DataSourceField.inapplicable", "attr:DataSourceField.group", 
         "attr:DataSourceField.javaClass","attr:DataSourceField.javaCollectionClass", "attr:DataSourceField.javaKeyClass", 
         "attr:DataSourceField.exportTitle","attr:DataSourceField.canExport", "attr:DataSourceField.uploadFieldName", 
         "attr:DataSourceField.showFileInline","attr:DataSourceField.customSQL", "attr:DataSourceField.tableName", 
         "attr:DataSourceField.canSortClientOnly","attr:DataSourceField.storeWithHash", "attr:DataSourceField.maxFileSize", 
         "attr:DataSourceField.validOperators"],
        description:"Metadata about a DataSourceField, including its type and validators.",
        name:"DataSourceField",
        seeAlso:[],
        type:"class",
        serverDS:"allowed"
    },
    "attr:ListGridField.filterEditorType":{
        ref:"attr:ListGridField.filterEditorType",
        flags:"IRWA",
        description:"If this ListGrid is showing a filter row, this property can be used to\n      specify the form item class to use for the filter form item associated with this \n      field\n      (Only used if this field is not canFilter:false).<br>\n      Note: If this is not specified, the edit-form item type may be derived from the\n      'editorType' property, typically inherited from datasource fields, or from the\n      'type' of the field (showing the appropriate form item for the data-type).",
        name:"filterEditorType",
        seeAlso:[],
        valueType:"FormItem className",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "method:Tree.move":{
        ref:"method:Tree.move",
        flags:"",
        description:"\n Moves the specified node to a new parent.\n\n",
        name:"move",
        seeAlso:[],
        params:[
            {description:"node to move", optional:false, name:"node", defaultValue:null, type:"TreeNode"},
            {description:"new parent to move the node to", optional:false, name:"newParent",
             defaultValue:null, type:"TreeNode"},
            {description:"Position of the new node in the children list. If not	                                specified, the node will be added at the end of the list.",
             optional:true, name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:ComboBoxItem.optionOperationId":{
        ref:"attr:ComboBoxItem.optionOperationId",
        flags:"IR",
        description:"If this item has a specified <code>optionDataSource</code>, this attribute may be set\n to specify an explicit ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} when performing a fetch against the\n option dataSource to pick up display value mapping.",
        name:"optionOperationId",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:DataSource.criteriaPolicy":{
        ref:"attr:DataSource.criteriaPolicy",
        flags:"IRWA",
        description:"Decides under what conditions the ${isc.DocUtils.linkForRef('class:ResultSet')} cache should be dropped when the\n ${isc.DocUtils.linkForRef('attr:ResultSet.criteria')} changes.",
        name:"criteriaPolicy",
        seeAlso:[
            {ref:"method:DataSource.compareCriteria", description:null}
        ],
        valueType:"CriteriaPolicy",
        defaultValue:"\"dropOnShortening\"",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:WebService.globalNamespaces":{
        ref:"attr:WebService.globalNamespaces",
        flags:"IRW",
        description:"Namespaces definitions to add to the root element of outbound XML messages sent to a web\n service, as a mapping from namespace prefix to namespace URI.\n <P>\n The default value is:\n <pre>\n   globalNamespaces : {\n      xsi: \"http://www.w3.org/2001/XMLSchema-instance\",\n      xsd: \"http://www.w3.org/2001/XMLSchema\"\n   },\n </pre>\n This default value allows the use of the xsi:type and xsi:nil attributes without further\n declarations.\n <P>\n Note that some web services will only accept specific revisions of the XML Schema URI.\n If xsi-namespaced attributes seem to be ignored by an older webservice, try the URI\n \"http://www.w3.org/1999/XMLSchema-instance\" instead.",
        name:"globalNamespaces",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"...",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:WebService"
    },
    "method:Label.setContents":{
        ref:"method:Label.setContents",
        flags:"",
        description:" Changes the contents of a widget to newContents, an HTML string.\n <P>\n When ${isc.DocUtils.linkForRef('attr:Canvas.dynamicContents','dynamicContents')} is set, <code>setContents()</code> can\n also be called with no arguments to cause contents to be re-evaluated.\n\n",
        name:"setContents",
        seeAlso:[],
        params:[
            {description:"an HTML string to be set as the contents of this widget", optional:true,
             name:"newContents", defaultValue:null, type:"string"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('setContents')}"],
        definingClass:"class:Label"
    },
    "method:DynamicForm.clearFieldErrors":{
        ref:"method:DynamicForm.clearFieldErrors",
        flags:"",
        description:" Clear any validation errors on the field passed in.\n",
        name:"clearFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to clear errors from", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"If true this method will fall through to ${isc.DocUtils.linkForRef('method:DynamicForm.showFieldErrors')}  to update the display",
             optional:false, name:"show", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "method:Class.getID":{
        ref:"method:Class.getID",
        flags:"",
        description:"			Return the global identifier for this object.\n\n",
        name:"getID",
        seeAlso:[],
        returns:{description:"global identifier for this canvas", type:"string"},
        type:"method",
        definingClass:"class:Class"
    },
    "attr:Criterion.end":{
        ref:"attr:Criterion.end",
        flags:"IR",
        description:"End value of a criterion with an operator of type <code>\"valueRange\"</code>.",
        name:"end",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "attr:FormItem.criteriaField":{
        ref:"attr:FormItem.criteriaField",
        flags:"IR",
        description:"When using ${isc.DocUtils.linkForRef('attr:FormItem.operator')}, the name of the DataSource field for the\n ${isc.DocUtils.linkForRef('object:Criterion')} this FormItem generates.  If not specified, defaults to\n ${isc.DocUtils.linkForRef('attr:FormItem.name')}.\n <P>\n Generally, because <code>criteriaField</code> defaults to <code>item.name</code>, you don't\n need to specify it.  However, if more than one FormItem specifies criteria for the same\n DataSource field, they will need unique values for ${isc.DocUtils.linkForRef('attr:FormItem.name')} but should set\n ${isc.DocUtils.linkForRef('attr:FormItem.criteriaField')} to the name of DataSource field they both target.\n <P>\n For example, if two DateItems are used to provide a min and max date for a single field called\n \"joinDate\", set ${isc.DocUtils.linkForRef('attr:FormItem.criteriaField')} to \"joinDate\" on both fields but give the fields\n distinct names (eg \"minDate\" and \"maxDate\") and use those names for any programmatic access,\n such as ${isc.DocUtils.linkForRef('method:DynamicForm.setValue')}.",
        name:"criteriaField",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:CubeGrid.saveEdits":{
        ref:"method:CubeGrid.saveEdits",
        flags:"A",
        description:"\n Validates and saves edits for some cell. If rowNum and colNum are not passed in, the current\n edit cell will be saved.\n <P>\n The 'callback' parameter provides a notification when the save attempt completes.\n Cases under which the callback will fire are:\n <ul><li>Save completed successfully</li>\n <li>No changes to the edited cell, so save not required</li>\n <li>Validation failure occurred on the client or on the server</li></ul>\n Note that if no rowNum/colNum were passed in and the editor is not showing for the cell,\n the callback will NOT fire - in this case, the method is a no-op.\n <P>\n Other, standard callbacks such as ${isc.DocUtils.linkForRef('method:ListGrid.editComplete','editComplete()')},\n ${isc.DocUtils.linkForRef('method:ListGrid.editFailed','editFailed()')} and ${isc.DocUtils.linkForRef('method:ListGrid.cellChanged','cellChanged()')} will\n fire normally.\n <P>\n Note this method does not hide the inline editors if they are showing - to explicitly save \n and end editing, use the method 'endEditing()'\n\n",
        name:"saveEdits",
        seeAlso:[
            {ref:"method:ListGrid.endEditing", description:null}
        ],
        params:[
            {description:"Event used to complete cell editing.          Optional, and defaults to <code>\"programmatic\"</code>.  Can be used by the \n          <code>callback</code> method to perform custom actions such as navigation when the \n          save completes.",
             optional:true, name:"editCompletionEvent", defaultValue:null, type:"EditCompletionEvent"},
            {description:"Callback to fire on completion of the saving process.                                    If no edits were made or client-side validation fails \n                                    the callback will be fired synchronously at the end of this\n                                    method.<br>\n                                    Takes the following parameters:\n                                    <br>- rowNum <i>(Number) edited row number</i>\n                                    <br>- colNum <i>(Number) edited column number</i>\n                                    <br>- editCompletionEvent <i>(EditCompletionEvent) event \n                                      passed in (defaults to <code>\"programmatic\"</code>)</i>\n                                    <br>- success <i>(boolean) false if the save was unable to\n                                      complete due to a validation failure or server-side \n                                      error.</i>",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"Which row should be saved. If unspecified the current edit row                            is saved by default. Note that if there is no current edit cell\n                            this method will no op.",
             optional:true, name:"rowNum", defaultValue:null, type:"number"},
            {description:"Which row should be saved. If unspecified the current edit column                            is saved by default. Note that if there is no current edit cell\n                            this method will no op.",
             optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["editing"]
    },
    "type:ListGridEditEvent":{
        ref:"type:ListGridEditEvent",
        values:[
            {description:"A single mouse click triggers inline editing", value:"\"click\""},
            {description:"A double click triggers inline editing", value:"\"doubleClick\""},
            {description:"No mouse event will trigger editing.  Editing must be                      programmatically started via ${isc.DocUtils.linkForRef('method:ListGrid.startEditing')}\n                      (perhaps from an external button) or may be triggered by \n                      keyboard navigation if ${isc.DocUtils.linkForRef('attr:ListGrid.editOnFocus')} is set.",
             value:"\"none\""}
        ],
        description:"Event that will trigger inline editing.",
        name:"ListGridEditEvent",
        seeAlso:[],
        type:"type",
        groups:["editing"]
    },
    "class:Selection":{
        methods:["method:Selection.isSelected", "method:Selection.anySelected", 
         "method:Selection.multipleSelected","method:Selection.getSelection", "method:Selection.getSelectedRecord", 
         "method:Selection.setSelected","method:Selection.select", "method:Selection.deselect", "method:Selection.selectSingle", 
         "method:Selection.selectList","method:Selection.deselectList", "method:Selection.selectAll", 
         "method:Selection.deselectAll","method:Selection.selectItem", "method:Selection.deselectItem", 
         "method:Selection.selectRange","method:Selection.deselectRange"],
        ref:"class:Selection",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:["attr:Selection.selectionRangeNotLoadedMessage"],
        description:"Maintains a 'selected' subset of a List or Array of objects, such as records in a record\n set, or widgets in a selectable header.<br><br>\n\n Includes methods for selecting objects and checking which objects are selected, and also for\n selecting objects as a result of mouse events, including drag selection support.<br>\n The selection object is used automatically to handle selection APIs on ${isc.DocUtils.linkForRef('class:ListGrid')}\n and ${isc.DocUtils.linkForRef('class:TreeGrid')} instances.<br><br>",
        name:"Selection",
        seeAlso:[],
        type:"class"
    },
    "classMethod:isc.getKeyForValue":{
        ref:"classMethod:isc.getKeyForValue",
        flags:"",
        description:" Given a value and an object of <code>key:value</code> pairs, return the value that corresponds\n to that key.\n <P>\n If the key is not found, <code>defaultValue</code> will be returned if provided, otherwise the\n value will be returned.\n\n",
        name:"getKeyForValue",
        seeAlso:[],
        params:[
            {description:"value to look for", optional:false, name:"key", defaultValue:null,
             type:"string or number"},
            {description:"object of key:value pairs", optional:false, name:"valueMap",
             defaultValue:null, type:"object"},
            {description:"default key to return if value not found", optional:true, name:"defaultKey",
             defaultValue:null, type:"any"}
        ],
        returns:{description:"returns first key in valueMap with value, or                                              defaultKey if value not found",
         type:"any"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:ResultTree.discardParentlessNodes":{
        ref:"attr:ResultTree.discardParentlessNodes",
        flags:"IRA",
        description:"When data is loaded from the server, should nodes with an explicit value for\n the ${isc.DocUtils.linkForRef('attr:Tree.parentIdField')} which doesn't map to a valid parent node be dropped?\n If set to false, for ${isc.DocUtils.linkForRef('attr:TreeGrid.loadDataOnDemand')}:false trees, parentless nodes will be\n added as children of the root node - for ${isc.DocUtils.linkForRef('attr:TreeGrid.loadDataOnDemand')}:true, they will be\n added as children of the folder currently requesting children.\n <P>\n This effectively allows nodes to be loaded into the current (or root) folder without\n needing an explicit ${isc.DocUtils.linkForRef('attr:Tree.parentIdField','parentIdField value')} that matches the folder's\n ID or <code>rootValue</code> for the resultTree.\n <P>\n Note: For <code>loadDataOnDemand:false</code> trees, if this property is unset at init time,\n it will default to <code>true</code> if an explicit ${isc.DocUtils.linkForRef('attr:ResultTree.rootNode')} has been\n specified. This ensures that if the data tree retrieved from the server includes ancestors\n of the desired root-node we don't display them. Otherwise this property always defaults to\n false.",
        name:"discardParentlessNodes",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultTree"
    },
    "attr:JSONEncoder.strictQuoting":{
        ref:"attr:JSONEncoder.strictQuoting",
        flags:"IR",
        description:"Whether all property names should be quoted, or only those property names that are not valid\n identifiers or are JavaScript reserved words (such as \"true\").\n <P>\n Encoding only where required produces slightly shorter, more readable output which is still\n compatible with JavaScript's eval():\n <pre>\n {\n     someProp : \"someValue\",\n     \"true\" : \"otherValue\",\n     otherProp : \"otherValue\"\n }\n </pre>\n .. but is not understood by many server-side JSON parser implementations.",
        name:"strictQuoting",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:JSONEncoder"
    },
    "attr:Calendar.dateChooser":{
        ref:"attr:Calendar.dateChooser",
        flags:"R",
        description:"${isc.DocUtils.linkForRef('class:DateChooser')} used to select the date for which events will be displayed.",
        name:"dateChooser",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "attr:ListGridField.groupPrecision":{
        ref:"attr:ListGridField.groupPrecision",
        flags:"IR",
        description:"For fields of type:\"float\" or derived from float, number of digits after the decimal point\n to consider when grouping.\n <P>\n For example, <code>groupPrecision:2</code> indicates that 45.238 and 45.231 group together,\n but 45.22 and 45.27 are separate.\n <P>\n See also ${isc.DocUtils.linkForRef('attr:ListGridField.groupGranularity','groupGranularity')} for grouping by broader\n ranges.",
        name:"groupPrecision",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "attr:FormItemIcon.src":{
        ref:"attr:FormItemIcon.src",
        flags:"IRW",
        description:"If set, this property determines this icon's image source.\n      If unset the form item's <code>defaultIconSrc</code> property will be used\n      instead.<br>\n      As with <code>defaultIconSrc</code> this URL will be modified by adding\n      \"_Over\" or \"_Disabled\" if appropriate to show the icons over or disabled state.",
        name:"src",
        seeAlso:[
            {ref:"attr:FormItem.defaultIconSrc", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('formIcons')}"],
        definingClass:"object:FormItemIcon",
        groups:["formIcons"]
    },
    "classMethod:SimpleType.applySummaryFunction":{
        ref:"classMethod:SimpleType.applySummaryFunction",
        flags:"",
        description:" Applies a ${isc.DocUtils.linkForRef('type:SummaryFunction')} to an array of records\n",
        name:"applySummaryFunction",
        seeAlso:[],
        params:[
            {description:"set of records to retrieve a summary value for", optional:false,
             name:"records", defaultValue:null, type:"Array of Objects"},
            {description:"field for which we're picking up a summary value", optional:false,
             name:"field", defaultValue:null, type:"DataSourceField"},
            {description:"SummaryFunction to apply to the records  in order to retrieve the summary value. May be specified as an explicit function\n  or string of script to execute, or a SummaryFunction identifier",
             optional:false, name:"summaryFunction", defaultValue:null, type:"SummaryFunction"}
        ],
        returns:{description:"summary value generated from the applied SummaryFunction", type:"any"},
        type:"classMethod",
        definingClass:"class:SimpleType"
    },
    "attr:Slider.flipValues":{
        ref:"attr:Slider.flipValues",
        flags:"IRW",
        description:"Specifies whether the value range of the slider should be flipped so that values increase as\n the thumb is moved down (for a vertical slider) or to the left (for a horizontal slider).",
        name:"flipValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Slider"
    },
    "method:Canvas.getScrollbarSize":{
        ref:"method:Canvas.getScrollbarSize",
        flags:"A",
        description:"  Returns the thickness of this widget's scrollbars.<br>\n  For canvases showing custom scrollbars this is determined from <code>this.scrollbarSize</code>\n\n",
        name:"getScrollbarSize",
        seeAlso:[
            {ref:"attr:Canvas.scrollbarSize", description:null}
        ],
        returns:{description:"thickness of the scrollbars, in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:Calendar.removeEvent":{
        ref:"method:Calendar.removeEvent",
        flags:"",
        description:" Remove an event from this calendar.\n\n",
        name:"removeEvent",
        seeAlso:[],
        params:[
            {description:"The event object to remove from the calendar", optional:false, name:"event",
             defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "group:customQuerying":{
        ref:"group:customQuerying",
        title:"Custom Querying Overview",
        refs:["attr:OperationBinding.selectClause", "attr:OperationBinding.tableClause", 
         "attr:OperationBinding.whereClause","attr:OperationBinding.groupClause", "attr:OperationBinding.orderClause", 
         "attr:OperationBinding.valuesClause","attr:OperationBinding.customSQL", "attr:OperationBinding.customHQL", 
         "attr:OperationBinding.customValueFields","attr:OperationBinding.customCriteriaFields", 
         "attr:OperationBinding.excludeCriteriaFields","attr:OperationBinding.useForCacheSync", "attr:OperationBinding.cacheSyncOperation", 
         "attr:OperationBinding.canSyncCache","attr:OperationBinding.sqlType", "type:DefaultQueryClause", "type:SQLType"],
        description:"The SmartClient server provides a number of ways to let you customize the SQL or Hibernate\n query it generates to fetch data from or update your database.  You can provide full \n custom queries in either ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL','SQL')} or \n ${isc.DocUtils.linkForRef('attr:OperationBinding.customHQL','HQL')}, or you can replace individual parts of the query \n (${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','the WHERE clause')}, for example) while letting\n SmartClient generate the rest.  Full custom queries provide complete flexibility, but \n they cannot be used for automatic data paging; if you use a full custom query, all of its\n data will be returned to the client in one shot.  This may, of course, be perfectly \n reasonable, depending on the number of rows involved.  However, if you need automatic \n paging support, you should try to implement your customizations by replacing pieces of the\n query.\n <p>\n Query customization is done per ${isc.DocUtils.linkForRef('class:OperationBinding')}, so it is entirely feasible to \n provide multiple custom queries for the same ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operation type')} \n on a given ${isc.DocUtils.linkForRef('class:DataSource')}.\n\n <h4>Using criteria and values</h4>\n Whether using full custom queries or individual clauses, your code has access to the \n criteria values submitted with the operation; if this is an \"add\" or \"update\" operation,\n it also has access to the new field values sent from the client.\n <p>\n Fields are accessed in your SQL or HQL code using the Velocity template language.  You\n can refer to container variables <b>$criteria</b> and <b>$values</b> in your queries or \n clause snippets, and SmartClient will insert the appropriate values.  A simple \n ${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','whereClause')} example:\n <p>\n <code>&lt;whereClause&gt;continent = $critieria.continent AND population > $criteria.minPop&lt;/whereClause&gt;</code>\n <p>\n Please see ${isc.DocUtils.linkForRef('group:velocitySupport')} for full details of accessing SmartClient \n Server context variables using Velocity. \n \n <h4>Other template variables</h4>\n In addition to the Velocity template variables described above, we also provide a number of\n template variables containing generally-useful values.  Please see\n ${isc.DocUtils.linkForRef('group:velocitySupport')} for details.\n\n <h4>Using the default clauses</h4>\n You also have access to the ${isc.DocUtils.linkForRef('type:DefaultQueryClause','default subclauses')} generated by \n SmartClient.  You can use these in full custom queries to allow a certain part of the query\n code to be generated:<p>\n <code>SELECT foo, bar FROM $defaultTableClause WHERE baz > $criteria.baz</code>.  \n <p>\n You can also use them, with care, as a foundation for your own additions:<p>\n <code>SELECT $defaultSelectClause, foo, bar FROM $defaultTableClause WHERE ($defaultWhereClause) AND baz > $criteria.baz</code>.\n <p>\n You can also use them within query snippets in the various subclause properties:<p>\n <code>&lt;selectClause&gt;$defaultSelectClause, foo, bar&lt;/selectClause&gt;</code>\n\n <h4>Stored procedures</h4>\n It is possible to include templated calls to SQL stored procedures in a\n ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL','customSQL')} clause, for the ultimate in flexibility.  For \n example, the deletion of an order might require a number of actions: deletion of the order\n record itself, messages sent to other systems (data warehousing, maybe, or a central accounts\n system running on a mainframe), an event log written, and so on.  You could write a stored \n procedure to do all this, and then invoke it with a customSQL clause:\n <pre>\n    &lt;operationBinding operationType=\"remove\"&gt;\n        &lt;customSQL&gt;call deleteOrder($criteria.orderNo)&lt;customSQL&gt;\n    &lt;/operationBinding&gt;\n </pre>\n <h4>Custom queries are safe</h4>\n Custom queries are protected from <a href=http://en.wikipedia.org/wiki/SQL_injection>\n SQL injection attacks</a>, because anything coming from the client is quoted and escaped \n in accordance with the syntax of the underlying database before use (though see the warning\n about using <code>$rawValue</code> in the article on ${isc.DocUtils.linkForRef('group:velocitySupport')}).\n So, in a typical SQL injection attack an attacker might enter his User ID as <br>\n &nbsp;&nbsp;<code>123' OR '1' = '1</code><p>\n in the hope that this will generate a query\n with a where clause like this<br>\n  &nbsp;&nbsp;<code>WHERE userID = '123' OR '1' = '1'</code><p>\n which would of course return every row.  With SmartClient custom queries, this does not happen; \n the client-provided string is escaped, and the resultant clause would look like this: <br>\n &nbsp;&nbsp;<code>WHERE userID = '123'' OR ''1'' = ''1'</code><p>\n This clause only returns those records where the userID column contains the literal value that \n the user typed: <br>\n &nbsp;&nbsp;<code>123' OR '1' = '1</code>\n <p>\n Further, custom queries can be protected from buggy or ad-hoc client requests because the \n query is specified on the server.  For example you could add a custom where clause, as shown\n in the above section on default clauses, to ensure that certain records are never seen by\n the client.  For instance: <p>\n <code>&lt;whereClause&gt;($defaultWhereClause) AND confidential = '0'&lt;/whereClause&gt;</code>.\n\n <h4>Column case-sensitivity issues</h4>\n Different database products have different rules concerning case-sensitivity in column \n names.  Consider the following query:\n <br><br><code>&nbsp;&nbsp;SELECT orderNumber FROM Order</code>\n <ul>\n <li>MySQL and Microsoft SQL Server are not case-sensitive with regard to column names, so \n this query will work whether the column is called \"orderNumber\" or \"ORDERNUMBER\" or any \n other variation.</li>\n <li>Oracle, HSQLDB and DB2 default to upper-case column names.  Therefore, this query will\n fail if the column is actually called \"orderNumber\"; it will only work if the underlying\n column name is \"ORDERNUMBER\"</li>\n <li>PostgreSQL defaults to lower-case column names, so this query will fail unless the \n underlying column name is actually \"ordernumber\"</li>\n </ul>\n These differences are one barrier to writing SQL that is portable from one database product\n to another.  There are two ways to work around them.\n <p>\n Firstly, you can simply accept the database's default behavior.  So, your table would \n contain a column called \"orderNumber\" on MySQL, \"ORDERNUMBER\" on Oracle and \"ordernumber\"\n on PostgreSQL (note that you can still use mixed-case DataSource field names; SmartClient\n will map DataSource field \"orderNumber\" to Oracle column \"ORDERNUMBER\" transparently). This\n is the simplest approach.\n <p>\n If you can't, or don't want to, accept the database default - if you are working with an\n existing schema, for example - then you will need to quote column names in your queries.\n Unfortunately, the way you do this also differs by database product, so quoting a column\n name correctly in one database's syntax will mean that the query cannot be ported to a\n different database without change.\n <p>\n To help with this case, we provide two extra container variables that you can use.\n <b>$fields</b> contains the names of all the fields in your DataSource, but quoted in\n accordance with the column-quoting rules of the target database.  <b>$qfields</b> also\n contains a list of field names, but in this case each one is qualified with its table\n name.<p>\n As an example of how to use <b>$fields</b> and <b>$qfields</b>, consider a DataSource with\n a field called \"itemID\", bound to a column also called \"itemID\", and a tableName property \n of \"orderItem\".  Here are three ways to write a ${isc.DocUtils.linkForRef('attr:OperationBinding.selectClause','selectClause')} \n for a custom SQL query that returns that field:<ul>\n <li><code>orderItem.\"itemID\"</code>\n <li><code>orderItem.$fields.itemID</code>\n <li><code>$qfields.itemID</code>\n </ul>\n The first of these is not portable.  It will work fine in HSQL and Oracle, but will fail \n with a syntax error in MySQL, amongst others, because you quote a field name with backticks \n in MySQL, not quote marks.\n <p>\n The usages via <b>$fields</b> and <b>$qfields</b> <em>are</em> portable.  The second line, \n when targeting Oracle, will be translated to <code>orderItem.\"itemID\"</code>; when targeting\n MySQL, it will be translated to <code>orderItem.itemID</code>, or <code>orderItem.`itemID`</code>\n if column quoting is enabled for that database (it generally isn't required, since MySQL \n preserves case by default).",
        name:"customQuerying",
        seeAlso:[],
        type:"group"
    },
    "method:ListGridField.getGroupTitle":{
        ref:"method:ListGridField.getGroupTitle",
        flags:"",
        description:" Return the title that should be shown to the user for the group with the\n <code>groupValue</code> passed as a parameter.\n <P>\n Default title is the groupValue itself. \n\n",
        name:"getGroupTitle",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null},
            {ref:"method:ListGridField.getGroupValue", description:null}
        ],
        params:[
            {description:"the value from the group is created, the result of   ${isc.DocUtils.linkForRef('method:ListGridField.getGroupValue')}",
             optional:false, name:"groupValue", defaultValue:null, type:"any"},
            {description:"the node in the grid containing the group.", optional:false,
             name:"groupNode", defaultValue:null, type:"groupNode"},
            {description:"Field object for which to get group value", optional:false, name:"field",
             defaultValue:null, type:"Object"},
            {description:"The name of the field", optional:false, name:"fieldName", defaultValue:null,
             type:"String"},
            {description:"the ListGrid displaying the cell", optional:false, name:"grid",
             defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"Group value to which this record belongs", type:"any"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('customGrouping')}"],
        definingClass:"object:ListGridField",
        groups:["grouping"]
    },
    "attr:ListGrid.canRemoveRecords":{
        ref:"attr:ListGrid.canRemoveRecords",
        flags:"IR",
        description:"If set, provide UI for the user to remove records from the grid. This is achieved by\n rendering an additional field in the listGrid which, when clicked, will remove the\n record associated with the clicked row via a call to ${isc.DocUtils.linkForRef('method:ListGrid.removeData')}.\n <P>\n If ${isc.DocUtils.linkForRef('attr:ListGrid.animateRemoveRecord')} is true, the removed record will appear to shrink\n out of view when it is removed.\n <P>\n By default the field will display the ${isc.DocUtils.linkForRef('attr:ListGrid.removeIcon')} next to each record, and\n will be rendered as the leftmost column. Two mechanisms exist to further modify this field:\n <ul>\n <li>To change the position of the remove-field, include an explicitly specified field with\n     the attribute ${isc.DocUtils.linkForRef('attr:ListGridField.isRemoveField','isRemoveField:true')} set. This will then\n     be used as the remove field instead of adding a field to the beginning of the set of\n     columns.</li>\n <li>Additional direct configuration of the remove field may be achieved by modifying\n     ${isc.DocUtils.linkForRef('attr:ListGrid.removeFieldProperties')}.</li>\n </ul>",
        name:"canRemoveRecords",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Calendar.setCurrentViewName":{
        ref:"method:Calendar.setCurrentViewName",
        flags:"",
        description:" Sets the currently visible view\n\n",
        name:"setCurrentViewName",
        seeAlso:[],
        params:[
            {description:"The name of the view that should be made visible.", optional:false,
             name:"viewName", defaultValue:null, type:"ViewName"}
        ],
        returns:{description:"The name of the visible view.", type:"ViewName"},
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:SelectItem.hiliteOnFocus":{
        ref:"attr:SelectItem.hiliteOnFocus",
        deprecated:"As of SmartClient version 5.5, use ${isc.DocUtils.linkForRef('attr:SelectItem.showFocused')} instead.",
        flags:"IRWA",
        description:"Should this SelectItem show a hilite when it receives keyboard focus?",
        name:"hiliteOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:DataSourceField.uploadFieldName":{
        ref:"attr:DataSourceField.uploadFieldName",
        flags:"IR",
        description:"Used by the ${isc.DocUtils.linkForRef('class:BatchUploader')} to map a field in an upload file to this \n dataSourceField.  This is only necessary if the dataSourceField's name and title differ \n from the name of the field in the upload file (SmartClient will automatically map upload \n fields using the dataSourceField's title, if possible, if it does not get a direct match \n on field name).",
        name:"uploadFieldName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "classAttr:ValidatorTypes.matchesField":{
        ref:"classAttr:ValidatorTypes.matchesField",
        flags:"IR",
        description:"Tests whether the value for this field matches the value of some other field.\n  The field to compare against is specified via the <code>otherField</code> property\n  on the validator object (should be set to a field name).<br>\n  Note this validator type is only supported for items being edited within a \n  DynamicForm - it cannot be applied to a ${isc.DocUtils.linkForRef('object:ListGridField','ListGrid field')}.",
        name:"matchesField",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        examples:["${isc.DocUtils.linkForExampleId('matchValue')}"],
        definingClass:"class:ValidatorTypes"
    },
    "attr:Tree.modelType":{
        ref:"attr:Tree.modelType",
        flags:"IRWA",
        description:"Selects the model used to construct the tree representation.  See ${isc.DocUtils.linkForRef('type:TreeModelType')} for\n the available options and their implications.\n <P>\n If the \"parent\" modelType is used, you can provide the initial parent-linked data set to the\n tree via the ${isc.DocUtils.linkForRef('attr:Tree.data')} attribute.  If the \"children\" modelType is used, you can\n provide the initial tree structure to the Tree via the ${isc.DocUtils.linkForRef('attr:Tree.root')} attribute.",
        name:"modelType",
        seeAlso:[
            {ref:"attr:Tree.data", description:null},
            {ref:"attr:Tree.root", description:null}
        ],
        valueType:"TreeModelType",
        defaultValue:"\"children\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('nodeTitles')}"],
        definingClass:"class:Tree"
    },
    "type:FieldType":{
        ref:"type:FieldType",
        values:[
            {description:"Generic text, e.g. <code>\"John Doe\"</code>. This is the default field type.                    Use <code>field.length</code> to set length.",
             value:"\"text\""},
            {description:"A boolean value, e.g. <code>true</code>", value:"\"boolean\""},
            {description:"A whole number, e.g. <code>123</code>", value:"\"integer\""},
            {description:"A floating point (decimal) number, e.g. <code>1.23</code>",
             value:"\"float\""},
            {description:"A logic date, with no time value (such as a holiday or birthday).                    Represented on the client as a JavaScript <code>Date</code> object with\n                    all time fields set to zero in browser local time.  Transmitted in\n                    UTC/GMT by default.  See ${isc.DocUtils.linkForRef('group:dateFormatAndStorage')} for more\n                    information on date display and serialization formats. See also\n                    ${isc.DocUtils.linkForRef('class:Date')} for SmartClient extensions to\n                    the <code>Date</code> object.",
             value:"\"date\""},
            {description:"A time of day, with no date.  Represented on the client as a JavaScript                    <code>Date</code> object in UTC/GMT by default (see also\n                    ${isc.DocUtils.linkForRef('group:dateFormatAndStorage')} and the ${isc.DocUtils.linkForRef('class:Time')} class).",
             value:"\"time\""},
            {description:"A date and time, accurate to the second. Represented on the client as a                     JavaScript <code>Date</code> object. See also\n                    ${isc.DocUtils.linkForRef('group:dateFormatAndStorage')}\n                    and ${isc.DocUtils.linkForRef('class:Date')} for\n                    SmartClient extensions to the <code>Date</code> object.",
             value:"\"datetime\""},
            {description:"A text value constrained to a set of legal values specified by the                    field's ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap','valueMap')}, as though an\n                    ${isc.DocUtils.linkForRef('classAttr:ValidatorTypes.isOneOf','isOneOf')} validator had been declared.",
             value:"\"enum\""},
            {description:"An enum whose values are numeric.", value:"\"intEnum\""},
            {description:"If you are using the SmartClient SQL datasource connector, a                    <code>sequence</code> is a unique, increasing whole number, incremented\n                    whenever a new record is added. Otherwise, <code>sequence</code> behaves\n                    identically to <code>integer</code>. This type is typically used with\n                    <code>field.primaryKey</code> to auto-generate unique primary keys.",
             value:"\"sequence\""},
            {description:"A string representing a well-formed URL.  Some components                    will render this as an HTML link (using an anchor tag for example).",
             value:"\"link\""},
            {description:"A string representing a well-formed URL that points to an                    image.  Some components will render an IMG tag with the value of this field\n                    as the 'src' attribute to render the image.",
             value:"\"image\""},
            {description:"Arbitrary binary data.  When this field type is present, three additional fields are automatically generated.  They are: &lt;fieldName&gt;_filename, &lt;fieldName&gt;_filesize, and\n &lt;fieldName&gt;_date_created where &lt;fieldName&gt; is the value of the <code>name</code>\n attribute of this field.  These fields are marked as\n ${isc.DocUtils.linkForRef('attr:DataSourceField.hidden')}<code>:true</code> to suppress their rendering by default.  You\n can show one or more of these fields by specifying the field with a <code>hidden:false</code>\n override in the fields array of the databound component.\n <i>Stream / view file support for non-SQL DataSources: a non-SQL DataSource should provide an \n instance of the Java type InputStream as a field value in DSResponse.data in order to allow \n SmartClient to download the file.</i>",
             value:"\"binary\""},
            {description:"Binary data comprising an image.", value:"\"imageFile\""},
            {description:"Fields of this type are automatically populated by the SmartClient Server                     with the current authenticated userId as part of \"add\" and \"update\" \n                    operations.  By default, fields of this type are hidden and not editable;\n                    the server ignores any value that the client sends in a field of this type.",
             value:"\"modifier\""},
            {description:"Fields of this type are automatically populated by the SmartClient                    Server with the current date and time as part of \"add\" and \"update\" \n                    operations.  By default, fields of this type are hidden and not editable;\n                    the server ignores any value that the client sends in a field of this type.",
             value:"\"modifierTimestamp\""},
            {description:"Fields of this type are automatically populated by the SmartClient Server                     with the current authenticated userId as part of \"add\" operations.\n                    By default, fields of this type are hidden and not editable;\n                    the server ignores any value that the client sends in a field of this type.",
             value:"\"creator\""},
            {description:"Fields of this type are automatically populated by the SmartClient                    Server with the current date and time as part of \"add\" and \"update\" \n                    operations.  By default, fields of this type are hidden and not editable;\n                    the server ignores any value that the client sends in a field of this type.",
             value:"\"creatorTimestamp\""}
        ],
        description:"The types listed below are built-in types that ${isc.DocUtils.linkForRef('interface:DataBoundComponent','databound\n components')} understand and treat specially (using type-specific form controls, validators,\n formatters, sorting logic, etc).  \n <P>\n You can declare custom types via ${isc.DocUtils.linkForRef('class:SimpleType','SimpleType.create()')}, with settings that\n will influence DataBound components.  You can also create your own subclasses of databound\n components to add further custom, reusable behaviors based on field.type.\n <P>\n <code>field.type</code> can also be the ID of another ${isc.DocUtils.linkForRef('class:DataSource')}, which allows you to\n model nested structures such as XML documents (in fact, ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadXMLSchema')}\n models XML schema in this way).  Nested DataSource declarations affect how XML and JSON data\n is deserialized into JavaScript objects in the \n ${isc.DocUtils.linkForRef('group:clientDataIntegration','client-side integration')} pipeline, so that you can load\n complex XML documents and have them deserialized into a correctly typed JavaScript object\n model.\n <P>\n Note: to declare related but <i>separate</i> objects, as in an \"Account\" object that can be\n related to both a \"Contact\" object and \"Order\" objects, use\n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey')}, <b>not</b> a nested structure declaration.",
        name:"FieldType",
        seeAlso:[
            {ref:"type:ListGridFieldType", description:null},
            {ref:"type:FormItemType", description:null}
        ],
        type:"type",
        serverDS:"allowed"
    },
    "type:Direction":{
        ref:"type:Direction",
        values:[
            {description:"above", value:"Canvas.UP"},
            {description:"below", value:"Canvas.DOWN"},
            {description:"to the left of", value:"Canvas.LEFT"},
            {description:"to the right of", value:"Canvas.RIGHT"}
        ],
        description:null,
        name:"Direction",
        seeAlso:[],
        type:"type",
        groups:["appearance"]
    },
    "method:CubeGrid.facetLabelHoverHTML":{
        ref:"method:CubeGrid.facetLabelHoverHTML",
        flags:"",
        description:"\n      Get the HTML for the facet label hover.  Default implementation returns null.\n\n      <br><i>methodType</i> callback\n",
        name:"facetLabelHoverHTML",
        seeAlso:[],
        params:[
            {description:"ID for the facet", optional:false, name:"facetId", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:Canvas.edgeOpacity":{
        flags:"IR",
        type:"attr",
        ref:"attr:Canvas.edgeOpacity",
        description:"Opacity of the edges.  Defaults to matching this.opacity.\n if ${isc.DocUtils.linkForRef('method:Canvas.setOpacity')} is called on a Canvas where edgeOpacity is set,\n edgeOpacity will be considered a percentage of the parent's opacity (so 50% opaque parent plus\n edgeOpacity 50 means 25% opaque edges)",
        name:"edgeOpacity",
        setter:["setEdgeOpacity"],
        valueType:"int",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        defaultValue:"null",
        groups:["imageEdges"],
        definingClass:"class:Canvas"
    },
    "attr:Canvas.noDoubleClicks":{
        ref:"attr:Canvas.noDoubleClicks",
        flags:"IRWA",
        description:"If true, this canvas will receive all mouse-clicks as single click events rather than\n doubleClicks.",
        name:"noDoubleClicks",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["events"]
    },
    "method:DynamicForm.setMethod":{
        ref:"method:DynamicForm.setMethod",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:DynamicForm.method','method')} for this form.\n",
        name:"setMethod",
        seeAlso:[],
        params:[
            {description:"html form submission method (get or post)", optional:false, name:"method",
             defaultValue:null, type:"FormMethod"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "classMethod:DataSource.combineCriteria":{
        ref:"classMethod:DataSource.combineCriteria",
        flags:"",
        description:" Combines two criteria (either simple criteria objects or AdvancedCriteria) using the \n \"outerOperator\".  Note that the combined criteria object will be an AdvancedCriteria\n unless <ul>\n <li>both input criteria objects are simple, and</li>\n <li>the \"outerOperator\" is \"and\", and</li>\n <li>there is no collision of key names on the two criteria</li>\n </ul>\n",
        name:"combineCriteria",
        seeAlso:[],
        params:[
            {description:"first criteria object", optional:false, name:"criteria1", defaultValue:null,
             type:"Criteria"},
            {description:"second criteria object", optional:false, name:"criteria2", defaultValue:null,
             type:"Criteria"},
            {description:"operator to use to combine the criteria.                                           Defaults to \"and\"",
             optional:true, name:"outerOperator", defaultValue:null, type:"CriteriaCombineOperator"},
            {description:"style of matching text, if it is necessary to                                          convert a simple criteria object to an \n                                          AdvancedCriteria.  Defaults to \"substring\"",
             optional:true, name:"textMatchStyle", defaultValue:null, type:"TextMatchStyle"}
        ],
        returns:{description:"The combined criteria", type:"Object"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:ImgSplitbar.skinImgDir":{
        ref:"attr:ImgSplitbar.skinImgDir",
        flags:"IR",
        description:"Default directory for skin images (those defined by the class), relative to the\n Page-wide ${isc.DocUtils.linkForRef('classMethod:Page.getSkinDir','skinDir')}.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"images/SplitBar/\"",
        type:"attr",
        definingClass:"class:ImgSplitbar",
        groups:["images"]
    },
    "attr:DSRequest.exportResults":{
        ref:"attr:DSRequest.exportResults",
        flags:"IR",
        description:"When set, causes the results of the DSRequest to be exported to a file, whose name \n and format are indicated by ${isc.DocUtils.linkForRef('attr:DSRequest.exportFilename')} and \n ${isc.DocUtils.linkForRef('attr:DSRequest.exportAs')} respectively.  When no exportFilename is provided, the default is\n <i>Results</i> and the default value of exportAs is <i>csv</i>.  \n <P>\n The export field-list can also be configured, see ${isc.DocUtils.linkForRef('attr:DSRequest.exportFields')}.\n <P>\n Once the operation completes, ${isc.DocUtils.linkForRef('attr:DSRequest.exportDisplay')} specifies whether the exported\n data should be downloaded to the file-system or displayed in a new window.  The default value\n of exportDisplay is \"download\" which displays the Save As dialog.  See ${isc.DocUtils.linkForRef('type:ExportDisplay')} \n for more information.\n <P>\n You can also configure the style of line-breaks to use when generating the output.  See\n ${isc.DocUtils.linkForRef('type:LineBreakStyle')} for more information.\n <P>\n Note that an export initiated using dsRequest properties does not provide support for JSON\n format (see \n <a href=\"http://forums.smartclient.com/showthread.php?t=235\">this post</a> for more detail).\n <P>\n As well as setting dsRequest.exportResults and related properties, exports can be initiated\n in two other ways, via ${isc.DocUtils.linkForRef('class:OperationBinding')}s and via custom server code which sets \n export-related properties on the ${isc.DocUtils.linkForRef('class:DSResponse')}.  Both of those methods support exporting\n to JSON format.\n <P>\n <b>Format Examples</b>\n XML format\n <pre>\n     &lt;List&gt;\n         &lt;Object&gt;\n             &lt;id&gt;10101&lt;/id&gt;\n             &lt;displayName&gt;Record 10101&lt;/displayName&gt;\n         &lt;/Object&gt;\n    &lt;/List&gt;\n </pre>\n JSON Format\n <pre>\n     [\n         { id: 10101, displayName: \"Record 10101\" }\n     ]\n </pre>\n CSV Format\n <pre>\n     id,displayName\n     10101,\"Record 10101\"\n </pre>",
        name:"exportResults",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:SelectOtherItem.separatorValue":{
        ref:"attr:SelectOtherItem.separatorValue",
        flags:"IRWA",
        description:"Value for the separator item between normal items and the <code>Other...</code> value. \n If necessary the value may be changed to ensure it doesn't collide with any data values in\n this item's ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')}.",
        name:"separatorValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"----\"",
        type:"attr",
        definingClass:"class:SelectOtherItem",
        groups:["appearance"]
    },
    "attr:Splitbar.vResizeCursor":{
        ref:"attr:Splitbar.vResizeCursor",
        flags:"IR",
        description:"Cursor to display if this Splitbar is to be used for vertical resize of widgets.",
        name:"vResizeCursor",
        seeAlso:[],
        valueType:"Cursor",
        defaultValue:"\"row-resize\"",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["cursor"]
    },
    "attr:ListGridField.includeInRecordSummary":{
        ref:"attr:ListGridField.includeInRecordSummary",
        flags:"IR",
        description:"If a listGrid is showing a field of type summary, should this field be passed to the\n recordSummaryFunction when calculating the summary value to display.\n If unset, fields are included if they are of type \"integer\" or \"float\" only (since most\n summary functions perform numeric calculations). See also\n ${isc.DocUtils.linkForRef('attr:ListGridField.includeInRecordSummaryFields')}.",
        name:"includeInRecordSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "group:contentLoading":{
        ref:"group:contentLoading",
        refs:["method:ViewLoader.handleError", "method:ViewLoader.getView", 
         "method:ViewLoader.viewLoaded","method:HTMLFlow.setContentsURL", "method:HTMLFlow.loadingContent", 
         "method:HTMLFlow.handleError","method:HTMLFlow.transformHTML", "method:HTMLFlow.contentLoaded", "class:HTMLFlow", 
         "attr:ViewLoader.httpMethod","attr:HTMLFlow.contentsURL", "attr:HTMLFlow.loadingMessage", 
         "attr:HTMLFlow.contentsURLParams","attr:HTMLFlow.httpMethod", "attr:HTMLFlow.contentsType", "attr:HTMLFlow.evalScriptBlocks", 
         "attr:HTMLFlow.captureSCComponents"],
        name:"contentLoading",
        seeAlso:[],
        type:"group"
    },
    "method:Array.intersect":{
        ref:"method:Array.intersect",
        flags:"",
        description:" Return the list of items that are in both this list and the passed-in list(s).\n\n",
        name:"intersect",
        seeAlso:[],
        params:[
            {description:"lists to intersect with", optional:false, name:"lists", defaultValue:null,
             type:"all List arguments"}
        ],
        returns:{description:"intersection", type:"List"},
        type:"method",
        definingClass:"object:Array",
        groups:["arrayMath"]
    },
    "method:Array.setProperty":{
        ref:"method:Array.setProperty",
        flags:"",
        description:"	Set item[property] = value for each item in this array.\n",
        name:"setProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to set", optional:false, name:"property",
             defaultValue:null, type:"string"},
            {description:"value to set to", optional:false, name:"value", defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"object:Array",
        groups:["iteration"]
    },
    "method:Canvas.scrollTo":{
        ref:"method:Canvas.scrollTo",
        flags:"",
        description:" Scrolls the content of the widget so that the origin (top-left corner) of the content\n is left pixels to the left and top pixels above the widget's top-left corner (but still\n clipped by the widget's dimensions).\n <p>\n This is guaranteed to be called whenever this Canvas is scrolled, whether scrolling is\n initiated programmatically, by custom scrollbars, or a by a native scrollbar.\n\n",
        name:"scrollTo",
        seeAlso:[],
        params:[
            {description:"the left coordinate", optional:true, name:"left", defaultValue:null,
             type:"number"},
            {description:"the top coordiante", optional:true, name:"top", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:ListGrid.hoverStyle":{
        ref:"attr:ListGrid.hoverStyle",
        flags:"IRWA",
        description:"Style to apply to hovers shown over this grid.",
        name:"hoverStyle",
        seeAlso:[
            {ref:"attr:ListGrid.showHover", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"gridHover\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["hovers"]
    },
    "classMethod:Class.addProperties":{
        ref:"classMethod:Class.addProperties",
        flags:"",
        description:"\n	Add default properties and methods to all instances of this class.<br><br>\n\n	These properties can then be accessed as <code>myInstance.property</code>, \n  and methods can be called via <code>myInstance.methodName()</code>\n\n",
        name:"addProperties",
        seeAlso:[],
        params:[
            {description:"objects with properties to add (think named parameters).										all the properties of each argument will be applied\n										as instance-level property defaults.",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"object"}
        ],
        returns:{description:"the class after properties have been added to it", type:"object"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:MultiSortDialog.cancelButtonTitle":{
        ref:"attr:MultiSortDialog.cancelButtonTitle",
        flags:"IR",
        description:"The title-text to appear on the cancelButton",
        name:"cancelButtonTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Cancel\"",
        type:"attr",
        definingClass:"class:MultiSortDialog",
        groups:["i18nMessages"]
    },
    "method:DateItem.deselectValue":{
        ref:"method:DateItem.deselectValue",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is true, falls through to standard\n ${isc.DocUtils.linkForRef('method:FormItem.deselectValue')} implementation on this items freeform text entry field.\n Otherwise has no effect.\n",
        name:"deselectValue",
        seeAlso:[],
        params:[
            {description:"If this parameter is passed, new cursor insertion position will be   moved to the start, rather than the end of this item's value.",
             optional:true, name:"start", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:DateItem"
    },
    "attr:ListGrid.groupByMaxRecords":{
        ref:"attr:ListGrid.groupByMaxRecords",
        flags:"IRW",
        description:"Maximum number of records to which a groupBy can be applied. If there are more records,\n grouping will not be available via the default header context menu, and calls to \n ${isc.DocUtils.linkForRef('method:ListGrid.groupBy')} will be ignored.",
        name:"groupByMaxRecords",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"Number",
        defaultValue:"1000",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "method:Tree.closeAll":{
        ref:"method:Tree.closeAll",
        flags:"",
        description:" Close all nodes under a particular node\n\n",
        name:"closeAll",
        seeAlso:[],
        params:[
            {description:"node from which to close folders (if not specified, the root                              node is used)",
             optional:true, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:ToolbarItem.vertical":{
        ref:"attr:ToolbarItem.vertical",
        flags:"IRA",
        description:"Should the toolbar stack its buttons vertically or horizontally?",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ToolbarItem"
    },
    "attr:ListGrid.unspannedHeaderVAlign":{
        ref:"attr:ListGrid.unspannedHeaderVAlign",
        flags:"IR",
        description:"When ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpans')} are in use, this property sets the default\n vertical alignment for for fields which do <b>not</b> have a headerSpan.",
        name:"unspannedHeaderVAlign",
        seeAlso:[],
        valueType:"vAlign",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "method:Tree.unloadChildren":{
        ref:"method:Tree.unloadChildren",
        flags:"",
        description:"			Unload the children of a node\n",
        name:"unloadChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree",
        groups:["loadState"]
    },
    "class:isc":{
        methods:[],
        ref:"class:isc",
        classAttrs:[],
        classMethods:["classMethod:isc.getErrorsHTML", "classMethod:isc.getIconLeft", 
         "classMethod:isc.getIconTop","classMethod:isc.getIconRect", "classMethod:isc.getIconPageRect", "classMethod:isc.clone", 
         "classMethod:isc.shallowClone","classMethod:isc.logWarn", "classMethod:isc.echo", "classMethod:isc.echoAll", 
         "classMethod:isc.echoLeaf","classMethod:isc.logEcho", "classMethod:isc.logEchoAll", "classMethod:isc.addProperties", 
         "classMethod:isc.getKeys","classMethod:isc.firstKey", "classMethod:isc.getValues", "classMethod:isc.sortObject", 
         "classMethod:isc.sortObjectByProperties","classMethod:isc.addDefaults", "classMethod:isc.propertyDefined", 
         "classMethod:isc.defineClass","classMethod:isc.eval", "classMethod:isc.timeStamp", "classMethod:isc.getValueForKey", 
         "classMethod:isc.getKeyForValue","classMethod:isc.makeReverseMap", "classMethod:isc.setAutoDraw", 
         "classMethod:isc.showPrompt","classMethod:isc.clearPrompt", "classMethod:isc.warn", "classMethod:isc.say", 
         "classMethod:isc.ask","classMethod:isc.confirm", "classMethod:isc.askForValue", 
         "classMethod:isc.showLoginDialog"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"The <code>isc</code> object contains global methods and objects of the Isomorphic SmartClient\n framework.\n <P>\n When the ISC framework runs in \"simple names\" mode (the default), all ISC Classes and several\n global methods are installed as global variables, that is, properties of the browser's\n \"window\" object.  When simple names mode is disabled (called \"portal mode\"), the framework\n uses only the global variable: \"isc\" and global variables prefixed with \"isc_\".  Portal mode\n is intended for applications which must integrate with fairly arbitrary JavaScript code\n written by third-party developers, and/or third party JavaScript frameworks, where it is\n important that each framework stays within it's own namespace.\n <P>\n In portal mode, all references to ISC classes and global functions must be prefixed with\n \"isc.\", for example:<pre>\n \n      Canvas.create(addProperties({}, myDefaults))\n\n </pre>would become<pre>\n\n      isc.Canvas.create(isc.addProperties({}, myDefaults));\n\n </pre>\n Portal mode is enabled by setting <code>window.isc_useSimpleNames = false</code> <b>before</b>\n ISC is loaded.",
        name:"isc",
        seeAlso:[],
        type:"class"
    },
    "attr:Canvas.canAcceptDrop":{
        ref:"attr:Canvas.canAcceptDrop",
        flags:"IRWA",
        description:"Indicates that this object can receive dropped widgets (i.e. other widgets can be\n      dropped on top of it).",
        name:"canAcceptDrop",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragCreate')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:TreeGrid.openerIconSize":{
        ref:"attr:TreeGrid.openerIconSize",
        flags:"IRW",
        description:"Width and height in pixels of the opener icons, that is, the icons which show the open\n or closed state of the node, typically a [+] or [-] symbol.\n <P>\n If ${isc.DocUtils.linkForRef('attr:TreeGrid.showConnectors')} is true, the opener icon includes the connector line, and\n defaults to ${isc.DocUtils.linkForRef('attr:ListGrid.cellHeight','cellHeight')}.\n <P>\n Otherwise, <code>openerIconSize</code> defaults to ${isc.DocUtils.linkForRef('attr:TreeGrid.iconSize')}.",
        name:"openerIconSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:HibernateBrowser.mappingTree":{
        ref:"attr:HibernateBrowser.mappingTree",
        flags:"IR",
        description:"Instance of TreeGrid used to display the Hibernate mappings (beans and properties)",
        name:"mappingTree",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "method:ListGrid.getFieldState":{
        ref:"method:ListGrid.getFieldState",
        flags:"",
        description:" Returns a snapshot of the current presentation of this listGrid's fields as \n a ${isc.DocUtils.linkForRef('type:listGridFieldState')} object.\n <P>\n This object can later be passed to ${isc.DocUtils.linkForRef('method:ListGrid.setFieldState')} to reset this grid's\n fields to the current state.\n <P>\n Note that the information stored includes the current width and visibility of each of this \n grid's fields, as well as any ${isc.DocUtils.linkForRef('attr:ListGrid.canAddFormulaFields','formula')} or\n ${isc.DocUtils.linkForRef('attr:ListGrid.canAddSummaryFields','summary fields')} added by the user.\n\n",
        name:"getFieldState",
        seeAlso:[
            {ref:"method:ListGrid.setFieldState", description:null}
        ],
        returns:{description:"current state of this grid's fields.", type:"listGridFieldState"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "method:SliderItem.setNumValues":{
        ref:"method:SliderItem.setNumValues",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.numValues','number of values')} for the slider \n\n",
        name:"setNumValues",
        seeAlso:[],
        params:[
            {description:"the new number of values", optional:false, name:"newNumValues",
             defaultValue:null, type:"float"}
        ],
        type:"method",
        definingClass:"class:SliderItem"
    },
    "method:Class.getStackTrace":{
        ref:"method:Class.getStackTrace",
        flags:"",
        description:" Returns a \"stack trace\" - one line per method in the current call stack, showing the method\n name and any parameters passed.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getStackTrace\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n <P>\n This API can currently only be supported in Internet Explorer.  See\n ${isc.DocUtils.linkForRef('group:debugging')} for information on how to get equivalent information from other\n browsers.\n\n",
        name:"getStackTrace",
        seeAlso:[],
        returns:{description:"stack trace.  Use eg ${isc.DocUtils.linkForRef('method:Class.logWarn')} to log to the Developer Console.",
         type:"String"},
        type:"method",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:Tree.reloadChildren":{
        ref:"method:Tree.reloadChildren",
        flags:"",
        description:"			Reload the children of a node\n",
        name:"reloadChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree",
        groups:["loadState"]
    },
    "classMethod:History.getHistoryData":{
        ref:"classMethod:History.getHistoryData",
        flags:"",
        description:"\n Returns the data associated with the specified history id.\n\n",
        name:"getHistoryData",
        seeAlso:[],
        params:[
            {description:"The id for which to fetch history data.", optional:false, name:"id",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"The data associated with the specified history id.", type:"any"},
        type:"classMethod",
        definingClass:"class:History"
    },
    "attr:SelectItem.height":{
        ref:"attr:SelectItem.height",
        flags:"IRW",
        description:"Default height for select items is 19px.",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"19",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "method:Array.getLength":{
        ref:"method:Array.getLength",
        flags:"",
        description:" Return the number of items in this list\n\n",
        name:"getLength",
        seeAlso:[],
        returns:{description:"number of items in the list", type:"Number"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "method:ListGridField.getGroupSummary":{
        ref:"method:ListGridField.getGroupSummary",
        flags:"A",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, and this method is specified it will be\n called to generate the field summary value to be displayed for each group level summary row.\n Note that this is called instead of making use of the ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')}.\n",
        name:"getGroupSummary",
        seeAlso:[],
        params:[
            {description:"records for which a summary is being generated  (so all records in the group).",
             optional:false, name:"records", defaultValue:null, type:"Array of ListGridRecord"},
            {description:"pointer to the field for which summary value is being generated",
             optional:false, name:"field", defaultValue:null, type:"ListGridField"},
            {description:"object with specified groupValue and groupName for this group",
             optional:true, name:"groupNode", defaultValue:null, type:"object"}
        ],
        returns:{description:"summary value to display", type:"any"},
        type:"method",
        definingClass:"object:ListGridField"
    },
    "type:DisplayNodeType":{
        ref:"type:DisplayNodeType",
        values:[
            {description:"operate on both folders and leaves", value:"null/unset"},
            {description:"operate on folders only, ignoring leaves", value:"\"folders\""},
            {description:"operate on leaves only, ignoring folders", value:"\"leaves\""}
        ],
        description:"Flag passed to functions as displayNodeType, telling the function whether it should work on\n folders, leaves or both at once.",
        name:"DisplayNodeType",
        seeAlso:[],
        type:"type",
        groups:["ancestry"]
    },
    "method:Tree.getParentPath":{
        ref:"method:Tree.getParentPath",
        flags:"",
        description:"\n Given a node, return the path to it's parent.  This works just like\n ${isc.DocUtils.linkForRef('method:Tree.getPath')} except the node itself is not reported as part of the path.\n\n",
        name:"getParentPath",
        seeAlso:[
            {ref:"method:Tree.getPath", description:null}
        ],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"path to the node's parent", type:"string"},
        type:"method",
        definingClass:"class:Tree"
    },
    "group:valueIcons":{
        ref:"group:valueIcons",
        refs:["method:FormItem.setValueIcons", "method:FormItem.getValueIcon", 
         "attr:FormItem.valueIcons","attr:FormItem.emptyValueIcon", "attr:FormItem.showValueIconOnly", 
         "attr:FormItem.suppressValueIcon","attr:FormItem.valueIconWidth", "attr:FormItem.valueIconHeight", 
         "attr:FormItem.valueIconSize","attr:FormItem.valueIconLeftPadding", "attr:FormItem.valueIconRightPadding", 
         "attr:FormItem.imageURLPrefix","attr:FormItem.imageURLSuffix", "attr:CheckboxItem.valueIconWidth", 
         "attr:CheckboxItem.valueIconHeight"],
        name:"valueIcons",
        seeAlso:[],
        type:"group"
    },
    "class:TileLayout":{
        methods:["method:TileLayout.getTile", "method:TileLayout.addTile", "method:TileLayout.removeTile", 
         "method:TileLayout.transferDragData","method:TileLayout.getDragData", "method:TileLayout.setTileSize", 
         "method:TileLayout.setTileWidth","method:TileLayout.setTileHeight", "method:TileLayout.setTileMargin", 
         "method:TileLayout.setTileHMargin","method:TileLayout.setTileVMargin"],
        ref:"class:TileLayout",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:TileLayout.tiles", "attr:TileLayout.layoutPolicy", "attr:TileLayout.tileSize", 
         "attr:TileLayout.tileWidth","attr:TileLayout.tileHeight", "attr:TileLayout.tileMargin", "attr:TileLayout.tileHMargin", 
         "attr:TileLayout.tileVMargin","attr:TileLayout.layoutMargin", "attr:TileLayout.paddingAsLayoutMargin", 
         "attr:TileLayout.animateTileChange","attr:TileLayout.orientation", "attr:TileLayout.tilesPerLine", "attr:TileLayout.overflow", 
         "attr:TileLayout.expandMargins","attr:TileLayout.autoWrapLines", "attr:TileLayout.dragLine", 
         "attr:TileLayout.dragDataAction"],
        description:"Lays out a series of components, calls \"tiles\", in a grid with multiple tiles per row.",
        name:"TileLayout",
        seeAlso:[],
        type:"class"
    },
    "attr:CubeGrid.facetLabelHoverWidth":{
        ref:"attr:CubeGrid.facetLabelHoverWidth",
        flags:"IRWA",
        description:"If specified and <code>this.showHover</code> is true, this is the default width to apply\n to hover tips shown for facetLabels. If unset, hover canvas will be sized to \n <code>this.hoverWidth</code> if specified instead.",
        name:"facetLabelHoverWidth",
        seeAlso:[
            {ref:"attr:Canvas.hoverWidth", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "method:TreeGrid.filterData":{
        ref:"method:TreeGrid.filterData",
        flags:"",
        description:" Retrieves data that matches the provided criteria and displays the matching data in this\n component.\n <P>\n This method behaves exactly like ${isc.DocUtils.linkForRef('method:TreeGrid.fetchData')} except that\n ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle')} is automatically set to \"substring\" so that String-valued\n fields are matched by case-insensitive substring comparison.\n\n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required; see\n                                          ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')} for details",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"for databound components only - optional                            additional properties to set on the DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DataSourceField.canSortClientOnly":{
        ref:"attr:DataSourceField.canSortClientOnly",
        flags:"IRW",
        description:"When true, this field can only be used for sorting if the data is entirely client-side.",
        name:"canSortClientOnly",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "method:SectionStack.collapseSection":{
        ref:"method:SectionStack.collapseSection",
        flags:"",
        description:" \n Collapse a section or sections.  This action hides all the items assigned to the\n section.  Calling this method is equivalent to the user clicking on the SectionHeader of\n an expanded section.\n\n",
        name:"collapseSection",
        seeAlso:[
            {ref:"method:SectionStack.hideSection", description:null}
        ],
        params:[
            {description:"Section(s) to collapse.  For this parameter, you can pass the position \n                      of the section in the SectionStack, the ID of the section, or a\n                      List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"},
            {description:"callback to fire when the section has been collapsed", optional:true,
             name:"callback", defaultValue:null, type:null}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"],
        definingClass:"class:SectionStack"
    },
    "method:Layout.membersChanged":{
        ref:"method:Layout.membersChanged",
        flags:"",
        description:" Fires once at initialization if the layout has any initial members, and then fires whenever\n members are added, removed or reordered.\n \n",
        name:"membersChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:Layout"
    },
    "attr:TreeGrid.dataSource":{
        ref:"attr:TreeGrid.dataSource",
        flags:"IRW",
        description:"The DataSource that this component should bind to for default fields and for performing\n ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')}.\n <P>\n Can be specified as either a DataSource instance or the String ID of a DataSource.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dataSourceFields')}"],
        definingClass:"class:TreeGrid",
        groups:["databinding"]
    },
    "attr:ListGridField.displayField":{
        ref:"attr:ListGridField.displayField",
        flags:"IRW",
        description:"Specifies the ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')} field used to retrieve the display\n values that are to be mapped from the internal values specified by ${isc.DocUtils.linkForRef('attr:ListGridField.valueField')}.\n <P>\n If no <code>optionDataSource</code> is defined for the field, the cell will display\n the displayField value for the current record instead of the underlying value for\n this field.  This approach can be used for situations where field values need a stored\n value to displayed value mapping, but the set of all possible values is too large to\n load as a ${isc.DocUtils.linkForRef('type:ValueMap')} - see ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')} for more details\n on this approach.  Note that if this field is editable this will also be applied to this\n field's editors.\n <p>\n The display value for a record with a specified <code>displayField</code> can be \n picked up via ${isc.DocUtils.linkForRef('method:ListGrid.getDisplayValue')}.",
        name:"displayField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "attr:SummaryBuilder.fields":{
        ref:"attr:SummaryBuilder.fields",
        flags:"IRW",
        description:"DataSource providing the available fields for the SummaryBuilder.\n <P>\n By default the SummaryBuilder will include all fields.  Set ${isc.DocUtils.linkForRef('attr:SummaryBuilder.fields')} to \n override this.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "method:ListGrid.getGridSummary":{
        ref:"method:ListGrid.getGridSummary",
        flags:"A",
        description:" When ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is <code>true</code> this method is called for each field\n which will show a grid summary value (as described in ${isc.DocUtils.linkForRef('attr:ListGridField.showGridSummary')}\n to get the summary value to display below the relevant column.\n <P>\n The default implementation is as follows:\n <ul><li>If this is a databound grid and not all data is loaded, returns null for every field</li>\n     <li>Otherwise if ${isc.DocUtils.linkForRef('method:ListGridField.getGridSummary')} is defined, calls that method passing\n         in the current data set for the grid</li>\n     <li>If ${isc.DocUtils.linkForRef('method:ListGridField.getGridSummary')} is undefined, makes use of the\n          ${isc.DocUtils.linkForRef('method:ListGrid.getGridSummaryFunction','standard summary function')} for the field to\n          calculate the summary based on the current data set</li>\n </ul>\n This method may be overridden to completely customize the summary value displayed for\n columns in this grid. An example use case would be when summary information is available on\n the client and does note need to be calculated directly from the data.\n <b>Note:</b> this method will not be called if ${isc.DocUtils.linkForRef('listGrid.gridSummaryDataSource')} is\n specified.\n\n",
        name:"getGridSummary",
        seeAlso:[],
        params:[
            {description:"field for which the summary value should be returned", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"}
        ],
        returns:{description:"summary value to display for the specified field.", type:"any"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:ImgButton.showDisabledIcon":{
        ref:"attr:ImgButton.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:DataSourceField.customSQL":{
        ref:"attr:DataSourceField.customSQL",
        flags:"IR",
        description:"If set, this property indicates that this dataSourceField is only intended for use as context in\n a ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL','customSQL')} or ${isc.DocUtils.linkForRef('attr:OperationBinding.customHQL','customHQL')}\n operationBinding, and should not be considered by the server when generating SQL or \n Hibernate queries to fetch or update data.  A common reason for needing to set this flag\n is where you have a value that you want to send to the server for use in selection criteria,\n but that value actually comes from another table.\n <p>\n Note that this property only applies to users of the SmartClient server using dataSources of\n type \"sql\" or \"hibernate\".",
        name:"customSQL",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField"
    },
    "method:ListGrid.getSummaryTitle":{
        ref:"method:ListGrid.getSummaryTitle",
        flags:"A",
        description:" Return the summary title of particular field.  This is the title of\n the field to be used in the show / hide fields context menu. Default implementation will\n use  ${isc.DocUtils.linkForRef('method:ListGridField.getSummaryTitle')} or ${isc.DocUtils.linkForRef('attr:ListGridField.summaryTitle')} if specified,\n otherwise ${isc.DocUtils.linkForRef('attr:ListGridField.title')}.\n",
        name:"getSummaryTitle",
        seeAlso:[],
        params:[
            {description:"field for which we're returning the title", optional:false, name:"field",
             defaultValue:null, type:"listGridField"}
        ],
        returns:{description:"Field summary title.", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["i18nMessages", "display"]
    },
    "attr:ComboBoxItem.optionFilterContext":{
        ref:"attr:ComboBoxItem.optionFilterContext",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, and this property is\n not null, this will be passed to the datasource as ${isc.DocUtils.linkForRef('class:DSRequest')} properties when\n performing the filter operation on the dataSource to obtain the set of options for the\n list.",
        name:"optionFilterContext",
        seeAlso:[],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "classMethod:isc.addProperties":{
        ref:"classMethod:isc.addProperties",
        flags:"",
        description:"\n Add all properties and methods from any number of objects to a destination object, \n overwriting properties in the destination object.\n <p>\n Common uses of <code>addProperties</code> include creating a shallow copy of an object:<pre>\n\n     isc.addProperties({}, someObject);\n\n </pre>Combining settings in order of precedence:<pre>\n\n     isc.addProperties({}, defaults, overrides, skinOverrides);\n\n </pre>\n <P>\n <b>NOTE</b>: do not use <code>addProperties</code> to add defaults to an ISC class.  Use\n <code>Class.addProperties()</code>, as in: <i>MyClassName</i><code>.addProperties()</code>.\n\n",
        name:"addProperties",
        seeAlso:[
            {ref:"method:Class.addProperties", description:null}
        ],
        params:[
            {description:"object to add properties to", optional:false, name:"destination",
             defaultValue:null, type:"object"},
            {description:"objects to obtain properties from.  Properties of all 											arguments other than destination are applied in turn.",
             optional:true, name:"(arguments 1-N)", defaultValue:null, type:"object"}
        ],
        returns:{description:"returns the destination object", type:"object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:ButtonItem.buttonConstructor":{
        ref:"attr:ButtonItem.buttonConstructor",
        flags:"IRA",
        description:"Constructor class for the button.",
        name:"buttonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"Button",
        type:"attr",
        definingClass:"class:ButtonItem"
    },
    "method:ValuesManager.valuesHaveChanged":{
        ref:"method:ValuesManager.valuesHaveChanged",
        flags:"",
        description:"          Compares the current set of values with the values stored by the call to the\n          <code>rememberValues()</code> method. Returns true if the values have changed, and false\n          otherwise.\n",
        name:"valuesHaveChanged",
        seeAlso:[],
        returns:{description:"true if current values do not match remembered values", type:"boolean"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "classMethod:EventHandler.getTarget":{
        ref:"classMethod:EventHandler.getTarget",
        flags:"",
        description:" Return the canvas that is the target of the mouse event.\n Returns null if no canvas found.\n\n",
        name:"getTarget",
        seeAlso:[],
        returns:{description:"event target canvas", type:"Canvas"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "attr:DataSource.dropExtraFields":{
        ref:"attr:DataSource.dropExtraFields",
        flags:"IR",
        description:"Indicates that for server responses, for any data being interpreted as DataSource records, \n only data that\n corresponds to declared fields should be retained; any extra fields should be discarded.\n <P>\n For ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','JSON')} data, this means extra properties in selected\n objects are dropped.\n <P>\n By default, for DMI DSResponses, DSResponse.data is filtered on the server to just\n the set of fields defined on the DataSource.  This type of filtering can also be enabled\n for non-DMI DSResponses (see the overview in ${isc.DocUtils.linkForRef('class:DMI')}).  Setting this property to\n <code>false</code> disables this filtering for this DataSource only.  This setting\n overrides the configuration in <code>server.properties</code>.  This setting can\n be overridden by ${isc.DocUtils.linkForRef('attr:ServerObject.dropExtraFields')}.",
        name:"dropExtraFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration"]
    },
    "method:ImgButton.setDisabled":{
        ref:"method:ImgButton.setDisabled",
        flags:"",
        description:" Enable or disable this object\n",
        name:"setDisabled",
        seeAlso:[],
        params:[
            {description:"true if this widget is to be disabled", optional:false, name:"disabled",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["enable", "state"]
    },
    "attr:ListGrid.useAllDataSourceFields":{
        ref:"attr:ListGrid.useAllDataSourceFields",
        flags:"IRW",
        description:"If true, the set of fields given by the \"default binding\" (see \n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}) is used, with any fields specified in\n <code>component.fields</code> acting as overrides that can suppress or modify the\n display of individual fields, without having to list the entire set of fields that\n should be shown.\n <P>\n If <code>component.fields</code> contains fields that are not found in the DataSource,\n they will be shown after the most recently referred to DataSource field.  If the new\n fields appear first, they will be shown first.\n <P>\n ${isc.DocUtils.linkForExampleId('validationFieldBinding', 'This example')} shows a mixture of component\n fields and DataSource fields, and how they interact for validation.",
        name:"useAllDataSourceFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('validationFieldBinding')}"],
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "method:GridRenderer.findRowNum":{
        ref:"method:GridRenderer.findRowNum",
        flags:"",
        description:" Given a record displayed in this grid, find and return the rowNum in which the record\n appears.\n <P>\n As with ${isc.DocUtils.linkForRef('method:GridRenderer.getCellRecord')} implementing this method is optional as a valid\n grid may be created without any notion of records.\n\n",
        name:"findRowNum",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"index of the row containing the record or -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:Canvas.dragStartDistance":{
        ref:"attr:Canvas.dragStartDistance",
        flags:"IRWA",
        description:"Number of pixels the cursor needs to move before the EventHandler starts a drag operation.",
        name:"dragStartDistance",
        seeAlso:[],
        valueType:"number",
        defaultValue:"5",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:SectionStack.getSectionNumber":{
        ref:"method:SectionStack.getSectionNumber",
        flags:"",
        description:"\n Returns the position of the specified section in the SectionStack.  The numbering is\n zero-based.  \n\n",
        name:"getSectionNumber",
        seeAlso:[],
        params:[
            {description:"ID of a section for which you want to obtain the position.", optional:false,
             name:"sectionID", defaultValue:null, type:"string"}
        ],
        returns:{description:"Position of the section in the SectionStack or -1 if the specified                      section is not a member of this SectionStack.",
         type:"number"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:Facet.rollupValue":{
        ref:"attr:Facet.rollupValue",
        flags:"IR",
        description:"facetValueId of the rollup facetValue for this facet",
        name:"rollupValue",
        seeAlso:[],
        valueType:"String",
        defaultValue:"cubeGrid.rollupValue",
        type:"attr",
        definingClass:"class:Facet"
    },
    "attr:Canvas.parentElement":{
        ref:"attr:Canvas.parentElement",
        flags:"IRA",
        description:"This Canvas's immediate parent, if any.\n <BR>\n Can be initialized, but any subsequent manipulation should be via \n ${isc.DocUtils.linkForRef('method:Canvas.addChild','addChild()')} and ${isc.DocUtils.linkForRef('method:Canvas.removeChild','removeChild()')} calls\n on the parent.",
        name:"parentElement",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "attr:ToolbarItem.buttonBaseStyle":{
        ref:"attr:ToolbarItem.buttonBaseStyle",
        flags:"IRW",
        description:"If specified this baseStyle will be applied to the buttons in this toolbar.",
        name:"buttonBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["appearance"]
    },
    "classMethod:isc.getValueForKey":{
        ref:"classMethod:isc.getValueForKey",
        flags:"",
        description:" Given a key and an object of <code>key:value</code> pairs, return the value that corresponds to\n that key.\n <P>\n If the key is not found, <code>defaultValue</code> will be returned if provided, otherwise the\n key will be returned.\n\n",
        name:"getValueForKey",
        seeAlso:[],
        params:[
            {description:"key to look for", optional:false, name:"key", defaultValue:null,
             type:"string or number"},
            {description:"object of key:value pairs", optional:false, name:"valueMap",
             defaultValue:null, type:"object"},
            {description:"default value to return if key not found", optional:true,
             name:"defaultValue", defaultValue:null, type:"any"}
        ],
        returns:{description:"returns value in valueMap under name key, or                                              defaultValue if key not found",
         type:"any"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:TextAreaItem.wrap":{
        ref:"attr:TextAreaItem.wrap",
        flags:"IRW",
        description:"Text wrapping style.",
        name:"wrap",
        seeAlso:[],
        valueType:"TEXTAREA_WRAP",
        defaultValue:"TextAreaItem.VIRTUAL",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["appearance"]
    },
    "class:Validator":{
        methods:["method:Validator.condition"],
        ref:"class:Validator",
        classAttrs:["classAttr:Validator.notABoolean", "classAttr:Validator.notAString", 
         "classAttr:Validator.notAnInteger","classAttr:Validator.notADecimal", "classAttr:Validator.notADate", 
         "classAttr:Validator.mustBeLessThan","classAttr:Validator.mustBeGreaterThan", "classAttr:Validator.mustBeLaterThan", 
         "classAttr:Validator.mustBeEarlierThan","classAttr:Validator.mustBeShorterThan", "classAttr:Validator.mustBeLongerThan", 
         "classAttr:Validator.mustBeExactLength","classAttr:Validator.requiredField", "classAttr:Validator.notOneOf"],
        classMethods:["classMethod:Validator.addValidator", "classMethod:Validator.addValidators"],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:Validator.type", "attr:Validator.serverCondition", "attr:Validator.errorMessage", 
         "attr:Validator.stopIfFalse","attr:Validator.clientOnly", "attr:Validator.serverOnly"],
        description:"A validator describes a check that should be performed on a value the user is trying to\n save.\n <p>\n Validators are specified for DataSource fields via the ${isc.DocUtils.linkForRef('attr:DataSourceField.validators')}\n property.  Validators that need not be run on the server can also be specified for a\n specific ${isc.DocUtils.linkForRef('class:FormItem')} or ${isc.DocUtils.linkForRef('object:ListGridField')}.\n <p>\n SmartClient supports a powerful library of ${isc.DocUtils.linkForRef('class:ValidatorTypes')} which have identical \n behavior on both the client and the server.  \n <p> \n Beyond this, custom validators can be defined on the client and custom validation logic\n added on the server.  Note that the <code>regexp</code> and <code>mask</code> validator\n types are very flexible and can be used to perform virtually any kind of formatting check\n that doesn't involve some large external dataset.\n <p>\n Custom validators can be reused on the client by adding them to the global validator list,\n via the ${isc.DocUtils.linkForRef('classMethod:Validator.addValidator')} method.",
        name:"Validator",
        seeAlso:[
            {ref:"class:ValidatorTypes", description:null}
        ],
        type:"class"
    },
    "classMethod:Date.getInputFormat":{
        ref:"classMethod:Date.getInputFormat",
        flags:"",
        description:" Retrieves the the default format for strings being parsed into dates via \n <code>Date.parseInput()</code>\n",
        name:"getInputFormat",
        seeAlso:[
            {ref:"classMethod:Date.setInputFormat", description:null}
        ],
        type:"classMethod",
        definingClass:"class:Date"
    },
    "type:FormMethod":{
        ref:"type:FormMethod",
        values:[
            {description:"GET request -- URL encoding (~4K max)", value:"DynamicForm.GET"},
            {description:"POST request -- separate field encoding (no max)", value:"DynamicForm.POST"}
        ],
        description:"Form METHOD parameters - how the form fields are submitted to the server",
        name:"FormMethod",
        seeAlso:[],
        type:"type",
        groups:["submitting"]
    },
    "class:Slider":{
        methods:["method:Slider.setValue", "method:Slider.getValue", "method:Slider.valueChanged", 
         "method:Slider.valueIsChanging","method:Slider.setMinValue", "method:Slider.setMaxValue", "method:Slider.setNumValues"],
        ref:"class:Slider",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:Slider.title", "attr:Slider.length", "attr:Slider.vertical", 
         "attr:Slider.thumbThickWidth","attr:Slider.thumbThinWidth", "attr:Slider.trackWidth", "attr:Slider.thumbSrc", 
         "attr:Slider.trackSrc","attr:Slider.trackCapSize", "attr:Slider.trackImageType", "attr:Slider.showTitle", 
         "attr:Slider.showRange","attr:Slider.showValue", "attr:Slider.labelWidth", "attr:Slider.labelHeight", 
         "attr:Slider.labelSpacing","attr:Slider.value", "attr:Slider.minValue", "attr:Slider.minValueLabel", 
         "attr:Slider.maxValue","attr:Slider.maxValueLabel", "attr:Slider.numValues", "attr:Slider.roundValues", 
         "attr:Slider.roundPrecision","attr:Slider.flipValues", "attr:Slider.sliderTarget", "attr:Slider.canFocus", 
         "attr:Slider.stepPercent","attr:Slider.animateThumb", "attr:Slider.animateThumbTime", "attr:Slider.animateThumbInit"],
        description:"The Slider class implements a GUI slider widget allowing the user to select a numeric \n  value from within a range by dragging a visual indicicator up and down a track.\n	<p>\n  The slider will generate events as the user interacts with it and changes its value.\n  If slider.sliderTarget is specified, moving the slider thumb generates a custom\n	event named 'sliderMove', sent to the sliderTarget.\n  If a <code>sliderMove</code> handler stringMethod is defined on the target, it will be \n  fired when the slider is moved. The second parameter (available via the variable name\n  <code>eventInfo</code> if the handler is a string) is a pointer back to the slider.\n  <p>\n  The slider will also fire a <code>valueChanged()</code> method whenever its value is \n  changed.  This can be observed or overridden on the Slider instance to perform some action.",
        name:"Slider",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"]
    },
    "attr:Button.radioGroup":{
        ref:"attr:Button.radioGroup",
        flags:"IRWA",
        description:"String identifier for this canvas's mutually exclusive selection group.",
        name:"radioGroup",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonRadioToggle')}"],
        definingClass:"class:Button",
        groups:["state", "event handling"]
    },
    "classAttr:Validator.mustBeEarlierThan":{
        ref:"classAttr:Validator.mustBeEarlierThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>dateRange</code> type validator\n  returns false because the value passed in is less than the specified maximum date.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeEarlierThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be earlier than ${max}\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:ResultSet.neverDropUpdatedRows":{
        ref:"attr:ResultSet.neverDropUpdatedRows",
        flags:"IRA",
        description:"By default when a row is returned by the server, the current ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','filter\n criteria')} are applied to it, and it may disappear from the cache.\n <P>\n Set this flag to true to disable this behavior.",
        name:"neverDropUpdatedRows",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["cacheSync"]
    },
    "attr:Facet.summaryValue":{
        ref:"attr:Facet.summaryValue",
        flags:"IR",
        description:"Value for facet summary.",
        name:"summaryValue",
        seeAlso:[],
        valueType:"String",
        defaultValue:"cubeGrid.summaryValue",
        type:"attr",
        definingClass:"class:Facet"
    },
    "classMethod:Page.setIsomorphicDir":{
        ref:"classMethod:Page.setIsomorphicDir",
        flags:"",
        description:"		Specify the root directory for Isomorphic-supplied files.\n\n",
        name:"setIsomorphicDir",
        seeAlso:[],
        params:[
            {description:"New IsomorphicDir URL.", optional:true, name:"URL", defaultValue:null,
             type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files"]
    },
    "group:animation":{
        ref:"group:animation",
        refs:["method:Canvas.animateMove", "method:Canvas.animateResize", "method:Canvas.animateRect", 
         "method:Canvas.animateFade","method:Canvas.animateScroll", "method:Canvas.animateShow", "method:Canvas.animateHide", 
         "attr:Slider.animateThumb","attr:Slider.animateThumbTime", "attr:Slider.animateThumbInit", 
         "attr:TreeGrid.animateFolders","attr:TreeGrid.animateFolderTime", "attr:TreeGrid.animateFolderSpeed", 
         "attr:TreeGrid.animateFolderEffect","attr:ListGrid.animateRemoveTime", "attr:ListGrid.animateRemoveSpeed", 
         "attr:Window.animateMinimize","attr:Window.minimizeTime", "attr:Window.minimizeAcceleration", "attr:Canvas.animateTime", 
         "attr:Canvas.animateAcceleration","attr:Canvas.animateShowEffect", "attr:Canvas.animateHideEffect", 
         "attr:Canvas.animateMoveTime","attr:Canvas.animateResizeTime", "attr:Canvas.animateRectTime", 
         "attr:Canvas.animateFadeTime","attr:Canvas.animateScrollTime", "attr:Canvas.animateShowTime", 
         "attr:Canvas.animateHideTime","attr:Canvas.animateMoveAcceleration", "attr:Canvas.animateResizeAcceleration", 
         "attr:Canvas.animateRectAcceleration","attr:Canvas.animateScrollAcceleration", "attr:Canvas.animateShowAcceleration", 
         "attr:Canvas.animateHideAcceleration","attr:Layout.animateMembers", "attr:Layout.animateMemberTime", 
         "attr:SectionStack.animateSections"],
        name:"animation",
        seeAlso:[],
        type:"group"
    },
    "group:springIntegration":{
        ref:"group:springIntegration",
        title:"Integration with Spring",
        treeLocation:["Concepts"],
        refs:[],
        description:"<b>Overview</b>\n <P>\n The Spring framework has many different parts, from integration with Object Relational\n Mapping (ORM) and transaction management systems, to a Model View Controller (MVC)\n architecture.\n <P>\n If you are building a new application from scratch and/or you are trying to \n modernize the presentation layer of an existing application, most of Spring MVC is\n inapplicable in the ${isc.DocUtils.linkForRef('group:smartArchitecture','SmartClient architecture')}.  Specifically,\n SmartClient renders <b>all</b> HTML on the client, and the server is responsible only for\n retrieving data and enforcing business rules.  This means that Spring's ModelAndView and all\n functionality related to retrieving and rendering Views is unnecessary in SmartClient.\n SmartClient only needs the Model, and provides methods to deliver that Model to SmartClient\n components (the server side method DSResponse.setData()).\n <P>\n However, Spring's DispatchServlet, Handler chain, and Controller architecture is applicable\n to SmartClient.  See \"Using Spring Controllers\" below.\n <P>\n <b>Existing Spring Application</b>\n <P>\n As discussed under the general ${isc.DocUtils.linkForRef('group:clientServerIntegration','server integration')}\n topic, integrating SmartClient into your application involves finding a way to provide data\n that fulfills the ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')} sent by SmartClient components.\n <P>\n There are 2 approaches for integrating SmartClient into an existing Spring application:\n <ul>\n <li> <b>call Spring beans via SmartClient DMI</b> <span\n style=\"color:red\">[Recommended]</span>: use SmartClient Direct Method Invocation\n (DMI) to map ${isc.DocUtils.linkForRef('class:DSRequest','DataSource requests')} to beans managed by Spring, via\n ${isc.DocUtils.linkForRef('attr:ServerObject.lookupStyle')}:\"spring\".   Return data to the browser by either simply\n returning it from your method, or via creating a DSResponse and calling DSResponse.setData()\n (server-side method). \n <P>\n This is the easiest method and produces the best result.  A Collection of Java Beans, such\n as EJB or Hibernate-managed beans, can be directly returned to SmartClient as the result of\n a DMI method, without the need to create an intervening\n +link{http://en.wikipedia.org/wiki/Data_transfer_object,Data Transfer Object} to express\n which fields should be delivered to the browser - instead, only the fields declared on the\n DataSource are returned to the browser (see\n ${isc.DocUtils.linkForRef('attr:DataSource.dropExtraFields','dropExtraFields')}.  In this integration scenario, the\n majority of the features of the SmartClient Server framework still apply - see this\n ${isc.DocUtils.linkForRef('group:featuresCustomPersistence','overview')}.\n <P>\n <li> <b>configure Spring to return XML or JSON responses</b>: create variants\n on existing Spring workflows that use a different type of View in order to output XML or\n JSON data instead of complete HTML pages.  The SmartClient ${isc.DocUtils.linkForRef('class:RestDataSource')} provides a\n standard \"REST\" XML or JSON-based protocol you can implement, or you can adapt generic\n ${isc.DocUtils.linkForRef('class:DataSource','DataSources')} to existing formats.\n <P>\n In some Spring applications, all existing Spring workflows can be made callable by\n SmartClient with a generic View class capable of serializing the Model to XML or JSON,\n combined with a Controller that always uses this View.  Consider the following Java\n anonymous class, which uses the SmartClient JSTranslater class to dump the entire \n Spring Model as a JSON response.\n <pre>\n  new View() {\n        public void render(Map model, HttpServletRequest request,\n                           HttpServletResponse response) throws IOException {\n                final ServletOutputStream outputStream = response.getOutputStream();\n                response.setContentType(\"application/x-javascript\");\n                outputStream.println(JSTranslater.get().toJS(model));\n                outputStream.close();\n        }\n        public String getContentType() {\n                return \"application/x-javascript\";\n        }\n  }\n </pre>\n <P>\n If you use this approach, you do not need to install the SmartClient server, and can\n ${isc.DocUtils.linkForRef('group:iscInstall','deploy')} SmartClient as simple web content (JS/media/HTML files).  If you\n are already familiar with how to generate XML from objects that typically appear in your\n Spring Models, this may be the easiest path.\n </ul>\n <P>\n <b>Using Spring Controllers with SmartClient DMI</b>\n <P>\n You can create a Controller that invokes standard SmartClient server request processing,\n including DMI, like so:\n <pre>\n public class SmartClientRPCController extends AbstractController\n {\n     public ModelAndView handleRequest(HttpServletRequest request, \n                                       HttpServletResponse response)\n         throws Exception\n     {\n         // invoke SmartClient server standard request processing\n         com.isomorphic.rpc.RPCManager.processRequest(request, response);\n         return null; // avoid default rendering\n     }\n }\n </pre>\n This lets you use Spring's DispatchServlet, Handler chain and Controller architecture as a\n pre- and post-processing model wrapped around SmartClient DMI.",
        name:"springIntegration",
        seeAlso:[],
        type:"group"
    },
    "attr:TabSet.showTabPicker":{
        ref:"attr:TabSet.showTabPicker",
        flags:"IR",
        description:"If there is not enough space to display all the tab-buttons in this tabSet, should\n a drop-down \"picker\" be displayed to allow selection of tabs that are clipped?",
        name:"showTabPicker",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "classAttr:Operators.startsWithTitle":{
        ref:"classAttr:Operators.startsWithTitle",
        flags:"IR",
        description:"Title for the \"startsWith\" operator",
        name:"startsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"starts with (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classAttr:Dialog.YES_BUTTON_TITLE":{
        ref:"classAttr:Dialog.YES_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"Yes\"</code> button.",
        name:"YES_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"Yes\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "method:List.remove":{
        ref:"method:List.remove",
        flags:"",
        description:" Remove first instance of the passed object from this array, sliding other items around to\n fill gaps.\n\n",
        name:"remove",
        seeAlso:[],
        params:[
            {description:"item to remove", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"true if a matching object was found and removed, false if no matching object was found and the list remains unchanged.",
         type:"boolean"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:Menu.cellHeight":{
        ref:"attr:Menu.cellHeight",
        flags:"IRW",
        description:"The height of each item in the menu, in pixels.",
        name:"cellHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:Menu",
        groups:["sizing", "sizing"]
    },
    "method:TreeGrid.openFolder":{
        ref:"method:TreeGrid.openFolder",
        flags:"A",
        description:" Opens a folder.\n <p>\n Executed when a folder node receives a 'doubleClick' event. This handler must be\n specified as a function, whose single parameter is a reference to the relevant folder\n node in the tree's data.<br>\n See the ListGrid Widget Class for inherited recordClick and recordDoubleClick events.\n\n",
        name:"openFolder",
        seeAlso:[
            {ref:"method:TreeGrid.closeFolder", description:null},
            {ref:"method:TreeGrid.folderOpened", description:null},
            {ref:"class:ListGrid", description:null}
        ],
        params:[
            {description:"node to open", optional:false, name:"node", defaultValue:null, type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "attr:DynamicForm.itemHoverHeight":{
        ref:"attr:DynamicForm.itemHoverHeight",
        flags:"IRW",
        description:"A default height for hovers shown for items",
        name:"itemHoverHeight",
        seeAlso:[
            {ref:"attr:FormItem.hoverHeight", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "group:mail":{
        ref:"group:mail",
        refs:["class:Mail", "attr:OperationBinding.mail", "attr:Mail.templateFile", 
         "attr:Mail.messageTemplate","attr:Mail.to", "attr:Mail.cc", "attr:Mail.bcc", "attr:Mail.from", "attr:Mail.replyTo", 
         "attr:Mail.subject","attr:Mail.multiple", "attr:Mail.messageData"],
        name:"mail",
        seeAlso:[],
        type:"group"
    },
    "attr:Label.showDisabledIcon":{
        ref:"attr:Label.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "classMethod:Class.logIsInfoEnabled":{
        ref:"classMethod:Class.logIsInfoEnabled",
        flags:"",
        description:" Check whether a message logged at \"info\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsInfoEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:MenuItem.dynamicIcon":{
        ref:"method:MenuItem.dynamicIcon",
        flags:"",
        description:" Contains the condition that will change the curent items' icon when met. The handler must be specified\n as a function or string of script.\n <p>\n If you don't need to set this state dynamically, use ${isc.DocUtils.linkForRef('attr:MenuItem.icon')} instead.\n <p>\n May be defined as a ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')}.\n <p>\n",
        name:"dynamicIcon",
        seeAlso:[],
        params:[
            {description:"${isc.DocUtils.linkForRef('attr:Menu.target','target')} attribute for the top level menu.",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"${isc.DocUtils.linkForRef('class:Menu','menu')} contains the reference to the menu that contains the current item",
             optional:false, name:"menu", defaultValue:null, type:"Menu"},
            {description:"contains the reference to the current item", optional:false, name:"item",
             defaultValue:null, type:"MenuItem"}
        ],
        returns:{description:"the url of this menuItems icon", type:"SCImgURL"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('menuDynamicItems')}"],
        definingClass:"object:MenuItem",
        groups:["dynamicMenuItem"]
    },
    "attr:DynamicForm.longTextEditorType":{
        ref:"attr:DynamicForm.longTextEditorType",
        flags:"IRW",
        description:"Name of the Form Item class to use for text fields which exceed the \n longTextEditorThreshold for this form.",
        name:"longTextEditorType",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"textArea\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["appearance"]
    },
    "attr:Canvas.dropTypes":{
        ref:"attr:Canvas.dropTypes",
        flags:"IRWA",
        description:"The \"type\" of thing(s) that can be dropped on this widget specified as a string or an\n array of strings (indicating multiple types). Leave this with the value null to indicate\n that this widget can accept anything dropped on it from the page.",
        name:"dropTypes",
        seeAlso:[
            {ref:"method:Canvas.willAcceptDrop", description:null},
            {ref:"attr:Canvas.dragType", description:null}
        ],
        valueType:"string | array of strings",
        defaultValue:"Canvas.ANYTHING",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "classMethod:Log.logFatal":{
        ref:"classMethod:Log.logFatal",
        flags:"",
        description:" Log a message at \"fatal\" priority\n \n",
        name:"logFatal",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Log"
    },
    "attr:ImgButton.icon":{
        ref:"attr:ImgButton.icon",
        flags:"IRW",
        description:"Optional icon to be shown with the button title text.  \n <P>\n Specify as the partial URL to an image, relative to the imgDir of this component.",
        name:"icon",
        setter:["setIcon"],
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "class:MathFunction":{
        methods:[],
        ref:"class:MathFunction",
        classAttrs:[],
        classMethods:["classMethod:MathFunction.registerFunction", 
         "classMethod:MathFunction.getRegisteredFunctionIndex","classMethod:MathFunction.getDefaultFunctionIndex"],
        treeLocation:["Client Reference/Data Binding/FormulaBuilder"],
        attrs:["attr:MathFunction.name", "attr:MathFunction.description", "attr:MathFunction.jsFunction", 
         "attr:MathFunction.defaultSortPosition"],
        description:"The definition of a function for use in the ${isc.DocUtils.linkForRef('class:FormulaBuilder')}.  A function consists of \n a name (what the user actually types to use the function), a description (shown in help) and \n an actual JavaScript function that executes the calculation.\n <P>\n The built-in functions cover all static functionality on the JavaScript Math object:\n <ul>\n <li><b>max(val1,val2)</b>: Maximum of two values</li>\n <li><b>min(val1,val2)</b>: Minimum of two values</li>\n <li><b>round(value,decimalDigits)</b>: Round a value up or down, optionally providing \n     <i>decimalDigits</i> as the maximum number of decimal places to round to.  For fixed \n     or precision rounding, use <i>toFixed()</i> and <i>toPrecision()</i> respectively.\n </li>\n <li><b>ceil(value)</b>: Round a value up</li>\n <li><b>floor(value)</b>: Round a value down</li>\n <li><b>abs(value)</b>: Absolute value</li>\n <li><b>pow(value1,value2)</b>: value1 to the power of value2</li>\n <li><b>sin(value)</b>: Sine of a value</li>\n <li><b>cos(value)</b>: Cosine of a value</li>\n <li><b>tan(value)</b>: Tangent of a value</li>\n <li><b>ln(value)</b>: natural logarithm of a value</li>\n <li><b>log(base,value)</b>: logarithm of a value with the specified <i>base</i></li>\n <li><b>asin(value)</b>: Arcsine of a value</li>\n <li><b>acos(value)</b>: Arccosine of a value</li>\n <li><b>atan(value)</b>: Arctangent of a value (-PI/2 to PI/2 radians)</li>\n <li><b>atan2(value1,value2)</b>: Angle theta of a point (-PI to PI radians)</li>\n <li><b>exp(value)</b>: The value of E<sup>value</sup></li>\n <li><b>random()</b>: Random number between 0 and 1</li>\n <li><b>sqrt(value)</b>: Square root of a value</li>\n\n <li><b>toPrecision(value,precision)</b>: Format a number to a length of <i>precision</i> digits, \n     rounding or adding a decimal point and zero-padding as necessary.  Note that the \n     values 123, 12.3 and 1.23 have an equal precision of 3.  Returns a formatted string \n     and should be used as the outermost function call in a formula. For rounding, use \n     <i>round()</i>.\n </li>\n <li><b>toFixed(value,digits)</b>: Round or zero-pad a number to <i>digits</i> decimal places.  \n     Returns a formatted string and should be used as the outermost function call in a \n     formula.  To round values or restrict precision, use <i>round()</i> and \n     <i>toPrecision()</i> respectively.\n </li>\n </ul>",
        name:"MathFunction",
        seeAlso:[],
        type:"class",
        groups:["formulaFields"]
    },
    "method:ValuesManager.editNewRecord":{
        ref:"method:ValuesManager.editNewRecord",
        flags:"",
        description:"\n Prepare to edit a new record by clearing the current set of values (or replacing them \n with initialValues if specified).\n <br>\n This method will also call ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')} to ensure\n subsequent calls to <code>saveData()</code> will use an <code>add</code> rather than\n an <code>update</code> operation.\n\n",
        name:"editNewRecord",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"initial set of values for the editor as a map of field names to their corresponding\n     values",
             optional:true, name:"initialValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "attr:Chart.labelValues":{
        ref:"attr:Chart.labelValues",
        flags:"IR",
        description:"Whether to show labels on each individual value (bar, column or line point)",
        name:"labelValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:Chart",
        groups:["chartAppearance"]
    },
    "attr:TileLayout.tilesPerLine":{
        ref:"attr:TileLayout.tilesPerLine",
        flags:"IR",
        description:"Number of tiles to show in each line.  Auto-derived from ${isc.DocUtils.linkForRef('attr:TileLayout.tileSize')} for some layout\n modes.  See ${isc.DocUtils.linkForRef('type:TileLayoutPolicy')}.\n This can also affect ${isc.DocUtils.linkForRef('attr:TileLayout.tileWidth')} or ${isc.DocUtils.linkForRef('attr:TileLayout.tileHeight')}. See those properties for details.",
        name:"tilesPerLine",
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutPolicy"]
    },
    "attr:TileGrid.fields":{
        ref:"attr:TileGrid.fields",
        flags:"IR",
        description:"Array of field definitions to control the default rendering of tiles.\n <P>\n If not specified, if the DataSource has an ${isc.DocUtils.linkForRef('attr:DataSource.iconField','iconField')}, only the\n <code>iconField</code> and ${isc.DocUtils.linkForRef('attr:DataSource.titleField','titleField')} will be shown.\n Otherwise, all non-${isc.DocUtils.linkForRef('attr:DataSourceField.hidden','hidden')}\n non-${isc.DocUtils.linkForRef('attr:DataSourceField.detail','detail')} fields will be shown, similar to the default set of\n fields shown by a ${isc.DocUtils.linkForRef('class:ListGrid')}.\n <P>\n Only applicable if using the default ${isc.DocUtils.linkForRef('class:SimpleTile')} class for tiles.\n <P>\n For SimpleTiles, it is possible to use ${isc.DocUtils.linkForRef('method:DetailViewerField.getCellStyle')} and \n ${isc.DocUtils.linkForRef('method:StatefulCanvas.getStateSuffix')} to make a single field statefully styled:\n <pre>\n isc.TileGrid.create({\n      fields:[\n          {name:'animalName',\n           getCellStyle : function (value, field, record, viewer) {\n               if (value == \"Tiger\") return \"tigerStyle\" + viewer.currentTile.getStateSuffix();\n               else return view.tileGrid.tileValueStyle + viewer.currentTile.getStateSuffix();\n           }\n          }\n      ]\n });\n </pre>",
        name:"fields",
        seeAlso:[],
        valueType:"Array of DetailViewerField",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:DatabaseBrowser.schemaTree":{
        ref:"attr:DatabaseBrowser.schemaTree",
        flags:"IR",
        description:"Instance of TreeGrid used to display the database schema (tables and columns)",
        name:"schemaTree",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:TabSet.scrollerButtonSize":{
        ref:"attr:TabSet.scrollerButtonSize",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller')} is true, this property governs the size of scroller\n buttons. Applied as the width of buttons if the tabBar is horizontal, or the height\n if tabBar is vertical. Note that the other dimension is determined by \n ${isc.DocUtils.linkForRef('attr:TabSet.tabBarThickness','this.tabBarThickness')}",
        name:"scrollerButtonSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "attr:Slider.showValue":{
        ref:"attr:Slider.showValue",
        flags:"IRW",
        description:"Indicates whether a label for the value of the slider should be displayed. The\n default position for this label is to the right of a horizontal slider, or below a vertical\n slider.",
        name:"showValue",
        seeAlso:[
            {ref:"attr:Slider.value", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:DataSource.strictSQLFiltering":{
        ref:"attr:DataSource.strictSQLFiltering",
        flags:"IR",
        description:"[A] If set to true, both client and server-side advanced filtering used by SmartClient will follow\n SQL99 behavior for dealing with NULL values, which is often counter-intuitive to users.\n Specifically, when a field has NULL value, all of the following expressions are false:\n <pre>\n    field == \"someValue\"  (normally false)\n    field != \"someValue\"  (normally true)\n    not (field == \"someValue\")   (normally true)\n    not (field != \"someValue\")   (normally false)\n </pre>",
        name:"strictSQLFiltering",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "classAttr:Dialog.WARN_TITLE":{
        ref:"classAttr:Dialog.WARN_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed in response to the ${isc.DocUtils.linkForRef('classMethod:isc.warn')} method.\n Note that a custom title can be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"WARN_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Note\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:Img.imageWidth":{
        ref:"attr:Img.imageWidth",
        flags:"IR",
        description:"Explicit size for the image, for ${isc.DocUtils.linkForRef('attr:Img.imageType')} settings that would normally use the\n image's natural size (applies to ${isc.DocUtils.linkForRef('attr:Img.imageType')} \"center\" and \"normal\" only).",
        name:"imageWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Img"
    },
    "method:ListGrid.getHeaderSpanContextMenuItems":{
        ref:"method:ListGrid.getHeaderSpanContextMenuItems",
        flags:"",
        description:" Return the menus items that should be shown in a menu triggered from a\n ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans','headerSpan')}.  Default behavior is to return false, meaning no\n contextMenu is shown.\n\n",
        name:"getHeaderSpanContextMenuItems",
        seeAlso:[],
        params:[
            {description:"name of the header span", optional:false, name:"name", defaultValue:null,
             type:"String"},
            {description:"the component representing the headerSpan.  This component will                            have all the properties specified via ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans')}.",
             optional:false, name:"headerSpan", defaultValue:null, type:"Canvas"}
        ],
        returns:{description:"return false instead to avoid showing a menu", type:"Array of MenuItem"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "attr:TestFunctionResult.emptyTestValue":{
        ref:"attr:TestFunctionResult.emptyTestValue",
        flags:"IRW",
        description:"Set to true if the formula or summary definition passed in was empty.",
        name:"emptyTestValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "attr:MenuButton.menuButtonImageUp":{
        ref:"attr:MenuButton.menuButtonImageUp",
        flags:"IRA",
        description:"Image for menu button indicating that the button expands a menu.  This image is shown\n for menus expanding up from the button.   Menu direction is controlled by\n ${isc.DocUtils.linkForRef('attr:MenuButton.showMenuBelow')}.",
        name:"menuButtonImageUp",
        seeAlso:[
            {ref:"attr:MenuButton.menuButtonImage", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]menu_button_up.gif\"",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "attr:ListGridRecord.canExpand":{
        ref:"attr:ListGridRecord.canExpand",
        flags:"IR",
        description:"Default property name denoting whether this record can be expanded. Property name may be \n modified for some grid via ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecordProperty')}.",
        name:"canExpand",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord",
        groups:["expansionField"]
    },
    "attr:SpacerItem.height":{
        ref:"attr:SpacerItem.height",
        flags:"IRW",
        description:"default height for the spacer",
        name:"height",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:SpacerItem",
        groups:["appearance"]
    },
    "attr:Tree.childrenProperty":{
        ref:"attr:Tree.childrenProperty",
        flags:"IRW",
        description:"For trees with the modelType \"children\", this property specifies the name of the property\n that contains the list of children for a node.",
        name:"childrenProperty",
        seeAlso:[
            {ref:"attr:Tree.modelType", description:null}
        ],
        valueType:"string",
        defaultValue:"\"children\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('childrenArrays')}"],
        definingClass:"class:Tree"
    },
    "method:ValuesManager.getValues":{
        ref:"method:ValuesManager.getValues",
        flags:"",
        description:" Returns the current set of values for the values manager instance.  This includes the\n values from any form managed by this manager, as well as any values explicitly applied\n via ${isc.DocUtils.linkForRef('method:ValuesManager.setValues')}.\n",
        name:"getValues",
        seeAlso:[],
        returns:{description:"a map of the values for this manager", type:"object"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "method:Array.get":{
        ref:"method:Array.get",
        flags:"",
        description:" Return the item at a particular position\n",
        name:"get",
        seeAlso:[],
        params:[
            {description:"position of the element to get", optional:false, name:"pos",
             defaultValue:null, type:"Number"}
        ],
        returns:{description:"whatever's at that position, undef if not found", type:"object"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:EdgedCanvas.edgeOffsetTop":{
        ref:"attr:EdgedCanvas.edgeOffsetTop",
        flags:"IRA",
        description:"Amount the contained Canvas should be offset from the top.  Defaults to the size for \n the top edge.  Set smaller to allow the contained Canvas to overlap the edge and \n corner media.",
        name:"edgeOffsetTop",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:DataSource.recordXPath":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.recordXPath",
        description:"See ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath')}.  <code>recordXPath</code> can be specified\n directly on the DataSource for a simple read-only DataSource only capable of \"fetch\"\n operations.",
        name:"recordXPath",
        valueType:"XPathExpression",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('xmlDataSource')}", 
         "${isc.DocUtils.linkForExampleId('jsonXPath')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration"],
        definingClass:"class:DataSource"
    },
    "attr:LoginDialog.errorStyle":{
        ref:"attr:LoginDialog.errorStyle",
        flags:"IR",
        description:"Specifies the CSS style of the error text shown for a login failure.",
        name:"errorStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"formCellError\"",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:FormItem.showPicker":{
        ref:"method:FormItem.showPicker",
        flags:"A",
        description:" Method to show a picker for this item. By default this method is called if the user\n clicks on a ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon','pickerIcon')}.  May also be called programatically.\n <P>\n Default implementation lazily creates and shows the ${isc.DocUtils.linkForRef('attr:FormItem.picker','Picker Autochild')}.\n May be overridden to implement some custom picker for this item.\n\n",
        name:"showPicker",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:SectionItem.canCollapse":{
        ref:"attr:SectionItem.canCollapse",
        flags:"IR",
        description:"Whether this section header can be collapsed.  If set false, suppresses open/close state\n icon",
        name:"canCollapse",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SectionItem"
    },
    "method:SummaryBuilder.getHelpText":{
        ref:"method:SummaryBuilder.getHelpText",
        flags:"",
        description:" Call to retrieve the text the SummaryBuilder would show by default for help, or override to\n provide alternate help text.\n\n",
        name:"getHelpText",
        seeAlso:[],
        returns:{description:"By default, the results of getHoverText()", type:"String"},
        type:"method",
        definingClass:"class:SummaryBuilder",
        groups:["summaryFields"]
    },
    "attr:CubeGrid.padTitles":{
        ref:"attr:CubeGrid.padTitles",
        flags:"IRW",
        description:"Whether to pad titles so they aren't flush with header borders",
        name:"padTitles",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "method:ImgButton.addToRadioGroup":{
        ref:"method:ImgButton.addToRadioGroup",
        flags:"",
        description:" Add this widget to the specified mutually exclusive selection group with the ID\n passed in.\n Selecting this widget will then deselect any other StatefulCanvases with the same\n radioGroup ID.\n StatefulCanvases can belong to only one radioGroup, so this method will remove from \n any other radiogroup of which this button is already a member.\n",
        name:"addToRadioGroup",
        seeAlso:[],
        params:[
            {description:"- ID of the radiogroup to which this widget should be added",
             optional:false, name:"groupID", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state", "event handling"]
    },
    "attr:Validator.serverCondition":{
        ref:"attr:Validator.serverCondition",
        flags:"IR",
        description:"An expression in the Velocity Template Language that will run on the server.  Unlike most \n other SmartClient Server features involving Velocity, server-side custom validators do not\n have access to the standard set of context variables.  This is deliberate, to ensure that \n custom validators can run in the widest possible variety of circumstances, without being \n tied to operating in the context of, say, an <code>HttpSession</code> or a \n <code>DSRequest</code>.  Server-side custom validators only have the following variables \n available:\n <ul>\n <li><b>dataSources</b> - The list of all DataSources, accessible by name (so, for example, \n     <code>$dataSources.supplyItem</code> refers to the <code>supplyItem</code> DataSource\n     object).</li>\n <li><b>dataSource</b> - The current DataSource</li>\n <li><b>record</b> - The DataSource record being validated, if available</li>\n <li><b>value</b> - The value of the field</li>\n <li><b>validator</b> - The config of this validator, presented as a <code>Map</code></li>\n <li><b>field</b> - The field (as a <code>DSField</code> object)</li>\n <li><b>util</b> - A <code>DataTools</code> object, giving you access to all of that\n     class's useful helper functions</li>\n </ul>",
        name:"serverCondition",
        seeAlso:[],
        valueType:"VelocityExpression",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Validator"
    },
    "attr:Calendar.otherDayBodyBaseStyle":{
        ref:"attr:Calendar.otherDayBodyBaseStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to the day body of the month view\n of the calendar. This style will have \"Dark\", \"Over\", \"Selected\", or \"Disabled\"\n appended to it according to the state of the cell.",
        name:"otherDayBodyBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calMonthDayBody\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "method:ListGrid.deselectAllRecords":{
        ref:"method:ListGrid.deselectAllRecords",
        flags:"",
        description:"\n Deselect all records\n\n",
        name:"deselectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "group:debugging":{
        ref:"group:debugging",
        title:"Debugging",
        treeLocation:["Concepts"],
        refs:[],
        description:"<h4>Built-in Diagnostics</h4>\n <P>\n In any page in which ISC has been loaded, you have access to the Developer Console,\n which can be opened by entering the following URL into your browser from the running\n application:\n <pre>\n     javascript:isc.showConsole()</pre>\n Basic information on the features of the Developer Console can be found in the QuickStart\n Guide.  This topic focuses on use of the log system and related debugging facilities.\n <P>\n The Developer Console contains a \"Results\" pane that displays a list of diagnostic\n messages logged by the SmartClient framework. The \"Logging Preferences\" menu lets you\n enable and disable SmartClient's built-in diagnostics in several categories. Because\n important diagnostic messages may be logged at any time, you should have the Developer\n Console open whenever you are working with SmartClient (and you should bookmark the\n \"javascript:\" expression above to make this easier).\n <P>\n Log messages are of the format:\n <P>\n &nbsp;&nbsp;&nbsp;<i>timestamp</i>:<i>priority</i>:<i>category</i>:<i>message</i>\n <P>\n For example, the following log message:\n <pre>\n     11:59:25:806:INFO:Page:Page loading complete.</pre>\n Occured at 11:59:25 local time and 806 milliseconds.  It's priority was <code>INFO</code>,\n it occurred in the category <i>Page</i>, and the message is \"Page loading complete.\".\n <P>\n Each logging <i>category</i> has a <i>priority</i> associated with it.  If a message's\n priority is lower than the current priority for the category it is logged in, the\n message will be suppressed (will not appear in the \"Results\" pane).  \n <p>\n It is critical to be familiar with the diagnostic categories built-in to SmartClient - \n you will use them in most debugging sessions.  Open the Logging Preferences menu and select\n \"More..\" to see a list of diagnostic log categories.   Hover over each category name to\n see a description of what kind of messages are logged in the category.  \n <P>\n <h4>Debugging JavaScript Errors</h4>\n <P>\n Isomorphic recommends the FireFox browser as the primary development platform.  In FireFox,\n JavaScript errors are reported both in the SmartClient Developer Console and in the FireFox\n JavaScript console, which can be opened by entering the special\n url \"javascript:\" in the address bar.\n <P>\n Isomorphic currently recommends that every SmartClient developer install either the \n <a href='https://addons.mozilla.org/firefox/1815/' onclick=\"window.open('https://addons.mozilla.org/firefox/1815/');return false;\">Console2</a> or\n <a href='http://www.getfirebug.com/' onclick=\"window.open('http://www.getfirebug.com/');return false;\">Firebug</a> extensions for FireFox.  \"Console2\" simply\n replaces the default FireFox JavaScript console with a more functional console;\n Firebug aims to be a true debugger.  Both tools will typically load or identify the correct\n file and line number where a JS error occured.\n <P>\n In Internet Explorer, when JS errors occur, SmartClient is able to report full stack traces\n in the Developer Console.  This can be invaluable when your code triggers a JS error\n in the SmartClient libraries themselves, or when it is unclear how your code is being\n called.  Stack traces from Internet Explorer should <i>always</i> be included in issue\n reports sent to Isomorphic Software, if at all possible. \n <P>\n <h4>Inspecting application state</h4>\n <P>\n The \"Evaluate JS Expression\" area of the Results Pane in the Developer Console can be used\n to inspect the current state of a SmartClient application.  Any SmartClient or browser\n built-in API can be called from the \"Evaluate JS Expression\" area, and the results will\n be intelligently summarized (via ${isc.DocUtils.linkForRef('classMethod:Log.echo')}).  For example, simply typing a\n component's ID and pressing the \"Eval JS\" button will give you a dump of it's current\n property values.  \n <P>\n Many, many SmartClient APIs can be usefully called while troubleshooting, eg, \n ${isc.DocUtils.linkForRef('attr:ListGrid.data')} is a ${isc.DocUtils.linkForRef('class:ResultSet')} when a grid is DataBound and \n ${isc.DocUtils.linkForRef('method:ResultSet.get')} can be called to inspect the current values on records.  In addition,\n new application code can be tried out, for example, you might repeatedly instantiate a new\n component, trying variants on the properties you could give it.\n <P>\n <b>Inspecting transient application state with logs</b>\n <P>\n Transient state, such as the values of local variables in a method that is crashing, can be\n sent to the Developer Console via using the ${isc.DocUtils.linkForRef('class:Log')} class.  For example, to dump the\n value of the local variable \"request\": \n <pre>\n     isc.logWarn(\"request is: \" + isc.echo(request));\n </pre>\n <P>\n It's a good idea to dump the values of local variables in any method that is crashing or\n behaving unexpectedly.\n <P>\n Note the use of ${isc.DocUtils.linkForRef('classMethod:isc.logWarn','logWarn()')} above: in typical debugging sessions,\n it's best\n to simply use <code>logWarn</code> method to output diagnostics to ensure your message will\n not be suppressed by log priority settings.\n <P>\n NOTE: never use the native <code>alert()</code> method to output diagnostics.  Among other\n issues, <code>alert()</code> can affect timing, masking or altering the behavior you were\n trying to debug.  SmartClient's logging system doesn't suffer from these problems and\n provides much more control.\n <P>\n <h4>Issue Reports</h4>\n <P>\n If you believe you've discovered a bug in SmartClient or you are having trouble using\n SmartClient APIs, you can report it at <a href='http://forums.smartclient.com/' onclick=\"window.open('http://forums.smartclient.com/');return false;\">http://forums.smartclient.com/</a>, or, if\n you have Enterprise Support, at the \n <a href='http://support.isomorphic.com/' onclick=\"window.open('http://support.isomorphic.com/');return false;\">Customer Support Extranet</a>.\n <P>\n <b>How quickly your issue is resolved is entirely up to you</b>.  If you follow the steps\n below and submit an appropriate issue report, you will generally receive a rapid solution\n from Isomorphic Support, regardless of what support level you have, because Isomorphic\n aggressively corrects bugs and legitimate usage issues.  If you skip steps you are likely to\n be directed back to this document and asked to submit a more complete issue report.\n <P>\n Before reporting an issue, ensure that you:\n <ul>\n <li> Have read the ${isc.DocUtils.linkForDocNode('QuickStartGuide', 'QuickStart Guide')} cover to\n cover.  Later chapters cover more advanced topics and provide links to further examples and\n reference.\n <li> Have searched the ${isc.DocUtils.linkForDocNode('FeatureExplorer', 'Feature Explorer')} for examples that show\n what you are trying to do\n <li> Have searched this reference, trying multiple searches using different, common and\n related terms for what you are trying to do (eg for search, try \"search\", \"filter\",\n \"criteria\", \"find\", \"match\", etc)\n <li> Have searched the public <a href='http://forums.smartclient.com' onclick=\"window.open('http://forums.smartclient.com');return false;\">forums</a>\n </ul>\n Always include:\n <ul>\n <li> A description of what you are trying to accomplish <b>from a user's perspective</b>.\n The best answers often point out a simpler approach.\n <li> The browser(s), operating system(s) and SmartClient version(s) you experience the error\n on (SmartClient version is available in the lower-left handle corner of the Developer\n Console)\n </ul>\n Then, include <b>either</b> a standalone test case (see below), <b>or</b>:\n <ul>\n <li> For JS errors, Stack traces from Firebug (for Firefox) or the Developer Console (for\n IE), as covered under \"Debugging JavaScript Errors\" above\n <li> Results of calling <code>echo()</code> on local variables or other application\n state you think is relevant (see \"Inspecting Application State\" above)\n <li> What server platform and ${isc.DocUtils.linkForRef('group:clientServerIntegration','databinding approach')} you\n are using, if applicable\n <li> contents of the SmartClient Developer Console \"Log messages\" area, with appropriate\n diagnostic categories set the DEBUG or INFO level (see \"Built-in Diagnostics\" above)\n <li> sample code and sample data\n </ul>\n <b>Preparing a standalone test case</b>\n <P>\n A standalone test case is one of:\n <ol>\n <li> a chunk of JavaScript code that can be executed from the \"Eval JS\" area of the\n Developer Console on some specified page within the unmodified SmartClient SDK,\n demonstrating your issue\n <li> an .html or .jsp file that can be dropped at a specified location into an unmodified\n SmartClient SDK and will run without changes, demonstrating your issue.\n <li> a .zip file that includes a standalone .html/.jsp file  as above, as well as\n dependencies required to make the test case runnable, such as XML datasets\n </ol>\n <P>\n Submitting a standalone test case removes any ambiguity as to whether there is a bug in\n SmartClient or a bug in your code, and eliminates the possibility of Isomorphic Support\n responding with a \"works for me\" result due to incomplete information.  Issues with verified\n test cases are routed directly to the engineer that authored the relevant SmartClient\n subsystem, often as the new highest priority task.  In addition, the process of preparing a\n test case very often allows you to solve the issue yourself.\n <P>\n There are two approaches to test case preparation:\n <ol>\n <li> Add code to an existing SmartClient example until you can reproduce the problem\n <li> Remove code from your application until it minimally shows the problem and runs standalone\n </ol>\n <P>\n For approach #1, find the nearest match to your use case in the\n ${isc.DocUtils.linkForDocNode('FeatureExplorer')} examples or in the other examples accessible from the Examples\n folder of the SDK, then try to minimally modify that example to demonstrate your issue.\n Feature Explorer examples are a particularly good starting point because you can simply copy\n the code from the Feature Explorer to the Eval JS area of the Developer Console and begin\n changing it, and if successful this yields a type #1 test case, the easiest for you to\n submit and most efficient for Isomorphic to work with.\n <P>\n For approach #2,\n <ol>\n <li> If a server is involved in initial page generation (eg a .jsp file), in most cases you\n can eliminate many server dependencies <b>and</b> create an easily modifiable starting point\n by using the browser's \"View Source\" feature to save a copy of the generated HTML output as\n an .html file in the same directory as the .jsp file that generated it.  Such a file will\n generally continue to function (all relative paths are still correct), and can be modified\n freely without the need to later revert changes to a .jsp.\n <li> Eliminate any code that isn't involved in the interaction.  Keep running the test case\n as you eliminate code to ensure you are still seeing the issue (you may solve it this way,\n or find key preconditions that you can report to Isomorphic)\n <li> For any issue that isn't cosmetic, revert to a default SmartClient skin\n <li> For any necessary RPC/DataSource interactions, spoof the interaction with one of these\n approaches:\n <ul>\n <li> switch any DataSources to one of the sample DataSources from the SDK (eg \"supplyItem\")\n if your issue can still be reproduced in this case.\n <li> create a small sample dataset in JavaScript directly in the .html file, and use a\n ${isc.DocUtils.linkForRef('attr:DataSource.clientOnly','clientOnly DataSource')} with that dataset.\n <li> capture server responses verbatim by setting the RPCManager log category to DEBUG, save\n the responses as flat files, and set ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')} to point at them.\n <li> for RPCs, instead of calling the RPCManager, directly call your own callback function,\n passing a spoofed RPCResponse that includes just the fields your code depends upon\n </ul>\n <li> Finally, move your .html file into the stock SmartClient SDK along with any remaining\n dependencies and verify the problem can still be reproduced\n </ol>\n Having prepared the test case, combine it with the other required issue report information\n covered above, and submit it to the <a href='http://forums.smartclient.com/' onclick=\"window.open('http://forums.smartclient.com/');return false;\">forums</a>,\n or, if you have Enterprise Support, at the\n <a href='http://support.isomorphic.com/' onclick=\"window.open('http://support.isomorphic.com/');return false;\">Customer Support Extranet</a>.\n <P>\n <h4>Adding your own diagnostic categories</h4>\n <P>\n Calling <code>logWarn()</code> is fine for a log statement you plan to delete at the end of\n the debugging session.  However, many log statements have lasting value if you could enable\n or disable them only when you need the relevant diagnostics, like SmartClient's built-in\n diagnostic categories.  To do this, pick a priority level less than <code>WARN</code>\n (<code>INFO</code> or <code>DEBUG</code>), and call the corresponding method on the Log\n class (<code>logInfo()</code> or <code>logDebug()</code>), passing the category name as a\n second parameter.  For example:\n <pre>\n   isc.Log.logInfo(\"first record is: \" + \n                   isc.Log.echo(myGrid.data.get(0)), \n                  \"myGridLoading\");\n </pre>\n This message will no longer appear in the Results Pane by default, because its priority\n (<code>INFO</code>) is less than the default of <code>WARN</code>.  To see this message,\n open the Logging Preferences menu and pick \"More..\", then click the \"Add\" button, enter\n \"myGridLoading\" as the category name and set the priority to <code>INFO</code>.  The message\n will now appear next time it is logged.\n <P>\n Now you have a custom log category that you and other developers can use to debug your\n application, subsystem by subsystem.  These diagnostics will be available to you both in\n development and production environments.\n <P>\n As with SmartClient's built-in diagnostics, you may choose to log certain messages in your\n custom category at the <code>DEBUG</code> level and a lesser number of messages at the\n <code>INFO</code> level, to create different depths of diagnostic output.\n <P>\n <h4>Logging refinements</h4>\n <P>\n The core log methods (<code>logDebug()</code>, <code>logInfo()</code>,\n <code>logWarn()</code>) and the \"echo\" facilities (<code>echo()</code> and\n <code>echoAll()</code>) are available on every SmartClient component and Class.  Hence,\n in many cases, the special JavaScript value \"this\" will refer to an object that supports\n <code>logWarn()</code> et al.  For example:\n <pre>\n     Canvas.create({\n        ID:\"canvasExample\",\n        contents:\"Hello World!\",\n        click:\"this.logWarn('the Canvas is: ' + this.echo(this))\"\n     });\n </pre>\n The special value \"this\" is not always set to a SmartClient component, for example, in some\n kinds of callbacks (eg ${isc.DocUtils.linkForRef('method:ListGrid.fetchData','fetchData()')}).  When in doubt, use these\n methods via the Log class as <code>isc.Log.logWarn()</code>.\n <P>\n <b>Logging performance</b>\n <P>\n Because the log message is actually formed <i>before</i> the call to the log system, logs\n that are suppressed can still carry a performance penalty.  This is particularly true of\n logs that output a lot of data or occur frequently.  To avoid this penalty, you can check in\n advance whether a message will be suppressed using \n ${isc.DocUtils.linkForRef('classMethod:Class.logIsDebugEnabled','isc.Log.logIsDebugEnabled()')} and\n ${isc.DocUtils.linkForRef('classMethod:Class.logIsInfoEnabled','isc.Log.logIsInfoEnabled()')}.  For example:\n <pre>\n   if (isc.Log.logIsInfoEnabled(\"myGridLoading\")) {\n      isc.Log.logInfo(\"first record is: \" +\n                      isc.Log.echo(myGrid.data.get(0)),\n                      \"myGridLoading\");\n   }\n </pre>\n Generally, it is only important to do this for logs that will occur multiple times during a\n given user interaction (eg a mousedown or keypress) and/or that call <code>echo()</code> on\n objects with many properties.",
        name:"debugging",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.waitForSave":{
        ref:"attr:ListGrid.waitForSave",
        flags:"IRWA",
        description:"If this is an editable listGrid, this property determines whether the user will be\n able to dismiss the edit form, or navigate to another cell while the save is in \n process (before the asynchronous server response returns).",
        name:"waitForSave",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:parsing":{
        ref:"group:parsing",
        refs:[],
        name:"parsing",
        seeAlso:[],
        type:"group"
    },
    "attr:ResultSet.initialData":{
        ref:"attr:ResultSet.initialData",
        flags:"IA",
        description:"Initial set of data for the ResultSet.\n <P>\n This data will be treated exactly as though it were the data returned from the\n ResultSet's first server fetch.\n <P>\n By default, <code>initialData</code> will be considered a complete response\n (all rows that match the ${isc.DocUtils.linkForRef('type:Criteria')} which the ResultSet was initialized with).\n <P>\n Set ${isc.DocUtils.linkForRef('attr:ResultSet.initialLength')} to treat <code>initialData</code> as a partial response,\n equivalent to receiving a ${isc.DocUtils.linkForRef('class:DSResponse')} with <code>startRow:0</code>,\n <code>endRow:initialData.length</code> and <code>totalRows:initialLength</code>.\n Normal data paging will then occur if data is requested for row indices not filled via \n <code>initialData</code>.",
        name:"initialData",
        seeAlso:[],
        valueType:"Array of Record",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching", "cacheSync"]
    },
    "attr:ListGrid.removeFieldDefaults":{
        ref:"attr:ListGrid.removeFieldDefaults",
        flags:"IR",
        description:"Default configuration properties for the \"remove field\" displayed when\n ${isc.DocUtils.linkForRef('attr:ListGrid.canRemoveRecords')} is enabled. ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults')} should be used\n when modifying this object. \n <P>\n The default configuration includes a ${isc.DocUtils.linkForRef('method:ListGridField.recordClick')} handler which\n calls ${isc.DocUtils.linkForRef('method:ListGrid.removeData')} to actually perform the data removal.",
        name:"removeFieldDefaults",
        seeAlso:[],
        valueType:"ListGridField properties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Canvas.dragRepositionStop":{
        ref:"method:Canvas.dragRepositionStop",
        flags:"A",
        description:"\n Executed when the mouse button is released at the end of the drag. Your\n widget can use this opportunity to fire custom code based upon where the\n mouse button was released, etc. \n <p>\n Returning true from this handler will cause the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline if\n ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance')} is set to \"outline\") to be left in its current\n location. Returning false from this handler will cause it to snap back to its\n original location.\n\n",
        name:"dragRepositionStop",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to snap the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline) back to its                   original location or true to leave it at the current cursor position.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "method:Calendar.getActiveTime":{
        ref:"method:Calendar.getActiveTime",
        flags:"",
        description:" Gets a date object representing the date over which the mouse is hovering for the current\n selected view. For month view, the time will be set to midnight of the active day. For dayview\n and week view, the time will be the rounded to the closest half hour relative to the mouse\n position.\n",
        name:"getActiveTime",
        seeAlso:[],
        returns:{description:"the date that the mouse is over", type:"Date"},
        type:"method",
        definingClass:"class:Calendar"
    },
    "object:Record":{
        methods:[],
        ref:"object:Record",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:[],
        description:"A Record is an ordinary JavaScript Object with properties that are treated as data to\n be displayed and edited by a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}. \n <P>\n ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}s have a concept of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields','named fields')},\n where values for each field are found under the same-named property in a Record.\n <P>\n A Record is always an ordinary JavaScript Object regardless of how the record is \n loaded (static data, java server, XML web service, etc), and so supports the\n normal behaviors of JavaScript Objects, including accessing and assigning to properties\n via dot notation:\n <pre>\n     var fieldValue = record.<i>fieldName</i>;\n     record.<i>fieldName</i> = newValue;\n </pre>\n <P>\n The concept of working with Records is common to all ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}s, although\n individual DataBoundComponents may work with singular records (${isc.DocUtils.linkForRef('class:DynamicForm')}) or\n may work with lists (${isc.DocUtils.linkForRef('class:ListGrid')}), trees (${isc.DocUtils.linkForRef('class:TreeGrid')}), or cubes\n (${isc.DocUtils.linkForRef('class:CubeGrid')}) of records.\n <P>\n Individual DataComponents may also look for special properties on Records which\n control styling or behavior for those records, such as\n ${isc.DocUtils.linkForRef('attr:ListGrid.recordEditProperty','<code>record.canEdit</code>')}.",
        name:"Record",
        seeAlso:[],
        type:"object"
    },
    "attr:Window.minimizeTime":{
        ref:"attr:Window.minimizeTime",
        flags:"IRWA",
        description:"If this window is minimizeable, and animateMinimize is true, what should the duration of \n the minize / maximize be (in ms)? If unset defaults to <code>canvas.animationTime</code>.",
        name:"minimizeTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('windowMinimize')}"],
        definingClass:"class:Window",
        groups:["appearance", "header", "animation"]
    },
    "attr:TestFunctionResult.result":{
        ref:"attr:TestFunctionResult.result",
        flags:"IRW",
        description:"When a formula or summary format is valid, <i>result</i> contains the result returned by the\n generated function when it was executed.",
        name:"result",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TestFunctionResult",
        groups:["formulaFields"]
    },
    "attr:ListGridField.width":{
        ref:"attr:ListGridField.width",
        flags:"IRW",
        description:"The width of this field, specified as either an absolute number of pixels,\n a percentage of the remaining space like \"25%\", or \"*\" to split remaining space among\n all fields which have \"*\".\n <P>\n See also ${isc.DocUtils.linkForRef('attr:ListGrid.minFieldWidth')} to ensure no field goes below a minimum size.\n <P>\n Use ${isc.DocUtils.linkForRef('method:ListGrid.resizeField')} to programmatically change field width after creation.",
        name:"width",
        seeAlso:[],
        valueType:"Number or String",
        defaultValue:"\"*\"",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["appearance"]
    },
    "attr:Window.modalMaskStyle":{
        ref:"attr:Window.modalMaskStyle",
        flags:"IR",
        description:"Specifies the CSS style for the modal mask.",
        name:"modalMaskStyle",
        seeAlso:[
            {ref:"attr:Window.modalMask", description:null}
        ],
        valueType:"string",
        defaultValue:"\"modalMask\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["modal", "appearance"]
    },
    "group:hibernateIntegration":{
        ref:"group:hibernateIntegration",
        title:"Integration with Hibernate",
        treeLocation:["Concepts"],
        refs:[],
        description:"SmartClient can integrate with Hibernate in two main ways:\n <ul>\n <li> With pre-existing Hibernate configuration and Java beans, SmartClient\n server-side DSRequests can be transformed into Hibernate <code>Criteria</code> objects,\n and results returned via a SmartClient DSResponse.  \n <li> SmartClient can drive Hibernate as a storage layer only, automatically generating\n Hibernate configuration from a SmartClient DataSource file (<i>dataSourceID</i>.ds.xml).  In\n this case, you do not write a Java bean; Hibernate's beanless \n <a href='http://www.hibernate.org/hib_docs/v3/reference/en/html_single/#persistent-classes-dynamicmodels' onclick=\"window.open('http://www.hibernate.org/hib_docs/v3/reference/en/html_single/#persistent-classes-dynamicmodels');return false;\">\"dynamic model\"</a>\n mode is used.  This is enabled via ${isc.DocUtils.linkForRef('attr:DataSource.serverType')}:\"hibernate\".\n </ul>\n <P>\n <b>Mapping DSRequest/DSResponse to Hibernate</b>\n <P>\n This integration strategy uses the server-side Java APIs described in\n ${isc.DocUtils.linkForRef('group:serverDataIntegration')}.  Two complete examples of Hibernate integration are\n provided in the SDK, one using ${isc.DocUtils.linkForRef('class:DMI')} and one using ${isc.DocUtils.linkForRef('class:RPCManager')} dispatch.  Both\n are accessible from the SDK Explorer as Examples->Server Integration, and both include\n sample Hibernate configuration as well as a sample UI capable of loading and saving data.\n <ul>\n <li>\n <a href='/examples/server_integration/index.html#customDataSourceIntegrationHibernate' onclick=\"window.open('/examples/server_integration/index.html#customDataSourceIntegrationHibernate');return false;\">\"Custom DataSource Integration with Hibernate\"</a> \n shows how to create a .jsp that uses RPCManager dispatch to implement all four DataSource\n request ${isc.DocUtils.linkForRef('attr:DSRequest.operationType')}s via Hibernate for a sample bean.\n <li>\n <a href='/examples/server_integration/index.html#springDSHiberateDMI' onclick=\"window.open('/examples/server_integration/index.html#springDSHiberateDMI');return false;\">\"Spring DataSource Integration using DMI to Hibernate\"</a>\n shows how to use SmartClient Direct Method Invocation (${isc.DocUtils.linkForRef('class:DMI')}) to implement all four\n DataSource operations with Hibernate.  This particular example uses\n ${isc.DocUtils.linkForRef('group:springIntegration','Spring')}-based lookup of the target for DMI, however, the same\n approach will work without Spring - see other DMI examples on the same page for non-Spring\n methods of DMI target lookup.\n </ul>\n As discussed under ${isc.DocUtils.linkForRef('class:DMI')}, whether to use DMI or RPCManager dispatch is largely a matter\n of preference, however if you are already using Spring, a DMI-based approach is probably\n a better fit.\n <P>\n <b>serverType:\"hibernate\"</b>\n <P>\n As with DataSources using SmartClient's built-in ${isc.DocUtils.linkForRef('group:sqlDataSource','SQL engine')}, you\n need only ${isc.DocUtils.linkForRef('group:dataSourceDeclaration','create a DataSource')} in XML format - no Java\n classes or other configuration are required.  The ${isc.DocUtils.linkForRef('group:adminConsole','Admin Console')}'s\n \"Import DataSources\" section can be used to import test data into serverType:\"hibernate\"\n DataSources in the same manner as SQLDataSources.",
        name:"hibernateIntegration",
        seeAlso:[
            {ref:"attr:DataSource.beanClassName", description:null}
        ],
        type:"group"
    },
    "type:AutoChild":{
        ref:"type:AutoChild",
        description:"An autoChild is an automatically generated subcomponent that a component creates to\n handle part of its presentation or functionality.\n <P>\n An example is the Window component and its subcomponent the \"header\".\n <P>\n AutoChildren support a standard set of properties that can be used to customize or skin\n them.  The names of these properties are derived from the name of the autoChild itself.\n These properties will generally not be separately documented for every autoChild unless\n there are special usage instructions - the existance of the properties is implied\n whenever you see an autoChild documented.\n <P>\n The properties are:\n <dl>\n <dt> <b>\"show\" + name</b> (eg showHeader)\n <dd> Controls whether this autoChild should be created and shown at all.  Note that the\n first letter of the autoChild name is uppercased for this property (\"header\" -> \"Header\").\n\n <dt> <b>name + \"Properties\"</b> (eg headerProperties)\n <dd> Properties to apply to the autoChild created by this particular instance of the\n parent component.  For example:\n <pre>\n        isc.Window.create({\n            headerProperties: { layoutMargin : 10 }\n        });\n </pre>\n Would apply a ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin','layoutMargin')} to the header of the Window,\n increasing the empty space around the subcomponents of the header (buttons, title label,\n etc).\n <P>\n Generally, *Properties is null.  <b>Do not</b> use the *Properties mechanism for\n skinning.  See below. \n\n <dt> <b>name + \"Defaults\"</b> (eg headerDefaults)\n <dd> Defaults that will be applied to the autoChild in every instance of the parent\n component.  *Defaults is used for skinning.  This property should never be set when\n creating an instance of the parent component, as it will generally wipe out defaults\n required for the component's operation.  Use ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','changeDefaults')}\n to alter defaults instead, generally as part of a custom skin and/or custom component\n creation - see the \n ${isc.DocUtils.linkForRef('group:autoChildren','overview of AutoChildren for component development')} for details\n and examples.\n <dt> <b>name + \"Constructor\"</b> (eg headerConstructor)\n <dd> SmartClient Class of the component to be created.  An advanced option, this\n property should generally only be used when there is documentation encouraging you to do\n so.  For example, the ListGrid offers the ability to use simple CSS-based headers or\n more complex ${isc.DocUtils.linkForRef('class:StretchImg')} based headers via\n ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor')}.  The constructor can also be specified using\n the <code>_constructor</code> property in the defaults for the autoChild.\n </dl>\n The AutoChild system can be used to create both ${isc.DocUtils.linkForRef('attr:Canvas.children','direct children')} \n and indirect children (children of children).  For example, the\n ${isc.DocUtils.linkForRef('attr:Window.minimizeButton','minimizeButton')} of the Window is also an autoChild, even\n though it is actually located within the window header.\n <P>\n <h4>Skinning AutoChildren</h4>\n <P>\n Skinning autoChildren, done via ${isc.DocUtils.linkForRef('classMethod:Class.changeDefaults','changeDefaults()')}, is done\n for two purposes:\n <ul>\n <li> Changing the default appearance or behavior of a component, for example, making all\n Window headers shorter\n <li> Creating a customized variation of an existing component <i>while retaining the\n base component unchanged</i>.  For example, creating a subclass of Window called\n \"PaletteWindow\" with very compact appearance, while leaving the base Window class\n unchanged so that warning dialogs and other core uses of Windows do not look like\n PaletteWindows.\n </ul>\n The best code examples for skinning are in the load_skin.js file for the \"SmartClient\"\n skin, in <code>isomorphic/skins/SmartClient/load_skin.js</code>.\n <P>\n <h4>Passthroughs (eg window.headerStyle)</h4>\n <P>\n In many cases a component will provide shortcuts to skinning or customizing its\n autoChildren, such as ${isc.DocUtils.linkForRef('attr:Window.headerStyle')}, which becomes header.styleName.  When\n these shortcuts exist, they must be used instead of the more general autoChild skinning\n system.\n <P>\n <h4>Safe Skinning</h4>\n <P>\n Before skinning an autoChild consider the \n ${isc.DocUtils.linkForRef('group:safeSkinning','safe skinning guidelines')}.\n <P>\n <h4>Accessing AutoChildren Dynamically</h4>\n <P>\n For a component \"Window\" with an autoChild named \"header\", if you create a Window\n with ID \"myWindow\", the header autoChild is available as \"myWindow.header\".\n <P>\n Unless documented otherwise, an autoChild should be considered an internal part of a\n component.  Always configure autoChildren by APIs on the parent component when they\n exist.  It makes sense to access an autoChild for troubleshooting purposes or for\n workarounds, but in general, an autoChild's type, behavior, and internal structure are\n subject to change without notice in future SmartClient versions.\n <P>\n Accessing an autoChild may give you a way to make a dynamic change to a component that\n is not otherwise supported by the parent component (for example, changing a text label\n where there is no setter on the parent).  Before using this approach, consider whether\n simply recreating the parent component from scratch is a viable option - this approach\n is more than fast enough for most smaller components, and will not create a reliance on\n unsupported APIs.",
        name:"AutoChild",
        seeAlso:[],
        type:"type",
        groups:["autoChildren"]
    },
    "attr:Tab.ID":{
        ref:"attr:Tab.ID",
        flags:"IRW",
        description:"Optional ID for the tab, which can later be used to reference the tab.\n APIs requiring a referenct a tab will accept the tabs ID \n [including  ${isc.DocUtils.linkForRef('method:TabSet.selectTab')}, ${isc.DocUtils.linkForRef('method:TabSet.updateTab')}, ${isc.DocUtils.linkForRef('method:TabSet.removeTab')}].<br>\n The ID will also be passed to the ${isc.DocUtils.linkForRef('method:TabSet.tabSelected')} and ${isc.DocUtils.linkForRef('method:TabSet.tabDeselected')}\n handler functions, if specified.\n <p>\n Note that if you provide an ID, it must be globally unique.",
        name:"ID",
        seeAlso:[],
        valueType:"identifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab"
    },
    "method:TileGrid.selectAllRecords":{
        ref:"method:TileGrid.selectAllRecords",
        flags:"",
        description:" Select all records\n\n",
        name:"selectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "method:Calendar.fetchData":{
        ref:"method:Calendar.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component.\n <P>\n If there are a large number of matching records, paging will automatically be enabled, so\n that initially a smaller number of records will be retrieved and further records will\n be fetched as the user navigates the dataset.\n <P>\n When first called, this method will create a ${isc.DocUtils.linkForRef('class:ResultSet')}, which will be\n configured based on component settings such as ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fetchOperation')}\n and ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataPageSize')}, as well as the general purpose\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')}.  The ResultSet is then available as\n <code>component.data</code>.\n <P>\n Subsequent calls to fetchData() will simply call ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria','setCriteria()')}\n on the created ResultSet with the passed criteria.\n <P>\n In some cases fetchData() will not need to contact the server as the new criteria can be\n satisfied by performing a client-side filter against the currently cached set of data.\n You can determine whether criteria will cause a fetch by calling \n ${isc.DocUtils.linkForRef('method:ResultSet.willFetchData')}.  If you need to force a server fetch, you can call\n ${isc.DocUtils.linkForRef('method:ListGrid.invalidateCache','invalidateCache()')} to do so.\n <P>\n This method takes an optional callback parameter (set to a ${isc.DocUtils.linkForRef('type:DSCallback')}) to fire when\n the fetch completes. Note that this callback will not fire if no server fetch is performed.\n In this case the data is updated synchronously, so as soon as this method completes you\n can interact with the new data. If necessary, you can use\n {dataBoundComponent.willFetchData,willFetchData()} to determine whether or not a server\n fetch will occur when <code>fetchData()</code> is called with new criteria.\n <P>\n In addition to the callback parameter for this method, developers can use \n ${isc.DocUtils.linkForRef('method:ListGrid.dataArrived','dataArrived()')} to be notified every time data is loaded.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundFetch')}"],
        definingClass:"class:Calendar",
        groups:["dataBoundComponentMethods"]
    },
    "attr:MultiSortDialog.deleteLevelButton":{
        ref:"attr:MultiSortDialog.deleteLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for deleting levels\n from the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.deleteLevelButtonProperties</code> and \n <code>multiSortPanel.deleteLevelButtonDefaults</code>.",
        name:"deleteLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:ListGrid.clearCriteria":{
        ref:"method:ListGrid.clearCriteria",
        flags:"",
        description:" Clear the current criteria used to filter data.\n\n",
        name:"clearCriteria",
        seeAlso:[
            {ref:"method:ListGrid.fetchData", description:null}
        ],
        params:[
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundFilter')}"],
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DataSourceField.title":{
        ref:"attr:DataSourceField.title",
        flags:"IR",
        description:"Default user-visible title for this field.\n <p>\n This will be picked up by DataBound components and other views over this DataSource.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "class:DatabaseBrowser":{
        methods:["method:DatabaseBrowser.getGeneratedDataSource", "method:DatabaseBrowser.getResults"],
        ref:"class:DatabaseBrowser",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:DatabaseBrowser.serverType", "attr:DatabaseBrowser.dbName", 
         "attr:DatabaseBrowser.catalog","attr:DatabaseBrowser.schema", "attr:DatabaseBrowser.schemaTree", 
         "attr:DatabaseBrowser.databaseList","attr:DatabaseBrowser.dataGrid", "attr:DatabaseBrowser.selectButton", 
         "attr:DatabaseBrowser.title","attr:DatabaseBrowser.includeSubstring", "attr:DatabaseBrowser.excludeSubstring"],
        description:"A component that connects to a database (and, depending on the RDBMS in use, optionally a \n particular catalog and/or schema) and displays the currently-defined tables, optionally \n filtered in a variety of ways.  It also allows you to view the column details\n of a selected table, and optionally retrieves and displays the data currently in that table.\n <p>\n The DatabaseBrowser can also create a SmartClient DataSource from any existing SQL table.",
        name:"DatabaseBrowser",
        seeAlso:[],
        type:"class"
    },
    "attr:TileGrid.data":{
        ref:"attr:TileGrid.data",
        flags:"IRW",
        description:"A List of TileRecord objects, specifying the data to be used to create the\n tiles.  \n <p>\n This property will typically not be explicitly specified for databound TileGrids, where\n the data is returned from the server via databound component methods such as\n ${isc.DocUtils.linkForRef('method:TileGrid.fetchData')}. In this case the data objects will be set to a \n ${isc.DocUtils.linkForRef('class:ResultSet','resultSet')} rather than a simple array.",
        name:"data",
        setter:["TileGrid.setData"],
        seeAlso:[
            {ref:"object:TileRecord", description:null}
        ],
        valueType:"List of TileRecord",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileGrid",
        groups:["data"]
    },
    "object:isA":{
        methods:[],
        ref:"object:isA",
        classAttrs:[],
        classMethods:["classMethod:isA.List", "classMethod:isA.emptyString", "classMethod:isA.nonemptyString", 
         "classMethod:isA.Object","classMethod:isA.emptyObject", "classMethod:isA.emptyArray", "classMethod:isA.String", 
         "classMethod:isA.Array","classMethod:isA.Function", "classMethod:isA.Number", "classMethod:isA.Boolean", 
         "classMethod:isA.Date","classMethod:isA.RegularExpression", "classMethod:isA.Instance", 
         "classMethod:isA.ClassObject","classMethod:isA.Interface"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"A library of functions for determining the types of other objects.<br><br>\n\n  The \"isA\" methods for the basic JavaScript types are much faster and more consistent across\n  platforms than JavaScript's \"typeof\" operator.<br><br>\n\n  An isA method is automatically created for every ISC Class and Interface definition, for\n  example, isA.Canvas().<br><br>",
        name:"isA",
        seeAlso:[],
        type:"object",
        examples:[
         "<code>if (isA.Number(myVariable)) ...</code>\n	Note: <code>is</code> and <code>isAn</code> are synonyms of <code>isA</code> and can be used\n			interchangably when it looks better syntactically, eg:\n				<code>if (myObject == null) ...</code>\n			or\n				<code>if (isAn.Array(myObject)) ...</code>"]
    },
    "class:HibernateBrowser":{
        methods:["method:HibernateBrowser.getGeneratedDataSource", "method:HibernateBrowser.getResults"],
        ref:"class:HibernateBrowser",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:HibernateBrowser.mappingTree", "attr:HibernateBrowser.dataGrid", 
         "attr:HibernateBrowser.title","attr:HibernateBrowser.selectButton", "attr:HibernateBrowser.includeSubstring", 
         "attr:HibernateBrowser.excludeSubstring"],
        description:"A component that connects to a Hibernate configuration and displays the currently-mapped \n entities.  It also allows you to view the bean properties of a selected entity, and optionally \n retrieves and displays the data currently in the corresponding database table.\n <p>\n The HibernateBrowser can also create a SmartClient DataSource from any existing Hibernate\n mapping.",
        name:"HibernateBrowser",
        seeAlso:[],
        type:"class"
    },
    "attr:LoginDialog.dismissOnEscape":{
        ref:"attr:LoginDialog.dismissOnEscape",
        flags:"IRW",
        description:"Do not set LoginDialog.dismissOnEscape; it is controlled by the \n ${isc.DocUtils.linkForRef('attr:LoginDialog.dismissable')}\n property.",
        name:"dismissOnEscape",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:Canvas.initWidget":{
        ref:"method:Canvas.initWidget",
        flags:"",
        description:"\n For custom components, perform any initialization specific to your widget subclass.\n <P>\n When creating a subclass of any Canvas-based component, you should generally override\n this method rather than overriding ${isc.DocUtils.linkForRef('method:Class.init')}.  This is because Canvas has its own\n ${isc.DocUtils.linkForRef('method:Class.init')} override which performs some generally desirable initialization - see\n ${isc.DocUtils.linkForRef('method:Canvas.init')} for details.\n <p>\n This method is called by ${isc.DocUtils.linkForRef('method:Canvas.init')} when a component is create()d.  When\n overriding this method, You must call the superClass initWidget implementation, like\n so:\n <pre>\n    this.Super(\"initWidget\", arguments);\n </pre>\n <P>\n In general, if you are going to call functionality supported by your superclass (eg calling\n addTab() when your superclass is a TabSet), call Super() first.  However, you can generally\n assign properties to <code>this</code> before calling Super() as a way of mimicing the\n effect of the property being passed to ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')} on normal instance\n construction.  For example, when subclassing a DynamicForm, you could set this.items to a\n generated set of items before calling Super().\n <P>\n NOTE: child creation: if you are creating a component that auto-creates certain children (eg\n a Window which creates a Header child), typical practice is to create those children\n immediately before drawing by overriding draw().  This postpones work until it is really\n necessary and avoids having to update children if settings are changed between creation and\n draw().  Alternatively, if you prefer callers to directly manipulate auto-created children,\n it's best to create them earlier in initWidget(), in order to allow manipulation before\n draw.\n\n",
        name:"initWidget",
        seeAlso:[],
        params:[
            {description:"All arguments initially passed to ${isc.DocUtils.linkForRef('classMethod:Class.create')}",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:Canvas.animateShow":{
        ref:"method:Canvas.animateShow",
        flags:"",
        description:" Show a canvas by growing it vertically to its fully drawn height over a period of time.\n This method will not fire if the widget is already drawn and visible, or has overflow\n other than <code>\"visible\"</code> or <code>\"hidden\"</code>.\n",
        name:"animateShow",
        seeAlso:[],
        params:[
            {description:"Animation effect to use      when revealing the widget. If ommitted, default behavior can be configured via\n      ${isc.DocUtils.linkForRef('attr:Canvas.animateShowEffect')}",
             optional:true, name:"effect", defaultValue:null,
             type:"animateShowEffectId | animateShowEffect"},
            {description:"When the show completes this callback will be fired. Single                              'earlyFinish' parameter will be passed if the animation was\n                              cut short by a call to finishAnimation.",
             optional:true, name:"callback", defaultValue:null, type:"callback"},
            {description:"Duration in ms of the animated show. If unset, duration will be   picked up from ${isc.DocUtils.linkForRef('attr:Canvas.animateShowTime')}",
             optional:true, name:"duration", defaultValue:null, type:"number"},
            {description:"Optional acceleration effect function to    bias the animation ratios.  If unset, acceleration will be picked up from\n   ${isc.DocUtils.linkForRef('attr:Canvas.animateShowAcceleration')}",
             optional:true, name:"acceleration", defaultValue:null, type:"AnimationAcceleration"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('animateWipe')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:Button.select":{
        ref:"method:Button.select",
        flags:"",
        description:" Select this object.\n",
        name:"select",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:DSRequestModifier.value":{
        ref:"attr:DSRequestModifier.value",
        flags:"IR",
        description:"The value to assign to the field named by ${isc.DocUtils.linkForRef('attr:DSRequestModifier.fieldName')}.  This value can be static, or it\n can be an expression in the Velocity template language.  In this latter case, there are a \n number of context variables available to you:\n <ul><li><b>dsRequest</b>. The DSRequest object being processed</li>\n <li><b>servletRequest</b>. The HttpServletRequest object representing the queue of requests\n sent from the client</li>\n <li><b>responseData</b>. The data member of a DSResponse object associated with an earlier \n DSRequest in the same queue</li></ul>\n (Note that the standard Velocity context variables provided by SmartClient Server are also \n available to you - see ${isc.DocUtils.linkForRef('group:velocitySupport')}).\n <p>\n The most interesting of these is the last. An example of when this might be useful would be a\n queued batch of \"add\" operations for an order header and its details - the detail additions need\n to know the unique primary key that was assigned to the order.  This value will be in the \n response to the DSRequest that added the order header.  See this example:\n ${isc.DocUtils.linkForExampleId('queuedAdd')}.\n\n You can obtain the responseData of prior responses using the following Velocity syntax:<p>\n <code>$responseData.first.myField</code> is the myField property of the first response in the queue<br>\n <code>$responseData.first('order').myField</code> is the myField property of the first response to an \n operation (any operation) on the \"order\" DataSource<br>\n <code>$responseData.first('order', 'add').myField</code> is the myField property of the first \n response to an \"add\" operation on the \"order\" DataSource<p>\n All of these syntactic variations are also available on the <code>$responseData.last</code> object - \n \"last\" here meaning the most recent response matching the DataSource and operation type (if \n applicable).",
        name:"value",
        seeAlso:[
            {ref:"group:velocitySupport", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:DSRequestModifier"
    },
    "class:Progressbar":{
        methods:["method:Progressbar.setPercentDone", "method:Progressbar.percentChanged", 
         "method:Progressbar.getLength","method:Progressbar.getBreadth", "method:Progressbar.setLength", 
         "method:Progressbar.setBreadth"],
        ref:"class:Progressbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:Progressbar.percentDone", "attr:Progressbar.length", "attr:Progressbar.breadth", 
         "attr:Progressbar.vertical","attr:Progressbar.src"],
        description:"The Progressbar widget class extends the StretchImg class to implement image-based progress\n bars (graphical bars whose lengths represent percentages, typically of task completion).",
        name:"Progressbar",
        seeAlso:[],
        type:"class"
    },
    "attr:SubmitItem.title":{
        ref:"attr:SubmitItem.title",
        flags:"IRW",
        description:"SubmitItems show a title of <code>\"Submit\"</code> by default. May be overridden.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Submit\"",
        type:"attr",
        definingClass:"class:SubmitItem"
    },
    "method:List.indexOf":{
        ref:"method:List.indexOf",
        flags:"",
        description:" Return the position in the list of the first instance of the specified object.\n <p>\n If pos is specified, starts looking after that position.\n <p>\n Returns -1 if not found.\n\n",
        name:"indexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"earliest index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"last index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:ImgSplitbar.vertical":{
        ref:"attr:ImgSplitbar.vertical",
        flags:"R",
        description:"Is this split bar vertically orientated?<br>\n When a <code>Splitbar</code> is created by a layout to be the resizeBar for some\n member of the layout, the <code>vertical</code> property will be set to <code>true</code>\n if the layout is horizontal, meaning this resizeBar will be taller than it is wide, \n and will allow horizontal resizing of the member.",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "attr:CalendarEvent.canEdit":{
        ref:"attr:CalendarEvent.canEdit",
        flags:"IRW",
        description:"Optional boolean value controlling the editability of this particular calendarEvent.\n  The name of this field within the CalendarEvent can be changed via \n  ${isc.DocUtils.linkForRef('attr:Calendar.canEditField')}.",
        name:"canEdit",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "attr:ValuesManager.saveOperationType":{
        ref:"attr:ValuesManager.saveOperationType",
        flags:"IRW",
        getter:["getSaveOperationType"],
        description:"Default ${isc.DocUtils.linkForRef('type:DSOperationType')} to be performed when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is called.\n This property is automatically set on a call to ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} or\n ${isc.DocUtils.linkForRef('method:DynamicForm.editNewRecord')}, or may be set directly via \n ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')}.\n <P>\n If <code>saveOperationType</code> is unset, the form will heuristically determine\n whether an \"add\" or \"update\" operation is intended based on whether the primaryKey field\n is present and editable.",
        name:"saveOperationType",
        setter:["setSaveOperationType"],
        seeAlso:[],
        valueType:"DSOperationType",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ValuesManager"
    },
    "method:ListGrid.getFocusRow":{
        ref:"method:ListGrid.getFocusRow",
        flags:"A",
        description:" Get the row that currently has keyboard focus.  Arrow key navigation moves relative to this\n row.\n \n",
        name:"getFocusRow",
        seeAlso:[],
        returns:{description:"rowNum of the current focus row", type:"Number"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:FacetValue.id":{
        ref:"attr:FacetValue.id",
        flags:"IRW",
        description:"id of this facetValue.  Any string or number.",
        name:"id",
        seeAlso:[],
        valueType:"String | Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "class:TextItem":{
        methods:["method:TextItem.getHint", "method:TextItem.getSelectionRange", 
         "method:TextItem.setSelectionRange","method:TextItem.selectValue", "method:TextItem.deselectValue", 
         "method:TextItem.setKeyPressFilter","method:TextItem.setMask"],
        exampleConfig:"TextItem.xml",
        ref:"class:TextItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:TextItem.width", "attr:TextItem.height", "attr:TextItem.textBoxStyle", 
         "attr:TextItem.length","attr:TextItem.selectOnFocus", "attr:TextItem.showHintInField", 
         "attr:TextItem.characterCasing","attr:TextItem.keyPressFilter", "attr:TextItem.mask", "attr:TextItem.maskOverwriteMode", 
         "attr:TextItem.maskSaveLiterals","attr:TextItem.maskPadChar", "attr:TextItem.maskPromptChar"],
        description:"FormItem for managing a text field.",
        name:"TextItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('textItem')}"]
    },
    "attr:FormItem.textBoxStyle":{
        ref:"attr:FormItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class name for a form item's text box element.\n <P>\n NOTE: See the ${isc.DocUtils.linkForRef('group:CompoundFormItem_skinning')} discussion for special skinning considerations.",
        name:"textBoxStyle",
        seeAlso:[
            {ref:"attr:FormItem.cellStyle", description:null}
        ],
        valueType:"FormItemBaseStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:TileGrid.dataArrived":{
        ref:"method:TileGrid.dataArrived",
        flags:"A",
        description:" Notification method fired when new data arrives from the server to be displayed in this\n tileGrid, (for example in response to the user scrolling a new set of tiles into view).\n Only applies to databound tileGrid where the ${isc.DocUtils.linkForRef('attr:TileGrid.data','data')} attribute is a\n ${isc.DocUtils.linkForRef('class:ResultSet')}.\n This method is fired directly in\n response to ${isc.DocUtils.linkForRef('method:ResultSet.dataArrived','dataArrived()')} firing on the data object.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"starting index of the newly loaded set of records", optional:false,
             name:"startRecord", defaultValue:null, type:"integer"},
            {description:"ending index of the newly loaded set of records (non inclusive).",
             optional:false, name:"endRecord", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:TileGrid"
    },
    "method:Selection.anySelected":{
        ref:"method:Selection.anySelected",
        flags:"",
        description:" Whether at least one item is selected\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:StatefulCanvas.setState":{
        ref:"method:StatefulCanvas.setState",
        flags:"A",
        description:" Set the 'state' of this object, this changes it's appearance.\n\n",
        name:"setState",
        seeAlso:[
            {ref:"method:StatefulCanvas.setDisabled", description:"which also affects state values."}
        ],
        params:[
            {description:"new state", optional:false, name:"newState", defaultValue:null, type:"State"}
        ],
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state", "appearance"]
    },
    "attr:ListGrid.showDetailFields":{
        ref:"attr:ListGrid.showDetailFields",
        flags:"IR",
        description:"Whether to include fields marked <code>detail:true</code> from this component's \n <code>DataSource</code>.\n <P>\n Fields may also be included directly in this component's <code>fields</code> array in\n which case they will be present regardless of the <code>detail</code> attribute.\n <p>\n When this property is <code>true</code>, <code>ListGrid</code>s will include all \n detail fields, but they will be initially hidden.  The user may show\n these fields via the default header context menu (${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu')}).\n Setting this property to false will completely exclude all detail fields from the list\n grid's fields array, such that they cannot be shown by the user or programmatically.\n <P>\n To override the visibility of individual fields, use the standard\n ${isc.DocUtils.linkForRef('method:ListGrid.showField')}, ${isc.DocUtils.linkForRef('method:ListGrid.hideField')} and ${isc.DocUtils.linkForRef('method:ListGridField.showIf')}\n APIs, for example, set showIf:\"true\" to show a detail field initially.",
        name:"showDetailFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["databinding"]
    },
    "attr:Canvas.snapOffsetLeft":{
        ref:"attr:Canvas.snapOffsetLeft",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')} is defined for this widget, this property can be used to\n specify an offset in px or percentage for the left coordinate of this widget.\n <P>\n For example if <code>snapTo</code> is specifed as <code>\"L\"</code> and \n <code>snapOffsetLeft</code> is set to 6, this widget will be rendered 6px inside the left\n edge of its parent or master element.",
        name:"snapOffsetLeft",
        seeAlso:[
            {ref:"attr:Canvas.snapTo", description:null}
        ],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:FormItem.setTop":{
        ref:"method:FormItem.setTop",
        flags:"A",
        description:" For a form with ${isc.DocUtils.linkForRef('attr:DynamicForm.itemLayout','itemLayout')}:\"absolute\" only, set the top\n coordinate of this form item.\n <P>\n Causes the form to redraw.\n",
        name:"setTop",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:ListGrid.fixedRecordHeights":{
        ref:"attr:ListGrid.fixedRecordHeights",
        flags:"IRWA",
        description:"Should we vertically clip cell contents, or allow rows to expand vertically to show all\n contents?\n <P>\n If we allow rows to expand, the row height as derived from\n ${isc.DocUtils.linkForRef('method:GridRenderer.getRowHeight','getRowHeight()')} or the\n default ${isc.DocUtils.linkForRef('attr:ListGrid.cellHeight')} is treated as a minimum.\n <P>\n <b>NOTE:</b> by default, for performance reasons, clipping is not enforced for some\n kinds of content (such as images) on all browsers.  Set\n ${isc.DocUtils.linkForRef('attr:ListGrid.enforceVClipping','enforceVClipping:true')} to enforce clipping for all types of\n content on all browsers.",
        name:"fixedRecordHeights",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('autofitValues')}"],
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "group:webService":{
        ref:"group:webService",
        refs:["method:WebService.getOperationNames", "method:WebService.getSchema", 
         "method:WebService.callOperation","method:WebService.getFetchDS", "method:WebService.setLocation", "class:WebService", 
         "classMethod:WebService.get","classMethod:WebService.getByName", "attr:SchemaSet.schemaNamespace", 
         "attr:WebService.serviceNamespace"],
        name:"webService",
        seeAlso:[],
        type:"group"
    },
    "attr:Calendar.canCreateEvents":{
        ref:"attr:Calendar.canCreateEvents",
        flags:"IR",
        description:"If true, users can create new events",
        name:"canCreateEvents",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["allowedOperations"]
    },
    "method:TabSet.scrollForward":{
        ref:"method:TabSet.scrollForward",
        flags:"",
        description:" If there is not enough space to display all the tabs in this tabSet, this method will \n scroll the next tab (that first tab that is clipped at the end of the tab-bar) into view.\n",
        name:"scrollForward",
        seeAlso:[],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:HTMLFlow.contentsURL":{
        ref:"attr:HTMLFlow.contentsURL",
        flags:"IRW",
        description:"URL to load content from.\n <P>\n If specified, this component will load HTML content from the specified URL when it is\n first drawn.\n <p>\n This feature relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.",
        name:"contentsURL",
        setter:["setContentsURL"],
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "method:ColorPicker.getRed":{
        ref:"method:ColorPicker.getRed",
        flags:"",
        description:" Returns the Red element of the currently-selected color, as an integer from 0-255\n",
        name:"getRed",
        seeAlso:[
            {ref:"method:ColorPicker.setRed", description:null}
        ],
        returns:{description:"red color component", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:TileGrid.dragTrackerStyle":{
        ref:"attr:TileGrid.dragTrackerStyle",
        flags:"IRW",
        description:"CSS Style to apply to the drag tracker when dragging occurs on this component.",
        name:"dragTrackerStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"gridDragTracker\"",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "group:loadISCTag":{
        ref:"group:loadISCTag",
        title:"&lt;isomorphic:loadISC&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> HTML\n <p>\n This tag sets window.isomorphicDir and loads a default set of SmartClient modules and a\n skin.  SmartClient modules are JavaScript files that need to be loaded via \n <code>&lt;SCRIPT SRC=/some/filename.js&gt;&lt;/SCRIPT&gt;</code>\n blocks as part of the loading HTML page.  The modules themselves are stored in the\n <code>[webroot]/isomorphic/system/modules</code> directory.  A skin defines the look and feel of your\n website and typically consists of at least one CSS file and some SmartClient UI Component\n default overrides.  Skins are located in <code>[webroot]/isomorphic/skins</code>.  Loading a skin\n consists of loading the corresponding <code>load_skin.js</code> - residing at top-level in\n each skin directory.\n <p>\n The default set of modules loaded by this tag are: <code>Core, Foundation, Containers,\n Grids, Forms, DataBinding</code>.  The default skin is the <code>SmartClient</code> skin.\n So, for example inserting this into your JSP:\n <pre>\n &lt;isomorphic:loadISC/&gt;\n </pre>\n Produces the following output:\n <pre>\n &lt;SCRIPT&gt;window.isomorphicDir='isomorphic/';&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Core.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Foundation.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Containers.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Grids.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Forms.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_DataBinding.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT src=isomorphic/skins/SmartClient/load_skin.js&gt;&lt;/SCRIPT&gt;\n </pre>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>skin</b><br>\n <i>value format</i>: Name of skin to load.  See the <code>[webroot]/isomorphic/skins</code> directory\n for a list of available skins.  You can also create your own skin by copying an existing\n skin directory recursively, modifying anything you need, and using your new skin name as the\n value here.<br>\n <i>default value</i>: \"SmartClient\"\n <p>\n This selects a skin other than the default.  The value of this attribute is used as a path\n component to <code>load_skin.js</code>.  For example, specifying:\n <pre>\n &lt;isomorphic:loadISC skin=\"standard\"/&gt;\n </pre>\n Produces the following <code>load_skin.js</code> directive (in addition to all the standard\n module loads):\n <pre>\n &lt;SCRIPT src=isomorphic/skins/standard/load_skin.js&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n <b>skinDir</b><br>\n <i>value format</i>: path or URL up to and including the name of the skin<br>\n <i>default value</i>: automatically derived from the value of the <code>skin</code>\n attribute\n <p>\n This attribute is usable in lieu of the <code>skin</code> attribute and allows you to\n specify the full path or URL to the directory containing the load_skin.js for your skin.\n This allows you to move the directory containing the skin anywhere you want - including to a\n completely different server.\n <p>\n <b>includeModules</b><br>\n <i>value format</i>: Comma separated list of additional modules to load.  See the\n <code>[webroot]/isomorphic/system/modules</code> directory for a list of loadable modules.  The\n leading \"ISC_\" is optional, spaces after commas are also optional.<br>\n <i>default value</i>: NONE\n <p>\n This attribute allows you to specify SmartClient modules that you want to load in addition\n to the default set of modules. For example, specifying:\n <pre>\n &lt;isomorphic:loadISC includeModules=\"Analytics\"/&gt;\n </pre>\n Produces the following output:\n <pre>\n &lt;SCRIPT&gt;window.isomorphicDir='isomorphic/';&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Core.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Foundation.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Containers.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Grids.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Forms.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_DataBinding.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT SRC=isomorphic/system/development/ISC_Analytics.js&gt;&lt;/SCRIPT&gt;\n &lt;SCRIPT src=isomorphic/skins/SmartClient/load_skin.js&gt;&lt;/SCRIPT&gt;\n </pre>\n Notice the addition of the line that loads <code>ISC_Analytics.js</code>.  SmartClient\n modules must be loaded in a particular order and may have dependencies on other modules.\n The <code>loadISC</code> automatically resolves any missing dependencies and includes them\n in the list of modules to load.  The list of modules to load is also automatically reordered\n as required by this tag, so you can specify just the modules you care about, in any order,\n and <code>loadISC</code> will do the right thing.\n <p>\n <b>modules</b><br>\n <i>value format</i>: Comma separated list of modules to load.  See the\n <code>[webroot]/isomorphic/system/modules</code> directory for a list of loadable modules.  The\n leading \"ISC_\" is optional, spaces after commas are also optional.<br>\n <i>default value</i>: \"Core, Foundation, Containers, Grids, Forms, DataBinding\"\n <p>\n Specifying this attribute overrides the default set of modules loaded by\n <code>loadISC</code>. This is useful if you don't need to load all of the default modules on\n a particular page.\n <p>\n <b>isomorphicURI</b><br>\n <i>value format</i>: absolute or relative (from current URI) path to the\n <code>isomorphic</code> directory (by default, located in webRoot).<br>\n <i>default value</i>: automatically derived\n <p>\n This value sets <code>window.isomorphicDir</code> and defines the base path to SmartClient\n modules from the currently exectuing JSP.  By default, this value is automatically derived\n and you don't need to override it.  Generally, an override is only required if you're\n pointing multiple web application deployments at one base URL for cache coherence reasons.\n <p>\n <b>defer</b><br>\n <i>value format</i>: boolean - acceptable values: \"true\" or \"false\"<br>\n <i>default value</i>: \"false\"\n <p>\n This attribute applies only if you're using the optional Network Performance component of\n SmartClient.  By default, the modules specified by the <code>loadISC</code> tag are loaded\n synchronously - that is, the browser waits for each module to download before rendering\n any HTML on the page.  Specifying <code>defer=\"true\"</code> makes the module load\n asynchronous.  In this mode, all the page content except the modules specified by\n <code>loadISC</code> is loaded and rendered first, and then the SmartClient modules are\n loaded.  The asynchronous mode makes it possible to render some content and allow the user\n to interact with the site immediately, while SmartClient modules load in the background.\n <p>\n Note that typically SmartClient modules are delivered compressed with cache headers, so the\n download delay really only affects first time users of your site, those whose cached\n versions have expired, or if you've upgraded your SmartClient module to a newer version that\n the user has cached.\n <p>\n Using this directive is equivalent to calling ${isc.DocUtils.linkForRef('classMethod:FileLoader.loadISC')}.\n <p>\n <b>cacheOnly</b><br>\n <i>value format</i>: boolean - acceptable values: \"true\" or \"false\"<br>\n <i>default value</i>: \"false\"\n <p>\n This attribute applies only if you're using the optional Network Performance component of\n SmartClient.  This works like <code>defer</code> (see above), except that the modules are\n only loaded into the browser cache, but not executed.  This means that when the specified\n modules download, they'll be added to the browser cache, but will not be available to the\n execution context of the page.  Actually executing the module code takes some processing\n time on the client, whereas simply caching it does not.  The typical use case for this\n directive is on some intermediary page before the user gets to a SmartClient-enabled page.\n For example, you could include a cacheOnly <code>loadISC</code> tag on a login page that\n does not use SmartClient components, but protects a SmartClient-enabled application.  While\n the user is typing in their credentials, the SmartClient modules will cache in the\n background, then once the user logs in and is sent to a SmartClient-enabled page, the\n modules are simply read from cache.\n <p>\n Using this directive is equivalent to calling ${isc.DocUtils.linkForRef('classMethod:FileLoader.cacheISC')}.\n <p>\n <b>onload</b><br>\n <i>value format</i>: String - any javascript code<br>\n <i>default value</i>: NONE\n <p>\n This attribute applies only if you're using the optional Network Performance component of\n SmartClient.  This attribute is usable in conjuction with <code>defer</code> and\n <code>cacheOnly</code> attributes.  This attribute allows you to specify a callback to\n execute when loading or caching of the specified modules has completed.  You can use this\n to, for example, replace portions of the page with SmartClient components once SmartClient\n has loaded.\n <p>\n The value of this attribute is passed as the <code>onload</code> handler to the generated\n ${isc.DocUtils.linkForRef('classMethod:FileLoader.cacheISC')} or ${isc.DocUtils.linkForRef('classMethod:FileLoader.loadISC')} call as appropriate.\n <p>\n <b>useSimpleNames</b><br>\n <i>value format</i>: boolean - acceptable values: \"true\" or \"false\"<br>\n <i>default value</i>: \"true\"\n <p>\n Setting this attribute to false puts SmartClient in namespace mode.  See ${isc.DocUtils.linkForRef('class:isc')}\n for more info.",
        name:"loadISCTag",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "attr:ComboBoxItem.mask":{
        ref:"attr:ComboBoxItem.mask",
        flags:null,
        description:"Not applicable to a ComboBoxItem.",
        name:"mask",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:DateChooser.baseButtonStyle":{
        ref:"attr:DateChooser.baseButtonStyle",
        flags:"IRW",
        description:"Base css style applied to this picker's buttons. Will have \"Over\", \"Selected\" and \"Down\"\n suffix appended as the user interacts with buttons.",
        name:"baseButtonStyle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"dateChooserButton\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "class:MultiView":{
        methods:["method:MultiView.recordClick", "method:MultiView.fetchData", 
         "method:MultiView.filterData","method:MultiView.fetchRelatedData", "method:MultiView.removeSelectedData", 
         "method:MultiView.anySelected","method:MultiView.getSelection", "method:MultiView.selectRecord", 
         "method:MultiView.selectRecords","method:MultiView.selectAllRecords", "method:MultiView.deselectRecord", 
         "method:MultiView.deselectRecords","method:MultiView.deselectAllRecords"],
        ref:"class:MultiView",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:MultiView.dataSource", "attr:MultiView.currentView", "attr:MultiView.currentDBC", 
         "attr:MultiView.showFilterEditor"],
        description:"MultiView is a compound UI component, based on the VLayout class, which allows switching\n and data-sharing between hosted ${isc.DocUtils.linkForRef('interface:DataBoundComponent','data-bound components')}.\n \n <ul>\n <li> ListGrid (default), with inline master/detail and editing support</li>\n <li> TileGrid, to display records as tiles, including an image as required</i>\n </ul>\n\n The MultiView manages data binding for its child components.",
        name:"MultiView",
        implementsInterfaces:["DataBoundComponent"],
        seeAlso:[],
        type:"class"
    },
    "method:MultiView.removeSelectedData":{
        ref:"method:MultiView.removeSelectedData",
        flags:"",
        description:" Remove the currently selected records from this component.\n If this is a databound grid, the records will be removed directly from the DataSource.\n <P>\n If no records are selected, no action is taken. The grid will automatically be\n updated if the record deletion succeeds.\n\n",
        name:"removeSelectedData",
        seeAlso:[],
        params:[
            {description:"callback to fire when the data has been removed", optional:true,
             name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('removeOperation')}"],
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "method:ValuesManager.getMember":{
        ref:"method:ValuesManager.getMember",
        flags:"",
        description:"  Returns a pointer to a specific member.\n",
        name:"getMember",
        seeAlso:[],
        params:[
            {description:"ID of the member component to retrieve", optional:false, name:"ID",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"member (or null if unable to find a member with the  specified ID).",
         type:"Canvas"},
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "attr:HibernateBrowser.selectButton":{
        ref:"attr:HibernateBrowser.selectButton",
        flags:"IR",
        description:"Instance of Button used to continue once a table has been selected.",
        name:"selectButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "attr:FilterBuilder.topOperatorForm":{
        ref:"attr:FilterBuilder.topOperatorForm",
        flags:"IR",
        description:"With ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')}:\"bracket\", form that appears to the left of the stack of\n clauses and allows picking the ${isc.DocUtils.linkForRef('type:LogicalOperator')} for the overall FilterBuilder.\n <P>\n By default, consists of a simple SelectItem.",
        name:"topOperatorForm",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:Operator.title":{
        ref:"attr:Operator.title",
        flags:"IR",
        description:"User-visible title for this operator, such as \"doesn't contain\".\n <P>\n To simplify internationalization by separating titles from operator code, you can use\n specify ${isc.DocUtils.linkForRef('attr:Operator.titleProperty')} instead of this property.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "attr:DataBoundComponent.preventDuplicates":{
        ref:"attr:DataBoundComponent.preventDuplicates",
        flags:"IR",
        description:"If set, detect and prevent duplicate records from being transferred to this component, either via\n drag and drop or via ${isc.DocUtils.linkForRef('method:DataBoundComponent.transferSelectedData')}.  When a duplicate transfer is detected,\n a dialog will appear showing the ${isc.DocUtils.linkForRef('attr:DataBoundComponent.duplicateDragMessage')}.\n <P>\n If the component either does not have a ${isc.DocUtils.linkForRef('class:DataSource')} or has a DataSource with no\n ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')} declared, duplicate checking is off by\n default.  If duplicate checking is enabled, it looks for an existing record in the dataset\n that has <b>all</b> of the properties of the dragged record, and considers that a duplicate.\n <P>\n For ${isc.DocUtils.linkForRef('type:DragDataAction')}:\"copy\" where the target DataSource is related to the source\n DataSource by foreignKey, a duplicate means that the target list, as filtered by the current\n criteria, already has a record whose value for the foreignKey field matches the\n primaryKey of the record being transferred.\n <P>\n For example, consider dragging \"employees\" to \"teams\", where \"teams\" has a field\n \"teams.employeeId\" which is a foreignKey pointing to \"employees.id\", and the target\n grid has search criteria causing it to show all the members of one team.  A duplicate -\n adding an employee to the same team twice - is when the target grid's dataset contains an\n record with \"employeeId\" matching the \"id\" field of the dropped employee.",
        name:"preventDuplicates",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:StretchImgButton.actionType":{
        flags:"IRW",
        getter:["getActionType"],
        type:"attr",
        ref:"attr:StretchImgButton.actionType",
        description:"Behavior on state changes -- BUTTON, RADIO or CHECKBOX",
        name:"actionType",
        setter:["setActionType"],
        valueType:"SelectionType",
        seeAlso:[],
        defaultValue:"\"button\"",
        groups:["state", "event handling"],
        definingClass:"class:StretchImgButton"
    },
    "attr:OperationBinding.dataURL":{
        ref:"attr:OperationBinding.dataURL",
        flags:"IR",
        description:"URL to contact to fulfill DSRequests for this operationBinding.\n <P>\n <code>dataURL</code> is typically set as DataSource.dataURL rather than on each individual\n operationBinding.\n <P>\n <code>dataURL</code> can be omitted for a DataSource using a Web Service\n (${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')} is set).",
        name:"dataURL",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "class:XMLTools":{
        methods:[],
        ref:"class:XMLTools",
        classAttrs:[],
        classMethods:["classMethod:XMLTools.loadXML", "classMethod:XMLTools.parseXML", 
         "classMethod:XMLTools.nativeXMLAvailable","classMethod:XMLTools.toJS", "classMethod:XMLTools.selectObjects", 
         "classMethod:XMLTools.selectNodes","classMethod:XMLTools.selectString", "classMethod:XMLTools.selectNumber", 
         "classMethod:XMLTools.transformNodes","classMethod:XMLTools.serializeToString", "classMethod:XMLTools.loadXMLSchema", 
         "classMethod:XMLTools.loadWSDL"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:[],
        description:"Utility methods for dealing with XML elements, XML Schema, WSDL files, XSLT, and other\n XML-related functionality.",
        name:"XMLTools",
        seeAlso:[],
        type:"class"
    },
    "classAttr:Validator.notABoolean":{
        ref:"classAttr:Validator.notABoolean",
        flags:"IRA",
        description:"Default error message to display when standard <code>isBoolean</code> type validator\n  returns false.",
        name:"notABoolean",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be a true/false value\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:Calendar.workdayEnd":{
        ref:"attr:Calendar.workdayEnd",
        flags:"IR",
        description:"When using ${isc.DocUtils.linkForRef('attr:Calendar.showWorkday')}:true, <code>workdayStart</code> and <code>workdayEnd</code>\n specify the time of day when the workday starts and ends, specified as a\n String acceptable to ${isc.DocUtils.linkForRef('classMethod:Time.parseInput')}.\n <P>\n Both start and end time must fall on a 30 minute increment (eg 9:30, but not 9:45).",
        name:"workdayEnd",
        seeAlso:[],
        valueType:"Time",
        defaultValue:"\"5:00pm\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday", "date"]
    },
    "attr:ValuesManager.dataSource":{
        ref:"attr:ValuesManager.dataSource",
        flags:"IRWA",
        getter:["getDataSource"],
        description:"Specifies a dataSource for this valuesManager.  This dataSource will then be used for\n validation and client-server flow methods.  Can be specified as a dataSource object or\n an identifier for the dataSource.<br>\n Note that member forms should have the same dataSource applied to them to allow their\n items to inherit properties from the DataSource fields.",
        name:"dataSource",
        setter:["setDataSource"],
        seeAlso:[
            {ref:"method:ValuesManager.setDataSource", description:null},
            {ref:"method:ValuesManager.getDataSource", description:null}
        ],
        valueType:"DataSource | string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ValuesManager"
    },
    "method:FormItem.transformInput":{
        ref:"method:FormItem.transformInput",
        flags:"",
        description:" Called when a FormItem's value is about to change as the result of user interaction.  This\n method fires after the user performed an action that would change the value of this field,\n and allows the developer to modify / reformat the value before it gets validated / saved.\n Fires before ${isc.DocUtils.linkForRef('method:FormItem.change')}.\n <P>\n Return the reformatted value.\n\n",
        name:"transformInput",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"The new value of the form item", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"The previous (current) value of the form item", optional:false,
             name:"oldValue", defaultValue:null, type:"any"}
        ],
        returns:{description:"The desired new value for the form item", type:"any"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:ColumnTree.deselectAllRecords":{
        ref:"method:ColumnTree.deselectAllRecords",
        flags:"",
        description:" Deselect all records in the supplied column (the first column if none is passed)\n\n",
        name:"deselectAllRecords",
        seeAlso:[],
        params:[
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "attr:ServerObject.visibleMethods":{
        ref:"attr:ServerObject.visibleMethods",
        flags:"IR",
        description:"When the ${isc.DocUtils.linkForRef('class:ServerObject')} appears in a .app.xml file (for RPC DMI), this property\n specifies the list of methods on the ServerObject that are callable from the client.  See\n the builtin.app.xml file in the /shared/app directory of the SDK for an example of a\n visibleMethods declaration block.",
        name:"visibleMethods",
        seeAlso:[
            {ref:"class:DMI", description:null}
        ],
        valueType:"List",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:ServerObject"
    },
    "attr:WSRequest.xmlNamespaces":{
        ref:"attr:WSRequest.xmlNamespaces",
        flags:"IR",
        description:"Optional object declaring namespace prefixes for use in evaluating the\n <code>resultType</code> parameter of ${isc.DocUtils.linkForRef('method:WebService.callOperation')}, if resultType is an\n XPath.\n <P>\n Format is identical to ${isc.DocUtils.linkForRef('attr:OperationBinding.xmlNamespaces')}, and default namespaces\n bindings are also identical.",
        name:"xmlNamespaces",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:WSRequest"
    },
    "method:Array.getValueMap":{
        ref:"method:Array.getValueMap",
        flags:"",
        description:" Get a map of the form <code>{ item[idField] -&gt; item[displayField] }</code>, for all \n items in the list.  Note that if more than one item has the same <code>idProperty</code>, \n the value for the later item in the list will clobber the value for the earlier item.\n\n",
        name:"getValueMap",
        seeAlso:[],
        params:[
            {description:"Property to use as ID (data value) in the valueMap", optional:false,
             name:"idField", defaultValue:null, type:"string"},
            {description:"Property to use a display value in the valueMap", optional:false,
             name:"displayField", defaultValue:null, type:"string"}
        ],
        returns:{description:"valueMap object", type:"object"},
        type:"method",
        definingClass:"object:Array"
    },
    "classMethod:JSON.decode":{
        ref:"classMethod:JSON.decode",
        flags:"",
        description:" De-serialize an object from JSON.  Currently, this is simply a JavaScript eval() and should\n be used for trusted data only.\n\n",
        name:"decode",
        seeAlso:[],
        params:[
            {description:"JSON data to be de-serialized", optional:false, name:"jsonString",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"object derived from JSON String", type:"String"},
        type:"classMethod",
        definingClass:"class:JSON"
    },
    "attr:HibernateBrowser.title":{
        ref:"attr:HibernateBrowser.title",
        flags:"IR",
        description:"A title to show in the header button of the HibernateBrowser's mapping tree.",
        name:"title",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Hibernate Mappings\"",
        type:"attr",
        definingClass:"class:HibernateBrowser"
    },
    "type:TitleOrientation":{
        ref:"type:TitleOrientation",
        values:[
            {description:null, value:"\"left\""},
            {description:null, value:"\"top\""},
            {description:null, value:"\"right\""}
        ],
        description:"Orientation of titles relative to the FormItem being labelled.  Can be set a the\n DynamicForm level as a default, or on individual items.",
        name:"TitleOrientation",
        seeAlso:[
            {ref:"attr:DynamicForm.titleOrientation", description:null},
            {ref:"attr:FormItem.titleOrientation", description:null}
        ],
        type:"type",
        groups:["formTitles"]
    },
    "method:ListGrid.cellMouseDown":{
        ref:"method:ListGrid.cellMouseDown",
        flags:"",
        description:" Called when a cell receives a mousedown event.\n \n",
        name:"cellMouseDown",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:HeaderSpan.fields":{
        ref:"attr:HeaderSpan.fields",
        flags:"IR",
        description:"List of fields that this header spans.  Fields should be identified by their value for\n ${isc.DocUtils.linkForRef('attr:ListGridField.name')}.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "attr:Canvas.hoverVAlign":{
        ref:"attr:Canvas.hoverVAlign",
        flags:"IRW",
        description:"If <code>this.showHover</code> is true, this property can be used to customize the\n vertical alignment of content in the hover canvas.",
        name:"hoverVAlign",
        seeAlso:[
            {ref:"attr:Canvas.showHover", description:null}
        ],
        valueType:"VerticalAlignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "method:GridRenderer.cellClick":{
        ref:"method:GridRenderer.cellClick",
        flags:"A",
        description:" Called when a cell receives a click event.\n\n",
        name:"cellClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "method:DynamicForm.clearValues":{
        ref:"method:DynamicForm.clearValues",
        flags:"",
        description:" Reset to default form values and clear errors\n",
        name:"clearValues",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ListGrid.singleCellValueProperty":{
        ref:"attr:ListGrid.singleCellValueProperty",
        flags:"IRW",
        description:"If <code>record[this.singleCellValueProperty]</code> is set for some record, the \n record will be displayed as a single cell spanning every column in the grid, with \n contents set to the value of <code>record[this.singleCellValueProperty]</code>.",
        name:"singleCellValueProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"singleCellValue\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:Button.getState":{
        ref:"method:Button.getState",
        flags:"",
        description:" Return the state of this StatefulCanvas\n",
        name:"getState",
        seeAlso:[],
        returns:{description:null, type:"State"},
        type:"method",
        definingClass:"class:Button",
        groups:["state"]
    },
    "class:CheckboxItem":{
        methods:["method:CheckboxItem.setValueMap"],
        exampleConfig:"CheckboxItem.xml",
        ref:"class:CheckboxItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:CheckboxItem.titleStyle", "attr:CheckboxItem.textBoxStyle", 
         "attr:CheckboxItem.showTitle","attr:CheckboxItem.labelAsTitle", "attr:CheckboxItem.showLabel", 
         "attr:CheckboxItem.checkedImage","attr:CheckboxItem.uncheckedImage", "attr:CheckboxItem.showUnsetImage", 
         "attr:CheckboxItem.unsetImage","attr:CheckboxItem.valueIconWidth", "attr:CheckboxItem.valueIconHeight", 
         "attr:CheckboxItem.valueMap","attr:CheckboxItem.allowEmptyValue"],
        description:"Checkbox form item, implemented with customizeable checkbox images",
        name:"CheckboxItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('checkboxItem')}"]
    },
    "method:ListGrid.getHeaderContextMenuItems":{
        ref:"method:ListGrid.getHeaderContextMenuItems",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu')} is <code>true</code> this method returns\n the menu items to be displayed in the default header context menu.\n <P>\n This method will be called each time the menu is displayed, allowing for dynamic content\n depending on the current state of the fields.\n <P>\n The default set of menu items will includes menu items for freezing fields, showing and\n hiding fields, grouping by fields, or other listGrid features \n\n",
        name:"getHeaderContextMenuItems",
        seeAlso:[],
        params:[
            {description:"Index of the field the user clicked in the    ${isc.DocUtils.linkForRef('attr:ListGrid.fields','fields')} array. <b>Note:</b> if the user right-clicked the sorter button\n    this parameter will be <code>null</code>.",
             optional:true, name:"fieldNum", defaultValue:null, type:"integer"}
        ],
        returns:{description:null, type:"Array of MenuItem"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridHeader"]
    },
    "attr:DataBoundComponent.hiliteProperty":{
        ref:"attr:DataBoundComponent.hiliteProperty",
        flags:"IRW",
        description:"Marker that can be set on a record to flag that record as hilited.  Should be set to a value\n that matches ${isc.DocUtils.linkForRef('attr:Hilite.id')} for a hilite defined on this component.",
        name:"hiliteProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"_hilite\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:ResultSet.getLength":{
        ref:"method:ResultSet.getLength",
        flags:"",
        description:" Return the total number of records that match the current filter criteria.\n <P>\n This length can only be known, even approximately, when the first results are retrieved from\n the server.  Before then, the ResultSet returns a large length in order to encourage viewers\n to ask for rows.  ${isc.DocUtils.linkForRef('method:ResultSet.lengthIsKnown','ResultSet.lengthIsKnown()')} can be called to\n determine whether an actual length is known.\n\n",
        name:"getLength",
        seeAlso:[],
        returns:{description:"number of items in the list", type:"Number"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access"]
    },
    "attr:SelectOtherItem.selectOtherPrompt":{
        ref:"attr:SelectOtherItem.selectOtherPrompt",
        flags:"IR",
        description:"Title to show in prompt for \"other\" value.\n Note this is a dynamic string. JavaScript content is supported within <code>\\${...}</code>\n tags, with local variables for <code>item</code> (a pointer to this item) and \n <code>value</code> a pointer to the currently selected item value.",
        name:"selectOtherPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Other value for <br>${item.getTitle()}?\"",
        type:"attr",
        definingClass:"class:SelectOtherItem",
        groups:["i18nMessages"]
    },
    "method:ListGrid.formatCellValue":{
        ref:"method:ListGrid.formatCellValue",
        flags:"",
        description:" Formatter to apply to values displayed within cells.\n <P>\n The value passed to this method is either the field value found in the cell record \n or, if there are unsaved edits, the current user-entered value for the cell.\n <b>NOTE:</b> unsaved user edits may contain nulls, bad values or values of the wrong\n type, so formatters used for editable data should be bulletproof.  For example, if you\n have a function \"myNumberFormatter\" that should only be passed actual Numbers, you might\n define formatCellValue like so:\n <pre>\n     isc.isA.Number(parseInt(value)) ? \n            myNumberFormatter(parseInt(value)) : value\n </pre>\n Note that this formatter will not be applied to the value displayed within editors for\n cells - use <code>formatEditorValue</code> to achieve this.\n <P>\n If <code>formatCellValue</code> is defined at the field level for some cell being edited,\n the field level method will be used to format the edit value and this method will not\n be called for that cell.<br>\n\n",
        name:"formatCellValue",
        seeAlso:[
            {ref:"method:ListGridField.formatCellValue", description:null},
            {ref:"method:ListGrid.formatEditorValue", description:null}
        ],
        params:[
            {description:"raw value for the cell being", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"Record object for the cell. Note: If this is a new row that has not been saved, in an \n   editable grid, it has no associated record object. In this case the edit values will\n   be passed in as this parameter.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"formatted value to display in the cell.", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["display_values"]
    },
    "attr:PickList.pickListHeight":{
        ref:"attr:PickList.pickListHeight",
        flags:"IRW",
        description:"Maximum height to show the pick list before it starts to scroll.\n Note that by default the pickList will be sized to the height required by its content\n so it will be taller when more rows are available as selectable options",
        name:"pickListHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"300",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:CubeGrid.bodyStyleName":{
        ref:"attr:CubeGrid.bodyStyleName",
        flags:"IRW",
        description:"CSS class for the CubeGrid body",
        name:"bodyStyleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"cubeGridBody\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "attr:SpinnerItem.maskSaveLiterals":{
        ref:"attr:SpinnerItem.maskSaveLiterals",
        flags:null,
        description:"Not applicable to a SpinnerItem.",
        name:"maskSaveLiterals",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:SpinnerItem"
    },
    "method:Canvas.getID":{
        ref:"method:Canvas.getID",
        flags:"",
        description:"      When a widget instance is created, it is assigned a unique global identifier that can be\n      used to access the instance by name. The getID method returns this ID for a particular\n      instance. Global IDs are essential when you need to embed a widget reference in a string,\n      usually a string that will be evaluated in the future and/or in another object, where you\n      may not have access to a variable or parameter holding the widget's reference.\n\n",
        name:"getID",
        seeAlso:[],
        returns:{description:"global identifier for this canvas", type:"string"},
        type:"method",
        definingClass:"class:Canvas"
    },
    "method:GridRenderer.getEventRow":{
        ref:"method:GridRenderer.getEventRow",
        flags:"",
        description:" Returns the row number of the most recent mouse event.\n",
        name:"getEventRow",
        seeAlso:[],
        params:[
            {description:"optional y-coordinate to obtain row number, in lieue of the y                        coordinate of the last mouse event",
             optional:true, name:"y", defaultValue:null, type:"number"}
        ],
        returns:{description:"row number, or -2 if beyond last drawn row", type:"number"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events", "selection"]
    },
    "attr:ListGrid.filterByCell":{
        ref:"attr:ListGrid.filterByCell",
        flags:"IRWA",
        description:"If we're showing the filterEditor (this.showFilterEditor is true), this property \n determines whether this list should be filtered every time the user puts focus in\n a different field in the filter editor.",
        name:"filterByCell",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "attr:FormItem.showTitle":{
        ref:"attr:FormItem.showTitle",
        flags:"IRW",
        description:"Should we show a title cell for this formItem?\n <p>\n Note: the default value of this attribute is overridden by some subclasses.",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "attr:RowSpacerItem.endRow":{
        ref:"attr:RowSpacerItem.endRow",
        flags:"IRW",
        description:"these items are in a row by themselves by default",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RowSpacerItem",
        groups:["appearance"]
    },
    "attr:ImgSplitbar.target":{
        ref:"attr:ImgSplitbar.target",
        flags:"R",
        description:"When a <code>Splitbar</code> is created by a layout, the <code>target</code> property\n of the Splitbar will be a pointer to the member for which it is acting as a resizeBar.\n The Splitbar will be positioned next to its target, and will resize it on drag completion.",
        name:"target",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "group:items":{
        ref:"group:items",
        title:"Form Items",
        refs:["method:DynamicForm.getItem", "method:DynamicForm.getField", "attr:DynamicForm.items", 
         "attr:DynamicForm.fields","attr:ToolbarItem.buttons"],
        description:"Manipulating the items that belong to a form.\n <BR><br>\n An item manages an atomic value (eg a String, Number, Date, etc) that appears as one of the\n properties in the overall form's values.  Some items exist purely for layout or appearance\n purposes (eg SpacerItem) and do not manage a value.",
        name:"items",
        seeAlso:[],
        type:"group"
    },
    "attr:ImgButton.src":{
        ref:"attr:ImgButton.src",
        flags:"IRW",
        description:"The base filename for the image.\n <P>\n If <code>img.state</code> is set, it's value will be appended to the URL before the\n file extension.  \n <P>\n For example, given an <code>src</code> of \"findIcon.gif\" and a state of\n \"Disabled\", the resulting image name would be \"findIcon_Disabled.gif\".  Compound states\n such as \"Selected\", \"Focused\" and \"Over\" or \"Down\" will have an intervening underscore,\n resulting in, for example, <code>\"findIcon_Selected_Down.gif\"</code>.",
        name:"src",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/ImgButton/button.png\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonAppearance')}"],
        definingClass:"class:ImgButton",
        groups:["appearance"]
    },
    "classAttr:Operators.iNotContainsTitle":{
        ref:"classAttr:Operators.iNotContainsTitle",
        flags:"IR",
        description:"Title for the \"iNotContains\" operator",
        name:"iNotContainsTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not contain\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classMethod:Page.getSkinImgDir":{
        ref:"classMethod:Page.getSkinImgDir",
        flags:"",
        description:"		Return the directory for a skin image.\n\n",
        name:"getSkinImgDir",
        seeAlso:[],
        params:[
            {description:"Partial URL (relative to Page._skinDir) where the image lives.										If not supplied, will use \"images/\"",
             optional:true, name:"imgDir", defaultValue:null, type:"URL"}
        ],
        returns:{description:"URL for page-specific images.", type:"string"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "method:ImgButton.getState":{
        ref:"method:ImgButton.getState",
        flags:"",
        description:" Return the state of this StatefulCanvas\n",
        name:"getState",
        seeAlso:[],
        returns:{description:null, type:"State"},
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "attr:FormulaBuilder.formulaField":{
        ref:"attr:FormulaBuilder.formulaField",
        flags:"IR",
        description:"TextItem that users type into when entering a formula.",
        name:"formulaField",
        seeAlso:[],
        valueType:"AutoChild TextItem",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:DetailViewer.dateFormatter":{
        ref:"attr:DetailViewer.dateFormatter",
        flags:"IR",
        description:"Display format to use for fields specified as type 'date'.  Default is to use the\n system-wide default normal date format, configured via\n ${isc.DocUtils.linkForRef('classMethod:Date.setNormalDisplayFormat')}.  Specify any valid ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} to\n change the format used by this detailViewer.",
        name:"dateFormatter",
        seeAlso:[],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DetailViewer"
    },
    "attr:Calendar.workdayBaseStyle":{
        ref:"attr:Calendar.workdayBaseStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:Calendar.showWorkday')} is set, style used for cells that are within the workday, as defined by\n ${isc.DocUtils.linkForRef('attr:Calendar.workdayStart')} and ${isc.DocUtils.linkForRef('attr:Calendar.workdayEnd')}.",
        name:"workdayBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calendarWorkday\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday", "appearance"]
    },
    "attr:TabSet.destroyPanes":{
        ref:"attr:TabSet.destroyPanes",
        flags:"IR",
        description:"Whether ${isc.DocUtils.linkForRef('method:Canvas.destroy','destroy()')} should be called on ${isc.DocUtils.linkForRef('attr:Tab.pane')} when it a tab is\n removed via ${isc.DocUtils.linkForRef('method:TabSet.removeTab')}}.  \n <P>\n An application might set this to false in order to re-use panes in different tabs or in\n different parts of the application.",
        name:"destroyPanes",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:Layout.layoutRightMargin":{
        ref:"attr:Layout.layoutRightMargin",
        flags:"IRW",
        description:"Space outside of all members, on the right-hand side.  Defaults to ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')}.\n <P>\n Requires a manual call to <code>setLayoutMargin()</code> if changed on the fly.",
        name:"layoutRightMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "method:SectionStack.getSections":{
        ref:"method:SectionStack.getSections",
        flags:"",
        description:"\n Returns a list of all sectionIDs in the order in which they appear in the SectionStack.\n\n",
        name:"getSections",
        seeAlso:[],
        returns:{description:"list of all sectionIDs in the order in which they appear in the SectionStack.",
         type:"List"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "class:DOMGrid":{
        methods:["method:DOMGrid.setRootElement"],
        ref:"class:DOMGrid",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:DOMGrid.rootElement"],
        description:"Provides a tree view of any DOM-compliant structure, such as an XML or HTML document.",
        name:"DOMGrid",
        seeAlso:[],
        type:"class"
    },
    "method:ListGrid.removeData":{
        ref:"method:ListGrid.removeData",
        flags:"",
        description:" Remove a record from this listGrids ${isc.DocUtils.linkForRef('attr:ListGrid.data','data object')}.\n <P>\n If this grid is bound to a dataSource, and ${isc.DocUtils.linkForRef('attr:ListGrid.saveLocally')} is not true,\n this method will perform a DataSource \"remove\" operation to remove records from this component's\n DataSource.\n <P>\n Otherwise the data will be removed from the grids data array. \n\n",
        name:"removeData",
        seeAlso:[],
        params:[
            {description:"listGrid record, or primary key values of record to delete.",
             optional:false, name:"data", defaultValue:null, type:"Object"},
            {description:"method to call on operation completion.    Note that if this is method does not trigger a dataSource remove operation, the callback\n   will still be fired when the data has been removed, but the <code>dsResponse</code> parameter\n   will be null.",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on any DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:CubeGrid.valueProperty":{
        ref:"attr:CubeGrid.valueProperty",
        flags:"IR",
        description:"Name of the property in a cell record that holds the cell value.",
        name:"valueProperty",
        seeAlso:[],
        valueType:"id",
        defaultValue:"\"_value\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:FilterBuilder.search":{
        ref:"method:FilterBuilder.search",
        flags:"",
        description:" A StringMethod that is automatically invoked if ${isc.DocUtils.linkForRef('attr:FilterBuilder.saveOnEnter')} is set \n and the user presses Enter whilst in a text-item in any clause or subclause.\n\n",
        name:"search",
        seeAlso:[],
        params:[
            {description:"The criteria represented by the filterBuilder", optional:false,
             name:"criteria", defaultValue:null, type:"AdvancedCriteria"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "method:ColumnTree.selectAllRecords":{
        ref:"method:ColumnTree.selectAllRecords",
        flags:"",
        description:" Select all records in the supplied column (the first column if none is passed)\n\n",
        name:"selectAllRecords",
        seeAlso:[],
        params:[
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "attr:SliderItem.defaultValue":{
        ref:"attr:SliderItem.defaultValue",
        flags:"IRW",
        description:"Default value for this sliderItems is 1.",
        name:"defaultValue",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:SliderItem"
    },
    "attr:Splitbar.canDrag":{
        ref:"attr:Splitbar.canDrag",
        flags:"IRW",
        description:"<code>canDrag</code> set to true to allow dragging of the split bar. Dragging the\n Splitbar will resize it's ${isc.DocUtils.linkForRef('attr:Splitbar.target','target')}",
        name:"canDrag",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Splitbar"
    },
    "attr:Slider.stepPercent":{
        ref:"attr:Slider.stepPercent",
        flags:"IRW",
        description:"The percentage of the total slider that constitutes one discrete step. The slider will move\n one step when the appropriate arrow key is pressed.",
        name:"stepPercent",
        seeAlso:[],
        valueType:"float",
        defaultValue:"5",
        type:"attr",
        definingClass:"class:Slider"
    },
    "method:ListGrid.getEventColumn":{
        ref:"method:ListGrid.getEventColumn",
        flags:"",
        description:" Returns the column number of the most recent mouse event.\n",
        name:"getEventColumn",
        seeAlso:[],
        params:[
            {description:"optional x-coordinate to obtain column number for, in lieue of the x                        coordinate of the last mouse event",
             optional:true, name:"x", defaultValue:null, type:"number"}
        ],
        returns:{description:"column number, or -2 if beyond last drawn column", type:"number"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:TreeGrid.loadDataOnDemand":{
        ref:"attr:TreeGrid.loadDataOnDemand",
        flags:"IRW",
        description:"For databound treeGrid instances, should the entire tree of data be loaded on initial \n fetch, or should folders load their children as they are opened.\n <P>\n If unset the default ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand')} setting will be used.\n <P>\n Note that when using <code>loadDataOnDemand</code>, every node returned by the server is\n assumed be a folder which may load further children.  See\n ${isc.DocUtils.linkForRef('attr:ResultTree.defaultIsFolder')} for how to control this behavior.",
        name:"loadDataOnDemand",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('initialData')}"],
        definingClass:"class:TreeGrid",
        groups:["databinding"]
    },
    "attr:ListGrid.originalData":{
        ref:"attr:ListGrid.originalData",
        flags:"R",
        description:"When grouped, a copy of the original ungrouped data.",
        name:"originalData",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"List of ListGridRecord",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "classMethod:isA.Number":{
        ref:"classMethod:isA.Number",
        flags:"",
        description:"\n	Is <code>object</code> a Number object?<br><br>\n\n	NOTE: this returns false if <code>object</code> is an invalid number (<code>isNaN(object) == true</code>)\n\n",
        name:"Number",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Number", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:Date.toDateStamp":{
        ref:"method:Date.toDateStamp",
        flags:"",
        description:"			Return this date in the format (UTC timezone):\n				<code><i>YYYYMMDD</i>T<i>HHMMSS</i>[Z]</code>\n",
        name:"toDateStamp",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:Calendar.previous":{
        ref:"method:Calendar.previous",
        flags:"",
        description:" Move to the previous day, week, or month, depending on which tab is selected.\n\n",
        name:"previous",
        seeAlso:[],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:TreeNode.children":{
        ref:"attr:TreeNode.children",
        flags:"IRW",
        description:"For trees with the modelType \"children\", this property specifies the children of this\n TreeNode.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.childrenProperty')}",
        name:"children",
        seeAlso:[
            {ref:"attr:Tree.modelType", description:null},
            {ref:"attr:Tree.childrenProperty", description:null}
        ],
        valueType:"List of TreeNode",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "method:Toolbar.removeButtons":{
        ref:"method:Toolbar.removeButtons",
        flags:"",
        description:"	Remove a list of buttons from the toolbar\n\n",
        name:"removeButtons",
        seeAlso:[],
        params:[
            {description:"Array of buttons to remove. Buttons may be specified as pointers to  the button instances contained in this toolbar, or numbers indicating the index of the buttons\n in <code>this.buttons</code>.",
             optional:true, name:"buttons", defaultValue:null, type:"Array"}
        ],
        type:"method",
        definingClass:"class:Toolbar"
    },
    "method:StretchImgButton.isSelected":{
        ref:"method:StretchImgButton.isSelected",
        flags:"",
        description:" Find out if this object is selected\n",
        name:"isSelected",
        seeAlso:[],
        returns:{description:null, type:"boolean"},
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "attr:Calendar.workdayStart":{
        ref:"attr:Calendar.workdayStart",
        flags:"IR",
        description:"When using ${isc.DocUtils.linkForRef('attr:Calendar.showWorkday')}:true, <code>workdayStart</code> and <code>workdayEnd</code>\n specify the time of day when the workday starts and ends, specified as a\n String acceptable to ${isc.DocUtils.linkForRef('classMethod:Time.parseInput')}.\n <P>\n Both start and end time must fall on a 30 minute increment (eg 9:30, but not 9:45).",
        name:"workdayStart",
        seeAlso:[],
        valueType:"Time",
        defaultValue:"\"9:00am\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["workday", "date"]
    },
    "attr:SelectItem.showOver":{
        ref:"attr:SelectItem.showOver",
        flags:"IRWA",
        description:"When the user rolls over the select item, should the pickButton display it's \n <code>Over</code> state?",
        name:"showOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "attr:TextAreaItem.selectOnFocus":{
        ref:"attr:TextAreaItem.selectOnFocus",
        flags:"IRW",
        description:"Allows the ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus','selectOnFocus')} behavior to be configured on a\n per-FormItem basis.  Normally all items in a form default to the value of\n ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus')}.",
        name:"selectOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["focus"]
    },
    "attr:ListGrid.editFailedBaseStyle":{
        ref:"attr:ListGrid.editFailedBaseStyle",
        flags:"IRWA",
        description:"A base name for the CSS class applied to cells when editing has failed.<br>\n  If this listGrid is editable, this style will be applied to any edited cells for which\n  validation failed.<br>\n  As with the default 'baseStyle' property, this style will have \"Dark\", \"Over\", \"Selected\", \n  or \"Disabled\" appended to it according to the state of the cell.<br>\n If null, cells for which editing has failed will be rendered using the normal base style\n classNames, but with custom CSSText applied as derived from <code>this.editFailedCSSText</code>",
        name:"editFailedBaseStyle",
        seeAlso:[
            {ref:"attr:ListGrid.baseStyle", description:null},
            {ref:"attr:ListGrid.editFailedCSSText", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "attr:Operator.hidden":{
        ref:"attr:Operator.hidden",
        flags:"IR",
        description:"Whether this operator should be offered to users by default in interfaces such as the\n ${isc.DocUtils.linkForRef('class:FilterBuilder')}.  \n <P>\n Setting hidden:true means the operator can be used in a programmatic search, for example,\n by calling ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria')}, but does not appear in the UI.",
        name:"hidden",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "method:Canvas.showContextMenu":{
        ref:"method:Canvas.showContextMenu",
        flags:"A",
        description:" Executed when the right mouse button is clicked.  The default implementation of\n this method auto-creates a ${isc.DocUtils.linkForRef('class:Menu')} from the ${isc.DocUtils.linkForRef('attr:Canvas.contextMenu')} property on\n this component and then calls ${isc.DocUtils.linkForRef('method:Menu.showContextMenu')} on it to show it.\n <p>\n If you want to show a standard context menu, you can simply define your Menu and set it as the\n contextMenu property on your component - you do not need to override this method.\n <p>\n If you want to do some other processing before showing a menu or do something else entirely, then\n you should override this method.  Note that the return value from this method controls whether or\n not the native browser context menu is shown.\n \n",
        name:"showContextMenu",
        seeAlso:[
            {ref:"attr:Canvas.contextMenu", description:null},
            {ref:"method:Menu.showContextMenu", description:null},
            {ref:"method:Canvas.hideContextMenu", description:null},
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false == don't show native context menu, true == show native context menu",
         type:"boolean"},
        type:"method",
        platformNotes:"On the Mac platform, context menu functionality may be triggered by <code>Command+click</code><br>\n On the Opera browser, context menu functionality may be triggered by <code>Shift+Ctrl+click</code>",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:RPCRequest.httpMethod":{
        ref:"attr:RPCRequest.httpMethod",
        flags:"IRW",
        description:"Selects the HTTP method that will be used for the request.  Typical values are \"POST\" and\n \"GET\".\n <P>\n The more obscure \"PUT\", \"DELETE\" and \"HEAD\" methods are also valid, however, none of these\n are supported by the Safari browser previous to version 3.0.",
        name:"httpMethod",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"POST\"",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:DataSourceField.valueMap":{
        ref:"attr:DataSourceField.valueMap",
        flags:"IR",
        description:"A ${isc.DocUtils.linkForRef('type:ValueMap')} is a set of legal values for a field.\n <p>\n The valueMap can be specified as either an Array of legal values, or as an\n ${isc.DocUtils.linkForRef('type:Object')} where each property maps a stored value to a user-displayable value.\n <p>\n To enforce that a field should be constrained to only the values in the valueMap, either\n declare ${isc.DocUtils.linkForRef('attr:DataSourceField.type','field.type')} as \"enum\", or use an\n ${isc.DocUtils.linkForRef('classAttr:ValidatorTypes.isOneOf','isOneOf validator')} with explicitly listed values.  Otherwise,\n although a normal ${isc.DocUtils.linkForRef('class:SelectItem')} control will only allow values from the valueMap to be\n entered, other controls such as a ${isc.DocUtils.linkForRef('class:ComboBoxItem','ComboBox')} will allow other values \n to be entered.\n <p>\n In XML, a valueMap that specifies only a list of legal values is specified as follows:\n <pre>\n   &lt;valueMap&gt;\n   	&lt;value&gt;Pens &amp; Pencils&lt;/value&gt;\n   	&lt;value&gt;Stationery&lt;/value&gt;\n   	&lt;value&gt;Computer Products&lt;/value&gt;\n   	&lt;value&gt;Furniture&lt;/value&gt;\n   	&lt;value&gt;Misc&lt;/value&gt;\n   &lt;/valueMap&gt;\n </pre>\n A ValueMap that specifies stored values mapped to user-visible values is specified as\n follows:\n <pre>\n   &lt;valueMap&gt;\n   	&lt;value ID=\"1\"&gt;Pens &amp; Pencils&lt;/value&gt;\n   	&lt;value ID=\"2\"&gt;Stationery&lt;/value&gt;\n   	&lt;value ID=\"3\"&gt;Computer Products&lt;/value&gt;\n   	&lt;value ID=\"4\"&gt;Furniture&lt;/value&gt;\n   	&lt;value ID=\"5\"&gt;Misc&lt;/value&gt;\n   &lt;/valueMap&gt;\n </pre>",
        name:"valueMap",
        seeAlso:[],
        valueType:"ValueMap",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["dataType"]
    },
    "classMethod:isc.showPrompt":{
        ref:"classMethod:isc.showPrompt",
        flags:"",
        description:"\n	Method available on the isc object to show a modal prompt to the user.\n  This method will display the message using the Dialog.Prompt singleton object.<br>\n  Note: if this prompt is to be shown to the user during some slow JavaScript logic, we \n  advise calling this method, then using ${isc.DocUtils.linkForRef('method:Class.delayCall')} or ${isc.DocUtils.linkForRef('classMethod:Timer.setTimeout')}\n  to kick off the slow logic in a separate thread. This ensures that the prompt is showing\n  before the lengthy execution begins.\n  \n\n",
        name:"showPrompt",
        seeAlso:[
            {ref:"classAttr:Dialog.Prompt", description:null}
        ],
        params:[
            {description:"message to display", optional:false, name:"message", defaultValue:null,
             type:"string"},
            {description:"additional properties for the Dialog, applied before                                       the Dialog is shown",
             optional:true, name:"properties", defaultValue:null, type:"Dialog Properties"}
        ],
        type:"classMethod",
        definingClass:"class:isc",
        groups:["Prompting"]
    },
    "attr:DateChooser.disableWeekends":{
        ref:"attr:DateChooser.disableWeekends",
        flags:"IR",
        description:"Whether it should be valid to pick a weekend day.  If set to true, weekend days appear\n in disabled style and cannot be picked. \n <P>\n Which days are considered weekends is controlled by ${isc.DocUtils.linkForRef('classAttr:Date.weekendDays')}.",
        name:"disableWeekends",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:CheckboxItem.checkedImage":{
        ref:"attr:CheckboxItem.checkedImage",
        flags:"IRW",
        description:"URL for the image to display when this checkbox is selected, or checked.\n This is the base image name - if <code>showValueIconOver</code> et al are set, the\n state (<code>\"Over\"</code>, <code>\"Down\"</code> and <code>\"Disabled\"</code>) will be\n added to this name as the user intereacts with the checkbox, as well as the \n <code>\".gif\"</code> extension",
        name:"checkedImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/DynamicForm/checked.gif\"",
        type:"attr",
        definingClass:"class:CheckboxItem",
        groups:["appearance"]
    },
    "attr:Calendar.startDateField":{
        ref:"attr:Calendar.startDateField",
        flags:"IR",
        description:"The name of the start date field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"startDateField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"startDate\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "attr:Operator.editorType":{
        ref:"attr:Operator.editorType",
        flags:"IR",
        description:"For an operator with ${isc.DocUtils.linkForRef('attr:Operator.valueType')}:\"custom\", indicates what kind of FormItem to use to\n provide a user interface for creating a valid ${isc.DocUtils.linkForRef('object:Criterion')}.  The default of\n <code>null</code> means an ordinary TextItem is fine.",
        name:"editorType",
        seeAlso:[],
        valueType:"FormItem classname",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "type:ExportFormat":{
        ref:"type:ExportFormat",
        values:[
            {description:"Export data as XML records", value:"\"xml\""},
            {description:"Export data as JSON objects", value:"\"json\""},
            {description:"Export data in comma-seperated format", value:"\"csv\""}
        ],
        description:"One of the supported formats for data-export.",
        name:"ExportFormat",
        seeAlso:[],
        type:"type",
        serverDS:"allowed"
    },
    "attr:TreeGrid.canReorderRecords":{
        ref:"attr:TreeGrid.canReorderRecords",
        flags:"IRWA",
        description:"Indicates whether records can be reordered by dragging within this listGrid.",
        name:"canReorderRecords",
        seeAlso:[
            {ref:"attr:TreeNode.canDrag", description:null},
            {ref:"attr:TreeNode.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragReparent')}"],
        definingClass:"class:TreeGrid",
        groups:["dragdrop"]
    },
    "attr:ColorItem.pickerIconHeight":{
        ref:"attr:ColorItem.pickerIconHeight",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"18",
        type:"attr",
        definingClass:"class:ColorItem"
    },
    "group:auth":{
        ref:"group:auth",
        refs:["attr:DataSource.requiresAuthentication", "attr:DataSource.requiresRole", 
         "attr:DataSource.requires","attr:OperationBinding.requiresAuthentication", "attr:OperationBinding.requiresRole", 
         "attr:OperationBinding.requires"],
        name:"auth",
        seeAlso:[],
        type:"group"
    },
    "group:menuIcons":{
        ref:"group:menuIcons",
        refs:["attr:MenuItem.checked", "attr:MenuItem.icon", "attr:MenuItem.iconWidth", 
         "attr:MenuItem.iconHeight"],
        name:"menuIcons",
        seeAlso:[],
        type:"group"
    },
    "method:Button.setSelected":{
        ref:"method:Button.setSelected",
        flags:"",
        description:" Select this object.\n",
        name:"setSelected",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["state"]
    },
    "method:Tree.closeFolder":{
        ref:"method:Tree.closeFolder",
        flags:"",
        description:"\n Closes a folder\n\n",
        name:"closeFolder",
        seeAlso:[],
        params:[
            {description:"folder to close", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:Window.headerControls":{
        ref:"attr:Window.headerControls",
        flags:"IR",
        description:"Array of members to show in the Window header.  \n <P>\n The default value of <code>headerControls</code> is an Array of Strings listing the\n standard header controls in their default order:\n <pre>\n    headerControls : [\"headerIcon\", \"headerLabel\", \n                      \"minimizeButton\", \"maximizeButton\", \"closeButton\"]\n </pre>\n You can override <code>headerControls</code> to change the order of standard controls in\n the header.  You can also omit standard controls this way, although it more efficient to\n use the related \"show\" property if available (eg ${isc.DocUtils.linkForRef('attr:Window.showMinimizeButton')}).  \n <P>\n By embedding a Canvas directly in this list you can add arbitrary additional controls to\n the header, for example, an additional button (eg return to dock) or a DynamicForm with\n various kinds of input controls.  \n <P>\n Note that having added controls to headerControls, you can still call APIs directly on\n those controls to change their appearance, and you can also show() and hide() them if\n they should not be shown in some circumstances.\n <P>\n Tip: custom controls need to set layoutAlign:\"center\" to appear vertically centered.",
        name:"headerControls",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"(see below)",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('windowHeaderControls')}"],
        definingClass:"class:Window"
    },
    "attr:HTMLFlow.evalScriptBlocks":{
        ref:"attr:HTMLFlow.evalScriptBlocks",
        flags:"IR",
        description:"If <code>evalScriptBlocks</code> is true, HTMLFlow will pre-process the loaded HTML in order to\n mimic how the HTML would execute if it were loaded as an independent page or loaded via an\n IFRAME.  \n <P>\n This feature is intended to assist with migrating existing applications to SmartClient.\n <P>\n Note that, if evalScriptBlocks is false, &lt;SCRIPT&gt; blocks will still be detected and disabled\n to avoid the inconsistent results across different browsers.\n <P>\n Only applies when contentsType is <b>not</b> \"page\".",
        name:"evalScriptBlocks",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:ListGrid.canExpandRecords":{
        ref:"attr:ListGrid.canExpandRecords",
        flags:"IRWA",
        description:"When set to true, shows an additional field at the beginning of the field-list \n (respecting RTL) to allow users to expand and collapse individual records.",
        name:"canExpandRecords",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "class:RichTextItem":{
        methods:[],
        ref:"class:RichTextItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:RichTextItem.showTitle", "attr:RichTextItem.startRow", "attr:RichTextItem.endRow", 
         "attr:RichTextItem.colSpan","attr:RichTextItem.controlGroups"],
        description:"FormItem for rich text (HTML) editing. Makes use of a ${isc.DocUtils.linkForRef('class:RichTextEditor')} as the \n editing interface.",
        name:"RichTextItem",
        seeAlso:[],
        type:"class"
    },
    "attr:RowSpacerItem.colSpan":{
        ref:"attr:RowSpacerItem.colSpan",
        flags:"IRW",
        description:"by default, separators span all remaining columns",
        name:"colSpan",
        seeAlso:[],
        valueType:"number",
        defaultValue:"\"*\"",
        type:"attr",
        definingClass:"class:RowSpacerItem",
        groups:["appearance"]
    },
    "classMethod:Array.compareAscending":{
        ref:"classMethod:Array.compareAscending",
        flags:"A",
        description:" Compare two values for an ascending order sort, using locale-sensitive comparison.\n",
        name:"compareAscending",
        seeAlso:[],
        params:[
            {description:"first value to compare", optional:false, name:"a", defaultValue:null,
             type:"any"},
            {description:"second value to compare", optional:false, name:"b", defaultValue:null,
             type:"any"}
        ],
        returns:{description:"negative == second is larger, 0 == same value, positive == first is larger",
         type:"number"},
        type:"classMethod",
        definingClass:"object:Array",
        groups:["sorting"]
    },
    "method:ImgButton.setActionType":{
        ref:"method:ImgButton.setActionType",
        flags:"",
        description:" Update the 'actionType' for this canvas (radio / checkbox / button)\n If the canvas is currently selected, and the passed in actionType is 'button'\n this method will deselect the canvas.\n",
        name:"setActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state", "event handling"]
    },
    "method:Canvas.isVisible":{
        ref:"method:Canvas.isVisible",
        flags:"",
        description:" Returns true if the widget is visible, taking all parents into account, so that a widget\n which is not hidden might still report itself as not visible if it is within a hidden\n parent.\n <P>\n NOTE: Undrawn widgets will report themselves as visible if they would be visible if drawn.\n\n",
        name:"isVisible",
        seeAlso:[],
        returns:{description:"true if the widget is visible, false otherwise", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["visibility"]
    },
    "method:DataBoundComponent.setDataSource":{
        ref:"method:DataBoundComponent.setDataSource",
        flags:"",
        description:" Bind to a new DataSource.\n <P>\n Like passing the \"dataSource\" property on creation, binding to a DataSource means that the\n component will use the DataSource to provide default data for its fields.\n <P>\n When binding to a new DataSource, if the component has any existing \"fields\" or has a dataset,\n these will be discarded by default, since it is assumed the new DataSource may represent a\n completely unrelated set of objects.  If the old \"fields\" are still relevant, pass them to\n setDataSource().\n \n",
        name:"setDataSource",
        seeAlso:[],
        params:[
            {description:"DataSource to bind to", optional:false, name:"dataSource", defaultValue:null,
             type:"ID or DataSource"},
            {description:"optional array of fields to use", optional:false, name:"fields",
             defaultValue:null, type:"Array of Fields"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('WSDLDataSource')}"],
        definingClass:"interface:DataBoundComponent"
    },
    "attr:DSRequest.startRow":{
        ref:"attr:DSRequest.startRow",
        flags:"IR",
        description:"Starting row of requested results, used only with fetch operations.  If unset, 0 is assumed.\n <p>\n Note that startRow and endRow are zero-based, so startRow: 0, endRow: 1 is a request\n for the first two records.",
        name:"startRow",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest",
        groups:["paging"]
    },
    "attr:Canvas.resizeFrom":{
        ref:"attr:Canvas.resizeFrom",
        flags:"IRWA",
        description:"Allows resizing in certain edges or corners. The default value of null indicates that\n      the widget is resizable from any corner or edge. To restrict resizing to only certain\n      corners, set resizeFrom to an array of any of the values listed:<br>\n      T      top edge<br>\n      B      bottom edge<br>\n      L      left edge<br>\n      R      right edge<br>\n      TL     top-left corner<br>\n      TR     top-right corner<br>\n      BL     bottom-left corner<br>\n      BR     bottom-right corner<br>\n	    E.g. setting this property to a value of [\"R\",\"TR\",\"BR\"] would restrict resizing to\n      the right edge, top-right corner and bottom-right corner only",
        name:"resizeFrom",
        seeAlso:[],
        valueType:"array",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragResize')}"],
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "method:Array.dataChanged":{
        ref:"method:Array.dataChanged",
        flags:"A",
        description:" Method called when this array changes in some way.  Observe the method to react to changes in\n this list.\n <P>\n Note: dataChanged() will only fire when items are added, removed or rearranged.  If a list\n contains objects, dataChanged() will not fire if changes are made to objects within the list\n without changing their position within the list.  If an observer of dataChanged() needs to react\n to such a change, you can manually fire dataChanged() by simply calling it.\n <P>\n Note: may be called multiple times as the result of a multi-item add or remove, etc.\n\n",
        name:"dataChanged",
        seeAlso:[],
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "type:Orientation":{
        ref:"type:Orientation",
        values:[
            {description:"members laid out vertically", value:"Layout.VERTICAL"},
            {description:"members laid out horizontally", value:"Layout.HORIZONTAL"}
        ],
        description:null,
        name:"Orientation",
        seeAlso:[],
        type:"type",
        groups:["orientation"]
    },
    "method:GridRenderer.getNearestRowToEvent":{
        ref:"method:GridRenderer.getNearestRowToEvent",
        flags:"",
        description:"			Returns the nearest row to the event coordinates\n",
        name:"getNearestRowToEvent",
        seeAlso:[],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events", "selection"]
    },
    "method:SectionStack.removeSection":{
        ref:"method:SectionStack.removeSection",
        flags:"",
        description:"\n Remove a section or set of sections from the SectionStack.  The removed sections' header\n and items (if any) are automatically destroyed.\n\n",
        name:"removeSection",
        seeAlso:[],
        params:[
            {description:"Section(s) to remove.  For this                  parameter, you can pass the position of the section in the\n                  SectionStack, the ID of the section, or a List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('sectionsAddAndRemove')}"],
        definingClass:"class:SectionStack"
    },
    "attr:ResultSet.dataSource":{
        ref:"attr:ResultSet.dataSource",
        flags:"IR",
        description:"What ${isc.DocUtils.linkForRef('class:DataSource')} is this resultSet associated with?",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet"
    },
    "attr:Canvas.destroying":{
        ref:"attr:Canvas.destroying",
        flags:"RA",
        description:"This property is set to true when the ${isc.DocUtils.linkForRef('method:Canvas.destroy')} method is called on a widget.\n If this property is true, but ${isc.DocUtils.linkForRef('attr:Canvas.destroyed')} is not, this indicates the canvas is\n in the process of being destroyed.",
        name:"destroying",
        seeAlso:[
            {ref:"method:Canvas.destroy", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "method:CellSelection.selectCell":{
        ref:"method:CellSelection.selectCell",
        flags:"",
        description:"			Select a particular cell\n",
        name:"selectCell",
        seeAlso:[],
        params:[
            {description:"row index of cell to select", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of cell to select", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection"]
    },
    "method:TileGrid.addData":{
        ref:"method:TileGrid.addData",
        flags:"",
        description:" Perform a DataSource \"add\" operation to add new records to this component's DataSource.\n\n",
        name:"addData",
        seeAlso:[],
        params:[
            {description:"new record", optional:false, name:"newRecord", defaultValue:null,
             type:"Object"},
            {description:"method to call on operation completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                          that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundAdd')}"],
        definingClass:"class:TileGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:ListGridField.formatGridSummary":{
        ref:"attr:ListGridField.formatGridSummary",
        flags:"IR",
        description:"Optional stringMethod to format the summary value displayed\n in the ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary','grid summary')}.\n Takes a single parameter <code>value</code> and should return the formatted version\n of that value\n <P>\n Note that for fields with a specified summary function of \"count\", if this method is \n undefined we default to formatting the count value by appending \n <code>field.pluralTitle</code> if defined, otherwise <code>field.title</code> to the\n numeric count value returned by the standard count function. To change this behavior for\n such fields, specify an explicit 'formatGridSummary' and/or 'formatGroupSummary' method",
        name:"formatGridSummary",
        seeAlso:[],
        valueType:"stringMethod",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "group:communication":{
        ref:"group:communication",
        refs:["type:SendMethod"],
        name:"communication",
        seeAlso:[],
        type:"group"
    },
    "group:formulaFields":{
        ref:"group:formulaFields",
        refs:["method:FormulaBuilder.setFormula", "method:FormulaBuilder.getHelpText", 
         "method:FormulaBuilder.getUpdatedFieldObject","method:FormulaBuilder.testFunction", "method:FormulaBuilder.getTestRecord", 
         "method:FormulaBuilder.save","method:FormulaBuilder.fireOnClose", "method:SummaryBuilder.setSummary", 
         "method:SummaryBuilder.save","method:DataBoundComponent.addFormulaField", "method:DataBoundComponent.editFormulaField", 
         "class:FormulaBuilder","class:MathFunction", "object:TestFunctionResult", 
         "classMethod:MathFunction.registerFunction","classMethod:MathFunction.getRegisteredFunctionIndex", 
         "classMethod:MathFunction.getDefaultFunctionIndex","attr:ListGrid.canAddFormulaFields", "attr:FormulaBuilder.dataSource", 
         "attr:FormulaBuilder.fields","attr:FormulaBuilder.field", "attr:FormulaBuilder.editMode", 
         "attr:FormulaBuilder.formulaField","attr:FormulaBuilder.titleField", "attr:FormulaBuilder.showHelpIcon", 
         "attr:FormulaBuilder.helpIcon","attr:FormulaBuilder.showAutoHideCheckBox", "attr:FormulaBuilder.autoHideCheckBox", 
         "attr:FormulaBuilder.builderTypeText","attr:FormulaBuilder.mathFunctions", "attr:FormulaBuilder.cancelled", 
         "attr:FormulaBuilder.fieldKey","attr:FormulaBuilder.cancelButton", "attr:FormulaBuilder.testButton", 
         "attr:FormulaBuilder.saveButton","attr:FormulaBuilder.autoTest", "attr:FormulaBuilder.autoTestDelay", 
         "attr:FormulaBuilder.testRecord","attr:SummaryBuilder.testRecord", "attr:DataBoundComponent.canAddFormulaFields", 
         "attr:MathFunction.name","attr:MathFunction.description", "attr:MathFunction.jsFunction", 
         "attr:MathFunction.defaultSortPosition","attr:TestFunctionResult.failedGeneration", "attr:TestFunctionResult.failedExecution", 
         "attr:TestFunctionResult.emptyTestValue","attr:TestFunctionResult.errorText", "attr:TestFunctionResult.result", 
         "attr:TestFunctionResult.record"],
        description:"Fields with values calculated from other fields in the grid.",
        name:"formulaFields",
        seeAlso:[],
        type:"group"
    },
    "attr:FormItem.errorCellClassName":{
        ref:"attr:FormItem.errorCellClassName",
        deprecated:"As of SmartClient version 5.5 deprecated in favor of ${isc.DocUtils.linkForRef('attr:FormItem.cellStyle')}",
        flags:"IRW",
        description:"CSS class for a form item's cell when a validation error is showing.",
        name:"errorCellClassName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"formError\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:DataSource.supportsTextMatchStyle":{
        ref:"method:DataSource.supportsTextMatchStyle",
        flags:"A",
        description:" Does this dataSource support the specified \"textMatchStyle\" when performing a filter\n operation against a text field.\n",
        name:"supportsTextMatchStyle",
        seeAlso:[],
        params:[
            {description:"textMatchStyle to check. If passed a null value,      assume an exact match is being requested.",
             optional:false, name:"textMatchStyle", defaultValue:null, type:"TextMatchStyle"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:PickTreeItem.dataSource":{
        ref:"attr:PickTreeItem.dataSource",
        flags:"IRA",
        description:"If specified, the tree of possible options will be derived from the dataSource as a\n ResultTree, rather than using this.valueTree.  Options can be loaded on demand or up\n front according tp ${isc.DocUtils.linkForRef('attr:PickTreeItem.loadDataOnDemand')}.",
        name:"dataSource",
        seeAlso:[],
        valueType:"datasource",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:PickTreeItem"
    },
    "attr:StatefulCanvas.showDown":{
        ref:"attr:StatefulCanvas.showDown",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes down in this object?",
        name:"showDown",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "class:DetailViewer":{
        exampleConfig:"DetailViewer.xml",
        methods:["method:DetailViewer.viewSelectedData", "method:DetailViewer.setData", 
         "method:DetailViewer.getCellCSSText","method:DetailViewer.getCellStyle", "method:DetailViewer.formatCellValue", 
         "method:DetailViewer.getRecordIndex","method:DetailViewer.exportData"],
        type:"class",
        ref:"class:DetailViewer",
        classAttrs:[],
        treeLocation:["Client Reference/Grids"],
        classMethods:[],
        description:"Displays one or more records \"horizontally\" with one property per line.",
        attrs:["attr:DetailViewer.data", "attr:DetailViewer.dataFetchMode", "attr:DetailViewer.fields", 
         "attr:DetailViewer.fieldIdProperty","attr:DetailViewer.recordsPerBlock", "attr:DetailViewer.blockSeparator", 
         "attr:DetailViewer.showEmptyField","attr:DetailViewer.emptyCellValue", "attr:DetailViewer.labelPrefix", 
         "attr:DetailViewer.labelSuffix","attr:DetailViewer.wrapLabel", "attr:DetailViewer.wrapValues", 
         "attr:DetailViewer.styleName","attr:DetailViewer.blockStyle", "attr:DetailViewer.labelStyle", 
         "attr:DetailViewer.cellStyle","attr:DetailViewer.headerStyle", "attr:DetailViewer.separatorStyle", 
         "attr:DetailViewer.dateFormatter","attr:DetailViewer.datetimeFormatter", "attr:DetailViewer.showEmptyMessage", 
         "attr:DetailViewer.emptyMessage","attr:DetailViewer.emptyMessageStyle", "attr:DetailViewer.loadingMessage", 
         "attr:DetailViewer.loadingMessageStyle"],
        implementsInterfaces:["DataBoundComponent"],
        name:"DetailViewer",
        seeAlso:[]
    },
    "attr:ListGridField.editorIconHeight":{
        ref:"attr:ListGridField.editorIconHeight",
        flags:"IRA",
        description:"If this field is editable, this property will be passed to editors displayed for\n cells within this field as ${isc.DocUtils.linkForRef('attr:FormItem.iconHeight')}.<br>\n If this property unset, the iconHeight property from the editor can be picked up from \n ${isc.DocUtils.linkForRef('attr:ListGridField.iconHeight')} instead.",
        name:"editorIconHeight",
        seeAlso:[
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:GridRenderer.snapToCells":{
        ref:"attr:GridRenderer.snapToCells",
        flags:"IRW",
        description:"Should drag-and-drop operations snap the dragged object into line with the nearest cell?",
        name:"snapToCells",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["dragdrop"]
    },
    "attr:ListGrid.emptyCellValue":{
        ref:"attr:ListGrid.emptyCellValue",
        flags:"IRW",
        description:"The value to display for cells whose value is null or the empty string after applying\n formatCellValue and valueMap (if any).\n <p>\n This is the grid-wide attribute.  You may also set the emptyCellValue on a per-field basis.",
        name:"emptyCellValue",
        seeAlso:[
            {ref:"attr:ListGridField.emptyCellValue", description:null}
        ],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('emptyValues')}"],
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "method:DynamicForm.resetValues":{
        ref:"method:DynamicForm.resetValues",
        flags:"",
        description:"\n Same as ${isc.DocUtils.linkForRef('method:DynamicForm.reset')}.\n\n",
        name:"resetValues",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:FormItem.accessKey":{
        ref:"attr:FormItem.accessKey",
        flags:"IRW",
        description:"AccessKey - a keyboard shortcut to trigger a form item's default behavior.<br>\n      Note: Alt+ (or in Moz Firefox 2.0 and above, Shift+Alt+) the specified key will \n      trigger the form item's default behavior.",
        name:"accessKey",
        seeAlso:[],
        valueType:"keyChar",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["focus"]
    },
    "attr:ListGridRecord.isSeparator":{
        ref:"attr:ListGridRecord.isSeparator",
        flags:"IR",
        description:"Default property name denoting a separator row.<br>\n When set to <code>true</code>, defines a horizontal separator in the listGrid\n object. Typically this is specified as the only property of a record object, since a\n record with <code>isSeparator:true</code> will not display any values.<br>\n Note: this attribute name is governed by ${isc.DocUtils.linkForRef('attr:ListGrid.isSeparatorProperty')}.",
        name:"isSeparator",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridRecord"
    },
    "attr:SelectItem.defaultValue":{
        ref:"attr:SelectItem.defaultValue",
        flags:"IRW",
        description:"Static default value for this SelectItem. To default to the first option use\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption')} instead.",
        name:"defaultValue",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "method:Date.toShortDate":{
        ref:"method:Date.toShortDate",
        flags:"",
        description:" Returns the date as a formatted string using the format set up via the \n <code>setShortDisplayFormat()</code> method.\n",
        name:"toShortDate",
        seeAlso:[],
        params:[
            {description:"Optional Format for the date returned", optional:false, name:"format",
             defaultValue:null, type:"DateDisplayFormat"},
            {description:"If true, format the date using the timezone set via  Time.setDefaultDisplayTimezone() rather than the native browser locale.",
             optional:false, name:"useCustomTimezone", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:Menu.hideContextMenu":{
        ref:"method:Menu.hideContextMenu",
        flags:"",
        description:" Hide the context menu - alias for hide()\n",
        name:"hideContextMenu",
        seeAlso:[],
        type:"method",
        definingClass:"class:Menu",
        groups:["visibility"]
    },
    "classAttr:Page.unsupportedBrowserAction":{
        ref:"classAttr:Page.unsupportedBrowserAction",
        flags:"IRA",
        description:"Action to take when ${isc.DocUtils.linkForRef('classMethod:Page.checkBrowserAndRedirect')} is called in a browser for which\n support is not guaranteed. Valid settings are:\n <ul>\n <li><code>\"continue\"</code> Load the page without notifying the user of potential issues</li>\n <li><code>\"confirm\"</code> Notify the user via a standard confirm dialog that their browser is\n  not supported. Provide options to continue anyway, or redirect to another page.\n <li><code>\"redirect\"</code> Automatically redirect to the another URL\n </ul>",
        name:"unsupportedBrowserAction",
        seeAlso:[
            {ref:"classMethod:Page.checkBrowserAndRedirect", description:null},
            {ref:"classAttr:Page.defaultUnsupportedBrowserURL", description:null}
        ],
        valueType:"string",
        defaultValue:"\"continue\"",
        type:"classAttr",
        definingClass:"class:Page"
    },
    "classAttr:Operators.isNullTitle":{
        ref:"classAttr:Operators.isNullTitle",
        flags:"IR",
        description:"Title for the \"isNull\" operator",
        name:"isNullTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"is null\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "method:Menu.setItemEnabled":{
        ref:"method:Menu.setItemEnabled",
        flags:"",
        description:" Enables or disables the menu item according to the value of newState, and redraws\n the menu if necessary. Returns true if there's a change in the enabled state.\n\n",
        name:"setItemEnabled",
        seeAlso:[],
        params:[
            {description:"MenuItem in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem or number"},
            {description:"true to enable the menu item, false to disable it.  If not                             passed, true is assumed",
             optional:true, name:"newState", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if the enabled state was changed", type:"boolean"},
        type:"method",
        definingClass:"class:Menu"
    },
    "method:Canvas.draw":{
        ref:"method:Canvas.draw",
        flags:"",
        description:"      Draws the widget on the page.\n",
        name:"draw",
        seeAlso:[],
        returns:{description:"Pointer to this canvas.  Returned so statements like the following will						work:<br>\n                          var myCanvas = Canvas.newInstance({...}).draw();",
         type:"canvas"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "method:Canvas.getBottom":{
        ref:"method:Canvas.getBottom",
        flags:"",
        description:" Return the bottom coordinate of this object as rendered, relative to its enclosing context,\n in pixels.\n\n",
        name:"getBottom",
        seeAlso:[],
        returns:{description:"bottom coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "sizing"]
    },
    "attr:Slider.showRange":{
        ref:"attr:Slider.showRange",
        flags:"IRW",
        description:"Indicates whether labels for the min and max values of the slider should be displayed. The\n default positions for these labels are below the start/end of a horizontal slider, or to the\n right of the start/end of a vertical slider.",
        name:"showRange",
        seeAlso:[
            {ref:"attr:Slider.minValueLabel", description:null},
            {ref:"attr:Slider.maxValueLabel", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider"
    },
    "type:DateInputFormat":{
        ref:"type:DateInputFormat",
        description:"3 character string containing the <code>\"M\"</code>, <code>\"D\"</code> and <code>\"Y\"</code>\n  characters to indicate the format of strings being parsed into Date instances via \n  <code>Date.parseInput()</code>.\n  <P>\n  As an example - an input format of \"MDY\" would parse \"01/02/1999\" to Jan 2nd 1999\n  <P>\n  Note: In addition to these standard formats, a custom date string parser function may be\n  passed directly to ${isc.DocUtils.linkForRef('classMethod:Date.setInputFormat')} or passed into ${isc.DocUtils.linkForRef('classMethod:Date.parseInput')} as \n  the inputFormat parameter.",
        name:"DateInputFormat",
        seeAlso:[],
        type:"type"
    },
    "class:Layout":{
        methods:["method:Layout.getMemberOffset", "method:Layout.setLayoutMargin", 
         "method:Layout.getMemberDefaultBreadth","method:Layout.getMemberSizes", "method:Layout.reflow", "method:Layout.reflowNow", 
         "method:Layout.getMember","method:Layout.getMemberNumber", "method:Layout.hasMember", "method:Layout.getMembers", 
         "method:Layout.addMember","method:Layout.addMembers", "method:Layout.removeMember", "method:Layout.removeMembers", 
         "method:Layout.showMember","method:Layout.hideMember", "method:Layout.setVisibleMember", 
         "method:Layout.reorderMember","method:Layout.reorderMembers", "method:Layout.membersChanged", 
         "method:Layout.getDropComponent","method:Layout.drop", "method:Layout.getDropPosition", "method:Layout.hideDropLine"],
        ref:"class:Layout",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:Layout.members", "attr:Layout.overflow", "attr:Layout.orientation", 
         "attr:Layout.vertical","attr:Layout.vPolicy", "attr:Layout.hPolicy", "attr:Layout.minMemberSize", 
         "attr:Layout.enforcePolicy","attr:Layout.defaultLayoutAlign", "attr:Layout.align", "attr:Layout.reverseOrder", 
         "attr:Layout.paddingAsLayoutMargin","attr:Layout.layoutMargin", "attr:Layout.layoutLeftMargin", 
         "attr:Layout.layoutRightMargin","attr:Layout.layoutTopMargin", "attr:Layout.layoutBottomMargin", 
         "attr:Layout.membersMargin","attr:Layout.leaveScrollbarGap", "attr:Layout.memberOverlap", 
         "attr:Layout.defaultResizeBars","attr:Layout.resizeBarClass", "attr:Layout.resizeBarSize", "attr:Layout.animateMembers", 
         "attr:Layout.animateMemberTime","attr:Layout.canDropComponents", "attr:Layout.dropLineThickness", 
         "attr:Layout.showDragPlaceHolder","attr:Layout.placeHolderProperties", "attr:Layout.stackZIndex", 
         "attr:Layout.managePercentBreadth","attr:Layout.placeHolderDefaults", "attr:Layout.dropLine"],
        description:"A subclass of Canvas that automatically arranges other Canvases according to a layout policy.\n <br><br>\n A Layout manages a set of \"member\" Canvases initialized via the \"members\" property.  Layouts\n can have both \"members\", which are managed by the Layout, and normal Canvas children, which\n are unmanaged.\n <br><br>\n Rather than using the Layout class directly, use the HLayout, VLayout, HStack and VStack\n classes, which are subclasses of Layout preconfigured for horizontal or vertical stacking,\n with the \"fill\" (VLayout) or \"none\" (VStack) ${isc.DocUtils.linkForRef('type:LayoutPolicy','policies')} already set.\n <br><br>\n Layouts and Stacks may be nested to create arbitrarily complex layouts.\n <br><br>\n To show a resizer bar after (to the right or bottom of) a layout member, set showResizeBar to\n true on that member component (not on the HLayout or VLayout).  Resizer bars override\n membersMargin spacing.\n <br><br> \n Like other Canvas subclasses, Layout and Stack components may have % width and height\n values. To create a dynamically-resizing layout that occupies the entire page (or entire\n parent component), set width and height to \"100%\".",
        name:"Layout",
        seeAlso:[
            {ref:"type:LayoutPolicy", description:"for available policies"},
            {ref:"class:VLayout", description:null},
            {ref:"class:HLayout", description:null},
            {ref:"class:VStack", description:null},
            {ref:"class:HStack", description:null},
            {ref:"class:LayoutSpacer", description:null}
        ],
        type:"class"
    },
    "attr:StaticTextItem.textBoxStyle":{
        ref:"attr:StaticTextItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class for this item",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"staticTextItem\"",
        type:"attr",
        definingClass:"class:StaticTextItem",
        groups:["appearance"]
    },
    "method:FormItem.getDisplayFieldName":{
        ref:"method:FormItem.getDisplayFieldName",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} for this form item.\n If unset, and ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is explicitly specified, this method\n will return the title field for the <code>optionDataSource</code>\n",
        name:"getDisplayFieldName",
        seeAlso:[],
        returns:{description:"display field name, or null", type:"String"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "method:Canvas.handleHover":{
        ref:"method:Canvas.handleHover",
        flags:"A",
        description:" Handler fired on a delay when the user hovers the mouse over this hover-target.\n Default implementation will fire <code>this.hover()</code> (if defined), and handle \n showing the hover canvas if <code>this.showHover</code> is true.\n",
        name:"handleHover",
        seeAlso:[
            {ref:"attr:Canvas.canHover", description:null},
            {ref:"attr:Canvas.showHover", description:null},
            {ref:"method:Canvas.hover", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["hovers"]
    },
    "attr:NativeCheckboxItem.showLabel":{
        ref:"attr:NativeCheckboxItem.showLabel",
        flags:"IRW",
        description:"Should we should the label text next to the checkbox item.",
        name:"showLabel",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:NativeCheckboxItem"
    },
    "classMethod:Canvas.hiliteCharacter":{
        ref:"classMethod:Canvas.hiliteCharacter",
        flags:"A",
        description:"			Given a string and a character, hilite the first occurrance of the character in the\n          string (if it occurs), preferring uppercase to lowercase.\n\n",
        name:"hiliteCharacter",
        seeAlso:[],
        params:[
            {description:"String to return with hilited character", optional:false, name:"string",
             defaultValue:null, type:"string"},
            {description:"Character to hilite", optional:false, name:"character", defaultValue:null,
             type:"character"},
            {description:"Prefix to apply to hilighted character - defaults to                                      \"&lt;span style='text-decoration:underline;'&gt;\"",
             optional:true, name:"hilitePrefix", defaultValue:null, type:"string"},
            {description:"Suffix to apply to hiliteed character - defaults to                                          \"&lt;/span&gt;\"",
             optional:true, name:"hiliteSuffix", defaultValue:null, type:"string"}
        ],
        returns:{description:"The string passed in, with the first occurrance of the hilite                          character enclosed by the 'hilitePrefix' and 'hiliteSuffix'",
         type:"string"},
        type:"classMethod",
        definingClass:"class:Canvas",
        groups:["utils"]
    },
    "method:ValuesManager.showFieldErrors":{
        ref:"method:ValuesManager.showFieldErrors",
        flags:"",
        description:" Method to explicitly show the latest set of validation errors present on some field \n within this ValuesManager.<br>\n If the field in question is present as a visible item in a member form, the form item\n will be redrawn to display the error message(s).\n Otherwise ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')} will be fired to allow \n custom handling of hidden errors.\n",
        name:"showFieldErrors",
        seeAlso:[],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:ImgButton.iconSize":{
        ref:"attr:ImgButton.iconSize",
        flags:"IR",
        description:"Size in pixels of the icon image.\n <P>\n The <code>iconWidth</code> and <code>iconHeight</code> properties can be used to\n configure width and height separately.",
        name:"iconSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:Chart.chartType":{
        ref:"attr:Chart.chartType",
        flags:"IR",
        description:"Type of chart to draw, see ${isc.DocUtils.linkForRef('type:ChartType')} for a list of known types.",
        name:"chartType",
        seeAlso:[],
        valueType:"ChartType",
        defaultValue:"\"Column\"",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "method:FormItem.getCriterion":{
        ref:"method:FormItem.getCriterion",
        flags:"A",
        description:" Override this method if you need to provide a specialized criterion from this formItem\n when creating an AdvancedCriteria via ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}.\n <P>\n This API is provided to allow you to specify a more complex criterion than the \n \"field-operator-value\" criterions that are built-in.  Note that the built-in behavior is\n generally quite flexible and powerful enough for most requirements.  An example of a case\n where you might want to override this method is if you wanted to implement a date range \n selection (ie, date &gt; x AND date &lt; y) on a form that was combining its other criteria \n fields with an OR.\n\n",
        name:"getCriterion",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:ColorPicker.okButton":{
        ref:"attr:ColorPicker.okButton",
        flags:"R",
        description:"\"OK\" button for the ColorPicker",
        name:"okButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Canvas.addPeer":{
        ref:"method:Canvas.addPeer",
        flags:"",
        description:"      Adds newPeer as a peer of this widget (also making it a child of this widget's parent, if\n      any), set up a named object reference (i.e., this[name]) to the new widget if name is\n      provided, and draw the peer if this widget has been drawn already.<br>\n      The widget to be added as a peer will be removed from its old master and/or parent, if any,\n      and it will be added as a child to the parent of this canvas (if any)\n",
        name:"addPeer",
        seeAlso:[],
        params:[
            {description:"new peer widget to add", optional:false, name:"newPeer", defaultValue:null,
             type:"canvas"},
            {description:"name to assign to peer (eg: this[peer] == child)", optional:true,
             name:"name", defaultValue:null, type:"string"},
            {description:"if true, peer will not automatically be drawn (only                                  for advanced use)",
             optional:true, name:"autoDraw", defaultValue:null, type:"boolean"},
            {description:"if true, when draw is called on the master widget, the peer\n                                  will be drawn before the master",
             optional:true, name:"preDraw", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"the new peer, or null if it couldn't be added", type:"canvas"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "attr:Canvas.canDragScroll":{
        ref:"attr:Canvas.canDragScroll",
        flags:"IRWA",
        description:"If this Canvas is canAcceptDrop:true, when the user drags a droppable widget over\n      an edge of the widget, should we scroll to show the rest of the widget's content?\n      Returned from canvas.shouldDragScroll().",
        name:"canDragScroll",
        seeAlso:[
            {ref:"method:Canvas.shouldDragScroll", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragging"]
    },
    "attr:TreeGrid.folderDropImage":{
        ref:"attr:TreeGrid.folderDropImage",
        deprecated:"as part of SmartClient release 5.5 in favor of ${isc.DocUtils.linkForRef('attr:TreeGrid.folderIcon')}",
        flags:"IRW",
        description:"The filename of the icon displayed for a folder node that will accept drag-and-drop\n data when the mouse is released.",
        name:"folderDropImage",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "attr:SelectItem.pickButtonSrc":{
        ref:"attr:SelectItem.pickButtonSrc",
        deprecated:"As of SmartClient version 5.5, pickButtonSrc has been deprecated in   favor of ${isc.DocUtils.linkForRef('attr:SelectItem.pickerIconSrc')}.",
        flags:"IRWA",
        description:"Source for image to show for the pick button",
        name:"pickButtonSrc",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "group:sqlDataSource":{
        ref:"group:sqlDataSource",
        title:"SQL DataSources",
        treeLocation:["Client Reference/Data Binding/DataSource"],
        refs:["attr:DataSourceField.sequenceName"],
        description:"The SmartClient Server supports comprehensive, codeless SQL connectivity for DataSources\n defined in XML.  Our SQL connectivity is mature, feature-rich, protected against injection\n attacks and easily customizable to support user-written SQL and additional business logic\n of any complexity.  ${isc.DocUtils.linkForRef('group:sqlVsJPA','This article')} compares the built-in SQL DataSource\n to other persistence approaches based on Javabeans.\n <P>\n To use the built-in SQL engine, declare a ${isc.DocUtils.linkForRef('class:DataSource')} in XML format with\n ${isc.DocUtils.linkForRef('attr:DataSource.serverType')} set to \"sql\", and place it in the shared dataSources directory\n ([webroot]/shared/ds by default) in a file called \"[dataSourceId].ds.xml\".\n <p>\n You can then use the ${isc.DocUtils.linkForRef('group:adminConsole','Admin Console')} to configure database access,\n as well as to automatically create and populate a database table based on your DataSource.\n By default, DataSources will use the \"default database\" from the admin console, however you\n can set ${isc.DocUtils.linkForRef('attr:DataSource.dbName')} to the name of a specific database configuration you have\n configured via the Admin Console.\n <P>\n The list of databases known to work with the the built-in SQL logic is as follows:\n <p>\n <table class='normal'>\n <tr><td width=40></td><td width=200><i>HSQLDB 1.7.x, 1.8.x</i></td><td></td></tr>\n <tr><td></td><td><i>IBM DB2 8.x, 9.x</i></td><td></td></tr>\n <tr><td></td><td><i>MS SQL Server 2000, 2005</i></td><td></td></tr>\n <tr><td></td><td><i>MySQL 3.2.x, 4.0.x, 4.1.x, 5.0.x, 5.1.x</i></td><td></td></tr>\n <tr><td></td><td><i>Oracle 8.0.5, 8i, 9i, 10g</i></td><td></td></tr>\n <tr><td></td><td><i>PostgreSQL 7.x, 8.x</i></td><td></td></tr>\n </table>\n <p>You will also need a JDBC driver for your specific database. Licensing restrictions\n prevent us including any JDBC driver other than the one for HSQLDB.  However, you\n can download these drivers for free from the vendors' websites.  If your specific database\n server or version is not listed above, please go to the\n <a href=\"http://forums.smartclient.com\">SmartClient forums</a> for assistance.\n <P>\n You can also connect to an existing database table - the Visual Builder DataSource Wizard\n (\"New\" button in DataSources pane) allows you to pick an existing database table and have a\n DataSource generated from it.  The Batch DataSource Generator tool will allow you to\n generate DataSources for any number of SQL DataSources, and customize the process so that\n organization-specific rules are applied (eg ignoring certain columns, or using naming\n conventions to generate more complete DataSources).\n <P>\n Once you have your SQL DataSource connected to a table, in a default SDK installation,\n DSRequests for your DataSource will be sent to the default\n ${isc.DocUtils.linkForRef('classAttr:RPCManager.actionURL','actionURL')}, and hence handled automatically, without you having\n to write any Java code, by the ${isc.DocUtils.linkForRef('group:servletDetails','IDACall servlet registered in web.xml')}.\n IDACall is a very simple servlet that just calls the server-side method\n dsRequest.execute() for all inbound requests.  For more details on how DataSource requests\n are processed by SmartClient Server, and how you can alter and add to this processing, see\n this description of ${isc.DocUtils.linkForRef('group:serverDataIntegration','server data integration')}.",
        name:"sqlDataSource",
        seeAlso:[],
        type:"group",
        requiresModules:["SCServer"]
    },
    "method:ImgButton.setAutoFit":{
        ref:"method:ImgButton.setAutoFit",
        flags:"",
        description:" Setter method for the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.autoFit')} property. Pass in true or false to turn\n autoFit on or off. When autoFit is set to <code>false</code>, canvas will be resized to\n it's previously specified size.\n",
        name:"setAutoFit",
        seeAlso:[],
        params:[
            {description:"New autoFit setting.", optional:false, name:"autoFit", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ImgButton"
    },
    "class:MultiSortDialog":{
        methods:["method:MultiSortDialog.getNumLevels", "method:MultiSortDialog.getSortLevel", 
         "method:MultiSortDialog.getSort","method:MultiSortDialog.validate"],
        ref:"class:MultiSortDialog",
        classAttrs:[],
        classMethods:["classMethod:MultiSortDialog.askForSort"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:["attr:MultiSortDialog.addLevelButtonTitle", "attr:MultiSortDialog.deleteLevelButtonTitle", 
         "attr:MultiSortDialog.copyLevelButtonTitle","attr:MultiSortDialog.applyButtonTitle", "attr:MultiSortDialog.cancelButtonTitle", 
         "attr:MultiSortDialog.addLevelButton","attr:MultiSortDialog.deleteLevelButton", "attr:MultiSortDialog.copyLevelButton", 
         "attr:MultiSortDialog.levelUpButton","attr:MultiSortDialog.levelDownButton", "attr:MultiSortDialog.fields", 
         "attr:MultiSortDialog.optionsGrid","attr:MultiSortDialog.initialSort", "attr:MultiSortDialog.maxLevels", 
         "attr:MultiSortDialog.invalidListPrompt"],
        description:"A dialog that allows the user to set up complex sorting arrangments by defining a group of\n ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n <P>\n Each ${isc.DocUtils.linkForRef('object:SortSpecifier')} applies to a single property and direction - so, for instance, in \n a grid with two columns, <code>year</code> and <code>monthNumber</code>, you could sort first \n by <code>year</code> in descending order and then by <code>monthNumber</code> in ascending \n order.  This would producing a grid sorted by year from largest (most \n recent) to smallest (least recent) and, within each year, by monthNumber from smallest \n (January) to largest (December).",
        name:"MultiSortDialog",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.removeFieldTitle":{
        ref:"attr:ListGrid.removeFieldTitle",
        flags:null,
        description:"(String : \"[Remove record]\" : IRW The title to use for the ${isc.DocUtils.linkForRef('listGrid.removeField','remove field')}.  Note that this\n text will appear in the column-picker but the not in the field-header because the removeField\n has ${isc.DocUtils.linkForRef('listGridField.showTitle','showTitle')} set to false by default.  This can be\n changed via ${isc.DocUtils.linkForRef('attr:ListGrid.removeFieldProperties')}.",
        name:"removeFieldTitle",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.editorIconWidth":{
        ref:"attr:ListGridField.editorIconWidth",
        flags:"IRA",
        description:"If this field is editable, this property will be passed to editors displayed for\n cells within this field as ${isc.DocUtils.linkForRef('attr:FormItem.iconWidth')}.<br>\n If this property unset, the iconWidth property from the editor can be picked up from \n ${isc.DocUtils.linkForRef('attr:ListGridField.iconWidth')} instead.",
        name:"editorIconWidth",
        seeAlso:[
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:RPCRequest.callbackParam":{
        ref:"attr:RPCRequest.callbackParam",
        flags:"IRW",
        description:"For use only with the ${isc.DocUtils.linkForRef('type:RPCTransport','scriptInclude')} transport, this attribute\n specifies the name of the URL parameter which is used to specify the callback function that\n the server is expected to call by writing out JavaScript code.  The actual function to call\n is automatically generated and differs for every request (to allow concurrency).\n <P>\n For example, with <code>callbackParam</code> set to it's default value of \"callback\", the\n server might be contacted with a URL like:\n <pre>\n    loadData?callback=isc_scriptIncludeCallback_5\n </pre>\n .. then the server's response should look like:\n <pre>\n    isc_scriptIncludeCallback_5({ .. data .. });\n </pre>\n The name \"isc_scriptIncludeCallback_5\" is automatically generated and will differ each time\n the server is contacted.\n <P>\n SmartClient makes of this server-provided callback mechanism, then calls\n ${isc.DocUtils.linkForRef('attr:RPCRequest.callback')} normally.\n <p>\n <code>rpcRequest.callbackParam</code> is ignored by all transport other than\n <code>scriptInclude</code>.",
        name:"callbackParam",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"callback\"",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:Canvas.scrollToPercent":{
        ref:"method:Canvas.scrollToPercent",
        flags:"",
        description:"  Scroll this widget to some position specified as a percentage of scroll size in either \n (or both) direction(s).\n\n",
        name:"scrollToPercent",
        seeAlso:[],
        params:[
            {description:"Left Percentage position to scroll to                                         Will accept either a numeric percent value, or a \n                                        string like \"10%\".",
             optional:false, name:"left", defaultValue:null, type:"number | string"},
            {description:"Top Percentage position to scroll to                                        Will accept either a numeric percent value, or a \n                                       string like \"10%\".",
             optional:false, name:"top", defaultValue:null, type:"number | string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:FormulaBuilder.autoTest":{
        ref:"attr:FormulaBuilder.autoTest",
        flags:"IRWA",
        description:"When set to true, automatically tests the formula by calling \n ${isc.DocUtils.linkForRef('method:FormulaBuilder.testFunction','testFunction()')} whenever typing into the \n ${isc.DocUtils.linkForRef('attr:FormulaBuilder.formulaField','formulaField')} pauses.\n <P>\n The default is true.",
        name:"autoTest",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "classMethod:EventHandler.getWheelDelta":{
        ref:"classMethod:EventHandler.getWheelDelta",
        flags:"",
        description:" Applies to ${isc.DocUtils.linkForRef('method:Canvas.mouseWheel','mouseWheel')} events only.\n Returns an integer indicating how far the mouse wheel was rotated. This value will be\n positive if the user scrolled the mousewheel forward or up, or negative if scrolled in the\n other direction and will be a multiple of 1 where 1 indicates the smallest possible rotation\n of the wheel.\n",
        name:"getWheelDelta",
        seeAlso:[],
        returns:{description:"integer indicating how far the mouse wheel was rotated.", type:"integer"},
        type:"classMethod",
        definingClass:"class:EventHandler"
    },
    "method:TileLayout.removeTile":{
        ref:"method:TileLayout.removeTile",
        flags:"",
        description:" Remove a tile from the layout.\n\n",
        name:"removeTile",
        seeAlso:[],
        params:[
            {description:"index or String ID of the tile", optional:false, name:"tileID",
             defaultValue:null, type:"Canvas or int or ID"}
        ],
        returns:{description:"whether a tile was found and removed", type:"boolean"},
        type:"method",
        definingClass:"class:TileLayout"
    },
    "type:TEXTAREA_WRAP":{
        ref:"type:TEXTAREA_WRAP",
        values:[
            {description:"don't allow wrapping at all", value:"TextAreaItem.OFF"},
            {description:"when the entered text reaches the edge of the text area,                                  wrap visibly but don't include line breaks in the textarea\n                                  value",
             value:"TextAreaItem.SOFT"},
            {description:"when the entered text reaches the edge of the text area,                                   insert a line break",
             value:"TextAreaItem.HARD"}
        ],
        description:null,
        name:"TEXTAREA_WRAP",
        seeAlso:[],
        type:"type"
    },
    "attr:Window.bodyDefaults":{
        ref:"attr:Window.bodyDefaults",
        flags:"IRWA",
        description:"Default properties for the body of the Window<br>\n You can change the class-level bodyDefaults for all Windows by changing this item\n or set  instance.body to be another object of properties to override for your instance only",
        name:"bodyDefaults",
        seeAlso:[],
        valueType:"object",
        defaultValue:"...",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "method:DataBoundComponent.getDragData":{
        ref:"method:DataBoundComponent.getDragData",
        flags:"A",
        description:"\n During a drag-and-drop interaction, this method returns the set of records being dragged out\n of the component.  In the default implementation, this is the list of currently selected\n records.<p>\n \n This method is consulted by ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')}.\n\n",
        name:"getDragData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"}
        ],
        returns:{description:"Array of ${isc.DocUtils.linkForRef('object:Record')}s that are currently selected.",
         type:"Array of Record"},
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["dragging", "data"]
    },
    "attr:FilterBuilder.validateOnChange":{
        ref:"attr:FilterBuilder.validateOnChange",
        flags:"IR",
        description:"If true (the default), validates each entered value when it changes, to make sure it is a \n a valid value of its type (valid string, number, and so on).  No other validation is \n carried out.  If you switch this property off, it is still possible to validate the \n <code>FilterBuilder</code> by calling ${isc.DocUtils.linkForRef('filterBuilder.validate')} from your own code.",
        name:"validateOnChange",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "method:Canvas.scrolled":{
        ref:"method:Canvas.scrolled",
        flags:"",
        description:" Notification that this component has just scrolled.  Use with\n ${isc.DocUtils.linkForRef('method:Class.observe','observation')}.\n <P>\n Fires for both CSS and ${isc.DocUtils.linkForRef('class:Scrollbar','\"synthetic\" scrollbars')}.\n\n",
        name:"scrolled",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:DateTimeItem.displayFormat":{
        ref:"attr:DateTimeItem.displayFormat",
        flags:"IRW",
        description:"This property can be used to customize the format in which datetimes are displayed.<br>\n Should be set to a standard ${isc.DocUtils.linkForRef('type:DateDisplayFormat')} or\n a function which will return a formatted date time string.\n <P>\n If unset, the standard shortDateTime format as set up in \n ${isc.DocUtils.linkForRef('classMethod:Date.setShortDatetimeDisplayFormat')} will be used.\n <P>\n <B>NOTE: you may need to update the ${isc.DocUtils.linkForRef('attr:DateTimeItem.inputFormat','inputFormat')}\n to ensure the DateItem is able to parse user-entered date strings back into Dates</B>",
        name:"displayFormat",
        seeAlso:[
            {ref:"attr:DateTimeItem.inputFormat", description:null}
        ],
        valueType:"DateDisplayFormat",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateTimeItem"
    },
    "classMethod:isc.timeStamp":{
        ref:"classMethod:isc.timeStamp",
        flags:"",
        description:"  Shorthand for <code>new Date().getTime();</code>, this returns a timeStamp - a large number\n  which is incremented by 1 every millisecond.  Can be used to generate unique identifiers,\n  or perform timing tasks.\n\n",
        name:"timeStamp",
        seeAlso:[],
        returns:{description:"a large integer (actually the number of milliseconds since 1/1/1970)",
         type:"number"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:ComboBoxItem.optionDataSource":{
        ref:"attr:ComboBoxItem.optionDataSource",
        flags:"IRA",
        description:"If set, this FormItem will derive data to show in the PickList by fetching records from\n the specified <code>optionDataSource</code>.  The fetched data will be used as a\n ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} by extracting the\n ${isc.DocUtils.linkForRef('attr:FormItem.valueField','valueField')} and ${isc.DocUtils.linkForRef('attr:FormItem.displayField','displayField')} in\n the loaded records, to derive one valueMap entry per record loaded from the\n optionDataSource.  Multiple fields from the fetched data may be shown in the pickList by\n setting ${isc.DocUtils.linkForRef('attr:ComboBoxItem.pickListFields')}.\n <P>\n The data will be retrieved via a \"fetch\" operation on the DataSource, passing the \n ${isc.DocUtils.linkForRef('attr:PickList.pickListCriteria')} (if set) as criteria, and passing\n ${isc.DocUtils.linkForRef('attr:ComboBoxItem.optionFilterContext')} (if set) as DSRequest properties.\n <P>\n The fetch will be triggered when the pickList is first shown, or, you can set\n ${isc.DocUtils.linkForRef('attr:SelectItem.autoFetchData','autoFetchData:true')} to fetch when the FormItem is\n first drawn.  \n <P>\n Note that providing an initial value when\n ${isc.DocUtils.linkForRef('attr:FormItem.fetchMissingValues','fetchMissingValues')} is enabled, or enabling\n ${isc.DocUtils.linkForRef('attr:SelectItem.defaultToFirstOption','defaultToFirstOption')}, can also cause a fetch to\n be initiated immediately upon form creation.  You can also call ${isc.DocUtils.linkForRef('method:PickList.fetchData')}\n at any time to manually trigger a fetch.\n <P>\n Data paging is automatically enabled if the optionDataSource supports it.  As the\n pickList is scrolled by the user, requests for additional data will be automatically\n issued.\n <P>\n For a pickList attached to a ComboBoxItem, new fetches are issued as the user types,\n with criteria set as described under ${isc.DocUtils.linkForRef('method:ComboBoxItem.getPickListFilterCriteria')}.\n If your dataSource is not capable of filtering results by search criteria (eg the\n dataSource is backed by an XML flat file), you can set ${isc.DocUtils.linkForRef('attr:ComboBoxItem.filterLocally')} to have the\n entire dataset loaded up front and filtering performed in the browser.  This disables\n data paging.\n <P>\n Note that if a normal, static ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','valueMap')} is <b>also</b> specified\n for the field (either directly in the form item or as part of the field definition in\n the dataSource), it will be preferred to the data derived from the optionDataSource for\n whatever mappings are present.",
        name:"optionDataSource",
        seeAlso:[],
        valueType:"DataSource | String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:ListGrid.freezeOnLeftText":{
        ref:"attr:ListGrid.freezeOnLeftText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid and\n ${isc.DocUtils.linkForRef('attr:ListGrid.canFreezeFields','this.canFreezeFields')} is true, this string will be shown as the\n title for the menu item to freeze fields on the left of the scrollable body.",
        name:"freezeOnLeftText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Freeze on left\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "classMethod:FileLoader.cacheStretchImgStates":{
        ref:"classMethod:FileLoader.cacheStretchImgStates",
        flags:"",
        description:"\n Caches a set of state stretch images derived from a base image.  This is useful for caching a set\n of images for a single component.  For example this code:\n <pre>\n isc.FileLoader.cacheStretchImgStates(\"/isomorphic/skins/SmartClient/images/button/button.png\",\n                                      \"Down,Over\");\n </pre>\n Will cause the following images to be cached:\n <pre>\n /isomorphic/skins/SmartClient/images/button/button_start.png\n /isomorphic/skins/SmartClient/images/button/button_stretch.png\n /isomorphic/skins/SmartClient/images/button/button_end.png\n /isomorphic/skins/SmartClient/images/button/button_Down_start.png\n /isomorphic/skins/SmartClient/images/button/button_Down_stretch.png\n /isomorphic/skins/SmartClient/images/button/button_Down_end.png\n /isomorphic/skins/SmartClient/images/button/button_Over_start.png\n /isomorphic/skins/SmartClient/images/button/button_Over_stretch.png\n /isomorphic/skins/SmartClient/images/button/button_Over_end.png\n </pre>\n\n",
        name:"cacheStretchImgStates",
        seeAlso:[
            {ref:"classMethod:FileLoader.cacheImgStates", description:null}
        ],
        params:[
            {description:"List of base image URLs from which stateful names are to be                                    derived.  This can either be a\n                                    string with comma separated baseURLs or an Array of baseURLs.",
             optional:false, name:"baseURLs", defaultValue:null, type:"String or Array"},
            {description:"List of states to load.  If none specified, The following                                    default list will be used:\n                                    \"Down,Over,Selected,Selected_Down,Selected_Over\".  This can\n                                    either be a string with comma separated states or an Array\n                                    of states",
             optional:true, name:"states", defaultValue:null, type:"String or Array"},
            {description:"List of StretchImg pieces to cache.  If not specified, the                                    following list will be used: \"start,stretch,end\".  This can\n                                    either be a string with comma separated piece names or an Array\n                                    of piece names.",
             optional:true, name:"pieces", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified URLs                                      has completed caching.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "method:Canvas.moveTo":{
        ref:"method:Canvas.moveTo",
        flags:"",
        description:" Moves the widget so that its top-left corner is at the specified coordinates. \n <P>\n This method will also accept a single parameter as an object array with left and top given\n as properties.\n\n",
        name:"moveTo",
        seeAlso:[],
        params:[
            {description:"x-coordinate to move to in LOCAL coordinates						                       or Object with left and top properties",
             optional:true, name:"left", defaultValue:null, type:"number or Object"},
            {description:"y-coordinate to move to in LOCAL coordinates", optional:true, name:"top",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"whether the component actually moved", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('move')}"],
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "classAttr:Hover.hoverCanvasDefaults":{
        ref:"classAttr:Hover.hoverCanvasDefaults",
        flags:"IRW",
        description:"Defaults to apply to the Hover canvas shown when the user hovers over some widget.\n By default this property is set to this object:<br><pre>\n       { defaultWidth:100, \n         defaultHeight:1,\n         baseStyle:\"canvasHover\",\n         align:\"left\",\n         valign:\"top\",\n         opacity:null\n        }\n </pre><br>\n Note that these properties can be overridden by individual widgets showing hovers, by\n modifying ${isc.DocUtils.linkForRef('attr:Canvas.hoverWidth')}, ${isc.DocUtils.linkForRef('attr:Canvas.hoverHeight')}, \n ${isc.DocUtils.linkForRef('attr:Canvas.hoverStyle')}, ${isc.DocUtils.linkForRef('attr:Canvas.hoverAlign')}, ${isc.DocUtils.linkForRef('attr:Canvas.hoverVAlign')}, \n ${isc.DocUtils.linkForRef('attr:Canvas.hoverOpacity')}, and ${isc.DocUtils.linkForRef('attr:Canvas.hoverWrap')}.",
        name:"hoverCanvasDefaults",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{...}",
        type:"classAttr",
        definingClass:"class:Hover"
    },
    "attr:FormItem.hintStyle":{
        ref:"attr:FormItem.hintStyle",
        flags:"IRW",
        description:"CSS class for the \"hint\" string.",
        name:"hintStyle",
        setter:["setHintStyle"],
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"string",
        defaultValue:"\"formHint\"",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:CanvasItem.canvasConstructor":{
        ref:"attr:CanvasItem.canvasConstructor",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:CanvasItem.canvas','this.canvas')} is not specified as a canvas instance at init\n time, a canvas will be created instead. This property denotes the class of that widget\n (Should be set to the name of a subclass of Canvas).",
        name:"canvasConstructor",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Canvas\"",
        type:"attr",
        definingClass:"class:CanvasItem"
    },
    "method:ListGrid.deselectRecords":{
        ref:"method:ListGrid.deselectRecords",
        flags:"",
        description:"\n Deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecords(records, false)</code>\n\n",
        name:"deselectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to deselect", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:ColumnTree.fetchData":{
        ref:"method:ColumnTree.fetchData",
        flags:"",
        description:" Uses a \"fetch\" operation on the current ${isc.DocUtils.linkForRef('class:DataSource','grid.dataSource')} to retrieve data\n that matches the provided criteria, and displays the matching data in this component as a\n tree.\n <P>\n This method will create a ${isc.DocUtils.linkForRef('class:ResultTree')} to manage tree data, which will\n subsequently be available as <code>columnTree.data</code>.  DataSource records\n returned by the \"fetch\" operation are linked into a tree structure according to\n ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')} and\n ${isc.DocUtils.linkForRef('attr:DataSourceField.foreignKey','foreignKey')} declarations on DataSource fields.  See the\n ${isc.DocUtils.linkForRef('group:treeDataBinding')} topic for complete details.\n <P>\n By default, the created ResultTree will use folder-by-folder load on demand, asking the\n server for the children of each folder as the user opens it.\n <P>\n The ${isc.DocUtils.linkForRef('class:ResultTree')} created by <code>fetchData()</code> can be customized by setting\n ${isc.DocUtils.linkForRef('attr:ListGrid.dataProperties')} to an Object containing properties and methods to apply to\n the created ResultTree.  For example, the property that determines whether a node is a\n folder (${isc.DocUtils.linkForRef('attr:Tree.isFolderProperty','isFolderProperty')}) can be customized, or\n level-by-level loading can be disabled via\n ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand','loadDataOnDemand:false')}.\n <P>\n The callback passed to <code>fetchData</code> will fire once, the first time that data is\n loaded from the server.  If using folder-by-folder load on demand, use the\n ${isc.DocUtils.linkForRef('method:ResultTree.dataArrived')} notification to be notified each time new nodes are loaded.\n <P>\n Note that, if criteria are passed to <code>fetchData()</code>, they will be passed every\n time a new \"fetch\" operation is sent to the server.  This allows you to retrieve multiple\n different tree structures from the same DataSource.  However note that the server is expected\n to always respond with an intact tree - returned nodes which do not have parents are dropped\n from the dataset and not displayed.\n\n",
        name:"fetchData",
        seeAlso:[],
        params:[
            {description:"Search criteria.                       If a ${isc.DocUtils.linkForRef('class:DynamicForm')} is passed in as this argument\n                      instead of a raw criteria object, will be derived by calling\n                      ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}",
             optional:true, name:"criteria", defaultValue:null, type:"Criteria"},
            {description:"callback to invoke when a fetch is complete.  Fires                                          only if server contact was required",
             optional:true, name:"callback", defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["dataBoundComponentMethods"]
    },
    "method:FormulaBuilder.getHelpText":{
        ref:"method:FormulaBuilder.getHelpText",
        flags:"",
        description:" Call to retrieve the text the FormulaBuilder would show by default for help, or override to\n provide alternate help text.\n\n",
        name:"getHelpText",
        seeAlso:[],
        returns:{description:"The results of getHoverText()", type:"String"},
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:Button.showDisabled":{
        ref:"attr:Button.showDisabled",
        flags:"IRW",
        description:"Should we visibly change state when disabled?",
        name:"showDisabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonStates')}"],
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:DataSourceField.summaryFunction":{
        ref:"attr:DataSourceField.summaryFunction",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} or ${isc.DocUtils.linkForRef('attr:ListGrid.showGroupSummary')} is true, \n this attribute can be used to specify\n an explicit ${isc.DocUtils.linkForRef('type:SummaryFunction')} for calculating the summary value to\n display.",
        name:"summaryFunction",
        seeAlso:[],
        valueType:"SummaryFunction",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DataSourceField"
    },
    "classMethod:XMLTools.toJS":{
        ref:"classMethod:XMLTools.toJS",
        flags:"",
        description:" Translates an XML fragment to JavaScript collections.  This method works just like the\n server-side method XML.toJS(Element, Writer):\n <ul>\n <li>  Elements become JavaScript Objects with each attribute becoming a property\n <li>  Subelements with just text (no child elements or attributes) become properties\n <li>  Subelements with child elements or attributes become sub objects\n </ul>\n For example, if you pass the following fragment to this method:\n <pre>\n &lt;foo&nbsp;bar=\"zoo\"&gt;\n     &nbsp;&nbsp;&lt;x&gt;y&lt;/x&gt;\n &lt;/foo&gt;\n </pre>\n You will get back the following JS structure: \n <pre>\n { bar:\"zoo\", x:\"y\"}\n </pre>\n All atomic property values will be of String type.  Use ${isc.DocUtils.linkForRef('method:DataSource.recordsFromXML')}\n to do schema-driven XML to JS transform, which can produce correctly typed values.\n \n",
        name:"toJS",
        seeAlso:[],
        params:[
            {description:"The element to tranform to JS", optional:false, name:"element",
             defaultValue:null, type:"XMLElement or XMLDocument"}
        ],
        returns:{description:"The resulting JavaScript collection.", type:"Object"},
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('xmlServerValidationErrors')}"],
        definingClass:"class:XMLTools"
    },
    "attr:TreeGrid.showFullConnectors":{
        ref:"attr:TreeGrid.showFullConnectors",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:TreeGrid.showConnectors')} is true, this property determines whether we should show\n showing vertical continuation lines for each level of indenting within the tree. Setting to\n false will show only the hierachy lines are only shown for the most indented path (\"sparse\"\n connectors).",
        name:"showFullConnectors",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["treeIcons"]
    },
    "attr:TreeGrid.canReparentNodes":{
        ref:"attr:TreeGrid.canReparentNodes",
        flags:"IRW",
        description:"If set this property allows the user to reparent nodes by dragging them from their\n current folder to a new folder.<br>\n <b>Backcompat:</b> For backwards compatibility with versions prior to SmartClient 1.5,\n if this property is unset, but <code>this.canAcceptDroppedRecords</code> is true, we\n allow nodes to be dragged to different folders.",
        name:"canReparentNodes",
        seeAlso:[
            {ref:"attr:TreeNode.canDrag", description:null},
            {ref:"attr:TreeNode.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["dragdrop"]
    },
    "attr:DSRequest.exportDisplay":{
        ref:"attr:DSRequest.exportDisplay",
        flags:"IR",
        description:"Specifies whether the exported data will be downloaded to the file-system or displayed in a \n new window. See ${isc.DocUtils.linkForRef('type:ExportDisplay')} for more information.",
        name:"exportDisplay",
        seeAlso:[],
        valueType:"ExportDisplay",
        defaultValue:"\"download\"",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:TextAreaItem.iconVAlign":{
        ref:"attr:TextAreaItem.iconVAlign",
        flags:"IR",
        description:"Align icons with the top edge of text area icons by default.",
        name:"iconVAlign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.TOP",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["formIcons"]
    },
    "attr:RichTextEditor.value":{
        ref:"attr:RichTextEditor.value",
        flags:"IRW",
        getter:["getValue"],
        description:"Initial value for the edit area.    Use <code>getValue()</code> and \n <code>setValue()</code> to update at runtime.",
        name:"value",
        setter:["setValue"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "attr:ViewLoader.loadingMessage":{
        ref:"attr:ViewLoader.loadingMessage",
        flags:"IR",
        description:"Message to show while the view is loading",
        name:"loadingMessage",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Loading View...\"",
        type:"attr",
        definingClass:"class:ViewLoader",
        groups:["viewLoading"]
    },
    "attr:Label.showFocusedIcon":{
        ref:"attr:Label.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "group:location":{
        ref:"group:location",
        refs:["method:Tree.findById", "method:Tree.find", "attr:Window.autoCenter"],
        name:"location",
        seeAlso:[],
        type:"group"
    },
    "attr:ColumnTree.showCustomIconOpen":{
        ref:"attr:ColumnTree.showCustomIconOpen",
        flags:"IRWA",
        description:"Should folder nodes showing custom icons (set via the ${isc.DocUtils.linkForRef('attr:ColumnTree.customIconProperty')}),\n show open state images when the folder is opened.\n If true, the ${isc.DocUtils.linkForRef('attr:ColumnTree.openIconSuffix')} will be appended to the image URL\n (so <code>\"customFolder.gif\"</code> might be replaced with \n <code>\"customFolder_open.gif\"</code>).<br>\n <b>Note</b> that the ${isc.DocUtils.linkForRef('attr:ColumnTree.closedIconSuffix')} is never appended to custom folder icons.<br>\n Can be overridden at the node level via the default property ${isc.DocUtils.linkForRef('attr:TreeNode.showOpenIcon')}\n and that property can be renamed via ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconOpenProperty')}.",
        name:"showCustomIconOpen",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "attr:TreeGrid.fields":{
        ref:"attr:TreeGrid.fields",
        flags:"IRW",
        description:"An array of field objects, specifying the order, layout, dynamic calculation, and\n sorting behavior of each field in the treeGrid object. In TreeGrids, the fields\n array specifies columns. Each field in the fields array is TreeGridField object.\n <p>\n If ${isc.DocUtils.linkForRef('attr:TreeGrid.dataSource')} is also set, this value acts as a set of overrides as\n explained in ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}.",
        name:"fields",
        seeAlso:[
            {ref:"object:TreeGridField", description:null}
        ],
        valueType:"Array of TreeGridField",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["databinding"]
    },
    "method:Slider.valueChanged":{
        ref:"method:Slider.valueChanged",
        flags:"A",
        description:" This method is called when the slider value changes. This occurs when the setValue method is\n called, or when the slider is moved. Observe this method to be notified when the slider value\n changes.\n\n",
        name:"valueChanged",
        seeAlso:[
            {ref:"method:Class.observe", description:null}
        ],
        params:[
            {description:"the new value", optional:false, name:"value", defaultValue:null, type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:Slider"
    },
    "type:KeyName":{
        ref:"type:KeyName",
        description:"Strings to identify the various keys on the keyboard.\n <ul>\n <li>  For alpha keys, the single (uppercase) character value is used, such as \"Q\"\n <li>  For Numeric keys, the number is used as in a single character string, like \"1\"\n <li>  Function keys are identified as <code>\"f1\"</code> - <code>\"f12\"</code>\n <li>  Non alpha-numeric character keys (such as the key for \"[\" / \"{\") are identified by\n       their unmodified character value (the value obtained by hitting the key without \n       holding shift down), by default - exceptions are listed below.\n <li>  Additional key names:<br>\n      - <code>Space</code><br>    \n      - <code>Tab</code><br>    \n      - <code>Enter</code><br>\n      - <code>Escape</code><br>\n      - <code>Backspace</code><br>\n      - <code>Insert</code><br>\n      - <code>Delete</code><br>\n      - <code>Arrow_Up</code><br>\n      - <code>Arrow_Down</code><br>\n      - <code>Arrow_Left</code><br>\n      - <code>Arrow_Right</code><br>            \n      - <code>Home</code><br>\n      - <code>End</code><br>\n      - <code>Page_Up</code><br>    \n      - <code>Page_Down</code><br>   \n      - <code>Shift</code><br>\n      - <code>Ctrl</code><br>\n      - <code>Alt</code>\n </ul>\n [Note: Some keys may not be available for capture on every platform]",
        name:"KeyName",
        seeAlso:[],
        type:"type"
    },
    "attr:TreeGrid.canDragRecordsOut":{
        ref:"attr:TreeGrid.canDragRecordsOut",
        flags:"IRW",
        description:"Indicates whether records can be dragged from this listGrid and dropped elsewhere.",
        name:"canDragRecordsOut",
        seeAlso:[
            {ref:"attr:TreeNode.canDrag", description:null},
            {ref:"attr:TreeNode.canAcceptDrop", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('treeDropEvents')}"],
        definingClass:"class:TreeGrid",
        groups:["dragdrop"]
    },
    "attr:PickList.showOptionsFromDataSource":{
        ref:"attr:PickList.showOptionsFromDataSource",
        flags:"IRWA",
        description:"If this item is part of a databound form, and has a specified <code>valueMap</code>,\n by default we show the valueMap options in the pickList for the item.\n Setting this property to true will ensure that the options displayed in our pickList\n are derived from the form's <code>dataSource</code>.",
        name:"showOptionsFromDataSource",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList",
        groups:["databinding"]
    },
    "method:TileGrid.removeTile":{
        ref:"method:TileGrid.removeTile",
        flags:"",
        description:" This is not allowed for tileGrid. Instead, use ${isc.DocUtils.linkForRef('method:TileGrid.removeData')}.\n\n",
        name:"removeTile",
        seeAlso:[],
        type:"method",
        definingClass:"class:TileGrid"
    },
    "attr:LoginDialog.loginForm":{
        ref:"attr:LoginDialog.loginForm",
        flags:"R",
        description:"Form used to request login credentials from the user.",
        name:"loginForm",
        seeAlso:[
            {ref:"attr:LoginDialog.formFields", description:null}
        ],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "classAttr:GroupingMessages.byMinutesTitle":{
        ref:"classAttr:GroupingMessages.byMinutesTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a time field by minutes.",
        name:"byMinutesTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Minutes\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:StretchImg.vertical":{
        ref:"attr:StretchImg.vertical",
        flags:"IRW",
        description:"Indicates whether the list of images is drawn vertically from top to bottom (true),\n or horizontally from left to right (false).",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StretchImg",
        groups:["appearance"]
    },
    "method:TextAreaItem.getSelectionRange":{
        ref:"method:TextAreaItem.getSelectionRange",
        flags:"",
        description:" For text-based items, this method returns the indices of the start/end of the current\n selection. Returns null if the item doesn't have focus. \n",
        name:"getSelectionRange",
        seeAlso:[],
        returns:{description:"2 element array showing character index of the current selection's  start and end point within this item's value. May be null if the item doesn't have focus.",
         type:"array | null"},
        type:"method",
        definingClass:"class:TextAreaItem"
    },
    "attr:Img.size":{
        ref:"attr:Img.size",
        flags:"IR",
        description:"Convenience for setting height and width to the same value, at init time only",
        name:"size",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Img",
        groups:["sizing"]
    },
    "method:CubeGrid.facetValueOver":{
        ref:"method:CubeGrid.facetValueOver",
        flags:"",
        description:"    StringMethod handler fired when mouseover occurs over a facetValues header button\n  <br><i>methodType</i> handler\n",
        name:"facetValueOver",
        seeAlso:[],
        params:[
            {description:"facetValuesObject for the appropriate header button", optional:false,
             name:"facetValues", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events"]
    },
    "attr:ListGrid.summaryRowCriteria":{
        ref:"attr:ListGrid.summaryRowCriteria",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showGridSummary')} is true, and a ${isc.DocUtils.linkForRef('listGrid.summaryDataSource')} is specified\n this property may be used to specify fetch criteria to apply when retrieving summary data\n to show in the summary row. If unset, and any filter criteria have been specified for the\n grid, they will be used.",
        name:"summaryRowCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:DynamicForm.itemHoverHTML":{
        ref:"method:DynamicForm.itemHoverHTML",
        flags:"A",
        description:"  Retrieves the HTML to display in a hover canvas when the user holds the mousepointer over\n  some item.  Return null to suppress the hover canvas altogether.<br>\n  Default implementation returns the prompt for the item if defined.<br>\n  Can be overridden via <code>item.itemHoverHTML()</code>\n\n",
        name:"itemHoverHTML",
        seeAlso:[
            {ref:"attr:FormItem.prompt", description:null},
            {ref:"method:FormItem.itemHoverHTML", description:null}
        ],
        params:[
            {description:"Item the user is hovering over.", optional:false, name:"item",
             defaultValue:null, type:"FormItem"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:OperationBinding.customCriteriaFields":{
        ref:"attr:OperationBinding.customCriteriaFields",
        flags:"IR",
        description:"Specifies, for this operationBinding only, a list of extra field names that should be included in\n the default ${isc.DocUtils.linkForRef('attr:OperationBinding.whereClause','whereClause')} SmartClient generates.  Here, \"extra\" refers to\n fields that would not normally be included because they are marked as \n ${isc.DocUtils.linkForRef('attr:DataSourceField.customSQL','customSQL')}.\n You can specify this property as either an array or a string containing a comma-separated list \n of field names (eg, \"foo, bar, baz\").\n <p>\n This property is only applicable to DataSources of type \"sql\".",
        name:"customCriteriaFields",
        seeAlso:[
            {ref:"attr:OperationBinding.customValueFields", description:null},
            {ref:"attr:OperationBinding.excludeCriteriaFields", description:null}
        ],
        valueType:"String or Array",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "method:Array.slideRange":{
        ref:"method:Array.slideRange",
        flags:"A",
        description:" Slide a range of elements from start to end to position destination, moving all the other\n elements to cover the gap.\n\n",
        name:"slideRange",
        seeAlso:[],
        params:[
            {description:"start position", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"end position (exclusive, like substring() and slice())", optional:false,
             name:"end", defaultValue:null, type:"number"},
            {description:"destination position for the range", optional:false, name:"destination",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"object:Array"
    },
    "attr:Canvas.children":{
        ref:"attr:Canvas.children",
        flags:"IRWA",
        description:"Array of all Canvii that are immediate children of this Canvas.",
        name:"children",
        seeAlso:[],
        valueType:"Array of Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "attr:MultiSortPanel.copyLevelButton":{
        ref:"attr:MultiSortPanel.copyLevelButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:Button')} providing a mechanism for duplicating levels\n in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.copyLevelButtonProperties</code> and \n <code>multiSortPanel.copyLevelButtonDefaults</code>.",
        name:"copyLevelButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "method:ValuesManager.filterData":{
        ref:"method:ValuesManager.filterData",
        flags:"",
        description:" Retrieve data that matches the provided criteria, and edit the first record returned.<br>\n Differs from ${isc.DocUtils.linkForRef('method:DynamicForm.fetchData')} in that a case insensitive substring match\n will be performed against the criteria to retrieve the data.\n \n",
        name:"filterData",
        seeAlso:[],
        params:[
            {description:"search criteria", optional:true, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the DSRequest                                            that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DataSource.resultTreeClass":{
        ref:"attr:DataSource.resultTreeClass",
        flags:"IRA",
        description:"Class for ResultTrees used by this datasource.  If null, defaults to using\n ${isc.DocUtils.linkForRef('class:ResultTree')}.\n <P>\n This can be set to a custom subclass of ResultTree that, for example, hangs on to extra\n information necessary for integration with web services.",
        name:"resultTreeClass",
        seeAlso:[],
        valueType:"Class Object",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "group:visualBuilder":{
        ref:"group:visualBuilder",
        title:"Visual Builder",
        treeLocation:["Concepts"],
        refs:[],
        description:"The SmartClient Visual Builder tool, accessible from the SDK Explorer as Tools->Visual\n Builder, is intended for:\n <ul>\n <li> business analysts and others doing functional application design, who want to create\n functional prototypes in a codeless, \"what you see is what you get\" environment\n <li> developers new to SmartClient who want to get a basic familiarity with component\n layout, component properties and SmartClient code structure\n <li> developers building simple applications that can be completed entirely within Visual\n Builder\n </ul>\n <P>\n <h4>Using Visual Builder</h4>\n <P>\n Basic usage instructions are embedded in Visual Builder itself, in the \"About Visual\n Builder\" pane.  Click on it to open it.\n <P>\n <b>Visual Builder for Functional Design</b>\n <P>\n Visual Builder has several advantages over other tools typically used for functional design:\n <ul>\n <li> Visual Builder allows simple drag and drop manipulation of components, form-based\n editing of component properties, and simple connection of events to actions - all without\n requiring any code to be written.  It is actually simpler to use than\n DreamWeaver or other code-oriented prototyping tools\n <li> because Visual Builder generates clean code, designs will not have to be converted to\n another technology before development can proceed.  This reduces both effort and the\n potential for miscommunication\n <li> developers can add custom skinning, components with custom behaviors, and custom\n DataSources with sample datasets to Visual Builder so that the design environment is an even\n closer match to the final application.  This helps eliminate many types of unimplementable\n designs \n <li> because Visual Builder is built in SmartClient itself, Visual Builder is simply a \n web page, and does not require installation.  Visual Builder can be deployed to \n an internal network to allow teams with a mixture of technical and semi-technical \n users to collaboratively build and share prototypes of SmartClient-based applications.  \n </ul>\n <P>\n <b>Loading and Saving</b>\n <P>\n The \"File\" menu within Visual Builder allows screens to be saved and reloaded for further\n editing.  Saved screens <b>can</b> be edited outside of Visual Builder and successfully\n reloaded, however, as with any design tool that provides a drag and drop, dialog-driven\n approach to screen creation, Visual Builder cannot work with entirely free-form code.  In\n particular, when a screen is loaded and then re-saved:\n <ul>\n <li> any indenting or spacing changes are not preserved \n <li> order of property or method defintions will revert to Visual Builder's default\n <li> while method definitions on components are preserved, any code <b>outside of</b>\n      component definitions will be dropped (in some cases adding such code will cause\n      loading to fail)\n <li> each Canvas-based component will be output separately, in the order these components\n      appear in the project tree, deepest first\n </ul>\n Generally speaking, screen definitions that you edit within Visual Builder should consist of\n purely declarative code.  Rather than appearing in screen definitions, custom components and\n JavaScript libraries should be added to Visual Builder itself via the customization\n facilities described below.\n <P>\n <h4>Installing Visual Builder</h4>\n <P>\n Visual Builder comes already installed and working in the SDK, and can be used from there out\n of the box.  This is the simplest thing to do during initial prototyping.\n <P>\n Further on in the development cycle, it may be advantageous to have Visual Builder available \n outside the SDK, for example in your test environment.  Installing Visual Builder into \n such an environment is very easy:\n <ul>\n <li>Perform a normal installation procedure, as discussed ${isc.DocUtils.linkForRef('group:iscInstall','here')}</li>\n <li>Copy the following .jar files from the SDK <code>lib</code> folder to the target \n <code>WEB-INF/lib</code> folder: \n <ul>\n <li><code>isomorphic_tools.jar</code></li>\n <li><code>isomorphic_sql.jar</code></li>\n <li><code>isomorphic_hibernate.jar</code></li>\n </ul></li>\n <li>Copy the SDK <code>tools</code> folder to the target application root</li>\n <li>Add the following line to the end of <code>WEB-INF/server.properties</code>:\n <ul>\n <li><code>FilesystemDataSource.enabled: true</code></li>\n </ul>\n </li>\n </ul>\n Note that it is safe to include Visual Builder even in a production environment, so long \n as you ensure that the <code>tools</code> folder is protected with any normal HTTP\n authentication/authorization mechanism - for example, an authentication filter.\n <P>\n <h4>Customizing Visual Builder</h4>\n <P>\n The rest of this topic focuses on how Visual Builder can be customized and deployed by\n developers to make it more effective as a functional design tool for a particular\n organization.\n <P>\n <b>Adding Custom DataSources to Visual Builder</b>\n <P>\n DataSources placed in the project dataSources directory ([webroot]/shared/ds by default)\n will be detected by Visual Builder whenever it is started, and appear in the DataSource\n listing in the lower right-hand corner automatically.\n <P>\n If you have created a custom subclass of DataSource (eg, as a base class for several\n DataSources that contact the same web service), you can use it with Visual Builder by:\n <ul>\n <li> creating an XML version of the DataSource using the XML tag &lt;DataSource&gt; and the\n <code>constructor</code> property set to the name of your custom DataSource subclass (as\n described ${isc.DocUtils.linkForRef('group:componentXML')} under the heading <i>Custom Components</i>)\n <li> modifying [webroot]/tools/visualBuilder/globalDependencies.xml to load the JavaScript\n code for your custom DataSource class.  See examples in that file.\n </ul>\n <P>\n <b>Adding Custom Components to Visual Builder</b>\n <P>\n The Component Library on the right hand side of Visual Builder loads component definitions\n from two XML files in the [webroot]/tools/visualBuilder directory: customComponents.xml and\n defaultComponents.xml.  customComponents.xml is empty and is intended for developers to add\n their own components.  defaultComponents.xml can also be customized, but the base version\n will change between SmartClient releases.\n <P>\n As can be seen by looking at defaultComponents.xml, components are specified using a tree\n structure similar to that shown in the \n ${isc.DocUtils.linkForExampleId('treeLoadXML', 'tree XML loading example')}.  The properties that can be set on\n nodes are:\n <ul>\n <li> <code>className</code>: name of the SmartClient Class on which ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')} will be\n called in order to construct the component.  <code>className</code> can be omitted to create\n a folder that cannot be dropped\n <li> <code>title</code>: title for the node\n <li> <code>defaults</code>: an Object specifying defaults to be passed to\n ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')}.\n For example, you could add an \"EditableGrid\" node by using <code>className:\"ListGrid\"</code>\n and specifying:\n <pre>\n &lt;defaults canEdit=\"true\"/&gt;</pre>\n NOTE: if you set any defaults that are not Canvas properties, you need to provide explicit\n type as documented under <i>Custom Properties</i> for ${isc.DocUtils.linkForRef('group:componentXML')}.\n <li> <code>children</code>: components that should appear as children in the tree under this\n node\n <li> <code>icon</code>: icon to show in the Visual Builder component tree (if desired)\n <li> <code>iconWidth/Height/Size</code>: dimensions of the icon in pixels (\"iconSize\" sets\n both)\n <li> <code>showDropIcon</code>: for components that allow children, whether to show a\n special drop icon on valid drop (like ${isc.DocUtils.linkForRef('attr:TreeGrid.showDropIcons')}).\n </ul>\n <P>\n In order to use custom classes in Visual Builder, you must modify\n <code>[webroot]/tools/visualBuilder/globalDependences.xml</code> to include:\n <ul>\n <li> the JavaScript class definition for the custom class (in other words, the\n ${isc.DocUtils.linkForRef('classMethod:isc.defineClass','defineClass()')} call)\n <li> a ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} for the custom component\n </ul>\n See globalDependencies.xml for examples.\n <P>\n <h4>Component Schema and Visual Builder</h4>\n <P>\n When you provide ${isc.DocUtils.linkForRef('group:componentSchema','custom schema')} for a component, Visual Builder\n uses that schema to drive component editing (Component Properties pane) and to drive drag\n and drop screen building functionality.\n <P>\n <b>Component Editing</b>\n <P>\n Newly declared fields will appear in the Component Editor in the \"Other\" category at the\n bottom by default.  You can create your own category by simply setting field.group to the\n name of a new group and using this on multiple custom fields.\n <P>\n The ComponentEditor will pick a FormItem for a custom field by the\n ${isc.DocUtils.linkForRef('type:FormItemType','same rules')} used for ordinary databinding, including the ability to\n set field.editorType to use a custom FormItem.\n <P>\n When the \"Apply\" button is clicked, Visual Builder will look for an appropriate \"setter\n function\" for the custom field, for example, for a field named \"myProp\", Visual Builder will\n look for \"setMyProp\".  The target component will also be ${isc.DocUtils.linkForRef('method:Canvas.redraw','redrawn')}.\n <P>\n <b>Event -&gt; Action Bindings</b>\n <P>\n The Component Properties pane contains an Events tab that allows you wire components events\n to actions on any other component currently in the project.\n <P>\n Events are simply ${isc.DocUtils.linkForRef('group:stringMethods','StringMethods')} defined on the component.  In\n order to be considered events, method definitions must have been added to the class via\n ${isc.DocUtils.linkForRef('classMethod:Class.registerStringMethods')} and either be publicly documented SmartClient methods or,\n for custom classes, have a methods definition in the ${isc.DocUtils.linkForRef('group:componentSchema','component\n schema')}.\n Examples of events are: ${isc.DocUtils.linkForRef('method:ListGrid.recordClick')} and ${isc.DocUtils.linkForRef('method:DynamicForm.itemChange')}.\n <P>\n Actions are methods on any component that have a method definition in the\n ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} and specify action=\"true\".\n <P>\n All available events (stringMethods) on a component are shown in the Events tab of the\n Component Editor.  Clicking the plus (+) sign next to the event name brings up a menu that\n shows a list of all components currently in the project and their available actions.\n Selecting an action from this submenu binds the action to the selected event.  When an event\n is bound to an action in this manner, automatic type matching is performed to pass arguments\n from the event to the action as follows:\n <ul>\n <li>Only non-optional parameters of the action are bound.\n <li>For each non-optional parameter of the action method, every parameter of the\n event method is inspected in order to either directly match the type (for non-object types)\n or to match an isAssignableFrom type check via a SmartClient schema inheritance check.\n <li>The 'type' of a parameter is determined from the type documented in the SmartClient\n reference for built-in components, or from the <code>type</code> atribute on the method\n param in the ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} definition of a custom component.\n <li>When a matching parameter is found, it is assigned to the current slot of the action and\n not considered for further parameter matching.\n <li>The above pattern is repeated until all non-optional parameters are exhausted, all\n event parameters are exhausted, or until no further type matches can be inferred.\n </ul>\n The \"actionBinding\" log category can be enabled in the Developer Console to troubleshoot\n issues with automatic binding for custom methods.\n <P>\n <b>Component Drag and Drop</b>\n <P>\n Visual Builder uses component schema to determine whether a given drop is allowed and what\n methods should be called to accomplish the drop.  For example, any Canvas-based component\n can be dropped on a VLayout because VLayout has a \"members\" field of type \"Canvas\", and an \n ${isc.DocUtils.linkForRef('method:Layout.addMember','addMember()')} function.\n <P>\n Because of these rules, any subclass of Canvas will be automatically eligible to be dropped\n into any container that accepts a Canvas (eg, a Layout or Tab).  Any subclass of a FormItem\n will be, likewise, automatically eligible to be dropped into a DynamicForm.\n <P>\n You can declare custom containment relations, such as a custom class \"Wizard\" that accepts\n instances of the custom class \"Pane\" by simply declaring a\n ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} that says that Wizard has a property called\n \"panes\" of type \"Pane\".  Then, provide methods that allow components to be added and removed:\n <ul>\n <li> for a ${isc.DocUtils.linkForRef('attr:DataSourceField.multiple','multiple')} field, provide \"add\" and \"remove\"\n functions based on the name of the field.  For example, for a field \"panes\" of type \"Pane\",\n provide \"addPane()\" that takes a Pane instance, and \"removePane()\" that takes a pane\n instance or pane ID \n <li> for a singular field (such as ${isc.DocUtils.linkForRef('attr:Canvas.contextMenu')} or ${isc.DocUtils.linkForRef('attr:Tab.pane')}), provide a\n setter method named after the field (eg setContextMenu()) that takes either an instance of\n the component or null for removal\n </ul>\n <P>\n The \"editing\" log category can be enabled in the Developer Console to troubleshoot issues\n with schema-driven drag and drop and automatic lookup of getter/setter and adder/remover\n methods.\n <P>\n <B>NOTE:</B> after modifying component schema, it may be necessary to restart the servlet\n engine and reload Visual Builder\n <P>\n <b>Presenting simplified components</b>\n <P>\n SmartClient components expose many methods and properties.  For some environments, it is\n more appropriate to provide a simplified list of properties, events, and actions on either\n built-in SmartClient components or your custom components.  This can be done by providing a\n custom ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} for an existing component that exposes\n your minimal set.  You also need to provide a trivial subclass of the class you're exposing\n so that it can be instantiated.\n <P>\n For example, let's say you want to make a simplified button called EButton that exposes only\n the 'title' property and the 'click' event of a standard Button.  The following steps will\n accomplish this:\n <p>\n 1. Edit /tools/visualBuilder/customComponents.xml and add a block similar to the following\n to make your custom component appear in the Component Library:\n <pre>\n &lt;PaletteNode&gt;\n     &lt;title&gt;EButton&lt;/title&gt;\n     &lt;className&gt;EButton&lt;/className&gt;\n     &lt;icon&gt;button.gif&lt;/icon&gt;\n &lt;/PaletteNode&gt;\n </pre>\n 2. Next, create a custom schema: /isomorphic/system/schema/EButton.ds.xml as follows:\n <pre>\n &lt;DataSource ID=\"EButton\" inheritsFrom=\"Button\" Constructor=\"EButton\"\n             showLocalFieldsOnly=\"true\" showSuperClassActions=\"false\"\n             showSuperClassEvents=\"false\"&gt;\n 	   &lt;fields&gt;\n         &lt;field name=\"title\"  type=\"HTML\"/&gt;\n     &lt;/fields&gt;\n     &lt;methods&gt;\n         &lt;method name=\"click\"&gt;\n             &lt;description&gt;Fires when this button is clicked.&lt;/description&gt;\n         &lt;/method&gt;\n     &lt;/methods&gt;\n &lt;/DataSource&gt;\n </pre>\n See documentation above and also ${isc.DocUtils.linkForRef('group:componentSchema','component schema')} for what the\n properties above do.\n 3.  Finally, you'll need to define an EButton class as a simple subclass of Button, as\n follows:\n <pre>\n isc.defineClass(\"EButton\", \"Button\");\n </pre>\n To make sure that the Visual Builder will load the above definition, you'll need to place it\n into a JavaScript file being loaded by the Visual Builder.  If you do not already have\n such a file, you can create one and add it to the list of Visual Builder dependencies by\n adding an entry in /tools/visualBuilder/globalDependencies.xml.  See examples in that file\n for specifics.\n <P>\n <h4>Deploying Visual Builder for Functional Designers</h4>\n <P>\n The normal ${isc.DocUtils.linkForRef('group:iscInstall')} deployment instructions apply to Visual Builder <b>except\n that</b> the \"BuiltinRPCs\", which are configured via server.properties, must be enabled\n in order for Visual Builder to load and save files to the SmartClient server.  This also\n means that Visual Builder should only be deployed within trusted environments.  \n <P>\n Note that the Visual Builder provides a \"live\" interface to the provided DataSources.  In\n other words, if a DataSource supports saving and a designer enables inline editing in a grid,\n real saves will be initiated.  The Visual Builder tool should be configured to use the same\n sample data that developers use during development.",
        name:"visualBuilder",
        seeAlso:[],
        type:"group"
    },
    "attr:DynamicForm.canSubmit":{
        ref:"attr:DynamicForm.canSubmit",
        flags:"IRWA",
        description:"Governs whether this form will be used to perform a standard HTML form submission.\n Note that if true, ${isc.DocUtils.linkForRef('method:DynamicForm.submit')} will perform a native HTML submission\n to the specified ${isc.DocUtils.linkForRef('attr:DynamicForm.action')} URL.<br>\n Wherever possible we strongly recommend using the \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')} to send data to\n the server as they provide a far more sophisticated interface, with built in \n options for server validation, required fields, etc.<br>",
        name:"canSubmit",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "method:DataBoundComponent.dropComplete":{
        ref:"method:DataBoundComponent.dropComplete",
        flags:"",
        description:" This method is invoked whenever a drop operation or ${isc.DocUtils.linkForRef('method:DataBoundComponent.transferSelectedData')} \n targetting this component completes.  A drop is considered to be complete when all the client-\n side transfer operations have finished.  This includes any server turnarounds SmartClient \n needs to make to check for duplicate records in the target component; it specifically does \n not include any add or update operations sent to the server for databound components.  If \n you want to be notified when the entire drag operation - including server updates and cache\n synchronization - has completed, override ${isc.DocUtils.linkForRef('method:DataBoundComponent.dragComplete','dragComplete')}\n on the source component.<p>\n There is no default implementation of this method; you are intended to override it if you \n are interested in being notified when drop operations complete.\n\n",
        name:"dropComplete",
        seeAlso:[
            {ref:"method:DataBoundComponent.dragComplete", description:null}
        ],
        params:[
            {description:"The list of records actually transferred to                    this component (note that this is not necessarily the same thing as the\n                    list of records dragged out of the source component because it doesn't\n                    include records that were excluded because of collisions with existing\n                    records)",
             optional:false, name:"transferredRecords", defaultValue:null, type:"List of Records"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["dragging"]
    },
    "attr:Splitbar.gripImgSuffix":{
        ref:"attr:Splitbar.gripImgSuffix",
        flags:"IRA",
        description:"part name for the 'grip' image if ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true",
        name:"gripImgSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"grip\"",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["grip"]
    },
    "method:ResultTree.invalidateCache":{
        ref:"method:ResultTree.invalidateCache",
        flags:"A",
        description:" Manually invalidate this ResultTree's cache.\n <P>\n Generally a ResultTree will observe and incorporate updates to the DataSource that provides it's\n records, but when this is not possible, <code>invalidateCache()</code> allows manual cache\n invalidation.\n <P>\n Components bound to this ResultTree will typically re-request the currently visible portion\n of the dataset, causing the ResultTree to re-fetch data from the server.\n",
        name:"invalidateCache",
        seeAlso:[],
        type:"method",
        definingClass:"class:ResultTree"
    },
    "method:TileLayout.setTileHMargin":{
        ref:"method:TileLayout.setTileHMargin",
        flags:"",
        description:" sets the horizontal margin of tiles\n",
        name:"setTileHMargin",
        seeAlso:[],
        params:[
            {description:"width", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "method:DataSource.compareCriteria":{
        ref:"method:DataSource.compareCriteria",
        flags:"",
        description:" Given two sets of criteria, determine whether they are equivalent, the new criteria is\n guaranteed more restrictive, or the new criteria is not guaranteed more restrictive,\n returning 0, 1 or -1 respectively.\n <P>\n Comparisons between ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} are made via recursively calling\n ${isc.DocUtils.linkForRef('method:Operator.compareCriteria')} for all criteria involved.\n <P>\n For simple ${isc.DocUtils.linkForRef('type:Criteria')}, by default (${isc.DocUtils.linkForRef('type:CriteriaPolicy')}:\"dropOnShortening\"), returns:\n <ul>\n <li> -1 if the new criteria has fewer properties than the old criteria (indicating that it\n      isn't more restrictive)\n <li> -1 if the value for any property in the old criteria is an array and 1) the value for\n      the same property in the new criteria isn't an array, or 2) is an array but\n      of different length, or 3) the arrays do not contain the exact same set of objects\n      (order can be different)\n <li> -1 if the value for any given property in the old criteria is not an array, and the\n      the value for the same property property in the new criteria is different\n <li> -1 if both values for a given property are strings and the new criteria value doesn't\n      contain the old criteria value\n <li> 1 if none of the above are true and, for at least one of the properties, the\n      respective criteria values are both strings, and the old criteria value is a substring\n      of, and is shorter than, the new criteria value\n <li> 0 otherwise (indicating the sets of criteria are equivalent)\n </ul>\n <P>\n For (${isc.DocUtils.linkForRef('type:CriteriaPolicy')}:\"dropOnChange\"), returns:\n <ul>\n <li> -1 if the two sets of criteria have a different number of properties\n <li> -1 if the value for any property in the old criteria is an array and 1) the value for\n      the same property in the new criteria isn't an array, or 2) is an array but\n      of different length, or 3) the arrays do not contain the exact same set of objects\n      (order can be different)\n <li> -1 if the value for any given property in the old criteria is not an array, and the\n      the value for the same property in the new criteria is different\n <li> 0 otherwise (indicating the sets of criteria are equivalent)\n </ul>\n This method is called by ${isc.DocUtils.linkForRef('method:ResultSet.compareCriteria')} to determine whether a change\n in criteria should cause the cache to be invalidated. You may want\n to override this method in order to mimic the filtering behavior that your server performs.\n\n",
        name:"compareCriteria",
        seeAlso:[
            {ref:"type:CriteriaPolicy", description:null}
        ],
        params:[
            {description:"new filter criteria", optional:false, name:"newCriteria", defaultValue:null,
             type:"Criteria"},
            {description:"previous filter criteria", optional:false, name:"oldCriteria",
             defaultValue:null, type:"Criteria"},
            {description:"dataSource request properties", optional:true, name:"requestProperties",
             defaultValue:null, type:"DSRequest Properties"},
            {description:"overrides ${isc.DocUtils.linkForRef('type:CriteriaPolicy')}", optional:true,
             name:"policy", defaultValue:null, type:"string"}
        ],
        returns:{description:"0 if the filters are equivalent, 1 if newCriteria is guaranteed more                      restrictive, and -1 if newCriteria is not guaranteed more restrictive",
         type:"Number"},
        type:"method",
        definingClass:"class:DataSource"
    },
    "method:CubeGrid.closeFacet":{
        ref:"method:CubeGrid.closeFacet",
        flags:"",
        description:"\n  Handler fired when facet is closed\n\n      <br><i>methodType</i> handler\n",
        name:"closeFacet",
        seeAlso:[],
        params:[
            {description:"ID of facet that was closed", optional:false, name:"facetId",
             defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["columnControls"]
    },
    "attr:Layout.placeHolderProperties":{
        ref:"attr:Layout.placeHolderProperties",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:Layout.showDragPlaceHolder','this.showDragPlaceHolder')} is true, this \n properties object can be used to customize the appearance of the placeholder displayed\n when the user drags a widget out of this layout.",
        name:"placeHolderProperties",
        seeAlso:[],
        valueType:"canvas properties",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('dragMove')}"],
        definingClass:"class:Layout",
        groups:["dragdrop"]
    },
    "attr:DynamicForm.autoFetchTextMatchStyle":{
        ref:"attr:DynamicForm.autoFetchTextMatchStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.autoFetchData')} is <code>true</code>, this attribute allows the developer to\n specify a textMatchStyle for the initial ${isc.DocUtils.linkForRef('method:DynamicForm.fetchData')} call.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["databinding"]
    },
    "class:Flashlet":{
        methods:["method:Flashlet.setSrc", "method:Flashlet.getPluginHandle"],
        ref:"class:Flashlet",
        classAttrs:[],
        classMethods:["classMethod:Flashlet.flashAvailable", "classMethod:Flashlet.getFlashVersion"],
        treeLocation:["Optional Modules/Client Bridges"],
        attrs:["attr:Flashlet.name", "attr:Flashlet.params", "attr:Flashlet.classID", 
         "attr:Flashlet.codeBase","attr:Flashlet.pluginsPage", "attr:Flashlet.src"],
        description:"ISC abstraction for Flashlets.",
        name:"Flashlet",
        seeAlso:[],
        type:"class",
        requiresModules:["PluginBridges"]
    },
    "attr:ResultSet.fetchDelay":{
        ref:"attr:ResultSet.fetchDelay",
        flags:"IRWA",
        description:"Delay in milliseconds before fetching rows.\n <P>\n When a get() or getRange() call asked for rows that haven't been loaded, the\n ResultSet will wait before actually triggering the request.  If, during the delay, more\n get() or getRange() calls are made for missing rows, the final fetch to the server will\n reflect the most recently requested rows.\n <P>\n The intent of this delay is to avoid triggering many unnecessary fetches during\n drag-scrolling and similar user interactions.",
        name:"fetchDelay",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching"]
    },
    "method:ValuesManager.setValue":{
        ref:"method:ValuesManager.setValue",
        flags:"",
        description:" Set the value for some field.\n",
        name:"setValue",
        seeAlso:[],
        params:[
            {description:"Which field to set the value for", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"New value for the field.", optional:false, name:"newValue", defaultValue:null,
             type:"any"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "attr:OperationBinding.requiresAuthentication":{
        flags:"IR",
        type:"attr",
        requiresModules:["SCServer"],
        ref:"attr:OperationBinding.requiresAuthentication",
        description:"Whether a user must be authenticated in order to access this operation.  For details of \n what is meant by \"authenticated\", see ${isc.DocUtils.linkForRef('attr:DataSource.requiresAuthentication')}.\n <P>\n To protect access to an entire operationType (eg, all \"fetch\" operations), declare an\n operationBinding with <code>requiresAuthentication=\"true\"</code>, ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType')} set\n to the operationType to be protected, but no ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId')}.  This will then\n prevent access to the \"fetch\" operationType unless another ${isc.DocUtils.linkForRef('class:OperationBinding')}\n declares requiresAuthentication=\"false\" with a specific\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId','operationId')}.",
        name:"requiresAuthentication",
        valueType:"boolean",
        seeAlso:[],
        serverDS:"only",
        defaultValue:"null",
        groups:["auth"],
        definingClass:"class:OperationBinding"
    },
    "method:SectionStack.getExpandedSections":{
        ref:"method:SectionStack.getExpandedSections",
        flags:"",
        description:"\n Returns the list of currently expanded sections.  The list items are section IDs.\n\n",
        name:"getExpandedSections",
        seeAlso:[],
        returns:{description:"list of currently expanded sections", type:"List"},
        type:"method",
        definingClass:"class:SectionStack"
    },
    "method:ColorPicker.getGreen":{
        ref:"method:ColorPicker.getGreen",
        flags:"",
        description:" Returns the Green element of the currently-selected color, as an integer from 0-255\n",
        name:"getGreen",
        seeAlso:[
            {ref:"method:ColorPicker.setGreen", description:null}
        ],
        returns:{description:"green color component", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "attr:StatefulCanvas.radioGroup":{
        ref:"attr:StatefulCanvas.radioGroup",
        flags:"IRWA",
        description:"String identifier for this canvas's mutually exclusive selection group.",
        name:"radioGroup",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state", "event handling"]
    },
    "attr:CubeGrid.facetValueHoverAlign":{
        ref:"attr:CubeGrid.facetValueHoverAlign",
        flags:"IRWA",
        description:"Allows the developer to override the horizontal text alignment of hover tips shown for\n facet values.  If unspecified the hover canvas content alignment will be set by\n <code>this.hoverAlign</code> if specified.",
        name:"facetValueHoverAlign",
        seeAlso:[
            {ref:"attr:Canvas.hoverAlign", description:null}
        ],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:DSRequestModifier.fieldName":{
        ref:"attr:DSRequestModifier.fieldName",
        flags:"IR",
        description:"The name of the field to add or replace on the DSRequest - whether this appears in the DSRequest's\n values or critiera depends on whether this is part of a ${isc.DocUtils.linkForRef('attr:OperationBinding.values')} or\n ${isc.DocUtils.linkForRef('attr:OperationBinding.criteria')} attribute.",
        name:"fieldName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:DSRequestModifier"
    },
    "method:Canvas.mouseStillDown":{
        ref:"method:Canvas.mouseStillDown",
        flags:"A",
        description:"\n Executed repeatedly (every ${isc.DocUtils.linkForRef('attr:Canvas.mouseStillDownDelay')} by default) when the system is idle -\n i.e. not busy running other scripts - and the left mouse button is held down after having been\n pressed in the object. This event is not native to JavaScript, but is provided by the ISC system.\n <p>\n Note: The event handling system waits ${isc.DocUtils.linkForRef('attr:Canvas.mouseStillDownInitialDelay')} before\n calling mouseStillDown for the first time on the widget.  Subsequently the method is called every\n ${isc.DocUtils.linkForRef('attr:Canvas.mouseStillDownDelay')}.  Both attributes are configurable per widget instance.\n <p>\n This method is called only when the left mouse is held down.\n\n",
        name:"mouseStillDown",
        seeAlso:[
            {ref:"attr:Canvas.mouseStillDownInitialDelay", description:null},
            {ref:"attr:Canvas.mouseStillDownDelay", description:null},
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        platformNotes:"Some end user systems only have one mouse button.",
        examples:["${isc.DocUtils.linkForExampleId('customMouseEvents')}"],
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:ColumnTree.autoFetchTextMatchStyle":{
        ref:"attr:ColumnTree.autoFetchTextMatchStyle",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ColumnTree.autoFetchData')} is <code>true</code>, this attribute allows the developer to\n specify a textMatchStyle for the initial ${isc.DocUtils.linkForRef('method:ColumnTree.fetchData')} call.",
        name:"autoFetchTextMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["databinding"]
    },
    "method:DynamicForm.hasFieldErrors":{
        ref:"method:DynamicForm.hasFieldErrors",
        flags:"",
        description:" Does this form currently h ave any validation errors on the field passed in?<br>\n Validation errors are set up automatically by validation, or may be explicitly set via\n ${isc.DocUtils.linkForRef('method:DynamicForm.setErrors')} or ${isc.DocUtils.linkForRef('method:DynamicForm.setFieldErrors')}.\n",
        name:"hasFieldErrors",
        seeAlso:[],
        params:[
            {description:"field to test for validation errors", optional:false, name:"fieldName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true if the form has outstanding errors for the field in question.",
         type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["errors"]
    },
    "attr:OperationBinding.operationId":{
        ref:"attr:OperationBinding.operationId",
        flags:"IR",
        description:"Optional operationId if this DataSource supports two or more variants of one of the basic\n DataSource operations, for instance, a \"fetch\" that uses full text search and a \"fetch\" that\n accepts per-field search criteria.  See ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} for usage.",
        name:"operationId",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:ListGrid.frozenHeaderBaseStyle":{
        ref:"attr:ListGrid.frozenHeaderBaseStyle",
        flags:"IR",
        description:"If this listGrid contains any frozen fields, this property can be used to apply a custom\n headerBaseStyle to the frozen fields set of fields. If unset, the standard headerBaseStyle\n will be used for both frozen and unfrozen cells.",
        name:"frozenHeaderBaseStyle",
        seeAlso:[
            {ref:"attr:ListGrid.headerBaseStyle", description:null},
            {ref:"attr:ListGridField.frozen", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance", "frozenFields"]
    },
    "method:ListGrid.getEventRow":{
        ref:"method:ListGrid.getEventRow",
        flags:"",
        description:" Returns the row number of the most recent mouse event.\n",
        name:"getEventRow",
        seeAlso:[],
        params:[
            {description:"optional y-coordinate to obtain row number, in lieue of the y                        coordinate of the last mouse event",
             optional:true, name:"y", defaultValue:null, type:"number"}
        ],
        returns:{description:"row number, or -2 if beyond last drawn row", type:"number"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:TileGrid.initialCriteria":{
        ref:"attr:TileGrid.initialCriteria",
        flags:"IR",
        description:"Criteria to be used when ${isc.DocUtils.linkForRef('attr:TileGrid.autoFetchData')} is set.",
        name:"initialCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:Calendar.weekViewTitle":{
        ref:"attr:Calendar.weekViewTitle",
        flags:"IR",
        description:"The title for the week view",
        name:"weekViewTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Week\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.baseStyle":{
        ref:"attr:ListGrid.baseStyle",
        flags:"IR",
        getter:["getBaseStyle"],
        description:"The base name for the CSS class applied to cells. This style will have \"Dark\", \n \"Over\", \"Selected\", or \"Disabled\" appended to it according to the state of the cell.",
        name:"baseStyle",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null},
            {ref:"method:ListGrid.getBaseStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"\"cell\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "classMethod:RPCManager.sendProxied":{
        ref:"classMethod:RPCManager.sendProxied",
        flags:"",
        description:" Send an HTTP request to a remote host, potentially through the HttpProxy servlet installed\n on the ISC server.\n <P>\n This API allows contacting services which are hosted on servers other than the origin server\n if the HttpProxy servlet is enabled on the ISC server.\n <P>\n The HttpProxy will be used if the ${isc.DocUtils.linkForRef('attr:RPCRequest.actionURL')} starts with \"http\" and uses a\n hostname other than \"localhost\" or <code>window.location.hostname</code>, or if the port\n number differs, or if <code>request.useHttpProxy</code> is explicitly set.  Otherwise the\n request goes to the origin server (the server that returned the current page).\n <P>\n The ${isc.DocUtils.linkForRef('class:RPCRequest')} properties that will be respected when relaying requests via the\n HttpProxy are: \n ${isc.DocUtils.linkForRef('attr:RPCRequest.actionURL','actionURL')}, ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod','httpMethod')}, \n ${isc.DocUtils.linkForRef('attr:RPCRequest.params','params')}, ${isc.DocUtils.linkForRef('attr:RPCRequest.contentType','contentType')}, \n ${isc.DocUtils.linkForRef('attr:RPCRequest.httpHeaders','httpHeaders')}, and ${isc.DocUtils.linkForRef('attr:RPCRequest.data','data')}.  In this case\n \"data\", if set, will be used as the request body for an HTTP POST.\n <P>\n Higher-level APIs like ${isc.DocUtils.linkForRef('class:DataSource')} or ${isc.DocUtils.linkForRef('class:WebService')} call through this API, and so\n automatically use the HttpProxy if ${isc.DocUtils.linkForRef('attr:DataSource.dataURL')} or\n ${isc.DocUtils.linkForRef('method:WebService.setLocation','webService.location')} is set to a foreign server.\n <P>\n This API is only suitable for direct use when loading unstructured data that will not be\n shown in a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')}.  For a WSDL-described web service, use\n ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')} instead.  For other web services, use a ${isc.DocUtils.linkForRef('class:DataSource')} with\n ${isc.DocUtils.linkForRef('attr:DataSource.dataURL','dataURL')}, and use ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} and\n ${isc.DocUtils.linkForRef('method:DataSource.transformResponse')} as necessary to form requests for the service and\n transform responses for display.\n\n",
        name:"sendProxied",
        seeAlso:[],
        params:[
            {description:"rpcRequest to be routed through the HttpProxy", optional:false,
             name:"request", defaultValue:null, type:"RPCRequest Properties"}
        ],
        type:"classMethod",
        definingClass:"class:RPCManager",
        requiresModules:["SCServer"]
    },
    "attr:Applet.codeBase":{
        ref:"attr:Applet.codeBase",
        flags:"IR",
        description:"The base URL of the applet if the ${isc.DocUtils.linkForRef('attr:Applet.code')} attribute is relative",
        name:"codeBase",
        seeAlso:[
            {ref:"attr:Applet.code", description:null},
            {ref:"attr:Applet.archive", description:null}
        ],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:ListGridField.iconSize":{
        ref:"attr:ListGridField.iconSize",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGridField.icon')} is specified, this property can be used to specify the \n size of the icon to be displayed in the ListGrid header button. \n (See ${isc.DocUtils.linkForRef('attr:StatefulCanvas.iconSize')})",
        name:"iconSize",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null}
        ],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "method:Tree.openFolders":{
        ref:"method:Tree.openFolders",
        flags:"",
        description:"\n Open a set of folders, specified by path or as pointers to nodes.\n\n",
        name:"openFolders",
        seeAlso:[
            {ref:"method:ResultTree.dataArrived", description:null}
        ],
        params:[
            {description:"List of nodes or node paths.", optional:false, name:"nodeList",
             defaultValue:null, type:"List of TreeNode"}
        ],
        type:"method",
        definingClass:"class:Tree"
    },
    "class:SearchForm":{
        methods:["method:SearchForm.search"],
        ref:"class:SearchForm",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms"],
        attrs:["attr:SearchForm.showFilterFieldsOnly"],
        description:"A SearchForm is a DynamicForm specialized for a user to enter search criteria.\n <P>\n All DynamicForm properties and methods work on SearchForm.  SearchForm extends and\n specializes DynamicForm for searching, for example, SearchForm sets\n <code>hiliteRequiredFields</code> false by default because fields are typically required in\n a search.",
        name:"SearchForm",
        seeAlso:[
            {ref:"class:DynamicForm", description:null}
        ],
        type:"class"
    },
    "attr:DataBoundComponent.editSummaryFieldText":{
        ref:"attr:DataBoundComponent.editSummaryFieldText",
        flags:"IRW",
        description:"Text for a menu item allowing users to edit the formatter for a field",
        name:"editSummaryFieldText",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Edit summary format...\"",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["i18nMessages"]
    },
    "attr:DynamicForm.sectionVisibilityMode":{
        ref:"attr:DynamicForm.sectionVisibilityMode",
        flags:"IRW",
        description:"If the form has sections, [implemented as ${isc.DocUtils.linkForRef('class:SectionItem')}s], this attribute controls\n whether multiple sections can be expanded at once.",
        name:"sectionVisibilityMode",
        seeAlso:[
            {ref:"type:VisibilityMode", description:null},
            {ref:"class:SectionItem", description:null}
        ],
        valueType:"VisibilityMode",
        defaultValue:"\"multiple\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "attr:Menu.canSelectParentItems":{
        ref:"attr:Menu.canSelectParentItems",
        flags:"IRW",
        description:"If true, clicking or pressing Enter on a menu item that has a submenu will\n		select that item (with standard behavior of hiding the menus, calling click\n		handlers, etc) instead of showing the submenu.",
        name:"canSelectParentItems",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('treeBinding')}"],
        definingClass:"class:Menu",
        groups:["selection"]
    },
    "method:ListGrid.getCellCSSText":{
        ref:"method:ListGrid.getCellCSSText",
        flags:"",
        description:" Return CSS text for styling this cell, which will be applied in addition to the CSS class\n for the cell, as overrides.\n <p>\n \"CSS text\" means semicolon-separated style settings, suitable for inclusion in a CSS\n stylesheet or in a STYLE attribute of an HTML element.\n\n",
        name:"getCellCSSText",
        seeAlso:[
            {ref:"method:ListGrid.getCellStyle", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS text for this cell", type:"string"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('addStyle')}"],
        definingClass:"class:ListGrid"
    },
    "group:timing":{
        ref:"group:timing",
        refs:[],
        name:"timing",
        seeAlso:[],
        type:"group"
    },
    "group:caching":{
        ref:"group:caching",
        title:"Caching",
        treeLocation:["Optional Modules/Network Performance"],
        refs:[],
        description:"Standard web browsers can cache server responses, associating the locally-cached files with\n the URLs (including query parameters) that were used to fetch the files from the\n server. Each file may be assigned an explicit expiration time. Requests for the associated\n URL will always be served from the local cache, without accessing the server, until the file\n expires.\n <p>\n The recommended approach is to move as much content as possible into cacheable assets\n (these can be images, html, css, and js) and tell the browser to cache those for as long as\n possible (ideally indefinitely).  Clearly, most things can't simply be cached permanently -\n new versions of the application will often require changes to these assets.  To allow for\n this, the pages that direct the loading of the cached assets should be dynamic and should\n create version-specific URLs to these cacheable assets.  This can be done by tacking the\n version number as a query parameter or as a path component.  Here's an example of loading a\n javascript file versioned with a query parameter:\n <pre>\n &lt;script src='/foo/bar.js?version=13'&gt;&lt;/script&gt;\n </pre>\n Generally, the version number wouldn't be hard-coded into the dynamic page, but would\n instead pick up the value of a variable, such that you can simply bump up the value in one\n configuration file and have all versioned URLs change dynamically.\n <p>\n Because SmartClient performs the assembly of image URLs dynamically, versioning of\n SmartClient skins must be done with a path component.  You can do this by changing the base\n skinDir using ${isc.DocUtils.linkForRef('classMethod:Page.setSkinDir')} - e.g:\n <pre>\n isc.Page.setSkinDir('/version/5.6/isomorphic/skins/SmartClient/');\n </pre>\n You can then either deploy the new skins under the versioned directory above or use a URL\n rewriting engine such as mod_rewrite for Apache to map all such versions into a single\n deploy directory.\n <p>\n To actually tell the browser to cache images for a longer length of time than the browser\n session, you need to set the HTTP 'Expires' header.\n If you're not using the SmartClient Java back-end there are several caching solutions\n available, depending on your server of choice.  Microsoft's IIS has built-in caching\n capability, please check the reference manual for details.  If you're using Apache, you can\n use <a href='http://httpd.apache.org/docs/2.0/mod/mod_expires.html' onclick=\"window.open('http://httpd.apache.org/docs/2.0/mod/mod_expires.html');return false;\">mod_expires</a>.  Some servlet\n containers also natively support the setting of caching headers.\n <p>\n The SmartClient Java back-end supports setting caching headers via the FileDownload service\n on a per-mimetype basis.  To use it, first register the FileDownload servlet in your web.xml\n as follows:\n <pre>\n     &lt;servlet&gt;\n       &lt;servlet-name&gt;FileDownload&lt;/servlet-name&gt;\n       &lt;init-param&gt;\n           &lt;param-name&gt;expires&lt;/param-name&gt;\n           &lt;param-value&gt;text/javascript:3600,image/gif:86400&lt;/param-value&gt;\n       &lt;/init-param&gt;\n       &lt;servlet-class&gt;com.isomorphic.servlet.FileDownload&lt;/servlet-class&gt;\n     &lt;/servlet&gt;\n </pre>\n The expires parameter controls the expiration time in seconds.  In the block above,\n javascript files are set to expire in 1 hour and gif images are set to expire in 1 day from\n the time they are served to the browser.  If you don't set explicit expires mappings, all\n images and css files will be set to expire in 1 day and javascript files will expire in 1\n hour, by default.\n <p>\n Next, map any resource that you want to serve with caching headers to the FileDownload\n servlet in your web.xml.  Typically, you'll want to serve the SmartClient modules and all\n skin images with caching headers.  You can do so by adding the following servlet-mapping\n directives to your web.xml:\n <pre>\n     &lt;servlet-mapping&gt;\n       &lt;servlet-name&gt;FileDownload&lt;/servlet-name&gt;\n       &lt;url-pattern&gt;/isomorphic/system/modules/*&lt;/url-pattern&gt;\n     &lt;/servlet-mapping&gt;\n \n     &lt;servlet-mapping&gt;\n       &lt;servlet-name&gt;FileDownload&lt;/servlet-name&gt;\n       &lt;url-pattern&gt;/isomorphic/skins/*&lt;/url-pattern&gt;\n     &lt;/servlet-mapping&gt;\n </pre>",
        name:"caching",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.saveEdits":{
        ref:"method:ListGrid.saveEdits",
        flags:"A",
        description:"\n Validates and saves edits within the row currently being edited (or another row with unsaved\n edits, if indicated).\n <P>\n This method can be called to manually trigger saves if the default mechanisms of\n ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell','cell by cell')} or row by row saving are not suitable.  \n <P>\n The 'callback' parameter provides a notification when the save attempt completes, which is\n likely to be asynchronous for databound grids.  Cases under which the callback will fire are:\n <ul><li>Save completed successfully</li>\n <li>No changes to the edited row, so save not required</li>\n <li>Validation failure occurred on the client or on the server</li></ul>\n Note that if this method was unable to determine the row to be saved, the callback will NOT\n fire - in this case, the method is a no-op.\n <P>\n Other, standard callbacks such as ${isc.DocUtils.linkForRef('method:ListGrid.editComplete')}, ${isc.DocUtils.linkForRef('method:ListGrid.editFailed')} and \n ${isc.DocUtils.linkForRef('method:ListGrid.cellChanged')} will fire normally.\n <P>\n Note this method does not hide the inline editors if they are showing - to explicitly save \n and end editing, use the method 'endEditing()'\n\n",
        name:"saveEdits",
        seeAlso:[
            {ref:"method:ListGrid.endEditing", description:null}
        ],
        params:[
            {description:"Event used to complete cell editing.          Optional, and defaults to <code>\"programmatic\"</code>.  Can be used by the \n          <code>callback</code> method to perform custom actions such as navigation when the \n          save completes.",
             optional:true, name:"editCompletionEvent", defaultValue:null, type:"EditCompletionEvent"},
            {description:"Callback to fire on completion of the saving process.                                    If no edits were made or client-side validation fails \n                                    the callback will be fired synchronously at the end of this\n                                    method.<br>\n                                    Takes the following parameters:\n                                    <br>- rowNum <i>(Number) edited row number</i>\n                                    <br>- colNum <i>(Number) edited column number</i>\n                                    <br>- editCompletionEvent <i>(EditCompletionEvent) event \n                                      passed in (defaults to <code>\"programmatic\"</code>)</i>\n                                    <br>- success <i>(boolean) false if the save was unable to\n                                      complete due to a validation failure or server-side \n                                      error.</i>",
             optional:true, name:"callback", defaultValue:null, type:"Callback"},
            {description:"Which row should be saved. If unspecified the current edit row                            is saved by default. Note that if there is no current edit row \n                            this method will no op.",
             optional:true, name:"rowNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "type:SelectionAppearance":{
        ref:"type:SelectionAppearance",
        values:[
            {description:"selected rows should be shown with different appearance - see                   ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle')} and optionally\n                   ${isc.DocUtils.linkForRef('attr:ListGrid.selectionCanvas')}.",
             value:"\"rowStyle\""},
            {description:"an extra, non-data column should be automatically added to the ListGrid,                   showing checkboxes that can be toggled to select rows.  \n                   See ${isc.DocUtils.linkForRef('method:ListGrid.getCheckboxField')}.",
             value:"\"checkbox\""}
        ],
        description:"How data selection should be presented to the user.",
        name:"SelectionAppearance",
        seeAlso:[],
        type:"type"
    },
    "attr:DynamicForm.cellBorder":{
        ref:"attr:DynamicForm.cellBorder",
        flags:"IRW",
        description:"Width of border for the table that form is drawn in. This is primarily used for debugging\n form layout.",
        name:"cellBorder",
        seeAlso:[],
        valueType:"number",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "attr:BatchUploader.gridFields":{
        ref:"attr:BatchUploader.gridFields",
        flags:"IRW",
        description:"Fields to apply to ${isc.DocUtils.linkForRef('attr:BatchUploader.grid')}.  These will override the field definitions\n in the ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadDataSource')} on a field by field basis, as described under\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}.",
        name:"gridFields",
        seeAlso:[],
        valueType:"Array of ListGridFields",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "classMethod:Canvas.getEventEdge":{
        ref:"classMethod:Canvas.getEventEdge",
        flags:"",
        description:"		Check if an event is within an \"edge\" of this canvas.\n\n",
        name:"getEventEdge",
        seeAlso:[
            {ref:"attr:Canvas.resizeFrom", description:null}
        ],
        params:[
            {description:"Array of legal edges.  Default is all the resizeFrom mask of this canvas.",
             optional:true, name:"edgeMask", defaultValue:null, type:"array"}
        ],
        returns:{description:"\"T\", \"TR\", etc. for corner the event is within, or null if not within a legal edge.",
         type:"string"},
        type:"classMethod",
        definingClass:"class:Canvas",
        groups:["dragdrop", "dragResize"]
    },
    "attr:SelectItem.pickListCriteria":{
        ref:"attr:SelectItem.pickListCriteria",
        flags:"IRWA",
        description:"If this item has a databound pick-list (for example ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')} is\n set), this property can be used to provide static filter criteria when retrieving the data\n for the pickList.",
        name:"pickListCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "class:RPCRequest":{
        methods:[],
        ref:"class:RPCRequest",
        classAttrs:["classAttr:RPCRequest.useCursorTracker"],
        classMethods:["classMethod:RPCRequest.create"],
        treeLocation:["Client Reference/RPC"],
        attrs:["attr:RPCRequest.data", "attr:RPCRequest.actionURL", "attr:RPCRequest.promptStyle", 
         "attr:RPCRequest.promptCursor","attr:RPCRequest.prompt", "attr:RPCRequest.showPrompt", "attr:RPCRequest.callback", 
         "attr:RPCRequest.clientContext","attr:RPCRequest.willHandleError", "attr:RPCRequest.timeout", "attr:RPCRequest.params", 
         "attr:RPCRequest.evalResult","attr:RPCRequest.evalVars", "attr:RPCRequest.callbackParam", 
         "attr:RPCRequest.serverOutputAsString","attr:RPCRequest.transport", "attr:RPCRequest.useXmlHttpRequest", 
         "attr:RPCRequest.httpMethod","attr:RPCRequest.contentType", "attr:RPCRequest.httpHeaders", 
         "attr:RPCRequest.containsCredentials","attr:RPCRequest.ignoreTimeout", "attr:RPCRequest.sendNoQueue", 
         "attr:RPCRequest.paramsOnly","attr:RPCRequest.useSimpleHttp", "attr:RPCRequest.bypassCache", 
         "attr:RPCRequest.omitNullMapValuesInResponse"],
        description:"Encapsulates a client/server RPC request.  You'll need to provide an instance of this class (or a\n constructor for it) to the ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendRequest')} method.  If you use the \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.send')} method, an instance of RPCRequest will be created for you.",
        name:"RPCRequest",
        seeAlso:[
            {ref:"classMethod:RPCManager.send", description:null},
            {ref:"classMethod:RPCManager.sendRequest", description:null}
        ],
        type:"class"
    },
    "method:FormItem.titleHoverHTML":{
        ref:"method:FormItem.titleHoverHTML",
        flags:"A",
        description:"  If defined, this method should return the HTML to display in a hover canvas when the \n  user holds the mousepointer over this item's title.  Return null to suppress the hover \n  canvas altogether.<br>\n  If not defined, <code>dynamicForm.titleHoverHTML()</code> will be evaluated to \n  determine hover content instead.\n\n",
        name:"titleHoverHTML",
        seeAlso:[
            {ref:"attr:FormItem.prompt", description:null},
            {ref:"method:FormItem.titleHover", description:null},
            {ref:"method:FormItem.itemHoverHTML", description:null}
        ],
        params:[
            {description:"Pointer to this item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"This items form", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["Hovers"]
    },
    "attr:DSResponse.totalRows":{
        ref:"attr:DSResponse.totalRows",
        flags:"R",
        description:"Total number of rows available from the server that match the current filter criteria,\n when using paged result fetching.",
        name:"totalRows",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["paging"]
    },
    "attr:OperationBinding.exportResults":{
        ref:"attr:OperationBinding.exportResults",
        flags:"IR",
        description:"When set, causes the results of the DataSource Operation to be exported to a file, whose \n name and format are indicated by ${isc.DocUtils.linkForRef('attr:OperationBinding.exportFilename')} and \n ${isc.DocUtils.linkForRef('attr:OperationBinding.exportAs')} respectively.  When no exportFilename is provided, the \n default is <i>Results</i> and the default value of exportAs is <i>csv</i>.  Once the Operation \n completes, ${isc.DocUtils.linkForRef('attr:DSRequest.exportDisplay')} specifies whether the \n exported data will be downloaded to the file-system or displayed in a new window.   The \n default value of exportDisplay is \"download\" which displays the Save As dialog.  See \n ${isc.DocUtils.linkForRef('type:ExportDisplay')} for more information.\n <P>\n The export field-list can also be configured, see ${isc.DocUtils.linkForRef('attr:DSRequest.exportFields')}.\n <P>\n You can also configure the style of line-breaks to use when generating the output.  See\n ${isc.DocUtils.linkForRef('type:LineBreakStyle')} for more information.\n <P>\n As well as setting this and other properties on the ${isc.DocUtils.linkForRef('class:OperationBinding')}, Exports can be \n initiated in two other ways.  You can set properties on the dsRequest by passing \n <i>requestProperties</i> into ${isc.DocUtils.linkForRef('method:DataSource.exportData')}.  Note that this method does\n not support exporting to JSON format (see \n <a href=\"http://forums.smartclient.com/showthread.php?t=235\">this post</a> for more detail).\n Additionally, custom server code may set export-related properties on the \n ${isc.DocUtils.linkForRef('class:DSResponse')}.\n <P>\n <b>Format Examples</b>\n <P>\n XML format\n <pre>\n     &lt;List&gt;\n         &lt;Object&gt;\n             &lt;id&gt;10101&lt;/id&gt;\n             &lt;displayName&gt;Record 10101&lt;/displayName&gt;\n         &lt;/Object&gt;\n    &lt;/List&gt;\n </pre>\n JSON Format\n <pre>\n     [\n         { id: 10101, displayName: \"Record 10101\" }\n     ]\n </pre>\n CSV Format\n <pre>\n     id,displayName\n     10101,\"Record 10101\"\n </pre>",
        name:"exportResults",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding"
    },
    "attr:TextItem.selectOnFocus":{
        ref:"attr:TextItem.selectOnFocus",
        flags:"IRW",
        description:"Allows the ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus','selectOnFocus')} behavior to be configured on a\n per-FormItem basis.  Normally all items in a form default to the value of\n ${isc.DocUtils.linkForRef('attr:DynamicForm.selectOnFocus')}.",
        name:"selectOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["focus"]
    },
    "method:ListGrid.refreshCellStyle":{
        ref:"method:ListGrid.refreshCellStyle",
        flags:"",
        description:" Refresh the styling of an individual cell without redrawing the grid.\n <P>\n The cell's CSS class and CSS text will be refreshed, to the current values returned by\n getCellStyle() and getCellCSSText() respectively.\n <P>\n The cell's contents (as returned by getCellValue()) will <b>not</b> be refreshed.  To\n refresh both styling and contents, call refreshCell() instead.\n\n",
        name:"refreshCellStyle",
        seeAlso:[
            {ref:"method:ListGrid.refreshCell", description:"to update cell contents too"}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column number of cell to refresh", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "object:ListGridField":{
        methods:["method:ListGridField.getGroupValue", "method:ListGridField.getGroupTitle", 
         "method:ListGridField.getFieldTitle","method:ListGridField.showIf", "method:ListGridField.getGridSummary", 
         "method:ListGridField.getGroupSummary","method:ListGridField.getRecordSummary", "method:ListGridField.getSummaryTitle", 
         "method:ListGridField.sortNormalizer","method:ListGridField.defaultDynamicValue", "method:ListGridField.editorEnter", 
         "method:ListGridField.editorExit","method:ListGridField.cellChanged", "method:ListGridField.recordClick", 
         "method:ListGridField.recordDoubleClick","method:ListGridField.getCellValue", "method:ListGridField.formatCellValue", 
         "method:ListGridField.formatEditorValue","method:ListGridField.parseEditorValue", "method:ListGridField.change", 
         "method:ListGridField.changed","method:ListGridField.hoverHTML", "method:ListGridField.getEditorValueMap"],
        ref:"object:ListGridField",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/ListGrid"],
        attrs:["attr:ListGridField.groupingModes", "attr:ListGridField.groupingMode", 
         "attr:ListGridField.defaultGroupingMode","attr:ListGridField.groupPrecision", "attr:ListGridField.groupGranularity", 
         "attr:ListGridField.canGroupBy","attr:ListGridField.canSortClientOnly", "attr:ListGridField.showDefaultContextMenu", 
         "attr:ListGridField.canExport","attr:ListGridField.valueIcons", "attr:ListGridField.valueIconSize", 
         "attr:ListGridField.valueIconWidth","attr:ListGridField.valueIconHeight", "attr:ListGridField.valueIconLeftPadding", 
         "attr:ListGridField.valueIconRightPadding","attr:ListGridField.editorValueIcons", "attr:ListGridField.editorValueIconWidth", 
         "attr:ListGridField.editorValueIconHeight","attr:ListGridField.showValueIconOnly", "attr:ListGridField.suppressValueIcon", 
         "attr:ListGridField.valueIconOrientation","attr:ListGridField.imageURLPrefix", "attr:ListGridField.imageURLSuffix", 
         "attr:ListGridField.editorImageURLPrefix","attr:ListGridField.editorImageURLSuffix", "attr:ListGridField.imageSize", 
         "attr:ListGridField.imageWidth","attr:ListGridField.imageHeight", "attr:ListGridField.type", "attr:ListGridField.name", 
         "attr:ListGridField.title","attr:ListGridField.target", "attr:ListGridField.frozen", "attr:ListGridField.canFreeze", 
         "attr:ListGridField.canHide","attr:ListGridField.showGridSummary", "attr:ListGridField.showGroupSummary", 
         "attr:ListGridField.summaryFunction","attr:ListGridField.formatGridSummary", "attr:ListGridField.formatGroupSummary", 
         "attr:ListGridField.recordSummaryFunction","attr:ListGridField.partialSummary", "attr:ListGridField.includeInRecordSummary", 
         "attr:ListGridField.includeInRecordSummaryFields","attr:ListGridField.icon", "attr:ListGridField.iconSize", "attr:ListGridField.iconWidth", 
         "attr:ListGridField.iconHeight","attr:ListGridField.iconOrientation", "attr:ListGridField.iconSpacing", 
         "attr:ListGridField.showDisabledIcon","attr:ListGridField.showRollOverIcon", "attr:ListGridField.showFocusedIcon", 
         "attr:ListGridField.showDownIcon","attr:ListGridField.showSelectedIcon", "attr:ListGridField.cellIcon", 
         "attr:ListGridField.showFileInline","attr:ListGridField.icons", "attr:ListGridField.editorIconWidth", 
         "attr:ListGridField.editorIconHeight","attr:ListGridField.defaultIconSrc", "attr:ListGridField.iconVAlign", 
         "attr:ListGridField.summaryTitle","attr:ListGridField.width", "attr:ListGridField.align", 
         "attr:ListGridField.headerBaseStyle","attr:ListGridField.headerTitleStyle", "attr:ListGridField.cellAlign", 
         "attr:ListGridField.baseStyle","attr:ListGridField.canSort", "attr:ListGridField.sortDirection", 
         "attr:ListGridField.canEdit","attr:ListGridField.defaultValue", "attr:ListGridField.enterKeyEditAction", 
         "attr:ListGridField.escapeKeyEditAction","attr:ListGridField.editorType", "attr:ListGridField.editorProperties", 
         "attr:ListGridField.validators","attr:ListGridField.validateOnChange", "attr:ListGridField.required", 
         "attr:ListGridField.displayFormat","attr:ListGridField.inputFormat", "attr:ListGridField.isRemoveField", 
         "attr:ListGridField.canFilter","attr:ListGridField.filterEditorValueMap", "attr:ListGridField.filterEditorType", 
         "attr:ListGridField.defaultFilterValue","attr:ListGridField.filterEditorProperties", "attr:ListGridField.filterOnKeypress", 
         "attr:ListGridField.linkText","attr:ListGridField.valueMap", "attr:ListGridField.editorValueMap", 
         "attr:ListGridField.emptyCellValue","attr:ListGridField.autoFetchDisplayMap", "attr:ListGridField.optionTextMatchStyle", 
         "attr:ListGridField.optionFilterContext","attr:ListGridField.optionOperationId", "attr:ListGridField.optionDataSource", 
         "attr:ListGridField.valueField","attr:ListGridField.displayField", "attr:ListGridField.showHover", 
         "attr:ListGridField.canToggle"],
        description:"An ordinary JavaScript object containing properties that configures the display of\n and interaction with the columns of a ${isc.DocUtils.linkForRef('class:ListGrid')}.",
        name:"ListGridField",
        seeAlso:[
            {ref:"attr:ListGrid.fields", description:null},
            {ref:"method:ListGrid.setFields", description:null}
        ],
        type:"object"
    },
    "attr:RichTextEditor.fontSizes":{
        ref:"attr:RichTextEditor.fontSizes",
        flags:"IRA",
        description:"ValueMap of css font size property values to font size titles to display in the font size\n  selector if <code>\"fontSizeSelector\"</code> is included in \n  ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')}.\n  Default value for this attribute:<br>\n <code>{\n &nbsp;&nbsp;\"1\":\"1 (8 pt)\",\n &nbsp;&nbsp;\"2\":\"2 (10 pt)\",\n &nbsp;&nbsp;\"3\":\"3 (12 pt)\",\n &nbsp;&nbsp;\"4\":\"4 (14 pt)\",\n &nbsp;&nbsp;\"5\":\"5 (18 pt)\",\n &nbsp;&nbsp;\"6\":\"6 (24 pt)\",\n &nbsp;&nbsp;\"7\":\"7 (36 pt)\"}</code>",
        name:"fontSizes",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "attr:DataSource.childrenField":{
        ref:"attr:DataSource.childrenField",
        flags:"IR",
        description:"fieldName for a field in the dataSource expected to contain an explicit array of child nodes.\n Enables loading a databound tree as a hierachical data structure, rather than a flat list of\n nodes linked by foreignKey.<br>\n Note this is an alternative to setting ${isc.DocUtils.linkForRef('attr:DataSourceField.childrenProperty')} directly on\n the childrenField object.<P>\n By default the children field will be assumed to be ${isc.DocUtils.linkForRef('attr:DataSourceField.multiple','multiple')},\n for XML databinding. This implies that child data should be delivered in the format:\n <pre>\n      &lt;childrenFieldName&gt;\n          &lt;item name=\"firstChild\" ...&gt;\n          &lt;item name=\"secondChild\" ...&gt;\n      &lt;/childrenFieldName&gt;\n </pre>\n However data may also be delivered as a direct list of <code>childrenFieldName</code> \n elements:\n <pre>\n      &lt;childrenFieldName name=\"firstChild\" ...&gt;\n      &lt;childrenFieldName name=\"secondChild\" ...&gt;\n </pre>\n If you want to return your data in this format, you will need to explicitly set \n <code>multiple</code> to false in the appropriate dataSource field definition.",
        name:"childrenField",
        seeAlso:[
            {ref:"attr:DataSourceField.childrenProperty", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["dataSourceRelations"]
    },
    "attr:ListGridField.editorImageURLSuffix":{
        ref:"attr:ListGridField.editorImageURLSuffix",
        flags:"IRWA",
        description:"When some cell in this field is being edited, this property can be used to apply \n an explicit ${isc.DocUtils.linkForRef('attr:FormItem.imageURLSuffix')} to the editor in question. \n This can be used to modify the valueIcons within the editor.<br>\n If unset, but ${isc.DocUtils.linkForRef('attr:ListGridField.imageURLPrefix')} is specified, that will be used\n instead.",
        name:"editorImageURLSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "group:formValues":{
        ref:"group:formValues",
        refs:["method:DynamicForm.setValues", "method:DynamicForm.rememberValues", 
         "method:DynamicForm.resetValues","method:DynamicForm.clearValues", "method:DynamicForm.valuesHaveChanged", 
         "method:DynamicForm.getValues","method:DynamicForm.getValuesAsCriteria", "method:DynamicForm.getValuesAsAdvancedCriteria", 
         "method:DynamicForm.getValue","method:DynamicForm.setValue", "method:DynamicForm.showItem", 
         "method:DynamicForm.hideItem","method:DynamicForm.setValueMap", "method:FormItem.getValue", 
         "method:FormItem.defaultDynamicValue","method:ValuesManager.getValues", "method:ValuesManager.setValues", 
         "method:ValuesManager.clearValues","method:ValuesManager.getMemberValues", "method:ValuesManager.setMemberValues", 
         "method:ValuesManager.rememberValues","method:ValuesManager.resetValues", "method:ValuesManager.valuesHaveChanged", 
         "method:ValuesManager.getValue","method:ValuesManager.setValue", "method:ValuesManager.clearValue", 
         "method:DynamicForm.reset","attr:DynamicForm.values", "attr:FormItem.shouldSaveValue"],
        name:"formValues",
        seeAlso:[],
        type:"group"
    },
    "method:Menu.setItemProperties":{
        ref:"method:Menu.setItemProperties",
        flags:"",
        description:" Set arbitrary properties for a particular menu item.\n\n",
        name:"setItemProperties",
        seeAlso:[],
        params:[
            {description:"index of the MenuItem", optional:false, name:"item", defaultValue:null,
             type:"int"},
            {description:"properties to apply to the item", optional:false, name:"properties",
             defaultValue:null, type:"MenuItem Properties"}
        ],
        type:"method",
        definingClass:"class:Menu"
    },
    "attr:DynamicForm.errorsPreamble":{
        ref:"attr:DynamicForm.errorsPreamble",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:DynamicForm.showInlineErrors')} is false we show all errors for the form item in \n a single item rendered at the top of the form.<br>\n This attribute specifies an introductory string rendered out before the individual error\n messages.",
        name:"errorsPreamble",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"The following errors were found\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation", "i18nMessages"]
    },
    "attr:Scrollbar.allowThumbDownState":{
        ref:"attr:Scrollbar.allowThumbDownState",
        flags:"IRA",
        description:"If true, the thumb's appearance changes when it's clicked on.",
        name:"allowThumbDownState",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "attr:Button.showRollOverIcon":{
        ref:"attr:Button.showRollOverIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image on mouse rollover.",
        name:"showRollOverIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "group:treeIcons":{
        ref:"group:treeIcons",
        refs:["method:TreeGrid.setNodeIcon", "attr:TreeNode.icon", "attr:TreeNode.showOpenIcon", 
         "attr:TreeNode.showDropIcon","attr:TreeGrid.iconSize", "attr:TreeGrid.openerIconSize", "attr:TreeGrid.folderIcon", 
         "attr:TreeGrid.dropIconSuffix","attr:TreeGrid.openIconSuffix", "attr:TreeGrid.closedIconSuffix", "attr:TreeGrid.nodeIcon", 
         "attr:TreeGrid.showOpenIcons","attr:TreeGrid.showDropIcons", "attr:TreeGrid.customIconProperty", 
         "attr:TreeGrid.customIconOpenProperty","attr:TreeGrid.customIconDropProperty", "attr:TreeGrid.showCustomIconOpen", 
         "attr:TreeGrid.showCustomIconDrop","attr:TreeGrid.showConnectors", "attr:TreeGrid.showFullConnectors", 
         "attr:TreeGrid.openerImage","attr:TreeGrid.connectorImage", "attr:ColumnTree.folderIcon", 
         "attr:ColumnTree.customIconProperty","attr:ColumnTree.nodeIcon", "attr:ColumnTree.openIconSuffix", 
         "attr:ColumnTree.closedIconSuffix","attr:ColumnTree.showOpenIcons", "attr:ColumnTree.showCustomIconOpen", 
         "attr:ColumnTree.customIconOpenProperty"],
        name:"treeIcons",
        seeAlso:[],
        type:"group"
    },
    "attr:Canvas.defaultHeight":{
        ref:"attr:Canvas.defaultHeight",
        flags:"IRWA",
        description:"For custom components, establishes a default height for the component.\n <P>\n For a component that should potentially be sized automatically by a Layout, set this\n property rather than ${isc.DocUtils.linkForRef('attr:Canvas.height')} directly, because Layouts regard a height setting as\n an explicit size that shouldn't be changed.",
        name:"defaultHeight",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:MenuItem.dynamicTitle":{
        ref:"method:MenuItem.dynamicTitle",
        flags:"",
        description:" Contains the condition that will change the curent items' title when met. The handler must be specified\n as a function or string of script.  \n <p>\n If you don't need to set this state dynamically, use ${isc.DocUtils.linkForRef('attr:MenuItem.title')} instead.\n <p>\n May be defined as a ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')}.\n <p>\n",
        name:"dynamicTitle",
        seeAlso:[],
        params:[
            {description:"${isc.DocUtils.linkForRef('attr:Menu.target','target')} attribute for the top level menu.",
             optional:false, name:"target", defaultValue:null, type:"Canvas"},
            {description:"${isc.DocUtils.linkForRef('class:Menu','menu')} contains the reference to the menu that contains the current item",
             optional:false, name:"menu", defaultValue:null, type:"Menu"},
            {description:"contains the reference to the current item", optional:false, name:"item",
             defaultValue:null, type:"MenuItem"}
        ],
        returns:{description:"the title of this menuItem", type:"String"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('menuDynamicItems')}"],
        definingClass:"object:MenuItem",
        groups:["dynamicMenuItem"]
    },
    "group:grip":{
        ref:"group:grip",
        refs:["attr:StretchImg.showGrip", "attr:Splitbar.showGrip", "attr:Splitbar.gripImgSuffix", 
         "attr:Splitbar.showDownGrip","attr:Splitbar.showRollOverGrip", "attr:Splitbar.showClosedGrip", "attr:Snapbar.showGrip", 
         "attr:Snapbar.showDownGrip","attr:Snapbar.showRollOverGrip", "attr:Snapbar.showClosedGrip"],
        name:"grip",
        seeAlso:[],
        type:"group"
    },
    "method:FilterBuilder.getFieldOperators":{
        ref:"method:FilterBuilder.getFieldOperators",
        flags:"",
        description:" Get the list of ${isc.DocUtils.linkForRef('class:Operators')} that are valid on this field.  By default, all operators\n returned by ${isc.DocUtils.linkForRef('method:DataSource.getFieldOperators')} are used.\n <P>\n Called whenever the fieldName is changed.\n \n",
        name:"getFieldOperators",
        seeAlso:[],
        returns:{description:"valid operators for this field", type:"Array of Operator"},
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "attr:Splitbar.showGrip":{
        ref:"attr:Splitbar.showGrip",
        flags:"IRA",
        description:"Should we show a \"grip\" image floating above the center of this widget?",
        name:"showGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["grip"]
    },
    "class:SectionHeader":{
        methods:[],
        ref:"class:SectionHeader",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/SectionStack"],
        attrs:["attr:SectionHeader.controls", "attr:SectionHeader.controlsLayout", 
         "attr:SectionHeader.icon","attr:SectionHeader.baseStyle"],
        description:"Simple SectionHeader class based on a Label with an icon, skinnable via CSS.",
        name:"SectionHeader",
        seeAlso:[],
        type:"class"
    },
    "attr:Splitbar.hSrc":{
        ref:"attr:Splitbar.hSrc",
        flags:"IR",
        description:"Base URL for the image if ${isc.DocUtils.linkForRef('attr:StretchImg.vertical')} is false and \n ${isc.DocUtils.linkForRef('attr:StretchImg.src')} is unset.",
        name:"hSrc",
        seeAlso:[
            {ref:"attr:StretchImg.src", description:null},
            {ref:"attr:StretchImg.vSrc", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Splitbar",
        groups:["appearance"]
    },
    "attr:SectionStackSection.showHeader":{
        ref:"attr:SectionStackSection.showHeader",
        flags:"I",
        description:"If true, a header will be shown for this section.  If false, no header will be shown.",
        name:"showHeader",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"object:SectionStackSection"
    },
    "method:Menu.getSubmenu":{
        ref:"method:Menu.getSubmenu",
        flags:"A",
        description:" Get the submenu for a particular menu item.\n <P>\n Override to provide dynamic generation of submenus.\n \n",
        name:"getSubmenu",
        seeAlso:[],
        params:[
            {description:"the item in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem | number"}
        ],
        returns:{description:"the submenu", type:"Menu"},
        type:"method",
        definingClass:"class:Menu"
    },
    "method:Calendar.getEventHoverHTML":{
        ref:"method:Calendar.getEventHoverHTML",
        flags:"",
        description:" gets the hover html for an event being hovered over. Override here to return custom \n html based upon the parameter event object.\n\n",
        name:"getEventHoverHTML",
        seeAlso:[],
        params:[
            {description:"The event being hovered", optional:false, name:"event", defaultValue:null,
             type:"Object"},
            {description:"the event window being hovered", optional:false, name:"eventWindow",
             defaultValue:null, type:"EventWindow"}
        ],
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:Button.showDownIcon":{
        ref:"attr:Button.showDownIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the mouse goes\n down on the button.",
        name:"showDownIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('buttonIcons')}"],
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "method:Class.setProperties":{
        ref:"method:Class.setProperties",
        flags:"",
        description:" Set multiple properties on an object, calling the appropriate setter methods if any are\n found.\n <p>\n Whenever you set a property on an ISC component, you should call either the specific setter\n for that property, or <code>setProperty()/setProperties()</code> if it doesn't have one.\n This future-proofs your code against the later addition of required setters.  \n <p>\n With <code>setProperties()</code> in particular, some classes may be able to take shortcuts\n and be more efficient when 2 or more related properties are set at the same time.\n\n",
        name:"setProperties",
        seeAlso:[
            {ref:"method:Class.setProperty", description:null}
        ],
        params:[
            {description:"objects with properties to add (think named parameters).										all the properties of each argument will be applied one\n										after another so later properties will override",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "method:ColumnTree.selectRecords":{
        ref:"method:ColumnTree.selectRecords",
        flags:"",
        description:"\n Select/deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n\n",
        name:"selectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to select", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"},
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "object:MenuItem":{
        methods:["method:MenuItem.enableIf", "method:MenuItem.checkIf", "method:MenuItem.dynamicTitle", 
         "method:MenuItem.dynamicIcon","method:MenuItem.click", "method:MenuItem.action"],
        ref:"object:MenuItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control/Menu"],
        attrs:["attr:MenuItem.title", "attr:MenuItem.submenu", "attr:MenuItem.canSelectParent", 
         "attr:MenuItem.isSeparator","attr:MenuItem.enabled", "attr:MenuItem.checked", "attr:MenuItem.icon", 
         "attr:MenuItem.iconWidth","attr:MenuItem.iconHeight", "attr:MenuItem.keys", "attr:MenuItem.keyTitle"],
        description:"Menu items are specified are object literals, not class instances.  For example, when\n developing in JavaScript, a typical initialization block for a Menu would look like\n this:\n <pre>\n Menu.create({\n     data: [\n         {title: \"item1\", click: \"alert(1)\"},\n         {title: \"item2\"}\n     ]\n });\n </pre>\n And in XML:\n <pre>\n &lt;Menu&gt;\n    &lt;data&gt;\n        &lt;MenuItem title=\"item1\" click=\"alert(1)\"/&gt;\n        &lt;MenuItem title=\"item2\"/&gt;\n    &lt;/data&gt;\n &lt;/Menu&gt;\n </pre>",
        name:"MenuItem",
        seeAlso:[],
        type:"object",
        inheritsFrom:"ListGridRecord"
    },
    "attr:StretchImgButton.state":{
        flags:"IRWA",
        getter:["getState"],
        type:"attr",
        ref:"attr:StretchImgButton.state",
        description:"Current \"state\" of this widget. StatefulCanvases will have a different appearance based\n on their current state. By default this is handled by changing the css className applied to\n the StatefulCanvas - see ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')} for a description of how this is\n done.<P>\n For ${isc.DocUtils.linkForRef('class:Img')} or ${isc.DocUtils.linkForRef('class:StretchImg')} based subclasses of StatefulCanvas, the \n appearance may also be updated by changing the src of the rendered image. See\n ${isc.DocUtils.linkForRef('attr:Img.src')} and ${isc.DocUtils.linkForRef('attr:StretchImgButton.src')} for a description of how the URL \n is modified to reflect the state of the widget in this case.",
        name:"state",
        setter:["setState"],
        valueType:"State",
        seeAlso:[
            {ref:"type:State", description:null},
            {ref:"group:state", description:null}
        ],
        defaultValue:"\"\"",
        groups:["state"],
        definingClass:"class:StretchImgButton"
    },
    "class:Date":{
        methods:["method:Date.duplicate", "method:Date.getShortDayName", "method:Date.getShortMonthName", 
         "method:Date.getShortYear","method:Date.getWeek", "method:Date.toDateStamp", "method:Date.toNormalDate", 
         "method:Date.toShortDate","method:Date.toShortDateTime", "method:Date.setDefaultDateSeparator", 
         "method:Date.getDefaultDateSeparator","method:Date.toUSShortDate", "method:Date.toUSShortDateTime", 
         "method:Date.toEuropeanShortDate","method:Date.toEuropeanShortDateTime", "method:Date.toJapanShortDate", 
         "method:Date.toJapanShortDateTime","method:Date.toSerializeableDate", "method:Date.setFormatter", 
         "method:Date.toPrettyString"],
        ref:"class:Date",
        classAttrs:["classAttr:Date.shortDayNames", "classAttr:Date.shortMonthNames", 
         "classAttr:Date.weekendDays"],
        classMethods:["classMethod:Date.create", "classMethod:Date.compareDates", 
         "classMethod:Date.compareLogicalDates","classMethod:Date.setInputFormat", "classMethod:Date.getInputFormat", 
         "classMethod:Date.parseInput","classMethod:Date.setNormalDisplayFormat", "classMethod:Date.setShortDisplayFormat", 
         "classMethod:Date.setShortDatetimeDisplayFormat"],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Extensions to the Date class, including added static methods on the Date object, and \n  additional instance methods available on all date instances.",
        name:"Date",
        seeAlso:[],
        type:"class"
    },
    "attr:Calendar.showWeekends":{
        ref:"attr:Calendar.showWeekends",
        flags:"IR",
        description:"Suppresses the display of weekend days in the week and month views, and disallows the\n creation of events on weekends.  Which days are considered weekends is controlled by\n ${isc.DocUtils.linkForRef('classAttr:Date.weekendDays')}.",
        name:"showWeekends",
        setter:["setShowWeekends"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["visibility"]
    },
    "method:Slider.setMinValue":{
        ref:"method:Slider.setMinValue",
        flags:"",
        description:" Sets the ${isc.DocUtils.linkForRef('attr:Slider.minValue','minimum value')} of the slider \n\n",
        name:"setMinValue",
        seeAlso:[],
        params:[
            {description:"the new minimum value", optional:false, name:"newValue", defaultValue:null,
             type:"float"}
        ],
        type:"method",
        definingClass:"class:Slider"
    },
    "attr:FormulaBuilder.fields":{
        ref:"attr:FormulaBuilder.fields",
        flags:"IRW",
        description:"Set this to override the underlying set of available fields.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "method:ComboBoxItem.dataArrived":{
        ref:"method:ComboBoxItem.dataArrived",
        flags:"",
        description:" If this item is showing a dataBound pickList, this notification method will be fired \n when new data arrives from the server.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"index of first row returned by the server", optional:false, name:"startRow",
             defaultValue:null, type:"number"},
            {description:"index of last row returned by the server", optional:false, name:"endRow",
             defaultValue:null, type:"number"},
            {description:"pointer to this pickList's data", optional:false, name:"data",
             defaultValue:null, type:"ResultSet"}
        ],
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "classMethod:Flashlet.getFlashVersion":{
        ref:"classMethod:Flashlet.getFlashVersion",
        flags:"",
        description:" Which version of Flash is installed on this browser?\n",
        name:"getFlashVersion",
        seeAlso:[],
        returns:{description:"flash version number, or null if flash is not installed", type:"number"},
        type:"classMethod",
        definingClass:"class:Flashlet"
    },
    "attr:Applet.code":{
        ref:"attr:Applet.code",
        flags:"IR",
        description:"A URL that points to the class of the applet.",
        name:"code",
        seeAlso:[
            {ref:"attr:Applet.codeBase", description:null},
            {ref:"attr:Applet.archive", description:null}
        ],
        valueType:"URL",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "method:ValuesManager.clearFieldErrors":{
        ref:"method:ValuesManager.clearFieldErrors",
        flags:"",
        description:" Clear all validation errors associated with some field in this form\n",
        name:"clearFieldErrors",
        seeAlso:[],
        params:[
            {description:"field for which errors should be cleared", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"if true, and the field is present in one of our member forms,                        redraw it to clear any currently visible validation errors",
             optional:false, name:"show", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "method:TabSet.removeTab":{
        ref:"method:TabSet.removeTab",
        flags:"A",
        description:" Remove a tab.\n <P>\n The pane associated with the removed tab is automatically destroyed when you\n call this method.  To avoid this, call ${isc.DocUtils.linkForRef('method:TabSet.updateTab')} with <code>null</code> as the new\n pane immediately before removing the tab.\n \n",
        name:"removeTab",
        seeAlso:[
            {ref:"method:TabSet.removeTabs", description:null}
        ],
        params:[
            {description:"list of tabs, tabIDs, or tab numbers", optional:false, name:"tabs",
             defaultValue:null, type:"Tab | ID | number | Array of Tab"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('tabsAddAndRemove')}"],
        definingClass:"class:TabSet"
    },
    "attr:FormulaBuilder.mathFunctions":{
        ref:"attr:FormulaBuilder.mathFunctions",
        flags:"IR",
        description:"The list of math functions available in this FormulaBuilder, as an array of \n ${isc.DocUtils.linkForRef('class:MathFunction','MathFunction')} names.\n <P>\n The following function list is supported in FormulaBuilders by default: min(), max(), \n round(), ceil(), floor(), abs(), pow(), sin(), cos(), tan(), ln() and log().",
        name:"mathFunctions",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:ColorPicker.swatchHeight":{
        ref:"attr:ColorPicker.swatchHeight",
        flags:"IR",
        description:"Displayed height of the color swatch image. The default height is approximately\n that used by the Windows&#174; XP color picking window",
        name:"swatchHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"170",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:TabSet.setTabProperties":{
        ref:"method:TabSet.setTabProperties",
        flags:"A",
        description:" Apply properties to an existing tab in a tabSet.\n",
        name:"setTabProperties",
        seeAlso:[],
        params:[
            {description:"Identifier for the tab to be modified", optional:false, name:"tab",
             defaultValue:null, type:"Tab | number | ID"},
            {description:"Javascript object containing the set of properties to be applied  to the tab.",
             optional:false, name:"properties", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:Calendar.detailsButtonTitle":{
        ref:"attr:Calendar.detailsButtonTitle",
        flags:"IR",
        description:"The title for the edit button in the quick event dialog",
        name:"detailsButtonTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Edit Details\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "method:MultiView.anySelected":{
        ref:"method:MultiView.anySelected",
        flags:"",
        description:" Whether at least one item is selected\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:DatabaseBrowser.databaseList":{
        ref:"attr:DatabaseBrowser.databaseList",
        flags:"IR",
        description:"Instance of DynamicForm used to select a database from the list for display in the \n ${isc.DocUtils.linkForRef('attr:DatabaseBrowser.schemaTree','schemaTree')}.",
        name:"databaseList",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "method:Class.getDefaultLogPriority":{
        ref:"method:Class.getDefaultLogPriority",
        flags:"",
        description:" Retrieves the default priority of messages for this class or instance.\n",
        name:"getDefaultLogPriority",
        seeAlso:[],
        returns:{description:"default priority for logging messages on this object.", type:"LogPriority"},
        type:"method",
        definingClass:"class:Class"
    },
    "method:FormItem.defaultDynamicValue":{
        ref:"method:FormItem.defaultDynamicValue",
        flags:"A",
        description:" Expression evaluated to determine the ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue')} when no value is \n provided for this item.\n <P>\n If you don't need dynamic evaluation, you can just use <code>item.defaultValue</code>.\n\n",
        name:"defaultDynamicValue",
        seeAlso:[
            {ref:"attr:FormItem.defaultValue", description:null}
        ],
        params:[
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the current set of values for the form as a whole", optional:false,
             name:"values", defaultValue:null, type:"Object"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["formValues"]
    },
    "classMethod:RPCRequest.create":{
        ref:"classMethod:RPCRequest.create",
        flags:"",
        description:" RPCRequest shouldn't be created directly. Instead, pass ${isc.DocUtils.linkForRef('type:Properties')} to \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.sendRequest')} and ${isc.DocUtils.linkForRef('classMethod:RPCManager.send')}.\n",
        name:"create",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:RPCRequest"
    },
    "object:SerializationContext":{
        methods:[],
        ref:"object:SerializationContext",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        attrs:["attr:SerializationContext.useFlatFields"],
        description:"Flags for XML serialization",
        name:"SerializationContext",
        seeAlso:[],
        type:"object"
    },
    "method:ListGrid.markForRedraw":{
        ref:"method:ListGrid.markForRedraw",
        flags:"",
        description:" Marks the widget as \"dirty\" so that it will be added to a queue for redraw. Redraw of dirty\n components is handled by a looping timer and will after a very short delay (typically less than\n 100ms). In most cases it is recommended that developers use <code>markForRedraw()</code>\n instead of calling ${isc.DocUtils.linkForRef('method:Canvas.redraw')} directly. Since this method queues the redraw, multiple\n calls to markForRedraw() within a single thread of excecution will only lead to a single DOM\n manipulation which greatly improves application performance.\n\n",
        name:"markForRedraw",
        seeAlso:[],
        params:[
            {description:"reason for performing the redraw", optional:true, name:"reason",
             defaultValue:"\"no reason provided\"", type:"string"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["drawing"]
    },
    "method:SelectItem.getDisplayFieldName":{
        ref:"method:SelectItem.getDisplayFieldName",
        flags:"A",
        description:" Returns the ${isc.DocUtils.linkForRef('attr:FormItem.displayField')} for this form item.\n If unset, and ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is explicitly specified, this method\n will return the title field for the <code>optionDataSource</code>\n",
        name:"getDisplayFieldName",
        seeAlso:[],
        returns:{description:"display field name, or null", type:"String"},
        type:"method",
        definingClass:"class:SelectItem"
    },
    "attr:Menu.useKeys":{
        ref:"attr:Menu.useKeys",
        flags:"IRW",
        description:"A boolean indicating whether this menu should use shortcut keys. Set useKeys to\n false in a menu's initialization block to explicitly disable shortcut keys.",
        name:"useKeys",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:ListGrid.cellHoverHTML":{
        ref:"method:ListGrid.cellHoverHTML",
        flags:"",
        description:" StringMethod to dynamically assemble an HTML string to show in a hover window over the\n appropriate cell/record when this.canHover and this.showHover are both true.\n Called when the mouse hovers over a cell.\n\n",
        name:"cellHoverHTML",
        seeAlso:[
            {ref:"attr:ListGrid.canHover", description:null},
            {ref:"attr:ListGrid.showHover", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"the html to be shown inside the hover for this cell", type:"html"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:SliderItem.roundValues":{
        ref:"attr:SliderItem.roundValues",
        flags:"IR",
        description:"Specifies whether the slider value should be rounded to the nearest integer.  If set to\n false, values will be rounded to a fixed number of decimal places controlled by\n ${isc.DocUtils.linkForRef('attr:SliderItem.roundPrecision')}.",
        name:"roundValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"],
        definingClass:"class:SliderItem"
    },
    "classAttr:Dialog.SAY_TITLE":{
        ref:"classAttr:Dialog.SAY_TITLE",
        flags:"IRW",
        description:"Default title for the dialog displayed in response to the ${isc.DocUtils.linkForRef('classMethod:isc.say')} method.\n Note that a custom title can be specified as the <code>title</code> attribute of the \n <code>properties</code> parameter passed to that method.",
        name:"SAY_TITLE",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"\"Note\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "attr:ImgProperties.width":{
        ref:"attr:ImgProperties.width",
        flags:"IRW",
        description:"Specifies the width of the image.",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ImgProperties"
    },
    "attr:DetailViewer.emptyMessageStyle":{
        ref:"attr:DetailViewer.emptyMessageStyle",
        flags:"IRWA",
        description:"CSS style to display this message in",
        name:"emptyMessageStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"normal\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["emptyMessage"]
    },
    "attr:HTMLFlow.allowCaching":{
        ref:"attr:HTMLFlow.allowCaching",
        flags:"IR",
        description:"By default an HTMLFlow will explicitly prevent browser caching.\n <P>\n Set to true to allow browser caching <b>if the browser would normally do so</b>, in other\n words, if the HTTP headers returned with the response indicate that the response can be\n cached.",
        name:"allowCaching",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:HTMLFlow"
    },
    "attr:StaticTextItem.outputAsHTML":{
        ref:"attr:StaticTextItem.outputAsHTML",
        flags:"IRW",
        description:"By default HTML values in a staticTextItem will be interpreted by the browser.\n Setting this flag to true will causes HTML characters to be escaped, meaning the\n raw value of the field (for example <code>\"&lt;b&gt;AAA&lt;/b&gt;\"</code>) is displayed\n to the user rather than the interpreted HTML (for example <code>\"<b>AAA</b>\"</code>)",
        name:"outputAsHTML",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StaticTextItem",
        groups:["appearance"]
    },
    "attr:JSONEncoder.circularReferenceMode":{
        ref:"attr:JSONEncoder.circularReferenceMode",
        flags:"IR",
        description:"What the JSONEncoder should do if it encounters a circular reference.",
        name:"circularReferenceMode",
        seeAlso:[],
        valueType:"JSONCircularReferenceMode",
        defaultValue:"\"path\"",
        type:"attr",
        definingClass:"class:JSONEncoder"
    },
    "attr:ListGrid.data":{
        flags:"IRW",
        type:"attr",
        ref:"attr:ListGrid.data",
        description:"A List of ListGridRecord objects, specifying the data to be used to populate the\n ListGrid.  In ListGrids, the data array specifies rows. Note that ListGrids\n automatically observe changes to the data List and redraw accordingly.\n <p>\n This property is settable directly only as part of a ${isc.DocUtils.linkForRef('class:ListGrid')} constructor.  If\n you want to change the ${isc.DocUtils.linkForRef('class:ListGrid')}'s data after initial creation, call\n ${isc.DocUtils.linkForRef('method:ListGrid.setData')}.\n <p>\n This property will typically not be explicitly specified for databound ListGrids, where\n the data is returned from the server via databound component methods such as\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')}. In this case the data objects will be set to a \n ${isc.DocUtils.linkForRef('class:ResultSet','resultSet')} rather than a simple array.",
        name:"data",
        setter:["setData"],
        valueType:"List of ListGridRecord",
        seeAlso:[
            {ref:"object:ListGridRecord", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('inlineData')}", 
         "${isc.DocUtils.linkForExampleId('localData')}"],
        defaultValue:"null",
        groups:["data"],
        definingClass:"class:ListGrid"
    },
    "group:columnControls":{
        ref:"group:columnControls",
        refs:["method:CubeGrid.closeFacet", "method:CubeGrid.closeColumn"],
        description:null,
        name:"columnControls",
        seeAlso:[],
        type:"group"
    },
    "classMethod:Canvas.getById":{
        ref:"classMethod:Canvas.getById",
        flags:"",
        description:" Retrieve a Canvas by it's global ${isc.DocUtils.linkForRef('attr:Canvas.ID','ID')}.\n",
        name:"getById",
        seeAlso:[],
        params:[
            {description:"global ID of the Canvas", optional:false, name:"ID", defaultValue:null,
             type:"String"}
        ],
        returns:{description:"the Canvas, or null if not found", type:"Canvas"},
        type:"classMethod",
        definingClass:"class:Canvas"
    },
    "attr:Snapbar.showDownGrip":{
        ref:"attr:Snapbar.showDownGrip",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether to show the\n 'Down' state on the grip image when the user mousedown's on this widget. \n Has no effect if ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showDown')} is false",
        name:"showDownGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar",
        groups:["grip"]
    },
    "attr:Tab.closeIcon":{
        ref:"attr:Tab.closeIcon",
        flags:"IRW",
        description:"Custom src for the close icon for this tab to display if it is closeable.\n See ${isc.DocUtils.linkForRef('attr:Tab.canClose')} and ${isc.DocUtils.linkForRef('attr:TabSet.canCloseTabs')}.",
        name:"closeIcon",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab"
    },
    "group:Hovers":{
        ref:"group:Hovers",
        refs:["method:DynamicForm.itemHoverHTML", "method:DynamicForm.titleHoverHTML", 
         "method:FormItem.stopHover","method:FormItem.itemHover", "method:FormItem.titleHover", 
         "method:FormItem.titleHoverHTML","method:FormItem.itemHoverHTML", "attr:DynamicForm.itemHoverDelay", 
         "attr:DynamicForm.itemHoverWidth","attr:DynamicForm.itemHoverHeight", "attr:DynamicForm.itemHoverAlign", 
         "attr:DynamicForm.itemHoverVAlign","attr:DynamicForm.itemHoverStyle", "attr:DynamicForm.itemHoverOpacity", 
         "attr:FormItem.hoverDelay","attr:FormItem.hoverWidth", "attr:FormItem.hoverHeight", "attr:FormItem.hoverAlign", 
         "attr:FormItem.hoverVAlign","attr:FormItem.hoverStyle", "attr:FormItem.hoverOpacity"],
        name:"Hovers",
        seeAlso:[],
        type:"group"
    },
    "attr:Window.animateMinimize":{
        ref:"attr:Window.animateMinimize",
        flags:"IRWA",
        description:"Should this window minimize, maximize, and restore as an animation, or as a \n simple 1-step transition?",
        name:"animateMinimize",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('windowMinimize')}"],
        definingClass:"class:Window",
        groups:["appearance", "header", "animation"]
    },
    "method:Canvas.setWidth":{
        ref:"method:Canvas.setWidth",
        flags:"",
        description:" Resizes the widget horizontally to the specified width (moves the right side of the\n widget). The width parameter can be expressed as a percentage of viewport size or as\n the number of pixels.\n <P>\n NOTE: if you're setting multiple coordinates, use resizeTo() or setRect() instead\n    \n",
        name:"setWidth",
        seeAlso:[],
        params:[
            {description:"new width", optional:false, name:"width", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:OperationBinding.recordName":{
        flags:"IR",
        type:"attr",
        ref:"attr:OperationBinding.recordName",
        description:"For an XML DataSource, tagName of the elements to be used as records.\n <p>\n This is a simple alternative to ${isc.DocUtils.linkForRef('attr:OperationBinding.recordXPath')} when the elements to be used as\n records all share a tagName.\n <p>\n When a DataSource has a WebService, <code>recordName</code> can also be set to the name\n of any <code>complexType</code> declared within the WebService's WSDL file.",
        name:"recordName",
        valueType:"String",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('wsdlBinding')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["clientDataIntegration"],
        definingClass:"class:OperationBinding"
    },
    "attr:Layout.resizeBarClass":{
        ref:"attr:Layout.resizeBarClass",
        flags:"AIRW",
        description:"Class to use for creating resizeBars.\n <P>\n A resize bar will be created for any Layout member that specifies\n ${isc.DocUtils.linkForRef('attr:Canvas.showResizeBar','<code>showResizeBar:true</code>')}.\n Resize bars will be instances of the class specified by this property, and will \n automatically be sized to the member's breadth and to the thickness\n given by ${isc.DocUtils.linkForRef('attr:Layout.resizeBarSize')}.<br>\n Classes that are valid by default are ${isc.DocUtils.linkForRef('class:Splitbar')} and ${isc.DocUtils.linkForRef('class:ImgSplitbar')}.\n <P>\n To customize the appearance or behavior of resizeBars within some layout a custom \n resize bar class can be created by subclassing ${isc.DocUtils.linkForRef('class:Splitbar')} or ${isc.DocUtils.linkForRef('class:ImgSplitbar')} and\n setting this property on your layout to use your new class.\n <P>\n Resize bars will automatically be sized to the member's breadth and to the thickness\n given by <code>layout.resizeBarSize</code>.  The built-in Splitbar class supports\n drag resizing of its target member, and clicking on the bar to hide the target member.",
        name:"resizeBarClass",
        seeAlso:[
            {ref:"class:Splitbar", description:null},
            {ref:"class:ImgSplitbar", description:null},
            {ref:"attr:Layout.resizeBarSize", description:null}
        ],
        valueType:"String",
        defaultValue:"\"Splitbar\"",
        type:"attr",
        definingClass:"class:Layout"
    },
    "attr:ListGrid.canEdit":{
        ref:"attr:ListGrid.canEdit",
        flags:"IRW",
        description:"Can the user edit cells in this listGrid? Can be set for the listGrid, and overridden for \n      individual fields.<br>\n      If 'canEdit' is false at the listGrid level, fields can never be edited - in this case\n      the canEdit property on individual fields will be ignored.<br>\n      If 'canEdit' is set to true at the listGrid level, setting the 'canEdit' property to\n      false at the field level will prevent the field from being edited inline.<br>\n      If 'canEdit' is not set at the listGrid level, setting 'canEdit' to true at the field \n      level enables the field to be edited inline.",
        name:"canEdit",
        seeAlso:[
            {ref:"method:ListGrid.startEditing", description:null},
            {ref:"attr:ListGridField.canEdit", description:null},
            {ref:"attr:ListGrid.recordEditProperty", description:null},
            {ref:"method:ListGrid.canEditCell", description:null},
            {ref:"attr:ListGrid.fields", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('editByRow')}"],
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "classMethod:Class.fireCallback":{
        ref:"classMethod:Class.fireCallback",
        flags:"",
        description:"\n Fire some arbitrary action specified as a ${isc.DocUtils.linkForRef('type:Callback')}.\n Returns the value returned by the action.\n \n",
        name:"fireCallback",
        seeAlso:[],
        params:[
            {description:"Action to fire.", optional:false, name:"callback", defaultValue:null,
             type:"Callback"},
            {description:"Comma separated string of variable names. If the callback                            passed in was a string of script, any arguments passed to the\n                            callback will be available as local variables with these names.",
             optional:true, name:"argNames", defaultValue:null, type:"string"},
            {description:"Array of arguments to pass to the mthod. Note that the number                           of arguments should match the number of argNames.",
             optional:true, name:"args", defaultValue:null, type:"array"},
            {description:"If specified the callback will be evaluated in the scope of this                          object - the <code>this</code> keyword will be a pointer to this\n                          target when the callback is fired.",
             optional:true, name:"target", defaultValue:null, type:"object"}
        ],
        returns:{description:"returns the value returned by the callback method passed in.", type:"any"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:Calendar.descriptionField":{
        ref:"attr:Calendar.descriptionField",
        flags:"IR",
        description:"The name of the description field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"descriptionField",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"description\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "attr:MultiFileItem.dataSource":{
        ref:"attr:MultiFileItem.dataSource",
        flags:"IR",
        description:"DataSource where files are stored.  \n <P>\n This DataSource is expected to have a field of type \"binary\" as well as a primaryKey and\n foreignKey declaration to some other DataSource; see the ${isc.DocUtils.linkForRef('class:MultiFileItem')} for an\n overview.\n <P>\n This DataSource need only be capable of \"fetch\", \"add\" and \"remove\" - \"update\" is unused.",
        name:"dataSource",
        seeAlso:[],
        valueType:"DataSource or ID",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiFileItem"
    },
    "attr:Calendar.canEditEvents":{
        ref:"attr:Calendar.canEditEvents",
        flags:"IR",
        description:"If true, users can edit existing events",
        name:"canEditEvents",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["allowedOperations"]
    },
    "method:DynamicForm.setFields":{
        ref:"method:DynamicForm.setFields",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('attr:DynamicForm.fields','items')} for this DynamicForm.  Takes an array of item\n definitions, which will be converted to ${isc.DocUtils.linkForRef('class:FormItem')}s and displayed in the form.\n <P>\n <span class=\"SmartClient\">\n Note: Do not attempt to create ${isc.DocUtils.linkForRef('class:FormItem')} instances directly. This method should be\n passed the raw properties for each item only.\n </span>\n <P>\n Objects passed to <code>setFields()</code> may not be reused in other forms and may not be\n used in subsequent calls to <code>setFields()</code> with the same form, new objects must be\n created instead.\n <P>\n To create a form where some items are conditionally present, rather than repeated calls to\n <code>setFields()</code> or <code>setItems()</code>, you should generally use\n ${isc.DocUtils.linkForRef('method:FormItem.hide')} and ${isc.DocUtils.linkForRef('method:FormItem.show')} and/or ${isc.DocUtils.linkForRef('method:FormItem.showIf')} rather than\n calling <code>setItems() or setFields()</code>.  <code>setItems()</code> and\n <code>setFields()</code> are appropriate for dynamically generated forms where there are\n few if any items that are the same each time the form is used.\n\n",
        name:"setFields",
        seeAlso:[],
        params:[
            {description:"list of new items to show in the form", optional:false, name:"itemList",
             defaultValue:null, type:"Array of FormItem Properties"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["elements"]
    },
    "attr:Menu.iconHeight":{
        ref:"attr:Menu.iconHeight",
        flags:"IRW",
        description:"The default height applied to custom icons in this menu. This is used whenever\n          item.iconHeight is not specified.",
        name:"iconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Menu"
    },
    "class:ToolStrip":{
        methods:[],
        ref:"class:ToolStrip",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:ToolStrip.members", "attr:ToolStrip.height", "attr:ToolStrip.styleName", 
         "attr:ToolStrip.vertical","attr:ToolStrip.resizeBarClass", "attr:ToolStrip.resizeBarSize", 
         "attr:ToolStrip.separatorClass","attr:ToolStrip.separatorSize"],
        description:"Base class for creating toolstrips like those found in browsers and office applications: a\n mixed set of controls including ${isc.DocUtils.linkForRef('class:ImgButton','icon buttons')}, \n ${isc.DocUtils.linkForRef('attr:Button.radioGroup','radio button groups')}, ${isc.DocUtils.linkForRef('class:MenuButton','menus')},\n ${isc.DocUtils.linkForRef('class:ComboBoxItem','comboBoxes')}, ${isc.DocUtils.linkForRef('class:LayoutSpacer','spacers')}, ${isc.DocUtils.linkForRef('class:Label','status displays')} and \n ${isc.DocUtils.linkForRef('class:SelectItem','drop-down selects')}.  \n <P>\n All of the above components are placed in the ${isc.DocUtils.linkForRef('attr:ToolStrip.members','members array')} to form\n a ToolStrip.  Note that the ${isc.DocUtils.linkForRef('class:FormItem','FormItems')} mentioned above (ComboBox and\n drop-down selects) need to be placed within a ${isc.DocUtils.linkForRef('class:DynamicForm')} as usual.\n <P>\n The special strings \"separator\" and \"resizer\" can be placed in the members array to create\n separators and resizers respectively.\n <P>\n Also see the ${isc.DocUtils.linkForExampleId('toolstrip')} example in the Feature Explorer.",
        name:"ToolStrip",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('toolstrip')}"]
    },
    "method:PanelHeader.iconClick":{
        ref:"method:PanelHeader.iconClick",
        flags:"",
        description:" If this button is showing an ${isc.DocUtils.linkForRef('panelHeader.icon','icon')}, a separate click\n handler for the icon may be defined as <code>this.iconClick</code>.\n Returning false will suppress the standard button click handling code.\n",
        name:"iconClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:PanelHeader",
        groups:["buttonIcon"]
    },
    "method:ListGrid.sort":{
        ref:"method:ListGrid.sort",
        flags:"",
        description:" Sorts a listGrid explicitly by the column given by sortFieldNum, if provided, in \n ascending or descending order if specified by sortDirection. If sortFieldNum is not\n provided and listGrid.sortFieldNum is undefined, the data will be sorted by the first\n sortable column according to ${isc.DocUtils.linkForRef('attr:ListGridField.sortDirection')} if specified, or\n ${isc.DocUtils.linkForRef('attr:ListGrid.sortDirection')}.\n <P>\n Updates the sortFieldNum and sortDirection to reflect the new sort order of the grid.\n\n",
        name:"sort",
        seeAlso:[
            {ref:"type:SortDirection", description:null}
        ],
        params:[
            {description:"the field name or column number to sort by", optional:true, name:"sortField",
             defaultValue:null, type:"string | number"},
            {description:"the direction to sort in", optional:true, name:"sortDirection",
             defaultValue:null, type:"SortDirection"}
        ],
        returns:{description:"sorting worked", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["sorting"]
    },
    "type:DSOperationType":{
        ref:"type:DSOperationType",
        values:[
            {description:"Fetch one or more records that match a set of search criteria.",
             value:"\"fetch\""},
            {description:"Store new records", value:"\"add\""},
            {description:"Update an existing record", value:"\"update\""},
            {description:"Remove (delete) an existing record", value:"\"remove\""},
            {description:"Perform some arbitrary custom logic", value:"\"custom\""}
        ],
        description:"One of the four basic operations that can be performed on DataSource data: \"fetch\",\n \"add\", \"update\", \"remove\".  Elsewhere called CRUD operations, where CRUD stands for\n \"create\", \"retrieve\", \"update\", \"delete\", which correspond to \"add\", \"fetch\", \"update\" and\n \"remove\" in SmartClient terminology.  See ${isc.DocUtils.linkForRef('group:dataSourceOperations')} for a full\n description.\n <p>\n There is also a fifth operation, \"custom\".  This is any operation that is not one of the \n four CRUD operations.  You can use operations of this type in circumstances where you \n might otherwise have used a plain RPC.",
        name:"DSOperationType",
        seeAlso:[],
        type:"type"
    },
    "group:selection":{
        ref:"group:selection",
        title:"Selection",
        refs:["method:CubeGrid.getEventRow", "method:CubeGrid.getEventColumn", 
         "method:CubeGrid.facetValueSelectionChanged","method:CubeGrid.getSelectedCells", "method:CubeGrid.getSelectedCellIds", 
         "method:CubeGrid.anyCellSelected","method:CubeGrid.cellIsSelected", "method:CubeGrid.selectCell", 
         "method:CubeGrid.deselectCell","method:CubeGrid.selectCells", "method:CubeGrid.deselectCells", 
         "method:CubeGrid.selectAllCells","method:CubeGrid.deselectAllCells", "method:CubeGrid.getSelectedFacetValues", 
         "method:CubeGrid.facetValuesSelected","method:CubeGrid.facetHasSelection", "method:CubeGrid.getFacetsHavingSelection", 
         "method:CubeGrid.selectFacetValues","method:CubeGrid.deselectFacetValues", "method:CubeGrid.selectFacetValue", 
         "method:CubeGrid.deselectFacetValue","method:CubeGrid.selectAllFacetValues", "method:CubeGrid.deselectAllFacetValues", 
         "method:CubeGrid.deselectAll","method:CubeGrid.cellSelectionChanged", "method:Selection.isSelected", 
         "method:Selection.anySelected","method:Selection.multipleSelected", "method:Selection.getSelection", 
         "method:Selection.getSelectedRecord","method:Selection.setSelected", "method:Selection.select", "method:Selection.deselect", 
         "method:Selection.selectSingle","method:Selection.selectList", "method:Selection.deselectList", 
         "method:Selection.selectAll","method:Selection.deselectAll", "method:Selection.selectItem", 
         "method:Selection.deselectItem","method:Selection.selectRange", "method:Selection.deselectRange", 
         "method:GridRenderer.cellIsEnabled","method:GridRenderer.getEventRow", "method:GridRenderer.getEventColumn", 
         "method:GridRenderer.getNearestRowToEvent","method:GridRenderer.getNearestColToEvent", "method:GridRenderer.selectionChanged", 
         "method:GridRenderer.cellSelectionChanged","method:ListGrid.selectRecord", "method:ListGrid.deselectRecord", 
         "method:ListGrid.selectRecords","method:ListGrid.deselectRecords", "method:ListGrid.selectAllRecords", 
         "method:ListGrid.deselectAllRecords","method:ListGrid.anySelected", "method:ListGrid.selectSingleRecord", 
         "method:ListGrid.selectionChanged","method:TileGrid.getSelection", "method:TileGrid.getSelectedRecord", 
         "method:TileGrid.selectRecord","method:TileGrid.deselectRecord", "method:TileGrid.selectRecords", 
         "method:TileGrid.deselectRecords","method:TileGrid.selectAllRecords", "method:TileGrid.deselectAllRecords", 
         "method:TileGrid.anySelected","method:TileGrid.selectionChanged", "method:ColumnTree.selectRecord", 
         "method:ColumnTree.deselectRecord","method:ColumnTree.selectRecords", "method:ColumnTree.deselectRecords", 
         "method:ColumnTree.selectAllRecords","method:ColumnTree.deselectAllRecords", "method:ColumnTree.anySelected", 
         "method:CellSelection.cellIsSelected","method:CellSelection.anySelected", "method:CellSelection.getSelectedCells", 
         "method:CellSelection.selectCell","method:CellSelection.deselectCell", "method:CellSelection.selectCellList", 
         "method:CellSelection.deselectCellList","method:CellSelection.selectSingleCell", "method:ListGrid.getSelection", 
         "method:ListGrid.getSelectedRecord","method:DataBoundComponent.selectRecord", "method:DataBoundComponent.deselectRecord", 
         "method:DataBoundComponent.selectRecords","method:DataBoundComponent.deselectRecords", "method:DataBoundComponent.selectAllRecords", 
         "method:DataBoundComponent.deselectAllRecords","method:DataBoundComponent.anySelected", "method:Toolbar.selectButton", 
         "method:Toolbar.deselectButton","attr:PickTreeItem.canSelectParentItems", "attr:ListGrid.selection", 
         "attr:ListGrid.selectionAppearance","attr:ListGrid.canSelectAll", "attr:ListGrid.selectionType", "attr:ListGrid.canDragSelect", 
         "attr:ListGrid.canDragSelectText","attr:Menu.canSelectParentItems", "attr:TileGrid.selectionType", "type:SelectionStyle"],
        description:"APIs for marking ${isc.DocUtils.linkForRef('object:Record')}s as selected and retrieving the selected record or records.\n <P>\n Only applicable to a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} that manages a list of Records, or manages a\n data model that can be viewed as a list (for example, the current list of visible nodes\n on a tree can be treated as a list for selection purposes).",
        name:"selection",
        seeAlso:[],
        type:"group"
    },
    "method:TextAreaItem.selectValue":{
        ref:"method:TextAreaItem.selectValue",
        flags:"",
        description:" Put focus in this item and select the entire value.\n Only applies to text based items\n",
        name:"selectValue",
        seeAlso:[],
        type:"method",
        definingClass:"class:TextAreaItem"
    },
    "group:timer":{
        ref:"group:timer",
        refs:[],
        name:"timer",
        seeAlso:[],
        type:"group"
    },
    "attr:DynamicForm.action":{
        ref:"attr:DynamicForm.action",
        flags:"IRW",
        description:"The URL to which the form will submit its values.\n <p>\n <b>NOTE:</b> this is used only in the very rare case that a form is used to submit data\n directly to a URL.  Normal server contact is through RPCManager.<br>\n See ${isc.DocUtils.linkForRef('attr:DynamicForm.canSubmit')} for more on this.",
        name:"action",
        setter:["setAction"],
        seeAlso:[
            {ref:"group:operations", description:null},
            {ref:"class:RPCManager", description:null}
        ],
        valueType:"string",
        defaultValue:"\"#\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "attr:Canvas.overflow":{
        ref:"attr:Canvas.overflow",
        flags:"IRW",
        description:"Controls what happens when the drawn size of the content of a Canvas is either\n			greater or smaller than the specified size of the Canvas.  Similar to the CSS\n			property overflow, but consistent across browsers.  See Overflow type for\n			details.",
        name:"overflow",
        setter:["setOverflow"],
        seeAlso:[],
        valueType:"Overflow",
        defaultValue:"Canvas.VISIBLE",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:Calendar.dayBodyClick":{
        ref:"method:Calendar.dayBodyClick",
        flags:"",
        description:" Called when the body area of a day in the month view is clicked on, outside of any links\n to a particular event.\n <P>\n By default, if the user can add events, shows a dialog for adding a new event for that\n day.  Return false to cancel this action.\n <P>\n Not called if the day falls outside the current month and ${isc.DocUtils.linkForRef('attr:Calendar.showOtherDays')} is false.\n\n",
        name:"dayBodyClick",
        seeAlso:[],
        params:[
            {description:"JavaScript Date object representing this day", optional:false, name:"date",
             defaultValue:null, type:"Date"},
            {description:"events that fall on this day", optional:false, name:"events",
             defaultValue:null, type:"Array of CalendarEvent"},
            {description:"the calendar itself", optional:false, name:"calendar", defaultValue:null,
             type:"Calendar"},
            {description:"the row number to which the parameter date belongs", optional:false,
             name:"rowNum", defaultValue:null, type:"Integer"},
            {description:"the column number to which the parameter date belongs", optional:false,
             name:"colNum", defaultValue:null, type:"Integer"}
        ],
        returns:{description:"false to cancel the default action", type:"boolean"},
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "attr:OperationBinding.defaultParams":{
        ref:"attr:OperationBinding.defaultParams",
        flags:"IRA",
        description:"HTTP parameters that should be submitted with every DSRequest.\n <P>\n Useful for authenticated services that require a sessionId with every request.\n <P>\n Can be set for all operations of a given DataSource as DataSource.defaultParams.",
        name:"defaultParams",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:DSRequest.textMatchStyle":{
        ref:"attr:DSRequest.textMatchStyle",
        flags:"I",
        description:"For \"fetch\" operations, how search criteria should be interpreted for text fields: either\n \"exact\" for exact match, \"startsWith\" for matching at the beginning only, or \"substring\" for\n case-insensitive substring match.\n <p>\n This setting is respected by the built-in SQLDataSource.  Your custom DataSource\n implementation can interpret the search criteria passed into \"fetch\" operations in arbitrary\n ways; you can safely ignore this flag and use others of your own devising.",
        name:"textMatchStyle",
        seeAlso:[],
        valueType:"TextMatchStyle",
        defaultValue:"\"exact\"",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:ListGrid.groupNodeStyle":{
        ref:"attr:ListGrid.groupNodeStyle",
        flags:"IRW",
        description:"The CSS style that group rows will have",
        name:"groupNodeStyle",
        seeAlso:[
            {ref:"group:grouping", description:null}
        ],
        valueType:"String",
        defaultValue:"\"groupNode\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "method:ListGridField.recordDoubleClick":{
        ref:"method:ListGridField.recordDoubleClick",
        flags:"",
        description:"\n Executed when this field is double-clicked.  Note that if\n ${isc.DocUtils.linkForRef('method:ListGrid.recordDoubleClick')} is also defined, it will be fired for fields that define\n a recordDoubleClick handler if the field-level handler returns true. Return false to prevent\n the grid-level handler from firing.\n\n\n",
        name:"recordDoubleClick",
        seeAlso:[
            {ref:"method:ListGrid.recordClick", description:null}
        ],
        params:[
            {description:"the listGrid that contains doubleclick event", optional:false, name:"viewer",
             defaultValue:null, type:"ListGrid"},
            {description:"the record that was double-clicked", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"number of the record clicked on in the current set of                                  displayed records (starts with 0)",
             optional:false, name:"recordNum", defaultValue:null, type:"number"},
            {description:"the field that was clicked on (field definition)", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"number of the field clicked on in the listGrid.fields                                  array",
             optional:false, name:"fieldNum", defaultValue:null, type:"number"},
            {description:"value of the cell (after valueMap, etc. applied)", optional:false,
             name:"value", defaultValue:null, type:"object"},
            {description:"raw value of the cell (before valueMap, etc applied)", optional:false,
             name:"rawValue", defaultValue:null, type:"object"}
        ],
        returns:{description:"false to stop event bubbling", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('recordClicks')}"],
        definingClass:"object:ListGridField",
        groups:["events"]
    },
    "attr:FormItem.errorIconHeight":{
        ref:"attr:FormItem.errorIconHeight",
        flags:"IRW",
        description:"Height of the error icon, if we're showing icons when validation errors occur.",
        name:"errorIconHeight",
        seeAlso:[
            {ref:"attr:FormItem.showErrorIcon", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["errorIcon"]
    },
    "attr:SelectItem.pickerIconStyle":{
        ref:"attr:SelectItem.pickerIconStyle",
        flags:"IRW",
        description:"Base CSS class name for a form item's picker icon cell. If unset inherits from \n this items <code>controlStyle</code>.",
        name:"pickerIconStyle",
        seeAlso:[
            {ref:"attr:SelectItem.controlStyle", description:null}
        ],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"selectItemPickerIcon\", [IRA]",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["appearance"]
    },
    "method:ResultSet.indexOf":{
        ref:"method:ResultSet.indexOf",
        flags:"",
        description:" Return the position in the list of the first instance of the specified object.\n <p>\n If pos is specified, starts looking after that position.\n <p>\n Returns -1 if not found.\n <p>\n <b>NOTE:</b> ResultSet.indexOf() only inspects the current cache of records, so it is only\n appropriate for temporary presentation purposes.  For example, it would not be appropriate\n to hold onto a record and attempt to use indexOf() to determine if it had been deleted.\n\n",
        name:"indexOf",
        seeAlso:[],
        params:[
            {description:"object to look for", optional:false, name:"obj", defaultValue:null, type:"any"},
            {description:"earliest index to consider", optional:true, name:"pos", defaultValue:null,
             type:"number"},
            {description:"last index to consider", optional:true, name:"endPos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"position of the item, if found, -1 if not found", type:"number"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access"]
    },
    "attr:Canvas.dataPath":{
        ref:"attr:Canvas.dataPath",
        flags:"IRWA",
        description:"A dataPath may be specified on any canvas. This provides a straightforward way to display or\n edit complex nested data.\n <P>\n For components which support displaying or editing data values, (such as ${isc.DocUtils.linkForRef('class:DynamicForm')} or\n ${isc.DocUtils.linkForRef('class:ListGrid')} components), the dataPath may be set to specify how the components data is\n accessed. In this case the dataPath essentially specifies a nested object to edit - typically\n a path to a field value within a dataSource record. Note that a ValuesManager will be required\n to handle connecting the dataBoundcomponent to the appropriate sub object. This may be explicitly\n specified on the component, or a parent of the component, or automatically generated\n if a DataSource is specified on either the component or a parent thereof.\n <P>\n To provide a simple example - if a complex object existed with the following format:\n <pre>\n { companyName:\"Some Company\",\n   address:{    street:\"123 Main Street\", city:\"New York\", state:\"NY\"  }\n }\n </pre>\n a developer could specify a DynamicForm instance with 'dataPath' set to \"address\" to edit\n the nested address object:\n <pre>\n isc.ValuesManager.create({\n      ID:'vm',\n      values: { companyName:\"Some Company\",\n              address:{    street:\"123 Main Street\", city:\"New York\", state:\"NY\"  }\n      }\n });\n\n isc.DynamicForm.create({\n      valuesManager:\"vm\",\n      dataPath:\"address\",\n      items:[{name:\"street\"}, {name:\"city\"}, {name:\"state\"}]\n });\n </pre>\n If a component is specified with a <code>dataPath</code> attribute but does not have an\n explicitly specified valuesManager, it will check its parent element chain for a specified\n valuesManager and automatically bind to that. This simplifies binding multiple components used\n to view or edit a nested data structure as the valuesManager needs only be defined once at a\n reasonably high level component. Here's an example of this approach:\n <pre>\n isc.ValuesManager.create({\n      ID:'vm',\n      values: { companyName:\"Some Company\",\n              address:{    street:\"123 Main Street\", city:\"New York\", state:\"NY\"  }\n      }\n });\n\n isc.Layout.create({\n      valuesManager:\"vm\",\n      members:[\n          isc.DynamicForm.create({\n              dataPath:\"/\",\n              items:[{name:\"companyName\"}]\n          }),\n          isc.DynamicForm.create({\n              dataPath:\"address\",\n              items:[{name:\"street\"}, {name:\"city\"}, {name:\"state\"}]\n          })\n      ]\n });\n </pre>\n Note that in this case the valuesManager is specified on a Layout, which has no 'values'\n management behavior of its own, but contains items with a specified dataPath which do. In this\n example you'd see 2 forms allowing editing of the nested data structure.\n <P>\n dataPaths from multiple nested components may also be combined. For example:\n <pre>\n isc.ValuesManager.create({\n      ID:'vm',\n      values: { companyName:\"Some Company\",\n              address:{    street:\"123 Main Street\", city:\"New York\", state:\"NY\"  }\n              parentCompany:{\n                  companyName:\"Some Coorporation\",\n                  address:{   street:\"1 High Street\", city:\"New York\", state:\"NY\" }\n              }\n      }\n });\n\n isc.Layout.create({\n      valuesManager:\"vm\",\n      members:[\n          isc.DynamicForm.create({\n              dataPath:\"/\",\n              items:[{name:\"companyName\"}]\n          }),\n          isc.DynamicForm.create({\n              dataPath:\"address\",\n              items:[{name:\"street\"}, {name:\"city\"}, {name:\"state\"}]\n          }),\n          isc.Layout.create({\n              dataPath:\"parentCompany\",\n              members:[\n                  isc.DynamicForm.create({\n                      dataPath:\"/\",\n                      items:[{name:\"companyName\", type:\"staticText\"}]\n                  }),\n                  isc.DetailViewer.create({\n                      dataPath:\"address\",\n                      fields:[{name:\"street\", name:\"city\", name:\"state\"}]\n                  })\n              ]\n          })\n      ]\n });\n </pre>\n In this example the detailViewer will display data from the <code>parentCompany.address</code>\n object within the base record.\n <P>\n Note that if a component has a specified  dataSource and shows child components with a\n specified dataPath, there is no need to explicitly declare a valuesManager at all. If a component\n with a dataPath has a dataSource, or an ancestor with a dataSource specified, it will, a\n valuesManager will automatically be generated on the higher level component (and be available as\n <code>component.valuesManager</code>).",
        name:"dataPath",
        setter:["setDataPath"],
        seeAlso:[],
        valueType:"dataPath",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas"
    },
    "classAttr:Validator.requiredField":{
        ref:"classAttr:Validator.requiredField",
        flags:"IRA",
        description:"Default error message to display when validation fails for a field marked as required,\n  or a field with a standard <code>requiredIf</code> type validator whose condition \n  evaluates to true, because the field has no value.",
        name:"requiredField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"'Field is required'",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "attr:SliderItem.sliderProperties":{
        ref:"attr:SliderItem.sliderProperties",
        flags:"IR",
        description:"Properties to add to the automatically created ${isc.DocUtils.linkForRef('class:Slider')} used by this\n FormItem.  See the ${isc.DocUtils.linkForRef('class:Slider')} class for reference.",
        name:"sliderProperties",
        seeAlso:[],
        valueType:"Slider properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SliderItem"
    },
    "attr:DetailViewer.labelPrefix":{
        ref:"attr:DetailViewer.labelPrefix",
        flags:"IRW",
        description:"text to put before a label",
        name:"labelPrefix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["labels"]
    },
    "attr:ListGridField.optionFilterContext":{
        ref:"attr:ListGridField.optionFilterContext",
        flags:"IR",
        description:"If this field has an optionDataSource specified and \n ${isc.DocUtils.linkForRef('attr:ListGridField.autoFetchDisplayMap','autoFetchDisplayMap')} is set, this attribute\n provides a way to customize the dataSource request issued to fetch the display map from\n the option dataSource.",
        name:"optionFilterContext",
        seeAlso:[],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "attr:Mail.multiple":{
        ref:"attr:Mail.multiple",
        flags:"IR",
        description:"By default, multiple mail messages are sent if the dsResponse contains multiple records. \n Set this property to false to prevent this behavior.",
        name:"multiple",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "method:MenuBar.addMenus":{
        ref:"method:MenuBar.addMenus",
        flags:"",
        description:" Dynamically update the menuBar to include additional menus. Will update the visible set\n of buttons as appropriate\n",
        name:"addMenus",
        seeAlso:[],
        params:[
            {description:"Array of new menus to add", optional:false, name:"newMenus",
             defaultValue:null, type:"array"},
            {description:"desired starting position of the new menus in the existing menus   array",
             optional:false, name:"position", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:MenuBar"
    },
    "attr:StretchImgButton.showDisabled":{
        ref:"attr:StretchImgButton.showDisabled",
        flags:"IRW",
        description:"Should we visibly change state when disabled?",
        name:"showDisabled",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["state"]
    },
    "method:List.getItems":{
        ref:"method:List.getItems",
        flags:"",
        description:" Return the items at a list of specified positions.\n\n",
        name:"getItems",
        seeAlso:[],
        params:[
            {description:"array of positions", optional:false, name:"itemList", defaultValue:null,
             type:"List of Number"}
        ],
        returns:{description:"subset of the array, in the same order as itemList", type:"array"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "attr:ColorPicker.colorButtonBaseStyle":{
        ref:"attr:ColorPicker.colorButtonBaseStyle",
        flags:"IR",
        description:"Base CSS style applied to the basic color boxes",
        name:"colorButtonBaseStyle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ColorChooserCell\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ListGrid.expansionDetails":{
        ref:"attr:ListGrid.expansionDetails",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:DetailViewer')} for displaying the details of a record\n in it's expanded section when\n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>details</code>.  Note that \n only those fields\n which do not already appear in the grid are displayed in the expanded section.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionDetailsProperties</code> and \n <code>listGrid.expansionDetailsDefaults</code>.\n <P>\n Note, however, that this is a multi-instance component (potentially one per record), \n so it is created using ${isc.DocUtils.linkForRef('method:Class.createAutoChild','createAutoChild()')} not \n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')}, and no default single instance is created by \n name on the grid.",
        name:"expansionDetails",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:StatefulCanvas.isSelected":{
        ref:"method:StatefulCanvas.isSelected",
        flags:"",
        description:" Find out if this object is selected\n",
        name:"isSelected",
        seeAlso:[],
        returns:{description:null, type:"boolean"},
        type:"method",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "classMethod:DataSource.get":{
        ref:"classMethod:DataSource.get",
        flags:"",
        description:" Synonym of ${isc.DocUtils.linkForRef('classMethod:DataSource.getDataSource')}: Lookup a DataSource by ID.\n\n",
        name:"get",
        seeAlso:[],
        params:[
            {description:"DataSource ID", optional:false, name:"ID", defaultValue:null,
             type:"identifier"}
        ],
        returns:{description:"the DataSource with this ID, if loaded, otherwise null.", type:"DataSource"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "classAttr:ValidatorTypes.contains":{
        ref:"classAttr:ValidatorTypes.contains",
        flags:"IR",
        description:"Determine whether a string value contains some substring specified via \n <code>validator.substring</code>.",
        name:"contains",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        definingClass:"class:ValidatorTypes"
    },
    "attr:GridRenderer.showEmptyMessage":{
        ref:"attr:GridRenderer.showEmptyMessage",
        flags:"IRW",
        description:"Indicates whether the text of the emptyMessage property should be displayed if no data is\n available.",
        name:"showEmptyMessage",
        seeAlso:[
            {ref:"attr:GridRenderer.emptyMessage", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["emptyMessage"]
    },
    "attr:ListGridField.editorProperties":{
        ref:"attr:ListGridField.editorProperties",
        flags:"IRWA",
        description:"Properties to apply the the form item created to edit this field. (Only used if\n this field is editable).\n <P>\n For example, if you have a field \"shoeSize\" with ${isc.DocUtils.linkForRef('attr:DataSourceField.editorType')} set\n to \"SpinnerItem\" in order to use a SpinnerItem as your field editor, and you want to pass the\n ${isc.DocUtils.linkForRef('attr:SpinnerItem.step')} property to the created SpinnerItem:\n <pre>\n    fields : [\n        { name:\"shoeSize\", editorType:\"SpinnerItem\", \n          editorProperties : { step:0.5 } },\n        ... other fields ...\n    ]\n </pre>",
        name:"editorProperties",
        seeAlso:[],
        valueType:"FormItem properties",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customEditors')}"],
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:Chart.setupChart":{
        ref:"method:Chart.setupChart",
        flags:"",
        description:" General facet and data model setup, including auto-derivation of facetValues from data if\n necessary.  Should be called by any concrete charting implementation before calling any\n other Chart method.\n\n",
        name:"setupChart",
        seeAlso:[],
        type:"method",
        definingClass:"interface:Chart"
    },
    "attr:Dialog.toolbarButtons":{
        ref:"attr:Dialog.toolbarButtons",
        flags:"IR",
        description:"Array of Buttons to show in the ${isc.DocUtils.linkForRef('attr:Dialog.showToolbar','toolbar')}, if shown.\n <P>\n The set of buttons to use is typically set by calling one of the shortcuts such as\n ${isc.DocUtils.linkForRef('classMethod:isc.say')} or ${isc.DocUtils.linkForRef('classMethod:isc.confirm')}.  A custom set of buttons can be passed to\n these shortcuts methods via the \"properties\" argument, or to a directly created Dialog.\n <P>\n In both cases, a mixture of ${isc.DocUtils.linkForRef('type:DialogButtons','built-in buttons')}, custom buttons,\n and other components (such as a ${isc.DocUtils.linkForRef('class:LayoutSpacer')}) can be passed.  Built-in buttons\n can be referred to as <code>isc.Dialog.OK</code>, for example:\n <pre>\n isc.Dialog.create({\n    toolbarButtons:[\n       isc.Dialog.OK, \n       isc.Dialog.CANCEL, \n       isc.LayoutSpacer.create({width:50}), \n       { title:\"Not now\", click:\"doSomething()\" }\n    ]\n })\n </pre>\n Built-in buttons will call standard methods on the Dialog itself, such as\n ${isc.DocUtils.linkForRef('method:Dialog.cancelClick')}, as explained in the \n ${isc.DocUtils.linkForRef('type:DialogButtons','list of built-in buttons')}.",
        name:"toolbarButtons",
        setter:["setToolbarButtons"],
        seeAlso:[],
        valueType:"Array of Button or Button Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Dialog"
    },
    "method:ListGrid.getRelatedDataSource":{
        ref:"method:ListGrid.getRelatedDataSource",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('class:DataSource')} containing data related to the passed record.  Used\n when ${isc.DocUtils.linkForRef('attr:ListGrid.canExpandRecords')} is true and ${isc.DocUtils.linkForRef('type:ExpansionMode')} is \"related\". The\n default implementation returns the DataSource specified in \n ${isc.DocUtils.linkForRef('attr:ListGridRecord.detailDS')}.\n \n",
        name:"getRelatedDataSource",
        seeAlso:[],
        params:[
            {description:"The record to get the Related dataSource for.", optional:false,
             name:"record", defaultValue:null, type:"ListGridRecord"}
        ],
        returns:{description:"The related DataSource for the \"record\" param", type:"DataSource"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:DSRequest.operationType":{
        ref:"attr:DSRequest.operationType",
        flags:"IR",
        description:"Type of operation being performed: \"fetch\", \"add\", \"remove\", \"update\" or \"custom\".\n <P>\n This property is generally automatically populated, for example when calling\n <code>fetchData()</code> on a DataSource or DataBound component the operationType is\n automatically set to \"fetch\".  Note that \"custom\" operations are never generated\n automatically, they are always fired by your code.",
        name:"operationType",
        seeAlso:[],
        valueType:"DSOperationType",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "attr:Snapbar.showRollOverGrip":{
        ref:"attr:Snapbar.showRollOverGrip",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:Splitbar.showGrip')} is true, this property determines whether to show the\n 'Over' state on the grip image when the user rolls over on this widget. \n Has no effect if ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showRollOver')} is false",
        name:"showRollOverGrip",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Snapbar",
        groups:["grip"]
    },
    "attr:DataSource.useFlatFields":{
        ref:"attr:DataSource.useFlatFields",
        flags:"IR",
        description:"Like ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useFlatFields')}, but applies to all DataBound components\n that bind to this DataSource.",
        name:"useFlatFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["fields"]
    },
    "attr:FacetValue.borderAfter":{
        ref:"attr:FacetValue.borderAfter",
        flags:"IR",
        description:"CSS line style to apply as a border after this facetValue",
        name:"borderAfter",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "classMethod:FileLoader.loadISC":{
        ref:"classMethod:FileLoader.loadISC",
        flags:"",
        description:"\n Loads the specified SmartClient modules and skin in defer mode.  Calling this method is\n equivalent to using the <code>loadISC</code> JSP tag with <code>defer=\"true\"</code>.\n\n",
        name:"loadISC",
        seeAlso:[],
        params:[
            {description:"Name of the skin to load.  If not specified, the skin specified by the                        default ${isc.DocUtils.linkForRef('classAttr:FileLoader.defaultSkin')} will be used.\n                        Instead of a skin name, you can specify a skinDir - this works just\n                        like ${isc.DocUtils.linkForRef('classMethod:Page.setSkinDir')} - the skin assumed to be a skinDir if\n                        there's at least one forward slash (/) in the name.",
             optional:true, name:"skin", defaultValue:null, type:"String"},
            {description:"List of modules to load.  If not specified, the list of modules                                   specified by the default\n                                   ${isc.DocUtils.linkForRef('classAttr:FileLoader.defaultModules')} will be used.  You\n                                   can specify modules as e.g: \"Core,Foundation\" or as [\"Core\",\n                                   \"Foundation\"]",
             optional:true, name:"modules", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when all specified modules and                                      skin have loaded.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "attr:GridRenderer.fixedRowHeights":{
        ref:"attr:GridRenderer.fixedRowHeights",
        flags:"IRWA",
        description:"Should we vertically clip cell contents, or allow rows to expand vertically to show all\n contents?\n <P>\n If we allow rows to expand, the row height as derived from\n ${isc.DocUtils.linkForRef('method:GridRenderer.getRowHeight','getRowHeight()')} or the\n default ${isc.DocUtils.linkForRef('attr:GridRenderer.cellHeight')} is treated as a minimum.",
        name:"fixedRowHeights",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:SortSpecifier.context":{
        ref:"attr:SortSpecifier.context",
        flags:"IR",
        description:"A DataBoundComponent providing the context for the sort-normalizer.",
        name:"context",
        seeAlso:[],
        valueType:"DataBoundComponent",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:SortSpecifier"
    },
    "classMethod:isc.getValues":{
        ref:"classMethod:isc.getValues",
        flags:"",
        description:"\n	Return all values of a given object\n\n",
        name:"getValues",
        seeAlso:[],
        params:[
            {description:"object to get properties from", optional:false, name:"object",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"values of all properties.  NOTE: never null", type:"Array"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:MultiSortDialog.levelUpButton":{
        ref:"attr:MultiSortDialog.levelUpButton",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ImgButton')} providing a mechanism for moving existing\n sort-levels up in the sort configuration.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.levelUpButtonProperties</code> and \n <code>multiSortPanel.levelUpButtonDefaults</code>.",
        name:"levelUpButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:Date.getDefaultDateSeparator":{
        ref:"method:Date.getDefaultDateSeparator",
        flags:"",
        description:" gets the default date separator string\n",
        name:"getDefaultDateSeparator",
        seeAlso:[],
        returns:{description:"the default date separator", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:Class.logIsWarnEnabled":{
        ref:"method:Class.logIsWarnEnabled",
        flags:"",
        description:" Check whether a message logged at \"warn\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsWarnEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "classAttr:Operators.notContainsTitle":{
        ref:"classAttr:Operators.notContainsTitle",
        flags:"IR",
        description:"Title for the \"notContains\" operator",
        name:"notContainsTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not contain (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.expansionLayout":{
        ref:"attr:ListGrid.expansionLayout",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:VLayout')} which fills a record's expanded section\n and contains other builtin ${isc.DocUtils.linkForRef('type:ExpansionMode','expansion-components')}.  You can also\n override ${isc.DocUtils.linkForRef('method:ListGrid.getExpansionComponent','getExpansionComponent()')} to provide\n components of your own specification.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionLayoutProperties</code> and \n <code>listGrid.expansionLayoutDefaults</code>.",
        name:"expansionLayout",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "attr:ActiveXControl.id":{
        ref:"attr:ActiveXControl.id",
        flags:"IR",
        description:"Sets the 'id' attribute on the object.  If a name is not provided it will be\n  auto-generated.  Note that in general you don't need to set this.  If you have a reference to\n  your ISC ActiveX control object you can simply call\n  ${isc.DocUtils.linkForRef('method:ActiveXControl.getPluginHandle')} to get a handle to the element.",
        name:"id",
        seeAlso:[
            {ref:"method:ActiveXControl.getPluginHandle", description:null},
            {ref:"method:ActiveXControl.getPluginID", description:null}
        ],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ActiveXControl"
    },
    "classAttr:Dialog.APPLY_BUTTON_TITLE":{
        ref:"classAttr:Dialog.APPLY_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"Apply\"</code> button.",
        name:"APPLY_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"Apply\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "method:Array.addList":{
        ref:"method:Array.addList",
        flags:"",
        description:" Add a list of items to this array.\n <P>\n Note: you can specify that a subset range be added by passing start and end indices\n\n",
        name:"addList",
        seeAlso:[],
        params:[
            {description:"list of items to add", optional:false, name:"list", defaultValue:null,
             type:"array"},
            {description:"optional start index in list", optional:true, name:"listStartRow",
             defaultValue:null, type:"number"},
            {description:"optional end index in list (non-inclusive)", optional:true,
             name:"listEndRow", defaultValue:null, type:"number"}
        ],
        returns:{description:"list of items that were added", type:"list"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "classMethod:History.getCurrentHistoryId":{
        ref:"classMethod:History.getCurrentHistoryId",
        flags:"",
        description:"\n Returns the current history id as reflected by the current URL.\n\n",
        name:"getCurrentHistoryId",
        seeAlso:[],
        returns:{description:"The current history id as reflected by the current URL.", type:"String"},
        type:"classMethod",
        definingClass:"class:History"
    },
    "classAttr:Operators.iNotStartsWithTitle":{
        ref:"classAttr:Operators.iNotStartsWithTitle",
        flags:"IR",
        description:"Title for the \"iNotStartsWith\" operator",
        name:"iNotStartsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"does not start with\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "object:Hilite":{
        methods:[],
        ref:"object:Hilite",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:["attr:Hilite.id", "attr:Hilite.cssText", "attr:Hilite.fieldName", "attr:Hilite.criteria", 
         "attr:Hilite.htmlBefore","attr:Hilite.htmlAfter", "attr:Hilite.htmlValue", "attr:Hilite.disabled", 
         "attr:Hilite.title"],
        description:"Definition of a hilite style.\n <P>\n See ${isc.DocUtils.linkForRef('group:hiliting')} for an overview.",
        name:"Hilite",
        seeAlso:[],
        type:"object",
        requiresModules:["Analytics"]
    },
    "attr:StretchImgButton.valign":{
        ref:"attr:StretchImgButton.valign",
        flags:"IRW",
        description:"Vertical alignment of this component's title.",
        name:"valign",
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:StretchImgButton",
        groups:["appearance"]
    },
    "attr:RichTextEditor.fontNames":{
        ref:"attr:RichTextEditor.fontNames",
        flags:"IRA",
        description:"ValueMap of css fontName properties to font name titles to display in the font selector \n  if <code>\"fontSelector\"</code> is included in ${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} \n  for this editor.\n  Default value for this attribute:<br>\n <code> {\n &nbsp;&nbsp;\"arial,helvetica,sans-serif\":\"Arial\",\n &nbsp;&nbsp;'courier new,courier,monospace':\"Courier New\",\n &nbsp;&nbsp;'georgia,times new roman,times,serif':\"Georgia\",\n &nbsp;&nbsp;'tahoma,arial,helvetica,sans-serif':\"Tahoma\",\n &nbsp;&nbsp;'times new roman,times,serif':\"Times New Roman\",\n &nbsp;&nbsp;'verdana,arial,helvetica,sans-serif':\"Verdana\",\n &nbsp;&nbsp;\"impact\":\"Impact\"}</code>",
        name:"fontNames",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:RichTextEditor"
    },
    "attr:MultiSortPanel.fields":{
        ref:"attr:MultiSortPanel.fields",
        flags:"IR",
        description:"The list of fields which the user can choose to sort by.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "attr:Operator.valueType":{
        ref:"attr:Operator.valueType",
        flags:"IR",
        description:"Indicates the kind of value expected in a ${isc.DocUtils.linkForRef('object:Criterion')} that uses this operator.\n ${isc.DocUtils.linkForRef('type:OperatorValueType')} lists possibilities.\n <P>\n The default of <code>null</code> is equivalent to \"fieldType\", indicating that\n ${isc.DocUtils.linkForRef('attr:Criterion.value')} is expected to contain a value of the same type as the field\n indicated by ${isc.DocUtils.linkForRef('attr:Criterion.fieldName')}.",
        name:"valueType",
        seeAlso:[],
        valueType:"OperatorValueType",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Operator",
        groups:["advancedFilter"]
    },
    "method:Calendar.setData":{
        ref:"method:Calendar.setData",
        flags:"",
        description:" Initialize the data object with the given array. Observes methods of the data object\n so that when the data changes, the calendar will redraw automatically.\n \n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"data to show in the list", optional:false, name:"newData", defaultValue:null,
             type:"List of CalendarEvent"}
        ],
        type:"method",
        definingClass:"class:Calendar",
        groups:["data"]
    },
    "attr:TabSet.bottomEdgeSizes":{
        ref:"attr:TabSet.bottomEdgeSizes",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeSizes</code>, \n <code>rightEdgeSizes</code>, <code>topEdgeSizes</code> and <code>bottomEdgeSizes</code> \n propertes allow the sizes of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge sizes where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"bottomEdgeSizes",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:ListGrid.headerBackgroundColor":{
        ref:"attr:ListGrid.headerBackgroundColor",
        flags:"IRW",
        description:"BackgroundColor for the header toolbar. Typically this is set to match the color\n of the header buttons.",
        name:"headerBackgroundColor",
        seeAlso:[],
        valueType:"color",
        defaultValue:"\"#CCCCCC\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:FilterBuilder.radioOperatorForm":{
        ref:"attr:FilterBuilder.radioOperatorForm",
        flags:"IR",
        description:"With ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')}:\"radio\", form that appears above the stack of clauses\n and allows picking the ${isc.DocUtils.linkForRef('type:LogicalOperator')} for the overall FilterBuilder.\n <P>\n By default, consists of a simple RadioGroupItem.",
        name:"radioOperatorForm",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:GridRenderer.cellHeight":{
        ref:"attr:GridRenderer.cellHeight",
        flags:"IRW",
        description:"The default height of each row in pixels.",
        name:"cellHeight",
        seeAlso:[
            {ref:"method:GridRenderer.getRowHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["cellStyling"]
    },
    "attr:TabSet.symmetricPickerButton":{
        ref:"attr:TabSet.symmetricPickerButton",
        flags:"IR",
        description:"If this TabSet is showing a ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker','tab picker button')}, this\n property determines whether the ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonHSrc')} and\n ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonVSrc')} media will be used for vertical and horizontal tab-bar\n picker buttons, or whether separate media should be used for each possible \n ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition','tabBarPosition')} based on the ${isc.DocUtils.linkForRef('attr:TabSet.pickerButtonSrc')}\n property  for this tabSet.",
        name:"symmetricPickerButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarScrolling"]
    },
    "classMethod:Array.compareDescending":{
        ref:"classMethod:Array.compareDescending",
        flags:"A",
        description:" Compare two values for a descending order sort, using locale-sensitive comparison.\n",
        name:"compareDescending",
        seeAlso:[],
        params:[
            {description:"first value to compare", optional:false, name:"first", defaultValue:null,
             type:"any"},
            {description:"second value to compare", optional:false, name:"second", defaultValue:null,
             type:"any"}
        ],
        returns:{description:"negative == first is larger, 0 == same value, positive == second is larger",
         type:"number"},
        type:"classMethod",
        definingClass:"object:Array",
        groups:["sorting"]
    },
    "attr:Canvas.canFocus":{
        flags:"IRWA",
        type:"attr",
        ref:"attr:Canvas.canFocus",
        description:"Can this widget be allowed to become the target of keyboard events?\n <P>\n If canFocus is unset (the default), only scrollable widgets with visible scrollbars are\n focusable, to allow for keyboard scrolling.\n <P>\n A widget normally receives focus by being clicked on or tabbed to.",
        name:"canFocus",
        setter:["setCanFocus"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('focus')}"],
        defaultValue:"null",
        groups:["focus", "events"],
        definingClass:"class:Canvas"
    },
    "attr:MultiView.showFilterEditor":{
        ref:"attr:MultiView.showFilterEditor",
        flags:"IR",
        description:"When true, show a UI for filtering the data in the current view.",
        name:"showFilterEditor",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:MultiView"
    },
    "group:imageEdgeStyles":{
        ref:"group:imageEdgeStyles",
        refs:["attr:EdgedCanvas.edgeStyleName"],
        name:"imageEdgeStyles",
        seeAlso:[],
        type:"group"
    },
    "method:GridRenderer.cellMouseUp":{
        ref:"method:GridRenderer.cellMouseUp",
        flags:"A",
        description:" Called when a cell receives a mouseup event.\n\n",
        name:"cellMouseUp",
        seeAlso:[],
        params:[
            {description:"Record object (retrieved from getCellRecord(rowNum, colNum))",
             optional:false, name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "classMethod:Class.getCallTrace":{
        ref:"classMethod:Class.getCallTrace",
        flags:"",
        description:" Returns a one-line summary of the current method call, showing method name and passed\n arguments. \n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getCallTrace()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"getCallTrace",
        seeAlso:[],
        params:[
            {description:"arguments object from the call to trace.  On IE, defaults to the                             calling function's arguments",
             optional:true, name:"args", defaultValue:null, type:"Arguments"}
        ],
        type:"classMethod",
        platformNotes:"IE only.",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "classMethod:Page.setSkinDir":{
        ref:"classMethod:Page.setSkinDir",
        flags:"",
        description:"		Specify the URL for media that's part of the skin\n\n",
        name:"setSkinDir",
        seeAlso:[],
        params:[
            {description:"New skinDir URL", optional:true, name:"URL", defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["skins", "files", "images"]
    },
    "attr:DateChooser.nextMonthIcon":{
        ref:"attr:DateChooser.nextMonthIcon",
        flags:"IRW",
        description:"Icon for the next month button",
        name:"nextMonthIcon",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"\"[SKIN]arrow_right.gif\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:DetailViewer.wrapValues":{
        ref:"attr:DetailViewer.wrapValues",
        flags:"IR",
        description:"Whether values should be allowed to wrap by default, or should be shown on one line\n regardless of length.",
        name:"wrapValues",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["labels"]
    },
    "attr:ListGrid.canFreezeFields":{
        ref:"attr:ListGrid.canFreezeFields",
        flags:"IRW",
        description:"Whether an interface should be shown to allow user is allowed to dynamically \"freeze\" or\n \"unfreeze\" columns with respect to horizontally scrolling. If unset, this property defaults\n to <code>true</code> unless:<ul>\n <li>${isc.DocUtils.linkForRef('attr:ListGrid.fixedRecordHeights','this.fixedRecordHeights')} is <code>false</code></li>\n <li>${isc.DocUtils.linkForRef('attr:ListGrid.bodyOverflow','this.bodyOverflow')} is <code>\"visible\"</code></li>\n <li>${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData','this.autoFitData')} is set to <code>\"horizontal\"</code> or \n <code>\"both\"</code></li>\n <li>Any field has overflow set to <code>\"visible\"</code></li></ul>\n <P>\n Note that the <code>canFreezeFields</code> setting enables or disables the user\n interface for freezing and unfreezing fields only.  Fields can be programmatically\n frozen via setting ${isc.DocUtils.linkForRef('attr:ListGridField.frozen','field.frozen')} to true when the grid is\n created, or dynamically frozen and unfrozen via ${isc.DocUtils.linkForRef('method:ListGrid.freezeField')} and\n ${isc.DocUtils.linkForRef('method:ListGrid.unfreezeField')}.",
        name:"canFreezeFields",
        setter:["setCanFreezeFields"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["frozenFields"]
    },
    "method:ListGrid.getFieldNum":{
        ref:"method:ListGrid.getFieldNum",
        flags:"A",
        description:"		Given a field or field id, return it's index in the fields array\n",
        name:"getFieldNum",
        seeAlso:[],
        params:[
            {description:"field number or field.name", optional:false, name:"fieldID",
             defaultValue:null, type:"string || number"}
        ],
        returns:{description:"index of the field within this.fields", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["display"]
    },
    "attr:Window.headerIcon":{
        ref:"attr:Window.headerIcon",
        flags:"R",
        description:"Header icon shown at left end of header by default.",
        name:"headerIcon",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:BatchUploader.uploadOperation":{
        ref:"attr:BatchUploader.uploadOperation",
        flags:"IR",
        description:"Optional ${isc.DocUtils.linkForRef('attr:DSRequest.operationId')} for the \"add\" operation used to add new\n records to the ${isc.DocUtils.linkForRef('attr:BatchUploader.uploadDataSource')}.",
        name:"uploadOperation",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:RichTextItem.controlGroups":{
        ref:"attr:RichTextItem.controlGroups",
        flags:"IA",
        description:"${isc.DocUtils.linkForRef('attr:RichTextEditor.controlGroups')} to display for this editor.\n Each controlGroup should be a property set either on this item or on the RichTextEditor\n prototype and should be set to an array of ${isc.DocUtils.linkForRef('type:ControlName')}s.",
        name:"controlGroups",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RichTextItem"
    },
    "attr:ToolbarItem.buttons":{
        ref:"attr:ToolbarItem.buttons",
        flags:"IRW",
        description:"List of buttons for the toolbar.\n  Each buttons should be specified as a simple JS object with properties to apply to the\n  button to be displayed. Note that any <code>click</code> stringMethod applied to the\n  button will be passed 2 parameters: <code>form</code> and <code>item</code>.",
        name:"buttons",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ToolbarItem",
        groups:["items"]
    },
    "method:Tree.removeList":{
        ref:"method:Tree.removeList",
        flags:"",
        description:"\n Remove a list of nodes (not necessarily from the same parent), and all children of those nodes.\n			\n",
        name:"removeList",
        seeAlso:[],
        params:[
            {description:"list of nodes to remove", optional:false, name:"nodeList", defaultValue:null,
             type:"List of TreeNode"}
        ],
        returns:{description:"true if the tree was changed as a result of this call", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "class:PanelHeader":{
        methods:["method:PanelHeader.setBaseStyle", "method:PanelHeader.setTitleStyle", 
         "method:PanelHeader.iconClick"],
        ref:"class:PanelHeader",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:PanelHeader.baseStyle", "attr:PanelHeader.titleStyle"],
        description:"A PanelHeader",
        name:"PanelHeader",
        seeAlso:[],
        type:"class"
    },
    "method:Canvas.moveBy":{
        ref:"method:Canvas.moveBy",
        flags:"",
        description:"			Moves the widget deltaX pixels to the right and deltaY pixels down. Pass negative\n          numbers to move up and/or to the left.\n",
        name:"moveBy",
        seeAlso:[],
        params:[
            {description:"amount to move horizontally (may be negative)", optional:false,
             name:"deltaX", defaultValue:null, type:"number"},
            {description:"amount to move vertically (may be negative)", optional:false, name:"deltaY",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"whether the component actually moved", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('move')}"],
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "attr:Calendar.otherDayBlankStyle":{
        ref:"attr:Calendar.otherDayBlankStyle",
        flags:"IR",
        description:"In the month CSS style applied to both the header and body of days from other months when\n ${isc.DocUtils.linkForRef('attr:Calendar.showOtherDays')} is false.",
        name:"otherDayBlankStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"calMonthOtherDayBlank\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "attr:StatefulCanvas.actionType":{
        flags:"IRW",
        getter:["getActionType"],
        type:"attr",
        ref:"attr:StatefulCanvas.actionType",
        description:"Behavior on state changes -- BUTTON, RADIO or CHECKBOX",
        name:"actionType",
        setter:["setActionType"],
        valueType:"SelectionType",
        seeAlso:[],
        defaultValue:"\"button\"",
        groups:["state", "event handling"],
        definingClass:"class:StatefulCanvas"
    },
    "attr:BlurbItem.showTitle":{
        ref:"attr:BlurbItem.showTitle",
        flags:"IRW",
        description:"Blurb items show no title by default.",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:BlurbItem",
        groups:["appearance"]
    },
    "classMethod:Time.compareTimes":{
        ref:"classMethod:Time.compareTimes",
        flags:"",
        description:" Compares the times of 2 dates, or strings. If a string is passed as one of the \n parameters it should be in a format that converts to a valid time such as <code>\"1:30pm\"</code>, \n <code>\"13:30\"</code>, or <code>\"1:30:45pm\"</code>\n",
        name:"compareTimes",
        seeAlso:[],
        params:[
            {description:"First time to compare", optional:false, name:"time1", defaultValue:null,
             type:"Date|string"},
            {description:"Second time to compare", optional:false, name:"time2", defaultValue:null,
             type:"Date|string"}
        ],
        returns:{description:"True if the times match, false if not", type:"boolean"},
        type:"classMethod",
        definingClass:"class:Time"
    },
    "attr:MultiSortPanel.optionsGrid":{
        ref:"attr:MultiSortPanel.optionsGrid",
        flags:"IR",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:ListGrid')} allowing the user to configure a set of \n ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>multiSortPanel.optionsGridProperties</code> and \n <code>multiSortPanel.optionsGridDefaults</code>.",
        name:"optionsGrid",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "method:Progressbar.setLength":{
        ref:"method:Progressbar.setLength",
        flags:"",
        description:" Sets the length of the progressbar to newLength. This is the width of a horizontal progressbar,\n or the height of a vertical progressbar.\n\n",
        name:"setLength",
        seeAlso:[],
        params:[
            {description:"the new length of the progressbar", optional:false, name:"newLength",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Progressbar"
    },
    "method:WebService.setLocation":{
        ref:"method:WebService.setLocation",
        flags:"A",
        description:" Set location can be used when the actual URL where a service will be accessible isn't\n known until runtime, or changes at runtime, hence can't be embedded in the service\n definition.\n <P>\n With an operation parameter, <code>setLocation()</code> can be used to set a distinct\n URL for each web service operation.  This is a development-time only feature that allows\n XML flat files to be placed at various URLs on a server, to serve as spoofed responses\n for each web service operation.\n\n",
        name:"setLocation",
        seeAlso:[],
        params:[
            {description:"URL where web service can be contacted", optional:false, name:"location",
             defaultValue:null, type:"URL"},
            {description:"optional operation name to set the location for, for                             debugging only",
             optional:true, name:"operation", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "attr:ColumnTree.autoFetchData":{
        ref:"attr:ColumnTree.autoFetchData",
        flags:"IR",
        description:"If true, when this component is first drawn, automatically call <code>this.fetchData()</code>.\n Criteria for this fetch may be picked up from ${isc.DocUtils.linkForRef('attr:ColumnTree.initialCriteria')}, and textMatchStyle may\n be specified via ${isc.DocUtils.linkForRef('attr:ColumnTree.autoFetchTextMatchStyle')}.\n <P>\n <span style='color:red'>NOTE:</span> if <code>autoFetchData</code> is set, calling\n ${isc.DocUtils.linkForRef('method:ColumnTree.fetchData')} before draw will cause two requests to be issued, one from the manual\n call to fetchData() and one from the autoFetchData setting.  The second request will use\n only ${isc.DocUtils.linkForRef('attr:ColumnTree.initialCriteria')} and not any other criteria or settings from the first request.\n Generally, turn off autoFetchData if you are going to manually call fetchData() at any time.",
        name:"autoFetchData",
        seeAlso:[
            {ref:"method:ColumnTree.fetchData", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["databinding"]
    },
    "method:ValuesManager.setFieldErrors":{
        ref:"method:ValuesManager.setFieldErrors",
        flags:"",
        description:" Sets validation errors for some field in the valuesManager.<br>\n Errors passed in should be a string (for a single error message) or an array of strings.\n Pass in the showErrors parameter to immediately display the errors to the user by \n redrawing the appropriate member form item (or if no visible item is found for the field\n firing ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')}.\n",
        name:"setFieldErrors",
        seeAlso:[],
        params:[
            {description:"name of field to apply errors to", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"error messages for the field", optional:false, name:"errors",
             defaultValue:null, type:"string | array of strings"},
            {description:"should the error(s) be immediately displayed to the user?", optional:false,
             name:"showErrors", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "attr:ListGridField.editorType":{
        ref:"attr:ListGridField.editorType",
        flags:"IRWA",
        description:"Name of form item class to use for the form item created to edit this field. \n      (Only used if this field is editable).<br>\n      Note: If this is not specified, the edit-form item type may be derived from the\n      <code>editorType</code> property, typically inherited from datasource fields, or \n      from the <code>type</code> of the field (showing the appropriate form item for \n      the data-type). See the ${isc.DocUtils.linkForRef('group:editing')} overview for more on editing ListGrid\n      fields.",
        name:"editorType",
        seeAlso:[
            {ref:"attr:ListGrid.canEdit", description:null}
        ],
        valueType:"FormItem className",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customEditors')}"],
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "method:TabSet.disableTab":{
        ref:"method:TabSet.disableTab",
        flags:"",
        description:" If the specified tab is enabled, disable it now.\n",
        name:"disableTab",
        seeAlso:[
            {ref:"attr:Tab.disabled", description:null}
        ],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"Tab | number | ID"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "group:cues":{
        ref:"group:cues",
        refs:["method:Canvas.setOpacity", "method:Canvas.setCursor", "attr:Canvas.cursor", 
         "attr:Canvas.disabledCursor","attr:Canvas.opacity", "attr:Canvas.contextMenu", "type:Cursor"],
        description:"Visual hints for the user that something can be done to this object",
        name:"cues",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.groupIcon":{
        ref:"attr:ListGrid.groupIcon",
        flags:"IRW",
        description:"The URL of the base icon for the group icons in this treegrid.",
        name:"groupIcon",
        seeAlso:[
            {ref:"group:grouping", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"\"[SKINIMG]/TreeGrid/opener.gif\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "class:ViewLoader":{
        methods:["method:ViewLoader.setViewURL", "method:ViewLoader.handleError", 
         "method:ViewLoader.getView","method:ViewLoader.viewLoaded"],
        type:"class",
        ref:"class:ViewLoader",
        classAttrs:[],
        treeLocation:["Client Reference/Foundation"],
        classMethods:[],
        description:"The ViewLoader component can be used to load new SmartClient-based user interfaces into a\n running application.\n <P>\n <b>NOTE:</b> before using a ViewLoader, be sure that you have read about and understood the\n ${isc.DocUtils.linkForRef('group:smartArchitecture','SmartClient Architecture')}.  The most responsive and\n scalable application architecture preloads views rather than using ViewLoaders.\n <P>\n A ViewLoader is a Canvas, and can be provided anywhere a Canvas can be provided: as a Tab\n pane, and Layout member, etc.  When a ViewLoader draws, it shows a\n ${isc.DocUtils.linkForRef('attr:ViewLoader.loadingMessage','loading message')}, then\n performs an RPC to the ${isc.DocUtils.linkForRef('attr:ViewLoader.viewURL','viewURL')} to load components.\n <P>\n The response from the viewURL should be SmartClient components defined in JavaScript, with no\n surrounding &lt;SCRIPT&gt; tags or other HTML framing.  The returned script can be\n dynamically generated, for example, it may be the result of a JSP containing an XML view\n description enclosed in ${isc.DocUtils.linkForRef('group:xmlTag','&lt;isomorphicXML&gt;')} tags.\n <P>\n In the returned script, the special variable \"viewLoader\" is available to refer to the\n ViewLoader instance that is loading components.  The intended usage is that the returned\n script creates a view consisting of SmartClient components, then calls\n <code>viewLoader.setView(myView)</code> to place the loaded view into the ViewLoader.\n If the view does not call setView() explicitly, the viewLoader will find the last top-level\n UI component (Canvas subclass) created by the view and set that as the current view.\n Top-level in this case means that the UI component is not contained in another UI component\n as a member or child.\n <p>\n The ViewLoader relies on the XMLHttpRequest object which can be disabled by end-users in some\n supported browsers.  See ${isc.DocUtils.linkForRef('group:platformDependencies')} for more information.",
        attrs:["attr:ViewLoader.loadingMessage", "attr:ViewLoader.viewURL", 
         "attr:ViewLoader.viewURLParams","attr:ViewLoader.viewRPCProperties", "attr:ViewLoader.httpMethod", 
         "attr:ViewLoader.allowCaching"],
        name:"ViewLoader",
        seeAlso:[
            {ref:"attr:RPCRequest.evalResult", description:null},
            {ref:"group:smartArchitecture",
             description:"for general discussion of architectural best practices in      SmartClient"}
        ],
        examples:["${isc.DocUtils.linkForExampleId('viewLoading')}"],
        groups:["viewLoading"]
    },
    "attr:ImgSplitbar.vSrc":{
        ref:"attr:ImgSplitbar.vSrc",
        flags:"IR",
        description:"Default src to display when ${isc.DocUtils.linkForRef('attr:ImgSplitbar.vertical')} is true, \n and ${isc.DocUtils.linkForRef('attr:ImgSplitbar.src')} is unset.",
        name:"vSrc",
        seeAlso:[
            {ref:"attr:ImgSplitbar.src", description:null}
        ],
        valueType:"SCImgURL",
        defaultValue:"[SKIN]vgrip.png",
        type:"attr",
        definingClass:"class:ImgSplitbar"
    },
    "attr:Window.showCloseButton":{
        ref:"attr:Window.showCloseButton",
        flags:"IRW",
        description:"If true, show a close button in the header, which will dismiss this window by \n calling ${isc.DocUtils.linkForRef('method:Window.closeClick')}.",
        name:"showCloseButton",
        setter:["setShowCloseButton"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "attr:DynamicForm.target":{
        ref:"attr:DynamicForm.target",
        flags:"IRWA",
        description:"The name of a window or frame that will receive the results returned by the form's\n action. The default null indicates to use the current frame.\n <p>\n <b>NOTE:</b> this is used only in the very rare case that a form is used to submit data\n directly to a URL.  Normal server contact is through\n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')}.",
        name:"target",
        setter:["setTarget"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "method:ListGrid.setEditorValueMap":{
        ref:"method:ListGrid.setEditorValueMap",
        flags:"A",
        description:"\n Set a valueMap to display for this field while editing.<br>\n This method sets the ${isc.DocUtils.linkForRef('attr:ListGridField.editorValueMap','field.editorValueMap')} property - \n note that if  ${isc.DocUtils.linkForRef('method:ListGrid.getEditorValueMap')} has been overridden it may not make use \n of this property.\n",
        name:"setEditorValueMap",
        seeAlso:[],
        params:[
            {description:"Field object or identifier", optional:false, name:"fieldID",
             defaultValue:null, type:"object | number | field name"},
            {description:"ValueMap to apply to the field", optional:false, name:"map",
             defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:EdgedCanvas.edgeImage":{
        ref:"attr:EdgedCanvas.edgeImage",
        flags:"IR",
        description:"Base name of images for edges.  Extensions for each corner or edge piece will be added\n to this image URL, before the extension.  For example, with the default base name of\n \"edge.gif\", the top-left corner image will be \"edge_TL.gif\".\n <P>\n The full list of extensions is: \"_TL\", \"_TR\", \"_BL\", \"_BR\", \"_T\", \"_L\", \"_B\", \"_R\",\n \"_center\".",
        name:"edgeImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]/rounded/frame/FFFFFF/6.png\"",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:ListGridField.filterEditorProperties":{
        ref:"attr:ListGridField.filterEditorProperties",
        flags:"IRWA",
        description:"If this ListGrid is showing a filter row\n (${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor','showFilterEditor')}:true), this property\n can be used to specify properties for the appropriate filter form item.",
        name:"filterEditorProperties",
        seeAlso:[],
        valueType:"FormItem properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "method:TileGrid.getSelection":{
        ref:"method:TileGrid.getSelection",
        flags:"",
        description:" Returns all selected records, as an Array.\n\n",
        name:"getSelection",
        seeAlso:[],
        returns:{description:"list of records, empty list if nothing selected",
         type:"Array of ListGridRecord"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('databoundRemove')}"],
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "attr:ListGrid.animateRemoveTime":{
        ref:"attr:ListGrid.animateRemoveTime",
        flags:"IRW",
        description:"When animating record removal \n ${isc.DocUtils.linkForRef('attr:ListGrid.animateRemoveRecord','(see animateRemoveRecord)')}, if \n ${isc.DocUtils.linkForRef('attr:ListGrid.animateRemoveSpeed')} is not\n set, this property designates the duration of the animation in ms.",
        name:"animateRemoveTime",
        seeAlso:[
            {ref:"attr:ListGrid.animateRemoveRecord", description:null}
        ],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["animation"]
    },
    "classAttr:Operators.greaterOrEqualTitle":{
        ref:"classAttr:Operators.greaterOrEqualTitle",
        flags:"IR",
        description:"Title for the \"greaterOrEqual\" operator",
        name:"greaterOrEqualTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"greater than or equal to\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "classAttr:RPCManager.fetchDataPrompt":{
        ref:"classAttr:RPCManager.fetchDataPrompt",
        flags:"IRW",
        description:"Default prompt displayed to the user while an operation is running to fetch data from\n the server.<br>\n Displayed as a result of ${isc.DocUtils.linkForRef('method:ListGrid.filterData')}, ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} and\n ${isc.DocUtils.linkForRef('method:ListGrid.clearCriteria')} code paths.",
        name:"fetchDataPrompt",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Finding Records that match your criteria...\"",
        type:"classAttr",
        definingClass:"class:RPCManager",
        groups:["i18nMessages"]
    },
    "group:changeHandling":{
        ref:"group:changeHandling",
        refs:["attr:FormItem.validateOnChange"],
        name:"changeHandling",
        seeAlso:[],
        type:"group"
    },
    "attr:RPCRequest.params":{
        ref:"attr:RPCRequest.params",
        flags:"IRW",
        description:"Values to be sent as simple HTTP params, as a JavaScript Object where each property/value\n pair will become an HTTP parameter name and value.  These parameters are then accessible on\n the server, for example, using servletRequest.getParameter(paramName) in Java Servlets.  \n <P>\n Array-valued parameters will be submitted as multiple instances of the same parameter,\n similar to an HTML form with a multi-select (?paramName=value1&amp;paramName=value2 ...),\n accessible as getParameterValues(paramName) in Java Servlets.  Any non-atomic type, such as\n an Object, will be serialized to <a href='http://www.json.org/' onclick=\"window.open('http://www.json.org/');return false;\">JSON</a> by the\n ${isc.DocUtils.linkForRef('class:JSONEncoder')}.  If this isn't desirable, serialize the data in advance so that the\n value provided in <code>rpcRequest.params</code> is a String.\n <P>\n Note that this API is primarily used in combination with ${isc.DocUtils.linkForRef('attr:RPCRequest.useSimpleHttp')} -\n when contacting the SmartClient Server, use ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} instead, which provides\n full JavaScript &lt;-&gt; Java translation of arbitrary structures.\n <code>rpcRequest.params</code> can also be used with the SmartClient Server, where it\n provides an an opportunity to send additional data aside from the main\n ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} payload.  This is useful for adding data to DataSource requests which\n will be kept separate from the automatically sent DataSource data, or for making parts of\n the request visible in the URL for HTTP-level logging or layer 4 switches.\n <P>\n Note that in contrast to ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} object, the data in\n <code>rpcRequest.params</code> is not deserialized by the SmartClient server, and\n all values arrive on the server as String type (like HTTP parameters always do).\n <p>\n <span class=\"smartclient\">\n The params value can also be specified as a componentID or component instance that provides\n a method getValues() that returns an Object literal.  SmartClient components\n ${isc.DocUtils.linkForRef('class:DynamicForm')}, ${isc.DocUtils.linkForRef('class:ValuesManager')} are two such classes.  Lastly, you may\n specify the ID of a native form element (retreivable via getElementById()) and the params\n will be populated from there.  If there is an error resolving your params directive, it will\n be logged to the Developer Console.\n </span>\n <p>\n Note: The params are submitted once per http transaction.  If you are using \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue','request queuing')} to bundle multiple RPCRequests or DSRequests\n into a single HTTP turnaround, the params from the various RPCRequests will be merged,\n with the later-queued transactions winning on parameter name collisions.  A warning will be\n logged in the Developer Console if multiple RPCRequests specified params.",
        name:"params",
        seeAlso:[],
        valueType:"Object, others - see below",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:DynamicForm.setValueMap":{
        ref:"method:DynamicForm.setValueMap",
        flags:"",
        description:"		Set the valueMap for a specified item\n",
        name:"setValueMap",
        seeAlso:[],
        params:[
            {description:"itemName of the item upon which the valueMap should be set.",
             optional:false, name:"itemName", defaultValue:null, type:"string"},
            {description:"new valueMap for the field in question.", optional:false, name:"valueMap",
             defaultValue:null, type:"ValueMap"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "method:CubeGrid.getFacetValueLayout":{
        ref:"method:CubeGrid.getFacetValueLayout",
        flags:"",
        description:" Get the current visual order and width for the facet values of a facet or facetValueGroup as\n an object of the form:\n <pre>\n [ {id:facetValueId, width:currentWidth }, ... ]\n </pre>\n\n",
        name:"getFacetValueLayout",
        seeAlso:[],
        params:[
            {description:"Which facet do we want details for?", optional:false, name:"id",
             defaultValue:null, type:"facetValueGroupId | facetId"}
        ],
        returns:{description:"array of {id:facetValueId, width:width} objects", type:"array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:OperationBinding.requestProperties":{
        ref:"attr:OperationBinding.requestProperties",
        flags:"IR",
        description:"Additional properties to pass through to the ${isc.DocUtils.linkForRef('class:DSRequest')} created for this operation.\n Note that these will be cumulative with and will override on a per-property basis any\n properties set via ${isc.DocUtils.linkForRef('attr:DataSource.requestProperties')}.\n <p>\n These properties are applied before ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} is called.",
        name:"requestProperties",
        seeAlso:[
            {ref:"class:DSRequest", description:null},
            {ref:"attr:DataSource.requestProperties", description:null}
        ],
        valueType:"DSRequest Properties",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration", "serverDataIntegration"]
    },
    "attr:Facet.collapsed":{
        ref:"attr:Facet.collapsed",
        flags:"IR",
        description:"For tree facets, default collapse state for parent nodes",
        name:"collapsed",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Facet"
    },
    "method:ListGrid.setSelectionAppearance":{
        ref:"method:ListGrid.setSelectionAppearance",
        flags:"",
        description:" Changes selectionAppearance on the fly.\n",
        name:"setSelectionAppearance",
        seeAlso:[],
        params:[
            {description:"new selection appearance", optional:false, name:"selectionAppearance",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:FormItem.setHint":{
        ref:"method:FormItem.setHint",
        flags:"",
        description:" Set the hint text for this item\n",
        name:"setHint",
        seeAlso:[],
        params:[
            {description:"new hint for the item", optional:false, name:"hintText", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:FormItem"
    },
    "type:AnimationAcceleration":{
        ref:"type:AnimationAcceleration",
        description:"Acceleration effect for animations. Can either be a ratio function or a string.\n Ratio functions take a value between 0 and 1 which represents how much of the \n animation's duration has elapsed, and return another value between 0 and 1 indicating\n how close the animation is to completion. For a completely linear animation, the \n function would return the value it was passed. This allows you to bias animations to\n [for example] speed up toward the end of the animation.<br>\n The following strings are also supported for common ratio bias effects:\n <ul>\n <li>\"smoothStart\" - animation will speed up as time elapses\n <li>\"smoothEnd\" - animation will slow down as time elapses\n <li>\"smoothStartEnd\" - animation will speed up in the middle\n <li>\"none\" - no bias</ul>",
        name:"AnimationAcceleration",
        seeAlso:[],
        type:"type"
    },
    "type:PartialCommitOption":{
        ref:"type:PartialCommitOption",
        values:[
            {description:"Silently allow the partial commit to proceed (note that this will result                  in the user losing those records that contain errors)",
             value:"\"allow\""},
            {description:"Pop up a message window and prevent the partial commit",
             value:"\"prevent\""},
            {description:"Pop up a confirmation window and allow the user to choose whether or not to                  proceed",
             value:"\"prompt\""}
        ],
        description:"Action to take if a user attempts to save the dataset produced by a ${isc.DocUtils.linkForRef('class:BatchUploader')}\n whilst it still contains errors.",
        name:"PartialCommitOption",
        seeAlso:[],
        type:"type"
    },
    "method:DynamicForm.getValuesAsAdvancedCriteria":{
        ref:"method:DynamicForm.getValuesAsAdvancedCriteria",
        flags:"",
        description:" Return an AdvancedCriteria object based on the current set of values within this form.\n <p>\n Similar to ${isc.DocUtils.linkForRef('method:DynamicForm.getValuesAsCriteria')}, except the returned criteria object\n is guaranteed to be an AdvancedCriteria object, even if none of the form's fields has a\n specified ${isc.DocUtils.linkForRef('attr:FormItem.operator')}\n\n",
        name:"getValuesAsAdvancedCriteria",
        seeAlso:[],
        returns:{description:"a ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} based on the form's current values",
         type:"AdvancedCriteria"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:DatabaseBrowser.dataGrid":{
        ref:"attr:DatabaseBrowser.dataGrid",
        flags:"IR",
        description:"Instance of ListGrid used to display the actual data in the table seleted in the \n ${isc.DocUtils.linkForRef('attr:DatabaseBrowser.schemaTree','schemaTree')}.",
        name:"dataGrid",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "method:DynamicForm.submitForm":{
        ref:"method:DynamicForm.submitForm",
        flags:"",
        description:" Submits the form to the URL defined by ${isc.DocUtils.linkForRef('attr:DynamicForm.action')}, \n identically to how a plain HTML &lt;form&gt; element would submit data,\n as either an HTTP GET or POST as specified by ${isc.DocUtils.linkForRef('attr:DynamicForm.method')}.\n <P>\n <b>Notes:</b>\n <ul>\n <li>this is used only in the very rare case that a form is used to submit data\n directly to a URL.  Normal server contact is through \n ${isc.DocUtils.linkForRef('group:dataBoundComponentMethods','DataBound Component Methods')}.</li>\n <li>For this method to reliably include values for every field in the grid, \n      ${isc.DocUtils.linkForRef('attr:DynamicForm.canSubmit')} must be set to <code>true</code></li>\n <li>To submit values for fields that do not have an editor, use ${isc.DocUtils.linkForRef('class:HiddenItem')} \n with a ${isc.DocUtils.linkForRef('attr:FormItem.defaultValue')} set.  This is analagous to &lt;input type=\"hidden\"&gt;\n in HTML forms.\n </ul>\n",
        name:"submitForm",
        seeAlso:[],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["submitting"]
    },
    "method:SectionItem.isExpanded":{
        ref:"method:SectionItem.isExpanded",
        flags:"",
        description:" Returns a boolean indicating whether this SectionItem is expanded.\n\n",
        name:"isExpanded",
        seeAlso:[],
        returns:{description:"true if the section is expanded false if not", type:"boolean"},
        type:"method",
        definingClass:"class:SectionItem"
    },
    "method:Window.closeClick":{
        ref:"method:Window.closeClick",
        flags:"",
        description:" Handles a click on the close button of this window. The default implementation\n hides the window and returns false to cancel bubbling.  Override this method if you want\n other actions to be taken.\n",
        name:"closeClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:Window",
        groups:["buttons"]
    },
    "attr:DataBoundComponent.exportFields":{
        ref:"attr:DataBoundComponent.exportFields",
        flags:"IRW",
        description:"The list of field-names to export.  If provided, the field-list in the exported output is \n limited and sorted as per the list.\n <P>\n If exportFields is not provided, the exported output includes all visible fields \n from this component, sorted as they appear.",
        name:"exportFields",
        seeAlso:[],
        valueType:"Array of String",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:DataBoundComponent"
    },
    "method:Progressbar.setPercentDone":{
        ref:"method:Progressbar.setPercentDone",
        flags:"",
        description:" Sets percentDone to newPercent.\n\n",
        name:"setPercentDone",
        seeAlso:[],
        params:[
            {description:"percent to show as done (0-100)", optional:false, name:"newPercent",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Progressbar"
    },
    "attr:DataSourceField.ignore":{
        ref:"attr:DataSourceField.ignore",
        flags:"IR",
        description:"Whether this field should be completely excluded from this dataSource, as if it had never been\n defined.",
        name:"ignore",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSourceField",
        groups:["componentBinding"]
    },
    "attr:FormItem.timeFormatter":{
        ref:"attr:FormItem.timeFormatter",
        flags:"IRWA",
        description:"Time-format to apply to date type values within this formItem.  If specified, any\n dates displayed in this item will be formatted as times using the appropriate format.<br>\n Has no effect if +lilnk{formItem.dateFormatter} is also set.<br>\n See also ${isc.DocUtils.linkForRef('attr:TimeItem.displayFormat')} for formatting values displayed in TimeItems.",
        name:"timeFormatter",
        seeAlso:[],
        valueType:"TimeFormatter",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "attr:Label.iconWidth":{
        ref:"attr:Label.iconWidth",
        flags:"IR",
        description:"Width in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "attr:BatchUploader.partialCommitPrompt":{
        ref:"attr:BatchUploader.partialCommitPrompt",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:BatchUploader.partialCommit')} is set to \"prompt\", the text to display to the user in the \n confirmation dialog.  By default, this text is \"There are errors in your data so it \n cannot all be saved.  If you proceed, you will lose the records with errors.  Click 'OK'\n to proceed anyway, or 'Cancel' to return to your data\"",
        name:"partialCommitPrompt",
        seeAlso:[],
        valueType:"String",
        defaultValue:"See below",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "attr:MenuButton.menu":{
        ref:"attr:MenuButton.menu",
        flags:"IRW",
        description:"The menu to show.",
        name:"menu",
        seeAlso:[],
        valueType:"Menu",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MenuButton"
    },
    "method:FormItem.changed":{
        ref:"method:FormItem.changed",
        flags:"",
        description:" Called when a FormItem's value has been changed as the result of user interaction.  This\n method fires after the newly specified value has been stored.\n",
        name:"changed",
        seeAlso:[],
        params:[
            {description:"the managing DynamicForm instance", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"the form item itself (also available as \"this\")", optional:false,
             name:"item", defaultValue:null, type:"FormItem"},
            {description:"The current value (after the change).", optional:false, name:"value",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["eventHandling"]
    },
    "method:Toolbar.selectButton":{
        ref:"method:Toolbar.selectButton",
        flags:"",
        description:" Given an identifier for a button, select it.\n The button identifier can be a number (index), string (id), or object (widget or init block),\n as with the getButtonNumber() method.\n \n",
        name:"selectButton",
        seeAlso:[
            {ref:"method:Toolbar.getButtonNumber", description:null}
        ],
        params:[
            {description:"Button / Button identifier", optional:false, name:"buttonID",
             defaultValue:null, type:"number | string | object | canvas"}
        ],
        type:"method",
        definingClass:"class:Toolbar",
        groups:["selection"]
    },
    "type:LogPriority":{
        ref:"type:LogPriority",
        values:[
            {description:"unrecoverable error", value:"Log.FATAL"},
            {description:"error, may be recoverable", value:"Log.ERROR"},
            {description:"apparent problem, misused API, partial result", value:"Log.WARN"},
            {description:"significant events in normal operation", value:"Log.INFO"},
            {description:"diagnostics for developers", value:"Log.DEBUG"}
        ],
        description:"Priority levels for log messages",
        name:"LogPriority",
        seeAlso:[
            {ref:"method:Class.logDebug", description:null}
        ],
        type:"type"
    },
    "attr:Scrollbar.btnSize":{
        ref:"attr:Scrollbar.btnSize",
        flags:"IRW",
        description:"The size of the square buttons (arrows) at the ends of this scrollbar. This \n overrides the width of a vertical scrollbar or the height of a horizontal scrollbar\n to set the scrollbar's thickness.",
        name:"btnSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["track"]
    },
    "classAttr:GroupingMessages.byMonthTitle":{
        ref:"classAttr:GroupingMessages.byMonthTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by month.",
        name:"byMonthTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Month\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "method:Canvas.adjustForContent":{
        ref:"method:Canvas.adjustForContent",
        flags:"A",
        description:" This method tells a component to adjust for size changes made to content by external code.\n <P>\n This is for very advanced use in which the contents of a Canvas are being directly updated by\n Ajax techniques, which is required for integration with some third-party libraries.\n Calling this method is required because browsers do not provide consistent events by which\n SmartClient can be notified of external content changes.\n <P>\n Only contents supplied to a widget via ${isc.DocUtils.linkForRef('attr:Canvas.contents')} or via an override of\n ${isc.DocUtils.linkForRef('method:Canvas.getInnerHTML')} should be manipulated directly.  Contents automatically\n generated by SmartClient components (such as the basic structure of a Button) should never be\n manipulated: these structures are considered internal, differ by platform, and will change\n without notice.\n",
        name:"adjustForContent",
        seeAlso:[],
        params:[
            {description:"By default the adjustment will occur on a small delay for performance reasons. \n  Pass in this parameter to force immediate adjustment.",
             optional:false, name:"immediate", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:ListGrid.getDragData":{
        ref:"method:ListGrid.getDragData",
        flags:"",
        description:"\n During a drag-and-drop interaction, this method returns the set of records being dragged out\n of the component.  In the default implementation, this is the list of currently selected\n records.<p>\n \n This method is consulted by ${isc.DocUtils.linkForRef('method:ListGrid.willAcceptDrop')}.\n\n",
        name:"getDragData",
        seeAlso:[],
        params:[
            {description:"source component from which the records will be tranferred", optional:false,
             name:"source", defaultValue:null, type:"DataBoundComponent"}
        ],
        returns:{description:"Array of ${isc.DocUtils.linkForRef('object:Record')}s that are currently selected.",
         type:"Array of Record"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dragging", "data"]
    },
    "method:MultiView.deselectAllRecords":{
        ref:"method:MultiView.deselectAllRecords",
        flags:"",
        description:"\n Deselect all records\n\n",
        name:"deselectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:MultiView",
        groups:["dataBoundComponentMethods"]
    },
    "attr:TileGrid.tileValueAlign":{
        ref:"attr:TileGrid.tileValueAlign",
        flags:"IR",
        description:"Horizontal alignment for tile values: \"left\", \"right\" or \"center\".",
        name:"tileValueAlign",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"class:TileGrid"
    },
    "attr:ListGridField.valueIconRightPadding":{
        ref:"attr:ListGridField.valueIconRightPadding",
        flags:"IRW",
        description:"How much padding should there be on the right of valueIcons for this field\n Overrides ${isc.DocUtils.linkForRef('attr:ListGrid.valueIconRightPadding')}",
        name:"valueIconRightPadding",
        seeAlso:[
            {ref:"attr:ListGridField.valueIcons", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "method:Canvas.markForDestroy":{
        ref:"method:Canvas.markForDestroy",
        flags:"",
        description:" ${isc.DocUtils.linkForRef('method:Canvas.destroy','destroy()')} this canvas on a timeout. This method should be used instead of\n calling <code>canvas.destroy()</code> directly unless there's a reason a the canvas needs to be\n destroyed synchronously. By using a timeout, this method ensures the <code>destroy()</code> will\n occur after the current thread of execution completes. This allows you to easily mark canvases\n for destruction while they're handling events, which must complete before the canvas can be\n destroyed.\n",
        name:"markForDestroy",
        seeAlso:[
            {ref:"method:Canvas.destroy", description:null}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "classMethod:Page.getAppFilesDir":{
        ref:"classMethod:Page.getAppFilesDir",
        flags:"",
        description:" Returns the directory for application-specific files (other than images).\n\n",
        name:"getAppFilesDir",
        seeAlso:[],
        params:[
            {description:"New app files URL.", optional:true, name:"URL", defaultValue:null,
             type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page",
        groups:["files", "images"]
    },
    "attr:CubeGrid.bodyMinHeight":{
        ref:"attr:CubeGrid.bodyMinHeight",
        flags:"IRWA",
        description:"Minimum height for the body of this cubeGrid",
        name:"bodyMinHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:Slider.animateThumb":{
        ref:"attr:Slider.animateThumb",
        flags:"IRW",
        description:"Should the thumb be animated to its new position when the value is changed programmatically,\n or by clicking in the slider track.",
        name:"animateThumb",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider",
        groups:["animation"]
    },
    "attr:StatefulCanvas.showDisabledIcon":{
        ref:"attr:StatefulCanvas.showDisabledIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image if the button becomes\n disabled.",
        name:"showDisabledIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "method:DataBoundComponent.getTitleFieldValue":{
        ref:"method:DataBoundComponent.getTitleFieldValue",
        flags:"",
        description:" Get the value of the titleField for the passed record\n <P>\n Override in subclasses \n\n",
        name:"getTitleFieldValue",
        seeAlso:[],
        params:[
            {description:"the record whose index is to be retrieved", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"valuethe value of the titleField for the passed record", type:"String"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:ListGrid.headerSpanConstructor":{
        ref:"attr:ListGrid.headerSpanConstructor",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('type:SCClassName','SmartClient Class')} to use for headerSpans.  Typically a ${isc.DocUtils.linkForRef('class:Button')} or\n ${isc.DocUtils.linkForRef('class:StretchImgButton')} subclass.\n <P>\n If unset, headerSpans will be created using the ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor')}.",
        name:"headerSpanConstructor",
        seeAlso:[],
        valueType:"SCClassName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerSpan"]
    },
    "classAttr:Hover.topOffset":{
        ref:"classAttr:Hover.topOffset",
        flags:"RW",
        description:"When positioining the hover canvas, this will be the default top offset from the \n mousepointer, if no explicit position was passed to the ${isc.DocUtils.linkForRef('classMethod:Hover.show')} method",
        name:"topOffset",
        seeAlso:[],
        valueType:"number",
        defaultValue:"15",
        type:"classAttr",
        definingClass:"class:Hover"
    },
    "method:TextItem.getSelectionRange":{
        ref:"method:TextItem.getSelectionRange",
        flags:"",
        description:" For text-based items, this method returns the indices of the start/end of the current\n selection. Returns null if the item doesn't have focus. \n",
        name:"getSelectionRange",
        seeAlso:[],
        returns:{description:"2 element array showing character index of the current selection's  start and end point within this item's value. May be null if the item doesn't have focus.",
         type:"array | null"},
        type:"method",
        definingClass:"class:TextItem"
    },
    "method:Canvas.isDrawn":{
        ref:"method:Canvas.isDrawn",
        flags:"",
        description:"      Returns the boolean true, if the widget has been completely drawn, and false otherwise.\n",
        name:"isDrawn",
        seeAlso:[],
        returns:{description:"true if drawn, false if not drawn", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "class:VStack":{
        methods:[],
        exampleConfig:"VStack.xml",
        ref:"class:VStack",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:[],
        description:"A subclass of Layout that simply stacks members on the vertical axis without trying to\n manage their height.  On the horizontal axis, any members that do not have explict widths\n will be sized to match the width of the stack.",
        name:"VStack",
        seeAlso:[
            {ref:"attr:Layout.vPolicy", description:null}
        ],
        type:"class"
    },
    "method:RestDataSource.transformRequest":{
        ref:"method:RestDataSource.transformRequest",
        flags:"",
        description:" RestDataSource overrides transformRequest and handles serializing the request in the\n appropriate format (determined by the specified\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','dataProtocol')}), including the submitted\n ${isc.DocUtils.linkForRef('attr:DSRequest.data','data')} as well as the meta data parameters, which may include -<br> \n ${isc.DocUtils.linkForRef('attr:DSRequest.dataSource','dataSource')},\n ${isc.DocUtils.linkForRef('attr:DSRequest.operationType','operationType')}, ${isc.DocUtils.linkForRef('attr:DSRequest.operationId','operationId')};<br>\n ${isc.DocUtils.linkForRef('attr:DSRequest.startRow','startRow')} and ${isc.DocUtils.linkForRef('attr:DSRequest.endRow','endRow')} (for fetches);<br>\n ${isc.DocUtils.linkForRef('attr:DSRequest.sortBy','sortBy')} and ${isc.DocUtils.linkForRef('attr:DSRequest.textMatchStyle','textMatchStyle')} \n (for fetches);<br>\n ${isc.DocUtils.linkForRef('attr:DSRequest.oldValues','oldValues')} (for update and remove operations);<br>\n and possibly ${isc.DocUtils.linkForRef('attr:DSRequest.componentId','componentId')}.\n <P>\n If you override this method in order to add additional data to the DSRequest, you must\n call ${isc.DocUtils.linkForRef('method:Class.Super','Super()')} or you will remove the functionality provided by\n RestDataSource.  For example:\n <pre>\n    transformRequest : function (dsRequest) {\n        // modify dsRequest.data here, for example, add fixed criteria\n        dsRequest.data.userId = myApplication.getCurrentUserId();\n  \n        return this.Super(\"transformRequest\", arguments);\n    }\n </pre>\n <P>\n See ${isc.DocUtils.linkForRef('class:RestDataSource','RestDataSource overview')} for a description of the \n standard formatting applied to requests.\n\n",
        name:"transformRequest",
        seeAlso:[],
        type:"method",
        definingClass:"class:RestDataSource"
    },
    "attr:StatefulCanvas.showOverCanvas":{
        ref:"attr:StatefulCanvas.showOverCanvas",
        flags:"IRWA",
        description:"When this property is set to true, this widget will create and show the\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.overCanvas')} on user rollover.",
        name:"showOverCanvas",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas"
    },
    "group:iteration":{
        ref:"group:iteration",
        title:"Iteration",
        refs:["method:ResultSet.getProperty", "method:List.getProperty", "method:Array.getProperty", 
         "method:Array.map","method:Array.setProperty", "method:Array.clearProperty"],
        description:"Operations on entire Arrays at once",
        name:"iteration",
        seeAlso:[],
        type:"group"
    },
    "type:listGridSortState":{
        ref:"type:listGridSortState",
        description:"An object containing the stored sort information for a listGrid.\n Note that this object is not intended to be interrogated directly, but may be stored \n (for example) as a blob on the server for state persistence across sessions.",
        name:"listGridSortState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "attr:DateChooser.headerHeight":{
        ref:"attr:DateChooser.headerHeight",
        flags:"IR",
        description:"Height of the header area (containing the next/previous month & year buttons) in pixels",
        name:"headerHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "method:TabSet.getSelectedTabNumber":{
        ref:"method:TabSet.getSelectedTabNumber",
        flags:"A",
        description:" Returns the index of the currently selected tab object.  \n",
        name:"getSelectedTabNumber",
        seeAlso:[],
        returns:{description:"the index of the currently selected tab object", type:"number"},
        type:"method",
        definingClass:"class:TabSet"
    },
    "method:Window.setSrc":{
        ref:"method:Window.setSrc",
        flags:"",
        description:" Sets the URL of the contents to display in the body of the window, redrawing if\n necessary.\n",
        name:"setSrc",
        seeAlso:[],
        params:[
            {description:"URL of new contents to be displayed in the window body", optional:false,
             name:"url", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "classMethod:WebService.getByName":{
        ref:"classMethod:WebService.getByName",
        flags:"",
        description:" Retrieve a WebService object by the name attribute declared on the &lt;wsdl:service&gt; tag.\n\n",
        name:"getByName",
        seeAlso:[],
        params:[
            {description:"name attribute from the &lt;wsdl:service&gt; tag", optional:false,
             name:"serviceName", defaultValue:null, type:"String"},
            {description:"optional serviceNamespace if needed to disambiguate", optional:true,
             name:"serviceNamespace", defaultValue:null, type:"String"}
        ],
        returns:{description:"the requested WebService, or null if not loaded", type:"WebService"},
        type:"classMethod",
        definingClass:"class:WebService",
        groups:["webService"]
    },
    "type:treeGridViewState":{
        ref:"type:treeGridViewState",
        description:"An object containing the \"view state\" information for a treeGrid. In addition to the \n state data contained by a ${isc.DocUtils.linkForRef('type:listGridViewState')} object, this will also contain the \n current open state of the treeGrid in question.<br>\n Note that this object is not intended to be interrogated directly, but may be stored \n (for example) as a blob on the server for view state persistence across sessions.",
        name:"treeGridViewState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "attr:ListGridField.required":{
        ref:"attr:ListGridField.required",
        flags:"IRW",
        description:"When the user edits cells in this field, is this value required to be non-empty \n in order for validation to pass.<br>\n Note: for databound listGrids, this property may be specified on the \n ${isc.DocUtils.linkForRef('class:DataSourceField')}, enabling both client and server side validation.",
        name:"required",
        seeAlso:[
            {ref:"attr:ListGridField.validators", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["gridValidation"]
    },
    "method:ColumnTree.getNodeTitle":{
        ref:"method:ColumnTree.getNodeTitle",
        flags:"",
        description:"\n Returns the title to show for a node in the ColumnTree.  The default implementation\n returns the result of calling ${isc.DocUtils.linkForRef('method:Tree.getTitle')} on the node.\n <br><br>\n You can override this method to return a custom title for nodes in the tree.\n\n",
        name:"getNodeTitle",
        seeAlso:[
            {ref:"method:Tree.getTitle", description:null}
        ],
        params:[
            {description:"The node for which the title is being requested.", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"The index of the node.", optional:false, name:"recordNum", defaultValue:null,
             type:"Number"},
            {description:"The field for which the title is being requested.", optional:false,
             name:"field", defaultValue:null, type:"DSField"}
        ],
        returns:{description:"the title to display.", type:"HTML"},
        type:"method",
        definingClass:"class:ColumnTree"
    },
    "method:ImgButton.setIconOrientation":{
        ref:"method:ImgButton.setIconOrientation",
        flags:"",
        description:" Changes the orientation of the icon relative to the text of the button.\n\n",
        name:"setIconOrientation",
        seeAlso:[],
        params:[
            {description:"The new orientation of the icon relative to the text of the button.",
             optional:false, name:"orientation", defaultValue:null, type:"\"left\" or \"right\""}
        ],
        type:"method",
        definingClass:"class:ImgButton",
        groups:["buttonIcon"]
    },
    "attr:EdgedCanvas.edgeColor":{
        ref:"attr:EdgedCanvas.edgeColor",
        flags:"IR",
        description:"CSS color (WITHOUT \"#\") for the edges.  If specified, will be used as part of image\n names.  Example: \"edge_88FF88_TL.gif\".",
        name:"edgeColor",
        seeAlso:[],
        valueType:"cssColor",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "classMethod:Class.addPropertyList":{
        ref:"classMethod:Class.addPropertyList",
        flags:"",
        description:"\n	Add default properties to all instances of this class\n\n",
        name:"addPropertyList",
        seeAlso:[],
        params:[
            {description:"listarray of objects with properties to add", optional:true, name:"",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"the class after properties have been added to it", type:"object"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:Layout.resizeBarSize":{
        ref:"attr:Layout.resizeBarSize",
        flags:"AIRW",
        description:"Thickness of the resizeBars in pixels",
        name:"resizeBarSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:Layout"
    },
    "attr:Canvas.dragScrollDelay":{
        ref:"attr:Canvas.dragScrollDelay",
        flags:"IRWA",
        description:"If this widget supports drag-scrolling, This property specifies how many ms the\n      user must hover over the drag-scroll threshold before scrolling begins.",
        name:"dragScrollDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragging"]
    },
    "attr:DSResponse.startRow":{
        ref:"attr:DSResponse.startRow",
        flags:"R",
        description:"Starting row of returned server results, when using paged result fetching\n <p>\n Note that startRow and endRow are zero-based - the first record is row zero.",
        name:"startRow",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["paging"]
    },
    "attr:StatefulCanvas.showFocusedIcon":{
        ref:"attr:StatefulCanvas.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["buttonIcon"]
    },
    "classMethod:Class.getInstanceProperty":{
        ref:"classMethod:Class.getInstanceProperty",
        flags:"",
        description:"\n	Gets a named property from the instance defaults for this object.\n\n",
        name:"getInstanceProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to return", optional:false, name:"property",
             defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:Array.find":{
        ref:"method:Array.find",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:Array.findIndex')}, but returns the object itself instead of its index.\n\n",
        name:"find",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"first matching object or null if not found", type:"Object"},
        type:"method",
        definingClass:"object:Array",
        groups:["access", "find"]
    },
    "attr:DataSource.preventHTTPCaching":{
        ref:"attr:DataSource.preventHTTPCaching",
        flags:"IR",
        description:"If set, the DataSource will ensure that it never uses a cached HTTP response, even if the\n server marks the response as cacheable.\n <P>\n Note that this does not disable caching at higher levels in the framework, for example, the\n caching performed by ${isc.DocUtils.linkForRef('class:ResultSet')}.",
        name:"preventHTTPCaching",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "method:Canvas.scrollToTop":{
        ref:"method:Canvas.scrollToTop",
        flags:"",
        description:" Vertically scrolls the content of the widget to 0\n\n",
        name:"scrollToTop",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:ListGrid.headerMenuButtonIconHeight":{
        ref:"attr:ListGrid.headerMenuButtonIconHeight",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, this property governs the height of the icon\n shown on the auto-generated <code>headerMenuButton</code>",
        name:"headerMenuButtonIconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "method:ListGrid.getValueIcon":{
        ref:"method:ListGrid.getValueIcon",
        flags:"",
        description:" Returns the appropriate valueIcon for a cell based on the field and the data value for the \n cell. Default implementation returns null if ${isc.DocUtils.linkForRef('attr:ListGridField.suppressValueIcon')} is true\n otherwise looks at ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')}.\n",
        name:"getValueIcon",
        seeAlso:[],
        params:[
            {description:"field associated with the cell", optional:false, name:"field",
             defaultValue:null, type:"ListGridField"},
            {description:"data value for the cell's record in this field.", optional:false,
             name:"value", defaultValue:null, type:"any"},
            {description:"record associated with this cell", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "classMethod:Log.getStackTrace":{
        ref:"classMethod:Log.getStackTrace",
        flags:"",
        description:" Returns a \"stack trace\" - one line per method in the current call stack, showing the method\n name and any parameters passed.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.getStackTrace\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n <P>\n This API can currently only be supported in Internet Explorer.  See\n ${isc.DocUtils.linkForRef('group:debugging')} for information on how to get equivalent information from other\n browsers.\n\n",
        name:"getStackTrace",
        seeAlso:[],
        returns:{description:"stack trace.  Use eg ${isc.DocUtils.linkForRef('method:Class.logWarn')} to log to the Developer Console.",
         type:"String"},
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "attr:SectionItem.sectionVisible":{
        ref:"attr:SectionItem.sectionVisible",
        deprecated:"As of SmartClient version 5.5, use ${isc.DocUtils.linkForRef('attr:SectionItem.sectionExpanded')}             instead.",
        flags:"IR",
        description:"Whether this form section should initially be visible.",
        name:"sectionVisible",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:SectionItem"
    },
    "method:ListGrid.formatEditorValue":{
        ref:"method:ListGrid.formatEditorValue",
        flags:"",
        description:" Formatter to apply to values displayed within editors while a cell is being edited.\n The value passed to this method is the raw value for the cell.<br>\n If <code>formatEditorValue</code> is defined at the field level for some cell being edited,\n the field level method will be used to format the edit value and this method will not\n be called for that cell.<br>\n To convert the formatted value displayed within an editor back to a raw value, the\n <code>parseEditorValue</code> method is used.\n\n",
        name:"formatEditorValue",
        seeAlso:[
            {ref:"method:ListGridField.formatEditorValue", description:null},
            {ref:"method:ListGrid.parseEditorValue", description:null}
        ],
        params:[
            {description:"raw value for the cell being edited", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"Record object for the cell. Note: If this is a new row that has not been saved, \n   it has no associated record object. In this case the edit values will\n   be passed in as this parameter.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number for the cell.", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"formatted value to display in the editor", type:"any"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "group:tabBar":{
        ref:"group:tabBar",
        refs:["attr:TabSet.tabBarPosition", "attr:TabSet.tabBarAlign", "attr:TabSet.tabBarThickness", 
         "attr:TabSet.selectedTab"],
        name:"tabBar",
        seeAlso:[],
        type:"group"
    },
    "attr:PickTreeItem.emptyMenuMessage":{
        ref:"attr:PickTreeItem.emptyMenuMessage",
        flags:"IRA",
        description:"This message will be displayed as a single, disabled option in any empty menu/submenu\n created from this item's data tree.",
        name:"emptyMenuMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"No items to display\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('pickTree')}"],
        definingClass:"class:PickTreeItem"
    },
    "group:stringProcessing":{
        ref:"group:stringProcessing",
        refs:["method:Number.stringify", "method:Number.toCurrencyString", "method:String.contains", 
         "method:String.startsWith","method:String.endsWith"],
        name:"stringProcessing",
        seeAlso:[],
        type:"group"
    },
    "attr:Toolbar.vertical":{
        ref:"attr:Toolbar.vertical",
        flags:"IRW",
        description:"Indicates whether the buttons are drawn horizontally from left to right (false), or\n vertically from top to bottom (true).",
        name:"vertical",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Toolbar",
        groups:["appearance"]
    },
    "method:Array.slide":{
        ref:"method:Array.slide",
        flags:"A",
        description:" Slide element at position start to position destination, moving all the other elements to cover\n the gap.\n\n",
        name:"slide",
        seeAlso:[],
        params:[
            {description:"start position", optional:false, name:"start", defaultValue:null,
             type:"number"},
            {description:"destinationdestination position for this", optional:true, name:"start",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"object:Array"
    },
    "method:ResultSet.get":{
        ref:"method:ResultSet.get",
        flags:"",
        description:" Returns the record at the specified position.\n <P>\n All List access methods of the ResultSet have the semantics described in <code>getRange()</code>.\n",
        name:"get",
        seeAlso:[
            {ref:"method:ResultSet.getRange", description:null}
        ],
        params:[
            {description:"position of the element to get", optional:false, name:"pos",
             defaultValue:null, type:"Number"}
        ],
        returns:{description:"whatever's at that position, undef if not found", type:"object"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access"]
    },
    "attr:Tab.canClose":{
        ref:"attr:Tab.canClose",
        flags:"IRW",
        description:"Determines whether this tab should show an icon allowing the user to dismiss the tab by\n clicking on it directly. The URL for this icon's image will be derived from \n ${isc.DocUtils.linkForRef('attr:TabSet.closeTabIcon')} by default, but may be overridden by explicitly specifying\n ${isc.DocUtils.linkForRef('attr:Tab.closeIcon')}.\n <P>\n If unset, this property is derived from ${isc.DocUtils.linkForRef('attr:TabSet.canCloseTabs')}",
        name:"canClose",
        seeAlso:[
            {ref:"method:TabSet.closeClick", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('closeableTabs')}"],
        definingClass:"object:Tab"
    },
    "attr:FormItem.pickerIconProperties":{
        ref:"attr:FormItem.pickerIconProperties",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this block of properties will\n be applied to the pickerIcon. Allows for advanced customization of this icon.",
        name:"pickerIconProperties",
        seeAlso:[],
        valueType:"FormItemIcon Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "object:Number":{
        methods:["method:Number.stringify", "method:Number.toCurrencyString"],
        ref:"object:Number",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Extra methods added to the Number object, available on all number variables.",
        name:"Number",
        seeAlso:[],
        type:"object"
    },
    "attr:DynamicForm.cellPadding":{
        ref:"attr:DynamicForm.cellPadding",
        flags:"IRW",
        description:"The amount of empty space, in pixels, surrounding each form item within its cell in\n the layout grid.",
        name:"cellPadding",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "method:GridRenderer.rowMouseDown":{
        ref:"method:GridRenderer.rowMouseDown",
        flags:"A",
        description:" Called when a row receives a mousedown event.\n \n",
        name:"rowMouseDown",
        seeAlso:[],
        params:[
            {description:"record object returned from 'getCellRecord()'", optional:false,
             name:"record", defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:ListGrid.groupStartOpen":{
        ref:"attr:ListGrid.groupStartOpen",
        flags:"IRW",
        description:"Describes the default state of ListGrid groups when groupBy is called. \n\n Possible values are:\n <ul>\n <li>\"all\": open all groups\n <li>\"first\": open the first group\n <li>\"none\": start with all groups closed \n <li>Array of values that should be opened\n </ul>",
        name:"groupStartOpen",
        seeAlso:[
            {ref:"method:ListGrid.groupBy", description:null}
        ],
        valueType:"String || Array",
        defaultValue:"\"first\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["grouping"]
    },
    "attr:Calendar.eventEditorFields":{
        ref:"attr:Calendar.eventEditorFields",
        flags:"IR",
        description:"The set of fields for the ${isc.DocUtils.linkForRef('attr:Calendar.eventEditor','event editor')}.\n <p>\n The default set of fields are: \n <pre>\n    {name: \"startHours\", title: \"From\", type: \"select\", width: 60},\n    {name: \"startMinutes\", showTitle: false, type: \"select\", width: 60},\n    {name: \"startAMPM\", showTitle: false, type: \"select\", width: 60},\n    {name: \"invalidDate\", type: \"blurb\", colSpan: 4, visible: false}\n    {name: \"endHours\", title: \"To\", type: \"select\", width: 60},\n    {name: \"endMinutes\", showTitle: false, type: \"select\", width: 60},\n    {name: \"endAMPM\", showTitle: false, type: \"select\", width: 60},\n    {name: \"name\", title: \"Name\", type: \"text\", colSpan: 4},\n    {name: \"description\", title: \"Description\", type: \"textArea\", colSpan: 4, height: 50}\n </pre>\n See the Customized Binding example below for more information on altering default datasource \n fields within forms.",
        name:"eventEditorFields",
        seeAlso:[],
        valueType:"Array of FormItem",
        defaultValue:"see below",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('customCalendar')}", 
         "${isc.DocUtils.linkForExampleId('validationFieldBinding')}"],
        definingClass:"class:Calendar",
        groups:["editing"]
    },
    "method:Menu.setItemChecked":{
        ref:"method:Menu.setItemChecked",
        flags:"",
        description:" Checks or unchecks the menu item according to the value of newState, and redraws\n the menu if necessary. Returns true if there's a change in the checked state.\n\n",
        name:"setItemChecked",
        seeAlso:[],
        params:[
            {description:"MenuItem in question, or it's index", optional:false, name:"item",
             defaultValue:null, type:"MenuItem or number"},
            {description:"true to check the menu item, false to uncheck it.  If not                             passed, true is assumed",
             optional:true, name:"newState", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"true if the checked state was changed", type:"boolean"},
        type:"method",
        definingClass:"class:Menu"
    },
    "attr:ListGrid.filterEditorHeight":{
        ref:"attr:ListGrid.filterEditorHeight",
        flags:"IRW",
        description:"Height for the filterEditor, if shown.",
        name:"filterEditorHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"22",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["filterEditor"]
    },
    "method:Canvas.markForRedraw":{
        ref:"method:Canvas.markForRedraw",
        flags:"",
        description:" Marks the widget as \"dirty\" so that it will be added to a queue for redraw. Redraw of dirty\n components is handled by a looping timer and will after a very short delay (typically less than\n 100ms). In most cases it is recommended that developers use <code>markForRedraw()</code>\n instead of calling ${isc.DocUtils.linkForRef('method:Canvas.redraw')} directly. Since this method queues the redraw, multiple\n calls to markForRedraw() within a single thread of excecution will only lead to a single DOM\n manipulation which greatly improves application performance.\n\n",
        name:"markForRedraw",
        seeAlso:[],
        params:[
            {description:"reason for performing the redraw", optional:true, name:"reason",
             defaultValue:"\"no reason provided\"", type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "attr:Tab.width":{
        ref:"attr:Tab.width",
        flags:"IRW",
        description:"You can specify an explicit width for the tab using this property.  Note that tabs\n automatically size to make room for the full title, but if you want to e.g. specify a\n uniform width for all tabs in a TabSet, this property enables you to do so.",
        name:"width",
        seeAlso:[],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"object:Tab"
    },
    "attr:TileLayout.tileHeight":{
        ref:"attr:TileLayout.tileHeight",
        flags:"IR",
        description:"Height of each tile in pixels.  See ${isc.DocUtils.linkForRef('attr:TileLayout.tileSize')}.\n If ${isc.DocUtils.linkForRef('type:LayoutPolicy')} is \"fit\", ${isc.DocUtils.linkForRef('attr:TileLayout.expandMargins')} is false, ${isc.DocUtils.linkForRef('attr:TileLayout.tilesPerLine')} is set, \n ${isc.DocUtils.linkForRef('type:Orientation')} is \"vertical\",\n and tileHeight is not set, tileHeight will be computed automatically based on ${isc.DocUtils.linkForRef('attr:TileLayout.tilesPerLine')}.",
        name:"tileHeight",
        setter:["setTileHeight"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["sizing"]
    },
    "method:DynamicForm.handleHiddenValidationErrors":{
        ref:"method:DynamicForm.handleHiddenValidationErrors",
        flags:"A",
        description:" Method to display validation error messages for fields that are not currently visible \n in this form.<br>\n This will be called when validation fails for<br>\n - a hidden field in this form<br>\n - if this form is databound, a datasource field with specified validators, for which we\n   have no specified form item.<br>\n Implement this to provide custom validation error handling for these fields.<br>\n By default hidden validation errors will be logged as warnings in the developerConsole.\n Return false from this method to suppress that behavior.\n",
        name:"handleHiddenValidationErrors",
        seeAlso:[],
        params:[
            {description:"The set of errors returned - this is an object of the form<br>                      &nbsp;&nbsp;<code>{fieldName:errors}</code><br>\n                      Where the 'errors' object is either a single string or an array\n                      of strings containing the error messages for the field.",
             optional:false, name:"errors", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "method:ValuesManager.addFieldErrors":{
        ref:"method:ValuesManager.addFieldErrors",
        flags:"",
        description:" Adds validation errors to the existing set of errors for the field in question.\n Errors passed in should be a string (for a single error message) or an array of strings.\n Pass in the showErrors parameter to immediately display the errors to the user by \n redrawing the appropriate member form item (or if no visible item is found for the field\n firing ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')}.\n",
        name:"addFieldErrors",
        seeAlso:[],
        params:[
            {description:"name of field to apply errors to", optional:false, name:"fieldName",
             defaultValue:null, type:"string"},
            {description:"error messages for the field", optional:false, name:"errors",
             defaultValue:null, type:"string | array of strings"},
            {description:"should the error(s) be immediately displayed to the user?", optional:false,
             name:"showErrors", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "method:Layout.hideDropLine":{
        ref:"method:Layout.hideDropLine",
        flags:"A",
        description:" Calling this method hides the dropLine shown during a drag and drop interaction with a\n Layout that has ${isc.DocUtils.linkForRef('attr:Layout.canDropComponents')} set to true.  This method is only useful for\n custom implementations of ${isc.DocUtils.linkForRef('method:Layout.drop')} as the default implementation calls this\n method automatically.\n\n",
        name:"hideDropLine",
        seeAlso:[],
        type:"method",
        definingClass:"class:Layout"
    },
    "method:CubeGrid.setFixedFacetValue":{
        ref:"method:CubeGrid.setFixedFacetValue",
        flags:"",
        description:"\n Modify fixedFacetValues for this cubeGrid.\n\n",
        name:"setFixedFacetValue",
        seeAlso:[],
        params:[
            {description:"facetId", optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"fixedFacetValue     New fixed value for the facet, to be added to cubeGrid.fixedFacetValues.  Default is the\n     rollup value for the facet.",
             optional:false, name:"fixedFacetValueId", defaultValue:null, type:"id"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:DataSource.quoteColumnNames":{
        ref:"attr:DataSource.quoteColumnNames",
        flags:"IR",
        description:"If set, tells the SQL engine to quote column names in all generated DML and DDL \n statements for this dataSource.  This will ensure that queries generated against\n tables that do not follow the database product's natural column-naming conventions \n will still work.\n <p>\n In general we recommend that you allow the database to use its natural naming scheme\n when creating tables (put more simply, just do not quote column names in the \n <code>CREATE TABLE</code> statement); if you do this, you will not need to worry about \n quoting column names when querying.  However, if you are dealing with pre-existing \n tables, or do not have control over the database naming conventions used, this property\n may become necessary.\n <p>\n <b>Note:</b> Only applicable to dataSources of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\".",
        name:"quoteColumnNames",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DataSource"
    },
    "method:DataBoundComponent.editFormulaField":{
        ref:"method:DataBoundComponent.editFormulaField",
        flags:"",
        description:" Method to display a ${isc.DocUtils.linkForRef('class:FormulaBuilder')} to edit a formula Field.  If the function is called\n without a paramater, a new field will be created when the formula is saved.\n\n",
        name:"editFormulaField",
        seeAlso:[],
        params:[
            {description:"Field to edit or null to add a new formula field", optional:false,
             name:"field", defaultValue:null, type:"Field"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["formulaFields"]
    },
    "attr:ListGrid.cellPadding":{
        ref:"attr:ListGrid.cellPadding",
        flags:"IRW",
        description:"The amount of empty space, in pixels, surrounding each value in its cell.",
        name:"cellPadding",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "method:ListGrid.bodyKeyPress":{
        ref:"method:ListGrid.bodyKeyPress",
        flags:"",
        description:" Handle a keyPress event on the body.  \n <P>\n Default implementation handles navigating between records with arrow keys, and activating\n records with space an enter.\n\n",
        name:"bodyKeyPress",
        seeAlso:[],
        params:[
            {description:"the event details", optional:false, name:"event", defaultValue:null,
             type:"ISCEvent"}
        ],
        returns:{description:"return value for the keyPress event.", type:"EventReturn"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "attr:TreeGridField.treeField":{
        ref:"attr:TreeGridField.treeField",
        flags:"IRW",
        description:"The field containing <code>treeField: true</code> will display the ${isc.DocUtils.linkForRef('class:Tree')}.  If no\n field specifies this property, if a field named after the ${isc.DocUtils.linkForRef('attr:Tree.titleProperty')} of\n the Tree is present in ${isc.DocUtils.linkForRef('attr:TreeGrid.fields')}, that field will show the tree.  Note that\n when using a DataSource, you typically define the title field via\n ${isc.DocUtils.linkForRef('attr:DataSource.titleField')} and the generated ${isc.DocUtils.linkForRef('class:ResultTree')} automatically uses this\n field.\n\n If none of the above rules apply, the first field in ${isc.DocUtils.linkForRef('attr:TreeGrid.fields')} is assigned to\n display the ${isc.DocUtils.linkForRef('class:Tree')}.",
        name:"treeField",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"see below",
        type:"attr",
        definingClass:"object:TreeGridField",
        groups:["treeField"]
    },
    "class:PasswordItem":{
        methods:[],
        exampleConfig:"PasswordItem.xml",
        ref:"class:PasswordItem",
        classAttrs:[],
        classMethods:[],
        attrs:[],
        description:"FormItem for password fields, where text input by the user should not be shown in readable text.",
        name:"PasswordItem",
        seeAlso:[],
        type:"class"
    },
    "type:EscapeKeyEditAction":{
        ref:"type:EscapeKeyEditAction",
        values:[
            {description:"cancels the current edit and discards edit values", value:"\"cancel\""},
            {description:"end editing", value:"\"done\""}
        ],
        description:"What to do if the user hits escape while editing a cell.",
        name:"EscapeKeyEditAction",
        seeAlso:[],
        type:"type",
        groups:["editing"]
    },
    "method:CubeGrid.getSelectedCells":{
        ref:"method:CubeGrid.getSelectedCells",
        flags:"",
        description:"\n  returns an array of the selected cell records\n\n  <br><i>methodType</i> getter\n",
        name:"getSelectedCells",
        seeAlso:[],
        returns:{description:"array of the selected cell records", type:"Array"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["selection"]
    },
    "method:Canvas.getRight":{
        ref:"method:Canvas.getRight",
        flags:"",
        description:" Return the right coordinate of this object as rendered, relative to its enclosing context,\n in pixels.\n\n",
        name:"getRight",
        seeAlso:[],
        returns:{description:"right coordinate", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning", "sizing"]
    },
    "method:Calendar.dateIsWorkday":{
        ref:"method:Calendar.dateIsWorkday",
        flags:"",
        description:" Should the parameter date be considered a workday? By default this method tries to find the\n parameter date day in ${isc.DocUtils.linkForRef('attr:Calendar.workdays')}, and returns true if found. Override this method to \n provide custom logic for determining workday, for example returning false on holidays.\n\n",
        name:"dateIsWorkday",
        seeAlso:[],
        params:[
            {description:"date to check for being a workday", optional:false, name:"date",
             defaultValue:null, type:"Date"}
        ],
        returns:{description:"true if date is a workday, false otherwise", type:"Boolean"},
        type:"method",
        definingClass:"class:Calendar"
    },
    "attr:ColorPicker.basicColorLabel":{
        ref:"attr:ColorPicker.basicColorLabel",
        flags:"IR",
        description:"The label shown above the basic color blocks.",
        name:"basicColorLabel",
        seeAlso:[],
        valueType:"text",
        defaultValue:"\"Basic Colors:\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Canvas.scrollToBottom":{
        ref:"method:Canvas.scrollToBottom",
        flags:"",
        description:" Vertically scrolls the content of the widget to the end of its content\n\n",
        name:"scrollToBottom",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "attr:ListGrid.wrapCells":{
        ref:"attr:ListGrid.wrapCells",
        flags:"IRWA",
        description:"Should content within cells be allowed to wrap?",
        name:"wrapCells",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('autofitValues')}"],
        definingClass:"class:ListGrid",
        groups:["cellStyling"]
    },
    "attr:DateChooser.skinImgDir":{
        ref:"attr:DateChooser.skinImgDir",
        flags:"IRWA",
        description:"Overridden directory where images for this widget (such as the month and year button icons)\n may be found.",
        name:"skinImgDir",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"images/common/\"",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:Canvas.percentSource":{
        ref:"attr:Canvas.percentSource",
        flags:"IRWA",
        description:"If this canvas has its size specified as a percentage, this property allows the user to\n explicitly designate another canvas upon which sizinng will be based.\n <P>\n If unset percentage sizing is based on<br>\n - the ${isc.DocUtils.linkForRef('attr:Canvas.masterElement','masterElement')} if there is one and\n   ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')} is set,<br> \n - otherwise on the amount of space available in this widget's parentElement, if this is\n   a child of some other widget<br>\n - otherwise the page size.",
        name:"percentSource",
        setter:["setPercentSource"],
        seeAlso:[
            {ref:"attr:Canvas.percentBox", description:null}
        ],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "class:StaticTextItem":{
        methods:[],
        ref:"class:StaticTextItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:StaticTextItem.wrap", "attr:StaticTextItem.clipValue", 
         "attr:StaticTextItem.textBoxStyle","attr:StaticTextItem.outputAsHTML"],
        description:"A FormItem that displays an uneditable value.",
        name:"StaticTextItem",
        seeAlso:[],
        type:"class"
    },
    "method:TreeGrid.canEditCell":{
        ref:"method:TreeGrid.canEditCell",
        flags:"",
        description:" Overridden to disallow editing of the ${isc.DocUtils.linkForRef('attr:TreeNode.name','name')} field of this grid's data\n tree. Also disallows editing of the auto-generated tree field, which displays the result\n of ${isc.DocUtils.linkForRef('method:Tree.getTitle')} on the node.\n",
        name:"canEditCell",
        seeAlso:[],
        returns:{description:"Whether to allow editing this cell", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:Date.toShortDateTime":{
        ref:"method:Date.toShortDateTime",
        flags:"",
        description:" Returns the datetime as a formatted string using the format set up via the \n <code>setShortDatetimeDisplayFormat()</code> method.\n",
        name:"toShortDateTime",
        seeAlso:[],
        params:[
            {description:"Optional Format for the date returned", optional:false, name:"format",
             defaultValue:null, type:"DateDisplayFormat"},
            {description:"If true, format the date using the timezone  Time.setDefaultDisplayTimezone() rather than the native browser locale. This is the\n  default behavior within SmartClient components for datetime fields.",
             optional:false, name:"useCustomTimezone", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "method:TreeGrid.nodeContextClick":{
        ref:"method:TreeGrid.nodeContextClick",
        flags:"",
        description:"\n This method is called when a context click occurs on a leaf or folder record.  Note that\n if you set up a callback for <code>nodeContextClick()</code> and\n e.g. ${isc.DocUtils.linkForRef('method:TreeGrid.leafContextClick')}, then both will fire (in that order) if a leaf\n is contextclicked - unless <code>nodeContextClick()</code> returns false, in which case\n no further contextClick callbacks will be called.\n\n",
        name:"nodeContextClick",
        seeAlso:[
            {ref:"method:TreeGrid.folderContextClick", description:null},
            {ref:"method:TreeGrid.leafContextClick", description:null}
        ],
        params:[
            {description:"The TreeGrid on which the contexclick occurred.", optional:false,
             name:"viewer", defaultValue:null, type:"TreeGrid"},
            {description:"The node (record) on which the contextclick occurred.", optional:false,
             name:"node", defaultValue:null, type:"TreeNode"},
            {description:"Index of the row where the contextclick occurred.", optional:false,
             name:"recordNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:CellSelection.selectSingleCell":{
        ref:"method:CellSelection.selectSingleCell",
        flags:"",
        description:"			select a single cell and deselect everything else\n",
        name:"selectSingleCell",
        seeAlso:[],
        params:[
            {description:"row index of cell to select", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"column index of cell to select", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection"]
    },
    "attr:DateChooser.todayButtonHeight":{
        ref:"attr:DateChooser.todayButtonHeight",
        flags:"IRW",
        description:"If set specifies a fixed height for the Today and Cancel buttons.",
        name:"todayButtonHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:DynamicForm.numCols":{
        ref:"attr:DynamicForm.numCols",
        flags:"IRW",
        description:"The number of columns of titles and items in this form's layout grid. A title and\n corresponding item each have their own column, so to display two form elements per\n row (each having a title and item), you would set this property to 4.",
        name:"numCols",
        seeAlso:[],
        valueType:"number",
        defaultValue:"2",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["tableLayout"]
    },
    "attr:TileLayout.layoutMargin":{
        ref:"attr:TileLayout.layoutMargin",
        flags:"IR",
        description:"A margin left around the outside of all tiles.",
        name:"layoutMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"5",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "attr:Window.hiliteHeaderStyle":{
        ref:"attr:Window.hiliteHeaderStyle",
        flags:"IRWA",
        description:"Highlight style for the Window header. Displayed when a window \n is ${isc.DocUtils.linkForRef('method:Window.flash','flashed')}",
        name:"hiliteHeaderStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"WindowHeader\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "classAttr:RPCResponse.STATUS_VALIDATION_ERROR":{
        ref:"classAttr:RPCResponse.STATUS_VALIDATION_ERROR",
        flags:"R",
        description:"Indicates a validation failure on the server.\n See the error handling section in ${isc.DocUtils.linkForRef('class:RPCManager','RPCManager documentation')}\n for more information.",
        name:"STATUS_VALIDATION_ERROR",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-4",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "method:ListGrid.cellDoubleClick":{
        ref:"method:ListGrid.cellDoubleClick",
        flags:"",
        description:" Called when a cell receives a double click event.\n\n",
        name:"cellDoubleClick",
        seeAlso:[],
        params:[
            {description:"Record object returned from getCellRecord()", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('cellClicks')}"],
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "classMethod:Time.setDefaultDisplayTimezone":{
        ref:"classMethod:Time.setDefaultDisplayTimezone",
        flags:"",
        description:" Sets the offset from UTC to use when formatting values of type ${isc.DocUtils.linkForRef('type:FieldType','datetime')} \n and ${isc.DocUtils.linkForRef('type:FieldType','time')} with standard display formatters.\n <P>\n If this method is never called, the default display timezone for times and datetimes will\n be derived from the native browser local timezone.\n",
        name:"setDefaultDisplayTimezone",
        seeAlso:[
            {ref:"group:dateFormatAndStorage", description:null}
        ],
        params:[
            {description:"offset from UTC. This should be a string in the format    <code>+/-HH:MM</code> for example <code>\"-08:00\"</code>",
             optional:false, name:"offset", defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "attr:ListGrid.expansionEditor":{
        ref:"attr:ListGrid.expansionEditor",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:DynamicForm')} for editing the details of a record\n in it's expanded section when\n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>editor</code>.  Note that only \n those fields which do not already appear in the grid will appear in the expanded section.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionEditorProperties</code> and \n <code>listGrid.expansionEditorDefaults</code>.\n <P>\n Note, however, that this is a multi-instance component (potentially one per record), \n so it is created using ${isc.DocUtils.linkForRef('method:Class.createAutoChild','createAutoChild()')} not \n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')}, and no default single instance is created by \n name on the grid.",
        name:"expansionEditor",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "classMethod:Time.createDate":{
        ref:"classMethod:Time.createDate",
        flags:"",
        description:" Creates a date object with the time set to the hours, minutes and seconds passed in.\n Unless the <code>UTCTime</code> parameter is passed in, parameters are assumed\n to specify the time in local display time -- offset from \n UTC by ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone','default display timezone')}.\n",
        name:"createDate",
        seeAlso:[],
        params:[
            {description:"Hours for the date (defaults to zero)", optional:true, name:"hours",
             defaultValue:null, type:"number"},
            {description:"Minutes for the date (defaults to zero)", optional:true, name:"minutes",
             defaultValue:null, type:"number"},
            {description:"Seconds for the date (defaults to zero)", optional:true, name:"seconds",
             defaultValue:null, type:"number"},
            {description:"Milliseconds for the date (defaults to zero)", optional:true,
             name:"milliseconds", defaultValue:null, type:"number"},
            {description:"If true, treat the time passed in as UTC time (so when the  time is displayed using normal formatters it will be offset by the specified\n  ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone','default display timezone')}). Otherwise assumes\n  time passed in is already in the specified display timezone.",
             optional:true, name:"UTCTime", defaultValue:null, type:"boolean"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "method:Date.toSerializeableDate":{
        ref:"method:Date.toSerializeableDate",
        flags:"A",
        description:" Return this date in 'serialized' format <code>YYYY-MM-DD HH:MM:SS</code>\n",
        name:"toSerializeableDate",
        seeAlso:[],
        returns:{description:"formatted date string", type:"String"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:TreeGrid.animateFolderTime":{
        ref:"attr:TreeGrid.animateFolderTime",
        flags:"IRW",
        description:"When animating folder opening / closing, if ${isc.DocUtils.linkForRef('attr:TreeGrid.animateFolderSpeed')} is not\n set, this property designates the duration of the animation in ms.",
        name:"animateFolderTime",
        seeAlso:[
            {ref:"attr:TreeGrid.animateFolderSpeed", description:null}
        ],
        valueType:"number",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["animation"]
    },
    "attr:ToolStrip.height":{
        ref:"attr:ToolStrip.height",
        flags:"IRW",
        description:"ToolStrips set a default ${isc.DocUtils.linkForRef('attr:Canvas.height','height')} to avoid being stretched by\n containing layouts.",
        name:"height",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:ToolStrip",
        groups:["sizing"]
    },
    "attr:OperationBinding.tableClause":{
        ref:"attr:OperationBinding.tableClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n table clause to use when constructing the SQL query to perform this operation.  The\n property should be a comma-separated list of tables and views, and you can use any\n special language constructs supported by the underlying database.  The server will \n insert the text of this property immediately after the \"FROM\" token.\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"tableClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:SelectItem.hiliteTextColor":{
        ref:"attr:SelectItem.hiliteTextColor",
        deprecated:"As of SmartClient version 5.5, if ${isc.DocUtils.linkForRef('attr:SelectItem.showFocused')} is true,  styling will be updated for this form item on focus. The hiliting effect can therefore\n  be achieved via ${isc.DocUtils.linkForRef('attr:SelectItem.textBoxStyle')} and ${isc.DocUtils.linkForRef('attr:SelectItem.controlStyle')} \n  instead.",
        flags:"IRWA",
        description:"Text color to apply to the select item's selected value when the SelectItem \n receives focus, if <code>hiliteOnFocus</code> is true.",
        name:"hiliteTextColor",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"white\"",
        type:"attr",
        definingClass:"class:SelectItem"
    },
    "method:EBay.setUseProduction":{
        ref:"method:EBay.setUseProduction",
        flags:"",
        description:"\n You can use web services in sandbox or production mode.  By default, this web service\n will work in sandbox mode.  Call this method with a true value to switch to production\n mode.\n\n",
        name:"setUseProduction",
        seeAlso:[],
        params:[
            {description:"true to use production, false to use sandbox", optional:false,
             name:"useProduction", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:EBay"
    },
    "attr:Scrollbar.thumbMinSize":{
        ref:"attr:Scrollbar.thumbMinSize",
        flags:"IRA",
        description:"The minimum pixel size of the draggable thumb regardless of how large the scrolling\n region becomes.",
        name:"thumbMinSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"12",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "class:SliderItem":{
        methods:["method:SliderItem.setMinValue", "method:SliderItem.setMaxValue", 
         "method:SliderItem.setNumValues"],
        ref:"class:SliderItem",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Forms/Form Items"],
        attrs:["attr:SliderItem.vertical", "attr:SliderItem.minValue", "attr:SliderItem.maxValue", 
         "attr:SliderItem.numValues","attr:SliderItem.roundValues", "attr:SliderItem.roundPrecision", 
         "attr:SliderItem.defaultValue","attr:SliderItem.sliderProperties"],
        description:"FormItem that uses a ${isc.DocUtils.linkForRef('class:Slider')} component to present an interface for picking\n from either a continuous range or a range with a small number of discrete values.",
        name:"SliderItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('sliderItem')}"]
    },
    "attr:Applet.scriptable":{
        ref:"attr:Applet.scriptable",
        flags:"IR",
        description:"Usable only in IE: if true allows JS->Java LiveConnect (other platforms that support\n  LiveConnect have it enabled by default and do not allow it to be disabled).",
        name:"scriptable",
        seeAlso:[
            {ref:"attr:Applet.mayScript", description:null}
        ],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        platformNotes:"Relevant only on IE.",
        definingClass:"class:Applet"
    },
    "attr:Canvas.animateShowTime":{
        ref:"attr:Canvas.animateShowTime",
        flags:"IRWA",
        description:"Default time for performing an animated show.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateShowTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:OperationBinding.sqlType":{
        ref:"attr:OperationBinding.sqlType",
        flags:"IR",
        description:"For dataSources of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\" and \"hibernate\" only, this \n property determines whether \"custom\" operations have their custom SQL or HQL sent to the  \n underlying database via a JDBC <code>executeQuery()</code> or a JDBC \n <code>executeUpdate()</code>.  The default value of null means the same as \"query\", so you\n only need to use this property when your custom SQL or HQL updates data.",
        name:"sqlType",
        seeAlso:[],
        valueType:"SQLType",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "attr:DynamicForm.itemHoverVAlign":{
        ref:"attr:DynamicForm.itemHoverVAlign",
        flags:"IRW",
        description:"Vertical text alignment for hovers shown for items",
        name:"itemHoverVAlign",
        seeAlso:[
            {ref:"attr:FormItem.hoverVAlign", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:DynamicForm.errors":{
        flags:"IRW",
        getter:["getErrors"],
        type:"attr",
        ref:"attr:DynamicForm.errors",
        description:"A property list of itemName:errorMessage pairs, specifying the set of error messages\n          displayed with the corresponding form elements. Each errorMessage may be either a\n          single string or an array of strings.",
        name:"errors",
        setter:["setErrors"],
        valueType:"array",
        seeAlso:[],
        defaultValue:"null",
        groups:["validation"],
        definingClass:"class:DynamicForm"
    },
    "attr:ListGrid.showGridSummary":{
        ref:"attr:ListGrid.showGridSummary",
        flags:"IRW",
        description:"Should this ListGrid show a summary row beneath the last record of the grid. This summary\n row will contain per-field summary information. See ${isc.DocUtils.linkForRef('attr:ListGridField.showGridSummary')} and\n ${isc.DocUtils.linkForRef('listGrid.getSummaryFunction')} for details on how the summary value to be displayed\n for each column will be caculated.\n <P>\n Note that the ${isc.DocUtils.linkForRef('attr:ListGrid.summaryRow','summaryRow autoChild')} will be created to actually\n display the summary row.",
        name:"showGridSummary",
        setter:["setShowGridSummary"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:DynamicForm.validateOnChange":{
        ref:"attr:DynamicForm.validateOnChange",
        flags:"IRW",
        description:"If true, form fields will be validated when each item's \"change\" handler is fired\n		as well as when the entire form is submitted or validated.<br>\n      Note that this property can also be set at the item level to enable finer granularity\n      validation in response to user interaction - if true at either level, validation\n      will occur on change.",
        name:"validateOnChange",
        seeAlso:[
            {ref:"attr:FormItem.validateOnChange", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["validation"]
    },
    "method:TabSet.enableTab":{
        ref:"method:TabSet.enableTab",
        flags:"",
        description:" If the specified tab is disabled, enable it now.\n",
        name:"enableTab",
        seeAlso:[
            {ref:"attr:Tab.disabled", description:null}
        ],
        params:[
            {description:null, optional:false, name:"tab", defaultValue:null, type:"Tab | number | ID"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "method:Progressbar.getBreadth":{
        ref:"method:Progressbar.getBreadth",
        flags:"",
        description:" Returns the current height of a horizontal progressbar, or width of a vertical progressbar.\n\n",
        name:"getBreadth",
        seeAlso:[],
        returns:{description:"the breadth of the progressbar", type:"number"},
        type:"method",
        definingClass:"class:Progressbar"
    },
    "attr:PickList.cachePickListResults":{
        ref:"attr:PickList.cachePickListResults",
        flags:"IR",
        description:"For databound pickLists (see ${isc.DocUtils.linkForRef('attr:PickList.optionDataSource')}), by default SmartClient\n will cache and re-use datasets shown by pickLists in an LRU (least recently used) caching\n pattern.\n <P>\n Setting this flag to false avoids this caching for situations where it is too aggressive.",
        name:"cachePickListResults",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "attr:FormItem.optionFilterContext":{
        ref:"attr:FormItem.optionFilterContext",
        flags:"IRA",
        description:"If this item has a specified <code>optionDataSource</code>, and this property is\n not null, this will be passed to the datasource as ${isc.DocUtils.linkForRef('class:RPCRequest')} properties when\n performing the fetch operation on the dataSource to obtain a data-value to display-value\n mapping",
        name:"optionFilterContext",
        seeAlso:[],
        valueType:"RPCRequest Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:ListGrid.dragTrackerStyle":{
        ref:"attr:ListGrid.dragTrackerStyle",
        flags:"IRW",
        description:"CSS Style to apply to the drag tracker when dragging occurs on this component.",
        name:"dragTrackerStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"gridDragTracker\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ListGrid.trackerImage":{
        ref:"attr:ListGrid.trackerImage",
        flags:"IRWA",
        description:"Default image to use for the dragTracker when things are dragged within or out of this \n list. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.",
        name:"trackerImage",
        seeAlso:[
            {ref:"attr:ListGrid.dragTrackerMode", description:null},
            {ref:"method:ListGrid.getDragTrackerIcon", description:null}
        ],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["dragTracker"]
    },
    "attr:Layout.minMemberSize":{
        ref:"attr:Layout.minMemberSize",
        flags:"IR",
        description:"Minimum size, in pixels, below which members should never be shrunk, even if this\n requires the Layout to overflow.",
        name:"minMemberSize",
        seeAlso:[],
        valueType:"int",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "method:GridRenderer.setColumnWidth":{
        ref:"method:GridRenderer.setColumnWidth",
        flags:"",
        description:" Sets the width of a single column.\n\n",
        name:"setColumnWidth",
        seeAlso:[],
        params:[
            {description:"the number of the column to resize", optional:false, name:"colNum",
             defaultValue:null, type:"number"},
            {description:"the new width", optional:false, name:"newWidth", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:Calendar.dayView":{
        ref:"attr:Calendar.dayView",
        flags:"R",
        description:"Listgrid used to display events that pertain to a given day.",
        name:"dayView",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:DynamicForm.setSaveOperationType":{
        ref:"method:DynamicForm.setSaveOperationType",
        flags:"",
        description:" Setter for the default ${isc.DocUtils.linkForRef('type:DSOperationType')} when ${isc.DocUtils.linkForRef('method:DynamicForm.saveData')} is called.\n Note that this property can also be set by calling ${isc.DocUtils.linkForRef('method:DynamicForm.editRecord')} or \n ${isc.DocUtils.linkForRef('method:DynamicForm.editNewRecord')}\n\n",
        name:"setSaveOperationType",
        seeAlso:[],
        params:[
            {description:"Operation type to use as a default. Valid values are  <code>\"add\"</code> or <code>\"update\"</code>.",
             optional:false, name:"operationType", defaultValue:null, type:"DSOperationType"}
        ],
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "classAttr:Operators.iregexpTitle":{
        ref:"classAttr:Operators.iregexpTitle",
        flags:"IR",
        description:"Title for the \"iregexp\" operator",
        name:"iregexpTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"matches expression\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:FormItem.titleAlign":{
        ref:"attr:FormItem.titleAlign",
        flags:"IRW",
        description:"Alignment of this item's title in its cell.\n <p>\n If null, dynamically set according to text direction.",
        name:"titleAlign",
        seeAlso:[],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["title"]
    },
    "method:Button.setActionType":{
        ref:"method:Button.setActionType",
        flags:"",
        description:" Update the 'actionType' for this canvas (radio / checkbox / button)\n If the canvas is currently selected, and the passed in actionType is 'button'\n this method will deselect the canvas.\n",
        name:"setActionType",
        seeAlso:[],
        type:"method",
        definingClass:"class:Button",
        groups:["state", "event handling"]
    },
    "attr:Facet.selectionBoundary":{
        ref:"attr:Facet.selectionBoundary",
        flags:"IR",
        description:"Selection boundary determining what facets / facetValues can be selected together by\n drag selection / shift+click selection",
        name:"selectionBoundary",
        seeAlso:[],
        valueType:"enum",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Facet"
    },
    "classMethod:isA.emptyString":{
        ref:"classMethod:isA.emptyString",
        flags:"",
        description:"\n	Is <code>object</code> the empty string?<br><br>\n	\n	NOTE: if you prefer, you can call this as <code>isAn.emptyString()</code>\n\n",
        name:"emptyString",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a null string", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "type:Visibility":{
        ref:"type:Visibility",
        values:[
            {description:"The widget visibility will match that of its parent (usually visible).",
             value:"Canvas.INHERIT"},
            {description:"The widget will always be visible whether its parent is or not.",
             value:"Canvas.VISIBLE"},
            {description:"The widget will always be hidden even when its parent is visible.",
             value:"Canvas.HIDDEN"}
        ],
        description:null,
        name:"Visibility",
        seeAlso:[],
        type:"type",
        groups:["visibility"]
    },
    "attr:Canvas.redrawOnResize":{
        ref:"attr:Canvas.redrawOnResize",
        flags:"IRWA",
        description:"Should this element be redrawn in response to a resize?\n <P>\n Should be set to true for components whose ${isc.DocUtils.linkForRef('method:Canvas.getInnerHTML','inner HTML')} will not\n automatically reflow to fit the component's new size.",
        name:"redrawOnResize",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["drawing"]
    },
    "method:TabSet.updateTab":{
        ref:"method:TabSet.updateTab",
        flags:"",
        description:" Set the pane for a tab.\n <P>\n Pass in the index of a tab (or a tab object), and a new pane.\n <P>\n NOTE: the old pane for the tab is not destroy()d\n \n",
        name:"updateTab",
        seeAlso:[],
        params:[
            {description:"tab to update", optional:false, name:"tab", defaultValue:null,
             type:"number | ID | Tab"},
            {description:"new pane for the tab", optional:false, name:"pane", defaultValue:null,
             type:"Canvas | ID"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:Calendar.endDateField":{
        ref:"attr:Calendar.endDateField",
        flags:"IR",
        description:"The name of the end date field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"endDateField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"endDate\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "classAttr:Time.displayFormat":{
        ref:"classAttr:Time.displayFormat",
        flags:"RWA",
        description:"Standard formatter to be used when converting a date to a time-string via ${isc.DocUtils.linkForRef('classMethod:Time.toTime')}",
        name:"displayFormat",
        setter:["Time.setNormalDisplayFormat"],
        seeAlso:[],
        valueType:"timeFormatter | function",
        defaultValue:"\"toTime\"",
        type:"classAttr",
        definingClass:"class:Time"
    },
    "group:formLayout":{
        ref:"group:formLayout",
        title:"Form Layout",
        treeLocation:["Client Reference/Forms"],
        refs:["attr:DynamicForm.itemLayout", "attr:FormItem.width", "attr:FormItem.height", 
         "attr:FormItem.colSpan","attr:FormItem.rowSpan", "attr:FormItem.startRow", "attr:FormItem.endRow", 
         "attr:ButtonItem.startRow","attr:ButtonItem.endRow"],
        description:"<b>FormItem Placement in Columns and Rows</b>\n <P>\n With the default tabular layout mechanism, items are laid out in rows from left to\n right until the number of columns, specified by ${isc.DocUtils.linkForRef('attr:DynamicForm.numCols','form.numCols')},\n is filled, then a new row is begun.  Flags on FormItems, including\n ${isc.DocUtils.linkForRef('attr:FormItem.startRow','startRow')}, ${isc.DocUtils.linkForRef('attr:FormItem.endRow','endRow')},\n ${isc.DocUtils.linkForRef('attr:FormItem.colSpan','colSpan')} and ${isc.DocUtils.linkForRef('attr:FormItem.rowSpan','rowSpan')}, control row and\n column placement and spanning.  \n <P>\n Note that the most common form items (TextItem, SelectItem, etc) take up <b>two</b>\n columns by default: one for the form control itself, and one for it's title.  The\n default setting of ${isc.DocUtils.linkForRef('attr:DynamicForm.numCols','form.numCols:2')} will result in one TextItem\n or SelectItem per row.\n <P>\n Note also that ButtonItems have both startRow:true and endRow:true by default.  You must\n set startRow and/or endRow to <code>false</code> on a ButtonItem in order to place a\n button in the same row as any other item.\n <P>\n The log category \"tablePlacement\" can be enabled from the Developer Console to watch\n items being placed.  You can also set ${isc.DocUtils.linkForRef('attr:DynamicForm.cellBorder','form.cellBorder:1')} to\n reveal the table structure for layout troubleshooting purposes.\n <P>\n <b>Row and Column Sizing</b>\n <P>\n ${isc.DocUtils.linkForRef('attr:DynamicForm.colWidths')} controls the widths of form columns.  FormItems that have\n \"*\" for ${isc.DocUtils.linkForRef('attr:FormItem.width')} will fill the column.  FormItems with a numeric width will\n have that width in pixels regardless of the column's specified width, which may cause the\n column to overflow as described under ${isc.DocUtils.linkForRef('attr:DynamicForm.fixedColWidths')}.\n <P>\n For row heights, the largest pixel height specified on any item in the row is taken as a\n minimum size for the row.  Then, any rows that have \"*\" or \"%\" height items will share\n any height not taken up by fixed-sized items.\n <P>\n <b>Managing Overflow</b>\n <P>\n Forms often contain labels, data values, or instructional text which can vary in\n size based on the skin, data values, or internationalization settings.  There are a few\n ways to deal with a form potentially varying in size:\n <ol>\n <li> Allow scrolling when necessary, using ${isc.DocUtils.linkForRef('attr:Canvas.overflow','overflow:auto')}, either\n on the immediate form, or on some parent.\n <li> Place the form in a Layout along with a component that can render any specified\n size, such as a ${isc.DocUtils.linkForRef('class:ListGrid')}.  In this case, the Layout will automatically shrink the\n grid in order to accomodate the form.\n <li> Ensure that the form can always render at a designed minimum size by reducing\n the number of cases of variable-sized text, and testing remaining cases across all\n supported skins.  For example, move help text into hovers on help icons, or clip \n long text values at a maximum length and provide a hover to see the rest.\n </ol>\n\n Several examples of Form Layout are available ${isc.DocUtils.linkForExampleId('formsLayout', 'here')}.",
        name:"formLayout",
        seeAlso:[],
        type:"group"
    },
    "method:Canvas.shouldDragScroll":{
        ref:"method:Canvas.shouldDragScroll",
        flags:"A",
        description:" If this widget is showing scrollbars, and a user drags close to the edge of the viewport,\n should we scroll the viewport in the appropriate direction?\n Returns this.canDragScroll by default.\n",
        name:"shouldDragScroll",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["events", "dragging"]
    },
    "method:DataBoundComponent.transferRecords":{
        ref:"method:DataBoundComponent.transferRecords",
        flags:"",
        description:"\n Transfer a list of ${isc.DocUtils.linkForRef('object:Record')}s from another component (does not have to be a databound\n component) into this component.  This method is only applicable to list-type components,\n such as ${isc.DocUtils.linkForRef('class:ListGrid','listGrid')}, ${isc.DocUtils.linkForRef('class:TreeGrid','treeGrid')} or ${isc.DocUtils.linkForRef('class:TileGrid','tileGrid')}\n <P>\n This method implements the automatic drag-copy and drag-move behaviors of components like\n ${isc.DocUtils.linkForRef('class:ListGrid')}, and calling it is equivalent to completing a drag and drop of the\n <code>dropRecords</code>.\n <P>\n Note that this method is asynchronous - it may need to perform server turnarounds to prevent\n duplicates in the target component's data.  If you wish to be notified when the transfer \n process has completed, you can either pass the optional callback to this method or implement\n the ${isc.DocUtils.linkForRef('method:DataBoundComponent.dropComplete')} method on this component.\n <P>\n See also ${isc.DocUtils.linkForRef('method:DataBoundComponent.transferSelectedData')}.\n\n",
        name:"transferRecords",
        seeAlso:[],
        params:[
            {description:"Records to transfer to this component", optional:false, name:"dropRecords",
             defaultValue:null, type:"Array of Record"},
            {description:"The target record (eg, of a drop interaction), for context", optional:false,
             name:"targetRecord", defaultValue:null, type:"Record"},
            {description:"Insert point in this component's data for the transferred records",
             optional:false, name:"index", defaultValue:null, type:"integer"},
            {description:"The databound or non-databound component from which the records                            are to be transferred.",
             optional:false, name:"sourceWidget", defaultValue:null, type:"Canvas"},
            {description:"optional callback to be fired when the transfer process has completed",
             optional:true, name:"callback", defaultValue:null, type:"Callback"}
        ],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["dragdrop"]
    },
    "classMethod:isA.emptyObject":{
        ref:"classMethod:isA.emptyObject",
        flags:"",
        description:"\n Is <code>object</code> an object with no properties (i.e.: <code>{}</code>)?\n <P>\n Note that an object that has properties with null values is considered non-empty, eg \n <code>{ propName:null }</code> is non-empty.\n <P>\n NOTE: if you prefer, you can call this as <code>isAn.emptyObject()</code>\n\n",
        name:"emptyObject",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is the empty object", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "classMethod:Time.parseInput":{
        ref:"classMethod:Time.parseInput",
        flags:"",
        description:" Converts a time-string such as <code>1:00pm</code> to a date object with the appropriate\n time set. Accepts most formats of time string.\n <P>\n Input time is expected to be in the local timezone specified by\n ${isc.DocUtils.linkForRef('classMethod:Time.setDefaultDisplayTimezone')} by default unless the UTCTime parameter is\n passed in. An explicit timezone offset from UTC\n may also be specified directly in the timestring passed in - for example\n <code>\"00:00:00+02:00\"</code>.\n",
        name:"parseInput",
        seeAlso:[],
        params:[
            {description:"time string to convert to a date", optional:false, name:"timeString",
             defaultValue:null, type:"string"},
            {description:"If this method is passed a timeString in an unrecognized format,  return null rather than a date object with time set to 00:00:00",
             optional:false, name:"validTime", defaultValue:null, type:"boolean"},
            {description:"if passed (and the time string passed in has no explicit timezone  offset specified), assume the time passed in is in UTC time rather than\n  the local display timezone.",
             optional:false, name:"UTCTime", defaultValue:null, type:"boolean"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "attr:Canvas.snapToGrid":{
        ref:"attr:Canvas.snapToGrid",
        flags:"IRW",
        description:"Causes this canvas to snap to its parent's grid when dragging.",
        name:"snapToGrid",
        seeAlso:[
            {ref:"attr:Canvas.childrenSnapToGrid", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["dragdrop"]
    },
    "attr:MultiSortDialog.maxLevels":{
        ref:"attr:MultiSortDialog.maxLevels",
        flags:"IR",
        description:"The maximum number of levels of sorting that can be applied.  Since each sort-property or\n field-name can be used only once in a given multi-sort operation, if no maxLevels value\n or a value larger than the total number of available properties is specified, it will \n default to the total number of available properties.",
        name:"maxLevels",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "attr:FormItem.valueIcons":{
        ref:"attr:FormItem.valueIcons",
        flags:"IRW",
        description:"A mapping of logical form item values to URLs.\n If specified, when the form item is set to the value in question, an icon will be \n displayed with the appropriate source URL.",
        name:"valueIcons",
        setter:["setValueIcons"],
        seeAlso:[
            {ref:"method:FormItem.getValueIcon", description:null}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "method:Canvas.setEdgeOpacity":{
        ref:"method:Canvas.setEdgeOpacity",
        flags:"",
        description:" Set the ${isc.DocUtils.linkForRef('attr:Canvas.edgeOpacity')} and mark the canvas for redraw\n",
        name:"setEdgeOpacity",
        seeAlso:[],
        params:[
            {description:"new edge-opacity level", optional:false, name:"newOpacity", defaultValue:null,
             type:"number"}
        ],
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        definingClass:"class:Canvas"
    },
    "attr:ListGrid.autoSaveEdits":{
        ref:"attr:ListGrid.autoSaveEdits",
        flags:"IRWA",
        description:"If this ListGrid is editable, should edits be saved out when the user finishes editing\n a row (or a cell if ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell')} is true).\n <P>\n The default of <code>true</code> indicates that edits will be\n ${isc.DocUtils.linkForRef('attr:ListGrid.saveByCell','automatically saved')} as the\n user navigates through the grid and/or ${isc.DocUtils.linkForRef('type:EnterKeyEditAction','hits 'Enter'')} to end\n editing.  See the ${isc.DocUtils.linkForRef('group:editing','Grid Editing')} overview for details. \n <P>\n Setting <code>autoSaveEdits</code> false creates a \"mass update\" / \"mass delete\"\n interaction where edits will be retained for all edited cells (across rows if\n appropriate) until ${isc.DocUtils.linkForRef('method:ListGrid.saveEdits')} is called to save a particular row, or\n ${isc.DocUtils.linkForRef('method:ListGrid.saveAllEdits')} is called to save all changes in a batch.",
        name:"autoSaveEdits",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:Canvas.getInnerContentHeight":{
        ref:"method:Canvas.getInnerContentHeight",
        flags:"A",
        description:"  Returns the amount of space available for interior content (or relatively positioned child\n  widget(s)) without introducing clipping, scrolling or overflow.<br>\n  This is the space within the viewport of the widget (not including padding, and excluding \n  margins, borders or scrollbars) rendered at its specified size.\n\n",
        name:"getInnerContentHeight",
        seeAlso:[
            {ref:"method:Canvas.getInnerContentWidth", description:null},
            {ref:"method:Canvas.getInnerHeight", description:null},
            {ref:"method:Canvas.getInnerWidth", description:null}
        ],
        returns:{description:"inner height of the widget in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "group:loadDSTag":{
        ref:"group:loadDSTag",
        title:"&lt;isomorphic:loadDS&gt;",
        treeLocation:["Java Server Reference/SmartClient JSP Tags"],
        refs:[],
        description:"<i>produces:</i> JavaScript\n <p>\n This tag converts a SmartClient DataSource defined in XML to JavaScript for use in\n databinding on the client (browser).\n <p>\n Note that this JSP tag must be surrounded by &lt;SCRIPT&gt; tags in the JSP because it\n generates JavaScript code.  Like other tags that generate JavaScript code, this tag can be\n used in a JSP that is included from your main page in order to create separate\n cacheability.  For example:<pre>\n     &lt;SCRIPT SRC=\"myDataSources.jsp\"&gt;&lt;/SCRIPT&gt;\n </pre>\n <p>\n <b><u>Tag Attributes:</u></b>\n <p>\n <b>ID</b><br>\n <i>value format</i>: String - ID of datasource to load<br>\n <i>default value</i>: NONE\n <p>\n This attribute specifies the name of the dataSource that you wish to load.  DataSources are\n located in <code>[webroot]/shared/ds</code> by default.  This location is changeable in\n <code>[webroot]/WEB-INF/classes/server.properties</code> by setting the config parameter\n <code>project.datasources</code> to the directory where your dataSources are located.\n We recommend that for prototying, at least, you use the default directory.\n <p>\n For example:\n <pre>\n &lt;isomorphic:loadDS ID=\"supplyItem\"/&gt;\n </pre>\n Would load the <code>supplyItem</code> DataSource.\n <p>\n You can also load multiple dataSources in one go by specifying a comma-separated list of \n dataSource names as the ID.  For example:\n <pre>\n &lt;isomorphic:loadDS ID=\"supplyItem, employees, worldDS\"/&gt;\n </pre>    \n See ${isc.DocUtils.linkForRef('group:dataSourceDeclaration')} for more details on creating DataSources and an\n example.\n <p>\n <b>name</b><br>\n <i>value format</i>: String - ID of datasource to load<br>\n <i>default value</i>: NONE\n <p>\n This is a synonym for the <code>ID</code> attribute.",
        name:"loadDSTag",
        seeAlso:[
            {ref:"group:dataSourceDeclaration", description:null}
        ],
        type:"group",
        requiresModules:["SCServer"]
    },
    "classAttr:RPCResponse.STATUS_LOGIN_SUCCESS":{
        ref:"classAttr:RPCResponse.STATUS_LOGIN_SUCCESS",
        flags:"R",
        description:"Indicates that the login succeeded.",
        name:"STATUS_LOGIN_SUCCESS",
        seeAlso:[
            {ref:"class:RPCRequest", description:null}
        ],
        valueType:"integer",
        defaultValue:"-8",
        type:"classAttr",
        definingClass:"class:RPCResponse",
        groups:["statusCodes"]
    },
    "method:HTMLFlow.contentLoaded":{
        ref:"method:HTMLFlow.contentLoaded",
        flags:"",
        description:" StringMethod fired when content is completely loaded in this htmlFlow. Has no default \n implementation. May be observed or overridden as a notification type method to fire custom\n logic when loading completes.\n <P>\n Note: Does not apply to htmlFlows with ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType','contentsType')} set \n to <code>\"page\"</code>\n\n",
        name:"contentLoaded",
        seeAlso:[],
        type:"method",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:Canvas.animateTime":{
        ref:"attr:Canvas.animateTime",
        flags:"IRWA",
        description:"Default total duration of animations. Can be overridden by setting animation times for\n specific animations, or by passing a <code>duration</code> parameter into the appropriate\n animate...() method.",
        name:"animateTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"300",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('animateMove')}"],
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "classMethod:Class.toString":{
        ref:"classMethod:Class.toString",
        flags:"",
        description:"\n  The default toString() for a ClassObject reports that you have a ClassObject and what class\n  it is.\n",
        name:"toString",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "method:ResultSet.findNextIndex":{
        ref:"method:ResultSet.findNextIndex",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.findNextIndex')}.  Checks only loaded rows and will not trigger a fetch.\n",
        name:"findNextIndex",
        seeAlso:[],
        params:[
            {description:"first index to consider", optional:false, name:"startIndex",
             defaultValue:null, type:"int"},
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"endIndex", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access", "find"]
    },
    "method:Tree.remove":{
        ref:"method:Tree.remove",
        flags:"",
        description:"\n Removes a node, along with all its children.\n\n",
        name:"remove",
        seeAlso:[],
        params:[
            {description:"node to remove", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"true if the tree was changed as a result of this call", type:"boolean"},
        type:"method",
        definingClass:"class:Tree"
    },
    "attr:ListGridField.editorValueIconWidth":{
        ref:"attr:ListGridField.editorValueIconWidth",
        flags:"IRW",
        description:"When some cell in this field is being edited, setting this property will specify the\n width for value icons in the cell's editor. If unset, the editor's valueIcon width and\n height will be determined in the same way as it would be for a static cell.",
        name:"editorValueIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "method:Canvas.dragResizeMove":{
        ref:"method:Canvas.dragResizeMove",
        flags:"A",
        description:"\n Executed every time the mouse moves while drag-resizing. If this method\n does not return false, the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} (or outline if\n ${isc.DocUtils.linkForRef('attr:Canvas.dragAppearance')} is set to \"outline\" will automatically be moved as appropriate\n whenever the mouse moves.\n\n",
        name:"dragResizeMove",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to suppress auto-resize of the ${isc.DocUtils.linkForRef('attr:Canvas.dragTarget')} or outline.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:FilterBuilder.showSubClauseButton":{
        ref:"attr:FilterBuilder.showSubClauseButton",
        flags:"IR",
        description:"Whether to show a button that allows the user to add subclauses.  Defaults to false if \n the ${isc.DocUtils.linkForRef('type:TopOperatorAppearance')} is \"radio\", true in all other cases.",
        name:"showSubClauseButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"See Description",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:Canvas.left":{
        flags:"IRW",
        getter:["getLeft"],
        type:"attr",
        ref:"attr:Canvas.left",
        description:"Number of pixels the left side of the widget is offset to the right from its default\n drawing context (either its parent's topleft corner, or the document flow, depending on\n the value of the ${isc.DocUtils.linkForRef('attr:Canvas.position')} property).\n <P>\n Can also be set as a percentage, specified as a String ending in '%', eg, \"50%\".  In\n this case the top coordinate is considered as a percentage of the specified width of\n the ${isc.DocUtils.linkForRef('attr:Canvas.parentElement','parent')}.",
        name:"left",
        setter:["setLeft"],
        valueType:"Number or String",
        seeAlso:[],
        defaultValue:"0",
        groups:["positioning"],
        definingClass:"class:Canvas"
    },
    "classMethod:Class.setDefaultLogPriority":{
        ref:"classMethod:Class.setDefaultLogPriority",
        flags:"",
        description:" Set the default priority of logging for messages logged on this Class or Instance object.\n All categories for which there is no explicit, instance level logging priority set will\n log at this level on this object.<br>  \n To set the default visible log priority across the entire page, use\n <code>isc.Log.setDefaultPriority()</code> instead.\n",
        name:"setDefaultLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.setPriority", description:null}
        ],
        params:[
            {description:"Category for which the log priority will be updated. If not                          all logs on this canvas will be logged at the priority passed in.",
             optional:false, name:"category", defaultValue:null, type:"string"},
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:ResultTree.updateCacheFromRequest":{
        ref:"attr:ResultTree.updateCacheFromRequest",
        flags:"IRA",
        description:"When a successful Add, Update or Remove type operation fires on this ResultTree's \n dataSource, if ${isc.DocUtils.linkForRef('attr:DSResponse.data')} is unset, should we integrate the submitted\n data values (from the request) into our data-set?",
        name:"updateCacheFromRequest",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ResultTree",
        groups:["cacheSync"]
    },
    "method:MenuBar.setMenus":{
        ref:"method:MenuBar.setMenus",
        flags:"",
        description:" Dynamically reset the set of menus displayed by this menu bar. \n",
        name:"setMenus",
        seeAlso:[],
        params:[
            {description:"array of new menus for this menubar", optional:false, name:"menus",
             defaultValue:null, type:"array"}
        ],
        type:"method",
        definingClass:"class:MenuBar"
    },
    "attr:TreeGrid.animateFolderEffect":{
        ref:"attr:TreeGrid.animateFolderEffect",
        flags:"IRW",
        description:"When animating folder opening / closing, this property can be set to apply an\n animated acceleration effect.designates the speed of the\n animation in pixels shown (or hidden) per second.",
        name:"animateFolderEffect",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["animation"]
    },
    "attr:OperationBinding.wsOperation":{
        ref:"attr:OperationBinding.wsOperation",
        flags:"IR",
        description:"Name of the web service operation that will be invoked in order to execute this DataSource\n operation.\n <P>\n Valid only for a DataSource that has a WebService (${isc.DocUtils.linkForRef('attr:DataSource.serviceNamespace')} is\n set).  Otherwise, use ${isc.DocUtils.linkForRef('attr:OperationBinding.dataURL')}.\n <P>\n Setting <code>wsOperation</code> means that ${isc.DocUtils.linkForRef('attr:DSRequest.data')} will be serialized as the\n request message for the specified web service operation, with namespacing and soap encoding\n handled automatically.  See ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} for how to customize what\n data is sent to the server.",
        name:"wsOperation",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('wsdlBinding')}"],
        definingClass:"class:OperationBinding",
        groups:["clientDataIntegration"]
    },
    "attr:FormItem.visible":{
        ref:"attr:FormItem.visible",
        flags:"IRW",
        description:"Whether this item is currently visible.\n <P>\n <code>visible</code> can only be set on creation.  After creation, use\n ${isc.DocUtils.linkForRef('method:FormItem.show')} and ${isc.DocUtils.linkForRef('method:FormItem.hide')} to manipulate visibility.",
        name:"visible",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["appearance"]
    },
    "method:Window.minimize":{
        ref:"method:Window.minimize",
        flags:"",
        description:" Minimize the window. Fired when the user clicks the minimize button if \n ${isc.DocUtils.linkForRef('attr:Window.showMinimizeButton','this.showMinimizeButton')} is true.<br>\n Default implementation shrinks the window to just the height of the header bar, hiding the\n body. If ${isc.DocUtils.linkForRef('attr:Window.animateMinimize','animateMinimize')} is true, the resize will be animated.\n A restore button will be displayed in place of the minimize button when the window is \n minimized.\n",
        name:"minimize",
        seeAlso:[],
        type:"method",
        definingClass:"class:Window"
    },
    "class:FlowLayout":{
        methods:[],
        ref:"class:FlowLayout",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids"],
        attrs:[],
        description:"Similar to a ${isc.DocUtils.linkForRef('class:TileLayout')}, but by default lays out tiles in 'flow' mode instead of\n 'fit' mode.",
        name:"FlowLayout",
        seeAlso:[
            {ref:"attr:TileLayout.layoutPolicy", description:null}
        ],
        type:"class"
    },
    "attr:DatabaseBrowser.selectButton":{
        ref:"attr:DatabaseBrowser.selectButton",
        flags:"IR",
        description:"Instance of Button used to continue once a table has been selected.",
        name:"selectButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:Layout.orientation":{
        ref:"attr:Layout.orientation",
        deprecated:"in favor of ${isc.DocUtils.linkForRef('attr:Layout.vertical','this.vertical')}, which, if specified takes  precedence over this setting",
        flags:"AIRW",
        description:"Orientation of this layout.",
        name:"orientation",
        seeAlso:[],
        valueType:"Orientation",
        defaultValue:"\"horizontal\"",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:TreeGrid.dragDataAction":{
        ref:"attr:TreeGrid.dragDataAction",
        flags:"IRWA",
        description:"Specifies what to do with data dragged from this TreeGrid (into another component, or\n another node in this TreeGrid.  The default action is to move the data.  A setting of\n \"none\" is not recommended for trees because Trees maintain the node open state on the nodes\n themselves, and hence having multiple Tree objects share a reference to a node can have\n unintended consequences (such as opening a folder in one tree also triggering an open in\n another tree that shares the same node).\n <br><br>\n For DataBound trees (${isc.DocUtils.linkForRef('class:ResultTree')}), the expectation is that\n ${isc.DocUtils.linkForRef('method:TreeGrid.folderDrop')} will be overridden to perform whatever action took\n place as the result of the drag and drop interaction.",
        name:"dragDataAction",
        seeAlso:[
            {ref:"group:sharingNodes", description:null}
        ],
        valueType:"DragDataAction",
        defaultValue:"ListGrid.MOVE",
        type:"attr",
        definingClass:"class:TreeGrid"
    },
    "attr:Button.showSelectedIcon":{
        ref:"attr:Button.showSelectedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n becomes selected.",
        name:"showSelectedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:TreeNode.showOpenIcon":{
        ref:"attr:TreeNode.showOpenIcon",
        flags:"IRWA",
        description:"For folder nodes showing custom icons (set via ${isc.DocUtils.linkForRef('attr:TreeNode.icon')}),\n this property allows the developer to specify on a per-node basis whether an\n open state icon should be displayed when the folder is open.\n Set <code>node.showOpenIcon</code> to true to show the open state\n icons, or false to suppress this.<br>\n If not specified, this behavior is determined by ${isc.DocUtils.linkForRef('attr:TreeGrid.showCustomIconOpen')}\n for this node.\n <P>You can change the name of this property by setting \n ${isc.DocUtils.linkForRef('attr:TreeGrid.customIconOpenProperty')}.",
        name:"showOpenIcon",
        seeAlso:[
            {ref:"attr:TreeGrid.customIconProperty", description:null},
            {ref:"attr:TreeGrid.showCustomIconOpen", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"object:TreeNode",
        groups:["treeIcons"]
    },
    "method:Date.toUSShortDateTime":{
        ref:"method:Date.toUSShortDateTime",
        flags:"",
        description:"  Return this date in the format: <code>MM/DD/YYYY HH:MM</code>\n   \n",
        name:"toUSShortDateTime",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:Chart.valueProperty":{
        ref:"attr:Chart.valueProperty",
        flags:"IR",
        description:"Property in each record that holds a data value.\n <P>\n Not used if there is an inline facet, see ${isc.DocUtils.linkForRef('attr:Chart.data')}.",
        name:"valueProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"_value\"",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:ColorPicker.cancelButtonConstructor":{
        ref:"attr:ColorPicker.cancelButtonConstructor",
        flags:"IRWA",
        description:"The class of the Cancel button. It is intended that you use either IButton or \n      Button - other classes are unlikely to work correctly.",
        name:"cancelButtonConstructor",
        seeAlso:[],
        valueType:"Class",
        defaultValue:"IButton",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:ValuesManager.unknownErrorMessage":{
        ref:"attr:ValuesManager.unknownErrorMessage",
        flags:"IRW",
        description:"The error message for a failed validator that does not specify its own errorMessage.",
        name:"unknownErrorMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Invalid value\"",
        type:"attr",
        definingClass:"class:ValuesManager",
        groups:["validation", "i18nMessages"]
    },
    "group:imageColumns":{
        ref:"group:imageColumns",
        refs:["method:ListGrid.getValueIcon", "method:ListGrid.getEditorValueIcons", 
         "attr:DetailViewerField.imageSize","attr:DetailViewerField.imageWidth", "attr:DetailViewerField.imageHeight", 
         "attr:DetailViewerField.imageURLPrefix","attr:ListGridField.valueIcons", "attr:ListGrid.valueIconSize", 
         "attr:ListGrid.valueIconWidth","attr:ListGrid.valueIconHeight", "attr:ListGridField.valueIconSize", 
         "attr:ListGridField.valueIconWidth","attr:ListGridField.valueIconHeight", "attr:ListGridField.valueIconLeftPadding", 
         "attr:ListGridField.valueIconRightPadding","attr:ListGridField.editorValueIcons", "attr:ListGridField.editorValueIconWidth", 
         "attr:ListGridField.editorValueIconHeight","attr:ListGridField.showValueIconOnly", "attr:ListGridField.suppressValueIcon", 
         "attr:ListGridField.valueIconOrientation","attr:ListGrid.valueIconLeftPadding", "attr:ListGrid.valueIconRightPadding", 
         "attr:ListGridField.imageURLPrefix","attr:ListGridField.imageURLSuffix", "attr:ListGrid.imageSize", 
         "attr:ListGridField.imageSize","attr:ListGridField.imageWidth", "attr:ListGridField.imageHeight", 
         "attr:ListGrid.booleanTrueImage","attr:ListGrid.booleanFalseImage", "attr:ListGrid.booleanImageWidth", 
         "attr:ListGrid.booleanImageHeight"],
        description:"Columns that show images either as their only appearance or in addition to text.",
        name:"imageColumns",
        seeAlso:[],
        type:"group"
    },
    "method:TreeGrid.getViewState":{
        ref:"method:TreeGrid.getViewState",
        flags:"",
        description:" Overridden to return a ${isc.DocUtils.linkForRef('type:treeGridViewState')} object for the grid.\n",
        name:"getViewState",
        seeAlso:[
            {ref:"type:treeGridViewState", description:null},
            {ref:"method:TreeGrid.setViewState", description:null}
        ],
        returns:{description:"current view state for the grid.", type:"treeGridViewState"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "attr:Canvas.topElement":{
        ref:"attr:Canvas.topElement",
        flags:"RA",
        description:"The top-most Canvas (i.e., not a child of any other Canvas), if any, in this widget's\n containment hierarchy.",
        name:"topElement",
        seeAlso:[],
        valueType:"Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "attr:CubeGrid.metricFacetId":{
        ref:"attr:CubeGrid.metricFacetId",
        flags:"IR",
        description:"In a CubeGrid that displays values of different types (eg \"Revenue\" and \"Income\"), the\n different types of values on display are enumerated as the facet values of the \"metric\n facet\".  \n <P>\n The metric facet is treated identically to any other facet by the CubeGrid: it can be\n represented as row or column headers, can be innermost or have other facets under it,\n can be moved around, etc.  However when a metric facet is used, ${isc.DocUtils.linkForRef('attr:CubeGrid.metricFacetId')}\n must be set to allow the CubeGrid generate meaningful descriptions of values show in\n cells for use in hovers and other situations; see ${isc.DocUtils.linkForRef('attr:CubeGrid.valueTitle')} for a full\n explanation.",
        name:"metricFacetId",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"metric\"",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:DateChooser.todayClick":{
        ref:"method:DateChooser.todayClick",
        flags:"",
        description:" Fired when the user clicks the Today button. Default implementation will select the current\n date in the date chooser.\n",
        name:"todayClick",
        seeAlso:[],
        type:"method",
        definingClass:"class:DateChooser"
    },
    "method:Class.logFatal":{
        ref:"method:Class.logFatal",
        flags:"",
        description:" Log a message at \"fatal\" priority\n \n",
        name:"logFatal",
        seeAlso:[
            {ref:"classMethod:Log.logDebug", description:"for usage info"}
        ],
        params:[
            {description:"message to log", optional:false, name:"message", defaultValue:null,
             type:"String"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "method:Class.fireCallback":{
        ref:"method:Class.fireCallback",
        flags:"",
        description:"\n	Method to fire a callback. Callback will be fired in the scope of the object on \n  which this method is called.<br>\n  Falls through to ${isc.DocUtils.linkForRef('classMethod:Class.fireCallback')}\n\n",
        name:"fireCallback",
        seeAlso:[],
        params:[
            {description:"Callback to fire", optional:false, name:"callback", defaultValue:null,
             type:"Callback"},
            {description:"comma separated string of variables", optional:true, name:"argNames",
             defaultValue:null, type:"string"},
            {description:"array of arguments to pass to the method", optional:true, name:"args",
             defaultValue:null, type:"array"}
        ],
        returns:{description:"returns the value returned by the callback method passed in.", type:"any"},
        type:"method",
        definingClass:"class:Class"
    },
    "interface:Chart":{
        methods:["method:Chart.setupChart", "method:Chart.getFacet", "method:Chart.getFacetValue", 
         "method:Chart.getValue","method:Chart.setData"],
        ref:"interface:Chart",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Optional Modules/Analytics"],
        attrs:["attr:Chart.valueProperty", "attr:Chart.data", "attr:Chart.facets", "attr:Chart.stacked", 
         "attr:Chart.chartType","attr:Chart.valueTitle", "attr:Chart.title", "attr:Chart.subTitle", "attr:Chart.threeD", 
         "attr:Chart.shouldAnimateShow","attr:Chart.labelValues"],
        description:"Generic Chart properties and interfaces to be mixed into concrete charting implementations.\n <P>\n Components such as the ${isc.DocUtils.linkForRef('method:ListGrid.chartData','ListGrid')} and\n ${isc.DocUtils.linkForRef('method:CubeGrid.makeChart','CubeGrid')} expect this interface and can drive charting engines\n that support it.\n <P>\n Concrete Chart implementations may expose whatever properties they want for configuration,\n however, to enable easy switching of charting engines (different engines may be used for\n different end users based on that user's installed plugins), they should support\n the properties of this interface to the maximum extent possible.\n <P>\n This interface also provides core data model management (see\n ${isc.DocUtils.linkForRef('method:Chart.getValue','getValue()')}) for charting engines.",
        name:"Chart",
        seeAlso:[],
        type:"interface",
        requiresModules:["Analytics"]
    },
    "method:TreeGrid.getOpenState":{
        ref:"method:TreeGrid.getOpenState",
        flags:"",
        description:" Returns a snapshot of the current open state of this grid's data as\n a ${isc.DocUtils.linkForRef('type:treeGridOpenState')} object.<br>\n This object can be passed to ${isc.DocUtils.linkForRef('method:TreeGrid.setOpenState')} to open the same set of folders\n within the treeGrid's data (assuming the nodes are still present in the data).\n",
        name:"getOpenState",
        seeAlso:[
            {ref:"method:TreeGrid.setOpenState", description:null}
        ],
        returns:{description:"current sort state for the grid.", type:"treeGridOpenState"},
        type:"method",
        definingClass:"class:TreeGrid",
        groups:["viewState"]
    },
    "attr:EdgedCanvas.edgeSize":{
        ref:"attr:EdgedCanvas.edgeSize",
        flags:"IR",
        description:"Size in pixels for corners and edges",
        name:"edgeSize",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "attr:Canvas.backgroundRepeat":{
        ref:"attr:Canvas.backgroundRepeat",
        flags:"IRW",
        description:"Specifies how the background image should be tiled if this widget\n      is larger than the image. It corresponds to the CSS background-repeat attribute.\n      See BkgndRepeat type for details.",
        name:"backgroundRepeat",
        seeAlso:[],
        valueType:"BkgndRepeat",
        defaultValue:"Canvas.REPEAT",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "method:Canvas.rightMouseDown":{
        ref:"method:Canvas.rightMouseDown",
        flags:"A",
        description:"\n Executed when the right mouse down is pressed on this widget.  No default implementation.\n\n",
        name:"rightMouseDown",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        platformNotes:"Some end user systems only have one mouse button.",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "group:facetLayout":{
        ref:"group:facetLayout",
        refs:["method:CubeGrid.getFacetValueLayout", "method:CubeGrid.getRowFacetLayout", 
         "method:CubeGrid.getColumnFacetLayout","method:CubeGrid.addRowFacet", "method:CubeGrid.addColumnFacet", 
         "method:CubeGrid.removeFacet","method:CubeGrid.setFixedFacetValue", "method:CubeGrid.facetValueReordered", 
         "method:CubeGrid.facetAdded","method:CubeGrid.facetRemoved", "method:CubeGrid.facetMoved", "attr:CubeGrid.columnFacets", 
         "attr:CubeGrid.rowFacets","attr:CubeGrid.hideEmptyFacetValues", "attr:CubeGrid.canReorderColumns", 
         "attr:CubeGrid.canMoveFacets"],
        description:null,
        name:"facetLayout",
        seeAlso:[],
        type:"group"
    },
    "attr:Button.selected":{
        ref:"attr:Button.selected",
        flags:"IRW",
        description:"Whether this component is selected.  For some components, selection affects appearance.",
        name:"selected",
        setter:["setSelected"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["state"]
    },
    "attr:TextAreaItem.showHintInField":{
        ref:"attr:TextAreaItem.showHintInField",
        flags:"IRWA",
        description:"If showing hint for this form item, should it be shown within the field?\n <P>CSS style for the hint is ${isc.DocUtils.linkForRef('attr:TextAreaItem.textBoxStyle')} with the suffix\n \"Hint\" appended to it.",
        name:"showHintInField",
        seeAlso:[
            {ref:"attr:FormItem.hint", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextAreaItem",
        groups:["appearance"]
    },
    "attr:Canvas.doubleClickDelay":{
        ref:"attr:Canvas.doubleClickDelay",
        flags:"IRWA",
        description:"Amount of time (in msec) between which two clicks are considered a single click",
        name:"doubleClickDelay",
        seeAlso:[],
        valueType:"number",
        defaultValue:"250",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["events"]
    },
    "group:loadingDataMessage":{
        ref:"group:loadingDataMessage",
        refs:["attr:ListGrid.loadingDataMessage", "attr:ListGrid.loadingDataMessageStyle"],
        name:"loadingDataMessage",
        seeAlso:[],
        type:"group"
    },
    "method:GridRenderer.rowOut":{
        ref:"method:GridRenderer.rowOut",
        flags:"A",
        description:" Called when the mouse pointer leaves a row\n\n",
        name:"rowOut",
        seeAlso:[],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"whether to cancel the event", type:"boolean"},
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:Window.modalMaskOpacity":{
        ref:"attr:Window.modalMaskOpacity",
        flags:"IR",
        description:"Controls the opacity of the modal mask displayed behind modal windows.",
        name:"modalMaskOpacity",
        seeAlso:[
            {ref:"attr:Window.modalMask", description:null}
        ],
        valueType:"number",
        defaultValue:"50",
        type:"attr",
        definingClass:"class:Window",
        groups:["modal", "appearance"]
    },
    "type:RowEndEditAction":{
        ref:"type:RowEndEditAction",
        values:[
            {description:"navigate to the first editable cell in the same record", value:"\"same\""},
            {description:"navigate to the first editable cell in the next record", value:"\"next\""},
            {description:"complete the edit.", value:"\"done\""},
            {description:"Leave focus in the cell being edited (take no action)", value:"\"stop\""},
            {description:"take no action", value:"\"none\""}
        ],
        description:"While editing a ListGrid, what cell should we edit when the user attempts to navigate \n  into a cell past the end of an editable row, via a Tab keypress, or a programmatic\n  saveAndEditNextCell() call?",
        name:"RowEndEditAction",
        seeAlso:[
            {ref:"attr:ListGrid.rowEndEditAction", description:null}
        ],
        type:"type",
        groups:["editing"]
    },
    "method:SectionStack.setSectionProperties":{
        ref:"method:SectionStack.setSectionProperties",
        flags:"",
        description:" Set arbitrary properties for a particular section in this SectionStack. Properties will\n be applied to the sectionHeader for the section.\n <P>\n Note that where APIs exist to explicitly manipulate section properties these should be\n used in preference to this method. For example, to add or remove items in a section use\n ${isc.DocUtils.linkForRef('method:SectionStack.addItem')} or ${isc.DocUtils.linkForRef('method:SectionStack.removeItem')}. To change the title of\n a section, use ${isc.DocUtils.linkForRef('method:SectionStack.setSectionTitle')}.\n <P>\n Also note that to modify properties of items within a section, call\n the appropriate setter methods directly on the item you want to modify.\n\n",
        name:"setSectionProperties",
        seeAlso:[],
        params:[
            {description:"ID or index of the section to modify", optional:false, name:"section",
             defaultValue:null, type:"String or Number"},
            {description:"properties to apply to the section.", optional:false, name:"properties",
             defaultValue:null, type:"section properties"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "method:DetailViewer.getRecordIndex":{
        ref:"method:DetailViewer.getRecordIndex",
        flags:"",
        description:" Get the index of the provided record.\n <P>\n Override in subclasses to provide more specific behaviour, for instance, when data holds a\n large number of records\n\n",
        name:"getRecordIndex",
        seeAlso:[],
        params:[
            {description:"the record whose index is to be retrieved", optional:false, name:"record",
             defaultValue:null, type:"Record"}
        ],
        returns:{description:"indexindex of the record, or -1 if not found", type:"Number"},
        type:"method",
        definingClass:"class:DetailViewer"
    },
    "attr:MultiSortDialog.fields":{
        ref:"attr:MultiSortDialog.fields",
        flags:"IR",
        description:"The list of fields which the user can choose to sort by.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of Field",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortDialog"
    },
    "method:FilterBuilder.setTopOperator":{
        ref:"method:FilterBuilder.setTopOperator",
        flags:"",
        description:" Programmatically change the ${isc.DocUtils.linkForRef('attr:FilterBuilder.topOperator')} for this FilterBuilder.\n",
        name:"setTopOperator",
        seeAlso:[],
        params:[
            {description:"new top-level operator", optional:false, name:"operator", defaultValue:null,
             type:"OperatiorId"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "method:DetailViewer.setData":{
        ref:"method:DetailViewer.setData",
        flags:"",
        description:" Sets the data displayed by this detail viewer.\n\n",
        name:"setData",
        seeAlso:[],
        params:[
            {description:"new data to be displayed", optional:false, name:"newData", defaultValue:null,
             type:"object or array"}
        ],
        type:"method",
        definingClass:"class:DetailViewer"
    },
    "method:ValuesManager.showErrors":{
        ref:"method:ValuesManager.showErrors",
        flags:"",
        description:" Method to explicitly show the latest set of validation errors present on this \n ValuesManager.<br>\n Will redraw all member forms to display (or clear) currently visible errors, and\n fire ${isc.DocUtils.linkForRef('method:ValuesManager.handleHiddenValidationErrors')} to allow custom handling of\n hidden errors.\n",
        name:"showErrors",
        seeAlso:[],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["errors"]
    },
    "group:rowEffects":{
        ref:"group:rowEffects",
        refs:[],
        name:"rowEffects",
        seeAlso:[],
        type:"group"
    },
    "classMethod:isA.Array":{
        ref:"classMethod:isA.Array",
        flags:"",
        description:"\n	Is <code>object</code> an Array object?<br><br>\n\n	NOTE: if you prefer, you can call this as <code>isAn.Array()</code>\n\n",
        name:"Array",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is an Array", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "attr:Facet.canMinimize":{
        ref:"attr:Facet.canMinimize",
        flags:"IR",
        description:"If facet minimizing is enabled, whether this facet should show controls to minimize the\n next facet.  Generally a tree facet should not also allow minimizing the next facet -\n the interaction of the two types of collapsing can be confusing.\n <P>\n See ${isc.DocUtils.linkForRef('attr:CubeGrid.canMinimizeFacets')}.",
        name:"canMinimize",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"cubeGrid.canMinimizeFacets",
        type:"attr",
        definingClass:"class:Facet"
    },
    "method:FormulaBuilder.getSamplePrompt":{
        ref:"method:FormulaBuilder.getSamplePrompt",
        flags:"",
        description:" Evaluates and returns the dynamic ${isc.DocUtils.linkForRef('attr:FormulaBuilder.samplePrompt')} string which is \n displayed beneath the formulaField and updated when typing pauses.\n \n",
        name:"getSamplePrompt",
        seeAlso:[],
        params:[
            {description:"The return value from a call to testFunction().", optional:false, name:"",
             defaultValue:null, type:"TestFunctionResult"}
        ],
        returns:{description:"Caption displaying dynamic row-title and the result of the formula",
         type:"string"},
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["i18nMessages"]
    },
    "method:ListGrid.cellHasErrors":{
        ref:"method:ListGrid.cellHasErrors",
        flags:"",
        description:" Given a rowNum and a colNum or fieldName, determine whether we currently have stored \n validation errors for the record/field in question.\n",
        name:"cellHasErrors",
        seeAlso:[
            {ref:"method:ListGrid.hasErrors", description:null},
            {ref:"method:ListGrid.rowHasErrors", description:null}
        ],
        params:[
            {description:"index of row to check for validation errors", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"name of field, or index of column to check for validation                                       errors",
             optional:false, name:"fieldID", defaultValue:null, type:"number|string"}
        ],
        returns:{description:"true if we have validation errors for the row/col in question", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "attr:StretchImgButton.baseStyle":{
        ref:"attr:StretchImgButton.baseStyle",
        flags:"IRW",
        description:"Base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n <P>\n When the component changes state (eg becomes disabled), a suffix will be appended to this\n style name, reflecting the following states: \"Over\", \"Down\", or \"Disabled\".\n <P>\n If the widget is selected, the suffixes will be \"Selected\", \"SelectedOver\", etc.\n <P>\n If the widget has focus and ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')} is true, and\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is false, the suffixes will be \"Focused\",\n \"FocusedOver\", etc, or if the widget is both selected and focused, \"SelectedFocused\",\n \"SelectedFocusedOver\", etc.\n <P>\n For example, if <code>baseStyle</code> is set to \"button\", this component is\n ${isc.DocUtils.linkForRef('method:StretchImgButton.isSelected','selected')} and the mouse cursor is over this component, the style\n \"buttonSelectedOver\" will be used.",
        name:"baseStyle",
        setter:["setBaseStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"stretchImgButton\"",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "attr:StatefulCanvas.baseStyle":{
        ref:"attr:StatefulCanvas.baseStyle",
        flags:"IRW",
        description:"Base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n <P>\n When the component changes state (eg becomes disabled), a suffix will be appended to this\n style name, reflecting the following states: \"Over\", \"Down\", or \"Disabled\".\n <P>\n If the widget is selected, the suffixes will be \"Selected\", \"SelectedOver\", etc.\n <P>\n If the widget has focus and ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')} is true, and\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is false, the suffixes will be \"Focused\",\n \"FocusedOver\", etc, or if the widget is both selected and focused, \"SelectedFocused\",\n \"SelectedFocusedOver\", etc.\n <P>\n For example, if <code>baseStyle</code> is set to \"button\", this component is\n ${isc.DocUtils.linkForRef('method:StatefulCanvas.isSelected','selected')} and the mouse cursor is over this component, the style\n \"buttonSelectedOver\" will be used.",
        name:"baseStyle",
        setter:["setBaseStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StatefulCanvas"
    },
    "method:ListGrid.getViewState":{
        ref:"method:ListGrid.getViewState",
        flags:"",
        description:" Returns a snapshot of the current view state of this ListGrid.<br>\n This includes the field state, sort state and selected state of the grid, returned as a\n ${isc.DocUtils.linkForRef('type:listGridViewState')} object.<br>\n This object can be passed to ${isc.DocUtils.linkForRef('method:ListGrid.setViewState')} to reset this grid's vew state\n to the current state (assuming the same data / fields are present in the grid).<br>\n",
        name:"getViewState",
        seeAlso:[
            {ref:"type:listGridViewState", description:null},
            {ref:"method:ListGrid.setViewState", description:null}
        ],
        returns:{description:"current view state for the grid.", type:"listGridViewState"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["viewState"]
    },
    "attr:DSResponse.data":{
        ref:"attr:DSResponse.data",
        flags:"R",
        description:"For DataSource operations, this is typically either an Array of Objects representing records\n (for \"fetch\" operations) or a single Object representing the updated record (for \"update\",\n \"add\" or \"remove\" operations).",
        name:"data",
        seeAlso:[
            {ref:"group:dataSourceOperations",
             description:"for standard return types for DataSource Operations"}
        ],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSResponse",
        groups:["dsResponse"]
    },
    "attr:Layout.vPolicy":{
        ref:"attr:Layout.vPolicy",
        flags:"IRWA",
        description:"Sizing policy applied to members on vertical axis",
        name:"vPolicy",
        seeAlso:[],
        valueType:"LayoutPolicy",
        defaultValue:"\"fill\"",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:TextItem.length":{
        ref:"attr:TextItem.length",
        flags:"IRW",
        description:"if set, maximum number of characters for this field",
        name:"length",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TextItem",
        groups:["validation"]
    },
    "attr:DateChooser.nextMonthIconWidth":{
        ref:"attr:DateChooser.nextMonthIconWidth",
        flags:"IRW",
        description:"Width of the icon for the next month button",
        name:"nextMonthIconWidth",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "group:zIndex":{
        ref:"group:zIndex",
        refs:["method:Canvas.bringToFront", "method:Canvas.sendToBack", "method:Canvas.moveAbove", 
         "method:Canvas.moveBelow"],
        description:"Object's \"stacking order\" above or below other objects",
        name:"zIndex",
        seeAlso:[],
        type:"group"
    },
    "method:ListGrid.editFailed":{
        ref:"method:ListGrid.editFailed",
        flags:"",
        description:" Called when an attempt to save inline edits fails, due to a validation error or other\n server error.\n <P>\n The default implementation of editFailed does nothing for normal validation errors,\n which are displayed before editFailed() is called.  For any other errors, the default\n implementation will call ${isc.DocUtils.linkForRef('classMethod:RPCManager.handleError')}, which by default\n will result in a warning dialog.\n\n",
        name:"editFailed",
        seeAlso:[],
        params:[
            {description:"current index of the row we attempted to save", optional:false,
             name:"rowNum", defaultValue:null, type:"number"},
            {description:"index of the column where the edit failed, if applicable", optional:false,
             name:"colNum", defaultValue:null, type:"number"},
            {description:"new values that we attempted to save", optional:false, name:"newValues",
             defaultValue:null, type:"Record"},
            {description:"the complete original values from before the save occurred", optional:false,
             name:"oldValues", defaultValue:null, type:"Record"},
            {description:"Edit completion event that led to the save                                                  attempt",
             optional:false, name:"editCompletionEvent", defaultValue:null, type:"editCompletionEvent"},
            {description:"DSResponse, for saves through a DataSource", optional:true,
             name:"dsResponse", defaultValue:null, type:"DSResponse"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "method:ValuesManager.clearValues":{
        ref:"method:ValuesManager.clearValues",
        flags:"",
        description:" Clear out all the values managed by this values manager.\n",
        name:"clearValues",
        seeAlso:[],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["formValues"]
    },
    "attr:DynamicForm.values":{
        flags:"IRW",
        getter:["getValues"],
        type:"attr",
        ref:"attr:DynamicForm.values",
        description:"An Object containing the initial values of the form as properties, where each\n propertyName is the name of a ${isc.DocUtils.linkForRef('group:Items','form item')} in the form, and each property\n value is the value held by that form item.\n <P>\n The form's values may contain values that are not managed by any FormItem, and these\n values will be preserved and available when the form values are subsequently retrieved\n via ${isc.DocUtils.linkForRef('method:DynamicForm.getValues')}.\n <P>\n Providing values on initialization is equivalent to calling ${isc.DocUtils.linkForRef('method:DynamicForm.setValues')}.\n <P>\n As the user manipulates form items to change values, change events fire\n ${isc.DocUtils.linkForRef('method:FormItem.change','on the items')} and \n ${isc.DocUtils.linkForRef('method:DynamicForm.itemChange','on the form as a whole')}.\n <P>\n Note that form values are logical values, for example, the value of a ${isc.DocUtils.linkForRef('class:DateItem')} is\n a JavaScript Date object, not a String, even if the user enters the date via a text\n input.  Likewise the value of a ${isc.DocUtils.linkForRef('class:TextItem')} or ${isc.DocUtils.linkForRef('class:CheckboxItem')} that started out\n null remains null until the user changes it; the value will not be automatically\n converted to the null string (\"\") or false respectively, as happens with native HTML\n elements.",
        name:"values",
        setter:["setValues"],
        valueType:"Object",
        seeAlso:[],
        defaultValue:"null",
        groups:["formValues"],
        definingClass:"class:DynamicForm"
    },
    "method:ListGrid.fieldStateChanged":{
        ref:"method:ListGrid.fieldStateChanged",
        flags:"",
        description:" Notification method executed when columns are resized or reordered, or fields are \n shown or hidden. Has no default implementation.\n\n",
        name:"fieldStateChanged",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["fieldState"]
    },
    "attr:ListGridField.defaultFilterValue":{
        ref:"attr:ListGridField.defaultFilterValue",
        flags:"IRWA",
        description:"If this ListGrid is showing a filter row, this property can be used to apply a default\n value to show in the filter editor for this field.",
        name:"defaultFilterValue",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["filterEditor"]
    },
    "attr:GridRenderer.canHover":{
        ref:"attr:GridRenderer.canHover",
        flags:"RW",
        description:"If true, cellHover and rowHover events will fire when the user leaves the mouse over a \n row / cell.",
        name:"canHover",
        seeAlso:[
            {ref:"method:GridRenderer.cellHover", description:null},
            {ref:"method:GridRenderer.rowHover", description:null},
            {ref:"attr:GridRenderer.showHover", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["events"]
    },
    "attr:HeaderItem.textBoxStyle":{
        ref:"attr:HeaderItem.textBoxStyle",
        flags:"IRW",
        description:"Base CSS class for this item",
        name:"textBoxStyle",
        seeAlso:[],
        valueType:"FormItemBaseStyle",
        defaultValue:"\"headerItem\"",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "attr:ActiveXControl.uuid":{
        ref:"attr:ActiveXControl.uuid",
        flags:"IR",
        description:"Set this to the uuid of your Active X control - ISC will then generate the appropriate\n  classID entry for you.",
        name:"uuid",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ActiveXControl"
    },
    "attr:StatefulCanvas.showRollOver":{
        ref:"attr:StatefulCanvas.showRollOver",
        flags:"IRW",
        description:"Should we visibly change state when the mouse goes over this object?",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "class:StretchImg":{
        methods:["method:StretchImg.setState", "method:StretchImg.setSrc"],
        ref:"class:StretchImg",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Foundation"],
        attrs:["attr:StretchImg.vertical", "attr:StretchImg.capSize", "attr:StretchImg.src", 
         "attr:StretchImg.hSrc","attr:StretchImg.vSrc", "attr:StretchImg.imageType", "attr:StretchImg.items", 
         "attr:StretchImg.showGrip","attr:StretchImg.showTitle", "attr:StretchImg.itemBaseStyle"],
        description:"The StretchImg widget class implements a widget type that displays a list of multiple images\n  that make up a single image.",
        name:"StretchImg",
        seeAlso:[],
        type:"class"
    },
    "attr:DetailViewer.wrapLabel":{
        ref:"attr:DetailViewer.wrapLabel",
        flags:"IRW",
        description:"Should the label be allowed to wrap, or be fixed to one line no matter how long",
        name:"wrapLabel",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["labels"]
    },
    "attr:HTMLFlow.loadingMessage":{
        ref:"attr:HTMLFlow.loadingMessage",
        flags:"IRW",
        description:"HTML to show while content is being fetched, active only if the <code>contentsURL</code>\n property has been set.\n <P>\n The loading message will show both during the initial load of content, and during reload if\n the contents are reloaded or the contentsURL changed.  For a first-time only loading\n message, initialize the <code>contents</code> property instead.<br>\n Note: the <code>loadingMessage</code> is never displayed when loading complete web pages \n rather than HTML fragments (see ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType')}).",
        name:"loadingMessage",
        seeAlso:[],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "method:Date.toJapanShortDate":{
        ref:"method:Date.toJapanShortDate",
        flags:"",
        description:" Return the date in this format: <code>YYYY/MM/DD</code>\n",
        name:"toJapanShortDate",
        seeAlso:[],
        returns:{description:"formatted date string", type:"string"},
        type:"method",
        definingClass:"class:Date",
        groups:["dateFormatting"]
    },
    "attr:Layout.layoutTopMargin":{
        ref:"attr:Layout.layoutTopMargin",
        flags:"IRW",
        description:"Space outside of all members, on the top side.  Defaults to ${isc.DocUtils.linkForRef('attr:Layout.layoutMargin')}.\n <P>\n Requires a manual call to <code>setLayoutMargin()</code> if changed on the fly.",
        name:"layoutTopMargin",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutMargin"]
    },
    "method:FilterBuilder.removeClause":{
        ref:"method:FilterBuilder.removeClause",
        flags:"",
        description:" Remove a clause this is currently showing.\n",
        name:"removeClause",
        seeAlso:[],
        params:[
            {description:"clause as retrieved from filterBuilder.clauses", optional:false,
             name:"clause", defaultValue:null, type:"Clause"}
        ],
        type:"method",
        definingClass:"class:FilterBuilder"
    },
    "attr:CubeGrid.data":{
        ref:"attr:CubeGrid.data",
        flags:"IRW",
        description:"An array of \"cellRecords\", each of which represents data for one cell of the body area.",
        name:"data",
        seeAlso:[
            {ref:"attr:CubeGrid.fixedFacetValues", description:null}
        ],
        valueType:"Array of CellRecord",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "method:DataBoundComponent.findByKey":{
        ref:"method:DataBoundComponent.findByKey",
        flags:"",
        description:" Attempt to find the record in the resultSet that has a primary key value that matches the \n passed in parameter value. Only the locally cached data will be searched. \n Checks only loaded rows and will not trigger a fetch. Returns null if there is no match, \n data is not loaded, or there is no ${isc.DocUtils.linkForRef('attr:ResultSet.dataSource','dataSource')}.\n\n",
        name:"findByKey",
        seeAlso:[],
        params:[
            {description:"primary key value to search for", optional:false, name:"keyValue",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the record with a matching primary key field, or null if not found",
         type:"Record"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:DataBoundComponent.showDetailFields":{
        ref:"attr:DataBoundComponent.showDetailFields",
        flags:"IRW",
        description:"Whether to show fields marked <code>detail:true</code> when a DataBoundComponent is \n given a DataSource but no <code>component.fields</code>.\n <p>\n The <code>detail</code> property is used on DataSource fields to mark fields that \n shouldn't appear by default in a view that tries to show many records in a small space.",
        name:"showDetailFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "attr:DataSourceField.childrenProperty":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSourceField.childrenProperty",
        description:"If true, this property indicates that this field will hold an explicit array of child nodes \n for the current node.\n This has the same effect as specifying ${isc.DocUtils.linkForRef('attr:DataSource.childrenField')} to this field's \n name.",
        name:"childrenProperty",
        valueType:"boolean",
        seeAlso:[
            {ref:"attr:DataSource.childrenField", description:null}
        ],
        examples:["${isc.DocUtils.linkForExampleId('treeLoadXML')}"],
        serverDS:"allowed",
        defaultValue:"false",
        groups:["dataSourceRelations"],
        definingClass:"class:DataSourceField"
    },
    "method:CellSelection.selectCellList":{
        ref:"method:CellSelection.selectCellList",
        flags:"",
        description:"			select an array of cells\n",
        name:"selectCellList",
        seeAlso:[],
        params:[
            {description:"listArray of cells to select. Each cell can be specified                                      as a 2 element array <code>[rowNum, colNum]</code>",
             optional:true, name:"", defaultValue:null, type:"array"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:CellSelection",
        groups:["selection"]
    },
    "attr:ListGrid.headerMenuButtonIconWidth":{
        ref:"attr:ListGrid.headerMenuButtonIconWidth",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderMenuButton')} is true, this property governs the width of the icon\n shown on the auto-generated <code>headerMenuButton</code>",
        name:"headerMenuButtonIconWidth",
        seeAlso:[],
        valueType:"number",
        defaultValue:"7",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["headerMenuButton"]
    },
    "attr:SelectItem.pickerIconProperties":{
        ref:"attr:SelectItem.pickerIconProperties",
        flags:"IRA",
        description:"If ${isc.DocUtils.linkForRef('attr:SelectItem.showPickerIcon')} is true for this item, this block of properties will\n be applied to the pickerIcon. Allows for advanced customization of this icon.",
        name:"pickerIconProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:SelectItem",
        groups:["pickerIcon"]
    },
    "classMethod:FileLoader.cacheFiles":{
        ref:"classMethod:FileLoader.cacheFiles",
        flags:"",
        description:"\n Caches the specified list of files.\n\n",
        name:"cacheFiles",
        seeAlso:[],
        params:[
            {description:"List of URLs to cache.  These may point to image, js, or css                               files.  This can either be a string with comma separated URLs or an\n                               Array of URLs.",
             optional:false, name:"URLs", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified                                      files has been cached.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"},
            {description:"Specifies the type of the files referenced by the URLs.  Valid values are:                        \"js\", \"css\", and \"image\".  If not specified, the type is auto-derived from\n                        the file extension.  If a type cannot be derived, the cache directive\n                        for that specific URL will be ignored and an error will be logged to\n                        the Developer Console. You must specify a type if you use a\n                        non-standard extension for your file - for example if you use a JSP\n                        or a servlet with no extension to generate images.",
             optional:true, name:"type", defaultValue:null, type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "attr:ImgButton.hiliteAccessKey":{
        ref:"attr:ImgButton.hiliteAccessKey",
        flags:"IRW",
        description:"If set to true, if the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.title','title')} of this button contains the\n specified ${isc.DocUtils.linkForRef('attr:Canvas.accessKey','accessKey')}, when the title is displayed to the user\n it will be modified to include HTML to underline the accessKey.<br>\n Note that this property may cause titles that include HTML (rather than simple strings)\n to be inappropriately modified, so should be disabled if your title string includes\n HTML characters.",
        name:"hiliteAccessKey",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ImgButton"
    },
    "method:WebService.getInputHeaderSchema":{
        ref:"method:WebService.getInputHeaderSchema",
        flags:"",
        description:" Get the schema for each part of the SOAP header for the input message of a given\n operation, as a mapping from part name to schema.  For example, given WSDL like:\n <pre>\n     &lt;soap:header part=\"SessionHeader\" message=\"tns:HeaderMessage\"/&gt;\n     &lt;soap:header part=\"CallOptions\" message=\"tns:HeaderMessage/&gt;\n </pre>\n The following schema would be returned:\n <pre>\n     { SessionHeader : <i>sessionHeaderPartSchema</i>,\n       CallOptions : <i>callOptionsPartSchema</i> }\n </pre>\n The schema are instances of ${isc.DocUtils.linkForRef('class:DataSource')} that can be inspected to discover the\n elements and types that are legal in that header part, and can construct a valid SOAP\n header part if ${isc.DocUtils.linkForRef('method:DataSource.xmlSerialize')} is invoked.\n \n",
        name:"getInputHeaderSchema",
        seeAlso:[],
        params:[
            {description:"name of an operation from this web service", optional:false,
             name:"operationName", defaultValue:null, type:"String"}
        ],
        returns:{description:"mapping from partName to schema", type:"Object"},
        type:"method",
        definingClass:"class:WebService"
    },
    "attr:Slider.canFocus":{
        ref:"attr:Slider.canFocus",
        flags:"IRW",
        description:"Indicates whether keyboard manipulation of the slider is allowed.",
        name:"canFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Slider"
    },
    "attr:FormItem.valueIconHeight":{
        ref:"attr:FormItem.valueIconHeight",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.valueIcons')} is specified, use this property to specify a height for the\n value icon written out.",
        name:"valueIconHeight",
        seeAlso:[
            {ref:"attr:FormItem.valueIconWidth", description:null},
            {ref:"attr:FormItem.valueIconSize", description:null}
        ],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["valueIcons"]
    },
    "attr:CheckboxItem.labelAsTitle":{
        ref:"attr:CheckboxItem.labelAsTitle",
        flags:"IRW",
        description:"By default a checkboxItem sets ${isc.DocUtils.linkForRef('attr:CheckboxItem.showTitle')}:true, and so takes up two cells with the\n default ${isc.DocUtils.linkForRef('type:TitleOrientation')} of \"left\" (see ${isc.DocUtils.linkForRef('group:formLayout','form layout\n overview')}).  However, the title cell is left blank by default, and the title specified\n by ${isc.DocUtils.linkForRef('attr:FormItem.title')} is shown inside the formItem's cell instead, in an element\n called the \"label\".\n <P>\n To instead show the title in it's original location, set <code>labelAsTitle:true</code>.\n You can also set ${isc.DocUtils.linkForRef('attr:CheckboxItem.showLabel')}:false to suppress the label and/or title altogether.",
        name:"labelAsTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CheckboxItem"
    },
    "method:FormItem.getOptionDataSource":{
        ref:"method:FormItem.getOptionDataSource",
        flags:"",
        description:" Returns the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} for this item. Default implementation\n will return the  ${isc.DocUtils.linkForRef('class:DataSource')} of this item's form if \n no explicit <code>optionDataSource</code> is specified.\n",
        name:"getOptionDataSource",
        seeAlso:[],
        type:"method",
        definingClass:"class:FormItem",
        groups:["display_values"]
    },
    "type:RPCCallback":{
        ref:"type:RPCCallback",
        treeLocation:["Client Reference/RPC"],
        description:"A ${isc.DocUtils.linkForRef('type:Callback')} to evaluate when an RPCRequest completes.\n <p>\n Parameters passed to this callback are:\n <ul>\n <li>rpcResponse: an ${isc.DocUtils.linkForRef('class:RPCResponse')} encapsulating the server response to your\n     request\n <li>data: just the \"data\" property from the RPCResponse, for convenience\n <li>rpcRequest: the ${isc.DocUtils.linkForRef('class:RPCRequest')} that was sent.  You can use\n     ${isc.DocUtils.linkForRef('attr:RPCRequest.clientContext')} to track state during the server turnaround.\n </ul>\n For example, to take the data returned by the server and display it in a previously created\n ListGrid with the ID \"myGrid\":\n <pre>\n     isc.RPCManager.send(\"getData\", \"myGrid.setData(data)\");\n </pre>\n Or\n <pre>\n     isc.RPCManager.send(\"getData\", function (rpcResponse, data, rpcRequest) { \n                                        myGrid.setData(data)\n                                    });\n </pre>",
        name:"RPCCallback",
        seeAlso:[
            {ref:"class:RPCRequest", description:null},
            {ref:"class:RPCResponse", description:null}
        ],
        type:"type"
    },
    "attr:ComboBoxItem.filterLocally":{
        ref:"attr:ComboBoxItem.filterLocally",
        flags:"IRA",
        description:"If <code>filterLocally</code> is set for this item, and this item is showing options \n from a dataSource, fetch the entire set of options from the server, and use these values\n to map the item value to the appropriate display value. Also use <code>\"local\"</code>\n type filtering on drop down list of options.\n <P>\n This means data will only be fetched once from the server, and then filtered on the\n client.",
        name:"filterLocally",
        seeAlso:[
            {ref:"attr:FormItem.filterLocally", description:null}
        ],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "attr:DateItem.enforceDate":{
        ref:"attr:DateItem.enforceDate",
        flags:"IRWA",
        description:"Can this field be set to a non-date value [other than null]?\n <P>\n When set to true, ${isc.DocUtils.linkForRef('method:FormItem.setValue')} will return false without setting the item value\n and log a warning if passed something other than a valid date value.\n If the dateItem is showing a ${isc.DocUtils.linkForRef('attr:DateItem.useTextField','free-form text entry field')},\n and a user enters a text value which cannot be parsed into a valid date, the item will\n automatically redraw and display the ${isc.DocUtils.linkForRef('attr:DateItem.invalidDateStringMessage')} (though at this\n point calling ${isc.DocUtils.linkForRef('method:FormItem.getValue')} will return the string entered by the user).\n <P>\n When set to false, a user may enter value that is not a valid date (for example, \"Not\n applicable\") and the value will not immediately be flagged as an error.  However note\n that for the value to actually pass validation you would need to declare the field as\n not of \"date\" type, for example:\n <pre>\n     {name:\"startDate\", type:\"dateOrOther\", editorType:\"DateItem\", useTextField:true },\n </pre>\n The type \"dateOrOther\" could be declared as a ${isc.DocUtils.linkForRef('class:SimpleType')}, with validators that\n will accept either a valid date or certain special Strings (like \"Not Available\").\n <P>\n Only applies to dateItems where ${isc.DocUtils.linkForRef('attr:DateItem.useTextField')} is true. Non-Date values\n are never supported in items where useTextField is false.",
        name:"enforceDate",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DateItem"
    },
    "method:TabSet.tabDeselected":{
        ref:"method:TabSet.tabDeselected",
        flags:"",
        description:"  Notification fired when a tab is deselected.        \n",
        name:"tabDeselected",
        seeAlso:[],
        params:[
            {description:"number of the tab", optional:false, name:"tabNum", defaultValue:null,
             type:"number"},
            {description:"pane for this tab", optional:false, name:"tabPane", defaultValue:null,
             type:"Canvas"},
            {description:"id of the tab", optional:false, name:"ID", defaultValue:null, type:"id"},
            {description:"the tab object (not tab button instance)", optional:false, name:"tab",
             defaultValue:null, type:"tab"}
        ],
        type:"method",
        definingClass:"class:TabSet"
    },
    "attr:EdgedCanvas.edgeOffsetBottom":{
        ref:"attr:EdgedCanvas.edgeOffsetBottom",
        flags:"IRA",
        description:"Amount the contained Canvas should be offset from the bottom.  Defaults to the size for\n the bottom edge.  Set smaller to allow the contained Canvas to overlap the edge and\n corner media.",
        name:"edgeOffsetBottom",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:EdgedCanvas",
        groups:["imageEdges"]
    },
    "classMethod:ClassFactory.getClass":{
        ref:"classMethod:ClassFactory.getClass",
        flags:"",
        description:"\n	Given a class name, return a pointer to the Class object for that class\n\n",
        name:"getClass",
        seeAlso:[],
        params:[
            {description:"name of a class", optional:false, name:"className", defaultValue:null,
             type:"string"}
        ],
        returns:{description:"Class object, or null if not found", type:"Class"},
        type:"classMethod",
        definingClass:"object:ClassFactory"
    },
    "attr:PickList.displayField":{
        ref:"attr:PickList.displayField",
        flags:"IRW",
        getter:["FormItem.getDisplayFieldName"],
        description:"Specifies an alternative field from which display values should be retrieved for this\n item.\n <P>\n The display field can be either another field value in the same record or a field that\n must be retrieved from a related ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource','optionDataSource')}.\n <P>\n If this item is not databound (${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} is unset), or bound \n to the same dataSource as the form as a whole, this item will call\n ${isc.DocUtils.linkForRef('method:DynamicForm.getValue','form.getValue()')} \n the form named after is implemented by picking up the\n value of the specified field from the Form's values object.\n <P>\n Otherwise this item will attempt to map its underlying value to a display value\n by retrieving a record from the ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')} where the \n ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} matches this item's value, and displaying the \n <code>displayField</code> value from that record.\n Note that if <code>optionDataSource</code> is set and this value is not\n set, ${isc.DocUtils.linkForRef('method:FormItem.getDisplayFieldName')} will return the dataSource title field by default.\n <P>\n This essentially enables the specified <code>optionDataSource</code> to be used as\n a server based ${isc.DocUtils.linkForRef('group:valueMap')}.",
        name:"displayField",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList",
        groups:["display_values"]
    },
    "attr:Window.autoSize":{
        flags:"IRW",
        type:"attr",
        ref:"attr:Window.autoSize",
        description:"If true, the window is resized automatically to accommodate the contents\n			of the body, if they would otherwise require scrolling.",
        name:"autoSize",
        setter:["setAutoSize"],
        valueType:"boolean",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('windowAutosize')}"],
        defaultValue:"false",
        groups:["appearance"],
        definingClass:"class:Window"
    },
    "method:CubeGrid.setFacetTitle":{
        ref:"method:CubeGrid.setFacetTitle",
        flags:"",
        description:" Set the title of a facet (appears in facet label)\n\n",
        name:"setFacetTitle",
        seeAlso:[],
        params:[
            {description:"facet to update", optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"title for the facet", optional:false, name:"newTitle", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["metadata"]
    },
    "attr:Img.imageType":{
        ref:"attr:Img.imageType",
        flags:"IRW",
        description:"Indicates whether the image should be tiled/cropped, stretched, or centered when the\n          size of this widget does not match the size of the image. See ImageStyle for\n          details",
        name:"imageType",
        setter:["setImageType"],
        seeAlso:[],
        valueType:"ImageStyle",
        defaultValue:"Img.STRETCH",
        type:"attr",
        definingClass:"class:Img",
        groups:["appearance"]
    },
    "classMethod:Page.isLoaded":{
        ref:"classMethod:Page.isLoaded",
        flags:"",
        description:"		Has the page finished loading?\n\n",
        name:"isLoaded",
        seeAlso:[],
        returns:{description:"true == page is done loading", type:"boolean"},
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:FormItem.picker":{
        ref:"attr:FormItem.picker",
        flags:"IRW",
        description:"The component that will be displayed when formItem.showPicker() is called due to a click\n on the ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon','pickerIcon')}.\n <P>\n Can be specified directly as a Canvas, or created automatically via the\n ${isc.DocUtils.linkForRef('type:AutoChild')} pattern.\n <P>\n Note that the picker is not automatically destroyed with the FormItem that uses it, in\n order to allow possibly recycling of picker components.  To destroy a single-use picker,\n override ${isc.DocUtils.linkForRef('method:Canvas.destroy')}.",
        name:"picker",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "classMethod:Class.addClassProperties":{
        ref:"classMethod:Class.addClassProperties",
        flags:"",
        description:"\n	Add static (Class-level) properties and methods to this object<br><br>\n\n	These properties can then be accessed as MyClass.property, or for functions, called as\n  MyClass.methodName()\n\n",
        name:"addClassProperties",
        seeAlso:[],
        params:[
            {description:"objects with properties to add (think named parameters).										all the properties of each argument will be applied\n										as class-level properties.",
             optional:true, name:"arguments 0-N", defaultValue:null, type:"object"}
        ],
        returns:{description:"the class after properties have been added to it", type:"object"},
        type:"classMethod",
        definingClass:"class:Class"
    },
    "attr:FacetValue.parentId":{
        ref:"attr:FacetValue.parentId",
        flags:"IRA",
        description:"For tree facets (${isc.DocUtils.linkForRef('attr:Facet.isTree','facet.isTree')}), id of this facetValue's\n parent facetValue.",
        name:"parentId",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FacetValue"
    },
    "attr:BatchUploader.uploadFormFields":{
        ref:"attr:BatchUploader.uploadFormFields",
        flags:"IR",
        description:"Optional fields for the uploadForm.",
        name:"uploadFormFields",
        seeAlso:[],
        valueType:"List of FormItem Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:BatchUploader"
    },
    "method:WebService.getInputDS":{
        ref:"method:WebService.getInputDS",
        flags:"",
        description:" Get a DataSource representing the input message to a web service operation.\n <P>\n This DataSource is suitable for use as\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','form.dataSource')} for a form that the user fills out\n when providing inputs to call this web service operation.\n\n",
        name:"getInputDS",
        seeAlso:[],
        params:[
            {description:"name of the web service operation whose inputs the                               returned DataSource will represent",
             optional:false, name:"operationName", defaultValue:null, type:"String"}
        ],
        returns:{description:"DataSource representing the input message of a web service                      operation",
         type:"DataSource"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('wsdlBinding')}"],
        definingClass:"class:WebService"
    },
    "class:SectionStack":{
        methods:["method:SectionStack.addItem", "method:SectionStack.removeItem", 
         "method:SectionStack.setSectionProperties","method:SectionStack.addSection", "method:SectionStack.removeSection", 
         "method:SectionStack.getSections","method:SectionStack.reorderSection", "method:SectionStack.moveSection", 
         "method:SectionStack.showSection","method:SectionStack.expandSection", "method:SectionStack.hideSection", 
         "method:SectionStack.collapseSection","method:SectionStack.sectionIsVisible", "method:SectionStack.getVisibleSections", 
         "method:SectionStack.sectionIsExpanded","method:SectionStack.getExpandedSections", "method:SectionStack.setSectionTitle", 
         "method:SectionStack.getSectionHeader","method:SectionStack.getSectionNumber", "method:SectionStack.sectionHeaderClick"],
        ref:"class:SectionStack",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:SectionStack.overflow", "attr:SectionStack.sectionHeaderClass", 
         "attr:SectionStack.headerHeight","attr:SectionStack.sections", "attr:SectionStack.canResizeSections", 
         "attr:SectionStack.canTabToHeaders","attr:SectionStack.scrollSectionIntoView", "attr:SectionStack.animateSections", 
         "attr:SectionStack.visibilityMode","attr:SectionStack.itemIndent", "attr:SectionStack.itemStartIndent", 
         "attr:SectionStack.itemEndIndent","attr:SectionStack.showExpandControls"],
        description:"A container that manages a list of sections of widgets, each with a header.  Sometimes called\n an \"Accordion\".\n <P>\n SectionStack can be configured so that only one section is visible at a time (similar to MS Outlook's\n left-hand Nav), or to allow multiple sections to be visible and share the available space.",
        name:"SectionStack",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('sectionsExpandCollapse')}"]
    },
    "attr:Calendar.currentViewName":{
        ref:"attr:Calendar.currentViewName",
        flags:"IRW",
        getter:["getCurrentViewName"],
        description:"The name of the view that should be visible initially by default.",
        name:"currentViewName",
        setter:["setCurrentViewName"],
        seeAlso:[],
        valueType:"ViewName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "classMethod:Page.checkBrowserAndRedirect":{
        ref:"classMethod:Page.checkBrowserAndRedirect",
        flags:"",
        description:" Check whether the browser is supported by the Isomorphic SmartClient system. Behavior depends\n upon the value of ${isc.DocUtils.linkForRef('classAttr:Page.unsupportedBrowserAction')}:\n <ul>\n <li><code>\"continue\"</code> Load the page without notifying the user of potential issues</li>\n <li><code>\"confirm\"</code> Notify the user via a standard confirm dialog that their browser is\n  not supported. Provide options to continue anyway, or redirect to another page. Text of the\n  confirm dialog is retrieved by calling ${isc.DocUtils.linkForRef('classMethod:Page.getUnsupportedBrowserPromptString')}.</li>\n <li><code>\"redirect\"</code> Automatically redirect to the another URL</li>\n </ul>\n If redirecting to another page is necessary, and no explicit URL is provided we will use\n ${isc.DocUtils.linkForRef('classAttr:Page.defaultUnsupportedBrowserURL')}.\n <p>\n This method is commonly called as part of the ${isc.DocUtils.linkForRef('group:skinning','skinning')} logic after page\n load.\n\n",
        name:"checkBrowserAndRedirect",
        seeAlso:[
            {ref:"classAttr:Page.unsupportedBrowserAction", description:null},
            {ref:"classMethod:Page.getUnsupportedBrowserPromptString", description:null},
            {ref:"classAttr:Page.defaultUnsupportedBrowserURL", description:null}
        ],
        params:[
            {description:"URL of redirect page. May include Isomorphic special directories      such as [SKIN].",
             optional:true, name:"URL", defaultValue:null, type:"string"}
        ],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "attr:Window.bodyColor":{
        ref:"attr:Window.bodyColor",
        flags:"IRW",
        description:"Color of the Window body. Overrides the background color specified in the style.",
        name:"bodyColor",
        seeAlso:[
            {ref:"method:Window.flash", description:null}
        ],
        valueType:"string",
        defaultValue:"\"#FFFFFF\"",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "attr:MultiSortPanel.initialSort":{
        ref:"attr:MultiSortPanel.initialSort",
        flags:"IR",
        description:"The initial sort configuration to show in the ${isc.DocUtils.linkForRef('attr:MultiSortPanel.optionsGrid')}.",
        name:"initialSort",
        seeAlso:[],
        valueType:"Array of SortSpecifier",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiSortPanel"
    },
    "classAttr:GroupingMessages.byMilisecondsTitle":{
        ref:"classAttr:GroupingMessages.byMilisecondsTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a time field by miliseconds.",
        name:"byMilisecondsTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Miliseconds\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "type:treeGridOpenState":{
        ref:"type:treeGridOpenState",
        description:"An object containing the open state for a treeGrid.\n Note that this object is not intended to be interrogated directly, but may be stored \n (for example) as a blob on the server for state persistence across sessions.",
        name:"treeGridOpenState",
        seeAlso:[],
        type:"type",
        groups:["viewState"]
    },
    "method:DynamicForm.setValue":{
        ref:"method:DynamicForm.setValue",
        flags:"",
        description:"   Sets the value for some field\n",
        name:"setValue",
        seeAlso:[],
        params:[
            {description:"Name of the field being updated. A ${isc.DocUtils.linkForRef('type:dataPath')} may                           be passed to set nested values",
             optional:false, name:"fieldName", defaultValue:null, type:"string"},
            {description:"New value.", optional:false, name:"value", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ColorPicker.selectedColorLabel":{
        ref:"attr:ColorPicker.selectedColorLabel",
        flags:"IR",
        description:"The label shown next to the selected color box.",
        name:"selectedColorLabel",
        seeAlso:[],
        valueType:"text",
        defaultValue:"\"Selected Color\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "method:Canvas.setDataPath":{
        ref:"method:Canvas.setDataPath",
        flags:"",
        description:" Setter for the ${isc.DocUtils.linkForRef('attr:Canvas.dataPath')} attribute. This method may be called directly at runtime\n to set the dataPath on a component, and will also be re-run automatically whenever a canvas'\n parentElement changes due to a call to addChild(). This method handles automatically binding\n the component to the appropriate valuesManager if necessary.\n",
        name:"setDataPath",
        seeAlso:[],
        params:[
            {description:"new dataPath", optional:false, name:"dataPath", defaultValue:null,
             type:"dataPath"}
        ],
        type:"method",
        definingClass:"class:Canvas"
    },
    "attr:Calendar.trailingDateField":{
        ref:"attr:Calendar.trailingDateField",
        flags:"IR",
        description:"The name of the end date field in a ${isc.DocUtils.linkForRef('object:CalendarEvent')}.",
        name:"trailingDateField",
        seeAlso:[
            {ref:"object:CalendarEvent", description:null}
        ],
        valueType:"String",
        defaultValue:"\"trailingDate\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["calendarEvent"]
    },
    "attr:ColumnTree.showHeaders":{
        ref:"attr:ColumnTree.showHeaders",
        flags:"IR",
        description:"If set, each column in the ColumnTree will show a header with the title of the selected\n node from the column to the left.",
        name:"showHeaders",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ColumnTree"
    },
    "method:FormItem.iconKeyPress":{
        ref:"method:FormItem.iconKeyPress",
        flags:"",
        description:"      StringMethod.\n      Default action to fire when an icon has keyboard focus and the user types a key.\n      May be overridden by setting <code>keyPress</code> on the form item icon directly.\n",
        name:"iconKeyPress",
        seeAlso:[],
        params:[
            {description:"name of the key pressed", optional:false, name:"keyName", defaultValue:null,
             type:"string"},
            {description:"character produced by the keypress", optional:false, name:"character",
             defaultValue:null, type:"character"},
            {description:"a pointer to this item's form", optional:false, name:"form",
             defaultValue:null, type:"DynamicForm"},
            {description:"a pointer to this form item", optional:false, name:"item", defaultValue:null,
             type:"FormItem"},
            {description:"a pointer to the icon that recieved the click event.", optional:false,
             name:"icon", defaultValue:null, type:"FormItemIcon"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "object:Operator":{
        methods:["method:Operator.condition", "method:Operator.compareCriteria"],
        type:"object",
        ref:"object:Operator",
        classAttrs:[],
        treeLocation:["Client Reference/Data Binding/DataSource"],
        classMethods:[],
        description:"Specification of an operator for use in filtering, for example \"equals\".\n Use with ${isc.DocUtils.linkForRef('method:DataSource.addSearchOperator')} to define custom filtering behaviors for\n client-side filtering.",
        attrs:["attr:Operator.ID", "attr:Operator.title", "attr:Operator.titleProperty", 
         "attr:Operator.fieldTypes","attr:Operator.requiresServer", "attr:Operator.hidden", "attr:Operator.valueType", 
         "attr:Operator.editorType"],
        name:"Operator",
        seeAlso:[],
        serverDS:"allowed",
        groups:["advancedFilter"]
    },
    "classAttr:Validator.mustBeLessThan":{
        ref:"classAttr:Validator.mustBeLessThan",
        flags:"IRA",
        description:"Default error message to display when standard <code>integerRange</code> type validator\n  returns false because the value passed in is greater than the specified maximum.\n This is a dynamic string - text within <code>\\${...}</code> will be evaluated as JS code\n when the message is displayed, with <code>max</code> and <code>min</code> available as\n variables mapped to <code>validator.max</code> and <code>validator.min</code>.",
        name:"mustBeLessThan",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Must be no more than ${max}\"",
        type:"classAttr",
        definingClass:"class:Validator",
        groups:["i18nMessages"]
    },
    "method:ListGrid.willAcceptDrop":{
        ref:"method:ListGrid.willAcceptDrop",
        flags:"A",
        description:"\n This method overrides ${isc.DocUtils.linkForRef('method:Canvas.willAcceptDrop')} and works as follows:<br>\n <ul>\n <li>If ${isc.DocUtils.linkForRef('method:Canvas.willAcceptDrop')} (the superclass definition) returns false, this \n     method always returns false.  This allows ${isc.DocUtils.linkForRef('attr:Canvas.dragType')} and\n     ${isc.DocUtils.linkForRef('attr:Canvas.dropTypes')} to be used to configure eligibility for drop.  By default,\n     a ListGrid has no dropTypes configured and so this check will not prevent a drop.</li>\n <li>If this is a self-drop, that is, the user is dragging a record within this list, this is\n     an attempted drag-reorder.  If ${isc.DocUtils.linkForRef('attr:ListGrid.canReorderRecords')} is \n     false, this method returns false.</li>\n <li>If the ${isc.DocUtils.linkForRef('classMethod:EventHandler.getDragTarget','dragTarget')} is another widget, if\n     ${isc.DocUtils.linkForRef('attr:ListGrid.canAcceptDroppedRecords')} is false this method returns false.</li>\n <li>If a call to ${isc.DocUtils.linkForRef('method:ListGrid.getDragData')} on the <code>dragTarget</code> fails to return\n     an record object or an array of records, this method returns false.</li> \n <li>If a the drop target record is disabled or has ${isc.DocUtils.linkForRef('attr:ListGridRecord.canAcceptDrop')} \n     set to false, return false.</li>\n </ul>\n Note that this method may be called repeatedly during a drag-drop interaction to update the\n UI and notify the user as to when they may validly drop data.\n\n",
        name:"willAcceptDrop",
        seeAlso:[
            {ref:"attr:ListGridRecord.canAcceptDrop", description:null},
            {ref:"method:ListGrid.getDragData", description:null}
        ],
        returns:{description:"true if this component will accept a drop of the dragData", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events", "dragging"]
    },
    "attr:Label.iconHeight":{
        ref:"attr:Label.iconHeight",
        flags:"IR",
        description:"Height in pixels of the icon image.\n <P>\n If unset, defaults to <code>iconSize</code>",
        name:"iconHeight",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "classMethod:XMLTools.loadXMLSchema":{
        ref:"classMethod:XMLTools.loadXMLSchema",
        flags:"A",
        description:" Load an XML file containing XML schema definitions and create DataSource and SimpleType\n objects to represent the schema.  You can use to loaded schema to bind ISC components,\n perform validation, create editing interfaces, and build other metadata-driven interfaces.\n You can also use ${isc.DocUtils.linkForRef('attr:DataSource.inheritsFrom','schema inheritance')} to overlay\n presentation-specific data (such as user-visible titles) on top of XML Schema.\n <p>\n In the loaded XML Schema, all &lt;xsd:complexType&gt; declarations become SmartClient\n DataSources, and all &lt;xsd:simpleType&gt; definitions become SmartClient\n ${isc.DocUtils.linkForRef('class:SimpleType','atomic type definitions')}.  \n <p>\n By default, named complexType definitions and named element definitions containing\n complexTypes become global DataSources, that is, they can be fetched with\n ${isc.DocUtils.linkForRef('classMethod:DataSource.getDataSource')}.  Inline complexType definitions get\n automatically generated names.\n <p>\n Named simpleType declarations become global ${isc.DocUtils.linkForRef('class:SimpleType','atomic types')}, that is,\n subsequently defined DataSources can use them for ${isc.DocUtils.linkForRef('attr:DataSourceField.type')}.  XML\n schema \"restrictions\" for simple types are automatically translated to\n ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap')} or ${isc.DocUtils.linkForRef('attr:DataSourceField.validators')} as\n appropriate.\n <P>\n The created SchemaSet object is available in the callback as the single parameter\n \"schemaSet\", or can retrieved via <code>SchemaSet.get(schemaNamespace)</code>.\n <P>\n NOTE: unless you are building an application that dynamically loads XML Schema \n without prior knowledge, instead of calling loadXMLSchema(), you should either:\n <ul>\n <li> use the ${isc.DocUtils.linkForRef('group:loadXMLSchemaTag')} tag to eliminate the need for an asynchronous\n download of an XML Schema file as part of application startup, <b>OR</b>\n <li> use the \"WSDL\" tab in the Developer Console to obtain the XML Schema definition as a\n JavaScript file that can be loaded via a normal HTML &lt;SCRIPT SRC=&gt; tag and/or\n combined with other JavaScript files.  \n </ul>\n <P>\n NOTE: required fields: the XML Schema concept of \"required\" for an attribute or subelement, \n expressed via use=\"required\" (for an attribute) or minOccurs > 0 (for a subelement), is\n that the attribute or element must be present in the XML document <i>but can have any\n value</i>, including being empty or null.  The SmartClient notion of required means\n non-null.  You can express the SmartClient notion of required in XML Schema with the\n combination of minOccurs>0 and a minLength or length \"restriction\", and SmartClient \n will recognize the field as SmartClient-required, with all of the behaviors that implies\n (eg, specially styled form titles, automatic validation, etc).\n\n",
        name:"loadXMLSchema",
        seeAlso:[],
        params:[
            {description:"URL to load the schema from", optional:false, name:"schemaURL",
             defaultValue:null, type:"URL"},
            {description:"signature is callback(schemaSet)", optional:false, name:"callback",
             defaultValue:null, type:"Callback"},
            {description:"additional properties to set on the RPCRequest                                         that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"RPCRequest"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('xmlSchemaImport')}"],
        definingClass:"class:XMLTools",
        groups:["xmlSchema"]
    },
    "attr:Menu.submenuDirection":{
        ref:"attr:Menu.submenuDirection",
        flags:"IRW",
        description:"Should submenus show up on our left or right. Can validly be set to <code>\"left\"</code>\n  or <code>\"right\"</code>",
        name:"submenuDirection",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"right\"",
        type:"attr",
        definingClass:"class:Menu"
    },
    "method:FormItem.setHintStyle":{
        ref:"method:FormItem.setHintStyle",
        flags:"",
        description:" Set the hintStyle for this item\n",
        name:"setHintStyle",
        seeAlso:[],
        params:[
            {description:"new style for hint text", optional:false, name:"hintStyle", defaultValue:null,
             type:"CSSStyleName"}
        ],
        type:"method",
        definingClass:"class:FormItem"
    },
    "attr:ListGrid.neverValidate":{
        ref:"attr:ListGrid.neverValidate",
        flags:"IRWA",
        description:"If true, validation will not occur as a result of cell editing for this grid.",
        name:"neverValidate",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridValidation"]
    },
    "method:Tree.getChildren":{
        ref:"method:Tree.getChildren",
        flags:"",
        description:"\n Returns all children of a node.  If the node is a leaf, this method returns null.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')} rather than a simple \n array - so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} \n interface instead of as a native Javascript Array.\n\n",
        name:"getChildren",
        seeAlso:[],
        params:[
            {description:"The node whose children you want to fetch.", optional:false, name:"node",
             defaultValue:null, type:"TreeNode"}
        ],
        returns:{description:"List of children for the node (empty List if node is a leaf                      or has no children)",
         type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:ListGrid.willFetchData":{
        ref:"method:ListGrid.willFetchData",
        flags:"",
        description:" Compares the specified criteria with the current criteria applied to this component's\n data object and determines whether the new criteria could be satisfied from the currently cached\n set of data, or if a new filter/fetch operation will be required.\n <P>\n This is equivalent to calling <code>this.data.willFetchData(...)</code>.\n Always returns true if this component is not showing a set of data from the dataSource. \n \n",
        name:"willFetchData",
        seeAlso:[],
        params:[
            {description:"new criteria to test.", optional:false, name:"newCriteria", defaultValue:null,
             type:"Criteria"},
            {description:"New text match style. If not passed assumes       textMatchStyle will not be modified.",
             optional:true, name:"textMatchStyle", defaultValue:null, type:"TextMatchStyle"}
        ],
        returns:{description:"true if server fetch would be required to satisfy new criteria.",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["dataBoundComponentMethods"]
    },
    "attr:CubeGrid.valueTitle":{
        ref:"attr:CubeGrid.valueTitle",
        flags:"IR",
        description:"A label for the data values shown in cells, such as \"Sales in Thousands\", typically used\n when the CubeGrid must generate a description for a cell value or set of cell values.\n <P>\n For example, in a CubeGrid showing \"Revenue\" by region and product, a cell with a\n CellRecord like:\n <pre> \n {product:\"chairs\", region:\"northwest\", _value:\"$5k\"}\n </pre>\n Should be described as \"Revenue for Charts for Northwest Region\", not \"Chairs for\n Revenue for Northwest Region\".\n <P>\n For CubeGrids that show multiple types of values at once (eg both \"Revenue\" and\n \"Income\") see ${isc.DocUtils.linkForRef('attr:CubeGrid.metricFacetId')}.",
        name:"valueTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid"
    },
    "attr:DateChooser.showDoubleYearIcon":{
        ref:"attr:DateChooser.showDoubleYearIcon",
        flags:"IRW",
        description:"If this property is set to true the previous and next year buttons will render out the \n previous and next month button icons twice rather than using the\n ${isc.DocUtils.linkForRef('attr:DateChooser.prevYearIcon')} and ${isc.DocUtils.linkForRef('attr:DateChooser.nextYearIcon')}.\n <P>\n Set to <code>true</code> by default as not all skins contain media for the year icons.",
        name:"showDoubleYearIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:DateChooser"
    },
    "attr:Flashlet.codeBase":{
        ref:"attr:Flashlet.codeBase",
        flags:"IR",
        description:"This attribute specifies the minimum version of the flash player required to show this\n  flashlet.\n  <p>\n  The default codeBase is: \"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=5,0,0,0\"\n  <p>",
        name:"codeBase",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:Flashlet"
    },
    "classMethod:isA.Date":{
        ref:"classMethod:isA.Date",
        flags:"",
        description:"\n	Is <code>object</code> a Date object?\n\n",
        name:"Date",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a Date", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "classMethod:Page.getScrollWidth":{
        ref:"classMethod:Page.getScrollWidth",
        flags:"",
        description:"		Get the width of the window contents as they have been drawn.\n		If the page scrolls, this may be larger than the page.getWidth().\n",
        name:"getScrollWidth",
        seeAlso:[],
        returns:{description:"width of the page as drawn", type:"number"},
        type:"classMethod",
        definingClass:"class:Page",
        groups:["sizing"]
    },
    "attr:DynamicForm.itemHoverWidth":{
        ref:"attr:DynamicForm.itemHoverWidth",
        flags:"IRW",
        description:"A default width for hovers shown for items",
        name:"itemHoverWidth",
        seeAlso:[
            {ref:"attr:FormItem.hoverWidth", description:null}
        ],
        valueType:"measure",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('itemHoverHTML')}"],
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "method:Array.addListAt":{
        ref:"method:Array.addListAt",
        flags:"",
        description:" Add list of items list to this array at item pos.  All items after array[pos] will slide down to\n fit new items.\n\n",
        name:"addListAt",
        seeAlso:[],
        params:[
            {description:"new array of items", optional:false, name:"list", defaultValue:null,
             type:"array"},
            {description:"position in this list to put the new items", optional:false, name:"pos",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"the list of items that was added", type:"array"},
        type:"method",
        definingClass:"object:Array",
        groups:["modification"]
    },
    "method:DataBoundComponent.transferDragData":{
        ref:"method:DataBoundComponent.transferDragData",
        flags:"A",
        description:"\n During a drag-and-drop interaction, this method is called to transfer a set of records that\n were dropped onto some other component.  This method is called after the set of records has\n been copied to the other component.  Whether or not this component's data is modified is \n determined by the value of ${isc.DocUtils.linkForRef('attr:DataBoundComponent.dragDataAction')}.\n <P>\n With a <code>dragDataAction</code> of \"move\", a databound component will issue \"remove\"\n dsRequests against its DataSource to actually remove the data, via\n ${isc.DocUtils.linkForRef('method:DataSource.removeData')}.\n\n",
        name:"transferDragData",
        seeAlso:[
            {ref:"method:DataBoundComponent.getDragData", description:null},
            {ref:"method:ListGrid.willAcceptDrop", description:null}
        ],
        returns:{description:"Array of objects that were dragged out of this ListGrid.", type:"Array"},
        type:"method",
        definingClass:"interface:DataBoundComponent"
    },
    "attr:Button.hiliteAccessKey":{
        ref:"attr:Button.hiliteAccessKey",
        flags:"IRW",
        description:"If set to true, if the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.title','title')} of this button contains the\n specified ${isc.DocUtils.linkForRef('attr:Canvas.accessKey','accessKey')}, when the title is displayed to the user\n it will be modified to include HTML to underline the accessKey.<br>\n Note that this property may cause titles that include HTML (rather than simple strings)\n to be inappropriately modified, so should be disabled if your title string includes\n HTML characters.",
        name:"hiliteAccessKey",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Button"
    },
    "attr:ResultSet.allRows":{
        ref:"attr:ResultSet.allRows",
        flags:"IA",
        description:"If the complete set of records for a resultSet is available when the resultSet is created,\n it can be made available to the resultSet via this property at initialization time.\n This data will then be considered cached meaning sorting and filtering can occur on\n the client (no need for server fetch).\n This cached data can be dropped via a call to ${isc.DocUtils.linkForRef('method:ResultSet.invalidateCache')}.",
        name:"allRows",
        seeAlso:[],
        valueType:"Array of records",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ResultSet",
        groups:["fetching", "cacheSync"]
    },
    "method:Canvas.getHeight":{
        ref:"method:Canvas.getHeight",
        flags:"",
        description:" Return the height of this object, in pixels.\n",
        name:"getHeight",
        seeAlso:[],
        returns:{description:"height", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:DynamicForm.valuesHaveChanged":{
        ref:"method:DynamicForm.valuesHaveChanged",
        flags:"",
        description:"          Compares the current set of values with the values stored by the call to the\n          <code>rememberValues()</code> method. Returns true if the values have changed, and false\n          otherwise.\n",
        name:"valuesHaveChanged",
        seeAlso:[],
        returns:{description:"true if current values do not match remembered values", type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["formValues"]
    },
    "attr:ButtonItem.startRow":{
        ref:"attr:ButtonItem.startRow",
        flags:"IRW",
        description:"These items are in a row by themselves by default",
        name:"startRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ButtonItem",
        groups:["formLayout"]
    },
    "object:SectionStackSection":{
        methods:[],
        ref:"object:SectionStackSection",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/SectionStack"],
        attrs:["attr:SectionStackSection.ID", "attr:SectionStackSection.title", 
         "attr:SectionStackSection.items","attr:SectionStackSection.showHeader", "attr:SectionStackSection.resizeable", 
         "attr:SectionStackSection.expanded","attr:SectionStackSection.hidden", "attr:SectionStackSection.canCollapse"],
        description:"Section descriptor used by a SectionStack to describe a section of items which are shown\n or hidden together, and their associated header.",
        name:"SectionStackSection",
        seeAlso:[],
        type:"object"
    },
    "attr:MathFunction.description":{
        ref:"attr:MathFunction.description",
        flags:"IR",
        description:"A short description of this function",
        name:"description",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MathFunction",
        groups:["formulaFields"]
    },
    "method:ListGrid.editorEnter":{
        ref:"method:ListGrid.editorEnter",
        flags:"",
        description:" Callback fired when the user starts editing a new cell.\n <P>\n This callback is typically used to establish dynamic default values via\n ${isc.DocUtils.linkForRef('method:ListGrid.setEditValue')} or ${isc.DocUtils.linkForRef('method:ListGrid.setEditValues')}.\n <P>\n Can also be overriden on a per-field basis via\n ${isc.DocUtils.linkForRef('method:ListGridField.editorEnter','field.editorEnter')}.\n\n",
        name:"editorEnter",
        seeAlso:[
            {ref:"method:ListGridField.editorEnter", description:null}
        ],
        params:[
            {description:"record for the cell being edited.  <b>Will be null</b>                                for a new, unsaved record.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"value for the cell being edited", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"integer"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "attr:CubeGrid.editByCell":{
        ref:"attr:CubeGrid.editByCell",
        flags:"R",
        description:"CubeGrids only support editing by cell",
        name:"editByCell",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["cellEditing"]
    },
    "group:layoutMargin":{
        ref:"group:layoutMargin",
        refs:["method:Layout.setLayoutMargin", "attr:TileLayout.tileMargin", 
         "attr:TileLayout.tileHMargin","attr:TileLayout.tileVMargin", "attr:TileLayout.layoutMargin", 
         "attr:TileLayout.paddingAsLayoutMargin","attr:TileLayout.expandMargins", "attr:Layout.paddingAsLayoutMargin", 
         "attr:Layout.layoutMargin","attr:Layout.layoutLeftMargin", "attr:Layout.layoutRightMargin", 
         "attr:Layout.layoutTopMargin","attr:Layout.layoutBottomMargin", "attr:Layout.membersMargin"],
        name:"layoutMargin",
        seeAlso:[],
        type:"group"
    },
    "attr:SVG.pluginsPage":{
        ref:"attr:SVG.pluginsPage",
        flags:"IR",
        description:"This attribute specifies the page the user should go to to get the plugin required to view\n  this SVG.\n  <p>\n  The default pluginsPage is: \"http://www.adobe.com/svg/viewer/install/\"\n  <p>",
        name:"pluginsPage",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"see below",
        type:"attr",
        definingClass:"class:SVG"
    },
    "method:Canvas.removePeer":{
        ref:"method:Canvas.removePeer",
        flags:"",
        description:" Remove a peer from this Canvas\n",
        name:"removePeer",
        seeAlso:[],
        params:[
            {description:"Peer to be removed from this canvas", optional:false, name:"peer",
             defaultValue:null, type:"Canvas instance"},
            {description:"If this peer was assigned a name it should be passed in here to ensure                      canvas[name] is cleared out",
             optional:true, name:"name", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["containment"]
    },
    "method:CubeGrid.getCellRow":{
        ref:"method:CubeGrid.getCellRow",
        flags:"",
        description:" Given a record in this grid, this method returns the rowNum on which the record is displayed.\n",
        name:"getCellRow",
        seeAlso:[],
        params:[
            {description:"record to find coordinates for", optional:false, name:"cellRecord",
             defaultValue:null, type:"record"}
        ],
        returns:{description:"Row number for the record. Returns -1 if the record is not found.",
         type:"integer"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "group:arrayMath":{
        ref:"group:arrayMath",
        title:"Array Math",
        refs:["method:List.intersect", "method:Array.intersect", "method:Array.max", "method:Array.min", 
         "method:Array.sum","method:Array.and", "method:Array.or"],
        description:"Math operations on entire Arrays at once",
        name:"arrayMath",
        seeAlso:[],
        type:"group"
    },
    "method:GridRenderer.getBaseStyle":{
        ref:"method:GridRenderer.getBaseStyle",
        flags:"A",
        description:" Return the base stylename for this cell.  Default implementation just returns this.baseStyle.\n See ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle','getCellStyle()')} for a general discussion of how to style cells.\n\n",
        name:"getBaseStyle",
        seeAlso:[
            {ref:"method:GridRenderer.getCellStyle", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS class for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:GridRenderer"
    },
    "attr:ComboBoxItem.maskPadChar":{
        ref:"attr:ComboBoxItem.maskPadChar",
        flags:null,
        description:"Not applicable to a ComboBoxItem.",
        name:"maskPadChar",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:ComboBoxItem"
    },
    "method:ListGrid.getEditorValueIcons":{
        ref:"method:ListGrid.getEditorValueIcons",
        flags:"",
        description:" Returns the valueIcons for a field when it is displayed in the editor while editing some\n record. Default implementation will return ${isc.DocUtils.linkForRef('attr:ListGridField.editorValueIcons')} if specified\n otherwise ${isc.DocUtils.linkForRef('attr:ListGridField.valueIcons')}\n",
        name:"getEditorValueIcons",
        seeAlso:[],
        params:[
            {description:"field definition", optional:false, name:"field", defaultValue:null,
             type:"object"},
            {description:"current edit values for the record", optional:false, name:"vaules",
             defaultValue:null, type:"object"}
        ],
        returns:{description:"valueIcons for the editor", type:"object"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "classMethod:Log.echoAll":{
        ref:"classMethod:Log.echoAll",
        flags:"",
        description:" Like echo(), except that if passed an Array, echoAll() will echo() every element of the\n Array.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoAll",
        seeAlso:[
            {ref:"classMethod:Log.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"classMethod",
        definingClass:"class:Log",
        groups:["debug"]
    },
    "attr:MenuBar.menus":{
        ref:"attr:MenuBar.menus",
        flags:"IRW",
        description:"An array of menu object initializers or instantiated menu objects. Buttons for each\n menu item will automatically be created. See the Menu Widget Class for fundamental\n menu properties and other properties.  Titles for the buttons are derived from the \n <code>title</code> property of each menu.",
        name:"menus",
        setter:["setMenus"],
        seeAlso:[
            {ref:"class:Menu", description:null}
        ],
        valueType:"Array of Menu",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MenuBar"
    },
    "attr:Chart.stacked":{
        ref:"attr:Chart.stacked",
        flags:"IR",
        description:"Whether to use stacking for charts that support this (eg bar and column charts).\n If stacked is not set and two facets are supplied, clustering is assumed.",
        name:"stacked",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"interface:Chart"
    },
    "attr:Label.iconSpacing":{
        ref:"attr:Label.iconSpacing",
        flags:"IR",
        description:"Pixels between icon and title text.",
        name:"iconSpacing",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"6",
        type:"attr",
        definingClass:"class:Label",
        groups:["buttonIcon"]
    },
    "method:FormulaBuilder.getUpdatedFieldObject":{
        ref:"method:FormulaBuilder.getUpdatedFieldObject",
        flags:"",
        description:" Returns the entire property-set for the updated field, including title and formula-related\n properties\n \n",
        name:"getUpdatedFieldObject",
        seeAlso:[],
        returns:{description:"The original field along with the updated title and formula", type:"Field"},
        type:"method",
        definingClass:"class:FormulaBuilder",
        groups:["formulaFields"]
    },
    "attr:Window.showTitle":{
        ref:"attr:Window.showTitle",
        flags:"IRW",
        description:"Show a title (typically just text) on the header for this window.",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "headerLabel"]
    },
    "method:Layout.getMemberNumber":{
        ref:"method:Layout.getMemberNumber",
        flags:"",
        description:"\n Given a member Canvas or member ID, return the index of that member within this\n layout's members array\n <p>\n If passed a number, just returns it.\n\n",
        name:"getMemberNumber",
        seeAlso:[
            {ref:"method:Layout.getMember", description:null}
        ],
        params:[
            {description:"identifier for the required member", optional:false, name:"memberID",
             defaultValue:null, type:"string | Canvas | number"}
        ],
        returns:{description:"index of the member canvas (or -1 if not found)", type:"number"},
        type:"method",
        definingClass:"class:Layout"
    },
    "classMethod:Page.scrollTo":{
        ref:"classMethod:Page.scrollTo",
        flags:"",
        description:"		Scroll the window to a specified top and left coordinate.\n\n",
        name:"scrollTo",
        seeAlso:[],
        params:[
            {description:"new left coordinate for window", optional:false, name:"left",
             defaultValue:null, type:"number"},
            {description:"new top coordinate for window", optional:false, name:"top", defaultValue:null,
             type:"number"}
        ],
        type:"classMethod",
        definingClass:"class:Page"
    },
    "method:ValuesManager.removeMembers":{
        ref:"method:ValuesManager.removeMembers",
        flags:"",
        description:"  Remove multiple member forms from this valuesManager.\n",
        name:"removeMembers",
        seeAlso:[
            {ref:"method:ValuesManager.removeMember", description:null}
        ],
        params:[
            {description:"array of forms to remove", optional:false, name:"members", defaultValue:null,
             type:"array"}
        ],
        type:"method",
        definingClass:"class:ValuesManager",
        groups:["members"]
    },
    "method:Canvas.getScrollBottom":{
        ref:"method:Canvas.getScrollBottom",
        flags:"",
        description:" Returns the scrollTop required to scroll vertically to the end of this widget's content.\n",
        name:"getScrollBottom",
        seeAlso:[],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:Canvas.scrollBy":{
        ref:"method:Canvas.scrollBy",
        flags:"",
        description:"  Scroll this widget by some pixel increment in either (or both) direction(s).\n\n",
        name:"scrollBy",
        seeAlso:[],
        params:[
            {description:"Number of pixels to scroll horizontally", optional:false, name:"dX",
             defaultValue:null, type:"number"},
            {description:"Number of pixels to scroll vertically", optional:false, name:"dY",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["scrolling"]
    },
    "method:StatefulCanvas.getStateSuffix":{
        ref:"method:StatefulCanvas.getStateSuffix",
        flags:"",
        description:" Returns the suffix that will be appended to the ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')}  \n as the component changes state and/or is selected.\n <P>\n When the component changes state (eg becomes disabled), a suffix will be appended to this\n style name, reflecting the following states: \"Over\", \"Down\", or \"Disabled\".\n <P>\n If the widget is selected, the suffixes will be \"Selected\", \"SelectedOver\", etc.\n <P>\n If the widget has focus and ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')} is true, and\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is false, the suffixes will be \"Focused\",\n \"FocusedOver\", etc, or if the widget is both selected and focused, \"SelectedFocused\",\n \"SelectedFocusedOver\", etc.\n <P>\n For example, if <code>baseStyle</code> is set to \"button\", this component is\n ${isc.DocUtils.linkForRef('method:StatefulCanvas.isSelected','selected')} and the mouse cursor is over this component, the style\n \"buttonSelectedOver\" will be used. \n \n",
        name:"getStateSuffix",
        seeAlso:[],
        type:"method",
        definingClass:"class:StatefulCanvas"
    },
    "attr:FormItem.iconHeight":{
        ref:"attr:FormItem.iconHeight",
        flags:"IRWA",
        getter:["getIconHeight"],
        description:"Default height for form item icons",
        name:"iconHeight",
        seeAlso:[],
        valueType:"number",
        defaultValue:"20",
        type:"attr",
        definingClass:"class:FormItem",
        groups:["formIcons"]
    },
    "method:Layout.getMemberDefaultBreadth":{
        ref:"method:Layout.getMemberDefaultBreadth",
        flags:"A",
        description:" Return the breadth for a member of this layout which either didn't specify a breadth or\n specified a percent breadth with ${isc.DocUtils.linkForRef('attr:Layout.managePercentBreadth')}:true.\n <P>\n Called only for Layouts which have a ${isc.DocUtils.linkForRef('type:LayoutPolicy','layout policy')} for the breadth\n axis of \"fill\", since Layouts with a breadth policy of \"none\" leave all member breadths alone.\n\n",
        name:"getMemberDefaultBreadth",
        seeAlso:[],
        params:[
            {description:"Component to be sized", optional:false, name:"member", defaultValue:null,
             type:"Canvas"},
            {description:"Value of the currently calculated member breadth. This      may be returned verbatim or manipulated in this method.",
             optional:false, name:"defaultBreadth", defaultValue:null, type:"Number"}
        ],
        type:"method",
        definingClass:"class:Layout",
        groups:["layoutMember"]
    },
    "method:Class.echoAll":{
        ref:"method:Class.echoAll",
        flags:"",
        description:" Like echo(), except that if passed an Array, echoAll() will echo() every element of the\n Array.\n This function is available as a static on every ISC Class and as an instance\n method on every instance of an ISC Class.<br>\n General best practice is to call the method as \"this.echo()\" whenever \"this\" is an\n instance, or call the static classMethod on the ${isc.DocUtils.linkForRef('class:Log')} class otherwise. \n\n",
        name:"echoAll",
        seeAlso:[
            {ref:"method:Class.echo", description:null}
        ],
        params:[
            {description:"object to echo", optional:false, name:"obj", defaultValue:null, type:"any"}
        ],
        returns:{description:"a short string representation of the object", type:"string"},
        type:"method",
        definingClass:"class:Class",
        groups:["debug"]
    },
    "method:Layout.removeMembers":{
        ref:"method:Layout.removeMembers",
        flags:"",
        description:"\n  Removes the specified members from the layout. If any of the removed members have resize\n  bars, the bars will be destroyed.\n\n",
        name:"removeMembers",
        seeAlso:[],
        params:[
            {description:"array of members to be removed, or reference to single 	                                    member.",
             optional:false, name:"members", defaultValue:null, type:"Array or canvas"}
        ],
        type:"method",
        definingClass:"class:Layout"
    },
    "method:Menu.getItem":{
        ref:"method:Menu.getItem",
        flags:"",
        description:" Get a particular MenuItem by index.\n <P>\n If passed a MenuItem, returns it.\n\n",
        name:"getItem",
        seeAlso:[],
        params:[
            {description:"index of the MenuItem", optional:false, name:"item", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"the MenuItem, Pointer to the item, or null if not defined", type:"MenuItem"},
        type:"method",
        definingClass:"class:Menu",
        groups:["menuItems"]
    },
    "attr:SVG.src":{
        ref:"attr:SVG.src",
        flags:"IR",
        description:"Location from which to load the SVG.\n  <p>\n  Note: if you do not specify a src value, ISC will load the special svg 'svgCanvas.svg' from\n  the helpers directory.  This SVG is simply an empty root element - essentially a blank\n  canvas.  You can use this feature to write components that programmatically manipulate the\n  SVG DOM without needing to ship placeholder SVG files.\n  <p>",
        name:"src",
        seeAlso:[],
        valueType:"URL",
        defaultValue:"[HELPERS]svgCanvas.svg",
        type:"attr",
        definingClass:"class:SVG"
    },
    "method:ListGrid.cellHasChanges":{
        ref:"method:ListGrid.cellHasChanges",
        flags:"",
        description:" If this listGrid can be edited, this method will return true if the cell passed in has been\n edited, but the edits have not yet been saved to the ListGrid's data object.\n",
        name:"cellHasChanges",
        seeAlso:[],
        params:[
            {description:"index of row to check for changes", optional:false, name:"rowNum",
             defaultValue:null, type:"number"},
            {description:"index of the col to check for changes", optional:false, name:"colNum",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"returns true if the cell has unsaved edits", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["editing"]
    },
    "class:ImgSectionHeader":{
        methods:[],
        ref:"class:ImgSectionHeader",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout/SectionStack"],
        attrs:["attr:ImgSectionHeader.controls", "attr:ImgSectionHeader.controlsLayout", 
         "attr:ImgSectionHeader.background"],
        description:"SectionHeader class based on an HLayout with ${isc.DocUtils.linkForRef('class:StretchImg')} background.",
        name:"ImgSectionHeader",
        seeAlso:[],
        type:"class"
    },
    "attr:ListGrid.showGroupSummary":{
        ref:"attr:ListGrid.showGroupSummary",
        flags:"IRW",
        description:"If this listGrid supports ${isc.DocUtils.linkForRef('attr:ListGrid.canGroupBy','grouping')}, setting this property will cause\n the grid to render an extra row at the end of each group when grouped, containing summary\n information for the fields.\n Summary information will be calculated by the ${isc.DocUtils.linkForRef('method:ListGridField.getGroupSummary')} method\n if specified, otherwise via the specified ${isc.DocUtils.linkForRef('attr:ListGridField.summaryFunction')}.",
        name:"showGroupSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "attr:ColorPicker.swatchImageURL":{
        ref:"attr:ColorPicker.swatchImageURL",
        flags:"IR",
        description:"The location of the color swatch image file",
        name:"swatchImageURL",
        seeAlso:[],
        valueType:"text",
        defaultValue:"\"[SKIN]ColorPicker/spectrum.png\"",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:CubeGrid.styleName":{
        ref:"attr:CubeGrid.styleName",
        flags:"IRW",
        description:"CSS class for the CubeGrid as a whole",
        name:"styleName",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"normal\"",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "attr:Mail.cc":{
        ref:"attr:Mail.cc",
        flags:"IR",
        description:"Comma-separated list of \"CC\" recipients.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n If you specify a \"Cc:\" line in the body of your email, recipient addresses will be parsed \n and this property will be ignored.",
        name:"cc",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "classMethod:Log.getDefaultLogPriority":{
        ref:"classMethod:Log.getDefaultLogPriority",
        flags:"",
        description:" Retrieves the default priority of messages for this class or instance.\n",
        name:"getDefaultLogPriority",
        seeAlso:[],
        returns:{description:"default priority for logging messages on this object.", type:"LogPriority"},
        type:"classMethod",
        definingClass:"class:Log"
    },
    "group:formTitles":{
        ref:"group:formTitles",
        title:"Form Titles",
        refs:["attr:DynamicForm.titleOrientation", "attr:DynamicForm.titlePrefix", 
         "attr:DynamicForm.rightTitlePrefix","attr:DynamicForm.titleSuffix", "attr:DynamicForm.rightTitleSuffix", 
         "attr:DynamicForm.titleWidth","attr:DynamicForm.wrapItemTitles", "attr:DynamicForm.hiliteRequiredFields", 
         "attr:DynamicForm.requiredTitlePrefix","attr:DynamicForm.requiredRightTitlePrefix", "attr:DynamicForm.requiredTitleSuffix", 
         "attr:DynamicForm.requiredRightTitleSuffix","type:TitleOrientation"],
        description:"Properties that affect form item title placement and styling.",
        name:"formTitles",
        seeAlso:[],
        type:"group"
    },
    "classMethod:isc.shallowClone":{
        ref:"classMethod:isc.shallowClone",
        flags:"",
        description:" Creates a shallow copy of the passed-in Object or Array of Objects, that is, copies all\n properties of an Object to a new Object, so that the clone now has exactly the same property\n values as the original Object.\n <P>\n If <code>shallowClone()</code> is passed an immutable type such as String and Number, it is returned\n unchanged.  Dates are copied via <code>new Date(originalDate.getTime())</code>.\n <P>\n Note that if an Array is passed, all members of the Array will be cloned.  For a copy of an\n Array that contains exactly the same members (not copies), use Array.duplicate().\n <P>\n Only an Array directly passed to <code>shallowClone()</code> is copied.  Arrays contained\n within Arrays will not be copied.\n\n",
        name:"shallowClone",
        seeAlso:[],
        params:[
            {description:"object to be cloned", optional:false, name:"object", defaultValue:null,
             type:"Object or Array of Object"}
        ],
        returns:{description:"a shallow copy of the passed-in data", type:"Object or Array of Object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "attr:PickTreeItem.valueTree":{
        ref:"attr:PickTreeItem.valueTree",
        flags:"IR",
        description:"A ${isc.DocUtils.linkForRef('class:Tree')} of options from which the user can select.",
        name:"valueTree",
        seeAlso:[],
        valueType:"tree",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('pickTree')}"],
        definingClass:"class:PickTreeItem"
    },
    "method:List.findNextIndex":{
        ref:"method:List.findNextIndex",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.findIndex')}, but inspects a range from startIndex to endIndex.\n\n",
        name:"findNextIndex",
        seeAlso:[],
        params:[
            {description:"first index to consider", optional:false, name:"startIndex",
             defaultValue:null, type:"int"},
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"},
            {description:"last index to consider", optional:true, name:"endIndex", defaultValue:null,
             type:"int"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"interface:List",
        groups:["access", "find"]
    },
    "classAttr:Operators.betweenInclusiveTitle":{
        ref:"classAttr:Operators.betweenInclusiveTitle",
        flags:"IR",
        description:"Title for the \"betweenInclusive\" operator",
        name:"betweenInclusiveTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"between (inclusive)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:Tab.pane":{
        ref:"attr:Tab.pane",
        flags:"IRW",
        description:"Specifies the pane associated with this tab.  You have two options for the value of\n the pane attribute:\n <ul>\n <li><b>ID</b> - The global ID of an already created Canvas (or subclass).\n <li><b>Canvas</b> - A live instance of a Canvas (or subclass).\n </ul>\n You can change the pane associated with a given tab after the TabSet has been created by\n calling ${isc.DocUtils.linkForRef('method:TabSet.updateTab')}.",
        name:"pane",
        seeAlso:[
            {ref:"method:TabSet.updateTab", description:null}
        ],
        valueType:"ID or Canvas",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:Tab"
    },
    "classAttr:GroupingMessages.byUpcomingTitle":{
        ref:"classAttr:GroupingMessages.byUpcomingTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by upcoming dates.",
        name:"byUpcomingTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Upcoming\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "classMethod:Time.toShortTime":{
        ref:"classMethod:Time.toShortTime",
        flags:"",
        description:" Given a date object, return the time associated with the date as a short string.\n If no formatter is passed, use the standard formatter set up via ${isc.DocUtils.linkForRef('classMethod:Time.setShortDisplayFormat')}\n",
        name:"toShortTime",
        seeAlso:[],
        params:[
            {description:"Date to convert to a time string.", optional:false, name:"date",
             defaultValue:null, type:"Date"},
            {description:"Optional custom formatter to use. Will accept  a function (which will be passed a pointer to the Date to format), or\n  a string designating a standard formatter",
             optional:true, name:"formatter", defaultValue:null, type:"timeFormatter | function"}
        ],
        type:"classMethod",
        definingClass:"class:Time"
    },
    "method:Progressbar.percentChanged":{
        ref:"method:Progressbar.percentChanged",
        flags:"A",
        description:" This method is called when the percentDone value changes. Observe this method to be notified upon\n a change to the percentDone value.\n\n",
        name:"percentChanged",
        seeAlso:[
            {ref:"method:Class.observe", description:null}
        ],
        type:"method",
        definingClass:"class:Progressbar"
    },
    "attr:MultiSortPanel.otherSortLevelTitle":{
        ref:"attr:MultiSortPanel.otherSortLevelTitle",
        flags:"IR",
        description:"The title-text to appear in the first column for all sort-levels other than the first.",
        name:"otherSortLevelTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Then by\"",
        type:"attr",
        definingClass:"class:MultiSortPanel",
        groups:["i18nMessages"]
    },
    "attr:DynamicForm.titleSuffix":{
        ref:"attr:DynamicForm.titleSuffix",
        flags:"IRW",
        description:"The string appended to the title of every item in this form.",
        name:"titleSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&nbsp;:\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "classAttr:Operators.notTitle":{
        ref:"classAttr:Operators.notTitle",
        flags:"IR",
        description:"Title for the \"not\" operator",
        name:"notTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"Match None\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:PickListMenu.dataProperties":{
        ref:"attr:PickListMenu.dataProperties",
        flags:"IRWA",
        description:"For databound ListGrids, this attribute can be used to customize the ${isc.DocUtils.linkForRef('class:ResultSet')}\n object created for this grid when data is fetched",
        name:"dataProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:PickListMenu",
        groups:["databinding"]
    },
    "method:DynamicForm.editRecord":{
        ref:"method:DynamicForm.editRecord",
        flags:"",
        description:"\n Edit an existing record.  Updates this editors values to match the values of the record \n passed in, via ${isc.DocUtils.linkForRef('method:DynamicForm.setValues')}.\n <P>\n This method will also call ${isc.DocUtils.linkForRef('method:DynamicForm.setSaveOperationType')} to ensure \n subsequent calls to <code>saveData()</code> will use an <code>update</code> rather than\n an <code>add</code> operation.\n\n",
        name:"editRecord",
        seeAlso:[
            {ref:"method:DynamicForm.saveData", description:null}
        ],
        params:[
            {description:"the record to be edited as a map of field names to their corresponding values",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"}
        ],
        type:"method",
        definingClass:"class:DynamicForm",
        groups:["dataBoundComponentMethods"]
    },
    "group:events":{
        ref:"group:events",
        refs:["method:CubeGrid.getEventRow", "method:CubeGrid.getEventColumn", 
         "method:CubeGrid.facetValueOver","method:CubeGrid.facetValueOut", "method:CubeGrid.facetValueHover", 
         "method:CubeGrid.facetValueContextClick","method:CubeGrid.facetLabelOver", "method:CubeGrid.facetLabelOut", 
         "method:CubeGrid.facetLabelHover","method:GridRenderer.getEventRow", "method:GridRenderer.getEventColumn", 
         "method:GridRenderer.getNearestRowToEvent","method:GridRenderer.getNearestColToEvent", "method:GridRenderer.cellOut", 
         "method:GridRenderer.cellOver","method:GridRenderer.rowOut", "method:GridRenderer.rowOver", 
         "method:GridRenderer.cellContextClick","method:GridRenderer.rowContextClick", "method:GridRenderer.cellMouseDown", 
         "method:GridRenderer.rowMouseDown","method:GridRenderer.cellMouseUp", "method:GridRenderer.rowMouseUp", 
         "method:GridRenderer.cellClick","method:GridRenderer.cellDoubleClick", "method:GridRenderer.rowClick", 
         "method:GridRenderer.rowDoubleClick","method:GridRenderer.cellHover", "method:GridRenderer.rowHover", 
         "method:GridRenderer.cellHoverHTML","method:ListGrid.cellOver", "method:ListGrid.rowOver", "method:ListGrid.cellOut", 
         "method:ListGrid.rowOut","method:ListGrid.cellHover", "method:ListGrid.rowHover", "method:ListGrid.cellHoverHTML", 
         "method:ListGrid.cellContextClick","method:ListGrid.rowContextClick", "method:ListGrid.cellMouseDown", 
         "method:ListGrid.rowMouseDown","method:ListGrid.cellMouseUp", "method:ListGrid.rowMouseUp", "method:ListGrid.cellClick", 
         "method:ListGrid.cellDoubleClick","method:ListGridField.recordClick", "method:ListGridField.recordDoubleClick", 
         "method:ListGrid.rowClick","method:ListGrid.rowDoubleClick", "method:ListGrid.recordClick", 
         "method:ListGrid.recordDoubleClick","method:ListGrid.willAcceptDrop", "method:ListGrid.getEventRow", 
         "method:ListGrid.getEventColumn","method:ListGrid.headerClick", "method:TreeGrid.getEventRow", 
         "method:TreeGrid.getDropFolder","method:TileGrid.recordClick", "method:TileGrid.recordDoubleClick", 
         "method:Canvas.containsEvent","method:Canvas.getOffsetX", "method:Canvas.getOffsetY", "method:Canvas.shouldDragScroll", 
         "attr:GridRenderer.canSelectOnRightMouse","attr:GridRenderer.canHover", "attr:GridRenderer.showHover", 
         "attr:ListGrid.arrowKeyAction","attr:Canvas.canSelectText", "attr:Canvas.canFocus", 
         "attr:Canvas.mouseStillDownInitialDelay","attr:Canvas.mouseStillDownDelay", "attr:Canvas.doubleClickDelay", 
         "attr:Canvas.noDoubleClicks"],
        description:"Handling mouse and keyboard events generated by the user",
        name:"events",
        seeAlso:[],
        type:"group"
    },
    "attr:DataBoundComponent.fields":{
        ref:"attr:DataBoundComponent.fields",
        flags:"IRW",
        description:"A DataBoundComponent manipulates records with one or more fields, and\n <code>component.fields</code> tells the DataBoundComponent which fields to present, in what\n order, and how to present each field.\n <p>\n When both <code>component.fields</code> and \n <code>${isc.DocUtils.linkForRef('attr:DataBoundComponent.dataSource','component.dataSource')}</code> are set,\n any fields in <code>component.fields</code> with the same name as a DataSource field\n inherit properties of the DataSource field.  This allows you to centralize data model\n information in the DataSource, but customize presentation of DataSource fields on a\n per-component basic.  For example, in a ListGrid, a shorter title or format for a field\n might be chosen to save space.\n <p>\n By default, only fields specified on the component are shown, in the order specified on\n the component.  The ${isc.DocUtils.linkForRef('attr:DataBoundComponent.useAllDataSourceFields')} flag can be set to show all fields\n from the DataSource, with <code>component.fields</code> acting as field-by-field\n overrides and/or additional fields.\n <p>\n If a DataBoundComponent is given a DataSource, but no <code>component.fields</code>, the\n \"default binding\" is used: fields are shown in DataSource order, according\n to the properties <code>${isc.DocUtils.linkForRef('attr:DataBoundComponent.showHiddenFields')}</code> and \n <code>${isc.DocUtils.linkForRef('attr:DataBoundComponent.showDetailFields')}</code>.",
        name:"fields",
        seeAlso:[],
        valueType:"Array of Field",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('mergedFields')}", 
         "${isc.DocUtils.linkForExampleId('validationFieldBinding')}"],
        definingClass:"interface:DataBoundComponent",
        groups:["databinding"]
    },
    "group:dragging":{
        ref:"group:dragging",
        refs:["method:ListGrid.willAcceptDrop", "method:ListGrid.transferSelectedData", 
         "method:ListGrid.getDragData","method:TreeGrid.transferSelectedData", "method:TileLayout.getDragData", 
         "method:TileGrid.getDragData","method:TileGrid.transferSelectedData", "method:Canvas.shouldDragScroll", 
         "method:DataBoundComponent.dragComplete","method:DataBoundComponent.dropComplete", "method:DataBoundComponent.getDragData", 
         "method:DataBoundComponent.transferSelectedData","attr:ListGrid.canDragRecordsOut", "attr:ListGrid.canAcceptDroppedRecords", 
         "attr:ListGrid.canReorderRecords","attr:ListGrid.canReorderFields", "attr:ListGrid.canResizeFields", 
         "attr:ListGrid.resizeFieldsInRealTime","attr:ListGrid.dragDataAction", "attr:TileLayout.dragDataAction", 
         "attr:TileGrid.dragDataAction","attr:Canvas.canDragScroll", "attr:Canvas.dragScrollDelay", 
         "attr:DataBoundComponent.addDropValues","attr:DataBoundComponent.dropValues", "attr:DataBoundComponent.dragDataAction", 
         "attr:Window.canDragReposition","attr:Window.canDragResize", "attr:Window.showResizer", "type:ReorderPosition"],
        description:null,
        name:"dragging",
        seeAlso:[],
        type:"group"
    },
    "attr:ListGrid.imageSize":{
        ref:"attr:ListGrid.imageSize",
        flags:"IRW",
        description:"Default size of thumbnails shown for fieldTypes image and imageFile.  Overrideable on a\n per-field basis via ${isc.DocUtils.linkForRef('attr:ListGridField.imageSize')} or\n ${isc.DocUtils.linkForRef('attr:ListGridField.imageWidth')}/${isc.DocUtils.linkForRef('attr:ListGridField.imageHeight')}",
        name:"imageSize",
        seeAlso:[],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["imageColumns"]
    },
    "attr:Canvas.edgeImage":{
        ref:"attr:Canvas.edgeImage",
        flags:"IR",
        description:"Base name of images for edges.  Extensions for each corner or edge piece will be added\n to this image URL, before the extension.  For example, with the default base name of\n \"edge.gif\", the top-left corner image will be \"edge_TL.gif\".\n <P>\n The full list of extensions is: \"_TL\", \"_TR\", \"_BL\", \"_BR\", \"_T\", \"_L\", \"_B\", \"_R\",\n \"_center\".",
        name:"edgeImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]edge.gif\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('edges')}"],
        definingClass:"class:Canvas",
        groups:["imageEdges"]
    },
    "method:Array.findAll":{
        ref:"method:Array.findAll",
        flags:"",
        description:" Find all objects where property == value in the object.\n <P>\n Pass an Object as the <code>propertyName</code> argument to match multiple properties.\n\n",
        name:"findAll",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"all matching Objects or null if none found", type:"Array"},
        type:"method",
        definingClass:"object:Array",
        groups:["access", "find"]
    },
    "classMethod:isc.getIconTop":{
        ref:"classMethod:isc.getIconTop",
        flags:"A",
        description:"  Returns the (offset) top-coordinate of an icon within its containing widget.\n",
        name:"getIconTop",
        seeAlso:[],
        params:[
            {description:"icon definition", optional:false, name:"icon", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"icon top position in px", type:"number"},
        type:"method",
        definingClass:"class:isc"
    },
    "method:ListGrid.getFilterEditorCriteria":{
        ref:"method:ListGrid.getFilterEditorCriteria",
        flags:"",
        description:" If ${isc.DocUtils.linkForRef('attr:ListGrid.showFilterEditor')} is true, this method will return the criteria currently\n displayed in the <code>filterEditor</code>. Note that these values may differ from the\n criteria returned by ${isc.DocUtils.linkForRef('method:ListGrid.getCriteria')} if the filter editor values have been modified\n without performing an actual filter.\n",
        name:"getFilterEditorCriteria",
        seeAlso:[],
        returns:{description:"criteria currently displayed in the filterEditor",
         type:"Criteria or AdvancedCriteria"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "classMethod:Class.logIsEnabledFor":{
        ref:"classMethod:Class.logIsEnabledFor",
        flags:"",
        description:" Check whether a message logged at the given priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsEnabledFor",
        seeAlso:[],
        params:[
            {description:"priority level", optional:false, name:"priority", defaultValue:null,
             type:"LogPriority"},
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"classMethod",
        definingClass:"class:Class"
    },
    "type:Cursor":{
        ref:"type:Cursor",
        values:[
            {description:"Use the default arrow cursor for this browser/OS.", value:"Canvas.DEFAULT"},
            {description:"Use the default cursor for this element type in this browser/OS",
             value:"Canvas.AUTO"},
            {description:"Use the wait cursor.", value:"Canvas.WAIT"},
            {description:"Use the hand cursor.", value:"Canvas.HAND"},
            {description:"Use the \"move\" (crosshairs) cursor.", value:"Canvas.MOVE"},
            {description:"Use the 'help' cursor.", value:"Canvas.HELP"},
            {description:"Use the 'text' (i-beam) cursor.", value:"Canvas.TEXT"},
            {description:"Use the 'crosshair' ( + ) cursor.", value:"Canvas.CROSSHAIR"},
            {description:"Use the column resize cursor (horizontal double-tipped arrow)",
             value:"\"col-resize\""},
            {description:"Use the row resize cursor (vertical double-tipped arrow)",
             value:"\"row-resize\""},
            {description:"Use the \"east resize\" cursor.", value:"\"e-resize\""},
            {description:"Use the \"west resize\" cursor.", value:"\"w-resize\""},
            {description:"Use the \"north resize\" cursor.", value:"\"n-resize\""},
            {description:"Use the \"south resize\" cursor.", value:"\"s-resize\""},
            {description:"Use the \"south-east resize\" cursor.", value:"\"se-resize\""},
            {description:"Use the \"north-east resize\" cursor.", value:"\"ne-resize\""},
            {description:"Use the \"north-west resize\" cursor.", value:"\"nw-resize\""},
            {description:"Use the \"south-west resize\" cursor.", value:"\"sw-resize\""},
            {description:"Use the \"not-allowed\" cursor.", value:"\"not-allowed\""}
        ],
        description:"You can use whatever cursors are valid for your deployment platforms, but keep in mind that\n visual representation may vary by browser and OS.",
        name:"Cursor",
        seeAlso:[
            {ref:"attr:Canvas.cursor", description:null}
        ],
        type:"type",
        examples:["${isc.DocUtils.linkForExampleId('cursors')}"],
        groups:["cues"]
    },
    "type:Alignment":{
        ref:"type:Alignment",
        values:[
            {description:"Center within container.", value:"Canvas.CENTER"},
            {description:"Stick to left side of container.", value:"Canvas.LEFT"},
            {description:"Stick to right side of container.", value:"Canvas.RIGHT"}
        ],
        description:null,
        name:"Alignment",
        seeAlso:[],
        type:"type",
        groups:["appearance"]
    },
    "attr:TabSet.tabBarControls":{
        ref:"attr:TabSet.tabBarControls",
        flags:"IRA",
        description:"This property determines what controls should show up after the tabBar for this tabSet.\n Standard controls can be included using the strings <code>\"tabScroller\"</code> and \n <code>\"tabPicker\"</code>. These show the standard controls to scroll to clipped tabs, \n or pick them directly from a menu, and show up only if ${isc.DocUtils.linkForRef('attr:TabSet.showTabScroller')} or\n  ${isc.DocUtils.linkForRef('attr:TabSet.showTabPicker')} is true and there is not enough space available to show all \n the tabs in the tab-bar.\n <P>\n Additional controls can be included by adding any widget to this array.  Controls will\n show up in the order in which they are specified.  For example, the following code would\n add a button in the tabBar area, while preserving the normal behavior of the tabScroller\n and tabPicker:\n <pre>\n isc.TabSet.create({\n     width:300,\n     tabs : [\n         { title: \"Tab one\" }\n     ],\n     tabBarControls : [\n         isc.ImgButton.create({\n             src:\"[SKINIMG]/actions/add.png\",\n             width:16, height:16,\n             layoutAlign:\"center\"\n         }),\n         \"tabScroller\", \"tabPicker\"\n     ]\n });\n </pre>",
        name:"tabBarControls",
        seeAlso:[],
        valueType:"Array",
        defaultValue:"[\"tabScroller\", \"tabPicker\"]",
        type:"attr",
        definingClass:"class:TabSet",
        groups:["tabBarControls"]
    },
    "method:List.sortByProperty":{
        ref:"method:List.sortByProperty",
        flags:"",
        description:" Sort a list of objects by a given property of each item.\n <P>\n The optional normalizer, if passed as a function, is called for each item in the List, and\n should return whatever value should be used for sorting, which does not have to agree with\n the property value. By passing a normalizer function you can achieve any kind of sorting\n you'd like, including sorting by multiple properties.\n <P>\n NOTE: string sort is case INsensitive by default\n\n",
        name:"sortByProperty",
        seeAlso:[],
        params:[
            {description:"name of the property to sort by", optional:false, name:"property",
             defaultValue:null, type:"string"},
            {description:"true == sort ascending, false == sort descending", optional:false, name:"up",
             defaultValue:null, type:"boolean"},
            {description:"May be specified as a function, with signature \n              <code>normalize(item, propertyName, context)</code>, where <code>item</code> is\n              a pointer to the item in the array, <code>propertyName</code> is the \n              property by which the array is being sorted, and <code>context</code> is the\n              arbitrary context passed into this method. Normalizer function should return\n              the value normalized for sorting.<br>\n              May also be specified as a ValueMap which maps property values to sortable values.",
             optional:true, name:"normalizer", defaultValue:null, type:"function or ValueMap"},
            {description:"Callers may pass an arbitrary context into the sort method, which                          will then be made available to the normalizer function",
             optional:true, name:"context", defaultValue:null, type:"any"}
        ],
        returns:{description:"the list itself", type:"List"},
        type:"method",
        definingClass:"interface:List",
        groups:["sorting"]
    },
    "method:Layout.getMemberOffset":{
        ref:"method:Layout.getMemberOffset",
        flags:"A",
        description:" An optional method that, if it exists, allows the manipulation of the calculated\n offset for members of this layout.  Returns the offset for the specified\n member.\n",
        name:"getMemberOffset",
        seeAlso:[],
        params:[
            {description:"Component to be positioned", optional:false, name:"member", defaultValue:null,
             type:"Canvas"},
            {description:"Value of the currently calculated member offset. This      may be returned verbatim or manupulated in this method.",
             optional:false, name:"defaultOffset", defaultValue:null, type:"Number"},
            {description:"alignment of the enclosing layout", optional:false, name:"alignment",
             defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:Layout",
        groups:["layoutMember"]
    },
    "attr:Canvas.animateShowAcceleration":{
        ref:"attr:Canvas.animateShowAcceleration",
        flags:"IRWA",
        description:"Default acceleration function for performing an animated show.  If unset, \n <code>this.animateAcceleration</code> will be used by default instead",
        name:"animateShowAcceleration",
        seeAlso:[],
        valueType:"AnimationAcceleration",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "method:ListGridField.sortNormalizer":{
        ref:"method:ListGridField.sortNormalizer",
        flags:"A",
        description:" Optional function to return the value that should be used when sorting this field.\n <P>\n Note that, if the dataset exceeds ${isc.DocUtils.linkForRef('attr:ListGrid.dataPageSize')} and hence paging is\n introduced, the grid relies on the server to provide sorting, and the sortNormalizer\n will no longer be called.\n\n",
        name:"sortNormalizer",
        seeAlso:[],
        params:[
            {description:"record to normalize", optional:false, name:"recordObject", defaultValue:null,
             type:"object"},
            {description:"name of the field on which sorting occurred", optional:false,
             name:"fieldName", defaultValue:null, type:"string"},
            {description:"A pointer back to the list grid displaying this field will   be available as the <code>context</code> argument. Note that you can also get a pointer\n   to the field definition object by calling <code>context.getField(fieldName)</code>",
             optional:false, name:"context", defaultValue:null, type:"ListGrid"}
        ],
        returns:{description:"normalized value for sorting", type:"any"},
        type:"method",
        examples:["${isc.DocUtils.linkForExampleId('dataTypes')}"],
        definingClass:"object:ListGridField",
        groups:["sorting"]
    },
    "method:TileLayout.setTileHeight":{
        ref:"method:TileLayout.setTileHeight",
        flags:"",
        description:" sets the height of tiles\n",
        name:"setTileHeight",
        seeAlso:[],
        params:[
            {description:"height", optional:false, name:"", defaultValue:null, type:"Integer"}
        ],
        type:"method",
        definingClass:"class:TileLayout",
        groups:["tileLayout"]
    },
    "method:ListGrid.anySelected":{
        ref:"method:ListGrid.anySelected",
        flags:"",
        description:" Whether at least one item is selected\n",
        name:"anySelected",
        seeAlso:[],
        returns:{description:"true == at least one item is selected								false == nothing at all is selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:DSRequest.oldValues":{
        ref:"attr:DSRequest.oldValues",
        flags:"IR",
        description:"For an <code>update</code> or <code>remove</code> operation, the original values from the\n record that is being updated or removed.  <code>oldValues</code> is automatically added to\n DSRequests submitted by DataBound Components.  Available on the server via\n <code>DSRequest.getOldValues()</code>.\n <P>\n The server can compare the <code>oldValues</code> to the most recent stored values in order\n to detect that the user was looking at stale values when the user submitted changes (NOTE:\n this means of detecting concurrent edit is sometimes called \"optimistic concurrency\" or\n \"long transactions\").  \n <P>\n In applications where a policy of \"last update wins\" is not appropriate when updating\n certain fields, special UI can be shown for this case.  For example, on detecting concurrent\n edit, the server may send back a special <code>dsResponse.status</code> code that the client\n application detects, offering the user a choice of proceeding with the operation,\n discarding edits, or reconciling new and old values in a special interface.",
        name:"oldValues",
        seeAlso:[],
        valueType:"Record",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "classMethod:Hover.hide":{
        ref:"classMethod:Hover.hide",
        flags:"",
        description:" Hide hover hover Canvas shown via ${isc.DocUtils.linkForRef('classMethod:Hover.show')}\n",
        name:"hide",
        seeAlso:[],
        type:"classMethod",
        definingClass:"class:Hover"
    },
    "attr:Canvas.animateMoveTime":{
        ref:"attr:Canvas.animateMoveTime",
        flags:"IRWA",
        description:"Default time for performing an animated move.  If unset, <code>this.animateTime</code>\n will be used by default instead",
        name:"animateMoveTime",
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["animation"]
    },
    "attr:PanelHeader.baseStyle":{
        ref:"attr:PanelHeader.baseStyle",
        flags:"IRW",
        description:null,
        name:"baseStyle",
        setter:["setBaseStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"sectionHeader\"",
        type:"attr",
        definingClass:"class:PanelHeader"
    },
    "attr:SelectItem.sortField":{
        ref:"attr:SelectItem.sortField",
        flags:"IR",
        description:"Specifies the field by which this item should be initially sorted.  Can be set to \n either a ${isc.DocUtils.linkForRef('attr:ListGridField.name','field name')} or the index of the field in the fields \n Array. Note that if <code>sortField</code> is initally specified as a number, it will be\n converted to a string (field name) after the item is initialized.\n <P>\n To programmatically change sort field or direction after initialization, call ${isc.DocUtils.linkForRef('sort')}.",
        name:"sortField",
        seeAlso:[],
        valueType:"String or integer",
        defaultValue:"null",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('sort')}"],
        definingClass:"class:SelectItem",
        groups:["sorting"]
    },
    "attr:Window.minimized":{
        ref:"attr:Window.minimized",
        flags:"IRW",
        description:"Is this window minimized. If true at init time, the window will be drawn minimized.\n To set this property at runtime use ${isc.DocUtils.linkForRef('method:Window.minimize')} or ${isc.DocUtils.linkForRef('method:Window.restore')}.",
        name:"minimized",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "header"]
    },
    "method:CubeGrid.setFacetValueTitleAlign":{
        ref:"method:CubeGrid.setFacetValueTitleAlign",
        flags:"",
        description:" Set the align for the title for a facet value\n\n",
        name:"setFacetValueTitleAlign",
        seeAlso:[],
        params:[
            {description:"facet to update", optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"facetValue to update", optional:false, name:"facetValueId", defaultValue:null,
             type:"id"},
            {description:"new alignment for facet value title", optional:false, name:"align",
             defaultValue:null, type:"alignEnum"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["gridLayout"]
    },
    "attr:StretchImg.itemBaseStyle":{
        ref:"attr:StretchImg.itemBaseStyle",
        flags:"IRW",
        description:"If specified this css class will be applied to the individual item images within this StretchImg.\n May be overridden by specifying item-specific base styles to each object in the\n ${isc.DocUtils.linkForRef('attr:StretchImg.items','items array')}. This base style will have standard stateful suffixes\n appended to indicate the state of this component (as described in \n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.baseStyle')}).",
        name:"itemBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:StretchImg"
    },
    "attr:ListGridField.imageSize":{
        ref:"attr:ListGridField.imageSize",
        flags:"IRW",
        description:"Size of images shown for fieldTypes image and imageFile in this field.\n This setting overrides the global ListGrid default ${isc.DocUtils.linkForRef('attr:ListGrid.imageSize')}.\n <P>\n If set to a String, assumed to be a property on each record that specifies the image\n height.  For example, if <code>field.imageSize</code> is \"logoSize\",\n <code>record.logoSize</code> will control the size of the image.",
        name:"imageSize",
        seeAlso:[
            {ref:"attr:ListGridField.imageWidth", description:null},
            {ref:"attr:ListGridField.imageHeight", description:null}
        ],
        valueType:"number",
        defaultValue:"16",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["imageColumns"]
    },
    "class:String":{
        methods:["method:String.contains", "method:String.startsWith", "method:String.endsWith"],
        ref:"class:String",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/System"],
        attrs:[],
        description:"Generic extensions to JavaScript Strings.  You can call these on any String.",
        name:"String",
        seeAlso:[],
        type:"class"
    },
    "attr:SectionStack.itemStartIndent":{
        ref:"attr:SectionStack.itemStartIndent",
        flags:"IRW",
        description:"Size, in pixels, of indentation of all member items relative to the start of\n the alignment axis. For instance, for left-aligned members, \n itemStartIndent specifies indentation for every item from the left side of the\n section stack. Overrides ${isc.DocUtils.linkForRef('attr:SectionStack.itemIndent')}.",
        name:"itemStartIndent",
        seeAlso:[],
        valueType:"Number",
        defaultValue:"undefined",
        type:"attr",
        definingClass:"class:SectionStack",
        groups:["layoutMember"]
    },
    "attr:TabSet.tabProperties":{
        ref:"attr:TabSet.tabProperties",
        flags:"IR",
        description:"Properties to apply to all Tabs created by this TabSet.",
        name:"tabProperties",
        setter:["setTabProperties"],
        seeAlso:[],
        valueType:"Tab Properties",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:FormItem.showPickerIcon":{
        ref:"attr:FormItem.showPickerIcon",
        flags:"IRW",
        description:"Should we show a special 'picker' ${isc.DocUtils.linkForRef('object:FormItemIcon','icon')} for this form item. Picker\n icons are customizeable via ${isc.DocUtils.linkForRef('attr:FormItem.pickerIconProperties')}. By default they will\n be rendered inside the Form Item's \"control box\" area, and will call\n ${isc.DocUtils.linkForRef('method:FormItem.showPicker')} when clicked.",
        name:"showPickerIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "method:ColumnTree.deselectRecords":{
        ref:"method:ColumnTree.deselectRecords",
        flags:"",
        description:"\n Deselect a list of ${isc.DocUtils.linkForRef('object:Record')}s passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecords(records, false)</code>\n\n",
        name:"deselectRecords",
        seeAlso:[],
        params:[
            {description:"records (or row numbers) to deselect", optional:false, name:"records",
             defaultValue:null, type:"Array of Record | numbers"},
            {description:"Column number", optional:true, name:"colNum", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:ColumnTree",
        groups:["selection"]
    },
    "attr:FilterBuilder.showRemoveButton":{
        ref:"attr:FilterBuilder.showRemoveButton",
        flags:"IR",
        description:"If set, a button will be shown for each clause allowing it to be removed.",
        name:"showRemoveButton",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:FilterBuilder"
    },
    "attr:FormulaBuilder.instructions":{
        ref:"attr:FormulaBuilder.instructions",
        flags:"IR",
        description:"Label displaying the instruction text above the fieldKey grid.",
        name:"instructions",
        seeAlso:[],
        valueType:"AutoChild Label",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormulaBuilder"
    },
    "attr:Scrollbar.thumbOverlap":{
        ref:"attr:Scrollbar.thumbOverlap",
        flags:"IRA",
        description:"Number of pixels the thumb is allowed to overlap the buttons at each end of the track.\n Default prevents doubling of 1px borders.  Set higher to allow media that shows curved\n joins between the track button and ScrollThumb.",
        name:"thumbOverlap",
        seeAlso:[],
        valueType:"number",
        defaultValue:"1",
        type:"attr",
        definingClass:"class:Scrollbar",
        groups:["thumb"]
    },
    "attr:DataSource.serviceNamespace":{
        flags:"IR",
        type:"attr",
        ref:"attr:DataSource.serviceNamespace",
        description:"For an XML DataSource, URN of the WebService to use to invoke operations.  This URN\n comes from the \"targetNamespace\" attribute of the &lt;wsdl:definitions&gt; element in a\n WSDL (Web Service Description Language) document, and serves as the unique identifier of\n the service.\n <P>\n Having loaded a WebService using ${isc.DocUtils.linkForRef('classMethod:XMLTools.loadWSDL')}, setting\n <code>serviceNamespace</code> combined with specifying \n ${isc.DocUtils.linkForRef('class:OperationBinding','operationBindings')}\n that set ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation')} will cause a DataSource to invoke web\n service operations to fulfill DataSource requests (${isc.DocUtils.linkForRef('class:DSRequest','DSRequests')}).\n <P>\n Setting <code>serviceNamespace</code> also defaults\n ${isc.DocUtils.linkForRef('attr:DataSource.dataURL','dataURL')} to the service's location,\n ${isc.DocUtils.linkForRef('attr:DataSource.dataFormat','dataFormat')} to \"xml\" and\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataProtocol','dataProtocol')} to \"soap\".",
        name:"serviceNamespace",
        valueType:"URN",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('wsdlBinding')}"],
        serverDS:"allowed",
        defaultValue:"null",
        groups:["wsdlBinding", "clientDataIntegration"],
        definingClass:"class:DataSource"
    },
    "classMethod:FileLoader.loadModules":{
        ref:"classMethod:FileLoader.loadModules",
        flags:"",
        description:"\n Loads the specified SmartClient modules into the context of the current page.\n\n",
        name:"loadModules",
        seeAlso:[],
        params:[
            {description:"List of SmartClient modules to load.  This can either be a                                  string with comma separated module names or an Array of module\n                                  names.",
             optional:false, name:"modules", defaultValue:null, type:"String or Array"},
            {description:"Optional code to execute when the last of the specified                                      modules has completed loading.",
             optional:true, name:"onload", defaultValue:null, type:"String or Function"}
        ],
        type:"classMethod",
        definingClass:"class:FileLoader"
    },
    "method:ResultSet.filterLocalData":{
        ref:"method:ResultSet.filterLocalData",
        flags:"A",
        description:" Derive the current filtered set of data from the cache of all matching rows.\n <P>\n This method is automatically called by ${isc.DocUtils.linkForRef('method:ResultSet.setCriteria')} when criteria have actually\n changed, as well as in various other situations.  You could only need to call this method\n directly if:\n <ul>\n <li> you know that client-side filtering is enabled (${isc.DocUtils.linkForRef('attr:ResultSet.useClientFiltering')}:true) and\n active ${isc.DocUtils.linkForRef('method:ResultSet.allMatchingRowsCached')}.\n <li> you have directly, programmatically modified data within the ResultSet such that it no\n longer matches the filter criteria\n <li> you want your modified records to disappear from the list of visible records (that is,\n those accesible via ${isc.DocUtils.linkForRef('method:ResultSet.get')})\n </ul>\n \n",
        name:"filterLocalData",
        seeAlso:[],
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:ListGridField.partialSummary":{
        ref:"attr:ListGridField.partialSummary",
        flags:"IR",
        description:"Only applies to fields of type <code>\"summary\"</code>.\n This attribute is set on a summary field, when calculating the summary value from\n some record, the summary function will only be passed the fields before this summary field.\n This may be useful for displaying running totals across a record.\n <P>\n Note that this feature would typically be used with\n ${isc.DocUtils.linkForRef('attr:ListGrid.canReorderFields','canReorderFields:false')}",
        name:"partialSummary",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:ListGridField.iconHeight":{
        ref:"attr:ListGridField.iconHeight",
        flags:"IR",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGridField.icon')} is specified, this property can be used to specify the \n height of the icon to be displayed in the ListGrid header button. \n (See ${isc.DocUtils.linkForRef('attr:StatefulCanvas.iconHeight')})<br>\n If this field is editable, and ${isc.DocUtils.linkForRef('attr:ListGridField.editorIconHeight')} is unset, this \n property will be passed onto the editors for this field as ${isc.DocUtils.linkForRef('attr:FormItem.iconWidth')}, \n which will effect the default size for ${isc.DocUtils.linkForRef('attr:ListGridField.icons','icons')} displayed \n in the editor.",
        name:"iconHeight",
        seeAlso:[
            {ref:"attr:ListGridField.icon", description:null},
            {ref:"attr:ListGridField.icons", description:null}
        ],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField"
    },
    "attr:ListGrid.expansionDetailField":{
        ref:"attr:ListGrid.expansionDetailField",
        flags:"RA",
        description:"Automatically generated ${isc.DocUtils.linkForRef('class:HTMLFlow')} for displaying the contents of \n ${isc.DocUtils.linkForRef('attr:ListGrid.detailField','a specified field')} in a record's expanded section when\n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is <code>detailField</code>.\n <P>\n This component is an ${isc.DocUtils.linkForRef('type:AutoChild')} and as such may be customized via \n <code>listGrid.expansionDetailFieldProperties</code> and \n <code>listGrid.expansionDetailFieldDefaults</code>.\n <P>\n Note, however, that this is a multi-instance component (potentially one per record), \n so it is created using ${isc.DocUtils.linkForRef('method:Class.createAutoChild','createAutoChild()')} not \n ${isc.DocUtils.linkForRef('method:Class.addAutoChild','addAutoChild()')}, and no default single instance is created by \n name on the grid.",
        name:"expansionDetailField",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["expansionField"]
    },
    "method:ListGrid.selectAllRecords":{
        ref:"method:ListGrid.selectAllRecords",
        flags:"",
        description:" Select all records\n\n",
        name:"selectAllRecords",
        seeAlso:[],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "attr:ListGrid.showHeader":{
        ref:"attr:ListGrid.showHeader",
        flags:"IRW",
        description:"Should we show the header for this ListGrid?",
        name:"showHeader",
        setter:["setShowHeader"],
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "method:Array.getItems":{
        ref:"method:Array.getItems",
        flags:"",
        description:" Return the items at a list of specified positions.\n\n",
        name:"getItems",
        seeAlso:[],
        params:[
            {description:"array of positions", optional:false, name:"itemList", defaultValue:null,
             type:"List of Number"}
        ],
        returns:{description:"subset of the array, in the same order as itemList", type:"array"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "method:FormItem.formatValue":{
        ref:"method:FormItem.formatValue",
        flags:"",
        description:" An optional ${isc.DocUtils.linkForRef('group:stringMethods','stringMethod')} which if provided, is evaluated to\n get the display value to show for this form items underlying data value.\n <P>\n This provides a way to perform a more complex data to display value manipulation \n than a simple ${isc.DocUtils.linkForRef('attr:FormItem.valueMap')}.\n Note that this method will not be applied to values displayed in a freeform text entry\n field (such as a ${isc.DocUtils.linkForRef('class:TextItem')} or ${isc.DocUtils.linkForRef('class:TextAreaItem')}), where an equivalent parser method\n would be required to convert a user-entered value back into a data value. \n Use ${isc.DocUtils.linkForRef('method:FormItem.formatEditorValue')} and ${isc.DocUtils.linkForRef('method:FormItem.parseEditorValue')} instead for\n these cases.\n \n",
        name:"formatValue",
        seeAlso:[],
        params:[
            {description:"Underlying data value to format. May be null.", optional:false, name:"value",
             defaultValue:null, type:"any"},
            {description:"The record currently being edited by this form.      Essentially the form's current values object.",
             optional:false, name:"record", defaultValue:null, type:"ListGridRecord"},
            {description:"pointer to the DynamicForm", optional:false, name:"form", defaultValue:null,
             type:"DynamicForm"},
            {description:"pointer to the FormItem", optional:false, name:"item", defaultValue:null,
             type:"FormItem"}
        ],
        returns:{description:"Display value to show.", type:"string"},
        type:"method",
        definingClass:"class:FormItem"
    },
    "group:smartArchitecture":{
        ref:"group:smartArchitecture",
        title:"SmartClient Architecture",
        treeLocation:["Concepts"],
        refs:[],
        description:"SmartClient can add interactivity and performance benefits to any web application with a\n variety of integration approaches.  This topic discusses the optimal architecture for a\n SmartClient application, which can be adopted in whole or in part.\n <p>\n In a typical HTML-based web application, every time a new view is shown to a user, a round\n trip to the server is required to retrieve new presentation information, such as a search\n screen.  However in an ISC-based application, showing a new view can be accomplished by\n simply hiding some components and showing others.\n <p>\n Because ISC components are expressed in a concise declarative form, and because ISC\n components have essentially no runtime performance impact until used, dozens of application\n views can be downloaded to the browser using the same bandwidth that would have been\n required to render just the initial view in plain HTML.  \n <p> \n This architectural pattern of \"preloading views\" has tremendous benefits.  View transitions\n which do not require new data from the server can be performed near-instantaneously and\n without server involvement, boosting both interactivity and scalability.\n <p>\n Showing a dialog containing a \"wizard\" is a straightforward example of showing a \"preloaded\n view\".  For example:\n <pre>\n    function showNewUserWizard() {\n        Window.create({\n            items:[\n                DynamicForm.create({ ... })\n            ]\n        });\n    }\n    Button.create({\n        title:\"New User..\",\n        click:\"showNewUserWizard()\"\n    });\n </pre>\n In this example, none of the components involved in a potentially multi-pane wizard are \n created until they are needed.  Showing the wizard has near-instantaneous response and\n causes no server load.\n <p>\n However, let's say that the first pane of the wizard is going to incorporate some dynamic\n user-specific data, such as the current user's name.  To load the username, we'll use an RPC\n operation targetting a .jsp called \"getUserName.jsp\" and show the wizard when it completes\n (see ${isc.DocUtils.linkForRef('class:RPCManager')} for information on RPCs and how to construct a .jsp that can\n send an RPC response).\n <pre>\n    function showNewUserWizard() {\n        RPCManager.sendRequest({\n            actionURL:\"getUserName.jsp\",\n            callback:\"doShow(rpcResponse)\"\n        });\n    }\n    function doShow(rpcResponse) {\n        Window.create({\n            items:[\n                Canvas.create({contents:\"Hello, \" + rpcResponse.userName}),\n                DynamicForm.create({ ... })\n            ]\n        });\n    }\n    Button.create({\n        title:\"New User..\",\n        click:\"showNewUserWizard()\"\n    });\n </pre>\n In this example, we've simply incorporated a user name into the first pane of a wizard.\n However, this pattern allows us to arbitrarily change user interactions based on data from\n the server.  For example, the RPCResponse might have contained a flag indicating that the\n wizard should skip the first two steps, or an arbitrary warning message for the user, or\n even JavaScript code to be evaluated on the client.\n <P>\n This architecture has several key advantages:\n <dl>\n\n <dt><b>Performance: Cacheable UI</b></dt>\n <dd>\n A dynamic, data-driven UI can be expressed completely in <i>cacheable</i> JavaScript.\n This is in contrast to any architecture based on server-side HTML generation, where static\n parts of the presentation are mixed in with dynamic data, preventing cacheability so that\n bandwidth and server time are wasted repeatedly delivering the same static presentation\n data.\n <br>\n Even generated JavaScript is cacheable.  For example, a SmartClient View expressed in XML\n and contained within a JSP is still a separately cacheable resource when loaded via a\n &lt;SCRIPT SRC&gt; tag and advertised as a cacheable resource via HTTP headers, because it\n is ultimately delivered to the browser as simple JavaScript.  Hence standard \n ${isc.DocUtils.linkForRef('group:i18n','internationalization')} techniques such as using JSTL tags in a JSP remain\n applicable.\n <br>\n The SmartClient Architecture even allows you to capture all the gradations of cacheability\n from completely static (changes once per application rollout) to completely dynamic\n (timestamp).  In the example above, the user name wouldn't actually change for the lifetime\n of the page, so could be loaded once only.\n <br>&nbsp;</dd>\n\n <dt><b>Performance: Minimal Server State</b></dt>\n <dd>\n Any architecture that relies on component descriptions being generated by the server\n must track a great deal of state, which, in the SmartClient Architecture, is either\n completely eliminated or greatly reduced.\n <br>&nbsp;</dd>\n\n <dt><b>True Presentation / Business Logic separation</b></dt>\n <dd>\n The RPCResponse object represents the client's exact, minimal needs for server data.\n This is much easier to understand and to audit than a slew of .jsp files which access and\n update miscellaneous state.  It is also far easier to spot reusable patterns of data access,\n which in server-side HTML generation systems often end up as duplicated code.\n <br>&nbsp;</dd>\n\n <dt><b>Parallel Development and Testability</b></dt>\n <dd>\n Using the SmartClient architecture allows you to build a complete, working application\n that can run without a server, based on sample data.  In the example above, it would be \n straightforward to create a testing mode that returned a faked RPC response consisting of\n simply <code>{ userName : \"Bob\" }</code>.\n <br>\n This allows better parallel development by enabling the client side of the system to be\n tested in isolation, and creates clearer communication between client and server-side\n developers since creation of test data tends to develop into data requirements\n specifications.\n <br>\n For more info on creating applications that support client-only testing, see\n ${isc.DocUtils.linkForRef('group:clientOnlyDataSources','Client Only DataSources')}.\n <br>&nbsp;</dd>\n </dl>\n <br>\n <h3>Refinements</h3>\n <br>\n <b>Creating vs Showing a View</b>\n <br>\n Many views will be shown to the user repeatedly, for example, the user may repeatedly switch\n back and forth between two panes of a TabSet.  In that usage it makes sense to make a\n distinction between <i>creating</i> a view and <i>showing</i> an existing view.  When\n showing an existing view, the same components and/or data may be able to be reused.\n <br>\n In the following variant on the original example, we only create the Window object and\n do the RPC to retrieve the user name the first time <code>showNewUserWizard()</code> is\n called.  Subsequently we reuse the existing window, and we assume the user name has not\n changed, so we need not do the RPC again. (<i>Note: \"New User\" button omitted for brevity\n from here on</i>)\n <pre>\n    function showNewUserWizard() {\n        if (!window.myWindow) {\n            Window.create({\n                ID:\"myWindow\",\n                autoDraw:false,\n                items:[\n                    Canvas.create({ ID: \"welcomeCanvas\" }),\n                    DynamicForm.create({ ... })\n                ]\n            });\n            RPCManager.sendRequest({\n                actionURL:\"getUserName.jsp\",\n                callback:\"doShow(rpcResponse)\"\n            });\n        } else {\n            myWindow.show();\n        }\n    }\n    function doShow(rpcResponse) {\n        welcomeCanvas.setContents(\"Hello, \" + rpcResponse.userName);\n        myWindow.show();\n    }\n </pre>\n \n <b>Batching Operations</b>\n <br>\n A view may incorporate multiple components, each of which requires data.  In the following\n example, a DataBound ListGrid has been incorporated into the wizard, and we'd like to fetch\n the user's name and the beginning dataset for the grid in the same batch.  We use \n ${isc.DocUtils.linkForRef('classMethod:RPCManager.startQueue')} to do so.\n <pre>\n    function showNewUserWizard() {\n        if (!window.myWindow) {\n            Window.create({\n                ID:\"myWindow\",\n                autoDraw:false,\n                items:[\n                    Canvas.create({ ID: \"welcomeCanvas\" }),\n                    <b>ListGrid.create({ \n                        ID: \"myGrid\",\n                        dataSource:\"myDataSource\"\n                    }),</b>\n                    DynamicForm.create({ ... })\n                ]\n            });\n            <b>RPCManager.startQueue();\n            myGrid.fetchData();</b>\n            RPCManager.sendRequest({\n                actionURL:\"getUserName.jsp\",\n                callback:\"doShow(rpcResponse)\"\n            });\n            <b>RPCManager.sendQueue();</b>\n        } else {\n            myWindow.show();\n        }\n    }\n    function doShow(rpcResponse) {\n        welcomeCanvas.setContents(\"Hello, \" + rpcResponse.userName);\n        myWindow.show();\n    }\n </pre>\n\n <b>Segmenting very large Applications</b>\n <P>\n If an application has many hundreds of views, but only a handful of views are used by a\n given user in a typical session, for the fastest loading performance you should consider\n loading only the most commonly used views initially then loading further views on demand.\n <P>\n You can use ${isc.DocUtils.linkForRef('classMethod:FileLoader.loadJSFiles')} to load a set of JavaScript files\n compromising an application module that defines a set of related views.  The loaded\n JavaScript files may define new component classes and new DataSources in addition to\n defining new views and their associated logic.",
        name:"smartArchitecture",
        seeAlso:[],
        type:"group"
    },
    "method:Array.isEmpty":{
        ref:"method:Array.isEmpty",
        flags:"",
        description:" Return whether or not this array is empty\n\n",
        name:"isEmpty",
        seeAlso:[],
        returns:{description:"true == this array is empty, false == some items in the array", type:"boolean"},
        type:"method",
        definingClass:"object:Array",
        groups:["access"]
    },
    "attr:Menu.items":{
        flags:"IRW",
        getter:["getItems"],
        type:"attr",
        ref:"attr:Menu.items",
        description:"Synonym for ${isc.DocUtils.linkForRef('attr:Menu.data')}",
        name:"items",
        setter:["setItems"],
        valueType:"Array of MenuItem",
        seeAlso:[],
        defaultValue:"null",
        groups:["data"],
        definingClass:"class:Menu"
    },
    "method:List.duplicate":{
        ref:"method:List.duplicate",
        flags:"A",
        description:" Return an Array that is a shallow copy of the list, that is, containing the same items.\n\n",
        name:"duplicate",
        seeAlso:[],
        returns:{description:"new array, pointing to the same items", type:"Array"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "method:ComboBoxItem.getValueFieldName":{
        ref:"method:ComboBoxItem.getValueFieldName",
        flags:"",
        description:" Getter method to retrieve the ${isc.DocUtils.linkForRef('attr:FormItem.valueField')} for this item.\n If unset, default behavior will return the ${isc.DocUtils.linkForRef('attr:FormItem.name')} of this field.\n",
        name:"getValueFieldName",
        seeAlso:[],
        returns:{description:"fieldName to use a \"value field\" in records from this items               ${isc.DocUtils.linkForRef('attr:FormItem.optionDataSource')}",
         type:"string"},
        type:"method",
        definingClass:"class:ComboBoxItem",
        groups:["display_values"]
    },
    "attr:Button.showFocusedIcon":{
        ref:"attr:Button.showFocusedIcon",
        flags:"IR",
        description:"If using an icon for this button, whether to switch the icon image when the button\n recieves focus.\n <P>\n If ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is true, the <code>\"Over\"</code> icon will be\n displayed when the canvas has focus, otherwise a seperate <code>\"Focused\"</code> icon\n will be displayed",
        name:"showFocusedIcon",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Button",
        groups:["buttonIcon"]
    },
    "attr:FormItem.pickerIconHeight":{
        ref:"attr:FormItem.pickerIconHeight",
        flags:"IRWA",
        description:"If ${isc.DocUtils.linkForRef('attr:FormItem.showPickerIcon')} is true for this item, this property governs the\n size of the picker icon. If unset picker icon will be sized as a square to fit in the\n avaliable height for the icon.",
        name:"pickerIconHeight",
        seeAlso:[],
        valueType:"int",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:FormItem"
    },
    "attr:ListGrid.sortFieldDescendingText":{
        ref:"attr:ListGrid.sortFieldDescendingText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, this\n attribute will be shown as the menu item title to sort a field in descending order.",
        name:"sortFieldDescendingText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Sort Descending\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:Window.showResizer":{
        ref:"attr:Window.showResizer",
        flags:"IRW",
        description:"If true, show a button in the lower right corner that allows users to resize the Window.\n Note that the resizer will only be displayed if the footer is showing for the window\n (${isc.DocUtils.linkForRef('attr:Window.showFooter')}) and ${isc.DocUtils.linkForRef('attr:Window.canDragResize')} is true.",
        name:"showResizer",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "dragging"]
    },
    "method:DataSource.getFieldOperatorMap":{
        ref:"method:DataSource.getFieldOperatorMap",
        flags:"",
        description:" Get the list of ${isc.DocUtils.linkForRef('object:Operator')}s available for this field, as a ${isc.DocUtils.linkForRef('type:ValueMap')} from\n ${isc.DocUtils.linkForRef('type:OperatorId')} to the ${isc.DocUtils.linkForRef('attr:Operator.title')} specified for the ${isc.DocUtils.linkForRef('object:Operator')}, or\n the corresponding property in ${isc.DocUtils.linkForRef('class:Operators')} if ${isc.DocUtils.linkForRef('attr:Operator.titleProperty')} is set.\n <P>\n This valueMap is suitable for use in a UI for building queries, similar to the\n ${isc.DocUtils.linkForRef('class:FilterBuilder')}, and optionally omits operators marked ${isc.DocUtils.linkForRef('attr:Operator.hidden')}:true.\n <p>\n It is also possible to have this function return only operators of a given \n ${isc.DocUtils.linkForRef('type:OperatorValueType')}, or everything except operators of that type. This is useful, \n for example, if you want to return all the logical operators (like \"and\"), or everything \n except the logical operators.\n\n",
        name:"getFieldOperatorMap",
        seeAlso:[
            {ref:"method:DataSource.getTypeOperatorMap", description:null}
        ],
        params:[
            {description:"Field (or field name) to obtain operator map for.", optional:false,
             name:"field", defaultValue:null, type:"string or DataSourceField"},
            {description:"whether to include Operators marked hidden:true", optional:true,
             name:"includeHidden", defaultValue:null, type:"boolean"},
            {description:"If passed, returns only operators of this ${isc.DocUtils.linkForRef('type:OperatorValueType')}",
             optional:true, name:"valueType", defaultValue:null, type:"OperatorValueType"},
            {description:"If set, reverses the meaning of the <code>valueType</code>         parameter, so operators of that ${isc.DocUtils.linkForRef('type:OperatorValueType')} are the only ones omitted",
             optional:true, name:"omitValueType", defaultValue:null, type:"boolean"}
        ],
        returns:{description:"mapping from ${isc.DocUtils.linkForRef('type:OperatorId')} to title, as described above",
         type:"ValueMap"},
        type:"method",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "group:serialization":{
        ref:"group:serialization",
        refs:["classMethod:isc.clone"],
        name:"serialization",
        seeAlso:[],
        type:"group"
    },
    "attr:DateChooser.firstDayOfWeek":{
        ref:"attr:DateChooser.firstDayOfWeek",
        flags:"IR",
        description:"Day of the week to show in the first column.  0=Sunday, 1=Monday, ..., 6=Saturday.",
        name:"firstDayOfWeek",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"0",
        type:"attr",
        definingClass:"class:DateChooser",
        groups:["appearance"]
    },
    "method:Class.setProperty":{
        ref:"method:Class.setProperty",
        flags:"",
        description:" Set a property on this object, calling the setter method if it exists.\n <p>\n Whenever you set a property on an ISC component, you should call either the specific setter\n for that property, or <code>setProperty()/setProperties()</code> if it doesn't have one.\n This future-proofs your code against the later addition of required setters.  \n\n",
        name:"setProperty",
        seeAlso:[
            {ref:"method:Class.setProperties", description:null}
        ],
        params:[
            {description:"name of the property to set", optional:false, name:"propertyName",
             defaultValue:null, type:"String"},
            {description:"new value for the property", optional:false, name:"newValue",
             defaultValue:null, type:"any"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "method:CubeGrid.addColumnFacet":{
        ref:"method:CubeGrid.addColumnFacet",
        flags:"",
        description:" Add a column facet to the view at index \"index\".  Handles the facet already being in the view\n (does a pivot).<br><br>\n\n The facet being added should currently have a fixed facet value (unless it's already part of the\n view), which will be removed from cubeGrid.fixedFacetValues.\n\n <br><i>methodType</i>  action\n",
        name:"addColumnFacet",
        seeAlso:[
            {ref:"method:CubeGrid.removeFacet", description:null},
            {ref:"attr:CubeGrid.fixedFacetValues", description:null}
        ],
        params:[
            {description:"facetId to add.  Definition must have been provided at init time.",
             optional:false, name:"facetId", defaultValue:null, type:"id"},
            {description:"index to add the facet at.  0 = outermost (default innermost)",
             optional:true, name:"index", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "attr:Tree.isFolderProperty":{
        ref:"attr:Tree.isFolderProperty",
        flags:"IRW",
        description:"Name of property that defines whether a node is a folder.  By default this is set to\n ${isc.DocUtils.linkForRef('attr:TreeNode.isFolder')}.",
        name:"isFolderProperty",
        seeAlso:[
            {ref:"attr:TreeNode.isFolder", description:null}
        ],
        valueType:"String",
        defaultValue:"\"isFolder\"",
        type:"attr",
        definingClass:"class:Tree"
    },
    "attr:DetailViewer.emptyCellValue":{
        ref:"attr:DetailViewer.emptyCellValue",
        flags:"IRWA",
        description:"Text to show for an empty cell",
        name:"emptyCellValue",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"&nbsp;\"",
        type:"attr",
        definingClass:"class:DetailViewer",
        groups:["appearance"]
    },
    "attr:CubeGrid.rowFacets":{
        ref:"attr:CubeGrid.rowFacets",
        flags:"IRW",
        description:"The list of facets that will appear to the left of the body",
        name:"rowFacets",
        seeAlso:[],
        valueType:"array",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["facetLayout"]
    },
    "type:Criteria":{
        ref:"type:Criteria",
        treeLocation:["Client Reference/Data Binding"],
        description:"Criteria for selecting only a matching set of records from a DataSource.  Criteria can\n be applied on the client and server.  Unless configured otherwise, criteria will generally\n be applied client-side by ${isc.DocUtils.linkForRef('class:ResultSet')}s via ${isc.DocUtils.linkForRef('method:ResultSet.applyFilter')}.  \n <P>\n Client- and server-side systems built into SmartClient understand two criteria formats by\n default: simple key-value pairs (Criteria) or the ${isc.DocUtils.linkForRef('object:AdvancedCriteria')} format.\n <P>\n Simple key-value Criteria are represented via a JavaScript Object where each property\n specifies the name and required value for a field.  Multiple legal values for a field can be\n provided as an Array.  For example:\n <pre>\n var criteria = {\n    field1 : \"value1\",\n    field2 : [\"value2\", \"value3\"]\n }\n </pre>\n Would select all records where field1 has value \"value1\" and where field2 has <i>either</i>\n \"value2\" or \"value3\".\n <P>\n When writing custom client and server-side filtering logic, criteria must be a JavaScript\n Object but the properties of that Object can contain whatever data you want.  When sent to\n the SmartClient server, the Java representation of the criteria is described\n ${isc.DocUtils.linkForRef('attr:RPCRequest.data','here')}.  When sent to other servers, the\n ${isc.DocUtils.linkForRef('type:DSProtocol','operationBinding.dataProtocol')} affects the format.",
        name:"Criteria",
        seeAlso:[
            {ref:"type:CriteriaPolicy", description:null}
        ],
        type:"type"
    },
    "method:Button.setWrap":{
        ref:"method:Button.setWrap",
        flags:"",
        description:" Set whether the title of this button should be allowed to wrap if too long for the button's\n specified width.\n\n",
        name:"setWrap",
        seeAlso:[],
        params:[
            {description:"whether to wrap the title", optional:false, name:"newWrap", defaultValue:null,
             type:"boolean"}
        ],
        type:"method",
        definingClass:"class:Button"
    },
    "attr:CubeGrid.facetValueHoverStyle":{
        ref:"attr:CubeGrid.facetValueHoverStyle",
        flags:"IRWA",
        description:"Allows the developer to override the css class applied to  hover tips shown for\n facet values.  If unspecified, and <code>this.hoverStyle</code> is not null, that \n css class will be applied to facet value hovers instead.",
        name:"facetValueHoverStyle",
        seeAlso:[
            {ref:"attr:Canvas.hoverStyle", description:null}
        ],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["hoverTips"]
    },
    "attr:ButtonItem.buttonProperties":{
        ref:"attr:ButtonItem.buttonProperties",
        flags:"IRA",
        description:"Custom Properties to apply to our button item.",
        name:"buttonProperties",
        seeAlso:[],
        valueType:"Object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ButtonItem"
    },
    "classAttr:Dialog.Prompt":{
        ref:"classAttr:Dialog.Prompt",
        flags:"A",
        description:"The \"Prompt\" object on the dialog class is a singleton Dialog instance.\n  The Prompt is used to show text to the user in a modal fashion - it will expand to show \n  all the text that you put into it.\n  By default this Dialog has no end-user controls and is expected to be programmatically\n  dismissed.<br>\n  Common use-case: During server-interactions, the Prompt will be used to display a suitable \n  wait message, and suppress user input.<br><br>\n\n Notes:<br>\n  Because this is a singleton object, properties set on the Prompt directly will persist each\n  time it is shown.<br>\n  Developers should use the <code>showPrompt()</code> and <code>clearPrompt()</code> methods\n  to show and hide the prompt rather than manipulating the prompt directly.",
        name:"Prompt",
        seeAlso:[
            {ref:"classMethod:isc.showPrompt", description:null},
            {ref:"classMethod:isc.clearPrompt", description:null}
        ],
        valueType:"Dialog Properties",
        defaultValue:"dialog instance properties",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["Prompting"]
    },
    "attr:GridRenderer.drawAheadRatio":{
        ref:"attr:GridRenderer.drawAheadRatio",
        flags:"IRWA",
        description:"How far should we render rows ahead of the currently visible area?  This is expressed as a\n ratio from viewport size to rendered area size.<br><br>\n \n Tweaking drawAheadRatio allows you to make tradeoffs between continuous scrolling speed vs\n initial render time and render time when scrolling by large amounts.<br><br>\n\n NOTE: Only applies when showAllRows is false.",
        name:"drawAheadRatio",
        seeAlso:[],
        valueType:"float",
        defaultValue:"1.3",
        type:"attr",
        definingClass:"class:GridRenderer",
        groups:["performance"]
    },
    "attr:Calendar.nextButtonHoverText":{
        ref:"attr:Calendar.nextButtonHoverText",
        flags:"IR",
        description:"The text to be displayed when a user hovers over the ${isc.DocUtils.linkForRef('attr:Calendar.nextButton','next')} \n toolbar button",
        name:"nextButtonHoverText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Next\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.enterKeyEditAction":{
        ref:"attr:ListGridField.enterKeyEditAction",
        flags:"IRW",
        description:"What to do when a user hits enter while editing this field?<br>\n Overrides the <code>enterKeyEditAction</code> as specified at the listGrid level while \n focus is in this field.",
        name:"enterKeyEditAction",
        seeAlso:[],
        valueType:"EnterKeyEditAction",
        defaultValue:"\"done\"",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["editing"]
    },
    "attr:StretchImgButton.showTitle":{
        ref:"attr:StretchImgButton.showTitle",
        flags:"IRW",
        description:"Determines whether any specified ${isc.DocUtils.linkForRef('method:StatefulCanvas.getTitle','title')} will be \n displayed for this component.<br>\n Applies to Image-based components only, where the title will be rendered out in a label\n floating over the component",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:StretchImgButton"
    },
    "attr:DynamicForm.initialCriteria":{
        ref:"attr:DynamicForm.initialCriteria",
        flags:"IR",
        description:"Criteria to be used when ${isc.DocUtils.linkForRef('attr:DynamicForm.autoFetchData')} is set.",
        name:"initialCriteria",
        seeAlso:[],
        valueType:"Criteria",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm"
    },
    "method:ImgButton.isSelected":{
        ref:"method:ImgButton.isSelected",
        flags:"",
        description:" Find out if this object is selected\n",
        name:"isSelected",
        seeAlso:[],
        returns:{description:null, type:"boolean"},
        type:"method",
        definingClass:"class:ImgButton",
        groups:["state"]
    },
    "class:RadioGroupItem":{
        methods:["method:RadioGroupItem.setValueDisabled"],
        exampleConfig:"RadioGroupItem.xml",
        ref:"class:RadioGroupItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:RadioGroupItem.vertical", "attr:RadioGroupItem.textBoxStyle", 
         "attr:RadioGroupItem.wrap","attr:RadioGroupItem.itemProperties"],
        description:"FormItem that shows a set of mutually exclusive options as a group of radio buttons.",
        name:"RadioGroupItem",
        seeAlso:[],
        type:"class"
    },
    "attr:Label.wrap":{
        ref:"attr:Label.wrap",
        flags:"IRW",
        description:"If false, the label text will not be wrapped to the next line.",
        name:"wrap",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Label",
        groups:["sizing"]
    },
    "attr:RPCRequest.data":{
        ref:"attr:RPCRequest.data",
        flags:"IRW",
        description:"This attribute specifies the payload of the RPCRequest.  When using the SmartClient server,\n any JavaScript simple type or arbitrarily nested set of Objects and Arrays can be sent\n to server and automatically translated to Java Objects.  Here are the \n mapping of JavaScript types to their corresponding server object types:<br><br>\n\n <table class='normal' border='1'>\n   <tr><td><b>JS Type</b></td>     <td><b>Java Type</b></td> <td><b>C# Type</b></td> <td><b>Perl Type</b></td></tr>\n\n   <tr><td>Object: {}</td>         <td>Map</td>              <td>IDictionary</td>      <td>Associative Array: {}</td></tr>\n   <tr><td>Array: []</td>          <td>List</td>             <td>IList</td>      <td>Array: []</td></tr>\n   <tr><td>String</td>             <td>String</td>           <td>string</td>         <td>string</td></tr>\n   <tr><td>Number</td>             <td>Long|Double</td>      <td>long|double</td>    <td>string</td></tr>\n   <tr><td>Boolean</td>            <td>Boolean</td>          <td>bool</td>           <td>string</td></tr>\n   <tr><td>Date</td>               <td>java.util.Date</td>   <td>DateTime</td>       <td>string</td></tr>\n \n </table>\n <br><br>\n Note that the order of keys/values in the Maps created on the server is not guaranteed\n because JavaScript Object literals do not guarantee order.\n <p>\n Server->client conversion follows the this table as well, with some extras.  See the toJS()\n method on JSTranslater in the server documentation for a description of additional\n behaviors.\n <P>\n When <b>not</b> communicating with the SmartClient server, <code>rpcRequest.data</code>\n becomes simple HTTP parameters or an HTTP request body - see ${isc.DocUtils.linkForRef('attr:RPCRequest.useSimpleHttp')}\n for details.",
        name:"data",
        seeAlso:[],
        valueType:"any serializeable",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "method:FormItem.getFieldName":{
        ref:"method:FormItem.getFieldName",
        flags:"A",
        description:"			Return the name for the this formItem.\n",
        name:"getFieldName",
        seeAlso:[],
        returns:{description:"name for this form item", type:"string"},
        type:"method",
        definingClass:"class:FormItem",
        groups:["drawing"]
    },
    "attr:Button.valign":{
        ref:"attr:Button.valign",
        flags:"IRW",
        description:"Vertical alignment of this component's title.",
        name:"valign",
        setter:["setVAlign"],
        seeAlso:[],
        valueType:"VerticalAlignment",
        defaultValue:"Canvas.CENTER",
        type:"attr",
        definingClass:"class:Button",
        groups:["appearance"]
    },
    "method:TileGrid.selectRecord":{
        ref:"method:TileGrid.selectRecord",
        flags:"",
        description:"\n Select/deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n\n",
        name:"selectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to select", optional:false, name:"record",
             defaultValue:null, type:"Record | number"},
            {description:"new selection state (if null, defaults to true)", optional:true,
             name:"newState", defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "method:TreeGrid.getOpenIcon":{
        ref:"method:TreeGrid.getOpenIcon",
        flags:"A",
        description:" Get the appropriate open/close opener icon for a node.\n\n",
        name:"getOpenIcon",
        seeAlso:[],
        params:[
            {description:"tree node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"URL for the icon to show the node's open state", type:"URL"},
        type:"method",
        definingClass:"class:TreeGrid"
    },
    "method:ListGrid.deselectRecord":{
        ref:"method:ListGrid.deselectRecord",
        flags:"",
        description:"\n Deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecord(record, false)</code>\n\n",
        name:"deselectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to deselect", optional:false, name:"record",
             defaultValue:null, type:"Record | number"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "method:Selection.getSelectedRecord":{
        ref:"method:Selection.getSelectedRecord",
        flags:"",
        description:"			Return the first item in the list that is selected.<br><br>\n\n			Note that this should only be used if you know that one only one item\n			 may be selected, or you really don't care about items after the first one.<br><br>\n\n			To get all selected objects, use <code>${isc.DocUtils.linkForRef('method:Selection.getSelection')}</code>\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"first selected record, or null if nothing selected", type:"object"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:Dialog.applyClick":{
        ref:"method:Dialog.applyClick",
        flags:"",
        description:" Handle a click on the 'apply' button of this Dialog.  \n Default implementation is to call <code>saveData()</code>, but NOT close the Dialog.\n",
        name:"applyClick",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        type:"method",
        definingClass:"class:Dialog",
        groups:["buttons"]
    },
    "method:ListGrid.recordDoubleClick":{
        ref:"method:ListGrid.recordDoubleClick",
        flags:"",
        description:" Executed when the listGrid receives a 'doubleClick' event on an enabled, non-separator\n record. The default implementation does nothing -- override to perform\n some action when any record or field is double clicked.<br>\n A record event handler can be specified either as a function to execute, or as a string\n of script to evaluate. If the handler is defined as a string of script, all the\n parameters below will be available as variables for use in the script.<br>\n To do something specific if a particular field is double clicked, add a\n recordDoubleClick method or string of script to that field (same parameters) when you're\n setting up the list.<br>\n <b>Notes:</b><ul>\n <li>This will not be called if the click is below the last item of the list.</li>\n <li>This method is called from the default implementation of ${isc.DocUtils.linkForRef('method:ListGrid.rowDoubleClick')},\n so if that method is overridden this method may not be fired.</li></ul>\n",
        name:"recordDoubleClick",
        seeAlso:[
            {ref:"method:ListGrid.rowDoubleClick", description:null}
        ],
        params:[
            {description:"the listGrid that contains the doubleclick event", optional:false,
             name:"viewer", defaultValue:null, type:"listGrid"},
            {description:"the record that was double-clicked", optional:false, name:"record",
             defaultValue:null, type:"ListGridRecord"},
            {description:"number of the record clicked on in the current set of                                  displayed records (starts with 0)",
             optional:false, name:"recordNum", defaultValue:null, type:"number"},
            {description:"the field that was clicked on (field definition)", optional:false,
             name:"field", defaultValue:null, type:"ListGridField"},
            {description:"number of the field clicked on in the listGrid.fields                                  array",
             optional:false, name:"fieldNum", defaultValue:null, type:"number"},
            {description:"value of the cell (after valueMap, etc. applied)", optional:false,
             name:"value", defaultValue:null, type:"object"},
            {description:"raw value of the cell (before valueMap, etc applied)", optional:false,
             name:"rawValue", defaultValue:null, type:"object"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["events"]
    },
    "attr:ListGrid.canSelectAll":{
        ref:"attr:ListGrid.canSelectAll",
        flags:"IRW",
        description:"Controls whether a checkbox for selecting all records appears in the header with \n ${isc.DocUtils.linkForRef('attr:ListGrid.selectionAppearance','selectionAppearance')} set to \"checkbox\"",
        name:"canSelectAll",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["selection"]
    },
    "classMethod:Date.compareDates":{
        ref:"classMethod:Date.compareDates",
        flags:"",
        description:" Compare two dates; returns 0 if equal, -1 if the first date is greater (later), or 1 if\n the second date is greater.\n",
        name:"compareDates",
        seeAlso:[],
        params:[
            {description:"first date to compare", optional:false, name:"date1", defaultValue:null,
             type:"date"},
            {description:"second date to compare", optional:false, name:"date2", defaultValue:null,
             type:"date"}
        ],
        returns:{description:"0 if equal, -1 if first date &gt; second date, 1 if second date &gt; first date",
         type:"number"},
        type:"classMethod",
        definingClass:"class:Date"
    },
    "classMethod:EventHandler.getKeyEventCharacter":{
        ref:"classMethod:EventHandler.getKeyEventCharacter",
        flags:"",
        description:"			Return the character for the current key being pressed.\n			Note that this is only set reliably for keyPress events on character keys.\n                  \n",
        name:"getKeyEventCharacter",
        seeAlso:[],
        returns:{description:"Character the user entered. May be null for non-character keys.", type:"string"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["keyboardEvents"]
    },
    "attr:ListGrid.autoFitData":{
        ref:"attr:ListGrid.autoFitData",
        flags:"IRW",
        description:"Should this ListGrid automatically expand to accomodate its content?\n <P>\n Valid settings are\n <ul><li><code>\"vertical\"</code>: expand vertically to accomodate records.</li>\n     <li><code>\"horizontal\"</code>: expand horizontally to accomodate fields.</li>\n     <li><code>\"both\"</code>: expand horizontally and vertically to accomodate content.</li>\n </ul>\n Note that how far the ListGrid will expand may be limited via the following properties:\n ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxHeight')}, ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxRecords')},\n ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxWidth')}, ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitMaxColumns')}.",
        name:"autoFitData",
        setter:["setAutoFitData"],
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "group:advancedFilter":{
        ref:"group:advancedFilter",
        refs:["method:Operator.condition", "method:Operator.compareCriteria", 
         "method:DataSource.addSearchOperator","method:DataSource.getSearchOperator", "method:DataSource.getTypeOperators", 
         "method:DataSource.setTypeOperators","method:DataSource.getFieldOperators", "method:DataSource.getFieldOperatorMap", 
         "method:DataSource.getTypeOperatorMap","method:DataSource.evaluateCriterion", "object:AdvancedCriteria", "object:Criterion", 
         "object:Operator","classMethod:DataSource.addSearchOperator", "classMethod:DataSource.setTypeOperators", 
         "attr:Criterion.operator","attr:Criterion.fieldName", "attr:Criterion.value", "attr:Criterion.criteria", 
         "attr:Criterion.start","attr:Criterion.end", "attr:Operator.ID", "attr:Operator.title", 
         "attr:Operator.titleProperty","attr:Operator.fieldTypes", "attr:Operator.requiresServer", "attr:Operator.hidden", 
         "attr:Operator.valueType","attr:Operator.editorType", "attr:DataSourceField.validOperators", 
         "attr:SimpleType.validOperators","type:OperatorValueType", "type:OperatorId"],
        name:"advancedFilter",
        seeAlso:[],
        type:"group"
    },
    "method:FormItem.setDisabled":{
        ref:"method:FormItem.setDisabled",
        flags:"A",
        description:"  Set this item to be enabled or disabled at runtime. \n",
        name:"setDisabled",
        seeAlso:[
            {ref:"attr:FormItem.disabled", description:null}
        ],
        params:[
            {description:"true if this item should be disabled", optional:false, name:"disabled",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:FormItem",
        groups:["enable"]
    },
    "attr:Calendar.showControlsBar":{
        ref:"attr:Calendar.showControlsBar",
        flags:"IR",
        description:"If false the controls bar at the top of the calendar will not be displayed. This consists \n of the autoChildren: ${isc.DocUtils.linkForRef('attr:Calendar.previousButton')}, ${isc.DocUtils.linkForRef('attr:Calendar.nextButton')}, ${isc.DocUtils.linkForRef('attr:Calendar.addEventButton')},\n ${isc.DocUtils.linkForRef('attr:Calendar.datePickerButton')}",
        name:"showControlsBar",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:ColorPicker.getHue":{
        ref:"method:ColorPicker.getHue",
        flags:"",
        description:" Returns the Hue of the currently-selected color, as an integer from 0-239\n",
        name:"getHue",
        seeAlso:[
            {ref:"method:ColorPicker.setHue", description:null}
        ],
        returns:{description:"hue value", type:"int"},
        type:"method",
        definingClass:"class:ColorPicker"
    },
    "class:JSON":{
        methods:[],
        ref:"class:JSON",
        classAttrs:[],
        classMethods:["classMethod:JSON.encode", "classMethod:JSON.decode"],
        treeLocation:["Client Reference/Data Binding"],
        attrs:[],
        description:"Utilities for working with JSON data.",
        name:"JSON",
        seeAlso:[],
        type:"class"
    },
    "method:StretchImgButton.setIcon":{
        ref:"method:StretchImgButton.setIcon",
        flags:"",
        description:" Change the icon being shown next to the title text.\n",
        name:"setIcon",
        seeAlso:[],
        params:[
            {description:"URL of new icon", optional:false, name:"icon", defaultValue:null, type:"URL"}
        ],
        type:"method",
        definingClass:"class:StretchImgButton",
        groups:["buttonIcon"]
    },
    "method:Tree.loadChildren":{
        ref:"method:Tree.loadChildren",
        flags:"",
        description:" 			Load the children of a given node.\n\n 			This implementation does nothing other than mark the node as loaded\n",
        name:"loadChildren",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"},
            {description:"Optional callback (stringMethod) to fire when loading                       completes. Has a single param <code>node</code> - the node whose \n                      children have been loaded, and is fired in the scope of the Tree.",
             optional:true, name:"callback", defaultValue:null, type:"callback"}
        ],
        type:"method",
        definingClass:"class:Tree",
        groups:["loadState"]
    },
    "attr:PickList.pickListProperties":{
        ref:"attr:PickList.pickListProperties",
        flags:"IR",
        description:"If specified this properties block will be applied to the ${isc.DocUtils.linkForRef('class:PickListMenu','pickList')}\n created for this Form Item.",
        name:"pickListProperties",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"interface:PickList"
    },
    "classAttr:GroupingMessages.byDayTitle":{
        ref:"classAttr:GroupingMessages.byDayTitle",
        flags:"IRW",
        description:"Title to use for the menu option which groups a date field by day.",
        name:"byDayTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"by Day\"",
        type:"classAttr",
        definingClass:"class:GroupingMessages",
        groups:["i18nMessages"]
    },
    "attr:ListGridField.canFreeze":{
        ref:"attr:ListGridField.canFreeze",
        flags:"IR",
        description:"Whether this field should display freezing/unfreezing options in its header context menu. \n See ${isc.DocUtils.linkForRef('group:frozenFields')}.",
        name:"canFreeze",
        seeAlso:[
            {ref:"method:ListGrid.getHeaderContextMenuItems", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["frozenFields"]
    },
    "method:List.removeAt":{
        ref:"method:List.removeAt",
        flags:"",
        description:" Remove the item at the specified position, rearranging all subsequent items to fill the gap\n\n",
        name:"removeAt",
        seeAlso:[],
        params:[
            {description:"position to remove", optional:false, name:"pos", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"item that was removed", type:"any"},
        type:"method",
        definingClass:"interface:List",
        groups:["modification"]
    },
    "attr:SimpleType.valueMap":{
        ref:"attr:SimpleType.valueMap",
        flags:"IR",
        description:"List of legal values for this type, like ${isc.DocUtils.linkForRef('attr:DataSourceField.valueMap')}.",
        name:"valueMap",
        seeAlso:[],
        valueType:"ValueMap",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:SimpleType",
        groups:["dataType"]
    },
    "attr:CubeGrid.rowHeaderBaseStyle":{
        ref:"attr:CubeGrid.rowHeaderBaseStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:Button.baseStyle','baseStyle')} for the buttons in this grid's row headers.",
        name:"rowHeaderBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"rowHeader",
        type:"attr",
        definingClass:"class:CubeGrid",
        groups:["appearance"]
    },
    "attr:DataSource.tagName":{
        ref:"attr:DataSource.tagName",
        flags:"IRA",
        description:"Tag name to use when serializing to XML.  If unspecified, the <code>dataSource.ID</code>\n will be used.",
        name:"tagName",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource",
        groups:["clientDataIntegration"]
    },
    "attr:DSRequest.operationId":{
        ref:"attr:DSRequest.operationId",
        flags:"I",
        description:"When a ${isc.DocUtils.linkForRef('interface:DataBoundComponent')} sends a DSRequest, the\n <code>dsRequest.operationId</code> will be automatically picked up from the\n <code>fetchOperation</code>, <code>addOperation</code>, etc properties of the\n DataBoundComponent.  \n <P>\n The <code>operationId</code> serves as an identifier that you can use to create\n variations on the 4 basic DataSource operations that are used by different components in\n different parts of your application.  For example, you may be using a standard\n <code>fetch</code> operation in one part of your application, however on another screen you\n want to perform a <code>fetch</code> operation on the same DataSource but interpret search\n criteria differently (eg full text search).  \n <P>\n If you declare more than one ${isc.DocUtils.linkForRef('class:OperationBinding')} for the same\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType')}, you can specify an <code>operationId</code>\n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationId','on the operationBinding')} which will cause that\n operationBinding to be used for dsRequests containing a matching <code>operationId</code>.\n This allows all the possible settings of an <code>operationBinding</code>, including\n ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','wsOperation')} or ${isc.DocUtils.linkForRef('class:DMI')} settings, to be switched on a\n per-component or per-request basis.  \n <P>\n For example, by setting the <code>fetchOperation</code> on a particular ListGrid, you could\n cause it to invoke a different server method via DMI, different\n ${isc.DocUtils.linkForRef('attr:OperationBinding.dataURL','dataURL')} or different ${isc.DocUtils.linkForRef('attr:OperationBinding.wsOperation','web service\n operation')}.\n <P>\n The <code>operationId</code> can also be directly received by the server in order to affect\n behavior.  When using the SmartClient Server, <code>operationId</code> can be accessed via\n dsRequest.getOperationId().  The ${isc.DocUtils.linkForRef('class:RestDataSource')} will also send the\n <code>operationId</code> to the server as part of the\n ${isc.DocUtils.linkForRef('attr:RestDataSource.metaDataPrefix','request metadata')}.  \n <P>\n Note that if you ${isc.DocUtils.linkForRef('method:DataSource.fetchData','manually invoke')} a DataSource operation, you can\n also specify operationId via the <code>requestProperties</code> parameter.",
        name:"operationId",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DSRequest",
        groups:["operations"]
    },
    "attr:StatefulCanvas.showFocus":{
        ref:"attr:StatefulCanvas.showFocus",
        deprecated:"as of SmartClient version 6.1 in favor of ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')}",
        flags:"IRW",
        description:"Should we visibly change state when the canvas recieves focus?  Note that by default the\n <code>over</code> state is used to indicate focus.",
        name:"showFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:StatefulCanvas",
        groups:["state"]
    },
    "classAttr:Operators.endsWithTitle":{
        ref:"classAttr:Operators.endsWithTitle",
        flags:"IR",
        description:"Title for the \"endsWith\" operator",
        name:"endsWithTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"ends with (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "attr:ListGrid.autoFitMaxColumns":{
        ref:"attr:ListGrid.autoFitMaxColumns",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')} is set to <code>\"horizontal\"</code> or <code>\"both\"</code>\n this property provides the maximum number of columns for which the ListGrid will expand.\n If more columns are present, scrolling will be introduced to reach them as normal.\n If unset the ListGrid will expand to accomodate as many columns as are defined for the\n grid.",
        name:"autoFitMaxColumns",
        setter:["setAutoFitMaxColumns"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"50",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "classMethod:DataSource.convertCriteria":{
        ref:"classMethod:DataSource.convertCriteria",
        flags:"",
        description:" Converts criteria expressed in SmartClient's simple criteria format to an AdvancedCriteria\n object.  \n\n",
        name:"convertCriteria",
        seeAlso:[],
        params:[
            {description:"simple criteria", optional:false, name:"criteria", defaultValue:null,
             type:"Criteria"},
            {description:"default style of matching text.  Defaults to                                          \"substring\"",
             optional:true, name:"textMatchStyle", defaultValue:null, type:"TextMatchStyle"}
        ],
        returns:{description:"equivalent AdvancedCriteria object", type:"AdvancedCriteria"},
        type:"classMethod",
        definingClass:"class:DataSource"
    },
    "attr:RichTextItem.endRow":{
        ref:"attr:RichTextItem.endRow",
        flags:"IRW",
        description:"By default RichTextItems take up an entire row",
        name:"endRow",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:RichTextItem"
    },
    "class:Button":{
        methods:["method:Button.getTitle", "method:Button.setTitle", "method:Button.setIconOrientation", 
         "method:Button.setIcon","method:Button.setAutoFit", "method:Button.setBaseStyle", "method:Button.select", 
         "method:Button.deselect","method:Button.isSelected", "method:Button.setSelected", "method:Button.addToRadioGroup", 
         "method:Button.removeFromRadioGroup","method:Button.setActionType", "method:Button.getActionType", "method:Button.setState", 
         "method:Button.setDisabled","method:Button.getState", "method:Button.action", "method:Button.setWrap", 
         "method:Button.setAlign","method:Button.setVAlign", "method:Button.iconClick"],
        ref:"class:Button",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:["attr:Button.title", "attr:Button.hiliteAccessKey", "attr:Button.icon", 
         "attr:Button.iconSize","attr:Button.iconWidth", "attr:Button.iconHeight", "attr:Button.iconOrientation", 
         "attr:Button.iconAlign","attr:Button.iconSpacing", "attr:Button.showDisabledIcon", "attr:Button.showRollOverIcon", 
         "attr:Button.showFocusedIcon","attr:Button.showDownIcon", "attr:Button.showSelectedIcon", "attr:Button.autoFit", 
         "attr:Button.baseStyle","attr:Button.selected", "attr:Button.radioGroup", "attr:Button.actionType", 
         "attr:Button.state","attr:Button.showDisabled", "attr:Button.showDown", "attr:Button.showFocused", 
         "attr:Button.showRollOver","attr:Button.align", "attr:Button.valign", "attr:Button.wrap"],
        description:"The Button widget class implements interactive, style-based button widgets.",
        name:"Button",
        seeAlso:[],
        type:"class"
    },
    "classMethod:isc.sortObjectByProperties":{
        ref:"classMethod:isc.sortObjectByProperties",
        flags:"",
        description:" Given a simple javascript object, return that object sorted by properties, such that when \n iterating through the properties of the object, values will show up in sorted order.<br>\n Usage example - may be used to sort a ${isc.DocUtils.linkForRef('attr:FormItem.valueMap','formItem valueMap')} defined\n as an object by display value.\n",
        name:"sortObjectByProperties",
        seeAlso:[],
        params:[
            {description:"Object to sort", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"Comparitor function to use when sorting the object properties",
             optional:true, name:"comparitor", defaultValue:null, type:"function"}
        ],
        returns:{description:"sorted version of the object passed in.", type:"object"},
        type:"classMethod",
        definingClass:"class:isc"
    },
    "method:Class.clearLogPriority":{
        ref:"method:Class.clearLogPriority",
        flags:"",
        description:" Clear this object's priority setting for a particular category, so that the category's \n effective priority returns to the specified priority for this category at the Log level\n (or <code>Log.defaultPriority</code> if not set).<br>\n To clear the Page-level priority setting for this log category use \n <code>isc.Log.clearPriority()</code> instead.\n\n",
        name:"clearLogPriority",
        seeAlso:[
            {ref:"classMethod:Log.clearPriority", description:null}
        ],
        params:[
            {description:"Category name. If not specified, all logging on this object                              will revert to default priority settings.",
             optional:false, name:"category", defaultValue:null, type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "attr:ListGrid.loadingDataMessage":{
        ref:"attr:ListGrid.loadingDataMessage",
        flags:"IRW",
        description:"The string to display in the body of a listGrid while data is being loaded.",
        name:"loadingDataMessage",
        seeAlso:[
            {ref:"attr:ListGrid.loadingDataMessageStyle", description:null}
        ],
        valueType:"string",
        defaultValue:"\"Loading data...\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["loadingDataMessage", "i18nMessages"]
    },
    "attr:DSRequest.parentNode":{
        ref:"attr:DSRequest.parentNode",
        flags:"R",
        description:"For advanced use in integrating trees that \n ${isc.DocUtils.linkForRef('attr:ResultTree.loadDataOnDemand','load data on demand')} with web services,\n <code>parentNode</code> is automatically set in \"fetch\" DSRequests issued by a\n ${isc.DocUtils.linkForRef('class:ResultTree')} that is loading children for that <code>parentNode</code>.\n <P>\n This is sometimes needed if a web service requires that additional properties beyond the\n ID of the parentNode must be passed in order to accomplished level-by-level loading.\n A custom implementation of ${isc.DocUtils.linkForRef('method:DataSource.transformRequest')} can access\n dsRequest.parentNode and add any such properties to ${isc.DocUtils.linkForRef('attr:DSRequest.data')}.\n <P>\n <code>parentNode</code> will also be automatically set by a TreeGrid performing databound\n reparenting of nodes, as implemented by ${isc.DocUtils.linkForRef('method:TreeGrid.folderDrop')}.\n <P>\n This property can only be read.  There is no meaning to setting this property\n yourself.",
        name:"parentNode",
        seeAlso:[],
        valueType:"TreeNode",
        defaultValue:"{}",
        type:"attr",
        definingClass:"class:DSRequest"
    },
    "method:ListGrid.dataArrived":{
        ref:"method:ListGrid.dataArrived",
        flags:"A",
        description:" Notification method fired when new data arrives from the server to be displayed in this\n ListGrid, (for example in response to the user scrolling a new set of rows into view).\n Only applies to databound listGrids where the ${isc.DocUtils.linkForRef('attr:ListGrid.data','data')} attribute is a\n ${isc.DocUtils.linkForRef('class:ResultSet')}. This ResultSet may have been created manually and applied to the grid via\n a call to ${isc.DocUtils.linkForRef('method:ListGrid.setData')} or may have been created and automatically assigned if\n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} was used to populate the grid.  This method is fired directly in\n response to ${isc.DocUtils.linkForRef('method:ResultSet.dataArrived','dataArrived()')} firing on the data object.\n",
        name:"dataArrived",
        seeAlso:[],
        params:[
            {description:"starting index of the newly loaded set of rows", optional:false,
             name:"startRow", defaultValue:null, type:"integer"},
            {description:"ending index of the newly loaded set of rows (non inclusive).",
             optional:false, name:"endRow", defaultValue:null, type:"integer"}
        ],
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:TextItem.setMask":{
        ref:"method:TextItem.setMask",
        flags:"",
        description:" Set the mask for this item.\n <P>\n Note that the current value of the field is cleared when changing the mask.\n",
        name:"setMask",
        seeAlso:[
            {ref:"attr:TextItem.mask", description:null}
        ],
        params:[
            {description:"mask to apply to text item", optional:false, name:"mask", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:TextItem"
    },
    "attr:ListGrid.sortAscendingImage":{
        ref:"attr:ListGrid.sortAscendingImage",
        flags:"IRWA",
        description:"Image to show when sorting ascending. See ${isc.DocUtils.linkForRef('object:ImgProperties')} for format.",
        name:"sortAscendingImage",
        seeAlso:[],
        valueType:"ImgProperties",
        defaultValue:"{...}",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "method:Canvas.getInnerWidth":{
        ref:"method:Canvas.getInnerWidth",
        flags:"A",
        description:"  Returns the amount of space available for (an) absolutely positioned child widget(s) or \n  HTML content, without introducing clipping, scrolling or overflow.<br>\n  This is the space within the viewport of the widget (including padding, but excluding \n  margins, borders or scrollbars) rendered at its specified size.\n\n",
        name:"getInnerWidth",
        seeAlso:[
            {ref:"method:Canvas.getInnerHeight", description:null},
            {ref:"method:Canvas.getInnerContentHeight", description:null},
            {ref:"method:Canvas.getInnerContentWidth", description:null}
        ],
        returns:{description:"inner width of the widget in pixels", type:"number"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:ListGridField.emptyCellValue":{
        ref:"attr:ListGridField.emptyCellValue",
        flags:"IRW",
        description:"The value to display for a cell whose value is null or the empty\n string after applying formatCellValue and valueMap (if any).\n <p>\n This is the field-specific attribute.  You may also set the emptyCellValue at the grid\n level to define the emptyCellValue for all empty fields in the grid.",
        name:"emptyCellValue",
        seeAlso:[
            {ref:"attr:ListGrid.emptyCellValue", description:null}
        ],
        valueType:"string",
        defaultValue:"\"&amp;nbsp;\"",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('emptyValues')}"],
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "attr:DynamicForm.titleWidth":{
        ref:"attr:DynamicForm.titleWidth",
        flags:"IRW",
        description:"The width in pixels allocated to the title of every item in this form.  If you\n          don't specify explicit ${isc.DocUtils.linkForRef('attr:DynamicForm.colWidths')}, you can set this\n          value to the string \"*\" to divide the usable space evenly between titles and\n          fields.",
        name:"titleWidth",
        seeAlso:[],
        valueType:"number or \"*\"",
        defaultValue:"100",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "method:Calendar.eventAdded":{
        ref:"method:Calendar.eventAdded",
        flags:"",
        description:" Notification fired whenever a user adds an event.\n <P>\n In a calendar with a DataSource, eventAdded() fires <b>after</b> the event has\n been successfully added to the server\n\n",
        name:"eventAdded",
        seeAlso:[],
        params:[
            {description:"the event that was added", optional:false, name:"event", defaultValue:null,
             type:"CalendarEvent"}
        ],
        type:"method",
        definingClass:"class:Calendar",
        groups:["monthViewEvents"]
    },
    "attr:Validator.type":{
        ref:"attr:Validator.type",
        flags:"IR",
        description:"Type of the validator.\n <p>\n This can be one of the built-in ${isc.DocUtils.linkForRef('class:ValidatorTypes')}, the string \"custom\" to define\n a custom validator, or the string \"serverCustom\" to define a server-only custom validator.",
        name:"type",
        seeAlso:[],
        valueType:"ValidatorTypes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Validator"
    },
    "attr:Applet.params":{
        ref:"attr:Applet.params",
        flags:"IR",
        description:"A map of key/value pairs to pass to the applet as parameters.  These are typically\n  accessible within the Java applet code.",
        name:"params",
        seeAlso:[],
        valueType:"object",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Applet"
    },
    "attr:Canvas.snapOffsetTop":{
        ref:"attr:Canvas.snapOffsetTop",
        flags:"IRW",
        description:"If ${isc.DocUtils.linkForRef('attr:Canvas.snapTo','snapTo')} is defined for this widget, this property can be used to\n specify an offset in px or percentage for the top coordinate of this widget.\n <P>\n For example if <code>snapTo</code> is specifed as <code>\"T\"</code> and \n <code>snapOffsetTop</code> is set to 6, this widget will be rendered 6px below the top\n edge of its parent or master element.",
        name:"snapOffsetTop",
        seeAlso:[
            {ref:"attr:Canvas.snapTo", description:null}
        ],
        valueType:"integer",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "attr:SpacerItem.showTitle":{
        ref:"attr:SpacerItem.showTitle",
        flags:"IRW",
        description:"we never show a separate title cell for spacers",
        name:"showTitle",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:SpacerItem",
        groups:["appearance"]
    },
    "type:EnumTranslateStrategy":{
        ref:"type:EnumTranslateStrategy",
        values:[
            {description:"Translates to/from a String matching the constant name. This is the default if not set.",
             value:"\"string\""},
            {description:"Translates to/from an integer matching the ordinal number of the constant within \n   the enumeration",
             value:"\"ordinal\""},
            {description:"Translates to/from a Javascript object containing one property for each property defined \n   within the enum. The constant itself and the ordinal number are included in the JS object.\n   By default they are called \"_constant\" and \"_ordinal\", but this can be overridden with \n   the ${isc.DocUtils.linkForRef('attr:DataSource.enumOrdinalProperty')} and ${isc.DocUtils.linkForRef('attr:DataSource.enumConstantProperty')}\n   properties",
             value:"\"bean\""}
        ],
        description:"Determines how Java enums are translated to and from Javascript by the SmartClient server.",
        name:"EnumTranslateStrategy",
        seeAlso:[
            {ref:"attr:DataSource.enumTranslateStrategy", description:null}
        ],
        type:"type",
        serverDS:"allowed"
    },
    "attr:Img.activeAreaHTML":{
        ref:"attr:Img.activeAreaHTML",
        flags:"IRWA",
        description:"Setting this attribute configures an image map for this image.  The value is expected as a\n sequence of &lg;AREA&gt tags - e.g:\n <pre>\n Img.create({ \n     src: \"myChart.gif\",\n     activeAreaHTML:\n         \"&lt;AREA shape='rect' coords='10,50,30,200' title='30' href='javascript:alert(\\\"30 units\\\")'&gt;\" +\n         \"&lt;AREA shape='rect' coords='50,90,80,200' title='22' href='javascript:alert(\\\"22 units\\\")'&gt;\"\n });\n </pre>\n <u>Implementation notes:</u>\n <ul>\n <li>Quotes in the activeAreaHTML must be escaped or alternated appropriately.</li>\n <li>Image maps do not stretch to fit scaled images. You must ensure that the dimensions of\n your Img component match the anticipated width and height of your image map (which will typically\n match the native dimensions of your image). </li>\n <li>To change the image map of an existing Img component, first set yourImg.activeAreaHTML,\n then call yourImg.markForRedraw(). Calls to yourImg.setSrc() will not automatically update the\n image map. </li>\n <li>activeAreaHTML is not supported on tiled Img components (imageType:\"tile\").</li> \n <li>Native browser support for image map focus/blur, keyboard events, and certain AREA tag\n attributes (eg NOHREF, DEFAULT...) varies by platform. If your image map HTML uses attributes\n beyond the basics (shape, coords, href, title), you should test on all supported browsers to\n ensure that it functions as expected.</li>\n </ul>",
        name:"activeAreaHTML",
        seeAlso:[],
        valueType:"String of HTML AREA tags",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Img",
        groups:["appearance"]
    },
    "classMethod:isA.String":{
        ref:"classMethod:isA.String",
        flags:"",
        description:"\n	Is <code>object</code> a String object?\n\n",
        name:"String",
        seeAlso:[],
        params:[
            {description:"object to test", optional:false, name:"object", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == <code>object</code> is a String", type:"boolean"},
        type:"classMethod",
        definingClass:"object:isA"
    },
    "method:ComboBoxItem.getSelectedRecord":{
        ref:"method:ComboBoxItem.getSelectedRecord",
        flags:"",
        description:" Returns the entire record object associated with the current value for this item\n (or null if no matching record exists in the PickList data).\n <P>\n Most commonly used for databound pickListItems to retrieve the values of other fields\n in the record.\n\n",
        name:"getSelectedRecord",
        seeAlso:[],
        returns:{description:"record object", type:"object"},
        type:"method",
        definingClass:"class:ComboBoxItem"
    },
    "method:DataSource.fetchRecord":{
        ref:"method:DataSource.fetchRecord",
        flags:"",
        description:" Fetch a single record from the DataSource by ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primary key')}.\n This simply calls ${isc.DocUtils.linkForRef('method:DataSource.fetchData')} after creating ${isc.DocUtils.linkForRef('type:Criteria')} that contain the primary\n key field and value.\n\n",
        name:"fetchRecord",
        seeAlso:[],
        params:[
            {description:"value for the field marked                      ${isc.DocUtils.linkForRef('attr:DataSourceField.primaryKey','primaryKey')}:true in this DataSource.",
             optional:false, name:"pkValue", defaultValue:null, type:"any"},
            {description:"callback to invoke on completion", optional:true, name:"callback",
             defaultValue:null, type:"DSCallback"},
            {description:"additional properties to set on the                                                   DSRequest that will be issued",
             optional:true, name:"requestProperties", defaultValue:null, type:"DSRequest Properties"}
        ],
        type:"method",
        definingClass:"class:DataSource"
    },
    "attr:Calendar.eventWindowStyle":{
        ref:"attr:Calendar.eventWindowStyle",
        flags:"IRW",
        description:"The base name for the CSS class applied to event windows within calendars.\n This style will have \"Header\", \"HeaderLabel\", and \"Body\" appended to it, according to \n which part of the event window is being styled. For example, to style the header, define\n a css class called 'eventWindowHeader'.",
        name:"eventWindowStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"eventWindow\"",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["appearance"]
    },
    "type:PanelHeaderPlacement":{
        ref:"type:PanelHeaderPlacement",
        values:[
            {description:"Add the ${isc.DocUtils.linkForRef('class:PanelHeader')} as a peer of this Canvas.",
             value:"\"peer\""},
            {description:"Add the PanelHeader as the first member of this Canvas.",
             value:"\"member\""},
            {description:"Don't attempt to place the PanelHeader, assuming that the component will do so itself.",
             value:"\"custom\""}
        ],
        description:null,
        name:"PanelHeaderPlacement",
        seeAlso:[],
        type:"type"
    },
    "method:Tree.getLeaves":{
        ref:"method:Tree.getLeaves",
        flags:"",
        description:"\n Return all the first-level leaves of a node.\n <br><br>\n For load on demand trees (those that only have a partial representation client-side), this\n method will return only nodes that have already been loaded from the server.  Furthermore,\n for databound trees the return value will be a ${isc.DocUtils.linkForRef('class:ResultSet')},\n so it's important to access the return value using the ${isc.DocUtils.linkForRef('interface:List')} interface\n instead of as a native Javascript Array.\n\n",
        name:"getLeaves",
        seeAlso:[],
        params:[
            {description:"node in question", optional:false, name:"node", defaultValue:null,
             type:"TreeNode"}
        ],
        returns:{description:"List of immediate children that are leaves.", type:"List"},
        type:"method",
        definingClass:"class:Tree"
    },
    "method:DataBoundComponent.addSummaryField":{
        ref:"method:DataBoundComponent.addSummaryField",
        flags:"",
        description:" Convenience method to display a ${isc.DocUtils.linkForRef('class:SummaryBuilder')} to create a new Summary Field.  This \n is equivalent to calling ${isc.DocUtils.linkForRef('method:DataBoundComponent.editSummaryField','editSummaryField()')} with \n no paramater.\n\n",
        name:"addSummaryField",
        seeAlso:[],
        type:"method",
        definingClass:"interface:DataBoundComponent",
        groups:["summaryFields"]
    },
    "attr:ListGrid.canReorderFields":{
        ref:"attr:ListGrid.canReorderFields",
        flags:"IRW",
        description:"Indicates whether fields in this listGrid can be reordered by dragging and\n          dropping header fields.",
        name:"canReorderFields",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('columnOrder')}"],
        definingClass:"class:ListGrid",
        groups:["dragging"]
    },
    "attr:HeaderItem.colSpan":{
        ref:"attr:HeaderItem.colSpan",
        flags:"IRW",
        description:"by default, headers span all remaining columns",
        name:"colSpan",
        seeAlso:[],
        valueType:"measure",
        defaultValue:"\"*\"",
        type:"attr",
        definingClass:"class:HeaderItem",
        groups:["appearance"]
    },
    "classAttr:Dialog.CANCEL_BUTTON_TITLE":{
        ref:"classAttr:Dialog.CANCEL_BUTTON_TITLE",
        flags:"IRW",
        description:"Title for the <code>\"Cancel\"</code> button.",
        name:"CANCEL_BUTTON_TITLE",
        seeAlso:[
            {ref:"type:DialogButtons", description:null}
        ],
        valueType:"HTML",
        defaultValue:"\"Cancel\"",
        type:"classAttr",
        definingClass:"class:Dialog",
        groups:["i18nMessages"]
    },
    "method:Selection.setSelected":{
        ref:"method:Selection.setSelected",
        flags:"A",
        description:" Select or deselect a particular item.<br><br>\n All other selection routines go through this one, so by observing this routine you can\n monitor all selection changes.\n",
        name:"setSelected",
        seeAlso:[],
        params:[
            {description:"object to select", optional:false, name:"item", defaultValue:null,
             type:"object"},
            {description:"turn selection on or off", optional:false, name:"newState", defaultValue:null,
             type:"boolean"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:DynamicForm.selectOnFocus":{
        ref:"attr:DynamicForm.selectOnFocus",
        flags:"IRW",
        description:"If this property is set to true, whenever a text-based field in this form \n (${isc.DocUtils.linkForRef('class:TextItem')}, ${isc.DocUtils.linkForRef('class:TextAreaItem')}) is given focus programmatically \n (see ${isc.DocUtils.linkForRef('method:DynamicForm.focusInItem')}), all text within the item will be selected.\n <P>\n Note that this flag affects only programmatic focus.  It's the normal behavior of text\n fields to select all text if the user navigates into them via keyboard, or if the user\n navigates via mouse, to place the text insertion point at the mouse click, and\n SmartClient preserves these behaviors.  <code>selectOnFocus</code> if only needed for\n cases like a form within a pop-up dialog that should have the first field selected.\n <P>\n If <code>selectOnFocus</code> is false, the selection is not modified on focus - any\n previous selection within the item will be maintained.\n <P>\n May be overridden at the form item level via ${isc.DocUtils.linkForRef('attr:FormItem.selectOnFocus')}.",
        name:"selectOnFocus",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["focus"]
    },
    "method:TileGrid.deselectRecord":{
        ref:"method:TileGrid.deselectRecord",
        flags:"",
        description:"\n Deselect a ${isc.DocUtils.linkForRef('object:Record')} passed in explicitly, or by index.\n <P>\n Synonym for <code>selectRecord(record, false)</code>\n\n",
        name:"deselectRecord",
        seeAlso:[],
        params:[
            {description:"record (or row number) to deselect", optional:false, name:"record",
             defaultValue:null, type:"Record | number"}
        ],
        type:"method",
        definingClass:"class:TileGrid",
        groups:["selection"]
    },
    "attr:Criterion.start":{
        ref:"attr:Criterion.start",
        flags:"IR",
        description:"Start value of a criterion with an operator of type <code>\"valueRange\"</code>.",
        name:"start",
        seeAlso:[],
        valueType:"any",
        defaultValue:"null",
        type:"attr",
        serverDS:"allowed",
        definingClass:"object:Criterion",
        groups:["advancedFilter"]
    },
    "attr:DynamicForm.itemHoverAlign":{
        ref:"attr:DynamicForm.itemHoverAlign",
        flags:"IRW",
        description:"Text alignment for hovers shown for items",
        name:"itemHoverAlign",
        seeAlso:[
            {ref:"attr:FormItem.hoverAlign", description:null}
        ],
        valueType:"Alignment",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["Hovers"]
    },
    "attr:Canvas.margin":{
        ref:"attr:Canvas.margin",
        flags:"IRW",
        description:"Set the CSS Margin, in pixels, for this component.  Margin provides blank space outside\n of the border.\n <P>\n This property sets the same thickness of margin on every side.  Differing per-side\n margins can be set in a CSS style and applied via ${isc.DocUtils.linkForRef('attr:Canvas.styleName')}.\n <P>\n Note that the specified size of the widget will be the size <b>including</b> the margin\n thickness on each side.",
        name:"margin",
        setter:["setMargin"],
        seeAlso:[],
        valueType:"number",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Canvas",
        groups:["appearance"]
    },
    "classAttr:ValidatorTypes.regexp":{
        ref:"classAttr:ValidatorTypes.regexp",
        flags:"IR",
        description:"<code>regexp</code> type validators will determine whether the value specified \n  matches a given regular expression.  The expression should be specified on the\n  <code>validator</code> object as the <code>expression</code> property.",
        name:"regexp",
        seeAlso:[],
        valueType:"validatorDefinition",
        defaultValue:"object",
        type:"classAttr",
        examples:["${isc.DocUtils.linkForExampleId('regularExpression')}"],
        definingClass:"class:ValidatorTypes"
    },
    "method:Chart.getFacet":{
        ref:"method:Chart.getFacet",
        flags:"",
        description:" Get a facet definition by facetId.\n",
        name:"getFacet",
        seeAlso:[
            {ref:"class:Facet", description:null}
        ],
        params:[
            {description:"the id of the facet to retrieve", optional:false, name:"facetId",
             defaultValue:null, type:"String"}
        ],
        returns:{description:"the Facet if found, or null", type:"Facet"},
        type:"method",
        definingClass:"interface:Chart"
    },
    "attr:ColorPicker.allowComplexMode":{
        ref:"attr:ColorPicker.allowComplexMode",
        flags:"IR",
        description:"Should the \"complex\" mode be allowed for this ColorPicker?\n If false, no \"More\" button is shown on the simple picker",
        name:"allowComplexMode",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ColorPicker"
    },
    "attr:DynamicForm.requiredRightTitleSuffix":{
        ref:"attr:DynamicForm.requiredRightTitleSuffix",
        flags:"IRW",
        description:"The string appended to the title of every required item in this form if\n highlightRequiredFields is true and the titleOrientation property is set to \"right\".",
        name:"requiredRightTitleSuffix",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"</B>\"",
        type:"attr",
        definingClass:"class:DynamicForm",
        groups:["formTitles"]
    },
    "method:Canvas.setSnapEdge":{
        ref:"method:Canvas.setSnapEdge",
        flags:"",
        description:" Set the snapEdge property of this canvas, and handle repositioning.\n\n",
        name:"setSnapEdge",
        seeAlso:[],
        params:[
            {description:"new snapEdge value", optional:false, name:"snapEdge", defaultValue:null,
             type:"string"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["positioning"]
    },
    "group:xml serialization":{
        ref:"group:xml serialization",
        refs:[],
        name:"xml serialization",
        seeAlso:[],
        type:"group"
    },
    "attr:Tree.reportCollisions":{
        ref:"attr:Tree.reportCollisions",
        flags:"IR",
        description:"If new nodes are added to a tree with modelType:\"parent\" which have the same\n ${isc.DocUtils.linkForRef('attr:Tree.idField','id field value')} as existing nodes, the existing nodess are removed when\n the new nodes are added.\n <P>\n If reportCollisions is true, the Tree will log a warning in the developer console about this.\n <P>\n Note that if an id collision occurs between a new node and its ancestor, the ancestor will be\n removed and the new node will not be added to the tree.",
        name:"reportCollisions",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Tree"
    },
    "attr:Facet.synchColumnLayout":{
        ref:"attr:Facet.synchColumnLayout",
        flags:"IR",
        description:"if true, treat all values in this facet as a facetValueGroup - causes\n synched header reorder and resize.\n  Note: only supported when all of a facets values are used.",
        name:"synchColumnLayout",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Facet"
    },
    "group:Prompting":{
        ref:"group:Prompting",
        treeLocation:["Client Reference/Control"],
        refs:["class:LoginDialog", "classAttr:Dialog.Prompt", "classAttr:Dialog.Warn", 
         "classMethod:isc.showPrompt","classMethod:isc.clearPrompt", "classMethod:isc.warn", "classMethod:isc.say", 
         "classMethod:isc.ask","classMethod:isc.confirm", "classMethod:isc.askForValue", 
         "classMethod:isc.showLoginDialog"],
        description:"Objects / methods used for displaying prompts and warnings to the user via (possibly modal)\n  isc Dialog objects.",
        name:"Prompting",
        seeAlso:[],
        type:"group"
    },
    "method:SectionStack.moveSection":{
        ref:"method:SectionStack.moveSection",
        flags:"",
        description:"\n Moves the specified section(s) to a new position in the SectionStack order.  If you pass\n in multiple sections, then each section will be moved to <code>newPosition</code> in the\n order specified by the <code>sections</code> argument.\n\n",
        name:"moveSection",
        seeAlso:[],
        params:[
            {description:"Section(s) to move.  For this                  parameter, you can pass the position of the section in the\n                  SectionStack, the ID of the section, or a List of sectionIDs",
             optional:false, name:"sections", defaultValue:null,
             type:"position|sectionId|list of sectionIDs"},
            {description:"new position index for the section(s).", optional:false, name:"position",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:SectionStack"
    },
    "attr:RPCRequest.paramsOnly":{
        ref:"attr:RPCRequest.paramsOnly",
        deprecated:"As of SmartClient 5.6, use ${isc.DocUtils.linkForRef('attr:RPCRequest.useSimpleHttp')} instead.",
        flags:"IRWA",
        description:"When set to true, assume the request is not going to the SmartClient server, and hence send\n a simple HTTP request.  Values specified in ${isc.DocUtils.linkForRef('attr:RPCRequest.params')} are sent to to the\n server as HTTP request parameters.  If ${isc.DocUtils.linkForRef('attr:RPCRequest.httpMethod')} method is POST and\n ${isc.DocUtils.linkForRef('attr:RPCRequest.data')} is supplied, it is assumed to be a string to post as the HTTP\n requestBody.\n <p>\n Setting this to true automatically defaults ${isc.DocUtils.linkForRef('attr:RPCRequest.serverOutputAsString')} to true\n as well.",
        name:"paramsOnly",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:RPCRequest"
    },
    "attr:Layout.reverseOrder":{
        ref:"attr:Layout.reverseOrder",
        flags:"IRW",
        description:"Reverse the order of stacking for this Layout, so that the last member is shown first.\n <P>\n Requires a manual call to <code>reflow()</code> if changed on the fly.\n <P>\n In RTL mode, for horizontal Layouts the value of this flag will be flipped during\n initialization.",
        name:"reverseOrder",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:Layout",
        groups:["layoutPolicy"]
    },
    "attr:Window.statusBar":{
        ref:"attr:Window.statusBar",
        flags:"R",
        description:"Simple Canvas-based status bar, shown in the footer.  ${isc.DocUtils.linkForRef('method:Window.setStatus')} can be used to\n show text here.",
        name:"statusBar",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "attr:TileLayout.tileMargin":{
        ref:"attr:TileLayout.tileMargin",
        flags:"IR",
        description:"Margin in between tiles.  Can be set on a per-axis basis with ${isc.DocUtils.linkForRef('attr:TileLayout.tileHMargin')} and\n ${isc.DocUtils.linkForRef('attr:TileLayout.tileVMargin')}.",
        name:"tileMargin",
        setter:["setTileMargin"],
        seeAlso:[],
        valueType:"int",
        defaultValue:"10",
        type:"attr",
        definingClass:"class:TileLayout",
        groups:["layoutMargin"]
    },
    "object:HeaderSpan":{
        methods:[],
        ref:"object:HeaderSpan",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Grids/ListGrid"],
        attrs:["attr:HeaderSpan.name", "attr:HeaderSpan.fields", "attr:HeaderSpan.title", 
         "attr:HeaderSpan.height","attr:HeaderSpan.valign", "attr:HeaderSpan.align"],
        description:"A header span appears as a second level of headers in a ListGrid, spanning one or more\n ListGrid columns and their associated headers.\n <P>\n See ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpans')}.\n <P>\n In addition to the properties documented here, all other properties specified on the\n headerSpan object will be passed to the ${isc.DocUtils.linkForRef('classMethod:Class.create','create()')} method of the\n ${isc.DocUtils.linkForRef('attr:ListGrid.headerSpanConstructor')}.  This allows you to set properties such as\n ${isc.DocUtils.linkForRef('attr:Button.baseStyle')} or ${isc.DocUtils.linkForRef('attr:StretchImgButton.src')} directly in a\n <code>headerSpan</code>.",
        name:"HeaderSpan",
        seeAlso:[],
        type:"object",
        groups:["headerSpan"]
    },
    "attr:OperationBinding.orderClause":{
        ref:"attr:OperationBinding.orderClause",
        flags:"IR",
        description:"For a dataSource of ${isc.DocUtils.linkForRef('attr:DataSource.serverType','serverType')} \"sql\", this \n property can be specified on an operationBinding to provide the server with a bespoke\n ORDER BY clause to use when constructing the SQL query to perform this operation.  The\n property should be a comma-separated list of column names and/or expressions, forming\n a valid ORDER BY clause in the syntax of the underlying database.  The server will \n insert the text of this property immediately after the \"ORDER BY\" token.\n <p>\n This property is only applicable to operationBindings of \n ${isc.DocUtils.linkForRef('attr:OperationBinding.operationType','operationType')} \"fetch\".\n <p>\n See the documentation for ${isc.DocUtils.linkForRef('attr:OperationBinding.customSQL')} for usage examples",
        name:"orderClause",
        seeAlso:[
            {ref:"attr:OperationBinding.customSQL", description:null}
        ],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        serverDS:"only",
        definingClass:"class:OperationBinding",
        groups:["customQuerying"]
    },
    "method:Canvas.keyDown":{
        ref:"method:Canvas.keyDown",
        flags:"A",
        description:"\n Executed when a key is pressed on a focusable widget (${isc.DocUtils.linkForRef('attr:Canvas.canFocus')}: true). \n <P>\n Use ${isc.DocUtils.linkForRef('classMethod:EventHandler.getKey')} to find out the ${isc.DocUtils.linkForRef('type:KeyName','keyName')} of the key that\n was pressed, and use ${isc.DocUtils.linkForRef('classMethod:EventHandler.shiftKeyDown')} and related functions to determine\n whether modifier keys were down.\n\n",
        name:"keyDown",
        seeAlso:[
            {ref:"attr:Canvas.canFocus", description:null}
        ],
        returns:{description:"false to prevent this event from bubbling to this widget's parent, true or undefined to bubble.",
         type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "attr:DynamicForm.fields":{
        flags:"IRW",
        getter:["getFields"],
        type:"attr",
        ref:"attr:DynamicForm.fields",
        description:"An array of field objects, specifying the order, layout, and types of each field in the\n DynamicForm.\n <p>\n When both <code>dynamicForm.fields</code> and <code>dynamicForm.dataSource</code> are\n set, <code>dynamicForm.fields</code> acts as a set of overrides as explained in\n ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields')}.\n <P>\n See ${isc.DocUtils.linkForRef('group:formLayout','Form Layout')} for information about how flags specified on\n the FormItems control how the form is laid out.",
        name:"fields",
        setter:["setFields"],
        valueType:"Array of FormItem Properties",
        seeAlso:[
            {ref:"class:FormItem", description:null}
        ],
        defaultValue:"null",
        groups:["items"],
        definingClass:"class:DynamicForm"
    },
    "attr:SpinnerItem.maskPadChar":{
        ref:"attr:SpinnerItem.maskPadChar",
        flags:null,
        description:"Not applicable to a SpinnerItem.",
        name:"maskPadChar",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:SpinnerItem"
    },
    "method:ListGrid.setShowHeader":{
        ref:"method:ListGrid.setShowHeader",
        flags:"",
        description:" Show or hide the ListGrid header.\n",
        name:"setShowHeader",
        seeAlso:[],
        params:[
            {description:"true to show the header, false to hide it.", optional:false, name:"show",
             defaultValue:null, type:"boolean"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["gridHeader"]
    },
    "method:ListGrid.isCheckboxField":{
        ref:"method:ListGrid.isCheckboxField",
        flags:"",
        description:" Identifies whether the passed-in field is the specially generated\n ${isc.DocUtils.linkForRef('attr:ListGrid.checkboxField','checkboxField')} used when ${isc.DocUtils.linkForRef('type:SelectionAppearance')} is\n \"checkbox\".  Use this method in your custom event handlers to avoid inappropriately\n performing actions when the checkboxField is clicked on.\n \n",
        name:"isCheckboxField",
        seeAlso:[],
        params:[
            {description:"field to test", optional:false, name:"field", defaultValue:null,
             type:"ListGridField"}
        ],
        returns:{description:"whether the provided field is the checkbox field", type:"boolean"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["checkboxField"]
    },
    "method:ListGrid.setAutoFitData":{
        ref:"method:ListGrid.setAutoFitData",
        flags:"",
        description:" Setter for ${isc.DocUtils.linkForRef('attr:ListGrid.autoFitData')}.\n",
        name:"setAutoFitData",
        seeAlso:[],
        params:[
            {description:"One of <code>\"vertical\"</code>, <code>\"horizontal\"</code>  or <code>\"both\"</code>. To disable auto fit behavior, pass in <code>null</code>.",
             optional:false, name:"autoFitData", defaultValue:null, type:"string"}
        ],
        type:"method",
        definingClass:"class:ListGrid",
        groups:["autoFitData"]
    },
    "class:Calendar":{
        methods:["method:Calendar.fetchData", "method:Calendar.filterData", "method:Calendar.setData", 
         "method:Calendar.getCurrentViewName","method:Calendar.setCurrentViewName", "method:Calendar.setShowWeekends", 
         "method:Calendar.addEvent","method:Calendar.removeEvent", "method:Calendar.updateEvent", 
         "method:Calendar.getEventHoverHTML","method:Calendar.setChosenDate", "method:Calendar.dateIsWorkday", "method:Calendar.next", 
         "method:Calendar.previous","method:Calendar.getDayBodyHTML", "method:Calendar.dayBodyClick", 
         "method:Calendar.dayHeaderClick","method:Calendar.eventChanged", "method:Calendar.eventRemoved", 
         "method:Calendar.eventAdded","method:Calendar.eventClick", "method:Calendar.eventRemoveClick", 
         "method:Calendar.eventMoved","method:Calendar.eventResized", "method:Calendar.selectTab", "method:Calendar.dateChanged", 
         "method:Calendar.getActiveDay","method:Calendar.getActiveTime", "method:Calendar.setTimelineRange"],
        type:"class",
        ref:"class:Calendar",
        classAttrs:[],
        treeLocation:["Client Reference/Calendar"],
        classMethods:[],
        description:"The Calendar component provides several different ways for a user to view and\n edit a set of events. Note that the <b>ISC_Calendar.js</b> module must be \n loaded to make use of the Calendar class.\n <P>\n <b>CalendarEvents</b>\n <P>\n Events are represented as ordinary JavaScript Objects (see ${isc.DocUtils.linkForRef('object:CalendarEvent')}).  \n The Calendar expects to be able to read and write a basic set of properties \n on events: name, startDate, endDate, description, etc, which can be stored \n under configurable property names (see eg ${isc.DocUtils.linkForRef('attr:Calendar.startDateField')}.\n <P>\n Much like a ${isc.DocUtils.linkForRef('class:ListGrid')} manages it's ListGridRecords, the Calendar can \n either be passed an ordinary Array of CalendarEvents or can fetch data from a \n DataSource.\n <P>\n If the calendar is bound to a DataSource, event changes by user action or by \n calling methods will be saved to the DataSource.\n <P>\n <b>Navigation</b>\n <P>\n The calendar supports a ${isc.DocUtils.linkForRef('attr:Calendar.weekView','WeekView')}, \n ${isc.DocUtils.linkForRef('attr:Calendar.dayView','DayView')} and ${isc.DocUtils.linkForRef('attr:Calendar.monthView','MonthView')} by \n default.  The user can navigate using back and forward buttons or via an \n attached ${isc.DocUtils.linkForRef('attr:Calendar.dateChooser','DateChooser')}.\n <P>\n <b>Event Manipulation</b>\n <P>\n Events can be created via clicking on the day, week or month views, or via the \n \"Add Event\" button.  In the day and week views, the user may click and drag \n to create an event of a specific duration.\n <P>\n Creating an event via click or click and drag pops up the\n ${isc.DocUtils.linkForRef('attr:Calendar.eventDialog','EventDialog')}, which provides a simple form for \n quick event entry (only one field, the description, is required by default).  \n <P>\n A separate editor called the ${isc.DocUtils.linkForRef('attr:Calendar.eventEditor','EventEditor')} provides \n an interface for editing all possible properties of an event, including custom \n properties.  The EventEditor is used whenever a pre-existing event is being \n edited, and can also be invoked\n by the user wherever the simpler EventDialog appears.\n <P>\n Events can also be programmatically ${isc.DocUtils.linkForRef('method:Calendar.addEvent','added')},\n ${isc.DocUtils.linkForRef('method:Calendar.removeEvent','removed')}, or ${isc.DocUtils.linkForRef('method:Calendar.updateEvent','updated')}.",
        attrs:["attr:Calendar.chosenDate", "attr:Calendar.firstDayOfWeek", "attr:Calendar.baseStyle", 
         "attr:Calendar.dayHeaderBaseStyle","attr:Calendar.dayBodyBaseStyle", "attr:Calendar.otherDayHeaderBaseStyle", 
         "attr:Calendar.otherDayBodyBaseStyle","attr:Calendar.otherDayBlankStyle", "attr:Calendar.selectedCellStyle", 
         "attr:Calendar.eventWindowStyle","attr:Calendar.workdayBaseStyle", "attr:Calendar.workdayStart", "attr:Calendar.workdayEnd", 
         "attr:Calendar.showWorkday","attr:Calendar.workdays", "attr:Calendar.scrollToWorkday", "attr:Calendar.nameField", 
         "attr:Calendar.descriptionField","attr:Calendar.startDateField", "attr:Calendar.endDateField", 
         "attr:Calendar.leadingDateField","attr:Calendar.trailingDateField", "attr:Calendar.eventTypeField", 
         "attr:Calendar.eventWindowStyleField","attr:Calendar.canEditField", "attr:Calendar.weekEventBorderOverlap", 
         "attr:Calendar.eventSnapGap","attr:Calendar.showQuickEventDialog", "attr:Calendar.eventEditorFields", 
         "attr:Calendar.eventDialogFields","attr:Calendar.canCreateEvents", "attr:Calendar.canEditEvents", 
         "attr:Calendar.canDeleteEvents","attr:Calendar.showDateChooser", "attr:Calendar.disableWeekends", 
         "attr:Calendar.showWeekends","attr:Calendar.showDayHeaders", "attr:Calendar.showOtherDays", 
         "attr:Calendar.showControlsBar","attr:Calendar.invalidDateMessage", "attr:Calendar.eventAutoArrange", 
         "attr:Calendar.eventOverlap","attr:Calendar.eventOverlapPercent", "attr:Calendar.eventOverlapIdenticalStartTimes", 
         "attr:Calendar.dayView","attr:Calendar.weekView", "attr:Calendar.monthView", "attr:Calendar.dateChooser", 
         "attr:Calendar.eventDialog","attr:Calendar.eventEditorLayout", "attr:Calendar.eventEditor", 
         "attr:Calendar.datePickerButton","attr:Calendar.addEventButton", "attr:Calendar.previousButton", "attr:Calendar.nextButton", 
         "attr:Calendar.data","attr:Calendar.autoFetchData", "attr:Calendar.autoFetchTextMatchStyle", 
         "attr:Calendar.initialCriteria","attr:Calendar.dayViewTitle", "attr:Calendar.weekViewTitle", 
         "attr:Calendar.monthViewTitle","attr:Calendar.timelineViewTitle", "attr:Calendar.eventNameFieldTitle", 
         "attr:Calendar.saveButtonTitle","attr:Calendar.detailsButtonTitle", "attr:Calendar.cancelButtonTitle", 
         "attr:Calendar.previousButtonHoverText","attr:Calendar.nextButtonHoverText", "attr:Calendar.addEventButtonHoverText", 
         "attr:Calendar.datePickerHoverText","attr:Calendar.currentViewName", "attr:Calendar.eventWindow"],
        implementsInterfaces:["DataBoundComponent"],
        name:"Calendar",
        seeAlso:[],
        examples:["${isc.DocUtils.linkForExampleId('simpleCalendar')}"]
    },
    "classMethod:Date.setInputFormat":{
        ref:"classMethod:Date.setInputFormat",
        flags:"",
        description:" Sets up the default format for strings being parsed into dates via <code>Date.parseInput()</code>\n",
        name:"setInputFormat",
        seeAlso:[
            {ref:"classMethod:Date.parseInput", description:null}
        ],
        params:[
            {description:"Default format for strings to be parsed into Dates. If this method is passed a function, it is expected to take a single parameter \n (the formatted date string), and return the appropriate Javascript Date object (or null if \n appropriate).",
             optional:false, name:"format", defaultValue:null, type:"DateInputFormat | function"}
        ],
        type:"classMethod",
        examples:["${isc.DocUtils.linkForExampleId('dateFormat')}", 
         "${isc.DocUtils.linkForExampleId('customDateFormat')}"],
        definingClass:"class:Date"
    },
    "group:errorhandling":{
        ref:"group:errorhandling",
        refs:[],
        name:"errorhandling",
        seeAlso:[],
        type:"group"
    },
    "method:DynamicForm.isNewRecord":{
        ref:"method:DynamicForm.isNewRecord",
        flags:"",
        description:" Returns true if ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')} is currently \"add\".  See\n ${isc.DocUtils.linkForRef('attr:DynamicForm.saveOperationType')}.\n\n",
        name:"isNewRecord",
        seeAlso:[],
        returns:{description:"whether this form will use an \"add\" operation when saving", type:"boolean"},
        type:"method",
        definingClass:"class:DynamicForm"
    },
    "attr:DatabaseBrowser.schema":{
        ref:"attr:DatabaseBrowser.schema",
        flags:"IR",
        description:"The name of the database schema to use.  Leave this property unset to include all \n schemata.  Note that this property is not applicable to the MySQL database, which\n does not recognize the concept of multiple schemata within a database.\n <p>\n Note that this property only applies where ${isc.DocUtils.linkForRef('attr:DatabaseBrowser.serverType')} is \"sql\"",
        name:"schema",
        seeAlso:[],
        valueType:"string",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DatabaseBrowser"
    },
    "attr:HeaderSpan.align":{
        ref:"attr:HeaderSpan.align",
        flags:"IR",
        description:"Horizontal alignment of the title of this headerSpan.",
        name:"align",
        seeAlso:[],
        valueType:"Align",
        defaultValue:"\"center\"",
        type:"attr",
        definingClass:"object:HeaderSpan",
        groups:["headerSpan"]
    },
    "type:SelectionType":{
        ref:"type:SelectionType",
        values:[
            {description:"object moves to \"down\" state temporarily (normal button)",
             value:"StatefulCanvas.BUTTON"},
            {description:"object remains in \"down\" state until clicked again (checkbox)",
             value:"StatefulCanvas.CHECKBOX"},
            {description:"object moves to \"down\" state, causing another object to go up (radio)",
             value:"StatefulCanvas.RADIO"}
        ],
        description:"Controls how an object changes state when clicked",
        name:"SelectionType",
        seeAlso:[],
        type:"type",
        groups:["state", "event handling"]
    },
    "attr:DateItem.invalidDateStringMessage":{
        ref:"attr:DateItem.invalidDateStringMessage",
        flags:"IRW",
        description:"Validation error message to display if the user enters an invalid date",
        name:"invalidDateStringMessage",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Invalid date\"",
        type:"attr",
        definingClass:"class:DateItem",
        groups:["i18nMessages"]
    },
    "attr:Calendar.addEventButton":{
        ref:"attr:Calendar.addEventButton",
        flags:"IR",
        description:"The addEventButton is an ImgButton that appears above the week/day/month views of the\n calendar and offers an alternative way to create a new event.",
        name:"addEventButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:ListGrid.getField":{
        ref:"method:ListGrid.getField",
        flags:"",
        description:" Given a column number or field name, return the field definition.\n <P>\n When using ${isc.DocUtils.linkForRef('attr:DataBoundComponent.fields','DataBinding')}, the field definition may be\n a mix of information derived from ${isc.DocUtils.linkForRef('attr:ListGrid.fields')} and ${isc.DocUtils.linkForRef('attr:ListGrid.dataSource')}.\n\n",
        name:"getField",
        seeAlso:[],
        params:[
            {description:"number or id of the field.", optional:false, name:"colNum", defaultValue:null,
             type:"number or ID"}
        ],
        returns:{description:"field definition", type:"ListGridField"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:CubeGrid.getBaseStyle":{
        ref:"method:CubeGrid.getBaseStyle",
        flags:"",
        description:" Return the base stylename for this cell.  Default implementation just returns this.baseStyle.\n See ${isc.DocUtils.linkForRef('method:ListGrid.getCellStyle','getCellStyle()')} for a general discussion of how to style cells.\n\n",
        name:"getBaseStyle",
        seeAlso:[
            {ref:"method:CubeGrid.getCellStyle", description:null}
        ],
        params:[
            {description:"cell record as returned by getCellRecord", optional:false, name:"record",
             defaultValue:null, type:"object"},
            {description:"row number for the cell", optional:false, name:"rowNum", defaultValue:null,
             type:"number"},
            {description:"column number of the cell", optional:false, name:"colNum", defaultValue:null,
             type:"number"}
        ],
        returns:{description:"CSS class for this cell", type:"CSSStyleName"},
        type:"method",
        definingClass:"class:CubeGrid"
    },
    "attr:TreeNode.title":{
        ref:"attr:TreeNode.title",
        flags:"IR",
        description:"The title of the node as it should appear next to the node icon in the ${isc.DocUtils.linkForRef('class:Tree')}.  If left\n unset, the value of ${isc.DocUtils.linkForRef('attr:TreeNode.name')} is used by default.  See the description in\n ${isc.DocUtils.linkForRef('method:Tree.getTitle')} for full details.\n <p>\n Note: the name of this property can be changed by setting ${isc.DocUtils.linkForRef('attr:Tree.titleProperty')}.",
        name:"title",
        seeAlso:[
            {ref:"attr:Tree.titleProperty", description:null},
            {ref:"method:Tree.getTitle", description:null}
        ],
        valueType:"HTML",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:TreeNode"
    },
    "attr:MenuItem.keyTitle":{
        ref:"attr:MenuItem.keyTitle",
        flags:"IR",
        description:"A string to display in the shortcut-key column for this item. If not\n specified, the first KeyName value in ${isc.DocUtils.linkForRef('attr:MenuItem.keys')} will be used by default.",
        name:"keyTitle",
        seeAlso:[],
        valueType:"string",
        defaultValue:"see below",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('fullMenu')}"],
        definingClass:"object:MenuItem",
        groups:["menuKeys"]
    },
    "attr:ListGrid.sortFieldAscendingText":{
        ref:"attr:ListGrid.sortFieldAscendingText",
        flags:"IRW",
        description:"If we're showing a ${isc.DocUtils.linkForRef('attr:ListGrid.showHeaderContextMenu','headerContextMenu')} for this grid, this\n attribute will be shown as the menu item title to sort a field in ascending order.",
        name:"sortFieldAscendingText",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"Sort Ascending\"",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["i18nMessages"]
    },
    "attr:TabSet.topEdgeOffsets":{
        ref:"attr:TabSet.topEdgeOffsets",
        flags:"IR",
        description:"If this tabSet will ${isc.DocUtils.linkForRef('attr:TabSet.showPaneContainerEdges','show edges')} for the paneContainer,\n and ${isc.DocUtils.linkForRef('attr:TabSet.symmetricEdges')} is set to false, the <code>leftEdgeOffsets</code>, \n <code>rightEdgeOffsets</code>, <code>topEdgeOffsets</code> and <code>bottomEdgeOffsets</code> \n propertes allow the offsets of edges for the paneContainer to be customized depending on\n the ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.\n <P>\n The attribute should be specified an ${isc.DocUtils.linkForRef('type:EdgeSizes','edgeSizes map')}, specifying the\n desired edge offsets where for the appropriate ${isc.DocUtils.linkForRef('attr:TabSet.tabBarPosition')}.",
        name:"topEdgeOffsets",
        seeAlso:[],
        valueType:"EdgeSizes",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:TabSet"
    },
    "attr:ListGrid.recordDetailDSProperty":{
        ref:"attr:ListGrid.recordDetailDSProperty",
        flags:"IRWA",
        description:"The name of the ListGridRecord property that specifies the DataSource to use when \n ${isc.DocUtils.linkForRef('type:ExpansionMode','listGrid.expansionMode')} is \"related\".  The default is \n ${isc.DocUtils.linkForRef('attr:ListGridRecord.detailDS')}.",
        name:"recordDetailDSProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"detailDS\"",
        type:"attr",
        definingClass:"class:ListGrid"
    },
    "method:GridRenderer.refreshRow":{
        ref:"method:GridRenderer.refreshRow",
        flags:"A",
        description:" Refresh an entire row of cells without redrawing the grid.\n <P>\n The cells' values, CSS classes, and CSS text will be refreshed, to the current values\n returned by getCellValue(), getCellStyle() and getCellCSSText() respectively.\n\n",
        name:"refreshRow",
        seeAlso:[
            {ref:"method:GridRenderer.refreshCellStyle", description:"to update just styling"},
            {ref:"method:GridRenderer.refreshCell", description:null}
        ],
        params:[
            {description:"row number of cell to refresh", optional:false, name:"rowNum",
             defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:GridRenderer",
        groups:["appearance"]
    },
    "attr:DOMGrid.rootElement":{
        ref:"attr:DOMGrid.rootElement",
        flags:"IRW",
        description:"Root element (or document) to view in the tree.",
        name:"rootElement",
        setter:["setRootElement"],
        seeAlso:[],
        valueType:"DOMElement",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:DOMGrid"
    },
    "attr:ImgButton.baseStyle":{
        ref:"attr:ImgButton.baseStyle",
        flags:"IRW",
        description:"Base CSS style.  As the component changes state and/or is selected, suffixes will be\n added to the base style.\n <P>\n When the component changes state (eg becomes disabled), a suffix will be appended to this\n style name, reflecting the following states: \"Over\", \"Down\", or \"Disabled\".\n <P>\n If the widget is selected, the suffixes will be \"Selected\", \"SelectedOver\", etc.\n <P>\n If the widget has focus and ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocused')} is true, and\n ${isc.DocUtils.linkForRef('attr:StatefulCanvas.showFocusedAsOver')} is false, the suffixes will be \"Focused\",\n \"FocusedOver\", etc, or if the widget is both selected and focused, \"SelectedFocused\",\n \"SelectedFocusedOver\", etc.\n <P>\n For example, if <code>baseStyle</code> is set to \"button\", this component is\n ${isc.DocUtils.linkForRef('method:ImgButton.isSelected','selected')} and the mouse cursor is over this component, the style\n \"buttonSelectedOver\" will be used.",
        name:"baseStyle",
        setter:["setBaseStyle"],
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"\"imgButton\"",
        type:"attr",
        definingClass:"class:ImgButton"
    },
    "attr:TreeGrid.manyItemsImage":{
        ref:"attr:TreeGrid.manyItemsImage",
        flags:"IRW",
        description:"The filename of the icon displayed use as the default drag tracker when for multiple\n files and/or folders are being dragged.",
        name:"manyItemsImage",
        seeAlso:[],
        valueType:"SCImgURL",
        defaultValue:"\"[SKIN]folder_file.gif\"",
        type:"attr",
        definingClass:"class:TreeGrid",
        groups:["dragdrop"]
    },
    "attr:Slider.animateThumbTime":{
        ref:"attr:Slider.animateThumbTime",
        flags:"IRW",
        description:"Duration of thumb animation, in milliseconds.",
        name:"animateThumbTime",
        seeAlso:[],
        valueType:"integer",
        defaultValue:"250",
        type:"attr",
        definingClass:"class:Slider",
        groups:["animation"]
    },
    "attr:ListGridField.autoFetchDisplayMap":{
        ref:"attr:ListGridField.autoFetchDisplayMap",
        flags:"IRW",
        description:"If true, automatically fetches records and derives a valueMap from\n ${isc.DocUtils.linkForRef('attr:ListGridField.optionDataSource')}.\n <p>\n Same as ${isc.DocUtils.linkForRef('attr:ListGrid.autoFetchDisplayMap')}, but defined on a per-field basis.",
        name:"autoFetchDisplayMap",
        seeAlso:[
            {ref:"attr:ListGrid.autoFetchDisplayMap", description:null}
        ],
        valueType:"boolean",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:ListGridField",
        groups:["display_values"]
    },
    "method:Canvas.dragRepositionStart":{
        ref:"method:Canvas.dragRepositionStart",
        flags:"A",
        description:"\n Executed when dragging first starts. No default implementation.  Create this handler to set\n things up for the drag reposition.\n\n",
        name:"dragRepositionStart",
        seeAlso:[
            {ref:"method:Canvas.getOffsetX", description:null},
            {ref:"method:Canvas.getOffsetY", description:null}
        ],
        returns:{description:"false to cancel the drag reposition action", type:"boolean"},
        type:"method",
        definingClass:"class:Canvas",
        groups:["widgetEvents"]
    },
    "classMethod:EventHandler.getDragTarget":{
        ref:"classMethod:EventHandler.getDragTarget",
        flags:"A",
        description:"\n Returns the current dragTarget.  This is the component on which the drag and drop\n interaction was initiated.  This only returns something meaningful during a drag and drop\n interaction.\n\n",
        name:"getDragTarget",
        seeAlso:[
            {ref:"attr:Canvas.dragTarget", description:null}
        ],
        returns:{description:"The dragTarget.", type:"Canvas"},
        type:"classMethod",
        definingClass:"class:EventHandler",
        groups:["mouseEvents"]
    },
    "method:ListGrid.chartRow":{
        ref:"method:ListGrid.chartRow",
        flags:"",
        description:" Chart a single row of data, with each cell value labelled by the column header.\n\n",
        name:"chartRow",
        seeAlso:[],
        params:[
            {description:"row to chart", optional:false, name:"rowNum", defaultValue:null, type:"Number"},
            {description:"optional list of fields to use as labels.  By      default, all fields are used.",
             optional:true, name:"dataFields", defaultValue:null, type:"Array of String"},
            {description:"properties to pass to the created chart", optional:true,
             name:"chartProperties", defaultValue:null, type:"Chart Properties"}
        ],
        returns:{description:"created Chart instance", type:"Chart"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "method:Canvas.setHeight":{
        ref:"method:Canvas.setHeight",
        flags:"",
        description:" Resizes the widget vertically to the specified height (moves the bottom side of the\n widget). The height parameter can be expressed as a percentage of viewport size or as\n the number of pixels.\n <P>\n NOTE: if you're setting multiple coordinates, use resizeTo() or setRect() instead\n\n",
        name:"setHeight",
        seeAlso:[],
        params:[
            {description:"new height", optional:false, name:"height", defaultValue:null, type:"number"}
        ],
        type:"method",
        definingClass:"class:Canvas",
        groups:["sizing"]
    },
    "method:ResultSet.findIndex":{
        ref:"method:ResultSet.findIndex",
        flags:"",
        description:" Like ${isc.DocUtils.linkForRef('method:List.findIndex')}.  Checks only loaded rows and will not trigger a fetch.\n",
        name:"findIndex",
        seeAlso:[],
        params:[
            {description:"property to match, or if an Object is passed, set of                                        properties and values to match",
             optional:false, name:"propertyName", defaultValue:null, type:"String or Object"},
            {description:"value to compare against (if propertyName is a string)", optional:true,
             name:"value", defaultValue:null, type:"any"}
        ],
        returns:{description:"index of the first matching Object or -1 if not found", type:"int"},
        type:"method",
        definingClass:"class:ResultSet",
        groups:["access", "find"]
    },
    "attr:Window.showBody":{
        ref:"attr:Window.showBody",
        flags:"IRWA",
        description:"If true, draw the body contents when this Window is drawn.",
        name:"showBody",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Window",
        groups:["appearance", "body"]
    },
    "method:Selection.isSelected":{
        ref:"method:Selection.isSelected",
        flags:"",
        description:"			Return true if a particular item is selected\n",
        name:"isSelected",
        seeAlso:[],
        params:[
            {description:"object to check", optional:false, name:"item", defaultValue:null,
             type:"object"}
        ],
        returns:{description:"true == object is selected								false == object is not selected",
         type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "method:Selection.selectItem":{
        ref:"method:Selection.selectItem",
        flags:"",
        description:" Select a particular item by its position in the list\n\n",
        name:"selectItem",
        seeAlso:[],
        params:[
            {description:"index of the item to be selected", optional:false, name:"position",
             defaultValue:null, type:"number"}
        ],
        returns:{description:"true == selection actually changed, false == no change", type:"boolean"},
        type:"method",
        definingClass:"class:Selection",
        groups:["selection"]
    },
    "attr:Calendar.weekView":{
        ref:"attr:Calendar.weekView",
        flags:"R",
        description:"Listgrid used to display events that pertain to a given week.",
        name:"weekView",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Calendar"
    },
    "method:Class.isObserving":{
        ref:"method:Class.isObserving",
        flags:"A",
        description:null,
        name:"isObserving",
        seeAlso:[],
        params:[
            {description:"object we may be observing", optional:false, name:"object", defaultValue:null,
             type:"object"},
            {description:"name of the method to observed", optional:false, name:"methodName",
             defaultValue:null, type:"string"}
        ],
        returns:{description:"true == already observing that method", type:"boolean"},
        type:"method",
        definingClass:"class:Class",
        groups:["observation"]
    },
    "method:List.get":{
        ref:"method:List.get",
        flags:"",
        description:" Return the item at a particular position\n",
        name:"get",
        seeAlso:[],
        params:[
            {description:"position of the element to get", optional:false, name:"pos",
             defaultValue:null, type:"Number"}
        ],
        returns:{description:"whatever's at that position, undef if not found", type:"object"},
        type:"method",
        definingClass:"interface:List",
        groups:["access"]
    },
    "group:htmlElement":{
        ref:"group:htmlElement",
        refs:["method:Canvas.setHtmlElement", "method:Canvas.setHtmlPosition", "attr:Canvas.htmlElement", 
         "attr:Canvas.htmlPosition"],
        name:"htmlElement",
        seeAlso:[],
        type:"group"
    },
    "attr:MultiFileItem.editButton":{
        ref:"attr:MultiFileItem.editButton",
        flags:"IR",
        description:"Button for launching a picker to add new files for upload.  Supports the properties of a\n ${isc.DocUtils.linkForRef('object:FormItemIcon')}.",
        name:"editButton",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:MultiFileItem"
    },
    "attr:CalendarEvent.description":{
        ref:"attr:CalendarEvent.description",
        flags:"IRW",
        description:"String which represents the description of a ${isc.DocUtils.linkForRef('object:CalendarEvent')}\n The name of this field within the CalendarEvent can be changed via \n ${isc.DocUtils.linkForRef('attr:Calendar.descriptionField')}",
        name:"description",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"object:CalendarEvent"
    },
    "method:HTMLFlow.loadingContent":{
        ref:"method:HTMLFlow.loadingContent",
        flags:"A",
        description:" Returns true if this htmlFlow is currently loading content from the server.<br>\n Note: Does not apply to htmlFlows with ${isc.DocUtils.linkForRef('attr:HTMLFlow.contentsType','contentsType')} set to \n <code>\"page\"</code>\n\n",
        name:"loadingContent",
        seeAlso:[
            {ref:"method:HTMLFlow.contentLoaded", description:null}
        ],
        type:"method",
        definingClass:"class:HTMLFlow",
        groups:["contentLoading"]
    },
    "attr:Slider.minValue":{
        ref:"attr:Slider.minValue",
        flags:"IRW",
        description:"The minimum slider value. The slider value is equal to minValue when the thumb is at the\n bottom or left of the slider (unless flipValues is true, in which case the minimum value\n is at the top/right of the slider)",
        name:"minValue",
        setter:["setMinValue"],
        seeAlso:[
            {ref:"attr:Slider.flipValues", description:null}
        ],
        valueType:"float",
        defaultValue:"1",
        type:"attr",
        examples:["${isc.DocUtils.linkForExampleId('slider')}"],
        definingClass:"class:Slider"
    },
    "attr:DataSource.canMultiSort":{
        ref:"attr:DataSource.canMultiSort",
        flags:"IR",
        description:"When true, indicates that this DataSource supports multi-level sorting.",
        name:"canMultiSort",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        serverDS:"allowed",
        definingClass:"class:DataSource"
    },
    "attr:ListGrid.showRollOver":{
        ref:"attr:ListGrid.showRollOver",
        flags:"IRW",
        description:"Should we show different styling for the cell the mouse is over?\n <br>\n If true, the cell style will have the suffix \"Over\" appended.",
        name:"showRollOver",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["appearance"]
    },
    "method:ListGrid.getCellValue":{
        ref:"method:ListGrid.getCellValue",
        flags:"A",
        description:"          Obtains the display value for a specific cell according to the given input \n          parameters.<br>\n          To format the value displayed in the cell, make use of the \n          <code>formatCellValue</code> methods rather than overriding this method directly.\n",
        name:"getCellValue",
        seeAlso:[
            {ref:"method:ListGrid.formatCellValue", description:null}
        ],
        params:[
            {description:"the current record object", optional:false, name:"record", defaultValue:null,
             type:"object"},
            {description:"number of the record in the current set of displayed                                       record (e.g. 0 for the first displayed record)",
             optional:false, name:"recordNum", defaultValue:null, type:"number"},
            {description:"number of the field in the listGrid.fields array", optional:false,
             name:"fieldNum", defaultValue:null, type:"number"}
        ],
        returns:{description:"display value for this cell", type:"string"},
        type:"method",
        definingClass:"class:ListGrid",
        groups:["data"]
    },
    "attr:Mail.from":{
        ref:"attr:Mail.from",
        flags:"IR",
        description:"The email address the message will appear to come from.  Like all other <code>String</code> \n properties of <code>Mail</code>, you can use Velocity substitution variables in this property.\n <p>\n If you specify a \"From:\" line in the body of your email, the from address will be parsed \n and this property will be ignored.",
        name:"from",
        seeAlso:[],
        valueType:"String",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Mail",
        groups:["mail"]
    },
    "classMethod:DataSource.addSearchOperator":{
        ref:"classMethod:DataSource.addSearchOperator",
        flags:"",
        description:" Add a new search operator to all DataSources.  \n <P>\n See also ${isc.DocUtils.linkForRef('method:DataSource.addSearchOperator')} for adding operators to specific\n DataSources only.\n\n",
        name:"addSearchOperator",
        seeAlso:[],
        params:[
            {description:"definition of the operator to add", optional:false, name:"operator",
             defaultValue:null, type:"Operator"}
        ],
        type:"classMethod",
        definingClass:"class:DataSource",
        groups:["advancedFilter"]
    },
    "attr:Calendar.disableWeekends":{
        ref:"attr:Calendar.disableWeekends",
        flags:"IR",
        description:"If set, weekend days appear in disabled style and events cannot be created on weekends.\n Which days are considered weekends is controlled by ${isc.DocUtils.linkForRef('classAttr:Date.weekendDays')}.",
        name:"disableWeekends",
        seeAlso:[],
        valueType:"boolean",
        defaultValue:"true",
        type:"attr",
        definingClass:"class:Calendar",
        groups:["visibility"]
    },
    "method:Class.logIsDebugEnabled":{
        ref:"method:Class.logIsDebugEnabled",
        flags:"",
        description:" Check whether a message logged at \"debug\" priority would be visible in the log.\n <P>\n As with logDebug, category is defaulted to the current className.  Use this method to avoid\n putting together expensive log messages if they will never appear in the log.\n\n",
        name:"logIsDebugEnabled",
        seeAlso:[],
        params:[
            {description:"category to log in", optional:true, name:"category", defaultValue:null,
             type:"String"}
        ],
        type:"method",
        definingClass:"class:Class"
    },
    "classMethod:Log.getDefaultPriority":{
        ref:"classMethod:Log.getDefaultPriority",
        flags:"",
        description:" Retrieves the default priority of messages that will be visible.\n\n",
        name:"getDefaultPriority",
        seeAlso:[],
        params:[
            {description:"Optional ISC class or instance - if passed the returns the default priority for \n     the class or instance only.",
             optional:true, name:"object", defaultValue:null, type:"Class or Instance object"}
        ],
        returns:{description:"default priority for which messages will be logged.", type:"LogPriority"},
        type:"classMethod",
        definingClass:"class:Log"
    },
    "method:MultiSortDialog.getSort":{
        ref:"method:MultiSortDialog.getSort",
        flags:"",
        description:" Returns all configured sorting levels, as an array of ${isc.DocUtils.linkForRef('object:SortSpecifier')}s.\n\n",
        name:"getSort",
        seeAlso:[],
        returns:{description:"the SortSpecifiers for all configured sorting levels",
         type:"Array of SortSpecifier"},
        type:"method",
        definingClass:"class:MultiSortDialog"
    },
    "method:ResultSet.setCriteria":{
        ref:"method:ResultSet.setCriteria",
        flags:"",
        description:" Set the filter criteria to use when fetching rows.\n <P>\n Setting new criteria will invalidate the current cache, if any.\n <P>\n Note: any field values in the criteria explicitly specified as null will be\n passed to the server. By default the server then returns only records whose value is null for\n that field. This differs from certain higher level methods such as \n ${isc.DocUtils.linkForRef('method:ListGrid.fetchData')} which prune null criteria fields before performing a fetch operation.\n\n",
        name:"setCriteria",
        seeAlso:[],
        params:[
            {description:"the filter criteria", optional:false, name:"newCriteria", defaultValue:null,
             type:"Criteria"}
        ],
        type:"method",
        definingClass:"class:ResultSet"
    },
    "attr:ListGrid.headerBaseStyle":{
        ref:"attr:ListGrid.headerBaseStyle",
        flags:"IR",
        description:"${isc.DocUtils.linkForRef('attr:Button.baseStyle')} to apply to the buttons in the header, and the sorter, for \n this ListGrid.\n Note that depending on the ${isc.DocUtils.linkForRef('attr:ListGrid.headerButtonConstructor','Class')} of the header\n buttons you may also need to set ${isc.DocUtils.linkForRef('attr:ListGrid.headerTitleStyle')}.",
        name:"headerBaseStyle",
        seeAlso:[],
        valueType:"CSSStyleName",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:ListGrid",
        groups:["gridHeader", "appearance"]
    },
    "attr:DetailViewer.fieldIdProperty":{
        ref:"attr:DetailViewer.fieldIdProperty",
        flags:"IRWA",
        description:"Name of the field in the DetailViewerRecord which specifies the data property for that record.",
        name:"fieldIdProperty",
        seeAlso:[],
        valueType:"string",
        defaultValue:"\"name\"",
        type:"attr",
        definingClass:"class:DetailViewer"
    },
    "attr:ColumnTree.customIconProperty":{
        ref:"attr:ColumnTree.customIconProperty",
        flags:"IRW",
        description:"This property allows the developer to customize the icon displayed next to a node.\n Set <code>node[grid.customIconProperty]</code> to the URL of the desired icon to display and\n it will be shown instead of the standard ${isc.DocUtils.linkForRef('attr:ColumnTree.nodeIcon')} for this node.<br>\n Note that if ${isc.DocUtils.linkForRef('attr:ColumnTree.showCustomIconOpen')} \n is true for this grid, customized icons for folder nodes will be appended with the \n ${isc.DocUtils.linkForRef('attr:ColumnTree.openIconSuffix')} suffix on state change,\n as with the standard ${isc.DocUtils.linkForRef('attr:ColumnTree.folderIcon')}.  Also note that for\n custom folder icons, the ${isc.DocUtils.linkForRef('attr:ColumnTree.closedIconSuffix')} will never be appened.",
        name:"customIconProperty",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"icon\"",
        type:"attr",
        definingClass:"class:ColumnTree",
        groups:["treeIcons"]
    },
    "group:inlineFormItems":{
        ref:"group:inlineFormItems",
        refs:[],
        name:"inlineFormItems",
        seeAlso:[],
        type:"group"
    },
    "attr:Window.body":{
        ref:"attr:Window.body",
        flags:"R",
        description:"Body of the Window, where ${isc.DocUtils.linkForRef('group:Items','contained components')} or ${isc.DocUtils.linkForRef('attr:Window.src','loaded content')}\n is shown.",
        name:"body",
        seeAlso:[],
        valueType:"AutoChild",
        defaultValue:"null",
        type:"attr",
        definingClass:"class:Window"
    },
    "classAttr:Operators.containsTitle":{
        ref:"classAttr:Operators.containsTitle",
        flags:"IR",
        description:"Title for the \"contains\" operator",
        name:"containsTitle",
        seeAlso:[],
        valueType:"String",
        defaultValue:"\"contains (match case)\"",
        type:"classAttr",
        definingClass:"class:Operators",
        groups:["i18nMessages"]
    },
    "method:CubeGrid.getEventColumn":{
        ref:"method:CubeGrid.getEventColumn",
        flags:"",
        description:" Returns the column number of the most recent mouse event.\n",
        name:"getEventColumn",
        seeAlso:[],
        params:[
            {description:"optional x-coordinate to obtain column number for, in lieue of the x                        coordinate of the last mouse event",
             optional:true, name:"x", defaultValue:null, type:"number"}
        ],
        returns:{description:"column number, or -2 if beyond last drawn column", type:"number"},
        type:"method",
        definingClass:"class:CubeGrid",
        groups:["events", "selection"]
    },
    "attr:LoginDialog.showLostPasswordLink":{
        ref:"attr:LoginDialog.showLostPasswordLink",
        flags:"IR",
        description:"If true, display a ${isc.DocUtils.linkForRef('class:LinkItem')} (${isc.DocUtils.linkForRef('attr:LoginDialog.lostPasswordItem')})\n meant for the user to click if the account's\n credentials are forgotten. The text of the link is controlled by\n ${isc.DocUtils.linkForRef('attr:LoginDialog.lostPasswordItemTitle')}. If clicked, the link will fire\n ${isc.DocUtils.linkForRef('method:LoginDialog.lostPassword')}.",
        name:"showLostPasswordLink",
        seeAlso:[],
        valueType:"Boolean",
        defaultValue:"false",
        type:"attr",
        definingClass:"class:LoginDialog"
    },
    "method:ListGrid.isSortField":{
        ref:"method:ListGrid.isSortField",
        flags:"A",
        description:"  Returns true if the passed fieldName is in the current sort-specification.\n",
        name:"isSortField",
        seeAlso:[],
        params:[
            {description:"The name of a field, visible, hidden or existing only in the dataSource",
             optional:false, name:"fieldName", defaultValue:null, type:"String"}
        ],
        returns:{description:"True if the passed field is sorted, false otherwise", type:"Boolean"},
        type:"method",
        definingClass:"class:ListGrid"
    },
    "class:IButton":{
        methods:[],
        ref:"class:IButton",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Control"],
        attrs:[],
        description:"The IButton widget class is a class that implements the same APIs as the \n ${isc.DocUtils.linkForRef('class:Button')} class.  Depending on the current skin, <code>IButton</code>s may be\n on the ${isc.DocUtils.linkForRef('class:StretchImgButton')} component, which renders via images, or may be based on the\n ${isc.DocUtils.linkForRef('class:Button')} component, which renders via CSS styles.",
        name:"IButton",
        seeAlso:[],
        type:"class"
    },
    "class:Snapbar":{
        methods:[],
        ref:"class:Snapbar",
        classAttrs:[],
        classMethods:[],
        treeLocation:["Client Reference/Layout"],
        attrs:["attr:Snapbar.showRollOver", "attr:Snapbar.showDown", "attr:Snapbar.showGrip", 
         "attr:Snapbar.showDownGrip","attr:Snapbar.showRollOverGrip", "attr:Snapbar.showClosedGrip", 
         "attr:Snapbar.gripImgSuffix"],
        description:"Subclass of the ${isc.DocUtils.linkForRef('class:Splitbar')} class that uses the <code>grip</code> functionality\n to show a stateful open / close indicator.",
        name:"Snapbar",
        seeAlso:[
            {ref:"class:Splitbar", description:null},
            {ref:"class:Layout", description:null}
        ],
        type:"class"
    },
    "attr:SpinnerItem.mask":{
        ref:"attr:SpinnerItem.mask",
        flags:null,
        description:"Not applicable to a SpinnerItem.",
        name:"mask",
        seeAlso:[],
        valueType:null,
        defaultValue:null,
        type:"attr",
        definingClass:"class:SpinnerItem"
    },
    "class:TextAreaItem":{
        methods:["method:TextAreaItem.getSelectionRange", "method:TextAreaItem.setSelectionRange", 
         "method:TextAreaItem.selectValue","method:TextAreaItem.deselectValue"],
        exampleConfig:"TextAreaItem.xml",
        ref:"class:TextAreaItem",
        classAttrs:[],
        classMethods:[],
        attrs:["attr:TextAreaItem.wrap", "attr:TextAreaItem.width", "attr:TextAreaItem.height", 
         "attr:TextAreaItem.textBoxStyle","attr:TextAreaItem.iconVAlign", "attr:TextAreaItem.selectOnFocus", 
         "attr:TextAreaItem.showHintInField"],
        description:"Class for editable multi-line text areas (uses HTML <code>&lt;TEXTAREA&gt;</code> object)",
        name:"TextAreaItem",
        seeAlso:[],
        type:"class",
        examples:["${isc.DocUtils.linkForExampleId('textAreaItem')}"]
    }
};

/*
 Isomorphic SmartClient web presentation layer
 Online documentation data
 Copyright 2000-2008 Isomorphic Software, Inc. All rights reserved.
*/

